/*

   HL7Daemon - Record Matcher (People, Tax Entities, Etc.)

   This module is responsible for taking the processed HL7 messages and then
   doing the following:

   * Determining existing records to be updated
   * Determining new records to be inserted
   * Determining records to mark expired or deleted, if any

*/

#ifndef HL7MATCH_H__
#define HL7MATCH_H__

#include "hl7process.h"

class HL7Matcher
{
protected:
   typedef HL7Processor::IntermediateMap::reverse_iterator reverse_interm_iterator;

   HL7Processor::IntermediateMap *im; // intermediates from HL7Processor
   PrometheusDB *db;                  // database connection
   PrometheusTransaction *tr;         // current transaction

   std::string daemonUserID;          // user ID to use for updates/inserts

   bool getBasicDatabaseInfo();
   void optimizeForPrimary(reverse_interm_iterator itr);
   void optimizeIntermediates();
   bool findBestMatches(HL7Person &p, pdb::vecmap &outvm);
   void doMatching();
   
public:
   HL7Matcher() : im(NULL), db(NULL), tr(NULL), daemonUserID()
   {
   }

   ~HL7Matcher();

   void matchIntermediates(HL7Processor::IntermediateMap &pIM);
};

#endif

// EOF

