/*

   HL7Segment Descendant Classes

*/

#ifndef HL7SEGMENTS_H__
#define HL7SEGMENTS_H__

#include "hl7parse.h"

//=============================================================================
//
// MSHSegment
//

class MSHSegment : public HL7Segment
{
   DECLARE_MSG_TYPE(MSHSegment, HL7Segment)

public:
   // MSH fields enumeration
   enum
   {
      MSH_ENCODINGCHARS,
      MSH_SENDINGAPP,
      MSH_SENDINGFAC,
      MSH_RECEIVINGAPP,
      MSH_RECEIVINGFAC,
      MSH_TIMESTAMP,
      MSH_SECURITY,
      MSH_MESSAGETYPE,
      MSH_MESSAGECTRLID,
      MSH_PROCESSINGID,
      MSH_VERSIONID,
      MSH_SEQUENCENUM,
      MSH_CONTINUATIONPTR,
      MSH_ACCEPTACKTYPE,
      MSH_APPACKTYPE,
      MSH_COUNTRYCODE,
      MSH_CHARSET,
      MSH_LANGUAGE,
      MSH_END,          // NB: for NULL terminator
      MSH_NUMFIELDS
   };

protected:
   static HL7FieldInfo ExpectedFields[MSH_NUMFIELDS];

public:
   virtual int getNumFields() const { return MSH_NUMFIELDS; }
   virtual const HL7FieldInfo *getFieldInfo() const { return ExpectedFields; }
};

//=============================================================================
//
// EVNSegment
//

class EVNSegment : public HL7Segment
{
   DECLARE_MSG_TYPE(EVNSegment, HL7Segment)

public:
   // EVN fields enumeration
   enum
   {
      EVN_TYPECODE,
      EVN_TIMESTAMP,
      EVN_PLANNEDSTAMP,
      EVN_REASONCODE,
      EVN_OPERATORID,
      EVN_EVENTOCCURED,
      EVN_END,
      EVN_NUMFIELDS
   };

protected:
   static HL7FieldInfo ExpectedFields[EVN_NUMFIELDS];

public:
   virtual int getNumFields() const { return EVN_NUMFIELDS; }
   virtual const HL7FieldInfo *getFieldInfo() const { return ExpectedFields; }
};

//=============================================================================
//
// PIDSegment
//

class PIDSegment : public HL7Segment
{
   DECLARE_MSG_TYPE(PIDSegment, HL7Segment)

public:
   // PID fields enumeration
   enum
   {
      PID_SETID,
      PID_EXTID,
      PID_INTID,
      PID_ALTID,
      PID_PATNAME,
      PID_MOTHER_MAIDEN_NAME,
      PID_DOB,
      PID_GENDER,
      PID_PATALIAS,
      PID_RACE,
      PID_PATADDRESS,
      PID_COUNTY_CODE,
      PID_PHONE_HOME,
      PID_PHONE_BUSINESS,
      PID_LANGUAGE,
      PID_MARITAL_STATUS,
      PID_REGION,
      PID_ACCOUNT_NUMBER,
      PID_PATSSN,
      PID_DRIVERS_LICENSE,
      PID_MOTHER_ID,
      PID_ETHNICITY,
      PID_BIRTH_PLACE,
      PID_MULTIBIRTH_INDICATOR,
      PID_BIRTH_ORDER,
      PID_CITIZENSHIP,
      PID_VET_MIL_STATUS,
      PID_NATIONALITY,
      PID_DEATH_TIMESTAMP,
      PID_DEATH_INDICATOR,
      PID_END,
      PID_NUMFIELDS
   };

protected:
   static HL7FieldInfo ExpectedFields[PID_NUMFIELDS];

public:
   virtual int getNumFields() const { return PID_NUMFIELDS; }
   virtual const HL7FieldInfo *getFieldInfo() const { return ExpectedFields; }
};

//============================================================================
//
// NK1Segment
//

class NK1Segment : public HL7Segment
{
   DECLARE_MSG_TYPE(NK1Segment, HL7Segment)

public:
   // NK1 fields enumeration
   enum
   {
      NK1_SETID,
      NK1_NAME,
      NK1_RELATIONSHIP,
      NK1_ADDRESS,
      NK1_PHONE_NUMBER,
      NK1_PHONE_BUSINESS,
      NK1_CONTACT_ROLE,
      NK1_START_DATE,
      NK1_END_DATE,
      NK1_JOB_TITLE,
      NK1_JOB_CODE,
      NK1_JOB_EMPLOYEENUM,
      NK1_ORGANIZATION_NAME,
      NK1_MARITAL_STATUS,
      NK1_ADMIN_GENDER,
      NK1_BIRTH_TIMESTAMP,
      NK1_LIVING_DEPENDENCY,
      NK1_AMBULATORY_STATUS,
      NK1_CITIZENSHIP,
      NK1_LANGUAGE,
      NK1_LIVING_ARRANGEMENT,
      NK1_PUBLICITY_CODE,
      NK1_PROTECTION_INDICATOR,
      NK1_STUDENT_INDICATOR,
      NK1_RELIGION,
      NK1_MOTHER_MAIDEN_NAME,
      NK1_NATIONALITY,
      NK1_ETHNICITY,
      NK1_CONTACT_REASON,
      NK1_CONTACT_NAME,
      NK1_CONTACT_PHONE,
      NK1_CONTACT_ADDRESS,
      NK1_NK_IDENTIFIER,
      NK1_JOB_STATUS,
      NK1_RACE,
      NK1_HANDICAP,
      NK1_CONTACT_SSN,
      NK1_END,
      NK1_NUMFIELDS
   };

protected:
   static HL7FieldInfo ExpectedFields[NK1_NUMFIELDS];

public:
   virtual int getNumFields() const { return NK1_NUMFIELDS; }
   virtual const HL7FieldInfo *getFieldInfo() const { return ExpectedFields; }
};

//=============================================================================
//
// PV1Segment
//

class PV1Segment : public HL7Segment
{
   DECLARE_MSG_TYPE(PV1Segment, HL7Segment)

public:
   // PV1 fields enumeration
   enum
   {
      PV1_SETID,
      PV1_PATIENT_CLASS,
      PV1_ASSIGNED_LOCATION,
      PV1_ADMISSION_TYPE,
      PV1_PREADMIT_NUMBER,
      PV1_PRIOR_LOCATION,
      PV1_ATTENDING_DOCTOR,
      PV1_REFERRING_DOCTOR,
      PV1_CONSULTING_DOCTOR,
      PV1_HOSPITAL_SERVICE,
      PV1_TEMPORARY_LOCATION,
      PV1_PREADMIT_TEST_INDICATOR,
      PV1_READMISSION_INDICATOR,
      PV1_ADMIT_SOURCE,
      PV1_AMBULATORY_STATUS,
      PV1_VIP_INDICATOR,
      PV1_ADMITTING_DOCTOR,
      PV1_PATIENT_TYPE,
      PV1_VISIT_NUMBER,
      PV1_FINANCIAL_CLASS,
      PV1_CHARGE_PRICE_INDICATOR,
      PV1_COURTESY_CODE,
      PV1_CREDIT_RATING,
      PV1_CONTRACT_CODE,
      PV1_CONTRACT_EFFECTIVE_DATE,
      PV1_CONTRACT_AMOUNT,
      PV1_CONTRACT_PERIOD,
      PV1_INTEREST_CODE,
      PV1_XFER_BAD_DEBT_CODE,
      PV1_XFER_BAD_DEBT_DATE,
      PV1_BAD_DEBT_AGENCY_CODE,
      PV1_BAD_DEBT_XFER_AMOUNT,
      PV1_BAD_DEBT_RECOVERED_AMOUNT,
      PV1_DELETE_ACCOUNT_INDICATOR,
      PV1_DELETE_ACCOUNT_DATE,
      PV1_DISCHARGE_DISPOSITION,
      PV1_DISCHARGED_TO_LOCATION,
      PV1_DIET_TYPE,
      PV1_SERVICING_FACILITY,
      PV1_BED_STATUS,
      PV1_ACCOUNT_STATUS,
      PV1_PENDING_LOCATION,
      PV1_PRIOR_TEMPORARY_LOCATION,
      PV1_ADMIT_TIMESTAMP,
      PV1_DISCHARGE_TIMESTAMP,
      PV1_CURRENT_PATIENT_BALANCE,
      PV1_TOTAL_CHARGE,
      PV1_TOTAL_ADJUSTMENT,
      PV1_TOTAL_PAYMENTS,
      PV1_ALTERNATE_VISIT_ID,
      PV1_VISIT_INDICATOR,
      PV1_OTHER_HEALTHCARE_PROVIDER,
      PV1_END,
      PV1_NUMFIELDS
   };

protected:
   static HL7FieldInfo ExpectedFields[PV1_NUMFIELDS];

public:
   virtual int getNumFields() const { return PV1_NUMFIELDS; }
   virtual const HL7FieldInfo *getFieldInfo() const { return ExpectedFields; }
};

//=============================================================================
//
// GT1Segment
//

class GT1Segment : public HL7Segment
{
   DECLARE_MSG_TYPE(GT1Segment, HL7Segment)

public:
   enum
   {
      GT1_SETID,
      GT1_GUARANTOR_NUMBER,
      GT1_GUARANTOR_NAME,
      GT1_SPOUSE_NAME,
      GT1_ADDRESS,
      GT1_PHONE_HOME,
      GT1_PHONE_BUSINESS,
      GT1_BIRTH_TIMESTAMP,
      GT1_GENDER,
      GT1_GUARANTOR_TYPE,
      GT1_RELATIONSHIP,
      GT1_GUARANTOR_SSN,
      GT1_BEGIN_DATE,
      GT1_END_DATE,
      GT1_PRIORITY,
      GT1_EMPLOYER_NAME,
      GT1_EMPLOYER_ADDRESS,
      GT1_EMPLOYER_PHONE,
      GT1_EMPLOYEE_ID_NUM,
      GT1_EMPLOYMENT_STATUS,
      GT1_ORGANIZATION,
      GT1_BILLING_HOLD_FLAG,
      GT1_CREDIT_RATING,
      GT1_DEATH_TIMESTAMP,
      GT1_DEATH_FLAG,
      GT1_CHARGE_ADJUST_CODE,
      GT1_HOUSEHOLD_ANNUAL_INCOME,
      GT1_HOUSEHOLD_SIZE,
      GT1_EMPLOYER_ID_NUM,
      GT1_MARITAL_STATUS,
      GT1_HIRE_EFFECTIVE_DATE,
      GT1_EMPLOYMENT_STOP_DATE,
      GT1_LIVING_DEPENDENCY,
      GT1_AMBULATORY_STATUS,
      GT1_CITIZENSHIP,
      GT1_PRIMARY_LANGUAGE,
      GT1_LIVING_ARRANGEMENT,
      GT1_PUBLICITY_INDICATOR,
      GT1_PROTECTION_INDICATOR,
      GT1_STUDENT_INDICATOR,
      GT1_RELIGION,
      GT1_MOTHER_MAIDEN_NAME,
      GT1_NATIONALITY,
      GT1_ETHNICITY,
      GT1_CONTACT_NAME,
      GT1_CONTACT_PHONE,
      GT1_CONTACT_REASON,
      GT1_CONTACT_RELATIONSHIP,
      GT1_JOB_TITLE,
      GT1_JOB_CODE,
      GT1_EMPLOYER_ORGANIZATION_NAME,
      GT1_HANDICAP,
      GT1_JOB_STATUS,
      GT1_FINANCIAL_CLASS,
      GT1_RACE,
      GT1_END,
      GT1_NUMFIELDS
   };

protected:
   static HL7FieldInfo ExpectedFields[GT1_NUMFIELDS];

public:
   virtual int getNumFields() const { return GT1_NUMFIELDS; }
   virtual const HL7FieldInfo *getFieldInfo() const { return ExpectedFields; }
};

//=============================================================================
//
// IN1Segment
//

class IN1Segment : public HL7Segment
{
   DECLARE_MSG_TYPE(IN1Segment, HL7Segment)

public:
   // IN1 fields enumeration
   enum
   {
      IN1_SETID,
      IN1_PLANID,
      IN1_COMPANYID,
      IN1_COMPANY_NAME,
      IN1_COMPANY_ADDRESS,
      IN1_COMPANY_CONTACT,
      IN1_COMPANY_PHONE,
      IN1_GROUP_NUMBER,
      IN1_GROUP_NAME,
      IN1_GROUP_EMPLOYER_ID,
      IN1_GROUP_EMPLOYER_NAME,
      IN1_PLAN_EFFECTIVE_DATE,
      IN1_PLAN_EXPIRATION_DATE,
      IN1_AUTHORIZATION_INFO,
      IN1_PLAN_TYPE,
      IN1_INSURED_NAME,
      IN1_INSURED_RELATION,
      IN1_INSURED_DOB,
      IN1_INSURED_ADDRESS,
      IN1_ASSIGNMENT_OF_BENEFITS,
      IN1_COORDINATION_OF_BENEFITS,
      IN1_COORDINATION_OF_BENEFITS_PRIORITY,
      IN1_NOTICE_OF_ADMISSION_FLAG,
      IN1_NOTICE_OF_ADMISSION_DATE,
      IN1_RPT_OF_ELIGIBILITY_FLAG,
      IN1_RPT_OF_ELIGIBILITY_DATE,
      IN1_RELEASE_INFORMATION_CODE,
      IN1_PREADMIT_CERTIFICATION,
      IN1_VERIFICATION_TIMESTAMP,
      IN1_VERIFICATION_BY,
      IN1_TYPE_OF_AGREEMENT_CODE,
      IN1_BILLING_STATUS,
      IN1_LIFETIME_RESERVE_DAYS,
      IN1_DELAY_BEFORE_LR_DAY,
      IN1_COMPANY_PLAN_CODE,
      IN1_POLICY_NUMBER,
      IN1_POLICY_DEDUCTABLE,
      IN1_POLICY_LIMIT_AMOUNT,
      IN1_POLICY_LIMIT_DAYS,
      IN1_ROOM_RATE_SEMIPRIVATE,
      IN1_ROOM_RATE_PRIVATE,
      IN1_INSURED_EMPLOYMENT_STATUS,
      IN1_INSURED_GENDER,
      IN1_INSURED_EMPLOYER_ADDRESS,
      IN1_VERIFICATION_STATUS,
      IN1_PRIOR_INSURANCE_PLAN_ID,
      IN1_COVERAGE_TYPE,
      IN1_HANDICAP,
      IN1_INSURED_ID_NUMBER,
      IN1_END,
      IN1_NUMFIELDS
   };

protected:
   static HL7FieldInfo ExpectedFields[IN1_NUMFIELDS];

public:
   virtual int getNumFields() const { return IN1_NUMFIELDS; }
   virtual const HL7FieldInfo *getFieldInfo() const { return ExpectedFields; }
};

//=============================================================================
//
// IN2Segment
//

class IN2Segment : public HL7Segment
{
   DECLARE_MSG_TYPE(IN2Segment, HL7Segment)

public:
   // IN2 field enumeration - O___o
   enum
   {
      IN2_INSURED_EMPLOYEE_ID,
      IN2_INSURED_SSN,
      IN2_INSURED_EMPLOYER_NAME,
      IN2_EMPLOYER_INFO_DATA,
      IN2_MAIL_CLAIM_PARTY,
      IN2_MEDICARE_CARD_NUMBER,
      IN2_MEDICAID_CASE_NAME,
      IN2_MEDICAID_CASE_NUMBER,
      IN2_CHAMPUS_SPONSOR_NAME,
      IN2_CHAMPUS_ID_NUMBER,
      IN2_DEPENDENT_OF_CHAMPUS_RECIPIENT,
      IN2_CHAMPUS_ORGANIZATION,
      IN2_CHAMPUS_STATION,
      IN2_CHAMPUS_SERVICE,
      IN2_CHAMPUS_RANK_GRADE,
      IN2_CHAMPUS_STATUS,
      IN2_CHAMPUS_RETIRE_DATE,
      IN2_CHAMPUS_NONAVAIL_CERTIFICATION_ON_FILE,
      IN2_BABY_COVERAGE,
      IN2_COMBINE_BABY_BILL,
      IN2_BLOOD_DEDUCTIBLE,
      IN2_SPECIAL_COVERAGE_APPROVAL_NAME,
      IN2_SPECIAL_COVERAGE_APPROVAL_TITLE,
      IN2_NONCOVERED_INSURANCE_CODE,
      IN2_PAYER_ID,
      IN2_PAYER_SUBSCRIBER_ID,
      IN2_ELIGIBILITY_SOURCE,
      IN2_ROOM_COVERAGE_TYPE_AMOUNT,
      IN2_POLICY_TYPE_AMOUNT,
      IN2_DAILY_DEDUCTIBLE,
      IN2_LIVING_DEPENDENCY,
      IN2_AMBULATORY_STATUS,
      IN2_CITIZENSHIP,
      IN2_PRIMARY_LANGUAGE,
      IN2_LIVING_ARRANGEMENT,
      IN2_PUBLICITY_INDICATOR,
      IN2_PROTECTION_INDICATOR,
      IN2_STUDENT_INDICATOR,
      IN2_RELIGION,
      IN2_MOTHER_MAIDEN_NAME,
      IN2_NATIONALITY,
      IN2_ETHNICITY,
      IN2_MARITAL_STATUS,
      IN2_INSURED_EMPLOYMENT_START_DATE,
      IN2_INSURED_EMPLOYMENT_STOP_DATE,
      IN2_JOB_TILE,
      IN2_JOB_CODE_CLASS,
      IN2_JOB_STATUS,
      IN2_EMPLOYER_CONTACT_NAME,
      IN2_EMPLOYER_CONTACT_PHONE,
      IN2_EMPLOYER_CONTACT_REASON,
      IN2_INSURED_CONTACT_PERSON_NAME,
      IN2_INSURED_CONTACT_PERSON_PHONE,
      IN2_INSURED_CONTACT_PERSON_REASON,
      IN2_RELATIONSHIP_TO_PATIENT_START,
      IN2_RELATIONSHIP_TO_PATIENT_STOP,
      IN2_INSURANCE_COMPANY_CONTACT_REASON,
      IN2_INSURANCE_COMPANY_CONTACT_PHONE,
      IN2_POLICY_SCOPE,
      IN2_POLICY_SOURCE,
      IN2_PATIENT_MEMBER_NUMBER,
      IN2_GUARANTOR_RELATIONSHIP_TO_INSURED,
      IN2_INSURED_HOME_PHONE,
      IN2_INSURED_WORK_PHONE,
      IN2_MILITARY_HANDICAP_PROGRAM,
      IN2_SUSPEND_FLAG,
      IN2_COPAY_LIMIT_FLAG,
      IN2_STOPLOSS_LIMIT_FLAG,
      IN2_INSURED_ORGANIZATION_NAME_ID,
      IN2_INSURED_EMPLOYER_ORGANIZATION_NAME_ID,
      IN2_RACE,
      IN2_HCFA_PATIENT_RELATIONSHIP_TO_INSURED,
      IN2_END,
      IN2_NUMFIELDS
   };

protected:
   static HL7FieldInfo ExpectedFields[IN2_NUMFIELDS];

public:
   virtual int getNumFields() const { return IN2_NUMFIELDS; }
   virtual const HL7FieldInfo *getFieldInfo() const { return ExpectedFields; }
};

//=============================================================================
//
// ZALSegment
//

class ZALSegment : public HL7Segment
{
   DECLARE_MSG_TYPE(ZALSegment, HL7Segment)

public:
   enum
   {
      ZAL_UNKNOWN_DATE,
      ZAL_END,
      ZAL_NUMFIELDS
   };

protected:
   static HL7FieldInfo ExpectedFields[ZAL_NUMFIELDS];

public:
   virtual int getNumFields() const { return ZAL_NUMFIELDS; }
   virtual const HL7FieldInfo *getFieldInfo() const { return ExpectedFields; }
};

#endif

// EOF


