
#include "scanner.h"

//---------------------------------------------------------------------------
bool IsDirectory(const AnsiString &filename)
{
  WIN32_FILE_ATTRIBUTE_DATA attrib;
  GetFileAttributesEx(filename.c_str(), GetFileExInfoStandard, (void *)(&attrib));
  return (attrib.dwFileAttributes & FILE_ATTRIBUTE_DIRECTORY);
};
//---------------------------------------------------------------------------
void format_date(const FILETIME *ft, AnsiString &outstr)
{
  SYSTEMTIME temp_time;
  FileTimeToSystemTime(ft, &temp_time);
  int bufsize = GetDateFormat(LOCALE_USER_DEFAULT, NULL, &temp_time, NULL, NULL, 0);
  char *buf = new char[bufsize];
  GetDateFormat(LOCALE_USER_DEFAULT, NULL, &temp_time, NULL, buf, bufsize);
  outstr = buf;
  delete [] buf; // jhaley - bug fix: memory leak.
}
//---------------------------------------------------------------------------
void format_time(const FILETIME *ft, AnsiString &outstr)
{
  SYSTEMTIME temp_time;
  FileTimeToSystemTime(ft, &temp_time);
  int bufsize = GetTimeFormat(LOCALE_USER_DEFAULT, NULL, &temp_time, NULL, NULL, 0);
  char *buf = new char[bufsize];
  GetTimeFormat(LOCALE_USER_DEFAULT, NULL, &temp_time, NULL, buf, bufsize);
  outstr = buf;
  delete [] buf; // jhaley - bug fix: memory leak.
}
//---------------------------------------------------------------------------
void get_file_specs(const WIN32_FIND_DATA &filename, fileinfo &info)
{
  info.filename = (filename.cFileName);
  format_date(&(filename.ftCreationTime), info.creation_date);
  format_time(&(filename.ftCreationTime), info.creation_time);
}
//---------------------------------------------------------------------------
// non-recursive (uses the queue you give it), uses Builder TSearchRec, etc.
void ScanForFiles(queue<AnsiString> &paths_to_scan, GenericFunctor &functor)
{
  while (!paths_to_scan.empty())
  {
    AnsiString working_dir = IncludeTrailingBackslash(paths_to_scan.front());
    TSearchRec sr;
    // Application->MessageBox(working_dir.c_str(), "Searching in ...", MB_OK);
    if (!FindFirst(working_dir + "*", faAnyFile, sr))
    {
      do
   	{
        if (sr.Name != "." && sr.Name != "..")
        {
          if (sr.Attr & faDirectory)
          {
            paths_to_scan.push(working_dir + sr.Name);
          } else
          {
            fileinfo fi;
            get_file_specs(sr.FindData, fi);
            fi.path = working_dir;
            functor(fi);
          }
        }
      } while (!FindNext(sr) /* && search_state == searching */);
      FindClose(sr);
    }
    paths_to_scan.pop();
  }
}
//---------------------------------------------------------------------------
void ScanForFiles(queue<AnsiString> &paths_to_scan, AddPath add_callback)
{
  class StaticFunctor : public GenericFunctor
  {
  public:
     AddPath static_func;
     StaticFunctor(AddPath pfunc) : static_func(pfunc) {}
     virtual void operator ()(fileinfo &fi) { static_func(fi); }
  };

  StaticFunctor functor = StaticFunctor(add_callback);
  ScanForFiles(paths_to_scan, functor);
}
//---------------------------------------------------------------------------
void ScanForFiles(queue<AnsiString> &paths_to_scan, TObject* member, MemberAddPath add_callback)
{
  class TObjectFunctor : public GenericFunctor
  {
  public:
    TObject *obj;
    MemberAddPath member_func;
    TObjectFunctor(TObject *pobj, MemberAddPath pfunc) : obj(pobj), member_func(pfunc) {}
    virtual void operator ()(fileinfo &fi) { (obj->*member_func)(fi); }
  };

  TObjectFunctor functor = TObjectFunctor(member, add_callback);
  ScanForFiles(paths_to_scan, functor);
}
//---------------------------------------------------------------------------
// jhaley - Because, why not? :P
void ScanForFiles(queue<AnsiString> &paths_to_scan, ClosureAddPath add_callback)
{
   class ClosureFunctor : public GenericFunctor
   {
   public:
      ClosureAddPath closure_func;
      ClosureFunctor(ClosureAddPath pcf) : closure_func(pcf) {}
      virtual void operator ()(fileinfo &fi) { closure_func(fi); }
   };

   ClosureFunctor functor = ClosureFunctor(add_callback);
   ScanForFiles(paths_to_scan, functor);
}
//---------------------------------------------------------------------------
