//DEIMOS
#pragma hdrstop
#include "sqlLib.h"
#include "utils.h"

// jhaley 20120620: HL7Daemon has no TApplication instance so we can't use it
// to output debug messages. Instead we need to route them to the debug output
// used by the rest of the application.
#define NO_TAPPLICATION_INSTANCE

#ifndef NO_TAPPLICATION_INSTANCE
#define AppDebugMsg(msg, cat, opt) Application->MessageBox(msg, cat, opt)
#else
#define AppDebugMsg(msg, cat, opt) DEBUGOUT(dbg_sql, cat << ": " << msg)
#endif


//---------------------------------------------------------------------------
sql_error last_sql_lib_error;
//---------------------------------------------------------------------------
// jhaley 20110318: Connect to a database
bool ConnectToDatabase(TIBDatabase *dbDatabase, const string &server,
                       const string &user_name, const string &password)
{
  TStringList *loginInfo = new TStringList;
  bool success = false;

  try
  {
    if (!dbDatabase->TestConnected())
    {
      loginInfo->Add(("user_name=" + user_name).c_str());
      loginInfo->Add(("password="  +  password).c_str());
      dbDatabase->DatabaseName = server.c_str();
      dbDatabase->Params       = loginInfo;
      dbDatabase->SQLDialect   = 3;
      dbDatabase->LoginPrompt  = false;
      dbDatabase->Connected    = true;
    }
    success = true;
  }
  catch(EIBError &error)
  {
    #ifdef use_tcpip_messaging
    if (debug_level[libraries]>=1 && tcpip_message_router)
      tcpip_message_router->SendMessageToAll("debug", "Error connecting to database " + server + ":\n" + error.Message.c_str());
    #endif
    last_sql_lib_error = error;
  }

  delete loginInfo;

  return success;
}
//--------------------------------------------------------------------------
bool LockRecordForUpdate (TIBTransaction* dbTransaction, string tableName, string id)
{
  #ifdef use_tcpip_messaging
  if(debug_level[libraries]>=2 && tcpip_message_router) tcpip_message_router->SendMessageToAll("debug", "Locking table " + tableName + " id " + id + " for update.");
  #endif
  if(dbTransaction->DefaultDatabase->TestConnected())
  {
    bool can_commit = true;
    if (dbTransaction->Active)
      can_commit = false;
    TIBSQL * dbSQL = new TIBSQL(NULL);
    string sqlstring;
    sqlstring = "update " + tableName + " set id = " + id + " where id = " + id;
    try
    {
      dbSQL->Database = dbTransaction->DefaultDatabase;
      dbSQL->Transaction = dbTransaction;
      if (can_commit)
        dbTransaction->StartTransaction();
      dbSQL->SQL->Text = sqlstring.c_str();
      dbSQL->ExecQuery();
      delete dbSQL;
      return true;
    }
    catch(EIBError &error)
    {
      #ifdef use_tcpip_messaging
      if(debug_level[libraries]>=1 && tcpip_message_router) tcpip_message_router->SendMessageToAll("debug", "Error locking table " + tableName + " id " + id + " for update:\n" + error.Message.c_str());
      #endif
      last_sql_lib_error = error;
      if(dbTransaction->Active == true && can_commit)
        dbTransaction->Rollback();
      delete dbSQL;
      return false;
    }
  }
  else
    return false;
}
//--------------------------------------------------------------------------
bool ExecuteUpdateStatement(TIBTransaction* dbTransaction, string tableName, map<string, string> fieldMap, map<string, int> field_options, bool show_error)
{
  #ifdef use_tcpip_messaging
  if(debug_level[libraries]>=2 && tcpip_message_router) tcpip_message_router->SendMessageToAll("debug", "Executing update statement on " + tableName + " with fieldMap:\n " + MapToCommaString(fieldMap) + " \nand show_error of " + IntToString(show_error));
  #endif                                                                                                                                                                                                           /*field_options:\n" + MapToDelimString(field_options, "{", "}", "", "'", "'", "'", "'", ",") + "'") + "\nand */
  if(dbTransaction->DefaultDatabase->TestConnected())
  {
    TIBSQL * dbSQL = new TIBSQL(NULL);
    string sqlstring;
    bool returnValue;

    bool can_commit = true;
    if (dbTransaction->Active)
     can_commit = false;

    // create sql statement:
    // update tableName set field = value, field = value, (...) where id = id_number
    sqlstring = "update " + tableName + " set ";
    for (map<string, string>::iterator itr = fieldMap.begin(); itr != fieldMap.end(); itr++)
    {
      sqlstring += itr->first;
      sqlstring += " = ";
      if (itr->second.empty())
      {
       sqlstring += "NULL";
      }
      else
      {
        map<string, int>::iterator opt_itr = field_options.find(itr->first);
        if (opt_itr == field_options.end())
         sqlstring += "'" + StripLeading(StripTrailing(ReplaceStringWithString(UppercaseString(itr->second), "'", "''"), " "), " ") + "'";
        else
        {
          string temp_string = "";
          int opts = opt_itr->second;
          if ((opts & sql_no_uppercasing) == 0)
            temp_string = UppercaseString(itr->second);
          else
            temp_string = itr->second;
          if ((opts & sql_no_left_trimming) == 0)
            temp_string = StripLeading(temp_string, " ");
          if ((opts & sql_no_right_trimming) == 0)
            temp_string = StripTrailing(temp_string, " ");
          if ((opts & sql_no_quoting) == 0)
            temp_string = "'" + ReplaceStringWithString(temp_string, "'", "''") + "'";
          sqlstring += temp_string;
        }
      }
      sqlstring += ", ";
    }
    if(sqlstring != "")
    {
      sqlstring = sqlstring.substr(0, (sqlstring.length()-2));
      sqlstring += " where id = " + fieldMap["id"];
      // try to execute statement (transaction should already be started from
      // locking the record for update)
      try
      {
        dbSQL->Database = dbTransaction->DefaultDatabase;
        dbSQL->Transaction = dbTransaction;
        if (can_commit)
         dbTransaction->StartTransaction();
        dbSQL->SQL->Text = sqlstring.c_str();
        dbSQL->ExecQuery();
        if (can_commit)
         dbTransaction->Commit();
        returnValue = true;
        #ifdef use_tcpip_messaging
        if(debug_level[libraries]>=4 && tcpip_message_router) tcpip_message_router->SendMessageToAll("debug", "Successfully executed update statement on " + tableName + " with fieldMap:\n " + MapToCommaString(fieldMap));
        #endif
      }
      catch(EIBError &the_error)
      {
        #ifdef use_tcpip_messaging
        if(debug_level[libraries]>=1 && tcpip_message_router) tcpip_message_router->SendMessageToAll("debug", "Failed to execute update statement on " + tableName + " with fieldMap:\n " + MapToCommaString(fieldMap));
        #endif
        last_sql_lib_error = the_error;
        if(dbTransaction->Active && can_commit)
          dbTransaction->Rollback();
        returnValue = false;
        if(show_error)
        {
          string message = the_error.Message.c_str();
          string library = "sqlLib:SqlExecuteUpdateStatement";
          string extended_info = "";
          #ifdef use_prometheus_util
          PSay(message, m_database_error, MB_OK, library, extended_info);
          #else
          AppDebugMsg(message.c_str(), "debug", MB_OK);
          #endif
        }
      }
      catch(...)
      {
        #ifdef use_tcpip_messaging
        if(debug_level[libraries]>=1 && tcpip_message_router) tcpip_message_router->SendMessageToAll("debug", "Failed to execute update statement on " + tableName + " with fieldMap:\n " + MapToCommaString(fieldMap) + "\n\nUnknown error\n");
        #endif
        if(dbTransaction->Active && can_commit)
          dbTransaction->Rollback();
        returnValue = false;
      }
    }
    else
      returnValue = true;
    delete dbSQL;
    return returnValue;
  }
  else
    return false;
}
//--------------------------------------------------------------------------
bool ExecuteUpdateStatement(TIBDatabase* dbDatabase, string tableName, map<string, string> fieldMap, map<string, int> field_options, bool show_error)
{
  if(dbDatabase->TestConnected())
  {
    bool return_value = true;
    TIBTransaction *dbTransaction = new TIBTransaction(NULL);
    try
    {
      StdTransaction (dbTransaction, dbDatabase, false);
      return_value = ExecuteUpdateStatement(dbTransaction, tableName, fieldMap, field_options, show_error);
    }
    catch(...)
    {
      if(dbTransaction->Active)
        dbTransaction->Rollback();
      return_value = false;
    }
    delete dbTransaction;
    return return_value;
  }
  else
    return false;
}
//--------------------------------------------------------------------------
bool ExecuteInsertStatement(TIBTransaction* dbTransaction, string tableName, map<string, string> fieldMap, map<string, int> field_options, bool show_error)
{
  #ifdef use_tcpip_messaging
  if(debug_level[libraries]>=2 && tcpip_message_router) tcpip_message_router->SendMessageToAll("debug", "Executing insert statement on " + tableName + " with fieldMap:\n " + MapToCommaString(fieldMap) + " \nand show_error of " + IntToString(show_error));
  #endif                                                                                                                                                                                                           /* field_options:\n" + SetToCommaString(fields_not_to_uppercase) + "\nand */
  if(dbTransaction->DefaultDatabase->TestConnected())
  {
    bool can_commit = true;

    if(dbTransaction->Active)
      can_commit = false;

    TIBSQL * dbSQL = new TIBSQL(NULL);
    string sqlstring = "";
    string fieldList = "";
    string valueList = "";

    for(map<string, string>::iterator itr = fieldMap.begin();
        itr != fieldMap.end(); itr++)
    {
      fieldList += itr->first;
      fieldList += ", ";
      if (itr->second.empty())
      {
       valueList += "NULL";
      }
      else
      {
        map<string, int>::iterator opt_itr = field_options.find(itr->first);
        if (opt_itr == field_options.end())
        {
          valueList += "'" + StripLeading(StripTrailing(ReplaceStringWithString(UppercaseString(itr->second), "'", "''"), " "), " ") + "'";
        }
        else
        {
          string temp_string = "";
          int opts = opt_itr->second;
          if ((opts & sql_no_uppercasing) == 0)
            temp_string = UppercaseString(itr->second);
          else
            temp_string = itr->second;
          if ((opts & sql_no_left_trimming) == 0)
            temp_string = StripLeading(temp_string, " ");
          if ((opts & sql_no_right_trimming) == 0)
            temp_string = StripTrailing(temp_string, " ");
          if ((opts & sql_no_quoting) == 0)
            temp_string = "'" + ReplaceStringWithString(temp_string, "'", "''") + "'";
          valueList += temp_string;
        }
      }
      valueList += ", ";
    }
    if (fieldList != "")
    {
      fieldList = fieldList.substr(0,(fieldList.length()-2));
      valueList = valueList.substr(0, (valueList.length()-2));
      sqlstring = "insert into " + tableName + "(" + fieldList + ") values (" + valueList + ")";
      try
      {
        //SendMessage (optionsIni["server"]["sql_put"], optionsIni["server"]["user_name"] + " <insert> : " + sqlstring);
        dbSQL->Database = dbTransaction->DefaultDatabase;
        dbSQL->Transaction = dbTransaction;
        if(can_commit)
          dbTransaction->StartTransaction();
        dbSQL->SQL->Text = sqlstring.c_str();
        dbSQL->ExecQuery();
        if(can_commit)
          dbTransaction->Commit();
        delete dbSQL;
        #ifdef use_tcpip_messaging
        if(debug_level[libraries]>=4 && tcpip_message_router) tcpip_message_router->SendMessageToAll("debug", "Successfully executed insert statement on " + tableName + " with fieldMap:\n " + MapToCommaString(fieldMap));
        #endif
        return true;
      }
      catch(EIBError &the_error)
      {
        #ifdef use_tcpip_messaging
        if(debug_level[libraries]>=1 && tcpip_message_router) tcpip_message_router->SendMessageToAll("debug", "Failed to execute insert statement on " + tableName + " with fieldMap:\n " + MapToCommaString(fieldMap) + "\n\n" + the_error.Message.c_str());
        #endif
        last_sql_lib_error = the_error;
        if(dbTransaction->Active && can_commit)
          dbTransaction->Rollback();
        delete dbSQL;
        if(show_error)
        {
          string message = the_error.Message.c_str();
          string library = "sqlLib:SqlExecuteInsertStatement";
          string extended_info = "";
          #ifdef use_prometheus_util
          PSay(message, m_database_error, MB_OK, library, extended_info);
          #else
          AppDebugMsg(message.c_str(), "debug", MB_OK);
          #endif
        }
        return false;
      }
      catch(...)
      {
        #ifdef use_tcpip_messaging
        if(debug_level[libraries]>=1 && tcpip_message_router) tcpip_message_router->SendMessageToAll("debug", "Failed to execute insert statement on " + tableName + " with fieldMap:\n " + MapToCommaString(fieldMap) + "\n\nUnknown error\n");
        #endif
        if(dbTransaction->Active && can_commit)
          dbTransaction->Rollback();
        delete dbSQL;
        return false;
      }
    }
    return true;
  }
  else
    return false;
}
//--------------------------------------------------------------------------
bool ExecuteInsertStatement(TIBDatabase* dbDatabase, string tableName, map<string, string> fieldMap, map<string, int> field_options, bool show_error)
{
  if(dbDatabase->TestConnected())
  {
    bool return_value = true;
    TIBTransaction *dbTransaction = new TIBTransaction(NULL);
    try
    {
      StdTransaction (dbTransaction, dbDatabase, false);
      return_value = ExecuteInsertStatement(dbTransaction, tableName, fieldMap, field_options, show_error);
    }
    catch(...)
    {
      if(dbTransaction->Active)
        dbTransaction->Rollback();
      return_value = false;
    }
    delete dbTransaction;
    return return_value;
  }
  else
    return false;
}
//--------------------------------------------------------------------------
string GetNextId (TIBTransaction* dbTransaction, string tableName)
{
  #ifdef use_tcpip_messaging
  if(debug_level[libraries]>=2 && tcpip_message_router) tcpip_message_router->SendMessageToAll("debug", "Getting next ID on table " + tableName);
  #endif
  if(dbTransaction->DefaultDatabase->TestConnected())
  {
    TIBDataSet * dbDataSet = new TIBDataSet(NULL);
    TStringList * sqlstmt = new TStringList;
    string returnString;
    string sqlstring = "select gen_id(" + tableName + "_gen, 1) from uno";

    sqlstmt->Add (sqlstring.c_str());
    try
    {
      dbDataSet->Database = dbTransaction->DefaultDatabase;;
      dbDataSet->Transaction = dbTransaction;
      dbDataSet->UniDirectional = false;
      dbDataSet->SelectSQL = sqlstmt;
      dbDataSet->Open();
      dbDataSet->First();
      if (dbDataSet->Eof == false)
      {
        returnString = dbDataSet->Fields->Fields[0]->AsString.c_str();
        #ifdef use_tcpip_messaging
        if(debug_level[libraries]>=4 && tcpip_message_router) tcpip_message_router->SendMessageToAll("debug", "Got next ID on table " + tableName + ":  " + returnString);
        #endif
      }
      else
        returnString = "";
      dbDataSet->Close();
    }
    catch (EIBError & error)
    {
      #ifdef use_tcpip_messaging
      if(debug_level[libraries]>=1 && tcpip_message_router) tcpip_message_router->SendMessageToAll("debug", "Error getting next ID on table " + tableName + ":\n" + error.Message.c_str());
      #endif
      last_sql_lib_error = error;
      returnString = "";
    }
    catch (...)
    {
      #ifdef use_tcpip_messaging
      if(debug_level[libraries]>=1 && tcpip_message_router) tcpip_message_router->SendMessageToAll("debug", "Error getting next ID on table " + tableName + ":  Unknown error");
      #endif
      returnString = "";
    }

    delete dbDataSet;
    delete sqlstmt;
    return returnString;
  }
  else
    return "";
}
//--------------------------------------------------------------------------
string GetNextId (TIBDatabase* dbDatabase, string tableName)
{
  string returnString = "";
  if(dbDatabase->TestConnected())
  {
    TIBTransaction * dbTransaction = new TIBTransaction(NULL);
    try
    {
      StdTransaction (dbTransaction, dbDatabase, false);
      returnString = GetNextId(dbTransaction, tableName);
    }
    catch (...)
    {
      if (dbTransaction->Active)
        dbTransaction->Rollback();
    }
    delete dbTransaction;
  }
  return returnString;
}
//--------------------------------------------------------------------------
string GetOneField(TIBTransaction * dbTransaction, string sqlstring)
{
  #ifdef use_tcpip_messaging
  if(debug_level[libraries]>=2 && tcpip_message_router) tcpip_message_router->SendMessageToAll("debug", "Getting one field with SQL:\n" + sqlstring);
  #endif
  string returnString = "";
  if(dbTransaction->DefaultDatabase->TestConnected())
  {
    bool canCommit = true;
    TIBDataSet * dbDataSet = new TIBDataSet(NULL);
    TStringList * sqlstmt = new TStringList;
    sqlstmt->Add(sqlstring.c_str());
    try
    {
      dbDataSet->Database = dbTransaction->DefaultDatabase;
      dbDataSet->Transaction = dbTransaction;
      dbDataSet->UniDirectional = false;
      if (dbTransaction->Active)
        canCommit = false;
      else
        dbTransaction->StartTransaction();
      dbDataSet->SelectSQL = sqlstmt;
      dbDataSet->Open();
      dbDataSet->First();
      if (dbDataSet->Eof == false)
      {
        returnString = dbDataSet->Fields->Fields[0]->AsString.c_str();
        #ifdef use_tcpip_messaging
        if(debug_level[libraries]>=4 && tcpip_message_router) tcpip_message_router->SendMessageToAll("debug", "Got one field:  " + returnString);
        #endif
      }
      dbDataSet->Close();
      if (canCommit)
        dbTransaction->Commit();
    }
    catch (EIBError & error)
    {
      #ifdef use_tcpip_messaging
      if(debug_level[libraries]>=1 && tcpip_message_router) tcpip_message_router->SendMessageToAll("debug", "Error getting one field:\n" + string(error.Message.c_str()));
      #endif
      last_sql_lib_error = error;
    }
    catch (...)
    {
      #ifdef use_tcpip_messaging
      if(debug_level[libraries]>=1 && tcpip_message_router) tcpip_message_router->SendMessageToAll("debug", "Error getting one field:  Unknown error");
      #endif
    }
    if ((canCommit) && (dbTransaction->Active))
      dbTransaction->Rollback();
    delete dbDataSet;
    delete sqlstmt;
  }
  #ifdef use_tcpip_messaging
  if(debug_level[libraries]>=4 && tcpip_message_router) tcpip_message_router->SendMessageToAll("debug", "GetOneField result: '" + returnString + "'");
  #endif
  return returnString;
}
//--------------------------------------------------------------------------
string GetOneField(TIBDatabase * dbDatabase, string sqlstring)
{
  string returnString = "";
  if(dbDatabase->TestConnected())
  {
    TIBTransaction * dbTransaction = new TIBTransaction(NULL);
    try
    {
      StdTransaction (dbTransaction, dbDatabase, false);
      returnString = GetOneField(dbTransaction, sqlstring);
    }
    catch (...)
    {
      if (dbTransaction->Active)
        dbTransaction->Rollback();
    }
    delete dbTransaction;
  }
  return returnString;
}
//--------------------------------------------------------------------------
bool FillSKVTable (TIBTransaction* dbTransaction, string sql, map<string, map<string, string> > & section_key_value_map)
{
  #ifdef use_tcpip_messaging
  if(debug_level[libraries]>=2 && tcpip_message_router) tcpip_message_router->SendMessageToAll("debug", "Filling SKV table with sql:\n" + sql);
  #endif
  if(dbTransaction->DefaultDatabase->TestConnected())
  {
   // empty map
   section_key_value_map.clear();

   bool canCommit = true;
   TIBDataSet * dbDataSet = new TIBDataSet(NULL);
   TStringList * sqlstmt = new TStringList;
   sqlstmt->Add(sql.c_str());
   try
   {
    dbDataSet->Database = dbTransaction->DefaultDatabase;
    dbDataSet->Transaction = dbTransaction;
    dbDataSet->UniDirectional = false;
    if (dbTransaction->Active)
     canCommit = false;
    else
     dbTransaction->StartTransaction();
    dbDataSet->SelectSQL = sqlstmt;
    dbDataSet->Open();
    dbDataSet->First();
    // skv processing
    while (!dbDataSet->Eof)
    {
     string v_section = LowercaseString(dbDataSet->Fields->Fields[0]->AsString.c_str());
     string v_key     = LowercaseString(dbDataSet->Fields->Fields[1]->AsString.c_str());
     string v_value   = LowercaseString(dbDataSet->Fields->Fields[2]->AsString.c_str());
     section_key_value_map[v_section][v_key] = v_value;
     dbDataSet->Next();
    }
    // skv processing
    dbDataSet->Close();
    if (canCommit)
     dbTransaction->Commit();
    #ifdef use_tcpip_messaging
    if(debug_level[libraries]>=1 && tcpip_message_router) tcpip_message_router->SendMessageToAll("debug", "Filled SKV table with sql:\n" + sql);
    #endif
   }
   catch (EIBError & error)
   {
    #ifdef use_tcpip_messaging
    if(debug_level[libraries]>=1 && tcpip_message_router) tcpip_message_router->SendMessageToAll("debug", "Error filling SKV table with sql:\n" + sql + "\n" + error.Message.c_str());
    #endif
    last_sql_lib_error = error;
   }
   catch (...)
   {
    #ifdef use_tcpip_messaging
    if(debug_level[libraries]>=1 && tcpip_message_router) tcpip_message_router->SendMessageToAll("debug", "Error filling SKV table with sql:\n" + sql + "\nUnknown error");
    #endif
   }
   if ((canCommit) && (dbTransaction->Active))
    dbTransaction->Rollback();
   delete dbDataSet;
   delete sqlstmt;
   return true;
  }
  else
   return false;
}
//--------------------------------------------------------------------------
string GetPersonName(TIBTransaction * dbTransaction, string person_id, int order)
{
  #ifdef use_tcpip_messaging
  if(debug_level[libraries]>=2 && tcpip_message_router) tcpip_message_router->SendMessageToAll("debug", "Getting person name for ID " + person_id + " in order " + IntToString(order));
  #endif
  if(dbTransaction->DefaultDatabase->TestConnected())
  {
    string returnString = "";
    if ((person_id != "0") && (person_id != "") && (person_id != "new"))
    {
      map<string, string> name_map;
      string sql = "select PN.prefix, PN.first_name, PN.middle_name, PN.last_name, PN.suffix "
                   "from people P inner join person_names PN on P.person_names_id = PN.id "
                   "where P.id = " + person_id;
      if (SqlToMap(dbTransaction, sql, name_map))
      {
        if (!name_map.empty())
        {
          if (order == last_first || order == last_first_middle)
          {
            returnString = name_map["last_name"] + (name_map["last_name"].size() > 0 ? ", " : "")
                         + name_map["first_name"];
            returnString += ((returnString.size() > 0 && returnString[returnString.length()-1] != ' ' ? " " : "")
                             + (order == last_first_middle ? name_map["middle_name"] : string("")));
            returnString += ((returnString.size() > 0 && returnString[returnString.length()-1] != ' ' ? " " : "")
                             + name_map["suffix"]);
          }
          else if (order == first_last || order == first_middle_last)
          {
            returnString = name_map["first_name"];
            returnString += (returnString.size() > 0 && returnString[returnString.length()-1] != ' ' ? " " : "")
                         + (order == first_middle_last ? name_map["middle_name"] : string(""));
            returnString += ((returnString.size() > 0 && returnString[returnString.length()-1] != ' ' ? " " : "")
                              + name_map["last_name"]);
            returnString += ((returnString.size() > 0 && returnString[returnString.length()-1] != ' ' ? " " : "")
                              + name_map["suffix"]);
          }

          returnString = PrettyString(returnString);

          if (order == first_last_initial)
          {
            returnString = PrettyString(name_map["first_name"]);
            returnString += ((returnString.size() > 0 && returnString[returnString.length()-1] != ' ' ? " " : "")
                              + AcronymString(name_map["last_name"]));
          }

          returnString = StripTrailing(returnString, " ");
          returnString = StripLeading(returnString, " ");
        }
        else
          returnString = "None";
      }
    }
    else
      returnString = "None";
    #ifdef use_tcpip_messaging
    if(debug_level[libraries]>=4 && tcpip_message_router) tcpip_message_router->SendMessageToAll("debug", "Got person name for ID " + person_id + " in order " + IntToString(order) + ":  " + returnString);
    #endif
    return returnString;
  }
  else
    return "";
}
//--------------------------------------------------------------------------
string GetPersonName(TIBDatabase * dbDatabase, string person_id, int order)
{
  string returnString = "";
  if(dbDatabase->TestConnected())
  {
    TIBTransaction * dbTransaction = new TIBTransaction(NULL);
    try
    {
      StdTransaction (dbTransaction, dbDatabase, false);
      returnString = GetPersonName(dbTransaction, person_id, order);
    }
    catch (...)
    {
      if (dbTransaction->Active)
        dbTransaction->Rollback();
    }
    delete dbTransaction;
  }
  return returnString;
}
//--------------------------------------------------------------------------
string GetUserName(TIBTransaction * dbTransaction, string user_id, int order)
{
  #ifdef use_tcpip_messaging
  if(debug_level[libraries]>=2 && tcpip_message_router) tcpip_message_router->SendMessageToAll("debug", "Getting user name for ID " + user_id + " in order " + IntToString(order));
  #endif
  if(dbTransaction->DefaultDatabase->TestConnected())
  {
    string returnString = "";
    if ((user_id != "0") && (user_id != "") && (user_id != "new"))
    {
      map<string, string> name_map;
      string sql = "select person_id from users where id = " + user_id;
      string person_id = GetOneField(dbTransaction, sql);
      if (!IsBlankOrZero(person_id))
        returnString = GetPersonName(dbTransaction, person_id, order);
      else
        returnString = "<unknown>";
    }
    else
      returnString = "None";

    #ifdef use_tcpip_messaging
    if(debug_level[libraries]>=4 && tcpip_message_router) tcpip_message_router->SendMessageToAll("debug", "Got user name for ID " + user_id + " in order " + IntToString(order) + ":  " + returnString);
    #endif
    return returnString;
  }

  return "";
}
//--------------------------------------------------------------------------
string GetUserName(TIBDatabase * dbDatabase, string user_id, int order)
{
  string returnString = "";
  if(dbDatabase->TestConnected())
  {
    TIBTransaction * dbTransaction = new TIBTransaction(NULL);
    try
    {
      StdTransaction (dbTransaction, dbDatabase, false);
      returnString = GetUserName(dbTransaction, user_id, order);
    }
    catch (...)
    {
      if (dbTransaction->Active)
        dbTransaction->Rollback();
    }
    delete dbTransaction;
  }
  return returnString;
}
//--------------------------------------------------------------------------
bool ExecuteStatement(TIBTransaction * dbTransaction, string sqlstring)
{
  #ifdef use_tcpip_messaging
  if(debug_level[libraries]>=2 && tcpip_message_router) tcpip_message_router->SendMessageToAll("debug", "Executing SQL:\n" + sqlstring);
  #endif
  if(dbTransaction->DefaultDatabase->TestConnected())
  {
    bool canCommit = true;
    bool toReturn = true;
    TIBSQL * dbSQL = new TIBSQL(NULL);
    if (dbTransaction->Active)
      canCommit = false;

    try
    {
      dbSQL->Database = dbTransaction->DefaultDatabase;
      dbSQL->Transaction = dbTransaction;
      if (canCommit)
        dbTransaction->StartTransaction();
      dbSQL->SQL->Text = sqlstring.c_str();
      dbSQL->ExecQuery();
      if (canCommit)
        dbTransaction->Commit();
      toReturn = true;
      #ifdef use_tcpip_messaging
      if(debug_level[libraries]>=4 && tcpip_message_router) tcpip_message_router->SendMessageToAll("debug", "Executed SQL:\n" + sqlstring);
      #endif
    }
    catch (EIBError & error)
    {
      #ifdef use_tcpip_messaging
      if(debug_level[libraries]>=1 && tcpip_message_router) tcpip_message_router->SendMessageToAll("debug", "Error executing SQL:\n" + sqlstring + "\n\n" + error.Message.c_str());
      #endif
      last_sql_lib_error = error;
      toReturn = false;
    }
    catch (...)
    {
      #ifdef use_tcpip_messaging
      if(debug_level[libraries]>=1 && tcpip_message_router) tcpip_message_router->SendMessageToAll("debug", "Error executing SQL:\n" + sqlstring + "\n\nUnknown error\n");
      #endif
      toReturn = false;
    }
    if ((canCommit) && (dbTransaction->Active))
      dbTransaction->Rollback();
    delete dbSQL;
    return toReturn;
  }
  else
    return false;
}
//--------------------------------------------------------------------------
bool ExecuteStatement(TIBDatabase * dbDatabase, string sqlstring)
{
  if(dbDatabase->TestConnected())
  {
    bool toReturn = true;
    TIBTransaction * dbTransaction = new TIBTransaction(NULL);
    try
    {
      StdTransaction (dbTransaction, dbDatabase, false);
      toReturn = ExecuteStatement(dbTransaction, sqlstring);
    }
    catch (...)
    {
      toReturn = false;
      if (dbTransaction->Active)
        dbTransaction->Rollback();
    }
    delete dbTransaction;
    return toReturn;
  }
  else
    return false;
}
//--------------------------------------------------------------------------
bool SqlToMap(TIBTransaction * dbTransaction, string sql, map<string, string> &field_map)
{
  #ifdef use_tcpip_messaging
  if(debug_level[libraries] >= 2 && tcpip_message_router)
    tcpip_message_router->SendMessageToAll("debug", "SqlToMap for sql\n" + sql);
  #endif
  if(dbTransaction->DefaultDatabase->TestConnected())
  {
    TIBDataSet * dbDataSet = new TIBDataSet(NULL);
    string field_name;
    bool toReturn = true;
    bool can_commit = true;
    if (dbTransaction->Active)
      can_commit = false;
    field_map.clear();
    try
    {
      dbDataSet->Database = dbTransaction->DefaultDatabase;
      dbDataSet->Transaction = dbTransaction;
      dbDataSet->UniDirectional = false;
      if (can_commit)
        dbTransaction->StartTransaction();
      dbDataSet->SelectSQL->Clear();
      dbDataSet->SelectSQL->Add(sql.c_str());

      dbDataSet->Open();
      dbDataSet->First();
      if (! dbDataSet->Eof)
      {
        for (int i = 0; i < dbDataSet->Fields->Count; i++)
        {
          field_name = LowercaseString(ws(dbDataSet->Fields->Fields[i]->FieldName));
          field_map[field_name] = dbDataSet->FieldByName(field_name.c_str())->AsString.c_str();
        }
      }
      #ifdef use_tcpip_messaging
      if(debug_level[libraries]>=1 && tcpip_message_router)
      {
        dbDataSet->Next();
        if (! dbDataSet->Eof)
          tcpip_message_router->SendMessageToAll("debug", "Warning:  Extra rows discarded for SqlToMap dataset; sql\n" + sql);
      }
      #endif
      dbDataSet->Close();
      if (can_commit)
        dbTransaction->Commit();
      toReturn = true;
      #ifdef use_tcpip_messaging
      if(debug_level[libraries] >= 4 && tcpip_message_router)
      {
        tcpip_message_router->SendMessageToAll("debug", "SqlToMap result: '"
          + MapToDelimString(field_map, "{", "}", "", "'", "'", "'", "'", ",") + "'");
      }
      #endif
    }
    catch (EIBError & error)
    {
      #ifdef use_tcpip_messaging
      if(debug_level[libraries]>=1 && tcpip_message_router)
        tcpip_message_router->SendMessageToAll("debug", "Error in SqlToMap for sql:\n" + sql + "\n\n" + error.Message.c_str());
      #endif
      last_sql_lib_error = error;
      toReturn = false;
      if ((dbTransaction->Active) && (can_commit))
       dbTransaction->Rollback();
    }
    catch (...)
    {
      #ifdef use_tcpip_messaging
      if(debug_level[libraries] >= 1 && tcpip_message_router)
        tcpip_message_router->SendMessageToAll("debug", "Error in SqlToMap for sql:\n" + sql + "\n\nUnknown error\n");
      #endif
      toReturn = false;
      if ((dbTransaction->Active) && (can_commit))
        dbTransaction->Rollback();
    }
    delete dbDataSet;
    return toReturn;
  }
  else
    return false;
}
//--------------------------------------------------------------------------
bool SqlToMap(TIBDatabase * dbDatabase, string sql, map<string, string> &field_map)
{
  if(dbDatabase->TestConnected())
  {
    TIBTransaction * dbTransaction = new TIBTransaction(NULL);
    bool toReturn = true;
    try
    {
      StdTransaction (dbTransaction, dbDatabase, false);
      toReturn = SqlToMap(dbTransaction, sql, field_map);
    }
    catch (...)
    {
      toReturn = false;
      if (dbTransaction->Active)
        dbTransaction->Rollback();
    }
    delete dbTransaction;
    return toReturn;
  }
  else
    return false;
}
//--------------------------------------------------------------------------
bool SqlToVecMap(TIBTransaction * dbTransaction, string sql, vector<map<string, string> > &field_vec, bool preserve_fieldname_case)
{
  #ifdef use_tcpip_messaging
  if(debug_level[libraries]>=2 && tcpip_message_router) tcpip_message_router->SendMessageToAll("debug", "SqlToVecMap for sql:\n" + sql);
  #endif
  if(dbTransaction->DefaultDatabase->TestConnected())
  {
    TIBDataSet * dbDataSet = new TIBDataSet(NULL);
    string field_name;
    bool toReturn = true;
    bool can_commit = true;
    if (dbTransaction->Active)
      can_commit = false;
    field_vec.clear();
    try
    {
      dbDataSet->Database = dbTransaction->DefaultDatabase;
      dbDataSet->Transaction = dbTransaction;
      dbDataSet->UniDirectional = false;
      if (can_commit)
        dbTransaction->StartTransaction();
      dbDataSet->SelectSQL->Clear();
      dbDataSet->SelectSQL->Add(sql.c_str());

      dbDataSet->Open();
      dbDataSet->First();
      while (! dbDataSet->Eof)
      {
        map<string, string> temp_map;
        for (int i = 0; i < dbDataSet->Fields->Count; i++)
        {
          field_name = ws(dbDataSet->Fields->Fields[i]->FieldName);
          if (!preserve_fieldname_case) field_name = LowercaseString(field_name); // default false; compat w/ Prometheus
          temp_map[field_name] = dbDataSet->FieldByName(field_name.c_str())->AsString.c_str();
        }
        field_vec.push_back(temp_map);
        dbDataSet->Next();
      }
      dbDataSet->Close();
      if (can_commit)
        dbTransaction->Commit();
      toReturn = true;
      #ifdef use_tcpip_messaging
      if(debug_level[libraries]>=4 && tcpip_message_router) tcpip_message_router->SendMessageToAll("debug", "Executed SqlToVecMap for sql:\n" + sql);
      #endif
    }
    catch (EIBError & error)
    {
      #ifdef use_tcpip_messaging
      if(debug_level[libraries]>=1 && tcpip_message_router) tcpip_message_router->SendMessageToAll("debug", "Error in SqlToVecMap for sql:\n" + sql + "\n\n" + error.Message.c_str());
      #endif
      last_sql_lib_error = error;
      toReturn = false;
      if ((dbTransaction->Active) && (can_commit))
       dbTransaction->Rollback();
    }
    catch (...)
    {
      #ifdef use_tcpip_messaging
      if(debug_level[libraries]>=1 && tcpip_message_router) tcpip_message_router->SendMessageToAll("debug", "Error in SqlToVecMap for sql:\n" + sql + "\n\nUnknown error\n");
      #endif
      toReturn = false;
      if ((dbTransaction->Active) && (can_commit))
        dbTransaction->Rollback();
    }
    delete dbDataSet;
    return toReturn;
  }
  else
    return false;
}
//--------------------------------------------------------------------------
bool SqlToVecMap(TIBDatabase * dbDatabase, string sql, vector<map<string, string> > &field_vec, bool preserve_fieldname_case)
{
  if(dbDatabase->TestConnected())
  {
    TIBTransaction * dbTransaction = new TIBTransaction(NULL);
    bool toReturn = true;
    try
    {
      StdTransaction (dbTransaction, dbDatabase, false);
      toReturn = SqlToVecMap(dbTransaction, sql, field_vec, preserve_fieldname_case);
    }
    catch (...)
    {
      toReturn = false;
      if (dbTransaction->Active)
        dbTransaction->Rollback();
    }
    delete dbTransaction;
    return toReturn;
  }
  else
    return false;
}
//--------------------------------------------------------------------------
bool SqlToListMap(TIBTransaction * dbTransaction, string sql, list<map<string, string> > &field_list)
{
  #ifdef use_tcpip_messaging
  if(debug_level[libraries]>=2 && tcpip_message_router) tcpip_message_router->SendMessageToAll("debug", "SqlToListMap for sql:\n" + sql);
  #endif
  if(dbTransaction->DefaultDatabase->TestConnected())
  {
    TIBDataSet * dbDataSet = new TIBDataSet(NULL);
    string field_name;
    bool toReturn = true;
    bool can_commit = true;
    if (dbTransaction->Active)
      can_commit = false;
    field_list.clear();
    try
    {
      dbDataSet->Database = dbTransaction->DefaultDatabase;
      dbDataSet->Transaction = dbTransaction;
      dbDataSet->UniDirectional = false;
      if (can_commit)
        dbTransaction->StartTransaction();
      dbDataSet->SelectSQL->Clear();
      dbDataSet->SelectSQL->Add(sql.c_str());

      dbDataSet->Open();
      dbDataSet->First();
      while (! dbDataSet->Eof)
      {
        map<string, string> temp_map;
        for (int i = 0; i < dbDataSet->Fields->Count; i++)
        {
          field_name = LowercaseString(ws(dbDataSet->Fields->Fields[i]->FieldName));
          temp_map[field_name] = dbDataSet->FieldByName(field_name.c_str())->AsString.c_str();
        }
        field_list.push_back(temp_map);
        dbDataSet->Next();
      }
      dbDataSet->Close();
      if (can_commit)
        dbTransaction->Commit();
      toReturn = true;
      #ifdef use_tcpip_messaging
      if(debug_level[libraries]>=4 && tcpip_message_router) tcpip_message_router->SendMessageToAll("debug", "Executed SqlToListMap for sql:\n" + sql);
      #endif
    }
    catch (EIBError & error)
    {
      #ifdef use_tcpip_messaging
      if(debug_level[libraries]>=1 && tcpip_message_router) tcpip_message_router->SendMessageToAll("debug", "Error in SqlToListMap for sql:\n" + sql + "\n\n" + error.Message.c_str());
      #endif
      last_sql_lib_error = error;
      toReturn = false;
      if ((dbTransaction->Active) && (can_commit))
       dbTransaction->Rollback();
    }
    catch (...)
    {
      #ifdef use_tcpip_messaging
      if(debug_level[libraries]>=1 && tcpip_message_router) tcpip_message_router->SendMessageToAll("debug", "Error in SqlToListMap for sql:\n" + sql + "\n\nUnknown error\n");
      #endif
      toReturn = false;
      if ((dbTransaction->Active) && (can_commit))
        dbTransaction->Rollback();
    }
    delete dbDataSet;
    return toReturn;
  }
  else
    return false;
}
//--------------------------------------------------------------------------
bool SqlToListMap(TIBDatabase * dbDatabase, string sql, list<map<string, string> > &field_list)
{
  if(dbDatabase->TestConnected())
  {
    TIBTransaction * dbTransaction = new TIBTransaction(NULL);
    bool toReturn = true;
    try
    {
      StdTransaction (dbTransaction, dbDatabase, false);
      toReturn = SqlToListMap(dbTransaction, sql, field_list);
    }
    catch (...)
    {
      toReturn = false;
      if (dbTransaction->Active)
       dbTransaction->Rollback();
     }
    delete dbTransaction;
    return toReturn;
  }
  else
    return false;
}
//--------------------------------------------------------------------------
// SqlToMapMap is NOT TESTED.  I thought I needed it, wrote it, then realized I didn't need it, so I didn't test it.
// I -think- it should work, but you should make -sure- before you use it!
//--------------------------------------------------------------------------
bool SqlToMapMap(TIBTransaction * dbTransaction, string sql, map<string, map<string, string> > &field_map, bool preserve_fieldname_case)
{
  #ifdef use_tcpip_messaging
  if(debug_level[libraries]>=2 && tcpip_message_router) tcpip_message_router->SendMessageToAll("debug", "SqlToMapMap for sql:\n" + sql);
  #endif
  if(dbTransaction->DefaultDatabase->TestConnected())
  {
    TIBDataSet * dbDataSet = new TIBDataSet(NULL);
    string field_name, key;
    bool toReturn = true;
    bool can_commit = true;
    if (dbTransaction->Active)
      can_commit = false;
    field_map.clear();
    try
    {
      dbDataSet->Database = dbTransaction->DefaultDatabase;
      dbDataSet->Transaction = dbTransaction;
      dbDataSet->UniDirectional = false;
      if (can_commit)
        dbTransaction->StartTransaction();
      dbDataSet->SelectSQL->Clear();
      dbDataSet->SelectSQL->Add(sql.c_str());

      dbDataSet->Open();
      dbDataSet->First();
      while (! dbDataSet->Eof)
      {
        // assume the data in the first column is the key
        key = dbDataSet->Fields->Fields[0]->AsString.c_str();
        // iterate
        map<string, string> temp_map;
        for (int i = 0; i < dbDataSet->Fields->Count; i++)
        {
          field_name = ws(dbDataSet->Fields->Fields[i]->FieldName);
          if (!preserve_fieldname_case) field_name = LowercaseString(field_name); // default false; compat w/ Prometheus
          temp_map[field_name] = dbDataSet->FieldByName(field_name.c_str())->AsString.c_str();
        }
        // slap it in the map.  overwrite if necessary.  not going to bother with the more generic multimap case.
        field_map[key] = temp_map;
        dbDataSet->Next();
      }
      dbDataSet->Close();
      if (can_commit)
        dbTransaction->Commit();
      toReturn = true;
      #ifdef use_tcpip_messaging
      if(debug_level[libraries]>=4 && tcpip_message_router) tcpip_message_router->SendMessageToAll("debug", "Executed SqlToVecMap for sql:\n" + sql);
      #endif
    }
    catch (EIBError & error)
    {
      #ifdef use_tcpip_messaging
      if(debug_level[libraries]>=1 && tcpip_message_router) tcpip_message_router->SendMessageToAll("debug", "Error in SqlToVecMap for sql:\n" + sql + "\n\n" + error.Message.c_str());
      #endif
      last_sql_lib_error = error;
      toReturn = false;
      if ((dbTransaction->Active) && (can_commit))
       dbTransaction->Rollback();
    }
    catch (...)
    {
      #ifdef use_tcpip_messaging
      if(debug_level[libraries]>=1 && tcpip_message_router) tcpip_message_router->SendMessageToAll("debug", "Error in SqlToVecMap for sql:\n" + sql + "\n\nUnknown error\n");
      #endif
      toReturn = false;
      if ((dbTransaction->Active) && (can_commit))
        dbTransaction->Rollback();
    }
    delete dbDataSet;
    return toReturn;
  }
  else
    return false;
}
//--------------------------------------------------------------------------
bool SqlToMapMap(TIBDatabase * dbDatabase, string sql, map<string, map<string, string> > &field_map, bool preserve_fieldname_case)
{
  if(dbDatabase->TestConnected())
  {
    TIBTransaction * dbTransaction = new TIBTransaction(NULL);
    bool toReturn = true;
    try
    {
      StdTransaction (dbTransaction, dbDatabase, false);
      toReturn = SqlToMapMap(dbTransaction, sql, field_map, preserve_fieldname_case);
    }
    catch (...)
    {
      toReturn = false;
      if (dbTransaction->Active)
        dbTransaction->Rollback();
    }
    delete dbTransaction;
    return toReturn;
  }
  else
    return false;
}
//--------------------------------------------------------------------------
bool SqlToSet(TIBTransaction * dbTransaction, string sql, set<string> &field_set)
{
  #ifdef use_tcpip_messaging
  if(debug_level[libraries]>=2 && tcpip_message_router) tcpip_message_router->SendMessageToAll("debug", "SqlToSet for sql:\n" + sql);
  #endif
  if(dbTransaction->DefaultDatabase->TestConnected())
  {
    TIBDataSet * dbDataSet = new TIBDataSet(NULL);
    bool toReturn = true;
    bool can_commit = true;
    if (dbTransaction->Active)
      can_commit = false;
    field_set.clear();
    try
    {
      dbDataSet->Database = dbTransaction->DefaultDatabase;
      dbDataSet->Transaction = dbTransaction;
      dbDataSet->UniDirectional = false;
      if (can_commit)
        dbTransaction->StartTransaction();
      dbDataSet->SelectSQL->Clear();
      dbDataSet->SelectSQL->Add(sql.c_str());

      dbDataSet->Open();
      dbDataSet->First();
      while (! dbDataSet->Eof)
      {
        field_set.insert(dbDataSet->Fields->Fields[0]->AsString.c_str());
        dbDataSet->Next();
      }
      dbDataSet->Close();
      if (can_commit)
        dbTransaction->Commit();
      #ifdef use_tcpip_messaging
      if(debug_level[libraries]>=4 && tcpip_message_router) tcpip_message_router->SendMessageToAll("debug", "SqlToSet result: '" + SetToCommaString(field_set) + "'");
      #endif
      toReturn = true;
    }
    catch (EIBError & error)
    {
      #ifdef use_tcpip_messaging
      if(debug_level[libraries]>=1 && tcpip_message_router) tcpip_message_router->SendMessageToAll("debug", "Error in SqlToSet for sql:\n" + sql + "\n\n" + error.Message.c_str());
      #endif
      last_sql_lib_error = error;
      toReturn = false;
      if ((dbTransaction->Active) && (can_commit))
       dbTransaction->Rollback();
    }
    catch (...)
    {
      #ifdef use_tcpip_messaging
      if(debug_level[libraries]>=1 && tcpip_message_router) tcpip_message_router->SendMessageToAll("debug", "Error in SqlToSet for sql:\n" + sql + "\n\nUnknown error\n");
      #endif
      toReturn = false;
      if ((dbTransaction->Active) && (can_commit))
        dbTransaction->Rollback();
    }
    delete dbDataSet;
    return toReturn;
  }
  else
    return false;
}
//--------------------------------------------------------------------------
bool SqlToSet(TIBDatabase * dbDatabase, string sql, set<string> &field_set)
{
  if(dbDatabase->TestConnected())
  {
    TIBTransaction * dbTransaction = new TIBTransaction(NULL);
    bool toReturn = true;
    try
    {
      StdTransaction (dbTransaction, dbDatabase, false);
      toReturn = SqlToSet(dbTransaction, sql, field_set);
    }
    catch (...)
    {
      toReturn = false;
      if (dbTransaction->Active)
        dbTransaction->Rollback();
    }
    delete dbTransaction;
    return toReturn;
  }
  else
    return false;
}
//--------------------------------------------------------------------------
bool SqlToList(TIBTransaction * dbTransaction, string sql, list<string> &result_list)
{
  #ifdef use_tcpip_messaging
  if(debug_level[libraries]>=2 && tcpip_message_router) tcpip_message_router->SendMessageToAll("debug", "SqlToList for sql:\n" + sql);
  #endif
  if(dbTransaction->DefaultDatabase->TestConnected())
  {
    TIBDataSet * dbDataSet = new TIBDataSet(NULL);
    bool toReturn = true;
    bool can_commit = true;
    if (dbTransaction->Active)
      can_commit = false;
    result_list.clear();
    try
    {
      dbDataSet->Database = dbTransaction->DefaultDatabase;
      dbDataSet->Transaction = dbTransaction;
      dbDataSet->UniDirectional = false;
      if (can_commit)
        dbTransaction->StartTransaction();
      dbDataSet->SelectSQL->Clear();
      dbDataSet->SelectSQL->Add(sql.c_str());

      dbDataSet->Open();
      dbDataSet->First();
      while (! dbDataSet->Eof)
      {
        result_list.push_back(dbDataSet->Fields->Fields[0]->AsString.c_str());
        dbDataSet->Next();
      }
      dbDataSet->Close();
      if (can_commit)
        dbTransaction->Commit();
      #ifdef use_tcpip_messaging
      if(debug_level[libraries]>=4 && tcpip_message_router) tcpip_message_router->SendMessageToAll("debug", "SqlToList result: '" + ListToCommaString(result_list) + "'");
      #endif
      toReturn = true;
    }
    catch (EIBError & error)
    {
      #ifdef use_tcpip_messaging
      if(debug_level[libraries]>=1 && tcpip_message_router) tcpip_message_router->SendMessageToAll("debug", "Error in SqlToList for sql:\n" + sql + "\n\n" + error.Message.c_str());
      #endif
      last_sql_lib_error = error;
      toReturn = false;
      if ((dbTransaction->Active) && (can_commit))
       dbTransaction->Rollback();
    }
    catch (...)
    {
      #ifdef use_tcpip_messaging
      if(debug_level[libraries]>=1 && tcpip_message_router) tcpip_message_router->SendMessageToAll("debug", "Error in SqlToList for sql:\n" + sql + "\n\nUnknown error\n");
      #endif
      toReturn = false;
      if ((dbTransaction->Active) && (can_commit))
        dbTransaction->Rollback();
    }
    delete dbDataSet;
    return toReturn;
  }
  else
    return false;
}
//--------------------------------------------------------------------------
bool SqlToList(TIBDatabase * dbDatabase, string sql, list<string> &result_list)
{
  if(dbDatabase->TestConnected())
  {
    TIBTransaction * dbTransaction = new TIBTransaction(NULL);
    bool toReturn = true;
    try
    {
      StdTransaction (dbTransaction, dbDatabase, false);
      toReturn = SqlToList(dbTransaction, sql, result_list);
    }
    catch (...)
    {
      toReturn = false;
      if (dbTransaction->Active)
        dbTransaction->Rollback();
    }
    delete dbTransaction;
    return toReturn;
  }
  else
    return false;
}
//--------------------------------------------------------------------------
bool SqlToVec(TIBTransaction * dbTransaction, string sql, vector<string> &result_vec)
{
  #ifdef use_tcpip_messaging
  if(debug_level[libraries]>=2 && tcpip_message_router) tcpip_message_router->SendMessageToAll("debug", "SqlToVec for sql:\n" + sql);
  #endif
  if(dbTransaction->DefaultDatabase->TestConnected())
  {
    TIBDataSet * dbDataSet = new TIBDataSet(NULL);
    bool toReturn = true;
    bool can_commit = true;
    if (dbTransaction->Active)
      can_commit = false;
    result_vec.clear();
    try
    {
      dbDataSet->Database = dbTransaction->DefaultDatabase;
      dbDataSet->Transaction = dbTransaction;
      dbDataSet->UniDirectional = false;
      if (can_commit)
        dbTransaction->StartTransaction();
      dbDataSet->SelectSQL->Clear();
      dbDataSet->SelectSQL->Add(sql.c_str());

      dbDataSet->Open();
      dbDataSet->First();
      while (! dbDataSet->Eof)
      {
        result_vec.push_back(dbDataSet->Fields->Fields[0]->AsString.c_str());
        dbDataSet->Next();
      }
      dbDataSet->Close();
      if (can_commit)
        dbTransaction->Commit();
      #ifdef use_tcpip_messaging
      if(debug_level[libraries]>=4 && tcpip_message_router) tcpip_message_router->SendMessageToAll("debug", "SqlToVec result: '" + VecToCommaString(result_vec) + "'");
      #endif
      toReturn = true;
    }
    catch (EIBError & error)
    {
      #ifdef use_tcpip_messaging
      if(debug_level[libraries]>=1 && tcpip_message_router) tcpip_message_router->SendMessageToAll("debug", "Error in SqlToVec for sql:\n" + sql + "\n\n" + error.Message.c_str());
      #endif
      last_sql_lib_error = error;
      toReturn = false;
      if ((dbTransaction->Active) && (can_commit))
       dbTransaction->Rollback();
    }
    catch (...)
    {
      #ifdef use_tcpip_messaging
      if(debug_level[libraries]>=1 && tcpip_message_router) tcpip_message_router->SendMessageToAll("debug", "Error in SqlToVec for sql:\n" + sql + "\n\nUnknown error\n");
      #endif
      toReturn = false;
      if ((dbTransaction->Active) && (can_commit))
        dbTransaction->Rollback();
    }
    delete dbDataSet;
    return toReturn;
  }
  else
    return false;
}
//--------------------------------------------------------------------------
bool SqlToVec(TIBDatabase * dbDatabase, string sql, vector<string> &result_vec)
{
  if(dbDatabase->TestConnected())
  {
    TIBTransaction * dbTransaction = new TIBTransaction(NULL);
    bool toReturn = true;
    try
    {
      StdTransaction (dbTransaction, dbDatabase, false);
      toReturn = SqlToVec(dbTransaction, sql, result_vec);
    }
    catch (...)
    {
      toReturn = false;
      if (dbTransaction->Active)
        dbTransaction->Rollback();
    }
    delete dbTransaction;
    return toReturn;
  }
  else
    return false;
}
//--------------------------------------------------------------------------
bool SqlToValueMap(TIBTransaction * dbTransaction, string sql, map<string, string> &field_map)
{
  #ifdef use_tcpip_messaging
  if(debug_level[libraries]>=2 && tcpip_message_router) tcpip_message_router->SendMessageToAll("debug", "SqlToValueMap for sql:\n" + sql);
  #endif
  if(dbTransaction->DefaultDatabase->TestConnected())
  {
    TIBDataSet * dbDataSet = new TIBDataSet(NULL);
    string key;
    bool toReturn = true;
    bool can_commit = true;
    if (dbTransaction->Active)
      can_commit = false;
    field_map.clear();
    try
    {
      dbDataSet->Database = dbTransaction->DefaultDatabase;
      dbDataSet->Transaction = dbTransaction;
      dbDataSet->UniDirectional = false;
      if (can_commit)
        dbTransaction->StartTransaction();
      dbDataSet->SelectSQL->Clear();
      dbDataSet->SelectSQL->Add(sql.c_str());

      dbDataSet->Open();
      dbDataSet->First();
      if(dbDataSet->FieldCount == 2)
      {
        while (! dbDataSet->Eof)
        {
          key = dbDataSet->Fields->Fields[0]->AsString.c_str();
          field_map[key] = dbDataSet->Fields->Fields[1]->AsString.c_str();
          dbDataSet->Next();
        }
      }
      dbDataSet->Close();
      if (can_commit)
        dbTransaction->Commit();
      toReturn = true;
      #ifdef use_tcpip_messaging
      if(debug_level[libraries]>=4 && tcpip_message_router) tcpip_message_router->SendMessageToAll("debug", "SqlToValueMap result: '" + MapToDelimString(field_map, "{", "}", "", "'", "'", "'", "'", ",") + "'");
      #endif
    }
    catch (EIBError & error)
    {
      #ifdef use_tcpip_messaging
      if(debug_level[libraries]>=1 && tcpip_message_router) tcpip_message_router->SendMessageToAll("debug", "Error in SqlToValueMap for sql:\n" + sql + "\n\n" + error.Message.c_str());
      #endif
      last_sql_lib_error = error;
      toReturn = false;
      if ((dbTransaction->Active) && (can_commit))
       dbTransaction->Rollback();
    }
    catch (...)
    {
      #ifdef use_tcpip_messaging
      if(debug_level[libraries]>=1 && tcpip_message_router) tcpip_message_router->SendMessageToAll("debug", "Error in SqlToValueMap for sql:\n" + sql + "\n\nUnknown error\n");
      #endif
      toReturn = false;
      if ((dbTransaction->Active) && (can_commit))
        dbTransaction->Rollback();
    }
    delete dbDataSet;
    return toReturn;
  }
  else
    return false;
}
//--------------------------------------------------------------------------
bool SqlToValueMap(TIBDatabase * dbDatabase, string sql, map<string, string> &field_map)
{
  if(dbDatabase->TestConnected())
  {
    TIBTransaction * dbTransaction = new TIBTransaction(NULL);
    bool toReturn = true;
    try
    {
      StdTransaction (dbTransaction, dbDatabase, false);
      toReturn = SqlToValueMap(dbTransaction, sql, field_map);
    }
    catch (...)
    {
      toReturn = false;
      if (dbTransaction->Active)
        dbTransaction->Rollback();
    }
    delete dbTransaction;
    return toReturn;
  }
  else
    return false;
}
//--------------------------------------------------------------------------
bool SqlToMapStringVec(TIBTransaction * dbTransaction, string sql, map<string, vector<string> > &field_map)
{
  #ifdef use_tcpip_messaging
  if(debug_level[libraries]>=2 && tcpip_message_router) tcpip_message_router->SendMessageToAll("debug", "SqlToMapStringVec for sql:\n" + sql);
  #endif
  if(dbTransaction->DefaultDatabase->TestConnected())
  {
    TIBDataSet * dbDataSet = new TIBDataSet(NULL);
    string key;
    bool toReturn = true;
    bool can_commit = true;
    if (dbTransaction->Active)
      can_commit = false;
    field_map.clear();
    try
    {
      dbDataSet->Database = dbTransaction->DefaultDatabase;
      dbDataSet->Transaction = dbTransaction;
      dbDataSet->UniDirectional = false;
      if (can_commit)
        dbTransaction->StartTransaction();
      dbDataSet->SelectSQL->Clear();
      dbDataSet->SelectSQL->Add(sql.c_str());

      dbDataSet->Open();
      dbDataSet->First();
      if(dbDataSet->FieldCount == 2)
      {
        while (! dbDataSet->Eof)
        {
          key = dbDataSet->Fields->Fields[0]->AsString.c_str();
          field_map[key].push_back(dbDataSet->Fields->Fields[1]->AsString.c_str());
          dbDataSet->Next();
        }
      }
      dbDataSet->Close();
      if (can_commit)
        dbTransaction->Commit();
      toReturn = true;
      #ifdef use_tcpip_messaging
      if(debug_level[libraries]>=1 && tcpip_message_router) tcpip_message_router->SendMessageToAll("debug", "Executed SqlToMapStringVec for sql:\n" + sql);
      #endif
    }
    catch (EIBError & error)
    {
      #ifdef use_tcpip_messaging
      if(debug_level[libraries]>=1 && tcpip_message_router) tcpip_message_router->SendMessageToAll("debug", "Error in SqlToMapStringVec for sql:\n" + sql + "\n\n" + error.Message.c_str());
      #endif
      last_sql_lib_error = error;
      toReturn = false;
      if ((dbTransaction->Active) && (can_commit))
       dbTransaction->Rollback();
    }
    catch (...)
    {
      #ifdef use_tcpip_messaging
      if(debug_level[libraries]>=1 && tcpip_message_router) tcpip_message_router->SendMessageToAll("debug", "Error in SqlToMapStringVec for sql:\n" + sql + "\n\nUnknown error\n");
      #endif
      toReturn = false;
      if ((dbTransaction->Active) && (can_commit))
        dbTransaction->Rollback();
    }
    delete dbDataSet;
    return toReturn;
  }
  else
    return false;
}
//--------------------------------------------------------------------------
bool SqlToMapStringVec(TIBDatabase * dbDatabase, string sql, map<string, vector<string> > &field_map)
{
  if(dbDatabase->TestConnected())
  {
    TIBTransaction * dbTransaction = new TIBTransaction(NULL);
    bool toReturn = true;
    try
    {
      StdTransaction (dbTransaction, dbDatabase, false);
      toReturn = SqlToMapStringVec(dbTransaction, sql, field_map);
    }
    catch (...)
    {
      toReturn = false;
      if (dbTransaction->Active)
        dbTransaction->Rollback();
    }
    delete dbTransaction;
    return toReturn;
  }
  else
    return false;
}
//--------------------------------------------------------------------------
bool SqlToMapStringList(TIBTransaction * dbTransaction, string sql, map<string, vector<string> > &field_map)
{
  #ifdef use_tcpip_messaging
  if(debug_level[libraries]>=2 && tcpip_message_router) tcpip_message_router->SendMessageToAll("debug", "SqlToMapStringList for sql:\n" + sql);
  #endif
  if(dbTransaction->DefaultDatabase->TestConnected())
  {
    TIBDataSet * dbDataSet = new TIBDataSet(NULL);
    string key;
    bool toReturn = true;
    bool can_commit = true;
    if (dbTransaction->Active)
      can_commit = false;
    field_map.clear();
    try
    {
      dbDataSet->Database = dbTransaction->DefaultDatabase;
      dbDataSet->Transaction = dbTransaction;
      dbDataSet->UniDirectional = false;
      if (can_commit)
        dbTransaction->StartTransaction();
      dbDataSet->SelectSQL->Clear();
      dbDataSet->SelectSQL->Add(sql.c_str());

      dbDataSet->Open();
      dbDataSet->First();
      if(dbDataSet->FieldCount == 2)
      {
        while (! dbDataSet->Eof)
        {
          key = dbDataSet->Fields->Fields[0]->AsString.c_str();
          field_map[key].push_back(dbDataSet->Fields->Fields[1]->AsString.c_str());
          dbDataSet->Next();
        }
      }
      dbDataSet->Close();
      if (can_commit)
        dbTransaction->Commit();
      toReturn = true;
      #ifdef use_tcpip_messaging
      if(debug_level[libraries]>=1 && tcpip_message_router) tcpip_message_router->SendMessageToAll("debug", "Executed SqlToMapStringList for sql:\n" + sql);
      #endif
    }
    catch (EIBError & error)
    {
      #ifdef use_tcpip_messaging
      if(debug_level[libraries]>=1 && tcpip_message_router) tcpip_message_router->SendMessageToAll("debug", "Error in SqlToMapStringList for sql:\n" + sql + "\n\n" + error.Message.c_str());
      #endif
      last_sql_lib_error = error;
      toReturn = false;
      if ((dbTransaction->Active) && (can_commit))
       dbTransaction->Rollback();
    }
    catch (...)
    {
      #ifdef use_tcpip_messaging
      if(debug_level[libraries]>=1 && tcpip_message_router) tcpip_message_router->SendMessageToAll("debug", "Error in SqlToMapStringList for sql:\n" + sql + "\n\nUnknown error\n");
      #endif
      toReturn = false;
      if ((dbTransaction->Active) && (can_commit))
        dbTransaction->Rollback();
    }
    delete dbDataSet;
    return toReturn;
  }
  else
    return false;
}
//--------------------------------------------------------------------------
bool SqlToMapStringList(TIBDatabase * dbDatabase, string sql, map<string, list<string> > &field_map)
{
  if(dbDatabase->TestConnected())
  {
    TIBTransaction * dbTransaction = new TIBTransaction(NULL);
    bool toReturn = true;
    try
    {
      StdTransaction (dbTransaction, dbDatabase, false);
      toReturn = SqlToMapStringList(dbTransaction, sql, field_map);
    }
    catch (...)
    {
      toReturn = false;
      if (dbTransaction->Active)
        dbTransaction->Rollback();
    }
    delete dbTransaction;
    return toReturn;
  }
  else
    return false;
}
//--------------------------------------------------------------------------
bool ExecutePreparedInsert (TIBTransaction * dbTransaction, string tableName,
                            vector<string> &field_names,
                            list <vector<string> > & insert_data,
                            map<string, int> field_options)
{
  #ifdef use_tcpip_messaging
  if(debug_level[libraries]>=2 && tcpip_message_router) tcpip_message_router->SendMessageToAll("debug", "Executing prepared statement into table " + tableName + " of fields\n" + VecToCommaString(field_names));
  #endif
  if(dbTransaction->DefaultDatabase->TestConnected())
  {
   bool can_commit = true;

   if(dbTransaction->Active)
    can_commit = false;

   if (!field_names.empty())
   {
    TIBSQL * dbSQL = new TIBSQL(NULL);

    try
    {
     dbSQL->Database = dbTransaction->DefaultDatabase;
     dbSQL->Transaction = dbTransaction;
     if(can_commit)
      dbTransaction->StartTransaction();

     // sqlstring with parameter names
     unsigned int n = field_names.size();
     vector<string> param_names = field_names;
     for (unsigned int i = 0; i < n; i++)
      param_names[i] = ":" + param_names[i];

     string sqlstring = "insert into " + tableName + "(" + VecToCommaString(field_names) + ") "
                        "values (" + VecToCommaString(param_names) + ")";
     dbSQL->SQL->Text = sqlstring.c_str();
     dbSQL->Prepare();
     string field_value;
     string field_name;
/* // okay, so, funny story, right?
   // turns out I don't need to look at types, because I don't *ever* need to quote *any* values,
   // when inserting data with a prepared statement.  If you read the commented-out section of
   // code below, which is what I was replacing, it doesn't quote.  So I don't need to quote.
   // .... *facepalm*
     map<string, int> param_types;
     for (unsigned int i = 0; i < n; i++)
       param_types[field_names[i]] = dbSQL->Params->ByName(field_names[i].c_str())->SQLType;
*/

     // loop through the rows
     for (list<vector<string> >::iterator i_row = insert_data.begin(); i_row != insert_data.end(); i_row++)
     {
      // get the params in there
      for (unsigned int i = 0; i < n; i++)
      {
       field_value = (*i_row)[i];
       field_name  = field_names[i];
       if (field_value.empty())
       {
        dbSQL->Params->ByName(field_name.c_str())->IsNull = true;
       }
       else
       {
/*
        if (save_option == uppercase_field && fields_not_to_uppercase.find(field_name) == fields_not_to_uppercase.end())
         field_value = UppercaseString(field_value);
        field_value = StripLeading(StripTrailing(ReplaceStringWithString(field_value, "'", "''"), " "), " ");
*/
        map<string, int>::iterator opt_itr = field_options.find(field_name);
        if (opt_itr == field_options.end())
        {
         field_value = StripLeading(StripTrailing(UppercaseString(field_value), " "), " ");
//         int type = param_types[field_name];
//         if (type==SQL_TEXT || type==SQL_VARYING || type==SQL_TYPE_DATE || type==SQL_TYPE_TIME || type==SQL_TIMESTAMP || type==SQL_BLOB || type==SQL_ARRAY) /* complete? */
//           field_value = "'" + ReplaceStringWithString(field_value, "'", "''") + "'";
        }
        else
        {
         int opts = opt_itr->second;
         if ((opts & sql_no_uppercasing) == 0)
           field_value = UppercaseString(field_value);
         if ((opts & sql_no_left_trimming) == 0)
           field_value = StripLeading(field_value, " ");
         if ((opts & sql_no_right_trimming) == 0)
           field_value = StripTrailing(field_value, " ");
//         if ((opts & sql_no_quoting) == 0)
//           field_value = "'" + ReplaceStringWithString(field_value, "'", "''") + "'";
        }
        dbSQL->Params->ByName(field_name.c_str())->AsString = field_value.c_str();
       }
      }
      dbSQL->ExecQuery();
     }

     if(can_commit)
      dbTransaction->Commit();
     delete dbSQL;
     #ifdef use_tcpip_messaging
     if(debug_level[libraries]>=4 && tcpip_message_router) tcpip_message_router->SendMessageToAll("debug", "Executed prepared statement into table " + tableName + " of fields\n" + VecToCommaString(field_names));
     #endif
     return true;
    }
    catch (EIBError & error)
    {
      #ifdef use_tcpip_messaging
      if(debug_level[libraries]>=1 && tcpip_message_router) tcpip_message_router->SendMessageToAll("debug", "Error executing prepared statement into table " + tableName + " of fields\n" + VecToCommaString(field_names) + "\n\n" + error.Message.c_str());
      #endif
      last_sql_lib_error = error;
      if (dbTransaction->Active && can_commit)
       dbTransaction->Rollback();
      delete dbSQL;
      return false;
    }
    catch (...)
    {
      #ifdef use_tcpip_messaging
      if(debug_level[libraries]>=1 && tcpip_message_router) tcpip_message_router->SendMessageToAll("debug", "Error executing prepared statement into table " + tableName + " of fields\n" + VecToCommaString(field_names) + "\n\nUnknown error\n");
      #endif
      if(dbTransaction->Active && can_commit)
       dbTransaction->Rollback();
      delete dbSQL;
      return false;
    }
   } else
   {
    // no fields to insert -- we're not going to insert with zero fields (theoretically useful, but not anywhere in prometheus, that i know of.)
    return false;
   }
  }
  else
   return false;
}
//--------------------------------------------------------------------------
bool ExecutePreparedInsert (TIBDatabase * dbDatabase, string tableName,
                            vector<string> &field_names,
                            list <vector<string> > & insert_data,
                            map<string, int> field_options)
{
  if(dbDatabase->TestConnected())
  {
    TIBTransaction * dbTransaction = new TIBTransaction(NULL);
    bool toReturn = true;
    try
    {
      StdTransaction (dbTransaction, dbDatabase, false);
      toReturn = ExecutePreparedInsert(dbTransaction, tableName, field_names, insert_data, field_options);
    }
    catch (...)
    {
      toReturn = false;
      if (dbTransaction->Active)
        dbTransaction->Rollback();
    }
    delete dbTransaction;
    return toReturn;
  }
  else
    return false;
}
//--------------------------------------------------------------------------
bool ExecutePreparedSetInsert (TIBTransaction * dbTransaction, string table_name,
                            string field_name, const set<string> & insert_data,
                            map<string, int> field_options)
{
  #ifdef use_tcpip_messaging
  if(debug_level[libraries]>=2 && tcpip_message_router) tcpip_message_router->SendMessageToAll("debug", "Executing prepared statement into table " + table_name + " of field " + field_name + " from a set");
  #endif
  if(dbTransaction->DefaultDatabase->TestConnected())
  {
   bool can_commit = true;

   if(dbTransaction->Active)
    can_commit = false;

   TIBSQL * dbSQL = new TIBSQL(NULL);

   try
   {
    dbSQL->Database = dbTransaction->DefaultDatabase;
    dbSQL->Transaction = dbTransaction;
    if(can_commit)
     dbTransaction->StartTransaction();

    string sqlstring = "insert into " + table_name + "(" + field_name + ") "
                       "values (:" + field_name + ")";
    dbSQL->SQL->Text = sqlstring.c_str();
    dbSQL->Prepare();
    string field_value;

    int param_type = dbSQL->Params->ByName(field_name.c_str())->SQLType;

    // loop through the rows
    for (set<string>::const_iterator i = insert_data.begin();
         i != insert_data.end(); i++)
    {
     field_value = (*i);
     if (field_value.empty())
      dbSQL->Params->ByName(field_name.c_str())->IsNull = true;
     else
     {
/*
      if (save_option == uppercase_field)
       field_value = UppercaseString(field_value);
      field_value = StripLeading(StripTrailing(ReplaceStringWithString(field_value, "'", "''"), " "), " ");
*/
      map<string, int>::iterator opt_itr = field_options.find(field_name);
      if (opt_itr == field_options.end())
      {
       field_value = StripLeading(StripTrailing(UppercaseString(field_value), " "), " ");
       if (param_type==SQL_TEXT || param_type==SQL_VARYING || param_type==SQL_TYPE_DATE || param_type==SQL_TYPE_TIME || param_type==SQL_TIMESTAMP || param_type==SQL_BLOB || param_type==SQL_ARRAY) /* complete? */
         field_value = "'" + ReplaceStringWithString(field_value, "'", "''") + "'";
      }
      else
      {
       int opts = opt_itr->second;
       if ((opts & sql_no_uppercasing) == 0)
         field_value = UppercaseString(field_value);
       if ((opts & sql_no_left_trimming) == 0)
         field_value = StripLeading(field_value, " ");
       if ((opts & sql_no_right_trimming) == 0)
         field_value = StripTrailing(field_value, " ");
       if ((opts & sql_no_quoting) == 0)
         field_value = "'" + ReplaceStringWithString(field_value, "'", "''") + "'";
      }
      dbSQL->Params->ByName(field_name.c_str())->AsString = field_value.c_str();
     }
     dbSQL->ExecQuery();
    }

    if(can_commit)
     dbTransaction->Commit();
    delete dbSQL;
    #ifdef use_tcpip_messaging
    if(debug_level[libraries]>=4 && tcpip_message_router) tcpip_message_router->SendMessageToAll("debug", "Executed prepared statement into table " + table_name + " of field " + field_name);
    #endif
    return true;
   }
   catch (EIBError & error)
   {
     #ifdef use_tcpip_messaging
     if(debug_level[libraries]>=1 && tcpip_message_router) tcpip_message_router->SendMessageToAll("debug", "Error executing prepared statement into table " + table_name + " of field " + field_name + "\n\n" + error.Message.c_str());
     #endif
     last_sql_lib_error = error;
     if (dbTransaction->Active && can_commit)
      dbTransaction->Rollback();
     delete dbSQL;
     return false;
   }
   catch (...)
   {
     #ifdef use_tcpip_messaging
     if(debug_level[libraries]>=1 && tcpip_message_router) tcpip_message_router->SendMessageToAll("debug", "Error executing prepared statement into table " + table_name + " of field " + field_name + "\n\nUnknown error\n");
     #endif
     if(dbTransaction->Active && can_commit)
      dbTransaction->Rollback();
     delete dbSQL;
     return false;
   }
  }
  else
   return false;
}
//--------------------------------------------------------------------------
bool ExecutePreparedSetInsert (TIBDatabase * dbDatabase, string table_name,
                            string field_name, const set<string> & insert_data,
                            map<string, int> field_options)
{
  if(dbDatabase->TestConnected())
  {
    TIBTransaction * dbTransaction = new TIBTransaction(NULL);
    bool toReturn = true;
    try
    {
      StdTransaction (dbTransaction, dbDatabase, false);
      toReturn = ExecutePreparedSetInsert(dbTransaction, table_name, field_name, insert_data, field_options);
    }
    catch (...)
    {
      toReturn = false;
      if (dbTransaction->Active)
        dbTransaction->Rollback();
    }
    delete dbTransaction;
    return toReturn;
  }
  else
    return false;
}
//--------------------------------------------------------------------------
// just starts it up with normal stuff -- must already have been created though.
bool StdTransaction (TIBTransaction * dbTransaction, TIBDatabase * dbDatabase, bool autostart)
{
 bool result = true;
 try
 {
   // create the object if needed, clear it out (maybe modify later?)
   dbTransaction->DefaultDatabase = dbDatabase;
   dbTransaction->DefaultAction = TARollback;
   dbTransaction->Params->Add("nowait");
   if (autostart)
     dbTransaction->Active = true;
 }
 catch (EIBError & error)
 {
   #ifdef use_tcpip_messaging
   if(debug_level[libraries]>=1 && tcpip_message_router) tcpip_message_router->SendMessageToAll("debug", "Error creating standard transaction:\n" + string(error.Message.c_str()));
   #endif
   last_sql_lib_error = error;
   result = false;
 }
 catch (...)
 {
   #ifdef use_tcpip_messaging
   if(debug_level[libraries]>=1 && tcpip_message_router) tcpip_message_router->SendMessageToAll("debug", "Error creating standard transaction.\nA non-Interbase error of some sort occurred.\n");
   #endif
   last_sql_lib_error.Reset();
   last_sql_lib_error.message = "A non-InterBase error of some sort occurred.";
   result = false;
 }
 return result;
}
//--------------------------------------------------------------------------
// read committed -- must already have been created though.
bool SpyTransaction (TIBTransaction * dbTransaction, TIBDatabase * dbDatabase, bool autostart)
{
 bool result = true;
 try
 {
   // create the object if needed, clear it out (maybe modify later?)
   dbTransaction->DefaultDatabase = dbDatabase;
   dbTransaction->DefaultAction = TARollback;
   dbTransaction->Params->Add("nowait");
   dbTransaction->Params->Add("read_committed");
   dbTransaction->Params->Add("rec_version");
   if (autostart)
     dbTransaction->Active = true;
 }
 catch (EIBError & error)
 {
   #ifdef use_tcpip_messaging
   if(debug_level[libraries]>=1 && tcpip_message_router) tcpip_message_router->SendMessageToAll("debug", "Error creating spy transaction:\n" + string(error.Message.c_str()));
   #endif
   last_sql_lib_error = error;
   result = false;
 }
 catch (...)
 {
   #ifdef use_tcpip_messaging
   if(debug_level[libraries]>=1 && tcpip_message_router) tcpip_message_router->SendMessageToAll("debug", "Error creating spy transaction.\nA non-Interbase error of some sort occurred.\n");
   #endif
   last_sql_lib_error.Reset();
   last_sql_lib_error.message = "A non-InterBase error of some sort occurred.";
   result = false;
 }
 return result;
}
//--------------------------------------------------------------------------
bool SqlToCommaString(TIBTransaction * dbTransaction, string sql, string &result_string)
{
  if(dbTransaction->DefaultDatabase->TestConnected())
  {
    bool to_return;
    vector<string> temp_vec;
    to_return = SqlToVec(dbTransaction, sql, temp_vec);
    if(! temp_vec.empty())
      result_string = VecToCommaString(temp_vec);
    else
      result_string = "";
    return to_return;
  }
  else
    return false;
}
//--------------------------------------------------------------------------
bool SqlToCommaString(TIBDatabase * dbDatabase, string sql, string &result_string)
{
  if(dbDatabase->TestConnected())
  {
    TIBTransaction * dbTransaction = new TIBTransaction(NULL);
    bool toReturn = true;
    try
    {
      StdTransaction (dbTransaction, dbDatabase, false);
      toReturn = SqlToCommaString(dbTransaction, sql, result_string);
    }
    catch (...)
    {
      toReturn = false;
      if (dbTransaction->Active)
        dbTransaction->Rollback();
    }
    delete dbTransaction;
    return toReturn;
  }
  else
    return false;
}
//--------------------------------------------------------------------------
void ResetLastSqlError ()
{
 last_sql_lib_error.Reset();
 return;
}
//--------------------------------------------------------------------------
bool ForeignKeyLookup (TIBTransaction * dbTransaction, string local_table_name, string local_field_name, string & foreign_table_name, string & foreign_field_name)
// Unlike other functions in sqllib, this one looks to REQUIRE an active transaction to work
{
  #ifdef use_tcpip_messaging
  if(debug_level[libraries]>=2 && tcpip_message_router) tcpip_message_router->SendMessageToAll("debug", "ForeignKeyLookup for " + local_table_name + "." + local_field_name);
  #endif
  if (!dbTransaction)
    return false;

  foreign_table_name = "";
  foreign_field_name = "";

  // shamelessly pulled from paul's codeSpecialDataQuery -- many thanks to paul for getting this "just right"
  if(dbTransaction->DefaultDatabase->TestConnected())
  {
    string gsql;
    gsql = "SELECT C.RDB$RELATION_NAME pk_table, E.RDB$FIELD_NAME pk_field "
           "FROM RDB$RELATION_CONSTRAINTS A "
           "   INNER JOIN RDB$INDICES B ON A.RDB$INDEX_NAME = B.RDB$INDEX_NAME "
           "   INNER JOIN RDB$INDICES C ON B.RDB$FOREIGN_KEY = C.RDB$INDEX_NAME "
           "   INNER JOIN RDB$INDEX_SEGMENTS D ON B.RDB$INDEX_NAME = D.RDB$INDEX_NAME "
           "   INNER JOIN RDB$INDEX_SEGMENTS E ON B.RDB$FOREIGN_KEY = E.RDB$INDEX_NAME "
           "WHERE A.RDB$RELATION_NAME='" + local_table_name + "' AND "
           "   B.RDB$RELATION_NAME='" + local_table_name + "' AND "
           "   D.RDB$FIELD_NAME='" + local_field_name + "' AND "
           "   A.RDB$CONSTRAINT_TYPE='FOREIGN KEY' "
           "ORDER BY A.RDB$CONSTRAINT_NAME ";
    vector<map<string, string> > gsql_results;
    if (!SqlToVecMap(dbTransaction, gsql, gsql_results))
      return false;
    if(!gsql_results.empty())
    {
      foreign_table_name = StripTrailing(gsql_results[0]["pk_table"], " ");
      foreign_field_name = StripTrailing(gsql_results[0]["pk_field"], " ");
      return true;
    }
    else
      return false;
  }
  else
    return false;
}
//--------------------------------------------------------------------------
bool ForeignKeyLookup (TIBDatabase * dbDatabase, string local_table_name, string local_field_name, string & foreign_table_name, string & foreign_field_name)
{
 if (!dbDatabase)
  return false;

 if(dbDatabase->TestConnected())
 {
   TIBTransaction * dbTransaction = new TIBTransaction(NULL);
   StdTransaction (dbTransaction, dbDatabase);
   bool toReturn = true;
   try
   {
     toReturn = ForeignKeyLookup (dbTransaction, local_table_name, local_field_name, foreign_table_name, foreign_field_name);
   }
   catch (...)
   {
     toReturn = false;
     if (dbTransaction->Active)
       dbTransaction->Rollback();
   }
   if (dbTransaction->Active)
     dbTransaction->Commit();
   delete dbTransaction;
   return toReturn;
 }
 else
   return false;
}
//--------------------------------------------------------------------------
bool ForeignKeyLookupByConstraint (TIBTransaction * dbTransaction, string constraint_name, string & local_table_name, string & local_field_name, string & foreign_table_name, string & foreign_field_name)
// Unlike other functions in sqllib, this one looks to REQUIRE an active transaction to work
{
  #ifdef use_tcpip_messaging
  if(debug_level[libraries]>=2 && tcpip_message_router) tcpip_message_router->SendMessageToAll("debug", "ForeignKeyLookupByConstraint for contraint " + constraint_name);
  #endif
  if (!dbTransaction)
    return false;
  if (dbTransaction->DefaultDatabase->TestConnected())
  {
    string info_sql = "select local_table.rdb$relation_name a, local_field.rdb$field_name b, foreign_table.rdb$relation_name c, foreign_field.rdb$field_name d "
                      "from rdb$ref_constraints "
                      "inner join rdb$relation_constraints local_constraint on local_constraint.rdb$constraint_name = rdb$ref_constraints.rdb$constraint_name "
                      "   and local_constraint.rdb$constraint_type = 'FOREIGN KEY' "
                      "inner join rdb$relation_constraints foreign_constraint on foreign_constraint.rdb$constraint_name = rdb$ref_constraints.rdb$const_name_uq "
                      "   and foreign_constraint.rdb$constraint_type = 'PRIMARY KEY' "
                      "inner join rdb$indices local_table on local_table.rdb$index_name = local_constraint.rdb$index_name "
                      "inner join rdb$indices foreign_table on foreign_table.rdb$index_name = foreign_constraint.rdb$index_name "
                      "inner join rdb$index_segments local_field on local_field.rdb$index_name = local_constraint.rdb$index_name "
                      "inner join rdb$index_segments foreign_field on foreign_field.rdb$index_name = foreign_constraint.rdb$index_name "
                      "where rdb$ref_constraints.rdb$constraint_name = '" + constraint_name + "'";
   map<string, string> info_result;
   if (!SqlToMap(dbTransaction, info_sql, info_result))
     return false;
   if (!info_result.empty())
   {
     local_table_name   = StripTrailing(info_result["a"], " ");
     local_field_name   = StripTrailing(info_result["b"], " ");
     foreign_table_name = StripTrailing(info_result["c"], " ");
     foreign_field_name = StripTrailing(info_result["d"], " ");
     return true;
   }
   else
     return false;
  }
  else
    return false;
}
//--------------------------------------------------------------------------
bool ForeignKeyLookupByConstraint (TIBDatabase * dbDatabase, string constraint_name, string & local_table_name, string & local_field_name, string & foreign_table_name, string & foreign_field_name)
{
 if (!dbDatabase)
  return false;

 if(dbDatabase->TestConnected())
 {
   TIBTransaction * dbTransaction = new TIBTransaction(NULL);
   StdTransaction (dbTransaction, dbDatabase);
   bool toReturn = true;
   try
   {
     toReturn = ForeignKeyLookupByConstraint (dbTransaction, constraint_name, local_table_name, local_field_name, foreign_table_name, foreign_field_name);
   }
   catch (...)
   {
     toReturn = false;
     if (dbTransaction->Active)
       dbTransaction->Rollback();
   }
   if (dbTransaction->Active)
     dbTransaction->Commit();
   delete dbTransaction;
   return toReturn;
 }
 else
   return false;
}
//--------------------------------------------------------------------------
// return the Primary Key that given Foreign Key links to.  This can be multiple fields in one table, so table is string and fields is a vector
bool GetPrimaryKey (TIBTransaction * dbTransaction, string fk_table, string fk_field, string & pk_table, vector<string> & pk_fields)
// Unlike other functions in sqllib, this one looks to REQUIRE an active transaction to work
{
  #ifdef use_tcpip_messaging
  if(debug_level[libraries]>=2 && tcpip_message_router) tcpip_message_router->SendMessageToAll("debug", "Getting primary key linked to by " + fk_table + "." + fk_field);
  #endif
  if (!dbTransaction)
   return false;

  pk_table = "";

  if(dbTransaction->DefaultDatabase->TestConnected())
  {
    string pk_sql = "select I2.rdb$relation_name pk_table, IS2.rdb$field_name pk_field "
                    "from rdb$relation_constraints RC "
                    "   inner join rdb$indices I1 on RC.rdb$index_name=I1.rdb$index_name "
                    "   inner join rdb$indices I2 on I1.rdb$foreign_key = I2.rdb$index_name "
                    "   inner join rdb$index_segments IS1 on I1.rdb$index_name = IS1.rdb$index_name "
                    "   inner join rdb$index_segments IS2 on I1.rdb$foreign_key = IS2.rdb$index_name "
                    "where RC.rdb$relation_name='" + fk_table + "' and "
                    "   I1.rdb$relation_name='" + fk_table + "' and "
                    "   IS1.rdb$field_name='" + fk_field + "' and "
                    "   RC.rdb$constraint_type='FOREIGN KEY' "
                    "order by RC.rdb$constraint_name ";
    vector<map<string, string> > pk_sql_results;
    if (!SqlToVecMap(dbTransaction, pk_sql, pk_sql_results))
      return false;
    if(!pk_sql_results.empty())
    {
      pk_table = StripTrailing(pk_sql_results[0]["pk_table"], " ");
      for (unsigned int i=0; i<pk_sql_results.size(); i++)
         pk_fields.push_back(StripTrailing(pk_sql_results[i]["pk_field"], " "));
      return true;
    }
    else
      return false;
  }
  else
    return false;
}
//--------------------------------------------------------------------------
// return the Primary Key that given Foreign Key links to.  This can be multiple fields in one table, so table is string and fields is a vector
bool GetPrimaryKey (TIBDatabase * dbDatabase, string fk_table, string fk_field, string & pk_table, vector<string> & pk_fields)
{
 if (!dbDatabase)
  return false;

 if(dbDatabase->TestConnected())
 {
   TIBTransaction * dbTransaction = new TIBTransaction(NULL);
   StdTransaction (dbTransaction, dbDatabase);
   bool toReturn = true;
   try
   {
     toReturn = GetPrimaryKey (dbTransaction, fk_table, fk_field, pk_table, pk_fields);
   }
   catch (...)
   {
     toReturn = false;
     if (dbTransaction->Active)
       dbTransaction->Rollback();
   }
   if (dbTransaction->Active)
     dbTransaction->Commit();
   delete dbTransaction;
   return toReturn;
 }
 else
   return false;
}
//--------------------------------------------------------------------------
// return the Foreign Keys that link to given Primary Key.
bool GetForeignKeys (TIBTransaction * dbTransaction, string pk_table, string pk_field, vector<map<string, string> > & fks)
// Unlike other functions in sqllib, this one looks to REQUIRE an active transaction to work
{
  #ifdef use_tcpip_messaging
  if(debug_level[libraries]>=2 && tcpip_message_router) tcpip_message_router->SendMessageToAll("debug", "Getting foreign keys to " + pk_table + "." + pk_field);
  #endif
  if (!dbTransaction)
   return false;

  if(dbTransaction->DefaultDatabase->TestConnected())
  {
    string fk_sql = "select RC.rdb$relation_name fk_table, IS1.rdb$field_name fk_field "
                    "from rdb$relation_constraints RC, rdb$indices I1, rdb$indices I2 "
                    "   inner join rdb$index_segments IS1 on I1.rdb$index_name = IS1.rdb$index_name "
                    "   inner join rdb$index_segments IS2 on I1.rdb$foreign_key = IS2.rdb$index_name "
                    "where I2.rdb$relation_name='" + pk_table + "' and "
                    "   IS2.rdb$field_name='" + pk_field + "' and "
                    "   RC.rdb$constraint_type='FOREIGN KEY' and "
                    "   I1.rdb$foreign_key = I2.rdb$index_name and "
                    "   RC.rdb$index_name=I1.rdb$index_name and "
                    "   RC.rdb$relation_name=I1.rdb$relation_name "
                    "order by RC.rdb$constraint_name ";
    #ifdef DERIVEDHEADERS_H
    DebugSql(NULL, &fk_sql, libraries, 9);
    #endif
    if (!SqlToVecMap(dbTransaction, fk_sql, fks))
      return false;
    if(!fks.empty())
    {
      for (unsigned int i=0; i<fks.size(); i++)
      {
         fks[i]["fk_table"] = pk_table = StripTrailing(fks[i]["fk_table"], " ");
         fks[i]["fk_field"] = pk_table = StripTrailing(fks[i]["fk_field"], " ");
      }
      return true;
    }
    else
      return false;              // should zero matching FKs return false?
  }
  else
    return false;
}
//--------------------------------------------------------------------------
// return the Foreign Keys that link to given Primary Key.
bool GetForeignKeys (TIBDatabase * dbDatabase, string pk_table, string pk_field, vector<map<string, string> > & fks)
{
 if (!dbDatabase)
  return false;

 if(dbDatabase->TestConnected())
 {
   TIBTransaction * dbTransaction = new TIBTransaction(NULL);
   StdTransaction (dbTransaction, dbDatabase);
   bool toReturn = true;
   try
   {
     toReturn = GetForeignKeys (dbTransaction, pk_table, pk_field, fks);
   }
   catch (...)
   {
     toReturn = false;
     if (dbTransaction->Active)
       dbTransaction->Rollback();
   }
   if (dbTransaction->Active)
     dbTransaction->Commit();
   delete dbTransaction;
   return toReturn;
 }
 else
   return false;
}
//--------------------------------------------------------------------------
bool GetFieldsFromTable (TIBTransaction * dbTransaction, string table, vector<string>& fields)
// Unlike other functions in sqllib, this one looks to REQUIRE an active transaction to work
{
  #ifdef use_tcpip_messaging
  if(debug_level[libraries]>=2 && tcpip_message_router) tcpip_message_router->SendMessageToAll("debug", "Getting fields from table " + table);
  #endif
  if (!dbTransaction)
   return false;

  string sql;
  if(dbTransaction->DefaultDatabase->TestConnected())
  {
    // get a list of all the fields in the table we were passed
    sql = "select RDB$FIELD_NAME "
          "from RDB$RELATION_FIELDS where RDB$RELATION_NAME = UPPER('" + table + "') "  // jhaley: added UPPER()
          "order by RDB$FIELD_POSITION ";
    if (!SqlToVec(dbTransaction, sql, fields))
    {
      return false;
    }
    if(!fields.empty())
    {
      for (unsigned int i=0; i<fields.size(); i++)
         fields[i] = StripTrailing(fields[i], " ");
      return true;
    }
    else
      return false;              // should zero matching fields return false?
  }
  else
    return false;
}
//--------------------------------------------------------------------------
bool GetFieldsFromTable (TIBDatabase * dbDatabase, string table, vector<string>& fields)
{
 if (!dbDatabase)
  return false;

 if(dbDatabase->TestConnected())
 {
   TIBTransaction * dbTransaction = new TIBTransaction(NULL);
   StdTransaction (dbTransaction, dbDatabase);
   bool toReturn = true;
   try
   {
     toReturn = GetFieldsFromTable (dbTransaction, table, fields);
   }
   catch (...)
   {
     toReturn = false;
     if (dbTransaction->Active)
       dbTransaction->Rollback();
   }
   if (dbTransaction->Active)
     dbTransaction->Commit();
   delete dbTransaction;
   return toReturn;
 }
 else
   return false;
}
//--------------------------------------------------------------------------
// jhaley 20110107: Routine to get a list of the fields in a table in set form, for O(log(N)) searchability.
bool GetFieldSetFromTable(TIBTransaction *dbTransaction, string table, set<string> &field_set)
{
  bool result;
  vector<string> fields;
  result = GetFieldsFromTable(dbTransaction, table, fields);
  VecToSet(fields, field_set);
  return result;
}
//--------------------------------------------------------------------------
// jhaley 20110107: TIBDatabase overload of GetFieldSetFromTable, for convenience and to match all the other routines.
bool GetFieldSetFromTable(TIBDatabase *dbDatabase, string table, set<string> &field_set)
{
  bool result;
  vector<string> fields;
  result = GetFieldsFromTable(dbDatabase, table, fields);
  VecToSet(fields, field_set);
  return result;
}
//--------------------------------------------------------------------------
// jhaley 20110107: Test if a field set contains a given field.
bool FieldSetHasField(set<string> &field_set, const string &field)
{
  return (field_set.find(UppercaseString(field)) != field_set.end());
}
//--------------------------------------------------------------------------
bool GetTablePrimaryKey (TIBTransaction * dbTransaction, string table, vector<string>& pk)
// Unlike other functions in sqllib, this one looks to REQUIRE an active transaction to work
{
  #ifdef use_tcpip_messaging
  if(debug_level[libraries]>=2 && tcpip_message_router) tcpip_message_router->SendMessageToAll("debug", "Getting table's primary key for table " + table);
  #endif
  if (!dbTransaction)
   return false;

  string sql;
  if(dbTransaction->DefaultDatabase->TestConnected())
  {
    // get a list of all the fields in the table we were passed
    sql = "select ISeg.rdb$field_name pk_field "
          "from rdb$relation_constraints RC inner join rdb$indices Idx on "
          "      RC.rdb$index_name=Idx.rdb$index_name and RC.rdb$relation_name=Idx.rdb$relation_name "
          "   inner join rdb$index_segments ISeg on Idx.rdb$index_name = ISeg.rdb$index_name "
          "where Idx.rdb$relation_name='" + table + "' and "
          "   RC.rdb$constraint_type='PRIMARY KEY' ";
    if (!SqlToVec(dbTransaction, sql, pk))
    {
      return false;
    }
    if(!pk.empty())
    {
      for (unsigned int i=0; i<pk.size(); i++)
         pk[i] = StripTrailing(pk[i], " ");
      return true;
    }
    else
      return false;              // should zero matching fields return false?
  }
  else
    return false;
}
//--------------------------------------------------------------------------
bool GetTablePrimaryKey (TIBDatabase * dbDatabase, string table, vector<string>& pk)
{
 if (!dbDatabase)
  return false;

 if(dbDatabase->TestConnected())
 {
   TIBTransaction * dbTransaction = new TIBTransaction(NULL);
   StdTransaction (dbTransaction, dbDatabase);
   bool toReturn = true;
   try
   {
     toReturn = GetTablePrimaryKey (dbTransaction, table, pk);
   }
   catch (...)
   {
     toReturn = false;
     if (dbTransaction->Active)
       dbTransaction->Rollback();
   }
   if (dbTransaction->Active)
     dbTransaction->Commit();
   delete dbTransaction;
   return toReturn;
 }
 else
   return false;
}
//--------------------------------------------------------------------------
bool CreateInsertMap (TIBTransaction * dbTransaction, string table, map<string, string>& ins_map)
// Unlike other functions in sqllib, this one looks to REQUIRE an active transaction to work
{
  #ifdef use_tcpip_messaging
  if(debug_level[libraries]>=2 && tcpip_message_router) tcpip_message_router->SendMessageToAll("debug", "Creating insert map for table " + table);
  #endif
  bool all_is_well = true;
  if (!dbTransaction)
  {
   // PSay("CreateInsertMap: no transaction", m_debug, MB_OK, "sqlLib::CreateInsertMap");
   all_is_well = false;
  }

  string sql;
  if(all_is_well && dbTransaction->DefaultDatabase->TestConnected())
  {
    vector<map<string, string> > fields;
    string sql = "select RelFld.rdb$field_name field_name, RelCon.rdb$constraint_type const_type "
                 "from rdb$relations Rel inner join rdb$relation_fields RelFld on Rel.rdb$relation_name = RelFld.rdb$relation_name "
                 "  left join ( "
                 "    rdb$relation_constraints RelCon inner join rdb$indices Idx on Idx.rdb$index_name = RelCon.rdb$index_name "
                 "      and Idx.rdb$relation_name = '" + UppercaseString(table) + "' and RelCon.rdb$relation_name = Idx.rdb$relation_name "
                 "      and RelCon.rdb$constraint_type = 'FOREIGN KEY' "
                 "    inner join rdb$index_segments ISeg on Idx.rdb$index_name = ISeg.rdb$index_name "
                 "  ) on RelFld.rdb$field_name = ISeg.rdb$field_name and RelFld.rdb$relation_name = Idx.rdb$relation_name "
                 "where Rel.rdb$relation_name = '" + UppercaseString(table) + "' ";
    if (all_is_well && !SqlToVecMap(dbTransaction, sql, fields))
    {
      // PSay("CreateInsertMap: query failed", m_debug, MB_OK, "sqlLib::CreateInsertMap");
      all_is_well = false;
    }
    if(all_is_well && !fields.empty())
    {
      for (unsigned int i=0; i<fields.size(); i++)
      {
        string field_value = LowercaseString(StripTrailing(fields[i]["field_name"], " "));
        if (ins_map.find(field_value)==ins_map.end())
        { // If this key doesn't already exist, add a default value
          // const_type is either going to be "" or "FOREIGN KEY"
          if (StripTrailing(fields[i]["const_type"], " ")!="")
           ins_map.insert(pair<string, string>(field_value, "0"));
          /* an empty string is going to be interpreted as NULL by ExecuteInsertStatement... this is problematic
             if you have a field with a DEFAULT declared that is defined as NOT NULL; if we leave it out of the set,
             Firebird will automagically assign DEFAULT and NULL as appropriate, saving us time and hacks */
          //else
          // ins_map.insert(pair<string, string>(field_value, ""));
        }
      }
      all_is_well = true;
    }
    else
    {
      // PSay("CreateInsertMap: no fields", m_debug, MB_OK, "sqlLib::CreateInsertMap");
      // not necessary // all_is_well = true;              // should zero matching fields return true?
    }
  }
  else
  {
    // PSay("CreateInsertMap: not connected", m_debug, MB_OK, "sqlLib::CreateInsertMap");
    all_is_well = false;
  }

  // The following are default values that should ALWAYS be set but are NOT foreign keys, by table
  // The most common are going to be booleans, which are stored as integers (which should normally default to NULL),
  // but boolean integers should always be 0 or 1.
  // Paul says:  can't you do default values per field at the DB level?  The above code only populates fiels with a
  // constraint type of FOREIGN KEY with default values ... I'm going to try doing it the default way, i.e.:
  // for new fields:

  // to fix existing fields:

  /*
  if (all_is_well)
  {
    if (table == "tasks")
    {
      ending_state["is_educational"] = "0";
      ending_state["hospitalized"] = "0";
      ending_state["in_progress"] = "0";
      ending_state["is_personal"] = "0";
      ending_state["given_room"] = "0";
      ending_state["new_client"] = "0";
    }
  {
  */
  // Paul, much later, says:  What?
  return all_is_well;
}
//--------------------------------------------------------------------------
bool CreateInsertMap (TIBDatabase * dbDatabase, string table, map<string, string>& ins_map)
{
 if (!dbDatabase)
  return false;

 if(dbDatabase->TestConnected())
 {
   TIBTransaction * dbTransaction = new TIBTransaction(NULL);
   StdTransaction (dbTransaction, dbDatabase);
   bool toReturn = true;
   try
   {
     toReturn = CreateInsertMap (dbTransaction, table, ins_map);
   }
   catch (...)
   {
     toReturn = false;
     if (dbTransaction->Active)
       dbTransaction->Rollback();
   }
   if (dbTransaction->Active)
     dbTransaction->Commit();
   delete dbTransaction;
   return toReturn;
 }
 else
   return false;
}
//--------------------------------------------------------------------------
void easy_streaming_dataset::SetTransaction (TIBTransaction * dbTransaction)
{
 _delete_transaction = false;
 if (!dbTransaction)
  throw(NULL);
 if (!dbTransaction->DefaultDatabase)
  throw(NULL);
 if (!dbTransaction->DefaultDatabase->TestConnected())
  throw(NULL);

 _transaction = dbTransaction;
 _database    = _transaction->DefaultDatabase;

 _dataset = NULL;
 while (!_dataset)
  _dataset = new TIBDataSet(NULL);
 _dataset->Database = _database;
 _dataset->Transaction = _transaction;
 _dataset->UniDirectional = true;

 _stringlist = NULL;
 while (!_stringlist)
  _stringlist = new TStringList();
}

void easy_streaming_dataset::SetDatabase (TIBDatabase * dbDatabase)
{
 if (!dbDatabase)
  throw(NULL);
 if (!dbDatabase->TestConnected())
  throw(NULL);

 _database = dbDatabase;

 _transaction = NULL;
 while (!_transaction)
  _transaction = new TIBTransaction(NULL);
 StdTransaction (_transaction, _database);

 SetTransaction(_transaction);
 _delete_transaction = true;
}

easy_streaming_dataset::easy_streaming_dataset(TIBTransaction * dbTransaction)
{
 _debug_level = 1;
 SetTransaction(dbTransaction);
}

easy_streaming_dataset::easy_streaming_dataset(TIBDatabase * dbDatabase)
{
 _debug_level = 1;
 SetDatabase(dbDatabase);
}

easy_streaming_dataset::easy_streaming_dataset(TIBTransaction * dbTransaction, string sql, int p_debug_level)
{
 _debug_level = p_debug_level;
 SetTransaction(dbTransaction);
 SetSql(sql);
}

easy_streaming_dataset::easy_streaming_dataset(TIBDatabase * dbDatabase, string sql, int p_debug_level)
{
 _debug_level = p_debug_level;
 SetDatabase(dbDatabase);
 SetSql(sql);
}

easy_streaming_dataset::~easy_streaming_dataset()
{
 if (_transaction && _transaction->Active)
  _dataset->Close();

 if (_delete_transaction)
 {
  if (_transaction)
   _transaction->Commit();
  delete _transaction;
 }

 delete _dataset;
 delete _stringlist;
}

void easy_streaming_dataset::SetSql(string sql)
{
 #ifdef use_tcpip_messaging
 if(debug_level[libraries]>=1 && tcpip_message_router) tcpip_message_router->SendMessageToAll("debug", "easy_streaming_dataset starting: " + sql);
 #endif

 _stringlist->Clear();
 _stringlist->Add(sql.c_str());
 #ifdef debug_easy_streaming_datasets
 if (_debug_level > 0) // jhaley: runtime debug toggle
   EditSql(NULL, &sql, _transaction);
 #endif

 _dataset->Close();
 _dataset->SelectSQL = _stringlist;
 _dataset->Open();
 _dataset->First();
 _row_count = 0;

 #ifdef use_tcpip_messaging
 if(debug_level[libraries]>=1 && tcpip_message_router) tcpip_message_router->SendMessageToAll("debug", "easy_streaming_dataset ready: " + sql);
 #endif
}

// jhaley 20110304: The EditSql calls in this object are known to sometimes cause problems with unanticipated recursion
// through Atlas event handlers, especially when a previously hidden list view is suddenly displayed and has a deferred
// refresh waiting on it in codePrometheus' queue. I don't want to disable debugging globally, so this method will turn
// it off locally and at runtime, so it can be toggled on or off as need be. Pass in 0 to disable all SQL views.
void easy_streaming_dataset::SetDebugLevel(int i)
{
  _debug_level = i;
}

bool easy_streaming_dataset::Done()
{
 return (_dataset->Eof);
}

void easy_streaming_dataset::Next()
{
 _dataset->Next();
 _row_count++;
}

void easy_streaming_dataset::Reset ()
{
 _dataset->Close();
 _dataset->Open();
 _dataset->First();
 _row_count = 0;
}

bool easy_streaming_dataset::GetRow(map<string, string> & field_map)
{
 if (!_dataset)
  throw(NULL);

 if (Done())
  return false;

 field_map.clear();
 string field_name;
 for (int i = 0; i < _dataset->Fields->Count; i++)
 {
  field_name = LowercaseString(ws(_dataset->Fields->Fields[i]->FieldName));
  field_map[field_name] = _dataset->Fields->Fields[i]->AsString.c_str();
 }
 return true;
}

bool easy_streaming_dataset::GetRow(vector<string> & field_vector)
{
 if (!_dataset)
  throw(NULL);

 if (Done())
  return false;

 field_vector.clear();
 for (int i = 0; i < _dataset->Fields->Count; i++)
  field_vector.push_back(_dataset->Fields->Fields[i]->AsString.c_str());

 return true;
}

unsigned int easy_streaming_dataset::GetRowNumber ()
{
 return _row_count;
}
//--------------------------------------------------------------------------
bool GetSqlPlan (TIBTransaction * dbTransaction, string sql_statement, string &plan)
{
 bool result = true;
 plan = "";
 if (dbTransaction && dbTransaction->Active)
 {
  TIBDataSet* dbDataSet = new TIBDataSet(NULL);
  try
  {
   dbDataSet->Database        = dbTransaction->DefaultDatabase;
   dbDataSet->Transaction     = dbTransaction;
   dbDataSet->SelectSQL->Text = sql_statement.c_str();
   dbDataSet->Prepare();
   plan = dbDataSet->Plan.c_str();
   #ifdef use_tcpip_messaging
   if(debug_level[libraries]>=4 && tcpip_message_router) tcpip_message_router->SendMessageToAll("debug", "Got plan for SQL:\n" + sql_statement + ":  " + plan);
   #endif
  }
  catch(EIBError &error)
  {
    #ifdef use_tcpip_messaging
    if(debug_level[libraries]>=1 && tcpip_message_router) tcpip_message_router->SendMessageToAll("debug", "Error getting plan for SQL:\n" + sql_statement + "\n\nError message:\n" + error.Message.c_str());
    #endif
    last_sql_lib_error = error;
    // not really necessary with a plan, neh?  It doesn't DO anything to roll back....
    //if(dbTransaction->Active)
    //  dbTransaction->Rollback();
    result = false;
  }
  catch (...)
  {
    #ifdef use_tcpip_messaging
    if(debug_level[libraries]>=1 && tcpip_message_router) tcpip_message_router->SendMessageToAll("debug", "Error getting plan for SQL:\n" + sql_statement + "\nA non-Interbase error of some sort occurred.\n");
    #endif
    last_sql_lib_error.Reset();
    last_sql_lib_error.message = "A non-InterBase error of some sort occurred.";
    result = false;
  }
  delete dbDataSet;
 }
 return result;
}
//--------------------------------------------------------------------------
bool GetSqlPlan (TIBDatabase * dbDatabase, string sql_statement, string &plan)
{
 bool all_is_well = true;
 if (!dbDatabase) all_is_well = false;

 if(dbDatabase->TestConnected() && all_is_well)
 {
   TIBTransaction * dbTransaction = new TIBTransaction(NULL);
   if(StdTransaction (dbTransaction, dbDatabase))
   {
     bool toReturn = true;
     try
     {
       toReturn = GetSqlPlan (dbTransaction, sql_statement, plan);
     }
     catch (...)
     {
       toReturn = false;
       //not really necessary with a plan, but we don't want to delete an Active transaction either.
       if (dbTransaction->Active)
         dbTransaction->Rollback();
     }
     //not really necessary with a plan, but we don't want to delete an Active transaction either.
     if (dbTransaction->Active)
       dbTransaction->Commit();
     delete dbTransaction;
     return toReturn;
   }
 }
 else all_is_well = false;

 //if we're here, we already know all_is_well is false.
 #ifdef use_tcpip_messaging
 if(debug_level[libraries]>=1 && tcpip_message_router) tcpip_message_router->SendMessageToAll("debug", "Error getting plan for SQL:\n" + sql_statement + "\nA non-Interbase error of some sort occurred.\n");
 #endif
 last_sql_lib_error.Reset();
 last_sql_lib_error.message = "A non-InterBase error of some sort occurred.";
 return false;
}
//--------------------------------------------------------------------------
string SqlOptionalBetween (string field, string a, string b)
{
 Pdate pa (a, global_mode);
 Pdate pb (b, global_mode);
 if (pa.IsValidDate() && pb.IsValidDate())
 {
  return field + " between '" + pa.OutDate(global_mode) + "' and '" + pb.OutDate(global_mode) + "'";
 } else
 if (pa.IsValidDate())
 {
  return field + " >= '" + pa.OutDate(global_mode) + "'";
 } else
 if (pb.IsValidDate())
 {
  return field + " <= '" + pb.OutDate(global_mode) + "'";
 } else
 {
  return "1 = 1";
 }
}
//--------------------------------------------------------------------------
string TranslateSqlPlan(TIBTransaction* dbTransaction, string plan)
{
 string answer;

 map<string, string> index_name_to_field_names;
 vector<string> big_chunks;
 set<string> index_names;
 SplitStringToVec(plan, " INDEX (", big_chunks);
 if (!big_chunks.empty())
  for (unsigned int i = 1; i < big_chunks.size(); i++)
  {
   SplitStringToSet(big_chunks[i].substr(0, big_chunks[i].find(")")), ",", index_names);
   for (set<string>::iterator j = index_names.begin(); j != index_names.end(); j++)
   {
    vector<string> fields_involved;
    string field_name = "'" + SafeSQLString(*j) + "'";
    if (index_name_to_field_names.find(field_name) == index_name_to_field_names.end())
    {
     // string table_name = GetOneField(dbTransaction, "select RDB$INDICES.RDB$RELATION_NAME from RDB$INDICES where RDB$INDICES.RDB$INDEX_NAME = " + field_name);
     // if (!table_name.empty())
     {
      SqlToVec(dbTransaction, "select RDB$INDEX_SEGMENTS.RDB$FIELD_NAME from RDB$INDEX_SEGMENTS where RDB$INDEX_NAME = " + field_name + " order by RDB$FIELD_POSITION", fields_involved);
      for (vector<string>::iterator k = fields_involved.begin(); k != fields_involved.end(); k++)
       *k = StripTrailing(*k, " ");
      index_name_to_field_names[*j] = VecToDelimString(fields_involved, "", "", " + ");
     }
    }
   }
  }
 answer = plan;
 for (map<string, string>::iterator k = index_name_to_field_names.begin(); k != index_name_to_field_names.end(); k++)
  answer = ReplaceStringWithString(answer, k->first, k->second);

 return answer;
}
//--------------------------------------------------------------------------
bool IsIndexed(TIBTransaction * dbTransaction, string table_name, string field_name)
{
 if ("1" == GetOneField(dbTransaction, "select '1' from RDB$INDEX_SEGMENTS inner join RDB$INDICES on RDB$INDICES.RDB$INDEX_NAME = RDB$INDEX_SEGMENTS.RDB$INDEX_NAME where RDB$INDEX_SEGMENTS.RDB$FIELD_POSITION = 0 and RDB$INDEX_SEGMENTS.RDB$FIELD_NAME = '" + SafeSQLString(field_name) + "' and RDB$INDICES.RDB$RELATION_NAME = '" + SafeSQLString(table_name) + "'"))
  return true;
 else
  return false;
}
//--------------------------------------------------------------------------
// defaults: SafeInClause( , , "0=1", false, 1000);
//
// firebird has a limit to the number of items in an in () clause, after which it craps out.  So we'll take a list of items,
// split it into max_values_in_groups sized chunks (at most), and write the in clause as necessary, e.g.
// field in (...)  ; or,  (field in (...) or field in (...) ...)  ; or,  result_if_blank
//
// the result_if_blank is aimed at compound statements.  e.g., say you have
// where x = y and z in (...)
// if there are no values to stick in the parens, rather than leave your sql with a trailing and ... and breaking it,
// we simply return false.  where x = y and false.  which is, actually, how you'd want that to evaluate in most compound clauses.
//
// note that if the /only/ statement you have in your where clause IS the in () clause, "where false" won't fly.  passing "1=0" for
// result_if_blank will though.  where 1=0 will return no results, which is what you want if there's nothing to match against.
//
// TODO -c oversight : if you pass a vector with only empty string(s), you'll end up with a in (), which is bad
// (and the very thing result_if_blank is designed to work around); this case needs to be specifically coded for.
//
string SafeInClause(string field, vector<string> values, string result_if_blank, bool quote_values, unsigned int max_values_in_group)
{
  if (values.empty())
    return result_if_blank;
  int beg_of_chunk = 0;
  int end_of_chunk = min(values.size(), max_values_in_group);
  vector<string> chunks;
  string sql;
  while (end_of_chunk > beg_of_chunk)
  {
    string ids = "";
    if (quote_values)
      for(int i=beg_of_chunk; i<end_of_chunk; i++)
        ids += ("'" + values[i] + "',");
    else
      for(int i=beg_of_chunk; i<end_of_chunk; i++)
        ids += (values[i] + ",");
    ids = ids.substr(0, ids.length() - 1);

    chunks.push_back(field + " in (" + ids + ")");
    beg_of_chunk = end_of_chunk;
    end_of_chunk = min(beg_of_chunk + max_values_in_group, values.size());
  }
  string result = VecToDelimString(chunks, "", "", " or ");
  if (chunks.size() > 1)
    result = "(" + result + ")";
  return result;
}
//--------------------------------------------------------------------------
string SafeInClause(const string &field, const vector<string> &values)
{
  return SafeInClause(field, values, "0=1", false, 1000);
}
//--------------------------------------------------------------------------
bool DataSetToCSV(TDataSet *DS, string FN, bool headers)
{
  fstream f;
  if (f.is_open()) f.close();
  try
  {
    f.open(FN.c_str(), ios::out | ios::trunc);
  }
  catch (...) { return false; }
  if (!f.is_open()) return false;

  try
  {
    if (!DS->Active)
      DS->Open();
    DS->First();
    string answer;
    if (headers)
    {
      for(int i = 0; i < DS->FieldCount; i++)
      {
        if (i > 0)
          answer += ",";
        answer += "\"" + ReplaceStringWithString(ReplaceAnyCharWithString(ws(DS->Fields->Fields[i]->FieldName), "\a\f\n\r\t", "?"), "\"", "\"\"") + "\"";
      }
      f << answer << endl;
    }
    while (!DS->Eof)
    {
      answer = "";
      for(int i = 0; i < DS->FieldCount; i++)
      {
        if (i > 0)
          answer += ",";
        if (!DS->Fields->Fields[i]->IsNull)
          answer += "\"" + ReplaceStringWithString(ReplaceAnyCharWithString(DS->Fields->Fields[i]->AsString.c_str(), "\a\f\n\r\t", "?"), "\"", "\"\"") + "\"";
      }
      f << answer << endl;
      DS->Next();
    }
  }
  catch (...) { return false; }
  if (f.is_open()) /* and it should be */ f.close();
  return true;
}
//--------------------------------------------------------------------------
// jhaley 20110224: I had to write this code for debugging back when I had issues with the Appt Outcome Details Report
// with TLargeintField's lacking of support in QuickReports, so I figured I'd keep it around. Handy for viewing the
// contents & format of a dataset in a human-readable form.
string DataSetDebugView(TIBDataSet *dbDataSet)
{
  string msg;
  int row_num = 1;

  while(!dbDataSet->Eof)
  {
    msg += string("Row ") + IntToString(row_num) + ":\n";
    for(int i = 0; i < dbDataSet->Fields->Count; i++)
    {
      TField *f = dbDataSet->Fields->Fields[i];
      string      field_name = ws(f->FieldName);
      TFieldKind  fk         = f->FieldKind;
      TFieldType  dt         = f->DataType;
      string      dt_str;
      string      fk_str;
      string      value;
      switch(fk) // Field Kind
      {
      case fkData:         fk_str = "Data";    break;
      case fkCalculated:   fk_str = "Calc";    break;
      case fkLookup:       fk_str = "Lookup";  break;
      case fkInternalCalc: fk_str = "ICalc";   break;
      case fkAggregate:    fk_str = "Aggr";    break;
      default:             fk_str = "Unknown"; break;
      }
      switch(dt) // Field Type
      {
      case ftInteger:    dt_str = "Integer";    break;
      case ftSmallint:   dt_str = "Smallint";   break;
      case ftLargeint:   dt_str = "Largeint";   break;
      case ftString:     dt_str = "String";     break;
      case ftWideString: dt_str = "WideString"; break;
      case ftWord:       dt_str = "Word";       break;
      case ftVariant:    dt_str = "Variant";    break;
      default:           dt_str = "Other (" + IntToString((int)dt) + ")"; break;
      }
      try // Value - this isn't guaranteed to succeed (for example, blobs may not support AsString)
      {
        value = f->AsString.c_str();
      }
      catch(...)
      {
        value = "(Exception in AsString)";
      }
      msg += string("  Field ") + field_name + ": " + fk_str + ", " + dt_str + ", " + value + "\n";
    }
    dbDataSet->Next();
    ++row_num;
  }
  dbDataSet->First(); // rewind the dataset

  return msg;
}

