/*

   HL7Daemon - String processing utilities

*/

#ifndef STRINGUTILS_H__
#define STRINGUTILS_H__

class Pdate;
class Ptime;
class Pstamp;

// Split a string similar to A=B into A and B
bool SplitKeyValue(const string &incoming, string &key, string &value);

// Strip off the first character of a string and return a new copy
string StripFirst(const string &incoming);

// Strip off the last character of a string and return a new copy
string StripLast(const string &incoming);

// Returns true if the string is surrounded with [], false otherwise.
bool IsBracketed(const string &incoming);

// Duplicate a string constant using new [] rather than malloc
char *cpp_strdup(const char *input);

// HL7 conversion routines
bool hl7_BLToBoolean(const string &value, const string &true_value);
void hl7_DTToPdate(const string &value, Pdate &answer);
void hl7_TMToPtime(const string &value, Ptime &answer);
void hl7_TSToPstamp(const string &value, Pstamp &answer);
void hl7_GenderToGenderID(const string &hl7gender, string &promID);
void hl7_RaceToRaceID(const string &hl7race, string &promID);
void hl7_MaritalStatusToMaritalID(const string &hl7ms, string &promID);
void hl7_SSNToNumeric(const string &hl7SSN, string &promSSN);
void hl7_StateToStateID(const string &hl7state, string &promID);
void hl7_NormalizeZip(const string &hl7zip, string &outZip);
void hl7_NormalizePhone(const string &hl7phone, string &outPhone);

#endif

// EOF

