/*

   Basic Utilities
   jhaley 20120525

*/

#include <cstdio>
#include <cstdlib>
#include <cstring>
#include <cstdarg>
#include <iostream>
#include <fstream>

#include "cached_files.h"
#include "globaldefines.h"
#include "utils.h"

using namespace std;

// I hate this warning: "Possibly incorrect assignment"
#pragma warn -8060

//
// Globals
//

// Controls the behavior of the DEBUGOUT macro
unsigned int debug_msg_flags;
ostream *debug_ostr = &cout;

//
// Module Statics
//

// debug flags
struct debugflag_t
{
   const char  *name;
   unsigned int flag;
};

static debugflag_t debug_flags[] =
{
   { "database",  dbg_database  },
   { "parser",    dbg_parser    },
   { "main",      dbg_main      },
   { "error",     dbg_error     },
   { "library",   dbg_library   },
   { "sql",       dbg_sql       },
   { "scanner",   dbg_scanner   },
   { "processor", dbg_processor },
   { "matcher",   dbg_matcher   },
   { "all",       0xFFFFFFFFu   },
   { NULL,        0             } // must be last.
};

//
// hl7_SetDebugFlags
//
// Setup debugging message properties.
//
void hl7_SetDebugFlags(const char *input)
{
   unsigned int tempflags;
   char *tempbuf = strdup(input);
   char *rover = tempbuf;

   char *end = NULL;
   unsigned long asNum;

   asNum = strtoul(input, &end, 16);

   if(end && *end == '\0') // it's a hex number?
   {
      tempflags = static_cast<unsigned int>(asNum);
   }
   else // must be a set of flag names.
   {
      for(; (rover = strtok(rover, ",+|")); rover = NULL)
      {
         int idx = 0;

         while(debug_flags[idx].name != NULL)
         {
            if(!stricmp(rover, debug_flags[idx].name))
            {
               tempflags |= debug_flags[idx].flag;
               break;
            }
            ++idx;
         }
      }
   }

   debug_msg_flags = tempflags;

   debug_msg_flags |= dbg_error; // ERROR messages are always given.

   free(tempbuf);
}

void hl7_DebugInitLog(bool usefile)
{
   using namespace FileCache;

   if(usefile) // want to write debug output to a file?
   {
      string filename = PathConcatenate(GetBasePath(), DEBUG_LOG_NAME);
      ofstream *logfile = new ofstream();

      logfile->open(filename.c_str(), ios::out|ios::app|ios::ate);
      if(logfile->is_open())
         debug_ostr = logfile;
      else
         delete logfile;
   }
   // Otherwise, output is to the console via cout.
}

//
// D_HashTableKeyCase
//
// Based on SGI STL string hash; fast, with good hash key properties.
// Available under BSD license.
//
unsigned int D_HashTableKeyCase(const char *str)
{
   const char *c = str;
   unsigned int h = 0;

   while(*c)
   {
      h = 5 * h + *c;
      ++c;
   }

   return h;
}

//
// LoadTextFile
//
// Load a text file from disk. Returns null if fails.
//
char *LoadTextFile(const char *filename)
{
   FILE *f;
   char *buffer;
   size_t size;

   if(!(f = fopen(filename, "rb")))
      return NULL; // can't open file

   fseek(f, 0, SEEK_END);
   size = ftell(f);
   fseek(f, 0, SEEK_SET);

   if(size <= 0)
   {
      fclose(f);
      return NULL; // nothing in the file
   }

   buffer = new char [size + 1]; // size+1 for null termination
   memset(buffer, 0, size + 1);

   if(fread(buffer, 1, size, f) != size)
   {
      delete [] buffer;
      fclose(f);
      return NULL; // can't read the file
   }

   fclose(f);

   return buffer; // success
}

//
// FatalError
//
// Will stop the program's execution after posting an error message.
//
void FatalError(const char *msg, ...)
{
   va_list args;

   va_start(args, msg);

   vprintf(msg, args);
   exit(-1);

   // Not really reachable, just a formality.
   va_end(args);
}

// EOF

