/*

   Basic Utilities
   jhaley 20120525

*/

#ifndef UTILS_H__
#define UTILS_H__

#include <ostream>


//=============================================================================
//
// Macros
//

// Debug message reasons
enum
{
  dbg_database  = 0x00000001u, // prometheusdb module
  dbg_parser    = 0x00000002u, // verbose parser logging
  dbg_main      = 0x00000004u, // main module
  dbg_error     = 0x00000008u, // for errors that might happen in the release build
  dbg_library   = 0x00000010u, // util errors
  dbg_sql       = 0x00000020u, // sqlLib errors
  dbg_scanner   = 0x00000040u, // scanner module
  dbg_processor = 0x00000080u, // processor module
  dbg_matcher   = 0x00000100u, // matcher module
};

extern unsigned int debug_msg_flags;
extern std::ostream *debug_ostr;
void hl7_DebugInitLog(bool usefile);

// Uncomment this to quash all debug output to the console
//#define NODEBUGOUTPUT

#ifndef NODEBUGOUTPUT
#define DEBUGOUT(reason, s) \
   do { if(debug_msg_flags & (reason)) (*debug_ostr) << s << endl; } while(0)
#else
#define DEBUGOUT(reason, s)
#endif

// Calculate the length of a statically defined array
#define arrlen(x) (sizeof(x) / sizeof(*(x)))

//=============================================================================
//
// Utility Functions
//

void hl7_SetDebugFlags(const char *input);
unsigned int D_HashTableKeyCase(const char *str);  // hash a string
char *LoadTextFile(const char *filename);          // load a text file
void FatalError(const char *msg, ...);             // GTFO the whole program.

#endif

// EOF

