/*
   HL7Daemon
   
   Person matching query
   
   Retrieves the set of people considered a possible match given the following
   input parameters from an HL7Intermediate:
     
     PERSON_ID  : ID number of person
     SSN        : Social security number
     GENDER_ID  : gender_types ID number
     BIRTH_DATE : person birth date
     LAST_NAME  : Last name
     FIRST_NAME : First name
     MI         : Middle initial; this is matched using a "like" operator
     SUFFIX     : Suffix (ie. Jr., Sr.)
     
   Returns: vector of (id, score), ordered by score descending
*/

#define NOTZERO(field, value)  (field <> 0  and field = value)
#define NOTBLANK(field, value) (field <> '' and upper(field) = value)
#define NOTBLIKE(field, value) (field <> '' and upper(field) like value)

select P.id,
  (case when P.id = PERSON_ID                        then 30 else 0 end +
   case when NOTBLANK(P.social_security_number, SSN) then 25 else 0 end +
   case when NOTZERO(P.gender_id, GENDER_ID)         then  5 else 0 end +
   case when P.birth_date = BIRTH_DATE               then 25 else 0 end +
   case when NOTBLANK(PN.last_name,   LAST_NAME)     then  7 else 0 end +
   case when NOTBLANK(PN.first_name,  FIRST_NAME)    then  5 else 0 end +
   case when NOTBLIKE(PN.middle_name, MI)            then  2 else 0 end +
   case when upper(PN.suffix) = SUFFIX               then  1 else 0 end) score
from people P
left join person_names PN on P.person_names_id = PN.id
where
  P.gender_id in (0, GENDER_ID) and
  (P.id = PERSON_ID or
   exists (
     select * from person_names IPN where IPN.person_id = P.id and 
     IPN.current_id >= 2 and 
     (upper(IPN.last_name) = LAST_NAME or
      upper(IPN.first_name) = FIRST_NAME)
   ) or
   P.birth_date = BIRTH_DATE or
   P.social_security_number = SSN)
order by score desc
