//------------------------------------------------------------------------

#ifndef Filter_H
#define Filter_H
//------------------------------------------------------------------------
#include <Classes.hpp>
#include <Controls.hpp>
#include <StdCtrls.hpp>
#include <Forms.hpp>
#include <Menus.hpp>
#include <ComCtrls.hpp>
#include <ToolWin.hpp>
#include <ImgList.hpp>
#include <Db.hpp>
#include <IBDatabase.hpp>
#include <IBCustomDataSet.hpp>
#include <IBSQL.hpp>
#include <Buttons.hpp>
#include <string>
#include <vector>
#include <map>
#include <stdlib.h>
#include <stdio.h>
#include "util.h"
#include <deque>
#ifdef use_tcpip_messaging
#include "tcpip_messaging.h"
#endif
#ifdef use_prometheus_util
#include "prometheus_util.h"
#endif
#include "fieldtypes.h"
#include "allsymbols.h"
#include "repeating.h"

using namespace std;

//------------------------------------------------------------------------
int CompareByType (string a, string b, sort_type as_what);
string FormatString (string tempstring, show_type columntype);
//------------------------------------------------------------------------
template <class T>
int TemplateCompare (T a, T b);
//------------------------------------------------------------------------
class  classCurrent
{
 public:
  bool showCurrent;
  string currentItem;
 public:
  classCurrent(bool d, string f);
};
//------------------------------------------------------------------------
enum filter_column_alignments {fc_default, fc_left, fc_right};
class classColumn
{
 public:
  bool showCol;
  string columnName;
  sort_type as_what;
  int ColumnWidth;
  int AutosizeWidth;
  show_type StringType;
  string requires;
  TListColumn * currently_at;
  TAlignment align_how;
  pair<int, int> indices; // jhaley 20100902: first == index into parent ColumnVec; second == original index into listview Columns
 public:
  classColumn(string a, string r, sort_type b, bool c, int d, show_type e, TAlignment al, int aw=0, int colnum=0);
};
//------------------------------------------------------------------------
LRESULT FilterRedirectedWndProc(HWND hwnd, UINT msg, WPARAM wparam, LPARAM lparam);

// empty little thing
class classFilterSource
{
 public:
  classFilterSource() {};
  virtual bool Begin(TIBTransaction * dbTransaction) = 0;
  virtual unsigned int GetFieldCount() = 0;
  virtual bool GetFieldNames(vector<string> & field_vec) = 0;
  virtual bool GetFieldTypes(vector<TFieldType> & field_vec) = 0;
  virtual bool GetRow(vector<string> & field_vec) = 0;
  virtual void End() = 0;
  virtual ~classFilterSource() {};
};

// smart little booger.
class classFilterSqlSource : public classFilterSource
{
 private:
  string FSqlStatement;
  TIBTransaction * FTransaction;
  TIBDataSet * FDataSet;
  TStringList * FStringList;
 public:
  classFilterSqlSource();
  virtual bool Begin(TIBTransaction * dbTransaction);
  virtual unsigned int GetFieldCount();
  virtual bool GetFieldNames(vector<string> & field_vec);
  virtual bool GetFieldTypes(vector<TFieldType> & field_vec);
  virtual bool GetRow(vector<string> & field_vec);
  virtual void End();
  virtual ~classFilterSqlSource();
  // additional
  virtual void SetSqlStatement(string sql);
  virtual string GetSqlStatement();
};

class classFilterObject;

class classFilterVecmapSource : public classFilterSource
{
 protected:
  // input
  classFilterObject * FFilter;

  // output
  vector<string> field_names;
  deque<vector<string> > field_values;
  vector<map<string,string> > source_vecmap;
  vector<TFieldType> field_types;
  unsigned int row_index;
  map<string, string> sql;
 public:
  classFilterVecmapSource();
  virtual bool Begin(TIBTransaction * dbTransaction);
  virtual bool SetSource(vector<map<string, string> > &src);
  virtual unsigned int GetFieldCount();
  virtual bool GetFieldNames(vector<string> & field_vec);
  virtual bool GetFieldTypes(vector<TFieldType> & field_vec);
  virtual bool GetRow(vector<string> & field_vec);
  virtual void End();
  virtual ~classFilterVecmapSource();

  virtual void SetFilter (classFilterObject * f);
};

// dumb as a rock, for hacking purposes only (or for deriving? but that'd be stupid too, what with the publicness.)
class classFilterSimpleSource : public classFilterSource
{
 private:
  unsigned int row_index;
 public:
  vector<string> field_names;
  vector<TFieldType> field_types;
  deque<vector<string> > field_values;
 public:
  classFilterSimpleSource();
  virtual bool Begin(TIBTransaction * dbTransaction);
  virtual unsigned int GetFieldCount();
  virtual bool GetFieldNames(vector<string> & field_vec);
  virtual bool GetFieldTypes(vector<TFieldType> & field_vec);
  virtual bool GetRow(vector<string> & field_vec);
  virtual void End();
  virtual ~classFilterSimpleSource();
};

// useful. more useful as a real component. with a T prefix and all that.
class classFilterObject
{
 public: // and static
  static TIBTransaction * default_transaction;
  static bool always_autosize_columns;
  static TNotifyEvent funcDoubleClick;
  static TLVColumnClickEvent funcColumnClick;
  static TLVAdvancedCustomDrawEvent funcAdvancedCustomDraw;

  static TNotifyEvent funcClick;
  static TKeyEvent funcKeyDown;
  static TLVOwnerDataEvent funcNeedData;
  static TLVOwnerDataFindEvent funcFindNextData;
  static TLVOwnerDataHintEvent funcNeedDataRange;
  static TLVOwnerDataStateChangeEvent funcStateChange;

  static map<string, map<string, string> >* user_permissions;
  static map<string, map<string, string> >* user_preferences;
  static TPopupMenu* default_popup_menu;
  static TImageList* default_image_list;
  static TIBDatabase* default_database;
  // static int upper_bound_sort_quantity;

  typedef void (*DeleteCallback)(classFilterObject *);
  static DeleteCallback funcDelete; // jhaley: notify callback when object is deleted

 private:
  classFilterSqlSource sql_source;
  classFilterSource * data_source; // delete in destructor only if != NULL and != &sql_source

  TIBTransaction * local_transaction;
  TListView      * listSearchResults;
  WNDPROC          original_window_proc;
  TStaticText    * listview_label;
  TObject        * associatedForm;
  TObject        * wheretoAddEdit;
  TObject        * wheretoCheck;
  TObject        * wheretoInform;
  AddRecord        howtoAdd;
  EditRecord       howtoEdit;
  CheckRecord      howtoCheck;
  InformForm       howtoInform;
  TListColumn    * sort_column_pointer;
  TPopupMenu     * popup_menu;
  TImageList     * image_list;
  TIBDatabase    * local_database;
  // displaced event handlers - jhaley 20110614
  bool             saved_onclick_handler;
  TNotifyEvent     DisplacedOnClickHandler;
  // attributes
  bool   is_multiselect;
  bool   is_checkboxes;
  bool   is_compress;
  bool   is_check_locked;
  bool   is_hiding_unchecked;
  bool   show_headers;
  bool   show_icons;
  bool   auto_column_specs;
  bool   auto_format_by_type;
  // status
  bool   is_pending;
  bool   is_pending_check_all;
  bool   is_pending_check_where_id;
  bool   is_make_checked_where_id;
  bool   is_pending_check_where_not_id;
  bool   is_make_checked_where_not_id;
  bool   is_refreshing_listview;
  bool   lack_columns;
  bool   lack_dataset;
  bool   sort_asc;
  int    prev_scroll;
  string listview_hint;
  string listview_text;
  set<string> set_where_id;
  set<string> set_where_not_id;

  // to replace dbDataSet
  vector<string> field_names;
  vector<TFieldType> field_types;
  deque<vector<string> > raw_data;
  deque<vector<string> > visible_data;
  set<string> checked_data;

  // features!
  int    column_id;
  int    column_icon;
  int    column_valid;
  int    column_color_swatch;

  int    visible_row_count; // shown objects
  int    all_row_count;     // includes hidden objects
  int    sort_column_index;

  string requires;
  string preferences_name;
  string original_preferences;
  TRect  floating_hint_pos;

  bool AutoGenerateColumnSpecs();

  bool CheckRawDataRowSizes(int index); // jhaley 20110907

  // debugging
  unsigned int compare_count;

 private: // and static
  static TPoint floating_hint_offset;
  static TTimer *timerFilterHint;
  static THintWindow *hintFloating;
  static TListView *listPrevious;

 public: // and static
  static bool EnableTransaction  (TIBDatabase * dbDatabase);
  static void DisableTransaction ();
  static bool EnableFloatingHint ();
  static void DisableFloatingHint ();
  static void SetAutosizeColumns(bool);
  static bool GetAutosizeColumns();

 public:
  vector<classColumn>  ColumnVec;
  vector<classCurrent> CurrentVec;

  classFilterObject();
 ~classFilterObject();

  // Get and Set
  string GetSQLStatement ();
  void   SetSQLStatement (const string &);
  bool   GetHeader ();
  void   SetHeader (bool);
  bool   GetIconStatus ();
  void   SetIconStatus (bool);
  string GetPreferences ();
  void   SetPreferences (const string & p_preferences);
  void   ThrowPreferences ();
  void   FetchPreferences (string p_name);
  string GetDefaultPreferences ();
  void   SetMulti (bool);
  bool   GetMulti ();
  void   SetCheck (bool);
  bool   GetCheck ();
  void   SetCompress (bool);
  bool   GetCompress ();
  void   SetAutoColumnSpecs(bool, bool);
  void   GetAutoColumnSpecs(bool&, bool&);
  void   SetCheckLocked (bool);
  bool   GetCheckLocked ();
  void   SetHideUnchecked (bool);
  bool   GetHideUnchecked ();
  void SetDataSource(classFilterSource * ds);
  classFilterSource * GetDataSource();

  // Set
  void   SetListViewHandle (TListView * tempListView);
  void   SetAddEdit (TObject * p_form, AddRecord p_howtoAdd, EditRecord p_howtoEdit);
  void   SetCheck (TObject * p_form, CheckRecord p_howtoCheck);
  void   SetInform (TObject * p_form, InformForm p_howtoInform);
  void   ChangeColumnIcon (TListColumn *Column, bool flip_direction = true);
  void   SetSecurity (const string & p_table_name);
  void   SetTransaction (TIBTransaction * p_local_transaction);
  void   SetHint (string);

  // Get
  TListView * GetListViewHandle ();
  bool   HasData ();
  int    GetResultSize ();
  int    GetDatasetSize ();
  string GetSelectedRecordID ();
  bool   SetSelectedRecordID (const string &id); // jhaley 20110906
  vector<string> GetMultiID ();
  AddRecord  HowToAdd ();
  EditRecord HowToEdit ();
  CheckRecord HowToCheck ();
  InformForm  HowToInform ();
  TObject*    WhereAction ();
  string GetSecurity ();
  bool   HasSecurityView ();
  bool   HasSecurityAdd ();
  bool   HasSecurityEdit ();
  vector<map<string, string> > GetAllData (bool use_original_field_names, bool selected_rows_only = false);
  vector<vector<string> > GetAllData (vector<string> &col_names, vector<show_type> &col_types, bool use_original_field_names, bool selected_rows_only = false);
  vector<string> GetAllID ();
  int    GetIconColumnIndex ();
  // planned functions that might be nice, eh?
  // vector<string> GetVisibleID (); -- see below
  // vector<string> GetIDWhere (form, function taking map<string, string> and returning bool); -- what about visibility?
  bool IsRefreshingListview() {return is_refreshing_listview;}

  const vector<string> & GetFieldNames();
  const vector<TFieldType> & GetFieldTypes();
  const deque<vector<string> > & GetRawData();

  // Add
  void   AddColumnSpec (string columnName, string requires_table, sort_type as_what, bool showCol = true,
                        int ColumnWidth = 0, show_type StringType = prettyString, filter_column_alignments al = fc_default);
  void   AddCurrentSpec (bool showCurrent, string currentItem);

  // Execute
  void   Sort(TListColumn *Column, bool flip_direction = true);
  void   RefreshDataSet  ();
  void   ForceRefreshDataSet ();
  void   RefreshListView (bool apply_hiding_unchecked_logic = false);
  void   RefreshHeaders  ();
  void   UpdateHint      ();
  void   UpdateText      ();
  void   AutosizeColumns ();
  bool   ClearColumnSpecs();

  // Utility
  int    CompareByColumn (long Param1,long Param2, long SortColumn);
  string FullPerm (string p_partial_pref);
  void   MoveToFirstUsefulItem ();
  void   RefreshIfPending ();

  // Because
  bool   CanFilter   (); // current_id must be a known field
  bool   CanIdentify (); // internal_id must be a known field

  // checking stuff
  void   CheckAll   (); // danger: asynchronous ... use SetCheck(form, function) for initial checking
  void   UncheckAll (); // danger, same
  void   FlipCheck  (); // danger, more of the same

  void CheckSelected ();
  void UncheckSelected ();
  void FlipCheckSelected ();

  void   SetCheckedWhereID (const set<string> & id_to_check, bool make_checked);
  void   SetCheckedWhereNotID (const set<string> & id_to_check, bool make_checked);

  // events
  void __fastcall listSearchResultsMouseMove(TObject *Sender, TShiftState Shift, int X, int Y);
  static void __fastcall timerFilterHintTimer(TObject *Sender);
  void SetRowAttributes(TListItem *Item);
  void KeyPressed(WORD &Key, TShiftState Shift);
  void Clicked(TObject *Sender);
  // void ListViewStateChange(int StartIndex, int EndIndex, TItemStates OldState, TItemStates NewState);
  friend LRESULT FilterRedirectedWndProc(HWND hwnd, UINT msg, WPARAM wparam, LPARAM lparam);
};
//------------------------------------------------------------------------
/* extern TIBDatabase * dbDatabase;
extern TImageList *imagesCurrent;
extern TImageList *imagesRecordTypes;
extern TPopupMenu *popupListActions;
extern map<string, map<string,string> > optionsPermissions;
extern map<string, map<string,string> > optionsPreferences; */
//------------------------------------------------------------------------
#endif