#ifndef allsymbols_h
#define allsymbols_h

#include <forms.hpp>
#include <string>
using namespace std;

// general purpose bit flag stuff... if using | and &, #define with these
// for consistency and so we can easily spot things in this file that should
// be enums instead!
// <!--  column 25 ...  v  -->
#define  bit_none       0      // no flags set
#define  bit_1          1      // first flag set
#define  bit_2          2      // second flag set
#define  bit_3          4      // third flag set
#define  bit_4          8      // ...
#define  bit_5          16     // ...
#define  bit_6          32     // ...
#define  bit_7          64     // ...
#define  bit_8          128    // a byte's worth of flags!
#define  bit_9          256    // ...
#define  bit_10         512    // ...
#define  bit_11         1024   // ...
#define  bit_12         2048   // ...
#define  bit_13         4096   // ...
#define  bit_14         8192   // ...
#define  bit_15         16384  // ...
#define  bit_16         32768  // an integer's worth of flags!
                               // TODO: go to longint if necessary...

// centralized list of defines, enums, typedefs, etc. that the libraries use.
// split into sections based on where they were used, including a description
// of their intent.

// util.h
//-----------------------------------------------------------------------------
// current - not used
// history - was used for the AddToSelectSQL function in hyperion
//#define contains        0
//#define ends_with       1
//#define exact_match     2
//#define starts_with     3

// current - used as a parameter to StripAllExcept, ints mostly
#define ints            "1234567890-+"
#define uints           "0123456789"
#define hexints         "0123456789ABCDEFabcdefXx"
#define floats          "1234567890.+-"
#define ufloats         "1234567890."
#define alphas          "abcdefghijklmnopqrstuvwxyz"
#define alpha_numerics  "abcdefghijklmnopqrstuvwxyz1234567890"
#define upper_alphas         "ABCDEFGHIJKLMNOPQRSTUVWXYZ"
#define upper_alpha_numerics "ABCDEFGHIJKLMNOPQRSTUVWXYZ1234567890"

// current - used by all Pdate/Ptime/Pstamp funcitons as "mode"
// notes   - can generally be cumulated with the pipe symbol
#define unknown         bit_none
#define yy              0
#define yyyy            bit_1
#define mdy             0
#define ymd             bit_2
#define dmy             bit_3
#define ampm            0
#define military        bit_4
#define timefirst       0
#define datefirst       bit_5
#define withsec         0
#define withoutsec      bit_6
#define no_zeros        0
#define left_zeros      bit_7

// current - used by all Pdate/Ptime/Pstamp functions as "metric"
// notes   - cannot be combined. this changes the meaning of "distance"
#define in_undef        bit_none
#define in_seconds      bit_1
#define in_minutes      bit_2
#define in_hours        bit_3
#define in_days         bit_4
#define in_weeks        bit_5
#define in_months       bit_6
#define in_years        bit_7

// current - used by Pdate/Ptime/Pstamp anytime a mode is not
//           specified by the user. reflects generally accepted parameters.
// mostly used by filter/edit_listview for sorting (comes this way from db)
 #define global_mode    yyyy | mdy | military | datefirst

// current - used by Pdate/Ptime/Pstamp as standard date/time separators
#define datechar        '/'
#define timechar        ':'
#define y2kcut          1905

// current - used by Like() as meta characters for simple regular expressions.
// notes   - corresponds to interbase/firebird's standard characters.
#define match_any       '%'
#define match_one       '_'

enum bool_types {true_false, yes_no, zero_one};
enum pad_string_types {pad_left, pad_right, pad_to_center};

// sqllib.h
//-----------------------------------------------------------------------------
// current - used to determine what ExecuteInsertStatement and
// ExecuteUpdateStatement do to individual fields passed to them
#define  sql_full_processing    bit_none
#define  sql_no_uppercasing     bit_1
#define  sql_no_quoting         bit_2
#define  sql_no_left_trimming   bit_3
#define  sql_no_right_trimming  bit_4
#define  sql_no_trimming        12
#define  sql_no_processing      15        

// current - not used
// history - used in hyperion for data validation
//#define look_up            0
//#define validate_int       1
//#define validate_date      2
//#define validate_timestamp 3
//#define text               4
//#define validate_float     5

// current - NOT used for ExecuteInsertStatement() and ExecuteUpdateStatement()
// notes   - default is uppercasee_field
// history - added because of usernames/passwords, usually just uppercase and
//           then lowercase for comparison (preferences, permissions)
//#define uppercase_field 0
//#define leave_alone     1

// current - used for GetPersonName(), to specify order
// notes   - default parameter is "last, first", other is "first last"
// history - added because of links on pages vs. reports
#define first_last         0
#define last_first         1
#define first_middle_last  2
#define last_first_middle  3
#define first_last_initial 4

// repeating.h
//-----------------------------------------------------------------------------
// current - controls drop-downs for repeating events, etc.
enum repeat_items {r_any, r_every, r_on, r_first, r_last};
enum day_items    {d_day, d_business_day, d_sunday, d_monday, d_tuesday, d_wednesday, d_thursday, d_friday, d_saturday};
enum week_items   {w_week};
enum month_items  {m_month, m_january, m_february, m_march, m_april, m_may, m_june, m_july, m_august, m_september, m_october, m_november, m_december};
enum year_items   {y_year, y_leap_year};

// relevant.h
//-----------------------------------------------------------------------------
// current - controls how atlas compares incoming messages to notificationrequests
// notes   - most often, only rel_e and rel_any will be used, and combination is
//           impossible (cummulative only, no intersections.) match to the main
//           part of the sql statement (usually a where statement about foreign keys)
#define rel_e           0  // equal
#define rel_l           1  // lesser than
#define rel_nge         1  // not greater/equal
#define rel_g           2  // greater than
#define rel_nle         2  // not lesser/equal
#define rel_le          3  // lesser than or equal to
#define rel_ng          3  // not greater than
#define rel_ge          4  // greater than or equal to
#define rel_nl          4  // not lesser than
#define rel_ne          5  // not equal
#define rel_like        6  // like
#define rel_any         7  // no matter what the change to fieldname

// current - used to identify the type of function that can be called by atlas upon
//           receipt of a matching request. since that function is a member of any
//           given form, this is in fact a closure of sorts. messy syntax, apologies.
typedef void (TObject::*RefreshHandler)(int, string);
// current - signatures of functions that allow for adding/editing of records
//           of the primary type of the table.
// notes   - some work-around should have been found to centralize this sort of
//           thing across the whole application, and security info, etc.
typedef void (TObject::*AddRecord)();
typedef void (TObject::*EditRecord)(const string &);
// current - function call-back used by Filter to ask if a given row should be [x]'ed
typedef bool (TObject::*CheckRecord)(const map<string, string> &);
// current - function call-back used for ... random stuff. all over the place. hehehe.
typedef void (TObject::*InformForm)();
typedef string (*TransformString)(const string &);

// used by RecordGeneric and TinyRecordGeneric
enum can_view_status {can_view_true, can_view_false, can_view_unknown};
enum can_insert_status {can_insert_true, can_insert_false, can_insert_unknown};
enum can_update_status {can_update_true, can_update_false, can_update_unknown};

// lookuptablewrapper.h
//-----------------------------------------------------------------------------
// current - controls how lookuptable values are displayed in combo boxes
#define as_pretty       0
#define as_uppercase    1
#define as_lowercase    2
#define as_sentence     3
#define as_nothing      4

// lookuptables.h
//-----------------------------------------------------------------------------
// current - not used
// history - used for hyperion to prevent mistyping of tablenames, since this was
//           a frequent problem. in prometheus, there are just too many tables to
//           do this (it would be ridiculous.)
// notes   - lookuptables.h contained only this, could be removed and deleted...
//#define diabetes_types  "diabetes_types"
//#define tribe_types     "tribe_types"
//#define state_types     "state_types"
//#define staff_types     "staff_types"
//#define contact_types   "contact_types"
//#define method_types    "method_types"
//#define direction_types "direction_types"
//#define program_types   "program_types"
//#define task_types      "task_types"
//#define referrer_types  "referrer_types"
//#define classification_types "classification_types"
//#define activity_types  "activity_types"
//#define gender_types    "gender_types"
//#define facility_types  "facility_types"
//#define event_types     "event_types"
//#define month_types     "month_types"

// lookuptableclass.h
//-----------------------------------------------------------------------------
// current - used to indicate which values should be shown in combo boxes
// notes   - default is... ?
#define showall         0
#define showvalid       1

// filter.h
//-----------------------------------------------------------------------------
// current - used to indicate how data in a column should be displayed
// notes   - fields are compared using a different set of names
//#define prettyString         0 // capitalize only first letter of each word, lowercase rest
//#define uppercaseString      1 // uppercase everything
//#define lowercaseString      2 // lowercase everything
//#define sentenceString       3 // sentence string capitalizes as best it can as per english grammar
//#define prettySSNString      4 // puts dashes in an american social security number
//#define prettyPhoneString    5 // puts parens and dashes in american phone numbers
//#define prettyZipString      6 // puts a dash for 5-4 zip codes, leaves alone otherwise
//#define prettyBoolString     7 // Yes/No
//#define amountString         8 // all numbers are right-justified 2-decimal floats
//#define repeatingString      9 // interpret as a repeating spec, and ... figure it out
//#define nonzeroAmountString 10 // 0.00 shows up as blank, others are right-justified 2-decimal floats
//#define prettyLogString     11 // "1" -> in, "0" -> out ("is_login")
//#define rawString           12 // do nothing
//#define intString           13 // right-justified integers
//#define statString          14 // right-justified 1-decimal floats
//#define dollarString        15 // amount string with dollar signs (useful when exporting)
//#define addressString       16 // USPS-happy addresses.  ish.

// if you add a new type, add an enum value, increment the array size below, and add a description in util.cpp
enum show_type {prettyString, uppercaseString, lowercaseString, sentenceString, prettySSNString,
                prettyPhoneString, prettyZipString, prettyBoolString, amountString, repeatingString,
                nonzeroAmountString, prettyLogString, rawString, intString, statString, dollarString,
                addressString};
#define num_show_types 17
// modify show_type_names in util.cpp

// fieldtypes.h
//-----------------------------------------------------------------------------
// current - controls how fields are compared, both in filter and in atlas,
//           though they should, but don't, share the same comparison function.
// notes   - fieldtypes.h may be deleted, as this was its entire contents

// if you add a new type, add an enum value, increment the array size below, and add a description in util.cpp
enum sort_type {as_text, as_date, as_stamp, as_time, as_int, as_double, as_bool};
#define num_sort_types 7
// modify sort_type_names in util.cpp

//-----------------------------------------------------------------------------

// current - on community health visit page, these values define what the
//           form will be.
#define child           1
#define newborn         2
#define prenatal        3
#define postpartum      4
#define adult           5
//------------------------------------------------------------------------------
// these values will be used to tell the community health form what type
// of elements should appear on the screen
/*
#define checkBox        0
#define edit            1
#define radioButton     2
#define button          3
#define label           4
#define comboBox        5
*/
//------------------------------------------------------------------------------
// current - account rules for the functions in components.* for account/
//           amount listviews (of doom)
#define acc_none        bit_none
#define acc_budget      bit_1
#define acc_non_budget  bit_2
#define acc_yearly      bit_3
#define acc_non_yearly  bit_4
#define acc_closed      bit_5
#define acc_non_closed  bit_6
#define acc_cash        bit_7
#define acc_payable     bit_8
#define acc_receivable  bit_9
#define acc_revenue     bit_10
#define acc_expense     bit_11
#define acc_equity      bit_12
#define acc_asset       bit_13
#define acc_indirect    bit_14

// only recently added, for future expansion ... should use these. or something.
enum account_links {balance_ref, pending_ref, cash_ref, budget_ref, actual_ref, repay_ref, unknown_ref};
enum account_types {equity_acc, payable_acc, receivable_acc, cash_acc, expense_acc, revenue_acc, unknown_acc};

// codeSpecialMessageBox.h
#define mb_yes          bit_1
#define mb_no           bit_2
#define mb_ok           bit_3
#define mb_cancel       bit_4
#define mb_abort        bit_5
#define mb_retry        bit_6
#define mb_ignore       bit_7
#define mb_allyes       bit_8
#define mb_allno        bit_9
#define mb_all          bit_10
#define mb_query        bit_11

#define mb_okcancel         (mb_ok | mb_cancel)
#define mb_yesno            (mb_yes | mb_no)
#define mb_yesnocancel      (mb_yes | mb_no | mb_cancel)
#define mb_abortretryignore (mb_abort | mb_retry | mb_ignore)
#define mb_retrycancel      (mb_retry | mb_cancel)

#define idyes           1
#define idno            2
#define idok            3
#define idcancel        4
#define idabort         5
#define idretry         6
#define idignore        7
#define idallyes        8
#define idallno         9
#define idall           10

// types of message boxes (determines the caption in the message box)
// if you add types here, add code in codePrometheus.h in the function PMessage
// to set the caption on the message box for the new type of message box.
// you should also add a new item to message_box_types in the database... make
// sure the id you assign the value here matches the id in the database.
enum prometheus_message_types {m_program_error = 0, m_warning = 1, m_invalid_data = 2, m_missing_data = 3, m_database_error = 4,
                               m_reminder = 5, m_message = 6, m_confirmation = 7, m_duplicate_data = 8, m_debug = 9};

// used on codeSpecialReferral
#define sent_referrals    1
#define waiting_referrals 2

// macros to catch EOutOfResource errors when creating new forms
#define max_form_try_attempts 100
#define sizeof_shin_kick 32768 // 32K
#define FORM_TRY \
  { \
    bool form_try_success = false; \
    unsigned int form_try_attempts = 0; \
    while (!form_try_success && form_try_attempts < max_form_try_attempts) \
    { \
      form_try_attempts++; \
      try \
      {

#define FORM_CATCH \
        form_try_success = true; \
      } \
      catch(EOutOfResources &) {} \
      catch(EOutOfMemory &) {} \
      catch(EAccessViolation &) {} \
      catch(EAbstractError &) {} \
      catch(EReadError &) {} \
      if (!form_try_success) \
      { \
        char *shin_kick_buffer; \
        bool shin_kick_ok = false; \
        try \
        { \
          shin_kick_buffer = new char[sizeof_shin_kick]; \
          shin_kick_ok = true;\
        } \
        catch (...) {} \
        if (shin_kick_ok) \
          delete [] shin_kick_buffer; \
      } \
    } \
    if (!form_try_success) \
      throw "Failed maximum attempts during a FORM_TRY block"; \
  }

// jhaley 20110719: Alternate FORM_CATCH which posts an error message and returns if it fails.
// Pass nothing as the second parameter if the function is void. PSay needs to be in-scope at point of use.
#define FORM_CATCH_RETURN(formname, retval) \
        form_try_success = true; \
      } \
      catch(EOutOfResources &) {} \
      catch(EOutOfMemory &) {} \
      catch(EAccessViolation &) {} \
      catch(EAbstractError &) {} \
      catch(EReadError &) {} \
      if (!form_try_success) \
      { \
        char *shin_kick_buffer; \
        bool shin_kick_ok = false; \
        try \
        { \
          shin_kick_buffer = new char[sizeof_shin_kick]; \
          shin_kick_ok = true;\
        } \
        catch (...) {} \
        if (shin_kick_ok) \
          delete [] shin_kick_buffer; \
      } \
    } \
    if (!form_try_success) \
    { \
      ::PSay(string("Could not create a ") + (formname), m_program_error, MB_OK, "FORM_CATCH_RETURN"); \
      return retval ; \
    } \
  }

//------------------------------------------------------------------------------
// Safe Stack Proxies - "Automatic" Pointers
//  jhaley 20120710: because I've had it with constructors and destructors that
//  throw and make collective allocation and freeing of objects impossible...

// One Parameter Version
template<typename T, typename P1> class AutoPtr
{
protected:
  T *ptr;

  void instantiate(P1 p) { try { ptr = new T(p);     } catch(...) {}             }
  void destroy()         { try { if(ptr) delete ptr; } catch(...) {} ptr = NULL; }

public:
  AutoPtr(P1 p) : ptr(NULL) { instantiate(p); }
  ~AutoPtr() { destroy(); }
  T *get() const { return ptr; }
  T *operator -> () const { return ptr;  }
  T &operator *  () const { return *ptr; }
};

// Partial specialization for no parameter
template<typename T> class AutoPtr<T, void>
{
protected:
  T *ptr;

  void instantiate() { try { ptr = new T();      } catch(...) {}             }
  void destroy()     { try { if(ptr) delete ptr; } catch(...) {} ptr = NULL; }

public:
  AutoPtr() : ptr(NULL) { instantiate(); }
  ~AutoPtr() { destroy(); }
  T *get() const { return ptr; }
  T *operator -> () const { return ptr;  }
  T &operator *  () const { return *ptr; }
};


// End Safe Stack Proxies
//------------------------------------------------------------------------------

enum debug_form_types {libraries, generic_forms, other_forms, numdebugformtypes};

enum monitor_actions {monitor_unknown, monitor_execute, monitor_fetch};

enum toolbar_view_option {view_icon, view_text, view_both};

const TColor clBad = (TColor)0x0043B5FF;

//------------------------------------------------------------------------------
#endif

