//---------------------------------------------------------------------------

#include <vcl.h>
#pragma hdrstop

#include "codeQueryMagic.h"
#include "codeQueryOperator.h"
#include "codeQueryMagicClosable.h"
#include "query_factories.h"
#include "util.h"
#include "query_load_save.h"
#include "db_hardcoded_metadata.h"
#include "Clipbrd.hpp"
//---------------------------------------------------------------------------
#pragma package(smart_init)
#pragma link "codeQueryGeneric"
#pragma link "codeHalfButton"
#pragma link "query_load_save"
#pragma link "codeQueryFileSave"
#pragma link "codeQueryFileLoad"
#pragma link "codeRubberContainer"
#pragma resource "*.dfm"
TframeQueryMagic *frameQueryMagic;
// string TframeQueryMagic::copy_buffer;
//---------------------------------------------------------------------------
__fastcall TframeQueryMagic::TframeQueryMagic(TComponent* Owner)
        : TframeQueryGeneric(Owner)
{
 comboPicker->DropDownCount = 15;
 comboPicker->OnKeyPress    = comboPickerKeyPress;
 comboPicker->OnChange      = comboPickerChange;

 FHowAppropriate = Appropriate;
 FHowCreate = Create;

 FClip  = false;
 FCache = false;
 FProxy = false;
}
//---------------------------------------------------------------------------
void __fastcall TframeQueryMagic::SetCache (bool Value)
{
 FCache = false;
 if (!FHostedQueries.empty())
  FHostedQueries[0]->Cache = Value;
}
//---------------------------------------------------------------------------
void __fastcall TframeQueryMagic::SetProxy (bool Value)
{
 FProxy = false;
 if (!FHostedQueries.empty())
  FHostedQueries[0]->Proxy = Value;
}
//---------------------------------------------------------------------------
void __fastcall TframeQueryMagic::SetClip (bool Value)
{
 // we never clip, but whatever we're told to do, we pass on to the child.
 // is this necessary? yes. actually.
 // most anything that requires its input to be clipped will ask -us- to clip, which we forward to the child!
 FClip = false;
 if (!FHostedQueries.empty())
  FHostedQueries[0]->Clip = Value;
}
//---------------------------------------------------------------------------
string TframeQueryMagic::PrepareImpl ()
{
 if (FHostedQueries.size() == 1)
  return FHostedQueries[0]->Prepare();
 else
  return "";
}
//---------------------------------------------------------------------------
void __fastcall TframeQueryMagic::ChangeHostedQuery (TframeQueryGeneric* Value)
{
 // keep only one at most.
 while (FHostedQueries.size() > 1)
 {
  delete FHostedQueries.back();
  FHostedQueries.pop_back();
 }
 // make sure we reflect the one we have
 if (FHostedQueries.size())
 {
  for (unsigned int i = 0; i < picker_content.size(); i++)
   if (FHostedQueries[0]->HowCreate == picker_content[i].create_pointer)
   {
    comboPicker->Text = comboPicker->Items->Strings[i];
    break;
   }
 }
}
//---------------------------------------------------------------------------
void __fastcall TframeQueryMagic::SetSourceTableName (AnsiString Value)
{
 TframeQueryGeneric::SetSourceTableName (Value);

 // could simply use HowAppropriate to see if the current query works or not. but this works too.

 // reload view of appropriate factories
 picker_content = query_factories.GetFactories(Value.c_str());
 comboPicker->Items->Clear();
 comboPicker->Text = "";
 bool last_factory_okay = false;
 int z = 0;
 for (vector<q_factory_info>::iterator i = picker_content.begin(); i != picker_content.end(); i++, z++)
 {
  // note that this would be -better- as a Relation-using control, with all the rights and duties thereof.
  comboPicker->Items->Add(UppercaseFirstChar(ReplaceStringWithString(i->description, "$", LowercaseString(dbPrettyPlural(LowercaseString(SourceTableName.c_str()))))).c_str());
  if (FHostedQueries.size() && FHostedQueries[0]->HowCreate == i->create_pointer)
  {
   last_factory_okay = true;
   comboPicker->Text = comboPicker->Items->Strings[z];
  }
 }

 // get rid of ours if not appropriate, otherwise inform of change
 if (FHostedQueries.size() && FHostedQueries[0]->HowCreate)
 {
  // ::PSay("Changing sources, but we already have a delegate ...", m_debug, MB_OK, "TframeQueryMagic");
  if (!last_factory_okay)
  {
   // ::PSay("Should delete the delegate and empty the string ...", m_debug, MB_OK, "TframeQueryMagic");
   delete FHostedQueries[0]; // also removes from FHostedQueries
  } else
  {
   // ::PSay("Just passing along the notification ...", m_debug, MB_OK, "TframeQueryMagic");
   FHostedQueries[0]->SourceTableName = SourceTableName;
  }
 }
}
//---------------------------------------------------------------------------
bool TframeQueryMagic::Valid ()
{
 comboPicker->Color = clWindow;
 if (FHostedQueries.empty())
  comboPicker->Color = clBad;

 // anything else is *bad*, mmmkay?
 return TframeQueryGeneric::Valid() && (FHostedQueries.size() == 1);
}
//---------------------------------------------------------------------------
TframeQueryGeneric* TframeQueryMagic::Create (TComponent* owner)
{
 TframeQueryGeneric* temp;
 FORM_TRY
  temp = new TframeQueryMagic(owner);
 FORM_CATCH
 return temp;
}
//---------------------------------------------------------------------------
bool TframeQueryMagic::Appropriate (string source_table_name)
{
 return false;
}
//---------------------------------------------------------------------------
void __fastcall TframeQueryMagic::comboPickerKeyPress(TObject *Sender, char &Key)
{
 bool is_enter = (Key == (char)VK_RETURN); // just in case it gets reset by the next function?
 FindComboBoxItem(Sender, Key);
 if (is_enter)
  comboPickerChange(Sender);
}
//---------------------------------------------------------------------------
void __fastcall TframeQueryMagic::comboPickerChange(TObject *Sender)
{
 string temp = UppercaseString(comboPicker->Text.c_str());
 int z = comboPicker->ItemIndex;
 // for (unsigned int i = 0; i < picker_content.size(); i++)
 //  if (temp == UppercaseString(picker_content[i].description))
 if (z != -1)
 {
  unsigned int i = z;
   if (FHostedQueries.empty() || (FHostedQueries[0]->HowCreate != picker_content[i].create_pointer))
   {
    TframeQueryGeneric * _old = NULL;
    TframeQueryGeneric * _new = NULL;

    string buf;

    if (FHostedQueries.size())
    {
     _old = FHostedQueries[0];
     buf = xAutoSave(this, true);
    }

    if (_old)
     _old->HostQuery = NULL;

    if (picker_content[i].create_pointer)
    {
     FORM_TRY
      _new = picker_content[i].create_pointer(this);
      _new->HostQuery = this;
      _new->Parent = Parent;
      _new->SourceTableName = SourceTableName;
      _new->Resized();
     FORM_CATCH
    }

    // only do it for operators. yes, creates new dependency, sorry!
    if (_new && _new->InheritsFrom(__classid(TframeQueryOperator)))
    {
     if (_old && _old->InheritsFrom(__classid(TframeQueryOperator)))
      _old->RehostTo(_new);
     else if (_old)
     {
      FORM_TRY
       TframeQueryMagicClosable* hub = new TframeQueryMagicClosable(_new);
       hub->HostQuery = _new;
       hub->Parent = Parent;
       hub->SourceTableName = _new->SourceTableName;
       hub->Indent = 0;
       hub->Resized();
       hub->Load(xContent(buf), true);
      FORM_CATCH
     } else
     {
      TframeQueryOperator* _new_op = (TframeQueryOperator*)_new;
      _new_op->buttonAddClick(_new_op->buttonAdd);
     }
    }

    if (_old && !_old->HostQuery)
     delete _old;

    if (_new)
     _new->Resized();
   }
 }
}
//---------------------------------------------------------------------------
void __fastcall TframeQueryMagic::menuCopyClick(TObject *Sender)
{
 // copy
 // copy_buffer = xAutoSave(this, true);
 Clipboard()->AsText = (xAutoSave(this, true) +
  "\r\n\r\n"
  "The lines above represent a custom query you built in Prometheus.  You can copy those lines to:\r\n"
  "(1) send to someone else by Chat or Email, so they can run the query just as you did, or\r\n"
  "(2) paste the query back into Prometheus anywhere that query-building is allowed.\r\n"
  "\r\n"
  "To paste it back into Prometheus (as an entire query or within an existing query), press the \r\n"
  "button that looks like a clipboard corresponding to the location where you want to drop the \r\n"
  "query.  Warning:  it will overwrite anything already \"within\" that \"branch\" of your query.").c_str();
}
//---------------------------------------------------------------------------
void __fastcall TframeQueryMagic::menuPasteClick(TObject *Sender)
{
 // bool did_hide = false;
 // if (Parent && Parent->Visible)
 //  Parent->Visible = false, did_hide = true;
 // paste

 if (UsefulClipboard())
 {
  string t = Clipboard()->AsText.c_str();
  this->Load(xContent(t), true);
  this->Resized();
 }
 // if (did_hide)
 //  Parent->Visible = true;
}
//---------------------------------------------------------------------------
void __fastcall TframeQueryMagic::menuSaveClick(TObject *Sender)
{
 // save
 if (Valid())
  QuerySave(SourceTableName, qf_query, xAutoSave(this, true));
}
//---------------------------------------------------------------------------
void __fastcall TframeQueryMagic::menuLoadClick(TObject *Sender)
{
 // load
 xValue possibly_loaded = QueryLoad(SourceTableName, qf_query);
 // bool did_hide = false;
 // if (Parent && Parent->Visible)
 //  Parent->Visible = false, did_hide = true;
 // in case they cancel, avoid error messages
 if (!possibly_loaded.empty())
 {
  this->Load(xContent(possibly_loaded), true);
  this->Resized();
 }
 // if (did_hide)
 //  Parent->Visible = true;
}
//---------------------------------------------------------------------------
void __fastcall TframeQueryMagic::menuDefaultsClick(TObject *Sender)
{
 // reset -- copy/paste, without the buffer, and without params
 string temp;
 temp = xAutoSave(this, false); // no params
 this->Load(xContent(temp), false); // no params
 this->Resized();
}
//---------------------------------------------------------------------------
void __fastcall TframeQueryMagic::buttonActionsClick(TObject *Sender)
{
 popupActions->PopupComponent = (TComponent*)Sender;
 popupActions->Popup(((TWinControl*)Sender)->ClientOrigin.x, ((TWinControl*)Sender)->ClientOrigin.y + ((TWinControl*)Sender)->Height - 2 /* half-button oddness */);
}
//---------------------------------------------------------------------------
void __fastcall TframeQueryMagic::popupActionsPopup(TObject *Sender)
{
 buttonActions->SetFocus(); // take focus away from date controls that won't have updated their date value until unfocused

 // check to see if there's something in the copy buffer that we can use -- check source_table_name on that query!
 menuPaste->Enabled = UsefulClipboard();
}
//---------------------------------------------------------------------------
bool TframeQueryMagic::UsefulClipboard()
{
 if (!Clipboard()->HasFormat(CF_TEXT))
  return false;
 string t = Clipboard()->AsText.c_str();
 if (t.empty())
  return false;
 string temp_tag = xTag(t);
 if (temp_tag.empty())
  return false;
 if (!LikeString("TframeQueryMagic%", temp_tag))
  return false;
 return true;
}
//---------------------------------------------------------------------------
void __fastcall TframeQueryMagic::SetSimple (bool Value)
{
 TframeQueryGeneric::SetSimple(Value);
 comboPicker->Visible   = !ShowComment && !Simple;
 rubberComment->Visible = ShowComment && !Simple;
 buttonActions->Visible = !Simple;
 labelType->Visible     = Simple;
 labelType->Caption     = (ShowComment ? editComment->Lines->Text : comboPicker->Text);
}
//---------------------------------------------------------------------------
bool TframeQueryMagic::Same (TframeQueryGeneric* other_query)
{
 bool answer = TframeQueryGeneric::Same(other_query);
 if (!answer)
 {
  if (!other_query)
   return false;
  if (other_query->SourceTableName != this->SourceTableName)
   return false;
  if (other_query->Clip == false && this->Clip == true)
   return false;

  string a, b;
  if (other_query->Prepared)
   a = other_query->CachedSave;
  else
   a = other_query->Save(true);

  if (this->Prepared)
   b = this->CachedSave;
  else
   b = this->Save(true);

  // we know they're different, TframeQueryGeneric::Same() already decided that for us ...
  if ((xTag(a) == "TframeQueryMagic" || xTag(a) == "TframeQueryMagicClosable") &&
      (xTag(b) == "TframeQueryMagic" || xTag(b) == "TframeQueryMagicClosable"))
   answer = (xContent(a) == xContent(b));
 }
 return answer;
}
//---------------------------------------------------------------------------
string TframeQueryMagic::Describe (bool as_html)
{
 if (!FHostedQueries.empty())
 {
  string temp;
  if (editComment->Lines->Text != "" && !LikeString("[summarize me]%", editComment->Lines->Text.c_str()))
  {
   temp += "\"";
   temp += StripLeading(StripTrailing(editComment->Lines->Text.c_str(), "\n"), "\n");
   temp += string("\", i.e.:");
   temp += (as_html ? "<br>" : "\r\n");
  }
  if (FHostedQueries[0]->Valid())
   temp += FHostedQueries[0]->Describe(as_html);
  else
   temp += "incomplete query";
  return temp;
 } else
  return "Undefined query (no sub-query.)" + string(as_html ? "<br>" : "\r\n");
}
//---------------------------------------------------------------------------
xValue TframeQueryMagic::Save (bool with_params)
{
 xValue answer = TframeQueryGeneric::Save(with_params);
 if (with_params && xTag(answer) == "map")
 {
  map<string, xValue> config_map = xToMap(answer);
  if (editComment->Lines->Text != "")
   config_map["magic_comment"] = xFromString(editComment->Lines->Text.c_str());
  config_map["magic_commented"] = xFromBool(ShowComment);
  answer = xFromMap(config_map);
 }
 return answer;
}
//---------------------------------------------------------------------------
void TframeQueryMagic::Load (xValue config, bool with_params)
{
 TframeQueryGeneric::Load(config, with_params);
 if (with_params && xTag(config) == "map")
 {
  map<string, xValue> config_map = xToMap(config);
  if (xTag(config_map["magic_comment"]) == "string")
   editComment->Lines->Text = xToString(config_map["magic_comment"]).c_str();
  else
   editComment->Lines->Text = "";
  if (xTag(config_map["magic_commented"]) == "bool")
   ShowComment = xToBool(config_map["magic_commented"]);
  else
   ShowComment = false;
 }
}
//---------------------------------------------------------------------------
void __fastcall TframeQueryMagic::SetShowComment (bool Value)
{
 FShowComment = Value;
 rubberComment->Visible = Value;
 buttonComment->Visible = !Value;
 comboPicker->Visible = !Value;
 buttonDefinition->Visible = Value;
 if (!Value && LikeString("[summarize me]%", editComment->Lines->Text.c_str()))
 {
  editComment->Lines->Text = "";
 } else
 if (Value && editComment->Lines->Text == "")
 {
  editComment->Lines->Text = ("[summarize me] " + Describe(false)).c_str();
  editComment->SelectAll(); // to make it easier to clear
 } else if (Value)
  editComment->SelStart = 0; // move caret to front
 for (vector<TframeQueryGeneric*>::iterator i = FHostedQueries.begin(); i != FHostedQueries.end(); i++)
  (*i)->Hidden = (Hidden || Value);

 int new_height = (Value ? 48 : 24);
 if (new_height != Height)
 {
  Height = new_height;
  Resized();
 }
}
//---------------------------------------------------------------------------
bool __fastcall TframeQueryMagic::GetShowComment ()
{
 return FShowComment;
}
//---------------------------------------------------------------------------
void __fastcall TframeQueryMagic::SetHidden (bool Value)
{
 TframeQueryGeneric::SetHidden(Value);
 ShowComment = ShowComment;
}
//---------------------------------------------------------------------------
void __fastcall TframeQueryMagic::buttonCommentClick(TObject *Sender)
{
 ShowComment = true;
 editComment->SetFocus();
}
//---------------------------------------------------------------------------
void __fastcall TframeQueryMagic::buttonDefinitionClick(TObject *Sender)
{
 ShowComment = false;
}
//---------------------------------------------------------------------------
void __fastcall TframeQueryMagic::menuExpandDefinitionsClick(
      TObject *Sender)
{
 RecurseShowDefinition();
}
//---------------------------------------------------------------------------
void __fastcall TframeQueryMagic::RecurseShowDefinition()
{
 ShowComment = false;
 TframeQueryGeneric::RecurseShowDefinition();
}
//---------------------------------------------------------------------------
void TframeQueryMagic::AfterPrepare(string dontcare)
{ // overridden
}
//---------------------------------------------------------------------------

