//---------------------------------------------------------------------------

#include <vcl.h>
#pragma hdrstop

#include "codeQueryOperator.h"
#include "codeQueryMagicClosable.h"
#include "db_hardcoded_metadata.h"
#include "prometheus_util.h"
#include "util.h"
//---------------------------------------------------------------------------
#pragma package(smart_init)
#pragma link "codeQueryGeneric"
#pragma resource "*.dfm"
TframeQueryOperator *frameQueryOperator;
//---------------------------------------------------------------------------
__fastcall TframeQueryOperator::TframeQueryOperator(TComponent* Owner)
        : TframeQueryGeneric(Owner)
{
 FHowAppropriate = Appropriate;
 FHowCreate = Create;
 Cache = true;
 // Proxy = false; // true; // considering the sql being run ... this seems like a good idea. neh?
}
//---------------------------------------------------------------------------
string TframeQueryOperator::PrepareImpl ()
{
 return "";
}
//---------------------------------------------------------------------------
void __fastcall TframeQueryOperator::SetSourceTableName (AnsiString Value)
{
 TframeQueryGeneric::SetSourceTableName (Value);

 // delete inappropriate ones, and inform appropriate ones of the change
 // this would not have worked: the Create and Appropriate functions are not virtual and cannot be
 // so ... constructors set "HowAppropriate" (*g*) which points to the correct static function, hopefully (wish it were automatic)
 // note: this will contain pickers, which, when told to change tables, will then go and reset their queries if they're not appropriate
 // but that'll result in -blank- pickers, not deleted ones. meh.
 for (vector<TframeQueryGeneric*>::iterator i = FHostedQueries.begin(); i != FHostedQueries.end(); i++)
  if (!((*i)->HowAppropriate(FSourceTableName.c_str())))
  {
   delete (*i);
   i = FHostedQueries.erase(i); // TODO: is this a bug? shouldn't the destructor have removed the item from the vector? (hard to test)
  } else
  {
   (*i)->SourceTableName = SourceTableName;
  }
}
//---------------------------------------------------------------------------
bool TframeQueryOperator::Valid ()
{
 if (FHostedQueries.empty())
  PSay("You must have at least one sub-query for each 'operator' query,\n(e.g. 'match all of', 'match any of', 'match none of', 'match some of', ...)\n\nClick a [+] button to add a new sub-query in the appropriate place.", m_missing_data, MB_OK, "TframeQueryOperator");
 return TframeQueryGeneric::Valid() && (!FHostedQueries.empty());
}
//---------------------------------------------------------------------------
TframeQueryGeneric * TframeQueryOperator::Create (TComponent* owner)
{
 TframeQueryGeneric * temp;
 FORM_TRY
  temp = new TframeQueryOperator(owner);
 FORM_CATCH
 return temp;
}
//---------------------------------------------------------------------------
bool TframeQueryOperator::Appropriate (string source_table_name)
{
 return dbIsAvailable(source_table_name);
}
//---------------------------------------------------------------------------
void __fastcall TframeQueryOperator::buttonAddClick(TObject *Sender)
{
 FORM_TRY
  TframeQueryMagicClosable* temp = NULL;
  temp = new TframeQueryMagicClosable(this);
  temp->HostQuery = this;
  temp->Parent = Parent;
  temp->SourceTableName = SourceTableName;
  temp->Indent = 0;
  temp->Resized();
 FORM_CATCH
}
//---------------------------------------------------------------------------
void __fastcall TframeQueryOperator::SetSimple (bool Value)
{
 TframeQueryGeneric::SetSimple(Value);
 buttonAdd->Enabled = !Simple;
 Height = (Simple ? 0 : 24);
 Resized();
}
//---------------------------------------------------------------------------
string TframeQueryOperator::Describe (bool as_html)
{
 string answer;
 if (as_html)
  answer += "<div style='padding-left: 24px;'>";
 for (int i = 0; i < FHostedQueries.size(); i++)
 {
  if (!as_html)
   answer += "  ";
  answer += FHostedQueries[i]->Describe(as_html);
 }
 if (as_html)
  answer += "</div>";
 return answer;
}
//---------------------------------------------------------------------------
void TframeQueryOperator::ProxyMultiChildren ()
{
 if(site_settings["proxy_all_queries"]=="1")
  if (FHostedQueries.size() > 1)
   for (vector<TframeQueryGeneric*>::iterator i = FHostedQueries.begin(); i != FHostedQueries.end(); i++)
    (*i)->Proxy = true;
}
//---------------------------------------------------------------------------

