//---------------------------------------------------------------------------

#include <vcl.h>
#pragma hdrstop

#include "codeQuery_PointsTo.h"
#include "codeQuery_Whole.h"
#include "codeQueryMagic.h"
//---------------------------------------------------------------------------
#pragma package(smart_init)
#pragma link "codeQueryGeneric"
#pragma resource "*.dfm"
TframeQuery_PointsTo *frameQuery_PointsTo;
//---------------------------------------------------------------------------
__fastcall TframeQuery_PointsTo::TframeQuery_PointsTo(TComponent* Owner)
        : TframeQueryGeneric(Owner)
{
 FHowAppropriate = Appropriate;
 FHowCreate = Create;

 FORM_TRY
  TframeQueryMagic* z = new TframeQueryMagic(this);
  z->HostQuery = this;
  z->SourceTableName = "-";
 FORM_CATCH

 comboType->OnKeyPress = comboTypeKeyPress;
 comboType->OnChange   = comboTypeChange;

 Clip = true;
}
//---------------------------------------------------------------------------
void __fastcall TframeQuery_PointsTo::ChangeHostedQuery (TframeQueryGeneric* Value)
{
 // in case it's auto-created for us?
 if (FHostedQueries.size())
  FHostedQueries[0]->Indent = 24;
}
//---------------------------------------------------------------------------
void __fastcall TframeQuery_PointsTo::SetSourceTableName (AnsiString Value)
{
 AnsiString old_stn = SourceTableName;
 TframeQueryGeneric::SetSourceTableName(Value);

 // list of tables we have FK's to
 if (old_stn != SourceTableName)
 {
  comboType->Items->Clear();
  picker_content = dbPointsFrom(SourceTableName.c_str());
  for (int i = 0; i < picker_content.size(); i++)
  {
   string a = picker_content[i].destination_table_singular + (picker_content[i].note.empty() ? string("") : " (" + picker_content[i].note + ")");
   comboType->Items->Add(a.c_str());
   if (a == comboType->Text.c_str())
    comboType->ItemIndex = i;
  }
 }

 if (FHostedQueries.size() && comboType->ItemIndex != -1)
  (FHostedQueries[0])->SourceTableName = picker_content[comboType->ItemIndex].destination_table.c_str();
}
//---------------------------------------------------------------------------
string TframeQuery_PointsTo::PrepareImpl ()
{
 string answer;

 if (FHostedQueries.size())
 {
  (FHostedQueries[0])->Clip = true;
  string data_source = (FHostedQueries[0])->Prepare();

  fk_info about_fk = picker_content[comboType->ItemIndex];
  // points to ...
  /*
  string sql = "select " + about_fk.source_table + ".id from " + data_source +
               " left join " + about_fk.source_table + " on " + data_source + ".id = " + about_fk.source_table + "." + about_fk.source_field +
               " where " + about_fk.source_table + ".id is not null";
  */
  string sql = "select " + about_fk.source_table_actual + ".id from " + about_fk.source_table_actual + " where " + about_fk.source_table_actual + "." + about_fk.source_field + " in (select " + data_source + ".id from " + data_source + ")";

  set<string> depends_on;
  depends_on.insert(data_source);
  answer = Context->CreateView(sql, depends_on);
 }

 return answer;
}
//---------------------------------------------------------------------------
bool TframeQuery_PointsTo::Valid ()
{
 comboType->Color = clWindow;
 if (comboType->ItemIndex == -1)
  comboType->Color = clBad;

 return TframeQueryGeneric::Valid() && (comboType->ItemIndex != -1);
}
//---------------------------------------------------------------------------
xValue TframeQuery_PointsTo::Save (bool with_params)
{
 xValue answer = TframeQueryGeneric::Save(with_params);
 if (with_params && xTag(answer) == "map")
 {
  map<string, xValue> config_map = xToMap(answer);
  // TODO : this could be dangerous ... must find a solution that doesn't depend on the text -and- doens't depend on ItemIndex (could change between versions)
  config_map["type"] = xFromString(comboType->Text.c_str());
  answer = xFromMap(config_map);
 }
 return answer;
}
//---------------------------------------------------------------------------
void TframeQuery_PointsTo::Load (xValue config, bool with_params)
{
 // let it auto-recreate the sub-query for us
 while (FHostedQueries.size())
  delete FHostedQueries[0];

 TframeQueryGeneric::Load(config, with_params);
 if (with_params && xTag(config) == "map")
 {
  map<string, xValue> config_map = xToMap(config);
  if (xTag(config_map["type"]) == "string")
  {
   comboType->Text = xToString(config_map["type"]).c_str();
   ResetItemIndex(comboType);
  }
 }

 // recreate stuff if necessary -- code copied from constructor?
 if (FHostedQueries.empty())
 {
  FORM_TRY
   TframeQueryMagic* z = new TframeQueryMagic(this);
   z->HostQuery = this;
   if (comboType->ItemIndex != -1)
    z->SourceTableName = picker_content[comboType->ItemIndex].destination_table.c_str();
  FORM_CATCH
 }
}
//---------------------------------------------------------------------------
bool TframeQuery_PointsTo::Appropriate (string source_table_name)
{
 // do we point to anything?
 return (!dbPointsFrom(source_table_name).empty());
}
//---------------------------------------------------------------------------
TframeQueryGeneric* TframeQuery_PointsTo::Create (TComponent* owner)
{
 TframeQueryGeneric* temp;
 FORM_TRY
  temp = new TframeQuery_PointsTo(owner);
 FORM_CATCH
 return temp;
}
//---------------------------------------------------------------------------
void __fastcall TframeQuery_PointsTo::comboTypeKeyPress(TObject *Sender, char &Key)
{
 FindComboBoxItem(Sender, Key);
}
//---------------------------------------------------------------------------
void __fastcall TframeQuery_PointsTo::comboTypeChange(TObject *Sender)
{
 if (comboType->Text != "" && comboType->ItemIndex != -1)
 {
  if (FHostedQueries.size())
   FHostedQueries[0]->SourceTableName = picker_content[comboType->ItemIndex].destination_table.c_str();
 } else
 {
  if (FHostedQueries.size())
   FHostedQueries[0]->SourceTableName = "-";
 }
}
//---------------------------------------------------------------------------
void __fastcall TframeQuery_PointsTo::SetSimple (bool Value)
{
 TframeQueryGeneric::SetSimple(Value);
 comboType->Visible = !Simple;
 labelType->Visible = Simple;
 labelType->Caption = comboType->Text;
}
//---------------------------------------------------------------------------
string TframeQuery_PointsTo::Describe (bool as_html)
{
 string desc;

 fk_info about_fk = picker_content[comboType->ItemIndex];
 string t_desc = dbPrettySingular(LowercaseString(about_fk.destination_table));
 // so, it occurs to me that i -added- singular and plural fields for source and destination in fk_info ...
 // so the above is probably redundant. don't feel like testing it right now. but ... yeah.
 // should look into it.
 // TODO: see above (philip being stupid?)

 if (!FHostedQueries.empty())
  return UppercaseFirstChar(LowercaseString(dbPrettyPlural(LowercaseString(SourceTableName.c_str()))) +
   " that point to a(n) " + LowercaseString(t_desc) + " among the following:") +
   (as_html ? "<br>" : "\r\n") +
   string(as_html ? "<div style='padding-left: 24px;'>" : "  ") + FHostedQueries[0]->Describe(as_html) + string(as_html ? "</div>" : "");
 else
  return "Undefined query (no sub-query.)" + string(as_html ? "<br>" : "\r\n");
}
//---------------------------------------------------------------------------

