//---------------------------------------------------------------------------

#include <vcl.h>
#pragma hdrstop

#include "codeQuery_Random.h"
#include "codeQueryMagic.h"
#include "db_hardcoded_metadata.h"
//---------------------------------------------------------------------------
#pragma package(smart_init)
#pragma link "codeQueryGeneric"
#pragma resource "*.dfm"
TframeQuery_Random *frameQuery_Random;
//---------------------------------------------------------------------------
__fastcall TframeQuery_Random::TframeQuery_Random(TComponent* Owner)
        : TframeQueryGeneric(Owner)
{
 FHowAppropriate = Appropriate;
 FHowCreate = Create;
 Cache = false;
 Proxy = false; // ... but we're already saving a hardcopy, don't need it duplicated, thank you!
 FORM_TRY
  TframeQueryMagic* z = new TframeQueryMagic(this);
  z->HostQuery = this;
  z->SourceTableName = SourceTableName;
 FORM_CATCH
 comboType->ItemIndex = 0;

 Clip  = false;  // see below, never clip (we clip input)
}
//---------------------------------------------------------------------------
void __fastcall TframeQuery_Random::SetClip (bool Value)
{
 // our method already clips
 FClip = false;
}
//---------------------------------------------------------------------------
void __fastcall TframeQuery_Random::SetCache (bool Value)
{
 // it's just a really bad idea to do this ... it makes things behave -oddly-
 // and(random(5, all), random(5, all)) --> gives 5 (if cacheing)
 // and(random(5, all), random(6, all)) --> gives 0, maybe 1 or 2, rarely more (can't cache)
 // ... solution: never cache, making it consistently a bad idea to re-use a 'random' query with expected results.
 FCache = false;
}
//---------------------------------------------------------------------------
void __fastcall TframeQuery_Random::ChangeHostedQuery (TframeQueryGeneric* Value)
{
 // in case it's auto-created for us?
 if (FHostedQueries.size())
  FHostedQueries[0]->Indent = 24;
}
//---------------------------------------------------------------------------
void __fastcall TframeQuery_Random::SetSourceTableName (AnsiString Value)
{
 TframeQueryGeneric::SetSourceTableName(Value);
 int z = comboType->ItemIndex;
 if (FHostedQueries.size())
  (FHostedQueries[0])->SourceTableName = SourceTableName;
 comboType->Items->Strings[0] = SentenceString(SourceTableName.c_str()).c_str();
 comboType->Items->Strings[1] = ("% of " + LowercaseString(SourceTableName.c_str())).c_str();
 comboType->ItemIndex = z;
}
//---------------------------------------------------------------------------
string TframeQuery_Random::PrepareImpl ()
{
 string answer;

 if (FHostedQueries.size())
 {
  (FHostedQueries[0])->Clip = true; // force clipping, because ... it's more efficient than clipping ourselves, and logically required ... (some of the incoming items could be disallowed -- we don't want to clip after picking them, because our count will be wrong)
  string data_source = (FHostedQueries[0])->Prepare();

  // unsigned int input_size = StringToInt(GetOneField(Context->Transaction, "select count(*) from " + data_source));
  list<string> input_data;
  list<string> output_data;
  // things got moved here because of divide-by-zero avoidance
  list<vector<string> > final_output_data;
  vector<string> f;
  f.resize(2);
  f[0] = "q";
  f[1] = "id";
  string phase1 = GetNextId(Context->Transaction, "temp_id");

  WriteLog("Starting data grab");
  if (!SqlToList(Context->Transaction, "select id from " + data_source, input_data))
  {
   PSay("Failed to retrieve in-memory copy for random selection", m_database_error, MB_OK, "TframeQueryRandom");
   return "";
  }
  WriteLog("Done grabbing data");

  unsigned int input_size = input_data.size(); // yes, it's on a linked list, but that avoids possibly running the query twice. and we could easily switch to vector or set or something.
  unsigned int wanted_size = 0;
  if (comboType->ItemIndex == 1)
   wanted_size = min(1.00, max(0.00, StringToFloat(editNumber->Text.c_str()) * 0.01)) * input_size;
  else
   wanted_size = min((unsigned int)StringToInt(editNumber->Text.c_str()), input_size);

  if (input_size) // oddly enough, dividing by zero is a -bad- idea ...
  {
   float likelihood = (1.00 * wanted_size / input_size);
   // PSay("wanted_size = " + IntToString(wanted_size), m_debug, MB_OK, "TframeQuery_Random::PrepareImpl");

   unsigned int output_size = 0;
   list<string>::iterator i = input_data.begin();
   list<string>::iterator j;
   randomize();
   while (output_size < wanted_size)
   {
    if (((rand() / (RAND_MAX + 0.0)) * 1.00) < likelihood)
    {
     output_data.push_back(*i);
     j = i++;
     input_data.erase(j);
     output_size++;
    } else
    {
     i++;
    }
    if (i == input_data.end())
    {
     i = input_data.begin();
     // PSay("Looping!", m_debug, MB_OK, "");
    }
   }

  WriteLog("Done picking data");

   // fix it up and send it (could do this earlier, i suppose -- would save time and memory? just less clean to look at.)
   id_queries.insert(phase1);
   vector<string> t;
   t.resize(2);
   t[0] = phase1;
   for (list<string>::iterator i = output_data.begin(); i != output_data.end(); i++)
   {
    t[1] = *i;
    final_output_data.push_back(t);
   }
  }

  if (!ExecutePreparedInsert(Context->Transaction, "temp_id", f, final_output_data))
  {
   PSay("Failed to insert random selection back into database.", m_database_error, MB_OK, "TframeQuery_Random");
   return "";
  }

  WriteLog("Done re-inserting data");

  answer = Context->CreateView("select temp_id.id from temp_id where q = " + phase1, set<string>());
 }
 return answer;
}
//---------------------------------------------------------------------------
void TframeQuery_Random::CleanUpImpl ()
{
 for(set<string>::iterator i = id_queries.begin(); i != id_queries.end(); i++)
 {
  if (!ExecuteStatement(Context->Transaction, "delete from temp_id where q = " + *i))
   PSay("Failed to delete temporary data, q = " + *i, m_database_error, MB_OK, "TframeQuery_Random");
 }
 id_queries.clear();
}
//---------------------------------------------------------------------------
bool TframeQuery_Random::Valid ()
{
 comboType->Color = clWindow;
 editNumber->Color = clWindow;

 if (comboType->ItemIndex == -1)
  comboType->Color = clBad;
 else
 if ((comboType->ItemIndex == 0 && (!IsInt(editNumber->Text.c_str()) || editNumber->Text == "" || StringToInt(editNumber->Text.c_str()) < 1)) ||
     (comboType->ItemIndex == 1 && (!IsFloat(editNumber->Text.c_str()) || editNumber->Text == "" || StringToFloat(editNumber->Text.c_str()) <= 0.00 || StringToFloat(editNumber->Text.c_str()) >= 100.00)))
  editNumber->Color = clBad;


 // mode must be selected; count -> int, % -> float
 return TframeQueryGeneric::Valid() &&
        comboType->Color != clBad &&
        editNumber->Color != clBad;
}
//---------------------------------------------------------------------------
xValue TframeQuery_Random::Save (bool with_params)
{
 xValue answer = TframeQueryGeneric::Save(with_params);
 if (with_params && xTag(answer) == "map")
 {
  map<string, xValue> config_map = xToMap(answer);
  string n;
  string t;
  switch(comboType->ItemIndex)
  {
   case 0: t = "i";
           n = xFromInt(StringToInt(editNumber->Text.c_str()));
           break;
   case 1: t = "%";
           n = xFromFloat(StringToFloat(editNumber->Text.c_str()));
           break;
   default: t = "";
  }
  config_map["num"] = n;
  config_map["type"] = xFromString(t);
  answer = xFromMap(config_map);
 }
 return answer;
}
//---------------------------------------------------------------------------
void TframeQuery_Random::Load (xValue config, bool with_params)
{
 // let it auto-recreate the sub-query for us
 while (FHostedQueries.size())
  delete FHostedQueries[0];

 TframeQueryGeneric::Load(config, with_params);
 if (with_params && xTag(config) == "map")
 {
  map<string, xValue> config_map = xToMap(config);

  if (xTag(config_map["num"]) == "int")
   editNumber->Text = IntToString(xToInt(config_map["num"])).c_str();
  else
  if (xTag(config_map["num"]) == "float")
   editNumber->Text = FloatToString(xToFloat(config_map["num"])).c_str();

  if (xTag(config_map["type"]) == "string")
  {
   if (xToString(config_map["type"]) == "i")
    comboType->ItemIndex = 0;
   else if (xToString(config_map["type"]) == "%")
    comboType->ItemIndex = 1;
   else
    comboType->Text = "";
  }
 }

 // recreate stuff if necessary -- code copied from codeQuery_Has
 if (FHostedQueries.empty())
 {
  FORM_TRY
   TframeQueryMagic* z = new TframeQueryMagic(this);
   z->HostQuery = this;
   z->SourceTableName = SourceTableName;
  FORM_CATCH
 }
}
//---------------------------------------------------------------------------
bool TframeQuery_Random::Appropriate (string source_table_name)
{
 // if there's a whole, there's a random subset of the whole.
 // TODO: do we need this?
 // seems like ... well, i guess we're just trying to make sure there's something -interesting- that can be done, but ... hmmm.
 return TframeQuery_Whole::Appropriate(source_table_name);
}
//---------------------------------------------------------------------------
TframeQueryGeneric* TframeQuery_Random::Create (TComponent* owner)
{
 TframeQueryGeneric* temp;
 FORM_TRY
  temp = new TframeQuery_Random(owner);
 FORM_CATCH
 return temp;
}
//---------------------------------------------------------------------------
string TframeQuery_Random::Describe (bool as_html)
{
 string desc;

 if (comboType->ItemIndex == 1)
  desc = FloatToString(min(float(100.00), max(StringToFloat(editNumber->Text.c_str()), float(0.00)))) + "% of";
 else
  desc = IntToString(max(0, StringToInt(editNumber->Text.c_str()))) + LowercaseString(dbPrettyPlural(LowercaseString(SourceTableName.c_str()))) + " among";

 if (!FHostedQueries.empty())
  return UppercaseFirstChar("A random selection of " + desc + " " + " the following:") +
   (as_html ? "<br>" : "\r\n") +
   string(as_html ? "<div style='padding-left: 24px;'>" : "  ") + FHostedQueries[0]->Describe(as_html) + string(as_html ? "</div>" : "");
 else
  return "Undefined query (no sub-query.)" + string(as_html ? "<br>" : "\r\n");
}
//---------------------------------------------------------------------------

