//---------------------------------------------------------------------------

#include <vcl.h>
#pragma hdrstop

#include "codeQuery_Some.h"
#include "util.h"
#include "db_hardcoded_metadata.h"
//---------------------------------------------------------------------------
#pragma package(smart_init)
#pragma link "codeQueryOperator"
#pragma resource "*.dfm"
TframeQuery_Some *frameQuery_Some;
//---------------------------------------------------------------------------
__fastcall TframeQuery_Some::TframeQuery_Some(TComponent* Owner)
        : TframeQueryOperator(Owner)
{
 FHowAppropriate = Appropriate;
 FHowCreate = Create;
 Cache = true;
 // Proxy = false; // for now?

 Proxy = true;
}
//---------------------------------------------------------------------------
string TframeQuery_Some::PrepareImpl ()
{
 // some
 // each entry must match z times, where z must be between x and y, inclusive.
 // x should be (preferably) less than the number of queries available.
 // superset of all and any ...
 // given N queries:
 //
 //        0 < M < P < N
 //
 // all:   N to N of N
 // least: M to N of N
 // some:  M to P of N
 // pass:  0 to N of N
 // any:   1 to N of N : need "whole"
 // upto:  0 to P of N : need "whole"
 // none:  0 to 0 of N : need "whole"
 // we optimize "at least" when M==1 and P==N (can't be unspecified), but do so in "some"

 ProxyMultiChildren();

 unsigned int _min, _max;
 _min = spinMin->Position;
 _max = spinMax->Position;

 string answer;

 set<string> dependent_views;

 set<string> hosted_views;
 for (vector<TframeQueryGeneric*>::iterator i = FHostedQueries.begin(); i != FHostedQueries.end(); i++)
  hosted_views.insert((*i)->Prepare());
 dependent_views = hosted_views;

 string whole_view;
 if (!_min)
 {
  whole_view = Context->FindWhole(SourceTableName.c_str());
  dependent_views.insert(whole_view);
 }

 // we clip because it's easier than clipping them?
 // TODO: reverify: should we sometimes request that all children clip? verify logic on that one.
 // result = ((A & W) & !(B & W)) | (!(A & W) & (B & W))
 // vs.
 // result = W & ((A & !B) | (!A & B))
 // remembering that & == intersection, | == union
 Clip = true;
 // NONE
 if (!_min && !_max)
 {
  // PSay("NONE", m_debug, MB_OK, "TframeQuery_Some");
  // TODO: this way of doing 'none' (same as codeQuery_None) is SLOW-ASS
  string sql;
  sql = "select " + whole_view + ".id from " + whole_view;
  for (set<string>::iterator i = hosted_views.begin(); i != hosted_views.end(); i++)
   sql += " left join " + *i + " on " + *i + ".id = " + whole_view + ".id";
  sql += " where ";
  for (set<string>::iterator i = hosted_views.begin(); i != hosted_views.end(); i++)
   sql += (i != hosted_views.begin() ? "and " : "") + *i + ".id is null ";
  answer = Context->CreateView(sql, dependent_views);
  // Proxy = false;
  Clip = false;
 } else
 // ALL
 if (_min == _max && _min == FHostedQueries.size())
 {
  // PSay("ALL", m_debug, MB_OK, "TframeQuery_Some");
  string sql;
  string b;
  for (set<string>::iterator i = hosted_views.begin(); i != hosted_views.end(); i ++)
  {
   if (i == hosted_views.begin())
    sql = "select " + *i + ".id from " + *i, b = *i;
   else
    sql += " inner join " + *i + " on " + *i + ".id = " + b + ".id";
  }
  answer = Context->CreateView(sql, dependent_views);
 } else
 // PASS-through
 // TODO: this is so dumb ... warn users?
 if (!_min && _max == FHostedQueries.size())
 {
  // PSay("PASS", m_debug, MB_OK, "TframeQuery_Some");
  answer = whole_view;
  Clip = false;
 } else
 // ANY (at least one)
 if (_min == 1 && _max == FHostedQueries.size())
 {
  // PSay("ANY", m_debug, MB_OK, "TframeQuery_Some");
  string sql;
  for (set<string>::iterator i = hosted_views.begin(); i != hosted_views.end(); i ++)
  {
   if (i != hosted_views.begin())
    sql += " union ";
   sql += "select " + *i + ".id from " + *i;
  }
  answer = Context->CreateView(sql, dependent_views);
 } else
 // UP TO N
 if (!_min)
 {
  // PSay("UPTO", m_debug, MB_OK, "TframeQuery_Some");
  string sql;

  string phase1 = GetNextId(Context->Transaction, "temp_id");
  string phase2 = GetNextId(Context->Transaction, "temp_id");

  for (set<string>::iterator i = hosted_views.begin(); i != hosted_views.end(); i++)
  {
   sql = "insert into temp_id (q, id) select " + phase1 + ", " + *i + ".id from " + *i;
   if (!ExecuteStatement(Context->Transaction, sql))
    PSay("Failed to insert union.", m_database_error, MB_OK, "TframeQuery_Some");
  }

  sql = "insert into temp_id (q, id) select " + phase2 + ", temp_id.id from temp_id where q = " + phase1 + " group by temp_id.id having count(*) > " + IntToString(_max);
  if (!ExecuteStatement(Context->Transaction, sql))
   PSay("Failed to insert overflow.", m_database_error, MB_OK, "TframeQuery_Some");
  id_queries.insert(phase2);

  sql = "delete from temp_id where q = " + phase1;
  if (!ExecuteStatement(Context->Transaction, sql))
   PSay("Failed to delete union.", m_database_error, MB_OK, "TframeQuery_Some");

  sql = "select " + whole_view + ".id from " + whole_view + " left join temp_id on temp_id.q = " + phase2 + " and temp_id.id = " + whole_view + ".id where temp_id.id is null";
  set<string> just_whole;
  just_whole.insert(whole_view);
  answer = Context->CreateView(sql, just_whole);
  Clip = false;
  // Proxy = false;
 } else
 // SOME, other case
 {
  // PSay("SOME", m_debug, MB_OK, "TframeQuery_Some");
  string sql;

  string phase1 = GetNextId(Context->Transaction, "temp_id");
  string phase2 = GetNextId(Context->Transaction, "temp_id");

  for (set<string>::iterator i = hosted_views.begin(); i != hosted_views.end(); i++)
  {
   sql = "insert into temp_id (q, id) select " + phase1 + ", " + *i + ".id from " + *i;
   if (!ExecuteStatement(Context->Transaction, sql))
    PSay("Failed to insert union.", m_database_error, MB_OK, "TframeQuery_Some");
  }

  sql = "insert into temp_id (q, id) select " + phase2 + ", temp_id.id from temp_id where q = " + phase1 + " group by temp_id.id having count(*) between " + IntToString(_min) + " and " + IntToString(_max);
  if (!ExecuteStatement(Context->Transaction, sql))
   PSay("Failed to insert selection.", m_database_error, MB_OK, "TframeQuery_Some");
  id_queries.insert(phase2);

  sql = "delete from temp_id where q = " + phase1;
  if (!ExecuteStatement(Context->Transaction, sql))
   PSay("Failed to delete union.", m_database_error, MB_OK, "TframeQuery_Some");

  sql = "select temp_id.id from temp_id where temp_id.q = " + phase2;
  answer = Context->CreateView(sql, set<string>());
  // Proxy = false;
 }

 return answer;
}
//---------------------------------------------------------------------------
void TframeQuery_Some::CleanUpImpl ()
{
 for(set<string>::iterator i = id_queries.begin(); i != id_queries.end(); i++)
 {
  if (!ExecuteStatement(Context->Transaction, "delete from temp_id where q = " + *i))
   PSay("Failed to delete temporary data, q = " + *i, m_database_error, MB_OK, "TframeQuery_Some");
 }
 id_queries.clear();
 return;
}
//---------------------------------------------------------------------------
bool TframeQuery_Some::Valid ()
{
 // avoid pass-through?
 if (spinMin->Position == 0 && spinMax->Position == FHostedQueries.size())
 {
  PSay("You have a 'some' query with a minimum of zero and a maximum of as many queries as are present.\nLogically, that means 'everything', and probably isn't what you meant to do.", m_invalid_data, MB_OK, "TframeQuery_Some");
  return false;
 }

 if (spinMin->Position == 0)
  return TframeQuery_Whole::Appropriate(SourceTableName.c_str()) && TframeQueryOperator::Valid();
 return TframeQueryOperator::Valid();
}
//---------------------------------------------------------------------------
xValue TframeQuery_Some::Save (bool with_params)
{
 xValue answer = TframeQueryOperator::Save(with_params);
 if (with_params && xTag(answer) == "map")
 {
  map<string, xValue> config_map = xToMap(answer);
  config_map["min"] = xFromInt(spinMin->Position);
  config_map["max"] = xFromInt(spinMax->Position);
  answer = xFromMap(config_map);
 }
 return answer;
}
//---------------------------------------------------------------------------
void TframeQuery_Some::Load (xValue config, bool with_params)
{
 TframeQueryOperator::Load(config, with_params);
 if (with_params && xTag(config) == "map")
 {
  map<string, xValue> config_map = xToMap(config);
  // max must increase first, because min must remain less than or equal to max (event handlers)
  if (xTag(config_map["max"]) == "int")
   spinMax->Position = xToInt(config_map["max"]);
  if (xTag(config_map["min"]) == "int")
   spinMin->Position = xToInt(config_map["min"]);
 }
}
//---------------------------------------------------------------------------
TframeQueryGeneric* TframeQuery_Some::Create (TComponent* owner)
{
 TframeQueryGeneric* temp;
 FORM_TRY
  temp = new TframeQuery_Some(owner);
 FORM_CATCH
 return temp;
}
//---------------------------------------------------------------------------
void __fastcall TframeQuery_Some::spinMaxChangingEx(TObject *Sender,
      bool &AllowChange, short NewValue, TUpDownDirection Direction)
{
 AllowChange = (NewValue >= spinMin->Position) && ((unsigned int)NewValue <= FHostedQueries.size());
}
//---------------------------------------------------------------------------
void __fastcall TframeQuery_Some::spinMinChangingEx(TObject *Sender,
      bool &AllowChange, short NewValue, TUpDownDirection Direction)
{
 AllowChange = ((NewValue <= spinMax->Position) && (NewValue >= 0));
}
//---------------------------------------------------------------------------
void __fastcall TframeQuery_Some::ChangeHostedQuery (TframeQueryGeneric* Value)
{
 TframeQueryOperator::ChangeHostedQuery(Value);
 spinMin->Position = min(min((unsigned int)(spinMax->Position), FHostedQueries.size()), (unsigned int)(spinMin->Position));
 spinMax->Position = min((unsigned int)(spinMax->Position), FHostedQueries.size());
 return;
}
//---------------------------------------------------------------------------
void __fastcall TframeQuery_Some::SetSimple (bool Value)
{
 TframeQueryOperator::SetSimple(Value);

 // override the size thing
 Height = (Simple ? 24 : 48);
 Resized();
}
//---------------------------------------------------------------------------
string TframeQuery_Some::Describe (bool as_html)
{
 unsigned int _min, _max;
 _min = spinMin->Position;
 _max = spinMax->Position;

 string desc;

 if (!_min && !_max)
  desc = "none";
 else
 if (_min == _max && _min == FHostedQueries.size())
  desc = "all";
 else
 if (!_min && _max == FHostedQueries.size())
  desc = "whole"; // special case, disallowed by Valid()
 else
 if (_min == 1 && _max == FHostedQueries.size())
  desc = "any";
 else
 if (!_min)
  desc = "at most " + IntToString(_max);
 else
 if (_max == FHostedQueries.size())
  desc = "at least " + IntToString(_min);
 else
 if (_max == _min)
  desc = "exactly " + IntToString(_max);
 else
  desc = "between " + IntToString(_min) + " and " + IntToString(_max);

 if (desc != "whole")
  return UppercaseFirstChar(LowercaseString(dbPrettyPlural(LowercaseString(SourceTableName.c_str()))) + " that match " + desc + " of the following:") + (as_html ? "<br>" : "\r\n") +
   TframeQueryOperator::Describe(as_html);
 else
  return UppercaseFirstChar("All " + LowercaseString(dbPrettyPlural(LowercaseString(SourceTableName.c_str())))) + (as_html ? "<br>" : "\r\n");
}
//---------------------------------------------------------------------------

