//---------------------------------------------------------------------------

#include <vcl.h>
#pragma hdrstop

#include "codeQuery_Whole.h"
#include "db_hardcoded_metadata.h"
#include "util.h"
#include "lookuptablewrapper.h"
#include "components.h"
//---------------------------------------------------------------------------
#pragma package(smart_init)
#pragma link "codeQueryGeneric"
#pragma resource "*.dfm"
TframeQuery_Whole *frameQuery_Whole;
//---------------------------------------------------------------------------
__fastcall TframeQuery_Whole::TframeQuery_Whole(TComponent* Owner)
        : TframeQueryGeneric(Owner)
{
 FHowAppropriate = Appropriate;
 FHowCreate = Create;
 Cache = true;  // always, always cache (stupid not to)
 Proxy = false; // never, ever, ever proxy (these should never be slow)
 Clip  = false; // never, ever, ever clip (these -define- how to clip, it'd be recursive)
}
//---------------------------------------------------------------------------
void __fastcall TframeQuery_Whole::SetCache (bool Value)
{
 FCache = true;
}
//---------------------------------------------------------------------------
void __fastcall TframeQuery_Whole::SetClip (bool Value)
{
 FClip = false;
}
//---------------------------------------------------------------------------
void __fastcall TframeQuery_Whole::SetProxy (bool Value)
{
 FProxy = false;
}
//---------------------------------------------------------------------------
string TframeQuery_Whole::PrepareImpl ()
{
 string answer;
 // TODO: modify to match permissions on each table (restrict by group_id, etc. as appropriate)
 // TODO: for things like COB's, may need to make sure we have a view that is -just- COB's to make db_hardcoded_data happy.

   // ugh -- by forcing it to not be indexed, we screw ourselves over when doing 'whole' searches by themselves -- an Indexed __property would almost make sense, but ... dude.
   // just imagine:
   // - Index(ed)
   // - Proxy
   // - Cache
   // ... that's a lot of tuning parameters to deal with.

 if (dbIsAvailable(SourceTableName.c_str()))
 {
  if (SourceTableName == "conditions" ||
      SourceTableName == "self-conditions" ||
      SourceTableName == "rel-conditions" ||
      SourceTableName == "unk-conditions")
  {
   // TODO: stolen from somewhere -- this really should be reorganized at some point ... not just centralized, but maybe replaced? that'd be nice.
   vector<string> allowed_groups;
   if (optionsPermissions["table.conditions.common"]["view"] == "true")
     allowed_groups.push_back("0");
   if (optionsPermissions["table.conditions.diab"]["view"] == "true")
     allowed_groups.push_back(lookupTables->IdOf("groups", "Diabetes"));
   if (optionsPermissions["table.conditions.beh"]["view"] == "true")
     allowed_groups.push_back(lookupTables->IdOf("groups", "Behavioral Health"));
   if (optionsPermissions["table.conditions.com"]["view"] == "true")
     allowed_groups.push_back(lookupTables->IdOf("groups", "Community Health"));
   if (optionsPermissions["table.conditions.lac"]["view"] == "true")
     allowed_groups.push_back(lookupTables->IdOf("groups", "Little Axe Clinic"));
   if (optionsPermissions["table.conditions.den"]["view"] == "true")
     allowed_groups.push_back(lookupTables->IdOf("groups", "Dental Clinic"));
   if (optionsPermissions["table.conditions.cob"]["view"] == "true")
     allowed_groups.push_back(lookupTables->IdOf("groups", "Contract Health"));
   if (optionsPermissions["table.conditions.pub"]["view"] == "true")
     allowed_groups.push_back(lookupTables->IdOf("groups", "Public Health"));
   // TODO: possibly dangerous ... members of a group might need stuff taken away from them, even from their own group?
   allowed_groups.push_back(current_groupid);

   string restrictions;

   if (SourceTableName == "conditions")
    restrictions = "and condition_id + 1 != 8 and task_id != 0 ";
   else
   if (SourceTableName == "self-conditions")
    restrictions = "and condition_id + 1 != 8 and task_id = 0 ";
   else
   if (SourceTableName == "rel-conditions")
    restrictions = "and condition_id + 1 = 8 and task_id = 0 ";

   answer = Context->CreateView("select id from conditions where "
                                "current_id + 1 > 3 " + restrictions +
                                "and (1 = 0 or conditions.group_id in (" + VecToCommaString(allowed_groups) + ")) ", // TODO -c HACK: as below, prevent index abuse
                                set<string>(), "whole");
  } else
  if (SourceTableName == "obs_response_sets")
  {
   set<string> allowed_groups;
   if (optionsPermissions["table.measurements.common"]["view"] == "true")
     allowed_groups.insert("0");
   if (optionsPermissions["table.measurements.diab"]["view"] == "true")
     allowed_groups.insert(lookupTables->IdOf("groups", "Diabetes"));
   if (optionsPermissions["table.measurements.beh"]["view"] == "true")
     allowed_groups.insert(lookupTables->IdOf("groups", "Behavioral Health"));
   if (optionsPermissions["table.measurements.com"]["view"] == "true")
     allowed_groups.insert(lookupTables->IdOf("groups", "Community Health"));
   if (optionsPermissions["table.measurements.lac"]["view"] == "true")
     allowed_groups.insert(lookupTables->IdOf("groups", "Little Axe Clinic"));
   if (optionsPermissions["table.measurements.den"]["view"] == "true")
     allowed_groups.insert(lookupTables->IdOf("groups", "Dental Clinic"));
   if (optionsPermissions["table.measurements.cob"]["view"] == "true")
     allowed_groups.insert(lookupTables->IdOf("groups", "Contract Health"));
   if (optionsPermissions["table.measurements.pub"]["view"] == "true")
     allowed_groups.insert(lookupTables->IdOf("groups", "Public Health"));
   // TODO: possibly dangerous ... members of a group might need stuff taken away from them, even from their own group?
   allowed_groups.insert(current_groupid);

   answer = Context->CreateView("select id from obs_response_sets "
                                "where current_id + 1 > 3 "
                                "and (1 = 0 or obs_response_sets.group_id in (" + SetToCommaString(allowed_groups) + ")) " // TODO -c HACK: as below, prevent index abuse
                                "and obs_response_sets.obs_question_set_id in (select oqs.id from obs_question_set_types oqs inner join permissions p on p.s = oqs.permission_name and p.k = 'VIEW' and p.v = 'TRUE' and p.user_id = " + current_userid + ") ",
                                set<string>(), "whole");
  } else
  if (SourceTableName == "tasks")
  {
   // DONE: stolen from somewhere -- this really should be reorganized at some point ... not just centralized, but maybe replaced? that'd be nice.
   vector<string> allowed_groups = SetToVec(GetGroupPermissions("table.tasks", "", "view", "true"));
/*
   // no 'common'? tasks should never have group_id 0?
   if (optionsPermissions["table.tasks.diab"]["view"] == "true")
     allowed_groups.push_back(lookupTables->IdOf("groups", "Diabetes"));
   if (optionsPermissions["table.tasks.beh"]["view"] == "true")
     allowed_groups.push_back(lookupTables->IdOf("groups", "Behavioral Health"));
   if (optionsPermissions["table.tasks.com"]["view"] == "true")
     allowed_groups.push_back(lookupTables->IdOf("groups", "Community Health"));
   if (optionsPermissions["table.tasks.lac"]["view"] == "true")
     allowed_groups.push_back(lookupTables->IdOf("groups", "Little Axe Clinic"));
   if (optionsPermissions["table.tasks.den"]["view"] == "true")
     allowed_groups.push_back(lookupTables->IdOf("groups", "Dental Clinic"));
   if (optionsPermissions["table.tasks.cob"]["view"] == "true")
     allowed_groups.push_back(lookupTables->IdOf("groups", "Contract Health"));
   if (optionsPermissions["table.tasks.pub"]["view"] == "true")
     allowed_groups.push_back(lookupTables->IdOf("groups", "Public Health"));
   // permissions exist for these two, so we might as well, even though ... at least transportation is rare. (never?)
   // have I mentioned this gives me the heeby-jeebies? this code being pasted? and the logic itself? and ... ew.
   if (optionsPermissions["table.tasks.care"]["view"] == "true")
     allowed_groups.push_back(lookupTables->IdOf("groups", "Caregiver Support Program"));
   if (optionsPermissions["table.tasks.trans"]["view"] == "true")
     allowed_groups.push_back(lookupTables->IdOf("groups", "Transportation"));
   // TODO: possibly dangerous ... members of a group might need stuff taken away from them, even from their own group?
   allowed_groups.push_back(current_groupid);
*/

   answer = Context->CreateView("select id from tasks "
                                "where current_id + 1 > 3 "
                                "and cob_id + 1 = 1 and (1 = 0 or " // TODO -c HACK: prevent it from abusing the index
                                "tasks.group_id in (" + VecToCommaString(allowed_groups) + ")) ", set<string>(), "whole");
  } else
   answer = Context->CreateView("select id from " + string(SourceTableName.c_str()) + " where current_id + 1 > 3 ", set<string>(), "whole");
 }
 return answer;
}
//---------------------------------------------------------------------------
bool TframeQuery_Whole::Appropriate (string source_table_name)
{
 return dbIsAvailable(source_table_name);
}
//---------------------------------------------------------------------------
TframeQueryGeneric* TframeQuery_Whole::Create (TComponent* owner)
{
 TframeQueryGeneric* temp;
 FORM_TRY
  temp = new TframeQuery_Whole(owner);
 FORM_CATCH
 return temp;
}
//---------------------------------------------------------------------------
string TframeQuery_Whole::Describe (bool as_html)
{
 return UppercaseFirstChar("All " + LowercaseString(dbPrettyPlural(LowercaseString(SourceTableName.c_str())))) + (as_html ? "<br>" : "\r\n");
}
//---------------------------------------------------------------------------

