//---------------------------------------------------------------------------

#include <vcl.h>
#pragma hdrstop

#include "codeQuery__PeopleByAge.h"
//---------------------------------------------------------------------------
#pragma package(smart_init)
#pragma link "codeQueryGeneric"
#pragma resource "*.dfm"
TframeQuery__PeopleByAge *frameQuery__PeopleByAge;
//---------------------------------------------------------------------------
__fastcall TframeQuery__PeopleByAge::TframeQuery__PeopleByAge(TComponent* Owner)
        : TframeQueryGeneric(Owner)
{
 FHowAppropriate = Appropriate;
 FHowCreate = Create;
 Cache = true;
 dateComp->DateTime = Date();
}
//---------------------------------------------------------------------------
string TframeQuery__PeopleByAge::PrepareImpl ()
{
 string answer;
 Pdate as_of = Pdate(dateComp->DateTime);

 bool has_min_age = (editMinAge->Text != "");
 int min_age = StringToInt(editMinAge->Text.c_str());
 Pdate max_date = as_of;
 max_date.mod(-min_age, in_years);

 bool has_max_age = (editMaxAge->Text != "");
 int max_age = StringToInt(editMaxAge->Text.c_str());
 Pdate min_date = as_of;
 min_date.mod(-(max_age + 1), in_years);

 // you don't have an age when you're dead.
 // death_date is null or death_date > as_of
 // birth_date is not null [and birth_date > min_date] [and birth_date =< max_date]
 // if they don't have a birth date, all we can calculate is that they were alive -- useless.

 // "3 to 6 years old on 6/1/2005"
 // min age = 3
 //  born on 6/1/2002 -> 3 years old
 //  born on 6/2/2002 -> 2 years old (bad)
 //  birth_date <= max date
 //  max date = this date - min age
 //  2002 = 2005 - 3
 // max age = 6
 //  born on 6/1/1998 -> 7 years old (bad)
 //  born on 6/2/1998 -> 6 years old
 //  birth_date > min date
 //  min date = this date - (max age + 1)
 //  1998 = 2005 - (6 + 1)

 string sql = "select people.id from people where people.birth_date is not null"
              " and (people.death_date is null or people.death_date > '" + as_of.OutDate(global_mode) + "')";
 if (has_max_age) // max age, min date
  sql += " and people.birth_date > '" + min_date.OutDate(global_mode) + "'";
 if (has_min_age) // min age, max date
  sql += " and people.birth_date <= '" + max_date.OutDate(global_mode) + "'";
 answer = Context->CreateView(sql, set<string>());
 return answer;
}
//---------------------------------------------------------------------------
bool TframeQuery__PeopleByAge::Valid ()
{
 editMinAge->Color = clWindow;
 editMaxAge->Color = clWindow;

 if (editMinAge->Text != "" && !IsInt(editMinAge->Text.c_str()))
  editMinAge->Color = clBad;
 if (editMaxAge->Text != "" && !IsInt(editMaxAge->Text.c_str()))
  editMaxAge->Color = clBad;
 if (editMinAge->Text != "" && IsInt(editMinAge->Text.c_str()) && StringToInt(editMinAge->Text.c_str()) < 0)
  editMinAge->Color = clBad;
 if (editMaxAge->Text != "" && IsInt(editMaxAge->Text.c_str()) && StringToInt(editMaxAge->Text.c_str()) < 0)
  editMaxAge->Color = clBad;
 if (editMinAge->Text != "" && IsInt(editMinAge->Text.c_str()) &&
     editMaxAge->Text != "" && IsInt(editMaxAge->Text.c_str()) &&
     StringToInt(editMinAge->Text.c_str()) > StringToInt(editMaxAge->Text.c_str()))
 {
  editMinAge->Color = clBad;
  editMaxAge->Color = clBad;
 }

 return TframeQueryGeneric::Valid() &&
        (editMinAge->Color == clWindow) &&
        (editMaxAge->Color == clWindow);
}
//---------------------------------------------------------------------------
xValue TframeQuery__PeopleByAge::Save (bool with_params)
{
 xValue answer = TframeQueryGeneric::Save(with_params);
 if (with_params && xTag(answer) == "map")
 {
  map<string, xValue> config_map = xToMap(answer);
  if (editMinAge->Text != "" && IsInt(editMinAge->Text.c_str()))
   config_map["min_age"] = xFromInt(StringToInt(editMinAge->Text.c_str()));
  if (editMaxAge->Text != "" && IsInt(editMaxAge->Text.c_str()))
   config_map["max_age"] = xFromInt(StringToInt(editMaxAge->Text.c_str()));
  config_map["as_of"] = xFromPdate(Pdate(dateComp->DateTime));
  // testing purposes
  // config_map["as_of"] = "global(as_of_date)";
  answer = xFromMap(config_map);
 }
 return answer;
}
//---------------------------------------------------------------------------
void TframeQuery__PeopleByAge::Load (xValue config, bool with_params)
{
 config = ReplaceWithGlobals(config);

 TframeQueryGeneric::Load(config, with_params);
 if (with_params && xTag(config) == "map")
 {
  map<string, xValue> config_map = xToMap(config);

  if (xTag(config_map["min_age"]) == "int")
   editMinAge->Text = IntToString(xToInt(config_map["min_age"])).c_str();
  else
   editMinAge->Text = "";
  if (xTag(config_map["max_age"]) == "int")
   editMaxAge->Text = IntToString(xToInt(config_map["max_age"])).c_str();
  else
   editMaxAge->Text = "";
  if (xTag(config_map["as_of"]) == "date")
   dateComp->DateTime = xToPdate(config_map["as_of"]).OutDate();
 }
}
//---------------------------------------------------------------------------
TframeQueryGeneric* TframeQuery__PeopleByAge::Create (TComponent* owner)
{
 TframeQueryGeneric* temp;
 FORM_TRY
  temp = new TframeQuery__PeopleByAge(owner);
 FORM_CATCH
 return temp;
}
//---------------------------------------------------------------------------
bool TframeQuery__PeopleByAge::Appropriate (string source_table_name)
{
 return (source_table_name == "people") && (optionsPermissions["table.people"]["view"] == "true");
}
//---------------------------------------------------------------------------
string TframeQuery__PeopleByAge::Describe (bool as_html)
{
 bool has_min_age = (editMinAge->Text != "");
 int min_age = StringToInt(editMinAge->Text.c_str());

 bool has_max_age = (editMaxAge->Text != "");
 int max_age = StringToInt(editMaxAge->Text.c_str());

 string desc;
 if (has_min_age && has_max_age)
  desc = IntToString(min_age) + " to " + IntToString(max_age);
 else
 if (has_min_age)
  desc = IntToString(min_age) + " and up";
 else
 if (has_max_age)
  desc = IntToString(max_age) + " and under";
 else
  desc = "unknown";

 return "People aged " + desc + " as of " + Pdate(dateComp->DateTime).OutDate(global_mode) + string(as_html ? ".<br>" : ".\r\n");
}
//---------------------------------------------------------------------------

