//---------------------------------------------------------------------------

#include <vcl.h>
#pragma hdrstop

#include "codeQuery__PeopleByMetabolicGoalsReached.h"
#include "control_break_objects.h"
#include "lookuptablewrapper.h"
//---------------------------------------------------------------------------
#pragma package(smart_init)
#pragma link "codeQueryGeneric"
#pragma resource "*.dfm"
TframeQuery__PeopleByMetabolicGoalsReached *frameQuery__PeopleByMetabolicGoalsReached;
//---------------------------------------------------------------------------
__fastcall TframeQuery__PeopleByMetabolicGoalsReached::TframeQuery__PeopleByMetabolicGoalsReached(TComponent* Owner)
        : TframeQueryGeneric(Owner)
{
 FHowAppropriate = Appropriate;
 FHowCreate = Create;
 Cache = true;

 lookupTables->RegisterComboBox("measurement_types", comboField);
 // TODO: set default dates

 list_goals.setup_listview(listGoals, false);
 list_goals.add_or_modify_column("key",  "Field", true, 0, 120, as_text, rawString);
 list_goals.add_or_modify_column("op",   "",      true, 1,  30, as_text, rawString);
 list_goals.add_or_modify_column("val",  "Value", true, 2,  80, as_double, rawString);
 list_goals.setup_callbacks((TFrame*)this, InformForm(&(TframeQuery__PeopleByMetabolicGoalsReached::GoalSelected)), NULL);
}
//---------------------------------------------------------------------------
__fastcall TframeQuery__PeopleByMetabolicGoalsReached::~TframeQuery__PeopleByMetabolicGoalsReached()
{
 lookupTables->UnregisterComboBox(comboField);
}
//---------------------------------------------------------------------------
void __fastcall TframeQuery__PeopleByMetabolicGoalsReached::comboFieldKeyPress(TObject *Sender,
      char &Key)
{
 FindComboBoxItem(Sender, Key);
}
//---------------------------------------------------------------------------
void __fastcall TframeQuery__PeopleByMetabolicGoalsReached::comboRelationKeyPress(TObject *Sender,
      char &Key)
{
 FindComboBoxItem(Sender, Key);
}
//---------------------------------------------------------------------------
void TframeQuery__PeopleByMetabolicGoalsReached::GoalSelected ()
{
 // select goal
 map<string, string> row_data;
 if (!list_goals.get_current_row(row_data))
 {
  // set default values
  row_data["key"] = "";
  row_data["op"]  = "";
  row_data["val"] = "0";
 }

 // update controls to reflect data in row_data
 for (int i = 0; i < comboField->Items->Count; i++)
  if (comboField->Items->Strings[i] == row_data["key"].c_str())
  {
   comboField->ItemIndex = i;
   break;
  }
 for (int i = 0; i < comboRelation->Items->Count; i++)
  if (comboRelation->Items->Strings[i] == row_data["op"].c_str())
  {
   comboRelation->ItemIndex = i;
   break;
  }
 editValue->Text = row_data["val"].c_str();
}
//---------------------------------------------------------------------------
void __fastcall TframeQuery__PeopleByMetabolicGoalsReached::buttonGoalAddClick(TObject *Sender)
{
 listGoals->Selected = NULL;
}
//---------------------------------------------------------------------------
void __fastcall TframeQuery__PeopleByMetabolicGoalsReached::buttonGoalRemoveClick(TObject *Sender)
{
 list_goals.remove_current_row(true);
}
//---------------------------------------------------------------------------
void __fastcall TframeQuery__PeopleByMetabolicGoalsReached::buttonGoalClearClick(TObject *Sender)
{
 list_goals.clear_data();
}
//---------------------------------------------------------------------------
void __fastcall TframeQuery__PeopleByMetabolicGoalsReached::buttonGoalSaveClick(TObject *Sender)
{
 if (comboField->ItemIndex == -1)
 {
  PSay("Please select a metric.", m_missing_data, MB_OK, "TframeQuery__PeopleByMetabolicGoalsReached");
  return;
 }

 if (comboRelation->ItemIndex == -1)
 {
  PSay("Please select a relational operator.", m_missing_data, MB_OK, "TframeQuery__PeopleByMetabolicGoalsReached");
  return;
 }

 if (editValue->Text == "" || !IsFloat(editValue->Text.c_str()))
 {
  PSay("Please enter a metric value.", m_missing_data, MB_OK, "TframeQuery__PeopleByMetabolicGoalsReached");
  return;
 }

 map<string, string> row_data;
 row_data["key"] = comboField->Text.c_str();
 row_data["op"]  = comboRelation->Text.c_str();
 row_data["val"] = FloatToString(StringToFloat(editValue->Text.c_str()));

 set<string> dupe_columns;
 dupe_columns.insert("key");
 if (list_goals.is_future_duplicate (row_data, dupe_columns))
 {
  PSay("Saving this row as it is would create a duplicate entry in the list.\nYou may not have two rows with the same field name.", m_duplicate_data, MB_OK, "TframeQuery__PeopleByMetabolicGoalsReached");
  return;
 }

 list_goals.add_or_modify_row_data(row_data, false);
 listGoals->Selected = NULL;
}
//---------------------------------------------------------------------------
string TframeQuery__PeopleByMetabolicGoalsReached::PrepareImpl ()
{
 string answer;

 // people who ...
 //  met goal 1 between A and B and did not meet goal 1 between C and D
 //  or who ...
 //  met goal 2 between A and B and did not meet goal 2 between C and D
 //  ...
 // "met" and "did not meet" ---> value was non-null, (non-zero), and did or did not meet the criterion

 vector<map<string, string> > all_goals;
 list_goals.export_data(all_goals);

 string clause_met, clause_unmet;

 Pdate met_min(dateMetMin->DateTime);
 Pdate met_max(editMetMax->Text.c_str(), global_mode);
 Pdate unmet_min(editUnmetMin->Text.c_str(), global_mode);
 Pdate unmet_max(dateUnmetMax->DateTime);

 if (met_max.IsValidDate())
  clause_met = "between '" + met_min.OutDate(global_mode) + "' and '" + met_max.OutDate(global_mode) + "'";
 else
  clause_met = ">= '" + met_min.OutDate(global_mode) + "'";

 if (unmet_min.IsValidDate())
  clause_unmet = "between '" + unmet_min.OutDate(global_mode) + "' and '" + unmet_max.OutDate(global_mode) + "'";
 else
  clause_unmet = "<= '" + unmet_max.OutDate(global_mode) + "'";

 string f, r, v;
 string sql = "select id from people where ";
 for (vector<map<string, string> >::iterator i = all_goals.begin(); i != all_goals.end(); i++)
 {
  if (i != all_goals.begin())
   sql += "or ";

  f = (*i)["key"];
  r = (*i)["op"];
  v = "'" + (*i)["val"] + "'";

  // met the goal at least once during period B
  // -> any value met requirements (could switch to maximum/minimum -did- meet requirements), or even avg()
  // did not meet the goal ever during period A
  // had a related measurement during period A
  // -> maximum/minimum during period A didn't meet requirements (could switch to avg())

  string rel_unmet, agg_unmet;
  if (r == "<")
   rel_unmet = ">=", agg_unmet = "min"; // unmet means min >= desired value
  else
  if (r == "<=")
   rel_unmet = ">", agg_unmet = "min"; // unmet means min > desired value
  else
  if (r == ">")
   rel_unmet = "<=", agg_unmet = "max"; // unmet means max <= desired value
  else
  if (r == ">=")
   rel_unmet = "<", agg_unmet = "max"; // unmet means max < desired value

  sql += "(exists(select " + f + " from measurements where person_id = people.id and " + f + " != 0 and current_id + 1 > 3 and date_of_measurement " + clause_met + " and " + f + " " + r + " " + v + ") "
         "and "
         "(select " + agg_unmet + "(" + f + ") from measurements where person_id = people.id and " + f + " != 0 and current_id + 1 > 3 and date_of_measurement " + clause_unmet + ") " + rel_unmet + " " + v + ") ";
 }

 answer = Context->CreateView(sql, set<string>());
 return answer;
}
//---------------------------------------------------------------------------
bool TframeQuery__PeopleByMetabolicGoalsReached::Valid ()
{
 bool answer;
 // & doesn't do short-circuit evaluation, && does, and we want not to
 answer = quValidSortedOptionalDate(dateMetMin, editMetMax) &
          quValidSortedOptionalDate(editUnmetMin, dateUnmetMax) &
          quSortedRequiredDate(dateUnmetMax, dateMetMin);

 if (!listGoals->Items->Count)
 {
  PSay("You must enter at least one goal.", m_missing_data, MB_OK, "TframeQuery__PeopleByMetabolicGoalsReached");
  answer = false;
 }

 return TframeQueryGeneric::Valid() & answer;
}
//---------------------------------------------------------------------------
xValue TframeQuery__PeopleByMetabolicGoalsReached::Save (bool with_params)
{
 xValue answer = TframeQueryGeneric::Save(with_params);
 if (with_params && xTag(answer) == "map")
 {
  map<string, xValue> config_map = xToMap(answer);

  quSaveRequiredDate(config_map, "met_start", dateMetMin);
  quSaveOptionalDate(config_map, "met_stop", editMetMax);
  quSaveOptionalDate(config_map, "unmet_start", editUnmetMin);
  quSaveRequiredDate(config_map, "unmet_stop", dateUnmetMax);

  vector<map<string, string> > raw_goals;
  list_goals.export_data(raw_goals);

  // sort using the most complicated method possible ...
  // why sort? so Same() can see them as being the same even if values are entered in the wrong order.
  // 'key' should be unique (no comparisons twice on the same field) so sorting by it alone is plenty.
  //
  // making this into a handy function wouldn't be bad ... maybe a va_arg function, where you can pass field names in order? (NULL-terminated to avoid having to send a count ahead of time? does "" equal NULL for blank field names?)
  vector<pair<string, cb_sorting_object<string> *> > sorting_methods;
  vector<map<string, string> > sorted_goals;
  sorting_methods.push_back(pair<string, cb_sorting_object<string> *>("key", new sort_less_than<string>()));
  multi_column_sorting_object<string, string> sorting_object (sorting_methods);
  sort(raw_goals.begin(), raw_goals.end(), sorting_object);
  cb_deallocate_sorting_map(sorting_methods);

  vector<xValue> saved_goals;
  for (vector<map<string, string> >::iterator i = raw_goals.begin(); i != raw_goals.end(); i++)
   saved_goals.push_back(xFromStringMap(*i));

  config_map["goals"] = xFromVec(saved_goals);

  answer = xFromMap(config_map);
 }
 return answer;
}
//---------------------------------------------------------------------------
void TframeQuery__PeopleByMetabolicGoalsReached::Load (xValue config, bool with_params)
{
 config = ReplaceWithGlobals(config);

 TframeQueryGeneric::Load(config, with_params);
 if (with_params && xTag(config) == "map")
 {
  map<string, xValue> config_map = xToMap(config);

  quLoadRequiredDate(config_map, "met_start", dateMetMin);
  quLoadOptionalDate(config_map, "met_stop", editMetMax);
  quLoadOptionalDate(config_map, "unmet_start", editUnmetMin);
  quLoadRequiredDate(config_map, "unmet_stop", dateUnmetMax);

  vector<map<string, string> > restored_goals;
  vector<xValue> saved_goals = xToVec(config_map["goals"]);
  for (vector<xValue>::iterator i = saved_goals.begin(); i != saved_goals.end(); i++)
   restored_goals.push_back(xToStringMap(*i));
  list_goals.clear_data();
  list_goals.import_data(restored_goals);
 }
}
//---------------------------------------------------------------------------
TframeQueryGeneric* TframeQuery__PeopleByMetabolicGoalsReached::Create (TComponent* owner)
{
 TframeQueryGeneric* temp;
 FORM_TRY
  temp = new TframeQuery__PeopleByMetabolicGoalsReached(owner);
 FORM_CATCH
 return temp;
}
//---------------------------------------------------------------------------
bool TframeQuery__PeopleByMetabolicGoalsReached::Appropriate (string source_table_name)
{
 return (source_table_name == "people") &&
        (optionsPermissions["table.people"]["view"] == "true") &&
        (optionsPermissions["table.measurements"]["view"] == "true");
}
//---------------------------------------------------------------------------