//---------------------------------------------------------------------------

#include <vcl.h>
#pragma hdrstop

#include "codeQuery__ProceduresByDate.h"
//---------------------------------------------------------------------------
#pragma package(smart_init)
#pragma link "codeQueryGeneric"
#pragma resource "*.dfm"
TframeQuery__ProceduresByDate *frameQuery__ProceduresByDate;
//---------------------------------------------------------------------------
__fastcall TframeQuery__ProceduresByDate::TframeQuery__ProceduresByDate(TComponent* Owner)
        : TframeQueryGeneric(Owner)
{
 FHowAppropriate = Appropriate;
 FHowCreate = Create;
 Cache = true;
}
//---------------------------------------------------------------------------
string TframeQuery__ProceduresByDate::PrepareImpl ()
{
 string answer;

 Pdate start_date = Pdate(editStartDate->Text.c_str(), global_mode);
 Pdate stop_date  = Pdate(editStopDate->Text.c_str(), global_mode);
 bool start_given = (editStartDate->Text != "");
 bool stop_given  = (editStopDate->Text != "");
 bool start_valid = (!start_given || start_date.IsValidDate());
 bool stop_valid  = (!stop_given || stop_date.IsValidDate());

 // experiment: use whole views for permissions and hope for the best
 string sql = "select procedures.id from procedures where procedures.start_date is not null" +
               ((start_given) ? string(" and start_date >= '" + start_date.OutDate(global_mode) + "'") : string("")) +
               ((stop_given) ? string(" and start_date <= '" + stop_date.OutDate(global_mode) + "'") : string(""));
 answer = Context->CreateView(sql, set<string>());
 return answer;
}
//---------------------------------------------------------------------------
bool TframeQuery__ProceduresByDate::Valid ()
{
 editStartDate->Color = clWindow;
 editStopDate->Color = clWindow;

 Pdate start_date = Pdate(editStartDate->Text.c_str(), global_mode);
 Pdate stop_date  = Pdate(editStopDate->Text.c_str(), global_mode);
 bool start_given = (editStartDate->Text != "");
 bool stop_given  = (editStopDate->Text != "");
 bool start_valid = (!start_given || start_date.IsValidDate());
 bool stop_valid  = (!stop_given || stop_date.IsValidDate());

 if (!start_valid)
  editStartDate->Color = clBad;

 if (!stop_valid)
  editStopDate->Color = clBad;

 if (start_given && stop_given && start_valid && stop_valid && start_date > stop_date)
  PSay("The start date should be earlier than the stop date, if both are provided.", m_invalid_data, MB_OK, "TframeQuery__ProceduresByDate");

 if (!start_given && !stop_given)
  PSay("To restrict by procedure date, you must provide at least one date.", m_missing_data, MB_OK, "TframeQuery__ProceduresByDate");

 bool answer = (TframeQueryGeneric::Valid() && start_valid && stop_valid && (start_given || stop_given) && (!start_given || !stop_given || start_date <= stop_date));
 return answer;
}
//---------------------------------------------------------------------------
xValue TframeQuery__ProceduresByDate::Save (bool with_params)
{
 xValue answer = TframeQueryGeneric::Save(with_params);
 if (with_params && xTag(answer) == "map")
 {
  map<string, xValue> config_map = xToMap(answer);

  if (Pdate(editStartDate->Text.c_str(), global_mode).IsValidDate())
   config_map["start"] = xFromPdate(Pdate(editStartDate->Text.c_str(), global_mode));
  if (Pdate(editStopDate->Text.c_str(), global_mode).IsValidDate())
   config_map["stop"] = xFromPdate(Pdate(editStopDate->Text.c_str(), global_mode));

  answer = xFromMap(config_map);
 }
 return answer;
}
//---------------------------------------------------------------------------
void TframeQuery__ProceduresByDate::Load (xValue config, bool with_params)
{
 config = ReplaceWithGlobals(config);

 TframeQueryGeneric::Load(config, with_params);
 if (with_params && xTag(config) == "map")
 {
  map<string, xValue> config_map = xToMap(config);

  if (xTag(config_map["start"]) == "date")
   editStartDate->Text = xToPdate(config_map["start"]).OutDate(global_mode).c_str();
  if (xTag(config_map["stop"]) == "date")
   editStopDate->Text = xToPdate(config_map["stop"]).OutDate(global_mode).c_str();
 }
}
//---------------------------------------------------------------------------
TframeQueryGeneric* TframeQuery__ProceduresByDate::Create (TComponent* owner)
{
 TframeQueryGeneric* temp;
 FORM_TRY
  temp = new TframeQuery__ProceduresByDate(owner);
 FORM_CATCH
 return temp;
}
//---------------------------------------------------------------------------
bool TframeQuery__ProceduresByDate::Appropriate (string source_table_name)
{
 return (source_table_name == "procedures") && (optionsPermissions["table.procedures"]["view"] == "true");
}
//---------------------------------------------------------------------------
