//---------------------------------------------------------------------------

#include <vcl.h>
#pragma hdrstop

#include "codeQuery__TasksByDate.h"
//---------------------------------------------------------------------------
#pragma package(smart_init)
#pragma link "codeQueryGeneric"
#pragma resource "*.dfm"
TframeQuery__TasksByDate *frameQuery__TasksByDate;
//---------------------------------------------------------------------------
__fastcall TframeQuery__TasksByDate::TframeQuery__TasksByDate(TComponent* Owner)
        : TframeQueryGeneric(Owner)
{
 FHowAppropriate = Appropriate;
 FHowCreate = Create;
 Cache = true;
}
//---------------------------------------------------------------------------
string TframeQuery__TasksByDate::PrepareImpl ()
{
 string answer;

 Pdate start_date = Pdate(editStartDate->Text.c_str(), global_mode);
 Pdate stop_date  = Pdate(editStopDate->Text.c_str(), global_mode);
 bool start_given = (editStartDate->Text != "");
 bool stop_given  = (editStopDate->Text != "");
 bool start_valid = (!start_given || start_date.IsValidDate());
 bool stop_valid  = (!stop_given || stop_date.IsValidDate());

 // TODO: see if we can use 'whole' ... inner joins inside the exists() clauses, to avoid issues with permissions, etc.
 string sql = "select tasks.id from tasks where ";
 if (start_given && !stop_given)
  sql += "date_of_task >= '" + start_date.OutDate(global_mode) + "'";
 else
 if (start_given && stop_given)
  sql += "date_of_task between '" + start_date.OutDate(global_mode) + "' and '" + stop_date.OutDate(global_mode) + "'";
 else
 if (!start_given && stop_given)
  sql += "date_of_task <= '" + stop_date.OutDate(global_mode) + "'";

 answer = Context->CreateView(sql, set<string>());
 return answer;
}
//---------------------------------------------------------------------------
bool TframeQuery__TasksByDate::Valid ()
{
 bool answer = true;

 editStartDate->Color = clWindow;
 editStopDate->Color = clWindow;

 Pdate start_date = Pdate(editStartDate->Text.c_str(), global_mode);
 Pdate stop_date  = Pdate(editStopDate->Text.c_str(), global_mode);
 bool start_given = (editStartDate->Text != "");
 bool stop_given  = (editStopDate->Text != "");
 bool start_valid = (!start_given || start_date.IsValidDate());
 bool stop_valid  = (!stop_given || stop_date.IsValidDate());

 if (!start_valid)
 {
  editStartDate->Color = clBad;
  answer = false;
 }

 if (!stop_valid)
 {
  editStopDate->Color = clBad;
  answer = false;
 }

 if (start_given && stop_given && start_valid && stop_valid && start_date > stop_date)
 {
  PSay("The start date should be earlier than the stop date, if both are provided.", m_invalid_data, MB_OK, "TframeQuery__TasksByDate");
  answer = false;
 }

 if (!start_given && !stop_given)
 {
  PSay("To restrict by task/visit date, you must provide at least one date.", m_missing_data, MB_OK, "TframeQuery__TasksByDate");
  answer = false;
 }

 return TframeQueryGeneric::Valid() && answer;
}
//---------------------------------------------------------------------------
xValue TframeQuery__TasksByDate::Save (bool with_params)
{
 xValue answer = TframeQueryGeneric::Save(with_params);
 if (with_params && xTag(answer) == "map")
 {
  map<string, xValue> config_map = xToMap(answer);
  if (Pdate(editStartDate->Text.c_str(), global_mode).IsValidDate())
   config_map["start"] = xFromPdate(Pdate(editStartDate->Text.c_str(), global_mode));
  if (Pdate(editStopDate->Text.c_str(), global_mode).IsValidDate())
   config_map["stop"] = xFromPdate(Pdate(editStopDate->Text.c_str(), global_mode));
  answer = xFromMap(config_map);
 }
 return answer;
}
//---------------------------------------------------------------------------
void TframeQuery__TasksByDate::Load (xValue config, bool with_params)
{
 config = ReplaceWithGlobals(config);

 TframeQueryGeneric::Load(config, with_params);
 if (with_params && xTag(config) == "map")
 {
  map<string, xValue> config_map = xToMap(config);
  if (xTag(config_map["start"]) == "date")
   editStartDate->Text = xToPdate(config_map["start"]).OutDate(global_mode).c_str();
  if (xTag(config_map["stop"]) == "date")
   editStopDate->Text = xToPdate(config_map["stop"]).OutDate(global_mode).c_str();
 }
}
//---------------------------------------------------------------------------
TframeQueryGeneric* TframeQuery__TasksByDate::Create (TComponent* owner)
{
 TframeQueryGeneric* temp;
 FORM_TRY
  temp = new TframeQuery__TasksByDate(owner);
 FORM_CATCH
 return temp;
}
//---------------------------------------------------------------------------
bool TframeQuery__TasksByDate::Appropriate (string source_table_name)
{
 return (source_table_name == "tasks") && (optionsPermissions["table.tasks"]["view"] == "true");
}
//---------------------------------------------------------------------------

