//---------------------------------------------------------------------------

#include <vcl.h>
#pragma hdrstop

#include "codeRelationMagic.h"
#pragma package(smart_init)
#include "codeRelationEventTimeline.h"
#include "codeRelationObservationTimeline.h"
#include "codeRelationTabular.h"
//---------------------------------------------------------------------------
// ValidCtrCheck is used to assure that the components created do not have
// any pure virtual functions.
//

static inline void ValidCtrCheck(TRelationMagic *)
{
        new TRelationMagic(NULL);
}
//---------------------------------------------------------------------------
__fastcall TRelationMagic::TRelationMagic(TComponent* Owner)
        : TRelationGeneric(Owner)
{
 FDelegate = NULL;
 FDelegateType = dtNone;

 _menuSwitch = new TMenuItem(this);
 _menuSwitchNone = new TMenuItem(this);
 _menuSwitchTabular = new TMenuItem(this);
 _menuSwitchEventTimeline = new TMenuItem(this);
 _menuSwitchObservationTimeline = new TMenuItem(this);

 _menuSwitch->Caption = "Switch to";
 _menuSwitchNone->Caption = "None";
 _menuSwitchTabular->Caption = "Table";
 _menuSwitchEventTimeline->Caption = "Event timeline";
 _menuSwitchObservationTimeline->Caption = "Observation timeline";

 _menuSwitchNone->OnClick = DefaultSwitchClick;
 _menuSwitchTabular->OnClick = DefaultSwitchClick;
 _menuSwitchEventTimeline->OnClick = DefaultSwitchClick;
 _menuSwitchObservationTimeline->OnClick = DefaultSwitchClick;

 _menuSwitchNone->GroupIndex = 1;
 _menuSwitchTabular->GroupIndex = 1;
 _menuSwitchEventTimeline->GroupIndex = 1;
 _menuSwitchObservationTimeline->GroupIndex = 1;

 _menuSwitchNone->RadioItem = true;
 _menuSwitchTabular->RadioItem = true;
 _menuSwitchEventTimeline->RadioItem = true;
 _menuSwitchObservationTimeline->RadioItem = true;

 _menuSwitch->Add(_menuSwitchNone);
 _menuSwitch->Add(_menuSwitchTabular);
 _menuSwitch->Add(_menuSwitchEventTimeline);
 _menuSwitch->Add(_menuSwitchObservationTimeline);

 _menuWhole->Items->NewTopLine();

 OnPopup = DefaultPopup; // why the -fuck- is this necessary? (if not done, inexplicably does not call our implementation of DefaultPopup, even though it's virtual!)
 DelegateType = dtNone;
}
//---------------------------------------------------------------------------
namespace Coderelationmagic
{
        void __fastcall PACKAGE Register()
        {
                 TComponentClass classes[1] = {__classid(TRelationMagic)};
                 RegisterComponents("prometheus", classes, 0);
        }
}
//---------------------------------------------------------------------------
void __fastcall TRelationMagic::SetDelegateType(RMDelegateTypes pValue)
{
 // if you enjoy resetting this value to what it already was, and somehow mind the cost, fuck you.
 // this should be rare and intentional.

 // make a copy of the pointer so we can copy over properties later ...
 TRelationGeneric * OldDelegate = FDelegate;

 switch (pValue)
 {
  case dtTabular:
  {
   FORM_TRY
    FDelegate = new TRelationTabular(this);
   FORM_CATCH
   break;
  }
  case dtEventTimeline:
  {
   FORM_TRY
    FDelegate = new TRelationEventTimeline(this);
   FORM_CATCH
   break;
  }
  case dtObservationTimeline:
  {
   FORM_TRY
    FDelegate = new TRelationObservationTimeline(this);
   FORM_CATCH
   break;
  }
  // TODO: the rest are missing!
 }

 if (FDelegate)
 {
  FDelegateType = pValue;

  FDelegate->Align          = alClient;
  FDelegate->TabOrder       = 0;
  FDelegate->TabStop        = true; // TODO: parent control needs to not be a tabstop itself, or at least auto-tab to child
  FDelegate->DoubleBuffered = this->DoubleBuffered;

  // include our own items in the delegate's popup
  if (_menuSwitch->Parent)
   _menuSwitch->Parent->Remove(_menuSwitch); // to be safe, in case Insert() doesn't properly do this later (paranoid)
  FDelegate->RelationPopupMenu->Items->Insert(0, _menuSwitch);
  FDelegate->RelationPopupMenu->Items->InsertNewLineAfter(_menuSwitch);
  // redirect popup-messages
  FDelegatePopup = FDelegate->OnPopup;
  FDelegate->OnPopup = DelegateDefaultPopup;

  if (OldDelegate)
  {
   FDelegate->Value      = OldDelegate->Value;
   FDelegate->InputState = OldDelegate->InputState;
   FDelegate->OnInitShow = OldDelegate->OnInitShow;
   FDelegate->OnValidate = OldDelegate->OnValidate;
  }

  FDelegate->Parent = this;
 }

 // will result in FDelegate being deleted and nothing created in its place -- resets to nothingness.
 if (pValue == dtNone)
 {
  if (_menuSwitch->Parent)
   _menuSwitch->Parent->Remove(_menuSwitch); // to be safe, in case Insert() doesn't properly do this later (paranoid)
  RelationPopupMenu->Items->Insert(0, _menuSwitch); // add to self instead
  FDelegatePopup = NULL;
  FDelegateType = pValue;
  FDelegate     = NULL;
 }

 // reset our radio items
 switch (FDelegateType)
 {
  case dtNone: _menuSwitchNone->Checked = true; break;
  case dtTabular: _menuSwitchTabular->Checked = true; break;
  case dtEventTimeline: _menuSwitchEventTimeline->Checked = true; break;
  case dtObservationTimeline: _menuSwitchObservationTimeline->Checked = true; break;
 }

 delete OldDelegate;
}
//---------------------------------------------------------------------------
relation __fastcall TRelationMagic::GetRelationValue()
{
 if (FDelegate)
  return FDelegate->Value;
 else
  return relation();
}
//---------------------------------------------------------------------------
void __fastcall TRelationMagic::SetRelationValue(const relation & pValue)
{
 if (FDelegate)
  FDelegate->Value = pValue;
}
//---------------------------------------------------------------------------
relation __fastcall TRelationMagic::GetRelationSelection()
{
 if (FDelegate)
  return FDelegate->Selection;
 else
  return relation();
}
//---------------------------------------------------------------------------
void __fastcall TRelationMagic::SetRelationSelection(const relation & pValue)
{
 if (FDelegate)
  FDelegate->Selection = pValue;
}
//---------------------------------------------------------------------------
relation __fastcall TRelationMagic::GetRelationHighlight()
{
 if (FDelegate)
  return FDelegate->Highlight;
 else
  return relation();
}
//---------------------------------------------------------------------------
void __fastcall TRelationMagic::SetRelationHighlight(const relation & pValue)
{
 if (FDelegate)
  FDelegate->Highlight = pValue;
}
//---------------------------------------------------------------------------
bool __fastcall TRelationMagic::GetAllowSelection()
{
 if (FDelegate)
  return FDelegate->AllowSelection;
 else
  return false;
}
//---------------------------------------------------------------------------
void __fastcall TRelationMagic::SetAllowSelection(bool pValue)
{
 if (FDelegate)
  FDelegate->AllowSelection = pValue;
}
//---------------------------------------------------------------------------
bool __fastcall TRelationMagic::GetAllowMultiSelection()
{
 if (FDelegate)
  return FDelegate->AllowMultiSelection;
 else
  return false;
}
//---------------------------------------------------------------------------
void __fastcall TRelationMagic::SetAllowMultiSelection(bool pValue)
{
 if (FDelegate)
  FDelegate->AllowMultiSelection = pValue;
}
//---------------------------------------------------------------------------
bool __fastcall TRelationMagic::GetAllowMultiHighlight()
{
 if (FDelegate)
  return FDelegate->AllowMultiHighlight;
 else
  return false;
}
//---------------------------------------------------------------------------
void __fastcall TRelationMagic::SetAllowMultiHighlight(bool pValue)
{
 if (FDelegate)
  FDelegate->AllowMultiHighlight = pValue;
}
//---------------------------------------------------------------------------
TNotifyEvent __fastcall TRelationMagic::GetOnValueChange()
{
 if (FDelegate)
  return FDelegate->OnValueChange;
 else
  return NULL;
}
//---------------------------------------------------------------------------
void __fastcall TRelationMagic::SetOnValueChange(TNotifyEvent pValue)
{
 if (FDelegate)
  FDelegate->OnValueChange = pValue;
}
//---------------------------------------------------------------------------
TNotifyEvent __fastcall TRelationMagic::GetOnSelectionChange()
{
 if (FDelegate)
  return FDelegate->OnSelectionChange;
 else
  return NULL;
}
//---------------------------------------------------------------------------
void __fastcall TRelationMagic::SetOnSelectionChange(TNotifyEvent pValue)
{
 if (FDelegate)
  FDelegate->OnSelectionChange = pValue;
}
//---------------------------------------------------------------------------
TNotifyEvent __fastcall TRelationMagic::GetOnHighlightChange()
{
 if (FDelegate)
  return FDelegate->OnHighlightChange;
 else
  return NULL;
}
//---------------------------------------------------------------------------
void __fastcall TRelationMagic::SetOnHighlightChange(TNotifyEvent pValue)
{
 if (FDelegate)
  FDelegate->OnHighlightChange = pValue;
}
//---------------------------------------------------------------------------
bool __fastcall TRelationMagic::Valid(VGValidationMethods Method)
{
 if (FDelegate)
  return FDelegate->Valid(Method);
 else
  return true;
}
//---------------------------------------------------------------------------
void __fastcall TRelationMagic::Paint()
{
 if (!FDelegate)
 {
  Canvas->Pen->Color = clGray;
  Canvas->Pen->Style = psDot;
  Canvas->Pen->Mode  = pmMask;
  Canvas->MoveTo(0,0);
  Canvas->LineTo(Width - 1, 0);
  Canvas->LineTo(Width - 1, Height - 1);
  Canvas->LineTo(0, Height - 1);
  Canvas->LineTo(0, 0);
 }
}
//---------------------------------------------------------------------------
void __fastcall TRelationMagic::DefaultSwitchClick(TObject * Sender)
{
 if (Sender == _menuSwitchNone)
  DelegateType = dtNone;
 if (Sender == _menuSwitchTabular)
  DelegateType = dtTabular;
 if (Sender == _menuSwitchEventTimeline)
  DelegateType = dtEventTimeline;
 if (Sender == _menuSwitchObservationTimeline)
  DelegateType = dtObservationTimeline;
}
//---------------------------------------------------------------------------
void __fastcall TRelationMagic::DelegateDefaultPopup(TObject * Sender)
{
 // do our own stuff
 // _menuSwitchTabular->Enabled = false;
 // ...

 // then call original one
 if (FDelegate && FDelegatePopup)
  FDelegatePopup(Sender);
}
//---------------------------------------------------------------------------
void __fastcall TRelationMagic::DefaultPopup(TObject * Sender)
{
 // just in case our own appears for some reason?
 // _menuSwitchTabular->Enabled = false;

 // _menuFilter->Visible = true;
 // _menuModify->Visible = true;
 // _menuExport->Visible = true;
 _menuInsert->Visible = false;
 _menuUpdate->Visible = false;
}
//---------------------------------------------------------------------------
