//---------------------------------------------------------------------------

#include <vcl.h>
#pragma hdrstop

#include "codeRubberContainer.h"
#include <vector>
using namespace std;

#pragma package(smart_init)

//---------------------------------------------------------------------------
// ValidCtrCheck is used to assure that the components created do not have
// any pure virtual functions.
//

static inline void ValidCtrCheck(TRubberContainer *)
{
        new TRubberContainer(NULL);
}
//---------------------------------------------------------------------------
__fastcall TRubberContainer::TRubberContainer(TComponent* Owner)
        : TCustomControl(Owner)
{
 ControlStyle << csAcceptsControls << csReplicatable;
 Width  = 193;
 Height = 57;

 first_paint = false;
 FPoppedOut = false;
 FCaptured  = false;
 FHidden    = false;
 FRightMargin  = 2;
 FBottomMargin = 2;

 DoubleBuffered = false;

 _FORM_TRY
  FGripHolder = new TPanel(this);
 _FORM_CATCH
 FGripHolder->Parent  = this;
 FGripHolder->Width   = 13;
 FGripHolder->Height  = 13;
 FGripHolder->Visible = true;
 FGripHolder->Anchors = FGripHolder->Anchors << akRight << akBottom >> akLeft >> akTop;
 FGripHolder->Left    = (Width - FGripHolder->Width) - FRightMargin;
 FGripHolder->Top     = (Height - FGripHolder->Width) - FBottomMargin;

 _FORM_TRY
  FGrip = new TPaintBox(this);
 _FORM_CATCH
 FGrip->Parent  = FGripHolder;
 FGrip->Width   = 13;
 FGrip->Height  = 13;
 FGrip->Visible = true;
 FGrip->Left    = 0;
 FGrip->Top     = 0;
 FGrip->Cursor      = crSizeNWSE;
 FGrip->OnMouseDown = this->GripMouseDown;
 FGrip->OnPaint     = this->GripPaint;

 // double-buffering didn't help with tearing, btw
 _FORM_TRY
  FRubberBand = new TForm(this);
 _FORM_CATCH
 FRubberBand->Width        = Width;
 FRubberBand->Height       = Height;
 FRubberBand->BorderStyle  = bsNone;
 FRubberBand->Color        = clBlack;
 FRubberBand->BorderWidth  = 1;
 FRubberBand->OnDeactivate = this->RubberExit;

 FRubberBack = NULL;

 OnEnter = this->RubberEnter;
}
//---------------------------------------------------------------------------
namespace Coderubbercontainer
{
        void __fastcall PACKAGE Register()
        {
                 TComponentClass classes[1] = {__classid(TRubberContainer)};
                 RegisterComponents("prometheus", classes, 0);
        }
}
//---------------------------------------------------------------------------
void __fastcall TRubberContainer::PopOut()
{
 if (!FPoppedOut)
 {
  FPoppedOut = true;

  TPoint b;
  b.x = 0; // this->Left;
  b.y = 0; // this->Top;
  b = this->ClientToScreen(b);
  FRubberBand->Left    = b.x - 1;
  FRubberBand->Top     = b.y - 1;
  FRubberBand->Width   = this->Width;
  FRubberBand->Height  = this->Height;
  FRubberBand->Constraints->MinWidth = this->Width + 2;
  FRubberBand->Constraints->MinHeight = this->Height + 2;

  while (ControlCount)
   Controls[0]->Parent = FRubberBand;

  // a background would be nice, neh?
  _FORM_TRY
   FRubberBack = new TPanel(this);
  _FORM_CATCH
  FRubberBack->Parent     = FRubberBand;
  FRubberBack->Left       = 0;
  FRubberBack->Top        = 0;
  FRubberBack->Width      = FRubberBand->ClientWidth;
  FRubberBack->Height     = FRubberBand->ClientHeight;
  FRubberBack->Caption    = "";
  FRubberBack->BevelOuter = bvNone;
  FRubberBack->Color      = this->Color;
  FRubberBack->Anchors    = FRubberBack->Anchors << akRight << akBottom << akLeft << akTop;
  FRubberBack->Visible    = true;
  FRubberBack->SendToBack();

  FRubberBand->Visible = true;
  ClipGrip();
  FRubberBand->SetFocus();

  if (FOnPopOut)
   FOnPopOut(this);
 }
}
//---------------------------------------------------------------------------
void __fastcall TRubberContainer::PopIn()
{
 if (FPoppedOut)
 {
  FPoppedOut = false;

  FRubberBand->Width  = this->Width + 2;
  FRubberBand->Height = this->Height + 2;

  // do this ahead of time so it doesn't get transferred back
  delete FRubberBack;
  FRubberBack = NULL;

  while (FRubberBand->ControlCount)
   FRubberBand->Controls[0]->Parent = this;

  // BottomMargin = BottomMargin;
  // RightMargin  = RightMargin;
  ClipGrip();
  FRubberBand->Visible = false;

  if (FOnPopIn)
   FOnPopIn(this);
 }
}
//---------------------------------------------------------------------------
void __fastcall TRubberContainer::ClipGrip()
{
 try
 {
  vector<POINT> points;
  long l = 0;
  long r = 13;
  long t = 0;
  long b = 13;
  points.push_back(_point(l, b));
  points.push_back(_point(r, b));
  points.push_back(_point(r, t));

  HRGN region_handle;
  region_handle = CreatePolygonRgn(&points[0], points.size(), WINDING);
  if (region_handle)
   if (SetWindowRgn(FGripHolder->Handle, region_handle, true))
    ; // success
   else
    ; // error
  else
   ; // error
 } catch (...)
 {
 }
}
//---------------------------------------------------------------------------
void __fastcall TRubberContainer::Paint()
{
 // just once, thank you very much
 if (!first_paint)
 {
  first_paint = true;
  // okay, I'm not actually sure why this is giving me problems.
  Hidden = FHidden;
  RightMargin = FRightMargin;
  BottomMargin = FBottomMargin;
  ClipGrip();
 }

 TRect rectAll;
 rectAll.Top = 0;
 rectAll.Bottom = Height;
 rectAll.Left = 0;
 rectAll.Right = Width;

 Canvas->Brush->Color = Color;
 Canvas->FillRect(rectAll);
}
//---------------------------------------------------------------------------
void __fastcall TRubberContainer::GripMouseDown (TObject* Sender, TMouseButton Button, TShiftState Shift, int X, int Y)
{
 if (!FCaptured)
 {
  sizegrip_x = X;
  sizegrip_y = Y;

  if (!FPoppedOut)
   PopOut();

//  FCaptured = true;
//  SetCaptureControl(FGrip);
// solution accidentally found when prettying up specialGeneric's sizegrip (making it 'go' again), used here. and it works!
   ReleaseCapture();
   FRubberBand->Perform(WM_SYSCOMMAND, SC_SIZE + 8, 0);
 }
}
//---------------------------------------------------------------------------
void __fastcall TRubberContainer::GripPaint (TObject* Sender)
{
 // try to paint the sizegrip
 RECT bob;
 bob.right  = FGrip->Width; // - 4;
 bob.bottom = FGrip->Height; // - 4;
 bob.left   = bob.right - 14; // it's 13, but right/bottom edges don't count.
 bob.top    = bob.bottom - 14;
 DrawFrameControl(FGrip->Canvas->Handle, &bob, DFC_SCROLL, DFCS_SCROLLSIZEGRIP);
}
//---------------------------------------------------------------------------
void __fastcall TRubberContainer::RubberEnter (TObject* Sender)
{
 FGripHolder->BringToFront();
 // pre-empt! (try to make it look the same?)
 // PopOut();
}
//---------------------------------------------------------------------------
void __fastcall TRubberContainer::RubberExit  (TObject* Sender)
{
 PopIn();
 // FGripHolder->SendToBack();
}
//---------------------------------------------------------------------------
void __fastcall TRubberContainer::SetHidden (bool Value)
{
 FHidden = Value;
 FGripHolder->Visible = !FHidden;
 if (FGripHolder->Visible)
  FGripHolder->BringToFront();
}
//---------------------------------------------------------------------------
void __fastcall TRubberContainer::SetRightMargin (int Value)
{
 FRightMargin = Value;
 FGripHolder->Left = (Width - FGripHolder->Width) - FRightMargin;
}
//---------------------------------------------------------------------------
void __fastcall TRubberContainer::SetBottomMargin (int Value)
{
 FBottomMargin = Value;
 FGripHolder->Top = (Height - FGripHolder->Height) - FBottomMargin;
}
//---------------------------------------------------------------------------
void __fastcall TRubberContainer::SetPoppedOut (bool Value)
{
 if (Value && !PoppedOut)
  PopOut();
 else if (!Value && PoppedOut)
  PopIn();
}
//---------------------------------------------------------------------------

