#include "filter_export.h"
#include <fstream>

//-column_description--------------------------------------------------------
column_description::column_description (string o_name, string db_name, bool b_show, bool b_sort, bool b_asc, show_type show_m, sort_type sort_m, int show_o, int sort_o, int db_o)
{
 official_name  = o_name;
 database_name  = db_name;
 is_show        = b_show;
 is_sort        = b_sort;
 is_ascending   = b_asc;
 show_method    = show_m;
 sort_method    = sort_m;
 show_order     = show_o;
 sort_order     = sort_o;
 database_order = db_o;
}

//-column_sort_functor-------------------------------------------------------
column_sort_functor::column_sort_functor (column_sort_criteria p_sort_criterium)
{
 sort_criterium = p_sort_criterium;
}
//---------------------------------------------------------------------------
bool column_sort_functor::operator () (const column_description * a, const column_description * b) // returns (a < b) according to sorting rules
{
 switch (sort_criterium)
 {
  case sort_by_show_order:
   return (a->show_order < b->show_order);
  case sort_by_sort_order:
   return (a->sort_order < b->sort_order);
  case sort_by_database_order:
   return (a->database_order < b->database_order);
  default:
   return false;
 }
}

//-data_sort_functor---------------------------------------------------------
data_sort_functor::data_sort_functor (vector<column_description *> & p_descriptions)
{
 sort_criterium = p_descriptions;
}
//---------------------------------------------------------------------------
bool data_sort_functor::operator () (const vector<string> & a, const vector<string> & b) // returns (a < b) according to sorting rules
{
 int compare_value;
 for (vector<column_description *>::iterator i_col_desc = sort_criterium.begin(); i_col_desc != sort_criterium.end(); i_col_desc++)
 {
  compare_value = CompareByType(a[(*i_col_desc)->result_order], b[(*i_col_desc)->result_order], (*i_col_desc)->sort_method);
  compare_value *= ((*i_col_desc)->is_ascending) ? 1 : -1;

  if (compare_value < 0)
   return true;
  else if (compare_value > 0)
   return false;
 }
 return false;
}

//-column_descriptions-------------------------------------------------------
column_descriptions::column_descriptions (classFilterObject * p_filter)
{
 data_source = p_filter;
 data_filter_column = -1;
}
//---------------------------------------------------------------------------
void column_descriptions::ImportSettings (classFilterObject * p_filter)
{
 string permission_name;
 int i = 0;
 for (vector<classColumn>::iterator i_col = p_filter->ColumnVec.begin(); i_col != p_filter->ColumnVec.end(); i_col++, i++)
 {
  permission_name = p_filter->FullPerm(i_col->requires);
  if ((permission_name == "") ||
      (optionsPermissions[permission_name][LikeString("table.%", permission_name) ? "view" : "enabled"] == "true"))
  {
   // not bothering to try to figure out if the column is currently sorted or not ...
   // if filter could (and maybe will someday) sort by multiple columns, this would be more useful.
   AddColumn(column_description(i_col->columnName, i_col->columnName, i_col->showCol, false, true, i_col->StringType, i_col->as_what, i, -1, i));
  }
 }
 data_filter_column = p_filter->GetIconColumnIndex();
 return;
}
//---------------------------------------------------------------------------
void column_descriptions::AddColumn      (column_description p_col)
{
 descriptions.push_back (p_col);
 return;
}
//---------------------------------------------------------------------------
bool column_descriptions::GetAllColumns  (vector<column_description *> & p_return)
{
 p_return.clear();
 for (list<column_description>::iterator i_col_desc = descriptions.begin(); i_col_desc != descriptions.end(); i_col_desc++)
  p_return.push_back(&(*i_col_desc));

 if (p_return.empty())
  return false;

 sort (p_return.begin(), p_return.end(), column_sort_functor(sort_by_database_order));
 return true;
}
//---------------------------------------------------------------------------
bool column_descriptions::GetShowColumns (vector<column_description *> & p_return)
{
 p_return.clear();
 for (list<column_description>::iterator i_col_desc = descriptions.begin(); i_col_desc != descriptions.end(); i_col_desc++)
  if (i_col_desc->is_show)
   p_return.push_back(&(*i_col_desc));

 if (p_return.empty())
  return false;

 sort (p_return.begin(), p_return.end(), column_sort_functor(sort_by_show_order));
 return true;
}
//---------------------------------------------------------------------------
bool column_descriptions::GetSortColumns (vector<column_description *> & p_return)
{
 p_return.clear();
 for (list<column_description>::iterator i_col_desc = descriptions.begin(); i_col_desc != descriptions.end(); i_col_desc++)
  if (i_col_desc->is_sort && i_col_desc->is_show)
   p_return.push_back(&(*i_col_desc));

 if (p_return.empty())
  return false;

 sort (p_return.begin(), p_return.end(), column_sort_functor(sort_by_sort_order));
 return true;
}
//---------------------------------------------------------------------------
bool column_descriptions::GetHeaders     (vector<string> & p_return)
{
 p_return.clear();
 vector<column_description *> column_order;
 if (GetShowColumns(column_order))
 {
  for (vector<column_description *>::iterator i_col_desc = column_order.begin(); i_col_desc != column_order.end(); i_col_desc++)
   p_return.push_back((*i_col_desc)->official_name);
  return true;
 } else
  return false;
}
//---------------------------------------------------------------------------
bool column_descriptions::GetData        (vector<vector<string> > & p_return, vector<int> & p_max_column_sizes, bool p_size_headers, vector<string> & p_valid_current_id)
{
 p_return.clear();
 p_max_column_sizes.clear();
 vector<column_description *> sort_order;
 vector<column_description *> show_order;
 vector<string> temp_row;
 bool valid_row;
 string row_current_id;

 if (data_source)
 {
  if (GetShowColumns(show_order))
  {
   // result order matters in this case, so we set it
   int i = 0;
   p_max_column_sizes.resize(show_order.size(), 0);
   for (vector<column_description *>::iterator i_col_desc = show_order.begin(); i_col_desc != show_order.end(); i_col_desc++, i++)
   {
    (*i_col_desc)->result_order = i;
    if (p_size_headers)
     p_max_column_sizes[i] = max(p_max_column_sizes[i], (int)((*i_col_desc)->official_name.length()));
    else
     p_max_column_sizes[i] = 0;
   }

   for (deque<vector<string> >::const_iterator z = data_source->GetRawData().begin(); z != data_source->GetRawData().end(); z++)
   {
    temp_row.clear();
    valid_row = false;

    if (data_filter_column == -1)
    {
     valid_row = true;
    } else
    {
     try
     {
      row_current_id = (*z)[data_filter_column];
     } catch (...)
     {
      PSay("Error while trying to retrieve Fields[data_filter_column]", m_program_error, MB_OK, "filter_export");
      return false;
     }
     for (vector<string>::iterator i_val = p_valid_current_id.begin(); i_val != p_valid_current_id.end(); i_val++)
     {
      if ((*i_val) == row_current_id)
      {
       valid_row = true;
       break;
      }
     }
    }

    if (valid_row)
    {
     for (vector<column_description *>::iterator i_col_desc = show_order.begin(); i_col_desc != show_order.end(); i_col_desc++)
     {
      try
      {
       temp_row.push_back(FormatString((*z)[(*i_col_desc)->database_order], (*i_col_desc)->show_method));
       p_max_column_sizes[(*i_col_desc)->result_order] = max (p_max_column_sizes[(*i_col_desc)->result_order],(int)(temp_row[(*i_col_desc)->result_order].length()));
      } catch (...)
      {
       PSay("Error while trying to retrieve Fields[database_order]", m_program_error, MB_OK, "filter_export");
       return false;
      }
     }
     p_return.push_back(temp_row);
    }
   }

   // we sort fields that are both visible and to be sorted, in order of importance
   if (GetSortColumns(sort_order))
    sort (p_return.begin(), p_return.end(), data_sort_functor(sort_order));

   return true;
  } else
   return false;
 } else
  return false;
}
//---------------------------------------------------------------------------
bool column_descriptions::ExportData     (string p_filename, export_types p_export_type, bool p_column_headers, vector<string> & p_valid_current_id)
{
 vector<vector<string> > exported_data;
 vector<int>             max_col_sizes;
 vector<string>          header_data;
 ofstream export_file;
 export_file.open(p_filename.c_str(), ios::trunc);
 if (export_file)
 {
  if (GetData(exported_data, max_col_sizes, p_column_headers, p_valid_current_id))
  {
   if (p_column_headers)
   {
    if (GetHeaders(header_data))
    {
     export_file << FormatForExport (header_data, max_col_sizes, p_export_type, '.') << endl;
    } else
    {
     export_file.close();
     return false;
    }
   }

   for (vector<vector<string> >::iterator i_row = exported_data.begin(); i_row != exported_data.end(); i_row++)
    export_file << FormatForExport ((*i_row), max_col_sizes, p_export_type, ' ') << endl;

   export_file.close();
   return true;
  } else
  {
   export_file.close();
   return false;
  }
 } else
  return false;
}
//---------------------------------------------------------------------------
string column_descriptions::FormatForExport (vector<string> & p_row_data, vector<int> & p_max_column_sizes, export_types p_export_type, char fill_right)
{
 string answer;
 string temp;
 int i;

 switch (p_export_type)
 {
  case export_csv:
  {
   for (vector<string>::iterator i_string = p_row_data.begin(); i_string != p_row_data.end(); i_string++)
   {
    if (i_string != p_row_data.begin())
     answer += ",";
    answer += "\"" + ReplaceStringWithString(ReplaceAnyCharWithString((*i_string), "\a\f\n\r\t", "?"), "\"", "\"\"") + "\"";
   }
   break;
  }
  case export_txt:
  {
   i = 0;
   for (vector<string>::iterator i_string = p_row_data.begin(); i_string != p_row_data.end(); i_string++, i++)
   {
    if (i_string != p_row_data.begin())
     answer += " ";
    temp = ReplaceAnyCharWithString((*i_string), "\a\f\n\r\t", "?");
    temp.resize(p_max_column_sizes[i], fill_right);
    answer += temp;
   }
   break;
  }
  default:
  {
   answer = "unknown export type";
  }
 }
 return answer;
}
//---------------------------------------------------------------------------

