#ifndef filter_export_h
#define filter_export_h

#include "prometheus_util.h" // well so much for modularity.  I don't much care though; I never use
                             // filter_export for anything.  but FYI:  this ought to be made independent of
                             // Prometheus at some point; a map<string, map<string, string>* member can be
                             // created and assigned the value of optionsIni in codePrometheus similar to
                             // what it does for filter and #ifdef stuff can switch PSay for Application->
                             // MessageBox like is done in lookuptables
#include "Filter.h"

// enums
enum column_sort_criteria {sort_by_show_order, sort_by_sort_order, sort_by_database_order};
enum export_types {export_csv, export_txt /*, export_xml */ };

// declarations
class column_description
{
 public:
  string official_name;
  string database_name;
  bool   is_show;
  bool   is_sort;
  bool   is_ascending;
  show_type show_method;
  sort_type sort_method;
  int    show_order;
  int    sort_order;
  int    database_order;
  
  int    result_order;
 public:
  column_description (string o_name, string db_name, bool b_show, bool b_sort, bool b_asc, show_type show_m, sort_type sort_m, int show_o, int sort_o, int db_o);
};

class column_sort_functor
{
 private:
  column_sort_criteria sort_criterium;
 public:
  column_sort_functor (column_sort_criteria p_sort_criterium);
  bool operator () (const column_description * a, const column_description * b); // returns (a < b) according to sorting rules
};

class data_sort_functor
{
 private:
  vector<column_description *> sort_criterium;
 public:
  data_sort_functor (vector<column_description *> & p_descriptions);
  bool operator () (const vector<string> & a, const vector<string> & b); // returns (a < b) according to sorting rules
};

class column_descriptions
{
 private:
  list<column_description> descriptions;
  classFilterObject*       data_source;
  int                      data_filter_column;

 public:
  column_descriptions (classFilterObject * p_filter);
  void ImportSettings (classFilterObject * p_filter);
  void AddColumn      (column_description p_col);
  bool GetAllColumns  (vector<column_description *> & p_return);
  bool GetShowColumns (vector<column_description *> & p_return);
  bool GetSortColumns (vector<column_description *> & p_return);
  bool GetHeaders     (vector<string> & p_return);
  bool GetData        (vector<vector<string> > & p_return, vector<int> & p_max_column_sizes, bool p_size_headers, vector<string> & p_valid_current_id);
  bool ExportData     (string p_filename, export_types p_export_type, bool p_column_headers, vector<string> & p_valid_current_id);
  string FormatForExport (vector<string> & p_row_data, vector<int> & p_max_column_sizes, export_types p_export_type, char fill_right);
};

#endif
 