#ifndef financial_h
#define financial_h
#include <math.h>

#define f_delta_value 0.005L

class financial
{
 private:
  long double actual_value;

 public:
  financial ()                                {actual_value = 0.00;}
  financial (const long double initial_value) {actual_value = initial_value;}

  long double AsLongDouble () const           {return actual_value;}

  // friends
  friend financial operator + (const financial & lhs, const financial & rhs);
  friend financial operator - (const financial & lhs, const financial & rhs);
  friend financial operator * (const financial & lhs, const financial & rhs);
  friend financial operator / (const financial & lhs, const financial & rhs);
  friend financial operator - (const financial & una);

  // non-const math
  financial & operator += (const financial & rhs) {actual_value += rhs.actual_value; return *this;}
  financial & operator -= (const financial & rhs) {actual_value -= rhs.actual_value; return *this;}
  financial & operator *= (const financial & rhs) {actual_value *= rhs.actual_value; return *this;}
  financial & operator /= (const financial & rhs) {actual_value /= rhs.actual_value; return *this;}

  // relational
  bool operator == (const financial & rhs) const {return fabsl(rhs.actual_value - actual_value) <  f_delta_value;}
  bool operator != (const financial & rhs) const {return fabsl(rhs.actual_value - actual_value) >= f_delta_value;}
  bool operator  < (const financial & rhs) const {return ((*this <= rhs) && (*this != rhs));}
  bool operator  > (const financial & rhs) const {return ((*this >= rhs) && (*this != rhs));}
  bool operator <= (const financial & rhs) const {return actual_value <= rhs.actual_value;}
  bool operator >= (const financial & rhs) const {return actual_value >= rhs.actual_value;}
};

#endif