//PHOBOS
#include "lookUpTableClass.h"
#include "sqlLib.h"
// TODO : UnsafeAddValue doesn't currently let you use custom SQL to insert a value
//TODO : GetValueList returns the default id,item,valid for the table; should probably add a way of getting value list from boxes created via custom SQL
//TODO : Id/Value Of functions return results based on id,item,valid for the table; this could be different from id/value pairs if using custom SQL

classLookUpTable::classLookUpTable (TIBDatabase * dbDatabase, TIBTransaction * transaction, string table,
                                    bool change, int display, int hide, TPopupMenu * popup, int autoload)
{
  dbTransaction   = transaction;
  tableName       = table;
  canChange       = change;
  display_option  = display;
  hide_option     = hide;
  autoload_option = autoload;

  isFine       = true;
  errorMessage = "";
  dbDataSet    = new TIBDataSet(NULL);
  dbSQL        = new TIBSQL (NULL);
  itemList     = new TStringList;
  lookup_popup = popup;

  try
  {
    dbDataSet->Database       = dbDatabase;
    dbDataSet->Transaction    = dbTransaction;
    dbDataSet->UniDirectional = false;
    dbSQL->Transaction = dbTransaction;
    dbSQL->Database    = dbDatabase;
    // RefreshAllBoxes();
    is_pending = true;
  }
  catch (EIBError &error)
  {
    isFine = false;
    errorMessage = error.Message.c_str();
  }
}
//--------------------------------------------------------------------------
// jhaley 20111010: cleanup macros for debug messaging - there's a point where debug code gets in the way of the
// ability to read and understand code, but here's how I deal with it. The same approach was employed in the Eternity
// Engine's zone heap, which contains extensive conditionally-enabled debugging capabilities.
#ifdef use_tcpip_messaging
  #define MSG_CHECKPOINT(func, checkpt) \
    if(debug_level[libraries] >= 9 && tcpip_message_router) \
      tcpip_message_router->SendMessageToAll("debug", func " checkpoint " checkpt)
#else
  #define MSG_CHECKPOINT(func, checkpt)
#endif
//--------------------------------------------------------------------------
void classLookUpTable::ExpectRefresh ()
{
 // the table needs to be updated, but we'll leave off if there aren't any
 // registered combo boxes at the moment. if one of them registers, or someone
 // asks for the value, -then- we'll go grab the value. if any boxes are registerd,
 // it'll see it needs to download, do so, and update the boxes.
 is_pending = true;
 if (boxes.size() > 0)
  RefreshAllBoxes();
}
//--------------------------------------------------------------------------
classLookUpTable::~classLookUpTable ()
{
  for (vector<lookuptable_box>::iterator itr = boxes.begin(); itr != boxes.end(); itr++)
    if (itr->item_list && itr->item_list != itemList)
      delete itr->item_list;
  boxes.clear();

  // delete dbTransaction;
  dbTransaction = NULL;

  if(dbDataSet)
    delete dbDataSet;
  dbDataSet = NULL;    // jhaley 20111010 - nullify all after deletion
  if(dbSQL)
    delete dbSQL;
  dbSQL = NULL;
  if(itemList)
    delete itemList;
  itemList = NULL;
}
//--------------------------------------------------------------------------
bool classLookUpTable::IsFine ()
{
  if (isFine == true)
    return true;
  else
    return false;
}
//--------------------------------------------------------------------------
string classLookUpTable::ErrorMessage ()
{
  return errorMessage;
}
//--------------------------------------------------------------------------
void classLookUpTable::RegisterComboBox(TComboBox * box, string sql, bool (*follow_up)(TIBTransaction *, const string &, const string &))
{
  // TODO: debug?
  box->Sorted = false; // just in case?

  #ifdef use_tcpip_messaging
  if(debug_level[libraries]>=2 && tcpip_message_router)
    tcpip_message_router->SendMessageToAll("debug", ("Registering combo box for table '"+tableName+"' with custom sql '"+sql+"'").c_str());
  #endif
  if (is_pending)
   RefreshDatasets();

  #ifdef use_tcpip_messaging
  if(debug_level[libraries]>=1 && tcpip_message_router)
  {
    for (vector<lookuptable_box>::iterator itr = boxes.begin(); itr != boxes.end(); itr++)
    {
      try
      {
        if (!itr->combo_box)
          tcpip_message_router->SendMessageToAll("debug", "lookUpTableClass::RegisterComboBox TComboBox* (combo_box) is null!");
        else if(itr->combo_box->ClassType() != __classid(TComboBox))
          tcpip_message_router->SendMessageToAll("debug", "lookUpTableClass::RegisterComboBox TComboBox* (combo_box) is not null but doesn't seem to point to a valid combo box!");
        else if (!itr->combo_box->Items)
          tcpip_message_router->SendMessageToAll("debug", "lookUpTableClass::RegisterComboBox TStringList* (Items) is null!");
        else if (itr->combo_box == box)
          tcpip_message_router->SendMessageToAll("debug", "lookUpTableClass::RegisterComboBox Registering a combo box that is already registered!");

        if (!itr->item_list)
          tcpip_message_router->SendMessageToAll("debug", "lookUpTableClass::RegisterComboBox TStrings* (item_list) is null!");
      }
      // the show must go on
      catch(EAccessViolation &av) { }
      catch (...) { }
    }
  }
  #endif

  if (sql == "")
  {
    MSG_CHECKPOINT("classLookUpTable::RegisterComboBox", "1");
    boxes.push_back(lookuptable_box(box, itemList));
  }
  else
  {
    MSG_CHECKPOINT("classLookUpTable::RegisterComboBox", "2");
    TStringList * string_list = new TStringList();
    boxes.push_back(lookuptable_box(box, string_list, sql, follow_up));
    RefreshDataset(/*boxes.back().item_list*/ string_list, sql);
    MSG_CHECKPOINT("classLookUpTable::RegisterComboBox", "3");
  }
  // TODO: debug?
  box->Sorted = (sql == ""); // true;
  box->DropDownCount = 15;
  if(! box->OnKeyPress)
    box->OnKeyPress = OnComboKeyPress;
  box->Items = boxes.back().item_list;

  box->PopupMenu = lookup_popup;
  MSG_CHECKPOINT("classLookUpTable::RegisterComboBox", "4");
}
//--------------------------------------------------------------------------
void classLookUpTable::UnregisterComboBox(TComboBox * box)
{
  #ifdef use_tcpip_messaging
  if(debug_level[libraries]>=2 && tcpip_message_router)
    tcpip_message_router->SendMessageToAll("debug", "Unregistering combo box for table '" + tableName + "'");
  #endif
  vector<lookuptable_box>::iterator itr = boxes.begin();
  while (itr != boxes.end())
  {
    if (itr->combo_box == box)
    {
       box->Items->Clear();
       itr = boxes.erase(itr);
       // return; // we found it... we don't -really- need to go on... but just in case it got registered twice or something?  Hmm.
    }
    else
       itr++;
  }
}
//--------------------------------------------------------------------------
bool classLookUpTable::IsInLookUpTable(string enteredString, TComboBox * box)
{
  if (is_pending)
   RefreshDatasets();

  enteredString = UppercaseString(enteredString);
  if (box)
  {
    for (unsigned int i=0; i<boxes.size(); i++)
      if (boxes[i].combo_box == box)
        for (int j=0; j<boxes[i].item_list->Count; j++)
          if (UppercaseString(boxes[i].item_list->Strings[j].c_str()) == enteredString)
            return true;
  }
  else
  {
    for (vector<record>::iterator itr = table_vec.begin(); itr != table_vec.end(); itr++)
    {
      if (UppercaseString(itr->item) == enteredString)
        return true;
    }
  }
  return false;
}
//--------------------------------------------------------------------------
void classLookUpTable::RefreshAllBoxes()
{
  if (is_pending)
   RefreshDatasets();

  for (vector<lookuptable_box>::iterator itr = boxes.begin(); itr != boxes.end(); itr++)
   itr->combo_box->Items = itr->item_list;
}
//--------------------------------------------------------------------------
string classLookUpTable::IdOf(string value, TComboBox * box)
{
  if (is_pending)
   RefreshDatasets();

  value = UppercaseString(value);
  if (box)
  {
    for (unsigned int i=0; i<boxes.size(); i++)
      if (boxes[i].combo_box == box)
        for (int j=0; j<boxes[i].item_list->Count; j++)
          if (UppercaseString(boxes[i].item_list->Strings[j].c_str()) == value)
            return IntToString((int)(boxes[i].item_list->Objects[j]));
  }
  else
  {
    for (vector<record>::iterator itr = table_vec.begin(); itr != table_vec.end(); itr++)
    {
      if (UppercaseString(itr->item) == value)
        return itr->id;
    }
  }
  return "";
}
//--------------------------------------------------------------------------
string classLookUpTable::ValueOf(string id, TComboBox * box)
{
  #ifdef use_tcpip_messaging
  if(debug_level[libraries]>=9 && tcpip_message_router)
    tcpip_message_router->SendMessageToAll("debug", "lookupTableClass::ValueOf() entry for " + tableName);
  #endif
  if (is_pending)
   RefreshDatasets();

  string to_return = "";
  if (box)
  {
    MSG_CHECKPOINT("classLookUpTable::ValueOf", "1a");
    for (unsigned int i=0; i<boxes.size(); i++)
    {
      MSG_CHECKPOINT("classLookUpTable::ValueOf", "2a");
      if (boxes[i].combo_box == box)
      {
        MSG_CHECKPOINT("classLookUpTable::ValueOf", "3a");
        for (int j=0; j<boxes[i].item_list->Count; j++)
        {
          MSG_CHECKPOINT("classLookUpTable::ValueOf", "4a");
          if (boxes[i].item_list->Objects[j])
          {
            MSG_CHECKPOINT("classLookUpTable::ValueOf", "5a");
            if ((int)(boxes[i].item_list->Objects[j]) == StringToInt(id))
            {
              #ifdef use_tcpip_messaging
              if(debug_level[libraries]>=9 && tcpip_message_router)
              {
                tcpip_message_router->SendMessageToAll("debug", "lookupTableClass::ValueOf() for " + tableName + ": "
                                                                "Match: '" + IntToString((int)(boxes[i].item_list->Objects[j])) + "'");
              }
              #endif
              to_return = boxes[i].item_list->Strings[j].c_str();
            }
          }
        }
      }
    }
  }
  else
  {
    MSG_CHECKPOINT("classLookUpTable::ValueOf", "1b");
    bool found_it = false;
    vector<record>::iterator itr = table_vec.begin();
    while (itr != table_vec.end() && !found_it)
    {
      MSG_CHECKPOINT("classLookUpTable::ValueOf", "2b");
      #ifdef use_tcpip_messaging
      if(debug_level[libraries]>=9 && tcpip_message_router)
        tcpip_message_router->SendMessageToAll("debug", "lookupTableClass::ValueOf() for " + tableName + ": '" + itr->id + "' =?= '" + id + "'");
      #endif
      if (itr->id == id)
      {
        MSG_CHECKPOINT("classLookUpTable::ValueOf", "3b");
        to_return = itr->item;
      }
      itr++;
    }
  }

  #ifdef use_tcpip_messaging
  if(debug_level[libraries]>=9 && tcpip_message_router)
    tcpip_message_router->SendMessageToAll("debug", "lookupTableClass::ValueOf() exit for " + tableName);
  #endif
  if(display_option == as_pretty)
    return PrettyString(to_return);
  else if (display_option == as_uppercase)
    return UppercaseString(to_return);
  else if (display_option == as_lowercase)
    return LowercaseString(to_return);
  else if (display_option == as_sentence)
    return SentenceString(to_return);
  else
    return to_return;
}
//--------------------------------------------------------------------------
bool classLookUpTable::IsValidId(string id, TComboBox * box)
{
  if (is_pending)
   RefreshDatasets();

  if (box)
  {
    for (unsigned int i=0; i<boxes.size(); i++)
      if (boxes[i].combo_box == box)
        for (int j=0; j<boxes[i].item_list->Count; j++)
          if (boxes[i].item_list->Objects[j])
            if ((int)(boxes[i].item_list->Objects[j]) == StringToInt(id))
              return true;
  }
  else
  {
    for (vector<record>::iterator itr = table_vec.begin(); itr != table_vec.end(); itr++)
    {
      if (itr->id == id)
      {
       return true;
      }
    }
  }
  return false;
}
//--------------------------------------------------------------------------
bool classLookUpTable::AddValue(string value, TComboBox *box)
{
  if (canChange)
  {
   return UnsafeAddValue(value, box);
  }
  else
    return false;
}
//--------------------------------------------------------------------------
bool classLookUpTable::UnsafeAddValue(string value, TComboBox *box)
{
  value = UppercaseString(value);
  value = ReplaceStringWithString(value, "'", "''");
  TIBTransaction * subTransaction = new TIBTransaction(NULL);
  TIBSQL * subSQL;
  subTransaction->DefaultDatabase = dbTransaction->DefaultDatabase;
  subTransaction->DefaultAction = TARollback;
  subTransaction->Params->Add("nowait");
  subTransaction->Active = true;
  subSQL = new TIBSQL (NULL);
  bool to_return;

  sqlstring = "insert into " + tableName + " (item, valid) values ('" + value + "', 1)";
  try
  {
    subSQL->Transaction = subTransaction;
    subSQL->Database = dbTransaction->DefaultDatabase;
    subSQL->SQL->Text = sqlstring.c_str();
    subSQL->ExecQuery();

    to_return = true;
    if (box)
    {
      for (unsigned int i=0; i<boxes.size(); i++)
        if (boxes[i].combo_box == box)
          if (boxes[i].follow_up)
            to_return = boxes[i].follow_up(subTransaction, tableName, GetOneField(subTransaction, "select max(id) from " + tableName));
    }
    subTransaction->Commit();
  }
  catch (EIBError &error)
  {
    isFine = false;
    errorMessage = error.Message.c_str();
    if (subTransaction->Active)
      subTransaction->Rollback();
    to_return = false;
  }
  delete subSQL;
  delete subTransaction;
  return to_return;
}
//--------------------------------------------------------------------------
bool classLookUpTable::CanAddToTable()
{
  return canChange;
}
//--------------------------------------------------------------------------
string classLookUpTable::TableName()
{
  return tableName;
}
//--------------------------------------------------------------------------
vector<string> classLookUpTable::GetValueList(TComboBox * box)
{
  if (is_pending)
   RefreshDatasets();

  vector<string> return_vec;
  if (box)
  {
    for (unsigned int i=0; i<boxes.size(); i++)
      if (boxes[i].combo_box == box)
        for (int j=0; j<boxes[i].item_list->Count; j++)
          return_vec.push_back(boxes[i].item_list->Strings[j].c_str());
  }
  else
  {
    for (int i = 0; i < itemList->Count; i++)
      return_vec.push_back(itemList->Strings[i].c_str());
  }
  return return_vec;
}
//--------------------------------------------------------------------------
bool classLookUpTable::BoxIsRegistered(TComboBox * box)
{
  for(vector<lookuptable_box>::iterator itr = boxes.begin(); itr != boxes.end(); itr++)
    if(itr->combo_box == box)
      return true;
  return false;
}
//--------------------------------------------------------------------------
void classLookUpTable::RefreshDatasets()
{
  is_pending = false;
  isFine = true;
  errorMessage = "";

  MSG_CHECKPOINT("classLookUpTable::RefreshDatasets", "1");
  #ifdef use_tcpip_messaging
  if(debug_level[libraries]>=2 && tcpip_message_router)
    tcpip_message_router->SendMessageToAll("debug", ("Refreshing datasets for table '" + tableName + "'").c_str());
  #endif
  // first refresh the default data set that most cbo boxes are probably using
  try
  {
    RefreshDataset(itemList, "");
    MSG_CHECKPOINT("classLookUpTable::RefreshDatasets", "2");
  }
  catch (EIBError &error)
  {
    MSG_CHECKPOINT("classLookUpTable::RefreshDatasets", "3");
    isFine = false;
    errorMessage = "Failure to download " + tableName + "\n" + string(error.Message.c_str());
    #ifdef use_prometheus_util
    // TODO -c FATAL : jhaley 20111010: potential infinite recursion - PSay -> ValueOf -> RefreshDatasets -> PSay -> ValueOf -> RefreshDatasets -> ...
    // TODO -c FATAL : jhaley 20111010: attempts to access database again, in response to an EIBError.
    PSay(errorMessage, m_database_error, MB_OK, "classLookUpTable");
    #else
    Application->MessageBox(errorMessage.c_str(), "Database error in lookUpTableClass::RefreshDatasets()", MB_OK);
    #endif
  }

  // if there are any cbo boxes running custom sql, update their data sets too
  MSG_CHECKPOINT("classLookUpTable::RefreshDatasets", "4");
  for(vector<lookuptable_box>::iterator itr = boxes.begin(); itr != boxes.end(); itr++)
  {
    MSG_CHECKPOINT("classLookUpTable::RefreshDatasets", "5");
    if (itr->sql != "")
    {
      MSG_CHECKPOINT("classLookUpTable::RefreshDatasets", "6");
      try
      {
        RefreshDataset(itr->item_list, itr->sql);
        MSG_CHECKPOINT("classLookUpTable::RefreshDatasets", "7");
      }
      catch (EIBError &error)
      {
        MSG_CHECKPOINT("classLookUpTable::RefreshDatasets", "8");
        isFine = false;
        errorMessage = error.Message.c_str();
        string extra_info = "\nwith custom sql:\n\n" + itr->sql;
        #ifdef use_prometheus_util
        // TODO -c FATAL : jhaley 20111010: potential infinite recursion - PSay -> ValueOf -> RefreshDatasets -> PSay -> ValueOf -> RefreshDatasets -> ...
        // TODO -c FATAL : jhaley 20111010: attempts to access database again, in response to an EIBError.
        PSay("Failure to download " + tableName + "\n" + errorMessage + extra_info, m_database_error, MB_OK, "classLookUpTable");
        #endif
      }
    }
  }
}
//--------------------------------------------------------------------------
void classLookUpTable::RefreshDataset(TStringList * p_item_list, string p_sql)
{
  // if we're using the default SQL (just grab id,item,valid for ALL records, no special processing,
  // but the option NOT to do this for this table is set, then... don't do it.
  // Aside:  autoload should be true in most cases; it says to go ahead and load the lookup table
  // straight from the database into memory, where it can be reused without having to query the DB
  // every time this lookup table's values are required.  This has always been the default functionality.
  // However, there are rare cases where we have a lookup table we don't want to load into memory as-is,
  // but still want the utility of the other handy functions this class offers; but, we'll -always- be
  // populating combo boxes with custom SQL.  If this is the case, we don't need to waste time and memory
  // auto-loading the entire contents of the table.  *takes deep breath*  See?  It makes sense.  I promise.  :-)
  if (p_sql == "" && !autoload_option)
  {
    #ifdef use_tcpip_messaging
    if(debug_level[libraries]>=3 && tcpip_message_router)
      tcpip_message_router->SendMessageToAll("debug", ("Skipping refreshing dataset for table '"+tableName+"' with custom sql '"+p_sql+"'").c_str());
    #endif
    return;
  }

  record temp_struct;

  MSG_CHECKPOINT("classLookUpTable::RefreshDataset", "1");
  #ifdef use_tcpip_messaging
  if(debug_level[libraries]>=2 && tcpip_message_router)
    tcpip_message_router->SendMessageToAll("debug", ("Refreshing dataset for table '"+tableName+"' with custom sql '"+p_sql+"'").c_str());
  #endif
  if (p_sql == "")
  {
    MSG_CHECKPOINT("classLookUpTable::RefreshDataset", "2");
    sqlstring = "select id, item, valid from ";
    sqlstring += tableName;
    sqlstring += " order by item";
    // only change global (to the lookuptable) table_vec if we're populating the "default" SQL list (not a custom SQL list)
    table_vec.clear();
  }
  else
  {
    MSG_CHECKPOINT("classLookUpTable::RefreshDataset", "3");
    sqlstring = p_sql;
  }

  // jhaley 20111010: more prophylaxis: test for loss of connection - we are probably shutting down the program.
  if(!dbDataSet || !dbDataSet->Database->TestConnected())
    return;

  MSG_CHECKPOINT("classLookUpTable::RefreshDataset", "4");
  p_item_list->Clear();
  MSG_CHECKPOINT("classLookUpTable::RefreshDataset", "5");
  dbDataSet->SelectSQL->Clear();
  dbDataSet->SelectSQL->Add(sqlstring.c_str());
  dbDataSet->Open();
  dbDataSet->First();
  MSG_CHECKPOINT("classLookUpTable::RefreshDataset", "6");
  while (dbDataSet->Eof == false)
  {
    MSG_CHECKPOINT("classLookUpTable::RefreshDataset", "7");
    // only change global (to the lookuptable) table_vec if we're populating the "default" SQL list (not a custom SQL list)
    if (p_sql == "")
    {
      MSG_CHECKPOINT("classLookUpTable::RefreshDataset", "8");
      temp_struct.id    = dbDataSet->FieldByName("id"   )->AsString.c_str();
      temp_struct.item  = dbDataSet->FieldByName("item" )->AsString.c_str();
      temp_struct.valid = dbDataSet->FieldByName("valid")->AsString.c_str();
      table_vec.push_back(temp_struct);
    }
    if (hide_option == showall
    #ifdef prometheus_externs
      || optionsPermissions["special.see_invalid_items"]["enabled"] == "true"
    #endif
      || (hide_option == showvalid && dbDataSet->FieldByName("valid")->AsString == "1"))
    {
      MSG_CHECKPOINT("classLookUpTable::RefreshDataset", "9");
      if(display_option == as_pretty)
        p_item_list->Add(PrettyString(dbDataSet->FieldByName("item")->AsString.c_str()).c_str());
      else if (display_option == as_uppercase)
        p_item_list->Add(UppercaseString(dbDataSet->FieldByName("item")->AsString.c_str()).c_str());
      else if (display_option == as_lowercase)
        p_item_list->Add(LowercaseString(dbDataSet->FieldByName("item")->AsString.c_str()).c_str());
      else if (display_option == as_sentence)
        p_item_list->Add(SentenceString(dbDataSet->FieldByName("item")->AsString.c_str()).c_str());
      else if (display_option == as_nothing)
        p_item_list->Add(dbDataSet->FieldByName("item")->AsString.c_str());
      // TODO -c HACK: So I'm just sticking the integer ID into this pointer field, as I'll need it later....
      p_item_list->Objects[p_item_list->Count-1] = (TObject*)(StringToInt(dbDataSet->FieldByName("id")->AsString.c_str()));
    }
    MSG_CHECKPOINT("classLookUpTable::RefreshDataset", "10");
    dbDataSet->Next();
  }
  dbDataSet->Close();
  MSG_CHECKPOINT("classLookUpTable::RefreshDataset", "11");
}
//--------------------------------------------------------------------------
void __fastcall classLookUpTable::OnComboKeyPress(TObject *Sender, char &Key)
{
  FindComboBoxItem(Sender, Key);
}
//--------------------------------------------------------------------------

