//PHOBOS
#include "lookuptablewrapper.h"

classLookUpTableWrapper::classLookUpTableWrapper (TIBDatabase * database)
{
  try
  {
    dbDatabase = database;
    dbTransaction = new TIBTransaction (NULL);
#ifdef sqlfunctions
    SpyTransaction(dbTransaction, database);
#else
    dbTransaction->DefaultDatabase = dbDatabase;
    dbTransaction->DefaultAction = TARollback;
    dbTransaction->Params->Add("nowait");
    dbTransaction->Params->Add("read_committed");
    dbTransaction->Params->Add("rec_version");
    dbTransaction->Active = true;
#endif
  }
  catch(...) {}
}
//--------------------------------------------------------------------------
classLookUpTableWrapper::~classLookUpTableWrapper ()
{
  for (map<string, classLookUpTable *>::iterator itr = lookuptables.begin();
       itr != lookuptables.end(); itr++)
  {
     delete itr->second;
  }
  if(dbTransaction && dbTransaction->DefaultDatabase->TestConnected() && dbTransaction->Active)
    dbTransaction->Commit();
  if(dbTransaction)
    delete dbTransaction;
}
//--------------------------------------------------------------------------
bool classLookUpTableWrapper::AddLookUpTable (string tableName, bool change, int display, int hide, TPopupMenu * popup, int autoload)
{
  bool result;
  if(dbDatabase->TestConnected())
  {
    canChange = change;
    classLookUpTable * new_table = new classLookUpTable(dbDatabase, dbTransaction, tableName, canChange, display, hide, popup, autoload);
    lookuptables[tableName] = new_table;
    result = lookuptables[tableName]->IsFine();
  }
  else
    result = false;
  return result;
}
//--------------------------------------------------------------------------
bool classLookUpTableWrapper::RegisterComboBox (string tableName, TComboBox * box, string sql, bool (*follow_up)(TIBTransaction *, const string &, const string &))
{
 #ifdef use_tcpip_messaging
 if(debug_level[libraries]>=2 && tcpip_message_router) tcpip_message_router->SendMessageToAll("debug", ("lookupTableWrapper::RegisterComboBox() called for table '"+tableName+"' with custom sql '"+sql+"'").c_str());
 #endif
 bool result;
 map<string, classLookUpTable *>::iterator found_class = lookuptables.find(tableName);
 if (found_class != lookuptables.end())
 {
  found_class->second->RegisterComboBox(box, sql, follow_up);
  result = true;
 }
 else
 {
  string errorMessage =  "Didn't find combo box for " + tableName + " to register.\n"
    "Please inform a programmer of this error immediately.\n"
    "Be sure to report on which screen you saw this error.";
  #ifdef prometheus_util
  PSay(errorMessage, m_program_error, MB_OK, "classLookUpTableWrapper");
  #else
  Application->MessageBox(errorMessage.c_str(), "Program error in classLookUpTableWrapper::RegisterComboBox()", MB_OK);
  #endif

  result = false;
 }
 #ifdef use_tcpip_messaging
 if(debug_level[libraries]>=3 && tcpip_message_router) tcpip_message_router->SendMessageToAll("debug", ("lookupTableWrapper::RegisterComboBox() call result: '"+IntToBoolString(BoolToInt(result))+"'").c_str());
 #endif
 return result;
}
//--------------------------------------------------------------------------
bool classLookUpTableWrapper::UnregisterComboBox (string tableName, TComboBox * box)
{
 #ifdef use_tcpip_messaging
 if(debug_level[libraries]>=2 && tcpip_message_router) tcpip_message_router->SendMessageToAll("debug", ("lookupTableWrapper::UnregisterComboBox() called for table '"+tableName+"'").c_str());
 #endif
 bool result;
 map<string, classLookUpTable *>::iterator found_class = lookuptables.find(tableName);
 if (found_class != lookuptables.end())
 {
  found_class->second->UnregisterComboBox(box);
  result = true;
 }
 else
 {
  string errorMessage =  "Didn't find combo box for " + tableName + " to unregister.\n"
    "Please inform a programmer of this error immediately\n"
    "Be sure to report on which screen you saw this error";
  #ifdef prometheus_util
  PSay(errorMessage, m_program_error, MB_OK, "classLookUpTableWrapper");
  #else
  Application->MessageBox(errorMessage.c_str(), "Program error in classLookUpTableWrapper::UnregisterComboBox()", MB_OK);
  #endif

  result = false;
 }
 #ifdef use_tcpip_messaging
 if(debug_level[libraries]>=3 && tcpip_message_router) tcpip_message_router->SendMessageToAll("debug", ("lookupTableWrapper::UnregisterComboBox() call result: '"+IntToBoolString(BoolToInt(result))+"'").c_str());
 #endif
 return result;
}
//--------------------------------------------------------------------------
bool classLookUpTableWrapper::UnregisterComboBox (TComboBox * box)
{
 for (map<string, classLookUpTable *>::iterator i = lookuptables.begin(); i != lookuptables.end(); i++)
 {
  if (i->second->BoxIsRegistered(box))
  {
   i->second->UnregisterComboBox(box);
   return true;
  }
 }
 return false;
}
//--------------------------------------------------------------------------
int classLookUpTableWrapper::IsInLookUpTable(string tableName, string enteredString)
{
 #ifdef use_tcpip_messaging
 if(debug_level[libraries]>=2 && tcpip_message_router) tcpip_message_router->SendMessageToAll("debug", ("lookupTableWrapper::IsInLookupTable() called for table '"+tableName+"' with string '"+enteredString+"'").c_str());
 #endif
 int result;
 map<string, classLookUpTable *>::iterator found_class = lookuptables.find(tableName);
 if (found_class != lookuptables.end())
  if (found_class->second->IsInLookUpTable(enteredString))
   result = 1; // in table
  else
   result = 0; // not in table
 else
  result = -1; // no such table
 #ifdef use_tcpip_messaging
 if(debug_level[libraries]>=3 && tcpip_message_router) tcpip_message_router->SendMessageToAll("debug", ("lookupTableWrapper::IsInLookupTable() call result: '"+IntToBoolString(BoolToInt(result))+"'").c_str());
 #endif
 return result;
}
//--------------------------------------------------------------------------
int classLookUpTableWrapper::IsInLookUpTable(string tableName, TComboBox * box)
{
 string enteredString = box->Text.c_str();
 #ifdef use_tcpip_messaging
 if(debug_level[libraries]>=2 && tcpip_message_router) tcpip_message_router->SendMessageToAll("debug", ("lookupTableWrapper::IsInLookupTable() called for table '"+tableName+"' with string '"+enteredString+"'").c_str());
 #endif
 int result;
 map<string, classLookUpTable *>::iterator found_class = lookuptables.find(tableName);
 if (found_class != lookuptables.end())
  if (found_class->second->IsInLookUpTable(enteredString, box)) // pass the TComboBox * too.  This only matters
   result = 1; // in table                                        // with custom SQL and autoload_option on
  else
   result = 0; // not in table
 else
  result = -1; // no such table
 #ifdef use_tcpip_messaging
 if(debug_level[libraries]>=3 && tcpip_message_router) tcpip_message_router->SendMessageToAll("debug", ("lookupTableWrapper::IsInLookupTable() call result: '"+IntToBoolString(BoolToInt(result))+"'").c_str());
 #endif
 return result;
}
//--------------------------------------------------------------------------
bool classLookUpTableWrapper::RefreshAllBoxes(string tableName)
{
 // this function should probably no longer be called -- ExpectRefresh should be called instead
 #ifdef use_tcpip_messaging
 if(debug_level[libraries]>=2 && tcpip_message_router) tcpip_message_router->SendMessageToAll("debug", ("lookupTableWrapper::RefreshAllBoxes() called for table '"+tableName+"'").c_str());
 #endif
 map<string, classLookUpTable *>::iterator found_class = lookuptables.find(tableName);
 bool result = true;
 if (found_class != lookuptables.end())
  found_class->second->RefreshAllBoxes();
 else
  result = false;
 #ifdef use_tcpip_messaging
 if(debug_level[libraries]>=3 && tcpip_message_router) tcpip_message_router->SendMessageToAll("debug", ("lookupTableWrapper::RefreshAllBoxes() call result: '"+IntToBoolString(BoolToInt(result))+"'").c_str());
 #endif
 return result;
}
//--------------------------------------------------------------------------
// this function was the original, and still works just fine, assuming you aren't using any
// custom SQL:  just look up that value in the global dataset for given table, return id
string classLookUpTableWrapper::IdOf(string tableName, string value)
{
 #ifdef use_tcpip_messaging
 if(debug_level[libraries]>=2 && tcpip_message_router) tcpip_message_router->SendMessageToAll("debug", ("lookupTableWrapper::IdOf() called for table '"+tableName+"' with string '"+value+"'").c_str());
 #endif
 map<string, classLookUpTable *>::iterator found_class = lookuptables.find(tableName);
 string result = "";
 if (found_class != lookuptables.end())
  result = found_class->second->IdOf(value);
 #ifdef use_tcpip_messaging
 if(debug_level[libraries]>=3 && tcpip_message_router) tcpip_message_router->SendMessageToAll("debug", ("lookupTableWrapper::IdOf() call result: '"+result+"'").c_str());
 #endif
 return result;
}
//--------------------------------------------------------------------------
// for custom SQL, we need the box; it is our index.  Can still use this with non-custom stuff
// By default we're going to assume they want the ID of the currently-selected-combo-box-value
string classLookUpTableWrapper::IdOf(string tableName, TComboBox * box)
{
 string value = box->Text.c_str();
 return IdOf(tableName, box->Text.c_str(), box);
}
//--------------------------------------------------------------------------
// but if they want to use the box as an index (important only for custom SQL), but look up a
// -different- value than the currently-selected one, then we'll do that too
string classLookUpTableWrapper::IdOf(string tableName, string value, TComboBox * box)
{
 #ifdef use_tcpip_messaging
 if(debug_level[libraries]>=2 && tcpip_message_router) tcpip_message_router->SendMessageToAll("debug", ("lookupTableWrapper::IdOf() called for table '"+tableName+"' with string '"+value+"'").c_str());
 #endif
 map<string, classLookUpTable *>::iterator found_class = lookuptables.find(tableName);
 string result = "";
 if (found_class != lookuptables.end())
  result = found_class->second->IdOf(value, box); // pass the TComboBox * too.  This only matters with custom SQL and autoload_option on
 #ifdef use_tcpip_messaging
 if(debug_level[libraries]>=3 && tcpip_message_router) tcpip_message_router->SendMessageToAll("debug", ("lookupTableWrapper::IdOf() call result: '"+result+"'").c_str());
 #endif
 return result;
}
//--------------------------------------------------------------------------
vector<string> classLookUpTableWrapper::IdsOf(string tableName, const vector<string> &values, TComboBox * box)
{
  vector<string> result;
  for (unsigned int i=0; i!=values.size(); i++)
    result.push_back(IdOf(tableName, values[i], box));
  return result;
}
//--------------------------------------------------------------------------
set<string> classLookUpTableWrapper::IdsOf(string tableName, const set<string> &values, TComboBox * box)
{
  set<string> result;
  for (set<string>::const_iterator itr=values.begin(); itr!=values.end(); itr++)
    result.insert(IdOf(tableName, *itr, box));
  return result;
}
//--------------------------------------------------------------------------
bool classLookUpTableWrapper::AddValue(string tableName, string value, TComboBox *box)
{
 #ifdef use_tcpip_messaging
 if(debug_level[libraries]>=2 && tcpip_message_router) tcpip_message_router->SendMessageToAll("debug", ("lookupTableWrapper::AddValue() called for table '"+tableName+"' with string '"+value+"'").c_str());
 #endif
 map<string, classLookUpTable *>::iterator found_class = lookuptables.find(tableName);
 bool result = false;
 if (found_class != lookuptables.end())
  result = found_class->second->AddValue(value, box);
 #ifdef use_tcpip_messaging
 if(debug_level[libraries]>=3 && tcpip_message_router) tcpip_message_router->SendMessageToAll("debug", ("lookupTableWrapper::AddValue() call result: '"+IntToBoolString(BoolToInt(result))+"'").c_str());
 #endif
 return result;
}
//--------------------------------------------------------------------------
bool classLookUpTableWrapper::UnsafeAddValue(string tableName, string value, TComboBox *box)
{
 #ifdef use_tcpip_messaging
 if(debug_level[libraries]>=2 && tcpip_message_router) tcpip_message_router->SendMessageToAll("debug", ("lookupTableWrapper::UnsafeAddValue() called for table '"+tableName+"' with string '"+value+"'").c_str());
 #endif
 bool result = false, found = false;
 map<string, classLookUpTable *>::iterator itr = lookuptables.begin();
 while(itr != lookuptables.end() && !found)
 {
   if (itr->first == tableName)
   {
     found = true;
     result = lookuptables[tableName]->UnsafeAddValue(value, box);
   }
   itr++;
 }
 #ifdef use_tcpip_messaging
 if(debug_level[libraries]>=3 && tcpip_message_router) tcpip_message_router->SendMessageToAll("debug", ("lookupTableWrapper::UnsafeAddValue() call result: '"+IntToBoolString(BoolToInt(result))+"'").c_str());
 #endif
 return result;
}
//--------------------------------------------------------------------------
string classLookUpTableWrapper::ValueOf(string tableName, string id, TComboBox * box)
{
 #ifdef use_tcpip_messaging
 if(debug_level[libraries]>=2 && tcpip_message_router)
   tcpip_message_router->SendMessageToAll("debug", ("lookupTableWrapper::ValueOf() called for table '"+tableName+"' with string '"+id+"'").c_str());
 #endif
 map<string, classLookUpTable *>::iterator found_class = lookuptables.find(tableName);
 string result = "";
 if (found_class != lookuptables.end())
  result = found_class->second->ValueOf(id, box);
 else
 {
   #ifdef use_tcpip_messaging
   if(debug_level[libraries]>=1 && tcpip_message_router)
     tcpip_message_router->SendMessageToAll("debug", ("lookupTableWrapper::ValueOf('" + tableName + "', '" + id + "') lookup table class could not be found!").c_str());
   #endif
 }
 #ifdef use_tcpip_messaging
 if(debug_level[libraries]>=3 && tcpip_message_router)
   tcpip_message_router->SendMessageToAll("debug", ("lookupTableWrapper::ValueOf() call result: '"+result+"'").c_str());
 #endif
 return result;
}
//--------------------------------------------------------------------------
vector<string> classLookUpTableWrapper::ValuesOf (string tableName, const vector<string> &ids, TComboBox * box)
{
  // if order matters to you, pass by vector, get a vector back
  vector<string> result;
  for (unsigned int i=0; i!=ids.size(); i++)
    result.push_back(ValueOf(tableName, ids[i], box));
  return result;
}
//--------------------------------------------------------------------------
set<string> classLookUpTableWrapper::ValuesOf (string tableName, const set<string> &ids, TComboBox * box)
{ // if not, sets will do
  set<string> result;
  for (set<string>::const_iterator itr=ids.begin(); itr!=ids.end(); itr++)
    result.insert(ValueOf(tableName, *itr, box));
  return result;
}
//--------------------------------------------------------------------------
bool classLookUpTableWrapper::IsValidId (string tableName, string id, TComboBox * box)
{
 #ifdef use_tcpip_messaging
 if(debug_level[libraries]>=2 && tcpip_message_router) tcpip_message_router->SendMessageToAll("debug", ("lookupTableWrapper::IsValidId() called for table '"+tableName+"' with string '"+id+"'").c_str());
 #endif
 map<string, classLookUpTable *>::iterator found_class = lookuptables.find(tableName);
 bool result = false;
 if (found_class != lookuptables.end())
  result = found_class->second->IsValidId(id, box);
 #ifdef use_tcpip_messaging
 if(debug_level[libraries]>=3 && tcpip_message_router) tcpip_message_router->SendMessageToAll("debug", ("lookupTableWrapper::IsValidId() call result: '"+IntToBoolString(BoolToInt(result))+"'").c_str());
 #endif
 return result;
}
//--------------------------------------------------------------------------
vector<string> classLookUpTableWrapper::ListTables()
{
 #ifdef use_tcpip_messaging
 if(debug_level[libraries]>=2 && tcpip_message_router) tcpip_message_router->SendMessageToAll("debug", "lookupTableWrapper::ListTables() called");
 #endif
 vector<string> answer;
 for (map<string, classLookUpTable *>::iterator i_table = lookuptables.begin();
      i_table != lookuptables.end(); i_table++)
  answer.push_back(i_table->first);
 return answer;
}
//--------------------------------------------------------------------------
bool classLookUpTableWrapper::CanAddToTable(string tableName)
{
 #ifdef use_tcpip_messaging
 if(debug_level[libraries]>=2 && tcpip_message_router) tcpip_message_router->SendMessageToAll("debug", ("lookupTableWrapper::CanAddToTable() called for table '"+tableName+"'").c_str());
 #endif
 bool result = false;
 map<string, classLookUpTable *>::iterator found_class = lookuptables.find(tableName);
 if (found_class != lookuptables.end())
  result = found_class->second->CanAddToTable();
 #ifdef use_tcpip_messaging
 if(debug_level[libraries]>=3 && tcpip_message_router) tcpip_message_router->SendMessageToAll("debug", ("lookupTableWrapper::CanAddToTable() call result: '"+IntToBoolString(BoolToInt(result))+"'").c_str());
 #endif
 return result;
}
//--------------------------------------------------------------------------
string classLookUpTableWrapper::TableName (TComboBox * box)
{
  #ifdef use_tcpip_messaging
  if(debug_level[libraries]>=2 && tcpip_message_router) tcpip_message_router->SendMessageToAll("debug", "lookupTableWrapper::TableName()");
  #endif
  string result = "";
  map<string, classLookUpTable *>::iterator itr = lookuptables.begin();
  while (itr != lookuptables.end() && result=="")
  {
    if (itr->second->BoxIsRegistered(box))
    {
      result = itr->second->TableName();
    }
    itr++;
  }
  #ifdef use_tcpip_messaging
  if(debug_level[libraries]>=3 && tcpip_message_router) tcpip_message_router->SendMessageToAll("debug", ("lookupTableWrapper::TableName() call result: '"+result+"'").c_str());
  #endif
  return result;
}
//--------------------------------------------------------------------------
void classLookUpTableWrapper::ExpectRefresh (string tableName)
{
 #ifdef use_tcpip_messaging
 if(debug_level[libraries]>=2 && tcpip_message_router) tcpip_message_router->SendMessageToAll("debug", ("lookupTableWrapper::ExpectRefresh() called for table '"+tableName+"'").c_str());
 #endif
 map<string, classLookUpTable *>::iterator found_class = lookuptables.find(tableName);
 if (found_class != lookuptables.end())
  found_class->second->ExpectRefresh();
 else
 {
  string errorMessage =  "A specific lookup table (" + tableName + ") could not be located, to be informed of changes it should know about.";
  #ifdef prometheus_util
  PSay(errorMessage, m_program_error, MB_OK, "classLookUpTableWrapper");
  #else
  Application->MessageBox(errorMessage.c_str(), "Program error in classLookUpTableWrapper::ExpectRefresh()", MB_OK);
  #endif
 }
}
//--------------------------------------------------------------------------
vector<string> classLookUpTableWrapper::GetValueList(string tableName, TComboBox * box)
{
  #ifdef use_tcpip_messaging
  if(debug_level[libraries]>=2 && tcpip_message_router) tcpip_message_router->SendMessageToAll("debug", ("lookupTableWrapper::GetValueList() called for table '"+tableName+"'").c_str());
  #endif
  vector<string> return_vec;
  for (map<string, classLookUpTable *>::iterator itr = lookuptables.begin();
       itr != lookuptables.end(); itr++)
    if (itr->first == tableName)
      return lookuptables[tableName]->GetValueList(box);
  return return_vec;
}
//--------------------------------------------------------------------------

