#include "mesg_q.h"
#include "util.h"
#include <iostream>

#ifdef __WIN32__
 #include <windows.h>
#else
 #include <unistd.h>
#endif

char read_buffer [max_chunk_size];

// checks to see if the buffer string is currently in use
bool   inbound_message_queue::IsMessageProcessing ()
{
 return (!current_message.empty());
}

// checks to see if the queue is non-empty
bool   inbound_message_queue::IsMessagePending ()
{
 return (!pending_messages.empty());
}

// returns the next string off the queue
// should be called only if IsMessagePending()
string inbound_message_queue::GetMessage ()
{
 string return_value = pending_messages.front();
 pending_messages.pop_front();
 return return_value;
}

// attempts to read from the file descriptor
// the file descriptor should have been marked as readable prior to call
// returns false if no bytes read (connection closed)
bool   inbound_message_queue::AttemptRead (TClientSocket *the_socket)
{
 AnsiString message_chunk;
 message_chunk = the_socket->Socket->ReceiveText();
 chunk_size = message_chunk.Length();
 if (!chunk_size)
  return false;
 current_message += message_chunk.c_str();

 PrepMessage();
 return true;
}

// takes a message, checks for completion, sticks completed message on the queue
void   inbound_message_queue::PrepMessage ()
{
 unsigned int first_colon;
 unsigned int first_char;
 while (IsMessageProcessing())
 {
  first_colon = current_message.find(size_separator);
  first_char  = current_message.find_first_not_of("0123456789");
  if (first_colon != string::npos)
  {
   if (first_char == string::npos || first_char >= first_colon)
   {
    chunk_size = StringToInt (current_message.substr(0, first_colon));
    if (current_message.length() >= (chunk_size + first_colon + 1))
    {
     pending_messages.push_back(current_message.substr(first_colon + 1, chunk_size));
     if (first_colon + chunk_size + 1 < current_message.length())
      current_message = current_message.substr(first_colon + chunk_size + 1);
     else
      current_message = "";
    }
    else
     break;
   }
   else
    current_message = "";
  }
  else
  {
   if (first_char != string::npos)
    current_message = "";
   break;
  }
 }
 return;
}

// checks to see if the buffer string is currently in use
bool outbound_message_queue::IsMessageProcessing ()
{
 return (!current_message.empty());
}

// checks to see if the queue is non-empty
bool outbound_message_queue::IsMessagePending ()
{
 return (!pending_messages.empty());
}

// adds a message to the queue to be sent
// if no message is currently being processed, and some are waiting,
// pulls next available message back off the queue and into the waiting area
void outbound_message_queue::AddMessage (string new_message)
{
 pending_messages.push_back(new_message);
 PrepMessage();
 return;
}

// will attempt to send the next available chunk on the file descriptor
// the file descriptor should have been marked as writable before calling
// returns true so long as nothing went wrong (zero bytes sent, for example)
bool outbound_message_queue::AttemptWrite (TClientSocket *the_socket)
{
 bool first_attempt = true;
 while (IsMessageProcessing())
 {
  chunk_size = the_socket->Socket->SendText(current_message.c_str());
  if (!chunk_size)
  {
   if (first_attempt)
    return false;
   else
    break;
  }
  if ((unsigned int)chunk_size < current_message.length())
   current_message = current_message.substr(chunk_size);
  else
   current_message = "";
  PrepMessage();
  first_attempt = false;
 }
 return true;
}

// take next available message for sending and stick it in the sending string, prepared
void outbound_message_queue::PrepMessage ()
{
 if (!IsMessageProcessing() && IsMessagePending())
 {
  current_message = pending_messages.front();
  current_message = IntToString(current_message.length()) + size_separator + current_message;
  pending_messages.pop_front();
 }
 return;
}
