#include "metric_conversion.h"
#include <float.h>

// can convert if both units are in the same dimension
bool CanConvertMetric (TIBTransaction * dbTransaction, string source_unit_id, string destination_unit_id)
{
 if (source_unit_id == destination_unit_id)
  return true;

 if (!IsInt(source_unit_id) || !IsInt(destination_unit_id))
  return false;

 if (dbTransaction && dbTransaction->Active)
 {
  if (GetOneField(dbTransaction, "select 1 from units src_units inner join units dest_units on src_units.id = " + source_unit_id + " and dest_units.id = " + destination_unit_id + " and src_units.dimension_id = dest_units.dimension_id") == "")
   return false;
  else
   return true;
 } else
  return false;
}

bool CanConvertMetric (TIBDatabase * dbConnection, string source_unit_id, string destination_unit_id)
{
 if (source_unit_id == destination_unit_id)
  return true;

 if (!IsInt(source_unit_id) || !IsInt(destination_unit_id))
  return false;

 if (!dbConnection)
  return false;
 if(dbConnection->TestConnected())
 {
  TIBTransaction * dbTransaction = new TIBTransaction(NULL);
  StdTransaction (dbTransaction, dbConnection);
  bool to_return;
  try
  {
   to_return = CanConvertMetric(dbTransaction, source_unit_id, destination_unit_id);
  }
  catch (...)
  {
   if (dbTransaction->Active)
    dbTransaction->Rollback();
  }
  if (dbTransaction->Active)
   dbTransaction->Commit();
  delete dbTransaction;
  return to_return;
 } else
  return false;
}

/*
new.value = new.ratio * ((old.value - old.offset) / old.ratio) + new.offset
old.value = old.ratio * ((new.value - new.offset) / new.ratio) + old.offset
*/
float FloatConvertMetric(TIBTransaction * dbTransaction, float source_value, string source_unit_id, string destination_unit_id)
{
 if (source_unit_id == destination_unit_id)
  return source_value;

 if (!IsInt(source_unit_id) || !IsInt(destination_unit_id))
  return source_value;

 if (dbTransaction && dbTransaction->Active)
 {
  string ans = GetOneField(dbTransaction,
   "select (dest_unit.conversion_offset + (dest_unit.conversion_ratio * ((" + FloatToString(source_value) + " - src_unit.conversion_offset) / src_unit.conversion_ratio))) dest_value "
   "from units dest_unit "
   "inner join units src_unit "
   "on dest_unit.id = " + destination_unit_id +
   " and src_unit.id = " + source_unit_id +
   " and dest_unit.dimension_id = src_unit.dimension_id");
  return StringToFloat(ans);
 } else
  return 0.00; // should throw. or something.
}

float FloatConvertMetric(TIBDatabase * dbConnection, float source_value, string source_unit_id, string destination_unit_id)
{
 if (source_unit_id == destination_unit_id)
  return source_value;

 if (!IsInt(source_unit_id) || !IsInt(destination_unit_id))
  return source_value;

 if (!dbConnection)
  return false;
 if(dbConnection->TestConnected())
 {
  TIBTransaction * dbTransaction = new TIBTransaction(NULL);
  StdTransaction (dbTransaction, dbConnection);
  float to_return = 0.00;
  try
  {
   to_return = FloatConvertMetric (dbTransaction, source_value, source_unit_id, destination_unit_id);
  }
  catch (...)
  {
   if (dbTransaction->Active)
    dbTransaction->Rollback();
  }
  if (dbTransaction->Active)
   dbTransaction->Commit();
  delete dbTransaction;
  return to_return;
 } else
  return 0.00; // should throw or something
}

string StringConvertMetric(TIBTransaction * dbTransaction, string source_value, string source_unit_id, string destination_unit_id)
{
 if (!IsInt(source_unit_id) || !IsInt(destination_unit_id) || source_unit_id == "" || destination_unit_id == "")
  return source_value;

 return FloatToString(FloatConvertMetric(dbTransaction, StringToFloat(source_value), source_unit_id, destination_unit_id));
}

string StringConvertMetric(TIBDatabase * dbConnection, string source_value, string source_unit_id, string destination_unit_id)
{
 if (!IsInt(source_unit_id) || !IsInt(destination_unit_id) || source_unit_id == "" || destination_unit_id == "")
  return source_value;

 return FloatToString(FloatConvertMetric(dbConnection, StringToFloat(source_value), source_unit_id, destination_unit_id));
}

