#include "query_action_utils.h"

/*

 See note at bottom.

*/
//---------------------------------------------------------------------------
bool quValidOptionalDate(TEdit* control)
{
 control->Color = clWindow;

 Pdate only_date = Pdate(control->Text.c_str(), global_mode);
 bool only_given = (control->Text != "");
 bool only_valid = (!only_given || only_date.IsValidDate());

 if (!only_valid)
 {
  control->Color = clBad;
  return false;
 }
 return true;
}
//---------------------------------------------------------------------------
bool quSomeOptionalDate(TEdit* control_a, TEdit* control_b)
{
 control_a->Color = clWindow;
 control_b->Color = clWindow;

 Pdate start_date = Pdate(control_a->Text.c_str(), global_mode);
 Pdate stop_date  = Pdate(control_b->Text.c_str(), global_mode);
 bool start_given = (control_a->Text != "");
 bool stop_given  = (control_b->Text != "");
 bool start_valid = (!start_given || start_date.IsValidDate());
 bool stop_valid  = (!stop_given || stop_date.IsValidDate());

 if (!start_given && !stop_given)
 {
  PSay("To restrict by date, at least one date must be provided.", m_missing_data, MB_OK, "TframeQueryGeneric");
  control_a->Color = clBad;
  control_b->Color = clBad;
  return false;
 }
 return true;
}
//---------------------------------------------------------------------------
bool quSortedOptionalDate(TEdit* control_a, TEdit* control_b)
{
 control_a->Color = clWindow;
 control_b->Color = clWindow;

 Pdate start_date = Pdate(control_a->Text.c_str(), global_mode);
 Pdate stop_date  = Pdate(control_b->Text.c_str(), global_mode);
 bool start_given = (control_a->Text != "");
 bool stop_given  = (control_b->Text != "");
 bool start_valid = (!start_given || start_date.IsValidDate());
 bool stop_valid  = (!stop_given || stop_date.IsValidDate());

 if (start_given && stop_given && start_valid && stop_valid && start_date > stop_date)
 {
  PSay("The start date should be earlier than the stop date, if both are provided.", m_invalid_data, MB_OK, "TframeQueryGeneric");
  control_a->Color = clBad;
  control_b->Color = clBad;
  return false;
 }
 return true;
}
//---------------------------------------------------------------------------
bool quSortedRequiredDate(TDateTimePicker* control_a, TDateTimePicker* control_b)
{
 control_a->Color = clWindow;
 control_b->Color = clWindow;

 if (Pdate(control_a->DateTime) > Pdate(control_b->DateTime))
 {
  PSay("The start date should be earlier than the stop date.", m_invalid_data, MB_OK, "TframeQueryGeneric");
  control_a->Color = clBad;
  control_b->Color = clBad;
  return false;
 }
 return true;
}
//---------------------------------------------------------------------------
void quSaveOptionalDate(map<string, xValue> & config_map, string attribute_name, TEdit* control)
{
 if (Pdate(control->Text.c_str(), global_mode).IsValidDate())
  config_map[attribute_name] = xFromPdate(Pdate(control->Text.c_str(), global_mode));
}
//---------------------------------------------------------------------------
void quSaveRequiredDate(map<string, xValue> & config_map, string attribute_name, TDateTimePicker* control)
{
 config_map[attribute_name] = xFromPdate(Pdate(control->DateTime));
}
//---------------------------------------------------------------------------
void quLoadOptionalDate(map<string, xValue> & config_map, string attribute_name, TEdit* control)
{
 if (xTag(config_map[attribute_name]) == "date")
  control->Text = xToPdate(config_map[attribute_name]).OutDate(global_mode).c_str();
 else
  control->Text = "";
}
//---------------------------------------------------------------------------
void quLoadRequiredDate(map<string, xValue> & config_map, string attribute_name, TDateTimePicker* control)
{
 if (xTag(config_map[attribute_name]) == "date")
  control->DateTime = xToPdate(config_map[attribute_name]).OutDate();
}
//---------------------------------------------------------------------------
bool quValidSortedOptionalDate(TEdit* control_a, TEdit* control_b)
{
 // don't short-circuit between the first two, but do short-circuit between the first group and the second
 bool answer = (quValidOptionalDate(control_a) &
                quValidOptionalDate(control_b)) &&
                quSortedOptionalDate(control_a, control_b);
 return answer;
}
//---------------------------------------------------------------------------
bool quValidSortedSomeOptionalDate(TEdit* control_a, TEdit* control_b)
{
 bool answer = quValidSortedOptionalDate(control_a, control_b) &&
               quSomeOptionalDate(control_a, control_b);
 return answer;
}
//---------------------------------------------------------------------------
bool quSortedOptionalDate(TDateTimePicker* control_a, TEdit* control_b)
{
 control_a->Color = clWindow;
 control_b->Color = clWindow;

 Pdate start_date = Pdate(control_a->DateTime);
 Pdate stop_date  = Pdate(control_b->Text.c_str(), global_mode);
 bool stop_given  = (control_b->Text != "");
 bool stop_valid  = (!stop_given || stop_date.IsValidDate());

 if (stop_given && stop_valid && start_date > stop_date)
 {
  PSay("The start date should be earlier than the stop date, if both are provided.", m_invalid_data, MB_OK, "TframeQueryGeneric");
  control_a->Color = clBad;
  control_b->Color = clBad;
  return false;
 }
 return true;
}
//---------------------------------------------------------------------------
bool quSortedOptionalDate(TEdit* control_a, TDateTimePicker* control_b)
{
 control_a->Color = clWindow;
 control_b->Color = clWindow;

 Pdate start_date = Pdate(control_a->Text.c_str(), global_mode);
 Pdate stop_date  = Pdate(control_b->DateTime);
 bool start_given = (control_a->Text != "");
 bool start_valid = (!start_given || start_date.IsValidDate());

 if (start_given && start_valid && start_date > stop_date)
 {
  PSay("The start date should be earlier than the stop date, if both are provided.", m_invalid_data, MB_OK, "TframeQueryGeneric");
  control_a->Color = clBad;
  control_b->Color = clBad;
  return false;
 }
 return true;
}
//---------------------------------------------------------------------------
bool quValidSortedOptionalDate(TDateTimePicker* control_a, TEdit* control_b)
{
 bool answer = quValidOptionalDate(control_b) &&
               quSortedOptionalDate(control_a, control_b);
 return answer;
}
//---------------------------------------------------------------------------
bool quValidSortedOptionalDate(TEdit* control_a, TDateTimePicker* control_b)
{
 bool answer = quValidOptionalDate(control_a) &&
               quSortedOptionalDate(control_a, control_b);
 return answer;
}
//---------------------------------------------------------------------------
bool quValidRequiredInt(TEdit* control)
{
 control->Color = clWindow;

 if (control->Text == "" || !IsInt(control->Text.c_str()))
 {
  control->Color = clBad;
  return false;
 }
 return true;
}
//---------------------------------------------------------------------------
bool quValidOptionalInt(TEdit* control)
{
 control->Color = clWindow;

 if (control->Text != "" && !IsInt(control->Text.c_str()))
 {
  control->Color = clBad;
  return false;
 }
 return true;
}
//---------------------------------------------------------------------------
bool quSomeOptionalInt(TEdit* control_a, TEdit* control_b)
{
 control_a->Color = clWindow;
 control_b->Color = clWindow;

 int start_number = StringToInt(control_a->Text.c_str());
 int stop_number  = StringToInt(control_b->Text.c_str());
 bool start_given = (control_a->Text != "");
 bool stop_given  = (control_b->Text != "");
 bool start_valid = (!start_given || IsInt(control_a->Text.c_str()));
 bool stop_valid  = (!stop_given || IsInt(control_b->Text.c_str()));

 if (!start_given && !stop_given)
 {
  PSay("At least one minimum or maximum must be provided.", m_missing_data, MB_OK, "TframeQueryGeneric");
  control_a->Color = clBad;
  control_b->Color = clBad;
  return false;
 }
 return true;
}
//---------------------------------------------------------------------------
bool quSortedOptionalInt(TEdit* control_a, TEdit* control_b)
{
 control_a->Color = clWindow;
 control_b->Color = clWindow;

 int start_number = StringToInt(control_a->Text.c_str());
 int stop_number  = StringToInt(control_b->Text.c_str());
 bool start_given = (control_a->Text != "");
 bool stop_given  = (control_b->Text != "");
 bool start_valid = (!start_given || IsInt(control_a->Text.c_str()));
 bool stop_valid  = (!stop_given || IsInt(control_b->Text.c_str()));

 if (start_given && stop_given && start_valid && stop_valid && start_number > stop_number)
 {
  PSay("The minimum should be less than the maximum, if both are provided.", m_invalid_data, MB_OK, "TframeQueryGeneric");
  control_a->Color = clBad;
  control_b->Color = clBad;
  return false;
 }
 return true;
}
//---------------------------------------------------------------------------
void quSaveOptionalInt(map<string, xValue> & config_map, string attribute_name, TEdit* control)
{
 if (control->Text != "" && IsInt(control->Text.c_str()))
  config_map[attribute_name] = xFromInt(StringToInt(control->Text.c_str()));
}
//---------------------------------------------------------------------------
void quLoadOptionalInt(map<string, xValue> & config_map, string attribute_name, TEdit* control)
{
 if (xTag(config_map[attribute_name]) == "int")
  control->Text = IntToString(xToInt(config_map[attribute_name])).c_str();
 else
  control->Text = "";
}
//---------------------------------------------------------------------------
void quLoadRequiredInt(map<string, xValue> & config_map, string attribute_name, TEdit* control, int default_int)
{
 if (xTag(config_map[attribute_name]) == "int")
  control->Text = IntToString(xToInt(config_map[attribute_name])).c_str();
 else
  control->Text = IntToString(default_int).c_str();
}
//---------------------------------------------------------------------------
bool quValidSortedOptionalInt(TEdit* control_a, TEdit* control_b)
{
 bool answer = (quValidOptionalInt(control_a) &
                quValidOptionalInt(control_b)) &&
                quSortedOptionalInt(control_a, control_b);
 return answer;
}
//---------------------------------------------------------------------------
bool quValidSortedSomeOptionalInt(TEdit* control_a, TEdit* control_b)
{
 bool answer = quValidSortedOptionalInt(control_a, control_b) &&
               quSomeOptionalInt(control_a, control_b);
 return answer;
}
//---------------------------------------------------------------------------
bool quValidSortedRequiredInt(TEdit* control_a, TEdit* control_b)
{
 bool answer = quValidRequiredInt(control_a) &
               quValidRequiredInt(control_b) &&
               quSortedOptionalInt(control_a, control_b);
 return answer;
}
//---------------------------------------------------------------------------
/*

 Yes, the use of & and && is purposeful. & does not short-circuit, && does.
 What we want is to sometimes perform several checks together (when independent)
 and sometimes only perform a check when preconditions are met.

 This is important because some of these aggregated checks reset fields to white.
 If you perform those checks right after another check that failed (and set the
 field to clBad), you'll confuse the user.

*/

//---------------------------------------------------------------------------
bool quValidRequiredText(TEdit* control)
{
 control->Color = clWindow;

 if (control->Text == "")
 {
  control->Color = clBad;
  return false;
 }
 return true;
}
//---------------------------------------------------------------------------
bool quSomeOptionalText(TEdit* control_a, TEdit* control_b)
{
 control_a->Color = clWindow;
 control_b->Color = clWindow;

 string start_text = control_a->Text.c_str();
 string stop_text  = control_b->Text.c_str();
 bool start_given = (!start_text.empty());
 bool stop_given  = (!stop_text.empty());

 if (!start_given && !stop_given)
 {
  PSay("At least one minimum or maximum must be provided.", m_missing_data, MB_OK, "TframeQueryGeneric");
  control_a->Color = clBad;
  control_b->Color = clBad;
  return false;
 }
 return true;
}
//---------------------------------------------------------------------------
bool quSortedOptionalText(TEdit* control_a, TEdit* control_b)
{
 control_a->Color = clWindow;
 control_b->Color = clWindow;

 string start_text = control_a->Text.c_str();
 string stop_text  = control_b->Text.c_str();
 bool start_given = (!start_text.empty());
 bool stop_given  = (!stop_text.empty());

 if (start_given && stop_given && start_text > stop_text)
 {
  PSay("The minimum should be less than the maximum, if both are provided.", m_invalid_data, MB_OK, "TframeQueryGeneric");
  control_a->Color = clBad;
  control_b->Color = clBad;
  return false;
 }
 return true;
}
//---------------------------------------------------------------------------
bool quValidSortedSomeOptionalText(TEdit* control_a, TEdit* control_b)
{
 bool answer = quSortedOptionalText(control_a, control_b) &&
               quSomeOptionalText(control_a, control_b);
 return answer;
}
//---------------------------------------------------------------------------
bool quValidSortedRequiredText(TEdit* control_a, TEdit* control_b)
{
 bool answer = quValidRequiredText(control_a) &
               quValidRequiredText(control_b) &&
               quSortedOptionalText(control_a, control_b);
 return answer;
}
//---------------------------------------------------------------------------
void quSaveOptionalText(map<string, xValue> & config_map, string attribute_name, TEdit* control)
{
 if (control->Text != "")
  config_map[attribute_name] = xFromString(control->Text.c_str());
}
//---------------------------------------------------------------------------
void quLoadOptionalText(map<string, xValue> & config_map, string attribute_name, TEdit* control)
{
 if (xTag(config_map[attribute_name]) == "string")
  control->Text = xToString(config_map[attribute_name]).c_str();
 else
  control->Text = "";
}
//---------------------------------------------------------------------------
void quLoadRequiredText(map<string, xValue> & config_map, string attribute_name, TEdit* control, string default_text)
{
 if (xTag(config_map[attribute_name]) == "string")
  control->Text = xToString(config_map[attribute_name]).c_str();
 else
  control->Text = default_text.c_str();
}
//---------------------------------------------------------------------------
bool quSortedRequiredTime(TDateTimePicker* control_a, TDateTimePicker* control_b)
{
 control_a->Color = clWindow;
 control_b->Color = clWindow;

 if (Ptime(control_a->DateTime) > Ptime(control_b->DateTime))
 {
  PSay("The start time should be earlier than the stop time.", m_invalid_data, MB_OK, "TframeQueryGeneric");
  control_a->Color = clBad;
  control_b->Color = clBad;
  return false;
 }
 return true;
}
//---------------------------------------------------------------------------
void quSaveOptionalTime(map<string, xValue> & config_map, string attribute_name, TEdit* control)
{
 if (Ptime(control->Text.c_str(), global_mode).IsValidTime())
  config_map[attribute_name] = xFromPtime(Ptime(control->Text.c_str(), global_mode));
}
//---------------------------------------------------------------------------
void quSaveRequiredTime(map<string, xValue> & config_map, string attribute_name, TDateTimePicker* control)
{
 config_map[attribute_name] = xFromPtime(Ptime(control->DateTime));
}
//---------------------------------------------------------------------------
void quLoadOptionalTime(map<string, xValue> & config_map, string attribute_name, TEdit* control)
{
 if (xTag(config_map[attribute_name]) == "time")
  control->Text = xToPtime(config_map[attribute_name]).OutTime(global_mode).c_str();
 else
  control->Text = "";
}
//---------------------------------------------------------------------------
void quLoadRequiredTime(map<string, xValue> & config_map, string attribute_name, TDateTimePicker* control)
{
 if (xTag(config_map[attribute_name]) == "time")
  control->DateTime = xToPtime(config_map[attribute_name]).OutTime();
}
//---------------------------------------------------------------------------
bool quValidRequiredFinancial(TEdit* control)
{
 control->Color = clWindow;

 if (control->Text == "" || !IsFloat(control->Text.c_str()))
 {
  control->Color = clBad;
  return false;
 }
 return true;
}
//---------------------------------------------------------------------------
bool quValidOptionalFinancial(TEdit* control)
{
 control->Color = clWindow;

 if (control->Text != "" && !IsFloat(control->Text.c_str()))
 {
  control->Color = clBad;
  return false;
 }
 return true;
}
//---------------------------------------------------------------------------
bool quSomeOptionalFinancial(TEdit* control_a, TEdit* control_b)
{
 control_a->Color = clWindow;
 control_b->Color = clWindow;

 financial start_number = StringToFinancial(control_a->Text.c_str());
 financial stop_number  = StringToFinancial(control_b->Text.c_str());
 bool start_given = (control_a->Text != "");
 bool stop_given  = (control_b->Text != "");
 bool start_valid = (!start_given || IsFloat(control_a->Text.c_str()));
 bool stop_valid  = (!stop_given || IsFloat(control_b->Text.c_str()));

 if (!start_given && !stop_given)
 {
  PSay("At least one minimum or maximum must be provided.", m_missing_data, MB_OK, "TframeQueryGeneric");
  control_a->Color = clBad;
  control_b->Color = clBad;
  return false;
 }
 return true;
}
//---------------------------------------------------------------------------
bool quSortedOptionalFinancial(TEdit* control_a, TEdit* control_b)
{
 control_a->Color = clWindow;
 control_b->Color = clWindow;

 financial start_number = StringToFinancial(control_a->Text.c_str());
 financial stop_number  = StringToFinancial(control_b->Text.c_str());
 bool start_given = (control_a->Text != "");
 bool stop_given  = (control_b->Text != "");
 bool start_valid = (!start_given || IsFloat(control_a->Text.c_str()));
 bool stop_valid  = (!stop_given || IsFloat(control_b->Text.c_str()));

 if (start_given && stop_given && start_valid && stop_valid && start_number > stop_number)
 {
  PSay("The minimum should be less than the maximum, if both are provided.", m_invalid_data, MB_OK, "TframeQueryGeneric");
  control_a->Color = clBad;
  control_b->Color = clBad;
  return false;
 }
 return true;
}
//---------------------------------------------------------------------------
void quSaveOptionalFinancial(map<string, xValue> & config_map, string attribute_name, TEdit* control)
{
 if (control->Text != "" && IsFloat(control->Text.c_str()))
  config_map[attribute_name] = xFromFinancial(StringToFinancial(control->Text.c_str()));
}
//---------------------------------------------------------------------------
void quLoadOptionalFinancial(map<string, xValue> & config_map, string attribute_name, TEdit* control)
{
 if (xTag(config_map[attribute_name]) == "financial")
  control->Text = FinancialToString(xToFinancial(config_map[attribute_name])).c_str();
 else
  control->Text = "";
}
//---------------------------------------------------------------------------
void quLoadRequiredFinancial(map<string, xValue> & config_map, string attribute_name, TEdit* control, financial default_financial)
{
 if (xTag(config_map[attribute_name]) == "financial")
  control->Text = FinancialToString(xToFinancial(config_map[attribute_name])).c_str();
 else
  control->Text = FinancialToString(default_financial).c_str();
}
//---------------------------------------------------------------------------
bool quValidSortedOptionalFinancial(TEdit* control_a, TEdit* control_b)
{
 bool answer = (quValidOptionalFinancial(control_a) &
                quValidOptionalFinancial(control_b)) &&
                quSortedOptionalFinancial(control_a, control_b);
 return answer;
}
//---------------------------------------------------------------------------
bool quValidSortedSomeOptionalFinancial(TEdit* control_a, TEdit* control_b)
{
 bool answer = quValidSortedOptionalFinancial(control_a, control_b) &&
               quSomeOptionalFinancial(control_a, control_b);
 return answer;
}
//---------------------------------------------------------------------------
bool quValidSortedRequiredFinancial(TEdit* control_a, TEdit* control_b)
{
 bool answer = quValidRequiredFinancial(control_a) &
               quValidRequiredFinancial(control_b) &&
               quSortedOptionalFinancial(control_a, control_b);
 return answer;
}
//---------------------------------------------------------------------------
bool quValidRequiredFloat(TEdit* control)
{
 control->Color = clWindow;

 if (control->Text == "" || !IsFloat(control->Text.c_str()))
 {
  control->Color = clBad;
  return false;
 }
 return true;
}
//---------------------------------------------------------------------------
bool quValidOptionalFloat(TEdit* control)
{
 control->Color = clWindow;

 if (control->Text != "" && !IsFloat(control->Text.c_str()))
 {
  control->Color = clBad;
  return false;
 }
 return true;
}
//---------------------------------------------------------------------------
bool quSomeOptionalFloat(TEdit* control_a, TEdit* control_b)
{
 control_a->Color = clWindow;
 control_b->Color = clWindow;

 float start_number = StringToFloat(control_a->Text.c_str());
 float stop_number  = StringToFloat(control_b->Text.c_str());
 bool start_given = (control_a->Text != "");
 bool stop_given  = (control_b->Text != "");
 bool start_valid = (!start_given || IsFloat(control_a->Text.c_str()));
 bool stop_valid  = (!stop_given || IsFloat(control_b->Text.c_str()));

 if (!start_given && !stop_given)
 {
  PSay("At least one minimum or maximum must be provided.", m_missing_data, MB_OK, "TframeQueryGeneric");
  control_a->Color = clBad;
  control_b->Color = clBad;
  return false;
 }
 return true;
}
//---------------------------------------------------------------------------
bool quSortedOptionalFloat(TEdit* control_a, TEdit* control_b)
{
 control_a->Color = clWindow;
 control_b->Color = clWindow;

 float start_number = StringToFloat(control_a->Text.c_str());
 float stop_number  = StringToFloat(control_b->Text.c_str());
 bool start_given = (control_a->Text != "");
 bool stop_given  = (control_b->Text != "");
 bool start_valid = (!start_given || IsFloat(control_a->Text.c_str()));
 bool stop_valid  = (!stop_given || IsFloat(control_b->Text.c_str()));

 if (start_given && stop_given && start_valid && stop_valid && start_number > stop_number)
 {
  PSay("The minimum should be less than the maximum, if both are provided.", m_invalid_data, MB_OK, "TframeQueryGeneric");
  control_a->Color = clBad;
  control_b->Color = clBad;
  return false;
 }
 return true;
}
//---------------------------------------------------------------------------
void quSaveOptionalFloat(map<string, xValue> & config_map, string attribute_name, TEdit* control)
{
 if (control->Text != "" && IsFloat(control->Text.c_str()))
  config_map[attribute_name] = xFromFloat(StringToFloat(control->Text.c_str()));
}
//---------------------------------------------------------------------------
void quLoadOptionalFloat(map<string, xValue> & config_map, string attribute_name, TEdit* control)
{
 if (xTag(config_map[attribute_name]) == "float")
  control->Text = FloatToString(xToFloat(config_map[attribute_name])).c_str();
 else
  control->Text = "";
}
//---------------------------------------------------------------------------
void quLoadRequiredFloat(map<string, xValue> & config_map, string attribute_name, TEdit* control, float default_float)
{
 if (xTag(config_map[attribute_name]) == "float")
  control->Text = FloatToString(xToFloat(config_map[attribute_name])).c_str();
 else
  control->Text = FloatToString(default_float).c_str();
}
//---------------------------------------------------------------------------
bool quValidSortedOptionalFloat(TEdit* control_a, TEdit* control_b)
{
 bool answer = (quValidOptionalFloat(control_a) &
                quValidOptionalFloat(control_b)) &&
                quSortedOptionalFloat(control_a, control_b);
 return answer;
}
//---------------------------------------------------------------------------
bool quValidSortedSomeOptionalFloat(TEdit* control_a, TEdit* control_b)
{
 bool answer = quValidSortedOptionalFloat(control_a, control_b) &&
               quSomeOptionalFloat(control_a, control_b);
 return answer;
}
//---------------------------------------------------------------------------
bool quValidSortedRequiredFloat(TEdit* control_a, TEdit* control_b)
{
 bool answer = quValidRequiredFloat(control_a) &
               quValidRequiredFloat(control_b) &&
               quSortedOptionalFloat(control_a, control_b);
 return answer;
}
//---------------------------------------------------------------------------

