#include "query_context.h"
#include "codeQueryGeneric.h"
#include "codeQuery_Whole.h"

query_context::query_context (TIBDatabase* dbDatabase)
{
 FLogOutput = NULL;
 FORM_TRY
  FGlobals = new xTuple();
 FORM_CATCH
 FORM_TRY
  FTransaction = new TIBTransaction(NULL);
  StdTransaction(FTransaction, dbDatabase);
 FORM_CATCH
}
//---------------------------------------------------------------------------
query_context::~query_context ()
{
 for (map<string, TframeQuery_Whole*>::iterator i = FWholeQueries.begin(); i != FWholeQueries.end(); i++)
  delete (i->second);
 FWholeQueries.clear();

 delete FGlobals;
 FGlobals = NULL;
 delete FTransaction;
 FTransaction = NULL;
}
//---------------------------------------------------------------------------
string query_context::CreateView (string statement, set<string> views_used, string purpose)
{
 string answer;
 if (FTransaction && FTransaction->Active)
 {
  string assigned_view_number = GetNextId(FTransaction, "temp_views");
  string desired_view_name = "temp_view_" + purpose + "_" + assigned_view_number; // IntToString((unsigned int)(void*)source_query);

/*
  PSay("Creating a view named " + desired_view_name + " with the following statement:\n" + statement, m_confirmation, MB_OK, "query_context");
  string plan;
  GetSqlPlan(FTransaction, statement, plan);
  PSay("Creating a view named " + desired_view_name + " for the following plan:\n" + plan, m_confirmation, MB_OK, "query_context");
*/

  string sql = "create view " + desired_view_name + " (id) as " + statement;
  if (!ExecuteStatement(FTransaction, sql))
   PSay("Failed to create view.\n" + sql, m_database_error, MB_OK, desired_view_name);
  else
  {
   if (FLogOutput)
    FLogOutput->Lines->Add(AnsiString("Done: ") + sql.c_str());
   FTransaction->Commit();
   FTransaction->StartTransaction();
   answer = desired_view_name;
   FViewDependencies[answer] = views_used;
   FViewDefinitions[answer] = statement;
  }
 }
 return answer;
}
//---------------------------------------------------------------------------
string query_context::CreateProxyView (string view_name)
{
 string answer;
 if (FTransaction && FTransaction->Active)
 {
  string sql;
  string assigned_query_number = GetNextId(FTransaction, "temp_id");
  FProxyQueries.push_back(assigned_query_number);

/*
  string plan;
  GetSqlPlan(FTransaction, "select * from " + view_name, plan);
  PSay("Proxying the following plan:\n" + plan + "\n" + DebugExplainView(view_name), m_confirmation, MB_OK, "query_context");
*/

  if (FLogOutput)
   FLogOutput->Lines->Add(AnsiString("Starting: ") + sql.c_str());
  sql = "insert into temp_id (q, id) select " + assigned_query_number + ", id from " + view_name;
  if (!ExecuteStatement(FTransaction, sql))
   PSay("Failed to create proxy data.\n" + sql, m_database_error, MB_OK, "query_context");
  else
  {
   if (FLogOutput)
    FLogOutput->Lines->Add(AnsiString("Done: ") + sql.c_str());
   answer = CreateView("select id from temp_id where q = " + assigned_query_number, set<string>(), "proxy");
  }
 }
 return answer;
}
//---------------------------------------------------------------------------
void query_context::CreateCache (TframeQueryGeneric* source_query, string result_view)
{
 if (source_query->Same(source_query))
 {
  FPreparedQueries[source_query] = result_view;
  source_query->WriteLog("Cache request accepted: " + result_view);
 }
 return;
}
//---------------------------------------------------------------------------
string query_context::FindCache (TframeQueryGeneric* query_to_match)
{
 string answer;
 if (query_to_match->Same(query_to_match))
  for (map<TframeQueryGeneric*, string>::iterator i = FPreparedQueries.begin(); i != FPreparedQueries.end(); i++)
   if (query_to_match->Same(i->first))
   {
    answer = i->second;
    i->first->WriteLog("Cache was useful: " + answer);
    query_to_match->WriteLog("Cache found: " + answer);
   }
 return answer;
}
//---------------------------------------------------------------------------
void query_context::Clear ()
{
 if (FTransaction && FTransaction->Active)
 {
  // the top has the last-to-delete stuff
  vector<set<string> > reverse_deletion_order;
  bool result = dependency_graph (FViewDependencies, reverse_deletion_order, true);
/* temporarily disabled for debugging! */
/*
  while (!reverse_deletion_order.empty())
  {
   set<string> to_delete = reverse_deletion_order.back();
   for (set<string>::iterator view_name = to_delete.begin(); view_name != to_delete.end(); view_name++)
    if (!ExecuteStatement(FTransaction, "drop view " + *view_name))
     PSay("Failed to drop view " + *view_name, m_database_error, MB_OK, "query_context", *view_name);
   reverse_deletion_order.pop_back();
  }

  // oh, and delete proxied data we inserted
  for (vector<string>::iterator i = FProxyQueries.begin(); i != FProxyQueries.end(); i++)
   if (!ExecuteStatement(FTransaction, "delete from temp_id where q = " + *i))
    PSay("Failed to delete proxy data " + *i, m_database_error, MB_OK, "query_context", *i);
*/

  for (map<string, TframeQuery_Whole*>::iterator i = FWholeQueries.begin(); i != FWholeQueries.end(); i++)
   i->second->CleanUp();
 }
 FViewDependencies.clear();
 FViewDefinitions.clear();
 FPreparedQueries.clear();
 FProxyQueries.clear();
}
//---------------------------------------------------------------------------
string query_context::DebugExplainView (string view_name)
{
 string message;

 list<string> to_learn_about;
 set<string> to_explain;
 list<string> to_say;

 to_learn_about.push_back(view_name);
 while (!to_learn_about.empty())
 {
  set<string> about_this_one = FViewDependencies[to_learn_about.back()];
  for (set<string>::iterator j = about_this_one.begin(); j != about_this_one.end(); j++)
   if (to_explain.find(*j) == to_explain.end())
    to_learn_about.push_front(*j);
  to_explain.insert(to_learn_about.back());
  to_say.push_front(to_learn_about.back());
  to_learn_about.pop_back();
 }

 message += "The final view is " + view_name + "\n";
 for (list<string>::iterator i = to_say.begin(); i != to_say.end(); i++)
  message += string("create view ") + *i + string(" as ") + FViewDefinitions[*i] + "\n";

 return message;
}
//---------------------------------------------------------------------------
string query_context::FindWhole (string table_name)
{
 // new as needed, add to FWholeQueries, and return result of Prepare(), otherwise return prepared result
 map<string, TframeQuery_Whole*>::iterator k = FWholeQueries.find(table_name);
 if (k == FWholeQueries.end())
 {
  TframeQuery_Whole* z = NULL;
  FORM_TRY
   z = new TframeQuery_Whole ((TframeQueryGeneric*)NULL);
   z->Context = this;
   z->SourceTableName = table_name.c_str();
  FORM_CATCH
  k = FWholeQueries.insert(pair<string, TframeQuery_Whole*>(table_name, z)).first;
 }
 // TODO: there is something fishy in here concerning whole_views being created for clipping, and having Prepare() called yet not being prepared on the next line!
 if (k != FWholeQueries.end())
 {
  if (!k->second->Prepared)
   k->second->Prepare();
  map<TframeQueryGeneric*, string>::iterator j = FPreparedQueries.find(k->second);
  if (j == FPreparedQueries.end())
  {
   PSay("A whole view was found for re-use, asked to prepare if not prepared, but then not found in the list of prepared queries.", m_debug, MB_OK, "");
   return "";
  } else
   return j->second;
 }
 PSay("No whole view found, one should have been created, but then it still wasn't found.", m_debug, MB_OK, "");
 return "";
}
//---------------------------------------------------------------------------
bool query_context::IsProxied (string view_name)
{
 for (vector<string>::iterator i = FProxyQueries.begin(); i != FProxyQueries.end(); i++)
  if (*i == view_name)
   return true;
 return false;
}
//---------------------------------------------------------------------------

