#include "query_factories.h"

// utility
#include "codeQueryMagic.h"
#include "codeQueryMagicClosable.h"

// generic-ish
#include "codeQuery_All.h"
#include "codeQuery_Any.h"
#include "codeQuery_None.h"
#include "codeQuery_Some.h"
#include "codeQuery_Random.h"
#include "codeQuery_Whole.h"
#include "codeQuery_Has.h"
#include "codeQuery_PointsTo.h"

#include "codeQuery_ByField.h"
#include "codeQuery_ByResultSet.h"
#include "codeQuery_CustomSql.h"

// specific
#include "codeQuery__PeopleByAge.h"
#include "codeQuery__PeopleByGender.h"
#include "codeQuery__PeopleByAffiliation.h"
#include "codeQuery__PeopleByFirstDiagnosed.h"
#include "codeQuery__PeopleByFirstProcedured.h" // yes -- i fucking know. "conditions / diagnosed" ... "procedures / treated" ... it's all funky.
#include "codeQuery__PeopleByAngelaAudit.h"
#include "codeQuery__PeopleByInsurance.h"
#include "codeQuery__PeopleByMetabolicGoalsReached.h"
#include "codeQuery__PeopleAlive.h"
#include "codeQuery__PeopleByLatestMeasurement.h"
#include "codeQuery__PeopleByMeasurementImprovement.h"

#include "codeQuery__ConditionsByDate.h"
#include "codeQuery__ConditionsByCode.h"

#include "codeQuery__TasksByDate.h"
#include "codeQuery__TasksByMethodOfContact.h"

#include "codeQuery__ProceduresByDate.h"
#include "codeQuery__ProceduresByCode.h"

#include "codeQuery__ObsResponseSetsByResponse.h"

// multi-table queries, may be re-usable with slight modifications
// things involving dates seem re-usable but we don't use consistent date field names everywhere, one vs. two fields, etc.
// plus, we might have several date fields in one table, and we'd need to know which one, and ... ugh.
// but for group_id ... it's fairly safe. as are any finger-pointing fields. and other such things.
#include "codeQuery__ByGroup.h"

void RegisterQueryFactories (set_query_factory_registry & factories)
{
 // utility
 // note: we simply don't provide an 'appropriate' function ...
 // this is because xAutoLoad tries to check the appropriateness of queries, if possible -- because it's not, it loads these
 // although 'Appropriate' returns false, and would prevent them from appearing in drop-downs ... yeah. ugh.
 factories.AddFactory("", "TframeQueryMagic", NULL, TframeQueryMagic::Create);
 factories.AddFactory("", "TframeQueryMagicClosable", NULL, TframeQueryMagicClosable::Create);

 // generic-ish
 factories.AddFactory("All possible $", "TframeQuery_Whole", TframeQuery_Whole::Appropriate, TframeQuery_Whole::Create);
 factories.AddFactory("$ that match all of ...", "TframeQuery_All", TframeQuery_All::Appropriate, TframeQuery_All::Create);
 factories.AddFactory("$ that match any of ...", "TframeQuery_Any", TframeQuery_Any::Appropriate, TframeQuery_Any::Create);
 factories.AddFactory("$ that match none of ...", "TframeQuery_None", TframeQuery_None::Appropriate, TframeQuery_None::Create);
 factories.AddFactory("$ that match some of ...", "TframeQuery_Some", TframeQuery_Some::Appropriate, TframeQuery_Some::Create);
 factories.AddSeparator();
 factories.AddFactory("Random set of $ among ...", "TframeQuery_Random", TframeQuery_Random::Appropriate, TframeQuery_Random::Create);
 factories.AddFactory("$ from a saved result set ...", "TframeQuery_ByResultSet", TframeQuery_ByResultSet::Appropriate, TframeQuery_ByResultSet::Create);
 factories.AddSeparator();
 factories.AddFactory("$ that have ...", "TframeQuery_Has", TframeQuery_Has::Appropriate, TframeQuery_Has::Create);
 factories.AddFactory("$ that point to a ...", "TframeQuery_PointsTo", TframeQuery_PointsTo::Appropriate, TframeQuery_PointsTo::Create);
 factories.AddFactory("$ by field ...", "TframeQuery_ByField", TframeQuery_ByField::Appropriate, TframeQuery_ByField::Create);
 factories.AddFactory("$ by SQL ...", "TframeQuery_CustomSql", TframeQuery_CustomSql::Appropriate, TframeQuery_CustomSql::Create);
 factories.AddSeparator();
 // still trying to find an ideal sort-order (with separators) for the above

 // specific
 // ... people
 factories.AddFactory("$ alive on ...", "TframeQuery__PeopleAlive", TframeQuery__PeopleAlive::Appropriate, TframeQuery__PeopleAlive::Create);
 factories.AddFactory("$ by age ...", "TframeQuery__PeopleByAge", TframeQuery__PeopleByAge::Appropriate, TframeQuery__PeopleByAge::Create);
 // factories.AddFactory("$ by gender ...", "TframeQuery__PeopleByGender", TframeQuery__PeopleByGender::Appropriate, TframeQuery__PeopleByGender::Create);
 factories.AddFactory("$ by tribe ...", "TframeQuery__PeopleByAffiliation", TframeQuery__PeopleByAffiliation::Appropriate, TframeQuery__PeopleByAffiliation::Create);
 factories.AddFactory("$ by insurance coverage ...", "TframeQuery__PeopleByInsurance", TframeQuery__PeopleByInsurance::Appropriate, TframeQuery__PeopleByInsurance::Create);
 factories.AddFactory("$ first diagnosed with ...", "TframeQuery__PeopleByFirstDiagnosed", TframeQuery__PeopleByFirstDiagnosed::Appropriate, TframeQuery__PeopleByFirstDiagnosed::Create);
 factories.AddFactory("$ first treated with ...", "TframeQuery__PeopleByFirstProcedured", TframeQuery__PeopleByFirstProcedured::Appropriate, TframeQuery__PeopleByFirstProcedured::Create);
 // factories.AddFactory("$ with improved metabolic control ...", "TframeQuery__PeopleByMetabolicGoalsReached", TframeQuery__PeopleByMetabolicGoalsReached::Appropriate, TframeQuery__PeopleByMetabolicGoalsReached::Create);
 factories.AddFactory("$ by measurement ...", "TframeQuery__PeopleByLatestMeasurement", TframeQuery__PeopleByLatestMeasurement::Appropriate, TframeQuery__PeopleByLatestMeasurement::Create);
 factories.AddFactory("$ by measurement improvement ...", "TframeQuery__PeopleByMeasurementImprovement", TframeQuery__PeopleByMeasurementImprovement::Appropriate, TframeQuery__PeopleByMeasurementImprovement::Create);
 factories.AddFactory("$ from the DMS Audit ...", "TframeQuery__PeopleByAngelaAudit", TframeQuery__PeopleByAngelaAudit::Appropriate, TframeQuery__PeopleByAngelaAudit::Create);

 // ... conditions
 // factories.AddFactory("$ by date ...", "TframeQuery__ConditionsByDate", TframeQuery__ConditionsByDate::Appropriate, TframeQuery__ConditionsByDate::Create);
 factories.AddFactory("$ by code ...", "TframeQuery__ConditionsByCode", TframeQuery__ConditionsByCode::Appropriate, TframeQuery__ConditionsByCode::Create);

 // ... tasks
 // factories.AddFactory("$ by date ...", "TframeQuery__TasksByDate", TframeQuery__TasksByDate::Appropriate, TframeQuery__TasksByDate::Create);
 // factories.AddFactory("$ by method of contact ...", "TframeQuery__TasksByMethodOfContact", TframeQuery__TasksByMethodOfContact::Appropriate, TframeQuery__TasksByMethodOfContact::Create);

 // ... multi-table
 // factories.AddFactory("$ by department / group ...", "TframeQuery__ByGroup", TframeQuery__ByGroup::Appropriate, TframeQuery__ByGroup::Create);

 // ... procedures
 // factories.AddFactory("$ by date ...", "TframeQuery__ProceduresByDate", TframeQuery__ProceduresByDate::Appropriate, TframeQuery__ProceduresByDate::Create);
 factories.AddFactory("$ by code ...", "TframeQuery__ProceduresByCode", TframeQuery__ProceduresByCode::Appropriate, TframeQuery__ProceduresByCode::Create);

 // ... observations
 factories.AddFactory("$ by value ...", "TframeQuery__ObsResponseSetsByResponse", TframeQuery__ObsResponseSetsByResponse::Appropriate, TframeQuery__ObsResponseSetsByResponse::Create);
}
