#include "query_factory_registry.h"
#include "codeQueryGeneric.h"

set_query_factory_registry query_factories;

vector<q_factory_info> set_query_factory_registry::GetFactories (string source_table_name)
{
 vector<q_factory_info> answer;

 // use callbacks to determine appropriateness
 for (vector<q_factory_info>::iterator i = factories.begin(); i != factories.end(); i++)
  if (i->is_separator || (i->appropriate_pointer && i->appropriate_pointer(source_table_name)))
   answer.push_back(*i);

 return answer;
}

void set_query_factory_registry::AddSeparator ()
{
 q_factory_info temp;

 temp.is_separator        = true;
 temp.description         = "";
 temp.serialization_tag   = "";
 temp.appropriate_pointer = NULL;
 temp.create_pointer      = NULL;

 factories.push_back(temp);
}

void set_query_factory_registry::AddFactory (string description, string serialization_tag, q_factory_appropriate appropriate_pointer, q_factory_create create_pointer)
{
 q_factory_info temp;

 temp.is_separator        = false;
 temp.description         = description;
 temp.serialization_tag   = serialization_tag;
 temp.appropriate_pointer = appropriate_pointer;
 temp.create_pointer      = create_pointer;

 // uniqueness check -- consider the "create_pointer" to be the PK (close enough?)
 for (vector<q_factory_info>::iterator i = factories.begin(); i != factories.end(); i++)
  if (i->create_pointer == create_pointer)
   return;

 // add if not otherwise found
 factories.push_back(temp);
}

q_factory_info set_query_factory_registry::GetFactoryByTag (string serialization_tag)
{
 for (vector<q_factory_info>::iterator i = factories.begin(); i != factories.end(); i++)
  if (!i->is_separator && i->serialization_tag == serialization_tag)
   return *i;
 return q_factory_info();
}

string set_query_factory_registry::GetTagByFactory (q_factory_create create_pointer)
{
 for (vector<q_factory_info>::iterator i = factories.begin(); i != factories.end(); i++)
  if (!i->is_separator && i->create_pointer == create_pointer)
   return i->serialization_tag;
 return "";
}

