#ifndef repeating_h
#define repeating_h

#include "util.h"
#include "allsymbols.h"
//-------------------------------------------------------------

// meaning of r_any
//   r_any should cause any of these specs to -not- change the range they're
//   given to refine.
// meaning of r_every
//   r_every should limit the range given to the range surrounding the given
//   date
// meaning of r_on
//   r_on should limit the range given to the range surrounding the specific
//   date, regardless of whether or not the given date fits within that zone
// meaning of r_first
//   r_first should attempt to find the first matching item within the range
//   regardless of whether or not the given date fits within that zone
// meaning of r_last
//   r_last should attempt to find the last matching item within the range
//   regardless of whether or not the given date fits within that zone

class date_range
{
 private:
  Pdate from;
  Pdate until;
 public:
  date_range ();
  date_range (Pdate, Pdate);
  bool  Contains (Pdate);
  bool  Contains (date_range);
  bool  IsDayType (day_items);
  bool  IsWeekType (week_items);
  bool  IsMonthType (month_items);
  bool  IsYearType (year_items);
  int   DayLength ();
  void  Truncate (date_range);
  Pdate From () {return from;};
  Pdate Until (){return until;};
};

class day_spec
{
 private:
  repeat_items repeats;
  int          specific;
  day_items    matching;
 public:
  day_spec ();
  day_spec (repeat_items, int, day_items);
  date_range   Refine (date_range, Pdate);
  repeat_items Repeats() {return repeats;};
  int          Specific() {return specific;};
  day_items    Matching() {return matching;};
  void Load (string);
  string Save ();
};

class week_spec
{
 private:
  repeat_items repeats;
  int          specific;
  week_items   matching;
 public:
  week_spec ();
  week_spec (repeat_items, int, week_items);
  date_range   Refine (date_range, Pdate);
  repeat_items Repeats() {return repeats;};
  int          Specific() {return specific;};
  week_items   Matching() {return matching;};
  void Load (string);
  string Save ();
};

class month_spec
{
 private:
  repeat_items repeats;
  int          specific;
  month_items  matching;
 public:
  month_spec ();
  month_spec (repeat_items, int, month_items);
  date_range   Refine (date_range, Pdate);
  repeat_items Repeats() {return repeats;};
  int          Specific() {return specific;};
  month_items  Matching() {return matching;};
  void Load (string);
  string Save ();
};

class year_spec
{
 private:
  repeat_items repeats;
  int          specific;
  year_items   matching;
 public:
  year_spec ();
  year_spec (repeat_items, int, year_items);
  date_range   Refine (date_range, Pdate);
  repeat_items Repeats() {return repeats;};
  int          Specific() {return specific;};
  year_items   Matching() {return matching;};
  void Load (string);
  string Save ();
};

template <class T>
class repeat_spec
{
 private:
  day_spec     when_day;
  week_spec    when_week;
  month_spec   when_month;
  year_spec    when_year;
  Pdate        when_start;
  Pdate        when_stop;
  T            when_cargo;
 public:
  repeat_spec ()
  {
   when_start.year = 2000;
   when_start.month = 1;
   when_start.day = 1;
   when_stop.year = 2010;
   when_stop.month = 12;
   when_stop.day = 31;
  };

  repeat_spec (day_spec p_when_day, week_spec p_when_week,
               month_spec p_when_month, year_spec p_when_year,
               Pdate p_when_start, Pdate p_when_stop, T p_when_cargo)
  {
   when_day   = p_when_day;
   when_week  = p_when_week;
   when_month = p_when_month;
   when_year  = p_when_year;
   when_start = p_when_start;
   when_stop  = p_when_stop;
   when_cargo = p_when_cargo;
  };

  repeat_spec (day_spec p_when_day, week_spec p_when_week,
               month_spec p_when_month, year_spec p_when_year,
               Pdate p_when_start, Pdate p_when_stop)
  {
   when_day   = p_when_day;
   when_week  = p_when_week;
   when_month = p_when_month;
   when_year  = p_when_year;
   when_start = p_when_start;
   when_stop  = p_when_stop;
  };

  repeat_spec (day_spec p_when_day, week_spec p_when_week,
               month_spec p_when_month, year_spec p_when_year)
  {
   when_day   = p_when_day;
   when_week  = p_when_week;
   when_month = p_when_month;
   when_year  = p_when_year;
   when_start.year = 2000;
   when_start.month = 1;
   when_start.day = 1;
   when_stop.year = 2010;
   when_stop.month = 12;
   when_stop.day = 31;
  };

  repeat_spec (string p_serialized, Pdate begin_date, Pdate end_date)
  {
   unsigned int left;
   unsigned int right;

   left = 0;
   right = p_serialized.find("|", left);
   when_day.Load(p_serialized.substr(left, right - left));

   left = right + 1;
   right = p_serialized.find("|", left);
   when_week.Load(p_serialized.substr(left, right - left));

   left = right + 1;
   right = p_serialized.find("|", left);
   when_month.Load(p_serialized.substr(left, right - left));

   left = right + 1;
//   right = p_serialized.find("|", left);
   when_year.Load(p_serialized.substr(left));

   when_start = end_date;
   when_stop = begin_date;

//   left = right + 1;
//   right = p_serialized.find("|", left);
//   when_start.InDate (p_serialized.substr(left, right - left), ymd | yyyy);

//   left = right + 1;
//   when_stop.InDate (p_serialized.substr(left), ymd | yyyy);
  };

  T& Cargo () {return when_cargo;};

  vector<Pdate> AllMatches (date_range p_search_area)
  {
   vector<Pdate> matches;
   date_range level_outer = p_search_area;
   date_range level_valid(when_start, when_stop);
   level_outer.Truncate (level_valid);

   Pdate i_test = level_outer.From();
   date_range level_year  = when_year.Refine(level_outer, i_test);
   date_range level_month = when_month.Refine(level_year, i_test);
   date_range level_week  = when_week.Refine(level_month, i_test);
   date_range level_day   = when_day.Refine(level_week, i_test);
   while (i_test <= level_outer.Until())
   {
    if (!level_year.Contains(i_test))
    {
     level_year = when_year.Refine (level_outer, i_test);
     level_month = when_month.Refine (level_year, i_test);
     level_week = when_week.Refine (level_month, i_test);
     level_day = when_day.Refine (level_week, i_test);
    } else
    if (!level_month.Contains(i_test))
    {
     level_month = when_month.Refine (level_year, i_test);
     level_week = when_week.Refine (level_month, i_test);
     level_day = when_day.Refine (level_week, i_test);
    } else
    if (!level_week.Contains(i_test))
    {
     level_week = when_week.Refine (level_month, i_test);
     level_day = when_day.Refine (level_week, i_test);
    } else
    if (!level_day.Contains(i_test))
    {
     level_day = when_day.Refine (level_week, i_test);
    }

    if (level_day.Contains (i_test))
     matches.push_back (i_test);
    i_test++;
   }
   return matches;
  }

// have a range
//  if it's a day level, and the date matches, add to list, increase day
//  if date no longer in range, need to increase the range to fit... which
//  means asking for a new day-refinement... until we reach the end of week,
//  when we need to leave the subset we were in and get a new week to work
//  with, etc.

  bool Matches (Pdate p_search)
  {
   date_range search_area(when_start, when_stop);
   if (search_area.Contains(p_search))
   {
    search_area = when_year.Refine(search_area, p_search);
    if (search_area.Contains(p_search))
    {
     search_area = when_month.Refine(search_area, p_search);
     if (search_area.Contains(p_search))
     {
      search_area = when_week.Refine(search_area, p_search);
      if (search_area.Contains(p_search))
      {
       search_area = when_day.Refine(search_area, p_search);
       if (search_area.Contains(p_search))
       {
        return true;
       }
      }
     }
    }
   }
   return false;
  };

  void Load(string p_serialized)
  {
   unsigned int left;
   unsigned int right;

   left = 0;
   right = p_serialized.find("|", left);
   when_day.Load(p_serialized.substr(left, right - left));

   left = right + 1;
   right = p_serialized.find("|", left);
   when_week.Load(p_serialized.substr(left, right - left));

   left = right + 1;
   right = p_serialized.find("|", left);
   when_month.Load(p_serialized.substr(left, right - left));

   left = right + 1;
   right = p_serialized.find("|", left);
   when_year.Load(p_serialized.substr(left, right - left));

   left = right + 1;
   right = p_serialized.find("|", left);
   when_start.InDate (p_serialized.substr(left, right - left), ymd | yyyy);

   left = right + 1;
   when_stop.InDate (p_serialized.substr(left), ymd | yyyy);
  };

  void Load(string p_serialized, Pdate begin_date, Pdate end_date)
  {
   unsigned int left;
   unsigned int right;

   left = 0;
   right = p_serialized.find("|", left);
   when_day.Load(p_serialized.substr(left, right - left));

   left = right + 1;
   right = p_serialized.find("|", left);
   when_week.Load(p_serialized.substr(left, right - left));

   left = right + 1;
   right = p_serialized.find("|", left);
   when_month.Load(p_serialized.substr(left, right - left));

   left = right + 1;
//   right = p_serialized.find("|", left);
   when_year.Load(p_serialized.substr(left));

   when_start = begin_date;
   when_stop = end_date;

//   left = right + 1;
//   right = p_serialized.find("|", left);
//   when_start.InDate (p_serialized.substr(left, right - left), ymd | yyyy);

//   left = right + 1;
//   when_stop.InDate (p_serialized.substr(left), ymd | yyyy);
  };

  string Save()
  {
   string serialized = "";
   serialized += when_day.Save();
   serialized += "|";
   serialized += when_week.Save();
   serialized += "|";
   serialized += when_month.Save();
   serialized += "|";
   serialized += when_year.Save();
   serialized += "|";
   serialized += when_start.OutDate(ymd | yyyy | left_zeros);
   serialized += "|";
   serialized += when_stop.OutDate(ymd | yyyy | left_zeros);
   return serialized;
  };

  string SaveWithoutDates()
  {
   string serialized = "";
   serialized += when_day.Save();
   serialized += "|";
   serialized += when_week.Save();
   serialized += "|";
   serialized += when_month.Save();
   serialized += "|";
   serialized += when_year.Save();
   return serialized;
  };

  T& GetCargo ()
  {
   return when_cargo;
  };

  void SetCargo (const T& p_new_cargo)
  {
   when_cargo = p_new_cargo;
  };

  day_spec Day()
  {
   return when_day;
  };

  week_spec Week()
  {
   return when_week;
  };

  month_spec Month()
  {
   return when_month;
  };

  year_spec Year()
  {
   return when_year;
  };

  Pdate Start()
  {
   return when_start;
  };

  Pdate Stop()
  {
   return when_stop;
  };
};

// function definition for a function that takes a string that is a repeating string,
// and returns ... a relevant date. for use in listviews and stuff. quite neat, really!

string RepeatingString (string p_rep); // takes ymd format

#endif