//DEIMOS
#ifndef sqlfunctions
#define sqlfunctions

#include <string>
#include <Classes.hpp>
#include <Controls.hpp>
#include <StdCtrls.hpp>
#include <Db.hpp>
#include <IBCustomDataSet.hpp>
#include <IBDatabase.hpp>
#include <IBSQL.hpp>
#include <map>
#include <vector>
#include <set>
#include <list>
#ifdef use_tcpip_messaging
#include "tcpip_messaging.h"
#endif
#ifdef use_prometheus_util
#include "prometheus_util.h"
#endif
#include "util.h"
#include "allsymbols.h"

using namespace std;

bool ConnectToDatabase(TIBDatabase *dbDatabase, string server, string user_name, string password);
bool LockRecordForUpdate (TIBTransaction* dbTransaction, string tableName, string id);
bool ExecuteUpdateStatement(TIBTransaction* dbTransaction, string tableName, map<string, string> fieldMap, map<string, int> field_options = map<string, int>(), bool show_error = false);
bool ExecuteUpdateStatement(TIBDatabase* dbDatabase, string tableName, map<string, string> fieldMap, map<string, int> field_options = map<string, int>(), bool show_error = false);
bool ExecuteInsertStatement(TIBTransaction* dbTransaction, string tableName, map<string, string> fieldMap, map<string, int> field_options = map<string, int>(), bool show_error = false);
bool ExecuteInsertStatement(TIBDatabase* dbDatabase, string tableName, map<string, string> fieldMap, map<string, int> field_options = map<string, int>(), bool show_error = false);
string GetNextId(TIBTransaction* dbTransaction, string tableName);
string GetNextId(TIBDatabase* dbDatabase, string tableName);
string GetOneField(TIBTransaction * dbTransaction, string sqlstmt);
string GetOneField(TIBDatabase * dbDatabase, string sqlstmt);
bool FillSKVTable (TIBTransaction* dbTransaction, string sql, map<string, map<string, string> > & section_key_value_map);
string GetPersonName(TIBTransaction* dbTransaction, string person_id, int order = last_first);
string GetPersonName(TIBDatabase * dbDatabase, string person_id, int order = last_first);
string GetUserName(TIBDatabase * dbDatabase, string user_id, int order);
string GetUserName(TIBTransaction * dbTransaction, string user_id, int order);
bool ExecuteStatement(TIBTransaction * dbTransaction, string sqlstring);
bool ExecuteStatement(TIBDatabase * dbDatabase, string sqlstring);
bool SqlToMap(TIBTransaction * dbTransaction, string sql, map<string, string> &field_map);
bool SqlToMap(TIBDatabase * dbDatabase, string sql, map<string, string> &field_map);
bool SqlToListMap(TIBTransaction * dbTransaction, string sql, list<map<string, string> > &field_list);
bool SqlToListMap(TIBDatabase * dbDatabase, string sql, list<map<string, string> > &field_list);
bool SqlToVecMap(TIBTransaction * dbTransaction, string sql, vector<map<string, string> > &field_vec, bool preserve_fieldname_case = false);
bool SqlToVecMap(TIBDatabase * dbDatabase, string sql, vector<map<string, string> > &field_vec, bool preserve_fieldname_case = false);
bool SqlToMapMap(TIBTransaction * dbTransaction, string sql, map<string, map<string, string> > &field_map, bool preserve_fieldname_case = false);
bool SqlToMapMap(TIBDatabase * dbDatabase, string sql, map<string, map<string, string> > &field_map, bool preserve_fieldname_case = false);
bool SqlToSet(TIBTransaction * dbTransaction, string sql, set<string> &field_set);
bool SqlToSet(TIBDatabase * dbDatabase, string sql, set<string> &field_set);
bool SqlToList(TIBTransaction * dbTransaction, string sql, list<string> &result_list);
bool SqlToList(TIBDatabase * dbDatabase, string sql, list<string> &result_list);
bool SqlToVec(TIBTransaction * dbTransaction, string sql, vector<string> &result_vec);
bool SqlToVec(TIBDatabase * dbDatabase, string sql, vector<string> &result_vec);
bool SqlToCommaString(TIBTransaction * dbTransaction, string sql, string &result_string);
bool SqlToCommaString(TIBDatabase * dbDatabase, string sql, string &result_string);
bool SqlToValueMap(TIBTransaction * dbTransaction, string sql, map<string, string> &field_map);
bool SqlToValueMap(TIBDatabase * dbDatabase, string sql, map<string, string> &field_map);
bool SqlToMapStringVec(TIBTransaction * dbTransaction, string sql, map<string, vector<string> > &field_map);
bool SqlToMapStringVec(TIBDatabase * dbDatabase, string sql, map<string, vector<string> > &field_map);
bool SqlToMapStringList(TIBTransaction * dbTransaction, string sql, map<string, list<string> > &field_map);
bool SqlToMapStringlist(TIBDatabase * dbDatabase, string sql, map<string, list<string> > &field_map);
bool ExecutePreparedInsert (TIBTransaction * dbTransaction, string tableName, vector<string> &field_names, list <vector<string> > & insert_data, map<string, int> field_options = map<string, int>());
bool ExecutePreparedInsert (TIBDatabase * dbDatabase, string tableName, vector<string> &field_names, list <vector<string> > & insert_data, map<string, int> field_options = map<string, int>());
bool StdTransaction (TIBTransaction * dbTransaction, TIBDatabase * dbDatabase, bool autostart = true);
bool SpyTransaction (TIBTransaction * dbTransaction, TIBDatabase * dbDatabase, bool autostart = true);
void ResetLastSqlError ();
bool ForeignKeyLookupByField (TIBTransaction * dbTransaction, string local_table_name, string local_field_name, string & foreign_table_name, string & foreign_field_name);
bool ForeignKeyLookupByField (TIBDatabase * dbDatabase, string local_table_name, string local_field_name, string & foreign_table_name, string & foreign_field_name);
bool ForeignKeyLookupByConstraint (TIBTransaction * dbTransaction, string constraint_name, string & local_table_name, string & local_field_name, string & foreign_table_name, string & foreign_field_name);
bool ForeignKeyLookupByConstraint (TIBDatabase * dbDatabase, string constraint_name, string & local_table_name, string & local_field_name, string & foreign_table_name, string & foreign_field_name);
bool GetPrimaryKey (TIBTransaction * dbTransaction, string fk_table, string fk_field, string & pk_table, vector<string> & pk_fields);
bool GetPrimaryKey (TIBDatabase * dbDatabase, string fk_table, string fk_field, string & pk_table, vector<string> & pk_fields);
bool GetForeignKeys (TIBTransaction * dbTransaction, string pk_table, string pk_field, vector<map<string, string> > & fks);
bool GetForeignKeys (TIBDatabase * dbDatabase, string pk_table, string pk_field, vector<map<string, string> > & fks);
bool GetFieldsFromTable (TIBDatabase * dbDatabase, string table, vector<string>& fields);
bool GetFieldsFromTable (TIBTransaction * dbTransaction, string table, vector<string>& fields);
bool GetFieldSetFromTable(TIBDatabase *dbTransaction, string table, set<string> &field_set);     // jhaley 20110107: GetFieldSetFromTable for codeRecordGeneric enhancements
bool GetFieldSetFromTable(TIBTransaction *dbTransaction, string table, set<string> &field_set);
bool FieldSetHasField(set<string> &field_set, const string &field);
bool GetTablePrimaryKey (TIBDatabase * dbDatabase, string table, vector<string>& pk);
bool GetTablePrimaryKey (TIBTransaction * dbDatabase, string table, vector<string>& pk);
bool CreateInsertMap (TIBTransaction * dbTransaction, string table, map<string, string>& ins_map);
bool CreateInsertMap (TIBDatabase * dbDatabase, string table, map<string, string>& ins_map);
bool ExecutePreparedSetInsert (TIBTransaction * dbTransaction, string table_name, string field_name, const set<string> & insert_data, map<string, int> field_options = map<string, int>());
bool ExecutePreparedSetInsert (TIBDatabase * dbDatabase, string table_name, string field_name, const set<string> & insert_data, map<string, int> field_options = map<string, int>());
//string GetSqlPlan (TIBTransaction * dbTransaction, string sql_statement);
bool GetSqlPlan (TIBTransaction * dbTransaction, string sql_statement, string &plan);
bool GetSqlPlan (TIBDatabase * dbDatabase, string sql_statement, string &plan);
string SqlOptionalBetween (string field, string a, string b);
string TranslateSqlPlan(TIBTransaction* dbTransaction, string plan);
bool IsIndexed(TIBTransaction * dbTransaction, string table_name, string field_name);
string SafeInClause(string field, vector<string> values, string result_if_blank = "0=1", bool quote_values = false, unsigned int max_values_in_group = 1000);
bool DataSetToCSV(TDataSet *DS, string FN, bool headers = true);

//--------------------------------------------------------------------------

class easy_streaming_dataset
{
 // data
 private:
  TIBDatabase *    _database;
  TIBTransaction * _transaction;
  TIBDataSet *     _dataset;
  TStringList *    _stringlist;
  bool             _delete_transaction;
  unsigned int     _row_count;
  int              _debug_level; // jhaley 20110304

 // functions
  void SetTransaction (TIBTransaction * dbTransaction);
  void SetDatabase    (TIBDatabase * dbDatabase);

 public:
  easy_streaming_dataset (TIBTransaction * dbTransaction);
  easy_streaming_dataset (TIBDatabase * dbDatabase);
  easy_streaming_dataset (TIBTransaction * dbTransaction, string sql, int p_debug_level = 1);
  easy_streaming_dataset (TIBDatabase * dbDatabase, string sql, int p_debug_level = 1);
  ~easy_streaming_dataset ();

  void SetSql (string sql);
  void SetDebugLevel (int i); // jhaley
  bool Done ();
  void Next ();
  void Reset ();
  bool GetRow (map<string, string> & field_map);
  bool GetRow (vector<string> & field_vector);
  unsigned int GetRowNumber ();

 template<class T>
 bool easy_streaming_dataset::GetRowsWhile (list<map<string, string> > & field_maps, T predicate)
 {
  map<string, string> temp;

  if (!_dataset)
   throw(NULL);

  field_maps.clear();
  while (!Done())
  {
   if (!GetRow(temp))
    return false;
   if (predicate(temp))
    field_maps.push_back(temp);
   else
    return true;
   Next();
  }

  return false;
 }

 template<class T>
 bool easy_streaming_dataset::GetRowsWhile (list<vector<string> > & field_vectors, T predicate)
 {
  vector<string> temp;

  if (!_dataset)
   throw(NULL);

  field_vectors.clear();
  while (!Done())
  {
   if (!GetRow(temp))
    return false;
   if (predicate(temp))
    field_vectors.push_back(temp);
   else
    return true;
   Next();
  }

  return false;
 }

 template<class T>
 bool easy_streaming_dataset::AdvanceWhileMap (T predicate)
 {
  map<string, string> temp;

  if (!_dataset)
   throw(NULL);

  while (!Done())
  {
   if (!GetRow(temp))
    return false;
   if (!predicate(temp))
    return true;
   Next();
  }

  return false;
 }

 template<class T>
 bool easy_streaming_dataset::AdvanceWhileVector (T predicate)
 {
  vector<string> temp;

  if (!_dataset)
   throw(NULL);

  while (!Done())
  {
   if (!GetRow(temp))
    return false;
   if (!predicate(temp))
    return true;
   Next();
  }

  return false;
 }
};

template<class T>
class streaming_dataset_map_value_predicate : public unary_function<map<string, string> &, bool>
{
 public:
  string _field_name;
  T      _value_operator;
  string _field_value;

 streaming_dataset_map_value_predicate (string field_name, T value_operator, string field_value)
  : _field_name (field_name), _value_operator (value_operator), _field_value (field_value) {}
 bool operator () (map<string, string> & row)
 {
  return _value_operator(row[_field_name], _field_value);
 }
};

template<class J, class K>
class streaming_dataset_map_value_predicate_and : public unary_function<map<string, string> &, bool>
{
 public:
  J _a;
  K _b;

 streaming_dataset_map_value_predicate_and (J a, K b)
  : _a (a), _b (b) {}
 bool operator () (map<string, string> & row)
 {
  return (_a(row) && _b(row));
 }
};

template<class J, class K>
class streaming_dataset_map_value_predicate_or : public unary_function<map<string, string> &, bool>
{
 public:
  J _a;
  K _b;

 streaming_dataset_map_value_predicate_or (J a, K b)
  : _a (a), _b (b) {}
 bool operator () (map<string, string> & row)
 {
  return (_a(row) || _b(row));
 }
};

template<class T>
class streaming_dataset_vector_value_predicate : public unary_function<vector<string> &, bool>
{
 public:
  unsigned int _field_index;
  T            _value_operator;
  string       _field_value;

 streaming_dataset_vector_value_predicate (unsigned int field_index, T value_operator, string field_value)
  : _field_index (field_index), _value_operator (value_operator), _field_value (field_value) {}
 bool operator () (vector<string> & row)
 {
  return _value_operator(row[_field_name], _field_value);
 }
};

// useful for the above stuff
struct less_strings_as_ints : public binary_function<string, string, bool>
{
 bool operator () (string x, string y)
 {
  return StringToInt(x) < StringToInt(y);
 }
};

struct great_strings_as_ints : public binary_function<string, string, bool>
{
 bool operator () (string x, string y)
 {
  return StringToInt(x) > StringToInt(y);
 }
};

struct equal_to_strings_as_ints : public binary_function<string, string, bool>
{
 bool operator () (string x, string y)
 {
  return StringToInt(x) == StringToInt(y);
 }
};

class sql_error
{
 public:
  int ibx_error;
  int sql_error;
  string message;
  sql_error () {Reset();}
  sql_error (EIBError & error)
  {
   ibx_error = error.IBErrorCode;
   sql_error = error.SQLCode;
   message   = error.Message.c_str();
  }
  void Reset ()
  {
   ibx_error = 0;
   sql_error = 0;
   message   = "No error since last reset.";
  }
  bool IsLockConflict () {return (ibx_error == 335544345L);}
  bool GetLockInfo (TIBTransaction * dbTransaction, string & local_table_name, string & local_field_name, string & foreign_table_name, string & foreign_field_name)
  {
   unsigned int dbl_quote_1 = message.find('"');
   if (dbl_quote_1 != string::npos)
   {
    unsigned int dbl_quote_2 = message.find('"', dbl_quote_1 + 1);
    if (dbl_quote_2 != string::npos)
    {
     string constraint_name = message.substr(dbl_quote_1 + 1, dbl_quote_2 - (dbl_quote_1 + 1));
     return ForeignKeyLookupByConstraint (dbTransaction, constraint_name, local_table_name, local_field_name, foreign_table_name, foreign_field_name);
    } else
     return false;
   } else
    return false;
  }
  bool GetLockInfo (TIBDatabase * dbDatabase, string & local_table_name, string & local_field_name, string & foreign_table_name, string & foreign_field_name)
  {
   unsigned int dbl_quote_1 = message.find('"');
   if (dbl_quote_1 != string::npos)
   {
    unsigned int dbl_quote_2 = message.find('"', dbl_quote_1 + 1);
    if (dbl_quote_2 != string::npos)
    {
     string constraint_name = message.substr(dbl_quote_1 + 1, dbl_quote_2 - (dbl_quote_1 + 1));
     return ForeignKeyLookupByConstraint (dbDatabase, constraint_name, local_table_name, local_field_name, foreign_table_name, foreign_field_name);
    } else
     return false;
   } else
    return false;
  }
};
//---------------------------------------------------------------------------
extern sql_error last_sql_lib_error;
//---------------------------------------------------------------------------
#endif
