#include "tcpip_messaging.h"
#ifdef _DEBUG
#include "util.h"
#endif

//---------------------------------------------------------------------------
classTcpipMessaging* tcpip_message_router = NULL; // default one
//---------------------------------------------------------------------------

classTcpipMessaging::classTcpipMessaging (TClientSocket * p_client_socket)
{
  client_socket = p_client_socket;
}

void classTcpipMessaging::SendMessageToAny (string map_key, string map_value, string channel, string message_text)
{
  if(client_socket && client_socket->Active)
  {
    string to_send = "any~" + map_key + "~" + map_value + "~" + channel + "~" + message_text;
    #ifdef _DEBUG
    LogIt(("dbg__" + channel + ".log").c_str(), to_send);
    #endif
    outbound_messages.AddMessage(to_send);
    outbound_messages.AttemptWrite(client_socket);
  } else
  {
   // TODO: for loopback server, would need to track our own properties to know if our message is destined to us?
  }
}

void classTcpipMessaging::SendMessageToAll (string channel, string message_text)
{
  if(client_socket && client_socket->Active)
  {
    string to_send = "all~" + channel + "~" + message_text;
    #ifdef _DEBUG
    LogIt(("dbg__" + channel + ".log").c_str(), to_send);
    #endif
    outbound_messages.AddMessage(to_send);
    outbound_messages.AttemptWrite(client_socket);
  } else
  {
   // TODO -c HACK : if server is missing in action, write back to self (loopback)
   inbound_message_map[channel].push_back(message_text);
  }
}

void classTcpipMessaging::SendMessageToConnection(int connection_id, string channel, string message_text)
{
  if(client_socket && client_socket->Active)
  {
    string connection_id_str = "";
    char floatnumber[30];
    sprintf(floatnumber, "%1i", connection_id);
    connection_id_str = floatnumber;

    string to_send = "to~" + connection_id_str + "~" + channel + "~" + message_text;
    #ifdef _DEBUG
    LogIt(("dbg__" + channel + ".log").c_str(), to_send);
    #endif
    outbound_messages.AddMessage(to_send);
    outbound_messages.AttemptWrite(client_socket);
  } else
  {
   // TODO: for loopback server, ... hmmm. do we even have our own connection number? eep?
  }
}

void classTcpipMessaging::CloseConnection(int connection_id)
{
  if(client_socket && client_socket->Active)
  {
    string connection_id_str = "";
    char floatnumber[30];
    sprintf(floatnumber, "%1i", connection_id);
    connection_id_str = floatnumber;

    SendMessageToServer("close~" + connection_id_str);
  } else
  {
   // TODO: loopback ... do we care? probably not.
  }
}

void classTcpipMessaging::SendMessageToServer (string message_text)
{
  if(client_socket && client_socket->Active)
  {
    string to_send = "server~" + message_text;
    #ifdef _DEBUG
    LogIt("dbg__server.log", to_send);
    #endif
    outbound_messages.AddMessage(to_send);
    outbound_messages.AttemptWrite(client_socket);
  } else
  {
   // TODO: loopback ... do we care? dunno. depends on message. likely not.
  }
}

bool classTcpipMessaging::IsMessageWaiting (string channel_name)
{
  if(inbound_message_map.find(channel_name) != inbound_message_map.end())
  {
    if (! inbound_message_map[channel_name].empty())
      return true;
    else
      return false;
  }
  else
    return false;
}

string classTcpipMessaging::GetNextMessage (string channel)
{
  string to_return;
  if(! inbound_message_map[channel].empty())
  {
    to_return = inbound_message_map[channel].front();
    inbound_message_map[channel].erase(inbound_message_map[channel].begin());
  }
  else
    to_return = "";
  return to_return;
}

list<string> classTcpipMessaging::GetAllChannels ()
{
  list<string> list_to_return;
  for(map<string, list<string> >::iterator itr = inbound_message_map.begin();
      itr != inbound_message_map.end(); itr++)
    list_to_return.push_back(itr->first);
  return list_to_return;
}

list<string> classTcpipMessaging::GetChannelsWithMessages ()
{
  list<string> list_to_return;
  for(map<string, list<string> >::iterator itr = inbound_message_map.begin();
      itr != inbound_message_map.end(); itr++)
  {
    if(! itr->second.empty())
      list_to_return.push_back(itr->first);
  }
  return list_to_return;
}

void classTcpipMessaging::SetValue (string map_key, string map_value)
{
  SendMessageToServer("set~" + map_key + "~" + map_value);
}

void classTcpipMessaging::GetValue (string map_key, string auth)
{
  SendMessageToServer("get~" + map_key + "~" + auth);
}

void classTcpipMessaging::UnsetValue (string map_key)
{
  SendMessageToServer("unset~" + map_key);
}

void classTcpipMessaging::GetServerStatus (string channel)
{
  SendMessageToServer("status~" + channel);
}

void classTcpipMessaging::RequestServerNotification (string channel, string event)
{
  SendMessageToServer("notify~" + channel + "~" + event);
}

void classTcpipMessaging::UnrequestServerNotification (string event)
{
  SendMessageToServer("unnotify~" + event);
}

void classTcpipMessaging::AttemptRead ()
{
  inbound_messages.AttemptRead(client_socket);

  set<string> newly_readable_channels;

  while (inbound_messages.IsMessagePending())
  {
   string raw_message = inbound_messages.GetMessage();
   string useful_message;
   string channel_name;
   int seperator = raw_message.find_first_of("~");

   channel_name = raw_message.substr(0, seperator);
   useful_message = raw_message.substr(seperator+1, raw_message.length()-seperator);

   if (!IsMessageWaiting(channel_name))
    newly_readable_channels.insert(channel_name);
   inbound_message_map[channel_name].push_back(useful_message);
  }

  set<string>::iterator i_new_channel, i_new_channel_end;
  i_new_channel_end = newly_readable_channels.end();
  map<string, set<classChannelReadableClosure> >::iterator i_channel, i_channel_end;
  i_channel_end = inbound_message_callbacks.end();
  set<classChannelReadableClosure>::iterator i_closure_this, i_closure_next;

  for(i_new_channel = newly_readable_channels.begin(); i_new_channel != i_new_channel_end; i_new_channel++)
  {
   i_channel = inbound_message_callbacks.find(*i_new_channel);
   if (i_channel != i_channel_end)
   {
    i_closure_this = i_closure_next = i_channel->second.begin();
    while (i_closure_this != i_channel->second.end())
    {
     i_closure_next++;
     if (!i_closure_this->Execute(i_channel->first))
      i_channel->second.erase(i_closure_this);
     i_closure_this = i_closure_next;
    }
   }
  }
}

void classTcpipMessaging::AttemptWrite ()
{
  outbound_messages.AttemptWrite(client_socket);
}

void classTcpipMessaging::GetProperties(string channel)
{
  SendMessageToServer("list~" + channel);
}

void classTcpipMessaging::AddCallback(string channel, TForm * form, ChannelReadable callback)
{
 inbound_message_callbacks[channel].insert(classChannelReadableClosure(form, callback));
 if (IsMessageWaiting(channel))
  classChannelReadableClosure(form, callback).Execute(channel);
}

void classTcpipMessaging::DelCallback(string channel, TForm * form, ChannelReadable callback)
{
 map<string, set<classChannelReadableClosure> >::iterator i_channel;
 set<classChannelReadableClosure>::iterator i_closure;
 i_channel = inbound_message_callbacks.find(channel);
 if (i_channel != inbound_message_callbacks.end())
 {
  i_closure = i_channel->second.find(classChannelReadableClosure(form, callback));
  if (i_closure != i_channel->second.end())
   i_channel->second.erase(i_closure);
 }
}

void classTcpipMessaging::DelCallback(string channel, TForm * form)
{
 map<string, set<classChannelReadableClosure> >::iterator i_channel;
 set<classChannelReadableClosure>::iterator i_this, i_next;
 i_channel = inbound_message_callbacks.find(channel);
 if (i_channel != inbound_message_callbacks.end())
 {
  i_next = i_this = i_channel->second.begin();
  while (i_this != i_channel->second.end())
  {
   i_next++;
   if (i_this->Matches(form))
    i_channel->second.erase(i_this);
   i_this = i_next;
  }
 }
}

void classTcpipMessaging::DelCallback(TForm * form)
{
 set<classChannelReadableClosure>::iterator i_next, i_this;
 for (map<string, set<classChannelReadableClosure> >::iterator i_channel = inbound_message_callbacks.begin(); i_channel != inbound_message_callbacks.end(); i_channel++)
 {
  i_next = i_this = i_channel->second.begin();
  while (i_this != i_channel->second.end())
  {
   i_next++;
   if (i_this->Matches(form))
    i_channel->second.erase(i_this);
   i_this = i_next;
  }
 }
}

