#ifndef tcpip_messaging_h
#define tcpip_messaging_h

#include "mesg_q.h"
#include <sysutils.hpp>
#include <vector>
#include <list>
#include <map>
#include <set>
#include <string>

#include <Classes.hpp>
#include <Controls.hpp>
#include <StdCtrls.hpp>
#include <Forms.hpp>
#include <Menus.hpp>
#include <ComCtrls.hpp>
#include <ToolWin.hpp>
#include <Buttons.hpp>
#include <stdlib.h>
#include <stdio.h>
using namespace std;

typedef void (TForm::*ChannelReadable)(string);

class classChannelReadableClosure
{
 private:
  TForm * form;
  ChannelReadable callback;

 public:
  classChannelReadableClosure (TForm * p_form, ChannelReadable p_callback)
   {form = p_form; callback = p_callback;}
  bool Execute(string p_channel) const
   {try {(form->*callback)(p_channel);} catch (...) {return false;} return true;}
  bool Matches (TForm * p_form) const
   {return (form == p_form);}
  bool Matches (TForm * p_form, ChannelReadable p_callback) const
   {return (form == p_form && callback == p_callback);}
  bool operator == (const classChannelReadableClosure & p_closure) const
   {return Matches(p_closure.form, p_closure.callback);}
  bool operator < (const classChannelReadableClosure & p_closure) const
   {return (form < p_closure.form || (form == p_closure.form && this < &p_closure));}
   // pardon the hack, but you quite simply can't compare callbacks via 'operator <' ...
   // not with callback, not with form->*callback,
   // not with casting, and not with green eggs or ham!
   // -- philip March 6, 2003
};

class classTcpipMessaging
{
private:
 inbound_message_queue inbound_messages;
 outbound_message_queue outbound_messages;
 map<string, list<string> > inbound_message_map;
 map<string, set<classChannelReadableClosure> > inbound_message_callbacks;
 TClientSocket * client_socket;

public:
 classTcpipMessaging(TClientSocket * p_client_socket);
 void SendMessageToAny (string map_key, string map_value, string channel, string message_text);
 void SendMessageToAll (string channel, string message_text);
 void SendMessageToServer(string message_text);
 void SendMessageToConnection(int connection_id, string channel, string message_text);
 void CloseConnection(int connection_id);
 bool IsMessageWaiting(string channel);
 string GetNextMessage (string channel);
 list<string> GetAllChannels ();
 list<string> GetChannelsWithMessages ();
 void SetValue (string map_key, string map_value);
 void GetValue (string map_key, string auth);
 void UnsetValue (string map_key);
 void GetServerStatus (string channel);
 void RequestServerNotification (string channel, string event);
 void UnrequestServerNotification (string event);
 // void GetValueOfAny (string map_key, string map_value, string map_key);
 void AttemptRead();
 void AttemptWrite();
 void GetProperties(string channel);
 void AddCallback(string channel, TForm * p_form, ChannelReadable p_callback);
 void DelCallback(string channel, TForm * p_form, ChannelReadable p_callback);
 void DelCallback(string channel, TForm * p_form);
 void DelCallback(TForm * p_form);
 // map<string, string> GetPropertiesOf (unique_id,
};

//---------------------------------------------------------------------------
extern classTcpipMessaging* tcpip_message_router;
//---------------------------------------------------------------------------

#endif


