//---------------------------------------------------------------------------
#ifndef ttreelistview_H
#define ttreelistview_H
//---------------------------------------------------------------------------
#include <SysUtils.hpp>
#include <Classes.hpp>
#include <ComCtrls.hpp>
#include <Controls.hpp>
#include <map>
#include <string>
#include <vector>
using namespace std;
//---------------------------------------------------------------------------
// Forward declarations
//---------------------------------------------------------------------------
class tlv_column_type;
class tlv_column_container_type;
class PACKAGE TTreeListView;
//---------------------------------------------------------------------------
// Enumerated types
//---------------------------------------------------------------------------
enum add_row_method {sib_first, sib_last, sib_previous, sib_next, child_first, child_last};
enum show_method {show_string, show_int, show_float, show_date, show_stamp, show_time,
                  show_bool, show_large_integer};
enum sort_method {sort_text, sort_date, sort_stamp, sort_time, sort_int,
                  sort_double, sort_bool};
typedef string (*ShowFunction)(string);

//------------------------------------------------------------------------
// for CustomSort on the treelistview
int __stdcall TTLVCompare(long Param1, long Param2, long columnToSort);
int CompareByType (string a, string b, sort_method as_what);
template <class T>
int TemplateCompare (T a, T b);
//------------------------------------------------------------------------
//this structure contains the data that determines the column's display
//format as well as how the data sorts.  Data that affects column behavior
//not shown here includes:
//from the vector<column_data>: column position (0 being tree data)
//---------------------------------------------------------------------------
// Class definitions
//---------------------------------------------------------------------------
class tlv_column_container_type
{
friend class tlv_column_type;
private:
  map<string, unsigned int> logical_index_lookup;
  TTreeListView *tlvParent;

  void Add (tlv_column_type *incoming);
  unsigned int NumColumns() {return column_data.size();};
public:
  // constructor/destructor
  tlv_column_container_type(TTreeListView *tlv);
  ~tlv_column_container_type();

  tlv_column_type* Column(string name);
  tlv_column_type* ColumnByLogicalIndex(unsigned int index);
  void Add (string title, int width, int logical_index,
       ShowFunction show_as = show_string, sort_method sort_as = sort_text,
       TAlignment alignment = taLeftJustify, bool visible = true);
  void Move (int src_col, int dest_col);

  vector<tlv_column_type *>   column_data;
  void LogicallyReindex();

  __property unsigned int Count = {read=NumColumns};
};
//---------------------------------------------------------------------------
class tlv_column_type
{
private:
   string        fTitle;
   unsigned int  fWidth;
   ShowFunction  fShowAs;
   sort_method   fSortAs;
   TAlignment    fAlignment;
   bool          fVisible;
   unsigned int  fLogicalIndex;
   tlv_column_container_type *Container;

   void SetTitle (string pTitle);
   void SetWidth (unsigned int pWidth);
   void SetShowType (ShowFunction pShowAs);
   void SetSortType (sort_method pSortAs);
   void SetAlignment (TAlignment pAlignment);
   void SetVisibility (bool pVisible);
   void SetLogicalIndex (unsigned int pLogicalIndex);
public:
   tlv_column_type (tlv_column_container_type *pContainer, int pLogicalIndex,
      string pTitle = "", int pWidth = 0, ShowFunction pShowAs = NULL,
      sort_method pSortAs = sort_text, TAlignment pAlignment = taLeftJustify,
      bool pVisible = true)
   {
     fLogicalIndex = pLogicalIndex; fTitle = pTitle; fWidth = pWidth; fShowAs = pShowAs;
     fSortAs = pSortAs; fAlignment = pAlignment; fVisible = pVisible;
     Container = pContainer;
   };

//__published:
  __property string        Title         = {read=fTitle,write=SetTitle};
  __property unsigned int  Width         = {read=fWidth,write=SetWidth};
  __property ShowFunction  ShowAs        = {read=fShowAs,write=SetShowType};
  __property sort_method   SortAs        = {read=fSortAs,write=SetSortType};
  __property TAlignment    Alignment     = {read=fAlignment,write=SetAlignment};
  __property bool          Visible       = {read=fVisible,write=SetVisibility};
  __property unsigned int  LogicalIndex  = {read=fLogicalIndex,write=SetLogicalIndex};
};
//---------------------------------------------------------------------------
class PACKAGE TTreeListView : public TWinControl
{
friend class tlv_column_container_type;
private: // data
  TPanel         *border;
  THeaderControl *header;
  TTreeView      *tree;
  TList          *ColorNodes;
  bool           header_visible;
  bool           select_row;
  bool           sort_asc;
  map<TTreeNode *, vector<string> > row_data;
  void Sort(THeaderSection* Column);
  string         sort_column_index;
  TImageList*    header_images;

private: // functions
  // these were silly.  I'm not using them currently.
  TRect ShiftRect(TRect rect, bool shift_bottom);
  void  SetSize(TControl * control, TRect rect);
public:  // data
  // TODO: make private again?

  tlv_column_container_type *Columns;
public:  // functions
  // constructor, destructor
  __fastcall TTreeListView (TComponent* AOwner);
  __fastcall ~TTreeListView ();

  void        RefreshHeaders ();
  int         GetColumnWidth(int logical_index);
  string      GetColumnValue (TTreeNode * row_node, unsigned int column);
  TTreeNode * AddRow (vector<string> data, add_row_method method, TTreeNode * node);
  bool        EditRow(vector<string> data, add_row_method method, TTreeNode * node);

  void        GetRow (TTreeNode* node, map<string, string>& row);
  void        GetSelectedRows (vector<map<string, string> >& rows);

  int CompareByColumn (TTreeNode* T1, TTreeNode* T2, long sortColumn);

  // OnEvents handled by this class
  void __fastcall HeaderSectionTrack (THeaderControl *HeaderControl, THeaderSection *Section, int Width, TSectionTrackState State);
  void __fastcall HeaderSectionResize (THeaderControl* HeaderControl, THeaderSection* Section);
  void __fastcall HeaderSectionDrag (TObject* Sender, THeaderSection* FromSection,THeaderSection* ToSection, bool &AllowDrag);
  void __fastcall HeaderSectionEndDrag (TObject* Sender);
  void __fastcall HeaderSectionClick (THeaderControl* HeaderControl, THeaderSection* Section);
  void __fastcall AdvancedCustomDraw (TCustomTreeView* Sender, const TRect &ARect, TCustomDrawStage Stage, bool &DefaultDraw);
  void __fastcall AdvancedCustomDrawItem (TCustomTreeView *Sender, TTreeNode *Node,TCustomDrawState State, TCustomDrawStage Stage, bool &PaintImages, bool &DefaultDraw);
  void __fastcall TLVMouseUp (TObject *Sender, TMouseButton Button,TShiftState Shift, int X, int Y);
  void __fastcall TLVMouseMove (TObject *Sender, TShiftState Shift, int X, int Y);

  // Tree/Header OnEvents accessible externally
  TTVChangedEvent __fastcall GetOnChange() {return tree->OnChange;}
  void __fastcall SetOnChange(TTVChangedEvent on_change) {tree->OnChange = on_change;}
  TTVExpandedEvent __fastcall GetOnExpanded() {return tree->OnExpanded;}
  void __fastcall SetOnExpanded(TTVExpandedEvent on_expanded) {tree->OnExpanded = on_expanded;}
  TTVExpandingEvent __fastcall GetOnExpanding() {return tree->OnExpanding;}
  void __fastcall SetOnExpanding(TTVExpandingEvent on_expanding) {tree->OnExpanding = on_expanding;}

//BEGIN_MESSAGE_MAP
//MESSAGE_HANDLER(WM_PAINT,TMessage,PaintIt)
//END_MESSAGE_MAP(TWinControl);

__published:
  __property bool   ShowHeader = {read=header_visible,write=header_visible};
  __property bool   RowSelect  = {read=select_row,write=select_row};
  __property TAlign Align      = {read=FAlign, write=SetAlign, default=0};
  __property TTVChangedEvent OnChange = {read=GetOnChange,write=SetOnChange};
  __property TTVExpandedEvent OnExpanded = {read=GetOnExpanded,write=SetOnExpanded};
  __property TTVExpandingEvent OnExpanding = {read=GetOnExpanding,write=SetOnExpanding};
};
//---------------------------------------------------------------------------
// tlv_column_type implementation
//---------------------------------------------------------------------------
void tlv_column_type::SetTitle(string pTitle)
{
   fTitle = pTitle;
   Container->LogicallyReindex();
   Container->tlvParent->RefreshHeaders();
}
//---------------------------------------------------------------------------
void tlv_column_type::SetWidth(unsigned int pWidth)
{
   fWidth = pWidth;
   Container->tlvParent->RefreshHeaders();
   Container->tlvParent->Repaint();
}
//---------------------------------------------------------------------------
void tlv_column_type::SetShowType(ShowFunction pShowAs)
{
   fShowAs = pShowAs;
   Container->tlvParent->RefreshHeaders();
   Container->tlvParent->Repaint();
}
//---------------------------------------------------------------------------
void tlv_column_type::SetSortType(sort_method pSortAs)
{
   fSortAs = pSortAs;
   Container->tlvParent->RefreshHeaders();
   Container->tlvParent->Repaint();
}
//---------------------------------------------------------------------------
void tlv_column_type::SetAlignment(TAlignment pAlignment)
{
   fAlignment = pAlignment;
   Container->tlvParent->RefreshHeaders();
   Container->tlvParent->Repaint();
}
//---------------------------------------------------------------------------
void tlv_column_type::SetVisibility(bool pVisible)
{
   fVisible = pVisible;
   Container->tlvParent->RefreshHeaders();
   Container->tlvParent->Repaint();
}
//---------------------------------------------------------------------------
void tlv_column_type::SetLogicalIndex(unsigned int pLogicalIndex)
{
   fLogicalIndex = pLogicalIndex;
   Container->tlvParent->RefreshHeaders();
   Container->tlvParent->Repaint();
}
//---------------------------------------------------------------------------
// tlv_column_container_type implementation
//---------------------------------------------------------------------------
tlv_column_container_type::tlv_column_container_type(TTreeListView *tlv)
{
  tlvParent = tlv;
}
//---------------------------------------------------------------------------
// destructor, delete column objects from vector
tlv_column_container_type::~tlv_column_container_type()
{
  for(vector<tlv_column_type*>::iterator itr = column_data.begin(); itr != column_data.end(); itr++)
    delete (*itr);
}
//---------------------------------------------------------------------------
tlv_column_type* tlv_column_container_type::Column(string name)
{
  tlv_column_type *result;
  if (logical_index_lookup.find(name) != logical_index_lookup.end())
    result = ColumnByLogicalIndex(logical_index_lookup[name]);
  else
    result = NULL;
  return result;
}
//---------------------------------------------------------------------------
tlv_column_type* tlv_column_container_type::ColumnByLogicalIndex(unsigned int index)
{
  if (index < column_data.size())
    return column_data[index];
  else
    return NULL;
}
//---------------------------------------------------------------------------
void tlv_column_container_type::Add(string title, int width, int logical_index,
       ShowFunction show_as, sort_method sort_as, TAlignment alignment, bool visible)
{
   Add(new tlv_column_type(this, logical_index, title, width, show_as, sort_as, alignment, visible));
}
//---------------------------------------------------------------------------
void tlv_column_container_type::Add(tlv_column_type *incoming)
{
   //int logical_pos = incoming.position;
   //int physical_pos = column_data.size();
   //incoming.position = physical_pos;  // where the column is about to go...
   column_data.push_back(incoming);   // put it there.
   logical_index_lookup[incoming->Title] = column_data.size() - 1;
   tlvParent->RefreshHeaders();
}
//---------------------------------------------------------------------------
void tlv_column_container_type::Move(int src_col, int dest_col)
{
   int inc;
   if(src_col < dest_col)
      inc = 1;
   else
      inc = -1;
   while(src_col != dest_col)
   {
      swap(column_data[src_col], column_data[src_col + inc]);
      src_col += inc;
   }
   LogicallyReindex();
}
//---------------------------------------------------------------------------
void tlv_column_container_type::LogicallyReindex()
{
   for (unsigned int i=0; i != column_data.size(); i++)
      logical_index_lookup[column_data[i]->Title] = i;
}
//---------------------------------------------------------------------------
#endif

