//PAN
#ifndef utilityfunctions
#define utilityfunctions

#include <Classes.hpp>
#include <Controls.hpp>
#include <StdCtrls.hpp>
#include <Db.hpp>
#include <IBCustomDataSet.hpp>
#include <IBDatabase.hpp>
#include <IBSQL.hpp>
#include <fstream.h>
#include <dir.h>
#include <string>
#include <vector>
#include <deque>
#include <list>
#include <map>
#include <set>
#include "allsymbols.h"
#include "financial.h"

// you MUST have a string class_name declared to use these macros.  They are already declared in all specialGeneric-derived forms.
#define AV_TRY { try {
#ifdef use_prometheus_util
 #define AV_CATCH } catch(EAccessViolation &av) {::PPretendToSay(av.Message.c_str(), m_program_error, class_name, "Access Violation caught: " + string(__FILE__) + "; " + string(__FUNC__) + "; " + IntToString(__LINE__));} }
#else
 #define AV_CATCH } catch(EAccessViolation &av) {Application->MessageBox((("Access Violation caught: " + string(__FILE__) + "; " + string(__FUNC__) + "; " + IntToString(__LINE__) + "\n\n") + av.Message.c_str()).c_str(), ("Program Error in " + string(class_name.c_str())).c_str(), MB_OK);} }
#endif

// jhaley 20111116: For when error reporting is pointless or detrimental, just catch them.
#define AV_CATCH_SILENT } catch(EAccessViolation &av) {} }

using namespace std;

//accept dates in one format, output in the other, play...
class Pdate
{
  public:
    int year;
    int month;
    int day;
    Pdate ();
    Pdate (const string &incoming, int mode);
    Pdate (TDate incoming);
    bool IsValidDate () const;
    void InDate (const string &incoming, int mode);
    void InDate (TDate incoming);
    void InDate (const Pdate & incoming);
    string OutDate (int mode) const;
    TDate OutDate () const;
    bool operator <  (const Pdate & rhs) const;
    bool operator <= (const Pdate & rhs) const;
    bool operator >  (const Pdate & rhs) const;
    bool operator >= (const Pdate & rhs) const;
    bool operator != (const Pdate & rhs) const;
    bool operator == (const Pdate & rhs) const;
    Pdate operator ++ (int);
    Pdate operator -- (int);
    int  mod (int p_distance, int p_metric);
    int  mod (const Pdate & p_distance);
    int  DayOfWeek() const;
    Pdate operator += (int rhs);
    Pdate operator -= (int rhs);
    string NameOfDay() const;
    string NameOfMonth() const;
};

//accept time in one format, output in the other, play...
class Ptime
{
  public:
    int hour;
    int minute;
    int second;
    Ptime ();
    Ptime (const string &incoming, int mode);
    Ptime (TDateTime incoming);
    bool IsValidTime () const;
    void InTime (const string &incoming, int mode);
    void InTime (TDateTime incoming);
    void InTime (const Ptime & incoming);
    string OutTime (int mode) const;
    TDateTime OutTime () const;
    bool operator <  (const Ptime & rhs) const;
    bool operator <= (const Ptime & rhs) const;
    bool operator >  (const Ptime & rhs) const;
    bool operator >= (const Ptime & rhs) const;
    bool operator != (const Ptime & rhs) const;
    bool operator == (const Ptime & rhs) const;
    Ptime operator ++ (int);
    Ptime operator -- (int);
    int  mod (int p_distance, int p_metric);
    int  mod (const Ptime & p_distance);
};

//a date and time together, to do stuff in formats...
class Pstamp : public Pdate, public Ptime
{
  public:
    Pstamp ();
    Pstamp (const string &incoming, int mode);
    Pstamp (TDateTime incoming);
    bool IsValidStamp () const;
    void InStamp (const string &incoming, int mode);
    void InStamp (TDateTime incoming);
    string OutStamp (int mode) const;
    bool operator <  (const Pstamp & rhs) const;
    bool operator <= (const Pstamp & rhs) const;
    bool operator >  (const Pstamp & rhs) const;
    bool operator >= (const Pstamp & rhs) const;
    bool operator != (const Pstamp & rhs) const;
    bool operator == (const Pstamp & rhs) const;
    int  mod (int p_distance, int p_metric);
    int  mod (const Pstamp & p_distance);
    int  mod (const Pdate & p_distance);
    int  mod (const Ptime & p_distance);
};

//outside pdate operators
Pdate  operator + (const Pdate & lhs, const int & rhs);
Pdate  operator - (const Pdate & lhs, const int & rhs);
int    operator - (const Pdate & lhs, const Pdate & rhs);
int CalcAge(Pdate lhs);
int CalcAge(Pdate lhs, Pdate rhs);
string GetTimeSince(Pdate now_date, Pdate then_date);
string GetTimeSince(Ptime now_date, Ptime then_date);
string GetTimeSince(Pstamp now_date, Pstamp then_date);
bool FriendlyDiffDate(string anchor_datestring, string relative_datestring, string &result, string relative_suffix, bool force_relative_syntax);
pair<Pdate, Pdate> GetPriorMonthDateRange(Pdate for_date);

//outside ptime operators
int operator - (const Ptime & lhs, const Ptime & rhs);

//outside pstamp operators
int operator - (const Pstamp & lhs, const Pstamp & rhs);

//other date/time/stamp stuff
string RightNow(bool compact = false);
string CurrentDate();
string CurrentTime();
Pdate  CurrentPdate();
Ptime  CurrentPtime();
Pstamp CurrentPstamp();

struct log_file_item
{
 fstream *file;
 Pstamp last_timestamp;
 log_file_item ()
 {
  file = NULL;
  last_timestamp = CurrentPstamp();
 }
 log_file_item (fstream* f, Pstamp ts)
 {
  file = f;
  last_timestamp = ts;
 }
};

void GetLog(AnsiString filename, fstream * &f);
void CloseLogs();
void LogIt(AnsiString filename, string message);

//query
bool   IsInt (const string &incoming);
bool   IsFloat (const string &incoming);
bool   IsInTStringList (TStringList *, AnsiString);
bool   IsLeapYear(int year);
int    LastDayOf (int month, int year);
void   FindComboBoxItem (TObject *, char &);
void   FindComboBoxItemAndSetItemIndex(TObject *Sender, char &Key);
void   ResetItemIndex(TObject *Sender);
bool   LikeString (const string &regular_expression_string, const string &string_to_search);
bool   IsEnter (char key);
bool   IsAmount (const string &incoming, int left_num = 15, int right_num = 2);
bool   IsInvalidDate(const string &date_entered);
bool   IsInvalidDate(TEdit * edit_box);
string ShowTypeName(int show_type);
string SortTypeName(int sort_type);
int    ShowTypeId(const string &show_type_name);
int    SortTypeId(const string &sort_type_name);
void   ShowTypeNames (vector<string> & p_return);
void   SortTypeNames (vector<string> & p_return);

//strip/pad/replace
string StripTrailing(const string &incoming, const string &characters);
string StripLeading(const string &incoming, const string &characters);
string StripAll(const string &incoming, const string &characters);
string StripAllExcept(const string &incoming, const string &characters);
string ReplaceCharWithChar(const string &incoming, char from, char to);
string ReplaceStringWithString(const string &incoming, const string &from, const string &to, bool recursive_collapsing = false);
string ReplaceAnyCharWithString(const string &incoming, const string &characters, const string &to);
string PadString(const string &incoming, pad_string_types how_to_pad, char pad_with, int final_size);
string CollapseSpaces(const string &input);
string ReplaceWordsWithWords(const string &incoming, const map<string, string> &replacement_map);

int RoundNearest(double D);
bool IsVisible (TControl * p_control);
bool IsEnabled (TControl * p_control);
bool HasFocusedAncestor (TWinControl * p_control);
bool HasAncestor (TWinControl *p_test, TWinControl *p_ancestor);
void InitUspsAbbrMap();

//convert
string twodigit(int incoming);
string StringToAmount (const string &incoming, int left_num = 15, int right_num = 2);
string AmountToString (const string &incoming, int left_num = 15, int right_num = 2);
string PrettyAmountString (const string &incoming, bool blank_zero_amounts = true);
string IntToString(int incoming);
string UIntToString(unsigned int incoming);
int    StringToInt(const string &incoming);
string IntToHexString(int incoming, int padto=0, bool prepend0x=false);
int    HexStringToInt(const string &incoming);
bool   IsHexInt(const string &incoming, int maxbits); // jhaley 20110816
string IntToRGBString(int incoming);
int    RGBStringToInt(const string &incoming);
string FlipRGB(const string &incoming);
string FloatToStringApprox(float incoming, int right_num);
string FloatToString(float incoming, int right_num = 2);
float  StringToFloat(const string &incoming);
string RawString (const string &incoming);
string SentenceString (const string &incoming);
string PrettyString (const string &incoming);
string UppercaseFirstChar(const string &incoming);
string LowercaseFirstChar(const string &incoming);
string PowerSearchString (const string &incoming);
string InitialsSearchString1 (const string &incoming);
string InitialsSearchString2 (const string &incoming);
string LowercaseString (const string &incoming);
string UppercaseString(const string &incoming);
string ConvertBinary(const string &incoming);
string ConvertTime(const string &incoming);
string ConvertDate(const string &incoming, int y2kcutoff);
string ConvertStamp(const string &incoming, int y2kcutoff);
string ConvertBool(const string &incoming);
void   septhree (const string &incoming, char separator, int &A, int&B, int&C);
string AddOptions(int searchOption, const string &field);
string PrettySSN(const string &incoming);
string PrettyDate(const string &incoming);
string TwoDecimals(const string &incoming);
string ConvertStupidNumber(const string &incoming);
string FormatPhoneNumber(const string &incoming);
string FormatPhoneNumberNoBreak(const string &incoming); // jhaley
string FormatZipCode(const string &incoming);
string FormatAddress (const string &incoming);
string AcronymString(const string &incoming);
string PrettySql(const string &incoming);
string SafeSQLString(const string &incoming, bool preserve_fieldname_case = false);
string IntToBoolString (int incoming, bool_types how = yes_no);
string BoolToBoolString (bool incoming, bool_types how = true_false);
int    BoolToInt(bool incoming);
void   SplitStringToVec(const string &incoming, const string &separator, vector<string> &vec);
void   SplitStringToSet(const string &incoming, const string &separator, set<string> &incoming_set);
string VecToDelimString(const vector<string> &p_vec, const string &left, const string &right, const string &between);
string VecToCommaString(const vector<string> &p_vec);
string SetToDelimString(const set<string> &p_set, const string &left, const string &right, const string &between);
string SetToCommaString(const set<string> &p_set);
string ListToDelimString(const list<string> &p_list, const string &left, const string &right, const string &between);
string ListToCommaString(const list<string> &p_list);
string MapToDelimString(const map<string,string> &p_map, const string &left_outer, const string &right_outer, const string &between_pair, const string &left_key, const string &right_key, const string &left_value, const string &right_value, const string &between_key_value);
string MapToCommaString(const map<string,string> &p_map);
string VecMapColumnToDelimString(const vector<map<string, string> > &p_vecmap, const string &field, const string &left, const string &right, const string &between);
string VecMapColumnToCommaString(const vector<map<string, string> > &p_vecmap, const string &field);
bool   VecMapColumnToVec(const vector<map<string, string> > &p_vecmap, vector<string> &p_vec, const string &field, bool include_blank = true);
void   SKVToPreparedInsert(const map<string, map<string, string> >& skv_map, list<vector<string> >& output_data);
void   TStringsToVec(TStrings * ts, vector<string> &vec);
string KeywordParseString(const string &incoming, const string &fieldname);
bool   DelimStringToVec(const string &raw_line, char quote_char, char delim_char, vector<string> &vecCSV, bool watch_doubles);
bool   CSVtoVecMap(const string &filename, char quote_char, char delim_char, vector<map<string,string> > &vecmap, string &err_message, bool watch_doubles);
bool   CopyCSVToClipboard(vector<vector<string> > &incoming, vector<string> headers = vector<string>(), bool include_headers = 1, bool always_quote = 1);

// Versioning, debug, and executable file magic stuff:
string ParseFileVersionInfo(AnsiString FQFN);
void   GetFileVersionMap(AnsiString FQFN, map<string, int> &version_map); // jhaley 20110215
bool   GetLogicalAddress(PVOID addr, DWORD &section, DWORD &offset);      // jhaley 20110428

bool   IsBlankOrZero(const string &id_field_value);

financial StringToFinancial(const string & text_value);
string    FinancialToStringEx(const financial & numeric_value); // for cases where you need a symmetrical object
string    FinancialToString(const financial numeric_value, bool with_comma = false, unsigned int decimal_places = 2);

bool      is_in (string val, ...); // use with care! takes char* for all other values, use NULL to terminate. example: if (in(some_string, "1", "2", "3", NULL)) {}
bool      is_in (AnsiString val, ...); // AnsiString version

struct financial_adder
{
 financial local_sum;
 financial_adder & reset() {local_sum = 0.00; return (*this);}
 financial_adder () {reset();}
 void operator () (const string &extra_value) {local_sum += StringToFinancial(extra_value);}
};

template<class T>
T & get_vec_map_column_sum(vector<map<string, string> > & data, const string &column_name, T & addition_object)
{
 string field_value;
 vector<map<string, string> >::iterator i_row;
 map<string, string>::iterator i_field;
 for (i_row = data.begin(); i_row != data.end(); i_row++)
 {
  i_field = (*i_row).find(column_name);
  if (i_field != (*i_row).end())
  {
   field_value = i_field->second;
   addition_object(field_value);
  }
 }
 return addition_object;
}

template <class T>
void VecToSet (const vector<T> & input, set<T> & output)
{
 output.clear();
 for(vector<T>::const_iterator i = input.begin(); i != input.end(); i++)
  output.insert(*i);
 return;
};

template <class T>
set<T> VecToSet (const vector<T> & input)
{
 set<T> output;
 output.clear();
 for(vector<T>::const_iterator i = input.begin(); i != input.end(); i++)
  output.insert(*i);
 return output;
};

template <class T>
void SetToVec (const set<T> & input, vector<T> & output)
{
 output.clear();
 for(set<T>::const_iterator i = input.begin(); i != input.end(); i++)
  output.push_back(*i);
 return;
};

template <class T>
vector<T> SetToVec (const set<T> & input)
{
 vector<T> output;
 output.clear();
 for(set<T>::const_iterator i = input.begin(); i != input.end(); i++)
  output.push_back(*i);
 return output;
};

template <class K, class V>
void MergeMap (map<K, V> & merge_into, map<K, V> & merge_from)
{
 for (map<K, V>::iterator i_from = merge_from.begin(); i_from != merge_from.end(); i_from++)
  merge_into[i_from->first] = i_from->second;
 return;
}

class numbered_string_generator
{
 private:
  unsigned int next_num;
  string base_text;

 public:
 numbered_string_generator (const string &initial_text, unsigned int initial_num) : next_num(initial_num), base_text(initial_text) {};
 string operator () ()
 {
  string answer = base_text + IntToString(next_num);
  next_num++;
  return answer;
 }
};

template <class item>
bool dependency_graph (map<item, set<item> > item_dependencies, vector<set<item> > & item_order, bool push_down = false)
{
 // return false if a conflict is found
 // see algorithm notes from the original class, below.

 item_order.clear();
 bool found_this_pass;
 set<item> add_this_pass;
 map<item, unsigned int> item_to_band;
 map<item, set<item> > item_children;
 map<item, set<item> > original_item_dependencies;
 if (push_down)
  original_item_dependencies = item_dependencies;

 do
 {
  add_this_pass.clear();
  found_this_pass = false;
  map<item, set<item> >::iterator i_next = item_dependencies.begin();
  for (map<item, set<item> >::iterator i_item = item_dependencies.begin(); i_item != item_dependencies.end(); i_item = i_next)
  {
   i_next = i_item;
   i_next++;
   if (i_item->second.empty())
   {
    add_this_pass.insert(i_item->first);
    item_dependencies.erase(i_item);
   } else
    found_this_pass = true;
  }

  // these dependencies have been fulfilled, remove from waiting list's lists. (confusing to say?)
  for (map<item, set<item> >::iterator i_item = item_dependencies.begin(); i_item != item_dependencies.end(); i_item++)
   for (set<item>::iterator i_added = add_this_pass.begin(); i_added != add_this_pass.end(); i_added++)
    i_item->second.erase(*i_added);

  if (!add_this_pass.empty())
  {
   if (push_down)
   {
    unsigned int band_number = item_order.size();
    for (set<item>::iterator i = add_this_pass.begin(); i != add_this_pass.end(); i++)
    {
     item_to_band[*i] = band_number;
     for (set<item>::iterator j = original_item_dependencies[*i].begin(); j != original_item_dependencies[*i].end(); j++)
      item_children[*j].insert(*i);
    }
   }
   item_order.push_back(add_this_pass);
  }
 } while (!add_this_pass.empty());

 // go back and push things to the right if possible
 if (push_down)
 {
  unsigned int max_band = item_order.size() - 1;
  set<item>::iterator l;
  for (vector<set<item> >::reverse_iterator i = item_order.rbegin(); i != item_order.rend(); i++)
  {
   for (set<item>::iterator j = i->begin(); j != i->end(); j++)
   {
    unsigned int currently_in = item_to_band[*j];
    unsigned int minimum_child = max_band;
    for (set<item>::iterator k = item_children[*j].begin(); k != item_children[*j].end(); k++)
     minimum_child = min(minimum_child, item_to_band[*k]);
    if (minimum_child - 1 > currently_in && minimum_child != max_band)
    {
     unsigned int should_be_in = minimum_child - 1;
     item_to_band[*j] = should_be_in;
    }
   }
  }

  // too many problems with modifying the vector on the fly, so ... fixing that by just re-inserting crap here. (yes, by that I mean I was an idiot of some sort.)
  for (vector<set<item> >::iterator i = item_order.begin(); i != item_order.end(); i++)
   i->clear();

  for (map<item, unsigned int>::iterator i = item_to_band.begin(); i != item_to_band.end(); i++)
   item_order[i->second].insert(i->first);
 }

 // if we had a pass where nothing was added, but things were waiting, deadlock.
 if (found_this_pass)
  return false;
 return true;
}

struct keyword_statement
{
 vector<string> and_words;
 vector<string> not_words;
};

template<class T>
set<T> subtract (const set<T> & a, const set<T> & b)
{
 set<T> answer;
 set_difference(a.begin(), a.end(),
                b.begin(), b.end(),
                insert_iterator<set<T> >(answer, answer.begin()));
 return answer;
}

template<class T>
set<T> add (const set<T> & a, const set<T> & b)
{
 set<T> answer;
 set_union(a.begin(), a.end(),
           b.begin(), b.end(),
           insert_iterator<set<T> >(answer, answer.begin()));
 return answer;
}

template<class T>
set<T> intersect (const set<T> & a, const set<T> & b)
{
 set<T> answer;
 set_intersection(a.begin(), a.end(),
                  b.begin(), b.end(),
                  insert_iterator<set<T> >(answer, answer.begin()));
 return answer;
}

/*template<class T>
T MapValue (map<string, T> & a_map, string a_key)
{
 return a_map[a_key];
}
*/
template <class K, class V>
V MapValue(map<K, V> &M, K &key)
{
 map<K, V>::iterator I = M.find(key);
 if (I != M.end()) return I->second;
 else return V();
 // "what in the everloving hell?" I hear you asking.  Well.  The debugger
 // won't let you Watch or Inspect some_map[some_value] with bracket notation,
 // but you CAN do both with a user defined function.  Thus, this.
}
//---------------------------------------------------------------------------

//---------------------------------------------------------------------------
//enum debug_form_types {libraries, generic_forms, other_forms}; // in allsymbols.h still but ... where does it belong?
//extern map<debug_form_types, int> debug_level; // don't have to look far; it's in util.cpp
extern int debug_level[numdebugformtypes];
//---------------------------------------------------------------------------
#endif



