//
// Exception Handling for Windows
//
// Based loosely on ExceptionHandler.cpp
// Author: Hans Dietrich (hdietrich2@hotmail.com)
// Original license: Public Domain
//

#ifndef VECTOREDEXCEPTION_H__
#define VECTOREDEXCEPTION_H__

// Only three externs for this module:
// * Call InstallVectoredExceptionHandler as early as possible at startup to register the exception handler.
// * Call SetVEHVersionInfo as soon as the EXE file version information is available.
// * DecodeBase64 is provided for decoding the stack data later. It returns the raw binary data, and places the length
//   of the data in bytes in "len". Use "delete []" on the return value when you're finished with it! It returns NULL
//   and sets len to 0 if the stack data cannot be decoded.

void InstallVectoredExceptionHandler();
void SetVEHVersionInfo(int pVersionMajor, int pVersionMinor, int pVersionRelease);
unsigned char *DecodeBase64(const string &encoded, size_t &len);

// jhaley 20110708: Global application state. Feel free to add more to distinguish different events of major import.
enum
{
  PROM_STATE_UNINIT,       // Nothing has been done yet; it will start with this value courtesy of the compiler.
  PROM_STATE_CONSTRUCTING, // Still inside constructor
  PROM_STATE_CONSTRUCTED,  // Constructor finished successfully
  PROM_STATE_STARTUP,      // In StartUp
  PROM_STATE_RUNNING,      // After StartUp
  PROM_STATE_WANTCLOSE,    // Close() has been called by our own code
  PROM_STATE_FORMCLOSE,    // In FormClose
  PROM_STATE_DESTROYING,   // In ~TformPrometheusMenu
  PROM_NUMSTATES
};
extern int PrometheusAppState;

#endif

// EOF


