#include "x_serialization.h"
#include "codeQueryGeneric.h"
#include "codeActionGeneric.h"

// TODO: switch to something like
// "tag~size~content"
// "map~96~a:string~3~hi!b:string~4~ugh.c:string~0~d:vector~44~string~2~IPint~3~204int~2~10int~1~1int~3~124"
// ==
// map(a:hi, b:ugh., c:, d:vector(IP, 204, 10, 3, 124))

xValue xVal(const string & Tag, const string & Content)
{
 xValue answer;
 answer = Tag + xSepTag + IntToString(Content.size()) + xSepSize + Content;
 return answer;
}

xValue xVal(const string & v, string::size_type start_at)
{
 return v.substr(start_at, xNextValue(v, start_at) - start_at);
}

string::size_type xNextValue(const xValue & v, string::size_type start_at)
{
 string::size_type answer = 0;
 string::size_type i = v.find(xSepTag, start_at);
 if (i != string::npos)
 {
  i++;
  string::size_type j = v.find(xSepSize, i);
  if (j != string::npos)
   answer = j + 1 + StringToInt(v.substr(i, j - i));
 }
 return answer;
}

string xTag(const xValue & v, string::size_type start_at)
{
 string::size_type i = v.find(xSepTag, start_at);
 return v.substr(start_at, i - start_at);
}

string::size_type xSize(const xValue & v, string::size_type start_at)
{
 string::size_type answer = 0;
 string::size_type i = v.find(xSepTag, start_at);
 if (i != string::npos)
 {
  i++;
  string::size_type j = v.find(xSepSize, i);
  if (j != string::npos)
   answer = StringToInt(v.substr(i, j - i));
 }
 return answer;
}

string xContent(const xValue & v, string::size_type start_at)
{
 string answer;
 string::size_type i = v.find(xSepTag, start_at);
 if (i != string::npos)
 {
  i++;
  string::size_type j = v.find(xSepSize, i);
  if (j != string::npos)
  {
   string::size_type size_to_get = StringToInt(v.substr(i, j - i));
   if (size_to_get)
    answer = v.substr(j + 1, size_to_get);
  }
 }
 return answer;
}

//---------------------------------------------------------------------------

TframeQueryGeneric* xAutoLoad(TframeQueryGeneric* host, TWinControl* parent, const xValue & settings, bool with_params, string::size_type start_at)
{
 TframeQueryGeneric* answer = NULL;
 if (!xTag(settings, start_at).empty())
 {
  q_factory_info temp_factory = query_factories.GetFactoryByTag (xTag(settings, start_at));
  if (temp_factory.create_pointer)
  {
   // if we can't verify, or there's no host against which to verify, or we can and do successfully verify ... create!
   if (!temp_factory.appropriate_pointer ||
       !host ||
        temp_factory.appropriate_pointer(host->SourceTableName.c_str()))
   {
    answer = temp_factory.create_pointer(host);
    answer->HostQuery = host;
    answer->Parent = parent;
    answer->Load(xContent(settings, start_at), with_params);
    answer->Resized();
   }
  }
 }
 return answer;
}

// query is responsible for asking sub-queries to save themselves via xAutoSave ?
xValue xAutoSave(TframeQueryGeneric* query, bool with_params)
{
 xValue answer;
 if (query)
  answer = xVal(query->GetTag(), query->Save(with_params));
 else
  answer = xVal("null", ""); // TODO : dangerous?
 return answer;
}

//---------------------------------------------------------------------------

bool xAutoLoad(TframeActionContainer* host, const xValue & settings, bool with_params, string::size_type start_at)
{
 bool answer = false;
 if (!xTag(settings, start_at).empty())
 {
  a_factory_info temp_factory = action_factories.GetFactoryByTag (xTag(settings, start_at));
  if (temp_factory.create_pointer)
  {
   // if we can't verify, or there's no host against which to verify, or we can and do successfully verify ... create!
   if (!temp_factory.appropriate_pointer ||
       !host ||
        temp_factory.appropriate_pointer(host->SourceTableName.c_str()))
   {
    TframeActionGeneric* thing = NULL;
    thing = temp_factory.create_pointer(host);
    thing->Container = host;
    thing->Load(xContent(settings, start_at), with_params);
    // thing->Resized();
    answer = true;
   }
  }
 }
 return answer;
}

xValue xAutoSave(TframeActionGeneric* action, bool with_params)
{
 xValue answer;
 if (action)
  answer = xVal(action->GetTag(), action->Save(with_params));
 else
  answer = xVal("null", ""); // TODO : dangerous?
 return answer;
}

//---------------------------------------------------------------------------

// WARNING: this will always return at least a one-element thing! if nothing else, a one-element vector of an empty string!
// THIS CODE WORKS -- BUT ON EVEN SLIGHTLY-DEEPLY-NESTED VALUES, IT'S ABOUT AS FAST AS [insert description of something slow and disgusting]!
/*
vector<xValue> xuSplitEscapedSequence(string v, string escape, string separator)
{
 vector<xValue> answer;
 unsigned int e = escape.size();
 unsigned int s = separator.size();
 unsigned int i, j;
 string t;
 string escesc = escape + escape;
 string escsep = escape + separator;
 i = 0; // we'll be selecting from here ...
 j = v.find(separator, i); // to right before here.
 while (j != string::npos)
 {
  // if could be escaped && is escaped, keep looking past the separator that we found
  if (j >= e && v.substr(j - e, e) == escape)
  {
   j = v.find(separator, j + s);
  } else
  // otherwise, grab up until here, move 'i' past the comma, and find next comma from that point on
  {
   t = v.substr(i, j - i);
   t = ReplaceStringWithString(ReplaceStringWithString(t, escsep, separator), escesc, escape);
   answer.push_back(t);
   i = j + s;
   j = v.find(separator, i);
  }
 }
 t = v.substr(i);
 t = ReplaceStringWithString(ReplaceStringWithString(t, escsep, separator), escesc, escape);
 answer.push_back(t);
 return answer;
}
*/

//---------------------------------------------------------------------------

// vector
/*
xValue xFromVec (vector<xValue> v)
{
 for(vector<xValue>::iterator i = v.begin(); i != v.end(); i++)
  (*i) = ReplaceStringWithString(ReplaceStringWithString(*i, "\\", "\\\\"), ",", "\\,"); // "\" -> "\\" and then "," -> "\,"
 return ("vector" + xSepLeft + VecToDelimString(v, "", "", ",") + xSepRight);
}
*/

xValue xFromVec (const vector<xValue> & v)
{
 string content;
 for(vector<xValue>::const_iterator i = v.begin(); i != v.end(); i++)
  content += (*i);
 return xVal("vector", content);
}

/*
vector<xValue> xToVec (xValue v)
{
 vector<xValue> answer;
 if (xTag(v) == "vector")
  answer = xuSplitEscapedSequence(xContent(v), "\\", ",");
 return answer;
}
*/

vector<xValue> xToVec (const xValue & v, string::size_type start_at)
{
 vector<xValue> answer;
 if (xTag(v, start_at) == "vector")
 {
  string::size_type i = v.find(xSepTag, start_at);
  string::size_type j = v.find(xSepSize, i + 1);
  string::size_type next_value = xNextValue(v, start_at);
  string::size_type next_chunk = j + 1;
  while (next_chunk < next_value)
  {
   answer.push_back(xVal(v, next_chunk));
   next_chunk = xNextValue(v, next_chunk);
  }
 }
 return answer;
}

/*
xValue xFromStringVec (vector<string> v)
{
 for(vector<xValue>::iterator i = v.begin(); i != v.end(); i++)
  (*i) = xFromString(*i);
 return xFromVec(v);
}
*/

xValue xFromStringVec (const vector<string> & v)
{
 string content;
 for(vector<string>::const_iterator i = v.begin(); i != v.end(); i++)
  content += xVal("string", (*i));
 return xVal("vector", content);
}

vector<string> xToStringVec (const xValue & v, string::size_type start_at)
{
 vector<string> answer;
 if (xTag(v, start_at) == "vector")
 {
  string::size_type i = v.find(xSepTag, start_at);
  string::size_type j = v.find(xSepSize, i + 1);
  string::size_type next_value = xNextValue(v, start_at);
  string::size_type next_chunk = j + 1;
  while (next_chunk < next_value)
  {
   answer.push_back(xContent(xVal(v, next_chunk)));
   next_chunk = xNextValue(v, next_chunk);
  }
 }
 return answer;
}

//---------------------------------------------------------------------------

// set
/*
xValue xFromSet (set<xValue> v)
{
 for(set<xValue>::iterator i = v.begin(); i != v.end(); i++)
  (*i) = ReplaceStringWithString(ReplaceStringWithString(*i, "\\", "\\\\"), ",", "\\,"); // "\" -> "\\" and then "," -> "\,"
 return ("set" + xSepLeft + SetToDelimString(v, "", "", ",") + xSepRight);
}
*/

xValue xFromSet (const set<xValue> & v)
{
 string content;
 for(set<xValue>::const_iterator i = v.begin(); i != v.end(); i++)
  content += (*i);
 return xVal("set", content);
}

/*
set<xValue> xToSet (xValue v)
{
 set<xValue> answer;
 if (xTag(v) == "set")
 {
  vector<xValue> temp = xuSplitEscapedSequence(xContent(v), "\\", ",");
  VecToSet(temp, answer);
 }
 return answer;
}
*/

set<xValue> xToSet (const xValue & v, string::size_type start_at)
{
 set<xValue> answer;
 if (xTag(v, start_at) == "set")
 {
  string::size_type i = v.find(xSepTag, start_at);
  string::size_type j = v.find(xSepSize, i + 1);
  string::size_type next_value = xNextValue(v, start_at);
  string::size_type next_chunk = j + 1;
  while (next_chunk < next_value)
  {
   answer.insert(xVal(v, next_chunk));
   next_chunk = xNextValue(v, next_chunk);
  }
 }
 return answer;
}

// needs to do extra conversion first (each string needs to be given the 'string' type)
/*
xValue xFromStringSet (set<string> v)
{
 for(set<xValue>::iterator i = v.begin(); i != v.end(); i++)
  (*i) = xFromString(*i);
 return xFromSet(v);
}
*/

xValue xFromStringSet (const set<string> & v)
{
 string content;
 for(set<string>::const_iterator i = v.begin(); i != v.end(); i++)
  content += xVal("string", (*i));
 return xVal("set", content);
}

set<string> xToStringSet (const xValue & v, string::size_type start_at)
{
 set<string> answer;
 if (xTag(v, start_at) == "set")
 {
  string::size_type i = v.find(xSepTag, start_at);
  string::size_type j = v.find(xSepSize, i + 1);
  string::size_type next_value = xNextValue(v, start_at);
  string::size_type next_chunk = j + 1;
  while (next_chunk < next_value)
  {
   answer.insert(xContent(xVal(v, next_chunk)));
   next_chunk = xNextValue(v, next_chunk);
  }
 }
 return answer;
}

//---------------------------------------------------------------------------

// map
/*
xValue xFromMap (map<string, xValue> v)
{
 vector<string> temp;
 for(map<string, xValue>::iterator i = v.begin(); i != v.end(); i++)
  temp.push_back(ReplaceStringWithString(ReplaceStringWithString(i->first + ":" + i->second, "\\", "\\\\"), ",", "\\,")); // "\" -> "\\" and then "," -> "\,"
 return ("map" + xSepLeft + VecToDelimString(temp, "", "", ",") + xSepRight);
}
*/

xValue xFromMap (const map<string, xValue> & v)
{
 string content;
 for(map<string, xValue>::const_iterator i = v.begin(); i != v.end(); i++)
  content += (i->first + xSepMapItemName + i->second);
 return xVal("map", content);
}

/*
map<string, xValue> xToMap (xValue v)
{
 map<string, xValue> answer;
 if (xTag(v) == "map")
 {
  // not technically xValues as they're name:type(value) not just type(value)
  vector<string> temp = xuSplitEscapedSequence(xContent(v), "\\", ",");
  int z;
  for (vector<string>::iterator i = temp.begin(); i != temp.end(); i++)
  {
   z = (*i).find(":");
   answer.insert(pair<string, xValue>((*i).substr(0, z), (*i).substr(z + 1))); // 1 = string(":").size()
  }
 }
 return answer;
}
*/

map<string, xValue> xToMap (const xValue & v, string::size_type start_at)
{
 map<string, xValue> answer;
 if (xTag(v, start_at) == "map")
 {
  string::size_type i = v.find(xSepTag, start_at);
  string::size_type j = v.find(xSepSize, i + 1);
  string::size_type next_value = xNextValue(v, start_at);
  string::size_type next_chunk = j + 1;
  while (next_chunk < next_value)
  {
   string::size_type k = v.find(xSepMapItemName, next_chunk);
   if (k >= next_value)
    break;
   answer[v.substr(next_chunk, k - next_chunk)] = (xVal(v, k + 1));
   next_chunk = xNextValue(v, k + 1);
  }
 }
 return answer;
}

/*
xValue xFromStringMap (map<string, string> v)
{
 for(map<string, xValue>::iterator i = v.begin(); i != v.end(); i++)
  i->second = xFromString(i->second);
 return xFromMap(v);
}
*/

xValue xFromStringMap (const map<string, string> & v)
{
 string content;
 for(map<string, string>::const_iterator i = v.begin(); i != v.end(); i++)
  content += (i->first + xSepMapItemName + xVal("string", i->second));
 return xVal("map", content);
}

map<string, string> xToStringMap (const xValue & v, string::size_type start_at)
{
 map<string, xValue> answer;
 if (xTag(v, start_at) == "map")
 {
  string::size_type i = v.find(xSepTag, start_at);
  string::size_type j = v.find(xSepSize, i + 1);
  string::size_type next_value = xNextValue(v, start_at);
  string::size_type next_chunk = j + 1;
  while (next_chunk < next_value)
  {
   string::size_type k = v.find(xSepMapItemName, next_chunk);
   if (k >= next_value)
    break;
   answer[v.substr(next_chunk, k - next_chunk)] = (xContent(xVal(v, k + 1)));
   next_chunk = xNextValue(v, k + 1);
  }
 }
 return answer;
}

//---------------------------------------------------------------------------

// string
// string(zero)
xValue xFromString (const string & v)
{
 return xVal("string", v);
}

string xToString (const xValue & v, string::size_type start_at)
{
 if (xTag(v, start_at) == "string")
  return xContent(v, start_at);
 else
  return "";
}

//---------------------------------------------------------------------------

// int
// int(45)
xValue xFromInt (int v)
{
 return xVal("int", IntToString(v));
}

int xToInt (const xValue & v, string::size_type start_at)
{
 if (xTag(v, start_at) == "int")
  return StringToInt(xContent(v, start_at));
 else
  return 0;
}

//---------------------------------------------------------------------------

// date
// date(12/11/1980)
xValue xFromPdate (Pdate v)
{
 return xVal("date", v.OutDate(global_mode));
}

Pdate xToPdate (const xValue & v, string::size_type start_at)
{
 if (xTag(v, start_at) == "date")
  return Pdate(xContent(v, start_at), global_mode);
 else
  return Pdate();
}

//---------------------------------------------------------------------------

// time
// time(13:45:00)
xValue xFromPtime (Ptime v)
{
 return xVal("time", v.OutTime(global_mode));
}

Ptime xToPtime (const xValue & v, string::size_type start_at)
{
 if (xTag(v, start_at) == "time")
  return Ptime(xContent(v, start_at), global_mode);
 else
  return Ptime();
}

//---------------------------------------------------------------------------

// float
// float(12.34)
xValue xFromFloat (float v)
{
 return xVal("float", FloatToString(v));
}

float xToFloat (const xValue & v, string::size_type start_at)
{
 if (xTag(v, start_at) == "float")
  return StringToFloat(xContent(v, start_at));
 else
  return 0;
}

//---------------------------------------------------------------------------

// financial
// financial(12.34)
xValue xFromFinancial (financial v)
{
 return xVal("financial", FinancialToString(v));
}

financial xToFinancial (const xValue & v, string::size_type start_at)
{
 if (xTag(v, start_at) == "financial")
  return StringToFinancial(xContent(v, start_at));
 else
  return 0;
}

//---------------------------------------------------------------------------

// bool
// bool(T)
xValue xFromBool (bool v)
{
 return xVal("bool", v ? "T" : "F");
}

bool xToBool (const xValue & v, string::size_type start_at)
{
 if (xTag(v, start_at) == "bool")
  return (xContent(v, start_at) == "T");
 else
  return false;
}

//---------------------------------------------------------------------------

// TODO : other types

//---------------------------------------------------------------------------
xValue xFromCodeDescription (const string & Value)
{
 map<string, xValue> three_parts;

 typedef vector<string> srange;

 vector<string> exact_parts;
 vector<string> like_parts;
 vector<srange> range_parts;

 string clean_value = StripAll(Value, "\a\b\f\n\r\t\v"); // need an entry in 'allsymbols.h' for this -- standard escaped symbols
 vector<string> chunks;
 SplitStringToVec(clean_value, ",", chunks);
 for (vector<string>::iterator i = chunks.begin(); i != chunks.end(); i++)
 {
  (*i) = StripTrailing(StripLeading((*i), " "), " ");
  if (LikeString("_%-_%", (*i)))
  {
   string::size_type z = i->find("-", 0);
   string a = i->substr(0, z);
   string b = i->substr(z + 1);
   srange to_add;
   to_add.resize(2);
   to_add[0] = ReplaceStringWithString(StripTrailing(StripLeading(a, " "), " "), "'", "''");
   to_add[1] = ReplaceStringWithString(StripTrailing(StripLeading(b, " "), " "), "'", "''");
   range_parts.push_back(to_add);
  } else
  if (i->find("%", 0) != string::npos ||
      i->find("_", 0) != string::npos)
  {
   string to_add = ReplaceStringWithString((*i), "'", "''");
   like_parts.push_back(to_add);
  } else
  if (*i != "")
  {
   string to_add = ReplaceStringWithString((*i), "'", "''");
   exact_parts.push_back(to_add);
  }
 }

 vector<xValue> range_parts_final;
 for (vector<srange>::iterator i = range_parts.begin(); i != range_parts.end(); i++)
  range_parts_final.push_back(xFromStringVec(*i));

 if (exact_parts.size())
  three_parts["exact"] = xFromStringVec(exact_parts);
 if (like_parts.size())
  three_parts["like"]  = xFromStringVec(like_parts);
 if (range_parts.size())
  three_parts["range"] = xFromVec(range_parts_final);

 xValue temp_answer = xFromMap(three_parts);
 xValue answer = xVal("code_matches", temp_answer);
 return answer;
}

string xToCodeDescription (const xValue & v, string::size_type start_at)
{
 string answer;

 if (xTag(v, start_at) == "code_matches")
 {
  xValue Value = xContent(v, start_at);

  map<string, xValue> three_parts = xToMap(Value);

  typedef vector<string> srange;
  vector<xValue> range_parts_initial = xToVec(three_parts["range"]);
  for (vector<xValue>::iterator i = range_parts_initial.begin(); i != range_parts_initial.end(); i++)
  {
   if (!answer.empty())
    answer += ", ";

   srange temp = xToVec(*i);
   if (temp.size() == 2)
   {
    string a = ReplaceStringWithString(xToString(temp[0]), "''", "'");
    string b = ReplaceStringWithString(xToString(temp[1]), "''", "'");
    answer += a + " - " + b;
   } else
    ; // PSay("The following condition code range doesn't seem right, please check for missing commas:\n" + (*i), m_debug, MB_OK, "xToCodeDescription");
  }

  vector<xValue> like_parts = xToVec(three_parts["like"]);
  for (vector<xValue>::iterator i = like_parts.begin(); i != like_parts.end(); i++)
  {
   if (!answer.empty())
    answer += ", ";

   string a = ReplaceStringWithString(xToString(*i), "''", "'");
   answer += a;
  }

  vector<xValue> exact_parts = xToVec(three_parts["exact"]);
  for (vector<xValue>::iterator i = exact_parts.begin(); i != exact_parts.end(); i++)
  {
   if (!answer.empty())
    answer += ", ";

   string a = ReplaceStringWithString(xToString(*i), "''", "'");
   answer += a;
  }
 }
 return answer;
}

string xToCodeMatchingSql (const xValue & v, const string & field_name, string::size_type start_at)
{
 string sql;

 if (xTag(v, start_at) == "code_matches")
 {
  xValue Value = xContent(v, start_at);

  map<string, xValue> three_parts = xToMap(Value);

  typedef vector<string> srange;
  vector<xValue> range_parts_initial = xToVec(three_parts["range"]);
  for (vector<xValue>::iterator i = range_parts_initial.begin(); i != range_parts_initial.end(); i++)
  {
   if (!sql.empty())
    sql += " or ";

   srange temp = xToVec(*i);
   if (temp.size() == 2)
   {
    string a = xToString(temp[0]);
    string b = xToString(temp[1]);
    sql += field_name + " between '" + a + "' and '" + b + "'";
   }
  }

  vector<xValue> like_parts = xToVec(three_parts["like"]);
  for (vector<xValue>::iterator i = like_parts.begin(); i != like_parts.end(); i++)
  {
   if (!sql.empty())
    sql += " or ";

    string a = xToString(*i);
    sql += field_name + " like '" + a + "'";
  }

  vector<xValue> exact_parts = xToVec(three_parts["exact"]);
  for (vector<xValue>::iterator i = exact_parts.begin(); i != exact_parts.end(); i++)
  {
   if (!sql.empty())
    sql += " or ";

    string a = xToString(*i);
    sql += field_name + " = '" + a + "'";
  }
 }

 if (!sql.empty())
  sql = "(" + sql + ")";

 return sql;
}
//---------------------------------------------------------------------------

