#ifndef x_serialization_h
#define x_serialization_h
#include "query_factory_registry.h"
#include "action_factory_registry.h"
#include <Controls.hpp>
#include "prometheus_util.h"
#include "util.h"
#include "codeActionContainer.h"

// i swear i didn't pick the 'x' prefix to match 'xml' ... it just seemed out-of-the-way.
// xValues are strings that represent typed values. the format is ...
// type:size:value
// the value is opaque, and must be interpreted in terms of the associated type.

typedef string xValue;
// "separator that comes after..."
// xSepTag comes after the tag
// xSepSize comes after the size
// xSepMapItemName comes after the name of each item in the map
#define xSepTag  string("~")
#define xSepSize string("~")
#define xSepMapItemName string(":")

typedef map<string, xValue> xTuple;

// basic manipulation
xValue       xVal(const string & Tag, const string & Content);
xValue       xVal(const string & v, string::size_type start_at);
unsigned int xNextValue(const xValue & v, string::size_type start_at = 0);
string       xTag(const xValue & v, string::size_type start_at = 0);
unsigned int xSize(const xValue & v, string::size_type start_at = 0);
string       xContent(const xValue & v, string::size_type start_at = 0);

// query
TframeQueryGeneric* xAutoLoad(TframeQueryGeneric* host, TWinControl* parent, const xValue & settings, bool with_params, string::size_type start_at = 0);
xValue xAutoSave(TframeQueryGeneric* query, bool with_params);

// action
bool xAutoLoad(TframeActionContainer* host, const xValue & settings, bool with_params, string::size_type start_at = 0);
xValue xAutoSave(TframeActionGeneric* action, bool with_params);

// utility (thus the 'u' in the name) for later functions
// deprecated for now -- nested escapes were a bad idea performance-wise (physically), though they worked quite well otherwise (logically)
// vector<xValue> xuSplitEscapedSequence(string v, string escape, string separator);

// vector
xValue xFromVec (const vector<xValue> & v);
vector<xValue> xToVec (const xValue & v, string::size_type start_at = 0);
xValue xFromStringVec (const vector<string> & v);
vector<string> xToStringVec (const xValue & v, string::size_type start_at = 0);

// set
xValue xFromSet (const set<xValue> & v);
set<xValue> xToSet (const xValue & v, string::size_type start_at = 0);
xValue xFromStringSet (const set<string> & v);
set<string> xToStringSet (const xValue & v, string::size_type start_at = 0);

// map
xValue xFromMap (const map<string, xValue> & v);
map<string, xValue> xToMap (const xValue & v, string::size_type start_at = 0);
xValue xFromStringMap (const map<string, string> & v);
map<string, string> xToStringMap (const xValue & v, string::size_type start_at = 0);

// string
xValue xFromString (const string & v);
string xToString (const xValue & v, string::size_type start_at = 0);

// int
xValue xFromInt (int v);
int xToInt (const xValue & v, string::size_type start_at = 0);

// date
xValue xFromPdate (Pdate v);
Pdate xToPdate (const xValue & v, string::size_type start_at = 0);

// time
xValue xFromPtime (Ptime v);
Ptime xToPtime (const xValue & v, string::size_type start_at = 0);

// float
xValue xFromFloat (float v);
float xToFloat (const xValue & v, string::size_type start_at = 0);

// financial
xValue xFromFinancial (financial v);
financial xToFinancial (const xValue & v, string::size_type start_at = 0);

// bool
xValue xFromBool (bool v);
bool xToBool (const xValue & v, string::size_type start_at = 0);

// codes (conditions, procedures) -- range, like, and exact matches
xValue xFromCodeDescription (const string & Value);
string xToCodeDescription (const xValue & Value, string::size_type start_at = 0);
string xToCodeMatchingSql (const xValue & Value, const string & field_name, string::size_type start_at = 0);

#define code_matches_help string(\
      "You may enter several codes, ranges of codes, and approximate codes at once.\n"\
      "Use commas to separate the following:\n"\
      "  Exact codes: 250.00, 250.01\n"\
      "  Ranges of codes: 250.00 - 250.99\n"\
      "  Approximate codes: 250.%, 250._3\n"\
      "\n"\
      "The _ character matches any single letter or number (exactly 1 letter).\n"\
      "The % character matches any number of letters or numbers in a row (0 or more letters).\n"\
      "\n"\
      "Spaces are optional (ignored) but make this easier to read.\n"\
      "\n"\
      "Example containing a bit of everything:\n"\
      "250.00, 250.01, 250.10-250.13, 250._2")

// note: each serialization should escape whatever characters it needs to escape in order to parse itself later
// if sets only need to look for "," to split up chunks, only "," need-be escaped to "\,"
// if maps need both ":" and ",", both should be escaped ...
// will this work?
// YES, THIS WORKED -- but it was slow. (note for future reference, when Philip goes having 'good' ideas again)
// note: not trying to build whole tree in memory (HL7), only trying to do one level at a time, each direction

#endif
