//---------------------------------------------------------------------------

#include <vcl.h>
#pragma hdrstop

#include "codeActionContainer.h"
#include "codeActionGeneric.h"
#include "action_factories.h"
#include "db_hardcoded_metadata.h"
#include "query_load_save.h"
//---------------------------------------------------------------------------
#pragma package(smart_init)
#pragma resource "*.dfm"
TframeActionContainer *frameActionContainer;
//---------------------------------------------------------------------------
__fastcall TframeActionContainer::TframeActionContainer(TComponent* Owner)
        : TFrame(Owner)
{
}
//---------------------------------------------------------------------------
__fastcall TframeActionContainer::~TframeActionContainer()
{
 while(!FHostedActions.empty())
  delete FHostedActions[0];
}
//---------------------------------------------------------------------------
void __fastcall TframeActionContainer::DeleteHostedAction (TMessage &Message)
{
 delete (TframeActionGeneric*)(Message.WParam);
}
//---------------------------------------------------------------------------
void __fastcall TframeActionContainer::buttonAddActionClick(
      TObject *Sender)
{
 int z = comboPicker->ItemIndex;
 if (z != -1)
 {
  FORM_TRY
   TframeActionGeneric* _new = NULL;
   _new = picker_content[z].create_pointer(this);
   _new->SourceTableName = SourceTableName;
   _new->Container = this;
   if (IsVisible(_new) && IsEnabled(_new))
    _new->SetFocus();
  FORM_CATCH
 }
}
//---------------------------------------------------------------------------
void __fastcall TframeActionContainer::scrollActionsMouseWheelDown(
      TObject *Sender, TShiftState Shift, TPoint &MousePos, bool &Handled)
{
  if(scrollActions->Visible)
    scrollActions->VertScrollBar->Position += scrollActions->VertScrollBar->Increment;
}
//---------------------------------------------------------------------------
void __fastcall TframeActionContainer::scrollActionsMouseWheelUp(
      TObject *Sender, TShiftState Shift, TPoint &MousePos, bool &Handled)
{
  if(scrollActions->Visible)
    scrollActions->VertScrollBar->Position -= scrollActions->VertScrollBar->Increment;
}
//---------------------------------------------------------------------------
void __fastcall TframeActionContainer::SetSourceTableName (AnsiString Value)
{
 // reload view of appropriate factories
 FSourceTableName = Value;
 picker_content = action_factories.GetFactories(SourceTableName.c_str());
 comboPicker->Items->Clear();
 comboPicker->Text = "";
 bool last_factory_okay = false;
 for (vector<a_factory_info>::iterator i = picker_content.begin(); i != picker_content.end(); i++)
  comboPicker->Items->Add(UppercaseFirstChar(ReplaceStringWithString(i->description, "$", LowercaseString(dbPrettyPlural(LowercaseString(SourceTableName.c_str()))))).c_str());
 // delete or inform
 vector<TframeActionGeneric*>::iterator j;
 for (vector<TframeActionGeneric*>::iterator i = FHostedActions.begin(); i != FHostedActions.end();)
 {
  j = i;
  i++;
  if (!(*j)->HowAppropriate(SourceTableName.c_str()))
   delete (*j);
  else
   (*j)->SourceTableName = SourceTableName;
 }
}
//---------------------------------------------------------------------------
bool TframeActionContainer::Execute (TIBTransaction* dbTransaction, string source_view_name)
{
 bool answer = true;
 for (vector<TframeActionGeneric*>::iterator i = FHostedActions.begin(); answer && i != FHostedActions.end(); i++)
  answer = (answer && (*i)->Execute(dbTransaction, source_view_name));
 return answer;
}
//---------------------------------------------------------------------------
bool TframeActionContainer::Valid ()
{
 bool answer = true;
 // we go all the way to the end, to allow them to color-code things that need attention regardless of what others are doing
 if (FHostedActions.empty())
 {
  answer = false;
  PSay("You have not requested any actions.", m_missing_data, MB_OK, "TframeActionContainer");
 }
 for (vector<TframeActionGeneric*>::iterator i = FHostedActions.begin(); i != FHostedActions.end(); i++)
  answer = (answer && (*i)->Valid());
 return answer;
}
//---------------------------------------------------------------------------
void TframeActionContainer::AddAction(TframeActionGeneric* Action)
{
 FHostedActions.push_back(Action);
 Action->Parent = scrollActions;
 Action->Visible = true;
 ReArrange();
}
//---------------------------------------------------------------------------
void TframeActionContainer::DelAction(TframeActionGeneric* Action)
{
 // might be safer to first make sure it's not here?
 FHostedActions.erase(remove(FHostedActions.begin(), FHostedActions.end(), Action), FHostedActions.end());
 Action->Visible = false;
 Action->Parent = NULL;
 ReArrange();
}
//---------------------------------------------------------------------------
void TframeActionContainer::ReArrange()
{
 for (vector<TframeActionGeneric*>::iterator i = FHostedActions.begin(); i != FHostedActions.end(); i++)
  (*i)->Visible = false;
 int h = 0;
 for (vector<TframeActionGeneric*>::iterator i = FHostedActions.begin(); i != FHostedActions.end(); i++)
 {
  (*i)->Top = h;
  h += (*i)->Height;
  (*i)->Left = 0;
 }
 for (vector<TframeActionGeneric*>::iterator i = FHostedActions.begin(); i != FHostedActions.end(); i++)
  (*i)->Visible = true;
}
//---------------------------------------------------------------------------
void __fastcall TframeActionContainer::comboPickerKeyPress(TObject *Sender,
      char &Key)
{
 bool is_enter = (Key == (char)VK_RETURN); // just in case it gets reset by the next function?
 FindComboBoxItem(Sender, Key);
 if (is_enter)
  buttonAddAction->Click();
}
//---------------------------------------------------------------------------
bool TframeActionContainer::RequestAction (a_factory_create desired_action)
{
 // just make sure it's ... allowed first.
 for (int z = 0; z < picker_content.size(); z++)
 {
  if (picker_content[z].create_pointer == desired_action)
  {
   FORM_TRY
    TframeActionGeneric* _new = NULL;
    _new = desired_action(this);
    _new->SourceTableName = SourceTableName;
    _new->Container = this;
   FORM_CATCH
   return true;
  }
 }
 return false;
}
//---------------------------------------------------------------------------
bool TframeActionContainer::Single ()
{
 return (FHostedActions.size() == 1);
}
//---------------------------------------------------------------------------
string TframeActionContainer::Save (bool with_params)
{
 xValue answer;
 map<string, xValue> config_map;
 config_map["source_table_name"] = xFromString(SourceTableName.c_str());
 vector<xValue> hosted_actions;
 // xAutoSave should automatically tack-on the correct type name for the query as a whole
 for (vector<TframeActionGeneric*>::iterator i = FHostedActions.begin(); i != FHostedActions.end(); i++)
  hosted_actions.push_back(xAutoSave(*i, with_params));
 // do not save an empty set!
 if (hosted_actions.size())
  config_map["hosted_actions"] = xFromVec(hosted_actions);
 answer = xVal("TframeActionContainer", xFromMap(config_map));
 return answer;
}
//---------------------------------------------------------------------------
void TframeActionContainer::Load (string config, bool with_params)
{
 if (xTag(config) == "TframeActionContainer")
 {
  config = xContent(config);

  if (xTag(config) == "map")
  {
   map<string, xValue> config_map = xToMap(config);
   if (xTag(config_map["source_table_name"]) == "string")
   {
    string attempted_stn = xToString(config_map["source_table_name"]);
    if (SourceTableName != "" && SourceTableName != attempted_stn.c_str()) // try to allow xAutoLoad to do its job?
    {
     ::PSay("The actions you are attempting to load are of the wrong type;\nthey are intended to work on " + dbPrettyPlural(LowercaseString(attempted_stn)) + " rather than " + dbPrettyPlural(LowercaseString(SourceTableName.c_str())) + ".", m_invalid_data, MB_OK, "TframeActionContainer::Load");
     return;
    }
    SourceTableName = attempted_stn.c_str();
   }
   // delete old content, as we'll be replacing it (most likely)
   while (FHostedActions.size())
    delete FHostedActions[0];
   // set up the source table name
   if (xTag(config_map["hosted_actions"]) == "vector")
   {
    vector<xValue> hosted_actions = xToVec(config_map["hosted_actions"]);
    for (vector<xValue>::iterator i = hosted_actions.begin(); i != hosted_actions.end(); i++)
     xAutoLoad(this, *i, with_params);
   }
  } else
   ::PSay("Parameters being loaded are of an unexpected format: no property map.", m_invalid_data, MB_OK, "TframeActionContainer::Load", config);
  return;
 } else
   ::PSay("Parameters being loaded are of an unexpected format: not a list of actions.", m_invalid_data, MB_OK, "TframeActionContainer::Load", config);
}
//---------------------------------------------------------------------------
void __fastcall TframeActionContainer::buttonLoadClick(TObject *Sender)
{
 xValue possibly_loaded = QueryLoad(SourceTableName, qf_actions);
 if (!possibly_loaded.empty())
  this->Load(possibly_loaded, true);
}
//---------------------------------------------------------------------------
void __fastcall TframeActionContainer::buttonSaveClick(TObject *Sender)
{
 if (Valid())
  QuerySave(SourceTableName, qf_actions, this->Save(true));
}
//---------------------------------------------------------------------------

