//---------------------------------------------------------------------------

#include <vcl.h>
#pragma hdrstop

#include "codeAction_ModifyResultSet.h"
#include "utilityheaders.h"
#include "db_hardcoded_metadata.h"
//---------------------------------------------------------------------------
#pragma package(smart_init)
#pragma link "codeActionGeneric"
#pragma resource "*.dfm"
TframeAction_ModifyResultSet *frameAction_ModifyResultSet;
//---------------------------------------------------------------------------
__fastcall TframeAction_ModifyResultSet::TframeAction_ModifyResultSet(TComponent* Owner)
        : TframeActionGeneric(Owner)
{
 FHowAppropriate = Appropriate;
 FHowCreate = Create;
 comboAction->ItemIndex = 2; // default = 'set'
}
//---------------------------------------------------------------------------
void __fastcall TframeAction_ModifyResultSet::SetSourceTableName (AnsiString Value)
{
 // remind me why I don't make everything virtual to start with? (this function wasn't, for example. stupid me.)
 TframeActionGeneric::SetSourceTableName(Value);

 // you only get to modify stuff you have permission to modify -- namely, stuff you created ...
 // oh. and the table names have to match. weird, eh?
 vector<map<string, string> > set_entries;
 if (!SqlToVecMap(dbDatabase, "select id, description from user_results where source_table_name = '" + SafeSQLString(LowercaseString(SourceTableName.c_str())) + "'", set_entries))
  PSay("Failed to retrieve the list of available result sets.", m_database_error, MB_OK, "TframeQuery_ByResultSet");
 else
 {
  comboSet->Items->Clear();
  set_ids.clear();
  int j = 0;
  for (vector<map<string, string> >::iterator i = set_entries.begin(); i != set_entries.end(); i++, j++)
  {
   set_ids.push_back((*i)["id"]);
   comboSet->Items->Add(SentenceString((*i)["description"]).c_str());
  }
 }
}
//---------------------------------------------------------------------------
bool TframeAction_ModifyResultSet::Execute (TIBTransaction* dbTransaction, string source_view_name)
{
 string result_id = "";
 if (radioExisting->Checked)
 {
  if (comboSet->ItemIndex > -1 && comboSet->ItemIndex < set_ids.size())
   result_id = set_ids[comboSet->ItemIndex];
 }
 else
 {
  result_id = GetNextId(dbTransaction, "user_results");
  map<string, string> insert_map;
  insert_map["id"] = result_id;
  insert_map["current_id"] = "4";
  insert_map["last_updated_by"] = current_userid;
  insert_map["last_update_stamp"] = RightNow();
  insert_map["source_table_name"] = LowercaseString(SourceTableName.c_str());
  insert_map["description"] = editSet->Text.c_str();
  if (!ExecuteInsertStatement(dbTransaction, "user_results", insert_map))
  {
   PSay("Failed to create new result set.", m_database_error, MB_OK, "TframeAction_ModifyResultSet");
   return false;
  }
 }

 if (!IsBlankOrZero(result_id))
 {
  switch (comboAction->ItemIndex)
  {
   case 0: // add
   {
    string sql = "insert into user_result_items (user_result_id, id) "
                 "select " + result_id + " user_result_id, " + source_view_name + ".id from " + source_view_name + " left join user_result_items on " + source_view_name + ".id = user_result_items.id and user_result_items.user_result_id = " + result_id + " where user_result_items.id is null";
    if (!ExecuteStatement(dbTransaction, sql))
    {
     PSay("Failed while adding new items to a result set.", m_database_error, MB_OK, "TframeAction_ModifyResultSet");
     return false;
    }
    break;
   }
   case 1: // remove
   {
    string sql = "delete from user_result_items where user_result_items.user_result_id = " + result_id + " and user_result_items.id in ("
                 "select " + source_view_name + ".id from " + source_view_name +
                 ")";
    if (!ExecuteStatement(dbTransaction, sql))
    {
     PSay("Failed while removing items from a result set.", m_database_error, MB_OK, "TframeAction_ModifyResultSet");
     return false;
    }
    break;
   }
   case 2: // set (clear and add)
   {
    string sql = "delete from user_result_items where user_result_id = " + result_id;
    if (!ExecuteStatement(dbTransaction, sql))
    {
     PSay("Failed clearing out a result set.", m_database_error, MB_OK, "TframeAction_ModifyResultSet");
     return false;
    }
    sql = "insert into user_result_items (user_result_id, id) "
          "select " + result_id + " user_result_id, " + source_view_name + ".id from " + source_view_name;
    if (!ExecuteStatement(dbTransaction, sql))
    {
     PSay("Failed while inserting items into a cleared result set.", m_database_error, MB_OK, "TframeAction_ModifyResultSet");
     return false;
    }
    break;
   }
  }
  return true;
 } else
  return false;// if nothing failed, ... we're okay. lots of things can fail though.
}
//---------------------------------------------------------------------------
bool TframeAction_ModifyResultSet::Valid ()
{
 comboAction->Color = clWindow;
 comboSet->Color = clWindow;
 editSet->Color = clWindow;

 if (comboAction->ItemIndex == -1)
  comboAction->Color = clBad;

 bool answer = true;
 if (radioExisting->Checked)
 {
  if (comboSet->Text == "" || comboSet->ItemIndex == -1)
  {
   comboSet->Color = clBad;
   answer = false;
  }
 } else
 if (radioCreate->Checked)
 {
  if (editSet->Text == "")
  {
   editSet->Color = clBad;
   answer = false;
  } else
  if (StringToInt(GetOneField(dbDatabase, "select count(*) from user_results where description = '" + SafeSQLString(editSet->Text.c_str()) + "'")) > 0)
  {
   PSay("Another saved result set already exists with this name.\nTo avoid confusion, please enter a different name.", m_duplicate_data, MB_OK, "TframeAction__ModifyResultSet");
   editSet->Color = clBad;
   answer = false;
  }
 } else
  answer = false;

 return TframeActionGeneric::Valid() && (comboAction->Color != clBad) && answer;
}
//---------------------------------------------------------------------------
TframeActionGeneric* TframeAction_ModifyResultSet::Create (TComponent* owner)
{
 TframeActionGeneric* temp;
 FORM_TRY
  temp = new TframeAction_ModifyResultSet(owner);
 FORM_CATCH
 return temp;
}
//---------------------------------------------------------------------------
bool TframeAction_ModifyResultSet::Appropriate (string source_table_name)
{
 return dbIsAvailable(source_table_name);
}
//---------------------------------------------------------------------------
void __fastcall TframeAction_ModifyResultSet::comboActionKeyPress(
      TObject *Sender, char &Key)
{
 FindComboBoxItem(Sender, Key);
}
//---------------------------------------------------------------------------
void __fastcall TframeAction_ModifyResultSet::comboSetKeyPress(
      TObject *Sender, char &Key)
{
 FindComboBoxItem(Sender, Key);
}
//---------------------------------------------------------------------------

