//---------------------------------------------------------------------------

#include <vcl.h>
#pragma hdrstop

#include "codeEnrollmentDiabRegistry.h"
#include "codeRecordEnrollment.h"
#include "codeRecordPerson.h"
#include "derivedheaders.h"
//---------------------------------------------------------------------------
#pragma package(smart_init)
#pragma link "codeEnrollmentGeneric"
#pragma link "codeRubberContainer"
#pragma link "codePaddedBox"
#pragma resource "*.dfm"
TformEnrollmentDiabRegistry *formEnrollmentDiabRegistry;
//---------------------------------------------------------------------------
__fastcall TformEnrollmentDiabRegistry::TformEnrollmentDiabRegistry(TComponent* Owner)
        : TformEnrollmentGeneric(Owner)
{
  instructions.push_back("Welcome to the diabetes registry.  If this is your first time using this screen, read the instructions below to get started.");
  instructions.push_back("This is the current diabetes registry.  Refer to the Welcome tab for specific instructions.");
  instructions.push_back("This list includes everyone diagnosed with diabetes not currently on the diabetes registry.");
  instructions.push_back("This list includes everyone not diagnosed with diabetes but at risk for diabetes, not currently on the diabetes registry.");

  InfoBarText = instructions[pageEnrollment->ActivePage->TabIndex];

  // we're using temp tables, so we'll be using a single transaction across all filters
  our_transaction = new TIBTransaction(NULL);
  StdTransaction(our_transaction, dbDatabase);

  classFilterObject *filter;

  listMembers->MultiSelect = true;
  listNewDiabetics->MultiSelect = true;
  listAtRisk->MultiSelect = true;

  filter = filter_atlas_handler["registry"].filter;

  RegistryDataSource = NULL;
  FORM_TRY
    RegistryDataSource = new classFilterRegistrySource();
  FORM_CATCH
  RegistryDataSource->SetFilter(filter);

  filter->SetListViewHandle(listMembers);
  filter->SetTransaction(our_transaction);
  filter->SetSecurity("enrollment");
  filter->AddColumnSpec("Name", "people", as_text, true, 150);
  filter->AddColumnSpec("DOB", "people", as_date,  true, 75);
  filter->AddColumnSpec("DOD", "people", as_date,  true, 75);
  filter->AddColumnSpec("Visits in Past Year", "tasks", as_int, true, 30);
  filter->AddColumnSpec("PCP", "tax_entities", as_text, true, 150, rawString);
  filter->AddColumnSpec("Most Recent Provider", "tax_entities", as_text, false, 150, rawString);
  filter->AddColumnSpec("Most Used Provider (in past year)", "tax_entities", as_text, false, 150, rawString);
  filter->AddColumnSpec("F/U in Past Year", "tasks", as_int, true, 40);
  filter->AddColumnSpec("DM Client", "people", as_text, true, 65, prettyBoolString);
  filter->AddColumnSpec("Status", "enrollment", as_text, true, 100);
  filter->AddColumnSpec("Suggested Status", "enrollment", as_text, true, 105);
  filter->AddColumnSpec("Current Id", "current_id", as_int, false, 80);
  filter->AddColumnSpec("Internal Id", "internal_id", as_int, false, 80);
  filter->FetchPreferences("specialdiabregistry.registry");
  filter->SetCheck(false);
  filter->SetCompress(true);
  filter->SetAddEdit(this, NULL, EditRecord(&(TformEnrollmentDiabRegistry::EditEnrollment)));
  filter->SetInform(this, InformForm(&(TformEnrollmentDiabRegistry::DoFilterStuff)));
  filter->SetHint("double click row to open that person's Diabetes Registry enrollment record");
  filter->GetListViewHandle()->ShowHint = true;
  //filter->SetSQLStatement(registry_sql);
  filter->SetDataSource(RegistryDataSource);
  RegistryDataSource->only_show_registry_changes = checkRegistryChanges->Checked;
  filter->RefreshDataSet();
  //filter->CheckAll();

  // visits in the past year
  // PCP is one of our internal referring physicians
  // # of f/u attempts in the past year

  filter = filter_atlas_handler["new_diabetics"].filter;

  NewDiabeticDataSource = NULL;
  FORM_TRY
    NewDiabeticDataSource = new classFilterNewDiabeticSource();
  FORM_CATCH
  NewDiabeticDataSource->SetFilter(filter);

  filter->SetListViewHandle(listNewDiabetics);
  filter->SetTransaction(our_transaction);
  filter->SetSecurity("people");
  filter->AddColumnSpec("Name", "people", as_text, true, 150);
  filter->AddColumnSpec("DOB", "people", as_date,  true, 75);
  filter->AddColumnSpec("Visits in Past Year", "tasks", as_int, true, 80);
  filter->AddColumnSpec("Current Id", "current_id", as_int, false, 80);
  filter->AddColumnSpec("Internal Id", "internal_id", as_int, false, 80);
  filter->FetchPreferences("specialdiabregistry.registry");
  filter->SetCheck(false);
  filter->SetCompress(true);
  filter->SetAddEdit(this, NULL, EditRecord(&(TformEnrollmentDiabRegistry::EditNewDiabetic)));
  filter->SetInform(this, InformForm(&(TformEnrollmentDiabRegistry::DoFilterStuff)));
  filter->SetHint("double click row to open that person's record screen");
  filter->GetListViewHandle()->ShowHint = true;
  //filter->SetSQLStatement(sql["new_dm"]);
  filter->SetDataSource(NewDiabeticDataSource);
  NewDiabeticDataSource->only_show_positive_visits = checkNewDiabeticVisits->Checked;
  filter->RefreshDataSet();
  //filter->CheckAll();

  AtRiskDataSource = NULL;
  FORM_TRY
    AtRiskDataSource = new classFilterAtRiskSource();
  FORM_CATCH

  filter = filter_atlas_handler["at_risk"].filter;
  filter->SetListViewHandle(listAtRisk);
  filter->SetTransaction(our_transaction);
  filter->SetSecurity("enrollment");
  filter->AddColumnSpec("Name", "people", as_text, true, 150);
  filter->AddColumnSpec("DOB", "people", as_date,  true, 75);
  filter->AddColumnSpec("Visits in Past Year", "tasks", as_int, true, 80);
  filter->AddColumnSpec("Current Id", "current_id", as_int, false, 80);
  filter->AddColumnSpec("Internal Id", "internal_id", as_int, false, 80);
  filter->FetchPreferences("specialdiabregistry.registry");
  filter->SetCheck(false);
  filter->SetCompress(true);
  filter->SetAddEdit(this, NULL, EditRecord(&(TformEnrollmentDiabRegistry::EditAtRisk)));
  filter->SetInform(this, InformForm(&(TformEnrollmentDiabRegistry::DoFilterStuff)));
  filter->SetHint("double click row to open that person's record screen");
  filter->GetListViewHandle()->ShowHint = true;
  filter->SetDataSource(AtRiskDataSource);
  //filter->SetSQLStatement(sql["at_risk"]);
  //listRegistry->MultiSelect = true;
  AtRiskDataSource->only_show_positive_visits = checkAtRiskVisits->Checked;
  filter->RefreshDataSet();

  enrollment_request = atlas->RequestNotificationWhen("enrollment.id", "", rel_any, as_int, this,
                       RefreshHandler(&(TformEnrollmentDiabRegistry::AtlasRefreshListView)));
}
//---------------------------------------------------------------------------
void __fastcall TformEnrollmentDiabRegistry::FormClose(TObject *Sender, TCloseAction &Action)
{
  try
  {
    if (our_transaction)
    {
      // once more for good measure:
      ExecuteStatement(our_transaction, "delete from temp_enrollments");
      if (our_transaction->Active)
        our_transaction->Commit();
      delete our_transaction;
    }
  }
  catch (...) { }

  TformSpecialGeneric::FormClose(Sender, Action);
}
//---------------------------------------------------------------------------
void __fastcall TformEnrollmentDiabRegistry::pageEnrollmentChange(TObject *Sender)
{
  InfoBarText = instructions[pageEnrollment->ActivePage->TabIndex];
}
//---------------------------------------------------------------------------
void __fastcall TformEnrollmentDiabRegistry::FormShow(TObject *Sender)
{
  ShowInfoBar();
}
//---------------------------------------------------------------------------
void TformEnrollmentDiabRegistry::EditEnrollment()
{
  map<string, string> temp_map;
  temp_map["id"] = filter_atlas_handler["registry"].filter->GetSelectedRecordID();
  FORM_TRY
    TformRecordEnrollment *view_enrollment = new TformRecordEnrollment(NULL, temp_map);
    view_enrollment->Show();
  FORM_CATCH
}
//---------------------------------------------------------------------------
void TformEnrollmentDiabRegistry::EditNewDiabetic()
{
  map<string, string> temp_map;
  temp_map["id"] = filter_atlas_handler["new_diabetics"].filter->GetSelectedRecordID();
  FORM_TRY
    TformRecordPerson * view_person = new TformRecordPerson(NULL, temp_map);
    view_person->Show();
  FORM_CATCH
}
//---------------------------------------------------------------------------
void TformEnrollmentDiabRegistry::EditAtRisk()
{
  map<string, string> temp_map;
  temp_map["id"] = filter_atlas_handler["at_risk"].filter->GetSelectedRecordID();
  FORM_TRY
    TformRecordPerson * view_person = new TformRecordPerson(NULL, temp_map);
    view_person->Show();
  FORM_CATCH
}
//---------------------------------------------------------------------------
void TformEnrollmentDiabRegistry::DoFilterStuff()
{
  vector <string> sections;
  sections.push_back("Diabetes Registry"); // always
  if (filter_atlas_handler["registry"].filter->HasData())
   sections.push_back(IntToString(filter_atlas_handler["registry"].filter->GetResultSize()) + " enrolled");
  if (filter_atlas_handler["new_diabetics"].filter->HasData())
   sections.push_back(IntToString(filter_atlas_handler["new_diabetics"].filter->GetResultSize()) + " new diabetics");
  if (filter_atlas_handler["at_risk"].filter->HasData())
   sections.push_back(IntToString(filter_atlas_handler["at_risk"].filter->GetResultSize()) + " at risk");
  this->Caption = VecToDelimString(sections, "", "", " - ").c_str();
  SetUpAtlasRequests();
}
//---------------------------------------------------------------------------
void TformEnrollmentDiabRegistry::SetUpAtlasRequests()
{
  vector<string> selected_id_vec = filter_atlas_handler["registry"].filter->GetAllID();
  for(vector<string>::iterator itr = selected_id_vec.begin(); itr != selected_id_vec.end(); itr++)
  {
    atlas->RequestNotificationWhen("enrollment.person_id", *itr, rel_e, as_int, this,
    RefreshHandler(&(TformEnrollmentDiabRegistry::AtlasRefreshListView)));
  }
}
//---------------------------------------------------------------------------
bool TformEnrollmentDiabRegistry::AtlasRefreshListView(int request_num, string matched_value)
{
  HandleAtlasRequest(request_num);

  if (request_num == enrollment_request)
  {
    // okay ... now ... how do we know what the change actually WAS?
    // we really need the itemvalue passed back to us here....
    // philip says the API doesn't do that.  so screw it...we'll just TELL them
    // there was a change and they can update at their convenience:
    panelRegistry->Visible = true;
    panelNewDiabetics->Visible = true;
    panelAtRisk->Visible = true;
    // ...
    // Hey look, I made the API do that now.  matched_value (above) is the
    // changed record.  now what to do with it?  suddenly I feel all lazy.
    // TODO: is matched_value an enrollments.id or what?  Could we use it to somehow
    // update the single row that has changed, rather than fetching the whole dataset again?
    // NB: Yes, I went to the trouble to update atlas to return this info, then did nothing
    // with it.  You're welcome to fix it.  :P
  }

  //filter_atlas_handler["registry"].filter->RefreshDataSet();
  //filter_atlas_handler["new_diabetics"].filter->RefreshDataSet();
  //filter_atlas_handler["at_risk"].filter->RefreshDataSet();

  return true;
}
//---------------------------------------------------------------------------
void __fastcall TformEnrollmentDiabRegistry::btnRefreshRegistryClick(TObject *Sender)
{
  filter_atlas_handler["at_risk"].filter->RefreshDataSet();
  panelAtRisk->Visible = false;
}
//---------------------------------------------------------------------------
void __fastcall TformEnrollmentDiabRegistry::btnRefreshNewDiabeticsClick(TObject *Sender)
{
  filter_atlas_handler["new_diabetics"].filter->RefreshDataSet();
  panelNewDiabetics->Visible = false;
}
//---------------------------------------------------------------------------
void __fastcall TformEnrollmentDiabRegistry::btnRefreshAtRiskClick(TObject *Sender)
{
  filter_atlas_handler["at_risk"].filter->RefreshDataSet();
  panelAtRisk->Visible = false;
}
//---------------------------------------------------------------------------
void __fastcall TformEnrollmentDiabRegistry::btnRefreshAllClick(TObject *Sender)
{
  filter_atlas_handler["registry"].filter->RefreshDataSet();
  filter_atlas_handler["new_diabetics"].filter->RefreshDataSet();
  filter_atlas_handler["at_risk"].filter->RefreshDataSet();
  panelRegistry->Visible = false;
  panelNewDiabetics->Visible = false;
  panelAtRisk->Visible = false;
}
//---------------------------------------------------------------------------
void __fastcall TformEnrollmentDiabRegistry::checkRegistryChangesClick(TObject *Sender)
{
  RegistryDataSource->only_show_registry_changes = checkRegistryChanges->Checked;
  filter_atlas_handler["registry"].filter->RefreshDataSet();
}
//---------------------------------------------------------------------------
void __fastcall TformEnrollmentDiabRegistry::checkNewDiabeticVisitsClick(TObject *Sender)
{
  NewDiabeticDataSource->only_show_positive_visits = checkNewDiabeticVisits->Checked;
  filter_atlas_handler["new_diabetics"].filter->RefreshDataSet();
}
//---------------------------------------------------------------------------
void __fastcall TformEnrollmentDiabRegistry::checkAtRiskVisitsClick(TObject *Sender)
{
  AtRiskDataSource->only_show_positive_visits = checkAtRiskVisits->Checked;
  filter_atlas_handler["at_risk"].filter->RefreshDataSet();
}
//---------------------------------------------------------------------------



//---------------------------------------------------------------------------
classFilterRegistrySource::classFilterRegistrySource() : classFilterEnrollmentSource()
{
}
//---------------------------------------------------------------------------
bool classFilterRegistrySource::Begin(TIBTransaction * dbTransaction)
{ // note:  dbTransaction is the active transaction passed by filter, which will be our_transaction above
  // some of this is duplicated from up above cause ... fuck all
  Pdate pointless_object = CurrentPdate();
  pointless_object.mod(-1, in_years);
  string year_begin = pointless_object.OutDate(mdy | yyyy);

  if (!dbTransaction->Active)
    dbTransaction->StartTransaction();

  ExecuteStatement(dbTransaction, "delete from temp_enrollments");

/*
  sql["visits"] = "select count(tasks.id) "
                  "from tasks "
                  "where tasks.person_id = people.id and tasks.current_id+1 >= 4 "
                  "and method_id = " + lookupTables->IdOf("method_types", "Visit") + " "
                  "and tasks.group_id in (" + lookupTables->IdOf("groups", "Diabetes") + "," +
                  lookupTables->IdOf("groups", "Little Axe Clinic") + "," + lookupTables->IdOf("groups", "Shawnee Clinic") + ") "
                  "and tasks.date_of_task between '" + year_begin + "' and current_date";

  // count of diabetes follow ups in the past year
  sql["fu"] = "select count(tasks.id) "
              "from tasks "
              "inner join diab_activity_types on tasks.diab_activity_id = diab_activity_types.id "
              "where tasks.person_id = people.id and tasks.current_id+1 >= 4 "
              "and diab_activity_types.item like '%FOLLOW UP%' "
              "and tasks.group_id in (" + lookupTables->IdOf("groups", "Diabetes") + ") "
              "and tasks.date_of_task between '" + year_begin + "' and current_date ";

  sql["last_visit"] = "select max(date_of_task) "
                      "from tasks "
                      "where person_id = people.id and method_id = " + lookupTables->IdOf("method_types", "Visit") + " "
                      "and tasks.group_id in (" + lookupTables->IdOf("groups", "Diabetes") + "," +
                      lookupTables->IdOf("groups", "Little Axe Clinic") + "," + lookupTables->IdOf("groups", "Shawnee Clinic") + "," +
                      lookupTables->IdOf("groups", "Contract Health") + ") " // include outside visits as well; at least, for now
                      "and tasks.date_of_task between '" + year_begin + "' and current_date";

  vector<string> internal_providers;                                               // kind of a hack, but ... eh.
  sql["internal_referring"] = "select id from tax_entities where (is_referring = 1 or name like 'ABSENTEE SHAWNEE %')";
  SqlToVec(dbTransaction, sql["internal_referring"], internal_providers);

//  sql["registry"] = "select first_name || (case when middle_name != '' then ' ' else '' end) || middle_name || (case when last_name != '' then ' ' else '' end) || last_name || (case when suffix != '' then ' ' else '' end) || suffix name, "
//                    "birth_date, death_date, (" + sql["visits"] + "), provider.name, (" + sql["fu"] + "), enrollment_status_types.item, "
//                    "enrollment.current_id, enrollment.id "
//                    "from enrollment "
//                    "inner join enrollment_status_types on enrollment.enrollment_status_id = enrollment_status_types.id "
//                    "inner join people on enrollment.person_id = people.id "
//                    "inner join person_names on people.person_names_id = person_names.id "
//                    "left join tax_entities provider on enrollment.provider_id = provider.id "
//                    "where enrollment.enrollment_id = " + lookupTables->IdOf("enrollment_types", "On Diabetes Registry");


  // override!  now you're thinking with temp tables.
  sql["registry.temp"] = "insert into temp_enrollments (id, person_id, person_name, dob, dod, visits_last_year, provider_id, "
                         "provider_name, follow_ups_last_year, last_visit_date, diabetes_client, status, suggested_status) "
                         "select enrollment.id, people.id, "
                         "first_name || (case when middle_name != '' then ' ' else '' end) || middle_name || (case when last_name != '' then ' ' else '' end) || last_name || (case when suffix != '' then ' ' else '' end) || suffix name, "
                         "birth_date, death_date, (" + sql["visits"] + "), provider.id, provider.name, (" + sql["fu"] + "), "
                         "(" + sql["last_visit"] + "), is_diabetes_client, enrollment_status_types.item, null "
                         "from enrollment "
                         "inner join enrollment_status_types on enrollment.enrollment_status_id = enrollment_status_types.id "
                         "inner join people on enrollment.person_id = people.id "
                         "inner join person_names on people.person_names_id = person_names.id "
                         "left join tax_entities provider on enrollment.provider_id = provider.id "
                         "where enrollment.enrollment_id = " + lookupTables->IdOf("enrollment_types", "On Diabetes Registry") + " " +
                         "and enrollment.current_id+1 >= 4 ";
*/

  set<string> referrers, employees, internal_providers;                            // kind of a hack, but ... eh.
  string provider_sql;
  // all referring physicians (and then some)
  provider_sql = "select id from tax_entities where (is_referring = 1 or upper(name) like 'ABSENTEE SHAWNEE %')";
  SqlToSet(dbTransaction, provider_sql, referrers);
  // employees (often used as "provider" on appointments/tasks as well)
  provider_sql = "select tax_entity_id from users " /* where valid = 1 */ ; // ?
  SqlToSet(dbTransaction, provider_sql, employees);
  // with your powers combined, I am Captain ... er ... both sets combined (union)
  internal_providers = add(referrers, employees);

  // override!  now you're thinking with temp tables.
  sql["registry.temp"] = "insert into temp_enrollments (id, person_id, person_name, dob, dod, provider_id, "
                         "provider_name, diabetes_client, status, suggested_status) "
                         "select registry_enrollment.id, people.id, "
                         "first_name || (case when middle_name != '' then ' ' else '' end) || middle_name || (case when last_name != '' then ' ' else '' end) || last_name || (case when suffix != '' then ' ' else '' end) || suffix name, "
                         "birth_date, death_date, provider.id, provider.name, "
                         "case when client_enrollment.id is not null then 1 else 0 end is_diabetes_client, "
                         "enrollment_status_types.item, null "
                         "from enrollment registry_enrollment "
                         "inner join enrollment_status_types on registry_enrollment.enrollment_status_id = enrollment_status_types.id "
                         "inner join people on registry_enrollment.person_id = people.id "
                         "inner join person_names on people.person_names_id = person_names.id "
                         "left join tax_entities provider on registry_enrollment.provider_id = provider.id "
                         "left join enrollment client_enrollment on client_enrollment.person_id = people.id and client_enrollment.current_id+1 >= 4 and "
                         " client_enrollment.enrollment_id = '" + lookupTables->IdOf("enrollment_types", "Diabetes Client") + "' "
                         "where registry_enrollment.enrollment_id = " + lookupTables->IdOf("enrollment_types", "On Diabetes Registry") + " " +
                         "and registry_enrollment.current_id+1 >= 4 ";

  DebugSql(NULL, &(sql["registry.temp"]), other_forms, 9);
  ExecuteStatement(dbTransaction, sql["registry.temp"]);

  sql["registry.temp_update"] = "update temp_enrollments set visits_last_year = ("
                                "select count(tasks.id) "
                                "from tasks "
                                "where tasks.person_id = temp_enrollments.person_id and tasks.current_id+1 >= 4 "
                                "and method_id = " + lookupTables->IdOf("method_types", "Visit") + " "
                                "and tasks.group_id in (" + lookupTables->IdOf("groups", "Diabetes") + "," +
                                lookupTables->IdOf("groups", "Little Axe Clinic") + "," + lookupTables->IdOf("groups", "Shawnee Clinic") + ") "
                                "and tasks.date_of_task between '" + year_begin + "' and current_date"
                                ")";
  DebugSql(NULL, &(sql["registry.temp_update"]), other_forms, 9);
  ExecuteStatement(dbTransaction, sql["registry.temp_update"]);

  // count of diabetes follow ups in the past year
  sql["registry.temp_update"] = "update temp_enrollments set follow_ups_last_year = ("
                                "select count(tasks.id) "
                                "from tasks "
                                "inner join diab_activity_types on tasks.diab_activity_id = diab_activity_types.id "
                                "where tasks.person_id = temp_enrollments.person_id and tasks.current_id+1 >= 4 "
                                "and diab_activity_types.item like '%FOLLOW UP%' "
                                "and tasks.group_id in (" + lookupTables->IdOf("groups", "Diabetes") + ") "
                                "and tasks.date_of_task between '" + year_begin + "' and current_date "
                                ")";
  DebugSql(NULL, &(sql["registry.temp_update"]), other_forms, 9);
  ExecuteStatement(dbTransaction, sql["registry.temp_update"]);

  sql["registry.temp_update"] = "update temp_enrollments set last_visit_date = ("
                                "select max(date_of_task) "
                                "from tasks "
                                "where person_id = temp_enrollments.person_id and method_id = " + lookupTables->IdOf("method_types", "Visit") + " "
                                "and tasks.group_id in (" + lookupTables->IdOf("groups", "Diabetes") + "," +
                                lookupTables->IdOf("groups", "Little Axe Clinic") + "," + lookupTables->IdOf("groups", "Shawnee Clinic") + "," +
                                lookupTables->IdOf("groups", "Contract Health") + ") " // include outside visits as well; at least, for now
                                "and tasks.date_of_task between '" + year_begin + "' and current_date"
                                ")";
  DebugSql(NULL, &(sql["registry.temp_update"]), other_forms, 9);
  ExecuteStatement(dbTransaction, sql["registry.temp_update"]);

  sql["registry.temp_update"] = "update temp_enrollments set most_recent_provider_id = ("
                                "select first 1 tasks.tax_entity_id "
                                "from tasks "
                                "inner join tax_entities on tasks.tax_entity_id = tax_entities.id "
                                "where person_id = temp_enrollments.person_id and method_id = " + lookupTables->IdOf("method_types", "Visit") + " "
                                "and tasks.group_id in (" + lookupTables->IdOf("groups", "Diabetes") + "," +
                                lookupTables->IdOf("groups", "Little Axe Clinic") + "," + lookupTables->IdOf("groups", "Shawnee Clinic") + "," +
                                lookupTables->IdOf("groups", "Contract Health") + ") " // include outside visits as well; at least, for now
                                "and tasks.date_of_task between '" + year_begin + "' and current_date "
                                "order by date_of_task desc"
                                ")";
  DebugSql(NULL, &(sql["registry.temp_update"]), other_forms, 9);
  ExecuteStatement(dbTransaction, sql["registry.temp_update"]);

  sql["registry.temp_update"] = "update temp_enrollments set most_recent_provider = ("
                                "select name from tax_entities where id = temp_enrollments.most_recent_provider_id"
                                ")";
  DebugSql(NULL, &(sql["registry.temp_update"]), other_forms, 9);
  ExecuteStatement(dbTransaction, sql["registry.temp_update"]);

  sql["registry.temp_update"] = "update temp_enrollments set most_frequent_provider_id = ("
                                "select first 1 tasks.tax_entity_id "
                                "from tasks "
                                "inner join tax_entities on tasks.tax_entity_id = tax_entities.id "
                                "where person_id = temp_enrollments.person_id and method_id = " + lookupTables->IdOf("method_types", "Visit") + " "
                                "and tasks.group_id in (" + lookupTables->IdOf("groups", "Diabetes") + "," +
                                lookupTables->IdOf("groups", "Little Axe Clinic") + "," + lookupTables->IdOf("groups", "Shawnee Clinic") + "," +
                                lookupTables->IdOf("groups", "Contract Health") + ") " // include outside visits as well; at least, for now
                                "and tasks.date_of_task between '" + year_begin + "' and current_date "
                                "group by 1 order by count(tasks.id) desc"
                                ")";
  DebugSql(NULL, &(sql["registry.temp_update"]), other_forms, 9);
  ExecuteStatement(dbTransaction, sql["registry.temp_update"]);

  sql["registry.temp_update"] = "update temp_enrollments set most_frequent_provider = ("
                                "select name from tax_entities where id = temp_enrollments.most_frequent_provider_id"
                                ")";
  DebugSql(NULL, &(sql["registry.temp_update"]), other_forms, 9);
  ExecuteStatement(dbTransaction, sql["registry.temp_update"]);

  // We need the most recent date they moved, if in the past year
  sql["registry.temp_update"] = "update temp_enrollments set last_move_date = "
                                "(select cast(max(change_stamp) as date) "
                                "from enrollment "
                                "inner join enrollment_changes on enrollment_changes.enrollment_id = enrollment.id "
                                "where enrollment.enrollment_id = " + lookupTables->IdOf("enrollment_types", "On Diabetes Registry") + " "
                                "and enrollment.person_id = temp_enrollments.person_id and enrollment_changes.change_stamp > '" + year_begin + "' "
                                "and enrollment_changes.enrollment_change_id = " + lookupTables->IdOf("enrollment_change_types", "Moved") + ")";
  DebugSql(NULL, &(sql["registry.temp_update"]), other_forms, 9);
  ExecuteStatement(dbTransaction, sql["registry.temp_update"]);

  // We need the most recent date they moved, if in the past year
  sql["registry.temp_update"] = "update temp_enrollments set last_provider_change_date = "
                                "(select cast(max(change_stamp) as date) "
                                "from enrollment "
                                "inner join enrollment_changes on enrollment_changes.enrollment_id = enrollment.id "
                                "where enrollment.enrollment_id = " + lookupTables->IdOf("enrollment_types", "On Diabetes Registry") + " "
                                "and enrollment.person_id = temp_enrollments.person_id and enrollment_changes.change_stamp > '" + year_begin + "' "
                                "and enrollment_changes.enrollment_change_id = " + lookupTables->IdOf("enrollment_change_types", "PCP Change") + ")";
  DebugSql(NULL, &(sql["registry.temp_update"]), other_forms, 9);
  ExecuteStatement(dbTransaction, sql["registry.temp_update"]);

  // DECEASED has got to be the easiest set of criteria, so let's get it out of the way...
  sql["registry.temp_update"] = "update temp_enrollments set suggested_status = 'Deceased' "
                                "where dod is not null ";
  DebugSql(NULL, &(sql["registry.temp_update"]), other_forms, 9);
  ExecuteStatement(dbTransaction, sql["registry.temp_update"]);

  // ACTIVE has at least one visit in the past year, and their PCP is "us"
  sql["registry.temp_update"] = "update temp_enrollments set suggested_status = 'Active' "
                                "where suggested_status is null and visits_last_year > 0 "
                                "and provider_id > 0 and provider_id in (" + SetToCommaString(internal_providers) + ") "
                                "and last_visit_date is not null and (last_move_date is null or last_visit_date > last_move_date) "
                                "and (last_provider_change_date is null or last_visit_date > last_provider_change_date) ";
  DebugSql(NULL, &(sql["registry.temp_update"]), other_forms, 9);
  ExecuteStatement(dbTransaction, sql["registry.temp_update"]);

  // TRANSIENT has at least one visit in the past year, and their PCP is NOT "us"
  sql["registry.temp_update"] = "update temp_enrollments set suggested_status = 'Transient' "
                                "where suggested_status is null and visits_last_year > 0 "
                                "and provider_id > 0 and provider_id not in (" + SetToCommaString(internal_providers) + ") ";
                                "and last_visit_date is not null and (last_move_date is null or last_visit_date > last_move_date) "
                                "and (last_provider_change_date is null or last_visit_date > last_provider_change_date) ";
  DebugSql(NULL, &(sql["registry.temp_update"]), other_forms, 9);
  ExecuteStatement(dbTransaction, sql["registry.temp_update"]);

  // LOST TO FOLLOW-UP has no visits in the past year, and < 3 follow-up attempts in the past year
  sql["registry.temp_update"] = "update temp_enrollments set suggested_status = 'Lost to Follow-Up' "
                                "where suggested_status is null and visits_last_year = 0 "
                                "and follow_ups_last_year < 3 ";
  DebugSql(NULL, &(sql["registry.temp_update"]), other_forms, 9);
  ExecuteStatement(dbTransaction, sql["registry.temp_update"]);

  // INACTIVE means one of three things:
  // 1) No visits in the past year, or they've moved more recently than their last visit in the past year
  // 2) No visits in the past year, or they've notified us that they've changed providers more recently than their last visit in the past year
  // 3) We've lost contact with them and have given up (PCP = us; no visits in the past year; and >= 3 follow-up attempts in the past year)
  sql["registry.temp_update"] = "update temp_enrollments set suggested_status = 'Inactive' "
                                "where suggested_status is null and "
                                "("
                                  "(last_move_date is not null and (last_visit_date is null or last_move_date > last_visit_date)) OR "
                                  "(last_provider_change_date is not null and (last_visit_date is null or last_provider_change_date > last_visit_date)) OR "
                                  "(visits_last_year = 0 and provider_id > 0 and provider_id in (" + SetToCommaString(internal_providers) + ") and follow_ups_last_year < 3) "
                                ")";
  DebugSql(NULL, &(sql["registry.temp_update"]), other_forms, 9);
  ExecuteStatement(dbTransaction, sql["registry.temp_update"]);

  // We'll be nice and indicate WHY the enrollment has an indeterminate status, since we know:
  sql["registry.temp_update"] = "update temp_enrollments set suggested_status = '>no PCP assigned<' "
                                "where suggested_status is null ";
  DebugSql(NULL, &(sql["registry.temp_update"]), other_forms, 9);
  ExecuteStatement(dbTransaction, sql["registry.temp_update"]);

  // everyone who wasn't marked with one of the above statuses is just going to have to be looked at manually, and their data fixed
  sql["registry.temp_update"] = "update temp_enrollments set suggested_status = '>indeterminate<' "
                                "where suggested_status is null ";
  DebugSql(NULL, &(sql["registry.temp_update"]), other_forms, 9);
  ExecuteStatement(dbTransaction, sql["registry.temp_update"]);

  // aahhh.
  sql["registry"] = "select person_name, dob, dod, visits_last_year, provider_name, most_recent_provider, most_frequent_provider, "
                    "follow_ups_last_year, diabetes_client, status, suggested_status, 4, id "
                    "from temp_enrollments ";
  if (only_show_registry_changes)
    sql["registry"] += "where UPPER(suggested_status) != UPPER(status) ";
  DebugSql(NULL, &(sql["registry"]), other_forms, 9);
  QueryData(NULL, dbTransaction, sql["registry"]);

  // setup default fields
  field_names.clear();
  field_types.clear();
  field_values.clear();
  row_index = 0;

  // setup data and 'extra' fields
  map<string, string> data_row;
  vector<string> fields_in_order;
  easy_streaming_dataset data_stream(dbTransaction, sql["registry"]);

  unsigned int next_column;
  next_column = 6;

  string prev_obs_set_id = "";
  deque<vector<string> >::iterator current_row;
  while (data_stream.GetRow(data_row))
  {
    data_stream.GetRow(fields_in_order);
    field_values.push_back(fields_in_order);

    data_stream.Next();
  }

  // Yeah...I don't see the need.  I mean, what could possibly go wrong?  ;-)
/* // make sure all rows are the same size.
   for (deque<vector<string> >::iterator j = field_values.begin(); j != field_values.end(); j++)
    j->resize(next_column);
*/

  ExecuteStatement(dbTransaction, "delete from temp_enrollments");
  
  if (dbTransaction->Active)
    dbTransaction->Commit();

  return true;
}
//---------------------------------------------------------------------------



//---------------------------------------------------------------------------
classFilterNewDiabeticSource::classFilterNewDiabeticSource() : classFilterEnrollmentSource()
{
}
//---------------------------------------------------------------------------
bool classFilterNewDiabeticSource::Begin(TIBTransaction * dbTransaction)
{
  Pdate pointless_object = CurrentPdate();
  pointless_object.mod(-1, in_years);
  string year_begin = pointless_object.OutDate(mdy | yyyy);

  if (!dbTransaction->Active)
    dbTransaction->StartTransaction();

  sql["visits"] = "select count(tasks.id) "
                  "from tasks "
                  "where tasks.person_id = p.id and tasks.current_id+1 >= 4 "
                  "and method_id = " + lookupTables->IdOf("method_types", "Visit") + " "
                  "and tasks.group_id in (" + lookupTables->IdOf("groups", "Diabetes") + "," +
                  lookupTables->IdOf("groups", "Little Axe Clinic") + "," + lookupTables->IdOf("groups", "Diabetes") + ") "
                  "and tasks.date_of_task between '" + year_begin + "' and current_date";

  // jhaley 20100818: must use alias 'p' throughout for Firebird 2.x compliance.
  sql["new_dm"] = "select distinct first_name || (case when middle_name != '' then ' ' else '' end) || middle_name || (case when last_name != '' then ' ' else '' end) || last_name || (case when suffix != '' then ' ' else '' end) || suffix name, "
                  "birth_date, (" + sql["visits"] + "), p.current_id, p.id "
                  "from people p "
                  "inner join person_names on p.person_names_id = person_names.id "
                  "inner join conditions c on c.person_id = p.id "
                  "where c.condition_id = " + lookupTables->IdOf("condition_types", "Condition") + " and c.current_id+1 >= 4 "
                  "and (c.code in ('250', '642.3') or c.code like '250.%' or c.code like '642.3_') " // type 1, type 2, or gestational
                  "and p.id not in "
                  "("
                    "select enrollment.person_id "
                    "from enrollment "
                    "where enrollment.enrollment_id = " + lookupTables->IdOf("enrollment_types", "On Diabetes Registry") + " "
                  ")";

  // setup default fields
  field_names.clear();
  field_types.clear();
  field_values.clear();
  row_index = 0;

  // setup data and 'extra' fields
  map<string, string> data_row;
  vector<string> fields_in_order;
  easy_streaming_dataset data_stream(dbTransaction, sql["new_dm"]);

  unsigned int next_column;
  next_column = 6;

  string prev_obs_set_id = "";
  deque<vector<string> >::iterator current_row;
  while (data_stream.GetRow(data_row))
  {
    data_stream.GetRow(fields_in_order);
    field_values.push_back(fields_in_order);

    data_stream.Next();
  }

  if (dbTransaction->Active)
    dbTransaction->Commit();

  // Yeah...I don't see the need.  I mean, what could possibly go wrong?  ;-)
/* // make sure all rows are the same size.
   for (deque<vector<string> >::iterator j = field_values.begin(); j != field_values.end(); j++)
    j->resize(next_column);
*/

  return true;
}
//---------------------------------------------------------------------------



//---------------------------------------------------------------------------
classFilterAtRiskSource::classFilterAtRiskSource() : classFilterEnrollmentSource()
{
}
//---------------------------------------------------------------------------
bool classFilterAtRiskSource::Begin(TIBTransaction * dbTransaction)
{
  // some of this is duplicated from up above cause ... fuck all
  Pdate pointless_object = CurrentPdate();
  pointless_object.mod(-1, in_years);
  string year_begin = pointless_object.OutDate(mdy | yyyy);

  if (!dbTransaction->Active)
    dbTransaction->StartTransaction();

  sql["visits"] = "select count(tasks.id) "
                  "from tasks "
                  "where tasks.person_id = p.id and tasks.current_id+1 >= 4 "
                  "and method_id = " + lookupTables->IdOf("method_types", "Visit") + " "
                  "and tasks.group_id in (" + lookupTables->IdOf("groups", "Diabetes") + "," +
                  lookupTables->IdOf("groups", "Little Axe Clinic") + "," + lookupTables->IdOf("groups", "Diabetes") + ") "
                  "and tasks.date_of_task between '" + year_begin + "' and current_date";

  // count of diabetes follow ups in the past year
  sql["fu"] = "select count(tasks.id) "
              "from tasks "
              "inner join diab_activity_types on tasks.diab_activity_id = diab_activity_types.id "
              "where tasks.person_id = p.id and tasks.current_id+1 >= 4 "
              "and diab_activity_types.item like '%FOLLOW UP%' "
              "and tasks.group_id in (" + lookupTables->IdOf("groups", "Diabetes") + ") "
              "and tasks.date_of_task between '" + year_begin + "' and current_date ";

  // jhaley 20100818: must use alias 'p' throughout for Firebird 2.x compliance
  sql["at_risk"] = "select first_name || (case when middle_name != '' then ' ' else '' end) || middle_name || (case when last_name != '' then ' ' else '' end) || last_name || (case when suffix != '' then ' ' else '' end) || suffix name, "
                   "birth_date, (" + sql["visits"] + "), p.current_id, p.id "
                   "from people p "
                   "inner join person_names on p.person_names_id = person_names.id "
                   "inner join obs_response_sets ors on ors.person_id = p.id "
                   "inner join obs_responses on obs_responses.obs_response_set_id = ors.id "
                   "inner join obs_questions oq on obs_responses.obs_question_id = oq.id "
                   "where not exists "
                   "(select * from conditions c "
                   "where c.person_id = p.id and c.condition_id = " + lookupTables->IdOf("condition_types", "Condition") + " "
                   "and c.current_id+1 >= 4 "
                   "and (c.code in ('250', '642.3') or c.code like '250.%' or c.code like '642.3_')) " // type 1, type 2, or gestational
                   "and p.id not in "
                   "("
                     "select enrollment.person_id "
                     "from enrollment "
                     "where enrollment.enrollment_id = " + lookupTables->IdOf("enrollment_types", "On Diabetes Registry") + " and enrollment.current_id+1 >= 4 "
                   ") and ors.date_of_observation > '" + year_begin + "' and "
                   "("
                     "(oq.short_item = 'TRIG' and obs_responses.response_metric > 150) or "
                     "(oq.short_item = 'LDL_C' and obs_responses.response_metric > 100) or "
                     "(oq.short_item = 'HDL_C' and obs_responses.response_metric > 35) or "
                     "(oq.short_item = 'CHOL' and obs_responses.response_metric > 200) or "
                     "(oq.short_item = 'Sys BP' and obs_responses.response_metric > 130) or "
                     "(oq.short_item = 'Dias BP' and obs_responses.response_metric > 80) or "
                     "(oq.short_item = 'HA1C' and obs_responses.response_metric >= 8) or "
                     "(oq.short_item = 'BMI' and obs_responses.response_metric >= 30) "
                   ")"
                   "group by 1, 2, 3, 4, 5";

  // setup default fields
  field_names.clear();
  field_types.clear();
  field_values.clear();
  row_index = 0;

  // setup data and 'extra' fields
  map<string, string> data_row;
  vector<string> fields_in_order;
  easy_streaming_dataset data_stream(dbTransaction, sql["at_risk"]);

  unsigned int next_column;
  next_column = 6;

  string prev_obs_set_id = "";
  deque<vector<string> >::iterator current_row;
  while (data_stream.GetRow(data_row))
  {
    data_stream.GetRow(fields_in_order);
    field_values.push_back(fields_in_order);

    data_stream.Next();
  }

  // Yeah...I don't see the need.  I mean, what could possibly go wrong?  ;-)
/* // make sure all rows are the same size.
   for (deque<vector<string> >::iterator j = field_values.begin(); j != field_values.end(); j++)
    j->resize(next_column);
*/

  if (dbTransaction->Active)
    dbTransaction->Commit();

  return true;
}
//---------------------------------------------------------------------------

