//----------------------------------------------------------------------------------------------------------------------

#include <vcl.h>
#pragma hdrstop

#include "codeFrameAssessmentActivityLevel.h"
#include "codeRecordPatientAssessment.h"
//----------------------------------------------------------------------------------------------------------------------
#pragma package(smart_init)
#pragma link "codeFrameAssessmentGeneric"
#pragma resource "*.dfm"
TframeAssessmentActivityLevel *frameAssessmentActivityLevel;
//----------------------------------------------------------------------------------------------------------------------
__fastcall TframeAssessmentActivityLevel::TframeAssessmentActivityLevel(TComponent* Owner)
        : TframeAssessmentGeneric(Owner)
{
}
//----------------------------------------------------------------------------------------------------------------------
// Initialize - Overridden to set parent properties and setup activity levels filtered listview.
bool TframeAssessmentActivityLevel::Initialize()
{
  // Set AssessmentGeneric properties
  SetBandCaption("Activity Level");
  SetPermission("activity_levels");

  // Setup notes filter
  levels_filter.SetListViewHandle(listLevels);
  levels_filter.SetSecurity("activity_levels");

  levels_filter.AddColumnSpec("Item",        "activity_levels", as_text, true,  160, rawString);
  levels_filter.AddColumnSpec("Details",     "activity_levels", as_text, true,  420, rawString);
  levels_filter.AddColumnSpec("Current Id",  "current_id",      as_int,  false,  80);
  levels_filter.AddColumnSpec("Internal Id", "internal_id",     as_int,  false,  80);

  levels_filter.FetchPreferences("assessment.levels.topfilter");
  levels_filter.SetSQLStatement("");
  levels_filter.RefreshDataSet();

  // setup activity_level_items input filter

  string items_sql =
    "select item, id from activity_item_types where valid = 1 and id > 0 and "
    "id <> " + lookupTables->IdOf("activity_item_types", "No Restrictions"); // don't include the NPI indicator

  // Checks Filter
  checks_filter.SetListViewHandle(listActivityTypes);
  checks_filter.SetSecurity("");
  checks_filter.AddColumnSpec("Item",        "activity_levels", as_text, true,  168, rawString);
  checks_filter.AddColumnSpec("Internal Id", "internal_id",     as_int,  false,  80);
  checks_filter.FetchPreferences("assessment.levels.activity_item_types");
  checks_filter.SetIconStatus(false);
  checks_filter.SetCheck(true);
  checks_filter.SetSQLStatement(items_sql);
  checks_filter.RefreshDataSet();

  // Set edit box change handlers
  SetAllEventHandlers(panelLevels);

  return TframeAssessmentGeneric::Initialize();
}
//----------------------------------------------------------------------------------------------------------------------
// Lock - Overridden to lock activity levels panel
void TframeAssessmentActivityLevel::Lock()
{
  TframeAssessmentGeneric::Lock(); // lock parent controls
  panelLevels->Enabled = false;    // lock all panel controls
}
//----------------------------------------------------------------------------------------------------------------------
// PutUpActivityLevels - Refreshes the controls to reflect the state of the activity levels list
void TframeAssessmentActivityLevel::PutUpActivityLevels(pvecmap &level_items)
{
  psetstr activity_item_ids;

  // Clear out the checkboxes and edit box
  checks_filter.UncheckAll();
  memoComments->Text = "";
  checkNPI->Checked = false;

  // Analyze the retrieved activity_levels items
  for(size_t i = 0; i < level_items.size(); i++)
  {
    // Skip items which are anything other than Current. This allows Deleted items to be updated back to Current status
    // when saving.
    if(level_items[i]["current_id"] != lookupTables->IdOf("current_types", "Current"))
      continue;

    if(level_items[i]["activity_item_id"] == lookupTables->IdOf("activity_item_types", "No Restrictions")) // NPI
      checkNPI->Checked = true;
    else
    {
      activity_item_ids.insert(level_items[i]["activity_item_id"]);

      if(level_items[i]["activity_item_id"] == lookupTables->IdOf("activity_item_types", "Other"))
        memoComments->Text = level_items[i]["comments"].c_str();
    }
  }

  // Set checkboxes
  checks_filter.SetCheckedWhereID(activity_item_ids, true);
}
//----------------------------------------------------------------------------------------------------------------------
// TaskIdChanging - Overridden to populate activity levels filter
void TframeAssessmentActivityLevel::TaskIdChanging(const string &new_id)
{
  string sql;

  TframeAssessmentGeneric::TaskIdChanging(new_id);

  sql = "select AIT.item, AL.comments, AL.current_id, AL.id "
        "from activity_levels AL "
        "left join activity_item_types AIT on AL.activity_item_id = AIT.id "
        "where AL.current_id >= 2 and AL.task_id = '" + new_id + "' ";

  levels_filter.SetSQLStatement(sql);
  levels_filter.RefreshDataSet();
  levels_filter.SetAddEdit(this, AddRecord (&TframeAssessmentActivityLevel::FilterAddActivityLevel),
                                 EditRecord(&TframeAssessmentActivityLevel::FilterEditActivityLevel));
  levels_request = parent_form->RequestNotificationWhen("activity_levels.task_id", new_id, rel_e, as_int);
}
//----------------------------------------------------------------------------------------------------------------------
// UpdateForEdit - Overridden to put fresh data into the editing_level_items vecmap and on screen.
void TframeAssessmentActivityLevel::UpdateForEdit()
{
  string sql =
    "select * "
    "from activity_levels AL "
    "where AL.task_id = '" + task_id + "' ";

  editing_level_items.clear();

  if(!SqlToVecMap(generic_transaction, sql, editing_level_items))
    parent_form->PSay("Could not retrieve up-to-date information for activity_levels items", m_warning, MB_OK);

  // Refresh controls
  PutUpActivityLevels(editing_level_items);

  TframeAssessmentGeneric::UpdateForEdit();
}
//----------------------------------------------------------------------------------------------------------------------
// LockRecords - Overridden to lock the activity_levels records
bool TframeAssessmentActivityLevel::LockRecords()
{
  // Must acquire all items. Any failure is a failure to unlock for editing, GTFO and rollback.
  if(!LockVecMapRecords(editing_level_items, "activity_levels"))
    return false;

  return TframeAssessmentGeneric::LockRecords();
}
//----------------------------------------------------------------------------------------------------------------------
// IsValidOnScreen - Overridden to do validation for activity_levels
bool TframeAssessmentActivityLevel::IsValidOnScreen()
{
  pvecstr checked_ids = checks_filter.GetMultiID();
  bool other_is_checked = false;
  
  // Make sure at least one activity level is specified
  if(!checkNPI->Checked &&
     editing_level_items.size() == 0 && // Only when the record is new, otherwise can't delete last one
     checked_ids.size() == 0)
  {
    parent_form->PSay("You have not specified any saveable data. Please check one or more activity levels or the "
                      "'No Restrictions' indicator.", m_missing_data, MB_OK);
    return false;
  }

  // If NPI is checked, nothing except Other can be specified in addition to it.
  if(checkNPI->Checked && checked_ids.size() > 0)
  {
    for(size_t i = 0; i < checked_ids.size(); i++)
    {
      if(checked_ids[i] != lookupTables->IdOf("activity_item_types", "Other"))
      {
        parent_form->PSay("You indicated that there were 'No Restrictions', but also checked additional problems "
                          "(not including 'Other', which you can use to provide relevant information even if there "
                          "were no tangible problems).\n\n"
                          "Please uncheck either the 'No Restrictions' indicator, or all items in the Activity "
                          "Levels list except 'Other'.", m_invalid_data, MB_OK);
        return false;
      }
    }
  }

  for(size_t i = 0; i < checked_ids.size(); i++)
  {
    // Still have to hard code it >_>
    if(checked_ids[i] == lookupTables->IdOf("activity_item_types", "Other"))
    {
      other_is_checked = true;
      if(memoComments->Text == "")
      {
        parent_form->PSay("You selected 'Other' but did not provide any additional information on the activity level.",
                          m_missing_data, MB_OK);
        return false;
      }
    }
  }

  if(!other_is_checked && memoComments->Text != "")
  {
    parent_form->PSay("You did not select 'Other', but specified information in the 'Other Information' field. This "
                      "information would be lost, so please either clear it out now, or select the 'Other' item in "
                      "the checkbox list as well.", m_invalid_data, MB_OK);
    return false;
  }

  return TframeAssessmentGeneric::IsValidOnScreen();
}
//----------------------------------------------------------------------------------------------------------------------
bool TframeAssessmentActivityLevel::UpdateActivityLevel(int record_index, string comment)
{
  pmapstrs ending_state;
  pmapstrtoint leave_alone;

  ending_state["id"]                = editing_level_items[record_index]["id"];
  ending_state["current_id"]        = lookupTables->IdOf("current_types", "Current");
  ending_state["last_updated_by"]   = current_userid;
  ending_state["last_update_stamp"] = RightNow();
  ending_state["task_id"]           = task_id;
  ending_state["activity_item_id"]  = editing_level_items[record_index]["activity_item_id"];
  ending_state["comments"]          = comment;

  leave_alone["comments"] = sql_no_uppercasing;

  final_ending_state.push_back(ending_state);

  return ExecuteUpdateStatement(generic_transaction, "activity_levels", ending_state, leave_alone);
}
//----------------------------------------------------------------------------------------------------------------------
bool TframeAssessmentActivityLevel::InsertActivityLevel(string activity_item_id, string comment)
{
  pmapstrs ending_state;
  pmapstrtoint leave_alone;

  ending_state["id"]                = GetNextId(generic_transaction, "activity_levels");
  ending_state["current_id"]        = lookupTables->IdOf("current_types", "Current");
  ending_state["last_updated_by"]   = current_userid;
  ending_state["last_update_stamp"] = RightNow();
  ending_state["created_by"]        = current_userid;
  ending_state["creation_stamp"]    = ending_state["last_update_stamp"];
  ending_state["task_id"]           = task_id;
  ending_state["activity_item_id"] = activity_item_id;
  ending_state["comments"]          = comment;

  leave_alone["comments"] = sql_no_uppercasing;

  final_ending_state.push_back(ending_state);

  return ExecuteInsertStatement(generic_transaction, "activity_levels", ending_state, leave_alone);
}
//----------------------------------------------------------------------------------------------------------------------
bool TframeAssessmentActivityLevel::DeleteActivityLevel(int record_index)
{
  pmapstrs ending_state;
  pmapstrtoint leave_alone;

  ending_state["id"]                = editing_level_items[record_index]["id"];
  ending_state["current_id"]        = lookupTables->IdOf("current_types", "Deleted"); // Mark as Deleted
  ending_state["last_updated_by"]   = current_userid;
  ending_state["last_update_stamp"] = RightNow();

  final_ending_state.push_back(ending_state);

  return ExecuteUpdateStatement(generic_transaction, "activity_levels", ending_state, leave_alone);
}
//----------------------------------------------------------------------------------------------------------------------
// DoSaveData - Overridden to provide core saving logic.
bool TframeAssessmentActivityLevel::DoSaveData()
{
  psetstr checked_ids;
  psetstr unchecked_ids;
  int record_idx;
  string npiID = lookupTables->IdOf("activity_item_types", "No Restrictions");

  final_ending_state.clear();

  // First determine what is checked and what is not in the filters.
  GetIDsFromCheckBoxFilter(checks_filter, checked_ids, unchecked_ids); // use parent class utility method

  // Add in the NPI
  if(checkNPI->Checked)
    checked_ids.insert(npiID);
  else
    unchecked_ids.insert(npiID);

  // Look at all IDs:

  // * If the ID is checked, and the ID is in the editing map, do an update on that record.
  // * If the ID is checked, and the ID is NOT in the editing map, do an insert.
  for(psetstr::iterator i_checked = checked_ids.begin(); i_checked != checked_ids.end(); ++i_checked)
  {
    string comments;

    if(*i_checked == lookupTables->IdOf("activity_item_types", "Other"))
      comments = memoComments->Text.c_str();

    record_idx = IsInVecMap(editing_level_items, "activity_item_id", *i_checked);
    if(record_idx >= 0)
    {
      // Do Update because this activity_levels item already exists.
      if(!UpdateActivityLevel(record_idx, comments))
        return false;
    }
    else
    {
      // Do Insert because this activity_levels item doesn't exist.
      if(!InsertActivityLevel(*i_checked, comments))
        return false;
    }
  }

  // * If the ID is NOT checked, and the id is in the editing map, mark that item as Deleted via the current_id.
  // * If the ID is NOT checked, and the id is NOT in the editing map, do nothing at all.
  for(psetstr::iterator i_unchecked = unchecked_ids.begin(); i_unchecked != unchecked_ids.end(); ++i_unchecked)
  {
    record_idx = IsInVecMap(editing_level_items, "activity_item_id", *i_unchecked);
    if(record_idx >= 0)
    {
      // Do Update and set the existing activity_levels record to 'Deleted'.
      if(!DeleteActivityLevel(record_idx))
        return false;
    }
    // Otherwise, we neither had a record to begin with, nor do we have one to insert now, so do nothing.
  }

  // If we get this far, nothing borked up above.
  return true;
}
//----------------------------------------------------------------------------------------------------------------------
// SetViewMode - Overridden to activate viewing of activity_levels information
void TframeAssessmentActivityLevel::SetViewMode()
{
  TframeAssessmentGeneric::SetViewMode();

  panelLevels->Enabled = false;                    // It is still not possible to enter data.
  editing_level_items.clear();                    // Clear out the items vecmap for safety
  checks_filter.UncheckAll();
  memoComments->Text = "";
  checkNPI->Checked = false;

  ClearControlsModified(panelLevels);
}
//----------------------------------------------------------------------------------------------------------------------
// SetEditMode - Overridden to activate editing of activity_levels records
void TframeAssessmentActivityLevel::SetEditMode()
{
  TframeAssessmentGeneric::SetEditMode();

  panelLevels->Enabled = true;               // Enable panel controls

  ClearControlsModified(panelLevels);
}
//----------------------------------------------------------------------------------------------------------------------
// FilterAddActivityLevel - "Add" handler for the activity_levels filter. This actually just activates editing mode here.
void TframeAssessmentActivityLevel::FilterAddActivityLevel()
{
  if(btnUnlockSave->Enabled && frame_mode == mode_viewing)
    btnUnlockSaveClick(NULL);
}
//----------------------------------------------------------------------------------------------------------------------
// FilterEditActivityLevel - Same as above.
void TframeAssessmentActivityLevel::FilterEditActivityLevel()
{
  if(btnUnlockSave->Enabled && frame_mode == mode_viewing)
    btnUnlockSaveClick(NULL);
}
//----------------------------------------------------------------------------------------------------------------------
// AtlasNotify - Overridden to refresh activity_levels listview
bool TframeAssessmentActivityLevel::AtlasNotify(int request_num, string matched_value)
{
  bool handled = false;

  if(request_num == levels_request)
  {
    levels_filter.RefreshDataSet();
    handled = true;
  }

  return handled;
}
//----------------------------------------------------------------------------------------------------------------------
// SendAtlas - Overridden to send out notifications for activity_levels
void TframeAssessmentActivityLevel::SendAtlas()
{
  atlas->Notify("activity_levels.task_id", task_id); // Is this good enough?
}
//----------------------------------------------------------------------------------------------------------------------
// WriteReport - Write out a report on the activity levels
bool TframeAssessmentActivityLevel::WriteReport(TComponent *compositeReportCtrl)
{
  bool success = false;

  try
  {
    string sql =
      "select AIT.item, AL.comments "
      "from activity_levels AL "
      "left join activity_item_types AIT on AL.activity_item_id = AIT.id "
      "where AL.current_id >= 3 and AL.task_id = '" + task_id + "' ";

    InitDataSet(sql);

    TformReportAssessment *reportForm = report_data.reportForm;
    TQuickRep *thisReport = reportForm->reportALevel;

    thisReport->DataSet = report_data.dbDataSet;
    SetupQRDBText(reportForm->txtALevelItem,   "item");
    SetupQRDBText(reportForm->txtALevelDetail, "comments");

    // NB: Need to pass it as TComponent to avoid problems with having QuickReport included in the headers
    TQRCompositeReport *compositeReport = dynamic_cast<TQRCompositeReport *>(compositeReportCtrl);
    if(compositeReport)
    {
      compositeReport->Reports->Add(thisReport);
      success = true;
    }
  }
  catch(...)
  {
  }

  return success;
}
//----------------------------------------------------------------------------------------------------------------------
void __fastcall TframeAssessmentActivityLevel::listActivityTypesClick(TObject *Sender)
{
  // Use parent class hit test logic
  FilterCheckBoxHitTest(listActivityTypes);
}
//----------------------------------------------------------------------------------------------------------------------
void __fastcall TframeAssessmentActivityLevel::checkNPIClick(TObject *Sender)
{
  if(frame_mode == mode_editing)
    SetModified(true);
}
//----------------------------------------------------------------------------------------------------------------------
void __fastcall TframeAssessmentActivityLevel::btnReferralClick(TObject *Sender)
{
  parent_form->ChangeToArea(TformRecordPatientAssessment::area_referrals);
}
//----------------------------------------------------------------------------------------------------------------------
