//----------------------------------------------------------------------------------------------------------------------

#include <vcl.h>
#pragma hdrstop

#include "codeFrameAssessmentAllergies.h"
#include "codeRecordPatientAssessment.h"
#include "codeRecordCondition.h"
#include "codeSearchCode.h"
//----------------------------------------------------------------------------------------------------------------------
#pragma package(smart_init)
#pragma link "codeFrameAssessmentGeneric"
#pragma link "codeHalfButton"
#pragma resource "*.dfm"
TframeAssessmentAllergies *frameAssessmentAllergies;
//----------------------------------------------------------------------------------------------------------------------
__fastcall TframeAssessmentAllergies::TframeAssessmentAllergies(TComponent* Owner)
        : TframeAssessmentGeneric(Owner), allergy_filter(), editing_condition(), ending_state(), allergy_request(-1),
          editing_id()
{
}
//----------------------------------------------------------------------------------------------------------------------
// Initialize - Overridden to set parent properties and setup conditions filtered listview.
bool TframeAssessmentAllergies::Initialize()
{
  // Set AssessmentGeneric properties
  SetBandCaption("Allergies");
  SetPermission("conditions");

  // Setup notes filter
  allergy_filter.SetListViewHandle(listAllergies);
  allergy_filter.SetSecurity("conditions");
  allergy_filter.AddColumnSpec("Condition Type", "conditions",  as_text, true,  120, prettyString);
  allergy_filter.AddColumnSpec("Allergy Type",   "conditions",  as_text, true,  120);
  allergy_filter.AddColumnSpec("Start Date",     "conditions",  as_date, true,   90);
  allergy_filter.AddColumnSpec("Code",           "conditions",  as_text, true,   90);
  allergy_filter.AddColumnSpec("Note",           "conditions",  as_text, true,  170, sentenceString);
  allergy_filter.AddColumnSpec("Current Id",     "current_id",  as_int,  false,  80);
  allergy_filter.AddColumnSpec("Internal Id",    "internal_id", as_int,  false,  80);
  allergy_filter.FetchPreferences("assessment.allergies.filter");
  allergy_filter.SetSQLStatement("");
  allergy_filter.RefreshDataSet();

  // Register condition types lookup with custom SQL - only Allergy and Drug Allergy conditions are available here
  string lookupName = "allergies.cboConditionType";
  string tableName  = "condition_types";
  string sql        = "select id, item, valid from condition_types where id in ("
                      + lookupTables->IdOf("condition_types", "Allergy") + ", "
                      + lookupTables->IdOf("condition_types", "Drug Allergy") + ") ";

  RegisterLookup(lookupName, tableName, cboConditionType, sql);

  // Register allergy types lookup
  lookupName = "allergies.cboAllergyType";
  tableName  = "allergy_types";
  RegisterLookup(lookupName, tableName, cboAllergyType);

  // Set standard OnChange handlers for all controls on panelAllergy
  SetAllEventHandlers(panelAllergy);

  return TframeAssessmentGeneric::Initialize();
}
//----------------------------------------------------------------------------------------------------------------------
// Lock - Overridden to lock Add button and allergy panel
void TframeAssessmentAllergies::Lock()
{
  TframeAssessmentGeneric::Lock(); // lock parent controls
  btnAdd->Enabled = false;         // disable add button
  panelAllergy->Enabled = false;   // lock all controls
}
//----------------------------------------------------------------------------------------------------------------------
// TaskIdChanging - Overridden to populate allergies filter
void TframeAssessmentAllergies::TaskIdChanging(const string &new_id)
{
  string sql;

  TframeAssessmentGeneric::TaskIdChanging(new_id);

  // All historic allergy condition data for the task's person is displayed here.
  // There is NOT any discrimination against group IDs because we do not hide vital information from people who are
  // supposed to have access to it. A per-department view of allergy conditions is not a real view of anything at all.
  // It's a liability waiting to happen.
  sql = "select condition_types.item, allergy_types.item, conditions.start_date, conditions.code, conditions.note, "
        "conditions.current_id, conditions.id "
        "from conditions "
        "left join condition_types on conditions.condition_id = condition_types.id "
        "left join allergy_types on conditions.allergy_id = allergy_types.id "
        "where conditions.current_id >= 2 and conditions.person_id = '" + person_id + "' and "
        "conditions.condition_id in ("
        + lookupTables->IdOf("condition_types", "Allergy") + ", "           // Allergy condition types only
        + lookupTables->IdOf("condition_types", "Drug Allergy") + ") ";

  allergy_filter.SetSQLStatement(sql);
  allergy_filter.RefreshDataSet();
  allergy_filter.SetAddEdit(this, AddRecord (&TframeAssessmentAllergies::FilterAddAllergy),
                                  EditRecord(&TframeAssessmentAllergies::FilterEditAllergy));
  allergy_request = parent_form->RequestNotificationWhen("conditions.person_id", person_id, rel_any, as_int);
}
//----------------------------------------------------------------------------------------------------------------------
// LockRecords - Overridden to lock conditions record for editing.
bool TframeAssessmentAllergies::LockRecords()
{
  if(!LockRecordForUpdate(generic_transaction, "conditions", editing_id))
    return false;

  return TframeAssessmentGeneric::LockRecords();
}
//----------------------------------------------------------------------------------------------------------------------
// CanUnlock - Overridden to test for a selected condition
bool TframeAssessmentAllergies::CanUnlock(bool &creating_new)
{
  editing_id = allergy_filter.GetSelectedRecordID();

  if(!creating_new && editing_id == "")
  {
    parent_form->PSay("Please select an allergy condition to edit in the list first.", m_reminder, MB_OK);
    return false;
  }

  return TframeAssessmentGeneric::CanUnlock(creating_new);
}
//----------------------------------------------------------------------------------------------------------------------
// UpdateForEdit - Overridden to put fresh data into the editing_condition map and on screen.
void TframeAssessmentAllergies::UpdateForEdit()
{
  string sql = "select * from conditions where id = '" + editing_id + "' ";

  if(!SqlToMap(generic_transaction, sql, editing_condition))
    parent_form->PSay("Could not retrieve up-to-date information for allergy condition", m_warning, MB_OK);

  string condition_type  = lookupTables->ValueOf("condition_types", editing_condition["condition_id"]);
  cboConditionType->Text = PrettyString(condition_type).c_str();
  cboAllergyType->Text   = lookupTables->ValueOf("allergy_types", editing_condition["allergy_id"]).c_str();
  editStartDate->Text    = editing_condition["start_date"].c_str();
  editCode->Text         = editing_condition["code"].c_str();
  editNotes->Text        = SentenceString(editing_condition["note"]).c_str();

  ClearControlsModified(panelAllergy);

  TframeAssessmentGeneric::UpdateForEdit();
}
//----------------------------------------------------------------------------------------------------------------------
// SavingNew - Overridden to return status of editing_condition as new record or unlocked record
bool TframeAssessmentAllergies::SavingNew()
{
  return editing_condition["id"] == "new";
}
//----------------------------------------------------------------------------------------------------------------------
// IsValidOnScreen - Overridden to validate lookup and date values
bool TframeAssessmentAllergies::IsValidOnScreen()
{
  if(!lookupTables->IsInLookUpTable("condition_types", cboConditionType))
  {
    parent_form->PSay("Please enter a valid condition type", m_invalid_data, MB_OK);
    return false;
  }

  if(!lookupTables->IsInLookUpTable("allergy_types", cboAllergyType) || cboAllergyType->Text == "")
  {
    parent_form->PSay("Please enter a valid allergy type", m_invalid_data, MB_OK);
    return false;
  }

  // An "Other" allergy type must have more specific information associated with it via the notes.
  // Any allergy that gets "coded" via the notes frequently enough will be manually added as an allergy type.
  string allergy_text = cboAllergyType->Text.c_str();
  if(allergy_text.find("Other") != string::npos && editNotes->Text == "")
  {
    parent_form->PSay("You have specified an 'Other' type of allergy but have provided no information in the "
                      "condition's notes to indicate what type of allergy this condition represents.\n\n"
                      "Please enter notes information or choose a more specific type of allergy.\n\n"
                      "If the allergy type you want is not available, please contact a programmer to have it added.",
                      m_missing_data, MB_OK);
    return false;
  }

  if(editStartDate->Text != "" && !Pdate(editStartDate->Text.c_str(), mdy | yy).IsValidDate())
  {
    parent_form->PSay("Please enter a valid start date", m_invalid_data, MB_OK);
    return false;
  }

  return TframeAssessmentGeneric::IsValidOnScreen();
}
//----------------------------------------------------------------------------------------------------------------------
// DoSaveData - Overridden to provide core saving logic.
bool TframeAssessmentAllergies::DoSaveData()
{
  map<string, int> leave_alone;
  ending_state = editing_condition;

  ending_state["current_id"]        = lookupTables->IdOf("current_types", "Current");
  ending_state["last_update_stamp"] = RightNow();
  ending_state["last_updated_by"]   = current_userid;
  ending_state["condition_id"]      = lookupTables->IdOf("condition_types", cboConditionType);
  ending_state["start_date"]        = editStartDate->Text != "" ?
                                        Pdate(editStartDate->Text.c_str(), mdy | yy).OutDate(mdy | yyyy).c_str() : "";
  ending_state["allergy_id"]        = lookupTables->IdOf("allergy_types", cboAllergyType);

  // This allergy will be associated with the most recent assessment of which it was considered a component.
  // This doesn't mean it didn't appear on earlier assessments, of course, but we're not terribly interested in that
  // fact. Allergy conditions carry over between assessments because they are a quality of the person and not the task.
  ending_state["patient_assessment_id"] = parent_form->GetAssessmentID();

  // Set the code. If the code is valid and the notes are blank, the code's description text may be adapted as the note.
  ending_state["code"] = editCode->Text.c_str();
  if(editCode->Text != "" && editNotes->Text == "")
  {
    string sql =
      string("select description from codes where code = '") + editCode->Text.c_str()
      + "' and code_id = " + lookupTables->IdOf("code_types", "ICD-9");
    ending_state["note"] = GetOneField(dbDatabase, sql);
  }
  else
    ending_state["note"] = editNotes->Text.c_str();

  leave_alone["note"] = sql_no_uppercasing; // Don't uppercase note text

  // Blank task_id is 0; otherwise, preserve any task attachment that already existed for the condition.
  // New conditions created through the patient assessment form are not associated with a task because that would
  // cause them to become GPRA reportable and billable, which is incorrect.
  if(ending_state["task_id"] == "")
    ending_state["task_id"] = "0";

  // Not seen:
  // person_id    - this either comes from the existing condition or is set in editing_condition on add
  // group_id     - this either comes from the existing condition or is set in editing_condition on add
  // stop_date    - this either comes from the existing condition or remains null
  // index_number - this either comes from the existing condition or remains null

  return DoInsertOrUpdate(ending_state, leave_alone, "conditions");
}
//----------------------------------------------------------------------------------------------------------------------
// SetViewMode - Overridden to activate viewing of task notes
void TframeAssessmentAllergies::SetViewMode()
{
  TframeAssessmentGeneric::SetViewMode();
  btnAdd->Enabled = HaveUnlockPermission(true); // It is now possible to add a note
  panelAllergy->Enabled = false;                // But it is still not possible to enter data.
  editing_condition.clear();                    // Clear out the note map for safety

  // Reset control contents
  ClearControlsText(panelAllergy);
  ClearControlsModified(panelAllergy);
}
//----------------------------------------------------------------------------------------------------------------------
// SetEditMode - Overridden to activate editing of a task note
void TframeAssessmentAllergies::SetEditMode()
{
  TframeAssessmentGeneric::SetEditMode();
  btnAdd->Enabled = false;                // Can't add notes right now
  panelAllergy->Enabled = true;           // Enable panel controls
}
//----------------------------------------------------------------------------------------------------------------------
// btnAddClick - Handler for toolbar button to add a new task note
void __fastcall TframeAssessmentAllergies::btnAddClick(TObject *Sender)
{
  if(DoUnlock(true))
  {
    // Reset control contents
    ClearControlsText(panelAllergy);
    ClearControlsModified(panelAllergy);

    editing_condition.clear();
    editing_condition["id"] = "new"; // Creating a new condition
    editing_condition["person_id"] = person_id; // For a new condition, person and group are propagated from the
    editing_condition["group_id"]  = group_id;  // parent assessment form's tasks.person_id and tasks.group_id
  }
}
//----------------------------------------------------------------------------------------------------------------------
// FilterAddCondition - "Add" handler for the conditions filter. We redirect this to the "Add a Note" code so that it
// works compatibly with the assessment frame.
void TframeAssessmentAllergies::FilterAddAllergy()
{
  if(btnAdd->Enabled) // Only when you could add using the button.
    btnAddClick(NULL);
}
//----------------------------------------------------------------------------------------------------------------------
// FilterEditCondition - Open the note in a task_notes record form for ordinary editing capability. Not encouraged, but
// allows adjustment of current_id's etc, which is important if a data entry mistake is made.
void TframeAssessmentAllergies::FilterEditAllergy()
{
  // Open the task note record
  map<string, string> field_map;
  field_map["id"] = allergy_filter.GetSelectedRecordID();

  FORM_TRY
    TformRecordCondition *condition_form = new TformRecordCondition(NULL, field_map);
    condition_form->Show();
  FORM_CATCH
}
//----------------------------------------------------------------------------------------------------------------------
// AtlasNotify - Overridden to refresh allergy listview
bool TframeAssessmentAllergies::AtlasNotify(int request_num, string matched_value)
{
  bool handled = false;

  if(request_num == allergy_request)
  {
    allergy_filter.RefreshDataSet();
    handled = true;
  }

  return handled;
}
//----------------------------------------------------------------------------------------------------------------------
// SendAtlas - Overridden to send out notifications for conditions
void TframeAssessmentAllergies::SendAtlas()
{
  bool new_record = (editing_condition["id"] == "new");

  SendAtlasIfChanged(new_record, "conditions", editing_condition, ending_state);
}
//----------------------------------------------------------------------------------------------------------------------
// WriteReport - Write out a report on the allergy conditions
bool TframeAssessmentAllergies::WriteReport(TComponent *compositeReportCtrl)
{
  bool success = false;

  try
  {
    string allergy_sql =
      "select condition_types.item condition_type, allergy_types.item allergy_type, conditions.start_date, "
      " conditions.note "
      "from conditions "
      "left join condition_types on conditions.condition_id = condition_types.id "
      "left join allergy_types on conditions.allergy_id = allergy_types.id "
      "where conditions.current_id >= 3 and conditions.person_id = '" + person_id + "' and "
      "conditions.condition_id in ("
      + lookupTables->IdOf("condition_types", "Allergy") + ", "           // Allergy condition types only
      + lookupTables->IdOf("condition_types", "Drug Allergy") + ") ";

    InitDataSet(allergy_sql);

    TformReportAssessment *reportForm = report_data.reportForm;
    TQuickRep *thisReport = report_data.reportForm->reportAllergies;

    thisReport->DataSet = report_data.dbDataSet;
    SetupQRDBText(reportForm->txtAllergyCondType, "condition_type");
    SetupQRDBText(reportForm->txtAllergyType,     "allergy_type");
    SetupQRDBText(reportForm->txtAllergyDate,     "start_date");
    SetupQRDBText(reportForm->txtAllergyComments, "note");

    // NB: Need to pass it as TComponent to avoid problems with having QuickReport included in the headers
    TQRCompositeReport *compositeReport = dynamic_cast<TQRCompositeReport *>(compositeReportCtrl);
    if(compositeReport)
    {
      compositeReport->Reports->Add(thisReport);
      success = true;
    }
  }
  catch(...)
  {
  }

  return success;
}
//----------------------------------------------------------------------------------------------------------------------
void __fastcall TframeAssessmentAllergies::btnCodeSearchClick(TObject *Sender)
{
  vector<string> return_vector;
  string sql;
  TformSearchCode *codeSearch = NULL;
  FORM_TRY
    codeSearch = new TformSearchCode(NULL, &return_vector);
  FORM_CATCH

  codeSearch->cboCode->Text = "ICD-9";
  codeSearch->ShowTaskModal(parent_form);

  if(!return_vector.empty())
  {
    sql = "select code from codes where id = '" + return_vector[0] + "' ";
    editCode->Text = GetOneField(dbDatabase, sql).c_str();
    if(editCode->Text != "")
    {
      string new_note;
      string entered_note = UppercaseString(editNotes->Text.c_str());
      string sql =
        string("select description from codes where code = '")
        + editCode->Text.c_str()
        + "' and code_id = " + lookupTables->IdOf("code_types", "ICD-9");
      string code_note = GetOneField(dbDatabase, sql);
      if(entered_note == "")
        new_note = code_note;
      else
      {
        if(entered_note.find(code_note) == string::npos)
          new_note = entered_note + "\n\n" + code_note;
        else
          new_note = entered_note;
      }
      editNotes->Text = SentenceString(new_note).c_str();
      SetModified(true);
    }
  }
}
//----------------------------------------------------------------------------------------------------------------------
void __fastcall TframeAssessmentAllergies::btnExpandCodeClick(TObject *Sender)
{
  TformSearchCode *codeSearch = NULL;
  FORM_TRY
    codeSearch = new TformSearchCode(NULL, NULL);
  FORM_CATCH

  codeSearch->cboCode->Text = "ICD-9";
  codeSearch->editCode->Text = editCode->Text;
  codeSearch->Show();
  if(editCode->Text != "")
    codeSearch->buttonSearch->Click();
}
//----------------------------------------------------------------------------------------------------------------------

