//----------------------------------------------------------------------------------------------------------------------

#include <vcl.h>
#pragma hdrstop

#include "codeFrameAssessmentCardio.h"
#include "codeRecordPatientAssessment.h"
//----------------------------------------------------------------------------------------------------------------------
#pragma package(smart_init)
#pragma link "codeFrameAssessmentGeneric"
#pragma resource "*.dfm"
TframeAssessmentCardio *frameAssessmentCardio;
//----------------------------------------------------------------------------------------------------------------------
__fastcall TframeAssessmentCardio::TframeAssessmentCardio(TComponent* Owner)
        : TframeAssessmentGeneric(Owner), cardio_filter(), editing_cardio(), ending_state(), cardio_request(-1),
          editing_id(), editing_cardio_items(), cardio_items_data()
{
  SetupCardioItemsMap();
}
//----------------------------------------------------------------------------------------------------------------------
// SetupCardioItemsMap - initializes the multi-level data structure used to get to the extremity pulses checkboxes
void TframeAssessmentCardio::SetupCardioItemsMap()
{
  cardio_items_data.clear();

  // Upper Right Items
  cardio_items_data["Strong"]    ["Right"]["Upper"] = checkUR1->Name.c_str();
  cardio_items_data["Faint"]     ["Right"]["Upper"] = checkUR2->Name.c_str();
  cardio_items_data["Absent"]    ["Right"]["Upper"] = checkUR3->Name.c_str();
  cardio_items_data["Cold"]      ["Right"]["Upper"] = checkUR4->Name.c_str();
  cardio_items_data["Discolored"]["Right"]["Upper"] = checkUR5->Name.c_str();
  // Lower Right Items
  cardio_items_data["Strong"]    ["Right"]["Lower"] = checkLR1->Name.c_str();
  cardio_items_data["Faint"]     ["Right"]["Lower"] = checkLR2->Name.c_str();
  cardio_items_data["Absent"]    ["Right"]["Lower"] = checkLR3->Name.c_str();
  cardio_items_data["Cold"]      ["Right"]["Lower"] = checkLR4->Name.c_str();
  cardio_items_data["Discolored"]["Right"]["Lower"] = checkLR5->Name.c_str();
  // Upper Left Items
  cardio_items_data["Strong"]    ["Left" ]["Upper"] = checkUL1->Name.c_str();
  cardio_items_data["Faint"]     ["Left" ]["Upper"] = checkUL2->Name.c_str();
  cardio_items_data["Absent"]    ["Left" ]["Upper"] = checkUL3->Name.c_str();
  cardio_items_data["Cold"]      ["Left" ]["Upper"] = checkUL4->Name.c_str();
  cardio_items_data["Discolored"]["Left" ]["Upper"] = checkUL5->Name.c_str();
  // Lower Left Items
  cardio_items_data["Strong"]    ["Left" ]["Lower"] = checkLL1->Name.c_str();
  cardio_items_data["Faint"]     ["Left" ]["Lower"] = checkLL2->Name.c_str();
  cardio_items_data["Absent"]    ["Left" ]["Lower"] = checkLL3->Name.c_str();
  cardio_items_data["Cold"]      ["Left" ]["Lower"] = checkLL4->Name.c_str();
  cardio_items_data["Discolored"]["Left" ]["Lower"] = checkLL5->Name.c_str();
}
//----------------------------------------------------------------------------------------------------------------------
// CheckBoxForItems - returns a TCheckBox * for a set of three lookup table descriptive item strings
// (ex: "Strong", "Right", "Upper") - returns NULL if no such item.
TCheckBox *TframeAssessmentCardio::CheckBoxForItems(string ext_pulse_type, string body_side_type, string extremity_type)
{
  if(ext_pulse_type == "" || body_side_type == "" || extremity_type == "")
    return NULL;

  string control_name = cardio_items_data[ext_pulse_type][body_side_type][extremity_type];
  if(control_name == "")
    return NULL;

  TControl *myCheckBox = groupExtPulse->FindChildControl(control_name.c_str());
  if(myCheckBox && myCheckBox->InheritsFrom(__classid(TCheckBox)))
    return dynamic_cast<TCheckBox *>(myCheckBox);

  return NULL;
}
//----------------------------------------------------------------------------------------------------------------------
// CheckBoxForIDs - returns a TCheckBox * given three lookup IDs by translating them into the corresponding values in
// the lookup tables and then calling the routine above. Returns NULL if no such item.
TCheckBox *TframeAssessmentCardio::CheckBoxForIDs(string ext_pulse_id, string body_side_id, string extremity_id)
{
  string ext_pulse_descr = lookupTables->ValueOf("ext_pulse_types", ext_pulse_id);
  string body_side_descr = lookupTables->ValueOf("body_side_types", body_side_id);
  string extremity_descr = lookupTables->ValueOf("extremity_types", extremity_id);

  return CheckBoxForItems(ext_pulse_descr, body_side_descr, extremity_descr);
}
//----------------------------------------------------------------------------------------------------------------------
// GetExtremityPulses - Returns a vector of cardio_save_data structures for every checkbox that is checked inside the
// Extremity Pulses group box. Unchecked boxes are simply not present in the return vector.
vector<TframeAssessmentCardio::cardio_save_data> TframeAssessmentCardio::GetExtremityPulses()
{
  vector<cardio_save_data> to_return;

  for(cardio_itemsmap::iterator i_extpulse = cardio_items_data.begin();
      i_extpulse != cardio_items_data.end();
      ++i_extpulse)
  {
    for(cardio_itemsmap::mapped_type::iterator i_side = i_extpulse->second.begin();
        i_side != i_extpulse->second.end();
        ++i_side)
    {
      for(map<string, string>::iterator i_extrem = i_side->second.begin(); i_extrem != i_side->second.end(); ++i_extrem)
      {
        TCheckBox *myCheckBox = CheckBoxForItems(i_extpulse->first, i_side->first, i_extrem->first);

        if(myCheckBox && myCheckBox->Checked)
        {
          cardio_save_data new_data;

          new_data.ext_pulse_id = lookupTables->IdOf("ext_pulse_types", i_extpulse->first);
          new_data.body_side_id = lookupTables->IdOf("body_side_types", i_side->first);
          new_data.extremity_id = lookupTables->IdOf("extremity_types", i_extrem->first);

          to_return.push_back(new_data);
        }
      }
    }
  }

  return to_return;
}
//----------------------------------------------------------------------------------------------------------------------
// Initialize - Overridden to set parent properties and setup cardio filtered listview.
bool TframeAssessmentCardio::Initialize()
{
  // Set AssessmentGeneric properties
  SetBandCaption("Cardiovascular");
  SetPermission("cardio_assessment");

  // Setup respiratory filter
  cardio_filter.SetListViewHandle(listCardio);
  cardio_filter.SetSecurity("cardio_assessment");

  cardio_filter.AddColumnSpec("NPI",          "cardio_assessment", as_text,   true,   65, prettyBoolString);
  cardio_filter.AddColumnSpec("Pacemaker",    "cardio_assessment", as_text,   true,   70, prettyBoolString);
  cardio_filter.AddColumnSpec("Tele",         "cardio_assessment", as_text,   true,   45, prettyBoolString);
  cardio_filter.AddColumnSpec("AICD",         "cardio_assessment", as_text,   true,   45, prettyBoolString);
  cardio_filter.AddColumnSpec("Heart Rate",   "cardio_assessment", as_int,    true,   65);
  cardio_filter.AddColumnSpec("Rhythm",       "cardio_assessment", as_text,   true,   60, rawString);
  cardio_filter.AddColumnSpec("Cap Refill",   "cardio_assessment", as_int,    true,   60);
  cardio_filter.AddColumnSpec("Has Edema",    "cardio_assessment", as_text,   true,   80, prettyBoolString);
  cardio_filter.AddColumnSpec("Edema Type",   "cardio_assessment", as_text,   true,   80, rawString);
  cardio_filter.AddColumnSpec("Location",     "cardio_assessment", as_text,    true, 110, rawString);
  cardio_filter.AddColumnSpec("Comments",     "cardio_assessment", as_text,   true,  110, rawString);
  cardio_filter.AddColumnSpec("Current Id",   "current_id",        as_int,    false,  80);
  cardio_filter.AddColumnSpec("Internal Id",  "internal_id",       as_int,    false,  80);

  cardio_filter.FetchPreferences("assessment.cardio.filter");
  cardio_filter.SetSQLStatement("");
  cardio_filter.RefreshDataSet();

  // Register lookups
  string lookupName = "cardio.cboRhythm";
  string tableName  = "rhythm_types";
  RegisterLookup(lookupName, tableName, cboRhythm);

  lookupName = "cardio.cboEdemaType";
  tableName  = "edema_types";
  RegisterLookup(lookupName, tableName, cboEdemaType);

  // Set standard modified event handlers for all controls on panelResp
  SetAllEventHandlers(panelCardio);
  SetCheckBoxHandlers(panelCardio);

  return TframeAssessmentGeneric::Initialize();
}
//----------------------------------------------------------------------------------------------------------------------
// Lock - Overridden to lock Add button and cardio panel
void TframeAssessmentCardio::Lock()
{
  TframeAssessmentGeneric::Lock(); // lock parent controls
  btnAdd->Enabled = false;         // disable add button
  panelCardio->Enabled = false;    // lock all controls
}
//----------------------------------------------------------------------------------------------------------------------
// PutUpCardioItems - loads the "Extremity Pulses" checkboxes with values from cardio_items records
// associated with the editing cardio_assessment
void TframeAssessmentCardio::PutUpCardioItems(pvecmap &cardio_items)
{
  ClearControlsChecked(groupExtPulse);

  for(size_t i = 0; i < cardio_items.size(); i++)
  {
    // Skip items which are anything other than Current. 
    if(cardio_items[i]["current_id"] != lookupTables->IdOf("current_types", "Current"))
      continue;

    // find the checkbox for this combination of ext_pulse_types, body_side_types, and extremity_types IDs
    TCheckBox *myCheckBox =
      CheckBoxForIDs(cardio_items[i]["ext_pulse_id"], cardio_items[i]["body_side_id"], cardio_items[i]["extremity_id"]);

    if(myCheckBox)
      myCheckBox->Checked = true;
  }
}
//----------------------------------------------------------------------------------------------------------------------
// TaskIdChanging - Overridden to populate cardiovascular filter
void TframeAssessmentCardio::TaskIdChanging(const string &new_id)
{
  string sql;

  TframeAssessmentGeneric::TaskIdChanging(new_id);

  sql = "select CA.is_npi, CA.is_pacemaker, CA.is_tele, CA.is_aicd, CA.heart_rate, RT.item, CA.cap_refill, "
        " CA.has_edema, ET.item, CA.edema_location, CA.comments, CA.current_id, CA.id "
        "from cardio_assessment CA "
        "left join rhythm_types RT on CA.rhythm_id = RT.id "
        "left join edema_types  ET on CA.edema_id  = ET.id "
        "where CA.current_id >= 2 and CA.task_id = '" + new_id + "' ";

  cardio_filter.SetSQLStatement(sql);
  cardio_filter.RefreshDataSet();
  cardio_filter.SetAddEdit(this, AddRecord (&TframeAssessmentCardio::FilterAddCardio),
                                 EditRecord(&TframeAssessmentCardio::FilterEditCardio));
  cardio_request = parent_form->RequestNotificationWhen("cardio_assessment.task_id", new_id, rel_e, as_int);
}
//----------------------------------------------------------------------------------------------------------------------
// LockRecords - Overridden to lock cardio_assessment record and cardio_items subrecords for editing.
bool TframeAssessmentCardio::LockRecords()
{
  if(!LockRecordForUpdate(generic_transaction, "cardio_assessment", editing_id))
    return false;

  // Must acquire all items. Any failure is a failure to unlock for editing, GTFO and rollback.
  if(!LockVecMapRecords(editing_cardio_items, "cardio_items"))
    return false;

  return TframeAssessmentGeneric::LockRecords();
}
//----------------------------------------------------------------------------------------------------------------------
// CanUnlock - Overridden to test for a selected cardio assessment
bool TframeAssessmentCardio::CanUnlock(bool &creating_new)
{
  editing_id = cardio_filter.GetSelectedRecordID();

  if(!creating_new && editing_id == "")
  {
    parent_form->PSay("Please select a cardiovascular assessment to edit in the list first.", m_reminder, MB_OK);
    return false;
  }

  return TframeAssessmentGeneric::CanUnlock(creating_new);
}
//----------------------------------------------------------------------------------------------------------------------
// UpdateForEdit - Overridden to put fresh data into the editing_cardio map and on screen.
void TframeAssessmentCardio::UpdateForEdit()
{
  string sql = "select * from cardio_assessment where id = '" + editing_id + "' ";

  if(!SqlToMap(generic_transaction, sql, editing_cardio))
    parent_form->PSay("Could not retrieve up-to-date information for cardio_assessment", m_warning, MB_OK);

  // Start all checkboxes cleared
  ClearControlsChecked(panelCardio);

  // Combo boxes
  string rhythm_type = lookupTables->ValueOf("rhythm_types", editing_cardio["rhythm_id"]);
  cboRhythm->Text = rhythm_type.c_str();

  string edema_type  = lookupTables->ValueOf("edema_types",  editing_cardio["edema_id"]);
  cboEdemaType->Text = edema_type.c_str();

  // Edits & Memos
  editBPM->Text       = editing_cardio["heart_rate"    ].c_str();
  editCapRefill->Text = editing_cardio["cap_refill"    ].c_str();
  editLocation->Text  = editing_cardio["edema_location"].c_str();
  memoComments->Text  = editing_cardio["comments"      ].c_str();

  // Checkboxes (external record)
  checkNPI->Checked       = (editing_cardio["is_npi"      ] == "1");
  checkPacemaker->Checked = (editing_cardio["is_pacemaker"] == "1");
  checkTele->Checked      = (editing_cardio["is_tele"     ] == "1");
  checkAICD->Checked      = (editing_cardio["is_aicd"     ] == "1");
  checkEdema->Checked     = (editing_cardio["has_edema"   ] == "1");

  // need also all cardio_items
  editing_cardio_items.clear();

  if(IsInt(editing_id) && !IsBlankOrZero(editing_id))
  {
    string sql =
      "select * "
      "from cardio_items CI "
      "where CI.cardio_assessment_id = '" + editing_id + "' ";

    if(!SqlToVecMap(generic_transaction, sql, editing_cardio_items))
      parent_form->PSay("Could not retrieve up-to-date information for extremity pulses", m_warning, MB_OK);

    // Refresh controls
    PutUpCardioItems(editing_cardio_items);
  }

  // Make sure all edits are marked unmodified
  ClearControlsModified(panelCardio);

  TframeAssessmentGeneric::UpdateForEdit();
}
//----------------------------------------------------------------------------------------------------------------------
// SavingNew - Overridden to return status of editing_cardio as new record or unlocked record
bool TframeAssessmentCardio::SavingNew()
{
  return editing_cardio["id"] == "new";
}
//----------------------------------------------------------------------------------------------------------------------
// IsValidOnScreen - Overridden to validate everything
bool TframeAssessmentCardio::IsValidOnScreen()
{
  vector<cardio_save_data> ext_pulses = GetExtremityPulses();

  // NPI is checked?
  if(checkNPI->Checked)
  {
    if(checkPacemaker->Checked || checkTele->Checked || checkAICD->Checked || checkEdema->Checked)
    {
      parent_form->PSay("You checked the patient as having No Known Problems, but also selected one of the other "
                        "problem indicators (pacemaker, tele, AICD, edema). Please clear one or more of the "
                        "conflicting indications.",
                        m_invalid_data, MB_OK);
      return false;
    }

    // check that all checked extremity pulses are "Strong", if any exist
    for(vector<cardio_save_data>::iterator item = ext_pulses.begin(); item != ext_pulses.end(); ++item)
    {
      if(item->ext_pulse_id != lookupTables->IdOf("ext_pulse_types", "Strong"))
      {
        parent_form->PSay("You checked the patient as having No Known Problems, but also selected one or more adverse "
                          "extremity pulse types (a type other than Strong) from the Extremity Pulses box. Please "
                          "clear one or more of the conflicting indications.",
                          m_invalid_data, MB_OK);
        return false;
      }
    }

    if(editLocation->Text != "")
    {
      parent_form->PSay("You checked the patient as having No Known Problems, but indicated that edema is present "
                        "at a given location. Please clear one or more of the conflicting indications.",
                        m_invalid_data, MB_OK);
      return false;
    }

    if(cboRhythm->Text == "Irregular")
    {
      parent_form->PSay("You checked the patient as having No Known Problems, but indicated the patient has an "
                        "irregular heart rate. Please clear one or more of these conflicting indications.",
                        m_invalid_data, MB_OK);
      return false;
    }

    if(cboEdemaType->Text != "")
    {
      parent_form->PSay("You checked the patient as having No Known Problems, but indicated that there is Pitting or "
                        "Non-Pitting Edema present. Please clear one or more of the conflicting indications.",
                        m_invalid_data, MB_OK);
      return false;
    }

    // Allowed with NPI: Heart Rate, Cap Refill, Comments
  }

  // NPI is NOT checked?
  if(!checkNPI->Checked)
  {
    // Something else at least semi-structured must be provided that indicates a problem.
    if(!checkPacemaker->Checked && !checkTele->Checked && !checkAICD->Checked &&
       editBPM->Text == "" && cboRhythm->Text == "" && editCapRefill->Text == "" && !checkEdema->Checked &&
       cboEdemaType->Text == "" && editLocation->Text == "" && ext_pulses.size() == 0)
    {
      parent_form->PSay("You did not check the No Known Problems indicator, but have not documented any issues with the "
                        "cardiovascular assessment. Please re-evaluate the patient status.", m_missing_data, MB_OK);
      return false;
    }
  }

  if(!lookupTables->IsInLookUpTable("rhythm_types", cboRhythm))
  {
    parent_form->PSay("Please enter a valid rhythm type.", m_invalid_data, MB_OK);
    return false;
  }
  if(!lookupTables->IsInLookUpTable("edema_types", cboEdemaType))
  {
    parent_form->PSay("Please enter a valid edema type.", m_invalid_data, MB_OK);
    return false;
  }
  if(editBPM->Text != "" && !IsInt(editBPM->Text.c_str()))
  {
    parent_form->PSay("Please enter a valid integer numeric value for Heart Rate (BPM).", m_invalid_data, MB_OK);
    return false;
  }
  if(editCapRefill->Text != "" && !IsInt(editCapRefill->Text.c_str()))
  {
    parent_form->PSay("Please enter a valid integer numeric value for Cap Refill (sec).", m_invalid_data, MB_OK);
    return false;
  }

  return TframeAssessmentGeneric::IsValidOnScreen();
}
//----------------------------------------------------------------------------------------------------------------------
// DoSaveData - Overridden to provide core saving logic.
bool TframeAssessmentCardio::DoSaveData()
{
  pmapstrtoint leave_alone;
  ending_state = editing_cardio;

  ending_state["current_id"]        = lookupTables->IdOf("current_types", "Current");
  ending_state["last_update_stamp"] = RightNow();
  ending_state["last_updated_by"]   = current_userid;

  if(ending_state["id"] == "new")
  {
    ending_state["creation_stamp"] = ending_state["last_update_stamp"];
    ending_state["created_by"]     = ending_state["last_updated_by"];
  }

  // scalar checkboxes
  ending_state["is_npi"]        = checkNPI->Checked       ? "1" : "0";
  ending_state["is_pacemaker"]  = checkPacemaker->Checked ? "1" : "0";
  ending_state["is_tele"]       = checkTele->Checked      ? "1" : "0";
  ending_state["is_aicd"]       = checkAICD->Checked      ? "1" : "0";
  ending_state["has_edema"]     = checkEdema->Checked     ? "1" : "0";

  // plain text edits
  ending_state["heart_rate"]     = editBPM->Text.c_str();
  ending_state["cap_refill"]     = editCapRefill->Text.c_str();
  ending_state["edema_location"] = editLocation->Text.c_str();
  ending_state["comments"]       = memoComments->Text.c_str();

  // do not capitalize these
  leave_alone["edema_location"] = sql_no_uppercasing;
  leave_alone["comments"]       = sql_no_uppercasing;

  // lookups
  ending_state["rhythm_id"] = lookupTables->IdOf("rhythm_types", cboRhythm->Text.c_str());
  ending_state["edema_id"]  = lookupTables->IdOf("edema_types",  cboEdemaType->Text.c_str());

  // Not seen:
  // task_id - this either comes from the existing cardio_assessment or is set in editing_cardio on add

  // Save the main record
  if(!DoInsertOrUpdate(ending_state, leave_alone, "cardio_assessment"))
    return false; // BURP!

  // Delete existing cardio_items (it really is truly the only manageable way to do this...)
  if(editing_cardio["id"] != "new")
  {
    if(!ExecuteStatement(generic_transaction,
                         "delete from cardio_items where cardio_assessment_id = " + editing_cardio["id"]))
    {
      return false; // BURP!
    }
  }

  // Insert new cardio_items
  vector<cardio_save_data> cardio_items = GetExtremityPulses();

  for(vector<cardio_save_data>::iterator item = cardio_items.begin(); item != cardio_items.end(); ++item)
  {
    pmapstrs     new_cardio_item;
    pmapstrtoint item_leave_alone;

    new_cardio_item["id"] = GetNextId(generic_transaction, "cardio_items");
    if(!IsInt(new_cardio_item["id"]))
      return false; // BURP!

    new_cardio_item["current_id"]           = lookupTables->IdOf("current_types", "Current");
    new_cardio_item["last_update_stamp"]    = ending_state["last_update_stamp"];
    new_cardio_item["last_updated_by"]      = ending_state["last_updated_by"];
    new_cardio_item["cardio_assessment_id"] = ending_state["id"];
    new_cardio_item["body_side_id"]         = item->body_side_id;
    new_cardio_item["extremity_id"]         = item->extremity_id;
    new_cardio_item["ext_pulse_id"]         = item->ext_pulse_id;

    if(!ExecuteInsertStatement(generic_transaction, "cardio_items", new_cardio_item, item_leave_alone))
      return false; // BURP!
  }

  return true; // success!
}
//----------------------------------------------------------------------------------------------------------------------
// SetViewMode - Overridden to activate viewing of cardio assessments
void TframeAssessmentCardio::SetViewMode()
{
  TframeAssessmentGeneric::SetViewMode();
  btnAdd->Enabled = HaveUnlockPermission(true); // It is now possible to add a respiratory assessment
  panelCardio->Enabled = false;                 // But it is still not possible to enter data.
  editing_cardio.clear();                       // Clear out the cardio map for safety

  // Reset control contents
  ClearControlsText(panelCardio);
  ClearControlsModified(panelCardio);
  ClearControlsChecked(panelCardio);
}
//----------------------------------------------------------------------------------------------------------------------
// SetEditMode - Overridden to activate editing of a cardio assessment
void TframeAssessmentCardio::SetEditMode()
{
  TframeAssessmentGeneric::SetEditMode();
  btnAdd->Enabled = false;                // Can't add cardio assessments right now
  panelCardio->Enabled = true;            // Enable panel controls
}
//----------------------------------------------------------------------------------------------------------------------
// btnAddClick - Handler for toolbar button to add a new cardiovascular assessment
void __fastcall TframeAssessmentCardio::btnAddClick(TObject *Sender)
{
  if(DoUnlock(true))
  {
    // Reset control contents
    ClearControlsText(panelCardio);
    ClearControlsModified(panelCardio);
    ClearControlsChecked(panelCardio);

    editing_cardio.clear();
    editing_cardio["id"] = "new"; // Creating a new cardio_assessment
    editing_cardio["task_id"] = task_id;
  }
}
//----------------------------------------------------------------------------------------------------------------------
// FilterAddCardio - "Add" handler for the cardio filter. We redirect this to the "Add a Cardio" code so that it
// works compatibly with the assessment frame.
void TframeAssessmentCardio::FilterAddCardio()
{
  if(btnAdd->Enabled) // Only when you could add using the button.
    btnAddClick(NULL);
}
//----------------------------------------------------------------------------------------------------------------------
// FilterEditCardio - Bleh.
void TframeAssessmentCardio::FilterEditCardio()
{
  if(btnUnlockSave->Enabled && frame_mode == mode_viewing) // Only when you can start editing via the button.
    btnUnlockSaveClick(NULL);
}
//----------------------------------------------------------------------------------------------------------------------
// AtlasNotify - Overridden to refresh cardio listview
bool TframeAssessmentCardio::AtlasNotify(int request_num, string matched_value)
{
  bool handled = false;

  if(request_num == cardio_request)
  {
    cardio_filter.RefreshDataSet();
    handled = true;
  }

  return handled;
}
//----------------------------------------------------------------------------------------------------------------------
// SendAtlas - Overridden to send out notifications for cardio_assessment
void TframeAssessmentCardio::SendAtlas()
{
  bool new_record = (editing_cardio["id"] == "new");

  SendAtlasIfChanged(new_record, "cardio_assessment", editing_cardio, ending_state);
}
//----------------------------------------------------------------------------------------------------------------------
string TframeAssessmentCardio::CaseForItem(string extremity_type, string body_side_type, string ext_pulse_type)
{
  return
    "case when exists "
    "(select * "
    " from cardio_items CI "
    " where CI.cardio_assessment_id = CA.id and CI.current_id >= 3 and "
    " CI.extremity_id = '" + lookupTables->IdOf("extremity_types", extremity_type) + "' and "
    " CI.body_side_id = '" + lookupTables->IdOf("body_side_types", body_side_type) + "' and "
    " CI.ext_pulse_id = '" + lookupTables->IdOf("ext_pulse_types", ext_pulse_type) + "') "
    "then 'X' else '' end ";
}
//----------------------------------------------------------------------------------------------------------------------
// WriteReport - Write out a report on the cardiovascular assessment
bool TframeAssessmentCardio::WriteReport(TComponent *compositeReportCtrl)
{
  bool success = false;

  try
  {
    string sql =
      "select "
      " case when CA.is_npi = 1 then 'X' else '' end npi_indicator, "
      " case when CA.is_pacemaker = 1 then 'X' else '' end pace_indicator, "
      " case when CA.is_tele = 1 then 'X' else '' end tele_indicator, "
      " case when CA.is_aicd = 1 then 'X' else '' end aicd_indicator, "
      " CA.heart_rate, RT.item rhythm, CA.cap_refill, "
      " case when CA.has_edema = 1 then 'X' else '' end edema_indicator, "
      " ET.item edema_type, CA.edema_location, CA.comments, "
      + CaseForItem("Upper", "Right", "Strong") + " ur_strong, "
      + CaseForItem("Upper", "Right", "Faint")  + " ur_faint,  "
      + CaseForItem("Upper", "Right", "Absent") + " ur_absent, "
      + CaseForItem("Upper", "Right", "Cold")   + " ur_cold, "
      + CaseForItem("Upper", "Right", "Discolored") + " ur_discol, "
      + CaseForItem("Lower", "Right", "Strong") + " lr_strong, "
      + CaseForItem("Lower", "Right", "Faint")  + " lr_faint,  "
      + CaseForItem("Lower", "Right", "Absent") + " lr_absent, "
      + CaseForItem("Lower", "Right", "Cold")   + " lr_cold, "
      + CaseForItem("Lower", "Right", "Discolored") + " lr_discol, "
      + CaseForItem("Upper", "Left",  "Strong") + " ul_strong, "
      + CaseForItem("Upper", "Left",  "Faint")  + " ul_faint,  "
      + CaseForItem("Upper", "Left",  "Absent") + " ul_absent, "
      + CaseForItem("Upper", "Left",  "Cold")   + " ul_cold, "
      + CaseForItem("Upper", "Left",  "Discolored") + " ul_discol, "
      + CaseForItem("Lower", "Left",  "Strong") + " ll_strong, "
      + CaseForItem("Lower", "Left",  "Faint")  + " ll_faint,  "
      + CaseForItem("Lower", "Left",  "Absent") + " ll_absent, "
      + CaseForItem("Lower", "Left",  "Cold")   + " ll_cold, "
      + CaseForItem("Lower", "Left",  "Discolored") + " ll_discol "
      "from cardio_assessment CA "
      "left join rhythm_types RT on CA.rhythm_id = RT.id "
      "left join edema_types  ET on CA.edema_id  = ET.id "
      "where CA.current_id >= 3 and CA.task_id = '" + task_id + "' ";

    InitDataSet(sql);

    TformReportAssessment *reportForm = report_data.reportForm;
    TQuickRep *thisReport = report_data.reportForm->reportCardio;

    thisReport->DataSet = report_data.dbDataSet;

    txtregister_t ctrlsToRegister[] =
    {
      { reportForm->txtCardioNPICheck,  "npi_indicator"   },
      { reportForm->txtCardioPacemaker, "pace_indicator"  },
      { reportForm->txtCardioTele,      "tele_indicator"  },
      { reportForm->txtCardioAICD,      "aicd_indicator"  },
      { reportForm->txtCardioBPM,       "heart_rate"      },
      { reportForm->txtCardioRhythm,    "rhythm"          },
      { reportForm->txtCardioCapRefill, "cap_refill"      },
      { reportForm->txtCardioEdema,     "edema_indicator" },
      { reportForm->txtCardioEdemaType, "edema_type"      },
      { reportForm->txtCardioLocation,  "edema_location"  },
      { reportForm->txtCardioComments,  "comments"        },
      { reportForm->txtURStrong,        "ur_strong"       },
      { reportForm->txtURFaint,         "ur_faint"        },
      { reportForm->txtURAbsent,        "ur_absent"       },
      { reportForm->txtURCold,          "ur_cold"         },
      { reportForm->txtURDiscol,        "ur_discol"       },
      { reportForm->txtLRStrong,        "lr_strong"       },
      { reportForm->txtLRFaint,         "lr_faint"        },
      { reportForm->txtLRAbsent,        "lr_absent"       },
      { reportForm->txtLRCold,          "lr_cold"         },
      { reportForm->txtLRDiscol,        "lr_discol"       },
      { reportForm->txtULStrong,        "ul_strong"       },
      { reportForm->txtULFaint,         "ul_faint"        },
      { reportForm->txtULAbsent,        "ul_absent"       },
      { reportForm->txtULCold,          "ul_cold"         },
      { reportForm->txtULDiscol,        "ul_discol"       },
      { reportForm->txtLLStrong,        "ll_strong"       },
      { reportForm->txtLLFaint,         "ll_faint"        },
      { reportForm->txtLLAbsent,        "ll_absent"       },
      { reportForm->txtLLCold,          "ll_cold"         },
      { reportForm->txtLLDiscol,        "ll_discol"       },
      { NULL,                           NULL              }
    };
    SetupQRDBTexts(ctrlsToRegister);

    // NB: Need to pass it as TComponent to avoid problems with having QuickReport included in the headers
    TQRCompositeReport *compositeReport = dynamic_cast<TQRCompositeReport *>(compositeReportCtrl);
    if(compositeReport)
    {
      compositeReport->Reports->Add(thisReport);
      success = true;
    }
  }
  catch(...)
  {
  }

  return success;
}
//----------------------------------------------------------------------------------------------------------------------

