//----------------------------------------------------------------------------------------------------------------------

#include <vcl.h>
#pragma hdrstop

#include "codeFrameAssessmentDME.h"
#include "codeRecordPatientAssessment.h"
//----------------------------------------------------------------------------------------------------------------------
#pragma package(smart_init)
#pragma link "codeFrameAssessmentGeneric"
#pragma resource "*.dfm"
TframeAssessmentDME *frameAssessmentDME;
//----------------------------------------------------------------------------------------------------------------------
__fastcall TframeAssessmentDME::TframeAssessmentDME(TComponent* Owner)
        : TframeAssessmentGeneric(Owner), dme_filter(), checks_filter(), dme_request(-1), editing_dme_items(),
          final_ending_state()
{
}
//----------------------------------------------------------------------------------------------------------------------
// Initialize - Overridden to set parent properties and setup equipment-in-use filtered listview.
bool TframeAssessmentDME::Initialize()
{
  // Set AssessmentGeneric properties
  SetBandCaption("Equipment");
  SetPermission("equipment_in_use");

  // Setup notes filter
  dme_filter.SetListViewHandle(listDME);
  dme_filter.SetSecurity("equipment_in_use");

  dme_filter.AddColumnSpec("Item",        "equipment_in_use", as_text, true,  160, rawString);
  dme_filter.AddColumnSpec("Details",     "equipment_in_use", as_text, true,  420, sentenceString);
  dme_filter.AddColumnSpec("Current Id",  "current_id",       as_int,  false,  80);
  dme_filter.AddColumnSpec("Internal Id", "internal_id",      as_int,  false,  80);

  dme_filter.FetchPreferences("assessment.dme.topfilter");
  dme_filter.SetSQLStatement("");
  dme_filter.RefreshDataSet();

  // setup equipment_items input filter

  string items_sql = "select item, id from equipment_types where valid = 1 and id > 0 ";

  // Checks Filter
  checks_filter.SetListViewHandle(listDMEChecks);
  checks_filter.SetSecurity("");
  checks_filter.AddColumnSpec("Item",        "equipment_in_use", as_text, true,  128, rawString);
  checks_filter.AddColumnSpec("Internal Id", "internal_id",      as_int,  false,  80);
  checks_filter.FetchPreferences("assessment.dme.equipment_items");
  checks_filter.SetIconStatus(false);
  checks_filter.SetCheck(true);
  checks_filter.SetSQLStatement(items_sql);
  checks_filter.RefreshDataSet();

  // Set edit box change handlers
  SetAllEventHandlers(panelDME);

  return TframeAssessmentGeneric::Initialize();
}
//----------------------------------------------------------------------------------------------------------------------
// Lock - Overridden to lock DME panel
void TframeAssessmentDME::Lock()
{
  TframeAssessmentGeneric::Lock(); // lock parent controls
  panelDME->Enabled = false;       // lock all panel controls
}
//----------------------------------------------------------------------------------------------------------------------
// PutUpEquipmentItems - Refreshes the controls to reflect the state of the DME list
void TframeAssessmentDME::PutUpEquipmentItems(pvecmap &dme_items)
{
  psetstr equipment_ids;

  // Clear out the checkboxes and edit box
  checks_filter.UncheckAll();
  editOther->Text = "";

  // Analyze the retrieved equipment_in_use items
  for(size_t i = 0; i < dme_items.size(); i++)
  {
    // Skip items which are anything other than Current. This allows Deleted items to be updated back to Current status
    // when saving.
    if(dme_items[i]["current_id"] != lookupTables->IdOf("current_types", "Current"))
      continue;
       
    equipment_ids.insert(dme_items[i]["equipment_id"]);

    if(dme_items[i]["equipment_id"] == lookupTables->IdOf("equipment_types", "Other"))
      editOther->Text = SentenceString(dme_items[i]["comments"]).c_str();
  }

  // Set checkboxes
  checks_filter.SetCheckedWhereID(equipment_ids, true);
}
//----------------------------------------------------------------------------------------------------------------------
// TaskIdChanging - Overridden to populate DME filter
void TframeAssessmentDME::TaskIdChanging(const string &new_id)
{
  string sql;

  TframeAssessmentGeneric::TaskIdChanging(new_id);

  sql = "select EQT.item, EQ.comments, EQ.current_id, EQ.id "
        "from equipment_in_use EQ "
        "left join equipment_types EQT on EQ.equipment_id = EQT.id "
        "where EQ.current_id >= 2 and EQ.task_id = '" + new_id + "' ";

  dme_filter.SetSQLStatement(sql);
  dme_filter.RefreshDataSet();
  dme_filter.SetAddEdit(this, AddRecord (&TframeAssessmentDME::FilterAddDME),
                              EditRecord(&TframeAssessmentDME::FilterEditDME));
  dme_request = parent_form->RequestNotificationWhen("equipment_in_use.task_id", new_id, rel_e, as_int);
}
//----------------------------------------------------------------------------------------------------------------------
// UpdateForEdit - Overridden to put fresh data into the editing_dme_items vecmap and on screen.
void TframeAssessmentDME::UpdateForEdit()
{
  string sql =
    "select EQ.equipment_id, EQ.comments, EQ.current_id, EQ.id "
    "from equipment_in_use EQ "
    "where EQ.task_id = '" + task_id + "' ";

  editing_dme_items.clear();

  if(!SqlToVecMap(generic_transaction, sql, editing_dme_items))
    parent_form->PSay("Could not retrieve up-to-date information for equipment_in_use items", m_warning, MB_OK);

  // Refresh controls
  PutUpEquipmentItems(editing_dme_items);

  TframeAssessmentGeneric::UpdateForEdit();
}
//----------------------------------------------------------------------------------------------------------------------
// LockRecords - Overridden to lock the equipment_in_use records (you might say they're really in use then ;)
bool TframeAssessmentDME::LockRecords()
{
  // Must acquire all items. Any failure is a failure to unlock for editing, GTFO and rollback.
  if(!LockVecMapRecords(editing_dme_items, "equipment_in_use"))
    return false;

  return TframeAssessmentGeneric::LockRecords();
}
//----------------------------------------------------------------------------------------------------------------------
// IsValidOnScreen - Overridden to do validation for equipment_in_use
bool TframeAssessmentDME::IsValidOnScreen()
{
  pvecstr checked_ids = checks_filter.GetMultiID();
  bool other_is_checked = false;
  
  // Make sure at least one equipment item is specified
  if(editing_dme_items.size() == 0 && // jhaley 20110629: Only when the record is new, otherwise can't delete last one
     checked_ids.size() == 0)
  {
    parent_form->PSay("You have not specified any saveable data. Please check one or more equipment items.",
                      m_missing_data, MB_OK);
    return false;
  }

  for(unsigned int i = 0; i < checked_ids.size(); i++)
  {
    // Still have to hard code it >_>
    if(checked_ids[i] == lookupTables->IdOf("equipment_types", "Other"))
    {
      other_is_checked = true;
      if(editOther->Text == "")
      {
        parent_form->PSay("You selected 'Other' but did not provide any information on the equipment.",
                          m_missing_data, MB_OK);
        return false;
      }
    }
  }

  if(!other_is_checked && editOther->Text != "")
  {
    parent_form->PSay("You did not select 'Other', but specified information in the 'Other Equipment' field. This "
                      "information would be lost, so please either clear it out now, or select the 'Other' item in "
                      "the checkbox list as well.", m_invalid_data, MB_OK);
    return false;
  }

  return TframeAssessmentGeneric::IsValidOnScreen();
}
//----------------------------------------------------------------------------------------------------------------------
bool TframeAssessmentDME::UpdateDMEItem(int record_index, string comment)
{
  pmapstrs ending_state;
  pmapstrtoint leave_alone;

  ending_state["id"]                 = editing_dme_items[record_index]["id"];
  ending_state["current_id"]         = lookupTables->IdOf("current_types", "Current");
  ending_state["last_updated_by"]    = current_userid;
  ending_state["last_update_stamp"]  = RightNow();
  ending_state["task_id"]            = task_id;
  ending_state["equipment_id"]       = editing_dme_items[record_index]["equipment_id"];
  ending_state["comments"]           = comment;

  final_ending_state.push_back(ending_state);

  return ExecuteUpdateStatement(generic_transaction, "equipment_in_use", ending_state, leave_alone);
}
//----------------------------------------------------------------------------------------------------------------------
bool TframeAssessmentDME::InsertDMEItem(string equipment_id, string comment)
{
  pmapstrs ending_state;
  pmapstrtoint leave_alone;

  ending_state["id"]                = GetNextId(generic_transaction, "equipment_in_use");
  ending_state["current_id"]        = lookupTables->IdOf("current_types", "Current");
  ending_state["last_updated_by"]   = current_userid;
  ending_state["last_update_stamp"] = RightNow();
  ending_state["created_by"]        = current_userid;
  ending_state["creation_stamp"]    = ending_state["last_update_stamp"];
  ending_state["task_id"]           = task_id;
  ending_state["equipment_id"]      = equipment_id;
  ending_state["comments"]          = comment;

  final_ending_state.push_back(ending_state);

  return ExecuteInsertStatement(generic_transaction, "equipment_in_use", ending_state, leave_alone);
}
//----------------------------------------------------------------------------------------------------------------------
bool TframeAssessmentDME::DeleteDMEItem(int record_index)
{
  pmapstrs ending_state;
  pmapstrtoint leave_alone;

  ending_state["id"]                = editing_dme_items[record_index]["id"];
  ending_state["current_id"]        = lookupTables->IdOf("current_types", "Deleted"); // Mark as Deleted
  ending_state["last_updated_by"]   = current_userid;
  ending_state["last_update_stamp"] = RightNow();

  final_ending_state.push_back(ending_state);

  return ExecuteUpdateStatement(generic_transaction, "equipment_in_use", ending_state, leave_alone);
}
//----------------------------------------------------------------------------------------------------------------------
// DoSaveData - Overridden to provide core saving logic.
bool TframeAssessmentDME::DoSaveData()
{
  psetstr checked_ids;
  psetstr unchecked_ids;
  int record_idx;

  final_ending_state.clear();

  // First determine what is checked and what is not in the filters.
  GetIDsFromCheckBoxFilter(checks_filter, checked_ids, unchecked_ids); // use parent class utility method

  // Look at all IDs:

  // * If the ID is checked, and the ID is in the editing map, do an update on that record.
  // * If the ID is checked, and the ID is NOT in the editing map, do an insert.
  for(psetstr::iterator i_checked = checked_ids.begin(); i_checked != checked_ids.end(); ++i_checked)
  {
    string comments;

    if(*i_checked == lookupTables->IdOf("equipment_types", "Other"))
      comments = editOther->Text.c_str();

    record_idx = IsInVecMap(editing_dme_items, "equipment_id", *i_checked);
    if(record_idx >= 0)
    {
      // Do Update because this equipment_in_use item already exists.
      if(!UpdateDMEItem(record_idx, comments))
        return false;
    }
    else
    {
      // Do Insert because this equipment_in_use item doesn't exist.
      if(!InsertDMEItem(*i_checked, comments))
        return false;
    }
  }

  // * If the ID is NOT checked, and the id is in the editing map, mark that item as Deleted via the current_id.
  // * If the ID is NOT checked, and the id is NOT in the editing map, do nothing at all.
  for(psetstr::iterator i_unchecked = unchecked_ids.begin(); i_unchecked != unchecked_ids.end(); ++i_unchecked)
  {
    record_idx = IsInVecMap(editing_dme_items, "equipment_id", *i_unchecked);
    if(record_idx >= 0)
    {
      // Do Update and set the existing equipment_in_use record to 'Deleted'.
      if(!DeleteDMEItem(record_idx))
        return false;
    }
    // Otherwise, we neither had a record to begin with, nor do we have one to insert now, so do nothing.
  }

  // If we get this far, nothing borked up above.
  return true;
}
//----------------------------------------------------------------------------------------------------------------------
// SetViewMode - Overridden to activate viewing of equipment_in_use information
void TframeAssessmentDME::SetViewMode()
{
  TframeAssessmentGeneric::SetViewMode();

  panelDME->Enabled = false;                    // But it is still not possible to enter data.
  editing_dme_items.clear();                    // Clear out the items vecmap for safety
  checks_filter.UncheckAll();
  editOther->Text = "";
}
//----------------------------------------------------------------------------------------------------------------------
// SetEditMode - Overridden to activate editing of equipment_in_use records
void TframeAssessmentDME::SetEditMode()
{
  TframeAssessmentGeneric::SetEditMode();

  panelDME->Enabled = true;               // Enable panel controls
}
//----------------------------------------------------------------------------------------------------------------------
// FilterAddDME - "Add" handler for the equipment_in_use filter. This actually just activates editing mode here.
void TframeAssessmentDME::FilterAddDME()
{
  if(btnUnlockSave->Enabled && frame_mode == mode_viewing)
    btnUnlockSaveClick(NULL);
}
//----------------------------------------------------------------------------------------------------------------------
// FilterEditDME - Same as above.
void TframeAssessmentDME::FilterEditDME()
{
  if(btnUnlockSave->Enabled && frame_mode == mode_viewing)
    btnUnlockSaveClick(NULL);
}
//----------------------------------------------------------------------------------------------------------------------
// AtlasNotify - Overridden to refresh equipment_in_use listview
bool TframeAssessmentDME::AtlasNotify(int request_num, string matched_value)
{
  bool handled = false;

  if(request_num == dme_request)
  {
    dme_filter.RefreshDataSet();
    handled = true;
  }

  return handled;
}
//----------------------------------------------------------------------------------------------------------------------
// SendAtlas - Overridden to send out notifications for equipment_in_use
void TframeAssessmentDME::SendAtlas()
{
  atlas->Notify("equipment_in_use.task_id", task_id); // Is this good enough?
}
//----------------------------------------------------------------------------------------------------------------------
// WriteReport - Write out a report on the equipment items
bool TframeAssessmentDME::WriteReport(TComponent *compositeReportCtrl)
{
  bool success = false;

  try
  {
    string sql =
      "select EQT.item, EQ.comments "
      "from equipment_in_use EQ "
      "left join equipment_types EQT on EQ.equipment_id = EQT.id "
      "where EQ.current_id >= 3 and EQ.task_id = '" + task_id + "' ";

    InitDataSet(sql);

    TformReportAssessment *reportForm = report_data.reportForm;
    TQuickRep *thisReport = reportForm->reportDME;

    thisReport->DataSet = report_data.dbDataSet;
    SetupQRDBText(reportForm->txtDMEItem,   "item");
    SetupQRDBText(reportForm->txtDMEDetail, "comments");

    // NB: Need to pass it as TComponent to avoid problems with having QuickReport included in the headers
    TQRCompositeReport *compositeReport = dynamic_cast<TQRCompositeReport *>(compositeReportCtrl);
    if(compositeReport)
    {
      compositeReport->Reports->Add(thisReport);
      success = true;
    }
  }
  catch(...)
  {
  }

  return success;
}
//----------------------------------------------------------------------------------------------------------------------
void __fastcall TframeAssessmentDME::listDMEChecksClick(TObject *Sender)
{
  // jhaley 20110629 - abstracted shared checkboxes logic into AssessmentGeneric class.
  FilterCheckBoxHitTest(listDMEChecks);
}
//----------------------------------------------------------------------------------------------------------------------

