//----------------------------------------------------------------------------------------------------------------------

#include <vcl.h>
#pragma hdrstop

#include "codeFrameAssessmentDiet.h"
#include "codeRecordPatientAssessment.h"
//----------------------------------------------------------------------------------------------------------------------
#pragma package(smart_init)
#pragma link "codeFrameAssessmentGeneric"
#pragma resource "*.dfm"
TframeAssessmentDiet *frameAssessmentDiet;
//----------------------------------------------------------------------------------------------------------------------
__fastcall TframeAssessmentDiet::TframeAssessmentDiet(TComponent* Owner)
        : TframeAssessmentGeneric(Owner), diet_filter(), checks_filter(), diet_request(-1), editing_diet_items(),
          final_ending_state()
{
}
//----------------------------------------------------------------------------------------------------------------------
// Initialize - Overridden to set parent properties and setup diet filtered listview.
bool TframeAssessmentDiet::Initialize()
{
  // Set AssessmentGeneric properties
  SetBandCaption("Nutrition/Diet");
  SetPermission("diet_items");

  // Setup notes filter
  diet_filter.SetListViewHandle(listDiet);
  diet_filter.SetSecurity("diet_items");

  diet_filter.AddColumnSpec("Item",        "diet_items",  as_text, true,  160, rawString);
  diet_filter.AddColumnSpec("Details",     "diet_items",  as_text, true,  420, rawString);
  diet_filter.AddColumnSpec("Current Id",  "current_id",  as_int,  false,  80);
  diet_filter.AddColumnSpec("Internal Id", "internal_id", as_int,  false,  80);

  diet_filter.FetchPreferences("assessment.diet.topfilter");
  diet_filter.SetSQLStatement("");
  diet_filter.RefreshDataSet();

  // setup diet_item_types input filter

  string items_sql =
    "select item, id from diet_item_types where valid = 1 and id > 0 and "
    "id <> " + lookupTables->IdOf("diet_item_types", "Regular Diet"); // don't include the NPI indicator

  // Checks Filter
  checks_filter.SetListViewHandle(listDietTypes);
  checks_filter.SetSecurity("");
  checks_filter.AddColumnSpec("Item",        "diet_items",  as_text, true,  168, rawString);
  checks_filter.AddColumnSpec("Internal Id", "internal_id", as_int,  false,  80);
  checks_filter.FetchPreferences("assessment.diet.diet_item_types");
  checks_filter.SetIconStatus(false);
  checks_filter.SetCheck(true);
  checks_filter.SetSQLStatement(items_sql);
  checks_filter.RefreshDataSet();

  // Set edit box change handlers
  SetAllEventHandlers(panelDiet);

  return TframeAssessmentGeneric::Initialize();
}
//----------------------------------------------------------------------------------------------------------------------
// Lock - Overridden to lock Diet panel
void TframeAssessmentDiet::Lock()
{
  TframeAssessmentGeneric::Lock(); // lock parent controls
  panelDiet->Enabled = false;      // lock all panel controls
}
//----------------------------------------------------------------------------------------------------------------------
// PutUpDietItems - Refreshes the controls to reflect the state of the diet items list
void TframeAssessmentDiet::PutUpDietItems(pvecmap &diet_items)
{
  psetstr diet_ids;

  // Clear out the checkboxes and edit box
  checks_filter.UncheckAll();
  memoComments->Text = "";
  checkNPI->Checked = false;

  // Analyze the retrieved diet items
  for(size_t i = 0; i < diet_items.size(); i++)
  {
    // Skip items which are anything other than Current. This allows Deleted items to be updated back to Current status
    // when saving.
    if(diet_items[i]["current_id"] != lookupTables->IdOf("current_types", "Current"))
      continue;

    if(diet_items[i]["diet_item_id"] == lookupTables->IdOf("diet_item_types", "Regular Diet")) // NPI
      checkNPI->Checked = true;
    else
    {
      diet_ids.insert(diet_items[i]["diet_item_id"]);

      if(diet_items[i]["diet_item_id"] == lookupTables->IdOf("diet_item_types", "Other"))
        memoComments->Text = diet_items[i]["comments"].c_str();
    }
  }

  // Set checkboxes
  checks_filter.SetCheckedWhereID(diet_ids, true);
}
//----------------------------------------------------------------------------------------------------------------------
// TaskIdChanging - Overridden to populate diet filter
void TframeAssessmentDiet::TaskIdChanging(const string &new_id)
{
  string sql;

  TframeAssessmentGeneric::TaskIdChanging(new_id);

  sql = "select DIT.item, DI.comments, DI.current_id, DI.id "
        "from diet_items DI "
        "left join diet_item_types DIT on DI.diet_item_id = DIT.id "
        "where DI.current_id >= 2 and DI.task_id = '" + new_id + "' ";

  diet_filter.SetSQLStatement(sql);
  diet_filter.RefreshDataSet();
  diet_filter.SetAddEdit(this, AddRecord (&TframeAssessmentDiet::FilterAddDiet),
                               EditRecord(&TframeAssessmentDiet::FilterEditDiet));
  diet_request = parent_form->RequestNotificationWhen("diet_items.task_id", new_id, rel_e, as_int);
}
//----------------------------------------------------------------------------------------------------------------------
// UpdateForEdit - Overridden to put fresh data into the editing_diet_items vecmap and on screen.
void TframeAssessmentDiet::UpdateForEdit()
{
  string sql =
    "select * "
    "from diet_items DI "
    "where DI.task_id = '" + task_id + "' ";

  editing_diet_items.clear();

  if(!SqlToVecMap(generic_transaction, sql, editing_diet_items))
    parent_form->PSay("Could not retrieve up-to-date information for diet_items", m_warning, MB_OK);

  // Refresh controls
  PutUpDietItems(editing_diet_items);

  TframeAssessmentGeneric::UpdateForEdit();
}
//----------------------------------------------------------------------------------------------------------------------
// LockRecords - Overridden to lock the diet_items records
bool TframeAssessmentDiet::LockRecords()
{
  // Must acquire all items. Any failure is a failure to unlock for editing, GTFO and rollback.
  if(!LockVecMapRecords(editing_diet_items, "diet_items"))
    return false;

  return TframeAssessmentGeneric::LockRecords();
}
//----------------------------------------------------------------------------------------------------------------------
// IsValidOnScreen - Overridden to do validation for diet_items
bool TframeAssessmentDiet::IsValidOnScreen()
{
  pvecstr checked_ids = checks_filter.GetMultiID();
  bool other_is_checked = false;
  
  // Make sure at least one diet item is specified
  if(!checkNPI->Checked &&
     editing_diet_items.size() == 0 && // Only when the record is new, otherwise can't delete last one
     checked_ids.size() == 0)
  {
    parent_form->PSay("You have not specified any saveable data. Please check one or more dietary status items or the "
                      "'Regular Diet' indicator.", m_missing_data, MB_OK);
    return false;
  }

  // If NPI is checked, nothing except Other can be specified in addition to it.
  if(checkNPI->Checked && checked_ids.size() > 0)
  {
    for(size_t i = 0; i < checked_ids.size(); i++)
    {
      if(checked_ids[i] != lookupTables->IdOf("diet_item_types", "Other"))
      {
        parent_form->PSay("You indicated that the patient is on a 'Regular Diet', but also checked additional statuses "
                          "(not including 'Other', which you can use to provide relevant information even if there "
                          "were no dietary status issues).\n\n"
                          "Please uncheck either the 'Regular Diet' indicator, or all items in the Dietary "
                          "Status list except 'Other'.", m_invalid_data, MB_OK);
        return false;
      }
    }
  }

  for(size_t i = 0; i < checked_ids.size(); i++)
  {
    // Still have to hard code it >_>
    if(checked_ids[i] == lookupTables->IdOf("diet_item_types", "Other"))
    {
      other_is_checked = true;
      if(memoComments->Text == "")
      {
        parent_form->PSay("You selected 'Other' but did not provide any additional information on the dietary status.",
                          m_missing_data, MB_OK);
        return false;
      }
    }
  }

  if(!other_is_checked && memoComments->Text != "")
  {
    parent_form->PSay("You did not select 'Other', but specified information in the 'Other Information' field. This "
                      "information would be lost, so please either clear it out now, or select the 'Other' item in "
                      "the checkbox list as well.", m_invalid_data, MB_OK);
    return false;
  }

  return TframeAssessmentGeneric::IsValidOnScreen();
}
//----------------------------------------------------------------------------------------------------------------------
bool TframeAssessmentDiet::UpdateDietItem(int record_index, string comment)
{
  pmapstrs ending_state;
  pmapstrtoint leave_alone;

  ending_state["id"]                 = editing_diet_items[record_index]["id"];
  ending_state["current_id"]         = lookupTables->IdOf("current_types", "Current");
  ending_state["last_updated_by"]    = current_userid;
  ending_state["last_update_stamp"]  = RightNow();
  ending_state["task_id"]            = task_id;
  ending_state["diet_item_id"]       = editing_diet_items[record_index]["diet_item_id"];
  ending_state["comments"]           = comment;

  leave_alone["comments"] = sql_no_uppercasing;

  final_ending_state.push_back(ending_state);

  return ExecuteUpdateStatement(generic_transaction, "diet_items", ending_state, leave_alone);
}
//----------------------------------------------------------------------------------------------------------------------
bool TframeAssessmentDiet::InsertDietItem(string diet_item_id, string comment)
{
  pmapstrs ending_state;
  pmapstrtoint leave_alone;

  ending_state["id"]                = GetNextId(generic_transaction, "diet_items");
  ending_state["current_id"]        = lookupTables->IdOf("current_types", "Current");
  ending_state["last_updated_by"]   = current_userid;
  ending_state["last_update_stamp"] = RightNow();
  ending_state["created_by"]        = current_userid;
  ending_state["creation_stamp"]    = ending_state["last_update_stamp"];
  ending_state["task_id"]           = task_id;
  ending_state["diet_item_id"]      = diet_item_id;
  ending_state["comments"]          = comment;

  leave_alone["comments"] = sql_no_uppercasing;

  final_ending_state.push_back(ending_state);

  return ExecuteInsertStatement(generic_transaction, "diet_items", ending_state, leave_alone);
}
//----------------------------------------------------------------------------------------------------------------------
bool TframeAssessmentDiet::DeleteDietItem(int record_index)
{
  pmapstrs ending_state;
  pmapstrtoint leave_alone;

  ending_state["id"]                = editing_diet_items[record_index]["id"];
  ending_state["current_id"]        = lookupTables->IdOf("current_types", "Deleted"); // Mark as Deleted
  ending_state["last_updated_by"]   = current_userid;
  ending_state["last_update_stamp"] = RightNow();

  final_ending_state.push_back(ending_state);

  return ExecuteUpdateStatement(generic_transaction, "diet_items", ending_state, leave_alone);
}
//----------------------------------------------------------------------------------------------------------------------
// DoSaveData - Overridden to provide core saving logic.
bool TframeAssessmentDiet::DoSaveData()
{
  psetstr checked_ids;
  psetstr unchecked_ids;
  int record_idx;
  string npiID = lookupTables->IdOf("diet_item_types", "Regular Diet");

  final_ending_state.clear();

  // First determine what is checked and what is not in the filters.
  GetIDsFromCheckBoxFilter(checks_filter, checked_ids, unchecked_ids); // use parent class utility method

  // Add in the NPI
  if(checkNPI->Checked)
    checked_ids.insert(npiID);
  else
    unchecked_ids.insert(npiID);

  // Look at all IDs:

  // * If the ID is checked, and the ID is in the editing map, do an update on that record.
  // * If the ID is checked, and the ID is NOT in the editing map, do an insert.
  for(psetstr::iterator i_checked = checked_ids.begin(); i_checked != checked_ids.end(); ++i_checked)
  {
    string comments;

    if(*i_checked == lookupTables->IdOf("diet_item_types", "Other"))
      comments = memoComments->Text.c_str();

    record_idx = IsInVecMap(editing_diet_items, "diet_item_id", *i_checked);
    if(record_idx >= 0)
    {
      // Do Update because this item already exists.
      if(!UpdateDietItem(record_idx, comments))
        return false;
    }
    else
    {
      // Do Insert because this item doesn't exist.
      if(!InsertDietItem(*i_checked, comments))
        return false;
    }
  }

  // * If the ID is NOT checked, and the id is in the editing map, mark that item as Deleted via the current_id.
  // * If the ID is NOT checked, and the id is NOT in the editing map, do nothing at all.
  for(psetstr::iterator i_unchecked = unchecked_ids.begin(); i_unchecked != unchecked_ids.end(); ++i_unchecked)
  {
    record_idx = IsInVecMap(editing_diet_items, "diet_item_id", *i_unchecked);
    if(record_idx >= 0)
    {
      // Do Update and set the existing record to 'Deleted'.
      if(!DeleteDietItem(record_idx))
        return false;
    }
    // Otherwise, we neither had a record to begin with, nor do we have one to insert now, so do nothing.
  }

  // If we get this far, nothing borked up above.
  return true;
}
//----------------------------------------------------------------------------------------------------------------------
// SetViewMode - Overridden to activate viewing of diet_items information
void TframeAssessmentDiet::SetViewMode()
{
  TframeAssessmentGeneric::SetViewMode();

  panelDiet->Enabled = false;                    // It is still not possible to enter data.
  editing_diet_items.clear();                    // Clear out the items vecmap for safety
  checks_filter.UncheckAll();
  memoComments->Text = "";
  checkNPI->Checked = false;

  ClearControlsModified(panelDiet);
}
//----------------------------------------------------------------------------------------------------------------------
// SetEditMode - Overridden to activate editing of diet_items records
void TframeAssessmentDiet::SetEditMode()
{
  TframeAssessmentGeneric::SetEditMode();

  panelDiet->Enabled = true;               // Enable panel controls

  ClearControlsModified(panelDiet);
}
//----------------------------------------------------------------------------------------------------------------------
// FilterAddDiet - "Add" handler for the topfilter. This actually just activates editing mode here.
void TframeAssessmentDiet::FilterAddDiet()
{
  if(btnUnlockSave->Enabled && frame_mode == mode_viewing)
    btnUnlockSaveClick(NULL);
}
//----------------------------------------------------------------------------------------------------------------------
// FilterEditDiet - Same as above.
void TframeAssessmentDiet::FilterEditDiet()
{
  if(btnUnlockSave->Enabled && frame_mode == mode_viewing)
    btnUnlockSaveClick(NULL);
}
//----------------------------------------------------------------------------------------------------------------------
// AtlasNotify - Overridden to refresh diet_items listview
bool TframeAssessmentDiet::AtlasNotify(int request_num, string matched_value)
{
  bool handled = false;

  if(request_num == diet_request)
  {
    diet_filter.RefreshDataSet();
    handled = true;
  }

  return handled;
}
//----------------------------------------------------------------------------------------------------------------------
// SendAtlas - Overridden to send out notifications for diet_items
void TframeAssessmentDiet::SendAtlas()
{
  atlas->Notify("diet_items.task_id", task_id); // Is this good enough?
}
//----------------------------------------------------------------------------------------------------------------------
// WriteReport - Write out a report on the nutrition items
bool TframeAssessmentDiet::WriteReport(TComponent *compositeReportCtrl)
{
  bool success = false;

  try
  {
    string sql =
      "select DIT.item, DI.comments "
      "from diet_items DI "
      "left join diet_item_types DIT on DI.diet_item_id = DIT.id "
      "where DI.current_id >= 3 and DI.task_id = '" + task_id + "' ";

    InitDataSet(sql);

    TformReportAssessment *reportForm = report_data.reportForm;
    TQuickRep *thisReport = reportForm->reportDiet;

    thisReport->DataSet = report_data.dbDataSet;
    SetupQRDBText(reportForm->txtDietItem,   "item");
    SetupQRDBText(reportForm->txtDietDetail, "comments");

    // NB: Need to pass it as TComponent to avoid problems with having QuickReport included in the headers
    TQRCompositeReport *compositeReport = dynamic_cast<TQRCompositeReport *>(compositeReportCtrl);
    if(compositeReport)
    {
      compositeReport->Reports->Add(thisReport);
      success = true;
    }
  }
  catch(...)
  {
  }

  return success;
}
//----------------------------------------------------------------------------------------------------------------------
void __fastcall TframeAssessmentDiet::listDietTypesClick(TObject *Sender)
{
  // Use parent class hit test logic
  FilterCheckBoxHitTest(listDietTypes);
}
//----------------------------------------------------------------------------------------------------------------------
void __fastcall TframeAssessmentDiet::checkNPIClick(TObject *Sender)
{
  if(frame_mode == mode_editing)
    SetModified(true);
}
//----------------------------------------------------------------------------------------------------------------------
