//----------------------------------------------------------------------------------------------------------------------

#include <vcl.h>
#pragma hdrstop

#include "codeFrameAssessmentEdu.h"
#include "codeRecordComHealth.h"
#include "codeRecordPatientAssessment.h"
//----------------------------------------------------------------------------------------------------------------------
#pragma package(smart_init)
#pragma link "codeFrameAssessmentGeneric"
#pragma resource "*.dfm"
TframeAssessmentEdu *frameAssessmentEdu;
//----------------------------------------------------------------------------------------------------------------------
__fastcall TframeAssessmentEdu::TframeAssessmentEdu(TComponent* Owner)
        : TframeAssessmentGeneric(Owner), edu_filter(), editing_edu(), ending_state(), edu_request(-1), editing_id()
{
}
//----------------------------------------------------------------------------------------------------------------------
// Initialize - Overridden to set parent properties and setup education filtered listview.
bool TframeAssessmentEdu::Initialize()
{
  // Set AssessmentGeneric properties
  SetBandCaption("Education");
  SetPermission("com_health_edu");

  // Setup notes filter
  edu_filter.SetListViewHandle(listEdu);
  edu_filter.SetSecurity("com_health_edu");
  edu_filter.AddColumnSpec("Topic",       "com_health_edu", as_text, true,  250);
  edu_filter.AddColumnSpec("Learner",     "com_health_edu", as_text, true,  100);
  edu_filter.AddColumnSpec("Method",      "com_health_edu", as_text, true,  100);
  edu_filter.AddColumnSpec("Evaluation",  "com_health_edu", as_text, true,  150);
  edu_filter.AddColumnSpec("Current Id",  "current_id",     as_int,  false, 80);
  edu_filter.AddColumnSpec("Internal Id", "internal_id",    as_int,  false, 80);
  edu_filter.FetchPreferences("assessment.edu.filter");
  edu_filter.SetSQLStatement("");
  edu_filter.RefreshDataSet();

  // Register lookups
  string lookupName = "edu.cboTopic";
  string tableName  = "com_adult_topic_types";
  RegisterLookup(lookupName, tableName, cboTopic);

  lookupName = "edu.cboLearner";
  tableName  = "com_learner_types";
  RegisterLookup(lookupName, tableName, cboLearner);

  lookupName = "edu.cboMethod";
  tableName  = "com_method_types";
  RegisterLookup(lookupName, tableName, cboMethod);

  lookupName = "edu.cboEvaluation";
  tableName  = "com_evaluation_types";
  RegisterLookup(lookupName, tableName, cboEvaluation);

  // Set standard modified event handlers for all controls on panelMed
  SetAllEventHandlers(panelEdu);

  return TframeAssessmentGeneric::Initialize();
}
//----------------------------------------------------------------------------------------------------------------------
// Lock - Overridden to lock Add button and edu panel
void TframeAssessmentEdu::Lock()
{
  TframeAssessmentGeneric::Lock(); // lock parent controls
  btnAdd->Enabled = false;         // disable add button
  btnReferral->Enabled = false;    // can't add referrals yet
  panelEdu->Enabled = false;       // lock all controls
}
//----------------------------------------------------------------------------------------------------------------------
// TaskIdChanging - Overridden to populate edu filter
void TframeAssessmentEdu::TaskIdChanging(const string &new_id)
{
  string sql;

  TframeAssessmentGeneric::TaskIdChanging(new_id);

  sql = "select CATT.item, CLT.item, CMT.item, CET.item, CHE.current_id, CHE.id "
        "from com_health_edu CHE "
        "left join com_adult_topic_types CATT on CHE.com_adult_topic_id = CATT.id "
        "left join com_learner_types     CLT  on CHE.com_learner_id     = CLT.id "
        "left join com_method_types      CMT  on CHE.com_method_id      = CMT.id "
        "left join com_evaluation_types  CET  on CHE.com_evaluation_id  = CET.id "
        "where CHE.current_id >= 2 and CHE.task_id = '" + new_id + "' ";  // NB: All can be seen @ person record

  edu_filter.SetSQLStatement(sql);
  edu_filter.RefreshDataSet();
  edu_filter.SetAddEdit(this, AddRecord (&TframeAssessmentEdu::FilterAddEdu),
                              EditRecord(&TframeAssessmentEdu::FilterEditEdu));
  edu_request = parent_form->RequestNotificationWhen("com_health_edu.task_id", new_id, rel_any, as_int);
}
//----------------------------------------------------------------------------------------------------------------------
// LockRecords - Overridden to lock com_health_edu record for editing.
bool TframeAssessmentEdu::LockRecords()
{
  if(!LockRecordForUpdate(generic_transaction, "com_health_edu", editing_id))
    return false;

  return TframeAssessmentGeneric::LockRecords();
}
//----------------------------------------------------------------------------------------------------------------------
// CanUnlock - Overridden to test for a selected education
bool TframeAssessmentEdu::CanUnlock(bool &creating_new)
{
  editing_id = edu_filter.GetSelectedRecordID();

  if(!creating_new && editing_id == "")
  {
    parent_form->PSay("Please select an education item to edit in the list first.", m_reminder, MB_OK);
    return false;
  }

  return TframeAssessmentGeneric::CanUnlock(creating_new);
}
//----------------------------------------------------------------------------------------------------------------------
// UpdateForEdit - Overridden to put fresh data into the editing_edu map and on screen.
void TframeAssessmentEdu::UpdateForEdit()
{
  string sql = "select * from com_health_edu where id = '" + editing_id + "' ";

  if(!SqlToMap(generic_transaction, sql, editing_edu))
    parent_form->PSay("Could not retrieve up-to-date information for education item", m_warning, MB_OK);

  string topic_type   = PrettyString(lookupTables->ValueOf("com_adult_topic_types", editing_edu["com_adult_topic_id"]));
  string learner_type = PrettyString(lookupTables->ValueOf("com_learner_types",     editing_edu["com_learner_id"]));
  string method_type  = PrettyString(lookupTables->ValueOf("com_method_types",      editing_edu["com_method_id"]));
  string eval_type    = PrettyString(lookupTables->ValueOf("com_evaluation_types",  editing_edu["com_evaluation_id"]));

  cboTopic->Text      = topic_type.c_str();
  cboLearner->Text    = learner_type.c_str();
  cboMethod->Text     = method_type.c_str();
  cboEvaluation->Text = eval_type.c_str();

  // No edits on this panel, so no need for clearing anything here.

  TframeAssessmentGeneric::UpdateForEdit();
}
//----------------------------------------------------------------------------------------------------------------------
// SavingNew - Overridden to return status of editing_edu as new record or unlocked record
bool TframeAssessmentEdu::SavingNew()
{
  return editing_edu["id"] == "new";
}
//----------------------------------------------------------------------------------------------------------------------
// IsValidOnScreen - Overridden to validate lookups
bool TframeAssessmentEdu::IsValidOnScreen()
{
  if(cboTopic->Text == "" || !lookupTables->IsInLookUpTable("com_adult_topic_types", cboTopic))
  {
    parent_form->PSay("Please enter a valid topic", m_invalid_data, MB_OK);
    return false;
  }
  if(cboLearner->Text == "" || !lookupTables->IsInLookUpTable("com_learner_types", cboLearner))
  {
    parent_form->PSay("Please enter a valid learner", m_invalid_data, MB_OK);
    return false;
  }
  if(cboMethod->Text == "" || !lookupTables->IsInLookUpTable("com_method_types", cboMethod))
  {
    parent_form->PSay("Please enter a valid method", m_invalid_data, MB_OK);
    return false;
  }
  if(cboEvaluation->Text == "" || !lookupTables->IsInLookUpTable("com_evaluation_types", cboEvaluation))
  {
    parent_form->PSay("Please enter a valid evaluation", m_invalid_data, MB_OK);
    return false;
  }

  return TframeAssessmentGeneric::IsValidOnScreen();
}
//----------------------------------------------------------------------------------------------------------------------
// DoSaveData - Overridden to provide core saving logic.
bool TframeAssessmentEdu::DoSaveData()
{
  pmapstrtoint leave_alone;
  ending_state = editing_edu;

  ending_state["current_id"]        = lookupTables->IdOf("current_types", "Current");
  ending_state["last_update_stamp"] = RightNow();
  ending_state["last_updated_by"]   = current_userid;

  ending_state["com_adult_topic_id"] = lookupTables->IdOf("com_adult_topic_types", cboTopic);
  ending_state["com_learner_id"]     = lookupTables->IdOf("com_learner_types",     cboLearner);
  ending_state["com_method_id"]      = lookupTables->IdOf("com_method_types",      cboMethod);
  ending_state["com_evaluation_id"]  = lookupTables->IdOf("com_evaluation_types",  cboEvaluation);

  // Not seen:
  // group_id  - this either comes from the existing edu or is set in editing_edu on add
  // task_id   - this either comes from the existing edu or is set in editing_edu on add

  return DoInsertOrUpdate(ending_state, leave_alone, "com_health_edu");
}
//----------------------------------------------------------------------------------------------------------------------
// SetViewMode - Overridden to activate viewing of medications
void TframeAssessmentEdu::SetViewMode()
{
  TframeAssessmentGeneric::SetViewMode();
  btnAdd->Enabled = HaveUnlockPermission(true); // It is now possible to add a medication
  btnReferral->Enabled = true;                  // Create referral button enabled
  panelEdu->Enabled = false;                    // But it is still not possible to enter data.
  editing_edu.clear();                          // Clear out the edu map for safety

  // Reset control contents
  ClearControlsText(panelEdu);
  ClearControlsModified(panelEdu);
}
//----------------------------------------------------------------------------------------------------------------------
// SetEditMode - Overridden to activate editing of a medication
void TframeAssessmentEdu::SetEditMode()
{
  TframeAssessmentGeneric::SetEditMode();
  btnAdd->Enabled = false;                // Can't add education items right now
  panelEdu->Enabled = true;               // Enable panel controls
}
//----------------------------------------------------------------------------------------------------------------------
// btnAddClick - Handler for toolbar button to add a new education item
void __fastcall TframeAssessmentEdu::btnAddClick(TObject *Sender)
{
  if(DoUnlock(true))
  {
    // Reset control contents
    ClearControlsText(panelEdu);
    ClearControlsModified(panelEdu);

    editing_edu.clear();
    editing_edu["id"] = "new"; // Creating a new education item
    editing_edu["group_id"] = group_id;  // Propagate parent assessment form's tasks.id and tasks.group_id
    editing_edu["task_id"]  = task_id;
  }
}
//----------------------------------------------------------------------------------------------------------------------
// FilterAddEdu - "Add" handler for the edu filter. We redirect this to the "Add an Item" code so that it
// works compatibly with the assessment frame.
void TframeAssessmentEdu::FilterAddEdu()
{
  if(btnAdd->Enabled) // Only when you could add using the button.
    btnAddClick(NULL);
}
//----------------------------------------------------------------------------------------------------------------------
// FilterEditEdu - Open the med in a com_health_edu record form for ordinary editing capability. Not encouraged,
// but allows adjustment of current_id's etc, which is important if a data entry mistake is made.
void TframeAssessmentEdu::FilterEditEdu()
{
  // Open the com_drug_regimen record
  pmapstrs field_map;
  field_map["id"] = edu_filter.GetSelectedRecordID();

  FORM_TRY
    TformRecordComHealth *edu_form = new TformRecordComHealth(NULL, field_map);
    edu_form->Show();
  FORM_CATCH
}
//----------------------------------------------------------------------------------------------------------------------
// AtlasNotify - Overridden to refresh edu listview
bool TframeAssessmentEdu::AtlasNotify(int request_num, string matched_value)
{
  bool handled = false;

  if(request_num == edu_request)
  {
    edu_filter.RefreshDataSet();
    handled = true;
  }

  return handled;
}
//----------------------------------------------------------------------------------------------------------------------
// SendAtlas - Overridden to send out notifications for com_health_edu
void TframeAssessmentEdu::SendAtlas()
{
  bool new_record = (editing_edu["id"] == "new");

  SendAtlasIfChanged(new_record, "com_health_edu", editing_edu, ending_state);
}
//----------------------------------------------------------------------------------------------------------------------
// WriteReport - Write out a report on the education items
bool TframeAssessmentEdu::WriteReport(TComponent *compositeReportCtrl)
{
  bool success = false;

  try
  {
    string sql =
      "select CATT.item topic, CLT.item learner, CMT.item method, CET.item eval "
      "from com_health_edu CHE "
      "left join com_adult_topic_types CATT on CHE.com_adult_topic_id = CATT.id "
      "left join com_learner_types     CLT  on CHE.com_learner_id     = CLT.id "
      "left join com_method_types      CMT  on CHE.com_method_id      = CMT.id "
      "left join com_evaluation_types  CET  on CHE.com_evaluation_id  = CET.id "
      "where CHE.current_id >= 3 and CHE.task_id = '" + task_id + "' ";

    InitDataSet(sql);

    TformReportAssessment *reportForm = report_data.reportForm;
    TQuickRep *thisReport = reportForm->reportEdu;

    thisReport->DataSet = report_data.dbDataSet;
    SetupQRDBText(reportForm->txtEduTopic,   "topic");
    SetupQRDBText(reportForm->txtEduLearner, "learner");
    SetupQRDBText(reportForm->txtEduMethod,  "method");
    SetupQRDBText(reportForm->txtEduEval,    "eval");

    // NB: Need to pass it as TComponent to avoid problems with having QuickReport included in the headers
    TQRCompositeReport *compositeReport = dynamic_cast<TQRCompositeReport *>(compositeReportCtrl);
    if(compositeReport)
    {
      compositeReport->Reports->Add(thisReport);
      success = true;
    }
  }
  catch(...)
  {
  }

  return success;
}
//----------------------------------------------------------------------------------------------------------------------
// btnReferralClick - Change the assessment form to the referrals area
void __fastcall TframeAssessmentEdu::btnReferralClick(TObject *Sender)
{
  parent_form->ChangeToArea(TformRecordPatientAssessment::area_referrals);
}
//----------------------------------------------------------------------------------------------------------------------

