//----------------------------------------------------------------------------------------------------------------------

#include <vcl.h>
#pragma hdrstop

#include "codeFrameAssessmentFallRisks.h"
#include "codeRecordPatientAssessment.h"
//----------------------------------------------------------------------------------------------------------------------
#pragma package(smart_init)
#pragma link "codeFrameAssessmentGeneric"
#pragma resource "*.dfm"
TframeAssessmentFallRisks *frameAssessmentFallRisks;
//----------------------------------------------------------------------------------------------------------------------
__fastcall TframeAssessmentFallRisks::TframeAssessmentFallRisks(TComponent* Owner)
        : TframeAssessmentGeneric(Owner), morse_fall_data(), editing_risk(), ending_state()
{
  editing_id = "";
}
//----------------------------------------------------------------------------------------------------------------------
// GetMorseAnswerSets - Private subroutine for Initialize; retrieves a set of Morse question answers and associates it
// with the corresponding dropbox
void TframeAssessmentFallRisks::GetMorseAnswerSets(string &question, TComboBox *cboBox)
{
  string question_id = lookupTables->IdOf("morse_questions", question); // Yes, Virginia, it is a lookup table.
  string lookup_sql =
    "select id, item, valid from morse_answer_types "
    "where valid = 1 and (id = 0 or morse_questions_id = '" + question_id + "') "
    "order by id ";
  string lookupName = "fall_risks.cboQuestion" + question_id;
  string tableName  = "morse_answer_types";

  RegisterLookup(lookupName, tableName, cboBox, lookup_sql);
}
//----------------------------------------------------------------------------------------------------------------------
// Initialize - Overridden to set parent properties and initialize controls
bool TframeAssessmentFallRisks::Initialize()
{
  // Set AssessmentGeneric properties
  SetBandCaption("Fall Risks");
  SetPermission("fall_risks");

  // Get captions for question labels
  string questions_sql = "select item from morse_questions where id <> 0 and valid = 1 order by id ";
  vector<string> question_strs;
  if(SqlToVec(dbDatabase, questions_sql, question_strs) && question_strs.size() >= 6)
  {
    lblQuestion1->Caption = question_strs[0].c_str();
    lblQuestion2->Caption = question_strs[1].c_str();
    lblQuestion3->Caption = question_strs[2].c_str();
    lblQuestion4->Caption = question_strs[3].c_str();
    lblQuestion5->Caption = question_strs[4].c_str();
    lblQuestion6->Caption = question_strs[5].c_str();

    GetMorseAnswerSets(question_strs[0], cboQuestion1);
    GetMorseAnswerSets(question_strs[1], cboQuestion2);
    GetMorseAnswerSets(question_strs[2], cboQuestion3);
    GetMorseAnswerSets(question_strs[3], cboQuestion4);
    GetMorseAnswerSets(question_strs[4], cboQuestion5);
    GetMorseAnswerSets(question_strs[5], cboQuestion6);
  }
  else
    parent_form->PSay("Could not retrieve Morse questions", m_warning, MB_OK); // Rut-row!

  return TframeAssessmentGeneric::Initialize();
}
//----------------------------------------------------------------------------------------------------------------------
// Lock - Overridden to lock fall risk assessment panel
void TframeAssessmentFallRisks::Lock()
{
  TframeAssessmentGeneric::Lock(); // lock parent controls
  panelFallRisks->Enabled = false; // lock all panel controls
}
//----------------------------------------------------------------------------------------------------------------------
// CalculateRiskPoints - Private method, calculates the total risk points from the answers selected. This is a local
// calculation; the score is not stored in the database as it could get out of sync with the answers (don't store the
// same information in two places, etc.)
void TframeAssessmentFallRisks::CalculateRiskPoints()
{
  vector<string> answer_ids;

  answer_ids.push_back(lookupTables->IdOf("morse_answer_types", cboQuestion1));
  answer_ids.push_back(lookupTables->IdOf("morse_answer_types", cboQuestion2));
  answer_ids.push_back(lookupTables->IdOf("morse_answer_types", cboQuestion3));
  answer_ids.push_back(lookupTables->IdOf("morse_answer_types", cboQuestion4));
  answer_ids.push_back(lookupTables->IdOf("morse_answer_types", cboQuestion5));
  answer_ids.push_back(lookupTables->IdOf("morse_answer_types", cboQuestion6));

  string total_risk_sql =
    "select sum(MAT.points) from morse_answer_types MAT where " + SafeInClause("MAT.id", answer_ids) + " ";
  string result = GetOneField(dbDatabase, total_risk_sql);
  editRiskPoints->Text = result.c_str();

  string assessment_sql =
    "select (item || ' - ' || action_text) from morse_assessments "
    "where id <> 0 and '" + result + "' >= min_score and '" + result + "' <= max_score ";
  editMorseAssessment->Text = GetOneField(dbDatabase, assessment_sql).c_str();
}
//----------------------------------------------------------------------------------------------------------------------
// PopulateComboBox - Private method; set one of the question combo boxes
void TframeAssessmentFallRisks::PopulateComboBox(TComboBox *cboBox, pmapstrs &datamap, const char *id)
{
  string id_str = id;
  if(!IsBlankOrZero(datamap[id_str]))
    cboBox->Text = lookupTables->ValueOf("morse_answer_types", datamap[id_str]).c_str();
  else
    cboBox->Text = "";
}
//----------------------------------------------------------------------------------------------------------------------
// DoFullUpdate - Private method; updates everything on the form using data from a map
void TframeAssessmentFallRisks::DoFullUpdate(pmapstrs &datamap)
{
  PopulateComboBox(cboQuestion1, datamap, "morse_hx_id");
  PopulateComboBox(cboQuestion2, datamap, "morse_dx_id");
  PopulateComboBox(cboQuestion3, datamap, "morse_amb_aid_id");
  PopulateComboBox(cboQuestion4, datamap, "morse_iv_id");
  PopulateComboBox(cboQuestion5, datamap, "morse_gait_id");
  PopulateComboBox(cboQuestion6, datamap, "morse_mental_id");
  CalculateRiskPoints();
}
//----------------------------------------------------------------------------------------------------------------------
// TaskIdChanging - Overridden to populate notes filter
void TframeAssessmentFallRisks::TaskIdChanging(const string &new_id)
{
  string sql;

  TframeAssessmentGeneric::TaskIdChanging(new_id);

  morse_fall_data.clear();
  // See if there is an existing current fall_risks record for the attached visit
  sql = "select id, morse_hx_id, morse_dx_id, morse_amb_aid_id, morse_iv_id, morse_gait_id, morse_mental_id "
        "from fall_risks "
        "where current_id = '" + lookupTables->IdOf("current_types", "Current") + "' and task_id = '" + new_id + "' ";

  if(!SqlToMap(dbDatabase, sql, morse_fall_data))
    parent_form->PSay("Could not get existing Morse fall assessment data", m_warning, MB_OK);

  DoFullUpdate(morse_fall_data);
}
//----------------------------------------------------------------------------------------------------------------------
// LockRecords - Overridden to lock fall_risks record for editing.
bool TframeAssessmentFallRisks::LockRecords()
{
  if(!LockRecordForUpdate(generic_transaction, "fall_risks", editing_id))
    return false;

  return TframeAssessmentGeneric::LockRecords();
}
//----------------------------------------------------------------------------------------------------------------------
// CanUnlock - Overridden to determine if editing an existing fall risk or creating a new one
bool TframeAssessmentFallRisks::CanUnlock(bool &creating_new)
{
  editing_id = morse_fall_data["id"];

  if(IsBlankOrZero(editing_id))
  {
    creating_new = true; // set creating_new flag so that the parent class will prepare for an insert
    editing_id = "new";
    editing_risk.clear();
    editing_risk["id"] = "new";
  }

  return TframeAssessmentGeneric::CanUnlock(creating_new);
}
//----------------------------------------------------------------------------------------------------------------------
// UpdateForEdit - Overridden to put fresh data into the editing_note map and on screen.
void TframeAssessmentFallRisks::UpdateForEdit()
{
  string sql = "select * from fall_risks where id = '" + editing_id + "' ";

  if(!SqlToMap(generic_transaction, sql, editing_risk))
    parent_form->PSay("Could not retrieve up-to-date information for fall_risks", m_warning, MB_OK);

  DoFullUpdate(editing_risk);

  TframeAssessmentGeneric::UpdateForEdit();
}
//----------------------------------------------------------------------------------------------------------------------
// SavingNew - Overridden to return status of editing_note as new record or unlocked record
bool TframeAssessmentFallRisks::SavingNew()
{
  return editing_risk["id"] == "new";
}
//----------------------------------------------------------------------------------------------------------------------
// IsValidOnScreen - Overridden to ensure all combo boxes contain valid answers
bool TframeAssessmentFallRisks::IsValidOnScreen()
{
  if(!lookupTables->IsInLookUpTable("morse_answer_types", cboQuestion1))
  {
    parent_form->PSay("Please select a valid Hx answer", m_invalid_data, MB_OK);
    return false;
  }
  if(!lookupTables->IsInLookUpTable("morse_answer_types", cboQuestion2))
  {
    parent_form->PSay("Please select a valid Secondary Dx answer", m_invalid_data, MB_OK);
    return false;
  }
  if(!lookupTables->IsInLookUpTable("morse_answer_types", cboQuestion3))
  {
    parent_form->PSay("Please select a valid Ambulatory Aid answer", m_invalid_data, MB_OK);
    return false;
  }
  if(!lookupTables->IsInLookUpTable("morse_answer_types", cboQuestion4))
  {
    parent_form->PSay("Please select a valid IV answer", m_invalid_data, MB_OK);
    return false;
  }
  if(!lookupTables->IsInLookUpTable("morse_answer_types", cboQuestion5))
  {
    parent_form->PSay("Please select a valid Gait answer", m_invalid_data, MB_OK);
    return false;
  }
  if(!lookupTables->IsInLookUpTable("morse_answer_types", cboQuestion6))
  {
    parent_form->PSay("Please select a valid Mental answer", m_invalid_data, MB_OK);
    return false;
  }

  return TframeAssessmentGeneric::IsValidOnScreen();
}
//----------------------------------------------------------------------------------------------------------------------
// DoSaveData - Overridden to provide core saving logic.
bool TframeAssessmentFallRisks::DoSaveData()
{
  pmapstrtoint leave_alone;
  ending_state = editing_risk;

  ending_state["current_id"]        = lookupTables->IdOf("current_types", "Current");
  ending_state["last_updated_by"]   = current_userid;
  ending_state["last_update_stamp"] = RightNow();
  ending_state["task_id"]           = task_id;
  ending_state["morse_hx_id"]       = lookupTables->IdOf("morse_answer_types", cboQuestion1);
  ending_state["morse_dx_id"]       = lookupTables->IdOf("morse_answer_types", cboQuestion2);
  ending_state["morse_amb_aid_id"]  = lookupTables->IdOf("morse_answer_types", cboQuestion3);
  ending_state["morse_iv_id"]       = lookupTables->IdOf("morse_answer_types", cboQuestion4);
  ending_state["morse_gait_id"]     = lookupTables->IdOf("morse_answer_types", cboQuestion5);
  ending_state["morse_mental_id"]   = lookupTables->IdOf("morse_answer_types", cboQuestion6);

  // Note: slightly different than DoInsertOrUpdate of parent class, so not changed.
  if(ending_state["id"] == "new")
  {
    ending_state["id"] = GetNextId(generic_transaction, "fall_risks");
    if(!IsInt(ending_state["id"]))
    {
      parent_form->PSay("Could not get new id", m_database_error, MB_OK, "Insert");
      return false;
    }
    // Set created_by/creation_stamp also when creating a new record
    ending_state["created_by"]     = current_userid;
    ending_state["creation_stamp"] = ending_state["last_update_stamp"];

    if(ExecuteInsertStatement(generic_transaction, "fall_risks", ending_state, leave_alone))
    {
      morse_fall_data = ending_state; // remember saved data
      return true;
    }
  }
  else
  {
    if(ExecuteUpdateStatement(generic_transaction, "fall_risks", ending_state, leave_alone))
    {
      morse_fall_data = ending_state; // remember saved data
      return true;
    }
  }

  // Error
  parent_form->PSay("Could not save changes.", m_database_error, MB_OK);
  return false;
}
//----------------------------------------------------------------------------------------------------------------------
// SetViewMode - Overridden to activate viewing of task notes
void TframeAssessmentFallRisks::SetViewMode()
{
  TframeAssessmentGeneric::SetViewMode();
  panelFallRisks->Enabled = false;              // Disable all controls on the panel
  editing_risk.clear();                         // Clear out the risk map for safety
  DoFullUpdate(morse_fall_data);
}
//----------------------------------------------------------------------------------------------------------------------
// SetEditMode - Overridden to activate editing of a task note
void TframeAssessmentFallRisks::SetEditMode()
{
  TframeAssessmentGeneric::SetEditMode();
  panelFallRisks->Enabled = true;           // Enable all controls on the panel
}
//----------------------------------------------------------------------------------------------------------------------
// SendAtlas - Overridden to send out notifications for fall_risks
void TframeAssessmentFallRisks::SendAtlas()
{
  bool new_record = (editing_risk["id"] == "new");

  SendAtlasIfChanged(new_record, "fall_risks", editing_risk, ending_state);
}
//----------------------------------------------------------------------------------------------------------------------
// WriteReport - Write out a report on the Morse fall risks assessment
bool TframeAssessmentFallRisks::WriteReport(TComponent *compositeReportCtrl)
{
  bool success = false;

  try
  {
    string sql =
      "select "
      " MQ1.item morse_hx, MAT1.item hx_answer, "
      " MQ2.item morse_dx, MAT2.item dx_answer, "
      " MQ3.item morse_aa, MAT3.item aa_answer, "
      " MQ4.item morse_iv, MAT4.item iv_answer, "
      " MQ5.item morse_gt, MAT5.item gt_answer, "
      " MQ6.item morse_mn, MAT6.item mn_answer, "
      " (MAT1.points + MAT2.points + MAT3.points + MAT4.points + MAT5.points + MAT6.points) points, "
      " MA.item assmt "
      "from fall_risks FR "
      "inner join morse_answer_types MAT1 on MAT1.id = FR.morse_hx_id "
      "inner join morse_answer_types MAT2 on MAT2.id = FR.morse_dx_id "
      "inner join morse_answer_types MAT3 on MAT3.id = FR.morse_amb_aid_id "
      "inner join morse_answer_types MAT4 on MAT4.id = FR.morse_iv_id "
      "inner join morse_answer_types MAT5 on MAT5.id = FR.morse_gait_id "
      "inner join morse_answer_types MAT6 on MAT6.id = FR.morse_mental_id "
      "inner join morse_questions MQ1 on MQ1.id = '" + lookupTables->IdOf("morse_questions", "Hx of Falls; Immediate or Within 3 Mos") + "' "
      "inner join morse_questions MQ2 on MQ2.id = '" + lookupTables->IdOf("morse_questions", "Secondary Dx")      + "' "
      "inner join morse_questions MQ3 on MQ3.id = '" + lookupTables->IdOf("morse_questions", "Ambulatory Aid")    + "' "
      "inner join morse_questions MQ4 on MQ4.id = '" + lookupTables->IdOf("morse_questions", "IV/Heparin Lock")   + "' "
      "inner join morse_questions MQ5 on MQ5.id = '" + lookupTables->IdOf("morse_questions", "Gait/Transferring") + "' "
      "inner join morse_questions MQ6 on MQ6.id = '" + lookupTables->IdOf("morse_questions", "Mental Status")     + "' "
      "inner join morse_assessments MA on MA.id <> 0 and "
      " (MAT1.points + MAT2.points + MAT3.points + MAT4.points + MAT5.points + MAT6.points) "
      " between MA.min_score and MA.max_score "
      "where FR.current_id >= 3 and FR.task_id = '" + task_id + "' ";

    InitDataSet(sql);

    TformReportAssessment *reportForm = report_data.reportForm;
    TQuickRep *thisReport = reportForm->reportFallRisks;

    thisReport->DataSet = report_data.dbDataSet;

    // Questions
    SetupQRDBText(reportForm->txtMorseHx, "morse_hx");
    SetupQRDBText(reportForm->txtMorseDx, "morse_dx");
    SetupQRDBText(reportForm->txtMorseAA, "morse_aa");
    SetupQRDBText(reportForm->txtMorseIV, "morse_iv");
    SetupQRDBText(reportForm->txtMorseGT, "morse_gt");
    SetupQRDBText(reportForm->txtMorseMN, "morse_mn");
    // Answers
    SetupQRDBText(reportForm->txtMorseHxAns, "hx_answer");
    SetupQRDBText(reportForm->txtMorseDxAns, "dx_answer");
    SetupQRDBText(reportForm->txtMorseAAAns, "aa_answer");
    SetupQRDBText(reportForm->txtMorseIVAns, "iv_answer");
    SetupQRDBText(reportForm->txtMorseGTAns, "gt_answer");
    SetupQRDBText(reportForm->txtMorseMNAns, "mn_answer");
    // Assessment
    SetupQRDBText(reportForm->txtMorsePoints, "points");
    SetupQRDBText(reportForm->txtMorseAssmt,  "assmt");

    // NB: Need to pass it as TComponent to avoid problems with having QuickReport included in the headers
    TQRCompositeReport *compositeReport = dynamic_cast<TQRCompositeReport *>(compositeReportCtrl);
    if(compositeReport)
    {
      compositeReport->Reports->Add(thisReport);
      success = true;
    }
  }
  catch(...)
  {
  }

  return success;
}
//----------------------------------------------------------------------------------------------------------------------
// btnUpdateClick - Manually update the risk assessment total and recommended action
void __fastcall TframeAssessmentFallRisks::btnUpdateClick(TObject *Sender)
{
  CalculateRiskPoints();
}
//----------------------------------------------------------------------------------------------------------------------
// cboQuestion1Change - This is actually an OnChange handler for all of the combo boxes on this frame. Mark the record
// as modified as soon as any change is detected to the combo boxes. Note that VCL does not fire OnChange if the boxes
// are modified programmatically, which is exactly what we want here.
void __fastcall TframeAssessmentFallRisks::cboQuestion1Change(TObject *Sender)
{
  if(frame_mode == mode_editing)
    SetModified(true);
}
//----------------------------------------------------------------------------------------------------------------------

