//----------------------------------------------------------------------------------------------------------------------

#include <vcl.h>
#pragma hdrstop

#include "codeFrameAssessmentGastro.h"
#include "codeRecordPatientAssessment.h"
//----------------------------------------------------------------------------------------------------------------------
#pragma package(smart_init)
#pragma link "codeFrameAssessmentGeneric"
#pragma resource "*.dfm"
TframeAssessmentGastro *frameAssessmentGastro;
//----------------------------------------------------------------------------------------------------------------------
__fastcall TframeAssessmentGastro::TframeAssessmentGastro(TComponent* Owner)
        : TframeAssessmentGeneric(Owner), gastro_filter(), checks_filter(), gastro_request(-1), editing_gastro_items(),
          final_ending_state()
{
}
//----------------------------------------------------------------------------------------------------------------------
// Initialize - Overridden to set parent properties and setup gastro problems filtered listview.
bool TframeAssessmentGastro::Initialize()
{
  // Set AssessmentGeneric properties
  SetBandCaption("Gastrointestinal");
  SetPermission("gastro_items");

  // Setup notes filter
  gastro_filter.SetListViewHandle(listGastro);
  gastro_filter.SetSecurity("gastro_items");

  gastro_filter.AddColumnSpec("Item",        "gastro_items", as_text, true,  160, rawString);
  gastro_filter.AddColumnSpec("Details",     "gastro_items", as_text, true,  420, rawString);
  gastro_filter.AddColumnSpec("Current Id",  "current_id",   as_int,  false,  80);
  gastro_filter.AddColumnSpec("Internal Id", "internal_id",  as_int,  false,  80);

  gastro_filter.FetchPreferences("assessment.gastro.topfilter");
  gastro_filter.SetSQLStatement("");
  gastro_filter.RefreshDataSet();

  // setup safety_types input filter

  // don't include the NPI indicator
  string items_sql =
    "select item, id from gastro_types where valid = 1 and id > 0 and "
    "id <> " + lookupTables->IdOf("gastro_types", "Abdomen soft, active bowel sounds x4, no tenderness, non-distended, reg BM");

  // Checks Filter
  checks_filter.SetListViewHandle(listGastroTypes);
  checks_filter.SetSecurity("");
  checks_filter.AddColumnSpec("Item",        "gastro_items", as_text, true,  168, rawString);
  checks_filter.AddColumnSpec("Internal Id", "internal_id",  as_int,  false,  80);
  checks_filter.FetchPreferences("assessment.gastro.gastro_items");
  checks_filter.SetIconStatus(false);
  checks_filter.SetCheck(true);
  checks_filter.SetSQLStatement(items_sql);
  checks_filter.RefreshDataSet();

  // Set edit box change handlers
  SetAllEventHandlers(panelGastro);

  return TframeAssessmentGeneric::Initialize();
}
//----------------------------------------------------------------------------------------------------------------------
// Lock - Overridden to lock Gastro panel
void TframeAssessmentGastro::Lock()
{
  TframeAssessmentGeneric::Lock(); // lock parent controls
  panelGastro->Enabled = false;    // lock all panel controls
}
//----------------------------------------------------------------------------------------------------------------------
// PutUpGastroItems - Refreshes the controls to reflect the state of the gastro problems list
void TframeAssessmentGastro::PutUpGastroItems(pvecmap &gastro_items)
{
  psetstr gastro_ids;

  // Clear out the checkboxes and edit box
  checks_filter.UncheckAll();
  editDrainage->Text = "";
  editFormula->Text  = "";
  editOstomy->Text   = "";
  editResidual->Text = "";
  memoComments->Text = "";
  checkNPI->Checked = false;

  // Analyze the retrieved safety_problems items
  for(size_t i = 0; i < gastro_items.size(); i++)
  {
    // Skip items which are anything other than Current. This allows Deleted items to be updated back to Current status
    // when saving.
    if(gastro_items[i]["current_id"] != lookupTables->IdOf("current_types", "Current"))
      continue;

    if(gastro_items[i]["gastro_id"] == lookupTables->IdOf("gastro_types", "Abdomen soft, active bowel sounds x4, no tenderness, non-distended, reg BM")) // NPI
      checkNPI->Checked = true;
    else
    {
      gastro_ids.insert(gastro_items[i]["gastro_id"]);

      if(gastro_items[i]["gastro_id"] == lookupTables->IdOf("gastro_types", "Drainage"))
        editDrainage->Text = gastro_items[i]["comments"].c_str();
      else if(gastro_items[i]["gastro_id"] == lookupTables->IdOf("gastro_types", "Formula"))
        editFormula->Text  = gastro_items[i]["comments"].c_str();
      else if(gastro_items[i]["gastro_id"] == lookupTables->IdOf("gastro_types", "Ostomy"))
        editOstomy->Text   = gastro_items[i]["comments"].c_str();
      else if(gastro_items[i]["gastro_id"] == lookupTables->IdOf("gastro_types", "Residual"))
        editResidual->Text = gastro_items[i]["comments"].c_str();
      else if(gastro_items[i]["gastro_id"] == lookupTables->IdOf("gastro_types", "Other"))
        memoComments->Text = gastro_items[i]["comments"].c_str();
    }
  }

  // Set checkboxes
  checks_filter.SetCheckedWhereID(gastro_ids, true);
}
//----------------------------------------------------------------------------------------------------------------------
// TaskIdChanging - Overridden to populate gastro filter
void TframeAssessmentGastro::TaskIdChanging(const string &new_id)
{
  string sql;

  TframeAssessmentGeneric::TaskIdChanging(new_id);

  sql = "select GT.item, GI.comments, GI.current_id, GI.id "
        "from gastro_items GI "
        "left join gastro_types GT on GI.gastro_id = GT.id "
        "where GI.current_id >= 2 and GI.task_id = '" + new_id + "' ";

  gastro_filter.SetSQLStatement(sql);
  gastro_filter.RefreshDataSet();
  gastro_filter.SetAddEdit(this, AddRecord (&TframeAssessmentGastro::FilterAddGastro),
                                 EditRecord(&TframeAssessmentGastro::FilterEditGastro));
  gastro_request = parent_form->RequestNotificationWhen("gastro_items.task_id", new_id, rel_e, as_int);
}
//----------------------------------------------------------------------------------------------------------------------
// UpdateForEdit - Overridden to put fresh data into the editing_gastro_items vecmap and on screen.
void TframeAssessmentGastro::UpdateForEdit()
{
  string sql =
    "select GI.gastro_id, GI.comments, GI.current_id, GI.id "
    "from gastro_items GI "
    "where GI.task_id = '" + task_id + "' ";

  editing_gastro_items.clear();

  if(!SqlToVecMap(generic_transaction, sql, editing_gastro_items))
    parent_form->PSay("Could not retrieve up-to-date information for gastro items", m_warning, MB_OK);

  // Refresh controls
  PutUpGastroItems(editing_gastro_items);

  TframeAssessmentGeneric::UpdateForEdit();
}
//----------------------------------------------------------------------------------------------------------------------
// LockRecords - Overridden to lock the gastro_items records
bool TframeAssessmentGastro::LockRecords()
{
  // Must acquire all items. Any failure is a failure to unlock for editing, GTFO and rollback.
  if(!LockVecMapRecords(editing_gastro_items, "gastro_items"))
    return false;

  return TframeAssessmentGeneric::LockRecords();
}
//----------------------------------------------------------------------------------------------------------------------
// IsValidOnScreen - Overridden to do validation for gastro_items
bool TframeAssessmentGastro::IsValidOnScreen()
{
  pvecstr checked_ids = checks_filter.GetMultiID();
  bool drainage_checked = false;
  bool formula_checked  = false;
  bool ostomy_checked   = false;
  bool residual_checked = false;
  bool other_is_checked = false;
  
  // Make sure at least one gastro item is specified
  if(!checkNPI->Checked &&
     editing_gastro_items.size() == 0 && // Only when the record is new, otherwise can't delete last one
     checked_ids.size() == 0)
  {
    parent_form->PSay("You have not specified any saveable data. Please check one or more gastrointestinal items or the "
                      "'No Problems Identified' indicator.", m_missing_data, MB_OK);
    return false;
  }

  // If NPI is checked, nothing except Other can be specified in addition to it.
  if(checkNPI->Checked && checked_ids.size() > 0)
  {
    for(int i = 0; i < checked_ids.size(); i++)
    {
      if(checked_ids[i] != lookupTables->IdOf("gastro_types", "Other"))
      {
        parent_form->PSay("You indicated that there were 'No Problems Identified', but also checked additional problems "
                          "(not including 'Other', which you can use to provide relevant information even if there "
                          "were no tangible problems).\n\n"
                          "Please uncheck either the 'No Problems Identified' indicator, or all items in the Gastrointestinal "
                          "Items list except 'Other'.", m_invalid_data, MB_OK);
        return false;
      }
    }
  }

  for(int i = 0; i < checked_ids.size(); i++)
  {
    if(checked_ids[i] == lookupTables->IdOf("gastro_types", "Drainage"))
      drainage_checked = true;
    else if(checked_ids[i] == lookupTables->IdOf("gastro_types", "Formula"))
      formula_checked = true;
    else if(checked_ids[i] == lookupTables->IdOf("gastro_types", "Ostomy"))
      ostomy_checked = true;
    else if(checked_ids[i] == lookupTables->IdOf("gastro_types", "Residual"))
      residual_checked = true;
    else if(checked_ids[i] == lookupTables->IdOf("gastro_types", "Other"))
    {
      other_is_checked = true;
      if(memoComments->Text == "")
      {
        parent_form->PSay("You selected 'Other' but did not provide any text in the Comments field.",
                          m_missing_data, MB_OK);
        return false;
      }
    }
  }

  if(!drainage_checked && editDrainage->Text != "")
  {
    parent_form->PSay("You did not select 'Drainage', but provided a description. Please either clear the description "
                      "or select the 'Drainage' item in the checkbox list as well.", m_invalid_data, MB_OK);
    return false;
  }
  if(!formula_checked && editFormula->Text != "")
  {
    parent_form->PSay("You did not select 'Formula', but provided a description. Please either clear the description "
                      "or select the 'Formula' item in the checkbox list as well.", m_invalid_data, MB_OK);
    return false;
  }
  if(!ostomy_checked && editOstomy->Text != "")
  {
    parent_form->PSay("You did not select 'Ostomy', but provided a description. Please either clear the description "
                      "or select the 'Ostomy' item in the checkbox list as well.", m_invalid_data, MB_OK);
    return false;
  }
  if(!residual_checked && editResidual->Text != "")
  {
    parent_form->PSay("You did not select 'Residual', but provided a description. Please either clear the description "
                      "or select the 'Residual' item in the checkbox list as well.", m_invalid_data, MB_OK);
    return false;
  }
  if(!other_is_checked && memoComments->Text != "")
  {
    parent_form->PSay("You did not select 'Other', but specified information in the 'Comments' field. This "
                      "information would be lost, so please either clear it out now, or select the 'Other' item in "
                      "the checkbox list as well.", m_invalid_data, MB_OK);
    return false;
  }

  return TframeAssessmentGeneric::IsValidOnScreen();
}
//----------------------------------------------------------------------------------------------------------------------
bool TframeAssessmentGastro::UpdateGastroItem(int record_index, string comment)
{
  pmapstrs ending_state;
  pmapstrtoint leave_alone;

  ending_state["id"]                 = editing_gastro_items[record_index]["id"];
  ending_state["current_id"]         = lookupTables->IdOf("current_types", "Current");
  ending_state["last_updated_by"]    = current_userid;
  ending_state["last_update_stamp"]  = RightNow();
  ending_state["task_id"]            = task_id;
  ending_state["gastro_id"]          = editing_gastro_items[record_index]["gastro_id"];
  ending_state["comments"]           = comment;

  leave_alone["comments"] = sql_no_uppercasing;

  final_ending_state.push_back(ending_state);

  return ExecuteUpdateStatement(generic_transaction, "gastro_items", ending_state, leave_alone);
}
//----------------------------------------------------------------------------------------------------------------------
bool TframeAssessmentGastro::InsertGastroItem(string gastro_id, string comment)
{
  pmapstrs ending_state;
  pmapstrtoint leave_alone;

  ending_state["id"]                = GetNextId(generic_transaction, "gastro_items");
  ending_state["current_id"]        = lookupTables->IdOf("current_types", "Current");
  ending_state["last_updated_by"]   = current_userid;
  ending_state["last_update_stamp"] = RightNow();
  ending_state["created_by"]        = current_userid;
  ending_state["creation_stamp"]    = ending_state["last_update_stamp"];
  ending_state["task_id"]           = task_id;
  ending_state["gastro_id"]         = gastro_id;
  ending_state["comments"]          = comment;

  leave_alone["comments"] = sql_no_uppercasing;

  final_ending_state.push_back(ending_state);

  return ExecuteInsertStatement(generic_transaction, "gastro_items", ending_state, leave_alone);
}
//----------------------------------------------------------------------------------------------------------------------
bool TframeAssessmentGastro::DeleteGastroItem(int record_index)
{
  pmapstrs ending_state;
  pmapstrtoint leave_alone;

  ending_state["id"]                = editing_gastro_items[record_index]["id"];
  ending_state["current_id"]        = lookupTables->IdOf("current_types", "Deleted"); // Mark as Deleted
  ending_state["last_updated_by"]   = current_userid;
  ending_state["last_update_stamp"] = RightNow();

  final_ending_state.push_back(ending_state);

  return ExecuteUpdateStatement(generic_transaction, "gastro_items", ending_state, leave_alone);
}
//----------------------------------------------------------------------------------------------------------------------
// DoSaveData - Overridden to provide core saving logic.
bool TframeAssessmentGastro::DoSaveData()
{
  psetstr checked_ids;
  psetstr unchecked_ids;
  int record_idx;
  string npiID = lookupTables->IdOf("gastro_types", "Abdomen soft, active bowel sounds x4, no tenderness, non-distended, reg BM");

  final_ending_state.clear();

  // First determine what is checked and what is not in the filters.
  GetIDsFromCheckBoxFilter(checks_filter, checked_ids, unchecked_ids); // use parent class utility method

  // Add in the NPI
  if(checkNPI->Checked)
    checked_ids.insert(npiID);
  else
    unchecked_ids.insert(npiID);

  // Look at all IDs:

  // * If the ID is checked, and the ID is in the editing map, do an update on that record.
  // * If the ID is checked, and the ID is NOT in the editing map, do an insert.
  for(psetstr::iterator i_checked = checked_ids.begin(); i_checked != checked_ids.end(); ++i_checked)
  {
    string comments;

    if(*i_checked == lookupTables->IdOf("gastro_types", "Drainage"))
      comments = editDrainage->Text.c_str();
    else if(*i_checked == lookupTables->IdOf("gastro_types", "Formula"))
      comments = editFormula->Text.c_str();
    else if(*i_checked == lookupTables->IdOf("gastro_types", "Ostomy"))
      comments = editOstomy->Text.c_str();
    else if(*i_checked == lookupTables->IdOf("gastro_types", "Residual"))
      comments = editResidual->Text.c_str();
    else if(*i_checked == lookupTables->IdOf("gastro_types", "Other"))
      comments = memoComments->Text.c_str();

    record_idx = IsInVecMap(editing_gastro_items, "gastro_id", *i_checked);
    if(record_idx >= 0)
    {
      // Do Update because this gastro_items row already exists.
      if(!UpdateGastroItem(record_idx, comments))
        return false;
    }
    else
    {
      // Do Insert because this gastro_items row doesn't exist.
      if(!InsertGastroItem(*i_checked, comments))
        return false;
    }
  }

  // * If the ID is NOT checked, and the id is in the editing map, mark that item as Deleted via the current_id.
  // * If the ID is NOT checked, and the id is NOT in the editing map, do nothing at all.
  for(psetstr::iterator i_unchecked = unchecked_ids.begin(); i_unchecked != unchecked_ids.end(); ++i_unchecked)
  {
    record_idx = IsInVecMap(editing_gastro_items, "gastro_id", *i_unchecked);
    if(record_idx >= 0)
    {
      // Do Update and set the existing gastro_items record to 'Deleted'.
      if(!DeleteGastroItem(record_idx))
        return false;
    }
    // Otherwise, we neither had a record to begin with, nor do we have one to insert now, so do nothing.
  }

  // If we get this far, nothing borked up above.
  return true;
}
//----------------------------------------------------------------------------------------------------------------------
// SetViewMode - Overridden to activate viewing of gastro_items information
void TframeAssessmentGastro::SetViewMode()
{
  TframeAssessmentGeneric::SetViewMode();

  panelGastro->Enabled = false;                    // It is still not possible to enter data.
  editing_gastro_items.clear();                    // Clear out the items vecmap for safety
  checks_filter.UncheckAll();
  ClearControlsText(panelGastro);
  checkNPI->Checked = false;
}
//----------------------------------------------------------------------------------------------------------------------
// SetEditMode - Overridden to activate editing of gastro_items records
void TframeAssessmentGastro::SetEditMode()
{
  TframeAssessmentGeneric::SetEditMode();

  panelGastro->Enabled = true;               // Enable panel controls
}
//----------------------------------------------------------------------------------------------------------------------
// FilterAddGastro - "Add" handler for the gastro_items filter. This actually just activates editing mode here.
void TframeAssessmentGastro::FilterAddGastro()
{
  if(btnUnlockSave->Enabled && frame_mode == mode_viewing)
    btnUnlockSaveClick(NULL);
}
//----------------------------------------------------------------------------------------------------------------------
// FilterEditGastro - Same as above.
void TframeAssessmentGastro::FilterEditGastro()
{
  if(btnUnlockSave->Enabled && frame_mode == mode_viewing)
    btnUnlockSaveClick(NULL);
}
//----------------------------------------------------------------------------------------------------------------------
// AtlasNotify - Overridden to refresh gastro_items listview
bool TframeAssessmentGastro::AtlasNotify(int request_num, string matched_value)
{
  bool handled = false;

  if(request_num == gastro_request)
  {
    gastro_filter.RefreshDataSet();
    handled = true;
  }

  return handled;
}
//----------------------------------------------------------------------------------------------------------------------
// SendAtlas - Overridden to send out notifications for gastro_items
void TframeAssessmentGastro::SendAtlas()
{
  atlas->Notify("gastro_items.task_id", task_id); // Is this good enough?
}
//----------------------------------------------------------------------------------------------------------------------
// WriteReport - Write out a report on the gastrointestinal conditions
bool TframeAssessmentGastro::WriteReport(TComponent *compositeReportCtrl)
{
  bool success = false;

  try
  {
    string sql =
      "select GT.item, GI.comments "
      "from gastro_items GI "
      "left join gastro_types GT on GI.gastro_id = GT.id "
      "where GI.current_id >= 3 and GI.task_id = '" + task_id + "' ";

    InitDataSet(sql);

    TformReportAssessment *reportForm = report_data.reportForm;
    TQuickRep *thisReport = reportForm->reportGastro;

    thisReport->DataSet = report_data.dbDataSet;
    SetupQRDBText(reportForm->txtGastroItem,   "item");
    SetupQRDBText(reportForm->txtGastroDetail, "comments");

    // NB: Need to pass it as TComponent to avoid problems with having QuickReport included in the headers
    TQRCompositeReport *compositeReport = dynamic_cast<TQRCompositeReport *>(compositeReportCtrl);
    if(compositeReport)
    {
      compositeReport->Reports->Add(thisReport);
      success = true;
    }
  }
  catch(...)
  {
  }

  return success;
}
//----------------------------------------------------------------------------------------------------------------------
// listGastroTypesClick - Pass on checkbox hit events inside the items listview to the parent class.
void __fastcall TframeAssessmentGastro::listGastroTypesClick(TObject *Sender)
{
  FilterCheckBoxHitTest(listGastroTypes);
}
//----------------------------------------------------------------------------------------------------------------------
// checkNPIClick - Set modified flag if the NPI checkbox is clicked.
void __fastcall TframeAssessmentGastro::checkNPIClick(TObject *Sender)
{
  if(frame_mode == mode_editing)
    SetModified(true);
}
//----------------------------------------------------------------------------------------------------------------------

