//----------------------------------------------------------------------------------------------------------------------

#include <vcl.h>
#pragma hdrstop

#include "baseheaders.h"
#include "codeFrameAssessmentGeneric.h"
#include "codeRecordPatientAssessment.h"
//----------------------------------------------------------------------------------------------------------------------
#pragma package(smart_init)
#pragma resource "*.dfm"
TframeAssessmentGeneric *frameAssessmentGeneric;
//----------------------------------------------------------------------------------------------------------------------
__fastcall TframeAssessmentGeneric::TframeAssessmentGeneric(TComponent* Owner)
        : TFrame(Owner), lookups(), report_data()
{
  generic_transaction = NULL;
  frame_mode = mode_none;
  modified = false;
  been_viewed = false;
  is_db_active = false;
  task_id = "";
  person_id = "";
  group_id = "";
  permission = "";

  // An AssessmentGeneric frame *must* be attached to a valid instance of TformRecordPatientAssessment.
  parent_form = dynamic_cast<TformRecordPatientAssessment *>(Owner);
}
//----------------------------------------------------------------------------------------------------------------------
__fastcall TframeAssessmentGeneric::~TframeAssessmentGeneric()
{
  // Don't leak an active locking transaction (it might rollback on its own, but I'm not chancing it; the VCL code for
  // these things is a gigantic gnarly mess).
  if(generic_transaction)
  {
    if(generic_transaction->Active)
      generic_transaction->Rollback();
  }
  // The transaction object itself is owned by the frame and so it *should* be destroyed by ~TComponent.

  // Unregister any combo boxes
  UnRegisterLookups();
}
//----------------------------------------------------------------------------------------------------------------------
// Initialize - called when the frame is created by the patient assessment form.
bool TframeAssessmentGeneric::Initialize()
{
  Lock();      // Start all controls locked
  return true;
}
//----------------------------------------------------------------------------------------------------------------------
// Lock - Overrideable. Something wants to lock all controls on the frame. Call your parent implementation!
void TframeAssessmentGeneric::Lock()
{
  // Disable the toolbar buttons on the coolbar
  btnUnlockSave->Enabled = false;
  btnDiscard->Enabled = false;
  SetStatus("Locked");
}
//----------------------------------------------------------------------------------------------------------------------
// TaskIdChanging - Overrideable. Called when the parent form calls SetTaskId.
void TframeAssessmentGeneric::TaskIdChanging(const string &new_id)
{
  // Nothing to do here (yet?)
}
//----------------------------------------------------------------------------------------------------------------------
// SetTaskId - the parent form has a valid task ID and wants us to know about it.
void TframeAssessmentGeneric::SetTaskId(const string &id)
{
  if(id != task_id && !IsBlankOrZero(id))
  {
    map<string, string> task_id_fields;
    SqlToMap(dbDatabase, "select group_id, person_id from tasks where id = '" + id + "' ", task_id_fields);
    group_id  = task_id_fields["group_id"];
    person_id = task_id_fields["person_id"];
    TaskIdChanging(id);
    task_id = id;
  }
  SetViewMode(); // Enter view mode now
}
//----------------------------------------------------------------------------------------------------------------------
// SetPermission - Sets the table permission needed to use an assessment frame
void TframeAssessmentGeneric::SetPermission(const string &str)
{
  permission = str;
}
//----------------------------------------------------------------------------------------------------------------------
// FirstVisible - Overrideable. Called when the parent form makes this frame visible for the first time.
void TframeAssessmentGeneric::FirstVisible()
{
  // Nothing to do here (yet?)
}
//----------------------------------------------------------------------------------------------------------------------
// MakeVisible - If visible is true, the parent form has made this frame the active editing frame
void TframeAssessmentGeneric::MakeVisible(bool visible)
{
  Visible = visible;

  if(Visible && !been_viewed) // First time?
  {
    been_viewed = true;
    FirstVisible();
  }
}
//----------------------------------------------------------------------------------------------------------------------
// DoHint - Called from parent form if it gets an OnHint event through its TApplicationEvents object
void TframeAssessmentGeneric::DoHint(TObject *Sender)
{
  statusGeneric->Panels->Items[2]->Text = Application->Hint;
}
//----------------------------------------------------------------------------------------------------------------------
// SetBandCaption - Call from child class implementation to change the caption text on the CoolBar to something
// area-specific.
void TframeAssessmentGeneric::SetBandCaption(const string &str)
{
  coolGeneric->Bands->Items[0]->Text = str.c_str();
}
//----------------------------------------------------------------------------------------------------------------------
// SetStatus - Set the status message in the first panel of the status bar
void TframeAssessmentGeneric::SetStatus(const string &str)
{
  statusGeneric->Panels->Items[0]->Text = str.c_str();
}
//----------------------------------------------------------------------------------------------------------------------
// SetModified - Call this rather than setting modified directly and it will update the status bar text accordingly
void TframeAssessmentGeneric::SetModified(bool p_modified)
{
  modified = p_modified;
  statusGeneric->Panels->Items[1]->Text = modified ? "Modified" : "";
}
//----------------------------------------------------------------------------------------------------------------------
// SetViewMode - Overrideable. Put the frame into viewing mode. Data is read-only.
void TframeAssessmentGeneric::SetViewMode()
{
  frame_mode = mode_viewing; // now in view mode
  SetModified(false);        // not modified yet
  btnUnlockSave->Caption = "Unlock and Edit";
  btnUnlockSave->Hint    = "Unlock|Unlock this panel to edit information";
  btnUnlockSave->Enabled = HaveUnlockPermission(false);
  btnDiscard->Enabled = false;
  SetStatus("Viewing");

  // Parent form can unlock for editing again
  parent_form->SetUnlockEnabled(true);
}
//----------------------------------------------------------------------------------------------------------------------
// SetEditMode - Overrideable. Put the frame into editing mode.
void TframeAssessmentGeneric::SetEditMode()
{
  frame_mode = mode_editing; // now in edit mode
  SetModified(false);        // not modified yet
  btnUnlockSave->Caption = "Save Information";
  btnUnlockSave->Hint    = "Save|Save your changes to this information";
  btnUnlockSave->Enabled = true;
  btnDiscard->Enabled = true;
  SetStatus("Editing");

  // Notify parent form that it should not unlock itself for editing
  parent_form->SetUnlockEnabled(false);
}
//----------------------------------------------------------------------------------------------------------------------
// HaveViewPermission - Returns true or false if the user has view permission to the table itself.
bool TframeAssessmentGeneric::HaveViewPermission()
{
  return (optionsPermissions["table." + permission]["view"] == "true");
}
//----------------------------------------------------------------------------------------------------------------------
// HaveUnlockPermission - Returns true or false if the user has unlock permission to the table itself.
bool TframeAssessmentGeneric::HaveUnlockPermission(bool creating_new)
{
  return (optionsPermissions["table." + permission][creating_new ? "insert" : "update"] == "true");
}
//----------------------------------------------------------------------------------------------------------------------
// OpenTransaction - Starts up the locking generic_transaction that protects records against concurrent modification
bool TframeAssessmentGeneric::OpenTransaction()
{
  // NB: Based on TformRecordGeneric::GenericTransaction
  if(generic_transaction)
  {
    if(generic_transaction->Active)
    {
      parent_form->PSay("Transaction already open - not sure whether to commit or rollback", m_database_error, MB_OK,
                        "OpenTransaction called while generic_transaction already active");
      return false;
    }
    else
      is_db_active = true;
  }
  else
  {
    generic_transaction = new TIBTransaction(this);
    StdTransaction(generic_transaction, dbDatabase);
    generic_transaction->Commit();
    is_db_active = true;
  }

  return true;
}
//----------------------------------------------------------------------------------------------------------------------
// LockRecords - Overrideable. Call to unlock one or more record(s) in the database. Note: generic_transaction is
// available during this method call and should be used for all locking updates.
bool TframeAssessmentGeneric::LockRecords()
{
  // Nothing to do here (yet?)
  return true;
}
//----------------------------------------------------------------------------------------------------------------------
// LockTransaction - Acquires lock(s) on record(s) in the database, for allowing exclusive editing.
bool TframeAssessmentGeneric::LockTransaction()
{
  // NB: Loosely based on TformRecordGeneric::GetLockFromDatabase
  is_db_active = false;
  OpenTransaction();
  if(is_db_active)
  {
    generic_transaction->StartTransaction();
    if(LockRecords())
      return true;
    else
    {
      parent_form->PSay("Could not lock record(s) for update. Someone else may be editing it/them currently.",
                        m_database_error, MB_OK);
      generic_transaction->Rollback();
    }
  }
  else
    parent_form->PSay("No locking transaction is available.", m_database_error, MB_OK);

  return false;
}
//----------------------------------------------------------------------------------------------------------------------
// CanUnlock - Overrideable. Return true if the record is good to be unlocked for editing. Call your parent!
bool TframeAssessmentGeneric::CanUnlock(bool &creating_new)
{
  return HaveUnlockPermission(creating_new);
}
//----------------------------------------------------------------------------------------------------------------------
// ClearControlsModified - Marks all TCustomEdit-descendant controls as unmodified
void TframeAssessmentGeneric::ClearControlsModified(TWinControl *parent)
{
  try
  {
    if(parent)
    {
      for(int i = 0; i < parent->ControlCount; i++)
      {
        if(parent->Controls[i]->InheritsFrom(__classid(TCustomEdit)))
          ((TCustomEdit *)parent->Controls[i])->Modified = false;

        if(parent->Controls[i]->InheritsFrom(__classid(TWinControl)))
          ClearControlsModified((TWinControl *)(parent->Controls[i]));
      }
    }
  }
  catch(...)
  {
    // Anything that goes wrong here is just VCL being typically braindead, so I really don't care, frankly.
  }
}
//----------------------------------------------------------------------------------------------------------------------
// ClearControlsText - Clears the Text property of all control classes known to have one.
void TframeAssessmentGeneric::ClearControlsText(TWinControl *parent)
{
  // NB: Based on components.cpp's RecursiveClear. This is more restricted to allow custom logic on descendant forms.
  try
  {
    if(parent)
    {
      for(int i = 0; i < parent->ControlCount; i++)
      {
        if(parent->Controls[i]->InheritsFrom(__classid(TEdit)))
          ((TEdit *)parent->Controls[i])->Text = "";
        else if(parent->Controls[i]->InheritsFrom(__classid(TRichEdit)))
          ((TRichEdit *)parent->Controls[i])->Text = "";
        else if(parent->Controls[i]->InheritsFrom(__classid(TMemo)))
          ((TMemo *)parent->Controls[i])->Text = "";
        else if(parent->Controls[i]->InheritsFrom(__classid(TComboBox)))
          ((TComboBox *)parent->Controls[i])->Text = "";

        if(parent->Controls[i]->InheritsFrom(__classid(TWinControl)))
          ClearControlsText((TWinControl *)(parent->Controls[i]));
      }
    }
  }
  catch(...)
  {
  }
}
//----------------------------------------------------------------------------------------------------------------------
// ClearControlsChecked - As with checkBoxClicked below, this is mainly separate from the above operation because it
// was implemented very late into the process and a lot of child frames therefore manage their own few checkboxes
// manually. This is for the Respiratory and other later assessment frames which have tons of checkboxes.
void TframeAssessmentGeneric::ClearControlsChecked(TWinControl *parent)
{
  try
  {
    if(parent)
    {
      for(int i = 0; i < parent->ControlCount; i++)
      {
        if(parent->Controls[i]->InheritsFrom(__classid(TCheckBox)))
          ((TCheckBox *)parent->Controls[i])->Checked = false;

        if(parent->Controls[i]->InheritsFrom(__classid(TWinControl)))
          ClearControlsChecked((TWinControl *)(parent->Controls[i]));
      }
    }
  }
  catch(...)
  {
  }
}
//----------------------------------------------------------------------------------------------------------------------
// UpdateForEdit - Overrideable. Refresh the information on the form with the newest data, queried after the locking
// transaction is opened.
void TframeAssessmentGeneric::UpdateForEdit()
{
  // Nothing to do here (yet?)
}
//----------------------------------------------------------------------------------------------------------------------
// DoUnlock - External interface function. Called when the frame should be unlocked for editing. Returns false if the
// process fails for any reason.
bool TframeAssessmentGeneric::DoUnlock(bool creating_new)
{
  bool result = false;

  if(CanUnlock(creating_new))
  {
    // If not a new record, create locking transaction and then update all fields to fresh values
    if(!creating_new)
    {
      if(LockTransaction())
        UpdateForEdit();
      else
        return false; // failed to get transaction (error message already posted above)
    }
       
    // Enter edit mode
    SetEditMode();

    result = true;
  }

  return result;
}
//----------------------------------------------------------------------------------------------------------------------
// SavingNew - Overrideable. Return true if an insert statement will be performed.
bool TframeAssessmentGeneric::SavingNew()
{
  // Default implementation - not inserting.
  return false;
}
//----------------------------------------------------------------------------------------------------------------------
// IsValidOnScreen - Overrideable. Do input validation before the data is saved.
bool TframeAssessmentGeneric::IsValidOnScreen()
{
  // Default implementation - input is considered valid.
  return true;
}
//----------------------------------------------------------------------------------------------------------------------
// DoInsertOrUpdate - This is a utility method for descendant classes which manage a single record at a time which needs
// an insert or update depending on the "id" value in the ending_state map. This operation occurs on almost all of the
// child class forms. Call from inside DoSaveData.
bool TframeAssessmentGeneric::DoInsertOrUpdate(pmapstrs &ending_state, pmapstrtoint &leave_alone, string table_name)
{
  if(ending_state["id"] == "new")
  {
    ending_state["id"] = GetNextId(generic_transaction, table_name);
    if(!IsInt(ending_state["id"]))
    {
      parent_form->PSay("Could not get new id", m_database_error, MB_OK, "Insert");
      return false;
    }
    if(ExecuteInsertStatement(generic_transaction, table_name, ending_state, leave_alone))
      return true;
  }
  else
  {
    if(ExecuteUpdateStatement(generic_transaction, table_name, ending_state, leave_alone))
      return true;
  }

  parent_form->PSay("Could not save changes.", m_database_error, MB_OK);
  return false; // Some kind of error occurred
}
//----------------------------------------------------------------------------------------------------------------------
// DoSaveData - Overrideable. Perform insert and update statements as needed using generic_transaction
bool TframeAssessmentGeneric::DoSaveData()
{
  // Default implementation - considered successful
  return true;
}
//----------------------------------------------------------------------------------------------------------------------
// SaveData - Private implementation of basic record saving logic.
bool TframeAssessmentGeneric::SaveData()
{
  // NB: Based loosely on TformRecordGeneric::PutDataInDatabase
  try // Thou shalt not throw.
  {
    if(SavingNew()) // Saving a new record?
    {
      if(OpenTransaction()) // Need a transaction for insert
      {
        generic_transaction->StartTransaction();
        if(DoSaveData())
        {
          try
          {
            generic_transaction->Commit(); // Success!
            return true;
          }
          catch(...)
          {
            parent_form->PSay("Could not commit insert of new record(s).", m_database_error, MB_OK, "Insert");
          }
        }
      }
      else
        parent_form->PSay("Could not open transaction for insert.", m_database_error, MB_OK, "Insert");
    }
    else // Inserting.
    {
      if(DoSaveData())
      {
        try
        {
          generic_transaction->Commit(); // Success!
          return true;
        }
        catch(...)
        {
          parent_form->PSay("Could not commit update of record(s).", m_database_error, MB_OK, "Update");
        }
      }
    }
  }
  catch(EIBInterBaseError &iberr)
  {
    parent_form->PSay(iberr.Message.c_str(), m_database_error, MB_OK);
  }
  catch(EIBError &err)
  {
    parent_form->PSay(err.Message.c_str(), m_database_error, MB_OK);
  }
  catch(...)
  {
    parent_form->PSay("Unknown error while trying to save. Please inform a programmer.", m_program_error, MB_OK);
  }

  DoDiscard();  // Error, must drop out of editing mode
  return false; // An error occurred.
}
//----------------------------------------------------------------------------------------------------------------------
// DoSave - External interface function. Called when the frame should be saved. Returns false if the process fails for
// any reason. Pass "forced" as true if the parent form is going to close. This changes the semantics when there are no
// unsaved changes.
bool TframeAssessmentGeneric::DoSave(bool forced)
{
  if(!modified) // Nothing to save?
  {
    if(!forced) // User clicked Save Information without doing anything? o_O
    {
      int result = parent_form->PSay("There is no useful information to save.\n\n"
                                     "No records will be changed or created.\n\n"
                                     "Choose OK to return to View mode, or Cancel to continue editing.",
                                     m_missing_data, MB_OKCANCEL);
      switch(result)
      {
      case IDOK: // Yes, they wish to discard the record.
        DoDiscard();
        return true;
      default:
        return false; // They don't know what the hell they're doing evidently.
      }
    }
    else
    {
      // Forced exit from saving mode - Discard any changes and return to viewing mode
      DoDiscard();
      return true;
    }
  }

  Screen->Cursor = crHourGlass; // busy bees!

  // NB: Based *very* loosely on TformRecordGeneric::DataSave
  if(IsValidOnScreen())
  {
    if(SaveData())
    {
      SendAtlas();
      // TODO: additional after-save actions?
      // return to viewing mode
      SetViewMode();
      Screen->Cursor = crDefault;
      return true;
    }
  }

  // There was an error, cannot save
  Screen->Cursor = crDefault;
  return false;
}
//----------------------------------------------------------------------------------------------------------------------
// DoDiscard - External interface function. Called when the frame should throw away unsaved data. Returns false if the
// process fails for any reason.
bool TframeAssessmentGeneric::DoDiscard()
{
  bool result = true;

  // NB: Based loosely on TformRecordGeneric::DataUndo
  // Clear out the transaction
  if(is_db_active)
  {
    if(generic_transaction && generic_transaction->Active)
      generic_transaction->Rollback();
  }
  // Not necessarily an error; we also pass through here when an "Add" mode implemented by a child frame is canceled.

  // return to viewing mode
  SetViewMode();

  return result;
}
//----------------------------------------------------------------------------------------------------------------------
// CheckModified - Returns whether or not the frame considers the data it holds to include unsaved modifications.
bool TframeAssessmentGeneric::CheckModified()
{
  return modified;
}
//----------------------------------------------------------------------------------------------------------------------
// AtlasNotify - Overrideable. Called by parent form when an Atlas notification is received.
bool TframeAssessmentGeneric::AtlasNotify(int request_num, string matched_value)
{
  // Default behavior: we did not handle this request.
  return false;
}
//----------------------------------------------------------------------------------------------------------------------
// SendAtlas - Overrideable. Called after a successful save to dispatch Atlas refresh notifications.
void TframeAssessmentGeneric::SendAtlas()
{
  // Nothing needed here yet.
}
//----------------------------------------------------------------------------------------------------------------------
// SendAtlasIfChanged - Utility function. Perform Atlas notifications when a record is saved. Both old and new values
// may be sent out depending on whether "new_record" is indicated.
void TframeAssessmentGeneric::SendAtlasIfChanged(bool new_record, string table_name, const pmapstrs &old_vals,
                                                 const pmapstrs &new_vals)
{
  // NB: Based on RecordGeneric::SendAtlas (TODO: should make a lib func?)
  if(!new_record)
  {
    for(pmapstrs::const_iterator i_initial = old_vals.begin(); i_initial != old_vals.end(); ++i_initial)
    {
      string key = table_name + "." + i_initial->first;
      string val = i_initial->second;

      if(LikeString("%id", LowercaseString(key)) || LikeString("%last_updated_by", LowercaseString(key)))
        atlas->Notify(LowercaseString(key), val);
    }
  }

  // regardless, we send ending_state
  for(pmapstrs::const_iterator i_ending = new_vals.begin(); i_ending != new_vals.end(); ++i_ending)
  {
    string key = table_name + "." + i_ending->first;
    string val = i_ending->second;
    pmapstrs::const_iterator i_old = old_vals.find(i_ending->first);
    bool have_oldval = (i_old != old_vals.end()); // jhaley: don't add it if it's not in there already (const map)

    if((LikeString("%id", LowercaseString(key)) || LikeString("%last_updated_by", LowercaseString(key))) &&
       (new_record || !have_oldval || i_old->second != val))
      atlas->Notify(LowercaseString(key), val);
  }
}
//----------------------------------------------------------------------------------------------------------------------
// RegisterLookup - Register a lookup table
void TframeAssessmentGeneric::RegisterLookup(string &identifier, string &tableName, TComboBox * box)
{
  if(lookups.find(identifier) == lookups.end())
  {
    pair<string, TComboBox *> new_pair;

    new_pair.first  = tableName;
    new_pair.second = box;

    lookups[identifier] = new_pair;

    lookupTables->RegisterComboBox(tableName, box);
  }
}
//----------------------------------------------------------------------------------------------------------------------
// RegisterLookup - Overload, register a lookup table w/custom SQL
void TframeAssessmentGeneric::RegisterLookup(string &identifier, string &tableName, TComboBox * box, string &sql)
{
  if(lookups.find(identifier) == lookups.end())
  {
    pair<string, TComboBox *> new_pair;

    new_pair.first  = tableName;
    new_pair.second = box;

    lookups[identifier] = new_pair;

    lookupTables->RegisterComboBox(tableName, box, sql);
  }
}
//----------------------------------------------------------------------------------------------------------------------
// UnRegisterLookups - Private routine called at destruction time; disassociates all lookup tables
void TframeAssessmentGeneric::UnRegisterLookups()
{
  map<string, pair<string, TComboBox *> >::iterator the_box;

  for(the_box = lookups.begin(); the_box != lookups.end(); ++the_box)
    lookupTables->UnregisterComboBox(the_box->second.first, the_box->second.second);

  lookups.clear();
}
//----------------------------------------------------------------------------------------------------------------------
// btnUnlockSaveClick - Toolbar button handler; saves if editing, unlocks if viewing
void __fastcall TframeAssessmentGeneric::btnUnlockSaveClick(TObject *Sender)
{
  if(frame_mode == mode_editing)
    DoSave(false);
  else
    DoUnlock();
}
//----------------------------------------------------------------------------------------------------------------------
// btnDiscardClick - Toolbar button handler; discards all unsaved modifications
void __fastcall TframeAssessmentGeneric::btnDiscardClick(TObject *Sender)
{
  if(frame_mode == mode_editing)
    DoDiscard();
}
//----------------------------------------------------------------------------------------------------------------------
// cboChangeModified - Set this to the OnChange event handler of a combo box on a descendant form and it will set the
// modified flag for the assessment frame when the user changes the value of the combo box in any manner.
void __fastcall TframeAssessmentGeneric::cboChangeModified(TObject *Sender)
{
  if(frame_mode == mode_editing)
    SetModified(true);
}
//----------------------------------------------------------------------------------------------------------------------
// editChangeModified - Set this to the OnChange event handler of an edit control on a descendant form and it will set
// the modified flag for the assessment frame when the user changes the value of the edit control in any manner.
void __fastcall TframeAssessmentGeneric::editChangeModified(TObject *Sender)
{
  if(frame_mode == mode_editing && Sender)
  {
    // In a normal, sane object-oriented system you don't have to branch on type info this way. VCL, in its infinite
    // wisdom, doesn't allow proper polymorphic access to Delphi properties. Otherwise these could have all been handled
    // through the common base class that they share in TCustomEdit... attempting to do so gives a "not accessible"
    // error, since Modified is protected in that parent class (ie., totally useless)

    if(Sender->InheritsFrom(__classid(TEdit)))
    {
      if(((TEdit *)Sender)->Modified)
        SetModified(true);
    }
    else if(Sender->InheritsFrom(__classid(TRichEdit)))
    {
      if(((TRichEdit *)Sender)->Modified)
        SetModified(true);
    }
    else if(Sender->InheritsFrom(__classid(TMemo)))
    {
      if(((TMemo *)Sender)->Modified)
        SetModified(true);
    }
  }
}
//----------------------------------------------------------------------------------------------------------------------
// checkBoxClicked - Set this to the OnClick event handler of a checkbox control on a descendant form and it will set
// the modified flag for the assessment frame when the user changes the state of the checkbox.
void __fastcall TframeAssessmentGeneric::checkBoxClicked(TObject *Sender)
{
  if(frame_mode == mode_editing)
    SetModified(true);
}
//----------------------------------------------------------------------------------------------------------------------
// SetAllEventHandlers - Sets all TEdit, TRichEdit, TMemo, and TComboBox descendant controls to use the above event
// handlers for OnChange events. Call ONLY if all controls on the descendant frame want/need to work this way.
void TframeAssessmentGeneric::SetAllEventHandlers(TWinControl *parent)
{
  try
  {
    if(parent)
    {
      for(int i = 0; i < parent->ControlCount; i++)
      {
        if(parent->Controls[i]->InheritsFrom(__classid(TEdit)))
          ((TEdit *)parent->Controls[i])->OnChange = editChangeModified;
        else if(parent->Controls[i]->InheritsFrom(__classid(TRichEdit)))
          ((TRichEdit *)parent->Controls[i])->OnChange = editChangeModified;
        else if(parent->Controls[i]->InheritsFrom(__classid(TMemo)))
          ((TMemo *)parent->Controls[i])->OnChange = editChangeModified;
        else if(parent->Controls[i]->InheritsFrom(__classid(TComboBox)))
          ((TComboBox *)parent->Controls[i])->OnChange = cboChangeModified;

        if(parent->Controls[i]->InheritsFrom(__classid(TWinControl)))
          SetAllEventHandlers((TWinControl *)(parent->Controls[i]));
      }
    }
  }
  catch(...)
  {
    // Again, anything that goes wrong here is just VCL being typically braindead, so I really don't care, frankly.
  }
}
//----------------------------------------------------------------------------------------------------------------------
// SetCheckBoxHandlers - This is separate from the above only because it was added significantly later, and therefore
// a lot of the child frames do their own checkbox click event handling - mainly due to the fact a lot of them only
// have one checkbox at most. This is for ones that have a LOT of checkboxes, such as TframeAssessmentRespiratory, and
// do not want to treat any of them specially.
void TframeAssessmentGeneric::SetCheckBoxHandlers(TWinControl *parent)
{
  try
  {
    if(parent)
    {
      for(int i = 0; i < parent->ControlCount; i++)
      {
        if(parent->Controls[i]->InheritsFrom(__classid(TCheckBox)))
          ((TCheckBox *)parent->Controls[i])->OnClick = checkBoxClicked;

        if(parent->Controls[i]->InheritsFrom(__classid(TWinControl)))
          SetCheckBoxHandlers((TWinControl *)(parent->Controls[i]));
      }
    }
  }
  catch(...)
  {
    // As above.
  }
}
//----------------------------------------------------------------------------------------------------------------------
// *** Filter Checkbox Utilities - a large number of descendants are just a bunch of checkboxes in a filter, so some
// ***   basic utilties have been abstracted up here into the parent class so that they can be effectively shared.
//----------------------------------------------------------------------------------------------------------------------
// FilterCheckBoxHitTest
// Call this from any OnClick event handler for a classFilterObject's TListView control and it will set the "modified"
// state automatically if a hit is detected on the filter's checkbox area. The filter object should probably use
// checkboxes in order for this to make any sense, but I didn't honestly think I'd have to type that for you to
// know!!!111 :P
void TframeAssessmentGeneric::FilterCheckBoxHitTest(TListView *listView)
{
  // Shamelessly grifted from classFilterObject::Clicked, after modifying same to call down here.
  TPoint current_mouse_position  = Mouse->CursorPos;
  TPoint current_window_position = listView->ClientOrigin;
  int x = current_mouse_position.x - current_window_position.x;
  int y = current_mouse_position.y - current_window_position.y;

  THitTests t = listView->GetHitTestInfoAt(x, y);

  if(t.Contains(htOnStateIcon) || // StateIndex
     t.Contains(htOnIcon))        // ImageIndex
  {
    if(frame_mode == mode_editing)
      SetModified(true);
  }
}
//----------------------------------------------------------------------------------------------------------------------
// GetIDsFromFilter - Convert the IDs in a checkboxes filter to a set of checked and unchecked IDs
void TframeAssessmentGeneric::GetIDsFromCheckBoxFilter(classFilterObject &filter,
                                                       psetstr &id_set_checked, psetstr &id_set_unchecked)
{
  pvecstr checked_vec = filter.GetMultiID();
  pvecmap all_data    = filter.GetAllData(true, false);
  psetstr local_set_checked;

  VecToSet(checked_vec, local_set_checked); // easy enough...

  // Go through the all_data vector and if it's not in the "checked" set, it's unchecked.
  for(size_t i = 0; i < all_data.size(); i++)
  {
    if(local_set_checked.find(all_data[i]["id"]) == local_set_checked.end())
      id_set_unchecked.insert(all_data[i]["id"]); // unchecked
    else
      id_set_checked.insert(all_data[i]["id"]);   // checked
  }
}
//----------------------------------------------------------------------------------------------------------------------
// IsInVecMap - Searches for a specific ID in all rows of a vecmap and returns the index of the item in the vector if
// it exists; returns -1 otherwise. Inefficient <3
int TframeAssessmentGeneric::IsInVecMap(pvecmap &vecmap, string fieldname, string id)
{
  for(size_t i = 0; i < vecmap.size(); i++)
  {
    if(vecmap[i][fieldname] == id)
      return i;
  }

  return -1; // narps.
}
//----------------------------------------------------------------------------------------------------------------------
// LockVecMapRecords - Tries to acquire a lock on a full vecmap of records. A failure to lock any individual record is
// considered a full failure, and generic_transaction should be made to rollback afterward by returning false from
// LockRecords. The "id" field should be present in every row of the vecmap for this to work.
bool TframeAssessmentGeneric::LockVecMapRecords(pvecmap &editing_items, string table_name)
{
  for(size_t i = 0; i < editing_items.size(); i++)
  {
    if(!LockRecordForUpdate(generic_transaction, table_name, editing_items[i]["id"]))
      return false; // Must acquire all items. Any failure is a failure to unlock for editing, GTFO and rollback.
  }

  return true;
}
//----------------------------------------------------------------------------------------------------------------------
// *** Reports *********************************************************************************************************
//----------------------------------------------------------------------------------------------------------------------
// InitDataSet - Utility method.
void TframeAssessmentGeneric::InitDataSet(const string &sql)
{
  report_data.dbDataSet->SelectSQL->Clear();
  report_data.dbDataSet->SelectSQL->Add(sql.c_str());
  report_data.dbDataSet->Open();
  report_data.dbDataSet->First();
}
//----------------------------------------------------------------------------------------------------------------------
// SetupQRDBText - NB: Shamelessly grifted from same routine in TformLaunchGeneric
void TframeAssessmentGeneric::SetupQRDBText(TControl *txtControl, const string &name)
{
  if(txtControl->ClassType() == __classid(TQRDBText))
  {
    TQRDBText *qrdbText = (TQRDBText *)txtControl;
    qrdbText->DataSet   = report_data.dbDataSet;
    qrdbText->DataField = name.c_str();
  }
}
//----------------------------------------------------------------------------------------------------------------------
// SetupQRDBTexts - Takes an array of structures bearing TControl *'s and names and calls the above. Make sure the
// array is terminated by a row of all NULLs.
void TframeAssessmentGeneric::SetupQRDBTexts(txtregister_t txtControls[])
{
  txtregister_t *ctrlToRegister = txtControls;

  while(ctrlToRegister->theName)
  {
    SetupQRDBText(ctrlToRegister->theControl, ctrlToRegister->theName);
    ++ctrlToRegister;
  }
}
//----------------------------------------------------------------------------------------------------------------------
// SetReportData - Called from TformRecordPatientAssessment before WriteReport. Not virtual, don't override :P
void TframeAssessmentGeneric::SetReportData(TformReportAssessment *reportForm, TIBTransaction *dbTransaction)
{
  TIBDataSet *dbDataSet = NULL;

  dbDataSet = new TIBDataSet(NULL); // Each report form needs its own dataset.

  dbDataSet->Database       = dbDatabase;
  dbDataSet->Transaction    = dbTransaction;
  dbDataSet->UniDirectional = false;

  report_data.dbDataSet     = dbDataSet;
  report_data.dbTransaction = dbTransaction;  // Transaction is global and is created by TformRecordPatientAssessment
  report_data.reportForm    = reportForm;     // Ditto for the instance of TformReportAssessment
}
//----------------------------------------------------------------------------------------------------------------------
// WriteReport - Overrideable. Writes out the assessment frame's report, if it has one.
bool TframeAssessmentGeneric::WriteReport(TComponent *compositeReportCtrl)
{
  return false; // Nothing to do here, yet. *Don't* call this from a child implementation.
}
//----------------------------------------------------------------------------------------------------------------------
// ReportDone - Clean up temporary report objects (virtual method, call your parent!)
//   NB: Necessitated by the behavior of TQRCompositeReport. The report controls, datasets, and transaction must all
//   remain valid until the report preview is closed, or QuickReport will meltdown in a storm of access violations.
void TframeAssessmentGeneric::ReportDone()
{
  if(report_data.dbDataSet)
  {
    delete report_data.dbDataSet;
    report_data.dbDataSet = NULL;
  }
  report_data.dbTransaction = NULL;
  report_data.reportForm = NULL;
}
//----------------------------------------------------------------------------------------------------------------------
