//---------------------------------------------------------------------------


#ifndef codeFrameAssessmentGenericH
#define codeFrameAssessmentGenericH
//---------------------------------------------------------------------------
#include <Classes.hpp>
#include <Controls.hpp>
#include <StdCtrls.hpp>
#include <Forms.hpp>
#include <ComCtrls.hpp>
#include <ImgList.hpp>
#include <ToolWin.hpp>
#include "sqlLib.h"
#include <ExtCtrls.hpp>
#include "reportAssessment.h"
#include <Db.hpp>
#include <IBCustomDataSet.hpp>
#include <IBDatabase.hpp>
class classFilterObject;
class TformRecordPatientAssessment;
//---------------------------------------------------------------------------
class TframeAssessmentGeneric : public TFrame
{
__published:	// IDE-managed Components
        TScrollBox *scrollGeneric;
        TCoolBar *coolGeneric;
        TToolBar *barGeneric;
        TToolButton *btnUnlockSave;
        TToolButton *sepOne;
        TToolButton *sepTwo;
        TToolButton *btnDiscard;
        TImageList *imagesGeneric;
        TStatusBar *statusGeneric;
        TSplitter *splitterNorth;
        void __fastcall btnUnlockSaveClick(TObject *Sender);
        void __fastcall btnDiscardClick(TObject *Sender);
private:	// User declarations
        bool OpenTransaction();   // Create generic_transaction if not already valid
        bool LockTransaction();   // Acquire lock(s) on records from the database
        bool SaveData();          // Start process of saving data to the database
        void UnRegisterLookups(); // Remove all registered combo boxes before being destroyed

protected:      // User declarations
        int frame_mode;    // viewing or editing?
        bool modified;     // have unsaved modifications? - please set using SetModified
        bool been_viewed;  // if true, have been viewed
        bool is_db_active; // if true, generic_transaction is alive
        string task_id;    // attached task, once one is available
        string person_id;  // person of attached task
        string group_id;   // group of attached task
        string permission; // table permission needed ("table." + permission)

        struct report_data_t
        {
          TIBDataSet            *dbDataSet;
          TIBTransaction        *dbTransaction;
          TformReportAssessment *reportForm;
        };
        report_data_t report_data;

        // lookups
        map<string, pair<string, TComboBox *> > lookups; // registered lookup tables <uniquename, <table, box>>

        TIBTransaction *generic_transaction;       // transaction used for exclusive updates, and inserts
        TformRecordPatientAssessment *parent_form; // TframeAssessmentGeneric works with/is owned by this form type

        // Basic properties (play with these in child classes)
        void SetBandCaption(const string &str); // Modify the left-hand-side caption on the Cool Bar (area title)
        void SetStatus(const string &str);      // Modify the first pane of the Status Bar (Viewing/Editing/Locked)
        void SetPermission(const string &str);  // Call with a table name to control permissions for view/edit/create
        void SetModified(bool p_modified);      // Call this to keep second pane of Status Bar in sync with mod state

        // Overrideables - Almost all should call parent implementation!
        virtual void TaskIdChanging(const string &new_id); // Called by SetTaskId before task_id is changed
        virtual void FirstVisible();                       // Called first time frame becomes visible
        virtual bool CanUnlock(bool &creating_new);        // Return true or false.
        virtual bool SavingNew();                          // Return true or false.
        virtual void SetViewMode();                        // Go into view mode
        virtual void SetEditMode();                        // Go into edit mode
        virtual bool LockRecords();                        // Call LockRecordForUpdate as needed in here
        virtual void UpdateForEdit();                      // Reset controls to fresh data when unlocking
        virtual bool IsValidOnScreen();                    // Validate input before save
        virtual bool DoSaveData();                         // Do the actual inserts and/or updates
        virtual void SendAtlas();                          // Send out Atlas refresh notifications on save

        // Utilities for clearing controls
        virtual void ClearControlsModified(TWinControl *parent); // Marks all edit controls unmodified
        virtual void ClearControlsText(TWinControl *parent);     // Clears all edit and combo box controls' text
        virtual void ClearControlsChecked(TWinControl *parent);  // Clears all checkboxes

        // Utilities
        void SendAtlasIfChanged(bool new_record, string table_name, const pmapstrs &old_vals, const pmapstrs &new_vals);
        void RegisterLookup(string &identifier, string &tableName, TComboBox* box);
        void RegisterLookup(string &identifier, string &tableName, TComboBox* box, string &sql);
        bool DoInsertOrUpdate(pmapstrs &ending_state, pmapstrtoint &leave_alone, string table_name);

        // Generic event handlers for controls on inheriting frames
        void __fastcall cboChangeModified(TObject *Sender);
        void __fastcall editChangeModified(TObject *Sender);
        void __fastcall checkBoxClicked(TObject *Sender);
        void SetAllEventHandlers(TWinControl *parent);
        void SetCheckBoxHandlers(TWinControl *parent);

        // Filter Checkbox Utilties
        // Mess required to detect changes to a listview's checkbox state (O VCL, mine soul thou hast slain)
        void FilterCheckBoxHitTest(TListView *listView);
        void GetIDsFromCheckBoxFilter(classFilterObject &filter, psetstr &id_set_checked, psetstr &id_set_unchecked);
        int  IsInVecMap(pvecmap &vecmap, string fieldname, string id);
        bool LockVecMapRecords(pvecmap &editing_items, string table_name);

        // Reports
        void InitDataSet(const string &sql);
        void SetupQRDBText(TControl *txtControl, const string &name);
        struct txtregister_t
        {
          TControl   *theControl;
          const char *theName;
        };
        void SetupQRDBTexts(txtregister_t txtControls[]);

public:		// User declarations
        __fastcall  TframeAssessmentGeneric(TComponent* Owner); // Owner must be TformRecordPatientAssessment.
        __fastcall ~TframeAssessmentGeneric();

        // External interface (methods invoked by TformRecordPatientAssessment)
        bool CheckModified();                     // True if user changed something after an unlock
        bool DoUnlock(bool creating_new = false); // Unlock for editing, or, create a new record
        bool DoSave(bool forced);                 // Save any pending modifications
        bool DoDiscard();                         // Discard any pending modifications and return to view mode
        void SetTaskId(const string &id);         // Notify of a change to parent task record ID
        void MakeVisible(bool visible);           // Modify the frame's visibility
        void DoHint(TObject *Sender);             // Notify of an application-level hint event

        // Set data needed for running the frame's report
        void SetReportData(TformReportAssessment *reportForm, TIBTransaction *dbTransaction);

        // Permissions
        bool HaveViewPermission();                    // Frame should never been shown if this returns false.
        bool HaveUnlockPermission(bool creating_new); // Pass in true if you're creating a new record.

        // Overrideables
        virtual bool Initialize(); // Call once when being created
        virtual void Lock();       // Lock against editing
        virtual bool AtlasNotify(int request_num, string matched_value); // Atlas support
        virtual bool WriteReport(TComponent *compositeReportCtrl); // Write out a report for this assessment frame
        virtual void ReportDone(); // Cleanup temporary report objects

        // Frame modes
        enum
        {
          mode_none,    // dunno yet
          mode_viewing, // just lookin', read-only access
          mode_editing  // wants to edit data
        };

        int GetFrameMode() const { return frame_mode; }
};
//---------------------------------------------------------------------------
extern PACKAGE TframeAssessmentGeneric *frameAssessmentGeneric;
//---------------------------------------------------------------------------
#endif
