//----------------------------------------------------------------------------------------------------------------------

#include <vcl.h>
#pragma hdrstop

#include "codeFrameAssessmentGenitourinary.h"
#include "codeRecordPatientAssessment.h"
//----------------------------------------------------------------------------------------------------------------------
#pragma package(smart_init)
#pragma link "codeFrameAssessmentGeneric"
#pragma resource "*.dfm"
TframeAssessmentGenitourinary *frameAssessmentGenitourinary;
//----------------------------------------------------------------------------------------------------------------------
__fastcall TframeAssessmentGenitourinary::TframeAssessmentGenitourinary(TComponent* Owner)
        : TframeAssessmentGeneric(Owner), genuri_filter(), urine_filter(), condition_filter(), detail_filter(),
          genuri_request(-1), editing_genuri_items(), final_ending_state()
{
}
//----------------------------------------------------------------------------------------------------------------------
// Initialize - Overridden to set parent properties and setup genuri_items filtered listview.
bool TframeAssessmentGenitourinary::Initialize()
{
  // Set AssessmentGeneric properties
  SetBandCaption("Genitourinary");
  SetPermission("genuri_items");

  // Setup top filter
  genuri_filter.SetListViewHandle(listGenuri);
  genuri_filter.SetSecurity("genuri_items");

  genuri_filter.AddColumnSpec("Item",        "genuri_items", as_text, true,  160, rawString);
  genuri_filter.AddColumnSpec("Category",    "genuri_items", as_text, true,   70, rawString);
  genuri_filter.AddColumnSpec("Details",     "genuri_items", as_text, true,  350, rawString);
  genuri_filter.AddColumnSpec("Current Id",  "current_id",   as_int,  false,  80);
  genuri_filter.AddColumnSpec("Internal Id", "internal_id",  as_int,  false,  80);

  genuri_filter.FetchPreferences("assessment.genuri.topfilter");
  genuri_filter.SetSQLStatement("");
  genuri_filter.RefreshDataSet();

  // setup input filters

  string urine_sql, condition_sql, detail_sql;

  urine_sql     = "select item, id from genuri_types where valid = 1 and genuri_value_id = '"
                   + lookupTables->IdOf("genuri_value_types", "Urine") + "' ";
  condition_sql = "select item, id from genuri_types where valid = 1 and genuri_value_id = '"
                   + lookupTables->IdOf("genuri_value_types", "Condition") + "' ";
  detail_sql    = "select item, id from genuri_types where valid = 1 and genuri_value_id = '"
                   + lookupTables->IdOf("genuri_value_types", "Detailed") + "' ";

  // Urine Appearance Filter
  urine_filter.SetListViewHandle(listUrineAppearance);
  urine_filter.SetSecurity("");
  urine_filter.AddColumnSpec("Item",        "genuri_items", as_text, true,  128, rawString);
  urine_filter.AddColumnSpec("Internal Id", "internal_id",  as_int,  false,  80);
  urine_filter.FetchPreferences("assessment.genuri.urineapp");
  urine_filter.SetIconStatus(false);
  urine_filter.SetCheck(true);
  urine_filter.SetSQLStatement(urine_sql);
  urine_filter.RefreshDataSet();

  // Condition Filter
  condition_filter.SetListViewHandle(listCondition);
  condition_filter.SetSecurity("");
  condition_filter.AddColumnSpec("Item",        "genuri_items", as_text, true,  128, rawString);
  condition_filter.AddColumnSpec("Internal Id", "internal_id",  as_int,  false,  80);
  condition_filter.FetchPreferences("assessment.genuri.condition");
  condition_filter.SetIconStatus(false);
  condition_filter.SetCheck(true);
  condition_filter.SetSQLStatement(condition_sql);
  condition_filter.RefreshDataSet();

  // Detailed Filter
  detail_filter.SetListViewHandle(listDetailed);
  detail_filter.SetSecurity("");
  detail_filter.AddColumnSpec("Item",        "genuri_items", as_text, true,  128, rawString);
  detail_filter.AddColumnSpec("Internal Id", "internal_id",  as_int,  false,  80);
  detail_filter.FetchPreferences("assessment.genuri.detailed");
  detail_filter.SetIconStatus(false);
  detail_filter.SetCheck(true);
  detail_filter.SetSQLStatement(detail_sql);
  detail_filter.RefreshDataSet();

  // Set edit box change handlers
  SetAllEventHandlers(panelGenuri);

  return TframeAssessmentGeneric::Initialize();
}
//----------------------------------------------------------------------------------------------------------------------
// Lock - Overridden to lock genuri panel
void TframeAssessmentGenitourinary::Lock()
{
  TframeAssessmentGeneric::Lock(); // lock parent controls
  panelGenuri->Enabled = false;    // lock all controls
}
//----------------------------------------------------------------------------------------------------------------------
// PutUpGenUriItems - Refreshes the controls to reflect the state of the genitourinary assessment
void TframeAssessmentGenitourinary::PutUpGenUriItems(pvecmap &genuri_items)
{
  // Clear out all checkbox filters, and the NPI item checkbox
  urine_filter.UncheckAll();
  condition_filter.UncheckAll();
  detail_filter.UncheckAll();

  bool have_NPI = false;
  psetstr urine_ids;
  psetstr condition_ids;
  psetstr detail_ids;

  //clear edits
  ClearControlsText(panelGenuri);

  // Analyze the retrieved genuri assessment items
  for(size_t i = 0; i < genuri_items.size(); i++)
  {
    // Skip items which are anything other than Current. This allows Deleted items to be updated back to Current status
    // when saving.
    if(genuri_items[i]["current_id"] != lookupTables->IdOf("current_types", "Current"))
      continue;

    if(genuri_items[i]["genuri_value_id"] == lookupTables->IdOf("genuri_value_types", "Urine"))
      urine_ids.insert(genuri_items[i]["genuri_id"]);
    else if(genuri_items[i]["genuri_value_id"] == lookupTables->IdOf("genuri_value_types", "Condition"))
      condition_ids.insert(genuri_items[i]["genuri_id"]);
    else if(genuri_items[i]["genuri_value_id"] == lookupTables->IdOf("genuri_value_types", "Detailed"))
      detail_ids.insert(genuri_items[i]["genuri_id"]);
    else if(genuri_items[i]["genuri_value_id"] == lookupTables->IdOf("genuri_value_types", "NPI"))
      have_NPI = true;

    // Put up comments on certain items
    if(genuri_items[i]["genuri_id"] == lookupTables->IdOf("genuri_types", "Other"))
      memoComments->Text = genuri_items[i]["comments"].c_str();
    if(genuri_items[i]["genuri_id"] == lookupTables->IdOf("genuri_types", "Catheter"))
      memoCatheter->Text = genuri_items[i]["comments"].c_str();
    if(genuri_items[i]["genuri_id"] == lookupTables->IdOf("genuri_types", "Frequency"))
      editFrequency->Text = genuri_items[i]["comments"].c_str();
    if(genuri_items[i]["genuri_id"] == lookupTables->IdOf("genuri_types", "Urgency"))
      editUrgency->Text = genuri_items[i]["comments"].c_str();
  }

  // Set checkboxes
  urine_filter.SetCheckedWhereID(urine_ids, true);
  condition_filter.SetCheckedWhereID(condition_ids, true);
  detail_filter.SetCheckedWhereID(detail_ids, true);

  checkNPI->Checked = have_NPI;
}
//----------------------------------------------------------------------------------------------------------------------
// TaskIdChanging - Overridden to populate genitourinary items filter
void TframeAssessmentGenitourinary::TaskIdChanging(const string &new_id)
{
  string sql;

  TframeAssessmentGeneric::TaskIdChanging(new_id);

  sql = "select GT.item, GVT.item, GI.comments, GI.current_id, GI.id "
        "from genuri_items GI "
        "left join genuri_types       GT  on GI.genuri_id       = GT.id "
        "left join genuri_value_types GVT on GT.genuri_value_id = GVT.id "
        "where GI.current_id >= 2 and GI.task_id = '" + new_id + "' ";

  genuri_filter.SetSQLStatement(sql);
  genuri_filter.RefreshDataSet();
  genuri_filter.SetAddEdit(this, AddRecord(&TframeAssessmentGenitourinary::FilterAddGenUri),
                                EditRecord(&TframeAssessmentGenitourinary::FilterEditGenUri));
  genuri_request = parent_form->RequestNotificationWhen("genuri_items.task_id", new_id, rel_e, as_int);
}
//----------------------------------------------------------------------------------------------------------------------
// UpdateForEdit - Overridden to put fresh data into the editing_genuri_items vecmap and on screen.
void TframeAssessmentGenitourinary::UpdateForEdit()
{
  string sql =
    "select GI.genuri_id, GT.genuri_value_id, GI.comments, GI.current_id, GI.id "
    "from genuri_items GI "
    "left join genuri_types GT on GI.genuri_id = GT.id "
    "where GI.task_id = '" + task_id + "' ";

  editing_genuri_items.clear();

  if(!SqlToVecMap(generic_transaction, sql, editing_genuri_items))
    parent_form->PSay("Could not retrieve up-to-date information for genitourinary items", m_warning, MB_OK);

  // Refresh controls
  PutUpGenUriItems(editing_genuri_items);

  TframeAssessmentGeneric::UpdateForEdit();
}
//----------------------------------------------------------------------------------------------------------------------
// LockRecords - Overridden to lock the genuri_items records
bool TframeAssessmentGenitourinary::LockRecords()
{
  // Must acquire all items. Any failure is a failure to unlock for editing, GTFO and rollback.
  if(!LockVecMapRecords(editing_genuri_items, "genuri_items"))
    return false;

  return TframeAssessmentGeneric::LockRecords();
}
//----------------------------------------------------------------------------------------------------------------------
// IsValidOnScreen - Overridden to do validation for genitourinary items
bool TframeAssessmentGenitourinary::IsValidOnScreen()
{
  pvecstr checked_urine_ids  = urine_filter.GetMultiID();
  pvecstr checked_cond_ids   = condition_filter.GetMultiID();
  pvecstr checked_detail_ids = detail_filter.GetMultiID();

  // Make sure at least one genuri assessment item is specified
  if(editing_genuri_items.size() == 0 && // jhaley 20110629: Only when the record is new, otherwise can't delete last one
     !checkNPI->Checked && checked_urine_ids.size() == 0 && checked_cond_ids.size() == 0 && checked_detail_ids.size() == 0)
  {
    parent_form->PSay("You have not specified any saveable data. Please check one or more assessment items.",
                      m_missing_data, MB_OK);
    return false;
  }

  // If NPI is specified, most options should be disallowed
  if(checkNPI->Checked)
  {
    if(checked_urine_ids.size() != 0)
    {
      parent_form->PSay("You indicated that the patient had no notable problems by checking the NPI indicator, but also "
                        "indicated problems in the Urine Appearance assessment category. Please correct this inconsistency "
                        "before saving.", m_invalid_data, MB_OK);
      return false;
    }
    if(checked_cond_ids.size() != 0)
    {
      parent_form->PSay("You indicated that the patient had no notable problems by checking the NPI indicator, but also "
                        "indicated problems in the Condition assessment category. Please correct this inconsistency "
                        "before saving.", m_invalid_data, MB_OK);
      return false;
    }
    // Note: Detailed area is allowed even with NPI indicator, as is not necessarily excluded by it.
  }

  bool other_checked = false, catheter_checked = false, frequency_checked = false, urgency_checked = false;

  // ones that require comments:
  // * Detailed - Other, Catheter, Frequency, Urgency
  for(int i = 0; i < checked_detail_ids.size(); i++)
  {
    if(checked_detail_ids[i] == lookupTables->IdOf("genuri_types", "Other"))
    {
      other_checked = true;
      if(memoComments->Text == "")
      {
        parent_form->PSay("You selected 'Other' but did not provide any information in the Comments field.",
                          m_missing_data, MB_OK);
        return false;
      }
    }
    if(checked_detail_ids[i] == lookupTables->IdOf("genuri_types", "Catheter"))
    {
      catheter_checked = true;
      if(memoCatheter->Text == "")
      {
        parent_form->PSay("You selected 'Catheter' but did not describe the catheter.",
                          m_missing_data, MB_OK);
        return false;
      }
    }
    if(checked_detail_ids[i] == lookupTables->IdOf("genuri_types", "Frequency"))
    {
      frequency_checked = true;
      if(editFrequency->Text == "")
      {
        parent_form->PSay("You selected 'Frequency' but did not describe the patient's urination frequency.",
                          m_missing_data, MB_OK);
        return false;
      }
    }
    if(checked_detail_ids[i] == lookupTables->IdOf("genuri_types", "Urgency"))
    {
      urgency_checked = true;
      if(editUrgency->Text == "")
      {
        parent_form->PSay("You selected 'Urgency' but did not describe the patient's urination urgency.",
                          m_missing_data, MB_OK);
        return false;
      }
    }
  }

  if(!other_checked && memoComments->Text != "")
  {
    parent_form->PSay("You did not check 'Other' but provided general comments. Please either select 'Other' in the "
                      "'Detailed' assessment item listbox, or clear the Comments field.", m_invalid_data, MB_OK);
    return false;
  }
  if(!catheter_checked && memoCatheter->Text != "")
  {
    parent_form->PSay("You did not check 'Catheter' but provided a description of one. Please either select 'Catheter' "
                      "in the 'Detailed' assessment item listbox, or clear the Catheter field.", m_invalid_data, MB_OK);
    return false;
  }
  if(!frequency_checked && editFrequency->Text != "")
  {
    parent_form->PSay("You did not check 'Frequency' but provided a description. Please either select 'Frequency' in "
                      "the 'Detailed' assessment item listbox, or clear the Frequency field.", m_invalid_data, MB_OK);
    return false;
  }
  if(!urgency_checked && editUrgency->Text != "")
  {
    parent_form->PSay("You did not check 'Urgency' but provided a description. Please either select 'Urgency' in the "
                      "'Detailed' assessment item listbox, or clear the Urgency field.", m_invalid_data, MB_OK);
    return false;
  }

  return TframeAssessmentGeneric::IsValidOnScreen();
}
//----------------------------------------------------------------------------------------------------------------------
bool TframeAssessmentGenitourinary::UpdateGenUriItem(int record_index, string comment)
{
  pmapstrs ending_state;
  pmapstrtoint leave_alone;

  ending_state["id"]                = editing_genuri_items[record_index]["id"];
  ending_state["current_id"]        = lookupTables->IdOf("current_types", "Current");
  ending_state["last_updated_by"]   = current_userid;
  ending_state["last_update_stamp"] = RightNow();
  ending_state["task_id"]           = task_id;
  ending_state["genuri_id"]         = editing_genuri_items[record_index]["genuri_id"];
  ending_state["comments"]          = comment;

  leave_alone["comments"] = sql_no_uppercasing;

  final_ending_state.push_back(ending_state);

  return ExecuteUpdateStatement(generic_transaction, "genuri_items", ending_state, leave_alone);
}
//----------------------------------------------------------------------------------------------------------------------
bool TframeAssessmentGenitourinary::InsertGenUriItem(string genuri_item_id, string comment)
{
  pmapstrs ending_state;
  pmapstrtoint leave_alone;

  ending_state["id"]                = GetNextId(generic_transaction, "genuri_items");
  ending_state["current_id"]        = lookupTables->IdOf("current_types", "Current");
  ending_state["last_updated_by"]   = current_userid;
  ending_state["last_update_stamp"] = RightNow();
  ending_state["created_by"]        = current_userid;
  ending_state["creation_stamp"]    = ending_state["last_update_stamp"];
  ending_state["task_id"]           = task_id;
  ending_state["genuri_id"]         = genuri_item_id;
  ending_state["comments"]          = comment;

  leave_alone["comments"] = sql_no_uppercasing;

  final_ending_state.push_back(ending_state);

  return ExecuteInsertStatement(generic_transaction, "genuri_items", ending_state, leave_alone);
}
//----------------------------------------------------------------------------------------------------------------------
bool TframeAssessmentGenitourinary::DeleteGenUriItem(int record_index)
{
  pmapstrs ending_state;
  pmapstrtoint leave_alone;

  ending_state["id"]                = editing_genuri_items[record_index]["id"];
  ending_state["current_id"]        = lookupTables->IdOf("current_types", "Deleted"); // Mark as Deleted
  ending_state["last_updated_by"]   = current_userid;
  ending_state["last_update_stamp"] = RightNow();

  leave_alone["comments"] = sql_no_uppercasing;

  final_ending_state.push_back(ending_state);

  return ExecuteUpdateStatement(generic_transaction, "genuri_items", ending_state, leave_alone);
}
//----------------------------------------------------------------------------------------------------------------------
// DoSaveData - Overridden to provide core saving logic.
bool TframeAssessmentGenitourinary::DoSaveData()
{
  psetstr checked_ids;
  psetstr unchecked_ids;
  int record_idx;

  final_ending_state.clear();

  // First determine what is checked and what is not in the filters.
  GetIDsFromCheckBoxFilter(urine_filter,     checked_ids, unchecked_ids);
  GetIDsFromCheckBoxFilter(condition_filter, checked_ids, unchecked_ids);
  GetIDsFromCheckBoxFilter(detail_filter,    checked_ids, unchecked_ids);

  // Throw in the NPI ID
  if(checkNPI->Checked)
    checked_ids.insert(lookupTables->IdOf("genuri_types", "Voiding without difficulty, continent, no pain or burning"));
  else
    unchecked_ids.insert(lookupTables->IdOf("genuri_types", "Voiding without difficulty, continent, no pain or burning"));

  // Look at all IDs:

  // * If the ID is checked, and the ID is in the editing map, do an update on that record.
  // * If the ID is checked, and the ID is NOT in the editing map, do an insert.
  for(psetstr::iterator i_checked = checked_ids.begin(); i_checked != checked_ids.end(); ++i_checked)
  {
    string comment;

    if(*i_checked == lookupTables->IdOf("genuri_types", "Other"))
      comment = memoComments->Text.c_str();
    else if(*i_checked == lookupTables->IdOf("genuri_types", "Catheter"))
      comment = memoCatheter->Text.c_str();
    else if(*i_checked == lookupTables->IdOf("genuri_types", "Frequency"))
      comment = editFrequency->Text.c_str();
    else if(*i_checked == lookupTables->IdOf("genuri_types", "Urgency"))
      comment = editUrgency->Text.c_str();

    // jhaley 20110629 - abstracted logic into parent class
    record_idx = IsInVecMap(editing_genuri_items, "genuri_id", *i_checked);
    if(record_idx >= 0)
    {
      // Do Update because this genuri assessment item already exists.
      if(!UpdateGenUriItem(record_idx, comment))
        return false;
    }
    else
    {
      // Do Insert because this genuri assessment item doesn't exist.
      if(!InsertGenUriItem(*i_checked, comment))
        return false;
    }
  }

  // * If the ID is NOT checked, and the id is in the editing map, mark that item as Deleted via the current_id.
  // * If the ID is NOT checked, and the id is NOT in the editing map, do nothing at all.
  for(psetstr::iterator i_unchecked = unchecked_ids.begin(); i_unchecked != unchecked_ids.end(); ++i_unchecked)
  {
    record_idx = IsInVecMap(editing_genuri_items, "genuri_id", *i_unchecked);
    if(record_idx >= 0)
    {
      // Do Update and set the existing genuri assessment item to 'Deleted'.
      if(!DeleteGenUriItem(record_idx))
        return false;
    }
    // Otherwise, we neither had a record to begin with, nor do we have one to insert now, so do nothing.
  }

  // If we get this far, nothing borked up above.
  return true;
}
//----------------------------------------------------------------------------------------------------------------------
// SetViewMode - Overridden to activate viewing of genuri assessment item information
void TframeAssessmentGenitourinary::SetViewMode()
{
  TframeAssessmentGeneric::SetViewMode();

  panelGenuri->Enabled = false;                   // It is still not possible to enter data.
  editing_genuri_items.clear();                   // Clear out the items vecmap for safety
  urine_filter.UncheckAll();
  condition_filter.UncheckAll();
  detail_filter.UncheckAll();
  checkNPI->Checked = false;

  ClearControlsText(panelGenuri);
}
//----------------------------------------------------------------------------------------------------------------------
// SetEditMode - Overridden to activate editing of genuri assessment items
void TframeAssessmentGenitourinary::SetEditMode()
{
  TframeAssessmentGeneric::SetEditMode();

  panelGenuri->Enabled = true;               // Enable panel controls
}
//----------------------------------------------------------------------------------------------------------------------
// FilterAddGenUri - "Add" handler for the genuri items filter. This actually just activates editing mode here.
void TframeAssessmentGenitourinary::FilterAddGenUri()
{
  if(btnUnlockSave->Enabled && frame_mode == mode_viewing)
    btnUnlockSaveClick(NULL);
}
//----------------------------------------------------------------------------------------------------------------------
// FilterEditGenUri - Same as above.
void TframeAssessmentGenitourinary::FilterEditGenUri()
{
  if(btnUnlockSave->Enabled && frame_mode == mode_viewing)
    btnUnlockSaveClick(NULL);
}
//----------------------------------------------------------------------------------------------------------------------
// AtlasNotify - Overridden to refresh genuri items listview
bool TframeAssessmentGenitourinary::AtlasNotify(int request_num, string matched_value)
{
  bool handled = false;

  if(request_num == genuri_request)
  {
    genuri_filter.RefreshDataSet();
    handled = true;
  }

  return handled;
}
//----------------------------------------------------------------------------------------------------------------------
// SendAtlas - Overridden to send out notifications for genuri_items
void TframeAssessmentGenitourinary::SendAtlas()
{
  atlas->Notify("genuri_items.task_id", task_id); // Is this good enough?
}
//----------------------------------------------------------------------------------------------------------------------
// WriteReport - Write out a report on the genitourinary conditions
bool TframeAssessmentGenitourinary::WriteReport(TComponent *compositeReportCtrl)
{
  bool success = false;

  try
  {
    string sql =
      "select GT.item item_type, GVT.item value_type, GI.comments "
      "from genuri_items GI "
      "left join genuri_types       GT  on GI.genuri_id       = GT.id "
      "left join genuri_value_types GVT on GT.genuri_value_id = GVT.id "
      "where GI.current_id >= 3 and GI.task_id = '" + task_id + "' ";

    InitDataSet(sql);

    TformReportAssessment *reportForm = report_data.reportForm;
    TQuickRep *thisReport = reportForm->reportGenuri;

    thisReport->DataSet = report_data.dbDataSet;
    SetupQRDBText(reportForm->txtGenuriItem,   "item_type");
    SetupQRDBText(reportForm->txtGenuriCat,    "value_type");
    SetupQRDBText(reportForm->txtGenuriDetail, "comments");

    // NB: Need to pass it as TComponent to avoid problems with having QuickReport included in the headers
    TQRCompositeReport *compositeReport = dynamic_cast<TQRCompositeReport *>(compositeReportCtrl);
    if(compositeReport)
    {
      compositeReport->Reports->Add(thisReport);
      success = true;
    }
  }
  catch(...)
  {
  }

  return success;
}
//----------------------------------------------------------------------------------------------------------------------
void __fastcall TframeAssessmentGenitourinary::checkNPIClick(TObject *Sender)
{
  if(frame_mode == mode_editing)
    SetModified(true);
}
//----------------------------------------------------------------------------------------------------------------------
void __fastcall TframeAssessmentGenitourinary::listUrineAppearanceClick(TObject *Sender)
{
  FilterCheckBoxHitTest(listUrineAppearance);
}
//----------------------------------------------------------------------------------------------------------------------
void __fastcall TframeAssessmentGenitourinary::listConditionClick(TObject *Sender)
{
  FilterCheckBoxHitTest(listCondition);
}
//----------------------------------------------------------------------------------------------------------------------
void __fastcall TframeAssessmentGenitourinary::listDetailedClick(TObject *Sender)
{
  FilterCheckBoxHitTest(listDetailed);
}
//----------------------------------------------------------------------------------------------------------------------
