//----------------------------------------------------------------------------------------------------------------------

#include <vcl.h>
#pragma hdrstop

#include "codeFrameAssessmentIVs.h"
#include "codeRecordPatientAssessment.h"
//----------------------------------------------------------------------------------------------------------------------
#pragma package(smart_init)
#pragma link "codeFrameAssessmentGeneric"
#pragma resource "*.dfm"
TframeAssessmentIVs *frameAssessmentIVs;
//----------------------------------------------------------------------------------------------------------------------
__fastcall TframeAssessmentIVs::TframeAssessmentIVs(TComponent* Owner)
        : TframeAssessmentGeneric(Owner), iv_filter(), editing_iv(), ending_state(), iv_request(-1), editing_id(),
          primary_iv_id()
{
}
//----------------------------------------------------------------------------------------------------------------------
// Initialize - Overridden to set parent properties and setup education filtered listview.
bool TframeAssessmentIVs::Initialize()
{
  // Set AssessmentGeneric properties
  SetBandCaption("Intravenous");
  SetPermission("intravenous_items");

  // Setup IV filter
  iv_filter.SetListViewHandle(listIVs);
  iv_filter.SetSecurity("intravenous_items");
  iv_filter.AddColumnSpec("Type",        "intravenous_items", as_text, true,   80);
  iv_filter.AddColumnSpec("Site",        "intravenous_items", as_text, true,   80, rawString);
  iv_filter.AddColumnSpec("Start Date",  "intravenous_items", as_date, true,   75);
  iv_filter.AddColumnSpec("Change Date", "intravenous_items", as_date, true,   75);
  iv_filter.AddColumnSpec("Fluid",       "intravenous_items", as_text, true,   80, rawString);
  iv_filter.AddColumnSpec("Rate",        "intravenous_items", as_int,  true,   40);
  iv_filter.AddColumnSpec("Size",        "intravenous_items", as_int,  true,   40);
  iv_filter.AddColumnSpec("Location",    "intravenous_items", as_text, true,  100, rawString);
  iv_filter.AddColumnSpec("Device",      "intravenous_items", as_text, true,  100, rawString);
  iv_filter.AddColumnSpec("Dressing",    "intravenous_items", as_text, true,  100, rawString);
  iv_filter.AddColumnSpec("Primary",     "intravenous_items", as_text, true,   60, prettyBoolString);
  iv_filter.AddColumnSpec("Current Id",  "current_id",        as_int,  false,  80);
  iv_filter.AddColumnSpec("Internal Id", "internal_id",       as_int,  false,  80);
  iv_filter.FetchPreferences("assessment.ivs.filter");
  iv_filter.SetSQLStatement("");
  iv_filter.RefreshDataSet();

  // Register lookups
  string lookupName = "ivs.cboIVType";
  string tableName = "iv_types";
  RegisterLookup(lookupName, tableName, cboIVType);

  lookupName = "ivs.cboIVSite";
  tableName  = "iv_site_types";
  RegisterLookup(lookupName, tableName, cboIVSite);

  // Set standard modified event handlers for all controls on panelIVs
  SetAllEventHandlers(panelIVs);

  staticIVLink->Font->Color = link_color;

  return TframeAssessmentGeneric::Initialize();
}
//----------------------------------------------------------------------------------------------------------------------
// Lock - Overridden to lock Add button and IV panel
void TframeAssessmentIVs::Lock()
{
  TframeAssessmentGeneric::Lock(); // lock parent controls
  btnAdd->Enabled = false;         // disable add button
  panelIVs->Enabled = false;       // lock all controls
}
//----------------------------------------------------------------------------------------------------------------------
// TaskIdChanging - Overridden to populate IV filter
void TframeAssessmentIVs::TaskIdChanging(const string &new_id)
{
  string sql;

  TframeAssessmentGeneric::TaskIdChanging(new_id);

  sql = "select IVT.item, IVS.item, IVI.start_date, IVI.change_date, IVI.fluid, IVI.rate, IVI.size, IVI.location, "
        " IVI.device, IVI.dressing, IVI.is_primary, IVI.current_id, IVI.id "
        "from intravenous_items IVI "
        "left join iv_types      IVT on IVI.iv_type_id = IVT.id "
        "left join iv_site_types IVS on IVI.iv_site_id = IVS.id "
        "where IVI.current_id >= 2 and IVI.task_id = '" + new_id + "' ";  // NB: All can be seen @ person record (someday?)

  iv_filter.SetSQLStatement(sql);
  iv_filter.RefreshDataSet();
  iv_filter.SetAddEdit(this, AddRecord (&TframeAssessmentIVs::FilterAddIV),
                             EditRecord(&TframeAssessmentIVs::FilterEditIV));

  iv_request = parent_form->RequestNotificationWhen("intravenous_items.task_id", new_id, rel_any, as_int);
}
//----------------------------------------------------------------------------------------------------------------------
// LockRecords - Overridden to lock intravenous_items record for editing.
bool TframeAssessmentIVs::LockRecords()
{
  if(!LockRecordForUpdate(generic_transaction, "intravenous_items", editing_id))
    return false;

  return TframeAssessmentGeneric::LockRecords();
}
//----------------------------------------------------------------------------------------------------------------------
// CanUnlock - Overridden to test for a selected intravenous item.
bool TframeAssessmentIVs::CanUnlock(bool &creating_new)
{
  editing_id = iv_filter.GetSelectedRecordID();

  if(!creating_new && editing_id == "")
  {
    parent_form->PSay("Please select an intravenous item to edit in the list first.", m_reminder, MB_OK);
    return false;
  }

  return TframeAssessmentGeneric::CanUnlock(creating_new);
}
//----------------------------------------------------------------------------------------------------------------------
// UpdateForEdit - Overridden to put fresh data into the editing_iv map and on screen.
void TframeAssessmentIVs::UpdateForEdit()
{
  string sql = "select * from intravenous_items where id = '" + editing_id + "' ";

  if(!SqlToMap(generic_transaction, sql, editing_iv))
    parent_form->PSay("Could not retrieve up-to-date information for intravenous item", m_warning, MB_OK);

  string iv_type      = lookupTables->ValueOf("iv_types",      editing_iv["iv_type_id"]);
  string iv_site_type = lookupTables->ValueOf("iv_site_types", editing_iv["iv_site_id"]);
  cboIVType->Text     = iv_type.c_str();
  cboIVSite->Text     = iv_site_type.c_str();

  editStartDate->Text  = editing_iv["start_date" ].c_str();
  editChangeDate->Text = editing_iv["change_date"].c_str();
  editLocation->Text   = editing_iv["location"   ].c_str();
  editDevice->Text     = editing_iv["device"     ].c_str();
  editDressing->Text   = editing_iv["dressing"   ].c_str();
  editFluid->Text      = editing_iv["fluid"      ].c_str();
  editRate->Text       = editing_iv["rate"       ].c_str();
  editSize->Text       = editing_iv["size"       ].c_str();

  checkPrimary->Checked = (editing_iv["is_primary"] == "1");

  primary_iv_id = editing_iv["primary_iv_id"];
  if(!IsBlankOrZero(primary_iv_id))
    staticIVLink->Caption = (string("IV #") + primary_iv_id).c_str();
  else
    staticIVLink->Caption = "None";

  ClearControlsModified(panelIVs);

  TframeAssessmentGeneric::UpdateForEdit();
}
//----------------------------------------------------------------------------------------------------------------------
// SavingNew - Overridden to return status of editing_iv as new record or unlocked record
bool TframeAssessmentIVs::SavingNew()
{
  return editing_iv["id"] == "new";
}
//----------------------------------------------------------------------------------------------------------------------
// IsValidOnScreen - Overridden to validate lookups
bool TframeAssessmentIVs::IsValidOnScreen()
{
  if(cboIVType->Text == "" || !lookupTables->IsInLookUpTable("iv_types", cboIVType))
  {
    parent_form->PSay("Please enter a valid IV Type", m_invalid_data, MB_OK);
    return false;
  }
  if(cboIVSite->Text == "" || !lookupTables->IsInLookUpTable("iv_site_types", cboIVSite))
  {
    parent_form->PSay("Please enter a valid IV Site", m_invalid_data, MB_OK);
    return false;
  }

  if(editStartDate->Text != "" && !Pdate(editStartDate->Text.c_str(), mdy | yy).IsValidDate())
  {
    parent_form->PSay("Please enter a valid Start Date", m_invalid_data, MB_OK);
    return false;
  }
  if(editChangeDate->Text != "" && !Pdate(editChangeDate->Text.c_str(), mdy | yy).IsValidDate())
  {
    parent_form->PSay("Please enter a valid Change Date", m_invalid_data, MB_OK);
    return false;
  }

  if(editFluid->Text == "")
  {
    parent_form->PSay("A description of the Fluid solution in use is required", m_invalid_data, MB_OK);
    return false;
  }
  if(editRate->Text == "" || !IsInt(editRate->Text.c_str()))
  {
    parent_form->PSay("Please enter the Drip Rate in whole drips per minute (do not include a slash or decimal point)",
                      m_invalid_data, MB_OK);
    return false;
  }
  if(editSize->Text == "" || !IsInt(editSize->Text.c_str()))
  {
    parent_form->PSay("Please enter the Size of the fluid bag in whole milliliters (do not include units or a decimal point)",
                      m_invalid_data, MB_OK);
    return false;
  }

  if(checkPrimary->Checked && !IsBlankOrZero(primary_iv_id))
  {
    parent_form->PSay("A primary IV should not have another IV set as its primary. Either:\n"
                      "1. Uncheck the \"Is Primary IV\" checkbox\n"
                      "2. Clear the \"Secondary To\" link using the \"Clear\" button.",
                      m_invalid_data, MB_OK);
    return false;
  }

  if(!checkPrimary->Checked && IsBlankOrZero(primary_iv_id))
  {
    parent_form->PSay("You neither marked this record as a primary IV, nor set its primary IV number. Please do one or "
                      "the other.", m_missing_data, MB_OK);
    return false;
  }

  return TframeAssessmentGeneric::IsValidOnScreen();
}
//----------------------------------------------------------------------------------------------------------------------
// DoSaveData - Overridden to provide core saving logic.
bool TframeAssessmentIVs::DoSaveData()
{
  pmapstrtoint leave_alone;
  ending_state = editing_iv;

  ending_state["current_id"]        = lookupTables->IdOf("current_types", "Current");
  ending_state["last_update_stamp"] = RightNow();
  ending_state["last_updated_by"]   = current_userid;

  // if creating new, set created_by/creation_stamp too
  if(editing_iv["id"] == "new")
  {
    ending_state["creation_stamp"] = ending_state["last_update_stamp"];
    ending_state["created_by"]     = ending_state["last_updated_by"];
  }

  ending_state["iv_type_id"]        = lookupTables->IdOf("iv_types",      cboIVType);
  ending_state["iv_site_id"]        = lookupTables->IdOf("iv_site_types", cboIVSite);
  ending_state["size"]              = editSize->Text.c_str();
  ending_state["start_date"]        = editStartDate->Text != "" ?
                                        Pdate(editStartDate->Text.c_str(), mdy | yy).OutDate(mdy | yyyy).c_str() : "";
  ending_state["change_date"]       = editChangeDate->Text != "" ?
                                        Pdate(editChangeDate->Text.c_str(), mdy | yy).OutDate(mdy | yyyy).c_str() : "";
  ending_state["location"]          = editLocation->Text.c_str();
  ending_state["device"]            = editDevice->Text.c_str();
  ending_state["dressing"]          = editDressing->Text.c_str();
  ending_state["fluid"]             = editFluid->Text.c_str();
  ending_state["rate"]              = editRate->Text.c_str();
  ending_state["is_primary"]        = checkPrimary->Checked ? "1" : "0";
  ending_state["primary_iv_id"]     = checkPrimary->Checked ? "0" : primary_iv_id.c_str();

  // Not seen:
  // task_id   - this either comes from the existing iv item or is set in editing_iv on add

  leave_alone["location"] = sql_no_uppercasing;
  leave_alone["device"]   = sql_no_uppercasing;
  leave_alone["dressing"] = sql_no_uppercasing;
  leave_alone["fluid"]    = sql_no_uppercasing;

  return DoInsertOrUpdate(ending_state, leave_alone, "intravenous_items");
}
//----------------------------------------------------------------------------------------------------------------------
// SetViewMode - Overridden to activate viewing of intravenous items
void TframeAssessmentIVs::SetViewMode()
{
  TframeAssessmentGeneric::SetViewMode();
  btnAdd->Enabled = HaveUnlockPermission(true); // It is now possible to add an IV
  panelIVs->Enabled = false;                    // But it is still not possible to enter data.
  editing_iv.clear();                           // Clear out the IV map for safety
  primary_iv_id = "0";

  // Reset control contents
  ClearControlsText(panelIVs);
  ClearControlsModified(panelIVs);

  staticIVLink->Caption = "None";
  checkPrimary->Checked = false;
}
//----------------------------------------------------------------------------------------------------------------------
// SetEditMode - Overridden to activate editing of an intravenous item
void TframeAssessmentIVs::SetEditMode()
{
  TframeAssessmentGeneric::SetEditMode();
  btnAdd->Enabled = false;                // Can't add IV items right now
  panelIVs->Enabled = true;               // Enable panel controls
}
//----------------------------------------------------------------------------------------------------------------------
// FilterAddIV - "Add" handler for the IV filter. We redirect this to the "Add an Item" code so that it
// works compatibly with the assessment frame.
void TframeAssessmentIVs::FilterAddIV()
{
  if(btnAdd->Enabled) // Only when you could add using the button.
    btnAddClick(NULL);
}
//----------------------------------------------------------------------------------------------------------------------
// FilterEditIV - As above, but for editing the selected record from the listview.
void TframeAssessmentIVs::FilterEditIV()
{
  if(btnUnlockSave->Enabled && frame_mode == mode_viewing) // Only when you can start editing via the button.
    btnUnlockSaveClick(NULL);
}
//----------------------------------------------------------------------------------------------------------------------
// AtlasNotify - Overridden to refresh edu listview
bool TframeAssessmentIVs::AtlasNotify(int request_num, string matched_value)
{
  bool handled = false;

  if(request_num == iv_request)
  {
    iv_filter.RefreshDataSet();
    handled = true;
  }

  return handled;
}
//----------------------------------------------------------------------------------------------------------------------
// SendAtlas - Overridden to send out notifications for intravenous_items
void TframeAssessmentIVs::SendAtlas()
{
  bool new_record = (editing_iv["id"] == "new");

  SendAtlasIfChanged(new_record, "intravenous_items", editing_iv, ending_state);
}
//----------------------------------------------------------------------------------------------------------------------
// WriteReport - Write out a report on the IVs
bool TframeAssessmentIVs::WriteReport(TComponent *compositeReportCtrl)
{
  bool success = false;

  try
  {
    string ivs_sql =
      "select IVI.id, IVT.item iv_type, IVS.item iv_site, IVI.start_date, IVI.change_date, IVI.fluid, IVI.rate, "
      " IVI.size, IVI.location, IVI.device, IVI.dressing, "
      " case when IVI.is_primary = 1 then 'X' else '' end is_primary_iv, "
      " case when IVI.primary_iv_id is null or IVI.primary_iv_id = 0 then "
      "   'None' else ('IV #' || IVI.primary_iv_id) end primary_iv "
      "from intravenous_items IVI "
      "left join iv_types      IVT on IVI.iv_type_id = IVT.id "
      "left join iv_site_types IVS on IVI.iv_site_id = IVS.id "
      "where IVI.current_id >= 3 and IVI.task_id = '" + task_id + "' ";

    InitDataSet(ivs_sql);

    TformReportAssessment *reportForm = report_data.reportForm;
    TQuickRep *thisReport = reportForm->reportIVs;

    thisReport->DataSet = report_data.dbDataSet;
    SetupQRDBText(reportForm->txtIVType,   "iv_type");
    SetupQRDBText(reportForm->txtIVSite,   "iv_site");
    SetupQRDBText(reportForm->txtIVStart,  "start_date");
    SetupQRDBText(reportForm->txtIVChg,    "change_date");
    SetupQRDBText(reportForm->txtIVLoc,    "location");
    SetupQRDBText(reportForm->txtIVDevice, "device");
    SetupQRDBText(reportForm->txtIVDress,  "dressing");
    SetupQRDBText(reportForm->txtIVCheck,  "is_primary_iv");
    SetupQRDBText(reportForm->txtIVIDNo,   "id");
    SetupQRDBText(reportForm->txtIVSecTo,  "primary_iv");
    SetupQRDBText(reportForm->txtIVFluid,  "fluid");
    SetupQRDBText(reportForm->txtIVDrip,   "rate");
    SetupQRDBText(reportForm->txtIVSize,   "size");

    // NB: Need to pass it as TComponent to avoid problems with having QuickReport included in the headers
    TQRCompositeReport *compositeReport = dynamic_cast<TQRCompositeReport *>(compositeReportCtrl);
    if(compositeReport)
    {
      compositeReport->Reports->Add(thisReport);
      success = true;
    }
  }
  catch(...)
  {
  }

  return success;
}
//----------------------------------------------------------------------------------------------------------------------
void __fastcall TframeAssessmentIVs::btnAddClick(TObject *Sender)
{
  if(DoUnlock(true))
  {
    // Reset control contents
    ClearControlsText(panelIVs);
    ClearControlsModified(panelIVs);

    staticIVLink->Caption = "None";
    checkPrimary->Checked = false;

    editing_iv.clear();
    primary_iv_id = "0";
    editing_iv["id"] = "new";        // Creating a new intravenous_items record
    editing_iv["task_id"] = task_id; // Set task id
  }
}
//----------------------------------------------------------------------------------------------------------------------
void __fastcall TframeAssessmentIVs::btnSetPrimaryClick(TObject *Sender)
{
  string iv_id = iv_filter.GetSelectedRecordID();

  if(checkPrimary->Checked)
  {
    parent_form->PSay("You have indicated that this IV is a primary IV, so it cannot have a primary itself.",
                      m_reminder, MB_OK);
    return;
  }

  if(IsBlankOrZero(iv_id))
  {
    parent_form->PSay("Please select an IV in the list above that is the parent of this IV record.", m_reminder, MB_OK);
    return;
  }

  if(iv_id == editing_iv["id"])
  {
    parent_form->PSay("An IV cannot be its own primary IV; please make a different selection.", m_invalid_data, MB_OK);
    return;
  }

  primary_iv_id = iv_id;
  staticIVLink->Caption = primary_iv_id.c_str();
  SetModified(true);
}
//----------------------------------------------------------------------------------------------------------------------
void __fastcall TframeAssessmentIVs::staticIVLinkClick(TObject *Sender)
{
  if(IsBlankOrZero(primary_iv_id))
    return;

  bool success = iv_filter.SetSelectedRecordID(primary_iv_id);
  if(success)
    listIVs->SetFocus();
}
//----------------------------------------------------------------------------------------------------------------------
void __fastcall TframeAssessmentIVs::btnClearClick(TObject *Sender)
{
  primary_iv_id = "0";
  staticIVLink->Caption = "None";
  SetModified(true);
}
//----------------------------------------------------------------------------------------------------------------------
void __fastcall TframeAssessmentIVs::checkPrimaryClick(TObject *Sender)
{
  if(frame_mode == mode_editing)
    SetModified(true);
}
//----------------------------------------------------------------------------------------------------------------------

