//----------------------------------------------------------------------------------------------------------------------

#include <vcl.h>
#pragma hdrstop

#include "codeFrameAssessmentLimits.h"
#include "codeRecordPatientAssessment.h"
//----------------------------------------------------------------------------------------------------------------------
#pragma package(smart_init)
#pragma link "codeFrameAssessmentGeneric"
#pragma resource "*.dfm"
TframeAssessmentLimits *frameAssessmentLimits;
//----------------------------------------------------------------------------------------------------------------------
__fastcall TframeAssessmentLimits::TframeAssessmentLimits(TComponent* Owner)
        : TframeAssessmentGeneric(Owner), limits_filter(), checks_filter(), limits_request(-1), editing_limits_items(),
          final_ending_state()
{
}
//----------------------------------------------------------------------------------------------------------------------
// Initialize - Overridden to set parent properties and setup functional limitations filtered listview.
bool TframeAssessmentLimits::Initialize()
{
  // Set AssessmentGeneric properties
  SetBandCaption("Functional Limits");
  SetPermission("functional_limits");

  // Setup notes filter
  limits_filter.SetListViewHandle(listLimits);
  limits_filter.SetSecurity("functional_limits");

  limits_filter.AddColumnSpec("Item",        "functional_limits", as_text, true,  160, rawString);
  limits_filter.AddColumnSpec("Details",     "functional_limits", as_text, true,  420, rawString);
  limits_filter.AddColumnSpec("Current Id",  "current_id",        as_int,  false,  80);
  limits_filter.AddColumnSpec("Internal Id", "internal_id",       as_int,  false,  80);

  limits_filter.FetchPreferences("assessment.limits.topfilter");
  limits_filter.SetSQLStatement("");
  limits_filter.RefreshDataSet();

  // setup func_limit_types input filter

  string items_sql =
    "select item, id from func_limit_types where valid = 1 and id > 0 and "
    "id <> " + lookupTables->IdOf("func_limit_types", "No Problems Identified"); // don't include the NPI indicator

  // Checks Filter
  checks_filter.SetListViewHandle(listLimitTypes);
  checks_filter.SetSecurity("");
  checks_filter.AddColumnSpec("Item",        "functional_limits", as_text, true,  188, rawString);
  checks_filter.AddColumnSpec("Internal Id", "internal_id",       as_int,  false,  80);
  checks_filter.FetchPreferences("assessment.limits.func_limit_types");
  checks_filter.SetIconStatus(false);
  checks_filter.SetCheck(true);
  checks_filter.SetSQLStatement(items_sql);
  checks_filter.RefreshDataSet();

  // Set edit box change handlers
  SetAllEventHandlers(panelLimits);

  return TframeAssessmentGeneric::Initialize();
}
//----------------------------------------------------------------------------------------------------------------------
// Lock - Overridden to lock Functional Limits panel
void TframeAssessmentLimits::Lock()
{
  TframeAssessmentGeneric::Lock(); // lock parent controls
  panelLimits->Enabled = false;    // lock all panel controls
}
//----------------------------------------------------------------------------------------------------------------------
// PutUpLimitsItems - Refreshes the controls to reflect the state of the functional limitations list
void TframeAssessmentLimits::PutUpLimitsItems(pvecmap &limits_items)
{
  psetstr func_limit_ids;

  // Clear out the checkboxes and edit box
  checks_filter.UncheckAll();
  memoComments->Text = "";
  checkNPI->Checked = false;

  // Analyze the retrieved functional_limits items
  for(size_t i = 0; i < limits_items.size(); i++)
  {
    // Skip items which are anything other than Current. This allows Deleted items to be updated back to Current status
    // when saving.
    if(limits_items[i]["current_id"] != lookupTables->IdOf("current_types", "Current"))
      continue;

    if(limits_items[i]["func_limit_id"] == lookupTables->IdOf("func_limit_types", "No Problems Identified")) // NPI
      checkNPI->Checked = true;
    else
    {
      func_limit_ids.insert(limits_items[i]["func_limit_id"]);

      if(limits_items[i]["func_limit_id"] == lookupTables->IdOf("func_limit_types", "Other"))
        memoComments->Text = limits_items[i]["comments"].c_str();
    }
  }

  // Set checkboxes
  checks_filter.SetCheckedWhereID(func_limit_ids, true);
}
//----------------------------------------------------------------------------------------------------------------------
// TaskIdChanging - Overridden to populate limits filter
void TframeAssessmentLimits::TaskIdChanging(const string &new_id)
{
  string sql;

  TframeAssessmentGeneric::TaskIdChanging(new_id);

  sql = "select FLT.item, FL.comments, FL.current_id, FL.id "
        "from functional_limits FL "
        "left join func_limit_types FLT on FL.func_limit_id = FLT.id "
        "where FL.current_id >= 2 and FL.task_id = '" + new_id + "' ";

  limits_filter.SetSQLStatement(sql);
  limits_filter.RefreshDataSet();
  limits_filter.SetAddEdit(this, AddRecord (&TframeAssessmentLimits::FilterAddLimits),
                                 EditRecord(&TframeAssessmentLimits::FilterEditLimits));
  limits_request = parent_form->RequestNotificationWhen("functional_limits.task_id", new_id, rel_e, as_int);
}
//----------------------------------------------------------------------------------------------------------------------
// UpdateForEdit - Overridden to put fresh data into the editing_limits_items vecmap and on screen.
void TframeAssessmentLimits::UpdateForEdit()
{
  string sql =
    "select * "
    "from functional_limits FL "
    "where FL.task_id = '" + task_id + "' ";

  editing_limits_items.clear();

  if(!SqlToVecMap(generic_transaction, sql, editing_limits_items))
    parent_form->PSay("Could not retrieve up-to-date information for functional_limits items", m_warning, MB_OK);

  // Refresh controls
  PutUpLimitsItems(editing_limits_items);

  TframeAssessmentGeneric::UpdateForEdit();
}
//----------------------------------------------------------------------------------------------------------------------
// LockRecords - Overridden to lock the functional_limits records
bool TframeAssessmentLimits::LockRecords()
{
  // Must acquire all items. Any failure is a failure to unlock for editing, GTFO and rollback.
  if(!LockVecMapRecords(editing_limits_items, "functional_limits"))
    return false;

  return TframeAssessmentGeneric::LockRecords();
}
//----------------------------------------------------------------------------------------------------------------------
// IsValidOnScreen - Overridden to do validation for functional_limits
bool TframeAssessmentLimits::IsValidOnScreen()
{
  pvecstr checked_ids = checks_filter.GetMultiID();
  bool other_is_checked = false;
  
  // Make sure at least one functional limitations item is specified
  if(!checkNPI->Checked &&
     editing_limits_items.size() == 0 && // Only when the record is new, otherwise can't delete last one
     checked_ids.size() == 0)
  {
    parent_form->PSay("You have not specified any saveable data. Please check one or more limitations or the "
                      "'No Problems Identified' indicator.", m_missing_data, MB_OK);
    return false;
  }

  // If NPI is checked, nothing except Other can be specified in addition to it.
  if(checkNPI->Checked && checked_ids.size() > 0)
  {
    for(size_t i = 0; i < checked_ids.size(); i++)
    {
      if(checked_ids[i] != lookupTables->IdOf("func_limit_types", "Other"))
      {
        parent_form->PSay("You indicated that there were 'No Problems Identified', but also checked additional problems "
                          "(not including 'Other', which you can use to provide relevant information even if there "
                          "were no tangible problems).\n\n"
                          "Please uncheck either the 'No Problems Identified' indicator, or all items in the Functional "
                          "Limitations list except 'Other'.", m_invalid_data, MB_OK);
        return false;
      }
    }
  }

  for(size_t i = 0; i < checked_ids.size(); i++)
  {
    // Still have to hard code it >_>
    if(checked_ids[i] == lookupTables->IdOf("func_limit_types", "Other"))
    {
      other_is_checked = true;
      if(memoComments->Text == "")
      {
        parent_form->PSay("You selected 'Other' but did not provide any additional information on the functional "
                          "limitation.", m_missing_data, MB_OK);
        return false;
      }
    }
  }

  if(!other_is_checked && memoComments->Text != "")
  {
    parent_form->PSay("You did not select 'Other', but specified information in the 'Other Information' field. This "
                      "information would be lost, so please either clear it out now, or select the 'Other' item in "
                      "the checkbox list as well.", m_invalid_data, MB_OK);
    return false;
  }

  return TframeAssessmentGeneric::IsValidOnScreen();
}
//----------------------------------------------------------------------------------------------------------------------
bool TframeAssessmentLimits::UpdateLimitsItem(int record_index, string comment)
{
  pmapstrs ending_state;
  pmapstrtoint leave_alone;

  ending_state["id"]                 = editing_limits_items[record_index]["id"];
  ending_state["current_id"]         = lookupTables->IdOf("current_types", "Current");
  ending_state["last_updated_by"]    = current_userid;
  ending_state["last_update_stamp"]  = RightNow();
  ending_state["task_id"]            = task_id;
  ending_state["func_limit_id"]      = editing_limits_items[record_index]["func_limit_id"];
  ending_state["comments"]           = comment;

  leave_alone["comments"] = sql_no_uppercasing;

  final_ending_state.push_back(ending_state);

  return ExecuteUpdateStatement(generic_transaction, "functional_limits", ending_state, leave_alone);
}
//----------------------------------------------------------------------------------------------------------------------
bool TframeAssessmentLimits::InsertLimitsItem(string func_limit_id, string comment)
{
  pmapstrs ending_state;
  pmapstrtoint leave_alone;

  ending_state["id"]                = GetNextId(generic_transaction, "functional_limits");
  ending_state["current_id"]        = lookupTables->IdOf("current_types", "Current");
  ending_state["last_updated_by"]   = current_userid;
  ending_state["last_update_stamp"] = RightNow();
  ending_state["created_by"]        = current_userid;
  ending_state["creation_stamp"]    = ending_state["last_update_stamp"];
  ending_state["task_id"]           = task_id;
  ending_state["func_limit_id"]     = func_limit_id;
  ending_state["comments"]          = comment;

  leave_alone["comments"] = sql_no_uppercasing;

  final_ending_state.push_back(ending_state);

  return ExecuteInsertStatement(generic_transaction, "functional_limits", ending_state, leave_alone);
}
//----------------------------------------------------------------------------------------------------------------------
bool TframeAssessmentLimits::DeleteLimitsItem(int record_index)
{
  pmapstrs ending_state;
  pmapstrtoint leave_alone;

  ending_state["id"]                = editing_limits_items[record_index]["id"];
  ending_state["current_id"]        = lookupTables->IdOf("current_types", "Deleted"); // Mark as Deleted
  ending_state["last_updated_by"]   = current_userid;
  ending_state["last_update_stamp"] = RightNow();

  final_ending_state.push_back(ending_state);

  return ExecuteUpdateStatement(generic_transaction, "functional_limits", ending_state, leave_alone);
}
//----------------------------------------------------------------------------------------------------------------------
// DoSaveData - Overridden to provide core saving logic.
bool TframeAssessmentLimits::DoSaveData()
{
  psetstr checked_ids;
  psetstr unchecked_ids;
  int record_idx;
  string npiID = lookupTables->IdOf("func_limit_types", "No Problems Identified");

  final_ending_state.clear();

  // First determine what is checked and what is not in the filters.
  GetIDsFromCheckBoxFilter(checks_filter, checked_ids, unchecked_ids); // use parent class utility method

  // Add in the NPI
  if(checkNPI->Checked)
    checked_ids.insert(npiID);
  else
    unchecked_ids.insert(npiID);

  // Look at all IDs:

  // * If the ID is checked, and the ID is in the editing map, do an update on that record.
  // * If the ID is checked, and the ID is NOT in the editing map, do an insert.
  for(psetstr::iterator i_checked = checked_ids.begin(); i_checked != checked_ids.end(); ++i_checked)
  {
    string comments;

    if(*i_checked == lookupTables->IdOf("func_limit_types", "Other"))
      comments = memoComments->Text.c_str();

    record_idx = IsInVecMap(editing_limits_items, "func_limit_id", *i_checked);
    if(record_idx >= 0)
    {
      // Do Update because this functional_limits item already exists.
      if(!UpdateLimitsItem(record_idx, comments))
        return false;
    }
    else
    {
      // Do Insert because this functional_limits item doesn't exist.
      if(!InsertLimitsItem(*i_checked, comments))
        return false;
    }
  }

  // * If the ID is NOT checked, and the id is in the editing map, mark that item as Deleted via the current_id.
  // * If the ID is NOT checked, and the id is NOT in the editing map, do nothing at all.
  for(psetstr::iterator i_unchecked = unchecked_ids.begin(); i_unchecked != unchecked_ids.end(); ++i_unchecked)
  {
    record_idx = IsInVecMap(editing_limits_items, "func_limit_id", *i_unchecked);
    if(record_idx >= 0)
    {
      // Do Update and set the existing functional_limits record to 'Deleted'.
      if(!DeleteLimitsItem(record_idx))
        return false;
    }
    // Otherwise, we neither had a record to begin with, nor do we have one to insert now, so do nothing.
  }

  // If we get this far, nothing borked up above.
  return true;
}
//----------------------------------------------------------------------------------------------------------------------
// SetViewMode - Overridden to activate viewing of functional_limits information
void TframeAssessmentLimits::SetViewMode()
{
  TframeAssessmentGeneric::SetViewMode();

  panelLimits->Enabled = false;                    // It is still not possible to enter data.
  editing_limits_items.clear();                    // Clear out the items vecmap for safety
  checks_filter.UncheckAll();
  memoComments->Text = "";
  checkNPI->Checked = false;

  ClearControlsModified(panelLimits);
}
//----------------------------------------------------------------------------------------------------------------------
// SetEditMode - Overridden to activate editing of functional_limits records
void TframeAssessmentLimits::SetEditMode()
{
  TframeAssessmentGeneric::SetEditMode();

  panelLimits->Enabled = true;               // Enable panel controls

  ClearControlsModified(panelLimits);
}
//----------------------------------------------------------------------------------------------------------------------
// FilterAddLimits - "Add" handler for the functional_limits filter. This actually just activates editing mode here.
void TframeAssessmentLimits::FilterAddLimits()
{
  if(btnUnlockSave->Enabled && frame_mode == mode_viewing)
    btnUnlockSaveClick(NULL);
}
//----------------------------------------------------------------------------------------------------------------------
// FilterEditLimits - Same as above.
void TframeAssessmentLimits::FilterEditLimits()
{
  if(btnUnlockSave->Enabled && frame_mode == mode_viewing)
    btnUnlockSaveClick(NULL);
}
//----------------------------------------------------------------------------------------------------------------------
// AtlasNotify - Overridden to refresh functional_limits listview
bool TframeAssessmentLimits::AtlasNotify(int request_num, string matched_value)
{
  bool handled = false;

  if(request_num == limits_request)
  {
    limits_filter.RefreshDataSet();
    handled = true;
  }

  return handled;
}
//----------------------------------------------------------------------------------------------------------------------
// SendAtlas - Overridden to send out notifications for functional_limits
void TframeAssessmentLimits::SendAtlas()
{
  atlas->Notify("functional_limits.task_id", task_id); // Is this good enough?
}
//----------------------------------------------------------------------------------------------------------------------
// WriteReport - Write out a report on the functional limitation conditions
bool TframeAssessmentLimits::WriteReport(TComponent *compositeReportCtrl)
{
  bool success = false;

  try
  {
    string sql =
      "select FLT.item, FL.comments "
      "from functional_limits FL "
      "left join func_limit_types FLT on FL.func_limit_id = FLT.id "
      "where FL.current_id >= 3 and FL.task_id = '" + task_id + "' ";

    InitDataSet(sql);

    TformReportAssessment *reportForm = report_data.reportForm;
    TQuickRep *thisReport = reportForm->reportLimits;

    thisReport->DataSet = report_data.dbDataSet;
    SetupQRDBText(reportForm->txtLimitsItem,   "item");
    SetupQRDBText(reportForm->txtLimitsDetail, "comments");

    // NB: Need to pass it as TComponent to avoid problems with having QuickReport included in the headers
    TQRCompositeReport *compositeReport = dynamic_cast<TQRCompositeReport *>(compositeReportCtrl);
    if(compositeReport)
    {
      compositeReport->Reports->Add(thisReport);
      success = true;
    }
  }
  catch(...)
  {
  }

  return success;
}
//----------------------------------------------------------------------------------------------------------------------
void __fastcall TframeAssessmentLimits::listLimitTypesClick(TObject *Sender)
{
  // Use parent class hit test logic
  FilterCheckBoxHitTest(listLimitTypes);
}
//----------------------------------------------------------------------------------------------------------------------
void __fastcall TframeAssessmentLimits::checkNPIClick(TObject *Sender)
{
  if(frame_mode == mode_editing)
    SetModified(true);
}
//----------------------------------------------------------------------------------------------------------------------
// btnReferralClick - Change the assessment form to the referrals area
void __fastcall TframeAssessmentLimits::btnReferralClick(TObject *Sender)
{
  parent_form->ChangeToArea(TformRecordPatientAssessment::area_referrals);
}
//----------------------------------------------------------------------------------------------------------------------
