//----------------------------------------------------------------------------------------------------------------------

#include <vcl.h>
#pragma hdrstop

#include "codeFrameAssessmentMedications.h"
#include "codeRecordPatientAssessment.h"
#include "codeRecordComDrug.h"
#include "codeSearchFDADrug.h"
//----------------------------------------------------------------------------------------------------------------------
#pragma package(smart_init)
#pragma link "codeFrameAssessmentGeneric"
#pragma link "codeHalfButton"
#pragma resource "*.dfm"
TframeAssessmentMedications *frameAssessmentMedications;
//----------------------------------------------------------------------------------------------------------------------
__fastcall TframeAssessmentMedications::TframeAssessmentMedications(TComponent* Owner)
        : TframeAssessmentGeneric(Owner), meds_filter(), editing_med(), ending_state(), meds_request(-1), editing_id()
{
}
//----------------------------------------------------------------------------------------------------------------------
// Initialize - Overridden to set parent properties and setup medications filtered listview.
bool TframeAssessmentMedications::Initialize()
{
  // Set AssessmentGeneric properties
  SetBandCaption("Medications");
  SetPermission("com_drug_regimen");

  // Setup notes filter
  meds_filter.SetListViewHandle(listMeds);
  meds_filter.SetSecurity("com_drug_regimen");

  meds_filter.AddColumnSpec("Medication",   "com_drug_regimen", as_text, true,  130, prettyString);
  meds_filter.AddColumnSpec("Med Type",     "com_drug_regimen", as_text, true,   70, prettyString);
  meds_filter.AddColumnSpec("Frequency",    "com_drug_regimen", as_text, true,   85, prettyString);
  meds_filter.AddColumnSpec("Dosage",       "com_drug_regimen", as_text, true,   70, lowercaseString);
  meds_filter.AddColumnSpec("Rx Number",    "com_drug_regimen", as_int,  false,  70);
  meds_filter.AddColumnSpec("Ordered",      "com_drug_regimen", as_date, false,  90);
  meds_filter.AddColumnSpec("Discontinued", "com_drug_regimen", as_date, false,  90);
  meds_filter.AddColumnSpec("Status",       "com_drug_regimen", as_text, true,   50);
  meds_filter.AddColumnSpec("Purpose",      "com_drug_regimen", as_text, true,  100, rawString);
  meds_filter.AddColumnSpec("Notes",        "com_drug_regimen", as_text, true,  110, rawString);
  meds_filter.AddColumnSpec("Current Id",   "current_id",       as_int,  false,  80);
  meds_filter.AddColumnSpec("Internal Id",  "internal_id",      as_int,  false,  80);

  meds_filter.FetchPreferences("assessment.meds.filter");
  meds_filter.SetSQLStatement("");
  meds_filter.RefreshDataSet();

  // Register lookups
  string lookupName = "meds.cboMedicationType";
  string tableName  = "com_medication_types";
  RegisterLookup(lookupName, tableName, cboMedicationType);

  lookupName = "meds.cboFrequency";
  tableName  = "com_frequency_types";
  RegisterLookup(lookupName, tableName, cboFrequency);

  lookupName = "meds.cboStatus";
  tableName  = "com_status_types";
  RegisterLookup(lookupName, tableName, cboStatus);

  // Set standard modified event handlers for all controls on panelMed
  SetAllEventHandlers(panelMed);

  return TframeAssessmentGeneric::Initialize();
}
//----------------------------------------------------------------------------------------------------------------------
// Lock - Overridden to lock Add button and meds panel
void TframeAssessmentMedications::Lock()
{
  TframeAssessmentGeneric::Lock(); // lock parent controls
  btnAdd->Enabled = false;         // disable add button
  btnLookUpMed->Enabled = false;   // can't lookup meds yet
  panelMed->Enabled = false;       // lock all controls
}
//----------------------------------------------------------------------------------------------------------------------
// TaskIdChanging - Overridden to populate medications filter
void TframeAssessmentMedications::TaskIdChanging(const string &new_id)
{
  string sql;

  TframeAssessmentGeneric::TaskIdChanging(new_id);

  sql = "select CDR.medication, com_medication_types.item, com_frequency_types.item, CDR.dosage, CDR.rx_number, "
        "CDR.date_ordered, CDR.date_discontinued, com_status_types.item, CDR.purpose, CDR.note, CDR.current_id, CDR.id "
        "from com_drug_regimen CDR "
        "left join com_medication_types on CDR.com_medication_id = com_medication_types.id "
        "left join com_frequency_types on CDR.com_frequency_id = com_frequency_types.id "
        "left join com_status_types on CDR.com_status_id = com_status_types.id "
        "where CDR.current_id >= 2 and CDR.task_id = '" + new_id + "' ";  // NB: All can be seen @ person record

  meds_filter.SetSQLStatement(sql);
  meds_filter.RefreshDataSet();
  meds_filter.SetAddEdit(this, AddRecord (&TframeAssessmentMedications::FilterAddMed),
                               EditRecord(&TframeAssessmentMedications::FilterEditMed));
  meds_request = parent_form->RequestNotificationWhen("com_drug_regimen.task_id", new_id, rel_e, as_int);
}
//----------------------------------------------------------------------------------------------------------------------
// LockRecords - Overridden to lock com_drug_regimen record for editing.
bool TframeAssessmentMedications::LockRecords()
{
  if(!LockRecordForUpdate(generic_transaction, "com_drug_regimen", editing_id))
    return false;

  return TframeAssessmentGeneric::LockRecords();
}
//----------------------------------------------------------------------------------------------------------------------
// CanUnlock - Overridden to test for a selected medication
bool TframeAssessmentMedications::CanUnlock(bool &creating_new)
{
  editing_id = meds_filter.GetSelectedRecordID();

  if(!creating_new && editing_id == "")
  {
    parent_form->PSay("Please select a medication to edit in the list first.", m_reminder, MB_OK);
    return false;
  }

  return TframeAssessmentGeneric::CanUnlock(creating_new);
}
//----------------------------------------------------------------------------------------------------------------------
// UpdateForEdit - Overridden to put fresh data into the editing_med map and on screen.
void TframeAssessmentMedications::UpdateForEdit()
{
  string sql = "select * from com_drug_regimen where id = '" + editing_id + "' ";

  if(!SqlToMap(generic_transaction, sql, editing_med))
    parent_form->PSay("Could not retrieve up-to-date information for medication", m_warning, MB_OK);

  string med_type  = PrettyString(lookupTables->ValueOf("com_medication_types", editing_med["com_medication_id"]));
  string freq_type = PrettyString(lookupTables->ValueOf("com_frequency_types",  editing_med["com_frequency_id" ]));
  string stat_type = PrettyString(lookupTables->ValueOf("com_status_types",     editing_med["com_status_id"    ]));

  editMedication->Text       = PrettyString(editing_med["medication"]).c_str();
  cboMedicationType->Text    = med_type.c_str();
  cboFrequency->Text         = freq_type.c_str();
  editDose->Text             = LowercaseString(editing_med["dosage"]).c_str();
  editRxNumber->Text         = editing_med["rx_number"].c_str();
  editDateOrdered->Text      = editing_med["date_ordered"].c_str();
  editDateDiscontinued->Text = editing_med["date_discontinued"].c_str();
  cboStatus->Text            = stat_type.c_str();
  editPurpose->Text          = editing_med["purpose"].c_str();
  editNotes->Text            = editing_med["note"].c_str();

  // Make sure all edits are marked unmodified (C++Builder requires one per line like this for properties 9_9)
  // DONE: NEED A PARENT CLASS METHOD "ClearControls", clearly. (Pun intentional; actual humorousness == low)  
  ClearControlsModified(panelMed);

  TframeAssessmentGeneric::UpdateForEdit();
}
//----------------------------------------------------------------------------------------------------------------------
// SavingNew - Overridden to return status of editing_med as new record or unlocked record
bool TframeAssessmentMedications::SavingNew()
{
  return editing_med["id"] == "new";
}
//----------------------------------------------------------------------------------------------------------------------
// IsValidOnScreen - Overridden to validate lookup and date values
bool TframeAssessmentMedications::IsValidOnScreen()
{
  if(editMedication->Text == "")
  {
    parent_form->PSay("Please specify a medication", m_invalid_data, MB_OK);
    return false;
  }
  if(!lookupTables->IsInLookUpTable("com_medication_types", cboMedicationType))
  {
    parent_form->PSay("Please enter a valid medication type", m_invalid_data, MB_OK);
    return false;
  }
  if(!lookupTables->IsInLookUpTable("com_status_types", cboStatus))
  {
    parent_form->PSay("Please enter a valid status type", m_invalid_data, MB_OK);
    return false;
  }
  if(!lookupTables->IsInLookUpTable("com_frequency_types", cboFrequency))
  {
    parent_form->PSay("Please enter a valid frequency", m_invalid_data, MB_OK);
    return false;
  }
  if(editDose->Text == "")
  {
    parent_form->PSay("Please enter a valid dosage", m_invalid_data, MB_OK);
    return false;
  }

  // Dates are optional, but must actually be dates if specified
  if(editDateOrdered->Text != "" && !Pdate(editDateOrdered->Text.c_str(), mdy|yy).IsValidDate())
  {
    parent_form->PSay("Please enter a valid date ordered", m_invalid_data, MB_OK);
    return false;
  }
  if(editDateDiscontinued->Text != "" && !Pdate(editDateDiscontinued->Text.c_str(), mdy|yy).IsValidDate())
  {
    parent_form->PSay("Please enter a valid date discontinued", m_invalid_data, MB_OK);
    return false;
  }

  return TframeAssessmentGeneric::IsValidOnScreen();
}
//----------------------------------------------------------------------------------------------------------------------
// DoSaveData - Overridden to provide core saving logic.
bool TframeAssessmentMedications::DoSaveData()
{
  pmapstrtoint leave_alone;
  ending_state = editing_med;

  ending_state["current_id"]        = lookupTables->IdOf("current_types", "Current");
  ending_state["last_update_stamp"] = RightNow();
  ending_state["last_updated_by"]   = current_userid;

  ending_state["date_ordered"]      = editDateOrdered->Text != "" ?
                                        Pdate(editDateOrdered->Text.c_str(), mdy|yy).OutDate(mdy|yyyy).c_str() : "";
  ending_state["date_discontinued"] = editDateDiscontinued->Text != "" ?
                                        Pdate(editDateDiscontinued->Text.c_str(), mdy|yy).OutDate(mdy|yyyy).c_str()
                                        : "";
  ending_state["medication"]        = editMedication->Text.c_str();
  ending_state["dosage"]            = editDose->Text.c_str();
  ending_state["com_medication_id"] = lookupTables->IdOf("com_medication_types", cboMedicationType);
  ending_state["com_frequency_id"]  = lookupTables->IdOf("com_frequency_types",  cboFrequency);
  ending_state["rx_number"]         = editRxNumber->Text.c_str();
  ending_state["com_status_id"]     = lookupTables->IdOf("com_status_types",     cboStatus);

  ending_state["purpose"] = editPurpose->Text.c_str();
  ending_state["note"]    = editNotes->Text.c_str();
  leave_alone["purpose"]  = sql_no_uppercasing; // Don't uppercase purpose text
  leave_alone["note"]     = sql_no_uppercasing; // Don't uppercase note text

  // Not seen:
  // person_id - this either comes from the existing med or is set in editing_med on add
  // group_id  - this either comes from the existing med or is set in editing_med on add
  // task_id   - this either comes from the existing med or is set in editing_med on add

  return DoInsertOrUpdate(ending_state, leave_alone, "com_drug_regimen");
}
//----------------------------------------------------------------------------------------------------------------------
// SetViewMode - Overridden to activate viewing of medications
void TframeAssessmentMedications::SetViewMode()
{
  TframeAssessmentGeneric::SetViewMode();
  btnAdd->Enabled = HaveUnlockPermission(true); // It is now possible to add a medication
  btnLookUpMed->Enabled = true;                 // Anybody can lookup meds
  panelMed->Enabled = false;                    // But it is still not possible to enter data.
  editing_med.clear();                          // Clear out the med map for safety

  // Reset control contents
  ClearControlsText(panelMed);
  ClearControlsModified(panelMed);
}
//----------------------------------------------------------------------------------------------------------------------
// SetEditMode - Overridden to activate editing of a medication
void TframeAssessmentMedications::SetEditMode()
{
  TframeAssessmentGeneric::SetEditMode();
  btnAdd->Enabled = false;                // Can't add notes right now
  panelMed->Enabled = true;               // Enable panel controls
}
//----------------------------------------------------------------------------------------------------------------------
// btnAddClick - Handler for toolbar button to add a new medication
void __fastcall TframeAssessmentMedications::btnAddClick(TObject *Sender)
{
  if(DoUnlock(true))
  {
    // Reset control contents
    ClearControlsText(panelMed);
    ClearControlsModified(panelMed);

    editing_med.clear();
    editing_med["id"] = "new"; // Creating a new medication
    editing_med["person_id"] = person_id; // For a new med, task, person, and group are propagated from the
    editing_med["group_id"]  = group_id;  // parent assessment form's tasks.id, tasks.person_id and tasks.group_id
    editing_med["task_id"]   = task_id;
  }
}
//----------------------------------------------------------------------------------------------------------------------
// FilterAddMed - "Add" handler for the meds filter. We redirect this to the "Add a Medication" code so that it
// works compatibly with the assessment frame.
void TframeAssessmentMedications::FilterAddMed()
{
  if(btnAdd->Enabled) // Only when you could add using the button.
    btnAddClick(NULL);
}
//----------------------------------------------------------------------------------------------------------------------
// FilterEditMed - Open the med in a com_drug_regimen record form for ordinary editing capability. Not encouraged,
// but allows adjustment of current_id's etc, which is important if a data entry mistake is made.
void TframeAssessmentMedications::FilterEditMed()
{
  // Open the com_drug_regimen record
  pmapstrs field_map;
  field_map["id"] = meds_filter.GetSelectedRecordID();

  FORM_TRY
    TformRecordComDrug *med_form = new TformRecordComDrug(NULL, field_map);
    med_form->Show();
  FORM_CATCH
}
//----------------------------------------------------------------------------------------------------------------------
// AtlasNotify - Overridden to refresh meds listview
bool TframeAssessmentMedications::AtlasNotify(int request_num, string matched_value)
{
  bool handled = false;

  if(request_num == meds_request)
  {
    meds_filter.RefreshDataSet();
    handled = true;
  }

  return handled;
}
//----------------------------------------------------------------------------------------------------------------------
// SendAtlas - Overridden to send out notifications for com_drug_regimen
void TframeAssessmentMedications::SendAtlas()
{
  bool new_record = (editing_med["id"] == "new");

  SendAtlasIfChanged(new_record, "com_drug_regimen", editing_med, ending_state);
}
//----------------------------------------------------------------------------------------------------------------------
// WriteReport - Write out a report on the medications
bool TframeAssessmentMedications::WriteReport(TComponent *compositeReportCtrl)
{
  bool success = false;

  try
  {
    string meds_sql =
      "select CDR.medication, com_medication_types.item med_item, com_frequency_types.item frequency, CDR.dosage, "
      " CDR.rx_number, CDR.date_ordered, CDR.date_discontinued, com_status_types.item status, CDR.purpose, CDR.note "
      "from com_drug_regimen CDR "
      "left join com_medication_types on CDR.com_medication_id = com_medication_types.id "
      "left join com_frequency_types on CDR.com_frequency_id = com_frequency_types.id "
      "left join com_status_types on CDR.com_status_id = com_status_types.id "
      "where CDR.current_id >= 3 and CDR.task_id = '" + task_id + "' ";

    InitDataSet(meds_sql);

    TformReportAssessment *reportForm = report_data.reportForm;
    TQuickRep *thisReport = report_data.reportForm->reportMeds;

    thisReport->DataSet = report_data.dbDataSet;
    SetupQRDBText(reportForm->txtMedName,    "medication");
    SetupQRDBText(reportForm->txtMedType,    "med_item");
    SetupQRDBText(reportForm->txtMedFreq,    "frequency");
    SetupQRDBText(reportForm->txtMedDose,    "dosage");
    SetupQRDBText(reportForm->txtMedRx,      "rx_number");
    SetupQRDBText(reportForm->txtMedDateO,   "date_ordered");
    SetupQRDBText(reportForm->txtMedDateD,   "date_discontinued");
    SetupQRDBText(reportForm->txtMedStatus,  "status");
    SetupQRDBText(reportForm->txtMedPurpose, "purpose");
    SetupQRDBText(reportForm->txtMedNotes,   "note");

    // NB: Need to pass it as TComponent to avoid problems with having QuickReport included in the headers
    TQRCompositeReport *compositeReport = dynamic_cast<TQRCompositeReport *>(compositeReportCtrl);
    if(compositeReport)
    {
      compositeReport->Reports->Add(thisReport);
      success = true;
    }
  }
  catch(...)
  {
  }

  return success;
}
//----------------------------------------------------------------------------------------------------------------------
// hbtnSearchFDAClick - Allow searching for a medication in the FDA drug database
void __fastcall TframeAssessmentMedications::hbtnSearchFDAClick(TObject *Sender)
{
  pvecstr drug_vec = TformSearchFDADrug::Do(parent_form);

  if(drug_vec.size() > 0)
  {
    editMedication->Text = PrettyString(drug_vec[0]).c_str();
    editMedication->Modified = true;
    SetModified(true);
  }
}
//----------------------------------------------------------------------------------------------------------------------
// btnLookUpMedClick - Toolbar button added to allow looking up an existing med in the FDA drug database
void __fastcall TframeAssessmentMedications::btnLookUpMedClick(TObject *Sender)
{
  string med_text;
  pvecmap all_data = meds_filter.GetAllData(true, true); // selected rows only!

  if(!all_data.empty())
  {
    // Return the IDs combined with the names as string values.
    // This allows a cascading behavior in case IDs should vanish in future versions of the FDA database -
    // we then instead search by name when trying to find the same drug again later. This avoids any hard links
    // between the two independent databases.
    med_text = all_data[0]["medication"];

    string drug_id;   // ID, assuming there is one in the string
    string drug_name; // name, used if there's no ID
    int as_int = StringToInt(med_text);
    if(as_int > 0)
      drug_id = IntToString(as_int);
    else
    {
      size_t first_letter = med_text.find_first_not_of("0123456789- ");
      if(first_letter != string::npos)
        drug_name = med_text.substr(first_letter);
    }

    TformSearchFDADrug::Do(drug_id, drug_name);
  }
  else
    TformSearchFDADrug::Do("", "");
}
//----------------------------------------------------------------------------------------------------------------------

