//----------------------------------------------------------------------------------------------------------------------

#include <vcl.h>
#pragma hdrstop

#include "codeFrameAssessmentMuscle.h"
#include "codeRecordPatientAssessment.h"
//----------------------------------------------------------------------------------------------------------------------
#pragma package(smart_init)
#pragma link "codeFrameAssessmentGeneric"
#pragma resource "*.dfm"
TframeAssessmentMuscle *frameAssessmentMuscle;
//----------------------------------------------------------------------------------------------------------------------
__fastcall TframeAssessmentMuscle::TframeAssessmentMuscle(TComponent* Owner)
        : TframeAssessmentGeneric(Owner), muscle_filter(), checks_filter(), muscle_request(-1), editing_muscle_items(),
          final_ending_state()
{
}
//----------------------------------------------------------------------------------------------------------------------
// Initialize - Overridden to set parent properties and setup musculo_items filtered listview.
bool TframeAssessmentMuscle::Initialize()
{
  // Set AssessmentGeneric properties
  SetBandCaption("Musculoskeletal");
  SetPermission("musculo_items");

  // Setup notes filter
  muscle_filter.SetListViewHandle(listMuscle);
  muscle_filter.SetSecurity("musculo_items");

  muscle_filter.AddColumnSpec("Item",        "musculo_items", as_text, true,  160, rawString);
  muscle_filter.AddColumnSpec("Details",     "musculo_items", as_text, true,  420, rawString);
  muscle_filter.AddColumnSpec("Current Id",  "current_id",    as_int,  false,  80);
  muscle_filter.AddColumnSpec("Internal Id", "internal_id",   as_int,  false,  80);

  muscle_filter.FetchPreferences("assessment.muscle.topfilter");
  muscle_filter.SetSQLStatement("");
  muscle_filter.RefreshDataSet();

  // setup musculo_item_types input filter

  string items_sql =
    "select item, id from musculo_item_types where valid = 1 and id > 0 and "
    "id <> " + lookupTables->IdOf("musculo_item_types", "No Limitations to ROM, Gait and Ambulation Approp."); // don't include the NPI indicator

  // Checks Filter
  checks_filter.SetListViewHandle(listMSTypes);
  checks_filter.SetSecurity("");
  checks_filter.AddColumnSpec("Item",        "musculo_items", as_text, true,  168, rawString);
  checks_filter.AddColumnSpec("Internal Id", "internal_id",   as_int,  false,  80);
  checks_filter.FetchPreferences("assessment.muscle.musculo_item_types");
  checks_filter.SetIconStatus(false);
  checks_filter.SetCheck(true);
  checks_filter.SetSQLStatement(items_sql);
  checks_filter.RefreshDataSet();

  // Set edit box change handlers
  SetAllEventHandlers(panelMuscle);

  return TframeAssessmentGeneric::Initialize();
}
//----------------------------------------------------------------------------------------------------------------------
// Lock - Overridden to lock Muscle panel
void TframeAssessmentMuscle::Lock()
{
  TframeAssessmentGeneric::Lock(); // lock parent controls
  panelMuscle->Enabled = false;    // lock all panel controls
}
//----------------------------------------------------------------------------------------------------------------------
// PutUpMuscleItems - Refreshes the controls to reflect the state of the muscle_items list
void TframeAssessmentMuscle::PutUpMuscleItems(pvecmap &musculo_items)
{
  psetstr musculo_item_ids;

  // Clear out the checkboxes and edit box
  checks_filter.UncheckAll();
  memoComments->Text = "";
  checkNPI->Checked = false;

  // Analyze the retrieved musculo_items items
  for(size_t i = 0; i < musculo_items.size(); i++)
  {
    // Skip items which are anything other than Current. This allows Deleted items to be updated back to Current status
    // when saving.
    if(musculo_items[i]["current_id"] != lookupTables->IdOf("current_types", "Current"))
      continue;

    if(musculo_items[i]["musculo_item_id"] == lookupTables->IdOf("musculo_item_types", "No Limitations to ROM, Gait and Ambulation Approp.")) // NPI
      checkNPI->Checked = true;
    else
    {
      musculo_item_ids.insert(musculo_items[i]["musculo_item_id"]);
      if(musculo_items[i]["musculo_item_id"] == lookupTables->IdOf("musculo_item_types", "Amputation"))
        editAmputation->Text = musculo_items[i]["comments"].c_str();
      else if(musculo_items[i]["musculo_item_id"] == lookupTables->IdOf("musculo_item_types", "Contracture"))
        editContractures->Text = musculo_items[i]["comments"].c_str();
      else if(musculo_items[i]["musculo_item_id"] == lookupTables->IdOf("musculo_item_types", "Joint Swelling"))
        editSwelling->Text = musculo_items[i]["comments"].c_str();
      else if(musculo_items[i]["musculo_item_id"] == lookupTables->IdOf("musculo_item_types", "Muscle Weakness"))
        editWeakness->Text = musculo_items[i]["comments"].c_str();
      else if(musculo_items[i]["musculo_item_id"] == lookupTables->IdOf("musculo_item_types", "Pain Locations"))
        editPain->Text = musculo_items[i]["comments"].c_str();
      else if(musculo_items[i]["musculo_item_id"] == lookupTables->IdOf("musculo_item_types", "Tenderness"))
        editTenderness->Text = musculo_items[i]["comments"].c_str();
      else if(musculo_items[i]["musculo_item_id"] == lookupTables->IdOf("musculo_item_types", "Other"))
        memoComments->Text = musculo_items[i]["comments"].c_str();
    }
  }

  // Set checkboxes
  checks_filter.SetCheckedWhereID(musculo_item_ids, true);
}
//----------------------------------------------------------------------------------------------------------------------
// TaskIdChanging - Overridden to populate musculo filter
void TframeAssessmentMuscle::TaskIdChanging(const string &new_id)
{
  string sql;

  TframeAssessmentGeneric::TaskIdChanging(new_id);

  sql = "select MIT.item, MI.comments, MI.current_id, MI.id "
        "from musculo_items MI "
        "left join musculo_item_types MIT on MI.musculo_item_id = MIT.id "
        "where MI.current_id >= 2 and MI.task_id = '" + new_id + "' ";

  muscle_filter.SetSQLStatement(sql);
  muscle_filter.RefreshDataSet();
  muscle_filter.SetAddEdit(this, AddRecord (&TframeAssessmentMuscle::FilterAddMuscle),
                                 EditRecord(&TframeAssessmentMuscle::FilterEditMuscle));
  muscle_request = parent_form->RequestNotificationWhen("musculo_items.task_id", new_id, rel_e, as_int);
}
//----------------------------------------------------------------------------------------------------------------------
// UpdateForEdit - Overridden to put fresh data into the editing_muscle_items vecmap and on screen.
void TframeAssessmentMuscle::UpdateForEdit()
{
  string sql =
    "select * "
    "from musculo_items MI "
    "where MI.task_id = '" + task_id + "' ";

  editing_muscle_items.clear();

  if(!SqlToVecMap(generic_transaction, sql, editing_muscle_items))
    parent_form->PSay("Could not retrieve up-to-date information for musculo_items", m_warning, MB_OK);

  // Refresh controls
  PutUpMuscleItems(editing_muscle_items);

  TframeAssessmentGeneric::UpdateForEdit();
}
//----------------------------------------------------------------------------------------------------------------------
// LockRecords - Overridden to lock the musculo_items records
bool TframeAssessmentMuscle::LockRecords()
{
  // Must acquire all items. Any failure is a failure to unlock for editing, GTFO and rollback.
  if(!LockVecMapRecords(editing_muscle_items, "musculo_items"))
    return false;

  return TframeAssessmentGeneric::LockRecords();
}
//----------------------------------------------------------------------------------------------------------------------
// IsValidOnScreen - Overridden to do validation for musculo_items
bool TframeAssessmentMuscle::IsValidOnScreen()
{
  pvecstr checked_ids = checks_filter.GetMultiID();
  bool other_checked = false;
  bool amputat_checked = false;
  bool contract_checked = false;
  bool joint_checked = false;
  bool weakness_checked = false;
  bool pain_checked = false;
  bool tender_checked = false;

  // Make sure at least one musculo item is specified
  if(!checkNPI->Checked &&
     editing_muscle_items.size() == 0 && // Only when the record is new, otherwise can't delete last one
     checked_ids.size() == 0)
  {
    parent_form->PSay("You have not specified any saveable data. Please check one or more musculoskeletal problems or "
                      "the 'No Limitations' indicator.", m_missing_data, MB_OK);
    return false;
  }

  // If NPI is checked, nothing except Other can be specified in addition to it.
  if(checkNPI->Checked && checked_ids.size() > 0)
  {
    for(size_t i = 0; i < checked_ids.size(); i++)
    {
      if(checked_ids[i] != lookupTables->IdOf("musculo_item_types", "Other"))
      {
        parent_form->PSay("You indicated that there were 'No Limitations', but also checked additional problems "
                          "(not including 'Other', which you can use to provide relevant information even if there "
                          "were no tangible problems).\n\n"
                          "Please uncheck either the 'No Limitations' indicator, or all items in the Musculoskeletal "
                          "Problems list except 'Other'.", m_invalid_data, MB_OK);
        return false;
      }
    }
  }

  for(size_t i = 0; i < checked_ids.size(); i++)
  {
    // Still have to hard code it >_>
    if(checked_ids[i] == lookupTables->IdOf("musculo_item_types", "Amputation"))
    {
      amputat_checked = true;
      if(editAmputation->Text == "")
      {
        parent_form->PSay("Please provide a description of the patient's amputations.", m_missing_data, MB_OK);
        return false;
      }
    }
    if(checked_ids[i] == lookupTables->IdOf("musculo_item_types", "Contracture"))
    {
      contract_checked = true;
      if(editContractures->Text == "")
      {
        parent_form->PSay("Please provide a description of the patient's contractures.", m_missing_data, MB_OK);
        return false;
      }
    }
    if(checked_ids[i] == lookupTables->IdOf("musculo_item_types", "Joint Swelling"))
    {
      joint_checked = true;
      // Description not required.
    }
    if(checked_ids[i] == lookupTables->IdOf("musculo_item_types", "Muscle Weakness"))
    {
      weakness_checked = true;
      // Description not required.
    }
    if(checked_ids[i] == lookupTables->IdOf("musculo_item_types", "Pain Locations"))
    {
      pain_checked = true;
      if(editPain->Text == "")
      {
        parent_form->PSay("Please describe the locations where the patient experiences pain.", m_missing_data, MB_OK);
        return false;
      }
    }
    if(checked_ids[i] == lookupTables->IdOf("musculo_item_types", "Tenderness"))
    {
      tender_checked = true;
      if(editTenderness->Text == "")
      {
        parent_form->PSay("Please describe the locations where the patient experiences tenderness.",
                          m_missing_data, MB_OK);
        return false;
      }
    }
    if(checked_ids[i] == lookupTables->IdOf("musculo_item_types", "Other"))
    {
      other_checked = true;
      if(memoComments->Text == "")
      {
        parent_form->PSay("You selected 'Other' but did not provide any additional information on the MS problems.",
                          m_missing_data, MB_OK);
        return false;
      }
    }
  }

  if(!amputat_checked && editAmputation->Text != "")
  {
    parent_form->PSay("You did not select 'Amputation', but specified information in the 'Amputation' field. This "
                      "information would be lost, so please either clear it out now, or select the 'Amputation' item in "
                      "the checkbox list as well.", m_invalid_data, MB_OK);
    return false;
  }
  if(!contract_checked && editContractures->Text != "")
  {
    parent_form->PSay("You did not select 'Contracture', but specified information in the 'Contracture' field. This "
                      "information would be lost, so please either clear it out now, or select the 'Contracture' item in "
                      "the checkbox list as well.", m_invalid_data, MB_OK);
    return false;
  }
  if(!joint_checked && editSwelling->Text != "")
  {
    parent_form->PSay("You did not select 'Joint Swelling', but specified information in the 'Joint Swelling' field. This "
                      "information would be lost, so please either clear it out now, or select the 'Joint Swelling' item in "
                      "the checkbox list as well.", m_invalid_data, MB_OK);
    return false;
  }
  if(!weakness_checked && editWeakness->Text != "")
  {
    parent_form->PSay("You did not select 'Muscle Weakness', but specified information in the 'Muscle Weakness' field. This "
                      "information would be lost, so please either clear it out now, or select the 'Muscle Weakness' item in "
                      "the checkbox list as well.", m_invalid_data, MB_OK);
    return false;
  }
  if(!pain_checked && editPain->Text != "")
  {
    parent_form->PSay("You did not select 'Pain Locations', but specified information in the 'Pain Locations' field. This "
                      "information would be lost, so please either clear it out now, or select the 'Pain Locations' item in "
                      "the checkbox list as well.", m_invalid_data, MB_OK);
    return false;
  }
  if(!tender_checked && editTenderness->Text != "")
  {
    parent_form->PSay("You did not select 'Tenderness', but specified information in the 'Tenderness' field. This "
                      "information would be lost, so please either clear it out now, or select the 'Tenderness' item in "
                      "the checkbox list as well.", m_invalid_data, MB_OK);
    return false;
  }
  if(!other_checked && memoComments->Text != "")
  {
    parent_form->PSay("You did not select 'Other', but specified information in the 'Other Comments' field. This "
                      "information would be lost, so please either clear it out now, or select the 'Other Comments' item in "
                      "the checkbox list as well.", m_invalid_data, MB_OK);
    return false;
  }

  return TframeAssessmentGeneric::IsValidOnScreen();
}
//----------------------------------------------------------------------------------------------------------------------
bool TframeAssessmentMuscle::UpdateMuscleItem(int record_index, string comment)
{
  pmapstrs ending_state;
  pmapstrtoint leave_alone;

  ending_state["id"]                 = editing_muscle_items[record_index]["id"];
  ending_state["current_id"]         = lookupTables->IdOf("current_types", "Current");
  ending_state["last_updated_by"]    = current_userid;
  ending_state["last_update_stamp"]  = RightNow();
  ending_state["task_id"]            = task_id;
  ending_state["musculo_item_id"]    = editing_muscle_items[record_index]["musculo_item_id"];
  ending_state["comments"]           = comment;

  leave_alone["comments"] = sql_no_uppercasing;

  final_ending_state.push_back(ending_state);

  return ExecuteUpdateStatement(generic_transaction, "musculo_items", ending_state, leave_alone);
}
//----------------------------------------------------------------------------------------------------------------------
bool TframeAssessmentMuscle::InsertMuscleItem(string musculo_item_id, string comment)
{
  pmapstrs ending_state;
  pmapstrtoint leave_alone;

  ending_state["id"]                = GetNextId(generic_transaction, "musculo_items");
  ending_state["current_id"]        = lookupTables->IdOf("current_types", "Current");
  ending_state["last_updated_by"]   = current_userid;
  ending_state["last_update_stamp"] = RightNow();
  ending_state["created_by"]        = current_userid;
  ending_state["creation_stamp"]    = ending_state["last_update_stamp"];
  ending_state["task_id"]           = task_id;
  ending_state["musculo_item_id"]   = musculo_item_id;
  ending_state["comments"]          = comment;

  leave_alone["comments"] = sql_no_uppercasing;

  final_ending_state.push_back(ending_state);

  return ExecuteInsertStatement(generic_transaction, "musculo_items", ending_state, leave_alone);
}
//----------------------------------------------------------------------------------------------------------------------
bool TframeAssessmentMuscle::DeleteMuscleItem(int record_index)
{
  pmapstrs ending_state;
  pmapstrtoint leave_alone;

  ending_state["id"]                = editing_muscle_items[record_index]["id"];
  ending_state["current_id"]        = lookupTables->IdOf("current_types", "Deleted"); // Mark as Deleted
  ending_state["last_updated_by"]   = current_userid;
  ending_state["last_update_stamp"] = RightNow();

  final_ending_state.push_back(ending_state);

  return ExecuteUpdateStatement(generic_transaction, "musculo_items", ending_state, leave_alone);
}
//----------------------------------------------------------------------------------------------------------------------
// DoSaveData - Overridden to provide core saving logic.
bool TframeAssessmentMuscle::DoSaveData()
{
  psetstr checked_ids;
  psetstr unchecked_ids;
  int record_idx;
  string npiID = lookupTables->IdOf("musculo_item_types", "No Limitations to ROM, Gait and Ambulation Approp.");

  final_ending_state.clear();

  // First determine what is checked and what is not in the filters.
  GetIDsFromCheckBoxFilter(checks_filter, checked_ids, unchecked_ids); // use parent class utility method

  // Add in the NPI
  if(checkNPI->Checked)
    checked_ids.insert(npiID);
  else
    unchecked_ids.insert(npiID);

  // Look at all IDs:

  // * If the ID is checked, and the ID is in the editing map, do an update on that record.
  // * If the ID is checked, and the ID is NOT in the editing map, do an insert.
  for(psetstr::iterator i_checked = checked_ids.begin(); i_checked != checked_ids.end(); ++i_checked)
  {
    string comments;

    if(*i_checked == lookupTables->IdOf("musculo_item_types", "Amputation"))
      comments = editAmputation->Text.c_str();
    else if(*i_checked == lookupTables->IdOf("musculo_item_types", "Contracture"))
      comments = editContractures->Text.c_str();
    else if(*i_checked == lookupTables->IdOf("musculo_item_types", "Joint Swelling"))
      comments = editSwelling->Text.c_str();
    else if(*i_checked == lookupTables->IdOf("musculo_item_types", "Muscle Weakness"))
      comments = editWeakness->Text.c_str();
    else if(*i_checked == lookupTables->IdOf("musculo_item_types", "Pain Locations"))
      comments = editPain->Text.c_str();
    else if(*i_checked == lookupTables->IdOf("musculo_item_types", "Tenderness"))
      comments = editTenderness->Text.c_str();
    else if(*i_checked == lookupTables->IdOf("musculo_item_types", "Other"))
      comments = memoComments->Text.c_str();

    record_idx = IsInVecMap(editing_muscle_items, "musculo_item_id", *i_checked);
    if(record_idx >= 0)
    {
      // Do Update because this musculo_items item already exists.
      if(!UpdateMuscleItem(record_idx, comments))
        return false;
    }
    else
    {
      // Do Insert because this musculo_items item doesn't exist.
      if(!InsertMuscleItem(*i_checked, comments))
        return false;
    }
  }

  // * If the ID is NOT checked, and the id is in the editing map, mark that item as Deleted via the current_id.
  // * If the ID is NOT checked, and the id is NOT in the editing map, do nothing at all.
  for(psetstr::iterator i_unchecked = unchecked_ids.begin(); i_unchecked != unchecked_ids.end(); ++i_unchecked)
  {
    record_idx = IsInVecMap(editing_muscle_items, "musculo_item_id", *i_unchecked);
    if(record_idx >= 0)
    {
      // Do Update and set the existing musculo_items record to 'Deleted'.
      if(!DeleteMuscleItem(record_idx))
        return false;
    }
    // Otherwise, we neither had a record to begin with, nor do we have one to insert now, so do nothing.
  }

  // If we get this far, nothing borked up above.
  return true;
}
//----------------------------------------------------------------------------------------------------------------------
// SetViewMode - Overridden to activate viewing of musculo_items information
void TframeAssessmentMuscle::SetViewMode()
{
  TframeAssessmentGeneric::SetViewMode();

  panelMuscle->Enabled = false;                    // It is still not possible to enter data.
  editing_muscle_items.clear();                    // Clear out the items vecmap for safety
  checkNPI->Checked = false;
  checks_filter.UncheckAll();

  ClearControlsModified(panelMuscle);
  ClearControlsText(panelMuscle);
}
//----------------------------------------------------------------------------------------------------------------------
// SetEditMode - Overridden to activate editing of musculo_items records
void TframeAssessmentMuscle::SetEditMode()
{
  TframeAssessmentGeneric::SetEditMode();

  panelMuscle->Enabled = true;               // Enable panel controls

  ClearControlsModified(panelMuscle);
}
//----------------------------------------------------------------------------------------------------------------------
// FilterAddMuscle - "Add" handler for the musculo_items filter. This actually just activates editing mode here.
void TframeAssessmentMuscle::FilterAddMuscle()
{
  if(btnUnlockSave->Enabled && frame_mode == mode_viewing)
    btnUnlockSaveClick(NULL);
}
//----------------------------------------------------------------------------------------------------------------------
// FilterEditMuscle - Same as above.
void TframeAssessmentMuscle::FilterEditMuscle()
{
  if(btnUnlockSave->Enabled && frame_mode == mode_viewing)
    btnUnlockSaveClick(NULL);
}
//----------------------------------------------------------------------------------------------------------------------
// AtlasNotify - Overridden to refresh musculo_items listview
bool TframeAssessmentMuscle::AtlasNotify(int request_num, string matched_value)
{
  bool handled = false;

  if(request_num == muscle_request)
  {
    muscle_filter.RefreshDataSet();
    handled = true;
  }

  return handled;
}
//----------------------------------------------------------------------------------------------------------------------
// SendAtlas - Overridden to send out notifications for musculo_items
void TframeAssessmentMuscle::SendAtlas()
{
  atlas->Notify("musculo_items.task_id", task_id); // Is this good enough?
}
//----------------------------------------------------------------------------------------------------------------------
// WriteReport - Write out a report on the musculoskeletal conditions
bool TframeAssessmentMuscle::WriteReport(TComponent *compositeReportCtrl)
{
  bool success = false;

  try
  {
    string sql =
      "select MIT.item, MI.comments "
      "from musculo_items MI "
      "left join musculo_item_types MIT on MI.musculo_item_id = MIT.id "
      "where MI.current_id >= 3 and MI.task_id = '" + task_id + "' ";

    InitDataSet(sql);

    TformReportAssessment *reportForm = report_data.reportForm;
    TQuickRep *thisReport = reportForm->reportMuscle;

    thisReport->DataSet = report_data.dbDataSet;
    SetupQRDBText(reportForm->txtMuscleItem,   "item");
    SetupQRDBText(reportForm->txtMuscleDetail, "comments");

    // NB: Need to pass it as TComponent to avoid problems with having QuickReport included in the headers
    TQRCompositeReport *compositeReport = dynamic_cast<TQRCompositeReport *>(compositeReportCtrl);
    if(compositeReport)
    {
      compositeReport->Reports->Add(thisReport);
      success = true;
    }
  }
  catch(...)
  {
  }

  return success;
}
//----------------------------------------------------------------------------------------------------------------------
void __fastcall TframeAssessmentMuscle::listMSTypesClick(TObject *Sender)
{
  // Use parent class hit test logic
  FilterCheckBoxHitTest(listMSTypes);
}
//----------------------------------------------------------------------------------------------------------------------
void __fastcall TframeAssessmentMuscle::checkNPIClick(TObject *Sender)
{
  if(frame_mode == mode_editing)
    SetModified(true);
}
//----------------------------------------------------------------------------------------------------------------------

