//----------------------------------------------------------------------------------------------------------------------

#include <vcl.h>
#pragma hdrstop

#include "codeFrameAssessmentNeuro.h"
#include "codeRecordPatientAssessment.h"
//----------------------------------------------------------------------------------------------------------------------
#pragma package(smart_init)
#pragma link "codeFrameAssessmentGeneric"
#pragma resource "*.dfm"
TframeAssessmentNeuro *frameAssessmentNeuro;
//----------------------------------------------------------------------------------------------------------------------
__fastcall TframeAssessmentNeuro::TframeAssessmentNeuro(TComponent* Owner)
        : TframeAssessmentGeneric(Owner), neuro_filter(), ori_filter(), disp_filter(), sympt_filter(), resp_filter(),
          speech_filter(), detail_filter(), neuro_request(-1), editing_neuro_items(), final_ending_state()
{
}
//----------------------------------------------------------------------------------------------------------------------
// Initialize - Overridden to set parent properties and setup neurology_items filtered listview.
bool TframeAssessmentNeuro::Initialize()
{
  // Set AssessmentGeneric properties
  SetBandCaption("Neurological");
  SetPermission("neurology_items");

  // Setup top filter
  neuro_filter.SetListViewHandle(listNeuro);
  neuro_filter.SetSecurity("neurology_items");

  neuro_filter.AddColumnSpec("Item",        "neurology_items", as_text, true,  160, rawString);
  neuro_filter.AddColumnSpec("Category",    "neurology_items", as_text, true,   70, rawString);
  neuro_filter.AddColumnSpec("Details",     "neurology_items", as_text, true,  350, sentenceString);
  neuro_filter.AddColumnSpec("Current Id",  "current_id",      as_int,  false,  80);
  neuro_filter.AddColumnSpec("Internal Id", "internal_id",     as_int,  false,  80);

  neuro_filter.FetchPreferences("assessment.neuro.topfilter");
  neuro_filter.SetSQLStatement("");
  neuro_filter.RefreshDataSet();

  // setup input filters

  string ori_sql, disp_sql, sympt_sql, resp_sql, speech_sql, detail_sql;

  ori_sql    = "select item, id from neuro_item_types where valid = 1 and neuro_value_id = '"
               + lookupTables->IdOf("neuro_value_types", "Orientation") + "' ";
  disp_sql   = "select item, id from neuro_item_types where valid = 1 and neuro_value_id = '"
               + lookupTables->IdOf("neuro_value_types", "Disposition") + "' ";
  sympt_sql  = "select item, id from neuro_item_types where valid = 1 and neuro_value_id = '"
               + lookupTables->IdOf("neuro_value_types", "Symptoms") + "' ";
  resp_sql   = "select item, id from neuro_item_types where valid = 1 and neuro_value_id = '"
               + lookupTables->IdOf("neuro_value_types", "Responses") + "' ";
  speech_sql = "select item, id from neuro_item_types where valid = 1 and neuro_value_id = '"
               + lookupTables->IdOf("neuro_value_types", "Speech") + "' ";
  detail_sql = "select item, id from neuro_item_types where valid = 1 and neuro_value_id = '"
               + lookupTables->IdOf("neuro_value_types", "Detailed") + "' ";

  // Orientation Filter
  ori_filter.SetListViewHandle(listOrientation);
  ori_filter.SetSecurity("");
  ori_filter.AddColumnSpec("Item",        "neurology_items", as_text, true,  128, rawString);
  ori_filter.AddColumnSpec("Internal Id", "internal_id",     as_int,  false,  80);
  ori_filter.FetchPreferences("assessment.neuro.orientation");
  ori_filter.SetIconStatus(false);
  ori_filter.SetCheck(true);
  ori_filter.SetSQLStatement(ori_sql);
  ori_filter.RefreshDataSet();

  // Disposition Filter
  disp_filter.SetListViewHandle(listDisposition);
  disp_filter.SetSecurity("");
  disp_filter.AddColumnSpec("Item",        "neurology_items", as_text, true,  128, rawString);
  disp_filter.AddColumnSpec("Internal Id", "internal_id",     as_int,  false,  80);
  disp_filter.FetchPreferences("assessment.neuro.orientation");
  disp_filter.SetIconStatus(false);
  disp_filter.SetCheck(true);
  disp_filter.SetSQLStatement(disp_sql);
  disp_filter.RefreshDataSet();

  // Symptoms Filter
  sympt_filter.SetListViewHandle(listSymptoms);
  sympt_filter.SetSecurity("");
  sympt_filter.AddColumnSpec("Item",        "neurology_items", as_text, true,  128, rawString);
  sympt_filter.AddColumnSpec("Internal Id", "internal_id",     as_int,  false,  80);
  sympt_filter.FetchPreferences("assessment.neuro.orientation");
  sympt_filter.SetIconStatus(false);
  sympt_filter.SetCheck(true);
  sympt_filter.SetSQLStatement(sympt_sql);
  sympt_filter.RefreshDataSet();

  // Responses Filter
  resp_filter.SetListViewHandle(listResponses);
  resp_filter.SetSecurity("");
  resp_filter.AddColumnSpec("Item",        "neurology_items", as_text, true,  128, rawString);
  resp_filter.AddColumnSpec("Internal Id", "internal_id",     as_int,  false,  80);
  resp_filter.FetchPreferences("assessment.neuro.orientation");
  resp_filter.SetIconStatus(false);
  resp_filter.SetCheck(true);
  resp_filter.SetSQLStatement(resp_sql);
  resp_filter.RefreshDataSet();

  // Speech Filter
  speech_filter.SetListViewHandle(listSpeech);
  speech_filter.SetSecurity("");
  speech_filter.AddColumnSpec("Item",        "neurology_items", as_text, true,  128, rawString);
  speech_filter.AddColumnSpec("Internal Id", "internal_id",     as_int,  false,  80);
  speech_filter.FetchPreferences("assessment.neuro.orientation");
  speech_filter.SetIconStatus(false);
  speech_filter.SetCheck(true);
  speech_filter.SetSQLStatement(speech_sql);
  speech_filter.RefreshDataSet();

  // Detailed Filter
  detail_filter.SetListViewHandle(listDetailed);
  detail_filter.SetSecurity("");
  detail_filter.AddColumnSpec("Item",        "neurology_items", as_text, true,  128, rawString);
  detail_filter.AddColumnSpec("Internal Id", "internal_id",     as_int,  false,  80);
  detail_filter.FetchPreferences("assessment.neuro.orientation");
  detail_filter.SetIconStatus(false);
  detail_filter.SetCheck(true);
  detail_filter.SetSQLStatement(detail_sql);
  detail_filter.RefreshDataSet();

  // Set edit box change handlers
  SetAllEventHandlers(panelNeuro);

  return TframeAssessmentGeneric::Initialize();
}
//----------------------------------------------------------------------------------------------------------------------
// Lock - Overridden to lock neuro panel
void TframeAssessmentNeuro::Lock()
{
  TframeAssessmentGeneric::Lock(); // lock parent controls
  panelNeuro->Enabled = false;     // lock all controls
}
//----------------------------------------------------------------------------------------------------------------------
// PutUpNeuroItems - Refreshes the controls to reflect the state of the neurology assessment
void TframeAssessmentNeuro::PutUpNeuroItems(pvecmap &neuro_items)
{
  // Clear out all checkbox filters, and the NPI item checkbox
  ori_filter.UncheckAll();
  disp_filter.UncheckAll();
  sympt_filter.UncheckAll();
  resp_filter.UncheckAll();
  speech_filter.UncheckAll();
  detail_filter.UncheckAll();

  bool have_NPI = false;
  psetstr ori_ids;
  psetstr disp_ids;
  psetstr sympt_ids;
  psetstr resp_ids;
  psetstr speech_ids;
  psetstr detail_ids;

  //clear edits
  ClearControlsText(panelNeuro);

  // Analyze the retrieved neuro assessment items
  for(size_t i = 0; i < neuro_items.size(); i++)
  {
    // Skip items which are anything other than Current. This allows Deleted items to be updated back to Current status
    // when saving.
    if(neuro_items[i]["current_id"] != lookupTables->IdOf("current_types", "Current"))
      continue;
       
    if(neuro_items[i]["neuro_value_id"] == lookupTables->IdOf("neuro_value_types", "Orientation"))
      ori_ids.insert(neuro_items[i]["neuro_item_id"]);
    else if(neuro_items[i]["neuro_value_id"] == lookupTables->IdOf("neuro_value_types", "Disposition"))
      disp_ids.insert(neuro_items[i]["neuro_item_id"]);
    else if(neuro_items[i]["neuro_value_id"] == lookupTables->IdOf("neuro_value_types", "Symptoms"))
      sympt_ids.insert(neuro_items[i]["neuro_item_id"]);
    else if(neuro_items[i]["neuro_value_id"] == lookupTables->IdOf("neuro_value_types", "Responses"))
      resp_ids.insert(neuro_items[i]["neuro_item_id"]);
    else if(neuro_items[i]["neuro_value_id"] == lookupTables->IdOf("neuro_value_types", "Speech"))
      speech_ids.insert(neuro_items[i]["neuro_item_id"]);
    else if(neuro_items[i]["neuro_value_id"] == lookupTables->IdOf("neuro_value_types", "Detailed"))
      detail_ids.insert(neuro_items[i]["neuro_item_id"]);
    else if(neuro_items[i]["neuro_value_id"] == lookupTables->IdOf("neuro_value_types", "NPI"))
      have_NPI = true;

    // Put up comments on certain items
    if(neuro_items[i]["neuro_item_id"] == lookupTables->IdOf("neuro_item_types", "Other"))
      memoSpeech->Text = neuro_items[i]["comments"].c_str();
    if(neuro_items[i]["neuro_item_id"] == lookupTables->IdOf("neuro_item_types", "Numbness"))
      editNumbness->Text = neuro_items[i]["comments"].c_str();
    if(neuro_items[i]["neuro_item_id"] == lookupTables->IdOf("neuro_item_types", "Weakness"))
      editWeakness->Text = neuro_items[i]["comments"].c_str();
    if(neuro_items[i]["neuro_item_id"] == lookupTables->IdOf("neuro_item_types", "Tingling"))
      editTingling->Text = neuro_items[i]["comments"].c_str(); // tingle tingle, kooloo-limpah!
  }

  // Set checkboxes
  ori_filter.SetCheckedWhereID(ori_ids, true);
  disp_filter.SetCheckedWhereID(disp_ids, true);
  sympt_filter.SetCheckedWhereID(sympt_ids, true);
  resp_filter.SetCheckedWhereID(resp_ids, true);
  speech_filter.SetCheckedWhereID(speech_ids, true);
  detail_filter.SetCheckedWhereID(detail_ids, true);

  checkNPI->Checked = have_NPI;
}
//----------------------------------------------------------------------------------------------------------------------
// TaskIdChanging - Overridden to populate neurology items filter
void TframeAssessmentNeuro::TaskIdChanging(const string &new_id)
{
  string sql;

  TframeAssessmentGeneric::TaskIdChanging(new_id);

  sql = "select NIT.item, NVT.item, NI.comments, NI.current_id, NI.id "
        "from neurology_items NI "
        "left join neuro_item_types  NIT on NI.neuro_item_id   = NIT.id "
        "left join neuro_value_types NVT on NIT.neuro_value_id = NVT.id "
        "where NI.current_id >= 2 and NI.task_id = '" + new_id + "' ";

  neuro_filter.SetSQLStatement(sql);
  neuro_filter.RefreshDataSet();
  neuro_filter.SetAddEdit(this, AddRecord(&TframeAssessmentNeuro::FilterAddNeuro),
                               EditRecord(&TframeAssessmentNeuro::FilterEditNeuro));
  neuro_request = parent_form->RequestNotificationWhen("neurology_items.task_id", new_id, rel_e, as_int);
}
//----------------------------------------------------------------------------------------------------------------------
// UpdateForEdit - Overridden to put fresh data into the editing_neuro_items vecmap and on screen.
void TframeAssessmentNeuro::UpdateForEdit()
{
  string sql =
    "select NI.neuro_item_id, NIT.neuro_value_id, NI.comments, NI.current_id, NI.id "
    "from neurology_items NI "
    "left join neuro_item_types  NIT on NI.neuro_item_id = NIT.id "
    "where NI.task_id = '" + task_id + "' ";

  editing_neuro_items.clear();

  if(!SqlToVecMap(generic_transaction, sql, editing_neuro_items))
    parent_form->PSay("Could not retrieve up-to-date information for neurology items", m_warning, MB_OK);

  // Refresh controls
  PutUpNeuroItems(editing_neuro_items);

  TframeAssessmentGeneric::UpdateForEdit();
}
//----------------------------------------------------------------------------------------------------------------------
// LockRecords - Overridden to lock the neurology_items records
bool TframeAssessmentNeuro::LockRecords()
{
  // Must acquire all items. Any failure is a failure to unlock for editing, GTFO and rollback.
  //   jhaley 20110629 - abstracted logic into parent class
  if(!LockVecMapRecords(editing_neuro_items, "neurology_items"))
    return false;

  return TframeAssessmentGeneric::LockRecords();
}
//----------------------------------------------------------------------------------------------------------------------
// IsValidOnScreen - Overridden to do validation for neurology items
bool TframeAssessmentNeuro::IsValidOnScreen()
{
  pvecstr checked_ori_ids    = ori_filter.GetMultiID();
  pvecstr checked_disp_ids   = disp_filter.GetMultiID();
  pvecstr checked_sympt_ids  = sympt_filter.GetMultiID();
  pvecstr checked_resp_ids   = resp_filter.GetMultiID();
  pvecstr checked_speech_ids = speech_filter.GetMultiID();
  pvecstr checked_detail_ids = detail_filter.GetMultiID();

  // Make sure at least one neuro assessment item is specified
  if(editing_neuro_items.size() == 0 && // jhaley 20110629: Only when the record is new, otherwise can't delete last one
     !checkNPI->Checked && checked_ori_ids.size() == 0 && checked_disp_ids.size() == 0 && checked_sympt_ids.size() == 0 &&
     checked_resp_ids.size() == 0 && checked_speech_ids.size() == 0 && checked_detail_ids.size() == 0)
  {
    parent_form->PSay("You have not specified any saveable data. Please check one or more assessment items.",
                      m_missing_data, MB_OK);
    return false;
  }

  // If NPI is specified, most options should be disallowed
  if(checkNPI->Checked)
  {
    if(checked_ori_ids.size() != 0)
    {
      parent_form->PSay("You indicated that the patient had no notable problems by checking the NPI indicator, but also "
                        "indicated problems in the Orientation assessment category. Please correct this inconsistency "
                        "before saving.", m_invalid_data, MB_OK);
      return false;
    }
    if(checked_disp_ids.size() != 0)
    {
      parent_form->PSay("You indicated that the patient had no notable problems by checking the NPI indicator, but also "
                        "indicated problems in the Disposition assessment category. Please correct this inconsistency "
                        "before saving.", m_invalid_data, MB_OK);
      return false;
    }
    if(checked_sympt_ids.size() != 0)
    {
      parent_form->PSay("You indicated that the patient had no notable problems by checking the NPI indicator, but also "
                        "indicated problems in the Symptoms assessment category. Please correct this inconsistency "
                        "before saving.", m_invalid_data, MB_OK);
      return false;
    }
    if(checked_resp_ids.size() != 0)
    {
      parent_form->PSay("You indicated that the patient had no notable problems by checking the NPI indicator, but also "
                        "indicated problems in the Responses assessment category. Please correct this inconsistency "
                        "before saving.", m_invalid_data, MB_OK);
      return false;
    }
    if(checked_speech_ids.size() != 0)
    {
      parent_form->PSay("You indicated that the patient had no notable problems by checking the NPI indicator, but also "
                        "indicated problems in the Speech assessment category. Please correct this inconsistency "
                        "before saving.", m_invalid_data, MB_OK);
      return false;
    }
    // Note: Detailed area is allowed even with NPI indicator, as is not necessarily excluded by it.
  }

  // ones that require comments:
  // * Speech - "Other" requires comment
  for(int i = 0; i < checked_speech_ids.size(); i++)
  {
    if(checked_speech_ids[i] == lookupTables->IdOf("neuro_item_types", "Other"))
    {
      if(memoSpeech->Text == "")
      {
        parent_form->PSay("You selected 'Other' but did not provide any information on the speech problem.",
                          m_missing_data, MB_OK);
        return false;
      }
    }
  }
  // * Detailed - Weakness, Tingling, Numbness require comments
  for(int i = 0; i < checked_detail_ids.size(); i++)
  {
    if(checked_detail_ids[i] == lookupTables->IdOf("neuro_item_types", "Numbness"))
    {
      if(editNumbness->Text == "")
      {
        parent_form->PSay("You selected 'Numbness' but did not provide any information on the location.",
                          m_missing_data, MB_OK);
        return false;
      }
    }
    if(checked_detail_ids[i] == lookupTables->IdOf("neuro_item_types", "Weakness"))
    {
      if(editWeakness->Text == "")
      {
        parent_form->PSay("You selected 'Weakness' but did not provide any information on the location.",
                          m_missing_data, MB_OK);
        return false;
      }
    }
    if(checked_detail_ids[i] == lookupTables->IdOf("neuro_item_types", "Tingling"))
    {
      if(editTingling->Text == "")
      {
        parent_form->PSay("You selected 'Tingling' but did not provide any information on the location.",
                          m_missing_data, MB_OK);
        return false;
      }
    }
  }

  return TframeAssessmentGeneric::IsValidOnScreen();
}
//----------------------------------------------------------------------------------------------------------------------
bool TframeAssessmentNeuro::UpdateNeuroItem(int record_index, string comment)
{
  pmapstrs ending_state;
  pmapstrtoint leave_alone;

  ending_state["id"]                = editing_neuro_items[record_index]["id"];
  ending_state["current_id"]        = lookupTables->IdOf("current_types", "Current");
  ending_state["last_updated_by"]   = current_userid;
  ending_state["last_update_stamp"] = RightNow();
  ending_state["task_id"]           = task_id;
  ending_state["neuro_item_id"]     = editing_neuro_items[record_index]["neuro_item_id"];
  ending_state["comments"]          = comment;

  leave_alone["comments"] = sql_no_uppercasing;

  final_ending_state.push_back(ending_state);

  return ExecuteUpdateStatement(generic_transaction, "neurology_items", ending_state, leave_alone);
}
//----------------------------------------------------------------------------------------------------------------------
bool TframeAssessmentNeuro::InsertNeuroItem(string neuro_item_id, string comment)
{
  pmapstrs ending_state;
  pmapstrtoint leave_alone;

  ending_state["id"]                = GetNextId(generic_transaction, "neurology_items");
  ending_state["current_id"]        = lookupTables->IdOf("current_types", "Current");
  ending_state["last_updated_by"]   = current_userid;
  ending_state["last_update_stamp"] = RightNow();
  ending_state["created_by"]        = current_userid;
  ending_state["creation_stamp"]    = ending_state["last_update_stamp"];
  ending_state["task_id"]           = task_id;
  ending_state["neuro_item_id"]     = neuro_item_id;
  ending_state["comments"]          = comment;

  leave_alone["comments"] = sql_no_uppercasing;

  final_ending_state.push_back(ending_state);

  return ExecuteInsertStatement(generic_transaction, "neurology_items", ending_state, leave_alone);
}
//----------------------------------------------------------------------------------------------------------------------
bool TframeAssessmentNeuro::DeleteNeuroItem(int record_index)
{
  pmapstrs ending_state;
  pmapstrtoint leave_alone;

  ending_state["id"]                = editing_neuro_items[record_index]["id"];
  ending_state["current_id"]        = lookupTables->IdOf("current_types", "Deleted"); // Mark as Deleted
  ending_state["last_updated_by"]   = current_userid;
  ending_state["last_update_stamp"] = RightNow();

  leave_alone["comments"] = sql_no_uppercasing;

  final_ending_state.push_back(ending_state);

  return ExecuteUpdateStatement(generic_transaction, "neurology_items", ending_state, leave_alone);
}
//----------------------------------------------------------------------------------------------------------------------
// DoSaveData - Overridden to provide core saving logic.
bool TframeAssessmentNeuro::DoSaveData()
{
  psetstr checked_ids;
  psetstr unchecked_ids;
  int record_idx;

  final_ending_state.clear();

  // First determine what is checked and what is not in the filters.
  //   jhaley 20110629 - abstracted shared logic into parent class routine
  GetIDsFromCheckBoxFilter(ori_filter,    checked_ids, unchecked_ids);
  GetIDsFromCheckBoxFilter(disp_filter,   checked_ids, unchecked_ids);
  GetIDsFromCheckBoxFilter(sympt_filter,  checked_ids, unchecked_ids);
  GetIDsFromCheckBoxFilter(resp_filter,   checked_ids, unchecked_ids);
  GetIDsFromCheckBoxFilter(speech_filter, checked_ids, unchecked_ids);
  GetIDsFromCheckBoxFilter(detail_filter, checked_ids, unchecked_ids);

  // Throw in the NPI ID
  if(checkNPI->Checked)
    checked_ids.insert(lookupTables->IdOf("neuro_item_types", "Awake, Alert, Oriented x3, Responds, Speech Clear, Handgrips Equal"));
  else
    unchecked_ids.insert(lookupTables->IdOf("neuro_item_types", "Awake, Alert, Oriented x3, Responds, Speech Clear, Handgrips Equal"));

  // Look at all IDs:

  // * If the ID is checked, and the ID is in the editing map, do an update on that record.
  // * If the ID is checked, and the ID is NOT in the editing map, do an insert.
  for(psetstr::iterator i_checked = checked_ids.begin(); i_checked != checked_ids.end(); ++i_checked)
  {
    string comment;

    if(*i_checked == lookupTables->IdOf("neuro_item_types", "Other"))
      comment = memoSpeech->Text.c_str();
    else if(*i_checked == lookupTables->IdOf("neuro_item_types", "Weakness"))
      comment = editWeakness->Text.c_str();
    else if(*i_checked == lookupTables->IdOf("neuro_item_types", "Numbness"))
      comment = editNumbness->Text.c_str();
    else if(*i_checked == lookupTables->IdOf("neuro_item_types", "Tingling"))
      comment = editTingling->Text.c_str();

    // jhaley 20110629 - abstracted logic into parent class
    record_idx = IsInVecMap(editing_neuro_items, "neuro_item_id", *i_checked);
    if(record_idx >= 0)
    {
      // Do Update because this neuro assessment item already exists.
      if(!UpdateNeuroItem(record_idx, comment))
        return false;
    }
    else
    {
      // Do Insert because this neuro assessment item doesn't exist.
      if(!InsertNeuroItem(*i_checked, comment))
        return false;
    }
  }

  // * If the ID is NOT checked, and the id is in the editing map, mark that item as Deleted via the current_id.
  // * If the ID is NOT checked, and the id is NOT in the editing map, do nothing at all.
  for(psetstr::iterator i_unchecked = unchecked_ids.begin(); i_unchecked != unchecked_ids.end(); ++i_unchecked)
  {
    record_idx = IsInVecMap(editing_neuro_items, "neuro_item_id", *i_unchecked);
    if(record_idx >= 0)
    {
      // Do Update and set the existing neuro assessment item to 'Deleted'.
      if(!DeleteNeuroItem(record_idx))
        return false;
    }
    // Otherwise, we neither had a record to begin with, nor do we have one to insert now, so do nothing.
  }

  // If we get this far, nothing borked up above.
  return true;
}
//----------------------------------------------------------------------------------------------------------------------
// SetViewMode - Overridden to activate viewing of neuro assessment item information
void TframeAssessmentNeuro::SetViewMode()
{
  TframeAssessmentGeneric::SetViewMode();

  panelNeuro->Enabled = false;                    // It is still not possible to enter data.
  editing_neuro_items.clear();                    // Clear out the items vecmap for safety
  ori_filter.UncheckAll();
  disp_filter.UncheckAll();
  sympt_filter.UncheckAll();
  resp_filter.UncheckAll();
  speech_filter.UncheckAll();
  detail_filter.UncheckAll();
  checkNPI->Checked = false;

  ClearControlsText(panelNeuro);
}
//----------------------------------------------------------------------------------------------------------------------
// SetEditMode - Overridden to activate editing of neuro assessment items
void TframeAssessmentNeuro::SetEditMode()
{
  TframeAssessmentGeneric::SetEditMode();

  panelNeuro->Enabled = true;               // Enable panel controls
}
//----------------------------------------------------------------------------------------------------------------------
// FilterAddNeuro - "Add" handler for the neuro items filter. This actually just activates editing mode here.
void TframeAssessmentNeuro::FilterAddNeuro()
{
  if(btnUnlockSave->Enabled && frame_mode == mode_viewing)
    btnUnlockSaveClick(NULL);
}
//----------------------------------------------------------------------------------------------------------------------
// FilterEditNeuro - Same as above.
void TframeAssessmentNeuro::FilterEditNeuro()
{
  if(btnUnlockSave->Enabled && frame_mode == mode_viewing)
    btnUnlockSaveClick(NULL);
}
//----------------------------------------------------------------------------------------------------------------------
// AtlasNotify - Overridden to refresh neuro items listview
bool TframeAssessmentNeuro::AtlasNotify(int request_num, string matched_value)
{
  bool handled = false;

  if(request_num == neuro_request)
  {
    neuro_filter.RefreshDataSet();
    handled = true;
  }

  return handled;
}
//----------------------------------------------------------------------------------------------------------------------
// SendAtlas - Overridden to send out notifications for neurology_items
void TframeAssessmentNeuro::SendAtlas()
{
  atlas->Notify("neurology_items.task_id", task_id); // Is this good enough?
}
//----------------------------------------------------------------------------------------------------------------------
// WriteReport - Write out a report on the neurological conditions
bool TframeAssessmentNeuro::WriteReport(TComponent *compositeReportCtrl)
{
  bool success = false;

  try
  {
    string sql =
      "select NIT.item value_item, NVT.item value_type, NI.comments "
      "from neurology_items NI "
      "left join neuro_item_types  NIT on NI.neuro_item_id   = NIT.id "
      "left join neuro_value_types NVT on NIT.neuro_value_id = NVT.id "
      "where NI.current_id >= 3 and NI.task_id = '" + task_id + "' ";

    InitDataSet(sql);

    TformReportAssessment *reportForm = report_data.reportForm;
    TQuickRep *thisReport = reportForm->reportNeuro;

    thisReport->DataSet = report_data.dbDataSet;
    SetupQRDBText(reportForm->txtNeuroItem,   "value_item");
    SetupQRDBText(reportForm->txtNeuroCat,    "value_type");
    SetupQRDBText(reportForm->txtNeuroDetail, "comments");

    // NB: Need to pass it as TComponent to avoid problems with having QuickReport included in the headers
    TQRCompositeReport *compositeReport = dynamic_cast<TQRCompositeReport *>(compositeReportCtrl);
    if(compositeReport)
    {
      compositeReport->Reports->Add(thisReport);
      success = true;
    }
  }
  catch(...)
  {
  }

  return success;
}
//----------------------------------------------------------------------------------------------------------------------
// checkNPIClick - Marks the frame's data as modified if the NPI checkbox is clicked during editing mode
void __fastcall TframeAssessmentNeuro::checkNPIClick(TObject *Sender)
{
  if(frame_mode == mode_editing)
    SetModified(true);
}
//----------------------------------------------------------------------------------------------------------------------
void __fastcall TframeAssessmentNeuro::listOrientationClick(TObject *Sender)
{
  FilterCheckBoxHitTest(listOrientation);
}
//----------------------------------------------------------------------------------------------------------------------
void __fastcall TframeAssessmentNeuro::listDispositionClick(TObject *Sender)
{
  FilterCheckBoxHitTest(listDisposition);
}
//----------------------------------------------------------------------------------------------------------------------
void __fastcall TframeAssessmentNeuro::listSymptomsClick(TObject *Sender)
{
  FilterCheckBoxHitTest(listSymptoms);
}
//----------------------------------------------------------------------------------------------------------------------
void __fastcall TframeAssessmentNeuro::listResponsesClick(TObject *Sender)
{
  FilterCheckBoxHitTest(listResponses);
}
//----------------------------------------------------------------------------------------------------------------------
void __fastcall TframeAssessmentNeuro::listSpeechClick(TObject *Sender)
{
  FilterCheckBoxHitTest(listSpeech);
}
//----------------------------------------------------------------------------------------------------------------------
void __fastcall TframeAssessmentNeuro::listDetailedClick(TObject *Sender)
{
  FilterCheckBoxHitTest(listDetailed);
}
//----------------------------------------------------------------------------------------------------------------------

