//---------------------------------------------------------------------------

#include <vcl.h>
#pragma hdrstop

#include "codeFrameAssessmentNotes.h"
#include "codeRecordPatientAssessment.h"
#include "codeRecordTaskNotes.h"
//----------------------------------------------------------------------------------------------------------------------
#pragma package(smart_init)
#pragma link "codeFrameAssessmentGeneric"
#pragma resource "*.dfm"
TframeAssessmentNotes *frameAssessmentNotes;
//----------------------------------------------------------------------------------------------------------------------
__fastcall TframeAssessmentNotes::TframeAssessmentNotes(TComponent* Owner)
        : TframeAssessmentGeneric(Owner), notes_filter(), editing_note(), ending_state(), editing_id(),
          notes_request(-1)
{
}
//----------------------------------------------------------------------------------------------------------------------
// Initialize - Overridden to set parent properties and setup task_notes filtered listview.
bool TframeAssessmentNotes::Initialize()
{
  // Set AssessmentGeneric properties
  SetBandCaption("Notes");
  SetPermission("task_notes");

  // Setup notes filter
  notes_filter.SetListViewHandle(listNotes);
  notes_filter.SetSecurity("task_notes");
  notes_filter.AddColumnSpec("Notes",       "task_notes",  as_text, true,  listNotes->Width - 16, sentenceString);
  notes_filter.AddColumnSpec("Current Id",  "current_id",  as_int,  false, 80);
  notes_filter.AddColumnSpec("Internal Id", "internal_id", as_int,  false, 80);
  notes_filter.FetchPreferences("assessment.notes.filter");
  notes_filter.SetSQLStatement("");
  notes_filter.RefreshDataSet();

  return TframeAssessmentGeneric::Initialize();
}
//----------------------------------------------------------------------------------------------------------------------
// Lock - Overridden to lock Add button and edit box.
void TframeAssessmentNotes::Lock()
{
  TframeAssessmentGeneric::Lock(); // lock parent controls
  btnAdd->Enabled = false;         // disable add button
  editTaskNote->Enabled = false;   // lock edit box 
}
//----------------------------------------------------------------------------------------------------------------------
// TaskIdChanging - Overridden to populate notes filter
void TframeAssessmentNotes::TaskIdChanging(const string &new_id)
{
  string sql;

  TframeAssessmentGeneric::TaskIdChanging(new_id);

  sql = "select task_notes.note, task_notes.current_id, task_notes.id "
        "from task_notes "
        "where task_notes.current_id >= 3 and task_notes.task_id = '" + new_id + "' ";

  notes_filter.SetSQLStatement(sql);
  notes_filter.RefreshDataSet();
  notes_filter.SetAddEdit(this, AddRecord (&TframeAssessmentNotes::FilterAddNote),
                                EditRecord(&TframeAssessmentNotes::FilterEditNote));
  notes_request = parent_form->RequestNotificationWhen("task_notes.task_id", new_id, rel_any, as_int);
}
//----------------------------------------------------------------------------------------------------------------------
// LockRecords - Overridden to lock task_notes record for editing.
bool TframeAssessmentNotes::LockRecords()
{
  if(!LockRecordForUpdate(generic_transaction, "task_notes", editing_id))
    return false;

  return TframeAssessmentGeneric::LockRecords();
}
//----------------------------------------------------------------------------------------------------------------------
// CanUnlock - Overridden to test for a selected task note
bool TframeAssessmentNotes::CanUnlock(bool &creating_new)
{
  editing_id = notes_filter.GetSelectedRecordID();

  if(!creating_new && editing_id == "")
  {
    parent_form->PSay("Please select a note to edit in the list first.", m_reminder, MB_OK);
    return false;
  }

  return TframeAssessmentGeneric::CanUnlock(creating_new);
}
//----------------------------------------------------------------------------------------------------------------------
// UpdateForEdit - Overridden to put fresh data into the editing_note map and on screen.
void TframeAssessmentNotes::UpdateForEdit()
{
  string sql = "select * from task_notes where id = '" + editing_id + "' ";

  if(!SqlToMap(generic_transaction, sql, editing_note))
    parent_form->PSay("Could not retrieve up-to-date information for note", m_warning, MB_OK);

  editTaskNote->Text = SentenceString(editing_note["note"]).c_str(); // NB: TformRecordTaskNotes uses SentenceString
  editTaskNote->Modified = false;

  TframeAssessmentGeneric::UpdateForEdit();
}
//----------------------------------------------------------------------------------------------------------------------
// SavingNew - Overridden to return status of editing_note as new record or unlocked record
bool TframeAssessmentNotes::SavingNew()
{
  return editing_note["id"] == "new";
}
//----------------------------------------------------------------------------------------------------------------------
// DoSaveData - Overridden to provide core saving logic.
bool TframeAssessmentNotes::DoSaveData()
{
  map<string, int> leave_alone;
  ending_state = editing_note;

  ending_state["current_id"]        = lookupTables->IdOf("current_types", "Current");
  ending_state["last_update_stamp"] = RightNow();
  ending_state["last_updated_by"]   = current_userid;
  ending_state["task_id"]           = task_id;
  ending_state["group_id"]          = group_id;
  ending_state["note"]              = editTaskNote->Text.c_str();
  leave_alone["note"]               = sql_no_uppercasing; // Don't uppercase note text

  return DoInsertOrUpdate(ending_state, leave_alone, "task_notes");
}
//----------------------------------------------------------------------------------------------------------------------
// SetViewMode - Overridden to activate viewing of task notes
void TframeAssessmentNotes::SetViewMode()
{
  TframeAssessmentGeneric::SetViewMode();
  btnAdd->Enabled = HaveUnlockPermission(true); // It is now possible to add a note
  editTaskNote->Enabled = false;                // But it is still not possible to type stuff.
  editing_note.clear();                         // Clear out the note map for safety

  // Reset box contents
  editTaskNote->Text = listNotes->Selected ? SentenceString(listNotes->Selected->Caption.c_str()).c_str() : "";
}
//----------------------------------------------------------------------------------------------------------------------
// SetEditMode - Overridden to activate editing of a task note
void TframeAssessmentNotes::SetEditMode()
{
  TframeAssessmentGeneric::SetEditMode();
  btnAdd->Enabled = false;                // Can't add notes right now
  editTaskNote->Enabled = true;           // Enable richedit control
}
//----------------------------------------------------------------------------------------------------------------------
// btnAddClick - Handler for toolbar button to add a new task note
void __fastcall TframeAssessmentNotes::btnAddClick(TObject *Sender)
{
  if(DoUnlock(true))
  {
    editTaskNote->Text = "";
    editTaskNote->Modified = false;
    editing_note.clear();
    editing_note["id"] = "new"; // Creating a new note
  }
}
//----------------------------------------------------------------------------------------------------------------------
// editTaskNoteChange - Handler for richedit control to track changes
void __fastcall TframeAssessmentNotes::editTaskNoteChange(TObject *Sender)
{
  if(frame_mode == mode_editing && editTaskNote->Modified)
    SetModified(true); // We now have unsaved data
}
//----------------------------------------------------------------------------------------------------------------------
// listNotesSelectItem - Handler for listview to display selected task note information
void __fastcall TframeAssessmentNotes::listNotesSelectItem(TObject *Sender, TListItem *Item, bool Selected)
{
  if(frame_mode == mode_viewing)
    editTaskNote->Text = (Item && Selected) ? SentenceString(Item->Caption.c_str()).c_str() : "";
}
//----------------------------------------------------------------------------------------------------------------------
// FilterAddNote - "Add" handler for the notes filter. We redirect this to the "Add a Note" code so that it works
// compatibly with the assessment frame.
void TframeAssessmentNotes::FilterAddNote()
{
  if(btnAdd->Enabled) // Only when you could add using the button.
    btnAddClick(NULL);
}
//----------------------------------------------------------------------------------------------------------------------
// FilterEditNote - Open the note in a task_notes record form for ordinary editing capability. Not encouraged, but
// allows adjustment of current_id's etc, which is important if a data entry mistake is made.
void TframeAssessmentNotes::FilterEditNote()
{
  // Open the task note record
  map<string, string> field_map;
  field_map["id"] = notes_filter.GetSelectedRecordID();

  FORM_TRY
    TformRecordTaskNotes *note_form = new TformRecordTaskNotes(NULL, field_map);
    note_form->Show();
  FORM_CATCH
}
//----------------------------------------------------------------------------------------------------------------------
// AtlasNotify - Overridden to refresh notes listview
bool TframeAssessmentNotes::AtlasNotify(int request_num, string matched_value)
{
  bool handled = false;

  if(request_num == notes_request)
  {
    notes_filter.RefreshDataSet();
    handled = true;
  }

  return handled;
}
//----------------------------------------------------------------------------------------------------------------------
// SendAtlas - Overridden to send out notifications for task_notes
void TframeAssessmentNotes::SendAtlas()
{
  bool new_record = (editing_note["id"] == "new");

  SendAtlasIfChanged(new_record, "task_notes", editing_note, ending_state);
}
//----------------------------------------------------------------------------------------------------------------------
// WriteReport - Write out a report containing all of the notes
bool TframeAssessmentNotes::WriteReport(TComponent *compositeReportCtrl)
{
  bool success = false;

  try
  {
    string sql =
      "select task_notes.note "
      "from task_notes "
      "where task_notes.current_id >= 3 and task_notes.task_id = '" + task_id + "' ";

    InitDataSet(sql);

    TformReportAssessment *reportForm = report_data.reportForm;
    TQuickRep *thisReport = reportForm->reportNotes;

    thisReport->DataSet = report_data.dbDataSet;
    SetupQRDBText(reportForm->txtNotes, "note");

    // NB: Need to pass it as TComponent to avoid problems with having QuickReport included in the headers
    TQRCompositeReport *compositeReport = dynamic_cast<TQRCompositeReport *>(compositeReportCtrl);
    if(compositeReport)
    {
      compositeReport->Reports->Add(thisReport);
      success = true;
    }
  }
  catch(...)
  {
  }

  return success;
}
//----------------------------------------------------------------------------------------------------------------------
