//----------------------------------------------------------------------------------------------------------------------

#include <vcl.h>
#pragma hdrstop

#include "codeFrameAssessmentReferrals.h"
#include "components.h"
#include "codeRecordPatientAssessment.h"
#include "codeRecordReferral.h"
#include "codeRecordReferralNote.h"
#include "codeSearchTaxEntity.h"
//----------------------------------------------------------------------------------------------------------------------
#pragma package(smart_init)
#pragma link "codeFrameAssessmentGeneric"
#pragma link "codeFrameFilterBar"
#pragma link "codePaddedBox"
#pragma resource "*.dfm"
TframeAssessmentReferrals *frameAssessmentReferrals;
//----------------------------------------------------------------------------------------------------------------------
__fastcall TframeAssessmentReferrals::TframeAssessmentReferrals(TComponent* Owner)
        : TframeAssessmentGeneric(Owner), refs_filter(), refnotes_filter(), editing_ref(), ending_state(),
          refs_request(-1), editing_id()
{
}
//----------------------------------------------------------------------------------------------------------------------
// Initialize - Overridden to set parent properties and setup filters.
bool TframeAssessmentReferrals::Initialize()
{
  // Set AssessmentGeneric properties
  SetBandCaption("Referrals");
  SetPermission("referrals");

  // Setup refs filter
  refs_filter.SetListViewHandle(listRefs);
  refs_filter.SetSecurity("referrals");
  refs_filter.AddColumnSpec("Date",                   "referrals",   as_date, true,   80);
  refs_filter.AddColumnSpec("Referred By (Facility)", "referrals",   as_text, true,  140);
  refs_filter.AddColumnSpec("Referred By (Provider)", "referrals",   as_text, true,  140);
  refs_filter.AddColumnSpec("Referred To (Facility)", "referrals",   as_text, true,  140);
  refs_filter.AddColumnSpec("Referred To (Provider)", "referrals",   as_text, true,  140);
  refs_filter.AddColumnSpec("Reason",                 "referrals",   as_text, true,  100);
  refs_filter.AddColumnSpec("Current Id",             "current_id",  as_int,  false,  80);
  refs_filter.AddColumnSpec("Internal Id",            "internal_id", as_int,  false,  80);
  refs_filter.FetchPreferences("assessment.refs.refs_filter");
  refs_filter.SetSQLStatement("");
  refs_filter.RefreshDataSet();

  // Setup notes filter
  refnotes_filter.SetListViewHandle(listRefNotes);
  refnotes_filter.SetSecurity("referral_notes");
  refnotes_filter.AddColumnSpec("Date",        "referral_notes", as_date, true,  100);
  refnotes_filter.AddColumnSpec("Note",        "referral_notes", as_text, true,  425, sentenceString);
  refnotes_filter.AddColumnSpec("Current Id",  "current_id",     as_int,  false,  80);
  refnotes_filter.AddColumnSpec("Internal Id", "internal_id",    as_int,  false,  80);
  refnotes_filter.FetchPreferences("assessment.refs.refnotes_filter");
  refnotes_filter.SetSQLStatement("");
  refnotes_filter.RefreshDataSet();

  // Register lookups
  string sql;
  string lookupName = "refs.cboStatus";
  string tableName  = "todo_status_types";
  RegisterLookup(lookupName, tableName, cboStatus);

  lookupName = "refs.cboReason";
  tableName  = "referral_reason_types";
  sql = string("select id, item, valid from referral_reason_types where group_id in (")
        + SetToCommaString(GetSpecialGroupPermissions("general")) + ") and valid = 1 order by item ";
  RegisterLookup(lookupName, tableName, cboReason, sql);

  // Set standard modified event handlers for all controls on panelRef
  SetAllEventHandlers(panelRef);

  // Setup the toolbar for the referral notes filter
  frameRefNoteBar->SetHandles(parent_form, &refnotes_filter);
  frameRefNoteBar->SetEditEnabled(false);
  frameRefNoteBar->SetAddEnabled(false);

  // Create the links initially blank
  BlankLinks();

  return TframeAssessmentGeneric::Initialize();
}
//----------------------------------------------------------------------------------------------------------------------
// BlankLinks - clears out all of the tax entity links on this frame
void TframeAssessmentReferrals::BlankLinks()
{
  CreateLink(txtByFacility, btnByFacilitySearch, parent_form, "", "tax_entities");
  CreateLink(txtByProvider, btnByProviderSearch, parent_form, "", "tax_entities");
  CreateLink(txtToFacility, btnToFacilitySearch, parent_form, "", "tax_entities");
  CreateLink(txtToProvider, btnToProviderSearch, parent_form, "", "tax_entities");
}
//----------------------------------------------------------------------------------------------------------------------
// Lock - Overridden to lock Add button and refs panel
void TframeAssessmentReferrals::Lock()
{
  TframeAssessmentGeneric::Lock();  // lock parent controls
  btnAdd->Enabled = false;          // disable add button
  panelRef->Enabled = false;        // lock all controls on the referral editing panel
  frameRefNoteBar->Enabled = false; // disable refnotes toolbar
}
//----------------------------------------------------------------------------------------------------------------------
// TaskIdChanging - Overridden to populate referrals filter
void TframeAssessmentReferrals::TaskIdChanging(const string &new_id)
{
  string sql;

  TframeAssessmentGeneric::TaskIdChanging(new_id);

  sql = "select REF.date_of_referral, by_facility.name, by_provider.name, to_facility.name, to_provider.name, "
        "RRT.item, REF.current_id, REF.id "
        "from referrals REF "
        "left join referral_reason_types RRT on REF.referral_reason_id = RRT.id "
        "left join tax_entities by_facility  on REF.by_facility_id = by_facility.id "
        "left join tax_entities by_provider  on REF.by_provider_id = by_provider.id "
        "left join tax_entities to_facility  on REF.to_facility_id = to_facility.id "
        "left join tax_entities to_provider  on REF.to_provider_id = to_provider.id "
        "where REF.current_id >= 2 and REF.task_id = '" + new_id + "' ";

  refs_filter.SetSQLStatement(sql);
  refs_filter.RefreshDataSet();
  refs_filter.SetAddEdit(this, AddRecord (&TframeAssessmentReferrals::FilterAddRef),
                               EditRecord(&TframeAssessmentReferrals::FilterEditRef));
  refs_request = parent_form->RequestNotificationWhen("referrals.task_id", new_id, rel_any, as_int);
}
//----------------------------------------------------------------------------------------------------------------------
// LockRecords - Overridden to lock referrals record for editing.
bool TframeAssessmentReferrals::LockRecords()
{
  if(!LockRecordForUpdate(generic_transaction, "referrals", editing_id))
    return false;

  return TframeAssessmentGeneric::LockRecords();
}
//----------------------------------------------------------------------------------------------------------------------
// CanUnlock - Overridden to test for a selected referral
bool TframeAssessmentReferrals::CanUnlock(bool &creating_new)
{
  editing_id = refs_filter.GetSelectedRecordID();

  if(!creating_new && editing_id == "")
  {
    parent_form->PSay("Please select a referral to edit in the list first.", m_reminder, MB_OK);
    return false;
  }

  return TframeAssessmentGeneric::CanUnlock(creating_new);
}
//----------------------------------------------------------------------------------------------------------------------
// UpdateForEdit - Overridden to put fresh data into the editing_ref map and on screen.
void TframeAssessmentReferrals::UpdateForEdit()
{
  string sql = "select * from referrals where id = '" + editing_id + "' ";

  if(!SqlToMap(generic_transaction, sql, editing_ref))
    parent_form->PSay("Could not retrieve up-to-date information for referral", m_warning, MB_OK);

  editDate->Text = editing_ref["date_of_referral"].c_str();

  string status = lookupTables->ValueOf("todo_status_types", editing_ref["todo_status_id"]);
  string reason = PrettyString(lookupTables->ValueOf("referral_reason_types", editing_ref["referral_reason_id"]));

  cboStatus->Text = status.c_str();
  cboReason->Text = reason.c_str();

  CreateLink(txtByFacility, btnByFacilitySearch, parent_form, editing_ref["by_facility_id"], "tax_entities");
  CreateLink(txtByProvider, btnByProviderSearch, parent_form, editing_ref["by_provider_id"], "tax_entities");
  CreateLink(txtToFacility, btnToFacilitySearch, parent_form, editing_ref["to_facility_id"], "tax_entities");
  CreateLink(txtToProvider, btnToProviderSearch, parent_form, editing_ref["to_provider_id"], "tax_entities");

  // Make sure all edits are marked unmodified
  ClearControlsModified(panelRef);

  TframeAssessmentGeneric::UpdateForEdit();
}
//----------------------------------------------------------------------------------------------------------------------
// SavingNew - Overridden to return status of editing_ref as new record or unlocked record
bool TframeAssessmentReferrals::SavingNew()
{
  return editing_ref["id"] == "new";
}
//----------------------------------------------------------------------------------------------------------------------
// IsValidOnScreen - Overridden to validate lookup and date values
bool TframeAssessmentReferrals::IsValidOnScreen()
{
  // check to make sure that the values entered in the combo boxe
  // corrospond to values in that look up table.
  // give the user a message box and prompt to re-enter if incorrect

  if(IsBlankOrZero(GetValidLinkId(txtByProvider)) && IsBlankOrZero(GetValidLinkId(txtByFacility)))
  {
    parent_form->PSay("Please enter either a facility or a provider by which the client was referred",
                      m_missing_data, MB_OK);
    return false;
  }

  if(IsBlankOrZero(GetValidLinkId(txtToProvider)) && IsBlankOrZero(GetValidLinkId(txtToFacility)))
  {
    parent_form->PSay("Please enter either a facility or a provider to which the client was referred",
                      m_missing_data, MB_OK);
    return false;
  }

  if(editDate->Text != "" && !Pdate(editDate->Text.c_str(), mdy | yy).IsValidDate())
  {
    parent_form->PSay("Please enter a valid date", m_invalid_data, MB_OK);
    return false;
  }

  if(!lookupTables->IsInLookUpTable("todo_status_types", cboStatus))
  {
    parent_form->PSay("Please enter a valid referral status", m_invalid_data, MB_OK);
    return false;
  }

  if(!lookupTables->IsInLookUpTable("referral_reason_types", cboReason))
  {
    parent_form->PSay("Please enter a valid referral reason", m_invalid_data, MB_OK);
    return false;
  }

  return TframeAssessmentGeneric::IsValidOnScreen();
}
//----------------------------------------------------------------------------------------------------------------------
// DoSaveData - Overridden to provide core saving logic.
bool TframeAssessmentReferrals::DoSaveData()
{
  pmapstrtoint leave_alone;
  ending_state = editing_ref;

  ending_state["last_update_stamp"] = RightNow();
  ending_state["last_updated_by"]   = current_userid;

  string by_provider_id = GetValidLinkId(txtByProvider);
  string by_facility_id = GetValidLinkId(txtByFacility);
  string to_provider_id = GetValidLinkId(txtToProvider);
  string to_facility_id = GetValidLinkId(txtToFacility);

  if(IsBlankOrZero(by_provider_id))
    ending_state["by_provider_id"] = "0";
  else
    ending_state["by_provider_id"] = by_provider_id;

  if(IsBlankOrZero(by_facility_id))
    ending_state["by_facility_id"] = "0";
  else
    ending_state["by_facility_id"] = by_facility_id;

  if(IsBlankOrZero(to_provider_id))
    ending_state["to_provider_id"] = "0";
  else
    ending_state["to_provider_id"] = to_provider_id;

  if(IsBlankOrZero(to_facility_id))
    ending_state["to_facility_id"] = "0";
  else
    ending_state["to_facility_id"] = to_facility_id;

  if(editDate->Text != "")
    ending_state["date_of_referral"] = Pdate(editDate->Text.c_str(), mdy | yy).OutDate(mdy | yyyy);
  else
    ending_state["date_of_referral"] = "";

  ending_state["todo_status_id"]     = lookupTables->IdOf("todo_status_types", cboStatus->Text.c_str());
  ending_state["referral_reason_id"] = lookupTables->IdOf("referral_reason_types", cboReason->Text.c_str());

  // Vary current_id by todo-status: Completed referrals are marked as "Expired"; anything else is "Current"
  if(ending_state["todo_status_id"] == lookupTables->IdOf("todo_status_types", "Complete"))
    ending_state["current_id"] = lookupTables->IdOf("current_types", "Expired");
  else
    ending_state["current_id"] = lookupTables->IdOf("current_types", "Current");

  // Not seen:
  // person_id - this either comes from the existing ref or is set in editing_ref on add
  // task_id   - this either comes from the existing ref or is set in editing_ref on add

  return DoInsertOrUpdate(ending_state, leave_alone, "referrals");
}
//----------------------------------------------------------------------------------------------------------------------
// SetViewMode - Overridden to activate viewing of referrals
void TframeAssessmentReferrals::SetViewMode()
{
  TframeAssessmentGeneric::SetViewMode();
  btnAdd->Enabled = HaveUnlockPermission(true); // It is now possible to add a referral
  panelRef->Enabled = false;                    // But it is still not possible to enter data.
  frameRefNoteBar->Enabled = true;              // Enable refnotes toolbar
  editing_ref.clear();                          // Clear out the ref map for safety

  // Reset control contents
  ClearControlsText(panelRef);
  ClearControlsModified(panelRef);
  BlankLinks();
}
//----------------------------------------------------------------------------------------------------------------------
// SetEditMode - Overridden to activate editing of a referral
void TframeAssessmentReferrals::SetEditMode()
{
  TframeAssessmentGeneric::SetEditMode();
  btnAdd->Enabled = false;                // Can't add referrals right now
  panelRef->Enabled = true;               // Enable panel controls
}
//----------------------------------------------------------------------------------------------------------------------
// btnAddClick - Handler for toolbar button to add a new referral
void __fastcall TframeAssessmentReferrals::btnAddClick(TObject *Sender)
{
  if(DoUnlock(true))
  {
    // Reset control contents
    ClearControlsText(panelRef);
    ClearControlsModified(panelRef);
    BlankLinks();

    editing_ref.clear();
    editing_ref["id"]        = "new";     // Creating a new referral
    editing_ref["person_id"] = person_id; // For a new referral, task and person are propagated from the
    editing_ref["task_id"]   = task_id;   // parent form's task record

    // Fill in the "By Provider" link automatically
    string user_taxentity_sql = "select tax_entity_id from users where id = '" + current_userid + "' ";
    string user_taxentity_id  = GetOneField(dbDatabase, user_taxentity_sql);
    CreateLink(txtByProvider, btnByProviderSearch, parent_form, user_taxentity_id, "tax_entities");
    editing_ref["by_provider_id"] = user_taxentity_id;
  }
}
//----------------------------------------------------------------------------------------------------------------------
// FilterAddRef - "Add" handler for the refs filter. We redirect this to the "Add a Referral" code so that it
// works compatibly with the assessment frame.
void TframeAssessmentReferrals::FilterAddRef()
{
  if(btnAdd->Enabled) // Only when you could add using the button.
    btnAddClick(NULL);
}
//----------------------------------------------------------------------------------------------------------------------
// FilterEditRef - Open the referral in a referrals record form for ordinary editing capability. Not encouraged,
// but allows adjustment of current_id's etc, which is important if a data entry mistake is made.
void TframeAssessmentReferrals::FilterEditRef()
{
  // Open the referrals record
  pmapstrs field_map;
  field_map["id"] = refs_filter.GetSelectedRecordID();

  FORM_TRY
    TformRecordReferral *ref_form = new TformRecordReferral(NULL, field_map);
    ref_form->Show();
  FORM_CATCH
}
//----------------------------------------------------------------------------------------------------------------------
// AtlasNotify - Overridden to refresh refs listview
bool TframeAssessmentReferrals::AtlasNotify(int request_num, string matched_value)
{
  bool handled = false;

  if(request_num == refs_request)
  {
    refs_filter.RefreshDataSet();
    handled = true;
  }

  return handled;
}
//----------------------------------------------------------------------------------------------------------------------
// SendAtlas - Overridden to send out notifications for referrals
void TframeAssessmentReferrals::SendAtlas()
{
  bool new_record = (editing_ref["id"] == "new");

  SendAtlasIfChanged(new_record, "referrals", editing_ref, ending_state);
}
//----------------------------------------------------------------------------------------------------------------------
// listRefsSelectItem - selection change event handler for the referrals filter. We repopulate the referral notes
// filter with notes for the currently selected referral.
void __fastcall TframeAssessmentReferrals::listRefsSelectItem(TObject *Sender, TListItem *Item, bool Selected)
{
  if(Item && Selected)
  {
    string sql;
    string ref_id = refs_filter.GetSelectedRecordID();

    if(!IsBlankOrZero(ref_id))
    {
      sql = "select NOTE.date_of_note, NOTE.note, NOTE.current_id, NOTE.id "
            "from referral_notes NOTE "
            "where NOTE.current_id >= 2 and NOTE.referral_id = '" + ref_id + "' ";

      refnotes_filter.SetSQLStatement(sql);
      refnotes_filter.RefreshDataSet();
      refnotes_filter.SetAddEdit(this, AddRecord (&TframeAssessmentReferrals::FilterAddRefNote),
                                       EditRecord(&TframeAssessmentReferrals::FilterEditRefNote));
      frameRefNoteBar->SetEditEnabled(true);
      frameRefNoteBar->SetAddEnabled(true);
    }
  }
}
//----------------------------------------------------------------------------------------------------------------------
// FilterAddRefNote - "Add" handler for the referral notes filter.
void TframeAssessmentReferrals::FilterAddRefNote()
{
  if(frame_mode == mode_viewing)
  {
    pmapstrs field_map;
    string ref_id = refs_filter.GetSelectedRecordID();

    if(!IsBlankOrZero(ref_id))
    {
      field_map["id"] = "new";
      field_map["referral_id"] = ref_id;
      FORM_TRY
        TformRecordReferralNote *note = new TformRecordReferralNote(NULL, field_map);
        note->ShowTaskModal(parent_form);
      FORM_CATCH
      listRefsSelectItem(NULL, listRefs->Selected, true);
    }
    else
    {
      parent_form->PSay("Please select a referral in the listview at the top of this frame in order to add a note to "
                        "that referral.", m_missing_data, MB_OK);
    }
  }
  else
  {
    // This is solely to avoid confusion and not because it would be an actual problem.
    parent_form->PSay("Please save or discard the referral you are working on first.", m_reminder, MB_OK);
  }
}
//----------------------------------------------------------------------------------------------------------------------
// FilterEditRefNote - Open the referral note in a referral notes record form for ordinary editing capability.
void TframeAssessmentReferrals::FilterEditRefNote()
{
  // Open the referrals record
  if(frame_mode == mode_viewing)
  {
    pmapstrs field_map;
    field_map["id"] = refnotes_filter.GetSelectedRecordID();

    FORM_TRY
      TformRecordReferralNote *note = new TformRecordReferralNote(NULL, field_map);
      note->ShowTaskModal(parent_form);
    FORM_CATCH
    listRefsSelectItem(NULL, listRefs->Selected, true);
  }
  else
  {
    // As above.
    parent_form->PSay("Please save or discard the referral you are working on first.", m_reminder, MB_OK);
  }
}
//----------------------------------------------------------------------------------------------------------------------
// WriteReport - Write out a report on the referrals
bool TframeAssessmentReferrals::WriteReport(TComponent *compositeReportCtrl)
{
  bool success = false;

  try
  {
    string meds_sql =
      "select REF.date_of_referral, "
      " case "
      "  when by_facility.name <> '' and by_provider.name <> '' then (by_facility.name || ' - ' || by_provider.name) "
      "  when by_facility.name <> '' then by_facility.name "
      "  when by_provider.name <> '' then by_provider.name "
      "  else '' "
      " end by_name, "
      " case "
      "  when to_facility.name <> '' and to_provider.name <> '' then (to_facility.name || ' - ' || to_provider.name) "
      "  when to_facility.name <> '' then to_facility.name "
      "  when to_provider.name <> '' then to_provider.name "
      "  else '' "
      " end to_name, "
      " RRT.item reason_type, TST.item todo_status, REF.id "
      "from referrals REF "
      "left join todo_status_types TST     on REF.todo_status_id = TST.id "
      "left join referral_reason_types RRT on REF.referral_reason_id = RRT.id "
      "left join tax_entities by_facility  on REF.by_facility_id = by_facility.id "
      "left join tax_entities by_provider  on REF.by_provider_id = by_provider.id "
      "left join tax_entities to_facility  on REF.to_facility_id = to_facility.id "
      "left join tax_entities to_provider  on REF.to_provider_id = to_provider.id "
      "where REF.current_id >= 3 and REF.task_id = '" + task_id + "' ";

    InitDataSet(meds_sql);

    TformReportAssessment *reportForm = report_data.reportForm;
    TQuickRep *thisReport = reportForm->reportRefs;

    thisReport->DataSet = report_data.dbDataSet;
    SetupQRDBText(reportForm->txtRefDate,   "date_of_referral");
    SetupQRDBText(reportForm->txtRefStatus, "todo_status");
    SetupQRDBText(reportForm->txtRefReason, "reason_type");
    SetupQRDBText(reportForm->txtRefBy,     "by_name");
    SetupQRDBText(reportForm->txtRefTo,     "to_name");

    // NB: Need to pass it as TComponent to avoid problems with having QuickReport included in the headers
    TQRCompositeReport *compositeReport = dynamic_cast<TQRCompositeReport *>(compositeReportCtrl);
    if(compositeReport)
    {
      compositeReport->Reports->Add(thisReport);
      success = true;
    }
  }
  catch(...)
  {
  }

  return success;
}
//----------------------------------------------------------------------------------------------------------------------
void __fastcall TframeAssessmentReferrals::btnByFacilitySearchClick(TObject *Sender)
{
  // create and show new person search form
  pvecstr return_vector;
  FORM_TRY
    TformSearchTaxEntity *facilitySearch = new TformSearchTaxEntity(NULL, &return_vector);
    facilitySearch->ShowTaskModal(parent_form);
  FORM_CATCH
  if(!return_vector.empty())
  {
    CreateLink(txtByFacility, btnByFacilitySearch, parent_form, return_vector[0], "tax_entities");
    SetModified(true);
  }
}
//----------------------------------------------------------------------------------------------------------------------
void __fastcall TframeAssessmentReferrals::btnByProviderSearchClick(TObject *Sender)
{
  // create and show new person search form
  pvecstr return_vector;
  FORM_TRY
    TformSearchTaxEntity *providerSearch = new TformSearchTaxEntity(NULL, &return_vector);
    providerSearch->ShowTaskModal(parent_form);
  FORM_CATCH
  if(!return_vector.empty())
  {
    CreateLink(txtByProvider, btnByProviderSearch, parent_form, return_vector[0], "tax_entities");
    SetModified(true);
  }
}
//----------------------------------------------------------------------------------------------------------------------
void __fastcall TframeAssessmentReferrals::btnToFacilitySearchClick(TObject *Sender)
{
  pvecstr return_vector;
  FORM_TRY
    TformSearchTaxEntity *facilitySearch = new TformSearchTaxEntity(NULL, &return_vector);
    facilitySearch->ShowTaskModal(parent_form);
  FORM_CATCH
  if(!return_vector.empty())
  {
    CreateLink(txtToFacility, btnToFacilitySearch, parent_form, return_vector[0], "tax_entities");
    SetModified(true);
  }
}
//----------------------------------------------------------------------------------------------------------------------
void __fastcall TframeAssessmentReferrals::btnToProviderSearchClick(TObject *Sender)
{
  // create and show new person search form
  pvecstr return_vector;
  FORM_TRY
    TformSearchTaxEntity *providerSearch = new TformSearchTaxEntity(NULL, &return_vector);
    providerSearch->ShowTaskModal(parent_form);
  FORM_CATCH
  if(!return_vector.empty())
  {
    CreateLink(txtToProvider, btnToProviderSearch, parent_form, return_vector[0], "tax_entities");
    SetModified(true);
  }
}
//----------------------------------------------------------------------------------------------------------------------

