//----------------------------------------------------------------------------------------------------------------------

#include <vcl.h>
#pragma hdrstop

#include "codeFrameAssessmentRespiratory.h"
#include "codeRecordPatientAssessment.h"
//----------------------------------------------------------------------------------------------------------------------
#pragma package(smart_init)
#pragma link "codeFrameAssessmentGeneric"
#pragma resource "*.dfm"
TframeAssessmentRespiratory *frameAssessmentRespiratory;
//----------------------------------------------------------------------------------------------------------------------
__fastcall TframeAssessmentRespiratory::TframeAssessmentRespiratory(TComponent* Owner)
        : TframeAssessmentGeneric(Owner), resp_filter(), editing_resp(), ending_state(), resp_request(-1), editing_id()
{
}
//----------------------------------------------------------------------------------------------------------------------
// respiratory_items data table
TframeAssessmentRespiratory::respiratory_items_data TframeAssessmentRespiratory::respiratory_items[] =
{
  { "Friction Rub", "check1R", "check1L" },
  { "Crackles",     "check2R", "check2L" },
  { "Wheezes",      "check3R", "check3L" },
  { "Absent",       "check4R", "check4L" },
  { "Diminished",   "check5R", "check5L" },
  { "Rhonchi",      "check6R", "check6L" },
  { "Chest Tube",   "check7R", "check7L" },

  // Must be last
  { NULL,           NULL,      NULL      }
};
//----------------------------------------------------------------------------------------------------------------------
// DataForRespiratoryID - static routine; returns a pointer to a respiratory_items_data structure given a
// respiratory_types ID. Returns NULL if invalid
TframeAssessmentRespiratory::respiratory_items_data *TframeAssessmentRespiratory::DataForRespiratoryID(const string &id)
{
  respiratory_items_data *item = respiratory_items;

  while(item->respiratory_type)
  {
    if(UppercaseString(item->respiratory_type) == UppercaseString(lookupTables->ValueOf("respiratory_types", id)))
      return item;

    ++item;
  }

  return NULL; // not found
}
//----------------------------------------------------------------------------------------------------------------------
// Initialize - Overridden to set parent properties and setup respiratory filtered listview.
bool TframeAssessmentRespiratory::Initialize()
{
  // Set AssessmentGeneric properties
  SetBandCaption("Respiratory");
  SetPermission("com_respiratory");

  // Setup respiratory filter
  resp_filter.SetListViewHandle(listResp);
  resp_filter.SetSecurity("com_respiratory");

  resp_filter.AddColumnSpec("NPI",          "com_respiratory", as_text,   true,   60, prettyBoolString);
  resp_filter.AddColumnSpec("Irregular",    "com_respiratory", as_text,   true,   60, prettyBoolString);
  resp_filter.AddColumnSpec("Labored",      "com_respiratory", as_text,   true,   60, prettyBoolString);
  resp_filter.AddColumnSpec("SOB",          "com_respiratory", as_text,   true,   50, prettyBoolString);
  resp_filter.AddColumnSpec("Suction",      "com_respiratory", as_text,   true,  110, rawString);
  resp_filter.AddColumnSpec("Oxygen",       "com_respiratory", as_text,   true,  110, rawString);
  resp_filter.AddColumnSpec("BiPap",        "com_respiratory", as_text,   true,  110, rawString);
  resp_filter.AddColumnSpec("Cough",        "com_respiratory", as_text,   true,  100);
  resp_filter.AddColumnSpec("Smokes PPD",   "com_respiratory", as_double, true,   80, amountString);
  resp_filter.AddColumnSpec("Smokes Years", "com_respiratory", as_int,    true,   80);
  resp_filter.AddColumnSpec("Comments",     "com_respiratory", as_text,   true,  110, rawString);
  resp_filter.AddColumnSpec("Current Id",   "current_id",      as_int,    false,  80);
  resp_filter.AddColumnSpec("Internal Id",  "internal_id",     as_int,    false,  80);

  resp_filter.FetchPreferences("assessment.resp.filter");
  resp_filter.SetSQLStatement("");
  resp_filter.RefreshDataSet();

  // Register lookups
  string lookupName = "resp.cboCough";
  string tableName  = "cough_types";
  RegisterLookup(lookupName, tableName, cboCough);

  // Set standard modified event handlers for all controls on panelResp
  SetAllEventHandlers(panelResp);
  SetCheckBoxHandlers(panelResp);

  return TframeAssessmentGeneric::Initialize();
}
//----------------------------------------------------------------------------------------------------------------------
// Lock - Overridden to lock Add button and resp panel
void TframeAssessmentRespiratory::Lock()
{
  TframeAssessmentGeneric::Lock(); // lock parent controls
  btnAdd->Enabled = false;         // disable add button
  panelResp->Enabled = false;      // lock all controls
}
//----------------------------------------------------------------------------------------------------------------------
// PutUpRespiratoryItems - loads the "Respiratory Problems" checkboxes with values from respiratory_items records
// associated with the editing com_respiratory
void TframeAssessmentRespiratory::PutUpRespiratoryItems(pvecmap &resp_items)
{
  ClearControlsChecked(groupProblems);

  for(size_t i = 0; i < resp_items.size(); i++)
  {
    // Skip items which are anything other than Current. This allows Deleted items to be updated back to Current status
    // when saving.
    if(resp_items[i]["current_id"] != lookupTables->IdOf("current_types", "Current"))
      continue;

    // find the respiratory_items_data for this id
    respiratory_items_data *data = DataForRespiratoryID(resp_items[i]["respiratory_id"]);

    if(data)
    {
      // determine which boxes to check
      if(resp_items[i]["body_side_id"] == lookupTables->IdOf("body_side_types", "Right") ||
         resp_items[i]["body_side_id"] == lookupTables->IdOf("body_side_types", "Both"))
      {
        TControl *myCheckBox = groupProblems->FindChildControl(data->controlNameR);
        if(myCheckBox && myCheckBox->InheritsFrom(__classid(TCheckBox)))
          dynamic_cast<TCheckBox *>(myCheckBox)->Checked = true;
      }
      if(resp_items[i]["body_side_id"] == lookupTables->IdOf("body_side_types", "Left") ||
         resp_items[i]["body_side_id"] == lookupTables->IdOf("body_side_types", "Both"))
      {
        TControl *myCheckBox = groupProblems->FindChildControl(data->controlNameL);
        if(myCheckBox && myCheckBox->InheritsFrom(__classid(TCheckBox)))
          dynamic_cast<TCheckBox *>(myCheckBox)->Checked = true;
      }
    }
  }
}
//----------------------------------------------------------------------------------------------------------------------
// TaskIdChanging - Overridden to populate respiratory filter
void TframeAssessmentRespiratory::TaskIdChanging(const string &new_id)
{
  string sql;

  TframeAssessmentGeneric::TaskIdChanging(new_id);

  sql = "select CR.is_noproblems, CR.is_irregular, CR.is_labored, CR.is_sob, CR.suction, CR.oxygen, CR.bipap, "
        " CT.item, CR.smoker_ppd, CR.smoker_years, CR.comments, CR.current_id, CR.id "
        "from com_respiratory CR "
        "left join cough_types CT on CR.cough_id = CT.id "
        "where CR.current_id >= 2 and CR.task_id = '" + new_id + "' ";

  resp_filter.SetSQLStatement(sql);
  resp_filter.RefreshDataSet();
  resp_filter.SetAddEdit(this, AddRecord (&TframeAssessmentRespiratory::FilterAddResp),
                               EditRecord(&TframeAssessmentRespiratory::FilterEditResp));
  resp_request = parent_form->RequestNotificationWhen("com_respiratory.task_id", new_id, rel_e, as_int);
}
//----------------------------------------------------------------------------------------------------------------------
// LockRecords - Overridden to lock com_respiratory record and respiratory_items subrecords for editing.
bool TframeAssessmentRespiratory::LockRecords()
{
  if(!LockRecordForUpdate(generic_transaction, "com_respiratory", editing_id))
    return false;

  // Must acquire all items. Any failure is a failure to unlock for editing, GTFO and rollback.
  if(!LockVecMapRecords(editing_resp_items, "respiratory_items"))
    return false;

  return TframeAssessmentGeneric::LockRecords();
}
//----------------------------------------------------------------------------------------------------------------------
// CanUnlock - Overridden to test for a selected respiratory assessment
bool TframeAssessmentRespiratory::CanUnlock(bool &creating_new)
{
  editing_id = resp_filter.GetSelectedRecordID();

  if(!creating_new && editing_id == "")
  {
    parent_form->PSay("Please select a respiratory assessment to edit in the list first.", m_reminder, MB_OK);
    return false;
  }

  return TframeAssessmentGeneric::CanUnlock(creating_new);
}
//----------------------------------------------------------------------------------------------------------------------
// UpdateForEdit - Overridden to put fresh data into the editing_resp map and on screen.
void TframeAssessmentRespiratory::UpdateForEdit()
{
  string sql = "select * from com_respiratory where id = '" + editing_id + "' ";

  if(!SqlToMap(generic_transaction, sql, editing_resp))
    parent_form->PSay("Could not retrieve up-to-date information for com_respiratory", m_warning, MB_OK);

  // Start all checkboxes cleared
  ClearControlsChecked(panelResp);

  // Combo boxes
  string cough_type = lookupTables->ValueOf("cough_types", editing_resp["cough_id"]);
  cboCough->Text = cough_type.c_str();

  // Edits & Memos
  editSuction->Text  = editing_resp["suction"     ].c_str();
  editOxygen->Text   = editing_resp["oxygen"      ].c_str();
  editBiPap->Text    = editing_resp["bipap"       ].c_str();
  editPpd->Text      = editing_resp["smoker_ppd"  ].c_str();
  editYears->Text    = editing_resp["smoker_years"].c_str();
  memoComments->Text = editing_resp["comments"    ].c_str();

  // Checkboxes (external record)
  checkNPI->Checked       = (editing_resp["is_noproblems"] == "1");
  checkIrregular->Checked = (editing_resp["is_irregular"]  == "1");
  checkLabored->Checked   = (editing_resp["is_labored"]    == "1");
  checkSOB->Checked       = (editing_resp["is_sob"]        == "1");

  // need also all respiratory_items
  editing_resp_items.clear();

  if(IsInt(editing_id) && !IsBlankOrZero(editing_id))
  {
    string sql =
      "select * "
      "from respiratory_items RI "
      "where RI.com_respiratory_id = '" + editing_id + "' ";

    if(!SqlToVecMap(generic_transaction, sql, editing_resp_items))
      parent_form->PSay("Could not retrieve up-to-date information for respiratory problems", m_warning, MB_OK);

    // Refresh controls
    PutUpRespiratoryItems(editing_resp_items);
  }

  // Make sure all edits are marked unmodified
  ClearControlsModified(panelResp);

  TframeAssessmentGeneric::UpdateForEdit();
}
//----------------------------------------------------------------------------------------------------------------------
// SavingNew - Overridden to return status of editing_resp as new record or unlocked record
bool TframeAssessmentRespiratory::SavingNew()
{
  return editing_resp["id"] == "new";
}
//----------------------------------------------------------------------------------------------------------------------
// GetRespItemsAsMap - Returns a map from respiratory_types ID to body_side_types ID for each set of checkboxes
void TframeAssessmentRespiratory::GetRespItemsAsMap(pmapstrs &to_return)
{
  respiratory_items_data *item = respiratory_items;

  to_return.clear();

  while(item->respiratory_type)
  {
    TControl  *myCheckBox;
    bool right_checked = false;
    bool left_checked  = false;
    string body_side_id;

    myCheckBox = groupProblems->FindChildControl(item->controlNameR);
    if(myCheckBox && myCheckBox->InheritsFrom(__classid(TCheckBox)))
      right_checked = dynamic_cast<TCheckBox *>(myCheckBox)->Checked;

    myCheckBox = groupProblems->FindChildControl(item->controlNameL);
    if(myCheckBox && myCheckBox->InheritsFrom(__classid(TCheckBox)))
      left_checked = dynamic_cast<TCheckBox *>(myCheckBox)->Checked;

    if(right_checked && left_checked)
      body_side_id = lookupTables->IdOf("body_side_types", "Both");
    else if(right_checked)
      body_side_id = lookupTables->IdOf("body_side_types", "Right");
    else if(left_checked)
      body_side_id = lookupTables->IdOf("body_side_types", "Left");
    else
      body_side_id = "0";

    to_return[lookupTables->IdOf("respiratory_types", item->respiratory_type)] = body_side_id;

    ++item;
  }
}
//----------------------------------------------------------------------------------------------------------------------
// IsValidOnScreen - Overridden to validate everything
bool TframeAssessmentRespiratory::IsValidOnScreen()
{
  pmapstrs resp_items;
  GetRespItemsAsMap(resp_items);

  // NPI is checked?
  if(checkNPI->Checked)
  {
    if(checkIrregular->Checked || checkLabored->Checked || checkSOB->Checked)
    {
      parent_form->PSay("You checked the patient as having No Known Problems, but also selected one of the other "
                        "problem indicators (irregular, labored, short of breath). Please clear one or more of the "
                        "conflicting indications.",
                        m_invalid_data, MB_OK);
      return false;
    }

    // check all respiratory problems are unchecked
    for(pmapstrs::iterator item = resp_items.begin(); item != resp_items.end(); ++item)
    {
      if(!IsBlankOrZero(item->second))
      {
        parent_form->PSay("You checked the patient as having No Known Problems, but also selected one or more adverse "
                          "conditions from the Respiratory Problems box. Please clear one or more of the conflicting "
                          "indications.",
                          m_invalid_data, MB_OK);
        return false;
      }
    }

    if(editSuction->Text != "" || editOxygen->Text != "" || editBiPap->Text != "")
    {
      parent_form->PSay("You checked the patient as having No Known Problems, but indicated that suction, oxygen, or "
                        "BiPap are in use. Please clear one or more of the conflicting indications.",
                        m_invalid_data, MB_OK);
      return false;
    }

    if(cboCough->Text != "")
    {
      parent_form->PSay("You checked the patient as having No Known Problems, but indicated the patient has a cough. "
                        "Please clear one or more of these conflicting indications.", m_invalid_data, MB_OK);
      return false;
    }

    // Allowed with NPI: Smoker status, generic comments
  }

  // NPI is NOT checked?
  if(!checkNPI->Checked)
  {
    bool all_problems_blank = true;

    for(pmapstrs::iterator item = resp_items.begin(); item != resp_items.end(); ++item)
    {
      if(!IsBlankOrZero(item->second))
      {
        all_problems_blank = false;
        break; // one is enough
      }
    }

    // Something else at least semi-structured must be provided that indicates a problem.
    if(!checkIrregular->Checked && !checkLabored->Checked && !checkSOB->Checked && editSuction->Text == "" &&
       editOxygen->Text == "" && editBiPap->Text == "" && cboCough->Text == "" && all_problems_blank)
    {
      parent_form->PSay("You did not check the No Known Problems indicator, but have not documented any issues with the "
                        "respiratory assessment. Please re-evaluate the patient status.", m_missing_data, MB_OK);
      return false;
    }
  }

  if(!lookupTables->IsInLookUpTable("cough_types", cboCough))
  {
    parent_form->PSay("Please enter a valid cough type", m_invalid_data, MB_OK);
    return false;
  }
  if(editPpd->Text != "" && !IsFloat(editPpd->Text.c_str()))
  {
    parent_form->PSay("Please enter a valid numeric value for Smoker Packs Per Day.", m_invalid_data, MB_OK);
    return false;
  }
  if(editYears->Text != "" && !IsInt(editYears->Text.c_str()))
  {
    parent_form->PSay("Please enter a valid integer numeric value for Smoker Years.", m_invalid_data, MB_OK);
    return false;
  }

  return TframeAssessmentGeneric::IsValidOnScreen();
}
//----------------------------------------------------------------------------------------------------------------------
// DoSaveData - Overridden to provide core saving logic.
bool TframeAssessmentRespiratory::DoSaveData()
{
  pmapstrtoint leave_alone;
  ending_state = editing_resp;

  ending_state["current_id"]        = lookupTables->IdOf("current_types", "Current");
  ending_state["last_update_stamp"] = RightNow();
  ending_state["last_updated_by"]   = current_userid;

  if(ending_state["id"] == "new")
  {
    ending_state["creation_stamp"] = ending_state["last_update_stamp"];
    ending_state["created_by"]     = ending_state["last_updated_by"];
  }

  // scalar checkboxes
  ending_state["is_noproblems"] = checkNPI->Checked       ? "1" : "0";
  ending_state["is_irregular"]  = checkIrregular->Checked ? "1" : "0";
  ending_state["is_labored"]    = checkLabored->Checked   ? "1" : "0";
  ending_state["is_sob"]        = checkSOB->Checked       ? "1" : "0";

  // plain text edits
  ending_state["suction"]  = editSuction->Text.c_str();
  ending_state["oxygen"]   = editOxygen->Text.c_str();
  ending_state["bipap"]    = editBiPap->Text.c_str();
  ending_state["comments"] = memoComments->Text.c_str();

  // do not capitalize these
  leave_alone["suction"]  = sql_no_uppercasing;
  leave_alone["oxygen"]   = sql_no_uppercasing;
  leave_alone["bipap"]    = sql_no_uppercasing;
  leave_alone["comments"] = sql_no_uppercasing;

  // numeric edits
  ending_state["smoker_ppd"]   = editPpd->Text.c_str();
  ending_state["smoker_years"] = editYears->Text.c_str();

  // lookups
  ending_state["cough_id"] = lookupTables->IdOf("cough_types", cboCough->Text.c_str());

  // Not seen:
  // task_id - this either comes from the existing com_respiratory or is set in editing_resp on add

  // Save the main record
  if(!DoInsertOrUpdate(ending_state, leave_alone, "com_respiratory"))
    return false; // BURP!

  // Delete existing respiratory_items (it really is truly the only manageable way to do this...)
  if(editing_resp["id"] != "new")
  {
    if(!ExecuteStatement(generic_transaction,
                         "delete from respiratory_items where com_respiratory_id = " + editing_resp["id"]))
    {
      return false; // BURP!
    }
  }

  // Insert new respiratory_items
  pmapstrs resp_items;
  GetRespItemsAsMap(resp_items);

  for(pmapstrs::iterator item = resp_items.begin(); item != resp_items.end(); ++item)
  {
    if(!IsBlankOrZero(item->second))
    {
      pmapstrs     new_resp_item;
      pmapstrtoint item_leave_alone;

      new_resp_item["id"] = GetNextId(generic_transaction, "respiratory_items");
      if(!IsInt(new_resp_item["id"]))
        return false; // BURP!

      new_resp_item["current_id"]         = lookupTables->IdOf("current_types", "Current");
      new_resp_item["com_respiratory_id"] = ending_state["id"];
      new_resp_item["body_side_id"]       = item->second;
      new_resp_item["respiratory_id"]     = item->first;

      if(!ExecuteInsertStatement(generic_transaction, "respiratory_items", new_resp_item, item_leave_alone))
        return false; // BURP!
    }
  }

  return true; // success!
}
//----------------------------------------------------------------------------------------------------------------------
// SetViewMode - Overridden to activate viewing of respirations
void TframeAssessmentRespiratory::SetViewMode()
{
  TframeAssessmentGeneric::SetViewMode();
  btnAdd->Enabled = HaveUnlockPermission(true); // It is now possible to add a respiratory assessment
  panelResp->Enabled = false;                   // But it is still not possible to enter data.
  editing_resp.clear();                         // Clear out the resp map for safety

  // Reset control contents
  ClearControlsText(panelResp);
  ClearControlsModified(panelResp);
  ClearControlsChecked(panelResp);
}
//----------------------------------------------------------------------------------------------------------------------
// SetEditMode - Overridden to activate editing of a respiratory assessment
void TframeAssessmentRespiratory::SetEditMode()
{
  TframeAssessmentGeneric::SetEditMode();
  btnAdd->Enabled = false;                // Can't add respiratory assessments right now
  panelResp->Enabled = true;              // Enable panel controls
}
//----------------------------------------------------------------------------------------------------------------------
// btnAddClick - Handler for toolbar button to add a new respiratory assessment
void __fastcall TframeAssessmentRespiratory::btnAddClick(TObject *Sender)
{
  if(DoUnlock(true))
  {
    // Reset control contents
    ClearControlsText(panelResp);
    ClearControlsModified(panelResp);
    ClearControlsChecked(panelResp);

    editing_resp.clear();
    editing_resp["id"] = "new"; // Creating a new com_respiratory
    editing_resp["task_id"] = task_id;
  }
}
//----------------------------------------------------------------------------------------------------------------------
// FilterAddResp - "Add" handler for the resp filter. We redirect this to the "Add a Respiration" code so that it
// works compatibly with the assessment frame.
void TframeAssessmentRespiratory::FilterAddResp()
{
  if(btnAdd->Enabled) // Only when you could add using the button.
    btnAddClick(NULL);
}
//----------------------------------------------------------------------------------------------------------------------
// FilterEditResp - Bleh.
void TframeAssessmentRespiratory::FilterEditResp()
{
  if(btnUnlockSave->Enabled && frame_mode == mode_viewing) // Only when you can start editing via the button.
    btnUnlockSaveClick(NULL);
}
//----------------------------------------------------------------------------------------------------------------------
// AtlasNotify - Overridden to refresh respiratory listview
bool TframeAssessmentRespiratory::AtlasNotify(int request_num, string matched_value)
{
  bool handled = false;

  if(request_num == resp_request)
  {
    resp_filter.RefreshDataSet();
    handled = true;
  }

  return handled;
}
//----------------------------------------------------------------------------------------------------------------------
// SendAtlas - Overridden to send out notifications for com_respiratory
void TframeAssessmentRespiratory::SendAtlas()
{
  bool new_record = (editing_resp["id"] == "new");

  SendAtlasIfChanged(new_record, "com_respiratory", editing_resp, ending_state);
}
//----------------------------------------------------------------------------------------------------------------------
// CaseForItem - utility for WriteReport
string TframeAssessmentRespiratory::CaseForItem(string body_side_type, string respiratory_type)
{
  return
    "case when exists "
    "(select * "
    " from respiratory_items RI "
    " where RI.com_respiratory_id = CR.id and RI.current_id >= 3 and "
    " RI.body_side_id = '"   + lookupTables->IdOf("body_side_types", body_side_type) + "' and "
    " RI.respiratory_id = '" + lookupTables->IdOf("respiratory_types", respiratory_type) + "') "
    "then 'X' else '' end ";
}
//----------------------------------------------------------------------------------------------------------------------
// WriteReport - Write out a report on the cardiovascular assessment
bool TframeAssessmentRespiratory::WriteReport(TComponent *compositeReportCtrl)
{
  bool success = false;

  try
  {
    string sql =
      "select "
      " case when CR.is_noproblems = 1 then 'X' else '' end npi_indicator, "
      " case when CR.is_irregular  = 1 then 'X' else '' end irreg_indicator, "
      " case when CR.is_labored    = 1 then 'X' else '' end labor_indicator, "
      " case when CR.is_sob        = 1 then 'X' else '' end sob_indicator, "
      " CR.suction, CR.oxygen, CR.bipap, CT.item cough_type, CR.smoker_ppd, CR.smoker_years, CR.comments, "
      + CaseForItem("Right", "Friction Rub") + " r_fric, "
      + CaseForItem("Right", "Crackles")     + " r_crack,  "
      + CaseForItem("Right", "Wheezes")      + " r_wheez, "
      + CaseForItem("Right", "Absent")       + " r_abs, "
      + CaseForItem("Right", "Diminished")   + " r_dim, "
      + CaseForItem("Right", "Rhonchi")      + " r_rhon, "
      + CaseForItem("Right", "Chest Tube")   + " r_tube,  "
      + CaseForItem("Left",  "Friction Rub") + " l_fric, "
      + CaseForItem("Left",  "Crackles")     + " l_crack,  "
      + CaseForItem("Left",  "Wheezes")      + " l_wheez, "
      + CaseForItem("Left",  "Absent")       + " l_abs, "
      + CaseForItem("Left",  "Diminished")   + " l_dim, "
      + CaseForItem("Left",  "Rhonchi")      + " l_rhon, "
      + CaseForItem("Left",  "Chest Tube")   + " l_tube  "
      "from com_respiratory CR "
      "left join cough_types CT on CR.cough_id = CT.id "
      "where CR.current_id >= 3 and CR.task_id = '" + task_id + "' ";

    InitDataSet(sql);

    TformReportAssessment *reportForm = report_data.reportForm;
    TQuickRep *thisReport = report_data.reportForm->reportResp;

    thisReport->DataSet = report_data.dbDataSet;

    txtregister_t ctrlsToRegister[] =
    {
      { reportForm->txtRespNPICheck,   "npi_indicator"   },
      { reportForm->txtRespIrregCheck, "irreg_indicator" },
      { reportForm->txtRespLabCheck,   "labor_indicator" },
      { reportForm->txtRespSOBCheck,   "sob_indicator"   },

      { reportForm->txtRespSuction,    "suction"         },
      { reportForm->txtRespOxygen,     "oxygen"          },
      { reportForm->txtRespBiPap,      "bipap"           },
      { reportForm->txtRespCough,      "cough_type"      },
      { reportForm->txtRespPpd,        "smoker_ppd"      },
      { reportForm->txtRespYears,      "smoker_years"    },
      { reportForm->txtRespComments,   "comments"        },

      { reportForm->txtRFric,          "r_fric"          },
      { reportForm->txtRCrack,         "r_crack"         },
      { reportForm->txtRWheez,         "r_wheez"         },
      { reportForm->txtRAbs,           "r_abs"           },
      { reportForm->txtRDim,           "r_dim"           },
      { reportForm->txtRRhon,          "r_rhon"          },
      { reportForm->txtRTube,          "r_tube"          },
      { reportForm->txtLFric,          "l_fric"          },
      { reportForm->txtLCrack,         "l_crack"         },
      { reportForm->txtLWheez,         "l_wheez"         },
      { reportForm->txtLAbs,           "l_abs"           },
      { reportForm->txtLDim,           "l_dim"           },
      { reportForm->txtLRhon,          "l_rhon"          },
      { reportForm->txtLTube,          "l_tube"          },
      { NULL,                           NULL             }
    };
    SetupQRDBTexts(ctrlsToRegister);

    // NB: Need to pass it as TComponent to avoid problems with having QuickReport included in the headers
    TQRCompositeReport *compositeReport = dynamic_cast<TQRCompositeReport *>(compositeReportCtrl);
    if(compositeReport)
    {
      compositeReport->Reports->Add(thisReport);
      success = true;
    }
  }
  catch(...)
  {
  }

  return success;
}
//----------------------------------------------------------------------------------------------------------------------

