//----------------------------------------------------------------------------------------------------------------------

#include <vcl.h>
#pragma hdrstop

#include "codeFrameAssessmentSafety.h"
#include "codeRecordPatientAssessment.h"
//----------------------------------------------------------------------------------------------------------------------
#pragma package(smart_init)
#pragma link "codeFrameAssessmentGeneric"
#pragma resource "*.dfm"
TframeAssessmentSafety *frameAssessmentSafety;
//----------------------------------------------------------------------------------------------------------------------
__fastcall TframeAssessmentSafety::TframeAssessmentSafety(TComponent* Owner)
        : TframeAssessmentGeneric(Owner), safety_filter(), checks_filter(), safety_request(-1), editing_safety_items(),
          final_ending_state()
{
}
//----------------------------------------------------------------------------------------------------------------------
// Initialize - Overridden to set parent properties and setup safety problems filtered listview.
bool TframeAssessmentSafety::Initialize()
{
  // Set AssessmentGeneric properties
  SetBandCaption("Safety");
  SetPermission("safety_problems");

  // Setup notes filter
  safety_filter.SetListViewHandle(listSafety);
  safety_filter.SetSecurity("safety_problems");

  safety_filter.AddColumnSpec("Item",        "safety_problems", as_text, true,  160, rawString);
  safety_filter.AddColumnSpec("Details",     "safety_problems", as_text, true,  420, rawString);
  safety_filter.AddColumnSpec("Current Id",  "current_id",      as_int,  false,  80);
  safety_filter.AddColumnSpec("Internal Id", "internal_id",     as_int,  false,  80);

  safety_filter.FetchPreferences("assessment.safety.topfilter");
  safety_filter.SetSQLStatement("");
  safety_filter.RefreshDataSet();

  // setup safety_types input filter

  string items_sql =
    "select item, id from safety_types where valid = 1 and id > 0 and "
    "id <> " + lookupTables->IdOf("safety_types", "No Problems Identified"); // don't include the NPI indicator

  // Checks Filter
  checks_filter.SetListViewHandle(listSafetyTypes);
  checks_filter.SetSecurity("");
  checks_filter.AddColumnSpec("Item",        "safety_problems", as_text, true,  168, rawString);
  checks_filter.AddColumnSpec("Internal Id", "internal_id",     as_int,  false,  80);
  checks_filter.FetchPreferences("assessment.safety.safety_types");
  checks_filter.SetIconStatus(false);
  checks_filter.SetCheck(true);
  checks_filter.SetSQLStatement(items_sql);
  checks_filter.RefreshDataSet();

  // Set edit box change handlers
  SetAllEventHandlers(panelSafety);

  return TframeAssessmentGeneric::Initialize();
}
//----------------------------------------------------------------------------------------------------------------------
// Lock - Overridden to lock Safety panel
void TframeAssessmentSafety::Lock()
{
  TframeAssessmentGeneric::Lock(); // lock parent controls
  panelSafety->Enabled = false;    // lock all panel controls
}
//----------------------------------------------------------------------------------------------------------------------
// PutUpSafetyItems - Refreshes the controls to reflect the state of the safety problems list
void TframeAssessmentSafety::PutUpSafetyItems(pvecmap &safety_items)
{
  psetstr safety_ids;

  // Clear out the checkboxes and edit box
  checks_filter.UncheckAll();
  editComments->Text = "";
  checkNPI->Checked = false;

  // Analyze the retrieved safety_problems items
  for(size_t i = 0; i < safety_items.size(); i++)
  {
    // Skip items which are anything other than Current. This allows Deleted items to be updated back to Current status
    // when saving.
    if(safety_items[i]["current_id"] != lookupTables->IdOf("current_types", "Current"))
      continue;

    if(safety_items[i]["safety_id"] == lookupTables->IdOf("safety_types", "No Problems Identified")) // NPI
      checkNPI->Checked = true;
    else
    {
      safety_ids.insert(safety_items[i]["safety_id"]);

      if(safety_items[i]["safety_id"] == lookupTables->IdOf("safety_types", "Other"))
        editComments->Text = safety_items[i]["comments"].c_str();
    }
  }

  // Set checkboxes
  checks_filter.SetCheckedWhereID(safety_ids, true);
}
//----------------------------------------------------------------------------------------------------------------------
// TaskIdChanging - Overridden to populate safety filter
void TframeAssessmentSafety::TaskIdChanging(const string &new_id)
{
  string sql;

  TframeAssessmentGeneric::TaskIdChanging(new_id);

  sql = "select ST.item, SP.comments, SP.current_id, SP.id "
        "from safety_problems SP "
        "left join safety_types ST on SP.safety_id = ST.id "
        "where SP.current_id >= 2 and SP.task_id = '" + new_id + "' ";

  safety_filter.SetSQLStatement(sql);
  safety_filter.RefreshDataSet();
  safety_filter.SetAddEdit(this, AddRecord (&TframeAssessmentSafety::FilterAddSafety),
                                 EditRecord(&TframeAssessmentSafety::FilterEditSafety));
  safety_request = parent_form->RequestNotificationWhen("safety_problems.task_id", new_id, rel_e, as_int);
}
//----------------------------------------------------------------------------------------------------------------------
// UpdateForEdit - Overridden to put fresh data into the editing_safety_items vecmap and on screen.
void TframeAssessmentSafety::UpdateForEdit()
{
  string sql =
    "select * "
    "from safety_problems SP "
    "where SP.task_id = '" + task_id + "' ";

  editing_safety_items.clear();

  if(!SqlToVecMap(generic_transaction, sql, editing_safety_items))
    parent_form->PSay("Could not retrieve up-to-date information for safety_problems items", m_warning, MB_OK);

  // Refresh controls
  PutUpSafetyItems(editing_safety_items);

  TframeAssessmentGeneric::UpdateForEdit();
}
//----------------------------------------------------------------------------------------------------------------------
// LockRecords - Overridden to lock the safety_problems records
bool TframeAssessmentSafety::LockRecords()
{
  // Must acquire all items. Any failure is a failure to unlock for editing, GTFO and rollback.
  if(!LockVecMapRecords(editing_safety_items, "safety_problems"))
    return false;

  return TframeAssessmentGeneric::LockRecords();
}
//----------------------------------------------------------------------------------------------------------------------
// IsValidOnScreen - Overridden to do validation for safety_problems
bool TframeAssessmentSafety::IsValidOnScreen()
{
  pvecstr checked_ids = checks_filter.GetMultiID();
  bool other_is_checked = false;
  
  // Make sure at least one safety item is specified
  if(!checkNPI->Checked &&
     editing_safety_items.size() == 0 && // Only when the record is new, otherwise can't delete last one
     checked_ids.size() == 0)
  {
    parent_form->PSay("You have not specified any saveable data. Please check one or more safety problems or the "
                      "'No Problems Identified' indicator.", m_missing_data, MB_OK);
    return false;
  }

  // If NPI is checked, nothing except Other can be specified in addition to it.
  if(checkNPI->Checked && checked_ids.size() > 0)
  {
    for(size_t i = 0; i < checked_ids.size(); i++)
    {
      if(checked_ids[i] != lookupTables->IdOf("safety_types", "Other"))
      {
        parent_form->PSay("You indicated that there were 'No Problems Identified', but also checked additional problems "
                          "(not including 'Other', which you can use to provide relevant information even if there "
                          "were no tangible problems).\n\n"
                          "Please uncheck either the 'No Problems Identified' indicator, or all items in the Safety "
                          "Problems list except 'Other'.", m_invalid_data, MB_OK);
        return false;
      }
    }
  }

  for(size_t i = 0; i < checked_ids.size(); i++)
  {
    // Still have to hard code it >_>
    if(checked_ids[i] == lookupTables->IdOf("safety_types", "Other"))
    {
      other_is_checked = true;
      if(editComments->Text == "")
      {
        parent_form->PSay("You selected 'Other' but did not provide any additional information on the safety problems.",
                          m_missing_data, MB_OK);
        return false;
      }
    }
  }

  if(!other_is_checked && editComments->Text != "")
  {
    parent_form->PSay("You did not select 'Other', but specified information in the 'Other Information' field. This "
                      "information would be lost, so please either clear it out now, or select the 'Other' item in "
                      "the checkbox list as well.", m_invalid_data, MB_OK);
    return false;
  }

  return TframeAssessmentGeneric::IsValidOnScreen();
}
//----------------------------------------------------------------------------------------------------------------------
bool TframeAssessmentSafety::UpdateSafetyItem(int record_index, string comment)
{
  pmapstrs ending_state;
  pmapstrtoint leave_alone;

  ending_state["id"]                 = editing_safety_items[record_index]["id"];
  ending_state["current_id"]         = lookupTables->IdOf("current_types", "Current");
  ending_state["last_updated_by"]    = current_userid;
  ending_state["last_update_stamp"]  = RightNow();
  ending_state["task_id"]            = task_id;
  ending_state["safety_id"]          = editing_safety_items[record_index]["safety_id"];
  ending_state["comments"]           = comment;

  leave_alone["comments"] = sql_no_uppercasing;

  final_ending_state.push_back(ending_state);

  return ExecuteUpdateStatement(generic_transaction, "safety_problems", ending_state, leave_alone);
}
//----------------------------------------------------------------------------------------------------------------------
bool TframeAssessmentSafety::InsertSafetyItem(string safety_id, string comment)
{
  pmapstrs ending_state;
  pmapstrtoint leave_alone;

  ending_state["id"]                = GetNextId(generic_transaction, "safety_problems");
  ending_state["current_id"]        = lookupTables->IdOf("current_types", "Current");
  ending_state["last_updated_by"]   = current_userid;
  ending_state["last_update_stamp"] = RightNow();
  ending_state["created_by"]        = current_userid;
  ending_state["creation_stamp"]    = ending_state["last_update_stamp"];
  ending_state["task_id"]           = task_id;
  ending_state["safety_id"]         = safety_id;
  ending_state["comments"]          = comment;

  leave_alone["comments"] = sql_no_uppercasing;

  final_ending_state.push_back(ending_state);

  return ExecuteInsertStatement(generic_transaction, "safety_problems", ending_state, leave_alone);
}
//----------------------------------------------------------------------------------------------------------------------
bool TframeAssessmentSafety::DeleteSafetyItem(int record_index)
{
  pmapstrs ending_state;
  pmapstrtoint leave_alone;

  ending_state["id"]                = editing_safety_items[record_index]["id"];
  ending_state["current_id"]        = lookupTables->IdOf("current_types", "Deleted"); // Mark as Deleted
  ending_state["last_updated_by"]   = current_userid;
  ending_state["last_update_stamp"] = RightNow();

  final_ending_state.push_back(ending_state);

  return ExecuteUpdateStatement(generic_transaction, "safety_problems", ending_state, leave_alone);
}
//----------------------------------------------------------------------------------------------------------------------
// DoSaveData - Overridden to provide core saving logic.
bool TframeAssessmentSafety::DoSaveData()
{
  psetstr checked_ids;
  psetstr unchecked_ids;
  int record_idx;
  string npiID = lookupTables->IdOf("safety_types", "No Problems Identified");

  final_ending_state.clear();

  // First determine what is checked and what is not in the filters.
  GetIDsFromCheckBoxFilter(checks_filter, checked_ids, unchecked_ids); // use parent class utility method

  // Add in the NPI
  if(checkNPI->Checked)
    checked_ids.insert(npiID);
  else
    unchecked_ids.insert(npiID);

  // Look at all IDs:

  // * If the ID is checked, and the ID is in the editing map, do an update on that record.
  // * If the ID is checked, and the ID is NOT in the editing map, do an insert.
  for(psetstr::iterator i_checked = checked_ids.begin(); i_checked != checked_ids.end(); ++i_checked)
  {
    string comments;

    if(*i_checked == lookupTables->IdOf("safety_types", "Other"))
      comments = editComments->Text.c_str();

    record_idx = IsInVecMap(editing_safety_items, "safety_id", *i_checked);
    if(record_idx >= 0)
    {
      // Do Update because this safety_problems item already exists.
      if(!UpdateSafetyItem(record_idx, comments))
        return false;
    }
    else
    {
      // Do Insert because this safety_problems item doesn't exist.
      if(!InsertSafetyItem(*i_checked, comments))
        return false;
    }
  }

  // * If the ID is NOT checked, and the id is in the editing map, mark that item as Deleted via the current_id.
  // * If the ID is NOT checked, and the id is NOT in the editing map, do nothing at all.
  for(psetstr::iterator i_unchecked = unchecked_ids.begin(); i_unchecked != unchecked_ids.end(); ++i_unchecked)
  {
    record_idx = IsInVecMap(editing_safety_items, "safety_id", *i_unchecked);
    if(record_idx >= 0)
    {
      // Do Update and set the existing safety_problems record to 'Deleted'.
      if(!DeleteSafetyItem(record_idx))
        return false;
    }
    // Otherwise, we neither had a record to begin with, nor do we have one to insert now, so do nothing.
  }

  // If we get this far, nothing borked up above.
  return true;
}
//----------------------------------------------------------------------------------------------------------------------
// SetViewMode - Overridden to activate viewing of safety_problems information
void TframeAssessmentSafety::SetViewMode()
{
  TframeAssessmentGeneric::SetViewMode();

  panelSafety->Enabled = false;                    // It is still not possible to enter data.
  editing_safety_items.clear();                    // Clear out the items vecmap for safety
  editComments->Text = "";
  checkNPI->Checked = false;
  checks_filter.UncheckAll();

  ClearControlsModified(panelSafety);
}
//----------------------------------------------------------------------------------------------------------------------
// SetEditMode - Overridden to activate editing of safety_problems records
void TframeAssessmentSafety::SetEditMode()
{
  TframeAssessmentGeneric::SetEditMode();

  panelSafety->Enabled = true;               // Enable panel controls

  ClearControlsModified(panelSafety);
}
//----------------------------------------------------------------------------------------------------------------------
// FilterAddSafety - "Add" handler for the safety_problems filter. This actually just activates editing mode here.
void TframeAssessmentSafety::FilterAddSafety()
{
  if(btnUnlockSave->Enabled && frame_mode == mode_viewing)
    btnUnlockSaveClick(NULL);
}
//----------------------------------------------------------------------------------------------------------------------
// FilterEditSafety - Same as above.
void TframeAssessmentSafety::FilterEditSafety()
{
  if(btnUnlockSave->Enabled && frame_mode == mode_viewing)
    btnUnlockSaveClick(NULL);
}
//----------------------------------------------------------------------------------------------------------------------
// AtlasNotify - Overridden to refresh safety_problems listview
bool TframeAssessmentSafety::AtlasNotify(int request_num, string matched_value)
{
  bool handled = false;

  if(request_num == safety_request)
  {
    safety_filter.RefreshDataSet();
    handled = true;
  }

  return handled;
}
//----------------------------------------------------------------------------------------------------------------------
// SendAtlas - Overridden to send out notifications for safety_problems
void TframeAssessmentSafety::SendAtlas()
{
  atlas->Notify("safety_problems.task_id", task_id); // Is this good enough?
}
//----------------------------------------------------------------------------------------------------------------------
// WriteReport - Write out a report on the safety problems
bool TframeAssessmentSafety::WriteReport(TComponent *compositeReportCtrl)
{
  bool success = false;

  try
  {
    string sql =
      "select ST.item, SP.comments "
      "from safety_problems SP "
      "left join safety_types ST on SP.safety_id = ST.id "
      "where SP.current_id >= 3 and SP.task_id = '" + task_id + "' ";

    InitDataSet(sql);

    TformReportAssessment *reportForm = report_data.reportForm;
    TQuickRep *thisReport = reportForm->reportSafety;

    thisReport->DataSet = report_data.dbDataSet;
    SetupQRDBText(reportForm->txtSafetyItem,   "item");
    SetupQRDBText(reportForm->txtSafetyDetail, "comments");

    // NB: Need to pass it as TComponent to avoid problems with having QuickReport included in the headers
    TQRCompositeReport *compositeReport = dynamic_cast<TQRCompositeReport *>(compositeReportCtrl);
    if(compositeReport)
    {
      compositeReport->Reports->Add(thisReport);
      success = true;
    }
  }
  catch(...)
  {
  }

  return success;
}
//----------------------------------------------------------------------------------------------------------------------
void __fastcall TframeAssessmentSafety::listSafetyTypesClick(TObject *Sender)
{
  // Use parent class hit test logic
  FilterCheckBoxHitTest(listSafetyTypes);
}
//----------------------------------------------------------------------------------------------------------------------
void __fastcall TframeAssessmentSafety::checkNPIClick(TObject *Sender)
{
  if(frame_mode == mode_editing)
    SetModified(true);
}
//----------------------------------------------------------------------------------------------------------------------
// btnReferralClick - Change the assessment form to the referrals area
void __fastcall TframeAssessmentSafety::btnReferralClick(TObject *Sender)
{
  parent_form->ChangeToArea(TformRecordPatientAssessment::area_referrals);
}
//----------------------------------------------------------------------------------------------------------------------

