//----------------------------------------------------------------------------------------------------------------------

#include <vcl.h>
#pragma hdrstop

#include "codeFrameAssessmentSensory.h"
#include "codeRecordPatientAssessment.h"
//----------------------------------------------------------------------------------------------------------------------
#pragma package(smart_init)
#pragma link "codeFrameAssessmentGeneric"
#pragma resource "*.dfm"
TframeAssessmentSensory *frameAssessmentSensory;
//----------------------------------------------------------------------------------------------------------------------
__fastcall TframeAssessmentSensory::TframeAssessmentSensory(TComponent* Owner)
        : TframeAssessmentGeneric(Owner), sensory_filter(), checks_filter(), sensory_request(-1), editing_sensory_items(),
          final_ending_state()
{
}
//----------------------------------------------------------------------------------------------------------------------
// Lookup Tables Lookup Table (heh-heh-heh)
static const char *sensoryLookupTables[][2] =
{
   // Combo Box Name, Lookup Table Value
   { "cboCataracts",  "Cataracts"       },
   { "cboGlaucoma",   "Glaucoma"        },
   { "cboProsthesis", "Prosthesis"      },
   { "cboDeafness",   "Deafness"        },
   { "cboHOH",        "Hard-of-Hearing" },
   { "cboHearingAid", "Hearing Aid"     },
   { "cboTinnitus",   "Tinnitus"        },
   { NULL,            NULL              }
};
//----------------------------------------------------------------------------------------------------------------------
// Initialize - Overridden to set parent properties and setup sensory problems filtered listview.
bool TframeAssessmentSensory::Initialize()
{
  // Set AssessmentGeneric properties
  SetBandCaption("Sensory");
  SetPermission("sensory_items");

  // Setup notes filter
  sensory_filter.SetListViewHandle(listSensory);
  sensory_filter.SetSecurity("sensory_items");

  sensory_filter.AddColumnSpec("Item",        "sensory_items", as_text, true,  160, rawString);
  sensory_filter.AddColumnSpec("Body Side",   "sensory_items", as_text, true,   80, rawString);
  sensory_filter.AddColumnSpec("Details",     "sensory_items", as_text, true,  360, rawString);
  sensory_filter.AddColumnSpec("Current Id",  "current_id",    as_int,  false,  80);
  sensory_filter.AddColumnSpec("Internal Id", "internal_id",   as_int,  false,  80);

  sensory_filter.FetchPreferences("assessment.sensory.topfilter");
  sensory_filter.SetSQLStatement("");
  sensory_filter.RefreshDataSet();

  // setup sensory_item_types input filter

  // NB: Excludes NPI, Cataracts, Glaucoma, Prosthesis, Deafness, Hard-of-Hearing, Hearing Aid, and Tinnitus
  string items_sql = "select item, id from sensory_item_types where valid = 1 and id > 0 and in_filter = 1 ";

  // Checks Filter
  checks_filter.SetListViewHandle(listSensoryTypes);
  checks_filter.SetSecurity("");
  checks_filter.AddColumnSpec("Item",        "sensory_items", as_text, true,  168, rawString);
  checks_filter.AddColumnSpec("Internal Id", "internal_id",   as_int,  false,  80);
  checks_filter.FetchPreferences("assessment.sensory.sensory_item_types");
  checks_filter.SetIconStatus(false);
  checks_filter.SetCheck(true);
  checks_filter.SetSQLStatement(items_sql);
  checks_filter.RefreshDataSet();

  // Register lookups - tablified above due to there being so many of them on this frame.
  for(int i = 0; ; i++)
  {
    if(sensoryLookupTables[i][0] == NULL)
      break; // finished.

    string controlName = sensoryLookupTables[i][0];
    string tableName   = "body_side_types";        // All on this frame use body_side_types lookup.
    string lookupName  = "sensory." + controlName; // build the unique name for parent class map functionality

    // Have to find the control on panelSensory
    TControl *theControl = panelSensory->FindChildControl(controlName.c_str());
    if(theControl && AnsiString(theControl->ClassName()) == "TComboBox")
    {
      TComboBox *theBox = dynamic_cast<TComboBox *>(theControl);
      RegisterLookup(lookupName, tableName, theBox);
    }
  }

  // Set edit box and combo box change handlers
  SetAllEventHandlers(panelSensory);

  return TframeAssessmentGeneric::Initialize();
}
//----------------------------------------------------------------------------------------------------------------------
// ComboBoxForSensoryType - Returns a TComboBox * given a sensory_item_types value, if one exists in the array;
//   otherwise, returns NULL.
TComboBox *TframeAssessmentSensory::ComboBoxForSensoryType(const string &sensory_type)
{
  for(int i = 0; ; i++)
  {
    if(sensoryLookupTables[i][0] == NULL)
      break; // finished.

    string controlName = sensoryLookupTables[i][0];
    string itemName    = UppercaseString(sensoryLookupTables[i][1]); // uppercase for insensitive comparison

    if(itemName == UppercaseString(sensory_type)) // Match?
    {
      // Have to find the control on panelSensory
      TControl *theControl = panelSensory->FindChildControl(controlName.c_str());
      if(theControl && AnsiString(theControl->ClassName()) == "TComboBox")
        return dynamic_cast<TComboBox *>(theControl);
    }
    // Keep lookin', bub.
  }

  return NULL; // Not found.
}
//----------------------------------------------------------------------------------------------------------------------
// Lock - Overridden to lock Sensory panel
void TframeAssessmentSensory::Lock()
{
  TframeAssessmentGeneric::Lock(); // lock parent controls
  panelSensory->Enabled = false;   // lock all panel controls
}
//----------------------------------------------------------------------------------------------------------------------
// PutUpSensoryItems - Refreshes the controls to reflect the state of the sensory problems list
void TframeAssessmentSensory::PutUpSensoryItems(pvecmap &sensory_items)
{
  psetstr  sensory_ids;
  pvecmap  bs_vecmap;
  pmapstrs need_body_sides;

  string bs_id_sql = "select id, need_body_side from sensory_item_types";
  SqlToVecMap(dbDatabase, bs_id_sql, bs_vecmap);

  // Build a map for fast, convenient lookup
  for(pvecmap::iterator i_bs = bs_vecmap.begin(); i_bs != bs_vecmap.end(); ++i_bs)
    need_body_sides[(*i_bs)["id"]] = (*i_bs)["need_body_side"];

  // Clear out the checkboxes and edit box
  checks_filter.UncheckAll();
  ClearControlsText(panelSensory);
  checkNPI->Checked = false;

  // Analyze the retrieved sensory_items items
  for(size_t i = 0; i < sensory_items.size(); i++)
  {
    // Skip items which are anything other than Current. This allows Deleted items to be updated back to Current status
    // when saving.
    if(sensory_items[i]["current_id"] != lookupTables->IdOf("current_types", "Current"))
      continue;

    string sensory_item_id = sensory_items[i]["sensory_item_id"];

    if(sensory_item_id == lookupTables->IdOf("sensory_item_types", "No Problems Identified")) // NPI
    {
      // Just set the checkbox
      checkNPI->Checked = true;
    }
    else if(sensory_item_id == lookupTables->IdOf("sensory_item_types", "Other Vision"))
    {
      // Add to ID set and populate memo
      sensory_ids.insert(sensory_item_id);
      memoOtherVision->Text = sensory_items[i]["comments"].c_str();
    }
    else if(sensory_item_id == lookupTables->IdOf("sensory_item_types", "Other Hearing"))
    {
      // Add to ID set and populate memo
      sensory_ids.insert(sensory_item_id);
      memoOtherHearing->Text = sensory_items[i]["comments"].c_str();
    }
    else if(need_body_sides[sensory_item_id] == "0")
    {
      // Just put it in the ID set
      sensory_ids.insert(sensory_item_id);
    }
    else
    {
      // Do NOT put it in the ID set; instead, populate its combo box
      // NB: Includes Cataracts, Glaucoma, Prosthesis, Deafness, HoH, Hearing Aid, Tinnitus
      TComboBox *theBox = ComboBoxForSensoryType(lookupTables->ValueOf("sensory_item_types", sensory_item_id));
      if(theBox)
        theBox->Text = lookupTables->ValueOf("body_side_types", sensory_items[i]["body_side_id"]).c_str();
    }
  }

  // Set checkboxes
  checks_filter.SetCheckedWhereID(sensory_ids, true);
}
//----------------------------------------------------------------------------------------------------------------------
// TaskIdChanging - Overridden to populate safety filter
void TframeAssessmentSensory::TaskIdChanging(const string &new_id)
{
  string sql;

  TframeAssessmentGeneric::TaskIdChanging(new_id);

  sql = "select SIT.item, BST.item, SI.comments, SI.current_id, SI.id "
        "from sensory_items SI "
        "left join sensory_item_types SIT on SI.sensory_item_id = SIT.id "
        "left join body_side_types BST on SI.body_side_id = BST.id "
        "where SI.current_id >= 2 and SI.task_id = '" + new_id + "' ";

  sensory_filter.SetSQLStatement(sql);
  sensory_filter.RefreshDataSet();
  sensory_filter.SetAddEdit(this, AddRecord(&TframeAssessmentSensory::FilterAddSensory),
                                 EditRecord(&TframeAssessmentSensory::FilterEditSensory));
  sensory_request = parent_form->RequestNotificationWhen("sensory_items.task_id", new_id, rel_e, as_int);
}
//----------------------------------------------------------------------------------------------------------------------
// UpdateForEdit - Overridden to put fresh data into the editing_sensory_items vecmap and on screen.
void TframeAssessmentSensory::UpdateForEdit()
{
  string sql =
    "select * "
    "from sensory_items SI "
    "where SI.task_id = '" + task_id + "' ";

  editing_sensory_items.clear();

  if(!SqlToVecMap(generic_transaction, sql, editing_sensory_items))
    parent_form->PSay("Could not retrieve up-to-date information for sensory_items", m_warning, MB_OK);

  // Refresh controls
  PutUpSensoryItems(editing_sensory_items);

  TframeAssessmentGeneric::UpdateForEdit();
}
//----------------------------------------------------------------------------------------------------------------------
// LockRecords - Overridden to lock the sensory_items records
bool TframeAssessmentSensory::LockRecords()
{
  // Must acquire all items. Any failure is a failure to unlock for editing, GTFO and rollback.
  if(!LockVecMapRecords(editing_sensory_items, "sensory_items"))
    return false;

  return TframeAssessmentGeneric::LockRecords();
}
//----------------------------------------------------------------------------------------------------------------------
// IsValidOnScreen - Overridden to do validation for sensory_items
bool TframeAssessmentSensory::IsValidOnScreen()
{
  pvecstr checked_ids = checks_filter.GetMultiID();
  bool other_hearing_checked = false, other_vision_checked = false;
  
  // Make sure at least one sensory item is specified
  if(!checkNPI->Checked &&
     editing_sensory_items.size() == 0 && // Only when the record is new, otherwise can't delete last one
     checked_ids.size() == 0 &&
     cboCataracts->Text == "" && cboGlaucoma->Text == "" && cboProsthesis->Text == "" && cboDeafness->Text == "" &&
     cboHOH->Text == "" && cboHearingAid->Text == "" && cboTinnitus->Text == "")
  {
    parent_form->PSay("You have not specified any saveable data. Please check one or more sensory problems or the "
                      "'No Problems Identified' indicator.", m_missing_data, MB_OK);
    return false;
  }

  // If NPI is checked, nothing except Other Hearing and Other Vision can be specified in addition to it.
  if(checkNPI->Checked && checked_ids.size() > 0)
  {
    for(size_t i = 0; i < checked_ids.size(); i++)
    {
      if(checked_ids[i] != lookupTables->IdOf("sensory_item_types", "Other Hearing") &&
         checked_ids[i] != lookupTables->IdOf("sensory_item_types", "Other Vision"))
      {
        parent_form->PSay("You indicated that there were 'No Problems Identified', but also checked additional problems "
                          "(not including 'Other Hearing/Vision', which you can use to provide relevant information even "
                          "if there were no tangible problems).\n\n"
                          "Please uncheck either the 'No Problems Identified' indicator, or all items in the Sensory "
                          "Problems list except 'Other Hearing/Vision'.", m_invalid_data, MB_OK);
        return false;
      }
    }
  }

  for(size_t i = 0; i < checked_ids.size(); i++)
  {
    // Still have to hard code it >_>
    if(checked_ids[i] == lookupTables->IdOf("sensory_item_types", "Other Hearing"))
    {
      other_hearing_checked = true;
      if(memoOtherHearing->Text == "")
      {
        parent_form->PSay("You selected 'Other Hearing' but did not provide any additional information on the sensory problem.",
                          m_missing_data, MB_OK);
        return false;
      }
    }
    if(checked_ids[i] == lookupTables->IdOf("sensory_item_types", "Other Vision"))
    {
      other_vision_checked = true;
      if(memoOtherVision->Text == "")
      {
        parent_form->PSay("You selected 'Other Vision' but did not provide any additional information on the sensory problem.",
                          m_missing_data, MB_OK);
        return false;
      }
    }
  }

  if(!other_hearing_checked && memoOtherHearing->Text != "")
  {
    parent_form->PSay("You did not select 'Other Hearing', but specified information in the 'Other Hearing' field. This "
                      "information would be lost, so please either clear it out now, or select the 'Other Hearing' item in "
                      "the checkbox list as well.", m_invalid_data, MB_OK);
    return false;
  }
  if(!other_vision_checked && memoOtherVision->Text != "")
  {
    parent_form->PSay("You did not select 'Other Vision', but specified information in the 'Other Vision' field. This "
                      "information would be lost, so please either clear it out now, or select the 'Other Vision' item in "
                      "the checkbox list as well.", m_invalid_data, MB_OK);
    return false;
  }

  return TframeAssessmentGeneric::IsValidOnScreen();
}
//----------------------------------------------------------------------------------------------------------------------
bool TframeAssessmentSensory::UpdateSensoryItem(int record_index, string body_side_id, string comment)
{
  pmapstrs ending_state;
  pmapstrtoint leave_alone;

  ending_state["id"]                 = editing_sensory_items[record_index]["id"];
  ending_state["current_id"]         = lookupTables->IdOf("current_types", "Current");
  ending_state["last_updated_by"]    = current_userid;
  ending_state["last_update_stamp"]  = RightNow();
  ending_state["task_id"]            = task_id;
  ending_state["sensory_item_id"]    = editing_sensory_items[record_index]["sensory_item_id"];
  ending_state["body_side_id"]       = body_side_id;
  ending_state["comments"]           = comment;

  leave_alone["comments"] = sql_no_uppercasing;

  final_ending_state.push_back(ending_state);

  return ExecuteUpdateStatement(generic_transaction, "sensory_items", ending_state, leave_alone);
}
//----------------------------------------------------------------------------------------------------------------------
bool TframeAssessmentSensory::InsertSensoryItem(string sensory_item_id, string body_side_id, string comment)
{
  pmapstrs ending_state;
  pmapstrtoint leave_alone;

  ending_state["id"]                = GetNextId(generic_transaction, "sensory_items");
  ending_state["current_id"]        = lookupTables->IdOf("current_types", "Current");
  ending_state["last_updated_by"]   = current_userid;
  ending_state["last_update_stamp"] = RightNow();
  ending_state["created_by"]        = current_userid;
  ending_state["creation_stamp"]    = ending_state["last_update_stamp"];
  ending_state["task_id"]           = task_id;
  ending_state["sensory_item_id"]   = sensory_item_id;
  ending_state["body_side_id"]      = body_side_id;
  ending_state["comments"]          = comment;

  leave_alone["comments"] = sql_no_uppercasing;

  final_ending_state.push_back(ending_state);

  return ExecuteInsertStatement(generic_transaction, "sensory_items", ending_state, leave_alone);
}
//----------------------------------------------------------------------------------------------------------------------
bool TframeAssessmentSensory::DeleteSensoryItem(int record_index)
{
  pmapstrs ending_state;
  pmapstrtoint leave_alone;

  ending_state["id"]                = editing_sensory_items[record_index]["id"];
  ending_state["current_id"]        = lookupTables->IdOf("current_types", "Deleted"); // Mark as Deleted
  ending_state["last_updated_by"]   = current_userid;
  ending_state["last_update_stamp"] = RightNow();

  final_ending_state.push_back(ending_state);

  return ExecuteUpdateStatement(generic_transaction, "sensory_items", ending_state, leave_alone);
}
//----------------------------------------------------------------------------------------------------------------------
// DoSaveData - Overridden to provide core saving logic.
bool TframeAssessmentSensory::DoSaveData()
{
  psetstr checked_ids;
  psetstr unchecked_ids;
  int record_idx;
  string npiID = lookupTables->IdOf("sensory_item_types", "No Problems Identified");
  pmapstrs body_side_for_sensory_item;

  final_ending_state.clear();

  // First determine what is checked and what is not in the filters.
  GetIDsFromCheckBoxFilter(checks_filter, checked_ids, unchecked_ids); // use parent class utility method

  // Add in the NPI
  if(checkNPI->Checked)
    checked_ids.insert(npiID);
  else
    unchecked_ids.insert(npiID);

  // Add in others based on combo boxen
  for(int i = 0; ; i++)
  {
    if(sensoryLookupTables[i][0] == NULL)
      break; // finished

    string controlName = sensoryLookupTables[i][0]; // name of cbo containing a body_side_types value
    string sensoryItem = sensoryLookupTables[i][1]; // name of the sensory_item_types value to which this corresponds

    TComboBox *theBox = NULL;
    TControl  *theControl = panelSensory->FindChildControl(controlName.c_str());
    if(theControl && AnsiString(theControl->ClassName()) == "TComboBox")
      theBox = dynamic_cast<TComboBox *>(theControl);

    if(theBox)
    {
      string sensory_item_id = lookupTables->IdOf("sensory_item_types", sensoryItem);
      if(theBox->Text != "")
      {
        checked_ids.insert(sensory_item_id);
        // Build a map of body_side_types IDs for the sensory items that need them, for update/insert below
        body_side_for_sensory_item[sensory_item_id] = lookupTables->IdOf("body_side_types", theBox);
      }
      else
        unchecked_ids.insert(sensory_item_id);
    }
  }

  // Look at all IDs:

  // * If the ID is checked, and the ID is in the editing map, do an update on that record.
  // * If the ID is checked, and the ID is NOT in the editing map, do an insert.
  for(psetstr::iterator i_checked = checked_ids.begin(); i_checked != checked_ids.end(); ++i_checked)
  {
    string comments;
    string body_side_id = "0";

    // Other Vision and Other Hearing support a comment
    if(*i_checked == lookupTables->IdOf("sensory_item_types", "Other Hearing"))
      comments = memoOtherHearing->Text.c_str();
    if(*i_checked == lookupTables->IdOf("sensory_item_types", "Other Vision"))
      comments = memoOtherVision->Text.c_str();

    // Get the body_side_id extracted from the combo boxes, if this sensory item type needs one.
    if(body_side_for_sensory_item[*i_checked] != "")
      body_side_id = body_side_for_sensory_item[*i_checked];

    record_idx = IsInVecMap(editing_sensory_items, "sensory_item_id", *i_checked);
    if(record_idx >= 0)
    {
      // Do Update because this sensory_items item already exists.
      if(!UpdateSensoryItem(record_idx, body_side_id, comments))
        return false;
    }
    else
    {
      // Do Insert because this sensory_items item doesn't exist.
      if(!InsertSensoryItem(*i_checked, body_side_id, comments))
        return false;
    }
  }

  // * If the ID is NOT checked, and the id is in the editing map, mark that item as Deleted via the current_id.
  // * If the ID is NOT checked, and the id is NOT in the editing map, do nothing at all.
  for(psetstr::iterator i_unchecked = unchecked_ids.begin(); i_unchecked != unchecked_ids.end(); ++i_unchecked)
  {
    record_idx = IsInVecMap(editing_sensory_items, "sensory_item_id", *i_unchecked);
    if(record_idx >= 0)
    {
      // Do Update and set the existing sensory_items record to 'Deleted'.
      if(!DeleteSensoryItem(record_idx))
        return false;
    }
    // Otherwise, we neither had a record to begin with, nor do we have one to insert now, so do nothing.
  }

  // If we get this far, nothing borked up above.
  return true;
}
//----------------------------------------------------------------------------------------------------------------------
// SetViewMode - Overridden to activate viewing of sensory_items information
void TframeAssessmentSensory::SetViewMode()
{
  TframeAssessmentGeneric::SetViewMode();

  panelSensory->Enabled = false;                    // It is still not possible to enter data.
  editing_sensory_items.clear();                    // Clear out the items vecmap for safety
  checkNPI->Checked = false;
  checks_filter.UncheckAll();

  ClearControlsText(panelSensory);
  ClearControlsModified(panelSensory);
}
//----------------------------------------------------------------------------------------------------------------------
// SetEditMode - Overridden to activate editing of sensory_items records
void TframeAssessmentSensory::SetEditMode()
{
  TframeAssessmentGeneric::SetEditMode();

  panelSensory->Enabled = true;               // Enable panel controls

  ClearControlsModified(panelSensory);
}
//----------------------------------------------------------------------------------------------------------------------
// FilterAddSensory - "Add" handler for the sensory_items filter. This actually just activates editing mode here.
void TframeAssessmentSensory::FilterAddSensory()
{
  if(btnUnlockSave->Enabled && frame_mode == mode_viewing)
    btnUnlockSaveClick(NULL);
}
//----------------------------------------------------------------------------------------------------------------------
// FilterEditSensory - Same as above.
void TframeAssessmentSensory::FilterEditSensory()
{
  if(btnUnlockSave->Enabled && frame_mode == mode_viewing)
    btnUnlockSaveClick(NULL);
}
//----------------------------------------------------------------------------------------------------------------------
// AtlasNotify - Overridden to refresh sensory_items listview
bool TframeAssessmentSensory::AtlasNotify(int request_num, string matched_value)
{
  bool handled = false;

  if(request_num == sensory_request)
  {
    sensory_filter.RefreshDataSet();
    handled = true;
  }

  return handled;
}
//----------------------------------------------------------------------------------------------------------------------
// SendAtlas - Overridden to send out notifications for sensory_items
void TframeAssessmentSensory::SendAtlas()
{
  atlas->Notify("sensory_items.task_id", task_id); // Is this good enough?
}
//----------------------------------------------------------------------------------------------------------------------
// WriteReport - Write out a report on the sensory conditions
bool TframeAssessmentSensory::WriteReport(TComponent *compositeReportCtrl)
{
  bool success = false;

  try
  {
    string sql =
      "select SIT.item item_type, BST.item bs_type, SI.comments "
      "from sensory_items SI "
      "left join sensory_item_types SIT on SI.sensory_item_id = SIT.id "
      "left join body_side_types BST on SI.body_side_id = BST.id "
      "where SI.current_id >= 3 and SI.task_id = '" + task_id + "' ";

    InitDataSet(sql);

    TformReportAssessment *reportForm = report_data.reportForm;
    TQuickRep *thisReport = reportForm->reportSensory;

    thisReport->DataSet = report_data.dbDataSet;
    SetupQRDBText(reportForm->txtSensoryItem,   "item_type");
    SetupQRDBText(reportForm->txtSensoryBS,     "bs_type");
    SetupQRDBText(reportForm->txtSensoryDetail, "comments");

    // NB: Need to pass it as TComponent to avoid problems with having QuickReport included in the headers
    TQRCompositeReport *compositeReport = dynamic_cast<TQRCompositeReport *>(compositeReportCtrl);
    if(compositeReport)
    {
      compositeReport->Reports->Add(thisReport);
      success = true;
    }
  }
  catch(...)
  {
  }

  return success;
}
//----------------------------------------------------------------------------------------------------------------------
void __fastcall TframeAssessmentSensory::listSensoryTypesClick(TObject *Sender)
{
  // Use parent class hit test logic
  FilterCheckBoxHitTest(listSensoryTypes);
}
//----------------------------------------------------------------------------------------------------------------------
void __fastcall TframeAssessmentSensory::checkNPIClick(TObject *Sender)
{
  if(frame_mode == mode_editing)
    SetModified(true);
}
//----------------------------------------------------------------------------------------------------------------------

