//----------------------------------------------------------------------------------------------------------------------

#include <vcl.h>
#pragma hdrstop

#include "codeFrameAssessmentSkin.h"
#include "codeRecordPatientAssessment.h"
//----------------------------------------------------------------------------------------------------------------------
#pragma package(smart_init)
#pragma link "codeFrameAssessmentGeneric"
#pragma resource "*.dfm"
TframeAssessmentSkin *frameAssessmentSkin;
//----------------------------------------------------------------------------------------------------------------------
__fastcall TframeAssessmentSkin::TframeAssessmentSkin(TComponent* Owner)
        : TframeAssessmentGeneric(Owner), skin_filter(), qualities_filter(), features_filter(), skin_request(-1),
          editing_skin_items(), final_ending_state()
{
}
//----------------------------------------------------------------------------------------------------------------------
// Initialize - Overridden to set parent properties and setup skin assessment filtered listview.
bool TframeAssessmentSkin::Initialize()
{
  // Set AssessmentGeneric properties
  SetBandCaption("Skin");
  SetPermission("skin_assessment_items");

  // Setup notes filter
  skin_filter.SetListViewHandle(listSkin);
  skin_filter.SetSecurity("skin_assessment_items");

  skin_filter.AddColumnSpec("Item",        "skin_assessment_items", as_text, true,  160, rawString);
  skin_filter.AddColumnSpec("Category",    "skin_assessment_items", as_text, true,   70, rawString);
  skin_filter.AddColumnSpec("Details",     "skin_assessment_items", as_text, true,  350, rawString);
  skin_filter.AddColumnSpec("Current Id",  "current_id",            as_int,  false,  80);
  skin_filter.AddColumnSpec("Internal Id", "internal_id",           as_int,  false,  80);

  skin_filter.FetchPreferences("assessment.skin.topfilter");
  skin_filter.SetSQLStatement("");
  skin_filter.RefreshDataSet();

  // setup skin_assessment_types dual input filters

  string qualities_sql, features_sql;

  qualities_sql = "select item, id from skin_assessment_types where valid = 1 and skin_value_id = '"
                  + lookupTables->IdOf("skin_value_types", "Quality") + "' ";

  features_sql  = "select item, id from skin_assessment_types where valid = 1 and skin_value_id = '"
                  + lookupTables->IdOf("skin_value_types", "Feature") + "' ";

  // Qualities Filter
  qualities_filter.SetListViewHandle(listQualities);
  qualities_filter.SetSecurity("");
  qualities_filter.AddColumnSpec("Item",        "skin_assessment_items", as_text, true,  128, rawString);
  qualities_filter.AddColumnSpec("Internal Id", "internal_id",           as_int,  false,  80);
  qualities_filter.FetchPreferences("assessment.skin.qualities");
  qualities_filter.SetIconStatus(false);
  qualities_filter.SetCheck(true);
  qualities_filter.SetSQLStatement(qualities_sql);
  qualities_filter.RefreshDataSet();

  // Features Filter
  features_filter.SetListViewHandle(listFeatures);
  features_filter.SetSecurity("");
  features_filter.AddColumnSpec("Item",        "skin_assessment_items", as_text, true,  128, rawString);
  features_filter.AddColumnSpec("Internal Id", "internal_id",           as_int,  false,  80);
  features_filter.FetchPreferences("assessment.skin.features");
  features_filter.SetIconStatus(false);
  features_filter.SetCheck(true);
  features_filter.SetSQLStatement(features_sql);
  features_filter.RefreshDataSet();

  // Set edit box change handlers
  SetAllEventHandlers(panelSkin);

  return TframeAssessmentGeneric::Initialize();
}
//----------------------------------------------------------------------------------------------------------------------
// Lock - Overridden to lock skin panel
void TframeAssessmentSkin::Lock()
{
  TframeAssessmentGeneric::Lock(); // lock parent controls
  panelSkin->Enabled = false;      // lock all controls
}
//----------------------------------------------------------------------------------------------------------------------
// PutUpSkinItems - Refreshes the controls to reflect the state of the skin assessment
void TframeAssessmentSkin::PutUpSkinItems(pvecmap &skin_items)
{
  // Clear out the qualities & features checkboxes, and the NPI item checkbox
  qualities_filter.UncheckAll();
  features_filter.UncheckAll();

  bool have_NPI = false;
  psetstr quality_ids;
  psetstr feature_ids;

  editOtherQuality->Text = "";
  editOtherFeature->Text = "";
  editDressing->Text     = "";

  // Analyze the retrieved skin assessment items
  for(size_t i = 0; i < skin_items.size(); i++)
  {
    // Skip items which are anything other than Current. This allows Deleted items to be updated back to Current status
    // when saving.
    if(skin_items[i]["current_id"] != lookupTables->IdOf("current_types", "Current"))
      continue;
       
    if(skin_items[i]["skin_value_id"] == lookupTables->IdOf("skin_value_types", "Quality"))
      quality_ids.insert(skin_items[i]["skin_assessment_id"]);
    else if(skin_items[i]["skin_value_id"] == lookupTables->IdOf("skin_value_types", "Feature"))
      feature_ids.insert(skin_items[i]["skin_assessment_id"]);
    else if(skin_items[i]["skin_value_id"] == lookupTables->IdOf("skin_value_types", "NPI"))
      have_NPI = true;

    // Put up comments on certain items
    if(skin_items[i]["requires_comment"] == "1")
    {
      if(skin_items[i]["skin_assessment_id"] == lookupTables->IdOf("skin_assessment_types", "Other Quality"))
        editOtherQuality->Text = skin_items[i]["comment"].c_str();
      if(skin_items[i]["skin_assessment_id"] == lookupTables->IdOf("skin_assessment_types", "Other Feature"))
        editOtherFeature->Text = skin_items[i]["comment"].c_str();
      if(skin_items[i]["skin_assessment_id"] == lookupTables->IdOf("skin_assessment_types", "Dressing"))
        editDressing->Text = skin_items[i]["comment"].c_str();
    }
  }

  // Set checkboxes
  qualities_filter.SetCheckedWhereID(quality_ids, true);
  features_filter.SetCheckedWhereID(feature_ids, true);
  checkNPI->Checked = have_NPI;
}
//----------------------------------------------------------------------------------------------------------------------
// TaskIdChanging - Overridden to populate skin filter
void TframeAssessmentSkin::TaskIdChanging(const string &new_id)
{
  string sql;

  TframeAssessmentGeneric::TaskIdChanging(new_id);

  sql = "select SAT.item, SVT.item, SAI.comment, SAI.current_id, SAI.id "
        "from skin_assessment_items SAI "
        "left join skin_assessment_types SAT on SAI.skin_assessment_id = SAT.id "
        "left join skin_value_types SVT on SAT.skin_value_id = SVT.id "
        "where SAI.current_id >= 2 and SAI.task_id = '" + new_id + "' ";

  skin_filter.SetSQLStatement(sql);
  skin_filter.RefreshDataSet();
  skin_filter.SetAddEdit(this, AddRecord (&TframeAssessmentSkin::FilterAddSkin),
                               EditRecord(&TframeAssessmentSkin::FilterEditSkin));
  skin_request = parent_form->RequestNotificationWhen("skin_assessment_items.task_id", new_id, rel_e, as_int);
}
//----------------------------------------------------------------------------------------------------------------------
// UpdateForEdit - Overridden to put fresh data into the editing_skin_items vecmap and on screen.
void TframeAssessmentSkin::UpdateForEdit()
{
  string sql =
    "select SAI.skin_assessment_id, SAT.skin_value_id, SAI.comment, SAI.current_id, SAI.id, SAT.requires_comment "
    "from skin_assessment_items SAI "
    "left join skin_assessment_types SAT on SAI.skin_assessment_id = SAT.id "
    "where SAI.task_id = '" + task_id + "' ";

  editing_skin_items.clear();

  if(!SqlToVecMap(generic_transaction, sql, editing_skin_items))
    parent_form->PSay("Could not retrieve up-to-date information for skin assessment items", m_warning, MB_OK);

  // Refresh controls
  PutUpSkinItems(editing_skin_items);

  TframeAssessmentGeneric::UpdateForEdit();
}
//----------------------------------------------------------------------------------------------------------------------
// LockRecords - Overridden to lock the skin_assessment_items records
bool TframeAssessmentSkin::LockRecords()
{
  // Must acquire all items. Any failure is a failure to unlock for editing, GTFO and rollback.
  //   jhaley 20110629 - abstracted logic into parent class
  if(!LockVecMapRecords(editing_skin_items, "skin_assessment_items"))
    return false;

  return TframeAssessmentGeneric::LockRecords();
}
//----------------------------------------------------------------------------------------------------------------------
// IsValidOnScreen - Overridden to do validation for skin assessment items
bool TframeAssessmentSkin::IsValidOnScreen()
{
  pvecstr checked_q_ids = qualities_filter.GetMultiID();
  pvecstr checked_f_ids = features_filter.GetMultiID();

  // Make sure at least one skin assessment item is specified
  if(editing_skin_items.size() == 0 && // jhaley 20110629: Only when the record is new, otherwise can't delete last one
     !checkNPI->Checked && checked_q_ids.size() == 0 && checked_f_ids.size() == 0)
  {
    parent_form->PSay("You have not specified any saveable data. Please check one or more assessment items.",
                      m_missing_data, MB_OK);
    return false;
  }

  // If NPI is specified, most options should be disallowed
  if(checkNPI->Checked)
  {
    if(checked_q_ids.size() != 0)
    {
      parent_form->PSay("You indicated that the patient's skin had no notable problems by checking \"Warm, Dry, Turgor "
                        "Good\", but also indicated problems in the Qualities assessment category. Please correct this "
                        "inconsistency before saving.", m_invalid_data, MB_OK);
      return false;
    }
    // DONE: should features specification be affected at all? Could have good condition skin but still have notable
    // features to record.
    // Request for clarification submitted to Roxann 6/10/2011
    // - Roxann sez it is fine as it is :)
  }

  psetstr ids_needing_input;
  string assmt_types_sql = "select id from skin_assessment_types where requires_comment = 1";
  SqlToSet(dbDatabase, assmt_types_sql, ids_needing_input);

  for(int i = 0; i < checked_q_ids.size(); i++)
  {
    if(ids_needing_input.find(checked_q_ids[i]) != ids_needing_input.end())
    {
      // Still have to hard code the cases >_>
      if(checked_q_ids[i] == lookupTables->IdOf("skin_assessment_types", "Other Quality"))
      {
        if(editOtherQuality->Text == "")
        {
          parent_form->PSay("You selected 'Other Quality' but did not provide any information on that quality.",
                            m_missing_data, MB_OK);
          return false;
        }
      }
    }
  }
  for(int i = 0; i < checked_f_ids.size(); i++)
  {
    if(ids_needing_input.find(checked_f_ids[i]) != ids_needing_input.end())
    {
      // Still have to hard code the cases >_>
      if(checked_f_ids[i] == lookupTables->IdOf("skin_assessment_types", "Other Feature"))
      {
        if(editOtherFeature->Text == "")
        {
          parent_form->PSay("You selected 'Other Feature' but did not provide any information on that feature.",
                            m_missing_data, MB_OK);
          return false;
        }
      }
      if(checked_f_ids[i] == lookupTables->IdOf("skin_assessment_types", "Dressing"))
      {
        if(editDressing->Text == "")
        {
          parent_form->PSay("You selected 'Dressing' but did not provide a description of the dressing.",
                            m_missing_data, MB_OK);
          return false;
        }
      }
    }
  }

  return TframeAssessmentGeneric::IsValidOnScreen();
}
//----------------------------------------------------------------------------------------------------------------------
bool TframeAssessmentSkin::UpdateSkinItem(int record_index, string comment)
{
  pmapstrs ending_state;
  pmapstrtoint leave_alone;

  ending_state["id"]                 = editing_skin_items[record_index]["id"];
  ending_state["current_id"]         = lookupTables->IdOf("current_types", "Current");
  ending_state["last_updated_by"]    = current_userid;
  ending_state["last_update_stamp"]  = RightNow();
  ending_state["task_id"]            = task_id;
  ending_state["skin_assessment_id"] = editing_skin_items[record_index]["skin_assessment_id"];
  ending_state["comment"]            = comment;

  leave_alone["comment"] = sql_no_uppercasing;
  final_ending_state.push_back(ending_state);

  return ExecuteUpdateStatement(generic_transaction, "skin_assessment_items", ending_state, leave_alone);
}
//----------------------------------------------------------------------------------------------------------------------
bool TframeAssessmentSkin::InsertSkinItem(string skin_assessment_id, string comment)
{
  pmapstrs ending_state;
  pmapstrtoint leave_alone;

  ending_state["id"]                 = GetNextId(generic_transaction, "skin_assessment_items");
  ending_state["current_id"]         = lookupTables->IdOf("current_types", "Current");
  ending_state["last_updated_by"]    = current_userid;
  ending_state["last_update_stamp"]  = RightNow();
  ending_state["created_by"]         = current_userid;
  ending_state["creation_stamp"]     = ending_state["last_update_stamp"];
  ending_state["task_id"]            = task_id;
  ending_state["skin_assessment_id"] = skin_assessment_id;
  ending_state["comment"]            = comment;

  leave_alone["comment"] = sql_no_uppercasing;
  final_ending_state.push_back(ending_state);

  return ExecuteInsertStatement(generic_transaction, "skin_assessment_items", ending_state, leave_alone);
}
//----------------------------------------------------------------------------------------------------------------------
bool TframeAssessmentSkin::DeleteSkinItem(int record_index)
{
  pmapstrs ending_state;
  pmapstrtoint leave_alone;

  ending_state["id"]                = editing_skin_items[record_index]["id"];
  ending_state["current_id"]        = lookupTables->IdOf("current_types", "Deleted"); // Mark as Deleted
  ending_state["last_updated_by"]   = current_userid;
  ending_state["last_update_stamp"] = RightNow();

  final_ending_state.push_back(ending_state);

  return ExecuteUpdateStatement(generic_transaction, "skin_assessment_items", ending_state, leave_alone);
}
//----------------------------------------------------------------------------------------------------------------------
// DoSaveData - Overridden to provide core saving logic.
bool TframeAssessmentSkin::DoSaveData()
{
  psetstr checked_ids;
  psetstr unchecked_ids;
  int record_idx;

  final_ending_state.clear();

  // First determine what is checked and what is not in the filters.
  //   jhaley 20110629 - abstracted shared logic into parent class routine
  GetIDsFromCheckBoxFilter(qualities_filter, checked_ids, unchecked_ids);
  GetIDsFromCheckBoxFilter(features_filter,  checked_ids, unchecked_ids);

  // Throw in the NPI ID
  if(checkNPI->Checked)
    checked_ids.insert(lookupTables->IdOf("skin_assessment_types", "Warm, Dry, Turgor Good"));
  else
    unchecked_ids.insert(lookupTables->IdOf("skin_assessment_types", "Warm, Dry, Turgor Good"));

  // Look at all IDs:

  // * If the ID is checked, and the ID is in the editing map, do an update on that record.
  // * If the ID is checked, and the ID is NOT in the editing map, do an insert.
  for(psetstr::iterator i_checked = checked_ids.begin(); i_checked != checked_ids.end(); ++i_checked)
  {
    string comment;

    if(*i_checked == lookupTables->IdOf("skin_assessment_types", "Other Quality"))
      comment = editOtherQuality->Text.c_str();
    else if(*i_checked == lookupTables->IdOf("skin_assessment_types", "Other Feature"))
      comment = editOtherFeature->Text.c_str();
    else if(*i_checked == lookupTables->IdOf("skin_assessment_types", "Dressing"))
      comment = editDressing->Text.c_str();

    // jhaley 20110629 - abstracted logic into parent class
    record_idx = IsInVecMap(editing_skin_items, "skin_assessment_id", *i_checked);
    if(record_idx >= 0)
    {
      // Do Update because this skin assessment item already exists.
      if(!UpdateSkinItem(record_idx, comment))
        return false;
    }
    else
    {
      // Do Insert because this skin assessment item doesn't exist.
      if(!InsertSkinItem(*i_checked, comment))
        return false;
    }
  }

  // * If the ID is NOT checked, and the id is in the editing map, mark that item as Deleted via the current_id.
  // * If the ID is NOT checked, and the id is NOT in the editing map, do nothing at all.
  for(psetstr::iterator i_unchecked = unchecked_ids.begin(); i_unchecked != unchecked_ids.end(); ++i_unchecked)
  {
    record_idx = IsInVecMap(editing_skin_items, "skin_assessment_id", *i_unchecked);
    if(record_idx >= 0)
    {
      // Do Update and set the existing skin assessment item to 'Deleted'.
      if(!DeleteSkinItem(record_idx))
        return false;
    }
    // Otherwise, we neither had a record to begin with, nor do we have one to insert now, so do nothing.
  }

  // If we get this far, nothing borked up above.
  return true;
}
//----------------------------------------------------------------------------------------------------------------------
// SetViewMode - Overridden to activate viewing of skin assessment item information
void TframeAssessmentSkin::SetViewMode()
{
  TframeAssessmentGeneric::SetViewMode();

  panelSkin->Enabled = false;                    // It is still not possible to enter data.
  editing_skin_items.clear();                    // Clear out the items vecmap for safety
  qualities_filter.UncheckAll();
  features_filter.UncheckAll();
  checkNPI->Checked      = false;
  editOtherQuality->Text = "";
  editOtherFeature->Text = "";
  editDressing->Text     = "";
}
//----------------------------------------------------------------------------------------------------------------------
// SetEditMode - Overridden to activate editing of skin assessment items
void TframeAssessmentSkin::SetEditMode()
{
  TframeAssessmentGeneric::SetEditMode();

  panelSkin->Enabled = true;               // Enable panel controls
}
//----------------------------------------------------------------------------------------------------------------------
// FilterAddSkin - "Add" handler for the skin items filter. This actually just activates editing mode here.
void TframeAssessmentSkin::FilterAddSkin()
{
  if(btnUnlockSave->Enabled && frame_mode == mode_viewing)
    btnUnlockSaveClick(NULL);
}
//----------------------------------------------------------------------------------------------------------------------
// FilterEditSkin - Same as above.
void TframeAssessmentSkin::FilterEditSkin()
{
  if(btnUnlockSave->Enabled && frame_mode == mode_viewing)
    btnUnlockSaveClick(NULL);
}
//----------------------------------------------------------------------------------------------------------------------
// AtlasNotify - Overridden to refresh skin items listview
bool TframeAssessmentSkin::AtlasNotify(int request_num, string matched_value)
{
  bool handled = false;

  if(request_num == skin_request)
  {
    skin_filter.RefreshDataSet();
    handled = true;
  }

  return handled;
}
//----------------------------------------------------------------------------------------------------------------------
// SendAtlas - Overridden to send out notifications for skin_assessment_items
void TframeAssessmentSkin::SendAtlas()
{
  atlas->Notify("skin_assessment_items.task_id", task_id); // Is this good enough?
}
//----------------------------------------------------------------------------------------------------------------------
// WriteReport - Write out a report on the skin conditions
bool TframeAssessmentSkin::WriteReport(TComponent *compositeReportCtrl)
{
  bool success = false;

  try
  {
    string sql =
      "select SAT.item value_item, SVT.item value_type, SAI.comment "
      "from skin_assessment_items SAI "
      "left join skin_assessment_types SAT on SAI.skin_assessment_id = SAT.id "
      "left join skin_value_types SVT on SAT.skin_value_id = SVT.id "
      "where SAI.current_id >= 3 and SAI.task_id = '" + task_id + "' ";

    InitDataSet(sql);

    TformReportAssessment *reportForm = report_data.reportForm;
    TQuickRep *thisReport = reportForm->reportSkin;

    thisReport->DataSet = report_data.dbDataSet;
    SetupQRDBText(reportForm->txtSkinQual,    "value_item");
    SetupQRDBText(reportForm->txtSkinValType, "value_type");
    SetupQRDBText(reportForm->txtSkinDesc,    "comment");

    // NB: Need to pass it as TComponent to avoid problems with having QuickReport included in the headers
    TQRCompositeReport *compositeReport = dynamic_cast<TQRCompositeReport *>(compositeReportCtrl);
    if(compositeReport)
    {
      compositeReport->Reports->Add(thisReport);
      success = true;
    }
  }
  catch(...)
  {
  }

  return success;
}
//----------------------------------------------------------------------------------------------------------------------
// checkNPIClick - Marks the frame's data as modified if the NPI checkbox is clicked during editing mode
void __fastcall TframeAssessmentSkin::checkNPIClick(TObject *Sender)
{
  if(frame_mode == mode_editing)
    SetModified(true);
}
//----------------------------------------------------------------------------------------------------------------------
void __fastcall TframeAssessmentSkin::listQualitiesClick(TObject *Sender)
{
  // jhaley 20110629 - abstracted shared checkboxes logic into AssessmentGeneric class.
  FilterCheckBoxHitTest(listQualities);
}
//----------------------------------------------------------------------------------------------------------------------
void __fastcall TframeAssessmentSkin::listFeaturesClick(TObject *Sender)
{
  // jhaley 20110629 - abstracted shared checkboxes logic into AssessmentGeneric class.
  FilterCheckBoxHitTest(listFeatures);
}
//----------------------------------------------------------------------------------------------------------------------

