//----------------------------------------------------------------------------------------------------------------------

#include <vcl.h>
#pragma hdrstop

#include "codeFrameAssessmentVitals.h"
#include "codeRecordPatientAssessment.h"
//----------------------------------------------------------------------------------------------------------------------
#pragma package(smart_init)
#pragma link "codeFrameAssessmentGeneric"
#pragma resource "*.dfm"
TframeAssessmentVitals *frameAssessmentVitals;
//----------------------------------------------------------------------------------------------------------------------
__fastcall TframeAssessmentVitals::TframeAssessmentVitals(TComponent* Owner)
        : TframeAssessmentGeneric(Owner), historical_filter(), obs_set_map(), editing_id(), editing_obs_set(),
          ending_state(), all_questions(), all_answers(), question_id_to_rownum(), rownum_to_question_id(),
          rownum_to_all_questions(), remembering_text()
{
}
//----------------------------------------------------------------------------------------------------------------------
// Initialize - Overridden to set captions, permissions, populate the map of all_questions data, and setup the
// historical data filter.
bool TframeAssessmentVitals::Initialize()
{
  SetBandCaption("Vitals");
  SetPermission("obs_response_sets");

  // Get the full set of vitals observation questions
  string questions_sql =
    "select Q.id, Q.item question_text, Q.short_item, units.item units_text, "
    "obs_question_types.item question_type, Q.obs_option_set_id, Q.is_calculated, Q.improvement_id, Q.cached_mean, "
    "Q.cached_stddev, Q.abs_min, Q.abs_max "
    "from obs_questions_in_sets SETS "
    "inner join obs_questions Q on SETS.obs_question_id = Q.id "
    "left join units on Q.unit_id = units.id "
    "left join obs_question_types on Q.obs_question_id = obs_question_types.id "
    "where SETS.obs_question_set_id = 5 " // FIXME: Magic
    "order by SETS.sequence ";

  if(!SqlToVecMap(dbDatabase, questions_sql, all_questions))
    parent_form->PSay("Could not retrieve 'Vitals' observation question set", m_warning, MB_OK);

  // Set up historical data filter
  historical_filter.SetListViewHandle(listVitalsDisplay);
  historical_filter.SetSecurity("obs_response_sets");
  historical_filter.AddColumnSpec("Observation", "obs_response_sets", as_text, true,  120, rawString);
  historical_filter.AddColumnSpec("Data",        "obs_response_sets", as_text, true,   80, rawString);
  historical_filter.AddColumnSpec("Units",       "obs_response_sets", as_text, true,   70, rawString);
  historical_filter.AddColumnSpec("Date",        "obs_response_sets", as_date, true,   70);
  historical_filter.AddColumnSpec("Current Id",  "current_id",        as_int,  false,  60);
  historical_filter.FetchPreferences("assessment.vitals.historical_filter");
  historical_filter.SetSQLStatement("");
  historical_filter.RefreshDataSet();

  return TframeAssessmentGeneric::Initialize();
}
//----------------------------------------------------------------------------------------------------------------------
// FirstVisible - Overridden to populate the string grid with its initial column/row setup.
void TframeAssessmentVitals::FirstVisible()
{
  // Set up string grid columns
  gridVitalsEntry->ColWidths[0] = 130;
  gridVitalsEntry->ColWidths[1] = 80;
  gridVitalsEntry->ColWidths[2] = 60;

  gridVitalsEntry->Cells[0][0] = "Observation";
  gridVitalsEntry->Cells[1][0] = "Data";
  gridVitalsEntry->Cells[2][0] = "Units";

  // Set up string grid rows
  gridVitalsEntry->RowCount  = (int)(all_questions.size() + 1);
  gridVitalsEntry->FixedRows = 1;
  gridVitalsEntry->FixedCols = 1;

  question_id_to_rownum.clear();
  rownum_to_question_id.clear();
  rownum_to_all_questions.clear();

  for(size_t i = 0; i < all_questions.size(); i++)
  {
    string question_field;

    // Some observation questions need to use the short_item as their description
    if(all_questions[i]["id"] == lookupTables->IdOf("obs_questions", "Blood Pressure (w/slash)") ||
       all_questions[i]["id"] == lookupTables->IdOf("obs_questions", "Frontal-Occipital Circumference"))
      question_field = "short_item";
    else
      question_field = "question_text";

    gridVitalsEntry->Cells[0][i+1] = all_questions[i][question_field].c_str();
    gridVitalsEntry->Cells[1][i+1] = "";
    gridVitalsEntry->Cells[2][i+1] = all_questions[i]["units_text"].c_str();

    // Build efficient, ease-of-access maps between question IDs and string grid rows
    question_id_to_rownum[all_questions[i]["id"]] = i+1;
    rownum_to_question_id[i+1] = all_questions[i]["id"];
    rownum_to_all_questions[i+1] = i;                    // convenient, though not strictly necessary (could use i-1)
  }

  PutUpCurrentVitals(obs_set_map["id"], NULL);

  TframeAssessmentGeneric::FirstVisible();
}
//----------------------------------------------------------------------------------------------------------------------
// Lock - Overridden to disable all editing controls
void TframeAssessmentVitals::Lock()
{
  TframeAssessmentGeneric::Lock();
  gridVitalsEntry->Enabled = false;
  dateObservation->Enabled = false;
}
//----------------------------------------------------------------------------------------------------------------------
// ClearVitalsDisplay - clear out the second column of the string grid
void TframeAssessmentVitals::ClearVitalsDisplay()
{
  for(int i = 1; i < gridVitalsEntry->RowCount; i++)
    gridVitalsEntry->Cells[1][i] = "";
}
//----------------------------------------------------------------------------------------------------------------------
// PutUpCurrentVitals - populate the string grid with any existing vitals observation data
void TframeAssessmentVitals::PutUpCurrentVitals(string obs_set_id, TIBTransaction *dbTransaction)
{
  ClearVitalsDisplay(); // always clear it to keep everything consistent.

  if(!IsBlankOrZero(obs_set_id))
  {
    string sql =
      "select Q.id, "        // FIXME: magic
      "case when Q.obs_question_id = 1 then OPT.item "
      "     when Q.obs_question_id = 2 then A.response_text "
      "     when Q.obs_question_id = 3 then OPT.item "
      "     when Q.obs_question_id = 4 then cast(A.response_metric as varchar(250)) "
      "     when Q.obs_question_id = 5 then cast(A.response_date   as varchar(250)) "
      "     else '' end answer_text "
      "from obs_questions Q "
      "inner join obs_responses A on A.id = "
      " (select first 1 IA.id "
      "  from obs_responses IA "
      "  where IA.current_id >= 4 and IA.obs_question_id = Q.id and IA.obs_response_set_id = '" + obs_set_id + "') "
      "inner join obs_option_types OPT on A.obs_option_id = OPT.id "
      "where exists "                                                                               // FIXME: magic
      "(select * from obs_questions_in_sets QSET where QSET.obs_question_id = Q.id and QSET.obs_question_set_id = 5) ";

    pvecmap cur_answers;
    bool result;

    if(dbTransaction) // On a specific transaction (ie. generic_transaction from UpdateForEdit)
      result = SqlToVecMap(dbTransaction, sql, cur_answers);
    else              // Use a transient transaction as usual
      result = SqlToVecMap(dbDatabase, sql, cur_answers);

    if(!result)
    {
      parent_form->PSay("Could not get existing Vitals values for this Patient Assessment", m_warning, MB_OK);
      return;
    }

    for(pvecmap::iterator i_answer = cur_answers.begin(); i_answer != cur_answers.end(); ++i_answer)
    {
      int rownum = question_id_to_rownum[(*i_answer)["id"]]; // find the string grid row for this observation question

      if(rownum > 0)
      {
        string answer_text = (*i_answer)["answer_text"];
        if(answer_text.find_first_of('.') != string::npos)
          answer_text = FloatToString(StringToFloat(answer_text), 2);
        gridVitalsEntry->Cells[1][rownum] = answer_text.c_str();
      }
    }
  }
}
//----------------------------------------------------------------------------------------------------------------------
// RefreshObsSetMap - jhaley 20111115: Separated out from TaskIdChanging to fix a major glitch
void TframeAssessmentVitals::RefreshObsSetMap(const string &task_id)
{
  // Not ready yet?
  if(IsBlankOrZero(task_id) || IsBlankOrZero(parent_form->GetAssessmentID()))
    return;

  // Get existing observation set data if such exists for this assessment.
  // Each assessment is only allocated a single observation set for vitals. If additional are ever needed they must
  // be created separately for that task/visit as usual. So, the one we are looking for is one which is current,
  // for the same visit, and for this particular assessment.

  obs_set_map.clear();
  string sql =
    string("select * from obs_response_sets where current_id = ") + lookupTables->IdOf("current_types", "Current") +
    " and task_id = '" + task_id + "' and patient_assessment_id = '" + parent_form->GetAssessmentID() + "' ";

  if(!SqlToMap(dbDatabase, sql, obs_set_map))
    parent_form->PSay("Could not get existing observation set data", m_warning, MB_OK);

  // Put up date
  Pdate obs_date = Pdate(obs_set_map["date_of_observation"], mdy | yyyy);
  if(obs_date.IsValidDate())
    dateObservation->Date = obs_date.OutDate();
  else
    dateObservation->Date = Date();
}
//----------------------------------------------------------------------------------------------------------------------
// TaskIdChanging - populate the obs_set_map and get historical observation data once a valid task ID is possessed.
void TframeAssessmentVitals::TaskIdChanging(const string &new_id)
{
  TframeAssessmentGeneric::TaskIdChanging(new_id);

  RefreshObsSetMap(new_id);

  // Get historical data
  string sql =
    "select Q.item, "
    "case when Q.obs_question_id = 1 then OPT.item "
    "     when Q.obs_question_id = 2 then A.response_text "
    "     when Q.obs_question_id = 3 then OPT.item "
    "     when Q.obs_question_id = 4 then cast(cast(A.response_metric as decimal(15, 2)) as varchar(250)) "
    "     when Q.obs_question_id = 5 then cast(A.response_date   as varchar(250)) "
    "     else '' end, "
    "units.item, cast(A.last_update_stamp as date), A.current_id "
    "from obs_questions Q "
    "inner join obs_responses A on A.id = "
    " (select first 1 IA.id "
    "  from obs_responses IA "
    "  inner join obs_response_sets ASET on IA.obs_response_set_id = ASET.id "
    "  where IA.current_id >= 4 and ASET.current_id >= 3 and ASET.person_id = '" + person_id + "' and "
    "  IA.obs_question_id = Q.id "
    "  order by IA.last_update_stamp desc) "
    "inner join obs_option_types OPT on A.obs_option_id = OPT.id "
    "inner join units on Q.unit_id = units.id "
    "where exists "                                                                               // FIXME: magic
    "(select * from obs_questions_in_sets QSET where QSET.obs_question_id = Q.id and QSET.obs_question_set_id = 5) "
    ;

  historical_filter.SetSQLStatement(sql);
  historical_filter.RefreshDataSet();
  historical_request = parent_form->RequestNotificationWhen("obs_response_sets.person_id", person_id, rel_e, as_int);
}
//----------------------------------------------------------------------------------------------------------------------
// LockRecords - Overridden to lock the obs_response_sets record.
bool TframeAssessmentVitals::LockRecords()
{
  if(!LockRecordForUpdate(generic_transaction, "obs_response_sets", editing_id))
    return false;

  return TframeAssessmentGeneric::LockRecords();
}
//----------------------------------------------------------------------------------------------------------------------
// CanUnlock - Overridden to remember the ID of the edited record and to handle insertion of new records.
bool TframeAssessmentVitals::CanUnlock(bool &creating_new)
{
  editing_id = obs_set_map["id"];

  if(IsBlankOrZero(editing_id))
  {
    creating_new = true; // set creating_new flag so that the parent class will prepare for an insert
    editing_id = "new";
    editing_obs_set.clear();
    editing_obs_set["id"] = "new";
  }

  return TframeAssessmentGeneric::CanUnlock(creating_new);
}
//----------------------------------------------------------------------------------------------------------------------
// UpdateForEdit - Overridden to populate the editing controls with data retrieved on generic_transaction
void TframeAssessmentVitals::UpdateForEdit()
{
  string sql = "select * from obs_response_sets where id = '" + editing_id + "' ";

  if(!SqlToMap(generic_transaction, sql, editing_obs_set))
    parent_form->PSay("Could not retrieve up-to-date information for obs_response_sets", m_warning, MB_OK);

  // Re-read the vitals question set (if one exists) on the generic transaction
  PutUpCurrentVitals(editing_id, generic_transaction);

  // Put up date
  Pdate obs_date = Pdate(editing_obs_set["date_of_observation"], mdy | yyyy);
  if(obs_date.IsValidDate())
    dateObservation->Date = obs_date.OutDate();
  else
    dateObservation->Date = Date();

  TframeAssessmentGeneric::UpdateForEdit();
}
//----------------------------------------------------------------------------------------------------------------------
// SavingNew - Overridden to return true if the edited record id is "new"
bool TframeAssessmentVitals::SavingNew()
{
  return editing_obs_set["id"] == "new";
}
//----------------------------------------------------------------------------------------------------------------------
// IsValidOnScreen - Performs validation for permissions and for each individual vitals observation question entry
bool TframeAssessmentVitals::IsValidOnScreen()
{
  string perm_sql;
  string vitals_id = lookupTables->IdOf("obs_question_set_types", "Vitals");

  perm_sql = string("select permission_name from obs_question_set_types where id = ") + vitals_id;
  string perm_str = LowercaseString(GetOneField(dbDatabase, perm_sql));

  if(editing_obs_set["id"] == "new")
  {
    if(optionsPermissions[perm_str]["insert"] != "true")
    {
      parent_form->PSay("You are not permitted to create 'Vitals' observation response sets.", m_warning, MB_OK);
      return false;
    }

    if(optionsPermissions[perm_str]["view"] != "true")
    {
      parent_form->PSay("You are not permitted to view responses to 'Vitals' observation sets.\n"
                        "Please contact a programmer.", m_warning, MB_OK);
      return false;
    }
  }
  else if(optionsPermissions[perm_str]["update"] != "true")
  {
    parent_form->PSay("You are not permitted to update responses to 'Vitals' observation sets.", m_warning, MB_OK);
    return false;
  }

  all_answers.clear();

  // Validate data entered in the observation string grid and build the all_answers map
  for(int rownum = 1; rownum < gridVitalsEntry->RowCount; rownum++)
  {
    map<string, string> this_answer;
    string answer_text   = gridVitalsEntry->Cells[1][rownum].c_str();
    int    question_idx  = rownum_to_all_questions[rownum];
    string question_id   = all_questions[question_idx]["id"];
    string question_type = all_questions[question_idx]["question_type"];
    string option_set_id = all_questions[question_idx]["obs_option_set_id"];
    string abs_min       = all_questions[question_idx]["abs_min"];
    string abs_max       = all_questions[question_idx]["abs_max"];
    string cached_mean   = all_questions[question_idx]["cached_mean"];
    string cached_stddev = all_questions[question_idx]["cached_stddev"];

    if(answer_text == "") // No response, nothing to validate.
      continue;

    if(question_type == "CHOICE" || question_type == "ORDINAL")
    {
      string upper_answer = UppercaseString(answer_text);
      string options_sql = "select UPPER(item) from obs_option_types where valid = 1 and obs_option_set_id = '"
                           + option_set_id + "' ";
      psetstr all_options;
      SqlToSet(dbDatabase, options_sql, all_options);
      if(all_options.find(upper_answer) == all_options.end())
      {
        parent_form->PSay(string("The value for '") + gridVitalsEntry->Cells[0][rownum].c_str() + "' is not valid.\n"
                          "Please enter one of (" + SetToCommaString(all_options) + "). ",
                          m_invalid_data, MB_OK);
        return false;
      }

      // set the option id for the response
      this_answer["obs_option_id"] =
        GetOneField(dbDatabase,
          "select id from obs_option_types where valid = 1 and obs_option_set_id = '" + option_set_id + "' "
          "and UPPER(item) = '" + upper_answer + "' ");
    }
    else if(question_type == "TEXT")
    {
      // There is no validation on this type of question.
      this_answer["response_text"] = answer_text;
    }
    else if(question_type == "METRIC")
    {
      // Numeric; answer must be an integer or float value
      if(!IsFloat(answer_text))
      {
        parent_form->PSay(string("The value for '") + gridVitalsEntry->Cells[0][rownum].c_str() + "' must be a number.",
                          m_invalid_data, MB_OK);
        return false;
      }

      float my_val = StringToFloat(answer_text);

      // Lower bound?
      if(abs_min != "")
      {
        if(my_val < StringToFloat(abs_min))
        {
          parent_form->PSay(string("The value for '") + gridVitalsEntry->Cells[0][rownum].c_str() + "' must be greater "
                            "than or equal to " + abs_min + ". ", m_invalid_data, MB_OK);
          return false;
        }
      }

      // Upper bound?
      if(abs_max != "")
      {
        if(my_val > StringToFloat(abs_max))
        {
          parent_form->PSay(string("The value for '") + gridVitalsEntry->Cells[0][rownum].c_str() + "' must be less "
                            "than or equal to " + abs_max + ". ", m_invalid_data, MB_OK);
          return false;
        }
      }

      // Standard deviations data is available?
      if(cached_mean != "" && cached_stddev != "")
      {
        float lower_bound = StringToFloat(cached_mean) - (3 * StringToFloat(cached_stddev));
        float upper_bound = StringToFloat(cached_mean) + (3 * StringToFloat(cached_stddev));

        if(my_val < lower_bound || my_val > upper_bound)
        {
          if(parent_form->PSay(string("The value for '") + gridVitalsEntry->Cells[0][rownum].c_str() + "' falls well "
                               "outside the statistically common range (3 standard deviations from the mean).\n\n"
                               "Are you sure you want to use this value?", m_warning, MB_YESNO) == IDNO)
          {
            return false;
          }
        }
      }

      // Good to go.
      this_answer["response_metric"] = FloatToString(StringToFloat(answer_text), 2);
    }
    else if(question_type == "DATE")
    {
      if(!Pdate(answer_text, mdy|yy).IsValidDate())
      {
        parent_form->PSay(string("The value for '") + gridVitalsEntry->Cells[0][rownum].c_str() + "' must be a valid "
                          "date.", m_invalid_data, MB_OK);
        return false;
      }

      this_answer["response_date"] = Pdate(answer_text, mdy|yy).OutDate(mdy|yyyy);
    }

    this_answer["current_id"]        = lookupTables->IdOf("current_types", "Current");
    this_answer["last_updated_by"]   = current_userid;
    this_answer["obs_question_id"]   = question_id;

    // Added later:
    // this_answers["id"]                  - determined at insertion time
    // this_answers["obs_response_set_id"] - not available until after it has been created
    // this_answers["last_update_stamp"]   - set while inserting for best accuracy

    // Remember all of this for insertion
    all_answers.push_back(this_answer);
  }

  // User has gone wack.
  if(all_answers.empty())
  {
    parent_form->PSay("There are no vitals to save. Click 'Discard Changes' if you want to stop editing.",
                      m_missing_data, MB_OK);
    return false;
  }

  return TframeAssessmentGeneric::IsValidOnScreen();
}
//----------------------------------------------------------------------------------------------------------------------
// DoSaveData - Save the obs_response_sets record and replace its observation question daughter records
bool TframeAssessmentVitals::DoSaveData()
{
  pmapstrtoint leave_alone;
  ending_state = editing_obs_set;

  ending_state["current_id"]          = lookupTables->IdOf("current_types", "Current");
  ending_state["last_updated_by"]     = current_userid;
  ending_state["last_update_stamp"]   = RightNow();
  ending_state["task_id"]             = task_id;   // parent task record
  ending_state["person_id"]           = person_id; // person from parent task
  ending_state["group_id"]            = group_id;  // group from parent task
  ending_state["obs_question_set_id"] = lookupTables->IdOf("obs_question_set_types", "Vitals");
  ending_state["date_of_observation"] = Pdate(dateObservation->Date).OutDate(mdy|yyyy);

  // These fields don't apply to this type of observation response set.
  ending_state["event_id"]             = "0";
  ending_state["participations_id"]    = "0";
  ending_state["is_steps_observation"] = "0";

  // Attach it to the current assessment as well
  ending_state["patient_assessment_id"] = parent_form->GetAssessmentID();

  if(DoInsertOrUpdate(ending_state, leave_alone, "obs_response_sets"))
  {
    // delete existing observations in this observation response set
    if(ExecuteStatement(generic_transaction,
         "delete from obs_responses where obs_response_set_id = " + ending_state["id"]))
    {
      pmapstrtoint fields_to_leave_raw;
      fields_to_leave_raw["response_text"] = sql_no_uppercasing;

      // Insert each answer as its own obs_responses
      for(pvecmap::iterator i_answer = all_answers.begin(); i_answer != all_answers.end(); ++i_answer)
      {
        (*i_answer)["id"]                  = GetNextId(generic_transaction, "obs_responses");
        (*i_answer)["obs_response_set_id"] = ending_state["id"];
        (*i_answer)["last_update_stamp"]   = RightNow();
        if(!ExecuteInsertStatement(generic_transaction, "obs_responses", *i_answer, fields_to_leave_raw))
          return false; // BORTCH! x_X
      }

      // Everything is snazzy.
      return true;
    }
  }

  // Some kind of error happened >_>
  return false;
}
//----------------------------------------------------------------------------------------------------------------------
// SetViewMode - Overridden to lock editing controls and put up data for display (if there is any).
void TframeAssessmentVitals::SetViewMode()
{
  TframeAssessmentGeneric::SetViewMode();
  editing_obs_set.clear();

  string perm_sql =
    string("select permission_name from obs_question_set_types where id = ")
    + lookupTables->IdOf("obs_question_set_types", "Vitals");
  string perm_name = LowercaseString(GetOneField(dbDatabase, perm_sql));

  // Cannot unlock if do not have update permission for this specific type of observation set
  if(optionsPermissions[perm_name]["update"] != "true")
    btnUnlockSave->Enabled = false;

  gridVitalsEntry->Enabled = false;
  dateObservation->Enabled = false;

  // Put up date
  Pdate obs_date = Pdate(obs_set_map["date_of_observation"], mdy | yyyy);
  if(obs_date.IsValidDate())
    dateObservation->Date = obs_date.OutDate();
  else
    dateObservation->Date = Date();

  // Put vitals back up
  RefreshObsSetMap(task_id);
  PutUpCurrentVitals(obs_set_map["id"], NULL);
}
//----------------------------------------------------------------------------------------------------------------------
// SetEditMode - Overridden to enable editing controls
void TframeAssessmentVitals::SetEditMode()
{
  TframeAssessmentGeneric::SetEditMode();

  gridVitalsEntry->Enabled = true;
  dateObservation->Enabled = true;
}
//----------------------------------------------------------------------------------------------------------------------
// AtlasNotify - Overridden to refresh the historical measurements filter
bool TframeAssessmentVitals::AtlasNotify(int request_num, string matched_value)
{
  bool handled = false;

  // Handle refresh of historical data list view
  if(request_num == historical_request)
  {
    historical_filter.RefreshDataSet();
    handled = true;
  }

  return handled;
}
//----------------------------------------------------------------------------------------------------------------------
// SendAtlas - Overridden to do Atlas update for obs_response_sets
void TframeAssessmentVitals::SendAtlas()
{
  bool new_record = (editing_obs_set["id"] == "new");

  SendAtlasIfChanged(new_record, "obs_response_sets", editing_obs_set, ending_state);
}
//----------------------------------------------------------------------------------------------------------------------
// gridVitalsEntryDrawCell - Custom drawing for the TStringGrid control, because I wasn't satisfied with the default
// visuals. Enhancements include bold for column title cells, a different background color for the selected cell, and
// responsiveness to the state of the assessment frame.
void __fastcall TframeAssessmentVitals::gridVitalsEntryDrawCell(TObject *Sender, int ACol, int ARow, TRect &Rect,
                                                                TGridDrawState State)
{
  if(State.Contains(gdFixed))
  {
    gridVitalsEntry->Canvas->Brush->Color = gridVitalsEntry->FixedColor;
    if(ARow == 0)
      gridVitalsEntry->Canvas->Font->Style = TFontStyles() << fsBold;
    else
      gridVitalsEntry->Canvas->Font->Style = TFontStyles();

    gridVitalsEntry->Canvas->Rectangle(Rect);
    gridVitalsEntry->Canvas->Pen->Width = 1;
    gridVitalsEntry->Canvas->Pen->Style = psSolid;

    // You would think a VCL canvas might have a "Draw3DControlBorder" method or something... but nope.

    // Outer border
    // Show Turtle
    gridVitalsEntry->Canvas->Pen->Color = clBtnHighlight;
    gridVitalsEntry->Canvas->MoveTo(Rect.Left, Rect.Top);
    gridVitalsEntry->Canvas->LineTo(Rect.Left, Rect.Bottom); // Left outer edge
    gridVitalsEntry->Canvas->MoveTo(Rect.Left, Rect.Top);
    gridVitalsEntry->Canvas->LineTo(Rect.Right, Rect.Top);   // Upper outer edge
    gridVitalsEntry->Canvas->Pen->Color = cl3DDkShadow;
    gridVitalsEntry->Canvas->MoveTo(Rect.Left, Rect.Bottom);
    gridVitalsEntry->Canvas->LineTo(Rect.Right, Rect.Bottom); // Bottom outer edge
    gridVitalsEntry->Canvas->MoveTo(Rect.Right, Rect.Top);
    gridVitalsEntry->Canvas->LineTo(Rect.Right, Rect.Bottom+1); // Right outer edge

    // Inner border
    gridVitalsEntry->Canvas->Pen->Color = cl3DLight;
    gridVitalsEntry->Canvas->MoveTo(Rect.Left+1, Rect.Top+1);
    gridVitalsEntry->Canvas->LineTo(Rect.Left+1, Rect.Bottom-1); // Left inner edge
    gridVitalsEntry->Canvas->MoveTo(Rect.Left+1, Rect.Top+1);
    gridVitalsEntry->Canvas->LineTo(Rect.Right-1, Rect.Top+1);   // Upper inner edge
    gridVitalsEntry->Canvas->Pen->Color = clBtnShadow;
    gridVitalsEntry->Canvas->MoveTo(Rect.Left+1, Rect.Bottom-1);
    gridVitalsEntry->Canvas->LineTo(Rect.Right-1, Rect.Bottom-1); // Bottom inner edge
    gridVitalsEntry->Canvas->MoveTo(Rect.Right-1, Rect.Top+1);
    gridVitalsEntry->Canvas->LineTo(Rect.Right-1, Rect.Bottom); // Right inner edge
  }
  else if(State.Contains(gdSelected))
  {
    gridVitalsEntry->Canvas->Brush->Color = static_cast<TColor>(RGB(255, 205, 155));
    gridVitalsEntry->Canvas->Font->Style = TFontStyles();
    gridVitalsEntry->Canvas->FillRect(Rect);
    gridVitalsEntry->Canvas->DrawFocusRect(Rect);
  }
  else
  {
    int all_questions_idx = rownum_to_all_questions[ARow];

    if(ACol == 1 && !IsBlankOrZero(all_questions[all_questions_idx]["is_calculated"])) // question is auto-calculated?
      gridVitalsEntry->Canvas->Brush->Color = clInactiveCaption;
    else
      gridVitalsEntry->Canvas->Brush->Color = gridVitalsEntry->Color;
    gridVitalsEntry->Canvas->FillRect(Rect);
  }

  AnsiString text = gridVitalsEntry->Cells[ACol][ARow];

  TRect textRect;
  textRect.Left   = Rect.Left   + 2;
  textRect.Top    = Rect.Top    + 2;
  textRect.Right  = Rect.Right  - 2;
  textRect.Bottom = Rect.Bottom - 2;
  gridVitalsEntry->Canvas->TextRect(textRect, textRect.Left, textRect.Top, text);
}
//----------------------------------------------------------------------------------------------------------------------
// gridVitalsEntrySelectCell - called when the user tries to select a grid cell.
void __fastcall TframeAssessmentVitals::gridVitalsEntrySelectCell(TObject *Sender, int ACol, int ARow, bool &CanSelect)
{
  CanSelect = true;

  if(ACol == 2)
  {
    // Until I figure out how to handle units...
    parent_form->PSay("Sorry, custom units are not implemented yet!", m_reminder, MB_OK);
    CanSelect = false;
  }
}
//----------------------------------------------------------------------------------------------------------------------
// gridVitalsEntryGetEditText - called when the user clicks a cell to start editing. Remember the current value of the
// cell so that we can tell if it has actually changed in the event handler below before marking the frame modified.
void __fastcall TframeAssessmentVitals::gridVitalsEntryGetEditText(
      TObject *Sender, int ACol, int ARow, AnsiString &Value)
{
  remembering_text = Value.c_str();
}
//----------------------------------------------------------------------------------------------------------------------
// gridVitalsEntrySetEditText - called when the user has stopped editing a cell. Here is where we do auto-calculation of
// "is_calculated" vitals, on the fly. Also, the frame will be set as modified if the contents of the cell actually
// changed from their state in the event handler above.
void __fastcall TframeAssessmentVitals::gridVitalsEntrySetEditText(
      TObject *Sender, int ACol, int ARow, const AnsiString Value)
{
  if(ACol == 1)
  {
    int all_questions_idx = rownum_to_all_questions[ARow];
    if(!IsBlankOrZero(all_questions[all_questions_idx]["is_calculated"])) // question is auto-calculated?
    {
      string msg = "\"" + all_questions[all_questions_idx]["short_item"] + "\" is an auto-calculated observation.";

      // Known cases:
      if(rownum_to_question_id[ARow] == lookupTables->IdOf("obs_questions", "Body-Mass Index"))
        msg += "\nPlease enter Height and Weight to set the result for this observation.";
      else if(rownum_to_question_id[ARow] == lookupTables->IdOf("obs_questions", "Blood Pressure (w/slash)"))
        msg += "\nPlease enter Systolic and Diastolic Blood Pressure to set the result for this observation.";

      // They need to make an actual change to the text before we throw a fit about it; otherwise, this message becomes
      // piss-off-level annoying.
      if(Value != remembering_text.c_str())
      {
        parent_form->PSay(msg, m_reminder, MB_OK);
        gridVitalsEntry->Cells[ACol][ARow] = remembering_text.c_str(); // restore previous value
      }
      return;
    }

    // User has edited an observation value; mark as modified if the value is not the same
    if(Value != remembering_text.c_str())
      SetModified(true);

    // Do auto-calculations for certain fields
    string height_id = lookupTables->IdOf("obs_questions", "Height");
    string weight_id = lookupTables->IdOf("obs_questions", "Weight");
    string bmi_id    = lookupTables->IdOf("obs_questions", "Body-Mass Index");

    string systolic_id  = lookupTables->IdOf("obs_questions", "Systolic Blood Pressure");
    string diastolic_id = lookupTables->IdOf("obs_questions", "Diastolic Blood Pressure");
    string slash_id     = lookupTables->IdOf("obs_questions", "Blood Pressure (w/slash)");

    // Editing height or weight? See if can calculate BMI
    if(rownum_to_question_id[ARow] == height_id || rownum_to_question_id[ARow] == weight_id)
    {
      string height_str = gridVitalsEntry->Cells[1][question_id_to_rownum[height_id]].c_str();
      string weight_str = gridVitalsEntry->Cells[1][question_id_to_rownum[weight_id]].c_str();

      if(height_str != "" && weight_str != "" &&       // not blanks;
         (IsInt(height_str) || IsFloat(height_str)) && // height is an integer or float value, and
         (IsInt(weight_str) || IsFloat(weight_str)))   // weight is an integer or float value
      {
        float height_in  = StringToFloat(height_str);
        float weight_lbs = StringToFloat(weight_str);

        // Yeah, I'm hardcoding the conversions. Sue me. I'll hopefully fix it up when I get user units editing working.
        // Frankly what the observation pane is doing with FloatConvertMetric is not any better (hardcoded ID values).
        float height_m   = height_in  * 0.0254f;
        float weight_kg  = weight_lbs * 0.453592f; // Warning to Octopi Men of the future: assumes G at Earth's surface :P

        if(height_m != 0.0f) // Patients having the properties of a black hole do not qualify for BMI calculation.
        {
          float bmi = weight_kg / (height_m * height_m);
          gridVitalsEntry->Cells[1][question_id_to_rownum[bmi_id]] = FloatToString(bmi, 2).c_str();
        }
        else
          gridVitalsEntry->Cells[1][question_id_to_rownum[bmi_id]] = "";
      }
      else
        gridVitalsEntry->Cells[1][question_id_to_rownum[bmi_id]] = "";
    }

    // Editing systolic or diastolic BP? See if can calculate Written BP
    if(rownum_to_question_id[ARow] == systolic_id || rownum_to_question_id[ARow] == diastolic_id)
    {
      string systolic_str  = gridVitalsEntry->Cells[1][question_id_to_rownum[systolic_id]].c_str();
      string diastolic_str = gridVitalsEntry->Cells[1][question_id_to_rownum[diastolic_id]].c_str();

      if(systolic_str != "" && diastolic_str != "" &&       // not blanks;
         (IsInt(systolic_str)  || IsFloat(systolic_str)) && // systolic BP is an int or float, and
         (IsInt(diastolic_str) || IsFloat(diastolic_str)))  // diastolic BP is an int or float
      {
        int systolic_bp  = (int)StringToFloat(systolic_str);
        int diastolic_bp = (int)StringToFloat(diastolic_str);

        gridVitalsEntry->Cells[1][question_id_to_rownum[slash_id]] =
          (IntToString(systolic_bp) + "/" + IntToString(diastolic_bp)).c_str();
      }
      else
        gridVitalsEntry->Cells[1][question_id_to_rownum[slash_id]] = "";
    }
  }
}
//----------------------------------------------------------------------------------------------------------------------
// dateObservationChange - Handles changes to the date control.
void __fastcall TframeAssessmentVitals::dateObservationChange(TObject *Sender)
{
  if(frame_mode == mode_editing)
    SetModified(true);
}
//----------------------------------------------------------------------------------------------------------------------
// WriteReport - Write out a report on the vitals assessment
bool TframeAssessmentVitals::WriteReport(TComponent *compositeReportCtrl)
{
  bool success = false;

  try
  {
    string sql =
      "select Q.item question_text, "        // FIXME: magic
      " case when Q.obs_question_id = 1 then OPT.item "
      "      when Q.obs_question_id = 2 then A.response_text "
      "      when Q.obs_question_id = 3 then OPT.item "
      "      when Q.obs_question_id = 4 then cast(A.response_metric as varchar(250)) "
      "      when Q.obs_question_id = 5 then cast(A.response_date   as varchar(250)) "
      "      else '' end answer_text, "
      " units.item units_text, "
      " cast(A.last_update_stamp as date) answer_date "
      "from obs_questions Q "
      "inner join obs_responses A on A.id = "
      " (select first 1 IA.id "
      "  from obs_responses IA "
      "  where IA.current_id >= 4 and IA.obs_question_id = Q.id and "
      "  IA.obs_response_set_id = "
      "  (select first 1 id from obs_response_sets where current_id = '" + lookupTables->IdOf("current_types", "Current") + "' "
      "   and task_id = '" + task_id + "' and patient_assessment_id = '" + parent_form->GetAssessmentID() + "')) "
      "inner join obs_option_types OPT on A.obs_option_id = OPT.id "
      "inner join units on Q.unit_id = units.id "
      "where exists "                                                                               // FIXME: magic
      "(select * from obs_questions_in_sets QSET where QSET.obs_question_id = Q.id and QSET.obs_question_set_id = 5) ";

    InitDataSet(sql);

    TformReportAssessment *reportForm = report_data.reportForm;
    TQuickRep *thisReport = report_data.reportForm->reportVitals;

    thisReport->DataSet = report_data.dbDataSet;

    SetupQRDBText(reportForm->txtVitalsObservation, "question_text");
    SetupQRDBText(reportForm->txtVitalsData,        "answer_text");
    SetupQRDBText(reportForm->txtVitalsUnits,       "units_text");
    SetupQRDBText(reportForm->txtVitalsDate,        "answer_date");

    // NB: Need to pass it as TComponent to avoid problems with having QuickReport included in the headers
    TQRCompositeReport *compositeReport = dynamic_cast<TQRCompositeReport *>(compositeReportCtrl);
    if(compositeReport)
    {
      compositeReport->Reports->Add(thisReport);
      success = true;
    }
  }
  catch(...)
  {
  }

  return success;
}
//----------------------------------------------------------------------------------------------------------------------

