//----------------------------------------------------------------------------------------------------------------------

#include <vcl.h>
#pragma hdrstop

#include "codeFrameAssessmentWound.h"
#include "codeRecordPatientAssessment.h"
//----------------------------------------------------------------------------------------------------------------------
#pragma package(smart_init)
#pragma link "codeFrameAssessmentGeneric"
#pragma resource "*.dfm"
TframeAssessmentWound *frameAssessmentWound;
//----------------------------------------------------------------------------------------------------------------------
__fastcall TframeAssessmentWound::TframeAssessmentWound(TComponent* Owner)
        : TframeAssessmentGeneric(Owner), wound_filter(), editing_wound(), ending_state(), wound_request(-1),
          editing_id()
{
}
//----------------------------------------------------------------------------------------------------------------------
// Lookup Tables Lookup Table (heh-heh-heh)
static const char *woundLookupTables[][2] =
{
  { "cboExudateAmt",   "exudate_amt_types"  },
  { "cboExudateType",  "exudate_types"      },
  { "cboOdor",         "wound_odor_types"   },
  { "cboMargins",      "wound_margin_types" },
  { "cboPeriTemp",     "skin_temp_types"    },
  { "cboPeriTexture",  "skin_texture_types" },
  { "cboColorPresent", "wound_color_types"  },
  { "cboWoundBase",    "wound_base_types"   },
  { NULL,              NULL                 }
};
//----------------------------------------------------------------------------------------------------------------------
// Initialize - Overridden to set parent properties and setup wounds filtered listview.
bool TframeAssessmentWound::Initialize()
{
  // Set AssessmentGeneric properties
  SetBandCaption("Wound Care");
  SetPermission("wound_care");

  // Setup notes filter
  wound_filter.SetListViewHandle(listWounds);
  wound_filter.SetSecurity("wound_care");

  /* TODO: add more fields? */
  wound_filter.AddColumnSpec("Location",    "wound_care",  as_text, true,  120, sentenceString);
  wound_filter.AddColumnSpec("Length",      "wound_care",  as_int,  true,   80);
  wound_filter.AddColumnSpec("Width",       "wound_care",  as_int,  true,   80);
  wound_filter.AddColumnSpec("Depth",       "wound_care",  as_int,  true,   80);
  wound_filter.AddColumnSpec("Pain",        "wound_care",  as_int,  true,   80);
  wound_filter.AddColumnSpec("Comments",    "wound_care",  as_text, true,  140, rawString);
  wound_filter.AddColumnSpec("Current Id",  "current_id",  as_int,  false,  80);
  wound_filter.AddColumnSpec("Internal Id", "internal_id", as_int,  false,  80);

  wound_filter.FetchPreferences("assessment.wounds.filter");
  wound_filter.SetSQLStatement("");
  wound_filter.RefreshDataSet();

  // Register lookups - tablified above due to there being so many of them on this frame.
  for(int i = 0; ; i++)
  {
    if(woundLookupTables[i][0] == NULL)
      break; // finished.

    string controlName = woundLookupTables[i][0];
    string tableName   = woundLookupTables[i][1];
    string lookupName  = "wounds." + controlName; // build the unique name for parent class map functionality

    // Have to find the control on panelWound
    TControl *theControl = panelWound->FindChildControl(controlName.c_str());
    if(theControl && AnsiString(theControl->ClassName()) == "TComboBox")
    {
      TComboBox *theBox = dynamic_cast<TComboBox *>(theControl);
      RegisterLookup(lookupName, tableName, theBox);
    }
  }

  // Set standard modified event handlers for all controls on panelMed
  SetAllEventHandlers(panelWound);

  return TframeAssessmentGeneric::Initialize();
}
//----------------------------------------------------------------------------------------------------------------------
// Lock - Overridden to lock Add button and wound panel
void TframeAssessmentWound::Lock()
{
  TframeAssessmentGeneric::Lock(); // lock parent controls
  btnAdd->Enabled     = false;     // disable add button
  panelWound->Enabled = false;     // lock all controls
}
//----------------------------------------------------------------------------------------------------------------------
// TaskIdChanging - Overridden to populate wounds filter
void TframeAssessmentWound::TaskIdChanging(const string &new_id)
{
  string sql;

  TframeAssessmentGeneric::TaskIdChanging(new_id);

  /* TODO: add more fields? */
  sql = "select WC.location, WC.wound_length, WC.wound_width, WC.wound_depth, WC.pain_scale, WC.comments, "
        "WC.current_id, WC.id "
        "from wound_care WC "
        "where WC.current_id >= 2 and WC.task_id = '" + new_id + "' ";

  wound_filter.SetSQLStatement(sql);
  wound_filter.RefreshDataSet();
  wound_filter.SetAddEdit(this, AddRecord(&TframeAssessmentWound::FilterAddWound),
                               EditRecord(&TframeAssessmentWound::FilterEditWound));
  wound_request = parent_form->RequestNotificationWhen("wound_care.task_id", new_id, rel_e, as_int);
}
//----------------------------------------------------------------------------------------------------------------------
// LockRecords - Overridden to lock wound_care record for editing.
bool TframeAssessmentWound::LockRecords()
{
  if(!LockRecordForUpdate(generic_transaction, "wound_care", editing_id))
    return false;

  return TframeAssessmentGeneric::LockRecords();
}
//----------------------------------------------------------------------------------------------------------------------
// CanUnlock - Overridden to test for a selected wound_care record
bool TframeAssessmentWound::CanUnlock(bool &creating_new)
{
  editing_id = wound_filter.GetSelectedRecordID();

  if(!creating_new && editing_id == "")
  {
    parent_form->PSay("Please select a Wound Care Encounter to edit in the list first.", m_reminder, MB_OK);
    return false;
  }

  return TframeAssessmentGeneric::CanUnlock(creating_new);
}
//----------------------------------------------------------------------------------------------------------------------
// UpdateForEdit - Overridden to put fresh data into the editing_wound map and on screen.
void TframeAssessmentWound::UpdateForEdit()
{
  string sql = "select * from wound_care where id = '" + editing_id + "' ";

  if(!SqlToMap(generic_transaction, sql, editing_wound))
    parent_form->PSay("Could not retrieve up-to-date information for Wound Care Encounter", m_warning, MB_OK);

  // Integers
  editLength->Text      = editing_wound["wound_length"].c_str();
  editWidth->Text       = editing_wound["wound_width"].c_str();
  editDepth->Text       = editing_wound["wound_depth"].c_str();
  editTunnelPos->Text   = editing_wound["tunneling_pos"].c_str();
  editMaxDistance->Text = editing_wound["max_distance"].c_str();
  editMinDistance->Text = editing_wound["min_distance"].c_str();
  editSinusPos->Text    = editing_wound["sinus_tract_pos"].c_str();
  editPain->Text        = editing_wound["pain_scale"].c_str();
  editSlough->Text      = editing_wound["slough"].c_str();
  editEschar->Text      = editing_wound["eschar"].c_str();

  // Lookups
  string skin_temp    = lookupTables->ValueOf("skin_temp_types",    editing_wound["skin_temp_id"]);
  string skin_texture = lookupTables->ValueOf("skin_texture_types", editing_wound["skin_texture_id"]);
  string wound_color  = lookupTables->ValueOf("wound_color_types",  editing_wound["wound_color_id"]);
  string wound_margin = lookupTables->ValueOf("wound_margin_types", editing_wound["wound_margin_id"]);
  string wound_odor   = lookupTables->ValueOf("wound_odor_types",   editing_wound["wound_odor_id"]);
  string wound_base   = lookupTables->ValueOf("wound_base_types",   editing_wound["wound_base_id"]);
  string exudate_type = lookupTables->ValueOf("exudate_types",      editing_wound["exudate_id"]);
  string exudate_amt  = lookupTables->ValueOf("exudate_amt_types",  editing_wound["exudate_amt_id"]);

  cboExudateAmt->Text   = exudate_amt.c_str();
  cboExudateType->Text  = exudate_type.c_str();
  cboOdor->Text         = wound_odor.c_str();
  cboMargins->Text      = wound_margin.c_str();
  cboPeriTemp->Text     = skin_temp.c_str();
  cboPeriTexture->Text  = skin_texture.c_str();
  cboColorPresent->Text = wound_color.c_str();
  cboWoundBase->Text    = wound_base.c_str();

  // Strings
  editGranulation->Text = SentenceString(editing_wound["granulation"]).c_str();
  editGranColor->Text   = SentenceString(editing_wound["granulation_color"]).c_str();
  editCleansing->Text   = SentenceString(editing_wound["cleansing_agents"]).c_str();
  editDressings->Text   = SentenceString(editing_wound["dressing_supplies"]).c_str();
  editLocation->Text    = SentenceString(editing_wound["location"]).c_str();
  editOtherType->Text   = PrettyString(editing_wound["other_info"]).c_str();
  editOtherMargin->Text = PrettyString(editing_wound["other_margin"]).c_str();
  editComments->Text    = editing_wound["comments"].c_str();

  // Make sure all edits are marked unmodified
  ClearControlsModified(panelWound);

  TframeAssessmentGeneric::UpdateForEdit();
}
//----------------------------------------------------------------------------------------------------------------------
// SavingNew - Overridden to return status of editing_wound as new record or unlocked record
bool TframeAssessmentWound::SavingNew()
{
  return editing_wound["id"] == "new";
}
//----------------------------------------------------------------------------------------------------------------------
// ValidateInteger - Subroutine for IsValidOnScreen
bool TframeAssessmentWound::ValidateInteger(TEdit *editBox, int min_value, int max_value, const char *name)
{
  if(editBox->Text != "") // All are optional.
  {
    string as_string  = editBox->Text.c_str();
    int    as_integer = StringToInt(as_string);

    if(!IsInt(as_string))
    {
      parent_form->PSay(string("Please enter an integer value for ") + name + ".", m_invalid_data, MB_OK);
      return false;
    }

    // Optional range checking
    if(min_value != -1)
    {
      if(as_integer < min_value || as_integer > max_value)
      {
        parent_form->PSay(
          string("Please enter an integer value between ")
          + IntToString(min_value) + " and " + IntToString(max_value) + " for " + name + ".",
          m_invalid_data, MB_OK);
        return false;
      }
    }
  }

  return true;
}
//----------------------------------------------------------------------------------------------------------------------
// ValidateLookup - Subroutine for IsValidOnScreen
bool TframeAssessmentWound::ValidateLookup(TComboBox *cboTheBox, const char *lookupTableName, const char *name)
{
  if(cboTheBox->Text != "") // All are optional.
  {
    if(!lookupTables->IsInLookUpTable(lookupTableName, cboTheBox))
    {
      parent_form->PSay(string("Please enter a valid ") + name + ".", m_invalid_data, MB_OK);
      return false;
    }
  }

  return true;
}
//----------------------------------------------------------------------------------------------------------------------
// IsValidOnScreen - Overridden to validate input
bool TframeAssessmentWound::IsValidOnScreen()
{
  // Integers
  if(!(ValidateInteger(editLength,      -1,  -1, "Length") &&
       ValidateInteger(editWidth,       -1,  -1, "Width")  &&
       ValidateInteger(editDepth,       -1,  -1, "Depth")  &&
       ValidateInteger(editTunnelPos,    1,  12, "Tunneling/Undermining Position") &&
       ValidateInteger(editMaxDistance, -1,  -1, "Max Distance") &&
       ValidateInteger(editMinDistance, -1,  -1, "Min Distance") &&
       ValidateInteger(editSinusPos,     1,  12, "Sinus Tract Position") &&
       ValidateInteger(editPain,         0,  10, "Pain Scale") &&
       ValidateInteger(editSlough,       0, 100, "Slough Percentage") &&
       ValidateInteger(editEschar,       0, 100, "Eschar Percentage")))
  {
    return false;
  }

  if(!(ValidateLookup(cboExudateAmt,   "exudate_amt_types",  "Exudate Amount") &&
       ValidateLookup(cboExudateType,  "exudate_types",      "Exudate Type") &&
       ValidateLookup(cboOdor,         "wound_odor_types",   "Odor") &&
       ValidateLookup(cboMargins,      "wound_margin_types", "Margins") &&
       ValidateLookup(cboPeriTemp,     "skin_temp_types",    "Peripheral Skin Temperature") &&
       ValidateLookup(cboPeriTexture,  "skin_texture_types", "Peripheral Skin Texture") &&
       ValidateLookup(cboColorPresent, "wound_color_types",  "Color Present") &&
       ValidateLookup(cboWoundBase,    "wound_base_types",   "Wound Base")))
  {
    return false;
  }

  // Strings
  // * Enforce consistency between "Other" values and the string fields the "other" information should be
  //   provided inside.
  if(UppercaseString(cboExudateType->Text.c_str()) == "OTHER")
  {
    if(editOtherType->Text == "")
    {
      parent_form->PSay("You have specified \"Other\" for Exudate Type, but did not provide any information on the "
                        "exudate. Please select a different Exudate Type, or provide additional information.",
                        m_missing_data, MB_OK);
      return false;
    }
  }
  else if(editOtherType->Text != "")
  {
    parent_form->PSay("Information entered in the \"Other\" field for Exudate Type will be ignored unless you select "
                      "\"Other\" for Exudate Type first. Please clear the \"Other\" field.",
                      m_invalid_data, MB_OK);
    return false;
  }

  if(UppercaseString(cboMargins->Text.c_str()) == "OTHER")
  {
    if(editOtherMargin->Text == "")
    {
      parent_form->PSay("You have specified \"Other\" for Wound Margins, but did not provide any information on the "
                        "margin. Please select a different Margins value, or provide additional information.",
                        m_missing_data, MB_OK);
      return false;
    }
  }
  else if(editOtherMargin->Text != "")
  {
    parent_form->PSay("Information entered in the \"Other\" field for Wound Margins will be ignored unless you select "
                      "\"Other\" for the Margins value first. Please clear the \"Other\" field.",
                      m_invalid_data, MB_OK);
    return false;
  }

  return TframeAssessmentGeneric::IsValidOnScreen();
}
//----------------------------------------------------------------------------------------------------------------------
// DoSaveData - Overridden to provide core saving logic.
bool TframeAssessmentWound::DoSaveData()
{
  pmapstrtoint leave_alone;
  ending_state = editing_wound;

  ending_state["current_id"]        = lookupTables->IdOf("current_types", "Current");
  ending_state["last_update_stamp"] = RightNow();
  ending_state["last_updated_by"]   = current_userid;

  // if creating new, set created_by/creation_stamp too
  if(editing_wound["id"] == "new")
  {
    ending_state["creation_stamp"] = ending_state["last_update_stamp"];
    ending_state["created_by"]     = ending_state["last_updated_by"];
  }

  // Integer fields
  ending_state["wound_length"]    = editLength->Text.c_str();
  ending_state["wound_width"]     = editWidth->Text.c_str();
  ending_state["wound_depth"]     = editDepth->Text.c_str();
  ending_state["tunneling_pos"]   = editTunnelPos->Text.c_str();
  ending_state["max_distance"]    = editMaxDistance->Text.c_str();
  ending_state["min_distance"]    = editMinDistance->Text.c_str();
  ending_state["sinus_tract_pos"] = editSinusPos->Text.c_str();
  ending_state["pain_scale"]      = editPain->Text.c_str();
  ending_state["slough"]          = editSlough->Text.c_str();
  ending_state["eschar"]          = editEschar->Text.c_str();

  // Lookup fields
  ending_state["skin_temp_id"]    = lookupTables->IdOf("skin_temp_types",    cboPeriTemp);
  ending_state["skin_texture_id"] = lookupTables->IdOf("skin_texture_types", cboPeriTexture);
  ending_state["wound_color_id"]  = lookupTables->IdOf("wound_color_types",  cboColorPresent);
  ending_state["wound_margin_id"] = lookupTables->IdOf("wound_margin_types", cboMargins);
  ending_state["wound_odor_id"]   = lookupTables->IdOf("wound_odor_types",   cboOdor);
  ending_state["wound_base_id"]   = lookupTables->IdOf("wound_base_types",   cboWoundBase);
  ending_state["exudate_id"]      = lookupTables->IdOf("exudate_types",      cboExudateType);
  ending_state["exudate_amt_id"]  = lookupTables->IdOf("exudate_amt_types",  cboExudateAmt);

  // String fields
  ending_state["granulation"]       = editGranulation->Text.c_str();
  ending_state["granulation_color"] = editGranColor->Text.c_str();
  ending_state["cleansing_agents"]  = editCleansing->Text.c_str();
  ending_state["dressing_supplies"] = editDressings->Text.c_str();
  ending_state["location"]          = editLocation->Text.c_str();
  ending_state["other_info"]        = editOtherType->Text.c_str();
  ending_state["other_margin"]      = editOtherMargin->Text.c_str();
  ending_state["comments"]          = editComments->Text.c_str();

  leave_alone["comments"] = sql_no_uppercasing; // Don't uppercase comments text

  // Not seen:
  // task_id - this either comes from the existing wound or is set in editing_wound on add

  return DoInsertOrUpdate(ending_state, leave_alone, "wound_care");
}
//----------------------------------------------------------------------------------------------------------------------
// SetViewMode - Overridden to activate viewing of wound care encounters
void TframeAssessmentWound::SetViewMode()
{
  TframeAssessmentGeneric::SetViewMode();
  btnAdd->Enabled = HaveUnlockPermission(true); // It is now possible to add a wound care encounter
  panelWound->Enabled = false;                  // But it is still not possible to enter data.
  editing_wound.clear();                        // Clear out the wound editing map for safety

  // Reset control contents
  ClearControlsText(panelWound);
  ClearControlsModified(panelWound);
}
//----------------------------------------------------------------------------------------------------------------------
// SetEditMode - Overridden to activate editing of a wound care encounter
void TframeAssessmentWound::SetEditMode()
{
  TframeAssessmentGeneric::SetEditMode();
  btnAdd->Enabled = false;                // Can't add wound care encounters right now
  panelWound->Enabled = true;             // Enable panel controls
}
//----------------------------------------------------------------------------------------------------------------------
// btnAddClick - Handler for toolbar button to add a new wound care encounter
void __fastcall TframeAssessmentWound::btnAddClick(TObject *Sender)
{
  if(DoUnlock(true))
  {
    // Reset control contents
    ClearControlsText(panelWound);
    ClearControlsModified(panelWound);

    editing_wound.clear();
    editing_wound["id"]      = "new";   // Creating a new wound care encounter
    editing_wound["task_id"] = task_id;
  }
}
//----------------------------------------------------------------------------------------------------------------------
// FilterAddMed - "Add" handler for the wounds filter. We redirect this to the "Add a Wound" code so that it
// works compatibly with the assessment frame.
void TframeAssessmentWound::FilterAddWound()
{
  if(btnAdd->Enabled) // Only when you could add using the button.
    btnAddClick(NULL);
}
//----------------------------------------------------------------------------------------------------------------------
// FilterEditMed - Edit the selected med in the assessment panel
void TframeAssessmentWound::FilterEditWound()
{
  if(btnUnlockSave->Enabled && frame_mode == mode_viewing) // Only when you can start editing via the button.
    btnUnlockSaveClick(NULL);
}
//----------------------------------------------------------------------------------------------------------------------
// AtlasNotify - Overridden to refresh wounds listview
bool TframeAssessmentWound::AtlasNotify(int request_num, string matched_value)
{
  bool handled = false;

  if(request_num == wound_request)
  {
    wound_filter.RefreshDataSet();
    handled = true;
  }

  return handled;
}
//----------------------------------------------------------------------------------------------------------------------
// SendAtlas - Overridden to send out notifications for wound_care
void TframeAssessmentWound::SendAtlas()
{
  bool new_record = (editing_wound["id"] == "new");

  SendAtlasIfChanged(new_record, "wound_care", editing_wound, ending_state);
}
//----------------------------------------------------------------------------------------------------------------------
// WriteReport - Write out a report on the wound care assessment
bool TframeAssessmentWound::WriteReport(TComponent *compositeReportCtrl)
{
  bool success = false;

  try
  {
    string sql =
      "select WC.wound_length, WC.wound_width, WC.wound_depth, WC.tunneling_pos, WC.max_distance, WC.min_distance, "
      " WC.pain_scale, WC.sinus_tract_pos, XAT.item exudate_amount, (XT.item || ' ' || WC.other_info) exudate_type, "
      " WOT.item wound_odor, (WMT.item || ' ' || WC.other_margin) wound_margins, STT.item skin_temp, "
      " WCT.item wound_color, WBT.item wound_base, STXT.item skin_texture, WC.slough, WC.eschar, "
      " WC.granulation, WC.granulation_color, WC.cleansing_agents, WC.dressing_supplies, WC.location, WC.comments "
      "from wound_care WC "
      "left join exudate_amt_types  XAT  on WC.exudate_amt_id  = XAT.id "
      "left join exudate_types      XT   on WC.exudate_id      = XT.id "
      "left join wound_odor_types   WOT  on WC.wound_odor_id   = WOT.id "
      "left join wound_margin_types WMT  on WC.wound_margin_id = WMT.id "
      "left join skin_temp_types    STT  on WC.skin_temp_id    = STT.id "
      "left join wound_color_types  WCT  on WC.wound_color_id  = WCT.id "
      "left join wound_base_types   WBT  on WC.wound_base_id   = WBT.id "
      "left join skin_texture_types STXT on WC.skin_texture_id = STXT.id "
      "where WC.current_id >= 3 and WC.task_id = '" + task_id + "' ";

    InitDataSet(sql);

    TformReportAssessment *reportForm = report_data.reportForm;
    TQuickRep *thisReport = report_data.reportForm->reportWound;

    thisReport->DataSet = report_data.dbDataSet;

    txtregister_t ctrlsToRegister[] =
    {
      { reportForm->txtWoundLen,     "wound_length"      },
      { reportForm->txtWoundWidth,   "wound_width"       },
      { reportForm->txtWoundDepth,   "wound_depth"       },
      { reportForm->txtWoundTUP,     "tunneling_pos"     },
      { reportForm->txtWoundMaxD,    "max_distance"      },
      { reportForm->txtWoundMinD,    "min_distance"      },
      { reportForm->txtWoundPain,    "pain_scale"        },
      { reportForm->txtWoundSTP,     "sinus_tract_pos"   },
      { reportForm->txtWoundExAmt,   "exudate_amount"    },
      { reportForm->txtWoundExType,  "exudate_type"      },
      { reportForm->txtWoundOdor,    "wound_odor"        },
      { reportForm->txtWoundMargin,  "wound_margins"     },
      { reportForm->txtWoundPSTmp,   "skin_temp"         },
      { reportForm->txtWoundColors,  "wound_color"       },
      { reportForm->txtWoundSlough,  "slough"            },
      { reportForm->txtWoundPSTex,   "skin_texture"      },
      { reportForm->txtWoundBase,    "wound_base"        },
      { reportForm->txtWoundEschar,  "eschar"            },
      { reportForm->txtWoundGran,    "granulation"       },
      { reportForm->txtWoundGCol,    "granulation_color" },
      { reportForm->txtWoundCAU,     "cleansing_agents"  },
      { reportForm->txtWoundDSU,     "dressing_supplies" },
      { reportForm->txtWoundLoc,     "location"          },
      { reportForm->txtWoundComment, "comments"          },
      { NULL,                        NULL                }
    };
    SetupQRDBTexts(ctrlsToRegister);

    // NB: Need to pass it as TComponent to avoid problems with having QuickReport included in the headers
    TQRCompositeReport *compositeReport = dynamic_cast<TQRCompositeReport *>(compositeReportCtrl);
    if(compositeReport)
    {
      compositeReport->Reports->Add(thisReport);
      success = true;
    }
  }
  catch(...)
  {
  }

  return success;
}
//----------------------------------------------------------------------------------------------------------------------

