//---------------------------------------------------------------------------


#ifndef codeFrameDependencyGraphH
#define codeFrameDependencyGraphH
//---------------------------------------------------------------------------
#include <Classes.hpp>
#include <Controls.hpp>
#include <StdCtrls.hpp>
#include <Forms.hpp>
#include <ExtCtrls.hpp>
#include "util.h"
#include "tensors.h"

enum DGNodeStates {ndHover, ndSelected, ndFocused, ndDragging};
typedef Set<DGNodeStates, ndHover, ndDragging> TDGNodeState;
enum DGEdgeStates {edHover};
typedef Set<DGNodeStates, edHover, edHover> TDGEdgeState;
typedef void __fastcall (__closure *TDGDrawNodeEvent)(System::TObject* Sender, Graphics::TCanvas* Canvas, string Item, Windows::TPoint Point);
typedef void __fastcall (__closure *TDGDrawEdgeEvent)(System::TObject* Sender, Graphics::TCanvas* Canvas, string ItemA, Windows::TPoint PointA, string ItemB, Windows::TPoint PointB);
typedef void __fastcall (__closure *TDGNodeClickEvent) (System::TObject* Sender, string Item);
typedef void __fastcall (__closure *TDGEdgeClickEvent) (System::TObject* Sender, string ItemA, string ItemB);
typedef void __fastcall (__closure *TDGNodeDblClickEvent) (System::TObject* Sender, string Item);
typedef void __fastcall (__closure *TDGEdgeDblClickEvent) (System::TObject* Sender, string ItemA, string ItemB);
// handle context-menu by providing TMenu properties for nodes and edges and 'nothing'? multi-selection of nodes, right-click?
// typedef void __fastcall (__closure *TDGNodeChangingEvent) (System::TObject* Sender, ..., bool & Accept);
// typedef void __fastcall (__closure *TDGEdgeChangingEvent) (System::TObject* Sender, ..., bool & Accept);
typedef void __fastcall (__closure *TDGNodeChangeEvent) (System::TObject* Sender, string Item, TDGNodeState OldState, TDGNodeState NewState);
typedef void __fastcall (__closure *TDGEdgeChangeEvent) (System::TObject* Sender, string ItemA, TDGEdgeState OldStateA, TDGEdgeState NewStateA, string ItemB, TDGEdgeState OldStateB, TDGEdgeState NewStateB);
typedef void __fastcall (__closure *TDGInfoHintEvent) (System::TObject* Sender, string Item, string & Hint);
// typedef void __fastcall (__closure *TDGNodeHitTest) (System::TObject* Sender, Windows::TPoint Point, string Item, bool & Confirm);
// typedef void __fastcall (__closure *TDGEdgeHitTest) (System::TObject* Sender, );

//---------------------------------------------------------------------------
class TframeDependencyGraph : public TFrame
{
__published:	// IDE-managed Components
        TScrollBox *scrollDependencyGraph;
        TPaintBox *paintDependencyGraph;
        TTimer *timerSimulateDependencyGraph;
        void __fastcall paintDependencyGraphMouseDown(TObject *Sender,
          TMouseButton Button, TShiftState Shift, int X, int Y);
        void __fastcall paintDependencyGraphMouseMove(TObject *Sender,
          TShiftState Shift, int X, int Y);
        void __fastcall paintDependencyGraphMouseUp(TObject *Sender,
          TMouseButton Button, TShiftState Shift, int X, int Y);
        void __fastcall paintDependencyGraphPaint(TObject *Sender);
        void __fastcall paintDependencyGraphClick(TObject *Sender);
        void __fastcall paintDependencyGraphContextPopup(TObject *Sender,
          TPoint &MousePos, bool &Handled);
        void __fastcall paintDependencyGraphDblClick(TObject *Sender);
        void __fastcall timerSimulateDependencyGraphTimer(TObject *Sender);
private:	// User declarations
protected:
        TDGDrawNodeEvent FOnDrawNode;
        TDGDrawEdgeEvent FOnDrawEdge;
        TDGNodeClickEvent FOnNodeClick;
        TDGEdgeClickEvent FOnEdgeClick;
        TDGNodeDblClickEvent FOnNodeDblClick;
        TDGEdgeDblClickEvent FOnEdgeDblClick;
        TDGNodeChangeEvent FOnNodeChange;
        TDGEdgeChangeEvent FOnEdgeChange;
        TDGInfoHintEvent FOnInfoHint;

        bool FActive;
        unsigned int FBandWidth;
        unsigned int FPadding;
        TColor FBandColor;
        map<string, set<string> >     items_to_requirements;
        vector<pair<string, string> > items_to_items;
        vector<set<string> >          bands_of_items;
        map<node_base<3> *,           string>  nodes_to_items;
        map<string, node_base<3> *>   items_to_nodes;
        world_base<3>                 world;
        node_base<3> *                node_drag;

        void virtual ResetNodePositions ();
        bool virtual ResetPaintboxExtents ();
        void __fastcall virtual SetActive(bool Value);
        void __fastcall virtual SetBandColor(TColor Value);
        void __fastcall virtual SetBandWidth(unsigned int Value);
        void __fastcall virtual SetPadding(unsigned int Value);
        void __fastcall DefaultDrawNode (System::TObject* Sender, Graphics::TCanvas* Canvas, std::string Item, Windows::TPoint Point);
        void __fastcall DefaultDrawEdge (System::TObject* Sender, Graphics::TCanvas* Canvas, std::string ItemA, Windows::TPoint PointA, std::string ItemB, Windows::TPoint PointB);
public:		// User declarations
        __fastcall TframeDependencyGraph(TComponent* Owner);
        void virtual SetDependencies (const map<string, set<string> > & item_dependencies, const vector<pair<string, string> > & item_relationships);
        TDGNodeState virtual GetNodeState (string item);

        __property bool Active = {read=FActive, write=SetActive, default=false};
        __property Graphics::TColor BandColor = {read=FBandColor, write=SetBandColor, default=clSilver};
        __property unsigned int BandWidth = {read=FBandWidth, write=SetBandWidth, default=30};
        __property unsigned int Padding = {read=FPadding, write=SetPadding, default=10};

        __property TDGDrawNodeEvent OnDrawNode = {read=FOnDrawNode, write=FOnDrawNode, nodefault};
        __property TDGDrawEdgeEvent OnDrawEdge = {read=FOnDrawEdge, write=FOnDrawEdge, nodefault};
        __property TDGNodeClickEvent OnNodeClick = {read=FOnNodeClick, write=FOnNodeClick, nodefault};
        __property TDGEdgeClickEvent OnEdgeClick = {read=FOnEdgeClick, write=FOnEdgeClick, nodefault};
        __property TDGNodeDblClickEvent OnNodeDblClick = {read=FOnNodeDblClick, write=FOnNodeDblClick, nodefault};
        __property TDGEdgeDblClickEvent OnEdgeDblClick = {read=FOnEdgeDblClick, write=FOnEdgeDblClick, nodefault};
        __property TDGNodeChangeEvent OnNodeChange = {read=FOnNodeChange, write=FOnNodeChange, nodefault};
        __property TDGEdgeChangeEvent OnEdgeChange = {read=FOnEdgeChange, write=FOnEdgeChange, nodefault};
        __property TDGInfoHintEvent OnInfoHint = {read=FOnInfoHint, write=FOnInfoHint, nodefault};
};
//---------------------------------------------------------------------------
extern PACKAGE TframeDependencyGraph *frameDependencyGraph;
//---------------------------------------------------------------------------
#endif
