//----------------------------------------------------------------------------------------------------------------------

#include <vcl.h>
#pragma hdrstop

#include "codeFrameFilterBar.h"
#include "codeSpecialSaveListView.h"
#include "codeSpecialExportListview.h"
#include "codeFilterPreferences.h"
#include "codeSpecialSqlViewer.h"
//----------------------------------------------------------------------------------------------------------------------
#pragma package(smart_init)
#pragma resource "*.dfm"
TframeFilterBar *frameFilterBar;
//----------------------------------------------------------------------------------------------------------------------
__fastcall TframeFilterBar::TframeFilterBar(TComponent* Owner) : TFrame(Owner), parent_form(NULL), filter(NULL)
{
}
//----------------------------------------------------------------------------------------------------------------------
// SetButtonsStatus - Enables/disables various toolbar buttons depending on the filter and on the user's permissions.
// Called from SetHandles when the filter pointer is made valid.
void TframeFilterBar::SetButtonsStatus()
{
  // NB: Edit/Add do their checks when the buttons are clicked, because otherwise the status of the buttons would have
  // to be updated constantly when the filter's properties and contents changed and that simply isn't manageable.
  btnCopy->Enabled      = false;
  btnDownload->Enabled  = false;
  btnExport->Enabled    = false;
  btnColumns->Enabled   = false;
  btnHeaders->Enabled   = false;
  btnPrefs->Enabled     = false;
  btnRefresh->Enabled   = false;
  btnModifySQL->Enabled = false;
  btnCheck->Enabled     = false;
  btnUncheck->Enabled   = false;
  btnFlipCheck->Enabled = false;

  if(!PtrCheck())
    return;

  btnPrefs->Enabled   = true; // Access to preferences is unconditional (so long as the filter pointer is valid)
  btnRefresh->Enabled = true; // Ability to refresh is unconditional
  btnColumns->Enabled = true; // Ability to size columns and toggle headers is unconditional
  btnHeaders->Enabled = true;

  // Has Export/Save permissions?
  if(optionsPermissions["special.save_listview_data"]["enabled"] == "true")
  {
    btnCopy->Enabled     = true;
    btnDownload->Enabled = true;
    btnExport->Enabled   = true;
  }

  // Has Modify SQL permission?
  if(optionsPermissions["special.modify_listview_sql"]["enabled"] == "true")
    btnModifySQL->Enabled = true;

  // Filter supports checkboxes?
  // Be sure to call this routine again if you change SetCheck on a filter any time other than when the filter is
  // created (this seems to be exceptionally rare in Prometheus)
  if(filter->GetCheck())
  {
    btnCheck->Enabled     = true;
    btnUncheck->Enabled   = true;
    btnFlipCheck->Enabled = true;
  }
}
//----------------------------------------------------------------------------------------------------------------------
// SetEditEnabled - Since these can't toggle automatically, an interface is provided to toggle them manually where it
// is so desired.
void TframeFilterBar::SetEditEnabled(bool enabled)
{
  btnEdit->Enabled = enabled;
}
//----------------------------------------------------------------------------------------------------------------------
// SetAddEnabled - Since these can't toggle automatically, an interface is provided to toggle them manually where it
// is so desired.
void TframeFilterBar::SetAddEnabled(bool enabled)
{
  btnAdd->Enabled = enabled;
}
//----------------------------------------------------------------------------------------------------------------------
void TframeFilterBar::SetHandles(TformSpecialGeneric *p_parent_form, classFilterObject *p_filter)
{
  parent_form = p_parent_form;
  filter      = p_filter;
  SetButtonsStatus();
}
//----------------------------------------------------------------------------------------------------------------------
bool TframeFilterBar::PtrCheck()
{
  return (parent_form != NULL && filter != NULL);
}
//----------------------------------------------------------------------------------------------------------------------
void __fastcall TframeFilterBar::btnEditClick(TObject *Sender)
{
  if(!PtrCheck())
    return;

  // Not editable. Parent form really should disable button.
  if(!filter->WhereAction() || !filter->HowToEdit())
    return;

  if(filter->GetSelectedRecordID().empty())
  {
    parent_form->PSay("Please select a record to edit in the listview first.", m_reminder, MB_OK);
    return;
  }

  // use filter object HowToEdit() value
  TObject    *where_to_call  = filter->WhereAction();
  EditRecord  how_to_edit    = filter->HowToEdit();
  string      record_to_edit = filter->GetSelectedRecordID();

  if(where_to_call && how_to_edit && !record_to_edit.empty())
    (where_to_call->*how_to_edit)(record_to_edit);
}
//----------------------------------------------------------------------------------------------------------------------
void __fastcall TframeFilterBar::btnAddClick(TObject *Sender)
{
  if(!PtrCheck())
    return;

  if(!filter->WhereAction() || !filter->HowToAdd())
    return;

  // use filter object's HowToAdd() value
  TObject   *where_to_call = filter->WhereAction();
  AddRecord  how_to_add    = filter->HowToAdd();

  if(where_to_call && how_to_add)
    (where_to_call->*how_to_add)();
}
//----------------------------------------------------------------------------------------------------------------------
void __fastcall TframeFilterBar::btnCopyClick(TObject *Sender)
{
  if(!PtrCheck())
    return;

  pvecstr headers;
  vector<show_type> header_types;
  vector<pvecstr> csv = filter->GetAllData(headers, header_types, false, false);
  CopyCSVToClipboard(csv, headers, true, false);
}
//----------------------------------------------------------------------------------------------------------------------
void __fastcall TframeFilterBar::btnDownloadClick(TObject *Sender)
{
  if(!PtrCheck())
    return;

  // download data from a list view into a file
  TListView *to_inform = filter->GetListViewHandle();

  if(to_inform)
  {
    FORM_TRY
      TformSpecialSaveListView *save_form = new TformSpecialSaveListView(NULL, to_inform);
      save_form->ShowTaskModal(parent_form);
    FORM_CATCH
  }
}
//----------------------------------------------------------------------------------------------------------------------
void __fastcall TframeFilterBar::btnExportClick(TObject *Sender)
{
  if(!PtrCheck())
    return;

  // advanced listview export (many more options!)
  TListView *to_inform = filter->GetListViewHandle();
  if(to_inform)
  {
    FORM_TRY
      TformSpecialExportListview *export_form = new TformSpecialExportListview(NULL, filter);
      export_form->ShowTaskModal(parent_form);
    FORM_CATCH
  }
}
//----------------------------------------------------------------------------------------------------------------------
void __fastcall TframeFilterBar::btnColumnsClick(TObject *Sender)
{
  if(!PtrCheck())
    return;

  // resize all
  filter->AutosizeColumns();
}
//----------------------------------------------------------------------------------------------------------------------
void __fastcall TframeFilterBar::btnHeadersClick(TObject *Sender)
{
  if(!PtrCheck())
    return;

  filter->SetHeader(!filter->GetHeader());
}
//----------------------------------------------------------------------------------------------------------------------
void __fastcall TframeFilterBar::btnPrefsClick(TObject *Sender)
{
  if(!PtrCheck())
    return;

  // tell the filter object to show its filter screen
  FORM_TRY
    TformFilterPreferences *filter_form = new TformFilterPreferences(NULL, filter);
    filter_form->ShowTaskModal(parent_form);
  FORM_CATCH
}
//----------------------------------------------------------------------------------------------------------------------
void __fastcall TframeFilterBar::btnRefreshClick(TObject *Sender)
{
  if(!PtrCheck())
    return;

  // refresh the filter
  filter->ForceRefreshDataSet();
}
//----------------------------------------------------------------------------------------------------------------------
void __fastcall TframeFilterBar::btnModifySQLClick(TObject *Sender)
{
  if(!PtrCheck())
    return;

  string sql = filter->GetSQLStatement();
  string original_sql = sql;

  FORM_TRY
    TformSpecialSqlViewer *viewer = new TformSpecialSqlViewer(NULL, &sql);
    viewer->ShowTaskModal(parent_form);
  FORM_CATCH

  if(sql != original_sql)
  {
    filter->SetSQLStatement(sql);
    filter->RefreshDataSet();
  }
}
//----------------------------------------------------------------------------------------------------------------------
void __fastcall TframeFilterBar::btnCheckClick(TObject *Sender)
{
  if(!PtrCheck())
    return;

  // Check all records
  filter->CheckAll();
}
//----------------------------------------------------------------------------------------------------------------------
void __fastcall TframeFilterBar::btnUncheckClick(TObject *Sender)
{
  if(!PtrCheck())
    return;

  // Uncheck all records
  filter->UncheckAll();
}
//----------------------------------------------------------------------------------------------------------------------
void __fastcall TframeFilterBar::btnFlipCheckClick(TObject *Sender)
{
  if(!PtrCheck())
    return;

  // Flip checked/unchecked states
  filter->FlipCheck();
}
//----------------------------------------------------------------------------------------------------------------------

