//---------------------------------------------------------------------------

#include <vcl.h>
#pragma hdrstop

#include "codeFrameObservationList.h"
#include "codeRecordObservationResponseSet.h"
#include "derivedheaders.h"
//---------------------------------------------------------------------------
#pragma package(smart_init)
#pragma resource "*.dfm"
TframeObservationList *frameObservationList;
//---------------------------------------------------------------------------
__fastcall TframeObservationList::TframeObservationList(TComponent* Owner) : TFrame(Owner)
{
 FDataSource = NULL;
 FORM_TRY
  FDataSource = new classFilterObservationSource();
 FORM_CATCH
 FDataSource->SetType(obs_list_none, "");
 FDataSource->SetFilter(&FFilter);

 FFilter.SetListViewHandle(listObservations);
 FFilter.SetSecurity("obs_response_sets"); // no specific permission, per-group stuff is done in the sql
 FFilter.SetIconStatus(true);
 FFilter.SetHeader(true);
 FFilter.SetDataSource(FDataSource); // filter will auto-delete this when it leaves. behavior might need to be changed later if we want to share these boogers.
 FFilter.RefreshDataSet();
}
//---------------------------------------------------------------------------
inline __fastcall TframeObservationList::~TframeObservationList(void)
{
  atlas->CancelRequestFrom(this);
}
//---------------------------------------------------------------------------
void TframeObservationList::SetSource (obs_list_types _Type, string _Id)
{
  if (_Type == obs_list_none || !IsBlankOrZero(_Id) || _Type == obs_list_custom)
  {
    FType = _Type;
    FId = _Id;

    // jhaley 20101210: disable Atlas refresh until the filter is fully ready.
    if (FRequest)
    {
      atlas->CancelRequest(FRequest);
      FRequest = 0;
    }

    string f = "";
    FDataSource->SetType(FType, FId);
    if (_Type == obs_list_none)
    {
      FFilter.SetAddEdit(this, NULL, NULL);
    }
    else
    {
      FFilter.SetAddEdit(this, AddRecord(&(TframeObservationList::AddObservation)), EditRecord(&(TframeObservationList::EditObservation)));
      switch (FType)
      {
      case obs_list_person: f = "person_id"; break;
      case obs_list_task: f = "task_id"; break;
      case obs_list_participation: f = "participations_id"; break;
      }
    }
    FFilter.RefreshDataSet();
    // jhaley 20101210: Do not re-enable Atlas refresh until after RefreshDataSet has returned.
    if (f != "")
      FRequest = atlas->RequestNotificationWhen("obs_response_sets." + f, FId, rel_e, as_int, this, RefreshHandler(&(TframeObservationList::AtlasRefresh)));
  }
}
//---------------------------------------------------------------------------
bool TframeObservationList::AtlasRefresh (int request_num, string matched_value)
{
  if (request_num == FRequest)
    FFilter.RefreshDataSet();
  return true;
}
//---------------------------------------------------------------------------
void TframeObservationList::AddObservation()
{
  map<string,string> temp_map;
  temp_map["id"] = "new";
  string f;
  switch (FType)
  {
   case obs_list_person: f = "person_id"; break;
   case obs_list_task: f = "task_id"; break;
   case obs_list_participation: f = "participations_id"; break;
  }
  temp_map[f] = FId;
  TformRecordObservationResponseSet::Do(temp_map);
}
//---------------------------------------------------------------------------
void TframeObservationList::EditObservation()
{
  map<string,string> temp_map;
  temp_map["id"] = FFilter.GetSelectedRecordID();
/*
  string f;
  switch (FType)
  {
   case obs_list_person: f = "person_id"; break;
   case obs_list_task: f = "task_id"; break;
   case obs_list_participation: f = "participations_id"; break;
  }
  temp_map[f] = FId;
*/
  TformRecordObservationResponseSet::Do(temp_map);
}
//---------------------------------------------------------------------------
void TframeObservationList::RestrictByGroups (vector<string> & group_ids)
{
 FDataSource->RestrictByGroups(group_ids);
 FFilter.RefreshDataSet();
}
//---------------------------------------------------------------------------
void TframeObservationList::UnRestrictByGroups ()
{
 FDataSource->UnRestrictByGroups();
 FFilter.RefreshDataSet();
}
//---------------------------------------------------------------------------
classFilterObject * TframeObservationList::GetFilter()
{
 return &FFilter;
}
//---------------------------------------------------------------------------
classFilterObservationSource::classFilterObservationSource() : classFilterSource()
{
 FType = obs_list_none;
 FRestricted = false;
}
bool classFilterObservationSource::Begin(TIBTransaction * dbTransaction)
{
 // from recordPerson
 // TODO -- some of these permissions don't actually exist (yet), just used the ones from conditions (superset)
 // TODO -- change these to obs_response_sets.* permissions?
 set<string> allowed_groups;
 if (optionsPermissions["table.measurements.common"]["view"] == "true")
   allowed_groups.insert("0");
 if (optionsPermissions["table.measurements.diab"]["view"] == "true")
   allowed_groups.insert(lookupTables->IdOf("groups", "Diabetes"));
 if (optionsPermissions["table.measurements.beh"]["view"] == "true")
   allowed_groups.insert(lookupTables->IdOf("groups", "Behavioral Health"));
 if (optionsPermissions["table.measurements.com"]["view"] == "true")
   allowed_groups.insert(lookupTables->IdOf("groups", "Community Health"));
 if (optionsPermissions["table.measurements.lac"]["view"] == "true")
   allowed_groups.insert(lookupTables->IdOf("groups", "Little Axe Clinic"));
 if (optionsPermissions["table.measurements.sc"]["view"] == "true")
   allowed_groups.insert(lookupTables->IdOf("groups", "Shawnee Clinic"));
 if (optionsPermissions["table.measurements.den"]["view"] == "true")
   allowed_groups.insert(lookupTables->IdOf("groups", "Dental Clinic"));
 if (optionsPermissions["table.measurements.cob"]["view"] == "true")
   allowed_groups.insert(lookupTables->IdOf("groups", "Contract Health"));
 if (optionsPermissions["table.measurements.pub"]["view"] == "true")
   allowed_groups.insert(lookupTables->IdOf("groups", "Public Health"));
 // TODO: possibly dangerous ... members of a group might need stuff taken away from them, even from their own group?
 allowed_groups.insert(current_groupid);

 if (FRestricted)
 {
  allowed_groups = intersect(allowed_groups, FGroups);
  // 'cause IN() doesn't like being empty, the fucker. and I don't feel like dealing with alternate sql.
  allowed_groups.insert("-1");
 }

 string sql =
  "select "
  + string(FType == obs_list_custom ? "person_names.first_name || ' ' || person_names.last_name person_name, " : "") +
  "obs_response_sets.id response_set_id, obs_response_sets.current_id, "
  "obs_response_sets.date_of_observation, groups.item group_item, obs_response_sets.is_steps_observation, "
  "obs_question_set_types.item question_set_item, "
  "obs_questions.id obs_question_id, obs_questions.short_item question_short_item, obs_questions.item question_item, "
  "obs_questions.cpt_code, obs_questions.is_calculated, "
  "obs_questions.obs_question_id question_type_id, obs_question_types.item question_type_item, "
  "obs_questions_in_sets.sequence, "
  "obs_responses.response_metric, units.item unit_item, "
  "obs_responses.obs_option_id, obs_option_types.item option_item, obs_option_types.rank, "
  "obs_responses.response_text, obs_responses.response_date "
  "from obs_response_sets "
  "left join groups on groups.id = obs_response_sets.group_id "
  + string(FType == obs_list_custom ? "left join people on people.id = obs_response_sets.person_id left join person_names on person_names.id = people.person_names_id " : "") +
  "left join obs_question_set_types on obs_question_set_types.id = obs_response_sets.obs_question_set_id "
  "left join obs_responses on obs_responses.obs_response_set_id = obs_response_sets.id "
  "left join obs_questions on obs_responses.obs_question_id = obs_questions.id "
  "left join obs_question_types on obs_question_types.id = obs_questions.obs_question_id "
  "left join units on units.id = obs_questions.unit_id "
  "left join obs_option_types on obs_option_types.id = obs_responses.obs_option_id "
  "left join obs_questions_in_sets on obs_questions_in_sets.obs_question_set_id = obs_question_set_types.id "
   "and obs_questions_in_sets.obs_question_id = obs_questions.id "
  "where "
  "(1 = 0 or obs_response_sets.group_id in (" + SetToCommaString(allowed_groups) + ")) "
  "and "
  "(1 = 0 or obs_response_sets.obs_question_set_id in (select oqs.id from obs_question_set_types oqs inner join permissions p on p.s = oqs.permission_name and p.k = 'VIEW' and p.v = 'TRUE' and p.user_id = " + current_userid + ")) "
  "and ";

 switch (FType)
 {
  case obs_list_person: sql += "obs_response_sets.person_id = " + FId; break;
  case obs_list_task: sql += "obs_response_sets.task_id = " + FId; break;
  case obs_list_participation: sql += "obs_response_sets.participations_id = " + FId; break;
  case obs_list_custom: sql += FId; break;
 }

 // to make sure control-break processing works out, and then to keep the column order more consistent?
 // TODO: not perfect. and no sort order here will fix it: it'd be nice if the columns were grouped by question-set, but:
 // - if you first get a smaller response set, then a larger one, for the same question, then the 'extra' columns go to the end
 // - questions appearing in multiple sets really foul that up too ...
 // so this -somewhat- helps (order by question set THEN by response set, which keeps responses together for a given set, then by sequence)
 // but ... still a few glitches. depending on what questions were answered.
 sql += " order by obs_response_sets.obs_question_set_id, obs_response_sets.id, obs_questions_in_sets.sequence";

 // setup default fields
 field_names.clear();
 field_types.clear();
 field_values.clear();
 row_index = 0;

 FFilter->ClearColumnSpecs();

 if (FType == obs_list_custom)
  FFilter->AddColumnSpec("Name", "person_names", as_text, true, 80);
 FFilter->AddColumnSpec("Internal Id", "internal_id", as_int, false, 80);
 FFilter->AddColumnSpec("Current Id", "current_id", as_int, false, 80);
 FFilter->AddColumnSpec("Date of Obs.", "obs_response_sets", as_date, true, 80);
 FFilter->AddColumnSpec("Group", "obs_response_sets", as_text, true, 80, prettyString);
 FFilter->AddColumnSpec("STEPS", "obs_response_sets", as_text, true, 80, prettyBoolString);
 FFilter->AddColumnSpec("Set", "obs_response_sets", as_text, true, 80, prettyString);

 if (FType == obs_list_custom)
  field_names.push_back("person_name");
 field_names.push_back("internal_id");
 field_names.push_back("current_id");
 field_names.push_back("date_of_observation");
 field_names.push_back("group");
 field_names.push_back("is_steps_observation");
 field_names.push_back("question_set");

 if (FType == obs_list_custom)
  field_types.push_back(ftString);
 field_types.push_back(ftString);
 field_types.push_back(ftString);
 field_types.push_back(ftDate);
 field_types.push_back(ftString);
 field_types.push_back(ftString);
 field_types.push_back(ftString);

 // setup data and 'extra' fields

 if (FType == obs_list_none)
  return true;

 map<string, string> data_row;
 // jhaley: Change the 0 here to 1 if you want to see EditSql windows for observation lists, and a lot of PITA crashes.
 easy_streaming_dataset data_stream(dbTransaction, sql, 0);

 map<string, unsigned int> question_to_column;
 unsigned int next_column;
 next_column = 6;
 if (FType == obs_list_custom)
  next_column++;

 string prev_obs_set_id = "";
 deque<vector<string> >::iterator current_row;
 map<string, unsigned int>::iterator current_col;
 while (data_stream.GetRow(data_row))
 {
  // new row, if necessary
  if (prev_obs_set_id != data_row["response_set_id"])
  {
   vector<string> temp_vec;
   temp_vec.resize(next_column);
   int i = 0;
   if (FType == obs_list_custom)
    temp_vec[i++] = data_row["person_name"];
   temp_vec[i++] = data_row["response_set_id"];
   temp_vec[i++] = data_row["current_id"];
   temp_vec[i++] = data_row["date_of_observation"];
   temp_vec[i++] = data_row["group_item"];
   temp_vec[i++] = data_row["is_steps_observation"];
   temp_vec[i++] = data_row["question_set_item"];
   field_values.push_back(temp_vec);
   current_row = (field_values.end() - 1);
   prev_obs_set_id = data_row["response_set_id"];
  }

  // if there are no observations at all in the set, avoid having a column with a blank title and values of 'unsupported'.
  if (!IsBlankOrZero(data_row["obs_question_id"]))
  {
   // new columns, if necessary
   current_col = question_to_column.find(data_row["obs_question_id"]);
   if (current_col == question_to_column.end())
   {
    question_to_column[data_row["obs_question_id"]] = next_column;
    next_column++;
    current_col = question_to_column.find(data_row["obs_question_id"]);

    field_names.push_back(data_row["obs_question_id"]); // hell of it, you know?
    string column_name = data_row["question_short_item"];
    if (data_row["question_type_id"] == "4")
     FFilter->AddColumnSpec(column_name, "obs_response_sets", as_double, true, 80, rawString, fc_right);
    else if (data_row["question_type_id"] == "5")
     FFilter->AddColumnSpec(column_name, "obs_response_sets", as_date, true, 80, rawString, fc_default);
    else
     FFilter->AddColumnSpec(column_name, "obs_response_sets", as_text, true, 80, rawString, fc_default);
    field_types.push_back(ftString);
   }

   // data, ol!
   string val;
   if (data_row["question_type_id"] == "5") // date
    val = data_row["response_date"];
   else if (data_row["question_type_id"] == "4") // metric
    val = data_row["response_metric"] + " " + data_row["unit_item"]; // TODO: units, maybe? maybe not?
   else if (data_row["question_type_id"] == "3") // ordinal
    val = data_row["option_item"];
   else if (data_row["question_type_id"] == "2") // text
    val = data_row["response_text"];
   else if (data_row["question_type_id"] == "1") // choice
    val = data_row["option_item"];
   else // unknown
    val = "unsupported";

   unsigned int c = current_col->second;
   if (c + 1 > current_row->size())
    current_row->resize(c + 1);
   (*current_row)[c] = val;
  }
  data_stream.Next();
 }

 // make sure all rows are the same size.
 for (deque<vector<string> >::iterator j = field_values.begin(); j != field_values.end(); j++)
  j->resize(next_column);

 return true;
}
unsigned int classFilterObservationSource::GetFieldCount()
{
 return field_names.size();
}
bool classFilterObservationSource::GetFieldNames(vector<string> & field_vec)
{
 field_vec = field_names;
 return true;
}
bool classFilterObservationSource::GetFieldTypes(vector<TFieldType> & field_vec)
{
 field_vec = field_types;
 return true;
}
bool classFilterObservationSource::GetRow(vector<string> & field_vec)
{
 if (row_index >= field_values.size())
  return false;
 field_vec = field_values[row_index];
 row_index++;
 return true;
}
void classFilterObservationSource::End()
{
 row_index = 0;
}
classFilterObservationSource::~classFilterObservationSource()
{
}
void classFilterObservationSource::SetType (obs_list_types t, string id)
{
 FType = t;
 FId = id;
}
void classFilterObservationSource::SetFilter (classFilterObject * f)
{
 FFilter = f;
}
void classFilterObservationSource::RestrictByGroups (vector<string> & group_ids)
{
 FRestricted = true;
 VecToSet(group_ids, FGroups);
}
void classFilterObservationSource::UnRestrictByGroups ()
{
 FRestricted = false;
 FGroups.clear();
}
//---------------------------------------------------------------------------

