//---------------------------------------------------------------------------

#include <vcl.h>
#pragma hdrstop

#include "codeFrameObservationResponse.h"
#include "codeRecordObservationQuestion.h"
#include "util.h"
//---------------------------------------------------------------------------
#pragma package(smart_init)
#pragma resource "*.dfm"
TframeObservationResponse *frameObservationResponse;
//---------------------------------------------------------------------------
__fastcall TframeObservationResponse::TframeObservationResponse(TComponent* Owner)
        : TFrame(Owner)
{
}
//---------------------------------------------------------------------------
__fastcall TframeObservationResponse::~TframeObservationResponse()
{
 lookupTables->UnregisterComboBox(comboOptions);
}
//---------------------------------------------------------------------------
obs_question_types TframeObservationResponse::obs_conversion (string Value)
{
 // meant to avoid MAGIC issues ... but really ... it comes out to the same, neh? if the names change, you're still screwed.
 if (Value == lookupTables->IdOf("obs_question_types", "Choice"))
  return obs_choice;
 if (Value == lookupTables->IdOf("obs_question_types", "Text"))
  return obs_text;
 if (Value == lookupTables->IdOf("obs_question_types", "Ordinal"))
  return obs_ordinal;
 if (Value == lookupTables->IdOf("obs_question_types", "Metric"))
  return obs_metric;
 if (Value == lookupTables->IdOf("obs_question_types", "Date"))
  return obs_date;
 return obs_unknown;
}
//---------------------------------------------------------------------------
void __fastcall TframeObservationResponse::SetRow (const map<string, string> & Value)
{
 _row = Value;

 // jhaley 20101207: I'm not actually sure why these names are set, as they don't seem to be used anywhere.
 // However, the fact they are being set has helped find problems higher up in the code that uses this frame. Leaving
 // it here for now until all these issues are resolved.
 try
 {
   this->Name = string("o_" + _row["obs_response_set_id"] + "_" + _row["obs_question_id"] + "_" + _row["id"]).c_str();
 }
 catch(EComponentError &e)
 {
   string message = string("Error caught in TframeObservationResponse::SetRow: ") + e.Message.c_str();
   LogIt("observationresponse.log", message.c_str());
   if(debug_level[other_forms] == 9 && message_router)
     message_router->SendMessageToAll("debug", message);
   throw; // Rethrow it! If we continue executing, VCL will suffer a serious meltdown - TComponent::Name seems to have a ref-counting glitch... 
 }

 // TODO: debug only
 panelObservation->ShowHint = false; // true;
 panelObservation->Hint = string("obs_response_set_id = " + _row["obs_response_set_id"] + "\n" + "obs_question_id = " + _row["obs_question_id"]).c_str();

 obs_question_types z = obs_conversion(_row["obs_question_type_id"]); /* renamed */

 comboOptions->Visible      = (z == obs_choice || z == obs_ordinal);
 editText->Visible          = (z == obs_text);
 editMetric->Visible        = (z == obs_metric || z == obs_date);
 labelUnits->Visible        = (z == obs_metric);

 switch (z)
 {
  case obs_text:
   editText->Text = _row["response_text"].c_str();
   break;
  // do it the same way here =)
  case obs_choice:
  case obs_ordinal:
   lookupTables->RegisterComboBox("obs_option_types", comboOptions, "select id, item, valid from obs_option_types where obs_option_set_id = " + _row["obs_option_set_id"] + " and valid = 1 order by rank, item");
   if (!IsBlankOrZero(_row["obs_option_id"]))
    comboOptions->Text = lookupTables->ValueOf("obs_option_types", _row["obs_option_id"], comboOptions).c_str();
   else
    comboOptions->Text = lookupTables->ValueOf("obs_option_types", _row["default_option_id"], comboOptions).c_str();
   break;
  case obs_metric:
   editMetric->Text = _row["response_metric"].c_str();
   labelUnits->Caption = lookupTables->ValueOf("units", _row["unit_id"]).c_str();
   break;
  case obs_date:
   editMetric->Text = _row["response_date"].c_str();
   break;
 }

 // checkObserved->Checked = (_row["id"] != ""); // no longer a has_response field to use
 // checkObserved->Caption = _row["item"].c_str(); // lookupTables->ValueOf("obs_questions", _row["obs_question_id"]).c_str();
 labelItem->Caption     = _row["item"].c_str();
 labelCPT->Caption    = _row["cpt_code"].c_str();
 labelCPT->ShowHint   = (labelCPT->Caption != "");

 bool is_autocalc       = (_row["is_calculated"] == "1");
 // checkObserved->Enabled = !is_autocalc;
 editText->Enabled      = !is_autocalc;
 comboOptions->Enabled  = !is_autocalc;
 editMetric->Enabled    = !is_autocalc;
 labelItem->Enabled     = !is_autocalc; // does this help?
}
//---------------------------------------------------------------------------
bool TframeObservationResponse::IsAutoCalc()
{
 return (_row["is_calculated"] == "1");
}
//---------------------------------------------------------------------------
map<string, string> __fastcall TframeObservationResponse::GetRow ()
{
 obs_question_types z = obs_conversion(_row["obs_question_type_id"]); /* renamed */
 map<string, string> answer; // = _row;

 // nothing renamed, usable directly.
 // finger-pointing fields get reset automatically, don't currently care about them here.
 // if inserting into the change_log, might matter, eh?
 answer["id"] = _row["id"];
 answer["obs_response_set_id"] = _row["obs_response_set_id"];
 answer["obs_question_id"]     = _row["obs_question_id"];
 answer["has_response"]        = (this->Blank ? "0" : "1"); // (checkObserved->Checked ? "1" : "0"); // not actually in db, used for intermediary code
 answer["obs_option_id"]       = "0";
 answer["response_text"]       = "";
 answer["response_metric"]     = "";
 answer["response_date"]       = "";

 // do it this way to prevent values from inappropriately being in multiple fields at once
 // not that such things would, of course, ever happen. ever.
 if (!this->Blank)
  switch (z)
  {
   case obs_choice:
   case obs_ordinal:
    answer["obs_option_id"] = lookupTables->IdOf("obs_option_types", comboOptions);
    break;
   case obs_text:
    answer["response_text"] = editText->Text.c_str();
    break;
   case obs_metric:
    answer["response_metric"] = editMetric->Text.c_str();
    break;
   case obs_date:
    answer["response_date"] = Pdate(editMetric->Text.c_str(), global_mode).OutDate(global_mode);
    break;
  }

 return answer;
}
//---------------------------------------------------------------------------
bool TframeObservationResponse::IsValid(bool silent)
{
 // if silent, simply return false if a value isn't acceptable. otherwise, inform the user usefully as to why the value cannot be accepted.
 // we could instead return a string indicating a reason + true/false, and let a single message box display all the errors together.
 // this would be interesting, architecture-wise, especially for record screens ...

 obs_question_types z = obs_conversion(_row["obs_question_type_id"]); /* renamed */

 if (!this->Blank)
  switch (z)
  {
   case obs_choice:
   case obs_ordinal:
    if (!lookupTables->IsInLookUpTable("obs_option_types", comboOptions))
    {
     if (!silent)
      PSay("Please enter something for '" + string(labelItem->Caption.c_str()) + "'", m_missing_data, MB_OK, "TframeObservationResponse");
     return false;
    }
    break;
/*
   case obs_text:
    if (editText->Text == "")
    {
     if (!silent)
      PSay("Please enter something for '" + string(checkObserved->Caption.c_str()) + "'", m_missing_data, MB_OK, "TframeObservationResponse");
     return false;
    }
    break;
*/
   case obs_metric:
    if (editMetric->Text == "" || !IsFloat(editMetric->Text.c_str()))
    {
     if (!silent)
      PSay("Please enter a valid number for '" + string(labelItem->Caption.c_str()) + "'", m_missing_data, MB_OK, "TframeObservationResponse");
     return false;
    } else
    {
     float my_val = StringToFloat(editMetric->Text.c_str());
     map<string, string> t;
     if (SqlToMap(dbDatabase, "select abs_min, abs_max, cached_mean, cached_stddev from obs_questions where obs_questions.id = " + _row["obs_question_id"], t))
     {
      if (t["abs_min"] != "")
       if (my_val < StringToFloat(t["abs_min"]))
       {
        PSay("Please enter a number greater than or equal to " + t["abs_min"] + " for '" + string(labelItem->Caption.c_str()) + "'", m_missing_data, MB_OK, "TframeObservationResponse");
        return false;
       }
      if (t["abs_max"] != "")
       if (my_val > StringToFloat(t["abs_max"]))
       {
        PSay("Please enter a number less than or equal to " + t["abs_max"] + " for '" + string(labelItem->Caption.c_str()) + "'", m_missing_data, MB_OK, "TframeObservationResponse");
        return false;
       }
      if (t["cached_mean"] != "" && t["cached_stddev"] != "")
      {
       float lower_bound = StringToFloat(t["cached_mean"]) - (3 * StringToFloat(t["cached_stddev"]));
       float upper_bound = StringToFloat(t["cached_mean"]) + (3 * StringToFloat(t["cached_stddev"]));
       if (my_val < lower_bound || my_val > upper_bound)
       {
        if (IDNO == PSay("'" + string(labelItem->Caption.c_str()) + "' of " + FloatToString(my_val) + " " + string(labelUnits->Caption.c_str()) + "\nThis value falls well outside the usual range ( 3 standard deviations from the mean.)\n\nAre you sure you want to continue?", m_warning, MB_YESNO, "TframeObservationResponse"))
         return false;
       }
      }
     } else
      PSay("Failed to get min/max/cached values.", m_debug, MB_OK, "");
    }
    break;
   case obs_date:
    if (editMetric->Text == "")
    {
     if (!silent)
      PSay("Please enter something for '" + string(labelItem->Caption.c_str()) + "'", m_missing_data, MB_OK, "TframeObservationResponse");
     return false;
    } else if (!Pdate(editMetric->Text.c_str(), global_mode).IsValidDate())
    {
     if (!silent)
      PSay("Please enter a valid date for '" + string(labelItem->Caption.c_str()) + "'", m_missing_data, MB_OK, "TframeObservationResponse");
     return false;
    }
    break;
  }

 return true;
}
//---------------------------------------------------------------------------
bool __fastcall TframeObservationResponse::GetBlank ()
{
 obs_question_types z = obs_conversion(_row["obs_question_type_id"]); /* renamed */

 switch (z)
 {
  case obs_choice:
  case obs_ordinal:
   if (comboOptions->Text != "")
    return false;
   return true;
  case obs_text:
   if (editText->Text != "")
    return false;
   return true;
  case obs_metric:
   if (editMetric->Text != "")
    return false;
   return true;
  case obs_date:
   if (editMetric->Text != "")
    return false;
   return true;
 }
 return true;
}
//---------------------------------------------------------------------------
void __fastcall TframeObservationResponse::ValueEntry(TObject *Sender)
{
  panelObservation->BevelInner = bvSpace;
  panelObservation->Color = title_bar_color;
}
//---------------------------------------------------------------------------
void __fastcall TframeObservationResponse::ValueExit(TObject *Sender)
{
  panelObservation->BevelInner = bvNone;
  panelObservation->ParentColor = true;
}
//---------------------------------------------------------------------------

