//---------------------------------------------------------------------------
#include <vcl.h>
#pragma hdrstop
#include "codeFrameSearchTaskCOBVisit.h"
#include "codeSearchTask.h"
//---------------------------------------------------------------------------
#pragma package(smart_init)
#pragma link "codeFrameSearchTaskGeneric"
#pragma resource "*.dfm"
TframeSearchTaskCOBVisit *frameSearchTaskCOBVisit;
//---------------------------------------------------------------------------
__fastcall TframeSearchTaskCOBVisit::TframeSearchTaskCOBVisit(TComponent* Owner)
        : TframeSearchTaskGeneric(Owner)
{
}
//---------------------------------------------------------------------------
void TframeSearchTaskCOBVisit::RegisterControls(TformSearchTask *task_form)
{
  // add fields which are widely shared with other frames
  task_form->AddTaxEntity(editTaxEntity);
  task_form->AddPerson("Person");
  task_form->AddTaskComments(editReason, "Reason");

  // Add editboxes
  //                    pretty_name,    box,             validation_type
  task_form->AddEditBox("COB number",   editCobNumber,   TformSearchTask::VALIDATE_INT);
  task_form->AddEditBox("PR number",    editPrNumber,    TformSearchTask::VALIDATE_INT);
  task_form->AddEditBox("check number", editCheckNumber, TformSearchTask::VALIDATE_INT);

  // Add query items
  //                      Field name                           Query Type                       Control
  task_form->AddQueryItem("T.parent_task_id",                  TformSearchTask::QUERY_CONSTANT, NULL, NULL, "!= 0"); // Should have a valid parent task
  task_form->AddQueryItem("cob.cob_number",                    TformSearchTask::QUERY_RAW,      editCobNumber);
  task_form->AddQueryItem("purchase_reqs.purchase_req_number", TformSearchTask::QUERY_RAW,      editPrNumber);
  task_form->AddQueryItem("checks.check_number",               TformSearchTask::QUERY_RAW,      editCheckNumber);

  // Special for COB Visits: disable some controls depending on various foreign-key-referenced table permissions
  if (optionsPermissions["table.conditions.cob"]["view"] != "true")
  {
    lblDiagnosis->Enabled = false;
    editDiagnosis->Enabled = false;
  }
  if (optionsPermissions["table.procedures"]["view"] != "true")
  {
    lblProcedure->Enabled = false;
    editProcedure->Enabled = false;
  }
  if (optionsPermissions["table.checks"]["view"] != "true")
  {
    lblCheckNumber->Enabled = false;
    editCheckNumber->Enabled = false;
  }
  if (optionsPermissions["table.purchase_reqs"]["view"] != "true")
  {
    lblPrNumber->Enabled = false;
    editPrNumber->Enabled = false;
  }
}
//---------------------------------------------------------------------------
void TframeSearchTaskCOBVisit::AddFilterColumnSpecs(classFilterObject *cfoTaskFilter)
{
  cfoTaskFilter->AddColumnSpec("Time", "tasks", as_time, true, 80);
  cfoTaskFilter->AddColumnSpec("COB Number", "tasks", as_int, true, 100);
}
//---------------------------------------------------------------------------
void TframeSearchTaskCOBVisit::SetFilterPreferences(classFilterObject *cfoTaskFilter)
{
  cfoTaskFilter->FetchPreferences("searchtaskcobvisit.filter");
}
//---------------------------------------------------------------------------
string TframeSearchTaskCOBVisit::GetSelectSQL()
{
  return "T.time_of_task, cob.cob_number, ";
}
//---------------------------------------------------------------------------
string TframeSearchTaskCOBVisit::GetFromSQL()
{
  string from = "left join tasks cob on cob.id = T.parent_task_id ";

  // optional joins which do some additional restrictions:
  if (editDiagnosis->Text != "")
  {
    from += "inner join conditions on conditions.task_id = T.id "
            "and conditions.code like '%" + SafeSQLString(editDiagnosis->Text.c_str()) + "%' "
            "and conditions.current_id + 1 > 3 "
            "and conditions.group_id = " + lookupTables->IdOf("groups", "Contract Health") + " ";
  }
  if (editProcedure->Text != "")
  {
    from += "inner join procedures on procedures.task_id = T.id "
            "and procedures.cpt_hcpcs like '%" + SafeSQLString(editProcedure->Text.c_str()) + "%' "
            "and procedures.current_id + 1 > 3 ";
  }
  if (editPrNumber->Text != "" || editCheckNumber->Text != "")
    from += "left join bills on bills.task_id = T.id left join purchase_reqs on purchase_reqs.id = bills.purchase_req_id ";
  if (editCheckNumber->Text != "")
    from += "left join checks on checks.id = purchase_reqs.check_id ";

  return from;
}
//---------------------------------------------------------------------------
vector<string> TframeSearchTaskCOBVisit::GetGroups()
{
  vector<string> group_vec;
  group_vec.push_back(lookupTables->IdOf("groups", "Contract Health"));
  return group_vec;
}
//---------------------------------------------------------------------------


