//---------------------------------------------------------------------------

#include <vcl.h>
#pragma hdrstop

#include "codeFrameTinyRecord_Affiliation.h"
//---------------------------------------------------------------------------
#pragma package(smart_init)
#pragma link "codeFrameTinyRecordGeneric"
#pragma link "codePaddedBox"
#pragma link "codeInlineContainer"
#pragma resource "*.dfm"
TframeTinyRecord_Affiliation *frameTinyRecord_Affiliation;
//---------------------------------------------------------------------------
__fastcall TframeTinyRecord_Affiliation::TframeTinyRecord_Affiliation(TComponent* Owner)
        : TframeTinyRecordGeneric(Owner)
{
 CanExpand = true;
 Expanded = false;
 lookupTables->RegisterComboBox("tribe_types", comboTribe);
}
//---------------------------------------------------------------------------
__fastcall TframeTinyRecord_Affiliation::~TframeTinyRecord_Affiliation()
{
 lookupTables->UnregisterComboBox(comboTribe);
}
//---------------------------------------------------------------------------
void __fastcall TframeTinyRecord_Affiliation::SetExpanded (bool Value)
{
 panelJoin->Visible = Value;
 panelRelinquish->Visible = Value;
 inlineAffiliation->Arrange();
 TframeTinyRecordGeneric::SetExpanded(Value);
}
//---------------------------------------------------------------------------
TframeTinyRecordGeneric* TframeTinyRecord_Affiliation::Create(TComponent* Owner)
{
 TframeTinyRecordGeneric * z = NULL;
 FORM_TRY
  z = new TframeTinyRecord_Affiliation(Owner);
 FORM_CATCH
 return z;
}
//---------------------------------------------------------------------------
void __fastcall TframeTinyRecord_Affiliation::inlineAffiliationArrange(
      TObject *Sender)
{
 Height = inlineAffiliation->Height;
}
//---------------------------------------------------------------------------
void __fastcall TframeTinyRecord_Affiliation::panelButtonsResize(
      TObject *Sender)
{
 inlineAffiliation->Left = ((panelButtons->Width + 8) / 8) * 8; // next multiple of 8, but not touching? (i hope)
 inlineAffiliation->Width = Width - inlineAffiliation->Left;
}
//---------------------------------------------------------------------------
void __fastcall TframeTinyRecord_Affiliation::SetId (string Value)
{
 // always needed?
 // some clearing ...
 editCDIB->Text   = "";
 comboTribe->Text = "";
 editJoin->Text   = "";
 editLeave->Text  = "";

 bool can_change = ((!IsBlankOrZero(Value) && optionsPermissions["table." + SourceTableName]["update"] == "true") ||
                    ( IsBlankOrZero(Value) && optionsPermissions["table." + SourceTableName]["insert"] == "true"));
 inlineAffiliation->Visible = (optionsPermissions["table." + SourceTableName]["view"] == "true");
 inlineAffiliation->Enabled = can_change;

 if (IsBlankOrZero(Value))
 {
  FCurrentId = "4";
  TframeTinyRecordGeneric::SetId(Value);
 } else
 {
  map<string, string> t;
  if (SqlToMap(dbDatabase, "select cdib, tribe_id, join_date, relinquish_date, current_id current_id from affiliations where id = " + Value, t))
  {
   FCurrentId = t["current_id"];

   editCDIB->Text   = PrettyString(t["cdib"]).c_str();
   comboTribe->Text = lookupTables->ValueOf("tribe_types", t["tribe_id"]).c_str();
   editJoin->Text   = t["join_date"].c_str();
   editLeave->Text  = t["relinquish_date"].c_str();

   TframeTinyRecordGeneric::SetId(Value);
  } else
   PSay("Failed to load a plausible record number", m_database_error, MB_OK, Value);
 }
}
//---------------------------------------------------------------------------
string __fastcall TframeTinyRecord_Affiliation::GetSourceTableName ()
{
 return "affiliations";
}
//---------------------------------------------------------------------------
bool TframeTinyRecord_Affiliation::Valid ()
{
 if (!lookupTables->IsInLookUpTable("tribe_types", comboTribe->Text.c_str()))
 {
  PSay("Please enter a valid tribe for each tribal affiliation.", m_invalid_data, MB_OK);
  if (IsVisible(comboTribe)) comboTribe->SetFocus();
  return false;
 }
 else if (!Pdate (editJoin->Text.c_str(), mdy | yy).IsValidDate() && ! (editJoin->Text == ""))
 {
  PSay("Please ensure that each tribal affiliation has a valid join date, or that the join date is left blank.", m_invalid_data, MB_OK);
  if (IsVisible(editJoin)) editJoin->SetFocus();
  return false;
 }
 else if (!Pdate(editLeave->Text.c_str(), mdy | yy).IsValidDate() && ! (editLeave->Text == ""))
 {
  PSay("Please ensure that each tribal affiliation has a valid relinquish date, or that the relinquish date is left blank.", m_invalid_data, MB_OK);
  if (IsVisible(editLeave)) editLeave->SetFocus();
  return false;
 }
 else if (editJoin->Text != "" && editLeave->Text != "" && Pdate(editJoin->Text.c_str(), mdy | yy) > Pdate(editLeave->Text.c_str(), mdy | yy))
 {
  PSay("Please ensure that each tribal affiliation has a join date that comes before the relinquish date.\nIf someone leaves and comes back, enter a relinquish date on the old record and create a new record for the new join date.", m_invalid_data, MB_OK);
  if (IsVisible(editJoin)) editJoin->SetFocus();
  return false;
 } else
  return true;
}
//---------------------------------------------------------------------------
bool TframeTinyRecord_Affiliation::Save (TIBTransaction * dbTransaction)
{
 AtlasBeforeSave(dbTransaction);

 map<string, string> row;
 if (!HasId)
 {
  CreateInsertMap (dbTransaction, SourceTableName, row);
  row["id"] = GetNextId(dbTransaction, SourceTableName);
 } else
  row["id"] = Id;

 row["person_id"] = PersonId;
 row["cdib"]      = editCDIB->Text.c_str();
 row["tribe_id"]  = lookupTables->IdOf("tribe_types", comboTribe);
 row["join_date"] = (editJoin->Text != "") ? Pdate (editJoin->Text.c_str(), mdy | yy).OutDate(mdy | yyyy) : string("");
 row["relinquish_date"] = (editLeave->Text != "") ? Pdate (editLeave->Text.c_str(), mdy | yy).OutDate(mdy | yyyy) : string("");

 row["current_id"]        = CurrentId;
 row["last_update_stamp"] = RightNow();
 row["last_updated_by"]   = current_userid;

 bool success = true;
 if (!HasId && (optionsPermissions["table." + SourceTableName]["insert"] == "true"))
  success = ExecuteInsertStatement(dbTransaction, SourceTableName, row);
 else if (HasId && (optionsPermissions["table." + SourceTableName]["update"] == "true"))
  if (WorthSaving(dbTransaction, row))
   success = ExecuteUpdateStatement(dbTransaction, SourceTableName, row);

 if (success)
  FId = row["id"];

 AtlasAfterSave(dbTransaction);

 return success;
}
//---------------------------------------------------------------------------
void __fastcall TframeTinyRecord_Affiliation::editAnyChange(
      TObject *Sender)
{
 should_send_change = true;
}
//---------------------------------------------------------------------------
void __fastcall TframeTinyRecord_Affiliation::editAnyExit(TObject *Sender)
{
 if (should_send_change)
  if (OnFieldChange)
   OnFieldChange(this);
 should_send_change = false;
}
//---------------------------------------------------------------------------

