//---------------------------------------------------------------------------

#include <vcl.h>
#pragma hdrstop

#include "codeFrameTinyRecord_Chart.h"
//---------------------------------------------------------------------------
#pragma package(smart_init)
#pragma link "codeFrameTinyRecordGeneric"
#pragma link "codeInlineContainer"
#pragma resource "*.dfm"
TframeTinyRecord_Chart *frameTinyRecord_Chart;
//---------------------------------------------------------------------------
__fastcall TframeTinyRecord_Chart::TframeTinyRecord_Chart(TComponent* Owner)
        : TframeTinyRecordGeneric(Owner)
{
 CanExpand = false;
 Expanded = false; // doesn't matter, does it?

 string location_groups_sql = "select id, item, valid from groups where has_charts = 1 or id = 0 order by item"; // clinics have charts
 lookupTables->RegisterComboBox("groups", comboLocation, location_groups_sql);
}
//---------------------------------------------------------------------------
__fastcall TframeTinyRecord_Chart::~TframeTinyRecord_Chart()
{
 lookupTables->UnregisterComboBox(comboLocation);
}
//---------------------------------------------------------------------------
TframeTinyRecordGeneric* TframeTinyRecord_Chart::Create(TComponent* Owner)
{
 TframeTinyRecordGeneric * z = NULL;
 FORM_TRY
  z = new TframeTinyRecord_Chart(Owner);
 FORM_CATCH
 return z;
}
//---------------------------------------------------------------------------
void __fastcall TframeTinyRecord_Chart::SetId (string Value)
{
 // always needed?
 // some clearing ...
 comboLocation->Text = "";
 editNumber->Text    = "";

 bool can_change = ((!IsBlankOrZero(Value) && optionsPermissions["table." + SourceTableName]["update"] == "true") ||
                    ( IsBlankOrZero(Value) && optionsPermissions["table." + SourceTableName]["insert"] == "true"));
 inlineChart->Visible = (optionsPermissions["table." + SourceTableName]["view"] == "true");
 inlineChart->Enabled = can_change;

 if (IsBlankOrZero(Value))
 {
  FCurrentId = "4";
  TframeTinyRecordGeneric::SetId(Value);
 } else
 {
  map<string, string> t;
  if (SqlToMap(dbDatabase, "select chart_number, group_id, current_id from charts where id = " + Value, t))
  {
   FCurrentId = t["current_id"];

   comboLocation->Text = PrettyString(lookupTables->ValueOf("groups", t["group_id"])).c_str();
   editNumber->Text    = t["chart_number"].c_str();

   TframeTinyRecordGeneric::SetId(Value);
  } else
   PSay("Failed to load a plausible record number", m_database_error, MB_OK, Value);
 }
}
//---------------------------------------------------------------------------
string __fastcall TframeTinyRecord_Chart::GetSourceTableName ()
{
 return "charts";
}
//---------------------------------------------------------------------------
bool TframeTinyRecord_Chart::Valid ()
{
 if (StringToInt(CurrentId) > 3 && comboLocation->Text == "" || !lookupTables->IsInLookUpTable("groups", comboLocation->Text.c_str()))
 {
  int answer = PSay("At least one chart on this screen is lacking a location.\n\n"
                    "If you know the location of the chart in question, you should supply this missing data.\n\n"
                    "There are times when you may not know the location of the chart in question, and it is valid to continue without it--for instance, if the chart belongs to a relative of the person you are updating.\n\n"
                    "Do you want to continue without a chart location?  If you say yes, the chart record in question will be marked as Incomplete due to missing data.\n\n"
                    "Continue with missing data and mark chart record as Incomplete?", m_confirmation, MB_YESNO);
  if (answer == IDYES)
    CurrentId = lookupTables->IdOf("current_types", "Incomplete");
  else
    return false;
 }
 if (StringToInt(CurrentId) > 3 && editNumber->Text == "")
 {
  int answer = PSay("At least one chart on this screen is lacking a number.\n\n"
                    "If you know the chart number in question, you should supply this missing data.\n\n"
                    "There are times when you may not know the chart number in question, and it is valid to continue without it--for instance, if the chart belongs to a relative of the person you are updating.\n\n"
                    "Do you want to continue without a chart number?  If you say yes, the chart record in question will be marked as Incomplete due to missing data.\n\n"
                    "Continue with missing data and mark chart record as Incomplete?", m_confirmation, MB_YESNO);
  if (answer == IDYES)
    CurrentId = lookupTables->IdOf("current_types", "Incomplete");
  else
    return false;
 }
 return true;
}
//---------------------------------------------------------------------------
bool TframeTinyRecord_Chart::Save (TIBTransaction * dbTransaction)
{
 AtlasBeforeSave(dbTransaction);

 map<string, string> row;
 if (!HasId)
 {
  CreateInsertMap (dbTransaction, SourceTableName, row);
  row["id"] = GetNextId(dbTransaction, SourceTableName);
 } else
  row["id"] = Id;

 // copied from recordChart, Paul's code (modified)
 /* TODO -c HACK: group names marked as locations match location_types by item; this is a
    temporary measure until I get the rest of my stuff committed */
 row["group_id"]     = lookupTables->IdOf("groups", comboLocation->Text.c_str());
 row["location_id"]  = lookupTables->IdOf("location_types", comboLocation->Text.c_str());
 row["chart_number"] = editNumber->Text.c_str();
 row["person_id"]    = PersonId;

 row["current_id"]        = CurrentId;
 row["last_update_stamp"] = RightNow();
 row["last_updated_by"]   = current_userid;

 bool success = true;
 if (!HasId && (optionsPermissions["table." + SourceTableName]["insert"] == "true"))
  success = ExecuteInsertStatement(dbTransaction, SourceTableName, row);
 else if (HasId && (optionsPermissions["table." + SourceTableName]["update"] == "true"))
  if (WorthSaving(dbTransaction, row))
   success = ExecuteUpdateStatement(dbTransaction, SourceTableName, row);

 if (success)
  FId = row["id"];

 AtlasAfterSave(dbTransaction);

 return success;
}
//---------------------------------------------------------------------------
void __fastcall TframeTinyRecord_Chart::inlineChartArrange(TObject *Sender)
{
 Height = inlineChart->Height;
}
//---------------------------------------------------------------------------
void __fastcall TframeTinyRecord_Chart::panelButtonsResize(TObject *Sender)
{
 inlineChart->Left = ((panelButtons->Width + 8) / 8) * 8; // next multiple of 8, but not touching? (i hope)
 inlineChart->Width = Width - inlineChart->Left;
}
//---------------------------------------------------------------------------
void __fastcall TframeTinyRecord_Chart::editAnyExit(TObject *Sender)
{
 if (should_send_change)
  if (OnFieldChange)
   OnFieldChange(this);
 should_send_change = false;
}
//---------------------------------------------------------------------------
void __fastcall TframeTinyRecord_Chart::editAnyChange(TObject *Sender)
{
 should_send_change = true;
}
//---------------------------------------------------------------------------
string TframeTinyRecord_Chart::GroupShortItem()
{
  string group_id;
  if (comboLocation->Text == "" || !lookupTables->IsInLookUpTable("location_types", comboLocation))
    group_id = GetPreviousState("group_id") != "" ? GetPreviousState("group_id") : string("0");
  else // good lookup table value
    group_id = lookupTables->IdOf("location_types", comboLocation);
  return LowercaseString(GetOneField(dbDatabase, "select short_item from groups where id = '" + group_id + "'"));
}
//---------------------------------------------------------------------------
bool TframeTinyRecord_Chart::_CanView() // yeah, we override the caching
{
  string si = GroupShortItem();
  return optionsPermissions["table.charts" + string(si!=""?".":"") + si]["view"] == "true";
}
//---------------------------------------------------------------------------
bool TframeTinyRecord_Chart::_CanUpdate() // yeah, we override the caching
{
  string si = GroupShortItem();
  return optionsPermissions["table.charts" + string(si!=""?".":"") + si]["update"] == "true";
}
//---------------------------------------------------------------------------
bool TframeTinyRecord_Chart::_CanInsert() // yeah, we override the caching
{
  string si = GroupShortItem();
  return optionsPermissions["table.charts" + string(si!=""?".":"") + si]["insert"] == "true";
}
//---------------------------------------------------------------------------

