//---------------------------------------------------------------------------
#include <sys/stat.h>
#include <dirent.h>
#include <vcl.h>
#pragma hdrstop
#pragma warn -8057

#include "codePrometheus.h"
#include "formheaders.h"
#include "codeRelationGeneric.h"
#include "action_factories.h"
#include "deferred_messages.h"
#include "framePrometheusMenu.h"
#include "vectoredexception.h" // jhaley 20110627
#include "codeSpecialIdleWarning.h"
#include "cached_files.h"      // jhaley 20120330
//---------------------------------------------------------------------------
#pragma package(smart_init)
#pragma link "codeSectionsGeneric"
#pragma resource "*.dfm"
TformPrometheusMenu *formPrometheusMenu;

// jhaley 20110708: See externheaders.h - this is slowly replacing all globals.
classPrometheusGlobals *PGlobals;

extern HINSTANCE g_hinst; // jhaley 20110415: needed by tray icon (see projectPrometheus.cpp, WinMain)

// TraceFlags -did- include:
// tfQExecute
// tfQFetch
// tfStmt

/* Question/note type thing:  I'm considering making some constants for "types" of
   things that we send debug messages for... like a constant for the required debug_level
   for an OnTimer event to send entry/exit messages.  You know, stuff that comes up in
   several places where, if we decide that those messages are getting annoying and want to
   up the required level of debug info required to send them, we can do it all at once.  Yeah? */

/* DONE: try/catch blocks for EOutOfResource errors were replaced by FORM_TRY and FORM_CATCH
         in this file (codePrometheus.cpp). */
/* TODO: However, this has not been done yet in all of the forms.  Also, there are forms
         to which I never finished (or started) adding the original try and catch blocks. */
                                               
// on the main menu...
// if you want a menu item to be not visible by default, unless the user has permissions,
// leave the tag property set to 0... the default
// if you want the menu item to be visible only if the connection to the database and
// login is sucessful, regardless of whether or not the user has permissions (for example,
// change password) set the tag to 1
// if you want the menu item to be visible no matter what, even if there is no database
// connection or the user hasn't logged in (they click Run away, etc.; e.g. About, Exit),
// set the tag to 2 -- Ashley (edited for clarity -- Paul (glanced at -- Philip))

// the globally-accessible objects,
// these are defined in the form create part of this form.

map<TStaticText *, link_struct> registered_links;
map<TMenuItem *, TStaticText *> registered_link_buttons;

TImageList * imagesCurrent;
TImageList * imagesRecordTypes;

TPopupMenu * popupListActions;
TPopupMenu * popupCurrentTypes;
TPopupMenu * popupLookupTables;

TIBDatabase * dbDatabase;
TIBDatabase * dbUpdateDatabase;

classNetworkRequestHandler * atlas;

// new stuff
classTcpipMessaging * message_router;
int connect_successful;

//sql_error last_sql_lib_error;

// list<TformRecordGeneric*>  generic_records;
// list<TformSearchGeneric*>  generic_searches;
list<TformSpecialGeneric*> generic_specials;
list<TformSpecialReminder*> reminder_forms;

map<string, map<string, string> > optionsIni;
map<string, map<string, string> > optionsPreferences; // [screen][option] = value  ["person"]["names-filter"] = "..."
map<string, map<string, string> > optionsPermissions; // [table][option] = value   ["person"]["update"] = "true"
map<string, string> site_settings;

map<string, int> form_count_map;
map<string, int> form_running_count_map;

TformSpecialChat * chat_window;
TformSpecialSchedule2 * scheduling_window;

classLookUpTableWrapper * lookupTables;

string current_userid;
string current_groupid;
string current_fullname;
string computer_name;
string user_taxid;

string auto_password;

TColor link_color;
TColor title_bar_color;
TColor today_circle_color;
TColor today_date_color;
TColor calendar_background;
TColor title_bar;
TColor title_text;
TColor week_titles;
TColor hor_line;
TColor date_color;
TColor grid_line_color;

// jhaley 20110119: listener for the main menu's deferred message queue
class classMainMenuListener : public classDeferredMessageListener
{
protected:
  TformPrometheusMenu *form;
public:
  classMainMenuListener(TformPrometheusMenu *f) : classDeferredMessageListener(), form(f) {}
  virtual const char *GetName();
  virtual bool CanAcceptMessage(classDeferredMessage &msg);
  virtual void AcceptMessage(classDeferredMessage &msg);
  virtual void MessageResult(classDeferredMessage &msg, int &result);
};

// jhaley 20110708: Global application state
int PrometheusAppState;

//---------------------------------------------------------------------------
__fastcall TformPrometheusMenu::TformPrometheusMenu(TComponent* Owner)
        : TForm(Owner)
{
  // Now beginning construction
  PrometheusAppState = PROM_STATE_CONSTRUCTING;

  // jhaley 20110708: Construct the globals object.
  PGlobals = new classPrometheusGlobals();

  // jhaley 20110627: Install a vectored exception handler so that we can intercept ALL exceptions to pass
  // down from the system before VCL gets a chance to throw away all the useful information on them.
  InstallVectoredExceptionHandler();

  // jhaley 20110119: create deferred message queue objects
  // 20110122: Use "new" here so that the classes don't have to be globally visible, due to linker problems...
  menu_msg_queue = new classDeferredMessageQueue(true, true, 500);
  menu_listener  = new classMainMenuListener(this);
  menu_msg_queue->AddListener(menu_listener);

  // jhaley 20110126: Create menu frame and attach it to panelMenuContainer, and make it aware of this form.
  FORM_TRY
    menuFrame = new TframePrometheusMenu(this);
  FORM_CATCH
  menuFrame->Parent = panelMenuContainer;
  menuFrame->Align  = alClient;
  menuFrame->SetParentForm(this);
  menuFrame->GetMainMenu()->Images = GimagesRecordTypes;
  menuFrame->GetTrayPopupMenu()->Images = GimagesRecordTypes;

  // Construction is now finished
  PrometheusAppState = PROM_STATE_CONSTRUCTED;
}
//---------------------------------------------------------------------------
__fastcall TformPrometheusMenu::~TformPrometheusMenu()
{
  if(PrometheusAppState == PROM_STATE_DESTROYING) // Already destroying?!?
    return;

  try
  {
    // jhaley 20110122: Shutdown queue objects - it'd be nice if the destructors did this, but the methods have to call
    // virtuals and those don't work properly during destruction (especially with an abstract base class!)
    if(menu_listener && menu_msg_queue)
    {
      menu_listener->ShutDown();
      menu_msg_queue->RemoveAllListeners();
    }

    if(menu_listener)
    {
      delete menu_listener;
      menu_listener = NULL;
    }
    if(menu_msg_queue)
    {
      delete menu_msg_queue;
      menu_msg_queue = NULL;
    }
  }
  catch(...) // jhaley 20110617: Hmmm... catch anything here.
  {
  }

  // jhaley 20110629: TformPrometheusMenu is destroying itself, so keep track.
  // Certain things should not happen after this point onward.
  PrometheusAppState = PROM_STATE_DESTROYING;
}

//---------------------------------------------------------------------------------------------------------------------
// *** SPLASH SCREEN METHODS ******************************************************************************************
//---------------------------------------------------------------------------------------------------------------------
//---------------------------------------------------------------------------
// SplashShow - jhaley 20110617: conditionally call down to the splash screen
void TformPrometheusMenu::SplashShow(string p_section)
{
  if(formSplash)
    formSplash->SplashShow(p_section);
}
//---------------------------------------------------------------------------
// SplashSet - jhaley 20110617: conditionally call down to the splash screen
void TformPrometheusMenu::SplashSet(string p_section, string p_value)
{
  if(formSplash)
    formSplash->SplashSet(p_section, p_value);
}

//---------------------------------------------------------------------------------------------------------------------
// *** CORE FRAME METHODS *********************************************************************************************
//---------------------------------------------------------------------------------------------------------------------
//---------------------------------------------------------------------------
void __fastcall TformPrometheusMenu::FormCreate(TObject *Sender)
{
  // constructor
  class_name = "codePrometheus";

  bool all_is_well;
  StartUp(all_is_well);
  LogIn(all_is_well);
}
//---------------------------------------------------------------------------
void __fastcall TformPrometheusMenu::FormClose(TObject *Sender, TCloseAction &Action)
{
  // Now in FormClose
  PrometheusAppState = PROM_STATE_FORMCLOSE;
  
  AV_TRY
    if(debug_level[other_forms]>=4 && message_router)
      message_router->SendMessageToAll("debug", "formPrometheusMenu::FormClose called");
  AV_CATCH
  AV_TRY
    // jhaley 20110121: If QuickReport previews are open, we need to give them a chance to close first.
    if(NumReportsOpen() > 0 && menu_msg_queue)
    {
      menu_msg_queue->CancelAllMessages();
      menu_msg_queue->ScheduleMessage(
        menu_msg_queue->CreateMessage("MainMenuListener", "MainMenuListener", "FormClose", "ReportsOpen", 5));
      if(!menu_msg_queue->IsLocked()) // If not locked, lock it, because we're going to shut down.
        menu_msg_queue->LockQueue();
      Action = caNone;
      return;
    }
    ShutDown();
  AV_CATCH

  try
  {
    // jhaley 20110415: be sure to remove tray icon
    // jhaley 20110701: moved here from destructor as that might be "too late"
    TrayMessage(NIM_DELETE);
  }
  catch(...) // Hmmm... catch anything here.
  {
    LogIt("systray.log", "Exception caught in TformPrometheusMenu::FormClose");
  }

  Action = caFree;
}
//---------------------------------------------------------------------------
// *** STARTUP SUBROUTINES ***
//---------------------------------------------------------------------------
// jhaley 20100823: Create the temporary directory where files such as reports can be stored. This directory and
// everything inside it will be removed by DeleteTempDir, which is called dead-last from ShutDown. If any portion of the
// process of creating the temporary directory fails, this will fall back on the old behavior of writing files to the
// current working directory by setting current_tempdir to ".", leaving the program in an operational state no matter
// what might happen.
void TformPrometheusMenu::CreateTempDir()
{
  // Get the system temporary directory with Win32 GetTempPath API
  char system_temp_dir[MAX_PATH];
  memset(system_temp_dir, 0, MAX_PATH); // very important!!!

  DWORD path_len = GetTempPath(MAX_PATH, system_temp_dir);

  if (path_len <= MAX_PATH && path_len != 0)
  {
    // If Windows slapped a slash on the end, it must be done away with. stat won't find it otherwise.
    if (system_temp_dir[path_len - 1] == '\\')
      system_temp_dir[path_len - 1] = '\0';
      
    // Verify that the path exists, that it is a directory, and that the user has full read/write/execute
    // permissions to that folder, because none of these facts are necessarily guaranteed (see MSDN). They
    // should be true on 99.999% of systems, however.
    struct stat s;
    if (!stat(system_temp_dir, &s) && S_ISDIR(s.st_mode) && (s.st_mode & S_IRWXU) == S_IRWXU)
    {
      // Create a new directory inside the temp directory.
      // This iteration limit is necessary only in case the user's file system changes somehow between stat
      // and mkdir and the permissions necessary become unavailable, or the parent directory was removed.
      // In that case we'd loop forever creating and throwing away GUIDs and increasing the entropy of the universe.
      int dirs_to_try = 100;
      while (dirs_to_try--)
      {
        string combined_dir = string(system_temp_dir) + "\\" + CreateClassID().c_str();

        // It should NOT exist already; if it does, somebody else could be using it.
        if (stat(combined_dir.c_str(), &s))
        {
          // Found a good non-existant name, so try to create it.
          if (!mkdir(combined_dir.c_str()))
          {
            // Got it.
            // we need to save the EXE here, since we can't delete it on exit.
            PGlobals->current_system_tempdir = system_temp_dir;
            PGlobals->current_tempdir = combined_dir;
            break;
          }
        }
      } // end while (dirs_to_try--)
    } // end if (!stat...)
  } // end if (path_len...)

  // check for any error condition - current_tempdir will not have been set if anything went wrong
  if (PGlobals->current_tempdir.length() == 0)
  {
    PGlobals->current_system_tempdir = ".";
    PGlobals->current_tempdir = "."; // use the current working directory.
  }
}
//---------------------------------------------------------------------------
// startupInitVersions - jhaley: Initializes EXE/DLL version info
void TformPrometheusMenu::startupInitVersions()
{
  AV_TRY
    PGlobals->current_version = ParseFileVersionInfo(Application->ExeName);
    GetFileVersionMap(Application->ExeName, PGlobals->current_version_map); // jhaley
    // jhaley 20110627: Pass on to the vectored exception handler module
    SetVEHVersionInfo(PGlobals->current_version_map["major"],
                      PGlobals->current_version_map["minor"],
                      PGlobals->current_version_map["release"]);

    char *temp = getenv("systemroot");
    string sysdir = "";
    if(temp)
    {
      sysdir = string(temp);
      PGlobals->gds_version = ParseFileVersionInfo((sysdir + "\\system32\\gds32.dll").c_str());
    }
  AV_CATCH
}
//---------------------------------------------------------------------------
// starutpInitDateFormats - jhaley: Initializes global date formats
void TformPrometheusMenu::startupInitDateFormats()
{
  ShortDateFormat = "m/d/yyyy";
  ShortTimeFormat = "hh:nn";
  LongTimeFormat  = "hh:nn:ss";
}
//---------------------------------------------------------------------------
// startupSetConstraints - jhaley: Sets constraints and size properties for this form
void TformPrometheusMenu::startupSetConstraints()
{
  this->AutoSize = true;

  startup_form_height = this->Height;                 // used later!
  this->Constraints->MinHeight = startup_form_height; // once established, lock in place!
  
  startup_form_width = this->Width;                   // used later!
  this->Constraints->MinWidth = startup_form_width;   // once established, lock in place!

  // set the top/left position for the menu form
  Top  = 50;
  Left = 50;
}
//---------------------------------------------------------------------------
// startupInitExterns - jhaley: Initializes a bunch of poorly encapsulated globals ;)
void TformPrometheusMenu::startupInitExterns()
{
  // get those externs going from form components -------------------------------
  imagesCurrent              = GimagesCurrent;
  imagesRecordTypes          = GimagesRecordTypes;
  popupListActions           = GpopupListActions;
  popupCurrentTypes          = GpopupCurrentTypes;
  popupLookupTables          = GpopupLookupTables;
  dbDatabase                 = GdbDatabase;
  dbUpdateDatabase           = GdbUpdateDatabase;
  dbTransaction              = NULL;
  dbUpdateTransaction        = NULL;
  current_userid             = "0"; // potential security flaw - valid userid, of a null person?
  link_color                 = clNavy;
  debug_level[other_forms]   = 0;
  debug_level[generic_forms] = 0;
  debug_level[libraries]     = 0;

  // TODO: color specs, should be overridden by optionsPreferences later
  title_bar_color     = TColor(0x00e1ffff);  // r=255, g=255, b=225  bgr = e1ffff
  today_circle_color  = TColor(0x000000ca);
  today_date_color    = clWhite;
  calendar_background = clWhite;
  title_bar           = TColor(0x00d6aa94);
  title_text          = clBlack;
  week_titles         = TColor(0x00bc6e48);
  hor_line            = clBlack;
  date_color          = clBlack;
  grid_line_color     = clGray;
}
//---------------------------------------------------------------------------
// startupInitRelationGeneric - jhaley: Initialize the TRelationGeneric utility class
void TformPrometheusMenu::startupInitRelationGeneric()
{
  // relationgeneric (and derived) static callbacks are initialized here (to 
  // null for now). Later, these will point to functions that will create forms
  // for them, using specialgeneric for consistency (without dependency issues,
  // except for their lack of a -default- implementation without Prometheus)
  TRelationGeneric::_funcFilter = NULL;
  TRelationGeneric::_funcModify = NULL;
  TRelationGeneric::_funcExport = NULL;
  TRelationGeneric::_funcInsert = NULL;
  TRelationGeneric::_funcUpdate = NULL;
}
//---------------------------------------------------------------------------
// startupInitSpecialActions - jhaley: Initialize the Special Actions and Settings classes
void TformPrometheusMenu::startupInitSpecialActions()
{
  TformSpecialAction::action_click = menuActionsClick;
  TformSpecialSettings::preferences_changed = PreferencesChanged;
  TformSpecialSettings::prometheus_menu = menuFrame->GetMainMenu();
}
//---------------------------------------------------------------------------
// startupInitClassFilterObject - jhaley: Initialize classFilterObject
void TformPrometheusMenu::startupInitClassFilterObject()
{
  // setup filter stuff (transaction started later)
  classFilterObject::SetAutosizeColumns(false);
 
  classFilterObject::funcDoubleClick        = DoubleClickListView;
  classFilterObject::funcColumnClick        = OnColumnClickListView;
  classFilterObject::funcAdvancedCustomDraw = FilterListViewAdvancedCustomDraw;
  classFilterObject::funcNeedData           = GetListViewData;
  classFilterObject::funcClick              = ListViewClick;
  classFilterObject::funcKeyDown            = ListViewKeyDown;
  classFilterObject::funcDelete             = FilterListViewDeleted; // jhaley 20100720
  classFilterObject::user_permissions       = &optionsPermissions;
  classFilterObject::user_preferences       = &optionsPreferences;
  classFilterObject::default_popup_menu     = popupListActions;
  classFilterObject::default_image_list     = imagesCurrent;
  classFilterObject::default_database       = dbDatabase;
  // classFilterObject::funcStateChange = ListViewStateChange;
}
//---------------------------------------------------------------------------
// startupInitEditListView - jhaley: Initialize the edit_listview class
void TformPrometheusMenu::startupInitEditListView()
{
  edit_listview::funcSelectItem  = formPrometheusMenu->OnEditListviewRowSelect;
  edit_listview::funcColumnClick = formPrometheusMenu->OnEditListviewColumnClick;
  edit_listview::funcDoubleClick = formPrometheusMenu->OnEditListviewDoubleClick;
}
//---------------------------------------------------------------------------
// startupInitComponents - jhaley: Initialize miscellaneous component classes
void TformPrometheusMenu::startupInitComponents()
{
  // other (components)
  funcLinkClick         = OnLinkClick;
  funcLinkMenuItemClick = OnLinkMenuItemClick;
  
  // other (specialgeneric)
  TformSpecialGeneric::funcGetDBOverrideSection = GetDBOverrideSection;
}
//---------------------------------------------------------------------------
// startupSplashScreen - jhaley - Display the splash screen
void TformPrometheusMenu::startupSplashScreen()
{
  try
  {
    formSplash = new TformPrometheusSplash(this);
  }
  catch(...) // jhaley: don't insist on creation of the splash screen, and catch ANY error here
  {
    LogIt("startup.log", "Failed to create splash screen due to an exception, ignoring.");
  }
  
  if(formSplash)
  {
    formSplash->Show();
    formSplash->allowclose = false;
  }
}
//---------------------------------------------------------------------------
// startupReadIni - jhaley: Read the configuration file
void TformPrometheusMenu::startupReadIni(bool &all_is_well)
{
  if(all_is_well)
  {
    SplashShow("ini");

    if(LoadOptionsFromFile("prometheus.ini", optionsIni))
      SplashSet("ini", "ok");
    else
      SplashSet("ini", "passed");
  }

  // Count the number of override sections in INI file to determine what to do
  int num_overrides = 0;
  string override_section;
  for(map<string,map<string,string> >::iterator itr = optionsIni.begin(); itr != optionsIni.end(); itr++)
  {
    if(itr->first != "server")
    {
      num_overrides++;
      override_section = itr->first;
    }
  }
  if(num_overrides > 0)
  {
    do_database_prompt = true;
    // Present login so they can choose which settings to use
    a_username = optionsIni["server"]["user_name"];
    a_password = auto_password;
    try
    {
      TformLoginBox * loginbox;
      FORM_TRY
        loginbox = new TformLoginBox(this, &a_username, &a_password, do_database_prompt, &db_override_section);
      FORM_CATCH
      all_is_well = (loginbox->ShowModal() == mrOk);
    }
    catch(...) // jhaley: catch any kind of error here.
    {
      all_is_well = false;
    }
    if(do_database_prompt && db_override_section != "" && db_override_section != "server")
      this->Caption = this->Caption + string("(" + db_override_section + ")").c_str();
  }

  if(optionsIni["server"]["run_mode"] == "hercules") 
    this->Caption = ReplaceStringWithString(this->Caption.c_str(), "Prometheus", "Hercules").c_str();

  // not loading the ini file is not critical, we'll create a new one with default options
  // server defaults if value missing
  if(optionsIni["server"]["atlas_interval"     ] == "") optionsIni["server"]["atlas_interval"     ] = "1000";
  if(optionsIni["server"]["admin_interval"     ] == "") optionsIni["server"]["admin_interval"     ] = "5000";
  if(optionsIni["server"]["server_name"        ] == "") optionsIni["server"]["server_name"        ] = "10.1.1.109";
  if(optionsIni["server"]["server_file"        ] == "") optionsIni["server"]["server_file"        ] = "/opt/interbase/db/prometheus.gdb";
  if(optionsIni["server"]["server_port"        ] == "") optionsIni["server"]["server_port"        ] = "3050";
  if(optionsIni["server"]["version_num"        ] == "") optionsIni["server"]["version_num"        ] = "0.0.0";
  if(optionsIni["server"]["perm_version_file"  ] == "") optionsIni["server"]["perm_version_file"  ] = "prometheus.exe";
  if(optionsIni["server"]["temp_version_file"  ] == "") optionsIni["server"]["temp_version_file"  ] = "prometheus_temp.exe";
  if(optionsIni["server"]["user_name"          ] == "") optionsIni["server"]["user_name"          ] = "default";
  if(optionsIni["server"]["auto_close_splash"  ] == "") optionsIni["server"]["auto_close_splash"  ] = "false";
  if(optionsIni["server"]["message_router_ip"  ] == "") optionsIni["server"]["message_router_ip"  ] = "10.1.1.109";
  if(optionsIni["server"]["message_router_port"] == "") optionsIni["server"]["message_router_port"] = "8945";
  if(optionsIni["server"]["base_help_url"      ] == "") optionsIni["server"]["base_help_url"      ] = "http://prometheus/";
  if(optionsIni["server"]["run_mode"           ] == "") optionsIni["server"]["run_mode"           ] = "prometheus";

  optionsIni["server"]["must_recopy"] = "false";
  optionsIni["server"]["atlas_get"  ] = "atlas";
  optionsIni["server"]["atlas_put"  ] = "atlas";
  optionsIni["server"]["admin_get"  ] = "commands";
  optionsIni["server"]["admin_put"  ] = "admin";
  optionsIni["server"]["other_put"  ] = "chat";
  optionsIni["server"]["sql_put"    ] = "sql";
  optionsIni["server"]["error_put"  ] = "error";
  optionsIni["server"]["store_put"  ] = "storage";

  // Do any special processing of ini, like setting debug info, which you want 
  // to also happen when router admin is used to manually set: a value in 
  // optionsIni remotely
  ProcessIni();
}
//---------------------------------------------------------------------------
// startupProcessCmdLine - jhaley: process command line options
void TformPrometheusMenu::startupProcessCmdLine()
{
  for(int i = 0; i <= ParamCount(); i++)
  {
    if(LikeString("--debug-level=%", LowercaseString(ParamStr(i).c_str())))
    {
      debug_level[other_forms]   = StringToInt(string(ParamStr(i).c_str()).substr(14,1));
      debug_level[generic_forms] = StringToInt(string(ParamStr(i).c_str()).substr(15,1));
      debug_level[libraries]     = StringToInt(string(ParamStr(i).c_str()).substr(16,1));
    }

    if(LikeString("--auto-password=%", LowercaseString(ParamStr(i).c_str())))
      auto_password = string(ParamStr(i).c_str()).substr(16);
  }
}
//---------------------------------------------------------------------------
// startupInitMsgRouter - jhaley: Connect to the message router server
void TformPrometheusMenu::startupInitMsgRouter()
{
  SplashShow("messaging");
  
  sockClientSocket->Address = optionsIni["server"]["message_router_ip"].c_str();
  sockClientSocket->Port    = StringToInt(optionsIni["server"]["message_router_port"]);
  tcpip_message_router      = new classTcpipMessaging(sockClientSocket);
  message_router            = tcpip_message_router;
  
  sockClientSocket->Open();
  
  while(connect_successful == 0)
  {
    Application->ProcessMessages();
    //  event_result = socket_event->WaitFor(100);
  }
  
  if(connect_successful == 1)
  {
    // we're connected
    SplashSet("messaging", "ok");
  }
  else if(connect_successful == 2)
  {
    SplashSet("messaging", "passed");
    timerReconnect->Enabled = true;
  }
}
//---------------------------------------------------------------------------
// startupGetComputerName - jhaley: get the name of the user's machine
void TformPrometheusMenu::startupGetComputerName()
{
  char          computer_name_buffer[256];
  unsigned long computer_name_size = 256;

  if(!GetComputerName(computer_name_buffer, &computer_name_size))
    message_router->SendMessageToAll(optionsIni["server"]["error_put"], "Could not determine computer's netbios name!");
  else
  {
    computer_name = computer_name_buffer;
    message_router->SetValue("computer_name", computer_name);
    message_router->SetValue("client_type",   "prometheus");
  }
}
//---------------------------------------------------------------------------
// startupConnectToDatabase - jhaley: Connect to the main Prometheus Firebird database server
void TformPrometheusMenu::startupConnectToDatabase(bool &all_is_well, TIBTransaction **created_transaction,
                                                   TIBDataSet **created_dataset)
{
  if(all_is_well)
  {
    SplashShow("db");
    try
    {
      string connect_to  = optionsIni["server"]["server_name"] + "/" +
                           optionsIni["server"]["server_port"] + ":" +
                           optionsIni["server"]["server_file"];
      string server_user = "ZEUS";
      string server_pass = "argon"; // TODO -c SECURITY: This ain't gonna pass muster with CMS...

      // global database
      ConnectToDatabase(dbDatabase, connect_to, server_user, server_pass);

      // local transaction
      TIBTransaction *dbTransaction = new TIBTransaction(NULL);
      StdTransaction(dbTransaction, dbDatabase);
      dbTransaction->Commit();

      // Return the transaction into the pointer passed in from StartUp
      *created_transaction = dbTransaction;

      // local dataset
      TIBDataSet *dbDataset = new TIBDataSet(NULL);
      dbDataset->Database = dbDatabase;
      dbDataset->Transaction = dbTransaction;
      dbDataset->UniDirectional = false;

      // Return the dataset into the pointer passed in from StartUp
      *created_dataset = dbDataset;

      // done with database setup
      SplashSet("db", "ok");
    }
    catch(EIBError &e)
    {
      message_router->SendMessageToAll(optionsIni["server"]["error_put"], 
                                       "Could not contact database from " + computer_name);
      SplashSet("db", "failed");
      PSay("Failed to connect to the database.  This could be due to a problem with the server, the network, or your "
           "computer.\n\n"
           "Check to see if you can access other network services, such as internet, email, and network drives.\n\n"
           "Please ensure that your network cable is securely connected, especially if you recently moved your "
           "computer.\n\n"
           "Close and start the program again.  If this problem persists, contact your technical support staff.\n\n"
           "\nThe specific error message caught is (" + IntToString(e.IBErrorCode) + "):\n"
           + string(e.Message.c_str()), m_database_error, MB_OK);
      all_is_well = false;
    }
    catch(...)
    {
      message_router->SendMessageToAll(optionsIni["server"]["error_put"],
                                       "Could not contact database from " + computer_name);
      SplashSet("db", "failed");
      PSay("Failed to connect to the database.  This could be due to a problem with the server, the network, or your "
           "computer.\n\n"
           "Check to see if you can access other network services, such as internet, email, and network drives.\n\n"
           "Please ensure that your network cable is securely connected, especially if you recently moved your "
           "computer.\n\n"
           "Close and start the program again.  If this problem persists, contact your technical support staff.",
           m_database_error, MB_OK);
      all_is_well = false;
    }
  }
}
//---------------------------------------------------------------------------
// startupInitAtlas - jhaley: Create the Atlas refresh system and enable its network timer
void TformPrometheusMenu::startupInitAtlas()
{
  atlas = NULL;
  atlas = new classNetworkRequestHandler(message_router);
  timerNetworkRefresh->Interval = StringToInt(optionsIni["server"]["atlas_interval"]);
  timerNetworkRefresh->Enabled = true;
}
//---------------------------------------------------------------------------
// startupExecNewVersion - jhaley: Starts up a new version of Prometheus when one was downloaded
void TformPrometheusMenu::startupExecNewVersion(bool &all_is_well, bool launch_new, int local_ver_a, int local_ver_b,
                                                int local_ver_c, const string &exe_to_launch)
{
  SplashShow("switch");

  if(all_is_well)
  {
    if(launch_new)
    {
      try
      {
        string execute_temp = "\"" + exe_to_launch + "\"";
        int new_success = WinExec(execute_temp.c_str(), 0);
        if(new_success < 32)
        {
          switch(new_success)
          {
          case 0: 
            message_router->SendMessageToAll(optionsIni["server"]["error_put"], 
              "Launch failed -- The system is out of memory or resources on " + computer_name); 
            break;
          case ERROR_BAD_FORMAT: 
            message_router->SendMessageToAll (optionsIni["server"]["error_put"], 
              "Launch failed -- The .exe file is invalid on " + computer_name); 
            break;
          case ERROR_FILE_NOT_FOUND: 
            message_router->SendMessageToAll(optionsIni["server"]["error_put"], 
              "Launch failed -- The specified file was not found on " + computer_name); 
            break;
          case ERROR_PATH_NOT_FOUND: 
            message_router->SendMessageToAll(optionsIni["server"]["error_put"], 
              "Launch failed -- The specified path was not found on " + computer_name); 
            break;
          default: 
            message_router->SendMessageToAll(optionsIni["server"]["error_put"], 
              "Launch failed -- Unknown WinExec error on " + computer_name); 
            break;
          }
        }
        else
        {
          message_router->SendMessageToAll(optionsIni["server"]["other_put"],
            "Launch succeeded -- all is well on " + computer_name);
        }

        timerDie->Enabled = true;
        all_is_well = false;
      }
      catch(...)
      {
        message_router->SendMessageToAll(optionsIni["server"]["error_put"], 
          "Launch failed -- unknown error was thrown on " + computer_name);
        optionsIni["server"]["version_num"] = IntToString(local_ver_a) + "." +
                                              IntToString(local_ver_b) + "." +
                                              IntToString(local_ver_c);
        optionsIni["server"]["copy_over_old"] = "false";
        SaveOptionsToFile("prometheus.ini", optionsIni);
        SplashSet("switch", "failed");
        all_is_well = false;
      }
    }
    else
      SplashSet("switch", "n/a");
  }
}
//---------------------------------------------------------------------------
// startupDownloadLookups - jhaley: Download all registered lookup tables and create the lookupTables global wrapper
void TformPrometheusMenu::startupDownloadLookups(bool &all_is_well, TIBTransaction *dbTransaction, TIBDataSet *dbDataset)
{
  SplashShow("lookup");

  if(all_is_well)
  {
    // now that we're connected, tell the messaging server where we are connected
    message_router->SetValue("database", optionsIni["server"]["server_name"] + ":" + 
                                         optionsIni["server"]["server_file"]);
    message_router->SetValue("version_number", PGlobals->current_version);
    AV_TRY
      message_router->SetValue("gds_version", PGlobals->gds_version);
    AV_CATCH

    // get lookup tables going
    lookupTables = NULL;
    lookupTables = new classLookUpTableWrapper(dbDatabase);
    string lookup_sql = "select table_name, can_add_to, display_option, valid_option, autoload_option "
                        "from lookup_tables "
                        "where id > 0";
    bool can_add = false;

    try
    {
      if(dbTransaction->Active) 
        dbTransaction->Commit();
      dbTransaction->StartTransaction();
      dbDataset->SelectSQL->Clear();
      dbDataset->SelectSQL->Add(lookup_sql.c_str());
      dbDataset->Open();
      dbDataset->First();
      while(!dbDataset->Eof)
      {
        if(dbDataset->FieldByName("can_add_to")->AsString == "1")
          can_add = true;
        else
          can_add = false;
        if(!lookupTables->AddLookUpTable(LowercaseString(dbDataset->FieldByName("table_name")->AsString.c_str()),
                                         can_add,
                                         dbDataset->FieldByName("display_option")->AsInteger,
                                         dbDataset->FieldByName("valid_option")->AsInteger, GpopupLookupTables,
                                         dbDataset->FieldByName("autoload_option")->AsInteger))
        {
          all_is_well = false;
        }

        dbDataset->Next();
      }
      dbDataset->Close();
      dbTransaction->Commit();
      SplashSet("lookup", "ok");
    }
    catch(...)
    {
      all_is_well = false;
    }
    if(!all_is_well)
    {
      message_router->SendMessageToAll(optionsIni["server"]["error_put"],
        "Could not download lookup tables on " + computer_name);
      SplashSet("lookup", "failed");
    }
  } // end if all_is_well
}
//---------------------------------------------------------------------------
// startupInitCurrentTypes - jhaley: Sets up the current_types dropdown menu
void TformPrometheusMenu::startupInitCurrentTypes()
{
  string     v_current_id;
  string     v_current_item;
  TMenuItem *v_current_menu;
  int        i_current_id;

  // start looking for values for the menu
  i_current_id   = 0;
  v_current_id   = IntToString(i_current_id);
  v_current_item = lookupTables->ValueOf("current_types", v_current_id);

  while(!v_current_item.empty())
  {
    v_current_item = PrettyString(v_current_item);
    v_current_menu = new TMenuItem(GpopupCurrentTypes);
    v_current_menu->ImageIndex = 3 + i_current_id;
    v_current_menu->Caption = v_current_item.c_str();
    v_current_menu->Enabled = true;
    v_current_menu->Visible = true;
    v_current_menu->OnClick = menuCurrentTypesChoice;
    GpopupCurrentTypes->Items->Add(v_current_menu);
    // next values
    i_current_id++;
    v_current_id   = IntToString(i_current_id);
    v_current_item = lookupTables->ValueOf("current_types", v_current_id);
  }
}
//---------------------------------------------------------------------------
// startupInitRemoteAdmin - jhaley: initializes the remote administration timer and capslock timer
void TformPrometheusMenu::startupInitRemoteAdmin()
{
  timerAdmin->Interval = StringToInt(optionsIni["server"]["admin_interval"]);
  timerAdmin->Enabled = true;
  timerDisableCapsLock->Enabled = true;
}
//---------------------------------------------------------------------------
// startupAtlasLookupTables - jhaley: create lookupAtlasTickets requests
void TformPrometheusMenu::startupAtlasLookupTables()
{
  pvecstr to_request_for = lookupTables->ListTables();
  for(pvecstr::iterator i_table = to_request_for.begin(); i_table != to_request_for.end(); i_table++)
  {
    int ticket_number =
      atlas->RequestNotificationWhen(*i_table, "", rel_any, as_int, this,
                                     RefreshHandler(&(TformPrometheusMenu::AtlasLookupTables)));
    lookupAtlasTickets.insert(pair<int, string>(ticket_number, *i_table));
  }
}
//---------------------------------------------------------------------------
// StartUp - Main startup and initialization routine for all of Prometheus.
void TformPrometheusMenu::StartUp(bool &all_is_well)
{
  // jhaley 20110531: Broke up gigantic monster routine into subroutines above. Moved most global declarations up here.
  TIBDataSet  *dbDataset       = NULL;
  TIBDataSet  *dbUpdateDataset = NULL;

  bool   copy_over_old  = false;
  bool   is_latest      = true;   // up-to-date version
  bool   launch_new     = false;
  int    local_ver_a    = 0;
  int    local_ver_b    = 0;
  int    local_ver_c    = 0;
  int    remote_ver_a   = 0;
  int    remote_ver_b   = 0;
  int    remote_ver_c   = 0;

  // Now in StartUp
  PrometheusAppState = PROM_STATE_STARTUP;

  CreateTempDir();          // jhaley 20100823: Create the temporary directory.
  DisableCapsLock();        // disable caps lock
  startupInitVersions();    // initialize EXE and DLL file version info structures
  startupInitDateFormats(); // initialize date format globals

  Application->OnHelp = FormHelp;

  startupSetConstraints(); // set form size and constraints

  // pointer safety
  lookupTables   = NULL;
  atlas          = NULL;
  message_router = NULL;

  startupInitExterns();           // initialize externs
  startupInitRelationGeneric();   // setup TRelationGeneric class
  startupInitSpecialActions();    // initialize SpecialActions
  startupInitClassFilterObject(); // initialize filter class
  startupInitEditListView();      // initialize edit_listview class
  startupInitComponents();        // initialize misc components

  // filter, again
  connect_successful = 0;

  // start splash screen --------------------------------------------------------
  startupSplashScreen();

  string exe_directory = ParamStr(0).c_str();
  exe_directory = exe_directory.substr(0, exe_directory.find_last_of("\\"));

  // get local options going, defaults ------------------------------------------
  all_is_well      = true;
  is_new_version   = false;  // freshly downloaded version
  computer_name    = "localhost";
  replace_attempts = 0;
  logged_in        = false;
  db_override_section = "";
  shutdown_due_to_inactivity = false;

  // get local options going, from file -----------------------------------------
  startupReadIni(all_is_well);

  // do any command line parameter processing here ------------------------------
  // This section can move elsewhere, but make sure it stays after the 
  // optionsIni["server"]["debug_level"] stuff as command line debug level 
  // settings should override ini file settings
  startupProcessCmdLine();

  // set up client socket and message router ------------------------------------
  startupInitMsgRouter();

  // settings
  copy_over_old = optionsIni["server"]["copy_over_old"] == "true";

  // jhaley: whether we try to update or not should be based on the EXE file's actual version, not what may or may
  // not be stored inside the configuration file. When these get out of sync, the entire update process becomes
  // broken and requires manual intervention to repair.

  //septhree(optionsIni["server"]["version_num"] , '.', local_ver_a, local_ver_b, local_ver_c);
  local_ver_a = PGlobals->current_version_map["major"];
  local_ver_b = PGlobals->current_version_map["minor"];
  local_ver_c = PGlobals->current_version_map["release"];

  // who are we? ----------------------------------------------------------------
  startupGetComputerName();

  // officially starting
  message_router->SendMessageToAll(optionsIni["server"]["other_put"], "Prometheus v " + 
                                   optionsIni["server"]["version_num"] + " starting on " + 
                                   computer_name + " from " + ParamStr(0).c_str());

  // copy over main file with temporary file ------------------------------------
  if(copy_over_old)
  {
    timerReplaceOld->Interval = 5000;
    timerReplaceOld->Enabled = true;
    is_new_version = true;
  }

  // get database going ---------------------------------------------------------
  startupConnectToDatabase(all_is_well, &dbTransaction, &dbDataset);

  if(all_is_well)
  {
    if(!classFilterObject::EnableTransaction(dbDatabase))
      PSay("Unable to globally enable transaction for Filter objects.", m_warning, MB_OK);
  }

  // I moved this up because I'm using Atlas to get my IP...
  // network notifications ------------------------------------------------------
  if(all_is_well)
    startupInitAtlas();

  // check version number -------------------------------------------------------
  SplashShow("version");
  if(all_is_well)
  {
    try
    {
      // figure out which update server to use
      vector<map<string, string> > update_databases;
      dbTransaction->StartTransaction();
      string update_sql = "select * from prometheus_update_sources where valid = 1 order by rule_priority desc";
      SqlToVecMap(dbTransaction, update_sql, update_databases);
      if(dbTransaction->Active) 
        dbTransaction->Commit();

      // I'm abusing atlas.  this is very much a hack.  dispatcher had to be hacked also to pass back
      // a response through atlas. Note that it's possible, but extremely unlikely, for there to be a
      // GUID collision.  in that event, worst case scenario, it downloads the exe from the wrong 
      // update server.  "darn"
      get_ip_guid = CreateClassID().c_str();
      my_ip = "";
      get_value_timed_out = false;
      timerGetValueTimeout->Enabled = true;

      // what's that make this?
      // a big damn hack, sir.
      // ain't it just.
      ip_request = atlas->RequestNotificationWhen(get_ip_guid, "", rel_any, as_text, this, 
                                                  RefreshHandler(&(TformPrometheusMenu::GetValueHandler)));
      message_router->GetValue("ip_address", get_ip_guid);

      while(my_ip == "" && !get_value_timed_out) 
        Application->ProcessMessages(); // do nothing

      atlas->CancelRequest(ip_request);

      bool have_update_connection = false;
      //is there a rule that applies to us?
      string likestring;
      for(unsigned int i = 0; i < update_databases.size(); i++)
      {
        likestring = update_databases[i]["ip_like"];
        if(LikeString(likestring, my_ip))
        {
          // rule matches!
          try
          {
            string connect_to = update_databases[i]["server_name"] + ":" + 
                                update_databases[i]["server_file"];
            PPretendToSay(computer_name + " trying update server at " + connect_to, m_debug,
                          "TformPrometheusMenu::StartUp()");
            message_router->SendMessageToAll("debug", computer_name + " trying update server at " + connect_to);
            //PSay(my_ip + " matched a rule; connecting to:  " + connect_to, m_debug, MB_OK, "codePrometheus");

            // jhaley 20110715: Crap, this was totally broken for months... use ConnectToDatabase here!
            // (Was passing a NULL loginInfo to the DB, causing an exception, and then deferring to the main database).
            if(ConnectToDatabase(dbUpdateDatabase, connect_to, "ZEUS", "argon"))
            {
              // local transaction
              dbUpdateTransaction = new TIBTransaction(NULL);
              StdTransaction(dbUpdateTransaction, dbUpdateDatabase);
              dbUpdateTransaction->Commit();

              // local dataset
              dbUpdateDataset = new TIBDataSet(NULL);
              dbUpdateDataset->Database = dbUpdateDatabase;
              dbUpdateDataset->Transaction = dbUpdateTransaction;
              dbUpdateDataset->UniDirectional = false;

              have_update_connection = true;
              break;
            }
          }
          catch (...)
          {
            message_router->SendMessageToAll(optionsIni["server"]["error_put"], 
              "Could not contact alternate update server database from " + computer_name);
            // it's all good; we fall back to the default database later on
          }
        }
      }

      if(!have_update_connection)
      { 
        // then we'll use the database we're already connected to
        dbUpdateDatabase    = dbDatabase;
        dbUpdateDataset     = dbDataset;
        dbUpdateTransaction = dbTransaction;
      }

      // find out what the latest version is
      dbUpdateTransaction->StartTransaction();
      string sql_get_version =
        "select version_number1, version_number2, version_number3 "
        "from prometheus_update2 "
        "order by version_number1 descending, version_number2 descending, version_number3 descending";
      dbUpdateDataset->SelectSQL->Add(sql_get_version.c_str());
      dbUpdateDataset->Open();
      dbUpdateDataset->First();

      // figure out who were are, and what's available
      remote_ver_a   = dbUpdateDataset->Fields->Fields[0]->AsInteger;
      remote_ver_b   = dbUpdateDataset->Fields->Fields[1]->AsInteger;
      remote_ver_c   = dbUpdateDataset->Fields->Fields[2]->AsInteger;

      if((local_ver_a < remote_ver_a) ||
         (local_ver_a == remote_ver_a && local_ver_b < remote_ver_b) ||
         (local_ver_a == remote_ver_a && local_ver_b == remote_ver_b && local_ver_c < remote_ver_c))
        is_latest = false;

      dbUpdateDataset->Close();
      dbUpdateTransaction->Commit();
      dbUpdateDataset->Close();
      SplashSet("version", "ok");
    }
    catch (...)
    {
      message_router->SendMessageToAll (optionsIni["server"]["error_put"], 
        "Could not identify new version on " + computer_name);
      SplashSet("version", "failed");
      all_is_well = false;
    }
  }

  string basepath = "";
  string dest_filename = optionsIni["server"]["temp_version_file"];

  // jhaley 20120330: If versions pulled from prometheus_update2 are running from the Program Files directory,
  // something has broken. We need to try to fix it.
  if(PIsRunningUnderPath("Program Files"))
    basepath = FileCache::PathConcatenate(FileCache::GetBasePath(), "Prometheus");
  if(basepath != "")
    dest_filename = FileCache::PathConcatenate(basepath, dest_filename);

  // download new version -------------------------------------------------------
  SplashShow("download");
  if(all_is_well)
  {
    if(!is_latest)
    {
      int checkpoint = 0; // An even better solution would be a custom data structure with message,
                          // mbox type, maybe checkpoint #, catch and re-throw one PSay at the end,
                          // no switch.  But, eh, this is improvement.
      try
      {
        // get file
        dbUpdateTransaction->StartTransaction();

        checkpoint = 1;
        string sql_get_file =
          "select file_exe "
          "from prometheus_update2 where "
          "version_number1 = '" + IntToString(remote_ver_a) + "' and "
          "version_number2 = '" + IntToString(remote_ver_b) + "' and "
          "version_number3 = '" + IntToString(remote_ver_c) + "'";
        dbUpdateDataset->SelectSQL->Clear();
        dbUpdateDataset->SelectSQL->Add(sql_get_file.c_str());
        dbUpdateDataset->Open();
        dbUpdateDataset->First();

        checkpoint = 2;
        // take out temporary file so we can download over it
        if(!DeleteFile(optionsIni["server"]["temp_version_file"].c_str()))
        {
          message_router->SendMessageToAll (optionsIni["server"]["error_put"], 
            "Deleting temporary executable file failed on " + computer_name);
        }

        checkpoint = 3;
        // save to temporary file
        ((TBlobField *)dbUpdateDataset->FieldByName("file_exe"))->SetFieldType(ftBlob);
        ((TBlobField *)dbUpdateDataset->FieldByName("file_exe"))->SaveToFile(dest_filename.c_str());

        checkpoint = 4;
        // update ini file with new version number
        optionsIni["server"]["version_num"] = IntToString(remote_ver_a) + "." + 
                                              IntToString(remote_ver_b) + "." + 
                                              IntToString(remote_ver_c);
        optionsIni["server"]["copy_over_old"] = "true";
        SaveOptionsToFile("prometheus.ini", optionsIni);

        checkpoint = 5;
        // close out
        dbUpdateDataset->Close();
        dbUpdateTransaction->Commit();
        launch_new = true;
        SplashSet("download", "ok");
        message_router->SendMessageToAll(optionsIni["server"]["other_put"], 
          "New version seems to have downloaded okay on " + computer_name);
      }
      catch(...)
      {
        message_router->SendMessageToAll(optionsIni["server"]["error_put"], 
          "Could not download new version on " + computer_name + "; checkpoint reached = " + IntToString(checkpoint));
        SplashSet("download", "failed");
        string permissions_yadda = "This likely indicates that your Windows permissions are not set "
                                   "correctly.  Please contact your network administrator.";
        switch(checkpoint)
        {
        case 0: 
          PSay("Failed to download new version:\nFailed to start transaction.",
               m_database_error, MB_OK);
          break;
        case 1:
          PSay("Failed to download new version:\nFailed to download version information.", 
               m_database_error, MB_OK); 
          break;
        case 2: 
          PSay("Failed to download new version:\nFailed to delete temporary executable file.\n\n" 
               + permissions_yadda, m_program_error, MB_OK); 
          break;
        case 3: 
          PSay("Failed to download new version:\nFailed to save new executable file.\n\n" 
               + permissions_yadda, m_program_error, MB_OK); 
          break;
        case 4: 
          PSay("Failed to save new preferences to disk.\n\n" 
               + permissions_yadda, m_program_error, MB_OK); 
          break;
        default: // jhaley 20111207: unknown problem
          PSay("Failed to download new version:\nAn unknown error occurred.\n\n"
               "Please exit Prometheus and try again. If this error persists, contact a programmer.",
               m_program_error, MB_OK);
          break;
        }
        all_is_well = false;
      }
    }
    else
      SplashSet("download", "n/a");
  }

  // start new version ----------------------------------------------------------
  startupExecNewVersion(all_is_well, launch_new, local_ver_a, local_ver_b, local_ver_c, dest_filename);

  // lookup tables --------------------------------------------------------------
  startupDownloadLookups(all_is_well, dbTransaction, dbDataset);

  // setup current_types dropdown -----------------------------------------------
  // setup the lookup values for the current_types drop-down, with images
  // (this is the pretty multi-colored drop-down with little circles... yeah)
  if(all_is_well)
    startupInitCurrentTypes();

  // remote administration ------------------------------------------------------
  if(all_is_well)
    startupInitRemoteAdmin();
    
  // request Atlas notifications for changes in LookupTables --------------------
  if(all_is_well)
    startupAtlasLookupTables();

  if(dbUpdateDataset && dbUpdateDataset != dbDataset)
    delete dbUpdateDataset;
  if (dbDataset)
    delete dbDataset;

  //Philip claims this stuff should go in startup, not login
  timerPendingFilters->Enabled = true;
  RegisterQueryFactories(query_factories);
  // set up possible actions, among other things for filter's popup menu
  RegisterActionFactories(action_factories);

  // Now in "Running" State (jhaley - unless about to die)
  if(!timerDie->Enabled)
    PrometheusAppState = PROM_STATE_RUNNING;

  // End of StartUp routine
  if(debug_level[other_forms] >= 4 && message_router)
    message_router->SendMessageToAll("debug", "formPrometheusMenu::StartUp exit");
}
//---------------------------------------------------------------------------
void TformPrometheusMenu::LogIn(bool &all_is_well)
{
  if(debug_level[other_forms]>=4 && message_router)
    message_router->SendMessageToAll("debug", "formPrometheusMenu::LogIn entry");

  if (formSplash)
  {
    if(!formSplash->Visible)
      formSplash->Visible = true;
    formSplash->allowclose = false;
  }

  // new order of things here
  // login as user --------------------------------------------------------------
  all_is_well = doLogin(all_is_well, do_database_prompt);
  if (all_is_well && is_new_version)
  {
    FORM_TRY
      TformSpecialPrometheusVersionLog * log = new TformSpecialPrometheusVersionLog(NULL, true);
      log->Show();
    FORM_CATCH
  }
  getPermissions(all_is_well);
  getPreferences(all_is_well);
  EnableMenus(all_is_well);
  if(all_is_well)
  {
    try
    {
      TrayMessage(NIM_ADD);    // jhaley 20110415: enable tray icon
      TrayMessage(NIM_MODIFY);
    }
    catch(...)
    {
      LogIt("systray.log", "Exception caught in TformPrometheusMenu::LogIn");
    }
  }
  PreferencesChanged(); // again, after menus have loaded, because of quick-menus

  // if they have the permission, host goes in title bar
  if (optionsPermissions["special.host_in_titlebar"]["enabled"] == "true")
    this->Caption = (string(this->Caption.c_str()) + "@" + optionsIni["server"]["server_name"]).c_str();

  // done -----------------------------------------------------------------------

  // jhaley 20110812: Upload crashlog
  try
  {
    UploadCrashLog();
  }
  catch(...)
  {
    // An error log for the crash log... metareferential.
    LogIt("crashlog.log", "Failed to upload crashlog");
  }

  // done
  if(formSplash)
    formSplash->allowclose = true;
  if (all_is_well)
  {
    if (formSplash && optionsIni["server"]["auto_close_splash"] == "true")
      formSplash->Visible = false;
  }
  else
  {
    // auto-terminate?
  }

  // This should go in at least ONE place outside of all_is_well business, or prometheus will never create
  // a default INI file for you to modify, if you can't connect using the default settings.
  SaveOptionsToFile("prometheus.ini", optionsIni);

  if(all_is_well)
  {
    bool must_change_password = false;
    if (GetOneField(dbTransaction, "select password_expired from users where id = " + current_userid) == "1")
    {
      PSay("Your password has expired and must be changed", m_warning, MB_OK);
      must_change_password = true;
    }
    if (!TformSpecialChangePassword::PasswordMeetsComplexityRequirements())
    {
      PSay("For security reasons, your password must meet various security requirements.\n\n"
           "The following screen outlines those requirements.  You may also refer to the Prometheus Rules of Behavior "
           "for password requirements.\n\n"
           "You will now be required to change your password.", m_warning, MB_OK);
      must_change_password = true;
    }
    if (must_change_password)
    {
      TformSpecialChangePassword * change_password = NULL;
      FORM_TRY
        change_password = new TformSpecialChangePassword(NULL, true);
      FORM_CATCH
      FORM_TRY
        change_password->ShowModal();
      FORM_CATCH
    }
  }

  if(all_is_well && (optionsPermissions["table.referrals"]["view"] == "true")) // jhaley 20111228: &&, not || here
  {
    if(!dbTransaction->Active)
      dbTransaction->StartTransaction();

    user_taxid = GetOneField(dbTransaction, "select tax_entity_id from users where id = " + current_userid);
    referrals_request = atlas->RequestNotificationWhen("hack.referrals.to_provider_id", user_taxid, rel_e, as_int, this,
                                                RefreshHandler(&(TformPrometheusMenu::DisplayNewReferralNotification)));
    if((user_taxid != "") && (user_taxid != "0"))
    {
      DisplayNewReferralNotification(referrals_request, user_taxid);
      /*
      string sql;
      sql = "select referrals.id from referrals where referrals.to_provider_id = " + user_taxid;
      vector<string> id_vec;
      SqlToVec(dbTransaction, "select users.tax_entity_id from users where users.valid = 1 and "
               "users.tax_entity_id > 0", id_vec);
      sql += " and referrals.by_provider_id in (" + VecToCommaString(id_vec) + ") "
             "and referrals.todo_status_id != " + lookupTables->IdOf("todo_status_types", "Complete");
      vector<string> referral_vec;
      SqlToVec(dbTransaction, sql, referral_vec);
      if(! referral_vec.empty())
      {
        PSay("You have interoffice referrals still pending", m_reminder, MB_OK);
        FORM_TRY
          TformSpecialWaitingReferrals *referral_form = new TformSpecialWaitingReferrals(NULL);
          referral_form->Show();
        FORM_CATCH
      }
      */
    } // end if user has a tax id
  }

  if(dbDatabase->TestConnected())
  {
    if(dbTransaction->Active)
    {
      string site_settings_sql = "select * from site_settings"; // where id = some magic number?
      SqlToMap(dbTransaction, site_settings_sql, site_settings);
    }
    else
      message_router->SendMessageToAll(optionsIni["server"]["error_put"], "Failed to fetch site settings on " + computer_name);
  }

  if(dbDatabase->TestConnected() && dbTransaction->Active)
    dbTransaction->Commit();

  stuff_happened = true;
  is_idle = false;

  message_router->SetValue("is_idle", "no");
  seconds_idle = 0;
  timerCheckIdle->Enabled = true;
  Application->OnMessage = HandleApplicationMessages;
  Application->OnActivate = FormActivate;
  Application->OnDeactivate = FormDeactivate;

  if(all_is_well)
  {
    timerAppointmentsTimer(NULL);
    timerAppointments->Enabled = true;
  }

  if(debug_level[other_forms]>=4 && message_router)
    message_router->SendMessageToAll("debug", "formPrometheusMenu::LogIn exit");

 // GdbMonitor->Enabled = true;
}
//---------------------------------------------------------------------------
void TformPrometheusMenu::LogOut()
{
  if(debug_level[other_forms] >= 4 && message_router)
    message_router->SendMessageToAll("debug", "formPrometheusMenu::LogOut entry");
  // GdbMonitor->Enabled = false;

  if(!logged_in)
  {
    if(debug_level[other_forms] >= 4 && message_router)
      message_router->SendMessageToAll("debug", "formPrometheusMenu::LogOut exiting; not logged in.");
    return;
  }

  // message_router->SendMessageToAll (optionsIni["server"]["admin_put"], "attempting formclose operation, event fired");
  // close all record forms
  /*
  if(AnyFormsBusy() > 1)
  {
    if (PSay("You have several forms still open that may need to ask you whether or not to save data.\n"
             "This can be confusing, since you may not be able to tell which form\n"
             "is asking you. You may answer 'yes' here to cancel the shutdown, and close the forms\n"
             "yourself. If you answer 'no', the application will close as soon as you\n"
             "finish answering the yes/no questions the forms ask you.\n\n"
             "Cancel shutdown, and close forms manually?", m_warning, MB_YESNO)== IDYES)
    {
      Action = caNone;
      return;
    }
  }
  */

  AV_TRY
    this->Caption = "Prometheus";
    Application->OnMessage = NULL;
    CloseAllForms();
  AV_CATCH

  // delete quick-menus
  for(vector<TSpeedButton *>::iterator k = quick_menus.begin(); k != quick_menus.end(); k++)
    delete (*k);
  quick_menus.clear();
  panelQuickMenus->Height = 0;

  AV_TRY
  if(dbDatabase->TestConnected())
    WriteLogs();
  AV_CATCH

  AV_TRY
    timerCheckIdle->Enabled = false;
    timerAppointments->Enabled = false;
  AV_CATCH

  AV_TRY
    message_router->SendMessageToAll(optionsIni["server"]["other_put"],
      "user " + optionsIni["server"]["user_name"] + " has logged out");
  AV_CATCH

  logged_in = false;

  // DONE: doesn't dispatcher show the user as still logged in at this point?
  current_userid    = "0";
  PGlobals->current_username  = "";
  current_groupid   = "";
  PGlobals->current_groupname = "";
  current_fullname  = "";
  computer_name     = "";
  user_taxid        = "";
  message_router->SetValue("user_name", "");
  message_router->SetValue("full_name", "");
  message_router->SetValue("user_id",   "");
  // some random stuff from login(), to try to mitigate problems from logging a user out at random?
  optionsPermissions.clear();
  optionsPreferences.clear();
  EnableMenus(true);
  TrayMessage(NIM_DELETE); // jhaley 20110415: remove tray icon
  PreferencesChanged();

  if(debug_level[other_forms] >= 4 && message_router)
    message_router->SendMessageToAll("debug", "formPrometheusMenu::LogOut exit");
}
//---------------------------------------------------------------------------
// jhaley 20100823: Delete all of the files in the temporary directory and then delete the directory as well.
// This is performed dead-last during ShutDown in hopes that all file handles are closed by that point, as otherwise
// the directory cannot be removed. This ought to be the case unless we are actively leaking file handles, but if all
// else fails, there's still the Windows disk cleanup wizard, right? ;)
void TformPrometheusMenu::DeleteTempDir()
{
  string &current_tempdir = PGlobals->current_tempdir;

  if (current_tempdir != "." && current_tempdir.length() > 0)
  {
    // Try to remove all the files in the directory first (this is required for rmdir to do anything);
    // use POSIX opendir since it's available in Builder, and is way simpler than Win32 findfirst/findnext.
    DIR *dir = opendir(current_tempdir.c_str());
    if (dir)
    {
      string filename;
      dirent *entry = readdir(dir);
      while (entry)
      {
        filename = entry->d_name;
        if (filename != "." && filename != "..") // don't bother trying to remove() directory pointer entries
        {
          filename = current_tempdir + "\\" + entry->d_name;
          remove(filename.c_str());
        }
        entry = readdir(dir);
      }
      closedir(dir);
    }
    // Try to remove the directory, and then go back to the default for safety, in case a temp file is needed after this point.
    rmdir(current_tempdir.c_str());
    current_tempdir = ".";
  }
}
//---------------------------------------------------------------------------
void TformPrometheusMenu::ShutDown()
{
  // Stop all timers -- jhaley 20111228: let's do this first, as fast as possible.
  AV_TRY
    timerNetworkRefresh->Enabled = false;
    timerNetworkRefresh->OnTimer = NULL;
    timerAdmin->Enabled = false;
    timerDisableCapsLock->Enabled = false;
    timerReplaceOld->Enabled = false;
    timerDie->Enabled = false;
    timerAppointments->Enabled = false;
    timerPendingFilters->Enabled = false;
    timerReconnect->Enabled = false;
    timerChatFlash->Enabled = false;
    timerCheckIdle->Enabled = false;
  AV_CATCH_SILENT

  // you do -not- want this enabled during shutdown. serious time issues.
  AV_TRY
    LogOut();

    if(debug_level[other_forms] >= 4 && message_router)
      message_router->SendMessageToAll("debug", "formPrometheusMenu::ShutDown called");
  AV_CATCH_SILENT

  // Close update transaction (moved here from login)
  AV_TRY
    if(dbUpdateTransaction && dbUpdateTransaction != dbTransaction)
    {
      if(dbUpdateTransaction->Active)
        dbUpdateTransaction->Commit();
      delete dbUpdateTransaction;
    }
  AV_CATCH_SILENT

  // Close main transaction
  AV_TRY
    if(dbTransaction)
    {
      if(dbTransaction->Active)
        dbTransaction->Commit();
      delete dbTransaction;
    }
  AV_CATCH_SILENT
  // end moved here from login

  // Shutdown Filters
  AV_TRY
    // moved here from logout
    if(dbDatabase->TestConnected())
    {
      if(connect_successful == 1)
      {
        classFilterObject::DisableTransaction();
        if(optionsPreferences["filter"]["no_persistent_hints"] != "true")
          classFilterObject::DisableFloatingHint();
      }
    }
    // end moved here from logout
  AV_CATCH_SILENT

  AV_TRY
    if(message_router) // jhaley: was missing check for null
    {
      message_router->SendMessageToAll(optionsIni["server"]["other_put"],
        "Prometheus v " + optionsIni["server"]["version_num"] + " stopping on " + computer_name);
    }
  AV_CATCH_SILENT

  // Delete Lookups
  AV_TRY
    if(lookupTables)
      delete lookupTables;
    lookupTables = NULL;
  AV_CATCH_SILENT

  // Close splash form if open
  AV_TRY
    if(formSplash)
      formSplash->Close();
  AV_CATCH_SILENT

  // Close network socket
  AV_TRY
    sockClientSocket->OnDisconnect = NULL;
    sockClientSocket->OnRead = NULL;
    sockClientSocket->OnWrite = NULL;
    sockClientSocket->OnError = NULL;
    sockClientSocket->Close();
  AV_CATCH_SILENT

  // Disconnect from the main database
  AV_TRY
    if(dbDatabase->TestConnected())
      dbDatabase->Close();
  AV_CATCH_SILENT

  if(dbUpdateDatabase != dbDatabase) // did we open a connection to a new database?
  {
    AV_TRY
      if(dbUpdateDatabase->TestConnected()) // and is it still open? (should be)
        dbUpdateDatabase->Close(); // then close it.
    AV_CATCH_SILENT
  }

  // Shutdown Atlas
  AV_TRY
    if(atlas)
      delete atlas;
    atlas = NULL;
  AV_CATCH_SILENT

  // Shutdown message router
  AV_TRY
    if(message_router)
      delete message_router;
    message_router = NULL;
    tcpip_message_router = NULL;
  AV_CATCH_SILENT

#if 0
  // jhaley 20111116: Prometheus doesn't shutdown for inactivity any longer
  AV_TRY
    if(shutdown_due_to_inactivity)
    {
      Application->MessageBox("Your Prometheus session is closing due to inactivity.\n\n"
                              "Remember to log out of Prometheus when you are done using it to avoid receiving this "
                              "message.",
                              "Inactive session closing", MB_OK | MB_ICONEXCLAMATION);
    }
  AV_CATCH_SILENT
#endif

  AV_TRY
    DeleteTempDir(); // jhaley 20100823: Remove everything in the temporary directory.
  AV_CATCH_SILENT

  // Application->MessageBox("ShutDown is done", "Debug", MB_OK | MB_ICONEXCLAMATION);
  // delete dbDatabase; - Nope, this object belongs to the main form; its scope ends automatically.
}
//---------------------------------------------------------------------------
// Log in.  Since it's dependent on things up to this point having worked properly, it needs all_is_well.
// suppress_prompt_once is mainly for do_database_prompt--situations where we've already prompted for a user/pass
// and want to try what we have once, and if it fails, prompt them until they give a valid one or run away
bool TformPrometheusMenu::doLogin(bool &all_is_well, bool suppress_prompt_once)
{
  if(debug_level[other_forms]>=4 && message_router)
    message_router->SendMessageToAll("debug", "TformPrometheusMenu::doLogin entry");

  SplashShow("login");

  if (all_is_well)
  {
    bool got_login = false;
    int loginresult = 0;
    if(!do_database_prompt)
    {
      a_username = optionsIni["server"]["user_name"];
      a_password = auto_password;
    }

    bool first_pass = true;
    while (all_is_well && !got_login)
    {
      TformLoginBox * loginbox;
      FORM_TRY
        loginbox = new TformLoginBox(this, &a_username, &a_password, false, NULL);
      FORM_CATCH
      // if we presented the login box earlier due to multiple databases to connect to, try the
      // username and password they entered earlier one time.
      if (suppress_prompt_once && first_pass)
        loginresult = mrOk;
      else
        loginresult = loginbox->ShowModal();
      first_pass = false;
      if(formSplash)
        formSplash->Update();
      if (loginresult == mrOk)
      {
        string temp_result_id = "";
        // TODO: encrypt passwords in database and over the wire (and then maybe make multiple actual logins? heh.)
        // change here if you want password encryption - encrypt a_password before sending,
        // and hunt down the password-change screen to modify it too (so encrypted passwords
        // are stored in the database.) we don't care, we like to spy on passwords.
        // jhaley 20110104: Ability to login now contingent on "can_log_in" field rather than "valid"
        string sql_get_userid =
          "select id from users where username = '" + ReplaceStringWithString(a_username, "'", "''") + "' "
          "and pass = '" + ReplaceStringWithString(a_password, "'", "''") + "' and can_log_in = '1'";
        temp_result_id = GetOneField (dbTransaction, sql_get_userid);
        if (temp_result_id != "")
        {
          map<string, string> log_map;
          map<string, int> sql_options;

          current_userid = temp_result_id;
          PGlobals->current_username = GetOneField(dbTransaction, "select username from users where id = " + current_userid);
          current_fullname = GetOneField(dbTransaction, "select item from users where id = " + current_userid);
          log_map["id"           ] = "1";
          log_map["user_id"      ] = current_userid;
          log_map["log_timestamp"] = "current_timestamp";
          log_map["computer_name"] = computer_name;
          log_map["is_log_in"    ] = "1";
          sql_options["log_timestamp"] = sql_no_processing;
          ExecuteInsertStatement(dbTransaction, "log_log", log_map, sql_options);
          atlas->Notify("log_log.users", current_userid);
          message_router->SetValue("user_name", a_username);
          message_router->SetValue("full_name", current_fullname);
          message_router->SetValue("user_id", current_userid);
          message_router->AddCallback("chat_messages", this, ChannelReadable(&(TformPrometheusMenu::HandleChatMessages)));
          got_login = true;
          string sql_group_id = "select groups.id from groups, users where users.group_id = groups.id and "
                                "users.id = " + current_userid;
          current_groupid = GetOneField(dbTransaction, sql_group_id);
          PGlobals->current_groupname = lookupTables->ValueOf("groups", current_groupid);
          optionsIni["server"]["user_name"] = a_username;
          message_router->SendMessageToAll (optionsIni["server"]["other_put"], "user " + a_username + " has logged in");
          this->Caption = string(string(this->Caption.c_str()) + " - " + a_username).c_str();
          SplashSet("login", "ok");
          logged_in = true;
        }
        else
        {
          got_login = false;
          SplashSet("login", "failed");
          message_router->SendMessageToAll(optionsIni["server"]["error_put"],
                                           "user " + a_username + " has forgotten his/her password");
        }
        a_password = "";
      }
      else
      {
        if(!logged_in)
        {
          all_is_well = false;
          //this->Caption = string("Prometheus").c_str();
          message_router->SendMessageToAll(optionsIni["server"]["other_put"], "User ran away");
          SplashSet("login", "failed");
          SplashSet("notes", "You must have a valid login/password to do anything.");
        }
        else
        {
          all_is_well = false; // still tell the switch users screen that they cancelled
          SplashSet("login", "passed");
        }
      }
    }
  }

  if(debug_level[other_forms] >= 4 && message_router)
    message_router->SendMessageToAll("debug", "TformPrometheusMenu::doLogin exit");

  return all_is_well;
}
//---------------------------------------------------------------------------
void TformPrometheusMenu::getPreferences(bool &all_is_well)
{
  if(debug_level[other_forms] >= 4 && message_router)
    message_router->SendMessageToAll("debug", "TformPrometheusMenu::getPreferences entry");

  SplashShow("preferences");

  if(all_is_well)
  {
    try
    {
      // get preferences
      string sql_get_preferences =
        "select s, k, v "
        "from preferences "
        "where user_id = '" + current_userid + "'";
      FillSKVTable(dbTransaction, sql_get_preferences, optionsPreferences);
      // set global link color and title bar color used by forms from options preferences
      PreferencesChanged();
      SplashSet("preferences", "ok");
    }
    catch(...)
    {
      message_router->SendMessageToAll(optionsIni["server"]["error_put"],
        "Could not download preferences on " + computer_name);
      SplashSet("preferences", "failed");
      all_is_well = false;
      optionsPreferences.clear();
      PreferencesChanged();
    }
  }
  else
  {
    SplashSet("preferences", "n/a");
    optionsPreferences.clear();
    PreferencesChanged();
  }

  if(debug_level[other_forms] >= 4 && message_router)
    message_router->SendMessageToAll("debug", "TformPrometheusMenu::getPreferences exit");
}
//---------------------------------------------------------------------------
void TformPrometheusMenu::getPermissions(bool &all_is_well)
{
 if(debug_level[other_forms]>=4 && message_router)
   message_router->SendMessageToAll("debug", "formPrometheusMenu::getPermissions entry");

 SplashShow("permissions");

 if (all_is_well)
 {
  try
  {
   // get permissions
   string sql_get_permissions =
    "select s, k, v "
    "from permissions "
    "where user_id = '" + current_userid + "'";
   FillSKVTable(dbTransaction, sql_get_permissions, optionsPermissions);
   SplashSet("permissions", "ok");

   // if this affects lookup tables, we should force them all to refresh or something.
   vector<string> all_lookup_tables = lookupTables->ListTables();
   for (vector<string>::iterator i_table = all_lookup_tables.begin(); i_table != all_lookup_tables.end(); i_table++)
    lookupTables->ExpectRefresh(*i_table);

  } // end try

  catch(...)
  {
   optionsPermissions.clear();
   message_router->SendMessageToAll (optionsIni["server"]["error_put"], "Could not download permissions on " + computer_name);
   SplashSet("permissions", "failed");
   all_is_well = false;
  }
 }
 else
 {
   SplashSet("permissions", "n/a");
   optionsPermissions.clear();
   // disable all menu items with tag of 1 because something didn't work
   RecurseMenu(menuFrame->GetMainMenu(), 1, false);
 }
 if(debug_level[other_forms]>=4 && message_router) message_router->SendMessageToAll("debug", "formPrometheusMenu::getPermissions exit");
}
//---------------------------------------------------------------------------
bool TformPrometheusMenu::SavePreferences ()
{
 if(debug_level[other_forms]>=4 && message_router)
   message_router->SendMessageToAll("debug", "formPrometheusMenu::SavePreferences entry");

 // went to the trouble to switch all the following code to a prepared insert statement,
 // then discovered that, really, it's fast anyway ... and if you take out the message_router->SendMessageToAll's
 // from the logout procedure, it closes in half a second. it's SHITTING fast, other than
 // the damn mailslots.
 if (logged_in && dbDatabase->TestConnected())
 {
  // save the SKV table optionsPreferences to the database
  // by first deleting all preferences from that table, for
  // that user, then re-uploading all the current preferences.
  // it was going to be hard to tell which preferences needed
  // inserting, and which needed updating.

  TIBTransaction * dbTransaction = NULL;
  string sql_statement;
  map<string, string> insert_map;

  try
  {
   dbTransaction = new TIBTransaction(NULL);
   StdTransaction(dbTransaction, dbDatabase);
   ExecuteStatement(dbTransaction, "delete from preferences where user_id = " + current_userid);

   list<vector<string> > to_save;
   SKVToPreparedInsert(optionsPreferences, to_save);

   for (list<vector<string> >::iterator i_row = to_save.begin();
        i_row != to_save.end(); i_row++)
    i_row->push_back(current_userid);

   vector<string> field_names;
   field_names.resize(4);
   field_names[0] = "s";
   field_names[1] = "k";
   field_names[2] = "v";
   field_names[3] = "user_id";

   if (!ExecutePreparedInsert (dbTransaction, "preferences", field_names, to_save))
   {
    dbTransaction->Rollback();
    PSay("For some reason, your preferences could not be saved.\n"
         "Ask your system administrator about this problem soon.", m_database_error, MB_OK);
   }
   else
    dbTransaction->Commit();
  }
  catch (...)
  {
   if (dbTransaction->Active)
    dbTransaction->Rollback();
   return false;
  }
  delete dbTransaction;
 }
 else
 {
  // PSay("Skipped saving preferences, not logged in", m_database_error, MB_OK);
 }
 if(debug_level[other_forms]>=4 && message_router)
   message_router->SendMessageToAll("debug", "formPrometheusMenu::SavePreferences exit");
 return true;
}
//---------------------------------------------------------------------------
int TformPrometheusMenu::PSay(string p_message, prometheus_message_types p_message_type, int p_buttons, string p_other_info)
{
  return ::PSay(p_message, p_message_type, p_buttons, "TformPrometheusMenu", p_other_info);
}
//---------------------------------------------------------------------------
void __fastcall TformPrometheusMenu::HandleApplicationMessages(tagMSG &Msg, bool &Handled)
{
  // Only put entry/exit messages in this function if you are TERMINALLY INSANE.  kthx.
  if((Msg.message == WM_MOUSEMOVE) || (Msg.message == WM_KEYUP))
    stuff_happened = true;
  // Handled = false; // this is the default; only need to set when true
}
//---------------------------------------------------------------------------
void __fastcall TformPrometheusMenu::HandleClosingExceptions(TObject *Sender, Exception *E)
{
  // do nothing
}
//---------------------------------------------------------------------------
bool TformPrometheusMenu::CloseAllForms ()
{
  if(debug_level[other_forms]>=4 && message_router)
    message_router->SendMessageToAll("debug", "formPrometheusMenu::CloseAllForms entry");

  // close all open forms like closing the application would
  bool no_errors_closing = true;

  // PSay("There are " + IntToString(generic_specials.size()) + " forms to close.", m_debug, MB_OK, "");
  while (generic_specials.size())
  {
    AV_TRY
    TformSpecialGeneric *to_close = generic_specials.front();

    // PSay("Looking at " + IntToString((int)(void*)(generic_specials.front())), m_debug, MB_OK, "");
    if (to_close)
    {
      AV_TRY
        // Detach it, then destroy it. why? because the parent may have aborted -its- constructor and we still want to
        // destroy this. It would actually be best if SpecialGeneric did more processing -only- if the constructor
        // succeeds, but with inheritance ... that's a mess. no? yes.
        if (to_close->DrawerOwner)
          to_close->DrawerOwner = NULL;

        ShortString temp_short = to_close->ClassName();
        AnsiString  temp_ansi(temp_short);
        string      temp_string = temp_ansi.c_str();
        if (!to_close->Visible)
        {
          PPretendToSay("Closing invisible form of type " + temp_string + " during shutdown.", m_debug,
                        "TformPrometheusMenu::CloseAllForms", temp_string);
        }

        if(to_close->FormState.Contains(fsModal))
          to_close->ModalResult = 99;
        else if (!to_close->CloseNow())
          no_errors_closing = false;
      AV_CATCH
      generic_specials.remove(to_close); // just in case
    }
    else
    {
      generic_specials.remove(to_close);
      PPretendToSay("NULL SpecialGeneric * found during CloseAllForms", m_debug, "TformPrometheusMenu::CloseAllForms");
      message_router->SendMessageToAll(optionsIni["server"]["error_put"],
                                       "NULL SpecialGeneric * found during CloseAllForms on " + computer_name);
    }
    AV_CATCH
    Application->ProcessMessages();
  }

  string about_running_count;
  for (map<string, int>::iterator i = form_running_count_map.begin(); i != form_running_count_map.end(); i++)
  {
    if (!about_running_count.empty())
      about_running_count += ",\n ";
    about_running_count += (i->first + ": " + IntToString(i->second));
  }

  if (!about_running_count.empty())
  {
    PPretendToSay("List of possible null pointer forms:\n" + about_running_count, m_debug,
                  "TformPrometheusMenu::CloseAllForms", about_running_count);
  }

  if (!no_errors_closing)
  {
    message_router->SendMessageToAll(optionsIni["server"]["error_put"],
                                     "Some forms refused to close for " + current_fullname);
  }

  if(debug_level[other_forms] >= 4 && message_router)
    message_router->SendMessageToAll("debug", "formPrometheusMenu::CloseAllForms exit");

  return no_errors_closing;
}
//---------------------------------------------------------------------------
int TformPrometheusMenu::AnyFormsBusy()
{
  if(debug_level[other_forms]>=4 && message_router)
    message_router->SendMessageToAll("debug", "formPrometheusMenu::AnyFormsBusy entry");

  int busy_forms = 0;

  for (list<TformSpecialGeneric*>::iterator i_spec = generic_specials.begin();
       i_spec != generic_specials.end(); i_spec++)
  {
    if ((*i_spec)->IsBusy())
      busy_forms++;
  }

  if(debug_level[other_forms] >= 4 && message_router)
    message_router->SendMessageToAll("debug", "formPrometheusMenu::AnyFormsBusy exit");

  return busy_forms;
}
//--------------------------------------------------------------------------
void TformPrometheusMenu::DoSwitchUsers()
{
  bool all_is_well = true;
  a_username = PGlobals->current_username;
  a_password = "";
  LogOut();
  if(formSplash)
    formSplash->SplashReset();
  LogIn(all_is_well);
}
//---------------------------------------------------------------------------
void TformPrometheusMenu::DisplayNewReferralNotification(int request_number, string matched_value)
{
  if(debug_level[other_forms]>=4 && message_router)
    message_router->SendMessageToAll("debug", "formPrometheusMenu::DisplayNewReferralNotification entry");

  // actually ignoring request_number, because ... it doesn't matter?
  //request_number;

  //string message_from_server = message_router->GetNextMessage("referrals");
  //string message_to_user = "A new interoffice referral has just been made to you by " + message_from_server;
  if (optionsPermissions["table.referrals"]["view"] == "true")
  {
    string referrals_sql =
      "select count(*) from referrals inner join tax_entities on referrals.by_provider_id = tax_entities.id "
      "inner join users on users.tax_entity_id = tax_entities.id "
      "where referrals.person_id > 0 and referral_reason_id is not null and to_provider_id = " + user_taxid +
      " and todo_status_id != " + lookupTables->IdOf("todo_status_types", "Complete");

    string count_important_referrals = GetOneField(dbDatabase, referrals_sql);

    /* DONE: if suspended ... actually go ahead and make them see it's suspended. that's not the same as cancelled,
       right? wait, there -is- no cancelled.
       -- Carolyn tells me that they shouldn't be able to cancel it.  Suspended should still constantly pester them, as
          they haven't met their obligation of receiving a referral yet.  If people complain, it's probably because
          they're not doing their jobs.  -- Paul
    */
    if (IsInt(count_important_referrals) && StringToInt(count_important_referrals) > 0)
    {
      set<TformSpecialGeneric *> current_forms = FindOpenFormsByType("TformSpecialWaitingReferrals", false);

      if (current_forms.empty())
      {
        FORM_TRY
          TformSpecialWaitingReferrals *referral_form = new TformSpecialWaitingReferrals(NULL);
          referral_form->Show();
        FORM_CATCH
      }
      else
      {
        TformSpecialGeneric *a_form = *(current_forms.begin());
        a_form->Show();
      }
    }
  }

  if(debug_level[other_forms] >= 4 && message_router)
    message_router->SendMessageToAll("debug", "formPrometheusMenu::DisplayNewReferralNotification exit");
}
//---------------------------------------------------------------------------
bool __fastcall TformPrometheusMenu::FormHelp(WORD Command, int Data, bool &CallHelp)
{
  if(debug_level[other_forms] >= 4 && message_router)
    message_router->SendMessageToAll("debug", "formPrometheusMenu::FormHelp entry");

  // system-wide help request
  if(Screen->ActiveForm && Screen->ActiveForm != this && Screen->ActiveForm->OnHelp)
    Screen->ActiveForm->OnHelp(Command, Data, CallHelp);

  CallHelp = false;

  if(debug_level[other_forms]>=4 && message_router)
    message_router->SendMessageToAll("debug", "formPrometheusMenu::FormHelp exit");

  return true;
}
//-----------------------------------------------------------------------
void TformPrometheusMenu::DisableCapsLock()
{
  // disable caps lock
  if(Application->Active)
  {
    unsigned char state[256];
    GetKeyboardState(state);
    if(state[VK_CAPITAL] == 1)
    {
      INPUT input[2];
      ::ZeroMemory(input, sizeof(input));
      input[0].type = input[1].type = INPUT_KEYBOARD;
      input[0].ki.wVk  = input[1].ki.wVk = VK_CAPITAL;
      input[1].ki.dwFlags = KEYEVENTF_KEYUP;  // THIS IS IMPORTANT
      ::SendInput(2, input, sizeof(INPUT));
    }
  }
}
//---------------------------------------------------------------------------
void __fastcall TformPrometheusMenu::timerDisableCapsLockTimer(TObject *Sender)
{
  if(PrometheusAppState == PROM_STATE_RUNNING && Application->Active) // jhaley 20111227: only while running normally.
    DisableCapsLock();
}
//---------------------------------------------------------------------------
void __fastcall TformPrometheusMenu::FormDeactivate(TObject *Sender)
{
  // jhaley 20111227: A well known VCL bug results in FormDeactivate events being sent to deleted forms. These should
  // always be ignored when possible. 
  if(PrometheusAppState == PROM_STATE_DESTROYING)
    return;

  if(debug_level[other_forms] >= 4 && message_router)
    message_router->SendMessageToAll("debug", "formPrometheusMenu::FormDeactivate entry");

  // this is actually pointed-to by the TApplication
  // drawers
  TForm *temp = Screen->ActiveForm;
  if(temp)
  {
    if(temp->InheritsFrom(__classid(TformSpecialGeneric)))
    {
      TformSpecialGeneric *active_form = (TformSpecialGeneric *)temp;
      active_form->OnDeactivate(NULL);
    }
  }

  if(debug_level[other_forms] >= 4 && message_router)
    message_router->SendMessageToAll("debug", "formPrometheusMenu::FormDeactivate exit");
}
//---------------------------------------------------------------------------
void __fastcall TformPrometheusMenu::FormActivate(TObject *Sender)
{
  if(debug_level[other_forms] >= 4 && message_router)
    message_router->SendMessageToAll("debug", "formPrometheusMenu::FormActivate entry");

  // this is actually pointed-to by the TApplication
  // drawers
  TForm *temp = Screen->ActiveForm;
  if(temp)
  {
    if (temp->InheritsFrom(__classid(TformSpecialGeneric)))
    {
      TformSpecialGeneric * active_form = (TformSpecialGeneric*)temp;
      active_form->OnActivate(NULL);
    }
  }

  if(debug_level[other_forms] >= 4 && message_router)
    message_router->SendMessageToAll("debug", "formPrometheusMenu::FormActivate exit");
}
//---------------------------------------------------------------------------
void TformPrometheusMenu::WriteLogs ()
{
  if(debug_level[other_forms]>=3 && message_router)
    message_router->SendMessageToAll("debug", "formPrometheusMenu::WriteLogs entry");

  TIBTransaction *dbCloseTransaction = new TIBTransaction(NULL);
  try
  {
   StdTransaction(dbCloseTransaction, dbDatabase);
   map<string, string> log_map;
   log_map["id"] = "1";
   log_map["user_id"] = current_userid;
   log_map["log_timestamp"] = "current_timestamp";
   log_map["computer_name"] = computer_name;
   log_map["is_log_in"] = "0";
   map<string, int> sql_options;
   sql_options["log_timestamp"] = sql_no_processing;
   ExecuteInsertStatement(dbCloseTransaction, "log_log", log_map, sql_options);
   message_router->SendMessageToAll(optionsIni["server"]["atlas_put"], "log_log.id|1");
   message_router->SendMessageToAll(optionsIni["server"]["atlas_put"], "log_log.user_id|" + current_userid);
   // atlas->Notify("log_log.users", current_userid);

   log_map.clear();
   log_map["event_date"] = CurrentDate();
   log_map["user_id"] = current_userid;
   for(map<string, int>::iterator itr = form_count_map.begin(); itr != form_count_map.end(); itr++)
   {
    log_map["id"] = "1";
    log_map["form_name"] = itr->first;
    log_map["event_name"] = "form opened";
    log_map["occurances"] = IntToString(itr->second); // yes, it's misspelled.  Ashley's fault.
    map<string, int> leave_alone;
    for (map<string, string>::iterator itr=log_map.begin(); itr!=log_map.end(); itr++)
      leave_alone[itr->first] = sql_no_uppercasing;
    ExecuteInsertStatement(dbCloseTransaction, "usage_log", log_map, leave_alone);
   }
   message_router->SendMessageToAll(optionsIni["server"]["atlas_put"], "usage_log.id|1");
   message_router->SendMessageToAll(optionsIni["server"]["atlas_put"], "usage_log.user_id|" + current_userid);
   dbCloseTransaction->Commit();
   if (!SavePreferences())
   {
    message_router->SendMessageToAll (optionsIni["server"]["error_put"], "Error saving preferences for user upon shutdown on " + computer_name);
    PSay("Error while saving user preferences.", m_database_error, MB_OK);
   }
  }
  catch(...)
  {
   if(dbCloseTransaction->Active)
    dbCloseTransaction->Rollback();
  }

  delete dbCloseTransaction;

  if(debug_level[other_forms] >= 3 && message_router)
    message_router->SendMessageToAll("debug", "formPrometheusMenu::WriteLogs exit");
}
//---------------------------------------------------------------------------
void TformPrometheusMenu::DebugWidth(string info)
{
 if(debug_level[other_forms]>=9 && message_router)
   message_router->SendMessageToAll("debug", "formPrometheusMenu::DebugWidth <" + info + ">");
 if(debug_level[other_forms]>=9 && message_router)
   message_router->SendMessageToAll("debug", "formPrometheusMenu::DebugWidth form MinWidth = " + IntToString(this->Constraints->MinWidth));
 if(debug_level[other_forms]>=9 && message_router)
   message_router->SendMessageToAll("debug", "formPrometheusMenu::DebugWidth form MaxWidth = " + IntToString(this->Constraints->MaxWidth));
 if(debug_level[other_forms]>=9 && message_router)
   message_router->SendMessageToAll("debug", "formPrometheusMenu::DebugWidth form Width = " + IntToString(this->Width));
 if(debug_level[other_forms]>=9 && message_router)
   message_router->SendMessageToAll("debug", "formPrometheusMenu::DebugWidth barMenu->Width = " + IntToString(menuFrame->GetControlBar()->Width));
 if(debug_level[other_forms]>=9 && message_router)
   message_router->SendMessageToAll("debug", "formPrometheusMenu::DebugWidth panelQuickMenus->Width = " + IntToString(panelQuickMenus->Width));
 if(debug_level[other_forms]>=9 && message_router)
   message_router->SendMessageToAll("debug", "formPrometheusMenu::DebugWidth </" + info + ">");
}
//---------------------------------------------------------------------------
void TformPrometheusMenu::ProcessIni()
{
  if(debug_level[other_forms] >= 4 && message_router)
    message_router->SendMessageToAll("debug", "formPrometheusMenu::ProcessIni entry");

  if (optionsIni["server"]["debug_level"] != "")
  {
    debug_level[other_forms]   = StringToInt(optionsIni["server"]["debug_level"].substr(0,1));
    debug_level[generic_forms] = StringToInt(optionsIni["server"]["debug_level"].substr(1,1));
    debug_level[libraries]     = StringToInt(optionsIni["server"]["debug_level"].substr(2,1));
  }

  if(debug_level[other_forms] >= 4 && message_router)
    message_router->SendMessageToAll("debug", "formPrometheusMenu::ProcessIni exit");
}
//---------------------------------------------------------------------------
// jhaley 20110118: Find the number of open QuickReport forms
int TformPrometheusMenu::NumReportsOpen()
{
  int count = 0;

  try
  {
    for(int i = 0; i < Screen->FormCount; i++)
    {
      if(AnsiString(Screen->Forms[i]->ClassName()) == "TQRStandardPreview")
        ++count;
    }
  }
  catch(...) // jhaley 20110617: Hmm... catch anything here.
  {
    LogIt("reportsclosing.log", "Exception thrown in TformPrometheusMenu::NumReportsOpen");
  }

  return count;
}
//---------------------------------------------------------------------------
// jhaley 20110121: Attempt to close all QuickReport Forms
void TformPrometheusMenu::CloseAllReports()
{
  for(int i = 0; i < Screen->FormCount; i++)
  {
    try
    {
      if(AnsiString(Screen->Forms[i]->ClassName()) == "TQRStandardPreview")
        Screen->Forms[i]->Close();
    }
    catch(...) // jhaley 20110617: Hmm... catch anything here.
    {
      LogIt("reportsclosing.log", "Exception thrown while closing TQRStandardPreview #" + IntToString(i));
    }
  }
}
//----------------------------------------------------------------------------------------------------------------------
// *** END CORE METHODS ************************************************************************************************
//----------------------------------------------------------------------------------------------------------------------
//----------------------------------------------------------------------------------------------------------------------
// *** TRAY ICON *******************************************************************************************************
//----------------------------------------------------------------------------------------------------------------------
// jhaley 20110415: Draw the tray icon
void __fastcall TformPrometheusMenu::DrawItem(TMessage &Msg)
{
  try
  {
    LPDRAWITEMSTRUCT lpdi = (LPDRAWITEMSTRUCT)(Msg.LParam);
    HICON hIcon;

    hIcon = (HICON)LoadImage(g_hinst, MAKEINTRESOURCE(lpdi->CtlID), IMAGE_ICON, 16, 16, 0);
    if(hIcon)
      DrawIconEx(lpdi->hDC, lpdi->rcItem.left, lpdi->rcItem.top, hIcon, 16, 16, 0, NULL, DI_NORMAL);

    TForm::Dispatch(&Msg);
  }
  catch(...) // jhaley 20110617: Hmm... catch anything here.
  {
    LogIt("systray.log", "Exception caught in TformPrometheusMenu::DrawItem");
  }
}
//----------------------------------------------------------------------------------------------------------------------
// jhaley 20110415: Tray icon notification message handler
void __fastcall TformPrometheusMenu::MyNotify(TMessage &Msg)
{
  try
  {
    POINT MousePos;

    switch(Msg.LParam)
    {
    case WM_RBUTTONUP:
      if(GetCursorPos(&MousePos))
      {
        // menu popup
        if(menuFrame)
        {
          TPopupMenu *menuPopup = menuFrame->GetTrayPopupMenu();
          menuPopup->PopupComponent = menuFrame;
          SetForegroundWindow(Handle);
          menuPopup->Popup(MousePos.x, MousePos.y);
        }
      }
      else
        Show();
      break;
    case WM_LBUTTONUP:
      // TODO: ???
      break;
    default:
      // TODO: ???
      break;
    }
    TForm::Dispatch(&Msg);
  }
  catch(...) // jhaley 20110617: Hmmm... catch anything here.
  {
    LogIt("systray.log", "Exception caught in TformPrometheusMenu::MyNotify");
  }
}
//----------------------------------------------------------------------------------------------------------------------
// jhaley 20110415: Tray messaging
bool __fastcall TformPrometheusMenu::TrayMessage(DWORD dwMessage)
{
  // 20110629: Do NOT accept any TrayMessage events if the form is being destroyed.
  if(PrometheusAppState == PROM_STATE_DESTROYING)
    return false;

  try
  {
    NOTIFYICONDATA tnd;
    string msg = string("Prometheus - Logged in as ") + PGlobals->current_username + "\nRight-click to access";

    tnd.cbSize = sizeof(NOTIFYICONDATA);
    tnd.hWnd   = Handle;
    tnd.uID    = IDC_MYICON;
    tnd.uFlags = NIF_MESSAGE | NIF_ICON | NIF_TIP;
    tnd.uCallbackMessage = MYWM_NOTIFY;

    if(dwMessage == NIM_MODIFY)
    {
      tnd.hIcon = Icon->Handle;
      lstrcpyn(tnd.szTip, msg.c_str(), sizeof(tnd.szTip));
    }
    else
    {
      tnd.hIcon = NULL;
      tnd.szTip[0] = '\0';
    }

    return Shell_NotifyIcon(dwMessage, &tnd);
  }
  catch(...) // jhaley 20110617: Hmmm... catch anything here.
  {
    LogIt("systray.log", "Exception caught in TformPrometheusMenu::TrayMessage");
    return false;
  }
}
//----------------------------------------------------------------------------------------------------------------------
// *** END TRAY ICON ***************************************************************************************************
//----------------------------------------------------------------------------------------------------------------------
//----------------------------------------------------------------------------------------------------------------------
// *** DEFERRED MESSAGE LISTENER ***************************************************************************************
//----------------------------------------------------------------------------------------------------------------------
// jhaley 20110119: classMainMenuListener virtual methods vvvvv
const char *classMainMenuListener::GetName()
{
  return "MainMenuListener";
}
//---------------------------------------------------------------------------
bool classMainMenuListener::CanAcceptMessage(classDeferredMessage &msg)
{
  // Cannot accept an event if the form is being destroyed
  if(PrometheusAppState == PROM_STATE_DESTROYING)
    return false;

  try
  {
    if (msg.GetReason() == "ReportsOpen")
    {
      if (msg.GetName() == "CloseAllForms" || msg.GetName() == "FormClose" || msg.GetName() == "SwitchUsers")
      {
        form->CloseAllReports(); // try to close all the report windows
        return (form->NumReportsOpen() == 0);
      }
    }
  }
  catch(...)
  {
    LogIt("menulistener.log", "Exception caught in classMainMenuListener::CanAcceptMessage");
  }

  return false; // unknown message, not interested
}
//---------------------------------------------------------------------------
void classMainMenuListener::AcceptMessage(classDeferredMessage &msg)
{
  // Cannot execute any more actions once the form is being destroyed.
  if(PrometheusAppState == PROM_STATE_DESTROYING)
    return;

  // The message was accepted, so execute it.
  try
  {
    if (msg.GetName() == "CloseAllForms")
    {
      form->CloseAllForms();
      form->menu_msg_queue->UnlockQueue();
    }
    else if (msg.GetName() == "FormClose")
    {
      PrometheusAppState = PROM_STATE_WANTCLOSE;
      form->Close();
    }
    else if (msg.GetName() == "SwitchUsers")
    {
      form->DoSwitchUsers();
      form->menu_msg_queue->UnlockQueue();
    }
  }
  catch(...)
  {
    LogIt("menulistener.log", "Exception caught in classMainMenuListener::AcceptMessage");
  }
}
//---------------------------------------------------------------------------
void classMainMenuListener::MessageResult(classDeferredMessage &msg, int &result)
{
  // Cannot execute any more actions once the form is being destroyed.
  if(PrometheusAppState == PROM_STATE_DESTROYING)
    return;

  try
  {
    // Some of the messages in this class should be executed even if the message fails or is canceled;
    // in the case this happens, things may blow up slightly - but that's nothing new.
    if (result == classDeferredMessage::RES_FAILED || result == classDeferredMessage::RES_CANCELED)
    {
      if (msg.GetName() == "CloseAllForms" || msg.GetName() == "FormClose" || msg.GetName() == "SwitchUsers")
        AcceptMessage(msg);
    }
  }
  catch(...)
  {
    LogIt("menulistener.log", "Exception caught in classMainMenuListener::MessageResult");
  }
}
//----------------------------------------------------------------------------------------------------------------------
// *** END DEFERRED MESSAGE LISTENER ***********************************************************************************
//----------------------------------------------------------------------------------------------------------------------

//---------------------------------------------------------------------------------------------------------------------
// *** MENU METHODS ***************************************************************************************************
//---------------------------------------------------------------------------------------------------------------------
// TODO -c ENCAPSULATION : Move to a superclass of TframePrometheusMenu?
void TformPrometheusMenu::RecurseMenu(TMainMenu *menu, int tagVal, bool boolVal)
{
  TMenuItem *menu_item;
  int total_items = menu->Items->Count;
  for(int i = 0; i < total_items; i++)
  {
    menu_item = menu->Items->Items[i];
    RecurseMenuItem(menu_item, tagVal, boolVal);
  }
}
//---------------------------------------------------------------------------
// TODO -c ENCAPSULATION : Move to a superclass of TframePrometheusMenu?
// sets all menu items and sub-menu items with a matching tag value to the specified Visible boolean value
void TformPrometheusMenu::RecurseMenuItem(TMenuItem *menu_item, int tagVal, bool boolVal)
{
  // we were passed a menu.  Look through all its menu items, setting Visible according to tag value
  int submenu_items = menu_item->Count;
  for(int j = 0; j < submenu_items; j++)
  {
    if(menu_item->Items[j]->Tag == tagVal)
    {
      menu_item->Items[j]->Visible = boolVal;
      //menu_item->Items[j]->Caption = (string(menu_item->Items[j]->Caption.c_str()) + "(" + IntToString(tagVal) + ")").c_str();
    }
    //menu_item->Items[j]->Caption += "diggity";
    // in addition, if this item is a submenu, we need to recursively call this function again with it for menu_item
    RecurseMenuItem(menu_item->Items[j], tagVal, boolVal);
  }
}
//---------------------------------------------------------------------------
void TformPrometheusMenu::EnableMenus(bool all_is_well)
{
  if(debug_level[other_forms] >= 4 && message_router)
    message_router->SendMessageToAll("debug", "TformPrometheusMenu::EnableMenus entry");

  // jhaley 20110126: Enable the active menu frame by calling its EnableMenus method.
  menuFrame->EnableMenus(all_is_well);

  if(debug_level[other_forms] >= 4 && message_router)
    message_router->SendMessageToAll("debug", "TformPrometheusMenu::EnableMenus exit");
}
//---------------------------------------------------------------------------
// TODO -c ENCAPSULATION : Move to a superclass of TframePrometheusMenu?
void TformPrometheusMenu::ShowUsefulFolders(TMenuItem *menu_item)
{
  // we were passed a menu.  Look through all its menu items, setting Visible according to tag value
  int submenu_items = menu_item->Count;
  bool any_visible = false;

  for(int j = 0; j < submenu_items; j++)
  {
    ShowUsefulFolders(menu_item->Items[j]);
    if(menu_item->Items[j]->Visible)
      any_visible = true;
  }

  if(any_visible)
    menu_item->Visible = true;
}
//----------------------------------------------------------------------------------------------------------------------
// *** END MENU METHODS ************************************************************************************************
//----------------------------------------------------------------------------------------------------------------------

//----------------------------------------------------------------------------------------------------------------------
// *** TIMERS **********************************************************************************************************
//----------------------------------------------------------------------------------------------------------------------
void __fastcall TformPrometheusMenu::timerCheckIdleTimer(TObject *Sender)
{
  //if(debug_level[other_forms]==9 && message_router)
  //  message_router->SendMessageToAll("debug", "formPrometheusMenu::timerCheckIdleTimer entry");

  // jhaley 20111227: How about we not bother with this if the application is doing anything other than running in the
  // normal fashion? Seems to occasionally cause a dramatic meltdown if it does run during the shutdown process for
  // whatever reason.
  if(PrometheusAppState != PROM_STATE_RUNNING)
    return;

  if(stuff_happened)
  {
    seconds_idle = 0;
    stuff_happened = false;
    if(is_idle)
    {
      message_router->SetValue("is_idle", "no");
      is_idle = false;
    }
  }
  else
    seconds_idle += timerCheckIdle->Interval / 1000;

  if(seconds_idle > 300 && !is_idle)
  {
    message_router->SetValue("is_idle", "yes");
    is_idle = true;
  }

  LASTINPUTINFO lii;
  lii.cbSize = sizeof(lii);
  DWORD idle_time = 0;
  if(GetLastInputInfo(&lii) != 0)
    idle_time = (GetTickCount() - lii.dwTime) / 1000;

  // jhaley - New desktop locking logic
  if(StringToInt(site_settings["disconnect_timeout_length"]) > 0 &&
     optionsPermissions["special.no_inactive_disconnect"]["enabled"] != "true" &&
     optionsIni["server"]["run_mode"] != "hercules" &&
     site_settings["disconnect_when_inactive"] == "1")
  {
    int time_limit = StringToInt(site_settings["disconnect_timeout_length"]);
    TformSpecialIdleWarning *idle_warn = NULL;

    // Ten seconds left to go?
    if((int)idle_time > time_limit - 10  /*seconds_idle > time_limit - 10*/)
    {
      // Look for an existing IdleWarning form:
      for(int fc = 0; fc < Screen->FormCount; fc++)
      {
        if(AnsiString(Screen->Forms[fc]->ClassName()) == "TformSpecialIdleWarning")
        {
          idle_warn = dynamic_cast<TformSpecialIdleWarning *>(Screen->Forms[fc]);
          break;
        }
      }

      if(idle_warn) // Already started a warning?
      {
        switch(idle_warn->GetLockState())
        {
        case TformSpecialIdleWarning::STATE_COUNTDOWN: // Already counting down.
        case TformSpecialIdleWarning::STATE_LOCKED:    // Already locked.
          break;
        case TformSpecialIdleWarning::STATE_WAITING:   // Restart the countdown.
          seconds_idle = 0;              // We've done our work, the dialog takes over now.
          idle_warn->SetTimerAmount(10); // 10 seconds til detonation!
          Application->Restore();        // Restore and bring to fjhaleront.
          idle_warn->BringToFront();
          message_router->SendMessageToAll(optionsIni["server"]["admin_put"],
                                           current_fullname + " is being locked out due to inactivity");
          break;
        }
      }
      else
      {
        // Create a new form.
        FORM_TRY
          idle_warn = new TformSpecialIdleWarning(NULL);
        FORM_CATCH_RETURN("TformSpecialIdleWarning", )
        if(idle_warn)
        {
          seconds_idle = 0;              // We've done our work, the dialog takes over now.
          idle_warn->SetTimerAmount(10); // 10 seconds til detonation!
          Application->Restore();        // Restore, show, and bring to front.
          idle_warn->Show();
          idle_warn->BringToFront();                                   // jhaley 20110816: Fix to lock-up problem:
          idle_warn->Left = min(Left, Screen->Width / 4);              // Position near TformPrometheusMenu, if possible,
          idle_warn->Top  = min(Top + Height + 8, Screen->Height / 4); // but NOT outside the first quarter of the screen!
          message_router->SendMessageToAll(optionsIni["server"]["admin_put"],
                                           current_fullname + " is being locked out due to inactivity");
        }
      }
    }
  }

  // Old code that logs user out, preserved for historical interest and/or future use:
#if 0
  if(StringToInt(site_settings["disconnect_timeout_length"]) > 0 &&
     optionsPermissions["special.no_inactive_disconnect"]["enabled"] != "true" &&
     optionsIni["server"]["run_mode"] != "hercules" &&
     seconds_idle > StringToInt(site_settings["disconnect_timeout_length"]) &&
     site_settings["disconnect_when_inactive"] == "1") // DONE -c MAGIC_NUMBER : put in database?
  {
    // jhaley 20110124: If reports are open, try to close all reports, and defer activation til the next timer event
    if(NumReportsOpen() > 0)
    {
      CloseAllReports();
      return;
    }

    message_router->SendMessageToAll(optionsIni["server"]["admin_put"],
                                     current_fullname + " is being logged off due to inactivity");
    shutdown_due_to_inactivity = true;
    timerDie->Enabled = true;
  }
#endif

  //if(debug_level[other_forms]==9 && message_router)
  //  message_router->SendMessageToAll("debug", "formPrometheusMenu::timerCheckIdleTimer exit");
}
//---------------------------------------------------------------------------
void __fastcall TformPrometheusMenu::timerNetworkRefreshTimer(TObject *Sender)
{
  //if(debug_level[other_forms]==9 && message_router)
  //  message_router->SendMessageToAll("debug", "formPrometheusMenu::timerNetworkRefreshTimer entry");

  timerNetworkRefresh->Enabled = false;
  if(atlas)
  {
    if(atlas->IsReady())
      atlas->Check();
  }

  if(!dbDatabase->TestConnected())
  {
    message_router->SetValue("db_connection", "no");
    optionsPermissions.clear();
    EnableMenus(false);
    timerNetworkRefresh->Enabled = false;
    Application->MessageBox("Your connection to the database has been lost.  Prometheus will now exit.",
                            "Database error", MB_OK);
    CloseAllReports(); // jhaley 20110124: Get all reports out of the way immediately when this happens.
    timerDie->Enabled = true;
    if(debug_level[other_forms] >= 1 && message_router)
      message_router->SendMessageToAll("debug", "Connection to the database has been lost on " + computer_name);
  }
  else
    timerNetworkRefresh->Enabled = true;

  //if(debug_level[other_forms]==9 && message_router)
  //  message_router->SendMessageToAll("debug", "formPrometheusMenu::timerNetworkRefreshTimer exit");
}
//---------------------------------------------------------------------------
void __fastcall TformPrometheusMenu::timerAdminTimer(TObject *Sender)
{
  timerAdmin->Enabled = false;
  // administrative whatnot
  string message = "";
  while (message_router->IsMessageWaiting(optionsIni["server"]["admin_get"]))
  {
    message = message_router->GetNextMessage(optionsIni["server"]["admin_get"]);
    if (message == "")
      ;
    else if (LikeString("run:%", LowercaseString(message)))
    {
     string p_run = message.substr(4);
     vector<string> run_vec;
     SplitStringToVec(p_run, "~", run_vec);
     //AnsiString x = p_run.c_str();
     LPCTSTR lpFile       = (run_vec.size()>0) ? static_cast<const char*>(AnsiString(run_vec[0].c_str()).data()) : NULL;
     LPCTSTR lpParameters = (run_vec.size()>1) ? static_cast<const char*>(AnsiString(run_vec[1].c_str()).data()) : NULL;
     LPCTSTR lpDirectory  = (run_vec.size()>2) ? static_cast<const char*>(AnsiString(run_vec[2].c_str()).data()) : NULL;
     int returnvalue = (int)ShellExecute(NULL, "open", lpFile, lpParameters, lpDirectory, SW_SHOW);
     if (returnvalue > 32)
       message_router->SendMessageToAll (optionsIni["server"]["admin_put"], "run:" + p_run + " acknowledged by user (" + optionsIni["server"]["user_name"] + ")");
     else
     {
       string error;
       switch (returnvalue)
       {
         case 0                      : error = "The operating system is out of memory or resources"; break;
         case ERROR_FILE_NOT_FOUND   : error = "The specified file was not found"; break;
         case ERROR_PATH_NOT_FOUND   : error = "The specified path was not found"; break;
         case ERROR_BAD_FORMAT       : error = "The .EXE file is invalid (non-Win32 .EXE or error in .EXE image)"; break;
         case SE_ERR_ACCESSDENIED    : error = "The operating system denied access to the specified file"; break;
         case SE_ERR_ASSOCINCOMPLETE : error = "The filename association is incomplete or invalid"; break;
         case SE_ERR_DDEBUSY	     : error = "The DDE transaction could not be completed because other DDE transactions were being processed"; break;
         case SE_ERR_DDEFAIL	     : error = "The DDE transaction failed"; break;
         case SE_ERR_DDETIMEOUT	     : error = "The DDE transaction could not be completed because the request timed out"; break;
         case SE_ERR_DLLNOTFOUND     : error = "The specified dynamic-link library was not found"; break;
         case SE_ERR_NOASSOC	     : error = "There is no application associated with the given filename extension"; break;
         case SE_ERR_OOM	     : error = "There was not enough memory to complete the operation"; break;
         case SE_ERR_SHARE	     : error = "A sharing violation occurred"; break;
         default                     : error = "Unknown error"; break;
       }
       message_router->SendMessageToAll (optionsIni["server"]["admin_put"], "run:" + p_run + " failed: " + error + " for user " + optionsIni["server"]["user_name"]);
     }
    }
    else if (LikeString("cmd:%", LowercaseString(message)))
    {
     string p_cmd = LowercaseString(message.substr(4));
     if (p_cmd == "ping")
     {
      message_router->SendMessageToAll (optionsIni["server"]["admin_put"], "cmd:ping acknowledged by user (" + optionsIni["server"]["user_name"] + ")");
     }
     if (p_cmd == "save")
     {
      if (SaveOptionsToFile ("prometheus.ini", optionsIni))
       message_router->SendMessageToAll (optionsIni["server"]["admin_put"], "cmd:save succeeded with user (" + optionsIni["server"]["user_name"] + ")");
      else
       message_router->SendMessageToAll (optionsIni["server"]["admin_put"], "cmd:save failed with user (" + optionsIni["server"]["user_name"] + ")");
     }
     if (p_cmd == "die" || p_cmd == "close" || p_cmd == "exit" || p_cmd == "quit")
     {
      message_router->SendMessageToAll (optionsIni["server"]["admin_put"], "cmd:die/close/exit/quit acknowledged by user (" + optionsIni["server"]["user_name"] + ")");
      // PSay("Chaotic shutdown in progress, my apologies!", m_warning, MB_OK);
      CloseAllReports(); // jhaley 20110124: Close all reports immediately in this case.
      timerAdmin->Enabled = false;
      timerDie->Enabled = true;
     }
     if(p_cmd == "closemessaging")
     {
       sockClientSocket->OnError = NULL;
       sockClientSocket->OnDisconnect = NULL;
       timerReconnect->Enabled = false;
     }
     if (p_cmd == "diagnose")
     {
      for (map<string, map<string, string> >::iterator i = optionsIni.begin(); i != optionsIni.end(); i++)
      {
       string section = i->first;
       message_router->SendMessageToAll (optionsIni["server"]["admin_put"], "[" + section + "]");
       for (map<string, string>::iterator j = i->second.begin(); j != i->second.end(); j++)
       {
        string key = j->first;
        string value = j->second;
        message_router->SendMessageToAll (optionsIni["server"]["admin_put"], key + "=" + value);
       }
      }
     }
    }
    else if (LikeString("get:%", LowercaseString(message)))
    {
     string p_get = message.substr(4);
     string section = p_get.substr(0, p_get.find_first_of("."));
     string key = p_get.substr(p_get.find_first_of(".") + 1);
     string value = optionsIni[section][key];
     message_router->SendMessageToAll (optionsIni["server"]["admin_put"], "get: " + section + "." + key + ":" + value);
    }
    else if (LikeString("say:%", LowercaseString(message)))
    {
     string p_say = message.substr(4);
     PSay(message.substr(4), m_message, MB_OK);
     message_router->SendMessageToAll (optionsIni["server"]["admin_put"], "say:... acknowledged by user (" + optionsIni["server"]["user_name"] + ")");
    }
    else if (LikeString("ack:%", LowercaseString(message)))
    {
     string p_ack = message.substr(4);
     message_router->SendMessageToAll (optionsIni["server"]["admin_put"], "ack:... received by user (" + optionsIni["server"]["user_name"] + ")");
     PSay(message.substr(4), m_message, MB_OK);
     message_router->SendMessageToAll (optionsIni["server"]["admin_put"], "ack:... acknowledged by user (" + optionsIni["server"]["user_name"] + ")");
    }
    else if (LikeString("set:%", LowercaseString(message)))
    {
     string p_get = message.substr(4);
     string section = p_get.substr(0, p_get.find_first_of("."));
     string keyvalue = p_get.substr(p_get.find_first_of(".") + 1);
     string key = keyvalue.substr(0, keyvalue.find_first_of("="));
     string value = keyvalue.substr(keyvalue.find_first_of("=") + 1);
     optionsIni[section][key] = value;
     message_router->SendMessageToAll (optionsIni["server"]["admin_put"], "set: " + section + "." + key + ":" + value + " for user (" + optionsIni["server"]["user_name"] + ")");
     ProcessIni();
    }
  /*  else if (LikeString("mb:%", LowercaseString(message)))
    {
     message_router->SendMessageToAll (optionsIni["server"]["admin_put"], "mb:... received by user (" + optionsIni["server"]["user_name"] + ")");
     string p_params = message.substr(3);
     vector<string> param_vec;
     SplitStringToVec(p_params, ",", param_vec);
     if (param_vec.size() < 3)
      message_router->SendMessageToAll (optionsIni["server"]["admin_put"], "mb:... wrong number of parameters received by user (" + optionsIni["server"]["user_name"] + ")");
     else
     {
      //param_vec[0] is the type of messagebox, param_vec[1] is the title bar string, and param_vec[2] is the dialog message
      param_vec[0] = LowercaseString(param_vec[0]);
      int result = 0;
      string query_response;
      string complete_message = "";
      for (unsigned int i = 2; i < param_vec.size(); i++)
         complete_message = complete_message + param_vec[i] + ",";
      complete_message = complete_message.substr(0, complete_message.length()-1);
      if ((param_vec[0] == "o") || (param_vec[0] == "ok") || (param_vec[0] == ""))
       result = PrometheusMessage(complete_message.c_str(), param_vec[1].c_str(), mb_ok);
      else if ((param_vec[0] == "oc") || (param_vec[0] == "okcancel"))
       result = PrometheusMessage(complete_message.c_str(), param_vec[1].c_str(), mb_okcancel);
      else if ((param_vec[0] == "rc") || (param_vec[0] == "retrycancel"))
       result = PrometheusMessage(complete_message.c_str(), param_vec[1].c_str(), mb_retrycancel);
      else if ((param_vec[0] == "yn") || (param_vec[0] == "yesno"))
       result = PrometheusMessage(complete_message.c_str(), param_vec[1].c_str(), mb_yesno);
      else if ((param_vec[0] == "ync") || (param_vec[0] == "yesnocancel"))
       result = PrometheusMessage(complete_message.c_str(), param_vec[1].c_str(), mb_yesnocancel);
      else if ((param_vec[0] == "ari") || (param_vec[0] == "abortretryignore"))
       result = PrometheusMessage(complete_message.c_str(), param_vec[1].c_str(), mb_abortretryignore);
      else if ((param_vec[0] == "q") || (param_vec[0] == "query"))
       result = PrometheusMessage(complete_message.c_str(), param_vec[1].c_str(), mb_ok | mb_query, &query_response);
      else
       message_router->SendMessageToAll (optionsIni["server"]["admin_put"], "mb:... invalid type (" + param_vec[0] + ") received by user (" + optionsIni["server"]["user_name"] + ")");
      // now say what the result was
      if (result > 0)
      {
       if (result == idok)
        message_router->SendMessageToAll (optionsIni["server"]["admin_put"], "mb:... received 'OK' from user (" + optionsIni["server"]["user_name"] + ")");
       else if (result == idcancel)
        message_router->SendMessageToAll (optionsIni["server"]["admin_put"], "mb:... received 'CANCEL' from user (" + optionsIni["server"]["user_name"] + ")");
       else if (result == idabort)
        message_router->SendMessageToAll (optionsIni["server"]["admin_put"], "mb:... received 'ABORT' from user (" + optionsIni["server"]["user_name"] + ")");
       else if (result == idretry)
        message_router->SendMessageToAll (optionsIni["server"]["admin_put"], "mb:... received 'RETRY' from user (" + optionsIni["server"]["user_name"] + ")");
       else if (result == idignore)
        message_router->SendMessageToAll (optionsIni["server"]["admin_put"], "mb:... received 'IGNORE' from user (" + optionsIni["server"]["user_name"] + ")");
       else if (result == idyes)
        message_router->SendMessageToAll (optionsIni["server"]["admin_put"], "mb:... received 'YES' from user (" + optionsIni["server"]["user_name"] + ")");
       else if (result == idno)
        message_router->SendMessageToAll (optionsIni["server"]["admin_put"], "mb:... received 'NO' from user (" + optionsIni["server"]["user_name"] + ")");
       else
        message_router->SendMessageToAll (optionsIni["server"]["admin_put"], "mb:... received unknown response from user (" + optionsIni["server"]["user_name"] + ")");

       if ((param_vec[0] == "q") || (param_vec[0] == "query"))
        message_router->SendMessageToAll (optionsIni["server"]["admin_put"], "mb:... query response from user (" + optionsIni["server"]["user_name"] + "): '" + query_response + "'");
      }
     }
    } */
    else message_router->SendMessageToAll (optionsIni["server"]["admin_put"], "???: unknown command to user (" + optionsIni["server"]["user_name"] + ")");
  }

  timerAdmin->Enabled = true;
}
//---------------------------------------------------------------------------
void __fastcall TformPrometheusMenu::timerReplaceOldTimer(TObject *Sender)
{
  replace_attempts += 1;
  if((replace_attempts <=  10 && replace_attempts %  5 == 0) ||
     (replace_attempts <= 100 && replace_attempts % 25 == 0) ||
     (replace_attempts % 100 == 0))
  {
    message_router->SendMessageToAll(optionsIni["server"]["admin_put"],
      "Attempting to replace current version with downloaded version (now for "
      + IntToString(replace_attempts) + " times)");
  }

  // Copy over old file with temp file
  if(CopyFile(optionsIni["server"]["temp_version_file"].c_str(), optionsIni["server"]["perm_version_file"].c_str(), 0))
  {
    timerReplaceOld->Enabled = false;
    optionsIni["server"]["copy_over_old"] = "false";
    SaveOptionsToFile ("prometheus.ini", optionsIni);
    message_router->SendMessageToAll(optionsIni["server"]["admin_put"], "file replaced, stopping timer");
  }
  else
  {
    if ((replace_attempts <=  10 && replace_attempts %  5 == 0) ||
        (replace_attempts <= 100 && replace_attempts % 25 == 0) ||
        (replace_attempts % 100 == 0))
    {
      message_router->SendMessageToAll(optionsIni["server"]["admin_put"],
        "Failed to replace file on " + computer_name + " " + IntToString(replace_attempts) + " times");
    }
  }
}
//---------------------------------------------------------------------------
void __fastcall TformPrometheusMenu::timerDieTimer(TObject *Sender)
{
  // jhaley 20111227: some insurance on order of execution
  switch(PrometheusAppState)
  {
  case PROM_STATE_WANTCLOSE:
  case PROM_STATE_FORMCLOSE:
  case PROM_STATE_DESTROYING:
    return; // we are already shutting down. Do not shutdown again.
  default:
    break;
  }

  // close application
  Application->OnException = HandleClosingExceptions;
  AV_TRY
    timerDie->Enabled = false;
  AV_CATCH
  AV_TRY
    message_router->SendMessageToAll (optionsIni["server"]["admin_put"], "Beginning timerDieTimer on " + computer_name);
  AV_CATCH
  AV_TRY
    CloseAllForms();
  AV_CATCH
  AV_TRY
    message_router->SendMessageToAll (optionsIni["server"]["admin_put"], "Ending timerDieTimer on " + computer_name);
  AV_CATCH

  PrometheusAppState = PROM_STATE_WANTCLOSE;
  Close();
}
//---------------------------------------------------------------------------
void __fastcall TformPrometheusMenu::timerAppointmentsTimer(TObject *Sender)
{
  if (optionsPreferences["appointments"]["reminders"] == "true")
  {
    PApptOccurrenceMap all_appointments;
    vector<string> user_ids;
    user_ids.push_back(current_userid);
    map<string, vector<string> > ids_to_match;
    ids_to_match["visit_outcome_id"].push_back("0");
    ids_to_match["visit_outcome_id"].push_back(lookupTables->IdOf("visit_outcome_types", "As Scheduled"));
    ids_to_match["visit_outcome_id"].push_back(lookupTables->IdOf("visit_outcome_types", "Walk-In"));
    ids_to_match["visit_outcome_id"].push_back(lookupTables->IdOf("visit_outcome_types", "Arrived Late"));
    ids_to_match["current_id"].push_back("3");
    ids_to_match["current_id"].push_back("4");
    ids_to_match["current_id"].push_back("5");
    DownloadAppointments(dbDatabase, CurrentPdate(), CurrentPdate(), current_userid, user_ids, vector<string>(), "",
                         all_appointments, ids_to_match, (optionsPermissions["admin.all_appointments"]["enabled"] == "true"),
                         false, "");
    PApptOccurrenceMap::iterator i_day = all_appointments.find(CurrentPdate());
    if(i_day != all_appointments.end())
    {
      int difference;
      Ptime current_time = CurrentPtime();
      for(PApptOccurrenceSet::iterator itr = i_day->second.begin(); itr != i_day->second.end(); itr++)
      {
        difference = itr->range_begin - current_time;
        if ((difference <= 900) && (difference > 0))
        {
          int new_top;
          int new_left;
          bool can_show = true;
          if (reminder_forms.empty())
          {
            new_top = 100;
            new_left = 100;
          }
          else
          {
            list<TformSpecialReminder *>::iterator form_itr = reminder_forms.end();
            form_itr--;
            if ((*form_itr)->Top >= 400)
            {
              new_top = 100;
              if ((*form_itr)->Left == 400)
                new_left = 150;
              else
                can_show = false;
            }
            else
            {
              new_top = (*form_itr)->Top + 50;
              new_left = (*form_itr)->Left + 50;
            }
          }
          if (can_show)
          {
            string to_display;
            if(itr->person != "")
              to_display = PrettyString(itr->person) + ": ";
            to_display += SentenceString(itr->description);
            FORM_TRY
              TformSpecialReminder * reminder_form = new TformSpecialReminder(NULL, itr->range_begin.OutTime(ampm | withoutsec),
                                                     to_display);
              reminder_form->Top = new_top;
              reminder_form->Left = new_left;
              reminder_forms.push_back(reminder_form);
              reminder_form->Show();
            FORM_CATCH
          }
        } // if difference <= 15 minutes
      } // end for
    }
  }// end if
}
//----------------------------------------------------------------------------------------------------------------------
// *** END TIMERS ******************************************************************************************************
//----------------------------------------------------------------------------------------------------------------------

//----------------------------------------------------------------------------------------------------------------------
// *** EXTERNAL GENERIC FORM HANDLERS **********************************************************************************
//----------------------------------------------------------------------------------------------------------------------
// For TformSpecialGeneric:
string __fastcall TformPrometheusMenu::GetDBOverrideSection()
{
  if(do_database_prompt && db_override_section != "server")
    return db_override_section;
  else
    return "";
}
//---------------------------------------------------------------------------
// For TformRecordGeneric:
void __fastcall TformPrometheusMenu::menuCurrentTypesChoice(TObject *Sender)
{
  // the tag of the component that calls this needs to have a pointer in it to
  // a TformRecordGeneric... we'll assume that this popup will only be called
  // by toolbars (most probably) at the bottom of the screen on TformRecordGeneric
  // screens... and this calls a function in that class to inform it of the value.

  // popupcomponent on a popup points to the object that made it appear
  TformRecordGeneric *to_inform  = (TformRecordGeneric *)(GpopupCurrentTypes->PopupComponent->Tag);
  TMenuItem          *is_clicked = (TMenuItem *)(Sender);
  string to_send_data = is_clicked->Caption.c_str();
  string to_send_id   = lookupTables->IdOf("current_types", to_send_data);

  if(to_inform)
    to_inform->PopupMessage("current_types", to_send_id, to_send_data);
  else
    PSay("Could not inform calling screen of the selected value", m_program_error, MB_OK);
}
//----------------------------------------------------------------------------------------------------------------------
// *** END EXTERNAL GENERIC FORM HANDLERS ******************************************************************************
//----------------------------------------------------------------------------------------------------------------------

//---------------------------------------------------------------------------------------------------------------------
// *** LOOKUP TABLE METHODS *******************************************************************************************
//---------------------------------------------------------------------------------------------------------------------
void TformPrometheusMenu::AtlasLookupTables(int request_number, string matched_value)
{
  if(debug_level[other_forms] >= 4 && message_router)
    message_router->SendMessageToAll("debug", "TformPrometheusMenu::AtlasLookupTables entry");

  string to_refresh = lookupAtlasTickets[request_number];
  if(!to_refresh.empty())
    lookupTables->ExpectRefresh(to_refresh); // RefreshAllBoxes(to_refresh);
  else
    PSay("Could not figure out which lookup table to refresh", m_program_error, MB_OK);

  if(debug_level[other_forms] >= 4 && message_router)
    message_router->SendMessageToAll("debug", "TformPrometheusMenu::AtlasLookupTables exit");
}
//---------------------------------------------------------------------------
void TformPrometheusMenu::GetValueHandler(int request_number, string matched_value)
{
  if(debug_level[other_forms] >= 4 && message_router)
    message_router->SendMessageToAll("debug", "TformPrometheusMenu::GetValueHandler entry");

  if(request_number == ip_request)
  {
    timerGetValueTimeout->Enabled = false;
    my_ip = matched_value;
  }

  if(debug_level[other_forms] >= 4 && message_router)
    message_router->SendMessageToAll("debug", "TformPrometheusMenu::GetValueHandler exit");
}
//---------------------------------------------------------------------------
void __fastcall TformPrometheusMenu::GpopupLookupTablesPopup(TObject *Sender)
{
  TComboBox * box_ptr = (TComboBox*)GpopupLookupTables->PopupComponent;

  // defaults
  menuAddValue->Enabled = false;

  if(lookupTables->CanAddToTable(lookupTables->TableName(box_ptr)))
    menuAddValue->Enabled = true;
}
//---------------------------------------------------------------------------
void __fastcall TformPrometheusMenu::menuAddValueClick(TObject *Sender)
{
  TComboBox * box_ptr = (TComboBox*)GpopupLookupTables->PopupComponent;
  map<string, string> temp_map;
  temp_map["table_name"] = lookupTables->TableName(box_ptr);
  temp_map["unsafe_add"] = "false";

  FORM_TRY
    TformSpecialAddValues * addValueForm = new TformSpecialAddValues(NULL, temp_map, box_ptr);
    addValueForm->Show();
  FORM_CATCH
}
//---------------------------------------------------------------------------
void __fastcall TformPrometheusMenu::timerGetValueTimeoutTimer(TObject *Sender)
{
  timerGetValueTimeout->Enabled = false;
  get_value_timed_out = true;
}
//----------------------------------------------------------------------------------------------------------------------
// *** END LOOKUP TABLE METHODS ****************************************************************************************
//----------------------------------------------------------------------------------------------------------------------

//----------------------------------------------------------------------------------------------------------------------
// *** CHAT ************************************************************************************************************
//----------------------------------------------------------------------------------------------------------------------
void TformPrometheusMenu::HandleChatMessages(string channel)
{
  if(debug_level[other_forms] >= 4 && message_router)
    message_router->SendMessageToAll("debug", "TformPrometheusMenu::HandleChatMessages called");

  if((optionsPermissions["special.chat"]["enabled"] == "true") &&
     (optionsPreferences["chat"]["do_not_receive_messages"] != "true"))
  {
    if(chat_window)
    {
      if((chat_window != Screen->ActiveForm) || (! Application->Active))
      {
        timerChatFlash->Enabled = true;
        chat_window->SetFlashTimer(timerChatFlash);
      }
    }
    else
    {
      FORM_TRY
        TformSpecialChat *chat_form = new TformSpecialChat(NULL);
        chat_window = chat_form;
        chat_form->Show();
        if((chat_window != Screen->ActiveForm) || (! Application->Active))
        {
          timerChatFlash->Enabled = true;
          chat_window->SetFlashTimer(timerChatFlash);
        }
      FORM_CATCH
    }
  }

  // TODO: if the user can't chat for some reason, send back a message saying so? "Johnny can't come out to play today."
}
//---------------------------------------------------------------------------
void __fastcall TformPrometheusMenu::timerChatFlashTimer(TObject *Sender)
{
  FlashWindow(chat_window->Handle, true);
}
//----------------------------------------------------------------------------------------------------------------------
// *** END CHAT ********************************************************************************************************
//----------------------------------------------------------------------------------------------------------------------

//----------------------------------------------------------------------------------------------------------------------
// *** LINKS ***********************************************************************************************************
//----------------------------------------------------------------------------------------------------------------------
void __fastcall TformPrometheusMenu::OnLinkClick(TObject *Sender)
{
  string id         = registered_links[(TStaticText *)Sender].id;
  string table_name = registered_links[(TStaticText *)Sender].table_name;

  // added because, well, it's stupid to click on links we should know to be useless.
  if(!IsBlankOrZero(id) && IsInt(id))
    DefaultAction(id, table_name);
}
//--------------------------------------------------------------------------
void __fastcall TformPrometheusMenu::OnLinkMenuItemClick(TObject *Sender)
{
  // TODO: if ! inherits from TMenuItem, return?  shouldn't be here?
  TMenuItem* M = (TMenuItem *)Sender;
  if (registered_link_buttons.find(M) != registered_link_buttons.end())
  {
    TStaticText* S         = registered_link_buttons[M];
    string new_id          = IntToString(M->Tag);
    registered_links[S].id = new_id;
    string table_name      = registered_links[S].table_name;
    string permission_name = registered_links[S].permission_name;
    int options            = registered_links[S].options;

    S->Caption = DefaultText(new_id, table_name, permission_name, options).c_str();
    if(!IsBlankOrZero(new_id) && IsInt(new_id) && !S->OnClick)
      S->OnClick = funcLinkClick;
  }
}
//----------------------------------------------------------------------------------------------------------------------
// *** END LINKS *******************************************************************************************************
//----------------------------------------------------------------------------------------------------------------------

//----------------------------------------------------------------------------------------------------------------------
// *** COMBO BOXES *****************************************************************************************************
//----------------------------------------------------------------------------------------------------------------------
void __fastcall TformPrometheusMenu::OnComboKeyPress(TObject *Sender, char &Key)
{
  FindComboBoxItem(Sender, Key);
}
//----------------------------------------------------------------------------------------------------------------------
// *** END COMBO BOXES *************************************************************************************************
//----------------------------------------------------------------------------------------------------------------------

//----------------------------------------------------------------------------------------------------------------------
// *** SOCKET HANDLERS *************************************************************************************************
//----------------------------------------------------------------------------------------------------------------------
void __fastcall TformPrometheusMenu::sockClientSocketRead(TObject *Sender, TCustomWinSocket *Socket)
{
  message_router->AttemptRead();
}
//---------------------------------------------------------------------------
void __fastcall TformPrometheusMenu::sockClientSocketWrite(TObject *Sender, TCustomWinSocket *Socket)
{
  message_router->AttemptWrite();
  sockClientSocket->OnWrite = sockClientSocketWrite;
}
//---------------------------------------------------------------------------
void __fastcall TformPrometheusMenu::sockClientSocketConnect(TObject *Sender, TCustomWinSocket *Socket)
{
 if(debug_level[other_forms]>=3 && message_router)
   message_router->SendMessageToAll("debug", "formPrometheusMenu::sockClientSocketConnect entry");

 try
 {
  connect_successful = 1;
  sockClientSocket->OnRead = sockClientSocketRead;
  sockClientSocket->OnWrite = sockClientSocketWrite;

  // added here rather than below
  timerReconnect->Enabled = false;

  message_router->SetValue("computer_name", computer_name);
  message_router->SetValue("user_name", PGlobals->current_username);
  message_router->SetValue("full_name", current_fullname);
  message_router->SetValue("database", optionsIni["server"]["server_name"] + ":" + optionsIni["server"]["server_file"]);
  message_router->SetValue("version_number", PGlobals->current_version);
  AV_TRY
    message_router->SetValue("gds_version", PGlobals->gds_version);
  AV_CATCH
  message_router->SetValue("client_type", "prometheus");
  message_router->SendMessageToAll(optionsIni["server"]["other_put"],
                                   PGlobals->current_username + " has reconnected to the message server");
 }
 catch(ESocketError & the_error)
 {
 }

 if(debug_level[other_forms]>=3 && message_router)
   message_router->SendMessageToAll("debug", "formPrometheusMenu::sockClientSocketConnect exit");
}
//---------------------------------------------------------------------------
void __fastcall TformPrometheusMenu::sockClientSocketError(TObject *Sender,
      TCustomWinSocket *Socket, TErrorEvent ErrorEvent, int &ErrorCode)
{
  // the following causes a recursive cascade of errors, results in a stack overflow, and kills P.
  // most likely cause during testing: connection to dispatcher was (temporarily?) down. (philip-to-kronos)
  /*
  if(debug_level[other_forms]>=3)
    message_router->SendMessageToAll("debug", "formPrometheusMenu::sockClientSocketError called");
  */
  // also called when the -other- side disconnects, or something inbetween breaks.
  connect_successful = 2;
  ErrorCode = 0;
  timerReconnect->Enabled = true;
}
//---------------------------------------------------------------------------
void __fastcall TformPrometheusMenu::sockClientSocketDisconnect(TObject *Sender, TCustomWinSocket *Socket)
{
  if(debug_level[other_forms]>=3 && message_router)
    message_router->SendMessageToAll("debug", "formPrometheusMenu::sockClientSocketDisconnect called");
  sockClientSocket->OnRead = NULL;
  sockClientSocket->OnWrite = NULL;
  timerReconnect->Enabled = true;
}
//---------------------------------------------------------------------------
void __fastcall TformPrometheusMenu::timerReconnectTimer(TObject *Sender)
{
  if(sockClientSocket->Active)
    sockClientSocket->Close();
  connect_successful = 0;
  sockClientSocket->Open();

  // trying to make 100% cpu usage go away when connection is taking a while ...
  /*
  while(connect_successful == 0)
   Application->ProcessMessages();
  if(connect_successful == 1)
  {
    timerReconnect->Enabled = false;

    message_router->SetValue("computer_name", computer_name);
    message_router->SetValue("user_name", PGlobals->current_username);
    message_router->SetValue("full_name", current_fullname);
    message_router->SetValue("database", optionsIni["server"]["server_name"] + ":" + optionsIni["server"]["server_file"]);
    message_router->SetValue("version_number", PGlobals->current_version);
    message_router->SetValue("client_type", "prometheus");
    message_router->SendMessageToAll (optionsIni["server"]["other_put"], PGlobals->current_username + " has reconnected to the message server");
  }
  */
}
//----------------------------------------------------------------------------------------------------------------------
// *** END SOCKET HANDLERS *********************************************************************************************
//----------------------------------------------------------------------------------------------------------------------

//----------------------------------------------------------------------------------------------------------------------
// *** DATABASE EVENT HANDLERS *****************************************************************************************
//----------------------------------------------------------------------------------------------------------------------
void __fastcall TformPrometheusMenu::GdbDatabaseAfterDisconnect(TObject *Sender)
{
  /* TODO : need to code for this to set value for messaging server and try to reconnect
     oh yeah, and to do all this gracefully without access violations everywhere */
}
//---------------------------------------------------------------------------
void __fastcall TformPrometheusMenu::GdbDatabaseAfterConnect(TObject *Sender)
{
  message_router->SetValue("db_connection", "yes");
}
//---------------------------------------------------------------------------
void __fastcall TformPrometheusMenu::GdbDatabaseBeforeConnect(TObject *Sender)
{
//
}
//---------------------------------------------------------------------------
void __fastcall TformPrometheusMenu::GdbDatabaseBeforeDisconnect(TObject *Sender)
{
//
}
//---------------------------------------------------------------------------
void __fastcall TformPrometheusMenu::GdbMonitorSQL(AnsiString EventText, TDateTime EventTime)
{
  // monitoring code.
  /*
  \r\n[Application: Prometheus]\r\n: [Fetch] <sql statement>\r\n\r\n  SEOFReached
  \r\n[Application: Prometheus]\r\n: [Fetch] <sql statement>\r\n
  \r\n[Application: Prometheus]\r\n: [Execute] <sql statement>\r\n

  \r\n[Application: Prometheus]\r\n: [Fetch] < ...
  ...
  ...\r\n

  \r\n[Application: Prometheus]\r\n: [Fetch] < ...
  ...
  ...\r\n\r\n  SEOFReached
  */

  vector<string> event_message_parts;
  string         sql           = "";
  bool           seofreached   = false;
  bool           whole_message = false;
  // string monitored_sql;
  // monitor_actions monitored_action;

  SplitStringToVec(EventText.c_str(), "\r\n", event_message_parts);

  if(event_message_parts.empty())
  {
    // message_router->SendMessageToAll("debug", string("Empty: ") + EventText.c_str());
    return;
  }

  if(event_message_parts.size() > 1 && event_message_parts[0] == "" && event_message_parts[1] == "[Application: Prometheus]")
  {
    monitored_sql = "";
    monitored_action = monitor_unknown;

   if(event_message_parts.size() > 2)
    if(event_message_parts[2].find(": [Fetch] ") == 0)
    {
      monitored_action = monitor_fetch;
      sql = event_message_parts[2].substr(10);
    }
    else if(event_message_parts[2].find(": [Execute] ") == 0)
    {
      monitored_action = monitor_execute;
      sql = event_message_parts[2].substr(12);
    }
  }
  else
    sql = event_message_parts[0];

  monitored_sql += sql;

  if(event_message_parts[event_message_parts.size() - 1] == "")
    whole_message = true;
  if(event_message_parts[event_message_parts.size() - 1] == "  SEOFReached")
  {
    seofreached = true;
    whole_message = true;
  }

  // if (!whole_message)
  //  message_router->SendMessageToAll("zool", TimeToStr(EventTime).c_str() + string(": ") + EventText.c_str());

  // if (action == monitor_fetch)
  //  for (unsigned int i = 0; i < event_message_parts.size(); i++)
  //   message_router->SendMessageToAll("debug", IntToString(i) + ":" + event_message_parts[i]);

  if(whole_message)
  {
    map<string, monitored_sql_query>::iterator i;

    if(monitored_action == monitor_execute)
    {
      i = monitored_sql_queries.insert(pair<string, monitored_sql_query>(monitored_sql, monitored_sql_query())).first;
      i->second.requested = EventTime;
      i->second.last_activity = EventTime;
      // message_router->SendMessageToAll("debug", "Requested: " + string(TimeToStr(EventTime).c_str()) + string(" - ") + monitored_sql);
    }
    else if(monitored_action == monitor_fetch)
    {
      i = monitored_sql_queries.find(monitored_sql);
      if(i != monitored_sql_queries.end())
      {
        if(seofreached)
        {
          if(i->second.answered == TDateTime())
            i->second.answered = EventTime;
          i->second.terminated = EventTime;
          i->second.last_activity = EventTime;
          monitored_sql_queries.erase(i);
          // message_router->SendMessageToAll("debug", "Terminated: " + string(TimeToStr(EventTime).c_str()) + string(" - ") + monitored_sql);
          // message_router->SendMessageToAll("monitor", "Pre-fetch: " + string(TimeToStr(i->second.answered - i->second.requested).c_str()) +
          // ", Fetch: " + string(TimeToStr(i->second.terminated - i->second.answered).c_str()) + ", Query: " + monitored_sql);
        }
        else
        {
          if(i->second.answered == TDateTime())
          {
            i->second.answered = EventTime;
            i->second.last_activity = EventTime;
            // message_router->SendMessageToAll("debug", "Answered: " + string(TimeToStr(EventTime).c_str()) + string(" - ") + monitored_sql);
            TDateTime pre_fetch_time = i->second.answered - i->second.requested;
            if(pre_fetch_time > StrToTime("00:00:01"))
              message_router->SendMessageToAll("monitor", "Pre-fetch: " + string(TimeToStr(pre_fetch_time).c_str()) + ", Query: " + monitored_sql);
          }
        }
      }
      else
      {
        // message_router->SendMessageToAll("debug", "Unrecognized fetch: " + monitored_sql);
      }
    }
    else
    {
      // message_router->SendMessageToAll("debug", "Indeterminate SQL event for statement: " + monitored_sql);
    }
  }
}
//----------------------------------------------------------------------------------------------------------------------
// *** END DATABASE EVENT HANDLERS *************************************************************************************
//----------------------------------------------------------------------------------------------------------------------

//---------------------------------------------------------------------------------------------------------------------
// *** EDIT_LISTVIEW METHODS AND HANDLERS *****************************************************************************
//---------------------------------------------------------------------------------------------------------------------
void __fastcall TformPrometheusMenu::OnEditListviewRowSelect(TObject *Sender, TListItem *Item, bool Selected)
{
  // selectitem example
  TListView     *to_click  = (TListView *)(Sender);
  edit_listview *to_inform = (edit_listview *)(to_click->Tag);
  AV_TRY
    to_inform->user_selected_row();
  AV_CATCH
}
//---------------------------------------------------------------------------
void __fastcall TformPrometheusMenu::OnEditListviewColumnClick(TObject *Sender, TListColumn *Column)
{
  // columnclick example
  TListView     *to_sort   = (TListView *)(Sender);
  edit_listview *to_inform = (edit_listview *)(to_sort->Tag);

  //sort the column
  AV_TRY
    to_inform->user_clicked_column(Column);
  AV_CATCH
}
//---------------------------------------------------------------------------
void __fastcall TformPrometheusMenu::OnEditListviewDoubleClick (TObject * Sender)
{
  // we "zoom" the record, which means finding out from the Sender (listview)'s
  // Tag property where the filter object is, and from there calling the howtoedit
  // function it should know about -if- it is not null.

  TListView     *to_click  = (TListView *)(Sender);
  edit_listview *to_inform = (edit_listview *)(to_click->Tag);

  AV_TRY
    to_inform->user_double_clicked_row();
  AV_CATCH
}
//----------------------------------------------------------------------------------------------------------------------
// *** END EDIT_LISTVIEW METHODS AND HANDLERS **************************************************************************
//----------------------------------------------------------------------------------------------------------------------

//----------------------------------------------------------------------------------------------------------------------
// *** FILTER METHODS AND HANDLERS *************************************************************************************
//----------------------------------------------------------------------------------------------------------------------
void __fastcall TformPrometheusMenu::menuHeadersClick(TObject *Sender)
{
  // popupcomponent on a popup points to the object that made it appear
  classFilterObject * to_inform = (classFilterObject *)(GpopupListActions->PopupComponent->Tag);
  if(to_inform)
    to_inform->SetHeader(!to_inform->GetHeader());
  else
    PSay("Could not show/hide headers in listview", m_program_error, MB_OK);
}
//---------------------------------------------------------------------------
void __fastcall TformPrometheusMenu::menuFilterClick(TObject *Sender)
{
  // tell the filter object to show its filter screen
  // popupcomponent on a popup points to the object that made it appear
  classFilterObject * to_inform = (classFilterObject *)(GpopupListActions->PopupComponent->Tag);

  if(to_inform)
  {
    TCustomForm *listview_parent = GetParentForm(to_inform->GetListViewHandle());
    TformSpecialGeneric *listview_parent_ex = NULL;
    if(listview_parent->InheritsFrom(__classid(TformSpecialGeneric)))
      listview_parent_ex = (TformSpecialGeneric*)listview_parent;

    FORM_TRY
      TformFilterPreferences * filterform = new TformFilterPreferences(NULL, to_inform);
      filterform->ShowTaskModal(listview_parent_ex);
    FORM_CATCH
  }
  else
    PSay("Could not ask calling listview to open preferences screen", m_program_error, MB_OK);
}
//---------------------------------------------------------------------------
void __fastcall TformPrometheusMenu::menuAddRecordClick(TObject *Sender)
{
  // use filter object's HowToAdd() value
  classFilterObject * to_inform = (classFilterObject *)(GpopupListActions->PopupComponent->Tag);
  TObject * where_to_call = to_inform->WhereAction();
  AddRecord how_to_add = to_inform->HowToAdd();
  AV_TRY
    if(where_to_call && how_to_add)
      (where_to_call->*how_to_add)();
  AV_CATCH

  //else
  //  PSay("Cannot add - form or function pointer missing", m_program_error, MB_OK);
}
//---------------------------------------------------------------------------
void __fastcall TformPrometheusMenu::menuEditRecordClick(TObject *Sender)
{
  // use filter object HowToEdit() value
  classFilterObject * to_inform = (classFilterObject *)(GpopupListActions->PopupComponent->Tag);
  TObject * where_to_call = to_inform->WhereAction();
  EditRecord how_to_edit = to_inform->HowToEdit();
  string record_to_edit = to_inform->GetSelectedRecordID();
  AV_TRY
    if(where_to_call && how_to_edit && (!record_to_edit.empty()))
      (where_to_call->*how_to_edit)(record_to_edit);
  AV_CATCH

  //else
  //  PSay("Cannot add - form or function pointer missing", m_program_error, MB_OK);
}
//---------------------------------------------------------------------------
void __fastcall TformPrometheusMenu::GpopupListActionsPopup(TObject *Sender)
{
  classFilterObject * to_inform = (classFilterObject *)(GpopupListActions->PopupComponent->Tag);

  // defaults
  menuFilter->Visible          = false;
  menuCopyListView->Visible    = false;
  menuSaveListView->Visible    = false;
  menuExportListview->Visible  = false;
  menuModifySQL->Visible       = false;
  menuAutosizeColumns->Visible = false;
  menuRefresh->Visible         = false;
  menuHeaders->Visible         = false;
  menuAddRecord->Visible       = false;
  menuEditRecord->Visible      = false;
  menuCheck->Visible           = false;

  // enable only what works
  if(to_inform)
  {
    // if (to_inform->CanFilter())
    menuFilter->Visible = true;
    if(optionsPermissions["special.save_listview_data"]["enabled"] == "true")
    {
      menuCopyListView->Visible = true;
      menuSaveListView->Visible = true;
      menuExportListview->Visible = true;
    }
    if(optionsPermissions["special.modify_listview_sql"]["enabled"] == "true")
    {
      menuModifySQL->Visible = true;
      menuRefresh->Visible = true; // temporary? here because programmers are normally the only ones who need it ...
    }

    menuAutosizeColumns->Visible = true; // TODO: debug, disabled until autosize can work properly on virtual listviews
    menuHeaders->Visible = true;
    menuHeaders->Checked = to_inform->GetHeader();

    if(to_inform->WhereAction()) // && to_inform->CanIdentify())
    {
      if (to_inform->HowToAdd())
        menuAddRecord->Visible = true;
      if (to_inform->HowToEdit() && (!(to_inform->GetSelectedRecordID()).empty()))
        menuEditRecord->Visible = true;
    }

    if(to_inform->GetCheck())
    {
      menuCheck->Visible = true;
    }
  }

  menuActions->Visible = false;
  vector<string> selected_ids = to_inform->GetMultiID();
  // TODO -c HACK: although GetSecurity() -usually- returns the table name, it's -possible- that's not always the case
  //  ... should be a separate notion.
  string stn = to_inform->GetSecurity();
  if(!selected_ids.empty())
    TformSpecialAction::FillActionMenu(menuActions, stn);
}
//---------------------------------------------------------------------------
void __fastcall TformPrometheusMenu::DoubleClickListView (TObject * Sender)
{
  // we "zoom" the record, which means finding out from the Sender (listview)'s
  // Tag property where the filter object is, and from there calling the howtoedit
  // function it should know about -if- it is not null.

  TListView         *to_click  = (TListView *)(Sender);
  classFilterObject *to_inform = (classFilterObject *)(to_click->Tag);
  TObject *where_to_call = to_inform->WhereAction();
  EditRecord how_to_edit = to_inform->HowToEdit();
  string  record_to_edit = to_inform->GetSelectedRecordID();
  AV_TRY
    if(where_to_call && how_to_edit && (!record_to_edit.empty()))
      (where_to_call->*how_to_edit)(record_to_edit);
  AV_CATCH

  //else
  //  PSay("Cannot edit - form or function pointer missing", m_program_error, MB_OK);
}
//--------------------------------------------------------------------------
void __fastcall TformPrometheusMenu::OnColumnClickListView(TObject * Sender, TListColumn * Column)
{
  // change column being sorted by to the new one, then execute the sort
  // on the listview
  TListView         *to_sort   = (TListView *)(Sender);
  classFilterObject *to_inform = (classFilterObject *)(to_sort->Tag);

  // jhaley 20100901: DEBUG, TEST
  // If the column has been dragged out of its original position, the Tag will no longer equal the Index.
  // There seems to be a bug in VCL listviews which causes click events to direct to the wrong column in
  // such a way that the clicks go to the column that was originally in that position.
  TListColumn *column_to_sort_by = Column;
  pair<int, int> *column_indices = (pair<int, int> *)(Column->Tag);
  if(Column->Index != column_indices->second)
  {
    // Find the column which originally matched this index;
    // The "second" member of the pair object tracks the original index into Columns.
    for(int i = 0; i < to_sort->Columns->Count; i++)
    {
      TListColumn *current_column = to_sort->Columns->Items[i];
      pair<int, int> *current_indices = (pair<int, int> *)(current_column->Tag);
      if(current_indices->second == Column->Index)
      {
        column_to_sort_by = current_column;
        break;
      }
    }
  }

  //sort the column
  AV_TRY
    //to_inform->Sort(Column, true);
    to_inform->Sort(column_to_sort_by, true);
  AV_CATCH
}
//---------------------------------------------------------------------------
void __fastcall TformPrometheusMenu::menuSaveListViewClick(TObject *Sender)
{
  // download data from a list view into a file
  TListView *to_inform = (TListView *)(GpopupListActions->PopupComponent);

  if(to_inform)
  {
    TCustomForm * listview_parent = GetParentForm(to_inform);
    TformSpecialGeneric * listview_parent_ex = NULL;
    if(listview_parent->InheritsFrom(__classid(TformSpecialGeneric)))
      listview_parent_ex = (TformSpecialGeneric*)listview_parent;

    FORM_TRY
      TformSpecialSaveListView * save_form = new TformSpecialSaveListView(NULL, to_inform);
      save_form->ShowTaskModal(listview_parent_ex);
    FORM_CATCH
  }
  else
    PSay("Could not ask calling listview to save contents to file", m_program_error, MB_OK);
}
//---------------------------------------------------------------------------
void __fastcall TformPrometheusMenu::menuExportListviewClick(TObject *Sender)
{
  // advanced listview export (many more options!)
  TListView *to_inform = (TListView *)(GpopupListActions->PopupComponent);
  classFilterObject *to_ask = (classFilterObject *)(to_inform->Tag);
  if(to_ask)
  {
    TCustomForm * listview_parent = GetParentForm(to_ask->GetListViewHandle());
    TformSpecialGeneric * listview_parent_ex = NULL;
    if(listview_parent->InheritsFrom(__classid(TformSpecialGeneric)))
      listview_parent_ex = (TformSpecialGeneric*)listview_parent;

    FORM_TRY
      TformSpecialExportListview * export_form = new TformSpecialExportListview(NULL, to_ask);
      export_form->ShowTaskModal(listview_parent_ex);
    FORM_CATCH
  }
  else
    PSay("Could not ask calling listview to export contents to file", m_program_error, MB_OK);
}
//---------------------------------------------------------------------------
void __fastcall TformPrometheusMenu::menuCopyListViewClick(TObject *Sender)
{
  TListView *to_inform = (TListView *)(GpopupListActions->PopupComponent);
  classFilterObject *to_ask = (classFilterObject *)(to_inform->Tag);
  if(to_ask)
  {
    vector<string> headers;
    vector<show_type> header_types;
    vector<vector<string> > csv = to_ask->GetAllData (headers, header_types, false, false);
    CopyCSVToClipboard(csv, headers, true, false);
  }
}
//---------------------------------------------------------------------------
void __fastcall TformPrometheusMenu::menuModifySQLClick(TObject *Sender)
{
  classFilterObject * to_inform = (classFilterObject *)(GpopupListActions->PopupComponent->Tag);
  if(to_inform)
  {
    TCustomForm *listview_parent = GetParentForm(to_inform->GetListViewHandle());
    TformSpecialGeneric *listview_parent_ex = NULL;
    if(listview_parent->InheritsFrom(__classid(TformSpecialGeneric)))
      listview_parent_ex = (TformSpecialGeneric*)listview_parent;

    // TForm * the_form = (TForm *)(GpopupListActions->PopupComponent->Owner);
    string sql = to_inform->GetSQLStatement();
    string original_sql = sql;
    FORM_TRY
      TformSpecialSqlViewer * viewer = new TformSpecialSqlViewer(NULL, &sql);
      viewer->ShowTaskModal(listview_parent_ex);
    FORM_CATCH
    if(sql != original_sql)
    {
      to_inform->SetSQLStatement(sql);
      to_inform->RefreshDataSet();
    }
  }
}
//---------------------------------------------------------------------------
void __fastcall TformPrometheusMenu::FilterListViewAdvancedCustomDraw(TCustomListView *Sender,
      const TRect &ARect, TCustomDrawStage Stage, bool &DefaultDraw)
{
  classFilterObject * to_inform = (classFilterObject*)Sender->Tag;
  if(to_inform)
  {
    PGlobals->pending_refreshes.push_back(to_inform);
    //to_inform->RefreshDataSet();
  }
  else
  {
    PSay("This listview does not seem to have an associated filter object. No updates will be sent.",
         m_program_error, MB_OK);
  }
  DefaultDraw = true;
}
//---------------------------------------------------------------------------
// jhaley 20100720: Should fix up the EAccessViolations occuring from timerPendingFiltersTimer below.
// Dangling pointers were being left in the pending_refreshes list when classFilterObjects were deleted
// with a pending refresh. This is registered above and is called from ~classFilterObject.
void TformPrometheusMenu::FilterListViewDeleted(classFilterObject *cfo)
{
  // if we're tracking any refreshes, remove this one from the list.
  if(!PGlobals->pending_refreshes.empty())
    PGlobals->pending_refreshes.remove(cfo);
}
//---------------------------------------------------------------------------
void __fastcall TformPrometheusMenu::timerPendingFiltersTimer(TObject *Sender)
{
  // check for pending filter objects needing to refresh,
  // thanks to calls to the function a bit above here
  classFilterObject *to_update = NULL;

  // GUB
  while(!PGlobals->pending_refreshes.empty())
  {
    to_update = PGlobals->pending_refreshes.front();
    PGlobals->pending_refreshes.pop_front();
    AV_TRY
      to_update->RefreshDataSet();
    AV_CATCH
  }
}
//---------------------------------------------------------------------------
void __fastcall TformPrometheusMenu::menuCheckAllClick(TObject *Sender)
{
  // check all items in given filter listview
  classFilterObject * to_inform = (classFilterObject *)(GpopupListActions->PopupComponent->Tag);
  if(to_inform)
    to_inform->CheckAll();
  else
    PSay("Could not check all items in listview", m_program_error, MB_OK);
}
//---------------------------------------------------------------------------
void __fastcall TformPrometheusMenu::menuCheckFlipClick(TObject *Sender)
{
  // flip checked/unchecked in a filter listview
  classFilterObject * to_inform = (classFilterObject *)(GpopupListActions->PopupComponent->Tag);
  if(to_inform)
    to_inform->FlipCheck();
  else
    PSay("Could not flip checked state of items in listview", m_program_error, MB_OK);
}
//---------------------------------------------------------------------------
void __fastcall TformPrometheusMenu::menuCheckNoneClick(TObject *Sender)
{
  // uncheck all items in a filter listview
  classFilterObject * to_inform = (classFilterObject *)(GpopupListActions->PopupComponent->Tag);
  if(to_inform)
    to_inform->UncheckAll();
  else
    PSay("Could not uncheck all items in listview", m_program_error, MB_OK);
}
//---------------------------------------------------------------------------
void __fastcall TformPrometheusMenu::timerFilterHintTimerProxy(TObject *Sender)
{
  if(debug_level[other_forms] >= 4 && message_router)
    message_router->SendMessageToAll("debug", "formPrometheusMenu::timerFilterHintTimerProxy entry");
  // since timerFilterHintTimer is static, we can't assign it to the OnTimer event
  // directly.  So we assign this function instead which calls it by proxy
  classFilterObject::timerFilterHintTimer(Sender);
  if(debug_level[other_forms] >= 4 && message_router)
    message_router->SendMessageToAll("debug", "formPrometheusMenu::timerFilterHintTimerProxy exit");
}
//---------------------------------------------------------------------------
void TformPrometheusMenu::PreferencesChanged()
{
  if(debug_level[other_forms]>=4 && message_router)
    message_router->SendMessageToAll("debug", "TformPrometheusMenu::PreferencesChanged entry");

  // see codeSpecialSettings
  link_color = clNavy;
  if(optionsPreferences["color"]["link_color"] != "")
    link_color = TColor(StringToInt(optionsPreferences["color"]["link_color"]));

  title_bar_color = TColor(0x00e1ffff);
  if(optionsPreferences["color"]["title_bar_color"] != "")
    title_bar_color = TColor(StringToInt(optionsPreferences["color"]["title_bar_color"]));

  if(optionsPreferences["filter"]["no_persistent_hints"] != "true")
    classFilterObject::EnableFloatingHint();
  else
    classFilterObject::DisableFloatingHint();

  if(optionsPreferences["icons"]["no_menu_icons"] != "true")
    menuFrame->GetMainMenu()->Images = GimagesRecordTypes;
  else
    menuFrame->GetMainMenu()->Images = NULL;

  classFilterObject::SetAutosizeColumns(optionsPreferences["filter"]["always_autosize_columns"] == "true");

  // quick-access menus
  for (vector<TSpeedButton *>::iterator k = quick_menus.begin(); k != quick_menus.end(); k++)
    delete (*k);
  quick_menus.clear();
  panelQuickMenus->Height = 0;
  DebugWidth("RESETTING form width to " + IntToString(startup_form_width));
  if(this->Constraints->MinWidth > 0)
    this->Constraints->MinWidth = startup_form_width;
  if(this->Constraints->MaxWidth > 0)
    this->Constraints->MaxWidth = startup_form_width;
  this->Width = startup_form_width;
  DebugWidth("SETTING panelQuickMenus to " + IntToString(menuFrame->GetControlBar()->Width));
  panelQuickMenus->Width = menuFrame->GetControlBar()->Width;
  DebugWidth("SET");

  vector<string> temp_quick_menus;
  SplitStringToVec(optionsPreferences["menu"]["quick_access"], ",", temp_quick_menus);
  unsigned int t = 0; // barMenu->Height;
  int max_item_width = 0, current_item_width /*, current_item_height*/;
  int item_left_padding = 40;

  for(unsigned int i = 0; i < temp_quick_menus.size(); i++)
  {
    if(temp_quick_menus[i] != "")
    {
      TMenuItem * z = (TMenuItem*)(FindMenuItemByNameInsensitive(menuFrame->GetMainMenu()->Items, temp_quick_menus[i]));
      if(z && z->Visible && z->Enabled)
      {
        TSpeedButton * y = new TSpeedButton((TComponent*)NULL);
        y->Parent = panelQuickMenus; // this;
        y->Top = t + 2;
        y->Left = 2;
        y->Width = menuFrame->GetControlBar()->Width - 4;
        y->Height = 21;
        y->Margin = 8;
        y->Anchors = y->Anchors << akLeft << akRight << akTop;
        y->Caption = FullyQualifiedMenuItemName(z).c_str();
        /*TODO -c HACK: should really look directly at y    */
        //current_item_width = GetTextExtent(toolsPrometheus->Canvas->Handle, toolsPrometheus->Canvas->Font,
        //  FullyQualifiedMenuItemName(z).c_str(), current_item_width, current_item_height);
        current_item_width = menuFrame->GetToolBar()->Canvas->TextWidth(FullyQualifiedMenuItemName(z).c_str());
        if(debug_level[other_forms] >= 9 && message_router)
        {
          message_router->SendMessageToAll("debug", "TformPrometheusMenu::PreferencesChanged current_item_width = "
             + IntToString(current_item_width));
        }
        if(current_item_width > max_item_width)
          max_item_width = current_item_width;
        y->Cursor = crHandPoint;
        y->Flat = true;
        y->OnClick = z->OnClick;
        y->Transparent = false;
        t += y->Height;
        quick_menus.push_back(y);
        GimagesRecordTypes->GetBitmap(z->ImageIndex, y->Glyph);
      }
    }
  }
  if(t)
    panelQuickMenus->Height = t + 4;
  if(debug_level[other_forms] >= 9 && message_router)
  {
    message_router->SendMessageToAll("debug", "TformPrometheusMenu::PreferencesChanged max_item_width = "
      + IntToString(max_item_width));
  }

  if((max_item_width + item_left_padding) > menuFrame->GetControlBar()->Width)
  {
    int width_diff = (max_item_width + item_left_padding) - menuFrame->GetControlBar()->Width;
    int new_form_width = this->Width + width_diff;
    if(debug_level[other_forms] >= 9 && message_router)
    {
      message_router->SendMessageToAll("debug", "TformPrometheusMenu::PreferencesChanged width_diff = "
        + IntToString(width_diff));
    }

    if(this->Constraints->MaxWidth > 0)
    {
      DebugWidth("SETTING MaxWidth to " + IntToString(this->Constraints->MaxWidth) + " + " + IntToString(width_diff) +
                 " = " + IntToString(this->Constraints->MaxWidth + width_diff));
      this->Constraints->MaxWidth = this->Constraints->MaxWidth + width_diff;
      DebugWidth("SET");
    }
    if(this->Constraints->MinWidth > 0)
    {
      DebugWidth("SETTING MinWidth to " + IntToString(this->Constraints->MinWidth) + " + " + IntToString(width_diff) +
                 " = " + IntToString(this->Constraints->MinWidth + width_diff));
      this->Constraints->MinWidth = this->Constraints->MinWidth + width_diff;
      DebugWidth("SET");
    }
    DebugWidth("SETTING form Width to " + IntToString(new_form_width));
    this->Width = new_form_width;
    DebugWidth("SET");

    //barMenu->Width = barMenu->Width + width_diff;
    DebugWidth("SETTING panelQuickMenus to " + IntToString(menuFrame->GetControlBar()->Width) + " + " +
               IntToString(width_diff) + " = " + IntToString(menuFrame->GetControlBar()->Width + width_diff));
    panelQuickMenus->Width = menuFrame->GetControlBar()->Width + width_diff;
    DebugWidth("SET");
    for(vector<TSpeedButton *>::iterator k = quick_menus.begin(); k != quick_menus.end(); k++)
      (*k)->Width = menuFrame->GetControlBar()->Width + width_diff - 4;
  }

  if(debug_level[other_forms]>=4 && message_router)
    message_router->SendMessageToAll("debug", "TformPrometheusMenu::PreferencesChanged exit");
}
//---------------------------------------------------------------------------
void __fastcall TformPrometheusMenu::menuAutosizeColumnsClick(TObject *Sender)
{
  // resize all
  classFilterObject * to_inform = (classFilterObject *)(GpopupListActions->PopupComponent->Tag);
  if(to_inform)
    to_inform->AutosizeColumns();
  else
    PSay("Could not ask calling listview to resize columns.", m_program_error, MB_OK);
}
//---------------------------------------------------------------------------
void __fastcall TformPrometheusMenu::menuRefreshClick(TObject *Sender)
{
  // resize all
  classFilterObject * to_inform = (classFilterObject *)(GpopupListActions->PopupComponent->Tag);
  if(to_inform)
    to_inform->ForceRefreshDataSet();
  else
    PSay("Could not ask calling listview to refresh.", m_program_error, MB_OK);
}
//---------------------------------------------------------------------------
void __fastcall TformPrometheusMenu::menuCheckTheseClick(TObject *Sender)
{
  classFilterObject * to_inform = (classFilterObject *)(GpopupListActions->PopupComponent->Tag);
  if(to_inform)
    to_inform->CheckSelected();
  else
    PSay("Could not check all selected items in listview", m_program_error, MB_OK);
}
//---------------------------------------------------------------------------
void __fastcall TformPrometheusMenu::menuCheckFlipTheseClick(TObject *Sender)
{
  classFilterObject * to_inform = (classFilterObject *)(GpopupListActions->PopupComponent->Tag);
  if(to_inform)
    to_inform->FlipCheckSelected();
  else
    PSay("Could not flip all selected items' checked state in listview", m_program_error, MB_OK);
}
//---------------------------------------------------------------------------
void __fastcall TformPrometheusMenu::menuUncheckTheseClick(TObject *Sender)
{
  classFilterObject * to_inform = (classFilterObject *)(GpopupListActions->PopupComponent->Tag);
  if(to_inform)
    to_inform->UncheckSelected();
  else
    PSay("Could not uncheck all selected items in listview", m_program_error, MB_OK);
}
//---------------------------------------------------------------------------
void __fastcall TformPrometheusMenu::menuActionsClick(TObject *Sender)
{
 if (((TMenuItem*)Sender)->Tag)
 {
  classFilterObject * to_inform = (classFilterObject *)(GpopupListActions->PopupComponent->Tag);
  if (to_inform)
  {
   // TODO -c HACK: although GetSecurity() -usually- returns the table name, it's -possible- that's not always the case
   // ... should be a separate notion.
   string stn = to_inform->GetSecurity();
   vector<string> selected_ids = to_inform->GetMultiID();
   a_factory_create to_create = (a_factory_create)(((TMenuItem*)Sender)->Tag);
   TformSpecialAction::Do(stn, to_create, selected_ids);
  }
  else
   PSay("Could not figure out how to initiate the action (no source table.)", m_program_error, MB_OK);
 }
}
//---------------------------------------------------------------------------
void __fastcall TformPrometheusMenu::GetListViewData(TObject *Sender, TListItem *Item)
{
  classFilterObject * to_inform = (classFilterObject *)(((TListView *)(Sender))->Tag);
  if(to_inform)
    to_inform->SetRowAttributes(Item);
  else
    PSay("Could not get requested data for listview.", m_program_error, MB_OK);
}
//---------------------------------------------------------------------------
void __fastcall TformPrometheusMenu::ListViewKeyDown(TObject *Sender, WORD &Key, TShiftState Shift)
{
  classFilterObject * to_inform = (classFilterObject *)(((TListView *)(Sender))->Tag);
  if(to_inform)
    to_inform->KeyPressed(Key, Shift);
  else
    PSay("Could not respond to keyboard event.", m_program_error, MB_OK);
}
//---------------------------------------------------------------------------
void __fastcall TformPrometheusMenu::ListViewClick(TObject *Sender)
{
  classFilterObject * to_inform = (classFilterObject *)(((TListView *)(Sender))->Tag);
  if(to_inform)
    to_inform->Clicked(Sender);
  else
    PSay("Could not respond to item click.", m_program_error, MB_OK);
}
//----------------------------------------------------------------------------------------------------------------------
// *** END FILTER METHODS AND HANDLERS *********************************************************************************
//----------------------------------------------------------------------------------------------------------------------



//---------------------------------------------------------------------------
// TODO : Dead(?) Code Below vvvvv
//---------------------------------------------------------------------------
void __fastcall TformPrometheusMenu::menuCreateCallBehClick(TObject *Sender)
{
  map<string,string> temp_map;
  temp_map["id"] = "new";
  temp_map["user_id"] = current_userid;
  temp_map["time_of_task"] = CurrentTime();
  temp_map["date_of_task"] = CurrentDate();
  temp_map["method_id"] = lookupTables->IdOf("method_types", "Phone");
  temp_map["group_id"]  = lookupTables->IdOf("groups", "Behavioral Health");
  FORM_TRY
    TformRecordTaskBeh * taskRecord = new TformRecordTaskBeh(NULL, temp_map);
    taskRecord->Show();
  FORM_CATCH
}
//---------------------------------------------------------------------------
void __fastcall TformPrometheusMenu::person1Click(TObject *Sender)
{
  // WTF?
  // dude, if you don't know, I sure as hell don't.
  map<string,string> temp_map;
  temp_map["id"] = "3"; // TODO -c MAGIC NUMBER :
  FORM_TRY
    TformRecordPerson * personRecord = new TformRecordPerson(NULL, temp_map);
    personRecord->Show();
  FORM_CATCH
}
//---------------------------------------------------------------------------

