//---------------------------------------------------------------------------
#include <vcl.h>
#pragma hdrstop

#include "codeRecordTransaction.h"
#include "codeRecordAccount.h"
#include "codeSearchAccount.h"
#include "launchBudgetReport.h"
#include "codeSpecialTransactionDetails.h"
//---------------------------------------------------------------------------
#pragma package(smart_init)
#pragma link "codeRecordGeneric"
#pragma resource "*.dfm"
TformRecordAccount *formRecordAccount;
//---------------------------------------------------------------------------
__fastcall TformRecordAccount::TformRecordAccount(TComponent* Owner, map<string,string> p_parameters)
        : TformRecordGeneric(Owner, p_parameters, string("accounts")), transactions_summary(this, listTransactions)
{
 ready_for_drop_down_change = false;

 accounts_filter.SetListViewHandle(listAccounts);
 accounts_filter.SetSecurity("accounts");
 accounts_filter.AddColumnSpec("Account Number", "accounts", as_text, true, 150);
 accounts_filter.AddColumnSpec("Fund", "accounts", as_text, false, 75);
 accounts_filter.AddColumnSpec("General Ledger", "accounts", as_text, false, 100);
 accounts_filter.AddColumnSpec("Department", "accounts", as_text, false, 75);
 accounts_filter.AddColumnSpec("Sub-department", "accounts", as_text, false, 75);
 accounts_filter.AddColumnSpec("Budget", "accounts", as_text, true, 60, prettyBoolString);
 accounts_filter.AddColumnSpec("Disabled", "accounts", as_text, true, 60, prettyBoolString);
 accounts_filter.AddColumnSpec("Yearly", "accounts", as_text, true, 60, prettyBoolString);
 accounts_filter.AddColumnSpec("Cycle", "accounts", as_text, true, 100);
 accounts_filter.AddColumnSpec("Type", "accounts", as_text, true, 100);
 accounts_filter.AddColumnSpec("Current Id", "current_id", as_int, false, 80);
 accounts_filter.AddColumnSpec("Internal Id", "internal_id", as_int, false, 80);
 accounts_filter.FetchPreferences("recordaccount.accounts_filter");
 accounts_filter.SetIconStatus(true);
 accounts_filter.SetSQLStatement("");
 accounts_filter.RefreshDataSet();

 staticTransactionDetails->Font->Color = link_color;
 staticBudgetReport->Font->Color = link_color;

 Initialize();
}
//---------------------------------------------------------------------------
bool TformRecordAccount::ScreenStart()
{
  // clear the rich edit box
  editDescription->Text = "";

  // register combo boxes
  lookupTables->RegisterComboBox("acc_level_one", comboOne);
  lookupTables->RegisterComboBox("acc_level_two", comboTwo);
  lookupTables->RegisterComboBox("acc_level_three", comboThree);
  lookupTables->RegisterComboBox("acc_level_four", comboFour);
  lookupTables->RegisterComboBox("account_types", comboAccountType);
  lookupTables->RegisterComboBox("month_types", comboCycle);

  TformRecordGeneric::ScreenStart();
  return true;
}
//---------------------------------------------------------------------------
bool TformRecordAccount::ScreenStop()
{
  // unregister combo boxes
  lookupTables->UnregisterComboBox("acc_level_one", comboOne);
  lookupTables->UnregisterComboBox("acc_level_two", comboTwo);
  lookupTables->UnregisterComboBox("acc_level_three", comboThree);
  lookupTables->UnregisterComboBox("acc_level_four", comboFour);
  lookupTables->UnregisterComboBox("account_types", comboAccountType);
  lookupTables->UnregisterComboBox("month_types", comboCycle);

  TformRecordGeneric::ScreenStop();
  return true;
}
//---------------------------------------------------------------------------
bool TformRecordAccount::IsValidOnScreen()
{
  // check to make sure that the values entered in the combo boxe
  // corrospond to values in that look up table.
  // give the user a message box and prompt to re-enter if incorrect
  // make sure there is a link to a person, although there's not much the user
  // can do to fix that if there's not
  // WHA???

  if (! lookupTables->IsInLookUpTable("acc_level_one", comboOne->Text.c_str()))
  {
    PSay("Please enter a valid 'fund'", m_invalid_data, MB_OK);
    comboOne->Show();
    comboOne->SetFocus();
    return false;
  }
  else if (! lookupTables->IsInLookUpTable("acc_level_two", comboTwo->Text.c_str()))
  {
    PSay("Please enter a valid 'general ledger'", m_invalid_data, MB_OK);
    comboTwo->Show();
    comboTwo->SetFocus();
    return false;
  }
  else if (! lookupTables->IsInLookUpTable("acc_level_three", comboThree->Text.c_str()))
  {
    PSay("Please enter a valid 'department'", m_invalid_data, MB_OK);
    comboThree->Show();
    comboThree->SetFocus();
    return false;
  }
  else if (! lookupTables->IsInLookUpTable("acc_level_four", comboFour->Text.c_str()))
  {
    PSay("Please enter a valid 'sub-department'", m_invalid_data, MB_OK);
    comboFour->Show();
    comboFour->SetFocus();
    return false;
  }
  else if (! lookupTables->IsInLookUpTable("account_types", comboAccountType->Text.c_str()))
  {
    PSay("Please enter a valid 'account-type'", m_invalid_data, MB_OK);
    comboAccountType->Show();
    comboAccountType->SetFocus();
    return false;
  }
  else if (! lookupTables->IsInLookUpTable("month_types", comboCycle->Text.c_str()))
  {
    PSay("Please enter a valid 'cycle beginning month'", m_invalid_data, MB_OK);
    comboCycle->Show();
    comboCycle->SetFocus();
    return false;
  }
  else if (! Pdate (editMinimumFiscalDate->Text.c_str(), mdy | yy).IsValidDate() && (editMinimumFiscalDate->Text != ""))
  {
    PSay("Please enter a valid minimum fiscal date, to be applied to any new transactions posted to this account.", m_invalid_data, MB_OK);
    editMinimumFiscalDate->Show();
    editMinimumFiscalDate->SetFocus();
    return false;
  }
  else
  {
   // check for duplicate
   string where_statement = "where "
    "acc_level_one_id = " + lookupTables->IdOf("acc_level_one", comboOne->Text.c_str()) + " and "
    "acc_level_two_id = " + lookupTables->IdOf("acc_level_two", comboTwo->Text.c_str()) + " and "
    "acc_level_three_id = " + lookupTables->IdOf("acc_level_three", comboThree->Text.c_str()) + " and "
    "acc_level_four_id = " + lookupTables->IdOf("acc_level_four", comboFour->Text.c_str()) + " and "
    "is_budget = " + (checkBudget->Checked ? string("1") : string("0"));
   if (!is_new_record)
    where_statement += " and id != " + initial_state["id"];
   string sql = "select id from accounts " + where_statement;
   if (GetOneField(dbDatabase, sql) != "")
   {
    // something found that isn't us! oh god!
    PSay("That account number/budget already exists, please select a different one.", m_invalid_data, MB_OK);
    editAccount->Show();
    editAccount->SetFocus();
    return false;
   }
   else
   {
    /* TODO :
       [ ] - look at new_ref_id and see if they've done something drastically wrong!!!
       [ ] - make sure that if they changed the account properties (type, budget, etc.) the existing refs were updated
       [x] - warn if references missing
       [x] - warn if extra references given [ask to remove for the user]
    */
    /*
    map<string, acc_info> will_be_refs;
    for (map<string, string>::iterator i_initial = initial_state.begin(); i_initial != initial_state.end(); i_initial++)
     if (LikeString("%_account_id", i_initial->first))
      will_be_refs[i_initial->first] = InfoById (i_initial->second);
    for (map<string, string>::iterator i_new = new_ref_id.begin(); i_new != new_ref_id.end(); i_new++)
     will_be_refs[i_new->first] = InfoById (i_new->second);
    */

    set<string> need_refs = GetRequiredAccountReferences(comboAccountType->Text.c_str(), checkBudget->Checked);

    // check needed ones
    // NOW verified in Validate...()
    /*
    for (set<string>::iterator i_n = need_refs.begin(); i_n != need_refs.end(); i_n++)
    {
     if (will_be_refs.find(*i_n) == will_be_refs.end() || will_be_refs[*i_n].IsFound() == false || will_be_refs[*i_n].id == "0")
     {
      if (IDNO == PSay(string("'" + *i_n + "' is a very important field for this type of account, but hasn't been specified.\nNot specifying reference accounts could result in the program not reacting correctly when the accounting system is used.\nWould you like to continue anyway?").c_str(), m_warning, MB_YESNO))
       return false;
     }
    }
    */

    // check for extra ones
    for (map<account_links, acc_info>::iterator i_w = new_ref_info.begin(); i_w != new_ref_info.end(); i_w++)
    {
     if (i_w->second.id != "0" && need_refs.find(LinkTypeToString(i_w->first)) == need_refs.end())
     {
      switch (PSay(string("'" + LinkTypeToString(i_w->first) + "' has a value, but is  not required. This won't hurt anything, but it looks mighty strange.\nWould you like the reference to be cleared for you?").c_str(), m_warning, MB_YESNOCANCEL))
      {
       case IDYES:
        // act as if the user had cleared the field
        // make the rest of the checking code not notice this field
        i_w->second = InfoById ("0");
        /* TODO: if there are several of these, and they say "yes, yes, cancel" -- the links won't update.
                 yes, that'd be ugly. and confusing. */
        break;
       case IDNO:
        break;
       case IDCANCEL:
        return false;
       default:
        break;
      }
     }
    }

    /*
    // NOW ... handled below automatically?
    // verify validity of accounts!
    // should remove the "0" ones; if the user -really- means to continue,
    // there's no point in bitching about those. they know they're missing.
    map<account_links, acc_info> to_verify;

    // all the non-zero ones that are needed (ignore zeroes, ignore non-imperitive)
    for (map<string, acc_info>::iterator i_w = will_be_refs.begin(); i_w != will_be_refs.end(); i_w++)
     if (i_w->second.id != "0" && need_refs.find(i_w->first) != need_refs.end())
      to_verify[StringToLinkType(i_w->first)] = i_w->second;
    */

    // this will give them "helpful" messages about their mistakes. it has the final word.
    if (!ValidateAccountReferences(new_ref_info, StringToAccountType(comboAccountType->Text.c_str()), false, checkClosed->Checked, checkBudget->Checked))
     return false;

    return true;
   }
  }
}
//---------------------------------------------------------------------------
bool TformRecordAccount::PutDataOnScreen()
{
  // put the values on the form from the map initial_state
  // values may have come from GetDataFromDatabase if it's an existing record,
  // or values may have been passed to the form in this map if it's a new record
  comboOne->Text = lookupTables->ValueOf("acc_level_one", initial_state["acc_level_one_id"]).c_str();
  comboTwo->Text = lookupTables->ValueOf("acc_level_two", initial_state["acc_level_two_id"]).c_str();
  comboThree->Text = lookupTables->ValueOf("acc_level_three", initial_state["acc_level_three_id"]).c_str();
  comboFour->Text = lookupTables->ValueOf("acc_level_four", initial_state["acc_level_four_id"]).c_str();
  comboCycle->Text = lookupTables->ValueOf("month_types", initial_state["bill_cycle_id"]).c_str();
  comboAccountType->Text = lookupTables->ValueOf("account_types", initial_state["account_type_id"]).c_str();

  if (comboOne->Text == "") comboOne->Text = "<blank>";
  if (comboTwo->Text == "") comboTwo->Text = "<blank>";
  if (comboThree->Text == "") comboThree->Text = "<blank>";
  if (comboFour->Text == "") comboFour->Text = "<blank>";

  // now that the drop-downs are set up, go ahead and set the text box thingie
  ready_for_drop_down_change = true;
  comboAnyChange(NULL);

  editName->Text = PrettyString(initial_state["name"]).c_str();
  editDescription->Text = SentenceString(initial_state["description"]).c_str();
  editMinimumFiscalDate->Text = initial_state["min_fiscal_date"].c_str();

  checkBudget->Checked = (initial_state["is_budget"] == "1");
  checkClosed->Checked = (initial_state["is_closed"] == "1");
  checkYearly->Checked = (initial_state["is_yearly"] == "1");

  // editAccount should be set up by the OnChange events, no? we'll see.

  CreateLink(staticBalance, buttonSearchBalance, this, initial_state["balance_account_id"], "accounts");
  CreateLink(staticPending, buttonSearchPending, this, initial_state["pending_account_id"], "accounts");
  CreateLink(staticCash,    buttonSearchCash,    this, initial_state["cash_account_id"],    "accounts");
  CreateLink(staticBudget,  buttonSearchBudget,  this, initial_state["budget_account_id"],  "accounts");
  CreateLink(staticActual,  buttonSearchActual,  this, initial_state["actual_account_id"],  "accounts");
  CreateLink(staticRepay,   buttonSearchRepay,   this, initial_state["repay_account_id"],   "accounts");
  RefreshLinkStatus();

  labelTotalThis->Caption    = GetCurrentAccountBalance(initial_state["id"]).c_str();
  labelTotalBalance->Caption = GetCurrentAccountBalance(initial_state["balance_account_id"]).c_str();
  labelTotalPending->Caption = GetCurrentAccountBalance(initial_state["pending_account_id"]).c_str();
  labelTotalCash->Caption    = GetCurrentAccountBalance(initial_state["cash_account_id"]).c_str();
  labelTotalBudget->Caption  = GetCurrentAccountBalance(initial_state["budget_account_id"]).c_str();
  labelTotalActual->Caption  = GetCurrentAccountBalance(initial_state["actual_account_id"]).c_str();
  labelTotalRepay->Caption   = GetCurrentAccountBalance(initial_state["repay_account_id"]).c_str();
  // just to help them out ...
  if (staticBudget->Enabled)
   labelTotalRemaining->Caption = FinancialToString(StringToFinancial(labelTotalBudget->Caption.c_str()) - StringToFinancial(labelTotalThis->Caption.c_str()), true).c_str();
  else if (staticActual->Enabled)
   labelTotalRemaining->Caption = FinancialToString(StringToFinancial(labelTotalThis->Caption.c_str()) - StringToFinancial(labelTotalActual->Caption.c_str()), true).c_str();
  else
   labelTotalRemaining->Caption = "unknown";

  new_ref_info.clear();
  // need to fill this in with InitialState, as other things will be expecting it ... (IsValid, etc.)
  for (map<string, string>::iterator i_field = initial_state.begin(); i_field != initial_state.end(); i_field++)
   if (StringToLinkType(i_field->first) != unknown_ref)
    new_ref_info[StringToLinkType(i_field->first)] = InfoById(i_field->second);

  TformRecordGeneric::PutDataOnScreen();
  return true; // what can really go wrong?
}
//---------------------------------------------------------------------------
bool TformRecordAccount::GetDataFromScreen()
{
  // place the values currently on the screen into
  // the map that will be used to update or insert the record
  // the parent form will call the insert or update function, and get the id
  // number if necessary and take care of current_id
  ending_state["acc_level_one_id"]   = lookupTables->IdOf("acc_level_one",   comboOne->Text.c_str());
  ending_state["acc_level_two_id"]   = lookupTables->IdOf("acc_level_two",   comboTwo->Text.c_str());
  ending_state["acc_level_three_id"] = lookupTables->IdOf("acc_level_three", comboThree->Text.c_str());
  ending_state["acc_level_four_id"]  = lookupTables->IdOf("acc_level_four",  comboFour->Text.c_str());
  ending_state["account_type_id"]    = lookupTables->IdOf("account_types",   comboAccountType->Text.c_str());
  ending_state["bill_cycle_id"]      = lookupTables->IdOf("month_types",     comboCycle->Text.c_str());

  ending_state["name"] = editName->Text.c_str();
  ending_state["description"] = editDescription->Text.c_str();
  if (editMinimumFiscalDate->Text != "" &&
      Pdate (editMinimumFiscalDate->Text.c_str(), global_mode).IsValidDate() )
  ending_state["min_fiscal_date"] =  Pdate (editMinimumFiscalDate->Text.c_str(), mdy | yy).OutDate(mdy | yyyy);
  // otherwise leave it null like we found it.  honestly.  PutDataInDatabase failures over this sort of thing are ridiculous.

  ending_state["is_budget"] = checkBudget->Checked ? "1" : "0";
  ending_state["is_closed"] = checkClosed->Checked ? "1" : "0";
  ending_state["is_yearly"] = checkYearly->Checked ? "1" : "0";

  ending_state["id"] = initial_state["id"];

  ending_state["balance_account_id"] = initial_state["balance_account_id"];
  ending_state["pending_account_id"] = initial_state["pending_account_id"];
  ending_state["cash_account_id"]    = initial_state["cash_account_id"];
  ending_state["budget_account_id"]  = initial_state["budget_account_id"];
  ending_state["actual_account_id"]  = initial_state["actual_account_id"];
  ending_state["repay_account_id"]   = initial_state["repay_account_id"];

  // if changes have been made to the above six FK's, they were recorded in new_ref_info
  for (map<account_links, acc_info>::iterator i_new = new_ref_info.begin(); i_new != new_ref_info.end(); i_new++)
   ending_state[LinkTypeToString(i_new->first)] = i_new->second.id;

  TformRecordGeneric::GetDataFromScreen();

  return true; // again, not much can go wrong here
}
//---------------------------------------------------------------------------
bool TformRecordAccount::ScreenLock()
{
  // disable panel containing edit boxes
  panelGroup->Enabled = false;
  editDescription->ReadOnly = true;
  editDescription->TabStop = false;
  panelGroupTotals->Enabled = false;
  // btnCreateFK->Enabled = false;
  TformRecordGeneric::ScreenLock();
  return true;
}
//---------------------------------------------------------------------------
bool TformRecordAccount::ScreenUnlock()
{
  // enable panel containing edit boxes
  panelGroup->Enabled = true;
  editDescription->ReadOnly = false;
  editDescription->TabStop = true;
  panelGroupTotals->Enabled = true;
  // btnCreateFK->Enabled = true;
  TformRecordGeneric::ScreenUnlock();
  return true;
}
//---------------------------------------------------------------------------
void __fastcall TformRecordAccount::editAccountChange(TObject *Sender)
{
 // refresh combo boxes based on /*mask*/edit box
 if (ready_for_drop_down_change)
 {
  map<string, string> account_parts = PartsByMask (editAccount);
  ready_for_drop_down_change = false;
  comboOne->Text   = GetOneField(dbDatabase, "select item from acc_level_one where short_item = '"   + account_parts["one"] + "'").c_str();
  comboTwo->Text   = GetOneField(dbDatabase, "select item from acc_level_two where short_item = '"   + account_parts["two"] + "'").c_str();
  comboThree->Text = GetOneField(dbDatabase, "select item from acc_level_three where short_item = '" + account_parts["three"] + "'").c_str();
  comboFour->Text  = GetOneField(dbDatabase, "select item from acc_level_four where short_item = '"  + account_parts["four"] + "'").c_str();
  ready_for_drop_down_change = true;
 }
 return;
}
//---------------------------------------------------------------------------
void __fastcall TformRecordAccount::comboAnyChange(TObject *Sender)
{
 // refresh /*mask*/edit box based on combo boxes (all of them at once)
 if (ready_for_drop_down_change)
 {
  map<string, string> account_parts;
  if (lookupTables->IsInLookUpTable("acc_level_one",   comboOne->Text.c_str()))
   account_parts["one"]   = GetOneField(dbDatabase, "select short_item from acc_level_one where id = "   + lookupTables->IdOf("acc_level_one",   comboOne->Text.c_str()));
  else
   account_parts["one"]   = "000";

  if (lookupTables->IsInLookUpTable("acc_level_two",   comboTwo->Text.c_str()))
   account_parts["two"]   = GetOneField(dbDatabase, "select short_item from acc_level_two where id = "   + lookupTables->IdOf("acc_level_two",   comboTwo->Text.c_str()));
  else
   account_parts["two"]   = "0000000";

  if (lookupTables->IsInLookUpTable("acc_level_three",   comboThree->Text.c_str()))
   account_parts["three"] = GetOneField(dbDatabase, "select short_item from acc_level_three where id = " + lookupTables->IdOf("acc_level_three", comboThree->Text.c_str()));
  else
   account_parts["three"]   = "000";

  if (lookupTables->IsInLookUpTable("acc_level_four",   comboFour->Text.c_str()))
   account_parts["four"]  = GetOneField(dbDatabase, "select short_item from acc_level_four where id = "  + lookupTables->IdOf("acc_level_four",  comboFour->Text.c_str()));
  else
   account_parts["four"]   = "000";

  ready_for_drop_down_change = false;
  MaskByParts (editAccount, account_parts);
  ready_for_drop_down_change = true;
 }
 return;
}
//---------------------------------------------------------------------------
void TformRecordAccount::SetupAccount (string fund, string gl, string dept, string sub_dept, string is_budget, TStaticText * link, string column_name)
{
 // deprecated

 /*
  string account = fund + "-" + gl + "-" + dept + "-" + sub_dept;
  string sql_to_get_account_id = "select id from accounts where name = ";
  sql_to_get_account_id += "\'" + account + "\'" + " and is_budget = " + is_budget;
  string internal_id = GetOneField(dbDatabase, sql_to_get_account_id);
  CreateLink(link, NULL, this, internal_id, "accounts", "");
  initial_state[column_name] = internal_id;
 */
}
//---------------------------------------------------------------------------
void __fastcall TformRecordAccount::btnCreateFKClick(TObject *Sender)
{
  PSay("This functionality is now deprecated.\nPlease use the search buttons to individually modify outside account references.", m_program_error, MB_OK);

  // deprecated code!

  return;

// these comments were at the top:
// does not work for accounts with dept's when working 1-3
// error messages
// check boxes are desired

  /*
  string accounts_to_be_processed;
  set <string> accounts;
  int answer;
  answer = PrometheusMessage("Please choose what relationships you would like set up:\n\n1. Balace Account\n2. Payable/Receivalbe Account\n3. Cash (Asset) Account\n4. Budget\n5. Expense\Revenue\n\nEnter your accounts in the following format: 1,3,5", "Question", mb_query | mb_ok | mb_cancel, &accounts_to_be_processed);
  SplitStringToSet(accounts_to_be_processed, ",", accounts);
  if (answer == idok)
  {
    if (accounts.empty())
    {
      PSay("You did not enter any accounts.", m_missing_data, MB_OK);
      return;
    }
    if (comboOne->Text == "" || comboTwo->Text == "" || comboOne->Text == "<blank>" || comboTwo->Text == "<blank>" || editAccount->Text == "000-0000000-000-000")
    {
      PSay("Please fill in the account number, and try again.", m_missing_data, MB_OK);
      return;
    }
    else
    {
       try
       {
         // get the parts from the account from the edit box
         map<string, string> account_parts = GetAccountParts(editAccount);
         string fund_num = account_parts["one"];
         string gl_num = account_parts["two"];
         string dept = account_parts["three"];
         string sub_dept = account_parts["four"];

         // set up accounts balance account
         if (accounts.find("1") != accounts.end())
         {
           string balance = GetOneField(dbDatabase, "select short_item from acc_level_two where acc_level_two.item = \'Fund Balance\'");
           SetupAccount (fund_num, balance, "000", sub_dept, "0", staticBalance, "balance_account_id");
         }
         // set up accounts receivable / accounts payable account
         if (accounts.find("2") != accounts.end())
         {
           // obtain the acceptable revenue/expense series
           set <string> revenue;
           set <string> expense;
           string response;
           PrometheusMessage("Please enter the first number of the general ledget accounts for revenue seperated by commas:", "Query", mb_query | mb_ok, &response);
           SplitStringToSet(response, ",", revenue);
           PrometheusMessage("Please enter the first number of the general ledget accounts for expense seperated by commas:", "Query", mb_query | mb_ok, &response);
           SplitStringToSet(response, ",", expense);
           string ar_pending = GetOneField(dbDatabase, "select short_item from acc_level_two where acc_level_two.item = \'Accounts Receivable\'");
           string ap_pending = GetOneField(dbDatabase, "select short_item from acc_level_two where acc_level_two.item = \'Accounts Payable\'");
           if (expense.empty() || revenue.empty())
           {
             PSay("You did not enter anything for either revenue or expense", m_missing_data, MB_OK);
             return;
           }
           // based on acceptable revenue/expense series numbers determine of the pending account will be for accounts receivable or accounts payable
           if (revenue.find(gl_num.substr(0,1)) != revenue.end())
           {
              SetupAccount (fund_num, ar_pending, "000", sub_dept, "0", staticPending, "pending_account_id");
           }
           else if (expense.find(gl_num.substr(0,1)) != expense.end())
           {
              SetupAccount (fund_num, ap_pending, "000", sub_dept, "0", staticPending, "pending_account_id");
           }
           else
           {
             PSay("You have confused the program.\nIf the general ledger does not start with one of the numbers you specifed then the program does not know whether to make the pending account receivable or payable.", m_warning, MB_OK);
             return;
           }
         }
         // set up accounts cash account
         if (accounts.find("3") != accounts.end())
         {
           string cash;
           if (fund_num == "003")
           {
             cash = GetOneField(dbDatabase, "select short_item from acc_level_two where acc_level_two.item = \'IHS Cash in Bank\'");
           }
           else if (fund_num == "024" || fund_num == "042" || fund_num == "064" || fund_num == "071")
           {
             cash = GetOneField(dbDatabase, "select short_item from acc_level_two where acc_level_two.item = \'Cash In Bank\'");
           }
           else
           {
              PSay("The program is unsure as to which cash account to use.", m_warning, MB_OK);
           }
           SetupAccount (fund_num, cash, "000", sub_dept, "0", staticCash, "cash_account_id");
         }
         // set up accounts budget account
         if (accounts.find("4") != accounts.end())
         {
           SetupAccount (fund_num, gl_num, dept, sub_dept, "1", staticBudget, "budget_account_id");
         }
         // set up accounts actual account
         if (accounts.find("5") != accounts.end())
         {
           SetupAccount (fund_num, gl_num, dept, sub_dept, "0", staticActual, "actual_account_id");
         }
       }
       catch(...)
       {
         PSay("The program read the users mind, but unforutnately the database can't follow directions.", m_database_error, MB_OK);
       }
    }
 }
 */
}
//---------------------------------------------------------------------------
void __fastcall TformRecordAccount::buttonSearchBalanceClick(TObject *Sender)
{
 // balance
 vector<string> results;
 FORM_TRY
  TformSearchAccount * find_acc = new TformSearchAccount(NULL, &results);
  find_acc->ShowTaskModal(this);
 FORM_CATCH
 if (!results.empty())
 {
  // new_ref_id["balance_account_id"] = results.front();
  new_ref_info[balance_ref] = InfoById(results.front());
  labelTotalBalance->Caption = GetCurrentAccountBalance(results.front()).c_str();
  CreateLink(staticBalance, buttonSearchBalance,  this, results.front(), "accounts", "", true);
 }
}
//---------------------------------------------------------------------------
void __fastcall TformRecordAccount::buttonSearchPendingClick(TObject *Sender)
{
 // pending
 vector<string> results;
 FORM_TRY
  TformSearchAccount * find_acc = new TformSearchAccount(NULL, &results);
  find_acc->ShowTaskModal(this);
 FORM_CATCH
 if (!results.empty())
 {
  // new_ref_id["pending_account_id"] = results.front();
  new_ref_info[pending_ref] = InfoById(results.front());
  labelTotalPending->Caption = GetCurrentAccountBalance(results.front()).c_str();
  CreateLink(staticPending, buttonSearchPending,  this, results.front(), "accounts", "", true);
 }
}
//---------------------------------------------------------------------------
void __fastcall TformRecordAccount::buttonSearchCashClick(TObject *Sender)
{
 // cash
 vector<string> results;
 FORM_TRY
  TformSearchAccount * find_acc = new TformSearchAccount(NULL, &results);
  find_acc->ShowTaskModal(this);
 FORM_CATCH
 if (!results.empty())
 {
  // new_ref_id["cash_account_id"] = results.front();
  new_ref_info[cash_ref] = InfoById(results.front());
  labelTotalCash->Caption    = GetCurrentAccountBalance(results.front()).c_str();
  CreateLink(staticCash,    buttonSearchCash,     this, results.front(),    "accounts", "", true);
 }
}
//---------------------------------------------------------------------------
void __fastcall TformRecordAccount::buttonSearchBudgetClick(TObject *Sender)
{
 // budget
 vector<string> results;
 FORM_TRY
  TformSearchAccount * find_acc = new TformSearchAccount(NULL, &results);
  find_acc->ShowTaskModal(this);
 FORM_CATCH
 if (!results.empty())
 {
  // new_ref_id["budget_account_id"] = results.front();
  new_ref_info[budget_ref] = InfoById(results.front());
  labelTotalBudget->Caption  = GetCurrentAccountBalance(results.front()).c_str();
  CreateLink(staticBudget,  buttonSearchBudget,   this, results.front(),  "accounts", "", true);
 }
}
//---------------------------------------------------------------------------
void __fastcall TformRecordAccount::buttonSearchActualClick(TObject *Sender)
{
 // actual
 vector<string> results;
 FORM_TRY
  TformSearchAccount * find_acc = new TformSearchAccount(NULL, &results);
  find_acc->ShowTaskModal(this);
 FORM_CATCH
 if (!results.empty())
 {
  // new_ref_id["actual_account_id"] = results.front();
  new_ref_info[actual_ref] = InfoById(results.front());
  labelTotalActual->Caption  = GetCurrentAccountBalance(results.front()).c_str();
  CreateLink(staticActual,  buttonSearchActual,   this, results.front(),  "accounts", "", true);
 }
}
//---------------------------------------------------------------------------
void __fastcall TformRecordAccount::buttonSearchRepayClick(TObject *Sender)
{
 // repay
 vector<string> results;
 FORM_TRY
  TformSearchAccount * find_acc = new TformSearchAccount(NULL, &results);
  find_acc->ShowTaskModal(this);
 FORM_CATCH
 if (!results.empty())
 {
  // new_ref_id["repay_account_id"] = results.front();
  new_ref_info[repay_ref] = InfoById(results.front());
  labelTotalRepay->Caption   = GetCurrentAccountBalance(results.front()).c_str();
  CreateLink(staticRepay,   buttonSearchRepay,   this, results.front(),   "accounts", "", true);
 }
}
//---------------------------------------------------------------------------
void TformRecordAccount::RefreshLinkStatus ()
{
 set<string> useful_refs = GetRequiredAccountReferences(comboAccountType->Text.c_str(), checkBudget->Checked);
 // sorry about redundancy of code, but apparently "a = b = c" doesn't work with properites. grrrrrr.
 if (useful_refs.find("balance_account_id") != useful_refs.end())
 {
  staticBalance->Enabled = true;
  buttonSearchBalance->Enabled = true;
  if (initial_state["is_budget"] != "0")
   labelBalance->Hint = "equity-budget account, for creating and destroying budgets";
  else
   labelBalance->Hint = "equity account, for year-end closing of revenues and expenses";
 } else
 {
  staticBalance->Enabled = false;
  buttonSearchBalance->Enabled = false;
  labelBalance->Hint = "not used";
 }

 if (useful_refs.find("pending_account_id") != useful_refs.end())
 {
  staticPending->Enabled = true;
  buttonSearchPending->Enabled = true;
  if (comboAccountType->Text == "Expense")
   labelPending->Hint = "payable account used when recording expenses";
  else if (comboAccountType->Text == "Revenue")
   labelPending->Hint = "receivable account used when recording revenues";
  else
   labelPending->Hint = "unknown -- please ask a programmer";
 } else
 {
  staticPending->Enabled = false;
  buttonSearchPending->Enabled = false;
  labelPending->Hint = "not used";
 }

 if (useful_refs.find("cash_account_id") != useful_refs.end())
 {
  staticCash->Enabled    = true;
  buttonSearchCash->Enabled    = true;
  if (comboAccountType->Text == "Expense")
   labelCash->Hint = "cash or asset account used when recording expenses";
  else if (comboAccountType->Text == "Revenue")
   labelCash->Hint = "cash or asset account used when recording revenues";
  else if (comboAccountType->Text == "Payable")
   labelCash->Hint = "cash or asset account used when converting payables";
  else if (comboAccountType->Text == "Receivable")
   labelCash->Hint = "cash or asset account used when converting receivables";
  else
   labelCash->Hint = "unknown -- please ask a programmer";
 } else
 {
  staticCash->Enabled    = false;
  buttonSearchCash->Enabled    = false;
  labelCash->Hint = "not used";
 }

 if (useful_refs.find("budget_account_id") != useful_refs.end())
 {
  staticBudget->Enabled  = true;
  buttonSearchBudget->Enabled  = true;
  labelBudget->Hint = "budget account that records budget-modifying transactions";
 } else
 {
  staticBudget->Enabled  = false;
  buttonSearchBudget->Enabled  = false;
  labelBudget->Hint = "not used";
 }

 if (useful_refs.find("actual_account_id") != useful_refs.end())
 {
  staticActual->Enabled  = true;
  buttonSearchActual->Enabled  = true;
  labelActual->Hint = (LowercaseString(comboAccountType->Text.c_str()) + " account for which this account records budget-modifying transactions").c_str();
 } else
 {
  staticActual->Enabled  = false;
  buttonSearchActual->Enabled  = false;
  labelActual->Hint = "not used";
 }

 if (useful_refs.find("repay_account_id") != useful_refs.end())
 {
  staticRepay->Enabled   = true;
  buttonSearchRepay->Enabled   = true;
  if (comboAccountType->Text == "Payable")
   labelRepay->Hint = "receivable account used when employees are asked to reimburse us for these payables";
  if (comboAccountType->Text == "Receivable")
   labelRepay->Hint = "payable account for which employees owe money on this account, for reimbursements";
 } else
 {
  staticRepay->Enabled   = false;
  buttonSearchRepay->Enabled   = false;
  labelRepay->Hint = "not used";
 }
 return;
}
//---------------------------------------------------------------------------
void __fastcall TformRecordAccount::checkBudgetClick(TObject *Sender)
{
 RefreshLinkStatus();
}
//---------------------------------------------------------------------------
void __fastcall TformRecordAccount::comboAccountTypeChange(TObject *Sender)
{
 RefreshLinkStatus();
}
//---------------------------------------------------------------------------
string TformRecordAccount::GetCurrentAccountBalance (string p_account_id)
{
 if (!IsInt(p_account_id))
 {
  return "unknown";
 } else
 if (p_account_id == "0")
 {
  return "0.00";
 }
 else
 {
  acc_info about_account = InfoById (p_account_id);
  if (!about_account.IsFound())
  {
   return "not found";
  } else
  {
   string result;
   if (about_account.IsDebitPositive())
    result = GetOneField(dbDatabase, "select sum(debit_amount - credit_amount) from transactions where account_id = " + p_account_id);
   else
    result = GetOneField(dbDatabase, "select sum(credit_amount - debit_amount) from transactions where account_id = " + p_account_id);
   if (result.empty())
    return "0.00";
   else
    return StringToAmount(result);
  }
 }
}
//---------------------------------------------------------------------------
bool TformRecordAccount::NotNewAnymore()
{
  // jhaley 20100910: Use transaction summarizer to display summary instead of details; details moved to own form.
  transactions_summary.SetSQLStatement("select * from transactions where account_id = " + initial_state["id"]);
  transactions_summary.SetRecord("accounts", initial_state["id"], "account_id", RefreshHandler(&(TformRecordAccount::AtlasRefreshListView)));

  if (optionsPermissions["table.accounts"]["view"] == "true")
  {
   string sql = "select accounts.name, acc_level_one.item, acc_level_two.item, acc_level_three.item, acc_level_four.item, "
                "accounts.is_budget, accounts.is_closed, accounts.is_yearly, month_types.item, account_types.item, accounts.current_id, accounts.id "
                "from accounts inner join month_types on month_types.id = accounts.bill_cycle_id inner join account_types on account_types.id = accounts.account_type_id "
                "inner join acc_level_one on acc_level_one.id = accounts.acc_level_one_id "
                "inner join acc_level_two on acc_level_two.id = accounts.acc_level_two_id "
                "inner join acc_level_three on acc_level_three.id = accounts.acc_level_three_id "
                "inner join acc_level_four on acc_level_four.id = accounts.acc_level_four_id "
                "where accounts.balance_account_id = " + initial_state["id"] +
                " or accounts.pending_account_id = " + initial_state["id"] +
                " or accounts.cash_account_id = " + initial_state["id"] +
                " or accounts.budget_account_id = " + initial_state["id"] +
                " or accounts.actual_account_id = " + initial_state["id"] +
                " or accounts.repay_account_id = " + initial_state["id"];
   accounts_filter.SetSQLStatement(sql);
   accounts_filter.RefreshDataSet();
   accounts_filter.SetAddEdit(this, NULL, EditRecord(&(TformRecordAccount::EditAccount)));
   accounts_request = atlas->RequestNotificationWhen("accounts.", initial_state["id"], rel_e, as_int, this,
                      RefreshHandler(&(TformRecordAccount::AtlasRefreshListView)));
   // is this a hack? NAW ...
   // it'll probably refresh just a bit too much, but ... it's okay. it's rare enough anyway.
  }

  staticTransactionDetails->Enabled = true;
  staticBudgetReport->Enabled       = true;
  TformRecordGeneric::NotNewAnymore();
  return true;
}
//---------------------------------------------------------------------------
bool TformRecordAccount::IsNewAgain()
{
  // jhaley 20100910: Use transaction summarizer
  transactions_summary.Clear();

  accounts_filter.SetSQLStatement(string(""));
  accounts_filter.RefreshDataSet();
  accounts_filter.SetAddEdit(this, NULL, NULL);
  TformRecordGeneric::IsNewAgain();
  return true;
}
//---------------------------------------------------------------------------
void TformRecordAccount::AtlasRefreshListView(int request_num, string matched_value)
{
  if (request_num == accounts_request)
    accounts_filter.RefreshDataSet();
  else
    transactions_summary.AtlasRefresh(request_num, matched_value); // jhaley: try the summarizer
}
//---------------------------------------------------------------------------
void TformRecordAccount::EditTransaction()
{
  /*
  map<string, string> temp_map;
  temp_map["id"] = transactions_filter.GetSelectedRecordID();
  FORM_TRY
    TformRecordTransaction * record_form = new TformRecordTransaction(NULL, temp_map);
    record_form->Show();
  FORM_CATCH
  */
}
//---------------------------------------------------------------------------
void TformRecordAccount::EditAccount()
{
  map<string, string> temp_map;
  temp_map["id"] = accounts_filter.GetSelectedRecordID();
  FORM_TRY
    TformRecordAccount * record_form = new TformRecordAccount(NULL, temp_map);
    record_form->Show();
  FORM_CATCH
}
//---------------------------------------------------------------------------
void __fastcall TformRecordAccount::staticTransactionDetailsClick(TObject *Sender)
{
  // jhaley 20100910: Launch transaction details form when link is clicked
  if (optionsPermissions["table.transactions"]["view"] == "true")
  {
    string sql = "select tax_entities.name, transactions.effective_date, transactions.fiscal_date, transactions.note, "
                 "transactions.debit_amount, transactions.credit_amount, transactions.transaction_number, "
                 "purchase_reqs.purchase_req_number, checks.check_number, transactions.current_id, transactions.id "
                 "from transactions "
                 "inner join tax_entities on transactions.tax_entity_id = tax_entities.id "
                 "inner join bills on transactions.bill_id = bills.id "
                 "inner join purchase_reqs on bills.purchase_req_id = purchase_reqs.id "
                 "inner join checks on purchase_reqs.check_id = checks.id "
                 "where transactions.account_id = " + initial_state["id"];

    TformSpecialTransactionDetails *details = NULL;
    FORM_TRY
      details = new TformSpecialTransactionDetails(NULL, initial_state["id"], "account_id");
    FORM_CATCH
    details->Filter->AddColumnSpec("Tax Entity",  "tax_entities",  as_text,   true,  120, rawString);
    details->Filter->AddColumnSpec("Eff. Date",   "transactions",  as_date,   true,   75);
    details->Filter->AddColumnSpec("Fisc. Date",  "transactions",  as_date,   true,   75);
    details->Filter->AddColumnSpec("Note",        "transactions",  as_text,   false, 120, sentenceString);
    details->Filter->AddColumnSpec("Debit",       "transactions",  as_double, true,   75, nonzeroAmountString);
    details->Filter->AddColumnSpec("Credit",      "transactions",  as_double, true,   75, nonzeroAmountString);
    details->Filter->AddColumnSpec("Trans #",     "transactions",  as_text,   true,   75);
    details->Filter->AddColumnSpec("PR #",        "purchase_reqs", as_text,   true,  100);
    details->Filter->AddColumnSpec("Check #",     "checks",        as_text,   true,  100);
    details->Filter->AddColumnSpec("Current Id",  "current_id",    as_int,    false,  80);
    details->Filter->AddColumnSpec("Internal Id", "internal_id",   as_int,    false,  80);
    details->Filter->FetchPreferences("recordaccount.transactions_filter");
    details->Execute(sql);
    details->Show();
  }
}
//---------------------------------------------------------------------------
void __fastcall TformRecordAccount::staticBudgetReportClick(TObject *Sender)
{
 // budget report
 FORM_TRY

  TformLaunchBudgetReport * rep = new TformLaunchBudgetReport(NULL);

  Pdate today_date(Date());
  Pdate begin_date = today_date;
  begin_date.day   = 1;

  if (StringToInt(initial_state["bill_cycle_id"]) > begin_date.month)
   begin_date.mod(-1, in_years);
  begin_date.month = StringToInt(initial_state["bill_cycle_id"]);

  Pdate end_date = begin_date;
  end_date.mod(1, in_years);
  end_date.mod(-1, in_days);

  rep->dateBeginDate->Date = begin_date.OutDate();
  rep->dateEndDate->Date   = end_date.OutDate();
  MaskById (rep->editAccountPattern, initial_state["id"]);
  rep->checkExactMatch->Checked = true;
  rep->comboCycle->Text = lookupTables->ValueOf("month_types", initial_state["bill_cycle_id"]).c_str();
  if (initial_state["account_type_id"] == lookupTables->IdOf("account_types", "Revenue"))
   rep->radioRevenue->Checked = true;
  rep->checkIgnoreDisabled->Checked = true;

  rep->Show();

 FORM_CATCH
}
//---------------------------------------------------------------------------
void TformRecordAccount::Do(map<string, string> & p_parameters)
{
 FORM_TRY
  TformRecordAccount * z = new TformRecordAccount(NULL, p_parameters);
  z->Show();
 FORM_CATCH
}
//---------------------------------------------------------------------------
void __fastcall TformRecordAccount::FormClose(TObject *Sender, TCloseAction &Action)
{
  // jhaley: shut down the transaction summary object
  transactions_summary.Close(Action);
  TformRecordGeneric::FormClose(Sender, Action);
}
//---------------------------------------------------------------------------
// jhaley 20100910: Added checkbox to control transaction summary grouping
void __fastcall TformRecordAccount::checkTransactionShowNameClick(TObject *Sender)
{
  transactions_summary.SetShowName(checkTransactionShowName->Checked); // set state
  transactions_summary.CheckBoxClicked();                              // update it!        
}
//---------------------------------------------------------------------------

