//---------------------------------------------------------------------------

#include <vcl.h>
#pragma hdrstop

#include "codeRecordActivity.h"
#include "codeSearchLogsheet.h"
//---------------------------------------------------------------------------
#pragma package(smart_init)
#pragma link "codeRecordGeneric"
#pragma resource "*.dfm"
TformRecordActivity *formRecordActivity;
//---------------------------------------------------------------------------
__fastcall TformRecordActivity::TformRecordActivity(TComponent* Owner, map<string, string> & p_parameters)
        : TformRecordGeneric(Owner, p_parameters, string("activities"))
{
 can_modify = true;
 Initialize();
 current_person_id = initial_state["person_id"];
 current_logsheet_person_id = initial_state["person_id"];

 if (initial_state["event_activity_id"] == "" )
   current_event_activity_id = "0";
 else
   current_event_activity_id = initial_state["event_activity_id"];
 if (current_event_activity_id != "0")
   can_modify = false; // Activities created for Events must be modified
                       // from recordEvent
}
//---------------------------------------------------------------------------
bool TformRecordActivity::ScreenStart()
{
 // register combo box
 lookupTables->RegisterComboBox("activity_code_types",cboActivity);
 lookupTables->RegisterComboBox("intensity_types",cboIntensity);
 TformRecordGeneric::ScreenStart();
 return true;
}
//---------------------------------------------------------------------------
bool TformRecordActivity::ScreenStop()
{
 // unregister combo box

 lookupTables->UnregisterComboBox("activity_code_types",cboActivity);
 lookupTables->UnregisterComboBox("intensity_types",cboIntensity);

 TformRecordGeneric::ScreenStop();
 return true;
}
//---------------------------------------------------------------------------
bool TformRecordActivity::IsValidOnScreen()
{
 if (!lookupTables->IsInLookUpTable ("activity_code_types", cboActivity->Text.c_str()))
 {
  PSay("Please enter a valid activity", m_invalid_data, MB_OK);
  cboActivity->SetFocus();
  return false;
 }
 else if (!lookupTables->IsInLookUpTable ("intensity_types", cboIntensity->Text.c_str()))
 {
  PSay("Please enter a valid intensity", m_invalid_data, MB_OK);
  cboIntensity->SetFocus();
  return false;
 }
 else if (! IsFloat(editTime->Text.c_str()))
 {
  PSay("Please enter a valid time", m_invalid_data, MB_OK);
  editTime->SetFocus();
  return false;
 }
 else if (! IsFloat(editDistance->Text.c_str()))
 {
  PSay("Please enter a valid distance", m_invalid_data, MB_OK);
  editDistance->SetFocus();
  return false;
 }
 else if (! IsFloat(editSteps->Text.c_str()))
 {
  PSay("Please enter a valid number of steps", m_invalid_data, MB_OK);
  editSteps->SetFocus();
  return false;
 }
 else if ((initial_state["person_id"] == "") || (initial_state["person_id"] == "0"))
 {
  PSay("There is no person attached to this record", m_missing_data, MB_OK);
  return false;
 }
 else if(current_person_id != current_logsheet_person_id && GetValidLinkId(txtLogsheet_ID) != "")
 {
  PSay("The selected logsheet does not belong to the correct person", m_invalid_data, MB_OK);
  return false;
 }
 else
   return true;
}
//---------------------------------------------------------------------------
bool TformRecordActivity::PutDataOnScreen()
{
 if (initial_state["date_of_activity"] != "")
   dateActivity->Date = Pdate(initial_state["date_of_activity"], mdy | yyyy).OutDate();
 else
   dateActivity->Date = Date();
 cboActivity->Text = PrettyString(lookupTables->ValueOf("activity_code_types", initial_state["activity_code_id"])).c_str();
 cboIntensity->Text = PrettyString(lookupTables->ValueOf("intensity_types",initial_state["intensity_id"])).c_str();
 editTime->Text = initial_state["time_spent"].c_str();
 editDistance->Text = initial_state["distance"].c_str();
 editSteps->Text = initial_state["steps"].c_str();

 CreateLink(txtPerson, NULL, this, initial_state["person_id"], "people");
 CreateLink(txtLogsheet_ID, NULL, this, initial_state["activity_logsheet_id"], "activity_logsheets");
 TformRecordGeneric::PutDataOnScreen();
 return true;
}
//---------------------------------------------------------------------------
bool TformRecordActivity::GetDataFromScreen()
{
 ending_state["date_of_activity"] = Pdate(dateActivity->Date).OutDate(mdy | yyyy);
 ending_state["activity_code_id"] = lookupTables->IdOf("activity_code_types", cboActivity->Text.c_str());
 ending_state["intensity_id"] = lookupTables->IdOf("intensity_types", cboIntensity->Text.c_str());
 ending_state["time_spent"] = StripAllExcept(editTime->Text.c_str(), floats);
 ending_state["distance"] = StripAllExcept(editDistance->Text.c_str(), floats);
 ending_state["steps"] = StripAllExcept(editSteps->Text.c_str(), floats);
 ending_state["person_id"] = initial_state["person_id"];
 ending_state["id"] = initial_state["id"];
 string current_logsheet_id = GetValidLinkId(txtLogsheet_ID);
 if (current_logsheet_id != "")
   ending_state["activity_logsheet_id"] = current_logsheet_id;
 else
   ending_state["activity_logsheet_id"] = "0";
 ending_state["event_activity_id"] = current_event_activity_id;
 TformRecordGeneric::GetDataFromScreen();
 return true;
}
//---------------------------------------------------------------------------
bool TformRecordActivity::ScreenLock()
{
 panelGroupBox->Enabled = false;
 TformRecordGeneric::ScreenLock();
 return true;
}
//---------------------------------------------------------------------------
bool TformRecordActivity::ScreenUnlock()
{
 panelGroupBox->Enabled = true;
 TformRecordGeneric::ScreenUnlock();
 return true;
}
//---------------------------------------------------------------------------
void __fastcall TformRecordActivity::btnLogsheetSearchClick(
      TObject *Sender)
{
  // create and show new person search form
  vector<string> return_vector;
  FORM_TRY
    TformSearchLogsheet * formLogsheetSearch = new TformSearchLogsheet(NULL,&return_vector, initial_state["person_id"]);
    formLogsheetSearch->ShowTaskModal(this);
  FORM_CATCH
  if (! return_vector.empty())
    CreateLink(txtLogsheet_ID, btnLogsheetSearch, this, return_vector[0], "activity_logsheets");
}
//---------------------------------------------------------------------------
void __fastcall TformRecordActivity::buttonAClick(TObject *Sender)
{
 if (can_modify)
   TformRecordGeneric::buttonAClick(this);
 else
   PSay("This activity was created by an Event and must be modified from the event screen", m_message, MB_OK);
}
//---------------------------------------------------------------------------
void TformRecordActivity::Do(map<string, string> & p_parameters)
{
 FORM_TRY
  TformRecordActivity * z = new TformRecordActivity(NULL, p_parameters);
  z->Show();
 FORM_CATCH
}
//---------------------------------------------------------------------------

