//---------------------------------------------------------------------------

#include <vcl.h>
#pragma hdrstop

#include "codeRecordAffiliations.h"
//---------------------------------------------------------------------------
#pragma package(smart_init)
#pragma link "codeRecordGeneric"
#pragma link "codeRecordGeneric"
#pragma resource "*.dfm"
TformRecordAffiliation *formRecordAffiliation;
//---------------------------------------------------------------------------
__fastcall TformRecordAffiliation::TformRecordAffiliation(TComponent* Owner, map<string,string> p_parameters)
        : TformRecordGeneric(Owner, p_parameters, string("affiliations"))
{
 Initialize();
}
//---------------------------------------------------------------------------
bool TformRecordAffiliation::ScreenStart()
{
  // set the title bar color
  panelSelectArea->Color = title_bar_color;

  // register combo box
  lookupTables->RegisterComboBox("tribe_types", cboTribe);
  TformRecordGeneric::ScreenStart();
  return true;
}
//---------------------------------------------------------------------------
bool TformRecordAffiliation::ScreenStop()
{
  // unregister combo box
  lookupTables->UnregisterComboBox("tribe_types", cboTribe);

  TformRecordGeneric::ScreenStop();
  return true;
}
//---------------------------------------------------------------------------
bool TformRecordAffiliation::IsValidOnScreen()
{
  // check to make sure that the values entered in the combo box
  // corrospond to values in that look up table.
  // check to make sure the dates entered are valid
  // give the user a message box and prompt to re-enter if incorrect
  // make sure there is a link to a person, although there's not much the user
  // can do to fix that if there's not
  if (! lookupTables->IsInLookUpTable("tribe_types", cboTribe->Text.c_str()))
  {
    PSay("Please enter a valid tribe", m_invalid_data, MB_OK);
    cboTribe->SetFocus();
    return false;
  }
  else if (! Pdate (editJoin->Text.c_str(), mdy | yy).IsValidDate() && ! (editJoin->Text == ""))
  {
    PSay("Please enter a valid join date", m_invalid_data, MB_OK);
    editJoin->SetFocus();
    return false;
  }
  else if (! Pdate(editRel->Text.c_str(), mdy | yy).IsValidDate() && ! (editRel->Text == ""))
  {
    PSay("Please enter a valid relinquish date", m_invalid_data, MB_OK);
    editRel->SetFocus();
    return false;
  }
  else if (editJoin->Text != "" && editRel->Text != "" && Pdate(editJoin->Text.c_str(), mdy | yy) > Pdate(editRel->Text.c_str(), mdy | yy))
  {
   PSay("Please enter a join date that comes before the relinquish date;\nsomeone has been overwriting the original join date and keeping the relinquish date, when a person re-joins.\nDon't. See note above for proper use.", m_invalid_data, MB_OK);
   editJoin->SetFocus();
   return false;
  }
  else if ((initial_state["person_id"] == "") || (initial_state["person_id"] == "0"))
  {
    PSay("There is no person associated with this record", m_invalid_data, MB_OK);
    return false;
  }
  else
    return true;
}
//---------------------------------------------------------------------------

bool TformRecordAffiliation::PutDataOnScreen()
{
  // put the values on the form from the map initial_state
  // values may have come from GetDataFromDatabase if it's an existing record,
  // or values may have been passed to the form in this map if it's a new record

  editCDIB->Text = LowercaseString(initial_state["cdib"]).c_str();
  cboTribe->Text = PrettyString(lookupTables->ValueOf("tribe_types", initial_state["tribe_id"])).c_str();
  editJoin->Text = initial_state["join_date"].c_str();
  editRel->Text = initial_state["relinquish_date"].c_str();

  // get the person name associated with the person id that was either selected
  // from the database if it's an existing record, or passed to the form if
  // it's a new record. display the name as a link to the person record.

  CreateLink(lblName, NULL, this, initial_state["person_id"], "people");


  TformRecordGeneric::PutDataOnScreen();  
  return true; // what can really go wrong?
}
//---------------------------------------------------------------------------
bool TformRecordAffiliation::GetDataFromScreen()
{
  // place the values currently on the screen into
  // the map that will be used to update or insert the record
  // the parent form will call the insert or update function, and get the id
  // number if necessary and take care of current_id
  ending_state["cdib"] = editCDIB->Text.c_str();
  ending_state["tribe_id"] = lookupTables->IdOf("tribe_types", cboTribe->Text.c_str());
  if (editJoin->Text != "")
    ending_state["join_date"] =  Pdate (editJoin->Text.c_str(), mdy | yy).OutDate(mdy | yyyy);
  else
    ending_state["join_date"] = "";
  if (editRel->Text != "")
    ending_state["relinquish_date"] = Pdate (editRel->Text.c_str(), mdy | yy).OutDate(mdy | yyyy);
  else
    ending_state["relinquish_date"] = "";
  ending_state["person_id"] = initial_state["person_id"];
  ending_state["id"] = initial_state["id"];

  TformRecordGeneric::GetDataFromScreen();

  return true; // again, not much can go wrong here
}
//---------------------------------------------------------------------------
bool TformRecordAffiliation::ScreenLock()
{
  // disable panel containing edit boxes
  panelGroup->Enabled = false;
  TformRecordGeneric::ScreenLock();
  return true;
}
//---------------------------------------------------------------------------
bool TformRecordAffiliation::ScreenUnlock()
{
  // enable panel containing edit boxes
  panelGroup->Enabled = true;
  TformRecordGeneric::ScreenUnlock();
  return true;
}
//---------------------------------------------------------------------------
void TformRecordAffiliation::Do(map<string, string> & p_parameters)
{
 FORM_TRY
  TformRecordAffiliation * z = new TformRecordAffiliation(NULL, p_parameters);
  z->Show();
 FORM_CATCH
}
//---------------------------------------------------------------------------

