//---------------------------------------------------------------------------

#include <vcl.h>
#pragma hdrstop

#include "codeRecordBehOutsideTreatment.h"
#include "codeSearchTaxEntity.h"
//---------------------------------------------------------------------------
#pragma package(smart_init)
#pragma link "codeRecordGeneric"
#pragma resource "*.dfm"
TformRecordBehOutsideTreatment *formRecordBehOutsideTreatment;
//---------------------------------------------------------------------------
__fastcall TformRecordBehOutsideTreatment::TformRecordBehOutsideTreatment(TComponent* Owner, map<string,string> & p_parameters)
        : TformRecordGeneric(Owner, p_parameters, string("beh_out_treatment"))
{
  Initialize();
}
//---------------------------------------------------------------------------
bool TformRecordBehOutsideTreatment::ScreenStart()
{
  editNotes->Text = "";
  editMedications->Text = "";

  lookupTables->RegisterComboBox("patient_types", cboPatientType);

  TformRecordGeneric::ScreenStart();
  return true;
}
//---------------------------------------------------------------------------
bool TformRecordBehOutsideTreatment::ScreenStop()
{
  // unregister combo box
  lookupTables->UnregisterComboBox("patient_types", cboPatientType);

  TformRecordGeneric::ScreenStop();
  return true;
}
//---------------------------------------------------------------------------
bool TformRecordBehOutsideTreatment::IsValidOnScreen()
{
  // check to make sure that the values entered in the combo boxe
  // corrospond to values in that look up table.
  // give the user a message box and prompt to re-enter if incorrect

  if (! lookupTables->IsInLookUpTable("patient_types", cboPatientType->Text.c_str()))
  {
    PSay("Please enter a valid patient type", m_invalid_data, MB_OK);
    cboPatientType->SetFocus();
    return false;
  }
  else if (! Pdate (editBeginDate->Text.c_str(), mdy | yy).IsValidDate() && ! (editBeginDate->Text == ""))
  {
    PSay("Please enter a valid begin date", m_invalid_data, MB_OK);
    editBeginDate->SetFocus();
    return false;
  }
  else if (! Pdate (editEndDate->Text.c_str(), mdy | yy).IsValidDate() && ! (editEndDate->Text == ""))
  {
    PSay("Please enter a valid end date", m_invalid_data, MB_OK);
    editEndDate->SetFocus();
    return false;
  }
  else if (GetValidLinkId(txtFacility) == "")
  {
    PSay("Please attach a facility to this referral", m_missing_data, MB_OK);
    btnFacilitySearch->SetFocus();
    return false;
  }  
  else if ((initial_state["person_id"] == "") || (initial_state["person_id"] == "0"))
  {
    PSay("There is no client attached to this record", m_missing_data, MB_OK);
    return false;
  }
  else
    return true;
}
//---------------------------------------------------------------------------

bool TformRecordBehOutsideTreatment::PutDataOnScreen()
{
  // put the values on the form from the map initial_state
  // values may have come from GetDataFromDatabase if it's an existing record,
  // or values may have been passed to the form in this map if it's a new record

  cboPatientType->Text = PrettyString(lookupTables->ValueOf("patient_types", initial_state["patient_id"])).c_str();
  editBeginDate->Text = initial_state["begin_date"].c_str();
  editEndDate->Text = initial_state["end_date"].c_str();
  editDoctor->Text = PrettyString(initial_state["doctor_name"]).c_str();
  editNotes->Text = SentenceString(initial_state["notes"]).c_str();
  editMedications->Text = SentenceString(initial_state["medications"]).c_str();

  // get the person name associated with the person id that was either selected
  // from the database if it's an existing record, or passed to the form if
  // it's a new record. display the name as a link to the person record.
  CreateLink(txtClient, NULL, this, initial_state["person_id"], "people");
  CreateLink(txtFacility, btnFacilitySearch, this, initial_state["tax_entity_id"], "tax_entities");
  TformRecordGeneric::PutDataOnScreen();
  return true;
}
//---------------------------------------------------------------------------
bool TformRecordBehOutsideTreatment::GetDataFromScreen()
{
  // place the values currently on the screen into
  // the map that will be used to update or insert the record
  // the parent form will call the insert or update function, and get the id
  // number if necessary and take care of current_id

  ending_state["id"] = initial_state["id"];
  ending_state["person_id"] = initial_state["person_id"];
  ending_state["tax_entity_id"] = GetValidLinkId(txtFacility);
  ending_state["patient_id"] = lookupTables->IdOf("patient_types", cboPatientType->Text.c_str());
  if (editBeginDate->Text != "")
    ending_state["begin_date"] =  Pdate (editBeginDate->Text.c_str(), mdy | yy).OutDate(mdy | yyyy);
  else
    ending_state["begin_date"] = "";
  if (editEndDate->Text != "")
    ending_state["end_date"] =  Pdate (editEndDate->Text.c_str(), mdy | yy).OutDate(mdy | yyyy);
  else
    ending_state["end_date"] = "";
  ending_state["doctor_name"] = editDoctor->Text.c_str();
  ending_state["notes"] = editNotes->Text.c_str();
  ending_state["medications"] = editMedications->Text.c_str();  

  TformRecordGeneric::GetDataFromScreen();

  return true;
}
//---------------------------------------------------------------------------
bool TformRecordBehOutsideTreatment::ScreenLock()
{
  panelGroup->Enabled = false;
  editMedications->ReadOnly = true;
  editMedications->TabStop = false;
  editNotes->ReadOnly = true;
  editNotes->TabStop = false;
  TformRecordGeneric::ScreenLock();
  return true;
}
//---------------------------------------------------------------------------
bool TformRecordBehOutsideTreatment::ScreenUnlock()
{
  panelGroup->Enabled = true;
  editMedications->ReadOnly = false;
  editMedications->TabStop = true;
  editNotes->ReadOnly = false;
  editNotes->TabStop = true;
  TformRecordGeneric::ScreenUnlock();
  return true;
}
//---------------------------------------------------------------------------
void __fastcall TformRecordBehOutsideTreatment::btnFacilitySearchClick(
      TObject *Sender)
{
  // create and show new person search form
  vector<string> return_vector;
  FORM_TRY
    TformSearchTaxEntity * facilitySearch = new TformSearchTaxEntity(NULL,&return_vector);
    facilitySearch->ShowTaskModal(this);
  FORM_CATCH
  if (! return_vector.empty())
    CreateLink(txtFacility, btnFacilitySearch, this, return_vector[0], "tax_entities");
}
//---------------------------------------------------------------------------
void TformRecordBehOutsideTreatment::Do(map<string, string> & p_parameters)
{
 FORM_TRY
  TformRecordBehOutsideTreatment * z = new TformRecordBehOutsideTreatment(NULL, p_parameters);
  z->Show();
 FORM_CATCH
}
//---------------------------------------------------------------------------

