//---------------------------------------------------------------------------

#include <vcl.h>
#pragma hdrstop

#include "codeRecordBug.h"
#include "codeRecordBugNotes.h"
//---------------------------------------------------------------------------
#pragma package(smart_init)
#pragma link "codeRecordGeneric"
#pragma link "codeRubberContainer"
#pragma resource "*.dfm"
TformRecordBug *formRecordBug;
//---------------------------------------------------------------------------
__fastcall TformRecordBug::TformRecordBug(TComponent* Owner, map<string,string> p_parameters)
        : TformRecordGeneric(Owner, p_parameters, string("bugs"))
{
  notes_filter.SetListViewHandle (listNotes);
  notes_filter.SetSecurity("bugs");
  notes_filter.AddColumnSpec("Date", "bugs", as_date, true, 85);
  notes_filter.AddColumnSpec("Posted By", "bugs", as_text, true, 90);
  notes_filter.AddColumnSpec("Type", "bugs", as_text, true, 95);
  notes_filter.AddColumnSpec("Note", "bugs", as_text, true, 400, rawString);
  notes_filter.AddColumnSpec("Current Id", "current_id", as_int, false, 80);
  notes_filter.AddColumnSpec("Internal Id", "internal_id", as_int, false, 80);
  notes_filter.FetchPreferences("recordbug.notes_filter");
  notes_filter.SetIconStatus(true);
  notes_filter.SetSQLStatement ("");
  notes_filter.RefreshDataSet();
  lastRadioChecked = 0;
  Initialize();
}
//---------------------------------------------------------------------------
bool TformRecordBug::ScreenStart()
{
  // register combo box
  lookupTables->RegisterComboBox("users", cboReportedBy);
  lookupTables->RegisterComboBox("users", cboClaimedBy);
  lookupTables->RegisterComboBox("users", cboFixedBy);
  lookupTables->RegisterComboBox("bug_priority_types", cboPriority);
  lookupTables->RegisterComboBox("bug_status_types", cboStatus);
  lookupTables->RegisterComboBox("bug_types", cboType);
  if (radioGroup->Checked)
     lookupTables->RegisterComboBox("groups", cboCreatedFor);
  else if (radioUser->Checked)
     lookupTables->RegisterComboBox("users", cboCreatedFor);

  if (optionsPermissions["admin.bugs_maintainer"]["enabled"] == "true")
  {
     cboClaimedBy->Enabled = true;
     editClaimedOn->Enabled = true;
     cboFixedBy->Enabled = true;
     editFixedOn->Enabled = true;
     cboPriority->Enabled = true;
     cboStatus->Enabled = true;
     cboType->Enabled = true;
  }
  // set the title bar color
  panelSelectArea->Color = title_bar_color;

  TformRecordGeneric::ScreenStart();
  return true;
}
//---------------------------------------------------------------------------
bool TformRecordBug::ScreenStop()
{
  // unregister combo box
  lookupTables->UnregisterComboBox("users", cboReportedBy);
  lookupTables->UnregisterComboBox("users", cboClaimedBy);
  lookupTables->UnregisterComboBox("users", cboFixedBy);
  lookupTables->UnregisterComboBox("bug_priority_types", cboPriority);
  lookupTables->UnregisterComboBox("bug_status_types", cboStatus);
  lookupTables->UnregisterComboBox("bug_types", cboType);
  if (radioGroup->Checked)
     lookupTables->UnregisterComboBox("groups", cboCreatedFor);
  else if (radioUser->Checked)
     lookupTables->UnregisterComboBox("users", cboCreatedFor);

  TformRecordGeneric::ScreenStop();
  return true;
}
//---------------------------------------------------------------------------
bool TformRecordBug::IsValidOnScreen()
{
  if (cboStatus->Text == "Complete" && optionsPermissions["admin.bugs_maintainer"]["enabled"] != "true")
  {
    PSay("A maintainer of the bug log has marked this bug as Complete.  You cannot modify it.  The reason(s) for marking "
         "it as completed will be in the Actual Cause/Location/Solution sections.  If you feel that your request was not "
         "brought to an agreeable conclusion, contact us.", m_warning, MB_OK);
    return false;
  }
  if (cboStatus->Text == "Rejected" && optionsPermissions["admin.bugs_maintainer"]["enabled"] != "true")
  {
    PSay("A maintainer of the bug log has marked this bug as Rejected.  You cannot modify it.  The reason(s) for rejection "
         "will be in the Probable Cause section.  If you feel that your request was rejected out of hand, contact us.", m_warning, MB_OK);
    return false;
  }

  if ( (optionsPermissions["admin.bugs_maintainer"]["enabled"] != "true" && cboReportedBy->Text == "") ||
       !lookupTables->IsInLookUpTable("users", cboReportedBy->Text.c_str()) )
  {
    PSay("Please enter a valid reported by user", m_invalid_data, MB_OK);
    cboReportedBy->SetFocus();
    return false;
  }
  if ( (optionsPermissions["admin.bugs_maintainer"]["enabled"] != "true" && editReportedOn->Text == "") ||
       (editReportedOn->Text != "" && !Pdate (editReportedOn->Text.c_str(), mdy | yy).IsValidDate()) )
  {
    PSay("Please enter a valid reported on date", m_invalid_data, MB_OK);
    editReportedOn->SetFocus();
    return false;
  }
  if ( (optionsPermissions["admin.bugs_maintainer"]["enabled"] != "true" && editDueOn->Text == "") ||
       (editDueOn->Text != "" && !Pdate (editDueOn->Text.c_str(), mdy | yy).IsValidDate()) )
  {
    PSay("Please enter a valid due on date", m_invalid_data, MB_OK);
    editDueOn->SetFocus();
    return false;
  }
  if ( (optionsPermissions["admin.bugs_maintainer"]["enabled"] != "true" && radioGroup->Checked && cboCreatedFor->Text == "") ||
       (radioGroup->Checked && (!lookupTables->IsInLookUpTable("groups", cboCreatedFor->Text.c_str()))) )
  {
    PSay("Please enter a department to be created for", m_invalid_data, MB_OK);
    cboCreatedFor->SetFocus();
    return false;
  }
  if ( (optionsPermissions["admin.bugs_maintainer"]["enabled"] != "true" && radioUser->Checked && cboCreatedFor->Text == "") ||
       (radioUser->Checked && (!lookupTables->IsInLookUpTable("users", cboCreatedFor->Text.c_str()))) )
  {
    PSay("Please enter a valid user to be created for", m_invalid_data, MB_OK);
    cboCreatedFor->SetFocus();
    return false;
  }

  if (! lookupTables->IsInLookUpTable("users", cboClaimedBy->Text.c_str()))
  {
    PSay("Please enter a valid claimed by user", m_invalid_data, MB_OK);
    cboClaimedBy->SetFocus();
    return false;
  }
  if (! lookupTables->IsInLookUpTable("users", cboFixedBy->Text.c_str()))
  {
    PSay("Please enter a valid fixed by user", m_invalid_data, MB_OK);
    cboFixedBy->SetFocus();
    return false;
  }
  if (! lookupTables->IsInLookUpTable("bug_priority_types", cboPriority->Text.c_str()))
  {
    PSay("Please enter a valid priority", m_invalid_data, MB_OK);
    cboPriority->SetFocus();
    return false;
  }
  if (! lookupTables->IsInLookUpTable("status_types", cboStatus->Text.c_str()))
  {
    PSay("Please enter a valid status", m_invalid_data, MB_OK);
    cboStatus->SetFocus();
    return false;
  }
  if (! lookupTables->IsInLookUpTable("bug_types", cboType->Text.c_str()))
  {
    PSay("Please enter a valid type", m_invalid_data, MB_OK);
    cboType->SetFocus();
    return false;
  }
  if (! Pdate (editClaimedOn->Text.c_str(), mdy | yy).IsValidDate() && ! (editClaimedOn->Text == ""))
  {
    PSay("Please enter a valid claimed on date", m_invalid_data, MB_OK);
    editClaimedOn->SetFocus();
    return false;
  }
  if (! Pdate (editFixedOn->Text.c_str(), mdy | yy).IsValidDate() && ! (editFixedOn->Text == ""))
  {
    PSay("Please enter a valid fixed on date", m_invalid_data, MB_OK);
    editFixedOn->SetFocus();
    return false;
  }
  return true;
}
//---------------------------------------------------------------------------
bool TformRecordBug::PutDataOnScreen()
{
  // put the values on the form from the map initial_state
  // values may have come from GetDataFromDatabase if it's an existing record,
  // or values may have been passed to the form in this map if it's a new record

  //PSay(MapToDelimString(initial_state, "", "", "\n", "'", "'", "'", "'", "="), m_debug, MB_OK);
  if (IsBlankOrZero(initial_state["created_by"]))
    initial_state["created_by"] = current_userid;
  lblCreatedByUser->Caption = lookupTables->ValueOf("users", initial_state["created_by"]).c_str();
  if (initial_state["created_on"] == "")
    initial_state["created_on"] = Pdate(Date()).OutDate(global_mode);
  lblCreatedOnDate->Caption = Pdate(initial_state["created_on"], mdy | yyyy).OutDate();
  editReportedOn->Text = initial_state["reported_on"].c_str();
  editClaimedOn->Text = initial_state["claimed_on"].c_str();
  editFixedOn->Text = initial_state["fixed_on"].c_str();
  editDueOn->Text = initial_state["due_on"].c_str();
  cboReportedBy->Text = lookupTables->ValueOf("users", initial_state["reported_by"]).c_str();
  cboClaimedBy->Text = lookupTables->ValueOf("users", initial_state["claimed_by"]).c_str();
  cboFixedBy->Text = lookupTables->ValueOf("users", initial_state["fixed_by"]).c_str();
  cboPriority->Text = lookupTables->ValueOf("bug_priority_types", initial_state["bug_priority_id"]).c_str();
  cboStatus->Text = lookupTables->ValueOf("bug_status_types", initial_state["status_id"]).c_str();
  if (initial_state["created_for_user"] != "0")
  {
     cboCreatedFor->Text = lookupTables->ValueOf("users", initial_state["created_for_user"]).c_str();
     radioUser->Checked = true;
  }
  else
  {
     cboCreatedFor->Text = lookupTables->ValueOf("groups", initial_state["created_for_group"]).c_str();
     radioGroup->Checked = true;
  }
  radioClick(NULL);

  // get the person name associated with the person id that was either selected
  // from the database if it's an existing record, or passed to the form if
  // it's a new record. display the name as a link to the person record.

  TformRecordGeneric::PutDataOnScreen();
  return true; // what can really go wrong?
}
//---------------------------------------------------------------------------
bool TformRecordBug::GetDataFromScreen()
{
  // place the values currently on the screen into
  // the map that will be used to update or insert the record
  // the parent form will call the insert or update function, and get the id
  // number if necessary and take care of current_id
  ending_state["created_by"] = initial_state["created_by"];
  if(lblCreatedOnDate->Caption != "")
    ending_state["created_on"] = Pdate(lblCreatedOnDate->Caption.c_str(), mdy | yy).OutDate(mdy | yyyy);
  else
    ending_state["created_on"] = "";
  if(editReportedOn->Text != "")
    ending_state["reported_on"] = Pdate(editReportedOn->Text.c_str(), mdy | yy).OutDate(mdy | yyyy);
  else
    ending_state["reported_on"] = "";
  if(editClaimedOn->Text != "")
    ending_state["claimed_on"] = Pdate(editClaimedOn->Text.c_str(), mdy | yy).OutDate(mdy | yyyy);
  else
    ending_state["claimed_on"] = "";
  if(editFixedOn->Text != "")
    ending_state["fixed_on"] = Pdate(editFixedOn->Text.c_str(), mdy | yy).OutDate(mdy | yyyy);
  else
    ending_state["fixed_on"] = "";
  if(editDueOn->Text != "")
    ending_state["due_on"] = Pdate(editDueOn->Text.c_str(), mdy | yy).OutDate(mdy | yyyy);
  else
    ending_state["due_on"] = "";
  ending_state["reported_by"] = lookupTables->IdOf("users", cboReportedBy->Text.c_str());
  ending_state["claimed_by"] = lookupTables->IdOf("users", cboClaimedBy->Text.c_str());
  ending_state["fixed_by"] = lookupTables->IdOf("users", cboFixedBy->Text.c_str());
  ending_state["bug_priority_id"] = lookupTables->IdOf("bug_priority_types", cboPriority->Text.c_str());
  ending_state["status_id"] = lookupTables->IdOf("bug_status_types", cboStatus->Text.c_str());
  ending_state["bug_type_id"] = lookupTables->IdOf("bug_types", cboType->Text.c_str());
  if (radioGroup->Checked)
  {
     ending_state["created_for_user"] = "0";
     ending_state["created_for_group"] = lookupTables->IdOf("groups", cboCreatedFor->Text.c_str());
  }
  else if (radioUser->Checked)
  {
     ending_state["created_for_group"] = "0";
     ending_state["created_for_user"] = lookupTables->IdOf("users", cboCreatedFor->Text.c_str());
  }
  ending_state["id"] = initial_state["id"];

  TformRecordGeneric::GetDataFromScreen();

  return true; // again, not much can go wrong here
}
//---------------------------------------------------------------------------
bool TformRecordBug::ScreenLock()
{
  // disable panels, edit boxes
  panelGroup->Enabled = false;
  TformRecordGeneric::ScreenLock();
  return true;
}
//---------------------------------------------------------------------------
bool TformRecordBug::ScreenUnlock()
{
  // enable panel containing edit boxes
  panelGroup->Enabled = true;
  TformRecordGeneric::ScreenUnlock();
  return true;
}
//---------------------------------------------------------------------------
void __fastcall TformRecordBug::radioClick(TObject *Sender)
{
   int currentRadioChecked;
   if (radioGroup->Checked)
      currentRadioChecked = 0;
   else if (radioUser->Checked)
      currentRadioChecked = 1;
   if (currentRadioChecked != lastRadioChecked)
   {
      if (radioGroup->Checked)
      {
         lookupTables->UnregisterComboBox("users", cboCreatedFor);
         lookupTables->RegisterComboBox("groups", cboCreatedFor);
         lblCreatedFor->Caption = "Department";
      }
      else if (radioUser->Checked)
      {
         lookupTables->UnregisterComboBox("groups", cboCreatedFor);
         lookupTables->RegisterComboBox("users", cboCreatedFor);
         lblCreatedFor->Caption = "Employee";
      }
   }
   lastRadioChecked = currentRadioChecked;
}
//---------------------------------------------------------------------------
bool TformRecordBug::CheckForUselessRecord()
{
  return true;
}
//---------------------------------------------------------------------------
bool TformRecordBug::IsNewAgain()
{
 notes_filter.SetSQLStatement(string(""));
 notes_filter.RefreshDataSet();
 notes_filter.SetAddEdit(this, NULL, NULL);
 TformRecordGeneric::IsNewAgain();
 return true;
}
//---------------------------------------------------------------------------
bool TformRecordBug::NotNewAnymore()
{
  string sql;
    sql = "select bug_notes.date_of_post, users.item, bug_note_types.item, bug_notes.note, bug_notes.current_id, bug_notes.id "
          "from bug_notes inner join bug_note_types on bug_notes.bug_note_id = bug_note_types.id "
          "inner join users on bug_notes.posted_by = users.id "
          "where bug_notes.bug_id = " + initial_state["id"];

  notes_filter.SetSQLStatement(sql);
  notes_filter.RefreshDataSet();
  notes_filter.SetAddEdit(this, AddRecord(&(TformRecordBug::AddNote)),
                                        EditRecord(&(TformRecordBug::EditNote)));
  notes_request = atlas->RequestNotificationWhen("bug_notes.bug_id", initial_state["id"], rel_e, as_int, this,
                               RefreshHandler(&(TformRecordBug::AtlasRefreshListViews)));
  TformRecordGeneric::NotNewAnymore();
  return true;
}
//---------------------------------------------------------------------------
bool TformRecordBug::AtlasRefreshListViews(int request_num, string matched_value)
{
  if (request_num == notes_request)
    notes_filter.RefreshDataSet();

  return true;
}
//---------------------------------------------------------------------------
void TformRecordBug::AddNote()
{
  map<string, string> temp_map;
  temp_map["id"] = "new";
  temp_map["bug_id"] = initial_state["id"];
  temp_map["status_id"] = initial_state["status_id"];
  FORM_TRY
    TformRecordBugNotes * note = new TformRecordBugNotes(NULL, temp_map);
    note->Show();
  FORM_CATCH
}
//---------------------------------------------------------------------------
void TformRecordBug::EditNote()
{
  map<string, string> temp_map;
  temp_map["id"] = notes_filter.GetSelectedRecordID();
  FORM_TRY
    TformRecordBugNotes * note = new TformRecordBugNotes(NULL, temp_map);
    note->Show();
  FORM_CATCH
}
//---------------------------------------------------------------------------
void __fastcall TformRecordBug::listNotesSelectItem(TObject *Sender,
      TListItem *Item, bool Selected)
{
  if (notes_filter.GetSelectedRecordID() != "")
  {
    string note = GetOneField(dbDatabase, "select note from bug_notes where id = " + notes_filter.GetSelectedRecordID());
    memoNote->Text = note.c_str();
  }
}
//---------------------------------------------------------------------------
void TformRecordBug::Do(map<string, string> & p_parameters)
{
 FORM_TRY
  TformRecordBug * z = new TformRecordBug(NULL, p_parameters);
  z->Show();
 FORM_CATCH
}
//---------------------------------------------------------------------------

