//---------------------------------------------------------------------------

#include <vcl.h>
#pragma hdrstop

#include "codeRecordChart.h"
//---------------------------------------------------------------------------
#pragma package(smart_init)
#pragma link "codeRecordGeneric"
#pragma resource "*.dfm"
TformRecordChart *formRecordChart;
//---------------------------------------------------------------------------
__fastcall TformRecordChart::TformRecordChart(TComponent* Owner, map<string,string> p_parameters)
        : TformRecordGeneric(Owner, p_parameters, string("charts"))
{
 Initialize();
}
//---------------------------------------------------------------------------
bool TformRecordChart::ScreenStart()
{
  set<string> can_view = GetGroupPermissions("table.charts", "", "view", "true");
  string location_groups_sql = "select id, item, valid from groups where " // has_charts = 1 or id = 0 order by item"; // clinics have charts
                               + SafeInClause("groups.id", SetToVec(can_view));

  lookupTables->RegisterComboBox("groups", cboLocation, location_groups_sql);

  TformRecordGeneric::ScreenStart();
  return true;
}
//---------------------------------------------------------------------------
bool TformRecordChart::ScreenStop()
{
  // unregister combo box
  lookupTables->UnregisterComboBox("groups", cboLocation);

  TformRecordGeneric::ScreenStop();
  return true;
}
//---------------------------------------------------------------------------
bool TformRecordChart::IsValidOnScreen()
{
  // check to make sure that the values entered in the combo boxe
  // corrospond to values in that look up table.
  // give the user a message box and prompt to re-enter if incorrect
  // make sure there is a link to a person, although there's not much the user
  // can do to fix that if there's not
  if (cboLocation->Text == "" || ! lookupTables->IsInLookUpTable("groups", cboLocation->Text.c_str()))
  {
    PSay("Please enter a valid chart location", m_invalid_data, MB_OK);
    cboLocation->SetFocus();
    return false;
  }
  else if ((initial_state["person_id"] == "") || (initial_state["person_id"] == "0"))
  {
    PSay("There is no person associated with this record", m_missing_data, MB_OK);
    return false;
  }
  else if (editChartNumber->Text == "")
  {
    PSay("Please enter a chart number.", m_missing_data, MB_OK);
    editChartNumber->SetFocus();
    return false;
  }
  // these should really be part of RecordGeneric ... except we don't call the ancestor IsValidOnScreen()
  // ANYwhere, so it would be kind of pointless.  Sigh.
  else if (is_new_record && !_CanInsert())
  {
    PSay("You do not have permission to insert a " + lookupTables->ValueOf("groups", initial_state["group_id"]) + " chart.", m_warning, MB_OK);
    return false;
  }
  else if (!is_new_record && !_CanUpdate())
  {
    PSay("You do not have permission to update a " + lookupTables->ValueOf("groups", initial_state["group_id"]) + " chart.", m_warning, MB_OK);
    return false;
  }
  // end "these should really be part of RecordGeneric"
  else
    return true;
}
//---------------------------------------------------------------------------
bool TformRecordChart::PutDataOnScreen()
{
  // put the values on the form from the map initial_state
  // values may have come from GetDataFromDatabase if it's an existing record,
  // or values may have been passed to the form in this map if it's a new record

  cboLocation->Text = PrettyString(lookupTables->ValueOf("groups", initial_state["group_id"])).c_str();
  if (is_new_record) // we'll be nice...if it's a new record, fill in the clinic name for them
   if (lookupTables->IsValidId("groups", current_groupid, cboLocation)) // is current user's group a clinic?  if so, set the clinic automatically...
   {
    cboLocation->Text = lookupTables->ValueOf("groups", current_groupid).c_str();
    editChartNumber->SetFocus();
   }

  editChartNumber->Text = initial_state["chart_number"].c_str();

  // get the person name associated with the person id that was either selected
  // from the database if it's an existing record, or passed to the form if
  // it's a new record. display the name as a link to the person record.
  CreateLink(txtPerson, NULL, this, initial_state["person_id"], "people");

  TformRecordGeneric::PutDataOnScreen();
  return true; // what can really go wrong?
}
//---------------------------------------------------------------------------
bool TformRecordChart::GetDataFromScreen()
{
  // place the values currently on the screen into
  // the map that will be used to update or insert the record
  // the parent form will call the insert or update function, and get the id
  // number if necessary and take care of current_id
  ending_state["group_id"] = lookupTables->IdOf("groups", cboLocation->Text.c_str());
  /* TODO -c HACK: group names marked as locations match location_types by item; this is a
     temporary measure until I get the rest of my stuff committed */
  ending_state["location_id"] = lookupTables->IdOf("location_types", cboLocation->Text.c_str());
  ending_state["chart_number"] = editChartNumber->Text.c_str();
  ending_state["person_id"] = initial_state["person_id"];
  ending_state["id"] = initial_state["id"];

  TformRecordGeneric::GetDataFromScreen();

  return true; // again, not much can go wrong here
}
//---------------------------------------------------------------------------
bool TformRecordChart::ScreenLock()
{
  // disable panel containing edit boxes
  panelGroup->Enabled = false;
  TformRecordGeneric::ScreenLock();
  return true;
}
//---------------------------------------------------------------------------
bool TformRecordChart::ScreenUnlock()
{
  // enable panel containing edit boxes
  panelGroup->Enabled = true;
  TformRecordGeneric::ScreenUnlock();
  return true;
}
//---------------------------------------------------------------------------
void TformRecordChart::Do(map<string, string> & p_parameters)
{
 FORM_TRY
  TformRecordChart * z = new TformRecordChart(NULL, p_parameters);
  z->Show();
 FORM_CATCH
}
//---------------------------------------------------------------------------
string TformRecordChart::GroupShortItem()
{
  string group_id;
  if (!is_editing || cboLocation->Text == "" || !lookupTables->IsInLookUpTable("location_types", cboLocation))
    group_id = initial_state["group_id"] != "" ? initial_state["group_id"] : string("0");
  else // is_editing AND a good lookup table value
    group_id = lookupTables->IdOf("location_types", cboLocation);
  return LowercaseString(GetOneField(dbDatabase, "select short_item from groups where id = '" + group_id + "'"));
}
//---------------------------------------------------------------------------
bool TformRecordChart::_CanView() // yeah, we override the caching
{
  string si = GroupShortItem();
  return optionsPermissions["table.charts" + string(si!=""?".":"") + si]["view"] == "true";
}
//---------------------------------------------------------------------------
bool TformRecordChart::_CanUpdate() // yeah, we override the caching
{
  string si = GroupShortItem();
  return optionsPermissions["table.charts" + string(si!=""?".":"") + si]["update"] == "true";
}
//---------------------------------------------------------------------------
bool TformRecordChart::_CanInsert() // yeah, we override the caching
{
  string si = GroupShortItem();
  return optionsPermissions["table.charts" + string(si!=""?".":"") + si]["insert"] == "true";
}
//---------------------------------------------------------------------------

