//---------------------------------------------------------------------------

#include <vcl.h>
#pragma hdrstop

#include "codeRecordCheck.h"
#include "codeRecordPurchaseReq.h"
#include "launchCheckJustification.h"
#include "codeSpecialVoidCheck.h"
#include "codeSpecialModifyCashAccount.h"
#include "codeWizardModifyTransactions.h"
#include "codeSpecialTransactionSummary.h"
//---------------------------------------------------------------------------
#pragma package(smart_init)
#pragma link "codeRecordGeneric"
#pragma resource "*.dfm"
TformRecordCheck *formRecordCheck;
//---------------------------------------------------------------------------
__fastcall TformRecordCheck::TformRecordCheck(TComponent* Owner, map<string,string> p_parameters)
        : TformRecordGeneric(Owner, p_parameters, string("checks"))
{
  purchase_reqs_filter.SetListViewHandle (listPRs);
  purchase_reqs_filter.SetSecurity("purchase_reqs");
  purchase_reqs_filter.AddColumnSpec("PR Number",   "purchase_reqs", as_int, true, 90);
  purchase_reqs_filter.AddColumnSpec("Tax Entity",  "tax_entities", as_text, true, 120, rawString);
  purchase_reqs_filter.AddColumnSpec("Date Sent",   "purchase_reqs", as_date, true, 90);
  purchase_reqs_filter.AddColumnSpec("Current Id",  "current_id", as_int, false, 80);
  purchase_reqs_filter.AddColumnSpec("Internal Id", "internal_id", as_int, false, 80);
  purchase_reqs_filter.SetIconStatus(true);
  purchase_reqs_filter.FetchPreferences("recordchecks.checks_filter");
  purchase_reqs_filter.SetSQLStatement ("");
  purchase_reqs_filter.RefreshDataSet();

  Initialize();
}
//---------------------------------------------------------------------------
bool TformRecordCheck::ScreenStart()
{   
  txtPrintCheckJustification->Font->Color = link_color;
  staticVoidCheck->Font->Color = link_color;
  staticBetaModifyTransactions->Font->Color = link_color;
  staticTransactionSummary->Font->Color = link_color;
  // staticModifyCheck->Font->Color = link_color; // link removed because Matt's screen sucks

  TformRecordGeneric::ScreenStart();
  return true;
}
//---------------------------------------------------------------------------
bool TformRecordCheck::ScreenStop()
{

  TformRecordGeneric::ScreenStop();
  return true;
}
//---------------------------------------------------------------------------
bool TformRecordCheck::IsValidOnScreen()
{
  if ((initial_state["tax_entity_id"] == "") || (initial_state["tax_entity_id"] == "0"))
  {
    PSay("There is no tax entity associated with this record", m_missing_data, MB_OK);
    return false;
  }
  else if(editCheckNumber->Text == "")
  {
    PSay("There is no check number for this record", m_missing_data, MB_OK);
    return false;
  }
  else
    return true;
}
//---------------------------------------------------------------------------
bool TformRecordCheck::PutDataOnScreen()
{
  // put the values on the form from the map initial_state
  // values may have come from GetDataFromDatabase if it's an existing record,
  // or values may have been passed to the form in this map if it's a new record

  editCheckNumber->Text = initial_state["check_number"].c_str();
  if(initial_state["date_sent"] != "")
    dateSent->Date = Pdate(initial_state["date_sent"], mdy | yyyy).OutDate();
  else
    dateSent->Date = Now();

  CreateLink(txtTaxEntity, NULL, this, initial_state["tax_entity_id"], "tax_entities");
  TformRecordGeneric::PutDataOnScreen();

  return true;
}
//---------------------------------------------------------------------------
bool TformRecordCheck::GetDataFromScreen()
{
  // place the values currently on the screen into
  // the map that will be used to update or insert the record
  // the parent form will call the insert or update function, and get the id
  // number if necessary and take care of current_id
  ending_state["tax_entity_id"] = initial_state["tax_entity_id"];
  ending_state["check_number"] = initial_state["check_number"];
  ending_state["date_sent"] = Pdate(dateSent->Date).OutDate(mdy | yyyy);
  ending_state["id"] = initial_state["id"];

  TformRecordGeneric::GetDataFromScreen();

  return true; // again, not much can go wrong here
}
//---------------------------------------------------------------------------
bool TformRecordCheck::ScreenLock()
{
  // disable panel containing edit boxes
  panelGroup->Enabled = false;
  TformRecordGeneric::ScreenLock();
  return true;
}
//---------------------------------------------------------------------------
bool TformRecordCheck::ScreenUnlock()
{
  // enable panel containing edit boxes
  panelGroup->Enabled = true;
  TformRecordGeneric::ScreenUnlock();
  return true;
}
//---------------------------------------------------------------------------
bool TformRecordCheck::NotNewAnymore()
{
  string sql;
  sql = "select distinct(purchase_reqs.purchase_req_number), tax_entities.name, purchase_reqs.date_sent, "
        "purchase_reqs.current_id, purchase_reqs.id "
        "from purchase_reqs, tax_entities, checks "
        "where purchase_reqs.check_id = checks.id and "
        "purchase_reqs.tax_entity_id = tax_entities.id and "
        "purchase_reqs.id > 0 and "
        "purchase_reqs.check_id = " + initial_state["id"];

  purchase_reqs_filter.SetSQLStatement (sql);
  purchase_reqs_filter.RefreshDataSet();
  purchase_reqs_filter.SetAddEdit(this, NULL, EditRecord(&(TformRecordCheck::EditPR)));
  purchase_reqs_request = atlas->RequestNotificationWhen("purchase_reqs.check_id", initial_state["id"], rel_e, as_int, this,
                          RefreshHandler(&(TformRecordCheck::AtlasRefreshListView)));
  TformRecordGeneric::NotNewAnymore();

  staticTransactionSummary->Enabled = true;
  staticBetaModifyTransactions->Enabled = true;
  return true;
}
//---------------------------------------------------------------------------
bool TformRecordCheck::AtlasRefreshListView(int request_num, string matched_value)
{
  if (request_num == purchase_reqs_request)
    purchase_reqs_filter.RefreshDataSet();
  return true;
}
//---------------------------------------------------------------------------
bool TformRecordCheck::IsNewAgain()
{
  purchase_reqs_filter.SetSQLStatement(string(""));
  purchase_reqs_filter.RefreshDataSet();
  purchase_reqs_filter.SetAddEdit(this, NULL, NULL);
  TformRecordGeneric::IsNewAgain();
  return true;
}
//---------------------------------------------------------------------------
void TformRecordCheck::EditPR()
{
  map<string, string> temp_map;
  temp_map["id"] = purchase_reqs_filter.GetSelectedRecordID();
  FORM_TRY
    TformRecordPurchaseReq * pr_record = new TformRecordPurchaseReq(NULL, temp_map);
    pr_record->Show();
  FORM_CATCH
}
//---------------------------------------------------------------------------
void __fastcall TformRecordCheck::txtPrintCheckJustificationClick(TObject *Sender)
{
  TformLaunchCheckJustification * check_justification_report = NULL;

  FORM_TRY
    check_justification_report = new TformLaunchCheckJustification(NULL, initial_state["id"]);
  FORM_CATCH_RETURN("TformLaunchCheckJustification", )

  check_justification_report->btnRunClick(NULL);
  check_justification_report->Close();
}
//---------------------------------------------------------------------------
void __fastcall TformRecordCheck::staticVoidCheckClick(TObject *Sender)
{
 // void check screen
 FORM_TRY
   TformSpecialVoidCheck * void_check = new TformSpecialVoidCheck(NULL, initial_state["id"]);
   void_check->Show();
 FORM_CATCH
 return;
}
//---------------------------------------------------------------------------
void __fastcall TformRecordCheck::staticModifyCheckClick(TObject *Sender)
{
 /* Deprecated!  Woo!
 FORM_TRY
   TformSpecialModifyCashAccount * modify_cash_account = new TformSpecialModifyCashAccount(NULL, initial_state["id"], "expense");
   modify_cash_account->Show();
 FORM_CATCH
 return;
 */
}
//---------------------------------------------------------------------------
void __fastcall TformRecordCheck::staticBetaModifyTransactionsClick(TObject *Sender)
{
 if (optionsPermissions["table.transactions"]["insert"] == "true")
 {
  FORM_TRY
   TformWizardModifyTransactions * wiz = new TformWizardModifyTransactions(NULL, initial_state["id"], mod_check);
   wiz->Show();
  FORM_CATCH
 }
}
//---------------------------------------------------------------------------
void __fastcall TformRecordCheck::staticTransactionSummaryClick(TObject *Sender)
{
 if (optionsPermissions["table.transactions"]["view"] == "true")
 {
  FORM_TRY
   TformSpecialTransactionSummary * sum = new TformSpecialTransactionSummary(NULL, initial_state["id"], "checks");
   sum->Show();
  FORM_CATCH
 }
}
//---------------------------------------------------------------------------
void TformRecordCheck::Do(map<string, string> & p_parameters)
{
 FORM_TRY
  TformRecordCheck * z = new TformRecordCheck(NULL, p_parameters);
  z->Show();
 FORM_CATCH
}
//---------------------------------------------------------------------------

