//---------------------------------------------------------------------------

#include <vcl.h>
#pragma hdrstop

#include "codeRecordCobRequest.h"
#include "codeSearchPerson.h"
//---------------------------------------------------------------------------
#pragma package(smart_init)
#pragma link "codeRecordGeneric"
#pragma resource "*.dfm"
TformRecordCobRequest *formRecordCobRequest;
//---------------------------------------------------------------------------
__fastcall TformRecordCobRequest::TformRecordCobRequest(TComponent* Owner, map<string, string> & p_parameters)
        : TformRecordGeneric(Owner, p_parameters, string("cob_requests"))
{
  Initialize();
}
//---------------------------------------------------------------------------
bool TformRecordCobRequest::ScreenStart()
{
 lookupTables->RegisterComboBox("level_types", cboLevel);

 referrer_filter.SetListViewHandle(listReferringPhysicians);
 referrer_filter.SetSecurity("tax_entities");
 referrer_filter.AddColumnSpec("Name", "tax_entities", as_text, true, 200, rawString);
 referrer_filter.AddColumnSpec("Current Id", "current_id", as_int, false, 80);
 referrer_filter.AddColumnSpec("Internal Id", "internal_id", as_int, false, 80);
 referrer_filter.FetchPreferences("recordcobrequest.referrer_filter");
 referrer_filter.SetHeader(false);
 referrer_filter.SetIconStatus(true);
 referrer_filter.SetSQLStatement("select tax_entities.name, tax_entities.current_id, tax_entities.id from tax_entities where tax_entities.is_referring = 1");
 referrer_filter.RefreshDataSet();

 if(optionsPermissions["special.handle_cob_requests"]["enabled"] != "true")
   checkCreated->Enabled = false;

 TformRecordGeneric::ScreenStart();
 return true;
}
//---------------------------------------------------------------------------
bool TformRecordCobRequest::ScreenStop()
{
 lookupTables->UnregisterComboBox("level_types", cboLevel);
 TformRecordGeneric::ScreenStop();
 return true;
}
//---------------------------------------------------------------------------
bool TformRecordCobRequest::IsValidOnScreen()
{
 if (! lookupTables->IsInLookUpTable ("level_types", cboLevel->Text.c_str()))
 {
    PSay("Please enter a valid level", m_invalid_data, MB_OK);
    cboLevel->SetFocus();
    return false;
 }
 else if (listReferringPhysicians->SelCount < 1)
 {
   PSay("Please select a referrer", m_missing_data, MB_OK);
   listReferringPhysicians->SetFocus();
   return false;
 }
 else if (GetValidLinkId(txtPerson) == "")
 {
   PSay("There is no person attached to this record", m_missing_data, MB_OK);
   return false;
 }
 else
 {
   string sql;
   string company;

   // note:  this may return more than one row, since insurance_id moved from tax_entities to insurance_policies, but GetOneField will ignore any extras
   sql = "select tax_entities.name from insurance_coverage, insurance_policies, tax_entities "
         "where insurance_coverage.person_id = '" + GetValidLinkId(txtPerson) + "' "
         " and insurance_coverage.insurance_policy_id = insurance_policies.id and "
         "insurance_policies.tax_entity_id = tax_entities.id and "
         "(insurance_policies.stop_date is null or insurance_policies.stop_date > '" +
         string(DateToStr(Date()).c_str()) + "') and insurance_policies.insurance_id != " +
         lookupTables->IdOf("insurance_types", "Veteran");
//   PSay(sql.c_str(), m_message, MB_OK);
   company = GetOneField(dbDatabase, sql);
   if(company != "")
   {
     PSay(string("This patient already has insurance coverage with " +
       PrettyString(company) + ". Please ask the patient to go through this insurance company for a referral").c_str(), m_warning, MB_OK);
     return false;
   }
   else
     return true;
 }
}
//---------------------------------------------------------------------------
bool TformRecordCobRequest::PutDataOnScreen()
{
 cboLevel->Text = lookupTables->ValueOf("level_types", initial_state["level_id"]).c_str();
 editDiagnosis->Text = SentenceString(initial_state["diagnosis"]).c_str();
 editService->Text = SentenceString(initial_state["services_required"]).c_str();
 if(initial_state["tax_entity_id"] != "")
 {
   AnsiString temp = initial_state["tax_entity_id"].c_str();
   for(int i = 0; i < listReferringPhysicians->Items->Count; i++)
   {
     if(listReferringPhysicians->Items->Item[i]->SubItems->Strings[0] == temp)
       listReferringPhysicians->Items->Item[i]->Selected = true;
   }
 }
 if(initial_state["cob_created"] == "1")
   checkCreated->Checked = true;
 else
   checkCreated->Checked = false;

 CreateLink(txtPerson, btnPersonSearch, this, initial_state["person_id"], "people");

 TformRecordGeneric::PutDataOnScreen();
 return true;
}
//---------------------------------------------------------------------------
bool TformRecordCobRequest::GetDataFromScreen()
{
 ending_state["level_id"] = lookupTables->IdOf("level_types", cboLevel->Text.c_str());
 ending_state["diagnosis"] = editDiagnosis->Text.c_str();
 ending_state["services_required"] = editService->Text.c_str();
 ending_state["tax_entity_id"] = referrer_filter.GetSelectedRecordID();

 if(checkCreated->Checked)
   ending_state["cob_created"] = "1";
 else
   ending_state["cob_created"] = "0";

 ending_state["person_id"] = GetValidLinkId(txtPerson);
 ending_state["id"] = initial_state["id"];

 TformRecordGeneric::GetDataFromScreen();
 return true;
}
//---------------------------------------------------------------------------
bool TformRecordCobRequest::ScreenLock()
{
 panelGroup->Enabled = false;
 editDiagnosis->ReadOnly = true;
 editDiagnosis->TabStop = false;
 editService->ReadOnly = true;
 editService->TabStop = false;
 TformRecordGeneric::ScreenLock();
 return true;
}
//---------------------------------------------------------------------------
bool TformRecordCobRequest::ScreenUnlock()
{
 panelGroup->Enabled = true;
 editDiagnosis->ReadOnly = false;
 editDiagnosis->TabStop = true;
 editService->ReadOnly = false;
 editService->TabStop = true;
 TformRecordGeneric::ScreenUnlock();
 return true;
}
//---------------------------------------------------------------------------
void __fastcall TformRecordCobRequest::btnPersonSearchClick(
      TObject *Sender)
{
  // create and show new person search form
  vector<string> return_vector;
  FORM_TRY
    TformSearchPerson * formPersonSearch = new TformSearchPerson(NULL,&return_vector);
    formPersonSearch->ShowTaskModal(this);
  FORM_CATCH
  if (! return_vector.empty())
    CreateLink(txtPerson, btnPersonSearch, this, return_vector[0], "people");
}
//---------------------------------------------------------------------------
void TformRecordCobRequest::Do(map<string, string> & p_parameters)
{
 FORM_TRY
  TformRecordCobRequest * z = new TformRecordCobRequest(NULL, p_parameters);
  z->Show();
 FORM_CATCH
}
//---------------------------------------------------------------------------

