//---------------------------------------------------------------------------

#include <vcl.h>
#pragma hdrstop

#include "codeRecordCondition.h"
#include "codeRecordProcedure.h"
#include "codeSearchCode.h"
//---------------------------------------------------------------------------
#pragma package(smart_init)
#pragma link "codeRecordGeneric"
#pragma link "codeHalfButton"
#pragma resource "*.dfm"
TformRecordCondition *formRecordCondition;
//---------------------------------------------------------------------------
__fastcall TformRecordCondition::TformRecordCondition(TComponent* Owner, map<string,string> p_parameters)
        : TformRecordGeneric(Owner, p_parameters, string("conditions"))
{
  procedures_filter.SetListViewHandle (listProcedures);
  procedures_filter.SetSecurity("procedures");
  procedures_filter.AddColumnSpec("Code", "procedures", as_text, true, 95);
  procedures_filter.AddColumnSpec("Note", "procedures", as_text, false, 95);
  procedures_filter.AddColumnSpec("From", "procedures", as_date, false, 95);
  procedures_filter.AddColumnSpec("Until", "procedures", as_date, false, 95);
  procedures_filter.AddColumnSpec("Current Id", "current_id", as_int, false, 80);
  procedures_filter.AddColumnSpec("Internal Id", "internal_id", as_int, false, 80);
  procedures_filter.AddColumnSpec("Linking Id", "procedure_reasons", as_int, false, 80);
  procedures_filter.SetIconStatus(false);
  procedures_filter.SetCheck(true);
  procedures_filter.SetCheck(this, CheckRecord(&(TformRecordCondition::CheckProcedure)));
  procedures_filter.SetHeader(false);
  procedures_filter.FetchPreferences("recordcondition.procedures_filter");
  procedures_filter.SetSQLStatement("");
  procedures_filter.RefreshDataSet();

  Initialize();
}
//---------------------------------------------------------------------------
bool TformRecordCondition::ScreenStart()
{
  lookupTables->RegisterComboBox("condition_types", cboCondition);
  lookupTables->RegisterComboBox("allergy_types",   cboAllergyType);
  TformRecordGeneric::ScreenStart();
  return true;
}
//---------------------------------------------------------------------------
bool TformRecordCondition::ScreenStop()
{
  lookupTables->UnregisterComboBox("condition_types", cboCondition);
  lookupTables->UnregisterComboBox("allergy_types",   cboAllergyType);
  TformRecordGeneric::ScreenStop();
  return true;
}
//---------------------------------------------------------------------------
bool TformRecordCondition::IsValidOnScreen()
{
  // check to make sure that the note is not longer than 255 characters
  // give the user a message box and prompt to re-enter if too long
  // make sure there is a link to a person, although there's not much the user
  // can do to fix that if there's not
  if (! lookupTables->IsInLookUpTable("condition_types", cboCondition->Text.c_str()))
  {
    PSay("Please enter a valid condition type", m_invalid_data, MB_OK);
    cboCondition->SetFocus();
    return false;
  }
  if (! lookupTables->IsInLookUpTable("allergy_types", cboAllergyType->Text.c_str()))
  {
    PSay("Please enter a valid allergy type", m_invalid_data, MB_OK);
    cboAllergyType->SetFocus();
    return false;
  }
  // An "Other" allergy type must have more specific information associated with it via the notes.
  // Any allergy that gets "coded" via the notes frequently enough will be manually added as an allergy type.
  string allergy_text = cboAllergyType->Text.c_str();
  if (allergy_text.find("Other") != string::npos && editNote->Text == "")
  {
    PSay("You have specified an 'Other' type of allergy but have provided no information in the "
         "condition's notes to indicate what type of allergy this condition represents.\n\n"
         "Please enter notes information or choose a more specific type of allergy.\n\n"
         "If the allergy type you want is not available, please contact a programmer to have it added.",
         m_missing_data, MB_OK);
    return false;
  }
  if (! Pdate (editStartDate->Text.c_str(), mdy | yy).IsValidDate() && ! (editStartDate->Text == ""))
  {
    PSay("Please enter a valid start date", m_invalid_data, MB_OK);
    editStartDate->SetFocus();
    return false;
  }
  if (! Pdate (editStopDate->Text.c_str(), mdy | yy).IsValidDate() && ! (editStopDate->Text == ""))
  {
    PSay("Please enter a valid stop date", m_invalid_data, MB_OK);
    editStopDate->SetFocus();
    return false;
  }
  if (! IsInt(editIndex->Text.c_str()))
  {
    PSay("Please enter a number for the index number", m_invalid_data, MB_OK);
    editIndex->SetFocus();
    return false;
  }
  if ((initial_state["person_id"] == "") || (initial_state["person_id"] == "0"))
  {
    PSay("There is no person associated with this record", m_missing_data, MB_OK);
    return false;
  }

  return true;
}
//---------------------------------------------------------------------------
bool TformRecordCondition::PutDataOnScreen()
{
  // put the values on the form from the map initial_state
  // values may have come from GetDataFromDatabase if it's an existing record,
  // or values may have been passed to the form in this map if it's a new record

  cboCondition->Text = PrettyString(lookupTables->ValueOf("condition_types", initial_state["condition_id"])).c_str();
  if((cboCondition->Text == "") && (is_new_record))
    cboCondition->Text = "Condition";
  cboAllergyType->Text = lookupTables->ValueOf("allergy_types", initial_state["allergy_id"]).c_str();
  editStartDate->Text = initial_state["start_date"].c_str();
  editStopDate->Text = initial_state["stop_date"].c_str();
  editCode->Text = initial_state["code"].c_str();
  editNote->Text = SentenceString(initial_state["note"]).c_str();
  editIndex->Text = initial_state["index_number"].c_str();

  // get the person name associated with the person id that was either selected
  // from the database if it's an existing record, or passed to the form if
  // it's a new record. display the name as a link to the person record.
  CreateLink(txtPerson, NULL, this, initial_state["person_id"], "people");
  CreateLink(txtTask, NULL, this, initial_state["task_id"], "tasks");

  if(optionsPermissions["table.codes"]["view"] != "true")              
    btnCodeSearch->Enabled = false;

  DownloadProcedures();
  if (IsBlankOrZero(initial_state["task_id"]))
   procedures_filter.SetAddEdit(this, NULL, NULL);
  else
   procedures_filter.SetAddEdit(this, AddRecord(&(TformRecordCondition::AddProcedure)),
                                EditRecord(&(TformRecordCondition::EditProcedure)));
  procedure_request = atlas->RequestNotificationWhen("procedures.task_id", initial_state["task_id"], rel_e, as_int, this,
                       RefreshHandler(&(TformRecordCondition::AtlasRefreshListView)));
  if(! is_new_record)
   procedure_reason_request = atlas->RequestNotificationWhen("procedure_reasons.condition_id", initial_state["id"],
                                rel_e, as_int, this, RefreshHandler(&(TformRecordCondition::AtlasRefreshListView)));

  // DONE : jhaley - set a flag, move this code to this::FormShow() event so that it actually has visible controls and
  // can calculate heights and thus show the infobar
  if (IsBlankOrZero(initial_state["task_id"]))
  {
    non_reportable_info_text =
      "Since this condition was not created as part of a task or visit (and thus with a physician/provider), "
      "it will be considered self-diagnosed.  "
      "For reporting purposes (GPRA, etc.), it won't be counted.";
  }
  else
    non_reportable_info_text = "";

  TformRecordGeneric::PutDataOnScreen();
  return true; // what can really go wrong?
}
//---------------------------------------------------------------------------
bool TformRecordCondition::GetDataFromScreen()
{
  // place the values currently on the screen into
  // the map that will be used to update or insert the record
  // the parent form will call the insert or update function, and get the id
  // number if necessary and take care of current_id
  ending_state["condition_id"] = lookupTables->IdOf("condition_types", cboCondition->Text.c_str());
  ending_state["allergy_id"] = lookupTables->IdOf("allergy_types",   cboAllergyType->Text.c_str());
  if (editStartDate->Text != "")
    ending_state["start_date"] = Pdate (editStartDate->Text.c_str(), mdy | yy).OutDate(mdy | yyyy);
  else
    ending_state["start_date"] = "";
  if (editStopDate->Text != "")
    ending_state["stop_date"] = Pdate (editStopDate->Text.c_str(), mdy | yy).OutDate(mdy | yyyy);
  else
    ending_state["stop_date"] = "";
  ending_state["code"] = editCode->Text.c_str();
  if ((editCode->Text != "") && (editNote->Text == ""))
  {
    string new_note;
    string entered_note = UppercaseString(editNote->Text.c_str());
    string sql = "select description from codes where code = '";
    sql += editCode->Text.c_str();
    sql += "' and code_id = " + lookupTables->IdOf("code_types", "ICD-9");
    string code_note = GetOneField(dbDatabase, sql);
    if (entered_note == "")
      new_note = code_note;
    else
    {
      if(entered_note.find(code_note) == string::npos)
        new_note = entered_note + "\n\n" + code_note;
      else
        new_note = entered_note;
    }
    ending_state["note"] = new_note;
  }
  else
    ending_state["note"] = editNote->Text.c_str();
  ending_state["index_number"] = editIndex->Text.c_str();

  if (initial_state["task_id"] != "")
    ending_state["task_id"] = initial_state["task_id"];
  else
    ending_state["task_id"] = "0";
  if (initial_state["group_id"] == "")
    ending_state["group_id"] = "0";
  else
    ending_state["group_id"] = initial_state["group_id"];
  ending_state["person_id"] = initial_state["person_id"];
  ending_state["id"] = initial_state["id"];

  // jhaley: don't lose a patient_assessment_id if one exists on this condition.
  ending_state["patient_assessment_id"] = initial_state["patient_assessment_id"];

  TformRecordGeneric::GetDataFromScreen();

  return true; // again, not much can go wrong here
}
//---------------------------------------------------------------------------
bool TformRecordCondition::ScreenLock()
{
  // disable panel containing edit boxes
  panelGroup->Enabled = false;
  editNote->ReadOnly = true;
  editNote->TabStop = false;
  procedures_filter.SetCheckLocked(true);
  DownloadProcedures();
  TformRecordGeneric::ScreenLock();
  return true;
}
//---------------------------------------------------------------------------
bool TformRecordCondition::ScreenUnlock()
{
  // enable panel containing edit boxes
  panelGroup->Enabled = true;
  editNote->ReadOnly = false;
  editNote->TabStop = true;
  procedures_filter.SetCheckLocked(false);
  DownloadProcedures();
  TformRecordGeneric::ScreenUnlock();
  return true;
}
//---------------------------------------------------------------------------
void __fastcall TformRecordCondition::btnCodeSearchClick(TObject *Sender)
{
  vector<string> return_vector;
  string sql;
  FORM_TRY
    TformSearchCode * codeSearch = new TformSearchCode(NULL, &return_vector);
    codeSearch->cboCode->Text = "ICD-9";
    codeSearch->ShowTaskModal(this);
  FORM_CATCH
  if (! return_vector.empty())
  {
    sql = "select code from codes where id = " + return_vector[0];
    editCode->Text = GetOneField(dbDatabase, sql).c_str();
    if (editCode->Text != "")
    {
      string new_note;
      string entered_note = UppercaseString(editNote->Text.c_str());
      string sql = "select description from codes where code = '";
      sql += editCode->Text.c_str();
      sql += "' and code_id = " + lookupTables->IdOf("code_types", "ICD-9");
      string code_note = GetOneField(dbDatabase, sql);
      if (entered_note == "")
        new_note = code_note;
      else
      {
        if(entered_note.find(code_note) == string::npos)
          new_note = entered_note + "\n\n" + code_note;
        else
          new_note = entered_note;
      }
      editNote->Text = SentenceString(new_note).c_str();
    }
  }
}
//---------------------------------------------------------------------------
void __fastcall TformRecordCondition::btnExpandCodeClick(TObject *Sender)
{
  FORM_TRY
    TformSearchCode * codeSearch = new TformSearchCode(NULL, NULL);
    codeSearch->cboCode->Text = "ICD-9";
    codeSearch->editCode->Text = editCode->Text;
    codeSearch->Show();
    if (editCode->Text != "")
     codeSearch->buttonSearch->Click();
  FORM_CATCH
}
//---------------------------------------------------------------------------
bool TformRecordCondition::BeforeSave()
{
  vector<string> id_vec;
  try
  {
    ExecuteStatement(lock_transaction, "delete from procedure_reasons where condition_id = " + ending_state["id"]);
    map<string, string> to_insert;
    id_vec = procedures_filter.GetMultiID();
    for(vector<string>::iterator itr = id_vec.begin(); itr != id_vec.end(); itr++)
    {
      to_insert["id"] = GetNextId(lock_transaction, "procedure_reasons");
      to_insert["last_updated_by"] = current_userid;
      to_insert["last_update_stamp"] = RightNow();
      to_insert["current_id"] = "4";
      to_insert["condition_id"] = ending_state["id"];
      to_insert["procedure_id"] = (*itr);
      ExecuteInsertStatement(lock_transaction, "procedure_reasons", to_insert);
    }
  }
  catch(...)
  {
   PSay("Failed to save changes to procedure justification (procedure-to-condition relationship)", m_program_error, MB_OK);
   return false;
  }
  return TformRecordGeneric::BeforeSave();
}
//---------------------------------------------------------------------------
bool TformRecordCondition::CheckProcedure(const map<string, string> & row_data)
{
 // if there's a link, check it!
 map<string, string>::const_iterator i_link = row_data.find("Linking Id");
 if (i_link != row_data.end() && i_link->second != "")
  return true;
 else
  return false;
}
//---------------------------------------------------------------------------
void TformRecordCondition::AtlasRefreshListView(int request_num, string matched_value)
{
 if (request_num == procedure_request || request_num == procedure_reason_request)
  DownloadProcedures();
  //conditions_filter.RefreshDataSet();
}
//---------------------------------------------------------------------------
void TformRecordCondition::DownloadProcedures()
{
 string sql;
 if (IsBlankOrZero(initial_state["task_id"]))
 {
  // some conditions don't have a task_id, and ... we were showing -odd- results in the listview.
  // this could be changed instead to a select that returns no rows, if you like.
  sql = "select '', '', NULL, NULL, NULL, NULL, NULL from uno where 1=0";
 } else if (is_new_record)
 {
  sql = "select procedures.cpt_hcpcs, procedures.local_use, procedures.start_date, procedures.stop_date, "
        "procedures.current_id, procedures.id, NULL "
        "from procedures "
        " where procedures.task_id = '" + initial_state["task_id"] + "'";
 } else
 {
  sql = "select procedures.cpt_hcpcs, procedures.local_use, procedures.start_date, procedures.stop_date, "
        "procedures.current_id, procedures.id, procedure_reasons.id linking_id "
        "from procedures "
        "left join procedure_reasons on procedure_reasons.procedure_id = procedures.id "
        "and procedure_reasons.condition_id = '" + initial_state["id"] + "' "
        "where procedures.task_id = '" + initial_state["task_id"] + "'";
 }
 procedures_filter.SetSQLStatement(sql);
 procedures_filter.RefreshDataSet();
}
//---------------------------------------------------------------------------
void __fastcall TformRecordCondition::listProceduresChanging(TObject *Sender, TListItem *Item, TItemChange Change,
                                                             bool &AllowChange)
{
 // find some good way to lock this so they don't think they're changing it when they're not ...
 // damn, this prevents Selected / double-clicked stuff, not just checked/unchecked. damn.

 if (Change == ctState)
  AllowChange = (is_editing || is_new_record || procedures_filter.IsRefreshingListview());
 else
  AllowChange = true;

 return;
}
//---------------------------------------------------------------------------
void TformRecordCondition::AddProcedure()
{
  map<string,string> temp_map;
  temp_map["id"] = "new";
  temp_map["task_id"] = initial_state["task_id"];
  if (initial_state["task_id"] != "" && initial_state["task_id"] != "0")
   temp_map["start_date"] = GetOneField(dbDatabase, "select date_of_task from tasks where id = " + initial_state["task_id"]);
  FORM_TRY
    TformRecordProcedure * procedureRecord = new TformRecordProcedure(NULL, temp_map);
    procedureRecord->Show();
  FORM_CATCH
}
//---------------------------------------------------------------------------
void TformRecordCondition::EditProcedure()
{
  map<string,string> temp_map;
  temp_map["id"] = procedures_filter.GetSelectedRecordID();
  FORM_TRY
    TformRecordProcedure * procedureRecord = new TformRecordProcedure(NULL, temp_map);
    procedureRecord->Show();
  FORM_CATCH
}
//---------------------------------------------------------------------------
void TformRecordCondition::Do(map<string, string> & p_parameters)
{
 FORM_TRY
  TformRecordCondition * z = new TformRecordCondition(NULL, p_parameters);
  z->Show();
 FORM_CATCH
}
//---------------------------------------------------------------------------
void __fastcall TformRecordCondition::FormShow(TObject *Sender)
{
  TformRecordGeneric::FormShow(Sender);

  if (non_reportable_info_text != "")
  {
    // TODO -c HACK: lblInformation starts out with Width = 0. This causes it to initially dimension the text as if it's
    // printing a single word on each row down the screen with a \n between them, returning a wildly incorrect
    // value for height to the code in ShowInfoBar that does form resizing.
    lblInformation->Width = scrollStandard->Width - 48;
    InfoBarText = non_reportable_info_text;
    ShowInfoBar();
  }
}
//---------------------------------------------------------------------------

