//---------------------------------------------------------------------------

#include <vcl.h>
#pragma hdrstop

#include "codeRecordConsent.h"
//---------------------------------------------------------------------------
#pragma package(smart_init)
#pragma link "codeRecordGeneric"
#pragma resource "*.dfm"
TformRecordConsent *formRecordConsent;
//---------------------------------------------------------------------------
__fastcall TformRecordConsent::TformRecordConsent(TComponent* Owner, map<string,string> p_parameters)
        : TformRecordGeneric(Owner, p_parameters, string("consents"))
{
 Initialize();
}
//---------------------------------------------------------------------------
bool TformRecordConsent::ScreenStart()
{
  lookupTables->RegisterComboBox("consent_types", cboType);

  TformRecordGeneric::ScreenStart();
  return true;
}
//---------------------------------------------------------------------------
bool TformRecordConsent::ScreenStop()
{                                                            
  lookupTables->UnregisterComboBox("consent_types", cboType);
  TformRecordGeneric::ScreenStop();
  return true;
}
//---------------------------------------------------------------------------
bool TformRecordConsent::IsValidOnScreen()
{
  // check to make sure the date entered is valid
  // give the user a message box and prompt to re-enter if incorrect
  // make sure there is a link to a person, although there's not much the user
  // can do to fix that if there's not
  if (! lookupTables->IsInLookUpTable("consent_types", cboType->Text.c_str()))
  {
    PSay("Please enter a valid type of consent", m_invalid_data, MB_OK);
    cboType->SetFocus();
    return false;
  }
  else if ((initial_state["person_id"] == "") || (initial_state["person_id"] == "0"))
  {
    PSay("There is no person associated with this record", m_invalid_data, MB_OK);
    return false;
  }
  else
    return true;
}
//---------------------------------------------------------------------------

bool TformRecordConsent::PutDataOnScreen()
{
  // put the values on the form from the map initial_state
  // values may have come from GetDataFromDatabase if it's an existing record,
  // or values may have been passed to the form in this map if it's a new record
  if (initial_state["signature_date"] != "")
    dateSignature->Date = Pdate(initial_state["signature_date"], mdy | yyyy).OutDate();
  else
    dateSignature->Date = Date();
  cboType->Text = PrettyString(lookupTables->ValueOf("consent_types", initial_state["consent_id"])).c_str();
  editNote->Text = SentenceString(initial_state["note"]).c_str();

  // get the person name associated with the person id that was either selected
  // from the database if it's an existing record, or passed to the form if
  // it's a new record. display the name as a link to the person record.
  CreateLink(txtPerson, NULL, this, initial_state["person_id"], "people");
  CreateLink(txtParticipation, NULL, this, initial_state["participations_id"], "participations");

  TformRecordGeneric::PutDataOnScreen();  
  return true; // what can really go wrong?
}
//---------------------------------------------------------------------------
bool TformRecordConsent::GetDataFromScreen()
{
  // place the values currently on the screen into
  // the map that will be used to update or insert the record
  // the parent form will call the insert or update function, and get the id
  // number if necessary and take care of current_id
  ending_state["signature_date"] = Pdate(dateSignature->Date).OutDate(mdy | yyyy);
  ending_state["consent_id"] = lookupTables->IdOf("consent_types", cboType->Text.c_str());
  ending_state["note"] = editNote->Text.c_str();
  ending_state["person_id"] = initial_state["person_id"];
  if (initial_state["participations_id"] == "")
    ending_state["participations_id"] = "0";
  else
    ending_state["participations_id"] = initial_state["participations_id"];
  /* TODO : what the... ???  We should probably always use the current_groupid or,
     Philip suggest we let them select from the groups they have permission to */
  if((optionsPermissions["special.lac.general"]["enabled"] != "true") &&
     (optionsPermissions["special.den.general"]["enabled"] != "true") &&
     (optionsPermissions["special.sc.general"]["enabled"] != "true") &&
     (optionsPermissions["special.diab.general"]["enabled"] != "true"))
    ending_state["group_id"] = lookupTables->IdOf("groups", "Third Party Resources");
  else
    ending_state["group_id"] = current_groupid;
  ending_state["id"] = initial_state["id"];

  TformRecordGeneric::GetDataFromScreen();

  return true; // again, not much can go wrong here
}
//---------------------------------------------------------------------------
bool TformRecordConsent::ScreenLock()
{
  // disable panel containing edit boxes
  panelGroup->Enabled = false;
  editNote->ReadOnly = true;
  editNote->TabStop = false;
  TformRecordGeneric::ScreenLock();
  return true;
}
//---------------------------------------------------------------------------
bool TformRecordConsent::ScreenUnlock()
{
  // enable panel containing edit boxes
  panelGroup->Enabled = true;
  editNote->ReadOnly = false;
  editNote->TabStop = true;
  TformRecordGeneric::ScreenUnlock();
  return true;
}
//---------------------------------------------------------------------------
void TformRecordConsent::Do(map<string, string> & p_parameters)
{
 FORM_TRY
  TformRecordConsent * z = new TformRecordConsent(NULL, p_parameters);
  z->Show();
 FORM_CATCH
}
//---------------------------------------------------------------------------

