//---------------------------------------------------------------------------

#include <vcl.h>
#pragma hdrstop

#include "codeRecordContactInfo.h"
//---------------------------------------------------------------------------
#pragma package(smart_init)
#pragma link "codeRecordGeneric"
#pragma link "codeHalfButton"
#pragma resource "*.dfm"
TformRecordContactInfo *formRecordContactInfo;
//---------------------------------------------------------------------------
__fastcall TformRecordContactInfo::TformRecordContactInfo(TComponent* Owner, map<string,string> p_parameters)
        : TformRecordGeneric(Owner, p_parameters, string("person_contact_info"))
{
 Initialize();
}
//---------------------------------------------------------------------------
bool TformRecordContactInfo::ScreenStart()
{
  // clear the rich edit box
  editNotes->Text = "";

  // register combo boxes
  lookupTables->RegisterComboBox("contact_info_types", cboLocation);
  lookupTables->RegisterComboBox("residence_types", cboResidence);
  lookupTables->RegisterComboBox("state_types", cboState);
  lookupTables->RegisterComboBox("country_types", cboCountry);

  TformRecordGeneric::ScreenStart();
  return true;
}
//---------------------------------------------------------------------------
bool TformRecordContactInfo::ScreenStop()
{
  // unregister combo boxes
  lookupTables->UnregisterComboBox("contact_info_types", cboLocation);
  lookupTables->UnregisterComboBox("residence_types", cboResidence);
  lookupTables->UnregisterComboBox("state_types", cboState);
  if (lookupTables->TableName(cboCounty) != "") // if this cbo box is registered
    lookupTables->UnregisterComboBox("county_types", cboCounty);
  lookupTables->UnregisterComboBox("country_types", cboCountry);

  TformRecordGeneric::ScreenStop();
  return true;
}
//---------------------------------------------------------------------------
bool TformRecordContactInfo::IsValidOnScreen()
{
  // check to make sure that the values entered in the combo boxe
  // corrospond to values in that look up table.
  // give the user a message box and prompt to re-enter if incorrect
  // make sure there is a link to a person, although there's not much the user
  // can do to fix that if there's not
  if (cboLocation->Text == "" || !lookupTables->IsInLookUpTable("contact_info_types", cboLocation->Text.c_str()))
  {
    PSay("Please enter a valid location", m_invalid_data, MB_OK);
    cboLocation->SetFocus();
    return false;
  }
  if (editLine1->Text != "" &&
      (editZip->Text == "" ||
       cboCity->Text == "" ||
       cboState->Text == "" ||
       cboCounty->Text == ""))
  {
    int answer = PSay("Since you appear to be entering an address, you should specify a city, state, county, and zip code as well.\n\n"
                      "Note that if you enter the zip code and press Enter, the city, state, and county will automatically be filled in for you.\n\n"
                      "You can continue, but this record will automatically be marked as incomplete due to missing data.\n\n"
                      "Continue with missing data and mark record as incomplete?", m_confirmation, MB_YESNO);
    if (answer == IDYES)
      ending_state["current_id"] = lookupTables->IdOf("current_types", "Incomplete");
    else
      return false;
  }
  if (! lookupTables->IsInLookUpTable("state_types", cboState->Text.c_str()))
  {
    PSay("Please enter a valid state.", m_invalid_data, MB_OK);
    cboState->SetFocus();
    return false;
  }
  else if (cboCounty->Text != "" && !lookupTables->IsInLookUpTable("county_types", cboCounty))
  {
    PSay("The county you entered does not appear within the list.  Please enter a valid county.", m_invalid_data, MB_OK);
    cboCounty->SetFocus();
    return false;
  }
  else if (! lookupTables->IsInLookUpTable("country_types", cboCountry->Text.c_str()))
  {
    PSay("Please enter a valid country.", m_invalid_data, MB_OK);
    cboCountry->SetFocus();
    return false;
  }
  else if (! lookupTables->IsInLookUpTable("residence_types", cboResidence->Text.c_str()))
  {
    PSay("Please enter a valid residence", m_invalid_data, MB_OK);
    cboResidence->SetFocus();
    return false;
  }
  else if ((initial_state["person_id"] == "") || (initial_state["person_id"] == "0"))
  {
    PSay("There is no person associated with this record.", m_missing_data, MB_OK);
    return false;
  }
  else
    return true;
}
//---------------------------------------------------------------------------
bool TformRecordContactInfo::PutDataOnScreen()
{
  // put the values on the form from the map initial_state
  // values may have come from GetDataFromDatabase if it's an existing record,
  // or values may have been passed to the form in this map if it's a new record

  cboLocation->Text = PrettyString(lookupTables->ValueOf("contact_info_types", initial_state["contact_info_id"])).c_str();
  editLine1->Text = FormatAddress(initial_state["line1"]).c_str();
  editLine2->Text = FormatAddress(initial_state["line2"]).c_str();
  cboCity->Text = PrettyString(initial_state["city"]).c_str();
  editZip->Text = FormatZipCode(initial_state["zip"]).c_str();
  cboState->Text = PrettyString(lookupTables->ValueOf("state_types", initial_state["state_id"])).c_str();
  string state_id, county_sql;
  if (!CountyLookupByState(cboState->Text.c_str(), state_id, county_sql, cboCounty)) /* whoops? */ ;
  cboCounty->Text = PrettyString(lookupTables->ValueOf("county_types", initial_state["county_id"], cboCounty)).c_str();
  cboCountry->Text = PrettyString(lookupTables->ValueOf("country_types", initial_state["country_id"])).c_str();
  cboResidence->Text = PrettyString(lookupTables->ValueOf("residence_types", initial_state["residence_id"])).c_str();
  if (initial_state["trust_lands"] == "1")
    checkTrust->Checked = true;
  else
    checkTrust->Checked = false;
  editPhone->Text = FormatPhoneNumber(initial_state["phone_number"]).c_str();
  editEmail->Text = LowercaseString(initial_state["email_address"]).c_str();
  editNotes->Text = SentenceString(initial_state["notes"]).c_str();

  // get the person name associated with the person id that was either selected
  // from the database if it's an existing record, or passed to the form if
  // it's a new record. display the name as a link to the person record.
  CreateLink(txtPerson, NULL, this, initial_state["person_id"], "people");

  if (is_new_record && cboCountry->Text == "")
     cboCountry->Text = "United States";

  TformRecordGeneric::PutDataOnScreen();
  return true; // what can really go wrong?
}
//---------------------------------------------------------------------------
bool TformRecordContactInfo::GetDataFromScreen()
{
  // place the values currently on the screen into
  // the map that will be used to update or insert the record
  // the parent form will call the insert or update function, and get the id
  // number if necessary and take care of current_id

  ending_state["contact_info_id"] = lookupTables->IdOf("contact_info_types", cboLocation->Text.c_str());
  ending_state["line1"] = FormatAddress(editLine1->Text.c_str());
  ending_state["line2"] = FormatAddress(editLine2->Text.c_str());
  ending_state["city"] = cboCity->Text.c_str();
  ending_state["state_id"] = lookupTables->IdOf("state_types", cboState->Text.c_str());
  ending_state["zip"] = StripAllExcept(editZip->Text.c_str(), ints);
  ending_state["county_id"] = lookupTables->IdOf("county_types", cboCounty); // need to pass the cbo box so it can look up the ID since it's got custom sql / item list
  ending_state["country_id"] = lookupTables->IdOf("country_types", cboCountry->Text.c_str());
  ending_state["residence_id"] = lookupTables->IdOf("residence_types", cboResidence->Text.c_str());
  if (checkTrust->Checked)
    ending_state["trust_lands"] = "1";
  else
    ending_state["trust_lands"] = "0";
  ending_state["phone_number"] = StripAllExcept(editPhone->Text.c_str(), uints);
  ending_state["email_address"] = editEmail->Text.c_str();
  ending_state["notes"] = editNotes->Text.c_str();
  ending_state["person_id"] = initial_state["person_id"];
  ending_state["id"] = initial_state["id"];

  TformRecordGeneric::GetDataFromScreen();

  return true; // again, not much can go wrong here
}
//---------------------------------------------------------------------------
bool TformRecordContactInfo::ScreenLock()
{
  // disable panel containing edit boxes
  panelGroup->Enabled = false;
  editNotes->ReadOnly = true;
  editNotes->TabStop = false;
  TformRecordGeneric::ScreenLock();
  return true;
}
//---------------------------------------------------------------------------
bool TformRecordContactInfo::ScreenUnlock()
{
  // enable panel containing edit boxes
  panelGroup->Enabled = true;
  editNotes->ReadOnly = false;
  editNotes->TabStop = true;
  TformRecordGeneric::ScreenUnlock();
  return true;
}
//---------------------------------------------------------------------------
void __fastcall TformRecordContactInfo::btnZipLookupClick(TObject *Sender)
{
  string zip_code = editZip->Text.c_str(), state_name, county_name, preferred_city_name;
  vector<map<string, string> > zip_vec;
  if (!ZipLookup(zip_code, zip_vec, state_name, county_name, preferred_city_name, cboState, cboCounty, cboCity))
  {
    PSay("Zip code not found.  Please check that it is in either 5 digit or 5-4 digit format, and that the number is "
         "correct.  If you are sure that this zip code is correct and continue to get this error, contact a programmer.",
         m_invalid_data, MB_OK);
  }
}
//---------------------------------------------------------------------------
void __fastcall TformRecordContactInfo::cboStateChange(TObject *Sender)
{
  string state_id, county_sql;
  if (!CountyLookupByState(cboState->Text.c_str(), state_id, county_sql, cboCounty)) /* whoops */ ;
}
//---------------------------------------------------------------------------
void __fastcall TformRecordContactInfo::cboStateExit(TObject *Sender)
{
  string state_id, county_sql;
  if (!CountyLookupByState(cboState->Text.c_str(), state_id, county_sql, cboCounty)) /* whoops */ ;
}
//---------------------------------------------------------------------------
void __fastcall TformRecordContactInfo::editZipKeyPress(TObject *Sender,char &Key)
{
  if (Key == VK_RETURN)
    btnZipLookup->Click();
}
//---------------------------------------------------------------------------
void __fastcall TformRecordContactInfo::editEmailKeyPress(TObject *Sender,char &Key)
{
  if (Key == VK_RETURN)
  {
    btnEmail->Click();
  }
}
//---------------------------------------------------------------------------
void __fastcall TformRecordContactInfo::btnEmailClick(TObject *Sender)
{
 AnsiString x = "mailto:" + editEmail->Text;
 ShellExecute(0, "open", static_cast<const char*>(x.data()), NULL, NULL, SW_SHOW);
}
//---------------------------------------------------------------------------
void TformRecordContactInfo::Do(map<string, string> & p_parameters)
{
 FORM_TRY
  TformRecordContactInfo * z = new TformRecordContactInfo(NULL, p_parameters);
  z->Show();
 FORM_CATCH
}
//---------------------------------------------------------------------------

