//---------------------------------------------------------------------------

#include <vcl.h>
#pragma hdrstop

#include "codeRecordCrashLog.h"
#include "vectoredexception.h"
//---------------------------------------------------------------------------
#pragma package(smart_init)
#pragma link "codeRecordGeneric"
#pragma resource "*.dfm"
TformRecordCrashLog *formRecordCrashLog;
//---------------------------------------------------------------------------
__fastcall TformRecordCrashLog::TformRecordCrashLog(TComponent* Owner, map<string, string> p_parameters)
        : TformRecordGeneric(Owner, p_parameters, string("crash_logs")), debugMapData()
{
  classFilterObject *filter = filter_atlas_handler["occurrences"].filter;
  filter->SetListViewHandle(listOccurrences);
  filter->SetSecurity("crash_logs");
  filter->AddColumnSpec("Uploaded By",     "crash_logs",  as_text,  true,  120);
  filter->AddColumnSpec("Uploaded At",     "crash_logs",  as_stamp, true,  110);
  filter->AddColumnSpec("Error Timestamp", "crash_logs",  as_stamp, true,  110);
  filter->AddColumnSpec("Current Id",      "current_id",  as_int,   false,  80);
  filter->AddColumnSpec("Internal Id",     "internal_id", as_int,   false,  80);
  filter->FetchPreferences("recordcrashlog.occurrences_filter");
  filter->SetAddEdit(this, NULL, NULL);
  filter->SetIconStatus(true);
  filter->SetSQLStatement("");
  filter->RefreshDataSet();

  Initialize();
}
//---------------------------------------------------------------------------
bool TformRecordCrashLog::ScreenStart()
{
  lookupTables->RegisterComboBox("bug_status_types",   cboBugStatus);
  lookupTables->RegisterComboBox("bug_priority_types", cboBugPriority);

  string user_sql =
    "select id, item, valid from users where id = 0 or "
    " (valid = 1 and group_id = " + lookupTables->IdOf("groups", "MIS") + ") "
    "order by item ";
  lookupTables->RegisterComboBox("users", cboAssignedTo, user_sql);

  return TformRecordGeneric::ScreenStart();
}
//---------------------------------------------------------------------------
bool TformRecordCrashLog::ScreenStop()
{
  lookupTables->UnregisterComboBox("bug_status_types",   cboBugStatus);
  lookupTables->UnregisterComboBox("bug_priority_types", cboBugPriority);
  lookupTables->UnregisterComboBox("users",              cboAssignedTo);
  return TformRecordGeneric::ScreenStop();
}
//---------------------------------------------------------------------------
const char *TformRecordCrashLog::DescriptionForAppState(int appstate)
{
  static const char *PromStateNames[PROM_NUMSTATES] =
  {
    "PROM_STATE_UNINIT",
    "PROM_STATE_CONSTRUCTING",
    "PROM_STATE_CONSTRUCTED",
    "PROM_STATE_STARTUP",
    "PROM_STATE_RUNNING",
    "PROM_STATE_WANTCLOSE",
    "PROM_STATE_FORMCLOSE",
    "PROM_STATE_DESTROYING"
  };

  return (appstate >= 0 && appstate < PROM_NUMSTATES) ? PromStateNames[appstate] : "Unknown";
}
//---------------------------------------------------------------------------
// PhraseForException
// Gets a text string describing the type of exception that occurred.
const char *TformRecordCrashLog::PhraseForException(DWORD code)
{
  struct exceptiondata_t
  {
    DWORD       code;
    const char *name;
  };

  static exceptiondata_t ExceptionData[] =
  {
    { 0x40010005, "Ctrl+C"                     },
    { 0x40010008, "Ctrl+Brk"                   },
    { 0x80000001, "Guard Page Violation"       },
    { 0x80000002, "Data Type Misalignment"     },
    { 0x80000003, "Breakpoint"                 },
    { 0xC0000005, "Access Violation"           },
    { 0xC0000006, "In-Page Error"              },
    { 0xC0000017, "No Memory"                  },
    { 0xC000001D, "Illegal Instruction"        },
    { 0xC0000025, "Non-continuable Exception"  },
    { 0xC0000026, "Invalid Disposition"        },
    { 0xC000008C, "Array Bounds Exceeded"      },
    { 0xC000008D, "Float Denormal Operand"     },
    { 0xC000008E, "Float Divide by Zero"       },
    { 0xC000008F, "Float Inexact Result"       },
    { 0xC0000090, "Float Invalid Operation"    },
    { 0xC0000091, "Float Overflow"             },
    { 0xC0000092, "Float Stack Check"          },
    { 0xC0000093, "Float Underflow"            },
    { 0xC0000094, "Integer Divide by Zero"     },
    { 0xC0000095, "Integer Overflow"           },
    { 0xC0000096, "Privileged Instruction"     },
    { 0xC00000FD, "Stack Overflow"             },
    { 0xC0000142, "DLL Initialization Failure" },
    { 0xE06D7363, "Microsoft C++"              },

    // jhaley 20110627: For Prometheus - Copied from Borland's RTL\rtlinc\xx.h file:
    { 0x0EEFFACE, "Borland C++"                },
    { 0x0EEDFADE, "Borland Delphi"             },

    // must be last
    { 0, NULL }
  };

  const char *ret = "Unknown";
  exceptiondata_t *ed = ExceptionData;

  while(ed->name)
  {
    if(code == ed->code)
    {
      ret = ed->name;
      break;
    }
    ++ed;
  }

  return ret;
}
//---------------------------------------------------------------------------
string TformRecordCrashLog::FormatBytesAtEIP(string source)
{
  string dest;

  for(size_t i = 0; i < source.size(); i++)
  {
    if(i && !(i % 2))
      dest += ' ';
    dest += tolower(source[i]);
  }

  return dest;
}
//---------------------------------------------------------------------------
void TformRecordCrashLog::ProcessStack(const string &input)
{
  memoStack->Lines->Clear();

  // Address  B - Y - T - E - S (16 per row)         ASCII chars (. for non-printables?)
  // aabbccdd xxxxxxxx xxxxxxxx xxxxxxxx xxxxxxxx    asciiasciiasciia
  //  ^^^ starting at ESP and +16 for each successive row (stack grows downward in memory with ESP at "top" which is
  //      the lowest address numerically)

  size_t stacklen = 0;
  unsigned char *stack = DecodeBase64(input, stacklen); // Decode the Base64 string into binary data
  if(stack != NULL && stacklen > 0) // successfully decoded?
  {
    DWORD  current_esp = (DWORD)(StringToInt(initial_state["ctx_esp"]));
    string current_line;
    string ascii;
    size_t numlines = stacklen / 16;
    size_t stackidx = 0;

    if(numlines == 0) // For the case of less than 16 bytes (which is terribly unlikely to happen)
      numlines = 1;

    for(size_t i = 0; i < numlines; i++)
    {
      current_line.erase();
      ascii.erase();

      // Add address to the beginning of the line
      current_line += IntToHexString(current_esp, 8, false) + ": ";
      current_esp  += 16;

      // Place 16 bytes on this line
      for(size_t j = 0; j < 16 && stackidx < stacklen; j++, stackidx++)
      {
        if(j && !(j % 4)) // space between every 4
          current_line += " ";
        current_line += IntToHexString((unsigned int)(stack[stackidx]), 2, false);
        ascii += (stack[stackidx] >= 32) ? (char)(stack[stackidx]) : '.';
      }

      // Add the ASCII output to the end of the line
      current_line += "    " + ascii;

      // Add to the memo control
      memoStack->Lines->Add(current_line.c_str());
    }
  }

  if(stack)
    delete [] stack;
}
//---------------------------------------------------------------------------
bool TformRecordCrashLog::PutDataOnScreen()
{
  // Basic editable record fields:
  cboBugStatus->Text   = lookupTables->ValueOf("bug_status_types",   initial_state["bug_status_id"]  ).c_str();
  cboBugPriority->Text = lookupTables->ValueOf("bug_priority_types", initial_state["bug_priority_id"]).c_str();
  cboAssignedTo->Text  = lookupTables->ValueOf("users",              initial_state["assigned_to"]    ).c_str();

  // Read-Only Crash Information

  // Basic Info
  editProgram->Text  = initial_state["program_name"].c_str();
  editVersion->Text  = (initial_state["exe_version_major"] + "." +
                        initial_state["exe_version_minor"] + "." +
                        initial_state["exe_version_build"]).c_str();
  editModule->Text   = initial_state["p_module_name"].c_str();
  editAppState->Text = DescriptionForAppState(StringToInt(initial_state["app_state"]));
  if(initial_state["log_segment"] != "" || initial_state["log_address"] != "") // This info is not always available
  {
    editLogicalAddr->Text = (IntToHexString(StringToInt(initial_state["log_segment"]), 4, false) + ":" +
                             IntToHexString(StringToInt(initial_state["log_address"]), 8, false)).c_str();
  }
  else
    editLogicalAddr->Text = "Indeterminate";
  editTimeStamp->Text = initial_state["error_timestamp"].c_str();

  // Concatenate exception code as hex string and exception description for known exception codes
  int   exception_code_int   = StringToInt(initial_state["code"]);
  DWORD exception_code_dword = (DWORD)(exception_code_int); // convert to unsigned
  editException->Text = (IntToHexString(exception_code_int, 8, true) + " - " +
                         PhraseForException(exception_code_dword) + " Exception").c_str(); 

  // User Information
  editRunBy->Text      = initial_state["run_by"].c_str(); // NB: Is Windows domain user name, not Prometheus user name
  editOSVersion->Text  = (initial_state["os_version_major"] + "." +
                          initial_state["os_version_minor"] + "." +
                          initial_state["os_version_build"] + " (Platform " +
                          initial_state["os_platform_id"]   + ")").c_str();
  editProcessors->Text = initial_state["num_procs"].c_str();
  editProcType->Text   = initial_state["proc_type"].c_str();
  editMemInUse->Text   = initial_state["mem_in_use"].c_str();
  editPhysMem->Text    = (initial_state["phys_mem_free"] + "/" + initial_state["phys_mem"]).c_str();
  editPageFile->Text   = initial_state["page_file"].c_str();
  editPageFree->Text   = initial_state["page_file_free"].c_str();
  editUserSpace->Text  = initial_state["user_addrspace"].c_str();
  editUserFree->Text   = initial_state["user_freespace"].c_str();

  // Access Violation Information (only valid if it was an Access Violation Exception)
  if(initial_state["av_address"] != "")
  {
    editAVAddress->Text = IntToHexString(StringToInt(initial_state["av_address"]), 8, true).c_str();
    if(StringToInt(initial_state["av_mode"]) != 0) // Any non-zero value means a write instruction caused the AV
      editAVMode->Text = "Write";
    else
      editAVMode->Text = "Read";
  }
  else
  {
    editAVAddress->Text = "(Not an access violation)";
    editAVMode->Text    = "";
  }

  // CPU Registers - tres x86 specifique, non?
  editEdi->Text    = IntToHexString(StringToInt(initial_state["ctx_edi"]),    8, false).c_str(); // Index Registers
  editEsi->Text    = IntToHexString(StringToInt(initial_state["ctx_esi"]),    8, false).c_str();
  editSegCs->Text  = IntToHexString(StringToInt(initial_state["ctx_segcs"]),  8, false).c_str(); // CS:EIP
  editEip->Text    = IntToHexString(StringToInt(initial_state["ctx_eip"]),    8, false).c_str();
  editSegSs->Text  = IntToHexString(StringToInt(initial_state["ctx_segss"]),  8, false).c_str(); // SS:ESP
  editEsp->Text    = IntToHexString(StringToInt(initial_state["ctx_esp"]),    8, false).c_str();
  editEFlags->Text = IntToHexString(StringToInt(initial_state["ctx_eflags"]), 8, false).c_str(); // EFlags
  editEbp->Text    = IntToHexString(StringToInt(initial_state["ctx_ebp"]),    8, false).c_str(); // EBP (Stack Frame)
  editEax->Text    = IntToHexString(StringToInt(initial_state["ctx_eax"]),    8, false).c_str(); // General Purpose Regs
  editEbx->Text    = IntToHexString(StringToInt(initial_state["ctx_ebx"]),    8, false).c_str();
  editEcx->Text    = IntToHexString(StringToInt(initial_state["ctx_ecx"]),    8, false).c_str(); // sometimes == "this"
  editEdx->Text    = IntToHexString(StringToInt(initial_state["ctx_edx"]),    8, false).c_str();

  // Format the Bytes-at-EIP string so it appears like "01 23 45 67 89 ab cd ef..."
  editEIPBytes->Text = FormatBytesAtEIP(initial_state["cs_eip"]).c_str();

  // Stack data
  ProcessStack(initial_state["stack"]);

  return TformRecordGeneric::PutDataOnScreen();
}
//---------------------------------------------------------------------------
bool TformRecordCrashLog::IsValidOnScreen()
{
  if(!lookupTables->IsInLookUpTable("bug_status_types", cboBugStatus))
  {
    PSay("Please enter a valid Bug Status value", m_invalid_data, MB_OK);
    return false;
  }
  if(!lookupTables->IsInLookUpTable("users", cboAssignedTo))
  {
    PSay("Please assign this bug to a valid user", m_invalid_data, MB_OK);
    return false;
  }
  if(!lookupTables->IsInLookUpTable("bug_priority_types", cboBugPriority))
  {
    PSay("Please enter a valid Bug Priority value", m_invalid_data, MB_OK);
    return false;
  }
  return true;
}
//---------------------------------------------------------------------------
bool TformRecordCrashLog::GetDataFromScreen()
{
  ending_state["id"]              = initial_state["id"];
  ending_state["bug_status_id"]   = lookupTables->IdOf("bug_status_types",   cboBugStatus  );
  ending_state["bug_priority_id"] = lookupTables->IdOf("bug_priority_types", cboBugPriority);
  ending_state["assigned_to"]     = lookupTables->IdOf("users",              cboAssignedTo );
  
  return TformRecordGeneric::GetDataFromScreen();
}
//---------------------------------------------------------------------------
bool TformRecordCrashLog::ScreenLock()
{
  cboBugStatus->Enabled   = false;
  cboBugPriority->Enabled = false;
  cboAssignedTo->Enabled  = false;
  return TformRecordGeneric::ScreenLock();
}
//---------------------------------------------------------------------------
bool TformRecordCrashLog::ScreenUnlock()
{
  cboBugStatus->Enabled   = true;
  cboBugPriority->Enabled = true;
  cboAssignedTo->Enabled  = true;
  return TformRecordGeneric::ScreenUnlock();
}
//---------------------------------------------------------------------------
bool TformRecordCrashLog::NotNewAnymore()
{
  classFilterObject *filter = filter_atlas_handler["occurrences"].filter;

  string occurrences_sql =
    "select U.item, CO.creation_stamp, CO.error_timestamp, 4, CO.id "
    "from crash_occurrences CO "
    "left join users U on CO.created_by = U.id "
    "where CO.crash_log_id = '" + initial_state["id"] + "' ";

  filter->SetSQLStatement(occurrences_sql);
  filter->RefreshDataSet();

  return TformRecordGeneric::NotNewAnymore();
}
//---------------------------------------------------------------------------
bool TformRecordCrashLog::IsNewAgain()
{
  return TformRecordGeneric::IsNewAgain();
}
//---------------------------------------------------------------------------
void __fastcall TformRecordCrashLog::btnAddrConvertClick(TObject *Sender)
{
  if(editStackAddress->Text == "")
  {
    PSay("Please enter an address first", m_missing_data, MB_OK);
    return;
  }

  string physical_addr_str = editStackAddress->Text.c_str();
  int    phys_addr_int = HexStringToInt(physical_addr_str);
  PVOID  phys_addr     = (PVOID)phys_addr_int;
  DWORD  segment;
  DWORD  offset;

  if(GetLogicalAddress(phys_addr, segment, offset))
  {
    editStackSeg->Text  = IntToHexString(segment, 4, false).c_str();
    editStackOffs->Text = IntToHexString(offset,  8, false).c_str();
  }
  else
    PSay("Not a valid address in this program", m_message, MB_OK);
}
//---------------------------------------------------------------------------
void TformRecordCrashLog::DebugMap::readMapFile(const string &filename)
{
  const char *data = PLoadTextFile(filename.c_str());
  const char *rover = data;
  unsigned int current_address;
  unsigned int largest_address = 0;
  string address, symbol;
  int state = 0;

  // Borland C++ programs are always based at 0x401000 by default.
  base = 0x401000u;

  if(!data)
    return; // can't load the file.

  while(*rover != '\0')
  {
    switch(state)
    {
    case 0: // reading in address
      switch(*rover)
      {
      case ' ': // space: end of the address
        state = 1;
        break;
      default: // anything else is part of the number
        address += *rover;
      }
      break;
    case 1: // reading in whitespace between address and symbol
      switch(*rover)
      {
      case ' ': // space: keep going
        break;
      default: // anything else is the start of the symbol
        symbol = "";
        symbol += *rover;
        state = 2;
        break;
      }
      break;
    case 2: // reading in symbol
      switch(*rover)
      {
      case '\r': // ignore \r
        break;
      case '\n': // end of the line
        // push the address/symbol pair to the map
        current_address = base + (unsigned int)(HexStringToInt(address)); // add in image base
        symbols[current_address] = symbol;
        // compare to largest address
        if(current_address > largest_address)
          largest_address = current_address;
        address = "";
        symbol  = "";
        state = 0;
        break;
      default: // anything else is part of the symbol
        symbol += *rover;
        break;
      }
      break;
    }
    ++rover;
  }

  len = (largest_address - base) + 0x1000u;

  delete [] data;
}
//---------------------------------------------------------------------------
string TformRecordCrashLog::DebugMap::nameForAddress(unsigned int address)
{
  string ret = "Unknown";
  SymbolMap::iterator firstGreater;

  if(address >= base && address < base + len)
  {
    firstGreater = symbols.upper_bound(address);

    // back up one - this should give us the function this address lies inside.
    if(firstGreater != symbols.begin())
      --firstGreater;

    if(firstGreater != symbols.end())
      ret = firstGreater->second; // return the name of the function.
  }

  return ret;
}
//---------------------------------------------------------------------------
bool TformRecordCrashLog::IsProbableRetnTarget(unsigned int address)
{
  unsigned char *ptr = reinterpret_cast<unsigned char *>(address);

  try
  {
    // Check for register calls
    unsigned char *twoback = ptr - 2;
    if(*twoback == 0xFFu)
    {
      switch(*(twoback + 1))
      {
      case 0x10u:
      case 0x11u:
      case 0x12u:
      case 0x13u:
      case 0x16u:
      case 0x17u:
      case 0xD0u:
      case 0xD1u:
      case 0xD2u:
      case 0xD3u:
      case 0xD4u:
      case 0xD5u:
      case 0xD6u:
      case 0xD7u:
        return true;
      default:
        break;
      }
    }

    // Check for three-byte calls
    unsigned char *threeback = ptr - 3;
    if(*threeback == 0xFFu)
    {
      switch(*(threeback + 1))
      {
      case 0x14u:
        return true;
      case 0x54u:
      case 0x55u:
        if(*(threeback + 2) == 0)
          return true;
        break;
      default:
        break;
      }
    }

    // Check first for relative call and far call
    unsigned char *fiveback = ptr - 5;
    if(*fiveback == 0xE8u || *fiveback == 0x9Au)
      return true;

    // Check for absolute call
    unsigned char *sixback = ptr - 6;
    if(*sixback == 0xFFu && *(sixback + 1) == 0x15u)
      return true;
  }
  __except(EXCEPTION_EXECUTE_HANDLER)
  {
    // Woops!
  }

  return false;
}
//---------------------------------------------------------------------------
void __fastcall TformRecordCrashLog::btnLoadMapClick(TObject *Sender)
{
  if(openDialog->Execute())
  {
    memoTrace->Lines->Clear();
    
    // Parse the file.
    debugMapData.readMapFile(openDialog->FileName.c_str());

    // TODO: walk the stack dump looking for DWORD-aligned addresses inside the base to len range.
    // When one is found, look for the function name in the debug map.
    // Then, add one line to the memo control for each function. The stack trace will read chronologically from
    // the newest call at the top to the oldest one at the bottom.
    size_t stacklen = 0;
    unsigned char *stack = DecodeBase64(initial_state["stack"], stacklen); // Decode the Base64 string into binary data

    if(stack != NULL && stacklen > 0) // successfully decoded?
    {
      size_t stackidx = 0;
      size_t stackwords = stacklen / 4;
      DWORD  current_word = 0;

      if(stackwords == 0) // For the case of less than 16 bytes (which is terribly unlikely to happen)
        stackwords = 1;

      for(size_t i = 0; i < stackwords; i++)
      {
        current_word = 0;

        // Grab 4 bytes for this address
        for(size_t j = 0; j < 4 && stackidx < stacklen; j++, stackidx++)
          current_word |= (unsigned int)(stack[stackidx]) << (j * 8);

        if(current_word >= debugMapData.getBase() && current_word < debugMapData.getBase() + debugMapData.getLen() &&
           IsProbableRetnTarget(current_word))
        {
          string newline = IntToHexString(current_word, 8, false) + ": " + debugMapData.nameForAddress(current_word);
          // Add to the memo control
          memoTrace->Lines->Add(newline.c_str());
        }
      }
    }

    if(stack)
      delete [] stack;
  }
}
//---------------------------------------------------------------------------

