//---------------------------------------------------------------------------

#include <vcl.h>
#pragma hdrstop

#include "codeRecordDeferredCOB.h"
#include "codeSearchPerson.h"
#include "codeSearchTaxEntity.h"
#include "codeSearchServiceProvided.h"
#include "codeSearchTaskCob.h"
#include "codeSearchDenial.h"
#include "launchFormLetter.h"
//---------------------------------------------------------------------------
#pragma package(smart_init)
#pragma link "codeRecordGeneric"
#pragma resource "*.dfm"
TformRecordDeferredCOB *formRecordDeferredCOB;
//---------------------------------------------------------------------------
__fastcall TformRecordDeferredCOB::TformRecordDeferredCOB(TComponent* Owner, map<string,string> p_parameters)
        : TformRecordGeneric(Owner, p_parameters, string("deferred_cobs"))
{
  Initialize();
}
//---------------------------------------------------------------------------
bool TformRecordDeferredCOB::ScreenStart()
{
  txtPrintLetter->Font->Color = link_color;

  lookupTables->RegisterComboBox("deferral_reason_types", cboReason);
  lookupTables->RegisterComboBox("deferral_status_types", cboStatus);

  return TformRecordGeneric::ScreenStart();
}
//---------------------------------------------------------------------------
bool TformRecordDeferredCOB::ScreenStop()
{
  lookupTables->UnregisterComboBox("deferral_reason_types", cboReason);
  lookupTables->UnregisterComboBox("deferral_status_types", cboStatus);

  return TformRecordGeneric::ScreenStop();
}
//---------------------------------------------------------------------------
bool TformRecordDeferredCOB::IsValidOnScreen()
{
  // required links:
  if(IsBlankOrZero(GetValidLinkId(txtPerson)))
  {
    PSay("Please attach a valid person to this deferred COB.", m_missing_data, MB_OK);
    btnPersonSearch->SetFocus();
    return false;
  }
  if(IsBlankOrZero(GetValidLinkId(txtService)))
  {
    PSay("Please specify the service to be provided if this COB is approved.", m_missing_data, MB_OK);
    btnServiceSearch->SetFocus();
    return false;
  }

  // required lookups:
  if(cboReason->Text == "" || !lookupTables->IsInLookUpTable("deferral_reason_types", cboReason))
  {
    PSay("Please provide a valid reason for deferring this COB.", m_invalid_data, MB_OK);
    cboReason->SetFocus();
    return false;
  }
  if(cboStatus->Text == "" || !lookupTables->IsInLookUpTable("deferral_status_types", cboStatus))
  {
    PSay("Please provide a valid status for this deferred COB.", m_invalid_data, MB_OK);
    cboStatus->SetFocus();
    return false;
  }

  // If status is "Pending", there should not be a COB or a denial attached.
  if(cboStatus->Text == "Pending" && (!IsBlankOrZero(GetValidLinkId(txtCOB)) || !IsBlankOrZero(GetValidLinkId(txtDenial))))
  {
    PSay("A deferred COB with status of 'Pending' should not have a COB or a Denial attached.", m_invalid_data, MB_OK);
    cboStatus->SetFocus();
    return false;
  }

  // If status is "COB Created", there should be a COB attached, but no denial.
  if(cboStatus->Text == "COB Created")
  {
    if(IsBlankOrZero(GetValidLinkId(txtCOB)))
    {
      PSay("A deferred COB with status of 'COB Created' should be linked to the COB.", m_missing_data, MB_OK);
      btnCOBSearch->SetFocus();
      return false;
    }
    if(!IsBlankOrZero(GetValidLinkId(txtDenial)))
    {
      PSay("A deferred COB with status of 'COB Created' should not be linked to a denial.", m_invalid_data, MB_OK);
      btnDenialSearch->SetFocus();
      return false;
    }
  }

  // If status is "Denied", there should be a denial attached, but no COB.
  if(cboStatus->Text == "Denied")
  {
    if(IsBlankOrZero(GetValidLinkId(txtDenial)))
    {
      PSay("A deferred COB with status of 'Denied' should be linked to the denial.", m_missing_data, MB_OK);
      btnDenialSearch->SetFocus();
      return false;
    }
    if(!IsBlankOrZero(GetValidLinkId(txtCOB)))
    {
      PSay("A deferred COB with status of 'Denied' should not be linked to a COB.", m_invalid_data, MB_OK);
      btnCOBSearch->SetFocus();
      return false;
    }
  }

  return true;
}
//---------------------------------------------------------------------------
bool TformRecordDeferredCOB::PutDataOnScreen()
{
  // create links
  CreateLink(txtPerson,     btnPersonSearch,   this, initial_state["person_id"],            "people");
  CreateLink(txtReferredBy, btnReferredSearch, this, initial_state["provider_id"],          "tax_entities");
  CreateLink(txtService,    btnServiceSearch,  this, initial_state["services_provided_id"], "services_provided");
  CreateLink(txtCOB,        btnCOBSearch,      this, initial_state["cob_id"],               "tasks");
  CreateLink(txtDenial,     btnDenialSearch,   this, initial_state["denial_id"],            "denials");

  // data
  if(initial_state["date_of_deferral"] != "")
    dateDeferred->Date = Pdate(initial_state["date_of_deferral"], mdy | yyyy).OutDate();
  else
    dateDeferred->Date = Date();

  checkApproved->Checked = (initial_state["is_approved"] == "1");

  cboReason->Text = lookupTables->ValueOf("deferral_reason_types", initial_state["deferral_reason_id"]).c_str();
  cboStatus->Text = lookupTables->ValueOf("deferral_status_types", initial_state["deferral_status_id"]).c_str();

  if(initial_state["id"] == "new" && cboStatus->Text == "")
    cboStatus->Text = "Pending";

  memoComments->Text = SentenceString(initial_state["comments"]).c_str();

  return TformRecordGeneric::PutDataOnScreen();
}
//---------------------------------------------------------------------------
bool TformRecordDeferredCOB::GetDataFromScreen()
{
  ending_state["id"] = initial_state["id"];

  // links
  ending_state["person_id"]            = GetValidLinkId(txtPerson);
  ending_state["provider_id"]          = GetValidLinkId(txtReferredBy);
  ending_state["services_provided_id"] = GetValidLinkId(txtService);
  ending_state["cob_id"]               = GetValidLinkId(txtCOB);
  ending_state["denial_id"]            = GetValidLinkId(txtDenial);

  // data
  ending_state["date_of_deferral"] = Pdate(dateDeferred->Date).OutDate(mdy | yyyy);

  ending_state["is_approved"] = checkApproved->Checked ? "1" : "0";

  ending_state["deferral_reason_id"] = lookupTables->IdOf("deferral_reason_types", cboReason);
  ending_state["deferral_status_id"] = lookupTables->IdOf("deferral_status_types", cboStatus);

  ending_state["comments"] = memoComments->Text.c_str();

  return TformRecordGeneric::GetDataFromScreen();
}
//---------------------------------------------------------------------------
bool TformRecordDeferredCOB::ScreenLock()
{
  panelSearchBtns->Enabled = false;
  panelDate->Enabled       = false;
  panelReason->Enabled     = false;
  panelSearch2->Enabled    = false;
  panelComments->Enabled   = false;

  return TformRecordGeneric::ScreenLock();
}
//---------------------------------------------------------------------------
bool TformRecordDeferredCOB::ScreenUnlock()
{
  panelSearchBtns->Enabled = true;
  panelDate->Enabled       = true;
  panelReason->Enabled     = true;
  panelSearch2->Enabled    = true;
  panelComments->Enabled   = true;

  return TformRecordGeneric::ScreenUnlock();
}
//---------------------------------------------------------------------------
bool TformRecordDeferredCOB::IsNewAgain()
{
  return TformRecordGeneric::IsNewAgain();
}
//---------------------------------------------------------------------------
bool TformRecordDeferredCOB::NotNewAnymore()
{
  return TformRecordGeneric::NotNewAnymore();
}
//---------------------------------------------------------------------------
void __fastcall TformRecordDeferredCOB::btnPersonSearchClick(TObject *Sender)
{
  // create and show new person search form
  vector<string> return_vector;
  TformSearchPerson *formPersonSearch = NULL;

  FORM_TRY
    formPersonSearch = new TformSearchPerson(NULL, &return_vector);
  FORM_CATCH_RETURN("TformSearchPerson", )

  formPersonSearch->ShowTaskModal(this);
  if(!return_vector.empty())
    CreateLink(txtPerson, btnPersonSearch, this, return_vector[0], "people");
}
//---------------------------------------------------------------------------
void __fastcall TformRecordDeferredCOB::btnReferredSearchClick(TObject *Sender)
{
  // create and show tax entity search form
  vector<string> return_vector;
  TformSearchTaxEntity *providerSearch = NULL;

  FORM_TRY
    providerSearch = new TformSearchTaxEntity(NULL,&return_vector);
  FORM_CATCH_RETURN("TformSearchTaxEntity", )

  providerSearch->ShowTaskModal(this);
  if(!return_vector.empty())
    CreateLink(txtReferredBy, btnReferredSearch, this, return_vector[0], "tax_entities");
}
//---------------------------------------------------------------------------
void __fastcall TformRecordDeferredCOB::btnServiceSearchClick(TObject *Sender)
{
  vector<string> return_vector = TformSearchServiceProvided::Do(this);

  if(!return_vector.empty())
    CreateLink(txtService, btnServiceSearch, this, return_vector[0], "services_provided");
}
//---------------------------------------------------------------------------
void __fastcall TformRecordDeferredCOB::btnCOBSearchClick(TObject *Sender)
{
  vector<string> return_vector = TformSearchTaskCob::Do(this);

  if(!return_vector.empty())
    CreateLink(txtCOB, btnCOBSearch, this, return_vector[0], "tasks");
}
//---------------------------------------------------------------------------
void __fastcall TformRecordDeferredCOB::btnDenialSearchClick(TObject *Sender)
{
  vector<string> return_vector = TformSearchDenial::Do(this);

  if(!return_vector.empty())
    CreateLink(txtDenial, btnDenialSearch, this, return_vector[0], "denials");
}
//---------------------------------------------------------------------------
void __fastcall TformRecordDeferredCOB::txtPrintLetterClick(TObject *Sender)
{
  pmapstrs variables;

  if(IsBlankOrZero(initial_state["id"]) || !IsInt(initial_state["id"]))
  {
    PSay("Please save this record before printing a letter for it.", m_reminder, MB_OK);
    return;
  }

  string referredby_name;

  if(!IsBlankOrZero(initial_state["provider_id"]))
    referredby_name = string("referral from ") + GetOneField(dbDatabase, "select name from tax_entities where id = " + initial_state["provider_id"]);
  else
    referredby_name = "emergency services notice"; // No referring provider implies it is a 72hr ER notice.

  string provider_name =
    GetOneField(dbDatabase,
                "select TE.name from services_provided SP inner join tax_entities TE on SP.tax_entity_id = TE.id "
                "where SP.id = " + initial_state["services_provided_id"]);

  string reason_text =
    GetOneField(dbDatabase,
                "select description from deferral_reason_types where id = " + initial_state["deferral_reason_id"]);

  variables["$referred_by$"] = referredby_name;
  variables["$provider$"]    = provider_name;
  variables["$reason$"]      = reason_text;

  TformLaunchFormLetter *formLetter = NULL;
  FORM_TRY
    formLetter = new TformLaunchFormLetter(NULL, initial_state["person_id"], &variables);
  FORM_CATCH_RETURN("TformLaunchFormLetter", )

  formLetter->cboLetterType->Text = "Deferred COB Letter";
  formLetter->Show();
  formLetter->btnRunReportClick(NULL);
}
//---------------------------------------------------------------------------

