//---------------------------------------------------------------------------

#include <vcl.h>
#pragma hdrstop

#include "codeRecordDenial.h"
#include "codeSearchPerson.h"
#include "codeSearchServiceProvided.h"
#include "codeRecordDenialLetter.h"
//---------------------------------------------------------------------------
#pragma package(smart_init)
#pragma link "codeRecordGeneric"
#pragma link "codeRubberContainer"
#pragma link "codeFrameFilterBar"
#pragma resource "*.dfm"
TformRecordDenial *formRecordDenial;
//---------------------------------------------------------------------------
__fastcall TformRecordDenial::TformRecordDenial(TComponent* Owner, map<string,string> p_parameters)
        : TformRecordGeneric(Owner, p_parameters, string("denials"))
{
  //person_id = p_parameters["person_id"];
  //services_provided_id = p_parameters["services_provided_id"];
  tax_entity_id = "";

  // jhaley 20100701: add filter for letters
  letters_filter.SetListViewHandle(listLetters);
  letters_filter.SetSecurity("denial_letters");
  letters_filter.AddColumnSpec("Authority", "denial_letters", as_text, true, 130, rawString);
  letters_filter.AddColumnSpec("Status", "denial_letters", as_text, true, 60);
  letters_filter.AddColumnSpec("Date Processed", "denial_letters", as_date, true, 100);
  letters_filter.AddColumnSpec("Effective Date", "denial_letters", as_date, true, 90);
  letters_filter.AddColumnSpec("Current Id", "current_id", as_int, false, 80);
  letters_filter.AddColumnSpec("Internal Id", "internal_id", as_int, false, 80);
  letters_filter.FetchPreferences("recorddenial.letters");
  letters_filter.SetSQLStatement("");
  letters_filter.RefreshDataSet();

  // jhaley 20110510: First testing ground for the filter bar. The filter bar makes all that well-hidden functionality
  // in the context menu for filters exposed to the end user. You wouldn't believe how many users don't even know you
  // can right-click a filtered listview... it's driving me nuts! Obviously not all filters are going to get a filter
  // bar but the most important ones, like this one, certainly will.
  frameFilterBar->SetHandles(this, &letters_filter);
  frameFilterBar->SetEditEnabled(false);
  frameFilterBar->SetAddEnabled(false);

  Initialize();
}
//---------------------------------------------------------------------------
bool TformRecordDenial::IsValidOnScreen()
{
  if (GetValidLinkId(txtPerson) == "")
  {
    PSay("Please enter a person", m_missing_data, MB_OK);
    btnPersonSearch->Show();
    btnPersonSearch->SetFocus();
    return false;
  }
  else if (GetValidLinkId(txtService) == "")
  {
    PSay("Please enter a service", m_missing_data, MB_OK);
    btnServiceSearch->Show();
    btnServiceSearch->SetFocus();
    return false;
  }

  return true;
}
//---------------------------------------------------------------------------
bool TformRecordDenial::PutDataOnScreen()
{
  CreateLink(txtPerson,  btnPersonSearch,  this, initial_state["person_id"],            "people");
  CreateLink(txtService, btnServiceSearch, this, initial_state["services_provided_id"], "services_provided");

  TformRecordGeneric::PutDataOnScreen();
  return true;
}
//---------------------------------------------------------------------------
bool TformRecordDenial::GetDataFromScreen()
{
  ending_state["id"] = initial_state["id"];
  ending_state["person_id"] = GetValidLinkId(txtPerson);
  ending_state["services_provided_id"] = GetValidLinkId(txtService);

  TformRecordGeneric::GetDataFromScreen();
  return true;
}
//---------------------------------------------------------------------------
bool TformRecordDenial::ScreenLock()
{
  panelGroup->Enabled = false;
  TformRecordGeneric::ScreenLock();
  return true;
}
//---------------------------------------------------------------------------
bool TformRecordDenial::ScreenUnlock()
{
  panelGroup->Enabled = true;
  TformRecordGeneric::ScreenUnlock();
  return true;
}
//---------------------------------------------------------------------------
void __fastcall TformRecordDenial::btnPersonSearchClick(TObject *Sender)
{
  vector<string> return_vector = TformSearchPerson::Do(this);
  if (!return_vector.empty())
    CreateLink(txtPerson, btnPersonSearch, this, return_vector[0], "people");
}
//---------------------------------------------------------------------------
void __fastcall TformRecordDenial::btnServiceSearchClick(TObject *Sender)
{
  vector<string> return_vector = TformSearchServiceProvided::Do(this, tax_entity_id);
  if (!return_vector.empty())
    CreateLink(txtService, btnServiceSearch, this, return_vector[0], "services_provided");
}
//---------------------------------------------------------------------------
bool TformRecordDenial::NotNewAnymore()
{
  letters_filter.SetSQLStatement("select denial_authority_types.item, denial_status_types.item, denial_letters.date_processed, denial_letters.effective_date, "
                                 "denial_letters.current_id, denial_letters.id "
                                 "from denial_letters "
                                 "inner join denials on denials.id = denial_letters.denial_id "
                                 "inner join denial_authority_types on denial_letters.denial_authority_id = denial_authority_types.id "
                                 "inner join denial_status_types on denial_letters.denial_status_id = denial_status_types.id "
                                 "where denials.id = '" + initial_state["id"] + "' ");
  letters_filter.RefreshDataSet();
  letters_filter.SetAddEdit(this,
                            AddRecord(&(TformRecordDenial::AddLetter)),
                            EditRecord(&(TformRecordDenial::EditLetter)));
  letters_request = atlas->RequestNotificationWhen("denial_letters.id", "", rel_any, as_int, this, RefreshHandler(&(TformRecordDenial::AtlasRefresh)));

  // Check HowToEdit/Add to make permissions kick in on this determination
  if(letters_filter.HowToEdit())
    frameFilterBar->SetEditEnabled(true);
  if(letters_filter.HowToAdd())
    frameFilterBar->SetAddEnabled(true);

  TformRecordGeneric::NotNewAnymore();
  return true;
}
//---------------------------------------------------------------------------
bool TformRecordDenial::IsNewAgain()
{
  letters_filter.SetSQLStatement("");
  letters_filter.RefreshDataSet();
  letters_filter.SetAddEdit(this, NULL, NULL);

  TformRecordGeneric::IsNewAgain();
  return true;
}
//---------------------------------------------------------------------------
bool TformRecordDenial::AfterSave()
{
  AtlasSend();
  return TformRecordGeneric::AfterSave();
}
//---------------------------------------------------------------------------
bool TformRecordDenial::BeforeSave()
{
  AtlasClear();

  // TODO: is any other handling needed here?
  return TformRecordGeneric::BeforeSave();
}
//---------------------------------------------------------------------------
bool TformRecordDenial::AtlasRefresh(int request_num, string matched_value)
{
  if (request_num == letters_request)
    letters_filter.RefreshDataSet();

  return true;
}
//---------------------------------------------------------------------------
void TformRecordDenial::AddLetter()
{
  map<string, string> temp_map;
  temp_map["id"] = "new";
  temp_map["denial_id"] = initial_state["id"];
  TformRecordDenialLetter::Do(temp_map);
}
//---------------------------------------------------------------------------
void TformRecordDenial::EditLetter()
{
  map<string, string> temp_map;
  temp_map["id"] = letters_filter.GetSelectedRecordID();
  TformRecordDenialLetter::Do(temp_map);
}
//---------------------------------------------------------------------------
/* I moved ending_state["id"] = initial_state["id"]; to GetDataFromScreen to fix the hack,
   and switched the code that was storing link data in initial_state to instead track that
   in private member variables so that CheckForUselessRecord will actually see differences
   between initial_state and ending_state and therefore not complain.  --Paul 1/10/2010 */
/*
// jhaley 20100708: overridden because a new denial only contains key fields,
//   which are ignored by the parent class implementation.
// 20100924: Turns out you can't just return true from here or you'll get into trouble when updating!
bool TformRecordDenial::CheckForUselessRecord()
{
  // Don't save identical data again, as there's no point to it.
  if (is_editing)
  {
    if (ending_state["person_id"] == person_id && ending_state["services_provided_id"] == services_provided_id)
    {
      PSay("You do not seem to have made any changes to the record that can be saved.", m_missing_data, MB_OK);
      return false;
    }
    // TODO -c HACK: Why is this needed? I have nooo idea. Somehow it's short-circuiting something in RecordGeneric and
    // the id never gets put in ending_state... haven't figured it out yet. Without this, fieldMap["id"] is blank in
    // ExecuteUpdateStatement and an "Unexpected end of query" error will occur.
    ending_state["id"] = initial_state["id"];
  }
  return true;
}
*/
//---------------------------------------------------------------------------

