//---------------------------------------------------------------------------
#include <vcl.h>
#pragma hdrstop

#include "codeRecordDenialLetter.h"
#include "codeSearchPerson.h"
#include "codeSearchServiceProvided.h"
#include "reportCHSBillingDenialLetter.h"
#include "reportDenialLetter.h"
//---------------------------------------------------------------------------
#pragma package(smart_init)
#pragma link "codeRecordGeneric"
#pragma link "codeRubberContainer"
#pragma resource "*.dfm"
TformRecordDenialLetter *formRecordDenialLetter;
//---------------------------------------------------------------------------
__fastcall TformRecordDenialLetter::TformRecordDenialLetter(TComponent* Owner, map<string,string> p_parameters)
        : TformRecordGeneric(Owner, p_parameters, string("denial_letters"))
{
  reasons_filter.SetListViewHandle(listReasons);
  reasons_filter.SetSecurity("denial_letters");
  reasons_filter.AddColumnSpec("Reason", "denial_letters", as_text, true, 200, rawString);
  reasons_filter.AddColumnSpec("Description", "denial_letters", as_text, false, 120, rawString);
  reasons_filter.AddColumnSpec("Internal Id", "internal_id", as_int, false, 80);
  reasons_filter.AddColumnSpec("Is Linked", "~", as_int, false, 80);
  reasons_filter.FetchPreferences("recorddenialletter.reasons");
  reasons_filter.SetCheck(true);
  reasons_filter.SetCompress(true);
  reasons_filter.SetSQLStatement("");
  reasons_filter.SetCheck(this, CheckRecord(&(TformRecordDenialLetter::CheckReason)));
  reasons_filter.SetCheckLocked(true);
  reasons_filter.SetHideUnchecked(true);
  reasons_filter.RefreshDataSet();

  // jhaley 20100630: add filter for denial_items view
  // jhaley 20101011: create vecmap source for denial items filter
  FORM_TRY
    items_source = new classFilterVecmapSource();
  FORM_CATCH
  items_source->SetFilter(&items_filter);

  // TEST:
  listDatesAndAmounts->OnSelectItem = OnDOSSelectItem;

  items_filter.SetListViewHandle(listDatesAndAmounts);
  items_filter.SetSecurity("denial_letters");
  items_filter.AddColumnSpec("Begin Date", "denial_letters", as_date, true, 95);
  items_filter.AddColumnSpec("End Date", "denial_letters", as_date, true, 70);
  items_filter.AddColumnSpec("Billed Amount", "denial_letters", as_double, true, 80, amountString);
  items_filter.AddColumnSpec("Patient Owes", "denial_letters", as_double, true, 80, amountString);
  items_filter.AddColumnSpec("Payable Amount", "denial_letters", as_double, true, 90, amountString);
  items_filter.AddColumnSpec("Current Id", "current_id", as_int, false, 80);
  items_filter.AddColumnSpec("Internal Id", "internal_id", as_int, false, 80);
  items_filter.AddColumnSpec("Edited", "denial_letters", as_text, false, 35); // DEBUG
  items_filter.FetchPreferences("recorddenialletter.items");
  items_filter.SetIconStatus(true);
  items_filter.SetSQLStatement("");
  items_filter.SetDataSource(items_source);
  items_filter.RefreshDataSet();
  items_filter.SetAddEdit(this,
                          AddRecord(&(TformRecordDenialLetter::AddItem)),
                          EditRecord(&(TformRecordDenialLetter::EditItem)));

  // jhaley 20110428: custom letter date always starts out as today's date
  dateCustomLetter->Date = Now();

  //person_id = p_parameters["person_id"];
  tax_entity_id = "";

  txtPrintDenialLetter->Font->Color = link_color;

  authority_permissions = GetSpecialGroupPermissions("general");

  SetDOSEnabled(false);

  Initialize();
}
//---------------------------------------------------------------------------
bool TformRecordDenialLetter::ScreenStart()
{
  lookupTables->RegisterComboBox("denial_status_types",    cboStatus);
  lookupTables->RegisterComboBox("denial_authority_types", cboDenialType);
  TformRecordGeneric::ScreenStart();
  return true;
}
//---------------------------------------------------------------------------
bool TformRecordDenialLetter::ScreenStop()
{
  lookupTables->UnregisterComboBox("denial_status_types",    cboStatus);
  lookupTables->UnregisterComboBox("denial_authority_types", cboDenialType);
  TformRecordGeneric::ScreenStop();
  return true;
}
//---------------------------------------------------------------------------
// jhaley 20100707: subroutine for IsValidOnScreen
// Returns true if the record conflicts (ala Enrollment method of same name)
// TODO: now used in 3 classes; could be abstracted into generic record class
bool TformRecordDenialLetter::ConflictsWithExistingRecord()
{
  set<string> has_auth_set; // set to track authority type IDs that have been seen during loop
  vector<map<string, string> > letter_map;
  string current_authority    = cboDenialType->Text.c_str();
  string current_authority_id = lookupTables->IdOf("denial_authority_types", current_authority);

  // get authority IDs and effective dates for all letters in the same denial batch:
  string sql_str = "select denial_letters.denial_authority_id, denial_letters.effective_date "
                   "from denial_letters "
                   "where denial_letters.denial_id = '" + initial_state["denial_id"] + "' ";

  // changing an existing record? do not match self.
  if (!is_new_record)
    sql_str += "and denial_letters.id != '" + initial_state["id"] + "' ";

  SqlToVecMap(dbDatabase, sql_str, letter_map);

  // Loop Checks:
  // * This denial letter should be the only one of its authority type in the current batch
  // * This denial letter needs a date consistent with its authority type vs the other letters
  for (vector<map<string, string> >::iterator i_item = letter_map.begin(); i_item != letter_map.end(); i_item++)
  {
    // check to see if a letter of the given denial type exists in this batch already
    if ((*i_item)["denial_authority_id"] == current_authority_id)
    {
      if (current_authority == "")
        current_authority = "current";
      PSay("There is already a denial letter in the current batch for the " + current_authority + " authority.",
           m_warning, MB_OK);
      cboDenialType->SetFocus();
      return true;
    }
    else // keep track of the authority types that have been seen by id (bit of a unique solution here)
      has_auth_set.insert((*i_item)["denial_authority_id"]);

    // Check effective date:
    // * If TPB, this should be earlier than any existing letter (though none should exist, ideally)
    // * If CMMR:
    //   * If authority of older item is TPB, this letter should be later than that date
    //   * If authority of older item is Executive Director, this should be earlier than that date
    // * If Executive Director, this should be later than any existing letter
    Pdate current_date = Pdate(dateEffective->Date);
    Pdate item_date    = Pdate();
    item_date.InDate((*i_item)["effective_date"], mdy | yyyy);
    if (current_authority_id == lookupTables->IdOf("denial_authority_types", "Third Party Billing")
        && current_date > item_date)
    {
      PSay("The effective date for this letter should be earlier than or equal to " + item_date.OutDate(mdy | yyyy),
           m_warning, MB_OK);
      dateEffective->SetFocus();
      return true;
    }
    else if (current_authority_id == lookupTables->IdOf("denial_authority_types", "CMMR"))
    {
      if ((*i_item)["denial_authority_id"] == lookupTables->IdOf("denial_authority_types", "Third Party Billing"))
      {
        // A TBP should be earlier or equal to a CMMR
        if (item_date > current_date)
        {
          PSay("The effective date for this letter should be later than or equal to " + item_date.OutDate(mdy | yyyy),
               m_warning, MB_OK);
          dateEffective->SetFocus();
          return true;
        }
      }
      else if ((*i_item)["denial_authority_id"] == lookupTables->IdOf("denial_authority_types", "Executive Director"))
      {
        // An Executive letter should be later or equal to a CMMR
        if (item_date < current_date)
        {
          PSay("The effective date for this letter should be earlier than or equal to " + item_date.OutDate(mdy | yyyy),
               m_warning, MB_OK);
          dateEffective->SetFocus();
          return true;
        }
      }
    }
    else if (current_authority_id == lookupTables->IdOf("denial_authority_types", "Executive Director")
             && current_date < item_date)
    {
      PSay("The effective date for this letter should be later than or equal to " + item_date.OutDate(mdy | yyyy),
           m_warning, MB_OK);
      dateEffective->SetFocus();
      return true;
    }
  } // end loop

  // Check that this letter has the pre-requisite letters existing for it:
  // * Third Party Billing: requires nothing.
  // * CMMR: As of 7/28/2010, this *likes* to have a TPB letter, but no longer requires it due to some special
  //   circumstances in the TPB department where they need to skip a step ahead in the process. However, if the
  //   CMMR letter is the only letter, it should not be an approval (8/16/2010).
  // * Executive: As of 7/28/2010, only requires a CMMR letter.
  if(current_authority_id == lookupTables->IdOf("denial_authority_types", "CMMR"))
  {
    if (has_auth_set.find(lookupTables->IdOf("denial_authority_types", "Third Party Billing")) == has_auth_set.end() &&
        cboStatus->Text == "Resolved")
    {
      PSay("When creating a CMMR letter without a pre-existing Third Party Billing letter, the status of the letter should be 'Denied' only.",
           m_warning, MB_OK);
      cboStatus->SetFocus();
      return true;
    }
  }
  else if(current_authority_id == lookupTables->IdOf("denial_authority_types", "Executive Director"))
  {
    if (has_auth_set.find(lookupTables->IdOf("denial_authority_types", "CMMR")) == has_auth_set.end())
    {
      PSay("This denial batch requires a CMMR letter before an Executive Director letter can be created.",
           m_warning, MB_OK);
      cboDenialType->SetFocus();
      return true;
    }
  }

  // All is well.
  return false;
}
//---------------------------------------------------------------------------
bool TformRecordDenialLetter::IsValidOnScreen()
{
  string current_id = ending_state["current_id"];
  if (current_id == "") current_id = initial_state["current_id"];

  // jhaley 20100708: verify at least one reason exists
  if (reasons_filter.GetMultiID().empty())
  {
    PSay("Please select one or more reasons for this denial letter.", m_invalid_data, MB_OK);
    return false;
  }

  // Validate all DOS items
  if (!AllDOSAreValid())
    return false;

  // need to check for valid authority type before looking for permissions:
  if (!lookupTables->IsInLookUpTable("denial_authority_types", cboDenialType) || cboDenialType->Text == "")
  {
    PSay("Please select a valid type of denial.", m_invalid_data, MB_OK);
    cboDenialType->SetFocus();
    return false;
  }
  else // check for permissions vs authority type
  {
    string group_id = GetOneField(dbDatabase, "select group_id from denial_authority_types where id = '"
                                               + lookupTables->IdOf("denial_authority_types", cboDenialType) + "' ");
    if (authority_permissions.find(group_id) == authority_permissions.end())
    {
      PSay("You do not have permission to edit that type of denial.", m_invalid_data, MB_OK);
      cboDenialType->SetFocus();
      return false;
    }
  }

  if (!lookupTables->IsInLookUpTable("denial_status_types", cboStatus) || cboStatus->Text == "")
  {
    PSay("Please select the status of this denial.", m_invalid_data, MB_OK);
    cboStatus->SetFocus();
    return false;
  }
  else if (cboStatus->Text == "Denied" && // Denied requires a current_id of Deleted, Primary, Current, or Incomplete and NO appeal approval date
           (current_id != "" && (current_id == lookupTables->IdOf("current_types", "Expired") || current_id == lookupTables->IdOf("current_types", "Incorrect"))))
  {
    string current_type = LowercaseString(lookupTables->ValueOf("current_types", current_id));
    PSay(
      "This denial is active, with a status of Denied, but the current state (" + current_type + ") doesn't make sense.\n\n"
      "If this denial is active, it should be marked as Current.\n"
      "If the denial is an error, it should be marked as Deleted.\n"
      , m_warning, MB_OK);
    return false;
  }
  else if (cboStatus->Text == "Resolved" && // Resolved requires a current_id of Deleted or Expired
           (current_id != "" && current_id != lookupTables->IdOf("current_types", "Deleted") && current_id != lookupTables->IdOf("current_types", "Expired")))
  {
    string current_type = LowercaseString(lookupTables->ValueOf("current_types", current_id));
    PSay(
      "This denial is inactive, with a status of Resolved, but the current state (" + current_type + ") doesn't make sense.\n\n"
      "If this denial is resolved, it should be marked as Expired.\n"
      "If the denial is an error, it should be marked as Deleted.\n"
      , m_warning, MB_OK);
    return false;
  }
  else if (cboStatus->Text == "Resolved" && // TPB letters should not have a status of Resolved
           initial_state["denial_authority_id"] == lookupTables->IdOf("denial_authority_types", "Third Party Billing"))
  {
    PSay("A letter for this authority should not have a status of Resolved.\n"
         "Please set status to Denied or create a new letter in this denial batch for your authority.",
         m_warning, MB_OK);
    return false;
  }
  else if (ConflictsWithExistingRecord()) // jhaley 20100707: check for conflicts with existing letters
    return false;

  /*    if (response == IDYES)
    { // I'm not sure which ones need to change, so change them all.
      // TODO: is this going to screw up CheckForUselessRecord?
      string expired = lookupTables->IdOf("current_types", "Expired");
      initial_state["current_id"] = expired;
      hack_state["current_id"] = expired;
      ending_state["current_id"] = expired;
    } else
  */

  return true;
}
//---------------------------------------------------------------------------
// jhaley 20101011: Call to refresh the denial items filter
void TformRecordDenialLetter::UpdateDOSFilter()
{
  vector<map<string, string> > dest_vecmap;

  // put field names in the first row - the numeric prefixes enforce a column order without requiring a custom comparator
  map<string, string> field_names;
  field_names["0_begin_date_of_service"] = "0_begin_date_of_service";
  field_names["1_end_date_of_service"]   = "1_end_date_of_service";
  field_names["2_billed_amount"]         = "2_billed_amount";
  field_names["3_patient_amount"]        = "3_patient_amount";
  field_names["4_payable_amount"]        = "4_payable_amount";
  field_names["5_current_id"]            = "5_current_id";
  field_names["6_id"]                    = "6_id";
  field_names["7_edited"]                = "7_edited";
  dest_vecmap.push_back(field_names);

  // put all the current denial items into the vecmap
  for (vector<map<string, string> >::iterator i_dos = current_dos_data.begin(); i_dos != current_dos_data.end(); i_dos++)
  {
    map<string, string> dest_row; // a bit unfortunate that this is necessary, but an ordering must be enforced.
    dest_row["0_begin_date_of_service"] = (*i_dos)["begin_date_of_service"];
    dest_row["1_end_date_of_service"]   = (*i_dos)["end_date_of_service"];
    dest_row["2_billed_amount"]         = (*i_dos)["billed_amount"];
    dest_row["3_patient_amount"]        = (*i_dos)["patient_amount"];
    dest_row["4_payable_amount"]        = (*i_dos)["payable_amount"];
    dest_row["5_current_id"]            = (*i_dos)["current_id"];
    dest_row["6_id"]                    = (*i_dos)["id"];
    dest_row["7_edited"]                = (*i_dos)["edited"];
    dest_vecmap.push_back(dest_row);
  }

  items_source->SetSource(dest_vecmap);
  items_filter.RefreshDataSet();
}
//---------------------------------------------------------------------------
// jhaley 20101011: Call to enable or disable the DOS editing control group
void TformRecordDenialLetter::SetDOSEnabled(bool p_enabled)
{
  // Changes for Paul: clear controls when locked
  if (!p_enabled)
  {
    dateDOSBegin->Date = Now();
    dateDOSEnd->Date   = Now();
    editBilledAmount->Text = "";
    editPatientOwes->Text = "";
    editPayableAmount->Text = "";
  }

  groupDOS->Enabled = p_enabled;
}
//---------------------------------------------------------------------------
bool TformRecordDenialLetter::PutDataOnScreen()
{
  // put the values on the form from the map initial_state
  // values may have come from GetDataFromDatabase if it's an existing record,
  // or values may have been passed to the form in this map if it's a new record

  // jhaley 20100630: moved to Denial DOS form: dateBegin, dateEnd, editAmount

  // This date is the date of the rollout for the new denial letters system
  Pdate cutoff_date = Pdate("7/21/2010", mdy | yyyy);
  Pdate letter_date_processed = Pdate(initial_state["date_processed"], mdy | yyyy);

  if (!is_new_record && letter_date_processed <= cutoff_date)
  {
    oldLetterInfoText = "Since this denial letter predates " + cutoff_date.OutDate(mdy | yyyy) +
                        ", the information on this screen may appear inaccurate or confusing.";
  }
  else
    oldLetterInfoText = "";

  if (!IsBlankOrZero(initial_state["denial_status_id"]))
    cboStatus->Text = lookupTables->ValueOf("denial_status_types", initial_state["denial_status_id"]).c_str();
  else // default
    cboStatus->Text = "Denied";
  cboStatusChange(NULL);

  // get group ID's for all the letters that already exist
  set<string> has_auth_set;
  string auth_sql = "select denial_authority_types.group_id from denial_authority_types "
                    "inner join denial_letters on denial_letters.denial_authority_id = denial_authority_types.id "
                    "where denial_letters.denial_id = '" + initial_state["denial_id"] + "' ";
  SqlToSet(dbDatabase, auth_sql, has_auth_set);

  if (!IsBlankOrZero(initial_state["denial_authority_id"]))
    cboDenialType->Text = lookupTables->ValueOf("denial_authority_types", initial_state["denial_authority_id"]).c_str();
  else // default
  {
    string ideal_default_authority;
    string group_perm_needed;

    // determine the ideal default authority to select based on the existing group ids
    if (has_auth_set.size() == 0) // none? select TPB.
      ideal_default_authority = lookupTables->IdOf("denial_authority_types", "Third Party Billing");
    else if (has_auth_set.find(lookupTables->IdOf("groups", "Third Party Billing")) != has_auth_set.end() && has_auth_set.size() == 1) // TPB is only letter? select CMMR.
      ideal_default_authority = lookupTables->IdOf("denial_authority_types", "CMMR");
    else if (has_auth_set.find(lookupTables->IdOf("groups", "Contract Health")) != has_auth_set.end()) // If a CMMR letter exists, select Executive Director
      ideal_default_authority = lookupTables->IdOf("denial_authority_types", "Executive Director");

    // get the group permission needed to view the ideal default
    group_perm_needed = GetOneField(dbDatabase, "select group_id from denial_authority_types where id = '" + ideal_default_authority + "' ");

    if (authority_permissions.find(group_perm_needed) != authority_permissions.end()) // has permission? set the default.
      cboDenialType->Text = lookupTables->ValueOf("denial_authority_types", ideal_default_authority).c_str();
    else
      cboDenialType->Text = ""; // Cannot set the default, just leave it blank. They will have to choose one to which they have access.
  }

  // jhaley 20100714: Check reasons from the previous letter if this is a new letter
  //        20100728: Revised for allowing CMMR letters to exist without TPB letters
  if (is_new_record && has_auth_set.size() > 0)
  {
    string prev_auth_id   = "";
    string third_party_id = GetOneField(dbDatabase, "select group_id from denial_authority_types where item = 'Third Party Billing'");
    string cmmr_id        = GetOneField(dbDatabase, "select group_id from denial_authority_types where item = 'CMMR'");

    if (has_auth_set.find(third_party_id) != has_auth_set.end()) // has a TPB letter?
    {
      if (has_auth_set.find(cmmr_id) != has_auth_set.end())
        prev_auth_id = lookupTables->IdOf("denial_authority_types", "CMMR"); // has TPB and CMMR letters; use CMMR letter's reasons
      else
        prev_auth_id = lookupTables->IdOf("denial_authority_types", "Third Party Billing"); // if no CMMR, get reasons from TPB
    }
    else // no TPB letter; check for a CMMR, since CMMR letters no longer require TPB letters to exist.
    {
      if (has_auth_set.find(cmmr_id) != has_auth_set.end())
        prev_auth_id = lookupTables->IdOf("denial_authority_types", "CMMR");
    }
    if (prev_auth_id != "")
    {
      // get a set of all the reasons from the previous letter in the sequence
      set<string> reason_id_set;
      string reasons_sql = "select denial_reason_types.id from denial_reason_types "
                           "inner join denial_reasons on denial_reason_types.id = denial_reasons.denial_reason_id "
                           "where denial_reasons.denial_letter_id = "
                           "(select denial_letters.id from denial_letters "
                           " where denial_letters.denial_authority_id = '" + prev_auth_id + "' "
                           " and denial_letters.denial_id = '" + initial_state["denial_id"] + "') ";
      SqlToSet(dbDatabase, reasons_sql, reason_id_set);

      // check the ones that are present, and uncheck the ones that are not
      reasons_filter.SetCheckedWhereNotID(reason_id_set, false);
      reasons_filter.SetCheckedWhereID(reason_id_set, true);

      // jhaley 20101011: transfer denial items also
      string items_sql = "select denial_items.begin_date_of_service, denial_items.end_date_of_service, denial_items.billed_amount, denial_items.patient_amount, "
                         "denial_items.payable_amount, denial_items.current_id, denial_items.id "
                         "from denial_items "
                         "where denial_items.current_id+1 > 4 and denial_items.denial_letter_id = "
                         "(select denial_letters.id from denial_letters "
                         " where denial_letters.denial_authority_id = '" + prev_auth_id + "' "
                         " and denial_letters.denial_id = '" + initial_state["denial_id"] + "') ";
      original_dos_data.clear();
      SqlToVecMap(dbDatabase, items_sql, original_dos_data); // fetch into original data cache
      // set temporary IDs for new items
      int i = 0;
      for (vector<map<string, string> >::iterator i_dos = original_dos_data.begin(); i_dos != original_dos_data.end(); i_dos++, i++)
      {
        (*i_dos)["id"] = "new" + IntToString(i); // Not a real ID, only for internal tracking.
        (*i_dos)["edited"] = "New";
        (*i_dos)["payable_amount"] = ""; // Payable amounts must be verified
        (*i_dos)["current_id"] = lookupTables->IdOf("current_types", "Incomplete");
      }
      current_dos_data = original_dos_data; // copy into the working data set
      UpdateDOSFilter();
      // Change for Paul: activate add mode immediately
      SetDOSAddMode(false);
    }
  }

  // jhaley 20100707: effective date handling (now required)
  if (initial_state["effective_date"] != "")
    dateEffective->Date = Pdate(initial_state["effective_date"], mdy | yyyy).OutDate();
  else
    dateEffective->Date = Date();

  if (initial_state["date_processed"] == "")
    initial_state["date_processed"] = CurrentDate();
  lblProcessed->Caption = ("Processed on " + initial_state["date_processed"]).c_str();

  // jhaley 20100630: removed search buttons, added person_id get and batch id
  string person_id_str = GetOneField(dbDatabase, "select denials.person_id from denials "
                                                 "where denials.id = '" + initial_state["denial_id"] + "' ");
  CreateLink(txtPerson, NULL, this, person_id_str, "people");
  CreateLink(txtBatch,  NULL, this, initial_state["denial_id"], "denials");

  TformRecordGeneric::PutDataOnScreen();
  return true;
}
//---------------------------------------------------------------------------
bool TformRecordDenialLetter::GetDataFromScreen()
{
  TformRecordGeneric::GetDataFromScreen();
  if (!CreateInsertMap(dbDatabase, "denial_letters", ending_state))
     return false;

  ending_state["id"] = initial_state["id"];
  ending_state["denial_id"] = initial_state["denial_id"]; // jhaley: changed to denial_id
  ending_state["denial_status_id"] = lookupTables->IdOf("denial_status_types", cboStatus);
  ending_state["denial_authority_id"] = lookupTables->IdOf("denial_authority_types", cboDenialType);

  ending_state["effective_date"] = Pdate(dateEffective->Date).OutDate(mdy | yyyy);

  ending_state["date_processed"] = initial_state["date_processed"];

  return true;
}
//---------------------------------------------------------------------------
bool TformRecordDenialLetter::ScreenLock()
{
  reasons_filter.SetCheckLocked(true);
  reasons_filter.SetHideUnchecked(true);
  panelGroup->Enabled = false;
  TformRecordGeneric::ScreenLock();

  // reset DOS filter and editing information
  is_dos_edit_active = false;
  current_editing_dos_id = "";
  SetDOSEnabled(false);
  current_dos_data = original_dos_data;
  UpdateDOSFilter();

  return true;
}
//---------------------------------------------------------------------------
bool TformRecordDenialLetter::ScreenUnlock()
{
  reasons_filter.SetCheckLocked(false);
  reasons_filter.SetHideUnchecked(false);
  panelGroup->Enabled = true;
  // Changes for Paul: Go into add mode immediately
  if(items_filter.GetSelectedRecordID() != "")
    SetDOSEditMode(false);
  else
    SetDOSAddMode(false);
  TformRecordGeneric::ScreenUnlock();
  return true;
}
//---------------------------------------------------------------------------
// jhaley 20101014: Refetch DOS items from the database. This is needed in NotNewAnymore, for bringing up an existing
// letter, and also in AfterSave, so that editing an existing letter refreshes the list.
void TformRecordDenialLetter::ReFetchDOS(string letter_id)
{
  // jhaley 20101011: pull in denial items for editing on this form
  string items_sql = "select denial_items.begin_date_of_service, denial_items.end_date_of_service, denial_items.billed_amount, denial_items.patient_amount, "
                     "denial_items.payable_amount, denial_items.current_id, denial_items.id, 0 "
                     "from denial_items "
                     "where denial_items.denial_letter_id = '" + letter_id + "' ";
  original_dos_data.clear();
  SqlToVecMap(dbDatabase, items_sql, original_dos_data); // fetch into original data cache
  current_dos_data = original_dos_data;                  // copy into the working data set
  UpdateDOSFilter();

  // changes for Paul: activate adding mode
  SetDOSAddMode(false);
}
//---------------------------------------------------------------------------
bool TformRecordDenialLetter::NotNewAnymore()
{
  reasons_filter.SetSQLStatement("select denial_reason_types.item, denial_reason_types.description, denial_reason_types.id, "
                                 "case when denial_reasons.denial_reason_id is not null then 1 else 0 end "
                                 "from denial_reason_types "
                                 "left join denial_reasons on denial_reasons.denial_reason_id = denial_reason_types.id and denial_reasons.denial_letter_id = '" + initial_state["id"] + "' "
                                 "where denial_reason_types.valid = 1 and denial_reason_types.id > 0");
  reasons_filter.RefreshDataSet();
  reason_request = atlas->RequestNotificationWhen("denial_reason_types.id", "", rel_any, as_int, this, RefreshHandler(&(TformRecordDenialLetter::AtlasRefresh)));

  ReFetchDOS(initial_state["id"]);
  //item_request = atlas->RequestNotificationWhen("denial_items.id", "", rel_any, as_int, this, RefreshHandler(&(TformRecordDenialLetter::AtlasRefresh)));

  txtPrintDenialLetter->OnClick = txtPrintDenialLetterClick;
  TformRecordGeneric::NotNewAnymore();
  return true;
}
//---------------------------------------------------------------------------
bool TformRecordDenialLetter::IsNewAgain()
{
  reasons_filter.SetSQLStatement("select denial_reason_types.item, denial_reason_types.description, denial_reason_types.id, 0 "
                                 "from denial_reason_types "
                                 "where denial_reason_types.valid = 1 and denial_reason_types.id > 0");
  reasons_filter.RefreshDataSet();
  reasons_filter.SetAddEdit(this, NULL, NULL);

  original_dos_data.clear();
  current_dos_data.clear();
  UpdateDOSFilter();

  // change for Paul: activate add mode immediately
  is_dos_edit_active = true;
  current_editing_dos_id = "new";
  SetDOSEnabled(true);
  groupDOS->Caption = "Add new dates and amounts";

  TformRecordGeneric::IsNewAgain();
  return true;
}
//---------------------------------------------------------------------------
bool TformRecordDenialLetter::AfterSave()
{
  AtlasSend();
  ReFetchDOS(ending_state["id"]);
  return TformRecordGeneric::AfterSave();
}
//---------------------------------------------------------------------------
bool TformRecordDenialLetter::BeforeSave()
{
  // stolen/adapted from record appointment
  AtlasClear();

  // jhaley 20100708: queue a notification on "fake" denial_letters.person_id field for the person form
  string person_id_str = GetOneField(dbDatabase, "select denials.person_id from denials "
                                                 "where denials.id = '" + ending_state["denial_id"] + "' ");
  if (!person_id_str.empty())
     AtlasAdd("denial_letters.person_id", person_id_str);

  vector<string> previously_checked_ids;
  // needed to send atlas refresh messages
  if (SqlToVec(lock_transaction, "select denial_reasons.denial_reason_id from denial_reasons where denial_letter_id = " + ending_state["id"], previously_checked_ids))
  {
    for (vector<string>::iterator i = previously_checked_ids.begin(); i != previously_checked_ids.end(); i++)
     AtlasAdd("denial_letters.denial_reason_id", *i);
    // delete existing
    if (ExecuteStatement(lock_transaction, "delete from denial_reasons where denial_letter_id = " + ending_state["id"]))
    {
      // add new
      vector<string> t_checked_ids = reasons_filter.GetMultiID();
      set<string> checked_ids;
      VecToSet(t_checked_ids, checked_ids);
      map<string, string> field_map;
      field_map["current_id"] = lookupTables->IdOf("current_types", "Current");
      field_map["last_update_stamp"] = RightNow();
      field_map["last_updated_by"] = current_userid;
      field_map["denial_letter_id"] = ending_state["id"];
      for (set<string>::iterator i = checked_ids.begin(); i != checked_ids.end(); i++)
      {
        field_map["id"] = GetNextId(lock_transaction, "denial_reasons");
        field_map["denial_reason_id"] = *i;
        ExecuteInsertStatement(lock_transaction, "denial_reasons", field_map);
        AtlasAdd("denial_reasons", field_map);
      }
    }
    else
    {
      PSay("Failed to save changes to the list of reasons for denial [deletion phase]", m_database_error, MB_OK);
      return false;
    }
  }
  else
  {
    PSay("Failed to get the previous list of denial reasons", m_database_error, MB_OK);
    return false;
  }

  // jhaley 20101011: save changes to denial items
  for (vector<map<string, string> >::iterator i_dos = current_dos_data.begin(); i_dos != current_dos_data.end(); i_dos++)
  {
    // If the item is new, and hasn't been marked as deleted already (ie. they changed their mind about adding it without
    // ever saving it first), insert the new denial_items record.
    if ((*i_dos)["edited"] == "New" && (*i_dos)["current_id"] != lookupTables->IdOf("current_types", "Deleted"))
    {
      map<string, string> field_map;
      //MergeMap(field_map, *i_dos); -- doesn't work, so don't try to change it to this.
      //field_map.erase("id");     // Remove the dummy ID
      //field_map.erase("edited"); // Get rid of the internal state tracking field
      field_map["id"] = GetNextId(lock_transaction, "denial_items");
      field_map["current_id"] = lookupTables->IdOf("current_types", "Current");
      field_map["last_update_stamp"] = RightNow();
      field_map["last_updated_by"] = current_userid;
      field_map["denial_letter_id"] = ending_state["id"];
      field_map["begin_date_of_service"] = (*i_dos)["begin_date_of_service"];
      field_map["end_date_of_service"] = (*i_dos)["end_date_of_service"];
      field_map["billed_amount"] = (*i_dos)["billed_amount"];
      field_map["payable_amount"] = (*i_dos)["payable_amount"];
      field_map["patient_amount"] = (*i_dos)["patient_amount"];
      if (ExecuteInsertStatement(lock_transaction, "denial_items", field_map))
        AtlasAdd("denial_items", field_map);
      else
      {
        PSay("Unable to save new denial item.", m_database_error, MB_OK);
        return false;
      }
    }
    else if ((*i_dos)["edited"] == "Edited" && !IsBlankOrZero((*i_dos)["id"])) // Has been edited?
    {
      map<string, string> field_map;
      //MergeMap(field_map, *i_dos); -- doesn't work, so don't try to change it to this.
      //field_map.erase("edited");
      field_map["id"] = (*i_dos)["id"];
      field_map["current_id"] = (*i_dos)["current_id"];
      field_map["last_update_stamp"] = RightNow();
      field_map["last_updated_by"] = current_userid;
      field_map["denial_letter_id"] = ending_state["id"];
      field_map["begin_date_of_service"] = (*i_dos)["begin_date_of_service"];
      field_map["end_date_of_service"] = (*i_dos)["end_date_of_service"];
      field_map["billed_amount"] = (*i_dos)["billed_amount"];
      field_map["payable_amount"] = (*i_dos)["payable_amount"];
      field_map["patient_amount"] = (*i_dos)["patient_amount"];
      if (ExecuteUpdateStatement(lock_transaction, "denial_items", field_map))
        atlas->Notify("denial_items.id", field_map["id"]);
      else
      {
        PSay("Unable to update denial item " + field_map["id"] + ".", m_database_error, MB_OK);
        return false;
      }
    }
  }

  return true;

  // habit, but unnecessary.  recordgeneric only returns true so no need for this
  // return result && TformRecordGeneric::BeforeSave();
}
//---------------------------------------------------------------------------
void __fastcall TformRecordDenialLetter::txtPrintDenialLetterClick(TObject *Sender)
{
  vector<string> letters;
  map<string, string> task_data;

  // jhaley 20111005: Don't try to print unsaved letters
  if(IsBlankOrZero(initial_state["id"]) || !IsInt(initial_state["id"]))
  {
    PSay("Please save the letter you are editing first.", m_reminder, MB_OK);
    return;
  }

  letters.push_back(initial_state["id"]);

  if(IsBlankOrZero(initial_state["denial_id"]))
  {
    PSay("An error has occurred; there is not a valid denial batch.\n\n"
         "Please contact a programmer immediately.", m_program_error, MB_OK);
    return;
  }

  string one_date = "";
  if (checkCustomDate->Checked)
    one_date = dateCustomLetter->Date.DateString().c_str();

  // jhaley 20110714: log to CHS mailout tasks
  string tax_entity_sql =
    "select DN.person_id, TE.name "
    "from denials DN "
    "inner join services_provided SP on DN.services_provided_id = SP.id "
    "inner join tax_entities TE on SP.tax_entity_id = TE.id "
    "where DN.id = '" + initial_state["denial_id"] + "' ";

  SqlToMap(dbDatabase, tax_entity_sql, task_data);

  string msg =
    "Printed "
    + string(cboDenialType->Text.c_str()) + " '"
    + string(cboStatus->Text.c_str()) +
    "' letter for services by " + task_data["name"] + " processed on " + initial_state["date_processed"];

  if(!InsertCHSMailTask(NULL, task_data["person_id"], msg))
    PSay("Failed to log denial letter mailout task", m_warning, MB_OK);

  // jhaley 20110315: allow dupes w/CHS chart #
  // jhaley 20110428: allow use of a single custom date from here, as well as on the report launch form
  TformReportDenialLetter::Do(letters, one_date, checkDuplicates->Checked);
}
//---------------------------------------------------------------------------
bool TformRecordDenialLetter::AtlasRefresh(int request_num, string matched_value)
{
  if (request_num == reason_request)
    reasons_filter.RefreshDataSet();
  //else if(request_num == item_request) // jhaley 20100630
  //  items_filter.RefreshDataSet();
  return true;
}
//---------------------------------------------------------------------------
bool TformRecordDenialLetter::CheckReason(const map<string, string> & row_data)
{
  // if there's a link, check it!
  map<string, string>::const_iterator i_link = row_data.find("Is Linked");
  return (i_link != row_data.end() && i_link->second == "1");
}
//---------------------------------------------------------------------------
void TformRecordDenialLetter::Do(map<string, string> & p_parameters)
{
  FORM_TRY
    TformRecordDenialLetter* z = new TformRecordDenialLetter(NULL, p_parameters);
    z->Show();
  FORM_CATCH
}
//---------------------------------------------------------------------------
void __fastcall TformRecordDenialLetter::cboStatusChange(TObject *Sender)
{
  string current_id = ending_state["current_id"];
  if (current_id == "") current_id = initial_state["current_id"];
  if (current_id == lookupTables->IdOf("current_types", "Deleted"))
    return; // Marked as deleted?  don't mess with it.

  if (cboStatus->Text == "Denied")
  {
    string current = lookupTables->IdOf("current_types", "Current");
    initial_state["current_id"] = current;
    hack_state["current_id"] = current;
    ending_state["current_id"] = current;

    txtPrintDenialLetter->Enabled = true;
    txtPrintDenialLetter->Caption = "Print denial letter";
  }
  else if (cboStatus->Text == "Resolved")
  {
    string expired = lookupTables->IdOf("current_types", "Expired");
    initial_state["current_id"] = expired;
    hack_state["current_id"] = expired;
    ending_state["current_id"] = expired;

    // jhaley 20100715: the new business process requires the ability to print approval letters as well.
    // The difference is handled entirely inside the Denial Letters report, but let's be consistent in
    // terminology here.
    txtPrintDenialLetter->Enabled = true;
    txtPrintDenialLetter->Caption = "Print approval letter";
  }
  else
    txtPrintDenialLetter->Enabled = false;

  UpdateCurrent();
}
//---------------------------------------------------------------------------
// jhaley 20100708: Test for permission to add or edit a DOS record.
bool TformRecordDenialLetter::CheckDOSPermissions(const string &action_type, bool silent)
{
  string denial_authority_id;
  string group_id;

  // allow adding to a new record by getting the authority from the combo box in that case
  if (is_new_record)
    denial_authority_id = lookupTables->IdOf("denial_authority_types", cboDenialType->Text.c_str());
  else
    denial_authority_id = initial_state["denial_authority_id"];

  if(!IsBlankOrZero(denial_authority_id))
    group_id = GetOneField(dbDatabase, "select group_id from denial_authority_types where id = '" + denial_authority_id + "' ");
  else
    group_id = "nothing should match"; // shouldn't happen, just for safety.

  if (authority_permissions.find(group_id) == authority_permissions.end() ||
      optionsPermissions["table.denial_items"][action_type] != "true")
  {
    if (!silent)
    {
      PSay("You do not have permission to " + action_type + " dates of service for this type of denial.\n"
           "Please contact your system administrator.", m_warning, MB_OK);
    }
    // Note: do NOT try to focus a control here, or you'll get a nice exception ;)
    return false; // no permission
  }
  else
    return true; // found a permission
}
//---------------------------------------------------------------------------
// jhaley 20101015: Changes for Paul - enabling add mode in various places automatically.
void TformRecordDenialLetter::SetDOSAddMode(bool setfocus)
{
  if (!(is_new_record || is_editing)) // only while creating or updating data
    return;

  // don't do if no insert permissions (be quiet about it though)
  if (!CheckDOSPermissions("insert", true))
    return;

  // jhaley 20101011: activate DOS editing
  is_dos_edit_active = true;
  current_editing_dos_id = "new";
  SetDOSEnabled(true);
  groupDOS->Caption = "Add new dates and amounts";

  dateDOSBegin->Date = Now();
  dateDOSEnd->Date   = Now();
  editBilledAmount->Text = "";
  editPatientOwes->Text = "";
  editPayableAmount->Text = "";
  if (setfocus)
    editBilledAmount->SetFocus();

  // jhaley 20100716: an item for a "Denied" status letter should have a $0.00 payable amount
  if (initial_state["denial_status_id"] == lookupTables->IdOf("denial_status_types", "Denied"))
    editPayableAmount->Text = "0.00";
}
//---------------------------------------------------------------------------
// jhaley 20101018: Changes for Paul - enabling edit mode in various places automatically.
void TformRecordDenialLetter::SetDOSEditMode(bool setfocus)
{
  if (!(is_new_record || is_editing)) // only while creating or updating data
    return;

  // don't do if no update permissions (be quiet about it though)
  if (!CheckDOSPermissions("update", true))
    return;

  // activate DOS editing
  string temp_id = items_filter.GetSelectedRecordID();

  // find the current_dos_data row that matches the selected id
  vector<map<string, string> >::iterator i_row;
  for (i_row = current_dos_data.begin(); i_row != current_dos_data.end(); i_row++)
  {
    if ((*i_row)["id"] == temp_id)
      break;
  }
  if (i_row != current_dos_data.end())
  {
    if ((*i_row)["current_id"] == lookupTables->IdOf("current_types", "Deleted"))
    {
      if (setfocus)
        PSay("You cannot edit a deleted dates-of-service item.", m_reminder, MB_OK);
      SetDOSAddMode(setfocus);
      return;
    }

    is_dos_edit_active = true;
    current_editing_dos_id = temp_id;
    SetDOSEnabled(true);
    groupDOS->Caption = "Edit dates and amounts";

    // "put data on screen"
    dateDOSBegin->Date = Pdate((*i_row)["begin_date_of_service"], mdy | yyyy).OutDate();
    dateDOSEnd->Date   = Pdate((*i_row)["end_date_of_service"],   mdy | yyyy).OutDate();
    editBilledAmount->Text  = AmountToString(StringToAmount((*i_row)["billed_amount"])).c_str();
    editPatientOwes->Text   = AmountToString(StringToAmount((*i_row)["patient_amount"])).c_str();

    if(groupDOS->Enabled && setfocus)
      editBilledAmount->SetFocus();

    // If payable amount isn't valid and the letter state is Resolved, start out with Billed Amount as Payable Amount
    if ((*i_row)["payable_amount"] == "" && cboStatus->Text == "Resolved")
      editPayableAmount->Text = editBilledAmount->Text;
    else
      editPayableAmount->Text = (*i_row)["payable_amount"] == "" ? "0.00" : AmountToString(StringToAmount((*i_row)["payable_amount"])).c_str();
  }
}
//---------------------------------------------------------------------------
// jhaley 20100630: added to allow click on items_filter
void TformRecordDenialLetter::AddItem()
{
  // editing controls must be enabled
  if (!groupDOS->Enabled)
    return;

  if (!CheckDOSPermissions("insert"))
    return;

  SetDOSAddMode();
}
//---------------------------------------------------------------------------
// jhaley 20100630: added to allow click on items_filter
void TformRecordDenialLetter::EditItem()
{
  // editing controls must be enabled
  if (!groupDOS->Enabled)
    return;

  if (!CheckDOSPermissions("update"))
    return;

  SetDOSEditMode(true);
}
//---------------------------------------------------------------------------
// jhaley 20100714: added to kill denial_reasons when deleting a letter
// jhaley 20101012: also delete denial_items now that they are managed by this form alone
bool TformRecordDenialLetter::DeleteRecord()
{
  if (!ExecuteStatement(delete_transaction, "delete from denial_reasons where denial_letter_id = " + initial_state["id"]))
  {
    PSay("Failed to delete denial_reasons; cancelled.", m_database_error, MB_OK);
    return false;
  }

  // jhaley 20101012: Also delete associated denial items
  if (!ExecuteStatement(delete_transaction, "delete from denial_items where denial_letter_id = " + initial_state["id"]))
  {
    PSay("Failed to delete denial_items; cancelled.", m_database_error, MB_OK);
    return false;
  }
  
  return TformRecordGeneric::DeleteRecord();
}
//---------------------------------------------------------------------------
// jhaley 20100719: added to show info bar for old letters
void __fastcall TformRecordDenialLetter::FormShow(TObject *Sender)
{
  TformRecordGeneric::FormShow(Sender);

  if (oldLetterInfoText != "")
  {
    // TODO -c HACK: lblInformation starts out with Width = 0. This causes it to initially dimension the text as if it's
    // printing a single word on each row down the screen with a \n between them, returning a wildly incorrect
    // value for height to the code in ShowInfoBar that does form resizing. My debugger won't work and I can't really
    // understand the code in ShowInfoBar without being able to step it, but here's a workaround: dimension the control
    // to a reasonable size beforehand and there will be significantly less of a problem. Without this, the denial
    // letter form grows to an amazing 729 pixels high, a third of which is blank space.
    lblInformation->Width = scrollStandard->Width - 48;
    InfoBarText = oldLetterInfoText;
    ShowInfoBar();
  }
}
//---------------------------------------------------------------------------
bool TformRecordDenialLetter::CanUpdate()
{
  bool can_update = TformRecordGeneric::CanUpdate();

  // For update permissions, you need permission to modify denial letters for the *original* authority saved for the
  // letter when it was created.
  string group_id = GetOneField(dbDatabase, "select group_id from denial_authority_types where id = '" + initial_state["denial_authority_id"] + "' ");

  if (authority_permissions.find(group_id) == authority_permissions.end())
    can_update = false;

  return can_update;
}
//---------------------------------------------------------------------------
bool TformRecordDenialLetter::CanInsert()
{
  bool can_insert = TformRecordGeneric::CanInsert();

  // For insert permissions, you need permission to insert for the *currently selected* authority.
  if (!lookupTables->IsInLookUpTable("denial_authority_types", cboDenialType) || cboDenialType->Text == "")
    can_insert = false; // Not a valid type? Can't do anything anyway.
  else
  {
    string group_id = GetOneField(dbDatabase, "select group_id from denial_authority_types where id = '"
                                              + lookupTables->IdOf("denial_authority_types", cboDenialType) + "' ");
    if (authority_permissions.find(group_id) == authority_permissions.end())
      can_insert = false;
  }

  return can_insert;
}
//---------------------------------------------------------------------------
// jhaley 20101011: Support for adding DOS items on-form.
void __fastcall TformRecordDenialLetter::btnDOSAddClick(TObject *Sender)
{
  AddItem();
}
//---------------------------------------------------------------------------
// jhaley 20101011: support for marking denial items as deleted on-form.
void __fastcall TformRecordDenialLetter::btnDOSDeleteClick(TObject *Sender)
{
  if (!CheckDOSPermissions("update"))
    return;

  string dos_id = items_filter.GetSelectedRecordID();

  // find the selected row in current_dos_data:
  vector<map<string, string> >::iterator i_row;
  for (i_row = current_dos_data.begin(); i_row != current_dos_data.end(); i_row++)
  {
    if ((*i_row)["id"] == dos_id)
      break;
  }
  if (i_row != current_dos_data.end())
  {
    (*i_row)["current_id"] = lookupTables->IdOf("current_types", "Deleted");
    if ((*i_row)["edited"] != "New")
      (*i_row)["edited"] = "Edited"; // mark as edited, unless it's new
    UpdateDOSFilter();
  }
}
//---------------------------------------------------------------------------
// jhaley 20101011: Discard all changes to denial items.
void __fastcall TformRecordDenialLetter::btnDOSUndoClick(TObject *Sender)
{
  current_dos_data = original_dos_data;
  UpdateDOSFilter();

  // wipe out current editing state as it's no longer valid
  // changes for Paul: return to add mode from here
  SetDOSAddMode(false);
}
//---------------------------------------------------------------------------
// jhaley 20101011: Validate DOS items already in the filter (when saving main record)
bool TformRecordDenialLetter::AllDOSAreValid()
{
  int valid_dos_count = 0;
  int review_dos_count = 0;
  int zero_approved_count = 0;

  // loop on all the current DOS items
  for (vector<map<string, string> >::iterator i_dos = current_dos_data.begin(); i_dos != current_dos_data.end(); i_dos++)
  {
    // Deleted items are not validated
    if ((*i_dos)["current_id"] == lookupTables->IdOf("current_types", "Deleted"))
      continue;
    // None should be marked as Incomplete
    else if ((*i_dos)["current_id"] == lookupTables->IdOf("current_types", "Incomplete"))
    {
      ++review_dos_count;
      continue; // don't do other checks below for ones that haven't been reviewed
    }
    else if ((*i_dos)["current_id"] == lookupTables->IdOf("current_types", "Current") ||
             (*i_dos)["current_id"] == lookupTables->IdOf("current_types", "Primary"))
      ++valid_dos_count;

    // these checks must be done again now because A) items adapted from an older letter need to be edited if the denial
    // status type has changed from Denied to Approved, and B) they might have gotten clever and changed the status
    // *after* adding DOS items.
    if (cboStatus->Text == "Denied")
    {
      // jhaley 20100716: The letter this DOS belongs to has "Denied" status, so the payable amount should be 0.
      if ((*i_dos)["payable_amount"] == "" || StringToFinancial((*i_dos)["payable_amount"]) != StringToFinancial("0.0"))
      {
        PSay("A dates-of-service item for a letter with Denied status should have a payable amount of $0.00.", m_invalid_data, MB_OK);
        return false;
      }
    }
    else if (cboStatus->Text == "Resolved")
    {
      // jhaley 20100716: The letter this DOS belongs to has "Resolved" status, so the payable amount should be
      // less than or equal to the patient responsibility.
      if ((*i_dos)["payable_amount"] == "" || StringToFinancial((*i_dos)["payable_amount"]) > StringToFinancial((*i_dos)["patient_amount"]))
      {
        PSay("A dates-of-service item for a letter with Resolved status should have a payable amount equal to or less than the patient's owed amount.",
             m_invalid_data, MB_OK);
        return false;
      }
      // jhaley 20101014: Zero-dollar payable amounts on an approval are allowed, but are odd, so they generate a warning below.
      if (StringToFinancial((*i_dos)["payable_amount"]) == StringToFinancial("0.0"))
        ++zero_approved_count;
    }
  }

  // If there are imported DOS items to review, that takes precedence over anything else.
  if (review_dos_count != 0)
  {
    PSay(IntToString(review_dos_count) + " dates-of-service item(s) imported from the previous letter in this denial batch need review.  "
         "You must explicitly specify a valid payable amount for each imported item.", m_invalid_data, MB_OK);
    return false;
  }

  // jhaley 20101012: verify at least one current denial item exists (do not count ones marked as deleted)
  if (valid_dos_count <= 0)
  {
    PSay("Please create one or more current dates-of-service items for this denial letter.", m_invalid_data, MB_OK);
    btnDOSAdd->SetFocus();
    return false;
  }

  // if approval has items with zero dollar payable amounts, warn:
  if (zero_approved_count != 0)
  {
    if(IDNO == PSay(IntToString(zero_approved_count) + " dates-of-service item(s) have a $0.00 payable amount. This is unusual for a 'Resolved' letter.\n\n"
                    "Are you sure you want to do this?\n\nYou can mark the item(s) as deleted if they should not appear on this letter at all.", m_confirmation, MB_YESNO))
      return false;
  }

  return true;
}
//---------------------------------------------------------------------------
// jhaley 20101011: Validate DOS items when saving to list from editing controls
bool TformRecordDenialLetter::IsDOSValidOnScreen()
{
  if (!IsAmount(editBilledAmount->Text.c_str()) || editBilledAmount->Text == "")
  {
     PSay("Please enter a valid billed amount.", m_invalid_data, MB_OK);
     editBilledAmount->SetFocus();
     return false;
  }
  else if (!IsAmount(editPatientOwes->Text.c_str()) || editPatientOwes->Text == "")
  {
    PSay("Please enter a valid patient owed amount.", m_invalid_data, MB_OK);
    editPatientOwes->SetFocus();
    return false;
  }
  else if (!IsAmount(editPayableAmount->Text.c_str()) || editPayableAmount->Text == "")
  {
    PSay("Please enter a valid payable amount.", m_invalid_data, MB_OK);
    editPayableAmount->SetFocus();
    return false;
  }
  else if (StringToFinancial(editPatientOwes->Text.c_str()) > StringToFinancial(editBilledAmount->Text.c_str()))
  {
    PSay("The patient owed amount should not exceed the billed amount.", m_invalid_data, MB_OK);
    editPatientOwes->SetFocus();
    return false;
  }
  else if (cboStatus->Text == "Denied")
  {
    // jhaley 20100716: The letter this DOS belongs to has "Denied" status, so the payable amount should be 0.
    // Compare both as financials.
    // Note that it will start out as 0 when coming here from a denial letter with denied status, but that
    // doesn't keep anybody from trying to change it on this form.
    if (StringToFinancial(editPayableAmount->Text.c_str()) != StringToFinancial("0.0"))
    {
      PSay("A DOS item for a letter with Denied status should have a payable amount of $0.00.", m_invalid_data, MB_OK);
      editPayableAmount->SetFocus();
      return false;
    }
  }
  else if (cboStatus->Text == "Resolved")
  {
    // jhaley 20100716: The letter this DOS belongs to has "Resolved" status, so the payable amount should be
    // less than or equal to the patient responsibility.
    if (StringToFinancial(editPayableAmount->Text.c_str()) > StringToFinancial(editPatientOwes->Text.c_str()))
    {
      PSay("A DOS item for a letter with Resolved status should have a payable amount equal to or less than the patient's owed amount.",
           m_invalid_data, MB_OK);
      editPayableAmount->SetFocus();
      return false;
    }
  }

  // verify date ordering
  Pdate dateFrom, dateTo;
  dateFrom = Pdate(dateDOSBegin->Date);
  dateTo   = Pdate(dateDOSEnd->Date);
  if (dateFrom > dateTo)
  {
     PSay("The beginning date should fall at or before the ending date.", m_invalid_data, MB_OK);
     dateDOSBegin->SetFocus();
     return false;
  }

  return true;
}
//---------------------------------------------------------------------------
// jhaley 20101011: Save the DOS item being edited or created into current_dos_data.
void __fastcall TformRecordDenialLetter::btnDOSSaveClick(TObject *Sender)
{
  if (is_dos_edit_active) // you shouldn't be able to even get here if this isn't true
  {
    // validate the DOS editing controls
    if (!IsDOSValidOnScreen())
      return;

    if (current_editing_dos_id == "new") // Adding a new entry
    {
      map<string, string> new_item;

      new_item["begin_date_of_service"] = Pdate(dateDOSBegin->Date).OutDate(mdy | yyyy);
      new_item["end_date_of_service"]   = Pdate(dateDOSEnd->Date).OutDate(mdy | yyyy);
      new_item["billed_amount"]  = editBilledAmount->Text.c_str();
      new_item["patient_amount"] = editPatientOwes->Text.c_str();
      new_item["payable_amount"] = editPayableAmount->Text.c_str();
      new_item["current_id"] = lookupTables->IdOf("current_types", "Current");
      new_item["id"] = "new" + IntToString(current_dos_data.size());
      new_item["edited"] = "New";

      current_dos_data.push_back(new_item);
    }
    else // Editing an existing entry
    {
      vector<map<string, string> >::iterator i_row;
      for (i_row = current_dos_data.begin(); i_row != current_dos_data.end(); i_row++)
      {
        if ((*i_row)["id"] == current_editing_dos_id)
          break;
      }
      if (i_row != current_dos_data.end())
      {
        (*i_row)["begin_date_of_service"] = Pdate(dateDOSBegin->Date).OutDate(mdy | yyyy);
        (*i_row)["end_date_of_service"]   = Pdate(dateDOSEnd->Date).OutDate(mdy | yyyy);
        (*i_row)["billed_amount"]  = editBilledAmount->Text.c_str();
        (*i_row)["patient_amount"] = editPatientOwes->Text.c_str();
        (*i_row)["payable_amount"] = editPayableAmount->Text.c_str();
        (*i_row)["current_id"] = lookupTables->IdOf("current_types", "Current");
        if ((*i_row)["edited"] != "New")
          (*i_row)["edited"] = "Edited"; // mark as edited unless it's a new item
      }
    }
    UpdateDOSFilter();

    // change for Paul: return to add mode from here
    SetDOSAddMode();
  }
}
//---------------------------------------------------------------------------
// jhaley 20101011: Save the denial item and add another new one.
void __fastcall TformRecordDenialLetter::btnDOSSaveAndAddClick(TObject *Sender)
{
  btnDOSSaveClick(Sender);

  // set up for a new one
  AddItem();
}
//---------------------------------------------------------------------------
// jhaley 20101018: Change for Paul - Auto-edit selected item when selection changes
void __fastcall TformRecordDenialLetter::OnDOSSelectItem(TObject *Sender, TListItem *Item, bool Selected)
{
  if (!Selected)
    SetDOSAddMode(false);
  else if (groupDOS->Enabled)
    SetDOSEditMode(false);
}
//---------------------------------------------------------------------------
