
//---------------------------------------------------------------------------

#include <vcl.h>
#pragma hdrstop

#include "codeRecordDiabScore.h"
#include "codeRecordTaskDiab.h"
//---------------------------------------------------------------------------
#pragma package(smart_init)
#pragma link "codeRecordGeneric"
#pragma resource "*.dfm"
TformRecordDiabScore *formRecordDiabScore;
//---------------------------------------------------------------------------
__fastcall TformRecordDiabScore::TformRecordDiabScore(TComponent* Owner, map<string,string> p_parameters)
        : TformRecordGeneric(Owner, p_parameters, string("diab_education"))
{
  task_filter.SetListViewHandle(listTasks);
  task_filter.SetSecurity("tasks.diab");
  task_filter.AddColumnSpec("Date", "tasks", as_date, true, 90);
  task_filter.AddColumnSpec("Employee", "tasks", as_text, true, 110);
  task_filter.AddColumnSpec("Time Spent", "tasks", as_double, true, 100);
  task_filter.AddColumnSpec("Activity", "tasks", as_text, true, 110);
  task_filter.AddColumnSpec("Notes", "tasks", as_text, true, 110, sentenceString);
  task_filter.AddColumnSpec("Current Id", "current_id", as_int, false, 80);
  task_filter.AddColumnSpec("Internal Id", "internal_id", as_int, false, 80);
  task_filter.FetchPreferences("recorddiabscore.task_filter");
  task_filter.SetIconStatus(true);
  task_filter.SetSQLStatement("");
  task_filter.RefreshDataSet();

  Initialize();
}
//---------------------------------------------------------------------------
bool TformRecordDiabScore::ScreenStart()
{
  lookupTables->RegisterComboBox("diab_topic_types", cboTopic);
  TformRecordGeneric::ScreenStart();
  return true;
}
//---------------------------------------------------------------------------
bool TformRecordDiabScore::ScreenStop()
{
  lookupTables->UnregisterComboBox("diab_topic_types", cboTopic);
  TformRecordGeneric::ScreenStop();
  return true;
}
//---------------------------------------------------------------------------
bool TformRecordDiabScore::IsValidOnScreen()
{
  if(! lookupTables->IsInLookUpTable("diab_topic_types", cboTopic->Text.c_str()))
  {
    PSay("Please enter a valid topic", m_invalid_data, MB_OK);
    cboTopic->SetFocus();
    return false;
  }
  else if (! Pdate (editPreTestDate->Text.c_str(), mdy | yy).IsValidDate() && ! (editPreTestDate->Text == ""))
  {
    PSay("Please enter a valid pre-test date", m_invalid_data, MB_OK);
    editPreTestDate->SetFocus();
    return false;
  }
  else if (! IsFloat(editPreScore->Text.c_str()))
  {
    PSay("Please enter a valid pre-score", m_invalid_data, MB_OK);
    editPreScore->SetFocus();
    return false;
  }
  else if (! Pdate (editPostTestDate->Text.c_str(), mdy | yy).IsValidDate() && ! (editPostTestDate->Text == ""))
  {
    PSay("Please enter a valid post-test date", m_invalid_data, MB_OK);
    editPostTestDate->SetFocus();
    return false;
  }
  else if (! IsFloat(editPostScore->Text.c_str()))
  {
    PSay("Please enter a valid post-score", m_invalid_data, MB_OK);
    editPostScore->SetFocus();
    return false;
  }
  else
    return true;
}
//---------------------------------------------------------------------------
bool TformRecordDiabScore::PutDataOnScreen()
{
  // put the values on the form from the map initial_state
  // values may have come from GetDataFromDatabase if it's an existing record,
  // or values may have been passed to the form in this map if it's a new record
  cboTopic->Text = lookupTables->ValueOf("diab_topic_types", initial_state["diab_topic_id"]).c_str();
  editPreTestDate->Text = initial_state["pre_test_date"].c_str();
  editPreScore->Text = initial_state["pre_score"].c_str();
  editPostTestDate->Text = initial_state["post_test_date"].c_str();
  editPostScore->Text = initial_state["post_score"].c_str();
  editNote->Text = initial_state["note"].c_str();

  // get the person name associated with the person id that was either selected
  // from the database if it's an existing record, or passed to the form if
  // it's a new record. display the name as a link to the person record.
  CreateLink(txtPerson, NULL, this, initial_state["person_id"], "people");

  TformRecordGeneric::PutDataOnScreen();
  return true;
}
//---------------------------------------------------------------------------
bool TformRecordDiabScore::GetDataFromScreen()
{
  // place the values currently on the screen into
  // the map that will be used to update or insert the record
  // the parent form will call the insert or update function, and get the id
  // number if necessary and take care of current_id
  ending_state["diab_topic_id"] = lookupTables->IdOf("diab_topic_types", cboTopic->Text.c_str());
  if(editPreTestDate->Text != "")
    ending_state["pre_test_date"] = Pdate(editPreTestDate->Text.c_str(), mdy | yy).OutDate(mdy | yyyy);
  else
    ending_state["pre_test_date"] = "";
  if(editPreScore->Text != "")
    ending_state["pre_score"] = editPreScore->Text.c_str();
  else
    ending_state["pre_score"] = "";
  if(editPostTestDate->Text != "")
    ending_state["post_test_date"] = Pdate(editPostTestDate->Text.c_str(), mdy | yy).OutDate(mdy | yyyy);
  else
    ending_state["post_test_date"] = "";
  if(editPostScore->Text != "")
    ending_state["post_score"] = editPostScore->Text.c_str();
  else
    ending_state["post_score"] = "";
  ending_state["note"] = editNote->Text.c_str();
  ending_state["person_id"] = initial_state["person_id"];
  ending_state["id"] = initial_state["id"];

  TformRecordGeneric::GetDataFromScreen();

  return true;
}
//---------------------------------------------------------------------------
bool TformRecordDiabScore::ScreenLock()
{
  // disable panel containing edit boxes
  panelGroup->Enabled = false;
  editNote->ReadOnly = true;
  editNote->TabStop = false;
  TformRecordGeneric::ScreenLock();
  return true;
}
//---------------------------------------------------------------------------
bool TformRecordDiabScore::ScreenUnlock()
{
  // enable panel containing edit boxes
  panelGroup->Enabled = true;
  editNote->ReadOnly = false;
  editNote->TabStop = true;
  TformRecordGeneric::ScreenUnlock();
  return true;
}
//---------------------------------------------------------------------------
bool TformRecordDiabScore::IsNewAgain()
{
  task_filter.SetSQLStatement(string(""));
  task_filter.RefreshDataSet();
  task_filter.SetAddEdit(this, NULL, NULL);
   TformRecordGeneric::IsNewAgain();
   return true;
}
//---------------------------------------------------------------------------
bool TformRecordDiabScore::NotNewAnymore()
{
  string sql;
  if (optionsPermissions["table.tasks.diab"]["view"] == "true")
  {
    sql = "select tasks.date_of_task, users.item, tasks.time_spent, diab_activity_types.item, "
          "tasks.task_comments, tasks.current_id, tasks.id "
          "from tasks, users, diab_activity_types "
          "where tasks.diab_education_id = " + initial_state["id"] + " and "
          "tasks.user_id = users.id and "
          "tasks.diab_activity_id = diab_activity_types.id";

    task_filter.SetSQLStatement(sql);
    task_filter.RefreshDataSet();
    task_filter.SetAddEdit(this, AddRecord(&(TformRecordDiabScore::AddTask)),
                            EditRecord(&(TformRecordDiabScore::EditTask)));
    task_request = atlas->RequestNotificationWhen("tasks.diab_education_id", initial_state["id"],
                    rel_e, as_int, this, RefreshHandler(&(TformRecordDiabScore::RefreshListView)));
  }
  TformRecordGeneric::NotNewAnymore();
  return true;
}
//---------------------------------------------------------------------------
void TformRecordDiabScore::RefreshListView(int request_num, string matched_value)
{
  if (request_num == task_request)
    task_filter.RefreshDataSet();
}
//---------------------------------------------------------------------------
void TformRecordDiabScore::AddTask()
{
  map<string, string> temp_map;
  temp_map["id"] = "new";
  temp_map["person_id"] = initial_state["person_id"];
  temp_map["diab_education_id"] = initial_state["id"];
  temp_map["group_id"] = lookupTables->IdOf("groups", "Diabetes");
  FORM_TRY
    TformRecordTaskDiab * diabTask = new TformRecordTaskDiab(NULL, temp_map);
    diabTask->Show();
  FORM_CATCH
}
//---------------------------------------------------------------------------
void TformRecordDiabScore::EditTask()
{
  map<string, string> temp_map;
  temp_map["id"] = task_filter.GetSelectedRecordID();
  FORM_TRY
    TformRecordTaskDiab * diabTask = new TformRecordTaskDiab(NULL, temp_map);
    diabTask->Show();
  FORM_CATCH
}
//---------------------------------------------------------------------------
void TformRecordDiabScore::Do(map<string, string> & p_parameters)
{
 FORM_TRY
  TformRecordDiabScore * z = new TformRecordDiabScore(NULL, p_parameters);
  z->Show();
 FORM_CATCH
}
//---------------------------------------------------------------------------

