//---------------------------------------------------------------------------
#include <vcl.h>
#pragma hdrstop

#include "codeRecordEnrollment.h"
//---------------------------------------------------------------------------
#pragma package(smart_init)
#pragma link "codeRecordGeneric"
#pragma link "codeRecordGeneric"
#pragma link "codeRubberContainer"
#pragma resource "*.dfm"
#include "codeSearchTaxEntity.h"
#include "derivedheaders.h"

TformRecordEnrollment *formRecordEnrollment;
//---------------------------------------------------------------------------
__fastcall TformRecordEnrollment::TformRecordEnrollment(TComponent* Owner, map<string,string> p_parameters)
        : TformRecordGeneric(Owner, p_parameters, string("enrollment"))
{
  // PSay(MapToDelimString(p_parameters, "{", "}", "", "'", "'", "'", "'", ","), m_debug, MB_OK);

  classFilterObject *filter = filter_atlas_handler["changes"].filter;
  filter->SetListViewHandle(listChanges);
  filter->SetSecurity("enrollment");
  filter->AddColumnSpec("Changed On", "enrollment", as_stamp, true, 100);
  filter->AddColumnSpec("Changed By", "users", as_text, true, 110);
  filter->AddColumnSpec("Type of Change", "enrollment", as_text, true, 110, rawString);
  filter->AddColumnSpec("Value Before", "enrollment", as_text, true, 130, rawString);
  filter->AddColumnSpec("Value After", "enrollment", as_text, true, 130, rawString);
  filter->AddColumnSpec("Current Id", "current_id", as_int, false, 80);
  filter->AddColumnSpec("Internal Id", "internal_id", as_int, false, 80);
  filter->FetchPreferences("recordenrollment.changes_filter");
  filter->SetAddEdit(this, NULL, NULL);
  filter_atlas_handler["changes"].request = atlas->RequestNotificationWhen("enrollment_changes.enrollment_id", initial_state["id"],
                                            rel_e, as_int, this, RefreshHandler(&(TformRecordEnrollment::RefreshListView)));
  filter->SetIconStatus(false);
  filter->SetSQLStatement("");
  filter->RefreshDataSet();

  Initialize();
}
//---------------------------------------------------------------------------
bool TformRecordEnrollment::ScreenStart()
{
  // register combo box
  if (is_new_record)
  {
    string sql = "select et.id, et.item, et.valid from "
                 "enrollment_types et "
                 "inner join permissions p1 on p1.s = et.permission_name and p1.k = 'INSERT' and p1.v = 'TRUE' and p1.user_id = " + current_userid + " "
                 "inner join permissions p2 on p2.s = et.permission_name and p2.k = 'VIEW' and p2.v = 'TRUE' and p2.user_id = " + current_userid + " ";
    lookupTables->RegisterComboBox("enrollment_types", cboType, sql);
    DebugSql(this, &sql, other_forms, 9);
  }
  else
    lookupTables->RegisterComboBox("enrollment_types", cboType);

  btnGenerate->Enabled = false;

  TformRecordGeneric::ScreenStart();
  return true;
}
//---------------------------------------------------------------------------
bool TformRecordEnrollment::ScreenStop()
{
  // unregister combo box
  lookupTables->UnregisterComboBox("enrollment_types", cboType);
  if (lookupTables->TableName(cboStatus) != "") // if this cbo box is registered
    lookupTables->UnregisterComboBox("enrollment_status_types", cboStatus);
  if (lookupTables->TableName(cboChange) != "") // if this cbo box is registered
    lookupTables->UnregisterComboBox("enrollment_change_types", cboChange);

  TformRecordGeneric::ScreenStop();
  return true;
}
//---------------------------------------------------------------------------
bool TformRecordEnrollment::IsValidOnScreen()
{
  string sql;

  // check to make sure that the values entered in the combo box
  // correspond to values in that look up table.
  // check to make sure the dates entered are valid
  // give the user a message box and prompt to re-enter if incorrect
  // make sure there is a link to a person, although there's not much the user
  // can do to fix that if there's not

  // jhaley: This was being queried 3 times in a row; no need for that.
  sql = "select permission_name from enrollment_types where id = '" + lookupTables->IdOf("enrollment_types", cboType) + "' ";
  string type_permission = LowercaseString(GetOneField(dbDatabase, sql));

  if (is_new_record) // nested cause ... there's no reason to run the SQL if new/editing is the wrong state (I hate relying on short-circuit evaluation)
  {
    if (optionsPermissions[type_permission]["insert"] != "true")
    {
      PSay("You are not permitted to enter '" + string(cboType->Text.c_str()) + "' enrollment records.", m_warning, MB_OK);
      return false;
    }
    if (optionsPermissions[type_permission]["view"] != "true")
    {
      PSay("You are not permitted to view '" + string(cboType->Text.c_str()) + "' enrollment records, yet are able to create enrollees in this program.  "
           "Please speak to a programmer about this.", m_program_error, MB_OK);
      return false;
    }
  }
  if (is_editing)
  {
    if (optionsPermissions[type_permission]["update"] != "true")
    {
      PSay("You are not permitted to update '" + string(cboType->Text.c_str()) + "' enrollment records.", m_warning, MB_OK);
      return false;
    }
  }

  if (!lookupTables->IsInLookUpTable("enrollment_types", cboType))
  {
    PSay("Please select the type of enrollment you wish to enter.", m_invalid_data, MB_OK);
    cboType->SetFocus();
    return false;
  }
  else if (!lookupTables->IsInLookUpTable("enrollment_status_types", cboStatus))
  {
    PSay("Please select the status of this person in this program.", m_invalid_data, MB_OK);
    cboStatus->SetFocus();
    return false;
  }
  else if (editAdmit->Text != "" && !Pdate(editAdmit->Text.c_str(), mdy | yy).IsValidDate())
  {
    PSay("Please enter a valid enrollment date", m_invalid_data, MB_OK);
    editAdmit->SetFocus();
    return false;
  }
  else if (editRemit->Text != "" && !Pdate(editRemit->Text.c_str(), mdy | yy).IsValidDate())
  {
    PSay("Please enter a valid un-enrollment date", m_invalid_data, MB_OK);
    editRemit->SetFocus();
    return false;
  }
  else if (editAdmit->Text != "" && editRemit->Text != "" && Pdate(editAdmit->Text.c_str(), mdy | yy) > Pdate(editRemit->Text.c_str(), mdy | yy))
  {
    PSay("Please enter an enrollment date that comes before the un-enrollment date.  "
         "Do not re-use this record if this person is re-enrolling.  Instead, cancel, and create a new enrollment record.", m_invalid_data, MB_OK);
    editAdmit->SetFocus();
    return false;
  }
  else if (IsBlankOrZero(initial_state["person_id"]))
  {
    PSay("There is no person associated with this record", m_program_error, MB_OK); // jhaley: program error; shouldn't happen
    return false;
  }
  else if (ConflictsWithExistingRecord())
  {
    PSay("This person is already enrolled in this program.  Please click the person link, go to their Enrollment page, and modify their existing enrollment as necessary.", m_invalid_data, MB_OK);
    return false;
  }
  else if (initial_state["remit_date"] != "" && (editRemit->Text == "" || !Pdate(editRemit->Text.c_str(), mdy | yy).IsValidDate()))
  {
    // jhaley 20110113: These checks must be last because they don't always return.
    if (optionsPermissions["admin.reuse_enrollment"]["enabled"] == "true")
    {
      // Special permission to reuse enrollment records - the only real reason for this is so that they can be fixed from within
      // Prometheus if a mistake was made; but not just anybody should be able to do it.
      string current_id = ending_state["current_id"];
      if (current_id == "") current_id = initial_state["current_id"];
      if (current_id != lookupTables->IdOf("current_types", "Current"))
      {
        PSay("You have permission to remove remit dates from enrollments, but please set the current type to 'Current' first.", m_reminder, MB_OK);
        return false;
      }
    }
    else
    {
      // jhaley: It doesn't make sense to do this if they're not supposed to re-use enrollments.
      PSay("This enrollment has already been expired, so you cannot clear the remit date.  "
           "If this person is re-enrolling, cancel, then create a new enrollment record.  "
           "If you expired this enrollment by mistake, please mark it as 'Incorrect' and contact a programmer for help.", m_invalid_data, MB_OK);
      editRemit->SetFocus();
      return false;
    }
  }

  return true;
}
//---------------------------------------------------------------------------
bool TformRecordEnrollment::IsNewAgain()
{
 filter_atlas_handler["changes"].Reset(this);

 generator_name = "";

 TformRecordGeneric::IsNewAgain();
 return true;
}
//---------------------------------------------------------------------------
bool TformRecordEnrollment::NotNewAnymore()
{
   string sql;
   sql = "select ec.change_stamp, users.item, ect.item, "
         "case when ec.enrollment_change_id = " + lookupTables->IdOf("enrollment_change_types", "Enrolled")      + " then old_admit_date "
         "     when ec.enrollment_change_id = " + lookupTables->IdOf("enrollment_change_types", "Un-Enrolled")   + " then old_remit_date "
         "     when ec.enrollment_change_id = " + lookupTables->IdOf("enrollment_change_types", "Status Change") + " then old_status.item "
         "     when ec.enrollment_change_id = " + lookupTables->IdOf("enrollment_change_types", "Enrollment Number Change") + " then old_enrollment_number "
         "     when ec.enrollment_change_id = " + lookupTables->IdOf("enrollment_change_types", "PCP Change")    + " then old_provider.name "
         "     else null end, "
         "case when ec.enrollment_change_id = " + lookupTables->IdOf("enrollment_change_types", "Enrolled")      + " then new_admit_date "
         "     when ec.enrollment_change_id = " + lookupTables->IdOf("enrollment_change_types", "Un-Enrolled")   + " then new_remit_date "
         "     when ec.enrollment_change_id = " + lookupTables->IdOf("enrollment_change_types", "Status Change") + " then new_status.item "
         "     when ec.enrollment_change_id = " + lookupTables->IdOf("enrollment_change_types", "Enrollment Number Change") + " then new_enrollment_number "
         "     when ec.enrollment_change_id = " + lookupTables->IdOf("enrollment_change_types", "PCP Change")    + " then new_provider.name "
         "     else null end, "
         "4, ec.id "
         "from enrollment_changes ec "
         "inner join enrollment_change_types ect on ec.enrollment_change_id = ect.id "
         "left  join enrollment_status_types old_status on ec.old_status_id = old_status.id "
         "left  join enrollment_status_types new_status on ec.new_status_id = new_status.id "
         "left  join tax_entities old_provider on ec.old_provider_id = old_provider.id "
         "left  join tax_entities new_provider on ec.new_provider_id = new_provider.id "
         "inner join users on ec.change_by = users.id "
         "where ec.enrollment_id = '" + initial_state["id"] + "' ";

  DebugSql(this, &sql, other_forms, 9);

  classFilterObject *filter = filter_atlas_handler["changes"].filter;
  filter->SetSQLStatement(sql);
  filter->RefreshDataSet();
  // elsewhere...
  //filter_atlas_handler["changes"].request = atlas->RequestNotificationWhen("enrollment_changes.enrollment_id", initial_state["id"],
  //                rel_e, as_int, this, RefreshHandler(&(TformRecordEnrollment::RefreshListView)));

  TformRecordGeneric::NotNewAnymore();
  return true;
}
//---------------------------------------------------------------------------
bool TformRecordEnrollment::PutDataOnScreen()
{
  // put the values on the form from the map initial_state
  // values may have come from GetDataFromDatabase if it's an existing record,
  // or values may have been passed to the form in this map if it's a new record

  editEnrollmentNumber->Text = LowercaseString(initial_state["enrollment_number"]).c_str();
  editAdmit->Text = initial_state["admit_date"].c_str();
  editRemit->Text = initial_state["remit_date"].c_str();
  cboTypeChange(NULL); // to enable Generate button if old record but blank Enroll #
  if (!is_new_record)
  {
    cboType->Text = lookupTables->ValueOf("enrollment_types", initial_state["enrollment_id"]).c_str();
    UpdateEnrollmentStatus();
    cboStatus->Text = PrettyString(lookupTables->ValueOf("enrollment_status_types", initial_state["enrollment_status_id"])).c_str();
  }
  if (is_new_record)
  {
    cboType->Text = "";
    UpdateEnrollmentStatus();
    cboType->Visible = true;
    lblTypeValue->Visible = false;
    if (editAdmit->Text == "") editAdmit->Text = CurrentDate().c_str();
  }
  else
  {
    cboType->Visible = false;
    lblTypeValue->Visible = true;
    lblTypeValue->Caption = lookupTables->ValueOf("enrollment_types", initial_state["enrollment_id"]).c_str();
  }
  cboChange->Text = "";

  // GetOneField ... generator name associated with enrollment type
  // if there is one, and the edit box is empty, enable the Generate button
  // TODO!

  // get the person name associated with the person id that was either selected
  // from the database if it's an existing record, or passed to the form if
  // it's a new record. display the name as a link to the person record.

  last_provider_id = initial_state["provider_id"];
  CreateLink(txtPCP, btnPCPSearch, this, last_provider_id, "tax_entities");
  CreateLink(txtLastPCP, NULL, this, initial_state["last_provider_id"], "tax_entities");
  CreateLink(lblName, NULL, this, initial_state["person_id"], "people");

  vector<string> available_values = lookupTables->GetValueList("enrollment_types", cboType);
  if (is_new_record && available_values.size() == 1)
   initial_state["enrollment_id"] = lookupTables->IdOf("enrollment_types", available_values[0], cboType);

  TformRecordGeneric::PutDataOnScreen();
  return true; // what can really go wrong?
}
//---------------------------------------------------------------------------
bool TformRecordEnrollment::GetDataFromScreen()
{
  // place the values currently on the screen into
  // the map that will be used to update or insert the record
  // the parent form will call the insert or update function, and get the id
  // number if necessary and take care of current_id
  ending_state["enrollment_number"] = editEnrollmentNumber->Text.c_str();
  ending_state["enrollment_id"] = lookupTables->IdOf("enrollment_types", cboType);
  ending_state["enrollment_status_id"] = lookupTables->IdOf("enrollment_status_types", cboStatus);
  if (editAdmit->Text != "")
    ending_state["admit_date"] =  Pdate (editAdmit->Text.c_str(), mdy | yy).OutDate(mdy | yyyy);
  else
    ending_state["admit_date"] = "";

  if (editRemit->Text != "")
    ending_state["remit_date"] = Pdate (editRemit->Text.c_str(), mdy | yy).OutDate(mdy | yyyy);
  else
    ending_state["remit_date"] = "";

  ending_state["person_id"] = initial_state["person_id"];
  ending_state["id"] = initial_state["id"];

  string provider_id = GetValidLinkId(txtPCP);
  ending_state["provider_id"] = provider_id;
  if (provider_id != last_provider_id) // if it's changed, record the previous value; unless the previous value was blank, then record 0
    ending_state["last_provider_id"] = (last_provider_id != "" ? last_provider_id : string("0"));

  // we don't want null values
  if (ending_state["provider_id"] == "")
   ending_state["provider_id"] = "0";

  TformRecordGeneric::GetDataFromScreen();
  return true; // again, not much can go wrong here
}
//---------------------------------------------------------------------------
bool TformRecordEnrollment::ScreenLock()
{
  // disable panel containing edit boxes
  panelGroup->Enabled = false;
  groupChange->Enabled = true;

  TformRecordGeneric::ScreenLock();
  return true;
}
//---------------------------------------------------------------------------
bool TformRecordEnrollment::ScreenUnlock()
{
  // enable panel containing edit boxes
  panelGroup->Enabled = true;
  groupChange->Enabled = false;

  TformRecordGeneric::ScreenUnlock();
  return true;
}
//---------------------------------------------------------------------------
bool TformRecordEnrollment::BeforeSave()
{
/*
  PSay(MapToDelimString(initial_state, "{", "}", "", "'", "'", "'", "'", ",") + "\n\n" +
       MapToDelimString(hack_state, "{", "}", "", "'", "'", "'", "'", ",") + "\n\n" +
       MapToDelimString(ending_state, "{", "}", "", "'", "'", "'", "'", ","), m_debug, MB_OK);
*/

 bool result = true;
/*
 list <map<string, string> > differences;
 map<string, string> this_difference, common_fields;

 // setup the basic fields for default_difference;
 common_fields ["enrollment_id"] = ending_state["id"];
 common_fields ["change_by"]     = current_userid;
 common_fields ["change_stamp"]  = RightNow();

 // changed to also use hack_state rather than initial_state, for the same reasons as in SendAtlas()
 map<string, string>::iterator i_after;
 map<string, string>::iterator i_after_end = ending_state.end();
 for (map<string, string>::iterator i_before = hack_state.begin();
      i_before != hack_state.end(); i_before++)
 {
   this_difference = common_fields;

   i_after = ending_state.find(i_before->first);
   if (i_after != i_after_end)
   {
    if (UppercaseString(i_after->second) != UppercaseString(i_before->second))
    {
     bool push_it = true;
     if      (i_before->first == "admit_date")
     {
       this_difference["old_admit_date"] = i_before->second;
       this_difference["new_admit_date"] = i_after->second;
       this_difference["enrollment_change_id"] = lookupTables->IdOf("enrollment_change_types", "Enrolled");
     }
     else if (i_before->first == "remit_date")
     {
       this_difference["old_remit_date"] = i_before->second;
       this_difference["new_remit_date"] = i_after->second;
       this_difference["enrollment_change_id"] = lookupTables->IdOf("enrollment_change_types", "Un-Enrolled");
     }
     else if (i_before->first == "enrollment_status_id")
     {
       this_difference["old_status_id"] = i_before->second;
       this_difference["new_status_id"] = i_after->second;
       this_difference["enrollment_change_id"] = lookupTables->IdOf("enrollment_change_types", "Status Change");
     }
     else if (i_before->first == "enrollment_number")
     {
       this_difference["old_enrollment_number"] = i_before->second;
       this_difference["new_enrollment_number"] = i_after->second;
       this_difference["enrollment_change_id"] = lookupTables->IdOf("enrollment_change_types", "Enrollment Number Change");
     }
     else if (i_before->first == "provider_id")
     {
       this_difference["old_provider_id"] = i_before->second;
       this_difference["new_provider_id"] = i_after->second;
       this_difference["enrollment_change_id"] = lookupTables->IdOf("enrollment_change_types", "PCP Change");
     }
     else push_it = false;

     if (push_it)
       differences.push_back (this_difference);
    }
   }
 }

 if (differences.size() > 0)
   atlas->Notify("enrollment_changes.enrollment_id", initial_state["id"]);

 for (list<map<string, string> >::iterator i_difference = differences.begin();
      i_difference != differences.end(); i_difference++)
 {
  if (!ExecuteInsertStatement(lock_transaction, "enrollment_changes", *i_difference))
  {
   PSay("Could not save log data to enrollment_changes!", m_database_error, MB_OK);
   result = false;
  }
  else
  {
    // TODO : send notification on each FK that was affected...
    // atlas->Notify ...
  }
 }
 */
 atlas->Notify("enrollment_changes.enrollment_id", initial_state["id"]);
 TformRecordGeneric::BeforeSave();
 return result;
}
//---------------------------------------------------------------------------
void TformRecordEnrollment::RefreshListView(int request_num, string matched_value)
{
  // for the basic "if (request==blah_request) blah_filter->RefreshDataSet()" tree,
  // this is sufficient, as we're using the filter_atlas_handler thing from specialgeneric
  HandleAtlasRequest(request_num);
}
//---------------------------------------------------------------------------
void TformRecordEnrollment::Do(map<string, string> & p_parameters)
{
 FORM_TRY
  TformRecordEnrollment * z = new TformRecordEnrollment(NULL, p_parameters);
  z->Show();
 FORM_CATCH
}
//---------------------------------------------------------------------------
void __fastcall TformRecordEnrollment::cboTypeChange(TObject *Sender)
{
  UpdateEnrollmentStatus();
  if (cboType->Text == "" || !lookupTables->IsInLookUpTable("enrollment_types", cboType))
  {
    btnGenerate->Enabled = false;
    return;
  }
  // SQL to fetch enrollment_types.generator_name
  generator_name = GetOneField(dbDatabase, "select generator_name from enrollment_types where id = '" + lookupTables->IdOf("enrollment_types", cboType) + "'");
  if (generator_name != "")
  {
    bool has_permissions = false;
    if (is_new_record || // nested cause ... there's no reason to run the SQL if new/editing is the wrong state (I hate relying on short-circuit evaluation)
        editEnrollmentNumber->Text == "") // so they can generate one later if they accidentally saved it without one
      if (optionsPermissions[LowercaseString(GetOneField(dbDatabase, "select permission_name from enrollment_types where id = " + lookupTables->IdOf("enrollment_types", cboType)))]["insert"] == "true")
        has_permissions = true;
/*  no need to check "update" ... they can't generate a new enrollment number after they've already set one.
    if (is_editing)
      if (optionsPermissions[LowercaseString(GetOneField(dbDatabase, "select permission_name from enrollment_types where id = " + lookupTables->IdOf("enrollment_types", cboType)))]["update"] != "true")
        has_permissions = true;
*/
    btnGenerate->Enabled = has_permissions && !ConflictsWithExistingRecord();
  }
  else
    btnGenerate->Enabled = false;
}
//---------------------------------------------------------------------------
// Fetches the appropriate status types for cboType into cboStatus
void TformRecordEnrollment::UpdateEnrollmentStatus()
{
  if (lookupTables->IsInLookUpTable("enrollment_types", cboType))
  {
    string sql;

    if (lookupTables->TableName(cboStatus) != "") // if this cbo box is registered
      lookupTables->UnregisterComboBox("enrollment_status_types", cboStatus);

    sql = "select ES.id, ES.item, ES.valid "
          "from enrollment_status_types ES "
          "where ES.enrollment_id = '" + lookupTables->IdOf("enrollment_types", cboType) + "' or "
          "(ES.id = 0 and (select ET.allow_blank from enrollment_types ET where ET.id = '" + lookupTables->IdOf("enrollment_types", cboType) + "') = 1) ";
    lookupTables->RegisterComboBox("enrollment_status_types", cboStatus, sql);
    cboStatus->Text = "";

    if (lookupTables->TableName(cboChange) != "") // if this cbo box is registered
      lookupTables->UnregisterComboBox("enrollment_change_types", cboChange);
    sql = "select ect.id, ect.item, ect.valid "
          "from enrollment_change_types ect "
          "where ect.enrollment_id = '" + lookupTables->IdOf("enrollment_types", cboType) + "'";
    lookupTables->RegisterComboBox("enrollment_change_types", cboChange, sql);
    cboChange->Text = "";

    RecurseEnabled(groupChange, cboChange->Items->Count > 0);
  }
}
//---------------------------------------------------------------------------
void __fastcall TformRecordEnrollment::btnPCPSearchClick(TObject *Sender)
{
  vector<string> return_vector;
  FORM_TRY
    TformSearchTaxEntity * pcp_search = new TformSearchTaxEntity(NULL,&return_vector);
    pcp_search->ShowTaskModal(this);
  FORM_CATCH
  if (! return_vector.empty())
    CreateLink(txtPCP, btnPCPSearch, this, return_vector[0], "tax_entities");
}
//---------------------------------------------------------------------------
void __fastcall TformRecordEnrollment::btnChangeClick(TObject *Sender)
{
 // jhaley: NB - only enrollment change types that *aren't* auto-generated can be set here.
 if (cboChange->Text == "" || !lookupTables->IsInLookUpTable("enrollment_change_types", cboChange))
 {
   PSay("Please select the type of change you wish to manually apply to this person's enrollment.", m_invalid_data, MB_OK);
   cboChange->SetFocus();
   return;
 }
 btnChange->Enabled = false;
 map<string, string> this_difference, temp_map;

/*
 bool close_tx_when_done = false;
 if (!lock_transaction->Active)
 {
   lock_transaction->StartTransaction();
   close_tx_when_done = true;
 }
*/ // fuck all, I'll just use my own

 // setup the basic fields for default_difference;
 temp_map ["enrollment_id"] = initial_state["id"];
 temp_map ["change_by"]     = current_userid;
 temp_map ["change_stamp"]  = RightNow();
 temp_map ["enrollment_change_id"] = lookupTables->IdOf("enrollment_change_types", cboChange);
 PSay(MapToDelimString(temp_map, "{", "}", "", "'", "'", "'", "'", ","), m_debug, MB_OK);

 if (!ExecuteInsertStatement(dbDatabase, "enrollment_changes", temp_map))
  PSay("Could not save log data to enrollment_changes!", m_database_error, MB_OK);
 else
 {
  // string id = GetOneField(lock_transaction, "select max(id) from enrollment_changes"); // no.
  atlas->Notify("enrollment_changes.enrollment_id", initial_state["id"]);
 }

// if (close_tx_when_done && lock_transaction->Active)
//   lock_transaction->Commit();
 cboChange->Text = "";
 timerReset->Enabled = true;
 return;
}
//---------------------------------------------------------------------------
void __fastcall TformRecordEnrollment::btnGenerateClick(TObject *Sender)
{
  if (cboType->Text == "" || !lookupTables->IsInLookUpTable("enrollment_types", cboType))
    return; // You shouldn't even be here.  Seriously.  How are you even here?

  // get next id, put it in the edit box, disable the button
  // and you know what else?  Go ahead and lock the Type combo too ... change
  // it to a label.
  if (generator_name == "") return;
  string generator_id = "";
  try
  {
    generator_id = GetOneField(dbDatabase, "select gen_id(" + generator_name + ", 1) from uno");
  }
  catch (...) { }
  if (generator_id != "")
  {
    editEnrollmentNumber->Text = generator_id.c_str();
    cboType->Visible = false;
    lblTypeValue->Visible = true;
    lblTypeValue->Caption = cboType->Text;
    btnGenerate->Enabled = false;
  }
}
//---------------------------------------------------------------------------
void __fastcall TformRecordEnrollment::timerResetTimer(TObject *Sender)
{
  timerReset->Enabled = false;
  btnChange->Enabled = true;
}
//---------------------------------------------------------------------------
bool TformRecordEnrollment::ConflictsWithExistingRecord()
{
  //bool result = false;
  /* are the records for this same person, same enrollment group / program, other than
     the one in front of them? */

  // if it's not valid anyway, don't get in their way

/*  PSay(MapToDelimString(initial_state, "{", "}", "", "'", "'", "'", "'", ",") + "\n\n" +
       MapToDelimString(hack_state, "{", "}", "", "'", "'", "'", "'", ",") + "\n\n" +
       MapToDelimString(ending_state, "{", "}", "", "'", "'", "'", "'", ","), m_debug, MB_OK);
*/
  if (is_in(ending_state["current_id"], "0", "1", "2", NULL))
    return false;

  TIBTransaction *tx = NULL;
  if (lock_transaction && lock_transaction->Active)
    tx = lock_transaction;
  else
  {
    tx = new TIBTransaction(NULL);
    StdTransaction(tx, dbDatabase);
  }
  string conflict_sql = "select id from enrollment "
                        "where enrollment_id = '" + lookupTables->IdOf("enrollment_types", cboType->Text.c_str()) + "' "
                        "and person_id = '" + initial_state["person_id"] + "' and current_id+1 >= 4 ";
  if (!is_new_record)
    conflict_sql += "and id != '" + initial_state["id"] + "' ";
  //QueryData(this, tx, conflict_sql);
  DebugSql(this, &conflict_sql, other_forms, 9);
  vector<string> conflict_vec;
  SqlToVec(dbDatabase, conflict_sql, conflict_vec);
  return !conflict_vec.empty();
}
//---------------------------------------------------------------------------
// jhaley 20101103: Allow cleaning up from inside Prometheus
bool TformRecordEnrollment::DeleteRecord()
{
  if (!ExecuteStatement(delete_transaction, "delete from enrollment_changes where enrollment_id = '" + initial_state["id"] + "' "))
  {
    PSay("Failed to delete enrollment_changes; cancelled.", m_database_error, MB_OK);
    return false;
  }
  return TformRecordGeneric::DeleteRecord();
}
//---------------------------------------------------------------------------
