//---------------------------------------------------------------------------

#include <vcl.h>
#pragma hdrstop

#include "codeRecordEntityContract.h"
#include "codeRecordTaxEntity.h"
//---------------------------------------------------------------------------
#pragma package(smart_init)
#pragma link "codeRecordGeneric"
#pragma resource "*.dfm"
TformRecordEntityContract *formRecordEntityContract;
//---------------------------------------------------------------------------
__fastcall TformRecordEntityContract::TformRecordEntityContract(TComponent* Owner, map<string,string> p_parameters)
        : TformRecordGeneric(Owner,  p_parameters, string("entity_contracts"))
{
 fields_to_leave_raw.insert("terms");
 Initialize();
}
//---------------------------------------------------------------------------
bool TformRecordEntityContract::ScreenStart()
{
 lookupTables->RegisterComboBox("entity_contract_types", comboContractType);
 TformRecordGeneric::ScreenStart();
 return true;
}
//---------------------------------------------------------------------------
bool TformRecordEntityContract::ScreenStop()
{
 lookupTables->UnregisterComboBox("entity_contract_types", comboContractType);
 TformRecordGeneric::ScreenStop();
 return true;
}
//---------------------------------------------------------------------------
bool TformRecordEntityContract::IsValidOnScreen()
{
 if (comboContractType->Text == "" || !lookupTables->IsInLookUpTable ("entity_contract_types", comboContractType))
 {
  PSay("Please enter a valid type of contract.", m_invalid_data, MB_OK);
  comboContractType->SetFocus();
  return false;
 } else
 if (editStart->Text != "" && !Pdate(editStart->Text.c_str(), global_mode).IsValidDate())
 {
  PSay("Please enter a valid start date.", m_missing_data, MB_OK);
  editStart->SetFocus();
  return false;
 } else
 if (editStop->Text != "" && !Pdate(editStop->Text.c_str(), global_mode).IsValidDate())
 {
  PSay("Please enter a valid stop date.", m_missing_data, MB_OK);
  editStop->SetFocus();
  return false;
 } else
 if (editStart->Text != "" &&
     editStop->Text != "" &&
     Pdate(editStart->Text.c_str(), global_mode).IsValidDate() &&
     Pdate(editStop->Text.c_str(), global_mode).IsValidDate() &&
     Pdate(editStart->Text.c_str(), global_mode) > Pdate(editStop->Text.c_str(), global_mode))
 {
  PSay("The start and stop dates are not in order.", m_missing_data, MB_OK);
  editStart->SetFocus();
  return false;
 }
 if (IsBlankOrZero(initial_state["tax_entity_id"]))
 {
  PSay("There is no tax entity associated with this record.", m_missing_data, MB_OK);
  return false;
 }


 // check for overlap, warn
 {
  string sql = "select count(*) from entity_contracts where tax_entity_id = " + initial_state["tax_entity_id"] + " and current_id > 2";
  // ignore self
  if (IsInt(initial_state["id"]) && !IsBlankOrZero(initial_state["id"]))
   sql += " and entity_contracts.id != " + initial_state["id"];
  // ignore those ending before we start
  if (editStart->Text != "" && Pdate(editStart->Text.c_str(), global_mode).IsValidDate())
   sql += " and (entity_contracts.stop_date is null or entity_contracts.stop_date >= '" + Pdate(editStart->Text.c_str(), global_mode).OutDate(global_mode) + "')";
  // ignore those starting after we stop
  if (editStop->Text != "" && Pdate(editStop->Text.c_str(), global_mode).IsValidDate())
   sql += " and (entity_contracts.start_date is null or entity_contracts.start_date <= '" + Pdate(editStop->Text.c_str(), global_mode).OutDate(global_mode) + "')";
  string c = GetOneField(dbDatabase, sql);
  if (IsInt(c) && StringToInt(c) > 0)
   if (IDNO == PSay("There are " + c + " other contract(s) for this tax entity covering the same date range.\nThis may include partially-overlapping dates, and contracts with no dates.\nThis could lead to confusion when deciding which contract should be enforced.\nAre you sure you want to continue?", m_duplicate_data, MB_YESNO))
    return false;
 }

 return true;
}
//---------------------------------------------------------------------------
bool TformRecordEntityContract::PutDataOnScreen()
{
 memoTerms->Text = initial_state["terms"].c_str();
 comboContractType->Text = PrettyString(lookupTables->ValueOf("entity_contract_types", initial_state["entity_contract_id"])).c_str();
 CreateLink(staticTaxEntity, NULL, this, initial_state["tax_entity_id"], "tax_entities");
 editStart->Text = initial_state["start_date"].c_str();
 editStop->Text = initial_state["stop_date"].c_str();
 TformRecordGeneric::PutDataOnScreen();
 return true;
}
//---------------------------------------------------------------------------
bool TformRecordEntityContract::GetDataFromScreen()
{
 ending_state["terms"] = memoTerms->Text.c_str();
 ending_state["tax_entity_id"] = initial_state["tax_entity_id"];
 ending_state["entity_contract_id"] = lookupTables->IdOf("entity_contract_types", comboContractType->Text.c_str());
 ending_state["start_date"] = (editStart->Text != "") ? Pdate(editStart->Text.c_str(), global_mode).OutDate(global_mode) : string("");
 ending_state["stop_date"] = (editStop->Text != "") ? Pdate(editStop->Text.c_str(), global_mode).OutDate(global_mode) : string("");
 ending_state["id"] = initial_state["id"];
 TformRecordGeneric::GetDataFromScreen();
 return true;
}
//---------------------------------------------------------------------------
bool TformRecordEntityContract::ScreenLock()
{
 panelGroup->Enabled = false;
 memoTerms->ReadOnly = true;
 memoTerms->TabStop = false;
 TformRecordGeneric::ScreenLock();
 return true;
}
//---------------------------------------------------------------------------
bool TformRecordEntityContract::ScreenUnlock()
{
 panelGroup->Enabled = true;
 memoTerms->ReadOnly = false;
 memoTerms->TabStop = true;
 TformRecordGeneric::ScreenUnlock();
 return true;
}
//---------------------------------------------------------------------------
void TformRecordEntityContract::Do(map<string, string> & p_parameters)
{
 FORM_TRY
  TformRecordEntityContract * z = new TformRecordEntityContract(NULL, p_parameters);
  z->Show();
 FORM_CATCH
}
//---------------------------------------------------------------------------
void TformRecordEntityContract::ShowAppropriate(string tax_entity_id, vector<string> possible_contract_ids)
{
 int c = possible_contract_ids.size();
 if (c > 1)
 {
  map<string, string> m;
  m["id"] = tax_entity_id;
  ::PSay("There are several relevant contracts.\nThe tax entity's screen will be opened instead so you can see all of them.", m_message, MB_OK, "TformRecordEntityContract (static)");
  TformRecordTaxEntity::Do(m);
 } else
 if (c == 1)
 {
  map<string, string> m;
  m["id"] = possible_contract_ids[0];
  TformRecordEntityContract::Do(m);
 } else
  ::PSay("There were no relevant contracts.", m_message, MB_OK, "TformRecordEntityContract (static)");
}
//---------------------------------------------------------------------------
void TformRecordEntityContract::FindAppropriate(string tax_entity_id)
{
 if (!IsInt(tax_entity_id) || IsBlankOrZero(tax_entity_id))
  return;
 vector<string> found_ids;
 string sql = "select id from entity_contracts where tax_entity_id = " + tax_entity_id;
 if (!SqlToVec(dbDatabase, sql, found_ids))
  ::PSay("Failed to find relevant contracts.", m_database_error, MB_OK, "TformRecordEntityContract (static)", "entity " + tax_entity_id + ", no date");
 else
  ShowAppropriate(tax_entity_id, found_ids);
}
//---------------------------------------------------------------------------
void TformRecordEntityContract::FindAppropriate(string tax_entity_id, Pdate as_of)
{
 if (!IsInt(tax_entity_id) || IsBlankOrZero(tax_entity_id))
  return;
 vector<string> found_ids;
 string sql = "select id from entity_contracts where tax_entity_id = " + tax_entity_id +
              " and (start_date is null or start_date <= '" + as_of.OutDate(global_mode) + "') "
              " and (stop_date is null or stop_date >= '" + as_of.OutDate(global_mode) + "')";
 if (!SqlToVec(dbDatabase, sql, found_ids))
  ::PSay("Failed to find relevant contracts.", m_database_error, MB_OK, "TformRecordEntityContract (static)", "entity " + tax_entity_id + ", date " + as_of.OutDate(global_mode));
 else
  ShowAppropriate(tax_entity_id, found_ids);
}
//---------------------------------------------------------------------------
// jhaley 20110921 - added for benefit of New CHS Item Wizard
int TformRecordEntityContract::CheckNumContracts(string tax_entity_id, Pdate as_of)
{
  if (!IsInt(tax_entity_id) || IsBlankOrZero(tax_entity_id))
    return -1;

  string sql =
    "select count(distinct id) "
    "from entity_contracts "
    "where tax_entity_id = '" + tax_entity_id + "' and "
    "(start_date is null or start_date <= '" + as_of.OutDate(global_mode) + "') and "
    "(stop_date  is null or stop_date  >= '" + as_of.OutDate(global_mode) + "') ";

  return StringToInt(GetOneField(dbDatabase, sql));
}
//---------------------------------------------------------------------------
