//---------------------------------------------------------------------------

#include <vcl.h>
#pragma hdrstop

#include "codeRecordEvent.h"
#include "codeRecordParticipationsDiab.h"
//#include "codeRecordParticipationsBeh.h"
#include "codeRecordTaskDiab.h"
#include "codeRecordTaskCare.h"
#include "codeRecordTaskBeh.h"
#include "codeRecordTaskCom.h"
#include "codeRecordEventActivity.h"
#include "codeRecordEventRollSheet.h"
#include "launchMailingLabelsWithTasksReport.h"
#include "codeSpecialAddEventTaskContacts.h"
#include "codeSpecialDisambiguate.h"
#include "codePrometheus.h" // for DefaultAction
//---------------------------------------------------------------------------
#pragma package(smart_init)
#pragma link "codeRecordGeneric"
#pragma link "codeRubberContainer"
#pragma resource "*.dfm"
TformRecordEvent *formRecordEvent;
//---------------------------------------------------------------------------
__fastcall TformRecordEvent::TformRecordEvent(TComponent* Owner, map<string,string> p_parameters)
        : TformRecordGeneric(Owner, p_parameters, string("events"))
{
  people_filter.SetListViewHandle(listParticipants);
  people_filter.SetSecurity("participations");
  people_filter.AddColumnSpec("Person Id", "internal_id", as_int, false, 80);
  people_filter.AddColumnSpec("Last Name", "people", as_text, true, 110);
  people_filter.AddColumnSpec("First Name", "people", as_text, true, 110);
  people_filter.AddColumnSpec("Group Id", "internal_id", as_int, false, 80);
  people_filter.AddColumnSpec("Current Id", "current_id", as_int, false, 80);
  people_filter.AddColumnSpec("Internal Id", "internal_id", as_int, false, 80);
  people_filter.FetchPreferences("recordevent.people_filter");
  people_filter.SetIconStatus(true);
  people_filter.SetSQLStatement("");
  people_filter.RefreshDataSet();

  task_filter.SetListViewHandle(listTasks);
  task_filter.SetSecurity("tasks");
  task_filter.AddColumnSpec("Date", "tasks", as_date, true, 90);
  task_filter.AddColumnSpec("Employee", "tasks", as_text, true, 110);
  task_filter.AddColumnSpec("Person", "tasks", as_text, true, 110);
  task_filter.AddColumnSpec("Type", "tasks", as_text, true, 130);
  task_filter.AddColumnSpec("Time Spent", "tasks", as_double, true, 100);
  task_filter.AddColumnSpec("Activity", "tasks", as_text, true, 130);
  task_filter.AddColumnSpec("Group Id", "internal_id", as_int, false, 80); // jhaley - I want to see group_id's here.
  task_filter.AddColumnSpec("Current Id", "current_id", as_int, false, 80);
  task_filter.AddColumnSpec("Internal Id", "internal_id", as_int, false, 80);
  task_filter.FetchPreferences("recordevent.task_filter");
  task_filter.SetIconStatus(true);
  task_filter.SetSQLStatement("");
  task_filter.RefreshDataSet();

  activity_filter.SetListViewHandle(listActivities);
  activity_filter.SetSecurity("activities");
  activity_filter.AddColumnSpec("Activity Id", "internal_id", as_int, false, 80);
  activity_filter.AddColumnSpec("Activity Code Id", "internal_id", as_int, false, 80);
  activity_filter.AddColumnSpec("Activity", "activities", as_text, true, 125);
  activity_filter.AddColumnSpec("Intensity Code Id", "internal_id", as_int, false, 80);
  activity_filter.AddColumnSpec("Intensity", "activities", as_text, true, 80);
  activity_filter.AddColumnSpec("Distance", "activities", as_double, false, 90);
  activity_filter.AddColumnSpec("Time Spent", "activities", as_double, true, 95);
  activity_filter.AddColumnSpec("Steps", "activities", as_double, false, 80);
  activity_filter.AddColumnSpec("Current Id", "current_id", as_int, false, 80);
  activity_filter.AddColumnSpec("Internal Id", "internal_id", as_int, false, 80);
  activity_filter.FetchPreferences("recordevent.activity_filter");
  activity_filter.SetIconStatus(true);
  activity_filter.SetSQLStatement("");
  activity_filter.RefreshDataSet();

  Initialize();

  txtAddContacts->Visible = true;
}
//---------------------------------------------------------------------------
bool TformRecordEvent::ScreenStart()
{
  txtLabels->Font->Color = link_color;
  txtAddContacts->Font->Color = link_color;
  txtViewRollSheet->Font->Color = link_color;

  // jhaley: If not a new record, we need the event's group_id (NOT one passed here externally, which could be wrong).
  // You might think you could just move TformRecordGeneric::ScreenStart up here and let it read the group_id using
  // GetDataFromDatabase, *BUT* "conveniently" it also calls PutDataOnScreen, and we must do the processing here before
  // that happens or we'll just end up staring at a screen full of blank controls.
  // If it IS a new record, the group_id was passed into the constructor and will already be valid. I verified every call
  // to make sure they all do it properly ;)
  if (!is_new_record)
    initial_state["group_id"] = GetOneField(dbDatabase, "select group_id from events where id = '" + initial_state["id"] + "' ");

  set<string> perm_set = GetSpecialGroupPermissions("general");
  string also_perm = initial_state["group_id"];

  // Public Health and Caregiver Support Program should see Community Health items as well.
  if (lookupTables->ValueOf("groups", initial_state["group_id"]) == "Public Health" ||
      lookupTables->ValueOf("groups", initial_state["group_id"]) == "Caregiver Support Program")
    also_perm = lookupTables->IdOf("groups", "Community Health");

  // jhaley 20100726: coalesced group event tables
  if (lookupTables->ValueOf("groups", initial_state["group_id"]) == "Caregiver Support Program")
    lookupTables->RegisterComboBox("care_service_types", cboEventType);
  else
  {
    string custom_type_sql = "select id, item, valid from event_type_types "
                             "where (group_id = 0 or group_id = '" + also_perm + "') and valid = 1 order by item ";
    lookupTables->RegisterComboBox("event_type_types", cboEventType, custom_type_sql);
  }

  string event_loc_sql = "select event_location_types.id, case "
                         "when event_location_types.zip_code_id = 0 then item "
                         "when item = '' then (city || ', ' || state) "
                         "else (city || ', ' || state || ' - ' || item) end item, event_location_types.valid "
                         "from event_location_types "
                         "inner join zip_codes on event_location_types.zip_code_id = zip_codes.id "
                         "where (group_id = 0 or group_id = '" + also_perm + "') "
                         "and valid = 1 order by state, city, item ";
  lookupTables->RegisterComboBox("event_location_types", cboLocation, event_loc_sql);

  string group_sql = "select id, item, valid from event_name_types "
                     "where (group_id = 0 or group_id = '" + also_perm + "') and valid = 1 order by item";
  lookupTables->RegisterComboBox("event_name_types", cboEventName, group_sql, SetGroupIdForNewLookupTableItem);

  // jhaley 20100913: program_types
  string event_program_sql = "select id, item, valid from program_types "
                             "where (group_id = 0 or group_id = '" + also_perm + "') and valid = 1 order by item ";
  lookupTables->RegisterComboBox("program_types", cboProgramType, event_program_sql);

  TformRecordGeneric::ScreenStart();
  return true;
}
//---------------------------------------------------------------------------
bool TformRecordEvent::ScreenStop()
{
  // unregister combo boxes

  // jhaley 20100726: coalesced group event tables
  if (lookupTables->ValueOf("groups", initial_state["group_id"]) == "Caregiver Support Program")
    lookupTables->UnregisterComboBox("care_service_types", cboEventType);
  else
    lookupTables->UnregisterComboBox("event_type_types", cboEventType);

  lookupTables->UnregisterComboBox("event_location_types", cboLocation);
  lookupTables->UnregisterComboBox("event_name_types", cboEventName);
  lookupTables->UnregisterComboBox("program_types", cboProgramType);

  TformRecordGeneric::ScreenStop();
  return true;
}
//---------------------------------------------------------------------------
bool TformRecordEvent::IsValidOnScreen()
{
  // check to make sure that the values entered in the combo boxes
  // corrospond to values in that look up table.
  // check to make sure the attendence is an integer
  // give the user a message box and prompt to re-enter if incorrect

  // jhaley 20100726: coalesced group event tables
  string event_type_table;

  if (lookupTables->ValueOf("groups", initial_state["group_id"]) == "Caregiver Support Program")
    event_type_table = "care_service_types";
  else
    event_type_table = "event_type_types";

  if (!lookupTables->IsInLookUpTable(event_type_table, cboEventType))
  {
    PSay("Please enter a valid event type", m_invalid_data, MB_OK);
    cboEventType->SetFocus();
    return false;
  }
  else if (!lookupTables->IsInLookUpTable("event_location_types", cboLocation))
  {
    PSay("Please enter a valid location", m_invalid_data, MB_OK);
    cboLocation->SetFocus();
    return false;
  }
  else if (!lookupTables->IsInLookUpTable("program_types", cboProgramType))
  {
    PSay("Please enter a valid program type", m_invalid_data, MB_OK);
    cboProgramType->SetFocus();
    return false;
  }
  else if (!IsInt(editAttendance->Text.c_str()))
  {
    PSay("Please enter a valid integer for attendance", m_invalid_data, MB_OK);
    editAttendance->SetFocus();
    return false;
  }
  else if (initial_state["group_id"] == "" || StringToInt(initial_state["group_id"]) <= 0)
  {
    // this should not ever happen; group_id is passed in from all callers.
    PSay("This event does not belong to a valid department. Please contact a programmer if this occured while creating a new event.",
         m_program_error, MB_OK);
    return false;
  }

  return true;
}
//---------------------------------------------------------------------------
bool TformRecordEvent::PutDataOnScreen()
{
  // put the values on the form from the map initial_state
  // values may have come from GetDataFromDatabase if it's an existing record,
  // or values may have been passed to the form in this map if it's a new record

  cboEventName->Text = lookupTables->ValueOf("event_name_types", initial_state["event_name_id"], cboEventName).c_str();

  if (initial_state["date_of_event"] != "")
    dateEvent->Date = Pdate(initial_state["date_of_event"], mdy | yyyy).OutDate();
  else
    dateEvent->Date = Date();

  editDescription->Text = SentenceString(initial_state["description"]).c_str();

  // jhaley 20100726: coalesced group event tables
  if (lookupTables->ValueOf("groups", initial_state["group_id"]) == "Caregiver Support Program")
    cboEventType->Text = PrettyString(lookupTables->ValueOf("care_service_types", initial_state["care_service_id"])).c_str();
  else
    cboEventType->Text = lookupTables->ValueOf("event_type_types", initial_state["event_type_id"], cboEventType).c_str();

  cboLocation->Text    = lookupTables->ValueOf("event_location_types", initial_state["event_location_id"], cboLocation).c_str();
  cboProgramType->Text = lookupTables->ValueOf("program_types", initial_state["program_id"], cboProgramType).c_str();

  editAttendance->Text = initial_state["number_attended"].c_str();

  string event_roll_id = GetRollSheetID();
  if(event_roll_id != "")
    txtViewRollSheet->Caption = "View roll sheet for this event";
  else
    txtViewRollSheet->Caption = "Create a roll sheet for this event";

  TformRecordGeneric::PutDataOnScreen();

  return true; // what can really go wrong?
}
//---------------------------------------------------------------------------
bool TformRecordEvent::GetDataFromScreen()
{
  // place the values currently on the screen into
  // the map that will be used to update or insert the record
  // the parent form will call the insert or update function, and get the id
  // number if necessary and take care of current_id
  ending_state["event_name_id"] = lookupTables->IdOf("event_name_types", cboEventName);
  ending_state["date_of_event"] = Pdate(dateEvent->Date).OutDate(mdy | yyyy);
  ending_state["description"] = editDescription->Text.c_str();

  // set event type depending on department
  // jhaley 20100726: coalesced group event tables

  if (lookupTables->ValueOf("groups", initial_state["group_id"]) == "Caregiver Support Program")
  {
    ending_state["care_service_id"] = lookupTables->IdOf("care_service_types", cboEventType->Text.c_str());
    ending_state["event_type_id"] = "0";
  }
  else
  {
    ending_state["care_service_id"] = "0";
    ending_state["event_type_id"] = lookupTables->IdOf("event_type_types", cboEventType);
  }

  ending_state["event_location_id"] = lookupTables->IdOf("event_location_types", cboLocation);
  ending_state["program_id"]  = lookupTables->IdOf("program_types",  cboProgramType);

  if (editAttendance->Text != "")
    ending_state["number_attended"] = editAttendance->Text.c_str();
  else
    ending_state["number_attended"] = "";

  if (initial_state["added_activities"] != "")
    ending_state["added_activities"] = initial_state["added_activities"];
  else
    ending_state["added_activities"] = "0";

  ending_state["group_id"] = initial_state["group_id"]; // group_id is constant across this form.
  ending_state["id"] = initial_state["id"];

  TformRecordGeneric::GetDataFromScreen();

  return true; // again, not much can go wrong here
}
//---------------------------------------------------------------------------
bool TformRecordEvent::ScreenLock()
{
  // disable panel containing edit boxes
  eventGroup->Enabled = false;
  editDescription->ReadOnly = true;
  editDescription->TabStop = false;
  TformRecordGeneric::ScreenLock();
  return true;
}
//---------------------------------------------------------------------------
bool TformRecordEvent::ScreenUnlock()
{
  // enable panel containing edit boxes
  eventGroup->Enabled = true;
  editDescription->ReadOnly = false;
  editDescription->TabStop = true;
  TformRecordGeneric::ScreenUnlock();
  return true;
}
//---------------------------------------------------------------------------
bool TformRecordEvent::IsNewAgain()
{
  people_filter.SetSQLStatement(string(""));
  people_filter.RefreshDataSet();
  people_filter.SetAddEdit(this, NULL, NULL);
  task_filter.SetSQLStatement(string(""));
  task_filter.RefreshDataSet();
  task_filter.SetAddEdit(this, NULL, NULL);
  activity_filter.SetSQLStatement(string(""));
  activity_filter.RefreshDataSet();
  activity_filter.SetAddEdit(this, NULL, NULL);
  TformRecordGeneric::IsNewAgain();
  txtAddContacts->Enabled = false; // jhaley: turn off the contacts link for new records
  txtViewRollSheet->Enabled = false;
  txtViewRollSheet->Caption = "Create a roll sheet for this event";
  return true;
}
//---------------------------------------------------------------------------
bool TformRecordEvent::NotNewAnymore()
{
  string sql;
  if(lookupTables->ValueOf("groups", initial_state["group_id"]) == "Diabetes" &&
     optionsPermissions["table.activities"]["view"] == "true")
  {
    sql = "select event_activities.id, event_activities.activity_code_id, activity_code_types.item, "
          "event_activities.intensity_id, intensity_types.item, event_activities.distance, event_activities.time_spent, "
          "event_activities.steps, event_activities.current_id, event_activities.id "
          "from event_activities, activity_code_types, intensity_types "
          "where event_activities.activity_code_id = activity_code_types.id and "
          "event_activities.intensity_id = intensity_types.id and "
          "event_activities.event_id = '" + initial_state["id"] + "'";

    activity_filter.SetSQLStatement(sql);
    activity_filter.ForceRefreshDataSet();
    activity_filter.SetAddEdit(this, AddRecord(&(TformRecordEvent::AddEventActivity)),
                                    EditRecord(&(TformRecordEvent::EditEventActivity)));
    activity_request = atlas->RequestNotificationWhen("event_activities.event_id", initial_state["id"],
                       rel_e, as_int, this, RefreshHandler(&(TformRecordEvent::RefreshListView)));
   }

   sql = "select participations.person_id, person_names.last_name, person_names.first_name, "
         "participations.group_id, participations.current_id, participations.id "
         "from participations, people, person_names "
         "where participations.event_id = " + initial_state["id"] + " and "
         "participations.person_id = people.id and "
         "person_names.id = people.person_names_id order by person_names.last_name, "
         "person_names.first_name";

  people_filter.SetSQLStatement(sql);
  people_filter.RefreshDataSet();
  people_filter.SetAddEdit(this, AddRecord(&(TformRecordEvent::AddPerson)), EditRecord(&(TformRecordEvent::EditPerson)));
  people_request = atlas->RequestNotificationWhen("participations.event_id", initial_state["id"], rel_e, as_int, this,
                     RefreshHandler(&(TformRecordEvent::RefreshListView)));

  // jhaley 20100729: setup the tasks filter in a more group-independent way
  string activity_table = "";
  string activity_id = "";
  bool has_activity_table = false;

  // Determine the activity lookup - these are quite different and so have not been merged together.
  // Not to mention they'd need special treatment with regard to COM and CSP groups anyway. Also some
  // groups who have new access to events won't have this table at all.
  if (lookupTables->ValueOf("groups", initial_state["group_id"]) == "Diabetes")
  {
    activity_table = "diab_activity_types";
    activity_id    = "diab_activity_id";
    has_activity_table = true;
  }
  else if (lookupTables->ValueOf("groups", initial_state["group_id"]) == "Behavioral Health")
  {
    activity_table = "beh_activity_types";
    activity_id    = "beh_activity_id";
    has_activity_table = true;
  }
  else if (lookupTables->ValueOf("groups", initial_state["group_id"]) == "Community Health" ||
           lookupTables->ValueOf("groups", initial_state["group_id"]) == "Public Health"    ||
           lookupTables->ValueOf("groups", initial_state["group_id"]) == "Caregiver Support Program")
  {
    // Public Health and CSP are odd ones out, and use Community Health's activities lookup
    activity_table = "com_activity_types";
    activity_id    = "com_activity_id";
    has_activity_table = true;
  }

  sql =  "select tasks.date_of_task, users.item, person_names.first_name || ' ' || person_names.last_name, "
         "task_purpose_types.item, tasks.time_spent, ";
  if (has_activity_table)
    sql += activity_table + ".item, ";
  else
    sql += "'', "; // empty if no table.
  sql += "tasks.group_id, tasks.current_id, tasks.id "
         "from tasks "
         "inner join users on tasks.user_id = users.id ";
  if (has_activity_table)
    sql += "inner join " + activity_table + " on tasks." + activity_id + " = " + activity_table + ".id ";
  sql += "inner join people on tasks.person_id = people.id "
         "inner join person_names on people.person_names_id = person_names.id "
         "inner join task_purpose_types on tasks.task_purpose_id = task_purpose_types.id "
         "where tasks.event_id = '" + initial_state["id"] + "' ";

  task_filter.SetSQLStatement(sql);
  task_filter.RefreshDataSet();
  task_filter.SetAddEdit(this, AddRecord(&(TformRecordEvent::AddTask)), EditRecord(&(TformRecordEvent::EditTask)));
  task_request = atlas->RequestNotificationWhen("tasks.event_id", initial_state["id"], rel_e, as_int, this,
                                                RefreshHandler(&(TformRecordEvent::RefreshListView)));

  // jhaley: let everybody print participant labels
  set<string> allowed_task_groups = GetGroupPermissions("table.tasks", "", "insert", "true");
  if (allowed_task_groups.find(initial_state["group_id"]) != allowed_task_groups.end() &&
      optionsPermissions["table.people"]["view"] == "true" &&
      optionsPermissions["table.person_contact_info"]["view"] == "true" &&
      optionsPermissions["table.affiliations"]["view"] == "true")
    txtLabels->Enabled = true;

  // jhaley: contacts link should be off until we have some valid data to work with (which is now)
  txtAddContacts->Enabled = true;
  txtViewRollSheet->Enabled = true;

  string roll_sheet_id = GetRollSheetID();
  if(roll_sheet_id != "")
    txtViewRollSheet->Caption = "View roll sheet for this event";
  else
    txtViewRollSheet->Caption = "Create a roll sheet for this event";
  event_roll_request = atlas->RequestNotificationWhen("event_roll_sheets.event_id", initial_state["id"], rel_e, as_int,
                                                      this, RefreshHandler(&(TformRecordEvent::RefreshListView)));

  TformRecordGeneric::NotNewAnymore();
  return true;
}
//---------------------------------------------------------------------------
void TformRecordEvent::RefreshListView(int request_num, string matched_value)
{
  if (request_num == people_request)
    people_filter.RefreshDataSet();
  else if (request_num == task_request)
    task_filter.RefreshDataSet();
  else if (request_num == activity_request)
    activity_filter.RefreshDataSet();
  else if (request_num == event_roll_request)
  {
    string roll_sheet_id = GetRollSheetID();
    if(roll_sheet_id != "")
      txtViewRollSheet->Caption = "View roll sheet for this event";
    else
      txtViewRollSheet->Caption = "Create a roll sheet for this event";
  }
}
//---------------------------------------------------------------------------
void TformRecordEvent::AddPerson()
{
  map<string, string> temp_map;
  temp_map["id"] = "new";
  temp_map["event_id"] = initial_state["id"];
  temp_map["date_of_event"] = Pdate(dateEvent->Date).OutDate(mdy | yyyy);
  temp_map["group_id"] = initial_state["group_id"];

  // jhaley 20100804: Everybody now uses the "Diab" participations form.
  FORM_TRY
    TformRecordParticipationsDiab *diabParticipation = new TformRecordParticipationsDiab(NULL, temp_map);
    diabParticipation->Show();
  FORM_CATCH
}
//---------------------------------------------------------------------------
void TformRecordEvent::EditPerson()
{
  map<string, string> temp_map;
  temp_map["id"] = people_filter.GetSelectedRecordID();

  // jhaley 20100804: Everybody now uses the "Diab" participations form.
  FORM_TRY
    TformRecordParticipationsDiab * diabParticipation = new TformRecordParticipationsDiab(NULL, temp_map);
    diabParticipation->Show();
  FORM_CATCH
}
//---------------------------------------------------------------------------
void TformRecordEvent::AddTask()
{
  map<string, string> field_map;

  // set up fields for the new task
  field_map["id"] = "new";
  field_map["event_id"] = initial_state["id"];
  field_map["user_id"] = current_userid;
  field_map["date_of_task"] = initial_state["date_of_event"];
  field_map["group_id"] = initial_state["group_id"]; // task group_id must equal event group_id!

  // should not happen; group_id is passed valid from all callers.
  if (field_map["group_id"] == "" || StringToInt(field_map["group_id"]) <= 0)
  {
    PSay("The event you are viewing does not belong to a valid department. You cannot attach a task to it. "
         "Please contact a programmer if this error appeared while creating a new event.",
         m_program_error, MB_OK);
  }
  else
    DefaultAction("tasks", field_map);
}
//---------------------------------------------------------------------------
void TformRecordEvent::EditTask()
{
  DefaultAction(task_filter.GetSelectedRecordID(), "tasks");

  // Note that public health uses a different form than this via DefaultAction.
  /*
  else if (group_name == "Public Health")
  {
    FORM_TRY
      TformRecordTaskCom *comTask = new TformRecordTaskCom(NULL, temp_map);
      comTask->Show();
    FORM_CATCH
  }
  */
}
//---------------------------------------------------------------------------
void TformRecordEvent::EditEventActivity()
{
  map<string,string> temp_map;
  temp_map["id"] = activity_filter.GetSelectedRecordID();
  FORM_TRY
    TformRecordEventActivity * activityRecord = new TformRecordEventActivity(NULL, temp_map);
    activityRecord->Show();
  FORM_CATCH
}
//---------------------------------------------------------------------------
void TformRecordEvent::AddEventActivity()
{
  map<string,string> temp_map;
  temp_map["id"] = "new";
  temp_map["event_id"] = initial_state["id"];
  temp_map["date_of_event"] = Pdate(dateEvent->Date).OutDate(mdy | yyyy);
  FORM_TRY
    TformRecordEventActivity * activityRecord = new TformRecordEventActivity(NULL, temp_map);
    activityRecord->Show();
  FORM_CATCH
}
//---------------------------------------------------------------------------
void __fastcall TformRecordEvent::txtLabelsClick(TObject *Sender)
{
  map<string, string> temp_map;
  vector<string> id_vec;
  SqlToVec(dbDatabase, "select person_id from participations where event_id = " + initial_state["id"], id_vec);
  if(! id_vec.empty())
  {
    temp_map["user_id"] = current_userid;
    temp_map["date_of_task"] = CurrentDate();
    temp_map["time_spent"] = "";
    temp_map["method_id"] = lookupTables->IdOf("method_types", "Mail");
    temp_map["direction_id"] = lookupTables->IdOf("direction_types", "Outgoing");
    temp_map["group_id"] = initial_state["group_id"]; // group_id is constant across this form.

    FORM_TRY
      TformLaunchMailingLabelsWithTasksReport *launch_form = new TformLaunchMailingLabelsWithTasksReport(NULL, temp_map, id_vec);
      launch_form->Show();
    FORM_CATCH
  }
  else
    PSay("There are no people who attended this event", m_missing_data, MB_OK);
}
//---------------------------------------------------------------------------
void __fastcall TformRecordEvent::txtAddContactsClick(TObject *Sender)
{
  // jhaley: although the add event contact form will show an error message on the form itself if any of the
  // prerequisite data is missing, this is sort of odd and irritating as far as workflow experience goes, so I
  // have added some quick checks here which will avoid having a useless form to close.
  string participant_count = GetOneField(dbDatabase, "select count(1) from participations where event_id = '" + initial_state["id"] + "' ");
  string task_count = GetOneField(dbDatabase, "select count(1) from tasks where event_id = '" + initial_state["id"] + "' "
                                              "and current_id >= 3 and task_purpose_id = (select task_purpose_types.id from task_purpose_types "
                                              "where UPPER(item) like 'EVENT TEACHING/LEADING')");

  if (StringToInt(participant_count) == 0)
  {
    PSay("Please add one or more participants to the event first.", m_missing_data, MB_OK);
  }
  else if (StringToInt(task_count) == 0)
  {
    PSay("Please add a task of type 'Event Teaching/Leading' to this event first.", m_missing_data, MB_OK);
  }
  else
  {
    FORM_TRY
      TformSpecialAddEventTaskContacts *task_contacts_form = new TformSpecialAddEventTaskContacts(NULL, initial_state);
      task_contacts_form->Show();
    FORM_CATCH
  }
}
//---------------------------------------------------------------------------
void TformRecordEvent::Do(pmapstrs & p_parameters)
{
  TformRecordEvent *z = NULL;
  FORM_TRY
    z = new TformRecordEvent(NULL, p_parameters);
  FORM_CATCH_RETURN("TformRecordEvent", )
  z->Show();
}
//---------------------------------------------------------------------------
// jhaley 20110411: Roll sheet functionality
void __fastcall TformRecordEvent::txtViewRollSheetClick(TObject *Sender)
{
  if(IsInt(initial_state["id"]))
  {
    map<string, string> field_map;
    string event_roll_id = GetRollSheetID();
    if(event_roll_id == "")
      field_map["id"] = "new";
    else
      field_map["id"] = event_roll_id;

    FORM_TRY
      TformRecordEventRollSheet *roll_sheet_form = new TformRecordEventRollSheet(NULL, field_map, initial_state["id"]);
      roll_sheet_form->Show();
    FORM_CATCH
  }
}
//---------------------------------------------------------------------------
string TformRecordEvent::GetRollSheetID()
{
  if(!IsInt(initial_state["id"]))
    return "";

  vector<string> event_roll_ids;
  string event_roll_sql =
    "select id from event_roll_sheets where event_id = '" + initial_state["id"] + "' ";
  SqlToVec(dbDatabase, event_roll_sql, event_roll_ids);

  if(event_roll_ids.empty())
    return "";
  else
    return event_roll_ids[0];
}
//---------------------------------------------------------------------------
void TformRecordEvent::DoCreateMedical(TForm *parent, TComponent* Owner, pmapstrs &p_parameters)
{
  string   group_id = "";
  pmapstrs params;

  if(p_parameters["id"] != "new")
    return; // This function is only for creating new events.

  params["prefix"] = "table.events";
  params["suffix"] = "";
  params["key"]    = "insert";
  params["value"]  = "true";
  params["instructions"] = "You have permission to create events for multiple clinics. "
                           "Please select the clinic for which to create events.";
  params["include"] = lookupTables->IdOf("groups", "Little Axe Clinic") + ", " +
                      lookupTables->IdOf("groups", "Shawnee Clinic");
  params["exclude"] = "";
  params["table"]   = "groups";
  params["sql"]     = "select id row_id, short_item row_value, item show_value from groups where valid = 1 and id > 0";

  group_id = TformSpecialDisambiguate::Disambiguate(parent, params, PrettyString);
  if (group_id != "")
  {
    p_parameters["group_id"] = group_id;
    TformRecordEvent *event_form = NULL;
    FORM_TRY
      event_form = new TformRecordEvent(Owner, p_parameters);
    FORM_CATCH_RETURN("TformRecordEvent", )
    event_form->Show();
  }
}
//---------------------------------------------------------------------------
