//---------------------------------------------------------------------------

#include <vcl.h>
#pragma hdrstop

#include "codeRecordEventRollItem.h"
#include "codeRecordEventRollSheet.h"
#include "codeSearchPerson.h"
//---------------------------------------------------------------------------
#pragma package(smart_init)
#pragma link "codeRecordGeneric"
#pragma resource "*.dfm"
TformRecordEventRollItem *formRecordEventRollItem;
//---------------------------------------------------------------------------
__fastcall TformRecordEventRollItem::TformRecordEventRollItem(TComponent* Owner, map<string,string> &p_parameters)
        : TformRecordGeneric(Owner, p_parameters, string("event_roll_items"))
{
  Initialize();
}
//---------------------------------------------------------------------------
bool TformRecordEventRollItem::ScreenStart()
{
  lookupTables->RegisterComboBox("gender_types", cboGender);
  txtRollSheet->Font->Color = link_color;
  txtRollSheet->Enabled = false;

  return TformRecordGeneric::ScreenStart();
}
//---------------------------------------------------------------------------
bool TformRecordEventRollItem::ScreenStop()
{
  lookupTables->UnregisterComboBox("gender_types", cboGender);

  return TformRecordGeneric::ScreenStop();
}
//---------------------------------------------------------------------------
bool TformRecordEventRollItem::IsValidOnScreen()
{
  string person_id = GetValidLinkId(txtPerson);

  // Should not happen. Cannot save without a valid event_roll_id.
  if(IsBlankOrZero(initial_state["event_roll_id"]))
  {
    PSay("This event roll sheet entry is not attached to a valid event roll sheet. Please contact a programmer.",
         m_program_error, MB_OK);
    return false;
  }

  // When saving a new record and a person_id is specified, none of the other fields may be set.
  // If it's being updated later to reflect the fact that person was added to Prometheus, we can allow that.
  // When editing a record and a person_id is specified, name/age/gender data cannot be added or edited unless it
  // was already present on the record.
  if(is_new_record && !IsBlankOrZero(person_id))
  {
    if(editPrefix->Text != "" || editFirstName->Text != "" || editMiddleName->Text != "" || editLastName->Text != "" ||
       editSuffix->Text != "" || editAge->Text != "" || cboGender->Text != "")
    {
      PSay("You have specified a person using the \"Person in Prometheus\" link, but have also specified information "
           "in one or more of the fields for a person who is not in Prometheus.\n\n"
           "Please either clear the link or remove the redundant information before saving this roll sheet entry.\n\n"
           "If the person can be found in Prometheus, you should use the link.", m_invalid_data, MB_OK);
      return false;
    }
  }
  else if(is_editing && !IsBlankOrZero(person_id))
  {
    if((initial_state["prefix"]      == ""  && editPrefix->Text     != "") ||
       (initial_state["first_name"]  == ""  && editFirstName->Text  != "") ||
       (initial_state["middle_name"] == ""  && editMiddleName->Text != "") ||
       (initial_state["last_name"]   == ""  && editLastName->Text   != "") ||
       (initial_state["suffix"]      == ""  && editSuffix->Text     != "") ||
       (initial_state["age"]         == ""  && editAge->Text        != "") ||
       (initial_state["gender_id"]   == "0" && cboGender->Text      != ""))
    {
      PSay("There is already a person attached to this record; you should not enter this information here.\n\n"
           "If you need to update the person's name, age, or gender information, do that by searching for the person "
           "(Search->Person... on the main menu bar), and edit the information on their Person record form.\n\n"
           "If you attached the wrong person by mistake, you may clear the link by searching for a person and then "
           "press the \"Select\" button at the bottom of the search form without choosing anyone.",
           m_invalid_data, MB_OK);
      return false;
    }
  }

  // It's not a useful record without either a person_id or at least one part of the person's name.
  if(IsBlankOrZero(person_id) && editPrefix->Text == "" && editFirstName->Text == "" && editMiddleName->Text == "" &&
     editLastName->Text == "" && editSuffix->Text == "")
  {
    PSay("Please either specify a person in Prometheus, or provide one or more parts of the person's name.",
         m_missing_data, MB_OK);
    editFirstName->SetFocus();
    return false;
  }
  if(editAge->Text != "" && !IsInt(editAge->Text.c_str()))
  {
    PSay("Please enter a numeric age in years", m_invalid_data, MB_OK);
    editAge->SetFocus();
    return false;
  }
  if(cboGender->Text != "" && !lookupTables->IsInLookUpTable("gender_types", cboGender->Text.c_str()))
  {
    PSay("Please enter a valid gender", m_invalid_data, MB_OK);
    cboGender->SetFocus();
    return false;
  }

  return TformRecordGeneric::IsValidOnScreen();
}
//---------------------------------------------------------------------------
bool TformRecordEventRollItem::PutDataOnScreen()
{
  if(!IsBlankOrZero(initial_state["event_roll_id"]))
  {
    txtRollSheet->Caption = "View roll sheet";
    txtRollSheet->Enabled = true;
  }

  editPrefix->Text     = PrettyString(initial_state["prefix"]     ).c_str();
  editFirstName->Text  = PrettyString(initial_state["first_name"] ).c_str();
  editMiddleName->Text = PrettyString(initial_state["middle_name"]).c_str();
  editLastName->Text   = PrettyString(initial_state["last_name"]  ).c_str();
  editSuffix->Text     = PrettyString(initial_state["suffix"]     ).c_str();

  editAge->Text = initial_state["age"].c_str();
  cboGender->Text = PrettyString(lookupTables->ValueOf("gender_types", initial_state["gender_id"])).c_str();

  CreateLink(txtPerson, btnPersonSearch, this, initial_state["person_id"], "people");

  return TformRecordGeneric::PutDataOnScreen();
}
//---------------------------------------------------------------------------
bool TformRecordEventRollItem::GetDataFromScreen()
{
  ending_state["id"] = initial_state["id"];
  ending_state["event_roll_id"] = initial_state["event_roll_id"];

  string person_id = GetValidLinkId(txtPerson);
  if(!IsBlankOrZero(person_id))
    ending_state["person_id"] = person_id;
  else
    ending_state["person_id"] = "0";

  ending_state["prefix"]      = SafeSQLString(editPrefix->Text.c_str());
  ending_state["first_name"]  = SafeSQLString(editFirstName->Text.c_str());
  ending_state["middle_name"] = SafeSQLString(editMiddleName->Text.c_str());
  ending_state["last_name"]   = SafeSQLString(editLastName->Text.c_str());
  ending_state["suffix"]      = SafeSQLString(editSuffix->Text.c_str());

  ending_state["age"] = editAge->Text.c_str();
  ending_state["gender_id"] = lookupTables->IdOf("gender_types", cboGender->Text.c_str());

  return TformRecordGeneric::GetDataFromScreen();
}
//---------------------------------------------------------------------------
bool TformRecordEventRollItem::BeforeSave()
{
  AtlasClear();

  string person_id     = GetValidLinkId(txtPerson);
  string event_roll_id = initial_state["event_roll_id"];

  // Need event id of the event that owns this roll sheet
  string event_id = GetOneField(lock_transaction,
                                "select event_id from event_roll_sheets where id = '" + event_roll_id + "' ");

  // If there is a valid person_id and event_id, add a participation for this person too, automatically.
  // Don't care that much if this fails for some reason. It shouldn't prevent them from saving the roll item.
  if(!IsBlankOrZero(person_id) && !IsBlankOrZero(event_id))
  {
    // Need event's group_id
    string group_id = GetOneField(lock_transaction, "select group_id from events where id = '" + event_id + "' ");
    if(!IsBlankOrZero(group_id))
    {
      vector<string> existing_participations;
      if(SqlToVec(lock_transaction, "select id from participations where person_id = '" + person_id + "' and "
                                    "event_id = '" + event_id + "' and current_id > 1 ", existing_participations))
      {
        // Only if there is no existing participation already, add a new one
        if(existing_participations.empty())
        {
          map<string, string> field_map;
          field_map["id"] = GetNextId(lock_transaction, "participations");
          field_map["current_id"] = lookupTables->IdOf("current_types", "Current");
          field_map["last_update_stamp"] = RightNow();
          field_map["last_updated_by"] = current_userid;
          field_map["person_id"] = person_id;
          field_map["event_id"] = event_id;
          field_map["group_id"] = group_id;
          field_map["diab_special_id"] = "0";
          field_map["note"] = "ADDED BY EVENT ROLL SHEET ENTRY";
          ExecuteInsertStatement(lock_transaction, "participations", field_map);
          AtlasAdd("participations", field_map); // Atlas refresh
        }
      }
    }
  }

  return TformRecordGeneric::BeforeSave();
}
//---------------------------------------------------------------------------
bool TformRecordEventRollItem::AfterSave()
{
  AtlasSend();
  return TformRecordGeneric::AfterSave();
}
//---------------------------------------------------------------------------
bool TformRecordEventRollItem::ScreenLock()
{
  // disable panel containing edit boxes
  panelGroup->Enabled = false;
  btnPersonSearch->Enabled = false;
  return TformRecordGeneric::ScreenLock();
}
//---------------------------------------------------------------------------
bool TformRecordEventRollItem::ScreenUnlock()
{
  // enable panel containing edit boxes
  panelGroup->Enabled = true;
  btnPersonSearch->Enabled = true;
  return TformRecordGeneric::ScreenUnlock();
}
//---------------------------------------------------------------------------
void __fastcall TformRecordEventRollItem::btnPersonSearchClick(TObject *Sender)
{
  vector<string> return_vector = TformSearchPerson::Do(this);
  if (!return_vector.empty())
    CreateLink(txtPerson, btnPersonSearch, this, return_vector[0], "people");
}
//---------------------------------------------------------------------------
void __fastcall TformRecordEventRollItem::txtRollSheetClick(TObject *Sender)
{
  map<string, string> field_map;
  field_map["id"] = initial_state["event_roll_id"];
  FORM_TRY
    TformRecordEventRollSheet *roll_sheet = new TformRecordEventRollSheet(NULL, field_map, "");
    roll_sheet->Show();
  FORM_CATCH
}
//---------------------------------------------------------------------------

