//---------------------------------------------------------------------------

#include <vcl.h>
#pragma hdrstop

#include "codeRecordEventRollSheet.h"
#include "codeRecordEventRollItem.h"
#include "launchRollSheetReport.h"
//---------------------------------------------------------------------------
#pragma package(smart_init)
#pragma link "codeRecordGeneric"
#pragma link "codeRubberContainer"
#pragma resource "*.dfm"
TformRecordEventRollSheet *formRecordEventRollSheet;
//---------------------------------------------------------------------------
__fastcall TformRecordEventRollSheet::TformRecordEventRollSheet(TComponent* Owner, map<string, string> &p_parameters, string p_event_id)
        : TformRecordGeneric(Owner, p_parameters, string("event_roll_sheets"))
{
  roll_entry_filter.SetListViewHandle(listEventRollItems);
  roll_entry_filter.SetSecurity("event_roll_items");
  roll_entry_filter.AddColumnSpec("Prefix",      "event_roll_items", as_text, false, 80,  prettyString);
  roll_entry_filter.AddColumnSpec("First Name",  "event_roll_items", as_text, true,  130, prettyString);
  roll_entry_filter.AddColumnSpec("Middle Name", "event_roll_items", as_text, true,  130, prettyString);
  roll_entry_filter.AddColumnSpec("Last Name",   "event_roll_items", as_text, true,  130, prettyString);
  roll_entry_filter.AddColumnSpec("Suffix",      "event_roll_items", as_text, false, 80,  prettyString);
  roll_entry_filter.AddColumnSpec("Age",         "event_roll_items", as_int,  true,  60);
  roll_entry_filter.AddColumnSpec("Gender",      "event_roll_items", as_text, true,  60,  prettyString);
  roll_entry_filter.AddColumnSpec("Current Id",  "current_id",       as_int,  false, 80);
  roll_entry_filter.AddColumnSpec("Internal Id", "internal_id",      as_int,  false, 80);
  roll_entry_filter.FetchPreferences("recordeventrollsheet.items");
  roll_entry_filter.SetSQLStatement("");
  roll_entry_filter.RefreshDataSet();

  event_id = p_event_id;

  lblPrintRollSheet->Font->Color = link_color;
  lblPrintRollSheet->Enabled = false;

  Initialize();
}
//---------------------------------------------------------------------------
bool TformRecordEventRollSheet::IsValidOnScreen()
{
  if(GetValidLinkId(txtEventName) == "")
  {
    PSay("This event roll sheet is not attached to a valid event. Please contact a programmer immediately.",
         m_program_error, MB_OK);
    return false;
  }

  return true;
}
//---------------------------------------------------------------------------
bool TformRecordEventRollSheet::PutDataOnScreen()
{
  if(!IsBlankOrZero(initial_state["event_id"]))
    event_id = initial_state["event_id"];

  if(!is_new_record)
    lblPrintRollSheet->Enabled = true;

  CreateLink(txtEventName, NULL, this, event_id, "events");

  TformRecordGeneric::PutDataOnScreen();
  return true;
}
//---------------------------------------------------------------------------
bool TformRecordEventRollSheet::GetDataFromScreen()
{
  ending_state["id"]       = initial_state["id"];
  ending_state["event_id"] = GetValidLinkId(txtEventName);

  TformRecordGeneric::GetDataFromScreen();
  return true;
}
//---------------------------------------------------------------------------
void __fastcall TformRecordEventRollSheet::FormShow(TObject *Sender)
{
  TformRecordGeneric::FormShow(Sender);
}
//---------------------------------------------------------------------------
bool TformRecordEventRollSheet::NotNewAnymore()
{
  string roll_entry_sql =
    "select "
    "case when PN.id is not null then PN.prefix      else ERI.prefix end, "
    "case when PN.id is not null then PN.first_name  else ERI.first_name end, "
    "case when PN.id is not null then PN.middle_name else ERI.middle_name end, "
    "case when PN.id is not null then PN.last_name   else ERI.last_name end, "
    "case when PN.id is not null then PN.suffix      else ERI.suffix end, "
    "case when P.id   is not null then (select p_age from calc_age(P.birth_date)) else ERI.age end, "
    "coalesce(GT.item, ''), ERI.current_id, ERI.id "
    "from event_roll_items ERI "
    "left join people P on ERI.person_id = P.id and P.id <> 0 "
    "left join person_names PN on P.person_names_id = PN.id "
    "left join gender_types GT on GT.id = (case when P.id is not null then P.gender_id else ERI.gender_id end) "
    "where ERI.event_roll_id = '" + initial_state["id"] + "' ";

  roll_entry_filter.SetSQLStatement(roll_entry_sql);
  roll_entry_filter.RefreshDataSet();
  roll_entry_filter.SetAddEdit(this,  AddRecord(&(TformRecordEventRollSheet::AddEntry)),
                                     EditRecord(&(TformRecordEventRollSheet::EditEntry)));
  roll_entry_request = atlas->RequestNotificationWhen("event_roll_items.id", "", rel_any, as_int, this,
                                                      RefreshHandler(&(TformRecordEventRollSheet::AtlasRefresh)));

  lblPrintRollSheet->Enabled = true;

  TformRecordGeneric::NotNewAnymore();
  return true;
}
//---------------------------------------------------------------------------
bool TformRecordEventRollSheet::IsNewAgain()
{
  roll_entry_filter.SetSQLStatement("");
  roll_entry_filter.RefreshDataSet();
  roll_entry_filter.SetAddEdit(this, NULL, NULL);

  lblPrintRollSheet->Enabled = false;

  TformRecordGeneric::IsNewAgain();
  return true;
}
//---------------------------------------------------------------------------
bool TformRecordEventRollSheet::AtlasRefresh(int request_num, string matched_value)
{
  if(request_num == roll_entry_request)
    roll_entry_filter.RefreshDataSet();

  return true;
}
//---------------------------------------------------------------------------
void TformRecordEventRollSheet::AddEntry()
{
  map<string, string> temp_map;
  temp_map["id"] = "new";
  temp_map["event_roll_id"] = initial_state["id"];

  TformRecordEventRollItem *roll_item_form = NULL;
  FORM_TRY
    roll_item_form = new TformRecordEventRollItem(NULL, temp_map);
  FORM_CATCH
  if(roll_item_form)
    roll_item_form->Show();
}
//---------------------------------------------------------------------------
void TformRecordEventRollSheet::EditEntry()
{
  map<string, string> temp_map;
  temp_map["id"] = roll_entry_filter.GetSelectedRecordID();

  TformRecordEventRollItem *roll_item_form = NULL;
  FORM_TRY
    roll_item_form = new TformRecordEventRollItem(NULL, temp_map);
  FORM_CATCH
  if(roll_item_form)
    roll_item_form->Show();
}
//---------------------------------------------------------------------------
void __fastcall TformRecordEventRollSheet::lblPrintRollSheetClick(TObject *Sender)
{
  string event_id = GetValidLinkId(txtEventName);

  if(event_id != "")
  {
    string group_id = GetOneField(dbDatabase, "select group_id from events where id = '" + event_id + "' ");
    TformLaunchRollSheetReport *roll_report = NULL;
    FORM_TRY
      roll_report = new TformLaunchRollSheetReport(NULL, event_id, group_id);
    FORM_CATCH
    if(roll_report)
      roll_report->Show();
  }
}
//---------------------------------------------------------------------------

