//---------------------------------------------------------------------------

#include <vcl.h>
#pragma hdrstop

#include "codeRecordFitnessMembership.h"
#include "codeRecordFitnessPerson.h"
#include "codeRecordTaskDiab.h"
//---------------------------------------------------------------------------
#pragma package(smart_init)
#pragma link "codeRecordGeneric"
#pragma link "codeHalfButton"
#pragma resource "*.dfm"
TformRecordFitnessMembership *formRecordFitnessMembership;
//---------------------------------------------------------------------------
__fastcall TformRecordFitnessMembership::TformRecordFitnessMembership(TComponent* Owner, map<string,string> p_parameters)
        : TformRecordGeneric(Owner, p_parameters, string("fitness_memberships"))
{
  people_filter.SetListViewHandle(listPeople);
  people_filter.SetSecurity("fitness_people");
  people_filter.AddColumnSpec("Name", "fitness_people", as_text, true, 120);
  people_filter.AddColumnSpec("Primary", "fitness_people", as_text, true, 80, prettyBoolString);
  people_filter.AddColumnSpec("Age", "people", as_int, true, 80);
  people_filter.AddColumnSpec("Doctor Name", "fitness_people", as_text, false, 100);
  people_filter.AddColumnSpec("Needs Release", "fitness_people", as_text, true, 100, prettyBoolString);
  people_filter.AddColumnSpec("Sent Letter", "fitness_people", as_text, false, 100, prettyBoolString);
  people_filter.AddColumnSpec("Current Id", "current_id", as_int, false, 80);
  people_filter.AddColumnSpec("Internal Id", "internal_id", as_int, false, 80);
  people_filter.FetchPreferences("recordfitnessmembership.people_filter");
  people_filter.SetIconStatus(true);
  people_filter.SetSQLStatement("");
  people_filter.RefreshDataSet();

  warning_filter.SetListViewHandle(listWarning);
  warning_filter.SetSecurity("tasks.diab");
  warning_filter.AddColumnSpec("Date", "tasks", as_date, true, 110);
  warning_filter.AddColumnSpec("Employee", "tasks", as_text, true, 100);
  warning_filter.AddColumnSpec("Current Id", "current_id", as_int, false, 80);
  warning_filter.AddColumnSpec("Internal Id", "internal_id", as_int, false, 80);
  warning_filter.FetchPreferences("recordfitnessmembership.warning_filter");
  warning_filter.SetIconStatus(true);
  warning_filter.SetSQLStatement("");
  warning_filter.RefreshDataSet();

  cancellation_filter.SetListViewHandle(listCancellation);
  cancellation_filter.SetSecurity("tasks.diab");
  cancellation_filter.AddColumnSpec("Date", "tasks", as_date, true, 110);
  cancellation_filter.AddColumnSpec("Employee", "tasks", as_text, true, 100);
  cancellation_filter.AddColumnSpec("Current Id", "current_id", as_int, false, 80);
  cancellation_filter.AddColumnSpec("Internal Id", "internal_id", as_int, false, 80);
  cancellation_filter.FetchPreferences("recordfitnessmembership.cancellation_filter");
  cancellation_filter.SetIconStatus(true);
  cancellation_filter.SetSQLStatement("");
  cancellation_filter.RefreshDataSet();

  Initialize();
}
//---------------------------------------------------------------------------
bool TformRecordFitnessMembership::ScreenStart()
{
  lookupTables->RegisterComboBox("fit_facility_types", cboFacility);
  editComments->Text = "";
  editBegin->Text = Now();
  TformRecordGeneric::ScreenStart();
  return true;
}
//---------------------------------------------------------------------------
bool TformRecordFitnessMembership::ScreenStop()
{
  lookupTables->UnregisterComboBox("fit_facility_types", cboFacility);

  TformRecordGeneric::ScreenStop();
  return true;
}
//---------------------------------------------------------------------------
bool TformRecordFitnessMembership::IsValidOnScreen()
{
  // make sure there is a link to a person, although there's not much the user
  // can do to fix that if there's not
  if (! lookupTables->IsInLookUpTable("fit_facility_types", cboFacility->Text.c_str()))
  {
    PSay("Please enter a valid facility", m_invalid_data, MB_OK);
    cboFacility->SetFocus();
    return false;
  }
  else if (! Pdate (editWaiting->Text.c_str(), mdy | yy).IsValidDate() && ! (editWaiting->Text == ""))
  {
    PSay("Please enter a valid date", m_invalid_data, MB_OK);
    editWaiting->SetFocus();
    return false;
  }
  else if (! Pdate (editBegin->Text.c_str(), mdy | yy).IsValidDate() && ! (editBegin->Text == ""))
  {
    PSay("Please enter a valid begin date", m_invalid_data, MB_OK);
    editBegin->SetFocus();
    return false;
  }
  else if (! Pdate (editCancel->Text.c_str(), mdy | yy).IsValidDate() && ! (editCancel->Text == ""))
  {
    PSay("Please enter a valid cancel date", m_invalid_data, MB_OK);
    editCancel->SetFocus();
    return false;
  }
  else
    return true;
}
//---------------------------------------------------------------------------

bool TformRecordFitnessMembership::PutDataOnScreen()
{
  // put the values on the form from the map initial_state
  // values may have come from GetDataFromDatabase if it's an existing record,
  // or values may have been passed to the form in this map if it's a new record
  if (initial_state["is_waiting"] == "1")
  {
    radioWaiting->Checked = true;
    panelMember->Enabled = false;
  }
  else
  {
    radioMember->Checked = true;
    panelWaiting->Enabled = false;
  }

//  radioClick(NULL);

  cboFacility->Text = PrettyString(lookupTables->ValueOf("fit_facility_types", initial_state["fit_facility_id"])).c_str();
  editBegin->Text = initial_state["begin_date"].c_str();
  editCancel->Text = initial_state["end_date"].c_str();
  editWaiting->Text = initial_state["waiting_list_date"].c_str();
  editComments->Text = SentenceString(initial_state["comments"]).c_str();
  editMemberNumber->Text = initial_state["membership_number"].c_str();
  TformRecordGeneric::PutDataOnScreen();
  return true; // what can really go wrong?
}
//---------------------------------------------------------------------------
bool TformRecordFitnessMembership::GetDataFromScreen()
{
  // place the values currently on the screen into
  // the map that will be used to update or insert the record
  // the parent form will call the insert or update function, and get the id
  // number if necessary and take care of current_id
  ending_state["fit_facility_id"] = lookupTables->IdOf("fit_facility_types", cboFacility->Text.c_str()).c_str();

  if (radioWaiting->Checked)
  {
    ending_state["is_waiting"] = "1";
    if (editWaiting->Text != "")
      ending_state["waiting_list_date"] =  Pdate (editWaiting->Text.c_str(), mdy | yyyy).OutDate(mdy | yyyy);
    else
      ending_state["waiting_list_date"] = "";
    ending_state["begin_date"] = "";
    ending_state["end_date"] = "";
  }
  else if(radioMember->Checked)
  {
    ending_state["is_waiting"] = "0";
    ending_state["waiting_list_date"] = "";
    if (editBegin->Text != "")
      ending_state["begin_date"] =  Pdate (editBegin->Text.c_str(), mdy | yyyy).OutDate(mdy | yyyy);
    else
      ending_state["begin_date"] = "";
    if (editCancel->Text != "")
      ending_state["end_date"] =  Pdate (editCancel->Text.c_str(), mdy | yyyy).OutDate(mdy | yyyy);
    else
      ending_state["end_date"] = "";
    ending_state["membership_number"] = editMemberNumber->Text.c_str();
  }
  ending_state["comments"] = editComments->Text.c_str();
  ending_state["id"] = initial_state["id"];

  TformRecordGeneric::GetDataFromScreen();

  return true; // again, not much can go wrong here
}
//---------------------------------------------------------------------------
bool TformRecordFitnessMembership::ScreenLock()
{
  // disable panel containing edit boxes
  panelGroup->Enabled = false;
  editComments->ReadOnly = true;
  editComments->TabStop = false;
  TformRecordGeneric::ScreenLock();
  return true;
}
//---------------------------------------------------------------------------
bool TformRecordFitnessMembership::ScreenUnlock()
{
  // enable panel containing edit boxes
  panelGroup->Enabled = true;
  editComments->ReadOnly = false;
  editComments->TabStop = true;
  TformRecordGeneric::ScreenUnlock();
  return true;
}
//---------------------------------------------------------------------------
bool TformRecordFitnessMembership::IsNewAgain()
{
 people_filter.SetSQLStatement(string(""));
 people_filter.RefreshDataSet();
 people_filter.SetAddEdit(this, NULL, NULL);
 TformRecordGeneric::IsNewAgain();
 return true;
}
//---------------------------------------------------------------------------
bool TformRecordFitnessMembership::NotNewAnymore()
{
  string sql;

  sql = "select person_names.first_name || ' ' || person_names.last_name || ' ' || person_names.suffix, "
        "fitness_people.is_primary, (select p_age from calc_age(people.birth_date)), "
        "fitness_people.doctor_name, fitness_people.needs_release, "
        "fitness_people.sent_letter, fitness_people.current_id, fitness_people.id "
        "from fitness_people, person_names, people "
        "where fitness_people.fitness_membership_id = " + initial_state["id"] + " and "
        "fitness_people.person_id = people.id and "
        "person_names.id = people.person_names_id";

  people_filter.SetSQLStatement(sql);
  people_filter.RefreshDataSet();
  people_filter.SetAddEdit(this, AddRecord(&(TformRecordFitnessMembership::AddPerson)),
                          EditRecord(&(TformRecordFitnessMembership::EditPerson)));
  people_request = atlas->RequestNotificationWhen("fitness_people.fitness_membership_id", initial_state["id"],
                  rel_e, as_int, this, RefreshHandler(&(TformRecordFitnessMembership::RefreshListView)));

  // jhaley: pop up a new fitness_people record screen automatically on save of a new fitness membership
  if (people_filter.GetAllData(false, false).empty() && optionsPermissions["table.fitness_people"]["insert"] == "true")
    AddPerson();

  sql = "select tasks.date_of_task, users.item, tasks.current_id, tasks.id "
        "from tasks, users "
        "where tasks.user_id = users.id and "
        "tasks.fitness_membership_id = " + initial_state["id"] + " and "
        "tasks.diab_activity_id = " + lookupTables->IdOf("diab_activity_types", "Attendance warning letter");
  warning_filter.SetSQLStatement(sql);
  warning_filter.RefreshDataSet();
  warning_filter.SetAddEdit(this, AddRecord(&(TformRecordFitnessMembership::AddWarning)),
                          EditRecord(&(TformRecordFitnessMembership::EditWarning)));
  warning_request = atlas->RequestNotificationWhen("tasks.fitness_membership_id", initial_state["id"],
                  rel_e, as_int, this, RefreshHandler(&(TformRecordFitnessMembership::RefreshListView)));

  sql = "select tasks.date_of_task, users.item, tasks.current_id, tasks.id "
        "from tasks, users "
        "where tasks.user_id = users.id and "
        "tasks.fitness_membership_id = " + initial_state["id"] + " and "
        "tasks.diab_activity_id = " + lookupTables->IdOf("diab_activity_types", "Fitness cancellation letter");
  cancellation_filter.SetSQLStatement(sql);
  cancellation_filter.RefreshDataSet();
  cancellation_filter.SetAddEdit(this, AddRecord(&(TformRecordFitnessMembership::AddWarning)),
                          EditRecord(&(TformRecordFitnessMembership::EditWarning)));
  cancellation_request = atlas->RequestNotificationWhen("tasks.fitness_membership_id", initial_state["id"],
                  rel_e, as_int, this, RefreshHandler(&(TformRecordFitnessMembership::RefreshListView)));

  TformRecordGeneric::NotNewAnymore();

  return true;
}
//---------------------------------------------------------------------------
bool TformRecordFitnessMembership::RefreshListView(int request_num, string matched_value)
{
  if (request_num == people_request)
    people_filter.RefreshDataSet();
  if (request_num == warning_request)
    warning_filter.RefreshDataSet();
  if (request_num == cancellation_request)
    cancellation_filter.RefreshDataSet();
  return true;
}
//---------------------------------------------------------------------------
void TformRecordFitnessMembership::AddPerson()
{
  map<string,string> temp_map;
  temp_map["id"] = "new";
  temp_map["fitness_membership_id"] = initial_state["id"];
  FORM_TRY
    TformRecordFitnessPerson * personRecord = new TformRecordFitnessPerson(NULL, temp_map);
    personRecord->Show();
  FORM_CATCH
}
//---------------------------------------------------------------------------
void TformRecordFitnessMembership::EditPerson()
{
  map<string,string> temp_map;
  temp_map["id"] = people_filter.GetSelectedRecordID();
  FORM_TRY
    TformRecordFitnessPerson * personRecord = new TformRecordFitnessPerson(NULL, temp_map);
    personRecord->Show();
  FORM_CATCH
}
//---------------------------------------------------------------------------
void __fastcall TformRecordFitnessMembership::radioClick(TObject *Sender)
{
  panelWaiting->Enabled = true;
  panelMember->Enabled = true;
  if(radioWaiting->Checked)
    panelMember->Enabled = false;
  else if(radioMember->Checked)
    panelWaiting->Enabled = false;
}
//---------------------------------------------------------------------------
void TformRecordFitnessMembership::AddWarning()
{
  map<string,string> temp_map;
  temp_map["id"] = "new";
  temp_map["fitness_membership_id"] = initial_state["id"];
  temp_map["group_id"] = lookupTables->IdOf("groups", "Diabetes");
  temp_map["method_id"] = lookupTables->IdOf("method_types", "Mail");
  temp_map["direction_id"] = lookupTables->IdOf("direction_types", "Outgoing");
  temp_map["task_comments"] = "fitness attendance warning letter";
  temp_map["diab_activity_id"] = lookupTables->IdOf("diab_activity_types", "Attendance warning letter");
  temp_map["date_of_task"] = CurrentDate();
  temp_map["user_id"] = current_userid;
  temp_map["group_id"] = lookupTables->IdOf("groups", "Diabetes");
  FORM_TRY
    TformRecordTaskDiab * warningRecord = new TformRecordTaskDiab(NULL, temp_map);
    warningRecord->Show();
  FORM_CATCH
}
//---------------------------------------------------------------------------
void TformRecordFitnessMembership::EditWarning()
{
  map<string,string> temp_map;
  temp_map["id"] = warning_filter.GetSelectedRecordID();
  FORM_TRY
    TformRecordTaskDiab * warningRecord = new TformRecordTaskDiab(NULL, temp_map);
    warningRecord->Show();
  FORM_CATCH
}
//---------------------------------------------------------------------------
void TformRecordFitnessMembership::AddCancellation()
{
  map<string,string> temp_map;
  temp_map["id"] = "new";
  temp_map["fitness_membership_id"] = initial_state["id"];
  temp_map["group_id"] = lookupTables->IdOf("groups", "Diabetes");
  temp_map["method_id"] = lookupTables->IdOf("method_types", "Mail");
  temp_map["direction_id"] = lookupTables->IdOf("direction_types", "Outgoing");
  temp_map["task_comments"] = "fitness membership cancellation letter";
  temp_map["diab_activity_id"] = lookupTables->IdOf("diab_activity_types", "Fitness cancellation letter");
  temp_map["date_of_task"] = CurrentDate();
  temp_map["user_id"] = current_userid;
  temp_map["group_id"] = lookupTables->IdOf("groups", "Diabetes");
  FORM_TRY
    TformRecordTaskDiab * warningRecord = new TformRecordTaskDiab(NULL, temp_map);
    warningRecord->Show();
  FORM_CATCH
}
//---------------------------------------------------------------------------
void TformRecordFitnessMembership::EditCancellation()
{
  map<string,string> temp_map;
  temp_map["id"] = cancellation_filter.GetSelectedRecordID();
  FORM_TRY
    TformRecordTaskDiab * warningRecord = new TformRecordTaskDiab(NULL, temp_map);
    warningRecord->Show();
  FORM_CATCH
}
//---------------------------------------------------------------------------
void __fastcall TformRecordFitnessMembership::buttonGetNextMembershipNumberClick(
      TObject *Sender)
{
 if (editMemberNumber->Text != "")
  if (IDNO == PSay("This record already has a member number; do you want to overwrite it?", m_confirmation, MB_YESNO))
   return;

 editMemberNumber->Text = GetNextId(dbDatabase, "membership_auto_num").c_str(); // membership_auto_num_gen
}
//---------------------------------------------------------------------------

