//---------------------------------------------------------------------------

#include <vcl.h>
#pragma hdrstop

#include "codeRecordFitnessPerson.h"
#include "codeSearchPerson.h"
#include "codeRecordFitnessAttendance.h"
//---------------------------------------------------------------------------
#pragma package(smart_init)
#pragma link "codeRecordGeneric"
#pragma resource "*.dfm"
TformRecordFitnessPerson *formRecordFitnessPerson;
//---------------------------------------------------------------------------
__fastcall TformRecordFitnessPerson::TformRecordFitnessPerson(TComponent* Owner, map<string,string> p_parameters)
        : TformRecordGeneric(Owner, p_parameters, string("fitness_people"))
{
  fitness_attendance_filter.SetListViewHandle(listAttendance);
  fitness_attendance_filter.SetSecurity("fitness_attendance");
  fitness_attendance_filter.AddColumnSpec("Date", "fitness_attendance", as_date, true, 130);
  fitness_attendance_filter.AddColumnSpec("Current Id", "current_id", as_int, false, 80);
  fitness_attendance_filter.AddColumnSpec("Internal Id", "internal_id", as_int, false, 80);
  fitness_attendance_filter.FetchPreferences("recordfitnessperson.attendance_filter");
  fitness_attendance_filter.SetIconStatus(true);
  fitness_attendance_filter.SetSQLStatement("");
  fitness_attendance_filter.RefreshDataSet();

//  initial_state["person_id"] = GetOneField(dbDatabase, "select person_id from fitness_people where id = " + initial_state["id"]);
//  initial_state["fitness_membership_id"] = GetOneField(dbDatabase, "select fitness_membership_id from fitness_people where id = " + initial_state["id"]);   

  Initialize();
}
//---------------------------------------------------------------------------
bool TformRecordFitnessPerson::ScreenStart()
{
  TformRecordGeneric::ScreenStart();
  return true;
}
//---------------------------------------------------------------------------
bool TformRecordFitnessPerson::ScreenStop()
{
  TformRecordGeneric::ScreenStop();
  return true;
}
//---------------------------------------------------------------------------
bool TformRecordFitnessPerson::IsValidOnScreen()
{
  // make sure there is a link to a person, although there's not much the user
  // can do to fix that if there's not
  if (GetValidLinkId(txtFacility) == "")
  {
    PSay("There is no fitness membership associated with this record", m_missing_data, MB_OK);
    return false;
  }
  else if (GetValidLinkId(txtPerson) == "")
  {
    PSay("There is no person associated with this record", m_missing_data, MB_OK);
    return false;
  }
  else
    return true;
}
//---------------------------------------------------------------------------

bool TformRecordFitnessPerson::PutDataOnScreen()
{
  // put the values on the form from the map initial_state
  // values may have come from GetDataFromDatabase if it's an existing record,
  // or values may have been passed to the form in this map if it's a new record
  editDoctor->Text = PrettyString(initial_state["doctor_name"]).c_str();
  if (initial_state["is_primary"] == "1")
    checkPrimary->Checked = true;
  else
    checkPrimary->Checked = false;
  if (initial_state["needs_release"] == "1")
    checkNeedsRelease->Checked = true;
  else
    checkNeedsRelease->Checked = false;
  if (initial_state["sent_letter"] == "1")
    checkSentLetter->Checked = true;
  else
    checkSentLetter->Checked = false;


  // get the person name associated with the person id that was either selected
  // from the database if it's an existing record, or passed to the form if
  // it's a new record. display the name as a link to the person record.
  CreateLink(txtPerson, btnPersonSearch, this, initial_state["person_id"], "people");
  CreateLink(txtFacility, NULL, this, initial_state["fitness_membership_id"], "fitness_memberships");

  TformRecordGeneric::PutDataOnScreen();
  return true; // what can really go wrong?
}
//---------------------------------------------------------------------------
bool TformRecordFitnessPerson::GetDataFromScreen()
{
  // place the values currently on the screen into
  // the map that will be used to update or insert the record
  // the parent form will call the insert or update function, and get the id
  // number if necessary and take care of current_id
  ending_state["doctor_name"] = editDoctor->Text.c_str();
  if (checkPrimary->Checked)
    ending_state["is_primary"] = "1";
  else
    ending_state["is_primary"] = "0";
  if (checkNeedsRelease->Checked)
    ending_state["needs_release"] = "1";
  else
    ending_state["needs_release"] = "0";
  if (checkSentLetter->Checked)
    ending_state["sent_letter"] = "1";
  else
    ending_state["sent_letter"] = "0";

  ending_state["fitness_membership_id"] = GetValidLinkId(txtFacility);
  ending_state["person_id"] = GetValidLinkId(txtPerson);
  ending_state["id"] = initial_state["id"];

  TformRecordGeneric::GetDataFromScreen();

  return true; // again, not much can go wrong here
}
//---------------------------------------------------------------------------
bool TformRecordFitnessPerson::ScreenLock()
{
  // disable panel containing edit boxes
  panelGroup->Enabled = false;
  TformRecordGeneric::ScreenLock();
  return true;
}
//---------------------------------------------------------------------------
bool TformRecordFitnessPerson::ScreenUnlock()
{
  // enable panel containing edit boxes
  panelGroup->Enabled = true;
  TformRecordGeneric::ScreenUnlock();
  return true;
}
//---------------------------------------------------------------------------
bool TformRecordFitnessPerson::CheckForUselessRecord()
{
  return true;
}
//---------------------------------------------------------------------------
void __fastcall TformRecordFitnessPerson::btnPersonSearchClick(
      TObject *Sender)
{
  // create and show new person search form
  vector<string> return_vector;
  FORM_TRY
    TformSearchPerson * formPersonSearch = new TformSearchPerson(NULL,&return_vector);
    formPersonSearch->ShowTaskModal(this);
  FORM_CATCH
  if (! return_vector.empty())
    CreateLink(txtPerson, btnPersonSearch, this, return_vector[0], "people");
}
//---------------------------------------------------------------------------
bool TformRecordFitnessPerson::NotNewAnymore()
{
  if (optionsPermissions["table.fitness_attendance"]["view"] == "true")
  {
    string sql = "select fitness_attendance.date_of_attendance, "
          "fitness_attendance.current_id, fitness_attendance.id "
          "from fitness_attendance "
          "where fitness_attendance.person_id = " + initial_state["person_id"] + " and "
          "fitness_attendance.fitness_membership_id = " + initial_state["fitness_membership_id"];
    fitness_attendance_filter.SetSQLStatement(sql);
    fitness_attendance_filter.RefreshDataSet();
    fitness_attendance_filter.SetAddEdit(this, AddRecord(&(TformRecordFitnessPerson::AddAttendance)),
                               EditRecord(&(TformRecordFitnessPerson::EditAttendance)));
    fitness_attendance_request = atlas->RequestNotificationWhen("fitness_attendance.person_id", initial_state["person_id"], rel_e, as_int, this,
                       RefreshHandler(&(TformRecordFitnessPerson::AtlasRefreshListView)));
  }
  TformRecordGeneric::NotNewAnymore();
  return true;
}
//---------------------------------------------------------------------------
bool TformRecordFitnessPerson::IsNewAgain()
{
  fitness_attendance_filter.SetSQLStatement(string(""));
  fitness_attendance_filter.RefreshDataSet();
  fitness_attendance_filter.SetAddEdit(this, NULL, NULL);
  TformRecordGeneric::IsNewAgain();
  return true;
}
//---------------------------------------------------------------------------
void TformRecordFitnessPerson::AtlasRefreshListView(int request_num, string matched_value)
{
  if(request_num == fitness_attendance_request)
    fitness_attendance_filter.RefreshDataSet();
}
//---------------------------------------------------------------------------
void TformRecordFitnessPerson::AddAttendance()
{
  if(FksAreSet())
  {
    map<string, string> temp_map;
    temp_map["id"] = "new";
    temp_map["person_id"] = GetValidLinkId(txtPerson);
    temp_map["fitness_membership_id"] = GetValidLinkId(txtFacility);
    FORM_TRY
      TformRecordFitnessAttendance *attendance_form = new TformRecordFitnessAttendance(NULL, temp_map);
      attendance_form->Show();
    FORM_CATCH
  }
}
//---------------------------------------------------------------------------
void TformRecordFitnessPerson::EditAttendance()
{
  map<string, string> temp_map;
  temp_map["id"] = fitness_attendance_filter.GetSelectedRecordID();
  FORM_TRY
    TformRecordFitnessAttendance *attendance_form = new TformRecordFitnessAttendance(NULL, temp_map);
    attendance_form->Show();
  FORM_CATCH
}
//---------------------------------------------------------------------------
bool TformRecordFitnessPerson::FksAreSet()
{
//  if((initial_state["person_id"] != "") && (initial_state["person_id"] != "0") &&
//    (initial_state["fitness_membership_id"] != "") && (initial_state["fitness_membership_id"] != "0"))
    return true;
//  else
//    return false;
}
//---------------------------------------------------------------------------
bool TformRecordFitnessPerson::AskUser()
{
  if (!IsBlankOrZero(initial_state["fitness_membership_id"]) && IsInt(initial_state["fitness_membership_id"]))
  {
    string sql = "select count(FP.id) from fitness_people FP where FP.fitness_membership_id = '" + initial_state["fitness_membership_id"] + "' ";
    int person_count = StringToInt(GetOneField(dbDatabase, sql));

    if (person_count <= 0)
    {
      int return_value = PSay("There are no people attached to the parent membership.\n"
                              "If you do not add a person, you may have trouble finding it later.\n"
                              "Click 'Yes' to cancel and then add a person, or 'No' to exit.",
                              m_confirmation, MB_YESNO);
      if (return_value == IDYES)
        return true;
      else
        return false;
    }
  }
  return TformRecordGeneric::AskUser();
}

