//---------------------------------------------------------------------------

#include <vcl.h>
#pragma hdrstop

#include "codeRecordForm.h"
//---------------------------------------------------------------------------
#pragma package(smart_init)
#pragma link "codeRecordGeneric"
#pragma resource "*.dfm"
TformRecordForm *formRecordForm;
//---------------------------------------------------------------------------
__fastcall TformRecordForm::TformRecordForm(TComponent* Owner, map<string,string> p_parameters)
        : TformRecordGeneric(Owner, p_parameters, string("forms"))
{
 Initialize();
}
//---------------------------------------------------------------------------
bool TformRecordForm::ScreenStart()
{
  string group_sql = "select id, item, valid from groups where id in ("+SetToCommaString(GetGroupPermissions("table.forms", "", "view", "true"))+") and valid = 1 order by item";
  lookupTables->RegisterComboBox("groups", cboDepartment, group_sql);

  // we'll be nice and fill in the clinic name for them
  if (lookupTables->IsValidId("groups", current_groupid, cboDepartment))
  {
   cboDepartment->Text = lookupTables->ValueOf("groups", current_groupid).c_str();
   this->ActiveControl = cboType/*->SetFocus()*/;
  }
  cboDepartmentChange(NULL);

  TformRecordGeneric::ScreenStart();
  return true;
}
//---------------------------------------------------------------------------
bool TformRecordForm::ScreenStop()
{
  lookupTables->UnregisterComboBox("groups", cboDepartment);
  lookupTables->UnregisterComboBox("form_types", cboType);
  TformRecordGeneric::ScreenStop();
  return true;
}
//---------------------------------------------------------------------------
bool TformRecordForm::IsValidOnScreen()
{
  // check to make sure the date entered is valid
  // give the user a message box and prompt to re-enter if incorrect
  // make sure there is a link to a person, although there's not much the user
  // can do to fix that if there's not
  if (cboDepartment->Text == "" || !lookupTables->IsInLookUpTable("groups", cboDepartment))
  {
    PSay("Please select a valid department", m_invalid_data, MB_OK);
    cboType->SetFocus();
    return false;
  }
  if (cboType->Text == "" || !lookupTables->IsInLookUpTable("form_types", cboType))
  {
    PSay("Please enter a valid type of form", m_invalid_data, MB_OK);
    cboType->SetFocus();
    return false;
  }
  else if ((initial_state["person_id"] == "") || (initial_state["person_id"] == "0"))
  {
    PSay("There is no person associated with this record", m_invalid_data, MB_OK);
    return false;
  }
  else
    return true;
}
//---------------------------------------------------------------------------
bool TformRecordForm::PutDataOnScreen()
{
  // put the values on the form from the map initial_state
  // values may have come from GetDataFromDatabase if it's an existing record,
  // or values may have been passed to the form in this map if it's a new record
  if (initial_state["date_of_completion"] != "")
    dateSignature->Date = Pdate(initial_state["date_of_completion"], mdy | yyyy).OutDate();
  else
    dateSignature->Date = Date();
  cboDepartment->Text = PrettyString(lookupTables->ValueOf("groups", initial_state["group_id"])).c_str();
  cboDepartmentChange(NULL);
  cboType->Text = PrettyString(lookupTables->ValueOf("form_types", initial_state["form_id"])).c_str();
  editNote->Text = SentenceString(initial_state["note"]).c_str();

  // get the person name associated with the person id that was either selected
  // from the database if it's an existing record, or passed to the form if
  // it's a new record. display the name as a link to the person record.
  CreateLink(txtPerson, NULL, this, initial_state["person_id"], "people");

  TformRecordGeneric::PutDataOnScreen();
  return true; // what can really go wrong?
}
//---------------------------------------------------------------------------
bool TformRecordForm::GetDataFromScreen()
{
  // place the values currently on the screen into
  // the map that will be used to update or insert the record
  // the parent form will call the insert or update function, and get the id
  // number if necessary and take care of current_id
  ending_state["date_of_completion"] = Pdate(dateSignature->Date).OutDate(mdy | yyyy);
  ending_state["form_id"] = lookupTables->IdOf("form_types", cboType->Text.c_str());
  ending_state["note"] = editNote->Text.c_str();
  ending_state["person_id"] = initial_state["person_id"];
  ending_state["group_id"] = lookupTables->IdOf("groups", cboDepartment->Text.c_str());
  ending_state["id"] = initial_state["id"];

  TformRecordGeneric::GetDataFromScreen();

  return true; // again, not much can go wrong here
}
//---------------------------------------------------------------------------
bool TformRecordForm::ScreenLock()
{
  // disable panel containing edit boxes
  panelGroup->Enabled = false;
  editNote->ReadOnly = true;
  editNote->TabStop = false;
  TformRecordGeneric::ScreenLock();
  return true;
}
//---------------------------------------------------------------------------
bool TformRecordForm::ScreenUnlock()
{
  // enable panel containing edit boxes
  panelGroup->Enabled = true;
  editNote->ReadOnly = false;
  editNote->TabStop = true;
  TformRecordGeneric::ScreenUnlock();
  return true;
}
//---------------------------------------------------------------------------
void __fastcall TformRecordForm::cboDepartmentChange(TObject *Sender)
{
  if (lookupTables->TableName(cboType) != "") // if this cbo box is registered
    lookupTables->UnregisterComboBox("form_types", cboType);
  if (cboDepartment->Text != "" && lookupTables->IsInLookUpTable("groups", cboDepartment))
  {
    string type_sql = "select id, item, valid from form_types where group_id = '" + lookupTables->IdOf("groups", cboDepartment) + "' and valid = 1 order by item";
    lookupTables->RegisterComboBox("form_types", cboType, type_sql, SetGroupIdForNewLookupTableItem);
  }
}
//---------------------------------------------------------------------------
void TformRecordForm::Do(map<string, string> & p_parameters)
{
 FORM_TRY
  TformRecordForm * z = new TformRecordForm(NULL, p_parameters);
  z->Show();
 FORM_CATCH
}
//---------------------------------------------------------------------------

