//---------------------------------------------------------------------------

#include <vcl.h>
#pragma hdrstop
#pragma warn -8057

#include "codeRecordGeneric.h"
#include "codeSpecialViewChangeLog.h"
#include "codeSpecialDataNavigate.h"
#include "codeSpecialAction.h"
// extern list<TformRecordGeneric*>  generic_records;
//---------------------------------------------------------------------------
#pragma package(smart_init)
#pragma link "codeSpecialGeneric"
#pragma resource "*.dfm"
//---------------------------------------------------------------------------
__fastcall TformRecordGeneric::TformRecordGeneric(TComponent* Owner, map<string,string> & p_parameters, string p_table_name)
        : TformSpecialGeneric(Owner)
{
 popupRecord->Images = imagesRecordTypes;
 ImageList = imagesCurrent;
 IconIndex = -1;

 can_view = can_view_unknown;
 can_insert = can_insert_unknown;
 can_update = can_update_unknown;
 initial_state = p_parameters;
 table_name = p_table_name;
 // title_name = this->Caption.c_str();
 Title = this->Caption;
 toolsStandard->Tag = int(this);
 lock_transaction = NULL;
 is_editing = false;
 record_useless_override = false;
 // generic_records.push_back(this);  // let prometheus know how to call CloseAsk() ...
 Screen->Cursor = crHourGlass;
 menuDeleteRecord->Caption = ("Delete " + table_name + " record").c_str();
}
//---------------------------------------------------------------------------
void __fastcall TformRecordGeneric::buttonBClick(TObject *Sender)
{
 // close / undo button
 if (is_editing)
  DataUndo();
 else
  Close();
  // whether not doing anything or making a new record, close.
  // no transaction has been started yet for "new" records.
}
//---------------------------------------------------------------------------
void __fastcall TformRecordGeneric::buttonAClick(TObject *Sender)
{
 // unlock / save button
 toolsStandard->SetFocus();
 if (!is_editing && !is_new_record)
  DataUnlock();
 else if ((is_editing && _CanUpdate()) ||
          (is_new_record && _CanInsert()))
  DataSave(); // knows the difference between insert and update
 else
  PSay("You do not have permissions to either insert or update this type of record (" + table_name + ").", m_message, MB_OK);
}
//---------------------------------------------------------------------------
void __fastcall TformRecordGeneric::FormClose(TObject *Sender,
      TCloseAction &Action)                                      //form close
{
 // no longer need to receive CloseAsk() ... messages
 // generic_records.remove(this);

 // by default, hitting the X button will ask about changes? can be changed to undo (for ashley)
 // --
 // this can also happen from CloseAsk, CloseUndo and CloseSave, which will do Close()
 // but not ScreenStop()... also, they will have called one of DataSave and DataUndo,
 // but this DataClose() is here JUST IN CASE... they hit the X button. Which they will. A lot.

 if (!DataClose())
 {
  // ABORT!
  Action = caNone;
  return;
 }

 ScreenStop();

 // this wasn't being done! memory leak! gah!
 if (lock_transaction)
 {
  if (lock_transaction->Active)
   lock_transaction->Rollback();
  delete lock_transaction;
 }

 // get scheduled to free memory, always
 Action = caFree;
 TformSpecialGeneric::FormClose(Sender, Action);
}
//---------------------------------------------------------------------------
bool TformRecordGeneric::ScreenStart()         // atlas, getdatafromdatabase, putdataonscreen, listviews, ...
{
 if (_CanView())
 {
  GetFieldSetFromTable(dbDatabase, table_name, db_table_fields); // jhaley 20110107: get RDB$RELATION_FIELDS mapping
  if (!is_new_record)
   if (!GetDataFromDatabase())
    return false;
  if (!PutDataOnScreen())
   return false;
 }
 return true;
}
//---------------------------------------------------------------------------
bool TformRecordGeneric::ScreenStop()          // atlas, clear listviews, ...
{
 // cancel all atlas requests
 atlas->CancelRequestFrom(this);
 UnregisterLinks(this);
 return true;
}
//---------------------------------------------------------------------------
bool TformRecordGeneric::GetDataFromDatabase() // database to initial_state
{
  // this function should only be called if we are viewing an existing
  // record... not creating a new one

  // WARNING: DOES NOT USE LOCK_TRANSACTION, THIS TRANSACTION NOT OTHERWISE AVAILABLE FOR PUTDATAONSCREEN
  // -- however, PutDataOnScreen isn't always called right after GetDataFromDatabase, ... we would need
  // a new global transaction like view_transaction, set to read_committed ... maybe that would help?
  // do all the reads on the same transaction, at least? but even so, not particularly good about non-repeatable reads ...

  string sql = "select * from " + table_name + " where id = " + initial_state["id"];
  bool successful;
  string field_name;
  TIBTransaction * dbTransaction = new TIBTransaction(NULL);

  try
  {
    StdTransaction(dbTransaction, dbDatabase);
    initial_state.clear();
    SqlToMap(dbTransaction, sql, initial_state);
    dbTransaction->Commit();
    successful = true;
  }
  catch(...)
  {
    if (dbTransaction->Active)
      dbTransaction->Rollback();
    successful = false;
  }
  delete dbTransaction;
  return successful;
}
//---------------------------------------------------------------------------
bool TformRecordGeneric::PutDataOnScreen()     // initial_state to screen
{
 // current/username/date
 UpdateCurrent();
 return true;
}
//---------------------------------------------------------------------------
bool TformRecordGeneric::GetDataFromScreen()   // screen to ending_state
{
  string current_time = RightNow(); // jhaley: call once, for guaranteed consistency

  // current_id might have been set by a call to PopupMessage, which leaves the
  // new value in ending_state... otherwise, for "new" forms, ending_state has
  // not yet received a value for this, but initial_state has a default value
  if (ending_state["current_id"] == "")
    ending_state["current_id"] = initial_state["current_id"];
  ending_state["last_update_stamp"] = current_time;
  ending_state["last_updated_by"] = current_userid;

 // jhaley 20110107: created_by/creation_stamp fields - these are only added when inserting a new record, and only if
 // the table *has* those fields, and they haven't been set to something valid already.
 if (is_new_record)
 {
   if (FieldSetHasField(db_table_fields, "created_by") && ending_state["created_by"] == "")
     ending_state["created_by"] = current_userid;
   if (FieldSetHasField(db_table_fields, "creation_stamp") && ending_state["creation_stamp"] == "")
     ending_state["creation_stamp"] = current_time;
 }

 return true;
}
//---------------------------------------------------------------------------
bool TformRecordGeneric::IsValidOnScreen()     // screen validation
{
 return true;
}
//---------------------------------------------------------------------------
bool TformRecordGeneric::GetLockFromDatabase() // lock record
{
 is_database_started = false;
 GenericTransaction();

 if (_CanUpdate())
 {
  if (is_database_started)
  {
   lock_transaction->StartTransaction();
   if (LockRecordForUpdate (lock_transaction, table_name, initial_state["id"]))
    return true;
   else
    lock_transaction->Rollback();
  }
  else
   PSay("No transaction object available (lock) - GenericTransaction() must have failed.", m_program_error, MB_OK);
 }
 else
  PSay("You do not have permission to update records of this type (" + table_name + "). Contact your systems administrator (or boss) to get these permissions.", m_message, MB_OK);
 return false;
}
//---------------------------------------------------------------------------
bool TformRecordGeneric::PutDataInDatabase()   // is_new_record: ending_state -> insert -> database (lock)
                                               // is_editing: ending_state -> update -> database (lock)
{
 try
 {
  if (is_editing)
  {
   // perform update based on table and ending_state["id"]
   if (_CanUpdate())
   {
    map<string, int> leave_alone;
    for (set<string>::iterator itr=fields_to_leave_raw.begin(); itr!=fields_to_leave_raw.end(); itr++)
     leave_alone[*itr] = sql_no_uppercasing;
    if (ExecuteUpdateStatement(lock_transaction, table_name, ending_state, leave_alone))
    {
     if (BeforeSave())
     {
      try
      {
       if (record_useless_override)
       {
        PPretendToSay("Warning about useless/blank records overriden: record " + ending_state["id"] + " in table " + table_name, m_debug, ending_state["id"]);
        record_useless_override = false;
       }
       lock_transaction->Commit();
       return true;
      }
      catch (...)
      {
       Screen->Cursor = crDefault;
       PSay("Could not commit change (PutDataInDatabase)", m_database_error, MB_OK, "Update");
      }
     }
     else
     {
      Screen->Cursor = crDefault;
      // PSay("Certain additional operations to be performed after saving your changes could not performed, causing all of the changes to be undone together.", m_program_error, MB_OK, "Update");
     }
    }
    else
    {
     Screen->Cursor = crDefault;
     PSay("Could not update record (PutDataInDatabase): statement failed", m_database_error, MB_OK, "ExecuteUpdateStatement failed");
    }
   }
   else
   {
    Screen->Cursor = crDefault;
    PSay("You do not have permission to update records of this type (" + table_name + "). Contact your systems administrator (or boss) to get these permissions.", m_message, MB_OK, "Update");
   }
  }
  else if (is_new_record)
   // perform insert based on table, get the new id first and store it
   if (_CanInsert())
    if (GenericTransaction())
    {
     lock_transaction->StartTransaction();
     ending_state["id"] = "nonyet";
     ending_state["id"] = GetNextId (lock_transaction, table_name);
     if (IsInt(ending_state["id"]))
     {
      map<string, int> leave_alone;
      for (set<string>::iterator itr=fields_to_leave_raw.begin(); itr!=fields_to_leave_raw.end(); itr++)
       leave_alone[*itr] = sql_no_uppercasing;
      if (ExecuteInsertStatement(lock_transaction, table_name, ending_state, leave_alone))
      {
       if (BeforeSave())
       {
        try
        {
         if (record_useless_override)
         {
          PPretendToSay("Warning about useless/blank records overriden: record " + ending_state["id"] + " in table " + table_name, m_debug, ending_state["id"]);
          record_useless_override = false;
         }
         lock_transaction->Commit();
         return true;
        }
        catch (...)
        {
         Screen->Cursor = crDefault;
         PSay("Could not commit new record (PutDataInDatabase)", m_database_error, MB_OK, "Insert");
        }
       }
       else
       {
        Screen->Cursor = crDefault;
        // PSay("Certain additional operations to be performed after saving your changes could not performed, causing all of the changes to be undone together.", m_program_error, MB_OK, "Insert");
       }
      }
      else
      {
       Screen->Cursor = crDefault;
       // not necessarily accurate, could be an actual error during insert. if so, damn. but the old error was just too imprecise.
       PSay("Could not insert this record -- lock conflict.\nMost likely you or someone else is editing a record this one depends on.\nYou probably started to edit a record, then decided to add a new entry to it, but didn't save/undo your previous changes first.\nIf not, try asking around the office.\nYou may save/undo the record causing a problem, then save this one; the problem should go away.\nIf nothing fixes the problem, ask a programmer for help.", m_program_error, MB_OK);
      }
     }
     else
     {
      Screen->Cursor = crDefault;
      PSay("Could not get new id (PutDataInDatabase)", m_database_error, MB_OK, "Insert");
     }
     lock_transaction->Rollback();
    }
    else
    {
     Screen->Cursor = crDefault;
     PSay("Could not start transaction (PutDataInDatabase)", m_database_error, MB_OK, "Insert");
    }
   else
   {
    Screen->Cursor = crDefault;
    PSay("You do not have permission to insert records of this type (" + table_name + "). Contact your systems administrator (or boss) to get these permissions.", m_message, MB_OK, "Insert");
   }
  else
  {
   Screen->Cursor = crDefault;
   PSay("Neither inserting nor updating: no good reason to be here (PutDataInDatabase)", m_program_error, MB_OK);
  }
 }
 catch (EIBInterBaseError & err)
 {
  Screen->Cursor = crDefault;
  PSay(err.Message.c_str(), m_database_error, MB_OK);
 }
 catch (EIBError & err)
 {
  Screen->Cursor = crDefault;
  PSay(err.Message.c_str(), m_database_error, MB_OK);
 }
 catch (...)
 {
  Screen->Cursor = crDefault; 
  PSay("Unknown PutDataInDatabase error: caught (...) rather than EIBInterBaseError or EIBError.\nPlease inform a programmer.", m_database_error, MB_OK);
 }
 return false;
}
//---------------------------------------------------------------------------
bool TformRecordGeneric::DataUnlock()          // getdatafromdatabase , putdataonscreen, getlockfromdatabase
{
 // WARNING: SHOULD WE NOT GET THE LOCK FIRST? -- fix later maybe ... if we don't get the lock, at least we updated initial_state.
 if (_CanUpdate())
  if (GetDataFromDatabase())
   if (PutDataOnScreen())
    if (GetLockFromDatabase())
     {
      is_editing = true;
      is_new_record = false;
      ScreenUnlock();
      UpdateTitle();
      return true;
     }
    else
     PSay("Could not lock this record for updating, you or someone else is either:\n - editing this record,\n - editing a record it depends on,\n - or editing a record that depends on it.\nTry asking around the office.", m_program_error, MB_OK);
   else
    PSay("Could not get the data to go on the form", m_database_error, MB_OK);
  else
   PSay("Could not seem to find this record in the database", m_database_error, MB_OK);
 else
   PSay("You do not have permission to update records of this type (" + table_name + "). Contact your systems administrator (or boss) to get these permissions.", m_message, MB_OK);
 return false;
}
//---------------------------------------------------------------------------
bool TformRecordGeneric::DataSave()            // getdatafromscreen , putdataindatabase
{
 FocusAway();
 Screen->Cursor = crHourGlass;
 if ((is_editing && is_database_started) || is_new_record)
  if (IsValidOnScreen())
   if (GetDataFromScreen())
    if (CheckForUselessRecord())
    { // Philip had been abusing short-circuit evaluation because he was too lazy to insert braces.  Fixed.
     if (!is_new_record)
      SqlToMap(dbDatabase, "select * from " + table_name + " where id = " + initial_state["id"], hack_state);
     if (PutDataInDatabase())
     {
      SendAtlas(); // uses initial_state and ending_state for foreign keys and id's
      AfterSave();

      if (is_editing)
       SaveLog();  // uses initial_ and ending_states to save differences to the log

      initial_state = ending_state;

      if (is_new_record)
       NotNewAnymore(); // uses initial_state to setup atlas / filter for new records

      is_editing = false;
      is_new_record = false;

      ScreenLock();
      UpdateTitle();
      Screen->Cursor = crDefault;
      return true;
     }
     else
      ; // PSay("Putting data in database failed", m_database_error, MB_OK);
    }
    else
     ; // user has chosen not to put in a record that is of no use (blank one)
   else
   {
    Screen->Cursor = crDefault;
    PSay("Getting data off screen failed", m_program_error, MB_OK);
   }
  else
   ; // don't need this message box because message boxes should be give in IsValidOnScreen
     // PSay("Some of your data looks invalid. You might try making sure you've got only existing values chosen in drop-downs, and making sure you've got the more useful fields entered.", m_invalid_data, MB_OK);
 else if (is_editing && !is_database_started)
 {
  Screen->Cursor = crDefault;
  PSay("Dude, where's my transaction... (update/insert)", m_database_error, MB_OK);
 }
 else
 {
  Screen->Cursor = crDefault;
  PSay("Neither inserting nor updating, or database connection lost: no good reason to be here (DataSave)", m_program_error, MB_OK);
 }
 Screen->Cursor = crDefault;
 return false;
}
//---------------------------------------------------------------------------
bool TformRecordGeneric::DataUndo()            // getdatafromdatabase, putdataonscreen
{
 if (is_editing)
  if (is_database_started)
   if (lock_transaction->Active)
   {
    lock_transaction->Rollback();
    if (PutDataOnScreen())
     if (ScreenLock())
     {
      is_editing = false;
      UpdateTitle();
      return true;
     }
     else
      PSay("Could not re-lock controls on screen", m_program_error, MB_OK);
    else
     PSay("Could not put data on screen (undo)", m_program_error, MB_OK);
   }
   else
    PSay("Cannot rollback without an active transaction", m_database_error, MB_OK);
  else
   PSay("Dude, where's my transaction... (rollback)", m_database_error, MB_OK);
 else
  ; // PSay("No good reason to be here (rollback)", m_program_error, MB_OK);
 return false;
}
//---------------------------------------------------------------------------
bool TformRecordGeneric::DataClose()           // is_locked: ask -> yes: usersave, no: userundo
{
 if (_CanView())
 {
  if (is_editing)
   if (_CanUpdate())
    if (AskUser())
     return DataSave();
    else
     return DataUndo();
   else
    ; // too bad!
  if (is_new_record)
   if (_CanInsert())
    if (AskUser())
     return DataSave();
    else
     ; // do nothing, let created record be abandoned
   else
    ; // no permission? oh well!
  UpdateTitle();
 }
 else
 ; // who cares? can't do anything about it!
 return true;
}
//---------------------------------------------------------------------------
bool TformRecordGeneric::CloseNow()            //if open, always undo
{
 // this gets called upon cmd:die, and i'd like for it not to ask ... i want
 // it to go away, right away! we used to have CloseAsk() and CloseSave(),
 // but CloseSave() is way too dangerous, and we rarely -need- to close the
 // program while at the same time letting the user select Save/Undo ... i
 // mean really -- if we can wait for that ... why not just go ahead and tell
 // them you want them to close it? and ... even with that, nothing was going
 // to guarantee they didn't leave a question open and go open more screens,
 // what with the messagebox being only locally modal and such (i think.)
 if (is_editing)
  DataUndo();
 else
  is_new_record = false;
 Close();
 return true;
}
//---------------------------------------------------------------------------
bool TformRecordGeneric::ScreenLock()          // user-defined
{
 // to be implemented by derived class
 // this parent can be called to save time ( this->TformRecordGeneric::ScreenLock(); )
 buttonCurrent->Enabled = true;
 buttonCurrent->DropdownMenu = NULL;
 buttonA->Enabled =  _CanUpdate(); // true;
 buttonA->Caption = "Unlock";
 buttonA->Hint = "Unlock the controls to allow exclusive editing of this record";
 buttonB->Enabled = true;
 buttonB->Caption = "Close";
 buttonB->Hint = "Close this form";
 return true;
}
//---------------------------------------------------------------------------
bool TformRecordGeneric::ScreenUnlock()        // user-defined
{
 // to be implemented by derived class
 // this parent should be called by the derived class
 buttonCurrent->Enabled = true;
 buttonCurrent->DropdownMenu = popupCurrentTypes;
 buttonA->Enabled = true;
 buttonA->Caption = "Save";
 buttonA->Hint = "Save this record and lock the screen (view only mode)";
 if (is_editing)
 {
  buttonB->Enabled = true;
  buttonB->Caption = "Undo";
  buttonB->Hint = "Undo your changes and lock the screen (view only mode) with original data";
 }
 else
 {
  buttonB->Enabled = true;
  buttonB->Caption = "Cancel";
  buttonB->Hint = "Cancel entry of this new record, and close form";
 }
 return true;
}
//---------------------------------------------------------------------------
bool TformRecordGeneric::AskUser()
{
 int return_value = PSay("You have made changes that will be lost if you do not save now.\nWould you like to save your changes?", m_confirmation, MB_YESNO);
 if (return_value == IDYES)
  return true;
 else
  return false;
}
//---------------------------------------------------------------------------
bool TformRecordGeneric::UpdateCurrent()
{
 string curr_id = initial_state["current_id"];
 string last_up = initial_state["last_update_stamp"];
 string last_id = initial_state["last_updated_by"];

 // honestly try to get the information
 string v_current = lookupTables->ValueOf("current_types", curr_id);
 string v_username = lookupTables->ValueOf("users", last_id);
 Pstamp v_stamp(last_up, datefirst | mdy | yyyy | military);
 string v_date = v_stamp.OutDate(yyyy | mdy);
 string v_time = v_stamp.OutTime(withoutsec | military);

 // funny defaults
 if (v_current.empty())
  v_current = "?";
 else
  v_current = PrettyString(v_current);
 if (v_username.empty())
  v_username = "?";
 if (!v_stamp.IsValidStamp())
 {
  v_date = "?/?/?";
  v_time = "?:?";
 }

 // put it together
 string message = v_current + " - " + v_username + " - " + v_date; // + " at " + v_time;

 // display it
 buttonCurrent->Caption = message.c_str();
 buttonCurrent->Hint = message.c_str();

 IconIndex = StringToInt(curr_id) + 3;
 // imagesCurrent->GetIcon(StringToInt(curr_id) + 3, this->Icon);

 return true;
}
//---------------------------------------------------------------------------
bool TformRecordGeneric::PopupMessage (string p_popup_name, string p_popup_id, string p_popup_data)
{
 if (p_popup_name == "current_types")
 {
  if (is_editing || is_new_record)
  {
   //ending_state["current_id"] = p_popup_id;
   // so as not to modify stuff too much... we want ending_state modified, but
   // we need initial_state to contain data - i didn't really think about making
   // refresh functions take from either initial or ending... we'll see about that.
   // think about making updates go from either initial or ending, and consider
   // that unlocking the screen may mean more than just letting users edit the data
   // on the screen - what about links, etc. that have values that need to be copied?
   string temp_space = initial_state["current_id"];
   initial_state["current_id"] = p_popup_id;
   UpdateCurrent();
   ending_state["current_id"] = p_popup_id;
   initial_state["current_id"] = temp_space;
  }
 }
 return true;
}
//---------------------------------------------------------------------------
bool TformRecordGeneric::AtlasScreen(int request_number, string matched_value)
{
 // ignore request_number
 if (!is_editing && !is_new_record)
 {
  can_view = can_view_unknown;
  can_insert = can_insert_unknown;
  can_update = can_update_unknown;
  GetDataFromDatabase();
  PutDataOnScreen();
 }
 return true;
}
//---------------------------------------------------------------------------
bool TformRecordGeneric::GenericTransaction()
{
 if (lock_transaction)
  if (lock_transaction->Active)
  {
   PSay("Transaction already open - not sure whether to commit or rollback (lock)", m_database_error, MB_OK, "GenericTransaction called while lock_transaction already active");
   return false;
  }
  else
   is_database_started = true;
 else
 {
  // create the object if needed, clear it out (maybe modify later?)
  // TODO -c WARNING: SWITCH TO USING STDTRANSACTION? -- only difference is that std transaction leaves it running ...
  lock_transaction = new TIBTransaction(this);
  StdTransaction(lock_transaction, dbDatabase);
  lock_transaction->Commit();
  is_database_started = true;
 }
 return true;
}
//---------------------------------------------------------------------------
bool TformRecordGeneric::SendAtlas()
{
 // send all of initial_state, then send the differences in ending_state
 // so as to avoid repeating key/value pairs when not necessary (repeating
 // keys makes sense though, since a value will most probably change.)
 // we don't send initial_state for new records, and for new records we also
 // send all of ending_state rather than just the changes

 // changed: use hack_state instead, which is refetched before putdataindatabase()
 // this guarantees we see changes in cases where we've stupidly overwritten initial_state while the record was being edited (links, particularly)
 if (!is_new_record)
  for (map<string, string>::iterator i_initial = hack_state.begin();
       i_initial != hack_state.end(); i_initial++)
  {
   string key = table_name + "." + i_initial->first;
   string val = i_initial->second;
   if (LikeString("%id", LowercaseString(key)) || LikeString("%last_updated_by", LowercaseString(key)))
    atlas->Notify(LowercaseString(key), val);
  }

 // regardless, we send ending_state
 for (map<string, string>::iterator i_ending = ending_state.begin();
      i_ending != ending_state.end(); i_ending++)
 {
  string key = table_name + "." + i_ending->first;
  string val = i_ending->second;
  if ((LikeString("%id", LowercaseString(key)) || LikeString("%last_updated_by", LowercaseString(key))) &&
      (is_new_record || hack_state[i_ending->first] != val))
   atlas->Notify(LowercaseString(key), val);
 }

 // in case we're modifying a lookuptable value that is -also- a record? shouldn't affect anything but lookuptables (refreshing in codeprometheus)
 atlas->Notify(table_name, "");
 return true;
}
//---------------------------------------------------------------------------
bool TformRecordGeneric::UpdateTitle()
{
 // caption
 string newcaption = title_name;
 if (is_editing)
  newcaption += " - editing";
 else if (is_new_record)
  newcaption += " - creating";

 if (optionsPermissions["special.see_record_id"]["enabled"] == "true")
  newcaption += " [" + initial_state["id"] + "]";

 this->Caption = newcaption.c_str();
 return true;
}
//---------------------------------------------------------------------------
bool TformRecordGeneric::Initialize()
{
 //find the id - if missing, we're creating a new record
 //zero id is probably not valid either!
 if (!IsInt(initial_state["id"]))
 {
  is_new_record = true;
  buttonC->Visible = true;
  buttonC->Enabled = true;
  // default values for finger-pointing fields
  if (initial_state["current_id"] == "") initial_state["current_id"] = "4";
  if (initial_state["last_updated_by"] == "") initial_state["last_updated_by"] = current_userid;
  if (initial_state["last_update_stamp"] == "") initial_state["last_update_stamp"] = RightNow();
  blank_state = initial_state;
 }
 else
 {
//  NotNewAnymore();
  is_new_record = false;
  blank_state.clear();
  buttonC->Visible = false;
  buttonC->Enabled = false;
 }

 // wouldn't recommend deriving this, but do so if you need to. and don't
 // forget to call this parent version!
 //get data initially, setup atlas stuff ...
 if (_CanView())
 {
  if (!IsInt(initial_state["id"]))
   IsNewAgain();
  ScreenStart();
  UpdateTitle();
  if (IsInt(initial_state["id"]))
   NotNewAnymore();
  //by default, the screen is locked unless is_new_record
  if (!is_new_record)
   ScreenLock();
  else
   ScreenUnlock();
 }
 else
 {
  if (IsInt(initial_state["id"]))
   NotNewAnymore();
  ScreenLock();
  scrollStandard->Enabled = false;
  buttonCurrent->Enabled = false;
  buttonA->Enabled = false;
  Screen->Cursor = crDefault;
  PSay("You do not have permission to view records of this type (" + table_name + "). Contact your systems administrator (or boss) to get these permissions.", m_message, MB_OK);
 }
 Screen->Cursor = crDefault;

 if (!is_new_record && buttonA->Enabled && optionsPreferences["record_generic"]["auto_unlock"] == "true")
  buttonA->Click();

 return true;
}
//---------------------------------------------------------------------------
bool TformRecordGeneric::NotNewAnymore()
{
 TformSpecialAction::FillActionMenu(menuActions, table_name, this);
 menuDeleteRecord->Visible = (optionsPermissions["admin.delete_record"]["enabled"] == "true") ? true : false;
 menuViewChangeLog->Visible = (optionsPermissions["admin.change_log"]["enabled"] == "true") ? true : false;
 menuDataNavigate->Visible = (optionsPermissions["special.data_navigate"]["enabled"] == "true") ? true : false;

 if (menuActions->Visible || menuDeleteRecord->Visible || menuViewChangeLog->Visible || menuDataNavigate->Visible)
  this->PopupMenu = popupRecord;
 else
  this->PopupMenu = NULL;

 atlas->RequestNotificationWhen((table_name + ".id"), initial_state["id"], rel_e, as_int, this, RefreshHandler(&(TformRecordGeneric::AtlasScreen)));
 return true;
}
//---------------------------------------------------------------------------
bool TformRecordGeneric::AfterSave()
{
 // override me! this function gets called anytime a save has succeeded,
 // but that shouldn't be needed except in very special cases. just in
 // case, don't forget to call
 // TformRecordGeneric::AfterSave();
 // from that new version of this function, so that code can be added here
 // and happen everywhere (forms that don't need it, and forms that do.)
 return true;
}
//---------------------------------------------------------------------------
bool TformRecordGeneric::SaveLog()
{
 list <map<string, string> > differences;
 map<string, string> this_difference, common_fields;

 // check for differences other than id, current_id, last_updated_by, last_update_stamp
 // use RightNow() for the current timestamp, current_userid for the user_id...

 // setup the basic fields for default_difference;
 common_fields ["change_by"]    = current_userid;
 common_fields ["change_stamp"] = RightNow();
 common_fields ["record_id"]    = ending_state["id"];
 common_fields ["table_name"]   = table_name;

 // changed to also use hack_state rather than initial_state, for the same reasons as in SendAtlas()
 map<string, string>::iterator i_after;
 map<string, string>::iterator i_after_end = ending_state.end();
 for (map<string, string>::iterator i_before = hack_state.begin();
      i_before != hack_state.end(); i_before++)
 {
  this_difference = common_fields;

  if (i_before->first != "id")
  {
   i_after = ending_state.find(i_before->first);
   if (i_after != i_after_end)
   {
    if (UppercaseString(i_after->second) != UppercaseString(i_before->second))
    {
     i_before->second = i_before->second.substr(0, 255);
     i_after->second = i_after->second.substr(0, 255);
     this_difference["field_name"]     = i_after->first;
     this_difference["previous_value"] = i_before->second;
     this_difference["present_value"]  = i_after->second;
     differences.push_back (this_difference);
    }
   }
  }
 }

 for (list<map<string, string> >::iterator i_difference = differences.begin();
      i_difference != differences.end(); i_difference++)
  if (!ExecuteInsertStatement(lock_transaction, "change_log", *i_difference))
   PSay("Could not save log data to change_log!", m_database_error, MB_OK);
 return true;
}
//---------------------------------------------------------------------------
void __fastcall TformRecordGeneric::buttonCurrentClick(TObject *Sender)
{
  buttonCurrent->CheckMenuDropdown();
}
//---------------------------------------------------------------------------
bool TformRecordGeneric::IsBusy()
{
 if (is_editing || is_new_record)
  return true;
 else
  return false;
}
//---------------------------------------------------------------------------
bool TformRecordGeneric::DataNext()
{
 if (is_editing || is_new_record)
  if (IsValidOnScreen())
   if (GetDataFromScreen())
    if (CheckForUselessRecord())
     if (PutDataInDatabase())
     {
      SendAtlas(); // uses initial_state and ending_state for foreign keys and id's
      AfterSave();
      return true;
     }
     else
      PSay("Putting data in database failed", m_database_error, MB_OK);
    else
     ; // user has elected not to go to the next thingie
   else
    PSay("Getting data off screen failed", m_program_error, MB_OK);
  else
   ; //PSay("Some of your data looks invalid. You might try making sure you've got only existing values chosen in drop-downs, and making sure you've got the more useful fields entered.", m_invalid_data, MB_OK);
 else
  PSay("No good reason to be here (update/insert)", m_program_error, MB_OK);
 return false;
}
//---------------------------------------------------------------------------
void __fastcall TformRecordGeneric::buttonCClick(TObject *Sender)
{
 if ((is_editing || is_new_record) && DataNext() || (!is_editing && !is_new_record))
 {
  is_editing = false;
  is_new_record = true;
  initial_state = blank_state;
  IsNewAgain();
  PutDataOnScreen();
  UpdateTitle();
  ScreenUnlock();
  scrollStandard->SetFocus();
 }
}
//---------------------------------------------------------------------------
bool TformRecordGeneric::IsNewAgain()
{
 // don't forget to call this parent version from the derived one!
 this->PopupMenu = NULL;
 atlas->CancelRequestFrom((TForm *)this);
 for (map<string, filter_atlas_wrapper>::iterator itr = filter_atlas_handler.begin(); itr != filter_atlas_handler.end(); itr++)
   itr->second.Reset(this);
 return true;
}
//---------------------------------------------------------------------------
bool TformRecordGeneric::CheckForUselessRecord()
{
 // overwrite this and return true if it's causing problems

 bool result = false;
 if (!is_new_record)
  result = true;
 else
 {
  map<string, string> initial_stripped = initial_state;
  map<string, string> ending_stripped = ending_state;
  // need to compare the two -other- than by the following fields
  // ending_state["id"] -- probably won't be different yet ... but just in case
  // ending_state["current_id"]
  // ending_state["last_update_stamp"]
  // ending_state["last_updated_by"]

  initial_stripped.erase("id");
  initial_stripped.erase("current_id");
  initial_stripped.erase("last_update_stamp");
  initial_stripped.erase("last_updated_by");
  ending_stripped.erase("id");
  ending_stripped.erase("current_id");
  ending_stripped.erase("last_update_stamp");
  ending_stripped.erase("last_updated_by");

  // okay, that's not enough. we also want to look for automatic changes handled
  // by GetDataFromScreen on the derived forms -- there, they might set drop-down
  // and checkboxes to '0' rather than '', and we need to see those too.

  // for every ending_stripped value that is "0" and the matching value is "" in
  // initial_stripped, remove that key entirely from the thingie. yup.
  set<string> remove_keys;
  for (map<string,string>::iterator i_key = ending_stripped.begin(); i_key != ending_stripped.end(); i_key++)
  {
   if (i_key->second == "0" && initial_stripped[i_key->first] == "")
    remove_keys.insert(i_key->first);
   // otherwise, well, if it's not 0, they probably won't match anyway ...
   // if it is, and it didn't exist, it's the same as blank, they'll both get removed ...
   // in other words, i think this is safe. the blank value that might get created will
   // get removed in a second anyway, so who cares, right? =)
  }

  // okay, we think these are bad (as in, they'll get in the way of the comparison below)
  for (set<string>::iterator i_key = remove_keys.begin(); i_key != remove_keys.end(); i_key++)
  {
   initial_stripped.erase(*i_key);
   ending_stripped.erase(*i_key);
  }

  // -now- we look, after having taken out all the automatics i can think of ... yup.
  if (initial_stripped == ending_stripped)
  {
   int user_answer = PSay("You do not seem to have put any information into this new record.\nThis may be necessary, and is allowed, but is discouraged.\nDo you wish to insert this record as it currently is?", m_missing_data, MB_YESNO);
   if (user_answer == IDYES)
   {
    record_useless_override = true;
    result = true;
   }
  }
  else
   result = true;
 }
 // While we're about fixing things, let's do this new thing with fixing abuse of fields_to_leave_raw by entering data in ALL CAPS,
 // replacing it with something more sane, unless they -specifically- have permission to do so
 if (optionsPermissions["special.can_save_all_caps"]["enabled"] != "true")
  for (set<string>::iterator raw=fields_to_leave_raw.begin(); raw!=fields_to_leave_raw.end(); raw++)
  {
   if (ending_state.find(*raw) != ending_state.end())               // field exists in output
    if (initial_state[*raw] != ending_state[*raw])                  // field has changed since input
     if (ending_state[*raw] == UppercaseString(ending_state[*raw])) // field is all caps
      ending_state[*raw] = SentenceString(ending_state[*raw]);      // change to suitable default
  }

 return result;
}
//---------------------------------------------------------------------------
void __fastcall TformRecordGeneric::menuDeleteRecordClick(TObject *Sender)
{
   if (is_editing)
   {
     PSay("Can't delete a record unlocked for editing", m_message, MB_OK);
   }
   else
   {
      int result = PSay("Are you SURE you want to delete this record?", m_confirmation, MB_YESNO);
      if (result == IDYES)
      {
         if (BeforeDeleteRecord())
         {
           if (DeleteRecord())
           {
             AfterDeleteRecord();
           }
         }
      }
   }
}
//---------------------------------------------------------------------------
bool TformRecordGeneric::BeforeDeleteRecord()
{
   try
   {
     delete_transaction = new TIBTransaction(NULL);
     StdTransaction (delete_transaction, dbDatabase);
     return true;
   }
   catch (...)
   {
     return false;
   }
}
//---------------------------------------------------------------------------
bool TformRecordGeneric::DeleteRecord()
{
   /* TODO: Could automagically change FKs pointing to this record to be zero.  Could also do a
            cascading delete.  Currently it's still set to try to do a simple deltion of this record
            with no further checks, and each record screen can override this function to remove FKs
            manually, etc. and then call the generic (this) function to take care of the rest. */
   string sqlstring = "delete from " + table_name + " where id = " + initial_state["id"];

   // Execute statement
   bool toReturn = true;
   TIBSQL * dbSQL = new TIBSQL(NULL);
   try
   {
     dbSQL->Database = dbDatabase;
     dbSQL->Transaction = delete_transaction;
     dbSQL->SQL->Text = sqlstring.c_str();
     dbSQL->ExecQuery();
     delete_transaction->Commit();
     toReturn = true;
   }
   catch (EIBInterBaseError & the_error)
   {
     PSay((the_error.Message).c_str(), m_database_error, MB_OK);
     toReturn = false;
   }
   catch (...)
   {
     toReturn = false;
   }
   if (delete_transaction->Active)
     delete_transaction->Rollback();
   delete dbSQL;
   delete delete_transaction;
   // End execute statement
   if (toReturn)
   {
      PSay("The deletion operation was completed successfully!", m_message, MB_OK);
      atlas->CancelRequestFrom(this);
      //atlas->Notify(table_name + ".id", initial_state["id"]);
      SendAtlas();
      Close();
   }
   else
   {
      PSay("The deletion operation failed to complete successfully!", m_database_error, MB_OK);
      /* TODO:  Ask if they want to try to do a cascading delete instead.  If they say yes ...
         should this be a separate screen (like DataNavigate and DataMerge), with a built-in
         log to show what it's doing (like DataMerge has)?  Probably, since, like DataMerge,
         anything deleted in this way really needs to be logged to the change_log, for IHS
         export and so forth.  Ugh.  Later.  */
   }

   return toReturn;
}
//---------------------------------------------------------------------------
bool TformRecordGeneric::AfterDeleteRecord()
{
  return true;
}
//---------------------------------------------------------------------------
void __fastcall TformRecordGeneric::menuViewChangeLogClick(TObject *Sender)
{
  FORM_TRY
    TformSpecialViewChangeLog * change_log = new TformSpecialViewChangeLog(NULL, table_name, initial_state["id"]);
    change_log->Show();
  FORM_CATCH
}
//---------------------------------------------------------------------------
void __fastcall TformRecordGeneric::FormShortCut(TWMKey &Msg,bool &Handled)
{
  TformSpecialGeneric::FormShortCut(Msg, Handled);
  // TODO: add & characters to captions to make these work automatically (if possible)

  if(HIBYTE(GetAsyncKeyState(VK_CONTROL)))
  {
    Handled = true;
    if (((Msg.CharCode == 's') || (Msg.CharCode == 'S')) && ((is_editing) || (is_new_record)))
      buttonAClick(NULL);  // save
    else if (( // ^Z is more standard; we don't need an extra shortcut for Undo // (Msg.CharCode == 'd') || (Msg.CharCode == 'D') ||
              (Msg.CharCode == 'z') || (Msg.CharCode == 'Z')) && (is_editing) && (! is_new_record))
      buttonBClick(NULL);  // undo
    else if (((Msg.CharCode == 'l') || (Msg.CharCode == 'L')) && (is_new_record))
      buttonBClick(NULL);  // cancel
    else if (((Msg.CharCode == 'n') || (Msg.CharCode == 'N')) && (is_new_record))
      buttonCClick(NULL);  // save and another (next)
    else if (((Msg.CharCode == 'u') || (Msg.CharCode == 'U')) && (! is_editing))
      buttonAClick(NULL);  // unlock
    else if (((Msg.CharCode == 't') || (Msg.CharCode == 'T')) && (! is_editing))
    { // "touch" a record
      buttonAClick(NULL);  // unlock
      buttonAClick(NULL);  // save
    }
    // Escape works for close.  Why do we need another shortcut, esp. one that interferes w/ the ubiquitious Copy shortcut?
    //else if (((Msg.CharCode == 'c') || (Msg.CharCode == 'C')) && (! is_editing))
    //  buttonBClick(NULL);  // close
    else
      Handled = false;
  }
  else
    Handled = false;
}
//---------------------------------------------------------------------------
bool TformRecordGeneric::BeforeSave()
{
 return true;
}
//---------------------------------------------------------------------------
void __fastcall TformRecordGeneric::menuDataNavigateClick(TObject *Sender)
{
 FORM_TRY
  TformSpecialDataNavigate * nav = new TformSpecialDataNavigate(NULL);
  nav->Show();
  nav->cboTable->ItemIndex = nav->cboTable->Items->IndexOf(UppercaseString(table_name).c_str());
  nav->cboTable->OnChange(NULL);
  nav->cboPk->ItemIndex = nav->cboPk->Items->IndexOf("ID");
  nav->editPkValue->Text = initial_state["id"].c_str();
  nav->btnGo->Click();
 FORM_CATCH
}
//---------------------------------------------------------------------------
AnsiString __fastcall TformRecordGeneric::GetRecordTitle()
{
 return title_name.c_str();
}
//---------------------------------------------------------------------------
void __fastcall TformRecordGeneric::SetRecordTitle(AnsiString Value)
{
 title_name = Value.c_str();
 UpdateTitle();
}
//---------------------------------------------------------------------------
void __fastcall TformRecordGeneric::menuActionsClick(TObject *Sender)
{
 if (((TMenuItem*)Sender)->Tag)
 {
  string stn = table_name;
  string selected_id = initial_state["id"];
  a_factory_create to_create = (a_factory_create)(((TMenuItem*)Sender)->Tag);
  TformSpecialAction::Do(stn, to_create, selected_id);
 }
}
//---------------------------------------------------------------------------
bool TformRecordGeneric::_CanView()
{ // no reason to override this function; override CanView() instead
  if (can_view == can_view_true)
    return true;
  if (can_view == can_view_false)
    return false;
  // otherwise, it's can_view_unknown, and we figure it out.

  if (CanView())
  {
    can_view = can_view_true;
    return true;
  }
  else
  {
    can_view = can_view_false;
    return false;
  }
}
//---------------------------------------------------------------------------
bool TformRecordGeneric::CanView()
{
  return optionsPermissions["table."+table_name]["view"] == "true";
}
//---------------------------------------------------------------------------
bool TformRecordGeneric::_CanInsert()
{ // no reason to override this function; override CanInsert() instead
  if (can_insert == can_insert_true)
    return true;
  if (can_insert == can_insert_false)
    return false;
  // otherwise, it's can_insert_unknown, and we figure it out.

  if (CanInsert())
  {
    can_insert = can_insert_true;
    return true;
  }
  else
  {
    can_insert = can_insert_false;
    return false;
  }
}
//---------------------------------------------------------------------------
bool TformRecordGeneric::CanInsert()
{
  return optionsPermissions["table."+table_name]["insert"] == "true";
}
//---------------------------------------------------------------------------
bool TformRecordGeneric::_CanUpdate()
{ // no reason to override this function; override CanUpdate() instead
  // if we already know, then go with that
  if (can_update == can_update_true)
    return true;
  if (can_update == can_update_false)
    return false;
  // otherwise it's can_update_unknown, and we figure it out.
  if (CanUpdate())
  {
    can_update = can_update_true;
    return true;
  }
  else
  {
    can_update = can_update_false;
    return false;
  }
}
//---------------------------------------------------------------------------
bool TformRecordGeneric::CanUpdate()
{
  // explicit update permissions to the table?  then yes, we're done
  if (optionsPermissions["table."+table_name]["update"] == "true")
    return true;
  // if they (still) have insert permissions to the table, and
  // they were the last user to modify the record, that's a pass too
  if (CanInsert() &&
      initial_state["last_updated_by"] == current_userid)
    return true;
  // if they have insert permissions to the table, and
  // the user who last modified the record is no longer valid, and
  // the current user is in the same department as the now-invalid user
  // then that's a pass (just let the whole department modify their old data)
  string last_updated_by_group = GetOneField(dbDatabase, "select group_id from users where id = '" + initial_state["last_updated_by"] + "'");
  string last_updated_by_user_valid = GetOneField(dbDatabase, "select valid from users where id = '" + initial_state["last_updated_by"] + "'");
  if (_CanInsert() &&
      IsBlankOrZero(last_updated_by_user_valid) &&
      current_groupid == last_updated_by_group && !IsBlankOrZero(last_updated_by_group))
    return true;

  // otherwise, so sorry, you can't modify this record
  return false;
}
//---------------------------------------------------------------------------

