//---------------------------------------------------------------------------

#include <vcl.h>
#pragma hdrstop

#include "codeRecordGoal.h"
//---------------------------------------------------------------------------
#pragma package(smart_init)
#pragma link "codeRecordGeneric"
#pragma resource "*.dfm"
TformRecordGoal *formRecordGoal;
//---------------------------------------------------------------------------
__fastcall TformRecordGoal::TformRecordGoal(TComponent* Owner, map<string,string> & p_parameters)
        : TformRecordGeneric(Owner, p_parameters, string("goals"))
{
 Initialize();
}
//---------------------------------------------------------------------------
bool TformRecordGoal::ScreenStart()
{
  // register combo boxes
  lookupTables->RegisterComboBox("activity_level_types", cboLevel);
  lookupTables->RegisterComboBox("frequency_types", cboFrequency);
  lookupTables->RegisterComboBox("length_types", cboLength);


  TformRecordGeneric::ScreenStart();
  return true;
}
//---------------------------------------------------------------------------
bool TformRecordGoal::ScreenStop()
{
  // unregister combo box
  lookupTables->UnregisterComboBox("activity_level_types", cboLevel);
  lookupTables->UnregisterComboBox("frequency_types", cboFrequency);
  lookupTables->UnregisterComboBox("length_types", cboLength);

  TformRecordGeneric::ScreenStop();
  return true;
}
//---------------------------------------------------------------------------
bool TformRecordGoal::IsValidOnScreen()
{
  // check to make sure that the values entered in the combo boxe
  // corrospond to values in that look up table.
  // check to make sure the dates entered are valid
  // give the user a message box and prompt to re-enter if incorrect
  // make sure there is a link to a person, although there's not much the user
  // can do to fix that if there's not
  if (! lookupTables->IsInLookUpTable("activity_level_types", cboLevel->Text.c_str()))
  {
    PSay("Please enter a valid activity level", m_invalid_data, MB_OK);
    cboLevel->SetFocus();
    return false;
  }
  else if (! lookupTables->IsInLookUpTable("frequency_types", cboFrequency->Text.c_str()))
  {
    PSay("Please enter a valid number of times per week", m_invalid_data, MB_OK);
    cboFrequency->SetFocus();
    return false;
  }
  else if (! lookupTables->IsInLookUpTable("length_types", cboLength->Text.c_str()))
  {
    PSay("Please enter a valid length", m_invalid_data, MB_OK);
    cboLength->SetFocus();
    return false;
  }
  else if ((initial_state["person_id"] == "") || (initial_state["person_id"] == "0"))
  {
    PSay("There is no person associated with this record", m_missing_data, MB_OK);
    return false;
  }
  else
    return true;
}
//---------------------------------------------------------------------------

bool TformRecordGoal::PutDataOnScreen()
{
  // put the values on the form from the map initial_state
  // values may have come from GetDataFromDatabase if it's an existing record,
  // or values may have been passed to the form in this map if it's a new record

  if(initial_state["date_entered"] != "")
    dateEntered->Date = Pdate(initial_state["date_entered"], mdy | yyyy).OutDate();
  else
    dateEntered->Date = Date();
  cboLevel->Text = lookupTables->ValueOf("activity_level_types", initial_state["activity_level_id"]).c_str();
  cboFrequency->Text = lookupTables->ValueOf("frequency_types", initial_state["frequency_id"]).c_str();
  cboLength->Text = lookupTables->ValueOf("length_types", initial_state["length_id"]).c_str();
  editActivityGoal->Text = SentenceString(initial_state["activity_goal"]).c_str();
  editTimesGoal->Text = SentenceString(initial_state["times_goal"]).c_str();
  editAmountGoal->Text = SentenceString(initial_state["amount_goal"]).c_str();

  CreateLink(txtPerson, NULL, this, initial_state["person_id"], "people");

  TformRecordGeneric::PutDataOnScreen();  
  return true; 
}
//---------------------------------------------------------------------------
bool TformRecordGoal::GetDataFromScreen()
{
  // place the values currently on the screen into
  // the map that will be used to update or insert the record
  // the parent form will call the insert or update function, and get the id
  // number if necessary and take care of current_id

  ending_state["date_entered"] = Pdate(dateEntered->Date).OutDate(mdy | yyyy);
  ending_state["activity_level_id"] = lookupTables->IdOf("activity_level_types", cboLevel->Text.c_str());
  ending_state["frequency_id"] = lookupTables->IdOf("frequency_types", cboFrequency->Text.c_str());
  ending_state["length_id"] = lookupTables->IdOf("length_types", cboLength->Text.c_str());
  ending_state["activity_goal"] = editActivityGoal->Text.c_str();
  ending_state["times_goal"] = editTimesGoal->Text.c_str();
  ending_state["amount_goal"] = editAmountGoal->Text.c_str();

  ending_state["person_id"] = initial_state["person_id"];
  ending_state["id"] = initial_state["id"];

  TformRecordGeneric::GetDataFromScreen();

  return true; // again, not much can go wrong here
}
//---------------------------------------------------------------------------
bool TformRecordGoal::ScreenLock()
{
  // disable panel containing edit boxes
  panelGroup->Enabled = false;
  editActivityGoal->ReadOnly = true;
  editActivityGoal->TabStop = false;
  editTimesGoal->ReadOnly = true;
  editTimesGoal->TabStop = false;
  editAmountGoal->ReadOnly = true;
  editAmountGoal->TabStop = false;
  TformRecordGeneric::ScreenLock();
  return true;
}
//---------------------------------------------------------------------------
bool TformRecordGoal::ScreenUnlock()
{
  // enable panel containing edit boxes
  panelGroup->Enabled = true;
  editActivityGoal->ReadOnly = false;
  editActivityGoal->TabStop = true;
  editTimesGoal->ReadOnly = false;
  editTimesGoal->TabStop = true;
  editAmountGoal->ReadOnly = false;
  editAmountGoal->TabStop = true;
  TformRecordGeneric::ScreenUnlock();
  return true;
}
//---------------------------------------------------------------------------


