//---------------------------------------------------------------------------

#include <vcl.h>
#pragma hdrstop

#include "codeRecordIncentive.h"
#include "codeSearchPerson.h"
//---------------------------------------------------------------------------
#pragma package(smart_init)
#pragma link "codeRecordGeneric"
#pragma resource "*.dfm"
TformRecordIncentive *formRecordIncentive;
//---------------------------------------------------------------------------
__fastcall TformRecordIncentive::TformRecordIncentive(TComponent* Owner, map<string,string> & p_parameters)
        : TformRecordGeneric(Owner, p_parameters, string("incentives"))
{
 Initialize();
}
//---------------------------------------------------------------------------
bool TformRecordIncentive::ScreenStart()
{
  // register combo box
  lookupTables->RegisterComboBox("incentive_types", cboType);
  lookupTables->RegisterComboBox("shoe_types", cboShoeType);
  lookupTables->RegisterComboBox("shirt_types", cboShirtType);
  lookupTables->RegisterComboBox("shoe_size_types", cboShoeSize);
  lookupTables->RegisterComboBox("shirt_size_types", cboShirtSize);
  TformRecordGeneric::ScreenStart();
  return true;
}
//---------------------------------------------------------------------------
bool TformRecordIncentive::ScreenStop()
{
  // unregister combo box
  lookupTables->UnregisterComboBox("incentive_types", cboType);
  lookupTables->UnregisterComboBox("shoe_types", cboShoeType);
  lookupTables->UnregisterComboBox("shirt_types", cboShirtType);
  lookupTables->UnregisterComboBox("shoe_size_types", cboShoeSize);
  lookupTables->UnregisterComboBox("shirt_size_types", cboShirtSize);

  TformRecordGeneric::ScreenStop();
  return true;
}
//---------------------------------------------------------------------------
bool TformRecordIncentive::IsValidOnScreen()
{
  // check to make sure that the values entered in the combo boxe
  // corrospond to values in that look up table.
  // check to make sure the dates entered are valid
  // give the user a message box and prompt to re-enter if incorrect
  // make sure there is a link to a person, although there's not much the user
  // can do to fix that if there's not
  if(GetValidLinkId(txtPerson) == "")
  {
    PSay("Please enter a person", m_missing_data, MB_OK);
    btnPersonSearch->SetFocus();
    return false;
  }
  else if (! lookupTables->IsInLookUpTable("incentive_types", cboType->Text.c_str()))
  {
    PSay("Please enter a valid type", m_invalid_data, MB_OK);
    cboType->SetFocus();
    return false;
  }
  else if (cboType->Text == "")
  {
    PSay("Please enter the type of incentive", m_missing_data, MB_OK);
    cboType->SetFocus();
    return false;
  }
  else if (! Pdate (editDateReceived->Text.c_str(), mdy | yy).IsValidDate() && ! (editDateReceived->Text == ""))
  {
    PSay("Please enter a valid date received", m_invalid_data, MB_OK);
    editDateReceived->SetFocus();
    return false;
  }
  else if (! lookupTables->IsInLookUpTable("shirt_types", cboShirtType->Text.c_str()))
  {
    PSay("Please enter a valid shirt type", m_invalid_data, MB_OK);
    cboShirtType->SetFocus();
    return false;
  }
  else if (! lookupTables->IsInLookUpTable("shirt_size_types", cboShirtSize->Text.c_str()))
  {
    PSay("Please enter a valid shirt size", m_invalid_data, MB_OK);
    cboShirtSize->SetFocus();
    return false;
  }
  else if (! Pdate (editDate->Text.c_str(), mdy | yy).IsValidDate() && ! (editDate->Text == ""))
  {
    PSay("Please enter a valid date sent to Finance", m_invalid_data, MB_OK);
    editDate->SetFocus();
    return false;
  }
  else if (! IsAmount(editAmount->Text.c_str()))
  {
    PSay("Please enter a valid amount", m_invalid_data, MB_OK);
    editAmount->SetFocus();
    return false;
  }
  else if (! lookupTables->IsInLookUpTable("shoe_types", cboShoeType->Text.c_str()))
  {
    PSay("Please enter a valid shoe type", m_invalid_data, MB_OK);
    cboShoeType->SetFocus();
    return false;
  }
  else if (! lookupTables->IsInLookUpTable("shoe_size_types", cboShoeSize->Text.c_str()))
  {
    PSay("Please enter a valid shoe size", m_invalid_data, MB_OK);
    cboShoeSize->SetFocus();
    return false;
  }  
  else if (! Pdate (editDateOrdered->Text.c_str(), mdy | yy).IsValidDate() && ! (editDateOrdered->Text == ""))
  {
    PSay("Please enter a valid order date", m_invalid_data, MB_OK);
    editDateOrdered->SetFocus();
    return false;
  }
  else
    return true;
}
//---------------------------------------------------------------------------

bool TformRecordIncentive::PutDataOnScreen()
{
  // put the values on the form from the map initial_state
  // values may have come from GetDataFromDatabase if it's an existing record,
  // or values may have been passed to the form in this map if it's a new record

  cboType->Text = lookupTables->ValueOf("incentive_types", initial_state["incentive_id"]).c_str();
  editDateReceived->Text = initial_state["date_received"].c_str();
  editNotes->Text = SentenceString(initial_state["note"]).c_str();
  cboShirtSize->Text = lookupTables->ValueOf("shirt_size_types", initial_state["shirt_size_id"]).c_str();
  cboShirtType->Text = lookupTables->ValueOf("shirt_types", initial_state["shirt_id"]).c_str();  
  editDate->Text = initial_state["date_sent_to_finance"].c_str();
  editCheckNum->Text = initial_state["check_number"].c_str();
  editAmount->Text = AmountToString(initial_state["amount"]).c_str();
  cboShoeType->Text = lookupTables->ValueOf("shoe_types", initial_state["shoe_id"]).c_str();
  cboShoeSize->Text = lookupTables->ValueOf("shoe_size_types", initial_state["shoe_size_id"]).c_str();
  editDateOrdered->Text = initial_state["date_ordered"].c_str();
  if (initial_state["received_by_person"] == "1")
    checkReceived->Checked = true;
  else
    checkReceived->Checked = false;
  if (initial_state["received_from_nike"] == "1")
    checkNike->Checked = true;
  else
    checkNike->Checked = false;

  CreateLink(txtPerson, btnPersonSearch, this, initial_state["person_id"], "people");

  TformRecordGeneric::PutDataOnScreen();  
  return true; 
}
//---------------------------------------------------------------------------
bool TformRecordIncentive::GetDataFromScreen()
{
  // place the values currently on the screen into
  // the map that will be used to update or insert the record
  // the parent form will call the insert or update function, and get the id
  // number if necessary and take care of current_id
  ending_state["incentive_id"] = lookupTables->IdOf("incentive_types", cboType->Text.c_str());
  if (editDateReceived->Text != "")
    ending_state["date_received"] =  Pdate (editDateReceived->Text.c_str(), mdy | yy).OutDate(mdy | yyyy);
  else
    ending_state["date_received"] = "";
  ending_state["shirt_size_id"] = lookupTables->IdOf("shirt_size_types", cboShirtSize->Text.c_str());
  ending_state["shirt_id"] = lookupTables->IdOf("shirt_types", cboShirtType->Text.c_str());
  ending_state["shoe_id"] = lookupTables->IdOf("shoe_types", cboShoeType->Text.c_str());
  ending_state["shoe_size_id"] = lookupTables->IdOf("shoe_size_types", cboShoeSize->Text.c_str());
  if (editDateOrdered->Text != "")
    ending_state["date_ordered"] =  Pdate (editDateOrdered->Text.c_str(), mdy | yy).OutDate(mdy | yyyy);
  else
    ending_state["date_ordered"] = "";
  if (checkReceived->Checked)
    ending_state["received_by_person"] = "1";
  else
    ending_state["received_by_person"] = "0";
  if (checkNike->Checked)
    ending_state["received_from_nike"] = "1";
  else
    ending_state["received_from_nike"] = "0";
  ending_state["note"] = editNotes->Text.c_str();
  if (editDate->Text != "")
    ending_state["date_sent_to_finance"] =  Pdate (editDate->Text.c_str(), mdy | yy).OutDate(mdy | yyyy);
  else
    ending_state["date_sent_to_finance"] = "";
  ending_state["check_number"] = editCheckNum->Text.c_str();
  ending_state["amount"] = StringToAmount(editAmount->Text.c_str());

  ending_state["person_id"] = GetValidLinkId(txtPerson);
  ending_state["id"] = initial_state["id"];

  TformRecordGeneric::GetDataFromScreen();

  return true; // again, not much can go wrong here
}
//---------------------------------------------------------------------------
bool TformRecordIncentive::ScreenLock()
{
  // disable panel containing edit boxes
  panelGroup->Enabled = false;
  editNotes->ReadOnly = true;
  editNotes->TabStop = false;
  TformRecordGeneric::ScreenLock();
  return true;
}
//---------------------------------------------------------------------------
bool TformRecordIncentive::ScreenUnlock()
{
  // enable panel containing edit boxes
  panelGroup->Enabled = true;
  editNotes->ReadOnly = false;
  editNotes->TabStop = true;
  TformRecordGeneric::ScreenUnlock();
  return true;
}
//---------------------------------------------------------------------------
void __fastcall TformRecordIncentive::btnPersonSearchClick(TObject *Sender)
{
  // create and show new person search form
  vector<string> return_vector;
  FORM_TRY
    TformSearchPerson * formPersonSearch = new TformSearchPerson(NULL,&return_vector);
    formPersonSearch->ShowTaskModal(this);
  FORM_CATCH
  if (! return_vector.empty())
    CreateLink(txtPerson, btnPersonSearch, this, return_vector[0], "people");
}
//---------------------------------------------------------------------------

