//---------------------------------------------------------------------------

#include <vcl.h>
#pragma hdrstop

#include "codeRecordIncomingBill.h"
#include "codeSearchTaxEntity.h"
#include "codeRecordTaxEntity.h"
#include "codeRecordTransaction.h"
#include "codeRecordProcedurePayment.h"
#include "codeSpecialModifyPayable.h"
#include "codeSpecialSelectGroups.h"
#include "codeWizardModifyTransactions.h"
#include "codeSpecialTransactionSummary.h"
#include "codeSpecialTransactionDetails.h"
#include "reportBulkEOB.h"
//---------------------------------------------------------------------------
#pragma package(smart_init)
#pragma link "codeRecordGeneric"
#pragma resource "*.dfm"
TformRecordIncomingBill *formRecordIncomingBill;
//---------------------------------------------------------------------------
__fastcall TformRecordIncomingBill::TformRecordIncomingBill(TComponent* Owner, map<string,string> &p_parameters)
        : TformRecordGeneric(Owner, p_parameters, string("bills")), transactions_summary(this, listTransactions)
{
  payments_filter.SetListViewHandle (listProcedurePayments);
  payments_filter.SetSecurity("procedure_payments");
  payments_filter.AddColumnSpec("Procedure", "procedures", as_text, true, 60, rawString);
  payments_filter.AddColumnSpec("Residual", "procedure_payments", as_double, true, 60, amountString);
  payments_filter.AddColumnSpec("Paid", "procedure_payments", as_double, true, 60, amountString);
  payments_filter.AddColumnSpec("Current Id", "current_id", as_int, false, 80);
  payments_filter.AddColumnSpec("Internal Id", "internal_id", as_int, false, 80);
  payments_filter.FetchPreferences("recordincomingbill.payments_filter");
  payments_filter.SetSQLStatement("");
  payments_filter.RefreshDataSet();

  staticTransactionDetails->Font->Color = link_color;
  staticBetaModifyTransactions->Font->Color = link_color;
  staticPrintEOB->Font->Color = link_color;
  lastRadioChecked = 1;

  Initialize();
}
//---------------------------------------------------------------------------
bool TformRecordIncomingBill::ScreenStart()
{
  // register combo box
  //staticModifyAccPayable->Font->Color = link_color; // link removed

  //lookupTables->RegisterComboBox("users", cboUser);
  /*
  if (radioGroups->Checked)
  {
     cboUser->Style = Stdctrls::csDropDownList;
     cboUser->Clear();
  }
  else if (radioUser->Checked)
  {
     cboUser->Style = Stdctrls::csDropDown;
     lookupTables->RegisterComboBox("users", cboUser);
     cboUser->PopupMenu = popupGroups;
  }
  */
  
 TformRecordGeneric::ScreenStart();
 return true;
}
//---------------------------------------------------------------------------
bool TformRecordIncomingBill::ScreenStop()
{
  // unregister combo box
  /*
  lookupTables->UnregisterComboBox("users", cboUser);

   if (radioGroups->Checked)
      ;
   else if (radioUser->Checked)
      lookupTables->UnregisterComboBox("users", cboUser);
  */
  
  TformRecordGeneric::ScreenStop();
  return true;
}
//---------------------------------------------------------------------------
bool TformRecordIncomingBill::IsValidOnScreen()
{
  // check to make sure that the values entered in the combo boxe
  // corrospond to values in that look up table.
  // check to make sure the dates entered are valid
  // give the user a message box and prompt to re-enter if incorrect

  string actual_tax_id   = GetValidLinkId(txtVendor);
  string provider_tax_id = GetValidLinkId(txtProvider);
  string facility_tax_id = GetValidLinkId(txtFacility);
  string group_tax_id    = GetValidLinkId(txtGroup);

  if((initial_state["bill_id"] == lookupTables->IdOf("bill_types", "Incoming provider bill")) &&
    ((! radioProvider->Checked) && (! radioFacility->Checked) && (! radioGroup->Checked)))
  {
    PSay("Please select the primary tax entity (provider, facility, or group)", m_missing_data, MB_OK);
    radioProvider->SetFocus();
    return false;
  }
  else if(radioProvider->Checked && provider_tax_id == "")
  {
    PSay("Please select a provider (disabled -- please ask a programmer for help)", m_missing_data, MB_OK);
    // btnProviderSearch->SetFocus();
    return false;
  }
  else if(radioFacility->Checked && facility_tax_id == "")
  {
    PSay("Please select a facility (disabled -- please ask a programmer for help)", m_missing_data, MB_OK);
    // btnFacilitySearch->SetFocus();
    return false;
  }
  else if(radioGroup->Checked && group_tax_id == "")
  {
    PSay("Please select a group (disabled -- please ask a programmer for help)", m_missing_data, MB_OK);
    // btnGroupSearch->SetFocus();
    return false;
  }
  else if (! IsAmount(editInvoiceAmount->Text.c_str()))
  {
    PSay("Please enter a valid invoice amount", m_invalid_data, MB_OK);
    editInvoiceAmount->SetFocus();
    return false;
  }
  else if (! IsAmount(editTotalCharge->Text.c_str()))
  {
    PSay("Please enter a valid total charge", m_invalid_data, MB_OK);
    editTotalCharge->SetFocus();
    return false;
  }
  else if (! IsAmount(editPaidAmount->Text.c_str()))
  {
    PSay("Please enter a valid paid amount", m_invalid_data, MB_OK);
    editPaidAmount->SetFocus();
    return false;
  }
  else if (UppercaseString(lookupTables->ValueOf("current_types", ending_state["current_id"])) == "DELETED")
  {
    int response;
    response = PSay(
               "Marking this bill as deleted will have no effect on the transactions attached to it.\n"
               "Marking the transactions as deleted will also have no effect.\n"
               "Please use the 'modify transactions' link at the bottom-left of this screen to make\n"
               "appropriate changes, such as reversing existing transactions. All non-zero totals\n"
               "shown in the 'summary' screen will be reflected everywhere else in Prometheus.\n"
               "Are you sure you want to continue anyway?", m_warning, MB_YESNO);
    if(response == IDNO)
       return false;
  }
  else if (actual_tax_id != "" &&
           ((radioProvider->Checked && provider_tax_id != actual_tax_id) ||
            (radioFacility->Checked && facility_tax_id != actual_tax_id) ||
            (radioGroup->Checked    && group_tax_id    != actual_tax_id)))
  {
    if (IDNO == PSay("You have changed who is to be paid for this bill.\n"
                     "If transactions have already been created for this bill, they will not be modified.\n"
                     "You must make sure that the transactions on this bill are as you would expect them\n"
                     "to be after your changes. If they are not, please use the 'modify transactions' link\n"
                     "at the bottom-left of this screen to make appropriate changes.\n"
                     "Are you sure you want to continue anyway?", m_warning, MB_YESNO))
    return false;
  }
  /*
  else if ((radioUser->Checked) && (! lookupTables->IsInLookUpTable("users", cboUser->Text.c_str())))
  {
    PSay("Please enter a valid employee", m_invalid_data, MB_OK);
    cboUser->SetFocus();
    return false;
  }
  */

  return true;
}
//---------------------------------------------------------------------------

bool TformRecordIncomingBill::PutDataOnScreen()
{
  // put the values on the form from the map initial_state
  // values may have come from GetDataFromDatabase if it's an existing record,
  // or values may have been passed to the form in this map if it's a new record

  // cboUser->Text = PrettyString(lookupTables->ValueOf("users", initial_state["user_id"])).c_str();
  if(initial_state["bill_date"] != "")
    dateReceived->Date = Pdate(initial_state["bill_date"], mdy | yyyy).OutDate();
  else
    dateReceived->Date = Date();
  editInvoiceAmount->Text = AmountToString(StringToAmount(initial_state["invoice_amount"])).c_str();
  editTotalCharge->Text = AmountToString(StringToAmount(initial_state["total_charge"])).c_str();
  editPaidAmount->Text = AmountToString(StringToAmount(initial_state["paid_amount"])).c_str();
  editNote->Text = SentenceString(initial_state["note"]).c_str();
  editInvoiceNumber->Text = SentenceString(initial_state["invoice_number"]).c_str();

  if(initial_state["bill_id"] == lookupTables->IdOf("bill_types", "Incoming vendor bill"))
  {
    radioVendor->Checked = true;
    radioProvider->Checked = false;
    radioFacility->Checked = false;
    radioGroup->Checked = false;
    radioProvider->Enabled = false;
    radioFacility->Enabled = false;
    radioGroup->Enabled = false;
    txtProvider->Enabled = false;
    txtFacility->Enabled = false;
    txtGroup->Enabled = false;
    btnProviderSearch->Enabled = false;
    btnFacilitySearch->Enabled = false;
    btnGroupSearch->Enabled = false;

    CreateLink(txtVendor, btnVendorSearch, this, initial_state["actual_tax_id"], "tax_entities");
    if((initial_state["purchase_req_id"] != "0") && (initial_state["purchase_req_id"] != ""))
      btnVendorSearch->Enabled = false;
  }
  else if(initial_state["bill_id"] == lookupTables->IdOf("bill_types", "Incoming provider bill"))
  {
    radioVendor->Checked = false;
    radioVendor->Enabled = false;
    txtVendor->Enabled = false;
    btnVendorSearch->Enabled = false;

    CreateLink(txtProvider, btnProviderSearch, this, initial_state["provider_tax_id"], "tax_entities");
    CreateLink(txtFacility, btnFacilitySearch, this, initial_state["facility_tax_id"], "tax_entities");
    CreateLink(txtGroup, btnGroupSearch, this, initial_state["group_tax_id"], "tax_entities");
    if(initial_state["provider_tax_id"] == initial_state["actual_tax_id"])
      radioProvider->Checked = true;
    else if(initial_state["facility_tax_id"] == initial_state["actual_tax_id"])
      radioFacility->Checked = true;
    if(initial_state["group_tax_id"] == initial_state["actual_tax_id"])
      radioGroup->Checked = true;

    if((initial_state["purchase_req_id"] != "0") && (initial_state["purchase_req_id"] != ""))
    {
      btnProviderSearch->Enabled = false;
      btnFacilitySearch->Enabled = false;
      btnGroupSearch->Enabled = false;
    }
    // lblUser->Enabled = false;
    // cboUser->Enabled = false;
  }
  // CreateLink(txtPr, NULL, this, initial_state["provider_tax_id"], "tax_entities");
  CreateLink(txtPr, NULL, this, initial_state["purchase_req_id"], "purchase_reqs");
  CreateLink(txtVisit, NULL, this, initial_state["task_id"], "tasks");

  /*
  string group_or_user = initial_state["group_or_user"];
  if (group_or_user == "0")
  {
     cboUser->Clear();
     cboUser->Items->Add(initial_state["groups"].c_str());
     cboUser->ItemIndex = cboUser->Items->Count - 1;
     radioGroups->Checked = true;
     radioClick(NULL);
  }
  else if ((group_or_user == "1") || (group_or_user == ""))
  {
     cboUser->Text = lookupTables->ValueOf("users", initial_state["user_id"]).c_str();
     radioUser->Checked = true;
  }
  */
  
  TformRecordGeneric::PutDataOnScreen();
  return true;
}
//---------------------------------------------------------------------------
bool TformRecordIncomingBill::GetDataFromScreen()
{
  // place the values currently on the screen into
  // the map that will be used to update or insert the record
  // the parent form will call the insert or update function, and get the id
  // number if necessary and take care of current_id

  ending_state["bill_date"] =  Pdate(dateReceived->Date).OutDate(mdy | yyyy);
  ending_state["invoice_amount"] = editInvoiceAmount->Text.c_str();
  ending_state["total_charge"] = editTotalCharge->Text.c_str();
  ending_state["paid_amount"] = editPaidAmount->Text.c_str();
  ending_state["note"] = editNote->Text.c_str();
  ending_state["invoice_number"] = editInvoiceNumber->Text.c_str();

  string actual_tax_id = GetValidLinkId(txtVendor);
  if (actual_tax_id == "")
    ending_state["actual_tax_id"] = "0";
  else
    ending_state["actual_tax_id"] = actual_tax_id;
  if (initial_state["purchase_req_id"] == "")
    ending_state["purchase_req_id"] = "0";
  else
    ending_state["purchase_req_id"] = initial_state["purchase_req_id"];

  string provider_tax_id = GetValidLinkId(txtProvider);
  if (provider_tax_id == "")
    ending_state["provider_tax_id"] = "0";
  else
    ending_state["provider_tax_id"] = provider_tax_id;

  string group_tax_id = GetValidLinkId(txtGroup);
  if (group_tax_id == "")
    ending_state["group_tax_id"] = "0";
  else
    ending_state["group_tax_id"] = group_tax_id;

  string facility_tax_id = GetValidLinkId(txtFacility);
  if (facility_tax_id == "")
    ending_state["facility_tax_id"] = "0";
  else
    ending_state["facility_tax_id"] = facility_tax_id;

  if (initial_state["task_id"] == "")
    ending_state["task_id"] = "0";
  else
    ending_state["task_id"] = initial_state["task_id"];

  if (initial_state["insurance_coverage_tax_id"] == "")
    ending_state["insurance_coverage_id"] = "0";
  else
    ending_state["insurance_coverage_id"] = initial_state["insurance_coverage_id"];

  if(initial_state["bill_id"] == lookupTables->IdOf("bill_types", "Incoming provider bill"))
  {
    if(radioProvider->Checked)
      ending_state["actual_tax_id"] = ending_state["provider_tax_id"];
    else if(radioFacility->Checked)
      ending_state["actual_tax_id"] = ending_state["facility_tax_id"];
    else if(radioGroup->Checked)
      ending_state["actual_tax_id"] = ending_state["group_tax_id"];
  }
  ending_state["bill_id"] = initial_state["bill_id"];

  ending_state["id"] = initial_state["id"];

  ending_state["user_id"] = initial_state["user_id"]; // lookupTables->IdOf("users", cboUser->Text.c_str());
  ending_state["group_or_user"] = initial_state["group_or_user"];
  ending_state["groups"] = initial_state["groups"];
  /*
  if (radioGroups->Checked)
  {
     ending_state["group_or_user"] = "0";
     ending_state["groups"] = cboUser->Text.c_str();
     ending_state["user_id"] = "0";
  }
  else if (radioUser->Checked)
  {
     ending_state["group_or_user"] = "1";
     ending_state["user_id"] = lookupTables->IdOf("users", cboUser->Text.c_str());
     ending_state["groups"] = "";
  }
  */
  
  TformRecordGeneric::GetDataFromScreen();

  return true; // again, not much can go wrong here
}
//---------------------------------------------------------------------------
bool TformRecordIncomingBill::ScreenLock()
{
  // disable panel containing edit boxes
  panelGroup->Enabled = false;
  TformRecordGeneric::ScreenLock();
  return true;
}
//---------------------------------------------------------------------------
bool TformRecordIncomingBill::ScreenUnlock()
{
  // enable panel containing edit boxes
  panelGroup->Enabled = true;
  TformRecordGeneric::ScreenUnlock();
  return true;
}
//---------------------------------------------------------------------------
bool TformRecordIncomingBill::NotNewAnymore()
{
  // jhaley 20100910: Use transactions summary object
  transactions_summary.SetSQLStatement("select * from transactions where bill_id = " + initial_state["id"]);
  transactions_summary.SetRecord("bills", initial_state["id"], "bill_id", RefreshHandler(&(TformRecordIncomingBill::AtlasRefreshListView)));

  payments_filter.SetSQLStatement("select procedures.cpt_hcpcs, procedure_payments.amount_charged, procedure_payments.amount_paid, procedure_payments.current_id, procedure_payments.id from procedure_payments left join procedures on procedures.id = procedure_payments.procedure_id where procedure_payments.bill_id = " + initial_state["id"]);
  payments_filter.RefreshDataSet();
  payments_filter.SetAddEdit(this, NULL /*AddRecord(&(TformRecordIncomingBill::AddPayment))*/,
                               EditRecord(&(TformRecordIncomingBill::EditPayment)));
  payment_request = atlas->RequestNotificationWhen("procedure_payments.bill_id", initial_state["id"], rel_e, as_int, this,
                     RefreshHandler(&(TformRecordIncomingBill::AtlasRefreshListView)));

  staticTransactionDetails->Enabled = true;
  staticBetaModifyTransactions->Enabled = true;

  if(initial_state["bill_id"] == lookupTables->IdOf("bill_types", "Incoming provider bill"))
   staticPrintEOB->Enabled = true;

  TformRecordGeneric::NotNewAnymore();
  return true;
}
//---------------------------------------------------------------------------
void TformRecordIncomingBill::AtlasRefreshListView(int request_num, string matched_value)
{
  if (request_num == payment_request)
    payments_filter.RefreshDataSet();
  else
    transactions_summary.AtlasRefresh(request_num, matched_value); // jhaley: check transaction summary
}
//---------------------------------------------------------------------------
void TformRecordIncomingBill::EditTransaction()
{
  /*
  map<string, string> temp_map;
  temp_map["id"] = transactions_filter.GetSelectedRecordID();
  FORM_TRY
    TformRecordTransaction * view_transaction = new TformRecordTransaction(NULL, temp_map);
    view_transaction->Show();
  FORM_CATCH
  */
}
//---------------------------------------------------------------------------
bool TformRecordIncomingBill::IsNewAgain()
{
  // jhaley 20100910: Use transactions summary object
  transactions_summary.Clear();

  payments_filter.SetSQLStatement(string(""));
  payments_filter.RefreshDataSet();
  payments_filter.SetAddEdit(this, NULL, NULL);

  staticTransactionDetails->Enabled = false;
  staticBetaModifyTransactions->Enabled = false;
  staticPrintEOB->Enabled = false;

  TformRecordGeneric::IsNewAgain();
  return true;
}
//---------------------------------------------------------------------------
void __fastcall TformRecordIncomingBill::btnVendorSearchClick(TObject *Sender)
{
  // create and show new person search form
  vector<string> return_vector;
  FORM_TRY
    TformSearchTaxEntity * providerSearch = new TformSearchTaxEntity(NULL,&return_vector);
    providerSearch->ShowTaskModal(this);
  FORM_CATCH
  if (! return_vector.empty())
    CreateLink(txtVendor, btnVendorSearch, this, return_vector[0], "tax_entities");
}
//---------------------------------------------------------------------------
void __fastcall TformRecordIncomingBill::btnProviderSearchClick(TObject *Sender)
{
  // create and show new person search form
  vector<string> return_vector;
  FORM_TRY
    TformSearchTaxEntity * providerSearch = new TformSearchTaxEntity(NULL,&return_vector);
    providerSearch->ShowTaskModal(this);
  FORM_CATCH
  if (! return_vector.empty())
    CreateLink(txtProvider, btnProviderSearch, this, return_vector[0], "tax_entities");
}
//---------------------------------------------------------------------------
void __fastcall TformRecordIncomingBill::btnFacilitySearchClick(TObject *Sender)
{
  // create and show new person search form
  vector<string> return_vector;
  FORM_TRY
    TformSearchTaxEntity * providerSearch = new TformSearchTaxEntity(NULL,&return_vector);
    providerSearch->ShowTaskModal(this);
  FORM_CATCH
  if (! return_vector.empty())
    CreateLink(txtFacility, btnFacilitySearch, this, return_vector[0], "tax_entities");
}
//---------------------------------------------------------------------------
void __fastcall TformRecordIncomingBill::btnGroupSearchClick(TObject *Sender)
{
  // create and show new person search form
  vector<string> return_vector;
  FORM_TRY
    TformSearchTaxEntity * providerSearch = new TformSearchTaxEntity(NULL,&return_vector);
    providerSearch->ShowTaskModal(this);
  FORM_CATCH
  if (! return_vector.empty())
    CreateLink(txtGroup, btnGroupSearch, this, return_vector[0], "tax_entities");
}
//---------------------------------------------------------------------------
void __fastcall TformRecordIncomingBill::staticModifyAccPayableClick(TObject *Sender)
{
 /* Deprecated!  Woo!
 FORM_TRY
   TformSpecialModifyPayable * edit_payable = new TformSpecialModifyPayable(NULL, initial_state["id"], "");
   edit_payable->Show();
 FORM_CATCH
 */
}
//---------------------------------------------------------------------------
void __fastcall TformRecordIncomingBill::radioVendorClick(TObject *Sender)
{
   // panelRadio->Enabled = true;
}
//---------------------------------------------------------------------------
void __fastcall TformRecordIncomingBill::radioOtherClick(TObject *Sender)
{
   // panelRadio->Enabled = false;
}
//---------------------------------------------------------------------------
void __fastcall TformRecordIncomingBill::menuAddByGroupsClick(TObject *Sender)
{
/*
  set<string> return_set;
  FORM_TRY
    TformSpecialSelectGroups * select_group_form = new TformSpecialSelectGroups(NULL, NULL, &return_set);
    select_group_form->ShowTaskModal(this);
  FORM_CATCH
  if(! return_set.empty())
  {
    string comma_sep = "";
    set<string>::iterator itr;
    for (itr = return_set.begin(); itr != return_set.end(); itr++)
    {
      comma_sep = comma_sep + (*itr) + ",";
    }
    comma_sep = comma_sep.substr(0, comma_sep.size() - 1);
    string sql = "select short_item from groups where id in (" + comma_sep + ");";
    vector<map<string, string> > names;
    SqlToVecMap(dbDatabase, sql, names);
    string departments = "";
    for (unsigned int i = 0; i != names.size(); i++)
    {
       departments = departments + names[i]["short_item"].c_str() + ", ";
    }
    cboUser->Clear();
    cboUser->Items->Add(departments.substr(0, departments.size() - 2).c_str());
    cboUser->ItemIndex = cboUser->Items->Count - 1;
  }
  else
     cboUser->Text = "";
*/
}
//---------------------------------------------------------------------------
void __fastcall TformRecordIncomingBill::radioClick(TObject *Sender)
{
/*
   int currentRadioChecked;
   if (radioGroups->Checked)
      currentRadioChecked = 0;
   else if (radioUser->Checked)
      currentRadioChecked = 1;
   if (currentRadioChecked != lastRadioChecked)
   {
      if (radioGroups->Checked)
      {
         lookupTables->UnregisterComboBox("users", cboUser);
         cboUser->Style = Stdctrls::csDropDownList;
         cboUser->Clear();
         cboUser->Items->Add("Right click to add departments");
         cboUser->ItemIndex = cboUser->Items->Count - 1;
         cboUser->PopupMenu = popupGroups;
         lblUser->Caption = "Department(s)";
      }
      else if (radioUser->Checked)
      {
         cboUser->Style = Stdctrls::csDropDown;
         cboUser->Clear();
         lookupTables->RegisterComboBox("users", cboUser);
         lblUser->Caption = "Employee";
      }
   }
   lastRadioChecked = currentRadioChecked;
*/
}
//---------------------------------------------------------------------------
void __fastcall TformRecordIncomingBill::staticBetaModifyTransactionsClick(TObject *Sender)
{
 if (optionsPermissions["table.transactions"]["insert"] == "true")
 {
  FORM_TRY
   TformWizardModifyTransactions * wiz = new TformWizardModifyTransactions(NULL, initial_state["id"], mod_in_bill);
   wiz->Show();
  FORM_CATCH
 }
}
//---------------------------------------------------------------------------
void __fastcall TformRecordIncomingBill::staticTransactionDetailsClick(TObject *Sender)
{
  // jhaley 20100910: Launch transaction details form when link is clicked
  if (optionsPermissions["table.transactions"]["view"] == "true")
  {
    string sql = "select tax_entities.name, transactions.effective_date, transactions.fiscal_date, "
                 "accounts.name, account_types.item, "
                 "transactions.note, transactions.debit_amount, transactions.credit_amount, "
                 "transactions.transaction_number, transactions.current_id, transactions.id "
                 "from transactions "
                 "inner join accounts on transactions.account_id = accounts.id "
                 "inner join account_types on accounts.account_type_id = account_types.id "
                 "left join tax_entities on transactions.tax_entity_id = tax_entities.id "
                 "where transactions.bill_id = " + initial_state["id"];

    TformSpecialTransactionDetails *details = NULL;
    FORM_TRY
      details = new TformSpecialTransactionDetails(NULL, initial_state["id"], "bill_id");
    FORM_CATCH
    details->Filter->AddColumnSpec("Tax Entity",  "tax_entities", as_text,   true,  120, rawString);
    details->Filter->AddColumnSpec("Eff. Date",   "transactions", as_date,   true,   75);
    details->Filter->AddColumnSpec("Fisc. Date",  "transactions", as_date,   true,   75);
    details->Filter->AddColumnSpec("Account",     "accounts",     as_text,   true,  140);
    details->Filter->AddColumnSpec("Type",        "accounts",     as_text,   true,   75);
    details->Filter->AddColumnSpec("Note",        "transactions", as_text,   false, 120, sentenceString);
    details->Filter->AddColumnSpec("Debit",       "transactions", as_double, true,   75, nonzeroAmountString);
    details->Filter->AddColumnSpec("Credit",      "transactions", as_double, true,   75, nonzeroAmountString);
    details->Filter->AddColumnSpec("Trans #",     "transactions", as_int,    true,   75);
    details->Filter->AddColumnSpec("Current Id",  "current_id",   as_int,    false,  80);
    details->Filter->AddColumnSpec("Internal Id", "internal_id",  as_int,    false,  80);
    details->Filter->FetchPreferences("recordincomingbill.transactions_filter");
    details->Execute(sql);
    details->Show();
  }
}
//---------------------------------------------------------------------------
void TformRecordIncomingBill::AddPayment()
{
  map<string,string> temp_map;
  temp_map["id"] = "new";
  temp_map["bill_id"] = initial_state["id"];
  FORM_TRY
    TformRecordProcedurePayment * pay = new TformRecordProcedurePayment(NULL, temp_map);
    pay->Show();
  FORM_CATCH
}
//---------------------------------------------------------------------------
void TformRecordIncomingBill::EditPayment()
{
  map<string,string> temp_map;
  temp_map["id"] = payments_filter.GetSelectedRecordID();
  FORM_TRY
    TformRecordProcedurePayment * pay = new TformRecordProcedurePayment(NULL, temp_map);
    pay->Show();
  FORM_CATCH
}
//---------------------------------------------------------------------------
void __fastcall TformRecordIncomingBill::staticPrintEOBClick(TObject *Sender)
{
 // print just this one EOB
 Processing(true);

 TIBTransaction * dbTransaction = new TIBTransaction(NULL);
 TIBDataSet * eobDataset = new TIBDataSet(NULL);

 try
 {
  StdTransaction(dbTransaction, dbDatabase);
  eobDataset->Database = dbDatabase;
  eobDataset->Transaction = dbTransaction;
  eobDataset->UniDirectional = false;

  string sql_get_eob =
   "select bills.id, "
   "affiliations.cdib, person_names.last_name || ', ' || person_names.first_name || ' ' || person_names.suffix name, tasks.patient_control_number, "
   "procedures.start_date, procedures.cpt_hcpcs code, procedures.charges charged, procedure_payments.amount_charged billed, procedure_payments.amount_allowed allowed, procedure_payments.amount_paid paid "
   "from bills "
   "left join tasks on tasks.id = bills.task_id "
   "left join procedures on procedures.task_id = tasks.id "
   "left join procedure_payments on procedure_payments.procedure_id = procedures.id "
   "left join people on tasks.person_id = people.id "
   "left join person_names on people.person_names_id = person_names.id "
   "left join affiliations on people.affiliations_id = affiliations.id "
   "where bills.id = " + initial_state["id"];
  eobDataset->SelectSQL->Add(sql_get_eob.c_str());
  eobDataset->Open();
  eobDataset->First();

  FORM_TRY
   TformReportBulkEOB * report = new TformReportBulkEOB(NULL);
   report->qpBulkEOB->DataSet = eobDataset;
   report->_labelProviderPaid->Caption = "";
   report->qrpProviderName->Caption = "";
   report->_labelPRNumber->Caption = "";
   report->qrpPR->Caption = "";
   report->lblUser->Caption = PrettyString(lookupTables->ValueOf("users", current_userid)).c_str();
   report->qrpCDIB->DataSet = eobDataset;
   report->qrpCDIB->DataField = "cdib";
   report->qrpName->DataSet = eobDataset;
   report->qrpName->DataField = "name";
   report->qrpPatientAccountNum->DataSet = eobDataset;
   report->qrpPatientAccountNum->DataField = "patient_control_number";
   report->qrpDate->DataSet = eobDataset;
   report->qrpDate->DataField = "start_date";
   report->qrpProcedure->DataSet = eobDataset;
   report->qrpProcedure->DataField = "code";
   report->bandGroup->Expression = "id";
   Processing(false);
   report->qpBulkEOB->Preview();
  FORM_CATCH
  eobDataset->Close();
 }
 catch(...)
 {
  Processing(false);
  PSay("Error printing EOB", m_database_error, MB_OK);
 }

 if(dbTransaction->Active)
  dbTransaction->Rollback();

 delete eobDataset;
 delete dbTransaction;
}
//---------------------------------------------------------------------------
void __fastcall TformRecordIncomingBill::FormClose(TObject *Sender, TCloseAction &Action)
{
  // jhaley 20100910: Shut down transactions summary object
  TformRecordGeneric::FormClose(Sender, Action);
  transactions_summary.Close(Action);
}
//---------------------------------------------------------------------------
// jhaley 20100910: Checkbox added to control transaction summary account grouping
void __fastcall TformRecordIncomingBill::checkBeforeAccountClick(TObject *Sender)
{
  transactions_summary.SetShowAccount(checkBeforeAccount->Checked); // set state
  transactions_summary.CheckBoxClicked();                           // update it.
}
//---------------------------------------------------------------------------
// jhaley 20100910: Checkbox added to control transaction summary tax entity grouping
void __fastcall TformRecordIncomingBill::checkBeforeNameClick(TObject *Sender)
{
  transactions_summary.SetShowName(checkBeforeName->Checked); // set state
  transactions_summary.CheckBoxClicked();                     // update it.
}
//---------------------------------------------------------------------------

