//---------------------------------------------------------------------------

#include <vcl.h>
#pragma hdrstop

#include "codeRecordInsuranceApplication.h"
#include "codeSearchTaxEntity.h"
#include "codeSearchInsurancePolicy.h"
//---------------------------------------------------------------------------
#pragma package(smart_init)
#pragma link "codeRecordGeneric"
#pragma resource "*.dfm"
TformRecordInsuranceApplication *formRecordInsuranceApplication;
//---------------------------------------------------------------------------
__fastcall TformRecordInsuranceApplication::TformRecordInsuranceApplication(TComponent* Owner, map<string, string> & p_parameters)
        : TformRecordGeneric(Owner, p_parameters, string("insurance_applications"))
{
  fields_to_leave_raw.insert("notes");
  Initialize();
}
//---------------------------------------------------------------------------
bool TformRecordInsuranceApplication::ScreenStart()
{
  lookupTables->RegisterComboBox("insurance_app_types", cboApplications);
  lookupTables->RegisterComboBox("users", cboMailedBy);
  lookupTables->RegisterComboBox("relationship_types", cboCompletedBy);
  lookupTables->RegisterComboBox("users", cboFaxedBy);
  lookupTables->RegisterComboBox("insurance_app_location_types", cboReceivedAt);
  lookupTables->RegisterComboBox("insurance_app_denial_types", cboDenied);
  TformRecordGeneric::ScreenStart();
  return true;
}
//---------------------------------------------------------------------------
bool TformRecordInsuranceApplication::ScreenStop()
{
  lookupTables->UnregisterComboBox("insurance_app_types", cboApplications);
  lookupTables->UnregisterComboBox("users", cboMailedBy);
  lookupTables->UnregisterComboBox("relationship_types", cboCompletedBy);
  lookupTables->UnregisterComboBox("users", cboFaxedBy);
  lookupTables->UnregisterComboBox("insurance_app_location_types", cboReceivedAt);
  lookupTables->UnregisterComboBox("insurance_app_denial_types", cboDenied);
  TformRecordGeneric::ScreenStop();
  return true;
}
//---------------------------------------------------------------------------
bool TformRecordInsuranceApplication::IsValidOnScreen()
{
  if(!lookupTables->IsInLookUpTable("insurance_app_types", cboApplications))
  {
    PSay("Please pick an application type", m_invalid_data, MB_OK);
    cboApplications->SetFocus();
    return false;
  }
  else if (checkMailed->Checked && (!Pdate (editDateMailed->Text.c_str(), mdy | yy).IsValidDate() || editDateMailed->Text == ""))
  {
    PSay("Please enter the date the application was mailed.", m_invalid_data, MB_OK);
    editDateMailed->SetFocus();
    return false;
  }
  else if(checkMailed->Checked && (cboMailedBy->Text == "" || !lookupTables->IsInLookUpTable("users", cboMailedBy)))
  {
    PSay("Please indicate who mailed the application.", m_invalid_data, MB_OK);
    cboMailedBy->SetFocus();
    return false;
  }
  else if(checkCompleted->Checked && (cboCompletedBy->Text == "" || !lookupTables->IsInLookUpTable("relationship_types", cboCompletedBy)))
  {
    PSay("Please indicate who completed the application.  If it was the patient him/her-self, select \"Self\".", m_invalid_data, MB_OK);
    cboCompletedBy->SetFocus();
    return false;
  }
  else if (checkReceived->Checked && (!Pdate (editDateReceived->Text.c_str(), mdy | yy).IsValidDate() || editDateReceived->Text == ""))
  {
    PSay("Please enter the date the application was received.", m_invalid_data, MB_OK);
    editDateReceived->SetFocus();
    return false;
  }
  else if(checkReceived->Checked && (cboReceivedAt->Text == "" || !lookupTables->IsInLookUpTable("insurance_app_location_types", cboReceivedAt)))
  {
    PSay("Please where the application was received.", m_invalid_data, MB_OK);
    cboReceivedAt->SetFocus();
    return false;
  }
  else if (checkFaxed->Checked && (!Pdate (editDateFaxed->Text.c_str(), mdy | yy).IsValidDate() || editDateFaxed->Text == ""))
  {
    PSay("Please enter the date the application was faxed.", m_invalid_data, MB_OK);
    editDateFaxed->SetFocus();
    return false;
  }
  else if(checkFaxed->Checked && (cboFaxedBy->Text == "" || !lookupTables->IsInLookUpTable("users", cboFaxedBy)))
  {
    PSay("Please indicate who faxed the application.", m_invalid_data, MB_OK);
    cboFaxedBy->SetFocus();
    return false;
  }
  else if (checkFaxed->Checked && GetValidLinkId(txtFaxedTo) == "")
  {
    PSay("Please specify to whom you sent the fax.", m_missing_data, MB_OK);
    return false;
  }
  else if (checkApproved->Checked && GetValidLinkId(txtPolicy) == "")
  {
    PSay("Please link to the approved policy.  You can search for an existing policy, and if it is not found, click \"New\" at the bottom to create it (at which point you can search for and Select it).", m_missing_data, MB_OK);
    return false;
  }
  else if(checkDenied->Checked && (cboDenied->Text == "" || !lookupTables->IsInLookUpTable("insurance_app_denial_types", cboDenied)))
  {
    PSay("Please pick a reason the application was denied from the list.", m_invalid_data, MB_OK);
    cboCompletedBy->SetFocus();
    return false;
  }
  else
    return true;
}
//---------------------------------------------------------------------------
bool TformRecordInsuranceApplication::PutDataOnScreen()
{
  CreateLink(txtPerson, NULL, this, initial_state["person_id"], "people");
  cboApplications->Text = lookupTables->ValueOf("insurance_app_types", initial_state["insurance_app_id"]).c_str();
  checkMailed->Checked = initial_state["is_mailed"] == "1";
  editDateMailed->Text = initial_state["mailed_on"].c_str();
  cboMailedBy->Text = lookupTables->ValueOf("users", initial_state["mailed_by"]).c_str();
  checkCompleted->Checked = initial_state["is_completed"] == "1";
  cboCompletedBy->Text = lookupTables->ValueOf("relationship_types", initial_state["completed_by"]).c_str();
  checkReceived->Checked = initial_state["is_received"] == "1";
  editDateReceived->Text = initial_state["received_on"].c_str();
  cboReceivedAt->Text = lookupTables->ValueOf("insurance_app_location_types", initial_state["received_at"]).c_str();
  checkFaxed->Checked = initial_state["is_faxed"] == "1";
  editDateFaxed->Text = initial_state["faxed_on"].c_str();
  cboFaxedBy->Text = lookupTables->ValueOf("users", initial_state["faxed_by"]).c_str();
  CreateLink(txtFaxedTo, btnFaxedToSearch, this, initial_state["faxed_to"], "tax_entities");
  checkApproved->Checked = initial_state["is_approved"] == "1";
  CreateLink(txtPolicy, btnPolicySearch, this, initial_state["insurance_policy_id"], "insurance_policies");
  checkDenied->Checked = initial_state["is_denied"] == "1";
  cboDenied->Text = lookupTables->ValueOf("insurance_app_denial_types", initial_state["insurance_app_denial_id"]).c_str();
  editNotes->Text = initial_state["notes"].c_str();
  checkChanged(NULL);
  TformRecordGeneric::PutDataOnScreen();
  return true;
}
//---------------------------------------------------------------------------
bool TformRecordInsuranceApplication::GetDataFromScreen()
{
  ending_state["id"] = initial_state["id"];
  ending_state["person_id"] = initial_state["person_id"];
  ending_state["insurance_app_id"] = lookupTables->IdOf("insurance_app_types", cboApplications);
  ending_state["is_mailed"] = BoolToBoolString(checkMailed->Checked, zero_one);
  if (checkMailed->Checked)
  {
    ending_state["mailed_on"] = Pdate(editDateMailed->Text.c_str(), mdy | yy).OutDate(mdy | yyyy);
    ending_state["mailed_by"] = lookupTables->IdOf("users", cboMailedBy);
  }
  else
  {
    ending_state["mailed_on"] = "";
    ending_state["mailed_by"] = "0";
  }
  ending_state["is_completed"] = BoolToBoolString(checkCompleted->Checked, zero_one);
  if (checkCompleted->Checked)
    ending_state["completed_by"] = lookupTables->IdOf("relationship_types", cboCompletedBy);
  else
    ending_state["completed_by"] = "0";
  ending_state["is_received"] = BoolToBoolString(checkReceived->Checked, zero_one);
  if (checkReceived->Checked)
  {
    ending_state["received_on"] = Pdate(editDateReceived->Text.c_str(), mdy | yy).OutDate(mdy | yyyy);
    ending_state["received_at"] = lookupTables->IdOf("insurance_app_location_types", cboReceivedAt);
  }
  else
  {
    ending_state["received_on"] = "";
    ending_state["received_at"] = "0";
  }
  ending_state["is_faxed"] = BoolToBoolString(checkFaxed->Checked, zero_one);
  if (checkFaxed->Checked)
  {
    string faxed_to_id = GetValidLinkId(txtFaxedTo);
    ending_state["faxed_on"] = Pdate(editDateFaxed->Text.c_str(), mdy | yy).OutDate(mdy | yyyy);
    ending_state["faxed_by"] = lookupTables->IdOf("users", cboFaxedBy);
    ending_state["faxed_to"] = faxed_to_id != "" ? faxed_to_id : string("0"); // shouldn't even BE blank
  }
  else
  {
    ending_state["faxed_on"] = "";
    ending_state["faxed_by"] = "0";
    ending_state["faxed_to"] = "0";
  }
  ending_state["is_approved"] = BoolToBoolString(checkApproved->Checked, zero_one);
  if (checkApproved->Checked)
  {
    string policy_id = GetValidLinkId(txtPolicy);
    ending_state["insurance_policy_id"] = policy_id != "" ? policy_id : string("0"); // shouldn't even BE blank
  }
  else
    ending_state["insurance_policy_id"] = "";
  ending_state["is_denied"] = BoolToBoolString(checkDenied->Checked, zero_one);
  if (checkDenied->Checked)
    ending_state["insurance_app_denial_id"] = lookupTables->IdOf("insurance_app_denial_types", cboDenied);
  else
    ending_state["insurance_app_denial_id"] = "0";
  ending_state["notes"] = editNotes->Text.c_str();
  TformRecordGeneric::GetDataFromScreen();
  return true;
}
//---------------------------------------------------------------------------
bool TformRecordInsuranceApplication::ScreenLock()
{
  panelGroup->Enabled = false;
  TformRecordGeneric::ScreenLock();
  return true;
}
//---------------------------------------------------------------------------
bool TformRecordInsuranceApplication::ScreenUnlock()
{
  panelGroup->Enabled = true;
  TformRecordGeneric::ScreenUnlock();
  return true;
}
//---------------------------------------------------------------------------
bool TformRecordInsuranceApplication::CheckForUselessRecord()
{
  return true;
}
//---------------------------------------------------------------------------
void TformRecordInsuranceApplication::Do(map<string, string> & p_parameters)
{
 FORM_TRY
  TformRecordInsuranceApplication * z = new TformRecordInsuranceApplication(NULL, p_parameters);
  z->Show();
 FORM_CATCH
}
//---------------------------------------------------------------------------
void __fastcall TformRecordInsuranceApplication::btnFaxedToSearchClick(TObject *Sender)
{
  // create and show new person search form
  vector<string> return_vector;
  FORM_TRY
    TformSearchTaxEntity * providerSearch = new TformSearchTaxEntity(NULL,&return_vector);
    providerSearch->ShowTaskModal(this);
  FORM_CATCH
  if (! return_vector.empty())
    CreateLink(txtFaxedTo, btnFaxedToSearch, this, return_vector[0], "tax_entities");
}
//---------------------------------------------------------------------------
void __fastcall TformRecordInsuranceApplication::btnPolicySearchClick(TObject *Sender)
{
  // create and show new person search form
  vector<string> return_vector;
  FORM_TRY
    TformSearchInsurancePolicy * policySearch = new TformSearchInsurancePolicy(NULL,&return_vector);
    policySearch->ShowTaskModal(this);
  FORM_CATCH
  if (! return_vector.empty())
    CreateLink(txtPolicy, btnPolicySearch, this, return_vector[0], "insurance_policies");
}
//---------------------------------------------------------------------------
void __fastcall TformRecordInsuranceApplication::checkChanged(TObject *Sender)
{
  panelMailed->Enabled = checkMailed->Checked;
  panelCompleted->Enabled = checkCompleted->Checked;
  panelReceived->Enabled = checkReceived->Checked;
  panelFaxed->Enabled = checkFaxed->Checked;
  panelApproved->Enabled = checkApproved->Checked;
  panelDenied->Enabled = checkDenied->Checked;
  /* if they check a box for the very first time, we should automatically pick their name
     from any user dropdowns ... as the person doing the data entry probably also did the
     work, and it seems like the user-friendly thing to do */
  AV_TRY // just in case
  if (Sender)
  { // defaults
    if (((TCheckBox*)Sender)->Checked && Sender == checkCompleted && initial_state["is_completed"] != "1")
      cboCompletedBy->Text = "Self";
    if (((TCheckBox*)Sender)->Checked && Sender == checkMailed && initial_state["is_mailed"] != "1")
      cboMailedBy->Text = lookupTables->ValueOf("users", current_userid).c_str();
    if (((TCheckBox*)Sender)->Checked && Sender == checkFaxed && initial_state["is_faxed"] != "1")
      cboFaxedBy->Text = lookupTables->ValueOf("users", current_userid).c_str();
  }
  AV_CATCH
}
//---------------------------------------------------------------------------
