//---------------------------------------------------------------------------

#include <vcl.h>
#pragma hdrstop

#include "codeRecordInsuranceCoverage.h"
#include "codeSearchPerson.h"
#include "codeSearchTaxEntity.h"
#include "codeSearchInsurancePolicy.h"
#include "codeSpecialTerminateInsurancePolicy.h"
#include "codeRecordPerson.h"
//---------------------------------------------------------------------------
#pragma package(smart_init)
#pragma link "codeRecordGeneric"
#pragma resource "*.dfm"
TformRecordInsuranceCoverage *formRecordInsuranceCoverage;
//---------------------------------------------------------------------------
__fastcall TformRecordInsuranceCoverage::TformRecordInsuranceCoverage(TComponent* Owner, map<string,string> p_parameters)
        : TformRecordGeneric(Owner, p_parameters, string("insurance_coverage"))
{
  Initialize();
}
//---------------------------------------------------------------------------
bool TformRecordInsuranceCoverage::ScreenStart()
{
  // set the font color for all links
  txtTerminateCoverage->Font->Color = link_color;
  TformRecordGeneric::ScreenStart();
  return true;
}
//---------------------------------------------------------------------------
bool TformRecordInsuranceCoverage::ScreenStop()
{
  TformRecordGeneric::ScreenStop();
  return true;
}
//---------------------------------------------------------------------------
bool TformRecordInsuranceCoverage::IsValidOnScreen()
{
  if((editPersonCode->Text != "") && (! IsInt(editPersonCode->Text.c_str())))
  {
    PSay("Please enter a valid person code (it must be a number)", m_invalid_data, MB_OK);
    editPersonCode->SetFocus();
    return false;
  }
  else if(GetValidLinkId(txtPerson) == "")
  {
    PSay("Please select a person", m_missing_data, MB_OK);
    btnPersonSearch->SetFocus();
    return false;
  }
  else if(GetValidLinkId(txtPolicy) == "")
  {
    PSay("Please select a policy", m_missing_data, MB_OK);
    btnPolicySearch->SetFocus();
    return false;
  }
  else if (! Pdate (editEffectiveDate->Text.c_str(), mdy | yy).IsValidDate()) // jhaley 20110922: required
  {
    PSay("Please enter a valid effective date", m_invalid_data, MB_OK);
    editEffectiveDate->SetFocus();
    return false;
  }
  else if (! Pdate (editTerminatedDate->Text.c_str(), mdy | yy).IsValidDate() && ! (editTerminatedDate->Text == ""))
  {
    PSay("Please enter a valid terminated date", m_invalid_data, MB_OK);
    editTerminatedDate->SetFocus();
    return false;
  }
  else
    return true;
}
//---------------------------------------------------------------------------
bool TformRecordInsuranceCoverage::PutDataOnScreen()
{
  // put the values on the form from the map initial_state
  // values may have come from GetDataFromDatabase if it's an existing record,
  // or values may have been passed to the form in this map if it's a new record
  editPersonCode->Text = twodigit(StringToInt(initial_state["person_code"])).c_str();
  editName->Text = PrettyString(initial_state["name_on_card"]).c_str();

  editEffectiveDate->Text  = initial_state["start_date"].c_str();
  editTerminatedDate->Text = initial_state["stop_date"].c_str();

  CreateLink(txtPerson, btnPersonSearch, this, initial_state["person_id"],           "people");
  CreateLink(txtPolicy, btnPolicySearch, this, initial_state["insurance_policy_id"], "insurance_policies");
  CreateLink(txtPcp,    btnPcpSearch,    this, initial_state["tax_entity_id"],       "tax_entities");
  TformRecordGeneric::PutDataOnScreen();
  return true;
}
//---------------------------------------------------------------------------
bool TformRecordInsuranceCoverage::GetDataFromScreen()
{
  // place the values currently on the screen into
  // the map that will be used to update or insert the record
  // the parent form will call the insert or update function, and get the id
  // number if necessary and take care of current_id

  ending_state["person_id"] = GetValidLinkId(txtPerson);
  ending_state["insurance_policy_id"] = GetValidLinkId(txtPolicy);
  ending_state["tax_entity_id"] = GetValidLinkId(txtPcp);

  // we don't want null values
  if (ending_state["person_id"] == "")
   ending_state["person_id"] = "0";
  if (ending_state["insurance_policy_id"] == "")
   ending_state["insurance_policy_id"] = "0";
  if (ending_state["tax_entity_id"] == "")
   ending_state["tax_entity_id"] = "0";

  if(editEffectiveDate->Text != "")
    ending_state["start_date"] = Pdate(editEffectiveDate->Text.c_str(), mdy | yy).OutDate(mdy | yyyy);
  else
    ending_state["start_date"] = "";
  if(editTerminatedDate->Text != "")
    ending_state["stop_date"] = Pdate(editTerminatedDate->Text.c_str(), mdy | yy).OutDate(mdy | yyyy);
  else
    ending_state["stop_date"] = "";

  ending_state["person_code"] = editPersonCode->Text.c_str();
  ending_state["name_on_card"] = editName->Text.c_str();
  if(ending_state["person_code"] == "")
    ending_state["person_code"] = "0";

  ending_state["id"] = initial_state["id"];

  UpdatePolicyholderInfo();

  TformRecordGeneric::GetDataFromScreen();

  return true; // again, not much can go wrong here
}
//---------------------------------------------------------------------------
bool TformRecordInsuranceCoverage::ScreenLock()
{
  // disable panel containing edit boxes
  panelGroup->Enabled = false;
  TformRecordGeneric::ScreenLock();
  return true;
}
//---------------------------------------------------------------------------
bool TformRecordInsuranceCoverage::ScreenUnlock()
{
  // enable panel containing edit boxes
  panelGroup->Enabled = true;
  TformRecordGeneric::ScreenUnlock();
  return true;
}
//---------------------------------------------------------------------------
void __fastcall TformRecordInsuranceCoverage::btnPcpSearchClick(TObject *Sender)
{
  vector<string> return_vector;
  FORM_TRY
    TformSearchTaxEntity * companySearch = new TformSearchTaxEntity(NULL,&return_vector);
    companySearch->ShowTaskModal(this);
  FORM_CATCH
  if (! return_vector.empty())
    CreateLink(txtPcp, btnPcpSearch, this, return_vector[0], "tax_entities");
}
//---------------------------------------------------------------------------
void __fastcall TformRecordInsuranceCoverage::btnPolicySearchClick(TObject *Sender)
{
  vector<string> return_vector;
  FORM_TRY
    TformSearchInsurancePolicy * policySearch = new TformSearchInsurancePolicy(NULL,&return_vector);
    policySearch->ShowTaskModal(this);
  FORM_CATCH
  if (! return_vector.empty())
    CreateLink(txtPolicy, btnPolicySearch, this, return_vector[0], "insurance_policies");
  UpdatePolicyholderInfo();
}
//---------------------------------------------------------------------------
void __fastcall TformRecordInsuranceCoverage::btnPersonSearchClick(TObject *Sender)
{
  vector<string> return_vector;
  FORM_TRY
    TformSearchPerson * personSearch = new TformSearchPerson(NULL,&return_vector);
    personSearch->ShowTaskModal(this);
  FORM_CATCH
  if (! return_vector.empty())
    CreateLink(txtPerson, btnPersonSearch, this, return_vector[0], "people");
}
//---------------------------------------------------------------------------
bool TformRecordInsuranceCoverage::NotNewAnymore()
{
  if ((optionsPermissions["table.insurance_policies"]["update"] == "true") &&
      (optionsPermissions["table.insurance_coverage"]["update"] == "true"))
  {
    txtTerminateCoverage->Enabled = true;
  }

  TformRecordGeneric::NotNewAnymore();
  return true;
}
//---------------------------------------------------------------------------
void __fastcall TformRecordInsuranceCoverage::txtTerminateCoverageClick(TObject *Sender)
{
  if (is_editing)
  {
    PSay("You cannot terminate insurance coverage while you are editing it.  "
         "Please press Save or Undo and then click the \"Terminate coverage\" link again.", m_warning, MB_OK);
  }
  else
  {
    set<string> coverage;
    coverage.insert(initial_state["id"]);
    FORM_TRY
      TformSpecialTerminateInsurancePolicy *term_form = new TformSpecialTerminateInsurancePolicy(NULL, initial_state["insurance_policy_id"], coverage);
      term_form->ShowTaskModal(this);
    FORM_CATCH
  }
}
//---------------------------------------------------------------------------
void TformRecordInsuranceCoverage::UpdatePolicyholderInfo()
{
  string insurance_policy_id = GetValidLinkId(txtPolicy);

  if (insurance_policy_id == "")
  {
    lblPolicyHolderRelationship->Caption = "Policy-holder cannot be determined as the policy has not yet been linked.";
    txtPolicyholder->Caption = "";
    txtPolicyholder->Visible = false;
    lblPolicyholderInfo->Caption = "Click the policy search button to select a policy.";
    lblPolicyholderInfo->Visible = true;
  }
  else
  {
    policyholder_id = GetOneField(dbDatabase, "select policy_holder_id from insurance_policies where id = '" + insurance_policy_id + "'");
    if (IsBlankOrZero(policyholder_id))
    {
      lblPolicyHolderRelationship->Caption = "Policy-holder is MISSING on the attached policy.";
      txtPolicyholder->Caption = "";
      txtPolicyholder->Visible = false;
      lblPolicyholderInfo->Caption = "Click the policy link to go to the policy and select the policy-holder.";
      lblPolicyholderInfo->Visible = true;
    }
    else
    {
      relationship_id = GetOneField(dbDatabase,
        "select first 1 rel_a_to_b_id "
        "from relationships_view "                      // TODO -c BUG : jhaley - is this supposed to use initial_state?
        "where current_id+1>3 and person_a_id = '" + policyholder_id + "' and person_b_id = '" + initial_state["person_id"] + "' "
        "order by current_id desc");
      string policyholder_name = GetPersonName(dbDatabase, policyholder_id, first_last);
      string cardholder_name = GetPersonName(dbDatabase, initial_state["person_id"], first_last);
      if (IsBlankOrZero(relationship_id))
      {
        lblPolicyHolderRelationship->Caption = ("No relationship exists between " + policyholder_name + " and " + cardholder_name + ".").c_str();
        txtPolicyholder->Caption = "Add a relationship of policy-holder to card-holder";
        txtPolicyholder->Visible = true;
        txtPolicyholder->OnClick = AddRelationship;
        lblPolicyholderInfo->Caption = "";
        lblPolicyholderInfo->Visible = false;
      }
      else
      {
        lblPolicyHolderRelationship->Caption = ("Policy-holder (" + policyholder_name + ") is card-holder's " + lookupTables->ValueOf("relationship_types", relationship_id)).c_str();
        txtPolicyholder->Caption = "Change relationship of policy-holder to card-holder";
        txtPolicyholder->Visible = true;
        txtPolicyholder->OnClick = ChangeRelationship;
        lblPolicyholderInfo->Caption = "";
        lblPolicyholderInfo->Visible = false;
      }
    }
  }
}
//---------------------------------------------------------------------------
void __fastcall TformRecordInsuranceCoverage::AddRelationship(TObject *Sender)
{
  //
}
//---------------------------------------------------------------------------
void __fastcall TformRecordInsuranceCoverage::ChangeRelationship(TObject *Sender)
{
  //
}
//---------------------------------------------------------------------------

