//---------------------------------------------------------------------------

#include <vcl.h>
#pragma hdrstop

#include "codeRecordInsurancePolicy.h"
#include "codeSearchTaxEntity.h"
#include "codeRecordInsuranceCoverage.h"
#include "codeSearchPerson.h"
#include "codeSpecialTerminateInsurancePolicy.h"
//---------------------------------------------------------------------------
#pragma package(smart_init)
#pragma link "codeRecordGeneric"
#pragma link "codeHalfButton"
#pragma resource "*.dfm"
TformRecordInsurancePolicy *formRecordInsurancePolicy;
//---------------------------------------------------------------------------
__fastcall TformRecordInsurancePolicy::TformRecordInsurancePolicy(TComponent* Owner, map<string,string> p_parameters)
        : TformRecordGeneric(Owner, p_parameters, string("insurance_policies"))
{
  //toolPolicyHolderSearch->Images = imagesRecordTypes;
  //btnPolicyHolderSearch->ImageIndex = 55;
  
  coverage_filter.SetListViewHandle (listCoverage);
  coverage_filter.SetSecurity("insurance_coverage");
  coverage_filter.AddColumnSpec("Person", "people", as_text, true, 110);
  coverage_filter.AddColumnSpec("PCP", "tax_entities", as_text, true, 120, rawString);
  coverage_filter.AddColumnSpec("Person Code", "insurance_coverage", as_int, true, 100);
  coverage_filter.AddColumnSpec("Name on Card", "insurance_coverage", as_text, true, 110);
  coverage_filter.AddColumnSpec("Covered From", "insurance_coverage", as_date, true, 70);
  coverage_filter.AddColumnSpec("Covered Until", "insurance_coverage", as_date, true, 70);
  coverage_filter.AddColumnSpec("Current Id", "current_id", as_int, false, 80);
  coverage_filter.AddColumnSpec("Internal Id", "internal_id", as_int, false, 80);
  coverage_filter.FetchPreferences("recordinsurancepolicy.coverage_filter");
  coverage_filter.SetIconStatus(true);
  coverage_filter.SetSQLStatement ("");
  coverage_filter.RefreshDataSet();

  middle_section = new ins_custom_row_container(scrollBenefits);

  Initialize();
}
//---------------------------------------------------------------------------
bool TformRecordInsurancePolicy::ScreenStart()
{
  // register combo boxes
  lookupTables->RegisterComboBox("insurance_types", cboInsurance);
  lookupTables->RegisterComboBox("coverage_types",  cboCoverage);
  lookupTables->RegisterComboBox("source_types",    cboSource);
  // set the font color for all links
  txtTerminatePolicy->Font->Color = link_color;
  txtTerminateCoverage->Font->Color = link_color;
  TformRecordGeneric::ScreenStart();
  return true;
}
//---------------------------------------------------------------------------
bool TformRecordInsurancePolicy::ScreenStop()
{
  // unregister combo boxes
  lookupTables->UnregisterComboBox("insurance_types", cboInsurance);
  lookupTables->UnregisterComboBox("coverage_types",  cboCoverage);
  lookupTables->UnregisterComboBox("source_types",    cboSource);
  delete middle_section;
  middle_section = NULL;
  TformRecordGeneric::ScreenStop();
  return true;
}
//---------------------------------------------------------------------------
bool TformRecordInsurancePolicy::IsValidOnScreen()
{
  string current_id = ending_state["current_id"]; // NB: nabbed from codeRecordDenialLetter
  if(current_id == "")
    current_id = initial_state["current_id"];

  if(GetValidLinkId(txtCompany) == "")
  {
    PSay("Please select an insurance company", m_missing_data, MB_OK);
    btnCompanySearch->SetFocus();
    return false;
  }
  // jhaley 20110818: Effective Date is now required information, as per Insurance Data Improvement findings
  if(editEffectiveDate->Text == "" || !Pdate(editEffectiveDate->Text.c_str(), mdy | yy).IsValidDate())
  {
    PSay("Please enter a valid effective date.\n\n"
         "If no date is available from either the patient or his/her insurance card, please use today's date "
         "as a placeholder.",
         m_invalid_data, MB_OK);
    editEffectiveDate->SetFocus();
    return false;
  }
  if(!Pdate(editTerminatedDate->Text.c_str(), mdy | yy).IsValidDate() && editTerminatedDate->Text != "")
  {
    PSay("Please enter a valid terminated date", m_invalid_data, MB_OK);
    editTerminatedDate->SetFocus();
    return false;
  }
  if(cboInsurance->Text == "")
  {
    // jhaley 20110819: Be super-obnoxious, since I'm not allowed to make this required yet.
    int result =
      PSay("You have not specified an Insurance Type. Although this is allowed, it is highly detrimental to reporting "
           "and general use of insurance data. If possible, please verify the type of this policy against the "
           "available items in the Insurance Type combo box, and then choose the one which is most correct.\n\n"
           "Do you want to STOP saving and re-check the insurance policy type now?",
           m_missing_data, MB_YESNO);
    if(result == IDYES)
    {
      cboInsurance->Show();
      cboInsurance->SetFocus();
      return false;
    }
  }
  if(!lookupTables->IsInLookUpTable("insurance_types", cboInsurance->Text.c_str()))
  {
    PSay("Please enter a valid Insurance Type", m_invalid_data, MB_OK);
    cboInsurance->Show();
    cboInsurance->SetFocus();
    return false;
  }
  if(!lookupTables->IsInLookUpTable("coverage_types", cboCoverage->Text.c_str()))
  {
    PSay("Please enter a valid type of coverage", m_invalid_data, MB_OK);
    cboCoverage->SetFocus();
    return false;
  }
  if(!lookupTables->IsInLookUpTable("source_types", cboSource->Text.c_str()))
  {
    PSay("Please enter a valid source", m_invalid_data, MB_OK);
    cboSource->SetFocus();
    return false;
  }

  // jhaley 20110822: One or more benefits required, as per Insurance Data Improvement meeting.
  if(middle_section->empty())
  {
    PSay("You have not specified any information about benefits provided through this policy.\n\n"
         "Please review the Coverage Information section and check one or more applicable areas of coverage.",
         m_missing_data, MB_OK);
    return false;
  }

  string duplicate_policy_id = "";
  string sql =
    "select id from insurance_policies "
    "where tax_entity_id = '" + GetValidLinkId(txtCompany) + "' and "
    "policy_number = '" + UppercaseString(editPolicyNumber->Text.c_str()) + "'" +
    ((IsInt(initial_state["id"])) ? (" and insurance_policies.id != " + initial_state["id"]) : string(""));
  if(editEffectiveDate->Text != "")
    sql += " and start_date = '" + string(editEffectiveDate->Text.c_str()) + "'";
  duplicate_policy_id = GetOneField(dbDatabase, sql);
  if(duplicate_policy_id != "")
  {
    int stupid_answer =
      PSay("A policy with this policy number already exists with this company.\n\n"
           "If you continue you will create the policy in addition to the existing one(s).\n"
           "In rare cases this may actually be desired, but the most likely cause is trying\n"
           "to create an insurance policy without searching for it first.  If this is the\n"
           "case and you proceed, a DUPLICATE policy will be created, and both policies\n"
           "will INCORRECTLY reflect who is covered under that policy.  Instead, please:\n"
           "1) Say NO below\n"
           "2) Push CANCEL on the insurance policy screen you were creating\n"
           "3) Say NO you do not wish to save it\n"
           "4) On the policy search screen, enter the policy number and SEARCH\n"
           "5) find the appropriate policy and SELECT it.\n\n"
           "Are you SURE you want to create this policy?",
           m_warning, MB_YESNO,
           "tax_entity = " + GetValidLinkId(txtCompany) + ", policy_number = "
           + string(editPolicyNumber->Text.c_str()));
    if(stupid_answer != IDYES)
      return false;
  }

  // jhaley 20110819: Automatic expiration of policies with a termination date
  Pdate term_date = Pdate(editTerminatedDate->Text.c_str(), mdy|yy);
  if(term_date.IsValidDate() && term_date <= Pdate(Now()) &&
     (current_id == lookupTables->IdOf("current_types", "Current") ||
      current_id == lookupTables->IdOf("current_types", "Primary")))
  {
    ending_state["current_id"] = lookupTables->IdOf("current_types", "Expired");
  }

  // jhaley 20110819: Automatic stop date for Expired policies with no stop date
  if(current_id == lookupTables->IdOf("current_types", "Expired") && editTerminatedDate->Text == "")
  {
    editTerminatedDate->Text = Pdate(Now()).OutDate(mdy | yyyy).c_str();
  }

  return true;
}
//---------------------------------------------------------------------------
bool TformRecordInsurancePolicy::PutDataOnScreen()
{
  // put the values on the form from the map initial_state
  // values may have come from GetDataFromDatabase if it's an existing record,
  // or values may have been passed to the form in this map if it's a new record
  editPolicyNumber->Text   = initial_state["policy_number"].c_str();
  editGroupNumber->Text    = initial_state["group_number"].c_str();
  editEffectiveDate->Text  = initial_state["start_date"].c_str();
  editTerminatedDate->Text = initial_state["stop_date"].c_str();

  cboInsurance->Text = PrettyString(lookupTables->ValueOf("insurance_types", initial_state["insurance_id"])).c_str();
  cboCoverage->Text  = lookupTables->ValueOf("coverage_types", initial_state["coverage_id"]).c_str();
  cboSource->Text    = lookupTables->ValueOf("source_types",   initial_state["source_id"]).c_str();
  editName->Text     = PrettyString(initial_state["name_on_policy"]).c_str();

  // raw, because they're weird numbers, like group/policy numbers
  editProcessorNumber->Text = initial_state["processor_control_number"].c_str();
  editBinNumber->Text       = initial_state["bin_number"].c_str();

  CreateLink(txtCompany,  btnCompanySearch,  this, initial_state["tax_entity_id"], "tax_entities");
  CreateLink(txtEmployer, btnEmployerSearch, this, initial_state["person_id"],     "people");

  vector<string> covered_people;
  if (IsInt(initial_state["id"]))
  {
    string people_sql =
      "select P.id from insurance_coverage IC "
      "inner join people P on IC.person_id = P.id "
      "where IC.insurance_policy_id = '" + initial_state["id"] + "'";
    SqlToVec(dbDatabase, people_sql, covered_people);
  }
  CreateLink(txtPolicyHolder, btnPolicyHolderSearch, this, initial_state["policy_holder_id"], "people", "", true, -1,
             VecToDelimString(covered_people, "", "", "|"));

  TIBTransaction *dbTransaction = new TIBTransaction(NULL);
  StdTransaction (dbTransaction, dbDatabase);
  try
  {
    scrollBenefits->VertScrollBar->Visible = false;
    if (!middle_section->load_entries(dbTransaction, initial_state["id"]))
      throw(1);
    scrollBenefits->VertScrollBar->Visible = true;
  }
  catch (...)
  {
    PSay ("Failed to update middle section to match rest of screen. (Not good.) "
          "Please inform a programmer of this.", m_database_error, MB_OK);
  }
  if (dbTransaction->Active)
    dbTransaction->Commit();
  delete dbTransaction;

  TformRecordGeneric::PutDataOnScreen();
  return true;
}
//---------------------------------------------------------------------------
bool TformRecordInsurancePolicy::GetDataFromScreen()
{
  // place the values currently on the screen into
  // the map that will be used to update or insert the record
  // the parent form will call the insert or update function, and get the id
  // number if necessary and take care of current_id

  // these values are modified directly by hitting the 'search' buttons.
  string tax_entity_id = GetValidLinkId(txtCompany);
  if (tax_entity_id != "")
    ending_state["tax_entity_id"] = tax_entity_id;
  else
    ending_state["tax_entity_id"] = "0";

  string employer_id = GetValidLinkId(txtEmployer);
  if (employer_id != "")
    ending_state["person_id"] = employer_id;
  else
    ending_state["person_id"] = "0";

  // jhaley: this one had already been converted as a test-of-concept apparently, but I normalized it to be like the others.
  string policy_holder_id = GetValidLinkId(txtPolicyHolder);
  if (policy_holder_id != "")
    ending_state["policy_holder_id"] = policy_holder_id;
  else
    ending_state["policy_holder_id"] = "0";

  ending_state["policy_number"] = editPolicyNumber->Text.c_str();
  ending_state["group_number"] = editGroupNumber->Text.c_str();
  if(editEffectiveDate->Text != "")
    ending_state["start_date"] = Pdate(editEffectiveDate->Text.c_str(), mdy | yy).OutDate(mdy | yyyy);
  else
    ending_state["start_date"] = "";
  if(editTerminatedDate->Text != "")
    ending_state["stop_date"] = Pdate(editTerminatedDate->Text.c_str(), mdy | yy).OutDate(mdy | yyyy);
  else
    ending_state["stop_date"] = "";
  ending_state["insurance_id"] = lookupTables->IdOf("insurance_types", cboInsurance->Text.c_str());
  ending_state["coverage_id"] = lookupTables->IdOf("coverage_types", cboCoverage->Text.c_str());
  ending_state["source_id"] = lookupTables->IdOf("source_types", cboSource->Text.c_str());
  ending_state["name_on_policy"] = editName->Text.c_str();
  ending_state["processor_control_number"] = editProcessorNumber->Text.c_str();
  ending_state["bin_number"] = editBinNumber->Text.c_str();

  ending_state["id"] = initial_state["id"];

  TformRecordGeneric::GetDataFromScreen();

  return true; // again, not much can go wrong here
}
//---------------------------------------------------------------------------
bool TformRecordInsurancePolicy::ScreenLock()
{
  // disable panel containing edit boxes
  panelGroup->Enabled = false;
  // scrollBenefits->Enabled = false;
  middle_section->set_lock(true);
  TformRecordGeneric::ScreenLock();
  return true;
}
//---------------------------------------------------------------------------
bool TformRecordInsurancePolicy::ScreenUnlock()
{
  // enable panel containing edit boxes
  panelGroup->Enabled = true;
  // scrollBenefits->Enabled = true;
  middle_section->set_lock(false);
  TformRecordGeneric::ScreenUnlock();
  return true;
}
//---------------------------------------------------------------------------
void __fastcall TformRecordInsurancePolicy::btnCompanySearchClick(TObject *Sender)
{
  vector<string> return_vector;
  FORM_TRY
    TformSearchTaxEntity * companySearch = new TformSearchTaxEntity(NULL,&return_vector);
    companySearch->ShowTaskModal(this);
  FORM_CATCH
  if (! return_vector.empty())
    CreateLink(txtCompany, btnCompanySearch, this, return_vector[0], "tax_entities");
}
//---------------------------------------------------------------------------
bool TformRecordInsurancePolicy::IsNewAgain()
{
  coverage_filter.SetSQLStatement(string(""));
  coverage_filter.RefreshDataSet();
  coverage_filter.SetAddEdit(this, NULL, NULL);
  TformRecordGeneric::IsNewAgain();
  return true;
}
//---------------------------------------------------------------------------
bool TformRecordInsurancePolicy::NotNewAnymore()
{
  string sql;

  if (optionsPermissions["table.insurance_coverage"]["view"] == "true")
  {
    sql = "select person_names.last_name || ', ' || person_names.first_name || ' ' || person_names.suffix, "
          "tax_entities.name, insurance_coverage.person_code, insurance_coverage.name_on_card, "
          "insurance_coverage.start_date, insurance_coverage.stop_date, "
          "insurance_coverage.current_id, insurance_coverage.id "
          "from insurance_coverage, tax_entities, people "
          "left join person_names on people.person_names_id = person_names.id "
          "where insurance_coverage.person_id = people.id and "
          "insurance_coverage.tax_entity_id = tax_entities.id and "
          "insurance_coverage.insurance_policy_id = " + initial_state["id"];
    coverage_filter.SetSQLStatement (sql);
    coverage_filter.RefreshDataSet();
    coverage_filter.SetAddEdit(this, AddRecord(&(TformRecordInsurancePolicy::AddCoverage)),
                               EditRecord(&(TformRecordInsurancePolicy::EditCoverage)));
    coverage_request = atlas->RequestNotificationWhen("insurance_coverage.insurance_policy_id", initial_state["id"], rel_e, as_int, this,
                          RefreshHandler(&(TformRecordInsurancePolicy::AtlasRefreshListView)));
  }

  if ((optionsPermissions["table.insurance_policies"]["update"] == "true") &&
      (optionsPermissions["table.insurance_coverage"]["update"] == "true"))
  {
    txtTerminatePolicy->Enabled = true;
    txtTerminateCoverage->Enabled = true;
  }

  TformRecordGeneric::NotNewAnymore();
  return true;
}
//---------------------------------------------------------------------------
bool TformRecordInsurancePolicy::AtlasRefreshListView(int request_num, string matched_value)
{
  if (request_num == coverage_request)
    coverage_filter.RefreshDataSet();
  return true;
}
//---------------------------------------------------------------------------
void TformRecordInsurancePolicy::AddCoverage()
{
  map<string,string> temp_map;
  temp_map["id"] = "new";
  temp_map["insurance_policy_id"] = initial_state["id"];
  FORM_TRY
    TformRecordInsuranceCoverage * coverageRecord = new TformRecordInsuranceCoverage(NULL, temp_map);
    coverageRecord->Show();
  FORM_CATCH
}
//---------------------------------------------------------------------------
void TformRecordInsurancePolicy::EditCoverage()
{
  map<string,string> temp_map;
  temp_map["id"] = coverage_filter.GetSelectedRecordID();
  FORM_TRY
    TformRecordInsuranceCoverage * coverageRecord = new TformRecordInsuranceCoverage(NULL, temp_map);
    coverageRecord->Show();
  FORM_CATCH
}
//---------------------------------------------------------------------------
void __fastcall TformRecordInsurancePolicy::btnEmployerSearchClick(
      TObject *Sender)
{
 vector<string> return_vector = TformSearchPerson::Do(this);
 if (!return_vector.empty())
   CreateLink(txtEmployer, btnEmployerSearch, this, return_vector[0], "people");
}
//---------------------------------------------------------------------------
bool TformRecordInsurancePolicy::BeforeSave()
{
 return (middle_section->save_entries(lock_transaction, ending_state["id"]));
}
//---------------------------------------------------------------------------
void __fastcall TformRecordInsurancePolicy::scrollBenefitsMouseWheelDown(
      TObject *Sender, TShiftState Shift, TPoint &MousePos, bool &Handled)
{
  if(scrollBenefits->Visible)
    scrollBenefits->VertScrollBar->Position += scrollBenefits->VertScrollBar->Increment;
}
//---------------------------------------------------------------------------
void __fastcall TformRecordInsurancePolicy::scrollBenefitsMouseWheelUp(
      TObject *Sender, TShiftState Shift, TPoint &MousePos, bool &Handled)
{
  if(scrollBenefits->Visible)
    scrollBenefits->VertScrollBar->Position -= scrollBenefits->VertScrollBar->Increment;
}
//---------------------------------------------------------------------------
void __fastcall TformRecordInsurancePolicy::txtTerminatePolicyClick(TObject *Sender)
{
  if (is_editing)
  {
    PSay("You cannot terminate an insurance policy while you are editing it.  "
         "Please press Save or Undo and then click the \"Terminate this policy\" link again.", m_warning, MB_OK);
  }
  else
  {
    TformSpecialTerminateInsurancePolicy *term_form = NULL;
    FORM_TRY
      term_form = new TformSpecialTerminateInsurancePolicy(NULL, initial_state["id"], VecToSet(coverage_filter.GetAllID()));
    FORM_CATCH_RETURN("TformSpecialTerminateInsurancePolicy", )
    term_form->ShowTaskModal(this);
  }
}
//---------------------------------------------------------------------------
void __fastcall TformRecordInsurancePolicy::txtTerminateCoverageClick(TObject *Sender)
{
  TformSpecialTerminateInsurancePolicy *term_form = NULL;
  FORM_TRY
    term_form = new TformSpecialTerminateInsurancePolicy(NULL, initial_state["id"], VecToSet(coverage_filter.GetMultiID()));
  FORM_CATCH_RETURN("TformSpecialTerminateInsurancePolicy", )

  term_form->ShowTaskModal(this);
}
//---------------------------------------------------------------------------
void __fastcall TformRecordInsurancePolicy::btnPolicyHolderSearchClick(TObject *Sender)
{
 vector<string> return_vector = TformSearchPerson::Do(this);
 if (!return_vector.empty())
 {
  if (editName->Text == "")
   editName->Text = GetPersonName(dbDatabase, return_vector[0], first_middle_last).c_str();
 }
 vector<string> covered_people;
 if (IsInt(initial_state["id"]))
 {
   string people_sql =
     "select P.id from insurance_coverage IC "
     "inner join people P on IC.person_id = P.id "
     "where IC.insurance_policy_id = '" + initial_state["id"] + "'";
   SqlToVec(dbDatabase, people_sql, covered_people);
 }
 CreateLink(txtPolicyHolder, btnPolicyHolderSearch, this, return_vector[0], "people", "", true, -1, VecToDelimString(covered_people, "", "", "|"));
}
//---------------------------------------------------------------------------
void __fastcall TformRecordInsurancePolicy::buttonGrabPolicyHolderNameClick(TObject *Sender)
{
  string t = GetPersonName(dbDatabase, GetLinkId(txtPolicyHolder), first_middle_last);
  if (!t.empty() && t != "None")
    editName->Text = t.c_str();
  else if (editName->Text != "")
  {
    // jhaley 20110819: Additional instructions to user: everyone should assume responsibility for data integrity.
    PSay("No policy-holder or policy-holder's name is blank;\nleaving current name value alone.\n\n"
         "If the person you selected for policy holder had a blank name, you should stop and repair that now by "
         "opening the policy holder's Person record screen and editing their name information.",
         m_missing_data, MB_OK);
  }
}
//---------------------------------------------------------------------------
// jhaley 20110819: Added AfterSave to make user aware that they need to create a coverage on this policy, even if the
// policy holder is the only person covered on the policy.
bool TformRecordInsurancePolicy::AfterSave()
{
  if(StringToInt(ending_state["current_id"]) >= 3 && !IsBlankOrZero(ending_state["id"]))
  {
    string coverage_sql =
      "select IC.id "
      "from insurance_coverage IC "
      "where IC.insurance_policy_id = '" + ending_state["id"] + "' and IC.current_id <> 0 ";
    vector<string> coverage_ids;
    if(SqlToVec(dbDatabase, coverage_sql, coverage_ids))
    {
      if(coverage_ids.empty())
      {
        pmapstrs temp_map;

        // Populate as much information as possible
        temp_map["id"] = "new";
        temp_map["insurance_policy_id"] = ending_state["id"];
        temp_map["person_id"]  = ending_state["policy_holder_id"];
        temp_map["start_date"] = ending_state["start_date"];
        temp_map["stop_date"]  = ending_state["stop_date"];

        PSay("There are no coverages for this insurance policy. You should create one now, even if the policy "
             "holder is the only person covered by this policy.", m_reminder, MB_OK);

        TformRecordInsuranceCoverage *coverage_form = NULL;
        FORM_TRY
          coverage_form = new TformRecordInsuranceCoverage(NULL, temp_map);
        FORM_CATCH_RETURN("TformRecordInsuranceCoverage", true)
        coverage_form->Show();
        coverage_form->BringToFront();
      }
    }
    else
      PSay("Failed to retrieve coverages for this insurance policy!", m_warning, MB_OK);
  }

  return TformRecordGeneric::AfterSave();
}
//---------------------------------------------------------------------------

