//---------------------------------------------------------------------------

#ifndef codeRecordInsurancePolicyH
#define codeRecordInsurancePolicyH
//---------------------------------------------------------------------------
#include <Classes.hpp>
#include <Controls.hpp>
#include <StdCtrls.hpp>
#include <Forms.hpp>
#include "codeRecordGeneric.h"
#include <ComCtrls.hpp>
#include <ToolWin.hpp>
#include <ExtCtrls.hpp>
#include <Buttons.hpp>
#include <Menus.hpp>
#include "codeHalfButton.h"
//---------------------------------------------------------------------------

// code taken from codeRecordComVisit.h, duped, modifed ... there's a base class here somewhere. *cough* ... in the fourth dimension.
struct ins_custom_row_pointers
{
  TPanel *    benefit_panel;
  TCheckBox * benefit_granted;
  TEdit *     benefit_note;
  string      benefit_id;
  string      area_id;
};

class ins_custom_row_container
{
 private:
  vector<ins_custom_row_pointers> custom_rows;
  bool should_be_locked;
  string current_policy_id;
  TWinControl * current_containing_window;
 public:
  ins_custom_row_container (TWinControl * containing_window) : current_containing_window(containing_window), should_be_locked(false) {}

  void set_lock (bool value)
  {
   should_be_locked = value;
   for (vector<ins_custom_row_pointers>::iterator i_row = custom_rows.begin(); i_row != custom_rows.end(); i_row++)
    i_row->benefit_panel->Enabled = !should_be_locked;
  }

  bool load_entries (TIBTransaction * dbTransaction, string policy_id)
  {
   // we hope it doesn't contain anything else of importance ... oops =)
   current_containing_window->DestroyComponents();
   current_containing_window->Repaint();
   custom_rows.clear();
   current_policy_id = policy_id;
   if (current_policy_id == "" || current_policy_id == "new")
    current_policy_id = "0";
   vector<map<string, string> > entries_and_values;
   string areas_sql =
     "select ATYP.id, ATYP.item, IB.id benefit_id, IB.is_covered, IB.note benefit_note "
     "from area_types ATYP "
     "left join insurance_benefits IB on IB.area_id = ATYP.id and IB.insurance_policy_id = " + current_policy_id + " "
     "where ATYP.id > 0 "
     "order by ATYP.item ";
   if (!SqlToVecMap(dbTransaction, areas_sql, entries_and_values))
    return false;
   else
   {
    unsigned int current_top = 1;
    unsigned int big_width = current_containing_window->ClientWidth - 6;
    for (vector<map<string, string> >::iterator i_entry = entries_and_values.begin(); i_entry != entries_and_values.end(); i_entry++)
    {
     ins_custom_row_pointers new_row;
     new_row.benefit_panel   = new TPanel(current_containing_window);
     new_row.benefit_granted = new TCheckBox(new_row.benefit_panel);
     new_row.benefit_note    = new TEdit(new_row.benefit_panel);
     new_row.benefit_id      = (*i_entry)["benefit_id"];
     new_row.area_id         = (*i_entry)["id"];

     current_containing_window->InsertControl(new_row.benefit_panel);

     new_row.benefit_panel->Top = current_top;
     new_row.benefit_panel->Left = 0;
     new_row.benefit_panel->Width = big_width;
     new_row.benefit_panel->Height = 24; // that's how much we change between rows
     new_row.benefit_panel->BevelInner = bvNone;
     new_row.benefit_panel->BevelOuter = bvNone;
     new_row.benefit_panel->Caption = "";
     new_row.benefit_panel->Enabled = !should_be_locked;
     new_row.benefit_panel->Anchors << akRight;

     new_row.benefit_granted->Checked = ((*i_entry)["is_covered"] == "1");
     new_row.benefit_granted->Caption = PrettyString((*i_entry)["item"]).c_str();
     new_row.benefit_granted->Left    = 6;   // the scroll box eats 2 already, so to get 8, add 6 to the left side
     new_row.benefit_granted->Top     = 0;
     new_row.benefit_granted->Width   = 97; // 214; // 222 - 8 -- (benefit_note->Left - 8)

     new_row.benefit_note->Text = SentenceString((*i_entry)["benefit_note"]).c_str();
     new_row.benefit_note->Left = 112 - 2; // 222; // 224 - 8 + 6 -- label minus other label plus scroll box left side (8-2)
     new_row.benefit_note->Top  = 0;
     new_row.benefit_note->Width = (new_row.benefit_panel->ClientWidth - new_row.benefit_note->Left); // (257 + 80) - 16; // width of text + width of label - scroll bar area
     new_row.benefit_note->MaxLength = 250;
     new_row.benefit_note->Anchors << akRight;

     new_row.benefit_panel->InsertControl(new_row.benefit_granted);
     new_row.benefit_panel->InsertControl(new_row.benefit_note);

     new_row.benefit_granted->Visible = true;
     new_row.benefit_note->Visible = true;
     new_row.benefit_panel->Visible = true;

     custom_rows.push_back(new_row);

     current_top += 24;
    }

    for (vector<ins_custom_row_pointers>::iterator i_row = custom_rows.begin(); i_row != custom_rows.end(); i_row++)
    {
     i_row->benefit_panel->Left = i_row->benefit_panel->Left + 1;
     i_row->benefit_panel->Left = i_row->benefit_panel->Left - 1;
     i_row->benefit_note->Left = i_row->benefit_note->Left + 1;
     i_row->benefit_note->Left = i_row->benefit_note->Left - 1;
    }

    // none selected?
    if (current_top == 1)
    {
     TLabel * oops_label = new TLabel(current_containing_window);
     oops_label->Top = current_top;
     oops_label->Left = 8;
     oops_label->AutoSize = true;
     oops_label->Caption = "Nothing was found: please contact a programmer.";
     oops_label->Visible = true;
     current_containing_window->InsertControl(oops_label);
    }
   }
   return true;
  }

  void get_notes (vector<string> &notes)
  {
    notes.clear();
    for (vector<ins_custom_row_pointers>::iterator i_row = custom_rows.begin(); i_row != custom_rows.end(); ++i_row)
    {
     if (i_row->benefit_granted->Checked || i_row->benefit_note->Text != "")
       notes.push_back(i_row->benefit_note->Text.c_str());
    }
  }

  // jhaley 20110822: test if any data is specified in the ins_custom_row_container
  bool empty()
  {
    bool found_one = false;
    for(vector<ins_custom_row_pointers>::iterator i_row = custom_rows.begin(); i_row != custom_rows.end(); ++i_row)
    {
      if(i_row->benefit_granted->Checked || i_row->benefit_note->Text != "")
        found_one = true;
    }
    return !found_one;
  }

  bool save_entries (TIBTransaction * dbTransaction, string policy_id)
  {
   if ((current_policy_id != "" && current_policy_id != "new" && current_policy_id != "0") && policy_id != current_policy_id)
   {
     PSay("ID values do not match!", m_debug, MB_OK, "save_entries");
     return false;
   }
   else
   {
    vector<string> valid_entries;
    for (vector<ins_custom_row_pointers>::iterator i_row = custom_rows.begin(); i_row != custom_rows.end(); ++i_row)
    {
     map<string, string> field_values;
     if (i_row->benefit_granted->Checked || i_row->benefit_note->Text != "")
      if (i_row->benefit_id == "")
      {
       // insert
       field_values["id"]                  = GetNextId(dbTransaction, "insurance_benefits");
       field_values["current_id"]          = "4";
       field_values["last_update_stamp"]   = RightNow();
       field_values["last_updated_by"]     = current_userid;
       field_values["area_id"]             = i_row->area_id;
       field_values["insurance_policy_id"] = policy_id;
       field_values["is_covered"]          = i_row->benefit_granted->Checked ? "1" : "0";
       field_values["note"]                = i_row->benefit_note->Text.c_str();
       valid_entries.push_back(field_values["id"]);
       if(!ExecuteInsertStatement(dbTransaction, "insurance_benefits", field_values))
       {
         PSay("Could not insert a row", m_debug, MB_OK, "save_entries");
         return false;
       }
      }
      else
      {
       // update
       field_values["id"]                = i_row->benefit_id;
       field_values["last_update_stamp"] = RightNow();
       field_values["last_updated_by"]   = current_userid;
       field_values["is_covered"]        = i_row->benefit_granted->Checked ? "1" : "0";
       field_values["note"]              = i_row->benefit_note->Text.c_str();
       valid_entries.push_back(field_values["id"]);
       if (!ExecuteUpdateStatement(dbTransaction, "insurance_benefits", field_values))
        {PSay("Could not update a row", m_debug, MB_OK, "save_entries"); return false;}
      }
    }
    // delete anything not inserted or updated
    if (valid_entries.empty())
    {
     if (!ExecuteStatement(dbTransaction, "delete from insurance_benefits where insurance_policy_id = " + policy_id))
     {
       PSay("Could not delete all non-essential dependent rows", m_debug, MB_OK, "save_entries");
       return false;
     }
    }
    else
    {
     if (!ExecuteStatement(dbTransaction, "delete from insurance_benefits where insurance_policy_id = " + policy_id + " and id not in (" + VecToCommaString(valid_entries) + ")"))
     {
       PSay("Could not delete some non-essential dependent rows", m_debug, MB_OK, "save_entries");
       return false;
     }
    }
   }
   return true;
  }
};

class TformRecordInsurancePolicy : public TformRecordGeneric
{
__published:	// IDE-managed Components
        TPanel *panelGroup;
        TLabel *lblCompany;
        TLabel *lblPolicyNumber;
        TLabel *lblEffectiveDate;
        TLabel *lblCoverage;
        TLabel *lblGroupNumber;
        TLabel *lblTerminated;
        TLabel *lblSource;
        TLabel *lblName;
        TEdit *editPolicyNumber;
        TEdit *editGroupNumber;
        TEdit *editEffectiveDate;
        TEdit *editTerminatedDate;
        TEdit *editName;
        TStaticText *txtCompany;
        TBitBtn *btnCompanySearch;
        TComboBox *cboCoverage;
        TComboBox *cboSource;
        TLabel *lblEmployer;
        TBitBtn *btnEmployerSearch;
        TStaticText *txtEmployer;
        TPanel *panelSplitting;
        TGroupBox *groupCoverage;
        TScrollBox *scrollBenefits;
        TSplitter *splitterMiddle;
        TPanel *panelCoverage;
        TLabel *_labelCovered;
        TListView *listCoverage;
        TEdit *editProcessorNumber;
        TEdit *editBinNumber;
        TComboBox *cboInsurance;
        TLabel *Label3;
        TLabel *Label4;
        TLabel *lblInsurance;
        TStaticText *txtTerminatePolicy;
        TStaticText *txtTerminateCoverage;
        TLabel *lblPolicyHolder;
        TStaticText *txtPolicyHolder;
        TBitBtn *btnPolicyHolderSearch;
        THalfButton *buttonGrabPolicyHolderName;
        void __fastcall btnCompanySearchClick(TObject *Sender);
        void __fastcall btnEmployerSearchClick(TObject *Sender);
        void __fastcall scrollBenefitsMouseWheelDown(TObject *Sender,
          TShiftState Shift, TPoint &MousePos, bool &Handled);
        void __fastcall scrollBenefitsMouseWheelUp(TObject *Sender,
          TShiftState Shift, TPoint &MousePos, bool &Handled);
        void __fastcall txtTerminatePolicyClick(TObject *Sender);
        void __fastcall txtTerminateCoverageClick(TObject *Sender);
        void __fastcall buttonGrabPolicyHolderNameClick(TObject *Sender);
        void __fastcall btnPolicyHolderSearchClick(TObject *Sender);
private:	// User declarations
        classFilterObject coverage_filter;
        int coverage_request;

        ins_custom_row_container * middle_section;

        // string tax_entity_id;
        // string employer_id;
        // string policy_holder_id; // switching to more better way of handling links
public:		// User declarations
        __fastcall TformRecordInsurancePolicy(TComponent* Owner, map<string,string> p_parameters);
        bool IsValidOnScreen();
        bool PutDataOnScreen();
        bool GetDataFromScreen();
        bool ScreenStart();
        bool ScreenStop();
        bool ScreenLock();
        bool ScreenUnlock();
        bool NotNewAnymore();        
        bool AtlasRefreshListView(int request_num, string matched_value);
        bool IsNewAgain();
        void AddCoverage();
        void EditCoverage();
        bool BeforeSave();
        bool AfterSave(); // jhaley 20110819
        static void Do(map<string, string> & p_parameters);
};
//---------------------------------------------------------------------------
extern PACKAGE TformRecordInsurancePolicy *formRecordInsurancePolicy;
//---------------------------------------------------------------------------
#endif
