//---------------------------------------------------------------------------

#include <vcl.h>
#pragma hdrstop

#include "codeRecordObservationQuestion.h"
#include "codeRecordObsExpectedRange.h"
#include "codeSearchCode.h"
#include "codeRecordMetricUnit.h"
#include "codeWizardObservationOptionSet.h"
#include "codeSpecialObservationQuestionSetEditor.h"
//---------------------------------------------------------------------------
#pragma package(smart_init)
#pragma link "codeRecordGeneric"
#pragma link "codeHalfButton"
#pragma link "codeRubberContainer"
#pragma resource "*.dfm"
TformRecordObservationQuestion *formRecordObservationQuestion;
//---------------------------------------------------------------------------
/*
   Here's a quick explanation of what the three question formats are used for:
     short_item  - IHS HL7 export, Expected Ranges, (opt) column on question set editor
     item        - dropdowns, (now; previously long_item) on recordObservationSet
     long_item   - printing reports they never print
*/
__fastcall TformRecordObservationQuestion::TformRecordObservationQuestion(TComponent* Owner, map<string,string> p_parameters)
        : TformRecordGeneric(Owner, p_parameters, string("obs_questions"))
{
  fields_to_leave_raw.insert("item");
  fields_to_leave_raw.insert("short_item");
  fields_to_leave_raw.insert("long_item");
  fields_to_leave_raw.insert("loinc_code"); // just in case?
  fields_to_leave_raw.insert("cpt_code"); // just in case?

  ranges_filter.SetListViewHandle (listRanges);
  ranges_filter.SetSecurity("obs_expected_ranges");
  ranges_filter.AddColumnSpec("Gender", "obs_expected_ranges", as_text, true, 120);
  ranges_filter.AddColumnSpec("Lower", "obs_expected_ranges", as_text, true, 120, rawString);
  ranges_filter.AddColumnSpec("Upper", "obs_expected_ranges", as_text, true, 120, rawString);
  ranges_filter.AddColumnSpec("Current Id", "current_id", as_int, false, 80);
  ranges_filter.AddColumnSpec("Internal Id", "internal_id", as_int, false, 80);
  ranges_filter.FetchPreferences("recordobservationquestion.ranges_filter");
  ranges_filter.SetSQLStatement("");
  ranges_filter.RefreshDataSet();

  buttonEditUnit->Enabled = (optionsPermissions["table.units"]["update"] == "true");
  buttonAddUnit->Enabled = (optionsPermissions["table.units"]["insert"] == "true");
  buttonEditOptionSet->Enabled = (optionsPermissions["table.obs_option_sets"]["update"] == "true"); // TODO: hmmm. not a precise permission.

  Initialize();
}
//---------------------------------------------------------------------------
void __fastcall TformRecordObservationQuestion::buttonExpandLoincClick(TObject *Sender)
{
  FORM_TRY
    TformSearchCode * codeSearch = new TformSearchCode(NULL, NULL);
    codeSearch->cboCode->Text = "LOINC"; // TODO: needs to be added?
    codeSearch->editCode->Text = editLOINC->Text;
    codeSearch->Show();
    if (editLOINC->Text != "")
     codeSearch->buttonSearch->Click();
  FORM_CATCH
}
//---------------------------------------------------------------------------
void __fastcall TformRecordObservationQuestion::buttonSearchLoincClick(TObject *Sender)
{
  vector<string> return_vector;
  string sql;
  FORM_TRY
    TformSearchCode * codeSearch = new TformSearchCode(NULL, &return_vector);
    codeSearch->cboCode->Text = "LOINC"; // TODO: needs to be added?
    codeSearch->ShowTaskModal(this);
  FORM_CATCH
  if (!return_vector.empty())
  {
   sql = "select code from codes where id = " + return_vector[0];
   editLOINC->Text = GetOneField(dbDatabase, sql).c_str();
   if (editLOINC->Text != "")
   {
    string new_note;
    string entered_note = UppercaseString(editItem->Text.c_str());
    string sql = "select description from codes where code = '";
    sql += editLOINC->Text.c_str(); // TODO: SAFESQLSTRING -- do this on condition and procedure screens too!
    sql += "' and code_id = '" + lookupTables->IdOf("code_types", "LOINC") + "' "; // TODO: still a problem
    string code_note = GetOneField(dbDatabase, sql);
    if (entered_note == "")
     new_note = code_note;
    else
    {
     if(entered_note.find(code_note) == string::npos)
      new_note = entered_note + "\n\n" + code_note;
     else
      new_note = entered_note;
    }
    editItem->Text = SentenceString(new_note).c_str();
   }
  }
}
//---------------------------------------------------------------------------
void __fastcall TformRecordObservationQuestion::buttonEditOptionSetClick(TObject *Sender)
{
 if (comboOptionSet->Text != "" && lookupTables->IsInLookUpTable("obs_option_set_types", comboOptionSet))
  TformWizardObservationOptionSet::Do(lookupTables->IdOf("obs_option_set_types", comboOptionSet->Text.c_str()));
 else
  TformWizardObservationOptionSet::Do();
}
//---------------------------------------------------------------------------
void __fastcall TformRecordObservationQuestion::buttonEditUnitClick(TObject *Sender)
{
 map<string, string> params;
 params["id"] = lookupTables->IdOf("units", comboUnit);
 if (!IsBlankOrZero(params["id"]))
  TformRecordMetricUnit::Do(params);
}
//---------------------------------------------------------------------------
bool TformRecordObservationQuestion::ScreenStart()
{
  staticQuestionSetEditor->Font->Color = link_color;
  lookupTables->RegisterComboBox("obs_question_types", comboType);
  lookupTables->RegisterComboBox("obs_option_set_types", comboOptionSet);
  string obs_id = lookupTables->IdOf("obs_option_set_types", comboOptionSet); if (obs_id == "") obs_id = "0";
  lookupTables->RegisterComboBox("obs_option_types", comboDefaultOption, "select id, item, valid from obs_option_types where obs_option_set_id = " + obs_id);
  lookupTables->RegisterComboBox("units", comboUnit);
  lookupTables->RegisterComboBox("improvement_types", comboImprovement);
  TformRecordGeneric::ScreenStart();
  return true;
}
//---------------------------------------------------------------------------
bool TformRecordObservationQuestion::ScreenStop()
{
  lookupTables->UnregisterComboBox("obs_question_types", comboType);
  lookupTables->UnregisterComboBox("obs_option_set_types", comboOptionSet);
  lookupTables->UnregisterComboBox("units", comboUnit);
  lookupTables->UnregisterComboBox("improvement_types", comboImprovement);
  TformRecordGeneric::ScreenStop();
  return true;
}
//---------------------------------------------------------------------------
bool TformRecordObservationQuestion::IsValidOnScreen()
{
 if (editItem->Text == "")
 {
  PSay("Please enter a full name", m_invalid_data, MB_OK);
  editItem->SetFocus();
  return false;
 }

 if (editShortItem->Text == "")
 {
  PSay("Please enter a short name", m_invalid_data, MB_OK);
  editShortItem->SetFocus();
  return false;
 }

 if (comboType->Text == "" || !lookupTables->IsInLookUpTable("obs_question_types", comboType))
 {
  PSay("Please select a question type", m_invalid_data, MB_OK);
  comboType->SetFocus();
  return false;
 }

 // why in a block? to make sure i don't accidentally re-use these variables elsewhere without realizing it (copy-pasting)
 {
  bool need_option_set = is_in(lookupTables->IdOf("obs_question_types", comboType), "1", "3", NULL); // TODO -c MAGIC:
  bool has_option_set = (comboOptionSet->Text != "" && lookupTables->IsInLookUpTable("obs_option_set_types", comboOptionSet->Text.c_str()));
  if (need_option_set && !has_option_set)
  {
   PSay("Please select an option set", m_invalid_data, MB_OK);
   comboOptionSet->SetFocus();
   return false;
  }
  if (!need_option_set && has_option_set)
  {
   PSay("Option set is being deselected for you as it's not needed here.", m_invalid_data, MB_OK);
   comboOptionSet->Text = "";
  }
 }

 {
  bool need_unit = (lookupTables->IdOf("obs_question_types", comboType) == "4"); // TODO -c MAGIC:
  bool has_unit = (comboUnit->Text != "" && lookupTables->IsInLookUpTable("units", comboUnit->Text.c_str()));
  if (need_unit && !has_unit)
  {
   PSay("Please select a unit of measure", m_invalid_data, MB_OK);
   comboUnit->SetFocus();
   return false;
  }
  if (!need_unit && has_unit)
  {
   PSay("Unit of measure is being deselected for you as it's not needed here.", m_invalid_data, MB_OK);
   comboUnit->Text = "";
  }
  if (need_unit)
  {
   if (editAbsMin->Text != "" && !IsFloat(editAbsMin->Text.c_str()))
   {
    PSay("Please enter a valid absolute minimum metric value.", m_invalid_data, MB_OK);
    return false;
   }
   if (editAbsMax->Text != "" && !IsFloat(editAbsMax->Text.c_str()))
   {
    PSay("Please enter a valid absolute maximum metric value.", m_invalid_data, MB_OK);
    return false;
   }
  }
 }

 {
  bool can_have_improvement = is_in(lookupTables->IdOf("obs_question_types", comboType), "3", "4", NULL);
  bool has_improvement = (comboImprovement->Text != "" && lookupTables->IsInLookUpTable("improvement_types", comboImprovement->Text.c_str()));
  if (!can_have_improvement && has_improvement)
  {
   PSay("Improvement is being deselected for you as it's not available here.", m_invalid_data, MB_OK);
   comboImprovement->Text = "";
  }
 }

 // may want to seriously warn users about changing the type, unit, option set of a question that already has responses!!!
 if (is_editing &&
    (initial_state["obs_question_id"] != lookupTables->IdOf("obs_question_types", comboType) ||
     initial_state["unit_id"] != lookupTables->IdOf("units", comboUnit) ||
     initial_state["obs_option_set_id"] != lookupTables->IdOf("obs_option_set_types", comboOptionSet)))
 {
  string c = GetOneField(dbDatabase, "select count(*) from obs_responses where obs_question_id = " + initial_state["id"]);
  if (!IsBlankOrZero(c))
   if (IDNO == PSay("There are currently " + c + " responses to this question.\nChanging the type, unit, or option set seems like a bad idea.\nAre you sure you want to continue?", m_confirmation, MB_YESNO))
    return false;
 }

 return true;
}
//---------------------------------------------------------------------------
bool TformRecordObservationQuestion::NotNewAnymore()
{
 bool need_option_set = is_in(lookupTables->IdOf("obs_question_types", comboType->Text.c_str()), "1", "3", NULL); // TODO -c MAGIC:
 bool need_unit = (lookupTables->IdOf("obs_question_types", comboType->Text.c_str()) == "4"); // TODO -c MAGIC:

 _labelExpectedRanges->Enabled = (need_option_set || need_unit); // just a general indicator

 string sql = "select gender_types.item, ";
 if (need_unit)
       sql += "cast(cast(obs_expected_ranges.min_response_metric as numeric (15,2)) as varchar(250)) || ' ' || units.item, "
              "cast(cast(obs_expected_ranges.max_response_metric as numeric (15,2)) as varchar(250)) || ' ' || units.item, ";
 else
 if (need_option_set)
       sql += "min_obs_option_types.item, "
              "max_obs_option_types.item, ";
       sql += "obs_expected_ranges.current_id, obs_expected_ranges.id from obs_expected_ranges "
              "left join gender_types on gender_types.id = obs_expected_ranges.gender_id "
              "left join obs_questions on obs_questions.id = obs_expected_ranges.obs_question_id "
              "left join units on units.id = obs_questions.unit_id "
              "left join obs_option_types min_obs_option_types on min_obs_option_types.id = obs_expected_ranges.min_obs_option_id "
              "left join obs_option_types max_obs_option_types on max_obs_option_types.id = obs_expected_ranges.max_obs_option_id "
              "where obs_expected_ranges.obs_question_id = " + initial_state["id"];

  ranges_filter.SetSQLStatement(sql);
  ranges_filter.RefreshDataSet();
  ranges_filter.SetAddEdit(this, AddRecord(&(TformRecordObservationQuestion::AddRange)),
                               EditRecord(&(TformRecordObservationQuestion::EditRange)));
  ranges_request = atlas->RequestNotificationWhen("obs_expected_ranges.obs_question_id", initial_state["id"], rel_e, as_int, this,
                     RefreshHandler(&(TformRecordObservationQuestion::AtlasRefreshListView)));

  return TformRecordGeneric::NotNewAnymore();
}
//---------------------------------------------------------------------------
bool TformRecordObservationQuestion::PutDataOnScreen()
{
  // put the values on the form from the map initial_state
  // values may have come from GetDataFromDatabase if it's an existing record,
  // or values may have been passed to the form in this map if it's a new record

  if(optionsPermissions["table.codes"]["view"] != "true")
    buttonSearchLoinc->Enabled = false;

  editItem->Text           = initial_state["item"].c_str();
  editShortItem->Text      = initial_state["short_item"].c_str();
  editLongItem->Text       = initial_state["long_item"].c_str();
  checkValid->Checked      = (initial_state["valid"] != "0"); // changed, default differently
  checkCalculated->Checked = (initial_state["is_calculated"] == "1");
  editLOINC->Text          = initial_state["loinc_code"].c_str();
  editCPT->Text            = initial_state["cpt_code"].c_str();
  editMSR->Text            = initial_state["msr_code"].c_str();
  // suggestion: there are also OBX-XAM exams, with no result, OBX-SKT skin-tests, OBX-CPT tests of another sort, etc. ...
  // all those codes could go here? which XAM, SKT, CPT, etc. it is? (individual fields?)

  editCachedMean->Text   = initial_state["cached_mean"].c_str();
  editCachedStdDev->Text = initial_state["cached_stddev"].c_str();
  editAbsMin->Text       = initial_state["abs_min"].c_str();
  editAbsMax->Text       = initial_state["abs_max"].c_str();

  comboType->Text        = lookupTables->ValueOf("obs_question_types", initial_state["obs_question_id"]).c_str();
  comboOptionSet->Text   = lookupTables->ValueOf("obs_option_set_types", initial_state["obs_option_set_id"]).c_str();
  comboOptionSetChange(NULL);

  comboDefaultOption->Text = lookupTables->ValueOf("obs_option_types", initial_state["default_option_id"]).c_str();
  comboUnit->Text          = lookupTables->ValueOf("units", initial_state["unit_id"]).c_str();
  comboImprovement->Text   = lookupTables->ValueOf("improvement_types", initial_state["improvement_id"]).c_str();
  comboTypeChange(comboType);
  comboUnitChange(comboUnit);

  TformRecordGeneric::PutDataOnScreen();
  return true;
}
//---------------------------------------------------------------------------
bool TformRecordObservationQuestion::GetDataFromScreen()
{
  ending_state["id"] = initial_state["id"];

  ending_state["item"]          = editItem->Text.c_str();
  ending_state["short_item"]    = editShortItem->Text.c_str();
  ending_state["long_item"]     = editLongItem->Text.c_str();
  ending_state["valid"]         = (checkValid->Checked ? "1" : "0");
  ending_state["is_calculated"] = (checkCalculated->Checked ? "1" : "0");
  ending_state["loinc_code"]    = editLOINC->Text.c_str();
  ending_state["cpt_code"]      = editCPT->Text.c_str();
  ending_state["msr_code"]      = editMSR->Text.c_str();

  ending_state["cached_mean"]   = initial_state["cached_mean"];
  ending_state["cached_stddev"] = initial_state["cached_stddev"];
  ending_state["abs_min"] = editAbsMin->Text.c_str();
  ending_state["abs_max"] = editAbsMax->Text.c_str();

  ending_state["obs_question_id"]   = lookupTables->IdOf("obs_question_types", comboType->Text.c_str());
  ending_state["obs_option_set_id"] = lookupTables->IdOf("obs_option_set_types", comboOptionSet->Text.c_str());
  ending_state["default_option_id"] = lookupTables->IdOf("obs_option_types", comboDefaultOption);

  ending_state["unit_id"]           = lookupTables->IdOf("units", comboUnit->Text.c_str());
  ending_state["improvement_id"]    = lookupTables->IdOf("improvement_types", comboImprovement->Text.c_str());

  TformRecordGeneric::GetDataFromScreen();
  return true;
}
//---------------------------------------------------------------------------
bool TformRecordObservationQuestion::ScreenLock()
{
  panelGroup->Enabled = false;
  TformRecordGeneric::ScreenLock();
  return true;
}
//---------------------------------------------------------------------------
bool TformRecordObservationQuestion::ScreenUnlock()
{
  panelGroup->Enabled = true;
  TformRecordGeneric::ScreenUnlock();
  return true;
}
//---------------------------------------------------------------------------
bool TformRecordObservationQuestion::IsNewAgain()
{
  ranges_filter.SetSQLStatement(string(""));
  ranges_filter.RefreshDataSet();
  ranges_filter.SetAddEdit(this, NULL, NULL);

  TformRecordGeneric::IsNewAgain();
  return true;
}
//---------------------------------------------------------------------------
void TformRecordObservationQuestion::AtlasRefreshListView(int request_num, string matched_value)
{
 if (request_num == ranges_request)
  ranges_filter.RefreshDataSet();
}
//---------------------------------------------------------------------------
void TformRecordObservationQuestion::AddRange()
{
  map<string,string> temp_map;
  temp_map["id"] = "new";
  temp_map["obs_question_id"] = initial_state["id"];
  TformRecordObsExpectedRange::Do(temp_map);
}
//---------------------------------------------------------------------------
void TformRecordObservationQuestion::EditRange()
{
  map<string,string> temp_map;
  temp_map["id"] = ranges_filter.GetSelectedRecordID();
  TformRecordObsExpectedRange::Do(temp_map);
}
//---------------------------------------------------------------------------
void TformRecordObservationQuestion::Do(map<string, string> & p_parameters)
{
 FORM_TRY
  TformRecordObservationQuestion * z = new TformRecordObservationQuestion(NULL, p_parameters);
  z->Show();
 FORM_CATCH
}
//---------------------------------------------------------------------------
void __fastcall TformRecordObservationQuestion::comboTypeChange(TObject *Sender)
{
 bool need_option_set = is_in(lookupTables->IdOf("obs_question_types", comboType->Text.c_str()), "1", "3", NULL); // TODO -c MAGIC:
 bool need_unit = (lookupTables->IdOf("obs_question_types", comboType->Text.c_str()) == "4"); // TODO -c MAGIC:
 bool can_have_improvement = is_in(lookupTables->IdOf("obs_question_types", comboType->Text.c_str()), "3", "4", NULL);

 comboOptionSet->Enabled = need_option_set;
 buttonEditOptionSet->Enabled = need_option_set;

 comboUnit->Enabled = need_unit;
 buttonEditUnit->Enabled = need_unit;

 editAbsMin->Enabled = need_unit;
 editAbsMax->Enabled = need_unit;
 if (!need_unit) // I'm feeling mean.
 {
  editAbsMin->Text = "";
  editAbsMax->Text = "";
 }

 comboImprovement->Enabled = can_have_improvement;
}
//---------------------------------------------------------------------------
void __fastcall TformRecordObservationQuestion::buttonAddUnitClick(TObject *Sender)
{
 map<string, string> params;
 params["id"] = "new";
 TformRecordMetricUnit::Do(params);
}
//---------------------------------------------------------------------------
void __fastcall TformRecordObservationQuestion::staticQuestionSetEditorClick(TObject *Sender)
{
 TformSpecialObservationQuestionSetEditor::Do();
}
//---------------------------------------------------------------------------
void __fastcall TformRecordObservationQuestion::buttonImprovementInfoClick(TObject *Sender)
{
 PSay("Improvement applies only to things which can be sorted: metrics and ordinals.\n"
      "\n"
      "With metrics, 'higher' and 'lower' seem obvious.\n"
      "\n"
      "Ordinal options are sorted by 'rank', and a higher 'rank' qualifies as 'higher' here.\n"
      "When setting up ordinal options, pay attention to the order in which they are ranked.\n"
      "\n"
      "Choice questions also place their options in order by 'rank', but for display purposes only.\n"
      "They cannot be compared by rank.", m_message, MB_OK);
}
//---------------------------------------------------------------------------
void __fastcall TformRecordObservationQuestion::comboUnitChange(TObject *Sender)
{
 labelUnitsMean->Caption = comboUnit->Text;
 labelUnitsStdDev->Caption = comboUnit->Text;
 labelUnitsMin->Caption = comboUnit->Text;
 labelUnitsMax->Caption = comboUnit->Text;
}
//---------------------------------------------------------------------------
void __fastcall TformRecordObservationQuestion::buttonSearchCptClick(TObject *Sender)
{
  vector<string> return_vector;
  string sql;
  FORM_TRY
    TformSearchCode * codeSearch = new TformSearchCode(NULL, &return_vector);
    codeSearch->cboCode->Text = "CPT-4";
    codeSearch->ShowTaskModal(this);
  FORM_CATCH
  if (!return_vector.empty())
  {
   sql = "select code from codes where id = " + return_vector[0];
   editCPT->Text = GetOneField(dbDatabase, sql).c_str();
   if (editCPT->Text != "")
   {
    string new_note;
    string entered_note = UppercaseString(editItem->Text.c_str());
    string sql = "select description from codes where code = '";
    sql += editCPT->Text.c_str(); // TODO: SAFESQLSTRING -- do this on condition and procedure screens too!
    sql += "' and code_id = " + lookupTables->IdOf("code_types", "CPT-4");
    string code_note = GetOneField(dbDatabase, sql);
    if (entered_note == "")
     new_note = code_note;
    else
    {
     if(entered_note.find(code_note) == string::npos)
      new_note = entered_note + "\n\n" + code_note;
     else
      new_note = entered_note;
    }
    editItem->Text = SentenceString(new_note).c_str();
   }
  }}
//---------------------------------------------------------------------------
void __fastcall TformRecordObservationQuestion::buttonExpandCptClick(TObject *Sender)
{
  FORM_TRY
    TformSearchCode * codeSearch = new TformSearchCode(NULL, NULL);
    codeSearch->cboCode->Text = "CPT-4";
    codeSearch->editCode->Text = editCPT->Text;
    codeSearch->Show();
    if (editCPT->Text != "")
     codeSearch->buttonSearch->Click();
  FORM_CATCH
}
//---------------------------------------------------------------------------
void __fastcall TformRecordObservationQuestion::comboOptionSetChange(TObject *Sender)
{
  if (lookupTables->TableName(comboDefaultOption) != "") // if this cbo box is registered
    lookupTables->UnregisterComboBox("obs_option_types", comboDefaultOption);
  string obs_id = lookupTables->IdOf("obs_option_set_types", comboOptionSet); if (obs_id == "") obs_id = "0";
  lookupTables->RegisterComboBox("obs_option_types", comboDefaultOption, "select id, item, valid from obs_option_types where obs_option_set_id = " + obs_id);
}
//---------------------------------------------------------------------------

