//---------------------------------------------------------------------------

#include <vcl.h>
#pragma hdrstop

#include "codeRecordObservationResponseSet.h"
#include "reportObsQuestionSet.h"
#include "codeSpecialObsExpectedRanges.h"
#include "codeSpecialObservationQuestionSetEditor.h"
//---------------------------------------------------------------------------
#pragma package(smart_init)
#pragma link "codeRecordGeneric"
#pragma link "codeRecordObservationResponseSet"
#pragma link "codeHalfButton"
#pragma resource "*.dfm"
TformRecordObservationResponseSet *formRecordObservationResponseSet;
//---------------------------------------------------------------------------
__fastcall TformRecordObservationResponseSet::TformRecordObservationResponseSet(TComponent* Owner, map<string,string> p_parameters)
        : TformRecordGeneric(Owner, p_parameters, string("obs_response_sets"))
{
 middle_section = new obs_row_container(scrollResponses);
 middle_section->set_ready(false);
 staticPrintFilled->Font->Color     = link_color;
 staticPrintSemiFilled->Font->Color = link_color;
 staticPrintEmpty->Font->Color      = link_color;
 do_not_refresh = false;

 // TODO -- inaccurate?
 buttonEditQuestionSet->Enabled = (optionsPermissions["table.obs_questions"]["update"] == "true");

 Initialize();
}
//---------------------------------------------------------------------------
bool TformRecordObservationResponseSet::ScreenStart()
{
 dateObserved->Date = Now();
 if (is_new_record)
 {
  string sql = "select obs_question_set_types.id, obs_question_set_types.item, obs_question_set_types.valid from "
               "obs_question_set_types "
               "inner join permissions p1 on p1.s = obs_question_set_types.permission_name and p1.k = 'INSERT' and p1.v = 'TRUE' and p1.user_id = " + current_userid + " "
               "inner join permissions p2 on p2.s = obs_question_set_types.permission_name and p2.k = 'VIEW' and p2.v = 'TRUE' and p2.user_id = " + current_userid + " ";
  lookupTables->RegisterComboBox("obs_question_set_types", comboQuestionSet, sql);
 } else
  lookupTables->RegisterComboBox("obs_question_set_types", comboQuestionSet);
 TformRecordGeneric::ScreenStart();
 if (is_editing)
  NotNewAnymore();
 return true;
}
//----------------------------------------------------------------------------
bool TformRecordObservationResponseSet::ScreenStop()
{
 lookupTables->UnregisterComboBox("obs_question_set_types", comboQuestionSet);
 delete middle_section;
 TformRecordGeneric::ScreenStop();
 return true;
}
//----------------------------------------------------------------------------
bool TformRecordObservationResponseSet::IsValidOnScreen()
{
 if (!lookupTables->IsInLookUpTable("obs_question_set_types", comboQuestionSet))
 {
   PSay("Please enter a valid question set.", m_invalid_data, MB_OK);
   comboQuestionSet->SetFocus();
   return false;
 }

 if (is_new_record && optionsPermissions[LowercaseString(GetOneField(dbDatabase, "select permission_name from obs_question_set_types where id = " + lookupTables->IdOf("obs_question_set_types", comboQuestionSet)))]["insert"] != "true")
 {
  PSay("You are not permitted to enter responses to this particular set of questions.", m_warning, MB_OK);
  return false;
 }

 if (is_new_record && optionsPermissions[LowercaseString(GetOneField(dbDatabase, "select permission_name from obs_question_set_types where id = " + lookupTables->IdOf("obs_question_set_types", comboQuestionSet)))]["view"] != "true")
 {
  PSay("You are not permitted to view responses to this particular set of questions;\nthis is an awkward situation (insert but not view,) please speak to a programmer.", m_warning, MB_OK);
  return false;
 }

 if (is_editing && optionsPermissions[LowercaseString(GetOneField(dbDatabase, "select permission_name from obs_question_set_types where id = " + lookupTables->IdOf("obs_question_set_types", comboQuestionSet)))]["update"] != "true")
 {
  PSay("You are not permitted to update responses to this particular set of questions.", m_warning, MB_OK);
  return false;
 }

 if (middle_section)
 {
  middle_section->auto_calc(); // pre-calc BMI and whatnot so they can be checked for dangerously stupid values.
  if (!middle_section->are_valid(false)) // not silent, ergo, will tell user what's wrong
   return false;
  if (middle_section->empty())
  {
   PSay("You must enter at least one response.", m_warning, MB_OK);
   return false;
  }
 }

 return true;
}
//----------------------------------------------------------------------------
bool TformRecordObservationResponseSet::PutDataOnScreen()
{
  do_not_refresh = true;
  CreateLink(staticPerson, NULL, this, initial_state["person_id"], "people");
  CreateLink(staticTask, NULL, this, initial_state["task_id"], "tasks");
  CreateLink(staticEvent, NULL, this, initial_state["event_id"], "events");
  CreateLink(staticParticipation, NULL, this, initial_state["participations_id"], "participations");

  if (initial_state["date_of_observation"] != "")
    dateObserved->Date = Pdate(initial_state["date_of_observation"], mdy | yyyy).OutDate();
  else
    dateObserved->Date = Date();
  checkSteps->Checked = (initial_state["is_steps_observation"] == "1");

  vector<string> available_values = lookupTables->GetValueList("obs_question_set_types", comboQuestionSet);
  if (is_new_record && available_values.size() == 1)
   initial_state["obs_question_set_id"] = lookupTables->IdOf("obs_question_set_types", available_values[0], comboQuestionSet);

  comboQuestionSet->Text = PrettyString(lookupTables->ValueOf("obs_question_set_types", initial_state["obs_question_set_id"])).c_str();
  comboQuestionSet->Enabled = !IsInt(initial_state["id"]); // don't let them change it once the record is saved
  comboQuestionSetChange(comboQuestionSet);

  TformRecordGeneric::PutDataOnScreen();
  do_not_refresh = false;
  return true;
}
//----------------------------------------------------------------------------
bool TformRecordObservationResponseSet::GetDataFromScreen()
{
 ending_state["id"] = initial_state["id"];
 // other finger-pointing handled by codeRecordGeneric

 ending_state["person_id"]         = IntToString(max(0, StringToInt(initial_state["person_id"])));
 ending_state["task_id"]           = IntToString(max(0, StringToInt(initial_state["task_id"])));
 ending_state["event_id"]          = IntToString(max(0, StringToInt(initial_state["event_id"])));
 ending_state["participations_id"] = IntToString(max(0, StringToInt(initial_state["participations_id"])));

 if (comboQuestionSet->Text != "")
  ending_state["obs_question_set_id"] = lookupTables->IdOf("obs_question_set_types", comboQuestionSet);
 else
  ending_state["obs_question_set_id"] = "0";

 ending_state["is_steps_observation"] = (checkSteps->Checked ? "1" : "0");

 // users never see this, determines security-ish stuff (again, could have been handled by a security_policy_id, or as Paul suggested, a permission_id)
 if(StringToInt(initial_state["group_id"]) > 0)
  ending_state["group_id"] = initial_state["group_id"];
 else if (is_new_record)
  ending_state["group_id"] = current_groupid;

 ending_state["date_of_observation"] = Pdate(dateObserved->Date).OutDate(mdy | yyyy);

 // jhaley 20110527: pass through any patient assessment ID that might be attached to this obs_response_set
 ending_state["patient_assessment_id"] = initial_state["patient_assessment_id"];

 TformRecordGeneric::GetDataFromScreen();
 return true;
}
//----------------------------------------------------------------------------
bool TformRecordObservationResponseSet::ScreenLock()
{
 // disable group boxes
 groupResponseSet->Enabled = false;
 middle_section->set_lock(true);
 comboQuestionSetChange(NULL);
 TformRecordGeneric::ScreenLock();
 buttonA->Enabled = (optionsPermissions[LowercaseString(GetOneField(dbDatabase, "select permission_name from obs_question_set_types where id = " + lookupTables->IdOf("obs_question_set_types", comboQuestionSet)))]["update"] == "true");
 return true;
}
//----------------------------------------------------------------------------
bool TformRecordObservationResponseSet::ScreenUnlock()
{
 // enable group boxes
 groupResponseSet->Enabled = true;
 middle_section->set_lock(false);
 comboQuestionSetChange(NULL);
 TformRecordGeneric::ScreenUnlock();
 return true;
}
//---------------------------------------------------------------------------
bool TformRecordObservationResponseSet::BeforeSave()
{
 return (middle_section->save_entries(lock_transaction, ending_state["id"]));
}
//-----------------------------------------------------------------------------
bool TformRecordObservationResponseSet::NotNewAnymore()
{
 // TODO: yes, this causes two refreshes in the record window that just 'saved' ... but ...
 // we avoid double-refreshes at each save/undo,
 // and we avoid not refreshing when another window modifies stuff,
 // and we avoid having to add atlas notifications to the .h file
 // (but we could do that, and get all benefits, with work -- except for transactionality,
 //  only sending updates when we commit, and the commit being in a different -class- altogether, and all that rot.)
 request_responses = atlas->RequestNotificationWhen("obs_response_sets.id", initial_state["id"], rel_e, as_int, this, RefreshHandler(&(TformRecordObservationResponseSet::AtlasRefreshListViews)));

 TformRecordGeneric::NotNewAnymore();
 return true;
}
//----------------------------------------------------------------------------
bool TformRecordObservationResponseSet::IsNewAgain()
{
 // ?

 // go ... fetch me some data, boy!

 if (!IsBlankOrZero(initial_state["task_id"]))
 {
  if (initial_state["date_of_observation"].empty())
   initial_state["date_of_observation"] = GetOneField(dbDatabase, "select date_of_task from tasks where id = " + initial_state["task_id"]);
  if (initial_state["person_id"].empty())
   initial_state["person_id"] = GetOneField(dbDatabase, "select person_id from tasks where id = " + initial_state["task_id"]);
  if (initial_state["group_id"].empty())
   initial_state["group_id"] = GetOneField(dbDatabase, "select group_id from tasks where id = " + initial_state["task_id"]);
 }

 if (!IsBlankOrZero(initial_state["participations_id"]))
 {
  if (initial_state["person_id"].empty())
   initial_state["person_id"] = GetOneField(dbDatabase, "select person_id from participations where id = " + initial_state["participations_id"]);
  if (initial_state["event_id"].empty())
   initial_state["event_id"] = GetOneField(dbDatabase, "select event_id from participations where id = " + initial_state["participations_id"]);
  if (initial_state["group_id"].empty())
   initial_state["group_id"] = GetOneField(dbDatabase, "select group_id from participations where id = " + initial_state["participations_id"]);
 }

 if (!IsBlankOrZero(initial_state["event_id"]))
 {
  if (initial_state["date_of_observation"].empty())
   initial_state["date_of_observation"] = GetOneField(dbDatabase, "select date_of_event from events where id = " + initial_state["event_id"]);
  if (initial_state["group_id"].empty())
   initial_state["group_id"] = GetOneField(dbDatabase, "select group_id from events where id = " + initial_state["event_id"]);
 }

 comboQuestionSetChange(comboQuestionSet);
 TformRecordGeneric::IsNewAgain();
 return true;
}
//---------------------------------------------------------------------------
bool TformRecordObservationResponseSet::AtlasRefreshListViews(int request_num, string matched_value)
{
 if (request_num == request_responses)
  comboQuestionSetChange(comboQuestionSet);
 return true;
}
//--------------------------------------------------------------------------
bool TformRecordObservationResponseSet::SendAtlas()
{
 if (StringToInt(ending_state["person_id"]) > 0)
  atlas->Notify("obs_response_sets.person_id", ending_state["person_id"]);
 if (StringToInt(ending_state["task_id"]) > 0)
  atlas->Notify("obs_response_sets.task_id", ending_state["task_id"]);
 if (StringToInt(ending_state["participations_id"]) > 0)
  atlas->Notify("obs_response_sets.participations_id", ending_state["participations_id"]);
 /* TODO: Why were these not already refreshing?  Looked at the code in RecordGeneric::SendAtlas
    and it looked fine; watched messages Atlas was sending and it sent id, event_id, group_id, and
    current_id, but not anything that frameObservationList actually listens for.  So, this hack. */
 return TformRecordGeneric::SendAtlas();
}
//--------------------------------------------------------------------------
bool TformRecordObservationResponseSet::DeleteRecord()
{
 if (ExecuteStatement(delete_transaction, "delete from obs_responses where obs_response_set_id = " + initial_state["id"]))
 {
  return TformRecordGeneric::DeleteRecord();
 } else
 {
  PSay("Failed to delete obs_responses; cancelled.", m_database_error, MB_OK);
  return false;
 }
}
//---------------------------------------------------------------------------
void __fastcall TformRecordObservationResponseSet::FormClose(TObject *Sender, TCloseAction &Action)
{
 TformRecordGeneric::FormClose(Sender, Action);
}
//---------------------------------------------------------------------------
void __fastcall TformRecordObservationResponseSet::comboQuestionSetChange(TObject *Sender)
{
 // reload right side
 if (lookupTables->IsInLookUpTable("obs_question_set_types", comboQuestionSet))
 {
  string q_id = lookupTables->IdOf("obs_question_set_types", comboQuestionSet);

  // NOTE: the following should be a reminder that codeRecordGeneric could've been better designed.
  // Namely, the transaction -it- uses should be available to us here. this is already true when saving (for rollback purposes).
  // Just one of those things, eh? (And yes, I have the right to say the above. I wrote the fucker, I should know.)
  // NOTE: the above applies less now that this isn't in PutDataOnScreen (at least not directly)
  // NOTE: also, is_editing and is_new_record get set at odd times, such that they're not available when you'd want them. fuckers.

  if (!do_not_refresh)
  {
   TIBTransaction * dbTransaction = new TIBTransaction(NULL);
   StdTransaction (dbTransaction, dbDatabase);
   try
   {
    if (q_id == "")
     q_id = "0";
    // avoid incessant refreshing and size oddities (hopefully)
    scrollResponses->Visible = false;
    if (!middle_section->load_entries(dbTransaction, q_id, initial_state["id"]))
     throw(1);
    scrollResponses->Visible = true;
   } catch (...)
   {
    PSay ("Failed to update right-hand side to match rest of screen. (Not good.)\nPlease inform a programmer.", m_database_error, MB_OK);
   }
   if (dbTransaction->Active)
    dbTransaction->Commit();
   delete dbTransaction;
  }

  buttonViewRanges->Enabled = IsInt(q_id);
  // i apparently had some doubts about the location of the following line? no longer sure why.
  staticPrintFilled->Enabled = IsInt(initial_state["id"]);
  staticPrintSemiFilled->Enabled = IsInt(initial_state["id"]);
  staticPrintEmpty->Enabled = IsInt(q_id);
 } else
 {
  staticPrintFilled->Enabled = false;
  staticPrintSemiFilled->Enabled = false;
  staticPrintEmpty->Enabled = false;
 }
}
//---------------------------------------------------------------------------
void __fastcall TformRecordObservationResponseSet::FormShow(TObject *Sender)
{
 TformRecordGeneric::FormShow(Sender);
 panelValues->Color = AverageColor(StringToColor("clBtnFace"), StringToColor("clWhite"));
 middle_section->set_ready(true);
}
//---------------------------------------------------------------------------
void __fastcall TformRecordObservationResponseSet::buttonEditQuestionSetClick(TObject *Sender)
{
 TformSpecialObservationQuestionSetEditor::Do(lookupTables->IdOf("obs_question_set_types", comboQuestionSet));
}
//---------------------------------------------------------------------------
void __fastcall TformRecordObservationResponseSet::buttonViewRangesClick(TObject *Sender)
{
 // don't ask me how this is different. maybe disable any sort of underlining? make it -not- look like there's room for responses?
 if (!lookupTables->IsInLookUpTable("obs_question_set_types", comboQuestionSet))
  return;

 string gender_id = "0";
 string obs_question_set_id = lookupTables->IdOf("obs_question_set_types", comboQuestionSet);

 if (!IsBlankOrZero(initial_state["person_id"]))
  gender_id = GetOneField(dbDatabase, "select gender_id from people where id = " + initial_state["person_id"]);

 TformSpecialObsExpectedRanges::Do(gender_id, obs_question_set_id);
}
//---------------------------------------------------------------------------
void __fastcall TformRecordObservationResponseSet::scrollResponsesMouseWheelDown(
      TObject *Sender, TShiftState Shift, TPoint &MousePos, bool &Handled)
{
  if(scrollResponses->Visible)
    scrollResponses->VertScrollBar->Position += scrollResponses->VertScrollBar->Increment;
}
//---------------------------------------------------------------------------

void __fastcall TformRecordObservationResponseSet::scrollResponsesMouseWheelUp(
      TObject *Sender, TShiftState Shift, TPoint &MousePos, bool &Handled)
{
  if(scrollResponses->Visible)
    scrollResponses->VertScrollBar->Position -= scrollResponses->VertScrollBar->Increment;
}
//---------------------------------------------------------------------------
void TformRecordObservationResponseSet::Do(map<string, string> & p_parameters)
{
 FORM_TRY
  TformRecordObservationResponseSet * z = new TformRecordObservationResponseSet(NULL, p_parameters);
  z->Show();
 FORM_CATCH
}
//---------------------------------------------------------------------------
void __fastcall TformRecordObservationResponseSet::staticPrintEmptyClick(TObject *Sender)
{
 if (!lookupTables->IsInLookUpTable("obs_question_set_types", comboQuestionSet))
  return;
 TformSpecialObsQuestionSetReport::Run(lookupTables->IdOf("obs_question_set_types", comboQuestionSet), "", initial_state["person_id"], "", "", false);

}
//---------------------------------------------------------------------------
void __fastcall TformRecordObservationResponseSet::staticPrintSemiFilledClick(TObject *Sender)
{
 if (!lookupTables->IsInLookUpTable("obs_question_set_types", comboQuestionSet))
  return;
 TformSpecialObsQuestionSetReport::Run(lookupTables->IdOf("obs_question_set_types", comboQuestionSet), initial_state["id"], initial_state["person_id"], "", "", true);
}
//---------------------------------------------------------------------------
void __fastcall TformRecordObservationResponseSet::staticPrintFilledClick(TObject *Sender)
{
 if (!lookupTables->IsInLookUpTable("obs_question_set_types", comboQuestionSet))
  return;
 TformSpecialObsQuestionSetReport::Run(lookupTables->IdOf("obs_question_set_types", comboQuestionSet), initial_state["id"], initial_state["person_id"], "", "", false);
}
//---------------------------------------------------------------------------
bool TformRecordObservationResponseSet::CheckForUselessRecord()
{
  // they'll get a warning anyway if they didn't enter any observations,
  // and they're currently getting these messages too often.
  // my guess is it had something to do with all the fields on the left being set in advance when adding obs to tasks,
  // particular for users with permission only to one obs-set such that the obs-set, the date, the group, the person,
  // the task, etc. were all already filled-in in initial_state, which meant initial_state == ending_state, ergo errors.
  return true;
}
//---------------------------------------------------------------------------



