//---------------------------------------------------------------------------

#ifndef codeRecordObservationResponseSetH
#define codeRecordObservationResponseSetH
//---------------------------------------------------------------------------
#include <Classes.hpp>
#include <Controls.hpp>
#include <StdCtrls.hpp>
#include <Forms.hpp>
#include "codeRecordGeneric.h"
#include <ComCtrls.hpp>
#include <Menus.hpp>
#include <ToolWin.hpp>
#include <ExtCtrls.hpp>
#include "codeFrameObservationResponse.h"
#include <Buttons.hpp>
#include "metric_conversion.h"
#include "codeHalfButton.h"
#include "derivedheaders.h"
//---------------------------------------------------------------------------
struct obs_row_pointers
{
 // yes, i know, i don't need the struct. just in case it -were- needed at some point, ya know?
 // this code was copied and modified from elsewhere, and i wanted to keep the same structure
 // (particularly to make it easier to convert to a single, nicely-designed, editor class thingie)
 // (you're probably reading this thinking "ha! that never happened. lame!". you're right. we suck.)
 TframeObservationResponse * entry_any;
};

class obs_row_container
{
 private:
  string _obs_question_set_id;
  string _obs_response_set_id;
  bool _ready;

  vector<obs_row_pointers> custom_rows;
  bool should_be_locked;
  bool is_loading; // jhaley 20101207: reentrancy protection in load_entries
  TWinControl * current_containing_window;
 public:
  obs_row_container (TWinControl * containing_window) : current_containing_window(containing_window), should_be_locked(false), is_loading(false) {}

  void set_ready (bool v)
  {
   _ready = v;
   for (vector<obs_row_pointers>::iterator i_row = custom_rows.begin(); i_row != custom_rows.end(); i_row++)
    (*i_row).entry_any->Ready = v;
  }

  TframeObservationResponse * find_by_q_id (string q_id)
  {
   map<string, string> t;
   for (vector<obs_row_pointers>::iterator i_row = custom_rows.begin(); i_row != custom_rows.end(); i_row++)
   {
    t = i_row->entry_any->Row;
    if (t["obs_question_id"] == q_id)
     return i_row->entry_any;
   }
   return NULL;
  }

  bool are_valid (bool silent)
  {
   for (vector<obs_row_pointers>::iterator i_row = custom_rows.begin(); i_row != custom_rows.end(); i_row++)
    // don't check auto-calc ones, there's nothing the user can do about it.
    // nevermind. this can be rather important -- two values could both be "allowed", but their ratio -way- out of whack. (73" tall, 5 pounds heavy. each allowed individually, but the BMI gets a warning.)
    if (/*!i_row->entry_any->IsAutoCalc() &&*/ !i_row->entry_any->IsValid(silent))
     return false;

   // extra curly braces to avoid problems when copy-pasting and forgetting to fix variable names =)

   // "are you in pain? if so, tell me about it"
   {
    TframeObservationResponse * pain_current = find_by_q_id("57");
    TframeObservationResponse * pain_current_desc = find_by_q_id("58");
    if (pain_current && pain_current_desc && !pain_current_desc->Blank && (pain_current->Blank || lookupTables->ValueOf("obs_option_types", pain_current->Row["obs_option_id"]) == "No"))
    {
     PSay("Please do not enter a pain description if the patient is not currently in pain.", m_missing_data, MB_OK, "");
     return false;
    }
   }

   // "do you smoke? if so, how much?"
   {
    TframeObservationResponse * smoking_current = find_by_q_id("59");
    TframeObservationResponse * smoking_current_quantity = find_by_q_id("125");
    if (smoking_current && smoking_current_quantity && !smoking_current_quantity->Blank && (smoking_current->Blank || lookupTables->ValueOf("obs_option_types", smoking_current->Row["obs_option_id"]) == "No"))
    {
     PSay("Please do not enter a smoking quantity if the patient is not currently smoking.", m_missing_data, MB_OK, "");
     return false;
    }
   }

   // "have you smoked? if so, for how long?"
   {
    TframeObservationResponse * smoking_history = find_by_q_id("60");
    TframeObservationResponse * smoking_history_duration = find_by_q_id("61");
    if (smoking_history && smoking_history_duration && !smoking_history_duration->Blank && (smoking_history->Blank || lookupTables->ValueOf("obs_option_types", smoking_history->Row["obs_option_id"]) == "No"))
    {
     PSay("Please do not enter a smoking history duration if the patient does not have a history of smoking.", m_missing_data, MB_OK, "");
     return false;
    }
   }

   // "are you under a doctor's care? if so, why?"
   {
    TframeObservationResponse * care_current = find_by_q_id("64");
    TframeObservationResponse * care_current_reason = find_by_q_id("65");
    if (care_current && care_current_reason && !care_current_reason->Blank && (care_current->Blank || lookupTables->ValueOf("obs_option_types", care_current->Row["obs_option_id"]) == "No"))
    {
     PSay("Please do not enter a reason for being under a doctor's care if the patient is not under a doctor's care.", m_missing_data, MB_OK, "");
     return false;
    }
   }

   // "have you been hospitalized? if so, why?"
   {
    TframeObservationResponse * hosp_history = find_by_q_id("66");
    TframeObservationResponse * hosp_history_reason = find_by_q_id("67");
    if (hosp_history && hosp_history_reason && !hosp_history_reason->Blank && (hosp_history->Blank || lookupTables->ValueOf("obs_option_types", hosp_history->Row["obs_option_id"]) == "No"))
    {
     PSay("Please do not enter a reason for being hospitalized if the patient has not been hospitalized.", m_missing_data, MB_OK, "");
     return false;
    }
   }

   // "have you been irradiated? if so, why?"
   {
    TframeObservationResponse * rad_history = find_by_q_id("68");
    TframeObservationResponse * rad_history_reason = find_by_q_id("69");
    if (rad_history && rad_history_reason && !rad_history_reason->Blank && (rad_history->Blank || lookupTables->ValueOf("obs_option_types", rad_history->Row["obs_option_id"]) == "No"))
    {
     PSay("Please do not enter a reason for having radiation treatment if the patient has not had radiation treatment.", m_missing_data, MB_OK, "");
     return false;
    }
   }

   // "have you had a pap smear in the last year? if so, when?"
   {
    TframeObservationResponse * pap_history = find_by_q_id("126");
    TframeObservationResponse * pap_history_date = find_by_q_id("127");
    if (pap_history && pap_history_date && !pap_history_date->Blank && (pap_history->Blank || lookupTables->ValueOf("obs_option_types", pap_history->Row["obs_option_id"]) == "No"))
    {
     PSay("Please do not enter a pap smear date if the patient has not had a pap smear in the last year.", m_missing_data, MB_OK, "");
     return false;
    }
   }
   return true;
  }

  void set_lock (bool value)
  {
   should_be_locked = value;
   for (vector<obs_row_pointers>::iterator i_row = custom_rows.begin(); i_row != custom_rows.end(); i_row++)
    i_row->entry_any->panelObservation->Enabled = !should_be_locked;
  }

  bool load_entries (TIBTransaction * dbTransaction, string obs_question_set_id, string obs_response_set_id)
  {
   // jhaley 20101207: This routine can execute recursively via one of the most tortured call stacks I've encountered.
   // If the application message queue is pumped during the running of this routine (for example via DebugSql, but there
   // are other ways for it to happen too), then the message gets sent and serviced by TformRecordObservationResponseSet,
   // which calls this again before it has exited from the original call. We must prevent this.
   if (is_loading)
     return true; // pretend is successful, or a spurious error message may appear.

   // Now in process of loading. Put out the proverbial Do-Not-Disturb sign.
   is_loading = true;

   int prev_scroll = -1;
   TScrollBox* scroll_box = (TScrollBox*)current_containing_window;
   if (current_containing_window->InheritsFrom(__classid(TScrollBox)))
    prev_scroll = scroll_box->VertScrollBar->Position;

   // we hope it doesn't contain anything else of importance ... oops =)
   current_containing_window->DestroyComponents();
   custom_rows.clear();
   current_containing_window->Repaint();

   _obs_question_set_id = obs_question_set_id;
   _obs_response_set_id = obs_response_set_id;
   if (_obs_response_set_id == "" || _obs_response_set_id == "new")
    _obs_response_set_id = "0";

   vector<map<string, string> > entries_and_values;

   string sql = "select "
                /* question */
                "obs_questions.id obs_question_id, obs_questions.obs_question_id obs_question_type_id, "
                "obs_questions.item, obs_questions.short_item, obs_questions.item, "
                "obs_questions.cpt_code, units.dimension_id, "
                "obs_questions.unit_id, obs_questions.obs_option_set_id, obs_questions.is_calculated, "
                "obs_questions_in_sets.obs_question_set_id, obs_questions.default_option_id, "
                /* response */
                "obs_responses.id, obs_responses.obs_response_set_id, obs_responses.obs_option_id, "
                "obs_responses.response_text, obs_responses.response_metric, obs_responses.response_date "
                /* and now, some joins! */
                "from obs_questions "
                "left join units on units.id = obs_questions.unit_id "
                "left outer join obs_questions_in_sets on "
                "(obs_questions_in_sets.obs_question_id = obs_questions.id "
                "and obs_questions_in_sets.obs_question_set_id = " + _obs_question_set_id + ") "
                "left outer join obs_responses on "
                "(obs_responses.obs_question_id = obs_questions.id "
                "and obs_responses.obs_response_set_id = " + _obs_response_set_id + ") "
                "where "
                "obs_responses.obs_response_set_id is not null "
                + string(should_be_locked ? "" : "or obs_questions_in_sets.obs_question_set_id is not null ") +
                "order by obs_questions_in_sets.sequence asc";
   // jhaley: I really don't need to see this any more. Reenable it if there are problems with observation sets.
   //DebugSql(NULL, &sql, other_forms, 9, dbTransaction);
   if (!SqlToVecMap(dbTransaction, sql, entries_and_values))
   {
    is_loading = false; // jhaley: finished loading.
    return false;
   }
   else
   {
    unsigned int current_top = 1; // the scroll box starts at 15, most items normally start at 16 (inside group box)

    if (!entries_and_values.empty())
    {
     TPanel * spacer = NULL;
     FORM_TRY
      spacer = new TPanel(current_containing_window);
     FORM_CATCH
     spacer->Parent = current_containing_window;
     spacer->ParentColor = true;
     spacer->Align = alTop;
     spacer->Height = 6;
     spacer->BevelOuter = bvNone;
     spacer->Caption = "";
     current_top += spacer->Height;
    }

    for (vector<map<string, string> >::iterator i_entry = entries_and_values.begin(); i_entry != entries_and_values.end(); i_entry++)
    {
     obs_row_pointers new_row;
     new_row.entry_any = NULL;
     FORM_TRY
      new_row.entry_any = new TframeObservationResponse(current_containing_window); // we want it owned elsewhere so it's destroyed automatically
     FORM_CATCH
     new_row.entry_any->Row = (*i_entry);
     new_row.entry_any->Ready = _ready;
     new_row.entry_any->Parent = current_containing_window;
     new_row.entry_any->Top = current_top; // try to make sure they actually appear in the right order (not flip-flopping the order)
     new_row.entry_any->Align = alTop;
     custom_rows.push_back(new_row);
     current_top += new_row.entry_any->Height; // 24;
    }

    // none selected? tell the user.
    if (current_top == 1)
    {
     TLabel * oops_label = NULL;
     FORM_TRY
      oops_label = new TLabel(current_containing_window);
     FORM_CATCH
     oops_label->Top = current_top + 4;
     oops_label->Left = 8;
     oops_label->AutoSize = true;
     oops_label->Caption = "Nothing to show (no questions and/or no responses.)";
     oops_label->Visible = true;
     oops_label->Parent = current_containing_window;
    }
   }

   set_lock (should_be_locked); // 'cause things happen out of order, and are t3h 5tup1d

   if (current_containing_window->InheritsFrom(__classid(TScrollBox)))
    scroll_box->VertScrollBar->Position = prev_scroll;

   is_loading = false; // jhaley: finished loading.
   return true;
  }

  bool empty()
  {
   for (vector<obs_row_pointers>::iterator i_row = custom_rows.begin(); i_row != custom_rows.end(); i_row++)
    if (!i_row->entry_any->Blank)
     return false;
   return true;
  }

  void auto_calc ()
  {
   // cache all the values. partially just 'cause, partially to avoid dealing with the "order" in which auto-calcs occur.
   // this way, everything breaks. yippee!
   map<string, pair<map<string, string>, TframeObservationResponse *> > inverted_values;
   map<string, string> temp_values;
   for (vector<obs_row_pointers>::iterator i_row = custom_rows.begin(); i_row != custom_rows.end(); i_row++)
   {
    temp_values = i_row->entry_any->Row;
    temp_values["unit_id"] = GetOneField(dbDatabase, "select unit_id from obs_questions where obs_questions.id = " + temp_values["obs_question_id"]);
    inverted_values[temp_values["obs_question_id"]] = pair<map<string, string>, TframeObservationResponse *>(temp_values, i_row->entry_any);
   }

   map<string, pair<map<string, string>, TframeObservationResponse *> >::iterator i_sink, i_source;

   // auto-calculate BMI if possible
   i_sink = inverted_values.find("1"); // TODO -c MAGIC: BMI
   if (i_sink != inverted_values.end())
   {
    // default back to blank
    i_sink->second.second->editMetric->Text = "";
    // i_sink->second.second->checkObserved->Checked = false;

    i_source = inverted_values.find("2"); // TODO -c MAGIC: height
    if (i_source != inverted_values.end())
    {
     bool b_height = (i_source->second.first["response_metric"] != "");
     float f_height = StringToFloat(i_source->second.first["response_metric"]);
     string u_height = i_source->second.first["unit_id"];
     i_source = inverted_values.find("3"); // TODO -c MAGIC: weight
     if (i_source != inverted_values.end())
     {
      bool b_weight = (i_source->second.first["response_metric"] != "");
      float f_weight = StringToFloat(i_source->second.first["response_metric"]);
      string u_weight = i_source->second.first["unit_id"];

      // PSay("FloatConvertMetric(dbDatabase, " + FloatToString(f_height) + ", " + u_height + ", \"18\")", m_debug, MB_OK, "");
      // PSay("FloatConvertMetric(dbDatabase, " + FloatToString(f_weight) + ", " + u_weight + ", \"17\")", m_debug, MB_OK, "");

      float c_height_m = FloatConvertMetric(dbDatabase, f_height, u_height, "18");
      float c_weight_kg = FloatConvertMetric(dbDatabase, f_weight, u_weight, "17");
      if (b_weight && b_height && c_height_m)
      {
       i_sink->second.second->editMetric->Text = (FloatToString(c_weight_kg / (c_height_m * c_height_m))).c_str();
       // i_sink->second.second->checkObserved->Checked = true;
       // units are automatically chosen in this case. but ...
       // TODO: when we start using a control that lets users play with units, BMI needs to reset its units appropriately here.
      }
     } // else PSay("3 not found", m_debug, MB_OK, "");
    } // else PSay("2 not found", m_debug, MB_OK, "");
   } // else PSay("1 not found", m_debug, MB_OK, "");

   // auto-calculate waist/hip ratio if possible
   i_sink = inverted_values.find("54"); // TODO -c MAGIC: Waist/Hip ratio
   if (i_sink != inverted_values.end())
   {
    // default back to blank
    i_sink->second.second->editMetric->Text = "";
    // i_sink->second.second->checkObserved->Checked = false;

    i_source = inverted_values.find("4"); // TODO -c MAGIC: waist
    if (i_source != inverted_values.end())
    {
     bool b_waist = (i_source->second.first["response_metric"] != "");
     float f_waist = StringToFloat(i_source->second.first["response_metric"]);
     string u_waist = i_source->second.first["unit_id"];
     i_source = inverted_values.find("5"); // TODO -c MAGIC: hip
     if (i_source != inverted_values.end())
     {
      bool b_hip = (i_source->second.first["response_metric"] != "");
      float f_hip = StringToFloat(i_source->second.first["response_metric"]);
      string u_hip = i_source->second.first["unit_id"];

      // PSay("FloatConvertMetric(dbDatabase, " + FloatToString(f_waist) + ", " + u_waist + ", " + u_waist + ")", m_debug, MB_OK, "");
      // PSay("FloatConvertMetric(dbDatabase, " + FloatToString(f_hip) + ", " + u_hip + ", " + u_waist + ")", m_debug, MB_OK, "");

      float c_waist_any = FloatConvertMetric(dbDatabase, f_waist, u_waist, u_waist);
      float c_hip_any = FloatConvertMetric(dbDatabase, f_hip, u_hip, u_waist); // just convert to same units. blah.
      if (b_waist && b_hip && c_hip_any)
      {
       i_sink->second.second->editMetric->Text = (FloatToString(c_waist_any / c_hip_any)).c_str();
       // i_sink->second.second->checkObserved->Checked = true;
       // units are automatically chosen in this case. but ...
       // TODO: when we start using a control that lets users play with units, Waist/Hip needs to reset its units appropriately here.
      }
     } // else PSay("5 not found", m_debug, MB_OK, "");
    } // else PSay("4 not found", m_debug, MB_OK, "");
   } // else PSay("54 not found", m_debug, MB_OK, "");

   // auto-calculate pulse blood pressure if possible
   i_sink = inverted_values.find("55"); // TODO -c MAGIC: pulse blood pressure
   if (i_sink != inverted_values.end())
   {
    // default back to blank
    i_sink->second.second->editMetric->Text = "";
    // i_sink->second.second->checkObserved->Checked = false;

    i_source = inverted_values.find("12"); // TODO -c MAGIC: systolic
    if (i_source != inverted_values.end())
    {
     bool b_sys = (i_source->second.first["response_metric"] != "");
     float f_sys = StringToFloat(i_source->second.first["response_metric"]);
     string u_sys = i_source->second.first["unit_id"];
     i_source = inverted_values.find("13"); // TODO -c MAGIC: diastolic
     if (i_source != inverted_values.end())
     {
      bool b_dia = (i_source->second.first["response_metric"] != "");
      float f_dia = StringToFloat(i_source->second.first["response_metric"]);
      string u_dia = i_source->second.first["unit_id"];

      // PSay("FloatConvertMetric(dbDatabase, " + FloatToString(f_waist) + ", " + u_waist + ", " + u_waist + ")", m_debug, MB_OK, "");
      // PSay("FloatConvertMetric(dbDatabase, " + FloatToString(f_hip) + ", " + u_hip + ", " + u_waist + ")", m_debug, MB_OK, "");

      float c_sys_any = FloatConvertMetric(dbDatabase, f_sys, u_sys, u_sys);
      float c_dia_any = FloatConvertMetric(dbDatabase, f_dia, u_dia, u_dia); // just convert to same units. blah.
      if (b_sys && b_dia)
      {
       i_sink->second.second->editMetric->Text = (FloatToString(c_sys_any - c_dia_any)).c_str();
       // i_sink->second.second->checkObserved->Checked = true;
       // units are automatically chosen in this case. but ...
       // TODO: when we start using a control that lets users play with units, Waist/Hip needs to reset its units appropriately here.
      }
     } // else PSay("13 not found", m_debug, MB_OK, "");
    } // else PSay("12 not found", m_debug, MB_OK, "");
   } // else PSay("55 not found", m_debug, MB_OK, "");

   // auto-calculate 'nice', presentable, slashed blood pressure if possible
   i_sink = inverted_values.find("56"); // TODO -c MAGIC: written blood pressure
   if (i_sink != inverted_values.end())
   {
    // default back to blank
    i_sink->second.second->editMetric->Text = "";
    // i_sink->second.second->checkObserved->Checked = false;

    i_source = inverted_values.find("12"); // TODO -c MAGIC: systolic
    if (i_source != inverted_values.end())
    {
     bool b_sys = (i_source->second.first["response_metric"] != "");
     float f_sys = StringToFloat(i_source->second.first["response_metric"]);
     string u_sys = i_source->second.first["unit_id"];
     i_source = inverted_values.find("13"); // TODO -c MAGIC: diastolic
     if (i_source != inverted_values.end())
     {
      bool b_dia = (i_source->second.first["response_metric"] != "");
      float f_dia = StringToFloat(i_source->second.first["response_metric"]);
      string u_dia = i_source->second.first["unit_id"];

      // PSay("FloatConvertMetric(dbDatabase, " + FloatToString(f_waist) + ", " + u_waist + ", " + u_waist + ")", m_debug, MB_OK, "");
      // PSay("FloatConvertMetric(dbDatabase, " + FloatToString(f_hip) + ", " + u_hip + ", " + u_waist + ")", m_debug, MB_OK, "");

      float c_sys_any = FloatConvertMetric(dbDatabase, f_sys, u_sys, u_sys);
      float c_dia_any = FloatConvertMetric(dbDatabase, f_dia, u_dia, u_dia); // just convert to same units. blah.
      if (b_sys && b_dia)
      {
       i_sink->second.second->editText->Text = (IntToString((int)c_sys_any) + "/" + IntToString((int)c_dia_any)).c_str();
       // i_sink->second.second->checkObserved->Checked = true;
       // units are automatically chosen in this case. but ...
       // TODO: when we start using a control that lets users play with units, Waist/Hip needs to reset its units appropriately here.
      }
     } // else PSay("13 not found", m_debug, MB_OK, "");
    } // else PSay("12 not found", m_debug, MB_OK, "");
   } // else PSay("56 not found", m_debug, MB_OK, "");

  }

  bool save_entries (TIBTransaction * dbTransaction, string obs_response_set_id)
  {
   auto_calc();
   map<string, int> fields_to_leave_raw;
   fields_to_leave_raw["response_text"] = sql_no_uppercasing;

   // PSay("obs_response_set_id = " + obs_response_set_id + ", _obs_response_set_id = " + _obs_response_set_id, m_debug, MB_OK, "obs_row");
   // if we already -had- a valid id, and they give us a different one, something's up.
   if ((_obs_response_set_id != "" && _obs_response_set_id != "new" && _obs_response_set_id != "0") && obs_response_set_id != _obs_response_set_id)
    {PSay("ID values do not match!", m_debug, MB_OK, "obs_save_entries"); return false;}
   else
   {
    vector<string> valid_entries;
    for (vector<obs_row_pointers>::iterator i_row = custom_rows.begin(); i_row != custom_rows.end(); i_row++)
    {
     // oddly, this contains fewer fields than what we sent. that's on purpose. makes this code easier.
     map<string, string> field_values = i_row->entry_any->Row;
      field_values["current_id"]         = "5";
      field_values["last_update_stamp"]  = RightNow();
      field_values["last_updated_by"]    = current_userid;
     if (field_values["has_response"] == "1")
     {
      field_values.erase("has_response"); // no longer in db, assumed to be '1'. it wasn't a great idea. but handy here.
      if (field_values["id"] == "")
      {
       // insert
       field_values["id"]                 = GetNextId(dbTransaction, "obs_responses");
       field_values["obs_response_set_id"]= obs_response_set_id;
       valid_entries.push_back(field_values["id"]);
       if(!ExecuteInsertStatement(dbTransaction, "obs_responses", field_values, fields_to_leave_raw))
        {PSay("Could not insert a row", m_debug, MB_OK, "obs_save_entries"); return false;}
      }
      else
      {
       // update
       // gave up on having has_response, stupid idea in the first place, eh?
       valid_entries.push_back(field_values["id"]);
       if (!ExecuteUpdateStatement(dbTransaction, "obs_responses", field_values, fields_to_leave_raw))
        {PSay("Could not update a row", m_debug, MB_OK, "obs_save_entries"); return false;}
      }
     }
    }
    // delete anything not inserted or updated
    // to those who think "hey, i can make this one thing", reminder: sql IN clauses don't accept empty lists.
    // TODO:  Hey, I can make this one thing; reminder:  SafeInClause() handles those cases.
    if (valid_entries.empty())
    {
     if (!ExecuteStatement(dbTransaction, "delete from obs_responses where obs_response_set_id = " + obs_response_set_id))
      {PSay("Could not delete all non-essential dependent rows", m_debug, MB_OK, "obs_save_entries"); return false;}
    }
    else
    {
     if (!ExecuteStatement(dbTransaction, "delete from obs_responses where obs_response_set_id = " + obs_response_set_id + " and id not in (" + VecToCommaString(valid_entries) + ")"))
      {PSay("Could not delete some non-essential dependent rows", m_debug, MB_OK, "obs_save_entries"); return false;}
    }
   }
   return true;
  }
};

//---------------------------------------------------------------------------
class TformRecordObservationResponseSet : public TformRecordGeneric
{
__published:	// IDE-managed Components
        TPanel *panelObservationSet;
        TGroupBox *groupResponseSet;
        TLabel *_labelPerson;
        TLabel *_labelTask;
        TLabel *_labelDate;
        TDateTimePicker *dateObserved;
        TStaticText *staticTask;
        TStaticText *staticPerson;
        TLabel *_labelEvent;
        TLabel *_labelSteps;
        TCheckBox *checkSteps;
        TStaticText *staticEvent;
        TLabel *Label1;
        TStaticText *staticParticipation;
        TLabel *Label2;
        THalfButton *buttonViewRanges;
        THalfButton *buttonEditQuestionSet;
        TStaticText *staticPrintFilled;
        TPanel *panelValues;
        TScrollBox *scrollResponses;
        TComboBox *comboQuestionSet;
        TStaticText *staticPrintSemiFilled;
        TStaticText *staticPrintEmpty;
        void __fastcall FormClose(TObject *Sender, TCloseAction &Action);
        void __fastcall comboQuestionSetChange(TObject *Sender);
        void __fastcall FormShow(TObject *Sender);
        void __fastcall buttonEditQuestionSetClick(TObject *Sender);
        void __fastcall buttonViewRangesClick(TObject *Sender);
        void __fastcall scrollResponsesMouseWheelDown(TObject *Sender,
          TShiftState Shift, TPoint &MousePos, bool &Handled);
        void __fastcall scrollResponsesMouseWheelUp(TObject *Sender,
          TShiftState Shift, TPoint &MousePos, bool &Handled);
        void __fastcall staticPrintEmptyClick(TObject *Sender);
        void __fastcall staticPrintSemiFilledClick(TObject *Sender);
        void __fastcall staticPrintFilledClick(TObject *Sender);
private:	// User declarations
        obs_row_container * middle_section;
        bool do_not_refresh;
        int request_responses;
protected:
        bool DeleteRecord();
public:		// User declarations
        bool ScreenStart();
        bool ScreenStop();
        bool IsValidOnScreen();
        bool PutDataOnScreen();
        bool GetDataFromScreen();
        bool ScreenLock();
        bool ScreenUnlock();
        bool BeforeSave();
        bool NotNewAnymore();
        bool IsNewAgain();
        bool SendAtlas();
        bool CheckForUselessRecord();
        bool AtlasRefreshListViews(int request_num, string matched_value);
        __fastcall TformRecordObservationResponseSet(TComponent* Owner, map<string,string> p_parameters);
        static void Do(map<string, string> & p_parameters);
};
//---------------------------------------------------------------------------
extern PACKAGE TformRecordObservationResponseSet *formRecordObservationResponseSet;
//---------------------------------------------------------------------------
#endif
