//---------------------------------------------------------------------------

#include <vcl.h>
#pragma hdrstop

#include "codeRecordOutgoingBill.h"
#include "codeRecordInsuranceCoverage.h"
#include "codeRecordTransaction.h"
#include "codeSearchTaxEntity.h"
#include "reportDentalBillingForm.h"
#include "reportHCFA1500.h"
#include "codeWizardModifyTransactions.h"
#include "codeSpecialTransactionSummary.h"
#include "codeSpecialTransactionDetails.h"
//---------------------------------------------------------------------------
#pragma package(smart_init)
#pragma link "codeRecordGeneric"
#pragma link "codeRubberContainer"
#pragma link "codeBaseDropDown"
#pragma link "codeCheckListDropDown"
#pragma resource "*.dfm"
TformRecordOutgoingBill *formRecordOutgoingBill;
//---------------------------------------------------------------------------
__fastcall TformRecordOutgoingBill::TformRecordOutgoingBill(TComponent* Owner, map<string,string> &p_parameters)
        : TformRecordGeneric(Owner, p_parameters, string("bills")), transactions_summary(this, listTransactions)
{
  missing_insurance = false;

  insurance_filter.SetListViewHandle(listInsurance);
  insurance_filter.SetSecurity("insurance_coverage");
  insurance_filter.AddColumnSpec("Dental", "insurance_benefits", as_text, true, 40, prettyBoolString);
  insurance_filter.AddColumnSpec("Medical", "insurance_benefits", as_text, true, 40, prettyBoolString);
  insurance_filter.AddColumnSpec("Rx", "insurance_benefits", as_text, false, 40, prettyBoolString);
  insurance_filter.AddColumnSpec("Vision", "insurance_benefits", as_text, false, 40, prettyBoolString);
  insurance_filter.AddColumnSpec("Company", "tax_entities", as_text, true, 120);
  insurance_filter.AddColumnSpec("Medical Type", "tax_entities", as_text, true, 60);
  insurance_filter.AddColumnSpec("Policy", "insurance_policies", as_text, true, 90, rawString);
  insurance_filter.AddColumnSpec("Group", "insurance_policies", as_text, true, 60, rawString);
  insurance_filter.AddColumnSpec("Effective", "insurance_policies", as_date, true, 60);
  insurance_filter.AddColumnSpec("Termination", "insurance_policies", as_date, false, 60);
  insurance_filter.AddColumnSpec("Coverage", "insurance_policies", as_text, false, 60);
  insurance_filter.AddColumnSpec("Source", "insurance_policies", as_text, false, 70);
  insurance_filter.AddColumnSpec("Name on Policy", "insurance_policies", as_text, false, 120);
  insurance_filter.AddColumnSpec("PCP", "tax_entities", as_text, true, 120);
  insurance_filter.AddColumnSpec("Person Code", "insurance_coverage", as_int, false, 70);
  insurance_filter.AddColumnSpec("Name on Card", "insurance_coverage", as_text, false, 120);
  insurance_filter.AddColumnSpec("Current Id", "current_id", as_int, false, 80);
  insurance_filter.AddColumnSpec("Internal Id", "internal_id", as_int, false, 80);

  insurance_filter.FetchPreferences("recordoutgoingbill.insurance_filter");
  insurance_filter.SetIconStatus(true);
  insurance_filter.SetSQLStatement("");
  insurance_filter.RefreshDataSet();

  staticTransactionDetails->Font->Color = link_color;
  staticBetaModifyTransactions->Font->Color = link_color;

  string teeth_sql =
    "select item, " + CleverCurrentId("valid", "1", "Current", "Expired") + ", id from tooth_types where id > 0";
  cbddTeeth->Filter->SetSecurity("");
  cbddTeeth->Filter->AddColumnSpec("Tooth", "", as_text, true, 100);
  cbddTeeth->Filter->AddColumnSpec("Current Id", "current_id", as_int, false, 40);
  cbddTeeth->Filter->AddColumnSpec("Internal Id", "internal_id", as_int, false, 60);
  cbddTeeth->Filter->SetSQLStatement(teeth_sql);
  cbddTeeth->Filter->FetchPreferences("outgoingbill.teeth");
  cbddTeeth->Filter->SetIconStatus(true);
  cbddTeeth->Filter->ForceRefreshDataSet();
  cbddTeeth->DisplayFieldName = "Tooth";
  cbddTeeth->DropDownHeight = 144;

  fields_to_leave_raw.insert("note");

  Initialize();
}
//---------------------------------------------------------------------------
bool TformRecordOutgoingBill::ScreenStart()
{
  txtOldDentalForm->Font->Color = link_color;
  txtADAForm->Font->Color = link_color;
  txtUbBilling->Font->Color = link_color;
  txtHcfaBilling->Font->Color = link_color;
  pageControl->ActivePageIndex = 0;
  lookupTables->RegisterComboBox("state_types", cboInjuryState);
  TformRecordGeneric::ScreenStart();
  return true;
}
//---------------------------------------------------------------------------
bool TformRecordOutgoingBill::ScreenStop()
{
  lookupTables->UnregisterComboBox("state_types", cboInjuryState);
  TformRecordGeneric::ScreenStop();
  return true;
}
//---------------------------------------------------------------------------
bool TformRecordOutgoingBill::IsValidOnScreen()
{
  // check to make sure that the values entered in the combo boxe
  // corrospond to values in that look up table.
  // check to make sure the dates entered are valid
  // give the user a message box and prompt to re-enter if incorrect
  if((! radioProvider->Checked) && (! radioFacility->Checked))
  {
    sheetGeneral->Show();
    PSay("Please select the primary tax entity (provider or facility)", m_missing_data, MB_OK);
    radioProvider->SetFocus();
    return false;
  }
  else if ((! IsAmount(editInvoiceAmount->Text.c_str()) && (editInvoiceAmount->Text != "")))
  {
    sheetGeneral->Show();
    PSay("Please enter a valid invoice amount", m_invalid_data, MB_OK);
    editInvoiceAmount->SetFocus();
    return false;
  }
  else if ((! IsAmount(editTotalCharge->Text.c_str()) && (editTotalCharge->Text != "")))
  {
    sheetGeneral->Show();
    PSay("Please enter a valid total charge", m_invalid_data, MB_OK);
    editTotalCharge->SetFocus();
    return false;
  }
  else if(insurance_filter.GetMultiID().empty())
  {
    sheetGeneral->Show();
    PSay("Please select which insurance coverage to bill under", m_missing_data, MB_OK);
    listInsurance->SetFocus();
    return false;
  }
  else if(missing_insurance)
  {
    sheetGeneral->Show();
    PSay("You cannot save this bill until the insurance problem is fixed.\nClick Undo to exit this form", m_missing_data, MB_OK);
    return false;
  }
  else if(!IsInt(editRadiographs->Text.c_str()))
  {
    sheetDental->Show();
    PSay("Radiograph enclosures must be a number, or left blank", m_invalid_data, MB_OK);
    editRadiographs->SetFocus();
    return false;
  }
  else if(!IsInt(editOralImages->Text.c_str()))
  {
    sheetDental->Show();
    PSay("Oral image enclosures must be a number, or left blank", m_invalid_data, MB_OK);
    editOralImages->SetFocus();
    return false;
  }
  else if(!IsInt(editModels->Text.c_str()))
  {
    sheetDental->Show();
    PSay("Model enclosures must be a number, or left blank", m_invalid_data, MB_OK);
    editModels->SetFocus();
    return false;
  }
  else if(IsInvalidDate(editOrthoDatePlaced))
  {
    sheetDental->Show();
    PSay("Orthodontics date placed must be a valid date, or left blank", m_invalid_data, MB_OK);
    editOrthoDatePlaced->SetFocus();
    return false;
  }
  else if(!IsInt(editOrthoMonthsRemaining->Text.c_str()))
  {
    sheetDental->Show();
    PSay("Orthodontics months remaining must be a number, or left blank", m_invalid_data, MB_OK);
    editOrthoMonthsRemaining->SetFocus();
    return false;
  }
  else if(IsInvalidDate(editInjuryDate))
  {
    sheetDental->Show();
    PSay("Auto injury date must be a valid date, or left blank", m_invalid_data, MB_OK);
    editInjuryDate->SetFocus();
    return false;
  }
  else if(!lookupTables->IsInLookUpTable("state_types", cboInjuryState))
  {
    sheetDental->Show();
    PSay("Enter a valid auto injury state, or leave it blank", m_invalid_data, MB_OK);
    cboInjuryState->SetFocus();
    return false;
  }
  else
    return true;
}
//---------------------------------------------------------------------------
bool TformRecordOutgoingBill::PutDataOnScreen()
{
  // put the values on the form from the map initial_state
  // values may have come from GetDataFromDatabase if it's an existing record,
  // or values may have been passed to the form in this map if it's a new record
  string sql;
  string person_id = GetOneField(dbDatabase, "select person_id from tasks where tasks.id = " + initial_state["task_id"]);
  if((person_id != "") && (person_id != "0"))
  {
   /* TODO -c meh: Modified after screen fallen into disuse, not tested.
      Moved insurance_id from tax_entities to insurance_policies, and since there's no grouping,
      it might be getting more rows per entity than it expects
  */
  string claim_date = (IsBlankOrZero(initial_state["bill_date"]) ? CurrentDate() : initial_state["bill_date"]);
  sql = "select "
        "case when den.id is not null then '1' else '0' end is_den, "
        "case when med.id is not null then '1' else '0' end is_med, "
        "case when rx.id  is not null then '1' else '0' end is_rx, "
        "case when vis.id is not null then '1' else '0' end is_vis, "
        "ins_co.name, insurance_types.item, insurance_policies.policy_number, insurance_policies.group_number, "
        "insurance_policies.start_date, insurance_policies.stop_date, coverage_types.item, "
        "source_types.item, insurance_policies.name_on_policy, pcp.name, insurance_coverage.person_code, "
        "insurance_coverage.name_on_card, insurance_coverage.current_id, insurance_coverage.id "
        "from insurance_coverage "
        "left join insurance_policies on insurance_policies.id = insurance_coverage.insurance_policy_id "
        "left join coverage_types on coverage_types.id = insurance_policies.coverage_id "
        "left join source_types on source_types.id = insurance_policies.source_id "
        "left join insurance_benefits med on med.id = "
        "  (select first 1 ib.id from insurance_benefits ib where ib.insurance_policy_id = insurance_policies.id and "
        "   ib.current_id+1>3 and ib.area_id = '" + lookupTables->IdOf("area_types", "Medical") + "') "
        "left join insurance_benefits den on den.id = "
        "  (select first 1 ib.id from insurance_benefits ib where ib.insurance_policy_id = insurance_policies.id and "
        "   ib.current_id+1>3 and ib.area_id = '" + lookupTables->IdOf("area_types", "Dental") + "') "
        "left join insurance_benefits rx  on rx.id  = "
        "  (select first 1 ib.id from insurance_benefits ib where ib.insurance_policy_id = insurance_policies.id and "
        "   ib.current_id+1>3 and ib.area_id = '" + lookupTables->IdOf("area_types", "Prescription") + "') "
        "left join insurance_benefits vis on vis.id = "
        "  (select first 1 ib.id from insurance_benefits ib where ib.insurance_policy_id = insurance_policies.id and "
        "   ib.current_id+1>3 and ib.area_id = '" + lookupTables->IdOf("area_types", "Vision") + "') "
        "left join tax_entities ins_co on ins_co.id = insurance_policies.tax_entity_id "
        "left join insurance_types on insurance_types.id = insurance_policies.insurance_id "
        "left join tax_entities pcp on pcp.id = insurance_coverage.tax_entity_id "
        "where insurance_coverage.person_id = " + person_id +
        " and (insurance_coverage.start_date is null or insurance_coverage.start_date <= '" + claim_date + "') "
        " and (insurance_coverage.stop_date is null or insurance_coverage.stop_date > '" + claim_date + "')";

    insurance_filter.SetSQLStatement (sql);
    insurance_filter.RefreshDataSet();
    insurance_filter.SetAddEdit(this, NULL, EditRecord(&(TformRecordOutgoingBill::EditInsurance)));
    insurance_request = atlas->RequestNotificationWhen("insurance_coverage.person_id", person_id, rel_e, as_int, this,
                        RefreshHandler(&(TformRecordOutgoingBill::AtlasRefreshListView)));
    if(! is_new_record)
      insurance_filter.SetInform(this, InformForm(&(TformRecordOutgoingBill::SelectInsurance)));
  }

  if(initial_state["bill_date"] != "")
    dateBilled->Date = Pdate(initial_state["bill_date"], mdy | yyyy).OutDate();
  else
    dateBilled->Date = Date();
  editInvoiceAmount->Text = AmountToString(StringToAmount(initial_state["invoice_amount"])).c_str();
  editTotalCharge->Text = AmountToString(StringToAmount(initial_state["total_charge"])).c_str();

  CreateLink(txtProvider, btnProviderSearch, this, initial_state["provider_tax_id"], "tax_entities");
  CreateLink(txtFacility, btnFacilitySearch, this, initial_state["facility_tax_id"], "tax_entities");
  if(initial_state["provider_tax_id"] == initial_state["actual_tax_id"])
    radioProvider->Checked = true;
  else if(initial_state["facility_tax_id"] == initial_state["actual_tax_id"])
    radioFacility->Checked = true;
  CreateLink(txtVisit, NULL, this, initial_state["task_id"], "tasks");
  editRemarks->Text = initial_state["note"].c_str();
  editRadiographs->Text = initial_state["ada_radiographs"].c_str();
  editOralImages->Text = initial_state["ada_oral_images"].c_str();
  editModels->Text = initial_state["ada_models"].c_str();
  checkOrtho->Checked = initial_state["ada_ortho_tx"] == "1";
  editOrthoDatePlaced->Text = initial_state["ada_ortho_placed"].c_str();
  editOrthoMonthsRemaining->Text = initial_state["ada_ortho_remaining"].c_str();
  checkInjuryJob->Checked = initial_state["ada_job_injury"] == "1";
  checkInjuryAuto->Checked = initial_state["ada_auto_injury"] == "1";
  checkInjuryOther->Checked = initial_state["ada_other_injury"] == "1";
  editInjuryDate->Text = initial_state["ada_injury_date"].c_str();
  cboInjuryState->Text = lookupTables->ValueOf("state_types", initial_state["ada_injury_state"]).c_str();

  TformRecordGeneric::PutDataOnScreen();
  return true;
}
//---------------------------------------------------------------------------
bool TformRecordOutgoingBill::GetDataFromScreen()
{
  // place the values currently on the screen into
  // the map that will be used to update or insert the record
  // the parent form will call the insert or update function, and get the id
  // number if necessary and take care of current_id

  ending_state["user_id"] = "0";
  ending_state["bill_date"] =  Pdate(dateBilled->Date).OutDate(mdy | yyyy);
  ending_state["invoice_amount"] = editInvoiceAmount->Text.c_str();
  ending_state["total_charge"] = editTotalCharge->Text.c_str();
  if(ending_state["total_charge"] == "")
    ending_state["total_charge"] = GetOneField(dbDatabase, "select sum(charges) from procedures where task_id = " + initial_state["task_id"]);
  if(ending_state["total_charge"] == "")
    ending_state["total_charge"] = "0";
  if (initial_state["purchase_req_id"] == "")
    ending_state["purchase_req_id"] = "0";
  else
    ending_state["purchase_req_id"] = initial_state["purchase_req_id"];
  if (initial_state["task_id"] == "")
    ending_state["task_id"] = "0";
  else
    ending_state["task_id"] = initial_state["task_id"];

  string provider_tax_id = GetValidLinkId(txtProvider);
  if (provider_tax_id == "")
    ending_state["provider_tax_id"] = "0";
  else
    ending_state["provider_tax_id"] = provider_tax_id;

  if (initial_state["group_tax_id"] == "")
    ending_state["group_tax_id"] = "0";
  else
    ending_state["group_tax_id"] = initial_state["group_tax_id"];

  string facility_tax_id = GetValidLinkId(txtFacility);
  if (facility_tax_id == "")
    ending_state["facility_tax_id"] = "0";
  else
    ending_state["facility_tax_id"] = facility_tax_id;

  if (radioProvider->Checked)
    ending_state["actual_tax_id"] = ending_state["provider_tax_id"]; // jhaley 20101028: bug fix - ending_state, not initial_state
  else if(radioFacility->Checked)
    ending_state["actual_tax_id"] = ending_state["facility_tax_id"];
  else
    ending_state["actual_tax_id"] = "0";
  ending_state["insurance_coverage_id"] = insurance_filter.GetSelectedRecordID();
  if(initial_state["bill_id"] == lookupTables->ValueOf("bill_types", "Outgoing provider bill"))
  {
    if(radioProvider->Checked)
      ending_state["actual_tax_id"] = ending_state["provider_tax_id"];
    else if(radioFacility->Checked)
      ending_state["actual_tax_id"] = ending_state["facility_tax_id"];
  }
  ending_state["bill_id"] = initial_state["bill_id"];

  ending_state["id"] = initial_state["id"];

  ending_state["note"] = editRemarks->Text.c_str();
  ending_state["ada_radiographs"] = editRadiographs->Text.c_str();
  ending_state["ada_oral_images"] = editOralImages->Text.c_str();
  ending_state["ada_models"] = editModels->Text.c_str();
  ending_state["ada_ortho_tx"] = checkOrtho->Checked ? "1" : "0";
  ending_state["ada_ortho_placed"] = editOrthoDatePlaced->Text.c_str();
  ending_state["ada_ortho_remaining"] = editOrthoMonthsRemaining->Text.c_str();
  ending_state["ada_job_injury"] = checkInjuryJob->Checked ? "1" : "0";
  ending_state["ada_auto_injury"] = checkInjuryAuto->Checked ? "1" : "0";
  ending_state["ada_other_injury"] = checkInjuryOther->Checked ? "1" : "0";
  ending_state["ada_injury_date"] = editInjuryDate->Text.c_str();
  ending_state["ada_injury_state"] = lookupTables->IdOf("state_types", cboInjuryState).c_str();

  //string

  TformRecordGeneric::GetDataFromScreen();

  return true; // again, not much can go wrong here
}
//---------------------------------------------------------------------------
bool TformRecordOutgoingBill::ScreenLock()
{
  // disable panel containing edit boxes
  panelGeneral->Enabled = false;
  panelDental->Enabled = false;
  listInsurance->ReadOnly = true;  // why would this deselect the selected row?  Whatever, we fix it.
  SelectInsurance();
  TformRecordGeneric::ScreenLock();
  return true;
}
//---------------------------------------------------------------------------
bool TformRecordOutgoingBill::ScreenUnlock()
{
  // enable panel containing edit boxes
  panelGeneral->Enabled = true;
  panelDental->Enabled = true;

  listInsurance->ReadOnly = false; // why would this deselect the selected row?  Whatever, we fix it.
  SelectInsurance();
  TformRecordGeneric::ScreenUnlock();
  return true;
}
//---------------------------------------------------------------------------
bool TformRecordOutgoingBill::NotNewAnymore()
{
  txtOldDentalForm->Enabled = true;
  txtADAForm->Enabled = true;
  txtUbBilling->Enabled = true;
  txtHcfaBilling->Enabled = true;

  // jhaley 20100910: Use transaction summary object
  transactions_summary.SetSQLStatement("select * from transactions where bill_id = " + initial_state["id"]);
  transactions_summary.SetRecord("bills", initial_state["id"], "bill_id", RefreshHandler(&(TformRecordOutgoingBill::AtlasRefreshListView)));

  staticTransactionDetails->Enabled = true;
  TformRecordGeneric::NotNewAnymore();
  return true;
}
//---------------------------------------------------------------------------
void TformRecordOutgoingBill::AtlasRefreshListView(int request_num, string matched_value)
{
  if(request_num == insurance_request)
  {
    insurance_filter.RefreshDataSet();
    SelectInsurance();
  }
  else
    transactions_summary.AtlasRefresh(request_num, matched_value); // jhaley: check transaction summary

  // ????
  //return true;
}
//---------------------------------------------------------------------------
void TformRecordOutgoingBill::EditTransaction()
{
  /*
  map<string, string> temp_map;
  temp_map["id"] = transactions_filter.GetSelectedRecordID();
  FORM_TRY
    TformRecordTransaction * view_transaction = new TformRecordTransaction(NULL, temp_map);
    view_transaction->Show();
  FORM_CATCH
  */
}
//---------------------------------------------------------------------------
bool TformRecordOutgoingBill::IsNewAgain()
{
  transactions_summary.Clear(); // jhaley: use transaction summary object
  TformRecordGeneric::IsNewAgain();
  return true;
}
//---------------------------------------------------------------------------
void __fastcall TformRecordOutgoingBill::btnProviderSearchClick(TObject *Sender)
{
  // create and show new person search form
  vector<string> return_vector;
  FORM_TRY
    TformSearchTaxEntity * providerSearch = new TformSearchTaxEntity(NULL,&return_vector);
    providerSearch->ShowTaskModal(this);
  FORM_CATCH
  if (! return_vector.empty())
    CreateLink(txtProvider, btnProviderSearch, this, return_vector[0], "tax_entities");
}
//---------------------------------------------------------------------------
void __fastcall TformRecordOutgoingBill::btnFacilitySearchClick(TObject *Sender)
{
  // create and show new person search form
  vector<string> return_vector;
  FORM_TRY
    TformSearchTaxEntity * providerSearch = new TformSearchTaxEntity(NULL,&return_vector);
    providerSearch->ShowTaskModal(this);
  FORM_CATCH
  if (! return_vector.empty())
    CreateLink(txtFacility, btnFacilitySearch, this, return_vector[0], "tax_entities");
}
//---------------------------------------------------------------------------
void TformRecordOutgoingBill::SelectInsurance()
{
  missing_insurance = false;
  string id = initial_state["insurance_coverage_id"];
  set<string> ids;
  ids.insert(id);
  Application->ProcessMessages();
  insurance_filter.SetCheckedWhereID (ids, true);
  //if(missing_insurance)
  /*
  if (listInsurance->SelCount < 1)
  {
    missing_insurance = true;
    PSay("The insurance coverage this bill was billed to is gone!\n"
         "The person who the bill was for may have changed. You might want "
         "to give your systems administrator a call", m_missing_data, MB_OK);
  }
  */
}
//---------------------------------------------------------------------------
void TformRecordOutgoingBill::EditInsurance()
{
  map<string, string> temp_map;
  temp_map["id"] = insurance_filter.GetSelectedRecordID();

  TformRecordInsuranceCoverage *view_insurance = NULL;
  FORM_TRY
    view_insurance = new TformRecordInsuranceCoverage(NULL, temp_map);
  FORM_CATCH_RETURN("TformRecordInsuranceCoverage", )
  
  view_insurance->Show();
}
//---------------------------------------------------------------------------
void __fastcall TformRecordOutgoingBill::txtOldDentalFormClick(TObject *Sender)
{
  if (is_editing)
  {
    PSay ("You're currently editing this bill record.  "
          "You should save or undo your changes before running any reports that rely on that data.", m_warning, MB_OK);
    return;
  }

  TformReportDentalBillingForm *report_form = NULL;
  FORM_TRY
    report_form = new TformReportDentalBillingForm(NULL);
  FORM_CATCH_RETURN("TformReportDentalBillingForm", )

  TIBTransaction * dbTransaction = new TIBTransaction(NULL);
  try
  {
    Processing(true);
    string sql;
    string patient_id;
    string primary_id;
    map<string, string> person_map;

    dbTransaction->DefaultDatabase = dbDatabase;
    dbTransaction->DefaultAction = TARollback;
    dbTransaction->StartTransaction();

    sql = "select people.id, person_names.first_name, person_names.middle_name, person_names.last_name, "
          "people.gender_id, people.birth_date, tax_entities.name ins_company, tax_addresses.line1 ins_address, "
          "tax_addresses.city || ', ' || state_types.abbreviation || '  ' || tax_addresses.zip ins_city, "
          "insurance_policies.policy_number, insurance_policies.group_number, tasks.date_of_task "
          "from tasks "
          "inner join bills on bills.task_id = tasks.id "
          "inner join insurance_coverage on bills.insurance_coverage_id = insurance_coverage.id "
          "inner join insurance_policies on insurance_coverage.insurance_policy_id = insurance_policies.id "
          "inner join tax_entities on insurance_policies.tax_entity_id = tax_entities.id "
          "inner join tax_addresses on tax_entities.billing_address_id = tax_addresses.id "
          "inner join state_types on tax_addresses.state_id = state_types.id "
          "inner join people on tasks.person_id = people.id "
          "left join person_names on people.person_names_id = person_names.id "
          "where bills.id = '" + initial_state["id"] + "' ";

    SqlToMap(dbTransaction, sql, person_map);
    patient_id = person_map["id"];
    report_form->lblVisitDate->Caption = person_map["date_of_task"].c_str();
    report_form->lblFirstName->Caption = PrettyString(person_map["first_name"]).c_str();
    if(person_map["middle_name"] == "")
      report_form->lblMiddleName->Caption = "";
    else
      report_form->lblMiddleName->Caption = person_map["middle_name"].substr(0, 1).c_str();
    report_form->lblLastName->Caption = PrettyString(person_map["last_name"]).c_str();
    report_form->lblInsNameTitle->Caption = person_map["ins_company"].c_str();
    report_form->lblInsAddressTitle->Caption = FormatAddress(person_map["ins_address"]).c_str();
    report_form->lblInsCityTitle->Caption = PrettyString(person_map["ins_city"]).c_str();
    if(person_map["gender_id"] != "")
    {
      if(person_map["gender_id"] == lookupTables->IdOf("gender_types", "Male"))
        report_form->lblFemale->Caption = "";
      else
        report_form->lblMale->Caption = "";
    }
    else
    {
      report_form->lblMale->Caption = "";
      report_form->lblFemale->Caption = "";
    }
    if(person_map["birth_date"] != "")
    {
      Pdate birthday(person_map["birth_date"], mdy | yyyy);
      report_form->lblBirthMonth->Caption = twodigit(birthday.month).c_str();
      report_form->lblBirthDate->Caption = twodigit(birthday.day).c_str();
      report_form->lblBirthYear->Caption = IntToString(birthday.year).c_str();
    }
    else
    {
      report_form->lblBirthMonth->Caption = "";
      report_form->lblBirthDate->Caption = "";
      report_form->lblBirthYear->Caption = "";
    }
    report_form->lblGroupNumber->Caption = person_map["group_number"].c_str();
    report_form->lblPolicyNumber->Caption = person_map["policy_number"].c_str();

    map<string, string> insurance_map;
    sql = "select first 1 people.id, person_names.first_name, person_names.middle_name, person_names.last_name, "
          "person_contact_info.line1, person_contact_info.city || ', ' || state_types.abbreviation || '  '|| "
          "person_contact_info.zip city, people.birth_date "
          "from bills "
          "inner join insurance_coverage on_bill on bills.insurance_coverage_id = on_bill.id "
          "inner join insurance_coverage on_policy on on_bill.insurance_policy_id = on_policy.insurance_policy_id "
          "inner join people on on_policy.person_id = people.id "
          "left join person_names on people.person_names_id = person_names.id "
          "left join person_contact_info on people.person_contact_info_id = person_contact_info.id "
          "left join state_types on person_contact_info.state_id = state_types.id "
          "where bills.id = '" + initial_state["id"] + "' "
          "order by on_policy.person_code ascending";

    SqlToMap(dbTransaction, sql, insurance_map);
    primary_id = insurance_map["id"];
    string name = insurance_map["first_name"] + " ";
    if(insurance_map["middle_name"] != "")
      name += insurance_map["middle_name"].substr(0, 1) + " ";
    name += insurance_map["last_name"].c_str();
    report_form->lblPrimaryName->Caption = PrettyString(name).c_str();
    report_form->lblPrimaryAddress->Caption = FormatAddress(insurance_map["line1"]).c_str();
    report_form->lblPrimaryCity->Caption = PrettyString(insurance_map["city"]).c_str();
    if(insurance_map["birth_date"] != "")
    {
      Pdate birthday(insurance_map["birth_date"], mdy | yyyy);
      report_form->lblPrimaryBirthMonth->Caption = twodigit(birthday.month).c_str();
      report_form->lblPrimaryBirthDate->Caption = twodigit(birthday.day).c_str();
      report_form->lblPrimaryBirthYear->Caption = IntToString(birthday.year).c_str();
    }
    else
    {
      report_form->lblPrimaryBirthMonth->Caption = "";
      report_form->lblPrimaryBirthDate->Caption = "";
      report_form->lblPrimaryBirthYear->Caption = "";
    }

    report_form->lblRelationship->Caption = "";
    if(patient_id == primary_id)
    {
      report_form->lblCheckSpouse->Caption = "";
      report_form->lblCheckChild->Caption = "";
      report_form->lblCheckOther->Caption = "";
    }
    else
    {
      report_form->lblCheckSelf->Caption = "";
      sql = "select first 1 rel_a_to_b_id "
            "from relationships "
            "where person_a_id = '" + patient_id + "' and person_b_id = '" + primary_id + "' and current_id >= 3";
      string relationship_id = GetOneField(dbTransaction, sql);
      if((relationship_id == lookupTables->IdOf("relationship_types", "Daughter")) ||
        (relationship_id == lookupTables->IdOf("relationship_types", "Son")))
      {
        report_form->lblCheckSelf->Caption = "";
        report_form->lblCheckSpouse->Caption = "";
        report_form->lblCheckOther->Caption = "";
      }
      else if((relationship_id == lookupTables->IdOf("relationship_types", "Husband")) ||
        (relationship_id == lookupTables->IdOf("relationship_types", "Wife")))
      {
        report_form->lblCheckSelf->Caption = "";
        report_form->lblCheckChild->Caption = "";
        report_form->lblCheckOther->Caption = "";
      }
      else
      {
        report_form->lblCheckSelf->Caption = "";
        report_form->lblCheckSpouse->Caption = "";
        report_form->lblCheckChild->Caption = "";
        report_form->lblRelationship->Caption = lookupTables->ValueOf("relationship_types", relationship_id).c_str();
      }
    }

    map<string, string> employer_map;
    sql = "select person_names.first_name || ' ' || person_names.last_name name, person_contact_info.line1, "
          "person_contact_info.city || ', ' || state_types.abbreviation || '  ' || person_contact_info.zip city "
          "from bills "
          "inner join insurance_coverage on bills.insurance_coverage_id = insurance_coverage.id "
          "inner join insurance_policies on insurance_coverage.insurance_policy_id = insurance_policies.id "
          "inner join people on insurance_policies.person_id = people.id "
          "left join person_names on people.person_names_id = person_names.id "
          "left join person_contact_info on people.person_contact_info_id = person_contact_info.id "
          "left join state_types on person_contact_info.state_id = state_types.id "
          "where bills.id = '" + initial_state["id"] + "' ";

    SqlToMap(dbTransaction, sql, employer_map);
    report_form->lblCompanyName->Caption = StripLeading(PrettyString(employer_map["name"]), " ").c_str();
    report_form->lblCompanyAddress->Caption = FormatAddress(employer_map["line1"]).c_str();
    report_form->lblCompanyCity->Caption = PrettyString(employer_map["city"]).c_str();

    string signature_date;
    sql = "select max(consents.signature_date) "
          "from consents "
          "inner join bills on bills.id = '" + initial_state["id"] + "' "
          "inner join tasks on bills.task_id = tasks.id "
          "inner join people on tasks.person_id = people.id "
          "where consents.person_id = people.id and consents.consent_id = 2 and " // TODO -c MAGIC : consent_types?
          "consents.group_id = '" + lookupTables->IdOf("groups", "Dental Clinic") + "' and consents.current_id >= 3";

    signature_date = GetOneField(dbTransaction, sql);
    if(signature_date != "")
    {
      report_form->lblSignature1->Caption = "Signature on file";
      report_form->lblSignature1Date->Caption = signature_date.c_str();
      report_form->lblSignature2->Caption = "Signature on file";
      report_form->lblSignature2Date->Caption = signature_date.c_str();
    }
    else
    {
      report_form->lblSignature1->Caption = "";
      report_form->lblSignature1Date->Caption = "";
      report_form->lblSignature2->Caption = "";
      report_form->lblSignature2Date->Caption = "";
    }

    map<string, string> tax_entity_map;
    sql = "select facility.name facility_name, facility_addr.line1, "
          "(facility_addr.city || ', ' || state_types.abbreviation || '  ' || facility_addr.zip) city, "
          "facility_tin.identifier, facility_addr.phone_number, provider_license.identifier, bills.total_charge "
          "from bills "
          "inner join tax_entities facility on bills.facility_tax_id = facility.id "
          "left  join tax_addresses facility_addr on facility.billing_address_id = facility_addr.id " // jhaley 20101208: forgotten join?
          "inner join state_types on facility_addr.state_id = state_types.id "
          "left  join entity_ids facility_tin on facility_tin.tax_entity_id = facility.id and facility_tin.current_id >= 3 "
          " and facility_tin.entity_id_id = '" + lookupTables->IdOf("entity_id_types", "TIN/EIN") + "' "
          "inner join tax_entities provider on bills.provider_tax_id = provider.id "
          "left  join entity_ids provider_license on provider_license.tax_entity_id = provider.id and provider_license.current_id >= 3 "
          " and provider_license.entity_id_id = '" + lookupTables->IdOf("entity_id_types", "OK License") + "' "
          "where bills.id = " + initial_state["id"];

    SqlToMap(dbTransaction, sql, tax_entity_map);
    report_form->lblFacilityName->Caption = PrettyString(tax_entity_map["facility_name"]).c_str();
    report_form->lblFacilityAddress->Caption = FormatAddress(tax_entity_map["line1"]).c_str();
    report_form->lblFacilityCity->Caption = PrettyString(tax_entity_map["city"]).c_str();
    report_form->lblFacilityPhone->Caption = FormatPhoneNumber(tax_entity_map["phone_number"]).c_str();
    report_form->lblTaxId->Caption = tax_entity_map["taxid"].c_str();
    report_form->lblDentistId1->Caption = tax_entity_map["id_number"].c_str();
    report_form->lblDentistId2->Caption = tax_entity_map["id_number"].c_str();
    report_form->lblDentistSigDate->Caption = DateToStr(Date());
    if(tax_entity_map["total_charge"] == "")
      report_form->lblTotalCharge->Caption = "0.00";
    else
      report_form->lblTotalCharge->Caption = StringToAmount(tax_entity_map["total_charge"]).c_str();

    vector<map<string, string> > procedure_vec;
    sql = "select procedures.cpt_hcpcs, codes.description, procedures.start_date, procedures.charges, "
          "       procedures.tooth_position, procedures.tooth_surface "
          "from procedures "
          "inner join bills on bills.id = '" + initial_state["id"] + "' "
          "inner join codes on procedures.cpt_hcpcs = codes.code "
          "inner join tasks on bills.task_id = tasks.id and procedures.task_id = tasks.id "; // FIXME : why joining to tasks?

    SqlToVecMap(dbTransaction, sql, procedure_vec);
    int current_top = 563;
    for(vector<map<string, string> >::iterator itr = procedure_vec.begin(); itr != procedure_vec.end(); itr++)
    {
      TQRLabel * tooth_position_label;
      tooth_position_label = new TQRLabel(report_form->titleBand);
      tooth_position_label->Font->Size = 7;
      tooth_position_label->AutoSize = false;
      tooth_position_label->Top = current_top;
      tooth_position_label->Left = 157;
      tooth_position_label->Width = 185;
      tooth_position_label->Height = 13;
      tooth_position_label->Caption = ((*itr)["tooth_position"]).c_str();
      tooth_position_label->Transparent = true;
      tooth_position_label->Parent = report_form->titleBand;

      TQRLabel * tooth_surface_label;
      tooth_surface_label = new TQRLabel(report_form->titleBand);
      tooth_surface_label->Font->Size = 7;
      tooth_surface_label->AutoSize = false;
      tooth_surface_label->Top = current_top;
      tooth_surface_label->Left = 186;
      tooth_surface_label->Width = 185;
      tooth_surface_label->Height = 13;
      tooth_surface_label->Caption = ((*itr)["tooth_surface"]).c_str();
      tooth_surface_label->Transparent = true;
      tooth_surface_label->Parent = report_form->titleBand;

      TQRLabel * description_label;
      description_label = new TQRLabel(report_form->titleBand);
      description_label->Font->Size = 7;
      description_label->AutoSize = false;
      description_label->Top = current_top;
      description_label->Left = 219;
      description_label->Width = 185;
      description_label->Height = 13;
      description_label->Caption = PrettyString((*itr)["description"]).c_str();
      description_label->Transparent = true;
      description_label->Parent = report_form->titleBand;

      Pdate start_date((*itr)["start_date"], mdy | yyyy);

      TQRLabel * month_label;
      month_label = new TQRLabel(report_form->titleBand);
      month_label->Font->Size = 7;
      month_label->AutoSize = false;
      month_label->Top = current_top;
      month_label->Left = 419;
      month_label->Width = 18;
      month_label->Height = 13;
      month_label->Caption = twodigit(start_date.month).c_str();
      month_label->Transparent = true;
      month_label->Parent = report_form->titleBand;

      TQRLabel * day_label;
      day_label = new TQRLabel(report_form->titleBand);
      day_label->Font->Size = 7;
      day_label->AutoSize = false;
      day_label->Top = current_top;
      day_label->Left = 447;
      day_label->Width = 20;
      day_label->Height = 13;
      day_label->Caption = twodigit(start_date.day).c_str();
      day_label->Transparent = true;
      day_label->Parent = report_form->titleBand;

      TQRLabel * year_label;
      year_label = new TQRLabel(report_form->titleBand);
      year_label->Font->Size = 7;
      year_label->AutoSize = false;
      year_label->Top = current_top;
      year_label->Left = 472;
      year_label->Width = 28;
      year_label->Height = 13;
      year_label->Caption = IntToString(start_date.year).c_str();
      year_label->Transparent = true;
      year_label->Parent = report_form->titleBand;

      TQRLabel * code_label;
      code_label = new TQRLabel(report_form->titleBand);
      code_label->Font->Size = 7;
      code_label->AutoSize = false;
      code_label->Top = current_top;
      code_label->Left = 515;
      code_label->Width = 46;
      code_label->Height = 13;
      code_label->Caption = (*itr)["cpt_hcpcs"].c_str();
      code_label->Transparent = true;
      code_label->Parent = report_form->titleBand;

      TQRLabel * fee_label;
      fee_label = new TQRLabel(report_form->titleBand);
      fee_label->Font->Size = 7;
      fee_label->AutoSize = false;
      fee_label->Alignment = taRightJustify;
      fee_label->Top = current_top;
      fee_label->Left = 570;
      fee_label->Width = 50;
      fee_label->Height = 13;
      fee_label->Caption = StringToAmount((*itr)["charges"]).c_str();
      fee_label->Transparent = true;
      fee_label->Parent = report_form->titleBand;

      current_top += 14;
    }

    report_form->lblOtherBirthMonth->Caption = "";
    report_form->lblOtherBirthDay->Caption = "";
    report_form->lblOtherBirthYear->Caption = "";

    report_form->report->Prepare();
    report_form->report->Print();
    report_form->CloseNow();
    dbTransaction->Commit();
    Processing(false);
  }
  catch(...)
  {
    Processing(false);
  }
  delete dbTransaction;
}
//---------------------------------------------------------------------------
// jhaley 20101210: For debugging the beast of a report below.
void TformRecordOutgoingBill::ValidateKey(string key_type, string key, const string &sql)
{
  if (debug_level[other_forms] == 9)
  {
    if (IsBlankOrZero(key) || !IsInt(key))
    {
      string msg = key_type + " key '" + key + "' looks suspicious.\n"
                   "Generated by SQL:\n" + PrettySql(sql);
      PSay(msg, m_warning, MB_OK);
    }
  }
}
//---------------------------------------------------------------------------
void __fastcall TformRecordOutgoingBill::txtHcfaBillingClick(TObject *Sender)
{
  if (is_editing)
  {
    PSay ("You're currently editing this bill record.  You should save or undo your changes before running any reports that rely on that data.", m_warning, MB_OK);
    return;
  }
  else if (IsBlankOrZero(initial_state["task_id"]) || !IsInt(initial_state["task_id"])) // jhaley 20101115: validate fields before trying to run for safety
  {
    PSay("There is no valid task attached to this bill.", m_missing_data, MB_OK);
    return;
  }

  // jhaley 20101210: Paul said the person_id involved here should originate from the task.
  string person_id = GetOneField(dbDatabase, "select person_id from tasks where id = '" + initial_state["task_id"] + "' ");
  if (IsBlankOrZero(person_id) || !IsInt(person_id))
  {
    PSay("There is no valid person attached to this bill's task.", m_missing_data, MB_OK);
    return;
  }

  TformReportHCFA1500 *med_form = NULL;
  FORM_TRY
    med_form = new TformReportHCFA1500(NULL);
  FORM_CATCH_RETURN("TformReportHCFA1500", )

  TIBTransaction * dbTransaction;
  map<string,string> result_map;
  vector<map<string,string> > procedure_vec;
  vector<map<string,string> > result_vec;
  vector<map<string,string> > condition_vec;
  string sql;
  TIBDataSet * dbDataSet = new TIBDataSet(NULL);
  dbTransaction = new TIBTransaction(NULL);
  map<string, string> stupid_code;

  Processing(true);

  med_form->page1->Prepare();

  try
  {
    dbTransaction->DefaultDatabase = dbDatabase;
    dbDataSet->Database = dbDatabase;
    dbDataSet->Transaction = dbTransaction;
    dbDataSet->UniDirectional = false;
    dbTransaction->StartTransaction();
    string field_name;

    //conditions
    sql = "select conditions.code "
          "from conditions "
          "where conditions.task_id = '" + initial_state["task_id"] + "' ";
    SqlToVecMap(dbTransaction, sql, result_vec);

    med_form->lblDiag1->Caption = "";
    med_form->lblDiag2->Caption = "";
    med_form->lblDiag3->Caption = "";
    med_form->lblDiag4->Caption = "";

    int count = 1;
    for(vector<map<string, string> >::iterator itr = result_vec.begin(); itr != result_vec.end(); itr++)
    {
      if (count ==1)
        med_form->lblDiag1->Caption = ((*itr)["code"]).c_str();
      else if (count == 2)
        med_form->lblDiag2->Caption = ((*itr)["code"]).c_str();
      else if (count == 3)
        med_form->lblDiag3->Caption = ((*itr)["code"]).c_str();
      else if (count == 4)
        med_form->lblDiag4->Caption = ((*itr)["code"]).c_str();
      count++;
    }
    string codes ="";

    //procedures
    sql = "select P.id, P.start_date, P.stop_date, P.place_of_service, P.type_of_service, P.cpt_hcpcs, P.modifier, "
          "P.charges, P.epsdt, P.emg, P.cob, P.days_units, P.local_use "
          "from procedures P "
          "where P.task_id = '" + initial_state["task_id"] + "' ";

    dbDataSet->SelectSQL->Add(sql.c_str());
    dbDataSet->Open();
    dbDataSet->First();

    med_form->page1->DataSet = dbDataSet;
    med_form->txtStartDate->DataSet = dbDataSet;
    med_form->txtStartDate->DataField = "start_date";
    med_form->txtEndDate->DataSet = dbDataSet;
    med_form->txtEndDate->DataField = "stop_date";
    med_form->txtPlace->DataSet = dbDataSet;
    med_form->txtPlace->DataField = "place_of_service";
    med_form->txtDiagCode->DataSet = dbDataSet;
    med_form->txtDiagCode->DataField = "id";
    med_form->txtServiceType->DataSet = dbDataSet;
    med_form->txtServiceType->DataField = "type_of_service";
    med_form->txtCPT->DataSet = dbDataSet;
    med_form->txtCPT->DataField = "cpt_hcpcs";
    med_form->txtModifier->DataSet = dbDataSet;
    med_form->txtModifier->DataField = "modifier";
    med_form->txtCharges->DataSet = dbDataSet;
    med_form->txtCharges->DataField = "charges";
    med_form->txtDayUnit->DataSet = dbDataSet;
    med_form->txtDayUnit->DataField = "days_units";
    med_form->txtEPSDT->DataSet = dbDataSet;
    med_form->txtEPSDT->DataField = "epsdt";
    med_form->txtEMG->DataSet = dbDataSet;
    med_form->txtEMG->DataField = "emg";
    med_form->txtCOB->DataSet = dbDataSet;
    med_form->txtCOB->DataField = "cob";
    med_form->txtReserved->DataSet = dbDataSet;
    med_form->txtReserved->DataField = "local_use";

    //insurance
    sql = "select people.birth_date, pci.notes, pci.phone_number, insurance_policies.policy_number, "
          "ins_co.name, ins_co_addr.line1, ins_co_addr.city, state_types.abbreviation, ins_co_addr.zip, "
          "insurance_policies.group_number, insurance_policies.start_date, insurance_policies.stop_date, coverage_types.item, "
          "source_types.item, insurance_policies.name_on_policy, insurance_policies.person_id, pcp.name, insurance_coverage.person_code, "
          "insurance_coverage.name_on_card, insurance_coverage.current_id, insurance_coverage.id "
          "from insurance_policies "
          "inner join insurance_coverage on insurance_coverage.insurance_policy_id = insurance_policies.id and insurance_coverage.person_id = '" + person_id + "' "
          "inner join coverage_types on insurance_policies.coverage_id = coverage_types.id "
          "inner join source_types on insurance_policies.source_id = source_types.id "
          "inner join tax_entities pcp on insurance_coverage.tax_entity_id = pcp.id "
          "inner join tax_entities ins_co on insurance_policies.tax_entity_id = ins_co.id "
          "inner join tax_addresses ins_co_addr on ins_co.billing_address_id = ins_co_addr.id "
          "inner join state_types on ins_co_addr.state_id = state_types.id "
          "inner join people on people.id = '" + person_id + "' "
          "inner join person_names on person_names.id = people.person_names_id "
          "inner join person_contact_info pci on pci.id = people.person_contact_info_id "
          "inner join affiliations on affiliations.id = people.affiliations_id "
          "inner join tribe_types on affiliations.tribe_id = tribe_types.id ";
    SqlToMap(dbTransaction,sql,result_map);

    med_form->lblInsurance->Caption = PrettyString(result_map["name"]).c_str();

    med_form->lblInsuranceAddress->Caption = FormatAddress(result_map["line1"]).c_str();
    med_form->lblInsuranceCity->Caption = PrettyString(result_map["city"]).c_str();
    med_form->lblInsuranceState->Caption = PrettyString(result_map["abbreviation"]).c_str();
    med_form->lblInsuranceZip->Caption = PrettyString(result_map["zip"]).c_str();
    med_form->lblInsuredID->Caption = PrettyString(result_map["group_number"]).c_str();
    med_form->lblInsuredPolicy->Caption = (result_map["policy_number"]).c_str();
    med_form->lblInsuredPlanName->Caption = PrettyString(result_map["policyName"]).c_str();
    string insured_id = result_map["person_id"];

    //ValidateKey("insured_id", insured_id, sql);

    // TODO -c meh: Modified after screen fallen into disuse, not tested.  Moved insurance_id from tax_entities to insurance_policies, and since there's no grouping, it might be getting more rows per entity than it expects
    sql = "select people.birth_date, pci.line1 , pci.city, state_types.abbreviation , pci.zip , pci.phone_number, "
          "insurance_policies.policy_number, people.gender_id, "
          "ins_co.name, ins_co_addr.line1, ins_co_addr.city, ins_co_addr.state_id, ins_co_addr.zip, insurance_policies.insurance_id, "
          "insurance_policies.group_number, insurance_policies.start_date, insurance_policies.stop_date, coverage_types.item, "
          "source_types.item, insurance_policies.name_on_policy, insurance_policies.person_id, pcp.name, insurance_coverage.person_code, "
          "insurance_coverage.name_on_card, insurance_coverage.current_id, insurance_coverage.id "
          "from insurance_policies "
          "inner join insurance_coverage on insurance_coverage.insurance_policy_id = insurance_policies.id and insurance_coverage.person_id = '" + person_id + "' "
          "inner join coverage_types on insurance_policies.coverage_id = coverage_types.id "
          "inner join source_types on insurance_policies.source_id = source_types.id "
          "inner join tax_entities pcp on insurance_coverage.tax_entity_id = pcp.id "
          "inner join tax_entities ins_co on insurance_policies.tax_entity_id = ins_co.id "
          "inner join tax_addresses ins_co_addr on ins_co.billing_address_id = ins_co_addr.id "
          "inner join people on people.id = '" + person_id + "' "
          "inner join person_names on person_names.id = people.person_names_id "
          "inner join person_contact_info pci on pci.id = people.person_contact_info_id "
          "inner join state_types on pci.state_id = state_types.id "
          "inner join affiliations on affiliations.id = people.affiliations_id "
          "inner join tribe_types on affiliations.tribe_id = tribe_types.id ";
    SqlToMap(dbTransaction,sql,result_map);
    med_form->lblInsuredPlanName->Caption = PrettyString(result_map["name"]).c_str();

    if (result_map["insurance_id"] == lookupTables->IdOf("insurance_types", "HMO Medicaid") ||
        result_map["insurance_id"] == lookupTables->IdOf("insurance_types", "Medicaid"))
      med_form->lblMedicaid->Caption = "x";
    else if (result_map["insurance_id"] == lookupTables->IdOf("insurance_types", "Medicare A") ||
             result_map["insurance_id"] == lookupTables->IdOf("insurance_types", "Medicare B") ||
             result_map["insurance_id"] == lookupTables->IdOf("insurance_types", "Medicare Advantage"))
      med_form->lblMedicare->Caption = "x";
    else if (result_map["insurance_id"] == lookupTables->IdOf("insurance_types", "Veteran"))
      med_form->lblChampVa->Caption = "x";
    else
      med_form->lblGroupHealth->Caption = "x";

    // second insurance

    sql = "select count(tax_entities.id) "
          "from insurance_coverage "
          "inner join insurance_policies on insurance_coverage.insurance_policy_id = insurance_policies.id "
          "inner join tax_entities on insurance_policies.tax_entity_id = tax_entities.id "
          "inner join coverage_types on insurance_policies.coverage_id = coverage_types.id "
          "inner join source_types on insurance_policies.source_id = source_types.id "
          "where insurance_coverage.person_id = '" + person_id + "' ";
    int insurance_count = StringToInt(GetOneField(dbTransaction, sql));

    if (insurance_count > 1)
    {
      sql = "select tax_entities.id, tax_entities.name, insurance_policies.policy_number, insurance_policies.group_number, "
            "insurance_policies.start_date, insurance_policies.stop_date, coverage_types.item, source_types.item, "
            "insurance_policies.name_on_policy, insurance_coverage.person_code, insurance_coverage.current_id, "
            "insurance_coverage.id, insurance_coverage.insurance_policy_id "
            "from insurance_coverage "
            "inner join insurance_policies on insurance_coverage.insurance_policy_id = insurance_policies.id "
            "inner join tax_entities on insurance_policies.tax_entity_id = tax_entities.id "
            "inner join coverage_types on insurance_policies.coverage_id = coverage_types.id "
            "inner join source_types on insurance_policies.source_id = source_types.id "
            "where insurance_coverage.person_id = '" + person_id + "' and "
            "insurance_coverage.current_id > 3 and insurance_coverage.current_id < 5 ";
      SqlToMap(dbTransaction, sql, result_map);
      med_form->lblCheckYesOther->Caption = "x";
      med_form->lblOtherInsurance->Caption = result_map["name"].c_str();
      string policy_number = result_map["insurance_policy_id"].c_str();
      med_form->lblInsuredOtherPolicy->Caption = result_map["policy_number"].c_str();

      //second insurance person
      if (!IsBlankOrZero(policy_number)) // jhaley 20101210: There isn't necessarily a second insurance policy
      {
        sql = "select person_names.last_name || ', ' || person_names.first_name || ' ' || person_names.suffix name, "
              "people.id the_person_id, people.gender_id, people.birth_date, tax_entities.id, "
              "insurance_coverage.person_code, insurance_coverage.name_on_card, "
              "insurance_coverage.current_id, insurance_coverage.id, insurance_policies.person_id "
              "from insurance_coverage "
              "inner join tax_entities on insurance_coverage.tax_entity_id = tax_entities.id "
              "inner join insurance_policies on insurance_coverage.insurance_policy_id = insurance_policies.id "
              "inner join people on insurance_coverage.person_id = people.id "
              "left join person_names on people.person_names_id = person_names.id "
              "where insurance_coverage.insurance_policy_id = '" + policy_number + "' and "
              "insurance_coverage.current_id > 3 and insurance_coverage.current_id < 5 ";
        SqlToMap(dbTransaction, sql, result_map);
        string Otherpolicy_owner = result_map["the_person_id"];

        med_form->lblInsuredOtherName->Caption = PrettyString(result_map["name"]).c_str();

        if(result_map["birth_date"] != "")
        {
          Pdate InsuredOtherDate(result_map["birth_date"], mdy | yyyy);
          med_form->lblInsuredOtherMonth->Caption = IntToString(InsuredOtherDate.month).c_str();
          med_form->lblInsuredOtherDay->Caption = IntToString(InsuredOtherDate.day).c_str();
          med_form->lblInsuredOtherYear->Caption = IntToString(InsuredOtherDate.year).c_str();
        }
        else
        {
          med_form->lblInsuredOtherMonth->Caption = "";
          med_form->lblInsuredOtherDay->Caption = "";
          med_form->lblInsuredOtherYear->Caption = "";
        }

        if (result_map["gender_id"] == lookupTables->IdOf("gender_types", "Male"))
          med_form->lblInsuredOtherMale->Caption = "x";
        else
          med_form->lblInsuredOtherFemale->Caption = "x";

        // second employer
        if (!IsBlankOrZero(Otherpolicy_owner)) // jhaley 20101210: There isn't necessarily another policy owner.
        {
          sql = "select first 1 person_b_id "
                "from relationships where person_a_id = '" + Otherpolicy_owner + "' and "
                "rel_a_to_b_id = '" + lookupTables->IdOf("relationship_types", "Employee") +  "' and current_id >= 3 ";
          string employer_id = GetOneField(dbTransaction, sql);

          if (!IsBlankOrZero(employer_id)) // jhaley 20101210: Not everybody has a job, unfortunately.
          {
            sql = "select person_names.last_name || ', ' || person_names.first_name || ' ' || person_names.suffix name "
                  "from people "
                  "left join person_names on person_names.id = people.person_names_id "
                  "left join affiliations on affiliations.id = people.affiliations_id "
                  "left join tribe_types on affiliations.tribe_id = tribe_types.id "
                  "left join person_contact_info on person_contact_info.id = people.person_contact_info_id "
                  "left join state_types on person_contact_info.state_id = state_types.id "
                  "where people.current_id >= 3  and people.id = '" + employer_id + "' ";
            med_form->lblOtherEmployer->Caption = PrettyString(GetOneField(dbTransaction, sql)).c_str();
          }
          else
            med_form->lblOtherEmployer->Caption = "";
        }
        else
          med_form->lblOtherEmployer->Caption = "";
      }
      else // jhaley 20101210: In case of no second policy...
      {
        med_form->lblInsuredOtherName->Caption  = "";
        med_form->lblInsuredOtherMonth->Caption = "";
        med_form->lblInsuredOtherDay->Caption   = "";
        med_form->lblInsuredOtherYear->Caption  = "";
        med_form->lblOtherEmployer->Caption     = "";
      }
    }
    else
    {
      med_form->lblCheckNoOther->Caption = "x";
      med_form->lblInsuredOtherName->Caption = "";
      med_form->lblInsuredOtherPolicy->Caption = "";
      med_form->lblInsuredOtherMonth->Caption = "";
      med_form->lblInsuredOtherDay->Caption = "";
      med_form->lblInsuredOtherYear->Caption = "";
      med_form->lblInsuredOtherMale->Caption = "";
      med_form->lblInsuredOtherFemale->Caption = "";
      med_form->lblOtherEmployer->Caption = "";
      med_form->lblOtherInsurance->Caption = "";
    }

    //person with policy
    sql = "select first 1 people.id, person_names.last_name || ', ' || person_names.first_name || ' ' || person_names.suffix name, "
          "person_contact_info.line1, person_contact_info.city, state_types.abbreviation, "
          "person_contact_info.zip, people.birth_date, person_contact_info.phone_number "
          "from bills "
          "inner join insurance_coverage on_bill on bills.insurance_coverage_id = on_bill.id "
          "inner join insurance_coverage on_policy on on_bill.insurance_policy_id = on_policy.insurance_policy_id "
          "inner join people on on_policy.person_id = people.id "
          "left join person_names on people.person_names_id = person_names.id "
          "left join person_contact_info on people.person_contact_info_id = person_contact_info.id "
          "left join state_types on person_contact_info.state_id = state_types.id "
          "where bills.id = '" + initial_state["id"] + "' "
          "order by on_policy.person_code ascending ";
    SqlToMap(dbTransaction, sql, result_map);
    string policy_owner = result_map["id"];

    //ValidateKey("people.id", policy_owner, sql);
    //ValidateKey("gender_id", result_map["gender_id"], sql);

    med_form->lblInsuredAddress->Caption = FormatAddress(result_map["line1"]).c_str();
    med_form->lblInsuredCity->Caption = PrettyString(result_map["city"]).c_str();
    med_form->lblInsuredZip->Caption = PrettyString(result_map["zip"]).c_str();
    med_form->lblInsuredState->Caption = PrettyString(result_map["abbreviation"]).c_str();
    med_form->lblInsuredPhone->Caption = FormatPhoneNumber(result_map["phone_number"]).c_str();
    med_form->lblInsuredName->Caption = PrettyString(result_map["name"]).c_str();

    if(result_map["birth_date"] != "")
    {
      Pdate InsuredDate(result_map["birth_date"], mdy | yyyy);
      med_form->lblInsuredMonth->Caption = IntToString(InsuredDate.month).c_str();
      med_form->lblInsuredDay->Caption = IntToString(InsuredDate.day).c_str();
      med_form->lblInsuredYear->Caption = IntToString(InsuredDate.year).c_str();
    }
    else
    {
      med_form->lblInsuredMonth->Caption = "";
      med_form->lblInsuredDay->Caption = "";
      med_form->lblInsuredYear->Caption = "";
    }

    if (result_map["gender_id"] == lookupTables->IdOf("gender_types", "Male"))
      med_form->lblInsuredMale->Caption = "x";
    else
      med_form->lblInsuredFemale->Caption = "x";

    //employer
    sql = "select person_a_id "
          "from relationships where person_b_id = '" + policy_owner + "' and "
          "rel_a_to_b_id = '" + lookupTables->IdOf("relationship_types", "Employer") + "' and current_id >= 3 ";
    string employer2_id = GetOneField(dbTransaction, sql);

    // jhaley 20101210: Removed duplicate joins to person_names in most remaining queries that broke Firebird 2.x

    if (!IsBlankOrZero(employer2_id)) // jhaley 20101210: Once again, not everybody is employed.
    {
      sql = "select person_names.last_name || ', ' || person_names.first_name || ' ' || person_names.suffix name "
            "from people "
            "left join person_names on person_names.id = people.person_names_id "
            "left join affiliations on affiliations.id = people.affiliations_id "
            "left join tribe_types on affiliations.tribe_id = tribe_types.id "
            "left join person_contact_info on person_contact_info.id = people.person_contact_info_id "
            "left join state_types on person_contact_info.state_id = state_types.id "
            "where people.current_id >= 3  and people.id = '" + employer2_id + "' ";
      SqlToMap(dbTransaction,sql, result_map);
      med_form->lblInsuredEmployer->Caption = PrettyString(result_map["name"]).c_str();
    }
    else
      med_form->lblInsuredEmployer->Caption = "";

    //patient
    sql = "select people.id, person_contact_info.line1, person_contact_info.city, affiliations.cdib, state_types.abbreviation, "
          "person_contact_info.zip, person_contact_info.phone_number, people.birth_date, people.gender_id, "
          "people.marital_id, person_names.last_name || ', ' || person_names.first_name || ' ' || person_names.suffix name "
          "from people "
          "left join person_names on person_names.id = people.person_names_id "
          "left join affiliations on affiliations.id = people.affiliations_id "
          "left join tribe_types on affiliations.tribe_id = tribe_types.id "
          "left join person_contact_info on person_contact_info.id = people.person_contact_info_id "
          "left join state_types on person_contact_info.state_id = state_types.id "
          "where people.current_id >= 3  and people.id = '" + person_id + "' ";
    SqlToMap(dbTransaction,sql, result_map);
    string patient_id = result_map["id"];
    med_form->lblPatientName->Caption = PrettyString(result_map["name"]).c_str();
    med_form->lblPatientAddress->Caption = FormatAddress(result_map["line1"]).c_str();
    med_form->lblPatientCity->Caption = PrettyString(result_map["city"]).c_str();
    med_form->lblPatientState->Caption = PrettyString(result_map["abbreviation"]).c_str();
    med_form->lblPatientZip->Caption = PrettyString(result_map["zip"]).c_str();
    med_form->lblPatientPhone->Caption = FormatPhoneNumber(result_map["phone_number"]).c_str();
    med_form->lblPatientCDIB->Caption = (result_map["cdib"]).c_str();

    //ValidateKey("patient_id", patient_id, sql);
    //ValidateKey("marital_id", result_map["marital_id"], sql);
    //ValidateKey("gender_id",  result_map["gender_id"],  sql);

    if (policy_owner == patient_id)
    {
      med_form->lblCheckSelf->Caption = "x";
      med_form->lblCheckSpouse->Caption = "";
      med_form->lblCheckChild->Caption = "";
      med_form->lblCheckOther->Caption = "";
    }
    else
    {
      med_form->lblCheckSelf->Caption = "";
      sql = "select first 1 rel_a_to_b_id "
            "from relationships "
            "where person_a_id = '" + patient_id + "' and person_b_id = '" + policy_owner + "' and current_id >= 3";
      string relationship_id = GetOneField(dbTransaction, sql);

      //ValidateKey("relationship_id", relationship_id, sql);

      if((relationship_id == lookupTables->IdOf("relationship_types", "Daughter")) ||
         (relationship_id == lookupTables->IdOf("relationship_types", "Son")))
      {
        med_form->lblCheckChild->Caption ="x";
        med_form->lblCheckSelf->Caption = "";
        med_form->lblCheckSpouse->Caption = "";
        med_form->lblCheckOther->Caption = "";
      }
      else if ((relationship_id == lookupTables->IdOf("relationship_types", "Husband")) ||
               (relationship_id == lookupTables->IdOf("relationship_types", "Wife")))
      {
        med_form->lblCheckSpouse->Caption = "x";
        med_form->lblCheckSelf->Caption = "";
        med_form->lblCheckChild->Caption = "";
        med_form->lblCheckOther->Caption = "";
      }
      else
      {
        med_form->lblCheckSelf->Caption = "";
        med_form->lblCheckSpouse->Caption = "";
        med_form->lblCheckChild->Caption = "";
        med_form->lblCheckOther->Caption = "x";
      }
    }

    if (result_map["birth_date"] != "")
    {
      Pdate PatientDate(result_map["birth_date"], mdy | yyyy);
      med_form->lblBirthMonth->Caption = IntToString(PatientDate.month).c_str();
      med_form->lblBirthDay->Caption = IntToString(PatientDate.day).c_str();
      med_form->lblBirthYear->Caption = IntToString(PatientDate.year).c_str();
    }
    else
    {
      med_form->lblBirthMonth->Caption = "";
      med_form->lblBirthDay->Caption = "";
      med_form->lblBirthYear->Caption = "";
    }

    string phone = result_map["phone_number"];
    if (result_map["marital_id"] == lookupTables->IdOf("marital_types", "Single"))
      med_form->lblPatientSingle->Caption = "x";
    else if (result_map["marital_id"] == lookupTables->IdOf("marital_types", "Married"))
      med_form->lblPatientMarried->Caption = "x";
    else
      med_form->lblPatientOther->Caption = "x";
    if (result_map["gender_id"] == lookupTables->IdOf("marital_types", "Male"))
      med_form->lblPatientMale->Caption = "x";
    else
      med_form->lblPatientFemale->Caption = "x";

    /*
    if (phone.length() == 10)
     string phone = phone.substr(0,3) + "  " + phone.substr(3, 3) + "-" + phone.substr(6,4);
    else if (phone.length() == 7)
     string phone = "    " + phone.substr(0,3) + "-" + phone.substr(3,4);
    med_form->lblPatientPhone->Caption = phone.c_str();
    */

    //consent
    sql = "select max(consents.signature_date) "
          "from consents "
          "inner join bills on bills.id = '" + initial_state["id"] + "' "
          "inner join tasks on bills.task_id = tasks.id "
          "inner join people on tasks.person_id = people.id "
          "where "
          "consents.person_id = people.id and "
          "consents.consent_id = 2 and "
          "consents.group_id = '" + lookupTables->IdOf("groups", "Little Axe Clinic") + "' and "
          "consents.current_id >= 3 ";
    string sign_date = GetOneField(dbTransaction, sql);

    if (sign_date != "")
    {
     med_form->lblConsent1->Caption = "Signature on file";
     med_form->lblConsent2->Caption = "Signature on file";
     med_form->lblConsentDate->Caption = sign_date.c_str();
    }
    else
    {
     med_form->lblConsent1->Caption = "";
     med_form->lblConsent2->Caption = "";
     med_form->lblConsentDate->Caption = "";
    }

    //tax entity
    // jhaley 20100818: fixed a typo that was keeping this whole thing from working (licese vs license)
    // jhaley 20101210: fixed ANOTHER typo which was matching conditions.person_id against bills.id
    sql = "select tax_entities.name, license.identifier, conditions.index_number, bills.actual_tax_id "
          "from tax_entities "
          "inner join tasks on tax_entities.id = tasks.tax_entity_id "
          "inner join bills on bills.task_id = tasks.id "
          "inner join conditions on conditions.task_id = tasks.id "
          "left join entity_ids license on license.tax_entity_id = tax_entities.id and license.current_id >= 3 and "
          " license.entity_id_id = '" + lookupTables->IdOf("entity_id_types", "OK License") + "' "
          "where conditions.person_id = '" + person_id + "' ";
    SqlToMap(dbTransaction,sql, result_map);

    // TODO : jhaley 20101210: ????? The results of the above ^^^^ are wiped out without being used!

    //facility
    sql = "select tax_entities.name, tax_addresses.line1, tax_addresses.city, state_types.abbreviation, "
          "tax_addresses.zip, tax_addresses.phone_number, tin.identifier taxid " // jhaley 20101210: 'taxid' row name was missing...
          "from bills "
          "inner join tax_entities on bills.facility_tax_id = tax_entities.id "
          "inner join tax_addresses on tax_entities.billing_address_id = tax_addresses.id "
          "inner join state_types on tax_addresses.state_id = state_types.id "
          "left join entity_ids tin on tin.tax_entity_id = tax_entities.id and tin.current_id >= 3 and tin.entity_id_id = '" + lookupTables->IdOf("entity_id_types", "TIN/EIN") + "' "
          "where bills.task_id = '" + initial_state["task_id"] + "' "; // jhaley 20100902: fixed bad syntax: extra 'and'
    SqlToMap(dbTransaction, sql, result_map);

    med_form->lblClinic->Caption = result_map["name"].c_str();
    med_form->lblAddress->Caption = FormatAddress(result_map["line1"]).c_str();
    med_form->lblCity->Caption = PrettyString(result_map["city"]).c_str();
    med_form->lblState->Caption = PrettyString(result_map["abbreviation"]).c_str();
    med_form->lblZip->Caption = result_map["zip"].c_str();
    med_form->lblPhone->Caption = FormatPhoneNumber(result_map["phone_number"]).c_str();
    med_form->lblPin->Caption = result_map["taxid"].c_str();
    med_form->lblFedTax->Caption = result_map["taxid"].c_str();

    //provider
    sql = "select bills.actual_tax_id, bills.provider_tax_id, bills.bill_date, tax_entities.name, license.identifier "
          "from bills "
          "inner join tax_entities on bills.provider_tax_id = tax_entities.id "
          "left join entity_ids license on license.tax_entity_id = tax_entities.id and license.current_id >= 3 and "
          " license.entity_id_id = '" + lookupTables->IdOf("entity_id_types", "OK License") + "' "
          "where bills.task_id = '" + initial_state["task_id"] + "' ";
    SqlToMap(dbTransaction, sql, result_map);

    med_form->lblTaxEntity->Caption = result_map["name"].c_str();
    med_form->lblTaxEntityNum->Caption = result_map["id_number"].c_str();
    med_form->lblPhysician->Caption = PrettyString(result_map["name"]).c_str();
    med_form->lblDate->Caption = result_map["bill_date"].c_str();

    //charges
    sql = "select bills.total_charge, bills.invoice_amount "
          "from bills "
          "where bills.task_id = '" + initial_state["task_id"] + "' ";
    SqlToMap(dbTransaction, sql, result_map);

    med_form->lblTotalCharge->Caption = (StringToAmount(result_map["total_charge"])).c_str();
    med_form->lblDue->Caption = (StringToAmount(result_map["invoice_amount"])).c_str();
    Processing(false);
    med_form->page1->Preview();

    dbDataSet->Close();
    dbTransaction->Commit();
  }
  catch(...)
  {
    Processing(false);
    PSay("Error running report", m_database_error, MB_OK);
    if (dbTransaction->Active)
     dbTransaction->Rollback();
  }
  
  delete dbTransaction;
  delete dbDataSet;
}
//---------------------------------------------------------------------------
void __fastcall TformRecordOutgoingBill::staticBetaModifyTransactionsClick(TObject *Sender)
{
 if (optionsPermissions["table.transactions"]["insert"] == "true")
 {
  FORM_TRY
   TformWizardModifyTransactions * wiz = new TformWizardModifyTransactions(NULL, initial_state["id"], mod_out_bill);
   wiz->Show();
  FORM_CATCH
 }
}
//---------------------------------------------------------------------------
void __fastcall TformRecordOutgoingBill::staticTransactionDetailsClick(TObject *Sender)
{
  // jhaley 20100910: Launch transaction details form when link is clicked
  if (optionsPermissions["table.transactions"]["view"] == "true")
  {
    string sql = "select tax_entities.name, transactions.effective_date, transactions.fiscal_date, accounts.name, "
                 "account_types.item, transactions.note, transactions.debit_amount, transactions.credit_amount, "
                 "transactions.transaction_number, transactions.current_id, transactions.id "
                 "from transactions "
                 "inner join accounts on transactions.account_id = accounts.id "
                 "inner join account_types on accounts.account_type_id = account_types.id "
                 "left join tax_entities on transactions.tax_entity_id = tax_entities.id "
                 "where transactions.bill_id = " + initial_state["id"];

    TformSpecialTransactionDetails *details = NULL;
    FORM_TRY
      details = new TformSpecialTransactionDetails(NULL, initial_state["id"], "bill_id");
    FORM_CATCH
    details->Filter->AddColumnSpec("Tax Entity",  "tax_entities", as_text,   true,  120, rawString);
    details->Filter->AddColumnSpec("Eff. Date",   "transactions", as_date,   true,   75);
    details->Filter->AddColumnSpec("Fisc. Date",  "transactions", as_date,   true,   75);
    details->Filter->AddColumnSpec("Account",     "accounts",     as_text,   true,  140);
    details->Filter->AddColumnSpec("Type",        "accounts",     as_text,   true,   75);
    details->Filter->AddColumnSpec("Note",        "transactions", as_text,   false, 120, sentenceString);
    details->Filter->AddColumnSpec("Debit",       "transactions", as_double, true,   75, nonzeroAmountString);
    details->Filter->AddColumnSpec("Credit",      "transactions", as_double, true,   75, nonzeroAmountString);
    details->Filter->AddColumnSpec("Trans #",     "transactions", as_int,    true,   75);
    details->Filter->AddColumnSpec("Current Id",  "current_id",   as_int,    false,  80);
    details->Filter->AddColumnSpec("Internal Id", "internal_id",  as_int,    false,  80);
    details->Filter->FetchPreferences("recordoutgoingbill.transactions_filter");
    details->Execute(sql);
    details->Show();
  }
}
//---------------------------------------------------------------------------
void TformRecordOutgoingBill::Do(map<string, string> & p_parameters)
{
 FORM_TRY
  TformRecordOutgoingBill * z = new TformRecordOutgoingBill(NULL, p_parameters);
  z->Show();
 FORM_CATCH
}
//---------------------------------------------------------------------------
void __fastcall TformRecordOutgoingBill::txtADAFormClick(TObject *Sender)
{
 if (is_editing) { PSay ("You're currently editing this bill record.  You should save or undo your changes before running any reports that rely on that data.", m_warning, MB_OK); return; }
 vector<string> bill_id;
 bill_id.push_back(initial_state["id"]);
 TformReportDentalBillingForm::DoADA(bill_id);
}
//---------------------------------------------------------------------------

void __fastcall TformRecordOutgoingBill::FormClose(TObject *Sender,
      TCloseAction &Action)
{
  TformRecordGeneric::FormClose(Sender, Action);
  transactions_summary.Close(Action); // jhaley: shutdown transactions summary        
}
//---------------------------------------------------------------------------
// jhaley 20100910: Added checkbox to support transaction summary grouping by account
void __fastcall TformRecordOutgoingBill::checkTransactionAccountClick(TObject *Sender)
{
  transactions_summary.SetShowAccount(checkTransactionAccount->Checked); // set state
  transactions_summary.CheckBoxClicked();                                // update it.
}
//---------------------------------------------------------------------------
// jhaley 20100910: Added checkbox to support transaction summary grouping by tax entity name
void __fastcall TformRecordOutgoingBill::checkTransactionNameClick(TObject *Sender)
{
  transactions_summary.SetShowName(checkTransactionName->Checked); // set state
  transactions_summary.CheckBoxClicked();                          // update it.
}
//---------------------------------------------------------------------------

