//---------------------------------------------------------------------------

#include <vcl.h>
#pragma hdrstop

#include "codeRecordParticipationsDiab.h"
#include "codeSearchPerson.h"
#include "codeSearchEvent.h"
#include "codeRecordConsent.h"
//---------------------------------------------------------------------------
#pragma package(smart_init)
#pragma link "codeRecordGeneric"
#pragma link "codeFrameObservationList"
#pragma link "codeRubberContainer"
#pragma resource "*.dfm"
TformRecordParticipationsDiab *formRecordParticipationsDiab;
//---------------------------------------------------------------------------
// jhaley 20100804: Please note that although this is still known as "Diab", it is in fact now used by all departments
// interested in viewing participations.
__fastcall TformRecordParticipationsDiab::TformRecordParticipationsDiab(TComponent* Owner, map<string,string> p_parameters)
        : TformRecordGeneric(Owner, p_parameters, string("participations"))
{
  consents_filter.SetListViewHandle (listConsents);
  consents_filter.SetSecurity("consents");
  consents_filter.AddColumnSpec("Date", "consents", as_date, true, 80);
  consents_filter.AddColumnSpec("Type of Consent", "consents", as_text, true, 120);
  consents_filter.AddColumnSpec("Note", "consents", as_text, true, 140);
  consents_filter.AddColumnSpec("Current Id", "current_id", as_int, false, 80);
  consents_filter.AddColumnSpec("Internal Id", "internal_id", as_int, false, 80);
  consents_filter.FetchPreferences("recordparticipationsdiab.consents_filter");
  consents_filter.SetIconStatus(true);
  consents_filter.SetSQLStatement ("");
  consents_filter.RefreshDataSet();

  Initialize();
}
//---------------------------------------------------------------------------
bool TformRecordParticipationsDiab::ScreenStart()
{
  string group_id = initial_state["group_id"];
  if (!is_new_record)
    group_id = GetOneField(dbDatabase, "select group_id from participations where id = '" + initial_state["id"] + "' ");

  if (group_id == lookupTables->IdOf("groups", "Diabetes"))
    lookupTables->RegisterComboBox("diab_special_types", cboSpecial);
  else
  {
    cboSpecial->Text    = "";
    cboSpecial->Enabled = false;
    cboSpecial->Visible = false;
    lblSpecial->Visible = false;
  }

  TformRecordGeneric::ScreenStart();
  return true;
}
//---------------------------------------------------------------------------
bool TformRecordParticipationsDiab::ScreenStop()
{
  if (initial_state["group_id"] == lookupTables->IdOf("groups", "Diabetes"))
    lookupTables->UnregisterComboBox("diab_special_types", cboSpecial);

  TformRecordGeneric::ScreenStop();
  return true;
}
//---------------------------------------------------------------------------
bool TformRecordParticipationsDiab::IsValidOnScreen()
{
  // check to make sure that the values entered in the combo boxe
  // corrospond to values in that look up table.
  // check to make sure the dates entered are valid
  // give the user a message box and prompt to re-enter if incorrect

  // jhaley: if there isn't a person, the user should search for one...

  if (GetValidLinkId(txtEvent) == "")
  {
    PSay("Please attach an event", m_missing_data, MB_OK);
    btnEventSearch->SetFocus();
    return false;
  }
  else if (GetValidLinkId(txtPerson) == "")
  {
    PSay("There is no person associated with this record", m_missing_data, MB_OK);
    btnPersonSearch->SetFocus();
    return false;
  }
  else if (initial_state["group_id"] == lookupTables->IdOf("groups", "Diabetes") &&
           !lookupTables->IsInLookUpTable("diab_special_types", cboSpecial->Text.c_str()))
  {
    PSay("Please enter valid special info", m_invalid_data, MB_OK);
    cboSpecial->SetFocus();
    return false;
  }
  else if (initial_state["group_id"] == "" || initial_state["group_id"] == "0")
  {
    PSay("This participation does not have a valid department. Please contact a programmer if this occured while creating a new participation.",
         m_program_error, MB_OK);
    return false;
  }

  return true;
}
//---------------------------------------------------------------------------
bool TformRecordParticipationsDiab::PutDataOnScreen()
{
  // put the values on the form from the map initial_state
  // values may have come from GetDataFromDatabase if it's an existing record,
  // or values may have been passed to the form in this map if it's a new record

  editNote->Text = SentenceString(initial_state["note"]).c_str();

  if (initial_state["group_id"] == lookupTables->IdOf("groups", "Diabetes"))
    cboSpecial->Text = PrettyString(lookupTables->ValueOf("diab_special_types", initial_state["diab_special_id"])).c_str();

  // get the person name associated with the person id that was either selected
  // from the database if it's an existing record, or passed to the form if
  // it's a new record. display the name as a link to the person record.

  CreateLink(txtPerson, btnPersonSearch, this, initial_state["person_id"], "people");
  CreateLink(txtEvent, btnEventSearch, this, initial_state["event_id"], "events");

  TformRecordGeneric::PutDataOnScreen();
  return true; // what can really go wrong?
}
//---------------------------------------------------------------------------
bool TformRecordParticipationsDiab::GetDataFromScreen()
{
  // place the values currently on the screen into
  // the map that will be used to update or insert the record
  // the parent form will call the insert or update function, and get the id
  // number if necessary and take care of current_id

  ending_state["group_id"] = initial_state["group_id"]; // jhaley: group_id is constant across this form.
  ending_state["event_id"] = GetValidLinkId(txtEvent);
  ending_state["person_id"] = GetValidLinkId(txtPerson);
  if (initial_state["group_id"] == lookupTables->IdOf("groups", "Diabetes"))
    ending_state["diab_special_id"] = lookupTables->IdOf("diab_special_types", cboSpecial->Text.c_str());
  else
    ending_state["diab_special_id"] = "0"; // adapted from old Behavioral Health participations form
  ending_state["note"] = editNote->Text.c_str();
  ending_state["id"] = initial_state["id"];

  TformRecordGeneric::GetDataFromScreen();
  return true; // again, not much can go wrong here
}
//---------------------------------------------------------------------------
bool TformRecordParticipationsDiab::ScreenLock()
{
  // disable panel containing edit boxes
  panelGroup->Enabled = false;
  editNote->ReadOnly = true;
  editNote->TabStop = false;
  TformRecordGeneric::ScreenLock();
  return true;
}
//---------------------------------------------------------------------------
bool TformRecordParticipationsDiab::ScreenUnlock()
{
  // enable panel containing edit boxes
  panelGroup->Enabled = true;
  editNote->ReadOnly = false;
  editNote->TabStop = true;
  TformRecordGeneric::ScreenUnlock();
  return true;
}
//---------------------------------------------------------------------------
void __fastcall TformRecordParticipationsDiab::btnPersonSearchClick(TObject *Sender)
{
  // create and show new person search form
  vector<string> return_vector;
  FORM_TRY
    TformSearchPerson * formPersonSearch = new TformSearchPerson(NULL,&return_vector);
    formPersonSearch->ShowTaskModal(this);
  FORM_CATCH
  if (! return_vector.empty())
    CreateLink(txtPerson, btnPersonSearch, this, return_vector[0], "people");
}
//---------------------------------------------------------------------------
void __fastcall TformRecordParticipationsDiab::btnEventSearchClick(TObject *Sender)
{
  // create and show new person search form
  vector<string> return_vector;
  FORM_TRY
    TformSearchEvent * eventSearch = new TformSearchEvent(NULL, &return_vector, lookupTables->IdOf("groups", "Diabetes"));
    eventSearch->ShowTaskModal(this);
  FORM_CATCH
  if (! return_vector.empty())
    CreateLink(txtEvent, btnEventSearch, this, return_vector[0], "events");
}
//---------------------------------------------------------------------------
void __fastcall TformRecordParticipationsDiab::cboSpecialKeyPress(TObject *Sender, char &Key)
{
  FindComboBoxItem(Sender, Key);
}
//---------------------------------------------------------------------------
bool TformRecordParticipationsDiab::NotNewAnymore()
{
  frameObs->SetSource(obs_list_participation, initial_state["id"]);

  string sql;
  bool refresh = false;

  sql = "select consents.signature_date, consent_types.item, consents.note, "
        "consents.current_id, consents.id "
        "from consents, consent_types "
        "where consents.consent_id = consent_types.id and "
        "consents.participations_id = " + initial_state["id"];
  consents_filter.SetSQLStatement (sql);
  consents_filter.RefreshDataSet();
  if(consents_filter.GetDatasetSize() > 1)
    btnPersonSearch->Enabled = false;
  consents_filter.SetAddEdit(this, AddRecord(&(TformRecordParticipationsDiab::AddConsent)),
                             EditRecord(&(TformRecordParticipationsDiab::EditConsent)));
  consents_request = atlas->RequestNotificationWhen("consents.participations_id", initial_state["id"], rel_e, as_int, this,
                     RefreshHandler(&(TformRecordParticipationsDiab::AtlasRefreshListView)));

  TformRecordGeneric::NotNewAnymore();
  return true;
}
//---------------------------------------------------------------------------
bool TformRecordParticipationsDiab::AtlasRefreshListView(int request_num, string matched_value)
{
 // TODO: wtf?
 // if(measure_filter.GetDatasetSize() > 0)
 //   btnPersonSearch->Enabled = false;

  if (request_num == consents_request)
  {
    consents_filter.RefreshDataSet();
    // TODO: more wtf?
    if(consents_filter.GetDatasetSize() > 0)
      btnPersonSearch->Enabled = false;
  }
  return true;
}
//---------------------------------------------------------------------------
/*
void TformRecordParticipationsDiab::AddMeasurement()
{
  map<string,string> temp_map;
  temp_map["id"] = "new";
  temp_map["person_id"] = initial_state["person_id"];
  temp_map["group_id"] = current_groupid;
  temp_map["participations_id"] = initial_state["id"];
  temp_map["date_of_measurement"] = GetOneField(dbDatabase, "select date_of_event from events where id = " + initial_state["event_id"]);
  FORM_TRY
    TformRecordMeasurement * measurementRecord = new TformRecordMeasurement(NULL, temp_map);
    measurementRecord->Show();
  FORM_CATCH
}
//---------------------------------------------------------------------------

void TformRecordParticipationsDiab::EditMeasurement()
{
  map<string,string> temp_map;
  temp_map["id"] = measure_filter.GetSelectedRecordID();
  FORM_TRY
    TformRecordMeasurement * measurementRecord = new TformRecordMeasurement(NULL, temp_map);
    measurementRecord->Show();
  FORM_CATCH
}
//---------------------------------------------------------------------------
*/
void TformRecordParticipationsDiab::AddConsent()
{
  map<string,string> temp_map;
  string person_id = GetValidLinkId(txtPerson);
  string event_id  = GetValidLinkId(txtEvent);

  // jhaley: These could be in the process of being changed when the form is unlocked for editing, so use of initial_state
  // in that circumstance would be wrong. Check and be sure there are valid IDs before attaching a consent.
  if (person_id != "" && event_id != "")
  {
    temp_map["id"] = "new";
    temp_map["person_id"] = person_id;
    temp_map["participations_id"] = initial_state["id"];
    temp_map["signature_date"] = GetOneField(dbDatabase, "select date_of_event from events where id = '" + event_id + "' ");
    FORM_TRY
      TformRecordConsent * consentRecord = new TformRecordConsent(NULL, temp_map);
      consentRecord->Show();
    FORM_CATCH
  }
  else
    PSay("This participation needs both a valid person and event before consents may be added", m_missing_data, MB_OK);
}
//---------------------------------------------------------------------------

void TformRecordParticipationsDiab::EditConsent()
{
  map<string,string> temp_map;
  temp_map["id"] = consents_filter.GetSelectedRecordID();
  FORM_TRY
    TformRecordConsent * consentRecord = new TformRecordConsent(NULL, temp_map);
    consentRecord->Show();
  FORM_CATCH
}
//---------------------------------------------------------------------------
bool TformRecordParticipationsDiab::IsNewAgain()
{
  frameObs->SetSource(obs_list_none, "0");

  consents_filter.SetSQLStatement(string(""));
  consents_filter.RefreshDataSet();
  consents_filter.SetAddEdit(this, NULL, NULL);

  TformRecordGeneric::IsNewAgain();
  return true;
}
//---------------------------------------------------------------------------
bool TformRecordParticipationsDiab::CheckForUselessRecord()
{
  return true;
}
//---------------------------------------------------------------------------
bool TformRecordParticipationsDiab::BeforeSave()
{
 Processing(true);
 vector<map<string, string> > people_vecmap;
 vector<map<string, string> > activity_vecmap;
 vector<map<string, string> > test_map;
 map<string, string> data_map;
 string sql;
 bool can_commit = true;

 sql = "select participations.person_id, person_names.last_name, person_names.first_name, "
         "participations.current_id, participations.id "
         "from participations, people, person_names "
         "where participations.event_id = " + ending_state["event_id"] + " and "
         "participations.person_id = people.id and "
         "person_names.id = people.person_names_id order by person_names.last_name, "
         "person_names.first_name";
 if (!SqlToVecMap(lock_transaction, sql, people_vecmap))
   can_commit = false;

 sql = "select event_activities.id, event_activities.activity_code_id, activity_code_types.item, "
          "event_activities.intensity_id, intensity_types.item, event_activities.distance, event_activities.time_spent, "
          "event_activities.steps, event_activities.current_id, event_activities.id, "
          "events.date_of_event "
          "from event_activities, activity_code_types, intensity_types, events "
          "where event_activities.activity_code_id = activity_code_types.id and "
          "event_activities.intensity_id = intensity_types.id and "
          "event_activities.event_id = '" + ending_state["event_id"] + "' and "
          "events.id = '" + ending_state["event_id"] + "'";
 if (!SqlToVecMap(lock_transaction, sql, activity_vecmap))
   can_commit = false;

   for (unsigned int i = 0; i < people_vecmap.size(); i++)
   {
    for (unsigned int j = 0; j < activity_vecmap.size(); j++)
    {
     sql = "select activities.person_id, "
                  "activities.activity_code_id, "
                  "activities.intensity_id, "
                  "activities.date_of_activity, "
                  "activities.distance, "
                  "activities.time_spent, "
                  "activities.steps, "
                  "activities.event_activity_id "
             "from activities, "
                  "event_activities, "
                  "participations "
             "where event_activities.id = '" + activity_vecmap[j]["id"] + "' "
                 "and participations.person_id = '" + people_vecmap[i]["person_id"] + "' "
                 "and activities.person_id = participations.person_id "
                 "and activities.event_activity_id = event_activities.id ";
     if (SqlToVecMap(lock_transaction, sql, test_map))
     {
      if (activity_vecmap[j]["current_id"] < people_vecmap[i]["current_id"])
        data_map["current_id"] = activity_vecmap[j]["current_id"];
      else
        data_map["current_id"] = people_vecmap[i]["current_id"];
      data_map["last_update_stamp"] = RightNow();
      data_map["last_updated_by"] = current_userid;
      data_map["person_id"] = people_vecmap[i]["person_id"];
      data_map["activity_code_id"] = activity_vecmap[j]["activity_code_id"];
      data_map["intensity_id"] = activity_vecmap[j]["intensity_id"];
      data_map["date_of_activity"] = activity_vecmap[j]["date_of_event"];
      data_map["distance"] = activity_vecmap[j]["distance"];
      data_map["time_spent"] = activity_vecmap[j]["time_spent"];
      data_map["steps"] = activity_vecmap[j]["steps"];
      data_map["activity_logsheet_id"]  = "0";
      data_map["event_activity_id"] = activity_vecmap[j]["id"];
      data_map["fitness_attendance_id"] = "0";
      if (test_map.size() == 0)
      {
       data_map["id"] = GetNextId (lock_transaction, "activities");
       if (!ExecuteInsertStatement(lock_transaction, "activities", data_map))
         can_commit = false;
      }
      else
      {
       sql = "select id from activities "
             "where activities.event_activity_id = '" + activity_vecmap[j]["id"] + "' "
             "and activities.person_id = '" + people_vecmap[i]["person_id"] + "'";
       data_map["id"] = GetOneField(lock_transaction,sql);
       if (!ExecuteUpdateStatement(lock_transaction, "activities", data_map))
         can_commit = false;
      }
     }
     else
     {
      can_commit = false;
     }
    }
   }
 Processing(false);
 return can_commit;
}
//---------------------------------------------------------------------------
void TformRecordParticipationsDiab::Do(map<string, string> & p_parameters)
{
 FORM_TRY
  TformRecordParticipationsDiab * z = new TformRecordParticipationsDiab(NULL, p_parameters);
  z->Show();
 FORM_CATCH
}
//---------------------------------------------------------------------------

