//----------------------------------------------------------------------------------------------------------------------

#include <vcl.h>
#pragma hdrstop

#include "codeRecordPatientAssessment.h"
#include "codeSearchTaskBeh.h"
#include "codeSearchTaskCare.h"
#include "codeSearchTaskCobVisit.h"
#include "codeSearchTaskCom.h"
#include "codeSearchTaskDiab.h"
#include "codeSearchTaskLac.h"
#include "codeSearchTaskTrans.h"
#include "codeSearchTaskPub.h"
#include "codeSpecialDisambiguate.h"
#include "codeFrameAssessmentFallRisks.h"
#include "codeFrameAssessmentNotes.h"
#include "codeFrameAssessmentAllergies.h"
#include "codeFrameAssessmentMedications.h"
#include "codeFrameAssessmentEdu.h"
#include "codeFrameAssessmentReferrals.h"
#include "codeFrameAssessmentVitals.h"
#include "codeFrameAssessmentSkin.h"
#include "codeFrameAssessmentWound.h"
#include "codeFrameAssessmentDME.h"
#include "codeFrameAssessmentSafety.h"
#include "codeFrameAssessmentIVs.h"
#include "codeFrameAssessmentNeuro.h"
#include "codeFrameAssessmentRespiratory.h"
#include "codeFrameAssessmentGastro.h"
#include "codeFrameAssessmentGenitourinary.h"
#include "codeFrameAssessmentCardio.h"
#include "codeFrameAssessmentLimits.h"
#include "codeFrameAssessmentActivityLevel.h"
#include "codeFrameAssessmentDiet.h"
#include "codeFrameAssessmentMuscle.h"
#include "codeFrameAssessmentSensory.h"
#include "reportAssessment.h"
//----------------------------------------------------------------------------------------------------------------------
#pragma package(smart_init)
#pragma link "codeRecordGeneric"
#pragma link "codePaddedBox"
#pragma resource "*.dfm"
TformRecordPatientAssessment *formRecordPatientAssessment;
//----------------------------------------------------------------------------------------------------------------------
__fastcall TformRecordPatientAssessment::TformRecordPatientAssessment(TComponent* Owner, map<string, string> &p_parameters)
        : TformRecordGeneric(Owner, p_parameters, string("patient_assessment")), area_names(), child_frames()
{
  current_frame = NULL; // We start out with no frame selected as the current frame
  lock_count = 0;
  CreateFrames();
  Initialize();

  txtPrintReport->Font->Color = link_color;
}
//----------------------------------------------------------------------------------------------------------------------
// SetAreaNames - Populates the left-hand list view with the hard-coded assessment area names. Each one represents one
// of the subclasses of TframeAssessmentGeneric, and clicking that item shows the corresponding frame in the right-
// hand panel. Also populates the area_names vector for lookup by area number.
void TformRecordPatientAssessment::SetAreaNames()
{
  static const char *static_area_names[num_assessment_areas] =
  {
    "Allergies", "Vitals", "Medications", "Skin", "Wound Care", "Neurological", "Respiratory", "Cardiovascular",
    "Gastrointestinal", "Genitourinary", "Musculoskeletal", "Safety", "Fall Risks", "Sensory", "Functional Limits",
    "Activity Level", "Nutrition", "Equipment", "Intravenous", "Education", "Notes", "Referrals"
  };

  listAreas->Items->Clear();
  for(int i = 0; i < num_assessment_areas; i++)
  {
    area_names.push_back(static_area_names[i]);

    // Add to list box
    TListItem *new_item = listAreas->Items->Add();
    new_item->Caption = static_area_names[i];
  }

  // Add a single column
  listAreas->Columns->Clear();
  TListColumn *new_col = listAreas->Columns->Add();
  new_col->Width = ColumnTextWidth;
}
//----------------------------------------------------------------------------------------------------------------------
// CreateFrames - All TframeAssessmentGeneric descendant instances are instantiated here and added to the child_frames
// vector under their corresponding area number.
void TformRecordPatientAssessment::CreateFrames()
{
  child_frames.resize(num_assessment_areas, NULL);

  child_frames[area_allergies]   = new TframeAssessmentAllergies(this);
  child_frames[area_vitals]      = new TframeAssessmentVitals(this);
  child_frames[area_medications] = new TframeAssessmentMedications(this);
  child_frames[area_skin]        = new TframeAssessmentSkin(this);
  child_frames[area_wound_care]  = new TframeAssessmentWound(this);
  child_frames[area_neurology]   = new TframeAssessmentNeuro(this);
  child_frames[area_respiratory] = new TframeAssessmentRespiratory(this);
  child_frames[area_cardio]      = new TframeAssessmentCardio(this);
  child_frames[area_gastro]      = new TframeAssessmentGastro(this);
  child_frames[area_genitouri]   = new TframeAssessmentGenitourinary(this);
  child_frames[area_musculo]     = new TframeAssessmentMuscle(this); 
  child_frames[area_safety]      = new TframeAssessmentSafety(this);
  child_frames[area_fall_risk]   = new TframeAssessmentFallRisks(this);
  child_frames[area_sensory]     = new TframeAssessmentSensory(this);
  child_frames[area_func_limits] = new TframeAssessmentLimits(this);
  child_frames[area_activitylvl] = new TframeAssessmentActivityLevel(this);
  child_frames[area_nutrition]   = new TframeAssessmentDiet(this);
  child_frames[area_equipment]   = new TframeAssessmentDME(this);
  child_frames[area_education]   = new TframeAssessmentEdu(this);
  child_frames[area_notes]       = new TframeAssessmentNotes(this);
  child_frames[area_referrals]   = new TframeAssessmentReferrals(this);
  child_frames[area_intravenous] = new TframeAssessmentIVs(this);

  for(int i = 0; i < num_assessment_areas; i++)
  {
    if(child_frames[i])
    {
      child_frames[i]->Parent  = panelRight; // parent control is the right-hand panel
      child_frames[i]->Align   = alClient;   // align to full area of parent panel
      child_frames[i]->Visible = false;      // all start out hidden
      child_frames[i]->Initialize();
    }
  }
}
//----------------------------------------------------------------------------------------------------------------------
// FormShow - Overridden from the parent class to populate the left-hand-side area list view. Done here because
// messing with list views inside form constructors needs to be deprecated to whatever extent possible, due to the
// seemingly random access violations it can cause.
void __fastcall TformRecordPatientAssessment::FormShow(TObject *Sender)
{
  SetAreaNames();

  TformRecordGeneric::FormShow(Sender);
}
//----------------------------------------------------------------------------------------------------------------------
// DataClose - Overridden from RecordGeneric to provide the ability to save any pending data in the various
// assessment frames before the form closes.
bool TformRecordPatientAssessment::DataClose()
{
  // check frames for modified status
  if(!is_editing && !is_new_record) // Not when editing or creating the parent form's record; frames are disabled then.
  {
    bool have_modified = false;

    for(size_t i = 0; i < child_frames.size(); i++)
    {
      if(child_frames[i] && child_frames[i]->CheckModified())
        have_modified = true;
    }
    if(have_modified)
    {
      string msg = "You have modified information in one or more assessment areas. Do you want to save these changes?";
      int result = PSay(msg, m_confirmation, MB_YESNOCANCEL);
      bool all_success = true;

      switch(result)
      {
      case IDYES:
        for(size_t i = 0; i < child_frames.size(); i++)
        {
          if(child_frames[i] && child_frames[i]->CheckModified())
            all_success = child_frames[i]->DoSave(true) && all_success; // any one failure will return false
        }
        return all_success;
      case IDNO:
        // Don't care what happens. Close process will clean up open records/transactions.
        return true;
      case IDCANCEL:
        return false; // Don't exit, user canceled
      }
    }
    return true;
  }
  else
    return TformRecordGeneric::DataClose(); // Defer to normal RecordGeneric logic if editing parent record
}
//----------------------------------------------------------------------------------------------------------------------
// SetUnlockEnabled - When a frame is in editing mode, the parent form cannot be unlocked for editing.
void TformRecordPatientAssessment::SetUnlockEnabled(bool enabled)
{
  if(enabled && lock_count > 0) // Re-enabling, decrement lock count
    --lock_count;

  if(lock_count == 0)           // State changes only when lock count is clear
    buttonA->Enabled = enabled;

  if(!enabled)                  // Disabling, increment lock count
    ++lock_count;
}
//----------------------------------------------------------------------------------------------------------------------
// AtlasRefresh - Registered for Atlas requests. Dispatches requests to child frames.
bool TformRecordPatientAssessment::AtlasRefresh(int request_num, string matched_value)
{
  for(int i = 0; i < num_assessment_areas; i++)
  {
    if(child_frames[i])
    {
      if(child_frames[i]->AtlasNotify(request_num, matched_value))
        return true; // Bail out early if it was handled.
    }
  }
  return true;
}
//----------------------------------------------------------------------------------------------------------------------
// RequestNotificationWhen - Wrapper for Atlas notifications, exposed to child frames.
int TformRecordPatientAssessment::RequestNotificationWhen(string key, string value, int relational_op, int field_type)
{
  return atlas->RequestNotificationWhen(key, value, relational_op, field_type, this,
                                        RefreshHandler(&TformRecordPatientAssessment::AtlasRefresh));
}
//----------------------------------------------------------------------------------------------------------------------
// GetAssessmentID - Return the ID # of the assessment record
string TformRecordPatientAssessment::GetAssessmentID()
{
  return initial_state["id"];
}
//----------------------------------------------------------------------------------------------------------------------
// ChangeToArea - Main routine to change to an indicated assessment area
bool TformRecordPatientAssessment::ChangeToArea(int area_id)
{
  if(area_id < 0 || area_id >= num_assessment_areas)
    return false; // invalid area id

  TframeAssessmentGeneric *target_frame = child_frames[area_id];
  if(target_frame)
  {
    if(target_frame->HaveViewPermission()) // Check for "view" permissions here before changing. Simplest solution.
    {
      if(current_frame)
        current_frame->MakeVisible(false);

      current_frame = target_frame;
      current_frame->MakeVisible(true);

      return true;
    }
    else
      PSay("You do not have permission to view information in this assessment area. Contact your supervisor.",
           m_message, MB_OK);
  }

  // Not valid, or no view permissions
  return false;
}
//----------------------------------------------------------------------------------------------------------------------
// *** Local Form Logic -- Saving/Unlocking the assessment record has nothing to do with the assessment areas, which
//     function as their own subordinate but independent record types. In fact when editing or creating the assessment,
//     all of the assessment area frames are disabled and cannot be edited/altered.
//----------------------------------------------------------------------------------------------------------------------
bool TformRecordPatientAssessment::IsValidOnScreen()
{
  // Need a valid visit task
  if(IsBlankOrZero(GetValidLinkId(txtVisit)))
  {
    PSay("The patient assessment must be associated with a visit.", m_missing_data, MB_OK);
    btnVisitSearch->SetFocus();
    return false;
  }

  // Need a valid person on the visit
  if(IsBlankOrZero(GetValidLinkId(txtPerson)))
  {
    PSay("The visit selected does not have a valid client. Please select a different visit, or add a client to that "
         "visit and then re-select that visit using the search form.", m_invalid_data, MB_OK);
    btnVisitSearch->SetFocus();
    return false;
  }

  return TformRecordGeneric::IsValidOnScreen();
}
//----------------------------------------------------------------------------------------------------------------------
bool TformRecordPatientAssessment::PutDataOnScreen()
{
  // Create link to visit
  CreateLink(txtVisit, btnVisitSearch, this, initial_state["task_id"], "tasks");

  // If visit is valid, also create link to visit person
  string task_id = GetValidLinkId(txtVisit);
  string person_id = "";
  if(!IsBlankOrZero(task_id))
  {
    string person_id_sql = "select person_id from tasks where id = '" + initial_state["task_id"] + "' ";
    person_id = GetOneField(dbDatabase, person_id_sql);
  }
  CreateLink(txtPerson, NULL, this, person_id, "people");

  // Notify all frames that we now have a valid task id
  if(!IsBlankOrZero(task_id) && !is_new_record)
  {
    for(int i = 0; i < num_assessment_areas; i++)
    {
      if(child_frames[i])
        child_frames[i]->SetTaskId(task_id);
    }
  }

  // There is no concept of "Save and create new" here; a task should generally only have one assessment associated
  // with it. The database design would allow more than one, because, well, who knows why they might want to do that,
  // but it might as well be possible. The assessment stores a task_id; we do not link tasks to an assessment- tasks is
  // a large enough table as it is.
  buttonC->Enabled = false;
  buttonC->Visible = false;

  return TformRecordGeneric::PutDataOnScreen();
}
//----------------------------------------------------------------------------------------------------------------------
bool TformRecordPatientAssessment::GetDataFromScreen()
{
  ending_state["id"]      = initial_state["id"];
  ending_state["task_id"] = GetValidLinkId(txtVisit);

  return TformRecordGeneric::GetDataFromScreen();
}
//----------------------------------------------------------------------------------------------------------------------
bool TformRecordPatientAssessment::ScreenLock()
{
  btnVisitSearch->Enabled = false;

  return TformRecordGeneric::ScreenLock();
}
//----------------------------------------------------------------------------------------------------------------------
bool TformRecordPatientAssessment::ScreenUnlock()
{
  btnVisitSearch->Enabled = true;

  // Note that this cannot be done while any of the child frames are in their editing modes because they notify this
  // form and it disables the Unlock/Save button while they are in their own editing mode. Thus we do not worry here
  // about whether or not the forms might have unsaved data.
  for(size_t i = 0; i < child_frames.size(); i++)
  {
    if(child_frames[i])
      child_frames[i]->Lock();
  }

  return TformRecordGeneric::ScreenUnlock();
}
//----------------------------------------------------------------------------------------------------------------------
bool TformRecordPatientAssessment::NotNewAnymore()
{
  string task_id = GetValidLinkId(txtVisit);

  // Notify all frames that we now have a valid task id
  if(!IsBlankOrZero(task_id))
  {
    for(int i = 0; i < num_assessment_areas; i++)
    {
      if(child_frames[i])
        child_frames[i]->SetTaskId(task_id);
    }
  }

  return TformRecordGeneric::NotNewAnymore();
}
//----------------------------------------------------------------------------------------------------------------------
// *** VCL Event Handlers
//----------------------------------------------------------------------------------------------------------------------
// Attach a visit when the search button is clicked
void __fastcall TformRecordPatientAssessment::btnVisitSearchClick(TObject *Sender)
{
  map<string, string> params;
  string group_id;

  if(!IsBlankOrZero(GetValidLinkId(txtVisit)))
  {
    int response = PSay("This assessment already has a valid visit. Changing the visit of an assessment once any data "
                        "has been entered into it will cause most of that data to be dropped from the assessment.\n\n"
                        "If individual areas of the assessment are incorrect, you can edit them individually instead.\n\n"
                        "Are you sure you want to do this?", m_warning, MB_YESNO);
    if(response != IDYES)
      return;
  }

  params["prefix"] = "table.tasks";
  params["suffix"] = "";
  params["key"]    = "view";
  params["value"]  = "true";
  params["instructions"] = "You have permission to search for visits created in multiple departments.  "
                           "Please select a department which you wish to find a visit within.";
  set<string> group_id_set;
  string group_id_sql = "select groups.id from groups inner join users on users.group_id = groups.id "
                        "where groups.id > 0 and groups.valid = 1 and users.valid = 1 group by groups.id ";
  SqlToSet(dbDatabase, group_id_sql, group_id_set);
  params["include"] = SetToCommaString(group_id_set);
  params["exclude"] = "";
  params["table"]   = "groups";
  params["sql"]     = "select id row_id, short_item row_value, item show_value from groups where valid = 1 and id > 0 ";

  group_id = TformSpecialDisambiguate::Disambiguate(this, params, PrettyString);

  // Here is why I should have finished that combined task search form already...
  if(!IsBlankOrZero(group_id))
  {
    vector<string> return_vec;

    if(group_id == lookupTables->IdOf("groups", "Behavioral Health"))
    {
      return_vec = TformSearchTaskBeh::Do(this);
    }
    else if(group_id == lookupTables->IdOf("groups", "Caregiver Support Program"))
    {
      return_vec = TformSearchTaskCare::Do(this);
    }
    else if(group_id == lookupTables->IdOf("groups", "Community Health"))
    {
      return_vec = TformSearchTaskCom::Do(this);
    }
    else if(group_id == lookupTables->IdOf("groups", "Contract Health"))
    {
      return_vec = TformSearchTaskCobVisit::Do(this);
    }
    else if(group_id == lookupTables->IdOf("groups", "Diabetes"))
    {
      return_vec = TformSearchTaskDiab::Do(this);
    }
    else if(group_id == lookupTables->IdOf("groups", "Little Axe Clinic") ||
            group_id == lookupTables->IdOf("groups", "Shawnee Clinic")    ||
            group_id == lookupTables->IdOf("groups", "Dental Clinic"))
    {
      return_vec = TformSearchTaskLac::Do(this);
    }
    else if(group_id == lookupTables->IdOf("groups", "Public Health"))
    {
      return_vec = TformSearchTaskPub::Do(this);
    }
    else if(group_id == lookupTables->IdOf("groups", "Transportation"))
    {
      return_vec = TformSearchTaskTrans::Do(this);
    }
    if(!return_vec.empty())
    {
      CreateLink(txtVisit, btnVisitSearch, this, return_vec[0], "tasks");

      string person_id_sql = "select person_id from tasks where id = '" + return_vec[0] + "' ";
      string person_id = GetOneField(dbDatabase, person_id_sql);
      CreateLink(txtPerson, NULL, this, person_id, "people");
    }
  }
}
//----------------------------------------------------------------------------------------------------------------------
// listAreasSelectItem - Select an assessment area.
void __fastcall TformRecordPatientAssessment::listAreasSelectItem(TObject *Sender, TListItem *Item, bool Selected)
{
  // Changing the currently visible/selected area
  if(Item && Selected)
    ChangeToArea(Item->Index);
}
//----------------------------------------------------------------------------------------------------------------------
// appEventsHint - Got an application-level OnHint event, and need to pass it down to the current frame
void __fastcall TformRecordPatientAssessment::appEventsHint(TObject *Sender)
{
  if(Active && current_frame)
    current_frame->DoHint(Sender);
}
//----------------------------------------------------------------------------------------------------------------------
// FormActivate - Overridden to elevate application events handler to head of global list when form activates
void __fastcall TformRecordPatientAssessment::FormActivate(TObject *Sender)
{
  appEvents->Activate();
  TformRecordGeneric::FormActivate(Sender);
}
//----------------------------------------------------------------------------------------------------------------------
// *** Reporting
//----------------------------------------------------------------------------------------------------------------------
// DoTaskReport - Do the miniature task report which forms the first page of the assessment report. The task data
//  doesn't have its own panel in the assessment, as it's contained in the visit which is considered the "parent
//  record" of the patient assessment.
bool TformRecordPatientAssessment::DoTaskReport()
{
  bool success = false;
  TformReportAssessment *reportForm = dynamic_cast<TformReportAssessment *>(task_report_form);
  if(reportForm)
  {
    try
    {
      string sql =
        "select U.item staff_name, "
        " (PN.last_name || ', ' || PN.first_name || ' ' || PN.middle_name || ' ' || PN.suffix) person_name, "
        "  CVT.item visit_type, VOT.item outcome_type, T.date_of_task, T.time_of_task, T.time_spent, T.visit_time_spent "
        "from tasks T "
        "left join users U on T.user_id = U.id "
        "left join people P on T.person_id = P.id "
        "left join person_names PN on P.person_names_id = PN.id "
        "left join com_visit_types CVT on T.com_visit_id = CVT.id "
        "left join visit_outcome_types VOT on T.visit_outcome_id = VOT.id "
        "where T.id = " + initial_state["task_id"];
      pmapstrs task_map;
      SqlToMap(report_transaction, sql, task_map);

      reportForm->lblVStaffName->Caption    = PrettyString(task_map["staff_name"]).c_str();
      reportForm->lblVPatientName->Caption  = PrettyString(CollapseSpaces(task_map["person_name"])).c_str();
      reportForm->lblVVisitType->Caption    = PrettyString(task_map["visit_type"]).c_str();
      reportForm->lblVVisitOutcome->Caption = PrettyString(task_map["outcome_type"]).c_str();
      reportForm->lblVDate->Caption         = task_map["date_of_task"].c_str();
      reportForm->lblVTime->Caption         = task_map["time_of_task"].c_str();
      reportForm->lblVTotTime->Caption      = task_map["time_spent"].c_str();
      reportForm->lblVPatTime->Caption      = task_map["visit_time_spent"].c_str();
      reportForm->lblVisitUser->Caption     = current_fullname.c_str();

      compositeReport->Reports->Add(reportForm->reportVisit);
      success = true;
    }
    catch(...)
    {
    }
  }

  return success;
}
//----------------------------------------------------------------------------------------------------------------------
// txtPrintReportClick - Do the assessment report thing.
void __fastcall TformRecordPatientAssessment::txtPrintReportClick(TObject *Sender)
{
  TIBTransaction *local_transaction = NULL;

  // Recursion? Heck no.
  if(task_report_form || report_transaction)
  {
    PSay("Please do not try to run more than one report at once.", m_warning, MB_OK);
    return;
  }

  // Possible rejection cases:
  // Editing this,   Editing frames,   Completely new record
  if(is_editing   || lock_count     || !IsInt(initial_state["id"]) || IsBlankOrZero(initial_state["task_id"]))
  {
    // Ummmm, no.
    PSay("Please save the assessment you are working on before attempting to print it.", m_reminder, MB_OK);
    return;
  }

  TformReportAssessment *reportForm = NULL;

  FORM_TRY
    reportForm = new TformReportAssessment(NULL);
  FORM_CATCH_RETURN("TformReportAssessment", )

  // jhaley 20111115: save a pointer for task report
  task_report_form = dynamic_cast<TForm *>(reportForm);

  try
  {
    // Activate transaction
    local_transaction = report_transaction = new TIBTransaction(NULL);
    StdTransaction(report_transaction, dbDatabase);

    // Lock self, and lock all child frames; also, disable ability to close form
    buttonB->Enabled = false;
    SetUnlockEnabled(false);
    for(size_t i = 0; i < child_frames.size(); i++)
    {
      if(child_frames[i])
      {
        child_frames[i]->SetReportData(reportForm, report_transaction);
        child_frames[i]->Lock();
      }
    }

    // Note this call is not effectively modal, even though it will stop the execution of this event handler, because
    // like other QuickReport previews, it continues to pump the VCL event loop using TApplication::ProcessMessages.
    // Because of that, we need some protection around this call.
    compositeReport->Preview();

    // Cleanup.
    for(size_t i = 0; i < child_frames.size(); i++)
    {
      if(child_frames[i])
        child_frames[i]->ReportDone();
    }
  }
  __finally // No matter what happened, we need to run this unlock code or the form will be unusable henceforth.
  {
    // Unlock self and child frames.
    buttonB->Enabled = true;
    SetUnlockEnabled(true);
    for(size_t i = 0; i < child_frames.size(); i++)
    {
      if(child_frames[i])
        child_frames[i]->DoDiscard();
    }
    task_report_form = NULL;
    report_transaction = NULL;
  }

  if(reportForm)
    reportForm->CloseNow();

  if(local_transaction)
  {
    if(local_transaction->Active)
      local_transaction->Rollback();
    delete local_transaction;
  }
}
//----------------------------------------------------------------------------------------------------------------------
void __fastcall TformRecordPatientAssessment::compositeReportAddReports(TObject *Sender)
{
  bool task_report_success = false;
  int  count_valid_reports = 0;

  // Do the basic task report first.
  try
  {
    if(DoTaskReport())
      ++count_valid_reports;
  }
  catch(...)
  {
    // See below.
  }

  // Call the WriteReport method of each valid child assessment frame
  for(int i = 0; i < num_assessment_areas; i++)
  {
    try
    {
      if(child_frames[i] && child_frames[i]->HaveViewPermission())
      {
        if(child_frames[i]->WriteReport(compositeReport))
          ++count_valid_reports;
      }
    }
    catch(...)
    {
      // Whatever. Really don't care. Shouldn't happen. Handle your own errors in each report, or ELSE.
    }
  }

  // Nothing to do, somehow?
  if(!count_valid_reports)
  {
    PSay("There is either nothing to print, or errors occurred while printing every assessment panel. Please check "
         "your assessment data, and if this message continues to occur, contact a programmer.", m_program_error, MB_OK);
  }
}
//----------------------------------------------------------------------------------------------------------------------
// FormClose - jhaley 20111103: Must override FormClose event handler to ensure that the assessment form cannot be
// closed from the form itself while the non-modal assessment report preview window is open, which would cause a
// virtual explosion of a scale not yet witnessed by mortal men. Like on TformLaunchGeneric, a close request issued
// from outside the form will still be processed (and may still explode, but that's not avoidable in that case...)
void __fastcall TformRecordPatientAssessment::FormClose(TObject *Sender, TCloseAction &Action)
{
  if(Sender == static_cast<TObject *>(this) && buttonB->Enabled == false)
    Action = caNone;
  else
    TformRecordGeneric::FormClose(Sender, Action);
}
//----------------------------------------------------------------------------------------------------------------------

