//---------------------------------------------------------------------------

#include <vcl.h>
#pragma hdrstop

#include "codeRecordPersonNotes.h"
//---------------------------------------------------------------------------
#pragma package(smart_init)
#pragma link "codeRecordGeneric"
#pragma resource "*.dfm"
TformRecordPersonNotes *formRecordPersonNotes;
//---------------------------------------------------------------------------
__fastcall TformRecordPersonNotes::TformRecordPersonNotes(TComponent* Owner, map<string,string> p_parameters)
        : TformRecordGeneric(Owner,  p_parameters, string("person_notes"))
{
  Initialize();
  InfoBarText = "Do not use this screen to enter OR edit family, responsible party, emergency contact, or employer information.  That information should all go under 'Relationships'.";
}
//---------------------------------------------------------------------------
bool TformRecordPersonNotes::ScreenStart()
{
  lookupTables->RegisterComboBox("warning_types",comboWarning);

  TformRecordGeneric::ScreenStart();
  return true;
}
//---------------------------------------------------------------------------
bool TformRecordPersonNotes::ScreenStop()
{
  lookupTables->UnregisterComboBox("warning_types", comboWarning);
  TformRecordGeneric::ScreenStop();
  return true;
}
//---------------------------------------------------------------------------
bool TformRecordPersonNotes::IsValidOnScreen()
{
  // check to make sure that the note is not longer than 255 characters
  // give the user a message box and prompt to re-enter if too long
  // make sure there is a link to a person, although there's not much the user
  // can do to fix that if there's not
  if (!lookupTables->IsInLookUpTable ("warning_types", comboWarning->Text.c_str()))
  {
   PSay("Please enter a valid warning level", m_invalid_data, MB_OK);
   comboWarning->SetFocus();
   return false;
  } else // trying to unset a block without permission
  if (optionsPermissions["special.note_block_override"]["enabled"] != "true" &&
      initial_state["warning_id"] == lookupTables->IdOf("warning_types", "Block") &&
      lookupTables->IdOf("warning_types", comboWarning->Text.c_str()) != lookupTables->IdOf("warning_types", "Block"))
  {
   string sql_get_users = "select users.item from users inner join permissions on permissions.user_id = users.id and permissions.s = 'SPECIAL.NOTE_BLOCK_OVERRIDE' and permissions.k = 'ENABLED' and permissions.v = 'TRUE' where valid = 1";
   vector<string> users_with_note_block_override;
   if (!SqlToVec(dbDatabase, sql_get_users, users_with_note_block_override))
    PSay("Failed to get a list of users with permission to override a block.", m_database_error, MB_OK);
   PSay("You do not have permission to un-set a block. Please find someone who does. Users who may be able to help you include:\n" + VecToCommaString(users_with_note_block_override), m_warning, MB_OK);
   return false;
  } else // trying to set a block they can't undo
  if (optionsPermissions["special.note_block_override"]["enabled"] != "true" &&
      initial_state["warning_id"] != lookupTables->IdOf("warning_types", "Block") &&
      lookupTables->IdOf("warning_types", comboWarning->Text.c_str()) == lookupTables->IdOf("warning_types", "Block") &&
      IDNO == PSay("You are about to set a block that you do not have permission to unset. Are you sure you want to continue?", m_warning, MB_YESNO))
  {
   return false;
  } else
  if ((initial_state["person_id"] == "") || (initial_state["person_id"] == "0"))
  {
    PSay("There is no person associated with this record", m_missing_data, MB_OK);
    return false;
  }
  else
    return true;
}
//---------------------------------------------------------------------------
bool TformRecordPersonNotes::IsNewAgain()
{
  return true;
}
//---------------------------------------------------------------------------
bool TformRecordPersonNotes::PutDataOnScreen()
{
  // put the values on the form from the map initial_state
  // values may have come from GetDataFromDatabase if it's an existing record,
  // or values may have been passed to the form in this map if it's a new record

  editNote->Text = SentenceString(initial_state["note"]).c_str();
  comboWarning->Text = PrettyString(lookupTables->ValueOf("warning_types", initial_state["warning_id"])).c_str();
  CreateLink(txtPerson, NULL, this, initial_state["person_id"], "people");

  TformRecordGeneric::PutDataOnScreen();
  return true; // what can really go wrong?
}
//---------------------------------------------------------------------------
bool TformRecordPersonNotes::GetDataFromScreen()
{
  // place the values currently on the screen into
  // the map that will be used to update or insert the record
  // the parent form will call the insert or update function, and get the id
  // number if necessary and take care of current_id
  ending_state["note"] = editNote->Text.c_str();
  if (initial_state["group_id"] == "")
    ending_state["group_id"] = "0";
  else
    ending_state["group_id"] = initial_state["group_id"];
  ending_state["person_id"] = initial_state["person_id"];
  ending_state["warning_id"] = lookupTables->IdOf("warning_types", comboWarning->Text.c_str());
  ending_state["id"] = initial_state["id"];

  TformRecordGeneric::GetDataFromScreen();

  return true; // again, not much can go wrong here
}
//---------------------------------------------------------------------------
bool TformRecordPersonNotes::ScreenLock()
{
  // disable panel containing edit boxes
  panelGroup->Enabled = false;
  editNote->ReadOnly = true;
  editNote->TabStop = false;
  TformRecordGeneric::ScreenLock();

  set<string> group_permissions = GetGroupPermissions("table.person_notes", "", "update", "true");
  buttonA->Enabled = group_permissions.find(initial_state["group_id"]) != group_permissions.end();
  return true;
}
//---------------------------------------------------------------------------
bool TformRecordPersonNotes::ScreenUnlock()
{
  // enable panel containing edit boxes
  panelGroup->Enabled = true;
  editNote->ReadOnly = false;
  editNote->TabStop = true;
  TformRecordGeneric::ScreenUnlock();
  return true;
}
//---------------------------------------------------------------------------
void TformRecordPersonNotes::Do(map<string, string> & p_parameters)
{
 FORM_TRY
  TformRecordPersonNotes * z = new TformRecordPersonNotes(NULL, p_parameters);
  z->Show();
 FORM_CATCH
}
//---------------------------------------------------------------------------
void __fastcall TformRecordPersonNotes::FormShow(TObject *Sender)
{
  ShowInfoBar();
}
//---------------------------------------------------------------------------

