//---------------------------------------------------------------------------

#include <vcl.h>
#pragma hdrstop

#include "codeRecordProcedure.h"
#include "codeRecordCondition.h"
#include "codeRecordProcedurePayment.h"
#include "codeSearchCode.h"
//---------------------------------------------------------------------------
#pragma package(smart_init)
#pragma link "codeRecordGeneric"
#pragma link "codeHalfButton"
#pragma resource "*.dfm"
TformRecordProcedure *formRecordProcedure;
//---------------------------------------------------------------------------
__fastcall TformRecordProcedure::TformRecordProcedure(TComponent* Owner, map<string,string> p_parameters)
        : TformRecordGeneric(Owner, p_parameters, string("procedures"))
{
  conditions_filter.SetListViewHandle (listConditions);
  conditions_filter.SetSecurity("conditions");
  conditions_filter.AddColumnSpec("Index", "conditions", as_int, true, 30);
  conditions_filter.AddColumnSpec("Code", "conditions", as_text, true, 95);
  conditions_filter.AddColumnSpec("Note", "conditions", as_text, false, 95);
  conditions_filter.AddColumnSpec("Type", "conditions", as_text, false, 95);
  conditions_filter.AddColumnSpec("From", "conditions", as_date, false, 95);
  conditions_filter.AddColumnSpec("Until", "conditions", as_date, false, 95);
  conditions_filter.AddColumnSpec("Current Id", "current_id", as_int, false, 80);
  conditions_filter.AddColumnSpec("Internal Id", "internal_id", as_int, false, 80);
  conditions_filter.AddColumnSpec("Linking Id", "procedure_reasons", as_int, false, 80);
  conditions_filter.SetIconStatus(false);
  conditions_filter.SetCheck(true);
  conditions_filter.SetCheck(this, CheckRecord(&(TformRecordProcedure::CheckCondition)));
  conditions_filter.SetHeader(false);
  conditions_filter.FetchPreferences("recordprocedure.conditions_filter");
  conditions_filter.SetSQLStatement("");
  conditions_filter.RefreshDataSet();

  payments_filter.SetListViewHandle (listPayments);
  payments_filter.SetSecurity("procedure_payments");
  payments_filter.AddColumnSpec("Bill date", "bills", as_date, true, 60);
  payments_filter.AddColumnSpec("Residual", "procedure_payments", as_double, true, 60, amountString);
  payments_filter.AddColumnSpec("Allowed", "procedure_payments", as_double, true, 60, amountString);
  payments_filter.AddColumnSpec("Paid", "procedure_payments", as_double, true, 60, amountString);
  payments_filter.AddColumnSpec("Current Id", "current_id", as_int, false, 80);
  payments_filter.AddColumnSpec("Internal Id", "internal_id", as_int, false, 80);
  payments_filter.FetchPreferences("recordprocedure.payments_filter");
  payments_filter.SetSQLStatement("");
  payments_filter.RefreshDataSet();

  Initialize();
}
//---------------------------------------------------------------------------
bool TformRecordProcedure::ScreenStart()
{
  TformRecordGeneric::ScreenStart();
  return true;
}
//---------------------------------------------------------------------------
bool TformRecordProcedure::ScreenStop()
{
  // unregister combo boxes
  TformRecordGeneric::ScreenStop();
  return true;
}
//---------------------------------------------------------------------------
bool TformRecordProcedure::IsValidOnScreen()
{
  if (! Pdate (editStartDate->Text.c_str(), mdy | yy).IsValidDate() && ! (editStartDate->Text == ""))
  {
    PSay("Please enter a valid start date", m_invalid_data, MB_OK);
    editStartDate->SetFocus();
    return false;
  }
  if (! Pdate (editStopDate->Text.c_str(), mdy | yy).IsValidDate() && ! (editStopDate->Text == ""))
  {
    PSay("Please enter a valid stop date", m_invalid_data, MB_OK);
    editStopDate->SetFocus();
    return false;
  }
  if (editStartDate->Text == "")
  {
    PSay("You must enter a start date", m_missing_data, MB_OK);
    editStartDate->SetFocus();
    return false;
  }
  if (editCode->Text == "")
  {
    if (IDNO == PSay("You have not provided a CPT/HCPCS procedure code.\n\n"
                     "Procedures without codes will be ignored in IHS and grant reporting, as will visits without any "
                     "procedure codes attached, and will not affect funding.  If this is not your intent, you need "
                     "somebody to generate a code from the procedure description.\n\n"
                     "Are you sure you want to continue?", m_confirmation, MB_YESNO))
    {
      editCode->SetFocus();
      return false;
    }
  }
  if (string(editCode->Text.c_str()).find('.') != string::npos)
  {
    // jhaley 20110921: Code check for strangely common input error...
    if (IDYES == PSay("You have entered what appears to be an ICD Diagnosis code or dollar amount into the Code field. "
                      "Please verify your data entry before continuing. The only value entered into the Code field "
                      "should be a CPT/HCPCS/CVX Procedure code, which do not contain periods.\n\n"
                      "Click Yes to stop saving and re-check your work, or No to continue.", m_confirmation, MB_YESNO))
    {
      editCode->SetFocus();
      return false;
    }
  }
  if ((initial_state["task_id"] == "") || (initial_state["task_id"] == "0"))
  {
    PSay("There is no task associated with this record", m_missing_data, MB_OK);
    return false;
  }
  if (editCode->Text != StripAll(editCode->Text.c_str(), ",; -").c_str())
  {
    PSay("The text you entered into the code field contains invalid characters.\n\n"
         "If you need to enter multiple procedures (code, charge, tooth position), do NOT enter them all on one record "
         "separated by spaces, commas, semicolons, etc.  Instead, enter them as separate records.  You can do this "
         "quickly and easily by utilizing the 'Save and Add Another' button (>>) instead of the Save button.  "
         "Press the >> button for all except the last record; press Save on the last record.\n\n"
         "Also remember that MODIFIER letters go in the Modifier field, NOT in the code field.", m_invalid_data, MB_OK);
    editCode->SetFocus();
    return false;
  }
  if (editCharges->Text != StripAll(editCharges->Text.c_str(), ",; -").c_str())
  {
    PSay("The text you entered into the charges field contains invalid characters.\n\n"
         "If you need to enter multiple procedures (code, charge, tooth position), do NOT enter them all on one record "
         "separated by spaces, commas, semicolons, etc.  Instead, enter them as separate records.  You can do this "
         "quickly and easily by utilizing the 'Save and Add Another' button (>>) instead of the Save button.  "
         "Press the >> button for all except the last record; press Save on the last record.\n\n"
         "Also remember that MODIFIER letters go in the Modifier field, NOT in the code field.", m_invalid_data, MB_OK);
    editToothPosition->SetFocus();
    return false;
  }
  if (editToothPosition->Text != StripAll(editToothPosition->Text.c_str(), ",; -").c_str())
  {
    PSay("The text you entered into the tooth position field contains invalid characters.\n\n"
         "If you need to enter multiple procedures (code, charge, tooth position), do NOT enter them all on one record "
         "separated by spaces, commas, semicolons, etc.  Instead, enter them as separate records.  You can do this "
         "quickly and easily by utilizing the 'Save and Add Another' button (>>) instead of the Save button.  "
         "Press the >> button for all except the last record; press Save on the last record.\n\n"
         "Also remember that MODIFIER letters go in the Modifier field, NOT in the code field.", m_invalid_data, MB_OK);
    editToothPosition->SetFocus();
    return false;
  }
  if (editToothSurface->Text != StripAll(editToothSurface->Text.c_str(), ",; -").c_str())
  {
    PSay("The text you entered into the tooth surface field contains invalid characters.\n\n"
         "Multiple tooth surfaces should be entered without any separators, e.g. MO and not M O or M,O.", m_invalid_data, MB_OK);
    editToothSurface->SetFocus();
    return false;
  }

  return true;
}
//---------------------------------------------------------------------------
bool TformRecordProcedure::NotNewAnymore()
{
  conditions_filter.SetAddEdit(this, AddRecord(&(TformRecordProcedure::AddCondition)),
                               EditRecord(&(TformRecordProcedure::EditCondition)));
  condition_request = atlas->RequestNotificationWhen("conditions.task_id", initial_state["task_id"], rel_e, as_int, this,
                       RefreshHandler(&(TformRecordProcedure::AtlasRefreshListView)));
  procedure_reason_request = atlas->RequestNotificationWhen("procedure_reasons.procedure_id", initial_state["id"], rel_e, as_int, this,
                              RefreshHandler(&(TformRecordProcedure::AtlasRefreshListView)));

  string payments_sql =
    "select bills.bill_date, procedure_payments.amount_charged, procedure_payments.amount_allowed, "
    " procedure_payments.amount_paid, procedure_payments.current_id, procedure_payments.id "
    "from procedure_payments "
    "left join bills on bills.id = procedure_payments.bill_id "
    "where procedure_payments.procedure_id = " + initial_state["id"];

  payments_filter.SetSQLStatement(payments_sql);
  payments_filter.RefreshDataSet();
  payments_filter.SetAddEdit(this, NULL /*AddRecord(&(TformRecordProcedure::AddPayment))*/,
                               EditRecord(&(TformRecordProcedure::EditPayment)));
  payment_request = atlas->RequestNotificationWhen("procedure_payments.procedure_id", initial_state["id"], rel_e, as_int, this,
                     RefreshHandler(&(TformRecordProcedure::AtlasRefreshListView)));

  return TformRecordGeneric::NotNewAnymore();
}
//---------------------------------------------------------------------------
void TformRecordProcedure::DownloadConditions()
{
 /*
  string sql;
  sql = "select conditions.index_number, conditions.code, conditions.current_id, conditions.id "
        "from conditions where task_id = " + initial_state["task_id"];
  conditions_filter.SetSQLStatement(sql);
  conditions_filter.RefreshDataSet();
 */
 string sql;
 if (is_new_record)
 {
  sql = "select conditions.index_number, conditions.code, conditions.note, "
        "condition_types.item, conditions.start_date, conditions.stop_date, "
        "conditions.current_id, conditions.id, NULL "
        "from conditions "
        "left join condition_types on condition_types.id = conditions.condition_id "
        " where conditions.task_id = " + initial_state["task_id"];
 } else
 {
  sql = "select conditions.index_number, conditions.code, conditions.note, "
        "condition_types.item, conditions.start_date, conditions.stop_date, "
        "conditions.current_id, conditions.id, procedure_reasons.id linking_id "
        "from conditions "
        "left join condition_types on condition_types.id = conditions.condition_id "
        "left join procedure_reasons on procedure_reasons.condition_id = conditions.id "
        "and procedure_reasons.procedure_id = " + initial_state["id"] +
        " where conditions.task_id = " + initial_state["task_id"];
 }
 conditions_filter.SetSQLStatement(sql);
 conditions_filter.RefreshDataSet();
}
//---------------------------------------------------------------------------
bool TformRecordProcedure::CheckCondition(const map<string, string> & row_data)
{
 // if there's a link, check it!
 map<string, string>::const_iterator i_link = row_data.find("Linking Id");
 if (i_link != row_data.end() && i_link->second != "")
  return true;
 else
  return false;
}
//---------------------------------------------------------------------------
void TformRecordProcedure::CheckConditions()
{/*
  string sql;
  set<string> id_set;
  sql = "select condition_id from procedure_reasons where procedure_id = " + initial_state["id"];
  SqlToSet(dbDatabase, sql, id_set);
  for(int i = 0; i < listConditions->Items->Count; i++)
  {
    if(id_set.find(listConditions->Items->Item[i]->SubItems->Strings[1].c_str()) != id_set.end())
      listConditions->Items->Item[i]->Checked = true;
    else
      listConditions->Items->Item[i]->Checked = false;
  }
 */
}
//---------------------------------------------------------------------------
bool TformRecordProcedure::PutDataOnScreen()
{
  // put the values on the form from the map initial_state
  // values may have come from GetDataFromDatabase if it's an existing record,
  // or values may have been passed to the form in this map if it's a new record

  DownloadConditions();

  if(optionsPermissions["table.codes"]["view"] != "true")
    btnCodeSearch->Enabled = false;

  editStartDate->Text = initial_state["start_date"].c_str();
  editStopDate->Text = initial_state["stop_date"].c_str();
  editPlace->Text = PrettyString(initial_state["place_of_service"]).c_str();
  editType->Text = PrettyString(initial_state["type_of_service"]).c_str();
  editCode->Text = initial_state["cpt_hcpcs"].c_str();
  editModifier->Text = PrettyString(initial_state["modifier"]).c_str();
  editCharges->Text = AmountToString(StringToAmount(initial_state["charges"])).c_str();
  editDaysUnits->Text = PrettyString(initial_state["days_units"]).c_str();
  editEpsdt->Text = PrettyString(initial_state["epsdt"]).c_str();
  editEmg->Text = PrettyString(initial_state["emg"]).c_str();
  editCob->Text = PrettyString(initial_state["cob"]).c_str();
  editLocalUse->Text = PrettyString(initial_state["local_use"]).c_str();
  editToothPosition->Text = initial_state["tooth_position"].c_str();
  editToothSurface->Text = UppercaseString(initial_state["tooth_surface"]).c_str();

  CreateLink(txtTask, NULL, this, initial_state["task_id"], "tasks");

  TformRecordGeneric::PutDataOnScreen();
  return true;
}
//---------------------------------------------------------------------------
bool TformRecordProcedure::GetDataFromScreen()
{
  // place the values currently on the screen into
  // the map that will be used to update or insert the record
  // the parent form will call the insert or update function, and get the id
  // number if necessary and take care of current_id

  ending_state["id"] = initial_state["id"];
  if(editStartDate->Text != "")
    ending_state["start_date"] = Pdate(editStartDate->Text.c_str(), mdy | yy).OutDate(mdy | yyyy);
  else
    ending_state["start_date"] = "";
  if(editStopDate->Text != "")
    ending_state["stop_date"] = Pdate(editStopDate->Text.c_str(), mdy | yy).OutDate(mdy | yyyy);
  else
    ending_state["stop_date"] = "";
  ending_state["place_of_service"] = editPlace->Text.c_str();
  ending_state["type_of_service"] = editType->Text.c_str();
  ending_state["cpt_hcpcs"] = editCode->Text.c_str();
  ending_state["modifier"] = editModifier->Text.c_str();
  if(editCharges->Text != "")
    ending_state["charges"] = editCharges->Text.c_str();
  else
    ending_state["charges"] = "0";
  ending_state["days_units"] = editDaysUnits->Text.c_str();
  ending_state["epsdt"] = editEpsdt->Text.c_str();
  ending_state["emg"] = editEmg->Text.c_str();
  ending_state["cob"] = editCob->Text.c_str();
  ending_state["local_use"] = editLocalUse->Text.c_str();
  ending_state["tooth_position"] = editToothPosition->Text.c_str();
  ending_state["tooth_surface"] = UppercaseString(editToothSurface->Text.c_str());
  ending_state["task_id"] = initial_state["task_id"];
  if ((editCharges->Text == "") && ((optionsPermissions["special.lac.general"]["enabled"] == "true") ||
      (optionsPermissions["special.den.general"]["enabled"] == "true")))
  {
     string sql = "select codes.clinic_cost from codes "
               "where codes.code = '" + SafeSQLString(editCode->Text.c_str()) + "'";
     string result = GetOneField(dbDatabase, sql);
     if (result == "")
        ending_state["charges"] = "0";
     else
        ending_state["charges"] = result;
  }
  TformRecordGeneric::GetDataFromScreen();

  return true;
}
//---------------------------------------------------------------------------
bool TformRecordProcedure::ScreenLock()
{
  // disable group boxes
  panelGroup->Enabled = false;
  panelTooth->Enabled = false;
  conditions_filter.SetCheckLocked(true);
  DownloadConditions();
  TformRecordGeneric::ScreenLock();
  return true;
}
//---------------------------------------------------------------------------
bool TformRecordProcedure::ScreenUnlock()
{
  // enable group boxes
  panelGroup->Enabled = true;
  // TODO: note: contract health received dental bills occasionally, no? wouldn't they also want to modify this if they enter it?
  if ((optionsPermissions["special.mis.general"]["enabled"] == "true") ||
      (optionsPermissions["special.den.general"]["enabled"] == "true"))
     panelTooth->Enabled = true;
  conditions_filter.SetCheckLocked(false);
  DownloadConditions();

  TformRecordGeneric::ScreenUnlock();

  editCode->SetFocus();
  return true;
}
//---------------------------------------------------------------------------
// DONE: this should be in BeforeSave(), using lock_transaction!
// DONE: (bad idea) (alt) we could instead modify records as users click on the [x] in the list?
bool TformRecordProcedure::BeforeSave()
{
  vector<string> id_vec;
  try
  {
    ExecuteStatement(lock_transaction, "delete from procedure_reasons where procedure_id = " + ending_state["id"]);
    map<string, string> to_insert;
    id_vec = conditions_filter.GetMultiID();
    for(vector<string>::iterator itr = id_vec.begin(); itr != id_vec.end(); itr++)
    {
      to_insert["id"] = GetNextId(lock_transaction, "procedure_reasons");
      to_insert["last_updated_by"] = current_userid;
      to_insert["last_update_stamp"] = RightNow();
      to_insert["current_id"] = "4";
      to_insert["condition_id"] = (*itr);
      to_insert["procedure_id"] = ending_state["id"];
      ExecuteInsertStatement(lock_transaction, "procedure_reasons", to_insert);
    }
  }
  catch(...)
  {
   PSay("Failed to save changes to procedure justification (procedure-to-condition relationship)", m_program_error, MB_OK);
   return false;
  }

  return TformRecordGeneric::BeforeSave();
}
//---------------------------------------------------------------------------
bool TformRecordProcedure::IsNewAgain()
{
  // put data on screen is actually taking care of this on this form since it
  // has to check the appropriate ones
  conditions_filter.SetSQLStatement(string(""));
  conditions_filter.RefreshDataSet();
  conditions_filter.SetAddEdit(this, NULL, NULL);

  payments_filter.SetSQLStatement(string(""));
  payments_filter.RefreshDataSet();
  payments_filter.SetAddEdit(this, NULL, NULL);

  TformRecordGeneric::IsNewAgain();
  return true;
}
//---------------------------------------------------------------------------
void TformRecordProcedure::AtlasRefreshListView(int request_num, string matched_value)
{
 if (request_num == condition_request || request_num == procedure_reason_request)
  DownloadConditions();

 if (request_num == payment_request)
  payments_filter.RefreshDataSet();
}
//---------------------------------------------------------------------------
void TformRecordProcedure::AddCondition()
{
  if((initial_state["task_id"] != "") && (initial_state["task_id"] != "0"))
  {
    TIBTransaction *dbTransaction = new TIBTransaction(NULL);
    try
    {
      dbTransaction->DefaultDatabase = dbDatabase;
      dbTransaction->StartTransaction();
      map<string,string> temp_map;
      temp_map["id"] = "new";
      temp_map["task_id"] = initial_state["task_id"];
      // TODO: do we see how many selects there are here, for no good reason?
      temp_map["person_id"] = GetOneField(dbTransaction, "select person_id from tasks where id = " + initial_state["task_id"]);
      temp_map["start_date"] = GetOneField(dbTransaction, "select date_of_task from tasks where id = " + initial_state["task_id"]);
      temp_map["group_id"] = GetOneField(dbTransaction, "select group_id from tasks where id = " + initial_state["task_id"]);
      dbTransaction->Commit();
      FORM_TRY
        TformRecordCondition * conditionRecord = new TformRecordCondition(NULL, temp_map);
        conditionRecord->Show();
      FORM_CATCH
    }
    catch(...)
    {
      if(dbTransaction->Active)
        dbTransaction->Rollback();
      PSay("Could not create a new condition", m_database_error, MB_OK);
    }
    delete dbTransaction;
  }
}
//---------------------------------------------------------------------------
void TformRecordProcedure::EditCondition()
{
  map<string,string> temp_map;
  temp_map["id"] = conditions_filter.GetSelectedRecordID();
  FORM_TRY
    TformRecordCondition * conditionRecord = new TformRecordCondition(NULL, temp_map);
    conditionRecord->Show();
  FORM_CATCH
}
//---------------------------------------------------------------------------
void TformRecordProcedure::AddPayment()
{
  map<string,string> temp_map;
  temp_map["id"] = "new";
  temp_map["procedure_id"] = initial_state["id"];
  FORM_TRY
    TformRecordProcedurePayment * pay = new TformRecordProcedurePayment(NULL, temp_map);
    pay->Show();
  FORM_CATCH
}
//---------------------------------------------------------------------------
void TformRecordProcedure::EditPayment()
{
  map<string,string> temp_map;
  temp_map["id"] = payments_filter.GetSelectedRecordID();
  FORM_TRY
    TformRecordProcedurePayment * pay = new TformRecordProcedurePayment(NULL, temp_map);
    pay->Show();
  FORM_CATCH
}
//---------------------------------------------------------------------------
void __fastcall TformRecordProcedure::listConditionsChanging(
      TObject *Sender, TListItem *Item, TItemChange Change,
      bool &AllowChange)
{
 if (Change == ctState)
  AllowChange = (is_editing || is_new_record || conditions_filter.IsRefreshingListview());
 else
  AllowChange = true;
}
//---------------------------------------------------------------------------
void __fastcall TformRecordProcedure::buttonExpandCodeClick(
      TObject *Sender)
{
  FORM_TRY
    TformSearchCode * codeSearch = new TformSearchCode(NULL, NULL);
    codeSearch->cboCode->Text = "";
    codeSearch->editCode->Text = editCode->Text;
    codeSearch->Show();
    if (editCode->Text != "")
     codeSearch->buttonSearch->Click();
  FORM_CATCH
}
//---------------------------------------------------------------------------
void TformRecordProcedure::Do(map<string, string> & p_parameters)
{
 FORM_TRY
  TformRecordProcedure * z = new TformRecordProcedure(NULL, p_parameters);
  z->Show();
 FORM_CATCH
}
//---------------------------------------------------------------------------
void __fastcall TformRecordProcedure::btnCodeSearchClick(TObject *Sender)
{
  vector<string> return_vector;
  string sql;
  FORM_TRY
    TformSearchCode * codeSearch = new TformSearchCode(NULL, &return_vector);
    codeSearch->cboCode->Text = "CPT-4";
    codeSearch->ShowTaskModal(this);
  FORM_CATCH
  if (! return_vector.empty())
  {
    sql = "select code from codes where id = " + return_vector[0];
    editCode->Text = GetOneField(dbDatabase, sql).c_str();
    /*
    if (editCode->Text != "")
    {
      string new_note;
      string entered_note = UppercaseString(editLocalUse->Text.c_str());
      string sql = "select description from codes where code = '";
      sql += editCode->Text.c_str();
      sql += "' and code_id = " + lookupTables->IdOf("code_types", "ICD-9");
      string code_note = GetOneField(dbDatabase, sql);
      if (entered_note == "")
        new_note = code_note;
      else
      {
        if(entered_note.find(code_note) == string::npos)
          new_note = entered_note + "\n\n" + code_note;
        else
          new_note = entered_note;
      }
      editLocalUse->Text = SentenceString(new_note).c_str();
    }
    */
  }
}
//---------------------------------------------------------------------------

