//---------------------------------------------------------------------------
#include <vcl.h>
#pragma hdrstop

#include "codeRecordReferral.h"
#include "codeSearchTaxEntity.h"
#include "codeSearchInsurancePolicy.h"
#include "codeRecordReferralNote.h"
#include "derivedheaders.h"
//---------------------------------------------------------------------------
#pragma package(smart_init)
#pragma link "codeRecordGeneric"
#pragma link "codeRubberContainer"
#pragma resource "*.dfm"
TformRecordReferral *formRecordReferral;
//---------------------------------------------------------------------------
__fastcall TformRecordReferral::TformRecordReferral(TComponent* Owner, map<string,string> & p_parameters)
        : TformRecordGeneric(Owner, p_parameters, string("referrals"))
{
  notes_filter.SetListViewHandle (listNotes);
  notes_filter.SetSecurity("referral_notes");
  notes_filter.AddColumnSpec("Date", "referral_notes", as_date, true, 100);
  notes_filter.AddColumnSpec("Note", "referral_notes", as_text, true, 400, sentenceString);
  notes_filter.AddColumnSpec("Current Id", "current_id", as_int, false, 80);
  notes_filter.AddColumnSpec("Internal Id", "internal_id", as_int, false, 80);
  notes_filter.FetchPreferences("recordreferral.notes_filter");
  notes_filter.SetIconStatus(true);
  notes_filter.SetSQLStatement ("");
  notes_filter.RefreshDataSet();

  Initialize();
}
//---------------------------------------------------------------------------
bool TformRecordReferral::ScreenStart()
{
//  editNotes->Text = "";

  lookupTables->RegisterComboBox("todo_status_types", cboStatus);
  string group_sql = "select id, item, valid from referral_reason_types where group_id in ("+SetToCommaString(GetSpecialGroupPermissions("general"))+") and valid = 1 order by item";
  lookupTables->RegisterComboBox("referral_reason_types", cboReason, group_sql);
  TformRecordGeneric::ScreenStart();
  return true;
}
//---------------------------------------------------------------------------
bool TformRecordReferral::ScreenStop()
{
  // unregister combo boxes
  lookupTables->UnregisterComboBox("todo_status_types", cboStatus);  
  lookupTables->UnregisterComboBox("referral_reason_types", cboReason);

  TformRecordGeneric::ScreenStop();
  return true;
}
//---------------------------------------------------------------------------
bool TformRecordReferral::IsValidOnScreen()
{
  // check to make sure that the values entered in the combo boxe
  // corrospond to values in that look up table.
  // give the user a message box and prompt to re-enter if incorrect
  if (GetValidLinkId(txtByProvider) == "" && GetValidLinkId(txtByFacility) == "")
  {
    PSay("Please enter either a facility or provider the client was referred by", m_missing_data, MB_OK);
    return false;
  }
  else if (GetValidLinkId(txtToProvider) == "" && GetValidLinkId(txtToFacility) == "")
  {
    PSay("Please enter either a facility or provider the client was referred to", m_missing_data, MB_OK);
    return false;
  }
  else if (! Pdate (editDate->Text.c_str(), mdy | yy).IsValidDate() && ! (editDate->Text == ""))
  {
    PSay("Please enter a valid date", m_invalid_data, MB_OK);
    editDate->SetFocus();
    return false;
  }
  else if(! lookupTables->IsInLookUpTable("todo_status_types", cboStatus->Text.c_str()))
  {
    PSay("Please enter a valid status", m_invalid_data, MB_OK);
    cboStatus->SetFocus();
    return false;
  }
  else if ((optionsPermissions["special.beh.general"]["enabled"] == "true") &&
           (! lookupTables->IsInLookUpTable("referral_reason_types", cboReason->Text.c_str())))
  {
    PSay("Please enter a valid reason", m_invalid_data, MB_OK);
    cboReason->SetFocus();
    return false;
  }
  else if ((initial_state["person_id"] == "") || (initial_state["person_id"] == "0"))
  {
    PSay("Please attach a client to this referral", m_missing_data, MB_OK);
    return false;
  }
  else
    return true;
}
//---------------------------------------------------------------------------
bool TformRecordReferral::PutDataOnScreen()
{
  // put the values on the form from the map initial_state
  // values may have come from GetDataFromDatabase if it's an existing record,
  // or values may have been passed to the form in this map if it's a new record

  editDate->Text = initial_state["date_of_referral"].c_str();
//  editNotes->Text = SentenceString(initial_state["notes"]).c_str();
  cboStatus->Text = lookupTables->ValueOf("todo_status_types", initial_state["todo_status_id"]).c_str();
  //if (optionsPermissions["special.beh.general"]["enabled"] == "true")
  cboReason->Text = PrettyString(lookupTables->ValueOf("referral_reason_types", initial_state["referral_reason_id"])).c_str();
  //else
  //  cboReason->Text = "";

  CreateLink(txtPerson, NULL, this, initial_state["person_id"], "people");
  CreateLink(txtTask, NULL, this, initial_state["task_id"], "tasks");
  CreateLink(txtByFacility, btnByFacilitySearch, this, initial_state["by_facility_id"], "tax_entities");
  CreateLink(txtByProvider, btnByProviderSearch, this, initial_state["by_provider_id"], "tax_entities");
  CreateLink(txtToFacility, btnToFacilitySearch, this, initial_state["to_facility_id"], "tax_entities");
  CreateLink(txtToProvider, btnToProviderSearch, this, initial_state["to_provider_id"], "tax_entities");
  CreateLink(txtPolicy, btnPolicySearch, this, initial_state["insurance_policy_id"], "insurance_policies");

  TformRecordGeneric::PutDataOnScreen();
  return true;
}
//---------------------------------------------------------------------------
bool TformRecordReferral::GetDataFromScreen()
{
  // place the values currently on the screen into
  // the map that will be used to update or insert the record
  // the parent form will call the insert or update function, and get the id
  // number if necessary and take care of current_id

  ending_state["id"] = initial_state["id"];
  ending_state["person_id"] = initial_state["person_id"];

  string by_provider_id = GetValidLinkId(txtByProvider);
  string by_facility_id = GetValidLinkId(txtByFacility);
  string to_provider_id = GetValidLinkId(txtToProvider);
  string to_facility_id = GetValidLinkId(txtToFacility);
  string policy_id = GetValidLinkId(txtPolicy);

  if (by_provider_id == "")
    ending_state["by_provider_id"] = "0";
  else
    ending_state["by_provider_id"] = by_provider_id;
  if (by_facility_id == "")
    ending_state["by_facility_id"] = "0";
  else
    ending_state["by_facility_id"] = by_facility_id;
  if (to_provider_id == "")
    ending_state["to_provider_id"] = "0";
  else
    ending_state["to_provider_id"] = to_provider_id;
  if (to_facility_id == "")
    ending_state["to_facility_id"] = "0";
  else
    ending_state["to_facility_id"] = to_facility_id;

//  ending_state["notes"] = editNotes->Text.c_str();
  if (editDate->Text != "")
    ending_state["date_of_referral"] =  Pdate (editDate->Text.c_str(), mdy | yy).OutDate(mdy | yyyy);
  else                                        
    ending_state["date_of_referral"] = "";
  ending_state["todo_status_id"] = lookupTables->IdOf("todo_status_types", cboStatus->Text.c_str());
  ending_state["referral_reason_id"] = lookupTables->IdOf("referral_reason_types", cboReason->Text.c_str());
  if (initial_state["task_id"] == "")
    ending_state["task_id"] = "0";
  else
    ending_state["task_id"] = initial_state["task_id"];
  TformRecordGeneric::GetDataFromScreen();

  return true;
}
//---------------------------------------------------------------------------
bool TformRecordReferral::ScreenLock()
{
  panelGroup->Enabled = false;
  TformRecordGeneric::ScreenLock();
  return true;
}
//---------------------------------------------------------------------------
bool TformRecordReferral::ScreenUnlock()
{
  panelGroup->Enabled = true;
  TformRecordGeneric::ScreenUnlock();
  return true;
}
//---------------------------------------------------------------------------
void __fastcall TformRecordReferral::btnToFacilitySearchClick(TObject *Sender)
{
  // create and show new person search form
  vector<string> return_vector;
  FORM_TRY
    TformSearchTaxEntity * facilitySearch = new TformSearchTaxEntity(NULL,&return_vector);
    facilitySearch->ShowTaskModal(this);
  FORM_CATCH
  if (! return_vector.empty())
    CreateLink(txtToFacility, btnToFacilitySearch, this, return_vector[0], "tax_entities");
}
//---------------------------------------------------------------------------
void __fastcall TformRecordReferral::btnToProviderSearchClick(TObject *Sender)
{
  // create and show new person search form
  vector<string> return_vector;
  FORM_TRY
    TformSearchTaxEntity * providerSearch = new TformSearchTaxEntity(NULL,&return_vector);
    providerSearch->ShowTaskModal(this);
  FORM_CATCH
  if (! return_vector.empty())
    CreateLink(txtToProvider, btnToProviderSearch, this, return_vector[0], "tax_entities");
}
//---------------------------------------------------------------------------
void __fastcall TformRecordReferral::btnByProviderSearchClick(TObject *Sender)
{
  // create and show new person search form
  vector<string> return_vector;
  FORM_TRY
    TformSearchTaxEntity * providerSearch = new TformSearchTaxEntity(NULL,&return_vector);
    providerSearch->ShowTaskModal(this);
  FORM_CATCH
  if (! return_vector.empty())
    CreateLink(txtByProvider, btnByProviderSearch, this, return_vector[0], "tax_entities");
}
//---------------------------------------------------------------------------
void __fastcall TformRecordReferral::btnByFacilitySearchClick(TObject *Sender)
{
  // create and show new person search form
  vector<string> return_vector;
  FORM_TRY
    TformSearchTaxEntity * facilitySearch = new TformSearchTaxEntity(NULL,&return_vector);
    facilitySearch->ShowTaskModal(this);
  FORM_CATCH
  if (! return_vector.empty())
    CreateLink(txtByFacility, btnByFacilitySearch, this, return_vector[0], "tax_entities");
}
//---------------------------------------------------------------------------
void __fastcall TformRecordReferral::btnPolicySearchClick(TObject *Sender)
{
  vector<string> return_vector;
  FORM_TRY
    TformSearchInsurancePolicy * policySearch = new TformSearchInsurancePolicy(NULL,&return_vector);
    policySearch->ShowTaskModal(this);
  FORM_CATCH
  if (! return_vector.empty())
    CreateLink(txtPolicy, btnPolicySearch, this, return_vector[0], "insurance_policies");
}
//---------------------------------------------------------------------------
bool TformRecordReferral::CheckForUselessRecord()
{
  return true;
}
//---------------------------------------------------------------------------
bool TformRecordReferral::IsNewAgain()
{
 notes_filter.SetSQLStatement(string(""));
 notes_filter.RefreshDataSet();
 notes_filter.SetAddEdit(this, NULL, NULL);
 TformRecordGeneric::IsNewAgain();
 return true;
}
//---------------------------------------------------------------------------
bool TformRecordReferral::NotNewAnymore()
{
  string sql;
    sql = "select referral_notes.date_of_note, referral_notes.note, "
          "referral_notes.current_id, referral_notes.id "
          "from referral_notes "
          "where referral_notes.referral_id = " + initial_state["id"];

  notes_filter.SetSQLStatement(sql);
  notes_filter.RefreshDataSet();
  notes_filter.SetAddEdit(this, AddRecord(&(TformRecordReferral::AddNote)),
                                        EditRecord(&(TformRecordReferral::EditNote)));
  notes_request = atlas->RequestNotificationWhen("referral_notes.referral_id", initial_state["id"], rel_e, as_int, this,
                               RefreshHandler(&(TformRecordReferral::AtlasRefreshListViews)));
  TformRecordGeneric::NotNewAnymore();
  return true;
}
//---------------------------------------------------------------------------
bool TformRecordReferral::AtlasRefreshListViews(int request_num, string matched_value)
{
  if (request_num == notes_request)
    notes_filter.RefreshDataSet();

  return true;
}
//---------------------------------------------------------------------------
void TformRecordReferral::AddNote()
{
  map<string, string> temp_map;
  temp_map["id"] = "new";
  temp_map["referral_id"] = initial_state["id"];
  FORM_TRY
    TformRecordReferralNote * note = new TformRecordReferralNote(NULL, temp_map);
    note->Show();
  FORM_CATCH
}
//---------------------------------------------------------------------------
void TformRecordReferral::EditNote()
{
  map<string, string> temp_map;
  temp_map["id"] = notes_filter.GetSelectedRecordID();
  FORM_TRY
    TformRecordReferralNote * note = new TformRecordReferralNote(NULL, temp_map);
    note->Show();
  FORM_CATCH
}
//---------------------------------------------------------------------------
bool TformRecordReferral::SendAtlas()
{
 if (ending_state["to_provider_id"] != user_taxid)
  atlas->Notify("hack.referrals.to_provider_id", ending_state["to_provider_id"]);
 // TODO: this is why I wish I had made atlas capable of just handing you a whole batch up messages that went "together",
 // and let you go through the whole thing (raw) if you so please to see if it interests you rather than resort to this hack.
 // it's all fine and good having it tell you something you cared about happened, but sometimes you want more information.
 // and in those cases, it should be layered to let you get to the source of stuff if you need it, and not care otherwise.
 return TformRecordGeneric::SendAtlas();
}
//---------------------------------------------------------------------------
void TformRecordReferral::Do(map<string, string> & p_parameters)
{
 FORM_TRY
  TformRecordReferral * z = new TformRecordReferral(NULL, p_parameters);
  z->Show();
 FORM_CATCH
}
//---------------------------------------------------------------------------

