//---------------------------------------------------------------------------

#include <vcl.h>
#pragma hdrstop

#include "codeSpecialHouseholdMod.h"
#include "codeRecordRelationship.h"
#include "codeSearchPerson.h"
//---------------------------------------------------------------------------
#pragma package(smart_init)
#pragma link "codeRecordGeneric"
#pragma resource "*.dfm"
TformRecordRelationship *formRecordRelationship;
//---------------------------------------------------------------------------
__fastcall TformRecordRelationship::TformRecordRelationship(TComponent* Owner, map<string,string> p_parameters)
        : TformRecordGeneric(Owner, p_parameters, string("relationships"))
{
 Initialize();
}
//---------------------------------------------------------------------------
void __fastcall TformRecordRelationship::btnPersonBSearchClick(
      TObject *Sender)
{
 // create and show new person search form
 vector<string> return_vector;
 FORM_TRY
   TformSearchPerson * formPersonSearch = new TformSearchPerson(NULL,&return_vector);
   formPersonSearch->ShowTaskModal(this);
 FORM_CATCH
 if (! return_vector.empty())
 {
   CreateLink(labelNameB, btnPersonBSearch, this, return_vector[0], "people");
   if(labelNameB->Caption == "None")
     labelNameB->Caption = "unknown person (B)";
   SetupNamedLabels();
 }
}
//---------------------------------------------------------------------------
bool TformRecordRelationship::IsValidOnScreen()
{
 if (!lookupTables->IsInLookUpTable("relationship_types", comboAtoB))
 {
  PSay("Please enter a valid relationship type (first drop-down)", m_invalid_data, MB_OK);
  comboAtoB->SetFocus();
  return false;
 }
 else if (!lookupTables->IsInLookUpTable("relationship_types", comboBtoA))
 {
  PSay("Please enter a valid relationship type (second drop-down)", m_invalid_data, MB_OK);
  comboBtoA->SetFocus();
  return false;
 }
 else if (!(StringToInt(GetValidLinkId(labelNameB)) > 0))
 {
  PSay("Please enter two people to be related to each other", m_missing_data, MB_OK);
  return false;
 }
 else
  return true;
}
//---------------------------------------------------------------------------
bool TformRecordRelationship::PutDataOnScreen()
{
 comboAtoB->Text = PrettyString(lookupTables->ValueOf("relationship_types", initial_state["rel_a_to_b_id"])).c_str();
 comboBtoA->Text = PrettyString(lookupTables->ValueOf("relationship_types", initial_state["rel_b_to_a_id"])).c_str();

 checkA_is_emergency_for_B->Checked      = (initial_state["a_is_emergency"] == "1");
 checkB_is_emergency_for_A->Checked      = (initial_state["b_is_emergency"] == "1");
 checkA_is_responsible_for_B->Checked    = (initial_state["a_is_responsible"] == "1");
 checkB_is_responsible_for_A->Checked    = (initial_state["b_is_responsible"] == "1");
 checkA_is_banned_from_B->Checked        = (initial_state["a_is_banned"] == "1");
 checkB_is_banned_from_A->Checked        = (initial_state["b_is_banned"] == "1");
 checkA_and_B_in_same_household->Checked = (initial_state["same_household"] == "1");

 CreateLink(labelNameA, NULL, this, initial_state["person_a_id"], "people");
 if(labelNameA->Caption == "None")
   labelNameA->Caption = "unknown person (A)";
 CreateLink(labelNameB, btnPersonBSearch, this, initial_state["person_b_id"], "people");
 if(labelNameB->Caption == "None")
   labelNameB->Caption = "unknown person (B)";

 SetupNamedLabels();

 TformRecordGeneric::PutDataOnScreen();
 return true;
}
//---------------------------------------------------------------------------
bool TformRecordRelationship::GetDataFromScreen()
{
 ending_state["id"] = initial_state["id"];

 string person_a_id = GetValidLinkId(labelNameA);
 if(StringToInt(person_a_id) > 0)
  ending_state["person_a_id"] = person_a_id;
 else
  ending_state["person_a_id"] = "0";

 string person_b_id = GetValidLinkId(labelNameB);
 if(StringToInt(person_b_id) > 0)
  ending_state["person_b_id"] = person_b_id;
 else
  ending_state["person_b_id"] = "0";

 ending_state["rel_a_to_b_id"]    = lookupTables->IdOf("relationship_types", comboAtoB);
 ending_state["rel_b_to_a_id"]    = lookupTables->IdOf("relationship_types", comboBtoA);
 ending_state["a_is_emergency"]   = (checkA_is_emergency_for_B->Checked ? "1" : "0");
 ending_state["b_is_emergency"]   = (checkB_is_emergency_for_A->Checked ? "1" : "0");
 ending_state["a_is_responsible"] = (checkA_is_responsible_for_B->Checked ? "1" : "0");
 ending_state["b_is_responsible"] = (checkB_is_responsible_for_A->Checked ? "1" : "0");
 ending_state["a_is_banned"]      = (checkA_is_banned_from_B->Checked ? "1" : "0");
 ending_state["b_is_banned"]      = (checkB_is_banned_from_A->Checked ? "1" : "0");
 ending_state["same_household"]   = (checkA_and_B_in_same_household->Checked ? "1" : "0");
 TformRecordGeneric::GetDataFromScreen();
 return true;
}
//---------------------------------------------------------------------------
bool TformRecordRelationship::ScreenStart()
{
 // register two combo boxes with the same tables
 string rel_sql = "select id, item, valid from relationship_types where is_self != 1 and valid = 1 order by item";
 lookupTables->RegisterComboBox("relationship_types", comboAtoB, rel_sql);
 lookupTables->RegisterComboBox("relationship_types", comboBtoA, rel_sql);

 TformRecordGeneric::ScreenStart();
 return true;
}
//---------------------------------------------------------------------------
bool TformRecordRelationship::ScreenStop()
{
 // unregister combo boxes
 lookupTables->UnregisterComboBox("relationship_types", comboAtoB);
 lookupTables->UnregisterComboBox("relationship_types", comboBtoA);

 TformRecordGeneric::ScreenStop();
 return true;
}
//---------------------------------------------------------------------------
bool TformRecordRelationship::ScreenLock()
{
 // disable group boxes
 panelGroup->Enabled = false;
 TformRecordGeneric::ScreenLock();
 return true;
}
//---------------------------------------------------------------------------
bool TformRecordRelationship::ScreenUnlock()
{
 // enable group boxes
 panelGroup->Enabled = true;
 TformRecordGeneric::ScreenUnlock();
 return true;
}
//---------------------------------------------------------------------------
void TformRecordRelationship::SetupNamedLabels()
{
 string person_a_name = labelNameA->Caption.c_str();
 string person_b_name = labelNameB->Caption.c_str();
 string person_a_id = GetValidLinkId(labelNameA);
 if (person_a_id != "")
   person_a_name = GetPersonName(dbDatabase, person_a_id, first_last);
 string person_b_id = GetValidLinkId(labelNameB);
 if (person_b_id != "")
   person_b_name = GetPersonName(dbDatabase, person_b_id, first_last);
 // setup all the text fields with names
 labelRelationshipOfAtoB->Caption = string(person_a_name + " is " + person_b_name + "'s").c_str();
 labelRelationshipOfBtoA->Caption = string(person_b_name + " is " + person_a_name + "'s").c_str();
 labelAChecks->Caption = string(person_a_name + " is...").c_str();
 labelBChecks->Caption = string(person_b_name + " is...").c_str();
 checkA_is_emergency_for_B->Caption = string("the emergency contact for " + person_b_name).c_str();
 checkB_is_emergency_for_A->Caption = string("the emergency contact for " + person_a_name).c_str();
 checkA_is_responsible_for_B->Caption = string("responsible for " + person_b_name).c_str();
 checkB_is_responsible_for_A->Caption = string("responsible for " + person_a_name).c_str();
 checkA_is_banned_from_B->Caption = string("banned from information regarding " + person_b_name).c_str();
 checkB_is_banned_from_A->Caption = string("banned from information regarding " + person_a_name).c_str();
 checkA_and_B_in_same_household->Caption = string(person_a_name + " and " + person_b_name + " are in the same household").c_str();
 return;
}
//---------------------------------------------------------------------------
bool TformRecordRelationship::BeforeSave()
{
 // okay, -only- if initial_state["same_household"] != ending_state["same_household"]
 // we have to figure out who was in the group with this person, so we have to:
 // figure out this peron's household number,
 // get a list of all relationships for people currently in that household where
 // same_household is true,
 // then go through and put people's id's into a growing list of sets of people
 // that are all in the same household together.
 // then assign id's to those, and update all those people to be in their correct
 // households.
 // will take a while to run the first time, when everone is in household zero...
 // and it has to figure out that they're all in individual households.
 // ah, but what about people who are currently being updated? we can't change their
 // info until everyone else is out of the way... which means waiting.
 // we could build this info when they ask for it, ie. who's in this household?
 // but then, what about who's in the family, etc? graphs?
 //
 // TODO: leaving this here for now, remove this for speed (minimal!)
 //
 // also, what's the proper way to do a smilie =) at the end of a paren block () ?

 set<string> people_involved;
 people_involved.insert(ending_state["person_a_id"]);
 people_involved.insert(ending_state["person_b_id"]);
 if (TformSpecialHouseholdMod::FindHousehold(lock_transaction, people_involved))
 {
  if (TformSpecialHouseholdMod::ResolveHouseholdInformation(lock_transaction, people_involved))
  {
   // PSay("Finished fixing household information", m_message, MB_OK);
   return true;
  } else
  {
   // error fixing stuff that may have broken
   // PSay("Error fixing household information", m_message, MB_OK); // they already get another error message
   return false;
  }
 } else
 {
  // error finding the rest of the household(s) based on relationships
  PSay("Error fetching household information", m_database_error, MB_OK);
  return false;
 }
}
//---------------------------------------------------------------------------
void TformRecordRelationship::Do(map<string, string> & p_parameters)
{
 FORM_TRY
  TformRecordRelationship * z = new TformRecordRelationship(NULL, p_parameters);
  z->Show();
 FORM_CATCH
}
//---------------------------------------------------------------------------

