//---------------------------------------------------------------------------

#include <vcl.h>
#pragma hdrstop

#include "codeRecordSurvey.h"
#include "codeRecordSurveyQuestion.h"
#include "codeRecordSurveyTaken.h"
//---------------------------------------------------------------------------
#pragma package(smart_init)
#pragma link "codeRecordGeneric"
#pragma resource "*.dfm"
TformRecordSurvey *formRecordSurvey;
//---------------------------------------------------------------------------
__fastcall TformRecordSurvey::TformRecordSurvey(TComponent* Owner, map<string, string> & p_parameters)
        : TformRecordGeneric(Owner, p_parameters, string("surveys"))
{
  question_filter.SetListViewHandle (listQuestions);
  question_filter.SetSecurity("survey_questions");
  question_filter.AddColumnSpec("Number", "survey_questions", as_int, true, 80);
  question_filter.AddColumnSpec("Question", "survey_questions", as_text, true, 400, sentenceString);
  question_filter.AddColumnSpec("Is Numeric", "survey_questions", as_text, true, 90, prettyBoolString);
  question_filter.AddColumnSpec("Min Value", "survey_questions", as_double, true, 90);
  question_filter.AddColumnSpec("Max Value", "survey_questions", as_double, true, 90);
  question_filter.AddColumnSpec("Current Id", "current_id", as_int, false, 80);
  question_filter.AddColumnSpec("Internal Id", "internal_id", as_int, false, 80);
  question_filter.FetchPreferences("recordsurvey.question_filter");
  question_filter.SetIconStatus(true);
  question_filter.SetSQLStatement ("");
  question_filter.RefreshDataSet();

  taken_filter.SetListViewHandle (listTaken);
  taken_filter.SetSecurity("surveys_taken");
  taken_filter.AddColumnSpec("Date", "surveys_taken", as_date, true, 90);
  taken_filter.AddColumnSpec("Person", "surveys_taken", as_text, true, 120);
  taken_filter.AddColumnSpec("Current Id", "current_id", as_int, false, 80);
  taken_filter.AddColumnSpec("Internal Id", "internal_id", as_int, false, 80);
  taken_filter.FetchPreferences("recordsurvey.taken_filter");
  taken_filter.SetIconStatus(true);
  taken_filter.SetSQLStatement ("");
  taken_filter.RefreshDataSet();

  Initialize();
}
//---------------------------------------------------------------------------
bool TformRecordSurvey::ScreenStart()
{
 TformRecordGeneric::ScreenStart();
 return true;
}
//---------------------------------------------------------------------------
bool TformRecordSurvey::ScreenStop()
{
 TformRecordGeneric::ScreenStop();
 return true;
}
//---------------------------------------------------------------------------
bool TformRecordSurvey::IsValidOnScreen()
{
  return true;
}
//---------------------------------------------------------------------------
bool TformRecordSurvey::PutDataOnScreen()
{
 editName->Text = PrettyString(initial_state["survey_name"]).c_str();

 TformRecordGeneric::PutDataOnScreen();
 return true;
}
//---------------------------------------------------------------------------
bool TformRecordSurvey::GetDataFromScreen()
{
 ending_state["survey_name"] = editName->Text.c_str();
 if(initial_state["group_id"] == "")
   ending_state["group_id"] = current_groupid;
 else
   ending_state["group_id"] = initial_state["group_id"];
 ending_state["id"] = initial_state["id"];

 TformRecordGeneric::GetDataFromScreen();
 return true;
}
//---------------------------------------------------------------------------
bool TformRecordSurvey::ScreenLock()
{
 panelGroup->Enabled = false;
 TformRecordGeneric::ScreenLock();
 return true;
}
//---------------------------------------------------------------------------
bool TformRecordSurvey::ScreenUnlock()
{
 panelGroup->Enabled = true;
 TformRecordGeneric::ScreenUnlock();
 return true;
}
//---------------------------------------------------------------------------
bool TformRecordSurvey::IsNewAgain()
{
 question_filter.SetSQLStatement(string(""));
 question_filter.RefreshDataSet();
 question_filter.SetAddEdit(this, NULL, NULL);
 taken_filter.SetSQLStatement(string(""));
 taken_filter.RefreshDataSet();
 taken_filter.SetAddEdit(this, NULL, NULL);
 TformRecordGeneric::IsNewAgain();
 return true;
}
//---------------------------------------------------------------------------
bool TformRecordSurvey::NotNewAnymore()
{
  string sql;
  sql = "select survey_questions.question_number, survey_questions.question, survey_questions.is_numeric, "
        "survey_questions.min_value, survey_questions.max_value, survey_questions.current_id, survey_questions.id "
        "from survey_questions where survey_questions.survey_id = " + initial_state["id"];

  question_filter.SetSQLStatement(sql);
  question_filter.RefreshDataSet();
  question_filter.SetAddEdit(this, AddRecord(&(TformRecordSurvey::AddQuestion)),
                                        EditRecord(&(TformRecordSurvey::EditQuestion)));
  question_request = atlas->RequestNotificationWhen("survey_questions.survey_id", initial_state["id"], rel_e, as_int, this,
                     RefreshHandler(&(TformRecordSurvey::AtlasRefreshListViews)));

  sql = "select surveys_taken.date_taken, "
        "person_names.last_name || ', ' || person_names.first_name || ' ' || person_names.suffix, "
        "surveys_taken.current_id, surveys_taken.id "
        "from surveys_taken left join people on surveys_taken.person_id = people.id "
        "left join person_names on people.person_names_id = person_names.id "
        "where surveys_taken.survey_id = " + initial_state["id"];

  taken_filter.SetSQLStatement(sql);
  taken_filter.RefreshDataSet();
  taken_filter.SetAddEdit(this, AddRecord(&(TformRecordSurvey::AddTaken)),
                          EditRecord(&(TformRecordSurvey::EditTaken)));
  taken_request = atlas->RequestNotificationWhen("surveys_taken.survey_id", initial_state["id"], rel_e, as_int, this,
                     RefreshHandler(&(TformRecordSurvey::AtlasRefreshListViews)));
  TformRecordGeneric::NotNewAnymore();
  return true;
}
//---------------------------------------------------------------------------
bool TformRecordSurvey::AtlasRefreshListViews(int request_num, string matched_value)
{
  if (request_num == question_request)
    question_filter.RefreshDataSet();
  else if (request_num == taken_request)
    taken_filter.RefreshDataSet();
  return true;
}
//---------------------------------------------------------------------------
void TformRecordSurvey::AddQuestion()
{
  map<string, string> temp_map;
  temp_map["id"] = "new";
  temp_map["survey_id"] = initial_state["id"];
  string max_question = GetOneField(dbDatabase, "select max(question_number) from survey_questions where survey_id = " + initial_state["id"]);
  if(max_question == "")
    max_question = "0";
  temp_map["question_number"] = IntToString(StringToInt(max_question) + 1);
  FORM_TRY
    TformRecordSurveyQuestion * question = new TformRecordSurveyQuestion(NULL, temp_map);
    question->Show();
  FORM_CATCH
}
//---------------------------------------------------------------------------
void TformRecordSurvey::EditQuestion()
{
  map<string, string> temp_map;
  temp_map["id"] = question_filter.GetSelectedRecordID();
  FORM_TRY
    TformRecordSurveyQuestion * question = new TformRecordSurveyQuestion(NULL, temp_map);
    question->Show();
  FORM_CATCH
}
//---------------------------------------------------------------------------
void TformRecordSurvey::AddTaken()
{
  map<string, string> temp_map;
  temp_map["id"] = "new";
  temp_map["survey_id"] = initial_state["id"];
  FORM_TRY
    TformRecordSurveyTaken * taken = new TformRecordSurveyTaken(NULL, temp_map);
    taken->Show();
  FORM_CATCH
}
//---------------------------------------------------------------------------
void TformRecordSurvey::EditTaken()
{
  map<string, string> temp_map;
  temp_map["id"] = taken_filter.GetSelectedRecordID();
  FORM_TRY
    TformRecordSurveyTaken * taken = new TformRecordSurveyTaken(NULL, temp_map);
    taken->Show();
  FORM_CATCH
}
//---------------------------------------------------------------------------
void TformRecordSurvey::Do(map<string, string> & p_parameters)
{
 FORM_TRY
  TformRecordSurvey * z = new TformRecordSurvey(NULL, p_parameters);
  z->Show();
 FORM_CATCH
}
//---------------------------------------------------------------------------

