//---------------------------------------------------------------------------

#include <vcl.h>
#pragma hdrstop

#include "codeRecordTaskBeh.h"
#include "codeSearchPerson.h"
#include "codeRecordCondition.h"
#include "codeRecordReferral.h"
#include "codeRecordProcedure.h"
#include "codeSearchEvent.h"
//---------------------------------------------------------------------------
#pragma package(smart_init)
#pragma link "codeRecordGeneric"
#pragma link "codeRubberContainer"
#pragma resource "*.dfm"
TformRecordTaskBeh *formRecordTaskBeh;
//---------------------------------------------------------------------------
__fastcall TformRecordTaskBeh::TformRecordTaskBeh(TComponent* Owner, map<string,string> p_parameters)
        : TformRecordGeneric(Owner, p_parameters, string("tasks"))
{
  fields_to_leave_raw.insert("task_comments");
  fields_to_leave_raw.insert("visit_comments");

  // set up filter objects
  referrals_filter.SetListViewHandle (listReferrals);
  referrals_filter.SetSecurity("referrals");
  referrals_filter.AddColumnSpec("Date", "referrals", as_date, true, 90);
  referrals_filter.AddColumnSpec("Referred To (Facility)", "referrals", as_text, true, 120);
  referrals_filter.AddColumnSpec("Referred To (Provider)", "referrals", as_text, true, 120);
  referrals_filter.AddColumnSpec("Referred By (Facility)", "referrals", as_text, true, 120);
  referrals_filter.AddColumnSpec("Referred By (Provider)", "referrals", as_text, true, 120);
  referrals_filter.AddColumnSpec("Reason", "referrals", as_text, false, 100);
  referrals_filter.AddColumnSpec("Current Id", "current_id", as_int, false, 80);
  referrals_filter.AddColumnSpec("Internal Id", "internal_id", as_int, false, 80);
  referrals_filter.FetchPreferences("recordtaskbeh.referrals_filter");
  referrals_filter.SetIconStatus(true);
  referrals_filter.SetSQLStatement ("");
  referrals_filter.RefreshDataSet();

  conditions_filter.SetListViewHandle (listConditions);
  conditions_filter.SetSecurity("conditions.beh");
  conditions_filter.AddColumnSpec("Condition Type", "conditions", as_text, true, 120);
  conditions_filter.AddColumnSpec("Start Date", "conditions", as_date, false, 100);
  conditions_filter.AddColumnSpec("Stop Date", "conditions", as_date, false, 100);
  conditions_filter.AddColumnSpec("Code", "conditions", as_text, true, 100);
  conditions_filter.AddColumnSpec("Note", "conditions", as_text, false, 100, sentenceString);
  conditions_filter.AddColumnSpec("Current Id", "current_id", as_int, false, 80);
  conditions_filter.AddColumnSpec("Internal Id", "internal_id", as_int, false, 80);
  conditions_filter.FetchPreferences("recordtaskdiab.conditions_filter");
  conditions_filter.SetIconStatus(true);
  conditions_filter.SetSQLStatement("");
  conditions_filter.RefreshDataSet();

  procedures_filter.SetListViewHandle (listProcedures);
  procedures_filter.SetSecurity("procedures");
  procedures_filter.AddColumnSpec("Start Date", "procedures", as_date, true, 100);
  procedures_filter.AddColumnSpec("Stop Date", "procedures", as_date, false, 100);
  procedures_filter.AddColumnSpec("Place of Service", "procedures", as_text, false, 110);
  procedures_filter.AddColumnSpec("Type of Service", "procedures", as_text, false, 110);
  procedures_filter.AddColumnSpec("Cpt/Hcpcs Code", "procedures", as_text, true, 105);
  procedures_filter.AddColumnSpec("Modifier", "procedures", as_text, false, 105);
  procedures_filter.AddColumnSpec("Charges", "procedures", as_text, false, 100);
  procedures_filter.AddColumnSpec("Days/Units", "procedures", as_text, false, 100);
  procedures_filter.AddColumnSpec("EPSDT", "procedures", as_text, false, 100);
  procedures_filter.AddColumnSpec("EMG", "procedures", as_text, false, 100);
  procedures_filter.AddColumnSpec("COB", "procedures", as_text, false, 100);
  procedures_filter.AddColumnSpec("Local Use", "procedures", as_text, false, 100);
  procedures_filter.AddColumnSpec("Current Id", "current_id", as_int, false, 80);
  procedures_filter.AddColumnSpec("Internal Id", "internal_id", as_int, false, 80);
  procedures_filter.FetchPreferences("recordtaskdiab.procedures_filter");
  procedures_filter.SetIconStatus(true);
  procedures_filter.SetSQLStatement("");
  procedures_filter.RefreshDataSet();

  appointment_to_attach = initial_state["appointment_id"];

 Initialize();
}
//---------------------------------------------------------------------------
bool TformRecordTaskBeh::ScreenStart()
{
  lookupTables->RegisterComboBox("users", cboStaff);
  lookupTables->RegisterComboBox("method_types", cboMethod);
  lookupTables->RegisterComboBox("direction_types", cboDirection);
  lookupTables->RegisterComboBox("classification_types", cboClassification);
  lookupTables->RegisterComboBox("beh_meeting_types", cboMeeting);
  lookupTables->RegisterComboBox("beh_activity_types", cboActivity);
  lookupTables->RegisterComboBox("beh_visit_types", cboVisitType);
  lookupTables->RegisterComboBox("visit_location_types", cboLocation);
  lookupTables->RegisterComboBox("visit_outcome_types", cboOutcome);
  lookupTables->RegisterComboBox("beh_appt_types", cboApptType);
  lookupTables->RegisterComboBox("beh_service_types", cboService);

  // jhaley 20100913: custom SQL for new program_types table
  string event_program_sql = "select id, item, valid from program_types "
                             "where (group_id = 0 or group_id = '" + lookupTables->IdOf("groups", "Behavioral Health") + "') "
                             "and valid = 1 order by item ";

  lookupTables->RegisterComboBox("program_types", cboProgramType, event_program_sql);

  string task_sql = "select id, item, valid from task_purpose_types where group_id = 0 or group_id = '"
                    + lookupTables->IdOf("groups", "Behavioral Health") + "' ";
  lookupTables->RegisterComboBox("task_purpose_types", cboTaskType, task_sql);

  string location_sql = "select event_location_types.id, case "
                        "when event_location_types.zip_code_id = 0 then item "
                        "when item = '' then (city || ', ' || state) "
                        "else (city || ', ' || state || ' - ' || item) end item, event_location_types.valid "
                        "from event_location_types "
                        "inner join zip_codes on event_location_types.zip_code_id = zip_codes.id "
                        "where (group_id = 0 or group_id = '" + lookupTables->IdOf("groups", "Behavioral Health") + "') "
                        "and valid = 1 order by state, city, item ";
  lookupTables->RegisterComboBox("event_location_types", cboCity, location_sql);

  TformRecordGeneric::ScreenStart();
  return true;
}
//---------------------------------------------------------------------------
bool TformRecordTaskBeh::ScreenStop()
{
  // unregister combo box
  lookupTables->UnregisterComboBox("users", cboStaff);
  lookupTables->UnregisterComboBox("method_types", cboMethod);
  lookupTables->UnregisterComboBox("direction_types", cboDirection);
  lookupTables->UnregisterComboBox("classification_types", cboClassification);
  lookupTables->UnregisterComboBox("beh_meeting_types", cboMeeting);
  lookupTables->UnregisterComboBox("beh_activity_types", cboActivity);
  lookupTables->UnregisterComboBox("beh_visit_types", cboVisitType);
  lookupTables->UnregisterComboBox("visit_location_types", cboLocation);
  lookupTables->UnregisterComboBox("visit_outcome_types", cboOutcome);
  lookupTables->UnregisterComboBox("event_location_types", cboCity);
  lookupTables->UnregisterComboBox("beh_appt_types", cboApptType);
  lookupTables->UnregisterComboBox("beh_service_types", cboService);
  lookupTables->UnregisterComboBox("program_types", cboProgramType);
  lookupTables->UnregisterComboBox("task_purpose_types", cboTaskType);

  TformRecordGeneric::ScreenStop();
  return true;
}
//---------------------------------------------------------------------------
bool TformRecordTaskBeh::IsValidOnScreen()
{
  // check to make sure that the values entered in the combo boxe
  // corrospond to values in that look up table.
  // give the user a message box and prompt to re-enter if incorrect

  if (! lookupTables->IsInLookUpTable("users", cboStaff->Text.c_str()))
  {
    PSay("Please enter a valid staff name", m_invalid_data, MB_OK);
    cboStaff->SetFocus();
    return false;
  }
  else if (cboStaff->Text == "")
  {
    PSay("You must enter a staff name", m_missing_data, MB_OK);
    cboStaff->SetFocus();
    return false;
  }
  else if (! IsFloat(editTimeSpent->Text.c_str()))
  {
    PSay("Please enter a valid time spent", m_invalid_data, MB_OK);
    editTimeSpent->SetFocus();
    return false;
  }
  else if (! IsFloat(editMileage->Text.c_str()))
  {
    PSay("Please enter a valid mileage", m_invalid_data, MB_OK);
    editMileage->SetFocus();
    return false;
  }
  else if (! lookupTables->IsInLookUpTable("method_types", cboMethod->Text.c_str()))
  {
    PSay("Please enter a valid method of contact", m_invalid_data, MB_OK);
    cboMethod->SetFocus();
    return false;
  }
  else if (! lookupTables->IsInLookUpTable("direction_types", cboDirection->Text.c_str()))
  {
    PSay("Please enter either incoming or outgoing", m_invalid_data, MB_OK);
    cboDirection->SetFocus();
    return false;
  }
  else if (! lookupTables->IsInLookUpTable("task_purpose_types", cboTaskType))
  {
    PSay("Please enter a valid task type", m_invalid_data, MB_OK);
    cboTaskType->SetFocus();
    return false;
  }
  else if (! lookupTables->IsInLookUpTable("classification_types", cboClassification->Text.c_str()))
  {
    PSay("Please enter a valid classification", m_invalid_data, MB_OK);
    cboClassification->SetFocus();
    return false;
  }
  else if (! lookupTables->IsInLookUpTable("beh_meeting_types", cboMeeting->Text.c_str()))
  {
    PSay("Please enter a valid meeting type", m_invalid_data, MB_OK);
    cboMeeting->SetFocus();
    return false;
  }
  else if (! lookupTables->IsInLookUpTable("beh_activity_types", cboActivity->Text.c_str()))
  {
    PSay("Please enter a valid activity type", m_invalid_data, MB_OK);
    cboActivity->SetFocus();
    return false;
  }
  else if (editTaskComments->Text.Length() > 255)
  {
    PSay("Comments can not be longer than 255 characters", m_invalid_data, MB_OK);
    editVisitComments->SetFocus();
    return false;
  }
  else if (! lookupTables->IsInLookUpTable("beh_appt_types", cboApptType->Text.c_str()))
  {
    PSay("Please enter a valid appointment type", m_invalid_data, MB_OK);
    cboApptType->SetFocus();
    return false;
  }
  else if (! lookupTables->IsInLookUpTable("beh_visit_types", cboVisitType->Text.c_str()))
  {
    PSay("Please enter a valid visit type", m_invalid_data, MB_OK);
    cboVisitType->SetFocus();
    return false;
  }
  else if (! lookupTables->IsInLookUpTable("visit_location_types", cboLocation->Text.c_str()))
  {
    PSay("Please enter a valid location", m_invalid_data, MB_OK);
    cboLocation->SetFocus();
    return false;
  }
  else if (! lookupTables->IsInLookUpTable("beh_service_types", cboService->Text.c_str()))
  {
    PSay("Please enter a service type", m_invalid_data, MB_OK);
    cboService->SetFocus();
    return false;
  }
  else if (cboProgramType->Text != "" && !lookupTables->IsInLookUpTable("program_types", cboProgramType))
  {
    PSay("Please enter a valid program type.", m_invalid_data, MB_OK);
    cboProgramType->SetFocus();
    return false;
  }
  else if (! IsFloat(editVisitTime->Text.c_str()))
  {
    PSay("Please enter a valid visit time spent", m_invalid_data, MB_OK);
    editVisitTime->SetFocus();
    return false;
  }
  else if (! lookupTables->IsInLookUpTable("visit_outcome_types", cboOutcome->Text.c_str()))
  {
    PSay("Please enter a valid outcome", m_invalid_data, MB_OK);
    cboOutcome->SetFocus();
    return false;
  }
  else if (cboCity->Text != "" && !lookupTables->IsInLookUpTable("event_location_types", cboCity))
  {
    PSay("Please enter a valid location", m_invalid_data, MB_OK);
    cboCity->SetFocus();
    return false;
  }
  else if (editVisitComments->Text.Length() > 255)
  {
    PSay("Comments can not be longer than 255 characters", m_invalid_data, MB_OK);
    editVisitComments->SetFocus();
    return false;
  }
  else if (initial_state["group_id"] == "" || StringToInt(initial_state["group_id"]) <= 0)
  {
    PSay("This task does not belong to a valid department. Please contact a programmer if this occured while creating a new task.",
         m_program_error, MB_OK);
    return false;
  }

  // jhaley 20110504: tasks with a person and a user are probably visits.
  if(cboMethod->Text == "" && cboStaff->Text != "" && !IsBlankOrZero(GetValidLinkId(txtPerson)))
  {
    int res = PSay("This task has both a provider and a client and is therefore probably a Visit, but you have not "
                   "marked it as such (Method of Contact is blank).\n"
                   "\n"
                   "If this task represents a billable contact between a provider and a client, it should be marked as "
                   "a visit in order to be GPRA- and IHS-workload reportable.\n"
                   "\n"
                   "Select 'Yes' to mark this task as a visit automatically.\n"
                   "Select 'No' to leave Method of Contact blank.\n"
                   "Select 'Cancel' to abort the saving of this task.", m_missing_data, MB_YESNOCANCEL);
    switch(res)
    {
    case IDYES:
      cboMethod->Text = "Visit"; // Set method to visit.
      break;
    case IDNO:                   // Do nothing.
      break;
    case IDCANCEL:               // Stop saving.
      return false;
    }
  }

  // jhaley 20100802: logic adapted from Diabetes task form to do special logic checks for task types
  string event_id = GetValidLinkId(txtEvent);
  if (event_id == "")
  {
    // no event...
    if (cboTaskType->Text == "Event Preparation" || cboTaskType->Text == "Event Teaching/Leading" ||
        cboTaskType->Text == "Event Contact")
    {
      // ...but this task type implies that the task should be attached to an event.
      PSay("This task is not attached to an event, but you have specified a 'Task Type' reserved for events.\n\n"
           "Please attach this task to the appropriate event, or select a different (or blank) Task Type.",
           m_invalid_data, MB_OK);
      cboTaskType->Show();
      cboTaskType->SetFocus();
      return false;
    }
  }
  else
  {
    // attached to an event...
    string person_id = GetValidLinkId(txtPerson);
    if(person_id == "" && cboTaskType->Text == "Event Contact")
    {
      // ...but this task type implies that a person should be specified, and one is not.
      PSay("This task is part of an event with a 'Task Type' indicating credit for contact with a client, but no client is "
           "selected.\n\nPlease search for the Client Name of the event participant, or select a different (or blank) Task "
           "Type.", m_invalid_data, MB_OK);
      btnPersonSearch->Show();
      btnPersonSearch->SetFocus();
      return false;
    }
    else if (StringToInt(person_id) > 0 &&
             (cboTaskType->Text == "Event Preparation" || cboTaskType->Text == "Event Teaching/Leading"))
    {
      // ..but this task type shouldn't have a person, and it does.
      PSay("This task is part of an event with a 'Task Type' indicating teaching/leading or preparation, but a Client Name "
           "is selected.\n\nPlease unlink the client that is selected and use the Event screen to create contacts for the "
           "appropriate participants. You can unlink the client by pressing the search button, then pressing 'Select' without "
           "performing a search (link text will change to 'None').", m_invalid_data, MB_OK);
      btnPersonSearch->Show();
      btnPersonSearch->SetFocus();
      return false;
    }
  }

  return true;
}
//---------------------------------------------------------------------------
bool TformRecordTaskBeh::PutDataOnScreen()
{
  // put the values on the form from the map initial_state
  // values may have come from GetDataFromDatabase if it's an existing record,
  // or values may have been passed to the form in this map if it's a new record

  cboStaff->Text = PrettyString(lookupTables->ValueOf("users", initial_state["user_id"])).c_str();
  if (initial_state["date_of_task"] != "")
    dateTask->Date = Pdate(initial_state["date_of_task"], mdy | yyyy).OutDate();
  else
    dateTask->Date = Date();
  if (initial_state["time_of_task"] != "")
    timeTask->Time = Ptime(initial_state["time_of_task"], withsec | military).OutTime();
  else
  {
    Ptime current_time(Time());
    current_time.second = 0;
    timeTask->Time = current_time.OutTime();
  }

  editTimeSpent->Text     = initial_state["time_spent"].c_str();
  editMileage->Text       = initial_state["mileage"].c_str();
  editVisitTime->Text     = initial_state["visit_time_spent"].c_str();
  editTaskComments->Text  = SentenceString(initial_state["task_comments"]).c_str();
  editVisitComments->Text = SentenceString(initial_state["visit_comments"]).c_str();
  cboMethod->Text         = PrettyString(lookupTables->ValueOf("method_types",         initial_state["method_id"])).c_str();
  cboDirection->Text      = PrettyString(lookupTables->ValueOf("direction_types",      initial_state["direction_id"])).c_str();
  cboTaskType->Text       = PrettyString(lookupTables->ValueOf("task_purpose_types",   initial_state["task_purpose_id"], cboTaskType)).c_str();
  cboClassification->Text = PrettyString(lookupTables->ValueOf("classification_types", initial_state["classification_id"])).c_str();
  cboMeeting->Text        = PrettyString(lookupTables->ValueOf("beh_meeting_types",    initial_state["beh_meeting_id"])).c_str();
  cboActivity->Text       = PrettyString(lookupTables->ValueOf("beh_activity_types",   initial_state["beh_activity_id"])).c_str();
  cboApptType->Text       = PrettyString(lookupTables->ValueOf("beh_appt_types",       initial_state["beh_appt_id"])).c_str();
  cboVisitType->Text      = PrettyString(lookupTables->ValueOf("beh_visit_types",      initial_state["beh_visit_id"])).c_str();
  cboLocation->Text       = PrettyString(lookupTables->ValueOf("visit_location_types", initial_state["visit_location_id"])).c_str();
  cboService->Text        = PrettyString(lookupTables->ValueOf("beh_service_types",    initial_state["beh_service_id"])).c_str();
  cboOutcome->Text        = PrettyString(lookupTables->ValueOf("visit_outcome_types",  initial_state["visit_outcome_id"])).c_str();
  cboCity->Text           = PrettyString(lookupTables->ValueOf("event_location_types", initial_state["event_location_id"], cboCity)).c_str();
  cboProgramType->Text    = lookupTables->ValueOf("program_types", initial_state["program_id"], cboProgramType).c_str();

  if (initial_state["new_client"] == "1")
    checkNew->Checked = true;
  else
    checkNew->Checked = false;

  CreateLink(txtPerson, btnPersonSearch, this, initial_state["person_id"], "people");
  CreateLink(txtEvent, btnEventSearch, this, initial_state["event_id"], "events");
  TformRecordGeneric::PutDataOnScreen();
  return true;
}
//---------------------------------------------------------------------------
bool TformRecordTaskBeh::GetDataFromScreen()
{
  TformRecordGeneric::GetDataFromScreen();
  if (!CreateInsertMap(dbDatabase, "tasks", ending_state))
    return false;

  // place the values currently on the screen into
  // the map that will be used to update or insert the record
  // the parent form will call the insert or update function, and get the id
  // number if necessary and take care of current_id

  ending_state["id"] = initial_state["id"];
  string person_id = GetValidLinkId(txtPerson);
  if(StringToInt(person_id) > 0)
    ending_state["person_id"] = person_id;
  else
    ending_state["person_id"] = "0";
  string event_id = GetValidLinkId(txtEvent);
  if(StringToInt(event_id) > 0)
    ending_state["event_id"] = event_id;
  else
    ending_state["event_id"] = "0";
  ending_state["group_id"] = initial_state["group_id"];
  ending_state["user_id"] = lookupTables->IdOf("users", cboStaff->Text.c_str());
  ending_state["date_of_task"] = Pdate(dateTask->Date).OutDate(mdy | yyyy);
  ending_state["time_of_task"] = Ptime(timeTask->Time).OutTime(military | withsec);
  if (editTimeSpent->Text != "")
    ending_state["time_spent"] = editTimeSpent->Text.c_str();
  else
    ending_state["time_spent"] = "";
  if (editMileage->Text != "")
    ending_state["mileage"] = editMileage->Text.c_str();
  else
    ending_state["mileage"] = "";

  ending_state["method_id"] = lookupTables->IdOf("method_types", cboMethod->Text.c_str());
  ending_state["direction_id"] = lookupTables->IdOf("direction_types", cboDirection->Text.c_str());
  ending_state["task_purpose_id"] = lookupTables->IdOf("task_purpose_types", cboTaskType->Text.c_str(), cboTaskType);
  ending_state["classification_id"] = lookupTables->IdOf("classification_types", cboClassification->Text.c_str());
  ending_state["beh_meeting_id"] = lookupTables->IdOf("beh_meeting_types", cboMeeting->Text.c_str());
  ending_state["beh_activity_id"] = lookupTables->IdOf("beh_activity_types", cboActivity->Text.c_str());
  ending_state["task_comments"] = editTaskComments->Text.c_str();
  ending_state["beh_appt_id"] = lookupTables->IdOf("beh_appt_types", cboApptType->Text.c_str());
  ending_state["beh_visit_id"] = lookupTables->IdOf("beh_visit_types", cboVisitType->Text.c_str());
  ending_state["visit_location_id"] = lookupTables->IdOf("visit_location_types", cboLocation->Text.c_str());
  ending_state["beh_service_id"] = lookupTables->IdOf("beh_service_types", cboService->Text.c_str());
  ending_state["program_id"] = lookupTables->IdOf("program_types", cboProgramType);
  ending_state["visit_time_spent"] = editVisitTime->Text.c_str();
  ending_state["visit_outcome_id"] = lookupTables->IdOf("visit_outcome_types", cboOutcome->Text.c_str());
  ending_state["event_location_id"] = lookupTables->IdOf("event_location_types", cboCity);
  ending_state["visit_comments"] = editVisitComments->Text.c_str();
  if (checkNew->Checked)
    ending_state["new_client"] = "1";
  else
    ending_state["new_client"] = "0";

  // No longer necessary.  CreateInsertMap takes care of default values for FKs
  // set the rest of the values in the tasks table, even though we don't use them
  // on this form, to avoid null values in the table
  // NOTE: when fields are added to the tasks table, they should be added here

  return true;
}
//---------------------------------------------------------------------------
bool TformRecordTaskBeh::ScreenLock()
{
  // disable group boxes
  taskInfoGroup->Enabled = false;
  visitInfoGroup->Enabled = false;
  editTaskComments->ReadOnly = true;
  editTaskComments->TabStop = false;
  editVisitComments->ReadOnly = true;
  editVisitComments->TabStop = false;
  listReferrals->Enabled = true;
  TformRecordGeneric::ScreenLock();
  return true;
}
//---------------------------------------------------------------------------
bool TformRecordTaskBeh::ScreenUnlock()
{
  // enable group boxes
  taskInfoGroup->Enabled = true;
  visitInfoGroup->Enabled = true;
  editTaskComments->ReadOnly = false;
  editTaskComments->TabStop = true;
  editVisitComments->ReadOnly = false;
  editVisitComments->TabStop = true;
  listReferrals->Enabled = false;
  TformRecordGeneric::ScreenUnlock();
  return true;
}
//---------------------------------------------------------------------------

void __fastcall TformRecordTaskBeh::btnPersonSearchClick(TObject *Sender)
{
  // create and show new person search form
  vector<string> return_vector;
  FORM_TRY
    TformSearchPerson * formPersonSearch = new TformSearchPerson(NULL,&return_vector);
    formPersonSearch->ShowTaskModal(this);
  FORM_CATCH
  if (! return_vector.empty())
  {
    CreateLink(txtPerson, btnPersonSearch, this, return_vector[0], "people");

    // jhaley 20110504: Tasks with a client should be marked as a visit automatically.
    // And in case they maliciously try to undo this, they will be warned thoroughly by IsValidOnScreen.
    // And if they ignore THAT warning, well, that's when I'll start kicking some ass ;)
    if(!IsBlankOrZero(GetValidLinkId(txtPerson)) && cboMethod->Text == "")
      cboMethod->Text = "Visit";
  }
}
//---------------------------------------------------------------------------
bool TformRecordTaskBeh::IsNewAgain()
{
 referrals_filter.SetSQLStatement(string(""));
 referrals_filter.RefreshDataSet();
 referrals_filter.SetAddEdit(this, NULL, NULL);
 conditions_filter.SetSQLStatement(string(""));
 conditions_filter.RefreshDataSet();
 conditions_filter.SetAddEdit(this, NULL, NULL);
 procedures_filter.SetSQLStatement(string(""));
 procedures_filter.RefreshDataSet();
 procedures_filter.SetAddEdit(this, NULL, NULL);
 TformRecordGeneric::IsNewAgain();
 return true;
}
//---------------------------------------------------------------------------
bool TformRecordTaskBeh::NotNewAnymore()
{
  string sql;
    sql = "select referrals.date_of_referral, a.name, b.name, c.name, d.name, referral_reason_types.item, "
          "referrals.current_id, referrals.id "
          "from referrals, tax_entities a, tax_entities b, tax_entities c, tax_entities d, referral_reason_types "
          "where referrals.by_facility_id = a.id and "
          "referrals.by_provider_id = b.id and "
          "referrals.to_facility_id = c.id and "
          "referrals.to_provider_id = d.id and "
          "referrals.referral_reason_id = referral_reason_types.id and "
          "referrals.task_id = " + initial_state["id"];

  referrals_filter.SetSQLStatement(sql);
  referrals_filter.RefreshDataSet();
  referrals_filter.SetAddEdit(this, AddRecord(&(TformRecordTaskBeh::AddReferral)),
                                        EditRecord(&(TformRecordTaskBeh::EditReferral)));
  referrals_request = atlas->RequestNotificationWhen("referrals.task_id", initial_state["id"], rel_e, as_int, this,
                               RefreshHandler(&(TformRecordTaskBeh::AtlasRefreshListViews)));

  if (optionsPermissions["table.conditions.beh"]["view"] == "true")
  {
    sql = "select condition_types.item, conditions.start_date, conditions.stop_date, "
          "conditions.code, conditions.note, conditions.current_id, conditions.id "
          "from conditions, condition_types "
          "where conditions.condition_id = condition_types.id and "
          "conditions.task_id = " + initial_state["id"];
    conditions_filter.SetSQLStatement(sql);
    conditions_filter.RefreshDataSet();
    conditions_filter.SetAddEdit(this, AddRecord(&(TformRecordTaskBeh::AddCondition)),
                                  EditRecord(&(TformRecordTaskBeh::EditCondition)));
    conditions_request = atlas->RequestNotificationWhen("conditions.task_id", initial_state["id"], rel_e, as_int, this,
                              RefreshHandler(&(TformRecordTaskBeh::AtlasRefreshListViews)));
  }

  if (optionsPermissions["table.procedures"]["view"] == "true")
  {
    sql = "select procedures.start_date, procedures.stop_date, procedures.place_of_service, "
          "procedures.type_of_service, procedures.cpt_hcpcs, procedures.modifier, "
          "procedures.charges, procedures.days_units, procedures.epsdt, procedures.emg, procedures.cob, "
          "procedures.local_use, procedures.current_id, procedures.id "
          "from procedures "
          "where procedures.task_id = " + initial_state["id"];
    procedures_filter.SetSQLStatement(sql);
    procedures_filter.RefreshDataSet();
    procedures_filter.SetAddEdit(this, AddRecord(&(TformRecordTaskBeh::AddProcedure)),
                                  EditRecord(&(TformRecordTaskBeh::EditProcedure)));
    procedures_request = atlas->RequestNotificationWhen("procedures.task_id", initial_state["id"], rel_e, as_int, this,
                              RefreshHandler(&(TformRecordTaskBeh::AtlasRefreshListViews)));
  }
  TformRecordGeneric::NotNewAnymore();
  return true;
}
//---------------------------------------------------------------------------
bool TformRecordTaskBeh::AtlasRefreshListViews(int request_num, string matched_value)
{
  if (request_num == referrals_request)
    referrals_filter.RefreshDataSet();
  else if (conditions_request == request_num)
    conditions_filter.RefreshDataSet();
  else if (procedures_request == request_num)
    procedures_filter.RefreshDataSet();

  return true;
}
//---------------------------------------------------------------------------
void TformRecordTaskBeh::AddReferral()
{
  string person_id = GetValidLinkId(txtPerson);
  if (person_id != "")
  {
    map<string, string> temp_map;
    temp_map["id"] = "new";
    temp_map["person_id"] = person_id;
    temp_map["task_id"] = initial_state["id"];
    FORM_TRY
      TformRecordReferral * referral = new TformRecordReferral(NULL, temp_map);
      referral->Show();
    FORM_CATCH
  }
}
//---------------------------------------------------------------------------
void TformRecordTaskBeh::EditReferral()
{
  map<string, string> temp_map;
  temp_map["id"] = referrals_filter.GetSelectedRecordID();
  FORM_TRY
    TformRecordReferral * referral = new TformRecordReferral(NULL, temp_map);
    referral->Show();
  FORM_CATCH
}
//---------------------------------------------------------------------------
void TformRecordTaskBeh::AddCondition()
{
  map<string,string> temp_map;
  temp_map["id"] = "new";
  temp_map["task_id"] = initial_state["id"];
  string person_id = GetValidLinkId(txtPerson);
  if (person_id != "")
    temp_map["person_id"] = person_id;
  if (initial_state["date_of_task"] != "")
    temp_map["start_date"] = initial_state["date_of_task"];
  temp_map["group_id"] = lookupTables->IdOf("groups", "Behavioral Health");
  FORM_TRY
    TformRecordCondition * conditionRecord = new TformRecordCondition(NULL, temp_map);
    conditionRecord->Show();
  FORM_CATCH
}
//---------------------------------------------------------------------------
void TformRecordTaskBeh::EditCondition()
{
  map<string,string> temp_map;
  temp_map["id"] = conditions_filter.GetSelectedRecordID();
  FORM_TRY
    TformRecordCondition * conditionRecord = new TformRecordCondition(NULL, temp_map);
    conditionRecord->Show();
  FORM_CATCH
}
//---------------------------------------------------------------------------
void TformRecordTaskBeh::AddProcedure()
{
  map<string,string> temp_map;
  temp_map["id"] = "new";
  temp_map["task_id"] = initial_state["id"];
  if (initial_state["date_of_task"] != "")
    temp_map["start_date"] = initial_state["date_of_task"];
  FORM_TRY
    TformRecordProcedure * procedureRecord = new TformRecordProcedure(NULL, temp_map);
    procedureRecord->Show();
  FORM_CATCH
}
//---------------------------------------------------------------------------
void TformRecordTaskBeh::EditProcedure()
{
  map<string,string> temp_map;
  temp_map["id"] = procedures_filter.GetSelectedRecordID();
  FORM_TRY
    TformRecordProcedure * procedureRecord = new TformRecordProcedure(NULL, temp_map);
    procedureRecord->Show();
  FORM_CATCH
}
//---------------------------------------------------------------------------
void __fastcall TformRecordTaskBeh::btnEventSearchClick(TObject *Sender)
{
  // create and show new person search form
  vector<string> return_vector;
  FORM_TRY
    TformSearchEvent * eventSearch = new TformSearchEvent(NULL,&return_vector,lookupTables->IdOf("groups", "Behavioral Health"));
    eventSearch->ShowTaskModal(this);
  FORM_CATCH
  if (! return_vector.empty())
    CreateLink(txtEvent, btnEventSearch, this, return_vector[0], "events");
}
//---------------------------------------------------------------------------
bool TformRecordTaskBeh::BeforeSave()
{
  bool result = true;
  if(is_new_record && (appointment_to_attach != ""))
  {
    map<string, string> field_map;
    field_map["id"] = appointment_to_attach;
    field_map["task_id"] = ending_state["id"];
    if (ExecuteUpdateStatement(lock_transaction, "appointments", field_map))
      atlas->Notify("appointments.id", field_map["id"]);
    else
    {
      result = false;
      PSay("Unable to attach appointment to task/visit.  If the appointment is still unlocked for editing, "
           "save the appointment and try again.", m_program_error, MB_OK);
    }
  }
  return result;
}
//---------------------------------------------------------------------------
void TformRecordTaskBeh::Do(map<string, string> & p_parameters)
{
 FORM_TRY
  TformRecordTaskBeh * z = new TformRecordTaskBeh(NULL, p_parameters);
  z->Show();
 FORM_CATCH
}
//---------------------------------------------------------------------------

