//---------------------------------------------------------------------------

#include <vcl.h>
#pragma hdrstop

#include "codeRecordTaskCare.h"
#include "codeSearchPerson.h"
#include "codeSearchEvent.h"
#include "codeRecordComHealth.h"
//---------------------------------------------------------------------------
#pragma package(smart_init)
#pragma link "codeRecordGeneric"
#pragma link "codeRubberContainer"
#pragma resource "*.dfm"
TformRecordTaskCare *formRecordTaskCare;
//---------------------------------------------------------------------------
__fastcall TformRecordTaskCare::TformRecordTaskCare(TComponent* Owner, map<string,string> p_parameters)
        : TformRecordGeneric(Owner, p_parameters, string("tasks"))
{
  fields_to_leave_raw.insert("task_comments");
  fields_to_leave_raw.insert("visit_comments");

  education_filter.SetListViewHandle(listComEducation);
  education_filter.SetSecurity("com_health_edu");
  education_filter.AddColumnSpec("Internal Id", "com_health_edu", as_int, false, 80);
  education_filter.AddColumnSpec("Current Id", "current_id", as_int, false, 80);
  education_filter.AddColumnSpec("Topic", "com_health_edu", as_text, true, 80);
  education_filter.AddColumnSpec("Learner", "com_health_edu", as_text, true,80);
  education_filter.AddColumnSpec("Method", "com_health_edu",as_text,true,80);
  education_filter.AddColumnSpec("Evaluation", "com_health_edu", as_text, true, 80);
  education_filter.FetchPreferences("recordCareTaskVisit.community_edu_filter");
  education_filter.SetSQLStatement("");
  education_filter.RefreshDataSet();

  appointment_to_attach = initial_state["appointment_id"];

  Initialize();
}
//---------------------------------------------------------------------------
bool TformRecordTaskCare::ScreenStart()
{
  lookupTables->RegisterComboBox("users", cboStaff);
  lookupTables->RegisterComboBox("method_types", cboMethod);
  lookupTables->RegisterComboBox("direction_types", cboInOut);
  lookupTables->RegisterComboBox("care_classification_types", cboClassification);
  lookupTables->RegisterComboBox("care_meeting_types", cboMeetingType);
  lookupTables->RegisterComboBox("com_activity_types", cboActivityType);
  string task_sql = "select id, item, valid from task_purpose_types where group_id = 0 or group_id = '"
                    + lookupTables->IdOf("groups", "Caregiver Support Program") + "' ";
  lookupTables->RegisterComboBox("task_purpose_types", cboTaskType, task_sql);
  // lookupTables->RegisterComboBox("care_service_types", cboServiceType);

  TformRecordGeneric::ScreenStart();
  return true;
}
//---------------------------------------------------------------------------
bool TformRecordTaskCare::ScreenStop()
{
  // unregister combo box
  lookupTables->UnregisterComboBox("users", cboStaff);
  lookupTables->UnregisterComboBox("method_types", cboMethod);
  lookupTables->UnregisterComboBox("direction_types", cboInOut);
  lookupTables->UnregisterComboBox("care_classification_types", cboClassification);
  lookupTables->UnregisterComboBox("care_meeting_types", cboMeetingType);
  lookupTables->UnregisterComboBox("com_activity_types", cboActivityType);
  lookupTables->UnregisterComboBox("task_purpose_types", cboTaskType);
  // lookupTables->UnregisterComboBox("care_service_types", cboServiceType);

  TformRecordGeneric::ScreenStop();
  return true;
}
//---------------------------------------------------------------------------
bool TformRecordTaskCare::IsValidOnScreen()
{
  // check to make sure that the values entered in the combo box
  // corrospond to values in that look up table.
  // give the user a message box and prompt to re-enter if incorrect

  if (! lookupTables->IsInLookUpTable("users", cboStaff->Text.c_str()))
  {
    PSay("Please enter a valid staff name", m_invalid_data, MB_OK);
    cboStaff->SetFocus();
    return false;
  }
  else if(cboStaff->Text == "")
  {
    PSay("Please enter a staff name", m_invalid_data, MB_OK);
    cboStaff->SetFocus();
    return false;
  }
  else if (! IsFloat(editTimeSpent->Text.c_str()))
  {
    PSay("Please enter a valid time spent", m_invalid_data, MB_OK);
    editTimeSpent->SetFocus();
    return false;
  }
  else if (! IsFloat(editMileage->Text.c_str()))
  {
    PSay("Please enter a valid mileage", m_invalid_data, MB_OK);
    editMileage->SetFocus();
    return false;
  }
  else if (cboStaff->Text == "")
  {
    PSay("You must enter an employee name", m_missing_data, MB_OK);
    cboStaff->SetFocus();
    return false;
  }
  else if (! lookupTables->IsInLookUpTable("method_types", cboMethod->Text.c_str()))
  {
    PSay("Please enter a valid method of contact", m_invalid_data, MB_OK);
    cboMethod->SetFocus();
    return false;
  }
  else if (! lookupTables->IsInLookUpTable("direction_types", cboInOut->Text.c_str()))
  {
    PSay("Please enter either incoming or outgoing", m_invalid_data, MB_OK);
    cboInOut->SetFocus();
    return false;
  }
  else if (! lookupTables->IsInLookUpTable("task_purpose_types", cboTaskType))
  {
    PSay("Please enter a valid task type", m_invalid_data, MB_OK);
    cboTaskType->SetFocus();
    return false;
  }
  else if (! lookupTables->IsInLookUpTable("care_meeting_types", cboMeetingType->Text.c_str()))
  {
    PSay("Please enter a valid meeting type", m_invalid_data, MB_OK);
    cboMeetingType->SetFocus();
    return false;
  }
  else if (! lookupTables->IsInLookUpTable("com_activity_types", cboActivityType->Text.c_str()))
  {
    PSay("Please enter a valid activity type", m_invalid_data, MB_OK);
    cboActivityType->SetFocus();
    return false;
  }
  else if (! lookupTables->IsInLookUpTable("care_classification_types", cboClassification->Text.c_str()))
  {
    PSay("Please enter a valid classification", m_invalid_data, MB_OK);
    cboClassification->SetFocus();
    return false;
  }
  else if(initial_state["group_id"] == "" || StringToInt(initial_state["group_id"]) <= 0)
  {
    PSay("This task does not belong to a valid department. Please contact a programmer if this occured while creating a new task.",
         m_program_error, MB_OK);
    return false;
  }
/*
  else if (! lookupTables->IsInLookUpTable("care_service_types", cboServiceType->Text.c_str()))
  {
    PSay("Please enter a valid service type", m_invalid_data, MB_OK);
    cboActivityType->SetFocus();
    return false;
  }
*/
  // jhaley 20100802: logic adapted from Diabetes task form to do special logic checks for task types
  string person_id = GetValidLinkId(txtPerson);
  string event_id  = GetValidLinkId(txtEvent);
  if (event_id == "")
  {
    // no event...
    if (cboTaskType->Text == "Event Preparation" || cboTaskType->Text == "Event Teaching/Leading" ||
        cboTaskType->Text == "Event Contact")
    {
      // ...but this task type implies that the task should be attached to an event.
      PSay("This task is not attached to an event, but you have specified a 'Task Type' reserved for events.\n\n"
           "Please attach this task to the appropriate event, or select a different (or blank) Task Type.",
           m_invalid_data, MB_OK);
      cboTaskType->Show();
      cboTaskType->SetFocus();
      return false;
    }
  }
  else
  {
    // attached to an event...
    if(person_id == "" && cboTaskType->Text == "Event Contact")
    {
      // ...but this task type implies that a person should be specified, and one is not.
      PSay("This task is part of an event with a 'Task Type' indicating credit for contact with a client, but no client is "
           "selected.\n\nPlease search for the Client Name of the event participant, or select a different (or blank) Task "
           "Type.", m_invalid_data, MB_OK);
      btnPersonSearch->Show();
      btnPersonSearch->SetFocus();
      return false;
    }
    else if (StringToInt(person_id) > 0 &&
             (cboTaskType->Text == "Event Preparation" || cboTaskType->Text == "Event Teaching/Leading"))
    {
      // ...but this task type shouldn't have a person, and it does.
      PSay("This task is part of an event with a 'Task Type' indicating teaching/leading or preparation, but a Client Name "
           "is selected.\n\nPlease unlink the client that is selected and use the Event screen to create contacts for the "
           "appropriate participants. You can unlink the client by pressing the search button, then pressing 'Select' without "
           "performing a search (link text will change to 'None').", m_invalid_data, MB_OK);
      btnPersonSearch->Show();
      btnPersonSearch->SetFocus();
      return false;
    }
  }

  return true;
}
//---------------------------------------------------------------------------
bool TformRecordTaskCare::PutDataOnScreen()
{
  // put the values on the form from the map initial_state
  // values may have come from GetDataFromDatabase if it's an existing record,
  // or values may have been passed to the form in this map if it's a new record

  cboStaff->Text = PrettyString(lookupTables->ValueOf("users", initial_state["user_id"])).c_str();

  if (initial_state["date_of_task"] != "")
    dateTask->Date = Pdate(initial_state["date_of_task"], mdy | yyyy).OutDate();
  else
    dateTask->Date = Date();

  if (initial_state["time_of_task"] != "")
    timeTask->Time = Ptime(initial_state["time_of_task"], withsec | military).OutTime();
  else
  {
    Ptime current_time(Time());
    current_time.second = 0;
    timeTask->Time = current_time.OutTime();
  }

  if (!DisplayServiceTypes())
    return false;
  else
    HighlightServiceTypes();

  editTimeSpent->Text = initial_state["time_spent"].c_str();
  editMileage->Text   = initial_state["mileage"].c_str();
  editComments->Text  = SentenceString(initial_state["task_comments"]).c_str();

  cboMethod->Text         = PrettyString(lookupTables->ValueOf("method_types",              initial_state["method_id"])).c_str();
  cboInOut->Text          = PrettyString(lookupTables->ValueOf("direction_types",           initial_state["direction_id"])).c_str();
  cboTaskType->Text       = PrettyString(lookupTables->ValueOf("task_purpose_types",        initial_state["task_purpose_id"], cboTaskType)).c_str();
  cboClassification->Text = PrettyString(lookupTables->ValueOf("care_classification_types", initial_state["care_classification_id"])).c_str();
  cboMeetingType->Text    = PrettyString(lookupTables->ValueOf("care_meeting_types",        initial_state["care_meeting_id"])).c_str();
  cboActivityType->Text   = PrettyString(lookupTables->ValueOf("com_activity_types",        initial_state["com_activity_id"])).c_str();
  //cboServiceType->Text  = PrettyString(lookupTables->ValueOf("care_service_types", initial_state["care_service_id"])).c_str();

  CreateLink(txtPerson, btnPersonSearch, this, initial_state["person_id"], "people");
  CreateLink(txtEvent,  btnEventSearch,  this, initial_state["event_id"],  "events");

  TformRecordGeneric::PutDataOnScreen();
  return true;
}
//---------------------------------------------------------------------------
bool TformRecordTaskCare::DisplayServiceTypes()
{
  TIBTransaction * dbTransaction;

  try
  {
    dbTransaction = new TIBTransaction(NULL);
    StdTransaction (dbTransaction, dbDatabase);
  }
  catch(...) { }

  bool result;
  try
  {
    vector<map<string, string> > services_vec;
    string sql = "select id, item from care_service_types where valid = 1 and id > 0 order by item ascending";
    SqlToVecMap(dbTransaction, sql, services_vec);
    TListItem* rows;
    listServices->Items->Clear();
    for (unsigned int i = 0; i < services_vec.size(); i++)
    {
      rows = listServices->Items->Add();
      rows->Caption = PrettyString(services_vec[i]["item"]).c_str();
      rows->SubItems->Add(services_vec[i]["id"].c_str());
    }
    dbTransaction->Commit();
    result = true;
  }
  catch(...)
  {
    if (dbTransaction->Active)
      dbTransaction->Rollback();
    result = false;
  }
  delete dbTransaction;
  return result;
}
//---------------------------------------------------------------------------
void TformRecordTaskCare::HighlightServiceTypes()
{
  TIBTransaction * dbTransaction;

  try
  {
    dbTransaction = new TIBTransaction(NULL);
    StdTransaction (dbTransaction, dbDatabase);
  }
  catch(...) { }

  if ((initial_state["id"] != "new") && (initial_state["id"] != ""))
  {
    string sql;
    TListItem* rows;
    string id;

    try
    {
      vector<map<string, string> > services_to_tasks;
      sql = "select care_service_id from tasks_to_care_services where task_id = " +
            initial_state["id"];
      SqlToVecMap(dbTransaction, sql, services_to_tasks);

      for (unsigned int i = 0; i < services_to_tasks.size(); i++)
      {
        for (int j = 0; j < listServices->Items->Count; j++)
        {
          rows = listServices->Items->Item[j];
          id = rows->SubItems->Strings[0].c_str();
          if (id == services_to_tasks[i]["care_service_id"])
            rows->Checked = true;
        }
      }
      dbTransaction->Commit();
    }
    catch(...)
    {
      if (dbTransaction->Active)
        dbTransaction->Rollback();
    }
  }
  delete dbTransaction;
}
//---------------------------------------------------------------------------
bool TformRecordTaskCare::GetDataFromScreen()
{
  TformRecordGeneric::GetDataFromScreen();
  if (!CreateInsertMap(dbDatabase, "tasks", ending_state))
     return false;

  // place the values currently on the screen into
  // the map that will be used to update or insert the record
  // the parent form will call the insert or update function, and get the id
  // number if necessary and take care of current_id

  ending_state["id"] = initial_state["id"];
  string person_id = GetValidLinkId(txtPerson);
  if(StringToInt(person_id) > 0)
    ending_state["person_id"] = person_id;
  else
    ending_state["person_id"] = "0";
  string event_id = GetValidLinkId(txtEvent);
  if(StringToInt(event_id) > 0)
    ending_state["event_id"] = event_id;
  else
    ending_state["event_id"] = "0";

  ending_state["group_id"] = initial_state["group_id"];
  ending_state["user_id"] = lookupTables->IdOf("users", cboStaff->Text.c_str());
  ending_state["date_of_task"] = Pdate(dateTask->Date).OutDate(mdy | yyyy);
  ending_state["time_of_task"] = Ptime(timeTask->Time).OutTime(military | withsec);

  if (editTimeSpent->Text != "")
    ending_state["time_spent"] = editTimeSpent->Text.c_str();
  else
    ending_state["time_spent"] = "0";
  if (editMileage->Text != "")
    ending_state["mileage"] = editMileage->Text.c_str();
  else
    ending_state["mileage"] = "0";


  ending_state["method_id"] = lookupTables->IdOf("method_types", cboMethod->Text.c_str());
  ending_state["direction_id"] = lookupTables->IdOf("direction_types", cboInOut->Text.c_str());
  ending_state["task_purpose_id"] = lookupTables->IdOf("task_purpose_types", cboTaskType->Text.c_str(), cboTaskType);
  ending_state["care_classification_id"] = lookupTables->IdOf("care_classification_types", cboClassification->Text.c_str());
  ending_state["task_comments"] = editComments->Text.c_str();
  ending_state["care_meeting_id"] = lookupTables->IdOf("care_meeting_types", cboMeetingType->Text.c_str());
  ending_state["com_activity_id"] = lookupTables->IdOf("com_activity_types", cboActivityType->Text.c_str());
  //ending_state["care_service_id"] = lookupTables->IdOf("care_service_types", cboServiceType->Text.c_str());

  // No longer necessary.  CreateInsertMap takes care of default values for FKs
  // set the rest of the values in the tasks table, even though we don't use them
  // on this form, to avoid null values in the table
  // NOTE: when fields are added to the tasks table, they should be added here

  return true;
}
//---------------------------------------------------------------------------
bool TformRecordTaskCare::ScreenLock()
{
  // disable group boxes

  panelGroup->Enabled = false;
  editComments->ReadOnly = true;
  editComments->TabStop = false;
  TformRecordGeneric::ScreenLock();
  return true;
}
//---------------------------------------------------------------------------
bool TformRecordTaskCare::ScreenUnlock()
{
  // enable group boxes

  panelGroup->Enabled = true;
  editComments->ReadOnly = false;
  editComments->TabStop = true;
  TformRecordGeneric::ScreenUnlock();
  return true;
}
//---------------------------------------------------------------------------
bool TformRecordTaskCare::NotNewAnymore()
{
  if (initial_state["group_id"] == lookupTables->IdOf("groups", "Caregiver Support Program"))
  {
   if (optionsPermissions["table.com_health_edu"]["view"] == "true")
   {
     string sql;
     sql = "select com_health_edu.id, com_health_edu.current_id, care_topic_types.item, "
           "care_classification_types.item, com_method_types.item, com_evaluation_types.item "
           "from "
           "com_health_edu, care_classification_types, care_topic_types, com_method_types, com_evaluation_types "
           "where "
           "care_classification_types.id = com_health_edu.care_classification_id and "
           "care_topic_types.id = com_health_edu.care_topic_id and "
           "com_method_types.id = com_health_edu.com_method_id and "
           "com_evaluation_types.id = com_evaluation_id and "
           "com_health_edu.task_id = " + initial_state["id"];

     education_filter.SetSQLStatement(sql);
     education_filter.RefreshDataSet();
     education_filter.SetAddEdit(this, AddRecord(&(TformRecordTaskCare::AddEdu)),
                                       EditRecord (&(TformRecordTaskCare::EditEdu)));
     community_edu_request = atlas->RequestNotificationWhen("com_health_edu.id", initial_state["task_id"], rel_any, as_int, this,
                                 RefreshHandler(&(TformRecordTaskCare::AtlasRefreshListViews)));
   }
  }
  return true; // TODO: added because SOMEBODY forgot to have warnings turned on, and missed this. this is wrong. true is not guaranteed.
}
//---------------------------------------------------------------------------
bool TformRecordTaskCare::IsNewAgain()
{

  education_filter.SetSQLStatement(string(""));
  education_filter.RefreshDataSet();
  education_filter.SetAddEdit(this, NULL, NULL);

  TformRecordGeneric::IsNewAgain();
  return true;
}
//---------------------------------------------------------------------------
bool TformRecordTaskCare::AtlasRefreshListViews(int request_num, string matched_value)
{
  if (request_num == community_edu_request)
    education_filter.RefreshDataSet();

  return true;
}
//---------------------------------------------------------------------------
void TformRecordTaskCare::EditEdu()
{
  map<string,string> temp_map;
  string person_id = GetValidLinkId(txtPerson);
  if (person_id != "")
    temp_map["person_id"] = person_id;    // Community Health
  temp_map["group_id"] = lookupTables->IdOf("groups", "Caregiver Support Program");
  temp_map["id"] = education_filter.GetSelectedRecordID();
  FORM_TRY
    TformRecordComHealth * comHealthRecord = new TformRecordComHealth(NULL, temp_map);
    comHealthRecord->Show();
  FORM_CATCH
}
//---------------------------------------------------------------------------
void TformRecordTaskCare::AddEdu()
{
  map<string,string> temp_map;
  temp_map["id"] = "new";
  temp_map["task_id"] = initial_state["id"];
  string person_id = GetValidLinkId(txtPerson);
  if (person_id != "")
    temp_map["person_id"] = person_id;    // Community Health
  temp_map["group_id"] = initial_state["group_id"];
  FORM_TRY
    TformRecordComHealth * comHealthRecord = new TformRecordComHealth(NULL, temp_map);
    comHealthRecord->Show();
  FORM_CATCH
}
//---------------------------------------------------------------------------
void __fastcall TformRecordTaskCare::btnEventSearchClick(TObject *Sender)
{
  vector<string> return_vector;
  FORM_TRY
    TformSearchEvent * eventSearch = new TformSearchEvent(NULL,&return_vector,lookupTables->IdOf("groups", "Caregiver Support Program"));
    eventSearch->ShowTaskModal(this);
  FORM_CATCH
  if (! return_vector.empty())
    CreateLink(txtEvent, btnEventSearch, this, return_vector[0], "events");
}
//---------------------------------------------------------------------------
void __fastcall TformRecordTaskCare::btnPersonSearchClick(TObject *Sender)
{
  // create and show new person search form
  vector<string> return_vector;
  FORM_TRY
    TformSearchPerson * formPersonSearch = new TformSearchPerson(NULL,&return_vector);
    formPersonSearch->ShowTaskModal(this);
  FORM_CATCH
  if (! return_vector.empty())
    CreateLink(txtPerson, btnPersonSearch, this, return_vector[0], "people");
}
//---------------------------------------------------------------------------
bool TformRecordTaskCare::BeforeSave()
{
  bool result = true;
  if(is_new_record && (appointment_to_attach != ""))
  {
    map<string, string> field_map;
    field_map["id"] = appointment_to_attach;
    field_map["task_id"] = ending_state["id"];
    if (ExecuteUpdateStatement(lock_transaction, "appointments", field_map))
      atlas->Notify("appointments.id", field_map["id"]);
    else
    {
      result = false;
      PSay("Unable to attach appointment to task/visit.  If the appointment is still unlocked for editing, "
           "save the appointment and try again.", m_program_error, MB_OK);
    }
  }

  // tasks_to_care_services
  string sql;
  map<string, string> field_map;
  TListItem* rows;
  string id;
  try
  {
    /*vector<map<string, string> > services_to_tasks;
    sql = "select user_id from appointment_viewers where appointment_id = " + ending_state["id"];
    SqlToVecMap(lock_transaction, sql, services_to_tasks);*/

    sql = "delete from tasks_to_care_services where task_id = " + ending_state["id"];
    ExecuteStatement(lock_transaction, sql);
    for (int i = 0; i < listServices->Items->Count; i++)
    {
      rows = listServices->Items->Item[i];
      id = rows->SubItems->Strings[0].c_str();
      if (listServices->Items->Item[i]->Checked)
      {
        field_map["id"] = GetNextId(lock_transaction, "tasks_to_care_services");
        field_map["current_id"] = "4";
        field_map["last_update_stamp"] = RightNow();
        field_map["last_updated_by"] = current_userid;
        field_map["task_id"] = ending_state["id"];
        field_map["care_service_id"] = id;
        ExecuteInsertStatement(lock_transaction, "tasks_to_care_services", field_map);
      }
    }
    result = result && true; // don't ignore the false above if it occurs
  }
  catch(...)
  {
    result = false;
  }
  return result;
}
//---------------------------------------------------------------------------
void TformRecordTaskCare::Do(map<string, string> & p_parameters)
{
 FORM_TRY
  TformRecordTaskCare * z = new TformRecordTaskCare(NULL, p_parameters);
  z->Show();
 FORM_CATCH
}
//---------------------------------------------------------------------------

