//--------------------------------------------------------------------------

#include <vcl.h>
#pragma hdrstop

#include "codeRecordTaskCobVisit.h"
#include "codeRecordIncomingBill.h"
#include "codeRecordOutgoingBill.h"
#include "codeRecordCondition.h"
#include "codeRecordProcedure.h"
#include "codeSearchTaxEntity.h"
//---------------------------------------------------------------------------
#pragma package(smart_init)
#pragma link "codeRecordGeneric"
#pragma link "codeRubberContainer"
#pragma resource "*.dfm"
TformRecordTaskCobVisit *formRecordTaskCobVisit;
//---------------------------------------------------------------------------
__fastcall TformRecordTaskCobVisit::TformRecordTaskCobVisit(TComponent* Owner, map<string,string> p_parameters)
        : TformRecordGeneric(Owner, p_parameters, string("tasks"))
{
  conditions_filter.SetListViewHandle (listDiagnosis);
  conditions_filter.SetSecurity("conditions.cob");
  conditions_filter.AddColumnSpec("Index Number", "conditions", as_int, true, 120);
  conditions_filter.AddColumnSpec("Condition Type", "conditions", as_text, false, 120);
  conditions_filter.AddColumnSpec("Start Date", "conditions", as_date, false, 100);
  conditions_filter.AddColumnSpec("Stop Date", "conditions", as_date, false, 100);
  conditions_filter.AddColumnSpec("Code", "conditions", as_text, true, 100);
  conditions_filter.AddColumnSpec("Note", "conditions", as_text, true, 130, sentenceString);
  conditions_filter.AddColumnSpec("Current Id", "current_id", as_int, false, 80);
  conditions_filter.AddColumnSpec("Internal Id", "internal_id", as_int, false, 80);
  conditions_filter.FetchPreferences("recordtaskcobvisit.conditions_filter");
  conditions_filter.SetIconStatus(true);
  conditions_filter.SetSQLStatement("");
  conditions_filter.RefreshDataSet();

  procedures_filter.SetListViewHandle (listProcedures);
  procedures_filter.SetSecurity("procedures");
  procedures_filter.AddColumnSpec("Start Date", "procedures", as_date, true, 100);
  procedures_filter.AddColumnSpec("Stop Date", "procedures", as_date, false, 100);
  procedures_filter.AddColumnSpec("Place of Service", "procedures", as_text, false, 110);
  procedures_filter.AddColumnSpec("Type of Service", "procedures", as_text, false, 110);
  procedures_filter.AddColumnSpec("Cpt/Hcpcs Code", "procedures", as_text, true, 105);
  procedures_filter.AddColumnSpec("Modifier", "procedures", as_text, false, 105);
  procedures_filter.AddColumnSpec("Charges", "procedures", as_text, false, 100);
  procedures_filter.AddColumnSpec("Days/Units", "procedures", as_text, false, 100);
  procedures_filter.AddColumnSpec("EPSDT", "procedures", as_text, false, 100);
  procedures_filter.AddColumnSpec("EMG", "procedures", as_text, false, 100);
  procedures_filter.AddColumnSpec("COB", "procedures", as_text, false, 100);
  procedures_filter.AddColumnSpec("Local Use", "procedures", as_text, false, 100);
  procedures_filter.AddColumnSpec("Current Id", "current_id", as_int, false, 80);
  procedures_filter.AddColumnSpec("Internal Id", "internal_id", as_int, false, 80);
  procedures_filter.FetchPreferences("recordtaskcobvisit.procedures_filter");
  procedures_filter.SetIconStatus(true);
  procedures_filter.SetSQLStatement("");
  procedures_filter.RefreshDataSet();

  bills_filter.SetListViewHandle (listBills);
  bills_filter.SetSecurity("bills");
  bills_filter.AddColumnSpec("Received Date", "bills", as_date, true, 120);
  bills_filter.AddColumnSpec("Employee", "bills", as_text, true, 100);
  bills_filter.AddColumnSpec("Tax Entity", "tax_entities", as_text, true, 140, rawString);
  bills_filter.AddColumnSpec("Invoice Number", "tax_entities", as_text, true, 120);
  bills_filter.AddColumnSpec("Total Charge", "bills", as_double, true, 120);
  bills_filter.AddColumnSpec("Invoice Amount", "bills", as_double, true, 120);
  bills_filter.AddColumnSpec("Bill Type", "bills", as_text, true, 100);
  bills_filter.AddColumnSpec("PR Number", "purchase_reqs", as_int, true, 100);
  bills_filter.AddColumnSpec("Current Id", "current_id", as_int, false, 80);
  bills_filter.AddColumnSpec("Internal Id", "internal_id", as_int, false, 80);
  bills_filter.FetchPreferences("recordtaskcobvisit.bills_filter");
  bills_filter.SetIconStatus(true);
  bills_filter.SetSQLStatement ("");
  bills_filter.RefreshDataSet();

  Initialize();
}
//---------------------------------------------------------------------------
bool TformRecordTaskCobVisit::ScreenStart()
{
//  pageControl->ActivePageIndex = 0;
  TformRecordGeneric::ScreenStart();
  return true;
}
//---------------------------------------------------------------------------

bool TformRecordTaskCobVisit::ScreenStop()
{
  // unregister combo boxes
  TformRecordGeneric::ScreenStop();
  return true;
}
//---------------------------------------------------------------------------
bool TformRecordTaskCobVisit::IsValidOnScreen()
{
  if((initial_state["parent_task_id"] == "") || (initial_state["parent_task_id"] == "0"))
  {
    PSay("There is no COB for this visit", m_missing_data, MB_OK);
    return false;
  }
  else if((initial_state["person_id"] == "") || (initial_state["person_id"] == "0"))
  {
    PSay("Please enter a person", m_missing_data, MB_OK);
    return false;
  }
  else if(GetValidLinkId(txtTaxEntity) == "")
  {
    PSay("Please enter a tax entity", m_missing_data, MB_OK);
    btnTaxEntitySearch->SetFocus();
    return false;
  }
  else
    return true;

}
//---------------------------------------------------------------------------

bool TformRecordTaskCobVisit::PutDataOnScreen()
{
  // put the values on the form from the map initial_state
  // values may have come from GetDataFromDatabase if it's an existing record,
  // or values may have been passed to the form in this map if it's a new record

  if (initial_state["date_of_task"] != "")
    dateTask->Date = Pdate(initial_state["date_of_task"], mdy | yyyy).OutDate();
  else
    dateTask->Date = Date();
  if (initial_state["time_of_task"] != "")
    timeTask->Time = Ptime(initial_state["time_of_task"], withsec | military).OutTime();
  else
  {
    Ptime current_time(Time());
    current_time.second = 0;
    timeTask->Time = current_time.OutTime();
  }
  editReason->Text = SentenceString(initial_state["task_comments"]).c_str();
  editPatientControlNumber->Text = PrettyString(initial_state["patient_control_number"]).c_str();

  CreateLink(txtCob, NULL, this, initial_state["parent_task_id"], "tasks");
  CreateLink(txtPerson, NULL, this, initial_state["person_id"], "people");
  // need to take care of this button myself, so passing null to createlink
  CreateLink(txtTaxEntity, NULL, this, initial_state["tax_entity_id"], "tax_entities");

  if(! is_new_record)
    original_tax_id = initial_state["tax_entity_id"];
  else
    original_tax_id = "";

  if(optionsPermissions["table.tax_entities"]["view"] == "true")
  {
    if(! is_new_record)
    {
      string sql;
      list<string> bill_list;
      sql = "select bills.purchase_req_id from bills where bills.task_id = " + initial_state["id"];
      SqlToList(dbDatabase, sql, bill_list);
      for(list<string>::iterator itr = bill_list.begin(); itr != bill_list.end(); itr++)
      {
        if (((*itr) != "0") && ((*itr) != ""))
        {
          btnTaxEntitySearch->Enabled = false;
          break;
        }
      } // end for
    } // end if ! new record
  }
  else
    btnTaxEntitySearch->Enabled = false;

  TformRecordGeneric::PutDataOnScreen();
  return true;
}
//---------------------------------------------------------------------------
bool TformRecordTaskCobVisit::GetDataFromScreen()
{
  TformRecordGeneric::GetDataFromScreen();
  if (!CreateInsertMap(dbDatabase, "tasks", ending_state))
     return false;

  // place the values currently on the screen into
  // the map that will be used to update or insert the record
  // the parent form will call the insert or update function, and get the id
  // number if necessary and take care of current_id

  ending_state["id"] = initial_state["id"];
  ending_state["parent_task_id"] = initial_state["parent_task_id"];
  ending_state["person_id"] = initial_state["person_id"];
  ending_state["tax_entity_id"] = GetValidLinkId(txtTaxEntity);
  ending_state["date_of_task"] = Pdate(dateTask->Date).OutDate(mdy | yyyy);
  ending_state["time_of_task"] = Ptime(timeTask->Time).OutTime(military | withsec);
  ending_state["task_comments"] = editReason->Text.c_str();
  ending_state["patient_control_number"] = editPatientControlNumber->Text.c_str();

  ending_state["group_id"] = lookupTables->IdOf("groups", "Contract Health");

  // set the rest of the values in the tasks table, even though we don't use them
  // on this form, to avoid null values in the table
  // NOTE: when fields (foreign keys, that is) are added to the tasks table, they should be added here

  if(initial_state["user_id"] != "")
    ending_state["user_id"] = initial_state["user_id"];
  else
    ending_state["user_id"] = "0";
  ending_state["new_client"] = "0";

  ending_state["method_id"] = lookupTables->IdOf("method_types", "Visit");

/*  // No longer necessary.  CreateInsertMap takes care of default values for FKs
*/
  return true;
}
//---------------------------------------------------------------------------
bool TformRecordTaskCobVisit::ScreenLock()
{
  // disable group boxes
  panelGroup->Enabled = false;
  editReason->ReadOnly = true;
  editReason->TabStop = false;
  TformRecordGeneric::ScreenLock();
  return true;
}
//---------------------------------------------------------------------------
bool TformRecordTaskCobVisit::ScreenUnlock()
{
  // enable group boxes
  panelGroup->Enabled = true;
  editReason->ReadOnly = false;
  editReason->TabStop = true;
  TformRecordGeneric::ScreenUnlock();
  return true;
}
//---------------------------------------------------------------------------
bool TformRecordTaskCobVisit::NotNewAnymore()
{
  string sql;
  if (optionsPermissions["table.conditions.cob"]["view"] == "true")
  {
    sql = "select conditions.index_number, condition_types.item, conditions.start_date, conditions.stop_date, "
          "conditions.code, conditions.note, conditions.current_id, conditions.id "
          "from conditions, condition_types "
          "where conditions.condition_id = condition_types.id and "
          "conditions.task_id = " + initial_state["id"];
    conditions_filter.SetSQLStatement(sql);
    conditions_filter.RefreshDataSet();
    conditions_filter.SetAddEdit(this, AddRecord(&(TformRecordTaskCobVisit::AddCondition)),
                                  EditRecord(&(TformRecordTaskCobVisit::EditCondition)));
    conditions_request = atlas->RequestNotificationWhen("conditions.task_id", initial_state["id"], rel_e, as_int, this,
                              RefreshHandler(&(TformRecordTaskCobVisit::AtlasRefreshListViews)));
  }

  if (optionsPermissions["table.procedures"]["view"] == "true")
  {
    sql = "select procedures.start_date, procedures.stop_date, procedures.place_of_service, "
          "procedures.type_of_service, procedures.cpt_hcpcs, procedures.modifier, "
          "procedures.charges, procedures.days_units, procedures.epsdt, procedures.emg, procedures.cob, "
          "procedures.local_use, procedures.current_id, procedures.id "
          "from procedures "
          "where procedures.task_id = " + initial_state["id"];
    procedures_filter.SetSQLStatement(sql);
    procedures_filter.RefreshDataSet();
    procedures_filter.SetAddEdit(this, AddRecord(&(TformRecordTaskCobVisit::AddProcedure)),
                                  EditRecord(&(TformRecordTaskCobVisit::EditProcedure)));
    procedures_request = atlas->RequestNotificationWhen("procedures.task_id", initial_state["id"], rel_e, as_int, this,
                              RefreshHandler(&(TformRecordTaskCobVisit::AtlasRefreshListViews)));
  }
  
  if(optionsPermissions["table.bills"]["view"] == "true")
  {
    sql = "select bills.bill_date, users.item, tax_entities.name, bills.invoice_number, bills.total_charge, "
          "bills.invoice_amount, bill_types.item, purchase_reqs.purchase_req_number, bills.current_id, bills.id "
          "from bills, users, tax_entities, bill_types, purchase_reqs "
          "where bills.user_id = users.id and "
          "bills.actual_tax_id = tax_entities.id and "
          "bills.bill_id = bill_types.id and "
          "bills.purchase_req_id = purchase_reqs.id and "
          "bills.task_id = " + initial_state["id"];
    bills_filter.SetSQLStatement (sql);
    bills_filter.RefreshDataSet();
    bills_filter.SetAddEdit(this, NULL, EditRecord(&(TformRecordTaskCobVisit::EditBill)));
    bills_request = atlas->RequestNotificationWhen("bills.task_id", initial_state["id"], rel_e, as_int, this,
                       RefreshHandler(&(TformRecordTaskCobVisit::AtlasRefreshListViews)));
  }
  TformRecordGeneric::NotNewAnymore();
  return true;
}
//---------------------------------------------------------------------------
bool TformRecordTaskCobVisit::IsNewAgain()
{
  conditions_filter.SetSQLStatement(string(""));
  conditions_filter.RefreshDataSet();
  conditions_filter.SetAddEdit(this, NULL, NULL);
  procedures_filter.SetSQLStatement(string(""));
  procedures_filter.RefreshDataSet();
  procedures_filter.SetAddEdit(this, NULL, NULL);
  bills_filter.SetSQLStatement(string(""));
  bills_filter.RefreshDataSet();
  bills_filter.SetAddEdit(this, NULL, NULL);
  TformRecordGeneric::IsNewAgain();
  return true;  
}
//---------------------------------------------------------------------------
bool TformRecordTaskCobVisit::AtlasRefreshListViews(int request_num, string matched_value)
{
  if (conditions_request == request_num)
    conditions_filter.RefreshDataSet();
  else if (procedures_request == request_num)
    procedures_filter.RefreshDataSet();
  else if (request_num == bills_request)
    bills_filter.RefreshDataSet();
  return true;
}
//---------------------------------------------------------------------------
void TformRecordTaskCobVisit::AddCondition()
{
  // TODO: whoaaa... shouldn't we be checking some additional permissions here?  To add to the filter, they need what?
  // conditions.cob / insert?  I'm not sure that's good enough, since it's adding as a LAC condition (???) they should
  // have conditions.lac / insert also, no?
  map<string,string> temp_map;
  temp_map["id"] = "new";
  temp_map["task_id"] = initial_state["id"];
  if ((initial_state["person_id"] != "") || (initial_state["person_id"] != "0"))
    temp_map["person_id"] = initial_state["person_id"];
  if (initial_state["date_of_task"] != "")
    temp_map["start_date"] = initial_state["date_of_task"];
  temp_map["group_id"] = lookupTables->IdOf("groups", "Little Axe Clinic");
  FORM_TRY
    TformRecordCondition * conditionRecord = new TformRecordCondition(NULL, temp_map);
    conditionRecord->Show();
  FORM_CATCH
}
//---------------------------------------------------------------------------
void TformRecordTaskCobVisit::EditCondition()
{
  map<string,string> temp_map;
  temp_map["id"] = conditions_filter.GetSelectedRecordID();
  FORM_TRY
    TformRecordCondition * conditionRecord = new TformRecordCondition(NULL, temp_map);
    conditionRecord->Show();
  FORM_CATCH
}
//---------------------------------------------------------------------------
void TformRecordTaskCobVisit::AddProcedure()
{
  map<string,string> temp_map;
  temp_map["id"] = "new";
  temp_map["task_id"] = initial_state["id"];
  if (initial_state["date_of_task"] != "")
    temp_map["start_date"] = initial_state["date_of_task"];
  FORM_TRY
    TformRecordProcedure * procedureRecord = new TformRecordProcedure(NULL, temp_map);
    procedureRecord->Show();
  FORM_CATCH
}
//---------------------------------------------------------------------------
void TformRecordTaskCobVisit::EditProcedure()
{
  map<string,string> temp_map;
  temp_map["id"] = procedures_filter.GetSelectedRecordID();
  FORM_TRY
    TformRecordProcedure * procedureRecord = new TformRecordProcedure(NULL, temp_map);
    procedureRecord->Show();
  FORM_CATCH
}
//---------------------------------------------------------------------------
void TformRecordTaskCobVisit::EditBill()
{
  map<string, string> temp_map;
  temp_map["id"] = bills_filter.GetSelectedRecordID();
  string bill_id = GetOneField(dbDatabase, "select bill_id from bills where id = " + temp_map["id"]);
  if((bill_id == lookupTables->IdOf("bill_types", "incoming provider bill")) ||
     (bill_id == lookupTables->IdOf("bill_types", "incoming vendor bill")))
  {
    FORM_TRY
      TformRecordIncomingBill * in_bill = new TformRecordIncomingBill(NULL, temp_map);
      in_bill->Show();
    FORM_CATCH
  }
  else if(bill_id == lookupTables->IdOf("bill_types", "outgoing provider bill"))
  {
    FORM_TRY
      TformRecordOutgoingBill * out_bill = new TformRecordOutgoingBill(NULL, temp_map);
      out_bill->Show();
    FORM_CATCH
  }
}
//--------------------------------------------------------------------------
void __fastcall TformRecordTaskCobVisit::btnTaxEntitySearchClick(TObject *Sender)
{

// no, this feature is -already- disabled if bills are attached. wtf?

// nevermind, that was checking for purchase requests. this is still bad.

// won't restrict it, but ... warn?

  PSay("This feature may not have been correctly updating bills and transactions.\n\nHave you been using this feature much (at all)?\nIf so, please tell a programmer immediately. Data may need fixing.", m_warning, MB_OK, "Notice me!");

  // create and show new person search form
  vector<string> return_vector;
  FORM_TRY
    TformSearchTaxEntity * taxEntitySearch = new TformSearchTaxEntity(NULL,&return_vector);
    taxEntitySearch->ShowTaskModal(this);
  FORM_CATCH
  if (! return_vector.empty())
    CreateLink(txtTaxEntity, btnTaxEntitySearch, this, return_vector[0], "tax_entities");
}
//---------------------------------------------------------------------------
bool TformRecordTaskCobVisit::BeforeSave()
{
  // TODO: investigate whether or not the following is needed, as the button for doing this is disabled if any bills are attached!

  string new_tax_id = ending_state["tax_entity_id"];
  // bool to_return = true;
  if(! is_new_record)
  {
    if(original_tax_id != new_tax_id)
    {
      // TIBTransaction * dbTransaction = new TIBTransaction(NULL);
      try
      {
        string sql;
        vector<map<string, string> > bill_map;
        list<string> transaction_list;
        map<string, string> update_map;
        // dbTransaction->DefaultDatabase = dbDatabase;
        // dbTransaction->StartTransaction();
        sql = "select bills.id, bills.actual_tax_id, bills.provider_tax_id, bills.facility_tax_id, "
              "bills.group_tax_id from bills where bills.task_id = " + ending_state["id"];
        SqlToVecMap(lock_transaction /*dbTransaction*/, sql, bill_map);
        for(vector<map<string, string> >::iterator itr = bill_map.begin(); itr != bill_map.end(); itr++)
        {
          update_map.clear();
          update_map["id"] = (*itr)["id"];
          if((*itr)["actual_tax_id"] == (*itr)["provider_tax_id"])
            update_map["provider_tax_id"] = new_tax_id;
          if((*itr)["actual_tax_id"] == (*itr)["facility_tax_id"])
            update_map["facility_tax_id"] = new_tax_id;
          if((*itr)["actual_tax_id"] == (*itr)["group_tax_id"])
            update_map["group_tax_id"] = new_tax_id;
          update_map["actual_tax_id"] = new_tax_id;
          ExecuteUpdateStatement(lock_transaction /*dbTransaction*/, "bills", update_map);
          sql = "select transactions.id from transactions where transactions.bill_id = " + (*itr)["id"];
          SqlToList(lock_transaction /*dbTransaction*/, sql, transaction_list);
          for(list<string>::iterator trans_itr = transaction_list.begin(); trans_itr != transaction_list.end(); trans_itr++)
          {
            update_map.clear();
            update_map["id"] = (*trans_itr);
            update_map["tax_entity_id"] = new_tax_id;
            ExecuteUpdateStatement(lock_transaction /*dbTransaction*/, "transactions", update_map);
          }
        }

        // dbTransaction->Commit();
      }
      catch(...)
      {
        // if(dbTransaction->Active)
        //   dbTransaction->Rollback();
        // to_return = false;
        PSay("You have changed the tax entity for this record and the bills "
                                "attached to it could not be updated.\nPlease let your systems "
                                "administrator know about this right away", m_warning, MB_OK);
        return false;
      }
      // delete dbTransaction;
    }
  }
  return TformRecordGeneric::BeforeSave();
  // return to_return;
}
//---------------------------------------------------------------------------
void TformRecordTaskCobVisit::Do(map<string, string> & p_parameters)
{
 FORM_TRY
  TformRecordTaskCobVisit * z = new TformRecordTaskCobVisit(NULL, p_parameters);
  z->Show();
 FORM_CATCH
}
//---------------------------------------------------------------------------

