//---------------------------------------------------------------------------

#include <vcl.h>
#pragma hdrstop

#include "codeRecordTaskCom.h"
#include "codeSearchPerson.h"
#include "codeSearchEvent.h"
//---------------------------------------------------------------------------
#pragma package(smart_init)
#pragma link "codeRecordGeneric"
#pragma link "codeRubberContainer"
#pragma resource "*.dfm"
TformRecordTaskCom *formRecordTaskCom;
//---------------------------------------------------------------------------
__fastcall TformRecordTaskCom::TformRecordTaskCom(TComponent* Owner, map<string,string> p_parameters)
       :filter() , TformRecordGeneric(Owner, p_parameters, string("tasks"))
{
  fields_to_leave_raw.insert("task_comments");
  fields_to_leave_raw.insert("visit_comments");

  appointment_to_attach = initial_state["appointment_id"];
  Initialize();
}
//---------------------------------------------------------------------------
bool TformRecordTaskCom::ScreenStart()
{
  lookupTables->RegisterComboBox("users", cboStaff);
  lookupTables->RegisterComboBox("method_types", cboMethod);
  lookupTables->RegisterComboBox("direction_types", cboInOut);
  lookupTables->RegisterComboBox("classification_types", cboClassification);
  lookupTables->RegisterComboBox("com_meeting_types", cboMeetingType);
  lookupTables->RegisterComboBox("com_activity_types", cboActivityType);
  string task_sql = "select id, item, valid from task_purpose_types where group_id = 0 or group_id = '"
                    + lookupTables->IdOf("groups", "Community Health") + "' ";
  lookupTables->RegisterComboBox("task_purpose_types", cboTaskType, task_sql);
  
  TformRecordGeneric::ScreenStart();
  return true;
}
//---------------------------------------------------------------------------
bool TformRecordTaskCom::ScreenStop()
{
  // unregister combo box
  lookupTables->UnregisterComboBox("users", cboStaff);
  lookupTables->UnregisterComboBox("method_types", cboMethod);
  lookupTables->UnregisterComboBox("direction_types", cboInOut);
  lookupTables->UnregisterComboBox("classification_types", cboClassification);
  lookupTables->UnregisterComboBox("com_meeting_types", cboMeetingType);
  lookupTables->UnregisterComboBox("com_activity_types", cboActivityType);
  lookupTables->UnregisterComboBox("task_purpose_types", cboTaskType);

  TformRecordGeneric::ScreenStop();
  return true;
}
//---------------------------------------------------------------------------
bool TformRecordTaskCom::IsValidOnScreen()
{
  // check to make sure that the values entered in the combo box
  // corrospond to values in that look up table.
  // give the user a message box and prompt to re-enter if incorrect

  if (! lookupTables->IsInLookUpTable("users", cboStaff->Text.c_str()))
  {
    PSay("Please enter a valid staff name", m_invalid_data, MB_OK);
    cboStaff->SetFocus();
    return false;
  }
  else if(cboStaff->Text == "")
  {
    PSay("Please enter a staff name", m_invalid_data, MB_OK);
    cboStaff->SetFocus();
    return false;
  }
  else if (! IsFloat(editTimeSpent->Text.c_str()))
  {
    PSay("Please enter a valid time spent", m_invalid_data, MB_OK);
    editTimeSpent->SetFocus();
    return false;
  }
  else if (! IsFloat(editMileage->Text.c_str()))
  {
    PSay("Please enter a valid mileage", m_invalid_data, MB_OK);
    editMileage->SetFocus();
    return false;
  }
  else if (cboStaff->Text == "")
  {
    PSay("You must enter an employee name", m_missing_data, MB_OK);
    cboStaff->SetFocus();
    return false;
  }
  else if (! lookupTables->IsInLookUpTable("method_types", cboMethod->Text.c_str()))
  {
    PSay("Please enter a valid method of contact", m_invalid_data, MB_OK);
    cboMethod->SetFocus();
    return false;
  }
  else if (! lookupTables->IsInLookUpTable("direction_types", cboInOut->Text.c_str()))
  {
    PSay("Please enter either incoming or outgoing", m_invalid_data, MB_OK);
    cboInOut->SetFocus();
    return false;
  }
  else if (! lookupTables->IsInLookUpTable("task_purpose_types", cboTaskType))
  {
    PSay("Please enter a valid task type", m_invalid_data, MB_OK);
    cboTaskType->SetFocus();
    return false;
  }
  else if (! lookupTables->IsInLookUpTable("com_meeting_types", cboMeetingType->Text.c_str()))
  {
    PSay("Please enter a valid meeting type", m_invalid_data, MB_OK);
    cboMeetingType->SetFocus();
    return false;
  }
  else if (! lookupTables->IsInLookUpTable("com_activity_types", cboActivityType->Text.c_str()))
  {
    PSay("Please enter a valid activity type", m_invalid_data, MB_OK);
    cboActivityType->SetFocus();
    return false;
  }
  else if (! lookupTables->IsInLookUpTable("classification_types", cboClassification->Text.c_str()))
  {
    PSay("Please enter a valid classification", m_invalid_data, MB_OK);
    cboClassification->SetFocus();
    return false;
  }
  else if (initial_state["group_id"] == "" || StringToInt(initial_state["group_id"]) <= 0)
  {
    PSay("This task does not belong to a valid department. Please contact a programmer if this occured while creating a new task.",
         m_program_error, MB_OK);
    return false;
  }

   // jhaley 20100802: logic adapted from Diabetes task form to do special logic checks for task types
  if (GetValidLinkId(txtEvent) == "")
  {
    // no event...
    if (cboTaskType->Text == "Event Preparation" || cboTaskType->Text == "Event Teaching/Leading" ||
        cboTaskType->Text == "Event Contact")
    {
      // ...but this task type implies that the task should be attached to an event.
      PSay("This task is not attached to an event, but you have specified a 'Task Type' reserved for events.\n\n"
           "Please attach this task to the appropriate event, or select a different (or blank) Task Type.",
           m_invalid_data, MB_OK);
      cboTaskType->Show();
      cboTaskType->SetFocus();
      return false;
    }
  }
  else
  {
    // attached to an event...
    string person_id = GetValidLinkId(txtPerson);
    if(person_id == "" && cboTaskType->Text == "Event Contact")
    {
      // ...but this task type implies that a person should be specified, and one is not.
      PSay("This task is part of an event with a 'Task Type' indicating credit for contact with a client, but no client is "
           "selected.\n\nPlease search for the Client Name of the event participant, or select a different (or blank) Task "
           "Type.", m_invalid_data, MB_OK);
      btnPersonSearch->Show();
      btnPersonSearch->SetFocus();
      return false;
    }
    else if (StringToInt(person_id) > 0 &&
             (cboTaskType->Text == "Event Preparation" || cboTaskType->Text == "Event Teaching/Leading"))
    {
      // ...but this task type shouldn't have a person, and it does.
      PSay("This task is part of an event with a 'Task Type' indicating teaching/leading or preparation, but a Client Name "
           "is selected.\n\nPlease unlink the client that is selected and use the Event screen to create contacts for the "
           "appropriate participants. You can unlink the client by pressing the search button, then pressing 'Select' without "
           "performing a search (link text will change to 'None').", m_invalid_data, MB_OK);
      btnPersonSearch->Show();
      btnPersonSearch->SetFocus();
      return false;
    }
  }

  return true;
}
//---------------------------------------------------------------------------

bool TformRecordTaskCom::PutDataOnScreen()
{
  // put the values on the form from the map initial_state
  // values may have come from GetDataFromDatabase if it's an existing record,
  // or values may have been passed to the form in this map if it's a new record

  if (initial_state["group_id"] == lookupTables->IdOf("groups", "Public Health"))
    SetRecordTitle("Public Health Task");

  cboStaff->Text = PrettyString(lookupTables->ValueOf("users", initial_state["user_id"])).c_str();
  if (initial_state["date_of_task"] != "")
    dateTask->Date = Pdate(initial_state["date_of_task"], mdy | yyyy).OutDate();
  else
    dateTask->Date = Date();
  if (initial_state["time_of_task"] != "")
    timeTask->Time = Ptime(initial_state["time_of_task"], withsec | military).OutTime();
  else
  {
    Ptime current_time(Time());
    current_time.second = 0;
    timeTask->Time = current_time.OutTime();
  }
  editTimeSpent->Text = initial_state["time_spent"].c_str();
  editMileage->Text = initial_state["mileage"].c_str();
  cboMethod->Text = PrettyString(lookupTables->ValueOf("method_types", initial_state["method_id"])).c_str();
  cboInOut->Text = PrettyString(lookupTables->ValueOf("direction_types", initial_state["direction_id"])).c_str();
  cboTaskType->Text = PrettyString(lookupTables->ValueOf("task_purpose_types", initial_state["task_purpose_id"], cboTaskType)).c_str();
  cboClassification->Text = PrettyString(lookupTables->ValueOf("classification_types", initial_state["classification_id"])).c_str();
  cboMeetingType->Text = PrettyString(lookupTables->ValueOf("com_meeting_types", initial_state["com_meeting_id"])).c_str();
  cboActivityType->Text = PrettyString(lookupTables->ValueOf("com_activity_types", initial_state["com_activity_id"])).c_str();
  string comment_string = initial_state["task_comments"];
  if (UppercaseString(comment_string) == comment_string || LowercaseString(comment_string) == comment_string)
    comment_string = SentenceString(comment_string);
  editComments->Text = comment_string.c_str();

  CreateLink(txtPerson, btnPersonSearch, this, initial_state["person_id"], "people");
  CreateLink(txtEvent, btnEventSearch, this, initial_state["event_id"], "events");

  TformRecordGeneric::PutDataOnScreen();
  return true;
}
//---------------------------------------------------------------------------
bool TformRecordTaskCom::GetDataFromScreen()
{
  TformRecordGeneric::GetDataFromScreen();
  if (!CreateInsertMap(dbDatabase, "tasks", ending_state))
     return false;

  // place the values currently on the screen into
  // the map that will be used to update or insert the record
  // the parent form will call the insert or update function, and get the id
  // number if necessary and take care of current_id

  ending_state["id"] = initial_state["id"];

  string person_id = GetValidLinkId(txtPerson);
  if(StringToInt(person_id) > 0)
    ending_state["person_id"] = person_id;
  else
    ending_state["person_id"] = "0";

  string event_id = GetValidLinkId(txtEvent);
  if(StringToInt(event_id) > 0)
    ending_state["event_id"] = event_id;
  else
    ending_state["event_id"] = "0";

  ending_state["group_id"]     = initial_state["group_id"];
  ending_state["user_id"]      = lookupTables->IdOf("users", cboStaff->Text.c_str());
  ending_state["date_of_task"] = Pdate(dateTask->Date).OutDate(mdy | yyyy);
  ending_state["time_of_task"] = Ptime(timeTask->Time).OutTime(military | withsec);

  if (editTimeSpent->Text != "")
    ending_state["time_spent"] = editTimeSpent->Text.c_str();
  else
    ending_state["time_spent"] = "0";

  if (editMileage->Text != "")
    ending_state["mileage"] = editMileage->Text.c_str();
  else
    ending_state["mileage"] = "0";

  ending_state["method_id"]         = lookupTables->IdOf("method_types", cboMethod->Text.c_str());
  ending_state["direction_id"]      = lookupTables->IdOf("direction_types", cboInOut->Text.c_str());
  ending_state["task_purpose_id"]   = lookupTables->IdOf("task_purpose_types", cboTaskType->Text.c_str(), cboTaskType);
  ending_state["classification_id"] = lookupTables->IdOf("classification_types", cboClassification->Text.c_str());
  ending_state["task_comments"]     = editComments->Text.c_str();
  ending_state["com_meeting_id"]    = lookupTables->IdOf("com_meeting_types", cboMeetingType->Text.c_str());
  ending_state["com_activity_id"]   = lookupTables->IdOf("com_activity_types", cboActivityType->Text.c_str());

  // No longer necessary.  CreateInsertMap takes care of default values for FKs
  // set the rest of the values in the tasks table, even though we don't use them
  // on this form, to avoid null values in the table
  // NOTE: when fields are added to the tasks table, they should be added here

  return true;
}
//---------------------------------------------------------------------------
bool TformRecordTaskCom::ScreenLock()
{
  // disable group boxes
  panelGroup->Enabled = false;
  editComments->ReadOnly = true;
  editComments->TabStop = false;
  TformRecordGeneric::ScreenLock();
  return true;
}
//---------------------------------------------------------------------------
bool TformRecordTaskCom::ScreenUnlock()
{
  // enable group boxes
  panelGroup->Enabled = true;
  editComments->ReadOnly = false;
  editComments->TabStop = true;
  TformRecordGeneric::ScreenUnlock();
  return true;
}
//---------------------------------------------------------------------------
void __fastcall TformRecordTaskCom::btnPersonSearchClick(TObject *Sender)
{
  // create and show new person search form
  vector<string> return_vector;
  FORM_TRY
    TformSearchPerson * formPersonSearch = new TformSearchPerson(NULL,&return_vector);
    formPersonSearch->ShowTaskModal(this);
  FORM_CATCH
  if (! return_vector.empty())
    CreateLink(txtPerson, btnPersonSearch, this, return_vector[0], "people");
}
//---------------------------------------------------------------------------
void __fastcall TformRecordTaskCom::btnEventSearchClick(TObject *Sender)
{
  vector<string> return_vector;

  FORM_TRY
    TformSearchEvent * eventSearch = new TformSearchEvent(NULL,&return_vector, initial_state["group_id"]);
    eventSearch->ShowTaskModal(this);
  FORM_CATCH
  if (! return_vector.empty())
    CreateLink(txtEvent, btnEventSearch, this, return_vector[0], "events");
}
//---------------------------------------------------------------------------
bool TformRecordTaskCom::BeforeSave()
{
  bool result = true;
  if(is_new_record && (appointment_to_attach != ""))
  {
    map<string, string> field_map;
    field_map["id"] = appointment_to_attach;
    field_map["task_id"] = ending_state["id"];
    if (ExecuteUpdateStatement(lock_transaction, "appointments", field_map))
      atlas->Notify("appointments.id", field_map["id"]);
    else
    {
      result = false;
      PSay("Unable to attach appointment to task/visit.  If the appointment is still unlocked for editing, "
           "save the appointment and try again.", m_program_error, MB_OK);
    }
  }
  return result;
}
//---------------------------------------------------------------------------
void TformRecordTaskCom::Do(map<string, string> & p_parameters)
{
 FORM_TRY
  TformRecordTaskCom * z = new TformRecordTaskCom(NULL, p_parameters);
  z->Show();
 FORM_CATCH
}
//---------------------------------------------------------------------------


