//---------------------------------------------------------------------------

#include <vcl.h>
#pragma hdrstop

#include "codeRecordTaskLac.h"
#include "codeSearchEvent.h"
#include "codeSearchPerson.h"
#include "codeSearchTaxEntity.h"
#include "codeRecordCondition.h"
#include "codeRecordProcedure.h"
#include "codeRecordAppointment.h"
#include "codeRecordOutgoingBill.h"
#include "codeSpecialAppointmentToTaskAttacher.h"
//---------------------------------------------------------------------------
#pragma package(smart_init)
#pragma link "codeRecordGeneric"
#pragma link "codeRubberContainer"
#pragma link "codeFrameObservationList"
#pragma resource "*.dfm"
TformRecordTaskLac *formRecordTaskLac;
//---------------------------------------------------------------------------
__fastcall TformRecordTaskLac::TformRecordTaskLac(TComponent* Owner, map<string,string> p_parameters, string p_id)
        : TformRecordGeneric(Owner, p_parameters, string("tasks"))
{
  // new records rely on a group_id being passed to them
  //   - jhaley 20100810: but guess what? They weren't always getting one! :) See TformRecordPerson::AddLacTask.
  clinic_id = p_id;
  fields_to_leave_raw.insert("task_comments");
  fields_to_leave_raw.insert("visit_comments");

  appointments_filter.SetListViewHandle (listAppointments);
  appointments_filter.SetSecurity("appointments.people");
  appointments_filter.AddColumnSpec("Provider", "appointments", as_text, false, 100, rawString);
  appointments_filter.AddColumnSpec("Patient", "appointments", as_text, false, 100);
  appointments_filter.AddColumnSpec("Date", "appointments", as_date, true, 90/*, repeatingString*/);
  appointments_filter.AddColumnSpec("Time", "appointments", as_text, true, 80);
  appointments_filter.AddColumnSpec("Description", "appointments", as_text, true, 170, sentenceString);
  appointments_filter.AddColumnSpec("Created For", "appointments", as_text, false, 100);
  appointments_filter.AddColumnSpec("Outcome", "appointments", as_text, false, 90);
  appointments_filter.AddColumnSpec("Current Id", "current_id", as_int, false, 80);
  appointments_filter.AddColumnSpec("Internal Id", "internal_id", as_int, false, 80);
  appointments_filter.FetchPreferences("recordtasklac.appointments_filter");
  appointments_filter.SetIconStatus(true);
  appointments_filter.SetSQLStatement ("");
  appointments_filter.RefreshDataSet();

  conditions_filter.SetListViewHandle (listConditions);
  conditions_filter.SetSecurity("conditions"); // tighten later, during PutDataOnScreen...
  conditions_filter.AddColumnSpec("Index Number", "conditions", as_int, true, 120);
  conditions_filter.AddColumnSpec("Condition Type", "conditions", as_text, false, 120);
  conditions_filter.AddColumnSpec("Start Date", "conditions", as_date, false, 100);
  conditions_filter.AddColumnSpec("Stop Date", "conditions", as_date, false, 100);
  conditions_filter.AddColumnSpec("Code", "conditions", as_text, true, 100);
  conditions_filter.AddColumnSpec("Note", "conditions", as_text, true, 130, sentenceString);
  conditions_filter.AddColumnSpec("Current Id", "current_id", as_int, false, 80);
  conditions_filter.AddColumnSpec("Internal Id", "internal_id", as_int, false, 80);
  conditions_filter.FetchPreferences("recordtasklac.conditions_filter");
  conditions_filter.SetIconStatus(true);
  conditions_filter.SetSQLStatement("");
  conditions_filter.RefreshDataSet();

  procedures_filter.SetListViewHandle (listProcedures);
  procedures_filter.SetSecurity("procedures");
  procedures_filter.AddColumnSpec("Start Date", "procedures", as_date, true, 100);
  procedures_filter.AddColumnSpec("Stop Date", "procedures", as_date, false, 100);
  procedures_filter.AddColumnSpec("Place of Service", "procedures", as_text, false, 110);
  procedures_filter.AddColumnSpec("Type of Service", "procedures", as_text, false, 110);
  procedures_filter.AddColumnSpec("Cpt/Hcpcs Code", "procedures", as_text, true, 105);
  procedures_filter.AddColumnSpec("Modifier", "procedures", as_text, false, 105);
  procedures_filter.AddColumnSpec("Charges", "procedures", as_text, false, 100);
  procedures_filter.AddColumnSpec("Days/Units", "procedures", as_text, false, 100);
  procedures_filter.AddColumnSpec("EPSDT", "procedures", as_text, false, 100);
  procedures_filter.AddColumnSpec("EMG", "procedures", as_text, false, 100);
  procedures_filter.AddColumnSpec("COB", "procedures", as_text, false, 100);
  procedures_filter.AddColumnSpec("Local Use", "procedures", as_text, false, 100);
  procedures_filter.AddColumnSpec("Current Id", "current_id", as_int, false, 80);
  procedures_filter.AddColumnSpec("Internal Id", "internal_id", as_int, false, 80);
  procedures_filter.FetchPreferences("recordtasklac.procedures_filter");
  procedures_filter.SetIconStatus(true);
  procedures_filter.SetSQLStatement("");
  procedures_filter.RefreshDataSet();

  bills_filter.SetListViewHandle (listBills);
  bills_filter.SetSecurity("bills.outgoing.lac");
  bills_filter.AddColumnSpec("Date Sent", "bills", as_date, true, 110);
  bills_filter.AddColumnSpec("Insurance Company", "tax_entities", as_text, true, 140, rawString);
  bills_filter.AddColumnSpec("Total Charge", "bills", as_double, true, 120);
  bills_filter.AddColumnSpec("Invoice Amount", "bills", as_double, false, 120);
  bills_filter.AddColumnSpec("Current Id", "current_id", as_int, false, 80);
  bills_filter.AddColumnSpec("Internal Id", "internal_id", as_int, false, 80);
  bills_filter.FetchPreferences("recordtasklac.bills_filter");
  bills_filter.SetIconStatus(true);
  bills_filter.SetSQLStatement ("");
  bills_filter.RefreshDataSet();

  staticAppointmentAutoAttacher->Font->Color = link_color;
  appointment_to_attach = initial_state["appointment_id"];

  Initialize();

  if (is_new_record && !IsBlankOrZero(initial_state["user_id"]) && IsBlankOrZero(initial_state["tax_entity_id"]))
  {
    // jhaley: if user_id has been set externally for a new task, we need to get the proper tax_entity_id
    string tax_entity_sql = "select tax_entity_id from users where users.id = '" + initial_state["user_id"] + "' ";
    initial_state["tax_entity_id"] = GetOneField(dbDatabase, tax_entity_sql);
  }
}
//---------------------------------------------------------------------------
bool TformRecordTaskLac::ScreenStart()
{
  if (!is_new_record) // existing records should already have this set
  {
    // jhaley 20100810: another chicken-and-egg problem: I need this before I can initialize cboTaskType, but
    // I must initialize cboTaskType before calling generic ScreenStart or the box will remain blank.
    clinic_id = GetOneField(dbDatabase, "select group_id from tasks where id = '" + initial_state["id"] + "' ");
  }

  string task_sql = "select id, item, valid from task_purpose_types where group_id = 0 or group_id = '" + clinic_id + "' ";
  lookupTables->RegisterComboBox("task_purpose_types", cboTaskType, task_sql);

  TformRecordGeneric::ScreenStart();
  return true;
}
//---------------------------------------------------------------------------
bool TformRecordTaskLac::ScreenStop()
{
  // unregister combo boxes
  lookupTables->UnregisterComboBox("task_purpose_types", cboTaskType);
  TformRecordGeneric::ScreenStop();
  return true;
}
//---------------------------------------------------------------------------
bool TformRecordTaskLac::IsValidOnScreen()
{
  // jhaley: if the task is not attached to an event, a person_id is required.
  // When it *is* attached to an event, whether a person is required or not can depend on the task_purpose_id as
  // sorted out below, as on other task forms.
  string event_id  = GetValidLinkId(txtEvent);
  string person_id = GetValidLinkId(txtPerson);
  string tax_id    = GetValidLinkId(txtProvider);

  if (event_id == "" && person_id == "")
  {
    PSay("Please enter a patient", m_missing_data, MB_OK);
    btnPersonSearch->Show();
    btnPersonSearch->SetFocus();
    return false;
  }
  else if (tax_id == "")
  {
    // check to see if there is a user_id; if so, get the proper tax_entity_id automatically.
    // tasks created by the mailing labels report form might have user_id but not tax_entity_id, and we don't want
    // to disqualify such tasks for later editing.
    if (initial_state["user_id"] != "" && IsInt(initial_state["user_id"]))
    {
      string tax_entity_sql = "select tax_entity_id from users where users.id = '" + initial_state["user_id"] + "' ";
      tax_id = GetOneField(dbDatabase, tax_entity_sql);
      if (!IsBlankOrZero(tax_id))
        CreateLink(txtProvider, btnProviderSearch, this, tax_id, "tax_entities"); // jhaley: must place in link.
    }
    // check again.
    tax_id = GetValidLinkId(txtProvider);
    if (tax_id == "")
    {
      PSay("Please enter a provider", m_missing_data, MB_OK);
      btnProviderSearch->Show();
      btnProviderSearch->SetFocus();
      return false;
    }
  }
  else if (!lookupTables->IsInLookUpTable("task_purpose_types", cboTaskType))
  {
    PSay("Please enter a valid task type", m_missing_data, MB_OK);
    cboTaskType->Show();
    cboTaskType->SetFocus();
    return false;
  }
  else if (clinic_id == "" || StringToInt(clinic_id) <= 0)
  {
    // this should not happen anymore; group_id *must* be valid.
    PSay("This task does not belong to a valid department. If this occured while creating a new task, please contact a programmer.",
         m_program_error, MB_OK);
    return false;
  }

  // jhaley 20100802: logic adapted from Diabetes task form to do special logic checks for task types
  if (event_id == "")
  {
    // no event...
    if (cboTaskType->Text == "Event Preparation" || cboTaskType->Text == "Event Teaching/Leading" ||
        cboTaskType->Text == "Event Contact")
    {
      // ...but this task type implies that the task should be attached to an event.
      PSay("This task is not attached to an event, but you have specified a 'Task Type' reserved for events.\n\n"
           "Please attach this task to the appropriate event, or select a different (or blank) Task Type.",
           m_invalid_data, MB_OK);
      cboTaskType->Show();
      cboTaskType->SetFocus();
      return false;
    }
  }
  else
  {
    // attached to an event...
    if(person_id == "" && cboTaskType->Text == "Event Contact")
    {
      // ...but this task type implies that a person should be specified, and one is not.
      PSay("This task is part of an event with a 'Task Type' indicating credit for contact with a patient, but no patient is "
           "selected.\n\nPlease search for the Patient Name of the event participant, or select a different (or blank) Task "
           "Type.", m_invalid_data, MB_OK);
      btnPersonSearch->Show();
      btnPersonSearch->SetFocus();
      return false;
    }
    else if (StringToInt(person_id) > 0 &&
             (cboTaskType->Text == "Event Preparation" || cboTaskType->Text == "Event Teaching/Leading"))
    {
      // ..but this task type shouldn't have a person, and it does.
      PSay("This task is part of an event with a 'Task Type' indicating teaching/leading or preparation, but a Patient Name "
           "is selected.\n\nPlease unlink the patient that is selected and use the Event screen to create contacts for the "
           "appropriate participants. You can unlink the patient by pressing the search button, then pressing 'Select' without "
           "performing a search (link text will change to 'None').", m_invalid_data, MB_OK);
      btnPersonSearch->Show();
      btnPersonSearch->SetFocus();
      return false;
    }
  }

  return true;
}
//---------------------------------------------------------------------------
bool TformRecordTaskLac::PutDataOnScreen()
{
  // put the values on the form from the map initial_state
  // values may have come from GetDataFromDatabase if it's an existing record,
  // or values may have been passed to the form in this map if it's a new record
  editTimeSpent->Text = initial_state["time_spent"].c_str();
  editComments->Text = SentenceString(initial_state["task_comments"]).c_str();
  if (initial_state["date_of_task"] != "")
    dateTask->Date = Pdate(initial_state["date_of_task"], mdy | yyyy).OutDate();
  else
    dateTask->Date = Date();
  if (initial_state["time_of_task"] != "")
    timeTask->Time = Ptime(initial_state["time_of_task"], withsec | military).OutTime();
  else
  {
    Ptime current_time(Time());
    current_time.second = 0;
    timeTask->Time = current_time.OutTime();
  }
  if (initial_state["new_client"] == "1")
    checkNewPatient->Checked = true;
  else
    checkNewPatient->Checked = false;

  if (clinic_id == lookupTables->IdOf("groups", "Little Axe Clinic"))
    conditions_filter.SetSecurity("conditions.lac");
  else if (clinic_id == lookupTables->IdOf("groups", "Shawnee Clinic"))
    conditions_filter.SetSecurity("conditions.sc");

  cboTaskType->Text = PrettyString(lookupTables->ValueOf("task_purpose_types", initial_state["task_purpose_id"], cboTaskType)).c_str();

  CreateLink(txtPerson,   btnPersonSearch,   this, initial_state["person_id"],     "people");
  CreateLink(txtProvider, btnProviderSearch, this, initial_state["tax_entity_id"], "tax_entities");
  CreateLink(txtEvent,    btnEventSearch,    this, initial_state["event_id"],      "events");
  if (!is_new_record)
  {
    btnPersonSearch->Enabled = false;
    btnProviderSearch->Enabled = false;
    btnEventSearch->Enabled = false;
  }

  TformRecordGeneric::PutDataOnScreen();
  return true;
}
//---------------------------------------------------------------------------
bool TformRecordTaskLac::GetDataFromScreen()
{
  TformRecordGeneric::GetDataFromScreen();
  if (!CreateInsertMap(dbDatabase, "tasks", ending_state))
     return false;

  // place the values currently on the screen into
  // the map that will be used to update or insert the record
  // the parent form will call the insert or update function, and get the id
  // number if necessary and take care of current_id

  ending_state["id"] = initial_state["id"];

  string person_id = GetValidLinkId(txtPerson);
  if(StringToInt(person_id) > 0)
    ending_state["person_id"] = person_id;
  else
    ending_state["person_id"] = "0";

  string tax_entity_id = GetValidLinkId(txtProvider);
  // if we have a user id but no tax entity id...
  if (!IsBlankOrZero(initial_state["user_id"]) && tax_entity_id == "")
  {
    // get the proper tax entity id
    string sql = "select tax_entity_id from users where users.id = '" + initial_state["user_id"] + "' ";
    tax_entity_id = GetOneField(dbDatabase, sql);
  }
  if (tax_entity_id != "" && StringToInt(tax_entity_id) > 0)
    ending_state["tax_entity_id"] = tax_entity_id;
  else
    ending_state["tax_entity_id"] = "0"; // just for safety.

  // jhaley 20100810: this should now always be valid and is constant across this form; the previous logic here was
  // semi-redundant anyway (tested clinic_id for a value, then assigned the same value to ending_state["group_id"])
  ending_state["group_id"] = clinic_id;

  string user_id = initial_state["user_id"];
  // if we have a tax entity id but no user id...
  if (!IsBlankOrZero(tax_entity_id) && IsBlankOrZero(initial_state["user_id"]))
  {
    // get the proper user_id
    string sql = "select id from users where tax_entity_id = '" + tax_entity_id + "' ";
    user_id = GetOneField(dbDatabase, sql);
  }
  if (user_id != "" && StringToInt(user_id) > 0)
    ending_state["user_id"] = user_id;
  else
    ending_state["user_id"] = "0"; // just for safety.

  // jhaley: set event_id
  string event_id = GetValidLinkId(txtEvent);
  if (event_id != "")
    ending_state["event_id"] = event_id;
  else
    ending_state["event_id"] = "0";

  ending_state["date_of_task"] = Pdate(dateTask->Date).OutDate(mdy | yyyy);
  ending_state["time_of_task"] = Ptime(timeTask->Time).OutTime(military | withsec);
  if (editTimeSpent->Text != "")
    ending_state["time_spent"] = editTimeSpent->Text.c_str();
  ending_state["task_comments"] = editComments->Text.c_str();
  if(checkNewPatient->Checked)
    ending_state["new_client"] = "1";
  else
    ending_state["new_client"] = "0";
  ending_state["method_id"] = lookupTables->IdOf("method_types", "Visit");
  ending_state["task_purpose_id"] = lookupTables->IdOf("task_purpose_types", cboTaskType->Text.c_str(), cboTaskType);

/* No longer necessary.  CreateInsertMap takes care of default values for FKs
   set the rest of the values in the tasks table, even though we don't use them
   on this form, to avoid null values in the table
   NOTE: when fields are added to the tasks table, they should be added here
*/
  return true;
}
//---------------------------------------------------------------------------
bool TformRecordTaskLac::ScreenLock()
{
  // disable group boxes
  panelGroup->Enabled = false;
  editComments->ReadOnly = true;
  editComments->TabStop = false;
  TformRecordGeneric::ScreenLock();
  return true;
}
//---------------------------------------------------------------------------
bool TformRecordTaskLac::ScreenUnlock()
{
  // enable group boxes
  panelGroup->Enabled = true;
  editComments->ReadOnly = false;
  editComments->TabStop = true;
  TformRecordGeneric::ScreenUnlock();
  return true;
}
//---------------------------------------------------------------------------
bool TformRecordTaskLac::NotNewAnymore()
{
  frameObs->SetSource(obs_list_task, initial_state["id"]);

  string sql;

  if (optionsPermissions["table.appointments.people"]["view"] == "true")
  {
    sql = "select tax_entities.name, person_names.last_name || ', ' || person_names.first_name, "
          // "appointments.repeating_string || '|' || appointments.begin_date || '|' || appointments.end_date, " // WTF?
          "appointments.begin_date, "
          "appointments.appointment_time, appointments.description, "
          "users.item, visit_outcome_types.item, appointments.current_id, appointments.id "
          "from appointments, users, visit_outcome_types, tax_entities, people "
          "left join person_names on people.person_names_id = person_names.id "
          "where appointments.created_for = users.id and "
          "appointments.person_id = people.id and "
          "appointments.visit_outcome_id = visit_outcome_types.id and "
          "appointments.tax_entity_id = tax_entities.id and "
          "appointments.task_id = " + initial_state["id"];
    appointments_filter.SetSQLStatement (sql);
    appointments_filter.RefreshDataSet();
    appointments_filter.SetAddEdit(this, AddRecord(&(TformRecordTaskLac::AddAppointment)),
                               EditRecord(&(TformRecordTaskLac::EditAppointment)));
    appointments_request = atlas->RequestNotificationWhen("appointments.task_id", initial_state["id"], rel_e, as_int, this,
                          RefreshHandler(&(TformRecordTaskLac::AtlasRefreshListViews)));
  }

  if (optionsPermissions["table.conditions.lac"]["view"] == "true" ||
      optionsPermissions["table.conditions.sc"]["view"] == "true")
  {
    sql = "select conditions.index_number, condition_types.item, conditions.start_date, conditions.stop_date, "
          "conditions.code, conditions.note, conditions.current_id, conditions.id "
          "from conditions, condition_types "
          "where conditions.condition_id = condition_types.id and "
          "conditions.task_id = " + initial_state["id"];
    conditions_filter.SetSQLStatement(sql);
    conditions_filter.RefreshDataSet();
    conditions_filter.SetAddEdit(this, AddRecord(&(TformRecordTaskLac::AddCondition)),
                                  EditRecord(&(TformRecordTaskLac::EditCondition)));
    conditions_request = atlas->RequestNotificationWhen("conditions.task_id", initial_state["id"], rel_e, as_int, this,
                              RefreshHandler(&(TformRecordTaskLac::AtlasRefreshListViews)));
  }

  if (optionsPermissions["table.procedures"]["view"] == "true")
  {
    sql = "select procedures.start_date, procedures.stop_date, procedures.place_of_service, "
          "procedures.type_of_service, procedures.cpt_hcpcs, procedures.modifier, "
          "procedures.charges, procedures.days_units, procedures.epsdt, procedures.emg, procedures.cob, "
          "procedures.local_use, procedures.current_id, procedures.id "
          "from procedures "
          "where procedures.task_id = " + initial_state["id"];
    procedures_filter.SetSQLStatement(sql);
    procedures_filter.RefreshDataSet();
    procedures_filter.SetAddEdit(this, AddRecord(&(TformRecordTaskLac::AddProcedure)),
                                  EditRecord(&(TformRecordTaskLac::EditProcedure)));
    procedures_request = atlas->RequestNotificationWhen("procedures.task_id", initial_state["id"], rel_e, as_int, this,
                              RefreshHandler(&(TformRecordTaskLac::AtlasRefreshListViews)));
  }

  if (optionsPermissions["table.bills.outgoing.lac"]["view"] == "true")
  {
    sql = "select bills.bill_date, tax_entities.name, bills.total_charge, "
          "bills.invoice_amount, bills.current_id, bills.id "
          "from bills, tax_entities, insurance_coverage, insurance_policies "
          "where bills.insurance_coverage_id = insurance_coverage.id and "
          "insurance_coverage.insurance_policy_id = insurance_policies.id and "
          "insurance_policies.tax_entity_id = tax_entities.id and "
          "bills.task_id = " + initial_state["id"];
    bills_filter.SetSQLStatement(sql);
    bills_filter.RefreshDataSet();
    bills_filter.SetAddEdit(this, AddRecord(&(TformRecordTaskLac::AddBill)),
                                  EditRecord(&(TformRecordTaskLac::EditBill)));
    bills_request = atlas->RequestNotificationWhen("bills.task_id", initial_state["id"], rel_e, as_int, this,
                              RefreshHandler(&(TformRecordTaskLac::AtlasRefreshListViews)));
  }

  staticAppointmentAutoAttacher->Enabled = true;
  TformRecordGeneric::NotNewAnymore();
  return true;
}
//---------------------------------------------------------------------------
void TformRecordTaskLac::AddCondition()
{
  map<string,string> temp_map;
  temp_map["id"] = "new";
  temp_map["task_id"] = initial_state["id"];
  string person_id = GetValidLinkId(txtPerson);
  if (person_id != "")
    temp_map["person_id"] = person_id;
  if (initial_state["date_of_task"] != "")
    temp_map["start_date"] = initial_state["date_of_task"];
  temp_map["group_id"] = clinic_id; // jhaley - always valid now.

  FORM_TRY
    TformRecordCondition * conditionRecord = new TformRecordCondition(NULL, temp_map);
    conditionRecord->Show();
  FORM_CATCH
}
//---------------------------------------------------------------------------
void TformRecordTaskLac::EditCondition()
{
  map<string,string> temp_map;
  temp_map["id"] = conditions_filter.GetSelectedRecordID();
  FORM_TRY
    TformRecordCondition * conditionRecord = new TformRecordCondition(NULL, temp_map);
    conditionRecord->Show();
  FORM_CATCH
}
//---------------------------------------------------------------------------
bool TformRecordTaskLac::AtlasRefreshListViews(int request_num, string matched_value)
{
  if (request_num == appointments_request)
    appointments_filter.RefreshDataSet();
  else if (conditions_request == request_num)
    conditions_filter.RefreshDataSet();
  else if (procedures_request == request_num)
    procedures_filter.RefreshDataSet();
  else if (bills_request == request_num)
    bills_filter.RefreshDataSet();
  return true;
}
//---------------------------------------------------------------------------
void __fastcall TformRecordTaskLac::btnPersonSearchClick(TObject *Sender)
{
  // create and show new person search form
  vector<string> return_vector;
  FORM_TRY
    TformSearchPerson * formPersonSearch = new TformSearchPerson(NULL,&return_vector);
    formPersonSearch->ShowTaskModal(this);
  FORM_CATCH
  if (! return_vector.empty())
    CreateLink(txtPerson, btnPersonSearch, this, return_vector[0], "people");
}
//---------------------------------------------------------------------------
void __fastcall TformRecordTaskLac::btnProviderSearchClick(TObject *Sender)
{
  // create and show new person search form
  vector<string> return_vector;
  FORM_TRY
    TformSearchTaxEntity * providerSearch = new TformSearchTaxEntity(NULL,&return_vector);
    providerSearch->ShowTaskModal(this);
  FORM_CATCH
  if (! return_vector.empty())
    CreateLink(txtProvider, btnProviderSearch, this, return_vector[0], "tax_entities");
}
//---------------------------------------------------------------------------
void TformRecordTaskLac::AddProcedure()
{
  map<string,string> temp_map;
  temp_map["id"] = "new";
  temp_map["task_id"] = initial_state["id"];
  if (initial_state["date_of_task"] != "")
    temp_map["start_date"] = initial_state["date_of_task"];
  FORM_TRY
    TformRecordProcedure * procedureRecord = new TformRecordProcedure(NULL, temp_map);
    procedureRecord->Show();
  FORM_CATCH
}
//---------------------------------------------------------------------------
void TformRecordTaskLac::EditProcedure()
{
  map<string,string> temp_map;
  temp_map["id"] = procedures_filter.GetSelectedRecordID();
  FORM_TRY
    TformRecordProcedure * procedureRecord = new TformRecordProcedure(NULL, temp_map);
    procedureRecord->Show();
  FORM_CATCH
}
//---------------------------------------------------------------------------
bool TformRecordTaskLac::IsNewAgain()
{
  frameObs->SetSource(obs_list_none, "0");

  appointments_filter.SetSQLStatement(string(""));
  appointments_filter.RefreshDataSet();
  appointments_filter.SetAddEdit(this, NULL, NULL);
  conditions_filter.SetSQLStatement(string(""));
  conditions_filter.RefreshDataSet();
  conditions_filter.SetAddEdit(this, NULL, NULL);
  procedures_filter.SetSQLStatement(string(""));
  procedures_filter.RefreshDataSet();
  procedures_filter.SetAddEdit(this, NULL, NULL);
  bills_filter.SetSQLStatement(string(""));
  bills_filter.RefreshDataSet();
  bills_filter.SetAddEdit(this, NULL, NULL);
  btnPersonSearch->Enabled = true;
  btnProviderSearch->Enabled = true;
  btnEventSearch->Enabled = true;
  TformRecordGeneric::IsNewAgain();
  return true;
}
//---------------------------------------------------------------------------
void TformRecordTaskLac::AddAppointment()
{
  map<string,string> temp_map;
  temp_map["id"] = "new";
  temp_map["task_id"] = initial_state["id"];
  string person_id = GetValidLinkId(txtPerson);
  if (person_id != "")
    temp_map["person_id"] = person_id;
  string tax_entity_id = GetValidLinkId(txtProvider);
  if (tax_entity_id != "")
  {
    temp_map["tax_entity_id"] = tax_entity_id;
    temp_map["created_for"] = GetOneField(dbDatabase, "select id from users where tax_entity_id = '" + tax_entity_id + "' ");
  }
  FORM_TRY
    TformRecordAppointment * appointmentRecord = new TformRecordAppointment(NULL, temp_map);
    appointmentRecord->Show();
  FORM_CATCH
}
//---------------------------------------------------------------------------
void TformRecordTaskLac::EditAppointment()
{
  map<string,string> temp_map;
  temp_map["id"] = appointments_filter.GetSelectedRecordID();
  FORM_TRY
    TformRecordAppointment * appointmentRecord = new TformRecordAppointment(NULL, temp_map);
    appointmentRecord->Show();
  FORM_CATCH
}
//---------------------------------------------------------------------------
void TformRecordTaskLac::AddBill()
{
  map<string,string> temp_map;
  temp_map["id"] = "new";
  temp_map["task_id"] = initial_state["id"];
  temp_map["bill_date"] = initial_state["date_of_task"];
  temp_map["bill_id"] = lookupTables->IdOf("bill_types", "Outgoing provider bill");
  string tax_entity_id = GetValidLinkId(txtProvider);
  temp_map["provider_tax_id"] = tax_entity_id;
  temp_map["facility_tax_id"] = "4"; // TODO -c MAGIC NUMBER :

  FORM_TRY
    TformRecordOutgoingBill * billRecord = new TformRecordOutgoingBill(NULL, temp_map);
    billRecord->Show();
  FORM_CATCH
}
//---------------------------------------------------------------------------
void TformRecordTaskLac::EditBill()
{
  map<string,string> temp_map;
  temp_map["id"] = bills_filter.GetSelectedRecordID();
  FORM_TRY
    TformRecordOutgoingBill * billRecord = new TformRecordOutgoingBill(NULL, temp_map);
    billRecord->Show();
  FORM_CATCH
}
//---------------------------------------------------------------------------
bool TformRecordTaskLac::BeforeSave()
{ // If you change code in this function, change it on the other TformRecordTask* screens as well!

  // if task is new, auto-attach the appt to the task for the appt from which the task
  // was created (if created from an appt) when they save the task
  bool result = true;
  vector<string> IDs_to_refresh;
  if(is_new_record)
  {
    map<string, string> field_map;
    field_map["id"] = appointment_to_attach;
    field_map["task_id"] = ending_state["id"];
    if (IsInt(appointment_to_attach))
    {
      if (appointment_to_attach != "")
      {
        if(ExecuteUpdateStatement(lock_transaction, "appointments", field_map))
          atlas->Notify("appointments.id", field_map["id"]);
        else
        {
          PSay("Unable to attach appointment to task/visit.  If you have the appointment unlocked for editing, "
               "save the appointment and then try saving the task/visit again.  Otherwise, someone else may have "
               "the appointment unlocked for editing.", m_warning, MB_OK);
          result = false;
        }
      }
      if (ending_state["person_id"] != "0")
      {
        int button_pressed;
        FORM_TRY
          //PSay(ReplaceStringWithString(MapToCommaString(ending_state), ", ", "\n"), m_debug, MB_OK);
          TformSpecialAppointmentToTaskAttacher * auto_attacher =
            new TformSpecialAppointmentToTaskAttacher(NULL, ending_state["id"], ending_state["person_id"], lock_transaction, &IDs_to_refresh, true);
          button_pressed = auto_attacher->ShowTaskModal(this);
        FORM_CATCH
        if(button_pressed == mrCancel || button_pressed == mrAbort)
        {
          PSay("The visit was not saved.", m_warning, MB_OK);
          result = false;  // don't save the task if they cancel
        }
        // since using lock_transaction, add to pending atlas requests and send them once transaction is committed
        for(vector<string>::iterator itr = IDs_to_refresh.begin(); itr != IDs_to_refresh.end(); itr++)
          AtlasAdd("appointments", "id", (*itr));
      }
    }
    else
    {
      PSay("Unable to attach appointment to task/visit.  It seems that you created a new appointment and forgot "
           "to save it before clicking on the task/visit link.\n\n"
           "If this is the case, close the visit, save the appointment, and then try creating and saving the task/visit again.  "
           "Otherwise, report this error to a programmer.", m_warning, MB_OK);
      result = false;
    }
  }
  return result;
}
//---------------------------------------------------------------------------
void __fastcall TformRecordTaskLac::staticAppointmentAutoAttacherClick(TObject *Sender)
{ // If you change code in this function, change it on the other TformRecordTask* screens as well!

 // they clicked the Attach link to manually alter appointment-to-task relationships
 if(optionsPermissions["table.appointments"]["view"] == "true")
 {
   TIBTransaction * transaction_to_use;
   if(is_editing)
      transaction_to_use = lock_transaction;
   else
   {
      transaction_to_use = new TIBTransaction(NULL);
      StdTransaction(transaction_to_use, dbDatabase);
   }
   string person_id = GetValidLinkId(txtPerson);
   if (person_id != "")
   {
     vector<string> IDs_to_refresh;
     FORM_TRY
       //PSay(ReplaceStringWithString(MapToCommaString(initial_state), ", ", "\n"), m_debug, MB_OK);
       TformSpecialAppointmentToTaskAttacher * auto_attacher =
         new TformSpecialAppointmentToTaskAttacher(NULL, initial_state["id"], person_id, transaction_to_use, &IDs_to_refresh);
       auto_attacher->ShowTaskModal(this);
     FORM_CATCH
     // add to pending atlas requests and send them once transaction is committed
     AtlasAdd("appointments", "task_id", initial_state["id"]);
     for(vector<string>::iterator itr = IDs_to_refresh.begin(); itr != IDs_to_refresh.end(); itr++)
       AtlasAdd("appointments", "id", (*itr));
   }
   if(!is_editing)
   {
     if (transaction_to_use->Active)
       transaction_to_use->Commit();
     if (transaction_to_use)
       delete transaction_to_use;
     AfterSave();  // since we're not editing, this isn't going to be called by recordGeneric, so we do it manually.
   }
 }
}
//---------------------------------------------------------------------------
bool TformRecordTaskLac::AfterSave()
{ // If you change code in this function, change it on the other TformRecordTask* screens as well!
   bool result = true;
   AtlasSend();

   // if we encounter an error (result == false) or the ancestor does, either
   // way, we should return false; only if both say true do we return true:
   return result && TformRecordGeneric::AfterSave();
}
//---------------------------------------------------------------------------
void TformRecordTaskLac::Do(map<string, string> & p_parameters, string p_id)
{
 FORM_TRY
  TformRecordTaskLac * z = new TformRecordTaskLac(NULL, p_parameters, p_id);
  z->Show();
 FORM_CATCH
}
//---------------------------------------------------------------------------
// jhaley 20100810: Added to allow attachment of clinic tasks to events
void __fastcall TformRecordTaskLac::btnEventSearchClick(TObject *Sender)
{
  // create and show new event search form
  vector<string> return_vector;
  FORM_TRY
    TformSearchEvent *eventSearch = new TformSearchEvent(NULL, &return_vector, clinic_id);
    eventSearch->ShowTaskModal(this);
  FORM_CATCH
  if (!return_vector.empty())
    CreateLink(txtEvent, btnEventSearch, this, return_vector[0], "events");
}
//---------------------------------------------------------------------------

