//---------------------------------------------------------------------------

#include <vcl.h>
#pragma hdrstop

#include "codeRecordTaskTrans.h"
#include "codeSearchPerson.h"
#include "codeRecordAppointment.h"
//---------------------------------------------------------------------------
#pragma package(smart_init)
#pragma link "codeRecordGeneric"
#pragma link "codeRubberContainer"
#pragma resource "*.dfm"
TformRecordTaskTrans *formRecordTaskTrans;
//---------------------------------------------------------------------------
__fastcall TformRecordTaskTrans::TformRecordTaskTrans(TComponent* Owner, map<string,string> p_parameters)
        : TformRecordGeneric(Owner, p_parameters, string("tasks"))
{
  fields_to_leave_raw.insert("task_comments");
  fields_to_leave_raw.insert("visit_comments");

  appointments_filter.SetListViewHandle (listAppointments);
  appointments_filter.SetSecurity("appointments.people");
  appointments_filter.AddColumnSpec("Date", "appointments", as_date, true, 90/*, repeatingString*/);
  appointments_filter.AddColumnSpec("Time", "appointments", as_text, true, 80);
  appointments_filter.AddColumnSpec("Description", "appointments", as_text, true, 130, sentenceString);
  appointments_filter.AddColumnSpec("Created For", "appointments", as_text, true, 100);
  appointments_filter.AddColumnSpec("Outcome", "appointments", as_text, false, 90);  
  appointments_filter.AddColumnSpec("Current Id", "current_id", as_int, false, 80);
  appointments_filter.AddColumnSpec("Internal Id", "internal_id", as_int, false, 80);
  appointments_filter.FetchPreferences("recordtasktrans.appointments_filter");
  appointments_filter.SetIconStatus(true);
  appointments_filter.SetSQLStatement ("");
  appointments_filter.RefreshDataSet();

  appointment_to_attach = initial_state["appointment_id"];

  Initialize();
}
//---------------------------------------------------------------------------
bool TformRecordTaskTrans::ScreenStart()
{
  // register combo boxes
  lookupTables->RegisterComboBox("users", cboStaff);
  lookupTables->RegisterComboBox("transp_action_types", comboTranspAction);

  TformRecordGeneric::ScreenStart();
  return true;
}
//---------------------------------------------------------------------------

bool TformRecordTaskTrans::ScreenStop()
{
  // unregister combo boxes
  lookupTables->UnregisterComboBox("users", cboStaff);
  lookupTables->RegisterComboBox("transp_action_types", comboTranspAction);

  TformRecordGeneric::ScreenStop();
  return true;
}
//---------------------------------------------------------------------------
bool TformRecordTaskTrans::IsValidOnScreen()
{
  // check to make sure that the values entered in the combo boxe
  // corrospond to values in that look up table.
  // give the user a message box and prompt to re-enter if incorrect

  if (! lookupTables->IsInLookUpTable("users", cboStaff->Text.c_str()))
  {
    PSay("Please enter a valid staff name", m_invalid_data, MB_OK);
    cboStaff->SetFocus();
    return false;
  }
  if (! lookupTables->IsInLookUpTable("transp_action_types", comboTranspAction->Text.c_str()))
  {
    PSay("Please enter a valid action", m_invalid_data, MB_OK);
    comboTranspAction->SetFocus();
    return false;
  }
  else if (! IsFloat(editTimeSpent->Text.c_str()))
  {
    PSay("Please enter a valid time spent", m_invalid_data, MB_OK);
    editTimeSpent->SetFocus();
    return false;
  }
  else if (! IsFloat(editMileage->Text.c_str()))
  {
    PSay("Please enter a valid mileage", m_invalid_data, MB_OK);
    editMileage->SetFocus();
    return false;
  }
  else
    return true;

}
//---------------------------------------------------------------------------

bool TformRecordTaskTrans::PutDataOnScreen()
{
  // put the values on the form from the map initial_state
  // values may have come from GetDataFromDatabase if it's an existing record,
  // or values may have been passed to the form in this map if it's a new record

  cboStaff->Text = PrettyString(lookupTables->ValueOf("users", initial_state["user_id"])).c_str();
  comboTranspAction->Text = lookupTables->ValueOf("transp_action_types", initial_state["transp_action_id"]).c_str();
  if (initial_state["date_of_task"] != "")
    dateTask->Date = Pdate(initial_state["date_of_task"], global_mode).OutDate();
  else
    dateTask->Date = Date();
  if (initial_state["time_of_task"] != "")
    timeTask->Time = Ptime(initial_state["time_of_task"], global_mode).OutTime();
  else
    timeTask->Time = Time();
  editTimeSpent->Text = initial_state["time_spent"].c_str();
  editMileage->Text = initial_state["mileage"].c_str();
  editComments->Text = SentenceString(initial_state["task_comments"]).c_str();

  CreateLink(txtPerson, buttonSearchPerson, this, initial_state["person_id"], "people");

  TformRecordGeneric::PutDataOnScreen();
  return true;
}
//---------------------------------------------------------------------------
bool TformRecordTaskTrans::GetDataFromScreen()
{
  TformRecordGeneric::GetDataFromScreen();
  if (!CreateInsertMap(dbDatabase, "tasks", ending_state))
     return false;

  // place the values currently on the screen into
  // the map that will be used to update or insert the record
  // the parent form will call the insert or update function, and get the id
  // number if necessary and take care of current_id

  ending_state["id"] = initial_state["id"];
  if(StringToInt(initial_state["person_id"]) > 0)
    ending_state["person_id"] = initial_state["person_id"];
  else
    ending_state["person_id"] = "0";
  if(StringToInt(initial_state["group_id"]) > 0)
    ending_state["group_id"] = initial_state["group_id"];
  else
    ending_state["group_id"] = lookupTables->IdOf("groups", "Transportation");
  ending_state["user_id"] = lookupTables->IdOf("users", cboStaff->Text.c_str());
  ending_state["transp_action_id"] = lookupTables->IdOf("transp_action_types", comboTranspAction->Text.c_str());
  ending_state["date_of_task"] = Pdate(dateTask->Date).OutDate(global_mode);
  ending_state["time_of_task"] = Ptime(timeTask->Time).OutTime(global_mode);
  if (editTimeSpent->Text != "")
    ending_state["time_spent"] = editTimeSpent->Text.c_str();
  else
    ending_state["time_spent"] = "";
  if (editMileage->Text != "")
    ending_state["mileage"] = editMileage->Text.c_str();
  else
    ending_state["mileage"] = "";
  ending_state["task_comments"] = editComments->Text.c_str();

/*  // No longer necessary.  CreateInsertMap takes care of default values for FKs
  // set the rest of the values in the tasks table, even though we don't use them
  // on this form, to avoid null values in the table
  // NOTE: when fields are added to the tasks table, they should be added here
*/
  return true;
}
//---------------------------------------------------------------------------
bool TformRecordTaskTrans::ScreenLock()
{
  // disable group boxes
  panelGroup->Enabled = false;
  editComments->ReadOnly = true;
  editComments->TabStop = false;
  TformRecordGeneric::ScreenLock();
  return true;
}
//---------------------------------------------------------------------------
bool TformRecordTaskTrans::ScreenUnlock()
{
  // enable group boxes
  panelGroup->Enabled = true;
  editComments->ReadOnly = false;
  editComments->TabStop = true;
  TformRecordGeneric::ScreenUnlock();
  return true;
}
//---------------------------------------------------------------------------
void __fastcall TformRecordTaskTrans::btnPersonSearchClick(TObject *Sender)
{
  // create and show new person search form
  vector<string> return_vector;
  FORM_TRY
    TformSearchPerson * formPersonSearch = new TformSearchPerson(NULL,&return_vector);
    formPersonSearch->ShowTaskModal(this);
  FORM_CATCH
  if (! return_vector.empty())
  {
    initial_state["person_id"] = return_vector[0];
    CreateLink(txtPerson, buttonSearchPerson, this, initial_state["person_id"], "people");
  }
}
//---------------------------------------------------------------------------
bool TformRecordTaskTrans::NotNewAnymore()
{
  string sql;

  if (optionsPermissions["table.appointments.people"]["view"] == "true")
  {
    sql = //"select appointments.repeating_string || '|' || appointments.begin_date || '|' || appointments.end_date, " // WTF?
          "select appointments.begin_date, "
          "appointments.appointment_time, appointments.description, "
          "users.item, visit_outcome_types.item, appointments.current_id, appointments.id "
          "from appointments, users, visit_outcome_types "
          "where appointments.created_for = users.id and "
          "appointments.visit_outcome_id = visit_outcome_types.id and "
          "appointments.task_id = " + initial_state["id"];
    appointments_filter.SetSQLStatement (sql);
    appointments_filter.RefreshDataSet();
    appointments_filter.SetAddEdit(this, AddRecord(&(TformRecordTaskTrans::AddAppointment)),
                               EditRecord(&(TformRecordTaskTrans::EditAppointment)));
    appointments_request = atlas->RequestNotificationWhen("appointments.task_id", initial_state["id"], rel_e, as_int, this,
                          RefreshHandler(&(TformRecordTaskTrans::AtlasRefreshListViews)));
  }
  TformRecordGeneric::NotNewAnymore();
  return true;
}
//---------------------------------------------------------------------------
bool TformRecordTaskTrans::AtlasRefreshListViews(int request_num, string matched_value)
{
  if (appointments_request == request_num)
    appointments_filter.RefreshDataSet();
  return true;
}
//---------------------------------------------------------------------------
bool TformRecordTaskTrans::IsNewAgain()
{
  appointments_filter.SetSQLStatement(string(""));
  appointments_filter.RefreshDataSet();
  appointments_filter.SetAddEdit(this, NULL, NULL);

  TformRecordGeneric::IsNewAgain();
  return true;
}
//---------------------------------------------------------------------------
void TformRecordTaskTrans::AddAppointment()
{
  map<string,string> temp_map;
  temp_map["id"] = "new";
  temp_map["task_id"] = initial_state["id"];
  temp_map["person_id"] = initial_state["person_id"];
  temp_map["created_for"] = initial_state["user_id"];
  Pdate entered_year(DateToStr(dateTask->Date).c_str(), mdy | yyyy);
  temp_map["repeating_string"] = "0,0,0|0,0,0|0,0,0|0,0,0";
  temp_map["begin_date"] = entered_year.OutDate(ymd | yyyy);
  temp_map["end_date"] = entered_year.OutDate(ymd | yyyy);
  temp_map["single_date"] = "1";
  FORM_TRY
    TformRecordAppointment * appointmentRecord = new TformRecordAppointment(NULL, temp_map);
    appointmentRecord->Show();
  FORM_CATCH
}
//---------------------------------------------------------------------------
void TformRecordTaskTrans::EditAppointment()
{
  map<string,string> temp_map;
  temp_map["id"] = appointments_filter.GetSelectedRecordID();
  FORM_TRY
    TformRecordAppointment * appointmentRecord = new TformRecordAppointment(NULL, temp_map);
    appointmentRecord->Show();
  FORM_CATCH
}
//---------------------------------------------------------------------------
bool TformRecordTaskTrans::BeforeSave()
{
  bool result = true;
  if(is_new_record && (appointment_to_attach != ""))
  {
    map<string, string> field_map;
    field_map["id"] = appointment_to_attach;
    field_map["task_id"] = ending_state["id"];
    if (ExecuteUpdateStatement(lock_transaction, "appointments", field_map))
      atlas->Notify("appointments.id", field_map["id"]);
    else
    {
      result = false;
      PSay("Unable to attach appointment to task/visit.  If the appointment is still unlocked for editing, "
           "save the appointment and try again.", m_program_error, MB_OK);
    }
  }
  return result;
}
//---------------------------------------------------------------------------
void TformRecordTaskTrans::Do(map<string, string> & p_parameters)
{
 FORM_TRY
  TformRecordTaskTrans * z = new TformRecordTaskTrans(NULL, p_parameters);
  z->Show();
 FORM_CATCH
}
//---------------------------------------------------------------------------

