//---------------------------------------------------------------------------

#include <vcl.h>
#pragma hdrstop

#include "codeRecordTaxAddress.h"
//---------------------------------------------------------------------------
#pragma package(smart_init)
#pragma link "codeRecordGeneric"
#pragma link "codeHalfButton"
#pragma resource "*.dfm"
TformRecordTaxAddress *formRecordTaxAddress;
//---------------------------------------------------------------------------
__fastcall TformRecordTaxAddress::TformRecordTaxAddress(TComponent* Owner, map<string,string> p_parameters)
        : TformRecordGeneric(Owner, p_parameters, string("tax_addresses"))
{
 Initialize();
}
//---------------------------------------------------------------------------
bool TformRecordTaxAddress::ScreenStart()
{
  // register combo boxes
  lookupTables->RegisterComboBox("tax_address_types", cboType);
  lookupTables->RegisterComboBox("state_types", cboState);
  lookupTables->RegisterComboBox("country_types", cboCountry);

  TformRecordGeneric::ScreenStart();
  return true;
}
//---------------------------------------------------------------------------
bool TformRecordTaxAddress::ScreenStop()
{
  // unregister combo boxes
  lookupTables->UnregisterComboBox("tax_address_types", cboType);
  lookupTables->UnregisterComboBox("state_types", cboState);
  if (lookupTables->TableName(cboCounty) != "") // if this cbo box is registered
    lookupTables->UnregisterComboBox("county_types", cboCounty);
  lookupTables->UnregisterComboBox("country_types", cboCountry);

  TformRecordGeneric::ScreenStop();
  return true;
}
//---------------------------------------------------------------------------
bool TformRecordTaxAddress::IsValidOnScreen()
{
  // check to make sure that the values entered in the combo box
  // correspond to values in that look up table.
  // give the user a message box and prompt to re-enter if incorrect
  // make sure there is a link to a person, although there's not much the user
  // can do to fix that if there's not
  if (! lookupTables->IsInLookUpTable("tax_address_types", cboType->Text.c_str()))
  {
    PSay("Please enter a valid type.", m_invalid_data, MB_OK);
    cboType->SetFocus();
    return false;
  }
  if (! lookupTables->IsInLookUpTable("state_types", cboState->Text.c_str()))
  {
    PSay("Please enter a valid state.", m_invalid_data, MB_OK);
    cboState->SetFocus();
    return false;
  }
  else if (cboCounty->Text != "" && !lookupTables->IsInLookUpTable("county_types", cboCounty))
  {
    PSay("The county you entered does not appear within the list.  Please enter a valid county.", m_invalid_data, MB_OK);
    cboCounty->SetFocus();
    return false;
  }
  else if (! lookupTables->IsInLookUpTable("country_types", cboCountry->Text.c_str()))
  {
    PSay("Please enter a valid country.", m_invalid_data, MB_OK);
    cboCountry->SetFocus();
    return false;
  }
  else if ((initial_state["tax_entity_id"] == "") || (initial_state["tax_entity_id"] == "0"))
  {
    PSay("There is no tax entity associated with this record.", m_missing_data, MB_OK);
    return false;
  }
  else
    return true;
}
//---------------------------------------------------------------------------
bool TformRecordTaxAddress::PutDataOnScreen()
{
  // put the values on the form from the map initial_state
  // values may have come from GetDataFromDatabase if it's an existing record,
  // or values may have been passed to the form in this map if it's a new record

  cboType->Text = lookupTables->ValueOf("tax_address_types", initial_state["tax_address_id"]).c_str();
  editLine1->Text = FormatAddress(initial_state["line1"]).c_str();
  editLine2->Text = FormatAddress(initial_state["line2"]).c_str();
  cboCity->Text = PrettyString(initial_state["city"]).c_str();
  editZip->Text = FormatZipCode(initial_state["zip"]).c_str();
  cboState->Text = lookupTables->ValueOf("state_types", initial_state["state_id"]).c_str();
  string state_id, county_sql;
  if (!CountyLookupByState(cboState->Text.c_str(), state_id, county_sql, cboCounty)) /* whoops? */ ;
  cboCounty->Text = PrettyString(lookupTables->ValueOf("county_types", initial_state["county_id"], cboCounty)).c_str();
  cboCountry->Text = PrettyString(lookupTables->ValueOf("country_types", initial_state["country_id"])).c_str();
  editPhone->Text = FormatPhoneNumber(initial_state["phone_number"]).c_str();
  editFax->Text = FormatPhoneNumber(initial_state["fax_number"]).c_str();
  editWebsite->Text = LowercaseString(initial_state["website"]).c_str();
  editEmail->Text = LowercaseString(initial_state["email_address"]).c_str();
  editNotes->Text = SentenceString(initial_state["notes"]).c_str();

  CreateLink(txtTaxEntity, NULL, this, initial_state["tax_entity_id"], "tax_entities");

  if (is_new_record && cboCountry->Text == "")
     cboCountry->Text = "United States";

  TformRecordGeneric::PutDataOnScreen();
  return true; // what can really go wrong?
}
//---------------------------------------------------------------------------
bool TformRecordTaxAddress::GetDataFromScreen()
{
  // place the values currently on the screen into
  // the map that will be used to update or insert the record
  // the parent form will call the insert or update function, and get the id
  // number if necessary and take care of current_id

  ending_state["tax_address_id"] = lookupTables->IdOf("tax_address_types", cboType->Text.c_str());
  ending_state["line1"] = FormatAddress(editLine1->Text.c_str());
  ending_state["line2"] = FormatAddress(editLine2->Text.c_str());
  ending_state["city"] = cboCity->Text.c_str();
  ending_state["state_id"] = lookupTables->IdOf("state_types", cboState->Text.c_str());
  ending_state["zip"] = StripAllExcept(editZip->Text.c_str(), ints);
  ending_state["county_id"] = lookupTables->IdOf("county_types", cboCounty); // need to pass the cbo box so it can look up the ID since it's got custom sql / item list
  ending_state["country_id"] = lookupTables->IdOf("country_types", cboCountry->Text.c_str());
  ending_state["phone_number"] = StripAllExcept(editPhone->Text.c_str(), uints);
  ending_state["fax_number"] = StripAllExcept(editFax->Text.c_str(), uints);
  ending_state["website"] = editWebsite->Text.c_str();
  ending_state["email_address"] = editEmail->Text.c_str();
  ending_state["notes"] = editNotes->Text.c_str();
  ending_state["tax_entity_id"] = initial_state["tax_entity_id"];
  ending_state["id"] = initial_state["id"];

  TformRecordGeneric::GetDataFromScreen();

  return true; // again, not much can go wrong here
}
//---------------------------------------------------------------------------
bool TformRecordTaxAddress::ScreenLock()
{
  // disable panel containing edit boxes
  panelGroup->Enabled = false;
  editNotes->ReadOnly = true;
  editNotes->TabStop = false;
  TformRecordGeneric::ScreenLock();
  return true;
}
//---------------------------------------------------------------------------
bool TformRecordTaxAddress::ScreenUnlock()
{
  // enable panel containing edit boxes
  panelGroup->Enabled = true;
  editNotes->ReadOnly = false;
  editNotes->TabStop = true;
  TformRecordGeneric::ScreenUnlock();
  return true;
}
//---------------------------------------------------------------------------
void __fastcall TformRecordTaxAddress::btnZipLookupClick(TObject *Sender)
{
/*  string zip = string(editZip->Text.c_str()).substr(0,5);
  vector<map<string, string> > zip_vec;
  string sql = "select * from zip_codes_view where zip_code = '" + zip + "'";
  SqlToVecMap(dbDatabase, sql, zip_vec);
  if (zip_vec.size() > 0)
  {
    // state and county will be the same for a given zip code, so just pick the first
    cboState->Text = PrettyString(zip_vec[0]["state_name"]).c_str();
    string state_id, county_sql;
    if (!CountyLookupByState(cboState->Text.c_str(), state_id, county_sql, cboCounty));
    cboCounty->Text = zip_vec[0]["county"].c_str();
    // city though might have some options, so let's loop through and populate the cbo box
    cboCity->Items->Clear();
    for (unsigned int i=0; i < zip_vec.size(); i++)
    {
      cboCity->Items->Add(zip_vec[i]["city"].c_str());
      if (zip_vec[i]["zip_code_type"] == "Preferred")
        cboCity->Text = zip_vec[i]["city"].c_str();
    }
  }
  else
*/
  string zip_code = editZip->Text.c_str(), state_name, county_name, preferred_city_name;
  vector<map<string, string> > zip_vec;
  if (!ZipLookup(zip_code, zip_vec, state_name, county_name, preferred_city_name, cboState, cboCounty, cboCity))
    PSay("Zip code not found.  Please check that it is in either 5 digit or 5-4 digit format, and that the number is correct.  If you are sure that this zip code is correct and continue to get this error, contact a programmer.", m_invalid_data, MB_OK);
}
//---------------------------------------------------------------------------
void __fastcall TformRecordTaxAddress::cboStateChange(TObject *Sender)
{
  string state_id, county_sql;
  if (!CountyLookupByState(cboState->Text.c_str(), state_id, county_sql, cboCounty)) /* whoops */ ;
}
//---------------------------------------------------------------------------
void __fastcall TformRecordTaxAddress::cboStateExit(TObject *Sender)
{
  string state_id, county_sql;
  if (!CountyLookupByState(cboState->Text.c_str(), state_id, county_sql, cboCounty)) /* whoops */ ;
}
//---------------------------------------------------------------------------
void __fastcall TformRecordTaxAddress::editZipKeyPress(TObject *Sender,char &Key)
{
  if (Key == VK_RETURN)
    btnZipLookup->Click();
}
//---------------------------------------------------------------------------
void __fastcall TformRecordTaxAddress::btnWebsiteClick(TObject *Sender)
{
 string temp = LowercaseString(StripTrailing(StripLeading(editWebsite->Text.c_str(), " "), " "));
 if(temp.substr(0, 7) != "http://" && temp.substr(0, 8) != "https://")
   temp = "http://" + temp;
 AnsiString x = temp.c_str();
 ShellExecute(0, "open", static_cast<const char*>(x.data()), NULL, NULL, SW_SHOW);
}
//---------------------------------------------------------------------------
void __fastcall TformRecordTaxAddress::btnEmailClick(TObject *Sender)
{
 AnsiString x = "mailto:" + editEmail->Text;
 ShellExecute(0, "open", static_cast<const char*>(x.data()), NULL, NULL, SW_SHOW);
}
//---------------------------------------------------------------------------
void __fastcall TformRecordTaxAddress::editEmailKeyPress(TObject *Sender,char &Key)
{
  if (Key == VK_RETURN)
  {
    btnEmail->Click();
  }
}
//---------------------------------------------------------------------------
void __fastcall TformRecordTaxAddress::editWebsiteKeyPress(TObject *Sender,char &Key)
{
  if (Key == VK_RETURN)
  {
    btnWebsite->Click();
  }
}
//---------------------------------------------------------------------------
void TformRecordTaxAddress::Do(map<string, string> & p_parameters)
{
 FORM_TRY
  TformRecordTaxAddress * z = new TformRecordTaxAddress(NULL, p_parameters);
  z->Show();
 FORM_CATCH
}
//---------------------------------------------------------------------------

