//---------------------------------------------------------------------------

#include <vcl.h>
#pragma hdrstop

#include "codeSearchAccount.h"
#include "codeRecordAccount.h"
#include "codeWizardNewAccount.h"
//---------------------------------------------------------------------------
#pragma package(smart_init) 
#pragma link "codeSearchGeneric"
#pragma link "codeRubberContainer"
#pragma resource "*.dfm"
TformSearchAccount *formSearchAccount;
//---------------------------------------------------------------------------
__fastcall TformSearchAccount::TformSearchAccount(TComponent* Owner, vector<string> * p_return_vector)
        : TformSearchGeneric(Owner, p_return_vector, string ("accounts"))
{
 Initialize();
}
//---------------------------------------------------------------------------
bool TformSearchAccount::IsValidOnScreen()
{
 bool answer = true;

 if (radioByMask->Checked)
 {
  // bah, let it return nothing if they typed a -stuuupid- number in. not so below, with combos:
 } else
 {
  if (comboOne->Text != "" && !(lookupTables->IsInLookUpTable("acc_level_one", comboOne->Text.c_str())))
  {
   PSay("The 'fund' level must either be left entirely blank, or be one of the available selections.", m_invalid_data, MB_OK);
   answer = false;
  }
  if (comboTwo->Text != "" && !(lookupTables->IsInLookUpTable("acc_level_two", comboTwo->Text.c_str())))
  {
   PSay("The 'general ledger' level must either be left entirely blank, or be one of the available selections.", m_invalid_data, MB_OK);
   answer = false;
  }
  if (comboThree->Text != "" && !(lookupTables->IsInLookUpTable("acc_level_three", comboThree->Text.c_str())))
  {
   PSay("The 'department' level must either be left entirely blank, or be one of the available selections.", m_invalid_data, MB_OK);
   answer = false;
  }
  if (comboFour->Text != "" && !(lookupTables->IsInLookUpTable("acc_level_four", comboFour->Text.c_str())))
  {
   PSay("The 'sub-department' level must either be left entirely blank, or be one of the available selections.", m_invalid_data, MB_OK);
   answer = false;
  }
 }
 if (comboAccountType->Text != "" && !(lookupTables->IsInLookUpTable("account_types", comboAccountType->Text.c_str())))
 {
  PSay("The 'account type' must either be left entirely blank, or be one of the available selections.", m_invalid_data, MB_OK);
  answer = false;
 }
 if (comboBillCycle->Text != "" && !(lookupTables->IsInLookUpTable("month_types", comboBillCycle->Text.c_str())))
 {
  PSay("The 'cycle' must either be left entirely blank, or be one of the available selections.", m_invalid_data, MB_OK);
  answer = false;
 }

 if (!TformSearchGeneric::IsValidOnScreen())
  answer = false;

 return answer;
}
//---------------------------------------------------------------------------
bool TformSearchAccount::Search()
{
 string              sql;
 map<string, string> requirements;

 if (radioByMask->Checked)
 {
  bool is_exact = checkIsExactMatch->Checked;
  map<string, string> account_parts = PartsByMask (maskAccount);

  for (map<string, string>::iterator i_part = account_parts.begin(); i_part != account_parts.end(); i_part++)
   if (is_exact || i_part->second.find_first_not_of ("0") != string::npos)
    requirements.insert(pair<string, string>("acc_level_" + i_part->first + ".short_item","'" + i_part->second + "'"));
 } else
 {
  if (comboOne->Text != "")
   requirements.insert(pair<string, string>("acc_level_one.item","'" + string(comboOne->Text.c_str()) + "'"));
  if (comboTwo->Text != "")
   requirements.insert(pair<string, string>("acc_level_two.item","'" + string(comboTwo->Text.c_str()) + "'"));
  if (comboThree->Text != "")
   requirements.insert(pair<string, string>("acc_level_three.item","'" + string(comboThree->Text.c_str()) + "'"));
  if (comboFour->Text != "")
   requirements.insert(pair<string, string>("acc_level_four.item","'" + string(comboFour->Text.c_str()) + "'"));
 }

 if (comboAccountType->Text != "")
  requirements.insert(pair<string, string>("accounts.account_type_id", lookupTables->IdOf("account_types", comboAccountType->Text.c_str())));

 if (comboBillCycle->Text != "")
  requirements.insert(pair<string, string>("accounts.bill_cycle_id", lookupTables->IdOf("month_types", comboBillCycle->Text.c_str())));

 if (checkIsBudget->State != cbGrayed)
  requirements.insert(pair<string, string>("accounts.is_budget", checkIsBudget->Checked ? "1" : "0"));

 if (checkIsClosed->State != cbGrayed)
  requirements.insert(pair<string, string>("accounts.is_closed", checkIsClosed->Checked ? "1" : "0"));

 if (checkIsYearly->State != cbGrayed)
  requirements.insert(pair<string, string>("accounts.is_yearly", checkIsYearly->Checked ? "1" : "0"));

 sql = "select accounts.name, acc_level_one.item, acc_level_two.item, acc_level_three.item, acc_level_four.item, "
       "accounts.is_budget, accounts.is_closed, accounts.is_yearly, month_types.item, account_types.item, accounts.current_id, accounts.id "
       "from accounts inner join month_types on month_types.id = accounts.bill_cycle_id inner join account_types on account_types.id = accounts.account_type_id "
       "inner join acc_level_one on acc_level_one.id = accounts.acc_level_one_id "
       "inner join acc_level_two on acc_level_two.id = accounts.acc_level_two_id "
       "inner join acc_level_three on acc_level_three.id = accounts.acc_level_three_id "
       "inner join acc_level_four on acc_level_four.id = accounts.acc_level_four_id "
       "where accounts.id > 0 ";
 for (map<string, string>::iterator i_req = requirements.begin(); i_req != requirements.end(); i_req++)
  sql += "and " + i_req->first + " = " + i_req->second + " ";
 sql += "order by accounts.name";

 Filter->SetSQLStatement (sql);
 Filter->RefreshDataSet ();

 return true;
}
//---------------------------------------------------------------------------
bool TformSearchAccount::ScreenStart()
{
 MaskSetup (maskAccount);

 lookupTables->RegisterComboBox("acc_level_one",   comboOne);
 lookupTables->RegisterComboBox("acc_level_two",   comboTwo);
 lookupTables->RegisterComboBox("acc_level_three", comboThree);
 lookupTables->RegisterComboBox("acc_level_four",  comboFour);
 lookupTables->RegisterComboBox("account_types",   comboAccountType);
 lookupTables->RegisterComboBox("month_types",     comboBillCycle);

 Filter->SetSecurity("accounts");
 Filter->AddColumnSpec("Account Number", "accounts", as_text, true, 150);
 Filter->AddColumnSpec("Fund", "accounts", as_text, false, 75);
 Filter->AddColumnSpec("General Ledger", "accounts", as_text, false, 100);
 Filter->AddColumnSpec("Department", "accounts", as_text, false, 75);
 Filter->AddColumnSpec("Sub-department", "accounts", as_text, false, 75);
 Filter->AddColumnSpec("Budget", "accounts", as_text, true, 60, prettyBoolString);
 Filter->AddColumnSpec("Disabled", "accounts", as_text, true, 60, prettyBoolString);
 Filter->AddColumnSpec("Yearly", "accounts", as_text, true, 60, prettyBoolString);
 Filter->AddColumnSpec("Cycle", "accounts", as_text, true, 100);
 Filter->AddColumnSpec("Type", "accounts", as_text, true, 100);
 Filter->AddColumnSpec("Current Id", "current_id", as_int, false, 80);
 Filter->AddColumnSpec("Internal Id", "internal_id", as_int, false, 80);
 Filter->FetchPreferences("searchaccount.filter");
 Filter->SetIconStatus(true);
 Filter->SetSQLStatement("");
 Filter->RefreshDataSet();

 TformSearchGeneric::ScreenStart();
 return true;
}
//---------------------------------------------------------------------------
bool TformSearchAccount::ScreenStop()
{
 lookupTables->UnregisterComboBox("acc_level_one",   comboOne);
 lookupTables->UnregisterComboBox("acc_level_two",   comboTwo);
 lookupTables->UnregisterComboBox("acc_level_three", comboThree);
 lookupTables->UnregisterComboBox("acc_level_four",  comboFour);
 lookupTables->UnregisterComboBox("account_types",   comboAccountType);
 lookupTables->UnregisterComboBox("month_types",     comboBillCycle);

 TformSearchGeneric::ScreenStop();
 return true;
}
//---------------------------------------------------------------------------
bool TformSearchAccount::Create()
{
 FORM_TRY
  TformWizardNewAccount * wiz = new TformWizardNewAccount(NULL);
  wiz->Show();
 FORM_CATCH
 return true;
}
//---------------------------------------------------------------------------
bool TformSearchAccount::Zoom(const string & p_id)
{
 map<string, string> temp_map;
 temp_map["id"] = p_id;
 FORM_TRY
  TformRecordAccount * accountRecord = new TformRecordAccount(NULL, temp_map);
  accountRecord->Show();
 FORM_CATCH

 return true;
}
//---------------------------------------------------------------------------
vector<string> TformSearchAccount::Do(TformSpecialGeneric * p_parent)
{
 vector<string> answer;
 FORM_TRY
  TformSearchAccount * z = new TformSearchAccount(NULL, &answer);
  z->ShowTaskModal(p_parent);
 FORM_CATCH
 return answer;
}
//---------------------------------------------------------------------------

