//---------------------------------------------------------------------------

#include <vcl.h>
#pragma hdrstop
 
#include "codeSearchAppointment.h"
#include "codeRecordAppointment.h"
#include "launchDiabRegistryReport.h"
#include "scheduling.h"
#include "derivedheaders.h"
//---------------------------------------------------------------------------
#pragma package(smart_init)
#pragma link "codeSearchGeneric"
#pragma link "codeRubberContainer"
#pragma link "codeBaseDropDown"
#pragma link "codeCheckListDropDown"
#pragma resource "*.dfm"
TformSearchAppointment *formSearchAppointment;
//---------------------------------------------------------------------------
__fastcall TformSearchAppointment::TformSearchAppointment(TComponent* Owner, vector<string> * p_return_vector)
        : TformSearchGeneric(Owner, p_return_vector, string ("appointments"))
{
  Initialize();
}
//---------------------------------------------------------------------------
bool TformSearchAppointment::IsValidOnScreen()
{
  // check to make sure that the values entered in the combo boxes
  // corrospond to values in that look up table.
  // give the user a message box and prompt to re-enter if incorrect
  if (! lookupTables->IsInLookUpTable("users", cboFor->Text.c_str()))
  {
    PSay("Please enter a valid employee that the appointment was created for", m_invalid_data, MB_OK);
    cboFor->SetFocus();
    return false;
  }
  else if (! lookupTables->IsInLookUpTable("users", cboBy->Text.c_str()))
  {
    PSay("Please enter a valid employee that the appointment was created by", m_invalid_data, MB_OK);
    cboBy->SetFocus();
    return false;
  }
  else if (! lookupTables->IsInLookUpTable("groups", cboDepartment->Text.c_str()))
  {
    PSay("Please enter a valid department", m_invalid_data, MB_OK);
    cboDepartment->SetFocus();
    return false;
  }
  else if (! lookupTables->IsInLookUpTable("appt_reason_types", cboReason))
  {
    PSay("Please enter a valid reason", m_invalid_data, MB_OK);
    cboReason->SetFocus();
    return false;
  }
  else if (UppercaseString(cboReason->Text.c_str()) == "BLOCK SCHEDULE" && checkNoBlocks->Checked)
  {
    // jhaley: not really harmful, but potentially confusing.
    PSay("Your selection of reason is inconsistent with the Block Schedule exclusion.\n\n"
         "Please either select a different reason or clear the check box.", m_invalid_data, MB_OK);
    checkNoBlocks->SetFocus();
    return false;
  }
  else if (! lookupTables->IsInLookUpTable("appt_location_types", cboLocation))
  {
    PSay("Please enter a valid location", m_invalid_data, MB_OK);
    cboLocation->SetFocus();
    return false;
  }
  else
    return true;
}
//---------------------------------------------------------------------------
bool TformSearchAppointment::Search()
{
  try
  {
    ExecuteStatement(dbTransaction, "delete from appointment_searches");
    string sql;
    PApptOccurrenceMap all_appointments;
    vector<string> user_ids;
    if(cboFor->Text != "")
      user_ids.push_back(lookupTables->IdOf("users", cboFor->Text.c_str()));
    else
      SqlToVec(dbTransaction, "select id from users where (valid = 1 or is_role = 1) and id > 0", user_ids);
    PApptIDMapVec ids_to_match;
    vector<string> outcomes = cbddOutcome->Filter->GetMultiID();
    if(!outcomes.empty())
      ids_to_match["visit_outcome_id"].push_back(VecToCommaString(outcomes));
    if(cboReason->Text != "")
      ids_to_match["appt_reason_id"].push_back(lookupTables->IdOf("appt_reason_types", cboReason));
    if(cboLocation->Text != "")
      ids_to_match["appt_location_id"].push_back(lookupTables->IdOf("appt_location_types", cboLocation));
    Pdate begin_date;
    begin_date.InDate(dateBegin->Date);
    Pdate end_date;
    end_date.InDate(dateEnd->Date);
    DownloadAppointments(dbDatabase, begin_date, end_date, current_userid, user_ids, vector<string>(), "",
                         all_appointments, ids_to_match,
                         (optionsPermissions["admin.all_appointments"]["enabled"] == "true"), false, "");
    pmapstrs insert_map;
    for(PApptOccurrenceMap::iterator itr = all_appointments.begin(); itr != all_appointments.end(); itr++)
    {
      for(PApptOccurrenceSet::iterator set_itr = itr->second.begin(); set_itr != itr->second.end(); set_itr++)
      {
        insert_map["id"] = (*set_itr).id;
        insert_map["appt_date"] = itr->first.OutDate(mdy | yyyy);
        insert_map["appt_time"] = (*set_itr).range_begin.OutTime(military | withsec);
        ExecuteInsertStatement(dbTransaction, "appointment_searches", insert_map);
      }
    }

    set<string> can_see_charts = GetSpecialGroupPermissions("charts");
    sql = "select appointment_searches.appt_date, appointment_searches.appt_time, "
          "person_names.first_name || ' ' || person_names.last_name || ' ' || person_names.suffix, "
          "tax_entities.name, created_for_user.item, created_by_user.item, "
          "appt_reason_types.item, appt_location_types.item, "
          "appointments.description, visit_outcome_types.item, appointments.within_24_hours, "
          "(select first 1 charts.chart_number from charts "
           "inner join groups on charts.location_id = groups.chart_location_id and groups.has_charts = 1 and charts.location_id > 0 "
           "where charts.person_id = people.id and " + SafeInClause("groups.id", SetToVec(can_see_charts)) + " "
            "and charts.current_id+1 > 3 order by groups.hl7_priority asc nulls last) chart_num, "
          "appointments.current_id, appointments.id "
          "from appointment_searches "
          "inner join appointments on appointment_searches.id = appointments.id "
          "left join people on appointments.person_id = people.id "
          "left join person_names on people.person_names_id = person_names.id "
          "left join tax_entities on appointments.tax_entity_id = tax_entities.id "
          "left join users created_for_user on appointments.created_for = created_for_user.id "
          "left join users created_by_user on appointments.created_by = created_by_user.id "
          "left join appt_reason_types on appointments.appt_reason_id = appt_reason_types.id "
          "left join appt_location_types on appointments.appt_location_id = appt_location_types.id "
          "left join visit_outcome_types on appointments.visit_outcome_id = visit_outcome_types.id ";

    vector<string> where_clause;
    if (cboBy->Text != "")
      where_clause.push_back("appointments.created_by = " + lookupTables->IdOf("users", cboBy->Text.c_str()) + " ");
    if (editDescription->Text != "")
      where_clause.push_back("appointments.description like '%" + SafeSQLString(editDescription->Text.c_str()) + "%' ");
    if (checkNoTask->Checked)
      where_clause.push_back("(appointments.task_id = 0 or appointments.task_id is null) ");
    if (check24Hours->Checked)
      where_clause.push_back("appointments.within_24_hours = 1 ");
    if (cboDepartment->Text != "")
      where_clause.push_back("appointments.group_id = " + lookupTables->IdOf("groups", cboDepartment) + " ");
    if (checkNoBlocks->Checked) // jhaley 20110720
      where_clause.push_back("appointments.appt_reason_id <> " + lookupTables->IdOf("appt_reason_types", "Block Schedule"));

    if (!where_clause.empty())
      sql += ("where " + VecToDelimString(where_clause, "", "", " and "));

    DebugSql(this, &sql, other_forms, 9, dbTransaction);
    Filter->SetSQLStatement(sql);
    Filter->RefreshDataSet();
  }
  catch(...)
  {
    PSay("An error occurred while trying to search for appointments.", m_program_error, MB_OK);
    return false; // warning: unchecked return value.
  }
  return true;
}
//---------------------------------------------------------------------------
bool TformSearchAppointment::ScreenStart()
{
  lookupTables->RegisterComboBox("users", cboFor);
  lookupTables->RegisterComboBox("users", cboBy);
  //set<string> can_view = GetGroupPermissions("table.appointments", "", "view", "true"); // unnecessarily restrictive, given appointment_viewers?
  string group_sql = "select id, item, valid from groups where has_appointments = 1 or id = 0 order by item";
                     //+ SafeInClause("groups.id", SetToVec(can_view));
  lookupTables->RegisterComboBox("groups", cboDepartment, group_sql);

  string custom_reason_sql   = "select id, item, valid from appt_reason_types order by item";
  string custom_location_sql = "select id, item, valid from appt_location_types order by item";

  lookupTables->RegisterComboBox("appt_reason_types",   cboReason,   custom_reason_sql);
  lookupTables->RegisterComboBox("appt_location_types", cboLocation, custom_location_sql);

  cbddOutcome->Filter->SetSecurity("");
  cbddOutcome->Filter->AddColumnSpec("Outcome", "", as_text, true, 180);
  cbddOutcome->Filter->AddColumnSpec("Current Id", "current_id", as_int, false, 40);
  cbddOutcome->Filter->AddColumnSpec("Internal Id", "internal_id", as_int, false, 60);
  cbddOutcome->Filter->SetSQLStatement("select item, " + CleverCurrentId("valid", "1", "Current", "Expired") + ", id from visit_outcome_types where id > 0");
  cbddOutcome->Filter->FetchPreferences("search_appointment.outcome");
  //cbddOutcome->Filter->ShowHeaders(false);
  cbddOutcome->Filter->SetIconStatus(true);
  cbddOutcome->Filter->ForceRefreshDataSet();
  cbddOutcome->DisplayFieldName = "Outcome";
  cbddOutcome->DropDownHeight = 145;

  if(optionsPermissions["table.tasks"]["view"] != "true")
    checkNoTask->Enabled = false;

  Pdate begin_date(DateToStr(Date()).c_str(), mdy | yyyy);
  begin_date.mod(-14, in_days);
  dateBegin->Date = StrToDate(begin_date.OutDate(mdy | yyyy).c_str());
  begin_date.mod(28, in_days);
  dateEnd->Date = StrToDate(begin_date.OutDate(mdy | yyyy).c_str());

  dbTransaction = new TIBTransaction(NULL);
  dbTransaction->DefaultDatabase = dbDatabase;
  dbTransaction->StartTransaction();
  /* Here's the thing.  We already do this at the beginning of the Search() function, so ...
     doing it here also seems unnecessary.  Besides being unnecessary, it it undesirable.  If
     there's a lot of crap in the temp table to delete, running this *now* makes the whole
     program seem to hang as soon as the user requests this screen be launched.  If it is slow
     to delete from the temp table at the beginning of Search() ... well at least the user
     requested that the program do something that might ought to take a bit, and it's more
     expected to be slow to respond then, yeah?
  */
  //ExecuteStatement(dbTransaction, "delete from appointment_searches");

  Filter->SetSecurity("appointments");
  Filter->SetTransaction(dbTransaction);
  Filter->AddColumnSpec("Date", "appointments", as_date, true, 85);
  Filter->AddColumnSpec("Time", "appointments", as_time, true, 85);
  Filter->AddColumnSpec("Person", "people", as_text, true, 120);
  Filter->AddColumnSpec("Tax Entity", "tax_entities", as_text, false, 130, rawString);
  Filter->AddColumnSpec("Created For", "appointments", as_text, false, 100);
  Filter->AddColumnSpec("Created By", "appointments", as_text, false, 100);
  Filter->AddColumnSpec("Reason", "appointments", as_text, false, 90);
  Filter->AddColumnSpec("Location", "appointments", as_text, false, 90);
  Filter->AddColumnSpec("Description", "appointments", as_text, true, 150, sentenceString);
  Filter->AddColumnSpec("Outcome", "appointments", as_text, false, 90);
  Filter->AddColumnSpec("24 Hours", "appointments", as_text, false, 90, prettyBoolString);
  Filter->AddColumnSpec("Chart No.", "charts", as_text, false, 90);
  Filter->AddColumnSpec("Current Id", "current_id", as_int, false, 80);
  Filter->AddColumnSpec("Internal Id", "internal_id", as_int, false, 80);
  Filter->FetchPreferences("searchappointment.filter");
  Filter->SetIconStatus(true);
  Filter->SetSQLStatement ("");
  Filter->RefreshDataSet();

  return true;
}
//---------------------------------------------------------------------------
bool TformSearchAppointment::ScreenStop()
{
  lookupTables->UnregisterComboBox("users", cboFor);
  lookupTables->UnregisterComboBox("users", cboBy);
  lookupTables->UnregisterComboBox("groups", cboDepartment);
  lookupTables->UnregisterComboBox("appt_reason_types", cboReason);
  lookupTables->UnregisterComboBox("appt_location_types", cboLocation);
  ExecuteStatement(dbTransaction, "delete from appointment_searches");
  dbTransaction->Commit();
  delete dbTransaction;
  return true;
}
//---------------------------------------------------------------------------
bool TformSearchAppointment::Create()
{
  map<string, string> temp_map;
  temp_map["id"] = "new";
  FORM_TRY
    TformRecordAppointment * appointmentRecord = new TformRecordAppointment(NULL, temp_map);
    appointmentRecord->Show();
  FORM_CATCH
  return true;
}
//---------------------------------------------------------------------------
bool TformSearchAppointment::Zoom(const string & p_id)
{
  map<string,string> temp_map;
  temp_map["id"] = p_id;
  FORM_TRY
    TformRecordAppointment * appointmentRecord = new TformRecordAppointment(NULL, temp_map);
    appointmentRecord->Show();
  FORM_CATCH
  return true;
}
//---------------------------------------------------------------------------
vector<string> TformSearchAppointment::Do(TformSpecialGeneric * p_parent)
{
 vector<string> answer;
 FORM_TRY
  TformSearchAppointment * z = new TformSearchAppointment(NULL, &answer);
  z->ShowTaskModal(p_parent);
 FORM_CATCH
 return answer;
}
//---------------------------------------------------------------------------

