//---------------------------------------------------------------------------

#include <vcl.h>
#pragma hdrstop

#include "codeSearchBill.h"
#include "codeRecordIncomingBill.h" 
#include "codeRecordOutgoingBill.h"
#include "codeRecordBill.h"
#include "codeSearchTaxEntity.h"
//---------------------------------------------------------------------------
#pragma package(smart_init)
#pragma link "codeSearchGeneric"
#pragma link "codeRubberContainer"
#pragma resource "*.dfm"
TformSearchBill *formSearchBill;
//---------------------------------------------------------------------------
__fastcall TformSearchBill::TformSearchBill(TComponent* Owner, vector<string> * p_return_vector, string p_bill_id)
        : TformSearchGeneric(Owner, p_return_vector, string ("bills"))
{
  bill_id = p_bill_id;
  Initialize();
}
//---------------------------------------------------------------------------
bool TformSearchBill::IsValidOnScreen()
{
  // check to make sure that the values entered in the combo boxes
  // corrospond to values in that look up table.
  // give the user a message box and prompt to re-enter if incorrect
  if (! Pdate (editBeginBillDate->Text.c_str(), mdy | yy).IsValidDate() && ! (editBeginBillDate->Text == ""))
  {
    PSay("Please enter a valid begin bill date", m_invalid_data, MB_OK);
    editBeginBillDate->SetFocus();
    return false;
  }
  else if (! Pdate (editEndBillDate->Text.c_str(), mdy | yy).IsValidDate() && ! (editEndBillDate->Text == ""))
  {
    PSay("Please enter a valid end bill date", m_invalid_data, MB_OK);
    editEndBillDate->SetFocus();
    return false;
  }
  else if (! IsAmount(editMinTotalCharge->Text.c_str()))
  {
    PSay("Please enter a valid min total charge", m_invalid_data, MB_OK);
    editMinTotalCharge->SetFocus();
    return false;
  }
  else if (! IsAmount(editMaxTotalCharge->Text.c_str()))
  {
    PSay("Please enter a valid max total charge", m_invalid_data, MB_OK);
    editMaxTotalCharge->SetFocus();
    return false;
  }
  else if (! IsAmount(editMinInvoiceAmount->Text.c_str()))
  {
    PSay("Please enter a valid min invoice amount", m_invalid_data, MB_OK);
    editMinInvoiceAmount->SetFocus();
    return false;
  }
  else if (! IsAmount(editMaxInvoiceAmount->Text.c_str()))
  {
    PSay("Please enter a valid max invoice amount", m_invalid_data, MB_OK);
    editMaxInvoiceAmount->SetFocus();
    return false;
  }
  else if (! IsInt(editCheckNumber->Text.c_str()))
  {
    PSay("Please enter a valid check number", m_invalid_data, MB_OK);
    editCheckNumber->SetFocus();
    return false;
  }
  else if (! IsInt(editPrNumber->Text.c_str()))
  {
    PSay("Please enter a valid PR number", m_invalid_data, MB_OK);
    editPrNumber->SetFocus();
    return false;
  }
  else if (! lookupTables->IsInLookUpTable("bill_types", comboBillType->Text.c_str()))
  {
    PSay("Please enter a type of bill (or none at all)", m_invalid_data, MB_OK);
    comboBillType->SetFocus();
    return false;
  }
  else
    return true;
}
//---------------------------------------------------------------------------

bool TformSearchBill::Search()
{
  string sql;
  string from;
  string where;

  sql = "select bills.bill_date, "
        "a.name, b.name, c.name, d.name, "
        "bills.invoice_number, purchase_reqs.purchase_req_number, checks.check_number, bills.invoice_amount, "
        "bills.total_charge, bills.paid_amount, bills.current_id, bills.id "
        "from bills, purchase_reqs, checks, tax_entities a, tax_entities b, tax_entities c, tax_entities d ";
        
  if((editBeginServiceDate->Text != "") || (editEndServiceDate->Text != ""))
   sql += ", tasks ";
  sql += "where bills.id > 0 and bills.purchase_req_id = purchase_reqs.id "
         "and purchase_reqs.check_id = checks.id ";

  if (comboBillType->Text != "")
  sql += "and bills.bill_id = " + lookupTables->IdOf("bill_types", comboBillType->Text.c_str()) + " ";

  if((editBeginServiceDate->Text != "") || (editEndServiceDate->Text != ""))
   sql += "and bills.task_id = tasks.id ";
  sql += "and bills.actual_tax_id = a.id "
         "and bills.provider_tax_id = b.id "
         "and bills.facility_tax_id = c.id "
         "and bills.group_tax_id = d.id ";

  string tax_entity_name = editTaxEntityName->Text.c_str();
  if(!tax_entity_name.empty())
    sql += "and ("
           "upper(a.name) like upper('%" + SafeSQLString(tax_entity_name) + "%') or "
           "upper(b.name) like upper('%" + SafeSQLString(tax_entity_name) + "%') or "
           "upper(c.name) like upper('%" + SafeSQLString(tax_entity_name) + "%') or "
           "upper(d.name) like upper('%" + SafeSQLString(tax_entity_name) + "%')) ";
  if(editBeginBillDate->Text != "")
    sql += "and bills.bill_date >= '" + Pdate(editBeginBillDate->Text.c_str(), mdy | yy).OutDate(mdy | yyyy) + "' ";
  if(editEndBillDate->Text != "")
    sql += "and bills.bill_date <= '" + Pdate(editEndBillDate->Text.c_str(), mdy | yy).OutDate(mdy | yyyy) + "' ";
  if(editBeginServiceDate->Text != "")
    sql += "and tasks.date_of_task >= '" + SafeSQLString(editBeginServiceDate->Text.c_str()) + "' ";
  if(editEndServiceDate->Text != "")
    sql += "and tasks.date_of_task <= '" + SafeSQLString(editEndServiceDate->Text.c_str()) + "' ";
  if(editMinTotalCharge->Text != "")
    sql += "and bills.total_charge >= " + SafeSQLString(editMinTotalCharge->Text.c_str()) + " ";
  if(editMaxTotalCharge->Text != "")
    sql += "and bills.total_charge <= " + SafeSQLString(editMaxTotalCharge->Text.c_str()) + " ";
  if(editMinInvoiceAmount->Text != "")
    sql += "and bills.invoice_amount >= " + SafeSQLString(editMinInvoiceAmount->Text.c_str()) + " ";
  if(editMaxInvoiceAmount->Text != "")
    sql += "and bills.invoice_amount <= " + SafeSQLString(editMaxInvoiceAmount->Text.c_str()) + " ";
  if(editInvoiceNumber->Text != "")
    sql += "and bills.invoice_number like '%" + SafeSQLString(editInvoiceNumber->Text.c_str()) + "%' ";
  if(editCheckNumber->Text != "")
    sql += "and checks.check_number = " + string(editCheckNumber->Text.c_str()) + " ";
  if(editPrNumber->Text != "")
    sql += "and purchase_reqs.purchase_req_number = " + string(editPrNumber->Text.c_str()) + " ";

  Filter->SetSQLStatement(sql);
  Filter->RefreshDataSet();

  return true;

}
//---------------------------------------------------------------------------
bool TformSearchBill::ScreenStart()
{
  // instead of formcreate, do startup stuff here or in the constructor
  // this gets called after stuff in the base classes's constructor
  lookupTables->RegisterComboBox("bill_types", comboBillType);
  if(bill_id != "")
   comboBillType->Text = lookupTables->ValueOf("bill_types", bill_id).c_str();

  Filter->SetListViewHandle(listSearchResults);
  Filter->SetSecurity("bills");
  Filter->AddColumnSpec("Date on bill", "bills", as_date, true, 110);
  Filter->AddColumnSpec("Selected Entity", "tax_entities", as_text, true, 150, rawString);
  Filter->AddColumnSpec("Provider", "tax_entities", as_text, true, 150, rawString);
  Filter->AddColumnSpec("Facility", "tax_entities", as_text, true, 150, rawString);
  Filter->AddColumnSpec("Group", "tax_entities", as_text, true, 150, rawString);
  Filter->AddColumnSpec("Invoice Number", "bills", as_text, true, 100);
  Filter->AddColumnSpec("PR Number", "purchase_reqs", as_int, true, 100);
  Filter->AddColumnSpec("Check Number", "checks", as_int, true, 100);
  Filter->AddColumnSpec("Invoice Amount", "bills", as_double, true, 110, amountString);
  Filter->AddColumnSpec("Total Amount", "bills", as_double, true, 110, amountString);    
  Filter->AddColumnSpec("Amount Paid", "bills", as_double, true, 110, amountString);
  Filter->AddColumnSpec("Current Id", "current_id", as_int, false, 80);
  Filter->AddColumnSpec("Internal Id", "internal_id", as_int, false, 80);
  Filter->FetchPreferences("searchbills.filter");
  Filter->SetSQLStatement("");
  Filter->RefreshDataSet();
  return true;
}
//---------------------------------------------------------------------------
bool TformSearchBill::ScreenStop()
{
 lookupTables->UnregisterComboBox("bill_types", comboBillType);
 return true;
}
//---------------------------------------------------------------------------
bool TformSearchBill::Create()
{
 PSay("This function is currently disabled.", m_debug, MB_OK);
 return true;
}
//---------------------------------------------------------------------------
bool TformSearchBill::Zoom(const string & p_id)
{
  map<string, string> temp_map;
  temp_map["id"] = p_id;
  string bill_type_id = GetOneField(dbDatabase, "select bills.bill_id from bills where id = " + temp_map["id"]);

  /*  Philip says he'd rather them not see this screen until he's tested it more
  FORM_TRY
   TformRecordBill * bill_record = new TformRecordBill(NULL, temp_map);
   bill_record->Show();
  FORM_CATCH
  */

  if((bill_type_id == lookupTables->IdOf("bill_types", "incoming vendor bill")) ||
     (bill_type_id == lookupTables->IdOf("bill_types", "incoming provider bill")))
  {
    FORM_TRY
      TformRecordIncomingBill * bill_record = new TformRecordIncomingBill(NULL, temp_map);
      bill_record->Show();
    FORM_CATCH
  }
  else if(bill_type_id == lookupTables->IdOf("bill_types", "outgoing provider bill"))
  {
    FORM_TRY
      TformRecordOutgoingBill * bill_record = new TformRecordOutgoingBill(NULL, temp_map);
      bill_record->Show();
    FORM_CATCH
  } else
   PSay ("Undetermined type of bill, cannot decide which screen to show!", m_missing_data, MB_OK);
  
  return true;
}
//---------------------------------------------------------------------------
vector<string> TformSearchBill::Do(TformSpecialGeneric * p_parent)
{
 return TformSearchBill::Do(p_parent, "");
}
//---------------------------------------------------------------------------
vector<string> TformSearchBill::Do(TformSpecialGeneric * p_parent, string p_bill_id)
{
 vector<string> answer;
 FORM_TRY
  TformSearchBill * z = new TformSearchBill(NULL, &answer, p_bill_id);
  z->ShowTaskModal(p_parent);
 FORM_CATCH
 return answer;
}
//---------------------------------------------------------------------------

