//---------------------------------------------------------------------------

#include <vcl.h>
#pragma hdrstop

#include "codeSearchCrashLog.h"
#include "codeRecordCrashLog.h"
//---------------------------------------------------------------------------
#pragma package(smart_init)
#pragma link "codeRubberContainer"
#pragma link "codeSearchGeneric"
#pragma resource "*.dfm"
TformSearchCrashLog *formSearchCrashLog;
//---------------------------------------------------------------------------
__fastcall TformSearchCrashLog::TformSearchCrashLog(TComponent* Owner, vector<string> *p_return_vector)
        : TformSearchGeneric(Owner, p_return_vector, string("crash_logs"))
{
  Initialize();
}
//---------------------------------------------------------------------------
bool TformSearchCrashLog::IsValidOnScreen()
{
  if(cboAssignedTo->Text != "" && !lookupTables->IsInLookUpTable("users", cboAssignedTo))
  {
    PSay("Please enter a valid Assigned To user name", m_invalid_data, MB_OK);
    cboAssignedTo->SetFocus();
    return false;
  }
  if(cboBugStatus->Text != "" && !lookupTables->IsInLookUpTable("bug_status_types", cboBugStatus))
  {
    PSay("Please enter a valid bug status", m_invalid_data, MB_OK);
    cboBugStatus->SetFocus();
    return false;
  }
  if(cboBugPriority->Text != "" && !lookupTables->IsInLookUpTable("bug_priority_types", cboBugPriority))
  {
    PSay("Please enter a valid bug priority", m_invalid_data, MB_OK);
    cboBugPriority->SetFocus();
    return false;
  }
  if(cboUploadUser->Text != "" && !lookupTables->IsInLookUpTable("users", cboUploadUser))
  {
    PSay("Please enter a valid Upload User", m_invalid_data, MB_OK);
    cboUploadUser->SetFocus();
    return false;
  }
  if(Pdate(dateUploadedMin->Date) > Pdate(dateUploadedMax->Date))
  {
    PSay("Upload Start date should fall before or on the Upload Max date", m_invalid_data, MB_OK);
    dateUploadedMin->SetFocus();
    return false;
  }
  if(Pdate(dateOccurredMin->Date) > Pdate(dateOccurredMax->Date))
  {
    PSay("Error Start date should fall before or on the Error Max date", m_invalid_data, MB_OK);
    dateOccurredMin->SetFocus();
    return false;
  }
  if(editCode->Text != "" && !IsHexInt(editCode->Text.c_str(), 32))
  {
    PSay("Please enter a valid hexadecimal Exception Code", m_invalid_data, MB_OK);
    editCode->SetFocus();
    return false;
  }
  if(editSegment->Text != "" && !IsHexInt(editSegment->Text.c_str(), 32))
  {
    PSay("Please enter a valid hexadecimal Segment value", m_invalid_data, MB_OK);
    editSegment->SetFocus();
    return false;
  }
  if(editOffset->Text != "" && !IsHexInt(editOffset->Text.c_str(), 32))
  {
    PSay("Please enter a valid hexadecimal Offset value", m_invalid_data, MB_OK);
    editOffset->SetFocus();
    return false;
  }
  if(editEip->Text != "" && !IsHexInt(editEip->Text.c_str(), 32))
  {
    PSay("Please enter a valid hexadecimal EIP value", m_invalid_data, MB_OK);
    editEip->SetFocus();
    return false;
  }
  if(editAVAddress->Text != "" && !IsHexInt(editAVAddress->Text.c_str(), 32))
  {
    PSay("Please enter a valid hexadecimal Access Violation Address value", m_invalid_data, MB_OK);
    editAVAddress->SetFocus();
    return false;
  }
  if(editVersionMajor->Text != "" && !IsInt(editVersionMajor->Text.c_str()))
  {
    PSay("Please enter a valid major version number", m_invalid_data, MB_OK);
    editVersionMajor->SetFocus();
    return false;
  }
  if(editVersionMinor->Text != "" && !IsInt(editVersionMinor->Text.c_str()))
  {
    PSay("Please enter a valid minor version number", m_invalid_data, MB_OK);
    editVersionMinor->SetFocus();
    return false;
  }
  if(editVersionBuild->Text != "" && !IsInt(editVersionBuild->Text.c_str()))
  {
    PSay("Please enter a valid build version number", m_invalid_data, MB_OK);
    editVersionBuild->SetFocus();
    return false;
  }

  return true;
}
//---------------------------------------------------------------------------
bool TformSearchCrashLog::ScreenStart()
{
  string assigned_user_sql =           // All MIS users, even if no longer valid
    "select id, item, 1 valid from users where id = 0 or group_id = " + lookupTables->IdOf("groups", "MIS") + " "
    "order by item ";
  lookupTables->RegisterComboBox("users",              cboAssignedTo, assigned_user_sql);
  lookupTables->RegisterComboBox("bug_status_types",   cboBugStatus);
  lookupTables->RegisterComboBox("bug_priority_types", cboBugPriority);
  lookupTables->RegisterComboBox("users",              cboUploadUser);

  Filter->SetSecurity("crash_logs");
  Filter->AddColumnSpec("Error Date",   "crash_logs",  as_date, true,   80);
  Filter->AddColumnSpec("Run By",       "crash_logs",  as_text, true,  100, rawString);
  Filter->AddColumnSpec("Upload Date",  "crash_logs",  as_date, true,   80);
  Filter->AddColumnSpec("Uploaded By",  "users",       as_text, true,  100);
  Filter->AddColumnSpec("Status",       "crash_logs",  as_text, true,  100);
  Filter->AddColumnSpec("Priority",     "crash_logs",  as_text, true,  100);
  Filter->AddColumnSpec("Assigned To",  "users",       as_text, true,  100);
  Filter->AddColumnSpec("Program Name", "crash_logs",  as_text, true,  110, rawString);
  Filter->AddColumnSpec("Version",      "crash_logs",  as_text, true,  120);
  Filter->AddColumnSpec("Module Name",  "crash_logs",  as_text, true,  120, rawString);
  Filter->AddColumnSpec("Current Id",   "current_id",  as_int,  false,  80);
  Filter->AddColumnSpec("Internal Id",  "internal_id", as_int,  false,  80);
  Filter->FetchPreferences("searchcrashlog.filter");
  Filter->SetIconStatus(true);
  Filter->SetSQLStatement("");
  Filter->RefreshDataSet();

  pair<Pdate, Pdate> date_range = GetPriorMonthDateRange(Pdate(Date()));
  date_range.second.mod(1, in_months);

  dateUploadedMin->Date = date_range.first.OutDate();
  dateUploadedMax->Date = date_range.second.OutDate();
  dateOccurredMin->Date = date_range.first.OutDate();
  dateOccurredMax->Date = date_range.second.OutDate();

  return TformSearchGeneric::ScreenStart();
}
//---------------------------------------------------------------------------
bool TformSearchCrashLog::ScreenStop()
{
  lookupTables->UnregisterComboBox("users",              cboAssignedTo);
  lookupTables->UnregisterComboBox("bug_status_types",   cboBugStatus);
  lookupTables->UnregisterComboBox("bug_priority_types", cboBugPriority);
  lookupTables->UnregisterComboBox("users",              cboUploadUser);
  
  return TformSearchGeneric::ScreenStop();
}
//---------------------------------------------------------------------------
bool TformSearchCrashLog::Create()
{
  return false; // Nope.
}
//---------------------------------------------------------------------------
bool TformSearchCrashLog::Search()
{
  string sql;

  string begin_create_date = Pdate(dateUploadedMin->Date).OutDate(mdy | yyyy);
  string end_create_date   = Pdate(dateUploadedMax->Date).OutDate(mdy | yyyy);
  string begin_error_date  = Pdate(dateOccurredMin->Date).OutDate(mdy | yyyy);
  string end_error_date    = Pdate(dateOccurredMax->Date).OutDate(mdy | yyyy);

  sql =
    "select cast(CL.error_timestamp as date), CL.run_by, cast(CL.creation_stamp as date), UBU.item, "
    " BST.item, BPT.item, ATU.item, CL.program_name, "
    " (CL.exe_version_major || '.' || CL.exe_version_minor || '.' || CL.exe_version_build), CL.p_module_name, "
    " CL.current_id, CL.id "
    "from crash_logs CL "
    "inner join users UBU on CL.created_by  = UBU.id and UBU.id > 0"
    "left  join users ATU on CL.assigned_to = ATU.id and ATU.id > 0 "
    "inner join bug_status_types BST on CL.bug_status_id = BST.id "
    "inner join bug_priority_types BPT on CL.bug_priority_id = BPT.id "
    "where "
    " ((CL.creation_stamp between '" + begin_create_date + "' and '" + end_create_date + "' and "
    "   CL.error_timestamp between '" + begin_error_date + "' and '" + end_error_date + "') ";

  // jhaley 20111013: Allow considering additional occurrences in date search
  if(checkAddlOccs->Checked)
  {
    sql += "or exists "
           "(select * from crash_occurrences CO where CO.crash_log_id = CL.id and "
           " (CO.creation_stamp between '" + begin_create_date + "' and '" + end_create_date + "' and "
           "  CO.error_timestamp between '" + begin_error_date + "' and '" + end_error_date + "'))";
  }
  sql += ") ";

  if(cboAssignedTo->Text != "")
    sql += "and CL.assigned_to = '" + lookupTables->IdOf("users", cboAssignedTo) + "' ";

  if(cboBugStatus->Text != "")
    sql += "and CL.bug_status_id = '" + lookupTables->IdOf("bug_status_types", cboBugStatus) + "' ";

  if(cboBugPriority->Text != "")
    sql += "and CL.bug_priority_id = '" + lookupTables->IdOf("bug_priority_types", cboBugPriority) + "' ";

  if(cboUploadUser->Text != "")
    sql += "and CL.created_by = '" + lookupTables->IdOf("users", cboUploadUser) + "' ";

  if(editCode->Text != "")
    sql += "and CL.code = '" + IntToString(HexStringToInt(editCode->Text.c_str())) + "' ";

  if(editSegment->Text != "")
    sql += "and CL.log_segment = '" + IntToString(HexStringToInt(editSegment->Text.c_str())) + "' ";

  if(editOffset->Text != "")
    sql += "and CL.log_offset = '" + IntToString(HexStringToInt(editOffset->Text.c_str())) + "' ";

  if(editEip->Text != "")
    sql += "and CL.ctx_eip = '" + IntToString(HexStringToInt(editEip->Text.c_str())) + "' ";

  if(editAVAddress->Text != "")
    sql += "and CL.av_address = '" + IntToString(HexStringToInt(editAVAddress->Text.c_str())) + "' ";

  if(editVersionMajor->Text != "")
    sql += "and CL.exe_version_major = '" + string(editVersionMajor->Text.c_str()) + "' ";
  if(editVersionMinor->Text != "")
    sql += "and CL.exe_version_minor = '" + string(editVersionMinor->Text.c_str()) + "' ";
  if(editVersionBuild->Text != "")
    sql += "and CL.exe_version_build = '" + string(editVersionBuild->Text.c_str()) + "' ";

  if(editRunBy->Text != "")
    sql += "and upper(CL.run_by) = '" + SafeSQLString(UppercaseString(editRunBy->Text.c_str())) + "' ";

  Filter->SetSQLStatement(sql);
  Filter->RefreshDataSet();

  return true;
}
//---------------------------------------------------------------------------
bool TformSearchCrashLog::Zoom(const string &p_id)
{
  pmapstrs temp_map;

  temp_map["id"] = p_id;
  TformRecordCrashLog *crashRecord = NULL;
  FORM_TRY
    crashRecord = new TformRecordCrashLog(NULL, temp_map);
  FORM_CATCH_RETURN("TformRecordCrashLog", false)

  crashRecord->Show();
  return true;
}
//---------------------------------------------------------------------------
vector<string> TformSearchCrashLog::Do(TformSpecialGeneric *p_parent)
{
  vector<string> answer;
  TformSearchCrashLog *z = NULL;
  FORM_TRY
    z = new TformSearchCrashLog(NULL, &answer);
  FORM_CATCH_RETURN("TformSearchCrashLog", answer)

  z->ShowTaskModal(p_parent);
  return answer;
}
//---------------------------------------------------------------------------


