//---------------------------------------------------------------------------

#include <vcl.h>
#pragma hdrstop

#include "codeSearchDeferredCOB.h"
#include "codeRecordDeferredCOB.h"
//---------------------------------------------------------------------------
#pragma package(smart_init)
#pragma link "codeRubberContainer"
#pragma link "codeSearchGeneric"
#pragma link "codeBaseDropDown"
#pragma link "codeCheckListDropDown"
#pragma resource "*.dfm"
TformSearchDeferredCOB *formSearchDeferredCOB;
//---------------------------------------------------------------------------
__fastcall TformSearchDeferredCOB::TformSearchDeferredCOB(TComponent* Owner, vector<string> *p_return_vector)
        : TformSearchGeneric(Owner, p_return_vector, string("deferred_cobs"))
{
  Initialize();
}
//---------------------------------------------------------------------------
bool TformSearchDeferredCOB::IsValidOnScreen()
{
  if(editBeginDate->Text != "")
  {
    Pdate dob = Pdate(editBeginDate->Text.c_str(), global_mode);
    Pdate now = Pdate(Now());
    if(!dob.IsValidDate() || dob.year > now.year + 1000)
    {
      PSay("Please enter a valid begin date", m_invalid_data, MB_OK);
      editBeginDate->SetFocus();
      return false;
    }
  }

  if(editEndDate->Text != "")
  {
    Pdate dob = Pdate(editEndDate->Text.c_str(), global_mode);
    Pdate now = Pdate(Now());
    if(!dob.IsValidDate() || dob.year > now.year + 1000)
    {
      PSay("Please enter a valid end date", m_invalid_data, MB_OK);
      editEndDate->SetFocus();
      return false;
    }
  }

  if(editCOBNum->Text != "" && !IsInt(editCOBNum->Text.c_str()))
  {
    PSay("Please enter a valid number value for the COB", m_invalid_data, MB_OK);
    editCOBNum->SetFocus();
    return false;
  }

  return true;
}
//---------------------------------------------------------------------------
bool TformSearchDeferredCOB::Search()
{
  string sql;

  sql =
    "select PN.last_name, PN.first_name, DCOB.date_of_deferral, REF.name, PROV.name, SERV.item, AREA.item, REAS.item, "
    " STAT.item, DCOB.is_approved, DCOB.current_id, DCOB.id "
    "from deferred_cobs DCOB "
    "inner join people P on DCOB.person_id = P.id "
    "inner join tax_entities REF on DCOB.provider_id = REF.id "
    "inner join services_provided SP on DCOB.services_provided_id = SP.id "
    "inner join tax_entities PROV on SP.tax_entity_id = PROV.id "
    "inner join service_types SERV on SP.service_id = SERV.id "
    "inner join area_types AREA on SP.area_id = AREA.id "
    "inner join deferral_reason_types REAS on DCOB.deferral_reason_id = REAS.id "
    "inner join deferral_status_types STAT on DCOB.deferral_status_id = STAT.id "
    "left  join person_names PN on P.person_names_id = PN.id "
    "left  join tasks COB on DCOB.cob_id = COB.id "
    "where DCOB.id > 0 ";

  if(editLastName->Text != "")
    sql += "and upper(PN.last_name) like '%" + SafeSQLString(editLastName->Text.c_str()) + "%' ";
  if(editFirstName->Text != "")
    sql += "and upper(PN.first_name) like '%" + SafeSQLString(editFirstName->Text.c_str()) + "%' ";
  if(editChartNo->Text != "")
  {
    sql += "and exists "
           "(select CH.id from charts CH where CH.person_id = P.id and "
           " CH.chart_number = '" + SafeSQLString(editChartNo->Text.c_str()) + "' and "
           " (CH.group_id = " + lookupTables->IdOf("groups", "Contract Health") + " or "
           "  CH.location_id = " + lookupTables->IdOf("location_types", "Contract Health") + ")) ";
  }
  if(editReferredBy->Text != "")
  {
    sql += "and (upper(REF.name) like '%" + SafeSQLString(ReplaceStringWithString(editReferredBy->Text.c_str(), " ", "%")) + "%' or "
                "upper(REF.name) like '" + SafeSQLString(InitialsSearchString1(editReferredBy->Text.c_str())) + "' or "
                "upper(REF.name) like '" + SafeSQLString(InitialsSearchString2(editReferredBy->Text.c_str())) + "') ";
  }
  if(editProvider->Text != "")
  {
    sql += "and (upper(PROV.name) like '%" + SafeSQLString(ReplaceStringWithString(editProvider->Text.c_str(), " ", "%")) + "%' or "
                "upper(PROV.name) like '" + SafeSQLString(InitialsSearchString1(editProvider->Text.c_str())) + "' or "
                "upper(PROV.name) like '" + SafeSQLString(InitialsSearchString2(editProvider->Text.c_str())) + "') ";
  }

  vector<string> service_ids = cbddService->Filter->GetMultiID();
  if(!service_ids.empty())
    sql += "and " + SafeInClause("SERV.id", service_ids) + " ";

  vector<string> area_ids = cbddArea->Filter->GetMultiID();
  if(!area_ids.empty())
    sql += "and " + SafeInClause("AREA.id", area_ids) + " ";

  if(editBeginDate->Text != "")
  {
    if(editEndDate->Text != "")
    {
      sql += "and DCOB.date_of_deferral between '" + Pdate(editBeginDate->Text.c_str(), mdy | yy).OutDate(mdy | yyyy) +
             "' and '" + Pdate(editEndDate->Text.c_str(), mdy | yy).OutDate(mdy | yyyy) + "' ";
    }
    else
      sql += "and DCOB.date_of_deferral >= '" + Pdate(editBeginDate->Text.c_str(), mdy | yy).OutDate(mdy | yyyy) + "' ";
  }
  else if(editEndDate->Text != "")
    sql += "and DCOB.date_of_deferral <= '" + Pdate(editEndDate->Text.c_str(), mdy | yy).OutDate(mdy | yyyy) + "' ";

  vector<string> reason_ids = cbddReason->Filter->GetMultiID();
  if(!reason_ids.empty())
    sql += "and " + SafeInClause("REAS.id", reason_ids) + " ";

  vector<string> status_ids = cbddStatus->Filter->GetMultiID();
  if(!status_ids.empty())
    sql += "and " + SafeInClause("STAT.id", status_ids) + " ";

  if(editCOBNum->Text != "")
    sql += "and COB.cob_number = " + IntToString(StringToInt(editCOBNum->Text.c_str())) + " ";

  switch(groupApproved->ItemIndex)
  {
  case 1: // Approved for COB
    sql += "and DCOB.is_approved = 1 ";
    break;
  case 2: // Unapproved
    sql += "and DCOB.is_approved = 0 ";
    break;
  default: // Match anything
    break;
  }

  Filter->SetSQLStatement(sql);
  Filter->RefreshDataSet();
  
  return true;
}
//---------------------------------------------------------------------------
bool TformSearchDeferredCOB::ScreenStart()
{
  if(optionsPermissions["table.person_names"]["view"] != "true")
  {
    editLastName->Enabled  = false;
    editFirstName->Enabled = false;
  }

  if(optionsPermissions["table.charts"]["view"] != "true")
    editChartNo->Enabled = false;

  Filter->SetSecurity("deferred_cobs");
  Filter->AddColumnSpec("Last Name",     "person_names",      as_text, true,  100);
  Filter->AddColumnSpec("First Name",    "person_names",      as_text, true,  100);
  Filter->AddColumnSpec("Date",          "deferred_cobs",     as_date, true,   80);
  Filter->AddColumnSpec("Referred By",   "tax_entities",      as_text, true,  120);
  Filter->AddColumnSpec("Provider",      "tax_entities",      as_text, true,  120);
  Filter->AddColumnSpec("Service",       "services_provided", as_text, true,  120);
  Filter->AddColumnSpec("Area",          "services_provided", as_text, true,   80);
  Filter->AddColumnSpec("Reason",        "deferred_cobs",     as_text, true,  120, rawString);
  Filter->AddColumnSpec("Status",        "deferred_cobs",     as_text, true,  100, rawString);
  Filter->AddColumnSpec("Approved",      "deferred_cobs",     as_text, true,   80, prettyBoolString);
  Filter->AddColumnSpec("Current Id",    "current_id",        as_int,  false,  80);
  Filter->AddColumnSpec("Internal Id",   "internal_id",       as_int,  false,  80);
  Filter->FetchPreferences("searchdeferredcob.filter");
  Filter->SetIconStatus(true);
  Filter->SetSQLStatement("");
  Filter->RefreshDataSet();

  string sql = "select item, id from service_types where valid = 1";
  cbddService->Filter->SetSecurity("");
  cbddService->Filter->AddColumnSpec("Service Type", "",            as_text, true,  153, prettyString);
  cbddService->Filter->AddColumnSpec("Internal Id",  "internal_id", as_int,  false,  60);
  cbddService->Filter->SetSQLStatement(sql);
  cbddService->Filter->FetchPreferences("searchdeferredcob.services_filter");

  sql = "select item, id from area_types where valid = 1";
  cbddArea->Filter->SetSecurity("");
  cbddArea->Filter->AddColumnSpec("Area Type",   "",            as_text, true,  153, prettyString);
  cbddArea->Filter->AddColumnSpec("Internal Id", "internal_id", as_int,  false,  60);
  cbddArea->Filter->SetSQLStatement(sql);
  cbddArea->Filter->FetchPreferences("searchdeferredcob.areas_filter");

  sql = "select item, id from deferral_reason_types where valid = 1";
  cbddReason->Filter->SetSecurity("");
  cbddReason->Filter->AddColumnSpec("Reason",      "",            as_text, true,  153, rawString);
  cbddReason->Filter->AddColumnSpec("Internal Id", "internal_id", as_int,  false,  60);
  cbddReason->Filter->SetSQLStatement(sql);
  cbddReason->Filter->FetchPreferences("searchdeferredcob.reason_filter");

  sql = "select item, id from deferral_status_types where valid = 1";
  cbddStatus->Filter->SetSecurity("");
  cbddStatus->Filter->AddColumnSpec("Status",      "",            as_text, true,  153, rawString);
  cbddStatus->Filter->AddColumnSpec("Internal Id", "internal_id", as_int,  false,  60);
  cbddStatus->Filter->SetSQLStatement(sql);
  cbddStatus->Filter->FetchPreferences("searchdeferredcob.status_filter");

  pair<Pdate, Pdate> dates = GetPriorMonthDateRange(Pdate(Date()));
  dates.second.mod(3, in_months);

  editBeginDate->Text = dates.first.OutDate(mdy | yyyy).c_str();
  editEndDate->Text   = dates.second.OutDate(mdy | yyyy).c_str();

  return TformSearchGeneric::ScreenStart();
}
//---------------------------------------------------------------------------
bool TformSearchDeferredCOB::Create()
{
  pmapstrs temp_map;
  temp_map["id"] = "new";

  TformRecordDeferredCOB *formDefCOB = NULL;
  FORM_TRY
    formDefCOB = new TformRecordDeferredCOB(NULL, temp_map);
  FORM_CATCH_RETURN("TformRecordDeferredCOB", false)

  formDefCOB->Show();
  return true;
}
//---------------------------------------------------------------------------
bool TformSearchDeferredCOB::Zoom(const string &p_id)
{
  pmapstrs temp_map;
  temp_map["id"] = p_id;

  TformRecordDeferredCOB *formDefCOB = NULL;
  FORM_TRY
    formDefCOB = new TformRecordDeferredCOB(NULL, temp_map);
  FORM_CATCH_RETURN("TformRecordDeferredCOB", false)

  formDefCOB->Show();
  return true;
}
//---------------------------------------------------------------------------
vector<string> TformSearchDeferredCOB::Do(TformSpecialGeneric *p_parent)
{
  vector<string> answer;

  TformSearchDeferredCOB *defCobSearch = NULL;
  FORM_TRY
    defCobSearch = new TformSearchDeferredCOB(NULL, &answer);
  FORM_CATCH_RETURN("TformSearchDeferredCOB", answer)

  defCobSearch->ShowTaskModal(p_parent);

  return answer;
}
//---------------------------------------------------------------------------
void __fastcall TformSearchDeferredCOB::cbddServiceDown(TObject *Sender)
{
  cbddService->Filter->ForceRefreshDataSet();
  cbddService->DisplayFieldName = "Service Type";
}
//---------------------------------------------------------------------------
void __fastcall TformSearchDeferredCOB::cbddAreaDown(TObject *Sender)
{
  cbddArea->Filter->ForceRefreshDataSet();
  cbddArea->DisplayFieldName = "Area Type";
}
//---------------------------------------------------------------------------
void __fastcall TformSearchDeferredCOB::cbddReasonDown(TObject *Sender)
{
  cbddReason->Filter->ForceRefreshDataSet();
  cbddReason->DisplayFieldName = "Reason";
}
//---------------------------------------------------------------------------
void __fastcall TformSearchDeferredCOB::cbddStatusDown(TObject *Sender)
{
  cbddStatus->Filter->ForceRefreshDataSet();
  cbddStatus->DisplayFieldName = "Status";
}
//---------------------------------------------------------------------------

