//---------------------------------------------------------------------------

#include <vcl.h>
#pragma hdrstop

#include "codeSearchDenial.h"
#include "codeRecordDenial.h"
//---------------------------------------------------------------------------
#pragma package(smart_init)
#pragma link "codeRubberContainer"
#pragma link "codeSearchGeneric"
#pragma link "codeBaseDropDown"
#pragma link "codeCheckListDropDown"
#pragma resource "*.dfm"
TformSearchDenial *formSearchDenial;
//---------------------------------------------------------------------------
__fastcall TformSearchDenial::TformSearchDenial(TComponent* Owner, vector<string> *p_return_vector)
        : TformSearchGeneric(Owner, p_return_vector, string("denials"))
{
  Initialize();
}
//---------------------------------------------------------------------------
bool TformSearchDenial::IsValidOnScreen()
{
  if(editProcessed->Text != "")
  {
    Pdate dob = Pdate(editProcessed->Text.c_str(), global_mode);
    Pdate now = Pdate(Now());
    if(!dob.IsValidDate() || dob.year > now.year + 1000)
    {
      PSay("Please enter a valid date processed", m_invalid_data, MB_OK);
      editProcessed->SetFocus();
      return false;
    }
  }

  if(editEffective->Text != "")
  {
    Pdate dob = Pdate(editEffective->Text.c_str(), global_mode);
    Pdate now = Pdate(Now());
    if(!dob.IsValidDate() || dob.year > now.year + 1000)
    {
      PSay("Please enter a valid effective date", m_invalid_data, MB_OK);
      editEffective->SetFocus();
      return false;
    }
  }

  return true;
}
//---------------------------------------------------------------------------
bool TformSearchDenial::Search()
{
  string sql;

  sql =
    "select PN.last_name, PN.first_name, TE.name, ST.item, AREA.item, NDLA.item, NDLS.item, DN.current_id, DN.id "
    "from denials DN "
    "inner join people P on DN.person_id = P.id "
    "inner join services_provided SP on DN.services_provided_id = SP.id "
    "inner join service_types ST on SP.service_id = ST.id "
    "inner join area_types AREA on SP.area_id = AREA.id "
    "inner join tax_entities TE on SP.tax_entity_id = TE.id "
    "left  join person_names PN on P.person_names_id = PN.id "
    "left  join denial_letters NDL on NDL.id = "
    " (select first 1 IDL.id from denial_letters IDL "
    "  where IDL.denial_id = DN.id and IDL.current_id >= 2 "
    "  order by IDL.last_update_stamp desc) "
    "left  join denial_authority_types NDLA on NDLA.id = NDL.denial_authority_id "
    "left  join denial_status_types    NDLS on NDLS.id = NDL.denial_status_id "
    "where DN.id > 0 ";

  if(editLastName->Text != "")
    sql += "and upper(PN.last_name) like '%" + SafeSQLString(editLastName->Text.c_str()) + "%' ";
  if(editFirstName->Text != "")
    sql += "and upper(PN.first_name) like '%" + SafeSQLString(editFirstName->Text.c_str()) + "%' ";
  if(editChartNo->Text != "")
  {
    sql += "and exists "
           "(select CH.id from charts CH where CH.person_id = P.id and "
           " CH.chart_number = '" + SafeSQLString(editChartNo->Text.c_str()) + "' and "
           " (CH.group_id = " + lookupTables->IdOf("groups", "Contract Health") + " or "
           "  CH.location_id = " + lookupTables->IdOf("location_types", "Contract Health") + ")) ";
  }
  if(editTaxEntity->Text != "")
  {
    sql += "and (upper(TE.name) like '%" + SafeSQLString(ReplaceStringWithString(editTaxEntity->Text.c_str(), " ", "%")) + "%' or "
                "upper(TE.name) like '" + SafeSQLString(InitialsSearchString1(editTaxEntity->Text.c_str())) + "' or "
                "upper(TE.name) like '" + SafeSQLString(InitialsSearchString2(editTaxEntity->Text.c_str())) + "') ";
  }

  vector<string> service_ids = cbddService->Filter->GetMultiID();
  if(!service_ids.empty())
    sql += "and " + SafeInClause("ST.id", service_ids) + " ";

  vector<string> area_ids = cbddArea->Filter->GetMultiID();
  if(!area_ids.empty())
    sql += "and " + SafeInClause("AREA.id", area_ids) + " ";

  vector<string> auth_ids = cbddAuthority->Filter->GetMultiID();
  if(!auth_ids.empty())
  {
    sql += "and exists (select * from denial_letters IDL1 where IDL1.denial_id = DN.id and "
           + SafeInClause("IDL1.denial_authority_id", auth_ids) + ") ";
  }

  vector<string> status_ids = cbddStatus->Filter->GetMultiID();
  if(!status_ids.empty())
  {
    sql += "and exists (select * from denial_letters IDL2 where IDL2.denial_id = DN.id and "
           + SafeInClause("IDL2.denial_status_id", status_ids) + ") ";
  }

  if(editProcessed->Text != "")
  {
    sql += "and exists (select * from denial_letters IDL3 where IDL3.denial_id = DN.id and "
           "IDL3.date_processed = '" + Pdate(editProcessed->Text.c_str(), mdy | yy).OutDate(mdy | yyyy) + "') ";
  }
  if(editEffective->Text != "")
  {
    sql += "and exists (select * from denial_letters IDL4 where IDL4.denial_id = DN.id and "
           "IDL4.effective_date = '" + Pdate(editEffective->Text.c_str(), mdy | yy).OutDate(mdy | yyyy) + "') ";
  }

  Filter->SetSQLStatement(sql);
  Filter->RefreshDataSet();

  return true;
}
//---------------------------------------------------------------------------
bool TformSearchDenial::ScreenStart()
{
  if(optionsPermissions["table.person_names"]["view"] != "true")
  {
    editLastName->Enabled  = false;
    editFirstName->Enabled = false;
  }

  if(optionsPermissions["table.charts"]["view"] != "true")
    editChartNo->Enabled = false;

  Filter->SetSecurity("denials");
  Filter->AddColumnSpec("Last Name",     "person_names",      as_text, true,  100);
  Filter->AddColumnSpec("First Name",    "person_names",      as_text, true,  100);
  Filter->AddColumnSpec("Provider",      "tax_entities",      as_text, true,  120);
  Filter->AddColumnSpec("Service",       "services_provided", as_text, true,  120);
  Filter->AddColumnSpec("Area",          "services_provided", as_text, true,   80);
  Filter->AddColumnSpec("Newest Letter", "denial_letters",    as_text, true,  100, rawString); // denial authority
  Filter->AddColumnSpec("Denial Status", "denial_letters",    as_text, true,   80); // denial status
  Filter->AddColumnSpec("Current Id",    "current_id",        as_int,  false,  80);
  Filter->AddColumnSpec("Internal Id",   "internal_id",       as_int,  false,  80);
  Filter->FetchPreferences("searchdenial.filter");
  Filter->SetIconStatus(true);
  Filter->SetSQLStatement("");
  Filter->RefreshDataSet();

  string sql = "select item, id from service_types where valid = 1";
  cbddService->Filter->SetSecurity("");
  cbddService->Filter->AddColumnSpec("Service Type", "",            as_text, true,  153, prettyString);
  cbddService->Filter->AddColumnSpec("Internal Id",  "internal_id", as_int,  false,  60);
  cbddService->Filter->SetSQLStatement(sql);
  cbddService->Filter->FetchPreferences("searchdenial.services_filter");

  sql = "select item, id from area_types where valid = 1";
  cbddArea->Filter->SetSecurity("");
  cbddArea->Filter->AddColumnSpec("Area Type",   "",            as_text, true,  153, prettyString);
  cbddArea->Filter->AddColumnSpec("Internal Id", "internal_id", as_int,  false,  60);
  cbddArea->Filter->SetSQLStatement(sql);
  cbddArea->Filter->FetchPreferences("searchdenial.areas_filter");

  sql = "select item, id from denial_authority_types where valid = 1";
  cbddAuthority->Filter->SetSecurity("");
  cbddAuthority->Filter->AddColumnSpec("Authority",   "",            as_text, true,  153, rawString);
  cbddAuthority->Filter->AddColumnSpec("Internal Id", "internal_id", as_int,  false,  60);
  cbddAuthority->Filter->SetSQLStatement(sql);
  cbddAuthority->Filter->FetchPreferences("searchdenial.authority_filter");

  sql = "select item, id from denial_status_types where valid = 1";
  cbddStatus->Filter->SetSecurity("");
  cbddStatus->Filter->AddColumnSpec("Status Type", "",            as_text, true,  153, rawString);
  cbddStatus->Filter->AddColumnSpec("Internal Id", "internal_id", as_int,  false,  60);
  cbddStatus->Filter->SetSQLStatement(sql);
  cbddStatus->Filter->FetchPreferences("searchdenial.status_filter");

  return TformSearchGeneric::ScreenStart();
}
//---------------------------------------------------------------------------
bool TformSearchDenial::ScreenStop()
{
  return TformSearchGeneric::ScreenStop();
}
//---------------------------------------------------------------------------
bool TformSearchDenial::Create()
{
  int return_val = PSay("You will create a new denial batch if you perform this action from here.\n"
                        "If you are trying to create a CMMR or Executive Director denial or approval letter, this is "
                        "NOT the appropriate action to take!\n\n"
                        "Are you sure you want to create a new denial batch?", m_confirmation, MB_YESNO);

  if(return_val == IDYES)
  {
    pmapstrs temp_map;
    temp_map["id"] = "new";

    TformRecordDenial *formDenial = NULL;
    FORM_TRY
      formDenial = new TformRecordDenial(NULL, temp_map);
    FORM_CATCH_RETURN("TformRecordDenial", false)

    formDenial->Show();
  }
  return true;
}
//---------------------------------------------------------------------------
bool TformSearchDenial::Zoom(const string &p_id)
{
  pmapstrs temp_map;
  temp_map["id"] = p_id;

  TformRecordDenial *formDenial = NULL;
  FORM_TRY
    formDenial = new TformRecordDenial(NULL, temp_map);
  FORM_CATCH_RETURN("TformRecordDenial", false)

  formDenial->Show();
  return true;
}
//---------------------------------------------------------------------------
vector<string> TformSearchDenial::Do(TformSpecialGeneric *p_parent)
{
  vector<string> answer;

  TformSearchDenial *denialSearch = NULL;
  FORM_TRY
    denialSearch = new TformSearchDenial(NULL, &answer);
  FORM_CATCH_RETURN("TformSearchDenial", answer)

  denialSearch->ShowTaskModal(p_parent);
  return answer;
}
//---------------------------------------------------------------------------
void __fastcall TformSearchDenial::cbddServiceDown(TObject *Sender)
{
  cbddService->Filter->ForceRefreshDataSet();
  cbddService->DisplayFieldName = "Service Type";
}
//---------------------------------------------------------------------------
void __fastcall TformSearchDenial::cbddAreaDown(TObject *Sender)
{
  cbddArea->Filter->ForceRefreshDataSet();
  cbddArea->DisplayFieldName = "Area Type";
}
//---------------------------------------------------------------------------
void __fastcall TformSearchDenial::cbddAuthorityDown(TObject *Sender)
{
  cbddAuthority->Filter->ForceRefreshDataSet();
  cbddAuthority->DisplayFieldName = "Authority";
}
//---------------------------------------------------------------------------
void __fastcall TformSearchDenial::cbddStatusDown(TObject *Sender)
{
  cbddStatus->Filter->ForceRefreshDataSet();
  cbddStatus->DisplayFieldName = "Status Type";
}
//---------------------------------------------------------------------------
