//---------------------------------------------------------------------------

#include <vcl.h>
#pragma hdrstop

#include "codeSearchEvent.h"
#include "codeRecordEvent.h"
#include "codeSpecialDisambiguate.h"
//---------------------------------------------------------------------------
#pragma package(smart_init)
#pragma link "codeSearchGeneric"
#pragma link "codeRubberContainer"
#pragma link "codeBaseDropDown"
#pragma link "codeCheckListDropDown"
#pragma resource "*.dfm"
TformSearchEvent *formSearchEvent;
//---------------------------------------------------------------------------
__fastcall TformSearchEvent::TformSearchEvent(TComponent* Owner, vector<string> * p_return_vector, string id)
        : TformSearchGeneric(Owner, p_return_vector, string ("events"))
{
  group_id = id;

  Initialize();
}
//---------------------------------------------------------------------------
bool TformSearchEvent::IsValidOnScreen()
{
  // check to make sure that the values entered in the combo boxes
  // corrospond to values in that look up table.
  // give the user a message box and prompt to re-enter if incorrect
  if (! Pdate (editBeginDate->Text.c_str(), mdy | yy).IsValidDate() && ! (editBeginDate->Text == ""))
  {
    PSay("Please enter a valid begin date", m_invalid_data, MB_OK);
    editBeginDate->SetFocus();
    return false;
  }
  else if (! Pdate (editEndDate->Text.c_str(), mdy | yy).IsValidDate() && ! (editEndDate->Text == ""))
  {
    PSay("Please enter a valid end date", m_invalid_data, MB_OK);
    editEndDate->SetFocus();
    return false;
  }
  else if ((editMinAttendance->Text != "") && (! IsInt(editMinAttendance->Text.c_str())))
  {
    PSay("Please enter a valid min attendance", m_invalid_data, MB_OK);
    editMinAttendance->SetFocus();
    return false;
  }
  else if ((editMaxAttendance->Text != "") && (! IsInt(editMaxAttendance->Text.c_str())))
  {
    PSay("Please enter a valid max attendance", m_invalid_data, MB_OK);
    editMaxAttendance->SetFocus();
    return false;
  }
  else if (! lookupTables->IsInLookUpTable("event_name_types", cboEventName))
  {
    PSay("Please enter a valid event name", m_invalid_data, MB_OK);
    cboEventName->SetFocus();
    return false;
  }
  else if (! lookupTables->IsInLookUpTable("program_types", cboProgramType))
  {
    PSay("Please enter a valid program type", m_invalid_data, MB_OK);
    cboProgramType->SetFocus();
    return false;
  }

  // jhaley 20100728: check this separately so it doesn't cancel out other checks above.
  if (lookupTables->IdOf("groups", "Caregiver Support Program") == group_id)
  {
    if (!lookupTables->IsInLookUpTable("care_service_types", cboEventType->Text.c_str()))
    {
      PSay("Please enter a valid event type", m_invalid_data, MB_OK);
      cboEventType->SetFocus();
      return false;
    }
  }
  else if (!lookupTables->IsInLookUpTable("event_type_types", cboEventType))
  {
    PSay("Please enter a valid event type", m_invalid_data, MB_OK);
    cboEventType->SetFocus();
    return false;
  }

  return true;
}
//---------------------------------------------------------------------------
bool TformSearchEvent::Search()
{
  // jhaley 20100726: refactored for new event_type_types and event_location_types tables
  string sql = "";
  string event_type_table = "event_type_types";
  string event_type_id    = "event_type_id";

  // CSP still has its own service types lookup, for now...
  if (lookupTables->IdOf("groups", "Caregiver Support Program") == group_id)
  {
    event_type_table = "care_service_types";
    event_type_id    = "care_service_id";
  }

  sql += "select events.date_of_event, event_name_types.item, " + event_type_table + ".item, case "
         "when event_location_types.zip_code_id = 0 then event_location_types.item "
         "when event_location_types.item = '' then (zip_codes.city || ', ' || zip_codes.state) "
         "else (zip_codes.city || ', ' || zip_codes.state || ' - ' || event_location_types.item) end event_location, "
         "program_types.item, events.number_attended, events.description, ";

  if (lookupTables->IdOf("groups", "Diabetes") == group_id)
    sql += "events.added_activities, ";

  sql += "events.group_id, events.current_id, events.id "
         "from events "
         "inner join " + event_type_table + " on events." + event_type_id + " = " + event_type_table + ".id "
         "inner join event_location_types on events.event_location_id = event_location_types.id "
         "inner join zip_codes on event_location_types.zip_code_id = zip_codes.id "
         "inner join program_types on events.program_id = program_types.id "
         "inner join event_name_types on events.event_name_id = event_name_types.id "
         "where events.group_id = '" + group_id + "' "; // match only the group_id that was passed in.

  if (editBeginDate->Text != "")
    sql += "and events.date_of_event >= '" + Pdate(editBeginDate->Text.c_str(), mdy | yy).OutDate(mdy | yyyy) + "' ";
  if (editEndDate->Text != "")
    sql += "and events.date_of_event <= '" + Pdate(editEndDate->Text.c_str(), mdy | yy).OutDate(mdy | yyyy) + "' ";
  if (cboEventType->Text != "")
    sql += "and events." + event_type_id + " = '" + lookupTables->IdOf(event_type_table, cboEventType) + "' ";

//  if (editName->Text != "")
//    sql += "and events.event_name like '%" + SafeSQLString(editName->Text.c_str()) + "%' ";
  if (cboEventName->Text != "")
    sql += "and events.event_name_id = '" + lookupTables->IdOf("event_name_types", cboEventName) + "' ";

  vector<string> location_ids = cbddLocation->Filter->GetMultiID();
  if (!location_ids.empty())
    sql += "and " + SafeInClause("events.event_location_id", location_ids) + " ";

  if (cboProgramType->Text != "")
    sql += "and events.program_id = '" + lookupTables->IdOf("program_types", cboProgramType) + "' ";

  if (editMinAttendance->Text != "")
    sql += "and events.number_attended >= " + string(editMinAttendance->Text.c_str()) + " ";
  if (editMaxAttendance->Text != "")
    sql += "and events.number_attended <= " + string(editMaxAttendance->Text.c_str()) + " ";
  if(radioWithActivities->Checked)
    sql += "and events.added_activities = 1 ";
  if(radioWithoutActivities->Checked)
    sql += "and events.added_activities = 0 ";

  Filter->SetSQLStatement(sql);
  Filter->RefreshDataSet();

  return true;
}
//---------------------------------------------------------------------------
bool TformSearchEvent::ScreenStart()
{
  // build the proper security string for the filter
  string filter_security = "events";
  string short_name = LowercaseString(GetOneField(dbDatabase, "select short_item from groups where id = '" + group_id + "' "));

  // hack for Public Health; use com permissions / view
  if (short_name == "pub")
    short_name = "com";
  filter_security += "." + short_name;
  Filter->SetSecurity(filter_security);

  // For Diabetes, enable the activities radio buttons
  if (lookupTables->IdOf("groups", "Diabetes") == group_id)
  {
    radioWithActivities->Enabled = true;
    radioWithoutActivities->Enabled = true;
  }
  else
  {
    radioWithActivities->Enabled = false;
    radioWithoutActivities->Enabled = false;
  }

  vector<string> group_perm_vec = SetToVec(GetSpecialGroupPermissions("general"));
  string also_perm = group_id;

  // Public Health and Caregiver Support should see Community Health items as well.
  if (lookupTables->IdOf("groups", "Public Health") == group_id ||
      lookupTables->IdOf("groups", "Caregiver Support Program") == group_id)
    also_perm = lookupTables->IdOf("groups", "Community Health");

  // jhaley 20100726: coalesced group event tables
  if (lookupTables->IdOf("groups", "Caregiver Support Program") == group_id)
    lookupTables->RegisterComboBox("care_service_types", cboEventType);
  else
  {
    string custom_type_sql = "select id, item, valid from event_type_types "
                             "where (group_id in (0, '" + also_perm + "')) and valid = 1 order by item ";
    lookupTables->RegisterComboBox("event_type_types", cboEventType, custom_type_sql);
  }

  // jhaley 20100913: Added program types to events
  string program_sql = "select id, item, valid from program_types "
                       "where (group_id in (0, '" + also_perm + "')) and valid = 1 order by item ";
  lookupTables->RegisterComboBox("program_types", cboProgramType, program_sql);

  string group_sql = "select id, item, valid from event_name_types "
                     "where (group_id in (0, '" + also_perm + "')) and valid = 1 order by item";
  lookupTables->RegisterComboBox("event_name_types", cboEventName, group_sql, SetGroupIdForNewLookupTableItem);

  Filter->AddColumnSpec("Event Date", "events", as_date, true, 100);
  Filter->AddColumnSpec("Event Name", "events", as_text, true, 150, rawString);
  Filter->AddColumnSpec("Event Type", "events", as_text, true, 140, rawString);
  Filter->AddColumnSpec("Location", "events", as_text, true, 170, rawString);
  Filter->AddColumnSpec("Program Type", "events", as_text, true, 100, rawString);
  Filter->AddColumnSpec("Attendance", "events", as_int, false, 100);
  Filter->AddColumnSpec("Description", "events", as_text, false, 150);
  if(lookupTables->IdOf("groups", "Diabetes") == group_id)
    Filter->AddColumnSpec("Activities Added", "events", as_text, true, 120, prettyBoolString);
  Filter->AddColumnSpec("Group Id",    "internal_id",  as_int, false, 80); // jhaley - I wanna SEE this and know it is right.
  Filter->AddColumnSpec("Current Id",  "current_id",  as_int, false, 80);
  Filter->AddColumnSpec("Internal Id", "internal_id", as_int, false, 80);
  Filter->FetchPreferences("searchevent.filter");
  Filter->SetIconStatus(true);
  Filter->SetSQLStatement("");
  Filter->RefreshDataSet();

  string event_loc_sql = "select case "
                         "when event_location_types.zip_code_id = 0 then item "
                         "when item = '' then (city || ', ' || state) "
                         "else (city || ', ' || state || ' - ' || item) end, " + CleverCurrentId("valid", "1", "Current", "Expired") + ", event_location_types.id "
                         "from event_location_types "
                         "inner join zip_codes on event_location_types.zip_code_id = zip_codes.id "
                         "where (group_id = 0 or group_id = '" + also_perm + "') "
                         "and valid = 1 order by state, city, item ";

  cbddLocation->Filter->SetSecurity("");
  cbddLocation->Filter->AddColumnSpec("Location", "", as_text, true, 250, rawString);
  cbddLocation->Filter->AddColumnSpec("Current Id", "current_id", as_int, false, 40);
  cbddLocation->Filter->AddColumnSpec("Internal Id", "internal_id", as_int, false, 60);
  cbddLocation->Filter->SetSQLStatement(event_loc_sql);
  cbddLocation->Filter->FetchPreferences("searchevent.location_filter");
  cbddLocation->Filter->SetIconStatus(true);

  // jhaley 20100913: Seems it is somehow too early for these to be run from here. See the OnDown event handler below for a
  // timing that seems to function properly.
  //cbddLocation->Filter->ForceRefreshDataSet();
  //cbddLocation->DisplayFieldName = "Location";

  return true;
}
//---------------------------------------------------------------------------
bool TformSearchEvent::ScreenStop()
{
  lookupTables->UnregisterComboBox(cboEventType);
  lookupTables->UnregisterComboBox("event_name_types", cboEventName);
  lookupTables->UnregisterComboBox("program_types", cboProgramType);
  return true;
}
//---------------------------------------------------------------------------
bool TformSearchEvent::Create()
{
  map<string,string> temp_map;
  temp_map["id"] = "new";
  temp_map["group_id"] = group_id; // pass on the group_id which we were passed
  FORM_TRY
    TformRecordEvent * eventRecord = new TformRecordEvent(NULL, temp_map);
    eventRecord->Show();
  FORM_CATCH
  return true;
}
//---------------------------------------------------------------------------
bool TformSearchEvent::Zoom(const string & p_id)
{
  map<string,string> temp_map;
  temp_map["id"] = p_id;
  FORM_TRY
    TformRecordEvent * eventRecord = new TformRecordEvent(NULL, temp_map);
    eventRecord->Show();
  FORM_CATCH
  return true;
}
//---------------------------------------------------------------------------
// jhaley: warning - calling this will cause the search to match events for no particular department. There are not any
// calls to this in Prometheus currently, and we probably shouldn't add any. The new events code doesn't like not having
// a group_id for an event anyway.
vector<string> TformSearchEvent::Do(TformSpecialGeneric * p_parent)
{
 return TformSearchEvent::Do(p_parent, "0");
}
//---------------------------------------------------------------------------
vector<string> TformSearchEvent::Do(TformSpecialGeneric * p_parent, string group_id)
{
  vector<string> answer;
  FORM_TRY
    TformSearchEvent * z = new TformSearchEvent(NULL, &answer, group_id);
    z->ShowTaskModal(p_parent);
  FORM_CATCH
  return answer;
}
//---------------------------------------------------------------------------
// jhaley 20110125: Code moved here from TformPrometheusMenu
void TformSearchEvent::DoMedical(TForm *parent)
{
  string group_id = "";
  map<string, string> params;
  params["prefix"] = "table.events";
  params["suffix"] = "";
  params["key"]    = "view";
  params["value"]  = "true";
  params["instructions"] = "You have permission to view events for multiple clinics. Please select the clinic for which to view events.";
  params["include"] = lookupTables->IdOf("groups", "Little Axe Clinic") + ", " + lookupTables->IdOf("groups", "Shawnee Clinic");
  params["exclude"] = "";
  params["table"]   = "groups";
  params["sql"]     = "select id row_id, short_item row_value, item show_value from groups where valid = 1 and id > 0";

  group_id = TformSpecialDisambiguate::Disambiguate(parent, params, PrettyString);
  if (group_id != "")
  {
    TformSearchEvent *event_search = NULL;
    FORM_TRY
      event_search = new TformSearchEvent(NULL, NULL, group_id);
    FORM_CATCH_RETURN("TformSearchEvent", )
    event_search->Show();
  }
}
//---------------------------------------------------------------------------
void __fastcall TformSearchEvent::cbddLocationDown(TObject *Sender)
{
  // jhaley 20100913: In the continuing intrigue surrounding invisible listview crashes, it seems calling these
  // from StartScreen is somehow too early. Instead, do it when the CBDD receives an arrow-down event. The only
  // side effect seems to be that the box has to refresh (almost?) every time you look at it, which is better
  // than an access violation at least. Otherwise it works very well.
  cbddLocation->Filter->ForceRefreshDataSet();
  cbddLocation->DisplayFieldName = "Location";
}
//---------------------------------------------------------------------------
