//---------------------------------------------------------------------------

#include <vcl.h>
#pragma hdrstop

#include "codeSearchFDADrug.h"
#include "codeSpecialFDADrug.h"
// Needed only if uncommenting DebugSql:
//#include "derivedheaders.h"
//---------------------------------------------------------------------------
#pragma package(smart_init)
#pragma link "codeRubberContainer"
#pragma link "codeSearchGeneric"
#pragma resource "*.dfm"
TformSearchFDADrug *formSearchFDADrug;
//---------------------------------------------------------------------------
__fastcall TformSearchFDADrug::TformSearchFDADrug(TComponent* Owner, pvecstr *p_return_vector)
        : TformSearchGeneric(Owner, p_return_vector, "fda_listings")
{
  no_perm_needed = true; // don't use permissions; this is public-domain data that anyone can search/see
  Initialize();
  buttonSearch->Enabled = true;
  name_return_vec = NULL;
}
//---------------------------------------------------------------------------
__fastcall TformSearchFDADrug::TformSearchFDADrug(TComponent *Owner, pvecstr *p_id_return_vec, pvecstr *p_name_return_vec)
        : TformSearchGeneric(Owner, p_id_return_vec, "fda_listings")
{
  no_perm_needed = true;
  Initialize();
  buttonSearch->Enabled = true;
  name_return_vec = p_name_return_vec;
}
//---------------------------------------------------------------------------
bool TformSearchFDADrug::Initialize()
{
  try
  {
    ConnectToDatabase(dbFDADatabase, "10.1.1.109:/opt/interbase/db/fdadrugs.gdb", "ZEUS", "argon");

    dbTransaction = new TIBTransaction(this);
    if(StdTransaction(dbTransaction, dbFDADatabase))
      Filter->SetTransaction(dbTransaction); // use custom transaction against FDA database
    else
    {
      PSay("Could not open transaction against FDA drug database", m_database_error, MB_OK);
      return false;
    }
  }
  catch(...)
  {
    PSay("An error occurred while connecting to the FDA drug database", m_database_error, MB_OK);
    return false;
  }

  return TformSearchGeneric::Initialize();
}
//---------------------------------------------------------------------------
bool TformSearchFDADrug::IsValidOnScreen()
{
  if(editFDAid->Text != "" && !IsInt(editFDAid->Text.c_str()))
  {
    PSay("Please enter a numeric FDA drug ID", m_invalid_data, MB_OK);
    editFDAid->SetFocus();
    return false;
  }

  if(editDEASchedule->Text != "" && !IsInt(editDEASchedule->Text.c_str()))
  {
    PSay("Please enter a valid DEA schedule number", m_invalid_data, MB_OK);
    editDEASchedule->SetFocus();
    return false;
  }


  return TformSearchGeneric::IsValidOnScreen();
}
//---------------------------------------------------------------------------
bool TformSearchFDADrug::Search()
{
  psetstr total_ids;

  // The drug database has some huge tables. If these queries are done in the usual manner (using where exists clauses),
  // the resulting full joins can take 30 minutes to run. Instead, we build a set of IDs manually by intersecting
  // separate smaller queries that run very fast. At the end, if the result set is too large, we request more input.
  // This otherwise runs into Firebird 2.5's 32K'ish query size limit (the actual cut-off is slightly higher than 32KB,
  // who knows why; for example it'll throw a syntax error at column 41772 - examination of the SQL shows it's just
  // fine syntactically - that's where Firebird says 'To hell with this!').

  // set_intersection is a standard library algorithm. This method of using it on std::set I must credit to "some
  // dude from stackoverflow".

  if(editDEASchedule->Text != "" && IsInt(editDEASchedule->Text.c_str()))
  {
    string sched_sql = string(
      "select fda_schedules.listing_id "
      "from fda_schedules "
      "where fda_schedules.schedule_num = ") + editDEASchedule->Text.c_str();
    SqlToSet(dbTransaction, sched_sql, total_ids);
  }
  if(editDoseForm->Text != "")
  {
    psetstr doseform_ids;
    string doseforms_sql = string(
      "select fda_doseforms.listing_id "
      "from fda_doseforms "
      "left join dosage_types on fda_doseforms.dosage_types_id = dosage_types.id "
      "where dosage_types.item like '%") + SafeSQLString(editDoseForm->Text.c_str()) + "%'";
    SqlToSet(dbTransaction, doseforms_sql, doseform_ids);
    if(total_ids.empty())
      total_ids = doseform_ids;
    else
    {
      psetstr temp_set = total_ids;
      total_ids.clear();
      set_intersection(temp_set.begin(), temp_set.end(), doseform_ids.begin(), doseform_ids.end(),
                       std::inserter(total_ids, total_ids.begin()));
    }
  }
  if(editPkgType->Text != "")
  {
    psetstr pkgtype_ids;
    string pkg_sql = string(
      "select fda_packages.listing_id "
      "from fda_packages "
      "left join package_types on fda_packages.package_type_id = package_types.id "
      "where package_types.item like '%") + SafeSQLString(editPkgType->Text.c_str()) + "%'";
    SqlToSet(dbTransaction, pkg_sql, pkgtype_ids);
    if(total_ids.empty())
      total_ids = pkgtype_ids;
    else
    {
      psetstr temp_set = total_ids;
      total_ids.clear();
      set_intersection(temp_set.begin(), temp_set.end(), pkgtype_ids.begin(), pkgtype_ids.end(),
                       std::inserter(total_ids, total_ids.begin()));
    }
  }
  if(editRoute->Text != "")
  {
    psetstr route_ids;
    string route_sql = string(
      "select fda_routes.listing_id "
      "from fda_routes "
      "left join route_names on fda_routes.route_name_id = route_names.id "
      "where route_names.item like '%") + SafeSQLString(editRoute->Text.c_str()) + "%'";
    SqlToSet(dbTransaction, route_sql, route_ids);
    if(total_ids.empty())
      total_ids = route_ids;
    else
    {
      psetstr temp_set = total_ids;
      total_ids.clear();
      set_intersection(temp_set.begin(), temp_set.end(), route_ids.begin(), route_ids.end(),
                       std::inserter(total_ids, total_ids.begin()));
    }
  }
  if(editIngredient->Text != "")
  {
    psetstr ing_ids;
    string ing_sql = string(
      "select fda_formulations.listing_id "
      "from fda_formulations "
      "left join formula_names on fda_formulations.formula_name_id = formula_names.id "
      "where formula_names.item like '%") + SafeSQLString(editIngredient->Text.c_str()) + "%'";
    SqlToSet(dbTransaction, ing_sql, ing_ids);
    if(total_ids.empty())
      total_ids = ing_ids;
    else
    {
      psetstr temp_set = total_ids;
      total_ids.clear();
      set_intersection(temp_set.begin(), temp_set.end(), ing_ids.begin(), ing_ids.end(),
                       std::inserter(total_ids, total_ids.begin()));
    }
  }

  // Do the main query. This one tends to run fine on its own.
  bool some_criteria_specified = false;
  string sql =
    "select fda_listings.id "
    "from fda_listings "
    "left join listing_names on fda_listings.name_id = listing_names.id "
    "left join listing_strengths on fda_listings.strength_id = listing_strengths.id "
    "left join listing_units on fda_listings.listing_unit_id = listing_units.id "
    "left join fda_firms on fda_listings.firm_id = fda_firms.id "
    "where 1=1 ";

  if(editName->Text != "")
  {
    some_criteria_specified = true;
    sql += "and listing_names.item like '%"
           + SafeSQLString(ReplaceStringWithString(editName->Text.c_str(), " ", "%")) + "%' ";
  }
  if(editManufacturer->Text != "")
  {
    some_criteria_specified = true;
    sql += "and fda_firms.firm_name like '%"
           + SafeSQLString(ReplaceStringWithString(editManufacturer->Text.c_str(), " ", "%")) + "%' ";
  }
  if(editFDAid->Text != "" && IsInt(editFDAid->Text.c_str()))
  {
    some_criteria_specified = true;
    sql += string("and fda_listings.id = '") + editFDAid->Text.c_str() + "' ";
  }
  if(editFDACode->Text != "")
  {
    some_criteria_specified = true;
    sql += string("and fda_listings.prodcode = '") + SafeSQLString(editFDACode->Text.c_str()) + "' ";
  }
  if(editStrength->Text != "")
  {
    some_criteria_specified = true;
    sql += "and listing_strengths.item like '%"
           + SafeSQLString(ReplaceStringWithString(editStrength->Text.c_str(), " ", "%")) + "%' ";
  }
  if(editUnits->Text != "")
  {
    some_criteria_specified = true;
    sql += "and listing_units.item like '%"
           + SafeSQLString(ReplaceStringWithString(editUnits->Text.c_str(), "%", "%%")) + "%' ";
  }
  if(editRxOTC->Text != "")
  {
    some_criteria_specified = true;
    sql += "and fda_listings.rx_otc = '" + SafeSQLString(editRxOTC->Text.c_str()) + "' ";
  }

  if(some_criteria_specified)
  {
    psetstr main_ids;
    SqlToSet(dbTransaction, sql, main_ids);
    if(total_ids.empty())
      total_ids = main_ids;
    else
    {
      psetstr temp_set = total_ids;
      total_ids.clear();
      set_intersection(temp_set.begin(), temp_set.end(), main_ids.begin(), main_ids.end(),
                       std::inserter(total_ids, total_ids.begin()));
    }
  }

  // Final SQL for the filter - use an "in" clause on the set of IDs that was built.
  string final_sql =
    "select listing_names.item listing_name, fda_firms.firm_name, fda_listings.id, fda_listings.prodcode, "
    "listing_strengths.item, listing_units.item, fda_listings.rx_otc, 4, fda_listings.id "
    "from fda_listings "
    "left join listing_names on fda_listings.name_id = listing_names.id "
    "left join listing_strengths on fda_listings.strength_id = listing_strengths.id "
    "left join listing_units on fda_listings.listing_unit_id = listing_units.id "
    "left join fda_firms on fda_listings.firm_id = fda_firms.id "
    "where ";

  // If some external criteria were specified but the criteria ended up being mutually exclusive, no results
  // should be returned.
  if(total_ids.empty())
    final_sql += "0=1 ";
  else
    final_sql += SafeInClause("fda_listings.id", SetToVec(total_ids)) + " ";

  //DebugSql(this, &final_sql, other_forms, 9);

  if(final_sql.length() >= 1024*32) // ~32KB is apparently the largest query that Firebird 2.5 can handle.
  {
    PSay("Too many results match your request. Please specify additional terms for your search.",
         m_missing_data, MB_OK);
    return false; // woops
  }

  Filter->SetSQLStatement(final_sql);
  Filter->RefreshDataSet();

  return true;
}
//---------------------------------------------------------------------------
bool TformSearchFDADrug::ScreenStart()
{
  Filter->SetSecurity("");
  Filter->AddColumnSpec("Name",         "", as_text, true,  130, prettyString);
  Filter->AddColumnSpec("Manufacturer", "", as_text, true,  130, prettyString);
  Filter->AddColumnSpec("FDA ID",       "", as_text, false,  90, rawString);
  Filter->AddColumnSpec("FDA Code",     "", as_text, false,  90, rawString);
  Filter->AddColumnSpec("Strengths",    "", as_text, true,   80, rawString);
  Filter->AddColumnSpec("Units",        "", as_text, true,   80, lowercaseString);
  Filter->AddColumnSpec("Rx/OTC",       "", as_text, true,   60, rawString);
  Filter->AddColumnSpec("Current Id",   "current_id",  as_int, false, 80);
  Filter->AddColumnSpec("Internal Id",  "internal_id", as_int, false, 80);
  Filter->FetchPreferences("searchfdadrug.filter");
  Filter->SetIconStatus(true);
  Filter->SetSQLStatement("");
  Filter->RefreshDataSet();

  return TformSearchGeneric::ScreenStart();
}
//---------------------------------------------------------------------------
bool TformSearchFDADrug::Zoom(const string &p_id)
{
  FORM_TRY
    TformSpecialFDADrug *drugForm = new TformSpecialFDADrug(NULL, p_id);
    drugForm->Show();
  FORM_CATCH

  return true;
}
//---------------------------------------------------------------------------
pvecstr TformSearchFDADrug::Do(TformSpecialGeneric *p_parent)
{
  pvecstr ids;
  pvecstr answer;
  FORM_TRY
    TformSearchFDADrug *z = new TformSearchFDADrug(NULL, &ids, &answer);
    z->ShowTaskModal(p_parent);
  FORM_CATCH
  return answer; // We want the names, not the IDs.
}
//---------------------------------------------------------------------------
void TformSearchFDADrug::Do(string init_id, string init_name)
{
  TformSearchFDADrug *z = NULL;
  FORM_TRY
    z = new TformSearchFDADrug(NULL, NULL);
  FORM_CATCH
  if(z)
  {
    z->Show();
    if(init_id != "")
      z->editFDAid->Text = init_id.c_str();
    if(init_name != "")
      z->editName->Text = init_name.c_str();
    z->buttonSearchClick(NULL);
  }
}
//---------------------------------------------------------------------------
void __fastcall TformSearchFDADrug::buttonSelectClick(TObject *Sender)
{
  // Custom logic is needed here for returning the drug names vector when so requested
  if(name_return_vec)
  {
    pvecmap all_data = Filter->GetAllData(true, true); // selected rows only!
    for(pvecmap::iterator i_dat = all_data.begin(); i_dat != all_data.end(); ++i_dat)
    {
      // Return the IDs combined with the names as string values.
      // This allows a cascading behavior in case IDs should vanish in future versions of the FDA database -
      // we then instead search by name when trying to find the same drug again later. This avoids any hard links
      // between the two independent databases.
      string combined_val = (*i_dat)["id"] + " - " + (*i_dat)["listing_name"];
      name_return_vec->push_back(combined_val);
    }
  }

  // Do normal parent class event handling
  TformSearchGeneric::buttonSelectClick(Sender);        
}
//---------------------------------------------------------------------------

