//---------------------------------------------------------------------------

#include <vcl.h>
#pragma hdrstop
 
#include "codeSearchInsurancePolicy.h"
#include "codeRecordInsurancePolicy.h"
#include "codeSearchTaxEntity.h"
//---------------------------------------------------------------------------
#pragma package(smart_init)
#pragma link "codeSearchGeneric"
#pragma link "codeRubberContainer"
#pragma resource "*.dfm"
TformSearchInsurancePolicy *formSearchInsurancePolicy;
//---------------------------------------------------------------------------
__fastcall TformSearchInsurancePolicy::TformSearchInsurancePolicy(TComponent* Owner, vector<string> * p_return_vector)
        : TformSearchGeneric(Owner, p_return_vector, string ("insurance_policies"))
{
  company_id = "";

  benefits_filter.SetListViewHandle(listBenefits);
  benefits_filter.SetSecurity("insurance_benefits");
  benefits_filter.AddColumnSpec("Area", "insurance_benefits", as_text, true, 120);
  benefits_filter.AddColumnSpec("Current Id", "current_id", as_int, false, 80);
  benefits_filter.AddColumnSpec("Internal Id", "internal_id", as_int, false, 80);
  benefits_filter.SetCompress(false);
  benefits_filter.SetIconStatus(false);
  benefits_filter.SetHeader(false);
  benefits_filter.SetCheck(true);
  benefits_filter.FetchPreferences("searchinsurancepolicy.benefits_filter");
  benefits_filter.SetSQLStatement("select item, 4, id from area_types where id > 0");
  benefits_filter.RefreshDataSet();

  Initialize();
}
//---------------------------------------------------------------------------
bool TformSearchInsurancePolicy::IsValidOnScreen()
{
  // check to make sure that the values entered in the combo boxes
  // corrospond to values in that look up table.
  // give the user a message box and prompt to re-enter if incorrect
  if (! Pdate (editBeginDate->Text.c_str(), mdy | yy).IsValidDate() && ! (editBeginDate->Text == ""))
  {
    PSay("Please enter a valid begin date", m_invalid_data, MB_OK);
    editBeginDate->SetFocus();
    return false;
  }
  else if (! Pdate (editEndDate->Text.c_str(), mdy | yy).IsValidDate() && ! (editEndDate->Text == ""))
  {
    PSay("Please enter a valid end date", m_invalid_data, MB_OK);
    editEndDate->SetFocus();
    return false;
  }
  else if (! lookupTables->IsInLookUpTable("coverage_types", cboCoverage->Text.c_str()))
  {
    PSay("Please enter a valid type of coverage", m_invalid_data, MB_OK);
    cboCoverage->SetFocus();
    return false;
  }
  else if (! lookupTables->IsInLookUpTable("insurance_types", cboInsurance->Text.c_str()))
  {
    PSay("Please enter a valid type of insurance", m_invalid_data, MB_OK);
    cboInsurance->SetFocus();
    return false;
  }
  else if (! lookupTables->IsInLookUpTable("source_types", cboSource->Text.c_str()))
  {
    PSay("Please enter a valid source", m_invalid_data, MB_OK);
    cboSource->SetFocus();
    return false;
  }
  else
    return true;
}
//---------------------------------------------------------------------------
bool TformSearchInsurancePolicy::Search()
{
 string sql = "";

  sql = "select distinct tax_entities.name, insurance_policies.policy_number, insurance_policies.group_number, "
        "insurance_policies.start_date, insurance_policies.stop_date, insurance_types.item, coverage_types.item, "
        "source_types.item, insurance_policies.name_on_policy, insurance_policies.bin_number, insurance_policies.processor_control_number, "
        "insurance_policies.current_id, insurance_policies.id "
        "from insurance_policies "
        "inner join tax_entities on insurance_policies.tax_entity_id = tax_entities.id "
        "left join tax_addresses on tax_addresses.tax_entity_id = tax_entities.id "
        "inner join coverage_types on insurance_policies.coverage_id = coverage_types.id "
        "inner join insurance_types on insurance_policies.insurance_id = insurance_types.id "
        "inner join source_types on insurance_policies.source_id = source_types.id "
        "where insurance_policies.id > 0 ";

  string policy_number_string = SafeSQLString(UppercaseString(PowerSearchString( StripAllExcept(LowercaseString(editPolicyNumber->Text.c_str()), alpha_numerics) )));
  if(editCompany->Text != "")
    sql += "and (upper(tax_entities.name) like '%" + SafeSQLString(ReplaceStringWithString(editCompany->Text.c_str(), " ", "%")) + "%' or "
                "upper(tax_entities.name) like '" + SafeSQLString(InitialsSearchString1(editCompany->Text.c_str())) + "' or "
                "upper(tax_entities.name) like '" + SafeSQLString(InitialsSearchString2(editCompany->Text.c_str())) + "' or "
                "upper(tax_addresses.plan_name) like '%" + SafeSQLString(ReplaceStringWithString(editCompany->Text.c_str(), " ", "%")) + "%' or "
                "upper(tax_addresses.plan_name) like '" + SafeSQLString(InitialsSearchString1(editCompany->Text.c_str())) + "' or "
                "upper(tax_addresses.plan_name) like '" + SafeSQLString(InitialsSearchString2(editCompany->Text.c_str())) + "') ";
  if(editPolicyNumber->Text != "")
    sql += "and insurance_policies.policy_number like '%" + policy_number_string + "%' ";
  if(editGroupNumber->Text != "")
    sql += "and insurance_policies.group_number like '%" + SafeSQLString(editGroupNumber->Text.c_str()) + "%' ";
  if (editBeginDate->Text != "")
    sql += "and insurance_policies.start_date >= '" + Pdate(editBeginDate->Text.c_str(), mdy | yy).OutDate(mdy | yyyy) + "' ";
  if (editEndDate->Text != "")
    sql += "and insurance_policies.start_date <= '" + Pdate(editEndDate->Text.c_str(), mdy | yy).OutDate(mdy | yyyy) + "' ";
  if (cboInsurance->Text != "")
    sql += "and insurance_policies.insurance_id = " + lookupTables->IdOf("insurance_types", cboInsurance->Text.c_str()) + " ";
  if (cboCoverage->Text != "")
    sql += "and insurance_policies.coverage_id = " + lookupTables->IdOf("coverage_types", cboCoverage->Text.c_str()) + " ";
  if (cboSource->Text != "")
    sql += "and insurance_policies.source_id = " + lookupTables->IdOf("source_types", cboSource->Text.c_str()) + " ";
  if(editName->Text != "")
    sql += "and insurance_policies.name_on_policy like '%" + SafeSQLString(editName->Text.c_str()) + "%' ";

  vector<string> wanted_areas = benefits_filter.GetMultiID();
  if (!wanted_areas.empty())
   sql += "and " + IntToString(wanted_areas.size()) +
          " = (select count(insurance_benefits.id) from insurance_benefits "
          "where insurance_benefits.insurance_policy_id = insurance_policies.id "
          "and insurance_benefits.area_id in (" + VecToCommaString(wanted_areas) + ") and insurance_benefits.is_covered = '1') ";

  Filter->SetSQLStatement (sql);
  Filter->RefreshDataSet ();

  return true;
}
//---------------------------------------------------------------------------
bool TformSearchInsurancePolicy::ScreenStart()
{
  lookupTables->RegisterComboBox("coverage_types", cboCoverage);
  lookupTables->RegisterComboBox("source_types", cboSource);
  lookupTables->RegisterComboBox("insurance_types", cboInsurance);

  Filter->SetSecurity("insurance_policies");
  Filter->AddColumnSpec("Company", "tax_entities", as_text, true, 140, rawString);
  Filter->AddColumnSpec("Policy Number", "insurance_policies", as_text, true, 110, rawString);
  Filter->AddColumnSpec("Group Number", "insurance_policies", as_text, false, 100, rawString);
  Filter->AddColumnSpec("Effective Date", "insurance_policies", as_date, true, 100);
  Filter->AddColumnSpec("Termination Date", "insurance_policies", as_date, false, 100);
  Filter->AddColumnSpec("Insurance", "tax_entities", as_text, true, 100);
  Filter->AddColumnSpec("Coverage", "insurance_policies", as_text, false, 100);
  Filter->AddColumnSpec("Source", "insurance_policies", as_text, false, 100);
  Filter->AddColumnSpec("Name on Policy", "insurance_policies", as_text, true, 120);
  Filter->AddColumnSpec("BIN #", "insurance_policies", as_text, false, 75, rawString);
  Filter->AddColumnSpec("PCN", "insurance_policies", as_text, false, 65, rawString);
  Filter->AddColumnSpec("Current Id", "current_id", as_int, false, 80);
  Filter->AddColumnSpec("Internal Id", "internal_id", as_int, false, 80);
  Filter->FetchPreferences("searchinsurancepolicy.filter");
  Filter->SetIconStatus(true);
  Filter->SetSQLStatement("");
  Filter->RefreshDataSet();

  return true;
}
//---------------------------------------------------------------------------
bool TformSearchInsurancePolicy::ScreenStop()
{
 // instead of formclose, do shutdown stuff here (combo boxes, etc.)
  lookupTables->UnregisterComboBox("coverage_types", cboCoverage);
  lookupTables->UnregisterComboBox("source_types", cboSource);
  lookupTables->UnregisterComboBox("insurance_types", cboInsurance);
  return true;
}
//---------------------------------------------------------------------------
bool TformSearchInsurancePolicy::Create()
{
  map<string,string> temp_map;
  temp_map["id"] = "new";
  FORM_TRY
    TformRecordInsurancePolicy * policyRecord = new TformRecordInsurancePolicy(NULL, temp_map);
    policyRecord->Show();
  FORM_CATCH
  return true;
}
//---------------------------------------------------------------------------
bool TformSearchInsurancePolicy::Zoom(const string & p_id)
{
  map<string,string> temp_map;
  temp_map["id"] = p_id;
  FORM_TRY
    TformRecordInsurancePolicy * policyRecord = new TformRecordInsurancePolicy(NULL, temp_map);
    policyRecord->Show();
  FORM_CATCH
  return true;
}
//---------------------------------------------------------------------------
vector<string> TformSearchInsurancePolicy::Do(TformSpecialGeneric * p_parent)
{
 vector<string> answer;
 FORM_TRY
  TformSearchInsurancePolicy * z = new TformSearchInsurancePolicy(NULL, &answer);
  z->ShowTaskModal(p_parent);
 FORM_CATCH
 return answer;
}
//---------------------------------------------------------------------------


