//---------------------------------------------------------------------------

#include <vcl.h>
#pragma hdrstop

#include "codeSearchPerson.h"
#include "codeRecordPerson.h"
#include "codeWizardNewPerson.h"
//---------------------------------------------------------------------------
#pragma package(smart_init)
#pragma link "codeSearchGeneric"
#pragma link "codeSearchGeneric"
#pragma link "codeRubberContainer"
#pragma link "codeBaseDropDown"
#pragma link "codeCheckListDropDown"
#pragma resource "*.dfm"
TformSearchPerson *formSearchPerson;
//---------------------------------------------------------------------------
__fastcall TformSearchPerson::TformSearchPerson(TComponent* Owner, vector<string> * p_return_vector)
        : TformSearchGeneric(Owner, p_return_vector, string ("people"))
{
  Initialize();
}
//---------------------------------------------------------------------------
bool TformSearchPerson::IsValidOnScreen()
{
  // check to make sure that the values entered in the combo boxes
  // corrospond to values in that look up table.
  // give the user a message box and prompt to re-enter if incorrect
  if (! lookupTables->IsInLookUpTable("tribe_types", cboTribe->Text.c_str()))
  {
    PSay("Please enter a valid tribe", m_invalid_data, MB_OK);
    cboTribe->SetFocus();
    return false;
  }
  else if (! lookupTables->IsInLookUpTable("state_types", cboState->Text.c_str()))
  {
    PSay("Please enter a valid state", m_invalid_data, MB_OK);
    cboState->SetFocus();
    return false;
  }
  else if (! lookupTables->IsInLookUpTable("gender_types", cboGender->Text.c_str()))
  {
    PSay("Please enter a valid gender", m_invalid_data, MB_OK);
    cboGender->SetFocus();
    return false;
  }

  if (editDOB->Text != "")
  {
    Pdate dob = Pdate(editDOB->Text.c_str(), global_mode);
    Pdate now = Pdate(Now());
    if(!dob.IsValidDate() || dob.year > now.year + 1000) // jhaley: catch unreasonable date typos here before they turn into IBExceptions
    {
      PSay("Please enter a valid date of birth", m_invalid_data, MB_OK);
      editDOB->SetFocus();
      return false;
    }
  }

  return true;
}
//---------------------------------------------------------------------------
bool TformSearchPerson::Search()
{
 string fields = "";
 string from = "";
 string where = "";
 string sql = "";

 /* Yes, there's a charts.group_id, and no, it's not what I want to join on.
    Here's why:  location_id will join Dental Clinic to LAC chart numbers,
    which is what we want (now that they use LAC charts).  It won't even
    matter if they have permission to see both sets of charts; it's select
    first 1 ... So yes, it looks weird, but it's intentional and proper.*/
 set<string> can_see_charts = GetSpecialGroupPermissions("charts");
 sql = "select "
       "person_names.last_name, person_names.first_name, person_names.middle_name, "
       "person_names.suffix, affiliations.cdib, "
       "tribe_types.item tribe, affiliations.join_date, affiliations.relinquish_date, "
       "people.birth_date, people.social_security_number, "
       "(select first 1 charts.chart_number from charts "
       "inner join groups on charts.location_id = groups.chart_location_id and groups.has_charts = 1 and charts.location_id > 0 "
       "where charts.person_id = people.id and " + SafeInClause("groups.id", SetToVec(can_see_charts)) +
/*
 if(optionsPermissions["special.lac.charts"]["enabled"] == "true")
   sql += " and charts.group_id = " + lookupTables->IdOf("groups", "Little Axe Clinic");
 else if(optionsPermissions["special.sc.charts"]["enabled"] == "true")
   sql += " and charts.group_id = " + lookupTables->IdOf("groups", "Shawnee Clinic");
 else if(optionsPermissions["special.den.charts"]["enabled"] == "true")
   sql += " and charts.group_id = " + lookupTables->IdOf("groups", "Little Axe Clinic");
 else if((optionsPermissions["special.cob.general"]["enabled"] == "true") ||
   (optionsPermissions["special.bill.general"]["enabled"] == "true") ||
   (optionsPermissions["special.res.general"]["enabled"] == "true"))
   sql += " and charts.group_id = " + lookupTables->IdOf("groups", "Contract Health");
*/
       " and charts.current_id+1 > 3 order by groups.hl7_priority asc nulls last) chart_num, "
       "(select first 1 conditions.code from conditions "
       "where conditions.person_id = people.id) code_num, "
       "person_contact_info.line1 || case when person_contact_info.line2 != '' then ' ' || person_contact_info.line2 else '' end address, "
       "person_contact_info.city, state_types.abbreviation state, person_contact_info.zip, "
       "person_contact_info.phone_number, gender_types.item, enrollment_status_types.item, "
       "people.current_id, people.id "                                              // v- they wanted it not to show affiliations that looked "current" because the person is current, when the affiliation isn't current.  So we want blank fields if not ... yeah.
       "from people "
       "left join affiliations on affiliations.id = people.affiliations_id and affiliations.current_id >= 3 "
       "left join tribe_types on affiliations.tribe_id = tribe_types.id "
       "left join person_names on person_names.id = people.person_names_id "
       "left join person_contact_info on person_contact_info.id = people.person_contact_info_id "
       "left join state_types on person_contact_info.state_id = state_types.id "
       "left join gender_types on people.gender_id = gender_types.id "
       "left join enrollment DCE on people.id = DCE.person_id and DCE.enrollment_id = '" + lookupTables->IdOf("enrollment_types", "Diabetes Client") + "' and "
       " DCE.current_id+1 >= 4 "
       "left join enrollment_status_types on DCE.enrollment_status_id = enrollment_status_types.id "
       "where people.id + 1 > 1 ";

  if (editLastName->Text != "")
    sql += "and exists (select person_names.last_name from person_names "
           "where person_names.person_id = people.id and person_names.current_id > 0 and "
           "person_names.last_name like '%" + SafeSQLString(ReplaceStringWithString(editLastName->Text.c_str(), " ", "%")) + "%') ";
  if (editFirstName->Text != "")
    sql += "and exists (select person_names.first_name from person_names "
           "where person_names.person_id = people.id and person_names.current_id > 0 and "
           "person_names.first_name like '%" + SafeSQLString(ReplaceStringWithString(editFirstName->Text.c_str(), " ", "%")) + "%') ";
  if (editCDIB->Text != "")
    sql += "and exists (select affiliations.cdib from affiliations "
           "where affiliations.person_id = people.id and person_names.current_id > 0 and "
           "affiliations.cdib like '" + SafeSQLString(editCDIB->Text.c_str()) + "%') ";
  if (cboTribe->Text != "")
    sql += "and exists (select tribe_types.id from affiliations inner join tribe_types on affiliations.tribe_id = tribe_types.id "
           "where tribe_types.id = " + lookupTables->IdOf("tribe_types", cboTribe->Text.c_str()) + " and "
           "affiliations.person_id = people.id and person_names.current_id > 0) ";
  if (editDOB->Text != "")
    sql += "and people.birth_date = '" + Pdate(editDOB->Text.c_str(), mdy | yy).OutDate(mdy | yyyy) + "' ";
  if (editSSN->Text != "")
    sql += "and people.social_security_number like '" + StripAllExcept(editSSN->Text.c_str(), uints) + "%' ";
  if (editChartNumber->Text != "")
  {
    if (optionsPermissions["special.search.person_id"]["enabled"] == "true")
    {
      set<string> id;
      // allow delimitation by various things, because I'm feeling nice.  which is the only reason they get delimitation at ALL, incidentally
      SplitStringToSet(ReplaceStringWithString(ReplaceAnyCharWithString(editChartNumber->Text.c_str(), " ;/", ","), ",,", ",", true), ",", id);
      set<string>::iterator itr = id.begin();
      while (itr != id.end())
      {
        if (!IsInt(*itr))
        { set<string>::iterator tmp = itr; tmp++; id.erase(itr); itr = tmp; }
        else itr++;
      }
      sql += "and people.id in (" + SetToCommaString(id) + ") ";
    }
    else
      sql += "and exists (select charts.chart_number from charts "
             "where charts.person_id = people.id and "
             "charts.chart_number like '%" + SafeSQLString(editChartNumber->Text.c_str()) + "%') ";
  }
  if (editCode->Text != "")
  {
    sql += "and exists (select conditions.code from conditions "
           "where conditions.person_id = people.id and "
           "conditions.condition_id != " + lookupTables->IdOf("condition_types", "Relative") + " and "
           "conditions.code like '%" + SafeSQLString(editCode->Text.c_str()) + "%' ";
    if (optionsPermissions["table.conditions.diab"]["view"] != "true")
      sql += "and conditions.group_id != " + lookupTables->IdOf("groups", "Diabetes") + " ";
    if (optionsPermissions["table.conditions.beh"]["view"] != "true")
      sql += "and conditions.group_id != " + lookupTables->IdOf("groups", "Behavioral Health") + " ";
    if (optionsPermissions["table.conditions.common"]["view"] != "true")
      sql += "and conditions.group_id != 0 ";
    if (optionsPermissions["table.conditions.com"]["view"] != "true")
      sql += "and conditions.group_id != " + lookupTables->IdOf("groups", "Community Health") + " ";
    sql += ") ";
  }
  // TODO -c BUG : jhaley - These all find non-current addresses, and I don't know if that's intentional or not.
  if (editCity->Text != "")
    sql += "and exists (select person_contact_info.city from person_contact_info "
           "where person_contact_info.person_id = people.id and "
           "person_contact_info.city like '%" + SafeSQLString(editCity->Text.c_str()) + "%') ";
  if (cboState->Text != "")
    sql += "and exists (select state_types.id from state_types, person_contact_info "
           "where state_types.id = " + lookupTables->IdOf("state_types", cboState->Text.c_str()) + " and "
           "person_contact_info.state_id = state_types.id and "
           "person_contact_info.person_id = people.id) ";
  if (editZip->Text != "")
    sql += "and exists (select person_contact_info.zip from person_contact_info "
           "where person_contact_info.person_id = people.id and "
           "person_contact_info.zip like '%" + StripAllExcept(editZip->Text.c_str(), uints) + "%') ";
  if (cboGender->Text != "")
    sql += "and people.gender_id = " + lookupTables->IdOf("gender_types", cboGender->Text.c_str()) + " ";

  // jhaley 20101101: Add enrollments - warning: duplicated code w/launchDiabClientReport.
  vector<map<string, string> > selected_vecmap = cbddEnrollments->Filter->GetAllData(true, true);
  if (selected_vecmap.size() > 0)
  {
    sql += "and exists (select ENROLL.id from enrollment ENROLL where ENROLL.person_id = people.id and ENROLL.current_id+1 >= 4 and (";

    // These must all be added as a single condition, only one of which must be true (OR logic)
    for(vector<map<string, string> >::iterator i_row = selected_vecmap.begin(); i_row != selected_vecmap.end(); i_row++)
    {
      int combo_ids = StringToInt((*i_row)["internal_id"]);
      string enrollment_type_id   = IntToString(combo_ids / 1000); // hack alert! - can I do this better? probably...
      string enrollment_status_id = IntToString(combo_ids % 1000);

      // If not the first item, add an "or".
      if (i_row != selected_vecmap.begin())
        sql += "or ";

      sql += "(ENROLL.enrollment_id = '" + enrollment_type_id + "' ";
      if (!IsBlankOrZero(enrollment_status_id))
        sql += "and ENROLL.enrollment_status_id = '" + enrollment_status_id + "' ";

      // close this predicate
      sql += ") ";
    }
    // Close the set of predicates and the 'exists' subquery
    sql += ")) ";
  }

  if (checkFitness->Checked)
    sql += "and exists (select fitness_people.person_id from fitness_people, fitness_memberships where "
           "people.id = fitness_people.person_id and fitness_memberships.id = "
           "fitness_people.fitness_membership_id and "
           "fitness_memberships.end_date is null and fitness_people.current_id >= 3 and "
           "fitness_memberships.current_id >= 3) ";
  if(!checkDeceased->Checked)
    sql += "and people.death_date is null ";

  // sql += " order by person_names.last_name, person_names.first_name";
  Filter->SetSQLStatement (sql);
  Filter->RefreshDataSet ();

  return true;
}
//---------------------------------------------------------------------------
bool TformSearchPerson::ScreenStart()
{
  // instead of formcreate, do startup stuff here or in the constructor
  // this gets called after stuff in the base classes's constructor
  lookupTables->RegisterComboBox("tribe_types", cboTribe);
  lookupTables->RegisterComboBox("state_types", cboState);
  lookupTables->RegisterComboBox("gender_types", cboGender);

  if (optionsPermissions["special.search.person_id"]["enabled"] == "true")
  {
   lblChartNumber->Caption = "ID Number";
  }
  if (optionsPermissions["table.person_names"]["view"] != "true")
  {
    editLastName->Enabled = false;
    editFirstName->Enabled = false;
  }
  if (optionsPermissions["table.affiliations"]["view"] != "true")
  {
    editCDIB->Enabled = false;
    cboTribe->Enabled = false;
  }
  if (optionsPermissions["table.charts"]["view"] != "true")
    editChartNumber->Enabled = false;
  if (optionsPermissions["table.conditions"]["view"] != "true")
    editCode->Enabled = false;
  if (optionsPermissions["table.person_contact_info"]["view"] != "true")
  {
    editCity->Enabled = false;
    cboState->Enabled = false;
    editZip->Enabled = false;
  }
  if((optionsPermissions["table.fitness_memberships"]["view"] != "true") ||
    (optionsPermissions["table.people.diab"]["view"] != "true"))
  {
    checkFitness->Enabled = false;
  }

  Filter->SetSecurity("people");
  Filter->AddColumnSpec("Last Name", "people", as_text, true, 100);
  Filter->AddColumnSpec("First Name", "people", as_text, true, 100);
  Filter->AddColumnSpec("Middle Name", "people", as_text, false, 100);
  Filter->AddColumnSpec("Suffix", "people", as_text, true, 50);
  Filter->AddColumnSpec("CDIB", "affiliations", as_text, true, 70, lowercaseString);
  Filter->AddColumnSpec("Tribe", "affiliations", as_text, true, 115);
  Filter->AddColumnSpec("Joined", "affiliations", as_date, true, 80);
  Filter->AddColumnSpec("Relinquished", "affiliations", as_date, true, 80);
  Filter->AddColumnSpec("Birthday", "people", as_date, true, 80);
  Filter->AddColumnSpec("SSN", "people", as_text, true, 90, prettySSNString);
  Filter->AddColumnSpec("Chart No.", "charts", as_text, false, 90);
  Filter->AddColumnSpec("Code", "conditions", as_text, false, 90);
  Filter->AddColumnSpec("Address", "person_contact_info", as_text, false, 140, addressString);
  Filter->AddColumnSpec("City", "person_contact_info", as_text, false, 80);
  Filter->AddColumnSpec("State", "person_contact_info", as_text, false, 80, uppercaseString);
  Filter->AddColumnSpec("Zip", "person_contact_info", as_text, false, 80, prettyZipString);
  Filter->AddColumnSpec("Phone Number", "person_contact_info", as_text, false, 120, prettyPhoneString);
  Filter->AddColumnSpec("Gender", "people", as_text, false, 80);
  Filter->AddColumnSpec("Client Type", "people", as_text, false, 80);
  Filter->AddColumnSpec("Current Id", "current_id", as_int, false, 80);
  Filter->AddColumnSpec("Internal Id", "internal_id", as_int, false, 80);
  Filter->FetchPreferences("searchperson.filter");
  Filter->SetIconStatus(true);
  Filter->SetSQLStatement("");
  Filter->RefreshDataSet();

  // jhaley 20101101: setup enrollments CBDD - warning: duplicated code w/launchDiabClientReport
  string enrollment_sql =
    "select enrollment_types.item || (case when enrollment_status_types.id = 0 then '' else (' - ' || enrollment_status_types.item) end) item, "
    "enrollment_types.id * 1000 + coalesce(enrollment_status_types.id, 0) internal_id "           // hack alert!
    "from enrollment_types "
    "inner join permissions P on P.s = enrollment_types.permission_name and P.k = 'VIEW' and P.v = 'TRUE' and P.user_id = '" + current_userid + "' "
    "left join enrollment_status_types on (enrollment_types.id = enrollment_status_types.enrollment_id or "
    " (enrollment_types.allow_blank = 1 and enrollment_status_types.id = 0)) and enrollment_status_types.valid = 1 "
    "where enrollment_types.id > 0 and enrollment_types.valid = 1 "
    "order by enrollment_types.permission_name, enrollment_types.item, enrollment_status_types.item ";
  cbddEnrollments->Filter->SetSecurity("");
  cbddEnrollments->Filter->AddColumnSpec("Enrollment Type", "", as_text, true, 210, rawString);
  cbddEnrollments->Filter->AddColumnSpec("Internal Id", "internal_id", as_int, false, 60);
  cbddEnrollments->Filter->SetSQLStatement(enrollment_sql);
  cbddEnrollments->Filter->FetchPreferences("searchperson.enrollment_filter");

  TformSearchGeneric::ScreenStart();
  return true;
}
//---------------------------------------------------------------------------
bool TformSearchPerson::ScreenStop()
{
  // instead of formclose, do shutdown stuff here (combo boxes, etc.)
  optionsPreferences["formSearchPerson"]["Filter"] = Filter->GetPreferences();
  // unregister combo boxes
  lookupTables->UnregisterComboBox("tribe_types", cboTribe);
  lookupTables->UnregisterComboBox("state_types", cboState);
  lookupTables->UnregisterComboBox("gender_types", cboGender);
  TformSearchGeneric::ScreenStop();
  return true;
}
//---------------------------------------------------------------------------
bool TformSearchPerson::Create()
{
  TformWizardNewPerson::Do();
  return true;
}
//---------------------------------------------------------------------------
bool TformSearchPerson::Zoom(const string & p_id)
{
  map<string,string> temp_map;
  temp_map["id"] = p_id;
  FORM_TRY
    TformRecordPerson * personRecord = new TformRecordPerson(NULL, temp_map);
    personRecord->Show();
  FORM_CATCH

  return true;
}
//---------------------------------------------------------------------------
vector<string> TformSearchPerson::Do(TformSpecialGeneric * p_parent)
{
  vector<string> answer;
  FORM_TRY
    TformSearchPerson * z = new TformSearchPerson(NULL, &answer);
    z->ShowTaskModal(p_parent);
  FORM_CATCH
  return answer;
}
//---------------------------------------------------------------------------
// jhaley 20101101: OnDown event handler for enrollment CBDD, to avoid exceptions if done elsewhere.
void __fastcall TformSearchPerson::cbddEnrollmentsDown(TObject *Sender)
{
  cbddEnrollments->Filter->ForceRefreshDataSet();
  cbddEnrollments->DisplayFieldName = "Enrollment Type";
}
//---------------------------------------------------------------------------

