//---------------------------------------------------------------------------

#include <vcl.h>
#pragma hdrstop

#include "codeSearchTask.h"
#include "codeFrameSearchTaskBeh.h"
#include "codeFrameSearchTaskCOBVisit.h"
//---------------------------------------------------------------------------
#pragma package(smart_init)
#pragma link "codeRubberContainer"
#pragma link "codeSearchGeneric"
#pragma resource "*.dfm"
TformSearchTask *formSearchTask;
//---------------------------------------------------------------------------
__fastcall TformSearchTask::TformSearchTask(TComponent* Owner, vector<string> * p_return_vector, string p_group_id, bool p_is_visit)
        : TformSearchGeneric(Owner, p_return_vector, "tasks")
{
  group_name = "";
  group_id = p_group_id;
  needs_distinct = false;

  is_visit = p_is_visit; // Some groups distinguish between 'task' tasks and 'visit' tasks

  // create the appropriate frame for the viewing group
  if (group_id == lookupTables->IdOf("groups", "Behavioral Health"))
  {
    TframeSearchTaskBeh *beh_frame = NULL;
    FORM_TRY
      beh_frame = new TframeSearchTaskBeh(this);
    FORM_CATCH
    search_frame = beh_frame;
    group_name = "Behavioral Health";
  }
  else if (group_id == lookupTables->IdOf("groups", "Contract Health"))
  {
    if (is_visit)
    {
      TframeSearchTaskCOBVisit *cob_visit_frame = NULL;
      FORM_TRY
        cob_visit_frame = new TframeSearchTaskCOBVisit(this);
      FORM_CATCH
      search_frame = cob_visit_frame;
      group_name = "COB";
      needs_distinct = true; // COB Visit uses "select distinct" in its SQL query
    }
    else
    {
      // TODO
    }
  }
  else
  {
    // Woops! This should definitely not happen.
    PSay("No valid group for which to view tasks. Please notify a programmer immediately.", m_program_error, MB_OK);
  }

  // attach the frame to the panelFrameHolder
  if (search_frame)
  {
    search_frame->Parent = panelFrameHolder;
    search_frame->Align  = alClient;

    // resize stuff
    scrollStandard->Height = lblBeginDate->Height + 16 + search_frame->Height + 4;
  }

  Initialize();
}
//---------------------------------------------------------------------------
__fastcall TformSearchTask::~TformSearchTask()
{
  if (search_frame)
  {
    delete search_frame; // NB: valid because all TObjects have virtual destructors.
    search_frame = NULL;
  }
}
//---------------------------------------------------------------------------
bool TformSearchTask::IsValidOnScreen()
{
  // validate shared date controls
  if (dateBegin->Date > dateEnd->Date)
  {
    PSay("End date must be greater than or equal to begin date.", m_invalid_data, MB_OK);
    dateEnd->SetFocus();
    return false;
  }

  // validate all registered lookups
  for (vector<structTaskLookUp>::iterator i_lookup = lookups.begin(); i_lookup != lookups.end(); i_lookup++)
  {
    if (!i_lookup->need_valid) // doesn't require validation?
      continue;

    if (!lookupTables->IsInLookUpTable(i_lookup->table_name, i_lookup->combo_box)
        || (i_lookup->disallow_blank && i_lookup->combo_box->Text == "")) // disallows blank specially?
    {
      PSay("Please enter a valid " + i_lookup->pretty_name, m_invalid_data, MB_OK);
      i_lookup->combo_box->SetFocus();
      return false;
    }
  }

  // validate all registered edit boxes
  for (vector<structTaskEditBox>::iterator i_box = edit_boxes.begin(); i_box != edit_boxes.end(); i_box++)
  {
    bool is_valid = true;

    switch(i_box->type)
    {
    case VALIDATE_INT:
      if (!IsInt(i_box->edit_box->Text.c_str()))
        is_valid = false;
      break;

    case VALIDATE_FLOAT:
      if (!IsFloat(i_box->edit_box->Text.c_str()))
        is_valid = false;
      break;

    case VALIDATE_DATE:
      if (i_box->edit_box->Text != "" && !Pdate(i_box->edit_box->Text.c_str(), mdy | yy).IsValidDate())
        is_valid = false;
      break;

    default:
      is_valid = false; // ???? - crazy validation type
      break;
    }
    if (!is_valid)
    {
      PSay("Please enter a valid " + i_box->pretty_name, m_invalid_data, MB_OK);
      i_box->edit_box->SetFocus();
      return false;
    }
  }

  return true;
}
//---------------------------------------------------------------------------
// Registers a query item, which are used to build the "where" clause of the task search query. Query type should be
// one of QUERY_WILDSTRING, LOOKUP, RAW, CHECK, or CONSTANT. The control should be of a sane type for the query type
// (combo, edit, or checkbox - this is checked below using VCL's TObject RTTI facility). Lookup and comparison operator
// are optional parameters which apply only to LOOKUP and RAW/CONSTANT items respectively.
bool TformSearchTask::AddQueryItem(string field_name, int query_type, TControl *control, const char *lookup_table,
                                   const char *comparison_op)
{
  structTaskQueryItem query_item;

  query_item.field   = field_name;
  query_item.type    = query_type;
  query_item.control = control;
  if (lookup_table)
    query_item.lookup = lookup_table;
  else
    query_item.lookup = "";
  if (comparison_op)
    query_item.comp_op = comparison_op;
  else
    query_item.comp_op = "=";

  query_items.push_back(query_item);

  return true;
}
//---------------------------------------------------------------------------
// Internal fields - these get added when you use methods such as AddStaff or AddMinMaxTime. They remove the
// requirement to select and do joins for the fields that are added through these methods. This is a private method.
void TformSearchTask::AddInternalField(string field_name, string joins)
{
  structTaskField field_item;

  field_item.sql_field = field_name;
  field_item.sql_joins = joins;

  internal_fields.push_back(field_item);
}
//---------------------------------------------------------------------------
// DoQueryItem is called for each query item added by the search frame. Provides a host of different matching types for
// building the task SQL query. Each call will add a single "and" condition to the where clause appropriate for the
// attached control.
void TformSearchTask::DoQueryItem(TformSearchTask::structTaskQueryItem *query_item, string &where_string)
{
  switch(query_item->type)
  {
  case QUERY_WILDSTRING:
    // Wildcard string matching - requires an edit box control
    // Example: if(editComments->Text != "") sql += "and upper(T.task_comments) like '%" + SafeSQLString(editComments->Text.c_str()) + "%' ";
    if (query_item->control->ClassType() == __classid(TEdit))
    {
      TEdit *edit_box = (TEdit *)(query_item->control);
      if (edit_box->Text != "")
        where_string += "and upper(" + query_item->field + ") like '%" + SafeSQLString(edit_box->Text.c_str()) + "%' ";
    }
    break;

  case QUERY_LOOKUP:
    // Lookup table matching - requires a combobox control. Custom SQL is supported.
    // Example: if(cboEventName->Text != "") sql += "and T.event_name_id = '" + lookupTables->IdOf("event_name_types", cboEventName) + "' ";
    if (query_item->control->ClassType() == __classid(TComboBox))
    {
      TComboBox *combo_box = (TComboBox *)(query_item->control);
      if (combo_box->Text != "")
        where_string += "and " + query_item->field + " = '" + lookupTables->IdOf(query_item->lookup, combo_box) + "' ";
    }
    break;

  case QUERY_RAW:
    // Raw string matching - requires an editbox control
    // Example: if(editMinTime->Text != "") sql += "and T.time_of_task >= '" + editMinTime->Text.c_str() + "' ";
    if (query_item->control->ClassType() == __classid(TEdit))
    {
      TEdit *edit_box = (TEdit *)(query_item->control);
      if (edit_box->Text != "")
        where_string += "and " + query_item->field + " " + query_item->comp_op + " '" + edit_box->Text.c_str() + "' ";
    }
    break;

  case QUERY_CHECK:
    // Checkbox matching - requires a checkbox control
    // Example: if(checkNewClient->Checked) sql += "and T.new_client = 1 ";
    if (query_item->control->ClassType() == __classid(TCheckBox))
    {
      TCheckBox *check_box = (TCheckBox *)(query_item->control);
      if (check_box->Checked)
        where_string += "and " + query_item->field + " = 1 ";
    }
    break;

  case QUERY_CONSTANT:
    // Check against a constant factor. No control is required, and the check is always done.
    // Example: ... and T.parent_task_id = 0 ...
    where_string += "and " + query_item->field + " " + query_item->comp_op + " ";
    break;

  default: // ???? - crazy item type
    break;
  }
}
//---------------------------------------------------------------------------
bool TformSearchTask::Search()
{
  string sql, select, from, where;

  if (!search_frame) // must have a valid frame
    return false;

  select = string("select ") + (needs_distinct ? "distinct " : "") + "T.date_of_task, "; // everybody has the date as the first item.
  from   = "from tasks T ";

  // add internally selected fields
  for (vector<structTaskField>::iterator i_field = internal_fields.begin(); i_field != internal_fields.end(); i_field++)
  {
    select += i_field->sql_field + ", "; // add the field to select
    from   += i_field->sql_joins + " ";  // add the joins it needs too
  }

  select += search_frame->GetSelectSQL();       // add custom fields for this group to the select
  select += "T.group_id, T.current_id, T.id ";  // add IDs, which everyone has, last.

  from += search_frame->GetFromSQL(); // add custom joins for this group's fields

  where = "where T.id+1 > 1 "
          "and T.date_of_task >= '" + Pdate(dateBegin->Date).OutDate(mdy|yyyy) + "' "
          "and T.date_of_task <= '" + Pdate(dateEnd->Date).OutDate(mdy|yyyy) + "' "
          "and " + SafeInClause("T.group_id", search_frame->GetGroups()) + " "; // add group restrictions

  // add custom query item restrictions
  for (vector<structTaskQueryItem>::iterator i_item = query_items.begin(); i_item != query_items.end(); i_item++)
    DoQueryItem(i_item, where);

  // add it all together and set it to the filter
  sql = select + from + where;
  Filter->SetSQLStatement(sql);
  Filter->RefreshDataSet();
  return true;
}
//---------------------------------------------------------------------------
// Add a custom lookup table to the list. Called by frames' RegisterControls method to add lookups. If custom SQL is
// not required, just pass the empty string. need_valid and disallow_blank are optional parameters to control, first,
// validation - if need_valid is false, the box doesn't have to contain a valid value in IsValidOnScreen. Second, some
// boxes specifically disallow "" even if it would pass the lookupTables->IsInLookUpTable() check.
bool TformSearchTask::AddLookUp(string table, string pretty_name, string custom_sql, TComboBox *box, bool need_valid,
                                bool disallow_blank)
{
  structTaskLookUp lookup_item;

  lookup_item.table_name     = table;
  lookup_item.pretty_name    = pretty_name;
  lookup_item.custom_sql     = custom_sql;
  lookup_item.combo_box      = box;
  lookup_item.need_valid     = need_valid;
  lookup_item.disallow_blank = disallow_blank;

  lookups.push_back(lookup_item);

  return true;
}
//---------------------------------------------------------------------------
// Add an edit box to the list. Called by frames' RegisterControls method to add edit boxes. Only edits which need
// some type of input validation must be registered in this manner.
bool TformSearchTask::AddEditBox(string pretty_name, TEdit *edit_box, int validation_type)
{
  structTaskEditBox editbox_item;

  editbox_item.pretty_name = pretty_name;
  editbox_item.edit_box    = edit_box;
  editbox_item.type        = validation_type;

  edit_boxes.push_back(editbox_item);

  return true;
}
//---------------------------------------------------------------------------
bool TformSearchTask::ScreenStart()
{
  // Handle shared date controls
  Pdate begin_date(DateToStr(Date()).c_str(), mdy | yyyy);
  begin_date.mod(-14, in_days);
  dateBegin->Date = StrToDate(begin_date.OutDate(mdy | yyyy).c_str());
  dateEnd->Date = Date();

  if (!search_frame) // must have a valid frame
    return false;

  // setup the search filter's initial shared properties
  search_frame->SetFilterSecurity(Filter);                   // set filter security (dependent on group)
  Filter->AddColumnSpec("Date", "tasks", as_date, true, 80); // everybody has dates so they're added first

  search_frame->RegisterControls(this); // get additional controls from frame

  // register the frame lookups
  for (vector<structTaskLookUp>::iterator i_lookup = lookups.begin(); i_lookup != lookups.end(); i_lookup++)
    lookupTables->RegisterComboBox(i_lookup->table_name, i_lookup->combo_box, i_lookup->custom_sql);

  // setup frame-specific custom columnspecs
  search_frame->AddFilterColumnSpecs(Filter);

  // do the rest of filter setup:
  Filter->AddColumnSpec("Group Id",    "internal_id", as_int, false, 80); // everybody has IDs as well, and they are added last
  Filter->AddColumnSpec("Current Id",  "current_id",  as_int, false, 80);
  Filter->AddColumnSpec("Internal Id", "internal_id", as_int, false, 80);
  search_frame->SetFilterPreferences(Filter);                             // get filter preferences (dependent on group)
  Filter->SetSQLStatement("");
  Filter->RefreshDataSet();

  return true;
}
//---------------------------------------------------------------------------
bool TformSearchTask::ScreenStop()
{
  // unregister frame lookups
  for (vector<structTaskLookUp>::iterator i_lookup = lookups.begin(); i_lookup != lookups.end(); i_lookup++)
    lookupTables->UnregisterComboBox(i_lookup->table_name, i_lookup->combo_box);

  return true;
}
//---------------------------------------------------------------------------
bool TformSearchTask::Create()
{
  map<string, string> temp_map;
  temp_map["id"] = "new";
  temp_map["group_id"] = group_id;

  // special cases:
  // The following groups are not allowed to create a task from their search form:
  // * COB / COB Visit
  // * LAC
  // * SC
  if (group_id == lookupTables->IdOf("groups", "Contract Health"))
  {
    PSay("Sorry, you cannot create a task or visit from here.", m_reminder, MB_OK);
    return false;
  }
  else if(group_id == lookupTables->IdOf("groups", "Little Axe Clinic") ||
          group_id == lookupTables->IdOf("groups", "Shawnee Clinic"))
  {
    PSay("Please create visits by opening the appointment form and clicking\non the Task/Visit link.", m_reminder, MB_OK);
    return false;
  }

  // TODO: visits vs tasks for Public Health, Community Health
  // TODO: For TASKPUB, use TformRecordComVisit::Do to allow choice of a com_dept_id before starting up the form.

  // OTHERWISE...
  DefaultAction("tasks", temp_map);
  return true;
}
//---------------------------------------------------------------------------
bool TformSearchTask::Zoom(const string & p_id)
{
  DefaultAction(Filter->GetSelectedRecordID(), "tasks");
  return true;
}
//---------------------------------------------------------------------------
vector<string> TformSearchTask::Do(TformSpecialGeneric * p_parent, string p_group_id, bool p_is_visit)
{
  vector<string> answer;
  FORM_TRY
    TformSearchTask *task_search = new TformSearchTask(NULL, &answer, p_group_id, p_is_visit);
    task_search->ShowTaskModal(p_parent);
  FORM_CATCH
  return answer;
}
//---------------------------------------------------------------------------
// Overridden to set a group-appropriate caption, so we know what kind of tasks we're looking at.
void TformSearchTask::SqlModified()
{
  TformSearchGeneric::SqlModified();
  string current_caption = Caption.c_str();

  if (is_visit)
    current_caption = ReplaceStringWithString(current_caption, "Task", "Visit");

  Caption = (group_name + " " + current_caption).c_str();
}
//---------------------------------------------------------------------------
//
// Code for shared items starts here. A major weakness of the old independent task search forms was an inability to
// effectively share this code, requiring it to be adjusted on every form if something such as custom SQL had to change.
// Sharing all of this logic here reduces the code on the frames to less than 100 lines a piece, even for the relatively
// complicated COB Visit.
//
//---------------------------------------------------------------------------
// Add a Staff item. It will bind to the given combobox for searching, and will appear in the filter.
void TformSearchTask::AddStaff(TComboBox *box)
{
  AddLookUp("users", "staff name", "", box);                                     // lookup for combo box
  Filter->AddColumnSpec("Staff", "tasks", as_text, true, 110);                   // column for filter
  AddQueryItem("T.user_id", QUERY_LOOKUP, box, "users");                         // query building item
  AddInternalField("users.item", "left join users on tasks.user_id = users.id"); // select & where clause additions
}
//---------------------------------------------------------------------------
// Add an Event Name item.
void TformSearchTask::AddEventName(TComboBox *box)
{
  string event_names_sql = "select id, item, valid from event_name_types "
                           "where (group_id = 0 or " + SafeInClause("group_id", search_frame->GetGroups()) + ") "
                           "and valid = 1 order by item";

  AddLookUp("event_name_types", "event name", event_names_sql, box);           // lookup for combo box
  Filter->AddColumnSpec("Event", "events", as_text, true, 110, rawString);     // column for filter
  AddQueryItem("events.event_name_id", QUERY_LOOKUP, box, "event_name_types"); // query building item
  AddInternalField("event_name_types.item",                                    // select & where clause additions
                   "left join events on T.event_id = events.id "
                   "left join event_name_types on events.event_name_id = event_name_types.id ");
}
//---------------------------------------------------------------------------
// Add Min and Max Time items.
void TformSearchTask::AddMinMaxTimes(TEdit *min_box, TEdit *max_box)
{
  AddEditBox("time spent", min_box, VALIDATE_FLOAT); // min editbox
  AddEditBox("time spent", max_box, VALIDATE_FLOAT); // max editbox

  Filter->AddColumnSpec("Time Spent", "tasks", as_double, true, 100); // filter item
  AddQueryItem("T.time_spent", QUERY_RAW, min_box, "", ">=");         // minimum time for query
  AddQueryItem("T.time_spent", QUERY_RAW, max_box, "", "<=");         // maximum time for query
  AddInternalField("T.time_spent", "");                               // select & where clause additions
}
//---------------------------------------------------------------------------
// Add Min and Max Mileage items.
void TformSearchTask::AddMinMaxMileage(TEdit *min_box, TEdit *max_box)
{
  AddEditBox("mileage", min_box, VALIDATE_FLOAT); // min editbox
  AddEditBox("mileage", max_box, VALIDATE_FLOAT); // max editbox

  Filter->AddColumnSpec("Mileage", "tasks", as_double, false, 80); // filter item
  AddQueryItem("T.mileage", QUERY_RAW, min_box, "", ">=");         // minimum time for query
  AddQueryItem("T.mileage", QUERY_RAW, max_box, "", "<=");         // maximum time for query
  AddInternalField("T.mileage", "");                               // select & where clause additions
}
//---------------------------------------------------------------------------
// Add a Person. No control is needed for this because searching for tasks by person isn't current supported (this could
// certainly be added in the future, of course). It does, however, take a string for terminology, as different groups
// refer to this field by different display names (ex: 'Person', 'Client', etc.).
void TformSearchTask::AddPerson(string field_term)
{
  // This just appears on the filter.
  Filter->AddColumnSpec(field_term, "people", as_text, true, 110);
  AddInternalField("person_names.first_name || ' ' || person_names.last_name || ' ' || person_names.suffix name",
                   "left join people on T.person_id = people.id "
                   "left join person_names on people.person_names_id = person_names.id");
}
//---------------------------------------------------------------------------
// Add Method of Contact.
void TformSearchTask::AddMethodOfContact(TComboBox *box)
{
  AddLookUp("method_types", "method of contact", "", box);
  Filter->AddColumnSpec("Method of Contact", "tasks", as_text, false, 130);
  AddQueryItem("T.method_id", QUERY_LOOKUP, box, "method_types");
  AddInternalField("method_types.item", "left join method_types on T.method_id = method_types.id");
}
//---------------------------------------------------------------------------
// Add Direction (In/Out)
void TformSearchTask::AddDirection(TComboBox *box)
{
  AddLookUp("direction_types", "incoming or outgoing", "", box);
  Filter->AddColumnSpec("In/Out", "tasks", as_text, true, 80);
  AddQueryItem("T.direction_id", QUERY_LOOKUP, box, "direction_types");
  AddInternalField("direction_types.item", "left join direction_types on T.direction_id = direction_types.id");
}
//---------------------------------------------------------------------------
// Add Classfication - combo box is optional, and if not provided, it will be display-only.
void TformSearchTask::AddClassification(TComboBox *box)
{
  if (box)
  {
    AddLookUp("classification_types", "classification", "", box);
    AddQueryItem("T.classification_id", QUERY_LOOKUP, box, "classification_types");
  }
  Filter->AddColumnSpec("Classification", "tasks", as_text, false, 110);
  AddInternalField("classification_types.item", "left join classification_types on T.classification_id = classification_types.id");
}
//---------------------------------------------------------------------------
// Add Task Purpose.
void TformSearchTask::AddTaskPurpose(TComboBox *box)
{
  string task_purpose_sql = "select id, item, valid from task_purpose_types "
                            "where (group_id = 0 or " + SafeInClause("group_id", search_frame->GetGroups()) + ") "
                            "and valid = 1 order by item ";

  AddLookUp("task_purpose_types", "task type", task_purpose_sql, box);
  Filter->AddColumnSpec("Task Type", "tasks", as_text, false, 110, rawString);
  AddQueryItem("T.task_purpose_id", QUERY_LOOKUP, box, "task_purpose_types");
  AddInternalField("task_purpose_types.item", "left join task_purpose_types on T.task_purpose_id = task_purpose_types.id");
}
//---------------------------------------------------------------------------
// Add Program Type.
void TformSearchTask::AddProgramType(TComboBox *box)
{
  string program_sql = "select id, item, valid from program_types "
                       "where (group_id = 0 or " + SafeInClause("group_id", search_frame->GetGroups()) + ") "
                       "and valid = 1 order by item ";

  AddLookUp("program_types", "program type", program_sql, box);
  Filter->AddColumnSpec("Program Type", "tasks", as_text, false, 110, rawString);
  AddQueryItem("T.program_id", QUERY_LOOKUP, box, "program_types");
  AddInternalField("program_types.item", "left join program_types on T.program_id = program_types.id");
}
//---------------------------------------------------------------------------
// Add Visit Location Types.
void TformSearchTask::AddVisitLocation(TComboBox *box)
{
  AddLookUp("visit_location_types", "visit location", "", box);
  Filter->AddColumnSpec("Location", "tasks", as_text, true, 110);
  AddQueryItem("T.visit_location_id", QUERY_LOOKUP, box, "visit_location_types");
  AddInternalField("visit_location_types.item",
                   "left join visit_location_types on T.visit_location_id = visit_location_types.id");
}
//---------------------------------------------------------------------------
// Add a New Client/Patient Checkbox
void TformSearchTask::AddNewClient(TCheckBox *box, string field_term)
{
  Filter->AddColumnSpec(field_term, "tasks", as_text, false, 100, prettyBoolString);
  AddQueryItem("T.new_client", QUERY_CHECK, box);
  AddInternalField("T.new_client", "");
}
//---------------------------------------------------------------------------
// Add Tax Entity - for COB Tasks & Visits
void TformSearchTask::AddTaxEntity(TEdit *box)
{
  Filter->AddColumnSpec("Tax Entity", "tax_entities", as_text, true, 140, rawString);
  AddQueryItem("tax_entities.name", QUERY_WILDSTRING, box);
  AddInternalField("tax_entities.name", "left join tax_entities on T.tax_entity_id = tax_entities.id");
}
//---------------------------------------------------------------------------
// Add Task Comments
void TformSearchTask::AddTaskComments(TEdit *box, string field_term)
{
  Filter->AddColumnSpec(field_term, "tasks", as_text, true, 130, sentenceString);
  AddQueryItem("T.task_comments", QUERY_WILDSTRING, box);
  AddInternalField("T.task_comments", "");
}
//---------------------------------------------------------------------------

