//---------------------------------------------------------------------------

#ifndef codeSearchTaskH
#define codeSearchTaskH
//---------------------------------------------------------------------------
#include <Classes.hpp>
#include <Controls.hpp>
#include <StdCtrls.hpp>
#include <Forms.hpp>
#include "codeRubberContainer.h"
#include "codeSearchGeneric.h"
#include <ComCtrls.hpp>
#include <ExtCtrls.hpp>
#include <ToolWin.hpp>
#include "codeFrameSearchTaskGeneric.h"
//---------------------------------------------------------------------------
class TformSearchTask : public TformSearchGeneric
{
__published:	// IDE-managed Components
        TPanel *panelFrameHolder;
        TLabel *lblBeginDate;
        TLabel *lblEndDate;
        TDateTimePicker *dateBegin;
        TDateTimePicker *dateEnd;
private:	// User declarations
        bool IsValidOnScreen();
        bool Search();
        bool ScreenStart();
        bool ScreenStop();
        bool Create();
        bool Zoom(const string & p_id);
        void SqlModified();

        // Lookups - these allow the task form to uniformly manage all lookups needed by the group-specific controls.
        // These are added by calling AddLookUp, or through one of the various built-in field adding methods.
        struct structTaskLookUp
        {
          string table_name;    // name of table
          string pretty_name;   // "pretty" name for error messages
          string custom_sql;    // custom SQL; "" if none.
          TComboBox *combo_box; // combo box control
          bool need_valid;      // true == validate in IsValidOnScreen
          bool disallow_blank;  // true == disallows blank value
        };
        vector<structTaskLookUp> lookups;

        // EditBoxes - these allow the task form to uniformly manage all editboxes needed by the group-specific controls.
        // These are added by calling AddEditBox, or through one of the various built-in field adding methods.
        struct structTaskEditBox
        {
          string pretty_name;   // "pretty" name for error messages
          TEdit *edit_box;      // edit box control
          int    type;          // type of validation to do on box
        };
        vector<structTaskEditBox> edit_boxes;

        // Task fields - these are internal and are only added when calling one of the built-in methods. These keep
        // track of fields to add to the select clause of the query, and the joins (if any) that are needed to put that
        // field into the query. If joins are not needed, they are set to "".
        struct structTaskField
        {
          string sql_field; // field to add to the select clause of the query
          string sql_joins; // join(s) required to display this field
        };
        vector<structTaskField> internal_fields;
        void AddInternalField(string field_name, string joins);

        // Query items - These are used to build the where clause of the SQL query, and represent different kinds of
        // restrictions on the type of tasks that will appear when searching.
        struct structTaskQueryItem
        {
          string field;      // field name
          string lookup;     // lookup table, only if type == QUERY_LOOKUP
          TControl *control; // relevant control
          int type;          // type of field
          string comp_op;    // comparison operator, if type == QUERY_RAW; include constant too if QUERY_CONSTANT
        };
        vector<structTaskQueryItem> query_items;
        void DoQueryItem(structTaskQueryItem *query_item, string &where_string);

        TframeSearchTaskGeneric *search_frame;
        string group_id;
        string group_name;
        bool   needs_distinct;
        bool   is_visit;

public:		// User declarations
        __fastcall TformSearchTask(TComponent* Owner, vector<string> * p_return_vector, string p_group_id, bool p_is_visit);
        __fastcall ~TformSearchTask();
        static vector<string> Do(TformSpecialGeneric * p_parent, string p_group_id, bool p_is_visit);
        bool AddLookUp(string table, string pretty_name, string custom_sql, TComboBox *box, bool need_valid = true, bool disallow_blank = false);
        bool AddEditBox(string pretty_name, TEdit *edit_box, int validation_type);
        bool AddQueryItem(string field_name, int query_type, TControl *control, const char *lookup_table = NULL, const char *comparison_op = NULL);

        // Common field adding methods - these are fields which are used on most of the frames (formerly, separate
        // task search forms). They were never able to share logic until now.
        void AddStaff(TComboBox *box);
        void AddEventName(TComboBox *box);
        void AddMinMaxTimes(TEdit *min_box, TEdit *max_box);
        void AddMinMaxMileage(TEdit *min_box, TEdit *max_box);
        void AddPerson(string field_term);
        void AddMethodOfContact(TComboBox *box);
        void AddDirection(TComboBox *box);
        void AddClassification(TComboBox *box = NULL);
        void AddTaskPurpose(TComboBox *box);
        void AddProgramType(TComboBox *box);
        void AddVisitLocation(TComboBox *box);
        void AddNewClient(TCheckBox *box, string field_term);
        void AddTaxEntity(TEdit *box);
        void AddTaskComments(TEdit *box, string field_term);

        // enumeration for task editbox validation types
        enum
        {
          VALIDATE_INT,
          VALIDATE_FLOAT,
          VALIDATE_DATE
        };

        // enumeration for query item types
        enum
        {
          QUERY_WILDSTRING, // a string using '%...%' matching
          QUERY_LOOKUP,     // a lookup table
          QUERY_RAW,        // raw comparison, for ints, floats, etc.
          QUERY_CHECK,      // checkbox
          QUERY_CONSTANT    // check against a constant value (no control required)
        };
};
//---------------------------------------------------------------------------
extern PACKAGE TformSearchTask *formSearchTask;
//---------------------------------------------------------------------------
#endif
