//---------------------------------------------------------------------------

#include <vcl.h>
#pragma hdrstop

#include "codeSearchTaskLac.h"
#include "codeRecordTaskLac.h"
#include "codeRecordTaxEntity.h"
#include "codeSearchTaxEntity.h"
#include "codeRecordTaskDen.h"
//---------------------------------------------------------------------------
#pragma package(smart_init)
#pragma link "codeSearchGeneric"
#pragma link "codeRubberContainer"
#pragma resource "*.dfm"
TformSearchTaskLac *formSearchTaskLac;
//---------------------------------------------------------------------------
__fastcall TformSearchTaskLac::TformSearchTaskLac(TComponent* Owner, vector<string> * p_return_vector)
        : TformSearchGeneric(Owner, p_return_vector, string ("tasks"))
{
  provider_id = "";
  clinic_id = "";

  Initialize();
}
//---------------------------------------------------------------------------
bool TformSearchTaskLac::IsValidOnScreen()
{
  // check to make sure that the values entered in the combo boxes
  // corrospond to values in that look up table.
  // give the user a message box and prompt to re-enter if incorrect
  if (! IsFloat(editMinTime->Text.c_str()))
  {
    PSay("Please enter a valid time spent", m_invalid_data, MB_OK);
    editMinTime->SetFocus();
    return false;
  }
  else if (! IsFloat(editMaxTime->Text.c_str()))
  {
    PSay("Please enter a valid time spent", m_invalid_data, MB_OK);
    editMaxTime->SetFocus();
    return false;
  }
  else if (cboClinic->Text == "" || !lookupTables->IsInLookUpTable("groups", cboClinic))
  {
     PSay("Please select a clinic.", m_message, MB_OK);
     cboClinic->SetFocus();
     return false;
  }
  else if (!HasLocationPermissions(GetSpecialGroupPermissions("general"), cboClinic->Text.c_str()))
  {
     PSay("You do not have permission to run this report for " + string(cboClinic->Text.c_str()) + ".", m_message, MB_OK);
     cboClinic->SetFocus();
     return false;
  }
  else if (!lookupTables->IsInLookUpTable("task_purpose_types", cboTaskType))
  {
    PSay("Please enter a valid task type", m_invalid_data, MB_OK);
    cboTaskType->SetFocus();
    return false;
  }

  return true;
}
//---------------------------------------------------------------------------
bool TformSearchTaskLac::Search()
{
  clinic_id = lookupTables->IdOf("groups", cboClinic);
  string sql;
  string from;
  string where;
  sql = "select distinct tasks.date_of_task, tasks.time_of_task, "
        "person_names.first_name || ' ' || person_names.last_name || ' ' || person_names.suffix name, "
        "tax_entities.name, tasks.time_spent, "
        "tasks.new_client, tasks.task_comments, "
        "tasks.current_id, tasks.id ";
  from = "from tasks left join tax_entities on tasks.tax_entity_id = tax_entities.id ";

  if(editDiagnosis->Text != "")
  {
    from += "inner join conditions on conditions.task_id = tasks.id and "
            "code like '%" + SafeSQLString(editDiagnosis->Text.c_str()) + "%' and "
            "conditions.condition_id + 1 != " + lookupTables->IdOf("condition_types", "Relative") + " + 1 and "
            "conditions.current_id + 1 > 3 ";
    if (!IsBlankOrZero(clinic_id))
     from += "and conditions.group_id = " + clinic_id + " ";
  }

  if(editProcedure->Text != "")
  {
    from += "inner join procedures on procedures.task_id = tasks.id and "
            "cpt_hcpcs like '%" + SafeSQLString(editProcedure->Text.c_str()) + "%' and "
            "procedures.current_id + 1 > 3 ";
  }

  if(cboEventName->Text != "")
  {
    from += "left join events on tasks.event_id = events.id left join event_name_types on events.event_name_id = event_name_types.id ";
  }

  where = " where tasks.id + 1 > 1 and "
          "tasks.date_of_task >= '" + Pdate(dateBegin->Date).OutDate(mdy | yyyy) + "' and "
          "tasks.date_of_task <= '" + Pdate(dateEnd->Date).OutDate(mdy | yyyy) + "' and ";
  if (!IsBlankOrZero(clinic_id))
  where += "tasks.group_id = " + clinic_id + " and ";

  if(editProvider->Text != "")
    where += "upper(tax_entities.name) like '%" + SafeSQLString(editProvider->Text.c_str()) + "%' and ";
  if(editMinTime->Text != "")
    where += "tasks.time_spent >= " + string(editMinTime->Text.c_str()) + " and ";
  if(editMaxTime->Text != "")
    where += "tasks.time_spent <= " + string(editMaxTime->Text.c_str()) + " and ";
  if(checkNewPatient->Checked)
    where += "tasks.new_client = 1 and ";
  if(cboTaskType->Text != "")
    where += "tasks.task_purpose_id = '" + lookupTables->IdOf("task_purpose_types", cboTaskType->Text.c_str(), cboTaskType) + "' and ";
  if(cboEventName->Text != "")
    where += "event_name_types.id = '" + lookupTables->IdOf("event_name_types", cboEventName) + "' and ";
  from += "left join people on tasks.person_id = people.id "
          "left join person_names on people.person_names_id = person_names.id ";
  where = where.substr(0, where.length()-5);
  sql = sql + " " + from + " " + where; // + " order by tasks.date_of_task, person_names.last_name, person_names.first_name";

  Filter->SetSQLStatement(sql);
  Filter->RefreshDataSet();
  return true;
}
//---------------------------------------------------------------------------
bool TformSearchTaskLac::ScreenStart()
{
 // instead of formcreate, do startup stuff here or in the constructor
 // this gets called after stuff in the base classes's constructor
  dateBegin->Date = Date();
  dateEnd->Date = Date();

  Filter->SetListViewHandle(listSearchResults);
  if(clinic_id == lookupTables->IdOf("groups", "Little Axe Clinic"))
    Filter->SetSecurity("tasks.lac");
  else if (clinic_id == lookupTables->IdOf("groups", "Shawnee Clinic"))
    Filter->SetSecurity("tasks.sc");
  Filter->AddColumnSpec("Date", "tasks", as_date, true, 80);
  Filter->AddColumnSpec("Time", "tasks", as_time, true, 80);
  Filter->AddColumnSpec("Patient", "people", as_text, true, 110);
  Filter->AddColumnSpec("Provider", "tax_entities", as_text, true, 110, rawString);
  Filter->AddColumnSpec("Time Spent", "tasks", as_double, false, 100);
  Filter->AddColumnSpec("New Patient", "tasks", as_text, true, 80, prettyBoolString);
  Filter->AddColumnSpec("Comments", "tasks", as_text, false, 150, sentenceString);
  Filter->AddColumnSpec("Current Id", "current_id", as_int, false, 80);
  Filter->AddColumnSpec("Internal Id", "internal_id", as_int, false, 80);
  Filter->FetchPreferences("searchtasklac.filter");
  Filter->SetSQLStatement("");
  Filter->RefreshDataSet();

  // include id(0) so there is a "blank" option
  string location_groups_sql = "select id, item, valid from groups where ((clinic_id > 0 and has_charts = 1) or id = 0) order by item"; // clinics have charts
  // TODO:  we really should remove the clinics they shouldn't have permission to...
  lookupTables->RegisterComboBox("groups", cboClinic, location_groups_sql);

  // we'll be nice and fill in the clinic name for them
  if (lookupTables->IsValidId("groups", current_groupid, cboClinic)) // is current user's group a clinic?  if so, set the clinic automatically...
  {
   cboClinic->Text = lookupTables->ValueOf("groups", current_groupid).c_str();
   this->ActiveControl = editProvider/*->SetFocus()*/;
  }

  string task_sql = "select id, item, valid from task_purpose_types where group_id = 0 or group_id in "
                    "(select groups.id from groups where ((clinic_id > 0 and has_charts = 1) or id = 0))";
  lookupTables->RegisterComboBox("task_purpose_types", cboTaskType, task_sql);

  // jhaley 20101026: I accidentally broke this by using clinic_id. But in fact we simply can't restrict this lookup to
  // a specific group because this form is being used by three groups, and the group can be changed dynamically on the
  // form. This is fixed in my new unified task search screen, though, which easily supports the clinics separately
  // without requiring a "Department" dropbox, or three duplicate forms. For now I'm restoring the code that was here
  // previously, which had the advantage of working.
  string event_sql = "select id, item, valid from event_name_types where "
                     + SafeInClause("group_id", SetToVec(GetSpecialGroupPermissions("general"))) + " and valid = 1 order by item";
                     // nope! - "(group_id = 0 or group_id = '" + clinic_id + "') and valid = 1 order by item";
  lookupTables->RegisterComboBox("event_name_types", cboEventName, event_sql);

  return true;
}
//---------------------------------------------------------------------------
bool TformSearchTaskLac::ScreenStop()
{
  lookupTables->UnregisterComboBox("groups", cboClinic);
  lookupTables->UnregisterComboBox("task_purpose_types", cboTaskType);
  lookupTables->UnregisterComboBox("event_name_types", cboEventName);

  return true;
}
//---------------------------------------------------------------------------
bool TformSearchTaskLac::Create()
{
  map<string,string> temp_map;
  temp_map["id"] = "new";
  if(clinic_id == lookupTables->IdOf("groups", "Little Axe Clinic") ||
     clinic_id == lookupTables->IdOf("groups", "Shawnee Clinic"))
  {
  /*
    FORM_TRY
      TformRecordTaskLac * taskRecord = new TformRecordTaskLac(NULL, temp_map);
      taskRecord->Show();
    FORM_CATCH
  */
    PSay("Please create visits by opening the appointment form and clicking\n"
         "on the Task/Visit link.", m_reminder, MB_OK);
  }
  else if(clinic_id == lookupTables->IdOf("groups", "Dental Clinic"))
  {
    FORM_TRY
      TformRecordTaskDen * taskRecord = new TformRecordTaskDen(NULL, temp_map);
      taskRecord->Show();
    FORM_CATCH
  }
 return true;
}
//---------------------------------------------------------------------------
bool TformSearchTaskLac::Zoom(const string & p_id)
{
  map<string, string> temp_map;
  temp_map["id"] = p_id;
  // okay, this didn't work for some reason
  // if(is_in(clinic_id, lookupTables->IdOf("groups", "Little Axe Clinic"), lookupTables->IdOf("groups", "Shawnee Clinic"), NULL))
  if(clinic_id == lookupTables->IdOf("groups", "Little Axe Clinic") ||
     clinic_id == lookupTables->IdOf("groups", "Shawnee Clinic"))
  {
    FORM_TRY
      TformRecordTaskLac * taskRecord = new TformRecordTaskLac(NULL, temp_map, clinic_id);
      taskRecord->Show();
    FORM_CATCH
  }
  else if(clinic_id == lookupTables->IdOf("groups", "Dental Clinic"))
  {
    FORM_TRY
      TformRecordTaskDen * taskRecord = new TformRecordTaskDen(NULL, temp_map);
      taskRecord->Show();
    FORM_CATCH
  }
  return true;
}
//---------------------------------------------------------------------------
vector<string> TformSearchTaskLac::Do(TformSpecialGeneric * p_parent)
{
 vector<string> answer;
 FORM_TRY
  TformSearchTaskLac * z = new TformSearchTaskLac(NULL, &answer);
  z->ShowTaskModal(p_parent);
 FORM_CATCH
 return answer;
}
//---------------------------------------------------------------------------

