//---------------------------------------------------------------------------

#include <vcl.h>
#pragma hdrstop

#include "codeSpecialAction.h"
#include "codePrometheus.h"
#include "db_hardcoded_metadata.h"
//---------------------------------------------------------------------------
#pragma package(smart_init)
#pragma link "codeSpecialGeneric"
#pragma resource "*.dfm"
TformSpecialAction *formSpecialAction;
TNotifyEvent TformSpecialAction::action_click;
//---------------------------------------------------------------------------
__fastcall TformSpecialAction::TformSpecialAction(TComponent* Owner, string source_table_name, a_factory_create chosen_action, const vector<string> & ids)
        : TformSpecialGeneric(Owner)
{
 id_vec = ids;

 FORM_TRY
  action_container = new TframeActionContainer(this);
  action_container->Parent = panelActions;
  action_container->Align  = alClient;
  action_container->SourceTableName = source_table_name.c_str();
  if (chosen_action)
   action_container->RequestAction(chosen_action);
 FORM_CATCH

 labelAbout->Caption = ("Action(s) to perform on " + IntToString(id_vec.size()) + " " + dbPrettyPlural(source_table_name) + ":").c_str();
}
//---------------------------------------------------------------------------
TformSpecialAction* __fastcall TformSpecialAction::Do(const string & source_table_name, const string & id)
{
 vector<string> temp;
 temp.push_back(id);
 return Do(source_table_name, temp);
}
//---------------------------------------------------------------------------
TformSpecialAction* __fastcall TformSpecialAction::Do(const string & source_table_name, const vector<string> & ids)
{
 return Do(source_table_name, NULL, ids);
}
//---------------------------------------------------------------------------
TformSpecialAction* __fastcall TformSpecialAction::Do(const string & source_table_name, a_factory_create chosen_action, const string & id)
{
 vector<string> temp;
 temp.push_back(id);
 return Do(source_table_name, chosen_action, temp);
}
//---------------------------------------------------------------------------
TformSpecialAction* __fastcall TformSpecialAction::Do(const string & source_table_name, a_factory_create chosen_action, const vector<string> & ids)
{
 TformSpecialAction* z = NULL;
 FORM_TRY
  z = new TformSpecialAction(NULL, source_table_name, chosen_action, ids);
  z->Show();
 FORM_CATCH
 return z;
}
//---------------------------------------------------------------------------
void __fastcall TformSpecialAction::buttonRunClick(TObject *Sender)
{
 bool should_close = false;
 // just enough to get the goddamn date controls to give up their values.
 FocusAway();
 FocusBack();
 // verify it's all valid,
 if (action_container->Valid())
 {
 // prep a transaction,
  TIBTransaction* dbTransaction = NULL;
  FORM_TRY
   dbTransaction = new TIBTransaction(NULL);
  FORM_CATCH
  StdTransaction(dbTransaction, dbDatabase);

  try
  {
   // TODO: alternative solution: for single-id, make the view be simply "select [id] from uno" rather than use temp_id

   string assigned_query_number = GetNextId(dbTransaction, "temp_id");
   vector<string> fields;
   fields.push_back("q");
   fields.push_back("id");
   list<vector<string> > values;
   vector<string> blank_values;
   blank_values.push_back(assigned_query_number);
   blank_values.push_back("");
   for(vector<string>::const_iterator i = id_vec.begin(); i != id_vec.end(); i++)
   {
    blank_values[1] = *i;
    values.push_back(blank_values);
   }
   if (!ExecutePreparedInsert (dbTransaction, "temp_id", fields, values))
    throw(99);

   string assigned_view_number = GetNextId(dbTransaction, "temp_views");
   string desired_view_name = "temp_view_action_" + assigned_view_number;
   string sql = "create view " + desired_view_name + " (id) as select id from temp_id where q = " + assigned_query_number;
   if (!ExecuteStatement(dbTransaction, sql))
   {
    PSay("Failed to create view.\n" + sql, m_database_error, MB_OK);
    throw(1);
   }

   // gotta make that view visible and all
   dbTransaction->Commit();
   dbTransaction->StartTransaction();

   if (!action_container->Execute(dbTransaction, desired_view_name))
    throw(0);

   sql = "delete from temp_id where q = " + assigned_query_number;
   if (!ExecuteStatement(dbTransaction, sql))
   {
    PSay("Failed to delete temporary data.\n" + sql, m_database_error, MB_OK);
    throw(2);
   }

   sql = "drop view " + desired_view_name;
   if (!ExecuteStatement(dbTransaction, sql))
   {
    PSay("Failed to drop view.\n" + sql, m_database_error, MB_OK);
    throw(1);
   }
   dbTransaction->Commit();
   should_close = true;
  } catch (...)
  {
   if (dbTransaction->Active)
    dbTransaction->Rollback();
  }

  delete dbTransaction;
 // drop stuff in a temp table,
 // create a view,
 // execute,
 // clean up
 }

 if (should_close)
  Close();
}
//---------------------------------------------------------------------------
void __fastcall TformSpecialAction::buttonCancelClick(TObject *Sender)
{
 Close();
}
//---------------------------------------------------------------------------
bool TformSpecialAction::FillActionMenu(TMenuItem *Parent, string source_table_name)
{
 bool answer = false;
 if (Parent)
 {
  Parent->Clear();

  vector<a_factory_info> picker_content = action_factories.GetFactories(source_table_name.c_str());
  TMenuItem * temp = NULL;
  for (vector<a_factory_info>::iterator i = picker_content.begin(); i != picker_content.end(); i++)
  {
   temp = new TMenuItem(Parent);
   temp->Caption = (SentenceString(ReplaceStringWithString(i->description, "$", LowercaseString(source_table_name.c_str())))).c_str();
   temp->OnClick = action_click;
   temp->Tag = (unsigned int)(void*)((*i).create_pointer);
   Parent->Add(temp);
   answer = true;
  }
 }

 Parent->Visible = answer;
 return answer;
}
//---------------------------------------------------------------------------
bool TformSpecialAction::FillActionMenu(TMenuItem *Parent, string source_table_name, TformRecordGeneric* record_form)
{
 bool answer = false;
 if (Parent)
 {
  Parent->Clear();

  vector<a_factory_info> picker_content = action_factories.GetFactories(source_table_name.c_str());
  TMenuItem * temp = NULL;
  for (vector<a_factory_info>::iterator i = picker_content.begin(); i != picker_content.end(); i++)
  {
   temp = new TMenuItem(Parent);
   temp->Caption = (SentenceString(ReplaceStringWithString(i->description, "$", LowercaseString(source_table_name.c_str())))).c_str();
   temp->OnClick = record_form->menuActionsClick;
   temp->Tag = (unsigned int)(void*)((*i).create_pointer);
   Parent->Add(temp);
   answer = true;
  }
 }

 Parent->Visible = answer;
 return answer;
}
//---------------------------------------------------------------------------

