//---------------------------------------------------------------------------

#include <vcl.h>
#pragma hdrstop

#include "codeSpecialAppointmentReminders.h"
#include "codeRecordAppointment.h"
#include "launchDiabRegistryReport.h"
#include "reportDiabAppointmentLetter.h"
#include "reportMailingLabels.h"
#include "scheduling.h"
//---------------------------------------------------------------------------
#pragma package(smart_init)
#pragma link "codeSpecialGeneric"
#pragma resource "*.dfm"
TformSpecialAppointmentReminders *formSpecialAppointmentReminders;
//---------------------------------------------------------------------------
__fastcall TformSpecialAppointmentReminders::TformSpecialAppointmentReminders(TComponent* Owner)
        : TformSpecialGeneric(Owner)
{
  this->IconIndex = 7;
  dbTransaction = new TIBTransaction(NULL);
  dbTransaction->DefaultDatabase = dbDatabase;
  dbTransaction->StartTransaction();
  lookupTables->RegisterComboBox("users", cboFor);
  lookupTables->RegisterComboBox("users", cboBy);

  Pdate two_weeks(DateToStr(Date()).c_str(), mdy | yyyy);
  two_weeks.mod(2, in_weeks);
  dateBegin->Date = Date();
  dateEnd->Date = StrToDate(two_weeks.OutDate(mdy | yyyy).c_str());

  result_filter.SetListViewHandle(listResults);
  result_filter.SetTransaction(dbTransaction);
  result_filter.SetSecurity("appointments");
  result_filter.AddColumnSpec("Date", "appointments", as_date, true, 100);
  result_filter.AddColumnSpec("Time", "appointments", as_time, true, 85);
  result_filter.AddColumnSpec("Person", "people", as_text, true, 120);
  result_filter.AddColumnSpec("Description", "appointments", as_text, true, 150, sentenceString);
  result_filter.AddColumnSpec("Created For", "appointments", as_text, false, 100);
  result_filter.AddColumnSpec("Created By", "appointments", as_text, false, 100);
  result_filter.AddColumnSpec("Current Id", "current_id", as_int, false, 80);
  result_filter.AddColumnSpec("Internal Id", "internal_id", as_int, false, 80);
  result_filter.FetchPreferences("specialappointmentreminders.result_filter");
  result_filter.SetAddEdit(this, NULL, EditRecord(&(TformSpecialAppointmentReminders::EditAppointment)));
  result_filter.SetIconStatus(true);
  result_filter.SetCheck(true);
  result_filter.SetSQLStatement ("");
  result_filter.RefreshDataSet();
}
//---------------------------------------------------------------------------
void __fastcall TformSpecialAppointmentReminders::btnCloseClick(TObject *Sender)
{
  Close();        
}
//---------------------------------------------------------------------------
bool TformSpecialAppointmentReminders::IsValidOnScreen()
{
  if (! lookupTables->IsInLookUpTable("users", cboFor->Text.c_str()))
  {
    PSay("Please enter a valid employee that the appointment was created for", m_invalid_data, MB_OK);
    cboFor->SetFocus();
    return false;
  }
  else if (! lookupTables->IsInLookUpTable("users", cboBy->Text.c_str()))
  {
    PSay("Please enter a valid employee that the appointment was created by", m_invalid_data, MB_OK);
    cboBy->SetFocus();
    return false;
  }
  else
    return true;
}
//---------------------------------------------------------------------------
bool TformSpecialAppointmentReminders::Search()
{
  Processing(true);
  try
  {
    ExecuteStatement(dbTransaction, "delete from appointment_searches");
    string sql;
    PApptOccurrenceMap all_appointments;
    vector<string> user_ids;
    if(cboFor->Text != "")
      user_ids.push_back(lookupTables->IdOf("users", cboFor->Text.c_str()));
    else
      SqlToVec(dbTransaction, "select id from users where valid = 1 and id > 0", user_ids);
    map<string, vector<string> > ids_to_match;
    ids_to_match["visit_outcome_id"].push_back("0");
    ids_to_match["visit_outcome_id"].push_back(lookupTables->IdOf("visit_outcome_types", "As Scheduled"));
    ids_to_match["visit_outcome_id"].push_back(lookupTables->IdOf("visit_outcome_types", "Walk-In"));
    ids_to_match["visit_outcome_id"].push_back(lookupTables->IdOf("visit_outcome_types", "Arrived Late"));
    Pdate begin_date;
    begin_date.InDate(dateBegin->Date);
    Pdate end_date;
    end_date.InDate(dateEnd->Date);
    DownloadAppointments(dbDatabase, begin_date, end_date, current_userid, user_ids, vector<string>(), "",
                         all_appointments, ids_to_match, (optionsPermissions["admin.all_appointments"]["enabled"] == "true"),
                         false, "");
    map<string, string> insert_map;
    for(PApptOccurrenceMap::iterator itr = all_appointments.begin(); itr != all_appointments.end(); itr++)
    {
      for(PApptOccurrenceSet::iterator set_itr = itr->second.begin(); set_itr != itr->second.end(); set_itr++)
      {
        insert_map["id"] = (*set_itr).id;
        insert_map["appt_date"] = itr->first.OutDate(mdy | yyyy);
        insert_map["appt_time"] = (*set_itr).range_begin.OutTime(military | withsec);
        ExecuteInsertStatement(dbTransaction, "appointment_searches", insert_map);
      }
    }
    sql = "select appointment_searches.appt_date, appointment_searches.appt_time, "
          "person_names.first_name || ' ' || person_names.last_name || ' ' || person_names.suffix, "
          "appointments.description, created_for_user.item, created_by_user.item, "
          "appointments.current_id, appointments.id "
          "from appointment_searches "
          "inner join appointments on appointment_searches.id = appointments.id "
          "left join people on appointments.person_id = people.id "
          "left join person_names on people.person_names_id = person_names.id "
          "left join tax_entities on appointments.tax_entity_id = tax_entities.id "
          "left join users created_for_user on appointments.created_for = created_for_user.id "
          "left join users created_by_user on appointments.created_by = created_by_user.id "
          "where appointments.person_id > 0";
    if (cboBy->Text != "")
      sql += "and appointments.created_by = " + lookupTables->IdOf("users", cboBy->Text.c_str()) + " ";
    if (editDescription->Text != "")
      sql += "and appointments.description like '%" + SafeSQLString(editDescription->Text.c_str()) + "%' ";
    sql += "order by appointment_searches.appt_date, appointment_searches.appt_time, "
           "person_names.last_name, person_names.first_name";
    result_filter.SetSQLStatement(sql);
    result_filter.RefreshDataSet();
  }
  catch(...)
  {
  }
  Processing(false); 
  return true;
}
//---------------------------------------------------------------------------
void __fastcall TformSpecialAppointmentReminders::btnSearchClick(TObject *Sender)
{
  if(IsValidOnScreen())
    Search();        
}
//---------------------------------------------------------------------------
void __fastcall TformSpecialAppointmentReminders::btnSelectAllClick(TObject *Sender)
{
  result_filter.CheckAll();        
}
//---------------------------------------------------------------------------
void __fastcall TformSpecialAppointmentReminders::btnUnselectAllClick(TObject *Sender)
{
  result_filter.UncheckAll();        
}
//---------------------------------------------------------------------------
void TformSpecialAppointmentReminders::EditAppointment()
{
  map<string,string> temp_map;
  temp_map["id"] = result_filter.GetSelectedRecordID();
  FORM_TRY
    TformRecordAppointment * appointmentRecord = new TformRecordAppointment(NULL, temp_map);
    appointmentRecord->Show();
  FORM_CATCH
}
//---------------------------------------------------------------------------
void __fastcall TformSpecialAppointmentReminders::FormClose(TObject *Sender, TCloseAction &Action)
{
  lookupTables->UnregisterComboBox("users", cboFor);
  lookupTables->UnregisterComboBox("users", cboBy);
  ExecuteStatement(dbTransaction, "delete from appointment_searches");
  dbTransaction->Commit();
  delete dbTransaction;
  TformSpecialGeneric::FormClose(Sender, Action);        
}
//---------------------------------------------------------------------------
void __fastcall TformSpecialAppointmentReminders::FormShortCut(TWMKey &Msg,bool &Handled)
{
  if(Msg.CharCode == VK_RETURN)
  {
    Handled = true;
    btnSearchClick(NULL);
  }
  else
    Handled = false;
}
//---------------------------------------------------------------------------
void __fastcall TformSpecialAppointmentReminders::btnPrintClick(TObject *Sender)
{
  vector<string> id_vec = result_filter.GetMultiID();
  if(! id_vec.empty())
  {
    TformReportDiabAppointmentLetter *report_form = NULL;
    FORM_TRY
      report_form = new TformReportDiabAppointmentLetter(NULL);
    FORM_CATCH
    if(!report_form) // woops?
      return;

    Processing(true);
    TIBDataSet *dbLetterDataset = new TIBDataSet(NULL);

    string sql;
    string temp = "";
    string id_string = VecToCommaString(id_vec);
    vector<string> employee_vec;
    try
    {
      dbLetterDataset->Database = dbDatabase;
      dbLetterDataset->Transaction = dbTransaction;
      sql = "select users.item || ' - ' || users.title employee "
            "from users where users.valid = 1 and "
            "users.group_id = " + lookupTables->IdOf("groups", "Diabetes") + " "
            "order by users.item";
      SqlToVec(dbTransaction, sql, employee_vec);
      for(vector<string>::iterator itr = employee_vec.begin(); itr != employee_vec.end(); itr++)
        report_form->memoEmployees->Lines->Add((*itr).c_str());

      sql = "select person_names.first_name || ' ' || person_names.last_name full_name, "
            "person_contact_info.line1, person_contact_info.line2, "
            "person_contact_info.city || ', ' || state_types.abbreviation || '  ' || "
            "person_contact_info.zip city, people.gender_id, "
            "appointments.appointment_date, appointments.appointment_time, "
            "cast(appointments.duration as int) || ' ' || duration_types.item duration, "
            "case when appt_location_types.item != '' then appt_location_types.item else '<unknown>' end location, "  // jhaley 20110412: was missing!
            "appointments.description reason "
            "from appointments "
            "inner join appt_location_types on appointments.appt_location_id = appt_location_types.id "
            "inner join duration_types on appointments.duration_id = duration_types.id "
            "inner join people on appointments.person_id = people.id "
            "inner join person_names on people.person_names_id = person_names.id "
            "inner join person_contact_info on people.person_contact_info_id = person_contact_info.id "
            "inner join state_types on person_contact_info.state_id = state_types.id "
            "where appointments.id in (" + id_string + ") "
            "order by person_names.first_name, person_names.last_name";

      dbLetterDataset->SelectSQL->Add(sql.c_str());
      dbLetterDataset->Open();
      report_form->report->DataSet = dbLetterDataset;
      report_form->txtPerson->DataSet = dbLetterDataset;
      report_form->txtPerson->DataField = "full_name";
      report_form->txtLine1->DataSet = dbLetterDataset;
      report_form->txtLine1->DataField = "line1";
      report_form->txtLine2->DataSet = dbLetterDataset;
      report_form->txtLine2->DataField = "line2";
      report_form->txtGreeting->DataSet = dbLetterDataset;
      report_form->txtGreeting->DataField = "gender_id";
      report_form->txtReason->DataSet = dbLetterDataset;
      report_form->txtReason->DataField = "reason";
      report_form->txtDate->DataSet = dbLetterDataset;
      report_form->txtDate->DataField = "appointment_date";
      report_form->txtTime->DataSet = dbLetterDataset;
      report_form->txtTime->DataField = "appointment_time";
      report_form->txtLocation->DataSet = dbLetterDataset; // jhaley 20110412: was missing, causing exception
      report_form->txtLocation->DataField = "location";
      report_form->txtDuration->DataSet = dbLetterDataset;
      report_form->txtDuration->DataField = "duration";
      Processing(false);
      report_form->report->Preview();

      TformReportMailingLabels *label_form = NULL;
      FORM_TRY
        label_form = new TformReportMailingLabels(NULL);
      FORM_CATCH
      if(label_form)
      {
        label_form->labelReport->DataSet = dbLetterDataset;
        label_form->txtName->DataSet = dbLetterDataset;
        label_form->txtName->DataField = "full_name";
        label_form->txtAddress1->DataSet = dbLetterDataset;
        label_form->txtAddress1->DataField = "line1";
        label_form->labelReport->Preview();
      }

      dbLetterDataset->Close();
    }
    catch(...)
    {
      Processing(false);
      PSay("Error printing letter", m_database_error, MB_OK);
    }
    delete dbLetterDataset;
  }
  else
    PSay("Please check the appointments you want to print", m_missing_data, MB_OK);
}
//---------------------------------------------------------------------------


