//---------------------------------------------------------------------------

#include <vcl.h>
#pragma hdrstop

#include "codeSpecialCachedFiles.h"
#include "codeSpecialUploadFile.h"
#include "cached_files.h"
//---------------------------------------------------------------------------
#pragma package(smart_init)
#pragma link "codeSpecialGeneric"
#pragma resource "*.dfm"
TformSpecialCachedFiles *formSpecialCachedFiles;
//---------------------------------------------------------------------------
__fastcall TformSpecialCachedFiles::TformSpecialCachedFiles(TComponent* Owner)
        : TformSpecialGeneric(Owner)
{
}
//---------------------------------------------------------------------------
void TformSpecialCachedFiles::AddDatabases(TIBTransaction *dbTransaction)
{
  string server_sql;
  pvecmap database_map;
  pmapstrs main_db;

  // TODO: add prometheus_update_sources databases (for now, just put in the main DB for testing purposes)
#if 0
  server_sql = "select distinct server_name, server_file from prometheus_update_sources where valid = 1 "
               "order by server_name, server_file";
  SqlToVecMap(dbTransaction, server_sql, database_map);
#endif

  // Add the main database, which is not included in the prometheus_update_sources table but is in the user cfg
  main_db["server_name"] = optionsIni["server"]["server_name"];
  main_db["server_file"] = optionsIni["server"]["server_file"];
  database_map.push_back(main_db);

  TTreeNode *prevnode = NULL;
  for(pvecmap::iterator i_db = database_map.begin(); i_db != database_map.end(); ++i_db)
  {
    // Create DB object
    sDBObject newDBObject;
    newDBObject.server_name = (*i_db)["server_name"];
    newDBObject.server_file = (*i_db)["server_file"];
    newDBObject.is_global_db = (newDBObject.server_name == main_db["server_name"]);

    string server_key = newDBObject.server_name + ":" + newDBObject.server_file;
    sDBObject &mapDBObject = databases[server_key] = newDBObject;

    // Create root node in the tree for this database
    prevnode = treeFiles->Items->AddObject(prevnode, server_key.c_str(), &mapDBObject);
    mapDBObject.root_node = prevnode;
  }
}
//---------------------------------------------------------------------------
TTreeNode *TformSpecialCachedFiles::CreateNodesForPath(string &path, TformSpecialCachedFiles::sDBObject &dbObject)
{
  TTreeNodes *nodes   = treeFiles->Items;
  TTreeNode  *parent  = dbObject.root_node;
  TTreeNode  *node    = NULL;
  string trimmed_path = FileCache::RemoveTrailingSlash(FileCache::RemoveLeadingSlash(path));
  string current_part;
  size_t len = trimmed_path.length();

  for(size_t i = 0; i < len; i++)
  {
    if(i == len - 1) // for sake of the final path component
      current_part.push_back(trimmed_path[i]);

    if(trimmed_path[i] == '\\' || i == len - 1) // end of the current path component
    {
      if(dbObject.node_map.find(current_part) == dbObject.node_map.end()) // keep path components unique
      {
        node = nodes->AddChild(parent, current_part.c_str());
        dbObject.node_map[current_part] = node;
      }
      else
      {
        node = dbObject.node_map[current_part];
      }
      parent = node;
      current_part = "";
    }
    else // build up the current path component
      current_part.push_back(trimmed_path[i]);
  }

  // Expand the database root down to the file name levels; this will keep individual versions collapsed, which is good
  // since there will be a lot of nodes for each version of the file.
  dbObject.root_node->Expand(true);

  return node; // return the last node created or found
}
//---------------------------------------------------------------------------
void TformSpecialCachedFiles::AddFiles(TformSpecialCachedFiles::sDBObject &dbObject)
{
  TIBDatabase    *dbCurrentDatabase  = NULL;
  TIBTransaction *dbLocalTransaction = NULL;

  if(dbObject.is_global_db)
    dbCurrentDatabase = dbDatabase;       // global Prometheus database
  else
    dbCurrentDatabase = dbUploadDatabase; // local update database, via TIBDatabase control on this form

  try
  {
    // Connect if necessary
    ConnectToDatabase(dbCurrentDatabase, dbObject.server_name + ":" + dbObject.server_file, "ZEUS", "argon");

    // Open transaction
    dbLocalTransaction = new TIBTransaction(NULL);
    StdTransaction(dbLocalTransaction, dbCurrentDatabase);

    // Get data for all files
    string files_sql =
      "select id, last_update_stamp, last_updated_by, valid, version_number, version_number1, version_number2, "
      "version_number3, version_number4, file_path, file_name, sha1_hash, comments "
      "from file_blobs "
      "order by file_path, file_name, "
      " version_number1 desc, version_number2 desc, version_number3 desc, version_number4 desc";
    SqlToVecMap(dbLocalTransaction, files_sql, dbObject.raw_files);

    // Create tree nodes for all files
    int num_files = dbObject.raw_files.size();
    for(int i = 0; i < num_files; i++)
    {
      sFileObject newFileObject;
      pmapstrs &row = dbObject.raw_files[i];

      string full_file_path = FileCache::PathConcatenate(row["file_path"], row["file_name"]);

      // Create tree nodes for the full file path and name
      TTreeNode  *path_node = CreateNodesForPath(full_file_path, dbObject);

      newFileObject.database = &dbObject;
      newFileObject.index    = i;

      string node_name = "v" + row["version_number"];
      sFileObject &mapFileObject = dbObject.file_objects[row["id"]] = newFileObject;

      // Create leaf nodes for each version of the file
      mapFileObject.node = treeFiles->Items->AddChildObject(path_node, node_name.c_str(), &mapFileObject);
    }

    dbLocalTransaction->Commit();
  }
  catch(EIBError &error)
  {
    if (dbLocalTransaction && dbLocalTransaction->Active)
      dbLocalTransaction->Rollback();
  }
  catch(...)
  {
    if (dbLocalTransaction && dbLocalTransaction->Active)
      dbLocalTransaction->Rollback();
  }

  // Disconnect from a local update database if we connected to one.
  if(dbUploadDatabase->TestConnected())
    dbUploadDatabase->Close();

  delete dbLocalTransaction;
}
//---------------------------------------------------------------------------
void TformSpecialCachedFiles::BuildFileTree()
{
  TIBTransaction *dbTransaction = new TIBTransaction(NULL);

  // Clear out the tree view and the cached database/file information
  treeFiles->Items->Clear();
  databases.clear();

  try
  {
    if(StdTransaction(dbTransaction, dbDatabase))
    {
      // Add all file server databases as root nodes
      AddDatabases(dbTransaction);

      // For each database, add all the files
      for(map<string, sDBObject>::iterator i_db = databases.begin(); i_db != databases.end(); ++i_db)
        AddFiles(i_db->second);
    }
  }
  catch(EIBError &error)
  {
    PSay(error.Message.c_str(), m_database_error, MB_OK);
  }
  catch(...)
  {
    PSay("An unknown error occurred while building the file tree", m_program_error, MB_OK);
  }

  delete dbTransaction;
}
//---------------------------------------------------------------------------
void __fastcall TformSpecialCachedFiles::FormShow(TObject *Sender)
{
  // Build the tree of files under each file server database
  BuildFileTree();

  // Clear the label captions and edit box
  lblTimestamp->Caption = "";
  lblUserName->Caption = "";
  lblHashCode->Caption = "";
  editChanges->Text = "";

  // Disable some context-sensitive buttons
  btnDownload->Enabled  = false;
  btnDropLocal->Enabled = false;

  TformSpecialGeneric::FormShow(Sender);
}
//---------------------------------------------------------------------------
void __fastcall TformSpecialCachedFiles::treeFilesChange(TObject *Sender, TTreeNode *Node)
{
  // If it has a valid object pointer, and is not one of the database root nodes, it's a file revision leaf node
  if(Node->Data != NULL && Node->Level > 0)
  {
    sFileObject *file     = static_cast<sFileObject *>(Node->Data);
    sDBObject   *database = file->database;
    pmapstrs &raw_file_info = database->raw_files[file->index];

    groupFileBox->Caption = FileCache::PathConcatenate(raw_file_info["file_path"], raw_file_info["file_name"]).c_str();
    Pstamp time_stamp(raw_file_info["last_update_stamp"], datefirst | mdy | yyyy | military);
    lblTimestamp->Caption = (time_stamp.OutDate(mdy | yyyy) + " " + time_stamp.OutTime(withoutsec | military)).c_str();
    lblUserName->Caption = lookupTables->ValueOf("users", raw_file_info["last_updated_by"]).c_str();
    lblHashCode->Caption = raw_file_info["sha1_hash"].c_str();

    checkIsValid->Checked = (raw_file_info["valid"] == "1");
    checkCachedLocal->Checked = FileCache::FileExistsInCache(raw_file_info["file_path"], raw_file_info["file_name"]);

    editChanges->Text = raw_file_info["comments"].c_str();

    // With a file revision selected, you can upload a new revision of that file, download it to the local cache,
    // or drop the local cached copy of the file if such exists.
    btnUploadNew->Caption = "Upload New Version";
    btnDownload->Enabled  = true;
    btnDropLocal->Enabled = checkCachedLocal->Checked;
  }
  else
  {
    // Clear all the informational controls
    groupFileBox->Caption = "(Click a file version to view information)";
    lblTimestamp->Caption = "";
    lblUserName->Caption = "";
    lblHashCode->Caption = "";
    checkIsValid->Checked = false;
    checkCachedLocal->Checked = false;

    // With anything other than a file revision node selected, you can upload a new file.
    btnUploadNew->Caption = "Upload New File";
    btnDownload->Enabled  = false;
    btnDropLocal->Enabled = false;
  }
}
//---------------------------------------------------------------------------
void __fastcall TformSpecialCachedFiles::btnCloseClick(TObject *Sender)
{
  Close();
}
//---------------------------------------------------------------------------
void __fastcall TformSpecialCachedFiles::btnUploadNewClick(TObject *Sender)
{
  pvecstr  temp_databases;
  pmapstrs temp_file;
  string   global_database;
  bool     did_upload = false;

  // Throw all the databases into a convenient vector
  for(map<string, sDBObject>::iterator i_db = databases.begin(); i_db != databases.end(); ++i_db)
  {
    temp_databases.push_back(i_db->first); // the key is the server_name:server_file string, which we need to connect.
    if (i_db->second.is_global_db)
      global_database = i_db->first;
  }

  // Get the data for the currently selected file node, if such exists (otherwise it's a new file upload)
  TTreeNode *the_node = treeFiles->Selected;
  if(the_node && the_node->Data != NULL && the_node->Level > 0)
  {
    sFileObject *file = static_cast<sFileObject *>(the_node->Data);
    temp_file = file->database->raw_files[file->index];
  }

  TformSpecialUploadFile *upload_form = NULL;
  FORM_TRY
    upload_form = new TformSpecialUploadFile(NULL, temp_databases, global_database, temp_file, &did_upload);
  FORM_CATCH_RETURN("TformSpecialUploadFile", )
  upload_form->ShowModal();

  // Update the file tree if necessary
  if(did_upload)
    BuildFileTree();
}
//---------------------------------------------------------------------------
void __fastcall TformSpecialCachedFiles::btnDownloadClick(TObject *Sender)
{
  pmapstrs     temp_file;
  sDBObject   *the_db;
  sFileObject *the_file;
  FileCache::get_file_flags flags = FileCache::gf_noresult;

  TTreeNode *the_node = treeFiles->Selected;
  if(the_node && the_node->Data != NULL && the_node->Level > 0)
  {
    the_file  = static_cast<sFileObject *>(the_node->Data);
    the_db    = the_file->database;
    temp_file = the_db->raw_files[the_file->index];
  }
  else
    return; // how'd you get here, anyway? :P

  TStream     *file_stream       = NULL;
  TIBDatabase *dbCurrentDatabase = NULL;
  try
  {
    if(the_db->is_global_db)
      dbCurrentDatabase = dbDatabase; // Global Prometheus database connection
    else
      dbCurrentDatabase = dbUploadDatabase; // file server database

    if(!dbCurrentDatabase->TestConnected())
      ConnectToDatabase(dbCurrentDatabase, the_db->server_name + ":" + the_db->server_file, "ZEUS", "argon");

    file_stream = FileCache::GetCacheableFile(dbCurrentDatabase, NULL, temp_file["file_path"], temp_file["file_name"], flags);
    if(!file_stream)
      PSay("Download failed", m_warning, MB_OK);
    else
    {
      // Check if it was saved to file or what...
      if(flags & FileCache::gf_sourcedisk)
        PSay("The file was retrieved from the local disk cache.", m_message, MB_OK);
      else if(flags & FileCache::gf_wrotetodisk)
        PSay("The file was downloaded and has been successfully cached.", m_message, MB_OK);
      else
        PSay("The file was downloaded, but could not be written to disk!", m_warning, MB_OK);
    }
  }
  catch(EIBError &error)
  {
    PSay(error.Message.c_str(), m_database_error, MB_OK);
  }
  catch(...)
  {
    PSay("Unknown error while downloading file", m_program_error, MB_OK);
  }

  if(file_stream)
    delete file_stream;

  // Close the form-local database connection if necessary
  if(dbUploadDatabase->TestConnected())
    dbUploadDatabase->Close();

  // Reset selection
  treeFiles->Selected = the_node;
  treeFilesChange(this, the_node);
}
//---------------------------------------------------------------------------
void __fastcall TformSpecialCachedFiles::btnDropLocalClick(TObject *Sender)
{
  pmapstrs     temp_file;
  sDBObject   *the_db;
  sFileObject *the_file;
  TTreeNode   *the_node = treeFiles->Selected;

  if (the_node && the_node->Data != NULL && the_node->Level > 0)
  {
    the_file  = static_cast<sFileObject *>(the_node->Data);
    the_db    = the_file->database;
    temp_file = the_db->raw_files[the_file->index];
  }
  else
    return; // how'd you get here, anyway? :P

  if(!FileCache::DropFromCache(temp_file["file_path"], temp_file["file_name"]))
    PSay("Could not delete local file", m_warning, MB_OK);

  // Reset selection
  treeFiles->Selected = the_node;
  treeFilesChange(this, the_node);
}
//---------------------------------------------------------------------------

