//---------------------------------------------------------------------------

#include <vcl.h>
#pragma hdrstop

#include "codeSpecialCancelPurchaseReq.h"
//---------------------------------------------------------------------------
#pragma package(smart_init)
#pragma link "codeSpecialGeneric"
#pragma resource "*.dfm"
TformSpecialCancelPurchaseReq *formSpecialCancelPurchaseReq;
//---------------------------------------------------------------------------
__fastcall TformSpecialCancelPurchaseReq::TformSpecialCancelPurchaseReq(TComponent* Owner, string purchase_req_number)
        : TformSpecialGeneric(Owner)
{
 purchase_req_id = purchase_req_number;
 panelSelectArea->Color = title_bar_color;
}
//---------------------------------------------------------------------------
void __fastcall TformSpecialCancelPurchaseReq::buttonFinishClick(
      TObject *Sender)
{
 int user_answer = PSay("This change could be difficult to undo. Please make sure you truly intended this action. Do you really want to cancel this PR?", m_confirmation, MB_YESNO);
 if (user_answer == IDNO)
  return;

 // on finish
 TIBTransaction * dbTransaction = new TIBTransaction(NULL);
 try
 {
  dbTransaction->DefaultDatabase = dbDatabase;
  dbTransaction->DefaultAction = TARollback;
  dbTransaction->StartTransaction();

  string sql_verify_purchase_req_number = "select purchase_reqs.id from purchase_reqs where id = " + purchase_req_id;
  purchase_req_id = GetOneField(dbTransaction, sql_verify_purchase_req_number);

  string current_transaction_number = GetNextId(dbTransaction, "transact_number");

  if (purchase_req_id != "")
  {
   // if the user wants to cancel a PR then mark it as deleted so they know it isn't valid anymore
   string sql;
   sql = "update purchase_reqs set purchase_reqs.current_id = 0 where purchase_reqs.id = " + purchase_req_id;
   if (!ExecuteStatement(dbTransaction, sql))
     {
      PSay("There is a balance of 0.00 on this bill, but the program is unable to mark it as deleted.", m_database_error, MB_OK);
     }

   // now make sure it's not already on a check -- if it is, they need to break apart the check too
   // this is gonna piss them off, oh yeah ...

   string sql_verify_no_check = "select purchase_reqs.check_id from purchase_reqs where id = " + purchase_req_id;
   string check_id = GetOneField(dbTransaction, sql_verify_no_check);

   if (check_id != "" && check_id != "0")
   {
    PSay("This purchase request is currently included on a check.\n"
         "No modification can be made to it so long as it remains on the check.\n"
         "Please void the check in order to detach the purchase request and make your changes.", m_warning, MB_OK);
   }
   else if (checkOne->Checked)
   {
    // continue by unlinking purchase_reqs from the check
    string sql_unlink_bills = "update bills set purchase_req_id = 0 where purchase_req_id = " + purchase_req_id;
    if (!ExecuteStatement(dbTransaction, sql_unlink_bills))
    {
     PSay("Failed to unlink bills from the purchase request.\nMake sure you don't have any of the bill records locked for update.", m_database_error, MB_OK);
     throw(7);
    }
   }
   else if (checkTwo->Checked)
   {
    // this time, we don't want to re-obligate stuff. instead ... we want to cancel the expense and the payable, rather than the cash and the anti-obligation
    // TODO: some sort of group by? select only fields needed?

    // mark bills as deleted that are no longer valid
    sql = "update bills set current_id = 0 where purchase_req_id = " + purchase_req_id;
    if (!ExecuteStatement(dbDatabase, sql))
      {
       PSay("There is a balance of 0.00 on this bill, but the program is unable to mark it as deleted.", m_database_error, MB_OK);
       throw(8);
      }


    //  string sql_select_expense = "select * from transactions inner join bills on transactions.bill_id = bills.id inner join accounts on transactions.account_id = accounts.id where purchase_reqs.check_id = " + purchase_req_id + " and accounts.account_type_id = " + lookupTables->IdOf("account_types", "Expense");
    // The above doesn't work.  I think Philip was trying to match purchase_reqs on bills.
    string sql_select_expense =
      "select * "
      "from transactions "
      "inner join bills on transactions.bill_id = bills.id "
      "inner join accounts on transactions.account_id = accounts.id "
      "where bills.purchase_req_id = " + purchase_req_id +
      " and accounts.account_type_id = " + lookupTables->IdOf("account_types", "Expense");

    vector<map<string, string> > expense_transactions;
    SqlToVecMap(dbTransaction, sql_select_expense, expense_transactions);

    for (vector<map<string, string> >::iterator i_old = expense_transactions.begin();
         i_old != expense_transactions.end(); i_old++)
    {
     map<string, string> reverse_transaction_core;
     reverse_transaction_core["transaction_number"] = current_transaction_number;
     reverse_transaction_core["effective_date"] = Date().CurrentDate().FormatString("mm/dd/yyyy").c_str(); // TODO: EFFECTIVE_DATE
     reverse_transaction_core["current_id"] = "4";
     reverse_transaction_core["last_update_stamp"] = RightNow();
     reverse_transaction_core["last_updated_by"] = current_userid;
     reverse_transaction_core["tax_entity_id"] = (*i_old)["tax_entity_id"];
     reverse_transaction_core["task_id"] = (*i_old)["task_id"];
     reverse_transaction_core["bill_id"] = (*i_old)["bill_id"];

     map<string, string> reverse_transaction_payable = reverse_transaction_core;
     map<string, string> reverse_transaction_self = reverse_transaction_core;

     string payable_account_id = EquivAccount (pending_ref, (*i_old)["account_id"]);
     string self_account_id = (*i_old)["account_id"];

     if (payable_account_id != "")
     {
      reverse_transaction_payable["note"] = "Cancelling of payable transactions on bills because purchase request is being cancelled";
      reverse_transaction_payable["id"] = GetNextId(dbTransaction, "transactions");
      reverse_transaction_payable["account_id"] = payable_account_id;
      reverse_transaction_payable["credit_amount"] = (*i_old)["credit_amount"];
      reverse_transaction_payable["debit_amount"] = (*i_old)["debit_amount"];
      if (!ExecuteInsertStatement(dbTransaction, "transactions", reverse_transaction_payable))
      {
       PSay("Failed to insert a reversal (payable) transaction.", m_database_error, MB_OK);
       throw(15);
      }
     }
     else
     {
      PSay("Could not identify the accounts payable account to undo based on the expense account used.", m_program_error, MB_OK);
      throw(16);
     }

     if (self_account_id != "")
     {
      reverse_transaction_self["note"] = "Cancelling of expense transactions on bills because purchase request is being cancelled";
      reverse_transaction_self["id"] = GetNextId(dbTransaction, "transactions");
      reverse_transaction_self["account_id"] = self_account_id;
      reverse_transaction_self["credit_amount"] = (*i_old)["debit_amount"];
      reverse_transaction_self["debit_amount"] = (*i_old)["credit_amount"];
      if (!ExecuteInsertStatement(dbTransaction, "transactions", reverse_transaction_self))
      {
       PSay("Failed to insert a reversal (expense) transaction.", m_database_error, MB_OK);
       throw(14);
      }
     }
     else
     {
      PSay("Could not identify the expense (self) account to undo based on the expense account used.", m_program_error, MB_OK);
      throw(17);
     }
    }
    // we don't care to unlink anything at all!
   }
   else
   {
    PSay("Magically, you've managed to not tell me what to do!", m_missing_data, MB_OK);
   }
  }
  else
  {
   PSay("The check identifier sent when this form opened could not be verified as valid.", m_program_error, MB_OK);
  }

  dbTransaction->Commit();
  Close();
 }
 catch (...)
 {
  if (dbTransaction->Active)
   dbTransaction->Rollback();
  PSay("Couldn't void the check, internal error.", m_database_error, MB_OK);
 }
 delete dbTransaction;
}
//---------------------------------------------------------------------------
void __fastcall TformSpecialCancelPurchaseReq::buttonCloseClick(
      TObject *Sender)
{
 // on close
 Close();
}
//---------------------------------------------------------------------------
void __fastcall TformSpecialCancelPurchaseReq::FormClose(TObject *Sender,
      TCloseAction &Action)
{
 // on form close
 TformSpecialGeneric::FormClose(Sender, Action);
}
//---------------------------------------------------------------------------
