//---------------------------------------------------------------------------

#include <vcl.h>
#pragma hdrstop

#include "codeSpecialChangePassword.h"
//---------------------------------------------------------------------------
#pragma package(smart_init)
#pragma link "codeSpecialGeneric"
#pragma resource "*.dfm"
TformSpecialChangePassword *formSpecialChangePassword;
//---------------------------------------------------------------------------
__fastcall TformSpecialChangePassword::TformSpecialChangePassword(TComponent* Owner, bool p_disable_close)
        : TformSpecialGeneric(Owner)
{
  disable_close = p_disable_close;
  properly_closing = false;

  if(disable_close)
    btnClose->Enabled = false;

  // set the database pointers to null so we don't try to delete them
  // later if the new fails
  dbTransaction = NULL;

  lblUser->Caption = PrettyString(lookupTables->ValueOf("users", current_userid)).c_str();

  try
  {
    // create database objects to be used on form
    dbTransaction = new TIBTransaction(NULL);

    // set up the objects... the database pointer is a global from the main form
    StdTransaction(dbTransaction, dbDatabase, false);
  }
  catch(...)
  {
    if (dbTransaction->Active)
      dbTransaction->Rollback();
  }
}
//---------------------------------------------------------------------------
void __fastcall TformSpecialChangePassword::FormClose(TObject *Sender, TCloseAction &Action)
{
  if (disable_close && !properly_closing)
  {
   // ABORT!
   Action = caNone;
   return;
  }

  // delete the database objects if they are not null
  if (dbTransaction)
    delete dbTransaction;
  // get scheduled to free memory, always
  Action = caFree;
  TformSpecialGeneric::FormClose(Sender, Action);
}
//---------------------------------------------------------------------------
void __fastcall TformSpecialChangePassword::btnCloseClick(TObject *Sender)
{
  Close();
}
//---------------------------------------------------------------------------
void __fastcall TformSpecialChangePassword::btnSaveClick(TObject *Sender)
{
  string sql = "select pass, username, item from users where id = " + current_userid;
  map<string, string> result_map;
  SqlToMap(dbTransaction, sql, result_map);
  string old_pass = result_map["pass"];
  string username = result_map["username"];
  string entered_old = editOld->Text.c_str();
  string new_pass = editNew->Text.c_str();
  string new_pass_repeat = editRepeatNew->Text.c_str();

  map<string, string> field_map;

  /* TODO -c extra_restrictions: it would be nice to disallow them using either their first or last name
     (note this gets slightly more complicated with alternate and nick names (debbie or deb instead of debra, etc.;
     and I'm not even going to suggest checking maiden name, family/pet names, date of birth, or combinations thereof.)
  */
  bool security_check_failed = false;
  if (entered_old == old_pass)
  {
    if (new_pass == new_pass_repeat)
    {
      if (TformSpecialChangePassword::PasswordMeetsComplexityRequirements(new_pass))
      {
        if (new_pass != username)
        {
          field_map["id"] = current_userid;
          field_map["pass"] = new_pass;
          field_map["password_expired"] = "0";
          field_map["last_password_change"] = "current_timestamp";
          map<string, int> sql_options;
          for (map<string, string>::iterator itr=field_map.begin(); itr!=field_map.end(); itr++)
          {
            if (itr->first == "last_password_change")
              sql_options[itr->first] = sql_no_processing;
            else
              sql_options[itr->first] = sql_no_uppercasing;
          }
          if (ExecuteUpdateStatement(dbTransaction, "users", field_map, sql_options))
          {
            properly_closing = true;
            Close();
          }
          else
            PSay("Error updating password... password not changed", m_warning, MB_OK);
        }
        else
          security_check_failed = true;
      }
      else
        security_check_failed = true;
    }
    else
    {
      editNew->SetFocus();
      PSay("The new passwords do not match", m_invalid_data, MB_OK);
    }
  }
  else
  {
    editOld->SetFocus();
    PSay("Your old password is incorrect", m_invalid_data, MB_OK);
  }
  if (security_check_failed)
  {
    PSay("For security reasons, your password must meet various security requirements.\n\n"
         "Refer to the information on the Change Password screen, and enter a password that meets these criteria.\n\n"
         "You may also refer to the Prometheus Rules of Behavior for password requirements.", m_warning, MB_OK);
    editNew->Text = "";
    editRepeatNew->Text = "";
    editNew->SetFocus();
  }

}
//---------------------------------------------------------------------------
void __fastcall TformSpecialChangePassword::FormShortCut(TWMKey &Msg,bool &Handled)
{
 Handled = true;
 if (Msg.CharCode == VK_RETURN)
  btnSave->Click();
 else
  Handled = false;
}
//---------------------------------------------------------------------------
bool TformSpecialChangePassword::PasswordMeetsComplexityRequirements(string p_pass)
{
  bool result = true;
  if (!IsBlankOrZero(current_userid))
  {
    string pass = p_pass;
    if (pass == "")
    {
      string sql = "select pass from users where id = " + current_userid;
      pass = GetOneField(dbDatabase, sql);
    }
    if (pass.length() < 8 ||             // too short
        UppercaseString(pass) == pass || // no lowercase letters
        LowercaseString(pass) == pass || // no uppercase letters
        pass.find_first_of("0123456789") == string::npos) // no numbers
    {
      result = false;
    }
  }
  return result;
}
//---------------------------------------------------------------------------

