//---------------------------------------------------------------------------
#include <vcl.h>
#pragma hdrstop

#include "codeSpecialChat.h"
#include "Clipbrd.hpp"
extern TformSpecialChat * chat_window;
//---------------------------------------------------------------------------
#pragma package(smart_init)
#pragma link "codeSpecialGeneric"
#pragma resource "*.dfm"
TformSpecialChat *formSpecialChat;
//---------------------------------------------------------------------------
__fastcall TformSpecialChat::TformSpecialChat(TComponent* Owner)
    : TformSpecialGeneric(Owner)
{
  this->IconIndex = 15;
  new_message_icon = 15;
  busy_icon = 16;
  idle_icon = 17;
  normal_icon = 18;
  first_property_set = true;
  if(optionsPreferences["chat"]["show_timestamp"] == "true")
    menuShowTimestamp->Checked = true;
  else
    menuShowTimestamp->Checked = false;
  flash_timer = NULL;
  Application->OnActivate = FormActivate;
  listUsers->SmallImages = imagesRecordTypes;
  pageMessages->Images = imagesRecordTypes;
  /* Now why would we be doing this this way, when we have a link to people, to person_names,
     where we know first, middle, and last names explicitly?  The problem with splitting on
     spaces is if you go by first space, you screw up Ca Trinna and Mary Ann, but if you go
     by last space, you screw up Kayci Little Axe.  Let's just do it right, neh?  And let's
     include last initial for all those Debra's, etc. while we're at it.
  this_user_name = current_fullname;
  unsigned int last_space = this_user_name.find_last_of(' ');
  if(last_space != string::npos)
    this_user_name = this_user_name.substr(0, last_space);
  */
  this_user_name = GetUserName(dbDatabase, current_userid, first_last_initial);
  try
  {
    TIBTransaction *dbTransaction = new TIBTransaction(NULL);
    dbTransaction->DefaultDatabase = dbDatabase;
    dbTransaction->StartTransaction();
    string sql = "select permissions.user_id from permissions where s = 'SPECIAL.CHAT' and "
                 "k = 'ENABLED' and v = 'TRUE'";
    SqlToSet(dbTransaction, sql, users_with_permissions);
    sql = "select preferences.user_id from preferences where s = 'CHAT' and "
          "k = 'DO_NOT_RECEIVE_MESSAGES' and v = 'TRUE'";
    SqlToSet(dbTransaction, sql, users_not_chatting);
    dbTransaction->Commit();
    not_chatting_request = atlas->RequestNotificationWhen("chat.users_not_chatting", "0", rel_any, as_int, this,
                           RefreshHandler(&(TformSpecialChat::AtlasRefresh)));
  }
  catch(...)
  {
    PSay("Error loading users", m_database_error, MB_OK);
  }
  message_router->AddCallback("chat_messages", this, ChannelReadable(&(TformSpecialChat::HandleIncomingMessages)));
  message_router->AddCallback("chat_notifications", this, ChannelReadable(&(TformSpecialChat::HandleIncomingMessages)));
  message_router->AddCallback("chat_properties", this, ChannelReadable(&(TformSpecialChat::HandleIncomingMessages)));
  message_router->RequestServerNotification ("chat_notifications", "set_property");
  message_router->RequestServerNotification ("chat_notifications", "client_disconnect");
  message_router->GetProperties("chat_properties");
}
//---------------------------------------------------------------------------
void __fastcall TformSpecialChat::btnCloseClick(TObject *Sender)
{
  Close();
}
//---------------------------------------------------------------------------
void __fastcall TformSpecialChat::FormClose(TObject *Sender, TCloseAction &Action)
{
  chat_window = NULL;
  Application->OnActivate = NULL;
  message_router->DelCallback(this);
  if(menuShowTimestamp->Checked)
    optionsPreferences["chat"]["show_timestamp"] = "true";
  else
    optionsPreferences["chat"]["show_timestamp"] = "false";
  TformSpecialGeneric::FormClose(Sender, Action);
}
//---------------------------------------------------------------------------
void TformSpecialChat::AtlasRefresh(int request_num, string matched_value)
{
  string sql = "select preferences.user_id from preferences where s = 'CHAT' and "
               "k = 'DO_NOT_RECEIVE_MESSAGES' and v = 'TRUE'";
  SqlToSet(dbDatabase, sql, users_not_chatting);
  UpdateIcons();
}
//---------------------------------------------------------------------------
void TformSpecialChat::HandleIncomingMessages(string channel)
{
  string message;
  if(channel == "chat_notifications")
  {
    while(message_router->IsMessageWaiting("chat_notifications"))
      message_router->GetNextMessage("chat_notifications");
    message_router->GetProperties("chat_properties");
  }
  else if(channel == "chat_properties")
  {
    while(message_router->IsMessageWaiting("chat_properties"))
    {
      message = message_router->GetNextMessage("chat_properties");
      if(message == "starting properties")
      {
        previous_user_info = user_info;
        connection_info.clear();
        user_info.clear();
      }
      else if(message == "ending properties")
      {
        CreateUserIdMap();
        if(UsersLoggedInChanged())
        {
          UpdateListView();
          ShowLoginTimestamps();
        }
        else
          UpdateIcons();
      }
      else
      {
        int first_sep;
        int second_sep;
        string connection_number;
        string property_name;
        string property_value;

        first_sep = message.find_first_of("~");
        second_sep = message.find("~", first_sep + 1);
        connection_number = message.substr(0, first_sep);
        property_name = message.substr(first_sep + 1, second_sep - first_sep - 1);
        property_value = message.substr(second_sep + 1, message.length() - second_sep);
        connection_info[connection_number][property_name] = property_value;
      }
    }
  }
  else if(channel == "chat_messages")
  {
    int first_sep;
    string user_id;
    while(message_router->IsMessageWaiting("chat_messages"))
    {
      message = message_router->GetNextMessage("chat_messages");
      first_sep = message.find(':', 0);
      user_id = message.substr(0, first_sep);
      if(tab_info.find(user_id) == tab_info.end())
        AddNewTab(user_id);
      message = message.substr(first_sep + 1, message.length() - first_sep);
      chat_messages[user_id].push_back(message);
    }
    ShowChatMessages();
  }
}
//---------------------------------------------------------------------------
void TformSpecialChat::CreateUserIdMap()
{
  string user_id;
  for(map<string, map<string, string> >::iterator outer_itr = connection_info.begin(); outer_itr != connection_info.end(); outer_itr++)
  {
    if(outer_itr->second.find("user_id") != outer_itr->second.end())
    {
      user_id = outer_itr->second["user_id"];
      if((outer_itr->second["client_type"] == "prometheus") && (users_with_permissions.find(user_id) != users_with_permissions.end()) &&
         (user_id != current_userid))
      {
        user_info[user_id]["connection_id"] = outer_itr->first;
        for(map<string, string>::iterator inner_itr = outer_itr->second.begin(); inner_itr != outer_itr->second.end(); inner_itr++)
          user_info[user_id][inner_itr->first] = inner_itr->second;
      } // end if prometheus connection and they have permissions
    } // end if user id is found in the map
  } // end outer for
}
//---------------------------------------------------------------------------
bool TformSpecialChat::UsersLoggedInChanged()
{
  for(map<string, map<string, string> >::iterator itr = user_info.begin(); itr != user_info.end(); itr++)
    if(previous_user_info.find(itr->first) == previous_user_info.end())
      return true;

  for(map<string, map<string, string> >::iterator itr = previous_user_info.begin(); itr != previous_user_info.end(); itr++)
    if(user_info.find(itr->first) == user_info.end())
      return true;

  // need to put messages in edit boxes still
  
  return false;
}
//---------------------------------------------------------------------------
void TformSpecialChat::ShowLoginTimestamps()
{
  if(! first_property_set)
  {
    Ptime current_time = CurrentPtime();
    string message;
    TRichEdit *edit_box;
    int last_char;
    for(map<string, map<string, string> >::iterator itr = user_info.begin(); itr != user_info.end(); itr++)
    {
      if((previous_user_info.find(itr->first) == previous_user_info.end()) && (tab_info.find(itr->first) != tab_info.end()))
      {
        edit_box = tab_info[itr->first].received_messages;
        message = tab_info[itr->first].short_name + " logged in at " + current_time.OutTime(military | withsec);
        message_class temp_class;
        temp_class.timestamp = "";
        temp_class.message = message;
        tab_info[itr->first].lines.push_back(temp_class);
        last_char = edit_box->Text.Length();
        edit_box->Lines->Add(message.c_str());
        edit_box->SelStart = last_char;
        edit_box->SelLength = message.length();
        edit_box->SelAttributes->Color = clBlue;
        edit_box->SelStart = edit_box->Text.Length();
        edit_box->SelLength = 0;
        edit_box->Perform(EM_SCROLLCARET, 0, 0);
      }
    }
    for(map<string, map<string, string> >::iterator itr = previous_user_info.begin(); itr != previous_user_info.end(); itr++)
    {
      if((user_info.find(itr->first) == user_info.end()) && (tab_info.find(itr->first) != tab_info.end()))
      {
        edit_box = tab_info[itr->first].received_messages;
        message = tab_info[itr->first].short_name + " logged out at " + current_time.OutTime(military | withsec);
        message_class temp_class;
        temp_class.timestamp = "";
        temp_class.message = message;
        tab_info[itr->first].lines.push_back(temp_class);
        last_char = edit_box->Text.Length();
        edit_box->Lines->Add(message.c_str());
        edit_box->SelStart = last_char;
        edit_box->SelLength = message.length();
        edit_box->SelAttributes->Color = clBlue;
        edit_box->SelStart = edit_box->Text.Length();
        edit_box->SelLength = 0;
        edit_box->Perform(EM_SCROLLCARET, 0, 0);
      }
    }
  }
  else
    first_property_set = false;
}
//---------------------------------------------------------------------------
void TformSpecialChat::UpdateListView()
{
  listUsers->Items->Clear();
  TListItem *row;
  map<string, map<string, string> > sorted_by_name;
  string name;
  for(map<string, map<string, string> >::iterator outer_itr = user_info.begin(); outer_itr != user_info.end(); outer_itr++)
  {
    name = outer_itr->second["full_name"];
    for(map<string, string>::iterator inner_itr = outer_itr->second.begin(); inner_itr != outer_itr->second.end(); inner_itr++)
      sorted_by_name[name][inner_itr->first] = inner_itr->second;
  }
  for(map<string, map<string, string> >::iterator itr = sorted_by_name.begin(); itr != sorted_by_name.end(); itr++)
  {
    row = listUsers->Items->Add();
    row->Caption = PrettyString(itr->first).c_str();
    row->SubItems->Add(itr->second["user_id"].c_str());
    if(users_not_chatting.find(itr->second["user_id"]) != users_not_chatting.end())
      row->ImageIndex = busy_icon;
    else if(itr->second["is_idle"] == "yes")
      row->ImageIndex = idle_icon;
    else
      row->ImageIndex = normal_icon;
  }
}
//---------------------------------------------------------------------------
void TformSpecialChat::UpdateIcons()
{
  int row_count = listUsers->Items->Count;
  TListItem *row;
  string user_id;
  for(int i = 0; i < row_count; i++)
  {
    row = listUsers->Items->Item[i];
    user_id = row->SubItems->Strings[0].c_str();
    if(users_not_chatting.find(user_id) != users_not_chatting.end())
    {
      row->ImageIndex = busy_icon;
//      PSay("user " + user_id + " is busy", m_debug, MB_OK);
    }
    else if(user_info[user_id]["is_idle"] == "yes")
      row->ImageIndex = idle_icon;
    else
      row->ImageIndex = normal_icon;
  }
}
//---------------------------------------------------------------------------
void TformSpecialChat::AddNewTab(string user_id)
{
  pageMessages->Visible = true;
  tab_info_class temp_class;

  TTabSheet * new_sheet = new TTabSheet(pageMessages);
  /* Now why would we be doing this this way, when we have a link to people, to person_names,
     where we know first, middle, and last names explicitly?  The problem with splitting on
     spaces is if you go by first space, you screw up Ca Trinna and Mary Ann, but if you go
     by last space, you screw up Kayci Little Axe.  Let's just do it right, neh?  And let's
     include last initial for all those Debra's, etc. while we're at it.
  string caption = lookupTables->ValueOf("users", user_id);
  unsigned int last_space = caption.find_last_of(' ');
  if(last_space != string::npos)
    caption = caption.substr(0, last_space);
  */
  string caption = GetUserName(dbDatabase, user_id, first_last_initial);
  temp_class.short_name = caption;
  new_sheet->Caption = caption.c_str();
  new_sheet->PageControl = pageMessages;
  new_sheet->Parent = pageMessages;
  new_sheet->OnShow = TabSheetShow;
  new_sheet->ImageIndex = -1;
  temp_class.tab_sheet = new_sheet;

  int margin = 8;
  int sending_edit_height = 65;
  int received_edit_height = new_sheet->Height - sending_edit_height - (margin * 4);
  TRichEdit *received_edit = new TRichEdit(new_sheet);
  TAnchors received_anchors;
  received_anchors << akLeft << akRight << akTop << akBottom;
  received_edit->Anchors = received_anchors;
  received_edit->Left = margin;
  received_edit->Top = margin;
  received_edit->Width = new_sheet->Width - (margin * 2);
  received_edit->Height = received_edit_height;
  received_edit->PlainText = true;
  received_edit->ReadOnly = true;
  received_edit->ScrollBars = ssVertical;
  received_edit->PopupMenu = popupOptions;
  received_edit->Parent = new_sheet;
  temp_class.received_messages = received_edit;

  TRichEdit *sending_edit = new TRichEdit(new_sheet);
  TAnchors sending_anchors;
  sending_anchors << akLeft << akRight << akBottom;
  sending_edit->Anchors = sending_anchors;
  sending_edit->Left = margin;
  sending_edit->Top = received_edit_height + (margin * 3);
  sending_edit->Width = new_sheet->Width - (margin * 2);
  sending_edit->Height = sending_edit_height;
  sending_edit->PlainText = true;
  sending_edit->ReadOnly = false;
  sending_edit->ScrollBars = ssVertical;
  sending_edit->PopupMenu = popupPaste;
  sending_edit->Parent = new_sheet;
  sending_edit->OnKeyPress = SendingEditKeyPress;
  temp_class.message_to_send = sending_edit;

  tab_info[user_id] = temp_class;
}
//---------------------------------------------------------------------------
void TformSpecialChat::ShowChatMessages()
{
  string message;
  string timestamp = CurrentTime();
  int last_char;
  int sel_length;
  TRichEdit *edit_box;

  for(map<string, list<string> >::iterator itr = chat_messages.begin(); itr != chat_messages.end(); itr++)
  {
    edit_box = tab_info[itr->first].received_messages;
    edit_box->ReadOnly = false;
    if(! itr->second.empty())
    {
      if(tab_info[itr->first].tab_sheet->PageIndex != pageMessages->ActivePageIndex)
        tab_info[itr->first].tab_sheet->ImageIndex = new_message_icon;
    }

    while(! itr->second.empty())
    {
      last_char = edit_box->Text.Length();
      message = tab_info[itr->first].short_name;
      message_class temp_class;
      temp_class.timestamp = timestamp;
      temp_class.message = message + ": " + itr->second.front();
      tab_info[itr->first].lines.push_back(temp_class);
      if(menuShowTimestamp->Checked)
        message += " (" + timestamp + ")";
      message += ": " + itr->second.front();
      edit_box->Lines->Add(message.c_str());
      edit_box->SelStart = last_char;
      sel_length = tab_info[itr->first].short_name.length() + 1;
      if(menuShowTimestamp->Checked)
        sel_length += timestamp.length() + 3;
      edit_box->SelLength = sel_length;
      edit_box->SelAttributes->Color = clBlue;
      edit_box->SelStart = edit_box->Text.Length();
      edit_box->SelLength = 0;
      edit_box->Perform(EM_SCROLLCARET, 0, 0);
      itr->second.pop_front();
    }
    edit_box->ReadOnly = true;
  }
}
//---------------------------------------------------------------------------
void TformSpecialChat::SendMessage(TRichEdit *sending_box)
{
  if(users_not_chatting.find(current_userid) != users_not_chatting.end())
  {
    PSay("You cannot send messages because you are currently marked as 'busy'.\n"
         "To change this setting, use the Settings screen under File on the main menu.\n", m_reminder, MB_OK);
  }
  else
  {
    string user_id;
    string message_to_send;
    string message_to_display;
    string timestamp = CurrentTime();
    int last_char;
    int sel_length;

    for(map<string, tab_info_class>::iterator itr = tab_info.begin(); itr != tab_info.end(); itr++)
    {
      if(itr->second.message_to_send == sending_box)
      {
        user_id = itr->first;
        break;
      }
    }

    if(users_not_chatting.find(user_id) != users_not_chatting.end())
    {
      PSay(lookupTables->ValueOf("users", user_id) + " is busy and is not currently accepting messages", m_reminder, MB_OK);
    }
    else if(user_info.find(user_id) != user_info.end())
    {
      message_to_send = current_userid + ":" + string(sending_box->Text.c_str());
      last_char = tab_info[user_id].received_messages->Text.Length();
      tab_info[user_id].received_messages->ReadOnly = false;
      message_to_display = this_user_name;
      if(menuShowTimestamp->Checked)
        message_to_display += " (" + timestamp + ")";
      message_to_display += ": " + string(sending_box->Text.c_str());
      tab_info[user_id].received_messages->Lines->Add(message_to_display.c_str());
      message_class temp_class;
      temp_class.timestamp = timestamp;
      temp_class.message = this_user_name + ": " + sending_box->Text.c_str();
      tab_info[user_id].lines.push_back(temp_class);
      tab_info[user_id].received_messages->SelStart = last_char;
      sel_length = this_user_name.length() + 1;
      if(menuShowTimestamp->Checked)
        sel_length += timestamp.length() + 3;
      tab_info[user_id].received_messages->SelLength =  sel_length;
      tab_info[user_id].received_messages->SelAttributes->Color = clRed;
      tab_info[user_id].received_messages->SelStart = tab_info[user_id].received_messages->Text.Length();
      tab_info[user_id].received_messages->SelLength = 0;
      tab_info[user_id].received_messages->Perform(EM_SCROLLCARET, 0, 0);
      tab_info[user_id].received_messages->ReadOnly = true;
      message_router->SendMessageToAny("user_id", user_id, "chat_messages", message_to_send);
      sending_box->Lines->Clear();
      sending_box->SetFocus();
    }
    else
    {
      PSay(string("Could not send your last message... " + tab_info[user_id].short_name + " is not logged in").c_str(), m_message, MB_OK);
    }
  }
}
//---------------------------------------------------------------------------
void __fastcall TformSpecialChat::listUsersDblClick(TObject *Sender)
{
  if(listUsers->SelCount > 0)
  {
    string user_id = listUsers->Selected->SubItems->Strings[0].c_str();
    if(users_not_chatting.find(user_id) != users_not_chatting.end())
      PSay(lookupTables->ValueOf("users", user_id) + " is busy and is not currently accepting messages", m_reminder, MB_OK);
    else
    {
      if(tab_info.find(user_id) == tab_info.end())
        AddNewTab(user_id);
      tab_info[user_id].tab_sheet->Show();
      tab_info[user_id].message_to_send->SetFocus();
    }
  }
}
//---------------------------------------------------------------------------
void __fastcall TformSpecialChat::SendingEditKeyPress(TObject *Sender, char &Key)
{
  if(IsEnter(Key))
  {
    Key = 0;
    SendMessage((TRichEdit *)Sender);
//    PSay("called it", m_debug, MB_OK);
  }
}
//---------------------------------------------------------------------------
void __fastcall TformSpecialChat::TabSheetShow(TObject *Sender)
{
  TTabSheet * tab_sheet = (TTabSheet *)Sender;
  if(tab_sheet->ImageIndex != -1)
    tab_sheet->ImageIndex = -1;
  for(map<string, tab_info_class>::iterator itr = tab_info.begin(); itr != tab_info.end(); itr++)
  {
    if(tab_sheet == itr->second.tab_sheet)
    {
      itr->second.message_to_send->SetFocus();
      break;
    }
  }
}
//---------------------------------------------------------------------------
void __fastcall TformSpecialChat::menuCloseTabClick(TObject *Sender)
{
  string user_id;
  TTabSheet *clicked_sheet = pageMessages->Pages[last_clicked_sheet_index];
  if(clicked_sheet)
  {
    for(map<string, tab_info_class>::iterator itr = tab_info.begin(); itr != tab_info.end(); itr++)
    {
      if(clicked_sheet == itr->second.tab_sheet)
      {
        user_id = itr->first;
        break;
      }
    }
    tab_info[user_id].received_messages->Parent = NULL;
    tab_info[user_id].message_to_send->Parent = NULL;
    tab_info[user_id].tab_sheet->PageControl = NULL;
    tab_info[user_id].tab_sheet->Parent = NULL;
    RemoveComponent(tab_info[user_id].received_messages);
    RemoveComponent(tab_info[user_id].message_to_send);
    RemoveComponent(tab_info[user_id].tab_sheet);
    tab_info.erase(user_id);
    if(pageMessages->PageCount == 0)
      pageMessages->Visible = false;
  }
}
//---------------------------------------------------------------------------
void TformSpecialChat::ShowTimestamps(bool p_to_show)
{
  int line_count;
  string new_message;
  string old_message;
  int first_colon;
  string this_name;
  TColor this_color;
  int sel_start;
  int sel_length;
  for(map<string, tab_info_class>::iterator itr = tab_info.begin(); itr != tab_info.end(); itr++)
  {
    line_count = itr->second.lines.size();
    itr->second.received_messages->Text = "";
    for(int i = 0; i < line_count; i++)
    {
      old_message = itr->second.lines[i].message;
      new_message = old_message;
      if(itr->second.lines[i].timestamp != "")
      {
        first_colon = old_message.find(":", 0);
        this_name = old_message.substr(0, first_colon);
        if(this_name == this_user_name)
          this_color = clRed;
        else
          this_color = clBlue;
        if(p_to_show)
        {
          new_message = this_name + " (" + itr->second.lines[i].timestamp + "):" + old_message.substr(first_colon+1, old_message.length()-first_colon);
          sel_length = string(this_name + " (" + itr->second.lines[i].timestamp + "):").length();
        }
        else
          sel_length = string(this_name + ":").length();
      }
      else
      {
        this_color = clBlue;
        sel_length = new_message.length();
      }
      sel_start = itr->second.received_messages->Text.Length();
      itr->second.received_messages->Lines->Add(new_message.c_str());
      itr->second.received_messages->SelStart = sel_start;
      itr->second.received_messages->SelLength = sel_length;
      itr->second.received_messages->SelAttributes->Color = this_color;
    }
    itr->second.received_messages->SelStart = itr->second.received_messages->Text.Length();
    itr->second.received_messages->SelLength = 0;
  }
}
//---------------------------------------------------------------------------
void __fastcall TformSpecialChat::pageMessagesMouseDown(TObject *Sender,
      TMouseButton Button, TShiftState Shift, int X, int Y)
{
  if(Button == mbRight)
    last_clicked_sheet_index = pageMessages->IndexOfTabAt(X, Y);
}
//---------------------------------------------------------------------------
void TformSpecialChat::SetFlashTimer(TTimer *p_timer)
{
  flash_timer = p_timer;
}
//---------------------------------------------------------------------------
void __fastcall TformSpecialChat::FormActivate(TObject *Sender)
{
  if((flash_timer) && (this->WindowState != wsMinimized))
  {
    flash_timer->Enabled = false;
    flash_timer = NULL;
  }
}
//---------------------------------------------------------------------------
void __fastcall TformSpecialChat::menuShowTimestampClick(TObject *Sender)
{
  if(menuShowTimestamp->Checked)
  {
    menuShowTimestamp->Checked = false;
    ShowTimestamps(false);
  }
  else
  {
    menuShowTimestamp->Checked = true;
    ShowTimestamps(true);
  }
}
//---------------------------------------------------------------------------
void __fastcall TformSpecialChat::menuCopySelectedClick(TObject *Sender)
{
  TTabSheet *active_tab = pageMessages->ActivePage;
  TRichEdit *edit_box;
  for(map<string, tab_info_class>::iterator itr = tab_info.begin(); itr != tab_info.end(); itr++)
  {
    if(itr->second.tab_sheet == active_tab)
    {
      edit_box = itr->second.received_messages;
      break;
    }
  }
  Clipboard()->AsText = edit_box->SelText;
}
//---------------------------------------------------------------------------
void __fastcall TformSpecialChat::menuCopyAllClick(TObject *Sender)
{
  TTabSheet *active_tab = pageMessages->ActivePage;
  TRichEdit *edit_box;
  for(map<string, tab_info_class>::iterator itr = tab_info.begin(); itr != tab_info.end(); itr++)
  {
    if(itr->second.tab_sheet == active_tab)
    {
      edit_box = itr->second.received_messages;
      break;
    }
  }
  Clipboard()->AsText = edit_box->Text;
}
//---------------------------------------------------------------------------
void __fastcall TformSpecialChat::menuPasteClick(TObject *Sender)
{
  TTabSheet *active_tab = pageMessages->ActivePage;
  TRichEdit *edit_box;
  for(map<string, tab_info_class>::iterator itr = tab_info.begin(); itr != tab_info.end(); itr++)
  {
    if(itr->second.tab_sheet == active_tab)
    {
      edit_box = itr->second.message_to_send;
      break;
    }
  }
  edit_box->PasteFromClipboard();
}
//---------------------------------------------------------------------------
void __fastcall TformSpecialChat::menuCopyClick(TObject *Sender)
{
  TTabSheet *active_tab = pageMessages->ActivePage;
  TRichEdit *edit_box;
  for(map<string, tab_info_class>::iterator itr = tab_info.begin(); itr != tab_info.end(); itr++)
  {
    if(itr->second.tab_sheet == active_tab)
    {
      edit_box = itr->second.message_to_send;
      break;
    }
  }
  Clipboard()->AsText = edit_box->SelText;
}
//---------------------------------------------------------------------------
void __fastcall TformSpecialChat::menuSaveConversationClick(TObject *Sender)
{
  TTabSheet *active_tab = pageMessages->ActivePage;
  if(dialogSave->Execute())
  {
    TRichEdit *edit_box;
    string user_id;
    for(map<string, tab_info_class>::iterator itr = tab_info.begin(); itr != tab_info.end(); itr++)
    {
      if(itr->second.tab_sheet == active_tab)
      {
        edit_box = itr->second.received_messages;
        user_id = itr->first;
        break;
      }
    }
    string file_name = dialogSave->FileName.c_str();
    fstream output_file;
    output_file.open(file_name.c_str(), ios::out | ios::trunc);
    if (output_file.is_open())
    {
      output_file << "<html><head><title>Prometheus chat</title></head><body>\n";
      int line_count = tab_info[user_id].lines.size();
      string line;
      string this_name;
      int first_colon;
      for(int i = 0; i < line_count; i++)
      {
        if(tab_info[user_id].lines[i].timestamp != "")
        {
          line = tab_info[user_id].lines[i].message;
          first_colon = line.find(":", 0);
          this_name = line.substr(0, first_colon);
          if(this_name == this_user_name)
            output_file << "<font color = \"red\">";
          else
            output_file << "<font color = \"blue\">";
          output_file << this_name;
          if(menuShowTimestamp->Checked)
            output_file << "(" << tab_info[user_id].lines[i].timestamp << "):";
          else
            output_file << ":";
          output_file << "</font>";
          output_file << line.substr(first_colon+1, line.length()-first_colon) << "<br>\n";
        }
        else
          output_file << "<font color=\"blue\">" << tab_info[user_id].lines[i].message << "</font><br>\n";
      }
      output_file << "</body></html>";
      output_file.close();
    }
  }        
}
//---------------------------------------------------------------------------

