//---------------------------------------------------------------------------

#include <vcl.h>
#pragma hdrstop

#include "codeSpecialDiabInactive.h"
#include "reportDiabInactive.h"
#include "codeRecordPerson.h"
//---------------------------------------------------------------------------
#pragma package(smart_init)
#pragma link "codeSpecialGeneric"
#pragma link "codeRubberContainer"
#pragma resource "*.dfm"
TformSpecialDiabInactive *formSpecialDiabInactive;
//---------------------------------------------------------------------------
__fastcall TformSpecialDiabInactive::TformSpecialDiabInactive(TComponent* Owner)
        : TformSpecialGeneric(Owner)
{
  this->IconIndex = 17;
  cboMailing->ItemIndex = 0;
  people_filter.SetListViewHandle(listPeople);
  people_filter.SetSecurity("people.diab");
  people_filter.AddColumnSpec("Name", "people", as_text, true, 140);
  people_filter.AddColumnSpec("Last Activity", "activities", as_date, true, 80);
  people_filter.AddColumnSpec("Last Measurement", "obs_response_sets", as_date, true, 110);
  people_filter.AddColumnSpec("Fitness Membership", "fitness_memberships", as_text, true, 120);
  people_filter.AddColumnSpec("On Mailing List", "people", as_text, true, 110, prettyBoolString);  
  people_filter.AddColumnSpec("Current Id", "current_id", as_int, false, 80);
  people_filter.AddColumnSpec("Internal Id", "internal_id", as_int, false, 80);
  people_filter.FetchPreferences("specialdiabinactive.people_filter");
  people_filter.SetCheck(true);
  people_filter.SetCompress(true);
  people_filter.SetSQLStatement("");
  people_filter.RefreshDataSet();

  dateEnd->Date = Date(); // jhaley: start out at a reasonable date rather than something circa 2003

  // DONE: consider replacing the view somehow. the whole thing is just really slow. view not used anywhere else.
  // -jhaley: Replaced with a pair of queries that do the same work in a miniscule fraction of the time.
}
//---------------------------------------------------------------------------
void __fastcall TformSpecialDiabInactive::btnSelectAllClick(TObject *Sender)
{
  people_filter.CheckAll();        
}
//---------------------------------------------------------------------------
void __fastcall TformSpecialDiabInactive::btnUnselectAllClick(TObject *Sender)
{
  people_filter.UncheckAll();
}
//---------------------------------------------------------------------------
void __fastcall TformSpecialDiabInactive::btnDisplayClick(TObject *Sender)
{
  string sql;
  Pdate six_months_ago(dateEnd->Date);
  six_months_ago.mod(-6, in_months);

  // jhaley 20101105: Using two queries for this is exponentially faster than using the old view, mainly because the old
  // view is written in an extremely inefficient manner (trying to group by person_names and current_id is what's causing
  // it to take ~10 minutes to run, plus that info could just be pulled here since it joined to people again anyway).
  sql = "select P.id from people P "
        "inner join enrollment on P.id = enrollment.person_id and enrollment.current_id+1 >= 4 and "
        " enrollment.enrollment_id = '" + lookupTables->IdOf("enrollment_types", "STEPS Participant") + "' "
        "where P.current_id+1 >= 4 and not exists "
        "(select A.id from activities A where A.person_id = P.id and A.current_id+1 >= 4 and A.date_of_activity >= '"
        + six_months_ago.OutDate(mdy | yyyy) + "') ";

  vector<string> person_ids;
  if (!SqlToVec(dbDatabase, sql, person_ids))
  {
    PSay("Could not get people IDs for STEPS Participants", m_database_error, MB_OK);
    return;
  }

  if (!person_ids.empty())
  {
    sql = "select (PN.last_name || ', ' || PN.first_name || ' ' || PN.suffix) name, "
          "(select max(A.date_of_activity) from activities A where A.person_id = P.id and A.current_id+1 >= 4) last_activity_date, "
          "(select max(ORS.date_of_observation) from obs_response_sets ORS where ORS.person_id = P.id and ORS.current_id+1 >= 4 and "
          " ORS.is_steps_observation = 1 and ORS.group_id = '" + lookupTables->IdOf("groups", "Diabetes") + "') last_observation_date, "
          "(select first 1 FFT.item from fitness_memberships FM inner join fit_facility_types FFT on FM.fit_facility_id = FFT.id "
          " inner join fitness_people FP on FM.id = FP.fitness_membership_id where FP.person_id = P.id and FP.current_id+1 >= 4 and "
          " FM.end_date is null and FM.is_waiting = 0 and FM.current_id+1 >= 4) fitness_facility, "
          "case when MLE.id is not null then 1 else 0 end on_diab_mailing_list, "
          "P.current_id, P.id "
          "from people P "
          "left join person_names PN on P.person_names_id = PN.id "
          "left join enrollment MLE on MLE.person_id = P.id and MLE.current_id+1 >= 4 and "
          " MLE.enrollment_id = '" + lookupTables->IdOf("enrollment_types", "On Diabetes Mailing List") + "' "
          "where " + SafeInClause("P.id", person_ids) + " ";

    if(cboMailing->Text == "Only clients on mailing list")
      sql += "and MLE.id is not null ";
    else if(cboMailing->Text == "Only clients not on mailing list")
      sql += "and MLE.id is null ";

    people_filter.SetSQLStatement(sql);
    Processing(true);
    people_filter.RefreshDataSet();
    people_filter.SetAddEdit(this, NULL, EditRecord(&(TformSpecialDiabInactive::ViewRecord)));
    this->Caption = string("Inactive STEPS Clients - " + IntToString(people_filter.GetResultSize()) + " Rows").c_str();
    Processing(false);
  }
}
//---------------------------------------------------------------------------
void TformSpecialDiabInactive::ViewRecord()
{
  map<string, string> temp_map;
  temp_map["id"] = people_filter.GetSelectedRecordID();
  FORM_TRY
    TformRecordPerson *person_form = new TformRecordPerson(NULL, temp_map);
    person_form->Show();
  FORM_CATCH
}
//---------------------------------------------------------------------------
void __fastcall TformSpecialDiabInactive::btnTakeOffListClick(TObject *Sender)
{
  // jhaley: User should have Diabetes enrollment permissions before they can do this.
  if (optionsPermissions["table.enrollment.diab"]["update"] != "true")
  {
    PSay("You do not have permission to edit Diabetes enrollments.", m_warning, MB_OK);
    return;
  }

  vector<string> id_vec = people_filter.GetMultiID();
  if (!id_vec.empty())
  {
    Processing(true);
    string sql;

    // jhaley: update enrollments
    Pdate date1(DateToStr(Now()).c_str(), mdy | yyyy);
    sql = "update enrollment set last_update_stamp = '" + RightNow() + "', last_updated_by = '" + current_userid + "', "
          "remit_date = '" + date1.OutDate(mdy | yyyy) + "' "
          "where enrollment_id = '" + lookupTables->IdOf("enrollment_types", "On Diabetes Mailing List") + "' and "
          + SafeInClause("person_id", id_vec) + " and current_id+1 >= 4 ";

    if (!ExecuteStatement(dbDatabase, sql))
    {
      Processing(false);
      PSay("Unable to update records", m_database_error, MB_OK);
    }
    else
    {
      Processing(false);
      PSay("Clients sucessfully taken off mailing list", m_message, MB_OK);

      // Send Atlas notifications
      // TODO : not all-encompassing; some forms listen for enrollment.id or enrollment_changes.enrollment_id;
      // the latter is particularly troublesome. Should we do special queries just for the notification?
      for (vector<string>::iterator i_person = id_vec.begin(); i_person != id_vec.end(); i_person++)
        atlas->Notify("enrollment.person_id", *i_person);
    }
  }
  else
    PSay("You have not selected any clients", m_missing_data, MB_OK);
}
//---------------------------------------------------------------------------
void __fastcall TformSpecialDiabInactive::btnPrintCheckedClick(TObject *Sender)
{
  // todo ... print this report
  // set up on click and atlas for list view

  vector<string> id_vec = people_filter.GetMultiID();
  if (!id_vec.empty())
  {
    Processing(true);
    Pdate six_months_ago(dateEnd->Date);
    six_months_ago.mod(-6, in_months);
    string sql;
    TIBTransaction *dbTransaction = new TIBTransaction(NULL);
    TIBDataSet *dbDataset = new TIBDataSet(NULL);

    try
    {
      TformReportDiabInactive *report_form = new TformReportDiabInactive(NULL);

      dbTransaction->DefaultDatabase = dbDatabase;
      dbDataset->Database = dbDatabase;
      dbDataset->Transaction = dbTransaction;
      dbTransaction->StartTransaction();

      // jhaley: No need for either last_activity_view or temp_diab_inactive here any longer.
      sql = "select (PN.last_name || ', ' || PN.first_name || ' ' || PN.suffix) person_name, "
            "(select max(A.date_of_activity) from activities A where A.person_id = P.id and A.current_id+1 >= 4) last_activity, "
            "(select max(ORS.date_of_observation) from obs_response_sets ORS where ORS.person_id = P.id and ORS.current_id+1 >= 4 and "
            " ORS.is_steps_observation = 1 and ORS.group_id = '" + lookupTables->IdOf("groups", "Diabetes") + "') last_measurement, "
            "(select first 1 FFT.item from fitness_memberships FM inner join fit_facility_types FFT on FM.fit_facility_id = FFT.id "
            " inner join fitness_people FP on FM.id = FP.fitness_membership_id where FP.person_id = P.id and FP.current_id+1 >= 4 and "
            " FM.end_date is null and FM.is_waiting = 0 and FM.current_id+1 >= 4) fitness_facility, "
            "case when MLE.id is not null then 1 else 0 end on_mailing_list "
            "from people P "
            "left join person_names PN on P.person_names_id = PN.id "
            "left join enrollment MLE on MLE.person_id = P.id and MLE.current_id+1 >= 4 and "
            " MLE.enrollment_id = '" + lookupTables->IdOf("enrollment_types", "On Diabetes Mailing List") + "' "
            "where " + SafeInClause("P.id", id_vec) + " "
            "order by PN.last_name, PN.first_name, PN.suffix ";

      dbDataset->SelectSQL->Add(sql.c_str());
      dbDataset->Open();
      
      report_form->report->DataSet = dbDataset;
      report_form->lblTitle->Caption = string("Inactive STEPS Clients as of " + six_months_ago.OutDate(mdy | yyyy)).c_str();
      report_form->lblUser->Caption = current_fullname.c_str();
      report_form->txtName->DataSet = dbDataset;
      report_form->txtName->DataField = "person_name";
      report_form->txtLastActivity->DataSet = dbDataset;
      report_form->txtLastActivity->DataField = "last_activity";
      report_form->txtLastMeasurement->DataSet = dbDataset;
      report_form->txtLastMeasurement->DataField = "last_measurement";
      report_form->txtFitness->DataSet = dbDataset;
      report_form->txtFitness->DataField = "fitness_facility";
      report_form->txtMailing->DataSet = dbDataset;
      report_form->txtMailing->DataField = "on_mailing_list";
      Processing(false);
      report_form->report->Preview();

      dbDataset->Close();
      dbTransaction->Commit();
    }
    catch(...)
    {
      Processing(false);
      if (dbTransaction->Active)
        dbTransaction->Rollback();
      PSay("Error printing letter", m_database_error, MB_OK);
    }
    delete dbDataset;
    delete dbTransaction;
  }
  else
  {
    PSay("You have not selected any clients", m_missing_data, MB_OK);
  }
}
//---------------------------------------------------------------------------
void __fastcall TformSpecialDiabInactive::btnCloseClick(TObject *Sender)
{
  Close();        
}
//---------------------------------------------------------------------------
/*
  jhaley - old query that used last_activity_view, preserved for historical interest:
  sql = "select last_activity_view.person_name, last_activity_view.last_activity, last_activity_view.last_measurement, "
        "(select first 1 fit_facility_types.item from fitness_memberships "
        " inner join fitness_people on fitness_people.fitness_membership_id = fitness_memberships.id "
        " inner join fitness_people on fitness_people.person_id = people.id and fitness_people.current_id+1 >= 4 "
        " inner join fit_facility_types on fitness_memberships.fit_facility_id = fit_facility_types.id "
        " where fitness_memberships.end_date is null and fitness_memberships.is_waiting = 0 and "
        " fitness_memberships.current_id+1 >= 4), "
        //"people.on_diab_mailing_list, "
        "case when MLE.id is not null then 1 else 0 end on_diab_mailing_list, "
        "last_activity_view.current_id, last_activity_view.person_id "
        "from people "
        "inner join last_activity_view on last_activity_view.person_id = people.id "
        "left join enrollment MLE on MLE.person_id = people.id and MLE.current_id+1 >= 4 and "
        " MLE.enrollment_id = '" + lookupTables->IdOf("enrollment_types", "On Diabetes Mailing List") + "' "
        "where (last_activity_view.last_activity <= '" + six_months_ago.OutDate(mdy | yyyy) + "' or "
        "last_activity_view.last_activity is null) ";
*/

