//---------------------------------------------------------------------------

#include <vcl.h>
#pragma hdrstop

#include "codeSpecialDiabWarningLetter.h"
#include "codeRecordFitnessMembership.h"
#include "reportDiabWarningLetter.h"
#include "reportMailingLabels.h"
#include "reportDiabCancellationLetter.h"
#include "reportDiabNewMembershipLetter.h"
#include "reportDiabMembershipsReady.h"
#include "reportDiabWarningLettersSent.h"
#include "reportDiabCancellationLettersSent.h"
//---------------------------------------------------------------------------
#pragma package(smart_init)
#pragma link "codeSpecialGeneric"
#pragma resource "*.dfm"
TformSpecialDiabWarningLetter *formSpecialDiabWarningLetter;
//---------------------------------------------------------------------------
__fastcall TformSpecialDiabWarningLetter::TformSpecialDiabWarningLetter(TComponent* Owner)
        : TformSpecialGeneric(Owner)
{
  this->IconIndex = 10;
  upDownYear->Position = CurrentPdate().year;

  memberships_filter.SetListViewHandle(listMemberships);
  memberships_filter.SetSecurity("fitness_memberships");
  memberships_filter.AddColumnSpec("Primary", "people", as_text, true, 140);
  memberships_filter.AddColumnSpec("Facility", "fitness_memberships", as_text, true, 120);
  memberships_filter.AddColumnSpec("Last Warning", "fitness_memberships", as_date, true, 100);
  memberships_filter.AddColumnSpec("Join Date", "fitness_memberships", as_date, true, 100);
  memberships_filter.AddColumnSpec("Notes", "fitness_memberships", as_text, true, 350, sentenceString);
  memberships_filter.AddColumnSpec("Current Id", "current_id", as_int, false, 80);
  memberships_filter.AddColumnSpec("Internal Id", "internal_id", as_int, false, 80);
  memberships_filter.FetchPreferences("specialdiabwarningletters.memberships_filter");
  memberships_filter.SetCheck(true);
  memberships_filter.SetCompress(true);
  memberships_filter.SetSQLStatement("");
  memberships_filter.RefreshDataSet();

}
//---------------------------------------------------------------------------
void TformSpecialDiabWarningLetter::ViewRecord()
{
  map<string, string> temp_map;
  temp_map["id"] = memberships_filter.GetSelectedRecordID();
  FORM_TRY
    TformRecordFitnessMembership *membership_form = new TformRecordFitnessMembership(NULL, temp_map);
    membership_form->Show();
  FORM_CATCH
}
//---------------------------------------------------------------------------
void __fastcall TformSpecialDiabWarningLetter::FormClose(TObject *Sender,
      TCloseAction &Action)
{
  TformSpecialGeneric::FormClose(Sender, Action);
}
//---------------------------------------------------------------------------

void __fastcall TformSpecialDiabWarningLetter::btnSelectAllClick(
      TObject *Sender)
{
  memberships_filter.CheckAll();        
}
//---------------------------------------------------------------------------

void __fastcall TformSpecialDiabWarningLetter::btnUnselectAllClick(
      TObject *Sender)
{
  memberships_filter.UncheckAll();
}
//---------------------------------------------------------------------------
bool TformSpecialDiabWarningLetter::IsValidOnScreen()
{
  if(cboMonth->Text == "")
  {
    PSay("Please enter a month", m_missing_data, MB_OK);
    cboMonth->SetFocus();
    return false;
  }
  else if(editYear->Text == "")
  {
    PSay("Please enter a year", m_missing_data, MB_OK);
    editYear->SetFocus();
    return false;
  }
  else if(! IsInt(editYear->Text.c_str()))
  {
    PSay("Please enter a valid year", m_invalid_data, MB_OK);
    editYear->SetFocus();
    return false;
  }
  else if(editYear->Text.Length() != 4)
  {
    PSay("Please enter a four digit year", m_invalid_data, MB_OK);
    editYear->SetFocus();
    return false;
  }
  else
  {
    Pdate today = CurrentPdate();
    int selected_month = StringToInt(lookupTables->IdOf("month_types", cboMonth->Text.c_str()));
    int selected_year = StringToInt(editYear->Text.c_str());
    int response;
    if(((selected_month > today.month) && (selected_year == today.year)) ||
        (selected_year > today.year))
    {
      string message = "You have selected " + string(cboMonth->Text.c_str()) + " " +
        string(editYear->Text.c_str()) + ", which is in the future. Are you sure you want to continue?";
      response = PSay(message, m_confirmation, MB_YESNO);
      if(response == IDYES)
        return true;
      else
        return false;
    }
    else
      return true;
  }
}
//---------------------------------------------------------------------------
void __fastcall TformSpecialDiabWarningLetter::btnDisplayClick(
      TObject *Sender)
{
  if(IsValidOnScreen())
  {
    Processing(true);

    string sql;
    Pdate eighteen_years_ago = CurrentPdate();
    eighteen_years_ago.mod(-18, in_years);
    bool finished = false;
    Pdate current_begin_date;
    Pdate current_end_date;
    int this_month;

    Pdate begin_date(lookupTables->IdOf("month_types", cboMonth->Text.c_str()) + "/1/" + string(editYear->Text.c_str()), mdy | yyyy);
    this_month = begin_date.month;
    if(begin_date.DayOfWeek() != 1)
    {
      int day_int = begin_date.DayOfWeek();
      int days_to_go_back = 0;
      for(int i = day_int; i > 1; i--)
        days_to_go_back++;
      days_to_go_back -= days_to_go_back * 2;
      begin_date.mod(days_to_go_back, in_days);
    }

    TIBTransaction *dbTransaction = new TIBTransaction(NULL);
    try
    {
      dbTransaction->DefaultDatabase = dbDatabase;
      dbTransaction->StartTransaction();
//      if(! ExecuteStatement(dbTransaction, "delete from temp_warning_letters"))
//        throw 1;
      if(! ExecuteStatement(dbTransaction, "delete from temp_warning_letters2"))
        throw 1;

      sql = "insert into temp_warning_letters2 "
            "select fitness_memberships.id, count(people.id) from fitness_memberships "
            "inner join fitness_people on fitness_people.fitness_membership_id = fitness_memberships.id "
            "inner join people on fitness_people.person_id = people.id "
            "where fitness_memberships.end_date is null and "
            "fitness_memberships.is_waiting != 1 and "
            "fitness_memberships.current_id >= 3 and "
            "fitness_people.is_primary = 1 and "
            "(people.birth_date < '" + eighteen_years_ago.OutDate(mdy | yyyy) + "' or people.birth_date is null) and (";
      current_begin_date = begin_date;
      while(! finished)
      {
        current_end_date = current_begin_date;
        current_end_date.mod(6, in_days);
        while(current_end_date.month != this_month)
          current_end_date.mod(-1, in_days);
        sql += "(not exists (select * from fitness_attendance  where fitness_attendance.person_id = people.id "
            "and fitness_attendance.fitness_membership_id = fitness_memberships.id and "
            "fitness_attendance.date_of_attendance between '" + current_begin_date.OutDate(mdy | yyyy) + "' "
            "and '" + current_end_date.OutDate(mdy | yyyy) + "')) OR ";
        current_begin_date = current_end_date;
        current_begin_date.mod(1, in_days);
        if(current_begin_date.month != this_month)
          finished = true;
      }
      sql = sql.substr(0, sql.length() - 3);
      sql += ") group by fitness_memberships.id";
      if(! ExecuteStatement(dbTransaction, sql))
        throw 1;
/*
      sql = "insert into temp_warning_letters "
            "select person_names.last_name || ', ' || person_names.first_name || ' ' || person_names.suffix, fit_facility_types.item, "
            "temp_warning_letters2.the_count, max(tasks.date_of_task), fitness_memberships.current_id, fitness_memberships.id, "
            "fitness_memberships.comments, fitness_memberships.begin_date "
            "from fitness_memberships "
            "left join tasks on tasks.fitness_membership_id = fitness_memberships.id and "
            "tasks.diab_activity_id = " + lookupTables->IdOf("diab_activity_types", "Attendance warning letter") + " "
            "left join temp_warning_letters2 on temp_warning_letters2.membership_id = fitness_memberships.id "
            "inner join fit_facility_types on fitness_memberships.fit_facility_id = fit_facility_types.id "
            "inner join fitness_people on fitness_people.fitness_membership_id = fitness_memberships.id "
            "inner join people on fitness_people.person_id = people.id "
            "left join person_names on people.person_names_id = person_names.id "
            "where fitness_memberships.end_date is null and "
            "fitness_memberships.is_waiting != 1 and "
            "fitness_memberships.current_id >= 3 and "
            "fitness_people.is_primary = 1 "
            "group by fitness_memberships.id, person_names.last_name, person_names.first_name, person_names.suffix, "
            "fit_facility_types.item, temp_warning_letters2.the_count, fitness_memberships.current_id, "
            "fitness_memberships.comments, fitness_memberships.begin_date";
      if(! ExecuteStatement(dbTransaction, sql))
        throw 1;
      */
      dbTransaction->Commit();
    }
    catch(...)
    {
      Processing(false);
      if(dbTransaction->Active)
        dbTransaction->Rollback();
      PSay("Error loading list of memberships", m_database_error, MB_OK);
    }
    delete dbTransaction;

    sql = "select person_names.last_name || ', ' || person_names.first_name || ' ' || person_names.suffix, "
          "fit_facility_types.item, max(tasks.date_of_task), "
          "fitness_memberships.begin_date, fitness_memberships.comments, fitness_memberships.current_id, "
          "fitness_memberships.id "
          "from fitness_memberships "
          "left join tasks on tasks.fitness_membership_id = fitness_memberships.id and "
          "tasks.diab_activity_id = " + lookupTables->IdOf("diab_activity_types", "Attendance warning letter") + " "
          "inner join temp_warning_letters2 on temp_warning_letters2.membership_id = fitness_memberships.id "
          "inner join fit_facility_types on fitness_memberships.fit_facility_id = fit_facility_types.id "
          "inner join fitness_people on fitness_people.fitness_membership_id = fitness_memberships.id "
          "inner join people on fitness_people.person_id = people.id "
          "left join person_names on people.person_names_id = person_names.id "
          "where fitness_memberships.end_date is null and "
          "fitness_memberships.is_waiting != 1 and "
          "fitness_memberships.current_id >= 3 and "
          "fitness_people.is_primary = 1 and "
          "temp_warning_letters2.the_count > 0 "
          "group by person_names.last_name, person_names.first_name, person_names.suffix, fitness_memberships.id, "
          "fit_facility_types.item, temp_warning_letters2.the_count, fitness_memberships.current_id, "
          "fitness_memberships.comments, fitness_memberships.begin_date";

    memberships_filter.SetAddEdit(this, NULL, EditRecord(&(TformSpecialDiabWarningLetter::ViewRecord)));
    memberships_filter.SetSQLStatement(sql);
    memberships_filter.RefreshDataSet();
    this->Caption = string("Warning Letters - " + IntToString(memberships_filter.GetResultSize())).c_str();
    Processing(false);
  }
}
//---------------------------------------------------------------------------
void __fastcall TformSpecialDiabWarningLetter::btnPrintWarningLettersClick(
      TObject *Sender)
{
  vector<string> id_vec = memberships_filter.GetMultiID();
  if(! id_vec.empty())
  {
    string id_string = VecToCommaString(id_vec);
    string sql;
    TIBTransaction *dbTransaction = new TIBTransaction(NULL);
    TIBDataSet *dbDataset = new TIBDataSet(NULL);
    TIBDataSet *dbLetterDataset = new TIBDataSet(NULL);

    try
    {
      TformReportDiabWarningLetter *letter_form = new TformReportDiabWarningLetter(NULL, dbLetterDataset);
      TformReportMailingLabels *label_form = new TformReportMailingLabels(NULL);
      TformReportDiabWarningLettersSent *report_form = new TformReportDiabWarningLettersSent(NULL);

      dbTransaction->DefaultDatabase = dbDatabase;
      dbDataset->Database = dbDatabase;
      dbDataset->Transaction = dbTransaction;
      dbLetterDataset->Database = dbDatabase;
      dbLetterDataset->Transaction = dbTransaction;
      dbTransaction->StartTransaction();

      sql = "select person_names.first_name || ' ' || person_names.last_name first_last, "
            "person_names.last_name || ', ' || person_names.first_name last_first, "
            "person_contact_info.line1, person_contact_info.line2, "
            "person_contact_info.city || ', ' || state_types.abbreviation || '  ' || "
            "person_contact_info.zip city, fit_facility_types.item facility "
            "from fitness_memberships "
            "inner join fit_facility_types on fitness_memberships.fit_facility_id = fit_facility_types.id "
            "inner join fitness_people on fitness_people.fitness_membership_id = fitness_memberships.id "
            "inner join people on fitness_people.person_id = people.id "
            "left join person_names on people.person_names_id = person_names.id "
            "left join person_contact_info on people.person_contact_info_id = person_contact_info.id "
            "inner join state_types on person_contact_info.state_id = state_types.id "
            "where fitness_people.is_primary = 1 and "
            "fitness_memberships.id in (" + id_string + ") "
            "order by person_names.last_name, person_names.first_name";

      dbDataset->SelectSQL->Add(sql.c_str());
      dbDataset->Open();

      sql = "select outer_memberships.id, person_names.first_name || ' ' || person_names.last_name first_last, "
            "person_names.last_name || ', ' || person_names.first_name last_first, "
            "fit_facility_types.item facility, "
            "(select first 1 person_names.last_name || ', ' || person_names.first_name "
            "from fitness_memberships inner_memberships "
            "inner join fitness_people on fitness_people.fitness_membership_id = fitness_memberships.id "
            "inner join people on fitness_people.person_id = people.id "
            "left join person_names on people.person_names_id = person_names.id "
            "where inner_memberships.id = outer_memberships.id and fitness_people.is_primary = 1) primary_last_first, "
            "(select first 1 person_names.first_name || ' ' || person_names.last_name "
            "from fitness_memberships inner_memberships "
            "inner join fitness_people on fitness_people.fitness_membership_id = fitness_memberships.id "
            "inner join people on fitness_people.person_id = people.id "
            "left join person_names on people.person_names_id = person_names.id "
            "where inner_memberships.id = outer_memberships.id and fitness_people.is_primary = 1) primary_first_last, "
            "(select first 1 person_contact_info.line1 "
            "from fitness_memberships inner_memberships "
            "inner join fitness_people on fitness_people.fitness_membership_id = fitness_memberships.id "
            "inner join people on fitness_people.person_id = people.id "
            "left join person_contact_info on people.person_contact_info_id = person_contact_info.id "
            "where inner_memberships.id = outer_memberships.id and fitness_people.is_primary = 1) line1, "
            "(select first 1 person_contact_info.city || ', ' || state_types.abbreviation || '  ' || person_contact_info.city "
            "from fitness_memberships inner_memberships "
            "inner join fitness_people on fitness_people.fitness_membership_id = fitness_memberships.id "
            "inner join people on fitness_people.person_id = people.id "
            "left join person_contact_info on people.person_contact_info_id = person_contact_info.id "
            "left join state_types on person_contact_info.state_id = state_types.id "
            "where inner_memberships.id = outer_memberships.id and fitness_people.is_primary = 1) line2 "
            "from fitness_memberships outer_memberships "
            "inner join fit_facility_types on fitness_memberships.fit_facility_id = fit_facility_types.id "
            "inner join fitness_people on fitness_people.fitness_membership_id = fitness_memberships.id "
            "inner join people on fitness_people.person_id = people.id "
            "left join person_names on people.person_names_id = person_names.id "
            "where outer_memberships.id in (" + id_string + ") and "
            "fitness_people.current_id >= 3 and fitness_memberships.current_id >= 3 "
            "order by 5, outer_memberships.id, person_names.last_name, person_names.first_name";

      dbLetterDataset->SelectSQL->Add(sql.c_str());
      dbLetterDataset->Open();
      dbLetterDataset->First();

      report_form->report->DataSet = dbDataset;
      report_form->lblDateTitle->Caption = CurrentDate().c_str();
      report_form->lblUser->Caption = current_fullname.c_str();
      report_form->txtName->DataSet = dbDataset;
      report_form->txtName->DataField = "last_first";
      report_form->txtFacility->DataSet = dbDataset;
      report_form->txtFacility->DataField = "facility";

      // i thought quick reports was stupid before, but today it has reached an all time
      // stupid high. for some reason on this report it is printing the last value in the
      // dataset on every row. so, i have a labels on the report that get the values
      // from the dataset directly and put them on the form in the beforeprint event.
      // the dbtext objects are still there just to make the bands print because
      // sometimes detail bands don't print if there is no data assigned.
      // their width is set to 0 so you can't see them.
      letter_form->warning_letter->DataSet = dbLetterDataset;
      letter_form->groupBand->Expression = "id";
      letter_form->txtPrimary->DataSet = dbDataset;
      letter_form->txtPrimary->DataField = "primary_name";
      letter_form->txtMember->DataSet = dbDataset;
      letter_form->txtMember->DataField = "first_last";

      label_form->labelReport->DataSet = dbDataset;
      label_form->txtName->DataSet = dbDataset;
      label_form->txtName->DataField = "first_last";
      label_form->txtAddress1->DataSet = dbDataset;
      label_form->txtAddress1->DataField = "line1";

      report_form->report->Preview();
      letter_form->warning_letter->Preview();
      label_form->labelReport->Preview();

      if(checkTasks->Checked)
      {
        Processing(true);
        sql = "insert into tasks (id, current_id, last_update_stamp, last_updated_by, person_id, group_id, "
              "date_of_task, method_id, direction_id, task_comments, diab_activity_id, user_id, fitness_membership_id) "
              "select (select gen_id(tasks_gen, 1) from uno), 4, '" + RightNow() + "', " + current_userid + ", "
              "fitness_people.person_id, " + lookupTables->IdOf("groups", "Diabetes") + ", '" +
              CurrentDate() + "', " + lookupTables->IdOf("method_types", "Mail") + ", " +
              lookupTables->IdOf("direction_types", "Outgoing") + ", 'FITNESS ATTENDANCE WARNING LETTER', " +
              lookupTables->IdOf("diab_activity_types", "Attendance warning letter") + ", " + current_userid + ", "
              "fitness_memberships.id "
              "from fitness_memberships "
              "inner join fitness_people on fitness_people.fitness_membership_id = fitness_memberships.id "
              "where fitness_memberships.id in (" + id_string + ") and "
              "fitness_people.is_primary = 1";
        if (! ExecuteStatement(dbTransaction, sql))
          throw 1;
        Processing(false);
      }

      dbDataset->Close();
      dbLetterDataset->Close();
      dbTransaction->Commit();
    }
    catch(...)
    {
      Processing(false);   
      if (dbTransaction->Active)
        dbTransaction->Rollback();
      PSay("Error printing letter", m_database_error, MB_OK);
    }
    delete dbDataset;
    delete dbLetterDataset;
    delete dbTransaction;
  }
  else
  {
    PSay("You have not selected any clients", m_missing_data, MB_OK);
  }
}
//---------------------------------------------------------------------------

void __fastcall TformSpecialDiabWarningLetter::btnCancelMembershipsClick(
      TObject *Sender)
{
  vector<string> id_vec = memberships_filter.GetMultiID();
  if(! id_vec.empty())
  {
    if(PSay("This will cancel the checked memberships. Are you sure you want to continue?", m_confirmation, MB_YESNO) == IDYES)
    {
      Processing(true);
      string id_string = VecToCommaString(id_vec);
      string sql;
      TIBTransaction *dbTransaction = new TIBTransaction(NULL);
      TIBDataSet *dbDataset = new TIBDataSet(NULL);

      try
      {
        TformReportDiabCancellationLetter *letter_form = new TformReportDiabCancellationLetter(NULL);
        TformReportMailingLabels *label_form = new TformReportMailingLabels(NULL);
        TformReportDiabCancellationLettersSent *report_form = new TformReportDiabCancellationLettersSent(NULL);

        dbTransaction->DefaultDatabase = dbDatabase;
        dbDataset->Database = dbDatabase;
        dbDataset->Transaction = dbTransaction;
        dbTransaction->StartTransaction();

        sql = "select person_names.first_name || ' ' || person_names.last_name first_last, "
              "person_names.last_name || ', ' || person_names.first_name last_first, "
              "person_contact_info.line1, person_contact_info.line2, "
              "person_contact_info.city || ', ' || state_types.abbreviation || '  ' || "
              "person_contact_info.zip city, fit_facility_types.item facility "
              "from fitness_memberships "
              "inner join fit_facility_types on fitness_memberships.fit_facility_id = fit_facility_types.id "
              "inner join fitness_people on fitness_people.fitness_membership_id = fitness_memberships.id "
              "inner join people on fitness_people.person_id = people.id "
              "left join person_names on people.person_names_id = person_names.id "
              "left join person_contact_info on people.person_contact_info_id = person_contact_info.id "
              "inner join state_types on person_contact_info.state_id = state_types.id "
              "where fitness_people.is_primary = 1 and "
              "fitness_memberships.id in (" + id_string + ") "
              "order by person_names.last_name, person_names.first_name";

        dbDataset->SelectSQL->Add(sql.c_str());
        dbDataset->Open();

        report_form->report->DataSet = dbDataset;
        report_form->lblDateTitle->Caption = CurrentDate().c_str();
        report_form->lblUser->Caption = current_fullname.c_str();
        report_form->txtName->DataSet = dbDataset;
        report_form->txtName->DataField = "last_first";
        report_form->txtFacility->DataSet = dbDataset;
        report_form->txtFacility->DataField = "facility";

        letter_form->cancel_letter->DataSet = dbDataset;
        letter_form->txtCancelPerson->DataSet = dbDataset;
        letter_form->txtCancelPerson->DataField = "first_last";
        letter_form->txtCancelLine1->DataSet = dbDataset;
        letter_form->txtCancelLine1->DataField = "line1";
        letter_form->txtCancelLine2->DataSet = dbDataset;
        letter_form->txtCancelLine2->DataField = "city";

        label_form->labelReport->DataSet = dbDataset;
        label_form->txtName->DataSet = dbDataset;
        label_form->txtName->DataField = "first_last";
        label_form->txtAddress1->DataSet = dbDataset;
        label_form->txtAddress1->DataField = "line1";

        if(checkCancellation->Checked)
        {
          sql = "update fitness_memberships set end_date = '" + CurrentDate() + "' "
                "where id in (" + id_string + ")";
          ExecuteStatement(dbTransaction, sql);
        }
        Processing(false);
        report_form->report->Preview();
        letter_form->cancel_letter->Preview();
        label_form->labelReport->Preview();
        Processing(true);
        TformReportMailingLabels *label_form2 = new TformReportMailingLabels(NULL);
        TformReportDiabMembershipsReady *waiting_report = new TformReportDiabMembershipsReady(NULL);
        TformReportDiabNewMembershipLetter *waiting_letter = new TformReportDiabNewMembershipLetter(NULL);
        sql = "select fitness_memberships.fit_facility_id facility, count(id) the_count from fitness_memberships "
              "where fitness_memberships.id in (" + id_string + ") "
              "group by fitness_memberships.fit_facility_id";
        map<string, string> facility_map;
        SqlToValueMap(dbTransaction, sql, facility_map);
        vector<string> membership_id_vec;
        ExecuteStatement(dbTransaction, "delete from temp_waiting_list");
        for(map<string, string>::iterator itr = facility_map.begin(); itr != facility_map.end(); itr++)
        {
          sql = "select first " + itr->second + " id from fitness_memberships where "
                "fitness_memberships.fit_facility_id = " + itr->first + " and "
                "fitness_memberships.is_waiting = 1 and fitness_memberships.current_id >= 3 "
                "order by fitness_memberships.waiting_list_date ascending";
          SqlToVec(dbTransaction, sql, membership_id_vec);
          for(vector<string>::iterator itr = membership_id_vec.begin(); itr != membership_id_vec.end(); itr++)
          {
            sql = "insert into temp_waiting_list (id) values (" + *itr + ")";;
            ExecuteStatement(dbTransaction, sql);
          }
        }
        sql = "select person_names.last_name || ', ' || person_names.first_name || ' ' || person_names.suffix last_first, "
              "person_names.first_name || ' ' || person_names.last_name first_last, "
              "person_contact_info.line1, person_contact_info.line2, "
              "person_contact_info.city || ', ' || state_types.abbreviation || '  ' || "
              "person_contact_info.zip city, person_contact_info.phone_number, "
              "fit_facility_types.item facility, fitness_memberships.waiting_list_date "
              "from fitness_memberships "
              "inner join fit_facility_types on fitness_memberships.fit_facility_id = fit_facility_types.id "
              "inner join fitness_people on fitness_people.fitness_membership_id = fitness_memberships.id "
              "inner join people on fitness_people.person_id = people.id "
              "left join person_names on people.person_names_id = person_names.id "
              "left join person_contact_info on people.person_contact_info_id = person_contact_info.id "
              "left join state_types on person_contact_info.state_id = state_types.id "
              "where fitness_people.is_primary = 1 and "
              "fitness_memberships.id in (select id from temp_waiting_list) "
              "order by fitness_memberships.waiting_list_date";
        dbDataset->Close();
        dbDataset->SelectSQL->Clear();
        dbDataset->SelectSQL->Add(sql.c_str());
        dbDataset->Open();

        waiting_report->waiting_list_report->DataSet = dbDataset;
        waiting_report->lblDateTitle->Caption = CurrentDate().c_str();
        waiting_report->lblUser->Caption = current_fullname.c_str();
        waiting_report->txtFacility->DataSet = dbDataset;
        waiting_report->txtFacility->DataField = "facility";
        waiting_report->txtName->DataSet = dbDataset;
        waiting_report->txtName->DataField = "last_first";
        waiting_report->txtDate->DataSet = dbDataset;
        waiting_report->txtDate->DataField = "waiting_list_date";
        waiting_report->txtPhone->DataSet = dbDataset;
        waiting_report->txtPhone->DataField = "phone_number";

        waiting_letter->new_letter->DataSet = dbDataset;
        waiting_letter->txtNewPerson->DataSet = dbDataset;
        waiting_letter->txtNewPerson->DataField = "first_last";
        waiting_letter->txtNewLine1->DataSet = dbDataset;
        waiting_letter->txtNewLine1->DataField = "line1";
        waiting_letter->txtNewLine2->DataSet = dbDataset;
        waiting_letter->txtNewLine2->DataField = "city";

        label_form2->labelReport->DataSet = dbDataset;
        label_form2->txtName->DataSet = dbDataset;
        label_form2->txtName->DataField = "first_last";
        label_form2->txtAddress1->DataSet = dbDataset;
        label_form2->txtAddress1->DataField = "line1";

        Processing(false);
        waiting_report->waiting_list_report->Preview();
        waiting_letter->new_letter->Preview();
        label_form2->labelReport->Preview();

        if(checkTasks->Checked)
        {
          Processing(true);
          sql = "insert into tasks (id, current_id, last_update_stamp, last_updated_by, person_id, group_id, "
                "date_of_task, method_id, direction_id, task_comments, diab_activity_id, user_id, fitness_membership_id) "
                "select (select gen_id(tasks_gen, 1) from uno), 4, '" + RightNow() + "', " + current_userid + ", "
                "fitness_people.person_id, " + lookupTables->IdOf("groups", "Diabetes") + ", '" +
                CurrentDate() + "', " + lookupTables->IdOf("method_types", "Mail") + ", " +
                lookupTables->IdOf("direction_types", "Outgoing") + ", 'FITNESS MEMBERSHIP CANCELLATION LETTER', " +
                lookupTables->IdOf("diab_activity_types", "Fitness cancellation letter") + ", " + current_userid + ", "
                "fitness_memberships.id "
                "from fitness_memberships "
                "inner join fitness_people on fitness_people.fitness_membership_id = fitness_memberships.id "
                "where fitness_memberships.id in (" + id_string + ") and "
                "fitness_people.is_primary = 1";
          if (! ExecuteStatement(dbTransaction, sql))
            throw 1;
            // The following is commented out because Diabetes doesn't
            // want to do fitness membership letters this way (for now)
            // I'm leaving it in place just in case they decide to change
            // back to this method. Would any of us be surprised?
/*          sql = "insert into tasks (id, current_id, last_update_stamp, last_updated_by, person_id, group_id, "
                "date_of_task, method_id, direction_id, task_comments, diab_activity_id, user_id, fitness_membership_id) "
                "select (select gen_id(tasks_gen, 1) from uno), 4, '" + RightNow() + "', " + current_userid + ", "
                "fitness_people.person_id, " + lookupTables->IdOf("groups", "Diabetes") + ", '" +
                CurrentDate() + "', " + lookupTables->IdOf("method_types", "Mail") + ", " +
                lookupTables->IdOf("direction_types", "Outgoing") + ", 'NEW FITNESS MEMBERSHIP AVAILABLE LETTER', " +
                lookupTables->IdOf("diab_activity_types", "New fitness membership letter") + ", " + current_userid + ", "
                "fitness_memberships.id "
                "from fitness_memberships "
                "inner join fitness_people on fitness_people.fitness_membership_id = fitness_memberships.id "
                "where fitness_memberships.id in (select id from temp_waiting_list) and "
                "fitness_people.is_primary = 1";
          if (! ExecuteStatement(dbTransaction, sql))
            throw 1; */
          Processing(false);
        }

        ExecuteStatement(dbTransaction, "delete from temp_waiting_list");

        dbDataset->Close();
        dbTransaction->Commit();
      }
      catch(...)
      {
        Processing(false);
        if (dbTransaction->Active)
          dbTransaction->Rollback();
        PSay("Error printing letter", m_database_error, MB_OK);
      }
      delete dbDataset;
      delete dbTransaction;
    }
  }
  else
    PSay("You have not selected any clients", m_missing_data, MB_OK);
}
//---------------------------------------------------------------------------

void __fastcall TformSpecialDiabWarningLetter::btnCloseClick(
      TObject *Sender)
{
  Close();        
}
//---------------------------------------------------------------------------

