//---------------------------------------------------------------------------

#include <vcl.h>
#pragma hdrstop

#include "codeSpecialDisambiguate.h"
// Couldn't think of a better name.  This form allows the user to manually
// disambiguate their intended action, when their permissions allow a given
// action they performed to have more than one possible result
// e.g.:  They click the Create task/visit link on appointments, but they
// are able to create Little Axe Clinic visits as well as Diabetes visits.
// Which one do they actually intend to create?
//---------------------------------------------------------------------------
#pragma package(smart_init)
#pragma link "codeSpecialGeneric"
#pragma resource "*.dfm"
TformSpecialDisambiguate *formSpecialDisambiguate;
//---------------------------------------------------------------------------
__fastcall TformSpecialDisambiguate::TformSpecialDisambiguate(TComponent* Owner, string p_instructions, string table, string sql, set<string> ids, string *result, TransformString how_to_format)
        : TformSpecialGeneric(Owner)
// input:  instructions - the instructions to display to the user
// input:  table - it occurs to me that this function doesn't need or use this parameter.  It's needed by the statid function below, to look up permissions for the current user
// input:  sql - the SQL that contains the columns
//         (1) "row_id" - the unique identifier that will be used to indicate which item they picked in the combo box
//         (2) "row_value" - this was used by the static function below to look up permissions.  It will also be the value
//             displayed to the user in the combo box, unless you supply
//         (3) "show_value" - optional alternate value to show the user in the combo box.  e.g. "Little Axe Clinic" rather than "lac"
// input:  ids - the "row_id" rows to actually display, post permission-filtering
// input:  result* - pointer to the string where you want the result stored.  This will store the "row_id" of the item the user picks in the combo box.
// input:  how_to_format - string transformation function to use (PrettyString, etc.).  Default is RawString.
{
  if (how_to_format == NULL) how_to_format = RawString;
  ok_to_close = false;
  // unnecessary to run the SQL again ... but whaddayagonnado?
  // get the list of values corresponding to the IDs we've determined they have permission to pick from
  vector<map<string, string> > table_vecmap;
  SqlToVecMap(dbDatabase, sql, table_vecmap);        // we expect a "row_id" and an "row_value" column, which can (probably will) be column aliases
  for(unsigned int i=0; i<table_vecmap.size(); i++)  // these names were picked as they don't conflict with any existing column names that I know of
    if (ids.find(table_vecmap[i]["row_id"]) != ids.end())
      // if a show_value column (alias) exists, then we want to show the user something other than the permission string;
      // for instance, in groups, short_item will be "lac", which is used in permission names; however, we want to show
      // the user "Little Axe Clinic" in the dropdown.  So we just select both columns and alias them appropriately.
      // (and if show_value doesn't exist, we fall back to showing row_value)
      if (table_vecmap[i].find("show_value") != table_vecmap[i].end())
        value_to_id[how_to_format(table_vecmap[i]["show_value"])] = table_vecmap[i]["row_id"];
      else
        value_to_id[how_to_format(table_vecmap[i]["row_value"])] = table_vecmap[i]["row_id"];
  // the map will necessarily be sorted by value
  for(map<string, string>::iterator itr = value_to_id.begin(); itr != value_to_id.end(); itr++)
  {
    //cboItems->Items->Add(itr->first.c_str());
    TListItem* item = listItems->Items->Add();
    item->Caption = itr->first.c_str();
  }

  instructions = p_instructions;

  selected_id = result;
}
//---------------------------------------------------------------------------
void __fastcall TformSpecialDisambiguate::FormShow(TObject *Sender)
{
  InfoBarText = instructions;
  ShowInfoBar();
}
//---------------------------------------------------------------------------
void __fastcall TformSpecialDisambiguate::FormCloseQuery(TObject *Sender, bool &CanClose)
{
  CanClose = ok_to_close;
}
//---------------------------------------------------------------------------
void __fastcall TformSpecialDisambiguate::listItemsSelectItem(TObject *Sender, TListItem *Item, bool Selected)
{
  btnOk->Enabled = listItems->Selected;
}
//---------------------------------------------------------------------------
void __fastcall TformSpecialDisambiguate::btnOkClick(TObject *Sender)
{
  *selected_id = value_to_id[listItems->Selected->Caption.c_str()];
  ok_to_close = true;
  Close();
}
//---------------------------------------------------------------------------
string TformSpecialDisambiguate::Disambiguate(TObject *Owner, string perm_prefix, string perm_suffix, string perm_key, string perm_value, string instructions, TransformString how_to_format, set<string> &to_include, set<string> &to_exclude, string table, string sql)
{
  string result;
  // We need to check and see if they have permission to do multiple things,
  // and if so, let them pick the thing they'd like to do.
  set<string> ids = GetPermissions(perm_prefix, perm_suffix, perm_key, perm_value, table, sql);
  //::PPretendToSay("ids: " + SetToDelimString(ids, "{", "}", ""), m_message, " static codeSpecialDisambiguate::Disambiguate");
  //::PPretendToSay("exclude: " + SetToDelimString(to_exclude, "{", "}", ""), m_message, "static codeSpecialDisambiguate::Disambiguate");
  set<string> temp;
  // exclude the things we were requested to exclude (to_exclude); user doesn't get to pick them even if they have permission
  temp.clear();
  temp = subtract(ids, to_exclude);
  ids = temp; // according to the help files, we can't do all this in one go... set_difference doesn't seem to like x = x - y; syntax
  //::PPretendToSay("ids: " + SetToDelimString(ids, "{", "}", ""), m_message, "static codeSpecialDisambiguate::Disambiguate");

  //::PPretendToSay("include: " + SetToDelimString(to_include, "{", "}", ""), m_message, "static codeSpecialDisambiguate::Disambiguate");
  // if we were given an explicit set of allowed values (to_include), restrict it further, to just those
  if (to_include.size() > 0)
  {
    temp.clear();
    temp = intersect(ids, to_include);
    ids = temp; // according to the help files, we can't do all this in one go... set_difference doesn't seem to like x = x - y; syntax
  }
  //::PPretendToSay("ids: " + SetToDelimString(ids, "{", "}", ""), m_message, "static codeSpecialDisambiguate::Disambiguate");

  if (ids.size() > 1) // let them pick a permission-action
  {
    FORM_TRY
     TformSpecialDisambiguate *d = new TformSpecialDisambiguate(NULL, instructions, table, sql, ids, &result, how_to_format);
     TformSpecialGeneric * parent = NULL;
     if (Owner->InheritsFrom(__classid(TformSpecialGeneric)))
       parent = (TformSpecialGeneric*)Owner;
     d->ShowTaskModal(parent);
    FORM_CATCH
  }
  else if (ids.size() == 1)
    result = SetToVec(ids)[0]; // kind of cheap; I needed the value of the only member of the set...
  else
    result = ""; // no permission!

  return result;
}
//---------------------------------------------------------------------------
string TformSpecialDisambiguate::Disambiguate(TObject *Owner, map<string, string> &params, TransformString how_to_format)
{
  set<string> to_include, to_exclude;
  // include/exclude are usually going to be IDs, so numeric, but we'll just strip the minimum to be safe.
  // NOTE:  For obvious reasons, you don't want to use this function if you're identifying rows with a field that may have commas in it.
  //        Should that be a problem for you, call the overloaded version above, with sets instead.
  params["include"] = ReplaceStringWithString(params["include"], ", ", ","); // convert comma space to just comma
  SplitStringToSet(params["include"], ",", to_include);
  params["exclude"] = ReplaceStringWithString(params["exclude"], ", ", ","); // convert comma space to just comma
  SplitStringToSet(params["exclude"], ",", to_exclude);
  return Disambiguate(Owner, params["prefix"], params["suffix"], params["key"], params["value"], params["instructions"], how_to_format, to_include, to_exclude, params["table"], params["sql"]);
}
//---------------------------------------------------------------------------

