//---------------------------------------------------------------------------

#include <vcl.h>
#pragma hdrstop

#include "codeSpecialDischargeBeh.h"
#include "codeRecordTreatmentBeh.h"
#include "reportBehDischarge.h"
//---------------------------------------------------------------------------
#pragma package(smart_init)
#pragma link "codeSpecialGeneric"
#pragma resource "*.dfm"
TformSpecialDischargeBeh *formSpecialDischargeBeh;
//---------------------------------------------------------------------------
__fastcall TformSpecialDischargeBeh::TformSpecialDischargeBeh(TComponent* Owner)
        : TformSpecialGeneric(Owner)
{
  this->IconIndex = 17;
  person_filter.SetListViewHandle(listClients);
  person_filter.SetSecurity("beh_treatment");
  person_filter.AddColumnSpec("Name", "people", as_text, true, 150);
  person_filter.AddColumnSpec("Birthday", "people", as_date, true, 90);
  person_filter.AddColumnSpec("CDIB", "affiliations", as_text, true, 70);
  person_filter.AddColumnSpec("Tribe", "affiliations", as_text, true, 160);
  person_filter.AddColumnSpec("Admit Date", "tasks", as_date, true, 90);
  person_filter.AddColumnSpec("Last Contact", "tasks", as_date, true, 90);
  person_filter.AddColumnSpec("Current Id", "current_id", as_int, false, 80);
  person_filter.AddColumnSpec("Internal Id", "internal_id", as_int, false, 80);
  person_filter.FetchPreferences("specialdischargebeh.person_filter");
  //person_filter.SetCompress(true);
  person_filter.SetCheck(true);
  person_filter.SetSQLStatement("");
  person_filter.RefreshDataSet();

  DisplayClients();
}
//---------------------------------------------------------------------------
void TformSpecialDischargeBeh::DisplayClients()
{
  string sql;
  string three_months;
  Pdate look_up_date(Date());
  look_up_date.mod(-3, in_months);

  sql = "select person_names.last_name || ', ' || person_names.first_name || ' ' || person_names.suffix full_name, "
        "people.birth_date, affiliations.cdib, tribe_types.item, "
        "beh_treatment.admit_date, max(tasks.date_of_task), beh_treatment.current_id, beh_treatment.id "
        "from people "
        "inner join beh_treatment on beh_treatment.person_id = people.id "
        "left join person_names on people.person_names_id = person_names.id "
        "left join affiliations on people.affiliations_id = affiliations.id "
        "left join tribe_types on affiliations.tribe_id = tribe_types.id "
        "left join tasks on tasks.person_id = people.id and "
        "tasks.group_id = " + lookupTables->IdOf("groups", "Behavioral Health") + " "
        "where beh_treatment.admit_date <= '" + look_up_date.OutDate(mdy | yyyy) + "' and "
        "beh_treatment.discharge_date is null and "
        "beh_treatment.id > 0 "
        "group by beh_treatment.id, person_names.last_name, "
        "person_names.first_name, person_names.suffix, beh_treatment.admit_date, beh_treatment.current_id, "
        "affiliations.cdib, tribe_types.item, people.birth_date "
        "having max(tasks.date_of_task) is null or max(tasks.date_of_task) <= '" + look_up_date.OutDate(mdy | yyyy) + "' "
        "order by person_names.last_name, person_names.first_name";

  person_filter.SetSQLStatement(sql);
  person_filter.RefreshDataSet();
  person_filter.SetAddEdit(this, NULL, EditRecord(&(TformSpecialDischargeBeh::EditTreatment)));
}
//---------------------------------------------------------------------------

void __fastcall TformSpecialDischargeBeh::btnCloseClick(TObject *Sender)
{
  Close();
}
//---------------------------------------------------------------------------
void __fastcall TformSpecialDischargeBeh::btnSaveClick(TObject *Sender)
{
  Processing(true);

  TIBTransaction *dbTransaction = new TIBTransaction(NULL);
  TIBDataSet *dbDataset = new TIBDataSet(NULL);
  vector<string> id_vec = person_filter.GetMultiID();
  map<string, string> update_map;
  // set up some strings so these functions are only called once instead
  // of every time in the loop
  string comment;
  string username = lookupTables->ValueOf("users", current_userid);
  string discharge_id = lookupTables->IdOf("beh_discharge_types", "Inactive/No Contact In 90 Days");
  string id_string;
  string sql;
  string title;
  string now = RightNow();
  string today = DateToStr(Date()).c_str();
  map<string, string> comment_map;
  map<string, string> discharge_id_map;
  list<string> person_id_list;
  try
  {
    if(! id_vec.empty())
    {
      StdTransaction(dbTransaction, dbDatabase);
      dbDataset->Database = dbDatabase;
      dbDataset->Transaction = dbTransaction;
      dbDataset->UniDirectional = false;

      id_string = "(" + VecToCommaString(id_vec) + ")";
      sql = "select id, comments from beh_treatment where id in " + id_string;
      SqlToValueMap(dbTransaction, sql, comment_map);
      sql = "select id, beh_discharge_id from beh_treatment where id in " + id_string;
      SqlToValueMap(dbTransaction, sql, discharge_id_map);
      sql = "select person_id from beh_treatment where id in " + id_string;
      SqlToList(dbTransaction, sql, person_id_list);

      for(vector<string>::iterator itr = id_vec.begin(); itr != id_vec.end(); itr++)
      {
        if(comment_map[*itr] != "")
          comment = comment_map[*itr] + "\n";
        else
          comment = "";
        comment += "Discharged by " + username + ".";

        update_map.clear();
        update_map["id"] = *itr;
        update_map["last_update_stamp"] = now;
        update_map["last_updated_by"] = current_userid;
        update_map["discharge_date"] = today;
        update_map["beh_discharge_id"] = discharge_id;
        update_map["comments"] = comment;
        if (! ExecuteUpdateStatement(dbTransaction, "beh_treatment", update_map))
          throw 0;

        update_map.clear();
        update_map["id"] = "1"; // has a trigger so it doesn't matter
        update_map["table_name"] = "beh_treatment";
        update_map["field_name"] = "discharge_date";
        update_map["previous_value"] = "";
        update_map["present_value"] = today;
        update_map["change_stamp"] = now;
        update_map["change_by"] = current_userid;
        update_map["record_id"] = *itr;
        if (! ExecuteInsertStatement(dbTransaction, "change_log", update_map))
          throw 0;

        update_map["field_name"] = "comments";
        update_map["previous_value"] = comment_map[*itr];
        update_map["present_value"] = comment;
        if (! ExecuteInsertStatement(dbTransaction, "change_log", update_map))
          throw 0;

        if(discharge_id_map[*itr] != discharge_id)
        {
          update_map["field_name"] = "beh_treatment_id";
          update_map["previous_value"] = discharge_id_map[*itr];
          update_map["present_value"] = discharge_id;
          if (! ExecuteInsertStatement(dbTransaction, "change_log", update_map))
            throw 0;
        }
        atlas->Notify("beh_treatment.id", *itr);
        for(list<string>::iterator person_itr = person_id_list.begin(); person_itr != person_id_list.end(); person_itr++)
          atlas->Notify("beh_treatment.person_id", *person_itr);

      }
      dbTransaction->Commit();

      sql = "select person_names.last_name || ', ' || person_names.first_name || ' ' || person_names.suffix full_name, "
            "affiliations.cdib, tribe_types.item tribe, people.birth_date, "
            "beh_treatment.admit_date, max(tasks.date_of_task) last_task "
            "from people "
            "inner join beh_treatment on beh_treatment.person_id = people.id "
            "left join person_names on people.person_names_id = person_names.id "
            "left join affiliations on people.affiliations_id = affiliations.id "
            "left join tribe_types on affiliations.tribe_id = tribe_types.id "
            "left join tasks on tasks.person_id = people.id and "
            "tasks.group_id = " + lookupTables->IdOf("groups", "Behavioral Health") + " "
            "where beh_treatment.id in " + id_string + " group by person_names.last_name, "
            "person_names.first_name, person_names.suffix, beh_treatment.admit_date, "
            "affiliations.cdib, tribe_types.item, people.birth_date";
      title = "Clients Discharged from AST Behavioral Health Program - " + today;

      dbTransaction->StartTransaction();
      dbDataset->SelectSQL->Add(sql.c_str());
      dbDataset->Open();
      dbDataset->First();

      FORM_TRY
        TformReportBehDischarge * report = new TformReportBehDischarge(NULL);
        report->report->DataSet = dbDataset;
        report->lblTitle->Caption = title.c_str();
        report->lblUser->Caption = PrettyString(lookupTables->ValueOf("users", current_userid)).c_str();
        report->txtName->DataSet = dbDataset;
        report->txtName->DataField = "full_name";
        report->txtCdib->DataSet = dbDataset;
        report->txtCdib->DataField = "cdib";
        report->txtTribe->DataSet = dbDataset;
        report->txtTribe->DataField = "tribe";
        report->txtDob->DataSet = dbDataset;
        report->txtDob->DataField = "birth_date";
        report->txtAdmitDate->DataSet = dbDataset;
        report->txtAdmitDate->DataField = "admit_date";
        report->txtLastContact->DataSet = dbDataset;
        report->txtLastContact->DataField = "last_task";

        Processing(false);
        report->report->Preview();
      FORM_CATCH


      dbDataset->Close();
      dbTransaction->Commit();

    } // end if vector not empty
  }
  catch(...)
  {
    Processing(false);  
    if (dbTransaction->Active)
      dbTransaction->Rollback();
    PSay("Could not perform update", m_database_error, MB_OK);
  }
  delete dbDataset;
  delete dbTransaction;
  Close();

}
//---------------------------------------------------------------------------
void TformSpecialDischargeBeh::EditTreatment()
{
  map<string,string> temp_map;
  temp_map["id"] = person_filter.GetSelectedRecordID();
  FORM_TRY
    TformRecordTreatmentBeh * treatmentRecord = new TformRecordTreatmentBeh(NULL, temp_map);
    treatmentRecord->Show();
  FORM_CATCH
}
//---------------------------------------------------------------------------
void __fastcall TformSpecialDischargeBeh::btnCheckAllClick(TObject *Sender)
{
  person_filter.CheckAll();
}
//---------------------------------------------------------------------------
void __fastcall TformSpecialDischargeBeh::btnUncheckAllClick(TObject *Sender)
{
  person_filter.UncheckAll();        
}
//---------------------------------------------------------------------------

