//---------------------------------------------------------------------------

#include <vcl.h>
#pragma hdrstop

#include "codeSpecialDropTempViews.h"
//---------------------------------------------------------------------------
#pragma package(smart_init)
#pragma link "codeSpecialGeneric"
#pragma resource "*.dfm"
TformSpecialDropTempViews *formSpecialDropTempViews;
//---------------------------------------------------------------------------
__fastcall TformSpecialDropTempViews::TformSpecialDropTempViews(TComponent* Owner)
        : TformSpecialGeneric(Owner)
{
 string sql_find_temp_views = "select rdb$relation_name, rdb$relation_name id from rdb$relations where rdb$relation_name like 'TEMP_VIEW_%' order by rdb$relation_name";

 view_filter.SetListViewHandle(listViews);
 view_filter.SetSecurity("");
 view_filter.AddColumnSpec("Name", "", as_text, true, 320, rawString);
 view_filter.AddColumnSpec("Internal Id", "internal_id", as_text, false, 160, rawString); // admittedly, unorthodox ... as it's sort of a text field, eh?
 view_filter.FetchPreferences("specialdroptempviews.views_filter");
 view_filter.SetIconStatus(true);
 view_filter.SetCheck(true);
 view_filter.SetSQLStatement(sql_find_temp_views);
 view_filter.RefreshDataSet();
}
//---------------------------------------------------------------------------
void __fastcall TformSpecialDropTempViews::buttonDropClick(TObject *Sender)
{
 //
 // rdb$view_relations has the dependency graph!
 // rdb$view_name 'depends on' rdb$relation_name

 vector<string> to_drop = view_filter.GetMultiID();
 if (to_drop.empty())
  return;

 // check for things -not- being dropped that do -depend- on things being dropped (thus, errors)
 string sql_find_errors = "select distinct rdb$view_name from rdb$view_relations where "
                          "rdb$relation_name in (" + VecToDelimString(to_drop, "'", "'", ", ") + ") and "
                          "rdb$view_name like 'TEMP_VIEW_%' and "
                          "rdb$view_name not in (" + VecToDelimString(to_drop, "'", "'", ", ") + ")";
 vector<string> would_fail;
 if (!SqlToVec(dbDatabase, sql_find_errors, would_fail))
 {
  PSay("Failed to find dependent views that would prevent this from working, aborting.", m_database_error, MB_OK, sql_find_errors);
  return;
 }

 if (!would_fail.empty())
 {
  PSay("The following views (not being dropped) depend on ones you've requested dropped:\n" + VecToDelimString(would_fail, "", "", ",\n"), m_invalid_data, MB_OK);
  return;
 }

 // get list of things to drop, put them into the dependency graph so we can sort them usefully
 string sql_find_dependencies = "select rdb$view_name child, rdb$relation_name parent from rdb$view_relations where "
                                "rdb$view_name in (" + VecToDelimString(to_drop, "'", "'", ", ") + ")";
 vector<map<string, string> > view_relations;
 if (!SqlToVecMap(dbDatabase, sql_find_dependencies, view_relations))
 {
  PSay("Failed to build the dependency graph between view names, aborting.", m_database_error, MB_OK, sql_find_dependencies);
  return;
 }

 map<string, set<string> > dependencies;
 map<string, set<string> >::iterator k;
 set<string> empty_set;

 // pre-create all the possible items
 for (vector<string>::iterator j = to_drop.begin(); j != to_drop.end(); j++)
  dependencies[*j] = empty_set;

 // go back and add in the actual dependencies
 for (vector<map<string, string> >::iterator i = view_relations.begin(); i != view_relations.end(); i++)
 {
  string c = (*i)["child"];
  string p = (*i)["parent"];
  k = dependencies.find(p);
  if (k != dependencies.end())
  {
   k = dependencies.find(c);
   k->second.insert(p);
  } else
   ; // ignore this relationship, as the parent isn't being deleted. if we added it, the graph would fail -- impossible to resolve?
 }

 if (IDNO == PSay("Everything's set to go -- are you really sure?", m_confirmation, MB_YESNO))
  return;

 // and the figure it all out and start from the correct end
 // the top has the last-to-delete stuff, so we start from the bottom
 vector<set<string> > reverse_deletion_order;
 bool result = dependency_graph (dependencies, reverse_deletion_order, true);
 while (!reverse_deletion_order.empty())
 {
  set<string> to_delete = reverse_deletion_order.back();
  for (set<string>::iterator view_name = to_delete.begin(); view_name != to_delete.end(); view_name++)
   if (!ExecuteStatement(dbDatabase, "drop view " + *view_name))
   {
    PSay("Failed to drop view " + *view_name + "\nThis is odd, shouldn't happen, aborting.\nThis is a rare bug -- please figure out why the dependency graph didn't see this one coming.", m_database_error, MB_OK, *view_name);
    view_filter.RefreshDataSet();
    return;
   }
  reverse_deletion_order.pop_back();
 }
 view_filter.RefreshDataSet();
}
//---------------------------------------------------------------------------
void __fastcall TformSpecialDropTempViews::buttonCloseClick(TObject *Sender)
{
 Close();
}
//---------------------------------------------------------------------------
void __fastcall TformSpecialDropTempViews::buttonClearClick(
      TObject *Sender)
{
 if (!ExecuteStatement(dbDatabase, "delete from temp_id"))
  PSay("Failed to delete the temp table's contents -- darn?", m_database_error, MB_OK);
}
//---------------------------------------------------------------------------

