//---------------------------------------------------------------------------

#include <vcl.h>
#pragma hdrstop

#include "codeSpecialExportHL7ForIHS.h"
#include "codeSpecialHL7Settings.h"
#include "hl7_db.h"
#include "derivedheaders.h"
//---------------------------------------------------------------------------
#pragma package(smart_init)
#pragma link "codeSpecialGeneric"
#pragma link "codeRubberContainer"
#pragma resource "*.dfm"
TformSpecialExportHL7ForIHS *formSpecialExportHL7ForIHS;

//---------------------------------------------------------------------------
__fastcall TformSpecialExportHL7ForIHS::TformSpecialExportHL7ForIHS(TComponent* Owner)
        : TformSpecialGeneric(Owner)
{
 radioFileOptionsClick(NULL);

 floating_log_window = NULL;
 editLog = NULL;

 FORM_TRY
  floating_log_window = new TForm((TComponent*)NULL);
 FORM_CATCH
 FORM_TRY
  editLog = new TRichEdit(floating_log_window);
 FORM_CATCH
 floating_log_window->BorderStyle = bsSizeToolWin;
 floating_log_window->AutoSize = false;
 floating_log_window->Width = Screen->Width;
 floating_log_window->Height = 350;
 floating_log_window->Left = 0;
 floating_log_window->Top = Screen->Height - floating_log_window->Height;
 floating_log_window->Caption = "Debugging log";
 editLog->Parent = floating_log_window;
 editLog->HideSelection = false;
 editLog->ReadOnly = false;
 editLog->ScrollBars = ssVertical;
 editLog->Align = alClient;
 floating_log_window->Visible = true;
}
//---------------------------------------------------------------------------
void __fastcall TformSpecialExportHL7ForIHS::buttonRunClick(TObject *Sender)
{
 if (floating_log_window && editLog)
 {
  editLog->Lines->Clear();
  floating_log_window->BringToFront();
 }
 last_message_stamp = CurrentPstamp();

 Log("Initializing...", true);
 bool answer = false;
 bool show_about_to = false;
 unsigned int progress_counter = 0;
 unsigned int progress_interval = 50 /* 1 */;

 // TODO: see list below
 // our ASUFAC codes are
 // 555813 for the "Little Axe Health Center" (Little Axe Clinic)
 // 555816 for the "Absentee Shawnee Health Center" (Shawnee Clinic)
 //
 // sending_facility_id (AKA DBID) is 17133 (it's always this number; we only have one database)

 // at least one row has a date in the tooth_number field and the tooth_number in the tooth_surface instead
 // a lot of rows have non-NULL tooth_number, but no value (blank, '')
 // a lot of rows have a comma-separated list of tooth_number's instead of separate rows. this is bad.

 // IGNORE: fix POS, non-numerics -- not present in export
 //  select count(*), place_of_service from procedures group by place_of_service;
 // IGNORE: fix TOS, non-numerics -- not present in export
 //  select count(*), type_of_service from procedures group by type_of_service;

 // when updating, consider the use of a trigger on the temp tables (extra "last_export" column)
 // such that updates to the temp table update the base table? (as would be done with views, in theory)
 //
 // END TODO

 // step 1 -- clear temp tables (visits, people, pharmacy)
 // step 2 -- find visits during date period
 //         + visits that need to be resent
 //         - visits that need not be sent at all
 //           [includes pharmacy]
 // step 3 -- match those up with people
 //         + people that need to be resent
 //         - people that need not be sent at all
 // step 4 -- ouput people
 //         & mark as sent as we go (speed?)
 //         , stop if we hit limit
 // step 5 -- output visits
 //         & mark as sent as we go (speed?)
 //         , stop if we hit limit
 //           [includes pharmacy]
 // step 6 -- clean up temp tables

 bool combined = radioCombined->Checked;
 bool npirs    = radioNPIRS->Checked;
 bool pharm    = checkPharmacy->Checked;
 bool notag    = checkNoExportTags->Checked;
 bool use_people_limit = checkLimitPeople->Checked;
 bool use_visits_limit = checkLimitVisits->Checked;
 unsigned int people_limit = 0;
 unsigned int visits_limit = 0;
 bool use_start_date = (editVisitStart->Text != "");
 bool use_stop_date  = (editVisitStop->Text != "");
 Pdate start_date;
 Pdate stop_date;
 Pstamp export_stamp(CurrentPstamp());
 unsigned int people_count = 0;
 unsigned int visits_count = 0;
 unsigned int pharmacy_visits_count = 0;
   hl7_run_type run_as = npirs ? hl7_run_full : hl7_run_incremental;

 if (use_start_date)
 {
  if (IsInvalidDate(editVisitStart))
  {
   PSay("Please enter a valid beginning date.", m_invalid_data, MB_OK);
   return;
  }
  start_date = Pdate(editVisitStart->Text.c_str(), global_mode);
 }
 if (use_stop_date)
 {
  if (IsInvalidDate(editVisitStop))
  {
   PSay("Please enter a valid end date.", m_invalid_data, MB_OK);
   return;
  }
  stop_date = Pdate(editVisitStop->Text.c_str(), global_mode);
 }
 if (use_people_limit)
 {
  if (!IsInt(editLimitPeople->Text.c_str()))
  {
   PSay("Please enter a valid maximum number of people to export.", m_invalid_data, MB_OK);
   return;
  }
  people_limit = StringToInt(editLimitPeople->Text.c_str());
 }
 if (use_visits_limit)
 {
  if (!IsInt(editLimitVisits->Text.c_str()))
  {
   PSay("Please enter a valid maximum number of visits to export.", m_invalid_data, MB_OK);
   return;
  }
  visits_limit = StringToInt(editLimitVisits->Text.c_str());
 }

 // get the files open ahead of time in case that's going to fail for some reason.
 fstream combined_file;
 fstream people_file;
 fstream visits_file;
 fstream   * export_file  = NULL;
 hl7_batch * export_batch = NULL;

 string log_filename = "AST" + people_batch.sending_facility + RightNow(true) + ".log";
 log_file.open(log_filename.c_str(), ios::out | ios::trunc);
 if (!log_file.is_open())
  PSay("Log file could not be opened for writing.", m_program_error, MB_OK, log_filename);
// else
//  log_file.close(); // ???
  
 if (combined)
 {
  combined_file.open(editCombinedFile->Text.c_str(), ios::out | ios::trunc);
  if (!combined_file.is_open())
  {
   PSay("Combined export file could not be opened for writing.\nPlease check the path and file name and try again.", m_program_error, MB_OK, editCombinedFile->Text.c_str());
   return;
  }
 } else
 {
  people_file.open(editPeopleFile->Text.c_str(), ios::out | ios::trunc);
  visits_file.open(editVisitsFile->Text.c_str(), ios::out | ios::trunc);
  if (!people_file.is_open() || !visits_file.is_open())
  {
   if (!people_file.is_open())
    PSay("People export file could not be opened for writing.\nPlease check the path and file name and try again.", m_program_error, MB_OK, editPeopleFile->Text.c_str());
   else
    people_file.close();
   if (!visits_file.is_open())
    PSay("Visits export file could not be opened for writing.\nPlease check the path and file name and try again.", m_program_error, MB_OK, editVisitsFile->Text.c_str());
   else
    visits_file.close();
   return;
  }
 }

 TIBTransaction * dbTransaction = NULL;
 try
 {
  while (!dbTransaction)
   dbTransaction = new TIBTransaction(NULL);
  StdTransaction(dbTransaction, dbDatabase);

  // initial export file and settings
  if (combined)
  {
   export_file  = &combined_file;
   export_batch = &combined_batch;
  } else
  {
   export_file  = &people_file;
   export_batch = &people_batch;
  }

  // DEBUG: for debugging, turn off output buffer
  export_file->rdbuf()->pubsetbuf(0, 0);
  log_file.rdbuf()->pubsetbuf(0, 0);

  // once more into the breach, my friends, once more into the breach. in peace, ...
  Processing(true);

  // record this batch in hl7_exports
  string hl7_exports_id = GetNextId(dbTransaction, "hl7_exports");
  map<string, string> temp;
  temp.clear();
  temp["id"] = hl7_exports_id;
  temp["begin_date"] = start_date.OutDate(global_mode);
  temp["end_date"] = stop_date.OutDate(global_mode);
  temp["hl7_run_id"] = lookupTables->IdOf("hl7_run_types", (npirs ? "Full" : "Incremental"));
  temp["batch_number"] = export_batch->batch_number;
  temp["was_sent"] = "0"; /* initial value; IF THIS IS A FINAL RUN of an IHS/NDW export,
                             i.e. IHS/NDW finds no problems with it, manually update
                             hl7_exports and set was_sent to 1!*/
  temp["run_date"] = export_stamp.OutStamp(mdy|yyyy|military|withsec|datefirst);
  Log("Inserting into hl7_exports:  " + MapToDelimString(temp, "{","}","","'","'","'","'",","));
  if (!ExecuteInsertStatement(dbTransaction, "hl7_exports", temp))
    Log("ERROR:  hl7_exports insert failed!");

  // BHS
  // ZHS
  *export_file << hl7_BHS(*export_batch, export_stamp).GetSegmentValue(*export_batch);
  *export_file << hl7_ZHS(start_date, stop_date, hl7_exports_id /* export id */).GetSegmentValue(*export_batch);

  // step 1
  // clear temp tables
  if (!ExecuteStatement(dbTransaction, "delete from temp_hl7_export_people_union"))
   throw(string("Could not clear temporary people (non-distinct) table"));
  if (!ExecuteStatement(dbTransaction, "delete from temp_hl7_export_people"))
   throw(string("Could not clear temporary people table"));
  if (!ExecuteStatement(dbTransaction, "delete from temp_hl7_export_tasks"))
   throw(string("Could not clear temporary visits (tasks) table"));
  if (!ExecuteStatement(dbTransaction, "delete from temp_hl7_export_pharm"))
   throw(string("Could not clear temporary visits (pharmacy) table"));

  // include them if (1) not included in previous export (2) changed since last export
  string ndw_date_string = "exports.id is null or "
                           "(exports.end_date is not null and last_update_stamp > exports.end_date) or "
                           "(exports.end_date is null and exports.begin_date is not null and last_update_stamp > exports.run_date)";
  // DONE: the following is just wrong. don't ask me why I wrote it this way. there should be an OR in there again.
  // TODO: current_id should be checked. not sure about doing that on visits. could just do it at runtime.
  // step 2
  string final_concat1;
  vector<string> date_params1, combined_params1;
  // dot_params will contain date criteria
  // combined_params will contain either/both/none of ((ndw criteria) OR (date criteria))
  // using VecToDelimString in this way will eliminate extraneous if statements
  if (!npirs)
  {
   string ndw_full_string = ndw_date_string;
   if (use_stop_date)
     ndw_full_string = "tasks.date_of_task <= '" + stop_date.OutDate(global_mode) + "' and (" + ndw_date_string + ")";
   combined_params1.push_back(ndw_full_string);
  }
  else // added because I don't see how this should apply to NDW data; NDW date string above should cover all possibilities;
  {
    if (use_start_date)
      date_params1.push_back("tasks.date_of_task >= '" + start_date.OutDate(global_mode) + "'");
    if (use_stop_date)
      date_params1.push_back("tasks.date_of_task <= '" + stop_date.OutDate(global_mode) + "'");
  }
  if (!date_params1.empty())
    combined_params1.push_back("(" + VecToDelimString(date_params1, "", "", " and ") + ")");
  else // my logic is this:  if we're in npirs mode, but NO DATE RANGE is specified, then we shouldn't
       // restrict to JUST records changed since last export, should we?  No date range means include
       // them all.  So saying ((task updated since export) OR (true)) will return them all, see?
   if (!combined_params1.empty() && npirs) combined_params1.push_back("true");
  // put it together, add it to the SQL
  final_concat1 = VecToDelimString(combined_params1, "(", ")", " or ");
  string extra_params1;
  if (final_concat1 != "")
   extra_params1 = " and (" + final_concat1 + ") ";
  else
   extra_params1 = "";

  string temp_sql;
  temp_sql =  "insert into temp_hl7_export_tasks "
              "select tasks.id, tasks.person_id "
              "from tasks ";
  if (!npirs)
   temp_sql += "left join hl7_visits on hl7_visits.id = " // we're going to consider the most-recently-performed export (by ID) that WAS_SENT to be the one we should compare to
               "(select first 1 i_visits.id from hl7_visits i_visits "
                "inner join hl7_exports i_exports on i_visits.hl7_export_id = i_exports.id "
                "where i_exports.was_sent = 1 and i_visits.task_id = tasks.id "
                "order by i_exports.end_date desc, i_exports.id desc) "
               "left join hl7_exports exports on exports.id = hl7_visits.hl7_export_id ";
  temp_sql += "where person_id != 0 and cob_number is null " + extra_params1;
  Log(temp_sql);
  DebugSql(this, &temp_sql, other_forms, 9, dbTransaction);
  if (!ExecuteStatement(dbTransaction, temp_sql))
   throw(string("Could not create temporary list of visits (tasks) to export"));

  /* TODO:  at this point we really should add to temp_hl7_export_tasks all tasked linked to from observation sets which were updated within the reporting period
           (where not already in the temp table) ... otherwise, once a task is exported, lab data, vitals, etc. added later will not be picked up the next time around
           The question that concerns me about this approach though is:  what do we set the A/Y/N status to on the task?  any special handling need to happen there? */

  // for pharmacy data:  lather, rinse, repeat
  string final_concat2;
  vector<string> date_params2, combined_params2;
  if (!npirs)
  {
   /* Here's another idea:  hell no.
   string ndw_full_string = ndw_date_string;
   if (use_stop_date)
     ndw_full_string = "pharmacy_info.dispense_date <= '" + stop_date.OutDate(global_mode) + "' and (" + ndw_date_string + ")";
   combined_params2.push_back(ndw_full_string);
   */
  }
  // ALL we need to do is restrict by date.  That's it.  Seriously.  Promise.
  if (use_start_date)
   date_params2.push_back("pharmacy_info.dispense_date >= '" + start_date.OutDate(global_mode) + "'");
  if (use_stop_date)
   date_params2.push_back("pharmacy_info.dispense_date <= '" + stop_date.OutDate(global_mode) + "'");
  if (!date_params2.empty())
   combined_params2.push_back("(" + VecToDelimString(date_params2, "", "", " and ") + ")");
  else if (npirs)
   if (!combined_params2.empty()) combined_params2.push_back("true");
  // put it together, add it to the SQL
  final_concat2 = VecToDelimString(combined_params2, "(", ")", " or ");
  string extra_params2;
  if (final_concat2 != "")
   extra_params2 = " and (" + final_concat2 + ") ";
  else
   extra_params2 = "";

  Log("Fetching people info", true);
/*  string hl7_people_sql = "left join hl7_people on hl7_people.person_id = people.id "
                          "left join hl7_exports exports on exports.id = " // we're going to consider the most-recently-performed export (by ID) that WAS_SENT to be the one we should compare to
                          "(select first 1 id from hl7_exports i_exports where i_exports.id = hl7_people.hl7_export_id and was_sent = 1 order by i_exports.id desc) ";
*/
  string hl7_people_sql = "left join hl7_people on hl7_people.id = " // we're going to consider the most-recently-performed export (by ID) that WAS_SENT to be the one we should compare to
                          "(select first 1 i_people.id from hl7_people i_people "
                           "inner join hl7_exports i_exports on i_people.hl7_export_id = i_exports.id "
                           "where i_exports.was_sent = 1 and i_people.person_id = people.id order by i_exports.end_date desc, i_exports.id desc) "
                          "left join hl7_exports exports on exports.id = hl7_people.hl7_export_id ";
  if (pharm)
  {
   temp_sql =  "insert into temp_hl7_export_pharm "
               "select pharmacy_info.id, pharmacy_info.person_id "
               "from pharmacy_info ";
   if (!npirs) // slightly different from hl7_people_sql ... it joins to pharmacy_info.person_id
    temp_sql += ReplaceStringWithString(hl7_people_sql, "= people.id", "= pharmacy_info.person_id");
   temp_sql += "where pharmacy_info.person_id > 0" + extra_params2;
   Log(temp_sql);
   DebugSql(this, &temp_sql, other_forms, 9, dbTransaction);
   //QueryData(this, dbTransaction, temp_sql);
   if (!ExecuteStatement(dbTransaction, temp_sql))
    throw(string("Could not create temporary list of visits (pharmacy) to export"));
  }

  // step 3
  if (!ExecuteStatement(dbTransaction, "insert into temp_hl7_export_people_union"
                                       " select distinct temp_hl7_export_tasks.person_id from temp_hl7_export_tasks"))
   throw(string("Could not create temporary list of people to export (tasks)"));

  if (pharm)
   if (!ExecuteStatement(dbTransaction, "insert into temp_hl7_export_people_union"
                                        " select temp_hl7_export_pharm.person_id from temp_hl7_export_pharm"))
    throw(string("Could not create temporary list of people to export (pharmacy)"));

/* // what?  why?  no.
  if (!npirs)
  {
   temp_sql = "insert into temp_hl7_export_people_union "
              "select people.id from people " + hl7_people_sql +
              "where people.id > 0 and (" + ndw_date_string + ")";
   Log(temp_sql);
   DebugSql(this, &temp_sql, other_forms, 9, dbTransaction);
   if (!ExecuteStatement(dbTransaction, temp_sql))
    throw(string("Could not create temporary list of people to export (people)"));
  }
*/

  // step 3.5
  if (!ExecuteStatement(dbTransaction, "insert into temp_hl7_export_people"
                                       " select distinct person_id from temp_hl7_export_people_union"));
  //QueryData(this, dbTransaction, "select distinct person_id from temp_hl7_export_people");
  // step 3.5 (totals for progress info)
  labelProgressTotalPeople->Caption = GetOneField(dbTransaction, "select count(distinct(person_id)) from temp_hl7_export_people").c_str();
  labelProgressTotalVisitPeople->Caption = GetOneField(dbTransaction, "select count(distinct person_id) from temp_hl7_export_tasks").c_str();
  labelProgressTotalVisits->Caption = GetOneField(dbTransaction, "select count(task_id) from temp_hl7_export_tasks").c_str();
  labelProgressTotalPharmacyVisits->Caption = GetOneField(dbTransaction, "select count(pharmacy_info_id) from temp_hl7_export_pharm").c_str();

  // step 4
  map<string, string> empty_row;
  list<map<string, string> > empty_rows;
  map<string, string> person_row;
  list<map<string, string> > person_name_rows;
  list<map<string, string> > person_affiliation_rows;
  list<map<string, string> > person_contact_info_rows;
  list<map<string, string> > person_chart_rows;
  list<map<string, string> > person_insurance_rows;

  /* if you want easy_streaming_dataset to pop up EditSql windows... */
  #define debug_easy_streaming_datasets

  Log("Starting person_stream", true);

  string easy_sql;
  easy_sql =  "select people.*, "
              "case when charts.id is not null then charts.id else people.id end chart_id, "
              "case when charts.id is not null then charts.group_id else 0 end chart_location, "
              "case when charts.id is not null then charts.chart_number else '' end chart_number, "
              "case when charts.id is not null then charts.current_id else 4 end chart_current_id ";
  if (!npirs)
   easy_sql += ", exports.id export_id, case when exports.end_date is not null then exports.end_date else cast(exports.run_date as date) end export_date, "
               "case when exports.id is not null then 1 else 0 end hl7_have_sent_person ";
    easy_sql += "from temp_hl7_export_people "
              "inner join people on temp_hl7_export_people.person_id = people.id "
   /* TODO -c HACK: they want a chart number in the people file too, and they said to pick the one at our "biggest" facility.  this is so dumb.
      Also, I'd like to change this to use the same code as elsewhere, i.e.:
          set<string> can_see_charts = GetSpecialGroupPermissions("charts");
          sql += "(select first 1 charts.chart_number from charts "
                  "inner join groups on charts.location_id = groups.chart_location_id and groups.has_charts = 1 and charts.location_id > 0 "
                  "where charts.person_id = people.id and " + SafeInClause("groups.id", SetToVec(can_see_charts)) +
                  "and charts.current_id > 2 order by groups.hl7_priority asc nulls last) chart_number, ";
      The problem with that being that has_charts isn't 1:1 with the list of group_id's below.  Hrm.  No time now.  Later, maybe. */
              "left  join charts on charts.id = (select first 1 charts.id from charts inner join groups on charts.group_id = groups.id where charts.person_id = people.id "
                "and charts.group_id in (" + lookupTables->IdOf("groups", "Little Axe Clinic") + "," + lookupTables->IdOf("groups", "Shawnee Clinic")
                + "," + lookupTables->IdOf("groups", "Contract Health") + "," + lookupTables->IdOf("groups", "Dental Clinic")
                + "," + lookupTables->IdOf("groups", "IHS/NDW Export") + ") order by groups.hl7_priority asc nulls last, charts.current_id desc) ";
  if (!npirs)
   easy_sql += hl7_people_sql;                                                 /* ehm, no.  They want our old/inactive data too.  If we exclude it, we'll never update their records during incremental exports, now will we? */
  easy_sql += "where people.id > 0 order by temp_hl7_export_people.person_id"; /* and charts.current_id+1 > 3 and charts.chart_number not like '%INACTIVE%' */
  Log(easy_sql);
  easy_streaming_dataset person_stream (dbTransaction, easy_sql);
  easy_sql = "select person_names.* ";
  if (!npirs)
    easy_sql += ", exports.id export_id, people.hl7_have_sent_aliases, case when exports.end_date is not null then exports.end_date else cast(exports.run_date as date) end export_date ";
  easy_sql += "from temp_hl7_export_people "
              "inner join person_names on temp_hl7_export_people.person_id = person_names.person_id ";
  if (!npirs)
   easy_sql += "left join people on person_names.person_id = people.id " + hl7_people_sql;
  easy_sql += "where person_names.current_id + 1 > 3 "
              "order by temp_hl7_export_people.person_id";
  Log(easy_sql);
  easy_streaming_dataset person_name_stream (dbTransaction, easy_sql);
  easy_sql =  "select affiliations.*, tribe_types.code tribe_code ";
  if (!npirs)
    easy_sql += ", exports.id export_id, people.hl7_have_sent_aliases, case when exports.end_date is not null then exports.end_date else cast(exports.run_date as date) end export_date ";
  easy_sql += "from temp_hl7_export_people "
              "inner join affiliations on temp_hl7_export_people.person_id = affiliations.person_id "
              "inner join tribe_types on affiliations.tribe_id = tribe_types.id ";
  if (!npirs)
   easy_sql += "left join people on temp_hl7_export_people.person_id = people.id " + hl7_people_sql;
  easy_sql += "where affiliations.current_id + 1 > 3 "
              "order by temp_hl7_export_people.person_id";
  Log(easy_sql);
  easy_streaming_dataset person_affiliation_stream (dbTransaction, easy_sql);
  easy_sql =  "select person_contact_info.*, ihs_equiv_states.ihs_state_id, ihs_equiv_counties.ihs_county_id, ihs_equiv_communities_by_city.ihs_community_id ";
  if (!npirs)
    easy_sql += ", exports.id export_id, people.hl7_have_sent_aliases, case when exports.end_date is not null then exports.end_date else cast(exports.run_date as date) end export_date ";
  easy_sql += "from temp_hl7_export_people "
              "inner join person_contact_info on temp_hl7_export_people.person_id = person_contact_info.person_id "
              "left join ihs_equiv_states on ihs_equiv_states.state_id = person_contact_info.state_id "
              "left join ihs_equiv_communities_by_city on ihs_equiv_communities_by_city.ihs_state_id = ihs_equiv_states.ihs_state_id and ihs_equiv_communities_by_city.city = person_contact_info.city "
              "left join ihs_equiv_counties on ihs_equiv_counties.county_id = person_contact_info.county_id ";
  if (!npirs)
   easy_sql += "left join people on temp_hl7_export_people.person_id = people.id " + hl7_people_sql;
  easy_sql += "where person_contact_info.current_id + 1 > 3 "
              "order by temp_hl7_export_people.person_id";
  Log(easy_sql);
  easy_streaming_dataset person_contact_info_stream (dbTransaction, easy_sql);
  easy_sql = "select charts.*, groups.hl7_asufac ";
  if (!npirs)
    easy_sql += ", exports.id export_id, people.hl7_have_sent_charts, case when exports.end_date is not null then exports.end_date else cast(exports.run_date as date) end export_date ";
  easy_sql += "from temp_hl7_export_people "
              "inner join charts on temp_hl7_export_people.person_id = charts.person_id "
              "left join groups on charts.group_id = groups.id ";
  if (!npirs)
    easy_sql += "left join people on charts.person_id = people.id " + hl7_people_sql;
  easy_sql += "where charts.current_id + 1 > 3 and charts.group_id in (" + lookupTables->IdOf("groups", "Little Axe Clinic") +
              "," + lookupTables->IdOf("groups", "Shawnee Clinic") + "," + lookupTables->IdOf("groups", "Contract Health") +
              "," + lookupTables->IdOf("groups", "Dental Clinic") + "," + lookupTables->IdOf("groups", "IHS/NDW Export") + ") " + // TODO: temporary, until we (maybe) add 6-character FACILITY codes to location_types? and add location_id to visits?
                //" and chart_number not like '%INACTIVE%' "
              "order by temp_hl7_export_people.person_id, charts.current_id desc";
  Log(easy_sql);
  easy_streaming_dataset person_chart_stream (dbTransaction, easy_sql);
  // nasty because of renaming fields and the like
  easy_sql = "select insurance_coverage.id c_id, "
                    "insurance_coverage.last_update_stamp c_last_update_stamp, "
                    "insurance_coverage.person_id c_person_id, "
                    "insurance_coverage.tax_entity_id c_tax_entity_id, "
                    "insurance_coverage.name_on_card c_name_on_card, "
                    "insurance_coverage.insurance_policy_id c_insurance_policy_id, "
                    "insurance_coverage.person_code c_person_code, "
                    "insurance_policies.id p_id, "
                    "insurance_policies.last_update_stamp p_last_update_stamp, "
                    "insurance_policies.tax_entity_id p_tax_entity_id, "
                    "insurance_policies.policy_number p_policy_number, "
                    "insurance_policies.group_number p_group_number, "
                    "insurance_policies.start_date p_start_date, "
                    "insurance_policies.stop_date p_stop_date, "
                    "insurance_policies.coverage_id p_coverage_id, "
                    "insurance_policies.source_id p_source_id, "
                    "insurance_policies.name_on_policy p_name_on_policy, "
                    "insurance_policies.person_id p_person_id, "
                    /* these three fields moved from tax_entities to insurance_policies */
                    "insurance_policies.insurance_id t_insurance_id, "
                    "insurance_policies.processor_control_number t_processor_control_number, "
                    "insurance_policies.bin_number t_bin_number, "
                    "tax_entities.id t_id, "
                    "tax_entities.last_update_stamp t_last_update_stamp, "
                    "tax_entities.tax_entity_id t_tax_entity_id, "
                    "tax_entities.name t_name, "
                    /* tax_entities.* address fields are no longer in use, will be dropped */
                    "tax_entities.specialty t_specialty, "
                    "tin.identifier t_taxid ";
  if (!npirs)
    easy_sql += ", exports.id export_id, people.hl7_have_sent_insurance, case when exports.end_date is not null then exports.end_date else cast(exports.run_date as date) end export_date ";
  easy_sql += "from temp_hl7_export_people "
              "inner join insurance_coverage on temp_hl7_export_people.person_id = insurance_coverage.person_id "
              "left join insurance_policies on insurance_coverage.insurance_policy_id = insurance_policies.id "
              "left join tax_entities on insurance_policies.tax_entity_id = tax_entities.id "
              "left join entity_ids tin on tin.tax_entity_id = tax_entities.id and tin.current_id >= 3 and tin.entity_id_id = '" + lookupTables->IdOf("entity_id_types", "TIN/EIN") + "' ";
  if (!npirs)
    easy_sql += "left join people on insurance_coverage.person_id = people.id " + hl7_people_sql;
  easy_sql += "where "
              "insurance_coverage.current_id + 1 > 3 and "
              "insurance_policies.current_id + 1 > 3 "
              "order by temp_hl7_export_people.person_id";
  Log(easy_sql);
  easy_streaming_dataset person_insurance_stream (dbTransaction, easy_sql);
  Log("Writing people file", true);
  while (person_stream.GetRow(person_row) && (!use_people_limit || people_count < people_limit))
  {
   // MSH
   // EVN (A31)
   // PID
   //   ZRB
   //   ZP2
   //   ZRD
   //  {ZRL}
   //  {ZRC}
   //  {IN1}
   //     ZIN

   // build in-memory lists
   person_name_stream.AdvanceWhileMap(streaming_dataset_map_value_predicate<less_strings_as_ints>("person_id", less_strings_as_ints(), person_row["id"]));
   person_name_stream.GetRowsWhile(person_name_rows, streaming_dataset_map_value_predicate<equal_to_strings_as_ints>("person_id", equal_to_strings_as_ints(), person_row["id"]));

   person_affiliation_stream.AdvanceWhileMap(streaming_dataset_map_value_predicate<less_strings_as_ints>("person_id", less_strings_as_ints(), person_row["id"]));
   person_affiliation_stream.GetRowsWhile(person_affiliation_rows, streaming_dataset_map_value_predicate<equal_to_strings_as_ints>("person_id", equal_to_strings_as_ints(), person_row["id"]));

   person_contact_info_stream.AdvanceWhileMap(streaming_dataset_map_value_predicate<less_strings_as_ints>("person_id", less_strings_as_ints(), person_row["id"]));
   person_contact_info_stream.GetRowsWhile(person_contact_info_rows, streaming_dataset_map_value_predicate<equal_to_strings_as_ints>("person_id", equal_to_strings_as_ints(), person_row["id"]));

   person_chart_stream.AdvanceWhileMap(streaming_dataset_map_value_predicate<less_strings_as_ints>("person_id", less_strings_as_ints(), person_row["id"]));
   person_chart_stream.GetRowsWhile(person_chart_rows, streaming_dataset_map_value_predicate<equal_to_strings_as_ints>("person_id", equal_to_strings_as_ints(), person_row["id"]));

   person_insurance_stream.AdvanceWhileMap(streaming_dataset_map_value_predicate<less_strings_as_ints>("c_person_id", less_strings_as_ints(), person_row["id"]));
   person_insurance_stream.GetRowsWhile(person_insurance_rows, streaming_dataset_map_value_predicate<equal_to_strings_as_ints>("c_person_id", equal_to_strings_as_ints(), person_row["id"]));

   // update change_code on all records that will need it
   hl7_change_code (person_row, "hl7_have_sent_person", false, list<map<string,string> >(), run_as);   // ZRB
   hl7_change_code (person_row, "hl7_have_sent_charts", true, person_chart_rows, run_as);              // ZRC
   hl7_change_code (person_row, "hl7_have_sent_demographics", true, person_contact_info_rows, run_as); // ZRD #1
   hl7_change_code (person_row, "hl7_have_sent_demographics", true, person_affiliation_rows, run_as);  // ZRD #2
   hl7_change_code (person_row, "hl7_have_sent_aliases", true, person_name_rows, run_as);              // ZRL
   hl7_change_code (person_row, "hl7_have_sent_insurance", true, person_insurance_rows, run_as);       // ZIN

   /* TODO: insert logic right here, to determine what needs sending, for incremental exports
      That logic is this:
        A set is defined to be the things that are 1:M with people:  charts, aliases, insurance
        Think of each type of set as a pointer from people to the set.  Person->Aliases, Person->Insurances, Person->Charts
        If that pointer is null (empty), and you want to add a set, you should specify "A"
          However, their import utility will now accept "Y" as well, under the assumption that it's still new data, you just
            weren't keeping track of what you had and hadn't sent them so far as well as you should.
        If that pointer is non-null, then "A" is an error; you can't "add a set" that already exists.
        If you at any point fail to send a set, the set is deleted.  E.g., sending a person without sending their charts deletes their
          charts and sets the pointer back to null.
        A set's members must all have the same change code.  Or to be exactly correct, the first change code sets the change code
          for the entire set.  So you might get lucky and have the right code on the first member, but the safe way to handle it
          is to look through the set and figure out what change code should be applied to the set, and use that for the whole set.

        Since we therefore have to send all records of all types for a person every time that person needs to be sent...
          If the contents of a set all have change codes of "N", use "N"
          If ANY members of the set have a change code of "A" but none have "Y", then use "A"
          If ANY members of the set have a change code of "Y", use "Y"
        "Y" is both a fail-safe and logically tells us "this set has been sent before" in which case "A" doesn't make sense, see?
   */
   if (!npirs) // actually won't hurt anything to run these for a full export, but it wastes time
   {
    hl7_assimilate_set(person_name_rows);
    hl7_assimilate_set(person_chart_rows);
    hl7_assimilate_set(person_insurance_rows);
   }

   temp.clear();
   //temp["id"] = GetNextId(dbTransaction, "hl7_people"); // it's on a trigger.
   temp["hl7_export_id"] = hl7_exports_id;
   temp["person_id"] = person_row["id"];
   Log("Inserting into hl7_people:  " + MapToDelimString(temp, "{","}","","'","'","'","'",","));
   if (!ExecuteInsertStatement(dbTransaction, "hl7_people", temp))
     Log("ERROR:  hl7_people insert failed!");

   *export_file << hl7_MSH(*export_batch, export_stamp, "A31", "0" /* MSH number */).GetSegmentValue(*export_batch);
   *export_file << hl7_EVN("A31", export_stamp).GetSegmentValue(*export_batch);
   *export_file << hl7_PID(*export_batch, person_row, empty_row, person_name_rows, person_contact_info_rows).GetSegmentValue(*export_batch);
   *export_file << hl7_ZRB(person_row).GetSegmentValue(*export_batch);
   *export_file << hl7_ZP2(person_row, person_name_rows, person_affiliation_rows, person_contact_info_rows).GetSegmentValue(*export_batch);
   *export_file << hl7_ZRD(person_row, person_name_rows, person_affiliation_rows, person_contact_info_rows).GetSegmentValue(*export_batch);
   unsigned int n = 1;
   for (list<map<string, string> >::iterator i = person_name_rows.begin(); i != person_name_rows.end(); i++, n++)
    if ((*i)["id"] != person_row["person_names_id"])             // not primary name
     *export_file << hl7_ZRL(*i).GetSegmentValue(*export_batch); // alias
//   if (n == 1)
//    *export_file << hl7_ZRL(empty_row).GetSegmentValue(*export_batch);

   unsigned int h = 1;
   for (list<map<string, string> >::iterator i = person_chart_rows.begin(); i != person_chart_rows.end(); i++, h++)
    *export_file << hl7_ZRC(*export_batch, person_row, *i).GetSegmentValue(*export_batch); // chart number
//   if (h == 1)
//    *export_file << hl7_ZRC(person_row, empty_row).GetSegmentValue(*export_batch); // chart number

   unsigned int k = 1;
   for (list<map<string, string> >::iterator i = person_insurance_rows.begin(); i != person_insurance_rows.end(); i++, k++)
   {
    *export_file << hl7_IN1(k, *i).GetSegmentValue(*export_batch); // insurance
    *export_file << hl7_ZIN(*i).GetSegmentValue(*export_batch);
   }
//   if (k == 1)
//   {
//    *export_file << hl7_IN1(k, empty_row).GetSegmentValue(*export_batch); // insurance
//    *export_file << hl7_ZIN(empty_row).GetSegmentValue(*export_batch);
//   }

   *export_file << flush;
   person_stream.Next();
   people_count++;

   progress_counter++;
   if (!(progress_counter % progress_interval))
   {
    labelProgressPeople->Caption = IntToString(people_count).c_str();
    richDebug->Text = string(
                      "people: " + IntToString(person_stream.GetRowNumber()) + "\n"
                      "names: " + IntToString(person_name_stream.GetRowNumber()) + "\n"
                      "affiliations: " + IntToString(person_affiliation_stream.GetRowNumber()) + "\n"
                      "addresses: " + IntToString(person_contact_info_stream.GetRowNumber()) + "\n"
                      "charts: " + IntToString(person_chart_stream.GetRowNumber()) + "\n"
                      "coverages: " + IntToString(person_insurance_stream.GetRowNumber()) + "\n"
                      ).c_str();
    Application->ProcessMessages();
   }
  }

  // switch export file and settings
  if (!combined)
  {
   // ZTS
   // BTS
   *export_file << hl7_ZTS(0, people_count).GetSegmentValue(*export_batch);
   *export_file << hl7_BTS(0, "").GetSegmentValue(*export_batch);

   export_file  = &visits_file;
   export_batch = &visits_batch;

   // DEBUG: for debugging, turn off output buffer
   export_file->rdbuf()->pubsetbuf(0, 0);

   // BHS
   // ZHS
   *export_file << hl7_BHS(*export_batch, export_stamp).GetSegmentValue(*export_batch);
   *export_file << hl7_ZHS(start_date, stop_date, hl7_exports_id /* export id */).GetSegmentValue(*export_batch);
  }

  // let's re-use a few streams here, as someone thought it was a good idea to retransmit crap all over the place.
  Log("Resetting streams", true);
  person_stream.Reset();
  person_name_stream.Reset();
  person_affiliation_stream.Reset();
  person_contact_info_stream.Reset();

  // step 5
  map<string, string> visit_row;
  list<map<string, string> > visit_condition_rows;
  list<map<string, string> > visit_procedure_rows;
  list<map<string, string> > visit_reason_rows;
  list<map<string, string> > visit_bill_rows;
  list<map<string, string> > visit_education_rows;
  list<map<string, string> > visit_measurement_rows;
  map<string, string> pharmacy_row;

  Log("Starting visit_stream", true);
  // PSay("Start visits?", m_debug, MB_OK);

  string sql =
   "select tasks.person_id e_person_id, tasks.id, tasks.person_id, "
   "tasks.last_update_stamp, tasks.current_id, tasks.group_id, "
   "date_of_task, time_of_task, time_spent, visit_time_spent, tasks.tax_entity_id, user_id, "
   "parent_task_id, " /* auth_service_id, auth_cob_number, " */ // actually coming from the parent_task_id (sub-select)
   "method_id, visit_location_id, event_id, "
   "com_visit_id, com_dept_id, "
   "program_id, diab_activity_id, " /* appt_reason_id, " */ // actually in APPOINTMENTS
   "charts.id chart_id, charts.group_id chart_location, charts.chart_number chart_number, "
   "user_discipline_types.ihs_id user_discipline "
   /* "beh_care_id " */ //actually in TAX_ENTITIES
   "from tasks "
   "inner join temp_hl7_export_tasks on temp_hl7_export_tasks.task_id = tasks.id "
   "left  join charts on charts.id = (select first 1 charts.id from charts where charts.person_id = tasks.person_id "
   "and charts.group_id = tasks.group_id and charts.current_id+1 >= 4 and charts.chart_number not like '%INACTIVE%') "
   "left  join users on tasks.user_id = users.id "
   "left  join user_discipline_types on users.user_discipline_id = user_discipline_types.id "
   "where tasks.current_id + 1 > 3 "
   "order by tasks.person_id asc, tasks.id asc";

  // string the_plan;
  // GetSqlPlan (dbTransaction, sql, the_plan);
  Log(sql);
  easy_streaming_dataset visit_stream (dbTransaction, sql);
  Log("Starting visit_condition_stream", true);
  // PSay("Start conditions?", m_debug, MB_OK);
  easy_sql = "select temp_hl7_export_tasks.person_id e_person_id, conditions.* "
   "from temp_hl7_export_tasks inner join conditions on temp_hl7_export_tasks.task_id = conditions.task_id "
   "where current_id + 1 > 3 "
   "order by temp_hl7_export_tasks.person_id, temp_hl7_export_tasks.task_id, conditions.index_number";
  Log(easy_sql);
  easy_streaming_dataset visit_condition_stream (dbTransaction, easy_sql);
  Log("Starting visit_procedure_stream", true);
  // PSay("Start procedures?", m_debug, MB_OK);
  // Nice planning, Philip.  Unlike conditions, procedures doesn't HAVE a group_id, so you're going to have to
  // join to tasks to GET a group_id, and you NEED one the way you coded dental stuff, below.  Fixed.  --Paul
  easy_sql = "select temp_hl7_export_tasks.person_id e_person_id, tasks.group_id, procedures.* "
   "from temp_hl7_export_tasks "
   "inner join procedures on temp_hl7_export_tasks.task_id = procedures.task_id "
   "inner join tasks on temp_hl7_export_tasks.task_id = tasks.id " // kludgey, but needed for group_id
   "where tasks.current_id + 1 > 3 "
   "order by temp_hl7_export_tasks.person_id, temp_hl7_export_tasks.task_id";
  Log(easy_sql);
  easy_streaming_dataset visit_procedure_stream (dbTransaction, easy_sql);
  Log("Starting visit_reason_stream", true);
  // PSay("Start procedure reasons?", m_debug, MB_OK);
  easy_sql = "select temp_hl7_export_tasks.person_id e_person_id, temp_hl7_export_tasks.task_id task_id, procedure_reasons.* "
   "from temp_hl7_export_tasks "
   "inner join procedures on temp_hl7_export_tasks.task_id = procedures.task_id "
   "inner join procedure_reasons on procedure_reasons.procedure_id = procedures.id "
   "where procedures.current_id + 1 > 3 and procedure_reasons.current_id + 1 > 3 "
   "order by temp_hl7_export_tasks.person_id, temp_hl7_export_tasks.task_id";
  Log(easy_sql);
  easy_streaming_dataset visit_reason_stream (dbTransaction, easy_sql);
  Log("Starting visit_bill_stream", true);
  // PSay("Start bills?", m_debug, MB_OK);
  easy_sql = "select temp_hl7_export_tasks.person_id e_person_id, bills.* "
   "from temp_hl7_export_tasks "
   "inner join bills on temp_hl7_export_tasks.task_id = bills.task_id "
   "where current_id + 1 > 3 "
   "order by temp_hl7_export_tasks.person_id, temp_hl7_export_tasks.task_id";
  Log(easy_sql);
  easy_streaming_dataset visit_bill_stream (dbTransaction, easy_sql);
  Log("Starting visit_education_stream", false);
  // PSay("Start education?", m_debug, MB_OK);
  easy_sql = "select temp_hl7_export_tasks.person_id e_person_id, com_health_edu.* "
   "from temp_hl7_export_tasks "
   "inner join com_health_edu on temp_hl7_export_tasks.task_id = com_health_edu.task_id "
   "where current_id + 1 > 3 "
   "order by temp_hl7_export_tasks.person_id, temp_hl7_export_tasks.task_id";
  Log(easy_sql);
  easy_streaming_dataset visit_education_stream (dbTransaction, easy_sql);
  Log("Starting visit_measurement_stream", true);
  // PSay("Start measurements?", m_debug, MB_OK);
  easy_sql = "select temp_hl7_export_tasks.person_id e_person_id, "
   "obs_response_sets.id, obs_response_sets.task_id, "
   "obs_questions.short_item question_item, obs_questions.loinc_code, obs_questions.msr_code, "
   "obs_responses.obs_question_id, obs_responses.response_metric, units.item unit_item, "
   "obs_expected_ranges.min_response_metric, obs_expected_ranges.max_response_metric "
   "from temp_hl7_export_tasks "
   "inner join obs_response_sets on temp_hl7_export_tasks.task_id = obs_response_sets.task_id "
   "inner join obs_responses on obs_responses.obs_response_set_id = obs_response_sets.id "
   "left join obs_questions on obs_questions.id = obs_responses.obs_question_id "
   "left join units on units.id = obs_questions.unit_id "
   "left join obs_expected_ranges on obs_expected_ranges.obs_question_id = obs_responses.obs_question_id and "
   "obs_expected_ranges.gender_id in (0, (select gender_id from people where people.id = temp_hl7_export_tasks.person_id)) "
   "where obs_response_sets.current_id + 1 > 3 "
   "order by temp_hl7_export_tasks.person_id, temp_hl7_export_tasks.task_id";
  Log(easy_sql);
  easy_streaming_dataset visit_measurement_stream (dbTransaction, easy_sql);
  Log("Starting pharmacy_stream", true);
  // PSay("Start pharmacy?", m_debug, MB_OK);
  easy_sql = "select '" + lookupTables->IdOf("groups", "Pharmacy") + "' group_id, temp_hl7_export_pharm.person_id e_person_id, pharmacy_info.*, "
   "pharmacy_locations.store_number, pharmacy_locations.address, pharmacy_locations.city, pharmacy_locations.state, pharmacy_locations.zip, pharmacy_locations.phone, pharmacy_locations.pharmacy_id, "
   "conditions.code, conditions.note, "
   // field translation so PV1 and ZV1 have the values they need for pharmacy data
   // TODO:  Will pharmacy_info.id conflict with cob_number for other tasks?  do we need to prefix with a 9 and pad, like elsewhere (somewhere)?
   "pharmacy_info.dispense_date date_of_task, 132 auth_service_id, pharmacy_info.id auth_cob_number "
   "from temp_hl7_export_pharm "
   "inner join pharmacy_info on temp_hl7_export_pharm.pharmacy_info_id = pharmacy_info.id "
   "left join pharmacy_locations on pharmacy_locations.id = pharmacy_info.pharmacy_location_id "
   "left join tasks on pharmacy_info.task_id = tasks.id "
   "left join conditions on conditions.id = (select first 1 id from conditions IC where IC.person_id = pharmacy_info.person_id and IC.task_id = tasks.id and IC.current_id > 2 and IC.condition_id != 7) "
   "order by temp_hl7_export_pharm.person_id";
  Log(easy_sql);
  easy_streaming_dataset pharmacy_stream (dbTransaction, easy_sql);

  people_count = 0;
  visits_count = 0;

  if (show_about_to)
   Log("About to loop through people", false);

  map<string, string> this_person_stats; // debug
  while (person_stream.GetRow(person_row) && (!use_people_limit || people_count < people_limit) && (!use_visits_limit || visits_count < visits_limit))
  {
   ////string this_person_name = GetPersonName(dbTransaction, person_row["id"], first_middle_last);
   ////Log("Exporting visits for person: " + person_row["id"] + " (" + this_person_name + ")");

   // MSH
   // EVN (A08)
   // PID
   //   ZP2
   // PV1
   //   ZV1
   //   ZEN
   //  {ZVP}
   //  {DG1}
   //     ZDX
   //  [ZPN]
   // {[ZDN]}
   //  [ZDP]
   // {[ZIM]}
   // {[ZMD]}
   // {[OBX]} (health, measurements, exams, diagnosis, labs, education, skin tests)

   // build in-memory lists
   if (show_about_to)
    Log("About to advance name stream", false);

   person_name_stream.AdvanceWhileMap(streaming_dataset_map_value_predicate<less_strings_as_ints>("person_id", less_strings_as_ints(), person_row["id"]));
   person_name_stream.GetRowsWhile(person_name_rows, streaming_dataset_map_value_predicate<equal_to_strings_as_ints>("person_id", equal_to_strings_as_ints(), person_row["id"]));

   if (show_about_to)
    Log("About to advance affiliation stream", false);

   person_affiliation_stream.AdvanceWhileMap(streaming_dataset_map_value_predicate<less_strings_as_ints>("person_id", less_strings_as_ints(), person_row["id"]));
   person_affiliation_stream.GetRowsWhile(person_affiliation_rows, streaming_dataset_map_value_predicate<equal_to_strings_as_ints>("person_id", equal_to_strings_as_ints(), person_row["id"]));

   if (show_about_to)
    Log("About to advance address stream", false);

   person_contact_info_stream.AdvanceWhileMap(streaming_dataset_map_value_predicate<less_strings_as_ints>("person_id", less_strings_as_ints(), person_row["id"]));
   person_contact_info_stream.GetRowsWhile(person_contact_info_rows, streaming_dataset_map_value_predicate<equal_to_strings_as_ints>("person_id", equal_to_strings_as_ints(), person_row["id"]));

   // abort if no row, if row is for a different person, we stay here (we don't call .Next()) and the next loop should pick it up?
   // tasks
   visit_stream.AdvanceWhileMap(streaming_dataset_map_value_predicate<less_strings_as_ints>("e_person_id", less_strings_as_ints(), person_row["id"]));
   if (show_about_to)
    labelCurrentState->Caption = "About to loop through visits";
   Application->ProcessMessages();
   // for debug log only;
   this_person_stats.clear();
   this_person_stats["visits"] = IntToString(visit_stream.GetRowNumber());
   this_person_stats["conditions"] = IntToString(visit_condition_stream.GetRowNumber());
   this_person_stats["procedures"] = IntToString(visit_procedure_stream.GetRowNumber());
   this_person_stats["reasons"] = IntToString(visit_reason_stream.GetRowNumber());
   this_person_stats["bills"] = IntToString(visit_bill_stream.GetRowNumber());
   this_person_stats["educations"] = IntToString(visit_education_stream.GetRowNumber());
   this_person_stats["measurements"] = IntToString(visit_measurement_stream.GetRowNumber());
   this_person_stats["pharmacy visits"] = IntToString(pharmacy_stream.GetRowNumber());
   while (visit_stream.GetRow(visit_row) && (visit_row["e_person_id"] ==  person_row["id"]) && (!use_visits_limit || visits_count < visits_limit))
   {
    // build in-memory lists
    // first find beginning for this person, then find beginning for this visit, grab all information related to the visit
    // (this apparently is ... yeah. how you do that. if sorting by multiple fields. and looking for crap.)
    // if the current row matches, we stay there, so there's not an off-by-one problem when searching by one thing then the next.

    if (show_about_to)
     Log("About to retrieve authorization information", false);

    if (!IsBlankOrZero(visit_row["parent_task_id"]))
    {
     map<string, string> temp;
     if (!SqlToMap(dbTransaction, "select tasks.cob_number, services_provided.service_id from tasks left join services_provided on services_provided.id = tasks.services_provided_id where tasks.id = " + visit_row["parent_task_id"], temp))
      throw(string("Could not retrieve authorization-related information for parent_task_id (" + visit_row["parent_task_id"] + ")"));
     visit_row["auth_cob_number"] = temp["cob_number"];
     visit_row["auth_service_id"] = temp["service_id"];
    }

    if (show_about_to)
     Log("About to advance condition stream", false);

    // advance while ("person_id" < ["person_id"] || ("person_id" == ["person_id"] && "task_id" < ["task_id"]))

    // or (<, and(==, <))
    // could have been done in a single statement, but it got even harder to read, neh?
    streaming_dataset_map_value_predicate<less_strings_as_ints> task_id_less_than("task_id", less_strings_as_ints(), visit_row["id"]);
    streaming_dataset_map_value_predicate<less_strings_as_ints> person_id_less_than("e_person_id", less_strings_as_ints(), visit_row["person_id"]);
    streaming_dataset_map_value_predicate<equal_to_strings_as_ints> person_id_equal_to("e_person_id", equal_to_strings_as_ints(), visit_row["person_id"]);
    streaming_dataset_map_value_predicate_and<streaming_dataset_map_value_predicate<equal_to_strings_as_ints>, streaming_dataset_map_value_predicate<less_strings_as_ints> > person_id_equal_to_and_task_id_less_than(person_id_equal_to, task_id_less_than);
    streaming_dataset_map_value_predicate_or<streaming_dataset_map_value_predicate<less_strings_as_ints>, streaming_dataset_map_value_predicate_and<streaming_dataset_map_value_predicate<equal_to_strings_as_ints>, streaming_dataset_map_value_predicate<less_strings_as_ints> > > person_id_less_than_or_person_id_equal_to_and_task_id_less_than(person_id_less_than, person_id_equal_to_and_task_id_less_than);
    // hell, might as well name this condition too
    streaming_dataset_map_value_predicate<equal_to_strings_as_ints> task_id_equal_to("task_id", equal_to_strings_as_ints(), visit_row["id"]);

    visit_condition_stream.AdvanceWhileMap(person_id_less_than_or_person_id_equal_to_and_task_id_less_than);
    visit_condition_stream.GetRowsWhile(visit_condition_rows, task_id_equal_to);

    if (show_about_to)
     Log("About to advance procedure stream", false);

    visit_procedure_stream.AdvanceWhileMap(person_id_less_than_or_person_id_equal_to_and_task_id_less_than);
    visit_procedure_stream.GetRowsWhile(visit_procedure_rows, task_id_equal_to);

    if (show_about_to)
     Log("About to advance procedure reason stream", false);

    visit_reason_stream.AdvanceWhileMap(person_id_less_than_or_person_id_equal_to_and_task_id_less_than);
    visit_reason_stream.GetRowsWhile(visit_reason_rows, task_id_equal_to);

    if (show_about_to)
     Log("About to advance bill stream", false);

    visit_bill_stream.AdvanceWhileMap(person_id_less_than_or_person_id_equal_to_and_task_id_less_than);
    visit_bill_stream.GetRowsWhile(visit_bill_rows, task_id_equal_to);

    if (show_about_to)
     Log("About to advance education stream", false);

    visit_education_stream.AdvanceWhileMap(person_id_less_than_or_person_id_equal_to_and_task_id_less_than);
    visit_education_stream.GetRowsWhile(visit_education_rows, task_id_equal_to);

    if (show_about_to)
     Log("About to advance measurement stream", false);

    visit_measurement_stream.AdvanceWhileMap(person_id_less_than_or_person_id_equal_to_and_task_id_less_than);
    visit_measurement_stream.GetRowsWhile(visit_measurement_rows, task_id_equal_to);

    if (show_about_to)
     Log("About to write visit to file", false);

    temp.clear();
    //temp["id"] = GetNextId(dbTransaction, "hl7_visits"); // it's on a trigger.
    temp["hl7_export_id"] = hl7_exports_id;
    temp["task_id"] = visit_row["id"];
    Log("Inserting into hl7_visits:  " + MapToDelimString(temp, "{","}","","'","'","'","'",","));
    if (!ExecuteInsertStatement(dbTransaction, "hl7_visits", temp))
      Log("ERROR:  hl7_visits insert failed!");

    *export_file << hl7_MSH(*export_batch, export_stamp, "A08", "0" /* MSH number */).GetSegmentValue(*export_batch);
    *export_file << hl7_EVN("A08", export_stamp).GetSegmentValue(*export_batch);
    *export_file << hl7_PID(*export_batch, person_row, visit_row, person_name_rows, person_contact_info_rows).GetSegmentValue(*export_batch);
    *export_file << hl7_ZP2(person_row, person_name_rows, person_affiliation_rows, person_contact_info_rows).GetSegmentValue(*export_batch);

    *export_file << hl7_PV1(*export_batch, visit_row, visit_condition_rows, visit_procedure_rows).GetSegmentValue(*export_batch);
    *export_file << hl7_ZV1(*export_batch, visit_row, visit_condition_rows, visit_procedure_rows).GetSegmentValue(*export_batch);
    *export_file << hl7_ZEN(visit_measurement_rows).GetSegmentValue(*export_batch);

    // empty ones to make them happy
    *export_file << hl7_ZVP(1, visit_row).GetSegmentValue(*export_batch); // multiple attending providers, need more data!

    unsigned int c = 1;
    for (list<map<string, string> >::iterator i = visit_condition_rows.begin(); i != visit_condition_rows.end(); i++, c++)
    {
     *export_file << hl7_DG1(c, *i).GetSegmentValue(*export_batch); // condition
     *export_file << hl7_ZDX(*i).GetSegmentValue(*export_batch); // condition (extended)
    }
    // empty ones to make them happy
//    if (c == 1)
//    {
//     *export_file << hl7_DG1(c, empty_row).GetSegmentValue(*export_batch);
//     *export_file << hl7_ZDX(empty_row).GetSegmentValue(*export_batch);
//    }

    // Non-Dental Procedures
    unsigned int p = 1;
    for (list<map<string, string> >::iterator i = visit_procedure_rows.begin(); i != visit_procedure_rows.end(); i++, p++)
     //if ((*i)["tooth_number"] == "" && (*i)["cpt_hcpcs"] != "") // no-code procedures don't seem meaningful
     if ((*i)["group_id"] != lookupTables->IdOf("groups", "Dental Clinic") && (*i)["cpt_hcpcs"] != "") // no-code procedures don't seem meaningful
     {
      //*export_file << hl7_PR1(p, *i).GetSegmentValue(*export_batch); // procedure // WRONG.  CPT codes go in OBX-CPT segment
      *export_file << hl7_OBX_CPT(p, *i).GetSegmentValue(*export_batch); // procedure // Correct.  Like this.
      *export_file << hl7_ZPR(*i, visit_reason_rows, visit_condition_rows).GetSegmentValue(*export_batch); // procedure (extended)
     }
    // empty ones to make them happy
//    if (p == 1)
//    {
//     *export_file << hl7_PR1(p, empty_row).GetSegmentValue(*export_batch);
//     *export_file << hl7_ZPR(empty_row, empty_rows, empty_rows).GetSegmentValue(*export_batch);
//    }

    // Dental Procedures
    unsigned int d = 1;
    for (list<map<string, string> >::iterator i = visit_procedure_rows.begin(); i != visit_procedure_rows.end(); i++, d++)
     //if ((*i)["tooth_number"] != "")
     if ((*i)["group_id"] == lookupTables->IdOf("groups", "Dental Clinic"))
     {
      *export_file << hl7_ZDN(p, *i).GetSegmentValue(*export_batch); // dental procedure
      *export_file << hl7_ZDP(p, *i).GetSegmentValue(*export_batch); // dental procedure (extended)
     }
    vector<hl7_value> measurement_segments = hl7_OBX_measurements (visit_measurement_rows);
    for (vector<hl7_value>::iterator m = measurement_segments.begin(); m != measurement_segments.end(); m++)
     *export_file << (*m).GetSegmentValue(*export_batch); // measurements, both MSR and LAB (but no other types yet)


/*
hl7_value hl7_ZPN (unsigned int sequence_number, map<string, string> & visit_row);
hl7_value hl7_ZIM (unsigned int sequence_number, map<string, string> & visit_immunization_row);
hl7_value hl7_ZMD (unsigned int sequence_number, map<string, string> & pharm_medication_row);
*/

    if (show_about_to)
     Log("About to print visit statistics", false);

    *export_file << flush;
    visit_stream.Next();
    visits_count++;

    progress_counter++;
    if (!(progress_counter % progress_interval))
    {
     labelProgressVisits->Caption = IntToString(visits_count).c_str();
     richDebug->Text = string(
                       "people: " + IntToString(person_stream.GetRowNumber()) + "\n"
                       "names: " + IntToString(person_name_stream.GetRowNumber()) + " (" + IntToString(person_name_rows.size()) + ")\n"
                       "affiliations: " + IntToString(person_affiliation_stream.GetRowNumber()) + " (" + IntToString(person_affiliation_rows.size()) + ")\n"
                       "addresses: " + IntToString(person_contact_info_stream.GetRowNumber()) + " (" + IntToString(person_contact_info_rows.size()) + ")\n"
                       "visits: " + IntToString(visit_stream.GetRowNumber()) + "\n"
                       "conditions: " + IntToString(visit_condition_stream.GetRowNumber()) + " (" + IntToString(visit_condition_rows.size()) + ")\n"
                       "procedures: " + IntToString(visit_procedure_stream.GetRowNumber()) + " (" + IntToString(visit_procedure_rows.size()) + ")\n"
                       "reasons: " + IntToString(visit_reason_stream.GetRowNumber()) + " (" + IntToString(visit_reason_rows.size()) + ")\n"
                       "bills: " + IntToString(visit_bill_stream.GetRowNumber()) + " (" + IntToString(visit_bill_rows.size()) + ")\n"
                       "educations: " + IntToString(visit_education_stream.GetRowNumber()) + " (" + IntToString(visit_education_rows.size()) + ")\n"
                       "measurements: " + IntToString(visit_measurement_stream.GetRowNumber()) + " (" + IntToString(visit_measurement_rows.size()) + ")\n"
                       "pharmacy visits: " + IntToString(pharmacy_stream.GetRowNumber()) + "\n"
                       ).c_str();
     Application->ProcessMessages();
    }
   }

   // pharmacy!
   pharmacy_stream.AdvanceWhileMap(streaming_dataset_map_value_predicate<less_strings_as_ints>("e_person_id", less_strings_as_ints(), person_row["id"]));
   if (show_about_to)
    labelCurrentState->Caption = "About to loop through pharmacy visits";
   Application->ProcessMessages();
   while (pharmacy_stream.GetRow(pharmacy_row) && (pharmacy_row["e_person_id"] ==  person_row["id"]) && (!use_visits_limit || visits_count < visits_limit))
   {
    if (show_about_to)
     Log("About to write pharmacy visit to file", false);

    *export_file << hl7_MSH(*export_batch, export_stamp, "A08", "0" /* MSH number */).GetSegmentValue(*export_batch);
    *export_file << hl7_EVN("A08", export_stamp).GetSegmentValue(*export_batch);
    *export_file << hl7_PID(*export_batch, person_row, empty_row, person_name_rows, person_contact_info_rows).GetSegmentValue(*export_batch);
    *export_file << hl7_ZP2(person_row, person_name_rows, person_affiliation_rows, person_contact_info_rows).GetSegmentValue(*export_batch);

    *export_file << hl7_PV1(*export_batch, pharmacy_row, empty_rows, empty_rows).GetSegmentValue(*export_batch);
    //*export_file << hl7_ZV1(*export_batch, empty_row, empty_rows, empty_rows).GetSegmentValue(*export_batch);
    *export_file << hl7_ZV1(*export_batch, pharmacy_row, empty_rows, empty_rows).GetSegmentValue(*export_batch);
    *export_file << hl7_ZEN(empty_rows).GetSegmentValue(*export_batch);
    *export_file << hl7_ZVP(1, empty_row).GetSegmentValue(*export_batch); // multiple attending providers, need more data!
    *export_file << hl7_DG1(1, pharmacy_row).GetSegmentValue(*export_batch);
    *export_file << hl7_ZDX(empty_row).GetSegmentValue(*export_batch);
    *export_file << hl7_ZMD(1, pharmacy_row).GetSegmentValue(*export_batch); // we don't know how these "go together" // TODO: some sort of grouping?
    *export_file << hl7_PR1(1, empty_row).GetSegmentValue(*export_batch);
    *export_file << hl7_ZPR(empty_row, empty_rows, empty_rows).GetSegmentValue(*export_batch);

    if (show_about_to)
     Log("About to print pharmacy visit statistics", false);

    *export_file << flush;
    pharmacy_stream.Next();
    visits_count++;
    pharmacy_visits_count++;

    progress_counter++;
    if (!(progress_counter % progress_interval))
    {
     labelProgressPharmacyVisits->Caption = IntToString(pharmacy_visits_count).c_str();
     richDebug->Text = string(
                       "people: " + IntToString(person_stream.GetRowNumber()) + "\n"
                       "names: " + IntToString(person_name_stream.GetRowNumber()) + " (" + IntToString(person_name_rows.size()) + ")\n"
                       "affiliations: " + IntToString(person_affiliation_stream.GetRowNumber()) + " (" + IntToString(person_affiliation_rows.size()) + ")\n"
                       "addresses: " + IntToString(person_contact_info_stream.GetRowNumber()) + " (" + IntToString(person_contact_info_rows.size()) + ")\n"
                       "visits: " + IntToString(visit_stream.GetRowNumber()) + "\n"
                       "conditions: " + IntToString(visit_condition_stream.GetRowNumber()) + " (" + IntToString(visit_condition_rows.size()) + ")\n"
                       "procedures: " + IntToString(visit_procedure_stream.GetRowNumber()) + " (" + IntToString(visit_procedure_rows.size()) + ")\n"
                       "reasons: " + IntToString(visit_reason_stream.GetRowNumber()) + " (" + IntToString(visit_reason_rows.size()) + ")\n"
                       "bills: " + IntToString(visit_bill_stream.GetRowNumber()) + " (" + IntToString(visit_bill_rows.size()) + ")\n"
                       "educations: " + IntToString(visit_education_stream.GetRowNumber()) + " (" + IntToString(visit_education_rows.size()) + ")\n"
                       "measurements: " + IntToString(visit_measurement_stream.GetRowNumber()) + " (" + IntToString(visit_measurement_rows.size()) + ")\n"
                       "pharmacy visits: " + IntToString(pharmacy_stream.GetRowNumber()) + "\n"
                       ).c_str();
     Application->ProcessMessages();
    }
   }

   if (show_about_to)
    labelCurrentState->Caption = "About to print person statistics";
   Application->ProcessMessages();

   // for debug log only
   this_person_stats["visits"] = IntToString(visit_stream.GetRowNumber() - StringToInt(this_person_stats["visits"]));
   this_person_stats["conditions"] = IntToString(visit_condition_stream.GetRowNumber() - StringToInt(this_person_stats["conditions"]));
   this_person_stats["procedures"] = IntToString(visit_procedure_stream.GetRowNumber() - StringToInt(this_person_stats["procedures"]));
   this_person_stats["reasons"] = IntToString(visit_reason_stream.GetRowNumber() - StringToInt(this_person_stats["reasons"]));
   this_person_stats["bills"] = IntToString(visit_bill_stream.GetRowNumber() - StringToInt(this_person_stats["bills"]));
   this_person_stats["educations"] = IntToString(visit_education_stream.GetRowNumber() - StringToInt(this_person_stats["educations"]));
   this_person_stats["measurements"] = IntToString(visit_measurement_stream.GetRowNumber() - StringToInt(this_person_stats["measurements"]));
   this_person_stats["pharmacy visits"] = IntToString(pharmacy_stream.GetRowNumber() - StringToInt(this_person_stats["pharmacy visits"]));
   ////Log("Exported for person: " + person_row["id"] + " (" + this_person_name + "):\n" +
   ////    MapToDelimString(this_person_stats, "   ","","\n","","","","",": "));

   person_stream.Next();
   people_count++;

   progress_counter++;
   if (!(progress_counter % progress_interval))
   {
    labelProgressVisitPeople->Caption = IntToString(people_count).c_str();
    richDebug->Text = string(
                      "people: " + IntToString(person_stream.GetRowNumber()) + "\n"
                      "names: " + IntToString(person_name_stream.GetRowNumber()) + " (" + IntToString(person_name_rows.size()) + ")\n"
                      "affiliations: " + IntToString(person_affiliation_stream.GetRowNumber()) + " (" + IntToString(person_affiliation_rows.size()) + ")\n"
                      "addresses: " + IntToString(person_contact_info_stream.GetRowNumber()) + " (" + IntToString(person_contact_info_rows.size()) + ")\n"
                      "visits: " + IntToString(visit_stream.GetRowNumber()) + "\n"
                      "conditions: " + IntToString(visit_condition_stream.GetRowNumber()) + " (" + IntToString(visit_condition_rows.size()) + ")\n"
                      "procedures: " + IntToString(visit_procedure_stream.GetRowNumber()) + " (" + IntToString(visit_procedure_rows.size()) + ")\n"
                      "reasons: " + IntToString(visit_reason_stream.GetRowNumber()) + " (" + IntToString(visit_reason_rows.size()) + ")\n"
                      "bills: " + IntToString(visit_bill_stream.GetRowNumber()) + " (" + IntToString(visit_bill_rows.size()) + ")\n"
                      "educations: " + IntToString(visit_education_stream.GetRowNumber()) + " (" + IntToString(visit_education_rows.size()) + ")\n"
                      "measurements: " + IntToString(visit_measurement_stream.GetRowNumber()) + " (" + IntToString(visit_measurement_rows.size()) + ")\n"
                      "pharmacy visits: " + IntToString(pharmacy_stream.GetRowNumber()) + "\n").c_str();
    Application->ProcessMessages();
   }
  }

  // ZTS
  // BTS
  *export_file << hl7_ZTS(visits_count, (combined) ? visits_count + people_count : visits_count).GetSegmentValue(*export_batch);
  *export_file << hl7_BTS(0, "").GetSegmentValue(*export_batch);

  // step 6
  // clear temp tables again
  if (!ExecuteStatement(dbTransaction, "delete from temp_hl7_export_people_union"))
   throw(string("Could not clear temporary people (non-distinct) table while cleaning up"));
  if (!ExecuteStatement(dbTransaction, "delete from temp_hl7_export_people"))
   throw(string("Could not clear temporary people table while cleaning up"));
  if (!ExecuteStatement(dbTransaction, "delete from temp_hl7_export_tasks"))
   throw(string("Could not clear temporary visits (tasks) table while cleaning up"));
  if (!ExecuteStatement(dbTransaction, "delete from temp_hl7_export_pharm"))
   throw(string("Could not clear temporary visits (pharmacy) table while cleaning up"));

  // nothing went wrong, let the user know
  dbTransaction->Commit();
  answer = true;
 } catch (string e)
 {
  dbTransaction->Rollback();
  PSay("The following error occurred: " + e, m_program_error, MB_OK, e);
 } catch (...)
 {
  dbTransaction->Rollback();
  PSay("An unknown error occured (not numbered).", m_program_error, MB_OK, "caught (...)");
 }

 // clean up!
 delete dbTransaction;

 // close files
 if (combined)
 {
  combined_file.close();
 } else
 {
  people_file.close();
  visits_file.close();
 }
 log_file.close();
 Processing(false);

 // moved this here to make -sure- the file closing would happen first, in case of stupid auto-log-out
 if (answer)
  PSay("All export data written to file(s) successfully.", m_message, MB_OK);

 return;
}
//---------------------------------------------------------------------------
void __fastcall TformSpecialExportHL7ForIHS::buttonCloseClick(TObject *Sender)
{
 Close();
}
//---------------------------------------------------------------------------
void __fastcall TformSpecialExportHL7ForIHS::radioFileOptionsClick(TObject *Sender)
{
 // change enabled/disabled controls
 bool combined = radioCombined->Checked;

 _labelCombinedFile->Enabled     = combined;
 editCombinedFile->Enabled       = combined;
 buttonCombinedFile->Enabled     = combined;
 buttonCombinedSettings->Enabled = combined;

 _labelPeopleFile->Enabled     = !combined;
 editPeopleFile->Enabled       = !combined;
 buttonPeopleFile->Enabled     = !combined;
 buttonPeopleSettings->Enabled = !combined;
 _labelVisitsFile->Enabled     = !combined;
 editVisitsFile->Enabled       = !combined;
 buttonVisitsFile->Enabled     = !combined;
 buttonVisitsSettings->Enabled = !combined;

 UpdateFilenames();
}
//---------------------------------------------------------------------------
void __fastcall TformSpecialExportHL7ForIHS::radioDataOptionsClick(TObject *Sender)
{
 UpdateFilenames();
}
//---------------------------------------------------------------------------
void TformSpecialExportHL7ForIHS::UpdateFilenames()
{
 if (radioCombined->Checked)
   editCombinedFile->Text = "AST" + AnsiString(combined_batch.sending_facility.c_str()) + AnsiString(RightNow(true).c_str())
   + "_" + (radioNDW->Checked ? "inc" : "full") + "_combined" + ".BDW";
 else if (radioSeparated->Checked)
 {
   editPeopleFile->Text = "AST" + AnsiString(people_batch.sending_facility.c_str()) + AnsiString(RightNow(true).c_str())
   + "_" + (radioNDW->Checked ? "inc" : "full") + "_registry" + ".BDW";
   editVisitsFile->Text = "AST" + AnsiString(visits_batch.sending_facility.c_str()) + AnsiString(RightNow(true).c_str())
   + "_" + (radioNDW->Checked ? "inc" : "full") + "_encounters" + ".BDW";
 }
};
//---------------------------------------------------------------------------
void __fastcall TformSpecialExportHL7ForIHS::buttonCombinedFileClick(TObject *Sender)
{
 // file (combined)
 if (dialogSave->Execute())
  editCombinedFile->Text = dialogSave->FileName;
}
//---------------------------------------------------------------------------
void __fastcall TformSpecialExportHL7ForIHS::buttonPeopleFileClick(TObject *Sender)
{
 // file (people)
 if (dialogSave->Execute())
  editPeopleFile->Text = dialogSave->FileName;
}
//---------------------------------------------------------------------------
void __fastcall TformSpecialExportHL7ForIHS::buttonVisitsFileClick(TObject *Sender)
{
 // file (visit)
 if (dialogSave->Execute())
  editVisitsFile->Text = dialogSave->FileName;
}
//---------------------------------------------------------------------------
void __fastcall TformSpecialExportHL7ForIHS::buttonCombinedSettingsClick(TObject *Sender)
{
 // settings (combined)
 FORM_TRY
  TformSpecialHL7Settings * settings = new TformSpecialHL7Settings(NULL, combined_batch);
  settings->ShowTaskModal(this);
 FORM_CATCH
}
//---------------------------------------------------------------------------
void __fastcall TformSpecialExportHL7ForIHS::buttonPeopleSettingsClick(TObject *Sender)
{
 // settings (people)
 FORM_TRY
  TformSpecialHL7Settings * settings = new TformSpecialHL7Settings(NULL, people_batch);
  settings->ShowTaskModal(this);
 FORM_CATCH
}
//---------------------------------------------------------------------------
void __fastcall TformSpecialExportHL7ForIHS::buttonVisitsSettingsClick(TObject *Sender)
{
 // settings (visit)
 FORM_TRY
  TformSpecialHL7Settings * settings = new TformSpecialHL7Settings(NULL, visits_batch);
  settings->ShowTaskModal(this);
 FORM_CATCH
}
//---------------------------------------------------------------------------
void __fastcall TformSpecialExportHL7ForIHS::FormClose(TObject *Sender, TCloseAction &Action)
{
  if (floating_log_window) delete floating_log_window;
}
//---------------------------------------------------------------------------
void TformSpecialExportHL7ForIHS::Log(string message, bool update_caption)
{
  Pstamp now_stamp = CurrentPstamp();
  string full_message = "[" + now_stamp.OutStamp(datefirst | ymd | yyyy | left_zeros | military) + "]" +
                        "[" + GetTimeSince(now_stamp, last_message_stamp) + "] " + StripTrailing(StripTrailing(message, "\n"), "\r") + "\r\n";
  log_file << full_message;
  last_message_stamp = now_stamp;
  if (editLog)
  {
    if (editLog->Lines->Count > 500)
      editLog->Lines->Delete(0);
    editLog->Lines->Add(ReplaceStringWithString(full_message, "\r", "").c_str());
  }
  if (update_caption) labelCurrentState->Caption = message.c_str();
  Application->ProcessMessages();
}
//---------------------------------------------------------------------------

