//---------------------------------------------------------------------------

#include <vcl.h>
#pragma hdrstop

#include "codeSpecialExportListview.h"
#include "derivedheaders.h"
//---------------------------------------------------------------------------
#pragma package(smart_init)
#pragma link "codeSpecialGeneric"
#pragma resource "*.dfm"
TformSpecialExportListview *formSpecialExportListview;
//---------------------------------------------------------------------------
__fastcall TformSpecialExportListview::TformSpecialExportListview(TComponent* Owner, classFilterObject * p_filter)
        : TformSpecialGeneric(Owner)
{
 filter_object = p_filter;
 // make sure variables are valid
 is_selecting = false;
 is_closing = false;
 export_data = NULL;
 column_list = NULL;

 // setup buttons
 buttonShowUp->Enabled = false;
 buttonShowDown->Enabled = false;
 buttonSortUp->Enabled = false;
 buttonSortDown->Enabled = false;

 // setup combo boxes
 vector<string> show_styles;
 vector<string> sort_styles;
 ShowTypeNames(show_styles);
 SortTypeNames(sort_styles);
 for (vector<string>::iterator i_item = show_styles.begin(); i_item != show_styles.end(); i_item++)
  comboShowStyle->Items->Add(i_item->c_str());
 for (vector<string>::iterator i_item = sort_styles.begin(); i_item != sort_styles.end(); i_item++)
  comboSortStyle->Items->Add(i_item->c_str());

 // setup listviews
 if (filter_object)
 {
  // get data from filter, prepare export object
  export_data = new column_descriptions(filter_object); // this lets GetData() work later -- originally, we passed -just- the dataset here, not the whole filter
  export_data->ImportSettings(p_filter);                // this gets the current column show/hide settings (user's choice)

  vector<column_description *> all_columns;
  vector<column_description *> show_columns;
  vector<column_description *> sort_columns;

  export_data->GetAllColumns(all_columns);
  export_data->GetShowColumns(show_columns);
  export_data->GetSortColumns(sort_columns);

  // prepare main listview
  column_list = new edit_listview();
  column_list->setup_listview(listList, false);
  column_list->setup_callbacks(this, InformForm(&(TformSpecialExportListview::ListSelect)), NULL);
  column_list->add_or_modify_column("col_name", "Name", true, 0, 100, as_text, rawString);
  column_list->add_or_modify_column("is_show", "Show", true, 1, 50, as_text, prettyBoolString);
  column_list->add_or_modify_column("show_type", "Format as", true, 2, 100, as_text, rawString);
  column_list->add_or_modify_column("is_sort", "Sort", true, 3, 50, as_text, prettyBoolString);
  column_list->add_or_modify_column("sort_type", "Sort as", true, 4, 75, as_text, rawString);
  column_list->add_or_modify_column("sort_dir", "Sort order", true, 5, 75, as_text, rawString);

  vector<map<string, string> > list_data;
  map<string, string>          list_row;
  for (vector<column_description *>::iterator i_col_desc = all_columns.begin(); i_col_desc != all_columns.end(); i_col_desc++)
  {
   list_row["col_name"]  = (*i_col_desc)->official_name;
   list_row["is_show"]   = (*i_col_desc)->is_show ? "1" : "0";
   list_row["show_type"] = ShowTypeName((*i_col_desc)->show_method);
   list_row["is_sort"]   = (*i_col_desc)->is_sort ? "1" : "0";
   list_row["sort_type"] = SortTypeName((*i_col_desc)->sort_method);
   list_row["sort_dir"]  = (*i_col_desc)->is_ascending ? "Ascending" : "Descending";
   list_row["column_pointer"] = IntToString((int)(*i_col_desc));
   list_data.push_back(list_row);
  }
  column_list->import_data(list_data);

  // show listview
  TListItem * item;
  for (vector<column_description *>::iterator i_col_desc = show_columns.begin(); i_col_desc != show_columns.end(); i_col_desc++)
  {
   item = listShow->Items->Add();
   item->Caption = (*i_col_desc)->official_name.c_str();
   item->Data = (void *)((*i_col_desc));
  }

  // sort listview
  for (vector<column_description *>::iterator i_col_desc = sort_columns.begin(); i_col_desc != sort_columns.end(); i_col_desc++)
  {
   item = listSort->Items->Add();
   item->Caption = (*i_col_desc)->official_name.c_str();
   item->Data = (void *)((*i_col_desc));
  }

  // current listview -- practically entirely lifted from codeFilterPreferences' constructor
  listCurrent->SmallImages = imagesCurrent;
  int i = 0;
  TListItem *rows;
  if (filter_object->CanFilter())
  {
   //go through the vector listing the current types and current columns so the user can choose
   //his/her preferences
   for (vector<classCurrent>::iterator itr = filter_object->CurrentVec.begin();
        itr != filter_object->CurrentVec.end(); itr++, i++)
   {
    rows = listCurrent->Items->Add();
    rows->ImageIndex = i + 3; // current specs are in order 0-5, images are 3-8
    rows->Caption = itr->currentItem.c_str();
    rows->Checked = itr->showCurrent;
   }
  } else
  {
   listCurrent->Enabled = false;
   groupCurrent->Enabled = false;
  }
  filter_sql = filter_object->GetSQLStatement();
  //DebugSql(NULL, &filter_sql, libraries, 9);
 }
}
//---------------------------------------------------------------------------
void __fastcall TformSpecialExportListview::buttonCloseClick(TObject *Sender)
{
 Close();
}
//---------------------------------------------------------------------------
void __fastcall TformSpecialExportListview::buttonExportClick(TObject *Sender)
{
 // save!
 string filename;
 vector<string> valid_current_id;

 if (radioComma->Checked)
 {
  dialogSaveData->Filter = "Comma-separated files (*.csv)|*.csv";
  dialogSaveData->DefaultExt = ".csv";
 } else
 {
  dialogSaveData->Filter = "Plain-text fixed-width files (*.txt)|*.txt";
  dialogSaveData->DefaultExt = ".txt";
 }

 if (dialogSaveData->Execute())
 {
  filename = dialogSaveData->FileName.c_str();

  if (listCurrent->Enabled)
   for (int i = 0; i < listCurrent->Items->Count; i++)
    if (listCurrent->Items->Item[i]->Checked)
     valid_current_id.push_back(IntToString(i));

  if (export_data->ExportData(filename, (radioComma->Checked ? export_csv : export_txt), checkHeaders->Checked, valid_current_id))
  {
   if (checkOpenFile->Checked)
   {
    AnsiString x = filename.c_str();
    ShellExecute(0, "open", static_cast<const char*>(x.data()), NULL, NULL, SW_SHOW);
   }
   if (checkSaveSql->Checked)
   {
     string sql_filename;
     unsigned int ext_at = filename.find_last_of(".", filename.length()-1);
     if(ext_at != string::npos)
       sql_filename = filename.substr(0, ext_at) + ".sql";
     else
       sql_filename = filename + ".sql";

     ofstream sql_file;
     sql_file.open(sql_filename.c_str(), ios::trunc);
     if (sql_file)
     {
       sql_file << filter_sql << endl;
       sql_file.close();
     }
   }
  } else
  {
   // failed to save
  }
 }
}
//---------------------------------------------------------------------------
void __fastcall TformSpecialExportListview::FormClose(TObject *Sender,TCloseAction &Action)
{
 is_closing = true;

 if (export_data)
 {
  delete export_data;
  export_data = NULL;
 }

 if (column_list)
 {
  delete column_list;
  column_list = NULL;
 }

 Action = caFree;
 TformSpecialGeneric::FormClose(Sender, Action);
}
//---------------------------------------------------------------------------
void __fastcall TformSpecialExportListview::buttonShowUpClick(TObject *Sender)
{
 // move column up (showing)
 TListItem * i_this = listShow->Selected;
 TListItem * i_that = NULL;
 if (i_this)
 {
  if (i_this->Index > 0)
  {
   i_that = listShow->Items->Item[i_this->Index - 1];
   column_description * c_that = (column_description *)(i_that->Data);
   column_description * c_this = (column_description *)(i_this->Data);
   swap(c_this->show_order, c_that->show_order);
   i_this->Delete();
   i_this = listShow->Items->Insert(i_that->Index);
   i_this->Caption = c_this->official_name.c_str();
   i_this->Data = (void *)(c_this);
   i_this->Selected = true;
  }
 }
}
//---------------------------------------------------------------------------
void __fastcall TformSpecialExportListview::buttonShowDownClick(TObject *Sender)
{
 // move column down (showing)
 TListItem * i_this = listShow->Selected;
 TListItem * i_that = NULL;
 if (i_this)
 {
  if (i_this->Index < listShow->Items->Count - 1)
  {
   i_that = listShow->Items->Item[i_this->Index + 1];
   column_description * c_that = (column_description *)(i_that->Data);
   column_description * c_this = (column_description *)(i_this->Data);
   swap(c_this->show_order, c_that->show_order);
   i_that->Delete();
   i_that = listShow->Items->Insert(i_this->Index);
   i_that->Caption = c_that->official_name.c_str();
   i_that->Data = (void *)(c_that);
   i_this->Selected = true;
  }
 }
}
//---------------------------------------------------------------------------
void __fastcall TformSpecialExportListview::buttonSortUpClick(TObject *Sender)
{
 // move column up (sorting)
 TListItem * i_this = listSort->Selected;
 TListItem * i_that = NULL;
 if (i_this)
 {
  if (i_this->Index > 0)
  {
   i_that = listSort->Items->Item[i_this->Index - 1];
   column_description * c_that = (column_description *)(i_that->Data);
   column_description * c_this = (column_description *)(i_this->Data);
   swap(c_this->sort_order, c_that->sort_order);
   i_this->Delete();
   i_this = listSort->Items->Insert(i_that->Index);
   i_this->Caption = c_this->official_name.c_str();
   i_this->Data = (void *)(c_this);
   i_this->Selected = true;
  }
 }
}
//---------------------------------------------------------------------------
void __fastcall TformSpecialExportListview::buttonSortDownClick(TObject *Sender)
{
 // move column down (sorting)
 TListItem * i_this = listSort->Selected;
 TListItem * i_that = NULL;
 if (i_this)
 {
  if (i_this->Index < listSort->Items->Count - 1)
  {
   i_that = listSort->Items->Item[i_this->Index + 1];
   column_description * c_that = (column_description *)(i_that->Data);
   column_description * c_this = (column_description *)(i_this->Data);
   swap(c_this->sort_order, c_that->sort_order);
   i_that->Delete();
   i_that = listSort->Items->Insert(i_this->Index);
   i_that->Caption = c_that->official_name.c_str();
   i_that->Data = (void *)(c_that);
   i_this->Selected = true;
  }
 }
}
//---------------------------------------------------------------------------
void __fastcall TformSpecialExportListview::checkShowColumnClick(TObject *Sender)
{
 if (is_selecting || is_closing)
  return;

 // change checked state on column visible
 TListItem * item = listList->Selected;
 TListItem * show_item = NULL;
 map<string, string> row_map;

 if (item)
 {
  if (column_list->get_current_row(row_map))
  {
   RecurseEnabled(panelShowColumn, checkShowColumn->Checked);

   column_description * c_this = (column_description *)(StringToInt(row_map["column_pointer"]));
   if (checkShowColumn->Checked)
   {
    vector<column_description *> show_columns;
    if (export_data->GetShowColumns(show_columns))
    {
     // goes at the end
     c_this->show_order = (show_columns.back())->show_order + 1;
    } else
    {
     // no other shown columns, so it gets first dibs
     c_this->show_order = 0;
    }
    // regardless, visible, and at the end of the list
    c_this->is_show = true;
    show_item = listShow->Items->Add();
    show_item->Caption = c_this->official_name.c_str();
    show_item->Data = (void *)(c_this);
   } else
   {
    c_this->is_show    = false;
    c_this->show_order = -1;
    show_item = listShow->FindData(0, (void *)(c_this), true, false);
    if (show_item)
     show_item->Delete();
   }
   row_map["is_show"] = c_this->is_show ? "1" : "0";
   column_list->add_or_modify_row_data(row_map, false);

   if (!(checkShowColumn->Checked))
   {
    checkSortColumn->Checked = false;
    checkSortColumn->Enabled = false;
    RecurseEnabled(panelSortColumn, false);
   } else
   {
    checkSortColumn->Enabled = true;
   }
  }
 }
}
//---------------------------------------------------------------------------
void __fastcall TformSpecialExportListview::checkSortColumnClick(TObject *Sender)
{
 if (is_selecting || is_closing)
  return;

 // change checked state on column visible
 TListItem * item = listList->Selected;
 TListItem * sort_item = NULL;
 map<string, string> row_map;

 if (item)
 {
  if (column_list->get_current_row(row_map))
  {
   RecurseEnabled(panelSortColumn, checkSortColumn->Checked);

   column_description * c_this = (column_description *)(StringToInt(row_map["column_pointer"]));
   if (checkSortColumn->Checked)
   {
    vector<column_description *> sort_columns;
    if (export_data->GetSortColumns(sort_columns))
    {
     // goes at the end
     c_this->sort_order = (sort_columns.back())->sort_order + 1;
    } else
    {
     // no other sortn columns, so it gets first dibs
     c_this->sort_order = 0;
    }
    // regardless, visible, and at the end of the list
    c_this->is_sort = true;
    sort_item = listSort->Items->Add();
    sort_item->Caption = c_this->official_name.c_str();
    sort_item->Data = (void *)(c_this);
   } else
   {
    c_this->is_sort    = false;
    c_this->sort_order = -1;
    sort_item = listSort->FindData(0, (void *)(c_this), true, false);
    if (sort_item)
     sort_item->Delete();
   }
   row_map["is_sort"] = c_this->is_sort ? "1" : "0";
   column_list->add_or_modify_row_data(row_map, false);
  }
 }
}
//---------------------------------------------------------------------------
void __fastcall TformSpecialExportListview::radioSortClick(TObject *Sender)
{
 if (is_selecting || is_closing)
  return;

 // change sort order on selected column
 TListItem * item = listList->Selected;
 map<string, string> row_map;
 if (item)
 {
  if (column_list->get_current_row(row_map))
  {
   column_description * c_this = (column_description *)(StringToInt(row_map["column_pointer"]));
   c_this->is_ascending = radioSortAsc->Checked;
   row_map["sort_dir"] = c_this->is_ascending ? "Ascending" : "Descending";
   column_list->add_or_modify_row_data(row_map, false);
  }
 }
}
//---------------------------------------------------------------------------
void __fastcall TformSpecialExportListview::comboShowStyleChange(TObject *Sender)
{
 if (is_selecting || is_closing)
  return;

 // change column display format
 TListItem * item = listList->Selected;
 map<string, string> row_map;
 if (item)
 {
  if (column_list->get_current_row(row_map))
  {
   column_description * c_this = (column_description *)(StringToInt(row_map["column_pointer"]));
   c_this->show_method = ShowTypeId(comboShowStyle->Text.c_str());
   row_map["show_type"] = comboShowStyle->Text.c_str();
   column_list->add_or_modify_row_data(row_map, false);
  }
 }
}
//---------------------------------------------------------------------------
void __fastcall TformSpecialExportListview::comboSortStyleChange(TObject *Sender)
{
 if (is_selecting || is_closing)
  return;

 // change column sort method
 TListItem * item = listList->Selected;
 map<string, string> row_map;
 if (item)
 {
  if (column_list->get_current_row(row_map))
  {
   column_description * c_this = (column_description *)(StringToInt(row_map["column_pointer"]));
   c_this->sort_method = SortTypeId(comboSortStyle->Text.c_str());
   row_map["sort_type"] = comboSortStyle->Text.c_str();
   column_list->add_or_modify_row_data(row_map, false);
  }
 }
}
//---------------------------------------------------------------------------
void TformSpecialExportListview::ListSelect ()
{
 if (is_closing)
  return;

 is_selecting = true;
 TListItem * item = listList->Selected;
 if (item)
 {
  map<string, string> row_map;
  if (column_list->get_current_row(row_map))
  {
   checkShowColumn->Enabled = true;
   checkSortColumn->Enabled = true;

  // enable, so we can set values
   checkSortColumn->Enabled = true;
   RecurseEnabled(panelShowColumn, true);
   RecurseEnabled(panelSortColumn, true);

   // set values
   checkShowColumn->Checked = row_map["is_show"] == "1";
   comboShowStyle->ItemIndex = comboShowStyle->Items->IndexOf(row_map["show_type"].c_str());
   checkSortColumn->Checked = row_map["is_sort"] == "1";
   comboSortStyle->ItemIndex = comboSortStyle->Items->IndexOf(row_map["sort_type"].c_str());
   if (row_map["sort_dir"] == "Ascending")
    radioSortAsc->Checked = true;
   else
    radioSortDesc->Checked = true;

   // disable controls they shouldn't have
   if (!checkShowColumn->Checked)
   {
    checkSortColumn->Checked = false;
    checkSortColumn->Enabled = false;
   }
   RecurseEnabled(panelShowColumn, checkShowColumn->Checked);
   RecurseEnabled(panelSortColumn, checkSortColumn->Checked);
  }
 } else
 {
  checkShowColumn->Checked = false;
  checkSortColumn->Checked = false;

  RecurseEnabled(panelShowColumn, false);
  RecurseEnabled(panelSortColumn, false);

  checkShowColumn->Enabled = false;
  checkSortColumn->Enabled = false;
 }
 is_selecting = false;
}
//---------------------------------------------------------------------------
void __fastcall TformSpecialExportListview::listShowSelectItem(
      TObject *Sender, TListItem *Item, bool Selected)
{
 if (is_closing)
  return;

 // select show column
 if (listShow->Selected)
 {
  column_list->select_items_where("column_pointer", IntToString(((int)(listShow->Selected->Data))));
  buttonShowUp->Enabled = true;
  buttonShowDown->Enabled = true;
 } else
 {
  buttonShowUp->Enabled = false;
  buttonShowDown->Enabled = false;
 }
}
//---------------------------------------------------------------------------
void __fastcall TformSpecialExportListview::listSortSelectItem(
      TObject *Sender, TListItem *Item, bool Selected)
{
 if (is_closing)
  return;

 // select sort column
 if (listSort->Selected)
 {
  column_list->select_items_where("column_pointer", IntToString(((int)(listSort->Selected->Data))));
  buttonSortUp->Enabled = true;
  buttonSortDown->Enabled = true;
 } else
 {
  buttonSortUp->Enabled = false;
  buttonSortDown->Enabled = false;
 }
}
//---------------------------------------------------------------------------
void __fastcall TformSpecialExportListview::FormShow(TObject *Sender)
{
 TformSpecialGeneric::FormShow(Sender);
 ListSelect();
}
//---------------------------------------------------------------------------
void __fastcall TformSpecialExportListview::menuCSVallClick(TObject *Sender)
{
 vector<map<string,string> > all_data = filter_object->GetAllData(false, false);
 vector<string> the_column;
 map<string,string> selected_row;
 column_list->get_current_row(selected_row);
 VecMapColumnToVec(all_data, the_column, selected_row["col_name"]);
 ViewSql(VecToCommaString(the_column));
}
//---------------------------------------------------------------------------
void __fastcall TformSpecialExportListview::menuCSVselectedClick(TObject *Sender)
{
 vector<map<string,string> > all_data = filter_object->GetAllData(false, true);
 vector<string> the_column;
 map<string,string> selected_row;
 column_list->get_current_row(selected_row);
 VecMapColumnToVec(all_data, the_column, selected_row["col_name"]);
 string selected = VecToCommaString(the_column);
 if (selected == "")
   selected = "Either no rows are selected, or there is no \"id\" column in the dataset.";
 ViewSql(selected);
}
//---------------------------------------------------------------------------

