//---------------------------------------------------------------------------

#include <vcl.h>
#pragma hdrstop

#include "codeSpecialFDADrug.h"
#include "util.h"
//---------------------------------------------------------------------------
#pragma package(smart_init)
#pragma link "codeSpecialGeneric"
#pragma link "codeRubberContainer"
#pragma resource "*.dfm"
TformSpecialFDADrug *formSpecialFDADrug;
//---------------------------------------------------------------------------
__fastcall TformSpecialFDADrug::TformSpecialFDADrug(TComponent* Owner, const string &p_listing_id)
        : TformSpecialGeneric(Owner)
{
  listing_id = p_listing_id;
}
//---------------------------------------------------------------------------
void __fastcall TformSpecialFDADrug::btnCloseClick(TObject *Sender)
{
  Close();        
}
//---------------------------------------------------------------------------
// PutDataOnScreen - Connects to the FDA drug database and then displays all information
void TformSpecialFDADrug::PutDataOnScreen()
{
  TIBTransaction *dbLocalTransaction = NULL;

  try
  {
    ConnectToDatabase(dbFDADatabase, "10.1.1.109:/opt/interbase/db/fdadrugs.gdb", "ZEUS", "argon");

    // Open transaction
    dbLocalTransaction = new TIBTransaction(NULL);
    StdTransaction(dbLocalTransaction, dbFDADatabase);

    // Get the basic data
    string basic_sql =
      "select listing_names.item listing_name, fda_listings.id fda_id, fda_listings.prodcode, "
      "listing_strengths.item listing_str, listing_units.item listing_unit, fda_listings.rx_otc, fda_firms.firm_name, "
      "(select first 1 schedule_num from fda_schedules where listing_id = fda_listings.id order by schedule_num desc) dea_schedule "
      "from fda_listings "
      "left join listing_names on fda_listings.name_id = listing_names.id "
      "left join listing_strengths on fda_listings.strength_id = listing_strengths.id "
      "left join listing_units on fda_listings.listing_unit_id = listing_units.id "
      "left join fda_firms on fda_listings.firm_id = fda_firms.id "
      "where fda_listings.id = '" + listing_id + "' ";

    map<string, string> fda_record;
    if(SqlToMap(dbLocalTransaction, basic_sql, fda_record))
    {
      editName->Text         = PrettyString(fda_record["listing_name"]).c_str();
      editManufacturer->Text = PrettyString(fda_record["firm_name"   ]).c_str();
      editUnits->Text        = LowercaseString(fda_record["listing_unit"]).c_str();
      editFDAid->Text        = fda_record["fda_id"      ].c_str();
      editFDACode->Text      = fda_record["prodcode"    ].c_str();
      editStrength->Text     = fda_record["listing_str" ].c_str();
      editRxOTC->Text        = fda_record["rx_otc"      ].c_str();
      editDEASchedule->Text  = fda_record["dea_schedule"].c_str();
    }

    // Populate Dose Forms Listview
    string dose_sql =
      "select dosage_types.item dosage_name "
      "from fda_doseforms "
      "left join dosage_types on fda_doseforms.dosage_types_id = dosage_types.id "
      "where fda_doseforms.listing_id = '" + listing_id + "' "
      "order by dosage_types.item ";
    vector<map<string, string > > doseforms;
    if(SqlToVecMap(dbLocalTransaction, dose_sql, doseforms))
    {
      // Add columns
      listDoseForms->Columns->Clear();
      TListColumn *new_col = listDoseForms->Columns->Add();
      new_col->Width = 190;
      new_col->Caption = "Type";

      // Add items
      listDoseForms->Items->Clear();
      for(size_t i = 0; i < doseforms.size(); i++)
      {
        // Add to list box
        TListItem *new_item = listDoseForms->Items->Add();
        new_item->Caption = PrettyString(doseforms[i]["dosage_name"]).c_str();
      }
    }

    // Populate Dose Routes Listview
    string routes_sql =
      "select route_names.item route_name, fda_routes.route_id "
      "from fda_routes "
      "left join route_names on fda_routes.route_name_id = route_names.id "
      "where fda_routes.listing_id = '" + listing_id + "' "
      "order by route_names.item ";
    vector<map<string, string> > doseroutes;
    if(SqlToVecMap(dbLocalTransaction, routes_sql, doseroutes))
    {
      // Add columns
      listDoseRoutes->Columns->Clear();
      TListColumn *new_col = listDoseRoutes->Columns->Add();
      new_col->Width = 110;
      new_col->Caption = "Route";
      TListColumn *new_col2 = listDoseRoutes->Columns->Add();
      new_col2->Width = 80;
      new_col2->Caption = "FDA No.";

      listDoseRoutes->Items->Clear();
      for(size_t i = 0; i < doseroutes.size(); i++)
      {
        TListItem *new_item = listDoseRoutes->Items->Add();
        new_item->Caption = PrettyString(doseroutes[i]["route_name"]).c_str();
        new_item->SubItems->Add(doseroutes[i]["route_id"].c_str());
      }
    }

    // Populate Packaging Listview
    string packaging_sql =
      "select fda_packages.pkgcode, package_sizes.item pkg_size, package_types.item pkg_type "
      "from fda_packages "
      "left join package_sizes on fda_packages.package_size_id = package_sizes.id "
      "left join package_types on fda_packages.package_type_id = package_types.id "
      "where fda_packages.listing_id = '" + listing_id + "' "
      "order by package_types.item, package_sizes.item ";
    vector<map<string, string> > packages;
    if(SqlToVecMap(dbLocalTransaction, packaging_sql, packages))
    {
      // Add columns
      listPackaging->Columns->Clear();
      TListColumn *new_col = listPackaging->Columns->Add();
      new_col->Width = 80;
      new_col->Caption = "FDA Code";
      TListColumn *new_col2 = listPackaging->Columns->Add();
      new_col2->Width = 80;
      new_col2->Caption = "Size";
      TListColumn *new_col3 = listPackaging->Columns->Add();
      new_col3->Width = 250;
      new_col3->Caption = "Type";

      // Add items
      listPackaging->Items->Clear();
      for(size_t i = 0; i < packages.size(); i++)
      {
        TListItem *new_item = listPackaging->Items->Add();
        new_item->Caption = packages[i]["pkgcode"].c_str();
        new_item->SubItems->Add(packages[i]["pkg_size"].c_str());
        new_item->SubItems->Add(LowercaseString(packages[i]["pkg_type"]).c_str());
      }
    }

    // Populate Fomulations Listview
    string formulas_sql =
      "select fda_formulations.strength, formula_units.item unit_name, formula_names.item ingredient "
      "from fda_formulations "
      "left join formula_units on fda_formulations.unit_id = formula_units.id "
      "left join formula_names on fda_formulations.formula_name_id = formula_names.id "
      "where fda_formulations.listing_id = '" + listing_id + "' "
      "order by formula_names.item ";
    vector<map<string, string> > formulas;
    if(SqlToVecMap(dbLocalTransaction, formulas_sql, formulas))
    {
      // Add columns
      listFormulation->Columns->Clear();
      TListColumn *new_col = listFormulation->Columns->Add();
      new_col->Width = 80;
      new_col->Caption = "Strengths";
      TListColumn *new_col2 = listFormulation->Columns->Add();
      new_col2->Width = 100;
      new_col2->Caption = "Units";
      TListColumn *new_col3 = listFormulation->Columns->Add();
      new_col3->Width = 230;
      new_col3->Caption = "Ingredient";

      // Add items
      listFormulation->Items->Clear();
      for(size_t i = 0; i < formulas.size(); i++)
      {
        TListItem *new_item = listFormulation->Items->Add();
        new_item->Caption = formulas[i]["strength"].c_str();
        new_item->SubItems->Add(LowercaseString(formulas[i]["unit_name"]).c_str());
        new_item->SubItems->Add(LowercaseString(formulas[i]["ingredient"]).c_str());
      }
    }
  }
  catch(...)
  {
    PSay("Error connecting to or querying the FDA drug database", m_database_error, MB_OK);
  }

  if(dbLocalTransaction)
  {
    if(dbLocalTransaction->Active)
      dbLocalTransaction->Rollback();
    delete dbLocalTransaction;
  }

  // Disconnect from the FDA drugs database if we connected to it.
  if(dbFDADatabase->TestConnected())
    dbFDADatabase->Close();
}
//---------------------------------------------------------------------------
void __fastcall TformSpecialFDADrug::FormShow(TObject *Sender)
{
  PutDataOnScreen();
  TformSpecialGeneric::FormShow(Sender);
}
//---------------------------------------------------------------------------
