//---------------------------------------------------------------------------

#include <vcl.h>
#pragma hdrstop
#pragma warn -8057

#include "codeSpecialGeneric.h"
#include "codeDrawerHelp.h"

extern list<TformSpecialGeneric*> generic_specials;
extern map<string, int> form_count_map;
TformSpecialGeneric::TGetDBOverrideSection TformSpecialGeneric::funcGetDBOverrideSection;
//---------------------------------------------------------------------------
#pragma package(smart_init)
#pragma resource "*.dfm"
//---------------------------------------------------------------------------
// Used by the system menu
#define SysMenu_Database                0x1000
#define SysMenu_Admin_ShowInfoBar       0x1001
#define SysMenu_Admin_HideInfoBar       0x1002
const unsigned int SysMenu_Admin_Debug_Libraries[10] = { 0x1003, 0x1004, 0x1005, 0x1006, 0x1007, 0x1008, 0x1009, 0x100A, 0x100B, 0x100C };
const unsigned int SysMenu_Admin_Debug_Generic[10]   = { 0x100D, 0x100E, 0x100F, 0x1010, 0x1011, 0x1012, 0x1013, 0x1014, 0x1015, 0x1016 };
const unsigned int SysMenu_Admin_Debug_Other[10]     = { 0x1017, 0x1018, 0x1019, 0x101A, 0x101B, 0x101C, 0x101D, 0x101E, 0x101F, 0x1020 };
//---------------------------------------------------------------------------
extern map<string, int> form_running_count_map;
void inc_dec_running_count(const string & form_name, bool inc)
{
 int a = inc ? +1 : -1;
 map<string, int>::iterator i = form_running_count_map.find(form_name);
 if (i == form_running_count_map.end())
  form_running_count_map[form_name] = a;
 else
 {
  i->second += a;
  if (!(i->second))
   form_running_count_map.erase(i);
 }
 return;
}
void inc_running_count(const string & form_name)
{
 inc_dec_running_count(form_name, true);
}
void dec_running_count(const string & form_name)
{
 inc_dec_running_count(form_name, false);
}
//---------------------------------------------------------------------------
// jhaley 20100831: Despite all guarantees to the contrary in the VCL documentation, forms can be sent events during the
// process of destruction. There is virtually no way to stop this, as it's a flaw in the framework - I've verified the events
// are not coming from Prometheus' code itself. To work around this problem, I am keeping a global list of pointers to forms
// that have been destroyed. Forms that have been destroyed will not have any event handlers executed for them that access
// fields such as DrawerOwner, etc.
static set<TformSpecialGeneric *> destroyed_forms;
static bool IsDestroyedForm(TformSpecialGeneric *form, const char *func, int line)
{
  bool return_code = destroyed_forms.find(form) != destroyed_forms.end();

  if (return_code) // woops!
  {
    char ptrbuff[33];              // could also use IntToHexString()
    itoa((int)form, ptrbuff, 16);
    string message = string("TformSpecialGeneric::") + func + "@" + IntToString(line) + ": using destroyed form 0x" + ptrbuff;
    LogIt("destroyedforms.log", message);
  }

  return return_code;
}
//---------------------------------------------------------------------------
__fastcall TformSpecialGeneric::TformSpecialGeneric(TComponent* Owner, TformSpecialGeneric * pDrawerOwner) : drawers(), TForm(Owner)
{
  LogIt("specialgeneric.log", "constructor entry");
  ShortString temp_short = this->ClassName();
  AnsiString temp_ansi(temp_short);
  class_name = temp_ansi.c_str(); // this stuff should be first so the debug message is sent as early as possible...
  if(debug_level[generic_forms]>=2 && message_router) message_router->SendMessageToAll("debug", "Launching form " + class_name);

  // jhaley: In case the memory allocator reuses a memory block, we could see the same pointer again that we saw earlier. We shouldn't
  // continue to treat a form as erased if that address passes through here again. So far I haven't seen this happen though.
  destroyed_forms.erase(this);

  // jhaley 20100831: We can also apparently get events before the constructor has run. Use the generic_form_state variable to see if
  // we've reached the constructor yet or not. I assume that everything from here down is safe to run on the form at this point...
  generic_form_state = FORM_STATE_CONSTRUCTED;

  in_task_modal_loop = false;

  // not gonna change the name, but ... just FYI, this is what it does. wooha.
  ImageList = imagesRecordTypes;
  IconIndex = -1;

  // EMBEDDED FORMS ONLY
  // 1 - we're using Owner here to specify Parent, effectively.
  // 2 - if this form is going to be embedded in another form, we'll go ahead and set it to alClient, remove all constraints, and just let it flow.
  //     because of this, you'd better be pretty sure your form can resize correctly in all directions. this does avoid scrollbars (or the need for them) in the host window.
  // 3 - the toolbar gets eliminated to make it more space-efficient. this is a hack for now.
  //     most likely, we should move to having forms eliminate their own toolbar if they know they're embedded, if they don't use it.
  //     embedded toolbars look funky, so there's the prettiness issue.
  // 4 - also, the caption bar won't be shown if we're embedded, and the [?] is set to not show if that's the case. (otherwise it shows in a very -wrong- place, inside the window.)
  if (Owner)
  {
   this->Constraints->MaxHeight = 0;
   this->Constraints->MinHeight = 0;
   this->Constraints->MaxWidth = 0;
   this->Constraints->MinWidth = 0;
   this->BorderStyle = bsNone;       // fairly safe, all considered. who needs to resize, anyway?
   this->toolsStandard->Visible = false; // TODO: not sure if this is good.
   this->Parent = (TWinControl*)Owner; // some magic. please never screw this up.
   this->Align = alClient; // see? resizing is for wimps. *cough*
  }

  // stuff that was here, and now runs first, continues here.
  form_count_map[class_name]++;
  inc_running_count(class_name);
  if (!(this->Parent))
   generic_specials.push_front(this);
  last_active_control = NULL;

 /*
  if (!(this->Parent))
   PSay("Creating " + class_name, m_debug, MB_OK);
 */

  sizegrip_dragging = false;

  is_drawer_destroyable = false;
  is_drawer_expanded = false;
  is_drawer_attached = true;
  drawer_position = akRight;
  active_count = 0;
  disable_count = 0;

  ResetHelpButtonRect(this->Width);

  if(optionsPermissions["admin.glass"]["enabled"] == "true" && optionsPreferences["special_generic"]["glass"] == "true")
  {
    // jhaley 20110602: Disabling the help button enables Aero glass on all windows. Unfortunately end-users can't see
    // it because they need the help button enabled :(
    HelpButtonVisible = false;
    HelpButtonEnabled = false;
  }
  else
  {
    // debugging, should be based on whether or not topics have been registered (which means functions for doing so
    // need to be ready and working, etc.) // or rather, make it always visible, but change "enabled"? but then,
    // hopefully it'll always be enabled ...
    HelpButtonVisible = true;
    HelpButtonEnabled = true;
  }
  is_help_button_captured = false;
  is_help_button_pushed   = false;
  help_drawer = NULL;

  DrawerOwner = pDrawerOwner;
  AddSysMenuItems();

  // I'm hoping this bit of code will fix the issue we have of Prometheus not handling Windows themes well.  The XP themes in particular create bigger windows, mostly
  // due to the title bar height.  27 and 8 below are the values I get for border sizes vertically and horizontally, respectively, and we'll just expand the window
  // to accomodate if their values are larger....
  // Tested A-OK.  I think we have a weiner.  I even tested drawers, at Philip's request.  :-D
  int delta_height = this->Height - this->ClientHeight;
  int delta_width  = this->Width - this->ClientWidth;
  int designed_height = 27; // TODO -c MAGIC_NUMBER
  int designed_width = 8; // TODO -c MAGIC_NUMBER

  if (delta_height > designed_height)
  {
    if (this->Constraints->MinHeight > 0)
      this->Constraints->MinHeight += (delta_height - designed_height);
    if (this->Constraints->MaxHeight > 0)
      this->Constraints->MaxHeight += (delta_height - designed_height);
    this->Height += (delta_height - designed_height);
  }
  if (delta_width > designed_width)
  {
    if (this->Constraints->MinWidth > 0)
      this->Constraints->MinWidth += (delta_width - designed_width);
    if (this->Constraints->MaxWidth > 0)
      this->Constraints->MaxWidth += (delta_width - designed_width);
    this->Width += (delta_width - designed_width);
  }

 /**/
  bevelStandardB->Top = toolsStandard->Top - 1;
  toolsStandard->AutoSize = true;
  toolsStandard->EdgeBorders = toolsStandard->EdgeBorders >> ebTop >> ebRight >> ebLeft >> ebBottom;
 /**/

  panelInfoBar->Color = title_bar_color;
  ignore_info_panel_resize = false;
  info_bar_height_before_resize = panelInfoBar->Height; // should start out 0

  LogIt("specialgeneric.log", "constructor exit");
}
//---------------------------------------------------------------------------
// jhaley 20100830: Added a destructor to support tracking deleted forms. Doing this in FormClose is
// a little bit too early and will cause false positives in FormDeactivate and WindowMoved.
__fastcall TformSpecialGeneric::~TformSpecialGeneric()
{
  LogIt("specialgeneric.log", "destructor entry");
  destroyed_forms.insert(this);

  // jhaley 20100914: When certain exceptions (commonly IBException while initializing a CBDD or
  // other filtered listview) occur during the construction of a form, VCL will execute the destructor
  // but does *not* send the form an OnClose event first, as it was never opened (I suppose that's
  // the logic). Leaving such forms in the generic_specials list will cause the Prometheus main menu
  // to try to close them on shutdown. No deleted forms should ever be in the list.
  generic_specials.remove(this);
  LogIt("specialgeneric.log", "destructor exit");
}
//---------------------------------------------------------------------------
void __fastcall TformSpecialGeneric::CreateParams(TCreateParams &Params)
{
  LogIt("specialgeneric.log", "CreateParams(...) entry");
  TForm::CreateParams(Params);
  if (optionsPreferences["windows"]["to_taskbar"] == "true")
    Params.ExStyle |= WS_EX_APPWINDOW;
}
//---------------------------------------------------------------------------
void __fastcall TformSpecialGeneric::WMSysCommand(TWMSysCommand &Msg)
{
  LogIt("specialgeneric.log", "WMSysCommand({" + IntToString(Msg.Msg) + " " + IntToString(Msg.CmdType) + " " + IntToString(Msg.XPos) + " " + IntToString(Msg.YPos) + " " + IntToString(Msg.Result) + " " + IntToString(Msg.Key) + "}) entry");
  // looks like debugging stuff (from paul?) ... not currently used (disabled in the Dispatch function below)
  if ( (Msg.CmdType & 0xFFF0) == SC_MINIMIZE)
    PSay("Minimize", m_message, MB_OK);
  else if ( (Msg.CmdType & 0xFFF0) == SC_RESTORE)
    PSay("Restore", m_message, MB_OK);

  TForm::Dispatch(&Msg);
}
//---------------------------------------------------------------------------
bool TformSpecialGeneric::CloseNow()
{
  LogIt("specialgeneric.log", "CloseNow() entry");
  Close();
  return true;
}
//---------------------------------------------------------------------------
void __fastcall TformSpecialGeneric::FormClose(TObject *Sender, TCloseAction &Action)
{
  LogIt("specialgeneric.log", "FormClose(...) entry");
  if (IsDestroyedForm(this, "FormClose", __LINE__)) // jhaley: Do not run on destroyed forms!
    return;
  if (!generic_form_state) // jhaley: Do not run on unconstructed forms either.
    return;

  // tell all drawers to close
  // (take it from the top, remembering that as they close, they're going to remove themselves from our list)
  if(debug_level[generic_forms]>=2 && message_router) { ShortString temp_short = this->ClassName(); AnsiString temp_ansi(temp_short);
                                 message_router->SendMessageToAll("debug", "Closing form " + string(temp_ansi.c_str())); }
  AV_TRY
   set<TformSpecialGeneric *>::iterator itr_drawer = drawers.begin();
   while (itr_drawer != drawers.end())
   {
    (*itr_drawer)->is_drawer_destroyable = true;
    (*itr_drawer)->CloseNow();
    itr_drawer = drawers.begin();
   }
  AV_CATCH

  // remove self from drawerowner
  DrawerOwner = NULL;

  //TerminateAllFilters();

  UnregisterLinks(this);
  atlas->CancelRequestFrom(this);
 /*
  if (!(this->Parent))
   PSay("Destroying " + class_name, m_debug, MB_OK);
 */
  dec_running_count(class_name);
  if (!(this->Parent))
   generic_specials.remove(this);
  Action = caFree;
  LogIt("specialgeneric.log", "FormClose() exit");
}
//---------------------------------------------------------------------------
bool TformSpecialGeneric::IsBusy()
{
  LogIt("specialgeneric.log", "IsBusy() entry");
  // override this if you want your form to block a call to formclose on the
  // main form, in case this form needs special closing... the user can still
  // force the issue, and CloseNow() will be called...
  return false;
}
//---------------------------------------------------------------------------
void TformSpecialGeneric::Processing (bool p_is_processing)
{
  LogIt("specialgeneric.log", "Processing(" + BoolToBoolString(p_is_processing, true_false) + ") entry");
  // cannot call RecurseEnabled since some buttons might be in random states
  // not worrying much about the current mouse state ... it's ... a generic thing.
  if (p_is_processing)
  {
   if (!is_processing)
   {
    FocusAway();
    Screen->Cursor = crHourGlass;
    toolsStandard->Enabled = false;
    is_processing = true;
   }
  } else
  {
   if (is_processing)
   {
    FocusBack();
    Screen->Cursor = crDefault;
    toolsStandard->Enabled = true;
    is_processing = false;
   }
  }
  this->Update();
  return;
}
//---------------------------------------------------------------------------
bool TformSpecialGeneric::IsProcessing ()
{
  LogIt("specialgeneric.log", "IsProcessing() entry");
  return is_processing;
}
//---------------------------------------------------------------------------
int TformSpecialGeneric::PSay(string p_message, prometheus_message_types p_message_type, int p_buttons, string p_other_info)
{
  return ::PSay(p_message, p_message_type, p_buttons, class_name, p_other_info, this->Handle);
}
//---------------------------------------------------------------------------
void TformSpecialGeneric::PPretendToSay(string p_message, prometheus_message_types p_message_type, string p_other_info)
{
  ::PPretendToSay(p_message, p_message_type, class_name, p_other_info);
}
//---------------------------------------------------------------------------
void TformSpecialGeneric::FocusAway()
{
  LogIt("specialgeneric.log", "FocusAway() entry");
  if (IsVisible(toolsStandard))
  {
   last_active_control = this->ActiveControl;
   if(toolsStandard->Enabled)
     // toolsStandard->SetFocus();
     this->ActiveControl = toolsStandard;
   else
   {
     toolsStandard->Enabled = true;
     // toolsStandard->SetFocus();
     this->ActiveControl = toolsStandard;
     toolsStandard->Enabled = false;
   }
  }
}
//---------------------------------------------------------------------------
void TformSpecialGeneric::FocusBack()
{
  LogIt("specialgeneric.log", "FocusBack() entry");
  // play nice and set the focus back
  if(last_active_control)
    if(IsVisible(last_active_control) && IsEnabled(last_active_control))
      this->ActiveControl = last_active_control;
  // and ready for the next call to FocusAway()
  last_active_control = NULL;
}
//---------------------------------------------------------------------------
void TformSpecialGeneric::SwitchRadioSelection (TRadioButton * to_check)
{
  LogIt("specialgeneric.log", "SwitchRadioSelection(...) entry");
  if (!to_check->Checked)
  {
    to_check->Checked = true;
    // make sure the above doesn't get undone
    if (last_active_control && last_active_control != to_check)
    {
     // if the last_active_control is one of the other radio buttons in the group,
     // then switching focus back to it is going to cause that radio button to be checked,
     // rather than the one we want.
     // thus, we need to find, for the given control, all other radio buttons that go with it,
     // and if it's one of them, switch last_active_control to to_check ... but not otherwise (to avoid oddities)
     for (int i = 0; i < to_check->Parent->ControlCount; i++)
     {
        if (to_check->Parent->Controls[i] == last_active_control &&
            AnsiString(to_check->Parent->Controls[i]->ClassName()) == "TRadioButton")
        {
          last_active_control = to_check;
          break;
        }
      }
    }
  }
}
//---------------------------------------------------------------------------
void __fastcall TformSpecialGeneric::FormShortCut(TWMKey &Msg, bool &Handled)
{
  LogIt("specialgeneric.log", "FormShortCut(...) entry");
  // try to auto-handle anything in the toolbar.
  Handled = false;

  if(!HIBYTE(GetAsyncKeyState(VK_CONTROL)) && (Msg.CharCode == VK_F1))
  {
    Handled = true;
    if (HelpButtonVisible && HelpButtonEnabled)
      HelpButtonPressed();
  }

  if(HIBYTE(GetAsyncKeyState(VK_CONTROL)) && HIBYTE(GetAsyncKeyState(VK_MENU)) && (Msg.CharCode == VK_F1))
  {
    Handled = true;
    Processing(false);
  }

  // by default, alt+key is recognized if you use & in the caption ...
  // here we add the ability to use control+key for toolbars,
  // which might help with conflicts (alt+d for a button in the window,
  // control+d for a toolbar button -- but who says how it's resolved?)
  // thus we can make it unique for toolbars, but the alt+key code may still have problems resolving elsewhere (taborder? unsure.)

  // except this made it so control+c would be seen as a shortcut to buttons rather than default copy/paste stuff ... alt is reserved, should just resolve conflicts everywhere.
  /*
  if(HIBYTE(GetAsyncKeyState(VK_CONTROL)))
  {
    TToolButton * temp = NULL;
    for (int i = 0; i < toolsStandard->ButtonCount; i++)
    {
      temp = toolsStandard->Buttons[i];
      if (temp->Style == tbsButton || temp->Style == tbsCheck) // the ones that can do something
        if (temp->Enabled && IsVisible(temp)) // should help with the << Back button being clicked even though it's disabled (common sense)
          if (IsAccel(Msg.CharCode, temp->Caption)) // oh yeah, shoot, foot, matching ...
          {
            Handled = true;
            temp->Click();
          }
    }
  }
  */
}
//---------------------------------------------------------------------------
void TformSpecialGeneric::AtlasClear()
{
  LogIt("specialgeneric.log", "AtlasClear() entry");
  pending_atlas_notifications.clear();
}
//---------------------------------------------------------------------------
void TformSpecialGeneric::AtlasAdd(string key, string value)
{
  LogIt("specialgeneric.log", "AtlasAdd(" + key + ", " + value + ") entry");
  pending_atlas_notifications[LowercaseString(key)].insert(value);
}
//---------------------------------------------------------------------------
void TformSpecialGeneric::AtlasAdd(string table_name, string field_name, string field_value)
{
  LogIt("specialgeneric.log", "AtlasAdd(" + table_name + ", " + field_value + ", " + field_value + ") entry");
  pending_atlas_notifications[LowercaseString(table_name + "." + field_name)].insert(field_value);
}
//---------------------------------------------------------------------------
void TformSpecialGeneric::AtlasAdd(string table_name, string field_name, vector<string> & field_values)
{
  LogIt("specialgeneric.log", "AtlasAdd(" + table_name + ", {" + VecToCommaString(field_values) + "}) entry");
  for (vector<string>::iterator i = field_values.begin(); i != field_values.end(); i++)
    pending_atlas_notifications[LowercaseString(table_name + "." + field_name)].insert(*i);
}
//---------------------------------------------------------------------------
void TformSpecialGeneric::AtlasAdd(string table_name, map<string, string> & field_names_values)
{
  LogIt("specialgeneric.log", "AtlasAdd(" + table_name + ", " + MapToDelimString(field_names_values, "{", "}", "", "'", "'", "'", "'", ",") + ") entry");
  for (map<string, string>::iterator i = field_names_values.begin(); i != field_names_values.end(); i++)
    if (LikeString("%id", LowercaseString(i->first)) || LikeString("%last_updated_by", LowercaseString(i->first)))
      AtlasAdd(table_name, i->first, i->second);
}
//---------------------------------------------------------------------------
void TformSpecialGeneric::AtlasAdd(string table_name, vector<map<string, string> > & field_names_values)
{
  LogIt("specialgeneric.log", "AtlasAdd(" + table_name + ", ...) entry"); // TODO -c lazy: write a VecMapToDelimString function that does what it ought
  for (vector<map<string, string> >::iterator j = field_names_values.begin(); j != field_names_values.end(); j++)
    for (map<string, string>::iterator i = j->begin(); i != j->end(); i++)
      if (LikeString("%id", LowercaseString(i->first)) || LikeString("%last_updated_by", LowercaseString(i->first)))
        AtlasAdd(table_name, i->first, i->second);
}
//---------------------------------------------------------------------------
void TformSpecialGeneric::AtlasSend()
{
  LogIt("specialgeneric.log", "AtlasSend() entry");
  for (map<string, set<string> >::iterator i = pending_atlas_notifications.begin(); i != pending_atlas_notifications.end(); i++)
    for (set<string>::iterator j = i->second.begin(); j != i->second.end(); j++)
      atlas->Notify(i->first, *j);
  AtlasClear();
}
//---------------------------------------------------------------------------
void __fastcall TformSpecialGeneric::SetDrawerOwner (TformSpecialGeneric * Value)
{
  LogIt("specialgeneric.log", "SetDrawerOwner(...) entry");
  if (IsDestroyedForm(this, "SetDrawerOwner", __LINE__)) // jhaley: Do not run on destroyed forms!
     return;
  if (!generic_form_state) // jhaley: Do not run on unconstructed forms either.
     return;

  TformSpecialGeneric *owner = DrawerOwner;

  if (Value == owner)
    return;

  if (owner && !IsDestroyedForm(owner, "SetDrawerOwner", __LINE__)) // jhaley: Not if owner is destroyed
  {
    AV_TRY
      set<TformSpecialGeneric*>::iterator itr_drawer = owner->drawers.find(this);
      if (itr_drawer != owner->drawers.end())
        owner->drawers.erase(itr_drawer);
      owner->ResetMaximizeButton();
    AV_CATCH
  }

  if (IsDestroyedForm(Value, "SetDrawerOwner", __LINE__)) // jhaley: cannot be owned by a destroyed form
    return;

  owner = drawer_owner = Value;
  if (owner)
  {
    owner->drawers.insert(this);
    owner->ResetMaximizeButton();

    // take this one out if we're a drawer ... CreateParams doesn't seem to be catching on.
    long temp_style = GetWindowLong(this->Handle, GWL_EXSTYLE);
    SetWindowLong(this->Handle, GWL_EXSTYLE, temp_style & (~WS_EX_APPWINDOW));
    // TODO: shouldn't the reverse also happen?
  }
  WindowMoved(true);
}
//---------------------------------------------------------------------------
void __fastcall TformSpecialGeneric::SetDrawerExpanded (bool Value)
{
  LogIt("specialgeneric.log", "SetDrawerExpanded(" + BoolToBoolString(Value, true_false) + ") entry");
  if (IsDestroyedForm(this, "SetDrawerExpanded", __LINE__)) // jhaley: Do not run on destroyed forms!
    return;
  if (!generic_form_state) // jhaley: Do not run on unconstructed forms either.
    return;

  if (Value == DrawerExpanded)
    return;

  is_drawer_expanded = Value;

  if (DrawerOwner)
    DrawerOwner->ResetMaximizeButton();

  // if closed by resizing, need to make sure it won't be stupid when we re-open it. ideally ... we should have an "ideal size" for this, rather than guessing.
  if (DrawerExpanded)
  {
    if (DrawerPosition == akLeft || DrawerPosition == akRight)
      Width = max(Width, drawer_autoopen_size);
    if (DrawerPosition == akTop || DrawerPosition == akBottom)
      Height = max(Height, drawer_autoopen_size);

    if (DrawerOwner)
      GetWindowRect(DrawerOwner->Handle, &(DrawerOwner->window_rect));
  }

  // this tries to hide others. needs to be fixed. it'd be better if we could have several on the same side at once.
  // a function that "organizes child thingies" perhaps? just call it from here, let it adjust everyone to fit?
  // also ... for small windows ... particularly for the "help" window ... can we allow for drawer bigger than parent?
  // the organizing function should be called in the above function to, when adding/removing a drawer from a parent. (as the container isn't a property.)
  // TODO: disabled drawer-closing, as we can now move stuff around, etc. ... either we solve it with stacking, or we don't. or something else.
  /*
  if (DrawerOwner && DrawerExpanded)
    for (set<TformSpecialGeneric*>::iterator itr_drawer = DrawerOwner->drawers.begin(); itr_drawer != DrawerOwner->drawers.end(); itr_drawer++)
      if ((*itr_drawer) != this && (*itr_drawer)->DrawerPosition == DrawerPosition)
        (*itr_drawer)->DrawerExpanded = false;
  */

  WindowMoved(true);
  DrawerOwner->DrawerIsChanging(this, Value);
}
//---------------------------------------------------------------------------
void __fastcall TformSpecialGeneric::SetDrawerPosition (TAnchorKind Value)
{
  LogIt("specialgeneric.log", "SetDrawerPosition(...) entry");
  if (IsDestroyedForm(this, "SetDrawerPosition", __LINE__)) // jhaley: Do not run on destroyed forms!
    return;
  if (!generic_form_state) // jhaley: Do not run on unconstructed forms either.
     return;

  if (Value == DrawerPosition)
    return;

  int new_height = Height;
  int new_width  = Width;
  if (DrawerAttached)
  {
    if ((drawer_position == akLeft || drawer_position == akRight) &&
        (Value == akTop || Value == akBottom))
      new_height = Width;
    if ((drawer_position == akTop || drawer_position == akBottom) &&
        (Value == akLeft || Value == akRight))
      new_width = Height;
  }

  drawer_position = Value;
  WindowMoved(true);

  if (DrawerAttached)
  {
    Height = new_height;
    Width = new_width;
  }
}
//---------------------------------------------------------------------------
void __fastcall TformSpecialGeneric::SetDrawerAttached (bool Value)
{
  LogIt("specialgeneric.log", "SetDrawerAttached(" + BoolToBoolString(Value, true_false) + ") entry");
  if (IsDestroyedForm(this, "SetDrawerAttached", __LINE__)) // jhaley: Do not run on destroyed forms!
    return;
  if (!generic_form_state) // jhaley: Do not run on unconstructed forms either.
    return;

  if (Value == DrawerAttached)
    return;

  if (DrawerOwner && Value == false && DrawerAttached == true)
  {
    // must change this before trying to change size/position, or they'll get reset as you change 'em.
    is_drawer_attached = Value;

    switch (DrawerPosition)
    {
      case akLeft:
        Left = DrawerOwner->Left - Width;
        Top = DrawerOwner->Top;
        Constraints->MinHeight = 0;
        Constraints->MaxHeight = 0;
        Height = DrawerOwner->Height;
        break;
      case akRight:
        Left = DrawerOwner->Left + DrawerOwner->Width;
        Top = DrawerOwner->Top;
        Constraints->MinHeight = 0;
        Constraints->MaxHeight = 0;
        Height = DrawerOwner->Height;
        break;
      case akTop:
        Top = DrawerOwner->Top - Height;
        Left = DrawerOwner->Left;
        Constraints->MinWidth = 0;
        Constraints->MaxWidth = 0;
        Width = DrawerOwner->Width;
        break;
      case akBottom:
        Top = DrawerOwner->Top + DrawerOwner->Height;
        Left = DrawerOwner->Left;
        Constraints->MinWidth = 0;
        Constraints->MaxWidth = 0;
        Width = DrawerOwner->Width;
        break;
    }
  }
  else
    is_drawer_attached = Value;


  WindowMoved(true);
}
//---------------------------------------------------------------------------
POINT TformSpecialGeneric::point(long x, long y)
{
  LogIt("specialgeneric.log", "point(" + UIntToString(x) + ", " + UIntToString(y) + ") entry");
  POINT temp;
  temp.x = x;
  temp.y = y;
  return temp;
}
//---------------------------------------------------------------------------
void TformSpecialGeneric::DrawerRefresh ()
{
  LogIt("specialgeneric.log", "DrawerRefresh() entry");
  if (IsDestroyedForm(this, "DrawerRefresh", __LINE__)) // jhaley: Do not run on destroyed forms!
   return;
  if (!generic_form_state) // jhaley: Do not run on unconstructed forms either.
    return;

  // if we're a parent window, do things ...
  if (!drawers.empty())
  {
   // nothing currently?
  }

  /* TODO: for ashley: if they resize the drawer small enough, make it close.
           that, in turn, means adding events for drawers opening/closing.
           we need those events, particularly if we go adding automatic buttons for open/close on things!
           and even more so if i go modifying the borders of things to make them become buttons and whatnot. */

  // TODO: prevent flashiness
  //         virtual void __fastcall SetBounds(int ALeft, int ATop, int AWidth, int AHeight);

  // if we're a child window, do things ...
  if (DrawerOwner)
  {
    long height_of_caption = GetSystemMetrics(SM_CYCAPTION);
    long width_of_border   = (this->BorderStyle == bsSizeable) ? GetSystemMetrics(SM_CXSIZEFRAME) : GetSystemMetrics(SM_CXFIXEDFRAME);
    long height_of_border  = (this->BorderStyle == bsSizeable) ? GetSystemMetrics(SM_CYSIZEFRAME) : GetSystemMetrics(SM_CYFIXEDFRAME);

    // the following were done to add a bit of margin on the side being eaten
    width_of_border -= 1;
    height_of_border -= 1;

    // position self
    this->Visible = (DrawerExpanded) && (DrawerOwner->Visible) && (DrawerOwner->WindowState != wsMinimized);
    // GetWindowRect(DrawerOwner->Handle, &(DrawerOwner->window_rect));
    RECT parent_rect = DrawerOwner->window_rect;

    int intended_value = 0;

    if (DrawerAttached)
      switch (DrawerPosition)
      {
       case akLeft:
         this->Left   = parent_rect.left /*DrawerOwner->Left */ - this->Width + width_of_border;
         this->Top    = parent_rect.top /* DrawerOwner->Top */ + height_of_caption;
         intended_value        = (parent_rect.bottom - parent_rect.top) /* DrawerOwner->Height */ - 2*height_of_caption;
         this->Constraints->MinHeight = intended_value;
         this->Constraints->MaxHeight = intended_value;
         this->Height                 = intended_value;
         this->Constraints->MinWidth  = 0;
         this->Constraints->MaxWidth  = 0;
         break;
       case akRight:
         this->Left   = parent_rect.right /* DrawerOwner->Left + DrawerOwner->Width */ - width_of_border + 1;
         this->Top    = parent_rect.top /* DrawerOwner->Top */ + height_of_caption;
         intended_value        = (parent_rect.bottom - parent_rect.top) /* DrawerOwner->Height */ - 2*height_of_caption;
         this->Constraints->MinHeight = intended_value;
         this->Constraints->MaxHeight = intended_value;
         this->Height                 = intended_value;
         this->Constraints->MinWidth  = 0;
         this->Constraints->MaxWidth  = 0;
         break;
       case akTop:
         this->Left   = parent_rect.left /* DrawerOwner->Left */;
         this->Top    = parent_rect.top /* DrawerOwner->Top */ - this->Height + height_of_border;
         intended_value        = (parent_rect.right - parent_rect.left) /* DrawerOwner->Width */;
         this->Constraints->MinWidth  = intended_value;
         this->Constraints->MaxWidth  = intended_value;
         this->Width                  = intended_value;
         this->Constraints->MinHeight = 0;
         this->Constraints->MaxHeight = 0;
         break;
       case akBottom:
         this->Left   = parent_rect.left /* DrawerOwner->Left */;
         this->Top    = parent_rect.bottom /* DrawerOwner->Top + DrawerOwner->Height */ - height_of_border + 1;
         intended_value        = (parent_rect.right - parent_rect.left) /* DrawerOwner->Width */;
         this->Constraints->MinWidth  = intended_value;
         this->Constraints->MaxWidth  = intended_value;
         this->Width                  = intended_value;
         this->Constraints->MinHeight = 0;
         this->Constraints->MaxHeight = 0;
         break;
      }
    else
    {
      // the first time, it might be good to set a default position that is clearly not where it was before. but not here.
      this->Constraints->MinWidth = 0;
      this->Constraints->MaxWidth = 0;
      this->Constraints->MinHeight = 0;
      this->Constraints->MaxHeight = 0;
    }

    // remove border that we don't want
    // note: we want to go anti-clockwise, to include stuff in the middle
    //    .. (normally, 3D graphics have you going clockwise, as I recall)

    vector<POINT> points;

    // left/right/top/bottom outer/inner x/y
    long lox = 0;
    long lix = lox + width_of_border - 1;
    long rox = this->Width;
    long rix = rox - width_of_border;

    long toy = 0;
    long tiy = toy + height_of_border - 1;
    long boy = this->Height;
    long biy = boy - height_of_border;

    if (DrawerAttached)
      switch (DrawerPosition)
      {
        case akLeft:
          points.push_back(point(lox,toy));
          points.push_back(point(lox,boy));
          points.push_back(point(rix,boy));
          points.push_back(point(rix,toy));
          break;
        case akRight:
          points.push_back(point(lix,toy));
          points.push_back(point(lix,boy));
          points.push_back(point(rox,boy));
          points.push_back(point(rox,toy));
          break;
        case akTop:
          points.push_back(point(lox,toy));
          points.push_back(point(lox,biy));
          points.push_back(point(rox,biy));
          points.push_back(point(rox,toy));
          break;
        case akBottom:
          points.push_back(point(lox,tiy));
          points.push_back(point(lox,boy));
          points.push_back(point(rox,boy));
          points.push_back(point(rox,tiy));
          break;
      }
    else
    {
      points.push_back(point(lox, toy));
      points.push_back(point(lox, boy));
      points.push_back(point(rox, boy));
      points.push_back(point(rox, toy));
    }

    HRGN region_handle;
    region_handle = CreatePolygonRgn(&points[0], points.size(), WINDING);
    if (region_handle)
      if (SetWindowRgn(this->Handle, region_handle, true))
        ; // success
      else
        ; // error
    else
      ; // error

  // note quite working right, so ... disabled. until further notice. and stuff.
  /*
     if (DrawerPosition == akLeft || DrawerPosition == akRight)
       if (Height < drawer_minimum_static_size)
       {
         DrawerAttached = false;
         Height = drawer_minimum_static_size;
       }
     if (DrawerPosition == akTop || DrawerPosition == akBottom)
       if (Width < drawer_minimum_static_size)
       {
         DrawerAttached = false;
         Width = drawer_minimum_static_size;
       }
  */
  }
}
//---------------------------------------------------------------------------
void TformSpecialGeneric::WindowMoved (bool snap_self)
{
  LogIt("specialgeneric.log", "WindowMoved(" + BoolToBoolString(snap_self, true_false) + ") entry");
  if (IsDestroyedForm(this, "WindowMoved", __LINE__)) // jhaley: Do not run on destroyed forms!
    return;
  if (!generic_form_state) // jhaley: Do not run on unconstructed forms either.
    return;

   UpdateWindowState();

   if (snap_self)
     DrawerRefresh();

   for (set<TformSpecialGeneric*>::iterator itr_drawer = drawers.begin(); itr_drawer != drawers.end(); itr_drawer++)
     (*itr_drawer)->WindowMoved(true);
}
//---------------------------------------------------------------------------
void __fastcall TformSpecialGeneric::Dispatch(void *Message)
{
  TMessage * msg = (TMessage*)Message;
  int    hit_test_result = (int)(msg->WParam);
  bool   should_refresh = false;
  POINT  cursor_position;
  POINTSTOPOINT(cursor_position, MAKEPOINTS(msg->LParam));
  // bool left_button_down = false;
  // if (msg->WParam & MK_LBUTTON)
  //  left_button_down = true;
  RECT* pRect = reinterpret_cast<RECT*>(msg->LParam);

  switch  (((PMessage)Message)->Msg)
  {
    // note: do not try to set the max size here.
    // doing so causes resizable forms with anchors to suddenly have stuff splattered all over the galaxy.
    case WM_GETMINMAXINFO:
    {
      TForm::Dispatch(Message);
      TMessage * msg = (TMessage*)Message;
      MINMAXINFO * info = (MINMAXINFO*)(msg->LParam);
      info->ptMinTrackSize.x = this->Constraints->MinWidth;
      info->ptMinTrackSize.y = this->Constraints->MinHeight;
      break;
    }

    // enforce correct sizing that does NOT move the window when resizing from the side
    case WM_SIZING:
    {
      // minimum width
      if (pRect->right - pRect->left < this->Constraints->MinWidth && this->Constraints->MinWidth)
      {
        switch (msg->WParam)
        {
          case WMSZ_LEFT:
          case WMSZ_TOPLEFT:
          case WMSZ_BOTTOMLEFT:
          {
            pRect->left = pRect->right - this->Constraints->MinWidth;
            break;
          }
          case WMSZ_RIGHT:
          case WMSZ_TOPRIGHT:
          case WMSZ_BOTTOMRIGHT:
          {
            pRect->right = pRect->left + this->Constraints->MinWidth;
            break;
          }
        }
      }
      // minimum height
      if (pRect->bottom - pRect->top < this->Constraints->MinHeight && this->Constraints->MinHeight)
      {
        switch (msg->WParam)
        {
          case WMSZ_TOPLEFT:
          case WMSZ_TOP:
          case WMSZ_TOPRIGHT:
          {
            pRect->top = pRect->bottom - this->Constraints->MinHeight;
            break;
          }
          case WMSZ_BOTTOMLEFT:
          case WMSZ_BOTTOM:
          case WMSZ_BOTTOMRIGHT:
          {
            pRect->bottom = pRect->top + this->Constraints->MinHeight;
            break;
          }
        }
      }
      // maximum width
      if (pRect->right - pRect->left > this->Constraints->MaxWidth && this->Constraints->MaxWidth)
      {
        switch (msg->WParam)
        {
          case WMSZ_LEFT:
          case WMSZ_TOPLEFT:
          case WMSZ_BOTTOMLEFT:
          {
            pRect->left = pRect->right - this->Constraints->MaxWidth;
            break;
          }
          case WMSZ_RIGHT:
          case WMSZ_TOPRIGHT:
          case WMSZ_BOTTOMRIGHT:
          {
            pRect->right = pRect->left + this->Constraints->MaxWidth;
            break;
          }
        }
      }
      // maximum height
      if (pRect->bottom - pRect->top > this->Constraints->MaxHeight && this->Constraints->MaxHeight)
      {
        switch (msg->WParam)
        {
          case WMSZ_TOPLEFT:
          case WMSZ_TOP:
          case WMSZ_TOPRIGHT:
          {
            pRect->top = pRect->bottom - this->Constraints->MaxHeight;
            break;
          }
          case WMSZ_BOTTOMLEFT:
          case WMSZ_BOTTOM:
          case WMSZ_BOTTOMRIGHT:
          {
            pRect->bottom = pRect->top + this->Constraints->MaxHeight;
            break;
          }
        }
      }
      // continue to the drawing and dispatching
      window_rect = *pRect;
      ResetHelpButtonRect(pRect->right - pRect->left);
      TForm::Dispatch(Message);
      break;
    }

    case WM_NCPAINT:
      TForm::Dispatch(Message);
      DrawHelpButton();
      break;

    case WM_SETTEXT:
      TForm::Dispatch(Message);
      DrawHelpButton();
      break;

    case WM_NCACTIVATE:
      TForm::Dispatch(Message);
      DrawHelpButton();
      break;

    // START - HELP BUTTON STUFF (specifically)

    case WM_NCHITTEST:
      // coordinates are originally relative to the screen,
      // here RECT coordinates are relative to the window
      cursor_position.x -= this->Left;
      cursor_position.y -= this->Top;
      last_cursor_position = cursor_position;
      if (HelpButtonVisible && HelpButtonEnabled && PtInRect(&help_button_rect, cursor_position))
        msg->Result = HTHELP;
      else
        TForm::Dispatch(Message);

      // prevent or change resizing arrows in edges that can't be resized currently based on constraints
      {
        bool can_size_width = (Constraints->MinWidth == 0 || Constraints->MinWidth != Constraints->MaxWidth);
        bool can_size_height = (Constraints->MinHeight == 0 || Constraints->MinHeight != Constraints->MaxHeight);

        if (!can_size_width && !can_size_height)
        {
          switch (msg->Result)
          {
            case HTTOPLEFT:
            case HTTOP:
            case HTTOPRIGHT:
            case HTLEFT:
            case HTRIGHT:
            case HTBOTTOMLEFT:
            case HTBOTTOM:
            case HTBOTTOMRIGHT:
             msg->Result = HTCLIENT;
             break;
          }
        }
        else if (!can_size_width)
        {
          switch (msg->Result)
          {
           case HTTOPLEFT:
           case HTTOP:
           case HTTOPRIGHT:
            msg->Result = HTTOP;
            break;
           case HTLEFT:
           case HTRIGHT:
            msg->Result = HTCLIENT;
            break;
           case HTBOTTOMLEFT:
           case HTBOTTOM:
           case HTBOTTOMRIGHT:
            msg->Result = HTBOTTOM;
            break;
          }
        }
        else if (!can_size_height)
        {
          switch (msg->Result)
          {
            case HTTOPLEFT:
            case HTLEFT:
            case HTBOTTOMLEFT:
              msg->Result = HTLEFT;
              break;
            case HTTOP:
            case HTBOTTOM:
              msg->Result = HTCLIENT;
              break;
            case HTTOPRIGHT:
            case HTRIGHT:
            case HTBOTTOMRIGHT:
              msg->Result = HTRIGHT;
              break;
          }
        }
      }

      // simpler solution to the code below ... no longer trying to change anything at the inner corners.
      if (DrawerOwner && DrawerExpanded && DrawerAttached)
      {
        if ((msg->Result == HTLEFT && DrawerPosition == akRight) ||
            (msg->Result == HTRIGHT && DrawerPosition == akLeft) ||
            (msg->Result == HTTOP && DrawerPosition == akBottom) ||
            (msg->Result == HTBOTTOM && DrawerPosition == akTop))
         msg->Result = HTCLIENT;
      }

      // now prevent resizing on the inner edge of a drawer.
      /*
      if (DrawerOwner && DrawerExpanded)
      {
       DWORD style      = GetWindowLong(this->Handle, GWL_STYLE);
       int frame_width  = GetSystemMetrics(( style & WS_THICKFRAME ) ? SM_CXSIZEFRAME : SM_CXFIXEDFRAME);
       int frame_height = GetSystemMetrics(( style & WS_THICKFRAME ) ? SM_CYSIZEFRAME : SM_CYFIXEDFRAME);
       switch (DrawerPosition)
       {
        case akTop:
         switch (msg->Result)
         {
          case HTBOTTOMLEFT: msg->Result = HTLEFT; break;
          case HTBOTTOMRIGHT: msg->Result = HTRIGHT; break;
         }
         if (cursor_position.y >= Height - frame_height &&
             cursor_position.x >= frame_width &&
             cursor_position.x <= Width - frame_width)
          msg->Result = HTCLIENT;
         break;
        case akLeft:
         switch (msg->Result)
         {
          case HTTOPRIGHT: msg->Result = HTTOP; break;
          case HTBOTTOMRIGHT: msg->Result = HTBOTTOM; break;
         }
         if (cursor_position.x >= Width - frame_width &&
             cursor_position.y >= frame_height &&
             cursor_position.y <= Height - frame_height)
          msg->Result = HTCLIENT;
         break;
        case akBottom:
         switch (msg->Result)
         {
          case HTTOPLEFT: msg->Result = HTLEFT; break;
          case HTTOPRIGHT: msg->Result = HTRIGHT; break;
         }
         if (cursor_position.y <= frame_height &&
             cursor_position.x >= frame_width &&
             cursor_position.x <= Width - frame_width)
          msg->Result = HTCLIENT;
         break;
        case akRight:
         switch (msg->Result)
         {
          case HTTOPLEFT: msg->Result = HTTOP; break;
          case HTBOTTOMLEFT: msg->Result = HTBOTTOM; break;
         }
         if (cursor_position.x <= frame_width &&
             cursor_position.y >= frame_height &&
             cursor_position.y <= Height - frame_height)
          msg->Result = HTCLIENT;
         break;
       }
      }
      // to restrictive, odd things on corners. above solution fixed that pretty much. hooray.
      /*
      if ((DrawerPosition == akTop && (msg->Result == HTBOTTOM || msg->Result == HTBOTTOMLEFT || msg->Result == HTBOTTOMRIGHT)) ||
          (DrawerPosition == akLeft && (msg->Result == HTRIGHT || msg->Result == HTTOPRIGHT || msg->Result == HTBOTTOMRIGHT)) ||
          (DrawerPosition == akBottom && (msg->Result == HTTOP || msg->Result == HTTOPRIGHT || msg->Result == HTTOPLEFT)) ||
          (DrawerPosition == akRight && (msg->Result == HTLEFT || msg->Result == HTTOPLEFT || msg->Result == HTBOTTOMLEFT)))
       msg->Result = HTCLIENT;
      */

      // DrawHelpButton();
      break;

    case WM_MOUSEMOVE:
      // coordinates are originally relative to the client area, here
      // RECT coordinates are relative to the window
      ::ClientToScreen(this->Handle, &cursor_position);
      // coordinates now relative to screen, transferring to window coordinates
      cursor_position.x -= this->Left;
      cursor_position.y -= this->Top;
      last_cursor_position = cursor_position;
      if (is_help_button_captured)
      {
        if (HelpButtonVisible && HelpButtonEnabled && PtInRect(&help_button_rect, cursor_position))
          if (!is_help_button_pushed)
            is_help_button_pushed = true;
        else
          if (is_help_button_pushed)
            is_help_button_pushed = false;
      }
      TForm::Dispatch(Message);
      if (should_refresh)
        DrawHelpButton();
      break;

    case WM_NCLBUTTONDOWN:
      // no coordinates given, only the hit-test
      if (HelpButtonVisible && HelpButtonEnabled && hit_test_result == HTHELP && !is_help_button_captured)
      {
        HWND previous_capture = GetCapture();
        if (previous_capture == SetCapture(this->Handle)) // we want to know about the mouse
        {
          is_help_button_pushed   = true;
          is_help_button_captured = true;
          should_refresh = true;
        }
      }
      else
        TForm::Dispatch(Message);
      if (should_refresh)
        DrawHelpButton();
      break;

    case WM_CAPTURECHANGED:
      if (is_help_button_captured)
      {
        is_help_button_captured = false;
        is_help_button_pushed   = false;
        should_refresh = true;
        if (HelpButtonVisible && HelpButtonEnabled && PtInRect(&help_button_rect, last_cursor_position))
         HelpButtonPressed(); // DO SOMETHING HERE
      }
      else
        TForm::Dispatch(Message);
      if (should_refresh)
        DrawHelpButton();
      break;

    // END - HELP BUTTON STUFF

    case WM_EXITSIZEMOVE:
      if (DrawerOwner && DrawerExpanded && DrawerAttached)
      {
       if ((DrawerPosition == akLeft || DrawerPosition == akRight) &&
           Width < drawer_autoclose_size)
        DrawerExpanded = false;
       if ((DrawerPosition == akTop || DrawerPosition == akBottom) &&
           Height < drawer_autoclose_size)
        DrawerExpanded = false;
      }
      TForm::Dispatch(Message);
      // DrawHelpButton();
      WindowMoved(true);
      break;
     case WM_MOVING:
      window_rect = *pRect;
      if (pRect->left != this->Left ||
          pRect->top != this->Top)
        should_refresh = true;
      TForm::Dispatch(Message);
      if (should_refresh)
      {
        // DrawHelpButton();
        WindowMoved(false);
      }
      break;

    case WM_SYSCOMMAND:
      //WMSysCommand(*((TWMSysCommand *)Message));
      if (msg->WParam == SysMenu_Database) ;
      if (msg->WParam == SysMenu_Admin_ShowInfoBar) { ShowInfoBar(); }
      if (msg->WParam == SysMenu_Admin_HideInfoBar) { HideInfoBar(); }
      for (int i = 0; i < 10; i++)
        if (msg->WParam == (int)(SysMenu_Admin_Debug_Libraries[i]))
        {
          debug_level[libraries] = i;
          CheckMenuItem(DebugLibrariesMenu, msg->WParam, MF_BYCOMMAND);
        }
      for (int i = 0; i < 10; i++)
        if (msg->WParam == (int)(SysMenu_Admin_Debug_Generic[i]))
        {
          debug_level[generic_forms] = i;
          CheckMenuItem(DebugGenericMenu, msg->WParam, MF_BYCOMMAND);
        }
      for (int i = 0; i < 10; i++)
        if (msg->WParam == (int)(SysMenu_Admin_Debug_Other[i]))
        {
          debug_level[other_forms] = i;
          CheckMenuItem(DebugGenericMenu, msg->WParam, MF_BYCOMMAND);
        }
      TForm::Dispatch(Message);
      break;

    default:
      TForm::Dispatch(Message);
      break;
  }
}
//---------------------------------------------------------------------------
void __fastcall TformSpecialGeneric::FormShow(TObject *Sender)
{
  LogIt("specialgeneric.log", "FormShow(" + IntToHexString((int)Sender) + ") entry");
  // move to a function that determines what should be possible? and makes that be true?
  ResetMaximizeButton();

  if (optionsPreferences["special_generic"]["auto_open_help"] == "true" && HelpButtonVisible && HelpButtonEnabled)
    HelpButtonPressed();

  // WindowMoved(true);
  SendMessage(this->Handle, WM_NCACTIVATE, Boolean(True), 0);
}
//---------------------------------------------------------------------------
void __fastcall TformSpecialGeneric::FormCloseQuery(TObject *Sender, bool &CanClose)
{
  LogIt("specialgeneric.log", "FormCloseQuery(" + IntToHexString((int)Sender) + ", " + BoolToBoolString(CanClose, true_false) + ") entry");
  if (DrawerOwner && !is_drawer_destroyable)
    CanClose = false;
  else
    CanClose = true;
// TODO: P is going to have to -not- try to close forms that currently have a non-null DrawerOwner ... that should fix this, at least.
}
//---------------------------------------------------------------------------
void __fastcall TformSpecialGeneric::FormResize(TObject *Sender)
{
  ResetHelpButtonRect(this->Width);

  // Redraw caption bar
  SetWindowPos(this->Handle, this->Handle, this->Left, this->Top, this->Width, this->Height, SWP_NOMOVE | SWP_NOSIZE | SWP_NOZORDER | SWP_FRAMECHANGED);
/*
  if (DrawerOwner && DrawerExpanded)
  {
    if ((DrawerPosition == akLeft || DrawerPosition == akRight) &&
        Width < drawer_autoclose_size)
      DrawerExpanded = false;
    if ((DrawerPosition == akTop || DrawerPosition == akBottom) &&
        Height < drawer_autoclose_size)
      DrawerExpanded = false;
  }
*/

  AdjustInfoBarSize();
  // Make sure drawers follow window
  WindowMoved(true);
  // Fill in array struct so on the next pass it has a record of what happened
  GetWindowRect(this->Handle, &(this->window_rect));
}
//---------------------------------------------------------------------------
void __fastcall TformSpecialGeneric::FormActivate(TObject *Sender)
{
  LogIt("specialgeneric.log", "FormActivate(" + IntToHexString((int)Sender) + ") entry");
  if (IsDestroyedForm(this, "FormActivate", __LINE__)) // jhaley: Do not run on destroyed forms!
    return;
  if (!generic_form_state) // jhaley: Do not run on unconstructed forms either.
     return;

 // if (optionsPermissions["special.can_maximize"]["enabled"] == "true")
 //   this->BorderIcons << biMaximize;
  AV_TRY
    TformSpecialGeneric *drawer_leader = DrawerLeader();
    if (drawer_leader)
      drawer_leader->ActiveCount = 1;
    GetWindowRect(this->Handle, &(this->window_rect));
    WindowMoved(true);
  AV_CATCH
}
//---------------------------------------------------------------------------
void __fastcall TformSpecialGeneric::FormDeactivate(TObject *Sender)
{
  LogIt("specialgeneric.log", "FormDeactivate(" + IntToHexString((int)Sender) + ") entry");
  if (IsDestroyedForm(this, "FormDeactivate", __LINE__)) // jhaley: Do not run on destroyed forms!
    return;
  if (!generic_form_state) // jhaley: Do not run on unconstructed forms either.
     return;

  AV_TRY
    TformSpecialGeneric *drawer_leader = DrawerLeader();
    if (drawer_leader)
      drawer_leader->ActiveCount = 0;
  AV_CATCH
}
//---------------------------------------------------------------------------
void __fastcall TformSpecialGeneric::SetActiveCount (int Value)
{
  LogIt("specialgeneric.log", "SetActiveCount(" + IntToString(Value) + ") entry");
  if (IsDestroyedForm(this, "SetActiveCount", __LINE__)) // jhaley: Do not run on destroyed forms!
    return;
  if (!generic_form_state) // jhaley: Do not run on unconstructed forms either.
    return;

 /*
  if (itr_drawer == NULL)
    break;     // Something's wrong.  This should SO not be necessary, it's amazing...
               // not really sure what's going on with begin() being NULL and end() not.
 */

  active_count = Value;

  if (active_count)
  {
    TformSpecialGeneric *drawer_leader = DrawerLeader();
    if (drawer_leader && drawer_leader == this)
      SetWindowPos(drawer_leader->Handle, HWND_TOP, 0, 0, 0, 0, SWP_NOMOVE | SWP_NOSIZE | SWP_NOACTIVATE);
    if (drawer_owner)
      SetWindowPos(this->Handle, drawer_owner->Handle, 0, 0, 0, 0, SWP_NOMOVE | SWP_NOSIZE | SWP_NOACTIVATE);
  }

  for (set<TformSpecialGeneric*>::iterator itr_drawer = drawers.begin(); itr_drawer != drawers.end(); itr_drawer++)
  {
    AV_TRY
     if (*itr_drawer)
       (*itr_drawer)->ActiveCount = Value;
     else /* whoops? */ ;
    AV_CATCH
  }

  if (active_count)
    SendMessage(this->Handle, WM_NCACTIVATE, Boolean(True), 0);
  else
    SendMessage(this->Handle, WM_NCACTIVATE, Boolean(False), 0);
}
//---------------------------------------------------------------------------
TformSpecialGeneric * TformSpecialGeneric::DrawerLeader ()
{
  LogIt("specialgeneric.log", "DrawerLeader() entry");
  if (IsDestroyedForm(this, "DrawerLeader", __LINE__)) // jhaley 20100913: Do not run on destroyed forms.
    return NULL;

  if (DrawerOwner)
    return DrawerOwner->DrawerLeader();
  else
    return this;
}
//---------------------------------------------------------------------------
void __fastcall TformSpecialGeneric::FormHide(TObject *Sender)
{
  LogIt("specialgeneric.log", "FormHide(" + IntToHexString((int)Sender) + ") entry");
  WindowMoved(true);
}
//---------------------------------------------------------------------------
void TformSpecialGeneric::ResetHelpButtonRect(int form_width)
{
  LogIt("specialgeneric.log", "ResetHelpButtonRect(" + IntToString(form_width) + ") entry");
  DWORD style        = GetWindowLong(this->Handle, GWL_STYLE);
  int button_width   = GetSystemMetrics(SM_CXSIZE);
  int button_height  = GetSystemMetrics(SM_CYSIZE);
  int frame_width    = GetSystemMetrics(( style & WS_THICKFRAME ) ? SM_CXSIZEFRAME : SM_CXFIXEDFRAME);
  int frame_height   = GetSystemMetrics(( style & WS_THICKFRAME ) ? SM_CYSIZEFRAME : SM_CYFIXEDFRAME);
  int caption_width  = form_width - (2 * frame_width);

  SetRect (&help_button_rect,
           /*left*/   frame_width + caption_width - (4 * button_width),
           /*top*/    frame_height + 2,
           /*right*/  frame_width + caption_width - (3 * button_width) - 2,
           /*bottom*/ frame_height + button_height - 2);
}
//---------------------------------------------------------------------------
void TformSpecialGeneric::DrawHelpButton()
{
  LogIt("specialgeneric.log", "DrawHelpButton() entry");
  HDC caption_dc = GetWindowDC(this->Handle);

  /* other option: DFCS_HOT, not particularly useful, eh? */
  if (HelpButtonVisible)
  {
    if (HelpButtonEnabled)
    {
      if (is_help_button_pushed)
        DrawFrameControl(caption_dc, &help_button_rect, DFC_CAPTION, DFCS_CAPTIONHELP | DFCS_PUSHED);
      else
        DrawFrameControl(caption_dc, &help_button_rect, DFC_CAPTION, DFCS_CAPTIONHELP);
    }
    else
     DrawFrameControl(caption_dc, &help_button_rect, DFC_CAPTION, DFCS_CAPTIONHELP | DFCS_INACTIVE);
  }

  ReleaseDC(this->Handle, caption_dc);
}
//---------------------------------------------------------------------------
void TformSpecialGeneric::HelpButtonPressed()
{
  LogIt("specialgeneric.log", "HelpButtonPressed() entry");
  if (!help_drawer)
  {
    FORM_TRY
      help_drawer = new TformDrawerHelp(NULL);
    FORM_CATCH
    help_drawer->DrawerOwner    = this;
    help_drawer->DrawerPosition = akRight;
    help_drawer->DrawerExpanded = true;
    help_drawer->DrawerAttached = true;
    help_drawer->Width          = 300;
    help_drawer->Height         = 400;
    help_drawer->SetSourceForm (this); // update with relevant topics
  }
  else
  {
    // in or out? depends.
    help_drawer->DrawerExpanded = !help_drawer->DrawerExpanded;
    help_drawer->DrawerAttached = false;
    // oh, keep it around already. it'll be fine.
    /*
    help_drawer->DrawerOwner = NULL;
    help_drawer->Close();
    help_drawer = NULL;
    */
    // orly?  is that why it's commented out, Philip?
  }
}
//---------------------------------------------------------------------------
void __fastcall TformSpecialGeneric::SetHelpButtonEnabled (bool Value)
{
  LogIt("specialgeneric.log", "SetHelpButtonEnabled(" + IntToBoolString(Value, true_false) + ") entry");
  if (Owner)
    Value = false;

  if (Value != is_help_button_enabled)
  {
    is_help_button_enabled = Value;
    SetWindowPos(this->Handle, this->Handle, this->Left, this->Top, this->Width, this->Height, SWP_NOMOVE | SWP_NOSIZE | SWP_NOZORDER | SWP_FRAMECHANGED);
  /*
    HRGN window_region;
    GetWindowRgn(this->Handle, window_region);
    SendMessage(this->Handle, WM_NCPAINT, (WPARAM)window_region, 0);
  */
  }
}
//---------------------------------------------------------------------------
void __fastcall TformSpecialGeneric::SetHelpButtonVisible (bool Value)
{
  LogIt("specialgeneric.log", "SetHelpButtonVisible(" + IntToBoolString(Value, true_false) + ") entry");
  if (Owner)
    Value = false;

  if (Value != is_help_button_visible)
  {
    is_help_button_visible = Value;
    SetWindowPos(this->Handle, this->Handle, this->Left, this->Top, this->Width, this->Height, SWP_NOMOVE | SWP_NOSIZE | SWP_NOZORDER | SWP_FRAMECHANGED);
  /*
    HRGN window_region;
    GetWindowRgn(this->Handle, window_region);
    SendMessage(this->Handle, WM_NCPAINT, (WPARAM)window_region, 0);
  */
  }
}
//---------------------------------------------------------------------------
void TformSpecialGeneric::DrawerIsChanging (TformSpecialGeneric * Drawer, bool Expanding)
{
  LogIt("specialgeneric.log", "DrawerIsChanging(" + IntToHexString((int)Drawer) + ", " + BoolToBoolString(Expanding, true_false) + ") entry");
  // prevent memory problems
  if (!Expanding && Drawer == help_drawer)
  {
    // no no no ... it'll be fine. created when button first pushed, keep alive after that.
    /*
    help_drawer->DrawerOwner = NULL;
    help_drawer->Close();
    help_drawer = NULL;
    */
  }
  return;
}
//---------------------------------------------------------------------------
void __fastcall TformSpecialGeneric::buttonEscapeClick(TObject *Sender)
{
  LogIt("specialgeneric.log", "FormDeactivate(" + IntToHexString((int)Sender) + ") entry");
  if (!is_processing)
    if (DrawerOwner)
      DrawerExpanded = false;
   else
     Close();
  else
   ; // do nothing. something important is happening.
}
//---------------------------------------------------------------------------
void TformSpecialGeneric::SetFormIcon ()
{
  LogIt("specialgeneric.log", "SetFormIcon() entry");
  // reset icon_index to something else if it's out of bounds, unless it's -1. in that case, draw blank.
  if (icon_index > image_list->Count - 1 || icon_index < -1)
   icon_index = -1;

  // Maybe this will get rid of "Error reading formSpecialGeneric->Icon.Data: Out of system resources", eh?  *crosses fingers*
  // okay, original try/catch didn't work. since it's a resources error, we'll just try re-using the previous trick. =)
  FORM_TRY
   image_list->GetIcon(icon_index, this->Icon);
  FORM_CATCH
}
//---------------------------------------------------------------------------
void __fastcall TformSpecialGeneric::SetImageList (TImageList* Value)
{
  LogIt("specialgeneric.log", "SetImageList(" + IntToHexString((int)Value) + ") entry");
  image_list = Value;
  SetFormIcon();
}
//---------------------------------------------------------------------------
void __fastcall TformSpecialGeneric::SetIconIndex (int Value)
{
  LogIt("specialgeneric.log", "SetIconIndex(" + IntToString(Value) + ") entry");
  icon_index = Value;
  SetFormIcon();
}
//---------------------------------------------------------------------------
void __fastcall TformSpecialGeneric::SetMaximizeButtonEnabled (bool Value)
{
  LogIt("specialgeneric.log", "FormShow(" + BoolToBoolString(Value, true_false) + ") entry");
  DWORD style = GetWindowLong(this->Handle, GWL_STYLE);

  if (Value)
  {
    style |= WS_MAXIMIZEBOX;
    this->BorderIcons << biMaximize;
  }
  else
  {
    if (this->WindowState == wsMaximized)
     this->WindowState = wsNormal;

    // if you don't remove the "WS_MAXIMIZE", it thinks it's still maximized when you close the drawer again, which ... freaks stuff out.
    style &= ~WS_MAXIMIZE;
    style &= ~WS_MAXIMIZEBOX;
    this->BorderIcons >> biMaximize;
  }

  SetWindowLong(this->Handle, GWL_STYLE, style);

  // for some reason, re-enabling doesn't refresh the window. go figure.
  SetWindowPos(this->Handle, this->Handle, this->Left, this->Top, this->Width, this->Height, SWP_NOMOVE | SWP_NOSIZE | SWP_NOZORDER | SWP_FRAMECHANGED);
}
//---------------------------------------------------------------------------
void TformSpecialGeneric::ResetMaximizeButton ()
{
 unsigned int open_drawers = 0;
 for (set<TformSpecialGeneric*>::iterator i_d = drawers.begin(); i_d != drawers.end(); i_d++)
  if ((*i_d)->DrawerExpanded)
   open_drawers++;

 if (optionsPermissions["special.can_maximize"]["enabled"] == "true" && this->Constraints->MaxWidth == 0 && this->Constraints->MaxHeight == 0 && open_drawers == 0)
  SetMaximizeButtonEnabled(true);
 else
  SetMaximizeButtonEnabled(false);
}
//---------------------------------------------------------------------------
void __fastcall TformSpecialGeneric::toolsStandardAdvancedCustomDraw(TToolBar *Sender, const TRect &ARect, TCustomDrawStage Stage, bool &DefaultDraw)
{
 // try to paint that  /;.  resizer (sizegrip)?
 DefaultDraw = true;
 // if (Stage == cdPostPaint)

 bool can_size_width = (Constraints->MinWidth == 0 || Constraints->MinWidth != Constraints->MaxWidth);
 bool can_size_height = (Constraints->MinHeight == 0 || Constraints->MinHeight != Constraints->MaxHeight);

 if ((can_size_width || can_size_height) && toolsStandard->Visible && this->WindowState != wsMaximized && toolsStandard->Height)
 {
  RECT bob;
  bob.right  = toolsStandard->Width - 0;
  bob.bottom = toolsStandard->Height - 0;
  bob.left   = bob.right - 14; // it's 13, but right/bottom edges don't count.
  bob.top    = bob.bottom - 14;

  DrawFrameControl(toolsStandard->Canvas->Handle, &bob, DFC_SCROLL, DFCS_SCROLLSIZEGRIP);
 }
}
//---------------------------------------------------------------------------
void __fastcall TformSpecialGeneric::toolsStandardResize(TObject *Sender)
{
 toolsStandard->Repaint();
}
//---------------------------------------------------------------------------
void __fastcall TformSpecialGeneric::toolsStandardMouseDown(TObject *Sender, TMouseButton Button, TShiftState Shift, int X, int Y)
{
 // disable this to disable sizegrip dragging!
 bool can_size_width = (Constraints->MinWidth == 0 || Constraints->MinWidth != Constraints->MaxWidth);
 bool can_size_height = (Constraints->MinHeight == 0 || Constraints->MinHeight != Constraints->MaxHeight);

 if ((can_size_width || can_size_height) && toolsStandard->Visible && this->WindowState != wsMaximized && toolsStandard->Height)
 {
  if (Y > (toolsStandard->Width - 14 + toolsStandard->Height - X)) // using the y=ax+b formula, clipping off a triangular corner?
  // if (X > toolsStandard->Width - 14 && Y > toolsStandard->Height - 14)
  {
   ReleaseCapture();
   Perform(WM_SYSCOMMAND, SC_SIZE + 8, 0);
/*
   sizegrip_x = X;
   sizegrip_y = Y;
   sizegrip_dragging = true;
   SetCapture(((TWinControl*)Sender)->Handle);
*/
  }
 }
}
//---------------------------------------------------------------------------
void __fastcall TformSpecialGeneric::toolsStandardMouseMove(TObject *Sender, TShiftState Shift, int X, int Y)
{
 // TODO: this doesn't work quite right in relation to "[ ] show windows content while resizing/moving"
 // *should* be able to override WM_NCHITTEST and say we're on an edge, but that doesn't seem to work.
 // (same was true in DrawerGeneric, moving by dragging the caption -- WM_NCHITTEST didn't work like delphi examples show.)

 // not needed, what with the Perform() trick.
/*
 if (sizegrip_dragging)
 {
  int old_width = this->Width;
  this->Width += (X - sizegrip_x);
  this->Height += (Y - sizegrip_y);
  if (old_width != this->Width)
   sizegrip_x = X;
 }
*/

 bool can_size_width = (Constraints->MinWidth == 0 || Constraints->MinWidth != Constraints->MaxWidth);
 bool can_size_height = (Constraints->MinHeight == 0 || Constraints->MinHeight != Constraints->MaxHeight);

 toolsStandard->Cursor = crDefault;
 if ((can_size_width || can_size_height) && toolsStandard->Visible && this->WindowState != wsMaximized && toolsStandard->Height)
 {
  if (Y > (toolsStandard->Width - 14 + toolsStandard->Height - X)) // using the y=ax+b formula, clipping off a triangular corner?
  // if (X > toolsStandard->Width - 14 && Y > toolsStandard->Height - 14)
  {
   if (can_size_width && can_size_height)
    toolsStandard->Cursor = crSizeNWSE;
   else if (can_size_width)
    toolsStandard->Cursor = crSizeWE;
   else
    toolsStandard->Cursor = crSizeNS;
  }
 }
}
//---------------------------------------------------------------------------
void __fastcall TformSpecialGeneric::toolsStandardMouseUp(TObject *Sender, TMouseButton Button, TShiftState Shift, int X, int Y)
{
 // not needed, what with the Perform() trick.
/*
 if (sizegrip_dragging)
 {
  sizegrip_dragging = false;
  ReleaseCapture();
 }
*/
}
//---------------------------------------------------------------------------
void TformSpecialGeneric::AddSysMenuItems()
{
  AppendMenu(GetSystemMenu(Handle, FALSE), MF_SEPARATOR, 0, "");
  if (optionsPermissions["admin.admin_sysmenu"]["enabled"] == "true")
  {
    AppendMenu(GetSystemMenu(Handle, FALSE), MF_STRING, SysMenu_Admin_ShowInfoBar, "Show info bar");
    AppendMenu(GetSystemMenu(Handle, FALSE), MF_STRING, SysMenu_Admin_HideInfoBar, "Hide info bar");
    HMENU DebugMenu = CreateMenu();
    AppendMenu(GetSystemMenu(Handle, FALSE), MF_STRING | MF_POPUP, UINT(DebugMenu), "Set debug level");
    DebugLibrariesMenu = CreateMenu();
    AppendMenu(DebugMenu, MF_STRING | MF_POPUP, UINT(DebugLibrariesMenu), "Libraries");
    for (int i = 0; i < 10; i++)
    {
      UINT checked = (debug_level[libraries] == i) ? MFS_CHECKED : MFS_UNCHECKED;
      const AnsiString MenuString = IntToString(i).c_str();
      MENUITEMINFO info = {  sizeof(MENUITEMINFO),
                             MIIM_TYPE | MIIM_STATE | MIIM_ID,
                             MFT_STRING | MFT_RADIOCHECK,      // menu type is string
                             MFS_ENABLED | checked,            // menu item is enabled
                             SysMenu_Admin_Debug_Libraries[i], // id of menu item
                             NULL,                             // sub menu
                             NULL,                             // no check bitmap
                             NULL,                             // no uncheck bitmap
                             0,                                // no item data
                             MenuString.c_str(),               // string of menu item
                             0};                               // last item not used

      InsertMenuItem (DebugLibrariesMenu, SC_CLOSE, TRUE, &info);
    }

    DebugGenericMenu = CreateMenu();
    AppendMenu(DebugMenu, MF_STRING | MF_POPUP, UINT(DebugGenericMenu), "Generic forms");
    for (int i = 0; i < 10; i++)
    {
      UINT checked = (debug_level[generic_forms] == i) ? MFS_CHECKED : MFS_UNCHECKED;
      const AnsiString MenuString = IntToString(i).c_str();
      MENUITEMINFO info = {  sizeof(MENUITEMINFO),
                             MIIM_TYPE | MIIM_STATE | MIIM_ID,
                             MFT_STRING | MFT_RADIOCHECK,      // menu type is string
                             MFS_ENABLED | checked,            // menu item is enabled
                             SysMenu_Admin_Debug_Generic[i],   // id of menu item
                             NULL,                             // sub menu
                             NULL,                             // no check bitmap
                             NULL,                             // no uncheck bitmap
                             0,                                // no item data
                             MenuString.c_str(),               // string of menu item
                             0};                               // last item not used

      InsertMenuItem (DebugGenericMenu, SC_CLOSE, TRUE, &info);
    }

    DebugOtherMenu = CreateMenu();
    AppendMenu(DebugMenu, MF_STRING | MF_POPUP, UINT(DebugOtherMenu), "Other forms");
    for (int i = 0; i < 10; i++)
    {
      UINT checked = (debug_level[other_forms] == i) ? MFS_CHECKED : MFS_UNCHECKED;
      const AnsiString MenuString = IntToString(i).c_str();
      MENUITEMINFO info = {  sizeof(MENUITEMINFO),
                             MIIM_TYPE | MIIM_STATE | MIIM_ID,
                             MFT_STRING | MFT_RADIOCHECK,      // menu type is string
                             MFS_ENABLED | checked,            // menu item is enabled
                             SysMenu_Admin_Debug_Other[i],     // id of menu item
                             NULL,                             // sub menu
                             NULL,                             // no check bitmap
                             NULL,                             // no uncheck bitmap
                             0,                                // no item data
                             MenuString.c_str(),               // string of menu item
                             0};                               // last item not used

      InsertMenuItem (DebugOtherMenu, SC_CLOSE, TRUE, &info);
    }
  }


  string override_section = funcGetDBOverrideSection();
  if(override_section != "")
  {
    const AnsiString MenuString = ("Connected to " + override_section + " database").c_str();
    MENUITEMINFO info = {  sizeof(MENUITEMINFO),
                           MIIM_TYPE | MIIM_STATE | MIIM_ID,
                           MFT_STRING,                 // menu type is string
                           MFS_ENABLED,                // menu item is enabled
                           SysMenu_Database,           // id of menu item
                           NULL,                       // no sub menu
                           NULL,                       // no check bitmap
                           NULL,                       // no uncheck bitmap
                           0,                          // no item data
                           MenuString.c_str(),         // string of menu item
                           0};                         // last item not used
    // obtain the menu handle for the system menu
    HMENU SysMenu = GetSystemMenu(Handle, FALSE);
    // Insert the new menu item just after the close menu item. If the TRUE
    InsertMenuItem (SysMenu, SC_CLOSE, TRUE, &info);
  }
}
//---------------------------------------------------------------------------
set<TformSpecialGeneric*> FindOpenFormsByType(string class_name, bool exact_match_only)
{
 set<TformSpecialGeneric*> answer;

 TClass ClassRef;
 string ClassName;

 for (list<TformSpecialGeneric*>::iterator i_f = generic_specials.begin(); i_f != generic_specials.end(); i_f++)
 {
  for (ClassRef = (*i_f)->ClassType(); ClassRef != NULL; ClassRef = ClassRef->ClassParent())
  {
   ClassName = AnsiString(String(ClassRef->ClassName())).c_str();
   if (ClassName == class_name)
    answer.insert(*i_f);
   if (exact_match_only)
    break; // only try one-deep
   if (ClassName == "TformSpecialGeneric")
    break; // stop here, because the rest is pointless.
           // you nihilist
  }
 }

 return answer;
}
//---------------------------------------------------------------------------
void TformSpecialGeneric::DisableIncrease ()
{
 for (set<TformSpecialGeneric*>::iterator i = drawers.begin(); i != drawers.end(); i++)
  (*i)->DisableIncrease();
 disable_count++;
 if (disable_count == 1)
  this->Enabled = false;
}
//---------------------------------------------------------------------------
void TformSpecialGeneric::DisableDecrease ()
{
 for (set<TformSpecialGeneric*>::iterator i = drawers.begin(); i != drawers.end(); i++)
  (*i)->DisableDecrease();
 disable_count--;
 if (disable_count == 0)
  this->Enabled = true;
 else if (disable_count < 0)
  disable_count = 0;
}
//---------------------------------------------------------------------------
int __fastcall TformSpecialGeneric::ShowTaskModal(TformSpecialGeneric * ModalParent)
{
 if (!ModalParent)
  return ShowModal();

 // TODO: as per the codeproject article (link below) there is a chance that a form with multiple modal dialogs will get re-enabled early
 // http://www.codeproject.com/dialog/notmodaldialogs.asp
 // some sort of fix involving a count (in the parent form) of the number of times it's been disabled would be good. later.

 // some of the pascal code (shown below) couldn't be easily translated here, and was ommitted.
 // it mostly involves saving cursor states, and could be replaced if rethought. I was too lazy.
 HWND    ActiveWindow;

 CancelDrag();
 if (this->Visible || !this->Enabled || FFormState.Contains(fsModal) || FormStyle == fsMDIChild)
  throw(new EInvalidOperation(65473 /*SCannotShowModal*/));
 if (GetCapture())
  SendMessage(GetCapture(), WM_CANCELMODE, 0, 0);
 ReleaseCapture();
 FFormState << fsModal;
 ActiveWindow = GetActiveWindow();
 if (ModalParent)
  // ModalParent->Enabled = false;
  ModalParent->DisableIncrease();

 // jhaley 20100812: The call to Release done below when CloseAction is caFree ought to wait until the window can no longer
 // receive events before freeing it, but that isn't what's happening. The calls to SendMessage and particularly Hide(), which
 // are really redundant for a form that is closing and freeing itself anyway, can cause events to be enqueued and then executed
 // after ~TForm has already started dismantling the window (in particular, when the form's HWND is being released). We will
 // avoid it entirely by simply not sending extraneous events when caFree is returned from the OnClose handler.
 bool should_hide = true;
 bool should_deactivate = true;

 ModalParent->in_task_modal_loop = true; // jhaley 20101210: Flag the fact that the parent is running a task modal form
 try
 {
  // having problems with Task-Modal forms popping up with waiting cursor?
  this->Cursor = crDefault;
  Show();
  this->SetFocus();
  try
  {
   SendMessage(Handle, CM_ACTIVATE, 0, 0);
   ModalResult = 0;
   do
   {
    Application->HandleMessage();
    if (Application->Terminated)
     ModalResult = mrCancel;
    else
     if (ModalResult)
     {
      TCloseAction CloseAction;
      try
      {
       CloseAction = caNone;
       if (CloseQuery())
       {
        CloseAction = caHide;
        if (OnClose)
         OnClose(this, CloseAction);
       }
       switch (CloseAction)
       {
        case caNone:
         ModalResult = 0;
         break;
        case caFree:
         Release();
         // jhaley: Don't do these actions when releasing the form; VCL will not wait for the resulting events to be handled.
         should_hide = false;
         should_deactivate = false;
       }
      }
      catch(...)
      {
       ModalResult = 0;
       Application->HandleException(this);
      }
     }
   } while (!ModalResult);
   if (should_deactivate) // jhaley: don't deactivate a form that is freeing itself
    SendMessage(Handle, CM_DEACTIVATE, 0, 0);
   if (GetActiveWindow() != Handle)
    ActiveWindow = NULL;
  }
  __finally
  {
   if (should_hide) // jhaley: don't hide a form that is freeing itself
     Hide();
  }
 }
 __finally
 {
  if (ModalParent)
   // ModalParent->Enabled = true;
   ModalParent->DisableDecrease();
  if (ActiveWindow)
   SetActiveWindow(ActiveWindow);
  FFormState >> fsModal;
  ModalParent->in_task_modal_loop = false; // jhaley 20101210: Turn off task modal flag.
 }
 return ModalResult;
}

/*
function TCustomForm.ShowModal: Integer;
var
  WindowList: Pointer;
  SaveFocusCount: Integer;
  SaveCursor: TCursor;
  SaveCount: Integer;
  ActiveWindow: HWnd;
begin
  CancelDrag;
  if Visible or not Enabled or (fsModal in FFormState) or
    (FormStyle = fsMDIChild) then
    raise EInvalidOperation.Create(SCannotShowModal);
  if GetCapture <> 0 then SendMessage(GetCapture, WM_CANCELMODE, 0, 0);
  ReleaseCapture;
  Include(FFormState, fsModal);
  ActiveWindow := GetActiveWindow;
  SaveFocusCount := FocusCount;
  Screen.FSaveFocusedList.Insert(0, Screen.FFocusedForm);
  Screen.FFocusedForm := Self;
  SaveCursor := Screen.Cursor;
  Screen.Cursor := crDefault;
  SaveCount := Screen.FCursorCount;
  WindowList := DisableTaskWindows(0);
  try
    Show;
    try
      SendMessage(Handle, CM_ACTIVATE, 0, 0);
      ModalResult := 0;
      repeat
        Application.HandleMessage;
        if Application.FTerminate then ModalResult := mrCancel else
          if ModalResult <> 0 then CloseModal;
      until ModalResult <> 0;
      Result := ModalResult;
      SendMessage(Handle, CM_DEACTIVATE, 0, 0);
      if GetActiveWindow <> Handle then ActiveWindow := 0;
    finally
      Hide;
    end;
  finally
    if Screen.FCursorCount = SaveCount then
      Screen.Cursor := SaveCursor
    else Screen.Cursor := crDefault;
    EnableTaskWindows(WindowList);
    if Screen.FSaveFocusedList.Count > 0 then
    begin
      Screen.FFocusedForm := Screen.FSaveFocusedList.First;
      Screen.FSaveFocusedList.Remove(Screen.FFocusedForm);
    end else Screen.FFocusedForm := nil;
    if ActiveWindow <> 0 then SetActiveWindow(ActiveWindow);
    FocusCount := SaveFocusCount;
    Exclude(FFormState, fsModal);
  end;
end;
*/

/*
procedure TCustomForm.CloseModal;
var
  CloseAction: TCloseAction;
begin
  try
    CloseAction := caNone;
    if CloseQuery then
    begin
      CloseAction := caHide;
      if Assigned(FOnClose) then FOnClose(Self, CloseAction);
    end;
    case CloseAction of
      caNone: ModalResult := 0;
      caFree: Release;
    end;
  except
    ModalResult := 0;
    Application.HandleException(Self);
  end;
end;
*/
//---------------------------------------------------------------------------
string __fastcall TformSpecialGeneric::GetInfoBarText ()
{
  return lblInformation->Caption.c_str();
}
//---------------------------------------------------------------------------
void __fastcall TformSpecialGeneric::SetInfoBarText (string Value)
{
  if(debug_level[generic_forms]>=8 && message_router) message_router->SendMessageToAll("debug", "SetInfoBarText called with value " + Value);
  lblInformation->Caption = Value.c_str();
  if(debug_level[generic_forms]>=2 && message_router && !this->Visible) message_router->SendMessageToAll("debug", "SetInfoBarText called but the form is invisible!");
  AdjustInfoBarSize();
  // If changing text and already visible, set new height; if not yet visible, this will get caught in ShowInfoBar
/*  if (IsVisible(panelInfoBar))
  {
    if(debug_level[generic_forms]>=8 && message_router) message_router->SendMessageToAll("debug", "SetInfoBarText adjusting lblInformation->Height by " + IntToString(lblInformation->Height + 13) + " pixels.");
    panelInfoBar->Height = lblInformation->Height + 13;
  }
*/
}
//---------------------------------------------------------------------------
void __fastcall TformSpecialGeneric::panelInfoBarCanResize(TObject *Sender,
      int &NewWidth, int &NewHeight, bool &Resize)
{
  if(debug_level[generic_forms]>=8 && message_router) message_router->SendMessageToAll("debug", "panelInfoBar CanResize called with CURRENT width:  " + IntToString(panelInfoBar->Width) + " and height:  " + IntToString(panelInfoBar->Height) + " and NEW width:  " + IntToString(NewWidth) + " and height:  " + IntToString(NewHeight));
  // Only needed when changing text and panel is already visible
  if (!ignore_info_panel_resize)
  {
    info_bar_height_before_resize = panelInfoBar->Height;
    form_height_before_resize = this->Height;
    if(debug_level[generic_forms]>=8 && message_router) message_router->SendMessageToAll("debug", "panelInfoBar CanResize set form_height_before_reize to:  " + IntToString(panelInfoBar->Width) + " and height:  " + IntToString(form_height_before_resize));
  }
}
//---------------------------------------------------------------------------
void __fastcall TformSpecialGeneric::panelInfoBarConstrainedResize(TObject *Sender,
      int &MinWidth, int &MinHeight, int &MaxWidth, int &MaxHeight)
{
  //
}
//---------------------------------------------------------------------------
void __fastcall TformSpecialGeneric::panelInfoBarResize(TObject *Sender)
{
  // We only want one instance of this function called at any given time.  Somehow it was getting called in the middle of already running.
  if (!is_panelInfoBar_resizing)
  {
    is_panelInfoBar_resizing = true;
    if(debug_level[generic_forms]>=8 && message_router) message_router->SendMessageToAll("debug", "panelInfoBar Resize called");
    // If changing text and already visible, then adjust to fit new text
    if (IsVisible(panelInfoBar) && !ignore_info_panel_resize)
    {
      if (!form_height_before_resize > 0)
      {
       form_height_before_resize = this->Height;
       if(debug_level[generic_forms]>=8 && message_router) message_router->SendMessageToAll("debug", "panelInfoBar Resize set form_height_before_reize to:  " + IntToString(panelInfoBar->Width) + " and height:  " + IntToString(form_height_before_resize));
      }
      int info_bar_height_after_resize = panelInfoBar->Height;
      int delta_info_bar_height = info_bar_height_after_resize - info_bar_height_before_resize;
      if(debug_level[generic_forms]>=8 && message_router) message_router->SendMessageToAll("debug", "panelInfoBar Resize is adjusting heights by " + IntToString(delta_info_bar_height) + " pixels.");
      if(debug_level[generic_forms]>=8 && message_router) message_router->SendMessageToAll("debug", "panelInfoBar Resize MinHeight on entry = " + IntToString(this->Constraints->MinHeight));
      if(debug_level[generic_forms]>=8 && message_router) message_router->SendMessageToAll("debug", "panelInfoBar Resize MaxHeight on entry = " + IntToString(this->Constraints->MaxHeight));
      if(debug_level[generic_forms]>=8 && message_router) message_router->SendMessageToAll("debug", "panelInfoBar Resize form height on entry = " + IntToString(this->Height));
      // if MinHeight and MaxHeight are both non-zero, try to preserve their relative relationship
      int rel_delta_form_height = -1;
      if (this->Constraints->MinHeight > 0 && this->Constraints->MaxHeight > 0)
        rel_delta_form_height = this->Constraints->MaxHeight - this->Constraints->MinHeight;
      if (this->Constraints->MinHeight > 0)
        this->Constraints->MinHeight = this->Constraints->MinHeight + delta_info_bar_height;
      if (this->Constraints->MaxHeight > 0)
      {
        if (rel_delta_form_height > -1) // if they were relative before, keep them relative
          this->Constraints->MaxHeight = this->Constraints->MinHeight + rel_delta_form_height;
        else // otherwise, just increase MaxHeight by the difference, like we did with MinHeight
          this->Constraints->MaxHeight = this->Constraints->MaxHeight + delta_info_bar_height;
      }
      this->Height = /*this->Height*/ form_height_before_resize + delta_info_bar_height;
      if(debug_level[generic_forms]>=8 && message_router) message_router->SendMessageToAll("debug", "panelInfoBar Resize MinHeight on exit = " + IntToString(this->Constraints->MinHeight));
      if(debug_level[generic_forms]>=8 && message_router) message_router->SendMessageToAll("debug", "panelInfoBar Resize MaxHeight on exit = " + IntToString(this->Constraints->MaxHeight));
      if(debug_level[generic_forms]>=8 && message_router) message_router->SendMessageToAll("debug", "panelInfoBar Resize form height on exit = " + IntToString(this->Height));
    }
    is_panelInfoBar_resizing = false;
  }
}
//---------------------------------------------------------------------------
void TformSpecialGeneric::AdjustInfoBarSize()
{
 LogIt("specialgeneric.log", "AdjustInfoBarSize() entry");
 if (IsVisible(panelInfoBar) && lblInformation->Visible)
 {
  lblInformation->AutoSize = false;
  lblInformation->Width = panelInfoBar->Width - (lblInformation->Left * 2); // jhaley: This works.
  lblInformation->AutoSize = true;
  if (panelInfoBar->Height != lblInformation->Height + 13)
    panelInfoBar->Height = lblInformation->Height + 13;
  LogIt("specialgeneric.log", "lblInformation sized to " + IntToString(lblInformation->Width) + "x" + IntToString(lblInformation->Height));
 }
}
//---------------------------------------------------------------------------
void TformSpecialGeneric::ShowInfoBar()
{
  if(debug_level[generic_forms]>=8 && message_router) message_router->SendMessageToAll("debug", "ShowInfoBar called");
  if (this->Visible && !panelInfoBar->Visible)
  {
    if (lblInformation->Visible) // If using default controls, adjust panel height automatically
    {
      ignore_info_panel_resize = true;
      panelInfoBar->Visible = true;
      // sometimes the first pass through, one or the other will still have a height of 0, causing the ->Height
      // of the other to return some really bizarre results.  So we loop until height has stabilized before
      // counting on it being accurate...
      int prev_delta_info_bar_height;
      int delta_info_bar_height = panelInfoBar->Height;
      do
      {
        prev_delta_info_bar_height = delta_info_bar_height;
        delta_info_bar_height = lblInformation->Height + 13;
        if(debug_level[generic_forms]>=8 && message_router) message_router->SendMessageToAll("debug", "ShowInfoBar is setting panel height to " + IntToString(delta_info_bar_height) + " pixels.");
        panelInfoBar->Height = delta_info_bar_height;
      } while (delta_info_bar_height != prev_delta_info_bar_height);
      ignore_info_panel_resize = false;
      panelInfoBarResize(this);
    }
/*
    if (this->Constraints->MinHeight > 0)
      this->Constraints->MinHeight = this->Constraints->MinHeight + delta_info_bar_height;
    if (this->Constraints->MaxHeight > 0)
      this->Constraints->MaxHeight = this->Constraints->MaxHeight + delta_info_bar_height;
    this->Height = this->Height + delta_info_bar_height;
*/
    panelInfoBar->Visible = true;
    AdjustHeight(CalcHeight());
  }
}
//---------------------------------------------------------------------------
void TformSpecialGeneric::HideInfoBar()
{
  if(debug_level[generic_forms]>=8 && message_router) message_router->SendMessageToAll("debug", "HideInfoBar called");
  if (panelInfoBar->Visible)
  {
    if (lblInformation->Visible) // If not using default controls, don't mess with the height
    {

      int delta_info_bar_height = panelInfoBar->Height;
      if(debug_level[generic_forms]>=8 && message_router) message_router->SendMessageToAll("debug", "HideInfoBar is adjusting heights by " + IntToString(delta_info_bar_height) + " pixels.");
      ignore_info_panel_resize = true;
      panelInfoBar->Height = 0;
      ignore_info_panel_resize = false;
    }
/*
    if (this->Constraints->MinHeight > 0)
      this->Constraints->MinHeight = this->Constraints->MinHeight - delta_info_bar_height;
    if (this->Constraints->MaxHeight > 0)
      this->Constraints->MaxHeight = this->Constraints->MaxHeight - delta_info_bar_height;
    this->Height = this->Height - delta_info_bar_height;
*/
    panelInfoBar->Visible = false;
    AdjustHeight(CalcHeight());
  }
}
//---------------------------------------------------------------------------
int TformSpecialGeneric::CalcHeight()
{
  if(debug_level[generic_forms]>=8 && message_router) message_router->SendMessageToAll("debug", "CalcHeight called");
  int title_and_border = this->Height - this->ClientHeight + 1;
  if(debug_level[generic_forms]>=8 && message_router) message_router->SendMessageToAll("debug", "CalcHeight title_and_border = " + IntToString(title_and_border));
  int info_bar_height  = (panelInfoBar->Visible) ? panelInfoBar->Height : 0;
  if(debug_level[generic_forms]>=8 && message_router) message_router->SendMessageToAll("debug", "CalcHeight info_bar_height = " + IntToString(info_bar_height));
  int total_height = title_and_border + info_bar_height + scrollStandard->Height + toolsStandard->Height;
  if(debug_level[generic_forms]>=8 && message_router) message_router->SendMessageToAll("debug", "CalcHeight scroll_standard_height = " + IntToString(scrollStandard->Height));
  if(debug_level[generic_forms]>=8 && message_router) message_router->SendMessageToAll("debug", "CalcHeight tools_standard_height = " + IntToString(toolsStandard->Height));
  if(debug_level[generic_forms]>=8 && message_router) message_router->SendMessageToAll("debug", "CalcHeight total_height = " + IntToString(total_height));
  //TODO -c HACK:  I'm still off a pixel?  Why am I off a pixel?
  return total_height + 1;
}
//---------------------------------------------------------------------------
void TformSpecialGeneric::AdjustHeight(int total_height)
// Set the height of the form based on the heights of the individual stacked controls (and which are currently visible)
{
  // If current form constraints do not accomodate all the things, adjust them
  // Note that if Min and Max Height are the same, they will get "pulled" along with the other, when the
  // other changes.  Thus for these forms, the form will always "fit" the content.  For forms that set a Min
  // but not Max height, Min will get pushed out when it doesn't accomodate all content inside the form, but
  // will NOT get pushed back in, if the content shrinks again (InfoBar goes away, etc.).  Not sure how much
  // of a problem, if any, this is.
  if(debug_level[generic_forms]>=8 && message_router) message_router->SendMessageToAll("debug", "AdjustHeight MinHeight on entry = " + IntToString(this->Constraints->MinHeight));
  if(debug_level[generic_forms]>=8 && message_router) message_router->SendMessageToAll("debug", "AdjustHeight MaxHeight on entry = " + IntToString(this->Constraints->MaxHeight));
  if(debug_level[generic_forms]>=8 && message_router) message_router->SendMessageToAll("debug", "AdjustHeight form height on entry = " + IntToString(this->Height));
  if (this->Constraints->MaxHeight > 0 && this->Constraints->MaxHeight < total_height)
  {
    if (this->Constraints->MinHeight == this->Constraints->MaxHeight) // if matched before, make them match again
      this->Constraints->MinHeight = total_height;
    this->Constraints->MaxHeight = total_height;
  }
  if (this->Constraints->MinHeight > 0 && this->Constraints->MinHeight > total_height)
  {
    if (this->Constraints->MaxHeight == this->Constraints->MinHeight) // if matched before, make them match again
      this->Constraints->MaxHeight = total_height;
    this->Constraints->MinHeight = total_height;
  }
  this->Height = total_height;
  if(debug_level[generic_forms]>=8 && message_router) message_router->SendMessageToAll("debug", "AdjustHeight MinHeight on exit = " + IntToString(this->Constraints->MinHeight));
  if(debug_level[generic_forms]>=8 && message_router) message_router->SendMessageToAll("debug", "AdjustHeight MaxHeight on exit = " + IntToString(this->Constraints->MaxHeight));
  if(debug_level[generic_forms]>=8 && message_router) message_router->SendMessageToAll("debug", "AdjustHeight form height on entry = " + IntToString(this->Height));
}
//---------------------------------------------------------------------------
bool TformSpecialGeneric::RegisterNewFilter(string name)
{
  if (filter_atlas_handler.find(name)!=filter_atlas_handler.end()) // key already in map
    return false;
  filter_atlas_handler.insert(pair<string, filter_atlas_wrapper>(name, filter_atlas_wrapper(new classFilterObject()  ) ));
  return true;
}
//---------------------------------------------------------------------------
void TformSpecialGeneric::ResetAllFilters()
{
  for (map<string, filter_atlas_wrapper>::iterator itr = filter_atlas_handler.begin(); itr != filter_atlas_handler.end(); itr++)
    filter_atlas_handler[itr->first].Reset(this);
}
//---------------------------------------------------------------------------
/*
void TformSpecialGeneric::TerminateAllFilters()
{
  for (map<string, filter_atlas_wrapper>::iterator itr = filter_atlas_handler.begin(); itr != filter_atlas_handler.end(); itr++)
    filter_atlas_handler[itr->first].Terminate();
}
*/
//---------------------------------------------------------------------------
bool TformSpecialGeneric::HandleAtlasRequest(int request_num)
{
  bool result = false;
  for (map<string, filter_atlas_wrapper>::iterator itr = filter_atlas_handler.begin(); itr != filter_atlas_handler.end(); itr++)
    result = result || filter_atlas_handler[itr->first].Handle(request_num);
  return result;
}
//---------------------------------------------------------------------------
// jhaley 20101210: Atlas refresh events can be received at any time that the application message pump is running. Unfortunately
// that includes during the execution of task-modal forms/dialogs, when the form doesn't want to change (for example if EditSql
// is called from any method dispatched from an Atlas refresh handler). Call this from forms that are having such problems to
// find out if the form is in a good state for handling Atlas refreshes.
bool TformSpecialGeneric::IsAtlasReady()
{
  // We never want to be running refreshes on forms that are in weird conditions (destroyed or unconstructed).
  // The latter is unlikely, but the former is within the realm of possibilties.
  if (IsDestroyedForm(this, "IsAtlasReady", __LINE__) || !generic_form_state)
    return false;

  // Performing Atlas refresh during task modal loops has proven to be exceptionally dangerous.
  if (in_task_modal_loop)
    return false;

  return true;
}
//---------------------------------------------------------------------------
