//---------------------------------------------------------------------------

#ifndef codeSpecialGenericH
#define codeSpecialGenericH
//---------------------------------------------------------------------------
#include <Classes.hpp>
#include <Controls.hpp>
#include <StdCtrls.hpp>
#include <Forms.hpp>
#include <ComCtrls.hpp>
#include <ToolWin.hpp>

#include "utilityheaders.h"
#include "baseheaders.h"
#include "components.h"
#include <ExtCtrls.hpp>
#include <Menus.hpp>

class TformDrawerHelp;
//---------------------------------------------------------------------------
class filter_atlas_wrapper
{
private:
  mutable bool terminate_when_done;
public:
  filter_atlas_wrapper(classFilterObject* p_filter = NULL, int p_request = 0)
  {
    terminate_when_done = true;

    if (p_filter)
      filter = p_filter;
    else
      filter  = new classFilterObject();
    request = p_request;
  }
  // if the copy constructor is called, preserve existing filter pointer
  filter_atlas_wrapper(const filter_atlas_wrapper & old_fa)
  {
    terminate_when_done = old_fa.terminate_when_done; // migrates to new object
    old_fa.terminate_when_done = false; // old object should NOT actually destruct

    filter  = old_fa.filter;
    request = old_fa.request;
  }
  ~filter_atlas_wrapper()
  {
    if(terminate_when_done)
      Terminate();
  }
  bool Handle(int received_request)
  {
    if (request && request == received_request)
    {
      if (!filter)
        return false;

      filter->RefreshDataSet();
      return true;
    }
    return false;
  }
  bool Reset(TObject* callback_form)
  {
    if (!filter) return false;
    filter->SetSQLStatement(string(""));
    filter->RefreshDataSet();
    filter->SetAddEdit(callback_form, NULL, NULL);
    return true;
  }
  void Terminate()
  {
    if (filter)  delete filter;
    if (request) atlas->CancelRequest (request);
  }
  classFilterObject* filter;
  int                request;
};
//---------------------------------------------------------------------------
class TformSpecialGeneric : public TForm
{
public:
        typedef string __fastcall (__closure *TGetDBOverrideSection)();
        static TGetDBOverrideSection funcGetDBOverrideSection;
__published:	// IDE-managed Components
        TToolBar *toolsStandard;
        TScrollBox *scrollStandard;
        TToolButton *sepStandard;
        TButton *buttonEscape;
        TPanel *panelInfoBar; // Do not set panelInfoBar->Visible directly!  Use Show/HideInfoBar functions.
        TLabel *lblInformation;
        TBevel *bevelStandardB; // Set lblInformation->Text via the InfoBarText property!  Is public only because (a) run time errors when made private and (b) should be able to make invisible if you want to use only your own controls in the info bar
        void __fastcall FormClose(TObject *Sender, TCloseAction &Action);
        void __fastcall CreateParams(TCreateParams &Params);
        void __fastcall FormShortCut(TWMKey &Msg, bool &Handled);
        void __fastcall FormShow(TObject *Sender);
        void __fastcall FormCloseQuery(TObject *Sender, bool &CanClose);
        void __fastcall FormResize(TObject *Sender);
        void __fastcall FormActivate(TObject *Sender);
        void __fastcall FormDeactivate(TObject *Sender);
        void __fastcall FormHide(TObject *Sender);
        void __fastcall buttonEscapeClick(TObject *Sender);
        void __fastcall toolsStandardAdvancedCustomDraw(TToolBar *Sender, const TRect &ARect, TCustomDrawStage Stage, bool &DefaultDraw);
        void __fastcall toolsStandardResize(TObject *Sender);
        void __fastcall toolsStandardMouseDown(TObject *Sender, TMouseButton Button, TShiftState Shift, int X, int Y);
        void __fastcall toolsStandardMouseMove(TObject *Sender, TShiftState Shift, int X, int Y);
        void __fastcall toolsStandardMouseUp(TObject *Sender, TMouseButton Button, TShiftState Shift, int X, int Y);
        void __fastcall panelInfoBarResize(TObject *Sender);
        void __fastcall panelInfoBarConstrainedResize(TObject *Sender, int &MinWidth, int &MinHeight, int &MaxWidth, int &MaxHeight);
        void __fastcall panelInfoBarCanResize(TObject *Sender, int &NewWidth, int &NewHeight, bool &Resize);

private:	// User declarations
        void __fastcall WMSysCommand(TWMSysCommand &Msg);
        bool is_processing;
        int info_bar_height_before_resize;
        int form_height_before_resize;
        bool in_task_modal_loop; // jhaley 20101210: some things shouldn't happen during task modal loops
protected:
        // jhaley 20100831: Keep track of the form's state with respect to whether it has been fully constructed yet.
        // Seems VCL can also send events to forms before their constructor sequence is finished....
        enum
        {
          FORM_STATE_CONSTRUCTED   = -1, // set all bits when asserted, for maximum confidence.
          FORM_STATE_UNCONSTRUCTED =  0  // should be zero because it is the default uninitialized value in C++.
        } generic_form_state;

        string class_name;
        TWinControl *last_active_control; // debug -- sometimes during Processing(true) i needed to set the ->Checked status of a radiobutton ... but when it switches back, it unsets my change.

        // forms like RecordPerson that have a large number of filters to register with atlas...this stuff simplifies the process
        map<string, filter_atlas_wrapper> filter_atlas_handler;
        bool RegisterNewFilter(string name);
        bool HandleAtlasRequest(int request_num);
        void ResetAllFilters();
        //void TerminateAllFilters(); // not anymore ... the filter_atlas_wrapper has a proper destructor now that plays nice with the copy constructor

        bool IsAtlasReady(); // jhaley 20101210: Sometimes forms need to be able to ignore an Atlas refresh based on things going on in this class.

        map<string, set<string> > pending_atlas_notifications;
        void AtlasClear();
        void AtlasAdd(string key, string value);
        void AtlasAdd(string table_name, string field_name, string field_value);
        void AtlasAdd(string table_name, string field_name, vector<string> & field_values);
        void AtlasAdd(string table_name, map<string, string> & field_names_values);
        void AtlasAdd(string table_name, vector<map<string, string> > & field_names_values);
        void AtlasAdd(map<string, set<string> > & entire_atlas_chunk);
        void AtlasSend();

        bool virtual IsProcessing ();

        // related to the top-left icon so we can reuse stuff from imagelists
        int icon_index;
        TImageList* image_list;
        void SetFormIcon ();
        void __fastcall virtual SetImageList (TImageList* Value);
        void __fastcall virtual SetIconIndex (int Value);

        // related to resizing via sizegrip
        bool sizegrip_dragging;
        int  sizegrip_x;
        int  sizegrip_y;

        // related to caption buttons
        // ... data
        RECT help_button_rect;
        POINT last_cursor_position;
        bool is_help_button_captured;
        bool is_help_button_pushed;
        bool is_help_button_enabled;
        bool is_help_button_visible;
        set<string> available_help_topics;
        TformDrawerHelp * help_drawer;
        // ... functions
        void ResetHelpButtonRect(int form_width);
        void DrawHelpButton();
        void virtual HelpButtonPressed();
        void __fastcall virtual SetHelpButtonEnabled (bool Value);
        void __fastcall virtual SetHelpButtonVisible (bool Value);

        void __fastcall virtual SetMaximizeButtonEnabled (bool Value);
        void virtual ResetMaximizeButton ();

        bool is_panelInfoBar_resizing;

        // related to side-windows and other optional assisting thingies (drawers in MacOS terminology)
        // ... data
        RECT window_rect;
        TformSpecialGeneric * drawer_owner;
        bool                  is_drawer_destroyable;
        bool                  is_drawer_expanded;
        bool                  is_drawer_attached;
        TAnchorKind           drawer_position;
        set<TformSpecialGeneric *> drawers;
        int                   active_count;
        int                   disable_count;
        static const int      drawer_autoclose_size = 12;
        static const int      drawer_autoopen_size = 32; // should be bigger than the above -- replacement for an "ideal" size?
        static const int      drawer_minimum_static_size = 150; // detach automatically if it's going to be tiny.

        bool ignore_info_panel_resize;
        HMENU DebugLibrariesMenu;
        HMENU DebugGenericMenu;
        HMENU DebugOtherMenu;
        
        // ... functions
        void __fastcall virtual SetDrawerOwner    (TformSpecialGeneric * Value);
        void __fastcall virtual SetDrawerExpanded (bool Value);
        void __fastcall virtual SetDrawerPosition (TAnchorKind Value);
        void __fastcall virtual SetDrawerAttached (bool Value);
        void __fastcall virtual SetActiveCount (int Value);
        string __fastcall virtual GetInfoBarText ();
        void   __fastcall virtual SetInfoBarText (string Value);
        int virtual CalcHeight();
        void virtual AdjustHeight(int total_height);
        void virtual AdjustInfoBarSize();
        void virtual ShowInfoBar();
        void virtual HideInfoBar();
        POINT point(long x, long y);
        void virtual DrawerRefresh ();
        void virtual WindowMoved (bool snap_self);
        TformSpecialGeneric * DrawerLeader ();
        void virtual DrawerIsChanging (TformSpecialGeneric * Drawer, bool Expanding);
        void AddSysMenuItems();

public:		// User declarations
        __fastcall TformSpecialGeneric(TComponent* Owner, TformSpecialGeneric * pDrawerOwner = NULL);
        __fastcall virtual ~TformSpecialGeneric();
        bool virtual IsBusy();    // asked so that the user is prompted to close or cancel on [X]
        bool virtual CloseNow();            // force a remote closing
        int PSay(string p_message, prometheus_message_types p_message_type, int p_buttons, string p_other_info = "");
        void PPretendToSay(string p_message, prometheus_message_types p_message_type, string p_other_info = "");
        void FocusAway();
        void FocusBack();
        void SwitchRadioSelection (TRadioButton * to_check);

        void DisableIncrease ();
        void DisableDecrease ();

        void virtual Processing (bool p_is_processing);

        __property TImageList* ImageList = {read=image_list, write=SetImageList};
        __property int         IconIndex = {read=icon_index, write=SetIconIndex};

        __property bool        HelpButtonEnabled = {read=is_help_button_enabled, write=SetHelpButtonEnabled, default=false};
        __property bool        HelpButtonVisible = {read=is_help_button_visible, write=SetHelpButtonVisible, default=false};

        __property TformSpecialGeneric * DrawerOwner = {read=drawer_owner, write=SetDrawerOwner};
        __property bool        DrawerExpanded = {read=is_drawer_expanded, write=SetDrawerExpanded, default=false};
        __property TAnchorKind DrawerPosition = {read=drawer_position, write=SetDrawerPosition, default=akRight};
        __property bool        DrawerAttached = {read=is_drawer_attached, write=SetDrawerAttached, default=true};
        __property int         ActiveCount = {read=active_count, write=SetActiveCount, default=0};
        __property string      InfoBarText = {read=GetInfoBarText, write=SetInfoBarText};
        // saying 'virtual' doesn't do anything here, as it's declared as such in an ancestor ... can't make it -not- be virtual ('final' in Java?)
        // IMPORTANT: specialGeneric redirects the flow of messages, possibly usurping power and otherwise claiming a throne!  King me.
        virtual void __fastcall Dispatch(void *Message);
	virtual int __fastcall ShowTaskModal(TformSpecialGeneric * ModalParent);
        // rather than override CloseModal (which is private, non-virtual), let's change the OnClose to do stuff.
};
//---------------------------------------------------------------------------
//extern PACKAGE TformSpecialGeneric *formSpecialGeneric;
//---------------------------------------------------------------------------
set<TformSpecialGeneric *> FindOpenFormsByType(string class_name, bool exact_match_only = false);

#endif
