//---------------------------------------------------------------------------

#include <vcl.h>
#pragma hdrstop

#include "codeSpecialHouseholdMod.h"
#include "codeRecordPerson.h"
//---------------------------------------------------------------------------
#pragma package(smart_init)
#pragma link "codeSpecialGeneric"
#pragma resource "*.dfm"
TformSpecialHouseholdMod *formSpecialHouseholdMod;
//---------------------------------------------------------------------------
__fastcall TformSpecialHouseholdMod::TformSpecialHouseholdMod(TComponent* Owner, string p_household_id, TIBTransaction * p_dbTransaction)
        : TformSpecialGeneric(Owner)
{
  dbTransaction = p_dbTransaction;

  household_filter.SetListViewHandle(listMembers);
  household_filter.SetSecurity("people");
  household_filter.SetTransaction(dbTransaction); // read stuff from this transaction rather than read-commited
  household_filter.AddColumnSpec("Name", "people", as_text, true, 150);
  household_filter.AddColumnSpec("Birthday", "people", as_date, true, 90);
  household_filter.AddColumnSpec("Gender", "people", as_text, true, 90);
  household_filter.AddColumnSpec("Current Id", "current_id", as_int, false, 80);
  household_filter.AddColumnSpec("Internal Id", "internal_id", as_int, false, 80);
  household_filter.FetchPreferences("specialhouseholdmod.household_filter");
  household_filter.SetAddEdit(this, NULL, EditRecord(&(TformSpecialHouseholdMod::EditPerson)));
  household_filter.SetCheck(false);
  household_filter.SetCompress(false);
  string sql;

  if (!p_household_id.empty() && IsInt(p_household_id) && StringToInt(p_household_id) != 0)
  {
   household_id = p_household_id;
   sql = "select person_names.last_name || ', ' || person_names.first_name || ' ' || person_names.suffix full_name, "
         "people.birth_date, gender_types.item, people.current_id, people.id  "
         "from people "
         "left join person_names on people.person_names_id = person_names.id "
         "left join gender_types on people.gender_id = gender_types.id "
         "where people.head_household_id = " + household_id +
         "order by person_names.last_name, person_names.first_name";
  }
  household_filter.SetSQLStatement(sql);
  household_filter.ForceRefreshDataSet();
  set<string> bob;
  bob.insert(household_id);
  household_filter.SetCheckedWhereID (bob, true);

  // fixed: force refresh, then select the right row. period.

  // done -- want to be able to check [x] just the one that is currently it ... but because
  // of changes in the way filter works, RefreshDataSet() isn't guaranteed to have been called.
  // need some way to do something about it. a callback that happens when is_pending is reset to
  // false, perhaps? do "this" when you -actually- can? or the feature of saying "select all rows
  // if "blah" is true ... possibly also as a callback, returning true or false based on row data)
  //
  // this was done in filter, but not used here. instead, the code selects a new head of household,
  // if not already one... and lets you change it. but no multiple-head thing going on.
}
//---------------------------------------------------------------------------
void __fastcall TformSpecialHouseholdMod::buttonSaveClick(TObject *Sender)
{
 // save the changes
 if (!household_id.empty() && IsInt(household_id) && StringToInt(household_id) != 0)
 {
  string selected_head = household_filter.GetSelectedRecordID();
  if (selected_head == "")
  {
   PSay("You must select a member to be the head of the household.", m_missing_data, MB_OK);
  } else
  {
   // actually save the change!
   try
   {
    string sql = "update people set head_household_id = " + selected_head + " where head_household_id = " + household_id;
    if (ExecuteStatement(dbTransaction, sql))
    {
     ModalResult = mrOk;
    } else
    {
     PSay("Something went wrong while trying to update the head of household information.\nPerhaps someone else is doing this at the same time?", m_database_error, MB_OK);
     ModalResult = mrAbort;
    }
   }
   catch(...)
   {
    PSay("Something went wrong while trying to update the head of household information.\nPerhaps someone else is doing this at the same time?", m_database_error, MB_OK);
    ModalResult = mrAbort;
   }
  }
 } else
 {
  PSay("This screen was not given a valid household identifier.", m_program_error, MB_OK);
  ModalResult = mrAbort;
 }
}
//---------------------------------------------------------------------------
void TformSpecialHouseholdMod::EditPerson()
{
  map<string,string> temp_map;
  temp_map["id"] = household_filter.GetSelectedRecordID();
  FORM_TRY
    TformRecordPerson * personRecord = new TformRecordPerson(NULL, temp_map);
    personRecord->Show();
  FORM_CATCH
}
//---------------------------------------------------------------------------
void __fastcall TformSpecialHouseholdMod::buttonCancelClick(TObject *Sender)
{
 ModalResult = mrCancel;
}
//---------------------------------------------------------------------------
bool TformSpecialHouseholdMod::FindHousehold(TIBDatabase * p_dbDatabase, set<string> & p_people)
{
 TIBTransaction * t = new TIBTransaction(NULL);
 StdTransaction(t, p_dbDatabase);
 bool result = FindHousehold(t, p_people);
 if (result)
  t->Commit();
 else
  t->Rollback();
 delete t;
 return result;
}
//---------------------------------------------------------------------------
bool TformSpecialHouseholdMod::FindHousehold(TIBTransaction * p_dbTransaction, set<string> & p_people)
{
 // find all people who are, through the same_household boolean, related to people in p_people
 bool commit_when_done = true;
 string sql;
 string related_to;
 string not_one_of;
 set<string> temp_found;
 set<string> check_next = p_people;

 if (p_dbTransaction->Active)
  commit_when_done = false;
 else
  p_dbTransaction->StartTransaction();

 while (!check_next.empty())
 {
  related_to = SetToCommaString(check_next);
  not_one_of = SetToCommaString(p_people);
  check_next.clear();

  // a
  temp_found.clear();
  sql = "select relationships.person_a_id from relationships where "
        "relationships.person_b_id in (" + related_to + ") and "
        "relationships.person_a_id not in (" + not_one_of + ") and "
        "relationships.same_household = 1 and "
        "relationships.person_a_id != relationships.person_b_id and "
        "relationships.current_id > 2";
  if (SqlToSet(p_dbTransaction, sql, temp_found))
   check_next.insert(temp_found.begin(), temp_found.end());
  else
   return false;

  // b
  temp_found.clear();
  sql = "select relationships.person_b_id from relationships where "
        "relationships.person_a_id in (" + related_to + ") and "
        "relationships.person_b_id not in (" + not_one_of + ") and "
        "relationships.same_household = 1 and "
        "relationships.person_a_id != relationships.person_b_id and "
        "relationships.current_id > 2";
  if (SqlToSet(p_dbTransaction, sql, temp_found))
   check_next.insert(temp_found.begin(), temp_found.end());
  else
   return false;

  p_people.insert(check_next.begin(), check_next.end());
 }

 if (commit_when_done)
  p_dbTransaction->Commit();
 return true;
}
//---------------------------------------------------------------------------
bool TformSpecialHouseholdMod::FindRelations(TIBDatabase * p_dbDatabase, vector<string> & p_people)
{
 TIBTransaction * t = new TIBTransaction(NULL);
 StdTransaction(t, p_dbDatabase);
 bool result = FindRelations(t, p_people);
 if (result)
  t->Commit();
 else
  t->Rollback();
 delete t;
 return result;
}
//---------------------------------------------------------------------------
bool TformSpecialHouseholdMod::FindRelations(TIBTransaction * p_dbTransaction, vector<string> & p_people, bool put_in_temp_table)
{
 // find all people who are, through any relationship, related to people in p_people
 bool commit_when_done = true;
 string sql1, sql2;
 string related_to;
 string not_one_of;
 set<string> temp_found;
 set<string> check_next;
 VecToSet(p_people, check_next);

 if (p_dbTransaction->Active)
  commit_when_done = false;
 else
  p_dbTransaction->StartTransaction();

 if (put_in_temp_table)
 {
   ExecuteStatement(p_dbTransaction, "delete from temp_relationship_data");
 }

 while (!check_next.empty())
 {
  related_to = SetToCommaString(check_next);
  not_one_of = VecToCommaString(p_people);
  check_next.clear();

  // a
  temp_found.clear();
  sql1 = "select relationships.person_a_id from relationships where "
         "relationships.person_b_id in (" + related_to + ") and "
         "relationships.person_a_id not in (" + not_one_of + ") and "
         "relationships.person_a_id != relationships.person_b_id and "
         "relationships.current_id > 2";
  if (SqlToSet(p_dbTransaction, sql1, temp_found))
   check_next.insert(temp_found.begin(), temp_found.end());
  else
   return false;

  // b
  temp_found.clear();
  sql2 = "select relationships.person_b_id from relationships where "
         "relationships.person_a_id in (" + related_to + ") and "
         "relationships.person_b_id not in (" + not_one_of + ") and "
         "relationships.person_a_id != relationships.person_b_id and "
         "relationships.current_id > 2";
  if (SqlToSet(p_dbTransaction, sql2, temp_found))
   check_next.insert(temp_found.begin(), temp_found.end());
  else
   return false;

  p_people.insert(p_people.end(), check_next.begin(), check_next.end());

  if (put_in_temp_table)
  {
    ExecuteStatement(p_dbTransaction, "insert into temp_relationship_data " + sql1); // and not exists in table already?  Or just select distinct later?
    ExecuteStatement(p_dbTransaction, "insert into temp_relationship_data " + sql2); // and not exists in table already?  Or just select distinct later?
  }
 }

 if (commit_when_done)
  p_dbTransaction->Commit();
 return true;
}
//---------------------------------------------------------------------------
bool TformSpecialHouseholdMod::ResolveHouseholdInformation(TIBDatabase * p_dbDatabase, set<string> p_people)
{
 TIBTransaction * t = new TIBTransaction(NULL);
 StdTransaction(t, p_dbDatabase);
 bool result = ResolveHouseholdInformation(t, p_people);
 if (result)
  t->Commit();
 else
  t->Rollback();
 delete t;
 return result;
}
//---------------------------------------------------------------------------
bool TformSpecialHouseholdMod::ResolveHouseholdInformation(TIBTransaction * p_dbTransaction, set<string> p_people)
{
 if (p_people.empty())
 {
  ::PSay("Warning: a programmer forgot to pass a list of people for whom to fix household information.\nThis is not being treated as an error, but someone should fix this. Please tell your programmer where you saw this happen.", m_debug, MB_OK, "TformSpecialHouseholdMod");
  return true;
 }

 // take the set of people given (hopefully the set contains all known people who may
 // be affected) and figure out what to do with them, household-wise.
 // for example,
 // after modifying a relationship record, you should call FindHousehold() with a set
 // containing the id's of the two people involved. that set should then be passed here,
 // where the household(s) will be recalculated for all people involved. if anything needs
 // to be done, the user will be asked.

 // people.household_id needs to at least be set = to people.id on creation of the person

 // algorithm:
 // get the full listing of tuples (id, id) from same_household relationships, involving only these people
 //
 // for each record,
 //  if neither id is in any group yet, create a new group, put both id's in there
 //  if one id is in a group, and not the other, add the other to the group
 //  if both id's are in the same group, keep going
 //  if both id's are in different groups, migrate all members of one group to the other
 //
 // for each person not already in a group, create new group just for them
 //
 // for each group,
 //  find out what all household_id's exist among them
 //  if none, select a head, and ask the user if more than one possible head
 //  if one,
 //   if the head of household is one of the members, keep going
 //   if the head of household isn't, then make it so, then ask the user to figure it out if several are possible
 //  if multiple, select one, and make the user chose a head.

 // get the full listing of relationships pertaining to this action
 // TODO: this would be better as a list<...> but no such SqlLib function exists yet, too lazy to create one
 bool commit_when_done = true;
 vector<map<string, string> > vec_links;
 string one_of = SetToCommaString(p_people);

 if (p_dbTransaction->Active)
  commit_when_done = false;
 else
  p_dbTransaction->StartTransaction();

 string sql_get_relationships = "select relationships.person_a_id a, relationships.person_b_id b "
                                "from relationships where "
                                "relationships.person_a_id in (" + one_of + ") and "
                                "relationships.person_b_id in (" + one_of + ") and "
                                "relationships.same_household = 1 and "
                                "relationships.person_a_id != relationships.person_b_id and "
                                "relationships.current_id > 2";
 if (!SqlToVecMap(p_dbTransaction, sql_get_relationships, vec_links))
 {
  // error getting the list. abort.
  ::PSay("Failed to retrieve the list of relevant relationships.", m_database_error, MB_OK, "TformSpecialHouseholdMod");
  if (commit_when_done)
   p_dbTransaction->Rollback();
  return false;
 }

 // group the person_id's into groups that make sense, a list of households. (list of sets)
 list<set<string> > list_households;
 list<set<string> >::iterator i_e;
 list<set<string> >::iterator i_a;
 list<set<string> >::iterator i_b;
 list<set<string> >::iterator i_t;
 string a;
 string b;

 for (vector<map<string, string> >::iterator i_link = vec_links.begin(); i_link != vec_links.end(); i_link++)
 {
  i_e = list_households.end();
  i_a = i_e;
  i_b = i_e;
  a = (*i_link)["a"];
  b = (*i_link)["b"];
  for (list<set<string> >::iterator i_hh = list_households.begin(); i_hh != i_e; i_hh++)
  {
   if (i_hh->find(a) != i_hh->end())
    i_a = i_hh;
   if (i_hh->find(b) != i_hh->end())
    i_b = i_hh;
   if (i_a != i_e && i_b != i_e)
    break; // stop looking early if we've found them both
  }
  if (i_a != i_e && i_b != i_e)
  {
   // found both id's somewhere
   if (i_a != i_b)
   {
    // a and b are in different groups, merge b into a, and clear b.
    i_a->insert(i_b->begin(), i_b->end());
    i_b->clear();
   } else
   {
    // a and b are in the same group already, just ignore
   }
  } else
  if (i_a != i_e)
  {
   // found a, but not b
   i_a->insert(b);
  } else
  if (i_b != i_e)
  {
   // found b, but not a
   i_b->insert(a);
  } else
  {
   // found neither
   set<string> new_set;
   new_set.insert(a);
   new_set.insert(b);
   list_households.push_back(new_set);
  }
 }

 // all people who perhaps -used- to be part of a household and are now alone,
 // need to have their own household even though there are no links. yeah. that.
 bool already_somewhere;
 for (set<string>::iterator i_person = p_people.begin(); i_person != p_people.end(); i_person++)
 {
  i_e = list_households.end();
  already_somewhere = false;
  for (list<set<string> >::iterator i_hh = list_households.begin(); i_hh != i_e; i_hh++)
  {
   if (i_hh->find(*i_person) != i_hh->end())
   {
    already_somewhere = true;
    break;
   }
  }
  if (!already_somewhere)
  {
   set<string> new_set;
   new_set.insert(*i_person);
   list_households.push_back(new_set);
  }
 }

 set<string> need_more_work;

 // resolve each group as a single item -- a transaction might be good here.
 string comma_members;
 string sql_hh;
 string sql_update;
 string sql_pre_update;
 string new_head;
 bool should_ask;
 set<string> existing_heads_of_hh;
 i_e = list_households.end();

 for (list<set<string> >::iterator i_hh = list_households.begin(); i_hh != i_e; i_hh++)
 {
  if (i_hh->size() == 0)
  {
   // empty set, ignore.
  } else
  {
   existing_heads_of_hh.clear();
   comma_members = SetToCommaString(*i_hh);
   sql_hh = "select distinct people.head_household_id from people where "
            "people.id in (" + comma_members + ") and people.head_household_id > 0";
   if (SqlToSet(p_dbTransaction, sql_hh, existing_heads_of_hh))
   {
    should_ask = false;
    if (existing_heads_of_hh.size() == 0)
    {
     // nobody had a head_household_id, so we'll make them decide on one.
     new_head = *(i_hh->begin());
     should_ask = true;
    } else
    {
     // use the first known head of household that is in the group, otherwise default to first person
     new_head = *(i_hh->begin());
     should_ask = true;
     for (set<string>::iterator i_head = existing_heads_of_hh.begin(); i_head != existing_heads_of_hh.end(); i_head++)
     {
      if (i_hh->find(*i_head) != i_hh->end())
      {
       new_head = (*i_head);
       if (existing_heads_of_hh.size() == 1)
        should_ask = false;
       break;
      }
     }
    }

    // regardless, do the following: (and try to limit the number of people getting updated, to keep the possible conflicts at a minimum)
    sql_update = "update people set head_household_id = " + new_head + " where id in (" + comma_members + ") and head_household_id != " + new_head;
    if (!ExecuteStatement(p_dbTransaction, sql_update))
    {
     ::PSay("Failed to update some of the people in this household;\none of them is probably being modified by someone:\nplease make sure you do not currently have their record open for editing.", m_database_error, MB_OK, "TformSpecialHouseholdMod");
     if (commit_when_done)
      p_dbTransaction->Rollback();
     return false;
    }

    if (should_ask && i_hh->size() > 1)
     need_more_work.insert(new_head);
   } else
   {
    // error, bail out.
    ::PSay("Failed to retrieve the list of possible heads of household for this household.", m_database_error, MB_OK, "TformSpecialHouseholdMod");
    if (commit_when_done)
     p_dbTransaction->Rollback();
    return false;
   }
  }
 }

 // for every need_more_work, ask the user.
 for (set<string>::iterator i_work = need_more_work.begin(); i_work != need_more_work.end(); i_work++)
 {
  TformSpecialHouseholdMod * ask_user_form = new TformSpecialHouseholdMod(NULL, *i_work, p_dbTransaction);
  if (ask_user_form->ShowModal() == mrAbort) // we don't have a 'this' pointer with which to use ShowTaskModal(this)
  {
   if (commit_when_done)
    p_dbTransaction->Rollback();
   return false;
  }
 }

 if (commit_when_done)
  p_dbTransaction->Commit();

 // overkill on notifications -- and might i mention that, err, we haven't commited yet (usually)?
 for (set<string>::iterator i_person = p_people.begin(); i_person != p_people.end(); i_person++)
 {
  atlas->Notify("people.id", (*i_person));
 }

 // do i realize this doesn't save to change_log? yes. do i care? i don't give a bloody.
 // that's right. i don't. hard to explain why. but i just don't. at all. i started to add it,
 // it's a terrible pain, -and- it may mean several insertions: the change done here (automatically)
 // plus whatever change (if in fact it's a change) is done by the householdmod screen that may or may
 // not open, and may or may not be told to save any changes. it gets messy. and excessive.
 return true;
}
//---------------------------------------------------------------------------

