//---------------------------------------------------------------------------

#ifndef codeSpecialLedgerH
#define codeSpecialLedgerH
//---------------------------------------------------------------------------
#include <Classes.hpp>
#include <Controls.hpp>
#include <StdCtrls.hpp>
#include <Forms.hpp>
#include "codeSpecialGeneric.h"
#include <ComCtrls.hpp>
#include <ToolWin.hpp>
#include <ExtCtrls.hpp>
#include <Mask.hpp>
#include <Buttons.hpp>
#include <string>
#include <map>
#include <algorithm>
using namespace std;
//---------------------------------------------------------------------------
struct level_info
{
 string level_name;
 string field_name;
 bool   show_field;
 level_info (string p_level_name, string p_field_name, bool p_show_field) :
  level_name (p_level_name), field_name (p_field_name), show_field (p_show_field) {};
};

bool operator == (const level_info & a, const level_info & b)
{
 if (a.level_name == b.level_name &&
     a.field_name == b.field_name &&
     a.show_field == b.show_field)
  return true;
 else
  return false;
}
//---------------------------------------------------------------------------
struct account_level
{
 string id;
 string short_name;
 string long_name;
 account_level (string p_id, string p_short_name, string p_long_name) :
  id(p_id), short_name(p_short_name), long_name(p_long_name) {};
 account_level () {};
};

struct account_info
{
 map<string, account_level> account_desc;
 string id;
 string account_name;
 string account_pure;
 string account_note;
 string id_type;
 string long_type;
 string id_cycle;
 string month_cycle;
 bool is_budget;
 bool is_closed;
 bool is_yearly;
 account_info () {};
};

struct account_compare
{
 vector<string> compare_order;
 bool operator() (const account_info & acc_one, const account_info & acc_two)
 {
  bool is_smaller = false;
  for (vector<string>::iterator i_table_name = compare_order.begin();
       i_table_name != compare_order.end(); i_table_name++)
  {
   string table_name = *i_table_name;
   map<string, account_level>::const_iterator i_acc_one_info = acc_one.account_desc.find(table_name);
   map<string, account_level>::const_iterator i_acc_two_info = acc_two.account_desc.find(table_name);
   int acc_one_id = StringToInt(i_acc_one_info->second.short_name);
   int acc_two_id = StringToInt(i_acc_two_info->second.short_name);
   if (acc_one_id < acc_two_id)
   {
    is_smaller = true;
    break;
   }
   else if (acc_one_id > acc_two_id)
   {
    is_smaller = false;
    break;
   }
  }
  return is_smaller;
 };
};

//---------------------------------------------------------------------------
class TformSpecialLedger : public TformSpecialGeneric
{
__published:	// IDE-managed Components
        TToolButton *buttonClose;
        TToolButton *buttonUseless;
        TPageControl *pageLedger;
        TTabSheet *sheetTransactions;
        TTabSheet *sheetTree;
        TTreeView *treeAccounts;
        TLabel *labelSumNet;
        TLabel *labelDollarNet;
        TLabel *labelNetTitle;
        TTreeView *treeDemo;
        TLabel *labelDemoTree;
        TLabel *labelViewedDimension;
        TBitBtn *buttonDecreaseDimension;
        TBitBtn *buttonIncreaseDimension;
        TBitBtn *buttonResetDimension;
        TLabel *labelAccountTree;
        TListView *listViewedDimensions;
        TLabel *labelAccountCount;
        TPanel *panelCriteria;
        TLabel *labelAccountSearch;
        TEdit *editAccount;
        TComboBox *comboAccountType;
        TCheckBox *checkExactMatch;
        TButton *buttonSearchAccounts;
        TCheckBox *checkIsClosed;
        TCheckBox *checkIsBudget;
        TCheckBox *checkIsYearly;
        TCheckBox *checkNotIsYearly;
        TCheckBox *checkNotIsBudget;
        TCheckBox *checkNotIsClosed;
        TLabel *labelAccountType;
        TLabel *labelOrBudget;
        TLabel *labelOrClosed;
        TLabel *labelOrYearly;
        TPanel *panelAccountsAndTransactions;
        TListView *listAccounts;
        TSplitter *splitterAccountTransactions;
        TPanel *PanelTransactions;
        TListView *listTransactions;
        TPanel *PanelTransactionSearch;
        TLabel *labelStartDate;
        TEdit *editStartDate;
        TLabel *labelEndDate;
        TEdit *editEndDate;
        TButton *buttonSearchTransactions;
        TBitBtn *buttonCheckAll;
        TBitBtn *buttonUncheckAll;
        TLabel *labelSumDRTitle;
        TLabel *labelDollarDR;
        TLabel *labelSumDR;
        TLabel *labelSumCRTitle;
        TLabel *labelDollarCR;
        TLabel *labelSumCR;
        void __fastcall buttonSearchAccountsClick(TObject *Sender);
        void __fastcall buttonSearchTransactionsClick(TObject *Sender);
        void __fastcall listViewedDimensionsChange(TObject *Sender,
          TListItem *Item, TItemChange Change);
        void __fastcall buttonIncreaseDimensionClick(TObject *Sender);
        void __fastcall buttonDecreaseDimensionClick(TObject *Sender);
        void __fastcall buttonResetDimensionClick(TObject *Sender);
        void __fastcall treeAccountsClick(TObject *Sender);
        void __fastcall treeAccountsDblClick(TObject *Sender);
        void __fastcall buttonCloseClick(TObject *Sender);
        void __fastcall FormClose(TObject *Sender, TCloseAction &Action);
        void __fastcall checkIsClosedClick(TObject *Sender);
        void __fastcall buttonCheckAllClick(TObject *Sender);
        void __fastcall buttonUncheckAllClick(TObject *Sender);
private:	// User declarations
        // variables
        vector<level_info> viewed_dimensions;
        vector<account_info> accounts;
        account_compare full_tree_compare;
        bool enable_list_checking;
        map<TTreeNode*, string> tree_clicking; // for real accounts (id)
        map<TTreeNode*, string> tree_partial; // seperate, for non-accounts
        classFilterObject accounts_filter;
        int accounts_request;
        classFilterObject transactions_filter;
        int transactions_request;
        // functions
        void RedrawLevelDemo ();
        void RedrawLevelList ();
        void RedrawLevelFull ();
        void RedrawAccounts ();
        void RedrawTransactions ();
        void ResetLevelInfo ();
        void ResetAccounts ();
public:		// User declarations
        __fastcall TformSpecialLedger(TComponent* Owner);
        void AtlasScreen(int request_num, string matched_value);
        void AddAccount ();
        void EditAccount ();
        void EditTransaction ();
};
//---------------------------------------------------------------------------
extern PACKAGE TformSpecialLedger *formSpecialLedger;
//---------------------------------------------------------------------------
#endif
