//---------------------------------------------------------------------------

#include <vcl.h>
#pragma hdrstop

#include "codeSpecialModifyCashAccount.h"
#include "codeRecordIncomingBill.h"
#include "codeRecordOutgoingBill.h"
//---------------------------------------------------------------------------
#pragma package(smart_init)
#pragma link "codeSpecialGeneric"
#pragma resource "*.dfm"
TformSpecialModifyCashAccount *formSpecialModifyCashAccount;
//---------------------------------------------------------------------------
__fastcall TformSpecialModifyCashAccount::TformSpecialModifyCashAccount(TComponent* Owner, string incoming_check_id, string type)
        : TformSpecialGeneric(Owner)
{
  // internal check id
  check_id = incoming_check_id;
  bill_id = "";

  if (type == "expense")
    is_expense = true;
  else
    is_expense = false;
  if (type == "receivable")
    is_receivable = true;
  else
    is_receivable = false;

  // set up list view for bills
  bills_filter.SetListViewHandle(listBills);
  bills_filter.SetSecurity("bills");
  bills_filter.AddColumnSpec("DOS", "tasks", as_date, true, 75);
  bills_filter.AddColumnSpec("Date Received", "bills", as_date, true, 85);
  bills_filter.AddColumnSpec("PR Num", "purchase_reqs", as_int, true, 55);
  bills_filter.AddColumnSpec("CDIB", "affiliations", as_text, true, 40);
  bills_filter.AddColumnSpec("Total Charge", "bills", as_double, true, 75, amountString);
  bills_filter.AddColumnSpec("Invoice Amount", "bills", as_double, true, 90, amountString);
  bills_filter.AddColumnSpec("Paid Amount", "bills", as_double, true, 85, amountString);
  bills_filter.AddColumnSpec("Current Id", "current_id", as_int, false, 80);
  bills_filter.AddColumnSpec("Internal Id", "internal_id", as_int, false, 80);
  bills_filter.FetchPreferences("specialmodifycashaccount.bills_filter");
  bills_filter.SetIconStatus(true);

  string sql;
  sql = "select tasks.date_of_task, bills.bill_date, purchase_reqs.purchase_req_number, affiliations.cdib, "
               "bills.total_charge, bills.invoice_amount, bills.paid_amount, bills.current_id, bills.id "
        "from checks "
        "inner join purchase_reqs on checks.id = purchase_reqs.check_id and checks.id = " + check_id + " "
        "inner join bills on purchase_reqs.id = bills.purchase_req_id "
        "inner join tasks on bills.task_id = tasks.id "
        "inner join people on tasks.person_id = people.id "
        "inner join affiliations on people.affiliations_id = affiliations.id "
        "order by tasks.date_of_task, bills.bill_date, purchase_reqs.purchase_req_number, affiliations.cdib";
  bills_filter.SetSQLStatement (sql);
  bills_filter.RefreshDataSet();
  bills_filter.SetAddEdit(this, NULL, EditRecord(&(TformSpecialModifyCashAccount::EditBill)));
  check_request = atlas->RequestNotificationWhen("bills.check_id", check_id, rel_e, as_int, this,
                     RefreshHandler(&(TformSpecialModifyCashAccount::AtlasRefreshListView)));

  // setup acceptable paymennts
  acceptable_payments = acc_none;
  acceptable_payments |= acc_non_budget;
  acceptable_payments |= acc_non_closed;
  acceptable_payments |= acc_non_yearly;
  acceptable_payments |= acc_yearly;

  if (is_expense)
    {
     acceptable_payments |= acc_expense;
    }
  else if (is_receivable)
    {
     acceptable_payments |= acc_receivable;
    }
  else
    {
     PSay("This screen has been called w/o specifying whether it is for account receivable or accounts payable.", m_program_error, MB_OK);
     return;
    }

  // refresh Amount info
  SetupAmountInfo();

  PSay("This screen should likely no longer be used;\nif you have a real need to use this screen, please contact a programmer.", m_warning, MB_OK, "deprecated");
}
//---------------------------------------------------------------------------
void TformSpecialModifyCashAccount::EditBill()
{
  map<string, string> temp_map;
  temp_map["id"] = bills_filter.GetSelectedRecordID();
  string bill_id = GetOneField(dbDatabase, "select bill_id from bills where id = " + temp_map["id"]);
  if((bill_id == lookupTables->IdOf("bill_types", "incoming provider bill")) ||
     (bill_id == lookupTables->IdOf("bill_types", "incoming vendor bill")))
  {
    FORM_TRY
      TformRecordIncomingBill * in_bill = new TformRecordIncomingBill(NULL, temp_map);
      in_bill->Show();
    FORM_CATCH
  }
  else if(bill_id == lookupTables->IdOf("bill_types", "outgoing provider bill"))
  {
    FORM_TRY
      TformRecordOutgoingBill * out_bill = new TformRecordOutgoingBill(NULL, temp_map);
      out_bill->Show();
    FORM_CATCH
  }
}
//--------------------------------------------------------------------------
bool TformSpecialModifyCashAccount::AtlasRefreshListView(int request_num, string matched_value)
{
  if (request_num == check_request)
    bills_filter.RefreshDataSet();
  return true;
}
//---------------------------------------------------------------------------
void TformSpecialModifyCashAccount::SetupAmountInfo()
{
  bill_id = bills_filter.GetSelectedRecordID();
  if (bill_id != "")
    {
     // Find the balance of all transactions
     string sql = "select (sum(transactions.debit_amount) - sum(transactions.credit_amount)) balance "
                  "from bills "
                  "left join transactions on bills.id = transactions.bill_id "
                  "left join accounts on transactions.account_id = accounts.id "
                  "where "
                  "(accounts.account_type_id = '" + lookupTables->IdOf("account_types", "EXPENSE") + "' or "
                  "accounts.account_type_id = '" + lookupTables->IdOf("account_types", "RECEIVABLE") + "') and "
                  "bills.id = " + bill_id;
     lblBillTotal->Caption = StringToAmount(GetOneField(dbDatabase, sql)).c_str();
    }
  // Get total for check, and put it in a vec map we can use later (RefreshAmountInfo)
  float result = 0.00;
  string sql = "select bills.id id, (sum(transactions.debit_amount) - sum(transactions.credit_amount)) balance "
               "from checks "
               "inner join purchase_reqs on checks.id = purchase_reqs.check_id "
               "inner join bills on purchase_reqs.id = bills.purchase_req_id and bills.current_id >= 3 "
               "left join transactions on bills.id = transactions.bill_id "
               "left join accounts on transactions.account_id = accounts.id "
               "where "
               "(accounts.account_type_id = '" + lookupTables->IdOf("account_types", "EXPENSE") + "' or "
               "accounts.account_type_id = '" + lookupTables->IdOf("account_types", "RECEIVABLE") + "') and "
               "checks.id = " + check_id + " "
               "group by bills.id";
  if (!SqlToVecMap(dbDatabase, sql, bill_ids))
  {
    PSay("Could not obtain bill id's.", m_database_error, MB_OK);
  }
  for (vector<map<string, string> >::iterator map_itr = bill_ids.begin(); map_itr != bill_ids.end(); map_itr++)
  {
       result += StringToFloat((*map_itr)["balance"]);
  }
  lblCheckTotal->Caption = FloatToString(result).c_str();
  SetAccountAmount(editAccount, editAmount, listPayments, acceptable_payments);
}
//---------------------------------------------------------------------------
void TformSpecialModifyCashAccount::RefreshAmountInfo(bool refresh_check)
{
 // we don't want to fill our changed amounts with empty values, or the screen will cancel everything!
 map<string, map<string, string> > temp_changed_amounts = changed_amounts;
 float answer = 0.00;
 TListItem * current_row = NULL;
 for (int i = 0; i < listPayments->Items->Count; i++)
 {
  current_row = listPayments->Items->Item[i];
  answer += StringToFloat(current_row->SubItems->Strings[0].c_str());
 }
 lblBillTotal->Caption = StringToAmount(FloatToString(answer)).c_str();

 // go through all of the orginal data, and if the user has made a change then sum that bill instead
 if (refresh_check)
 {
   float result = 0.00;
   for (vector<map<string, string> >::iterator map_itr = bill_ids.begin(); map_itr != bill_ids.end(); map_itr++)
   {
     map <string, string> account_info = temp_changed_amounts[(*map_itr)["id"]];
     // if user has changed this bill (current bill == (*map_itr)["id"]) then sum all of the account(s) they changed
     if (!account_info.empty())
     {
       for (map<string, string>::iterator changed_itr = account_info.begin(); changed_itr != account_info.end(); changed_itr++)
       {
         result += StringToFloat((*changed_itr).second);
       }
     }
     else  // else sum the original data from database
     {
       result += StringToFloat((*map_itr)["balance"]);
     }
   }
   lblCheckTotal->Caption = FloatToString(result).c_str();
 }
}
//---------------------------------------------------------------------------
void __fastcall TformSpecialModifyCashAccount::buttonDeletePaymentClick(
      TObject *Sender)
{
 // use Philip's accouting functions
 map<string, string> wanted_account = PartsByMask (editAccount);
 bool looking_for_budget = (acc_budget & acceptable_payments);
 acc_info requested_account = InfoByParts (wanted_account, looking_for_budget);

 if (requested_account.Matches(acceptable_payments))
 {
   string given_amount = AmountToString(editAccount->Text.c_str());
   // if this was a modification then kill it from changed_amounts
   if (changed_amounts[bills_filter.GetSelectedRecordID()][requested_account.id] != "")
   {
     changed_amounts[bills_filter.GetSelectedRecordID()].erase(requested_account.id);
   }
   else
   {
     // the blank one we create here will let the finish function know to cancel all moneys on this account
   }
 }

 // remove account/amount
 DelAccountAmount (editAccount, editAmount, listPayments, acceptable_payments);
 RefreshAmountInfo(true);
 return;
}
//---------------------------------------------------------------------------
void __fastcall TformSpecialModifyCashAccount::buttonSavePaymentClick(
      TObject *Sender)
{
 // store changes the user has made in map<string, map<string,     string> > changed_amounts
 //                                        bill_id,    account_id, amount
 // use Philip's accouting functions

 map<string, string> wanted_account = PartsByMask (editAccount);
 bool looking_for_budget = (acc_budget & acceptable_payments);
 acc_info requested_account = InfoByParts (wanted_account, looking_for_budget);

 AddAccountAmount (editAccount, editAmount, listPayments, acceptable_payments);
 RefreshAmountInfo(true);

 // put all of the accounts in the map of accounts b/c Philip's reversal algorithm reverses all accounts for a given bill
 map<string, string> all_payments = GetAccountAmount(listPayments);
 map<string, string> temp_map;
 for (map<string, string>::iterator i_payment = all_payments.begin(); i_payment != all_payments.end(); i_payment++)
 {
   temp_map[i_payment->first] = i_payment->second;
 }
 changed_amounts[bills_filter.GetSelectedRecordID()] = temp_map;
 return;
}
//---------------------------------------------------------------------------
void __fastcall TformSpecialModifyCashAccount::editAccountKeyPress(
      TObject *Sender, char &Key)
{
 PSay("You can't change an account number after a check has been posted.", m_warning, MB_OK);
 Key = '\0';
}
//---------------------------------------------------------------------------
void __fastcall TformSpecialModifyCashAccount::listPaymentsSelectItem(
      TObject *Sender, TListItem *Item, bool Selected)
{
 // select payment
 ModAccountAmount (editAccount, editAmount, listPayments, acceptable_payments);
}
//---------------------------------------------------------------------------
void __fastcall TformSpecialModifyCashAccount::listBillsClick(
      TObject *Sender)
{
 // we don't want to fill our changed amounts with empty values, or the screen will cancel everything!
 map<string, map<string, string> > temp_changed_amounts = changed_amounts;
 bill_id = bills_filter.GetSelectedRecordID();

 if (bill_id != "")
 {
    string sql_find_payments = "select accounts.name name, accounts.id id, sum(transactions.debit_amount - transactions.credit_amount) total_exp from transactions inner join accounts on transactions.account_id = accounts.id where accounts.account_type_id = " + lookupTables->IdOf("account_types", "Expense");
    sql_find_payments += " and transactions.bill_id = " + bill_id;
    sql_find_payments += " group by accounts.id, accounts.name";
    vector<map<string, string> > expenses_on_bill;

    SetAccountAmount(editAccount, editAmount, listPayments, acceptable_payments);

    map <string, string> account_info = temp_changed_amounts[bill_id];
    if (!account_info.empty())
    {
     for (map<string, string>::iterator i_exp = account_info.begin(); i_exp != account_info.end(); i_exp++)
     {
      TListItem * new_row = NULL;
      new_row = listPayments->Items->Add();
      new_row->Caption = GetOneField(dbDatabase, "select name from accounts where id = " + (*i_exp).first).c_str();
      new_row->SubItems->Clear();
      new_row->SubItems->Add((StringToAmount((*i_exp).second)).c_str());
      new_row->SubItems->Add((*i_exp).first.c_str());
      listPayments->Selected = NULL;
     }
    } else if (SqlToVecMap(dbDatabase, sql_find_payments, expenses_on_bill))
      {
        for (vector<map<string, string> >::iterator i_exp = expenses_on_bill.begin(); i_exp != expenses_on_bill.end(); i_exp++)
        {
         TListItem * new_row = NULL;
         new_row = listPayments->Items->Add();
         new_row->Caption = (*i_exp)["name"].c_str();
         new_row->SubItems->Clear();
         new_row->SubItems->Add((StringToAmount((*i_exp)["total_exp"])).c_str());
         new_row->SubItems->Add((*i_exp)["id"].c_str());
         listPayments->Selected = NULL;
        }
      }
    else
     PSay("Error while trying to fetch transactions to date on this item", m_database_error, MB_OK);
 }

 // reflect the bill total
 RefreshAmountInfo(false);
}
//---------------------------------------------------------------------------
void __fastcall TformSpecialModifyCashAccount::btnCloseClick(
      TObject *Sender)
{
 Close();
}
//---------------------------------------------------------------------------
void __fastcall TformSpecialModifyCashAccount::btnFinishClick(
      TObject *Sender)
{
 // finish
 int user_answer = PSay("This change could be difficult to undo. Please make sure you truly intended this action.", m_confirmation, MB_YESNO);
 if (user_answer == IDNO)
  return;

 TIBTransaction * dbTransaction = new TIBTransaction(NULL);
 try
 {
  dbTransaction->DefaultDatabase = dbDatabase;
  dbTransaction->DefaultAction = TARollback;
  dbTransaction->StartTransaction();

  // obtain transaction number
  string current_transaction_number = GetNextId(dbTransaction, "transact_number");

  // Philips code from modify accounts payable that I modified here to handle modifying cash accounts
  for (map<string, map<string, string> >::iterator map_itr = changed_amounts.begin(); map_itr != changed_amounts.end(); map_itr++)
  {
    bill_id = (*map_itr).first;
    string sql_get_task_id = "select task_id from bills where id = " + bill_id;
    string task_id = GetOneField(dbTransaction, sql_get_task_id);
    if (bill_id != "")
    {
      // this time, we don't want to re-obligate stuff. instead ... we want to cancel the expense and the cash, and put them back for the correct amounts
      // TODO: some sort of group by? select only fields needed?

      string sql_select_expense = "select * from transactions inner join accounts on transactions.account_id = accounts.id where accounts.account_type_id = " + lookupTables->IdOf("account_types", "Expense");

      sql_select_expense += " and transactions.bill_id = " + bill_id;

      vector<map<string, string> > expense_transactions;
      SqlToVecMap(dbTransaction, sql_select_expense, expense_transactions);

      for (vector<map<string, string> >::iterator i_old = expense_transactions.begin();
           i_old != expense_transactions.end(); i_old++)
      {
       map<string, string> reverse_transaction_core;
       reverse_transaction_core["transaction_number"] = current_transaction_number;
       reverse_transaction_core["effective_date"] = CurrentDate(); // TODO: EFFECTIVE_DATE
       reverse_transaction_core["current_id"] = "4";
       reverse_transaction_core["last_update_stamp"] = RightNow();
       reverse_transaction_core["last_updated_by"] = current_userid;
       reverse_transaction_core["tax_entity_id"] = (*i_old)["tax_entity_id"];
       reverse_transaction_core["task_id"] = (*i_old)["task_id"];
       reverse_transaction_core["bill_id"] = (*i_old)["bill_id"];

       map<string, string> reverse_transaction_cash = reverse_transaction_core;
       map<string, string> reverse_transaction_self = reverse_transaction_core;

       string payable_account_id = EquivAccount (cash_ref, (*i_old)["account_id"]);
       string self_account_id = (*i_old)["account_id"];

       if (payable_account_id != "")
       {
        reverse_transaction_cash["note"] = "Cancelling of cash transactions on bill because cash is being modified";
        reverse_transaction_cash["id"] = GetNextId(dbTransaction, "transactions");
        reverse_transaction_cash["account_id"] = payable_account_id;
        reverse_transaction_cash["credit_amount"] = (*i_old)["credit_amount"];
        reverse_transaction_cash["debit_amount"] = (*i_old)["debit_amount"];
        if (!ExecuteInsertStatement(dbTransaction, "transactions", reverse_transaction_cash))
        {
         PSay("Failed to insert a reversal (cash) transaction.", m_database_error, MB_OK);
         throw(15);
        }
       }
       else
       {
        PSay("Could not identify the cash account to undo based on the expense account used.", m_missing_data, MB_OK);
        throw(16);
       }

       if (self_account_id != "")
       {
        reverse_transaction_self["note"] = "Cancelling of expense transactions on bill because cash is being modified";
        reverse_transaction_self["id"] = GetNextId(dbTransaction, "transactions");
        reverse_transaction_self["account_id"] = self_account_id;
        reverse_transaction_self["credit_amount"] = (*i_old)["debit_amount"];
        reverse_transaction_self["debit_amount"] = (*i_old)["credit_amount"];
        if (!ExecuteInsertStatement(dbTransaction, "transactions", reverse_transaction_self))
        {
         PSay("Failed to insert a reversal (expense) transaction.", m_database_error, MB_OK);
         throw(14);
        }
       }
       else
       {
        PSay("Could not identify the expense (self) account to undo based on the expense account used.", m_missing_data, MB_OK);
        throw(17);
       }
      }

     string provider_id = "0";
     string sql_select_provider_id;
     sql_select_provider_id = "select bills.actual_tax_id from bills where bills.id = " + bill_id;


     provider_id = GetOneField(dbTransaction, sql_select_provider_id);
     if (provider_id == "")
      provider_id = "0";

     // now that we've reversed the cash, we want to put it back in, if any was given.
     map<string, string> insert_transaction_core;
     insert_transaction_core["current_id"] = "4";
     insert_transaction_core["last_update_stamp"] = RightNow();
     insert_transaction_core["last_updated_by"] = current_userid;
     insert_transaction_core["task_id"] = task_id;
     insert_transaction_core["bill_id"] = bill_id;
     insert_transaction_core["tax_entity_id"] = provider_id;
     insert_transaction_core["effective_date"] = CurrentDate(); // TODO: EFFECTIVE_DATE
     insert_transaction_core["credit_amount"] = "0.00";
     insert_transaction_core["debit_amount"] = "0.00";
     insert_transaction_core["transaction_number"] = current_transaction_number;

//     map<string, string> all_payments = GetAccountAmount(listPayments);
     map<string, string> all_payments = changed_amounts[bill_id];
     for (map<string, string>::iterator i_payment = all_payments.begin(); i_payment != all_payments.end(); i_payment++)
     {
      // for each, we see only the expense account listed, but we need to remember to also deal
      // with the cash account too! so for each of these, figure out what the cash account is,
      // and run transactions against it too

      acc_info pay_account = EquivAccountInfo (cash_ref, i_payment->first);
      if (pay_account.IsClosed() || !pay_account.IsFound())
      {
       PSay("A cash account number could not be found to match this expense account.", m_missing_data, MB_OK);
       throw(2);
      }
      string id_of_pay_account = pay_account.id;

      map<string, string> expense_transaction = insert_transaction_core;
      map<string, string> cash_transaction = insert_transaction_core;

      expense_transaction["id"] = GetNextId(dbTransaction, "transactions");
      expense_transaction["note"] = "Modified expense on bill";
      expense_transaction["account_id"] = i_payment->first;
      expense_transaction["debit_amount"] = AmountToString(i_payment->second);

      cash_transaction["id"] = GetNextId(dbTransaction, "transactions");
      cash_transaction["note"] = "Modified cash on bill";
      cash_transaction["account_id"] = id_of_pay_account;
      cash_transaction["credit_amount"] = AmountToString(i_payment->second);

      if (!ExecuteInsertStatement(dbTransaction, "transactions", expense_transaction))
       throw (3);

      if (!ExecuteInsertStatement(dbTransaction, "transactions", cash_transaction))
       throw (4);
     }
    }
    else
    {
      PSay("One of the bills could not be modified.", m_database_error, MB_OK);
    }
    // if there is a balance of zero left on the bill then mark it as deleted
    string sql = "select (sum(transactions.debit_amount) - sum(transactions.credit_amount)) balance "
                 "from bills "
                 "left join transactions on bills.id = transactions.bill_id "
                 "left join accounts on transactions.account_id = accounts.id "
                 "where accounts.account_type_id = '" + lookupTables->IdOf("account_types", "EXPENSE") + "' and "
                 "bills.id = " + bill_id;
    string transaction_balance = GetOneField(dbTransaction, sql);

    // set the paid amount for the user so they don't have to
    string update_bill = "update bills set paid_amount = " + transaction_balance + " where id = " + bill_id;
    ExecuteStatement(dbTransaction, update_bill);

    // if bill has a balance of 0.00 then let the user know this bill is no longer valid
    if (StringToFloat(transaction_balance) == 0.00)
      {
       sql = "update bills set current_id = 0 where id = " + bill_id;
       if (!ExecuteStatement(dbTransaction, sql))
         {
          PSay("There is a balance of 0.00 on this bill, but the the program is unable ot mark it as deleted.", m_database_error, MB_OK);
         }
      }
  }  // end of loop through changed bills
  dbTransaction->Commit();

  Close();
 }
 catch (...)
 {
  if (dbTransaction->Active)
   dbTransaction->Rollback();
  PSay("Couldn't modify the cash account related to this item.", m_database_error, MB_OK);
 }
 delete dbTransaction;

}
//---------------------------------------------------------------------------
