//---------------------------------------------------------------------------

#ifndef codeSpecialMultipleWarningsBlocksH
#define codeSpecialMultipleWarningsBlocksH
//---------------------------------------------------------------------------
#include <Classes.hpp>
#include <Controls.hpp>
#include <StdCtrls.hpp>
#include <Forms.hpp>
#include "codeSpecialGeneric.h"
#include <ComCtrls.hpp>
#include <ToolWin.hpp>
#include <ExtCtrls.hpp>
#include <ImgList.hpp>

enum warning_icon_types {warning_icon_none, warning_icon_unknown, warning_icon_error, warning_icon_okay, warning_icon_warning, warning_icon_block};
enum warning_check_types {warning_check_none, warning_check_unchecked, warning_check_checked, warning_check_gray_unchecked, warning_check_gray_checked};

struct warning_row
{
 warning_icon_types  icon;
 warning_check_types check;
 string              comment;
 void *              identifier;

 warning_row(warning_icon_types p_icon, warning_check_types p_check, string p_comment, void * p_identifier) :
  icon (p_icon),
  check (p_check),
  comment (p_comment),
  identifier (p_identifier)
 {}

 static warning_icon_types GetItemIcon (TListItem* row)
 {
  if (row)
  {
   int found_icon = -1;
   if (row->SubItemImages[0] > 3)
    found_icon = row->SubItemImages[0];
   else
    found_icon = row->ImageIndex;

   switch (found_icon)
   {
    case 4: return warning_icon_error;
    case 5: return warning_icon_okay;
    case 6: return warning_icon_warning;
    case 7: return warning_icon_block;
   }
  }
  return warning_icon_none;
 }

 static warning_check_types GetItemCheck (TListItem* row)
 {
  if (row)
  {
   int found_icon = -1;
   if (row->SubItemImages[0] < 4)
    found_icon = row->SubItemImages[0];

   switch (found_icon)
   {
    case 0: return warning_check_unchecked;
    case 1: return warning_check_checked;
    case 2: return warning_check_gray_unchecked;
    case 3: return warning_check_gray_checked;
   }
  }
  return warning_check_none;
 }

 static string GetItemComment (TListItem* row)
 {
  if (row)
   return row->SubItems->Strings[1].c_str();
  return "";
 }

 void GetFromListitem (TListItem* old_item)
 {
  icon = GetItemIcon(old_item);
  check = GetItemCheck(old_item);
  comment = GetItemComment(old_item);
 }

 TListItem* AddToListview (TListView* listview)
 {
  TListItem * new_item = listview->Items->Add();
  new_item->Caption = "";
  int warning_icon_index = -1;
  int check_icon_index   = -1;
  switch(icon)
  {
   case warning_icon_error:    warning_icon_index = 4; break;
   case warning_icon_okay:     warning_icon_index = 5; break;
   case warning_icon_warning:  warning_icon_index = 6; break;
   case warning_icon_block:    warning_icon_index = 7; break;
  }
  switch(check)
  {
   case warning_check_unchecked:      check_icon_index = 0; break;
   case warning_check_checked:        check_icon_index = 1; break;
   case warning_check_gray_unchecked: check_icon_index = 2; break;
   case warning_check_gray_checked:   check_icon_index = 3; break;
  }
  new_item->SubItems->Add("");
  if (check != warning_check_none)
  {
   new_item->ImageIndex = warning_icon_index;
   new_item->SubItemImages[0] = check_icon_index;
  } else
  {
   new_item->ImageIndex = -1;
   new_item->SubItemImages[0] = warning_icon_index;
  }
  new_item->SubItems->Add(comment.c_str());
  return new_item;
 }
};

bool IsUnchecked (const vector<warning_row> & warning_rows)
{
 for (vector<warning_row>::const_iterator i_row = warning_rows.begin(); i_row != warning_rows.end(); i_row++)
  if (i_row->check == warning_check_unchecked || i_row->check == warning_check_gray_unchecked)
   return true;
 return false;
}

//---------------------------------------------------------------------------
class TformSpecialMultipleWarningsBlocks : public TformSpecialGeneric
{
__published:	// IDE-managed Components
        TToolButton *buttonClose;
        TToolButton *buttonUseless;
        TListView *listItems;
        TImageList *imageList;
        TLabel *_labelExplanation;
        void __fastcall buttonCloseClick(TObject *Sender);
        void __fastcall listItemsMouseMove(TObject *Sender,
          TShiftState Shift, int X, int Y);
        void __fastcall listItemsClick(TObject *Sender);
        void __fastcall FormClose(TObject *Sender, TCloseAction &Action);
        void __fastcall listItemsKeyPress(TObject *Sender, char &Key);
private:	// User declarations
        int x_pos;
        vector<warning_row> & warning_rows;
        map<TListItem *, unsigned int> list_row_to_warn_row;
public:		// User declarations
        __fastcall TformSpecialMultipleWarningsBlocks(TComponent* Owner, vector<warning_row> & p_warning_rows);
};
//---------------------------------------------------------------------------

bool CanOverrideWarnings (vector<warning_row> & warning_rows)
{
 if (IsUnchecked(warning_rows))
 {
  FORM_TRY
   TformSpecialMultipleWarningsBlocks * warning_window = new TformSpecialMultipleWarningsBlocks(NULL, warning_rows);
   warning_window->ShowModal(); // we don't have a 'this' pointer with which to call ShowTaskModal(this)
  FORM_CATCH
 }
 return !IsUnchecked(warning_rows);
}

extern PACKAGE TformSpecialMultipleWarningsBlocks *formSpecialMultipleWarningsBlocks;
//---------------------------------------------------------------------------
#endif
