//---------------------------------------------------------------------------
#include <vcl.h>
#pragma hdrstop

#include "codeSpecialNewProviderBill.h"
#include "codeSearchPerson.h"
#include "codeSearchTaxEntity.h"
#include "codeSpecialPersonLimits.h"
#include "codeRecordInsuranceCoverage.h"
#include "codeRecordServiceProvided.h"
#include "codeRecordTaskCobVisit.h"
#include "codeSearchCode.h"
#include "codeRecordPerson.h"
//---------------------------------------------------------------------------
#pragma package(smart_init)
#pragma link "codeSpecialGeneric"
#pragma resource "*.dfm"
TformSpecialNewProviderBill *formSpecialNewProviderBill;


/*
forward (next >>)

first   -> fourth (old cob found)
first   -> second (new cob)
second  -> fourth
third   -> fifth
fourth  -> fifth (old cob found)
fourth  -> third (new cob)
fifth   -> sixth (hcfa)
fifth   -> seventh (ub)
sixth   -> eighth
seventh -> eighth
eighth  -> ninth
ninth   -> *****

backward (<< back)

ninth   -> eighth
eighth  -> seventh (ub)
eighth  -> sixth (hcfa)
seventh -> fifth
sixth   -> fifth
fifth   -> third (new cob)
fifth   -> fourth (old cob)
fourth  -> first (old cob)
fourth  -> second (new cob)
third   -> fourth
second  -> first
first   -> *****
*/
//---------------------------------------------------------------------------
__fastcall TformSpecialNewProviderBill::TformSpecialNewProviderBill(TComponent* Owner)
        : TformSpecialGeneric(Owner)
{
 panelSelectArea->Color = title_bar_color;
 Start();

 ResetFirstPage();
 GotoFirstPage();
}
//---------------------------------------------------------------------------
void TformSpecialNewProviderBill::ResetFirstPage()
{
 is_new_cob = false;
 checkHaveCOB->Checked = true;
 editKnownCOBNumber->Text = "";
 editCOBDOS->DateTime = Date().CurrentDate();

 // link to be based on COB number   
 CreateLink(txtPerson, NULL, this, "", "people");

 ResetSecondPage();
 ResetFourthPage();
}
//---------------------------------------------------------------------------
void TformSpecialNewProviderBill::ResetSecondPage()
{
 patient_id = "";
 provider_id = "";
 CreateLink(staticPatient, buttonPatientSearch, this, patient_id, "people", "");
 CreateLink(staticProvider, buttonProviderSearch, this, provider_id, "tax_entities", "");
 ResetServices();
 ResetInsurance();

 ResetFourthPage();
 ResetThirdPage();
}
//---------------------------------------------------------------------------
void TformSpecialNewProviderBill::ResetThirdPage()
{
 editDateService->DateTime = Date().CurrentDate();
 editTimeService->DateTime = Time().CurrentTime();
 editReason->Text = "";

 ResetFifthPage();
}
//---------------------------------------------------------------------------
void TformSpecialNewProviderBill::ResetFourthPage()
{
 // already been reset by changing the person id
 ResetFifthPage();
}
//---------------------------------------------------------------------------
void TformSpecialNewProviderBill::ResetFifthPage()
{
 tax_provider_id = "";
 tax_facility_id = "";
 tax_group_id    = "";
 tax_actual_id   = "";

 if (provider_id != "" && IsInt(provider_id))
  tax_provider_id = provider_id;

 RedrawTaxProvider();
 RedrawTaxFacility();
 RedrawTaxGroup();

 is_hcfa = true;
 checkIsHCFA->Checked = true;
 checkPayProvider->Checked = true;
 possible_dup_non_referal = false;

 ResetSixthPage();
 ResetSeventhPage();
}
//---------------------------------------------------------------------------
void TformSpecialNewProviderBill::ResetSixthPage()
{
 listHCFAConditions->Items->Clear();
 SetHCFACondition(NULL);

 listHCFAProcedures->Items->Clear();
 SetHCFAProcedure(NULL);    
 possible_dup_non_referal = false;

 ResetEighthPage();
}
//---------------------------------------------------------------------------
void TformSpecialNewProviderBill::ResetSeventhPage()
{
 listUBConditions->Items->Clear();
 SetUBCondition(NULL);

 listUBProcedures->Items->Clear();
 SetUBProcedure(NULL);

 ResetEighthPage();
}
//---------------------------------------------------------------------------
void TformSpecialNewProviderBill::ResetEighthPage()
{
 editOriginalCharges->Text = "";
 editResidualCharges->Text = "";
 editControlNumber->Text = "";

 editPaymentTotal->Text = "0.00";
 // The following aren't on the eigth page, but they need to be reset along with editPaymentTotal
 lblHCFAPaymentTotal->Caption = "0.00";
 lblUBPaymentTotal->Caption = "0.00";

 checkDoNotCancelObligated->Checked = false;

 SetAccountAmount(editAccount, editAmount, listPayments, acceptable_payments);

 ResetNinthPage();
}
//---------------------------------------------------------------------------
void TformSpecialNewProviderBill::ResetNinthPage()
{
 // the GotoNinthPage() should take care of this, right?
 return;
}
//---------------------------------------------------------------------------
void TformSpecialNewProviderBill::GotoFirstPage()
{
 checkCreateCOB->Checked = is_new_cob;

 staticHeader->Caption = "1 of 6 or 8, depending - Select previous / new COB";
 buttonBack->Enabled = false;
 buttonNext->Caption = "Next >>";
 pageBill->ActivePageIndex = 0;
 return;
}
//---------------------------------------------------------------------------
void TformSpecialNewProviderBill::GotoSecondPage()
{

 staticHeader->Caption = "2 of 8 - Select patient, provider service and referring physician";
 buttonBack->Enabled = true;
 buttonNext->Caption = "Next >>";
 pageBill->ActivePageIndex = 1;
 return;
}
//---------------------------------------------------------------------------
void TformSpecialNewProviderBill::GotoThirdPage()
{

 staticHeader->Caption = "4 of 8 - Select date, time, and reason for the non-referral";
 buttonBack->Enabled = true;
 buttonNext->Caption = "Next >>";
 pageBill->ActivePageIndex = 2;
 return;
}
//---------------------------------------------------------------------------
void TformSpecialNewProviderBill::GotoFourthPage()
{

 if (is_new_cob)
  staticHeader->Caption = "3 of 8 - Examine patient's insurance and limits";
 else
  staticHeader->Caption = "2 of 6 - Examine patient's insurance and limits";
 buttonBack->Enabled = true;
 buttonNext->Caption = "Next >>";
 pageBill->ActivePageIndex = 3;
 return;
}
//---------------------------------------------------------------------------
void TformSpecialNewProviderBill::GotoFifthPage()
{

 if (is_new_cob)
  staticHeader->Caption = "5 of 8 - Select tax entities on bill, type of bill submitted";
 else
  staticHeader->Caption = "3 of 6 - Select tax entities on bill, type of bill submitted";
 buttonBack->Enabled = true;
 buttonNext->Caption = "Next >>";
 pageBill->ActivePageIndex = 4;
 return;
}
//---------------------------------------------------------------------------
void TformSpecialNewProviderBill::GotoSixthPage()
{

 if (is_new_cob)
  staticHeader->Caption = "6 of 8 - Enter data from HCFA-1500 form";
 else
  staticHeader->Caption = "4 of 6 - Enter data from HCFA-1500 form";
 buttonBack->Enabled = true;
 buttonNext->Caption = "Next >>";
 pageBill->ActivePageIndex = 5;
 return;
}
//---------------------------------------------------------------------------
void TformSpecialNewProviderBill::GotoSeventhPage()
{

 if (is_new_cob)
  staticHeader->Caption = "6 of 8 - Enter data from UB-92 form";
 else
  staticHeader->Caption = "4 of 6 - Enter data from UB-92 form";
 buttonBack->Enabled = true;
 buttonNext->Caption = "Next >>";
 pageBill->ActivePageIndex = 6;
 return;
}
//---------------------------------------------------------------------------
void TformSpecialNewProviderBill::GotoEighthPage()
{

 if (is_new_cob)
  staticHeader->Caption = "7 of 8 - Enter charges and payment information";
 else
  staticHeader->Caption = "5 of 6 - Enter charges and payment information";
 buttonBack->Enabled = true;
 buttonNext->Caption = "Next >>";
 pageBill->ActivePageIndex = 7;
 return;
}
//---------------------------------------------------------------------------
void TformSpecialNewProviderBill::GotoNinthPage()
{
 labelActualTaxEntity->Caption = PrettyString(GetOneField(dbDatabase, "select name from tax_entities where id = " + tax_actual_id)).c_str();
 labelActualTaxID->Caption = GetOneField(dbDatabase, "select taxid from tax_entities where id = " + tax_actual_id).c_str();

 map<string, string> all_payments = GetAccountAmount (listPayments);
 float payment_amount = 0.00;
 for (map<string, string>::iterator i_payment = all_payments.begin(); i_payment != all_payments.end(); i_payment++)
  payment_amount += StringToFloat(AmountToString(i_payment->second));

 labelFinalPaymentAmount->Caption = StringToAmount(FloatToString(payment_amount)).c_str();

 if (is_new_cob)
  staticHeader->Caption = "8 of 8 - Enter charges and payment information";
 else
  staticHeader->Caption = "6 of 6 - Enter charges and payment information";
 buttonBack->Enabled = true;
 buttonNext->Caption = "Finish";
 pageBill->ActivePageIndex = 8;
 return;
}
//---------------------------------------------------------------------------
void TformSpecialNewProviderBill::Start()
{
 is_new_cob = false;
 is_hcfa = true;

 services_request = 0;
 insurance_request = 0;

 staticPersonLimits->Font->Color = link_color;

 acceptable_payments = acc_none;
 acceptable_payments |= acc_non_budget;
 acceptable_payments |= acc_non_closed;
 acceptable_payments |= acc_non_yearly;
 acceptable_payments |= acc_yearly;
 acceptable_payments |= acc_expense;

 services_filter.SetListViewHandle(listServicesProvided);
 services_filter.SetSecurity("services_provided");
 services_filter.AddColumnSpec("Service", "services_provided", as_text, true, 140);
 services_filter.AddColumnSpec("Area", "services_provided", as_text, true, 80);
 services_filter.AddColumnSpec("Current Id", "current_id", as_int, false, 80);
 services_filter.AddColumnSpec("Internal Id", "internal_id", as_int, false, 80);
 services_filter.FetchPreferences("recordspecialnewcob.services_filter");
 services_filter.SetIconStatus(true);
 services_filter.SetSQLStatement("");
 services_filter.RefreshDataSet();

 referrer_filter.SetListViewHandle(listReferringPhysicians);
 referrer_filter.SetSecurity("tax_entities");
 referrer_filter.AddColumnSpec("Name", "tax_entities", as_text, true, 240);
 referrer_filter.AddColumnSpec("Current Id", "current_id", as_int, false, 80);
 referrer_filter.AddColumnSpec("Internal Id", "internal_id", as_int, false, 80);
 referrer_filter.FetchPreferences("recordspecialnewcob.referrer_filter");
 referrer_filter.SetHeader(false);
 referrer_filter.SetIconStatus(true);
 referrer_filter.SetSQLStatement("select tax_entities.name, tax_entities.current_id, tax_entities.id from tax_entities where tax_entities.is_referring = 1");
 referrer_filter.RefreshDataSet();

 insurance_filter.SetListViewHandle(listInsurance);
 insurance_filter.SetSecurity("insurance_coverage");
 insurance_filter.AddColumnSpec("Company", "tax_entities", as_text, true, 140);
 insurance_filter.AddColumnSpec("Medical Type", "tax_entities", as_text, true, 100);
 insurance_filter.AddColumnSpec("Policy Number", "insurance_policies", as_text, true, 120, rawString);
 insurance_filter.AddColumnSpec("Group Number", "insurance_policies", as_text, true, 120, rawString);
 insurance_filter.AddColumnSpec("Coverage Type", "insurance_benefits", as_text, true, 100);
 insurance_filter.AddColumnSpec("Coverage Note", "insurance_benefits", as_text, true, 120);
 insurance_filter.AddColumnSpec("Effective Date", "insurance_policies", as_date, true, 120);
 insurance_filter.AddColumnSpec("Termination Date", "insurance_policies", as_date, false, 120);
 insurance_filter.AddColumnSpec("Coverage", "insurance_policies", as_text, false, 120);
 insurance_filter.AddColumnSpec("Source", "insurance_policies", as_text, false, 120);
 insurance_filter.AddColumnSpec("Name on Policy", "insurance_policies", as_text, false, 120);
 insurance_filter.AddColumnSpec("PCP", "tax_entities", as_text, true, 120);
 insurance_filter.AddColumnSpec("Person Code", "insurance_coverage", as_int, false, 100);
 insurance_filter.AddColumnSpec("Name on Card", "insurance_coverage", as_text, false, 100);
 insurance_filter.AddColumnSpec("Current Id", "current_id", as_int, false, 80);
 insurance_filter.AddColumnSpec("Internal Id", "internal_id", as_int, false, 80);
 insurance_filter.FetchPreferences("recordspecialnewcob.insurance_filter");
 insurance_filter.SetIconStatus(true);
 insurance_filter.SetSQLStatement("");
 insurance_filter.RefreshDataSet();
 return;
}
//---------------------------------------------------------------------------
void TformSpecialNewProviderBill::Stop()
{
 return;
}
//---------------------------------------------------------------------------
bool TformSpecialNewProviderBill::AtlasScreen (int request_num)
{
 if (request_num == insurance_request)
  insurance_filter.RefreshDataSet();
 else
 if (request_num == services_request)
  services_filter.RefreshDataSet();
 return true;
}
//---------------------------------------------------------------------------
void __fastcall TformSpecialNewProviderBill::buttonBackClick(
      TObject *Sender)
{
 // back
 if (pageBill->ActivePageIndex == 1)
  GotoFirstPage();
 else
 if (pageBill->ActivePageIndex == 2)
  GotoFourthPage();
 else
 if (pageBill->ActivePageIndex == 3)
  if (is_new_cob)
   GotoSecondPage();
  else
   GotoFirstPage();
 else
 if (pageBill->ActivePageIndex == 4)
  if (is_new_cob)
   GotoThirdPage();
  else
   GotoFourthPage();
 else
 if (pageBill->ActivePageIndex == 5)
  GotoFifthPage();
 else
 if (pageBill->ActivePageIndex == 6)
  GotoFifthPage();
 else
 if (pageBill->ActivePageIndex == 7)
  if (is_hcfa)
    GotoSixthPage();
  else
   GotoSeventhPage();
 else
 if (pageBill->ActivePageIndex == 8)
  GotoEighthPage();
 return;
}
//---------------------------------------------------------------------------
void __fastcall TformSpecialNewProviderBill::buttonNextClick(
      TObject *Sender)
{
 // next
 if (pageBill->ActivePageIndex == 0)
 {
  // either find the cob, grab the id number from it and person_id ... and ... the provider?
  // then go to fourth, or
  // go to second to create a new cob, then put a bill on it ...
  if (checkHaveCOB->Checked)
  {
   if (editKnownCOBNumber->Text != "" && IsInt(editKnownCOBNumber->Text.c_str()))
   {
    string sql_find_cob = "select tasks.id, tasks.person_id, tasks.services_provided_id, services_provided.tax_entity_id from tasks inner join services_provided on services_provided.id = tasks.services_provided_id where tasks.cob_number = " + string(editKnownCOBNumber->Text.c_str());
    map<string, string> found_cob;
    if (SqlToMap(dbDatabase, sql_find_cob, found_cob))
    {
     if (found_cob["id"] != "" && IsInt(found_cob["id"]))
     {
      ResetFourthPage();
      cob_id = found_cob["id"];
      patient_id = found_cob["person_id"];
      service_id = found_cob["services_provided_id"];
      provider_id = found_cob["tax_entity_id"];
      tax_provider_id = provider_id;
      RedrawTaxProvider();

      // Philip forgot to reset the insurance list view.
      // But the sad (or funny) part is it took the users 4 monthes before
      // they figured out the insurance info wasn't showing.
      ResetInsurance();

      // now check to make sure they haven't already put a bill on here for this COB?
      string sql_find_visits = "select tasks.id from tasks where parent_task_id = " + cob_id;
      set<string> found_tasks;
      if (SqlToSet(dbDatabase, sql_find_visits, found_tasks))
      {
       if (found_tasks.size() > 0)
       {
        string count_of_tasks = IntToString(found_tasks.size());
        string warning_string = "There " + ((found_tasks.size() == 1) ? string("has ") : string("have ")) + "already been " + count_of_tasks + " bills entered for this COB. "
                                "\nIt is recommended that you check the person's previous bills (person screen) to make sure this is not a duplicate."
                                "\nWould you like to see the " + ((found_tasks.size() == 1) ? string("visit ") : string("visits ")) + " posted to this COB?";
        int answer = PSay(warning_string.c_str(), m_warning, MB_YESNO);
        if (answer == IDYES)
          {
           set<string>::iterator found_tasks_itr = found_tasks.begin();
           while (found_tasks_itr != found_tasks.end())
             {
              map <string, string> temp_map;
              temp_map["id"] = (*found_tasks_itr);
              FORM_TRY
                TformRecordTaskCobVisit *visit_with_a_bill_already_posted = new TformRecordTaskCobVisit(NULL, temp_map);
                visit_with_a_bill_already_posted->Show();
              FORM_CATCH
              found_tasks_itr++;
             }
          }
       }
      }
      else
      {
       PSay("Could not check for previously entered bills, internal error.", m_database_error, MB_OK);
      }
      is_new_cob = false;
      GotoFourthPage();
     }
     else
     {
      PSay("The COB number you have entered could not be found.", m_missing_data, MB_OK);
      return;
     }
    }
    else
    {
     PSay("The specified COB could not be found.", m_missing_data, MB_OK);
     return;
    }
   }
   else
   {
    PSay("In order to find a COB, you'll have to provide a numeric COB number.", m_invalid_data, MB_OK);
    return;
   }
  }
  else
  {
   is_new_cob = true;
   GotoSecondPage();
  }
 } else
 if (pageBill->ActivePageIndex == 1)
 {
  // select patient, provider, service, referring physician ... then go to fourth
  if (patient_id == "" || patient_id == "0")
  {
   PSay("You must select a patient in order to continue.", m_missing_data, MB_OK);
   return;
  } else
  if (service_id == "" || service_id == "0")
  {
   PSay("You must select a provider and a provider service.", m_missing_data, MB_OK);
   return;
  }

  ResetFourthPage();
  GotoFourthPage();
 } else
 if (pageBill->ActivePageIndex == 2)
 {
  // check date, time, reason? then go to fifth
  // check for duplicates
  string sql = "select id from tasks where person_id = " + patient_id + " and tax_entity_id = " + provider_id + " and date_of_task = \'" + Pdate(editDateService->Date).OutDate(mdy | yyyy) + "\'";
  vector<map<string, string> > visit_vec;
  if (!SqlToVecMap(dbDatabase, sql, visit_vec))
  {
    PSay("Unable to check for duplicates", m_database_error, MB_OK);
    return;
  }
  if (visit_vec.size() > 0)
  {
    possible_dup_non_referal = true;
    string error_message = "The patient " + string(staticPatient->Caption.c_str()) + " already has a visit for provider " +
                           staticProvider->Caption.c_str() + " on " + Pdate(editDateService->Date).OutDate(mdy | yyyy) +
                           "\n\nYou are most probably about to enter a duplicate bill." +
                           "\n\n\nAre you absolutely sure you want to continue?";
    int answer = PSay(error_message, m_confirmation, MB_YESNO);
    if (answer == IDNO)
      return;
  }
  if (editReason->Text == "")
  {
   int user_answer = PSay("You have not specified a reason/diagnosis for this COB. Would you like to continue anyway?", m_confirmation, MB_YESNO);
   if (user_answer == IDNO)
    return;
  }

  TIBTransaction * dbTransaction = new TIBTransaction(NULL);
  StdTransaction (dbTransaction, dbDatabase);
  set<string> problems_found;
  if (!TformRecordPerson::GetWarningsPreventingContractHealthActions (dbTransaction, patient_id, Pdate(editDateService->DateTime), problems_found))
  {
   PSay ("Unable to check on the validity of this person's tribal affiliation, stop-notes, etc.", m_database_error, MB_OK);
   return;
  }
  else if (!problems_found.empty())
  {
   string final_message = "The following problems were encountered for this person, for the date of service " + Pdate(editDateService->DateTime).OutDate(global_mode) + ":\n\n";
   for (set<string>::iterator i_pb = problems_found.begin(); i_pb != problems_found.end(); i_pb++)
    final_message += " - " + (*i_pb) + "\n";
   final_message += "\nWould you like to continue in spite of these problems, and override the warnings?";
   if (IDNO == PSay(final_message, m_warning, MB_YESNO))
    return;
   else
   {
    // TODO: record stuff here? (should be moved to a function in TformRecordPerson::* when this all works -- it's here for testing only)
   }
  }

  GotoFifthPage();
 } else
 if (pageBill->ActivePageIndex == 3)
 {
  // nothing to check, go to either fifth or third
  if (checkHaveCOB->Checked)
   GotoFifthPage();
  else
   GotoThirdPage();
 } else
 if (pageBill->ActivePageIndex == 4)
 {
  // make sure we have at least one tax entity selected, and move to either sixth or seventh
  if (checkPayProvider->Checked)
  {
   if (labelProviderTaxID->Caption == "no known tax ID number (cannot pay)" || labelProviderTaxID->Caption == "")
   {
    PSay("You cannot pay a tax entity without its tax ID number.", m_missing_data, MB_OK);
    return;
   }
   tax_actual_id = tax_provider_id;
  } else
  if (checkPayFacility->Checked)
  {
   if (labelFacilityTaxID->Caption == "no known tax ID number (cannot pay)" || labelFacilityTaxID->Caption == "")
   {
    PSay("You cannot pay a tax entity without its tax ID number.", m_missing_data, MB_OK);
    return;
   }
   tax_actual_id = tax_facility_id;
  } else
  if (checkPayGroup->Checked)
  {
   if (labelGroupTaxID->Caption == "no known tax ID number (cannot pay)" || labelGroupTaxID->Caption == "")
   {
    PSay("You cannot pay a tax entity without its tax ID number.", m_missing_data, MB_OK);
    return;
   }
   tax_actual_id = tax_group_id;
  }
  // move along now!
  if (checkIsHCFA->Checked)
   GotoSixthPage();
  else
  {                  
   is_hcfa = false;
   GotoSeventhPage();
  }
 } else
 if (pageBill->ActivePageIndex == 5)
 {
  // make sure this is not a duplicate payment
  if (possible_dup_non_referal)
  {
   string sql_to_find_duplicates = "select tasks.id from tasks inner join procedures on tasks.id = procedures.task_id "
                                   "where procedures.current_id >= 3 and "
                                   "person_id = " + patient_id + " and "
                                   "tax_entity_id = " + provider_id + " and "
                                   "date_of_task = \'" + Pdate(editDateService->Date).OutDate(mdy | yyyy) + "\' ";
   vector<vector<string> > all_given_procedures;
   GetFromList(listHCFAProcedures, all_given_procedures);

   vector<string> one_proc;
   for (unsigned int i = 0; i < all_given_procedures.size(); i++)
   {
    if (i == 0)
      sql_to_find_duplicates += " and (";
    one_proc = all_given_procedures[i];
    sql_to_find_duplicates += "procedures.cpt_hcpcs = \'" + UppercaseString(one_proc[4]) + "\' or ";
    if (i == (all_given_procedures.size() - 1))
    {
      sql_to_find_duplicates = sql_to_find_duplicates.substr(0, sql_to_find_duplicates.size() - 3);
      sql_to_find_duplicates += ")";
    }
    set<string> duplicate_bill_ids;
    if (!SqlToSet(dbDatabase, sql_to_find_duplicates, duplicate_bill_ids))
    {
       PSay("Unable to determine if this is a duplicate bill", m_database_error, MB_OK);
    }
    if (duplicate_bill_ids.size() > 0)
    {
       int answer = PSay("You are making a duplicate payment. Are you sure you want to continue?", m_confirmation, MB_OK);
       if(answer == IDNO)
         return;
    }
   }
  }
  // check hcfa1500
  GotoEighthPage();
 } else
 if (pageBill->ActivePageIndex == 6)
 {
  // make sure this is not a duplicate payment
  if (possible_dup_non_referal)
  {
   string sql_to_find_duplicates = "select tasks.id from tasks inner join procedures on tasks.id = procedures.task_id "
                                   "where procedures.current_id >= 3 and "
                                   "person_id = " + patient_id + " and "
                                   "tax_entity_id = " + provider_id + " and "
                                   "date_of_task = \'" + Pdate(editDateService->Date).OutDate(mdy | yyyy) + "\' ";
   vector<vector<string> > all_given_procedures;
   GetFromList(listUBProcedures, all_given_procedures);

   vector<string> one_proc;
   for (unsigned int i = 0; i < all_given_procedures.size(); i++)
   {
    if (i == 0)
      sql_to_find_duplicates += " and (";
    one_proc = all_given_procedures[i];
    sql_to_find_duplicates += "procedures.cpt_hcpcs = \'" + UppercaseString(one_proc[4]) + "\' or ";
    if (i == (all_given_procedures.size() - 1))
    {
      sql_to_find_duplicates = sql_to_find_duplicates.substr(0, sql_to_find_duplicates.size() - 3);
      sql_to_find_duplicates += ")";
    }
    set<string> duplicate_bill_ids;
    if (!SqlToSet(dbDatabase, sql_to_find_duplicates, duplicate_bill_ids))
    {
     PSay("Unable to determine if this is a duplicate bill", m_database_error, MB_OK);
    }
    if (duplicate_bill_ids.size() > 0)
    {
       int answer = PSay("You are making a duplicate payment (based on date of service, provider, and patient information.) Are you sure you want to continue?", m_confirmation, MB_OK);
       if(answer == IDNO)
         return;
    }
   }
  }
  // check the ub92
  GotoEighthPage();
 } else
 if (pageBill->ActivePageIndex == 7)
 {
  // verify payment, charges and residual (and the pesky checkbox thingie)
  if (editOriginalCharges->Text != "" && !IsFloat(editOriginalCharges->Text.c_str()))
  {
   PSay("Please enter a valid charge.", m_invalid_data, MB_OK);
   editOriginalCharges->SetFocus();
   return;
  } else
  if (editResidualCharges->Text != "" && !IsFloat(editResidualCharges->Text.c_str()))
  {
   PSay("Please enter a valid charge", m_invalid_data, MB_OK);
   editResidualCharges->SetFocus();
   return;
  }

  if (editOriginalCharges->Text == "")
  {
   int user_answer = PSay("You have not specified an original charge for this bill. Would you like to continue anyway?", m_confirmation, MB_YESNO);
   if (user_answer == IDNO)
    return;
  }
  if (editResidualCharges->Text == "")
  {
   int user_answer = PSay("You have not specified a residual charge for this bill. Would you like to continue anyway?", m_confirmation, MB_YESNO);
   if (user_answer == IDNO)
    return;
  }
  if (editControlNumber->Text == "")
  {
   int user_answer = PSay("You have not specified a patient account or control number. Would you like to continue anyway?", m_confirmation, MB_YESNO);
   if (user_answer == IDNO)
    return;
  }

  map<string, string> all_payments = GetAccountAmount (listPayments);
  float payment_amount = 0.00;
  for (map<string, string>::iterator i_payment = all_payments.begin(); i_payment != all_payments.end(); i_payment++)
   payment_amount += StringToFloat(AmountToString(i_payment->second));

  if (payment_amount <= 0.00)
  {
   int user_answer = PSay("You have chosen not to pay any moneys at all on this bill. Would you like to continue anyway?", m_confirmation, MB_YESNO);
   if (user_answer == IDNO)
    return;
  }

  if (StringToFinancial(editResidualCharges->Text.c_str()) != 0.00 &&
      StringToFinancial(editOriginalCharges->Text.c_str()) != 0.00 &&
      StringToFinancial(editResidualCharges->Text.c_str()) > StringToFinancial(editOriginalCharges->Text.c_str()))
  {
   int user_answer = PSay("You have entered residual charges greater than the original charges. This seems a bit dangerous, no? Would you like to continue anyway?", m_confirmation, MB_YESNO);
   // int user_answer = PSay("You have given an amount for the residual charges, but your payment differs from this amount. Would you like to continue anyway?", m_confirmation, MB_YESNO);
   if (user_answer == IDNO)
    return;
  }

  if (StringToFinancial(editResidualCharges->Text.c_str()) != 0.00 &&
      StringToFinancial(editResidualCharges->Text.c_str()) != payment_amount)
  {
   int user_answer = PSay("You have given an amount for the residual charges, but your payment differs from this amount. Would you like to continue anyway?", m_confirmation, MB_YESNO);
   // int user_answer = PSay("You have entered residual charges greater than the original charges. This seems a bit dangerous, no? Would you like to continue anyway?", m_confirmation, MB_YESNO);
   if (user_answer == IDNO)
    return;
  }

  GotoNinthPage();
 } else
 if (pageBill->ActivePageIndex == 8)
 {
  // finish, nothing to verify anymore
  TIBTransaction * dbTransaction = new TIBTransaction(NULL);
  try
  {
   dbTransaction->DefaultDatabase = dbDatabase;
   dbTransaction->DefaultAction = TARollback;
   dbTransaction->StartTransaction();

   if (is_new_cob)
   {
    map<string, string> insert_cob;
    insert_cob["id"] = GetNextId(dbTransaction, "tasks");
    insert_cob["current_id"] = "4";
    insert_cob["last_update_stamp"] = RightNow();
    insert_cob["last_updated_by"] = current_userid;
    insert_cob["person_id"] = patient_id;
    insert_cob["event_id"] = "0";
    insert_cob["group_id"] = "5"; // TODO -c MAGIC NUMBER :
    insert_cob["date_of_task"] = CurrentDate();
    insert_cob["time_of_task"] = CurrentTime();
    insert_cob["time_spent"] = "";
    insert_cob["mileage"] = "";
    insert_cob["task_comments"] = editReason->Text.c_str();
    insert_cob["visit_time_spent"] = "";
    insert_cob["visit_comments"] = "";
    insert_cob["method_id"] = "0"; // TODO :
    insert_cob["direction_id"] = "0"; // TODO :
    insert_cob["classification_id"] = "0"; // stupid field that needs to go away // TODO :
    insert_cob["beh_meeting_id"] = "0";
    insert_cob["beh_visit_id"] = "0";
    insert_cob["beh_activity_id"] = "0";
    insert_cob["visit_location_id"] = "0";
    insert_cob["is_educational"] = "0";
    insert_cob["is_personal"] = "0";
    insert_cob["diab_activity_id"] = "0";
    insert_cob["diab_contact_id"] = "0";
    insert_cob["diab_meeting_id"] = "0";
    insert_cob["diab_task_id"] = "0";
    insert_cob["diab_program_id"] = "0";
    insert_cob["user_id"] = "0"; // TODO :
    insert_cob["visit_outcome_id"] = "0";
    insert_cob["com_meeting_id"] = "0";
    insert_cob["com_activity_id"] = "0";
    insert_cob["com_visit_id"] = "0";
    insert_cob["new_client"] = "0";
    insert_cob["beh_city_id"] = "0";
    insert_cob["beh_appt_id"] = "0";
    insert_cob["beh_service_id"] = "0";
    insert_cob["com_dept_id"] = "0";
    insert_cob["tax_entity_id"] = "0"; // intentionally left this way
    insert_cob["parent_task_id"] = "0";
    insert_cob["patient_control_number"] = "";
    insert_cob["services_provided_id"] = service_id;
    insert_cob["in_progress"] = "0";
    insert_cob["preconditions_note"] = "";
    insert_cob["followup_note"] = "";
    insert_cob["reported_by_note"] = "";
    insert_cob["hospitalized"] = "0";
    insert_cob["given_room"] = "0";
    insert_cob["cob_number"] = insert_cob["id"];

    if (!ExecuteInsertStatement(dbTransaction, "tasks", insert_cob))
    {
     message_router->SendMessageToAll("debug", "Error on insert_cob");
     throw (3);
    }
    else
     PSay(string("The COB number assigned to this non-referral is " + insert_cob["cob_number"] + "\nYou may write this on the bill for future reference."), m_reminder, MB_OK);

    map<string, string> insert_appt;
    insert_appt["id"] = GetNextId(dbTransaction, "appointments");
    insert_appt["current_id"] = "4";
    insert_appt["last_update_stamp"] = RightNow();
    insert_appt["last_updated_by"] = current_userid;
    insert_appt["created_by"] = current_userid;
    insert_appt["created_for"] = "0"; // current_userid;
    insert_appt["description"] = "COB Number " +  insert_cob["cob_number"];
    Pdate appt_date(editDateService->Date);
    insert_appt["repeating_string"] = "0,0,0|0,0,0|0,0,0|0,0,0";
    insert_appt["begin_date"] = appt_date.OutDate(mdy | yyyy);
    insert_appt["end_date"] = appt_date.OutDate(mdy | yyyy);
    insert_appt["appointment_date"] = appt_date.OutDate(mdy | yyyy);
    insert_appt["appointment_time"] = Ptime(editTimeService->Time).OutTime(military | withoutsec);
    insert_appt["single_date"] = "1";
    insert_appt["duration"] = "30";
    insert_appt["duration_id"] = lookupTables->IdOf("duration_types", "Minutes");
    insert_appt["person_id"] = patient_id;
    insert_appt["visit_outcome_id"] = "0";
    insert_appt["tax_entity_id"] = provider_id;
    insert_appt["task_id"] = insert_cob["id"];
    Pstamp begin_stamp(insert_appt["appointment_date"] + " " + insert_appt["appointment_time"], mdy | yyyy | military | withsec | datefirst);
    Pstamp end_stamp = begin_stamp;
    end_stamp.mod(30, in_minutes);
    insert_appt["final_end_stamp"] = end_stamp.OutStamp(mdy | yyyy | military | withoutsec | datefirst);
    if (!ExecuteInsertStatement(dbTransaction, "appointments", insert_appt))
    {
     message_router->SendMessageToAll("debug", "Error on insert_appt");
     throw (7);
    }

    cob_id = insert_cob["id"];

    if (referring_id != "")
    {
     map<string, string> insert_ref;
     insert_ref["id"] = GetNextId(dbTransaction, "referrals");
     insert_ref["current_id"] = "4";
     insert_ref["last_update_stamp"] = RightNow();
     insert_ref["last_updated_by"] = current_userid;
     insert_ref["person_id"] = insert_cob["person_id"];
     insert_ref["by_provider_id"] = referring_id;
     insert_ref["by_facility_id"] = "0";
     insert_ref["to_provider_id"] = (provider_id != "") ? provider_id : string("0");
     insert_ref["to_facility_id"] = "0";
     insert_ref["date_of_referral"] = CurrentDate();
     insert_ref["notes"] = "NonRef-style referral";
     insert_ref["beh_refer_id"] = "0";
     insert_ref["com_refer_id"] = "0";
     insert_ref["diab_refer_id"] = "0";
     insert_ref["task_id"] = insert_cob["id"];
     if (!ExecuteInsertStatement(dbTransaction, "referrals", insert_ref))
     {
      message_router->SendMessageToAll("debug", "Error on insert_ref");
      throw (8);
     }
    }
   }


   map<string, string> insert_visit;
   insert_visit["id"] = GetNextId(dbTransaction, "tasks");
   insert_visit["current_id"] = "4";
   insert_visit["last_update_stamp"] = RightNow();
   insert_visit["last_updated_by"] = current_userid;
   insert_visit["person_id"] = patient_id;
   insert_visit["event_id"] = "0";
   insert_visit["group_id"] = "5"; // TODO -c MAGIC NUMBER :

   // added 6/11/2002 by Matt Green
   // This should be the visit date, not the cob creation date

   // 2/13/2003 -- Philip
   // well, while you're being so brilliant, you might note that the users have now gone 6 months
   // without noticing that transactions.effective_date should be equal to the date you just fixed.
   // 6 months. lovely. so i'll go fix that below, and just -hope- that date_of_task always has a
   // reasonable value. and we'll have to fix all the transactions for 2003 involving tasks in 2002
   // so we can close the accounts. ugh.
   if (is_new_cob)
   {
    insert_visit["date_of_task"] = Pdate(editDateService->Date).OutDate(mdy | yyyy);
   }
   else
   {
    insert_visit["date_of_task"] = Pdate(editCOBDOS->Date).OutDate(mdy | yyyy);
   }
   insert_visit["time_of_task"] = "";
// RRRRRRRRRRRR Philip, 2 weeks of work need to be fixed b/c of this!!!   insert_visit["date_of_task"] = Date().CurrentDate().FormatString("mm/dd/yyyy").c_str();
// RRRRRRRRRRRR insert_visit["time_of_task"] = Time().CurrentTime().FormatString("hh:nn:ss").c_str(); // minutes are "n", months are "m"
   insert_visit["time_spent"] = "";
   insert_visit["mileage"] = "";
   insert_visit["task_comments"] = "";
   insert_visit["visit_time_spent"] = "";
   insert_visit["visit_comments"] = ""; // editUsage->Text.c_str();
   insert_visit["method_id"] = lookupTables->IdOf("method_types", "Visit");
   insert_visit["direction_id"] = "0"; // TODO :
   insert_visit["classification_id"] = "0"; // stupid field that needs to go away // TODO :
   insert_visit["beh_meeting_id"] = "0";
   insert_visit["beh_visit_id"] = "0";
   insert_visit["beh_activity_id"] = "0";
   insert_visit["visit_location_id"] = "0";
   insert_visit["is_educational"] = "0";
   insert_visit["is_personal"] = "0";
   insert_visit["diab_activity_id"] = "0";
   insert_visit["diab_contact_id"] = "0";
   insert_visit["diab_meeting_id"] = "0";
   insert_visit["diab_task_id"] = "0";
   insert_visit["diab_program_id"] = "0";
   insert_visit["user_id"] = "0"; // TODO :
   insert_visit["visit_outcome_id"] = "0";
   insert_visit["com_meeting_id"] = "0";
   insert_visit["com_activity_id"] = "0";
   insert_visit["com_visit_id"] = "0";
   insert_visit["new_client"] = "0";
   insert_visit["beh_city_id"] = "0";
   insert_visit["beh_appt_id"] = "0";
   insert_visit["beh_service_id"] = "0";
   insert_visit["com_dept_id"] = "0";
   insert_visit["tax_entity_id"] = (provider_id != "" ) ? provider_id : string("0");
   insert_visit["parent_task_id"] = cob_id;
   insert_visit["patient_control_number"] = editControlNumber->Text.c_str();
   insert_visit["services_provided_id"] = "0";
   insert_visit["in_progress"] = "0";
   insert_visit["preconditions_note"] = "";
   insert_visit["followup_note"] = "";
   insert_visit["reported_by_note"] = "";
   insert_visit["hospitalized"] = "0";
   insert_visit["given_room"] = "0";
   insert_visit["cob_number"] = ""; // insert_visit["id"];

   if (!ExecuteInsertStatement(dbTransaction, "tasks", insert_visit))
   {
    message_router->SendMessageToAll("debug", "Error on insert_visit");
    throw (1);
   }

   map<string, string> insert_bill;
   insert_bill["id"] = GetNextId(dbTransaction, "bills"); // this is what has been eating task id's ... and caused bills to have (max(id) > gen_id(bills_gen,0))
   insert_bill["current_id"] = "4";
   insert_bill["last_update_stamp"] = RightNow();
   insert_bill["last_updated_by"] = current_userid;
   insert_bill["task_id"] = insert_visit["id"];
   insert_bill["user_id"] = "0";
   insert_bill["insurance_coverage_id"] = "0";
   insert_bill["provider_tax_id"] = (tax_provider_id != "") ? tax_provider_id : string("0");
   insert_bill["facility_tax_id"] = (tax_facility_id != "") ? tax_facility_id : string("0");
   insert_bill["group_tax_id"] = (tax_group_id != "") ? tax_group_id : string("0");
   insert_bill["actual_tax_id"] = (tax_actual_id != "") ? tax_actual_id : string("0");
   insert_bill["purchase_req_id"] = "0";
   if (IsFloat(editOriginalCharges->Text.c_str()))
    insert_bill["total_charge"] = editOriginalCharges->Text.c_str();
   if (IsFloat(editResidualCharges->Text.c_str()))
    insert_bill["invoice_amount"] = editResidualCharges->Text.c_str();
   insert_bill["bill_id"] = lookupTables->IdOf ("bill_types", "Incoming Provider Bill");
   insert_bill["bill_date"] = CurrentDate();
   insert_bill["invoice_number"] = "";
   insert_bill["paid_amount"] = editPaymentTotal->Text.c_str();

   if (!ExecuteInsertStatement(dbTransaction, "bills", insert_bill))
   {
    message_router->SendMessageToAll("debug", "Error on insert_bill");
    throw (8);
   }

   // insert procedures and conditions

   map<string, string> insert_cond_core;
   // insert_cond_core["id"]
   insert_cond_core["current_id"] = "4";
   insert_cond_core["last_update_stamp"] = RightNow();
   insert_cond_core["last_updated_by"] = current_userid;
   insert_cond_core["person_id"] = patient_id;
   insert_cond_core["condition_id"] = lookupTables->IdOf("condition_types", "Condition");
   // insert_cond_core["note"]
   insert_cond_core["start_date"] = insert_visit["date_of_task"];
   // insert_cond_core["stop_date"]
   // insert_cond_core["code"]
   insert_cond_core["group_id"] = "5"; // TODO -c MAGIC NUMBER :
   insert_cond_core["task_id"] = insert_visit["id"];
   // insert_cond_core["index_number"]

   vector<vector<string> > all_given_conditions;
   if (is_hcfa)
    GetFromList(listHCFAConditions, all_given_conditions);
   else
    GetFromList(listUBConditions, all_given_conditions);

   map<string, string> insert_cond;
   vector<string> one_cond;

   map<string, string> index_to_condition_id;

   for (unsigned int i = 0; i < all_given_conditions.size(); i++)
   {
    insert_cond = insert_cond_core;
    one_cond = all_given_conditions[i];
    insert_cond["id"] = GetNextId(dbTransaction, "conditions");
    if (is_hcfa)
    {
     insert_cond["code"] = one_cond[1];
     if (one_cond[0] != "" && IsInt(one_cond[0]))
     {
      insert_cond["index_number"] = one_cond[0];
      index_to_condition_id[insert_cond["index_number"]] = insert_cond["id"];
     }
    }
    else
     insert_cond["code"] = one_cond[0];
    if (!ExecuteInsertStatement(dbTransaction, "conditions", insert_cond))
    {
     message_router->SendMessageToAll("debug", "Error on insert_cond");
     for (map<string, string>::iterator i_thingie = insert_cond.begin(); i_thingie != insert_cond.end(); i_thingie++)
     {
      message_router->SendMessageToAll("debug", "[" + i_thingie->first + "] = " + i_thingie->second);
     }

     throw (13);
    }
   }

   map<string, string> insert_proc_core;
   // insert_proc_core["id"]
   insert_proc_core["current_id"] = "4";
   insert_proc_core["last_update_stamp"] = RightNow();
   insert_proc_core["last_updated_by"] = current_userid;
   // insert_proc_core["start_date"]
   // insert_proc_core["stop_date"]
   // insert_proc_core["place_of_service"]
   // insert_proc_core["type_of_service"]
   // insert_proc_core["cpt_hcpcs"]
   // insert_proc_core["modifier"]
   // insert_proc_core["charges"]
   // insert_proc_core["days_units"]
   // insert_proc_core["epsdt"]
   // insert_proc_core["emg"]
   // insert_proc_core["cob"]
   // insert_proc_core["local_use"]
   insert_proc_core["task_id"] = insert_visit["id"];

   map<string, string> insert_proc_cond_link_core;
   // insert_proc_core["id"];
   insert_proc_cond_link_core["current_id"] = "4";
   insert_proc_cond_link_core["last_update_stamp"] = RightNow();
   insert_proc_cond_link_core["last_updated_by"] = current_userid;
   // insert_proc_cond_link_core["procedure_id"]
   // insert_proc_cond_link_core["condition_id"]
   map<string, string> insert_proc_cond_link;

   vector<vector<string> > all_given_procedures;
   if (is_hcfa)
    GetFromList(listHCFAProcedures, all_given_procedures);
   else
    GetFromList(listUBProcedures, all_given_procedures);

   map<string, string> insert_proc;
   vector<string> one_proc;
   for (unsigned int i = 0; i < all_given_procedures.size(); i++)
   {
    one_proc = all_given_procedures[i];
    insert_proc = insert_proc_core;
    insert_proc ["id"] = GetNextId(dbTransaction, "procedures");
    if (is_hcfa)
    {
     if (one_proc[0] != "")
      insert_proc["start_date"] = one_proc[0];
     if (one_proc[1] != "")
      insert_proc["stop_date"] = one_proc[1];
     insert_proc["place_of_service"] = one_proc[2];
     insert_proc["type_of_service"] = one_proc[3];
     insert_proc["cpt_hcpcs"] = one_proc[4];
     insert_proc["modifier"] = one_proc[5];
     insert_proc["charges"] = one_proc[7];
     if (one_proc[8] != "" && IsInt(one_proc[8]))
      insert_proc["days_units"] = one_proc[8];
//     insert_proc["paid_amount"] = one_proc[9];
     insert_proc["local_use"] = one_proc[10];
    }
    else
    {
     if (one_proc[2] != "")
      insert_proc["start_date"] = one_proc[2];
     insert_proc["cpt_hcpcs"] = one_proc[1];
     if (one_proc[4] != "")
      insert_proc["charges"] = one_proc[4];
     if (one_proc[3] != "" && IsInt(one_proc[3]))
     insert_proc["days_units"] = one_proc[3];
     insert_proc["local_use"] = one_proc[0];
    }
    if (!ExecuteInsertStatement(dbTransaction, "procedures", insert_proc))
    {
     message_router->SendMessageToAll("debug", "Error on insert_proc");
     throw (13);
    }

    if (is_hcfa)
    {
     for (map<string, string>::iterator i_cond = index_to_condition_id.begin();
          i_cond != index_to_condition_id.end(); i_cond++)
     {
      if (one_proc[6].find(i_cond->first) != string::npos)
      {
       insert_proc_cond_link = insert_proc_cond_link_core;
       insert_proc_cond_link["id"] = GetNextId(dbTransaction, "procedure_reasons");
       insert_proc_cond_link["procedure_id"] = insert_proc["id"];
       insert_proc_cond_link["condition_id"] = i_cond->second;
       if (!ExecuteInsertStatement(dbTransaction, "procedure_reasons", insert_proc_cond_link))
       {
        message_router->SendMessageToAll("debug", "Error on insert_proc_cond_link");
        throw (13);
       }
      }
     }
    }
   }

   string current_transaction_number = GetNextId(dbTransaction, "transact_number");


   // TODO: if this payment does NOT cancel existing obligated moneys, should it decrement them?
   // ie.
   // obligate 500, pay 400, leave 100 obligated, and pay that later?
   // currently, obligate 500, pay 400, leave 0 obligated, then pay out of nowhere the other 100 ... 
   if (!checkDoNotCancelObligated->Checked && !is_new_cob)
   {
    string sql_select_stuff = "select * from transactions where task_id = " + cob_id;
    vector<map<string, string> > cob_transactions;
    SqlToVecMap(dbTransaction, sql_select_stuff, cob_transactions);

    for (vector<map<string, string> >::iterator i_old = cob_transactions.begin();
         i_old != cob_transactions.end(); i_old++)
    {
     map<string, string> reverse_transaction;
     reverse_transaction["transaction_number"] = current_transaction_number;
     reverse_transaction["effective_date"] = insert_visit["date_of_task"]; // CurrentDate(); -- changed 2/13/2003 by philip, at user request
     reverse_transaction["current_id"] = "4";
     reverse_transaction["last_update_stamp"] = RightNow();
     reverse_transaction["last_updated_by"] = current_userid;
     reverse_transaction["tax_entity_id"] = (*i_old)["tax_entity_id"];
     reverse_transaction["note"] = "Cancelling of COB expense/payable because of first bill";
     reverse_transaction["task_id"] = (*i_old)["task_id"];
     reverse_transaction["bill_id"] = (*i_old)["bill_id"];
     reverse_transaction["id"] = GetNextId(dbTransaction, "transactions");
     reverse_transaction["account_id"] = (*i_old)["account_id"];
     reverse_transaction["credit_amount"] = (*i_old)["debit_amount"];
     reverse_transaction["debit_amount"] = (*i_old)["credit_amount"];

     if (!ExecuteInsertStatement(dbTransaction, "transactions", reverse_transaction))
     {
      message_router->SendMessageToAll("debug", "Error on reverse_transaction");
      PSay("Failed to insert a reverse (on COB) transaction.", m_database_error, MB_OK);
      throw(6);
     }
    }
   }

   // new transactions on ... the bill?
   map<string, string> insert_transaction_core;
   insert_transaction_core["current_id"] = "4";
   insert_transaction_core["last_update_stamp"] = RightNow();
   insert_transaction_core["last_updated_by"] = current_userid;
   insert_transaction_core["task_id"] = "0";
   insert_transaction_core["bill_id"] = insert_bill["id"];
   insert_transaction_core["tax_entity_id"] = tax_actual_id;
   insert_transaction_core["effective_date"] = insert_visit["date_of_task"]; // CurrentDate(); -- changed 2/13/2003 by philip, at user request
   insert_transaction_core["credit_amount"] = "0.00";
   insert_transaction_core["debit_amount"] = "0.00";
   insert_transaction_core["transaction_number"] = current_transaction_number;

   map<string, string> all_payments = GetAccountAmount(listPayments);
   for (map<string, string>::iterator i_payment = all_payments.begin(); i_payment != all_payments.end(); i_payment++)
   {
    acc_info pay_account = EquivAccountInfo (pending_ref, i_payment->first);

    if (pay_account.IsClosed() || !pay_account.IsFound())
    {
     PSay("An accounts payable account number could not be found to match this expense account.", m_missing_data, MB_OK);
     throw(2);
    }

    string id_of_pay_account = pay_account.id;

    map<string, string> expense_transaction = insert_transaction_core;
    map<string, string> payable_transaction = insert_transaction_core;

    expense_transaction["id"] = GetNextId(dbTransaction, "transactions");
    expense_transaction["note"] = "Expense for Bill " + insert_visit["id"];
    expense_transaction["account_id"] = i_payment->first;
    expense_transaction["debit_amount"] = AmountToString(i_payment->second);

    payable_transaction["id"] = GetNextId(dbTransaction, "transactions");
    payable_transaction["note"] = "Payable for Bill " + insert_visit["id"];
    payable_transaction["account_id"] = id_of_pay_account;
    payable_transaction["credit_amount"] = AmountToString(i_payment->second);

    if (!ExecuteInsertStatement(dbTransaction, "transactions", expense_transaction))
    {
     message_router->SendMessageToAll("debug", "Error on expense_transaction");
     throw (3);
    }

    if (!ExecuteInsertStatement(dbTransaction, "transactions", payable_transaction))
    {
     message_router->SendMessageToAll("debug", "Error on payable_transaction");
     throw (4);
    }
   }

   dbTransaction->Commit();
   ResetFirstPage();
   GotoFirstPage();
  }
  catch (int e)
  {
   if (dbTransaction->Active)
    dbTransaction->Rollback();
   PSay("Couldn't save the COB.\nError check-point: " + IntToString(e), m_database_error, MB_OK);
  }
  catch (...)
  {
   if (dbTransaction->Active)
    dbTransaction->Rollback();
   PSay("Couldn't save the COB.", m_database_error, MB_OK);
  }
  delete dbTransaction;
 }
 return;
}
//---------------------------------------------------------------------------

void __fastcall TformSpecialNewProviderBill::buttonCancelClick(
      TObject *Sender)
{
 // cancel
 ResetFirstPage();
 GotoFirstPage();
 return;
}
//---------------------------------------------------------------------------

void __fastcall TformSpecialNewProviderBill::buttonCloseClick(
      TObject *Sender)
{
 // close
 Close();
 return;
}
//---------------------------------------------------------------------------

void __fastcall TformSpecialNewProviderBill::FormClose(TObject *Sender,
      TCloseAction &Action)
{
 // on form close
 Stop();
 TformSpecialGeneric::FormClose(Sender, Action);
 return;
}
//---------------------------------------------------------------------------

void __fastcall TformSpecialNewProviderBill::buttonPatientSearchClick(
      TObject *Sender)
{
 // search patient
 vector<string> return_vector;
 FORM_TRY
   TformSearchPerson * find_person = new TformSearchPerson(NULL, &return_vector);
   find_person->ShowTaskModal();
 FORM_CATCH

 if (return_vector.size() == 1)
 {
  patient_id = return_vector[0];
  CreateLink(staticPatient, buttonPatientSearch, this, patient_id, "people", "");
  ResetInsurance();
 }
 return;
}
//---------------------------------------------------------------------------

void __fastcall TformSpecialNewProviderBill::buttonProviderSearchClick(
      TObject *Sender)
{
 // search provider
 vector<string> return_vector;
 FORM_TRY
   TformSearchTaxEntity * find_provider = new TformSearchTaxEntity(NULL, &return_vector);
   find_provider->ShowTaskModal();
 FORM_CATCH

 if (return_vector.size() == 1)
 {
  provider_id = return_vector[0];
  service_id = "";
  CreateLink(staticProvider, buttonProviderSearch, this, provider_id, "tax_entities", "");
  ResetServices();
 }
 return;
}
//---------------------------------------------------------------------------

void __fastcall TformSpecialNewProviderBill::staticPersonLimitsClick(
      TObject *Sender)
{
 // see person's info
 map<string, string> params;
 params["id"] = patient_id;
 FORM_TRY
   TformSpecialPersonLimits * see_limits = new TformSpecialPersonLimits(NULL, params);
   see_limits->Show();
 FORM_CATCH
 return;
}
//---------------------------------------------------------------------------
void __fastcall TformSpecialNewProviderBill::buttonTaxProviderSearchClick(
      TObject *Sender)
{
 // search provider to pay
 vector<string> return_vector;
 FORM_TRY
   TformSearchTaxEntity * find_provider = new TformSearchTaxEntity(NULL, &return_vector);
   find_provider->ShowTaskModal();
 FORM_CATCH

 if (return_vector.size() == 1)
 {
  tax_provider_id = return_vector[0];
  RedrawTaxProvider();
 }
 return;
}
//---------------------------------------------------------------------------

void __fastcall TformSpecialNewProviderBill::buttonTaxFacilitySearchClick(
      TObject *Sender)
{
 // search facility to pay
 vector<string> return_vector;
 FORM_TRY
   TformSearchTaxEntity * find_facility = new TformSearchTaxEntity(NULL, &return_vector);
   find_facility->ShowTaskModal();
 FORM_CATCH

 if (return_vector.size() == 1)
 {
  tax_facility_id = return_vector[0];
  RedrawTaxFacility();
 }
 return;
}
//---------------------------------------------------------------------------

void __fastcall TformSpecialNewProviderBill::buttonTaxGroupSearchClick(
      TObject *Sender)
{
 // search group to pay
 vector<string> return_vector;
 FORM_TRY
   TformSearchTaxEntity * find_group = new TformSearchTaxEntity(NULL, &return_vector);
   find_group->ShowTaskModal();
 FORM_CATCH

 if (return_vector.size() == 1)
 {
  tax_group_id = return_vector[0];
  RedrawTaxGroup();
 }
 return;
}
//---------------------------------------------------------------------------

void __fastcall TformSpecialNewProviderBill::buttonHCFADeleteConditionClick(
      TObject *Sender)
{
 // delete hcfa condition
 if (listHCFAConditions->Selected)
 {
  listHCFAConditions->Selected->Delete();
  SetHCFACondition(NULL);
 }
 return;
}
//---------------------------------------------------------------------------

void __fastcall TformSpecialNewProviderBill::buttonHCFAAddConditionClick(
      TObject *Sender)
{
 // add hcfa condition

 if (editHCFACondition->Text == "")
 {
  PSay("Please enter a condition code.", m_invalid_data, MB_OK);
  return;
 }


 TListItem * edit_row;
 if (listHCFAConditions->Selected)
  edit_row = listHCFAConditions->Selected;
 else
  edit_row = listHCFAConditions->Items->Add();

 // TODO: check for duplicate position number (except for one currently selected)

 edit_row->Caption = editHCFAConditionPosition->Text;
 edit_row->SubItems->Clear();
 edit_row->SubItems->Add (editHCFACondition->Text);

 SetHCFACondition(NULL);
 listHCFAConditions->Selected = NULL;
 editHCFAConditionPosition->SetFocus();
 return;
}
//---------------------------------------------------------------------------

void __fastcall TformSpecialNewProviderBill::buttonHCFADeleteProcedureClick(
      TObject *Sender)
{
 // delete hcfa procedure
 if (listHCFAProcedures->Selected)
 {
  // subtract from payment total
  TListItem * edit_row;
  edit_row = listHCFAProcedures->Selected;
  editPaymentTotal->Text = StringToFloat(editPaymentTotal->Text.c_str()) - StringToFloat(edit_row->SubItems->Strings[8].c_str());
  lblHCFAPaymentTotal->Caption = editPaymentTotal->Text;

  listHCFAProcedures->Selected->Delete();
  SetHCFAProcedure(NULL);
 }
 return;
}
//---------------------------------------------------------------------------

void __fastcall TformSpecialNewProviderBill::buttonHCFAAddProcedureClick(
      TObject *Sender)
{
 // add hcfa procedure

 Pdate test_from;
 Pdate test_to;

 if (editHCFAFrom->Text != "")
 {
  test_from.InDate(editHCFAFrom->Text.c_str(), mdy | yy);
  if (!test_from.IsValidDate())
  {
   PSay("Please enter a valid from date.", m_invalid_data, MB_OK);
   editHCFAFrom->SetFocus();
   return;
  }
 }
 else
 {
  PSay("Please enter a from date.", m_missing_data, MB_OK);
  editHCFAFrom->SetFocus();
  return;
 }

 if (editHCFATo->Text != "")
 {
  test_to.InDate(editHCFATo->Text.c_str(), mdy | yy);
  if (!test_to.IsValidDate())
  {
   PSay("Please enter a valid to date.", m_invalid_data, MB_OK);
   editHCFATo->SetFocus();
   return;
  }
 }

 if (!IsFloat(editHCFACharges->Text.c_str()))
 {
  PSay("Please enter a numeric value for charges.", m_invalid_data, MB_OK);
  editHCFACharges->SetFocus();
  return;
 }

 if (editHCFAUnits->Text != "")
 {
  if (!IsInt(editHCFAUnits->Text.c_str()))
  {
   PSay("Please enter a numeric value for units.", m_invalid_data, MB_OK);
   editHCFAUnits->SetFocus();
   return;
  }
 }

 if (!IsFloat(editHCFAPayment->Text.c_str()))
 {
  PSay("Please enter a numeric value for payment.", m_invalid_data, MB_OK);
  editHCFAPayment->SetFocus();
  return;
 }

 if (editHCFAFrom->Text == "" &&
     editHCFATo->Text == "" &&
     editHCFAPOS->Text == "" &&
     editHCFATOS->Text == "" &&
     editHCFACode->Text == "" &&
     editHCFAMOD->Text == "" &&
     editHCFACharges->Text == "" &&
     editHCFAUnits->Text == "" &&
     editHCFAPayment->Text == "")
 {
  PSay("Please enter something, at least.", m_invalid_data, MB_OK);
  return;
 }

 TListItem * edit_row;
 if (listHCFAProcedures->Selected)
 {
  edit_row = listHCFAProcedures->Selected;

  // subtract out the old amount that is selected, and then add in the new amount from the edit box
  float old_total = StringToFloat(editPaymentTotal->Text.c_str());
  float old_amount = StringToFloat(edit_row->SubItems->Strings[8].c_str());
  float new_amount = StringToFloat(editHCFAPayment->Text.c_str());
  float new_total = (old_total - old_amount) + new_amount;
  editPaymentTotal->Text = FloatToString(new_total).c_str();
  lblHCFAPaymentTotal->Caption = FloatToString(new_total).c_str();
 }
 else
 {
  edit_row = listHCFAProcedures->Items->Add();

  // setup payment total
  editPaymentTotal->Text = FloatToString(StringToFloat(editPaymentTotal->Text.c_str()) + StringToFloat(editHCFAPayment->Text.c_str())).c_str();
  lblHCFAPaymentTotal->Caption = editPaymentTotal->Text;
 }

 edit_row->Caption = editHCFAFrom->Text;
 edit_row->SubItems->Clear();
 edit_row->SubItems->Add(editHCFATo->Text);
 edit_row->SubItems->Add(editHCFAPOS->Text);
 edit_row->SubItems->Add(editHCFATOS->Text);
 edit_row->SubItems->Add(editHCFACode->Text);
 edit_row->SubItems->Add(editHCFAMOD->Text);
 edit_row->SubItems->Add(editHCFAConditionLink->Text);
 edit_row->SubItems->Add(editHCFACharges->Text);
 edit_row->SubItems->Add(editHCFAUnits->Text);
 edit_row->SubItems->Add(editHCFAPayment->Text);
 edit_row->SubItems->Add(editHCFAReserved->Text);

 SetHCFAProcedure(NULL);
 listHCFAProcedures->Selected = NULL;
 editHCFAFrom->SetFocus();
 return;
}
//---------------------------------------------------------------------------

void __fastcall TformSpecialNewProviderBill::buttonUBDeleteConditionClick(
      TObject *Sender)
{
 // delete ub condition
 if (listUBConditions->Selected)
 {
  listUBConditions->Selected->Delete();
  SetUBCondition(NULL);
 }
 return;
}
//---------------------------------------------------------------------------

void __fastcall TformSpecialNewProviderBill::buttonUBAddConditionClick(
      TObject *Sender)
{
 if (editUBCondition->Text == "")
 {
  PSay("Please enter a condition code.", m_invalid_data, MB_OK);
  return;
 }

 // add ub condition
 TListItem * edit_row;
 if (listUBConditions->Selected)
  edit_row = listUBConditions->Selected;
 else
  edit_row = listUBConditions->Items->Add();

 edit_row->Caption = editUBCondition->Text;
 edit_row->SubItems->Clear();

 SetUBCondition(NULL);
 listUBConditions->Selected = NULL;
 editUBCondition->SetFocus();
 return;
}
//---------------------------------------------------------------------------

void __fastcall TformSpecialNewProviderBill::buttonUBDeleteProcedureClick(
      TObject *Sender)
{
 // delete ub procedure
 if (listUBProcedures->Selected)
 {
  // subtract from payment total
  TListItem * edit_row;
  edit_row = listUBProcedures->Selected;
  editPaymentTotal->Text = FloatToString(StringToFloat(editPaymentTotal->Text.c_str()) - StringToFloat(edit_row->SubItems->Strings[4].c_str())).c_str();
  lblUBPaymentTotal->Caption = editPaymentTotal->Text;

  listUBProcedures->Selected->Delete();
  SetUBProcedure(NULL);
 }
 return;
}
//---------------------------------------------------------------------------

void __fastcall TformSpecialNewProviderBill::buttonUBAddProcedureClick(
      TObject *Sender)
{
 // add ub procedure

 Pdate test_date;

 if (!IsFloat(editUBPayment->Text.c_str()))
 {
  PSay("Please enter a numeric value for payment.", m_invalid_data, MB_OK);
  editUBPayment->SetFocus();
  return;
 }

 if (editUBDate->Text != "")
 {
  test_date.InDate(editUBDate->Text.c_str(), mdy | yy);
  if (!test_date.IsValidDate())
  {
   PSay("Please enter a valid date.", m_invalid_data, MB_OK);
   editUBDate->SetFocus();
   return;
  }
 }

 if (editUBUnits->Text != "")
 {
  if (!IsInt(editUBUnits->Text.c_str()))
  {
   PSay("Please enter a numeric value for units.", m_invalid_data, MB_OK);
   editUBUnits->SetFocus();
   return;
  }
 }

 if (!IsFloat(editUBCharges->Text.c_str()))
 {
  PSay("Please enter a numeric value for charges.", m_invalid_data, MB_OK);
  editUBCharges->SetFocus();
  return;
 }

 // no blanks, at least
 if (editUBDescription->Text == "" &&
     editUBCode->Text == "" &&
     editUBDate->Text == "" &&
     editUBUnits->Text == "" &&
     editUBCharges->Text == "")
 {
  PSay("Please enter something, at least.", m_invalid_data, MB_OK);
  return;
 }


 TListItem * edit_row;
 if (listUBProcedures->Selected)
 {
  // subtract out the old amount that is selected, and then add in the new amount from the edit box
  edit_row = listUBProcedures->Selected;
  editPaymentTotal->Text = FloatToString(StringToFloat(editPaymentTotal->Text.c_str()) - StringToFloat(edit_row->SubItems->Strings[4].c_str())).c_str();
  editPaymentTotal->Text = FloatToString(StringToFloat(editPaymentTotal->Text.c_str()) + StringToFloat(editUBPayment->Text.c_str())).c_str();
  lblUBPaymentTotal->Caption = editPaymentTotal->Text;
 }
 else
 {
  edit_row = listUBProcedures->Items->Add();

  // setup payment total
  editPaymentTotal->Text = FloatToString(StringToFloat(editPaymentTotal->Text.c_str()) + StringToFloat(editUBPayment->Text.c_str())).c_str();
  lblUBPaymentTotal->Caption = editPaymentTotal->Text;
 }

 edit_row->Caption = editUBDescription->Text;
 edit_row->SubItems->Clear();
 edit_row->SubItems->Add(editUBCode->Text);
 edit_row->SubItems->Add(editUBDate->Text);
 edit_row->SubItems->Add(editUBUnits->Text);
 edit_row->SubItems->Add(editUBCharges->Text);
 edit_row->SubItems->Add(editUBPayment->Text);

 SetUBProcedure(NULL);
 listUBProcedures->Selected = NULL;
 editUBDescription->SetFocus();

 return;
}
//---------------------------------------------------------------------------

void __fastcall TformSpecialNewProviderBill::buttonDeletePaymentClick(
      TObject *Sender)
{
 // delete payment
 DelAccountAmount (editAccount, editAmount, listPayments, acceptable_payments);
 RefreshBudgetInfo();
}
//---------------------------------------------------------------------------

void __fastcall TformSpecialNewProviderBill::buttonAddPaymentClick(
      TObject *Sender)
{
 // add payment
 AddAccountAmount (editAccount, editAmount, listPayments, acceptable_payments);
 RefreshBudgetInfo();
}
//---------------------------------------------------------------------------

void __fastcall TformSpecialNewProviderBill::listServicesProvidedSelectItem(
      TObject *Sender, TListItem *Item, bool Selected)
{
 // select provider service
 service_id = services_filter.GetSelectedRecordID();
}
//---------------------------------------------------------------------------

void __fastcall TformSpecialNewProviderBill::listReferringPhysiciansSelectItem(
      TObject *Sender, TListItem *Item, bool Selected)
{
 // select (or unselect) referring physician
 referring_id = referrer_filter.GetSelectedRecordID();
}
//---------------------------------------------------------------------------

void __fastcall TformSpecialNewProviderBill::listHCFAConditionsSelectItem(
      TObject *Sender, TListItem *Item, bool Selected)
{
 // select hcfa condition
 SetHCFACondition(listHCFAConditions->Selected);
 return;
}
//---------------------------------------------------------------------------

void __fastcall TformSpecialNewProviderBill::listHCFAProceduresSelectItem(
      TObject *Sender, TListItem *Item, bool Selected)
{
 // select hcfa procedure
 SetHCFAProcedure(listHCFAProcedures->Selected);
 return;
}
//---------------------------------------------------------------------------

void __fastcall TformSpecialNewProviderBill::listUBConditionsSelectItem(
      TObject *Sender, TListItem *Item, bool Selected)
{
 // select ub condition
 SetUBCondition(listUBConditions->Selected);
 return;
}
//---------------------------------------------------------------------------

void __fastcall TformSpecialNewProviderBill::listUBProceduresSelectItem(
      TObject *Sender, TListItem *Item, bool Selected)
{
 // select ub procedure
 SetUBProcedure(listUBProcedures->Selected);
 return;
}
//---------------------------------------------------------------------------

void __fastcall TformSpecialNewProviderBill::listPaymentsSelectItem(
      TObject *Sender, TListItem *Item, bool Selected)
{
 // select payment
 ModAccountAmount (editAccount, editAmount, listPayments, acceptable_payments);
 RefreshBudgetInfo();
}
//---------------------------------------------------------------------------
void TformSpecialNewProviderBill::RefreshBudgetInfo()
{
 map<string, string> all_payments = GetAccountAmount (listPayments);
 float payment_amount = 0.00;
 for (map<string, string>::iterator i_payment = all_payments.begin(); i_payment != all_payments.end(); i_payment++)
  payment_amount += StringToFloat(AmountToString(i_payment->second));

 labelAmountPayment->Caption = StringToAmount(FloatToString(payment_amount)).c_str();
 return;
}
//---------------------------------------------------------------------------
void TformSpecialNewProviderBill::ResetServices()
{
 atlas->CancelRequest(services_request);
 if (provider_id == "" || !IsInt(provider_id))
 {
  services_filter.SetSQLStatement("");
  services_filter.SetAddEdit(this, NULL, NULL);
 }
 else
 {
  string sql_select_services = "select service_types.item, area_types.item, services_provided.current_id, services_provided.id "
                               "from services_provided "
                               "inner join service_types on services_provided.service_id = service_types.id "
                               "inner join area_types on services_provided.area_id = area_types.id "
                               "where services_provided.current_id >  2 and "
                               "services_provided.tax_entity_id = " + provider_id;

  services_filter.SetSQLStatement(sql_select_services);
  services_filter.RefreshDataSet();
  services_filter.SetAddEdit(this, AddRecord(&(TformSpecialNewProviderBill::AddServiceProvided)), EditRecord(&(TformSpecialNewProviderBill::EditServiceProvided)));
  services_request = atlas->RequestNotificationWhen("services_provided.tax_entity_id", provider_id, rel_e, as_int, this,
                      RefreshHandler(&(TformSpecialNewProviderBill::AtlasScreen)));
 }
 services_filter.RefreshDataSet();
 return;
}
//---------------------------------------------------------------------------
void TformSpecialNewProviderBill::ResetInsurance()
{
 atlas->CancelRequest(insurance_request);
 if((patient_id != "") && (patient_id != "0"))
 {
   string sql;
   sql = "select a.name, insurance_types.item, insurance_policies.policy_number, insurance_policies.group_number, "
         "area_types.item, insurance_benefits.note, "
         "insurance_policies.start_date, insurance_policies.stop_date, coverage_types.item, "
         "source_types.item, insurance_policies.name_on_policy, b.name, insurance_coverage.person_code, "
         "insurance_coverage.name_on_card, insurance_coverage.current_id, insurance_coverage.id "
         "from insurance_coverage "
         "left join insurance_policies on insurance_policies.id = insurance_coverage.insurance_policy_id "
         "left join coverage_types on coverage_types.id = insurance_policies.coverage_id "
         "left join source_types on source_types.id = insurance_policies.source_id "
         "left join insurance_benefits on insurance_policies.id = insurance_benefits.insurance_policy_id "
         "left join area_types on area_types.id = insurance_benefits.area_id "
         "left join tax_entities a on a.id = insurance_policies.tax_entity_id "
         "left join insurance_types on insurance_types.id = a.insurance_id "
         "left join tax_entities b on b.id = insurance_coverage.tax_entity_id "
         "where insurance_coverage.person_id = " + patient_id;
   insurance_filter.SetSQLStatement (sql);
   insurance_filter.SetAddEdit(this, AddRecord(&(TformSpecialNewProviderBill::AddInsurance)), EditRecord(&(TformSpecialNewProviderBill::EditInsurance)));
   insurance_request = atlas->RequestNotificationWhen("insurance_coverage.person_id", patient_id, rel_e, as_int, this,
                       RefreshHandler(&(TformSpecialNewProviderBill::AtlasScreen)));
 }
 else
 {
  insurance_filter.SetSQLStatement("");
  insurance_filter.SetAddEdit(this, NULL, NULL);
 }
 insurance_filter.RefreshDataSet();
 return;
}
//---------------------------------------------------------------------------
void TformSpecialNewProviderBill::AddInsurance()
{
 map<string, string> params;
 params["person_id"] = patient_id;
 FORM_TRY
   TformRecordInsuranceCoverage * add_coverage = new TformRecordInsuranceCoverage(NULL, params);
   add_coverage->Show();
 FORM_CATCH
 return;
}
//---------------------------------------------------------------------------
void TformSpecialNewProviderBill::EditInsurance()
{
 map<string, string> params;
 params["id"] = insurance_filter.GetSelectedRecordID();
 FORM_TRY
   TformRecordInsuranceCoverage * edit_coverage = new TformRecordInsuranceCoverage(NULL, params);
   edit_coverage->Show();
 FORM_CATCH
 return;
}
//---------------------------------------------------------------------------
void TformSpecialNewProviderBill::AddServiceProvided()
{
 if (provider_id != "")
 {
  map<string, string> params;
  params["tax_entity_id"] = provider_id;
  FORM_TRY
   TformRecordServiceProvided * add_service = new TformRecordServiceProvided(NULL, params);
   add_service->Show();
  FORM_CATCH
 }
 else
  PSay("Cannot add a service provided without specifying which provider is to have the new service.", m_missing_data, MB_OK);
 return;
}
//---------------------------------------------------------------------------
void TformSpecialNewProviderBill::EditServiceProvided()
{
 map<string, string> params;
 params["id"] = services_filter.GetSelectedRecordID();
 FORM_TRY
  TformRecordServiceProvided * edit_service = new TformRecordServiceProvided(NULL, params);
   edit_service->Show();
 FORM_CATCH
 return;
}
//---------------------------------------------------------------------------
void TformSpecialNewProviderBill::SetHCFACondition(TListItem * based_on)
{
 if (based_on)
 {
  editHCFAConditionPosition->Text = based_on->Caption;
  editHCFACondition->Text = based_on->SubItems->Strings[0];
 }
 else
 {
  editHCFAConditionPosition->Text = "";
  editHCFACondition->Text = "";
 }
 return;
}
//---------------------------------------------------------------------------
void TformSpecialNewProviderBill::SetUBCondition(TListItem * based_on)
{
 if (based_on)
 {
  editUBCondition->Text = based_on->Caption;
 }
 else
 {
  editUBCondition->Text = "";
 }
 return;
}
//---------------------------------------------------------------------------
void TformSpecialNewProviderBill::SetHCFAProcedure(TListItem * based_on)
{
 if (based_on)
 {
  editHCFAFrom->Text = based_on->Caption;
  editHCFATo->Text = based_on->SubItems->Strings[0];
  editHCFAPOS->Text = based_on->SubItems->Strings[1];
  editHCFATOS->Text = based_on->SubItems->Strings[2];
  editHCFACode->Text = based_on->SubItems->Strings[3];
  editHCFAMOD->Text = based_on->SubItems->Strings[4];
  editHCFAConditionLink->Text = based_on->SubItems->Strings[5];
  editHCFACharges->Text = based_on->SubItems->Strings[6];
  editHCFAUnits->Text = based_on->SubItems->Strings[7];
  editHCFAPayment->Text = based_on->SubItems->Strings[8];
  editHCFAReserved->Text = based_on->SubItems->Strings[9];
 }
 else
 {
  editHCFAFrom->Text = "";
  editHCFATo->Text = "";
  editHCFAPOS->Text = "";
  editHCFATOS->Text = "";
  editHCFACode->Text = "";
  editHCFAMOD->Text = "";
  editHCFAConditionLink->Text = "";
  editHCFACharges->Text = "";
  editHCFAUnits->Text = "";
  editHCFAPayment->Text = "";
  editHCFAReserved->Text = "";
 }
 return;
}
//---------------------------------------------------------------------------
void TformSpecialNewProviderBill::SetUBProcedure(TListItem * based_on)
{
 if (based_on)
 {
  editUBDescription->Text = based_on->Caption;
  editUBCode->Text = based_on->SubItems->Strings[0];
  editUBDate->Text = based_on->SubItems->Strings[1];
  editUBUnits->Text = based_on->SubItems->Strings[2];
  editUBCharges->Text = based_on->SubItems->Strings[3];
  editUBPayment->Text = based_on->SubItems->Strings[4];
 }
 else
 {
  editUBDescription->Text = "";
  editUBCode->Text = "";
  editUBDate->Text = "";
  editUBUnits->Text = "";
  editUBCharges->Text = ""; 
  editUBPayment->Text = "";
 }
 return;
}
//---------------------------------------------------------------------------
void TformSpecialNewProviderBill::RedrawTaxProvider()
{
 CreateLink(staticTaxProvider, buttonTaxProviderSearch, this, tax_provider_id, "tax_entities", "");
 labelProviderTaxID->Caption = "";

 if (tax_provider_id != "" && IsInt(tax_provider_id))
  labelProviderTaxID->Caption = GetOneField(dbDatabase, "select taxid from tax_entities where id = " + tax_provider_id).c_str();

 if (labelProviderTaxID->Caption == "")
  labelProviderTaxID->Caption = "no known tax ID number (cannot pay)";
 return;
}
//---------------------------------------------------------------------------
void TformSpecialNewProviderBill::RedrawTaxFacility()
{
 CreateLink(staticTaxFacility, buttonTaxFacilitySearch, this, tax_facility_id, "tax_entities", "");
 labelFacilityTaxID->Caption = "";

 if (tax_facility_id != "" && IsInt(tax_facility_id))
  labelFacilityTaxID->Caption = GetOneField(dbDatabase, "select taxid from tax_entities where id = " + tax_facility_id).c_str();

 if (labelFacilityTaxID->Caption == "")
  labelFacilityTaxID->Caption = "no known tax ID number (cannot pay)";
 return;
}
//---------------------------------------------------------------------------
void TformSpecialNewProviderBill::RedrawTaxGroup()
{
 CreateLink(staticTaxGroup, buttonTaxGroupSearch, this, tax_group_id, "tax_entities", "");
 labelGroupTaxID->Caption = "";

 if (tax_group_id != "" && IsInt(tax_group_id))
  labelGroupTaxID->Caption = GetOneField(dbDatabase, "select taxid from tax_entities where id = " + tax_group_id).c_str();

 if (labelGroupTaxID->Caption == "")
  labelGroupTaxID->Caption = "no known tax ID number (cannot pay)";
 return;
}
//---------------------------------------------------------------------------
void __fastcall TformSpecialNewProviderBill::editHCFAFromKeyPress(
      TObject *Sender, char &Key)
{
 if (Key == VK_RETURN)
 {

  if (editHCFAFrom->Focused())
   editHCFATo->SetFocus();
  else if (editHCFATo->Focused())
   editHCFAPOS->SetFocus();
  else if (editHCFAPOS->Focused())
   editHCFATOS->SetFocus();
  else if (editHCFATOS->Focused())
   editHCFACode->SetFocus();
  else if (editHCFACode->Focused())
   editHCFAMOD->SetFocus();
  else if (editHCFAMOD->Focused())
   editHCFAConditionLink->SetFocus();
  else if (editHCFAConditionLink->Focused())
   editHCFACharges->SetFocus();
  else if (editHCFACharges->Focused())
   editHCFAUnits->SetFocus();
  else if (editHCFAUnits->Focused())
   editHCFAPayment->SetFocus();
  else if (editHCFAPayment->Focused())
   editHCFAReserved->SetFocus();
  else if (editHCFAReserved->Focused())
   buttonHCFAAddProcedure->Click();

  if (editHCFAConditionPosition->Focused())
   editHCFACondition->SetFocus();
  else if (editHCFACondition->Focused())
   buttonHCFAAddCondition->Click();

  if (editUBDescription->Focused())
   editUBPayment->SetFocus();
  else if (editUBPayment->Focused())
   editUBCode->SetFocus();
  else if (editUBCode->Focused())
   editUBDate->SetFocus();
  else if (editUBDate->Focused())
   editUBUnits->SetFocus();
  else if (editUBUnits->Focused())
   editUBCharges->SetFocus();
  else if (editUBCharges->Focused())
   buttonUBAddProcedure->Click();
  else if (editUBCondition->Focused())
   buttonUBAddCondition->Click();

  Key = 0;
 } else
 if (Key == VK_BACK)
 {
  if (editHCFATo->Focused() && editHCFATo->Text == "")
   editHCFAFrom->SetFocus();
  else if (editHCFAPOS->Focused() && editHCFAPOS->Text == "")
   editHCFATo->SetFocus();
  else if (editHCFATOS->Focused() && editHCFATOS->Text == "")
   editHCFAPOS->SetFocus();
  else if (editHCFACode->Focused() && editHCFACode->Text == "")
   editHCFATOS->SetFocus();
  else if (editHCFAMOD->Focused() && editHCFAMOD->Text == "")
   editHCFACode->SetFocus();
  else if (editHCFAConditionLink->Focused() && editHCFAConditionLink->Text == "")
   editHCFAMOD->SetFocus();
  else if (editHCFACharges->Focused() && editHCFACharges->Text == "")
   editHCFAConditionLink->SetFocus();
  else if (editHCFAUnits->Focused() && editHCFAUnits->Text == "")
   editHCFACharges->SetFocus();
  else if (editHCFAPayment->Focused() && editHCFAPayment->Text == "")
   editHCFAUnits->SetFocus();
  else if (editHCFAReserved->Focused() && editHCFAReserved->Text == "")
   editHCFAPayment->SetFocus();
  else if (editHCFACondition->Focused() && editHCFACondition->Text == "")
   editHCFAConditionPosition->SetFocus();

  if (editUBPayment->Focused() && editUBPayment->Text == "")
   editUBDescription->SetFocus();
  else if (editUBCode->Focused() && editUBCode->Text == "")
   editUBPayment->SetFocus();
  else if (editUBDate->Focused() && editUBDate->Text == "")
   editUBCode->SetFocus();
  else if (editUBUnits->Focused() && editUBUnits->Text == "")
   editUBDate->SetFocus();
  else if (editUBCharges->Focused() && editUBCharges->Text == "")
   editUBUnits->SetFocus();
  else if (buttonUBAddProcedure->Focused())
   editUBCharges->SetFocus();
  else if (buttonUBAddCondition->Focused())
   editUBCondition->SetFocus();

  // The following makes it to where the user can't delete out of the edit box.  I don't know why Philip did this.
  // Key = 0;
 }
}
//---------------------------------------------------------------------------
void __fastcall TformSpecialNewProviderBill::OnChangeCOBNumber(TObject *Sender)
{
   string sql = "select person_id from tasks where tasks.cob_number = ";
   sql += editKnownCOBNumber->Text.c_str();
   string person_id = GetOneField(dbDatabase, sql);
   CreateLink(txtPerson, NULL, this, person_id, "people");
}
//---------------------------------------------------------------------------

