//---------------------------------------------------------------------------

#include <vcl.h>
#pragma hdrstop

#include "codeSpecialNewTransportationTasks.h"
#include "codeSearchPerson.h"
#include "control_break_objects.h"
//---------------------------------------------------------------------------
#pragma package(smart_init)
#pragma link "codeSpecialGeneric"
#pragma resource "*.dfm"
TformSpecialNewTransportationTasks *formSpecialNewTransportationTasks;
//---------------------------------------------------------------------------
__fastcall TformSpecialNewTransportationTasks::TformSpecialNewTransportationTasks(TComponent* Owner)
        : TformSpecialGeneric(Owner)
{
 lookupTables->RegisterComboBox("users", comboUser);

 list_stops.setup_listview(listStops, false);
 list_stops.add_or_modify_column("mileage", "Mileage", true, 0, 80, as_int, rawString);
 list_stops.add_or_modify_column("time", "Time", true, 1, 80, as_time, rawString);
 list_stops.add_or_modify_column("action", "Action", true, 2, 90, as_text, rawString);
 list_stops.add_or_modify_column("person", "Person", true, 3, 90, as_text, rawString);
 list_stops.add_or_modify_column("note", "Note", true, 4, 120, as_text, rawString);
 list_stops.add_or_modify_column("action_id", "Action (id)", false, 5, 60, as_int, rawString);
 list_stops.add_or_modify_column("person_id", "Person (id)", false, 6, 60, as_int, rawString);
 list_stops.setup_callbacks((TFrame*)this, InformForm(&(TformSpecialNewTransportationTasks::StopSelected)), NULL);

 ClearScreen();
}
//---------------------------------------------------------------------------
__fastcall TformSpecialNewTransportationTasks::~TformSpecialNewTransportationTasks()
{
 lookupTables->UnregisterComboBox(comboUser);
}
//---------------------------------------------------------------------------
void __fastcall TformSpecialNewTransportationTasks::comboActionChange(
      TObject *Sender)
{
 // if it's a leave-office action, show no further controls
 // if it's a pick-up-person action, switch to person link and button
 // if it's a drop-off-person action, switch to person drop-down and populate it
 // if it's a drop-off-meds action, switch to person link and button
 // if it's a return-to-office action, show no further controls
 int chosen_action = comboAction->ItemIndex;
 comboPerson->Visible = (chosen_action == 2);
 staticPerson->Visible = (chosen_action == 1 || chosen_action == 3);
 buttonSearchPerson->Visible = staticPerson->Visible;
 labelPerson->Visible = (comboPerson->Visible || staticPerson->Visible);
 if (comboPerson->Visible)
  RefreshPersonCombo(person_id);
 editNote->Visible = (chosen_action != 1 && !(listStops->Items->Count == 0 && chosen_action == 0)); // just ... trying to prevent notes from being lost, you know?
 labelNote->Visible = editNote->Visible;
}
//---------------------------------------------------------------------------
void __fastcall TformSpecialNewTransportationTasks::comboActionKeyPress(
      TObject *Sender, char &Key)
{
 FindComboBoxItemAndSetItemIndex(Sender, Key);
}
//---------------------------------------------------------------------------
void __fastcall TformSpecialNewTransportationTasks::buttonStopSaveClick(
      TObject *Sender)
{
 // meh. not optional. makes life easier elsewhere, and ... isn't this rather the purpose?
 if (IsBlankOrZero(editMileage->Text.c_str()))
 {
  PSay("Please enter a mileage for this step along your route.", m_invalid_data, MB_OK);
  return;
 }

 if (comboAction->ItemIndex == -1)
 {
  PSay("Please select an action.", m_missing_data, MB_OK);
  return;
 }

 if ((comboPerson->Visible && comboPerson->Text == "") ||
     (staticPerson->Visible && IsBlankOrZero(person_id)))
 {
  PSay("Please select a person associated with this action.", m_missing_data, MB_OK);
  return;
 }

 if (comboPerson->Visible)
  person_id = InterpretPersonCombo();
 else if (staticPerson->Visible)
  ; // person_id is set by the search button, don't try to change it here
 else
  person_id = "0";

 map<string, string> row_data;
 row_data["mileage"]   = editMileage->Text.c_str();
 row_data["time"]      = Ptime(timeStop->DateTime).OutTime(global_mode | withoutsec); // strip them there thar seconds off, 'cause they're ugly and pointless.
 row_data["action"]    = comboAction->Text.c_str();
 row_data["person"]    = (IsBlankOrZero(person_id) ? string("") : DefaultText(person_id, "people"));
 row_data["action_id"] = IntToString(comboAction->ItemIndex);
 row_data["person_id"] = person_id;
 row_data["note"]      = editNote->Text.c_str();

 // TODO: other checking, as follows
 // - watch for picking the same person up twice without dropping them off
 // - watch for dropping the same person off twice without picking them up again
 // - watch for dropping someone off before picking them up
 // ...

 list_stops.add_or_modify_row_data(row_data, false);
 listStops->Selected = NULL;

 editMileage->SetFocus();
}
//---------------------------------------------------------------------------
void __fastcall TformSpecialNewTransportationTasks::buttonStopAddClick(
      TObject *Sender)
{
 listStops->Selected = NULL;
}
//---------------------------------------------------------------------------
void __fastcall TformSpecialNewTransportationTasks::buttonStopRemoveClick(
      TObject *Sender)
{
 list_stops.remove_current_row(true);
}
//---------------------------------------------------------------------------
void __fastcall TformSpecialNewTransportationTasks::buttonStopClearClick(
      TObject *Sender)
{
 list_stops.clear_data();
}
//---------------------------------------------------------------------------
void __fastcall TformSpecialNewTransportationTasks::buttonSearchPersonClick(
      TObject *Sender)
{
 vector<string> return_vector = TformSearchPerson::Do(this);
 if (!return_vector.empty())
 {
  person_id = return_vector[0];
  CreateLink(staticPerson, buttonSearchPerson, this, person_id, "people");
 }
}
//---------------------------------------------------------------------------
void TformSpecialNewTransportationTasks::StopSelected ()
{
 // select goal
 map<string, string> row_data;
 if (!list_stops.get_current_row(row_data))
 {
  // set default values
  row_data["mileage"] = ""; // reset to previous? would that even be helpful?
  row_data["time"]    = "08:00"; // 'cause?
  row_data["action"]  = "Leave office"; // maybe pick a different default if there are already rows? but which? argh.
  row_data["person"]  = "";
  row_data["action_id"] = "0";
  row_data["person_id"] = "0";
  row_data["note"]      = "";

  // grab latest, 'cause ... at least maybe that'll remind them where they were?
  vector<map<string, string> > sorted_data;
// paul asked me to disable this, because Geraldine is a slow typist and this was getting in the way
/*
  GetStopsSortedByMileage(sorted_data);
  if (!sorted_data.empty())
   row_data["mileage"] = sorted_data[sorted_data.size() - 1]["mileage"];
*/
  GetStopsSortedByTime(sorted_data);
  if (!sorted_data.empty())
   row_data["time"] = sorted_data[sorted_data.size() - 1]["time"];
 }

 // update controls to reflect data in row_data
 person_id = row_data["person_id"];
 editMileage->Text = row_data["mileage"].c_str();
 timeStop->Time = Ptime(row_data["time"], global_mode).OutTime();
 comboAction->ItemIndex = StringToInt(row_data["action_id"]);
 editNote->Text = row_data["note"].c_str();
 if (comboAction->OnChange)
  comboAction->OnChange(comboAction);
 CreateLink(staticPerson, buttonSearchPerson, this, person_id, "people");
}
//---------------------------------------------------------------------------
void TformSpecialNewTransportationTasks::RefreshPersonCombo(string id_to_select)
{
 person_id_to_name.clear();
 comboPerson->Items->Clear();
 comboPerson->Text = "";

 // this makes most sense if sorted by mileage. wouldn't want a drop-off to somehow come before a pick-up and confuse things.
 vector<map<string, string> > sorted_data;
 GetStopsSortedByMileage(sorted_data);

 // when editing or adding a row, allow only the value of the row, and values of people who have been picked up but not dropped off ... yay.
 int j = 0;
 string a, z;
 set<string> added;
 for (vector<map<string, string> >::iterator i = sorted_data.begin(); i != sorted_data.end(); i++)
 {
  a = (*i)["action_id"];
  z = (*i)["person_id"];
  if (a == "1" && !IsBlankOrZero(z))
   added.insert(z);
  else if (a == "2")
   added.erase(z);
 }

 // for the purpose of editing a row, once saved -- we want the value of the row (if it's a drop-off) to still be available
 map<string, string> row_data;
 if (list_stops.get_current_row(row_data))
  if (!IsBlankOrZero(row_data["person_id"]))
   added.insert(row_data["person_id"]);

 // now actually put it back in the list
 for (set<string>::iterator i = added.begin(); i != added.end(); i++, j++)
 {
  z = *i;
  if (!IsBlankOrZero(z))
  {
   comboPerson->Items->Add(DefaultText(z, "people").c_str());
   person_id_to_name.push_back(z);
   if (z == id_to_select)
    comboPerson->ItemIndex = j;
  }
 }
}
//---------------------------------------------------------------------------
string TformSpecialNewTransportationTasks::InterpretPersonCombo()
{
 if (comboPerson->ItemIndex > -1 && comboPerson->ItemIndex < person_id_to_name.size())
  return person_id_to_name[comboPerson->ItemIndex];
 return "";
}
//---------------------------------------------------------------------------
void TformSpecialNewTransportationTasks::Do ()
{
 FORM_TRY
  TformSpecialNewTransportationTasks * z = new TformSpecialNewTransportationTasks(NULL);
  z->Show();
 FORM_CATCH
}
//---------------------------------------------------------------------------
void __fastcall TformSpecialNewTransportationTasks::buttonSaveClick(
      TObject *Sender)
{
 // warn if no leave-office (important: reduces number of miles!)
 // warn if no return-to-office (less important: no admin time.)
 // warn if fewer than 2 items, as there won't be a difference to calculate ... ?
 // warn for any other weirdness. right. like that's easy.
 // warn if people were picked up but not dropped off? or any mismatch? (dropped multiple times?) dropped, then picked up? ew, what if they were picked up without mileage, and dropped off with ... how does that count? out of order? code-wise, yes, but ... argh. wound up disabling null-mileage to get around this. ha!

 vector<map<string, string> > sorted_data;
 GetStopsSortedByMileage(sorted_data);

 // error
 if (comboUser->Text == "" || !lookupTables->IsInLookUpTable("users", comboUser->Text.c_str()))
 {
  PSay("Please select a driver.", m_invalid_data, MB_OK);
  return;
 }

 // error
 if (sorted_data.empty())
 {
  PSay("There are no tasks to save.", m_missing_data, MB_OK);
  return;
 }

 // warning
 if (sorted_data.size() == 1)
 {
  if (IDNO == PSay("With only one step in the route, no mileage-difference can be calculated;\nyour task will not show any distance traveled, nor any time spent.\nAre you sure you want to save anyway?", m_confirmation, MB_YESNO))
   return;
 }

 // warning
 // I like curly braces.
 if (sorted_data[0]["action_id"] != "0")
 {
  if (IDNO == PSay("Your first step was not 'Admin', which means the second step won't get properly credited with mileage.\nIt will appear you started in the middle of your route.\nAre you sure you want to save anyway?", m_confirmation, MB_YESNO))
   return;
 } else
 // warning
 {
  if (sorted_data[0]["note"] != "")
   if (IDNO == PSay("You have a note associated with your very first ('Admin') stop;\nthis note will not be saved, as tasks are saved for the 'difference' between stops.\nNotes on later 'Admin' tasks (such as oil, gas, tires, wash, etc.) will be saved.\nAre you sure you want to save anyway?", m_confirmation, MB_YESNO))
    return;
 }

 // warning (less important, only concerns admin mileage)
 if (sorted_data[sorted_data.size() - 1]["action_id"] != "0")
 {
  if (IDNO == PSay("Your last step was not 'Admin', which means the last step won't get properly credited with administrative mileage.\nIt will appear you stopped in the middle of your route.\nAre you sure you want to save anyway?", m_confirmation, MB_YESNO))
   return;
 }

 // warnings
 // TODO: will warn them (possibly) if they drop off and pick up the same person in a single mile, but hadn't previously picked them up (int is not float)
 // this is hard to do, but is possible:
 // - pick up a person at mile 15
 // - drop off a person at mile 17
 // - delete the row for the pick-up
 // - pick up that person at mile 17
 // stable-sort (or, worse, unstable-sort) will/may wind up placing the pick-up after the drop-off (particularly because of the row insertion) ... leading to a warning.
 string a, z, m;
 set<string> currently_picked_up;
 for (vector<map<string, string> >::iterator i = sorted_data.begin(); i != sorted_data.end(); i++)
 {
  a = (*i)["action_id"];
  z = (*i)["person_id"];
  if (a == "1")
  {
   if (currently_picked_up.find(z) != currently_picked_up.end())
   {
    if (IDNO == PSay("Your log indicates that " + (*i)["person"] + " was already in the vehicle when you picked him/her up again at mileage " + (*i)["mileage"] + ".\nAre you sure you want to save anyway?", m_confirmation, MB_OK))
     return;
   }
   currently_picked_up.insert(z);
  } else
  if (a == "2")
  {
   if (currently_picked_up.find(z) == currently_picked_up.end())
   {
    if (IDNO == PSay("Your log indicates that " + (*i)["person"] + " was not in the vehicle when you dropped him/her off at mileage " + (*i)["mileage"] + ".\nAre you sure you want to save anyway?", m_confirmation, MB_OK))
     return;
   }
   currently_picked_up.erase(z);
  }
 }
 if (!currently_picked_up.empty())
 {
  string person_list;
  for (set<string>::iterator i = currently_picked_up.begin(); i != currently_picked_up.end(); i++)
   person_list += DefaultText(*i, "people") + "\n";
  if (IDNO == PSay("At the end of your trip, you still had the following people in your vehicle:\n" + person_list + "Are you sure you want to save anyway?", m_invalid_data, MB_YESNO))
   return;
 }

 // warnings (should be errors, maybe?)
 vector<map<string, string> > sorted_data_by_time;
 GetStopsSortedByTime(sorted_data_by_time);
 int previous_mileage = -1;
 int this_mileage = -1;
 for (vector<map<string, string> >::iterator i = sorted_data_by_time.begin(); i != sorted_data_by_time.end(); i++)
 {
  this_mileage = StringToInt((*i)["mileage"]);
  if (previous_mileage > this_mileage)
  {
   if (IDNO == PSay("Your odometer and clock readings are not in the same order (time travel.)\nYou reached odometer reading " + IntToString(previous_mileage) + " before reaching " + IntToString(this_mileage) + ", according to your time entries.\nAre you sure you want to save anyway?", m_invalid_data, MB_YESNO))
    return;
  }
  previous_mileage = this_mileage;
 }

 bool should_clear = false;

 TIBTransaction * dbTransaction = NULL;
 try
 {
  FORM_TRY
   dbTransaction = new TIBTransaction(NULL);
  FORM_CATCH
  StdTransaction(dbTransaction, dbDatabase);

  AtlasClear();

  map<string, string> default_row;
  default_row["current_id"]        = "4";
  default_row["last_update_stamp"] = RightNow();
  default_row["last_updated_by"]   = current_userid;

  string pk_visit;
  map<string, string> insert_row;
  map<string, string> default_task;
  if (!CreateInsertMap(dbTransaction, "tasks", default_task))
   throw(007);
  MergeMap(default_task, default_row);

  default_task["group_id"]               = lookupTables->IdOf("groups", "Transportation");
  default_task["date_of_task"]           = Pdate(dateLog->Date).OutDate(global_mode);
  default_task["user_id"]                = lookupTables->IdOf("users", comboUser->Text.c_str());
  map<string, int> leave_reason_alone;
  leave_reason_alone["task_comments"] = sql_no_uppercasing;

  vector<map<string, string> > sorted_data;
  GetStopsSortedByMileage(sorted_data);

  vector<pick_up_info> pick_ups;

  // one task at a time
  Ptime prev_time, curr_time;
  int prev_odometer, curr_odometer, diff_odometer;
  financial diff_time;
  string a, p;
  bool has_done_transp = false;
  bool has_done_deliv = false;
  for (vector<map<string, string> >::iterator i = sorted_data.begin(); i != sorted_data.end(); i++)
  {
   curr_time = Ptime((*i)["time"], global_mode);
   curr_odometer = StringToInt((*i)["mileage"]);

   diff_time = 0;
   diff_odometer = 0;

   if (i != sorted_data.begin())
   {
    diff_time = (curr_time - prev_time) / 3600.0; // note: time - time is by default in seconds
    diff_odometer = curr_odometer - prev_odometer;
   }

   // we overwrite curr_time later, so we'll remember it here just in case, rather than at the end of the loop (sorry about the confusion)
   prev_time = curr_time;
   prev_odometer = curr_odometer;

   // i hate repeating myself!
   // i hate repeating myself!
   a = (*i)["action_id"];
   p = (*i)["person_id"];

   // save a pick-up for later
   if (a == "1")
    pick_ups.push_back(pick_up_info(p, diff_odometer, diff_time, curr_time));

   // find the latest pick_up_info for our person, and ...
   if (a == "2")
    for (vector<pick_up_info>::reverse_iterator b = pick_ups.rbegin(); b != pick_ups.rend(); b++)
     if (b->person_id == p)
     {
      // use their time, rather than ours (would match appointments better, you know?)
      curr_time = b->pick_up_time;
      // add previous diff_time to our own
      diff_time += b->diff_time;
      // add previous diff_odometer to our own
      diff_odometer += b->diff_odometer;
      break;
     }

   if (a != "1") // not a pick-up, as those don't generate tasks by themselves
   {
    // skip first admin entry, but if they didn't start with admin (woe to them), look at it anyway
    if (a != "0" || i != sorted_data.begin())
    {
     insert_row = default_task;
     insert_row["id"]            = GetNextId(dbTransaction, "tasks");
     insert_row["person_id"]     = (*i)["person_id"];
     insert_row["time_of_task"]  = curr_time.OutTime(global_mode);
     insert_row["time_spent"]    = FinancialToString(diff_time);
     insert_row["mileage"]       = IntToString(diff_odometer);
     insert_row["task_comments"] = (*i)["note"];

     if (a == "0")
     {
      vector<map<string, string> >::iterator test_for_end = i;
      test_for_end++;
      if (test_for_end == sorted_data.end())
      {
       // they want the last admin task to count as whatever they did, in the following order:
       if (has_done_transp)
        insert_row["transp_action_id"] = lookupTables->IdOf("transp_action_types", "Transportation");
       else if (has_done_deliv)
        insert_row["transp_action_id"] = lookupTables->IdOf("transp_action_types", "Delivery");
       else
        insert_row["transp_action_id"] = lookupTables->IdOf("transp_action_types", "Administrative");
      } else
       insert_row["transp_action_id"] = lookupTables->IdOf("transp_action_types", "Administrative");
     } else
     if (is_in(a, "2", "4", NULL))
     {
      insert_row["transp_action_id"] = lookupTables->IdOf("transp_action_types", "Transportation");
      has_done_transp = true;
     } else
     if (is_in(a, "3", "5", NULL))
     {
      insert_row["transp_action_id"] = lookupTables->IdOf("transp_action_types", "Delivery"); // not specifying 'medication' because, well, ... that seems too narrow.
      has_done_deliv = true;
     } else
      insert_row["transp_action_id"] = "0"; // meh.

     if (!ExecuteInsertStatement(dbTransaction, "tasks", insert_row, leave_reason_alone))
      throw(006);
     AtlasAdd("tasks", insert_row);

     if (!IsBlankOrZero(insert_row["person_id"])) // extra safety
     {
      string sql = "select appointments.id from appointments where "
                   "appointments.task_id = '0'  and "
                   "appointments.begin_date = '" + default_task["date_of_task"] + "' and "
                   "appointments.created_for in ("
                    "select users.id from users where users.group_id = ("
                     "select first 1 users.group_id from users where users.id = '" + default_task["user_id"] + "')) and "
                   "appointments.person_id = " + insert_row["person_id"];
      vector<string> appointments_to_update;
      if (!SqlToVec(dbTransaction, sql, appointments_to_update))
       throw(005);
      if (!appointments_to_update.empty())
      {
       if (!ExecuteStatement(dbTransaction, "update appointments set task_id = " + insert_row["id"] + " where id in (" + VecToCommaString(appointments_to_update) + ")"))
        throw(004);
       for (vector<string>::iterator j = appointments_to_update.begin(); j != appointments_to_update.end(); j++)
        AtlasAdd("appointments", "id", *j);
      }
     }
    }
   }
  }

  dbTransaction->Commit();
  AtlasSend();
  should_clear = true;
 } catch(...)
 {
  if (dbTransaction->Active)
   dbTransaction->Rollback();
  PSay("An error was encountered; your data was not saved.", m_database_error, MB_OK);
 }
 delete dbTransaction;

 // clear!
 if (should_clear)
  ClearScreen();
}
//---------------------------------------------------------------------------
void __fastcall TformSpecialNewTransportationTasks::buttonCloseClick(
      TObject *Sender)
{
 Close();
}
//---------------------------------------------------------------------------
void __fastcall TformSpecialNewTransportationTasks::comboPersonKeyPress(
      TObject *Sender, char &Key)
{
 FindComboBoxItemAndSetItemIndex(Sender, Key);
}
//---------------------------------------------------------------------------
// argh -- we get no respect. and no anonymous functions. i've gotten too used to them in other languages. (they ARE neat!) AND no partial functions -- passing Ptime(, global_mode) below, and having it understand that it should always use 'global_mode' as the second parameter?
// which reminds me -- we really should make global_mode the default. period.
Ptime StringToPtime(const string &a)
{
 return Ptime(a, global_mode);
}
//---------------------------------------------------------------------------
void TformSpecialNewTransportationTasks::GetStopsSortedByMileage(vector<map<string, string> > & sorted_data)
{
 sorted_data.clear();
 list_stops.export_data(sorted_data);
 vector<pair<string, cb_sorting_object<string> *> > sorting_methods;
 sorting_methods.push_back(pair<string, cb_sorting_object<string> *>("mileage", new sort_callback_wrapper<string, int, sort_less_than<int> >(StringToInt))); // pretty, neh? i think it's cute.
 sorting_methods.push_back(pair<string, cb_sorting_object<string> *>("time", new sort_callback_wrapper<string, Ptime, sort_less_than<Ptime> >(StringToPtime))); // pretty, neh? i think it's cute.
 multi_column_sorting_object<string, string> sorting_object (sorting_methods);
 sort(sorted_data.begin(), sorted_data.end(), sorting_object);
 cb_deallocate_sorting_map(sorting_methods);
}
//---------------------------------------------------------------------------
void TformSpecialNewTransportationTasks::GetStopsSortedByTime(vector<map<string, string> > & sorted_data)
{
 sorted_data.clear();
 list_stops.export_data(sorted_data);
 vector<pair<string, cb_sorting_object<string> *> > sorting_methods;
 sorting_methods.push_back(pair<string, cb_sorting_object<string> *>("time", new sort_callback_wrapper<string, Ptime, sort_less_than<Ptime> >(StringToPtime))); // pretty, neh? i think it's cute.
 sorting_methods.push_back(pair<string, cb_sorting_object<string> *>("mileage", new sort_callback_wrapper<string, int, sort_less_than<int> >(StringToInt))); // pretty, neh? i think it's cute.
 multi_column_sorting_object<string, string> sorting_object (sorting_methods);
 sort(sorted_data.begin(), sorted_data.end(), sorting_object);
 cb_deallocate_sorting_map(sorting_methods);
}
//---------------------------------------------------------------------------
void TformSpecialNewTransportationTasks::ClearScreen()
{
 comboUser->ItemIndex = -1;
 dateLog->Date = Date();

 list_stops.clear_data();
 editMileage->Text = "";
 timeStop->DateTime = Ptime("08:00", global_mode).OutTime();
 person_id = "0";
 person_id_to_name.clear();
 comboPerson->Items->Clear();
 comboPerson->ItemIndex = -1;
 comboAction->ItemIndex = 0;
 comboActionChange(comboAction);
 editNote->Text = "";
}
//---------------------------------------------------------------------------
