//---------------------------------------------------------------------------

#include <vcl.h>
#pragma hdrstop

#include "codeSpecialObsExpectedRanges.h"
#include "codeRecordObservationQuestion.h"
//---------------------------------------------------------------------------
#pragma package(smart_init)
#pragma link "codeSpecialGeneric"
#pragma resource "*.dfm"
TformSpecialObsExpectedRanges *formSpecialObsExpectedRanges;
//---------------------------------------------------------------------------
__fastcall TformSpecialObsExpectedRanges::TformSpecialObsExpectedRanges(TComponent* Owner)
        : TformSpecialGeneric(Owner)
{
 IconIndex = 54;

 ranges_filter.SetListViewHandle(listRanges);
 ranges_filter.SetSecurity("obs_expected_ranges");
 ranges_filter.AddColumnSpec("Internal Id", "internal_id", as_int, false, 60);
 ranges_filter.AddColumnSpec("Current Id", "current_id", as_int, false, 60);

 // not precisely the permissions expected, but obs_questions doesn't have its own permission set at the moment (weird?)
 ranges_filter.AddColumnSpec("Question", "obs_questions", as_text, true, 120, rawString);
 ranges_filter.AddColumnSpec("Question Abbr.", "obs_questions", as_text, false, 75, rawString);
 ranges_filter.AddColumnSpec("CPT", "obs_questions", as_text, true, 75, rawString);
 ranges_filter.AddColumnSpec("Lower", "obs_expected_ranges", as_text, true, 75, rawString);
 ranges_filter.AddColumnSpec("Upper", "obs_expected_ranges", as_text, true, 75, rawString);

 ranges_filter.FetchPreferences("specialobsexpectedranges.ranges_filter");
 ranges_filter.SetIconStatus(true);
 ranges_filter.SetSQLStatement("");
 ranges_filter.RefreshDataSet();

 lookupTables->RegisterComboBox("obs_question_set_types", comboObsQuestionSet);
 lookupTables->RegisterComboBox("gender_types", comboGender);

 ranges_request = atlas->RequestNotificationWhen("obs_expected_ranges.id", "", rel_any, as_int, this, RefreshHandler(&(TformSpecialObsExpectedRanges::AtlasRefreshListView)));
 ranges_filter.SetAddEdit(this, AddRecord(&(TformSpecialObsExpectedRanges::AddQuestion)), EditRecord(&(TformSpecialObsExpectedRanges::EditQuestion)));
}
//---------------------------------------------------------------------------
void TformSpecialObsExpectedRanges::EditQuestion()
{
 map<string,string> temp_map;
 temp_map["id"] = ranges_filter.GetSelectedRecordID();
 TformRecordObservationQuestion::Do(temp_map);
}
//---------------------------------------------------------------------------
void TformSpecialObsExpectedRanges::AddQuestion()
{
 map<string,string> temp_map;
 temp_map["id"] = "new";
 TformRecordObservationQuestion::Do(temp_map);
}
//---------------------------------------------------------------------------
void TformSpecialObsExpectedRanges::AtlasRefreshListView(int request_num, string matched_value)
{
 if (request_num == ranges_request)
  ranges_filter.RefreshDataSet();
}
//---------------------------------------------------------------------------
void __fastcall TformSpecialObsExpectedRanges::buttonCloseClick(
      TObject *Sender)
{
 Close();
}
//---------------------------------------------------------------------------
void __fastcall TformSpecialObsExpectedRanges::comboObsQuestionSetChange(
      TObject *Sender)
{
 // refresh the list!
 if (!lookupTables->IsInLookUpTable("obs_question_set_types", comboObsQuestionSet))
  return;

 string gender_id           = lookupTables->IdOf("gender_types", comboGender);
 string obs_question_set_id = lookupTables->IdOf("obs_question_set_types", comboObsQuestionSet);

 string sql = "select obs_questions.id obs_question_id, obs_questions.current_id, "
              "obs_questions.item, obs_questions.short_item, "
              "obs_questions.cpt_code, "
              "coalesce(cast(cast(obs_expected_ranges.min_response_metric as numeric (15,2)) as varchar(250)) || ' ' || units.item, '') || coalesce(min_option.item, ''), "
              "coalesce(cast(cast(obs_expected_ranges.max_response_metric as numeric (15,2)) as varchar(250)) || ' ' || units.item, '') || coalesce(max_option.item, '')  "
              "from "
              "obs_questions_in_sets "
              "inner join obs_questions on "
              "obs_questions_in_sets.obs_question_id = obs_questions.id "
              "and obs_questions_in_sets.obs_question_set_id = " + obs_question_set_id + " "
              "and obs_questions.obs_question_id in (3, 4) "
              "left join obs_expected_ranges on "
              "obs_expected_ranges.obs_question_id = obs_questions.id and ";
 if (IsBlankOrZero(gender_id))
       sql += "(obs_expected_ranges.gender_id = 0) "; // won't know what to show in these cases, don't want to show "all"
 else
       sql += "(obs_expected_ranges.gender_id = " + gender_id + " or obs_expected_ranges.gender_id = 0) ";
       sql += "left join units on units.id = obs_questions.unit_id "
              "left join obs_option_types min_option on min_option.id = obs_expected_ranges.min_obs_option_id "
              "left join obs_option_types max_option on max_option.id = obs_expected_ranges.max_obs_option_id "
              "order by obs_questions_in_sets.sequence asc";

 ranges_filter.SetSQLStatement(sql);
 ranges_filter.RefreshDataSet();
}
//---------------------------------------------------------------------------
void TformSpecialObsExpectedRanges::Do(string gender_id, string obs_question_set_id)
{
 FORM_TRY
  TformSpecialObsExpectedRanges * z = new TformSpecialObsExpectedRanges(NULL);
  if (!IsBlankOrZero(gender_id))
   z->comboGender->Text = lookupTables->ValueOf("gender_types", gender_id).c_str();
  if (!IsBlankOrZero(obs_question_set_id))
   z->comboObsQuestionSet->Text = lookupTables->ValueOf("obs_question_set_types", obs_question_set_id).c_str();
  z->comboObsQuestionSetChange(NULL); // the above don't count as 'changed' events?
  z->Show();
 FORM_CATCH
}
//---------------------------------------------------------------------------
void __fastcall TformSpecialObsExpectedRanges::FormClose(TObject *Sender,
      TCloseAction &Action)
{
 TformSpecialGeneric::FormClose(Sender, Action);
 if (Action == caFree)
 {
  lookupTables->UnregisterComboBox("obs_question_set_types", comboObsQuestionSet);
  lookupTables->UnregisterComboBox("gender_types", comboGender);
 }
}
//---------------------------------------------------------------------------
