//---------------------------------------------------------------------------

#include <vcl.h>
#pragma hdrstop

#include "codeSpecialObservationQuestionSetEditor.h"
#include "codeRecordObservationQuestion.h"
//---------------------------------------------------------------------------
#pragma package(smart_init)
#pragma link "codeHalfButton"
#pragma link "codeSpecialGeneric"
#pragma resource "*.dfm"
TformSpecialObservationQuestionSetEditor *formSpecialObservationQuestionSetEditor;
//---------------------------------------------------------------------------
__fastcall TformSpecialObservationQuestionSetEditor::TformSpecialObservationQuestionSetEditor(TComponent* Owner)
        : TformSpecialGeneric(Owner)
{
 IconIndex = 54;

 list_all_questions.SetListViewHandle(listQuestions);
 list_all_questions.SetSecurity("obs_questions");
 list_all_questions.AddColumnSpec("Internal Id", "internal_id", as_int, false, 80);
 list_all_questions.AddColumnSpec("Current Id", "current_id", as_int, false, 80);
 list_all_questions.AddColumnSpec("Valid", "people", as_text, false, 40, prettyBoolString);
 list_all_questions.AddColumnSpec("Name", "people", as_text, true, 180, rawString);
 list_all_questions.AddColumnSpec("Abbrev.", "affiliations", as_text, false, 70, rawString);
 list_all_questions.AddColumnSpec("CPT", "affiliations", as_text, false, 80, rawString);
 list_all_questions.AddColumnSpec("Type", "tasks", as_text, false, 60, prettyString);
 list_all_questions.AddColumnSpec("Options", "tasks", as_text, false, 90, prettyString);
 list_all_questions.SetMulti(true);
 list_all_questions.FetchPreferences("specialobservationquestionseteditor.question_filter");
 list_all_questions.SetAddEdit(this, AddRecord(&(TformSpecialObservationQuestionSetEditor::AddQuestion)), EditRecord(&(TformSpecialObservationQuestionSetEditor::EditQuestion)));
 request_all_questions = atlas->RequestNotificationWhen("obs_questions.id", "", rel_any, as_int, this, RefreshHandler(&(TformSpecialObservationQuestionSetEditor::AtlasRefreshListView)));

 list_some_questions.setup_listview(listQuestionSetQuestions, /* multiselect */ true);
 list_some_questions.add_or_modify_column("id", "Id",              false,  0,  60, as_int,  rawString);
 list_some_questions.add_or_modify_column("sequence", "Sequence",   true,  1,  60, as_int,  rawString);
 list_some_questions.add_or_modify_column("item", "Name",           true,  2, 120, as_text, rawString);
 // do we need to show more columns?
 // do we need to select 'full' items when something's selected here, to make up for the lesser amount of information?
 list_some_questions.setup_callbacks((TForm*)this, InformForm(&(TformSpecialObservationQuestionSetEditor::QuestionSelected)), InformForm(&(TformSpecialObservationQuestionSetEditor::QuestionZoomed)));

 lookupTables->RegisterComboBox("obs_question_set_types", comboQuestionSet);

 // all sorts of priming pumps (first time stuff)
 comboQuestionSetChange(comboQuestionSet);
 QuestionSelected();
 RefreshAllQuestions();
}
//---------------------------------------------------------------------------
__fastcall TformSpecialObservationQuestionSetEditor::~TformSpecialObservationQuestionSetEditor()
{
 lookupTables->UnregisterComboBox(comboQuestionSet);
}
//---------------------------------------------------------------------------
void __fastcall TformSpecialObservationQuestionSetEditor::buttonMoveUpClick(TObject *Sender)
{
 list_some_questions.decrease_rank("sequence");
}
//---------------------------------------------------------------------------
void __fastcall TformSpecialObservationQuestionSetEditor::buttonMoveDownClick(TObject *Sender)
{
 list_some_questions.increase_rank("sequence");
}
//---------------------------------------------------------------------------
void __fastcall TformSpecialObservationQuestionSetEditor::buttonAddQuestionClick(TObject *Sender)
{
 // add
 vector<map<string, string> > all_the_data = list_all_questions.GetAllData(true);
 vector<string> selected_data_ids = list_all_questions.GetMultiID();
 set<string> set_of_ids;
 VecToSet(selected_data_ids, set_of_ids);
 for (vector<map<string, string> >::iterator i = all_the_data.begin(); i != all_the_data.end(); i++)
  if (set_of_ids.find((*i)["id"]) != set_of_ids.end())
  {
   map<string, string> temp;
   temp["id"] = (*i)["id"];
   temp["item"] = (*i)["item"];
   temp["sequence"] = IntToString(listQuestionSetQuestions->Items->Count); // place at end?
   listQuestionSetQuestions->Selected = NULL;
   list_some_questions.add_or_modify_row_data(temp, false);
  }
 // TODO: do we test for dupes? we ought to, even though the following will make that less likely ... (but only less!)
 RefreshAllQuestions();
}
//---------------------------------------------------------------------------
void __fastcall TformSpecialObservationQuestionSetEditor::buttonRemoveQuestionClick(TObject *Sender)
{
 list_some_questions.remove_current_row(true);
 list_some_questions.normalize_rank("sequence");
 RefreshAllQuestions();
}
//---------------------------------------------------------------------------
void __fastcall TformSpecialObservationQuestionSetEditor::buttonCloseClick(TObject *Sender)
{
 Close();
}
//---------------------------------------------------------------------------
void __fastcall TformSpecialObservationQuestionSetEditor::buttonSaveClick(TObject *Sender)
{
 // save

 /*
  if we have an question set, and the list is non-empty:
  1) delete everything from the linking table, for that question set
  2) reinsert everything into the linkint table, for that question set, from the list.
  3) clear the lists, reset the screen, etc.
 */

 string obs_question_set_id = lookupTables->IdOf("obs_question_set_types", comboQuestionSet);
 if (!IsBlankOrZero(obs_question_set_id))
 {
  if (listQuestionSetQuestions->Items->Count == 0)
   if (IDNO == PSay("No questions are associated with this question set.\nAre you sure you want to continue?", m_missing_data, MB_YESNO))
    return;

  TIBTransaction * dbTransaction = NULL;
  FORM_TRY
   dbTransaction = new TIBTransaction(NULL);
  FORM_CATCH

  try
  {
   StdTransaction(dbTransaction, dbDatabase);

   string sql;

   if (!LockRecordForUpdate(dbTransaction, "obs_question_set_types", obs_question_set_id))
   {
    PSay("Failed to get exclusive access to the question-set: it is already being modified elsewhere.", m_database_error, MB_OK);
    throw(0);
   }

   string up = UppercaseString(editPermission->Text.c_str());
   if (LikeString("TABLE.%", up))
   {
    string a = GetOneField(dbTransaction, "select count(*) from possible_permissions where item = '" + SafeSQLString(up) + "'");
    if (IsBlankOrZero(a))
    {
     if (IDNO == PSay("This permission does not exist yet.\nWould you like to create it now?", m_confirmation, MB_YESNO))
      throw(5);
     else
     {
      map<string, string> temp_row;
      // temp_row["id"] is auto-assigned by trigger
      temp_row["item"] = up;
      temp_row["valid"] = "1";
      if (!ExecuteInsertStatement(dbTransaction, "possible_permissions", temp_row))
      {
       PSay("Failed to create the permission.", m_database_error, MB_OK);
       throw(6);
      }
     }
    }
   } else
   {
    PSay("You must use a table.% permission for this to work correctly.", m_warning, MB_OK);
    throw(7);
   }

   map<string, string> update_map;
   update_map["permission_name"] = up;
   update_map["id"] = obs_question_set_id;
   if (!ExecuteUpdateStatement(dbTransaction, "obs_question_set_types", update_map))
   {
    PSay("Failed to update the permission name on the question set.", m_database_error, MB_OK);
    throw(8);
   }

   sql = "delete from obs_questions_in_sets where obs_question_set_id = " + obs_question_set_id;
   if (!ExecuteStatement(dbTransaction, sql))
   {
    PSay("Failed to delete current question-to-set associations.", m_database_error, MB_OK);
    throw(1);
   }

   vector<map<string, string> > all_data;
   list_some_questions.export_data(all_data);

   map<string, string> insert_row;
   insert_row["obs_question_set_id"] = obs_question_set_id;
   for (vector<map<string, string> >::iterator i = all_data.begin(); i != all_data.end(); i++)
   {
    insert_row["id"] = GetNextId(dbTransaction, "obs_questions_in_sets");
    insert_row["obs_question_id"] = (*i)["id"];
    insert_row["sequence"] = (*i)["sequence"];
    if (!ExecuteInsertStatement(dbTransaction, "obs_questions_in_sets", insert_row))
    {
     PSay("Failed to set a question-to-set association.", m_database_error, MB_OK);
     throw(2);
    }
   }

   dbTransaction->Commit();

   comboQuestionSetChange(comboQuestionSet);
   QuestionSelected();
  } catch (...)
  {
   if (dbTransaction->Active)
    dbTransaction->Rollback();
  }
  delete dbTransaction;

 } else
  PSay("No question set selected.", m_missing_data, MB_OK);
}
//---------------------------------------------------------------------------
void TformSpecialObservationQuestionSetEditor::Do ()
{
 FORM_TRY
  TformSpecialObservationQuestionSetEditor * z = new TformSpecialObservationQuestionSetEditor(NULL);
  z->Show();
 FORM_CATCH
}
//---------------------------------------------------------------------------
void TformSpecialObservationQuestionSetEditor::Do (string obs_question_set_id)
{
 FORM_TRY
  TformSpecialObservationQuestionSetEditor * z = new TformSpecialObservationQuestionSetEditor(NULL);
  z->comboQuestionSet->Text = lookupTables->ValueOf("obs_question_set_types", obs_question_set_id).c_str();
  z->comboQuestionSetChange(z->comboQuestionSet);
  z->Show();
 FORM_CATCH
}
//---------------------------------------------------------------------------
void __fastcall TformSpecialObservationQuestionSetEditor::comboQuestionSetChange(
      TObject *Sender)
{
 string obs_question_set_id = lookupTables->IdOf("obs_question_set_types", comboQuestionSet);
 bool set_selected = true;
 if (IsBlankOrZero(obs_question_set_id))
 {
  set_selected = false;
  obs_question_set_id = "0";
 }
 buttonAddQuestion->Enabled = set_selected;
 buttonSave->Enabled = set_selected;

 editPermission->Text = LowercaseString(GetOneField(dbDatabase, "select permission_name from obs_question_set_types where id = " + obs_question_set_id)).c_str();

 vector<map<string, string> > questions_in_set;
 string sql = "select obs_questions.id, obs_questions_in_sets.sequence, obs_questions.item "
              "from obs_questions_in_sets "
              "left join obs_questions on obs_questions.id = obs_questions_in_sets.obs_question_id "
              "where obs_questions.valid = 1 and obs_questions_in_sets.obs_question_set_id = " + obs_question_set_id;
 list_some_questions.clear_data();
 if (!SqlToVecMap(dbDatabase, sql, questions_in_set))
  PSay("Failed to retrieve the list of questions in this set.", m_database_error, MB_OK);
 else
  list_some_questions.import_data(questions_in_set);
 RefreshAllQuestions();
}
//---------------------------------------------------------------------------
void TformSpecialObservationQuestionSetEditor::AtlasRefreshListView(int request_num, string matched_value)
{
 if (request_num == request_all_questions)
  list_all_questions.RefreshDataSet();
 // TODO: plus, do an 'intersection' between the id's in the left set and in the right, to prune any now-unavailables?
 // what about fixing up problems with names that have changed? users might be confused, if they change a question's
 // name/wording on the right, and the change isn't reflected on the left.
 // -> no good way (in edit_listview) to modify rows -without changing the ->Selected property to go to the right row
 // -> don't want to empty/fill the list, now that i've gone to the trouble *not* to do that for up/down buttons
 // -> why do i have a feeling this means, either:
 // a) a "update ... set ... where ..." function, or
 // b) the ability to re-import data on top of the current data, replacing rows precisely?
 // neither is particularly easy to code.
}
//---------------------------------------------------------------------------
void TformSpecialObservationQuestionSetEditor::AddQuestion()
{
  map<string,string> temp_map;
  temp_map["id"] = "new";
  TformRecordObservationQuestion::Do(temp_map);
}
//---------------------------------------------------------------------------
void TformSpecialObservationQuestionSetEditor::EditQuestion()
{
  map<string,string> temp_map;
  temp_map["id"] = list_all_questions.GetSelectedRecordID();
  TformRecordObservationQuestion::Do(temp_map);
}
//---------------------------------------------------------------------------
void TformSpecialObservationQuestionSetEditor::QuestionSelected ()
{
 bool some_selected = (listQuestionSetQuestions->Selected != NULL);
 buttonRemoveQuestion->Enabled = some_selected;
 buttonMoveUp->Enabled         = some_selected;
 buttonMoveDown->Enabled       = some_selected;
}
//---------------------------------------------------------------------------
void TformSpecialObservationQuestionSetEditor::QuestionZoomed ()
{
 bool some_selected = (listQuestionSetQuestions->Selected != NULL);
 map<string, string> temp_map;
 map<string, string> row_data;
 if (list_some_questions.get_current_row(row_data))
 {
  temp_map["id"] = row_data["id"];
  TformRecordObservationQuestion::Do(temp_map);
 }
}
//---------------------------------------------------------------------------
void TformSpecialObservationQuestionSetEditor::RefreshAllQuestions ()
{
 string sql =
   "select obs_questions.id, obs_questions.current_id, obs_questions.valid, "
   "obs_questions.item, obs_questions.short_item, obs_questions.cpt_code, "
   "obs_question_types.item question_type_item, obs_option_set_types.item option_type_item "
   "from obs_questions "
   "left join obs_question_types on obs_question_types.id = obs_questions.obs_question_id "
   "left join obs_option_set_types on obs_option_set_types.id = obs_questions.obs_option_set_id "
   "where obs_questions.valid = 1";
 vector<map<string, string> > temp;
 if (list_some_questions.export_data(temp))
  if (!temp.empty())
  {
   string t;
   for (vector<map<string, string> >::iterator i = temp.begin(); i != temp.end(); i++)
   {
    if (i != temp.begin())
     t += ", ";
    t += (*i)["id"];
   }
   sql += " and obs_questions.id not in (" + t + ")";
  }
 list_all_questions.SetSQLStatement(sql);
 list_all_questions.RefreshDataSet();
}
//---------------------------------------------------------------------------
