//---------------------------------------------------------------------------

#include <vcl.h>
#pragma hdrstop

#include "codeSpecialObservationStatistics.h"
#include "codeRecordObservationQuestion.h"
//---------------------------------------------------------------------------
#pragma package(smart_init)
#pragma link "codeSpecialGeneric"
#pragma resource "*.dfm"
TformSpecialObservationStatistics *formSpecialObservationStatistics;
//---------------------------------------------------------------------------
__fastcall TformSpecialObservationStatistics::TformSpecialObservationStatistics(TComponent* Owner)
        : TformSpecialGeneric(Owner)
{
 IconIndex = 54;

 list_stats.SetListViewHandle(listStats);
 list_stats.SetSecurity("obs_questions");
 list_stats.AddColumnSpec("Current Id", "current_id", as_int, false, 80);
 list_stats.AddColumnSpec("Internal Id", "internal_id", as_int, false, 80);
 list_stats.AddColumnSpec("Question", "obs_questions", as_text, true, 200);
 list_stats.AddColumnSpec("Mean", "", as_double, true, 80, amountString);
 list_stats.AddColumnSpec("StdDev", "", as_double, true, 80, amountString);
 list_stats.AddColumnSpec("Low (-3 )", "", as_double, true, 80, amountString);
 list_stats.AddColumnSpec("High (+3 )", "", as_double, true, 80, amountString);

 list_stats.SetAddEdit(this, AddRecord(&(TformSpecialObservationStatistics::AddQuestion)), EditRecord(&(TformSpecialObservationStatistics::EditQuestion)));

 list_stats.SetSQLStatement("select current_id, id, item, cached_mean, cached_stddev, cached_mean - (3 * cached_stddev), cached_mean + (3 * cached_stddev) from obs_questions where id > 0 and obs_question_id = 4"); // metric
 list_stats.RefreshDataSet();
}
//---------------------------------------------------------------------------
float CalcMedian (TIBTransaction * dbTransaction, string question_id)
{
 if (IsBlankOrZero(question_id))
  throw("Invalid question id");
 string sql = "select count(*) from obs_responses where obs_question_id = " + question_id;
 string c = GetOneField(dbTransaction, sql);
 if (IsBlankOrZero(c))
  throw("Invalid response count, or no responses");
 int response_count = StringToInt(c);
 int responses_to_keep = 0;
 if (response_count % 2) // odd
  responses_to_keep = 1;
 else
  responses_to_keep = 2;
 sql = "select first " + IntToString(responses_to_keep) + " skip " + IntToString((response_count - responses_to_keep) / 2) + " response_metric from obs_responses where obs_question_id = " + question_id + " order by response_metric asc";
 vector<string> u;
 if (!SqlToVec(dbTransaction, sql, u))
  throw("Failure to retrieve data points");
 if (u.empty())
  throw("Empty result set");
 float answer = 0.0;
 if (u.size() == 1)
  answer = StringToFloat(u[0]);
 else
  answer = (StringToFloat(u[0]) + StringToFloat(u[1])) / 2.0;
 return answer;
}

float CalcMean (TIBTransaction * dbTransaction, string question_id)
{
 if (IsBlankOrZero(question_id))
  throw("Invalid question id");
 string sql = "select avg(response_metric) from obs_responses where obs_question_id = " + question_id;
 string a = GetOneField(dbTransaction, sql);
 if (a.empty() || !IsFloat(a))
  throw("Invalid average, or no responses");
 float answer = StringToFloat(a);
 return answer;
}

float CalcSomeMean (TIBTransaction * dbTransaction, string question_id, float percentage)
{
 if (percentage > 1.00 || percentage < 0.00)
  throw("Invalid percentage");
 if (IsBlankOrZero(question_id))
  throw("Invalid question id");
 string sql = "select count(*) from obs_responses where obs_question_id = " + question_id;
 string c = GetOneField(dbTransaction, sql);
 if (IsBlankOrZero(c))
  throw("Invalid response count, or no responses");
 int response_count = StringToInt(c);
 int responses_to_keep = response_count * percentage;
 int margin = (response_count - responses_to_keep) / 2;

 // jhaley: Change the 0 here to 1 if you want to see EditSql windows for observation lists, and a lot of PITA crashes.
 easy_streaming_dataset j (dbTransaction,
   "select first " + IntToString(responses_to_keep) + " skip " + IntToString(margin) + " response_metric "
   "from obs_responses where obs_question_id = " + question_id + " order by response_metric asc", 0);
 vector<string> k;
 float accumulator = 0.0;
 int counter = 0;
 while (j.GetRow(k))
 {
  accumulator += StringToFloat(k[0]);
  counter++;
  j.Next();
 }
 if (!counter)
  throw("Division by zero avoided");
 float answer = accumulator / counter;
 return answer;
}

float CalcSomeMin (TIBTransaction * dbTransaction, string question_id, float percentage)
{
 if (percentage > 1.00 || percentage < 0.00)
  throw("Invalid percentage");
 if (IsBlankOrZero(question_id))
  throw("Invalid question id");
 string sql = "select count(*) from obs_responses where obs_question_id = " + question_id;
 string c = GetOneField(dbTransaction, sql);
 if (IsBlankOrZero(c))
  throw("Invalid response count, or no responses");
 int response_count = StringToInt(c);
 int responses_to_keep = response_count * percentage;
 int margin = (response_count - responses_to_keep) / 2;

 string a = GetOneField(dbTransaction, "select first 1 skip " + IntToString(margin) + " response_metric from obs_responses where obs_question_id = " + question_id + " order by response_metric asc");
 float answer = StringToFloat(a);
 return answer;
}

float CalcSomeMax (TIBTransaction * dbTransaction, string question_id, float percentage)
{
 if (percentage > 1.00 || percentage < 0.00)
  throw("Invalid percentage");
 if (IsBlankOrZero(question_id))
  throw("Invalid question id");
 string sql = "select count(*) from obs_responses where obs_question_id = " + question_id;
 string c = GetOneField(dbTransaction, sql);
 if (IsBlankOrZero(c))
  throw("Invalid response count, or no responses");
 int response_count = StringToInt(c);
 int responses_to_keep = response_count * percentage;
 int margin = (response_count - responses_to_keep) / 2;

 string a = GetOneField(dbTransaction, "select first 1 skip " + IntToString(margin + responses_to_keep - 1) + " response_metric from obs_responses where obs_question_id = " + question_id + " order by response_metric asc");
 float answer = StringToFloat(a);
 return answer;
}

float CalcSomeStdDev (TIBTransaction * dbTransaction, string question_id, float mean_or_median, float percentage)
{
 if (percentage > 1.00 || percentage < 0.00)
  throw("Invalid percentage");
 if (IsBlankOrZero(question_id))
  throw("Invalid question id");
 string sql = "select count(*) from obs_responses where obs_question_id = " + question_id;
 string c = GetOneField(dbTransaction, sql);
 if (IsBlankOrZero(c))
  throw("Invalid response count, or no responses");
 int response_count = StringToInt(c);
 int responses_to_keep = response_count * percentage;
 int margin = (response_count - responses_to_keep) / 2;

 // generally: sqrt( sum((val-avg)^2) / n) ... close enough (n vs. n-1) for our purposes.
 // jhaley: Change the 0 here to 1 if you want to see EditSql windows for observation lists, and a lot of PITA crashes.
 easy_streaming_dataset j (dbTransaction,
   "select first " + IntToString(responses_to_keep) + " skip " + IntToString(margin) + " response_metric "
   "from obs_responses where obs_question_id = " + question_id + " order by response_metric asc", 0);
 vector<string> k;
 float accumulator = 0.0;
 int counter = 0;
 float z;
 while (j.GetRow(k))
 {
  z = (StringToFloat(k[0]) - mean_or_median);
  accumulator += (z * z);
  counter++;
  j.Next();
 }
 if (!counter)
  throw("Division by zero avoided");
 float answer = sqrt(accumulator / counter);
 return answer;
}
//---------------------------------------------------------------------------
void __fastcall TformSpecialObservationStatistics::buttonRegenerateClick(TObject *Sender)
{
 TIBTransaction * dbTransaction = NULL;
 FORM_TRY
  dbTransaction = new TIBTransaction(NULL);
 FORM_CATCH
 StdTransaction(dbTransaction, dbDatabase);
 try
 {
  string sql = "select id, item from obs_questions where obs_question_id = 4"; // metric
  map<string, string> question_id_to_name;
  if (!SqlToValueMap(dbTransaction, sql, question_id_to_name))
   throw("Failed to get questions");

  for (map<string, string>::iterator i = question_id_to_name.begin(); i != question_id_to_name.end(); i++)
  {
   try
   {
    float avg_95;
    float std_95;
    avg_95 = CalcSomeMean(dbTransaction, i->first, 0.95);
    std_95 = CalcSomeStdDev(dbTransaction, i->first, avg_95, 0.95);

    string update_sql = "update obs_questions set cached_mean = " + FloatToString(avg_95) + ", cached_stddev = " + FloatToString(std_95) + " where id = " + i->first;
    if (!ExecuteStatement(dbTransaction, update_sql))
    {
     PSay("Failed to update a question (possible record lock.)", m_database_error, MB_OK);
     throw(1L);
    }
   } catch (char * err)
   {
    // we'll, uh, just ... ignore that. 'cause ... yeah. meh.
    // PSay(err, m_database_error, MB_OK);
    delete err;
   }
  }
  dbTransaction->Commit();
 } catch (char* err)
 {
  if (dbTransaction->Active)
   dbTransaction->Rollback();
  delete err;
  PSay(err, m_database_error, MB_OK);
 } catch (...)
 {
  if (dbTransaction->Active)
   dbTransaction->Rollback();
  PSay("Unknown error", m_database_error, MB_OK);
 }
 delete dbTransaction;
 list_stats.RefreshDataSet();
}
//---------------------------------------------------------------------------
void __fastcall TformSpecialObservationStatistics::buttonCloseClick(TObject *Sender)
{
 Close();
}
//---------------------------------------------------------------------------
void TformSpecialObservationStatistics::Do ()
{
 FORM_TRY
  TformSpecialObservationStatistics * z = new TformSpecialObservationStatistics(NULL);
  z->Show();
 FORM_CATCH
}
//---------------------------------------------------------------------------
void TformSpecialObservationStatistics::EditQuestion()
{
 map<string,string> temp_map;
 temp_map["id"] = list_stats.GetSelectedRecordID();
 TformRecordObservationQuestion::Do(temp_map);
}
//---------------------------------------------------------------------------
void TformSpecialObservationStatistics::AddQuestion()
{
 map<string,string> temp_map;
 temp_map["id"] = "new";
 TformRecordObservationQuestion::Do(temp_map);
}
//---------------------------------------------------------------------------
