//---------------------------------------------------------------------------

#include <vcl.h>
#pragma hdrstop

#include "codeSpecialPermissions.h"
#include "codeSpecialAddTable.h"
#include "codeSpecialAddUser.h"
#include "codeSpecialMakeLike.h"
//---------------------------------------------------------------------------
#pragma package(smart_init)
#pragma link "codeSpecialGeneric"
#pragma resource "*.dfm"
TformSpecialPermissions *formSpecialPermissions;
//---------------------------------------------------------------------------
__fastcall TformSpecialPermissions::TformSpecialPermissions(TComponent* Owner)
        : TformSpecialGeneric(Owner)
{
  this->IconIndex = 44;
  dbTransaction = new TIBTransaction(NULL);
  StdTransaction(dbTransaction, dbDatabase, false);

  string sql = "select users.item, groups.item, users.valid, users.id "
               "from users, groups "
               "where users.group_id = groups.id and "
               "users.id > 0"; // and users.valid = 1"; // not no mo', mamba jamba!
  user_filter.SetListViewHandle(listUsers);
  user_filter.SetSecurity("users");
  user_filter.AddColumnSpec("User", "users", as_text, true, 115);
  user_filter.AddColumnSpec("Group", "users", as_text, true, 115);
  user_filter.AddColumnSpec("Valid", "users", as_text, true, 35, prettyBoolString);
  user_filter.AddColumnSpec("Internal Id", "internal_id", as_int, false, 80);
  user_filter.FetchPreferences("specialpermissions.user_filter");
  user_filter.SetCompress(true);
  user_filter.SetSQLStatement(sql);
  user_filter.RefreshDataSet();
  user_filter.SetAddEdit(this, AddRecord(&(TformSpecialPermissions::AddUser)),
                         EditRecord(&(TformSpecialPermissions::EditUser)));
  user_request = atlas->RequestNotificationWhen("users.id", "0", rel_any, as_int, this,
                 RefreshHandler(&(TformSpecialPermissions::AtlasRefresh)));

  DisplayAllPermissions();
  possible_permissions_request = atlas->RequestNotificationWhen("possible_permissions", "0", rel_any, as_int, this,
                                 RefreshHandler(&(TformSpecialPermissions::AtlasRefresh)));
}
//---------------------------------------------------------------------------
bool TformSpecialPermissions::AtlasRefresh(int request_num, string matched_value)
{
 if (request_num == possible_permissions_request)
 {
   DisplayAllPermissions();
   if (listUsers->SelCount > 0)
   {
     user_id = user_filter.GetSelectedRecordID();
     DisplayUserPermissions(user_id);
   }
 }
 else if(request_num == user_request)
 {
   user_filter.RefreshDataSet();
   DisplayAllPermissions();
 }
 return true;
}
//---------------------------------------------------------------------------
void TformSpecialPermissions::DisplayAllPermissions()
{
  all_permissions_vec = lookupTables->GetValueList("possible_permissions");
  table_permissions_vec.clear();
  other_permissions_vec.clear();
  for(vector<string>::iterator itr = all_permissions_vec.begin(); itr != all_permissions_vec.end(); itr++)
  {
    if((*itr).find("table.") != string::npos)
      table_permissions_vec.push_back(*itr);
    else
      other_permissions_vec.push_back(*itr);
  }

  TListItem * row;
  listTablePermissions->Items->Clear();
  listSinglePermissions->Items->Clear();
  for(vector<string>::iterator itr = table_permissions_vec.begin(); itr != table_permissions_vec.end(); itr++)
  {
    row = listTablePermissions->Items->Add();
    row->Caption = LowercaseString((*itr)).c_str();
    row->SubItems->Add("");
    row->SubItemImages[0] = 0;
    row->SubItems->Add("");
    row->SubItemImages[1] = 0;
    row->SubItems->Add("");
    row->SubItemImages[2] = 0;
    row->ImageIndex = -1;
  }
  for(vector<string>::iterator itr = other_permissions_vec.begin(); itr != other_permissions_vec.end(); itr++)
  {
    row = listSinglePermissions->Items->Add();
    row->Caption = LowercaseString((*itr)).c_str();
    row->SubItems->Add("");
    row->SubItemImages[0] = 0;
    row->ImageIndex = -1;
  }
}
//---------------------------------------------------------------------------
void TformSpecialPermissions::DownloadUserPermissions(string user_id)
{
  string sql;
  permission_map.clear();
  sql = "select s, k, v from permissions where user_id = " + user_id;
  FillSKVTable(dbTransaction, sql, permission_map);

}
//---------------------------------------------------------------------------
void TformSpecialPermissions::DisplayUserPermissions(string user_id)
{
  if(user_id != "")
  {
    TListItem * row;
    // set the images to "unchecked" for both list views
    for(unsigned int i = 0; i < table_permissions_vec.size(); i++)
    {
      row = listTablePermissions->Items->Item[i];
      row->SubItemImages[0] = 0;
      row->SubItemImages[1] = 0;
      row->SubItemImages[2] = 0;
    }
    for(unsigned int i = 0; i < other_permissions_vec.size(); i++)
    {
      row = listSinglePermissions->Items->Item[i];
      row->SubItemImages[0] = 0;
    }

    // set the appropriate images to checked
    for(unsigned int i = 0; i < table_permissions_vec.size(); i++)
    {
      row = listTablePermissions->Items->Item[i];
      if(permission_map[table_permissions_vec[i]]["view"] == "true")
        row->SubItemImages[0] = 1;
      if(permission_map[table_permissions_vec[i]]["update"] == "true")
        row->SubItemImages[1] = 1;
      if(permission_map[table_permissions_vec[i]]["insert"] == "true")
        row->SubItemImages[2] = 1;
    }
    for(unsigned int i = 0; i < other_permissions_vec.size(); i++)
    {
      row = listSinglePermissions->Items->Item[i];
      if(permission_map[other_permissions_vec[i]]["enabled"] == "true")
        row->SubItemImages[0] = 1;
    }
  }
}
//---------------------------------------------------------------------------
void __fastcall TformSpecialPermissions::btnCloseClick(TObject *Sender)
{
  Close();
}
//---------------------------------------------------------------------------
void __fastcall TformSpecialPermissions::btnSaveClick(TObject *Sender)
{
  if(listUsers->SelCount > 0)
  {
    Processing(true);

    try
    {
      string sql;
      map<string, string> insert_map;
      TListItem * row;

      dbTransaction->StartTransaction();
      user_id = user_filter.GetSelectedRecordID();
      sql = "delete from permissions where user_id = " + user_id;
      ExecuteStatement(dbTransaction, sql);
      insert_map["user_id"] = user_id;
      for(unsigned int i = 0; i < table_permissions_vec.size(); i++)
      {
        row = listTablePermissions->Items->Item[i];
        insert_map["s"] = table_permissions_vec[i];
        if(row->SubItemImages[0] == 1)
        {
          insert_map["k"] = "view";
          insert_map["v"] = "true";
          ExecuteInsertStatement(dbTransaction, "permissions", insert_map);
        }
        if(row->SubItemImages[1] == 1)
        {
          insert_map["k"] = "update";
          insert_map["v"] = "true";
          ExecuteInsertStatement(dbTransaction, "permissions", insert_map);
        }
        if(row->SubItemImages[2] == 1)
        {
          insert_map["k"] = "insert";
          insert_map["v"] = "true";
          ExecuteInsertStatement(dbTransaction, "permissions", insert_map);
        }
      }
      for(unsigned int i = 0; i < other_permissions_vec.size(); i++)
      {
        row = listSinglePermissions->Items->Item[i];
        insert_map["s"] = other_permissions_vec[i];
        if(row->SubItemImages[0] == 1)
        {
          insert_map["k"] = "enabled";
          insert_map["v"] = "true";
          ExecuteInsertStatement(dbTransaction, "permissions", insert_map);
        }
      }

      dbTransaction->Commit();
      Processing(false);
    }
    catch(...)
    {
      Processing(false);
      if(dbTransaction->Active)
        dbTransaction->Rollback();
      PSay("Error saving permissions", m_database_error, MB_OK);
    }
  }
}
//---------------------------------------------------------------------------
void __fastcall TformSpecialPermissions::FormClose(TObject *Sender, TCloseAction &Action)
{
  delete dbTransaction;
  TformSpecialGeneric::FormClose(Sender, Action);        
}
//---------------------------------------------------------------------------
void __fastcall TformSpecialPermissions::listTablePermissionsMouseMove(TObject *Sender, TShiftState Shift, int X, int Y)
{
  x_pos = X;
}
//---------------------------------------------------------------------------
void __fastcall TformSpecialPermissions::listTablePermissionsClick(TObject *Sender)
{
  // determine x and y position
  if (listTablePermissions->SelCount > 0)
  {
    TListItem * row = listTablePermissions->Selected;

    x_pos -= row->Position.x - 2; // TODO -c MAGIC : no idea where '2' comes from exactly, or when it might change.

    int i;
    int left = 0;
    for (i = 0; i < listTablePermissions->Columns->Count; i++)
    {
     if (x_pos >= left && x_pos < left + min(imageList->Width, listTablePermissions->Columns->Items[i]->Width))
      break;
     left += listTablePermissions->Columns->Items[i]->Width;
    }

    if (i > 0 && (i-1 < row->SubItems->Count))
    {
     row->SubItemImages[i-1] = (row->SubItemImages[i-1] + 1) % 2;

     // now, for all currently selected rows, modify rows such that their column value matches this one exactly
     TItemStates selected = TItemStates() << isSelected;
     for (TListItem * Item = listTablePermissions->Selected; Item != NULL; Item = listTablePermissions->GetNextItem(Item, sdAll, selected))
      Item->SubItemImages[i-1] = row->SubItemImages[i-1];
    }
  }
}
//---------------------------------------------------------------------------
void TformSpecialPermissions::AddUser()
{
  map<string,string> to_pass;
//  vector<string> return_vec;
  to_pass["id"] = "new";
  FORM_TRY
    TformSpecialAddUser * editUserForm = new TformSpecialAddUser(NULL, to_pass, NULL);
    editUserForm->Show();
  FORM_CATCH
}
//---------------------------------------------------------------------------
void TformSpecialPermissions::EditUser()
{
  user_id = user_filter.GetSelectedRecordID();
  map<string,string> to_pass;
  vector<string> return_vec;
  to_pass["id"] = user_id;
  FORM_TRY
    TformSpecialAddUser * editUserForm = new TformSpecialAddUser(NULL, to_pass, &return_vec);
    editUserForm->Show();
  FORM_CATCH
}
//---------------------------------------------------------------------------
void __fastcall TformSpecialPermissions::menuAddTableClick(TObject *Sender)
{
  FORM_TRY
    TformSpecialAddTable * new_permission = new TformSpecialAddTable(NULL);
    new_permission->ShowTaskModal(this);
  FORM_CATCH
}
//---------------------------------------------------------------------------

void __fastcall TformSpecialPermissions::menuNoneClick(TObject *Sender)
{
  if ((listTablePermissions->SelCount > 0) && (listTablePermissions->SelCount > 0))
  {
    TListItem * row;
    for(unsigned int i = 0; i < table_permissions_vec.size(); i++)
    {
      row = listTablePermissions->Items->Item[i];
      if(row->Selected)
      {
        row->SubItemImages[0] = 0;
        row->SubItemImages[1] = 0;
        row->SubItemImages[2] = 0;
      }
    }
  }
}
//---------------------------------------------------------------------------

void __fastcall TformSpecialPermissions::menuViewClick(TObject *Sender)
{
  if ((listTablePermissions->SelCount > 0) && (listTablePermissions->SelCount > 0))
  {
    TListItem * row;
    for(unsigned int i = 0; i < table_permissions_vec.size(); i++)
    {
      row = listTablePermissions->Items->Item[i];
      if(row->Selected)
      {
        row->SubItemImages[0] = 1;
        row->SubItemImages[1] = 0;
        row->SubItemImages[2] = 0;
      }
    }
  }
}
//---------------------------------------------------------------------------
void __fastcall TformSpecialPermissions::menuViewUpdateClick(TObject *Sender)
{
  if ((listTablePermissions->SelCount > 0) && (listTablePermissions->SelCount > 0))
  {
    TListItem * row;
    for(unsigned int i = 0; i < table_permissions_vec.size(); i++)
    {
      row = listTablePermissions->Items->Item[i];
      if(row->Selected)
      {
        row->SubItemImages[0] = 1;
        row->SubItemImages[1] = 1;
        row->SubItemImages[2] = 0;
      }
    }
  }
}
//---------------------------------------------------------------------------
void __fastcall TformSpecialPermissions::menuAllClick(TObject *Sender)
{
  if ((listTablePermissions->SelCount > 0) && (listTablePermissions->SelCount > 0))
  {
    TListItem * row;
    for(unsigned int i = 0; i < table_permissions_vec.size(); i++)
    {
      row = listTablePermissions->Items->Item[i];
      if(row->Selected)
      {
        row->SubItemImages[0] = 1;
        row->SubItemImages[1] = 1;
        row->SubItemImages[2] = 1;
      }
    }
  }
}
//---------------------------------------------------------------------------
void __fastcall TformSpecialPermissions::menuDisabledClick(TObject *Sender)
{
  if ((listSinglePermissions->SelCount > 0) && (listSinglePermissions->SelCount > 0))
  {
    TListItem * row;
    for(unsigned int i = 0; i < other_permissions_vec.size(); i++)
    {
      row = listSinglePermissions->Items->Item[i];
      if(row->Selected)
      {
        row->SubItemImages[0] = 0;
      }
    }
  }
}
//---------------------------------------------------------------------------
void __fastcall TformSpecialPermissions::menuEnabledClick(TObject *Sender)
{
  if ((listSinglePermissions->SelCount > 0) && (listSinglePermissions->SelCount > 0))
  {
    TListItem * row;
    for(unsigned int i = 0; i < other_permissions_vec.size(); i++)
    {
      row = listSinglePermissions->Items->Item[i];
      if(row->Selected)
      {
        row->SubItemImages[0] = 1;
      }
    }
  }
}
//---------------------------------------------------------------------------
void __fastcall TformSpecialPermissions::listSinglePermissionsClick(TObject *Sender)
{
  // determine x and y position
  if (listSinglePermissions->SelCount > 0)
  {
    TListItem * row = listSinglePermissions->Selected;

    x_pos -= row->Position.x - 2; // TODO -c MAGIC : no idea where '2' comes from exactly, or when it might change.

    int i;
    int left = 0;
    for (i = 0; i < listSinglePermissions->Columns->Count; i++)
    {
     if (x_pos >= left && x_pos < left + min(imageList->Width, listSinglePermissions->Columns->Items[i]->Width))
      break;
     left += listSinglePermissions->Columns->Items[i]->Width;
    }

    if (i > 0 && (i-1 < row->SubItems->Count))
    {
     row->SubItemImages[i-1] = (row->SubItemImages[i-1] + 1) % 2;

     // now, for all currently selected rows, modify rows such that their column value matches this one exactly
     TItemStates selected = TItemStates() << isSelected;
     for (TListItem * Item = listSinglePermissions->Selected; Item != NULL; Item = listSinglePermissions->GetNextItem(Item, sdAll, selected))
      Item->SubItemImages[i-1] = row->SubItemImages[i-1];
    }
  }
}
//---------------------------------------------------------------------------
void __fastcall TformSpecialPermissions::btnMakeLikeClick(TObject *Sender)
{
  // make permissions like another users
  if (listUsers->SelCount > 0)
  {
    user_id = user_filter.GetSelectedRecordID();
    FORM_TRY
      TformSpecialMakeLike * makeLikeForm = new TformSpecialMakeLike(NULL, user_id);
      makeLikeForm->ShowTaskModal(this);
    FORM_CATCH
    DownloadUserPermissions(user_id);
    DisplayUserPermissions(user_id);
  }        
}
//---------------------------------------------------------------------------
void __fastcall TformSpecialPermissions::btnMakeInvalidClick(TObject *Sender)
{
  string user_name = lookupTables->ValueOf("users", user_filter.GetSelectedRecordID());
  string message = "Remember that users marked as invalid cannot log in and do not show up in combo boxes.\n"
                   "Are you sure you want to mark " + user_name + " as invalid?";
  if(PSay(message, m_confirmation, MB_YESNO) == IDYES)
  {
    user_id = user_filter.GetSelectedRecordID();
    if(ExecuteStatement(dbDatabase, "update users set valid = 0 where id = " + user_id))
    {
      lookupTables->ExpectRefresh("users");
      atlas->Notify("users.id", user_id);
    }
    else
      PSay("Error updating user record in database", m_database_error, MB_OK);
  }
}
//---------------------------------------------------------------------------
void __fastcall TformSpecialPermissions::dropdownPopupPopup(TObject *Sender)
{
  btnMakeAll->Down = false;
}
//---------------------------------------------------------------------------
void __fastcall TformSpecialPermissions::listUsersSelectItem(TObject *Sender, TListItem *Item, bool Selected)
{
  user_id = user_filter.GetSelectedRecordID();
  if(user_id != "")
  {
    DownloadUserPermissions(user_id);
    DisplayUserPermissions(user_id);
 }
}
//---------------------------------------------------------------------------



