//---------------------------------------------------------------------------

#include <vcl.h>
#pragma hdrstop

#include "codeSpecialPermissionsToUsers.h"
#include "codeSpecialAddTable.h"
#include "codeSpecialAddUser.h"
//---------------------------------------------------------------------------
#pragma package(smart_init)
#pragma link "codeSpecialGeneric"
#pragma resource "*.dfm"
TformSpecialPermissionsToUsers *formSpecialPermissionsToUsers;
//---------------------------------------------------------------------------
__fastcall TformSpecialPermissionsToUsers::TformSpecialPermissionsToUsers(TComponent* Owner)
        : TformSpecialGeneric(Owner)
{
  this->IconIndex = 44;
  user_filter.SetListViewHandle(listUsers);
  user_filter.SetSecurity("users");
  user_filter.AddColumnSpec("User", "users", as_text, true, 115);
  user_filter.AddColumnSpec("Group", "users", as_text, true, 115);
  user_filter.AddColumnSpec("Valid", "users", as_text, true, 35, prettyBoolString);
  user_filter.AddColumnSpec("Internal Id", "internal_id", as_int, false, 80);
  user_filter.FetchPreferences("specialpermissionstousers.user_filter");
  user_filter.SetCheck(true);
  user_filter.SetCompress(true);
  user_filter.SetSQLStatement("");
  user_filter.RefreshDataSet();
  user_filter.SetAddEdit(this, AddRecord(&(TformSpecialPermissionsToUsers::AddUser)),
                         EditRecord(&(TformSpecialPermissionsToUsers::EditUser)));
  user_request = atlas->RequestNotificationWhen("users.id", "0", rel_any, as_int, this,
                 RefreshHandler(&(TformSpecialPermissionsToUsers::AtlasRefresh)));

  DisplayAllPermissions();

  possible_permissions_request = atlas->RequestNotificationWhen("possible_permissions", "0", rel_any, as_int, this,
                                 RefreshHandler(&(TformSpecialPermissionsToUsers::AtlasRefresh)));
}
//---------------------------------------------------------------------------
void TformSpecialPermissionsToUsers::DisplayAllPermissions()
{
  all_permissions_vec = lookupTables->GetValueList("possible_permissions");
  table_permissions_vec.clear();
  other_permissions_vec.clear();
  for(vector<string>::iterator itr = all_permissions_vec.begin(); itr != all_permissions_vec.end(); itr++)
  {
    if((*itr).find("table.") != string::npos)
      table_permissions_vec.push_back(*itr);
    else
      other_permissions_vec.push_back(*itr);
  }

  TListItem * row;
  listTablePermissions->Items->Clear();
  listSinglePermissions->Items->Clear();
  for(vector<string>::iterator itr = table_permissions_vec.begin(); itr != table_permissions_vec.end(); itr++)
  {
    row = listTablePermissions->Items->Add();
    row->Caption = LowercaseString((*itr)).c_str();
    row->SubItems->Add("");
    row->SubItemImages[0] = 2;
    row->SubItems->Add("");
    row->SubItemImages[1] = 2;
    row->SubItems->Add("");
    row->SubItemImages[2] = 2;
    row->ImageIndex = -1;
  }
  for(vector<string>::iterator itr = other_permissions_vec.begin(); itr != other_permissions_vec.end(); itr++)
  {
    row = listSinglePermissions->Items->Add();
    row->Caption = LowercaseString((*itr)).c_str();
    row->SubItems->Add("");
    row->SubItemImages[0] = 2;
    row->ImageIndex = -1;
  }
}
//---------------------------------------------------------------------------
void TformSpecialPermissionsToUsers::AtlasRefresh(int request_num, string matched_value)
{
 if (request_num == possible_permissions_request)
 {
   user_filter.SetSQLStatement("");
   user_filter.RefreshDataSet(); 
   DisplayAllPermissions();
 }
 else if(request_num == user_request)
 {
//   user_filter.SetSQLStatement("");
//   user_filter.RefreshDataSet();
   btnSearchClick(NULL);
 }
}
//---------------------------------------------------------------------------
void TformSpecialPermissionsToUsers::AddUser()
{
  map<string,string> to_pass;
//  vector<string> return_vec;
  to_pass["id"] = "new";
  FORM_TRY
    TformSpecialAddUser * editUserForm = new TformSpecialAddUser(NULL, to_pass, NULL);
    editUserForm->Show();
  FORM_CATCH
}
//---------------------------------------------------------------------------
void TformSpecialPermissionsToUsers::EditUser()
{
  map<string,string> to_pass;
  vector<string> return_vec;
  to_pass["id"] = user_filter.GetSelectedRecordID();
  FORM_TRY
    TformSpecialAddUser * editUserForm = new TformSpecialAddUser(NULL, to_pass, &return_vec);
    editUserForm->Show();
  FORM_CATCH
}
//---------------------------------------------------------------------------
void __fastcall TformSpecialPermissionsToUsers::btnCloseClick(TObject *Sender)
{
  Close();
}
//---------------------------------------------------------------------------
// jhaley 20100819: helper method for btnSearchClick: build an SQL statement for permissions.
// Pass the check value (0 or 1) for "has" or "has not" query. Generalized from code previously
// in btnSearchClick and corrected to run on Firebird 2.x, which requires aliases to disambiguate
// the multiple joins to permissions.
string TformSpecialPermissionsToUsers::BuildPermissionsQuery(int check_has)
{
  TListItem * row;
  string sql = "select users.id from users ";

  // jhaley: I hope that this does not join too many times... this is how it worked before,
  // and my attempts to change it were not successful.
  for (unsigned int i = 0; i < table_permissions_vec.size(); i++)
  {
    string view_alias = "pview" + IntToString(i);
    string update_alias = "pupdate" + IntToString(i);
    string insert_alias = "pinsert" + IntToString(i);
    row = listTablePermissions->Items->Item[i];
    if (row->SubItemImages[0] == check_has)
      sql += "inner join permissions " + view_alias + " on " + view_alias + ".user_id = users.id "
             "and " + view_alias + ".s = '" + SafeSQLString(row->Caption.c_str()) + "' "
             "and " + view_alias + ".k = 'VIEW' "
             "and " + view_alias + ".v = 'TRUE' ";
    if (row->SubItemImages[1] == check_has)
      sql += "inner join permissions " + update_alias + " on " + update_alias + ".user_id = users.id "
             "and " + update_alias + ".s = '" + SafeSQLString(row->Caption.c_str()) + "' "
             "and " + update_alias + ".k = 'UPDATE' "
             "and " + update_alias + ".v = 'TRUE' ";
    if (row->SubItemImages[2] == check_has)
      sql += "inner join permissions " + insert_alias + " on " + insert_alias + ".user_id = users.id "
             "and " + insert_alias + ".s = '" + SafeSQLString(row->Caption.c_str()) + "' "
             "and " + insert_alias + ".k = 'INSERT' "
             "and " + insert_alias + ".v = 'TRUE' ";
  }
  for (unsigned int i = 0; i < other_permissions_vec.size(); i++)
  {
    string special_alias = "pspecial" + IntToString(i);
    row = listSinglePermissions->Items->Item[i];
    if (row->SubItemImages[0] == check_has)
    {
      sql += "inner join permissions " + special_alias + " on " + special_alias + ".user_id = users.id "
             "and " + special_alias + ".s = '" + SafeSQLString(row->Caption.c_str()) + "' "
             "and " + special_alias + ".k = 'ENABLED' "
             "and " + special_alias + ".v = 'TRUE' ";
    }
  }

  return sql;
}
//---------------------------------------------------------------------------
void __fastcall TformSpecialPermissionsToUsers::btnSearchClick(TObject *Sender)
{
  Processing(true);

  set<string> check_id, uncheck_id;
  to_check_id.clear();

  string sql;

  // jhaley 20100818: refactored to use a subroutine and to work with Firebird 2.x
  sql = BuildPermissionsQuery(1);
  SqlToSet(dbDatabase, sql, check_id);

  sql = BuildPermissionsQuery(0);
  if (sql != "select users.id from users ")
    SqlToSet(dbDatabase, sql, uncheck_id);

  // subtract
  insert_iterator<set<string> > insert_itr(to_check_id, to_check_id.begin());
  set_difference(check_id.begin(), check_id.end(), uncheck_id.begin(), uncheck_id.end(), insert_itr);

  user_filter.SetCheck(this, CheckRecord(&(TformSpecialPermissionsToUsers::CheckEachRecord)));
  sql = "select users.item, groups.item, users.valid, users.id "
        "from users, groups "
        "where users.group_id = groups.id and users.id > 0"; // and users.valid = 1";
  user_filter.SetSQLStatement(sql);
  user_filter.RefreshDataSet();

  Processing(false);
}
//---------------------------------------------------------------------------
bool TformSpecialPermissionsToUsers::CheckEachRecord (map<string, string> & row_data)
{
 set<string>::iterator i_found_id = to_check_id.find(row_data["Internal Id"]);
 if (i_found_id != to_check_id.end())
   return true;
 else
   return false;
}
//---------------------------------------------------------------------------

void __fastcall TformSpecialPermissionsToUsers::listTablePermissionsClick(TObject *Sender)
{
  if (listTablePermissions->SelCount > 0)
  {
    TListItem * row = listTablePermissions->Selected;

    x_pos -= row->Position.x - 2; // TODO -c MAGIC : no idea where '2' comes from exactly, or when it might change.

    int i;
    int left = 0;
    for (i = 0; i < listTablePermissions->Columns->Count; i++)
    {
     if (x_pos >= left && x_pos < left + min(imageList->Width, listTablePermissions->Columns->Items[i]->Width))
      break;
     left += listTablePermissions->Columns->Items[i]->Width;
    }

    if (i > 0 && (i-1 < row->SubItems->Count))
    {
     row->SubItemImages[i-1] = (row->SubItemImages[i-1] + 1) % 3;

     // now, for all currently selected rows, modify rows such that their column value matches this one exactly
     TItemStates selected = TItemStates() << isSelected;
     for (TListItem * Item = listTablePermissions->Selected; Item != NULL; Item = listTablePermissions->GetNextItem(Item, sdAll, selected))
      Item->SubItemImages[i-1] = row->SubItemImages[i-1];
    }
  }
}
//---------------------------------------------------------------------------
void __fastcall TformSpecialPermissionsToUsers::listSinglePermissionsClick(TObject *Sender)
{
  if (listSinglePermissions->SelCount > 0)
  {
    TListItem * row = listSinglePermissions->Selected;

    x_pos -= row->Position.x - 2; // TODO -c MAGIC : no idea where '2' comes from exactly, or when it might change.

    int i;
    int left = 0;
    for (i = 0; i < listSinglePermissions->Columns->Count; i++)
    {
     if (x_pos >= left && x_pos < left + min(imageList->Width, listSinglePermissions->Columns->Items[i]->Width))
      break;
     left += listSinglePermissions->Columns->Items[i]->Width;
    }

    if (i > 0 && (i-1 < row->SubItems->Count))
    {
     row->SubItemImages[i-1] = (row->SubItemImages[i-1] + 1) % 3;

     // now, for all currently selected rows, modify rows such that their column value matches this one exactly
     TItemStates selected = TItemStates() << isSelected;
     for (TListItem * Item = listSinglePermissions->Selected; Item != NULL; Item = listSinglePermissions->GetNextItem(Item, sdAll, selected))
      Item->SubItemImages[i-1] = row->SubItemImages[i-1];
    }
  }
}
//---------------------------------------------------------------------------
void __fastcall TformSpecialPermissionsToUsers::listTablePermissionsMouseMove(TObject *Sender, TShiftState Shift, int X, int Y)
{
  x_pos = X;
}
//---------------------------------------------------------------------------
void __fastcall TformSpecialPermissionsToUsers::btnUpdateClick(TObject *Sender)
{
  vector<string> checked_id_vec = user_filter.GetMultiID();
  if(! checked_id_vec.empty())
  {
    Processing(true);
    TIBTransaction *dbTransaction = new TIBTransaction(NULL);
    TListItem *row;
    map<string, string> insert_map;
    string permission_name;
    string sql_permission_name;
    try
    {
      dbTransaction->DefaultDatabase = dbDatabase;
      dbTransaction->StartTransaction();
      for(vector<string>::iterator itr = checked_id_vec.begin(); itr != checked_id_vec.end(); itr++)
      {
        insert_map["user_id"] = (*itr);
        for(unsigned int i = 0; i < table_permissions_vec.size(); i++)
        {
          row = listTablePermissions->Items->Item[i];
          permission_name = row->Caption.c_str();
          sql_permission_name = "'" + SafeSQLString(permission_name) + "'";
          insert_map["s"] = permission_name;

          // if it's non-shaded, then we delete anything related to it first, and re-insert it if it's meant to be TRUE
          if(row->SubItemImages[0] != 2)
           ExecuteStatement(dbTransaction, "delete from permissions where s = " + sql_permission_name +
                                           " and user_id = " + (*itr) + " and k = 'VIEW'");
          if(row->SubItemImages[1] != 2)
           ExecuteStatement(dbTransaction, "delete from permissions where s = " + sql_permission_name +
                                           " and user_id = " + (*itr) + " and k = 'UPDATE'");
          if(row->SubItemImages[2] != 2)
           ExecuteStatement(dbTransaction, "delete from permissions where s = " + sql_permission_name +
                                           " and user_id = " + (*itr) + " and k = 'INSERT'");

          if(row->SubItemImages[0] == 1)
          {
            insert_map["k"] = "view";
            insert_map["v"] = "true";
            ExecuteInsertStatement(dbTransaction, "permissions", insert_map);
          }
          if(row->SubItemImages[1] == 1)
          {
            insert_map["k"] = "update";
            insert_map["v"] = "true";
            ExecuteInsertStatement(dbTransaction, "permissions", insert_map);
          }
          if(row->SubItemImages[2] == 1)
          {
            insert_map["k"] = "insert";
            insert_map["v"] = "true";
            ExecuteInsertStatement(dbTransaction, "permissions", insert_map);
          }
        } // end for permissions with view, update, and insert
        for(unsigned int i = 0; i < other_permissions_vec.size(); i++)
        {
          row = listSinglePermissions->Items->Item[i];
          permission_name = row->Caption.c_str();
          sql_permission_name = "'" + SafeSQLString(permission_name) + "'";
          insert_map["s"] = permission_name;
          if(row->SubItemImages[0] != 2)
           ExecuteStatement(dbTransaction, "delete from permissions where s = " + sql_permission_name +
                                           " and user_id = " + (*itr) + " and k = 'ENABLED'");
          if (row->SubItemImages[0] == 1)
          {
            insert_map["k"] = "enabled";
            insert_map["v"] = "true";
            ExecuteInsertStatement(dbTransaction, "permissions", insert_map);
          }
        } // end for permissions with view, update, and insert
      } // end for on checked_id_vec
      dbTransaction->Commit();
      Processing(false);
    } // end try
    catch(...)
    {
      Processing(false);
      if(dbTransaction->Active)
        dbTransaction->Rollback();
      PSay("Error updating permissions", m_database_error, MB_OK);
    }
  } // end if any users checked
}
//---------------------------------------------------------------------------
void __fastcall TformSpecialPermissionsToUsers::menuAddTableClick(TObject *Sender)
{
  FORM_TRY
    TformSpecialAddTable * new_permission = new TformSpecialAddTable(NULL);
    new_permission->ShowTaskModal(this);
  FORM_CATCH
}
//---------------------------------------------------------------------------


