//---------------------------------------------------------------------------
/*
IM from Ashley
agent2187: you can determine how much vision money they've spent based on the area_id in services_provided (the services_provided_id linked to the cob)
agent2187: the area_types are things like medical, vision, dental, etc...
*/

#include <vcl.h>
#pragma hdrstop

#include "codeSpecialPersonLimits.h"
#include "control_break_objects.h"
//---------------------------------------------------------------------------
#pragma package(smart_init)
#pragma link "codeSpecialGeneric"
#pragma link "ttreelistview"
#pragma resource "*.dfm"
TformSpecialPersonLimits *formSpecialPersonLimits;
//---------------------------------------------------------------------------
__fastcall TformSpecialPersonLimits::TformSpecialPersonLimits(TComponent* Owner, map<string, string> p_parameters)
        : TformSpecialGeneric(Owner)
{
 Processing(true);
 if (IsInt(p_parameters["id"]))
  person_id = p_parameters["id"];

 CreateLink(staticPerson, NULL, this, person_id, "people", "");
 Pdate date_as_of(Date());
 selectPeriod->Position = date_as_of.year;

 totals_listview = new edit_listview();
 totals_listview->setup_listview(listTotals);
 totals_listview->setup_callbacks((TForm*)this, InformForm(&(TformSpecialPersonLimits::total_selected)), NULL);

 FetchAmounts();
 ProcessAmounts();

 Processing(false);
}
//---------------------------------------------------------------------------
void TformSpecialPersonLimits::total_selected ()
{
 // user has selected data (listview->data box)
 if (listTotals->Selected)
 {
  map<string, string> row_data;
  if (totals_listview->get_current_row(row_data))
  {
   // something selected
  } else
  {
   PSay("You have selected a row, but for some reason the data cannot be retrieved to be processed. You should report this immediately.", m_program_error, MB_OK);
  }
 } else
 {
  // nothing selected
 }
}
//---------------------------------------------------------------------------
void TformSpecialPersonLimits::ProcessAmounts()
{
 // headers
 totals_listview->clear_data();

 vector<map<string, string> > copy_of_data = processed_combo_data;

 Processing (true);

 checkLimits->Enabled = (checkArea->Checked && checkPeriod->Checked && checkAccount->Checked);

 totals_listview->add_or_modify_column("area_name",        "Area",           checkArea->Checked,        0, 90,   as_text, prettyString);
 totals_listview->add_or_modify_column("period",           "Period",         checkPeriod->Checked,      1, 70,   as_date, prettyString);
 totals_listview->add_or_modify_column("account_name",     "Account",        checkAccount->Checked,     2, 130,  as_text, prettyString);
 totals_listview->add_or_modify_column("cob_expenses",     "COB Expenses",   checkCOBExpenses->Checked, 5, 80, as_double, amountString);
 totals_listview->add_or_modify_column("vis_expenses",     "Bill Expenses",  checkVisExpenses->Checked, 6, 80, as_double, amountString);
 totals_listview->add_or_modify_column("tot_expenses",     "Total Expenses", checkTotExpenses->Checked, 7, 80, as_double, amountString);
 totals_listview->add_or_modify_column("limit_amount",     "Limit",          checkLimits->Checked && checkLimits->Enabled,      8, 80, as_double, nonzeroAmountString);
 totals_listview->add_or_modify_column("remaining_amount", "Remaining",      checkLimits->Checked && checkLimits->Enabled,      9, 80, as_double, amountString);

 vector<map<string, string> > grouped_data;

 if (checkLimits->Checked && checkLimits->Enabled)
 {
  int this_year = Pdate(Date()).year;
  for (vector<map<string, string> >::iterator i_row = copy_of_data.begin(); i_row != copy_of_data.end(); i_row++)
  {
   if ((*i_row)["area_name"] == "Psych Inpatient")
    (*i_row)["period"] = "Lifetime";
   else
   if ((*i_row)["area_name"] == "Vision" && (*i_row)["account_name"] == "003-6300000-061-063")
    (*i_row)["period"] = "Lifetime";
   else
   if ((*i_row)["area_name"] == "Orthodontics")
    (*i_row)["period"] = "Lifetime";
   else
   if ((*i_row)["area_name"] == "Hearing")
   {
    int temp_year = StringToInt((*i_row)["period"]);
    (*i_row)["period"] = YearList(this_year, temp_year, 3);
   }
  }
 }

 vector<pair<string, cb_sorting_object<string> *> > sorting_methods;
 map<string, cb_aggregate_object<string> *> summing_methods;

 if (checkAccount->Checked) sorting_methods.push_back(pair<string, cb_sorting_object<string> *>("account_name", new sort_less_than<string>()));
 if (checkArea->Checked) sorting_methods.push_back(pair<string, cb_sorting_object<string> *>("area_name", new sort_less_than<string>()));
 if (checkPeriod->Checked) sorting_methods.push_back(pair<string, cb_sorting_object<string> *>("period", new sort_less_than<string>()));

 summing_methods["cob_expenses"] = new aggregate_callback_wrapper<string, financial, aggregate_sum<financial> >(StringToFinancial, FinancialToStringEx);
 summing_methods["vis_expenses"] = new aggregate_callback_wrapper<string, financial, aggregate_sum<financial> >(StringToFinancial, FinancialToStringEx);
 summing_methods["tot_expenses"] = new aggregate_callback_wrapper<string, financial, aggregate_sum<financial> >(StringToFinancial, FinancialToStringEx);

 grouped_data.reserve(copy_of_data.size());
 do_control_break(copy_of_data, grouped_data, sorting_methods, summing_methods);

 // TODO: limits, remaining amounts?

 if (checkLimits->Checked && checkLimits->Enabled)
 {
  for (vector<map<string, string> >::iterator i_row = grouped_data.begin(); i_row != grouped_data.end(); i_row++)
  {
   if ((*i_row)["area_name"] == "Psych Inpatient")
    (*i_row)["limit_amount"] = "5000.00";
   else
   if ((*i_row)["area_name"] == "Vision" && (*i_row)["account_name"] == "003-6300000-061-063")
    (*i_row)["limit_amount"] = "100.00";
   else
   if ((*i_row)["area_name"] == "Vision" && (*i_row)["account_name"] == "003-6300000-061-000")
   {
    // business rule:
    // before 2005, the limit is $200/yr
    // after 2005, the limit is $350/yr
    // during 2005, the limit is $200/yr for those who used any vision money between 1/1/2005 and 8/15/2005
    // during 2005, the limit is $350/yr for those who did not use any vision money between 1/1/2005 and 8/15/2005
    int y = StringToInt((*i_row)["period"]);
    if (y < 2005 || (y == 2005 && hack_used_2005_vision_money))
     (*i_row)["limit_amount"] = "200.00";
    else
     (*i_row)["limit_amount"] = "350.00";
   } else
   if ((*i_row)["area_name"] == "Dental")
    (*i_row)["limit_amount"] = "1000.00";
   else
   if ((*i_row)["area_name"] == "Orthodontics")
    (*i_row)["limit_amount"] = "2000.00";
   else
   if ((*i_row)["area_name"] == "Hearing")
    (*i_row)["limit_amount"] = "1500.00";

   // if there's a limit (none for medical) then we calculate the remaining amount
   if ((*i_row)["limit_amount"] != "")
    (*i_row)["remaining_amount"] = FinancialToString(StringToFinancial((*i_row)["limit_amount"]) - StringToFinancial((*i_row)["tot_expenses"]));
  }
 }

 // multi_column_sorting_object<string, string> sorting_object (sorting_methods);
 // sort(grouped_data.begin(), grouped_data.end(), sorting_object);

 // deallocate
 cb_deallocate_sorting_map(sorting_methods);
 cb_deallocate_aggregate_map(summing_methods);

 // finalize and visualize
 totals_listview->import_data(grouped_data);
 Processing(false);
}
//---------------------------------------------------------------------------
void TformSpecialPersonLimits::FetchAmounts()
{
 Processing (true);

 processed_combo_data.clear();
 vector<map<string, string> > raw_cob_data;
 vector<map<string, string> > raw_vis_data;
 vector<map<string, string> > processed_cob_data;
 vector<map<string, string> > processed_vis_data;

 map<string, string>          temp_row;
 map<string, string>          lookup_cob_to_area;

 /*
  COB: tasks.parent_task_id = 0
       tasks.group_id ... usually = 5
       tasks.services_provided_id -> services_provided.id where we find area_id -> area_types.id and area_types.item
       transactions.task_id -> tasks.id
       appointments.task_id -> tasks.id
       appointments.final_end_stamp should be good? pull the date out of here? end of the last appointment possible!
       appointments.visit_outcome_id -> visit_outcomes.id, looking for 1 and 0? mostly 1? all others indicate it didn't happen ...
       ... but what if there's no appointment? use tasks.date_of_task?

       ... we only really care about cob's that -have- a total on them. can we do that first somehow?

  Vis: tasks.parent_task_id != 0
       tasks.group_id ... usually = 5
       tasks.services_provided_id = 0 (must look up through cob's)
       tasks.date_of_task ... should be the date we want to use (regardless of appointments?)
       bills.task_id = tasks.id
       transactions.bill_id = bills.id

       ... we only really care about visits that -have- a total on them.
 */

 // we're gonna need a transaction for this, hunh.
 TIBTransaction * dbTransaction = new TIBTransaction(NULL);
 StdTransaction(dbTransaction, dbDatabase);

 string sql_get_raw_cob = "select tasks.id, accounts.name account_name, accounts.account_type_id, services_provided.area_id, sum(transactions.debit_amount - transactions.credit_amount) amount, max(appointments.final_end_stamp) end_stamp, tasks.date_of_task "
                          "from tasks "
                          "left join services_provided on services_provided.id = tasks.services_provided_id "
                          "left join transactions on transactions.task_id = tasks.id "
                          "left join accounts on accounts.id = transactions.account_id "
                          "left join appointments on appointments.task_id = tasks.id and appointments.visit_outcome_id + 1 < 2 + 1 and appointments.current_id + 1 > 2 + 1" // TODO -c HACK : speed?
                          "where tasks.person_id = " + person_id +
                          "and tasks.services_provided_id > 0 "
                          "group by tasks.id, accounts.name, accounts.account_type_id, services_provided.area_id, tasks.date_of_task ";

 string sql_get_raw_vis = "select tasks.id, accounts.name account_name, accounts.account_type_id, tasks.parent_task_id, sum(transactions.debit_amount - transactions.credit_amount) amount, tasks.date_of_task "
                          "from tasks "
                          "left join bills on bills.task_id = tasks.id "
                          "left join transactions on transactions.bill_id = bills.id "
                          "left join accounts on accounts.id = transactions.account_id "
                          "where tasks.person_id = " + person_id +
                          "and tasks.parent_task_id > 0 "
                          "group by tasks.id, accounts.name, accounts.account_type_id, tasks.parent_task_id, tasks.date_of_task ";

 if (SqlToVecMap(dbTransaction, sql_get_raw_cob, raw_cob_data))
 {
  if (SqlToVecMap(dbTransaction, sql_get_raw_vis, raw_vis_data))
  {
   // do stuff here
   for (vector<map<string, string> >::iterator i_raw_cob = raw_cob_data.begin(); i_raw_cob != raw_cob_data.end(); i_raw_cob++)
   {
    // keep this for the visit pass
    lookup_cob_to_area[(*i_raw_cob)["id"]] = (*i_raw_cob)["area_id"];

    // now make a well-defined row to be counted with the rest, grouped, etc.
    if ((*i_raw_cob)["account_type_id"] == lookupTables->IdOf("account_types", "Expense"))
    {
     temp_row.clear();
     temp_row["account_name"] = (*i_raw_cob)["account_name"];
     temp_row["area_name"] = lookupTables->ValueOf("area_types", (*i_raw_cob)["area_id"]);
     temp_row["cob_expenses"] = (*i_raw_cob)["amount"];
     temp_row["tot_expenses"] = (*i_raw_cob)["amount"];
     if ((*i_raw_cob)["end_stamp"].empty())
      temp_row["period"] = IntToString(Pdate((*i_raw_cob)["date_of_task"], global_mode).year);
     else
      temp_row["period"] = IntToString(Pstamp((*i_raw_cob)["end_stamp"], global_mode).year);
     processed_combo_data.push_back(temp_row);
    }
   }

   hack_used_2005_vision_money = false;
   for (vector<map<string, string> >::iterator i_raw_vis = raw_vis_data.begin(); i_raw_vis != raw_vis_data.end(); i_raw_vis++)
   {
    // now make a well-defined row to be counted with the rest, grouped, etc.
    if ((*i_raw_vis)["account_type_id"] == lookupTables->IdOf("account_types", "Expense"))
    {
     temp_row.clear();
     temp_row["account_name"] = (*i_raw_vis)["account_name"];
     temp_row["area_name"] = lookupTables->ValueOf("area_types", lookup_cob_to_area[(*i_raw_vis)["parent_task_id"]]);
     temp_row["vis_expenses"] = (*i_raw_vis)["amount"];
     temp_row["tot_expenses"] = (*i_raw_vis)["amount"];

     if (temp_row["area_name"] == "Vision" &&
         StringToFinancial((*i_raw_vis)["amount"]) > 0.00 &&
         Pdate((*i_raw_vis)["date_of_task"], global_mode) >= Pdate("1/1/2005", global_mode) &&
         Pdate((*i_raw_vis)["date_of_task"], global_mode) <  Pdate("8/15/2005", global_mode))
      hack_used_2005_vision_money = true;

     temp_row["period"] = IntToString(Pdate((*i_raw_vis)["date_of_task"], global_mode).year);
     processed_combo_data.push_back(temp_row);
    }
   }
   dbTransaction->Commit();
  } else
  {
   PSay("Request for visit expense information failed.", m_database_error, MB_OK);
   dbTransaction->Rollback();
  }
 } else
 {
  PSay("Request for COB expense information failed.", m_database_error, MB_OK);
  dbTransaction->Rollback();
 }

 delete dbTransaction;
 Processing (false);
}
//---------------------------------------------------------------------------
//
// THE FOLLOWING SECTION IS NOW DEPRECATED -- ONLY THE SECTIONS ABOVE MATTERS ANYMORE
//

void TformSpecialPersonLimits::RefreshAmounts()
{
 Processing(true);
 if (!IsInt(person_id))
 {
  PSay("No person was specified", m_program_error, MB_OK);
  return;
 }

 int given_year = 2000;
 if (IsInt(editPeriod->Text.c_str()))
  given_year = StringToInt(editPeriod->Text.c_str());
 else
 {
  PSay("The year specified is not valid", m_invalid_data, MB_OK);
  return;
 }

 // do important stuff based on person_id
 // if it's now 5/25/2002
 // this year's worth is 1/1/2002 to (not including) 1/1/2003
 // hearing goes from to 1/1/2000 to (not including) 1/1/2003
 Pdate begin_this_year;
 Pdate end_this_year;
 Pdate begin_hearing_year;

 // the between clause is inclusive, so we change end_this_year to 12/31 of the same year
 begin_this_year.InDate("01/01/" + IntToString(given_year), mdy | yyyy);
 end_this_year.InDate("12/31/" + IntToString(given_year), mdy | yyyy);
 begin_hearing_year.InDate("01/01/" + IntToString(given_year - 2), mdy | yyyy);

 string medical_area    = lookupTables->IdOf("area_types", "Medical");
 string dental_area     = lookupTables->IdOf("area_types", "Dental");
 string ortho_area      = lookupTables->IdOf("area_types", "Orthodontics");
 string vision_area     = lookupTables->IdOf("area_types", "Vision");
 string hearing_area    = lookupTables->IdOf("area_types", "Hearing");

 /* TODO: should be in configuration for the database
   (the settings table that doesn't exist, along with the name of the tribe and other crock) */
 string usual_account   = "858"; // TODO -c MAGIC NUMBER :
 string gaming_account  = "859"; // TODO -c MAGIC NUMBER :

 string payable_account = lookupTables->IdOf("account_types", "Payable");
 string expense_account = lookupTables->IdOf("account_types", "Expense");
 string cash_account    = lookupTables->IdOf("account_types", "Cash");

 string contract_health_group = lookupTables->IdOf("groups", "Contract Health");

 this->Update();

 // medical
 string sql_medical_cob_exp;
 string sql_medical_bill_exp;
 string medical_cob_exp;
 string medical_bill_exp;

 financial medical_cob_expensed  = 0.00;
 financial medical_bill_expensed = 0.00;
 financial medical_expensed      = 0.00;

 labelPeriodMedical->Caption = string(begin_this_year.OutDate(mdy | yyyy) + " - " + end_this_year.OutDate(mdy | yyyy)).c_str();
 sql_medical_cob_exp = "select sum(transactions.debit_amount - transactions.credit_amount) "
                       "from tasks "
                       "inner join services_provided on tasks.services_provided_id = services_provided.id "
                       "inner join transactions on transactions.task_id = tasks.id "
                       "inner join accounts on transactions.account_id = accounts.id "
                       "where tasks.person_id = " + person_id +
                       " and tasks.services_provided_id != 0" // and tasks.services_provided_id is not null"
                       " and tasks.group_id = " + contract_health_group +
                       " and services_provided.area_id = " + medical_area +
                       " and accounts.account_type_id = " + expense_account +
                       " and tasks.date_of_task between '" + begin_this_year.OutDate(mdy | yyyy) +
                       "' and '" + end_this_year.OutDate(mdy | yyyy) + "'";
 sql_medical_bill_exp = "select sum(transactions.debit_amount - transactions.credit_amount) "
                        "from tasks cob_tasks "
                        "inner join tasks visit_tasks on visit_tasks.parent_task_id = cob_tasks.id "
                        "inner join bills on bills.task_id = visit_tasks.id "
                        "inner join transactions on transactions.bill_id = bills.id "
                        "inner join services_provided on cob_tasks.services_provided_id = services_provided.id "
                        "inner join accounts on transactions.account_id = accounts.id "
                        "where cob_tasks.person_id = " + person_id +
                        " and cob_tasks.services_provided_id != 0" // and cob_tasks.services_provided_id is not null"
                        " and cob_tasks.group_id = " + contract_health_group +
                        " and visit_tasks.group_id = " + contract_health_group +
                        " and services_provided.area_id = " + medical_area +
                        " and visit_tasks.parent_task_id != 0" // and visit_tasks.parent_task_id is not null"
                        " and accounts.account_type_id = " + expense_account +
                        " and cob_tasks.date_of_task between '" + begin_this_year.OutDate(mdy | yyyy) +
                        "' and '" + end_this_year.OutDate(mdy | yyyy) + "'";
 medical_cob_exp = GetOneField(dbDatabase, sql_medical_cob_exp);
 medical_bill_exp = GetOneField(dbDatabase, sql_medical_bill_exp);
 medical_cob_expensed   = IsAmount(medical_cob_exp)  ? StringToFinancial(medical_cob_exp)  : financial(0.00);
 medical_bill_expensed  = IsAmount(medical_bill_exp) ? StringToFinancial(medical_bill_exp) : financial(0.00);
 medical_expensed       = medical_cob_expensed + medical_bill_expensed;

 labelExpensedMedical->Caption = FinancialToString(medical_expensed, true).c_str();
 labelObligatedMedical->Caption = FinancialToString(medical_cob_expensed, true).c_str();
 labelPaidMedical->Caption = FinancialToString(medical_bill_expensed, true).c_str();

 this->Update();

 // dental
 string sql_dental_cob_exp;
 string sql_dental_bill_exp;
 string dental_cob_exp;
 string dental_bill_exp;

 financial dental_allowed       = 1000.00;
 financial dental_cob_expensed  = 0.00;
 financial dental_bill_expensed = 0.00;
 financial dental_expensed      = 0.00;
 financial dental_remaining     = 0.00;

 labelPeriodDental->Caption = string(begin_this_year.OutDate(mdy | yyyy) + " - " + end_this_year.OutDate(mdy | yyyy)).c_str();
 sql_dental_cob_exp = "select sum(transactions.debit_amount - transactions.credit_amount) "
                      "from tasks "
                      "inner join transactions on transactions.task_id = tasks.id "
                      "inner join services_provided on tasks.services_provided_id = services_provided.id "
                      "inner join accounts on transactions.account_id = accounts.id "
                      "where tasks.person_id = " + person_id +
                      " and tasks.group_id = " + contract_health_group +
                      " and tasks.services_provided_id != 0" // and tasks.services_provided_id is not null"
                      " and services_provided.area_id = " + dental_area +
                      " and accounts.account_type_id = " + expense_account +
                      " and tasks.date_of_task between '" + begin_this_year.OutDate(mdy | yyyy) +
                      "' and '" + end_this_year.OutDate(mdy | yyyy) + "'";
 sql_dental_bill_exp = "select sum(transactions.debit_amount - transactions.credit_amount) "
                       "from tasks cob_tasks "
                       "inner join tasks visit_tasks on visit_tasks.parent_task_id = cob_tasks.id "
                       "inner join bills on bills.task_id = visit_tasks.id "
                       "inner join transactions on transactions.bill_id = bills.id "
                       "inner join services_provided on cob_tasks.services_provided_id = services_provided.id "
                       "inner join accounts on transactions.account_id = accounts.id "
                       "where cob_tasks.person_id = " + person_id +
                       " and cob_tasks.group_id = " + contract_health_group +
                       " and visit_tasks.group_id = " + contract_health_group +
                       " and cob_tasks.services_provided_id != 0" // and cob_tasks.services_provided_id is not null"
                       " and services_provided.area_id = " + dental_area +
                       " and visit_tasks.parent_task_id != 0" // and visit_tasks.parent_task_id is not null"
                       " and accounts.account_type_id = " + expense_account +
                       " and cob_tasks.date_of_task between '" + begin_this_year.OutDate(mdy | yyyy) +
                       "' and '" + end_this_year.OutDate(mdy | yyyy) + "'";
 dental_cob_exp = GetOneField(dbDatabase, sql_dental_cob_exp);
 dental_bill_exp = GetOneField(dbDatabase, sql_dental_bill_exp);
 dental_cob_expensed   = IsAmount(dental_cob_exp)  ? StringToFinancial(dental_cob_exp)  : financial(0.00);
 dental_bill_expensed  = IsAmount(dental_bill_exp) ? StringToFinancial(dental_bill_exp) : financial(0.00);
 dental_expensed       = dental_cob_expensed + dental_bill_expensed;
 dental_remaining      = dental_allowed - dental_expensed;

 labelAllowedDental->Caption = FinancialToString(dental_allowed, true).c_str();
 labelExpensedDental->Caption = FinancialToString(dental_expensed, true).c_str();
 labelObligatedDental->Caption = FinancialToString(dental_cob_expensed, true).c_str();
 labelPaidDental->Caption = FinancialToString(dental_bill_expensed, true).c_str();
 labelRemainingDental->Caption = FinancialToString(dental_remaining, true).c_str();

 this->Update();

 // ortho
 string sql_ortho_cob_exp;
 string sql_ortho_bill_exp;
 string ortho_cob_exp;
 string ortho_bill_exp;

 financial ortho_allowed       = 2000.00;
 financial ortho_cob_expensed  = 0.00;
 financial ortho_bill_expensed = 0.00;
 financial ortho_expensed      = 0.00;
 financial ortho_remaining     = 0.00;

 labelPeriodOrtho->Caption = "" /* this never had any dates, just doing this for consistency */ ;
 sql_ortho_cob_exp = "select sum(transactions.debit_amount - transactions.credit_amount) "
                     "from tasks "
                     "inner join transactions on transactions.task_id = tasks.id "
                     "inner join services_provided on tasks.services_provided_id = services_provided.id "
                     "inner join accounts on transactions.account_id = accounts.id "
                     "where tasks.person_id = " + person_id +
                     " and tasks.group_id = " + contract_health_group +
                     " and tasks.services_provided_id != 0" // and tasks.services_provided_id is not null"
                     " and services_provided.area_id = " + ortho_area +
                     " and accounts.account_type_id = " + expense_account;
 sql_ortho_bill_exp = "select sum(transactions.debit_amount - transactions.credit_amount) "
                      "from tasks cob_tasks "
                      "inner join tasks visit_tasks on visit_tasks.parent_task_id = cob_tasks.id "
                      "inner join bills on bills.task_id = visit_tasks.id "
                      "inner join transactions on transactions.bill_id = bills.id "
                      "inner join services_provided on cob_tasks.services_provided_id = services_provided.id "
                      "inner join accounts on transactions.account_id = accounts.id "
                      "where cob_tasks.person_id = " + person_id +
                      " and cob_tasks.group_id = " + contract_health_group +
                      " and visit_tasks.group_id = " + contract_health_group +
                      " and cob_tasks.services_provided_id != 0" // and cob_tasks.services_provided_id is not null"
                      " and services_provided.area_id = " + ortho_area +
                      " and visit_tasks.parent_task_id != 0" // and visit_tasks.parent_task_id is not null"
                      " and accounts.account_type_id = " + expense_account;
 ortho_cob_exp = GetOneField(dbDatabase, sql_ortho_cob_exp);
 ortho_bill_exp = GetOneField(dbDatabase, sql_ortho_bill_exp);
 ortho_cob_expensed   = IsAmount(ortho_cob_exp)  ? StringToFinancial(ortho_cob_exp)  : financial(0.00);
 ortho_bill_expensed  = IsAmount(ortho_bill_exp) ? StringToFinancial(ortho_bill_exp) : financial(0.00);
 ortho_expensed       = ortho_cob_expensed + ortho_bill_expensed;
 ortho_remaining      = ortho_allowed - ortho_expensed;

 labelAllowedOrtho->Caption = FinancialToString(ortho_allowed, true).c_str();
 labelExpensedOrtho->Caption = FinancialToString(ortho_expensed, true).c_str();
 labelObligatedOrtho->Caption = FinancialToString(ortho_cob_expensed, true).c_str();
 labelPaidOrtho->Caption = FinancialToString(ortho_bill_expensed, true).c_str();
 labelRemainingOrtho->Caption = FinancialToString(ortho_remaining, true).c_str();

 this->Update();

 // vision
 string sql_vision_cob_exp;
 string sql_vision_bill_exp;
 string vision_cob_exp;
 string vision_bill_exp;

 financial vision_allowed       = 200.00;
 financial vision_cob_expensed  = 0.00;
 financial vision_bill_expensed = 0.00;
 financial vision_expensed      = 0.00;
 financial vision_remaining     = 0.00;

 labelPeriodVision->Caption = string(begin_this_year.OutDate(mdy | yyyy) + " - " + end_this_year.OutDate(mdy | yyyy)).c_str();
 sql_vision_cob_exp = "select sum(transactions.debit_amount - transactions.credit_amount) "
                      "from tasks "
                      "inner join transactions on transactions.task_id = tasks.id "
                      "inner join services_provided on tasks.services_provided_id = services_provided.id "
                      "inner join accounts on transactions.account_id = accounts.id "
                      "where tasks.person_id = " + person_id +
                      " and tasks.group_id = " + contract_health_group +
                      " and tasks.services_provided_id != 0" // and tasks.services_provided_id is not null"
                      " and services_provided.area_id = " + vision_area +
                      " and accounts.account_type_id = " + expense_account +
                      " and accounts.id = " + usual_account +
                      " and tasks.date_of_task between '" + begin_this_year.OutDate(mdy | yyyy) +
                      "' and '" + end_this_year.OutDate(mdy | yyyy) + "'";
 sql_vision_bill_exp = "select sum(transactions.debit_amount - transactions.credit_amount) "
                       "from tasks cob_tasks "
                       "inner join tasks visit_tasks on visit_tasks.parent_task_id = cob_tasks.id "
                       "inner join bills on bills.task_id = visit_tasks.id "
                       "inner join transactions on transactions.bill_id = bills.id "
                       "inner join services_provided on cob_tasks.services_provided_id = services_provided.id "
                       "inner join accounts on transactions.account_id = accounts.id "
                       "where cob_tasks.person_id = " + person_id +
                       " and cob_tasks.group_id = " + contract_health_group +
                       " and visit_tasks.group_id = " + contract_health_group +
                       " and cob_tasks.services_provided_id != 0" // and cob_tasks.services_provided_id is not null"
                       " and services_provided.area_id = " + vision_area +
                       " and visit_tasks.parent_task_id != 0" // and visit_tasks.parent_task_id is not null"
                       // " and accounts.account_type_id = " + expense_account +
                       " and accounts.id = " + usual_account +
                       " and cob_tasks.date_of_task between '" + begin_this_year.OutDate(mdy | yyyy) +
                       "' and '" + end_this_year.OutDate(mdy | yyyy) + "'";
 vision_cob_exp = GetOneField(dbDatabase, sql_vision_cob_exp);
 vision_bill_exp = GetOneField(dbDatabase, sql_vision_bill_exp);
 vision_cob_expensed   = IsAmount(vision_cob_exp)  ? StringToFinancial(vision_cob_exp)  : financial(0.00);
 vision_bill_expensed  = IsAmount(vision_bill_exp) ? StringToFinancial(vision_bill_exp) : financial(0.00);
 vision_expensed       = vision_cob_expensed + vision_bill_expensed;
 vision_remaining      = vision_allowed - vision_expensed;

 labelAllowedVision->Caption = FinancialToString(vision_allowed, true).c_str();
 labelExpensedVision->Caption = FinancialToString(vision_expensed, true).c_str();
 labelObligatedVision->Caption = FinancialToString(vision_cob_expensed, true).c_str();
 labelPaidVision->Caption = FinancialToString(vision_bill_expensed, true).c_str();
 labelRemainingVision->Caption = FinancialToString(vision_remaining, true).c_str();

 this->Update();

 // hearing
 string sql_hearing_cob_exp;
 string sql_hearing_bill_exp;
 string hearing_cob_exp;
 string hearing_bill_exp;

 financial hearing_allowed       = 1500.00;
 financial hearing_cob_expensed  = 0.00;
 financial hearing_bill_expensed = 0.00;
 financial hearing_expensed      = 0.00;
 financial hearing_remaining     = 0.00;

 labelPeriodHearing->Caption = string(begin_hearing_year.OutDate(mdy | yyyy) + " - " + end_this_year.OutDate(mdy | yyyy)).c_str();
 sql_hearing_cob_exp = "select sum(transactions.debit_amount - transactions.credit_amount) "
                       "from tasks "
                       "inner join transactions on transactions.task_id = tasks.id "
                       "inner join services_provided on tasks.services_provided_id = services_provided.id "
                       "inner join accounts on transactions.account_id = accounts.id "
                       "where tasks.person_id = " + person_id +
                       " and tasks.group_id = " + contract_health_group +
                       " and tasks.services_provided_id != 0" // and tasks.services_provided_id is not null"
                       " and services_provided.area_id = " + hearing_area +
                       " and accounts.account_type_id = " + expense_account +
                       " and tasks.date_of_task between '" + begin_hearing_year.OutDate(mdy | yyyy) +
                       "' and '" + end_this_year.OutDate(mdy | yyyy) + "'";
 sql_hearing_bill_exp = "select sum(transactions.debit_amount - transactions.credit_amount) "
                        "from tasks cob_tasks "
                        "inner join tasks visit_tasks on visit_tasks.parent_task_id = cob_tasks.id "
                        "inner join bills on bills.task_id = visit_tasks.id "
                        "inner join transactions on transactions.bill_id = bills.id "
                        "inner join services_provided on cob_tasks.services_provided_id = services_provided.id "
                        "inner join accounts on transactions.account_id = accounts.id "
                        "where cob_tasks.person_id = " + person_id +
                        " and cob_tasks.group_id = " + contract_health_group +
                        " and visit_tasks.group_id = " + contract_health_group +
                        " and cob_tasks.services_provided_id != 0" // and cob_tasks.services_provided_id is not null"
                        " and services_provided.area_id = " + hearing_area +
                        " and visit_tasks.parent_task_id != 0" // and visit_tasks.parent_task_id is not null"
                        " and accounts.account_type_id = " + expense_account +
                        " and cob_tasks.date_of_task between '" + begin_hearing_year.OutDate(mdy | yyyy) +
                        "' and '" + end_this_year.OutDate(mdy | yyyy) + "'";
 hearing_cob_exp = GetOneField(dbDatabase, sql_hearing_cob_exp);
 hearing_bill_exp = GetOneField(dbDatabase, sql_hearing_bill_exp);
 hearing_cob_expensed   = IsAmount(hearing_cob_exp)  ? StringToFinancial(hearing_cob_exp)  : financial(0.00);
 hearing_bill_expensed  = IsAmount(hearing_bill_exp) ? StringToFinancial(hearing_bill_exp) : financial(0.00);
 hearing_expensed       = hearing_cob_expensed + hearing_bill_expensed;
 hearing_remaining      = hearing_allowed - hearing_expensed;

 labelAllowedHearing->Caption = FinancialToString(hearing_allowed, true).c_str();
 labelExpensedHearing->Caption = FinancialToString(hearing_expensed, true).c_str();
 labelObligatedHearing->Caption = FinancialToString(hearing_cob_expensed, true).c_str();
 labelPaidHearing->Caption = FinancialToString(hearing_bill_expensed, true).c_str();
 labelRemainingHearing->Caption = FinancialToString(hearing_remaining, true).c_str();

 this->Update();

 // gaming
 string sql_gaming_cob_exp;
 string sql_gaming_bill_exp;
 string gaming_cob_exp;
 string gaming_bill_exp;

 financial gaming_allowed       = 100.00;
 financial gaming_cob_expensed  = 0.00;
 financial gaming_bill_expensed = 0.00;
 financial gaming_expensed      = 0.00;
 financial gaming_remaining     = 0.00;

 labelPeriodGaming->Caption = "" /* string(begin_this_year.OutDate(mdy | yyyy) + " - " + end_this_year.OutDate(mdy | yyyy)).c_str() */ ;
 sql_gaming_cob_exp = "select sum(transactions.debit_amount - transactions.credit_amount) "
                      "from tasks "
                      "inner join transactions on transactions.task_id = tasks.id "
                      "inner join services_provided on tasks.services_provided_id = services_provided.id "
                      "inner join accounts on transactions.account_id = accounts.id "
                      "where tasks.person_id = " + person_id +
                      " and tasks.services_provided_id != 0" // and tasks.services_provided_id is not null"
                      " and tasks.group_id = " + contract_health_group +
                      " and services_provided.area_id = " + vision_area +
                      " and accounts.account_type_id = " + expense_account +
                      " and accounts.id = " + gaming_account /* +
                      " and tasks.date_of_task between '" + begin_this_year.OutDate(mdy | yyyy) +
                      "' and '" + end_this_year.OutDate(mdy | yyyy) + "'" */ ;
 sql_gaming_bill_exp = "select sum(transactions.debit_amount - transactions.credit_amount) "
                       "from tasks cob_tasks "
                       "inner join tasks visit_tasks on visit_tasks.parent_task_id = cob_tasks.id "
                       "inner join bills on bills.task_id = visit_tasks.id "
                       "inner join transactions on transactions.bill_id = bills.id "
                       "inner join services_provided on cob_tasks.services_provided_id = services_provided.id "
                       "inner join accounts on transactions.account_id = accounts.id "
                       "where cob_tasks.person_id = " + person_id +
                       " and cob_tasks.group_id = " + contract_health_group +
                       " and visit_tasks.group_id = " + contract_health_group +
                       " and cob_tasks.services_provided_id != 0" // and cob_tasks.services_provided_id is not null"
                       " and services_provided.area_id = " + vision_area +
                       " and visit_tasks.parent_task_id != 0" // and visit_tasks.parent_task_id is not null"
                       " and accounts.account_type_id = " + expense_account +
                       " and accounts.id = " + gaming_account /* +
                       " and cob_tasks.date_of_task between '" + begin_this_year.OutDate(mdy | yyyy) +
                       "' and '" + end_this_year.OutDate(mdy | yyyy) + "'" */ ;
 gaming_cob_exp = GetOneField(dbDatabase, sql_gaming_cob_exp);
 gaming_bill_exp = GetOneField(dbDatabase, sql_gaming_bill_exp);
 gaming_cob_expensed   = IsAmount(gaming_cob_exp)  ? StringToFinancial(gaming_cob_exp)  : financial(0.00);
 gaming_bill_expensed  = IsAmount(gaming_bill_exp) ? StringToFinancial(gaming_bill_exp) : financial(0.00);
 gaming_expensed       = gaming_cob_expensed + gaming_bill_expensed;
 gaming_remaining      = gaming_allowed - gaming_expensed;

 labelAllowedGaming->Caption = FinancialToString(gaming_allowed, true).c_str();
 labelExpensedGaming->Caption = FinancialToString(gaming_expensed, true).c_str();
 labelObligatedGaming->Caption = FinancialToString(gaming_cob_expensed, true).c_str();
 labelPaidGaming->Caption = FinancialToString(gaming_bill_expensed, true).c_str();
 labelRemainingGaming->Caption = FinancialToString(gaming_remaining, true).c_str();

 labelExpensedTotal->Caption = FinancialToString(medical_expensed + dental_expensed + vision_expensed + gaming_expensed, true).c_str();
 labelObligatedTotal->Caption = FinancialToString(medical_cob_expensed + dental_cob_expensed + vision_cob_expensed + gaming_cob_expensed, true).c_str();
 labelPaidTotal->Caption = FinancialToString(medical_bill_expensed + dental_bill_expensed + vision_bill_expensed /* + gaming_bill_expensed */, true).c_str();
 labelPeriodTotal->Caption = string(begin_this_year.OutDate(mdy | yyyy) + " - " + end_this_year.OutDate(mdy | yyyy)).c_str();

 Processing(false);
 return;
}
//---------------------------------------------------------------------------
void __fastcall TformSpecialPersonLimits::buttonCloseClick(TObject *Sender)
{
 Close();
}
//---------------------------------------------------------------------------
void __fastcall TformSpecialPersonLimits::editPeriodChange(TObject *Sender)
{
// RefreshAmounts();
}
//---------------------------------------------------------------------------
void __fastcall TformSpecialPersonLimits::FormClose(TObject *Sender, TCloseAction &Action)
{
 Action = caFree;
 TformSpecialGeneric::FormClose(Sender, Action);
 if (Action != caNone)
  delete totals_listview;
}
//---------------------------------------------------------------------------
void __fastcall TformSpecialPersonLimits::checkPeriodClick(TObject *Sender)
{
 checkPeriod->Checked = !checkPeriod->Checked;
 ProcessAmounts();
}
//---------------------------------------------------------------------------
void __fastcall TformSpecialPersonLimits::checkAreaClick(TObject *Sender)
{
 checkArea->Checked = !checkArea->Checked;
 ProcessAmounts();
}
//---------------------------------------------------------------------------
void __fastcall TformSpecialPersonLimits::checkAccountClick(TObject *Sender)
{
 checkAccount->Checked = !checkAccount->Checked;
 ProcessAmounts();
}
//---------------------------------------------------------------------------
void __fastcall TformSpecialPersonLimits::checkCOBExpensesClick(TObject *Sender)
{
 checkCOBExpenses->Checked = !checkCOBExpenses->Checked;
 ProcessAmounts();
}
//---------------------------------------------------------------------------
void __fastcall TformSpecialPersonLimits::checkVisExpensesClick(TObject *Sender)
{
 checkVisExpenses->Checked = !checkVisExpenses->Checked;
 ProcessAmounts();
}
//---------------------------------------------------------------------------
void __fastcall TformSpecialPersonLimits::checkTotExpensesClick(TObject *Sender)
{
 checkTotExpenses->Checked = !checkTotExpenses->Checked;
 ProcessAmounts();
}
//--------------------------------------------------------------------------- 
void __fastcall TformSpecialPersonLimits::checkLimitsClick(TObject *Sender)
{
 checkLimits->Checked = !checkLimits->Checked;
 ProcessAmounts();
}
//---------------------------------------------------------------------------

