//---------------------------------------------------------------------------

#include <vcl.h>
#pragma hdrstop

#include "codeSpecialPersonNotesSummary.h"
#include "codeRecordPersonNotes.h"
//---------------------------------------------------------------------------
#pragma package(smart_init)
#pragma link "codeSpecialListview"
#pragma resource "*.dfm"
TformSpecialPersonNotesSummary *formSpecialPersonNotesSummary;
//---------------------------------------------------------------------------
__fastcall TformSpecialPersonNotesSummary::TformSpecialPersonNotesSummary(TComponent* Owner, string &p_person_id,
  vector<string> &p_group_ids, int &p_num_checked)
        : TformSpecialListview(Owner), num_checks(p_num_checked)
{
  // Remember person and group IDs
  person_id   = p_person_id;
  group_ids   = p_group_ids;

  // Source table is always person_notes
  SourceTableName = "person_notes";

  // Setup filter properties
  filter_stuff.SetSecurity("person_notes");
  filter_stuff.AddColumnSpec("Date",        "person_notes", as_date, true,  80);
  filter_stuff.AddColumnSpec("Note",        "person_notes", as_text, true,  320, sentenceString);
  filter_stuff.AddColumnSpec("Status",      "person_notes", as_text, true,  80,  prettyString);
  filter_stuff.AddColumnSpec("Current Id",  "current_id",   as_int,  false, 80);
  filter_stuff.AddColumnSpec("Internal Id", "internal_id",  as_int,  false, 80);
  filter_stuff.SetIconStatus(false);
  filter_stuff.SetCheck(true);

  // Open transaction
  FORM_TRY
    dbTransaction = new TIBTransaction(NULL);
  FORM_CATCH
  try
  {
    StdTransaction(dbTransaction, dbDatabase);
    filter_stuff.SetTransaction(dbTransaction);
  }
  catch (...)
  {
    PSay("Failed to open transaction for filter", m_database_error, MB_OK);
  }
}
//---------------------------------------------------------------------------
__fastcall TformSpecialPersonNotesSummary::~TformSpecialPersonNotesSummary()
{
  // Shutdown/delete transaction object
  if (dbTransaction)
  {
    if (dbTransaction->Active)
      dbTransaction->Rollback();
    delete dbTransaction;
    dbTransaction = NULL;
  }
}
//---------------------------------------------------------------------------
void TformSpecialPersonNotesSummary::Execute(TIBTransaction* dbTransaction, string sql_statement)
{
  // protected override of base class Execute - do nothing.
}
//---------------------------------------------------------------------------
void TformSpecialPersonNotesSummary::AtlasRefreshListView(int request_num, string matched_value)
{
  // TODO: Atlas refresh?
}
//---------------------------------------------------------------------------
void __fastcall TformSpecialPersonNotesSummary::FormShow(TObject *Sender)
{
  TformSpecialGeneric::FormShow(Sender);

  string sql = "select cast(PN.last_update_stamp as date), PN.note, WT.item, PN.current_id, PN.id "
               "from person_notes PN "
               "inner join warning_types WT on PN.warning_id = WT.id "
               "where PN.person_id = '" + person_id + "' and " + SafeInClause("PN.group_id", group_ids) + " "
               "and PN.warning_id <> 0 and PN.current_id+1 > 3 ";

  filter_stuff.SetAddEdit(this, NULL, EditRecord(&(TformSpecialPersonNotesSummary::Zoom)));
  filter_stuff.SetSQLStatement(sql);
  filter_stuff.RefreshDataSet();

  lblInformation->Width = scrollStandard->Width - 48; // Stop over-growing
  InfoBarText = "Select the blocks you want to override. All blocks must be overridden before you can save.";
  ShowInfoBar();
}
//---------------------------------------------------------------------------
void __fastcall TformSpecialPersonNotesSummary::FormClose(TObject *Sender, TCloseAction &Action)
{
  vector<string> selected_ids = filter_stuff.GetMultiID();
  string sql = "select count(id) from person_notes where warning_id = '" + lookupTables->IdOf("warning_types", "Warn") + "' "
               "and " + SafeInClause("id", selected_ids);
  int num_warns = StringToInt(GetOneField(dbTransaction, sql));

  // Return the number that were selected minus the number that are just warnings.
  num_checks = selected_ids.size() - num_warns;

  TformSpecialGeneric::FormClose(Sender, Action);
}
//---------------------------------------------------------------------------
// Not so useless here. For speed, because otherwise you gotta right-click and navigate to a submenu to do this, and I
// have a feeling they'll do this most of the time 9_9
void __fastcall TformSpecialPersonNotesSummary::buttonUselessClick(TObject *Sender)
{
  filter_stuff.CheckAll();
}
//---------------------------------------------------------------------------
bool TformSpecialPersonNotesSummary::Zoom(const string &p_id)
{
  TformRecordPersonNotes *person_note = NULL;
  map<string,string> temp_map;
  temp_map["id"] = p_id;
  FORM_TRY
    person_note = new TformRecordPersonNotes(NULL, temp_map);
  FORM_CATCH
  person_note->Show();
}
//---------------------------------------------------------------------------
