//---------------------------------------------------------------------------

#include <vcl.h>
#pragma hdrstop

#include "codeSpecialPrometheusVersionLog.h"
//---------------------------------------------------------------------------
#pragma package(smart_init)
#pragma link "codeSpecialGeneric"
#pragma resource "*.dfm"
TformSpecialPrometheusVersionLog *formSpecialPrometheusVersionLog;
//---------------------------------------------------------------------------
__fastcall TformSpecialPrometheusVersionLog::TformSpecialPrometheusVersionLog(TComponent* Owner,
                                                                              bool p_show_new_version_notification)
        : TformSpecialGeneric(Owner), version_to_notes(), class_name()
{
  IconIndex = 19;
  class_name = "TformSpecialPrometheusVersionLog";

  allow_close = false;
  understands_current_release_notes = false;

  show_new_version_notification = p_show_new_version_notification;

  // jhaley 20110531: Moved extended initialization calls to FormShow
}
//---------------------------------------------------------------------------
void TformSpecialPrometheusVersionLog::UpdateList()
{
  TIBTransaction *dbTransaction = NULL;
  FORM_TRY
    dbTransaction = new TIBTransaction(NULL);
  FORM_CATCH

  TIBDataSet *dbDataset = NULL;
  FORM_TRY
    dbDataset = new TIBDataSet(NULL);
  FORM_CATCH

  // jhaley: got rid of hack loops and replaced with FORM_TRY above, and added this safety check: don't deliberately
  // allow an exception to be generated just because the above failed somehow.
  if(!dbTransaction || !dbDataset)
  {
    PSay("VCL refuses to create a transaction or dataset to retrieve version information.", m_program_error, MB_OK);
    return; // sorry >_>
  }

  try
  {
    StdTransaction(dbTransaction, dbDatabase);
    dbDataset->Database       = dbDatabase;
    dbDataset->Transaction    = dbTransaction;
    dbDataset->UniDirectional = true;

    string sql_get_comments =
      "select version_number, cast(last_update_stamp as date) version_date, comments "
      "from prometheus_update2 "
      "order by version_number1 desc, version_number2 desc, version_number3 desc";
    dbDataset->SelectSQL->Clear();
    dbDataset->SelectSQL->Add(sql_get_comments.c_str());
    dbDataset->Open();
    dbDataset->First();

    TListItem * row;
    listVersions->Items->BeginUpdate();
    listVersions->Items->Clear();

    while(!dbDataset->Eof)
    {
      ((TBlobField *)dbDataset->FieldByName("comments"))->SetFieldType(ftBlob);
      row = listVersions->Items->Add();
      row->Caption = dbDataset->FieldByName("version_number")->AsString;
      row->SubItems->Add(dbDataset->FieldByName("version_date")->AsString);

      // codeSpecialUploadVersion should soon stop uppercasing stuff. it's annoying. and ugly. and it's our data, so I
      // don't give a flip about users who are stupid with caps-lock.
      string comments = (((TBlobField *)dbDataset->FieldByName("comments"))->AsString.c_str());
      // check for all-uppercase ...
      if (comments.find_first_of(alphas) == string::npos)
        comments = SentenceString(comments);
      version_to_notes[dbDataset->FieldByName("version_number")->AsString.c_str()] = comments;

      dbDataset->Next();
    }
    listVersions->Items->EndUpdate();

    string current_version = listVersions->Items->Item[0]->Caption.c_str();
    string confirmation_id =
    GetOneField(dbDatabase,
      "select id, message_text from message_box_log "
      "where form_name = 'TformSpecialPrometheusVersionLog' and message_box_log.message_box_id = "
      + lookupTables->IdOf("message_box_types", "Confirmation") +
      " and message_text = '" + current_fullname + " read and understood release notes for version "
      + current_version + ".0'");

    if(confirmation_id != "")
    {
      // then they've already confirmed they understood this version's release notes; no reason to do it again.
      understands_current_release_notes = true;
      btnAcknowledge->Caption = "Close";
      btnMisunderstood->Visible = false;
      sep1->Visible = false;
      btnIgnore->Visible = false;
      sep2->Visible = false;
    }

    dbDataset->Close();
    dbTransaction->Commit();
  }
  catch(...)
  {
    if(dbTransaction && dbTransaction->Active)
      dbTransaction->Rollback();
    PSay("Error retrieving list of versions.", m_database_error, MB_OK);
  }

  listVersions->Selected = listVersions->Items->Item[0];

  delete dbDataset;
  delete dbTransaction;
}
//---------------------------------------------------------------------------
void __fastcall TformSpecialPrometheusVersionLog::listVersionsChange(TObject *Sender, TListItem *Item, TItemChange Change)
{
  // update right side!
  if(listVersions->Selected)
    editNotes->Text = version_to_notes[listVersions->Selected->Caption.c_str()].c_str();
  else
    editNotes->Text = "";
}
//---------------------------------------------------------------------------
void __fastcall TformSpecialPrometheusVersionLog::btnAcknowledgeClick(TObject *Sender)
{
  if(!understands_current_release_notes) // they didn't before, but they do now
    PPretendToSay(current_fullname + " read and understood release notes for version " + PGlobals->current_version, m_confirmation);
  allow_close = true;
  Close();
}
//---------------------------------------------------------------------------
void __fastcall TformSpecialPrometheusVersionLog::btnMisunderstoodClick(TObject *Sender)
{
  PPretendToSay(current_fullname + " did not understand release notes for version " + PGlobals->current_version, m_confirmation);
  allow_close = true;
  Close();
}
//---------------------------------------------------------------------------
void __fastcall TformSpecialPrometheusVersionLog::btnIgnoreClick(TObject *Sender)
{
  PPretendToSay(current_fullname + " is ignoring release notes for version " + PGlobals->current_version, m_confirmation);
  PSay("You may return here to read the latest version notes at a later time by selecting \"File\"->\"Version notes\" "
       "from the main menu.", m_reminder, MB_OK);
  allow_close = true;
  Close();
}
//---------------------------------------------------------------------------
void __fastcall TformSpecialPrometheusVersionLog::FormCloseQuery(TObject *Sender, bool &CanClose)
{
  // jhaley: Can't do this; if Prometheus tries to auto-shutdown with this form still open, things blow up big time.
  //CanClose = allow_close;
}
//---------------------------------------------------------------------------
void __fastcall TformSpecialPrometheusVersionLog::FormShow(TObject *Sender)
{
  TformSpecialGeneric::FormShow(Sender);

  // jhaley 20110531: Moved extended initialization to FormShow, as form constructor is too early to be doing listview
  // manipulations, and *especially* messing with the Update methods which control drawing.
  UpdateList();

  if(show_new_version_notification)
  {
    PSay("A new version has been downloaded.  Please read the version notes for changes since the last version.\n\n"
         "If any of the changes listed affect you, or if you are unsure what impact the changes may have on your job "
         "functions, contact a programmer for any necessary training.", m_reminder, MB_OK);
  }
}
//---------------------------------------------------------------------------

