//---------------------------------------------------------------------------

#include <vcl.h>
#pragma hdrstop

#include "codeSpecialQuery.h"
#include "db_hardcoded_metadata.h"
#include "query_load_save.h"
//---------------------------------------------------------------------------
#pragma package(smart_init)
#pragma link "codeSpecialGeneric"
#pragma resource "*.dfm"
TformSpecialQuery *formSpecialQuery;
//---------------------------------------------------------------------------
__fastcall TformSpecialQuery::TformSpecialQuery(TComponent* Owner, string source_table_name)
        : TformSpecialGeneric(Owner)
{
 IconIndex = 36;

 FSourceTableName = source_table_name;

 FORM_TRY
  context = new query_context(dbDatabase);
  context->Transaction->Commit();
 FORM_CATCH

 floating_log_window = NULL;
 labelStats->Caption = "";
 editLog->Visible = false;

 if (optionsPermissions["special.debug_query_action"]["enabled"] == "true")
 {
  editLog->Visible = true;
  context->LogOutput = editLog;
  FORM_TRY
   floating_log_window = new TForm((TComponent*)NULL);
  FORM_CATCH
  floating_log_window->BorderStyle = bsSizeToolWin;
  floating_log_window->AutoSize = false;
  editLog->Align = alClient;
  editLog->HostDockSite = floating_log_window;
  floating_log_window->Width = 300;
  floating_log_window->Height = Screen->Height;
  floating_log_window->Left = 0;
  floating_log_window->Top = 0;
  floating_log_window->Caption = "Query debugging log";
  floating_log_window->Visible = true;
 }

 _labelSource->Caption = ("Source (" + LowercaseString(dbPrettyPlural(LowercaseString(FSourceTableName))) + ")").c_str();

/*
 (*(context->Globals))["as_of_date"] = xFromPdate(Pdate("11/12/1980", global_mode));
 (*(context->Globals))["start_date"] = xFromPdate(Pdate("01/01/2004", global_mode));
 (*(context->Globals))["stop_date"]  = xFromPdate(Pdate("12/31/2004", global_mode));
*/

 FORM_TRY
  query_container = new TframeQueryContainer(this);
  query_container->Parent = panelQuery;
  query_container->Align  = alClient;
  query_container->Context = context;
  query_container->SourceTableName = FSourceTableName.c_str();
 FORM_CATCH

 FORM_TRY
  action_container = new TframeActionContainer(this);
  action_container->Parent = panelActions;
  action_container->Align  = alClient;
  action_container->SourceTableName = FSourceTableName.c_str();
 FORM_CATCH
}
//---------------------------------------------------------------------------
__fastcall TformSpecialQuery::~TformSpecialQuery()
{
 delete context;
}
//---------------------------------------------------------------------------
void __fastcall TformSpecialQuery::buttonCloseClick(TObject *Sender)
{
 Close();
}
//---------------------------------------------------------------------------
string PrettyTime (int seconds)
{
 int s = seconds;
 int m = seconds / 60;
 int h = seconds / (24 * 60);

 if (seconds < 60)
  return IntToString(s) + " second" + string(s == 1 ? "" : "s");
 else if (seconds < 60*60)
  return IntToString(m) + " minute" + string(m == 1 ? "" : "s") + " and " + PrettyTime(seconds % 60);
 else
  return IntToString(h) + " hour" + string(h == 1 ? "" : "s") + ", " + PrettyTime(seconds % (24 * 60));
}
//---------------------------------------------------------------------------
void __fastcall TformSpecialQuery::buttonRunClick(TObject *Sender)
{
 labelStats->Caption = "";

 // just enough to get the goddamn date controls to give up their values.
 FocusAway();
 FocusBack();

 // make sure the query is valid
 if (!query_container->Valid())
  return;

 // make sure every action is valid
 if (!action_container->Valid())
  return;

 Processing(true);
 try
 {
  editLog->Lines->Clear();
  if (floating_log_window)
   floating_log_window->BringToFront();

  Ptime A, B, C, D, E;

  A = Ptime(TDateTime::CurrentTime());

  context->Transaction->StartTransaction();
  context->Clear();

  B = Ptime(TDateTime::CurrentTime());
  editLog->Lines->Add((B.OutTime(global_mode) + ": Starting preparation.").c_str());

  // prepare the query
  // if (!action_container->Single())
  query_container->Proxy = true;
  query_container->Clip = true; // query-generic *was* setting this if there was no HostQuery ... interferes elsewhere.
  string resulting_view = query_container->Prepare();

  C = Ptime(TDateTime::CurrentTime());
  editLog->Lines->Add((C.OutTime(global_mode) + ": Done preparing, starting actions.").c_str());

  // run every action
  if (!action_container->Execute(context->Transaction, resulting_view))
   throw(0);

  D = Ptime(TDateTime::CurrentTime());
  editLog->Lines->Add((D.OutTime(global_mode) + ": Actions completed, cleaning up.").c_str());

  // clean up the query
  query_container->CleanUp();
  context->Clear();
  context->Transaction->Commit();

  E = Ptime(TDateTime::CurrentTime());
  editLog->Lines->Add((E.OutTime(global_mode) + ": Done cleaning up.").c_str());

  labelStats->Caption = ("Query completed in " + PrettyTime(C - A) + ", actions in " + PrettyTime(E - C)).c_str();
 } catch (...)
 {
  // clean up the query
  query_container->CleanUp();
  context->Clear();
  context->Transaction->Rollback();
  PSay("An error occured, and any actions intended to modify the database were cancelled.\nThis is not your fault, please contact a programmer.", m_program_error, MB_OK);
 }
 Processing(false);
}
//---------------------------------------------------------------------------
void __fastcall TformSpecialQuery::FormShow(TObject *Sender)
{
 TformSpecialGeneric::FormShow(Sender);
 //query_container->Color = AverageColor(StringToColor("clBtnFace"), StringToColor("clWhite"));
 query_container->Color = AverageColor(clBtnFace, clWindow);
}
//---------------------------------------------------------------------------
void __fastcall TformSpecialQuery::buttonDescribeClick(TObject *Sender)
{
 if (query_container && query_container->Valid())
  query_container->Describe(true);
}
//---------------------------------------------------------------------------
void __fastcall TformSpecialQuery::FormClose(TObject *Sender,
      TCloseAction &Action)
{
 if (floating_log_window)
 {
  editLog->Visible = false;
  editLog->HostDockSite = this;
  delete floating_log_window;
 }
 TformSpecialGeneric::FormClose(Sender, Action);
}
//---------------------------------------------------------------------------
void __fastcall TformSpecialQuery::buttonSaveAllClick(TObject *Sender)
{
 // TODO: this could be moved to a Save() function, eh? or should we create a TframeQueryAndAction to embed, and have -it- have Save() and Load()? woah. modular! like a ... singular ... pendular ... calendar ... *sigh*
 if (query_container->Valid() && action_container->Valid())
 {
  map<string, xValue> config_map;
  config_map["source_table_name"] = xFromString(FSourceTableName);
  config_map["query"] = query_container->Save(true);
  config_map["actions"] = action_container->Save(true);
  xValue combined_value = xVal("QueryAndActions", xFromMap(config_map));
  QuerySave(FSourceTableName.c_str(), qf_query_actions, combined_value);
 }
}
//---------------------------------------------------------------------------
void __fastcall TformSpecialQuery::buttonLoadAllClick(TObject *Sender)
{
 xValue possibly_loaded = QueryLoad(FSourceTableName.c_str(), qf_query_actions);
 if (!possibly_loaded.empty())
 {
  if (xTag(possibly_loaded) == "QueryAndActions")
  {
   xValue config = xContent(possibly_loaded);
   if (xTag(config) == "map")
   {
    map<string, xValue> config_map = xToMap(config);
    if (xTag(config_map["source_table_name"]) == "string")
    {
     string attempted_stn = xToString(config_map["source_table_name"]);
     if (FSourceTableName != attempted_stn.c_str())
     {
      PSay("The query and action combo you are attempting to load is of the wrong type;\nit is intended to find (and act on) " + dbPrettyPlural(LowercaseString(attempted_stn)) + " rather than " + dbPrettyPlural(LowercaseString(FSourceTableName.c_str())) + ".", m_invalid_data, MB_OK);
      return;
     }
     query_container->Load(config_map["query"], true);
     action_container->Load(config_map["actions"], true);
    } else
     PSay("The query and action combo you are attempting to load does not have a type;\nit seems to have been corrupted.", m_invalid_data, MB_OK);
   } else
    PSay("Parameters being loaded are of an unexpected format: no property map.", m_invalid_data, MB_OK, config);
  } else
   PSay("Unable to restore query and action combo, something's wrong.", m_invalid_data, MB_OK);
 }
}
//---------------------------------------------------------------------------

