//---------------------------------------------------------------------------
#include <vcl.h>
#pragma hdrstop

#include "codeSpecialRelationshipGraph.h"
#include "codeRecordRelationship.h"
#include "codeSpecialHouseholdMod.h"
#include "derivedheaders.h"
//---------------------------------------------------------------------------
#pragma package(smart_init)
#pragma link "codeSpecialGeneric"
#pragma link "codeSpecialGeneric"
#pragma resource "*.dfm"
TformSpecialRelationshipGraph *formSpecialRelationshipGraph;
//---------------------------------------------------------------------------
__fastcall TformSpecialRelationshipGraph::TformSpecialRelationshipGraph(TComponent* Owner, string p_person, bool is_family)
        : TformSpecialGeneric(Owner)
{
 is_family_graph = is_family;
 num_nodes = 0;
 num_edges = 0;
 num_nodes_shown = 0;
 num_edges_shown = 0;
 selected_node      = NULL;
 dragging_node      = NULL;
 prev_dragging_node = NULL;
 dragging_screen    = false;
 distance_dragged   = (dimension)0.00;
 zoom_level         = (dimension)1.00;
 FetchData (p_person);
 UpdateNums();
}
//---------------------------------------------------------------------------
bool TformSpecialRelationshipGraph::AtlasScreen (int p_request_num, string matched_value)
{
 FetchData (current_person);
 return true;
}
//---------------------------------------------------------------------------
void TformSpecialRelationshipGraph::FetchData (string p_person)
{
 if (is_family_graph)
  FetchFamilyData(p_person);
 else
  FetchAllData(p_person);
}
//---------------------------------------------------------------------------
void TformSpecialRelationshipGraph::FetchFamilyData (string p_person)
{
 timerRefresh->Enabled = false;
 timerUpdate->Enabled = false;
 RemoveObjects ();

 extended_node<3> * to_be_selected   = NULL;

 map<string, relationship_creator<numbered_string_generator, string> * > relationship_to_handler;

 numbered_string_generator * gen = new numbered_string_generator("unknown", 0);
 relationship_creator<numbered_string_generator, string> * rel_parent_child = new relationship_parent_child<numbered_string_generator, string>(false, *gen);
 relationship_creator<numbered_string_generator, string> * rel_child_parent = new relationship_parent_child<numbered_string_generator, string>(true, *gen);
 relationship_creator<numbered_string_generator, string> * rel_full_sibling = new relationship_full_sibling<numbered_string_generator, string>(false, *gen);
 relationship_creator<numbered_string_generator, string> * rel_grand_parent_child = new relationship_grand_parent_child<numbered_string_generator, string>(false, *gen);
 relationship_creator<numbered_string_generator, string> * rel_grand_child_parent = new relationship_grand_parent_child<numbered_string_generator, string>(true, *gen);
 relationship_creator<numbered_string_generator, string> * rel_uncle_nephew = new relationship_uncle_nephew<numbered_string_generator, string>(false, *gen);
 relationship_creator<numbered_string_generator, string> * rel_nephew_uncle = new relationship_uncle_nephew<numbered_string_generator, string>(true, *gen);
 relationship_creator<numbered_string_generator, string> * rel_cousin = new relationship_cousin<numbered_string_generator, string>(false, *gen);
 relationship_creator<numbered_string_generator, string> * rel_half_sibling = new relationship_half_sibling<numbered_string_generator, string>(false, *gen);
 relationship_creator<numbered_string_generator, string> * rel_grand_uncle_nephew = new relationship_grand_uncle_nephew<numbered_string_generator, string>(false, *gen);
 relationship_creator<numbered_string_generator, string> * rel_grand_nephew_uncle = new relationship_grand_uncle_nephew<numbered_string_generator, string>(true, *gen);

 relationship_to_handler[lookupTables->IdOf("relationship_types", "Mother")]   = rel_parent_child;
 relationship_to_handler[lookupTables->IdOf("relationship_types", "Father")]   = rel_parent_child;
 relationship_to_handler[lookupTables->IdOf("relationship_types", "Daughter")] = rel_child_parent;
 relationship_to_handler[lookupTables->IdOf("relationship_types", "Son")]      = rel_child_parent;

/*
  relationship_to_handler[lookupTables->IdOf("relationship_types", "Foster Parent")] = rel_parent_child;
  relationship_to_handler[lookupTables->IdOf("relationship_types", "Foster Child")]  = rel_child_parent;
*/

 relationship_to_handler[lookupTables->IdOf("relationship_types", "Brother")]      = rel_full_sibling;
 relationship_to_handler[lookupTables->IdOf("relationship_types", "Sister")]       = rel_full_sibling;
 relationship_to_handler[lookupTables->IdOf("relationship_types", "Half Brother")] = rel_half_sibling;
 relationship_to_handler[lookupTables->IdOf("relationship_types", "Half Sister")]  = rel_half_sibling;

 relationship_to_handler[lookupTables->IdOf("relationship_types", "Grandmother")]   = rel_grand_parent_child;
 relationship_to_handler[lookupTables->IdOf("relationship_types", "Grandfather")]   = rel_grand_parent_child;
 relationship_to_handler[lookupTables->IdOf("relationship_types", "Granddaughter")] = rel_grand_child_parent;
 relationship_to_handler[lookupTables->IdOf("relationship_types", "Grandson")]      = rel_grand_child_parent;

 relationship_to_handler[lookupTables->IdOf("relationship_types", "Aunt")]   = rel_uncle_nephew;
 relationship_to_handler[lookupTables->IdOf("relationship_types", "Uncle")]  = rel_uncle_nephew;
 relationship_to_handler[lookupTables->IdOf("relationship_types", "Niece")]  = rel_nephew_uncle;
 relationship_to_handler[lookupTables->IdOf("relationship_types", "Nephew")] = rel_nephew_uncle;

 relationship_to_handler[lookupTables->IdOf("relationship_types", "Cousin")] = rel_cousin;

 relationship_to_handler[lookupTables->IdOf("relationship_types", "Great Aunt")]   = rel_uncle_nephew;
 relationship_to_handler[lookupTables->IdOf("relationship_types", "Great Uncle")]  = rel_uncle_nephew;
 relationship_to_handler[lookupTables->IdOf("relationship_types", "Great Niece")]  = rel_nephew_uncle;
 relationship_to_handler[lookupTables->IdOf("relationship_types", "Great Nephew")] = rel_nephew_uncle;

 TIBTransaction * dbTransaction = new TIBTransaction(NULL);
 try
 {
  StdTransaction(dbTransaction, dbDatabase);

  set<string> people_asked_about;
  vector<string> people_to_ask_about;
  people_to_ask_about.push_back(p_person);

  map<string, set<string> > child_to_parents;

  while (!people_to_ask_about.empty())
  {
   vector<map<string, string> > result_map;
   string sql = "select relationships.person_a_id, relationships.person_b_id, relationships.rel_a_to_b_id, relationships.rel_b_to_a_id "
                "from relationships where "
                "current_id > 2 and "
                "(person_a_id in (" + VecToCommaString(people_to_ask_about) + ") or "
                " person_b_id in (" + VecToCommaString(people_to_ask_about) + "))";
   if (!SqlToVecMap(dbTransaction, sql, result_map))
    throw(1);

   people_asked_about.insert(people_to_ask_about.begin(), people_to_ask_about.end());
   people_to_ask_about.clear();

   for (vector<map<string, string> >::iterator i_link = result_map.begin(); i_link != result_map.end(); i_link++)
   {
    if(debug_level[other_forms]>=2 && message_router) message_router->SendMessageToAll("debug", "FetchFamilyData: a relationship ...");
    bool is_family_rel = false;

    // see if it's a family one or not
    if (relationship_to_handler.find((*i_link)["rel_a_to_b_id"]) != relationship_to_handler.end() ||
        relationship_to_handler.find((*i_link)["rel_b_to_a_id"]) != relationship_to_handler.end())
     is_family_rel = true;

    if (is_family_rel)
    {
     if(debug_level[other_forms]>=2 && message_router) message_router->SendMessageToAll("debug", "FetchFamilyData: family!");
     // prepare to ask about people, yay!
     if (people_asked_about.find((*i_link)["person_a_id"]) == people_asked_about.end())
      people_to_ask_about.push_back((*i_link)["person_a_id"]);
     if (people_asked_about.find((*i_link)["person_b_id"]) == people_asked_about.end())
      people_to_ask_about.push_back((*i_link)["person_b_id"]);

     // mess with it somehow
     if (relationship_to_handler.find((*i_link)["rel_a_to_b_id"]) != relationship_to_handler.end())
      // a_to_b defined
      (*(relationship_to_handler[(*i_link)["rel_a_to_b_id"]]))((*i_link)["person_a_id"], (*i_link)["person_b_id"], child_to_parents);
     else
      // fall back to b_to_a instead
      (*(relationship_to_handler[(*i_link)["rel_b_to_a_id"]]))((*i_link)["person_b_id"], (*i_link)["person_a_id"], child_to_parents);
    }
   }
  }

  // make sure the main person is in there, even if alone.
  people_asked_about.insert(p_person);
  if (child_to_parents.find(p_person) == child_to_parents.end())
   child_to_parents.insert(pair<string, set<string> >(p_person, set<string>()));


  // get actual names for people
  map<string, string> id_to_name;
  if (!SqlToValueMap(dbTransaction, "select people.id, person_names.first_name || ' ' || person_names.last_name || ' ' || person_names.suffix from people left join person_names on person_names.id = people.person_names_id where people.id in (" + SetToCommaString(people_asked_about) + ")", id_to_name))
   throw(2);

//  #ifdef use_tcpip_messaging
//  if(debug_level[other_forms]>=8 && message_router) message_router->SendMessageToAll("debug", "codeSpecialRelationshipsGraph::people ValueMap (id_to_name) is empty?  " + IntToBoolString(BoolToInt(id_to_name.empty())) + " ");
//  #endif

  // child_to_parents should now be nicely initialized?
  // we should do something with it.
  vector<set<string> > dep_graph;
  if (!dependency_graph(child_to_parents, dep_graph))
  {
   PSay("A cycle of some sort was found (someone being his/her own ancestor, for example.)\nProcessing halted.", m_invalid_data, MB_OK);
   throw(3);
  }

  // graph / nodes / tensors start here, hurray!
  map<string, extended_node<3> *> id_to_node_pointer;

  extended_node<3> * created_node     = NULL;
  extended_tensor<3> * created_tensor = NULL;

  num_nodes = 0;
  num_edges = 0;
  num_nodes_shown = 0;
  num_edges_shown = 0;

  dimension current_level = 0.00;
  for (vector<set<string> >::iterator i_level = dep_graph.begin(); i_level != dep_graph.end(); i_level++)
  {
   if(debug_level[other_forms]>=2 && message_router) message_router->SendMessageToAll("debug", "FetchFamilyData: a level of the dep graph ...");
   for (set<string>::iterator i_person = i_level->begin(); i_person != i_level->end(); i_person++)
   {
    atlas->RequestNotificationWhen("relationships.person_a_id", *i_person, rel_e, as_int, this, RefreshHandler(&(TformSpecialRelationshipGraph::AtlasScreen)));
    atlas->RequestNotificationWhen("relationships.person_b_id", *i_person, rel_e, as_int, this, RefreshHandler(&(TformSpecialRelationshipGraph::AtlasScreen)));

    created_node = new extended_node<3>(t_vector<3>(3, ((rand() / (RAND_MAX + 0.0)) * panelGraph->Width), current_level, ((rand() / (RAND_MAX + 0.0)) * 10.00)),
                                     std_mass, std_drag, std_minv, 4500 /* TODO -c MAGIC : */, false,
                                     *i_person, PrettyString(id_to_name[*i_person]), "",
                                     panelGraph, &LabelMouseDown, &LabelMouseMove, &LabelMouseUp,
                                     &visual_center, &zoom_level, true);
    created_node->SetYConstraint(true, current_level);
    num_nodes++;
    num_nodes_shown++;
    id_to_node_pointer[*i_person] = created_node;
    physical_simulation.nodes.insert(physical_simulation.nodes.end(), created_node);
    if(debug_level[other_forms]>=2 && message_router) message_router->SendMessageToAll("debug", "FetchFamilyData: new node");
    if (*i_person == p_person)
     to_be_selected = created_node;
   }
   current_level += 200; // TODO -c MAGIC :
  }

  for (map<string, set<string> >::iterator i_person = child_to_parents.begin(); i_person != child_to_parents.end(); i_person++)
  {
   for (set<string>::iterator i_dep = i_person->second.begin(); i_dep != i_person->second.end(); i_dep++)
   {
    created_tensor = new extended_tensor<3>((node_base<3> *)(id_to_node_pointer[i_person->first]),
                                         (node_base<3> *)(id_to_node_pointer[*i_dep]),
                                         std_tens, std_size,
                                         "", "", "", false,
                                         panelGraph, &PaintClick, &PaintPaint, &PaintMouseDown, &PaintMouseMove, &PaintMouseUp,
                                         &visual_center, &zoom_level, true);
    created_tensor->SetIsDiagonal(false);
    num_edges++;
    num_edges_shown++;
    physical_simulation.tensors.insert(physical_simulation.tensors.end(), created_tensor);
    if(debug_level[other_forms]>=2 && message_router) message_router->SendMessageToAll("debug", "FetchFamilyData: new tensor!");
   }
  }
  // graph / nodes / tensors end here. awww.

  dbTransaction->Commit();
 }
 catch (...)
 {
  // not like it matters which way we go ...
  PSay("Failed to load list of useful relationships.", m_database_error, MB_OK);
  dbTransaction->Rollback();
 }
 delete dbTransaction;

 delete gen;
 delete rel_parent_child;
 delete rel_child_parent;
 delete rel_full_sibling;
 delete rel_grand_parent_child;
 delete rel_grand_child_parent;
 delete rel_uncle_nephew;
 delete rel_nephew_uncle;
 delete rel_cousin;
 delete rel_half_sibling;
 delete rel_grand_uncle_nephew;
 delete rel_grand_nephew_uncle;

 ChangeFocus(to_be_selected, true);
 timerUpdate->Enabled = true;
 timerRefresh->Enabled = true;
}
//---------------------------------------------------------------------------
void TformSpecialRelationshipGraph::FetchAllData (string p_person)
{
 timerRefresh->Enabled = false;
 timerUpdate->Enabled = false;
 RemoveObjects ();

 // fetch data
 vector<string> found_people;
 found_people.push_back(p_person);

 TIBTransaction * dbTransaction = new TIBTransaction(NULL);
 try
 {
   StdTransaction(dbTransaction, dbDatabase);

   TformSpecialHouseholdMod::FindRelations(dbTransaction, found_people, true);

   string sql_get_person_info;
   string sql_get_relationship_info;
   string comma_person_id =  "select distinct person_id from temp_relationship_data";  // VecToCommaString(found_people);

   sql_get_person_info = "select people.id, "
                         "person_names.first_name || ' ' || person_names.last_name || ' ' || person_names.suffix name "
                         "from people left join person_names on people.person_names_id = person_names.id "
                         "where people.id in (" + comma_person_id + ")";
   sql_get_relationship_info = "select relationships.id, "
                               "relationships.person_a_id, relationships.a_is_emergency, a_is_banned, a_is_responsible, rel_a.item a_is, "
                               "relationships.person_b_id, relationships.b_is_emergency, b_is_banned, b_is_responsible, rel_b.item b_is, "
                               "relationships.same_household "
                               "from relationships "
                               "left join relationship_types rel_a on relationships.rel_a_to_b_id = rel_a.id "
                               "left join relationship_types rel_b on relationships.rel_b_to_a_id = rel_b.id "
                               "where relationships.current_id > 2 "
                               "and relationships.person_a_id in (" + comma_person_id + ") "
                               "and relationships.person_b_id in (" + comma_person_id + ") ";
   vector<map<string, string> > data_person_info;
   vector<map<string, string> > data_relationship_info;
   map<string, extended_node<3> *> id_to_node_pointer;

   if (!SqlToVecMap(dbTransaction, sql_get_person_info, data_person_info))
    PSay("Could not retrieve people", m_database_error, MB_OK);

   #ifdef use_tcpip_messaging
   if(debug_level[other_forms]>=8 && message_router) message_router->SendMessageToAll("debug", "codeSpecialRelationshipsGraph::people VecMap (data_person_info) is empty?  " + IntToBoolString(BoolToInt(data_person_info.empty())) + " ");
   #endif

   if (!SqlToVecMap(dbTransaction, sql_get_relationship_info, data_relationship_info))
    PSay("Could not retrieve relationships", m_database_error, MB_OK);

   #ifdef use_tcpip_messaging
   if(debug_level[other_forms]>=8 && message_router) message_router->SendMessageToAll("debug", "codeSpecialRelationshipsGraph::relationships VecMap (data_relationship_info) is empty?  " + IntToBoolString(BoolToInt(data_relationship_info.empty())) + " ");
   #endif

   /* TODO : sort data_relationship_info such that tensors between "early" nodes will be shown first ...
            found_people should be in order of "appearance"
            for each tensor, take the relative order of each node mentioned, and keep the minimum value
            then compare two tensors by the minimum value */

   // first pass, create the lookup thingie
   map<string, int> person_order;
   int i_order = 0;
   for (vector<string>::iterator i_entry = found_people.begin(); i_entry != found_people.end(); i_entry++, i_order++)
    person_order[*i_entry] = i_order;

   // second pass, use lookup thingie to get the minimum order for the tensor, store items by that order ...
   multimap<int, map<string, string> > tensor_order;
   for (vector<map<string, string> >::iterator i_tensor = data_relationship_info.begin(); i_tensor != data_relationship_info.end(); i_tensor++)
    tensor_order.insert(pair<int, map<string, string> >(min(person_order[(*i_tensor)["person_a_id"]], person_order[(*i_tensor)["person_b_id"]]), (*i_tensor)));

   string about_a;
   string about_b;
   extended_node<3> * created_node     = NULL;
   extended_node<3> * to_be_selected   = NULL;
   extended_tensor<3> * created_tensor = NULL;

   bool make_bold = false;
   num_nodes = 0;
   num_edges = 0;
   num_nodes_shown = 0;
   num_edges_shown = 0;

   for (vector<map<string, string> >::iterator i_person = data_person_info.begin();
        i_person != data_person_info.end(); i_person ++)
   {
    // while we're at it, register with atlas, eh? for both person_a_id and person_b_id;
    atlas->RequestNotificationWhen( "relationships.person_a_id", (*i_person)["id"], rel_e, as_int, this, RefreshHandler(&(TformSpecialRelationshipGraph::AtlasScreen)));
    atlas->RequestNotificationWhen( "relationships.person_b_id", (*i_person)["id"], rel_e, as_int, this, RefreshHandler(&(TformSpecialRelationshipGraph::AtlasScreen)));

    created_node = new extended_node<3>(t_vector<3>(3, ((rand() / (RAND_MAX + 0.0)) * panelGraph->Width),
                                                 ((rand() / (RAND_MAX + 0.0)) * panelGraph->Height),
                                                 ((rand() / (RAND_MAX + 0.0)) * 10.00)),
                                     std_mass, std_drag, std_minv, std_elec, false,
                                     (*i_person)["id"], PrettyString((*i_person)["name"]), "",
                                     panelGraph, &LabelMouseDown, &LabelMouseMove, &LabelMouseUp,
                                     &visual_center, &zoom_level, false);
    num_nodes++;
    id_to_node_pointer[(*i_person)["id"]] = created_node;
    physical_simulation.nodes.insert(physical_simulation.nodes.end(), created_node);
    if ((*i_person)["id"] == p_person)
    {
     to_be_selected = created_node;
     to_be_selected->SetExists(true);
     num_nodes_shown++;
    }
   }

   for (multimap<int, map<string, string> >::iterator i_rel = tensor_order.begin(); i_rel != tensor_order.end(); i_rel++)
   {
    if (i_rel->second["same_household"] == "1")
     make_bold = true;
    else
     make_bold = false;

    about_a = PrettyString(i_rel->second["a_is"]);
    about_a += (i_rel->second["a_is_emergency"] == "1") ? ", Emerg." : "";
    about_a += (i_rel->second["a_is_responsible"] == "1") ? ", Resp." : "";
    about_a += (i_rel->second["a_is_banned"] == "1") ? ", Ban." : "";

    about_b = PrettyString(i_rel->second["b_is"]);
    about_b += (i_rel->second["b_is_emergency"] == "1") ? ", Emerg." : "";
    about_b += (i_rel->second["b_is_responsible"] == "1") ? ", Resp." : "";
    about_b += (i_rel->second["b_is_banned"] == "1") ? ", Ban." : "";

    created_tensor = new extended_tensor<3>((node_base<3> *)(id_to_node_pointer[i_rel->second["person_a_id"]]),
                                         (node_base<3> *)(id_to_node_pointer[i_rel->second["person_b_id"]]),
                                         std_tens, std_size,
                                         i_rel->second["id"], about_a, about_b, make_bold,
                                         panelGraph, &PaintClick, &PaintPaint, &PaintMouseDown, &PaintMouseMove, &PaintMouseUp,
                                         &visual_center, &zoom_level, false);
    num_edges++;
    physical_simulation.tensors.insert(physical_simulation.tensors.end(), created_tensor);
   }

   ChangeFocus(to_be_selected, true);
   timerUpdate->Enabled = true;
   timerRefresh->Enabled = true;
 } // not sure where to end the try block so... here?
 catch (...)
 {
  PSay("Failed to do something or other.", m_database_error, MB_OK);
  dbTransaction->Rollback();
 }
 if (dbTransaction->Active)
   dbTransaction->Commit();
 delete dbTransaction;
}
//---------------------------------------------------------------------------
void TformSpecialRelationshipGraph::ChangeFocus (const extended_node<3> * new_focus, bool p_follow)
{
 #ifdef use_tcpip_messaging
 if(debug_level[libraries]>=8 && message_router) message_router->SendMessageToAll("debug", "TformSpecialRelationshipGraph::ChangeFocus(...) entry");
 #endif
 if (!new_focus)
  return;

 current_person = new_focus->GetID();
 CreateLink(staticPerson, NULL, this, current_person, "people");

 intended_center = new_focus->position;
 follow_selected_node = p_follow;

 for (list<node_base<3> *>::iterator i_node = physical_simulation.nodes.begin();
      i_node != physical_simulation.nodes.end(); i_node++)
 {
  ((extended_node<3>*)(void*)(*i_node))->SetRemoteData("");
  if ((void*)(*i_node) == (void*)new_focus)
   ((extended_node<3>*)(void*)(*i_node))->SetColor(clRed);
  else
   ((extended_node<3>*)(void*)(*i_node))->SetColor(clBlack);
 }

 #ifdef use_tcpip_messaging
 if(debug_level[libraries]>=8 && message_router) message_router->SendMessageToAll("debug", "TformSpecialRelationshipGraph::ChangeFocus(...) post-SetColor()");
 #endif

 for (list<tensor_base<3> *>::iterator i_tensor = physical_simulation.tensors.begin();
      i_tensor != physical_simulation.tensors.end(); i_tensor++)
  ((extended_tensor<3>*)(void*)(*i_tensor))->SetFocus(new_focus);

 #ifdef use_tcpip_messaging
 if(debug_level[libraries]>=8 && message_router) message_router->SendMessageToAll("debug", "TformSpecialRelationshipGraph::ChangeFocus(...) exit");
 #endif
}
//---------------------------------------------------------------------------
void TformSpecialRelationshipGraph::RemoveObjects ()
{
 // on refresh, this will cause an access violation most likely. between two notifications,
 // notifications for this form get purged. middle of the loop, gets confused, freaks out.
 // atlas patched, but in debug mode, you'll notice. atlas could probably be modified to
 // re-read the list after every request is fulfilled or somesuch but i'm lazy tonight.
 // atlas->CancelRequestFrom(this);
 // fuck it. i'll deal with it later, eh? yeah. later. that's what i said.
 for (list<tensor_base<3> *>::iterator i_tensor = physical_simulation.tensors.begin();
      i_tensor != physical_simulation.tensors.end(); i_tensor++)
  delete (*i_tensor);

 for (list<node_base<3> *>::iterator i_node = physical_simulation.nodes.begin();
      i_node != physical_simulation.nodes.end(); i_node++)
  delete (*i_node);
 num_nodes = 0;
 num_edges = 0;
 num_nodes_shown = 0;
 num_edges_shown = 0;
 physical_simulation.tensors.clear();
 physical_simulation.nodes.clear();
}
//---------------------------------------------------------------------------
void TformSpecialRelationshipGraph::UpdateObjects ()
{
 if (follow_selected_node)
  if (selected_node)
   intended_center = selected_node->position;
 visual_center = (visual_center + intended_center) / 2.00;

 for (list<tensor_base<3> *>::iterator i_tensor = physical_simulation.tensors.begin();
      i_tensor != physical_simulation.tensors.end(); i_tensor++)
  if (((extended_tensor<3>*)(void*)(*i_tensor))->GetExists())
   ((extended_tensor<3>*)(void*)(*i_tensor))->UpdateVisual();

 for (list<node_base<3> *>::iterator i_node = physical_simulation.nodes.begin();
      i_node != physical_simulation.nodes.end(); i_node++)
  if (((extended_node<3>*)(void*)(*i_node))->GetExists())
   ((extended_node<3>*)(void*)(*i_node))->UpdateVisual();
}
//---------------------------------------------------------------------------
void __fastcall TformSpecialRelationshipGraph::timerRefreshTimer(TObject *Sender)
{
 timerRefresh->Enabled = false; // stop for now

 // extra 3d bit here, for the heck of it, eh?
 // code lifted from changes made to pseudotheos' graph.
 // if (true /* if we want 3d-ness */)
 {
  for (list<node_base<3> *>::iterator i_node = physical_simulation.nodes.begin(); i_node != physical_simulation.nodes.end(); i_node++)
  {
   t_vector<3> vec_oa = (*i_node)->position;
   vec_oa.dimensions[0] = 0.00;
   vec_oa.dimensions[1] = 0.00;
   dimension dist_oa_squared = vec_oa.size_squared();
   dimension dist_oa = sqrt(dist_oa_squared);
   if (dist_oa_squared != 0.00)
   {
    t_vector<3> unit_vec_oa = vec_oa / dist_oa; // _squared;
    dimension force = 0.50 * (dist_oa);
    t_vector<3> force_a = unit_vec_oa * force * (-1.00);
    (*i_node)->push(force_a);
   }
  }
 }
 // realtime =) or maybe ten times normal speed. who cares? =)
 // re-adding node-to-node relationships here, as they were removed from tensors.h itself
   list<node_base<3> *>::iterator i_node_b;
   for (list<node_base<3> *>::iterator i_node_a = physical_simulation.nodes.begin(); i_node_a != physical_simulation.nodes.end(); i_node_a ++)
   {
    i_node_b = i_node_a;
    for (i_node_b ++; i_node_b != physical_simulation.nodes.end(); i_node_b++)
     (*i_node_a)->apply_charge_mutually((*i_node_b));
   }
 physical_simulation.simulate(timerRefresh->Interval / 10.00);

 // labelZoomIn->Caption = FinancialToString(bob, false, 4).c_str();

 if (num_edges_shown < num_edges)
 {
  dimension bob = physical_simulation.stat_node_max_velocity(); //stat_node_kinetic_energy();
  if (bob < velocity_threshhold /*kinetic_threshhold*/)
  {
   for (list<tensor_base<3> *>::iterator i_tensor = physical_simulation.tensors.begin();
        i_tensor != physical_simulation.tensors.end(); i_tensor++)
   {
    if (((extended_tensor<3>*)(void*)(*i_tensor))->GetExists() == false)
    {
     ((extended_tensor<3>*)(void*)(*i_tensor))->SetExists(true);

     num_edges_shown++;

     bool add_a = !((extended_node<3> *)(void*)((*i_tensor)->node_a))->GetExists();
     bool add_b = !((extended_node<3> *)(void*)((*i_tensor)->node_b))->GetExists();

     if (add_a)
     {
      ((extended_node<3> *)(void*)((*i_tensor)->node_a))->SetExists(true);
      num_nodes_shown++;
     }
     if (add_b)
     {
      ((extended_node<3> *)(void*)((*i_tensor)->node_b))->SetExists(true);
      num_nodes_shown++;
     }

     dimension current_size = 0.00;
     t_vector<3> add_to_existing = t_vector<3>(3, ((rand() / (RAND_MAX + 0.0)) * 1.00) - 0.50, ((rand() / (RAND_MAX + 0.0)) * 1.00) - 0.50, ((rand() / (RAND_MAX + 0.0)) * 1.00) - 0.50);
     current_size = add_to_existing.size();
     while (!current_size)
     {
      add_to_existing = t_vector<3>(3, ((rand() / (RAND_MAX + 0.0)) * 1.00) - 0.50, ((rand() / (RAND_MAX + 0.0)) * 1.00) - 0.50, ((rand() / (RAND_MAX + 0.0)) * 1.00) - 0.50);
      current_size = add_to_existing.size();
     }
     add_to_existing /= add_to_existing.size();
     add_to_existing *= std_size;

     if (add_b)
     {
      (*i_tensor)->node_b->position = (*i_tensor)->node_a->position + add_to_existing;
     } else
     if (add_a)
     {
      (*i_tensor)->node_a->position = (*i_tensor)->node_b->position + add_to_existing;
     }

     UpdateNums();

     break;
    }
   }
  }
 }
 timerRefresh->Enabled = true; // go again
}
//---------------------------------------------------------------------------
void __fastcall TformSpecialRelationshipGraph::PaintPaint(TObject *Sender)
{
 // on paint paintbox
 TPaintBox * p = (TPaintBox *)(Sender);
 extended_tensor<3> * t = (extended_tensor<3> *)(p->Tag);
 TCanvas * c = p->Canvas;
 TRect r = c->ClipRect;

 if (t->GetBold())
 {
  c->Pen->Color = clSilver;
  c->Pen->Style = psSolid;
 } else
 {
  c->Pen->Color = clSilver;
  c->Pen->Style = psDot;
 }

 if (t->GetIsDiagonal())
 {
  if (t->GetDirection() == falling)
  {
   c->MoveTo(0,0);
   c->LineTo(p->Width - 1, p->Height - 1);
  } else
  {
   c->MoveTo(0, p->Height - 1);
   c->LineTo(p->Width - 1, 0);
  }
 } else
 {
  // TODO: pipe drawing here
  if (t->GetDirection() == falling)
  {
   c->MoveTo(0,0);
   c->LineTo(0, (p->Height * 0.75) - 1);
   c->LineTo(p->Width - 1, (p->Height * 0.75) - 1);
   c->LineTo(p->Width - 1, p->Height - 1);
  } else
  {
   c->MoveTo(0, p->Height - 1);
   c->LineTo(0, (p->Height * 0.75) - 1);
   c->LineTo(p->Width - 1, (p->Height * 0.75) - 1);
   c->LineTo(p->Width - 1, 0);
  }
 }
}
//---------------------------------------------------------------------------
void __fastcall TformSpecialRelationshipGraph::buttonCloseClick(TObject *Sender)
{
 Close();
}
//---------------------------------------------------------------------------
void __fastcall TformSpecialRelationshipGraph::FormClose(TObject *Sender, TCloseAction &Action)
{
 atlas->CancelRequestFrom(this);
 timerRefresh->Enabled = false;
 timerUpdate->Enabled = false;
 RemoveObjects();
 TformSpecialGeneric::FormClose(Sender, Action);
}
//---------------------------------------------------------------------------
void __fastcall TformSpecialRelationshipGraph::buttonRefreshClick(TObject *Sender)
{
 if (selected_node)
  FetchData(selected_node->GetID());
}
//---------------------------------------------------------------------------
void __fastcall TformSpecialRelationshipGraph::trackZoomChange(TObject *Sender)
{
 // zoom changing
 zoom_level = (dimension)(((trackZoom->Max - trackZoom->Position) + 1) / 100.00);
}
//---------------------------------------------------------------------------
void __fastcall TformSpecialRelationshipGraph::LabelMouseDown(TObject *Sender, TMouseButton Button, TShiftState Shift, int X, int Y)
{
 // mouse down
 // ! start dragging a node, most likely
 TLabel * l = (TLabel *)Sender;
 extended_node<3> * n = (extended_node<3> *)(l->Tag);
 CaptureNode(n);
 wants_to_select = n;
}
//---------------------------------------------------------------------------
void __fastcall TformSpecialRelationshipGraph::LabelMouseUp(TObject *Sender, TMouseButton Button, TShiftState Shift, int X, int Y)
{
 // mouse up
 // ! let go of a node (not necessarily this one) if dragging
 TLabel * l = (TLabel *)Sender;
 Drag (X + l->Left, Y + l->Top);
 ReleaseNode();

 extended_node<3> * n = (extended_node<3> *)(l->Tag);
 // drag distance is remeasured against the zoom level ... so if you're zoomed out,
 // you don't have to be extra-careful.
 if (n == wants_to_select && distance_dragged < (drag_threshhold / zoom_level))
  ChangeFocus(n, false);
}
//---------------------------------------------------------------------------
void __fastcall TformSpecialRelationshipGraph::LabelMouseMove(TObject *Sender, TShiftState Shift, int X, int Y)
{
 // mouse move
 // ! move the node here, if dragging (regardless of which node it is)
 TLabel * l = (TLabel *)Sender;
 Drag (X + l->Left, Y + l->Top);
}
//---------------------------------------------------------------------------
void __fastcall TformSpecialRelationshipGraph::PaintMouseDown(TObject *Sender, TMouseButton Button, TShiftState Shift, int X, int Y)
{
 // mouse down
 // . nothing
 CaptureScreen ();
}
//---------------------------------------------------------------------------
void __fastcall TformSpecialRelationshipGraph::PaintMouseMove(TObject *Sender, TShiftState Shift, int X, int Y)
{
 // mouse move
 // ! possibly move a node being pulled
 TPaintBox * p = (TPaintBox *)Sender;
 Drag (X + p->Left, Y + p->Top);
}
//---------------------------------------------------------------------------
void __fastcall TformSpecialRelationshipGraph::PaintMouseUp(TObject *Sender, TMouseButton Button, TShiftState Shift, int X, int Y)
{
 // mouse up
 // ! possibly let go of a node being pulled
 TPaintBox * p = (TPaintBox *)Sender;
 Drag (X + p->Left, Y + p->Top);
 ReleaseNode ();
}
//---------------------------------------------------------------------------
void __fastcall TformSpecialRelationshipGraph::panelGraphMouseDown(TObject *Sender, TMouseButton Button, TShiftState Shift, int X, int Y)
{
 // mouse down
 // . nothing
 CaptureScreen ();
}
//---------------------------------------------------------------------------
void __fastcall TformSpecialRelationshipGraph::panelGraphMouseMove(TObject *Sender, TShiftState Shift, int X, int Y)
{
 // mouse move
 // ! possibly change position of a node being pulled
 Drag (X, Y);
}
//---------------------------------------------------------------------------

void __fastcall TformSpecialRelationshipGraph::panelGraphMouseUp(
      TObject *Sender, TMouseButton Button, TShiftState Shift, int X,
      int Y)
{
 // mouse up
 // ! possibly let go of a node being pulled
 Drag (X, Y);
 if (dragging_screen)
  ReleaseScreen ();
 else
  ReleaseNode();
}
//---------------------------------------------------------------------------
void TformSpecialRelationshipGraph::CaptureNode (extended_node<3> * p_node)
{
 if (dragging_node)
  ReleaseNode();

 dragging_node = p_node;
 prev_dragging_node = dragging_node;
 if (dragging_node)
 {
  distance_dragged = 0.00;
  p_node->infinite_mass = true;
  Screen->Cursor = crDrag;
 }
}
//---------------------------------------------------------------------------
void TformSpecialRelationshipGraph::ReleaseNode ()
{
 dragging_screen = false;
 if (dragging_node)
 {
  dragging_node->infinite_mass = false;
  prev_dragging_node = dragging_node;
  dragging_node = NULL;
 }
 if (Screen->Cursor == crDrag)
  Screen->Cursor = crArrow;
}
//---------------------------------------------------------------------------
void TformSpecialRelationshipGraph::ClickNearestRelationship ()
{
 // based on last_mouse_pos, and the fact that they click on a paintbox ...
 // find the nearest line, and go for it.
 extended_tensor<3> * nearest_tensor = NULL;
 dimension min_dist  = 0.00;
 dimension this_dist = 0.00;

 for (list<tensor_base<3> *>::iterator i_tensor = physical_simulation.tensors.begin();
      i_tensor != physical_simulation.tensors.end(); i_tensor++)
 {
  if (last_mouse_pos.dimensions[0] >= min((*i_tensor)->node_a->position.dimensions[0], (*i_tensor)->node_b->position.dimensions[0]) &&
      last_mouse_pos.dimensions[0] <= max((*i_tensor)->node_a->position.dimensions[0], (*i_tensor)->node_b->position.dimensions[0]) &&
      last_mouse_pos.dimensions[1] >= min((*i_tensor)->node_a->position.dimensions[1], (*i_tensor)->node_b->position.dimensions[1]) &&
      last_mouse_pos.dimensions[1] <= max((*i_tensor)->node_a->position.dimensions[1], (*i_tensor)->node_b->position.dimensions[1]))
  {
   this_dist = point_segment_distance((*i_tensor)->node_a->position, (*i_tensor)->node_b->position, last_mouse_pos);
   if ((nearest_tensor && this_dist < min_dist) || (!nearest_tensor))
   {
    nearest_tensor = (extended_tensor<3>*)(void*)(*i_tensor);
    min_dist = this_dist;
   }
  }
 }
 if (nearest_tensor && min_dist < near_threshhold)
 {
  map<string, string> params;
  params["id"] = nearest_tensor->GetID();
  if (IsInt(params["id"]) && params["id"] != "")
  {
   FORM_TRY
    TformRecordRelationship * f = new TformRecordRelationship(NULL, params);
    f->Show();
   FORM_CATCH
  }
 }
}
//---------------------------------------------------------------------------
void __fastcall TformSpecialRelationshipGraph::PaintClick(TObject *Sender)
{
 // click on a line
 ClickNearestRelationship();
}
//---------------------------------------------------------------------------
void TformSpecialRelationshipGraph::CaptureScreen ()
{
 dragging_screen      = true;
 follow_selected_node = false;
 Screen->Cursor = crDrag;
}
//---------------------------------------------------------------------------
void TformSpecialRelationshipGraph::ReleaseScreen ()
{
 if (dragging_screen)
  dragging_screen = false;
 if (Screen->Cursor == crDrag)
  Screen->Cursor = crArrow;
}
//---------------------------------------------------------------------------
void TformSpecialRelationshipGraph::Drag (int p_x, int p_y)
{
 follow_selected_node = false;
 
 t_vector<3> screen_position = t_vector<3>(3, (dimension)p_x, (dimension)p_y, 0.00);
 t_vector<3> screen_center   = t_vector<3>(3, panelGraph->Width / 2.00, panelGraph->Height / 2.00, 0.00);
 t_vector<3> actual_position;
 actual_position = ((screen_position - screen_center) / zoom_level) + visual_center;
 
 t_vector<3> actual_change = actual_position - last_mouse_pos;

 last_mouse_pos = actual_position;

 if (dragging_screen)
 {
  intended_center -= actual_change;
 }
 else if (dragging_node)
 {
  distance_dragged += (actual_position - dragging_node->position).size();
  dragging_node->position = actual_position;
 } else
 {
  if (Screen->Cursor == crDrag)
   Screen->Cursor = crArrow;
 }
}

//---------------------------------------------------------------------------
void __fastcall TformSpecialRelationshipGraph::timerUpdateTimer(TObject *Sender)
{
 // stuff
 if (timerRefresh->Enabled)
 {
  UpdateObjects ();
  // this->Update();
 }
}
//---------------------------------------------------------------------------
void TformSpecialRelationshipGraph::UpdateNums()
{
 // label stuff
 labelNums->Caption = string(IntToString(num_nodes_shown) + " of " + IntToString(num_nodes) + " people shown, " + IntToString(num_edges_shown) + " of " + IntToString(num_edges) + " links shown.").c_str();
}
//---------------------------------------------------------------------------


