//---------------------------------------------------------------------------

#ifndef codeSpecialRelationshipGraphH
#define codeSpecialRelationshipGraphH
//---------------------------------------------------------------------------
#include <Classes.hpp>
#include <Controls.hpp>
#include <StdCtrls.hpp>
#include <Forms.hpp>
#include "codeSpecialGeneric.h"
#include <ComCtrls.hpp>
#include <ExtCtrls.hpp>
#include <ToolWin.hpp>
#ifdef use_tcpip_messaging
#include "tcpip_messaging.h"
#endif
#include "codeSpecialGeneric.h"
#include <ComCtrls.hpp>
#include <ToolWin.hpp>
#include <ExtCtrls.hpp>

//---------------------------------------------------------------------------
#include "tensors.h"
#define std_gravity  0.50 // force of gravity, neh?
#define std_mass     0.75 // mass of a node
#define std_drag     0.99 // % of energy kept after drag operation
#define std_minv     1.00 // if moving slower than this, stop.
#define std_tens 10000.00 // tension between objects
#define foc_tens  5000.00
#define std_size   125.00 // distance between objects
#define foc_size   125.00
#define std_elec 45000.00 // repulsive electrical field
#define drag_threshhold     15.00 // distance to start dragging
#define near_threshhold     15.00 // distance for grabbing
#define kinetic_threshhold  25.00 // not used currently, but new edges spawned if total kinetic energy falls below this
#define velocity_threshhold 50.00 // new edges spawned if maximum velocity of any node falls below this

/*
in the simulation:
 t_node represents people
 t_tensor represents relationships between people
on the screen:
 static texts represent people (and info about them relative to the selected person)
 paint boxes represent the links between people (drawing the diagonal line)
to bind it all together:
 map<int, object(statictext*, t_node*, ...)> represents the association of <tag, t_node>
 list<object(paintbox*, t_tensor*, ...)> represents the relationship from diagonal to tensor

after simulation, for every linking object, take data from simulation and apply it visually
during mouseovers, using tag to find linking object to know what to change visually

fetchdata should:
 download all relationships spawning off of the given person id, giving us:
  people (id, name, ...)
  relationships (id, id, rel, rel, 6xbool, same_household)
*/

template <unsigned int D>
class extended_node : public node_standard<D>
{
 public:
  bool exists;
  
 public:
  extended_node (const t_vector<D> & p_position, dimension p_mass, dimension p_drag, dimension p_minimum_velocity, dimension p_charge, bool p_infinite_mass, const string & p_initial_id, const string & p_initial_local, const string & p_initial_remote, TWinControl * p_visual_container, TMouseEvent p_onmousedown, TMouseMoveEvent p_onmousemove, TMouseEvent p_onmouseup, t_vector<D> * p_visual_center, dimension * p_zoom_level, bool p_exists)
  : node_standard<D> (p_position, p_mass, p_drag, p_minimum_velocity, p_charge, p_infinite_mass), exists(p_exists)
  {
   #ifdef tcpip_messaging_h
   if(debug_level[libraries]>=8 && message_router) message_router->SendMessageToAll("debug", "extended_node::extended_node(...) entry");
   #endif
   is_x_constrained = false;
   is_y_constrained = false;

   drawing_object = NULL;
   visual_center  = p_visual_center;
   zoom_level     = p_zoom_level;

   SetID(p_initial_id);
   SetLocalData(p_initial_local);
   SetRemoteData(p_initial_remote);

   drawing_object = new TLabel((TComponent *)NULL);
   drawing_object->Parent      = p_visual_container;
   drawing_object->Tag         = (int)this;
   drawing_object->AutoSize    = true;
   drawing_object->Transparent = true;
   drawing_object->OnMouseDown = p_onmousedown;
   drawing_object->OnMouseUp   = p_onmouseup;
   drawing_object->OnMouseMove = p_onmousemove;
   drawing_object->Visible     = exists;
   #ifdef tcpip_messaging_h
   if(debug_level[libraries]>=8 && message_router) message_router->SendMessageToAll("debug", "extended_node::extended_node(...) exit");
   #endif
  }
  ~extended_node ()
  {
   #ifdef tcpip_messaging_h
   if(debug_level[libraries]>=8 && message_router) message_router->SendMessageToAll("debug", "extended_node::~extended_node() called");
   #endif
   delete drawing_object;
  }
  void SetID (const string & p_id)
  {
   #ifdef tcpip_messaging_h
   if(debug_level[libraries]>=8 && message_router) message_router->SendMessageToAll("debug", "extended_node::SetID(" + p_id + ") called");
   #endif
   id = p_id;
  }
  void SetLocalData (const string & p_text)
  {
   #ifdef tcpip_messaging_h
   if(debug_level[libraries]>=8 && message_router) message_router->SendMessageToAll("debug", "extended_node::SetLocalData(" + p_text + ") called");
   #endif
   local_data = p_text;
  }
  void SetRemoteData (const string & p_text)
  {
   #ifdef tcpip_messaging_h
   if(debug_level[libraries]>=8 && message_router) message_router->SendMessageToAll("debug", "extended_node::SetRemoteData(" + p_text + ") called");
   #endif
   remote_data = p_text;
  }
  string GetID () const
  {
   #ifdef tcpip_messaging_h
   if(debug_level[libraries]>=8 && message_router) message_router->SendMessageToAll("debug", "extended_node::GetID() called");
   #endif
   return id;
  }
  string GetLocalData () const
  {
   #ifdef tcpip_messaging_h
   if(debug_level[libraries]>=8 && message_router) message_router->SendMessageToAll("debug", "extended_node::GetLocalData() called");
   #endif
   return local_data;
  }
  string GetRemoteData () const
  {
   #ifdef tcpip_messaging_h
   if(debug_level[libraries]>=8 && message_router) message_router->SendMessageToAll("debug", "extended_node::GetRemoteData() called");
   #endif
   if (!remote_data.empty())
    return string("(" + remote_data + ")");
   else
    return string("");
  }
  void SetColor (TColor p_new_color)
  {
   #ifdef tcpip_messaging_h
   if(debug_level[libraries]>=8 && message_router) message_router->SendMessageToAll("debug", "extended_node::SetColor(" + IntToString((unsigned int)(p_new_color)) + ") called");
   #endif
   if (drawing_object->Font->Color != p_new_color)
    drawing_object->Font->Color = p_new_color;
  }
  void UpdateVisual ()
  {
   #ifdef tcpip_messaging_h
   if(debug_level[libraries]>=8 && message_router) message_router->SendMessageToAll("debug", "extended_node::UpdateVisual() entry");
   #endif
   if (is_x_constrained)
    position.dimensions[0] = x_constraint_value;
   if (is_y_constrained)
    position.dimensions[1] = y_constraint_value;

   t_vector<D> screen_center = t_vector<D>(3, drawing_object->Parent->Width / 2.00, drawing_object->Parent->Height / 2.00, 0.00);
   t_vector<D> visual_position = ((position - (*visual_center)) * (*zoom_level)) + screen_center;

   string new_cap = GetLocalData();
   if (!GetRemoteData().empty())
   {
    new_cap += "\n" + GetRemoteData();
    if (new_cap != drawing_object->Caption.c_str())
    {
     int line_one_width, line_two_width;
     drawing_object->AutoSize = true;
     drawing_object->Caption = GetLocalData().c_str();
     line_one_width = drawing_object->Width;
     drawing_object->Caption = GetRemoteData().c_str();
     line_two_width = drawing_object->Width;
     drawing_object->AutoSize = false;
     drawing_object->Alignment = taCenter;
     drawing_object->Width = max(line_one_width, line_two_width);
     drawing_object->Height = 2 * drawing_object->Height;
     drawing_object->Caption = new_cap.c_str();
    }
   } else
   {
    if (new_cap != drawing_object->Caption.c_str())
    {
     drawing_object->AutoSize = true;
     drawing_object->Caption = GetLocalData().c_str();
    }
   }

   t_vector<D> visible_dimensions = t_vector<D>(3, (dimension)(drawing_object->Width), (dimension)(drawing_object->Height), 0.00);
   t_vector<D> new_position = visual_position - 0.50 * visible_dimensions;

   int new_left = (int)(new_position.dimensions[0]);
   int new_top = (int)(new_position.dimensions[1]);

   if (new_left != drawing_object->Left)
    drawing_object->Left = new_left;
   if (new_top != drawing_object->Top)
    drawing_object->Top = new_top;
   #ifdef tcpip_messaging_h
   if(debug_level[libraries]>=8 && message_router) message_router->SendMessageToAll("debug", "extended_node::UpdateVisual() exit");
   #endif
  }
  bool GetExists ()
  {
   #ifdef tcpip_messaging_h
   if(debug_level[libraries]>=8 && message_router) message_router->SendMessageToAll("debug", "extended_node::GetExists() called");
   #endif
   return exists;
  }
  void SetExists (bool p_exists)
  {
   #ifdef tcpip_messaging_h
   if(debug_level[libraries]>=8 && message_router) message_router->SendMessageToAll("debug", "extended_node::SetExists(" + IntToBoolString(BoolToInt(p_exists)) + ") called");
   #endif
   exists = p_exists;
   drawing_object->Visible = exists;
  }
  void SetXConstraint(bool is_constrained, dimension x_constraint)
  {
   #ifdef tcpip_messaging_h
   if(debug_level[libraries]>=8 && message_router) message_router->SendMessageToAll("debug", "extended_node::SetXConstraint(" + IntToBoolString(BoolToInt(is_constrained)) + ", dimension) called");
   #endif
   is_x_constrained = is_constrained;
   x_constraint_value = x_constraint;
  }
  bool GetIsXConstrained()
  {
   #ifdef tcpip_messaging_h
   if(debug_level[libraries]>=8 && message_router) message_router->SendMessageToAll("debug", "extended_node::GetXConstrained() called");
   #endif
   return is_x_constrained;
  }
  dimension GetXConstraint()
  {
   #ifdef tcpip_messaging_h
   if(debug_level[libraries]>=8 && message_router) message_router->SendMessageToAll("debug", "extended_node::GetXConstraint() called");
   #endif
   return x_constraint_value;
  }
  void SetYConstraint(bool is_constrained, dimension y_constraint)
  {
   #ifdef tcpip_messaging_h
   if(debug_level[libraries]>=8 && message_router) message_router->SendMessageToAll("debug", "extended_node::SetYConstraint(" + IntToBoolString(BoolToInt(is_constrained)) + ", dimension) called");
   #endif
   is_y_constrained = is_constrained;
   y_constraint_value = y_constraint;
  }
  bool GetIsYConstrained()
  {
   #ifdef tcpip_messaging_h
   if(debug_level[libraries]>=8 && message_router) message_router->SendMessageToAll("debug", "extended_node::GetYConstrained() called");
   #endif
   return is_y_constrained;
  }
  dimension GetYConstraint()
  {
   #ifdef tcpip_messaging_h
   if(debug_level[libraries]>=8 && message_router) message_router->SendMessageToAll("debug", "extended_node::GetYConstraint() called");
   #endif
   return y_constraint_value;
  }
 private:
  TLabel * drawing_object;
  t_vector<D> * visual_center;
  dimension * zoom_level;

  // added for family stuff (plus a little bit)
  bool is_x_constrained;
  bool is_y_constrained;
  dimension x_constraint_value;
  dimension y_constraint_value;

  string id;
  string local_data;
  string remote_data;
};

enum direction {rising, falling};

template <unsigned int D>
class extended_tensor : public tensor_standard<D>
{
 public:
  bool exists;
 public:
  extended_tensor (node_base<D> * p_node_a, node_base<D> * p_node_b, dimension p_tension, dimension p_size, const string & p_initial_id, const string & p_initial_a, const string & p_initial_b, bool p_initial_bold, TWinControl * p_visual_container, TNotifyEvent p_onclick, TNotifyEvent p_onpaint, TMouseEvent p_onmousedown, TMouseMoveEvent p_onmousemove, TMouseEvent p_onmouseup, t_vector<D> * p_visual_center, dimension * p_zoom_level, bool p_exists)
  : tensor_standard<D> (p_node_a, p_node_b, p_tension, p_size), exists (p_exists)
  {
   #ifdef tcpip_messaging_h
   if(debug_level[libraries]>=8 && message_router) message_router->SendMessageToAll("debug", "extended_tensor::extended_tensor(...) entry");
   #endif
   is_diagonal = true;

   #ifdef tcpip_messaging_h
   if(debug_level[libraries]>=8 && message_router) message_router->SendMessageToAll("debug", "extended_tensor::extended_tensor(...) initializing variables...");
   #endif

   drawing_object = NULL;
   visual_center  = p_visual_center;
   zoom_level     = p_zoom_level;

   #ifdef tcpip_messaging_h
   if(debug_level[libraries]>=8 && message_router) message_router->SendMessageToAll("debug", "extended_tensor::extended_tensor(...) Calling SetID()");
   #endif
   SetID(p_initial_id);
   #ifdef tcpip_messaging_h
   if(debug_level[libraries]>=8 && message_router) message_router->SendMessageToAll("debug", "extended_tensor::extended_tensor(...) Calling SetAboutA(...)");
   #endif
   SetAboutA(p_initial_a);
   #ifdef tcpip_messaging_h
   if(debug_level[libraries]>=8 && message_router) message_router->SendMessageToAll("debug", "extended_tensor::extended_tensor(...) Calling SetAboutB(...)");
   #endif
   SetAboutB(p_initial_b);
   #ifdef tcpip_messaging_h
   if(debug_level[libraries]>=8 && message_router) message_router->SendMessageToAll("debug", "extended_tensor::extended_tensor(...) Calling SetBold(...)");
   #endif
   SetBold(p_initial_bold);

   #ifdef tcpip_messaging_h
   if(debug_level[libraries]>=8 && message_router) message_router->SendMessageToAll("debug", "extended_tensor::extended_tensor(...) Calling initializing drawing object...");
   #endif
   drawing_object = new TPaintBox(NULL);
   drawing_object->Parent      = p_visual_container;
   drawing_object->Tag         = (int)this;
   drawing_object->OnPaint     = p_onpaint;
   drawing_object->OnClick     = p_onclick;
   drawing_object->OnMouseDown = p_onmousedown;
   drawing_object->OnMouseMove = p_onmousemove;
   drawing_object->OnMouseUp   = p_onmouseup;
   drawing_object->Visible     = exists;

   #ifdef tcpip_messaging_h
   if(debug_level[libraries]>=8 && message_router) message_router->SendMessageToAll("debug", "extended_tensor::extended_tensor(...) Sending it to back...");
   #endif
   drawing_object->SendToBack();
   #ifdef tcpip_messaging_h
   if(debug_level[libraries]>=8 && message_router) message_router->SendMessageToAll("debug", "extended_tensor::extended_tensor(...) exit");
   #endif
  }
  ~extended_tensor ()
  {
   #ifdef tcpip_messaging_h
   if(debug_level[libraries]>=8 && message_router) message_router->SendMessageToAll("debug", "extended_tensor::~extended_tensor() called");
   #endif
   delete drawing_object;
  }
  void SetID (const string & p_id)
  {
   #ifdef tcpip_messaging_h
   if(debug_level[libraries]>=8 && message_router) message_router->SendMessageToAll("debug", "extended_tensor::SetID(" + p_id + ") called");
   #endif
   id = p_id;
  }
  string GetID () const
  {
   #ifdef tcpip_messaging_h
   if(debug_level[libraries]>=8 && message_router) message_router->SendMessageToAll("debug", "extended_tensor::GetID() called");
   #endif
   return id;
  }
  void SetAboutA(const string & p_text)
  {
   #ifdef tcpip_messaging_h
   if(debug_level[libraries]>=8 && message_router) message_router->SendMessageToAll("debug", "extended_tensor::SetAboutA(" + p_text + ") called");
   #endif
   about_a = p_text;
  }
  void SetAboutB(const string & p_text)
  {
   #ifdef tcpip_messaging_h
   if(debug_level[libraries]>=8 && message_router) message_router->SendMessageToAll("debug", "extended_tensor::SetAboutB(" + p_text + ") called");
   #endif
   about_b = p_text;
  }
  void SetBold(bool p_bold)
  {
   #ifdef tcpip_messaging_h
   if(debug_level[libraries]>=8 && message_router) message_router->SendMessageToAll("debug", "extended_tensor::SetBold(" + IntToBoolString(BoolToInt(p_bold)) + ") called");
   #endif
   is_bold = p_bold;
  }
  string GetAboutA() const
  {
   #ifdef tcpip_messaging_h
   if(debug_level[libraries]>=8 && message_router) message_router->SendMessageToAll("debug", "extended_tensor::GetAboutA() called");
   #endif
   return about_a;
  }
  string GetAboutB() const
  {
   #ifdef tcpip_messaging_h
   if(debug_level[libraries]>=8 && message_router) message_router->SendMessageToAll("debug", "extended_tensor::GetAboutB() called");
   #endif
   return about_b;
  }
  bool GetBold() const
  {
   #ifdef tcpip_messaging_h
   if(debug_level[libraries]>=8 && message_router) message_router->SendMessageToAll("debug", "extended_tensor::GetBold() called");
   #endif
   return is_bold;
  }
  direction GetDirection () const
  {
   #ifdef tcpip_messaging_h
   if(debug_level[libraries]>=8 && message_router) message_router->SendMessageToAll("debug", "extended_tensor::GetDirection() called");
   #endif
   return current_diagonal;
  }
  void SetFocus (const extended_node<D> * p_focus)
  {
   #ifdef tcpip_messaging_h
   if(debug_level[libraries]>=8 && message_router) message_router->SendMessageToAll("debug", "extended_tensor::SetFocus(extended_node<uint>&) called");
   #endif
   extended_node<D> * a = (extended_node<D> *)(void*)(this->node_a);
   extended_node<D> * b = (extended_node<D> *)(void*)(this->node_b);
   if (a == p_focus)
   {
    b->SetRemoteData(about_b);
    b->SetColor(clMaroon);
    tension = foc_tens;
    size = foc_size;
   } else
   if (b == p_focus)
   {
    a->SetRemoteData(about_a);
    a->SetColor(clMaroon);
    tension = foc_tens;
    size = foc_size;
   } else
   {
    tension = std_tens;
    size = std_size;
   }
  }
  void UpdateVisual ()
  {
   #ifdef tcpip_messaging_h
   if(debug_level[libraries]>=8 && message_router) message_router->SendMessageToAll("debug", "extended_tensor::UpdateVisual() entry");
   #endif
   // TODO: a == b leads to new drawing system, eh?
   t_vector<D> screen_center = t_vector<D>(3, drawing_object->Parent->Width / 2.00, drawing_object->Parent->Height / 2.00, 0.00);
   t_vector<D> visual_position_a = ((node_a->position - (*visual_center)) * (*zoom_level)) + screen_center;
   t_vector<D> visual_position_b = ((node_b->position - (*visual_center)) * (*zoom_level)) + screen_center;

   if (node_a->position.dimensions[0] < node_b->position.dimensions[0])
    if (node_a->position.dimensions[1] < node_b->position.dimensions[1])
     current_diagonal = falling;
    else
     current_diagonal = rising;
   else
    if (node_a->position.dimensions[1] < node_b->position.dimensions[1])
     current_diagonal = rising;
    else
     current_diagonal = falling;

   dimension left   = min(visual_position_a.dimensions[0], visual_position_b.dimensions[0]);
   dimension right  = max(visual_position_a.dimensions[0], visual_position_b.dimensions[0]);
   dimension top    = min(visual_position_a.dimensions[1], visual_position_b.dimensions[1]);
   dimension bottom = max(visual_position_a.dimensions[1], visual_position_b.dimensions[1]);

   int new_left   = (int)(left);
   int new_top    = (int)(top);
   int new_width  = ((int)(right - left) != 0) ? (int)(right - left) : 1;
   int new_height = ((int)(bottom - top) != 0) ? (int)(bottom - top) : 1;

   // avoid redrawing if possible
   if (drawing_object->Left != new_left)
    drawing_object->Left   = new_left;
   if (drawing_object->Top != new_top)
    drawing_object->Top    = new_top;
   if (drawing_object->Width != new_width)
    drawing_object->Width  = new_width;
   if (drawing_object->Left != new_height)
    drawing_object->Height = new_height;
   // zero height and width both cause drawing problems ... so we at least want 1px, right?
   #ifdef tcpip_messaging_h
   if(debug_level[libraries]>=8 && message_router) message_router->SendMessageToAll("debug", "extended_tensor::UpdateVisual() exit");
   #endif
  }
  bool GetExists ()
  {
   #ifdef tcpip_messaging_h
   if(debug_level[libraries]>=8 && message_router) message_router->SendMessageToAll("debug", "extended_tensor::GetExists() called");
   #endif
   return exists;
  }
  void SetExists (bool p_exists)
  {
   #ifdef tcpip_messaging_h
   if(debug_level[libraries]>=8 && message_router) message_router->SendMessageToAll("debug", "extended_tensor::SetExists(" + IntToBoolString(BoolToInt(p_exists)) + ") called");
   #endif
   exists = p_exists;
   drawing_object->Visible = exists;
  }
  void SetIsDiagonal(bool p_diag)
  {
   #ifdef tcpip_messaging_h
   if(debug_level[libraries]>=8 && message_router) message_router->SendMessageToAll("debug", "extended_tensor::SetIsDiagonal(" + IntToBoolString(BoolToInt(p_diag)) + ") called");
   #endif
   is_diagonal = p_diag;
  }
  bool GetIsDiagonal()
  {
   return is_diagonal;
  }
 private:
  t_vector<D> * visual_center;
  dimension * zoom_level;
  TPaintBox * drawing_object;
  string id;
  string about_a;
  string about_b;
  bool   is_bold;
  direction current_diagonal;

  // added for family stuff -- if false, draws square pipes rather than vectors
  bool is_diagonal;
};

template <class gen, class item>
class relationship_creator
{
 private:
  bool is_flipped;
 protected:
  gen & unknown_naming_method;
 public:
  relationship_creator () : is_flipped(false), unknown_naming_method(*(gen*)(NULL)) {};
  relationship_creator (bool flip, gen & howto) : is_flipped(flip), unknown_naming_method(howto) {};
  void virtual calc_rel (item a, item b, map<item, set<item> > & c) = 0;
  void operator () (item a, item b, map<item, set<item> > & c)
  {
   #ifdef tcpip_messaging_h
   if(debug_level[libraries]>=8 && message_router) message_router->SendMessageToAll("debug", "relationship_creator::operator() called");
   #endif
   if (is_flipped)
    calc_rel(b, a, c);
   else
    calc_rel(a, b, c);
  }
};

template <class gen, class item>
class relationship_parent_child : public relationship_creator <gen, item>
{
 public:
  relationship_parent_child (bool flip, gen & howto) : relationship_creator<gen, item> (flip, howto) {};
  void calc_rel (item a, item b, map<item, set<item> > & c)
  {
   #ifdef tcpip_messaging_h
   if(debug_level[libraries]>=8 && message_router) message_router->SendMessageToAll("debug", "relationship_parent_child::calc_rel(...) called");
   #endif
   if (c.find(a) == c.end())
    c[a] = set<item>();
   c[b].insert(a);
  }
};

template <class gen, class item>
class relationship_full_sibling : public relationship_creator <gen, item>
{
 public:
  relationship_full_sibling (bool flip, gen & howto) : relationship_creator<gen, item> (flip, howto) {};
  void calc_rel (item a, item b, map<item, set<item> > & c)
  {
   #ifdef tcpip_messaging_h
   if(debug_level[libraries]>=8 && message_router) message_router->SendMessageToAll("debug", "relationship_full_sibling::calc_rel(...) called");
   #endif
   item a_common_parent(unknown_naming_method());
   c[a_common_parent] = set<item>();
   item b_common_parent(unknown_naming_method());
   c[b_common_parent] = set<item>();
   c[a].insert(a_common_parent);
   c[a].insert(b_common_parent);
   c[b].insert(a_common_parent);
   c[b].insert(b_common_parent);
  }
};

template <class gen, class item>
class relationship_grand_parent_child : public relationship_creator <gen, item>
{
 public:
  relationship_grand_parent_child (bool flip, gen & howto) : relationship_creator<gen, item> (flip, howto) {};
  void calc_rel (item a, item b, map<item, set<item> > & c)
  {
   #ifdef tcpip_messaging_h
   if(debug_level[libraries]>=8 && message_router) message_router->SendMessageToAll("debug", "relationship_grand_parent_child::calc_rel(...) called");
   #endif
   if (c.find(a) == c.end())
    c[a] = set<item>();
   item common_middle(unknown_naming_method());
   c[common_middle].insert(a);
   c[b].insert(common_middle);
  }
};

template <class gen, class item>
class relationship_uncle_nephew : public relationship_creator <gen, item>
{
 public:
  relationship_uncle_nephew (bool flip, gen & howto) : relationship_creator<gen, item> (flip, howto) {};
  void calc_rel (item a, item b, map<item, set<item> > & c)
  {
   #ifdef tcpip_messaging_h
   if(debug_level[libraries]>=8 && message_router) message_router->SendMessageToAll("debug", "relationship_uncle_nephew::calc_rel(...) called");
   #endif
   item common_parent(unknown_naming_method());
   c[common_parent] = set<item>();
   c[a].insert(common_parent);
   item common_middle(unknown_naming_method());
   c[common_middle].insert(common_parent);
   c[b].insert(common_middle);
  }
};

template <class gen, class item>
class relationship_cousin : public relationship_creator <gen, item>
{
 public:
  relationship_cousin (bool flip, gen & howto) : relationship_creator<gen, item> (flip, howto) {};
  void calc_rel (item a, item b, map<item, set<item> > & c)
  {
   #ifdef tcpip_messaging_h
   if(debug_level[libraries]>=8 && message_router) message_router->SendMessageToAll("debug", "relationship_cousin::calc_rel(...) called");
   #endif
   item common_parent(unknown_naming_method());
   c[common_parent] = set<item>();
   item a_middle(unknown_naming_method());
   c[a_middle].insert(common_parent);
   c[a].insert(a_middle);
   item b_middle(unknown_naming_method());
   c[b_middle].insert(common_parent);
   c[b].insert(b_middle);
  }
};

template <class gen, class item>
class relationship_half_sibling : public relationship_creator <gen, item>
{
 public:
  relationship_half_sibling (bool flip, gen & howto) : relationship_creator<gen, item> (flip, howto) {};
  void calc_rel (item a, item b, map<item, set<item> > & c)
  {
   #ifdef tcpip_messaging_h
   if(debug_level[libraries]>=8 && message_router) message_router->SendMessageToAll("debug", "relationship_half_sibling::calc_rel(...) called");
   #endif
   item common_parent(unknown_naming_method());
   c[common_parent] = set<item>();
   c[a].insert(common_parent);
   c[b].insert(common_parent);
  }
};

template <class gen, class item>
class relationship_grand_uncle_nephew : public relationship_creator <gen, item>
{
 public:
  relationship_grand_uncle_nephew (bool flip, gen & howto) : relationship_creator<gen, item> (flip, howto) {};
  void calc_rel (item a, item b, map<item, set<item> > & c)
  {
   #ifdef tcpip_messaging_h
   if(debug_level[libraries]>=8 && message_router) message_router->SendMessageToAll("debug", "relationship_grand_uncle_nephew::calc_rel(...) called");
   #endif
   item common_parent(unknown_naming_method());
   c[common_parent] = set<item>();
   c[a].insert(common_parent);
   item top_middle(unknown_naming_method());
   c[top_middle].insert(common_parent);
   item bottom_middle(unknown_naming_method());
   c[bottom_middle].insert(top_middle);
   c[b].insert(bottom_middle);
  }
};

//---------------------------------------------------------------------------
class TformSpecialRelationshipGraph : public TformSpecialGeneric
{
__published:	// IDE-managed Components
        TToolButton *buttonClose;
        TTimer *timerRefresh;
        TLabel *labelCenteredOn;
        TStaticText *staticPerson;
        TToolButton *buttonRefresh;
        TToolButton *sepBetween;
        TTrackBar *trackZoom;
        TPanel *panelGraph;
        TLabel *labelZoomIn;
        TLabel *labelZoomOut;
        TTimer *timerUpdate;
        TLabel *labelNums;
        void __fastcall timerRefreshTimer(TObject *Sender);
        void __fastcall PaintPaint(TObject *Sender);
        void __fastcall buttonCloseClick(TObject *Sender);
        void __fastcall FormClose(TObject *Sender, TCloseAction &Action);
        void __fastcall buttonRefreshClick(TObject *Sender);
        void __fastcall trackZoomChange(TObject *Sender);
        void __fastcall LabelMouseDown(TObject *Sender,
          TMouseButton Button, TShiftState Shift, int X, int Y);
        void __fastcall LabelMouseUp(TObject *Sender, TMouseButton Button,
          TShiftState Shift, int X, int Y);
        void __fastcall LabelMouseMove(TObject *Sender, TShiftState Shift,
          int X, int Y);
        void __fastcall PaintMouseDown(TObject *Sender,
          TMouseButton Button, TShiftState Shift, int X, int Y);
        void __fastcall PaintMouseMove(TObject *Sender,
          TShiftState Shift, int X, int Y);
        void __fastcall PaintMouseUp(TObject *Sender,
          TMouseButton Button, TShiftState Shift, int X, int Y);
        void __fastcall panelGraphMouseDown(TObject *Sender,
          TMouseButton Button, TShiftState Shift, int X, int Y);
        void __fastcall panelGraphMouseMove(TObject *Sender,
          TShiftState Shift, int X, int Y);
        void __fastcall panelGraphMouseUp(TObject *Sender,
          TMouseButton Button, TShiftState Shift, int X, int Y);
        void __fastcall PaintClick(TObject *Sender);
        void __fastcall timerUpdateTimer(TObject *Sender);
private:	// User declarations
        world_base<3> physical_simulation;
        string current_person;
        bool AtlasScreen (int p_request_num, string matched_value);
        void FetchData (string p_person);
        void FetchFamilyData (string p_person);
        void FetchAllData (string p_person);
        void RemoveObjects ();
        void UpdateObjects ();
        void ChangeFocus (const extended_node<3> * p_node, bool p_follow_node);
        void CaptureNode (extended_node<3> * p_node);
        void ReleaseNode ();
        void ClickNearestRelationship ();
        void CaptureScreen ();
        void ReleaseScreen ();
        void Drag (int p_x, int p_y);
        void UpdateNums ();
        extended_node<3> * selected_node;
        extended_node<3> * dragging_node;
        extended_node<3> * prev_dragging_node;
        extended_node<3> * wants_to_select;
        dimension       distance_dragged;
        t_vector<3> last_mouse_pos;
        t_vector<3> intended_center;
        t_vector<3> visual_center;
        dimension zoom_level;
        bool follow_selected_node;
        bool dragging_screen;
        bool is_family_graph;
        unsigned int num_nodes;
        unsigned int num_edges;
        unsigned int num_nodes_shown;
        unsigned int num_edges_shown;
public:		// User declarations
        __fastcall TformSpecialRelationshipGraph(TComponent* Owner, string p_person, bool is_family = false);
};
//---------------------------------------------------------------------------
extern PACKAGE TformSpecialRelationshipGraph *formSpecialRelationshipGraph;
//---------------------------------------------------------------------------
#endif
