//---------------------------------------------------------------------------

#ifndef codeSpecialReusableAccountingInsertH
#define codeSpecialReusableAccountingInsertH
//---------------------------------------------------------------------------
#include <Classes.hpp>
#include <Controls.hpp>
#include <StdCtrls.hpp>
#include <Forms.hpp>
#include "codeSpecialGeneric.h"
#include <Buttons.hpp>
#include <ComCtrls.hpp>
#include <Mask.hpp>
#include <ToolWin.hpp>
#include <ExtCtrls.hpp>
//---------------------------------------------------------------------------
struct reusable_accounting_insert_parameters
{
 // data
 string default_inventory_tax_entity_id;
 string default_expense_tax_entity_id;
 string default_forward_tax_entity_id_from;
 string default_forward_tax_entity_id_to;
 string default_revenue_tax_entity_id;

 financial target_amount;

 bool is_change;

 bool show_inventory;
 bool show_expense;
 bool show_forward;
 bool show_revenue;

 vector<map<string, string> > inventory_data;
 vector<map<string, string> > expense_data;
 vector<map<string, string> > forward_data;
 vector<map<string, string> > revenue_data;

 map<string, map<string, financial> > prior_transactions;

 // functions
 public:
  reusable_accounting_insert_parameters () :
   target_amount(0.00),
   is_change (false),
   show_inventory (false),
   show_expense (false),
   show_forward (false),
   show_revenue (false) {};

  void ImportOldTransactions(vector<map<string, string> > & prior_data);

  map<string, map<string, financial> > GetNewRawTransactions ();
  map<string, map<string, financial> > GetOldRawTransactions ();
  map<string, map<string, financial> > GetAllRawTransactions ();

  // final transactions have debits and credits, hopefully in the right places
  vector<map<string, string> > GetFinalTransactions (map<string, string> default_row, map<string, map<string, financial> > change_data);
  vector<map<string, string> > GetNewFinalTransactions (map<string, string> default_row);
  vector<map<string, string> > GetOldFinalTransactions (map<string, string> default_row);
  vector<map<string, string> > GetAllFinalTransactions (map<string, string> default_row);

  // preview transactions just have an 'amount' field
  vector<map<string, string> > GetPreviewTransactions (map<string, map<string, financial> > change_data);
  vector<map<string, string> > GetNewPreviewTransactions ();
  vector<map<string, string> > GetOldPreviewTransactions ();
  vector<map<string, string> > GetAllPreviewTransactions ();

  static void SetupTransactionSummaryView (edit_listview & listview_handler, bool show_account_name, bool show_tax_entity_name);
  static void RefreshTransactionSummaryView (edit_listview & listview_handler, bool show_account_name, bool show_tax_entity_name, vector<map<string, string> > & preview_data);

 private:
  void AddFinancial (map<string, map<string, financial> > & data, string a, string b, financial c);
  void SubFinancial (map<string, map<string, financial> > & data, string a, string b, financial c);
};
//---------------------------------------------------------------------------
class TformSpecialReusableAccountingInsert : public TformSpecialGeneric
{
__published:	// IDE-managed Components
        TToolButton *buttonSave;
        TToolButton *buttonCancel;
        TPageControl *pageLeft;
        TTabSheet *sheetInventory;
        TTabSheet *sheetExpense;
        TTabSheet *sheetForward;
        TTabSheet *sheetRevenue;
        TToolButton *buttonInventory;
        TToolButton *sepLeftTabs;
        TToolButton *buttonExpense;
        TToolButton *buttonForward;
        TToolButton *buttonRevenue;
        TGroupBox *groupInventory;
        TLabel *_labelInventoryDollar0;
        TLabel *_labelInventoryCost;
        TLabel *_labelInventoryItemType;
        TLabel *_labelInventoryQuantity;
        TLabel *_labelInventoryUniqueNumber;
        TLabel *labelInventoryTimesQuantityType;
        TLabel *labelInventoryPerQuantityType;
        TButton *buttonInventoryAdd;
        TButton *buttonInventoryRemove;
        TEdit *editInventoryCost;
        TButton *buttonInventorySave;
        TListView *listInventory;
        TEdit *editInventoryQuantity;
        TEdit *editInventoryUniqueNumber;
        TBitBtn *buttonInventorySearchItemType;
        TStaticText *staticInventoryItemType;
        TGroupBox *groupExpense;
        TLabel *_labelExpenseAccount;
        TLabel *_labelExpenseDollar0;
        TLabel *_labelExpensePayable;
        TLabel *_labelExpensePaid;
        TLabel *_labelExpenseDollar1;
        TListView *listExpense;
        TButton *buttonExpenseAdd;
        TButton *buttonExpenseRemove;
        TEdit *editExpenseAccount;
        TButton *buttonExpenseExpandAccount;
        TEdit *editExpensePayable;
        TEdit *editExpensePaid;
        TButton *buttonExpenseSave;
        TBitBtn *buttonExpenseSearchAccount;
        TGroupBox *groupForward;
        TLabel *_labelForwardTaxEntityFrom;
        TLabel *_labelForwardAccount;
        TLabel *_labelForwardReceivable;
        TLabel *_labelReceivableDollar0;
        TLabel *_labelForwardDollar1;
        TLabel *_labelForwardReceived;
        TLabel *_labelForwardPaid;
        TLabel *_labelForwardDollar2;
        TButton *buttonForwardAdd;
        TButton *buttonForwardRemove;
        TListView *listForward;
        TStaticText *staticForwardTaxEntityFrom;
        TBitBtn *buttonForwardSearchTaxEntityFrom;
        TButton *buttonForwardExpandAccount;
        TEdit *editForwardAccount;
        TEdit *editForwardReceivable;
        TEdit *editForwardReceived;
        TButton *buttonForwardSave;
        TBitBtn *buttonForwardSearchAccount;
        TEdit *editForwardPaid;
        TGroupBox *groupRevenue;
        TLabel *_labelRevenueTaxEntity;
        TLabel *_labelRevenueAccount;
        TLabel *_labelRevenueReceivable;
        TLabel *_labelRevenueDollar0;
        TLabel *_labelRevenueDollar1;
        TLabel *_labelRevenueReceived;
        TButton *buttonRevenueAdd;
        TButton *buttonRevenueRemove;
        TListView *listRevenue;
        TStaticText *staticRevenueTaxEntity;
        TBitBtn *buttonRevenueSearchTaxEntity;
        TButton *buttonRevenueExpandAccount;
        TEdit *editRevenueAccount;
        TEdit *editRevenueReceivable;
        TEdit *editRevenueReceived;
        TBitBtn *buttonRevenueSearchAccount;
        TLabel *_labelExpenseTaxEntity;
        TStaticText *staticExpenseTaxEntity;
        TBitBtn *buttonExpenseSearchTaxEntity;
        TLabel *_labelForwardTaxEntityTo;
        TStaticText *staticForwardTaxEntityTo;
        TBitBtn *buttonForwardSearchTaxEntityTo;
        TTabSheet *sheetResult;
        TLabel *_labelInventoryTaxEntity;
        TStaticText *staticInventoryTaxEntity;
        TBitBtn *buttonInventorySearchTaxEntity;
        TLabel *_labelExpenseAccountType;
        TLabel *_labelForwardAccountType;
        TLabel *_labelRevenueAccountType;
        TToolButton *buttonResult;
        TButton *buttonExpenseClear;
        TButton *buttonForwardClear;
        TButton *buttonRevenueClear;
        TButton *buttonInventoryClear;
        TPageControl *pageTransactions;
        TTabSheet *sheetOld;
        TListView *listBefore;
        TTabSheet *sheetNew;
        TListView *listChange;
        TTabSheet *sheetAll;
        TListView *listAfter;
        TGroupBox *groupSummaryTree;
        TTreeView *treeSummary;
        TTabSheet *sheetBudgets;
        TListView *listBudgets;
        TCheckBox *checkBeforeShowAccount;
        TCheckBox *checkBeforeShowName;
        TCheckBox *checkChangesShowAccount;
        TCheckBox *checkChangesShowName;
        TCheckBox *checkAfterShowAccount;
        TCheckBox *checkAfterShowName;
        TLabel *labelSplitBefore;
        TLabel *labelSplitChange;
        TLabel *labelSplitAfter;
        TButton *buttonRevenueSave;
        void __fastcall buttonSaveClick(TObject *Sender);
        void __fastcall buttonCancelClick(TObject *Sender);
        void __fastcall buttonInventoryClick(TObject *Sender);
        void __fastcall buttonExpenseClick(TObject *Sender);
        void __fastcall buttonForwardClick(TObject *Sender);
        void __fastcall buttonRevenueClick(TObject *Sender);
        void __fastcall buttonInventoryAddClick(TObject *Sender);
        void __fastcall buttonInventoryRemoveClick(TObject *Sender);
        void __fastcall buttonInventorySaveClick(TObject *Sender);
        void __fastcall buttonInventorySearchTaxEntityClick(
          TObject *Sender);
        void __fastcall buttonInventorySearchItemTypeClick(
          TObject *Sender);
        void __fastcall buttonExpenseAddClick(TObject *Sender);
        void __fastcall buttonExpenseRemoveClick(TObject *Sender);
        void __fastcall buttonExpenseSaveClick(TObject *Sender);
        void __fastcall buttonExpenseExpandAccountClick(TObject *Sender);
        void __fastcall buttonExpenseSearchAccountClick(TObject *Sender);
        void __fastcall buttonExpenseSearchTaxEntityClick(TObject *Sender);
        void __fastcall buttonForwardAddClick(TObject *Sender);
        void __fastcall buttonForwardRemoveClick(TObject *Sender);
        void __fastcall buttonForwardSaveClick(TObject *Sender);
        void __fastcall buttonForwardSearchTaxEntityFromClick(
          TObject *Sender);
        void __fastcall buttonForwardExpandAccountClick(TObject *Sender);
        void __fastcall buttonForwardSearchAccountClick(TObject *Sender);
        void __fastcall buttonForwardSearchTaxEntityToClick(
          TObject *Sender);
        void __fastcall buttonRevenueAddClick(TObject *Sender);
        void __fastcall buttonRevenueRemoveClick(TObject *Sender);
        void __fastcall buttonRevenueSaveClick(TObject *Sender);
        void __fastcall buttonRevenueSearchTaxEntityClick(TObject *Sender);
        void __fastcall buttonRevenueExpandAccountClick(TObject *Sender);
        void __fastcall buttonRevenueSearchAccountClick(TObject *Sender);
        void __fastcall FormClose(TObject *Sender, TCloseAction &Action);
        void __fastcall buttonInventoryClearClick(TObject *Sender);
        void __fastcall buttonExpenseClearClick(TObject *Sender);
        void __fastcall buttonForwardClearClick(TObject *Sender);
        void __fastcall buttonRevenueClearClick(TObject *Sender);
        void __fastcall buttonResultClick(TObject *Sender);
        void __fastcall treeSummaryExpanded(TObject *Sender,
          TTreeNode *Node);
        void __fastcall treeSummaryCollapsed(TObject *Sender,
          TTreeNode *Node);
        void __fastcall treeSummaryAdvancedCustomDrawItem(
          TCustomTreeView *Sender, TTreeNode *Node, TCustomDrawState State,
          TCustomDrawStage Stage, bool &PaintImages, bool &DefaultDraw);
        void __fastcall FormResize(TObject *Sender);
        void __fastcall FormShortCut(TWMKey &Msg, bool &Handled);
        void __fastcall checkBeforeClick(TObject *Sender);
        void __fastcall checkChangeClick(TObject *Sender);
        void __fastcall checkAfterClick(TObject *Sender);
private:	// User declarations
        edit_listview edit_inventory;
        edit_listview edit_expense;
        edit_listview edit_forward;
        edit_listview edit_revenue;
        edit_listview edit_before;
        edit_listview edit_change;
        edit_listview edit_after;
        edit_listview edit_budget;

        reusable_accounting_insert_parameters * params;

        string current_inventory_tax_entity_id;
        string current_expense_tax_entity_id;
        string current_forward_tax_entity_id_from;
        string current_forward_tax_entity_id_to;
        string current_revenue_tax_entity_id;

        string current_expense_account_id; // do not rely on these when scraping screen: typing doesn't change 'em!
        string current_forward_account_id;
        string current_revenue_account_id;

        vector<map<string, string> > before_data;
        vector<map<string, string> > change_data;
        vector<map<string, string> > after_data;

        financial chg_expense;
        financial chg_expense_payable;
        financial chg_expense_paid;
        financial chg_forward;
        financial chg_forward_payable;
        financial chg_forward_paid;
        financial chg_forward_receivable;
        financial chg_forward_received;
        financial chg_revenue;
        financial chg_revenue_receivable;
        financial chg_revenue_received;
        financial total_expense;
        financial total_payable;
        financial total_paid;
        financial total_revenue;
        financial total_receivable;
        financial total_received;
        financial total_payment;
        financial total_target;

        void InventoryRowSelected  ();
        void ExpenseRowSelected    ();
        void ForwardRowSelected    ();
        void RevenueRowSelected    ();

        void RefreshSummary        ();
protected:
public:		// User declarations
        __fastcall TformSpecialReusableAccountingInsert(TComponent* Owner, reusable_accounting_insert_parameters * p_params);
};
//---------------------------------------------------------------------------
extern PACKAGE TformSpecialReusableAccountingInsert *formSpecialReusableAccountingInsert;
//---------------------------------------------------------------------------
#endif
