//---------------------------------------------------------------------------

#include <vcl.h>
#pragma hdrstop

#include "codeSpecialSqlViewer.h"
#include "codeSpecialDataQuery.h"
//---------------------------------------------------------------------------
#pragma package(smart_init)
#pragma link "codeSpecialGeneric"
#pragma resource "*.dfm"
TformSpecialSqlViewer *formSpecialSqlViewer;
//---------------------------------------------------------------------------
__fastcall TformSpecialSqlViewer::TformSpecialSqlViewer(TComponent* Owner, string* sql, TIBTransaction* t)
        : TformSpecialGeneric(Owner)
{
  // jhaley 20101007: Just calling DebugSql can mask bugs in SQL queries via PrettySql reformatting the string by
  // adding \n's after certain keywords. That's undesirable, since you'd expect the SQL to run unmodified unless you
  // manually edited it. So to that end, save the original SQL passed in and allow it to be used with the "Use Original SQL"
  // button.
  original_sql = *sql;
  editSQL->Text = PrettySql(*sql).c_str();
  sql_string = sql;
  DQTransaction = t;
  if (t)
  {
    sepDataQuery->Visible = true;
    btnDataQuery->Visible = true;
  }
}
//---------------------------------------------------------------------------
void __fastcall TformSpecialSqlViewer::btnOkClick(TObject *Sender)
{
  string sql = editSQL->Text.c_str();
  if (!HasPermissionToExecute())
    PSay("For security reasons, you cannot execute this SQL statement from this window", m_warning, MB_OK);
  else
   (*sql_string) = sql;

  Close();
}
//---------------------------------------------------------------------------
void __fastcall TformSpecialSqlViewer::btnCheckSQLClick(TObject *Sender)
{
   string sql = editSQL->Text.c_str();
   if (!HasPermissionToExecute())
     PSay("For security reasons, you cannot execute this SQL statement from this window", m_warning, MB_OK);
   else
   {
      // Execute statement
      bool executes_ok = true;
      string plan;
      TIBDataSet* dbDataSet = new TIBDataSet(NULL);
      TIBTransaction * dbTransaction = new TIBTransaction(NULL);
      try
      {
        if(StdTransaction(dbTransaction, dbDatabase))
        {
          dbDataSet->Database = dbDatabase;
          dbDataSet->Transaction = dbTransaction;
          dbDataSet->SelectSQL->Text = editSQL->Text;
          dbDataSet->Prepare();
          plan = dbDataSet->Plan.c_str();
          plan = TranslateSqlPlan(dbTransaction, plan); // 'cause ... prettier?
          dbTransaction->Commit();
        }
        else
        {
          PSay("Error setting up transaction", m_database_error, MB_OK);
          executes_ok = false;
        }
      }
      catch (EIBInterBaseError & the_error)
      {
        PSay((the_error.Message).c_str(), m_database_error, MB_OK);
        executes_ok = false;
      }
      catch (...)
      {
        executes_ok = false;
      }
      if (dbTransaction->Active)
        dbTransaction->Rollback();
      delete dbDataSet;
      delete dbTransaction;
      // End execute statement
      if (executes_ok)
         PSay("SQL validated successfully!\n\nPlan produced:\n\n" + plan, m_message, MB_OK);
      else
         PSay("SQL -failed- to validate successfully!", m_message, MB_OK);
   }
}
//---------------------------------------------------------------------------
void __fastcall TformSpecialSqlViewer::btnCancelClick(TObject *Sender)
{
//   int answer = PSay("You will lose any changes you've made to the SQL.  Continue?", m_warning, MB_YESNO);
//   if (answer == IDYES)
      Close();
}
//---------------------------------------------------------------------------
void __fastcall TformSpecialSqlViewer::editSQLSelectionChange(TObject *Sender)
{
   this->Caption = ("SQL Viewer (cursor row: " + IntToString(editSQL->CaretPos.y+1) +
      "; col: " + IntToString(editSQL->CaretPos.x+1) + "; abs. pos: " + IntToString(editSQL->SelStart) + ")").c_str();
}
//---------------------------------------------------------------------------
void __fastcall TformSpecialSqlViewer::FormResize(TObject *Sender)
{
   editSQL->Invalidate();
   editSQLSelectionChange(NULL);
   TformSpecialGeneric::FormResize(Sender);
}
//---------------------------------------------------------------------------
bool TformSpecialSqlViewer::HasPermissionToExecute()
{
//string query_type = LowercaseString(StripLeading(sql, " ").substr(0, 6));
/* return (((optionsPermissions["special.modify_listview_sql_select"]["enabled"] == "true") && (query_type == "select")) ||
         ((optionsPermissions["special.modify_listview_sql_update"]["enabled"] == "true") && (query_type == "update")) ||
         ((optionsPermissions["special.modify_listview_sql_insert"]["enabled"] == "true") && (query_type == "insert")) );
*/
 return optionsPermissions["special.sql_viewer_execute"]["enabled"] == "true";
}
//---------------------------------------------------------------------------
TformSpecialSqlViewer *TformSpecialSqlViewer::Launch(HowToLaunch launch_as, TformSpecialGeneric *modalParent, string *sql, TIBTransaction *t)
{
  TformSpecialSqlViewer *viewer = NULL;
  FORM_TRY
    viewer = new TformSpecialSqlViewer(NULL, sql, t);
    switch (launch_as)
    {
      case non_modal:   viewer->Show(); break;
      case modal:       viewer->ShowModal(); break;
      case task_modal:  viewer->ShowTaskModal(modalParent); break;
    }
  FORM_CATCH
  return viewer;
}
//---------------------------------------------------------------------------
void __fastcall TformSpecialSqlViewer::editSQLKeyPress(TObject *Sender, char &Key)
{
 // rich edit traps escape?  screw that!

/*  fuck it, I'll figure it out later
 Handled = false;

 if(!HIBYTE(GetAsyncKeyState(VK_CONTROL)) && (Msg.CharCode == VK_ESCAPE))
 {
  Handled = true;
  buttonEscapeClick(NULL);
 }
*/
}
//---------------------------------------------------------------------------
void __fastcall TformSpecialSqlViewer::btnDataQueryClick(TObject *Sender)
{
  string sql = editSQL->Text.c_str();
  // maybe I could just include derivedheaders, but it includes this form and I wasn't up
  // for dealing with a dependency loop at the moment
  //QueryData(this, DQTransaction, &sql);
  if (TformSpecialDataQuery::HasPermissionToExecute())
    TformSpecialDataQuery::Launch(this, DQTransaction, sql);
}
//---------------------------------------------------------------------------
void __fastcall TformSpecialSqlViewer::btnRunUnmodifiedClick(TObject *Sender)
{
  // jhaley 20101007: Button to run the original, unmodified SQL that was passed in, for debugging purposes.
  // Otherwise you can get side effects, and thorough testing would require turning the debug level up and down
  // constantly.
  if (IDNO == PSay("This will discard any changes you made. Are you sure?", m_warning, MB_OK))
    return;

  if (!HasPermissionToExecute())
    PSay("For security reasons, you cannot execute this SQL statement from this window", m_warning, MB_OK);
  else
   (*sql_string) = original_sql;

  Close();
}
//---------------------------------------------------------------------------

