//---------------------------------------------------------------------------
#include <vcl.h>
#pragma hdrstop

#include "codeSpecialTerminateInsurancePolicy.h"
#include "codeRecordInsuranceCoverage.h"
#include "derivedheaders.h"
//---------------------------------------------------------------------------
#pragma package(smart_init)
#pragma link "codeSpecialGeneric"
#pragma link "codeRubberContainer"
#pragma resource "*.dfm"
TformSpecialTerminateInsurancePolicy *formSpecialTerminateInsurancePolicy;
//---------------------------------------------------------------------------
__fastcall TformSpecialTerminateInsurancePolicy::TformSpecialTerminateInsurancePolicy(TComponent* Owner, string id, const set<string> &coverage_ids)
        : TformSpecialGeneric(Owner)
{
  policy_id = id;

  dateTerminated->Date = Date();
  AdjustHeight(CalcHeight());
  string company_sql = "select TE.id from tax_entities TE "
                       "inner join insurance_policies IP on IP.tax_entity_id = TE.id "
                       "where IP.id = '" + policy_id + "'";
  string company = GetOneField(dbDatabase, company_sql);
  CreateLink(txtCompany, NULL, this, company, "tax_entities");

  // Wanted what I inserted to be bold, but it didn't feel like working today
  // richInstructions->DefAttributes->Style << fsBold;

  coverage_filter.SetListViewHandle (listCoverages);
  coverage_filter.SetSecurity("insurance_coverage");
  coverage_filter.AddColumnSpec("Name", "people", as_text, true, 140);
  coverage_filter.AddColumnSpec("Company", "tax_entities", as_text, false, 100, rawString);
  coverage_filter.AddColumnSpec("Medical Type", "tax_entities", as_text, true, 60);
  coverage_filter.AddColumnSpec("Policy Number", "insurance_policies", as_text, true, 100, rawString);
  coverage_filter.AddColumnSpec("Group Number", "insurance_policies", as_text, true, 90, rawString);
  coverage_filter.AddColumnSpec("Effective Date", "insurance_policies", as_date, true, 90);
  coverage_filter.AddColumnSpec("Termination Date", "insurance_policies", as_date, false, 90);
  coverage_filter.AddColumnSpec("Coverage", "insurance_policies", as_text, false, 120);
  coverage_filter.AddColumnSpec("Source", "insurance_policies", as_text, false, 120);
  coverage_filter.AddColumnSpec("Name on Policy", "insurance_policies", as_text, false, 120);
  coverage_filter.AddColumnSpec("PCP", "tax_entities", as_text, true, 120, rawString);
  coverage_filter.AddColumnSpec("Person Code", "insurance_coverage", as_int, false, 100);
  coverage_filter.AddColumnSpec("Name on Card", "insurance_coverage", as_text, false, 100);
  coverage_filter.AddColumnSpec("Current Id", "current_id", as_int, false, 80);
  coverage_filter.AddColumnSpec("Internal Id", "internal_id", as_int, false, 80);
  coverage_filter.FetchPreferences("specialterminateinsurancepolicy.coverage_filter");
  coverage_filter.SetCompress(false);
  coverage_filter.SetIconStatus(true);
  coverage_filter.SetHeader(true);
  coverage_filter.SetCheck(true);

  string sql =
    "select first_name || "
    "  (case when middle_name != '' then (' ' || middle_name) else '' end) || ' ' || last_name || "
    "  (case when suffix != '' then (' ' || suffix) else '' end) name, "
    " ins_co.name, insurance_types.item, insurance_policies.policy_number, insurance_policies.group_number, "
    " insurance_coverage.start_date, insurance_coverage.stop_date, coverage_types.item, "
    " source_types.item, insurance_policies.name_on_policy, pcp.name, insurance_coverage.person_code, "
    " insurance_coverage.name_on_card, insurance_coverage.current_id, insurance_coverage.id "
    "from insurance_policies "
    "left join insurance_coverage on insurance_policies.id = insurance_coverage.insurance_policy_id "
    "left join coverage_types on coverage_types.id = insurance_policies.coverage_id "
    "left join source_types on source_types.id = insurance_policies.source_id "
    "left join insurance_types on insurance_types.id = insurance_policies.insurance_id "
    "left join tax_entities ins_co on ins_co.id = insurance_policies.tax_entity_id "
    "left join tax_entities pcp on pcp.id = insurance_coverage.tax_entity_id "
    "left join people on insurance_coverage.person_id = people.id "
    "left join person_names on people.person_names_id = person_names.id "
    "where insurance_policies.id = '" + policy_id + "' and insurance_coverage.current_id >= 3 and "
    " insurance_coverage.stop_date is null";

  DebugSql(this, &sql, other_forms, 9);
  coverage_filter.SetSQLStatement(sql);
  coverage_filter.RefreshDataSet();
  coverage_filter.SetAddEdit(this, NULL, EditRecord(&(TformSpecialTerminateInsurancePolicy::EditCoverage)));
  coverage_filter.SetCheckedWhereID(coverage_ids, true);
  coverage_request = atlas->RequestNotificationWhen("insurance_coverage.insurance_policy_id", policy_id, rel_e, as_int, this,
                       RefreshHandler(&(TformSpecialTerminateInsurancePolicy::AtlasRefreshListViews)));

  while(listCoverages->VisibleRowCount < listCoverages->Items->Count
        && (this->Height + 8) < Screen->Height) // just in case
  {
    this->Constraints->MinHeight = this->Constraints->MinHeight + 8;
  }

  // TODO -c HACK: This shouldn't be necessary, but out in the field, I've seen the listview not big enough to see a
  // single item.
  this->Height = this->Constraints->MinHeight + 40;
  this->ActiveControl = dateTerminated;
}
//---------------------------------------------------------------------------
void __fastcall TformSpecialTerminateInsurancePolicy::btnCancelClick(TObject *Sender)
{
  Close();
}
//---------------------------------------------------------------------------
void __fastcall TformSpecialTerminateInsurancePolicy::btnSaveClick(TObject *Sender)
{
  Processing(true);
  string term_date = Pdate(dateTerminated->Date).OutDate(mdy | yyyy);

  TIBTransaction *terminate_transaction = new TIBTransaction(NULL);

  if(StdTransaction(terminate_transaction, dbDatabase))
  {
    try // if anything fails, we roll it all back
    {
      // GetMultiID ... if size of result (selected IDs) is the same as the number of rows (GetDatasetSize) then
      // we've selected every coverage under this policy ... so terminate the policy too.
      vector<string> selected_insurance_coverages;
      selected_insurance_coverages = coverage_filter.GetMultiID();

      if(selected_insurance_coverages.size() == coverage_filter.GetDatasetSize())
      {
        string policy_sql =
          "update insurance_policies set current_id = '" + lookupTables->IdOf("current_types", "Expired") + "', "
          " last_updated_by = '" + current_userid + "', last_update_stamp = '" + RightNow() + "', stop_date = '" + term_date + "' "
          "where id = '" + policy_id + "'";
        DebugSql(this, &policy_sql, other_forms, 9);
        if(!ExecuteStatement(terminate_transaction, policy_sql))
          throw 1;
      }

      // update all the insurance coverages at once.  Fantastic.
      string coverage_sql =
        "update insurance_coverage set current_id = '" + lookupTables->IdOf("current_types", "Expired") + "', "
        " last_updated_by = '" + current_userid + "', last_update_stamp = '" + RightNow() + "', stop_date = '" + term_date + "' "
        "where " + SafeInClause("id", selected_insurance_coverages);
      DebugSql(this, &coverage_sql, other_forms, 9);
      if(!ExecuteStatement(terminate_transaction, coverage_sql))
        throw 2;

      // Still here?  Good.  Commit and notify Atlas of changes.
      if (terminate_transaction->Active)
        terminate_transaction->Commit();

      atlas->Notify("insurance_policies.id", policy_id);
      atlas->Notify("insurance_coverage.insurance_policy_id", policy_id);
      for(unsigned int i = 0; i < selected_insurance_coverages.size(); i++)
      {
        atlas->Notify("insurance_coverage.id", selected_insurance_coverages[i]);
        // record person needs love too
        string person_id = GetOneField(dbDatabase, "select person_id from insurance_coverage where id = '" + selected_insurance_coverages[i] + "'");
        if (!IsBlankOrZero(person_id))
          atlas->Notify("insurance_coverage.person_id", person_id);
      }

      Processing(false);
      Close();
    }
    catch (int i)
    {
      if(terminate_transaction->Active)
        terminate_transaction->Rollback();
      switch (i)
      {
        case 1:
          PSay("Unable to update insurance policy to reflect termination.\n\n"
               "Check to make sure you do not have the Insurance Policy screen unlocked.  "
               "If you do not, ask around the office to see if anyone else does.", m_message, MB_OK);
          break;
        case 2:
          PSay("Unable to update insurance coverages to reflect termination.\n\n"
               "Check to make sure you do not have the Insurance Coverage screen unlocked for any of the people "
               "covered under this policy.  If you do not, ask around the office to see if anyone else does.",
               m_message, MB_OK);
          break;
      }
    }
    catch(...)
    {
      if(terminate_transaction->Active)
        terminate_transaction->Rollback();
      PSay("An unknown error prevented termination of this policy.  Contact a programmer for assistance.", m_program_error, MB_OK);
    }
  }
  else
    PSay("Unable to create transaction.  Action was not successfully completed.", m_program_error, MB_OK);

  Processing(false);
}
//---------------------------------------------------------------------------
void TformSpecialTerminateInsurancePolicy::EditCoverage()
{
  pmapstrs temp_map;
  temp_map["id"] = coverage_filter.GetSelectedRecordID();

  TformRecordInsuranceCoverage *coverageRecord = NULL;
  FORM_TRY
     coverageRecord = new TformRecordInsuranceCoverage(NULL, temp_map);
  FORM_CATCH_RETURN("TformRecordInsuranceCoverage", )
  coverageRecord->Show();
}
//---------------------------------------------------------------------------
bool TformSpecialTerminateInsurancePolicy::AtlasRefreshListViews(int request_num, string matched_value)
{
  if (request_num == coverage_request)
    coverage_filter.RefreshDataSet();
  return true;
}
//---------------------------------------------------------------------------
void __fastcall TformSpecialTerminateInsurancePolicy::FormResize(TObject *Sender)
{
/*Application->ProcessMessages();
  lblDynamic1->AutoSize = false;
  lblDynamic3->AutoSize = false;
*/
}
//---------------------------------------------------------------------------
void __fastcall TformSpecialTerminateInsurancePolicy::FormCanResize(TObject *Sender, int &NewWidth, int &NewHeight, bool &Resize)
{
/*
  lblDynamic1->AutoSize = true;
  lblDynamic3->AutoSize = true;
  Resize = true;
*/
}
//---------------------------------------------------------------------------

