//---------------------------------------------------------------------------

#include <vcl.h>
#pragma hdrstop

#include "codeSpecialTransactionDetails.h"
//---------------------------------------------------------------------------
#pragma package(smart_init)
#pragma link "codeSpecialListview"
#pragma resource "*.dfm"
TformSpecialTransactionDetails *formSpecialTransactionDetails;
//---------------------------------------------------------------------------
__fastcall TformSpecialTransactionDetails::TformSpecialTransactionDetails(TComponent* Owner, const string &p_parent_record_id, const string &p_refresh_foreign_key)
        : TformSpecialListview(Owner)
{
  parent_record_id    = p_parent_record_id;     // id of the record whose form brought us here, for Atlas refresh purposes
  refresh_foreign_key = p_refresh_foreign_key;  // foreign key field in transactions which refers to the above id.

  // set table name to transactions - you should not override this yourself, as it won't work then.
  SourceTableName = "transactions";

  // default properties of the filter - make it unnecessary to set these at every call point:
  filter_stuff.SetSecurity("transactions");  // always transactions
  filter_stuff.SetIconStatus(true);          // always have icons
}
//---------------------------------------------------------------------------
__fastcall TformSpecialTransactionDetails::~TformSpecialTransactionDetails()
{
  // delete the private transaction object if it exists
  if (dbTransaction)
  {
    if (dbTransaction->Active)
      dbTransaction->Rollback(); // read-only so I don't think it matters too much...
    delete dbTransaction;
    dbTransaction = NULL;
  }
}
//---------------------------------------------------------------------------
// jhaley 20100907: This overload of Execute is inherited from the parent class. It is made protected
// in this subclass to avoid having an external transaction passed in, as this form is not task modal and
// uses Atlas refresh, and thus could persist beyond the allocation lifetime of the launching form.
void TformSpecialTransactionDetails::Execute(TIBTransaction* dbTransaction, string sql_statement)
{
  // call the superclass implementation to populate the filter
  TformSpecialListview::Execute(dbTransaction, sql_statement);

  // register Atlas notification
  string transaction_foreign_key = "transactions." + refresh_foreign_key;
  transactions_request = atlas->RequestNotificationWhen(transaction_foreign_key, parent_record_id, rel_e, as_int, this,
                                                        RefreshHandler(&(TformSpecialTransactionDetails::AtlasRefreshListView)));

  // override the form caption set in TformSpecialListview::Execute for relevance to this subclass
  this->Caption = string("Transaction Details - " + IntToString(filter_stuff.GetResultSize()) + " entries").c_str();
}
//---------------------------------------------------------------------------
// jhaley 20100907: This overload of Execute is the only public one for this class. See above for the override of the
// parent class version. Pass in the SQL query for the filter.
void TformSpecialTransactionDetails::Execute(string sql_statement)
{
  // create a private transaction object.
  if (!dbTransaction)
  {
    dbTransaction = new TIBTransaction(NULL);
    dbTransaction->DefaultDatabase = dbDatabase;
    dbTransaction->StartTransaction();
  }

  // call the normal version of this function overridden from the base class
  Execute(dbTransaction, sql_statement);
}
//---------------------------------------------------------------------------
void TformSpecialTransactionDetails::AtlasRefreshListView(int request_num, string matched_value)
{
  if (request_num == transactions_request)
    filter_stuff.RefreshDataSet();
}
//---------------------------------------------------------------------------

