//---------------------------------------------------------------------------

#include <vcl.h>
#pragma hdrstop

#include "codeSpecialTransactionEntry.h"
#include "control_break_objects.h"
#include "codeRecordAccount.h"
#include "codeSearchAccount.h"
#include "codeSearchTaxEntity.h"

//---------------------------------------------------------------------------
#pragma package(smart_init)
#pragma link "codeSpecialGeneric"
#pragma resource "*.dfm"
TformSpecialTransactionEntry *formSpecialTransactionEntry;
//---------------------------------------------------------------------------
__fastcall TformSpecialTransactionEntry::TformSpecialTransactionEntry(TComponent* Owner, const transaction_summary & p_original_data, transaction_summary * p_current_data, string p_default_tax_entity_id, financial p_default_amount, bool p_use_entities)
        : TformSpecialGeneric(Owner)
{
 // initial data
 use_entities = p_use_entities;

 return_current_data = p_current_data;
 original_data = p_original_data;
 current_data  = *p_current_data;
 current_tax_entity_id = default_tax_entity_id = p_default_tax_entity_id;
 default_amount = p_default_amount;
 was_balanced = true;

 if (!use_entities)
 {
  checkShowName->Checked = false;
  checkShowName->Enabled = false;
  buttonSearchTaxEntity->Enabled = false;
  default_tax_entity_id = "0";
  current_tax_entity_id = "0";
  original_data.RemoveEntities();
  current_data.RemoveEntities();
 }

 RefreshFrequentAccounts();

 // controls
 edit_budgets.setup_listview(listBudgets, false);
 edit_budgets.add_or_modify_column("account_name", "Account", true, 1, 130, as_text, rawString);
 edit_budgets.add_or_modify_column("bud_a", "Budg. before orig. exp.", true, 2, 75, as_double, rawString);
 edit_budgets.add_or_modify_column("exp_b", "Exp., orig.", true, 3, 75, as_double, rawString);
 edit_budgets.add_or_modify_column("bud_c", "Budg. after orig. exp.", true, 4, 75, as_double, rawString);
 edit_budgets.add_or_modify_column("exp_diff", "Exp., difference", true, 5, 75, as_double, rawString);
 edit_budgets.add_or_modify_column("exp_d", "Exp., modified", true, 6, 75, as_double, rawString);
 edit_budgets.add_or_modify_column("bud_e", "Budg. after mod. exp.", true, 7, 75, as_double, rawString);

 edit_transactions.setup_listview(listTransactions, false);
 edit_transactions.setup_callbacks((TForm*)this, InformForm(&(TformSpecialTransactionEntry::TransactionRowSelected)), NULL);
 MaskSetup (editAccount);
 CreateLink (staticTaxEntity, buttonSearchTaxEntity, this, current_tax_entity_id, "tax_entities", "table.tax_entities", use_entities);

 // camera, lights, action!
 checkShowClick(NULL);
 TransactionRowSelected();
}
//---------------------------------------------------------------------------
void __fastcall TformSpecialTransactionEntry::checkShowClick(TObject *Sender)
{
 // changed the way stuff is grouped -- if not both [x], disable buttons ... selecting rows should do nothing
 // (this is because the rows are no longer distinct)

 RefreshTransactionSummaryView (edit_transactions, checkShowAccount->Checked, (checkShowName->Checked && use_entities), current_data.GetPreviewOutput(false));
 // RefreshBudgets();

 bool buttons_enabled = (checkShowAccount->Checked && (checkShowName->Checked || !use_entities));
 buttonTransactionsAdd->Enabled = buttons_enabled;
 buttonTransactionsRemove->Enabled = buttons_enabled;
 // buttonTransactionsClear->Enabled = buttons_enabled;
 // buttonTransactionsRevert->Enabled = buttons_enabled;
 buttonTransactionsSave->Enabled = buttons_enabled;
 return;
}
//---------------------------------------------------------------------------
void __fastcall TformSpecialTransactionEntry::buttonTransactionsAddClick(TObject *Sender)
{
 // add blank row
 if (listTransactions->Items->Count)
  listTransactions->Items->Item[0]->Selected = true;
 listTransactions->Selected = NULL;
 return;
}
//---------------------------------------------------------------------------
void __fastcall TformSpecialTransactionEntry::buttonTransactionsRemoveClick(TObject *Sender)
{
 // remove current row (and associated items)
 // ... we don't actually remove it, we just subtract it. yes, it'll show a zero value. darn.
 if (listTransactions->Selected)
 {
  map<string, string> current_row;
  edit_transactions.get_current_row(current_row);
  if (current_row["tax_entity_id"].empty())
   current_row["tax_entity_id"] = "0";
  if (current_row["account_id"].empty())
   current_row["account_id"] = "0";
  string account_to_clear = current_row["account_id"];
  string entity_to_clear = current_row["tax_entity_id"];
  if (current_data.ChangeAmountTo(account_to_clear, entity_to_clear, 0.00, !checkDisableMagic->Checked))
  {
   checkShowClick(NULL);
   CheckCurrentBalance ();
  }
  else
   PSay("You may not modify the amount of this account/entity pair directly.\nPlease edit only expense, revenue, payable, and receivable entries.", m_invalid_data, MB_OK);
 }
 return;
}
//---------------------------------------------------------------------------
void __fastcall TformSpecialTransactionEntry::buttonTransactionsClearClick(TObject *Sender)
{
 // clear everything (somewhat of a doozie!)
 transaction_summary empty_summary;
 current_data = empty_summary;
 checkShowClick(NULL);
 CheckCurrentBalance ();
 return;
}
//---------------------------------------------------------------------------
void __fastcall TformSpecialTransactionEntry::buttonTransactionsRevertClick(TObject *Sender)
{
 // go back to original data ... easy, right?
 if (IDNO == PSay("Reverting will use the information still in the database to undo your changes (complete undo.)\nIf you only wish to cancel your very latest changes, press 'cancel', then re-open this screen if needed.\nAre you sure you want to revert to the original transaction data?", m_confirmation, MB_YESNO))
  return;

 current_data = original_data;
 checkShowClick(NULL);
 CheckCurrentBalance ();
 return;
}
//---------------------------------------------------------------------------
void __fastcall TformSpecialTransactionEntry::buttonTransactionsSaveClick(TObject *Sender)
{
 // save the current information, if possible (not a cash account, etc.)
 acc_info about_account = InfoByMask(editAccount, false);
 if (about_account.IsFound())
 {
  if ((!use_entities) || (use_entities && current_tax_entity_id != "0")) // prevent blank entities when we want them
  {
   if (use_entities || (!use_entities && current_tax_entity_id == "0")) // prevent non-blank entitites when we don't want them
   {
    if (IsAmount(editAmount->Text.c_str()))
    {
     if (current_data.ChangeAmountTo(about_account.id, current_tax_entity_id, StringToFinancial(editAmount->Text.c_str()), !checkDisableMagic->Checked))
     {
      checkShowClick(NULL);
      buttonTransactionsAddClick(NULL);
      CheckCurrentBalance();
     }
     else

      PSay("Your change has been refused.\nEither you attempted to edit an account type not supported,\nor you attempted to set anything but a cash account to a negative value.\nPlease only edit expense, payable, revenue, receivable, and cash accounts;\nand only attempt to set cash accounts to negative amounts.", m_invalid_data, MB_OK);
    } else
     PSay ("You must specify a valid dollar amount.", m_invalid_data, MB_OK);
   } else
    PSay ("You may not specify a tax entity: if you have this information, \nyou should create an actual bill to which transactions are attached.", m_invalid_data, MB_OK);
  } else
   PSay ("You must specify a tax entity, either to be paid or to pay.", m_missing_data, MB_OK);
 } else
  PSay ("The specified account could not be found.", m_invalid_data, MB_OK);
 return;
}
//---------------------------------------------------------------------------

void __fastcall TformSpecialTransactionEntry::buttonExpandAccountClick(
      TObject *Sender)
{
 // expand account for more info
 FORM_TRY
  string id;
  id = IdByMask (editAccount, false);
  if (!id.empty())
  {
   map<string, string> params;
   params["id"] = id;
   TformRecordAccount * expand = new TformRecordAccount(NULL, params);
   expand->Show();
  } else
   PSay("Unable to expand account information: account not found.", m_invalid_data, MB_OK);
 FORM_CATCH
 return;
}
//---------------------------------------------------------------------------
void __fastcall TformSpecialTransactionEntry::buttonSearchAccountClick(TObject *Sender)
{
 // search for an account
 FORM_TRY
  vector<string> results;
  TformSearchAccount * search = new TformSearchAccount(NULL, &results);
  // search->comboAccountType->Text = "Expense";
  search->checkIsBudget->Checked = false;
  search->checkIsClosed->Checked = false;
  search->ShowTaskModal(this);
  if (!results.empty())
   MaskByInfo (editAccount, current_data.GetCachedAccountInfo(results.front()));
 FORM_CATCH
 return;
}
//---------------------------------------------------------------------------
void __fastcall TformSpecialTransactionEntry::buttonSearchTaxEntityClick(TObject *Sender)
{
 // search for a tax entity
 FORM_TRY
  vector<string> results;
  TformSearchTaxEntity * search = new TformSearchTaxEntity(NULL, &results);
  search->ShowTaskModal(this);
  if (!results.empty())
  {
   current_tax_entity_id = results.front();
   CreateLink (staticTaxEntity, buttonSearchTaxEntity, this, current_tax_entity_id, "tax_entities", "table.tax_entities", use_entities);
  }
 FORM_CATCH
 return;
}
//---------------------------------------------------------------------------
void TformSpecialTransactionEntry::TransactionRowSelected()
{
 if (checkShowAccount->Checked && (checkShowName->Checked || !use_entities))
 {
  // a useful row's been selected, fill in the thingie!
  map<string, string> row_data;
  if (!edit_transactions.get_current_row(row_data))
  {
   // default values, for the columns being used
   row_data["account_id"]      = "0";
   row_data["tax_entity_id"]   = default_tax_entity_id;
   if (listTransactions->Items->Count)
    row_data["change_amount"]   = "0.00";
   else
    row_data["change_amount"]   = FinancialToString(default_amount);
  }

  if (row_data["tax_entity_id"].empty())
   row_data["tax_entity_id"] = "0";
  if (row_data["account_id"].empty())
   row_data["account_id"] = "0";

  string account_id = row_data["account_id"];
  current_tax_entity_id  = row_data["tax_entity_id"];
  acc_info about_account = current_data.GetCachedAccountInfo(account_id);
  MaskByInfo(editAccount, about_account);

  if (account_id == "0" && optionsPreferences["accounting"]["default_account"] != "")
   editAccount->Text = optionsPreferences["accounting"]["default_account"].c_str();

  CreateLink (staticTaxEntity, buttonSearchTaxEntity, this, current_tax_entity_id, "tax_entities", "table.tax_entities", use_entities);
  editAmount->Text = row_data["change_amount"].c_str(); // it should be okay, right? maybe not. we'll see.

  if (IsVisible(editAccount) && editAccount->Enabled)
   editAccount->SetFocus();
 } else
 {
  // they can keep whatever data they have there (might just be looking something up real quick) until they go back to the 'useful' mode.
  // no need to overwrite those fields at all. just let them select ... and ignore it.
 }
 return;
}
//---------------------------------------------------------------------------
void SetupTransactionSummaryView   (edit_listview & listview_handler, bool show_account_name, bool show_tax_entity_name)
{
 listview_handler.clear_columns();

 // jhaley 20100910: try to take advantage of all available space for tax entity name.
 int account_name_width  = (show_account_name ? 130 : 0);
 int tax_name_width      = (show_tax_entity_name ? (90 + (show_account_name ? 0 : 80)) : 0);
 int account_type_width  = 60  + (show_account_name ? 0 : 50) + (show_tax_entity_name ? 0 : 60) + (!show_account_name && !show_tax_entity_name ? 80 : 0);
 int charge_amount_width = 75;
 if (show_tax_entity_name)
 {
   int remaining_width = listview_handler.get_listview_width() - (account_name_width + tax_name_width + account_type_width + charge_amount_width + 24);
   if (remaining_width > 0)
     tax_name_width += remaining_width;
 }

 if (show_account_name)
 {
  listview_handler.add_or_modify_column("account_name",    "Account",    true,  1, account_name_width, as_text, rawString);
  listview_handler.add_or_modify_column("account_id",      "Account Id", false, 2, 60, as_int, rawString);
 }
 if (show_tax_entity_name)
 {
  listview_handler.add_or_modify_column("tax_entity_name", "Entity",     true,  3, tax_name_width, as_text, prettyString);
  listview_handler.add_or_modify_column("tax_entity_id",   "Entity Id",  false, 4, 60, as_int, rawString);
 }
 // always
 listview_handler.add_or_modify_column("account_type",    "Type",        true,  5, account_type_width,  as_text,   prettyString);
 listview_handler.add_or_modify_column("change_amount",   "Amount",      true,  6, charge_amount_width, as_double, amountString);
 return;
}
//---------------------------------------------------------------------------
void RefreshTransactionSummaryView (edit_listview & listview_handler, bool show_account_name, bool show_tax_entity_name, const vector<map<string, string> > & preview_data)
{
 listview_handler.clear_data();
 SetupTransactionSummaryView (listview_handler, show_account_name, show_tax_entity_name);

 vector<map<string, string> > local_data = preview_data;
 vector<map<string, string> > output_data;
 vector<pair<string, cb_sorting_object<string> *> > sorting_methods;
 map<string, cb_aggregate_object<string> *> summing_methods;

 if (show_account_name)
 {
  sorting_methods.push_back(pair<string, cb_sorting_object<string> *>("account_name", new sort_less_than<string>()));
  sorting_methods.push_back(pair<string, cb_sorting_object<string> *>("account_id", new sort_less_than<string>()));
 }
 sorting_methods.push_back(pair<string, cb_sorting_object<string> *>("account_type", new sort_less_than<string>()));
 if (show_tax_entity_name)
 {
  sorting_methods.push_back(pair<string, cb_sorting_object<string> *>("tax_entity_name", new sort_less_than<string>()));
  sorting_methods.push_back(pair<string, cb_sorting_object<string> *>("tax_entity_id", new sort_less_than<string>()));
 }

 summing_methods["change_amount"] = new aggregate_callback_wrapper<string, financial, aggregate_sum<financial> >(StringToFinancial, FinancialToStringEx);

 output_data.reserve(local_data.size());
 do_control_break(local_data, output_data, sorting_methods, summing_methods);

 listview_handler.import_data(output_data);

 // prevent memory leaks
 cb_deallocate_sorting_map(sorting_methods);
 cb_deallocate_aggregate_map(summing_methods);
 return;
}
//---------------------------------------------------------------------------
void __fastcall TformSpecialTransactionEntry::FormClose(TObject *Sender, TCloseAction &Action)
{
 // stuff when closing
 Action = caFree;
 TformSpecialGeneric::FormClose(Sender, Action);
 if (Action == caFree)
  edit_transactions.setup_listview(NULL, false);
 return;
}
//---------------------------------------------------------------------------
void __fastcall TformSpecialTransactionEntry::buttonSaveClick(TObject *Sender)
{
 *return_current_data = current_data;
 Close();
 return;
}
//---------------------------------------------------------------------------
void __fastcall TformSpecialTransactionEntry::buttonCancelClick(TObject *Sender)
{
 Close();
 return;
}
//---------------------------------------------------------------------------
void TformSpecialTransactionEntry::CheckCurrentBalance ()
{
 financial abs_balance_amount = current_data.AbsBalanceAmount();
 if (abs_balance_amount != 0.00)
 {
  labelBalance->Caption = ("$ " + FinancialToString(abs_balance_amount, true)).c_str();
  _labelBalance->Visible = true;
  labelBalance->Visible = true;

  // if (was_balanced)
  //  PSay("These transactions are now out of balance by a total of $" + FinancialToString(abs_balance_amount, true) + ".\nYou will not be allowed to save until they are.", m_warning, MB_OK);
  buttonSave->Enabled = false;
  was_balanced = false;
 } else
 {
  _labelBalance->Visible = false;
  labelBalance->Visible = false;
  buttonSave->Enabled = true;
  was_balanced = true;
 }
 return;
}
//---------------------------------------------------------------------------
void __fastcall TformSpecialTransactionEntry::buttonTransactionsClick(TObject *Sender)
{
 sheetTransactions->Show();
}
//---------------------------------------------------------------------------
void __fastcall TformSpecialTransactionEntry::buttonBudgetsClick(TObject *Sender)
{
 sheetBudgets->Show();
}
//---------------------------------------------------------------------------
void TformSpecialTransactionEntry::RefreshBudgets ()
{
 transaction_summary differences = (current_data - original_data);
 map<string, financial> orig_account_amounts = original_data.GetAccountOutput();
 map<string, financial> diff_account_amounts = differences.GetAccountOutput();
 map<string, financial> curr_account_amounts = current_data.GetAccountOutput();

 acc_info  this_account;
 string    budget_fk;
 acc_info  get_info;
 financial bud_a, exp_b, bud_c, exp_diff, exp_d, bud_e;

 vector<map<string, string> > all_rows;
 map<string, string> one_row;

 for (map<string, financial>::iterator i_acc = diff_account_amounts.begin(); i_acc != diff_account_amounts.end(); i_acc++)
 {
  this_account = differences.GetCachedAccountInfo(i_acc->first);
  budget_fk = this_account.EquivAccount(budget_ref);
  if (budget_fk != "" && budget_fk != "0")
  {
   if (budget_amounts.find(budget_fk) == budget_amounts.end())
   {
    get_info = InfoById(budget_fk);
    if (get_info.IsFound())
     budget_amounts.insert(pair<string, financial>(budget_fk, get_info.GetCurrentBalance() - this_account.GetCurrentBalance()));
    else
     break;
   }
   // assume we got it (it was purported to exist, and we didn't break; out of this loop when we went looking ...

   bud_c = budget_amounts[budget_fk];
   if (orig_account_amounts.find(i_acc->first) != orig_account_amounts.end())
    exp_b = orig_account_amounts[i_acc->first];
   else
    exp_b = 0.00;
   bud_a = bud_c + exp_b;
   if (curr_account_amounts.find(i_acc->first) != curr_account_amounts.end())
    exp_d = curr_account_amounts[i_acc->first];
   else
    exp_d = 0.00;
   exp_diff = exp_d - exp_b;
   bud_e = bud_a - exp_d;

   // don't bother with rows no longer showing up, but not actually "cleared" ... nothing ever happened, don't show
   if (exp_b != 0.00 || exp_d != 0.00)
   {
    one_row["account_name"] = this_account.GetTextName();
    one_row["bud_a"]        = FinancialToString(bud_a, true);
    one_row["exp_b"]        = FinancialToString(exp_b, true);
    one_row["bud_c"]        = FinancialToString(bud_c, true);
    one_row["exp_diff"]     = FinancialToString(exp_diff, true);
    one_row["exp_d"]        = FinancialToString(exp_d, true);
    one_row["bud_e"]        = FinancialToString(bud_e, true);
    all_rows.push_back(one_row);
   }
  }
 }
 edit_budgets.clear_data();
 edit_budgets.import_data(all_rows);
}
//---------------------------------------------------------------------------
void __fastcall TformSpecialTransactionEntry::buttonTransactionsBalanceClick(TObject *Sender)
{
 // re-balance
 // remove current row (and associated items)
 // ... we don't actually remove it, we just subtract it. yes, it'll show a zero value. darn.
 if (listTransactions->Selected)
 {
  map<string, string> current_row;
  edit_transactions.get_current_row(current_row);
  if (current_row["tax_entity_id"].empty())
   current_row["tax_entity_id"] = "0";
  if (current_row["account_id"].empty())
   current_row["account_id"] = "0";
  string account_to_clear = current_row["account_id"];
  string entity_to_clear = current_row["tax_entity_id"];
  financial abs_balance_amount = current_data.AbsBalanceAmount();
  if (abs_balance_amount != 0.00)
  {
   if (current_data.BalanceAccount(account_to_clear, entity_to_clear))
   {
    checkShowClick(NULL);
    CheckCurrentBalance();
   }
   else
    PSay("Unable to automatically balance this row, as it would most likely result in something rather invalid.", m_invalid_data, MB_OK);
  } else
   PSay("There is no off-balance amount to apply anywhere. These amounts balance just fine as they are.", m_invalid_data, MB_OK);
 } else
  PSay("You must first select a row to the left upon which to apply the current difference to bring things back into balance.", m_missing_data, MB_OK);
 return;
}
//---------------------------------------------------------------------------
void __fastcall TformSpecialTransactionEntry::sheetBudgetsShow(TObject *Sender)
{
 // refresh here only
 RefreshBudgets();
}
//---------------------------------------------------------------------------
void __fastcall TformSpecialTransactionEntry::menuConvertToAnyClick(TObject *Sender)
{
 // convert stuff, really.
}
//---------------------------------------------------------------------------

void __fastcall TformSpecialTransactionEntry::popupTransactionsConvertPopup(TObject *Sender)
{
 // prepare the popup by disabling what's not needed / creating what is (depending on how we feel)
}
//---------------------------------------------------------------------------
void __fastcall TformSpecialTransactionEntry::buttonTransactionsConvertClick(TObject *Sender)
{
 // try here?
 // convert stuff. yup. popup the menu.
 popupTransactionsConvert->PopupComponent = (TComponent*)Sender;
 popupTransactionsConvert->Popup(((TWinControl*)Sender)->ClientOrigin.x, ((TWinControl*)Sender)->ClientOrigin.y + ((TWinControl*)Sender)->Height);

 // so, at this point, am I willing to redo the way links between accounts are stored?
 // if so, what do I want?

 // (account_from, account_to, reason)
 // -> actual links

 // reason_types ...
 // -> used for some screens (see below) to figure out what they should be doing ... more analysis needed.

 // (account_from_type, account_to_type, reason, is_required, is_automatic)
 // -> for what could/should exist?
 // -> only one of each type from each type?
 // -> or should is_automatic be on actual links themselves? regardless, only one at a time (enforce as with person_names if need-be.)

 // places these fk's appear:
 // - components
 // -> see where these functions are used, and why/how? (transaction-entry for sure!)
 // - recordAccount
 // - specialCheckSentNotification (deprecated, remove)
 // - specialVoidCheck (is this still available? problems?)
 // - specialCancelPurchaseReq (as above)
 // - specialModifyCashAccount (trying to deprecate, see usage)
 // - specialGeneralJouranl (want to deprecate)
 // - specialModifyPayable (is this still accessible? should be deprecated.)
 // - specialReusableAccountingInsert (deprecated)
 // - wizardAccountCloseEquity
 // - wizardNewAcount
 // - wizardNewBudget
 // - launchPRReport
}
//---------------------------------------------------------------------------
void __fastcall TformSpecialTransactionEntry::buttonFrequentAccountsClick(TObject *Sender)
{
 popupFrequentAccounts->PopupComponent = (TComponent*)Sender;
 popupFrequentAccounts->Popup(((TWinControl*)Sender)->ClientOrigin.x, ((TWinControl*)Sender)->ClientOrigin.y + ((TWinControl*)Sender)->Height);
}
//---------------------------------------------------------------------------
void __fastcall TformSpecialTransactionEntry::menuAddFrequentAccountClick(TObject *Sender)
{
 // add current account to list
 string string_account = editAccount->Text.c_str();

 string string_accounts = optionsPreferences["accounting"]["frequent_accounts"];
 optionsPreferences["accounting"]["frequent_accounts"] = "";
 set<string> list_accounts;
 SplitStringToSet(string_accounts, "|", list_accounts);
 list_accounts.insert(string_account);
 list_accounts.erase("");

 for (set<string>::iterator i_a = list_accounts.begin(); i_a != list_accounts.end(); i_a++)
  optionsPreferences["accounting"]["frequent_accounts"] += ("|" + *i_a);

 RefreshFrequentAccounts();
}
//---------------------------------------------------------------------------
void __fastcall TformSpecialTransactionEntry::menuUseFrequentAccountClick(TObject *Sender)
{
 // generic, used by all added items
 editAccount->Text = ((TMenuItem*)Sender)->Caption;
}
//---------------------------------------------------------------------------
void TformSpecialTransactionEntry::RefreshFrequentAccounts()
{
 // remove all but the top two, then re-insert
 while (popupFrequentAccounts->Items->Count > 3)
  delete popupFrequentAccounts->Items->Items[3];

 string string_accounts = optionsPreferences["accounting"]["frequent_accounts"];
 set<string> list_accounts;
 SplitStringToSet(string_accounts, "|", list_accounts);
 for (set<string>::iterator i_a = list_accounts.begin(); i_a != list_accounts.end(); i_a++)
  if (*i_a != "")
  {
   TMenuItem * temp = new TMenuItem(popupFrequentAccounts);
   temp->Caption = i_a->c_str();
   temp->OnClick = menuUseFrequentAccountClick;
   temp->Enabled = true;
   temp->Visible = true;
   popupFrequentAccounts->Items->Add(temp);
  }
}
//---------------------------------------------------------------------------
void __fastcall TformSpecialTransactionEntry::menuRemoveFrequentAccountClick(TObject *Sender)
{
 // remove current account from list
 string string_account = editAccount->Text.c_str();

 string string_accounts = optionsPreferences["accounting"]["frequent_accounts"];
 optionsPreferences["accounting"]["frequent_accounts"] = "";
 set<string> list_accounts;
 SplitStringToSet(string_accounts, "|", list_accounts);
 list_accounts.erase(string_account);
 list_accounts.erase("");

 for (set<string>::iterator i_a = list_accounts.begin(); i_a != list_accounts.end(); i_a++)
  optionsPreferences["accounting"]["frequent_accounts"] += ("|" + *i_a);

 RefreshFrequentAccounts();
}
//---------------------------------------------------------------------------
void __fastcall TformSpecialTransactionEntry::editAmountKeyPress(TObject *Sender, char &Key)
{
 if (Key == VK_RETURN)
 {
  buttonTransactionsSave->Click();
  Key = 0;
 }
}
//---------------------------------------------------------------------------

