//---------------------------------------------------------------------------

#ifndef codeSpecialTransactionSummaryH
#define codeSpecialTransactionSummaryH
//---------------------------------------------------------------------------
#include <Classes.hpp>
#include <Controls.hpp>
#include <StdCtrls.hpp>
#include <Forms.hpp>
#include "codeSpecialGeneric.h"
#include <ComCtrls.hpp>
#include <ToolWin.hpp>
#include "codeSpecialTransactionEntry.h"
#include <ExtCtrls.hpp>
//---------------------------------------------------------------------------

// jhaley 20100908: transaction summarizer class, abstracted out of TformSpecialTransactionSummary so that this
// logic can be used in multiple places. Note TformSpecialTransactionSummary *is* still used in a few places that
// are not interested in showing either a summary -or- details of transactions directly. It has been reduced to a
// dedicated form which uses this class.
class classTransactionSummarizer
{
protected:
  string sql;            // SQL query
  bool   is_valid;       // has valid data?
  bool   show_account;   // show/group by account if true
  bool   show_name;      // show/group by name if true
  int    id_request_num; // Atlas request # for id field
  int    fk_request_num; // Atlas request # for foreign key field
  
  TformSpecialGeneric *parent_form; // form this belongs to
  TListView           *list_view;   // ListView control
  edit_listview        edit_before; // listview edit object
  transaction_summary  params;      // transaction summary object

public:
   classTransactionSummarizer(TformSpecialGeneric *p_parent_form, TListView *list_view);

   // methods
   void Close(TCloseAction Action); // call when the form closes
   void RefreshDataSet(); // call to refresh
   void AtlasRefresh(int request_num, string matched_value); // call from Atlas refresh handler
   void CheckBoxClicked(); // call when a checkbox representing show_account or show_name changes
   void Clear(); // call from IsNewAgain

   // setters
   void SetShowAccount(bool p_show_account);
   void SetShowName(bool p_show_name);
   void SetRecord(const string &p_item_table, const string &p_item_id, const string &p_tx_fk, RefreshHandler refresh_method);
   void SetSQLStatement(const string &p_sql);
};
//---------------------------------------------------------------------------
class TformSpecialTransactionSummary : public TformSpecialGeneric
{
__published:	// IDE-managed Components
        TToolButton *buttonClose;
        TToolButton *buttonUseless;
        TPanel *panelTopLink;
        TLabel *_labelItem;
        TStaticText *staticItem;
        TPanel *panelBottomList;
        TLabel *_labelSplitBefore;
        TCheckBox *checkBeforeShowAccount;
        TCheckBox *checkBeforeShowName;
        TListView *listBefore;
        void __fastcall checkBeforeClick(TObject *Sender);
        void __fastcall buttonCloseClick(TObject *Sender);
        void __fastcall FormClose(TObject *Sender, TCloseAction &Action);
private:	// User declarations
        classTransactionSummarizer summarizer; // jhaley: use the new utility class
        string item_id;
        string item_table;
        string tx_fk;

        void AtlasEvent(int request_num, string matched_value);
public:		// User declarations
        __fastcall TformSpecialTransactionSummary(TComponent* Owner, string p_item_id, string p_item_table);
};
//---------------------------------------------------------------------------
extern PACKAGE TformSpecialTransactionSummary *formSpecialTransactionSummary;
//---------------------------------------------------------------------------
#endif
