//---------------------------------------------------------------------------

#include <vcl.h>
#pragma hdrstop

#include "codeSpecialUpdatesNeeded.h"
#include "codeRecordPerson.h"
#include "reportUpdatesNeeded.h"
#include "reportMailingLabels.h"
//---------------------------------------------------------------------------
#pragma package(smart_init)
#pragma link "codeSpecialGeneric"
#pragma resource "*.dfm"
TformSpecialUpdatesNeeded *formSpecialUpdatesNeeded;
//---------------------------------------------------------------------------
__fastcall TformSpecialUpdatesNeeded::TformSpecialUpdatesNeeded(TComponent* Owner, bool show_consents)
        : TformSpecialGeneric(Owner)
{
  this->IconIndex = 17;
  children_date.InDate(Date());
  children_date.mod(-18, in_years);
  six_months.InDate(Date());
  six_months.mod(-6, in_months);
  six_months.mod(-1, in_weeks);
  five_months = six_months;
  five_months.mod(1, in_months); // plus one month (5-6 months back; ie., everyone that will become due this month)

  medicaid_weeks.InDate(Date());
  medicaid_weeks.mod(3, in_weeks);

  string sql = "select tax_entities.id from tax_entities "
               "where exists (select * from insurance_policies inner join insurance_types on insurance_policies.insurance_id = insurance_types.id "
                             "where insurance_policies.tax_entity_id = tax_entities.id and insurance_types.item like '%MEDICAID%')";
  SqlToVec(dbDatabase, sql, medicaid_companies_vec);

  consent_sql =
        "select person_names.last_name || ', ' || person_names.first_name || ' ' || person_names.suffix, "
        " affiliations.cdib, person_contact_info.phone_number, person_contact_info.city, "
        " state_types.abbreviation, max(consents.signature_date), people.current_id, people.id "
        "from people "
        "left join person_names on people.person_names_id = person_names.id "
        "left join affiliations on people.affiliations_id = affiliations.id "
        "left join person_contact_info on people.person_contact_info_id = person_contact_info.id "
        "left join state_types on person_contact_info.state_id = state_types.id "
        "left join consents on consents.person_id = people.id and consents.consent_id in (2, 9) "  // TODO -c MAGIC: bleh!
        //"and consents.group_id = '" + lookupTables->IdOf("groups", "Third Party Resources") + "' " -- jhaley 20111006: Mary does NOT want this, now.
        "where people.birth_date <= '" + children_date.OutDate(mdy | yyyy) + "' and "
        " people.death_date is null and "
        " affiliations.tribe_id = " + lookupTables->IdOf("tribe_types", "Absentee Shawnee Tribes of Oklahoma") + " "
        "group by person_names.last_name, person_names.first_name, person_names.suffix, "
        " person_contact_info.phone_number, affiliations.cdib, "
        " person_contact_info.city, state_types.abbreviation, people.current_id, people.id ";

  // jhaley 20110211: "having" clause for displaying all people, as usual
  consent_having_all_sql =
        "having max(consents.signature_date) <= '" + six_months.OutDate(mdy | yyyy) + "' "
        " or max(consents.signature_date) is null ";

  // jhaley 20110211: "having" clause for displaying only people who *had* a consent between six and five months ago and
  // will now need to sign a new one if they have additional services rendered.
  consent_having_sixmo_sql =
        "having max(consents.signature_date) between '" + six_months.OutDate(mdy | yyyy) + "' and '"
        + five_months.OutDate(mdy | yyyy) + "' ";

  // jhaley 20110211: Don't display policies that are explicitly marked as deleted, incorrect, or expired
  medicaid_sql =
        "select person_names.last_name || ', ' || person_names.first_name || ' ' || person_names.suffix, "
        "affiliations.cdib, person_contact_info.phone_number, tax_entities.name, insurance_policies.policy_number, "
        "insurance_policies.stop_date, people.current_id, people.id "
        "from people left join person_names on people.person_names_id = person_names.id "
        "left join affiliations on people.affiliations_id = affiliations.id "
        "left join person_contact_info on people.person_contact_info_id = person_contact_info.id "
        "inner join insurance_coverage on insurance_coverage.person_id = people.id and insurance_coverage.current_id > 2 "
        "inner join insurance_policies on insurance_coverage.insurance_policy_id = insurance_policies.id and insurance_policies.current_id > 2 "
        "left join tax_entities on insurance_policies.tax_entity_id = tax_entities.id "
        "where " + SafeInClause("insurance_policies.tax_entity_id", medicaid_companies_vec) + " and " // jhaley 20100818: use SafeInClause
        " (insurance_policies.stop_date <= '" + medicaid_weeks.OutDate(mdy | yyyy) + "' or "
        "  insurance_policies.stop_date is null) and "
        " people.death_date is null and "
        " affiliations.tribe_id = " + lookupTables->IdOf("tribe_types", "Absentee Shawnee Tribes of Oklahoma") +
        "order by person_names.last_name, person_names.first_name ";

  person_filter.SetListViewHandle(listPeople);
  person_filter.SetSecurity("people");
  person_filter.AddColumnSpec("Name", "people", as_text, true, 150);
  person_filter.AddColumnSpec("CDIB", "affiliations", as_text, true, 60);
  person_filter.AddColumnSpec("Phone Number", "person_contact_info", as_text, true, 90, prettyPhoneString);
  person_filter.AddColumnSpec("City", "person_contact_info", as_text, true, 90);
  person_filter.AddColumnSpec("State", "person_contact_info", as_text, true, 60, uppercaseString);
  person_filter.AddColumnSpec("Date of Last Consent", "consents", as_date, true, 80);
  person_filter.AddColumnSpec("Current Id", "current_id", as_int, false, 80);
  person_filter.AddColumnSpec("Internal Id", "internal_id", as_int, false, 80);
  person_filter.FetchPreferences("specialupdatesneeded.person_filter");
  person_filter.SetCheck(true);
  person_filter.SetCompress(true);
  person_filter.SetAddEdit(this, NULL, EditRecord(&(TformSpecialUpdatesNeeded::EditPerson)));
  person_filter.SetInform(this, InformForm(&(TformSpecialUpdatesNeeded::DoFilterStuff)));
  person_filter.SetHint("double click row to open person's record form");
  listPeople->MultiSelect = true;

  medicaid_filter.SetListViewHandle(listMedicaid);
  medicaid_filter.SetSecurity("insurance_policies");
  medicaid_filter.AddColumnSpec("Name", "people", as_text, true, 150);
  medicaid_filter.AddColumnSpec("CDIB", "affiliations", as_text, true, 60);
  medicaid_filter.AddColumnSpec("Phone Number", "person_contact_info", as_text, true, 90, prettyPhoneString);
  medicaid_filter.AddColumnSpec("Insurance Company", "insurance_policies", as_text, true, 120, rawString);
  medicaid_filter.AddColumnSpec("Policy Number", "insurance_policies", as_text, true, 100, rawString);
  medicaid_filter.AddColumnSpec("Date Expires", "consents", as_date, true, 90);
  medicaid_filter.AddColumnSpec("Current Id", "current_id", as_int, false, 80);
  medicaid_filter.AddColumnSpec("Internal Id", "internal_id", as_int, false, 80);
  medicaid_filter.FetchPreferences("specialupdatesneeded.medicaid_filter");
  medicaid_filter.SetCheck(true);
  medicaid_filter.SetCompress(true);
  medicaid_filter.SetAddEdit(this, NULL, EditRecord(&(TformSpecialUpdatesNeeded::EditPerson)));
  medicaid_filter.SetInform(this, InformForm(&(TformSpecialUpdatesNeeded::DoFilterStuff)));
  medicaid_filter.SetHint("double click row to open person's record form");
  listMedicaid->MultiSelect = true;

  if (show_consents)
    menuConsentsClick(NULL);
  else
    menuMedicaidClick(NULL);
}
//---------------------------------------------------------------------------
void TformSpecialUpdatesNeeded::DoFilterStuff()
{
  string caption;
  if(menuConsents->Checked)
    caption = "Consents to be signed - " + IntToString(person_filter.GetResultSize());
  else
    caption = "Medicaid coverage expiring - " + IntToString(medicaid_filter.GetResultSize());
  this->Caption = caption.c_str();
  SetUpAtlasRequests();
}
//---------------------------------------------------------------------------
void TformSpecialUpdatesNeeded::SetUpAtlasRequests()
{
  vector<string> selected_id_vec = person_filter.GetAllID();
  atlas_requests.clear();
//  if(selected_id_vec.empty())
//    PSay("the vector is empty", m_message, MB_OK);
  for(vector<string>::iterator itr = selected_id_vec.begin(); itr != selected_id_vec.end(); itr++)
  {
    atlas_requests.push_back(atlas->RequestNotificationWhen("person_contact_info.person_id", *itr, rel_e, as_int, this,
                             RefreshHandler(&(TformSpecialUpdatesNeeded::AtlasRefreshListView))));
    if(menuConsents->Checked)
      atlas_requests.push_back(atlas->RequestNotificationWhen("consents.person_id", *itr, rel_e, as_int, this,
                               RefreshHandler(&(TformSpecialUpdatesNeeded::AtlasRefreshListView))));
  }
}
//---------------------------------------------------------------------------
bool TformSpecialUpdatesNeeded::AtlasRefreshListView(int request_num, string matched_value)
{
  if(menuConsents->Checked)
    DisplayConsents();
  else
    DisplayMedicaid();
  return true;
}
//---------------------------------------------------------------------------
void __fastcall TformSpecialUpdatesNeeded::btnCloseClick(TObject *Sender)
{
  Close();
}
//---------------------------------------------------------------------------
void TformSpecialUpdatesNeeded::DisplayConsents()
{
  this->Caption = "Loading...";
  listMedicaid->Visible = false;
  listPeople->Visible = true;

  // jhaley 20110211: Build consents query based on indicated date option
  string sql;
  if (menuDatesLastSix->Checked)
    sql = consent_sql + consent_having_sixmo_sql;
  else
    sql = consent_sql + consent_having_all_sql;

  person_filter.SetSQLStatement(sql);
  person_filter.RefreshDataSet();
  person_filter.CheckAll();
}
//---------------------------------------------------------------------------
void TformSpecialUpdatesNeeded::DisplayMedicaid()
{
  this->Caption = "Loading...";
  listMedicaid->Visible = true;
  listPeople->Visible = false;
  medicaid_filter.SetSQLStatement(medicaid_sql);
  medicaid_filter.RefreshDataSet();
  medicaid_filter.CheckAll();
}
//---------------------------------------------------------------------------
void TformSpecialUpdatesNeeded::EditPerson()
{
  map<string, string> temp_map;
  if(menuConsents->Checked)
    temp_map["id"] = person_filter.GetSelectedRecordID();
  else
    temp_map["id"] = medicaid_filter.GetSelectedRecordID();
  FORM_TRY
    TformRecordPerson *view_person = new TformRecordPerson(NULL, temp_map);
    view_person->Show();
  FORM_CATCH
}
//---------------------------------------------------------------------------
void __fastcall TformSpecialUpdatesNeeded::btnSelectAllClick(TObject *Sender)
{
  if(menuConsents->Checked)
    person_filter.CheckAll();
  else
    medicaid_filter.CheckAll();
}
//---------------------------------------------------------------------------
void __fastcall TformSpecialUpdatesNeeded::btnUnselectAllClick(TObject *Sender)
{
  if(menuConsents->Checked)
    person_filter.UncheckAll();
  else
    medicaid_filter.UncheckAll();
}
//---------------------------------------------------------------------------
void __fastcall TformSpecialUpdatesNeeded::menuMedicaidClick(TObject *Sender)
{
  btnView->Down = false;
  menuConsents->Checked = false;
  menuMedicaid->Checked = true;
  DisplayMedicaid();
}
//---------------------------------------------------------------------------
void __fastcall TformSpecialUpdatesNeeded::menuConsentsClick(TObject *Sender)
{
  btnView->Down = false;
  menuConsents->Checked = true;
  menuMedicaid->Checked = false;
  DisplayConsents();
}
//---------------------------------------------------------------------------
void __fastcall TformSpecialUpdatesNeeded::menuPrintReportClick(TObject *Sender)
{
  vector<string> id_vec;
  if (menuConsents->Checked)
    id_vec = person_filter.GetMultiID();
  else if (menuMedicaid->Checked)
    id_vec = medicaid_filter.GetMultiID();
  if(! id_vec.empty())
  {
    Processing(true);
    TIBTransaction * dbTransaction = new TIBTransaction(NULL);
    TIBDataSet * dbDataSet = new TIBDataSet(NULL);
    string sql;
    string today = CurrentDate();

    if (menuConsents->Checked)
    {
      sql = "select person_names.last_name || ', ' || person_names.first_name || ' ' || person_names.suffix full_name, "
            "affiliations.cdib, person_contact_info.phone_number, person_contact_info.line1, person_contact_info.city, "
            "state_types.abbreviation, person_contact_info.zip, max(consents.signature_date) the_date "
            "from people left join person_names on people.person_names_id = person_names.id "
            "left join affiliations on people.affiliations_id = affiliations.id "
            "left join person_contact_info on people.person_contact_info_id = person_contact_info.id "
            "left join state_types on person_contact_info.state_id = state_types.id "
            "left join consents on consents.person_id = people.id and consents.consent_id in (2, 9) "
            " and consents.group_id = '" + lookupTables->IdOf("groups", "Third Party Resources") + "' "
            "where " + SafeInClause("people.id", id_vec) + " "                                    // jhaley 20100818: use SafeInClause
            "group by person_names.last_name, person_names.first_name, person_names.suffix, "
            "person_contact_info.phone_number, person_contact_info.line1, affiliations.cdib, "
            "person_contact_info.city, state_types.abbreviation, person_contact_info.zip ";

      // jhaley 20110211: use current date range setting
      if (menuDatesLastSix->Checked)
        sql += consent_having_sixmo_sql;
      else
        sql += consent_having_all_sql;
    }
    else
    {
      sql = "select person_names.last_name || ', ' || person_names.first_name || ' ' || person_names.suffix full_name, "
            "affiliations.cdib, person_contact_info.phone_number, person_contact_info.line1, person_contact_info.city, "
            "state_types.abbreviation, person_contact_info.zip, insurance_policies.stop_date the_date, "
            "tax_entities.name insurance_company, insurance_policies.policy_number "
            "from people left join person_names on people.person_names_id = person_names.id "
            "left join affiliations on people.affiliations_id = affiliations.id "
            "left join person_contact_info on people.person_contact_info_id = person_contact_info.id "
            "left join state_types on person_contact_info.state_id = state_types.id "
            "inner join insurance_coverage on insurance_coverage.person_id = people.id and insurance_coverage.current_id > 2 "
            "inner join insurance_policies on insurance_coverage.insurance_policy_id = insurance_policies.id and "
            + SafeInClause("insurance_policies.tax_entity_id", medicaid_companies_vec) + " and "  // jhaley 20100818: use SafeInClause
            "(insurance_policies.stop_date <= '" + medicaid_weeks.OutDate(mdy | yyyy) + "' or "
            "insurance_policies.stop_date is null) and insurance_policies.current_id > 2 "
            "left join tax_entities on insurance_policies.tax_entity_id = tax_entities.id "
            "where " + SafeInClause("people.id", id_vec) + " "                                    // jhaley 20100818: use SafeInClause
            "order by person_names.last_name, person_names.first_name";
    }

    try
    {
      dbTransaction->DefaultDatabase = dbDatabase;
      dbDataSet->Database = dbDatabase;
      dbDataSet->Transaction = dbTransaction;
      dbTransaction->StartTransaction();
      dbDataSet->SelectSQL->Add(sql.c_str());
      dbDataSet->Open();
      dbDataSet->First();

      TformReportUpdatesNeeded *report_form = NULL;
      FORM_TRY
         report_form = new TformReportUpdatesNeeded(NULL);
      FORM_CATCH
      
      report_form->report->DataSet = dbDataSet;
      if(menuConsents->Checked)
      {
        report_form->lblTitle->Caption = string("People needing to sign consents - " + today).c_str();
        report_form->lblDate->Caption = "Date of last consent";
        report_form->lblCompany->Caption = "";
        report_form->lblCompany->SendToBack();
        report_form->lblPolicyNumber->Caption = "";
        report_form->lblPolicyNumber->SendToBack();
        report_form->txtCompany->Width = 0;
        report_form->txtCompany->SendToBack();
        report_form->txtPolicyNumber->Width = 0;
        report_form->txtPolicyNumber->SendToBack();

        report_form->txtAddress->DataSet = dbDataSet;
        report_form->txtAddress->DataField = "line1";
        report_form->txtCity->DataSet = dbDataSet;
        report_form->txtCity->DataField = "city";
        report_form->txtState->DataSet = dbDataSet;
        report_form->txtState->DataField = "abbreviation";
        report_form->txtZip->DataSet = dbDataSet;
        report_form->txtZip->DataField = "zip";
      }
      else
      {
        report_form->lblTitle->Caption = string("People with Medicaid coverage expiring - " + today).c_str();
        report_form->lblDate->Caption = "Date coverage expires";
        report_form->lblAddress->Caption = "";
        report_form->lblAddress->SendToBack();
        report_form->lblCity->Caption = "";
        report_form->lblCity->SendToBack();
        report_form->lblState->Caption = "";
        report_form->lblState->SendToBack();
        report_form->lblZip->Caption = "";
        report_form->lblZip->SendToBack();

        report_form->txtCompany->DataSet = dbDataSet;
        report_form->txtCompany->DataField = "insurance_company";
        report_form->txtPolicyNumber->DataSet = dbDataSet;
        report_form->txtPolicyNumber->DataField = "policy_number";
      }
      report_form->lblUser->Caption = PrettyString(lookupTables->ValueOf("users", current_userid)).c_str();
      report_form->txtName->DataSet = dbDataSet;
      report_form->txtName->DataField = "full_name";
      report_form->txtCDIB->DataSet = dbDataSet;
      report_form->txtCDIB->DataField = "cdib";
      report_form->txtPhone->DataSet = dbDataSet;
      report_form->txtPhone->DataField = "phone_number";

      report_form->txtDate->DataSet = dbDataSet;
      report_form->txtDate->DataField = "the_date";
      Processing(false);
      report_form->report->PreviewModal(); // jhaley - need PreviewModal here

      dbDataSet->Close();
      dbTransaction->Commit();
    }
    catch (...)
    {
      Processing(false);
      PSay("Error running report", m_database_error, MB_OK);
      if (dbTransaction->Active)
       dbTransaction->Rollback();
    }
    delete dbDataSet;
    delete dbTransaction;
  }
  else
    PSay("No people are selected. Please check the people you want on the report", m_missing_data, MB_OK);
}
//---------------------------------------------------------------------------
void __fastcall TformSpecialUpdatesNeeded::btnViewClick(TObject *Sender)
{
  btnView->Down = false;        
}
//---------------------------------------------------------------------------

void __fastcall TformSpecialUpdatesNeeded::btnPrintClick(TObject *Sender)
{
  btnPrint->Down = false;        
}
//---------------------------------------------------------------------------
void __fastcall TformSpecialUpdatesNeeded::popupViewPopup(TObject *Sender)
{
  btnView->Down = false;        
}
//---------------------------------------------------------------------------
void __fastcall TformSpecialUpdatesNeeded::popupReportPopup(TObject *Sender)
{
  btnPrint->Down = false;
}
//---------------------------------------------------------------------------
void __fastcall TformSpecialUpdatesNeeded::popupSelectionPopup(TObject *Sender)
{
  btnSelection->Down = false;
}
//---------------------------------------------------------------------------
void __fastcall TformSpecialUpdatesNeeded::menuPrintLabelsClick(TObject *Sender)
{
  vector<string> id_vec;
  id_vec = person_filter.GetMultiID();
  if(! id_vec.empty())
  {
    Processing(true);
    TIBTransaction * dbTransaction = new TIBTransaction(NULL);
    TIBDataSet * dbDataSet = new TIBDataSet(NULL);
    string sql;
    string today = CurrentDate();

    sql = "select person_names.last_name || ', ' || person_names.first_name || ' ' || person_names.suffix full_name, "
          "person_contact_info.line1, person_contact_info.line2, person_contact_info.city || ', ' || "
          "state_types.abbreviation || '  ' || person_contact_info.zip city "
          "from people left join person_names on people.person_names_id = person_names.id "
          "left join person_contact_info on people.person_contact_info_id = person_contact_info.id "
          "left join state_types on person_contact_info.state_id = state_types.id "
          "where " + SafeInClause("people.id", id_vec) + " "                         // jhaley 20100818: use SafeInClause
          "order by person_names.last_name, person_names.first_name";
    try
    {
      dbTransaction->DefaultDatabase = dbDatabase;
      dbDataSet->Database = dbDatabase;
      dbDataSet->Transaction = dbTransaction;
      dbTransaction->StartTransaction();
      dbDataSet->SelectSQL->Add(sql.c_str());
      dbDataSet->Open();
      dbDataSet->First();

      TformReportMailingLabels * report_form;
      FORM_TRY
        report_form = new TformReportMailingLabels(NULL);
      FORM_CATCH
      report_form->labelReport->DataSet = dbDataSet;
      report_form->txtName->DataSet = dbDataSet;
      report_form->txtName->DataField = "full_name";
      report_form->txtAddress1->DataSet = dbDataSet;
      report_form->txtAddress1->DataField = "line1";
      Processing(false);
      report_form->labelReport->Preview();

      dbDataSet->Close();
      dbTransaction->Commit();

    }
    catch (...)
    {
      Processing(false); 
      PSay("Error running report", m_database_error, MB_OK);
      if (dbTransaction->Active)
       dbTransaction->Rollback();
    }
    delete dbDataSet;
    delete dbTransaction;    
  }
  else
    PSay("No people are selected. Please check the people you want on the report", m_missing_data, MB_OK);
}
//---------------------------------------------------------------------------
void __fastcall TformSpecialUpdatesNeeded::btnSelectionClick(TObject *Sender)
{
  TListView * listview = NULL;
  if (menuConsents->Checked)
    listview = listPeople;
  else if (menuMedicaid->Checked)
    listview = listMedicaid;
  if (listview->SelCount)
  {
   TItemStates selected = TItemStates() << isSelected;
   TListItem * Item = listview->Selected;
   while (Item)
   {
     if (menuCheck->Checked)
       Item->Checked = true;
     else if (menuUncheck->Checked)
       Item->Checked = false;
     else if (menuFlip->Checked)
       Item->Checked = !Item->Checked;
     Item = listview->GetNextItem(Item, sdAll, selected);
    }
  }
  btnSelection->Down = false;
}
//---------------------------------------------------------------------------
void __fastcall TformSpecialUpdatesNeeded::popupSelectionItemClick(TObject *Sender)
{
  ((TMenuItem *)Sender)->Checked = true;
}
//---------------------------------------------------------------------------
void __fastcall TformSpecialUpdatesNeeded::btnDatesClick(TObject *Sender)
{
  btnDates->Down = false;
}
//---------------------------------------------------------------------------
void __fastcall TformSpecialUpdatesNeeded::popupDatesPopup(TObject *Sender)
{
  btnDates->Down = false;
}
//---------------------------------------------------------------------------
void __fastcall TformSpecialUpdatesNeeded::menuDatesLastSixClick(TObject *Sender)
{
  if (Sender->InheritsFrom(__classid(TMenuItem)))
    ((TMenuItem *)Sender)->Checked = true;

  if (menuConsents->Checked)
    DisplayConsents();
  else
  {
    PSay("This option only applies to consents. Select \"Consents to be signed...\" in the View menu.",
         m_reminder, MB_OK);
  }
}
//---------------------------------------------------------------------------
void __fastcall TformSpecialUpdatesNeeded::menuDatesNoCutoffClick(TObject *Sender)
{
  if (Sender->InheritsFrom(__classid(TMenuItem)))
    ((TMenuItem *)Sender)->Checked = true;

  if (menuConsents->Checked)
    DisplayConsents();
  else
  {
    PSay("This option only applies to consents. Select \"Consents to be signed...\" in the View menu.",
         m_reminder, MB_OK);
  }
}
//---------------------------------------------------------------------------

