//---------------------------------------------------------------------------

#include <vcl.h>
#pragma hdrstop

#include "codeSpecialUploadFile.h"
#include "cached_files.h"
//---------------------------------------------------------------------------
#pragma package(smart_init)
#pragma link "codeSpecialGeneric"
#pragma resource "*.dfm"
TformSpecialUploadFile *formSpecialUploadFile;
//---------------------------------------------------------------------------
__fastcall TformSpecialUploadFile::TformSpecialUploadFile(TComponent* Owner, pvecstr &p_databases,
    string p_global_database, pmapstrs &init_file, bool *p_did_upload)
    : TformSpecialGeneric(Owner)
{
  databases       = p_databases;
  global_database = p_global_database;
  file_info       = init_file;
  did_upload      = p_did_upload;
}
//---------------------------------------------------------------------------
void __fastcall TformSpecialUploadFile::editFileNameDblClick(TObject *Sender)
{
  if(openDialog->Execute())
    editFileName->Text = openDialog->FileName;
}
//---------------------------------------------------------------------------
void __fastcall TformSpecialUploadFile::btnCloseClick(TObject *Sender)
{
  Close();
}
//---------------------------------------------------------------------------
void __fastcall TformSpecialUploadFile::FormShow(TObject *Sender)
{
  if(file_info["file_path"] != "" && file_info["file_name"] != "")
    editUploadTo->Text = FileCache::PathConcatenate(file_info["file_path"], file_info["file_name"]).c_str();

  if(file_info["version_number"] != "")
  {
    editVersion->Text = FileCache::IncrementVersionNum(file_info["version_number1"], file_info["version_number2"],
                           file_info["version_number3"], file_info["version_number4"], 0, 0, 0, 1).c_str();
  }
  else
    editVersion->Text = "1.1.1.1";

  progress_step = progressUpload->Max / databases.size();

  TformSpecialGeneric::FormShow(Sender);
}
//---------------------------------------------------------------------------
void __fastcall TformSpecialUploadFile::btnUploadClick(TObject *Sender)
{
  TIBDatabase *dbCurrentDatabase;
  string ib_error;
  FileCache::push_file_status result;

  // If necessary...
  //btnUpload->Enabled = false;
  progressUpload->Position = 0;

  // Validate input
  if(editFileName->Text == "")
  {
    PSay("Please select a file to upload", m_missing_data, MB_OK);
    editFileName->SetFocus();
    return;
  }
  else if(editUploadTo->Text == "")
  {
    PSay("Please enter a destination path and filename", m_missing_data, MB_OK);
    editUploadTo->SetFocus();
    return;
  }
  else if(editVersion->Text == "")
  {
    PSay("Please enter a version number", m_missing_data, MB_OK);
    editVersion->SetFocus();
    return;
  }

  pair<string, string> path_and_name = FileCache::GetFileSpec(editUploadTo->Text.c_str());
  if(path_and_name.first == "")
  {
    PSay("Need a valid destination path. Please add at least one folder component", m_missing_data, MB_OK);
    editUploadTo->SetFocus();
    return;
  }

  for(pvecstr::iterator i_db = databases.begin(); i_db != databases.end(); ++i_db)
  {
    try
    {
      if(*i_db == global_database) // is the global database?
        dbCurrentDatabase = dbDatabase; // use global Prometheus connection
      else
        dbCurrentDatabase = dbUploadDatabase; // use private database control on this form

      // Connect if needed
      if(!dbCurrentDatabase->TestConnected())
      {
        LogIt("upload_file", "Not connected; connecting...");
        ConnectToDatabase(dbCurrentDatabase, *i_db, "ZEUS", "argon");
      }

      // Upload the file
      result = FileCache::PushFileToDatabase(dbCurrentDatabase, NULL, current_userid, editFileName->Text.c_str(),
                                             editVersion->Text.c_str(), path_and_name.first, path_and_name.second,
                                             editComments->Text.c_str(), ib_error);

      // Error?
      if(result != FileCache::pf_ok)
      {
        string error_message = "Error: " + FileCache::PushFileError(result);
        if (ib_error != "")
          error_message += string(";\n\nIB Error: ") + ib_error;
        PSay(error_message, result == FileCache::pf_interbase_error ? m_database_error : m_program_error, MB_OK);
      }
      else if(did_upload)
        *did_upload = true; // Successfully uploaded at least one file, so let our parent form know to refresh the tree.
    }
    catch(EIBError &error)
    {
      PSay(error.Message.c_str(), m_database_error, MB_OK);
    }
    catch(...)
    {
      PSay("Unknown error during file upload", m_program_error, MB_OK);
    }
    if(dbUploadDatabase->TestConnected()) // If the database local to this form is still connected, disconnect now
      dbUploadDatabase->Close();

    int current_pos = progressUpload->Position;
    int desired_pos = progressUpload->Position + progress_step;
    int maximum_pos = progressUpload->Max;
    progressUpload->Position = desired_pos <= maximum_pos ? desired_pos : maximum_pos;
  }

  // If necessary...
  //btnUpload->Enabled = true;
}
//---------------------------------------------------------------------------

