//---------------------------------------------------------------------------
#include <vcl.h>
#pragma hdrstop

#include "codeSpecialUploadVersion.h"
#include "derivedheaders.h"
//---------------------------------------------------------------------------
#pragma package(smart_init)
#pragma link "codeSpecialGeneric"
#pragma resource "*.dfm"
TformSpecialUploadVersion *formSpecialUploadVersion;
//---------------------------------------------------------------------------
__fastcall TformSpecialUploadVersion::TformSpecialUploadVersion(TComponent* Owner) : TformSpecialGeneric(Owner)
{
  this->IconIndex = 47;
  // string sql;
  string errorMessage;
  file_name = "";

  editChanges->Text = "";

  TIBTransaction* dbTransaction = new TIBTransaction(NULL);
  StdTransaction(dbTransaction, dbDatabase);

  try
  {
    lblCurrent->Caption = GetOneField(dbTransaction, "select version_number from prometheus_update2 "
       "order by version_number1 descending, version_number2 descending, version_number3 descending").c_str();

    vector<map<string, string> > databases;
    string sql = "select distinct server_name, server_file from prometheus_update_sources where valid = 1 order by server_name, server_file";
    SqlToVecMap(dbTransaction, sql, databases);

    string primary_database_path = optionsIni["server"]["server_name"] + ":" + optionsIni["server"]["server_file"];
    upload_databases.push_back(pair<string, TCheckBox*>(primary_database_path, checkPrimary));
    checkPrimary->Caption = primary_database_path.c_str();
    checkPrimary->Hint = primary_database_path.c_str();
    checkPrimary->ShowHint = true;
    checkPrimary->Checked = true;
    unsigned int top = checkPrimary->Top + 24;
    for (unsigned int i=0; i<databases.size(); i++)
    {
      string database_path = databases[i]["server_name"] + ":" + databases[i]["server_file"];
      if (database_path != primary_database_path)
      {
        TCheckBox *checkItOut = new TCheckBox(scrollDatabases);
        checkItOut->Left = checkPrimary->Left;
        checkItOut->Top = top;
        top += 24;
        checkItOut->Width = checkPrimary->Width;
        checkItOut->Caption = database_path.c_str();
        checkItOut->Hint = database_path.c_str();
        checkItOut->ShowHint = true;
        checkItOut->Checked = true;
        scrollDatabases->InsertControl(checkItOut);
        checkItOut->Visible = true;
        upload_databases.push_back(pair<string, TCheckBox*>(database_path, checkItOut));
      }
    }
  }
  catch (EIBError &error)
  {
    errorMessage = "Error. Here is the message from InterBase:\n";
    errorMessage += error.Message.c_str();
    PSay(errorMessage, m_database_error, MB_OK);
  }
  if (dbTransaction->Active) dbTransaction->Commit();
  delete dbTransaction;
}
//---------------------------------------------------------------------------
void __fastcall TformSpecialUploadVersion::btnCloseClick(TObject *Sender)
{
  Close();
}
//---------------------------------------------------------------------------
void __fastcall TformSpecialUploadVersion::FormClose(TObject *Sender, TCloseAction &Action)
{
  TformSpecialGeneric::FormClose(Sender, Action);
}
//---------------------------------------------------------------------------
void __fastcall TformSpecialUploadVersion::editFileNameDblClick(TObject *Sender)
{
  if(openDialog->Execute())
  {
    file_name = openDialog->FileName.c_str();
    editFileName->Text = file_name.c_str();
    editVersion->SetFocus();
  }

// This block gets the version information from the specified file
  string version = "";
  VS_FIXEDFILEINFO *pFixedInfo; // pointer to fixed file info structure
  UINT  VersionLen;
  DWORD unused;
  try
  {
    DWORD VersionInfoSize = GetFileVersionInfoSize(openDialog->FileName.c_str(), &unused);
    if (VersionInfoSize != 0)
    {
      char *buffer = new char[VersionInfoSize];
      GetFileVersionInfo(openDialog->FileName.c_str(), unused, VersionInfoSize, (void *)buffer);
      VerQueryValue(buffer,"\\",(void**)&pFixedInfo,(UINT *)&VersionLen);
      char cver[30];
      sprintf(cver, "%u\.%u\.%u\.%u",
              HIWORD (pFixedInfo->dwProductVersionMS),
              LOWORD (pFixedInfo->dwProductVersionMS),
              HIWORD (pFixedInfo->dwProductVersionLS),
              LOWORD (pFixedInfo->dwProductVersionLS));
      version = cver;
      if (buffer)
        delete []buffer;
    }
  }
  catch (...) { }
// end get version information

  if (version == "")
    editVersion->Text = "error getting version info from exe";
  else if (version.substr(version.size() - 2, 2) == ".0")
    editVersion->Text = version.substr(0, version.size() - 2).c_str();
  else
    editVersion->Text = version.c_str();

  if (editVersion->Text != "")
    editChanges->SetFocus();
}
//---------------------------------------------------------------------------
void TformSpecialUploadVersion::UpdateDatabase(string database)
{
  LogIt("upload", "Starting up request for '" + AnsiString(database.c_str()) + "'");
  string errorMessage;
  string sql;
  fstream inputFile;
  string comments = editChanges->Text.c_str();
  string version = editVersion->Text.c_str();
  int first = version.find_first_of(".");
  int second = version.find_last_of(".");

  if (file_name == "")
    file_name = editFileName->Text.c_str();

  dbUploadTransaction    = new TIBTransaction(NULL);
  dbUploadSQL            = new TIBSQL(NULL);
  dbUploadDataSet        = new TIBDataSet(NULL);

  try
  {
    if (database == "")
      dbUploadDatabase = dbDatabase;
    else
      dbUploadDatabase = GdbUploadDatabase;

    // we really should already be connected, from codePrometheus, but if not...
    if (!dbUploadDatabase->TestConnected())
    {
      LogIt("upload", "Not connected; connecting...");
      ConnectToDatabase(dbUploadDatabase, database, "ZEUS", "argon");
    }
    LogIt("upload", "Setting up DB objects");
    dbUploadSQL->Database        = dbUploadDatabase;
    dbUploadDataSet->Database    = dbUploadDatabase;
    StdTransaction(dbUploadTransaction, dbUploadDatabase);
    dbUploadSQL->Transaction     = dbUploadTransaction;
    dbUploadDataSet->Transaction = dbUploadTransaction;

    /* TODO -c Add: */

    LogIt("upload", "Executing SQL to fetch version info");
    // get previous latest-version (note we haven't uploaded the newest-newest one yet)
    sql = "select first 1 version_number1, version_number2, version_number3 "
          "from prometheus_update2 "
          "order by version_number1 desc, version_number2 desc, version_number3 desc";
    DebugSql(this, &sql, other_forms, 9);

    pvecmap ver_map;
    if (SqlToVecMap(dbUploadTransaction, sql, ver_map))
    {
      // fill this in with -those- numbers to clear exe for everything else
      /* Note, this is probably overkill.  We could just set file_exe = null where
         version != version_number1 || '.' || version_number2 || '.' || version_number3
         but this seems safer if someone comes in and changes code on this screen ... I feel better about doing
         it explicitly.  Further note:  we're deleting every version less than the newest one.  This is before
         we upload the new version.  So we're actually keeping one reserve version "just in case" we need to take
         down the one we just uploaded and use the previous as the newest still (provided nobody's downloaded the
         newer version yet).  If the upload fails for some reason, this gets rolled back on the same transaction.
      */
      if (!ver_map.empty())
      {
        // Do we even need this anymore, since we do this farther down, after exe is successfully uploaded,
        // AND we do it more correctly (i.e. the last two versions, accounting for gaps in numbers and such...)
        LogIt("upload", "Executing SQL to drop old executables");
        sql = "update prometheus_update2 set file_exe = null "
              "where version_number1 < '" + ver_map[0]["version_number1"] + "' or "
              "(version_number1 = '" + ver_map[0]["version_number1"] + "' and version_number2 < '" + ver_map[0]["version_number2"] + "') or "
              "(version_number1 = '" + ver_map[0]["version_number1"] + "' and version_number2 = '" + ver_map[0]["version_number2"] +
              "' and version_number3 < '" + ver_map[0]["version_number3"] + "')";
        DebugSql(this, &sql, other_forms, 9);
        ExecuteStatement(dbUploadTransaction, sql);
      }
      else
        LogIt("upload", "SqlToVecMap returned empty set");
    }
    else
      LogIt("upload", "SqlToVecMap failed");

    LogIt("upload", "Checking to see if version already exists");
    sql = "select version_number from prometheus_update2 where version_number = '" + version + "'";
    dbUploadDataSet->SelectSQL->Add(sql.c_str());
    dbUploadDataSet->Open();

    if (dbUploadDataSet->Eof)
    {
      LogIt("upload", "Inserting version info");
      dbUploadDataSet->Close();
      // The hell?  No?  We can't roll back altogether if we do this.
      // dbUploadTransaction->Commit();
      // dbUploadTransaction->StartTransaction();

      sql = "insert into prometheus_update2 "
            "(version_number1, version_number2, version_number3, version_number, last_update_stamp, last_updated_by) values (" +
             version.substr(0, first) + ", " +
             version.substr(first + 1, (second - first) - 1) + ", " +
             version.substr(second + 1) + ", '" + version + "', '" + RightNow() + "', " + current_userid + ")";
      dbUploadSQL->SQL->Text = sql.c_str();
      dbUploadSQL->ExecQuery();

      // needed for the bit inside the if {} below
      LogIt("upload", "Fetching data we just inserted, to insert exe blob");
      sql = "select * from prometheus_update2 where version_number = '" + version + "'";
      dbUploadDataSet->SelectSQL->Clear();
      dbUploadDataSet->SelectSQL->Add(sql.c_str());

      LogIt("upload", "Inserting exe blob");
      sql = "update prometheus_update2 "
            " set file_exe = :file_exe, comments = :comments where version_number = '" + version + "'";
      dbUploadDataSet->ModifySQL->Add(sql.c_str());

      dbUploadDataSet->Open();
      dbUploadDataSet->Edit();
      inputFile.open(file_name.c_str(), ios::in || ios::nocreate);
      if(inputFile)
      {
        inputFile.close();
        dbUploadDataSet->FieldByName("file_exe")->SetFieldType(ftBlob);
        ((TBlobField *)dbUploadDataSet->FieldByName("file_exe"))->LoadFromFile(file_name.c_str());
        int size_to_write = ((TBlobField *)dbUploadDataSet->FieldByName("file_exe"))->BlobSize;
        lblByteExe->Caption = ("Writing " + IntToString(size_to_write) + " byte exe").c_str();
        Application->ProcessMessages();
        dbUploadDataSet->FieldByName("comments")->SetFieldType(ftBlob);
        ((TBlobField *)dbUploadDataSet->FieldByName("comments"))->AsString = comments.c_str();
        ((TBlobField *)dbUploadDataSet->FieldByName("comments"))->Modified = true;
        LogIt("upload", "Writing exe blob to database");
        dbUploadDataSet->Post();
        dbUploadDataSet->Close();

        LogIt("upload", "Fetching data we just wrote to verify it was successfully written");
        sql = "select * from prometheus_update2 where version_number = '" + version + "'";
        dbUploadDataSet->SelectSQL->Clear();
        dbUploadDataSet->SelectSQL->Add(sql.c_str());
        dbUploadDataSet->Open();
        dbUploadDataSet->First();
        int size_written = ((TBlobField *)dbUploadDataSet->FieldByName("file_exe"))->BlobSize;
        dbUploadDataSet->Close();
        if (size_written == size_to_write)
        {
           LogIt("upload", "Blob written successfully");
           // File uploaded successfully.  Rock!  NEW:  Let's remove the EXE's for old versions.
           // We'll keep the one we just uploaded (obviously) and one backup, and nullify the rest.
           // We keep forgetting manually, so we'll do it automatically.  This saves disk space on
           // the database server every time the database is rebuilt (backup/restore, currently
           // weekly).  If you're thinking "how much space could it possibly save?", remember that
           // these EXEs are currently 15 megs a pop, and we've been through hundreds of versions....
           // -- now 20+ megs each

           // We keep the latest two:  the one we just upped and one backup.
           LogIt("upload", "Nullifying exe blobs for all but the latest two versions");
           string remove_old_exes_sql =
             "update prometheus_update2 set file_exe = null "
             "where last_update_stamp not in "
             " (select first 2 last_update_stamp from prometheus_update2 order by last_update_stamp desc)";
           if (! ExecuteStatement(dbUploadTransaction, remove_old_exes_sql))
           {
             LogIt("upload", "Failed to remove old file_exe blobs.  You should do this manually.");
             PSay("Failed to remove old file_exe blobs.  You should do this manually.", m_database_error, MB_OK);
             // Not a critical error.  Just let them know and go ahead and commit.
           }
           dbUploadTransaction->Commit();
        }
        else
        {
           LogIt("upload", "Size of uploaded EXE differs from original exe (" +
                 AnsiString(IntToString(size_to_write).c_str()) + " vs. " +
                 AnsiString(IntToString(size_written).c_str()) + ")");
           PSay("Size of uploaded exe differs from original exe!\n\n"
                "original exe:  " + IntToString(size_to_write) + " bytes\n"
                "uploaded exe:  " + IntToString(size_written) + " bytes", m_program_error, MB_OK);
           dbUploadTransaction->Rollback();
        }
      }
      else
      {
       LogIt("upload", "Could not open exe file for upload; aborting.");
       PSay("Could not open exe file for upload", m_program_error, MB_OK);
       dbUploadDataSet->Close();
       dbUploadTransaction->Rollback();
      }
    }
    else
    {
      LogIt("upload", "Version already exists; aborting.");
      PSay("Version already exists. File not added", m_duplicate_data, MB_OK);
      dbUploadDataSet->Close();
      dbUploadTransaction->Rollback();
    }
  }
  catch (EIBError &error)
  {
    if (dbUploadTransaction->Active)
      dbUploadTransaction->Rollback();
    errorMessage = "Error. Here is the message from Interbase:\n";
    errorMessage += error.Message.c_str();
    PSay(errorMessage, m_database_error, MB_OK);
  }
  catch (...)
  {
   if (dbUploadTransaction->Active)
     dbUploadTransaction->Rollback();
   PSay("An unknown error has been thrown (not EIBError)", m_database_error, MB_OK);
  }

  delete dbUploadTransaction;
  delete dbUploadDataSet;
  delete dbUploadSQL;

  // only close DB connection if it's not the one we're connected to globally
  AV_TRY
  if(GdbUploadDatabase && GdbUploadDatabase->TestConnected())
    GdbUploadDatabase->Close();
  AV_CATCH
  dbUploadDatabase = NULL;
}
//---------------------------------------------------------------------------
void __fastcall TformSpecialUploadVersion::btnUploadClick(TObject *Sender)
{
  for (unsigned int i = 0; i < upload_databases.size(); i++)
  {
    if (upload_databases[i].second->Checked)
    {
      string db = upload_databases[i].first;
      LogIt("upload", "Upload loading to " + AnsiString(db.c_str()) + "...");
      UpdateDatabase(db);
    }
  }
  Close();
}
//---------------------------------------------------------------------------
// TODO -c MOVE: this should be in a utility library somewhere
bool TformSpecialUploadVersion::LogIt(AnsiString log_name, AnsiString message)
{
  bool all_is_well = true;
  try
  {
    AnsiString app_path  = ExtractFilePath(ParamStr(0));
    AnsiString run_stamp = RightNow().c_str();
    fstream log_file((app_path + "\\" + log_name + ".log").c_str(), ios::out | ios::app);
    log_file << "[" << run_stamp.c_str() << "] " << ReplaceStringWithString(message.c_str(), "\n\n", "\n", true) << '\n';
    log_file.close();
  }
  catch (...)
  {
    all_is_well = false;
  }
  return all_is_well;
}
//---------------------------------------------------------------------------

