//---------------------------------------------------------------------------

#include <vcl.h>
#pragma hdrstop

#include "codeSpecialUserColors.h"
//---------------------------------------------------------------------------
#pragma package(smart_init)
#pragma link "codeSpecialGeneric"
#pragma resource "*.dfm"
TformSpecialUserColors *formSpecialUserColors;
//---------------------------------------------------------------------------
__fastcall TformSpecialUserColors::TformSpecialUserColors(TComponent* Owner)
        : TformSpecialGeneric(Owner)
{
  IconIndex = 29;

  DisplayUsers();
}
//---------------------------------------------------------------------------
void TformSpecialUserColors::DisplayUsers()
{
  vector<map<string, string> > field_vec;
  shape_map.clear();
  edit_map.clear();
  string sql = "select users.item user_name, users.user_color, users.id, groups.item group_name, users.has_appointments "
               "from users "
               "inner join groups on users.group_id = groups.id "
               "where "
               "(users.valid = 1 or users.is_role = 1) and users.id > 0 "
               "order by groups.item, users.item ";

  SqlToVecMap(dbDatabase, sql, field_vec);
  if (! field_vec.empty())
  {
    string previous_group = "";
    int current_top = 8;
    int group_label_left = 8;
    int user_label_left = 24;
    int shape_left = 144;
    int edit_left = 192;

    for(vector<map<string, string> >::iterator itr = field_vec.begin(); itr != field_vec.end(); itr++)
    {
      if ((*itr)["group_name"] != previous_group)
      {
        previous_group = (*itr)["group_name"];
        TStaticText * group_label;
        group_label = new TStaticText(scrollStandard);
        group_label->AutoSize = false;
        group_label->Top = current_top;
        group_label->Left = group_label_left;
        group_label->Width = 105;
        group_label->Height = 17;
        group_label->Caption = PrettyString((*itr)["group_name"]).c_str();
        group_label->Parent = scrollStandard;
        current_top += 24;
      }

      TStaticText * user_label;
      user_label = new TStaticText(scrollStandard);
      user_label->AutoSize = false;
      user_label->Top = current_top;
      user_label->Left = user_label_left;
      user_label->Width = 105;
      user_label->Height = 17;
      user_label->Caption = PrettyString((*itr)["user_name"]).c_str();
      user_label->Parent = scrollStandard;

      TShape * user_color;
      user_color = new TShape(scrollStandard);
      user_color->Shape = stRectangle;
      user_color->Brush->Color = TColor(StringToInt((*itr)["user_color"]));
      user_color->Brush->Style = bsSolid;
      user_color->Cursor = crHandPoint;
      user_color->Pen->Style = psClear;
      user_color->Pen->Width = 0;
      user_color->Top = (current_top - 4);
      user_color->Left = shape_left;
      user_color->Width = 21;
      user_color->Height = 21;
      user_color->OnMouseDown = shapeClick;
      user_color->Parent = scrollStandard;

      TEdit * edit_int;
      edit_int = new TEdit(scrollStandard);
      edit_int->Text = IntToRGBString(StringToInt((*itr)["user_color"])).c_str();
      edit_int->Top = (current_top - 4);
      edit_int->Left = edit_left;
      edit_int->Width = 97;
      edit_int->Height = 21;
      edit_int->Tag = StringToInt((*itr)["id"]);
      edit_int->OnChange = editChange;
      edit_int->Parent = scrollStandard;
      edit_box_list.push_back(edit_int);

      shape_map[user_color] = edit_int;
      edit_map[edit_int] = user_color;

      current_top += 24;

      // jhaley 20111110: save data for later, with efficient mapping by user ID
      user_data_map[(*itr)["id"]] = *itr;
    } // end for
  } // end if vec not empty
}
//---------------------------------------------------------------------------
void __fastcall TformSpecialUserColors::btnCancelClick(TObject *Sender)
{
  Close();
}
//---------------------------------------------------------------------------
void __fastcall TformSpecialUserColors::btnSaveClick(TObject *Sender)
{
  TIBTransaction * dbTransaction = new TIBTransaction(NULL);
  map<string, string> field_map;
  try
  {
    dbTransaction->DefaultDatabase = dbDatabase;
    dbTransaction->StartTransaction();
    for(list<TEdit *>::iterator itr = edit_box_list.begin(); itr != edit_box_list.end(); itr++)
    {
      field_map.clear();
      field_map["id"] = IntToString((*itr)->Tag);
      field_map["user_color"] = IntToString(RGBStringToInt((*itr)->Text.c_str()));
      ExecuteUpdateStatement(dbTransaction, "users", field_map);
    }
    dbTransaction->Commit();
  }
  catch(...)
  {
    if(dbTransaction->Active)
      dbTransaction->Rollback();
    PSay("Could not update records", m_database_error, MB_OK);
  }
  delete dbTransaction;
  Close();
}
//---------------------------------------------------------------------------
void __fastcall TformSpecialUserColors::shapeClick(TObject *Sender, TMouseButton Button, TShiftState Shift, int X, int Y)
{
  TShape * my_shape = (TShape *) Sender;
  colorDialog->Color = my_shape->Brush->Color;
  colorDialog->CustomColors->Clear();
  colorDialog->CustomColors->Add(IntToRGBString((int)colorDialog->Color).c_str());
  if (colorDialog->Execute())
  {
    my_shape->Brush->Color = colorDialog->Color;
    shape_map[my_shape]->Text = IntToRGBString((int)colorDialog->Color).c_str();
  }
}
//---------------------------------------------------------------------------
void __fastcall TformSpecialUserColors::editChange(TObject *Sender)
{
  TEdit * my_box = (TEdit *) Sender;
  edit_map[my_box]->Brush->Color = TColor(RGBStringToInt(my_box->Text.c_str()));
}
//---------------------------------------------------------------------------
void __fastcall TformSpecialUserColors::menuShowHasAppsClick(TObject *Sender)
{
  // Record current state of check
  bool current_state = menuShowHasApps->Checked;

  // Invert state of check
  bool new_state = !current_state;

  if(current_state && !new_state) // Turning off? Show everybody again.
  {
    // Go through the checkboxes...
    for(list<TEdit *>::iterator itr = edit_box_list.begin(); itr != edit_box_list.end(); itr++)
    {
      (*itr)->Visible = true; // ... and make them all visible.
      edit_map[(*itr)]->Visible = true;
    }
  }
  else // Turning on? Show only users with appointments.
  {
    // Go through the checkboxes...
    for(list<TEdit *>::iterator itr = edit_box_list.begin(); itr != edit_box_list.end(); itr++)
    {
      string user_id = IntToString((*itr)->Tag); // get user ID

      if(user_data_map[user_id]["has_appointments"] == "0") // If not has_appointments, make invisible
      {
        (*itr)->Visible = false;
        edit_map[(*itr)]->Visible = false;
      }
    }
  }

  // Set new state to menu item
  menuShowHasApps->Checked = new_state;
}
//---------------------------------------------------------------------------

