//---------------------------------------------------------------------------

#include <vcl.h>
#pragma hdrstop

#include "codeWizardNewAccount.h"
//---------------------------------------------------------------------------
#pragma package(smart_init)
#pragma link "codeWizardGeneric"
#pragma resource "*.dfm"
TformWizardNewAccount *formWizardNewAccount;
//---------------------------------------------------------------------------
__fastcall TformWizardNewAccount::TformWizardNewAccount(TComponent* Owner)
        : TformWizardGeneric(Owner)
{
 pass = 0;
 start_page = 0;
 // SetExpectedPages(2);
 Initialize();

 lookupTables->RegisterComboBox("acc_level_one", comboLevelOne);
 lookupTables->RegisterComboBox("acc_level_two", comboLevelTwo);
 lookupTables->RegisterComboBox("acc_level_three", comboLevelThree);
 lookupTables->RegisterComboBox("acc_level_four", comboLevelFour);
 lookupTables->RegisterComboBox("account_types", comboAccountType);
 lookupTables->RegisterComboBox("month_types", comboBillCycle);
}
//---------------------------------------------------------------------------
bool TformWizardNewAccount::IsValidPage(int page_index)           // true/false: this page is without errors
{
 switch (page_index)
 {
  case 0:
  {
   // to do here:
   // [x] - make sure the account doesn't already exist
   // [x] - make sure they select everything

   map<string, string> new_account_parts;
   if (!GetNewAccountParts(new_account_parts))
    return false;

   // regardless, based on the type of account (second group box) let's see if it already exists. grrr if it does.
   // if that number + is_budget already exists, (even if this is, say, revenue and the other is expense) ... bad user.

   acc_info duplicate_account = InfoByParts (new_account_parts, checkIsBudget->Checked);
   if (duplicate_account.IsFound())
   {
    PSay("An account already exists that matches your request. You may not create any duplicates.", m_duplicate_data, MB_OK);
    return false;
   }

   if (!lookupTables->IsInLookUpTable("account_types", comboAccountType->Text.c_str()) || comboAccountType->Text == "")
   {
    PSay("You must select an account type. If you are creating a budget account,\nenter the type of the account this budget covers.", m_missing_data, MB_OK);
    comboAccountType->SetFocus();
    return false;
   } else
   {
    if (checkIsBudget->Checked)
    {
     account_types acc_type = StringToAccountType(comboAccountType->Text.c_str());
     if (acc_type != expense_acc && acc_type != revenue_acc && acc_type != equity_acc)
     {
      PSay("You are currently allowed to create budget accounts only for the following types of accounts:\nExpense, Revenue, and Equity.", m_invalid_data, MB_OK);
      checkIsBudget->SetFocus();
      return false;
     }
    }
   }

   if (!lookupTables->IsInLookUpTable("month_types", comboBillCycle->Text.c_str()))
   {
    PSay("You must select a month at the beginning of which actions may be taken on this account.\nIf this seems irrelevant, simply use the month during which related accounts cycle.\nThis setting is generally the same for all accounts within a fund.", m_invalid_data, MB_OK);
    comboBillCycle->SetFocus();
    return false;
   }

   if (editMinFiscalDate->Text != "" && !Pdate(editMinFiscalDate->Text.c_str(), global_mode).IsValidDate())
   {
    PSay("You must enter a valid minimum fiscal date, or none at all.\nIf none is specified, then the fiscal date will always be equal to the effective date.", m_invalid_data, MB_OK);
    editMinFiscalDate->SetFocus();
    return false;
   }

   // TODO: possibly warn them if the minimum fiscal date is in a month that is != the cycling month, if yearly?

   return true;
   // break;
  }
  case 1:
  {
   // to do here:
   // [x] - make sure that all the reference accounts exist and are the right types
   // [ ] - warn them if they're creating links outside a fund
   // [x] - don't let them auto-create budgets when
   //       a) they're creating a budget anyway
   //       b) budget already exists
   //       c) it's not something you can put a budget on
   // [x] - if auto-creating budgets, check that to-be-referenced accounts exist
   //       (if creating an expense account and its associated expense budget,
   //       it has to point to an equity account which itself has an equity budget account,
   //       which the expense budget account will point to. same with revenue.)
   map<account_links, pair<account_types, bool> > required_fields = GetRequiredAccountReferences(StringToAccountType(comboAccountType->Text.c_str()), checkIsBudget->Checked);
   // since we'll be modifying account_info (potentially) it's now stateful, to be remembered by ProcessPage() later
   account_info = GetRequiredAccountReferencesInfo(required_fields);

   if (checkIsBudget->Checked && checkAutoCreateBudget->Checked)
   {
    // shouldn't happen, but just in case!
    PSay("You may not auto-create budgets for budget accounts. You shouldn't even be able to ask for it.", m_invalid_data, MB_OK);
    return false;
   }

   // the following MODIFIES account_info in case the user declines to correct zero/unfound account references
   if (!ValidateAccountReferences(account_info, StringToAccountType(comboAccountType->Text.c_str()), checkAutoCreateBudget->Checked, checkIsClosed->Checked, checkIsBudget->Checked))
   {
    // they should already have been told why they're naughty in my sight
    return false;
   } else
   {
    if (checkAutoCreateBudget->Checked)
    {
     map<string, string> new_account_parts;
     if (!GetNewAccountParts(new_account_parts))
      return false;

     acc_info new_budget_account = InfoByParts (new_account_parts, true);
     if (new_budget_account.IsFound())
     {
      PSay("You may not auto-create a budget account, as such a budget account already exists.", m_duplicate_data, MB_OK);
      return false;
     }

     account_types acc_type = StringToAccountType(comboAccountType->Text.c_str());

     if (acc_type != expense_acc && acc_type != revenue_acc && acc_type != equity_acc)
     {
      PSay("You are currently allowed to (auto-)create budget accounts only for the following types of accounts:\nExpense, Revenue, and Equity.", m_invalid_data, MB_OK);
      return false;
     }

     if (acc_type == expense_acc || acc_type == revenue_acc)
     {
      map<account_links, acc_info>::iterator i_equity = account_info.find(balance_ref);
      if (i_equity != account_info.end())
      {
       if (i_equity->second.IsFound())
       {
        acc_info new_equity_budget_account = EquivAccountInfo (budget_ref, i_equity->second.id);
        if (new_equity_budget_account.IsFound())
        {
         if (new_equity_budget_account.GetType() == equity_acc && new_equity_budget_account.IsBudget())
         {
          if (new_equity_budget_account.IsClosed() && !checkIsClosed->Checked)
          {
           PSay("Your newly created account will use an equity account that has a disabled budget account.\nYou may therefore not auto-create a budget account.", m_warning, MB_OK);
           return false;
          }
         } else
         {
          PSay("Your newly created account will use an equity account that has a non-equity and/or non-budget budget account.\nYou may therefore not auto-create a budget account.", m_warning, MB_OK);
          return false;
         }
        } else
        {
         PSay("Your newly created account will use an equity account that does not have a budget account.\nYou may therefore not auto-create a budget account.", m_warning, MB_OK);
         return false;
        }
       } else
       {
        PSay("You have been erroneously allowed to select an equity account that does not exist.", m_program_error, MB_OK);
        return false;
       }
      } else
      {
       PSay("You have been erroneously allowed to not select an equity account at all, though one is needed.", m_program_error, MB_OK);
       return false;
      }
     }
    }
   }
   return true;
   // break;
  }
  default:
   return false;
   // break;
 }
}
//---------------------------------------------------------------------------
int TformWizardNewAccount::WhereNext(int page_index)           // page_index of the next page to visit, based on current page number
{
 if (page_index == 1)
  return TformWizardGeneric::last_wizard_page;
 else
  return (page_index + 1); // safe for this wizard, where pages are in order
}
//---------------------------------------------------------------------------
bool TformWizardNewAccount::ProcessPage(int page_index)           // actually go from A to B, ... no reset, no messing with other stuff?
{
 switch (page_index)
 {
  case 0:
  {
   // to do here:
   // - nothing
   return true;
  }
  case 1:
  {
   return true;
  }
  default:
   PSay("ProcessPage() is trying to access a page that does not exist.", m_program_error, MB_OK);
   return false;
 }
}
//---------------------------------------------------------------------------
void TformWizardNewAccount::SetupPage(int page_index)           // make this page ready to be shown based on other data
{
 switch (page_index)
 {
  case 0:
  {
   // to do here:
   // [x] - set combo boxes and masks back to nothing, default options
   if (pass == 0) // only do this stuff the first time
   {
     MaskSetup (maskNewAccount);
     comboLevelOne->Text    = "";
     comboLevelTwo->Text    = "";
     comboLevelThree->Text  = "";
     comboLevelFour->Text   = "";
     comboAccountType->Text = "Expense";
     comboBillCycle->Text   = "January";
     checkIsBudget->Checked = false;
     checkIsYearly->Checked = true;
   }
   break;
  }
  case 1:
  {
   // to do here:
   // [x] - disable controls on the second page based on selected account type
   // [x] - reset fields to defaults as necessary
   MaskSetup (maskBalanceAccount);
   MaskSetup (maskPendingAccount);
   MaskSetup (maskCashAccount);
   MaskSetup (maskBudgetAccount);
   MaskSetup (maskActualAccount);
   MaskSetup (maskRepayAccount);

   editDescription->Text  = "";
   checkIsClosed->Checked = false;
   checkAutoCreateBudget->Checked = true; // yes, true, fucktard

   // [x] - disable auto create if not allowed
   // [x] - set accounts to values that make sense if possible
   // [x] - disable masks that make no sense (EnableOnly...)

   checkAutoCreateBudget->Enabled = !checkIsBudget->Checked;

   string found_account_id;
   map<string, string> new_account_parts;
   if (!GetNewAccountParts(new_account_parts))
    return;

   // TODO: shouldn't be using the Like search -- should be looking at ... the individual lookup tables!

   account_types acc_type = StringToAccountType(comboAccountType->Text.c_str());
   map<account_links, pair<account_types, bool> > needed_refs = GetRequiredAccountReferences(StringToAccountType(comboAccountType->Text.c_str()), checkIsBudget->Checked);

   if (needed_refs.find(balance_ref) != needed_refs.end())
   {
    found_account_id = GetOneField(dbDatabase, "select first 1 accounts.id from accounts where accounts.account_type_id = " + lookupTables->IdOf("account_types", "Equity") + " and accounts.is_budget = " + (checkIsBudget->Checked ? string("1") : string("0")) + " and accounts.name like '" + new_account_parts["one"] + "%' order by accounts.is_closed");
    if (!found_account_id.empty())
     MaskById(maskBalanceAccount, found_account_id);
   }

   if (needed_refs.find(pending_ref) != needed_refs.end())
   {
    if (acc_type == expense_acc)
     found_account_id = GetOneField(dbDatabase, "select first 1 accounts.id from accounts where accounts.account_type_id = " + lookupTables->IdOf("account_types", "Payable") + " and accounts.name like '" + new_account_parts["one"] + "%' order by accounts.is_closed");
    else // "Revenue" or anything else ... at this point, might as well pick something.
     found_account_id = GetOneField(dbDatabase, "select first 1 accounts.id from accounts where accounts.account_type_id = " + lookupTables->IdOf("account_types", "Receivable") + " and accounts.name like '" + new_account_parts["one"] + "%' order by accounts.is_closed");
    if (!found_account_id.empty())
     MaskById(maskPendingAccount, found_account_id);
   }

   if (needed_refs.find(cash_ref) != needed_refs.end())
   {
    found_account_id = GetOneField(dbDatabase, "select first 1 accounts.id from accounts where accounts.account_type_id = " + lookupTables->IdOf("account_types", "Cash") + " and accounts.name like '" + new_account_parts["one"] + "%' order by accounts.is_closed");
    if (!found_account_id.empty())
     MaskById(maskCashAccount, found_account_id);
   }

   if (needed_refs.find(budget_ref) != needed_refs.end())
   {
    MaskByParts (maskBudgetAccount, new_account_parts);
   }

   if (needed_refs.find(actual_ref) != needed_refs.end())
   {
    MaskByParts (maskActualAccount, new_account_parts);
   }

   if (needed_refs.find(repay_ref) != needed_refs.end())
   {
    if (acc_type == revenue_acc)
     found_account_id = GetOneField(dbDatabase, "select first 1 accounts.id from accounts where accounts.account_type_id = " + lookupTables->IdOf("account_types", "Payable") + " and accounts.name like '" + new_account_parts["one"] + "%' order by accounts.is_closed");
    else // "Expense" or anything else ... at this point, might as well pick something.
     found_account_id = GetOneField(dbDatabase, "select first 1 accounts.id from accounts where accounts.account_type_id = " + lookupTables->IdOf("account_types", "Receivable") + " and accounts.name like '" + new_account_parts["one"] + "%' order by accounts.is_closed");
    if (!found_account_id.empty())
     MaskById(maskRepayAccount, found_account_id);
   }

   EnableOnlyRequiredAccountReferences (needed_refs);
   break;
  }
  default:
   break;
 }
 return;
}
//---------------------------------------------------------------------------
bool __fastcall TformWizardNewAccount::FormHelp(WORD Command, int Data, bool &CallHelp)
{
 // AnsiString x = "http://www.google.com/";
 // ShellExecute(0, "open", static_cast<const char*>(x.data()), NULL, NULL, SW_SHOW);
 CallHelp = false;
 return true;
}
//---------------------------------------------------------------------------
void __fastcall TformWizardNewAccount::FormClose(TObject *Sender, TCloseAction &Action)
{
 Action = caFree;
 TformWizardGeneric::FormClose(Sender, Action);
 if (Action == caFree)
 {
  // unregister
  lookupTables->UnregisterComboBox("acc_level_one", comboLevelOne);
  lookupTables->UnregisterComboBox("acc_level_two", comboLevelTwo);
  lookupTables->UnregisterComboBox("acc_level_three", comboLevelThree);
  lookupTables->UnregisterComboBox("acc_level_four", comboLevelFour);
  lookupTables->UnregisterComboBox("account_types", comboAccountType);
  lookupTables->UnregisterComboBox("month_types", comboBillCycle);
 }
}
//---------------------------------------------------------------------------
void TformWizardNewAccount::EnableOnlyRequiredAccountReferences (map<account_links, pair<account_types, bool> > fk_names)
{
 maskBalanceAccount->Enabled = (fk_names.find(balance_ref) != fk_names.end());
 maskPendingAccount->Enabled = (fk_names.find(pending_ref) != fk_names.end());
 maskCashAccount->Enabled    = (fk_names.find(cash_ref)    != fk_names.end());
 maskBudgetAccount->Enabled  = (fk_names.find(budget_ref)  != fk_names.end());
 maskActualAccount->Enabled  = (fk_names.find(actual_ref)  != fk_names.end());
 // maskRepayAccount->Enabled   = (fk_names.find(repay_ref)   != fk_names.end());
}
//---------------------------------------------------------------------------
map<account_links, acc_info> TformWizardNewAccount::GetRequiredAccountReferencesInfo (map<account_links, pair<account_types, bool> > fk_names)
{
 map<account_links, acc_info> answer;

 if (fk_names.find(balance_ref) != fk_names.end())
  answer.insert(pair<account_links, acc_info>(balance_ref, InfoByMask (maskBalanceAccount, checkIsBudget->Checked)));

 if (fk_names.find(pending_ref) != fk_names.end())
  answer.insert(pair<account_links, acc_info>(pending_ref, InfoByMask (maskPendingAccount, false)));

 if (fk_names.find(cash_ref) != fk_names.end())
  answer.insert(pair<account_links, acc_info>(cash_ref, InfoByMask (maskCashAccount, false)));

 if (fk_names.find(budget_ref) != fk_names.end())
  answer.insert(pair<account_links, acc_info>(budget_ref, InfoByMask (maskBudgetAccount, true)));

 if (fk_names.find(actual_ref) != fk_names.end())
  answer.insert(pair<account_links, acc_info>(actual_ref, InfoByMask (maskActualAccount, false)));

 //if (fk_names.find(repay_ref) != fk_names.end())
 // answer.insert(pair<account_links, acc_info>(repay_ref, InfoByMask (maskRepayAccount, false)));

 return answer;
}
//---------------------------------------------------------------------------
bool TformWizardNewAccount::GetNewAccountParts(map<string, string> & new_account_parts)
{
   new_account_parts.clear();

   if (radioJustName->Checked)
   {
    new_account_parts = PartsByMask (maskNewAccount);
   } else
   {
    if (!lookupTables->IsInLookUpTable("acc_level_one", comboLevelOne->Text.c_str()) || comboLevelOne->Text == "")
    {
     PSay("You must select a fund number.", m_missing_data, MB_OK);
     return false;
    }
    if (!lookupTables->IsInLookUpTable("acc_level_two", comboLevelTwo->Text.c_str()) || comboLevelTwo->Text == "")
    {
     PSay("You must select a general-ledger number.", m_missing_data, MB_OK);
     return false;
    }
    if (!lookupTables->IsInLookUpTable("acc_level_three", comboLevelThree->Text.c_str()) || comboLevelThree->Text == "")
    {
     PSay("You must select a department number.", m_missing_data, MB_OK);
     return false;
    }
    if (!lookupTables->IsInLookUpTable("acc_level_four", comboLevelFour->Text.c_str()) || comboLevelFour->Text == "")
    {
     PSay("You must select a sub-department number.", m_missing_data, MB_OK);
     return false;
    }

    new_account_parts["one"]    = GetOneField(dbDatabase, "select short_item from acc_level_one where id = "  + lookupTables->IdOf("acc_level_one",  comboLevelOne->Text.c_str()));
    new_account_parts["two"]    = GetOneField(dbDatabase, "select short_item from acc_level_two where id = "  + lookupTables->IdOf("acc_level_two",  comboLevelTwo->Text.c_str()));
    new_account_parts["three"]  = GetOneField(dbDatabase, "select short_item from acc_level_three where id = "  + lookupTables->IdOf("acc_level_three",  comboLevelThree->Text.c_str()));
    new_account_parts["four"]   = GetOneField(dbDatabase, "select short_item from acc_level_four where id = "  + lookupTables->IdOf("acc_level_four",  comboLevelFour->Text.c_str()));
   }
   return true;
}
//---------------------------------------------------------------------------
void TformWizardNewAccount::PageLockStatus (int page_index, bool locked)
{
 switch (page_index)
 {
  case 0:
   groupAccountName->Enabled = !locked;
   groupAccountType->Enabled = !locked;
   groupAccountCycle->Enabled = !locked;
   return;
  case 1:
   groupAccountReferences->Enabled = !locked;
   groupAdditionalOptions->Enabled = !locked;
   return;
  default:
   PSay("PageLockStatus() is trying to access a page that does not exist.", m_program_error, MB_OK);
   return;
 }
}
//---------------------------------------------------------------------------
bool TformWizardNewAccount::FinishWizard ()
{
 // to do here:
 // - create the account(s)
 TIBTransaction * dbTransaction = new TIBTransaction(NULL);

 if (dbTransaction)
 {
  StdTransaction(dbTransaction, dbDatabase);
  try
  {
   // information-gathering
   bool is_budget = checkIsBudget->Checked;
   bool is_yearly = checkIsYearly->Checked;
   bool is_closed = checkIsClosed->Checked;
   bool is_auto   = checkAutoCreateBudget->Checked;
   account_types account_type = StringToAccountType(comboAccountType->Text.c_str());
   string bill_cycle   = comboBillCycle->Text.c_str();
   string description  = editDescription->Text.c_str();

   map<string, string> account_parts;
   if (GetNewAccountParts(account_parts))
   {
    map<account_links, pair<account_types, bool> > needed_refs = GetRequiredAccountReferences(account_type, is_budget);
    map<account_links, acc_info> given_refs = account_info; // now stateful, using verified and corrected info from IsValidPage()! ... GetRequiredAccountReferencesInfo(needed_refs);

    map<string, string> base_record_account;
    map<string, string> expl_record_account;
    map<string, string> impl_record_account;

    // create accounts
    base_record_account["id"]                 = "0";
    base_record_account["current_id"]         = "4";
    base_record_account["last_update_stamp"]  = RightNow();
    base_record_account["last_updated_by"]    = current_userid;
    /* TODO -c error_checking: after each one, see if it's null.  if so, inform them that they need to create that account chunk, and launch the screen to let them add it.
       Note:  add-screen could be existing add-lookup-table-value screen, but needs to allow for arbitrary number of arbitrary type extra fields (i.e. be generic enough to
       handle any lookup table). */
    base_record_account["acc_level_one_id"]   = GetOneField(dbTransaction, "select id from acc_level_one where short_item = '"  + account_parts["one"] + "'");
    if (base_record_account["acc_level_one_id"] == "")
    {
      PSay("acc_level_one value not found.\n\n"
           "insert into acc_level_one (item, valid, short_item) values ('some name', 1, '" + account_parts["one"] + "');", m_message, MB_OK);
      throw(5);
    }
    base_record_account["acc_level_two_id"]   = GetOneField(dbTransaction, "select id from acc_level_two where short_item = '"  + account_parts["two"] + "'");
    if (base_record_account["acc_level_two_id"] == "")
    {
      PSay("acc_level_two value not found.\n\n"
           "insert into acc_level_two (item, valid, short_item) values ('some name', 1, '" + account_parts["two"] + "');", m_message, MB_OK);
      throw(6);
    }
    base_record_account["acc_level_three_id"] = GetOneField(dbTransaction, "select id from acc_level_three where short_item = '"  + account_parts["three"] + "'");
    if (base_record_account["acc_level_three_id"] == "")
    {
      PSay("acc_level_three value not found.\n\n"
           "insert into acc_level_three (item, valid, short_item) values ('some name', 1, '" + account_parts["three"] + "');", m_message, MB_OK);
      throw(7);
    }
    base_record_account["acc_level_four_id"]  = GetOneField(dbTransaction, "select id from acc_level_four where short_item = '"  + account_parts["four"] + "'");
    if (base_record_account["acc_level_four_id"] == "")
    {
      PSay("acc_level_four value not found.\n\n"
           "insert into acc_level_four (item, valid, short_item) values ('some name', 1, '" + account_parts["four"] + "');", m_message, MB_OK);
      throw(8);
    }

    base_record_account["name"]               = account_parts["one"] + "-" + account_parts["two"] + "-" + account_parts["three"] + "-" + account_parts["four"];
    base_record_account["bill_cycle_id"]      = lookupTables->IdOf("month_types", bill_cycle);
    base_record_account["is_closed"]          = (is_closed) ? "1" : "0";
    base_record_account["is_yearly"]          = (is_yearly) ? "1" : "0";
    base_record_account["account_type_id"]    = lookupTables->IdOf("account_types", AccountTypeToString(account_type));
    base_record_account["balance_account_id"] = "0";
    base_record_account["pending_account_id"] = "0";
    base_record_account["cash_account_id"]    = "0";
    base_record_account["budget_account_id"]  = "0";
    base_record_account["actual_account_id"]  = "0";
    base_record_account["repay_account_id"]   = "0";
    base_record_account["min_fiscal_date"]    = (editMinFiscalDate->Text == "") ? string("") : Pdate(editMinFiscalDate->Text.c_str(), global_mode).OutDate(global_mode);

    expl_record_account = base_record_account;
    impl_record_account = base_record_account;

    // normal (explicit)
    expl_record_account["id"]                 = GetNextId(dbTransaction, "accounts");
    expl_record_account["description"]        = description;
    expl_record_account["is_budget"]          = (is_budget) ? "1" : "0";
    for (map<account_links, acc_info>::iterator i_ref = given_refs.begin(); i_ref != given_refs.end(); i_ref++)
     expl_record_account[LinkTypeToString(i_ref->first)] = i_ref->second.id;

    // insert normal
    if (!ExecuteInsertStatement(dbTransaction, "accounts", expl_record_account))
     throw(2);

    if (is_auto)
    {
     // auto-budget (implicit)
     impl_record_account["id"]                 = GetNextId(dbTransaction, "accounts");
     impl_record_account["description"]        = string("Budget for " + description).substr(0, 250);
     impl_record_account["is_budget"]          = "1";
     if (account_type == expense_acc || account_type == revenue_acc)
     {
      map<account_links, acc_info>::iterator i_equity = given_refs.find(balance_ref);
      if (i_equity != given_refs.end() && i_equity->second.IsFound())
      {
       acc_info new_equity_budget_account = EquivAccountInfo (budget_ref, i_equity->second.id);
       if (new_equity_budget_account.IsFound())
        impl_record_account[LinkTypeToString(balance_ref)] = new_equity_budget_account.id;
      }
     }

     impl_record_account[LinkTypeToString(actual_ref)]  = expl_record_account["id"];

     // insert auto-budget
     if (!ExecuteInsertStatement(dbTransaction, "accounts", impl_record_account))
      throw(3);

     if (!ExecuteStatement(dbTransaction, "update accounts set budget_account_id = " + impl_record_account["id"] + " where id = " + expl_record_account["id"]))
      throw(4);
    }
    dbTransaction->Commit();
    delete dbTransaction;
    pass++;
    return true;
   } else
    throw(1);
  }
  catch (int i)
  {
   PSay("A program error was caught during account creation; no accounts were created.", m_program_error, MB_OK, IntToString(i));
   dbTransaction->Rollback();
   delete dbTransaction;
   return false;
  }
  catch (...)
  {
   PSay("A database error was caught during account creation; no accounts were created.", m_database_error, MB_OK);
   dbTransaction->Rollback();
   delete dbTransaction;
   return false;
  }
 } else
 {
  PSay("A transaction could not be created with the database; no accounts were created.", m_database_error, MB_OK);
  return false;
 }
}
//---------------------------------------------------------------------------
void TformWizardNewAccount::Do()
{
 FORM_TRY
  TformWizardNewAccount * z = new TformWizardNewAccount(NULL);
  z->Show();
 FORM_CATCH
}
//---------------------------------------------------------------------------

