//---------------------------------------------------------------------------

#include <vcl.h>
#pragma hdrstop

#include "codeWizardNewCheck.h"
#include "codeRecordTaxEntity.h"
#include "codeRecordPurchaseReq.h"
//---------------------------------------------------------------------------
#pragma package(smart_init)
#pragma link "codeWizardGeneric"
#pragma link "codeRubberContainer"
#pragma resource "*.dfm"
TformWizardNewCheck *formWizardNewCheck;
//---------------------------------------------------------------------------
__fastcall TformWizardNewCheck::TformWizardNewCheck(TComponent* Owner)
        : TformWizardGeneric(Owner)
{
 tax_request_any       = 0;
 tx_request_any        = 0;
 pr_request_any        = 0;
 tax_request_specific  = 0;
 tx_request_specific   = 0;
 pr_request_specific   = 0;

 tax_entities_filter.SetListViewHandle(listTaxEntities);
 tax_entities_filter.SetSecurity("tax_entities");
 tax_entities_filter.AddColumnSpec("Name" ,   "tax_entities", as_text, true,   185, rawString);
 tax_entities_filter.AddColumnSpec("Tax ID" , "tax_entities", as_text, true,   73,  prettyString);
 tax_entities_filter.AddColumnSpec("Payable", "transactions", as_double, true, 75,  nonzeroAmountString);
 tax_entities_filter.AddColumnSpec("Current Id",  "current_id",  as_int, false, 80);
 tax_entities_filter.AddColumnSpec("Internal Id", "internal_id", as_int, false, 80);
 tax_entities_filter.FetchPreferences("wizardnewcheck.tax_entities_filter");
 tax_entities_filter.SetSQLStatement("");
 tax_entities_filter.RefreshDataSet();
 tax_entities_filter.SetAddEdit(this, NULL, EditRecord(&(TformWizardNewCheck::EditTaxEntity)));

 pr_filter.SetListViewHandle(listPurchaseReqs);
 pr_filter.SetSecurity("purchase_reqs");
 pr_filter.AddColumnSpec("Tax Entity", "tax_entities", as_text, true, 120, rawString);
 pr_filter.AddColumnSpec("Number", "purchase_reqs", as_int, true, 75);
 pr_filter.AddColumnSpec("Date", "purchase_reqs", as_date, true, 75);
 pr_filter.AddColumnSpec("Expenses", "purchase_reqs", as_double, true, 75, nonzeroAmountString);
 pr_filter.AddColumnSpec("Payable", "purchase_reqs", as_double, true, 75, nonzeroAmountString);
 pr_filter.AddColumnSpec("Paid", "purchase_reqs", as_double, true, 75, nonzeroAmountString);
 pr_filter.AddColumnSpec("Revenue", "purchase_reqs", as_double, true, 75, nonzeroAmountString);
 pr_filter.AddColumnSpec("Receivable", "purchase_reqs", as_double, true, 75, nonzeroAmountString);
 pr_filter.AddColumnSpec("Received", "purchase_reqs", as_double, true, 75, nonzeroAmountString);
 pr_filter.AddColumnSpec("Current Id", "current_id", as_int, false, 80);
 pr_filter.AddColumnSpec("Internal Id", "internal_id", as_int, false, 80);
 pr_filter.FetchPreferences("wizardnewcheck.pr_filter");
 pr_filter.SetCheck(true);
 pr_filter.SetSQLStatement("");
 pr_filter.RefreshDataSet();
 pr_filter.SetAddEdit(this, NULL, EditRecord(&(TformWizardNewCheck::EditPurchaseReq)));

 // SetExpectedPages(2);
 Initialize();
}
//---------------------------------------------------------------------------
bool TformWizardNewCheck::IsValidPage (int page_index)
{
 switch (page_index)
 {
  case 0:
  {
   chosen_tax_entity_id = tax_entities_filter.GetSelectedRecordID();
   if (chosen_tax_entity_id.empty())
   {
    PSay ("You must select a tax entity for which to record the creation of a check.", m_missing_data, MB_OK);
    return false;
   } else
   {
    string known_federal_id = GetOneField(dbDatabase, "select tin.identifier "
                                                      "from tax_entities "
                                                      "left join entity_ids tin on tin.tax_entity_id = tax_entities.id and tin.current_id >= 3 and tin.entity_id_id = '" + lookupTables->IdOf("entity_id_types", "TIN/EIN") + "' "
                                                      "where tax_entities.id = " + chosen_tax_entity_id);
    string numeric_federal_id = StripAllExcept(known_federal_id, uints);

    // digits is a #define'd quoted string ... concatenation via operator+ would result in pointer-addition, which is bad.
    if (known_federal_id.find_first_not_of(uints "-") != string::npos)
    {
     PSay ("The federal tax id number found contains non-digit, non-dash characters. It must be properly formatted before being used.", m_invalid_data, MB_OK);
     return false;
    } else
    if (numeric_federal_id.empty())
    {
     PSay ("There is no federal tax id number for this tax entity.\nThis must be corrected before any checks may be recorded.", m_missing_data, MB_OK);
     return false;
    } else
    if (numeric_federal_id.length() != 9)
    {
     if (IDNO == PSay ("The federal tax id number found is not 9 digits long.\nAre you sure you want to use it anyway?", m_invalid_data, MB_YESNO))
      return false;
    } else
    {
     // yes, we could watch for even more cases... but nah. not worth it at the moment, until they get clever. and deceitful. and then it's a different problem.
     if (numeric_federal_id == "999999999" || numeric_federal_id == "000000000" || numeric_federal_id == "123456789")
      if (IDNO == PSay ("It seems you're using a bogus federal tax id number.\nIf you can fix this before continuing, please do.\nAre you sure you want to continue anyway?", m_invalid_data, MB_YESNO, numeric_federal_id))
       return false;
    }
   }
   return true;
  }
  case 1:
  {
   chosen_pr_id = pr_filter.GetMultiID();
   if (chosen_pr_id.empty())
   {
    PSay ("You must select at least one purchase request to include before continuing.\nIf no purchase requests are listed, please make sure your filter preferences are set appropriately.", m_missing_data, MB_OK);
    return false;
   }
   chosen_number = editCheckNumber->Text.c_str();
   chosen_date = Pdate(dateCheckDate->Date);

   if (!IsInt(chosen_number) || chosen_number.empty())
   {
    PSay ("You must enter a valid number to use as a check number.", m_invalid_data, MB_OK);
    return false;
   } else
   {
    string count_of_duplicates = GetOneField(dbDatabase, "select count(*) from checks where check_number = " + chosen_number);
    if (StringToInt(count_of_duplicates) > 0)
    {
     if (IDNO == PSay("There already exist " + count_of_duplicates + " checks with this same number.\nAre you sure you want to use it anyway?", m_duplicate_data, MB_YESNO, "check number = " + chosen_number))
      return false;
    }
   }

   //chosen_finance_id = editNavisionID->Text.c_str();
   chosen_tax_id = editTaxEntityID->Text.c_str();

   return true;
  }
  default:
   PSay("IsValidPage() is trying to access a page that does not exist.", m_program_error, MB_OK);
   return false;
 }
}
//---------------------------------------------------------------------------
int TformWizardNewCheck::WhereNext (int page_index)
{
 if (page_index == 1)
  return TformWizardGeneric::last_wizard_page;
 else
  return (page_index + 1);
}
//---------------------------------------------------------------------------
bool TformWizardNewCheck::ProcessPage (int page_index)
{
 switch (page_index)
 {
  case 0:
  {
   // chosen_tax_entity_id = tax_entities_filter.GetSelectedRecordID(); -- saved earlier for use here!
   return true;
  }
  case 1:
  {
   // chosen_pr_id  = pr_filter.GetMultiID(); -- saved earlier for use here!
   chosen_number = editCheckNumber->Text.c_str();
   chosen_date   = Pdate(dateCheckDate->Date);
   //chosen_finance_id = editNavisionID->Text.c_str();
   chosen_tax_id = editTaxEntityID->Text.c_str();
   return true;
  }
  default:
   PSay("ProcessPage() is trying to access a page that does not exist.", m_program_error, MB_OK);
   return false;
 }
}
//---------------------------------------------------------------------------
void TformWizardNewCheck::SetupPage (int page_index)
{
 switch (page_index)
 {
  case 0:
  {
   if (tax_request_any != 0) atlas->CancelRequest(tax_request_any);
   if (tx_request_any  != 0) atlas->CancelRequest(tx_request_any);
   if (pr_request_any  != 0) atlas->CancelRequest(pr_request_any);

   tax_request_any = atlas->RequestNotificationWhen ("tax_entities.id", "0", rel_any, as_int, this, RefreshHandler(&(TformWizardNewCheck::AtlasScreen)));
   tx_request_any  = atlas->RequestNotificationWhen ("transactions.id", "0", rel_any, as_int, this, RefreshHandler(&(TformWizardNewCheck::AtlasScreen)));
   pr_request_any  = atlas->RequestNotificationWhen ("purchase_reqs.id", "0", rel_any, as_int, this, RefreshHandler(&(TformWizardNewCheck::AtlasScreen)));

   string tax_sql = "select tax_entities.name, tin.identifier, "
                    "sum(transactions.credit_amount - transactions.debit_amount) ob, "
                    "tax_entities.current_id, tax_entities.id "
                    "from tax_entities "
                    "inner join purchase_reqs on purchase_reqs.tax_entity_id = tax_entities.id and purchase_reqs.check_id = 0 "
                    "inner join bills on bills.purchase_req_id = purchase_reqs.id "
                    "inner join transactions on transactions.bill_id = bills.id and transactions.tax_entity_id = bills.actual_tax_id and transactions.account_id in ("
                    "select accounts.id from accounts where account_type_id = " + lookupTables->IdOf("account_types", "Payable") + ") "
                    "left join entity_ids tin on tin.tax_entity_id = tax_entities.id and tin.current_id >= 3 and tin.entity_id_id = '" + lookupTables->IdOf("entity_id_types", "TIN/EIN") + "' "
                    "where tax_entities.id > 0 or 0 = 1 "
                    "group by tax_entities.name, tin.identifier, tax_entities.current_id, tax_entities.id "
                    "having sum(transactions.credit_amount - transactions.debit_amount) > 0.00";
   tax_entities_filter.SetSQLStatement(tax_sql);
   tax_entities_filter.RefreshDataSet();
   break;
  }
  case 1:
  {
   if (tax_request_specific != 0) atlas->CancelRequest(tax_request_specific);
   if (tx_request_specific  != 0) atlas->CancelRequest(tx_request_specific);
   if (pr_request_specific  != 0) atlas->CancelRequest(pr_request_specific);

   tax_request_specific = atlas->RequestNotificationWhen ("tax_entities.id", chosen_tax_entity_id, rel_e, as_int, this, RefreshHandler(&(TformWizardNewCheck::AtlasScreen)));
   tx_request_specific  = atlas->RequestNotificationWhen ("transactions.tax_entity_id", chosen_tax_entity_id, rel_e, as_int, this, RefreshHandler(&(TformWizardNewCheck::AtlasScreen)));
   pr_request_specific  = atlas->RequestNotificationWhen ("purchase_reqs.tax_entity_id", chosen_tax_entity_id, rel_e, as_int, this, RefreshHandler(&(TformWizardNewCheck::AtlasScreen)));

   string pr_sql = "select tax_entities.name, purchase_reqs.purchase_req_number, purchase_reqs.date_sent, "
                   "(select sum(debit_amount-credit_amount) from transactions inner join bills on transactions.bill_id = bills.id where bills.purchase_req_id = purchase_reqs.id and account_id in (select accounts.id from accounts where account_type_id = " + lookupTables->IdOf("account_types", "Expense") + ")) expense, "
                   "(select sum(credit_amount-debit_amount) from transactions inner join bills on transactions.bill_id = bills.id where bills.purchase_req_id = purchase_reqs.id and account_id in (select accounts.id from accounts where account_type_id = " + lookupTables->IdOf("account_types", "Payable") + ")) payable, "
                   "(select sum(credit_amount-debit_amount) from transactions inner join bills on transactions.bill_id = bills.id where bills.purchase_req_id = purchase_reqs.id and transactions.tax_entity_id = purchase_reqs.tax_entity_id and account_id in (select accounts.id from accounts where account_type_id = " + lookupTables->IdOf("account_types", "Cash") + ")) paid, "
                   "(select sum(credit_amount-debit_amount) from transactions inner join bills on transactions.bill_id = bills.id where bills.purchase_req_id = purchase_reqs.id and account_id in (select accounts.id from accounts where account_type_id = " + lookupTables->IdOf("account_types", "Revenue") + ")) revenue, "
                   "(select sum(debit_amount-credit_amount) from transactions inner join bills on transactions.bill_id = bills.id where bills.purchase_req_id = purchase_reqs.id and account_id in (select accounts.id from accounts where account_type_id = " + lookupTables->IdOf("account_types", "Receivable") + ")) receivable, "
                   "(select sum(debit_amount-credit_amount) from transactions inner join bills on transactions.bill_id = bills.id where bills.purchase_req_id = purchase_reqs.id and transactions.tax_entity_id != purchase_reqs.tax_entity_id and account_id in (select accounts.id from accounts where account_type_id = " + lookupTables->IdOf("account_types", "Cash") + ")) received, "
                   "purchase_reqs.current_id, purchase_reqs.id "
                   "from purchase_reqs "
                   "inner join tax_entities on purchase_reqs.tax_entity_id = tax_entities.id "
                   "where purchase_reqs.tax_entity_id = " + chosen_tax_entity_id +
                   " and purchase_reqs.check_id = 0 and "
                   "(select sum(credit_amount-debit_amount) from transactions inner join bills on transactions.bill_id = bills.id where bills.purchase_req_id = purchase_reqs.id and account_id in (select accounts.id from accounts where account_type_id = " + lookupTables->IdOf("account_types", "Payable") + ")) > 0.00 ";
                  // if aliases worked correctly, the above where clause could run a tad faster ... damn aliases not working! (assuming the 'having' clause doesn't know how to re-use expressions, that is.)
   pr_filter.SetSQLStatement(pr_sql);
   pr_filter.RefreshDataSet();
   pr_filter.CheckAll();

   dateCheckDate->DateTime = TDateTime::CurrentDate();
   editCheckNumber->Text   = "";

   string tax_entity_sql = "select identifier " /*, id_number "*/
                           "from entity_ids tin "
                           "where tin.current_id >= 3 and tin.entity_id_id = '" + lookupTables->IdOf("entity_id_types", "TIN/EIN") + "' "
                           "and tin.tax_entity_id = '" + chosen_tax_entity_id + "'";
   map<string, string> id_map;
   SqlToMap(dbDatabase, tax_entity_sql, id_map);

   /*
   existing_finance_id = id_map["id_number"];
   editNavisionID->Text = existing_finance_id.c_str();
   lblNavisionID->Caption = existing_finance_id.c_str();
   if (!IsBlankOrZero(existing_finance_id))
   {
     editNavisionID->Visible = false;
     lblNavisionID->Visible = true;
   }
   else
   {
     editNavisionID->Visible = true;
     lblNavisionID->Visible = false;
   }
   */
   existing_tax_id = id_map["taxid"];
   editTaxEntityID->Text = existing_tax_id.c_str();
   lblTaxEntityID->Caption = existing_tax_id.c_str();
   if (!IsBlankOrZero(existing_tax_id) &&
       !is_in(existing_tax_id, "999999999", "000000000", "123456789", NULL))
   {
     editTaxEntityID->Visible = false;
     lblTaxEntityID->Visible = true;
   }
   else
   {
     editTaxEntityID->Visible = true;
     lblTaxEntityID->Visible = false;
   }

   break;
  }
  default:
   PSay("SetupPage() is trying to access a page that does not exist.", m_program_error, MB_OK);
   break;
 }
}
//---------------------------------------------------------------------------
void TformWizardNewCheck::EditTaxEntity()
{
 map<string,string> temp_map;
 temp_map["id"] = tax_entities_filter.GetSelectedRecordID();
 FORM_TRY
  TformRecordTaxEntity * tax_entity = new TformRecordTaxEntity(NULL, temp_map);
  tax_entity->Show();
 FORM_CATCH
 return;
}
//---------------------------------------------------------------------------
void TformWizardNewCheck::EditPurchaseReq()
{
 map<string, string> params;
 params["id"] = pr_filter.GetSelectedRecordID();
 FORM_TRY
  TformRecordPurchaseReq * edit_pr = new TformRecordPurchaseReq(NULL, params);
  edit_pr->Show();
 FORM_CATCH
 return;
}
//---------------------------------------------------------------------------
void TformWizardNewCheck::AtlasScreen (int request_num, string matched_value)
{
 if (request_num == tax_request_any ||
     request_num == tx_request_any ||
     request_num == pr_request_any)
  tax_entities_filter.RefreshDataSet();

 if (request_num == tx_request_specific || request_num == pr_request_specific || request_num == tax_request_specific)
  pr_filter.RefreshDataSet();
}
//---------------------------------------------------------------------------
void TformWizardNewCheck::PageLockStatus (int page_index, bool locked)
{
 switch (page_index)
 {
  case 0:
   return;
  case 1:
   groupPurchaseReq->Enabled = !locked;
   return;
  default:
   PSay("PageLockStatus() is trying to access a page that does not exist.", m_program_error, MB_OK);
   return;
 }
}
//---------------------------------------------------------------------------
void __fastcall TformWizardNewCheck::listTaxEntitiesChanging(TObject *Sender,
   TListItem *Item, TItemChange Change, bool &AllowChange)
{
 AllowChange = !WizardLocked();
}
//---------------------------------------------------------------------------

void __fastcall TformWizardNewCheck::listPurchaseReqsChanging(TObject *Sender,
   TListItem *Item, TItemChange Change, bool &AllowChange)
{
 AllowChange = !WizardLocked();
}
//---------------------------------------------------------------------------
bool TformWizardNewCheck::FinishWizard ()
{
 // create check, attach prs to check, and don't launch anything
 TIBTransaction * dbTransaction = new TIBTransaction(NULL);
 StdTransaction(dbTransaction, dbDatabase);
 try
 {
  AtlasClear();
  map<string, string> new_check;

  new_check["id"]                = GetNextId(dbTransaction, "checks");
  new_check["current_id"]        = "4";
  new_check["last_update_stamp"] = RightNow();
  new_check["last_updated_by"]   = current_userid;
  new_check["tax_entity_id"]     = chosen_tax_entity_id;
  new_check["date_sent"]         = chosen_date.OutDate(global_mode);
  new_check["check_number"]      = chosen_number;

  AtlasAdd("checks", new_check);
  if (!ExecuteInsertStatement(dbTransaction, "checks", new_check))
  {
   PSay ("Failure creating check record.", m_database_error, MB_OK);
   throw (1);
  }

  vector<map<string, string> > about_updated_purchase_reqs;
  if (!SqlToVecMap(dbTransaction, "select * from purchase_reqs where id in (" + VecToCommaString(chosen_pr_id) + ") ", about_updated_purchase_reqs))
  {
   PSay ("Failure retrieving information about purchase requests in order to send network update.", m_database_error, MB_OK);
   throw(2);
  }
  AtlasAdd("purchase_reqs", about_updated_purchase_reqs);

  string update_sql = "update purchase_reqs set check_id = '" + new_check["id"] + "' where id in (" + VecToCommaString(chosen_pr_id) + ") ";
  if (!ExecuteStatement(dbTransaction, update_sql))
  {
   PSay ("Failure attaching purchase requests to newly created check.", m_database_error, MB_OK);
   throw(3);
  }

  string sql_get_payable_amounts = "select transactions.account_id, transactions.tax_entity_id, transactions.task_id, transactions.bill_id, "
                                   "min(transactions.effective_date) eff_date, sum(credit_amount - debit_amount) tot_amount "
                                   "from transactions inner join bills on transactions.bill_id = bills.id inner join purchase_reqs on bills.purchase_req_id = purchase_reqs.id "
                                   "where purchase_reqs.check_id = " + new_check["id"] +
                                   " group by transactions.task_id, transactions.bill_id, transactions.tax_entity_id, transactions.account_id "
                                   "having sum(debit_amount - credit_amount) != 0";
  vector<map<string, string> > grouped_payable_amounts;
  if (!SqlToVecMap(dbTransaction, sql_get_payable_amounts, grouped_payable_amounts))
  {
   PSay ("Failure retrieving total of current transactions.", m_database_error, MB_OK);
   throw (4);
  }

  acc_info about_payable_account;
  acc_info about_cash_account;
  financial tot_amount;
  map<string, string> transaction_core;
  map<string, string> do_payable;
  map<string, string> do_cash;
  transaction_core["current_id"]        = "4";
  transaction_core["last_update_stamp"] = RightNow(); // TODO: this function should try to be accurate to the database's current_timestamp() function
  transaction_core["last_updated_by"]   = current_userid;
  transaction_core["note"]              = "Payment of check #" + chosen_number;

  for (vector<map<string, string> >::iterator i_row = grouped_payable_amounts.begin(); i_row != grouped_payable_amounts.end(); i_row++)
  {
   // cache
   if (about_payable_account.id != (*i_row)["account_id"])
    about_payable_account = InfoById((*i_row)["account_id"]);

   if (!about_payable_account.IsFound() || about_payable_account.IsClosed())
   {
    PSay ("Payable account required is either not found, or closed and inaccessible at this time.", m_database_error, MB_OK, about_payable_account.id);
    throw(5);
   } else
   {
    if (!about_payable_account.IsType(payable_acc))
    {
     // TODO: maybe tell them about positive amounts on, say, receivables, and offer to open the record(s)?
    } else
    {
     tot_amount = StringToFinancial((*i_row)["tot_amount"]);
     if (tot_amount < 0)
     {                                                                                                                                                                 // vv-- tax_entity_id at least?
      PSay ("A negative payable amount has been found where it should not be.", m_database_error, MB_OK, "account: " + about_payable_account.id + ", entity: " + (*i_row)[""] + ", amount: " + FinancialToString(tot_amount, true));
      throw(6);
     } else
     {
      // cache
      if (about_cash_account.id != about_payable_account.EquivAccount(cash_ref))
       about_cash_account = about_payable_account.EquivAccountInfo(cash_ref);

      if (!about_cash_account.IsType(cash_acc))
      {
       PSay ("A payable account's equivalent cash account was not a cash account!", m_database_error, MB_OK, about_payable_account.id + " => " + about_cash_account.id);
       throw(7);
      } else
      if (!about_cash_account.IsFound() || about_cash_account.IsClosed())
      {
       PSay ("Cash account required is either not found, or closed and inaccessible at this time.", m_database_error, MB_OK, about_cash_account.id);
       throw(8);
      } else
      {
       // proceed with insertion of stuff
       transaction_core["transaction_number"] = GetNextId(dbTransaction, "transact_number");
       transaction_core["effective_date"]     = (*i_row)["eff_date"];
       transaction_core["tax_entity_id"]      = (*i_row)["tax_entity_id"];
       transaction_core["task_id"]            = (*i_row)["task_id"];
       transaction_core["bill_id"]            = (*i_row)["bill_id"];

       do_payable = transaction_core;
       do_payable["id"]            = GetNextId(dbTransaction, "transactions");
       do_payable["account_id"]    = about_payable_account.id;
       do_payable["credit_amount"] = "0.00";
       do_payable["debit_amount"]  = FinancialToString(tot_amount);

       do_cash = transaction_core;
       do_cash["id"]            = GetNextId(dbTransaction, "transactions");
       do_cash["account_id"]    = about_cash_account.id;
       do_cash["credit_amount"] = FinancialToString(tot_amount);
       do_cash["debit_amount"]  = "0.00";

       if (!ExecuteInsertStatement(dbTransaction, "transactions", do_payable))
       {
        PSay("Failed to insert a transaction to cancel payable amounts.", m_database_error, MB_OK);
        throw(9);
       }

       if (!ExecuteInsertStatement(dbTransaction, "transactions", do_cash))
       {
        PSay("Failed to insert a transaction to modify cash amounts.", m_database_error, MB_OK);
        throw(10);
       }
      }
     }
    }
   }
  }

  map<string, string> to_update;
  //if (existing_finance_id != chosen_finance_id)
  //  to_update["id_number"] = chosen_finance_id;
  if (existing_tax_id != chosen_tax_id)
    to_update["taxid"] = chosen_tax_id;
  if (!to_update.empty())
  {
    to_update["id"] = chosen_tax_entity_id;
    ExecuteUpdateStatement(dbTransaction, "tax_entities", to_update);
  }
  
  dbTransaction->Commit();
  delete dbTransaction;
  AtlasSend();
  return true;
 }
 catch (...)
 {
  PSay("Couldn't save a new check; perhaps some of the selected purchase requests were already locked by another user.", m_database_error, MB_OK);
  dbTransaction->Rollback();
  delete dbTransaction;
  AtlasClear();
  return false;
 }
}
//---------------------------------------------------------------------------
void TformWizardNewCheck::Do()
{
 FORM_TRY
  TformWizardNewCheck * z = new TformWizardNewCheck(NULL);
  z->Show();
 FORM_CATCH
}
//---------------------------------------------------------------------------

