//---------------------------------------------------------------------------

#include <vcl.h>
#pragma hdrstop

#include "codeWizardNewPerson.h"
#include "codeFrameTinyRecord_Relationship.h"
#include "codeFrameTinyRecord_InsuranceCoverage.h"
#include "codeRecordConsent.h"
#include "tinyrecord_concentrator.h"
#include "codeSpecialHouseholdMod.h"
//---------------------------------------------------------------------------
#pragma package(smart_init)
#pragma link "codeHalfButton"
#pragma link "codeWizardGeneric"
#pragma link "codeFrameTinyRecordContainer"
#pragma link "codePaddedBox"
#pragma link "codeInlineContainer"
#pragma link "codeFrameTinyRecord_Person"
#pragma link "codeFrameTinyRecordGeneric"
#pragma link "codeFrameTinyRecordMagic"
#pragma resource "*.dfm"
TformWizardNewPerson *formWizardNewPerson;
//---------------------------------------------------------------------------
__fastcall TformWizardNewPerson::TformWizardNewPerson(TComponent* Owner, string* p_return_person_id)
        : TformWizardGeneric(Owner)
{
 last_necessary_page = 6;
 return_person_id = p_return_person_id;

 is_handling_shortcut = false;
 is_loading_from_db = false;

 framePerson->OnSetId = framePersonSetId;
 framePerson->AllowSame = false;
 framePerson->AllowNone = false;
 framePerson->Label = "Person (main)";
 framePerson->AutoExpandOnSetId = true;

 done_onshow_person = false;
 done_onshow_relationships = false;
 done_onshow_insurance = false;

 frameRelationships->OnAdd = frameRelationshipsAdd;
 frameRelationships->OnChange = frameRelationshipsChange;
 frameRelationships->CreateMethod = TframeTinyRecord_Relationship::Create;

 frameInsurance->OnAdd = frameInsuranceAdd;
 frameInsurance->OnChange = frameInsuranceChange;
 frameInsurance->CreateMethod = TframeTinyRecord_InsuranceCoverage::Create;

 magic_concentrator = NULL;
 FORM_TRY
  magic_concentrator = new tr_magic_concentrator();
 FORM_CATCH
 framePerson->Concentrator = magic_concentrator;
 frameRelationships->Concentrator = magic_concentrator;
 frameInsurance->Concentrator = magic_concentrator;

 consents_filter.SetListViewHandle (listConsents);
 consents_filter.SetSecurity("consents");
 consents_filter.AddColumnSpec("Type of Consent", "consents", as_text, true, 250);
 consents_filter.AddColumnSpec("Consent Id", "internal_id", as_int, false, 80);
 consents_filter.AddColumnSpec("Latest Date", "consents", as_date, true, 80);
 consents_filter.AddColumnSpec("Time Elapsed", "consents", as_date, true, 100, rawString);
 consents_filter.AddColumnSpec("Latest Note", "consents", as_text, false, 100, sentenceString);
 consents_filter.AddColumnSpec("Latest Group", "consents", as_text, true, 100);
 consents_filter.AddColumnSpec("Current Id", "current_id", as_int, false, 80);
 consents_filter.AddColumnSpec("Internal Id", "hidden", as_int, false, 80);
 consents_filter.FetchPreferences("wizardnewperson.consents_filter");
 consents_filter.SetIconStatus(true);
 consents_filter.SetCheck(true);
 consents_filter.SetSQLStatement ("");
 consents_filter.RefreshDataSet();
 consents_filter.SetAddEdit(this, NULL, EditRecord(&(TformWizardNewPerson::EditConsent)));
 consents_data = NULL;
 FORM_TRY
  consents_data = new classFilterSimpleSource();
 FORM_CATCH
 consents_filter.SetDataSource(consents_data); // do it this way because these things get deleted when the filter gets deleted

 buttonRelationships->Enabled = (optionsPermissions["table.relationships"]["insert"] == "true");
 buttonInsurance->Enabled = (optionsPermissions["table.insurance_coverage"]["insert"] == "true");

 current_action = is_intake;  // default is to start a new person unless otherwise specified

 // Initialize(); -- this is done in formshow, because ... lemme think ... something about resizing? visible? something. change at your own risk.
}
//---------------------------------------------------------------------------
__fastcall TformWizardNewPerson::~TformWizardNewPerson()
{
 delete magic_concentrator;
}
//---------------------------------------------------------------------------
string UnblankValue(TEdit * e)
{
 return StripLeading(StripTrailing(e->Text.c_str(), " "), " ");
}
//---------------------------------------------------------------------------
bool TformWizardNewPerson::IsValidPage (int page_index)
{
 switch(page_index)
 {
  case 0:
   if (framePerson->Magic == trm_none)
   {
    PSay("Please either edit an existing person or enter a new one. 'None' isn't an option here.", m_missing_data, MB_OK);
    return false;
   }
   return framePerson->Valid();
  case 1:
  {
   bool success = true;
   for(int i = 0; i < frameRelationships->RecordCount && success; i++)
   {
    TframeTinyRecord_Relationship * z = dynamic_cast<TframeTinyRecord_Relationship *>(frameRelationships->Record[i]);
    success = (success && z->Valid());
   }
   return success;
  }
  case 2:
  {
   bool success = true;
   for(int i = 0; (i < frameInsurance->RecordCount) && success; i++)
   {
    TframeTinyRecord_InsuranceCoverage * z = dynamic_cast<TframeTinyRecord_InsuranceCoverage *>(frameInsurance->Record[i]);
    success = (success && z->Valid());
   }
   return success;
  }
  case 3: // med-history 1
   if (!radioPainYes->Checked && !UnblankValue(editPain).empty())
   {
    PSay("Please do not enter a pain description if the patient is not currently in pain.", m_missing_data, MB_OK, "");
    return false;
   }
   if (!radioSmokeYes->Checked && !UnblankValue(editSmoke).empty())
   {
    PSay("Please do not enter a smoking quantity if the patient is not currently smoking.", m_missing_data, MB_OK, "");
    return false;
   }
   if (!radioSmokedYes->Checked && !UnblankValue(editSmoked).empty())
   {
    PSay("Please do not enter a smoking history duration if the patient does not have a history of smoking.", m_missing_data, MB_OK, "");
    return false;
   }
   if (!radioCareYes->Checked && !UnblankValue(editCare).empty())
   {
    PSay("Please do not enter a reason for being under a doctor's care if the patient is not under a doctor's care.", m_missing_data, MB_OK, "");
    return false;
   }
   if (!radioHospitalizedYes->Checked && !UnblankValue(editHospitalized).empty())
   {
    PSay("Please do not enter a reason for being hospitalized if the patient has not been hospitalized.", m_missing_data, MB_OK, "");
    return false;
   }
   if (!radioRadiationYes->Checked && !UnblankValue(editRadiation).empty())
   {
    PSay("Please do not enter a reason for having radiation treatment if the patient has not had radiation treatment.", m_missing_data, MB_OK, "");
    return false;
   }
   if (!radioPapSmearYes->Checked && !UnblankValue(editPapSmear).empty())
   {
    PSay("Please do not enter a pap smear date if the patient has not had a pap smear in the last year.", m_missing_data, MB_OK, "");
    return false;
   }
   if (!UnblankValue(editMammogram).empty())
    if (!Pdate(UnblankValue(editMammogram), global_mode).IsValidDate())
    {
     PSay("Please enter a valid mammogram date.", m_missing_data, MB_OK, "");
     return false;
    }
   if (!UnblankValue(editDiabetesDiag).empty())
    if (!Pdate(UnblankValue(editDiabetesDiag), global_mode).IsValidDate())
    {
     PSay("Please enter a valid diabetes diagnosis date.", m_missing_data, MB_OK, "");
     return false;
    }
   if (!UnblankValue(editPapSmear).empty())
    if (!Pdate(UnblankValue(editPapSmear), global_mode).IsValidDate())
    {
     PSay("Please enter a valid pap smear date.", m_missing_data, MB_OK, "");
     return false;
    }
   return true;
  case 4: // med-history 2
   // what could -possibly- go wrong? i ask thee.
   // more questions on this page means stuff can go wrong now
   if (!radioUrgentYes->Checked && !radioInjuryDrugsNA->Checked)
   {
    PSay("Please do not enter a drugs/alcohol reason for an injury that did not occur.", m_missing_data, MB_OK, "");
    return false;
   }
   if (!radioAutoYes->Checked && !radioAutoDrugsNA->Checked)
   {
    PSay("Please do not enter a drugs/alcohol reason for an auto accident that did not occur.", m_missing_data, MB_OK, "");
    return false;
   }
   // should we also verify that if Yes to injury that drugs/alcohol is Y/N and not NA?
   return true;
  case 5: // optometry
   if (!radioGlassesYes->Checked && !UnblankValue(editGlassesOld).empty())
   {
    PSay("Please do not tell me how old the non-existent glasses are.", m_missing_data, MB_OK, "");
    return false;
   }
   if (!radioContactsYes->Checked && !UnblankValue(editContactsOld).empty())
   {
    PSay("Please do not tell me how old the non-existent contacts are.", m_missing_data, MB_OK, "");
    return false;
   }
   if (!radioContactsYes->Checked && !radioContactTypeNA->Checked)
   {
    PSay("Please do not tell me what type the non-existent contacts are.", m_missing_data, MB_OK, "");
    return false;
   }
   if (!radioContactsYes->Checked && !radioComfyNA->Checked)
   {
    PSay("Please do not tell me how comfy the non-existent contacts are.", m_missing_data, MB_OK, "");
    return false;
   }
   return true;
  case 6: // consents signed
   // either they check stuff, or they don't ... but i don't think we care? do we?
   // fine. we'll warn if they don't check any.
   if (consents_filter.GetMultiID().empty())
    if (IDNO == PSay("You have not indicated that any consents were signed.\nAre you sure you want to continue?", m_confirmation, MB_YESNO))
     return false;
   return true;
 }
 return false; // jhaley 20110824: unknown page? well whatever it is, it's certainly not valid.
}
//---------------------------------------------------------------------------
bool TformWizardNewPerson::SkipPage (int page_index)
{
 if (framePerson->Business && (page_index == 1 || page_index == 2))
  return true;

 // skip if no view permissions
 if (page_index == 1 && optionsPermissions["table.relationships"]["view"] != "true")
  return true;
 if (page_index == 2 && optionsPermissions["table.insurance_coverage"]["view"] != "true" && optionsPermissions["table.insurance_policies"]["view"] != "true")
  return true;

 // skip if no insert permissions (no point in asking)
 // jhaley 20110922: optometry page folded into this check
 if ((page_index == 3 || page_index == 4 || page_index == 5) && /* TODO -c MAGIC: 'medical history', maybe renamed? see below too */
   optionsPermissions[LowercaseString(GetOneField(dbDatabase, "select permission_name from obs_question_set_types where id = 3"))]["insert"] != "true")
  return true;

 return false;
}
//---------------------------------------------------------------------------
int TformWizardNewPerson::WhereNext (int page_index)
{
 if (last_necessary_page == 1) // HACK
   close_on_finish = true;
 if (page_index == last_necessary_page)
  return TformWizardGeneric::last_wizard_page;
 else
  return (page_index + 1);
}
//---------------------------------------------------------------------------
bool TformWizardNewPerson::ProcessPage (int page_index)
{
 return true;
}
//---------------------------------------------------------------------------
void TformWizardNewPerson::SetupPage (int page_index)
{
 last_active_control = NULL;

 is_loading_from_db = true;
 switch (page_index)
 {
  case 0:
   dateEverything->Date = Date(); // tada! i mean, uh, today!
   if ((framePerson->Magic == trm_existing && optionsPermissions["table.people"]["update"] == "true") ||
       (framePerson->Magic == trm_new && optionsPermissions["table.people"]["insert"] == "true"))
   {
    if (current_action == is_intake)
    {
      framePerson->Magic = trm_existing; // easy way to reset?
      framePerson->Magic = trm_new;
    }
    else // if we're updating, then Do() will have already set Magic and Id
      framePerson->Expanded = true; // the auto expand on set id thing isn't working and ... frankly I don't care.  we do this and move on.

   }
   else
   {
    framePerson->Magic = trm_existing;
    framePerson->Id = "0"; // reset? (i hope)
   }
   refresh_relationships = true;
   refresh_insurance = true;
   refresh_consents = true;
   break;
  case 1:
   if (refresh_relationships)
   {
    // has to be visible in order to properly resize the first time? I don't get it. Fucking with AutoSize,
    // ProcessMessages(), etc. doesn't seem to solve it, either. Stupid!
    sheetRelationships->Show();
    // frameRelationships->Visible = true;
    paddedRelationships->PaintSuspended = true;
    frameRelationships->Clear();
    // scrollRelationships->Visible = false;
    if (!IsBlankOrZero(framePerson->Id))
    {
     string sql;
     vector<string> ids;

     paddedRelationships->Visible = true;
     sql = "select id from relationships where (person_a_id = " + framePerson->Id + " or person_b_id = " + framePerson->Id + ") and current_id > 2";
     if (SqlToVec(dbDatabase, sql, ids))
      for (vector<string>::iterator i = ids.begin(); i != ids.end(); i++)
      {
       TframeTinyRecordGeneric * z = frameRelationships->Add();
       // make sure ParentPersonId is properly set -before- setting Id so that SetId() will load properly (pick the right FK to show, and the right one to hide)
       // dynamic_cast<TframeTinyRecord_Relationship *>(z)->ParentPersonId = framePerson->Id;
       dynamic_cast<TframeTinyRecord_Relationship *>(z)->ParentPerson = framePerson;
       z->Id = *i;
       z->Expanded = false;
      }
      else
       PSay("Failed to get list of relationship records", m_database_error, MB_OK, sql);
     if (ids.empty())
      paddedRelationships->Visible = false;
    }
    paddedRelationships->PaintSuspended = false;
    refresh_relationships = false;
   }
   // scrollRelationships->Visible = true;
   // scrollRelationships->Width = scrollRelationships->Width - 1;
   break;
  case 2:
   if (refresh_insurance)
   {
    // has to be visible in order to properly resize the first time? I don't get it. Fucking with AutoSize,
    // ProcessMessages(), etc. doesn't seem to solve it, either. Stupid!
    sheetInsurance->Show();
    // frameInsurance->Visible = true;
    paddedInsurance->PaintSuspended = true;
    frameInsurance->Clear();
    if (!IsBlankOrZero(framePerson->Id))
    {
     string sql;
     vector<string> ids;

     paddedInsurance->Visible = true;
     sql = "select id from insurance_coverage where person_id = " + framePerson->Id + " and current_id > 2";
     if (SqlToVec(dbDatabase, sql, ids))
      for (vector<string>::iterator i = ids.begin(); i != ids.end(); i++)
      {
       TframeTinyRecordGeneric * z = frameInsurance->Add();
       z->Id = *i;
       z->Expanded = false;
      } else
       PSay("Failed to get list of insurance coverage records", m_database_error, MB_OK, sql);
     if (ids.empty())
      paddedInsurance->Visible = false;
    }
    paddedInsurance->PaintSuspended = false;
    refresh_insurance = false;
   }
   break;
  case 3:
   ResetRadio(panelPain);
   ResetText(editPain);

   ResetRadio(panelSmoke);
   ResetText(editSmoke);

   ResetRadio(panelSmoked);
   ResetText(editSmoked);

   ResetText(editMedications);
   ResetText(editDrugAllergies);

   ResetRadio(panelCare);
   ResetText(editCare);

   ResetRadio(panelHospitalized);
   ResetText(editHospitalized);

   ResetRadio(panelRadiation);
   ResetText(editRadiation);

   ResetRadio(panelImmunizations);
   ResetRadio(panelAmputations);
   ResetRadio(panelDialisys);
   ResetRadio(panelPregnant);
   ResetRadio(panelAlcohol);

   ResetDate(editMammogram);

   ResetDate(editDiabetesDiag);

   ResetRadio(panelPapSmear);
   ResetDate(editPapSmear);
   break;
  case 4:
   ResetRadio(panelUrgentCare);   // jhaley 20110718: TIPP intake questions
   ResetRadio(panelInjuryDrugs);
   ResetRadio(panelAutoAccident);
   ResetRadio(panelAutoDrugs);

   ResetCheckboxes(panelMedLeft);
   ResetCheckboxes(panelMedRight);
   break;
  case 5:
   ResetDate(editEyeExam);
   ResetRadio(panelGlasses);
   ResetText(editGlassesOld);
   ResetRadio(panelContacts);
   ResetText(editContactsOld);
   ResetRadio(panelContactType);
   ResetRadio(panelComfyContacts);
   ResetRadio(panelHardToDriveContacts);
   ResetCheckboxes(panelOptSymptoms);
   ResetCheckboxes(panelOptProblems);
   ResetText(editOther);
   break;
  case 6:
  {
   // if (refresh_consents) // bah. if they want to hit 'clear page' ... that's their business.
   // which reminds me -- as it is, that won't work on relationships and insurance, as they only refresh if you've
   // changed who the person is on the front page. oops? no good undo. might need undo buttons on tinyrecord (but where?!)
   // i considered changing the [X] button to be a menu item on the current_id menu, maybe under a divider, and only
   // enabled when appropriate. could do the same with revert (only available if editing? clear if new?)
   RefreshConsents();
   consents_filter.UncheckAll();
   refresh_consents = false;
   break;
  }
 }
 is_loading_from_db = false;
}
//---------------------------------------------------------------------------
bool TformWizardNewPerson::FinishWizard ()
{
 bool success = false;
 last_sql_lib_error.Reset();
 TIBTransaction * dbTransaction = NULL;

 try
 {
  try
  {
   FORM_TRY
    dbTransaction = new TIBTransaction(NULL);
   FORM_CATCH
   if (!dbTransaction)
    throw(0);
   StdTransaction(dbTransaction, dbDatabase);
   AtlasClear();

   // we should have already checked validity before getting here!

   if (!framePerson->Save(dbTransaction))
    throw(1);
   framePerson->AtlasGet(pending_atlas_notifications);

   if (HistoryIncludes(1))
    for(int i = 0; i < frameRelationships->RecordCount; i++)
    {
     TframeTinyRecord_Relationship * z = dynamic_cast<TframeTinyRecord_Relationship *>(frameRelationships->Record[i]);
     // z->ParentPersonId = framePerson->Id;
     z->ParentPerson = framePerson;
     if (!z->Save(dbTransaction))
      throw(2);
     z->AtlasGet(pending_atlas_notifications);
    }

   if (HistoryIncludes(2))
    for(int i = 0; i < frameInsurance->RecordCount; i++)
    {
     TframeTinyRecord_InsuranceCoverage * z = dynamic_cast<TframeTinyRecord_InsuranceCoverage *>(frameInsurance->Record[i]);
     z->PersonId = framePerson->Id;
     if (!z->Save(dbTransaction))
      throw(3);
     z->AtlasGet(pending_atlas_notifications);
    }

   // do things

   // consents
   if (HistoryIncludes(6))
   {
    pmapstrs consent_row;
    CreateInsertMap (dbTransaction, "consents", consent_row);
    consent_row["current_id"]        = "4";
    consent_row["last_update_stamp"] = RightNow();
    consent_row["last_updated_by"]   = current_userid;
    vector<string> to_insert;

    to_insert = consents_filter.GetMultiID();

 /*
    vector<map<string, string> > selected_row_data = consents_filter.GetAllData (true, true);
    for (vector<map<string, string> >::iterator i = selected_row_data.begin(); i != selected_row_data.end(); i++)
     to_insert.push_back((*i)["consent_id"]);
 */

    for (vector<string>::iterator i = to_insert.begin(); i != to_insert.end(); i++)
    {
     consent_row["id"] = GetNextId(dbTransaction, "consents");
     consent_row["consent_id"] = (*i);
     consent_row["person_id"] = framePerson->Id;
     consent_row["signature_date"] = Pdate(dateEverything->Date).OutDate(global_mode);
     // :::::LOGIC COPIED FROM RECORDCONSENT:::::
     /* TODO : what the... ???  We should probably always use the current_groupid or,
        Philip suggests we let them select from the groups they have permission to */
     if((optionsPermissions["special.lac.general"]["enabled"] != "true") &&
       (optionsPermissions["special.den.general"]["enabled"] != "true") &&
       (optionsPermissions["special.sc.general"]["enabled"] != "true") &&
       (optionsPermissions["special.diab.general"]["enabled"] != "true"))
      consent_row["group_id"] = lookupTables->IdOf("groups", "Third Party Resources");
     else
      consent_row["group_id"] = current_groupid;

     if (!ExecuteInsertStatement(dbTransaction, "consents", consent_row))
      throw(4);
     AtlasAdd("consents", consent_row);
     // TODO: do we want atlas refreshing on tinyrecords? easy enough to do, i suppose ... record locking? meh.
    }
   }

   pvecmap medical_hx;
   pvecmap optometry;
   pmapstrs t;

   // medical history as radio buttons ...
   // TODO -c MAGIC: jhaley 20110718 - hard-coded IDs!!!!

   // I AM SORRY
   // the code above, and the code below, is messy.
   // - it doesn't use recordObsRespSet's code to auto-display stuff in frames, because it wouldn't at all have looked
   //   like the form
   // - i didn't write my own, because it seemed like it would have taken longer
   // - i was too lazy to write a recursive algorithm that would look for controls with non-zero ->Tag values
   //   (for obs_question_id) and auto-save based on the type of control found
   // you're welcome to implement whichever solutions you think are best (least messy for expansion) ...
   // i was admittedly going for quick, dirty, and working.
   // and if you're reading this, i probably screwed up. thus, i'm sorry.
   // - philip 5/2/2006
   // I AM SORRY

   if (HistoryIncludes(3))
   {
    AddFromRadio(dbTransaction, panelPain, "57", medical_hx);
    AddFromText(editPain, "58", medical_hx);

    AddFromRadio(dbTransaction, panelSmoke, "59", medical_hx);
    AddFromText(editSmoke, "125", medical_hx);

    AddFromRadio(dbTransaction, panelSmoked, "60", medical_hx);
    AddFromText(editSmoked, "61", medical_hx);

    AddFromText(editMedications, "62", medical_hx);
    AddFromText(editDrugAllergies, "63", medical_hx);

    AddFromRadio(dbTransaction, panelCare, "64", medical_hx);
    AddFromText(editCare, "65", medical_hx);

    AddFromRadio(dbTransaction, panelHospitalized, "66", medical_hx);
    AddFromText(editHospitalized, "67", medical_hx);

    AddFromRadio(dbTransaction, panelRadiation, "68", medical_hx);
    AddFromText(editRadiation, "69", medical_hx);

    AddFromRadio(dbTransaction, panelImmunizations, "70", medical_hx);
    AddFromRadio(dbTransaction, panelAmputations, "71", medical_hx);
    AddFromRadio(dbTransaction, panelDialisys, "72", medical_hx);
    AddFromRadio(dbTransaction, panelLasik, "173", medical_hx);
    AddFromRadio(dbTransaction, panelPregnant, "73", medical_hx);
    AddFromRadio(dbTransaction, panelAlcohol, "75", medical_hx);

    AddFromDate(editMammogram, "74", medical_hx);

    AddFromDate(editDiabetesDiag, "124", medical_hx);

    AddFromRadio(dbTransaction, panelPapSmear, "126", medical_hx);
    AddFromDate(editPapSmear, "127", medical_hx);
   }

   // self vs. relative checkboxes ...
   if (HistoryIncludes(4))
   {
    // jhaley 20110718: added new TIPP questions (require IDs to exist first...)
    AddFromRadio(dbTransaction, panelUrgentCare,   "191", medical_hx);
    AddFromRadio(dbTransaction, panelInjuryDrugs,  "192", medical_hx);
    AddFromRadio(dbTransaction, panelAutoAccident, "193", medical_hx);
    AddFromRadio(dbTransaction, panelAutoDrugs,    "194", medical_hx);

    AddFromCheck(dbTransaction, checkAsthmaSelf, "76", medical_hx);
    AddFromCheck(dbTransaction, checkAsthmaRelative, "77", medical_hx);

    AddFromCheck(dbTransaction, checkBleedingSelf, "78", medical_hx);
    AddFromCheck(dbTransaction, checkBleedingRelative, "79", medical_hx);

    AddFromCheck(dbTransaction, checkBoneSelf, "80", medical_hx);
    AddFromCheck(dbTransaction, checkBoneRelative, "81", medical_hx);

    AddFromCheck(dbTransaction, checkCancerSelf, "82", medical_hx);
    AddFromCheck(dbTransaction, checkCancerRelative, "83", medical_hx);

    AddFromCheck(dbTransaction, checkConvulsionsSelf, "84", medical_hx);
    AddFromCheck(dbTransaction, checkConvulsionsRelative, "85", medical_hx);

    AddFromCheck(dbTransaction, checkInfectionSelf, "86", medical_hx);
    AddFromCheck(dbTransaction, checkInfectionRelative, "87", medical_hx);

    AddFromCheck(dbTransaction, checkDiabetesSelf, "88", medical_hx);
    AddFromCheck(dbTransaction, checkDiabetesRelative, "89", medical_hx);

    AddFromCheck(dbTransaction, checkEyeSelf, "90", medical_hx);
    AddFromCheck(dbTransaction, checkEyeRelative, "91", medical_hx);

    AddFromCheck(dbTransaction, checkEmotionalSelf, "92", medical_hx);
    AddFromCheck(dbTransaction, checkEmotionalRelative, "93", medical_hx);

    AddFromCheck(dbTransaction, checkHeartSelf, "94", medical_hx);
    AddFromCheck(dbTransaction, checkHeartRelative, "95", medical_hx);

    AddFromCheck(dbTransaction, checkHepatitisSelf, "96", medical_hx);
    AddFromCheck(dbTransaction, checkHepatitisRelative, "97", medical_hx);

    AddFromCheck(dbTransaction, checkPressureSelf, "98", medical_hx);
    AddFromCheck(dbTransaction, checkPressureRelative, "99", medical_hx);

    AddFromCheck(dbTransaction, checkKidneySelf, "100", medical_hx);
    AddFromCheck(dbTransaction, checkKidneyRelative, "101", medical_hx);

    AddFromCheck(dbTransaction, checkLiverSelf, "102", medical_hx);
    AddFromCheck(dbTransaction, checkLiverRelative, "103", medical_hx);

    AddFromCheck(dbTransaction, checkMouthSelf, "104", medical_hx);
    AddFromCheck(dbTransaction, checkMouthRelative, "105", medical_hx);

    AddFromCheck(dbTransaction, checkNervousSelf, "106", medical_hx);
    AddFromCheck(dbTransaction, checkNervousRelative, "107", medical_hx);

    AddFromCheck(dbTransaction, checkRheumaticSelf, "108", medical_hx);
    AddFromCheck(dbTransaction, checkRheumaticRelative, "109", medical_hx);

    AddFromCheck(dbTransaction, checkSTDSelf, "110", medical_hx);
    AddFromCheck(dbTransaction, checkSTDRelative, "111", medical_hx);

    AddFromCheck(dbTransaction, checkStomachSelf, "112", medical_hx);
    AddFromCheck(dbTransaction, checkStomachRelative, "113", medical_hx);

    AddFromCheck(dbTransaction, checkSurgerySelf, "114", medical_hx);
    AddFromCheck(dbTransaction, checkSurgeryRelative, "115", medical_hx);

    AddFromCheck(dbTransaction, checkSwollenSelf, "116", medical_hx);
    AddFromCheck(dbTransaction, checkSwollenRelative, "117", medical_hx);

    AddFromCheck(dbTransaction, checkThyroidSelf, "118", medical_hx);
    AddFromCheck(dbTransaction, checkThyroidRelative, "119", medical_hx);

    AddFromCheck(dbTransaction, checkTuberculosisSelf, "120", medical_hx);
    AddFromCheck(dbTransaction, checkTuberculosisRelative, "121", medical_hx);

    AddFromCheck(dbTransaction, checkWeightSelf, "122", medical_hx);
    AddFromCheck(dbTransaction, checkWeightRelative, "123", medical_hx);
   }

   if ((HistoryIncludes(3) || HistoryIncludes(4)) && !medical_hx.empty())
   {
    map<string, string> set_row;
    CreateInsertMap (dbTransaction, "obs_response_sets", set_row);
    set_row["current_id"]           = "4";
    set_row["last_update_stamp"]    = RightNow();
    set_row["last_updated_by"]      = current_userid;
    set_row["id"]                   = GetNextId(dbTransaction, "obs_response_sets");
    set_row["person_id"]            = framePerson->Id;
    set_row["obs_question_set_id"]  = "3"; // TODO -c MAGIC: 'medical history' may wind up being renamed soon? not like any excuse really works.
    set_row["is_steps_observation"] = "0";
    set_row["group_id"]             = current_groupid;
    set_row["date_of_observation"]  = Pdate(dateEverything->Date).OutDate(global_mode);

    if (!ExecuteInsertStatement(dbTransaction, "obs_response_sets", set_row))
     throw(5);
    AtlasAdd("obs_response_sets", set_row);

    map<string, int> fields_to_leave_raw;
    fields_to_leave_raw["response_text"] = sql_no_uppercasing;

    map<string, string> _response_row;
    map<string, string> response_row;
    CreateInsertMap (dbTransaction, "obs_responses", _response_row);
    _response_row["current_id"]          = "4";
    _response_row["last_update_stamp"]   = RightNow();
    _response_row["last_updated_by"]     = current_userid;
    _response_row["obs_response_set_id"] = set_row["id"];
    for (vector<map<string, string> >::iterator i = medical_hx.begin(); i != medical_hx.end(); i++)
    {
     response_row = _response_row;
     response_row["id"] = GetNextId(dbTransaction, "obs_responses");
     MergeMap (response_row, *i);

     if(!ExecuteInsertStatement(dbTransaction, "obs_responses", response_row, fields_to_leave_raw))
      throw(6);
     AtlasAdd("obs_responses", response_row);
    }
   }

   // optometry questions
   if (HistoryIncludes(5))
   {
    AddFromDate(editEyeExam, "142", optometry);
    AddFromRadio(dbTransaction, panelGlasses,   "195", optometry);
    AddFromMetric(editGlassesOld,  "196", optometry);
    AddFromRadio(dbTransaction, panelContacts, "198", optometry);
    AddFromMetric(editContactsOld,  "197", optometry);
    AddFromRadio(dbTransaction, panelContactType, "199", optometry);
    AddFromRadio(dbTransaction, panelComfyContacts, "200", optometry);
    AddFromRadio(dbTransaction, panelHardToDriveContacts, "201", optometry);
    AddFromCheck(dbTransaction, checkHeadaches, "202", optometry);
    AddFromCheck(dbTransaction, checkMigraines, "203", optometry);
    AddFromCheck(dbTransaction, checkSeizures, "204", optometry);
    AddFromCheck(dbTransaction, checkVisionLoss, "205", optometry);
    AddFromCheck(dbTransaction, checkBlurredVision, "206", optometry);
    AddFromCheck(dbTransaction, checkDistortedVision, "207", optometry);
    AddFromCheck(dbTransaction, checkItching, "208", optometry);
    AddFromCheck(dbTransaction, checkBurning, "209", optometry);
    AddFromCheck(dbTransaction, checkSties, "210", optometry);
    AddFromCheck(dbTransaction, checkTiredEyes, "211", optometry);
    AddFromCheck(dbTransaction, checkAllergies, "212", optometry);
    AddFromCheck(dbTransaction, checkCongestion, "213", optometry);
    AddFromCheck(dbTransaction, checkRunnyNose, "214", optometry);
    AddFromCheck(dbTransaction, checkCough, "215", optometry);
    AddFromCheck(dbTransaction, checkDryMouth, "216", optometry);
    AddFromCheck(dbTransaction, checkWateryEyes, "217", optometry);
    AddFromCheck(dbTransaction, checkLightSensitive, "218", optometry);
    AddFromCheck(dbTransaction, checkFlashers, "219", optometry);
    AddFromCheck(dbTransaction, checkPeripheralLoss, "220", optometry);
    AddFromCheck(dbTransaction, checkDoubleVision, "221", optometry);
    AddFromCheck(dbTransaction, checkDryEyes, "222", optometry);
    AddFromCheck(dbTransaction, checkMucous, "223", optometry);
    AddFromCheck(dbTransaction, checkRedness, "224", optometry);
    AddFromCheck(dbTransaction, checkGritty, "225", optometry);
    AddFromCheck(dbTransaction, checkForeignBody, "226", optometry);
    AddFromCheck(dbTransaction, checkEyePain, "227", optometry);
    AddFromCheck(dbTransaction, checkPressure, "228", optometry);
    AddFromCheck(dbTransaction, checkCrossedEyesSelf, "229", optometry);
    AddFromCheck(dbTransaction, checkCrossedEyesRel, "230", optometry);
    AddFromCheck(dbTransaction, checkLazyEyeSelf, "231", optometry);
    AddFromCheck(dbTransaction, checkLazyEyeRel, "232", optometry);
    AddFromCheck(dbTransaction, checkDroopingEyeSelf, "233", optometry);
    AddFromCheck(dbTransaction, checkDroopingEyeRel, "234", optometry);
    AddFromCheck(dbTransaction, checkProminentEyeSelf, "235", optometry);
    AddFromCheck(dbTransaction, checkProminentEyeRel, "236", optometry);
    AddFromCheck(dbTransaction, checkGlaucomaSelf, "237", optometry);
    AddFromCheck(dbTransaction, checkGlaucomaRel, "238", optometry);
    AddFromCheck(dbTransaction, checkCataractsSelf, "239", optometry);
    AddFromCheck(dbTransaction, checkCataractsRel, "240", optometry);
    AddFromCheck(dbTransaction, checkLupusSelf, "241", optometry);
    AddFromCheck(dbTransaction, checkLupusRel, "242", optometry);
    AddFromCheck(dbTransaction, checkEyeInfectionSelf, "243", optometry);
    AddFromCheck(dbTransaction, checkEyeInfectionRel, "244", optometry);
    AddFromCheck(dbTransaction, checkEyeInjuriesSelf, "245", optometry);
    AddFromCheck(dbTransaction, checkEyeInjuriesRel, "246", optometry);
    AddFromCheck(dbTransaction, checkBlindnessSelf, "247", optometry);
    AddFromCheck(dbTransaction, checkBlindnessRel, "248", optometry);
    AddFromCheck(dbTransaction, checkMDSelf, "249", optometry);
    AddFromCheck(dbTransaction, checkMDRel, "250", optometry);
    AddFromCheck(dbTransaction, checkRetinalDiseaseSelf, "251", optometry);
    AddFromCheck(dbTransaction, checkRetinalDiseaseRel, "252", optometry);
    AddFromCheck(dbTransaction, checkDetachedRetinaSelf, "253", optometry);
    AddFromCheck(dbTransaction, checkDetachedRetinaRel, "254", optometry);
    AddFromText(editOther, "255", optometry);
   }

   if (HistoryIncludes(5) && !optometry.empty())
   {
    map<string, string> set_row;
    CreateInsertMap (dbTransaction, "obs_response_sets", set_row);
    set_row["current_id"]           = "4";
    set_row["last_update_stamp"]    = RightNow();
    set_row["last_updated_by"]      = current_userid;
    set_row["id"]                   = GetNextId(dbTransaction, "obs_response_sets");
    set_row["person_id"]            = framePerson->Id;
    set_row["obs_question_set_id"]  = "8"; // TODO -c MAGIC: 'optometry' ... this is the only thing different from above.  so sorry.
    set_row["is_steps_observation"] = "0";
    set_row["group_id"]             = current_groupid;
    set_row["date_of_observation"]  = Pdate(dateEverything->Date).OutDate(global_mode);

    if (!ExecuteInsertStatement(dbTransaction, "obs_response_sets", set_row))
     throw(5);
    AtlasAdd("obs_response_sets", set_row);

    map<string, int> fields_to_leave_raw;
    fields_to_leave_raw["response_text"] = sql_no_uppercasing;

    map<string, string> _response_row;
    map<string, string> response_row;
    CreateInsertMap (dbTransaction, "obs_responses", _response_row);
    _response_row["current_id"]          = "4";
    _response_row["last_update_stamp"]   = RightNow();
    _response_row["last_updated_by"]     = current_userid;
    _response_row["obs_response_set_id"] = set_row["id"];
    for (vector<map<string, string> >::iterator i = optometry.begin(); i != optometry.end(); i++)
    {
     response_row = _response_row;
     response_row["id"] = GetNextId(dbTransaction, "obs_responses");
     MergeMap (response_row, *i);

     if(!ExecuteInsertStatement(dbTransaction, "obs_responses", response_row, fields_to_leave_raw))
      throw(6);
     AtlasAdd("obs_responses", response_row);
    }
   }

   // head-of-household fix-ups, may or may not ask them anything ...
   if (HistoryIncludes(1))
   {
    set<string> people_involved;
    people_involved.insert(framePerson->Actual->Id);
    for(int i = 0; i < frameRelationships->RecordCount; i++)
    {
     TframeTinyRecord_Relationship * z = dynamic_cast<TframeTinyRecord_Relationship *>(frameRelationships->Record[i]);
     // string a = dynamic_cast<TframeTinyRecord_Person *>(z->ParentPerson->Actual)->Id;
     // people_involved.insert((z->ParentPerson && z->ParentPerson->Actual) ? a.c_str() : "0");
     people_involved.insert(z->framePerson->Actual->Id);
    }
    if (TformSpecialHouseholdMod::FindHousehold(dbTransaction, people_involved))
     if (TformSpecialHouseholdMod::ResolveHouseholdInformation(dbTransaction, people_involved))
      ;
     else
      throw(7);
    else
     throw(8);
   }

   dbTransaction->Commit();
   AtlasSend();

   Processing(false);
   success = true;
  }
  catch (int e)
  {
   PSay ("An error has been encountered. Your changes have not been saved.\n"
         "You may try again if the problem can be solved on your end (records open for editing, etc.)\n"
         "Please inform a programmer of this problem; the error log will have more information on its source.",
         m_program_error, MB_OK, "FinishWizard() failed on outer block for (" + IntToString(e) + ") reason");
  }
  catch (...)
  {
   PSay ("An error has been encountered. Your changes have not been saved.\n"
         "You may try again if the problem can be solved on your end (records open for editing, etc.)\n"
         "Please inform a programmer of this problem; the error log will have more information on its source.",
         m_program_error, MB_OK, "FinishWizard() failed on outer block for (...) reason");
  }
 }
 __finally
 {
  // note: this isn't working. GetLockInfo is returning false. don't feel like looking into why, but i don't think it's my fault.

  if (last_sql_lib_error.IsLockConflict())
  {
   PSay("ibx error number is " + IntToString(last_sql_lib_error.ibx_error) + "\n"
        "sql error number is " + IntToString(last_sql_lib_error.sql_error) +"\n"
        "message is " + last_sql_lib_error.message, m_debug, MB_OK);
   string local_table_name;
   string local_field_name;
   string foreign_table_name;
   string foreign_field_name;
   PSay(last_sql_lib_error.message, m_debug, MB_OK);
   if (last_sql_lib_error.GetLockInfo (dbTransaction, local_table_name, local_field_name, foreign_table_name, foreign_field_name))
   {
    PSay("If it's any help to you, it looks like the problem is that someone was editing something in '"
         + foreign_table_name + "' that was related to something you were doing in '" + local_table_name + "'.\n"
         "Make sure you don't have any record screens in editing mode that might conflict with what you're doing in "
         "this wizard, and if that doesn't fix it, ask around.", m_debug, MB_OK);
   }
   else
    PSay("Failed to get lock info", m_debug, MB_OK);
  }
  //else
  // PSay("Not a lock conflict", m_debug, MB_OK);

  if (dbTransaction && dbTransaction->Active)
   dbTransaction->Rollback();
  delete dbTransaction;
 }

 // i ... i dunno. i guess? maybe?
 if (success)
 {
  if (return_person_id)
    (*return_person_id) = framePerson->Id;
  done_onshow_person = true;
  done_onshow_relationships = true;
  done_onshow_insurance = true;
  current_action = is_intake; // default for every time around except the first (which is why it's not in Initialize or StartOver)
 }
 return success;
}
//---------------------------------------------------------------------------
void TformWizardNewPerson::AddFromRadio(TIBTransaction * dbTransaction, TPanel * p, string q_id, pvecmap & add_to)
{
 if (IsBlankOrZero(q_id))
  return;
 map<string, string> t;
 t["obs_question_id"] = q_id;
 for (int i = 0; i < p->ControlCount; i++)
 {
  if(p->Controls[i]->InheritsFrom(__classid(TRadioButton)))
  {
   TRadioButton * z = dynamic_cast<TRadioButton*>(p->Controls[i]);
   if (z && z->Checked)
   {
    t["obs_option_id"] = GetOneField(dbTransaction,
      "select obs_option_types.id "
      "from obs_option_types "
      "inner join obs_questions on obs_questions.obs_option_set_id = obs_option_types.obs_option_set_id "
      "where obs_questions.id = " + q_id + " and upper(obs_option_types.item) = '"
      + SafeSQLString(z->Caption.c_str()) + "'");
    break;
   }
  }
 }
 if (!IsBlankOrZero(t["obs_option_id"]))
  add_to.push_back(t);
 return;
}
//---------------------------------------------------------------------------
void TformWizardNewPerson::AddFromText(TEdit * e, string q_id, pvecmap & add_to)
{
 if (IsBlankOrZero(q_id))
  return;
 map<string, string> t;
 t["obs_question_id"] = q_id;
 t["response_text"] = UnblankValue(e);
 if (!t["response_text"].empty())
  add_to.push_back(t);
 return;
}
//---------------------------------------------------------------------------
void TformWizardNewPerson::AddFromMetric(TEdit * e, string q_id, pvecmap & add_to)
{
 if (IsBlankOrZero(q_id))
  return;
 map<string, string> t;
 t["obs_question_id"] = q_id;
 t["response_metric"] = UnblankValue(e);
 if (!t["response_metric"].empty())
  add_to.push_back(t);
 return;
}
//---------------------------------------------------------------------------
void TformWizardNewPerson::AddFromDate(TEdit * e, string q_id, pvecmap & add_to)
{
 if (IsBlankOrZero(q_id))
  return;
 map<string, string> t;
 t["obs_question_id"] = q_id;
 Pdate u(e->Text.c_str(), global_mode);
 if (e->Text != "" && u.IsValidDate())
 {
  t["response_date"] = u.OutDate(global_mode);
  add_to.push_back(t);
 }
 return;
}
//---------------------------------------------------------------------------
void TformWizardNewPerson::AddFromCheck(TIBTransaction * dbTransaction, TCheckBox * c, string q_id, pvecmap & add_to)
{
  if (IsBlankOrZero(q_id))
    return;
  map<string, string> t;
  t["obs_question_id"] = q_id;
  if (c && c->Checked)
  {
    t["obs_option_id"] = GetOneField(dbTransaction,
      "select obs_option_types.id "
      "from obs_option_types "
      "inner join obs_questions on obs_questions.obs_option_set_id = obs_option_types.obs_option_set_id "
      "where obs_questions.id = " + q_id + " and upper(obs_option_types.item) = 'YES'");
  }
  if (!IsBlankOrZero(t["obs_option_id"]))
    add_to.push_back(t);
}
//---------------------------------------------------------------------------
void TformWizardNewPerson::PageLockStatus (int page_index, bool locked)
{
 switch(page_index)
 {
  case 0:
   dateEverything->Enabled = !locked;
   framePerson->Enabled = !locked;
   break;
  case 1:
   buttonRelationships->Enabled = !locked && (optionsPermissions["table.relationships"]["insert"] == "true");
   for(int i = 0; i < frameRelationships->RecordCount; i++)
    (frameRelationships->Record[i])->Enabled = !locked;
   break;
  case 2:
   buttonInsurance->Enabled = !locked && (optionsPermissions["table.insurance_coverage"]["insert"] == "true");
   for(int i = 0; i < frameInsurance->RecordCount; i++)
    (frameInsurance->Record[i])->Enabled = !locked;
   break;
  case 3:
   panelPain->Enabled          = !locked;
   editPain->ReadOnly          =  locked;
   panelSmoke->Enabled         = !locked;
   editSmoke->ReadOnly         =  locked;
   panelSmoked->Enabled        = !locked;
   editSmoked->ReadOnly        =  locked;
   editMedications->ReadOnly   =  locked;
   editDrugAllergies->ReadOnly =  locked;
   panelCare->Enabled          = !locked;
   editCare->ReadOnly          =  locked;
   panelHospitalized->Enabled  = !locked;
   editHospitalized->ReadOnly  =  locked;
   panelRadiation->Enabled     = !locked;
   editRadiation->ReadOnly     =  locked;
   panelImmunizations->Enabled = !locked;
   panelAmputations->Enabled   = !locked;
   panelDialisys->Enabled      = !locked;
   panelPregnant->Enabled      = !locked;
   panelAlcohol->Enabled       = !locked;
   editMammogram->ReadOnly     =  locked;
   editDiabetesDiag->ReadOnly  =  locked;
   panelPapSmear->Enabled      = !locked;
   editPapSmear->ReadOnly      =  locked;
   break;
  case 4:
   panelUrgentCare->Enabled   = !locked; // jhaley 20110718: Added TIPP questions
   panelInjuryDrugs->Enabled  = !locked;
   panelAutoAccident->Enabled = !locked;
   panelAutoDrugs->Enabled    = !locked;
   panelMedLeft->Enabled      = !locked;
   panelMedRight->Enabled     = !locked;
   break;
  case 5:
   consents_filter.SetCheckLocked(locked);
   break;
 }
}
//---------------------------------------------------------------------------
void __fastcall TformWizardNewPerson::FormShow(TObject *Sender)
{
 TformWizardGeneric::FormShow(Sender);
 Initialize(); // are we doing this here because of sizing issues?
}
//---------------------------------------------------------------------------
void __fastcall TformWizardNewPerson::framePersonSetId(TObject *Sender)
{
 scrollDemographics->Width = scrollDemographics->Width - 1;
 refresh_relationships = true;
 refresh_insurance = true;
 refresh_consents = true;
}
//---------------------------------------------------------------------------
void __fastcall TformWizardNewPerson::frameRelationshipsChange(TObject *Sender)
{
 paddedRelationships->Visible = !frameRelationships->Empty;
 buttonRelationships->Caption =
   ("Relationships - Click here to add (" + IntToString(frameRelationships->RecordCount) + ")").c_str();
}
//---------------------------------------------------------------------------
void __fastcall TformWizardNewPerson::buttonRelationshipsClick(TObject *Sender)
{
 paddedRelationships->Visible = true;
 frameRelationships->Add();
}
//---------------------------------------------------------------------------
void __fastcall TformWizardNewPerson::FormClose(TObject *Sender, TCloseAction &Action)
{
 TformWizardGeneric::FormClose(Sender, Action);
 if (Action == caFree)
 {
  frameRelationships->Clear(); // should we have to?
  frameInsurance->Clear(); // should we have to?
 }
}
//---------------------------------------------------------------------------
void __fastcall TformWizardNewPerson::buttonInsuranceClick(TObject *Sender)
{
 paddedInsurance->Visible = true;
 frameInsurance->Add();
}
//---------------------------------------------------------------------------
void __fastcall TformWizardNewPerson::frameInsuranceChange(TObject *Sender)
{
 paddedInsurance->Visible = !frameInsurance->Empty;
 buttonInsurance->Caption =
   ("Insurance - Click here to add (" + IntToString(frameInsurance->RecordCount) + ")").c_str();
}
//---------------------------------------------------------------------------
void __fastcall TformWizardNewPerson::frameRelationshipsAdd(TObject *Sender)
{
 // note: this occurs for existing detail records as well as new ones,
 // but for existing ones, doesn't actually change anything:
 // this was already set right before they were given an Id to load
 // dynamic_cast<TframeTinyRecord_Relationship *>(Sender)->ParentPersonId = framePerson->Id;
 dynamic_cast<TframeTinyRecord_Relationship *>(Sender)->ParentPerson = framePerson;
}
//---------------------------------------------------------------------------
void __fastcall TformWizardNewPerson::frameInsuranceAdd(TObject *Sender)
{
 // dynamic_cast<TframeTinyRecord_Relationship *>(Sender)->ParentPersonId = framePerson->Id;
 // dynamic_cast<TframeTinyRecord_InsuranceCoverage *>(Sender)->PersonId = framePerson->Id;

 // note: this occurs for existing detail records as well as new ones ... try not to fuck it up?
 TframeTinyRecord_InsuranceCoverage * z = dynamic_cast<TframeTinyRecord_InsuranceCoverage *>(Sender);

 if (!is_loading_from_db)
 {
  // some nice defaults?
  z->framePolicy->editPolicyNumber->Text = StripAllExcept(framePerson->editSSN->Text.c_str(), uints).c_str();
  z->framePolicy->framePolicyHolder->Magic = trm_same;
  z->framePolicy->framePolicyHolder->SameAs = framePerson;
  z->editName->Text = framePerson->PersonName.c_str(); // name on coverage, not name on policy -- this should come from us (but shouldn't we do it on framePerson->OnNameChange(Sender) ?)
 }
}
//---------------------------------------------------------------------------
void __fastcall TformWizardNewPerson::sheetRelationshipsShow(
      TObject *Sender)
{
 if (!done_onshow_relationships)
 {
  // why did it stop working? no idea. but ... this certainly fixes it. ha.
  scrollRelationships->Align = alLeft;
  Application->ProcessMessages();
  scrollRelationships->Width = scrollRelationships->Width - 1;
  Application->ProcessMessages();
  scrollRelationships->Align = alClient;
  Application->ProcessMessages();
  done_onshow_relationships = true;
 }
}
//---------------------------------------------------------------------------
void __fastcall TformWizardNewPerson::FormShortCut(TWMKey &Msg,
      bool &Handled)
{
 // prevent recursion if we forward a shortcut to a tinyrecord, and it sends it back.
 if (is_handling_shortcut)
  return;

 is_handling_shortcut = true;
 TformWizardGeneric::FormShortCut(Msg, Handled);

 // locking a page and then having a user make use of the keyboard shortcut ... it's sad, you know?
 if (WizardLocked())
  return;

 if (ActiveControl)
  TframeTinyRecordGeneric::HandleShortCut(ActiveControl, Msg, Handled);

 if (sheetPerson->Visible)
  TframeTinyRecordGeneric::HandleShortCut(framePerson, Msg, Handled);

 if (sheetRelationships->Visible)
 {
  if(HIBYTE(GetAsyncKeyState(VK_CONTROL)) && !HIBYTE(GetAsyncKeyState(VK_MENU)) && Msg.CharCode == '5' && buttonRelationships->Visible && buttonRelationships->Enabled)
  {
   buttonRelationships->Click();
   Handled = true;
  }
 }

 if (sheetInsurance->Visible)
 {
  if(HIBYTE(GetAsyncKeyState(VK_CONTROL)) && !HIBYTE(GetAsyncKeyState(VK_MENU)) && Msg.CharCode == '6' && buttonInsurance->Visible && buttonInsurance->Enabled)
  {
   buttonInsurance->Click();
   Handled = true;
  }
 }
 is_handling_shortcut = false;
}
//---------------------------------------------------------------------------
void __fastcall TformWizardNewPerson::sheetPersonShow(TObject *Sender)
{
 if (!done_onshow_person)
 {
  scrollDemographics->Width = scrollDemographics->Width - 1; // nevermind, doesn't seem to really fix it.
  ActiveControl = framePerson->comboSource;
  done_onshow_person = true;
 }
}
//---------------------------------------------------------------------------
void __fastcall TformWizardNewPerson::scrollAnyMouseWheelDown(
      TObject *Sender, TShiftState Shift, TPoint &MousePos, bool &Handled)
{
 TScrollBox * z = dynamic_cast<TScrollBox*>(Sender);
 if (z && z->Visible)
  z->VertScrollBar->Position += 4; // z->VertScrollBar->Increment;
}
//---------------------------------------------------------------------------
void __fastcall TformWizardNewPerson::scrollAnyMouseWheelUp(
      TObject *Sender, TShiftState Shift, TPoint &MousePos, bool &Handled)
{
 TScrollBox * z = dynamic_cast<TScrollBox*>(Sender);
 if (z && z->Visible)
  z->VertScrollBar->Position -= 4; // z->VertScrollBar->Increment;
}
//---------------------------------------------------------------------------
void __fastcall TformWizardNewPerson::sheetInsuranceShow(TObject *Sender)
{
 if (!done_onshow_insurance)
 {
  scrollInsurance->Width = scrollInsurance->Width - 1;
  done_onshow_insurance = true;
 }
}
//---------------------------------------------------------------------------
void TformWizardNewPerson::EditConsent()
{
 map<string,string> params;
 // different field is marked as internal_id, so we have to reverse-match the fucker because filter doesn't have a "give me the selected row" function. and i don't want to add one right now.
 string c_id = consents_filter.GetSelectedRecordID();
 vector<map<string, string> > all_data = consents_filter.GetAllData(true, false);
 for (vector<map<string, string> >::iterator i = all_data.begin(); i != all_data.end(); i++)
  if (c_id == (*i)["consent_id"] && !IsBlankOrZero((*i)["id"]))
  {
   params["id"] = (*i)["id"];
   TformRecordConsent::Do(params);
  }
}
//---------------------------------------------------------------------------
void TformWizardNewPerson::RefreshConsents()
{
 consents_data->field_names.clear();
 consents_data->field_names.push_back("consent");
 consents_data->field_names.push_back("consent_id");
 consents_data->field_names.push_back("signature_date");
 consents_data->field_names.push_back("elapsed");
 consents_data->field_names.push_back("note");
 consents_data->field_names.push_back("group");
 consents_data->field_names.push_back("current_id");
 consents_data->field_names.push_back("id");

 consents_data->field_types.clear();
 consents_data->field_types.push_back(ftString);
 consents_data->field_types.push_back(ftInteger);
 consents_data->field_types.push_back(ftDate);
 consents_data->field_types.push_back(ftString);
 consents_data->field_types.push_back(ftString);
 consents_data->field_types.push_back(ftString);
 consents_data->field_types.push_back(ftInteger);
 consents_data->field_types.push_back(ftInteger);

 string sql;
 if (framePerson->Actual->HasId)
  sql = "select consent_types.item c_item, "
        "consent_types.id c_id, "
        "OC.signature_date, "
        "OC.note, "
        "groups.item g_item, "
        "OC.current_id, "
        "OC.id "
        "from consent_types "
        "left join consents OC on "
         "OC.id = (select first 1 IC.id from consents IC where IC.person_id = " + framePerson->Actual->Id +
         " and IC.consent_id = consent_types.id and IC.current_id > 2 order by IC.signature_date desc) "
        "left join groups on "
         "OC.group_id = groups.id "
        "where consent_types.valid = 1";
 else
  sql = "select consent_types.item c_item, consent_types.id c_id, NULL signature_date, NULL note, NULL g_item, NULL current_id, NULL id "
        "from consent_types "
        "where consent_types.valid = 1 and consent_types.id > 0";

 vector<map<string, string> > raw_data;
 SqlToVecMap(dbDatabase, sql, raw_data);

 vector<string> temp;
 temp.resize(8);
 consents_data->field_values.clear();
 Pdate today_date(Date());
 for (vector<map<string, string> >::iterator i = raw_data.begin(); i != raw_data.end(); i++)
 {
  Pdate then_date((*i)["signature_date"], global_mode);
  string time_since = "";
  if (then_date.IsValidDate())
   time_since = GetTimeSince(today_date, then_date);

  temp[0] = (*i)["c_item"];
  temp[1] = (*i)["c_id"];
  temp[2] = (*i)["signature_date"];
  temp[3] = time_since;
  temp[4] = (*i)["note"];
  temp[5] = (*i)["g_item"];
  temp[6] = (*i)["current_id"];
  temp[7] = (*i)["id"];
  consents_data->field_values.push_back(temp);
  // PSay(VecToCommaString(temp), m_debug, MB_OK);
 }

 consents_filter.RefreshDataSet();
}
//---------------------------------------------------------------------------
void TformWizardNewPerson::ResetRadio(TPanel * p)
{
 if (p)
  for (int i = 0; i < p->ControlCount; i++)
   if(p->Controls[i]->InheritsFrom(__classid(TRadioButton)))
   {
    TRadioButton * z = dynamic_cast<TRadioButton*>(p->Controls[i]);
    if (z && z->Caption == "N/A")
    {
     z->Checked = true;
     break;
    }
   }
 return;
}
//---------------------------------------------------------------------------
void TformWizardNewPerson::ResetCheckboxes(TPanel * p)
{
 if (p)
  for (int i = 0; i < p->ControlCount; i++)
   if(p->Controls[i]->InheritsFrom(__classid(TCheckBox)))
   {
    TCheckBox * z = dynamic_cast<TCheckBox*>(p->Controls[i]);
    z->Checked = false;
   }
 return;
}
//---------------------------------------------------------------------------
void TformWizardNewPerson::ResetText(TEdit * e)
{
 if (e)
  e->Text = "";
 return;
}
//---------------------------------------------------------------------------
void TformWizardNewPerson::ResetDate(TEdit * e)
{
 if (e)
  e->Text = "";
 return;
}
//---------------------------------------------------------------------------
void TformWizardNewPerson::ResetCheck(TCheckBox * c)
{
 if (c)
  c->Checked = false;
 return;
}
//---------------------------------------------------------------------------
void TformWizardNewPerson::Do ()
{
 FORM_TRY
  TformWizardNewPerson * z = new TformWizardNewPerson(NULL);
  if (z->framePerson->AllowNew)
   z->framePerson->Magic = trm_new;
  z->Show();
 FORM_CATCH
}
//---------------------------------------------------------------------------
void TformWizardNewPerson::Do (string person_id)
{
 TformWizardNewPerson * z;
 FORM_TRY
  z = new TformWizardNewPerson(NULL);
  if (!IsBlankOrZero(person_id))
  {
   z->current_action = is_update;
   z->framePerson->Magic = trm_existing;
   z->framePerson->Id = person_id;
   z->Show();
  }
 FORM_CATCH
}
//---------------------------------------------------------------------------
void __fastcall TformWizardNewPerson::framePersoncomboSourceChange(TObject *Sender)
{
  framePerson->comboSourceChange(Sender);
  if (framePerson->Magic == trm_existing)
    current_action = is_update;
  else if (framePerson->Magic == trm_new)
    current_action = is_intake;
}
//---------------------------------------------------------------------------

