//---------------------------------------------------------------------------

#include <vcl.h>
#pragma hdrstop

#include "codeWizardNewPurchaseRequest.h"
#include "launchPRReport.h"
#include "codeRecordTaxEntity.h"
#include "codeRecordIncomingBill.h"
//---------------------------------------------------------------------------
#pragma package(smart_init)
#pragma link "codeWizardGeneric"
#pragma resource "*.dfm"
TformWizardNewPurchaseRequest *formWizardNewPurchaseRequest;
//---------------------------------------------------------------------------
__fastcall TformWizardNewPurchaseRequest::TformWizardNewPurchaseRequest(TComponent* Owner)
        : TformWizardGeneric(Owner)
{
 tax_request_any       = 0;
 tx_request_any        = 0;
 bill_request_any      = 0;
 tax_request_specific  = 0;
 tx_request_specific   = 0;
 bill_request_specific = 0;

 tax_entities_filter.SetListViewHandle(listTaxEntities);
 tax_entities_filter.SetSecurity("tax_entities");
 tax_entities_filter.AddColumnSpec("Name" ,   "tax_entities", as_text, true, 160, rawString);
 tax_entities_filter.AddColumnSpec("Tax ID" , "tax_entities", as_text, true, 75, prettyString);
 tax_entities_filter.AddColumnSpec("Payable",   "transactions", as_double, true, 75, nonzeroAmountString);
 tax_entities_filter.AddColumnSpec("Current Id", "current_id", as_int, false, 80);
 tax_entities_filter.AddColumnSpec("Internal Id", "internal_id", as_int, false, 80);
 tax_entities_filter.FetchPreferences("wizardnewpurchaserequest.tax_entities_filter");
 tax_entities_filter.SetSQLStatement("");
 tax_entities_filter.RefreshDataSet();
 tax_entities_filter.SetAddEdit(this, NULL, EditRecord(&(TformWizardNewPurchaseRequest::EditTaxEntity)));

 bills_filter.SetListViewHandle(listBills);
 bills_filter.SetSecurity("bills");
 bills_filter.AddColumnSpec("Bill Type", "bills", as_text, true, 150, prettyString);
 bills_filter.AddColumnSpec("Person", "people", as_text, true, 120, prettyString);
 bills_filter.AddColumnSpec("Visit Date", "bills", as_date, true, 120);
 bills_filter.AddColumnSpec("Bill Date", "bills", as_date, true, 120);
 bills_filter.AddColumnSpec("Tax Entity", "tax_entities", as_text, true, 120, rawString);
 bills_filter.AddColumnSpec("Invoice Number", "bills", as_text, true, 120);
 bills_filter.AddColumnSpec("Note", "bills", as_text, true, 120, sentenceString);
 bills_filter.AddColumnSpec("Total Charge", "bills", as_double, true, 75, nonzeroAmountString);
 bills_filter.AddColumnSpec("Invoice Amount", "bills", as_double, true, 75, nonzeroAmountString);
 bills_filter.AddColumnSpec("Expenses", "bills", as_double, true, 75, nonzeroAmountString);
 bills_filter.AddColumnSpec("Payable", "bills", as_double, true, 75, nonzeroAmountString);
 bills_filter.AddColumnSpec("Paid", "bills", as_double, true, 75, nonzeroAmountString);
 bills_filter.AddColumnSpec("Revenue", "bills", as_double, true, 75, nonzeroAmountString);
 bills_filter.AddColumnSpec("Receivable", "bills", as_double, true, 75, nonzeroAmountString);
 bills_filter.AddColumnSpec("Received", "bills", as_double, true, 75, nonzeroAmountString);
 bills_filter.AddColumnSpec("User Id", "users", as_int, false, 80);
 bills_filter.AddColumnSpec("Current Id", "current_id", as_int, false, 80);
 bills_filter.AddColumnSpec("Internal Id", "internal_id", as_int, false, 80);
 bills_filter.FetchPreferences("wizardnewpurchaserequest.bills_filter");
 bills_filter.SetCheck(true);
 bills_filter.SetSQLStatement("");
 bills_filter.RefreshDataSet();
 bills_filter.SetAddEdit(this, NULL, EditRecord(&(TformWizardNewPurchaseRequest::EditBill)));

 if(optionsPermissions["special.edit_pr_number"]["enabled"] != "true")
 {
  editPurchaseReqNumber->Enabled = false;
  radioUseSpecialNumber->Enabled = false;
 } else
 {
  editPurchaseReqNumber->Enabled = true;
  radioUseSpecialNumber->Enabled = true;
 }

 // SetExpectedPages(3);
 Initialize();
}
//---------------------------------------------------------------------------
bool TformWizardNewPurchaseRequest::IsValidPage (int page_index)
{
 switch (page_index)
 {
  case 0:
  {
   chosen_tax_entity_id = tax_entities_filter.GetSelectedRecordID();
   if (chosen_tax_entity_id.empty())
   {
    PSay ("You must select a tax entity for which to create a purchase request.", m_missing_data, MB_OK);
    return false;
   } else
   {
    string known_federal_id = GetOneField(dbDatabase, "select tin.identifier "
                                                      "from tax_entities "
                                                      "left join entity_ids tin on tin.tax_entity_id = tax_entities.id and tin.current_id >= 3 and tin.entity_id_id = '" + lookupTables->IdOf("entity_id_types", "TIN/EIN") + "' "
                                                      "where tax_entities.id = " + chosen_tax_entity_id);
    string numeric_federal_id = StripAllExcept(known_federal_id, uints);

    // digits is a #define'd quoted string ... concatenation via operator+ would result in pointer-addition, which is bad.
    if (known_federal_id.find_first_not_of(uints "-") != string::npos)
    {
     PSay ("The federal tax id number found contains non-digit, non-dash characters. It must be properly formatted before being used.", m_invalid_data, MB_OK);
     return false;
    } else
    if (numeric_federal_id.empty())
    {
     PSay ("There is no federal tax id number for this tax entity.\nThis must be corrected before any purchase requests may be generated.", m_missing_data, MB_OK);
     return false;
    } else
    if (numeric_federal_id.length() != 9)
    {
     if (IDNO == PSay ("The federal tax id number found is not 9 digits long.\nAre you sure you want to use it anyway?", m_invalid_data, MB_YESNO))
      return false;
    } else
    {
     // yes, we could watch for even more cases... but nah. not worth it at the moment, until they get clever. and deceitful. and then it's a different problem.
     if (numeric_federal_id == "999999999" || numeric_federal_id == "000000000" || numeric_federal_id == "123456789")
      if (IDNO == PSay ("It seems you're using a bogus federal tax id number.\nIf you can fix this before continuing, please do.\nAre you sure you want to continue anyway?", m_invalid_data, MB_YESNO, numeric_federal_id))
       return false;
    }
   }
   return true;
  }
  case 1:
  {
   chosen_bill_id = bills_filter.GetMultiID();
   if (chosen_bill_id.empty())
   {
    PSay ("You must select at least one bill to pay before continuing.\nIf no bills are listed, make sure you filter settings are correct,\nand make sure no other users are creating purchase requests for the same tax entity.", m_missing_data, MB_OK);
    return false;
   }
   return true;
  }
  case 2:
  {
   use_chosen_number = !(radioUseGenerator->Checked);
   chosen_number = editPurchaseReqNumber->Text.c_str();
   chosen_date = Pdate(datePurchaseReqDate->Date);

   if (chosen_date < Pdate(TDateTime::CurrentDate()))
   {
    if (IDNO == PSay("You have specified that you expect to send this purchase request before today.\nAre you sure you want to continue with this odd date?", m_confirmation, MB_YESNO))
     return false;
   }

   if (use_chosen_number)
   {
    if (!IsInt(chosen_number) || chosen_number.empty())
    {
     PSay ("You must enter a valid number to use as a purchase request number, or let the wizard generate one for you.", m_invalid_data, MB_OK);
     return false;
    } else
    {
     string current_generator_value = GetOneField(dbDatabase, "select gen_id(pr_auto_num_gen, 0) from uno");
     if (StringToInt(chosen_number) >= StringToInt(current_generator_value))
     {
      PSay ("Your chosen purchase request number is higher than the highest number in the current sequence.\nUsing this number would interfere with the auto-numbering system, and is therefore not allowed.\nPlease select a lower number.", m_invalid_data, MB_OK);
      return false;
     } else
     {
      string count_of_duplicates = GetOneField(dbDatabase, "select count(*) from purchase_reqs where purchase_req_number = " + chosen_number);
      if (StringToInt(count_of_duplicates) > 0)
      {
       if (IDNO == PSay("There already exist " + count_of_duplicates + " purchase requests with this same number.\nAre you sure you want to use it anyway?", m_duplicate_data, MB_YESNO, "pr number = " + chosen_number))
        return false;
      }
     }
    }
   }
   return true;
  }
  default:
   PSay("IsValidPage() is trying to access a page that does not exist.", m_program_error, MB_OK);
   return false;
 }
}
//---------------------------------------------------------------------------
int TformWizardNewPurchaseRequest::WhereNext (int page_index)
{
 if (page_index == 2)
  return TformWizardGeneric::last_wizard_page;
 else
  return (page_index + 1);
}
//---------------------------------------------------------------------------
bool TformWizardNewPurchaseRequest::ProcessPage (int page_index)
{
 switch (page_index)
 {
  case 0:
  {
   // chosen_tax_entity_id = tax_entities_filter.GetSelectedRecordID(); -- saved elsewhere
   return true;
  }
  case 1:
  {
   // chosen_bill_id = bills_filter.GetMultiID(); -- saved elsewhere
   return true;
  }
  case 2:
  {
   return true;
  }
  default:
   PSay("ProcessPage() is trying to access a page that does not exist.", m_program_error, MB_OK);
   return false;
 }
}
//---------------------------------------------------------------------------
void TformWizardNewPurchaseRequest::SetupPage (int page_index)
{
 switch (page_index)
 {
  case 0:
  {
   if (tax_request_any != 0) atlas->CancelRequest(tax_request_any);
   if (tx_request_any != 0) atlas->CancelRequest(tx_request_any);
   if (bill_request_any != 0) atlas->CancelRequest(bill_request_any);

   tax_request_any  = atlas->RequestNotificationWhen ("tax_entities.id", "0", rel_any, as_int, this, RefreshHandler(&(TformWizardNewPurchaseRequest::AtlasScreen)));
   tx_request_any   = atlas->RequestNotificationWhen ("transactions.id", "0", rel_any, as_int, this, RefreshHandler(&(TformWizardNewPurchaseRequest::AtlasScreen)));
   bill_request_any = atlas->RequestNotificationWhen ("bills.id", "0", rel_any, as_int, this, RefreshHandler(&(TformWizardNewPurchaseRequest::AtlasScreen)));

   string tax_sql = "select tax_entities.name, tin.identifier, "
                    "sum(transactions.credit_amount - transactions.debit_amount) ob, "
                    "tax_entities.current_id, tax_entities.id "
                    "from tax_entities "
                    "right join bills on bills.actual_tax_id = tax_entities.id "
                    "left join entity_ids tin on tin.tax_entity_id = tax_entities.id and tin.current_id >= 3 and tin.entity_id_id = '" + lookupTables->IdOf("entity_id_types", "TIN/EIN") + "' "
                    "inner join transactions on transactions.bill_id = bills.id and transactions.account_id in ("
                    "select accounts.id from accounts where account_type_id = " + lookupTables->IdOf("account_types", "Payable") +
                    ") where bills.purchase_req_id = 0 and tax_entities.id > 0 "
                    "group by tax_entities.name, tin.identifier, tax_entities.current_id, tax_entities.id "
                    "having sum(transactions.credit_amount - transactions.debit_amount) > 0.00";
   tax_entities_filter.SetSQLStatement(tax_sql);
   tax_entities_filter.RefreshDataSet();
   break;
  }
  case 1:
  {
   if (tax_request_specific != 0) atlas->CancelRequest(tax_request_specific);
   if (tx_request_specific != 0) atlas->CancelRequest(tx_request_specific);
   if (bill_request_specific != 0) atlas->CancelRequest(bill_request_specific);

   tax_request_specific  = atlas->RequestNotificationWhen ("tax_entities.id", chosen_tax_entity_id, rel_e, as_int, this, RefreshHandler(&(TformWizardNewPurchaseRequest::AtlasScreen)));
   tx_request_specific   = atlas->RequestNotificationWhen ("transactions.tax_entity_id", chosen_tax_entity_id, rel_e, as_int, this, RefreshHandler(&(TformWizardNewPurchaseRequest::AtlasScreen)));
   bill_request_specific = atlas->RequestNotificationWhen ("bills.actual_tax_id", chosen_tax_entity_id, rel_e, as_int, this, RefreshHandler(&(TformWizardNewPurchaseRequest::AtlasScreen)));

   string bill_sql = "select bill_types.item, person_names.last_name || ', ' || person_names.first_name, "
                     "tasks.date_of_task, "
                     "bills.bill_date, tax_entities.name, bills.invoice_number, bills.note, "
                     "bills.total_charge, bills.invoice_amount, "
                     "(select sum(debit_amount-credit_amount) from transactions where bill_id = bills.id and account_id in (select accounts.id from accounts where account_type_id = " + lookupTables->IdOf("account_types", "Expense") + ")) expense, "
                     "(select sum(credit_amount-debit_amount) from transactions where bill_id = bills.id and account_id in (select accounts.id from accounts where account_type_id = " + lookupTables->IdOf("account_types", "Payable") + ")) payable, "
                     "(select sum(credit_amount-debit_amount) from transactions where bill_id = bills.id and transactions.tax_entity_id = bills.actual_tax_id and account_id in (select accounts.id from accounts where account_type_id = " + lookupTables->IdOf("account_types", "Cash") + ")) paid, "
                     "(select sum(credit_amount-debit_amount) from transactions where bill_id = bills.id and account_id in (select accounts.id from accounts where account_type_id = " + lookupTables->IdOf("account_types", "Revenue") + ")) revenue, "
                     "(select sum(debit_amount-credit_amount) from transactions where bill_id = bills.id and account_id in (select accounts.id from accounts where account_type_id = " + lookupTables->IdOf("account_types", "Receivable") + ")) receivable, "
                     "(select sum(debit_amount-credit_amount) from transactions where bill_id = bills.id and transactions.tax_entity_id != bills.actual_tax_id and account_id in (select accounts.id from accounts where account_type_id = " + lookupTables->IdOf("account_types", "Cash") + ")) received, "
                     "bills.last_updated_by, bills.current_id, bills.id "
                     "from bills "
                     "left join tax_entities on bills.actual_tax_id = tax_entities.id "
                     "left  join bill_types on bills.bill_id = bill_types.id "
                     "left join tasks on bills.task_id = tasks.id "
                     "left join people on tasks.person_id = people.id "
                     "left join person_names on people.person_names_id = person_names.id "
                     "where bills.actual_tax_id = " + chosen_tax_entity_id +
                     " and bills.purchase_req_id = 0 and "
                     "(select sum(credit_amount-debit_amount) from transactions where bill_id = bills.id and account_id in (select accounts.id from accounts where account_type_id = " + lookupTables->IdOf("account_types", "Payable") + ")) > 0.00 ";
                    // if aliases worked correctly, the above where clause could run a tad faster ... damn aliases not working! (assuming the 'having' clause doesn't know how to re-use expressions, that is.)
   bills_filter.SetSQLStatement(bill_sql);
   bills_filter.RefreshDataSet();
   // bills_filter.CheckAll();
   bills_filter.SetCheck(this, CheckRecord(&(TformWizardNewPurchaseRequest::CheckBill)));

   break;
  }
  case 2:
  {
   datePurchaseReqDate->DateTime = TDateTime::CurrentDate();
   editExpReason->Text           = "";
   radioUseGenerator->Checked    = true;
   editPurchaseReqNumber->Text   = "";
   RefreshTaxEntityInfo ();

   break;
  }
  default:
   PSay("SetupPage() is trying to access a page that does not exist.", m_program_error, MB_OK);
   break;
 }
}
//---------------------------------------------------------------------------
void TformWizardNewPurchaseRequest::EditTaxEntity()
{
 map<string,string> temp_map;
 temp_map["id"] = tax_entities_filter.GetSelectedRecordID();
 FORM_TRY
  TformRecordTaxEntity * tax_entity = new TformRecordTaxEntity(NULL, temp_map);
  tax_entity->Show();
 FORM_CATCH
 return;
}
//---------------------------------------------------------------------------
void TformWizardNewPurchaseRequest::EditBill()
{
 map<string, string> params;
 params["id"] = bills_filter.GetSelectedRecordID();
 FORM_TRY
  TformRecordIncomingBill * edit_bill = new TformRecordIncomingBill(NULL, params);
  edit_bill->Show();
 FORM_CATCH
 return;
}
//---------------------------------------------------------------------------
void TformWizardNewPurchaseRequest::AtlasScreen (int request_num, string matched_value)
{
 if (request_num == tax_request_any ||
     request_num == tx_request_any ||
     request_num == bill_request_any)
  tax_entities_filter.RefreshDataSet();

 if (request_num == tx_request_specific || request_num == bill_request_specific || request_num == tax_request_specific)
  bills_filter.RefreshDataSet();

 if (request_num == tax_request_specific)
  RefreshTaxEntityInfo();
}
//---------------------------------------------------------------------------
void TformWizardNewPurchaseRequest::RefreshTaxEntityInfo ()
{
 CreateLink (staticSummaryTaxEntity, NULL, this, chosen_tax_entity_id, "tax_entities", "table.tax_entities");
 map<string, string> info_about_entity;
 if (!SqlToMap(dbDatabase, "select tin.identifier o_tax_number, tax_addresses.line1 o_line_1, tax_addresses.line2 o_line_2, tax_addresses.city o_city, state_types.item o_state, tax_addresses.zip o_zip, country_types.item o_country "
                           "from tax_entities "
                           "inner join tax_addresses on tax_addresses.id = tax_entities.billing_address_id "
                           "inner join state_types on state_types.id = tax_addresses.state_id "
                           "inner join country_types on country_types.id = tax_addresses.country_id "
                           "left join entity_ids tin on tin.tax_entity_id = tax_entities.id and tin.current_id >= 3 and tin.entity_id_id = '" + lookupTables->IdOf("entity_id_types", "TIN/EIN") + "' "
                           "where tax_entities.id = " + chosen_tax_entity_id, info_about_entity))
  PSay("Failed to retrieve federal tax id number and billing address information, SqlToMap() failed.", m_database_error, MB_OK);

 labelSummaryTaxNumber->Caption         = (info_about_entity["o_tax_number"]).c_str();
 labelSummaryTaxEntityAddress1->Caption = FormatAddress(info_about_entity["o_line_1"]).c_str();
 labelSummaryTaxEntityAddress2->Caption = FormatAddress(info_about_entity["o_line_2"]).c_str();
 labelSummaryTaxEntityAddress3->Caption = PrettyString(info_about_entity["o_city"] + ", " + info_about_entity["o_state"] + ", " + info_about_entity["o_zip"]).c_str();
 if (!info_about_entity["o_country"].empty())
  labelSummaryTaxEntityAddress3->Caption = labelSummaryTaxEntityAddress3->Caption + PrettyString(" (" + info_about_entity["o_country"] + ")").c_str();
}
//---------------------------------------------------------------------------
void TformWizardNewPurchaseRequest::PageLockStatus (int page_index, bool locked)
{
 switch (page_index)
 {
  case 0:
   return;
  case 1:
   return;
  case 2:
   datePurchaseReqDate->Enabled = !locked;
   radioUseGenerator->Enabled = !locked;
   radioUseSpecialNumber->Enabled = !locked;
   editPurchaseReqNumber->Enabled = !locked; // && radioUseSpecialNumber->Checked; // we're not doing this here?
   return;
  default:
   PSay("PageLockStatus() is trying to access a page that does not exist.", m_program_error, MB_OK);
   return;
 }
}
//---------------------------------------------------------------------------
void __fastcall TformWizardNewPurchaseRequest::listBillsChanging(TObject *Sender,
   TListItem *Item, TItemChange Change, bool &AllowChange)
{
 AllowChange = !WizardLocked();
}
//---------------------------------------------------------------------------
void __fastcall TformWizardNewPurchaseRequest::listTaxEntitiesChanging(TObject *Sender,
   TListItem *Item, TItemChange Change, bool &AllowChange)
{
 AllowChange = !WizardLocked();
}
//---------------------------------------------------------------------------
bool TformWizardNewPurchaseRequest::CheckBill(const map<string, string> & p_row)
{
 map<string, string>::const_iterator i_field = p_row.find("User Id");
 if (i_field != p_row.end())
 {
  if (i_field->second == current_userid)
   return true;
 }
 return false;
}
//---------------------------------------------------------------------------
bool TformWizardNewPurchaseRequest::FinishWizard ()
{
 // create pr, attach bills to pr, and launch pr report
 TIBTransaction * dbTransaction = new TIBTransaction(NULL);
 StdTransaction(dbTransaction, dbDatabase);
 try
 {
  AtlasClear();
  map<string, string> new_purchase_req;

  string new_pr_number;
  if (use_chosen_number)
   new_pr_number = chosen_number;
  else
   new_pr_number = GetOneField(dbTransaction, "select gen_id(pr_auto_num_gen, 1) from uno");

  new_purchase_req["id"]                  = GetNextId(dbTransaction, "purchase_reqs");
  new_purchase_req["current_id"]          = "4";
  new_purchase_req["last_update_stamp"]   = RightNow();
  new_purchase_req["last_updated_by"]     = current_userid;
  new_purchase_req["check_id"]            = "0";
  new_purchase_req["tax_entity_id"]       = chosen_tax_entity_id;
  new_purchase_req["date_sent"]           = chosen_date.OutDate(global_mode);
  new_purchase_req["exp_reason"]          = editExpReason->Text.c_str();
  new_purchase_req["purchase_req_number"] = new_pr_number;

  AtlasAdd("purchase_reqs", new_purchase_req);
  if (!ExecuteInsertStatement(dbTransaction, "purchase_reqs", new_purchase_req))
   throw (3);

  // doubtful anything will be watching for the purchase_req_id above, but also doubtful they're watching for purchase_req_id = 0 ...
  // thus, it doesn't matter if we grab this before or after. ha!
  vector<map<string, string> > about_updated_bills;
  if (!SqlToVecMap(dbTransaction, "select * from bills where id in (" + VecToCommaString(chosen_bill_id) + ") ", about_updated_bills))
   throw(6);

  string update_sql = "update bills set purchase_req_id = '" + new_purchase_req["id"] + "' where id in (" + VecToCommaString(chosen_bill_id) + ") ";

  AtlasAdd("bills", about_updated_bills);
  if (!ExecuteStatement(dbTransaction, update_sql))
   throw(7);

  dbTransaction->Commit();
  delete dbTransaction;
  AtlasSend();

  // print!print!print!
  // oh, wait, is that fixed now? hehehehe ...
  // (in case the joke is lost on you, Matt's old code would print three copies of something, immediately, without warning, and sometimes the wrong stuff ... it was great.)
  FORM_TRY
   TformLaunchPRReport * print_pr = new TformLaunchPRReport(NULL, new_purchase_req["id"]);
   print_pr->Show();
  FORM_CATCH

  return true;
 }
 catch (...)
 {
  PSay("Couldn't save a new purchase request; perhaps some of the selected bills were already locked by another user.", m_database_error, MB_OK);
  dbTransaction->Rollback();
  delete dbTransaction;
  AtlasClear();
  return false;
 }
}
//---------------------------------------------------------------------------
void TformWizardNewPurchaseRequest::Do()
{
 FORM_TRY
  TformWizardNewPurchaseRequest * z = new TformWizardNewPurchaseRequest(NULL);
  z->Show();
 FORM_CATCH
}
//---------------------------------------------------------------------------

