//---------------------------------------------------------------------------

#include <vcl.h>
#pragma hdrstop

#include "codeWizardNewReferral.h"
#include "codeRecordReferral.h"
#include "codeSpecialNewInterofficeReferral.h"
#include "codeSearchPerson.h"
#include "codeSearchTaskBeh.h"
#include "codeSearchTaskCare.h"
#include "codeSearchTaskCobVisit.h"
#include "codeSearchTaskCom.h"
#include "codeSearchTaskDiab.h"
#include "codeSearchTaskLac.h"
#include "codeSearchTaskTrans.h"
#include "codeSearchTaskPub.h"
#include "codeSpecialDisambiguate.h"
//---------------------------------------------------------------------------
#pragma package(smart_init)
#pragma link "codeWizardGeneric"
#pragma resource "*.dfm"
TformWizardNewReferral *formWizardNewReferral;
//---------------------------------------------------------------------------
__fastcall TformWizardNewReferral::TformWizardNewReferral(TComponent* Owner)
        : TformWizardGeneric(Owner), chosen_referral_type(0)
{
  Initialize();
}
//---------------------------------------------------------------------------
bool TformWizardNewReferral::IsValidPage(int page_index)
{
  bool success = false;

  switch(page_index)
  {
  case 0:
    // Nothing to validate on this page.
    success = true;
    break;
  case 1:
    if(IsBlankOrZero(GetValidLinkId(txtPerson)) && IsBlankOrZero(GetValidLinkId(txtVisit)))
    {
      PSay("You must select either a person or a visit.", m_missing_data, MB_OK);
      success = false;
    }
    else
      success = true;
    break;
  default:
    PSay("IsValidPage() is trying to access a page that does not exist.", m_program_error, MB_OK);
    break;
  }

  return success;
}
//---------------------------------------------------------------------------
bool TformWizardNewReferral::ProcessPage(int page_index)
{
  bool success = false;

  switch(page_index)
  {
  case 0:
    chosen_referral_type = radioGroupType->ItemIndex;
    success = true;
    break;
  case 1:
    chosen_person_id = GetValidLinkId(txtPerson);
    chosen_visit_id  = GetValidLinkId(txtVisit);
    success = true;
    break;
  default:
    PSay("ProcessPage() is trying to access a page that does not exist.", m_program_error, MB_OK);
    break;
  }

  return success;
}
//---------------------------------------------------------------------------
void TformWizardNewReferral::SetupPage(int page_index)
{
  switch(page_index)
  {
  case 0:
    radioGroupType->ItemIndex = chosen_referral_type;
    break;
  case 1:
    CreateLink(txtPerson, btnPersonSearch, this, chosen_person_id, "people");
    CreateLink(txtVisit,  btnVisitSearch,  this, chosen_visit_id,  "tasks");
    break;
  default:
    PSay("SetupPage() is trying to access a page that does not exist.", m_program_error, MB_OK);
    break;
  }
}
//---------------------------------------------------------------------------
void TformWizardNewReferral::PageLockStatus(int page_index, bool locked)
{
  switch(page_index)
  {
  case 0:
    radioGroupType->Enabled = !locked;
    return;
  case 1:
    btnPersonSearch->Enabled = !locked;
    btnVisitSearch->Enabled  = !locked;
    return;
  default:
    PSay("PageLockStatus() is trying to access a page that does not exist.", m_program_error, MB_OK);
    return;
  }
}
//---------------------------------------------------------------------------
bool TformWizardNewReferral::FinishWizard()
{
  pmapstrs params;
  TformRecordReferral                *normal_referral_form = NULL;
  TformSpecialNewInterofficeReferral *io_referral_form = NULL;

  switch(chosen_referral_type)
  {
  case 0:
    params["id"] = "new";
    if(!IsBlankOrZero(chosen_person_id))
      params["person_id"] = chosen_person_id;
    if(!IsBlankOrZero(chosen_visit_id))
      params["task_id"] = chosen_visit_id;
    params["by_provider_id"] = GetOneField(dbDatabase, "select tax_entity_id from users where id = " + current_userid);
    params["todo_status_id"] = lookupTables->IdOf("todo_status_types", "Not Begun");
    FORM_TRY
      normal_referral_form = new TformRecordReferral(NULL, params);
    FORM_CATCH_RETURN("TformRecordReferral", false)
    normal_referral_form->editDate->Text = DateToStr(Date()).c_str();
    normal_referral_form->ShowTaskModal(this);
    break;
  case 1:
    params["id"] = "new";
    if(!IsBlankOrZero(chosen_person_id))
      params["person_id"] = chosen_person_id;
    if(!IsBlankOrZero(chosen_visit_id))
      params["task_id"] = chosen_visit_id;
    params["by_provider_id"]     = GetOneField(dbDatabase, "select tax_entity_id from users where id = " + current_userid);
    params["referral_reason_id"] = lookupTables->IdOf("referral_reason_types", "Non-AST Referral");
    params["todo_status_id"]     = lookupTables->IdOf("todo_status_types", "Not Begun");
    FORM_TRY
      normal_referral_form = new TformRecordReferral(NULL, params);
    FORM_CATCH_RETURN("TformRecordReferral", false)
    normal_referral_form->editDate->Text = DateToStr(Date()).c_str();
    normal_referral_form->cboReason->Enabled = false;
    Processing(false);
    normal_referral_form->ShowTaskModal(this);
    break;
  case 2:
    FORM_TRY
      io_referral_form = new TformSpecialNewInterofficeReferral(NULL, chosen_person_id);
    FORM_CATCH_RETURN("TformSpecialNewInterofficeReferral", false)
    io_referral_form->btnPersonSearch->Enabled = false;
    Processing(false);
    io_referral_form->ShowTaskModal(this);
    break;
  }

  return true;
}
//---------------------------------------------------------------------------
void TformWizardNewReferral::Do()
{
  TformWizardNewReferral *z = NULL;
  FORM_TRY
    z = new TformWizardNewReferral(NULL);
  FORM_CATCH_RETURN("TformWizardNewReferral", )
  z->Show();
}
//---------------------------------------------------------------------------
void __fastcall TformWizardNewReferral::btnPersonSearchClick(TObject *Sender)
{
  vector<string> return_vec = TformSearchPerson::Do(this);

  if(!return_vec.empty())
  {
    CreateLink(txtPerson, btnPersonSearch, this, return_vec[0], "people");
    CreateLink(txtVisit,  btnVisitSearch,  this, "0",           "tasks");  // clear visit link
  }
}
//---------------------------------------------------------------------------
void __fastcall TformWizardNewReferral::btnVisitSearchClick(TObject *Sender)
{
  map<string, string> params;
  string group_id;

  params["prefix"] = "table.tasks";
  params["suffix"] = "";
  params["key"]    = "view";
  params["value"]  = "true";
  params["instructions"] = "You have permission to search for visits created in multiple departments.  "
                           "Please select a department within which you wish to find a visit.";
  set<string> group_id_set;
  string group_id_sql = "select groups.id from groups inner join users on users.group_id = groups.id "
                        "where groups.id > 0 and groups.valid = 1 and users.valid = 1 group by groups.id ";
  SqlToSet(dbDatabase, group_id_sql, group_id_set);
  params["include"] = SetToCommaString(group_id_set);
  params["exclude"] = "";
  params["table"]   = "groups";
  params["sql"]     = "select id row_id, short_item row_value, item show_value from groups where valid = 1 and id > 0 ";

  group_id = TformSpecialDisambiguate::Disambiguate(this, params, PrettyString);

  // Here is why I should have finished that combined task search form already...
  if(!IsBlankOrZero(group_id))
  {
    vector<string> return_vec;

    if(group_id == lookupTables->IdOf("groups", "Behavioral Health"))
    {
      return_vec = TformSearchTaskBeh::Do(this);
    }
    else if(group_id == lookupTables->IdOf("groups", "Caregiver Support Program"))
    {
      return_vec = TformSearchTaskCare::Do(this);
    }
    else if(group_id == lookupTables->IdOf("groups", "Community Health"))
    {
      return_vec = TformSearchTaskCom::Do(this);
    }
    else if(group_id == lookupTables->IdOf("groups", "Contract Health"))
    {
      return_vec = TformSearchTaskCobVisit::Do(this);
    }
    else if(group_id == lookupTables->IdOf("groups", "Diabetes"))
    {
      return_vec = TformSearchTaskDiab::Do(this);
    }
    else if(group_id == lookupTables->IdOf("groups", "Little Axe Clinic") ||
            group_id == lookupTables->IdOf("groups", "Shawnee Clinic")    ||
            group_id == lookupTables->IdOf("groups", "Dental Clinic"))
    {
      return_vec = TformSearchTaskLac::Do(this);
    }
    else if(group_id == lookupTables->IdOf("groups", "Public Health"))
    {
      return_vec = TformSearchTaskPub::Do(this);
    }
    else if(group_id == lookupTables->IdOf("groups", "Transportation"))
    {
      return_vec = TformSearchTaskTrans::Do(this);
    }
    if(!return_vec.empty())
    {
      CreateLink(txtVisit, btnVisitSearch, this, return_vec[0], "tasks");

      string person_id_sql = "select person_id from tasks where id = '" + return_vec[0] + "' ";
      string person_id = GetOneField(dbDatabase, person_id_sql);
      CreateLink(txtPerson, btnPersonSearch, this, person_id, "people");
    }
  }
}
//---------------------------------------------------------------------------

