//---------------------------------------------------------------------------

#include <vcl.h>
#pragma hdrstop

#include "codeWizardObservationOptionSet.h"
#include "control_break_objects.h"
//---------------------------------------------------------------------------
#pragma package(smart_init)
#pragma link "codeWizardGeneric"
#pragma resource "*.dfm"
TformWizardObservationOptionSet *formWizardObservationOptionSet;
//---------------------------------------------------------------------------
__fastcall TformWizardObservationOptionSet::TformWizardObservationOptionSet(TComponent* Owner)
        : TformWizardGeneric(Owner)
{
 lookupTables->RegisterComboBox("obs_option_set_types", comboOptionSet);
 start_page = 0;

 list_questions.SetListViewHandle(listQuestions);
 list_questions.SetSecurity("obs_questions");
 list_questions.AddColumnSpec("Internal Id", "internal_id", as_int, false, 80);
 list_questions.AddColumnSpec("Current Id", "current_id", as_int, false, 80);
 list_questions.AddColumnSpec("Valid", "people", as_text, false, 150, prettyBoolString);
 list_questions.AddColumnSpec("Name", "people", as_text, true, 180, rawString);
 list_questions.AddColumnSpec("Abbrev.", "affiliations", as_text, true, 70, rawString);
 list_questions.AddColumnSpec("CPT", "affiliations", as_text, false, 160, rawString);
 list_questions.AddColumnSpec("Type", "tasks", as_text, true, 90, prettyString);
 list_questions.AddColumnSpec("Options", "tasks", as_text, false, 90, prettyString);
 list_questions.SetMulti(true);
 list_questions.FetchPreferences("wizardobservationoptionset.list_questions");
 // list_questions.SetAddEdit(this, AddRecord(&(TformWizardObservationOptionSet::AddQuestion)), EditRecord(&(TformWizardObservationOptionSet::EditQuestion)));
 // request_questions = atlas->RequestNotificationWhen("obs_questions.id", "", rel_any, as_int, this, RefreshHandler(&(TformWizardObservationOptionSet::AtlasRefreshListView)));
 comboOptionSetChange(comboOptionSet);

 list_options.setup_listview(listOptions, false);
 list_options.add_or_modify_column("option_rank", "Rank",   true, 0,  60,  as_int, rawString);
 list_options.add_or_modify_column("option_item", "Option", true, 1, 120, as_text, rawString);
 list_options.setup_callbacks((TForm*)this, InformForm(&(TformWizardObservationOptionSet::OptionSelected)), NULL);

 list_conversions.setup_listview(listConversions, false);
 list_conversions.add_or_modify_column("option_rank", "Old Rank", true, 0, 60, as_int, rawString);
 list_conversions.add_or_modify_column("old_item", "Old", true, 1, 80, as_text, rawString);
 list_conversions.add_or_modify_column("new_item", "New", true, 2, 80, as_text, rawString);
 list_conversions.setup_callbacks((TForm*)this, InformForm(&(TformWizardObservationOptionSet::ConversionSelected)), NULL);

 Initialize();
}
//---------------------------------------------------------------------------
__fastcall TformWizardObservationOptionSet::~TformWizardObservationOptionSet()
{
 lookupTables->UnregisterComboBox(comboOptionSet);
}
//---------------------------------------------------------------------------
void __fastcall TformWizardObservationOptionSet::buttonOptionAddClick(TObject *Sender)
{
 // add
 listOptions->Selected = NULL;
}
//---------------------------------------------------------------------------
void __fastcall TformWizardObservationOptionSet::buttonOptionRemoveClick(
      TObject *Sender)
{
 // remove
 list_options.remove_current_row(true);
 // remove any gaps created, including at bottom
 NormalizeOptions ();
}
//---------------------------------------------------------------------------
void __fastcall TformWizardObservationOptionSet::buttonOptionClearClick(
      TObject *Sender)
{
 // clear
 list_options.clear_data();
}
//---------------------------------------------------------------------------
void __fastcall TformWizardObservationOptionSet::buttonOptionSaveClick(TObject *Sender)
{
 // save
 // if new row, always add to end (high rank)
 // if not new row, save the rank from the current row! (there's no edit box for it, mind, nor even a variable!)
 // let them move it around themselves.

 if (editOptionItem->Text == "")
 {
  PSay("Please enter text to describe the option.", m_missing_data, MB_OK);
  return;
 }

 map<string, string> row_data;
 if (list_options.get_current_row(row_data))
 {
  // modifying a row
  // - keep rank
  // - modify item
  row_data["option_item"] = editOptionItem->Text.c_str();
 } else
 {
  // inserting a row
  // - set rank to highest value
  // - set item
  row_data["option_item"] = editOptionItem->Text.c_str();

  vector<map<string, string> > all_options;
  list_options.export_data(all_options);
  int lowest_number = 0;
  int temp_number = 0;
  for (vector<map<string, string> >::iterator i = all_options.begin(); i != all_options.end(); i++)
  {
   if (IsInt((*i)["option_rank"]))
   {
    temp_number = StringToInt((*i)["option_rank"]);
    lowest_number = max(lowest_number, temp_number);
   }
  }
  lowest_number++;
  row_data["option_rank"] = IntToString(lowest_number);
 }

 set<string> dupe_columns;
 dupe_columns.insert("option_item");
 if (list_options.is_future_duplicate (row_data, dupe_columns))
 {
  PSay("Saving this row as it is would create a duplicate entry in the list.\nYou may not have two rows with the same option text.", m_duplicate_data, MB_OK);
  return;
 }

 list_options.add_or_modify_row_data(row_data, false);
 listOptions->Selected = NULL;

 NormalizeOptions ();

 TWinControl * wanted_box = editOptionItem;
 if (wanted_box && wanted_box->Enabled && IsVisible(wanted_box))
  this->ActiveControl = wanted_box;
}
//---------------------------------------------------------------------------
void __fastcall TformWizardObservationOptionSet::buttonOptionUpClick(TObject *Sender)
{
 list_options.decrease_rank("option_rank");
}
//---------------------------------------------------------------------------
void __fastcall TformWizardObservationOptionSet::buttonOptionDownClick(TObject *Sender)
{
 list_options.increase_rank ("option_rank");
}
//---------------------------------------------------------------------------
void TformWizardObservationOptionSet::OptionSelected ()
{
 bool row_selected = true;
 map<string, string> row_data;
 if (!list_options.get_current_row(row_data))
 {
  // set default values
  row_data["option_rank"] = "";
  row_data["option_item"] = "";
  row_selected = false;
 }

 buttonOptionRemove->Enabled = row_selected; // i don't usually bother, but it wouldn't hurt to do this elsewhere?
 buttonOptionUp->Enabled     = row_selected;
 buttonOptionDown->Enabled   = row_selected;

 // update controls to reflect data in row_data
 // editOptionRank->Text = row_data["option_rank"].c_str();
 editOptionItem->Text = row_data["option_item"].c_str();

 // TWinControl * wanted_box = editOptionItem;
 // if (wanted_box && wanted_box->Enabled && IsVisible(wanted_box))
 //  this->ActiveControl = wanted_box;
}
//---------------------------------------------------------------------------
void TformWizardObservationOptionSet::ConversionSelected ()
{
 bool row_selected = true;
 map<string, string> row_data;
 if (!list_conversions.get_current_row(row_data))
 {
  // set default values
  row_data["old_item"] = "";
  row_data["new_item"] = "";
  row_selected = false;
 }

 buttonConversionSave->Enabled = row_selected;
 comboOption->Enabled          = row_selected;

 // update controls to reflect data in row_data
 comboOption->Text = row_data["new_item"].c_str();

 // TWinControl * wanted_box = comboOption;
 // if (wanted_box && wanted_box->Enabled && IsVisible(wanted_box))
 //  this->ActiveControl = wanted_box;
}
//---------------------------------------------------------------------------
void __fastcall TformWizardObservationOptionSet::buttonConversionSaveClick(
      TObject *Sender)
{
 // save conversion
 if (listConversions->Selected)
 {
  map<string, string> row_data;
  list_conversions.get_current_row(row_data);
  row_data["new_item"] = comboOption->Text.c_str();

  set<string> dupe_columns;
  dupe_columns.insert("old_item");
  if (list_conversions.is_future_duplicate (row_data, dupe_columns))
  {
   PSay("You may not convert an option to two different options.", m_duplicate_data, MB_OK);
   return;
  }

  list_conversions.add_or_modify_row_data(row_data, false);
  listConversions->Selected = NULL;

  TWinControl * wanted_box = comboOption;
  if (wanted_box && wanted_box->Enabled && IsVisible(wanted_box))
   this->ActiveControl = wanted_box;
 } else
  PSay("No new rows can be added to the conversion list.", m_program_error, MB_OK);
}
//---------------------------------------------------------------------------
bool TformWizardObservationOptionSet::SkipPage (int page_index)
{
 return (radioCreate->Checked && page_index == 2);
}
//---------------------------------------------------------------------------
bool TformWizardObservationOptionSet::IsValidPage (int page_index)
{
 switch (page_index)
 {
  case 0:
   if (radioModify->Checked && (comboOptionSet->Text == "" || !lookupTables->IsInLookUpTable("obs_option_set_types", comboOptionSet)))
   {
    PSay("Please select an option-set to modify.", m_missing_data, MB_OK);
    comboOptionSet->SetFocus();
    return false;
   }
   if (radioCreate->Checked && editOptionSetItem->Text == "")
   {
    PSay("Please enter a name for the new option-set.", m_missing_data, MB_OK);
    editOptionSetItem->SetFocus();
    return false;
   }
   if (radioCreate->Checked)
   {
    string sql = "select count(*) from obs_option_set_types where upper(item) = '" + SafeSQLString(editOptionSetItem->Text.c_str()) + "'";
    string c = GetOneField(dbDatabase, sql);
    if (StringToInt(c) > 0)
    {
     PSay("An option-set already exists with that name, or a very similar name (case-insensitive.)\nPlease enter a different name for the new option-set.", m_duplicate_data, MB_OK);
     return false;
    }
   }
   return true;
  case 1:
   return true;
  case 2:
  {
   // please make sure all old values have -some- equivalent in the new set?
   vector<map<string, string> > conversion_data;
   list_conversions.export_data(conversion_data);
   for (vector<map<string, string> >::iterator i = conversion_data.begin(); i != conversion_data.end(); i++)
    if ((*i)["new_item"] == "")
    {
     PSay("Please enter conversions for every old option.\n\nIf doing so would seem 'wrong', then it probably is. Stop and read the following:\n\nIf a semi-quantitative test is being done with new instruments/strips,\nand the result values have changed, you should:\n- create a new option-set,\n- create a new question, using the new option-set,\n- add the new question to question-sets,\n- remove the old question question-sets.\n\nThis will not disrupt old data, and will allow you to enter new data correctly.\nThe results of old and new questions will be difficult to compare, but that's to be expected.", m_missing_data, MB_OK);
     return false;
    }
   return true;
  }
 }
 return false;
}
//---------------------------------------------------------------------------
int TformWizardObservationOptionSet::WhereNext (int page_index)
{
 switch (page_index)
 {
  case 0: return 1;
  case 1: return 2;
  case 2: return TformWizardGeneric::last_wizard_page;
 }
 return TformWizardGeneric::last_wizard_page;
}
//---------------------------------------------------------------------------
bool TformWizardObservationOptionSet::ProcessPage (int page_index)
{
 switch (page_index)
 {
  case 0: // select option set
   chosen_obs_option_set_id = "0";
   if (comboOptionSet->Text != "" && lookupTables->IsInLookUpTable("obs_option_set_types", comboOptionSet))
    chosen_obs_option_set_id = lookupTables->IdOf("obs_option_set_types", comboOptionSet);
   return true;
  case 1:
  case 2:
   // nothing to do? (it's all in the edit_listviews)
   return true;
 }
 return false;
}
//---------------------------------------------------------------------------
void TformWizardObservationOptionSet::SetupPage (int page_index)
{
 switch (page_index)
 {
  case 0: // select option set
  {
   radioModify->Checked = true;
   radioModifyClick(radioModify);
   comboOptionSet->Text = "";
   editOptionSetItem->Text = "";
   return;
  }
  case 1: // modify option set
  {
   list_options.clear_data();

   if (radioModify->Checked)
   {
    vector<map<string, string> > temp_data;
    string sql = "select rank option_rank, item option_item from obs_option_types where obs_option_types.obs_option_set_id = " + chosen_obs_option_set_id;
    if (!SqlToVecMap(dbDatabase, sql, temp_data))
     PSay("Failed to retrieve the list of options currently in this option-set.", m_database_error, MB_OK);
    list_options.import_data(temp_data);
   }
   OptionSelected();
   return;
  }
  case 2: // optionally convert old options to new options
  {
   // need to match up those that are the same by "item"
   // need to put all new items into the custom drop-down

   list_conversions.clear_data();

   if (radioModify->Checked)
   {
    vector<map<string, string> > original_data;
    vector<map<string, string > > updated_data;

    // original
    string sql = "select rank option_rank, item option_item from obs_option_types where obs_option_types.obs_option_set_id = " + chosen_obs_option_set_id;
    if (!SqlToVecMap(dbDatabase, sql, original_data))
     PSay("Failed to retrieve the list of options currently in this option-set.", m_database_error, MB_OK);

    // update
    list_options.export_data(updated_data);

    // left join ...
    // ENTIRELY OVERKILL.
    // KEEP:
    // might make for fine code if there were more columns involved. consider making a 'join' function later.
    /*
    vector<pair<string, cb_sorting_object<string> *> > sorting_methods;
    sorting_methods.push_back(pair<string, cb_sorting_object<string> *>("option_item", new sort_less_than<string>()));
    multi_column_sorting_object<string, string> sorting_object (sorting_methods);
    sort(original_data.begin(), original_data.end(), sorting_object);
    sort(updated_data.begin(), updated_data.end(), sorting_object);
    cb_deallocate_sorting_map(sorting_methods);

    vector<map<string, string> >::iterator i_updated = updated_data.begin();
    for (vector<map<string, string> >::iterator i_original = original_data.begin(); i_original != original_data.end(); i_original++)
    {
     map<string, string> temp;
     temp["old_item"] = (*i_original)["option_item"];
     while ((*i_updated)["option_item"] < (*i_original)["option_item"] && i_updated != updated_data.end())
      i_updated++;
     if ((*i_updated)["option_item"] == (*i_original)["option_item"])
      temp["new_item"] = (*i_updated)["option_item"]; // duh!
     else
      temp["new_item"] = ""; // not found
    }
    */

    // in order by rank, to make them more obvious in the drop-down?
    vector<pair<string, cb_sorting_object<string> *> > sorting_methods;
    sorting_methods.push_back(pair<string, cb_sorting_object<string> *>("option_rank", new sort_callback_wrapper<string, int, sort_less_than<int> >(StringToInt)));
    multi_column_sorting_object<string, string> sorting_object (sorting_methods);
    sort(updated_data.begin(), updated_data.end(), sorting_object);
    cb_deallocate_sorting_map(sorting_methods);

    set<string> available_new_items;
    comboOption->Items->Clear();
    for (vector<map<string, string> >::iterator i_up = updated_data.begin(); i_up != updated_data.end(); i_up++)
    {
     available_new_items.insert((*i_up)["option_item"]);
     comboOption->Items->Add((*i_up)["option_item"].c_str());
    }

    for (vector<map<string, string> >::iterator i_or = original_data.begin(); i_or != original_data.end(); i_or++)
    {
     // drop a column (or not)
     // (*i_or).erase("option_rank");
     // rename a column
     (*i_or)["old_item"] = (*i_or)["option_item"];
     (*i_or).erase("option_item");
     // "where exists"
     if (available_new_items.find((*i_or)["old_item"]) != available_new_items.end())
      (*i_or)["new_item"] = (*i_or)["old_item"];
    }

    // conversion!
    list_conversions.import_data(original_data);
   }
   ConversionSelected();

   return;
  }
 }
}
//---------------------------------------------------------------------------
bool TformWizardObservationOptionSet::FinishWizard ()
{
 bool answer = false;
 AtlasClear();

 TIBTransaction * dbTransaction = NULL;
 FORM_TRY
  dbTransaction = new TIBTransaction(NULL);
 FORM_CATCH

 try
 {
  StdTransaction(dbTransaction, dbDatabase);

  // track id/item for old and new
  string option_set_id;
  map<string, string> old_item_to_id;
  map<string, string> new_item_to_id;
  map<string, string> new_item_to_rank;
  option_set_id = chosen_obs_option_set_id;

  // ... for old ...
  if (radioModify->Checked)
  {
   string sql = "select item, id from obs_option_types where obs_option_types.obs_option_set_id = " + chosen_obs_option_set_id;
   if (!SqlToValueMap(dbDatabase, sql, old_item_to_id))
   {
    PSay("Failed to retrieve the list of options currently in this option-set.", m_database_error, MB_OK);
    throw (1);
   }
  }

  // optionally, create option-set
  if (radioCreate->Checked)
  {
   map<string, string> to_insert;
   to_insert["item"] = editOptionSetItem->Text.c_str();
   to_insert["valid"] = "1";
   map<string, int> leave_alone;
   for (map<string, string>::iterator itr=to_insert.begin(); itr!=to_insert.end(); itr++)
    leave_alone[itr->first] = sql_no_uppercasing;
   if (!ExecuteInsertStatement(dbTransaction, "obs_option_set_types", to_insert, leave_alone))
   {
    PSay("Failed to create new option-set.", m_database_error, MB_OK);
    throw (2);
   }
   option_set_id = GetOneField(dbTransaction, "select max(id) from obs_option_set_types"); // lookuptableclass does this too, btw.
   to_insert["id"] = option_set_id;
   AtlasAdd("obs_option_set_types", to_insert);
   AtlasAdd("obs_option_set_types", ""); // lookuptables refresh on "$table_name|anything" -- had to add a function just for this!
  }

  // optionally, create new options
  vector<map<string, string> > new_options;
  list_options.export_data(new_options);
  for(vector<map<string, string> >::iterator i = new_options.begin(); i != new_options.end(); i++)
  {
   new_item_to_id[(*i)["option_item"]] = old_item_to_id[(*i)["option_item"]];
   new_item_to_rank[(*i)["option_item"]] = (*i)["option_rank"];
  }
  for (map<string, string>::iterator i = new_item_to_id.begin(); i != new_item_to_id.end(); i++)
  {
   if (i->second.empty())
   {
    map<string, string> to_insert;
    to_insert["item"] = i->first;
    to_insert["obs_option_set_id"] = option_set_id;
    to_insert["rank"] = new_item_to_rank[i->first];
    to_insert["valid"] = "1";
    map<string, int> leave_alone;
    for (map<string, string>::iterator itr=to_insert.begin(); itr!=to_insert.end(); itr++)
     leave_alone[itr->first] = sql_no_uppercasing;
    if (!ExecuteInsertStatement(dbTransaction, "obs_option_types", to_insert, leave_alone))
    {
     PSay("Failed to create new option.", m_database_error, MB_OK);
     throw (3);
    }
    i->second = GetOneField(dbTransaction, "select max(id) from obs_option_types"); // lookuptableclass does this too, btw.
    to_insert["id"] = i->second;
    AtlasAdd("obs_option_types", to_insert);
   }
  }

  // optionally, convert old to new
  vector<map<string, string> > conversion_rows;
  list_conversions.export_data(conversion_rows);
  for(vector<map<string, string> >::iterator i = conversion_rows.begin(); i != conversion_rows.end(); i++)
  {
   string old_id = old_item_to_id[(*i)["old_item"]];
   string new_id = new_item_to_id[(*i)["new_item"]];
   if (old_id != new_id)
   {
    if (!IsBlankOrZero(old_id) && !IsBlankOrZero(new_id))
    {
     string sql = "update obs_responses set obs_option_id = " + new_id + " where obs_option_id = " + old_id;
     if (!ExecuteStatement(dbTransaction, sql))
     {
      PSay("Failed to convert old options to new options.", m_database_error, MB_OK, "old = " + old_id + ", new = " + new_id + ", new.item was " + (*i)["new_item"]);
      throw (4);
     }
     // not terribly useful notifications.
     AtlasAdd("obs_responses", "obs_option_id", old_id);
     AtlasAdd("obs_responses", "obs_option_id", new_id);
    } else
    {
     PSay("Bad option conversion: the two id's are not both valid numerics!", m_program_error, MB_OK, "old = " + old_id + ", new = " + new_id);
     throw (5);
    }
   }
  }

  // optionally, delete old options
  for (map<string, string>::iterator i = old_item_to_id.begin(); i != old_item_to_id.end(); i++)
  {
   if (new_item_to_id.find(i->first) == new_item_to_id.end())
   {
    string sql = "delete from obs_option_types where id = " + i->second;
    if (!ExecuteStatement(dbTransaction, sql))
    {
     PSay("Failed to delete an obsolete option.", m_database_error, MB_OK, "id = " + i->second + ", item = " + i->first);
     throw (6);
    }
    AtlasAdd("obs_option_types", "id", i->second);
   }
  }

  // less optionally, make sure 'rank' is set properly
  for (map<string, string>::iterator i = new_item_to_rank.begin(); i != new_item_to_rank.end(); i++)
  {
   string sql = "update obs_option_types set rank = " + i->second + " where id = " + new_item_to_id[i->first];
   if (!ExecuteStatement(dbTransaction, sql))
   {
    PSay("Failed to update the ordering of option sets.", m_database_error, MB_OK, sql);
    throw(7);
   }
  }

  // done!
  dbTransaction->Commit();
  AtlasSend();
  answer = true;
 }
 catch (...)
 {
  if (dbTransaction->Active)
   dbTransaction->Rollback();
 }
 delete dbTransaction;

 return answer;
}
//---------------------------------------------------------------------------
void TformWizardObservationOptionSet::PageLockStatus (int page_index, bool locked)
{
 switch (page_index)
 {
  case 0: // select option set
  {
   radioModify->Enabled       = !locked; // permissions? (can't edit, can't create?)
   comboOptionSet->Enabled    = !locked;
   radioCreate->Enabled       = !locked; // should check for permissions? (can't create)
   editOptionSetItem->Enabled = !locked;
   return;
  }
  case 1: // modify option set
  {
   editOptionItem->Enabled     = !locked;
   buttonOptionAdd->Enabled    = !locked;
   buttonOptionRemove->Enabled = !locked;
   buttonOptionClear->Enabled  = !locked;
   buttonOptionUp->Enabled     = !locked;
   buttonOptionDown->Enabled   = !locked;
   buttonOptionSave->Enabled   = !locked;
   buttonOptionAdd->Enabled    = !locked;
   return;
  }
  case 2: // optionally convert old options to new options
  {
   comboOption->Enabled          = !locked;
   buttonConversionSave->Enabled = !locked;
   return;
  }
 }
}
//---------------------------------------------------------------------------
void TformWizardObservationOptionSet::Do()
{
 FORM_TRY
  TformWizardObservationOptionSet * z = new TformWizardObservationOptionSet(NULL);
  z->Show();
  z->radioCreate->Checked = true;
 FORM_CATCH
}
//---------------------------------------------------------------------------
void TformWizardObservationOptionSet::Do(string obs_option_set_id)
{
 FORM_TRY
  TformWizardObservationOptionSet * z = new TformWizardObservationOptionSet(NULL);
  z->Show();
  z->radioModify->Checked = true;
  z->comboOptionSet->Text = lookupTables->ValueOf("obs_option_set_types", obs_option_set_id).c_str();
 FORM_CATCH
}
//---------------------------------------------------------------------------
void __fastcall TformWizardObservationOptionSet::radioModifyClick(TObject *Sender)
{
 comboOptionSet->Enabled = radioModify->Checked;
 editOptionSetItem->Enabled = radioCreate->Checked;
/*
 if (comboOptionSet->Enabled)
  comboOptionSet->SetFocus();
 if (editOptionSetItem->Enabled)
  editOptionSetItem->SetFocus();
*/
}
//---------------------------------------------------------------------------
void __fastcall TformWizardObservationOptionSet::comboOptionKeyPress(TObject *Sender, char &Key)
{
 FindComboBoxItemAndSetItemIndex(Sender, Key);
}
//---------------------------------------------------------------------------
void TformWizardObservationOptionSet::NormalizeOptions ()
{
 list_options.normalize_rank("option_rank");
}
//---------------------------------------------------------------------------
void __fastcall TformWizardObservationOptionSet::comboOptionSetChange(TObject *Sender)
{
 string sql;
 if (lookupTables->IsInLookUpTable("obs_option_set_types", comboOptionSet))
  sql =
   "select obs_questions.id, obs_questions.current_id, obs_questions.valid, "
   "obs_questions.item, obs_questions.short_item, obs_questions.cpt_code, "
   "obs_question_types.item question_type_item, obs_option_set_types.item option_type_item "
   "from obs_questions "
   "left join obs_question_types on obs_question_types.id = obs_questions.obs_question_id "
   "left join obs_option_set_types on obs_option_set_types.id = obs_questions.obs_option_set_id "
   "where obs_questions.valid = 1 and obs_questions.obs_question_id in (1, 3) and obs_questions.obs_option_set_id = " + lookupTables->IdOf("obs_option_set_types", comboOptionSet);
 list_questions.SetSQLStatement(sql);
 list_questions.RefreshDataSet();
}
//---------------------------------------------------------------------------
