#include "components.h"
#include "codeSpecialMessageBox.h"
#include "codeSpecialDisambiguate.h"
#include "derivedheaders.h"

extern map<TStaticText *, link_struct> registered_links;
extern map<TMenuItem *, TStaticText *> registered_link_buttons;

// storage space for static
TNotifyEvent funcLinkClick;
TNotifyEvent funcLinkMenuItemClick;

//---------------------------------------------------------------------------
bool DefaultTextAvailable(string table_name, string permission_name, int options)
{
 // cases where we know the text always looks the same
 if (table_name == "" ||
     table_name == "tasks" ||
     table_name == "bills" ||
     table_name == "referrals" ||
     table_name == "activity_logsheets" ||
     table_name == "bugs")
  return false;

 string result = DefaultText("1", table_name, permission_name, options);
 if (result == "Not a valid table name" ||
     result == "Restricted")
  return false;

 return true;
}
//---------------------------------------------------------------------------
string DefaultText(string id, string table_name, string permission_name, int options)
{
 string permission_string;
 string to_display = "";

 if(table_name == "people")
 {
   if (options == -1)
     to_display = GetPersonName(dbDatabase, id);
   else
     to_display = GetPersonName(dbDatabase, id, options);
 }
 else if(table_name == "participations")
 {
   to_display = GetOneField(dbDatabase, string("select event_name_types.item from events "
                "inner join event_name_types on events.event_name_id = event_name_types.id "
                "inner join participations on participations.event_id = events.id and participations.id = " + id));
 }
 else if (table_name == "events")
 {
   to_display = GetOneField(dbDatabase, string("select event_name_types.item from events "
                "inner join event_name_types on events.event_name_id = event_name_types.id where events.id = " + id));
 }
 else if(table_name == "tasks")
 {
   map<string, string> task_map;
   SqlToMap(dbDatabase, string("select group_id, date_of_task, services_provided_id, "
            "cob_number from tasks where id = " + id), task_map);
   if(task_map["group_id"] == lookupTables->IdOf("groups", "Diabetes") ||
      task_map["group_id"] == lookupTables->IdOf("groups", "Behavioral Health"))
     to_display = "Go to task/visit";
   else if(task_map["group_id"] == lookupTables->IdOf("groups", "Community Health")          ||
           task_map["group_id"] == lookupTables->IdOf("groups", "Public Health")             ||
           task_map["group_id"] == lookupTables->IdOf("groups", "Caregiver Support Program") ||
           task_map["group_id"] == lookupTables->IdOf("groups", "Little Axe Clinic")         ||
           task_map["group_id"] == lookupTables->IdOf("groups", "Shawnee Clinic")            ||
           task_map["group_id"] == lookupTables->IdOf("groups", "Dental Clinic"))
     to_display = "Go to visit";
   else if(task_map["group_id"] == lookupTables->IdOf("groups", "Transportation"))
     to_display = "Go to task";
   else if(task_map["group_id"] == lookupTables->IdOf("groups", "Contract Health"))
   {
     if((task_map["services_provided_id"] == "0") || (task_map["services_provided_id"] == ""))
       to_display = "Visit on " + task_map["date_of_task"];
     else
       to_display = task_map["cob_number"];
   }
 }
 else if(table_name == "tax_entities")
   to_display = GetOneField(dbDatabase, string("select tax_entities.name from tax_entities where tax_entities.id = " + id));
 else if(table_name == "fitness_memberships")
 {
   to_display = PrettyString(GetOneField(dbDatabase,
      string("select fit_facility_types.item from fit_facility_types, fitness_memberships where "
             "fit_facility_types.id = fitness_memberships.fit_facility_id and fitness_memberships.id = " + id)));
 }
 else if(table_name == "purchase_reqs")
   to_display = GetOneField(dbDatabase, string("select purchase_req_number from purchase_reqs where id = " + id));
 else if(table_name == "accounts")
 {
   to_display = PrettyString(GetOneField(dbDatabase, string("select accounts.name from accounts where accounts.id = " + id)));
 }
 else if(table_name == "bills")
   to_display = "Go to bill";
 else if(table_name == "insurance_policies")
 {
   to_display = GetOneField(dbDatabase, string("select insurance_policies.policy_number from "
                "insurance_policies where insurance_policies.id = " + id));
   // jhaley 20101208: It would be unusual, but my testing indicates far from impossible, for this to happen.
   if (to_display == "")
     to_display = "Go to policy";
 }
 else if(table_name == "insurance_coverage")
 {
   map<string, string> temp;
   SqlToMap (dbDatabase, "select insurance_policies.policy_number, insurance_coverage.person_code "
                         "from insurance_coverage "
                         "left join insurance_policies on insurance_policies.id = insurance_coverage.insurance_policy_id "
                         "where insurance_coverage.id = " + id, temp);
   if (temp["person_code"] == "")
    to_display = temp["policy_number"] + " : ??";
   else if (!IsInt(temp["person_code"]))
    to_display = temp["policy_number"] + " : " + temp["person_code"];
   else
    to_display = temp["policy_number"] + " : " + twodigit(StringToInt(temp["person_code"]));
 }
 else if(table_name == "checks")
   to_display = PrettyString(GetOneField(dbDatabase, string("select checks.check_number from checks where checks.id = " + id)));
 else if(table_name == "services_provided")
 {
   to_display = GetOneField(dbDatabase, string("select tax_entities.name || ' (' || "
               "service_types.item || ')' from services_provided, tax_entities, service_types "
               "where services_provided.tax_entity_id = tax_entities.id and "
               "services_provided.service_id = service_types.id and services_provided.id = " + id));
   size_t paren = to_display.find_last_of("(");
   if (paren != string::npos)
     to_display = to_display.substr(0, paren) + PrettyString(to_display.substr(paren));
 }
 else if(table_name == "referrals")
   to_display = "Go to referral";
 else if(table_name == "bugs")
   to_display = "Go to bug/feature request";
 else if(table_name == "diab_education")
 {
   to_display = PrettyString(GetOneField(dbDatabase,
        string("select diab_topic_types.item "
               "from diab_education, diab_topic_types "
               "where diab_education.diab_topic_id = diab_topic_types.id and "
               "diab_education.id = " + id)));
 }
 else if(table_name == "surveys")
   to_display = PrettyString(GetOneField(dbDatabase, "select survey_name from surveys where id = " + id));
 else if(table_name == "activity_logsheets")
   to_display = "Go to logsheet";
 else if(table_name == "procedures")
   to_display = "Procedure: " + GetOneField(dbDatabase, "select cpt_hcpcs from procedures where id = " + id);
 else if(table_name == "conditions")
   to_display = "Condition: " + GetOneField(dbDatabase, "select code from conditions where id = " + id);
 else if(table_name == "denials")
   to_display = "Go to denial batch";  // jhaley 20100702
 else if(table_name == "denial_letters")
   to_display = "Go to denial letter"; // jhaley 20100706
 else
   to_display = "Not a valid table name";

 // set up the permission to be checked
 if(permission_name != "")
   permission_string = permission_name;
 else
   permission_string = "table." + table_name;

 if(optionsPermissions[permission_string]["view"] != "true")
   if(to_display != "None")
     to_display = "Restricted";

 return to_display;
}
//---------------------------------------------------------------------------
set<string> GetPermissions(string permission_prefix, string permission_suffix, string key, string value, string table, string sql)
{
  // input:  table VALUES;  output:  table IDs corresponding to values they have matching permissions for
  set<string> results;
  vector<map<string, string> > table_vecmap;
  SqlToVecMap(dbDatabase, sql, table_vecmap);        // we expect an "row_id" and an "row_value" column, which can (probably will) be column aliases
  for(unsigned int i=0; i<table_vecmap.size(); i++)  // these names were picked as they don't conflict with any existing column names that I know of
  {
    string section = permission_prefix + "." + LowercaseString(table_vecmap[i]["row_value"]);
    if (permission_suffix!="") section += ("." + permission_suffix);
    // let's use find() and be careful accessing the map; we don't want to add random keys by accident, as the contents of
    // optionsPermissions are written to the DB as the current user's permission set, when they log out.
    if (optionsPermissions.find(section) != optionsPermissions.end() && optionsPermissions[section].find(key) != optionsPermissions[section].end())
      if(optionsPermissions[section][key] == value)
        results.insert(PrettyString(table_vecmap[i]["row_id"]));
  }

  // Special "extra" handling, depending on table

  if (table == "groups")
  {
    // There are generally two scenarios for handing "common" permissions--either there's an explicit permission they must have
    // named "common" (e.g. table.person_notes.common rather than table.person_notes.diab), or there's no "common" and it's just
    // generally assumed that everyone can see records with a group_id of 0.  So first look in optionsPermissions and see if there's
    // a "common" "department" (used loosely), and if there is, see if they have permission to use it.  If there isn't a common,
    // then automatically give them the ability to see group 0.
    string common = permission_prefix + ".common";
    if (permission_suffix!="") common += ("." + permission_suffix);
    if (optionsPermissions.find(common) != optionsPermissions.end())
    {
      if(optionsPermissions[common].find(key) != optionsPermissions[common].end())
        if(optionsPermissions[common][key] == value)
          results.insert("0");
    }
    else
      results.insert("0");
  }

  return results;
}
//---------------------------------------------------------------------------
set<string> GetGroupPermissions(string permission_prefix, string permission_suffix, string key, string value)
{
  return GetPermissions(permission_prefix, permission_suffix, key, value, "groups", "select id row_id, short_item row_value from groups where valid = 1 and id > 0");
}
//---------------------------------------------------------------------------
set<string> GetSpecialGroupPermissions(string permission_suffix)
{ // provided for compatability with previous code
  return GetGroupPermissions("special", permission_suffix, "enabled", "true");
}
//---------------------------------------------------------------------------
set<string> GetUsersWithPermissions(string section, string key, string value)
{
  set<string> temp;
  string sql = "select permissions.user_id from permissions where s like '" + UppercaseString(section) + "' "
               "and k = '" + key + "' and v = '" + value + "'";
  SqlToSet(dbDatabase, sql, temp);
  return temp;
}
//---------------------------------------------------------------------------
set<string> GetUsersWithPermissions(string section, string key)
{
  string k = (UppercaseString(section.substr(0, 6)) == "TABLE.") ? UppercaseString(key) : string("ENABLED");
  return GetUsersWithPermissions(section, k, "TRUE");
}
//---------------------------------------------------------------------------
set<string> GetUsersWithGroupPermissions(string prefix, string suffix, string group_id, string key)
{
  string grp = GetOneField(dbDatabase, "select short_item from groups where id = '" + group_id + "'");
  return GetUsersWithPermissions(prefix + "." + grp + "." + suffix, key);
}
//---------------------------------------------------------------------------
bool HasLocationPermissions(const set<string> &perm_group_ids, string location_name)
{
  bool result = false;
  //Application->MessageBox(VecToCommaString(perm_group_ids).c_str(), IntToString(perm_group_ids.size()).c_str(), MB_OK);
  // we receive a list of groups they have appropriate permissions to, by id; look up the values.
  set<string> temp_vec = lookupTables->ValuesOf ("groups", perm_group_ids, NULL);
  //Application->MessageBox(VecToCommaString(temp_vec).c_str(), IntToString(temp_vec.size()).c_str(), MB_OK);
  // drop the values into a set for easy searching
  //set<string> perm_group_values;
  //VecToSet(temp_vec, perm_group_values);
  // they're requesting authorization for a particular location--is it among the permissions they have?
  if (temp_vec.find(location_name) != temp_vec.end())
    result = true;
  return result;

/* must be true:
   * when a new department is added, it has a short_item, which has a corresponding special.X.general permission
     which is actually assigned to users as needed
*/
}
//---------------------------------------------------------------------------
void CreateLink(TStaticText * text_object, TControl * search_button,
                TForm * parent_form, string id, string table_name,
                string permission_name, bool is_enabled, int options, string alt_favorites)
{

  // set up properties about the text object
  text_object->Font->Color = link_color;
  text_object->Cursor = crHandPoint;
  text_object->Font->Style = TFontStyles() << fsUnderline;
  string to_display = "";
  string permission_string;
  bool register_ok = false;

  if ((IsInt(id)) && (id != ""))
  {
     // if the id is 0 set the caption to None
     // else select the text to be displayed
    if(id == "0")
      to_display = "None";
    else
    {
      to_display = DefaultText(id, table_name, permission_name, options);
      if (to_display == "Not a valid table name")
        text_object->Font->Color = clRed;
    }

    to_display = ReplaceStringWithString(to_display, "&", "&&");
    text_object->Caption = to_display.c_str();

    // set up the permission to be checked
    if(permission_name != "")
      permission_string = permission_name;
    else
      permission_string = "table." + table_name;

    // if the user has view privileges on this permission, show the link and set up the on click
    // else set the caption to Restricted if there is text to display
    if(optionsPermissions[permission_string]["view"] == "true")
    {
      if (is_enabled)
       text_object->Enabled = true;
      if(search_button)
        RecurseEnabled((TWinControl*)search_button, is_enabled);
      if((id != "0") && (! text_object->OnClick))
        text_object->OnClick = funcLinkClick;
    }
    else
    {
      text_object->Enabled = false;
      if(search_button)
        RecurseEnabled((TWinControl*)search_button, false);
    }
    register_ok = true;
  }
  else
    text_object->Caption = "None";

  if (search_button)
  {
    if (search_button->InheritsFrom(__classid(TToolBar)))
    { // toolbar, with a dropdown button containing favorites.  yay!
      // one caveat comes to mind ... if you're going to use toolbars
      // (with the advantage of the favorites dropdown), make sure the
      // toolbar button's OnClick is already set to whatever you want
      // the menu item to notify, when the link changes.
      bool insert_ok = false;
      TToolBar* T = NULL;
      try
      {
        T = (TToolBar*)search_button;
        if (T->ButtonCount > 0)
        {
          T->Images = imagesRecordTypes;
          T->Buttons[0]->ImageIndex = 55;
          if (DefaultTextAvailable(table_name))
          { // we should know better than to make a link where we don't have text to display, but we'll play it safe
            T->Buttons[0]->Style = tbsDropDown;
            T->Caption = table_name.c_str();
            T->Buttons[0]->Caption = "search";
            string string_favorites = (alt_favorites=="" ? optionsPreferences["favorites"][table_name] : alt_favorites);
            set<string> set_favorites;
            SplitStringToSet(string_favorites, "|", set_favorites);
            set_favorites.erase("");
            if (!set_favorites.empty())
            {
              if (!T->Buttons[0]->DropdownMenu)
                T->Buttons[0]->DropdownMenu = new TPopupMenu(parent_form);
              TPopupMenu* popup = T->Buttons[0]->DropdownMenu;
              //if (!popup->Items)
              //  popup->Items = new TMenuItem(popup); // apparently it does this automatically, as it's read-only
              TMenuItem* root = popup->Items;
              root->Clear();
              T->Buttons[0]->Hint = ("Search " + ReplaceStringWithString(table_name, "_", " ") + " or pick from \"favorites\"").c_str();
              T->Buttons[0]->ShowHint = true;

              for (set<string>::iterator itr = set_favorites.begin(); itr != set_favorites.end(); itr++)
              {
                string fav_id = *itr;
                TMenuItem* M = new TMenuItem(root);
                M->Caption = DefaultText(fav_id, table_name, permission_name, options).c_str();
                M->OnClick = funcLinkMenuItemClick /*T->Buttons[0]->OnClick*/ ;
                M->Tag = StringToInt(fav_id);
                root->Add(M);
                registered_link_buttons[M] = text_object;
                register_ok = true;
              }
              insert_ok = true;
            }
          }
        }
      }
      catch (...) { }

      if (T && !insert_ok)
      {
        T->Buttons[0]->Hint = ("Search " + ReplaceStringWithString(table_name, "_", " ")).c_str();
        T->Buttons[0]->ShowHint = true;
        T->Buttons[0]->Style = tbsButton;
      }
    }
    else if (search_button->InheritsFrom(__classid(TButton)))
    {
      // jhaley 20101012: Support this behavior on TButton descendents for easy access to favorites
      bool insert_ok = false;
      TButton *B = NULL;
      try
      {
        B = (TButton *)search_button;

        if (DefaultTextAvailable(table_name))
        {
          // we should know better than to make a link where we don't have text to display, but we'll play it safe
          string string_favorites = (alt_favorites=="" ? optionsPreferences["favorites"][table_name] : alt_favorites);
          set<string> set_favorites;
          SplitStringToSet(string_favorites, "|", set_favorites);
          set_favorites.erase("");
          if (!set_favorites.empty())
          {
            if (!B->PopupMenu)
                B->PopupMenu = new TPopupMenu(parent_form);
            TPopupMenu *popup = B->PopupMenu;
            TMenuItem  *root  = popup->Items;
            root->Clear();
            B->Hint = ("Search " + ReplaceStringWithString(table_name, "_", " ") + " or right-click to choose a favorite").c_str();
            B->ShowHint = true;

            for (set<string>::iterator itr = set_favorites.begin(); itr != set_favorites.end(); itr++)
            {
              string fav_id = *itr;
              TMenuItem *M = new TMenuItem(root);
              M->Caption = DefaultText(fav_id, table_name, permission_name, options).c_str();
              M->OnClick = funcLinkMenuItemClick /*T->Buttons[0]->OnClick*/ ;
              M->Tag = StringToInt(fav_id);
              root->Add(M);
              registered_link_buttons[M] = text_object;
              register_ok = true;
            }
            insert_ok = true;
          }
        }
      }
      catch (...) { }

      if (B && !insert_ok)
      {
        B->Hint = ("Search " + ReplaceStringWithString(table_name, "_", " ")).c_str();
        B->ShowHint = true;
      }
    }
  }

  // jhaley 20110106: Links must *always* be registered now, or they will retain their previous values across calls to
  // TformRecordGeneric's IsNewAgain method, allowing a link which says "None" to validate as an expired ID.
  //if (register_ok)
  //{ // register the link if it has a value, OR if there are favorites that might update it
  link_struct temp_struct;
  temp_struct.id = id;
  temp_struct.table_name = table_name;
  temp_struct.parent_form = parent_form;
  temp_struct.permission_name = permission_name;
  temp_struct.options = options;
  registered_links[text_object] = temp_struct;
}
//---------------------------------------------------------------------------
string GetLinkId(TStaticText *link)
{
  string result = "";
  map<TStaticText *, link_struct>::iterator itr = registered_links.find(link);
  if (itr != registered_links.end())
    result = registered_links[link].id;
  return result;
}
//---------------------------------------------------------------------------
// jhaley 20101026: This is a stronger variant of the above function. It will return the empty string if the link is
// not registered, *or* if the link doesn't have a valid numeric ID (ie. not "new", "0", "", etc.)
string GetValidLinkId(TStaticText *link)
{
  string result = "";
  map<TStaticText *, link_struct>::iterator itr = registered_links.find(link);
  if (itr != registered_links.end())
  {
    result = registered_links[link].id;
    if (IsBlankOrZero(result) || !IsInt(result))
       result = "";
  }
  return result;
}
//---------------------------------------------------------------------------
void UnregisterLinks(TForm * parent_form)
{
  map<TStaticText *, link_struct>::iterator a = registered_links.begin();
  map<TStaticText *, link_struct>::iterator b;
  while (a != registered_links.end())
  {
    b = a;
    b++;
    if(a->second.parent_form == parent_form)
    {
      // clear all of the favorites menu items that link to the to-be-deleted link
      map<TMenuItem *, TStaticText *>::iterator itr1 = registered_link_buttons.begin();
      map<TMenuItem *, TStaticText *>::iterator itr2;
      while (itr1 != registered_link_buttons.end())
      {
        itr2 = itr1;
        itr2++;
        if (itr1->second == a->first)
          registered_link_buttons.erase(itr1);
        itr1 = itr2;
      }
      // dependencies cleared up, delete the link
      registered_links.erase(a);
    }
    a = b;
  }
}
//---------------------------------------------------------------------------
void GetFromList (TListView * p_list, vector<vector<string> > & answer)
{
 answer.clear();
 TListItem * current_row;
 vector<string> blank_row;
 for (int i = 0; i < p_list->Items->Count; i++)
 {
  current_row = p_list->Items->Item[i];
  blank_row.clear();
  blank_row.push_back(string(current_row->Caption.c_str()));
  for (int j = 0; j < current_row->SubItems->Count; j++)
  {
   blank_row.push_back(string(current_row->SubItems->Strings[j].c_str()));
  }
  answer.push_back(blank_row);
 }
 return;
}
//---------------------------------------------------------------------------
void RecurseEnabled (TWinControl * parent, bool make_enabled)
{
 try
 {
  if(parent)
  {
   if (IsVisible(parent))
   {
    parent->Enabled = make_enabled;
    for (int i = 0; i < parent->ControlCount; i++)
    {
     if(parent->Controls[i]->InheritsFrom(__classid(TWinControl)))
       RecurseEnabled((TWinControl*)parent->Controls[i], make_enabled);
     else if(parent->Controls[i]->InheritsFrom(__classid(TControl)))
       ((TControl*)parent->Controls[i])->Enabled = make_enabled; // end of the line
    }
   }
  }
 }
 catch(...)
 {
 }
 return;
}
//---------------------------------------------------------------------------
// slightly more generic. also takes care of captions, which are not TWinControls
void RecurseEnabledAny (TControl * parent, bool make_enabled)
{
 try
 {
  if(parent)
  {
   if (IsVisible(parent))
   {
    parent->Enabled = make_enabled;
    if(parent->InheritsFrom(__classid(TWinControl)))
    {
     TWinControl * win_parent = (TWinControl*)(parent);
     for (int i = 0; i < win_parent->ControlCount; i++)
      RecurseEnabledAny(win_parent->Controls[i], make_enabled);
    }
   }
  }
 }
 catch(...)
 {
 }
 return;
}
//---------------------------------------------------------------------------
bool GetTextExtent(TWinControl* WinControl, TFont* Font, string Text, int& Width, int& Height)
{
  HDC ctrlDC = GetDC(WinControl->Handle);
  bool result = true;

  if(ctrlDC)
  {
    result = GetTextExtent(ctrlDC, Font, Text, Width, Height);
    ReleaseDC(WinControl->Handle, ctrlDC); // jhaley 20110901: plugged a DC handle leak
  }

  return result;
}
//---------------------------------------------------------------------------
bool GetTextExtent(HDC DC, TFont* Font, string Text, int& Width, int& Height)
{
  bool result;
  try
  {
    // As usual, I'll use all caps to indicate Win32 API objects.  Seems somehow fitting....
    // We need the device context to the wincontrol, don't need the HWND though.
    //HDC DC = GetDC(WinControl->Handle);
    // and the HFONT of the font
    HFONT FONT = Font->Handle;
    // Need to make sure the font is the active one (save the previously selected one)
    HGDIOBJ PREVFONT = SelectObject(DC, FONT);
    // Let's get the text extents now
    SIZE SIZE;
    if (GetTextExtentPoint32(DC, Text.c_str(), Text.length(), &SIZE))
    {
      Width = SIZE.cx;
      Height = SIZE.cy;
      result = true;
    }
    else
      result = false;

    // Reset the font to whatever it was before
    SelectObject(DC, PREVFONT);
  }
  catch (...)
  {
    result = false;
  }

  return result;
}
//---------------------------------------------------------------------------
void RecursiveClear (TWinControl * parent)
{
  try
  {
    if(parent)
    {
      for (int i = 0; i < parent->ControlCount; i++)
      {
        if(parent->Controls[i]->ClassType() == __classid(TEdit))
          ((TEdit *)parent->Controls[i])->Text = "";
        else if(parent->Controls[i]->ClassType() == __classid(TMemo))
          ((TMemo *)parent->Controls[i])->Text = "";
        else if(parent->Controls[i]->ClassType() == __classid(TRichEdit))
          ((TRichEdit *)parent->Controls[i])->Text = "";
        else if(parent->Controls[i]->ClassType() == __classid(TMaskEdit))
          ((TMaskEdit *)parent->Controls[i])->Text = "";
        else if(parent->Controls[i]->ClassType() == __classid(TComboBox))
        {
          ((TComboBox *)parent->Controls[i])->Text = "";
          ((TComboBox *)parent->Controls[i])->ItemIndex = -1; // jhaley 20110531: fix identified by Paul and emailed in.
        }
        else if(parent->Controls[i]->ClassType() == __classid(TCheckBox))
          ((TCheckBox *)parent->Controls[i])->Checked = false;
        else if(parent->Controls[i]->ClassType() == __classid(TRadioButton))
          ((TRadioButton *)parent->Controls[i])->Checked = false;

        if(parent->Controls[i]->InheritsFrom(__classid(TWinControl)))
        RecursiveClear((TWinControl*)(parent->Controls[i]));
      }
    }
  }
  catch(...)
  {
  }
}
//---------------------------------------------------------------------------


//---------------------------------------------------------------------------
void AddAccountAmount (TEdit * edit_account, TEdit * edit_amount, TListView * list_amounts, int account_rule)
{
 map<string, string> wanted_account = PartsByMask (edit_account);
 bool looking_for_budget = (acc_budget & account_rule);
 acc_info requested_account = InfoByParts (wanted_account, looking_for_budget);
 string given_amount = AmountToString(edit_amount->Text.c_str());
 if (requested_account.Matches(account_rule))
 {
  if (StringToFloat(given_amount) > 0.00)
  {
   TListItem * new_row = NULL;
   if (list_amounts->Selected)
    new_row = list_amounts->Selected;
   else
    new_row = list_amounts->Items->Add();
   if (new_row)
   {
    new_row->Caption = edit_account->Text;
    new_row->SubItems->Clear();
    new_row->SubItems->Add((StringToAmount(given_amount)).c_str());
    new_row->SubItems->Add(requested_account.id.c_str());
    list_amounts->Selected = NULL;
    edit_account->Text = "000-0000000-000-000";
    edit_amount->Text = "0.00";
    edit_account->SetFocus();
   }
   else
   {
    PSay("Ouch, cannot add row.", m_program_error, MB_OK, "components");
   }
  }
  else
  {
   PSay("Amounts must be positive.", m_invalid_data, MB_OK, "components");
  }
 }
 else
 {
  PSay("The account you have specified either does not exist,\nor cannot be referenced on this screen.", m_invalid_data, MB_OK, "components");
  edit_account->SetFocus();
 }

}
//---------------------------------------------------------------------------
void DelAccountAmount (TEdit * edit_account, TEdit * edit_amount, TListView * list_amounts, int account_rule)
{
 if (list_amounts->Selected)
 {
  list_amounts->Selected->Delete();
  list_amounts->Selected = NULL;
  edit_account->Text = "000-0000000-000-000";
  edit_amount->Text = "0.00";
 }
 // else the user is stupid and shouldn't be pushing buttons!
}
//---------------------------------------------------------------------------
void ModAccountAmount (TEdit * edit_account, TEdit * edit_amount, TListView * list_amounts, int account_rule)
{
 if (list_amounts->SelCount)
 {
  //PSay("Modifying!", m_message, MB_OK);
  edit_account->Text = list_amounts->Selected->Caption;
  edit_amount->Text = list_amounts->Selected->SubItems->Strings[0];
 }
 else // otherwise, they unselected, so we want a new one entered
 {
  //PSay("Resetting...", m_message, MB_OK);
  edit_account->Text = "000-0000000-000-000";
  edit_amount->Text = "0.00";
 }
}
//---------------------------------------------------------------------------
void SetAccountAmount (TEdit * edit_account, TEdit * edit_amount, TListView * list_amounts, int account_rule)
{
 list_amounts->Columns->Clear();
 TListColumn * account_column = list_amounts->Columns->Add();
 account_column->Caption = "Account";
 account_column->Width = 175;
 TListColumn * amount_column = list_amounts->Columns->Add();
 amount_column->Caption = "Amount";
 amount_column->Width = 70;
 list_amounts->Items->Clear();
 list_amounts->ReadOnly = true;
 list_amounts->HideSelection = false;
 list_amounts->MultiSelect = false;
 list_amounts->RowSelect = true;
 list_amounts->ShowColumnHeaders = true;
 list_amounts->ViewStyle = vsReport;

 MaskSetup (edit_account);
 edit_amount->Text = "0.00";
}
//---------------------------------------------------------------------------
map<string, string> GetAccountAmount (TListView * list_amounts)
{
 map<string, string> answer;
 // return <id, amount> pairs
 TListItem * this_row = NULL;
 int num_rows = list_amounts->Items->Count;
 string this_account_id;
 string this_account_amount;

 for (int i = 0; i < num_rows; i++)
 {
  this_row = list_amounts->Items->Item[i];
  // this_row->Caption; // ignored
  this_account_id = (this_row->SubItems->Strings[1]).c_str();
  this_account_amount = AmountToString((this_row->SubItems->Strings[0]).c_str());
  answer.insert(pair<string, string>(this_account_id, this_account_amount));
 }

 return answer;
}
//---------------------------------------------------------------------------
/* Toasted!
// added for the sake of codeSpecialNewVendor bill when it needed to handle payments for employee's
// Matt Green 7/24/2002
multimap<string, string> GetMultiAccountAmount(TListView * list_amounts)
{
 multimap<string, string> answer;
 // return <id, amount> pairs
 TListItem * this_row = NULL;
 int num_rows = list_amounts->Items->Count;
 string this_account_id;
 string this_account_amount;

 for (int i = 0; i < num_rows; i++)
 {
  this_row = list_amounts->Items->Item[i];
  // this_row->Caption; // ignored
  this_account_id = (this_row->SubItems->Strings[1]).c_str();
  this_account_amount = AmountToString((this_row->SubItems->Strings[0]).c_str());
  answer.insert(pair<string, string>(this_account_id, this_account_amount));
 }

 return answer;
}
*/
//---------------------------------------------------------------------------
bool acc_info::Matches (int account_rule)
{
 if (!found)
  return false;

 int account_is = acc_none;

 if (budget == "1")
  account_is |= acc_budget;
 else
  account_is |= acc_non_budget;

 if (closed == "1")
  account_is |= acc_closed;
 else
  account_is |= acc_non_closed;

 if (yearly == "1")
  account_is |= acc_yearly;
 else
  account_is |= acc_non_yearly;

 if (UppercaseString(rulename) == "ASSET")
  account_is |= acc_asset;
 else if (UppercaseString(rulename) == "CASH")
  account_is |= acc_cash;
 else if (UppercaseString(rulename) == "PAYABLE")
  account_is |= acc_payable;
 else if (UppercaseString(rulename) == "RECEIVABLE")
  account_is |= acc_receivable;
 else if (UppercaseString(rulename) == "REVENUE")
  account_is |= acc_revenue;
 else if (UppercaseString(rulename) == "EXPENSE")
  account_is |= acc_expense;
 else if (UppercaseString(rulename) == "EQUITY")
  account_is |= acc_equity;
 else if (UppercaseString(rulename) == "INIDIRECT EXPENSES")
  account_is |= acc_indirect;

 account_rule = ~account_rule; // get inverse as mask (1 indicates must-not-be)

 if (account_rule & account_is)
 {
  // not equal to zero, ... any bits here indicate something is true in the
  // account that is not supposed to be true!
  // PSay(IntToString(account_rule & account_is), m_message, MB_OK);
  return false;
 }

 return true;
}
//---------------------------------------------------------------------------
/*
#define acc_none           0
#define acc_budget         1
#define acc_non_budget     2
#define acc_yearly         4
#define acc_non_yearly     8
#define acc_closed        16
#define acc_non_closed    32
#define acc_cash          64
#define acc_payable      128
#define acc_receivable   256
#define acc_revenue      512
#define acc_expense     1024
#define acc_equity      2048
#define acc_asset       4096
#define acc_indirect    8192
*/
//---------------------------------------------------------------------------
string AccountTypeToString(account_types account_type)
{
 switch (account_type)
 {
  case equity_acc:
   return "Equity";
  case payable_acc:
   return "Payable";
  case receivable_acc:
   return "Receivable";
  case cash_acc:
   return "Cash";
  case expense_acc:
   return "Expense";
  case revenue_acc:
   return "Revenue";
  default:
   return "";
 }
}
//---------------------------------------------------------------------------
account_types StringToAccountType(string account_type_name)
{
 if (UppercaseString(account_type_name) == "EQUITY")
  return equity_acc;
 if (UppercaseString(account_type_name) == "PAYABLE")
  return payable_acc;
 if (UppercaseString(account_type_name) == "RECEIVABLE")
  return receivable_acc;
 if (UppercaseString(account_type_name) == "CASH")
  return cash_acc;
 if (UppercaseString(account_type_name) == "EXPENSE")
  return expense_acc;
 if (UppercaseString(account_type_name) == "REVENUE")
  return revenue_acc;
 return unknown_acc;
}
//---------------------------------------------------------------------------
string LinkTypeToString(account_links to_follow)
{
 switch (to_follow)
 {
  case balance_ref:
   return "balance_account_id";
  case pending_ref:
   return "pending_account_id";
  case cash_ref:
   return "cash_account_id";
  case budget_ref:
   return "budget_account_id";
  case actual_ref:
   return "actual_account_id";
  case repay_ref:
   return "repay_account_id";
  default:
   return "erroneous_account_id";
 }
}
//---------------------------------------------------------------------------
account_links StringToLinkType(string link_name)
{
 if (UppercaseString(link_name) == "BALANCE_ACCOUNT_ID")
  return balance_ref;
 if (UppercaseString(link_name) == "PENDING_ACCOUNT_ID")
  return pending_ref;
 if (UppercaseString(link_name) == "CASH_ACCOUNT_ID")
  return cash_ref;
 if (UppercaseString(link_name) == "BUDGET_ACCOUNT_ID")
  return budget_ref;
 if (UppercaseString(link_name) == "ACTUAL_ACCOUNT_ID")
  return actual_ref;
 if (UppercaseString(link_name) == "REPAY_ACCOUNT_ID")
  return repay_ref;
 return unknown_ref;
}
//---------------------------------------------------------------------------
string EquivAccount (account_links to_follow, string account_id)
{
 if (IsInt(account_id) && !account_id.empty())
 {
  string link_name = LinkTypeToString(to_follow);
  string sql_find_equiv_account = "select " + link_name + " from accounts where id = " + account_id;
  string found_id = GetOneField(dbDatabase, sql_find_equiv_account);
  return found_id;
 } else
  return "";
}
//---------------------------------------------------------------------------
acc_info EquivAccountInfo (account_links to_follow, string account_id)
{
 return InfoById(EquivAccount(to_follow, account_id));
}
//---------------------------------------------------------------------------
string EquivAccount (account_links to_follow, acc_info account_info)
{
 return account_info.EquivAccount(to_follow);
 // return EquivAccount (to_follow, account_info.id);
}
//---------------------------------------------------------------------------
acc_info EquivAccountInfo (account_links to_follow, acc_info account_info)
{
 return account_info.EquivAccountInfo(to_follow);
 // return EquivAccountInfo (to_follow, account_info.id);
}
//---------------------------------------------------------------------------
bool IsDebitPositiveByAccountType (account_types this_type)
{
 if (this_type == expense_acc || this_type == receivable_acc || this_type == cash_acc || this_type == unknown_acc)
  return true;
 else
  return false;
}

bool acc_info::IsDebitPositive ()
{
 account_types this_type = GetType();
 return IsDebitPositiveByAccountType(this_type);
}
//---------------------------------------------------------------------------
set<string> GetRequiredAccountReferences(string account_type, bool is_budget)
{
 set<string> answer;
 map<account_links, pair<account_types, bool> > temp = GetRequiredAccountReferences(StringToAccountType(account_type), is_budget);
 for (map<account_links, pair<account_types, bool> >::iterator i = temp.begin(); i != temp.end(); i++)
  answer.insert(LinkTypeToString(i->first));
 return answer;
}
//---------------------------------------------------------------------------
map<account_links, pair<account_types, bool> > GetRequiredAccountReferences(account_types account_type, bool is_budget)
{
 map<account_links, pair<account_types, bool> > answer;

 if (is_budget)
 {
  if (account_type == equity_acc)
  {
   answer.insert(pair<account_links, pair<account_types, bool> >(actual_ref, pair<account_types, bool>(equity_acc, false)));
  } else
  if (account_type == expense_acc)
  {
   answer.insert(pair<account_links, pair<account_types, bool> >(balance_ref, pair<account_types, bool>(equity_acc, true)));
   answer.insert(pair<account_links, pair<account_types, bool> >(actual_ref, pair<account_types, bool>(expense_acc, false)));
  } else
  if (account_type == revenue_acc)
  {
   answer.insert(pair<account_links, pair<account_types, bool> >(balance_ref, pair<account_types, bool>(equity_acc, true)));
   answer.insert(pair<account_links, pair<account_types, bool> >(actual_ref, pair<account_types, bool>(revenue_acc, false)));
  }
 } else
 {
  if (account_type == equity_acc)
  {
   answer.insert(pair<account_links, pair<account_types, bool> >(budget_ref, pair<account_types, bool>(equity_acc, true)));
  } else
  if (account_type == expense_acc)
  {
   answer.insert(pair<account_links, pair<account_types, bool> >(balance_ref, pair<account_types, bool>(equity_acc, false)));
   answer.insert(pair<account_links, pair<account_types, bool> >(budget_ref, pair<account_types, bool>(expense_acc, true)));
   answer.insert(pair<account_links, pair<account_types, bool> >(pending_ref, pair<account_types, bool>(payable_acc, false)));
   answer.insert(pair<account_links, pair<account_types, bool> >(cash_ref, pair<account_types, bool>(cash_acc, false)));
  } else
  if (account_type == revenue_acc)
  {
   answer.insert(pair<account_links, pair<account_types, bool> >(balance_ref, pair<account_types, bool>(equity_acc, false)));
   answer.insert(pair<account_links, pair<account_types, bool> >(budget_ref, pair<account_types, bool>(revenue_acc, true)));
   answer.insert(pair<account_links, pair<account_types, bool> >(pending_ref, pair<account_types, bool>(receivable_acc, false)));
   answer.insert(pair<account_links, pair<account_types, bool> >(cash_ref, pair<account_types, bool>(cash_acc, false)));
  } else
  if (account_type == payable_acc)
  {
   answer.insert(pair<account_links, pair<account_types, bool> >(cash_ref, pair<account_types, bool>(cash_acc, false)));
  } else
  if (account_type == receivable_acc)
  {
   answer.insert(pair<account_links, pair<account_types, bool> >(cash_ref, pair<account_types, bool>(cash_acc, false)));
   // answer.insert(pair<account_links, pair<account_types, bool> >(repay_ref, pair<account_types, bool>(payable_acc, false)));
  }
 }

 return answer;
}
//---------------------------------------------------------------------------
bool ValidateAccountReferences(map<account_links, acc_info> & acc_refs, account_types account_type, bool is_autocreate, bool is_closed, bool is_budget)
{
 map<account_links, pair<account_types, bool> > temp = GetRequiredAccountReferences(account_type, is_budget);

 if (is_autocreate)
 {
  map<account_links, pair<account_types, bool> >::iterator i_budget = temp.find(budget_ref);
  if (i_budget != temp.end())
   temp.erase(i_budget);
 }

 // check given references for validity, and check for extras
 for (map<account_links, acc_info>::iterator i_given = acc_refs.begin(); i_given != acc_refs.end(); i_given++)
 {
  map<account_links, pair<account_types, bool> >::iterator i_should_be = temp.find(i_given->first);
  if (i_should_be != temp.end())
  {
   // if it -should- exist ... check that it's found, etc.

   if (!i_given->second.IsFound() || i_given->second.id == "0")
   {
    // not found or not allowed blank
    if (IDNO == PSay(("The following account could not be found, or was blank: " + LinkTypeToString(i_given->first) + "\nWould you like to continue anyway, without it?"), m_invalid_data, MB_YESNO, "components"))
     return false;
    else
     i_given->second = InfoById("0"); // in that case, we'll allow it, reset it to zero (just in case) and ... skip the rest of the testing, but be approving. 
   } else
   if (i_given->second.IsClosed() && !is_closed)
   {
    // it's closed, and shouldn't be
    PSay(("The following account was closed/disabled and therefore may not be referenced here: " + LinkTypeToString(i_given->first)), m_invalid_data, MB_OK, "components");
    return false;
   } else
   if (i_should_be->second.first != i_given->second.GetType())
   {
    // wrong type
    PSay(("The following account was of the wrong type: " + LinkTypeToString(i_given->first) + " is a " + AccountTypeToString(i_given->second.GetType()) + " account, rather than a " + AccountTypeToString(i_should_be->second.first) + " account."), m_invalid_data, MB_OK, "components");
    return false;
   } else
   if (i_should_be->second.second != i_given->second.IsBudget())
   {
    // wrong budget-type
    if (i_given->second.IsBudget())
     PSay(("The following account is a budget account and therefore may not be referenced here: " + LinkTypeToString(i_given->first)), m_invalid_data, MB_OK, "components");
    else
     PSay(("The following account is not a budget account and therefore may not be referenced here: " + LinkTypeToString(i_given->first)), m_invalid_data, MB_OK, "components");
    return false;
   } else
   {
    // no problems! oh my goodness! it made it through the firey swamp!
   }
  } else
  {
   // this account reference wasn't needed ... so we didn't check it. is that a problem? hmmmm ...
   // maybe we should have checked it anyway, and ... for auto_create, deleted it from the map in the first place?
   // we'll see how this goes on record account. hard to test (disabled controls) ...
   // couldn't find the correct rule -- this is an extra reference
   // for now, ignoring this (other code will decide what to do with extras)
  }
 }

 // make sure none are missing
 // actually, no. recordAccount strips out references with values of "0" so they won't be checked.
 // user has already been warned. wizardNewAccount doesn't do that -- and values of "0" are not allowed.
 // and to say, i had some neat code checking for missing ones! hehehe ... shouldn't be a problem.
 // but if it is ... well ... add it back in! just don't forget that sometimes, we allow users to be
 // missing those values. grrr to the users.

 // the above has been changed -- users are warned for empty references in places where there should be something,
 // but are being allowed to continue so long as it's not a -bad- reference, just a missing one. 2/4/2004, Philip.

 return true;
}
//---------------------------------------------------------------------------

//---------------------------------------------------------------------------
// new accounting functions ... 'cause stuff was getting messy.
//---------------------------------------------------------------------------

//---------------------------------------------------------------------------
void acc_info::SetFromMap (map<string, string> data)
{
 found          = true;
 id             = data["acc_id"];
 rulename       = data["acc_type"];
 budget         = data["acc_budget"];
 closed         = data["acc_closed"];
 yearly         = data["acc_yearly"];
 parts["one"]   = data["one"];
 parts["two"]   = data["two"];
 parts["three"] = data["three"];
 parts["four"]  = data["four"];

 account_links temp_link = unknown_ref;
 for (map<string, string>::iterator i_ref = data.begin(); i_ref != data.end(); i_ref++)
 {
  temp_link = StringToLinkType(i_ref->first);
  if (temp_link != unknown_ref)
   references[temp_link] = i_ref->second;
 }
}
//---------------------------------------------------------------------------
account_types GetAccountTypeByTypeName (string rulename)
{
 // unknowns are done are purpose -- we don't usually deal with them,
 // so for now we're somewhat shunning them. this can be changed later, i promise
 account_types bob = unknown_acc;
 if (UppercaseString(rulename) == "ASSET")
  bob = unknown_acc;
 else if (UppercaseString(rulename) == "CASH")
  bob = cash_acc;
 else if (UppercaseString(rulename) == "PAYABLE")
  bob = payable_acc;
 else if (UppercaseString(rulename) == "RECEIVABLE")
  bob = receivable_acc;
 else if (UppercaseString(rulename) == "REVENUE")
  bob = revenue_acc;
 else if (UppercaseString(rulename) == "EXPENSE")
  bob = expense_acc;
 else if (UppercaseString(rulename) == "EQUITY")
  bob = equity_acc;
 else if (UppercaseString(rulename) == "INIDIRECT EXPENSES")
  bob = unknown_acc;
 return bob;
}

account_types acc_info::GetType ()
{
 return GetAccountTypeByTypeName(rulename);
}
//---------------------------------------------------------------------------
bool acc_info::IsType (account_types query)
{
 return (query == GetType());
}
//---------------------------------------------------------------------------
string acc_info::EquivAccount (account_links to_follow)
{
 return references[to_follow];
}
//---------------------------------------------------------------------------
acc_info acc_info::EquivAccountInfo (account_links to_follow)
{
 return InfoById (references[to_follow]);
}
//---------------------------------------------------------------------------
financial acc_info::GetCurrentBalance ()
{
 string sql_balance;
 if (IsDebitPositive())
  sql_balance = "select sum(debit_amount - credit_amount) from transactions where account_id = " + id;
 else
  sql_balance = "select sum(credit_amount - debit_amount) from transactions where account_id = " + id;
 return StringToFinancial(GetOneField(dbDatabase, sql_balance));
}
//---------------------------------------------------------------------------
bool acc_info::IsBudget ()
{
 return (budget == "1");
}
//---------------------------------------------------------------------------
bool acc_info::IsYearly ()
{
 return (yearly == "1");
}
//---------------------------------------------------------------------------
bool acc_info::IsClosed ()
{
 return (closed == "1");
}
//---------------------------------------------------------------------------
bool acc_info::IsFound  ()
{
 return found;
}
//---------------------------------------------------------------------------
string acc_info::GetTextName ()
{
 return (parts["one"] + "-" + parts["two"] + "-" + parts["three"] + "-" + parts["four"]);
}
//---------------------------------------------------------------------------
void MaskSetup   (TEdit * box)
{
 if (box)
 {
  //box->EditMask = "000\-0000000\-000\-000;1;_";
  box->Text     = "000-0000000-000-000";
 }
}
//---------------------------------------------------------------------------
string IdByParts (map<string, string> parts, bool is_budget)
{
 acc_info info;
 info = InfoByParts (parts, is_budget);
 if (info.IsFound())
  return info.id;
 else
  return "";
}
//---------------------------------------------------------------------------
string IdByMask  (TEdit * box, bool is_budget)
{
 acc_info info;
 info = InfoByMask (box, is_budget);
 if (info.IsFound())
  return info.id;
 else
  return "";
}
//---------------------------------------------------------------------------
string IdByInfo  (acc_info info)
{
 if (info.IsFound())
  return info.id;
 else
  return "";
}
//---------------------------------------------------------------------------
map<string, string> PartsById   (string id)
{
 acc_info info;
 info = InfoById (id);
 return info.parts;
}
//---------------------------------------------------------------------------
map<string, string> PartsByMask (TEdit * box)
{
 map<string, string> answer;
 if (box)
 {
  string working_info;
  working_info =  box->Text.c_str();
  unsigned int first_dash  = working_info.find('-', 0);
  unsigned int second_dash = working_info.find('-', first_dash + 1);
  unsigned int third_dash  = working_info.find('-', second_dash + 1);
  answer["one"]   = working_info.substr(0, first_dash);
  answer["two"]   = working_info.substr(first_dash + 1, (second_dash - first_dash) - 1);
  answer["three"] = working_info.substr(second_dash + 1, (third_dash - second_dash) - 1);
  answer["four"]  = working_info.substr(third_dash + 1);
 }
 return answer;
}
//---------------------------------------------------------------------------
map<string, string> PartsByInfo (acc_info info)
{
 return info.parts;
}
//---------------------------------------------------------------------------
void MaskById    (TEdit * box, string id)
{
 if (box)
  MaskByInfo (box, InfoById(id));
}
//---------------------------------------------------------------------------
void MaskByParts (TEdit * box, map<string, string> parts)
{
 if (box)
  box->Text = string(parts["one"] + "-" + parts["two"] + "-" + parts["three"] + "-" + parts["four"]).c_str();
}
//---------------------------------------------------------------------------
void MaskByInfo  (TEdit * box, acc_info info)
{
 if (box)
  MaskByParts (box, info.parts);
}
//---------------------------------------------------------------------------
acc_info InfoById    (string id)
{
 acc_info answer;
 if (IsInt(id) && !id.empty())
 {
  string sql;
  map<string, string> result;

  sql = acc_info_select_statement +
        "where accounts.id = " + id;

  if (SqlToMap(dbDatabase, sql, result))
  {
   if (!result.empty())
    answer.SetFromMap(result);
  } else
   PSay("Request for account information was invalid. (InfoById)", m_program_error, MB_OK, "components");
 }
 return answer;
}
//---------------------------------------------------------------------------
acc_info InfoByParts (map<string, string> parts, bool is_budget)
{
 acc_info answer;
 string sql;
 map<string, string> result;

 sql = acc_info_select_statement +
       "where accounts.is_budget = " + (is_budget ? "1" : "0") + " "
       "and acc_level_one.short_item   = '" + parts["one"]   + "' "
       "and acc_level_two.short_item   = '" + parts["two"]   + "' "
       "and acc_level_three.short_item = '" + parts["three"] + "' "
       "and acc_level_four.short_item  = '" + parts["four"]  + "' ";

 if (SqlToMap(dbDatabase, sql, result))
 {
  if (!result.empty())
   answer.SetFromMap(result);
 } else
  PSay("Request for account information was invalid. (InfoById)", m_program_error, MB_OK, "components");
 return answer;
}
//---------------------------------------------------------------------------
acc_info InfoByMask  (TEdit * box, bool is_budget)
{
 acc_info answer;
 if (box)
  answer = InfoByParts(PartsByMask(box), is_budget);
 return answer;
}
//---------------------------------------------------------------------------

/*
  map<string, string> cache_entity_id_to_name;
  map<string, acc_info> cache_account_id_to_info;
  map<string, map<string, financial> > data_account_entity_amount;
*/
void transaction_summary::AddFinancial (string account, string entity, financial amount)
{
 map<string, map<string, financial> >::iterator i_acc;
 map<string, financial>::iterator i_ent;

 i_acc = data_account_entity_amount.find(account);
 if (i_acc != data_account_entity_amount.end())
 {
  i_ent = i_acc->second.find(entity);
  if (i_ent != i_acc->second.end())
   i_ent->second += amount;
  else
   i_acc->second[entity] = amount;
 } else
  data_account_entity_amount[account][entity] = amount;
 return;
}

void transaction_summary::SubFinancial (string account, string entity, financial amount)
{
 AddFinancial(account, entity, -amount);
}

financial transaction_summary::GetFinancial (string account, string entity)
{
 financial answer = 0.00;
 map<string, map<string, financial> >::iterator i_acc;
 map<string, financial>::iterator i_ent;

 i_acc = data_account_entity_amount.find(account);
 if (i_acc != data_account_entity_amount.end())
 {
  i_ent = i_acc->second.find(entity);
  if (i_ent != i_acc->second.end())
   answer = i_ent->second;
 }

 return answer;
}

void transaction_summary::MergeCaches (const transaction_summary & rhs)
{
 // merge the caches
 // if they overlap, and we can't compare, don't take any chances ... look in the database!
 // otherwise, look in the database if they're different.
 for (map<string, acc_info>::const_iterator i_acc_cache = rhs.cache_account_id_to_info.begin(); i_acc_cache != rhs.cache_account_id_to_info.end(); i_acc_cache++)
 {
  if (cache_account_id_to_info.find(i_acc_cache->first) == cache_account_id_to_info.end())
   cache_account_id_to_info[i_acc_cache->first] = i_acc_cache->second;
  else
   cache_account_id_to_info[i_acc_cache->first] = InfoById(i_acc_cache->first);
 }

 for (map<string, string>::const_iterator i_ent_cache = rhs.cache_entity_id_to_name.begin(); i_ent_cache != rhs.cache_entity_id_to_name.end(); i_ent_cache++)
 {
  if (cache_entity_id_to_name.find(i_ent_cache->first) == cache_entity_id_to_name.end())
   cache_entity_id_to_name[i_ent_cache->first] = i_ent_cache->second;
  else if (cache_entity_id_to_name[i_ent_cache->first] != i_ent_cache->second)
   cache_entity_id_to_name[i_ent_cache->first] = GetOneField(dbDatabase, "select tax_entities.name from tax_entities where tax_entities.id = " + i_ent_cache->first);
  else
   ; // do nothing, they're the same; why bother?
 }
 return;
}

map<string, map<account_types, financial> > transaction_summary::GetEntityTypeAmount ()
{
 map<string, map<account_types, financial> > answer;
 for (map<string, map<string, financial> >::const_iterator i_account = data_account_entity_amount.begin(); i_account != data_account_entity_amount.end(); i_account++)
 {
  account_types this_type = GetCachedAccountInfo(i_account->first).GetType();
  for (map<string, financial>::const_iterator i_entity = i_account->second.begin(); i_entity != i_account->second.end(); i_entity++)
  {
   if (answer[i_entity->first].find(this_type) == answer[i_entity->first].end())
    answer[i_entity->first][this_type] = i_entity->second;
   else
    answer[i_entity->first][this_type] += i_entity->second;
  }
 }
 return answer;
}

map<account_types, map<string, financial> > transaction_summary::GetTypeEntityAmount ()
{
 map<account_types, map<string, financial> > answer;
 for (map<string, map<string, financial> >::const_iterator i_account = data_account_entity_amount.begin(); i_account != data_account_entity_amount.end(); i_account++)
 {
  account_types this_type = GetCachedAccountInfo(i_account->first).GetType();
  for (map<string, financial>::const_iterator i_entity = i_account->second.begin(); i_entity != i_account->second.end(); i_entity++)
  {
   if (answer[this_type].find(i_entity->first) == answer[this_type].end())
    answer[this_type][i_entity->first] = i_entity->second;
   else
    answer[this_type][i_entity->first] += i_entity->second;
  }
 }
 return answer;
}

unsigned int transaction_summary::GetCountEntitiesOut ()
{
 set<string> set_entities;
 for (map<string, map<string, financial> >::const_iterator i_account = data_account_entity_amount.begin(); i_account != data_account_entity_amount.end(); i_account++)
 {
  account_types this_type = GetCachedAccountInfo(i_account->first).GetType();
  for (map<string, financial>::const_iterator i_entity = i_account->second.begin(); i_entity != i_account->second.end(); i_entity++)
   if ((this_type == expense_acc && i_entity->second > 0.00) ||
       (this_type == payable_acc && i_entity->second > 0.00) ||
       (this_type == cash_acc && i_entity->second < 0.00))
    set_entities.insert(i_entity->first);
 }
 return set_entities.size();
}

unsigned int transaction_summary::GetCountEntitiesIn ()
{
 set<string> set_entities;
 for (map<string, map<string, financial> >::const_iterator i_account = data_account_entity_amount.begin(); i_account != data_account_entity_amount.end(); i_account++)
 {
  account_types this_type = GetCachedAccountInfo(i_account->first).GetType();
  for (map<string, financial>::const_iterator i_entity = i_account->second.begin(); i_entity != i_account->second.end(); i_entity++)
   if ((this_type == revenue_acc && i_entity->second > 0.00) ||
       (this_type == receivable_acc && i_entity->second > 0.00) ||
       (this_type == cash_acc && i_entity->second > 0.00))
    set_entities.insert(i_entity->first);
 }
 return set_entities.size();
}

acc_info transaction_summary::GetCachedAccountInfo (string account_id)
{
 acc_info answer;
 if (cache_account_id_to_info.find(account_id) == cache_account_id_to_info.end())
  cache_account_id_to_info.insert(pair<string, acc_info>(account_id, InfoById(account_id)));
 answer = cache_account_id_to_info[account_id];
 return answer;
}

string transaction_summary::GetCachedEntityName (string entity_id)
{
 string answer;
 if (cache_entity_id_to_name.find(entity_id) == cache_entity_id_to_name.end())
  cache_entity_id_to_name.insert(pair<string, string>(entity_id, GetOneField(dbDatabase, "select tax_entities.name from tax_entities where tax_entities.id = " + entity_id)));
 answer = cache_entity_id_to_name[entity_id];
 return answer;
}

transaction_summary::transaction_summary ()
  : cache_entity_id_to_name(), cache_account_id_to_info(), data_account_entity_amount()  // jhaley - call constructors
{
 // do absolutely nothing, i guess
}

transaction_summary::transaction_summary (const vector<map<string, string> > & input_data)
  : cache_entity_id_to_name(), cache_account_id_to_info(), data_account_entity_amount()  // jhaley - call constructors
{
 RawInput (input_data);
}

void transaction_summary::RawInput (const vector<map<string, string> > & input_data)
{
 data_account_entity_amount.clear();
 vector<map<string, string> >::const_iterator i;
 map<string, string> temp_row;
 string entity_id;
 string account_id;
 financial debit_amount;
 financial credit_amount;
 financial change_amount;
 acc_info about_account;

 for (i = input_data.begin(); i != input_data.end(); i++)
 {
  entity_id     = (i->find("tax_entity_id"))->second;
  account_id    = (i->find("account_id"))->second;
  about_account = GetCachedAccountInfo(account_id);
  debit_amount  = StringToFinancial((i->find("debit_amount"))->second);
  credit_amount = StringToFinancial((i->find("credit_amount"))->second);
  change_amount = debit_amount - credit_amount;

  if (about_account.IsDebitPositive())
   AddFinancial(account_id, entity_id, change_amount);
  else
   SubFinancial(account_id, entity_id, change_amount);
 }
 return;
}

bool transaction_summary::ChangeAmountTo (string account, string entity, financial amount, bool do_magic)
{
 // this one has additional logic (these are when the -directly- affected agent is on the left:)
 // [x] - expense (+) -> payable (+)                           // by default, expense goes to payable
 // [x] - expense (-) -> payable (-) until 0, then cash (+)    // take expense from payable, then reimburse cash
 // [x] - revenue (+) -> receivable (+)                        // by default, revenue goes to receivable
 // [x] - revenue (-) -> receivable (-) until 0, then cash (-) // take revenue from receivable, then give the cash back to whoever
 // [x] - payable (+) -> cash (+)                              // take out a loan, more or less
 // [x] - payable (-) -> cash (-)                              // less payable, so more must have been paid, hand out cash
 // [x] - receivable (+) -> cash (-)                           // more to be received, give a loan
 // [x] - receivable (-) -> cash (+)                           // less to be received, we've been paid, take the cash
 // [!] - alternate case of payable <-> receivable             // for forwarding amounts
 // [ ] - instead, we'll let them modify cash, and ... just warn them that it won't balance. or something. even better -- warn them if this -leaves- it out of balance. (brings it back into balance?)

 // return false if trying to make an expense go negative, or a revenue go negative, or if the account isn't one of these types, etc.

 // we should warn users, you know ... that if they 'reimburse' someone, the better actually be giving them the cash!
 // not so bad if they're giving us our money back. fairly sure that happened. but i get the feeling we're going to gyp people here!

 acc_info about_account = GetCachedAccountInfo(account);
 financial change_amount = amount - GetFinancial(account, entity);

 if (!about_account.IsFound())
  return false;

 if (about_account.IsClosed() && optionsPermissions["special.modify_disabled_accounts"]["enabled"] != "true")
   return false;

 switch (about_account.GetType())
 {
  case expense_acc:
  {
   if (amount < 0.00)
    return false;
   acc_info pending_account_info = GetCachedAccountInfo(about_account.EquivAccount(pending_ref));
   acc_info cash_account_info    = GetCachedAccountInfo(about_account.EquivAccount(cash_ref));
   if (!pending_account_info.IsFound() || !pending_account_info.IsType(payable_acc) ||
       !cash_account_info.IsFound() || !cash_account_info.IsType(cash_acc))
    return false;

   AddFinancial(about_account.id, entity, change_amount);
   if (do_magic)
   {
    AddFinancial(pending_account_info.id, entity, change_amount);
    financial pending_amount = GetFinancial(pending_account_info.id, entity);
    // if it's negative, subtract it from pending (make that zero) and subtract it from cash (which adds to cash)
    if (pending_amount < 0.00)
    {
     SubFinancial(pending_account_info.id, entity, pending_amount);
     SubFinancial(cash_account_info.id, entity, pending_amount);
    }
   }
   break;
  }
  case revenue_acc:
  {
   if (amount < 0.00)
    return false;
   acc_info pending_account_info = GetCachedAccountInfo(about_account.EquivAccount(pending_ref));
   acc_info cash_account_info    = GetCachedAccountInfo(about_account.EquivAccount(cash_ref));
   if (!pending_account_info.IsFound() || !pending_account_info.IsType(receivable_acc) ||
       !cash_account_info.IsFound() || !cash_account_info.IsType(cash_acc))
    return false;

   AddFinancial(about_account.id, entity, change_amount);
   if (do_magic)
   {
    AddFinancial(pending_account_info.id, entity, change_amount);
    financial pending_amount = GetFinancial(pending_account_info.id, entity);
    // if it's negative, subtract it from pending (make that zero) and add it to cash (which subtracts from cash)
    if (pending_amount < 0.00)
    {
     SubFinancial(pending_account_info.id, entity, pending_amount);
     AddFinancial(cash_account_info.id, entity, pending_amount);
    }
   }
   break;
  }
  case payable_acc:
  {
   if (amount < 0.00)
    return false;
   acc_info cash_account_info    = GetCachedAccountInfo(about_account.EquivAccount(cash_ref));
   if (!cash_account_info.IsFound() || !cash_account_info.IsType(cash_acc))
    return false;

   AddFinancial(about_account.id, entity, change_amount);
   if (do_magic)
    AddFinancial(cash_account_info.id, entity, change_amount);
   break;
  }
  case receivable_acc:
  {
   if (amount < 0.00)
    return false;
   acc_info cash_account_info    = GetCachedAccountInfo(about_account.EquivAccount(cash_ref));
   if (!cash_account_info.IsFound() || !cash_account_info.IsType(cash_acc))
    return false;

   AddFinancial(about_account.id, entity, change_amount);
   if (do_magic)
    SubFinancial(cash_account_info.id, entity, change_amount);
   break;
  }
  case cash_acc:
  {
   // this will put it IN or OUT of balance, but probably not leave it alone.
   AddFinancial(about_account.id, entity, change_amount);
   // no magic to do!
   break;
  }
  default:
   return false;
 }

 return true;
}

bool transaction_summary::ChangeEntityTo (string old_entity, string new_entity)
{
 bool found = false;

 for (map<string, map<string, financial> >::iterator i = data_account_entity_amount.begin(); i != data_account_entity_amount.end(); i++)
  for (map<string, financial>::iterator j = i->second.begin(); j != i->second.end(); j++)
  {
   if (j->first == old_entity)
   {
    found = true;
    financial t = j->second;
    // if old = new, then this should work out okay. (no change.)
    AddFinancial(i->first, old_entity, -t);
    AddFinancial(i->first, new_entity, t);
   }
  }
 return found;
}

bool transaction_summary::BalanceAccount (string account, string entity)
{
 acc_info about_account = GetCachedAccountInfo(account);
 if (!about_account.IsFound())
  return false;

 // positive amounts should be subtracted from debit-positive accounts
 // positive amounts should be added to credit-positive accounts
 // negative amounts should be added to debit-positive accounts
 // negative amounts should be subtracted (abs() added) to credit-positive accounts
 // if it's not cash, needs to not make it go negative. apply partially?

 financial to_apply = BalanceAmount();

 if (about_account.IsDebitPositive())
  to_apply = -to_apply;

 if (!about_account.IsType(cash_acc) && (GetFinancial(about_account.id, entity) + to_apply < 0.00))
  return false;

 AddFinancial(account, entity, to_apply);

 return true;
}

financial transaction_summary::BalanceAmount ()
{
 // okay. this ... uhm ... does the thingie.
 // positive amounts indicate that the absolute amount here should be subtracted from debit-positive accounts, or added to the others. i think.
 financial answer = 0.00;
 acc_info  current_account;
 map<string, financial> account_totals = GetAccountOutput ();

 for (map<string, financial>::iterator i_acc = account_totals.begin(); i_acc != account_totals.end(); i_acc++)
 {
  current_account = GetCachedAccountInfo(i_acc->first);
  if (current_account.IsDebitPositive())
   answer += i_acc->second;
  else
   answer -= i_acc->second;
 }
 return answer;
}

financial transaction_summary::AbsBalanceAmount ()
{
 // show the difference in absolutes ... nobody will remember which way is which anyway
 financial answer = BalanceAmount ();
 if (answer < 0.00)
  answer = -answer;
 return answer;
}

transaction_summary & transaction_summary::operator += (const transaction_summary & rhs)
{
 // take on more stuff, -including- its cache of account information!
 // budget cache, however, is external ... because we may contain old transactions already applied to the budgets, as well as new ones!
 // someone else gets to do the math for -that- thing ...

 MergeCaches (rhs);

 for (map<string, map<string, financial> >::const_iterator i_account = rhs.data_account_entity_amount.begin(); i_account != rhs.data_account_entity_amount.end(); i_account++)
  for (map<string, financial>::const_iterator i_entity = i_account->second.begin(); i_entity != i_account->second.end(); i_entity++)
   AddFinancial(i_account->first, i_entity->first, i_entity->second);

 return (*this);
}

transaction_summary & transaction_summary::operator -= (const transaction_summary & rhs)
{
 // make sure than new names/info are available to us, if they weren't already ...
 MergeCaches (rhs);

 for (map<string, map<string, financial> >::const_iterator i_account = rhs.data_account_entity_amount.begin(); i_account != rhs.data_account_entity_amount.end(); i_account++)
  for (map<string, financial>::const_iterator i_entity = i_account->second.begin(); i_entity != i_account->second.end(); i_entity++)
   SubFinancial(i_account->first, i_entity->first, i_entity->second);

 return (*this);
}

vector<map<string, string> > transaction_summary::GetPreviewOutput (bool show_zero_values)
{
 // this seems like it should be easy enough ... do an internal join, and have at it!
 vector<map<string, string> > answer;
 map<string, string> temp_row;
 map<string, map<string, financial> >::iterator i_account;
 map<string, financial>::iterator i_entity;
 acc_info about_account;

 // put default values in for the calling function -- things like dates, bills, tasks, etc.
 // move amounts to the correct debit/credit column, etc.
 for (i_account = data_account_entity_amount.begin(); i_account != data_account_entity_amount.end(); i_account++)
 {
  about_account = GetCachedAccountInfo(i_account->first);

  for (i_entity = i_account->second.begin(); i_entity != i_account->second.end(); i_entity++)
  {
   // in a lot of cases, it just looks better to list zero-amounts. people get less confused about empty lists.
   // if they reduce something to zero, and change their mind ... it's less trouble to just click on the damn row again.
   if (show_zero_values || i_entity->second != 0.00)
   {
    temp_row.clear();
    temp_row["account_id"]      = i_account->first;
    temp_row["account_name"]    = about_account.GetTextName();
    temp_row["account_type"]    = AccountTypeToString(about_account.GetType());
    temp_row["tax_entity_id"]   = i_entity->first;
    temp_row["tax_entity_name"] = GetCachedEntityName(i_entity->first);
    temp_row["change_amount"]   = FinancialToString(i_entity->second);
    answer.push_back(temp_row);
   }
  }
 }
 return answer;
}

vector<map<string, string> > transaction_summary::GetFinalOutput (map<string, string> & initial_row)
{
 // less fun, but still managable: apply preview to a 'real' row, with debits and credits
 map<string, string> temp_row;
 vector<map<string, string> > answer;
 map<string, map<string, financial> >::iterator i_account;
 map<string, financial>::iterator i_entity;
 acc_info about_account;
 bool is_debit_positive;

 // put default values in for the calling function -- things like dates, bills, tasks, etc.
 // move amounts to the correct debit/credit column, etc.
 for (i_account = data_account_entity_amount.begin(); i_account != data_account_entity_amount.end(); i_account++)
 {
  about_account     = GetCachedAccountInfo(i_account->first);
  is_debit_positive = about_account.IsDebitPositive();

  for (i_entity = i_account->second.begin(); i_entity != i_account->second.end(); i_entity++)
  {
   if (i_entity->second != 0.00)
   {
    temp_row = initial_row;
    temp_row["account_id"] = i_account->first;
    temp_row["tax_entity_id"] = i_entity->first;
    // i'm sure there was a way of being more clever with the following, but ... bah humbug.
    if (i_entity->second > 0.00)
    {
     temp_row[is_debit_positive ? "debit_amount" : "credit_amount"] = FinancialToString(i_entity->second);
     temp_row[is_debit_positive ? "credit_amount" : "debit_amount"] = "0.00";
    } else
    {
     temp_row[is_debit_positive ? "credit_amount" : "debit_amount"] = FinancialToString(-(i_entity->second));
     temp_row[is_debit_positive ? "debit_amount" : "credit_amount"] = "0.00";
    }
    answer.push_back(temp_row);
   }
  }
 }
 return answer;
}

map<string, financial> transaction_summary::GetAccountOutput ()
{
 // this is so we can quickly assess budgets -- an account id, and an amount it has changed ...
 // for budgets, this means taking the difference between two transaction_summaries (one of which
 // has already happened, and has been committed) and seeing what that does to budget sums. should work.
 map<string, financial> answer;
 financial current_total;
 for (map<string, map<string, financial> >::iterator i_account = data_account_entity_amount.begin(); i_account != data_account_entity_amount.end(); i_account++)
 {
  current_total = 0.00;
  for (map<string, financial>::iterator i_entity = i_account->second.begin(); i_entity != i_account->second.end(); i_entity++)
   current_total += i_entity->second;
  answer[i_account->first] = current_total;
 }
 return answer;
}

void transaction_summary::ResetAccountCache ()
{
 cache_account_id_to_info.clear();
 return;
}

void transaction_summary::ResetEntityCache ()
{
 cache_entity_id_to_name.clear();
 return;
}

void transaction_summary::TrimZeroAmounts ()
{
 map<string, map<string, financial> >::iterator i_acc = data_account_entity_amount.begin();
 map<string, map<string, financial> >::iterator i_acc_next = i_acc;

 while (i_acc_next != data_account_entity_amount.end())
 {
  i_acc = i_acc_next;
  i_acc_next++;

  map<string, financial>::iterator i_ent = i_acc->second.begin();
  map<string, financial>::iterator i_ent_next = i_ent;
  while (i_ent_next != i_acc->second.end())
  {
   i_ent = i_ent_next;
   i_ent_next++;
   if (i_ent->second == 0.00)
    i_acc->second.erase(i_ent);
  }

  if (i_acc->second.empty())
   data_account_entity_amount.erase(i_acc);
 }
 return;
}

void transaction_summary::RemoveEntities ()
{
 // WORKING
 for (map<string, map<string, financial> >::iterator i_acc = data_account_entity_amount.begin(); i_acc != data_account_entity_amount.end(); i_acc++)
 {
  // for each account, replace list of entity/amount with "0, sum(amounts)"
  financial running_total = 0.00;
  for (map<string, financial>::iterator i_ent = i_acc->second.begin(); i_ent != i_acc->second.end(); i_ent++)
   running_total += i_ent->second;
  i_acc->second.clear();
  i_acc->second.insert(pair<string, financial>("0", running_total));
 }
 return;
}

//---------------------------------------------------------------------------
transaction_summary operator + (const transaction_summary & lhs, const transaction_summary & rhs)
{
 transaction_summary temp = lhs;
 temp += rhs;
 return temp;
}

transaction_summary operator - (const transaction_summary & lhs, const transaction_summary & rhs)
{
 transaction_summary temp = lhs;
 temp -= rhs;
 return temp;
}
//---------------------------------------------------------------------------
financial transaction_summary::GetEntityPayment (string entity)
{
 financial total_payment;

 map<string, map<account_types, financial> > type_ent_amount = GetEntityTypeAmount();
 if (type_ent_amount[entity].find(payable_acc) != type_ent_amount[entity].end())
  total_payment += type_ent_amount[entity][payable_acc];
 if (type_ent_amount[entity].find(cash_acc) != type_ent_amount[entity].end())
  total_payment -= type_ent_amount[entity][cash_acc];

 return total_payment;
}
//---------------------------------------------------------------------------
int ArrangeControlsInline (TWinControl * container, bool is_vertical, int secondary_padding)
{
 // find them in order by ->Tag
 multimap<int, TControl *> controls_in_order;
 for (int i = 0; i < container->ControlCount; i++)
  if (container->Controls[i]->Visible)
   controls_in_order.insert(pair<int, TControl *>(container->Controls[i]->Tag, container->Controls[i]));

 // order them visually
 int current_primary   = 0;
 int next_primary      = 0;
 int current_secondary = 0;
 TControl * current_control = NULL;
 bool current_control_fits = false;

 // loop in order, move to next appropriate position
 for (multimap<int, TControl *>::iterator i_control = controls_in_order.begin(); i_control != controls_in_order.end(); i_control++)
 {
  current_control = i_control->second;

  // does it fit here?
  if (is_vertical)
   current_control_fits = (current_secondary + current_control->Height <= container->ClientHeight);
  else
   current_control_fits = (current_secondary + current_control->Width <= container->ClientWidth);

  // if not, and it might possibly by itself, go there.
  if (!current_control_fits && current_secondary != 0)
  {
   current_primary = next_primary;
   current_secondary = 0;
  }

  // put it where it goes.
  if (is_vertical)
  {
   current_control->Top = current_secondary;
   current_control->Left = current_primary;
  } else
  {
   current_control->Left = current_secondary;
   current_control->Top = current_primary;
  }

  // update the next position, make sure primary groups don't overlap
  if (is_vertical)
  {
   next_primary = max(next_primary, current_primary + current_control->Width);
   current_secondary = current_control->Top + current_control->Height;
  } else
  {
   next_primary = max(next_primary, current_primary + current_control->Height);
   current_secondary = current_control->Left + current_control->Width;
  }
  current_secondary += secondary_padding;
 }
 // return the amount of primary space required
 return next_primary;
}
//---------------------------------------------------------------------------
void SetupControlsForInline (TWinControl * container, list<TWinControl*> & useful_controls)
{
 for (int i = 0; i < container->ControlCount; i++)
  container->Controls[i]->Visible = false;

 int j = 0;
 for (list<TWinControl *>::iterator i_control = useful_controls.begin(); i_control != useful_controls.end(); i_control++)
 {
  (*i_control)->TabOrder = j;
  (*i_control)->Tag = j;
  (*i_control)->Visible = true;
  j++;
 }
}
//---------------------------------------------------------------------------
TComponent * FindComponentInsensitive (TComponent * owner, string name)
{
 TComponent * t = NULL;
 try
 {
  if(owner)
  {
   if(UppercaseString(owner->Name.c_str()) == UppercaseString(name))
    return owner;

   for (int i = 0; i < owner->ComponentCount; i++)
   {
    t = FindComponentInsensitive (owner->Components[i], name);
    if (t)
     return t;
   }
  }
 }
 catch(...)
 {
 }
 return NULL;
}
//---------------------------------------------------------------------------
TMenuItem * FindMenuItemByNameInsensitive (TMenuItem * parent, string name)
{
 TMenuItem * t = NULL;
 try
 {
  if(parent)
  {
   if(UppercaseString(parent->Name.c_str()) == UppercaseString(name))
    return parent;

   for (int i = 0; i < parent->Count; i++)
   {
    t = FindMenuItemByNameInsensitive (parent->Items[i], name);
    if (t)
     return t;
   }
  }
 }
 catch(...)
 {
 }
 return NULL;
}
//---------------------------------------------------------------------------
vector<string> RecursiveMenuItemName (TMenuItem * leaf, bool stripped)
{
 vector<string> answer;
 if (leaf)
 {
  if (leaf->Parent)
   answer = RecursiveMenuItemName(leaf->Parent, stripped);

  string leaf_name;
  if (stripped)
   leaf_name = ReplaceStringWithString(
                ReplaceStringWithString(
                 ReplaceStringWithString(leaf->Caption.c_str()
                 , "&&", "@")
                , "&", "")
               , "@", "&");
  else
   leaf_name = leaf->Caption.c_str();

  if (StripAll(leaf_name, " ") != "")
   answer.push_back(leaf_name);
 }
 return answer;
}
//---------------------------------------------------------------------------
string FullyQualifiedMenuItemName (TMenuItem * leaf)
{
 vector<string> parts = RecursiveMenuItemName(leaf, true);
 return VecToDelimString(parts, "", "", " -> ");
}
//---------------------------------------------------------------------------
// Zip code functions, centralized logic
//---------------------------------------------------------------------------
bool CountyLookupByState(string &state_name, string &state_id, string &county_sql, TComboBox * county_combo)
{ // Look up county listing by state.  Can pass state in by name or ID, and you'll get the other value back.  You'll get the SQL
  // necessary to generate county listing, and if you pass a combo box, it'll automatically be populated with appropriate counties.
  try
  {
    if (state_name != "")
      if (lookupTables->IsInLookUpTable("state_types", state_name))
        state_id = lookupTables->IdOf("state_types", state_name);
      else return false;
    else if (state_id != "")
      state_name = lookupTables->ValueOf("state_types", state_id);
    else return false;

    // " or county_types.id = 0"  is because I needed the ability to leave the field blank and have it still validate okay.
    county_sql = "select id, item, valid from county_types where state_id = " + state_id + " or county_types.id = 0 order by item";
    if (county_combo)
      lookupTables->RegisterComboBox("county_types", county_combo, county_sql);
  }
  catch (...) // things like invalid TComboBox *'s could screw things up
  { return false; }

  return true;
}
//---------------------------------------------------------------------------
bool ZipLookup(string zip_code, vector<map<string, string> > &zip_vec, string &state_name, string &county_name, string &preferred_city_name, TComboBox *state_combo, TComboBox *county_combo, TComboBox *city_combo)
{ // Look up state, county, city name(s) for given zip code.  Passes contents of vecmap back, should you require it for any manual
  // processing.  If you pass a state combo box, its ->Text property will be set to the state_name.  If you pass a county combo box,
  // it'll automatically be populated with all the counties within the state that the zip corresponds to.  If you pass a city combo
  // box, it'll be populated with the city names(s) corresponding to the zip code, and the ->Text property will automatically be set
  // to the "Preferred" (by the US Postal Service) city name.
  try
  {
    zip_code = zip_code.substr(0,5);

    string sql = "select * from zip_codes_view where zip_code = '" + zip_code + "'";
    SqlToVecMap(dbDatabase, sql, zip_vec);
    if (zip_vec.size() > 0)
    {
      // state and county will be the same for a given zip code, so just pick the first
      state_name = PrettyString(zip_vec[0]["state_name"]);
      if (state_combo) state_combo->Text = state_name.c_str();
      string state_id, county_sql;
      if (!CountyLookupByState(state_name, state_id, county_sql, county_combo))
        return false;
      county_name = zip_vec[0]["county"].c_str();
      if (county_combo) county_combo->Text = county_name.c_str();
      // city though might have some options, so let's loop through and populate the cbo box
      if (city_combo) city_combo->Items->Clear();
      for (unsigned int i=0; i < zip_vec.size(); i++)
      {
        if (city_combo) city_combo->Items->Add(zip_vec[i]["city"].c_str());
        if (zip_vec[i]["zip_code_type"] == "Preferred")
        {
          preferred_city_name = zip_vec[i]["city"].c_str();
          if (city_combo) city_combo->Text = preferred_city_name.c_str();
        }
      }
    }
    else
      return false;
  }
  catch (...) // things like invalid TComboBox *'s could screw things up
  { return false; }

  return true;
}
//---------------------------------------------------------------------------
// Follow-up functions for lookup tables with custom SQL
//---------------------------------------------------------------------------
bool SetGroupIdForNewLookupTableItem(TIBTransaction *dbTransaction, const string &tableName, const string &id)
{
  string sql = "update " + tableName + " set group_id = '" + current_groupid + "' where id = '" + id + "'";
  DebugSql(NULL, &sql, libraries, 8);
  return ExecuteStatement(dbTransaction, sql);
}
//---------------------------------------------------------------------------
TQRPrinter *InitTQRPrinter()
{
  TQRPrinter *retval = NULL;

  try
  {
#if cQRVersion > 350
    retval = new TQRPrinter(NULL);
#else
    retval = new TQRPrinter();
#endif
  }
  catch(...)
  {
    // meh...
  }

  return retval;
}
//---------------------------------------------------------------------------
void ArrangeByTabOrder(TWinControl * p, bool vertical)
{
 multimap<int, TControl *> controls_in_order;
 TControl* t = NULL;
 for (int i = 0; i < p->ControlCount; i++)
 {
  t = p->Controls[i];
  if (t->Visible && ((vertical && (t->Align == alTop || t->Align == alBottom))|| (!vertical && (t->Align == alLeft || t->Align == alRight))))
  {
   int to = 0;
   if(t->InheritsFrom(__classid(TWinControl)))
    to = ((TWinControl*)t)->TabOrder;
   controls_in_order.insert(pair<int, TControl *>(to, t));
  }
 }

 // order them visually
 int z = 0;
 TControl * current_control = NULL;

 // loop in order, move to next appropriate position
 for (multimap<int, TControl *>::iterator i_control = controls_in_order.begin(); i_control != controls_in_order.end(); i_control++)
 {
  current_control = i_control->second;
  if (vertical)
  {
   current_control->Top = z;
   z += current_control->Height;
  } else
  {
   current_control->Left = z;
   z += current_control->Width;
  }
 }
}
//---------------------------------------------------------------------------
void ArrangeByTag(TToolBar * p, bool vertical)
{
 multimap<int, TControl *> controls_in_order;
 TControl* t = NULL;
 for (int i = 0; i < p->ButtonCount; i++)
 {
  t = p->Buttons[i];
  if (t->Visible)
  {
   int to = t->Tag;
   controls_in_order.insert(pair<int, TControl *>(to, t));
  }
 }

 // order them visually
 int z = 0;
 TControl * current_control = NULL;

 // loop in order, move to next appropriate position
 for (multimap<int, TControl *>::iterator i_control = controls_in_order.begin(); i_control != controls_in_order.end(); i_control++)
 {
  current_control = i_control->second;
  if (vertical)
  {
   current_control->Top = z;
   z += current_control->Height;
  } else
  {
   current_control->Left = z;
   z += current_control->Width;
  }
 }
}
//---------------------------------------------------------------------------
// wound up not being needed, but kinda cool. can be called as:
// TControl * z = FindNearestParentOfType(this, __classid(TSomething));
TControl* FindNearestParentOfType (TControl* Child, TMetaClass* Classtype)
{
 TControl * t = Child;
 while (t)
 {
  if (t->InheritsFrom(Classtype))
   break;
  else
   t = t->Parent;
 }
 return t;
}

//---------------------------------------------------------------------------
TColor AverageColor(TColor A, TColor B)
{
 // copied from codeSpecialQuery -- should centralize some day!
 // some quick color-average code ... including learning about order of bytes. damn little-endian architectures! (or is it the other way around? i don't know!)
 //PSay("ColorToRGB(StringToColor(...)) of clBtnFace: " + UIntToString(ColorToRGB(StringToColor("clBtnFace"))), m_message, MB_OK);
 //int c; IdentToColor("clBtnFace", c);
 //PSay("ColorToRGB(IdentToColor(...)) of clBtnFace: " + UIntToString(ColorToRGB(c)), m_message, MB_OK);
 //TColor g = (TColor)ColorToRGB(StringToColor("clBtnFace"));
 //PSay("(unsigned int)(TColor) of clBtnFace: " + UIntToString((unsigned int)(g)), m_message, MB_OK);
 //TColor w = (TColor)ColorToRGB(clWhite);

 // Fixed for Windows 7.  Damn but that was annoying.
 struct _rgb {unsigned char r; unsigned char g; unsigned char b; unsigned int z;};
 union rgb {unsigned int i; _rgb c; unsigned char a[4];};
 rgb bob, larry, result;
 //PSay("A: " + string(IntToHex((int)A,8).c_str()) + " ; B: " + string(IntToHex((int)B,8).c_str()), m_message, MB_OK, "components");
 bob.i   = ColorToRGB(A);
 larry.i = ColorToRGB(B);
 //PSay("ColorToRGB(A): " + string(IntToHex((int)bob.i,8).c_str()) + " ; ColorToRGB(B): " + string(IntToHex((int)larry.i,8).c_str()), m_message, MB_OK, "components");
 for (unsigned int k = 0; k < 3; k++) // starts at the end -- RR, GG, BB, other
  result.a[k] = (bob.a[k] >> 1) + (larry.a[k] >> 1);
 result.a[3] = 0; // highest order byte gets zeroed because we hates it, precioussss
 //PSay("color-averaging result: " + string(IntToHex((int)result.i,8).c_str()), m_message, MB_OK, "components");
 //PSay("ColorToRGB(result): " + string(IntToHex(ColorToRGB(result.i),8).c_str()), m_message, MB_OK, "components");
 return ColorToRGB(result.i);
}
//---------------------------------------------------------------------------
// jhaley 20101229: Code to examine person_notes for group-relevant blocks and warnings
bool CountPersonBlocksAndWarnings(int &numblocks, int &numwarnings, string person_id, set<string> &group_ids,
                                  string min_current_id)
{
  bool success = false;
  int tmp_numblocks = 0;
  int tmp_numwarnings = 0;
  string sql;
  vector<string> results;

  sql = "select warning_id from person_notes where person_id = '" + person_id + "' "
        "and " + SafeInClause("group_id", SetToVec(group_ids)) + " ";

  if (min_current_id != "")
    sql += "and current_id >= '" + min_current_id + "' ";

  if (SqlToVec(dbDatabase, sql, results))
  {
    success = true;

    // count the warnings and blocks
    for (vector<string>::iterator i_note = results.begin(); i_note != results.end(); ++i_note)
    {
      if (*i_note == lookupTables->IdOf("warning_types", "Block"))
        ++tmp_numblocks;
      if (*i_note == lookupTables->IdOf("warning_types", "Warn"))
        ++tmp_numwarnings;
    }

    // put the results into the reference parameters
    numblocks   = tmp_numblocks;
    numwarnings = tmp_numwarnings;
  }

  // A return value of false is a vote of no confidence; the parameters were not modified.
  return success;
}
//---------------------------------------------------------------------------
// jhaley 20101229: Helper routine to give a message when a user does not have block override permissions. Extracted
// from the New CHS Item Wizard.
void BlockOverrideErrorMessage()
{
  string sql = "select users.item "
               "from users "
               "inner join people on users.person_id = people.id "
               "inner join person_names on people.person_names_id = person_names.id "
               "inner join permissions on permissions.user_id = users.id and "
               " permissions.s = 'SPECIAL.NOTE_BLOCK_OVERRIDE' and permissions.k = 'ENABLED' and permissions.v = 'TRUE' "
               "where valid = 1 "
               "order by person_names.last_name ";

  vector<string> users_with_note_block_override;
  if (!SqlToVec(dbDatabase, sql, users_with_note_block_override))
  {
    PSay("Failed to get a list of users with permission to override a block.", m_database_error, MB_OK,
         "components::BlockOverrideErrorMessage");
  }
  else
  {
    PSay("You do not have permission to override notes tagged as 'blocks'.\n"
         "You will have to find someone who does to help you. Users who may be able to help you include:\n"
         + VecToCommaString(users_with_note_block_override),
         m_invalid_data, MB_OK, "components::BlockOverrideErrorMessage");
  }
}
//---------------------------------------------------------------------------
