#ifndef components
#define components
#include <Classes.hpp>
#include <Controls.hpp>
#include <StdCtrls.hpp>
#include <Forms.hpp>
#include <ComCtrls.hpp>
#include <ToolWin.hpp>
#include <ExtCtrls.hpp>
#include <Mask.hpp>
#include <Buttons.hpp>
#include <vcl.h>
#include <vector>
#include <string>
#include <Db.hpp>
#include <IBCustomDataSet.hpp>
#include <IBDatabase.hpp>
#include <IBSQL.hpp>
#include <map>
#include <set>
#include <list>
#include <QuickRpt.hpp>
#include "allsymbols.h"
#include "financial.h"

using namespace std;

bool DefaultTextAvailable(string table_name, string permission_name = "", int options = -1);
string DefaultText(string id, string table_name, string permission_name = "", int options = -1);

struct link_struct
{
  string id;
  string table_name;
  string permission_name;
  int options;
  TForm * parent_form;
};

extern TNotifyEvent funcLinkClick;
extern TNotifyEvent funcLinkMenuItemClick;
void CreateLink(TStaticText * text_object, TControl * search_button,
                TForm * parent_form, string id, string table_name,
                string permission_name = "", bool is_enabled = true, int options = -1, string alt_favorites = "");
void UnregisterLinks(TForm * the_parent_form);
string GetLinkId(TStaticText *link);
string GetValidLinkId(TStaticText *link);
void RecurseEnabled (TWinControl * parent, bool make_enabled);
void RecurseEnabledAny (TControl * parent, bool make_enabled);
bool GetTextExtent(TWinControl* WinControl, TFont* Font, string Text, int& Width, int& Height);
bool GetTextExtent(HDC DC, TFont* Font, string Text, int& Width, int& Height);
void RecursiveClear (TWinControl * parent);
bool SetGroupIdForNewLookupTableItem(TIBTransaction *dbTransaction, const string &tableName, const string &id);

// accounting
class acc_info
{
 private:
  bool   found;
  string rulename;
  string budget;
  string closed;
  string yearly;
 public:
  string id;
  map<string, string> parts;
  map<account_links, string> references;

 public:
  acc_info () : found(false), rulename(), budget(), closed(), yearly(), id(), parts(), references() {};
  bool Matches (int account_rule);
  bool IsDebitPositive ();
  void SetFromMap (map<string, string> data);
  account_types GetType ();
  bool IsType (account_types query);
  string EquivAccount (account_links to_follow);
  acc_info EquivAccountInfo (account_links to_follow);
  financial GetCurrentBalance ();
  bool IsBudget ();
  bool IsYearly ();
  bool IsClosed ();
  bool IsFound  ();
  string GetTextName ();
};

account_types GetAccountTypeByTypeName (string rulename);
bool IsDebitPositiveByAccountType (account_types this_type);

// deprecated, if possible (should be replaced with edit_listview, or the new functions below)
void AddAccountAmount (TEdit * edit_account, TEdit * edit_amount, TListView * list_amounts, int account_rule);
void DelAccountAmount (TEdit * edit_account, TEdit * edit_amount, TListView * list_amounts, int account_rule);
void ModAccountAmount (TEdit * edit_account, TEdit * edit_amount, TListView * list_amounts, int account_rule);
void SetAccountAmount (TEdit * edit_account, TEdit * edit_amount, TListView * list_amounts, int account_rule);
map<string, string> GetAccountAmount (TListView * list_amounts);
//multimap<string, string> GetMultiAccountAmount(TListView * list_amounts); // Matt's code, toasted!
void GetFromList (TListView * p_list, vector<vector<string> > & p_vec);

// cleaned-up versions:
string EquivAccount (account_links, string account_id);
acc_info EquivAccountInfo (account_links, string account_id);
string EquivAccount (account_links, acc_info account_info);
acc_info EquivAccountInfo (account_links, acc_info account_info);

// string conversions to the enums
string        AccountTypeToString (account_types account_type);
account_types StringToAccountType (string account_type_name);
string        LinkTypeToString    (account_links link_type);
account_links StringToLinkType    (string link_type_name);

// not deprecated, i think -- but trying to make it so!
set<string> GetRequiredAccountReferences(string account_type, bool is_budget);

// enum versions (removed old validate function entirely)
map<account_links, pair<account_types, bool> > GetRequiredAccountReferences(account_types account_type, bool is_budget);
bool ValidateAccountReferences(map<account_links, acc_info> & acc_refs, account_types account_type, bool is_autocreate, bool is_closed, bool is_budget);

void MaskSetup   (TEdit * box);

// in order 'id, parts, mask, info'
string IdByParts (map<string, string> parts, bool is_budget);
string IdByMask  (TEdit * box, bool is_budget);
string IdByInfo  (acc_info info);
map<string, string> PartsById   (string id);
map<string, string> PartsByMask (TEdit * box);
map<string, string> PartsByInfo (acc_info info);
void MaskById    (TEdit * box, string id);
void MaskByParts (TEdit * box, map<string, string> parts);
void MaskByInfo  (TEdit * box, acc_info info);
acc_info InfoById    (string id);
acc_info InfoByParts (map<string, string> parts, bool is_budget);
acc_info InfoByMask  (TEdit * box, bool is_budget);

class transaction_summary
{
 private:
  map<string, string> cache_entity_id_to_name;
  map<string, acc_info> cache_account_id_to_info;
  map<string, map<string, financial> > data_account_entity_amount;
  void      AddFinancial (string account, string entity, financial amount);
  void      SubFinancial (string account, string entity, financial amount);
  financial GetFinancial (string account, string entity);
  void      MergeCaches  (const transaction_summary & rhs);
 public:
  transaction_summary ();
  transaction_summary (const vector<map<string, string> > & input_data);
  void RawInput (const vector<map<string, string> > & input_data);
  bool ChangeAmountTo (string account, string entity, financial amount, bool do_magic);
  bool ChangeEntityTo (string old_entity, string new_entity);
  bool BalanceAccount (string account, string entity);
  financial BalanceAmount ();
  financial AbsBalanceAmount ();
  transaction_summary & operator += (const transaction_summary & rhs);
  transaction_summary & operator -= (const transaction_summary & rhs);
  vector<map<string, string> > GetPreviewOutput (bool show_zero_values);
  vector<map<string, string> > GetFinalOutput (map<string, string> & initial_row);
  map<string, financial>       GetAccountOutput ();
  map<string, map<account_types, financial> > GetEntityTypeAmount ();
  map<account_types, map<string, financial> > GetTypeEntityAmount ();
  acc_info GetCachedAccountInfo (string account_id);
  string   GetCachedEntityName (string entity_id);
  unsigned int GetCountEntitiesOut ();
  unsigned int GetCountEntitiesIn ();
  void ResetAccountCache ();
  void ResetEntityCache ();
  void TrimZeroAmounts ();
  void RemoveEntities ();
  financial GetEntityPayment (string entity);
};
transaction_summary operator + (const transaction_summary & lhs, const transaction_summary & rhs);
transaction_summary operator - (const transaction_summary & lhs, const transaction_summary & rhs);

int ArrangeControlsInline (TWinControl * container, bool is_vertical, int secondary_padding);
void SetupControlsForInline (TWinControl * container, list<TWinControl*> & useful_controls);

TComponent * FindComponentInsensitive (TComponent * owner, string name);
TMenuItem * FindMenuItemByNameInsensitive (TMenuItem * parent, string name);
vector<string> RecursiveMenuItemName (TMenuItem * leaf, bool stripped);
string FullyQualifiedMenuItemName (TMenuItem * leaf);

TQRPrinter *InitTQRPrinter();

// used in sql statements by some of the above functions (InfoById and InfoByParts directly, others indirectly)
#define acc_info_select_statement string(\
        "select accounts.id acc_id, account_types.item acc_type, accounts.is_budget acc_budget, " \
        "accounts.is_closed acc_closed, accounts.is_yearly acc_yearly, " \
        "acc_level_one.short_item one, acc_level_two.short_item two, " \
        "acc_level_three.short_item three, acc_level_four.short_item four, " \
        "accounts.actual_account_id, accounts.balance_account_id, " \
        "accounts.budget_account_id, accounts.cash_account_id, " \
        "accounts.repay_account_id, accounts.pending_account_id " \
        "from accounts " \
        "inner join acc_level_one   on accounts.acc_level_one_id   = acc_level_one.id   " \
        "inner join acc_level_two   on accounts.acc_level_two_id   = acc_level_two.id   " \
        "inner join acc_level_three on accounts.acc_level_three_id = acc_level_three.id " \
        "inner join acc_level_four  on accounts.acc_level_four_id  = acc_level_four.id  " \
        "inner join account_types on accounts.account_type_id = account_types.id ")

// ZIP code functions
bool CountyLookupByState(string &state_name, string &state_id, string &county_sql, TComboBox * county_combo);
bool ZipLookup(string zip_code, vector<map<string, string> > &zip_vec, string &state_name, string &county_name,
               string &preferred_city_name, TComboBox *state_combo, TComboBox *county_combo, TComboBox *city_combo);

// marginally less hackish permission stuff
set<string> GetPermissions(string permission_prefix, string permission_suffix, string key, string value, string table, string sql);
// hackish permission stuff
set<string> GetGroupPermissions(string permission_prefix, string permission_suffix, string key, string value);
set<string> GetSpecialGroupPermissions(string permission_suffix);
set<string> GetUsersWithPermissions(string section, string key, string value);
set<string> GetUsersWithPermissions(string section, string key = ""); // key is VIEW, UPDATE, or INSERT for table.* permissions; default (for admin, special, etc.) is ENABLED
set<string> GetUsersWithGroupPermissions(string prefix, string suffix, string group_id, string key = ""); // ditto
bool HasLocationPermissions(const set<string> &perm_group_ids, string location_name);

// other (from tiny-record stuff)
void ArrangeByTabOrder(TWinControl * p, bool vertical);
void ArrangeByTag(TToolBar * p, bool vertical);
TControl* FindNearestParentOfType (TControl* Child, TMetaClass* Classtype);

TColor AverageColor(TColor A, TColor B);

// jhaley 20101229: New person_notes code to enable easy use on appointments and anywhere else it might be needed.
bool CountPersonBlocksAndWarnings(int &numblocks, int &numwarnings, string person_id, set<string> &group_ids, string min_current_id);
void BlockOverrideErrorMessage();

#endif

// EOF

