#include "edit_listview.h"
#include "control_break_objects.h"
#include "values.h"

// storage for statics
TLVSelectItemEvent  edit_listview::funcSelectItem;
TLVColumnClickEvent edit_listview::funcColumnClick;
TNotifyEvent        edit_listview::funcDoubleClick;
//-----------------------------------------------------------------------
int __stdcall EditListviewCompare(long Param1, long Param2, long columnToSort)
{
 TListItem         * row        = NULL;
 TCustomListView   * listview   = NULL;
 edit_listview     * controller = NULL;

 row        = (TListItem *)Param1;
 listview   = row->ListView;
 controller = (edit_listview *)listview->Tag;

 if (controller)
  return controller->compare_values (Param1, Param2, columnToSort);
 return 0;
}
//-----------------------------------------------------------------------
bool edit_listview::data_to_listview (TListItem * row_to_fill, map<string, string> & row_data)
{
 string temp_value;
 string temp_column;
 if (row_to_fill)
 {
  row_to_fill->SubItems->Clear();
  for (multimap<int, string>::iterator i_order = visible_column_order.begin(); i_order != visible_column_order.end(); i_order++)
  {
   temp_column = i_order->second;
   temp_value = FormatString (row_data[temp_column], column_information[temp_column].column_show_type);
   if (i_order == visible_column_order.begin())
    row_to_fill->Caption = temp_value.c_str();
   else
    row_to_fill->SubItems->Add(temp_value.c_str());
  }
  return true;
 } else
  return false;
}
//-----------------------------------------------------------------------
void edit_listview::recalc_column_order ()
{
 visible_column_order.clear();
 for (map<string, column_structure>::iterator i_column = column_information.begin(); i_column != column_information.end(); i_column++)
  if (i_column->second.column_visible)
   visible_column_order.insert(pair<int, string>(i_column->second.column_position, i_column->second.column_name));
 return;
}
//-----------------------------------------------------------------------
void edit_listview::refresh_columns ()
{
 if (listview)
 {
  // jhaley 20110809: rewritten to use indices instead of pointers
  TListColumn * temp_column_pointer;
  sort_index = 0;
  listview->Columns->Clear();
  visible_column_index.clear();
  for (multimap<int, string>::iterator i_order = visible_column_order.begin(); i_order != visible_column_order.end(); ++i_order)
  {
   column_structure & temp_column_structure = column_information[i_order->second];
   temp_column_pointer = listview->Columns->Add();
   temp_column_pointer->Caption = temp_column_structure.column_title.c_str();
   temp_column_pointer->Width = temp_column_structure.column_width;
   if (temp_column_structure.column_show_type == amountString ||
       temp_column_structure.column_show_type == nonzeroAmountString ||
       temp_column_structure.column_show_type == intString ||
       temp_column_structure.column_show_type == statString)
   {
    temp_column_pointer->Alignment = taRightJustify;
   }
   temp_column_structure.column_index = temp_column_pointer->Index;
   visible_column_index[temp_column_pointer->Index] = temp_column_structure.column_name;
  }
 }
}
//-----------------------------------------------------------------------
void edit_listview::refresh_data ()
{
 // make sure all the data is up to date (all row pointers should already exist)
 for (map<TListItem *, map<string, string> >::iterator i_row = listview_data.begin(); i_row != listview_data.end(); ++i_row)
  data_to_listview (i_row->first, i_row->second);

 // re-sort the data, keep current order
 // jhaley 20110809: AV's coming from here were due to dangling pointers to TListColumn structures, which are dynamic
 // and are destroyed by the TListView control at its sole whim whenever certain operations occur. The comparison that
 // was previously being done in refresh_columns was unreliable as it used a pointer to a potentially freed structure,
 // and thus could leave sort_column unchanged as a result.

 sort_by_column_index (sort_index, false);
}
//-----------------------------------------------------------------------
edit_listview::edit_listview() : listview_data(), column_information(), visible_column_order(), visible_column_index()
{
 listview                         = NULL;
 sort_index                       = 0;
 callback_form                    = NULL;
 callback_when_row_selected       = NULL;
 callback_when_row_double_clicked = NULL;
}
//-----------------------------------------------------------------------
edit_listview::~edit_listview ()
{
 if (listview)
 {
  listview->OnSelectItem  = NULL;
  listview->OnColumnClick = NULL;
  listview->OnDblClick    = NULL;
 }

 callback_form                    = NULL;
 callback_when_row_selected       = NULL;
 callback_when_row_double_clicked = NULL;

 sort_index = 0;
 listview   = NULL;
}
//-----------------------------------------------------------------------
bool edit_listview::setup_listview (TListView * associated_listview, bool is_multiselect)
{
 listview = associated_listview;
 if (listview)
 {
  listview->Tag               = int(this);
  listview->ViewStyle         = vsReport;
  listview->ReadOnly          = true;
  listview->RowSelect         = true;
  listview->HideSelection     = false;
  listview->ShowColumnHeaders = true;
  listview->MultiSelect       = is_multiselect;
  listview->SmallImages       = imagesCurrent;
  listview->OnSelectItem      = funcSelectItem;
  listview->OnColumnClick     = funcColumnClick;
  listview->OnDblClick        = funcDoubleClick;

  return true;
 } else
  return false;
}
//-----------------------------------------------------------------------
bool edit_listview::add_or_modify_column (string column_name, string column_title, bool column_visible, int column_position, int column_width, sort_type column_sort_type, show_type column_show_type)
{
 column_structure & this_column = column_information[column_name];
 this_column.column_name        = column_name;
 this_column.column_title       = column_title;
 this_column.column_visible     = column_visible;
 this_column.column_position    = column_position;
 this_column.column_width       = column_width;
 this_column.column_sort_type   = column_sort_type;
 this_column.column_show_type   = column_show_type;

 recalc_column_order();
 refresh_columns();
 refresh_data();
 return true;
}
//-----------------------------------------------------------------------
bool edit_listview::remove_column (string column_name)
{
 column_information.erase(column_name);

 recalc_column_order();
 refresh_columns();
 refresh_data();
 return true;
}
//-----------------------------------------------------------------------
bool edit_listview::clear_columns ()
{
 column_information.clear();

 recalc_column_order();
 refresh_columns();
 refresh_data();
 return true;
}
//-----------------------------------------------------------------------
bool edit_listview::sort_by_column_name (string column_name)
{
 map<string, column_structure>::iterator i_column = column_information.find(column_name);
 if (i_column != column_information.end())
 {
  sort_index = i_column->second.column_index;
  sort_by_column_index (sort_index, false);
  return true;
 } else
  return false;
}
//-----------------------------------------------------------------------
//bool edit_listview::sort_by_column_pointer (TListColumn * column_pointer, bool flip_sort_order)
bool edit_listview::sort_by_column_index (int column_index, bool flip_sort_order)
{
  // jhaley 20110809: rewritten to use indices instead of pointers
  if (listview)
  {
    map<string, column_structure>::iterator i_col;

    // jhaley 20110809: safer access into column structures
    if(column_index >= 0 && (size_t)column_index < visible_column_index.size())
    {
      string colstr = visible_column_index[column_index];
      i_col = column_information.find(colstr);

      if(i_col == column_information.end())
        return false; // jhaley: failure, that column is not in the map!
    }
    else
      return false; // jhaley: failure, index out of range!

    bool &column_sort_ascending = i_col->second.column_sort_ascending;

    if (flip_sort_order)
      column_sort_ascending = !column_sort_ascending;

    if (listview->SmallImages)
    {
      for (int i = 0; i < listview->Columns->Count; i++)
        listview->Column[i]->ImageIndex = -1;
      listview->Column[column_index]->ImageIndex = (column_sort_ascending ? 2 : 1);
    }

    listview->CustomSort(EditListviewCompare, column_index);

    return true;
  }
  else
    return false;
}
//-----------------------------------------------------------------------
int edit_listview::compare_values (long param_a, long param_b, long column_index)
{
 TListItem * item_a, * item_b;
 AnsiString string_a, string_b;

 item_a =(TListItem *)param_a;
 item_b =(TListItem *)param_b;

 if (column_index > 0)
 {
  string_a = item_a->SubItems->Strings[column_index - 1];
  string_b = item_b->SubItems->Strings[column_index - 1];
 } else
 {
  string_a = item_a->Caption;
  string_b = item_b->Caption;
 }

 // do the comparison, and flip it if we're not sorting ascending
 // (CompareByType assumes ascending)
 return CompareByType(string_a.c_str(), string_b.c_str(), column_information[visible_column_index[column_index]].column_sort_type) * (column_information[visible_column_index[column_index]].column_sort_ascending? 1 : -1);
}
//-----------------------------------------------------------------------
// jhaley 20100910: Added for smarter column sizing in SetupTransactionSummaryView
int edit_listview::get_listview_width()
{
  if (listview)
    return listview->Width;
  else
    return 0;
}
//-----------------------------------------------------------------------
bool edit_listview::setup_callbacks (TObject * form, InformForm when_row_selected, InformForm when_row_double_clicked)
{
 callback_form                    = form;
 callback_when_row_selected       = when_row_selected;
 callback_when_row_double_clicked = when_row_double_clicked;
 return true;
}
//-----------------------------------------------------------------------
bool edit_listview::clear_data ()
{
 if (listview)
 {
  listview->Items->Clear();
  listview_data.clear();
  return true;
 } else
  return false;
}
//-----------------------------------------------------------------------
bool edit_listview::import_data (const vector<map<string, string> > & imported_data)
{
 if (listview)
 {
  TListItem * temp_item_pointer = NULL;
  for (vector<map<string, string> >::const_iterator i_row = imported_data.begin(); i_row != imported_data.end(); i_row++)
  {
   temp_item_pointer = listview->Items->Add();
   listview_data.insert(pair<TListItem *, map<string, string> >(temp_item_pointer, *i_row));
  }

  refresh_data();
  return true;
 } else
  return false;
}
//-----------------------------------------------------------------------
bool edit_listview::export_data (vector<map<string, string> > & exported_data)
{
 exported_data.clear();
 for (map<TListItem *, map<string, string> >::iterator i_row = listview_data.begin(); i_row != listview_data.end(); i_row++)
  exported_data.push_back(i_row->second);
 return true;
}
//-----------------------------------------------------------------------
bool edit_listview::export_column (string column_name, vector<string> & column_data, bool selected_only)
{
 column_data.clear();
 for (map<TListItem *, map<string, string> >::iterator i_row = listview_data.begin(); i_row != listview_data.end(); i_row++)
  if (!selected_only || i_row->first->Selected)
   column_data.push_back((i_row->second)[column_name]);
 return true;
}
//-----------------------------------------------------------------------
bool edit_listview::export_selected_data (vector<map<string, string> > & exported_data)
{
 exported_data.clear();
 for (map<TListItem *, map<string, string> >::iterator i_row = listview_data.begin(); i_row != listview_data.end(); i_row++)
  if (i_row->first->Selected)
   exported_data.push_back(i_row->second);
 return true;
}
//-----------------------------------------------------------------------
bool edit_listview::get_current_row (map<string, string> & row_data)
{
 row_data.clear();
 if (listview)
 {
  TListItem * selected_row = listview->Selected;
  if (selected_row)
  {
   map<TListItem *, map<string, string> >::iterator i_row = listview_data.find(selected_row);
   if (i_row != listview_data.end())
   {
    row_data = i_row->second;
    return true;
   } else
    return false;
  } else
   return false;
 } else
  return false;
}
//-----------------------------------------------------------------------
bool edit_listview::add_or_modify_row_data (map<string, string> & row_data, bool move_to_next_row)
{
 if (listview)
 {
  TListItem * this_row = listview->Selected;
  if (!this_row)
   this_row = listview->Items->Add();
  if (this_row)
  {
   listview_data[this_row] = row_data;
   data_to_listview (this_row, row_data);

   sort_by_column_index (sort_index, false);
   this_row->Selected = true;

   if (move_to_next_row)
    go_to_next_row();
   return true;
  } else
   return false;
 } else
  return false;
}
//-----------------------------------------------------------------------
bool edit_listview::remove_current_row (bool move_to_next_row)
{
 if (listview)
 {
  if (listview->SelCount)
  {
   TItemStates selected = TItemStates() << isSelected;
   TListItem * Item = listview->Selected;
   TListItem * Next = NULL;
   while (Item)
   {
    Next = listview->GetNextItem(Item, sdAll, selected);
    map<TListItem *, map<string, string> >::iterator i_row = listview_data.find(Item);
    if (i_row != listview_data.end())
     listview_data.erase(i_row);
    Item->Delete();
    Item = Next;
   }
   if (move_to_next_row)
    go_to_next_row();
   return true;
  }
 }
 return false;
}
//-----------------------------------------------------------------------
bool edit_listview::select_items_where (string key, string value)
{
 bool is_found = false;
 map<string, string>::iterator i_pair;

 for (map<TListItem *, map<string, string> >::iterator i_item = listview_data.begin(); i_item != listview_data.end(); i_item++)
 {
  i_pair = i_item->second.find(key);
  if (i_pair != i_item->second.end())
  {
   if (i_pair->second == value)
   {
    is_found = true;
    i_item->first->Selected = true;
    i_item->first->MakeVisible(false);
   }
  }
 }
 return is_found;
}
//-----------------------------------------------------------------------
void edit_listview::user_clicked_column (TListColumn * column_pointer)
{
  // jhaley 20110809: rewritten to use indices instead of pointers
  if (column_pointer->Index == sort_index)
    sort_by_column_index (column_pointer->Index, true);
  else
  {
    sort_index = column_pointer->Index;
    sort_by_column_index (column_pointer->Index, false);
  }
}
//-----------------------------------------------------------------------
void edit_listview::user_selected_row ()
{
 string class_name = "edit_listview";
 AV_TRY
  if (callback_form && callback_when_row_selected)
   (callback_form->*callback_when_row_selected)();
 AV_CATCH
 return;
}
//-----------------------------------------------------------------------
void edit_listview::user_double_clicked_row ()
{
 string class_name = "edit_listview";
 AV_TRY
  if (callback_form && callback_when_row_double_clicked)
   (callback_form->*callback_when_row_double_clicked)();
 AV_CATCH
}
//-----------------------------------------------------------------------
void edit_listview::go_to_next_row()
{
 // iff one selection
 if ((!listview->MultiSelect) && (listview->Selected))
 {
  // and that selection is not the last item in the listview
  if (listview->Items->Count - listview->Selected->Index > 1)
   // then select the next item
   listview->Items->Item[listview->Selected->Index + 1]->Selected = true;
  listview->Selected->MakeVisible(false);
 }
}
//-----------------------------------------------------------------------
bool edit_listview::is_future_duplicate (map<string, string> & row_data, set<string> & unique_columns)
{
 // no unique constraints? well, that'd fuck stuff up later, so ...
 // if no unique constraints, there cannot be duplicates. or they all are. but in this case, there are none.
 if (unique_columns.empty())
  return false;

 map<string, string> current_row_data;
 bool duplicate_row_found = false;

 for (map<TListItem *, map<string, string> >::iterator i_row = listview_data.begin(); i_row != listview_data.end(); i_row++)
 {
  duplicate_row_found = true;
  for (set<string>::iterator i_col = unique_columns.begin(); i_col != unique_columns.end(); i_col++)
   if ((i_row->second)[(*i_col)] != row_data[(*i_col)])
    duplicate_row_found = false;
  if (duplicate_row_found)
   break; // could have gone in the 'for' statement, but this might be more obvious ...
 }

 if (duplicate_row_found)
 {
  if (get_current_row(current_row_data))
  {
   // if this would replace an existing row, and would change the fields uniqueness depends on,
   // then this cannot have conflicted with itself. thus, changing the row would turn it into a duplicate.
   for (set<string>::iterator i_col = unique_columns.begin(); i_col != unique_columns.end(); i_col++)
   {
    if (row_data[*i_col] != current_row_data[*i_col])
     return true; // go right ahead and exit ... one of the unique-constraint columns has changed -- dupe!
   }
  } else
  {
   // if there is no current row, and we found a dupe, well ... there's no ambiguity. it's a dupe.
   return true;
  }
 }
 return false;
}
//-----------------------------------------------------------------------
void edit_listview::decrease_rank (string _f)
{
 change_rank (_f, true);
}
//-----------------------------------------------------------------------
void edit_listview::increase_rank (string _f)
{
 change_rank (_f, false);
}
//-----------------------------------------------------------------------
void edit_listview::change_rank (string _f, bool _dec)
{
 int _mod = (_dec ? -1 : +1);
 int _rev = (_dec ? +1 : -1);
 if (listview)
 {
  if (listview->SelCount)
  {
   // SECTION: make sure we don't fuck up
   int min_rank = MAXINT;
   int max_rank = 0;
   int min_sel = MAXINT;
   int max_sel = 0;

   // gonna reuse these ...
   TItemStates selected = TItemStates() << isSelected;
   TListItem * Item = NULL;

   int k = listview->Items->Count;
   for (int i = 0; i < k; i++)
   {
    Item = listview->Items->Item[i];
    map<TListItem *, map<string, string> >::iterator i_row = listview_data.find(Item);
    if (i_row != listview_data.end())
    {
     int u = StringToInt((i_row->second)[_f]);
     min_rank = min(min_rank, u);
     max_rank = max(max_rank, u);
     if (Item->Selected)
     {
      min_sel = min(min_sel, u);
      max_sel = max(max_sel, u);
     }
    }
   }

   if (_dec && min_sel == min_rank)
    return;
   if (!_dec && max_sel == max_rank)
    return;

   // SECTION: go through all selected items, and update rows to reflect new ranks
   set<int> gaps_created;
   set<int> dupes_created;

   Item = listview->Selected;
   while (Item)
   {
    map<TListItem *, map<string, string> >::iterator i_row = listview_data.find(Item);
    if (i_row != listview_data.end())
    {
     int u = StringToInt((i_row->second)[_f]);
     gaps_created.insert(u);
     dupes_created.insert(u + _mod);
     (i_row->second)[_f] = IntToString(u + _mod);
     data_to_listview (Item, i_row->second);
    }
    Item = listview->GetNextItem(Item, sdAll, selected);
   }

   set<int> remaining_gaps;
   set_difference(gaps_created.begin(), gaps_created.end(),
               dupes_created.begin(), dupes_created.end(),
               insert_iterator<set<int, less<int> > >(remaining_gaps, remaining_gaps.begin()));
   gaps_created.swap(remaining_gaps);

   // SECTION: then go back through all non-selected items and clean them up, by finding the next available gap for each.
   for (int i = 0; i < k; i++)
   {
    Item = listview->Items->Item[i];
    if (!Item->Selected)
    {
     map<TListItem *, map<string, string> >::iterator i_row = listview_data.find(Item);
     if (i_row != listview_data.end())
     {
      int u = StringToInt((i_row->second)[_f]);
      if (dupes_created.find(u) != dupes_created.end())
      {
       // we shouldn't be able to find it, so ... we should always find the next-greater value.
       set<int>::iterator gap = lower_bound(gaps_created.begin(), gaps_created.end(), u);
       if (!_dec)
       {
        if (gap != gaps_created.begin())
         gap--;
       }
       if (gap != gaps_created.end())
       {
        dupes_created.erase(u);
        gaps_created.erase(*gap);
        (i_row->second)[_f] = IntToString(*gap);
        data_to_listview (Item, i_row->second);
       }
      }
     }
    }
   }

   // best to resort, just in case?
   sort_by_column_index (sort_index, false);

   if (listview->Selected)
    listview->Selected->MakeVisible(false);
  }
 }
}

//-----------------------------------------------------------------------
bool temp_compare_items_by_rank (const pair<TListItem*, int> & a, const pair<TListItem*, int> & b)
{
 return (a.second < b.second);
}
void edit_listview::normalize_rank(string _f)
{
 if (listview)
 {
  vector<pair<TListItem *, int> > items_to_ranks;
  for (map<TListItem *, map<string, string> >::iterator i = listview_data.begin(); i != listview_data.end(); i++)
   items_to_ranks.push_back(pair<TListItem *, int>(i->first, StringToInt(i->second[_f])));
  sort(items_to_ranks.begin(), items_to_ranks.end(), temp_compare_items_by_rank);
  int n = 0;
  for (vector<pair<TListItem *, int> >::iterator i = items_to_ranks.begin(); i != items_to_ranks.end(); i++, n++)
  {
   listview_data[i->first][_f] = IntToString(n);
   data_to_listview (i->first, listview_data[i->first]);
  }

  // best to resort, just in case?
  sort_by_column_index (sort_index, false);

  // this_row->Selected = true;
 }
}
//-----------------------------------------------------------------------

