#ifndef edit_listview_h
#define edit_listview_h

#include "prometheus_util.h" // well so much for modularity.  I don't much care though; I never use
                             // edit_listview for anything.  but FYI:  this ought to be made independent of
                             // Prometheus at some point; imagesCurrent etc. could be static members of class
                             // edit_listview and set in codePrometheus similar to what it does for filter
#include "Filter.h" // filter does most of what we want ... but it wasn't at all designed for this,
                    // and deriving from it would be nasty. both of these should have been derived
                    // originally from some sort of listview handler or something. oh well.
                    // now we just want access to the classes designed for filter, functions, etc.
#include "util.h"

using namespace std;

class column_structure
{
 public:
  // data
  // set by programmer
  string column_name;
  string column_title;
  bool   column_visible;
  int    column_position;
  int    column_width;
  sort_type column_sort_type;
  show_type column_show_type;
  // internal
  bool   column_sort_ascending;
  int    column_index; // jhaley 20110809: replace column_pointer

  // constructor
  column_structure () : column_visible(true), column_position(0), column_width(0), column_sort_type(as_text), column_show_type(prettyString), column_sort_ascending(true), /*column_pointer(NULL)*/ column_index(0) {};
};

class edit_listview
{
 public: // and static
  static TLVSelectItemEvent  funcSelectItem;
  static TLVColumnClickEvent funcColumnClick;
  static TNotifyEvent        funcDoubleClick;
 private:
  bool data_to_listview        (TListItem * row_to_fill, map<string, string> & row_data);
  void recalc_column_order     ();
  void refresh_columns         ();
  void refresh_data            ();
  void go_to_next_row          ();
 protected:
  TListView   * listview;
  int           sort_index;    // jhaley 20110809: replace sort_column
  TObject     * callback_form;
  InformForm    callback_when_row_selected;
  InformForm    callback_when_row_double_clicked;

  map<TListItem *, map<string, string> > listview_data;
  map<string, column_structure> column_information;
  multimap<int, string>         visible_column_order;
  map<int, string>              visible_column_index;
 public:
  edit_listview ();
  ~edit_listview ();

  bool setup_listview          (TListView * associated_listview, bool is_multiselect = false);
  bool add_or_modify_column    (string column_name, string column_title, bool column_visible, int column_position, int column_width, sort_type column_sort_type, show_type column_show_type);
  bool remove_column           (string column_name);
  bool clear_columns           ();
  bool sort_by_column_name     (string column_name);
  bool sort_by_column_index    (int column_index, bool flip_sort_order); // jhaley 20110809: changed to use index
   int compare_values          (long param_a, long param_b, long column_index);
   int get_listview_width      (); // jhaley

  bool setup_callbacks         (TObject * form, InformForm when_row_selected, InformForm when_row_double_clicked);

  bool clear_data              ();
  bool import_data             (const vector<map<string, string> > & imported_data);
  bool export_data             (vector<map<string, string> > & exported_data);
  bool export_column           (string column_name, vector<string> & column_data, bool selected_only);
  bool export_selected_data    (vector<map<string, string> > & exported_data);

  bool get_current_row         (map<string, string> & row_data);
  bool add_or_modify_row_data  (map<string, string> & row_data, bool move_to_next_row);
  bool remove_current_row      (bool move_to_next_row);

  bool select_items_where      (string key, string value);

  void user_clicked_column     (TListColumn * column_pointer);
  void user_selected_row       ();
  void user_double_clicked_row ();

  bool is_future_duplicate     (map<string, string> & row_data, set<string> & unique_columns);

  template<class T>
  T & get_column_sum(string column_name, T & addition_object)
  {
   string field_value;
   if (column_information.find(column_name) != column_information.end())
   {
    for (map<TListItem *, map<string, string> >::iterator i_row = listview_data.begin(); i_row != listview_data.end(); i_row++)
    {
     field_value = i_row->second[column_name];
     addition_object(field_value);
    }
   }
   return addition_object;
  }

  // rank-related stuff ... _f should probably be made part of the whole class? but what if we have several rank fields?
  void change_rank (string _f, bool _dec);
  void decrease_rank (string _f);
  void increase_rank (string _f);
  void normalize_rank(string _f);
};


#endif