//---------------------------------------------------------------------------

#include <vcl.h>
#pragma hdrstop

#include "codeRecordDenialDOS.h"
//---------------------------------------------------------------------------
#pragma package(smart_init)
#pragma link "codeRecordGeneric"
#pragma resource "*.dfm"
TformRecordDenialDOS *formRecordDenialDOS;
//---------------------------------------------------------------------------
__fastcall TformRecordDenialDOS::TformRecordDenialDOS(TComponent* Owner, map<string,string> p_parameters)
        : TformRecordGeneric(Owner, p_parameters, string("denial_items")), last_billed_amount("")
{
  Initialize();
}
//---------------------------------------------------------------------------
bool TformRecordDenialDOS::PutDataOnScreen()
{
  string letter_sql = "select date_processed from denial_letters where id = '" + initial_state["denial_letter_id"] + "' ";
  string letter_date_processed_str = GetOneField(dbDatabase, letter_sql);

  // This date is the date of the rollout for the new denial letter system
  Pdate cutoff_date = Pdate("7/21/2010", mdy | yyyy);
  Pdate letter_date_processed = Pdate(letter_date_processed_str, mdy | yyyy);

  if (!is_new_record && letter_date_processed <= cutoff_date)
  {
    oldLetterInfoText = "Since this denial of service item predates " + cutoff_date.OutDate(mdy | yyyy) +
                        ", the information on this screen may appear inaccurate or confusing.";
  }
  else
    oldLetterInfoText = "";

  if (initial_state["begin_date_of_service"] != "")
    dateBegin->Date = Pdate(initial_state["begin_date_of_service"], mdy | yyyy).OutDate();
  else
    dateBegin->Date = Date();

  if (initial_state["end_date_of_service"] != "")
    dateEnd->Date = Pdate(initial_state["end_date_of_service"], mdy | yyyy).OutDate();
  else
    dateEnd->Date = Date();

  editBilledAmount->Text  = AmountToString(StringToAmount(initial_state["billed_amount"])).c_str();
  editPatient->Text       = AmountToString(StringToAmount(initial_state["patient_amount"])).c_str();
  editPayableAmount->Text = AmountToString(StringToAmount(initial_state["payable_amount"])).c_str();

  CreateLink(txtLetter, NULL, this, initial_state["denial_letter_id"], "denial_letters");

  TformRecordGeneric::PutDataOnScreen();
  return true;
}
//---------------------------------------------------------------------------
/* IsValidOnScreen should validate that amount is a number */
bool TformRecordDenialDOS::GetDataFromScreen()
{
  ending_state["id"] = initial_state["id"];
  ending_state["denial_letter_id"] = initial_state["denial_letter_id"];
  ending_state["begin_date_of_service"] = Pdate(dateBegin->Date).OutDate(mdy | yyyy);
  ending_state["end_date_of_service"] = Pdate(dateEnd->Date).OutDate(mdy | yyyy);
  ending_state["billed_amount"] = editBilledAmount->Text.c_str();
  ending_state["patient_amount"] = editPatient->Text.c_str();
  ending_state["payable_amount"] = editPayableAmount->Text.c_str();

  TformRecordGeneric::GetDataFromScreen();
  return true;
}
//---------------------------------------------------------------------------
bool TformRecordDenialDOS::ScreenLock()
{
  panelGroup->Enabled = false;
  TformRecordGeneric::ScreenLock();
  return true;
}
//---------------------------------------------------------------------------
bool TformRecordDenialDOS::ScreenUnlock()
{
  panelGroup->Enabled = true;
  TformRecordGeneric::ScreenUnlock();
  return true;
}
//---------------------------------------------------------------------------
bool TformRecordDenialDOS::IsValidOnScreen()
{
  string status_id = GetOneField(dbDatabase, "select denial_status_id from denial_letters where id = '"
                                             + initial_state["denial_letter_id"] + "' ");

  // jhaley 20100706: verify fields
  if (!IsAmount(editBilledAmount->Text.c_str()) || editBilledAmount->Text == "")
  {
     PSay("Please enter a valid billed amount.", m_invalid_data, MB_OK);
     editBilledAmount->SetFocus();
     return false;
  }
  else if (!IsAmount(editPatient->Text.c_str()) || editPatient->Text == "")
  {
    PSay("Please enter a valid patient owed amount.", m_invalid_data, MB_OK);
    editPatient->SetFocus();
    return false;
  }
  else if (!IsAmount(editPayableAmount->Text.c_str()) || editPayableAmount->Text == "") // jhaley 20100715: added payable amount
  {
    PSay("Please enter a valid payable amount.", m_invalid_data, MB_OK);
    editPayableAmount->SetFocus();
    return false;
  }
  else if (StringToFinancial(editPatient->Text.c_str()) > StringToFinancial(editBilledAmount->Text.c_str()))
  {
    PSay("The patient owed amount should not exceed the billed amount.", m_invalid_data, MB_OK);
    editPayableAmount->SetFocus();
    return false;
  }
  else if (status_id == lookupTables->IdOf("denial_status_types", "Denied"))
  {
    // jhaley 20100716: The letter this DOS belongs to has "Denied" status, so the payable amount should be 0.
    // Compare both as financials.
    // Note that it will start out as 0 when coming here from a denial letter with denied status, but that
    // doesn't keep anybody from trying to change it on this form.
    if (StringToFinancial(editPayableAmount->Text.c_str()) != StringToFinancial("0.0"))
    {
      PSay("A DOS item for a letter with Denied status should have a payable amount of $0.00.", m_invalid_data, MB_OK);
      editPayableAmount->SetFocus();
      return false;
    }
  }
  else if (status_id == lookupTables->IdOf("denial_status_types", "Resolved"))
  {
    // jhaley 20100716: The letter this DOS belongs to has "Resolved" status, so the payable amount should be
    // less than or equal to the patient responsibility.
    if (StringToFinancial(editPayableAmount->Text.c_str()) > StringToFinancial(editPatient->Text.c_str()))
    {
      PSay("A DOS item for a letter with Resolved status should have a payable amount equal to or less than the patient's owed amount.",
           m_invalid_data, MB_OK);
      editPayableAmount->SetFocus();
      return false;
    }
  }

  // verify date ordering
  Pdate dateFrom, dateTo;
  dateFrom = Pdate(dateBegin->Date);
  dateTo   = Pdate(dateEnd->Date);
  if (dateFrom > dateTo)
  {
     PSay("The beginning date should fall at or before the ending date.", m_invalid_data, MB_OK);
     dateBegin->SetFocus();
     return false;
  }

  return true;
}
//---------------------------------------------------------------------------
bool TformRecordDenialDOS::BeforeSave()
{
  AtlasClear();

  // jhaley 20100708: send out a "fake" Atlas notification on person_id for the sake of the
  // person record denials filter.
  string person_id = GetOneField(dbDatabase, "select denials.person_id from denials "
                                             "inner join denial_letters on denial_letters.denial_id = denials.id "
                                             "where denial_letters.id = '" + ending_state["denial_letter_id"] + "' ");
  if (!person_id.empty())
    AtlasAdd("denial_items.person_id", person_id);

  return TformRecordGeneric::BeforeSave();
}
//---------------------------------------------------------------------------
bool TformRecordDenialDOS::AfterSave()
{
  AtlasSend();
  return TformRecordGeneric::AfterSave();
}
//---------------------------------------------------------------------------
void __fastcall TformRecordDenialDOS::editBilledAmountExit(TObject *Sender)
{
  // jhaley: If a billed amount has been entered by the user and this is attached to a Resolved letter,
  // propagate the value to the payable amount.
  string status_id = GetOneField(dbDatabase, "select denial_status_id from denial_letters where id = '"
                                             + initial_state["denial_letter_id"] + "' ");

  if (is_new_record && editBilledAmount->Text != "" && IsAmount(editBilledAmount->Text.c_str()) &&
      status_id == lookupTables->IdOf("denial_status_types", "Resolved"))
  {
    if (editPayableAmount->Text == "" ||
        (last_billed_amount != "" && StringToFinancial(editPayableAmount->Text.c_str()) == StringToFinancial(last_billed_amount)))
    {
      editPayableAmount->Text = editBilledAmount->Text;
      last_billed_amount = editBilledAmount->Text.c_str();
    }
  }
}
//---------------------------------------------------------------------------
// jhaley 20100719: added to display info bar for old denial DOS items, which may have dates or $ amounts
// which are inconsistent with the new three-step business logic for denials.
void __fastcall TformRecordDenialDOS::FormShow(TObject *Sender)
{
  TformRecordGeneric::FormShow(Sender);
  if (oldLetterInfoText != "")
  {
    InfoBarText = oldLetterInfoText;
    ShowInfoBar();
  }
}
//---------------------------------------------------------------------------

