//---------------------------------------------------------------------------

#include <vcl.h>
#pragma hdrstop

#include "launchDiabEventReport.h"
#include "reportDiabEvent.h" // note that the filename and class don't match!
#include "derivedheaders.h"
//---------------------------------------------------------------------------
#pragma package(smart_init)
#pragma link "codeSpecialGeneric"
#pragma link "codeRubberContainer"
#pragma resource "*.dfm"
TformLaunchDiabEventReport *formLaunchDiabEventReport;
//---------------------------------------------------------------------------
__fastcall TformLaunchDiabEventReport::TformLaunchDiabEventReport(TComponent* Owner)
        : TformSpecialGeneric(Owner)
{
  dateBegin->Date = Now();
  dateEnd->Date = Now();

  event_filter.SetListViewHandle(listEvents);
  event_filter.SetSecurity("events");
  event_filter.AddColumnSpec("Event Name", "events", as_text, true, 184, prettyString);
  event_filter.AddColumnSpec("Internal Id", "internal_id", as_int, false, 80);
  event_filter.FetchPreferences("launchDiabEventReport.event_filter");
  event_filter.SetIconStatus(false);
  event_filter.SetAutosizeColumns(false);
  event_filter.SetCheck(true);
  event_filter.SetHeader(true);
  event_filter.RefreshDataSet();
  event_filter.SetSQLStatement("select item, id from event_name_types");
  event_filter.ForceRefreshDataSet();
//  event_filter.AutosizeColumns();
  event_filter.CheckAll();
}
//---------------------------------------------------------------------------
void __fastcall TformLaunchDiabEventReport::btnCloseClick(TObject *Sender)
{
  Close();        
}
//---------------------------------------------------------------------------
void __fastcall TformLaunchDiabEventReport::btnRunClick(TObject *Sender)
{
  vector<string> event_names_vec = event_filter.GetMultiID();

  Processing(true);

  TIBTransaction * dbTransaction = new TIBTransaction(NULL);
  TIBDataSet * dbDataSet = new TIBDataSet(NULL);
  string sql;
  string begin_date = Pdate(dateBegin->Date).OutDate(mdy | yyyy);
  string end_date =  Pdate(dateEnd->Date).OutDate(mdy | yyyy);
  string date_title = begin_date + " - " + end_date;

  try
  {
    StdTransaction(dbTransaction, dbDatabase);
    dbDataSet->Database = dbDatabase;
    dbDataSet->Transaction = dbTransaction;
    dbDataSet->UniDirectional = false;

    sql = "select events.date_of_event, event_name_types.item event_name, "
          "event_type_types.item event_type, case "
          "when event_location_types.zip_code_id = 0 then event_location_types.item "
          "when event_location_types.item = '' then (zip_codes.city || ', ' || zip_codes.state) "
          "else (zip_codes.city || ', ' || zip_codes.state || ' - ' || event_location_types.item) end location, "
          "events.number_attended "
          "from events "
          "inner join event_name_types on events.event_name_id = event_name_types.id "
          "inner join event_type_types on events.event_type_id = event_type_types.id "
          "inner join event_location_types on events.event_location_id = event_location_types.id "
          "inner join zip_codes on event_location_types.zip_code_id = zip_codes.id "
          "where events.group_id = '" + lookupTables->IdOf("groups", "Diabetes") + "' and "
          "events.date_of_event >= '" + begin_date + "' and "
          "events.date_of_event <= '" + end_date + "' and "
          "events.current_id >= 3 and "
          "events.event_name_id in (" + VecToCommaString(event_names_vec) + ") "
          "order by events.date_of_event";

    DebugSql(this, &sql, other_forms, 9);
    dbDataSet->SelectSQL->Add(sql.c_str());
    dbDataSet->Open();
    dbDataSet->First();

    FORM_TRY
      TformDiabEventReport * report = new TformDiabEventReport(NULL);
      report->eventReport->DataSet = dbDataSet;
      report->lblDateTitle->Caption = date_title.c_str();
      report->lblUser->Caption = PrettyString(lookupTables->ValueOf("users", current_userid)).c_str();
      report->txtDate->DataSet = dbDataSet;
      report->txtDate->DataField = "date_of_event";
      report->txtName->DataSet = dbDataSet;
      report->txtName->DataField = "event_name";
      report->txtEventType->DataSet = dbDataSet;
      report->txtEventType->DataField = "event_type";
      report->txtLocation->DataSet = dbDataSet;
      report->txtLocation->DataField = "location";
      report->txtAttendance->DataSet = dbDataSet;
      report->txtAttendance->DataField = "number_attended";
      report->exprTotAttendance->Expression = "SUM(number_attended)";
      Processing(false);
      report->eventReport->Preview();
    FORM_CATCH
    dbDataSet->Close();
    dbTransaction->Commit();
  }
  catch (...)
  {
    Processing(false);
    PSay("Error running report", m_database_error, MB_OK);
    if (dbTransaction->Active)
      dbTransaction->Rollback();
  }
  delete dbTransaction;
  delete dbDataSet;
}
//---------------------------------------------------------------------------
void __fastcall TformLaunchDiabEventReport::FormShortCut(TWMKey &Msg,
      bool &Handled)
{
  Handled = true;
  if (Msg.CharCode == VK_RETURN)
    btnRun->Click();
  else
    Handled = false;
}
//---------------------------------------------------------------------------

