#include "hl7.h"

// hl7_value
hl7_value::hl7_value ()
{
}

hl7_value::hl7_value (string _value)
{
 SetValue(_value);
}

hl7_value::hl7_value (char * _value)
{
 SetValue(string(_value));
}

void hl7_value::SetPreserve (bool _preserve)
{
 preserve = _preserve;
}

bool hl7_value::GetPreserve () const
{
 return preserve;
}

void hl7_value::SetValue (string _value)
{
 value = _value;
 values.clear();
}

void hl7_value::SetValue (hl7_value _value)
{
 value  = _value.value;
 values = _value.values;
}

void hl7_value::SetValue (unsigned int _index, string _value)
{
 if (_index + 1 > values.size())
  values.resize(_index + 1);
 values[_index] = _value;
}

void hl7_value::SetValue (unsigned int _index, hl7_value _value)
{
 if (_index + 1 > values.size())
  values.resize(_index + 1);
 values[_index] = _value;
}

hl7_value & hl7_value::operator[] (unsigned int _index)
{
 if (_index + 1 > values.size())
  values.resize(_index + 1);
 return values[_index];
}

string hl7_value::GetRawValue () const
{
 if (!values.empty())
  throw("Not a singleton!");
 return value;
}

string hl7_value::GetValue (const hl7_batch & _batch, hl7_value_type _type) const
{
 string answer;
 if (_type == hl7_subcomponent || !values.size())
  if (preserve)
   answer = value;
  else
  answer = _batch.GetEscapedValue(value);
 else
 {
  for (unsigned int i = 0; i < values.size(); i++)
  {
   if (i)
    answer += _batch.GetSeparator(_type + 1);
   answer += values[i].GetValue(_batch, _type + 1);
  }
 }
 return answer;
}

string hl7_value::GetSegmentValue (const hl7_batch & _batch) const
{
 return GetValue(_batch, hl7_segment) + _batch.GetSeparator(hl7_segment);
}

//---------------------------------------------------------------------------

// suspiciously vague function definitions
hl7_value hl7_values (unsigned int _count_values, ...)
{
 hl7_value answer;
 va_list argument_list;
 va_start (argument_list, _count_values);
 for (;_count_values > 0; _count_values--)
  answer.values.push_back (va_arg(argument_list, hl7_value));
 va_end (argument_list);
 return answer;
}

hl7_value hl7_values (string _value_description, ...)
{
 hl7_value answer;
 va_list argument_list;
 unsigned int _count_values = _value_description.length();
 va_start (argument_list, _value_description);
 for (unsigned int i = 0; i < _count_values; i++)
 {
  switch (_value_description[i])
  {
   case 's':
   case 'S':
    answer.values.push_back (hl7_value(va_arg(argument_list, string)));
    break;
   case 'c':
   case 'C':
    answer.values.push_back (hl7_value(string(va_arg(argument_list, char *))));
    break;
   case 'v':
   case 'V':
    answer.values.push_back (va_arg(argument_list, hl7_value));
    break;
   default:
    answer.values.push_back (hl7_value(""));
  }
 }
 va_end (argument_list);
 return answer;
}

//---------------------------------------------------------------------------

// hl7_batch
hl7_batch::hl7_batch ()
{
 SetSeparator(hl7_segment,      "\n");
 SetSeparator(hl7_element,      "|");
 SetSeparator(hl7_component,    "^");
 SetSeparator(hl7_repetition,   "~");
 SetSeparator(hl7_subcomponent, "&");
 SetEscapeSequence("\\");

 // hey look, magic numbers!
 // TODO -c MAGIC: let's formalize this todo!
 batch_number          = "";
 resend_batch_number   = "";
 sending_application   = "PROM"; // they lied again.  4 characters max.  they're updating the documentation to match.  *rolls eyes*
 sending_facility      = "555813";
 sending_facility_id   = "17133";
 receiving_application = "DW1";
 receiving_facility    = "IHS";
 comment               = "ABSENTEE SHAWNEE";
 status                = "P";
 version               = "2.4";
}

bool hl7_batch::SetSeparator (hl7_value_type _type, string _separator)
{
 for (unsigned int i = 0; i < separators.size(); i++)
  if (i != _type && separators[i] == _separator)
   return false;
 if (_type + 1 > separators.size())
  separators.resize(_type + 1);
 separators[_type] = _separator;
 return true;
}

string hl7_batch::GetSeparator (hl7_value_type _type) const
{
 if (_type + 1 > separators.size())
  return "";
 else
  return separators[_type];
}

bool hl7_batch::SetEscapeSequence (string _escape_sequence)
{
 for (unsigned int i = 0; i < separators.size(); i++)
  if (separators[i] == _escape_sequence)
   return false;
 escape_sequence = _escape_sequence;
 return true;
}

string hl7_batch::GetEscapeSequence () const
{
 return escape_sequence;
}

string hl7_batch::GetEscapedValue (string _value) const
{
 string answer = _value;
 answer = ReplaceStringWithString(_value, escape_sequence, escape_sequence + escape_sequence); // escape the escape chars
 answer = StripLeading(StripTrailing(answer, " "), " "); // remove leading/trailing spaces
 answer = StripAll(answer, "\a\b\f\n\r\t\v"); // remove escape sequences
 answer = StripAll(answer, GetSeparator(hl7_segment)); // just in case, remove segment separator (not escapable)
 for (unsigned int i = 0; i < separators.size(); i++)
  answer = ReplaceStringWithString(answer, separators[i], escape_sequence + separators[i]);
 return answer;
}

//---------------------------------------------------------------------------

// conversion functions
string hl7_FinancialToNM (financial _value)
{
 return FinancialToString(_value, false, 2);
}

financial hl7_NMToFinancial (string _value)
{
 return StringToFinancial(_value);
}

string hl7_BooleanToBL (bool _value, string _true_value, string _false_value)
{
 return (_value ? _true_value : _false_value);
}

bool hl7_BLToBoolean (string _value, string _true_value, string _false_value)
{
 return (_value == _true_value);
}

string hl7_PdateToDT (Pdate _value)
{
 if (_value.IsValidDate())
  return twodigit(_value.year) + twodigit(_value.month) + twodigit(_value.day);
 else
  return "";
}

Pdate hl7_DTToPdate (string _value)
{
 Pdate answer;
 if (_value.length() > 7)
 {
  Pdate temp;
  temp.year  = StringToInt(_value.substr(0, 4));
  temp.month = StringToInt(_value.substr(4, 2));
  temp.day   = StringToInt(_value.substr(6, 2));
  if (temp.IsValidDate())
   answer = temp;
 }
 return answer;
}

string hl7_PtimeToTM (Ptime _value)
{
 if (_value.IsValidTime())
  return twodigit(_value.hour) + twodigit(_value.minute);
 else
  return "";
}

Ptime hl7_TMToPtime (string _value)
{
 Ptime answer;
 if (_value.length() > 3)
 {
  Ptime temp;
  temp.hour   = StringToInt(_value.substr(0, 2));
  temp.minute = StringToInt(_value.substr(2, 2));
  if (temp.IsValidTime())
   answer = temp;
 }
 return answer;
}

string hl7_PstampToTS (Pstamp _value)
{
 if (_value.IsValidStamp())
  return hl7_PdateToDT(_value) + hl7_PtimeToTM(_value);
 else
  return "";
}

Pstamp hl7_TSToPstamp (string _value)
{
 Pstamp answer;
 if (_value.length() > 11)
 {
  Ptime temp_time = hl7_TMToPtime(_value.substr(8));
  Pdate temp_date = hl7_DTToPdate(_value.substr(0, 8));
  Pstamp temp_stamp;
  temp_stamp.year = temp_date.year;
  temp_stamp.month = temp_date.month;
  temp_stamp.day = temp_date.day;
  temp_stamp.hour = temp_time.hour;
  temp_stamp.minute = temp_time.minute;
  if (temp_stamp.IsValidStamp())
   answer = temp_stamp;
 }
 return answer;
}

