#include "hl7_db.h"
#include "lookuptablewrapper.h"
#include "util.h"

/*
hl7_value person_segment;
person_segment.SetValue(0, "PRS");
person_segment.SetValue(1, person_row[0]);
person_segment.SetValue(2, person_row[1]);
hl7_value person_segment = hl7_values("csv", "PRS", person_row[0], hl7_values("v", hl7_values("sv", person_row[1], hl7_values("cc", "wee", "cough"))));
*/

/*
hl7_value person_segment;
person_segment.SetValue(0, "PRS"); // should come from Prepare()
person_segment.SetValue(1, person_row[0]);
person_segment.SetValue(2, person_row[1]);
person_segment.SetValue(3, hl7_values("cccccc", "this", "is", "a", "six", "part", "message"));
person_segment.SetValue(4, hl7_values("cvv", "ouch", hl7_values("v", hl7_values("cc", "big", "small")), hl7_values("vv", hl7_values("cc", "cold", "hot"), hl7_values("ccc", "short", "medium", "tall"))));
(*export_file) << person_segment.GetValue(*export_batch, hl7_segment) << export_batch->GetSeparator(hl7_segment) << flush;
*/

// reminder: repetition is directly under element, so if you do
// answer[2] = hl7_values("cc", "blah", "stuff");
// you'll get blah~stuff rather than blah^stuff
// you want the following instead
// answer[2] = hl7_values(1, hl7_values("cc", "blah", "stuff"));
// as this creates 1 repetition with 2 parts to it
// sorry that it's so ... odd. didn't want to embed the "level" in the object because of how easily those'd get screwed up.

// be careful with hl7_values!
// doing the following will cause problems:
// hl7_values("s", "blah")
// also ... not sure how well string& can be passed as string, which is why you'll see
// hl7_values("s", string(some_map["field_name"]))
// i was too lazy to test to see what'd happen.

// how do we handle lookup table code here? shall we handle it outside before passing it to the function?

// DONE: they don't like the way I have the "modifications" stuff coded, because their system will refuse "Y" when it's "A"
// centralized change-code generating whatsit
void hl7_change_code (map<string, string> &entity, string have_sent_fieldname, bool look_at_sets, list<map<string, string> > &entity_sets, hl7_run_type full_or_inc)
{ // input:  entity is a single row (person or visit) we're looking at entity_sets on
  //         have_sent_fieldname is "0" or "1" indicating whether this "type of set" has been sent to IHS for this entity, ever
  //         entity_sets relates to the entity (for people: aliases, charts, insurance policies; for visits: procedures, diagnoses, measurements...)
  // output: "A" for add; otherwise, was it modified since last sent?  "Y" or "N" for Yes/No.
  // processing: current logic is as follows:
  //             have_sent_X (set) before?
  //               No:  change code is "A" (Add)
  //               Yes:  loop through records in entity_set, find max("last_update_stamp")
  //                     get "export_date" from entity (see codeSpecialExportHL7ForIHS.cpp for logic on that;
  //                     currently it is:  last export's end_date if not null, otherwise last export's run_date,
  //                     where "last export" is the last one sent to IHS
  //                     Is max(last_update_stamp) > export_date ?
  //                     Yes:  change code is "Y" (modified)
  //                     No:  change code is "N" (not modified)
  //             Then change_code will be added to each entity_set row and passed back, where it can be passed
  //             forward to the appropriate segment functions and applied directly (fetch row's "change_code"
  //             and slap it in the appropriate "field" in the output file).
  string result = "";
  if (full_or_inc == hl7_run_full)
    result = "A";
  else // incremental
  {
    if (entity[have_sent_fieldname] != "1")
      result = "A";
    else if (!look_at_sets)
    { // then they want the change code for the entity (person/visit), not the change code for the set
     if (entity["last_update_stamp"] > entity["export_date"])
      result = "Y";
     else
      result = "N";
    }
    else
    { // they want the change code for the set
     bool has_changed = false;
     for (list<map<string, string> >::iterator i = entity_sets.begin(); i != entity_sets.end(); i++)
      if (Pdate((*i)["last_update_stamp"], global_mode) > Pdate((*i)["export_date"], global_mode))
      {  has_changed = true; break;  }
     switch (has_changed)
     { case true: result = "Y"; break;
       case false: result = "N"; break; }
    }
  }

  // now propogate the code accordingly
  if (!look_at_sets)
    entity["change_code"] = result;
  else
   for (list<map<string, string> >::iterator i = entity_sets.begin(); i != entity_sets.end(); i++)
    (*i)["change_code"] = result;
}
//---------------------------------------------------------------------------
bool hl7_assimilate_set(list<map<string, string> > a_set)
{ // So far as precedence:  Y > A > N
 // get
 string cc = "N"; // default
 for (list<map<string, string> >::iterator i = a_set.begin(); i != a_set.end(); i++)
 {
  if (cc == "N" && (*i)["change_code"] == "A") // first A we come to makes it A
    cc = "A";
  if ((*i)["change_code"] == "Y")              // any Y we come to makes it Y
    cc = "Y";
 }
 for (list<map<string, string> >::iterator i = a_set.begin(); i != a_set.end(); i++)
  (*i)["change_code"] = cc;
 return true;
}
//---------------------------------------------------------------------------
// batch header
hl7_value hl7_BHS (hl7_batch & b, Pstamp export_stamp)
{
 hl7_value answer;
 answer[0] = "BHS";
 answer[1] = b.GetSeparator(hl7_component) + b.GetSeparator(hl7_repetition) + b.GetEscapeSequence() + b.GetSeparator(hl7_subcomponent);
 answer[1].SetPreserve(true); // we don't want to accidentally escape the escape sequence, eh?
 answer[2] = b.sending_application;
 answer[3] = b.sending_facility;
 answer[4] = b.receiving_application;
 answer[5] = b.receiving_facility;
 answer[6] = hl7_PdateToDT(Pdate(export_stamp));
 answer[8] = b.batch_number;
 answer[9] = b.comment;
 answer[10] = b.sending_facility + b.batch_number;
 answer[11] = b.resend_batch_number;
 return answer;
}

hl7_value hl7_ZHS (Pdate start, Pdate stop, string export_id)
{
 hl7_value answer;
 answer[0] = "ZHS";
 answer[1] = hl7_PdateToDT(start);
 answer[2] = hl7_PdateToDT(stop);
 answer[3] = export_id;
 answer[4] = "HL7";
 answer[5] =  hl7_values(1, hl7_values("cc", "EXO", "INCR")); // TODO: INCR(emental) should probably be something else for full exports....
 answer[6] = "2.4";
 return answer;
}
//---------------------------------------------------------------------------
// batch trailer
hl7_value hl7_BTS (unsigned int message_count, string message_comment)
{
 hl7_value answer;
 answer[0] = "BTS";
 answer[1] = IntToString(message_count); // research?
 answer[2] = "";
 answer[3] = ""; // "batch totals" research? [undescribed in ADT segment docs]
 return answer;
}

hl7_value hl7_ZTS (unsigned int visit_count, unsigned int record_count)
{
 hl7_value answer;
 answer[0] = "ZTS";
 answer[1] = IntToString(visit_count); // might be zero if only sending people
 answer[2] = ""; // skipped total (error + not appropriate)
 answer[3] = ""; // skipped error
 answer[4] = ""; // skipped demo
 answer[5] = IntToString(record_count + 2); // adding header and trailer messages to total count
 return answer;
}
//---------------------------------------------------------------------------
// more generics
hl7_value hl7_MSH (hl7_batch & b, Pstamp event_stamp, string trigger_type, string message_id)
{
 hl7_value answer;
 answer[0] = "MSH";
 answer[1] = b.GetSeparator(hl7_component) + b.GetSeparator(hl7_repetition) + b.GetEscapeSequence() + b.GetSeparator(hl7_subcomponent);
 answer[1].SetPreserve(true); // we don't want to accidentally escape the escape sequence, eh?
 answer[2] = b.sending_application;
 answer[3] = b.sending_facility;
 answer[4] = b.receiving_application;
 answer[5] = b.receiving_facility;
 answer[6] = hl7_PdateToDT(Pdate(event_stamp));
 answer[8] = hl7_values(1, hl7_values("cs", "ADT", trigger_type));
 answer[9] = message_id;
 answer[10] = b.status; // (D)ebug, (P)roduction, (T)raining
 answer[11] = b.version;
 answer[14] = "AL"; // always ack (accept)
 answer[15] = "AL"; // always ack (application)
 answer[20] = ""; // to end
 return answer;
}

hl7_value hl7_EVN (string trigger_type, Pstamp event_stamp)
{
 hl7_value answer;
 answer[0] = "EVN";
 answer[1] = trigger_type;
 answer[2] = hl7_PdateToDT(Pdate(event_stamp));
 answer[7] = ""; // to end
 return answer;
}
//---------------------------------------------------------------------------
// common to A31 and A08
hl7_value hl7_PID (hl7_batch & b, map<string, string> & person_row, map<string, string> & visit_row, list<map<string, string> > & person_name_rows, list<map<string, string> > & person_contact_info_rows)
{
 hl7_value answer;
 answer[0] = "PID";
 answer[1] = ""; // TODO: "set id" -- research?
 answer[3] = hl7_values(1, hl7_values("ss", PadString(b.sending_facility_id, pad_left, '0', 5) + PadString(person_row["id"], pad_left, '0', 10), string(person_row["id"])));

 // TODO -c HACK: hack hack hack. this export needs to be done and to them by the end of the day, so for now...
 string sending_facility = b.sending_facility;
 if (!visit_row.empty() && visit_row["group_id"] == "29") // Shawnee Clinic
  sending_facility = "555816";
// They're using this as an "alternate patient identifier" ... so for consistency, we'll continue leaving CHS out of this
// logic; it should theoretically match what's in the registry file; so for encounters that's going to be using the LAC
// ASUFAC, since visit_row will be empty.  So ... yeah.  Think of it like "where is their primary chart stored" ... except
// the people with no chart at either clinic (only ever referred by CHS) are considered to have a LAC chart too.  Which
// works fine, since we're using person_id as the chart# (due to length and consistency issues on LAC chart #s).
// if (!IsBlankOrZero(visit_row["parent_task_id"]))
//  sending_facility = "555897";                  // those with parent tasks are COB visits, so "CHS Other"

 // DONE: alternate id involving 6-char facility code? they want this after all?
 // DONE -c MAGIC: FAC code, facility code // TODO: we want this
 // TODO: chart number is supposed to be 6 chars
 string chart_number = "";
 if (!visit_row.empty()) // have a visit, use it // .find("chart_id") != visit_row.end()
   chart_number = visit_row["person_id"];
 else // if (person_row["primary_chart_number"] != "") // otherwise go off person
   chart_number = person_row["id"];
 chart_number = StripLeading(StripLeading(StripTrailing(chart_number, " "), " "), "0");
 // TODO: strip trailing -00 ... don't use StripTrailing though as that will strip trailing hyphens or zeroes.  whoops.  that is, if we ever switch back to sending them chart numbers.
 answer[4] = hl7_values("s", sending_facility + PadString(chart_number, pad_left, '0', 6));

 // primary name
 for (list<map<string, string> >::iterator i = person_name_rows.begin(); i != person_name_rows.end(); i++)
  if ((*i)["id"] == person_row["person_names_id"])
   answer[5] = hl7_values(1, hl7_values("sssss", StripSurrounding((*i)["last_name"], " "),
                                                 StripSurrounding((*i)["first_name"], " "),
                                                 StripSurrounding((*i)["middle_name"], " "),
                                                 StripSurrounding((*i)["suffix"], " "),
                                                 StripSurrounding((*i)["prefix"], " ")));
 answer[6] = ""; // mother's maiden name, not supported by prometheus
 answer[7] = hl7_PdateToDT(Pdate(person_row["birth_date"], global_mode));
 answer[8] = hl7_PrometheusToHL7_gender_types(person_row["gender_id"]); // they lied about the gender codes being numeric. (in the code set they sent me.)
 // primary address, phone numbers (sorta)
 for (list<map<string, string> >::iterator i = person_contact_info_rows.begin(); i != person_contact_info_rows.end(); i++)
  if ((*i)["id"] == person_row["person_contact_info_id"])
  {
   answer[11] = hl7_values(1, hl7_values("sssss", string((*i)["line1"]), string((*i)["line2"]), string((*i)["city"]), hl7_PrometheusToHL7_state_types_abbreviations((*i)["state_id"]) /* STATE code */, string((*i)["zip"]), string("") /* COUNTRY code */, string("") /* COUNTY */));
   answer[13] = ""; // (*i)["phone_number"]; // phone, home -- verify formatting?
   answer[14] = ""; // phone, work
  }
 answer[19] = StripAllExcept(person_row["social_security_number"], uints);
 answer[27] = ""; // TODO: veteran (Y/N)
 answer[29] = hl7_PdateToDT(Pdate(person_row["death_date"], global_mode));
 answer[33] = hl7_PstampToTS(Pstamp(person_row["last_update_stamp"], global_mode));
 answer[38] = ""; // to end
 return answer;
}

hl7_value hl7_ZP2 (map<string, string> & person_row, list<map<string, string> > & person_name_rows, list<map<string, string> > & person_affiliation_rows, list<map<string, string> > & person_contact_info_rows)
{
 hl7_value answer;
 answer[0] = "ZP2";
 answer[1] = ""; // TODO: date record created
 answer[2] = ""; // TODO: date record updated? ("last Reg update")
 // primary affiliation
 for (list<map<string, string> >::iterator i = person_affiliation_rows.begin(); i != person_affiliation_rows.end(); i++)
  if ((*i)["id"] == person_row["affiliations_id"])
  {
   answer[5] = (*i)["cdib"]; // CDIB
   //answer[5] = hl7_values("s", PadString((*i)["cdib"], pad_left, '0', 12)); // CDIB
   answer[6] = ""; // TODO: "CHS Tribal Affiliation" (CE, length 60)
   answer[9] = hl7_PdateToDT(Pdate((*i)["join_date"], global_mode)); // join-date
   answer[10] = hl7_PdateToDT(Pdate((*i)["relinquish_date"], global_mode)); // relinq-date
   answer[15] = PadString((*i)["tribe_code"], pad_left, '0', 3);
   //(*i)["tribe_code"];  // hl7_PrometheusToHL7_tribe_types((*i)["tribe_id"]); // TRIBE code
   //answer[15] = hl7_values("s", PadString((*i)["tribe_code"], pad_left, '0', 3)); // TRIBE code
   answer[16] = ""; // quantum
   answer[17] = ""; // QUANTUM code (all, lots, some, a bit, ...)
  }
 answer[11] = ""; // RPMS-only SSN verification status
 if (!is_in(answer[15].GetRawValue(), "000", "997", "999", "", NULL))
  answer[18] = "01" /* tribal */; // TODO: BENEF_CLASS code (why the're eligible?)

 // answer[19] = ""; // current residence date -- research
 // last-update stamp on the primary address
 for (list<map<string, string> >::iterator i = person_contact_info_rows.begin(); i != person_contact_info_rows.end(); i++)
  if ((*i)["id"] == person_row["person_contact_info_id"])
   answer[19] = hl7_PdateToDT(Pdate((*i)["last_update_stamp"], global_mode));

 answer[22] = "9999999"; // default if no person_contact_info rows at all!
 for (list<map<string, string> >::iterator i = person_contact_info_rows.begin(); i != person_contact_info_rows.end(); i++)
  if ((*i)["id"] == person_row["person_contact_info_id"])
  {
   // COMMUNITY code -- based on a lot of matching up at random, via city names and whatnot
   string st = ((*i)["ihs_state_id"].empty() ? string("99") : string((*i)["ihs_state_id"]));
   string co = ((*i)["ihs_county_id"].empty() ? string("99") : string((*i)["ihs_county_id"]));
   string cm = ((*i)["ihs_community_id"].empty() ? string("9" + st) : string((*i)["ihs_community_id"]));
   // this seemed weird to me:
   // if the community can't be figured out, we -resend- the state as the last two digits, after a 9?
   // who came up with -that- brilliant idea?
   answer[22] = st + co + cm;
  }
 answer[31] = ""; // TODO: father's name (last^first^middle)

 // ELIGIBILITY code
 // TODO: possibly inaccurate?
 if (answer[15].GetRawValue() == "141") // Absentee Shawnee Tribes Of Oklahoma
  answer[34] = "C"; // CHS & DIRECT
 else if (!is_in(answer[15].GetRawValue(), "000", "997", "999", "", NULL))
  answer[34] = "D"; // DIRECT only
 else
  answer[34] = "I"; // INELIGIBLE

 answer[35] = ""; // to end; TODO: also mother's name (last^first^middle)
 return answer;
}

//---------------------------------------------------------------------------
// only in A31 (about person)
hl7_value hl7_ZRB (map<string, string> & person_row)
/* technically also relies on relationships + other person info */
{
 hl7_value answer;
 Pstamp last_row_update(person_row["last_update_stamp"], global_mode); // this is very nearly correct; there are two pieces of info not in the people table that they want to affect it too though--if the person's mother or father's name change
 answer[0] = "ZRB";
 answer[1] = hl7_PdateToDT(Pdate(last_row_update));
 answer[2] = hl7_PtimeToTM(Ptime(last_row_update));
 answer[3] = person_row["change_code"];

 answer[5] = ""; // to end; also cause of death, not supported by prometheus (somewhat repeated in PV1 or ZV1 or something)
 return answer;
}

hl7_value hl7_ZRC (hl7_batch & b, map<string, string> & person_row, map<string, string> & person_chart_row)
{
 hl7_value answer;
 Pstamp last_row_update(person_chart_row["last_update_stamp"], global_mode);
 answer[0] = "ZRC";
 answer[1] = hl7_PdateToDT(Pdate(last_row_update));
 answer[2] = hl7_PtimeToTM(Ptime(last_row_update));
 answer[3] = person_chart_row["change_code"];
 answer[5] = person_chart_row["id"];
 answer[6] = (person_chart_row["hl7_asufac"] != "") ? person_chart_row["hl7_asufac"] : string("555813");
 /* b.sending_facility; // TODO -c MAGIC ""; // FAC code, facility code // TODO: we want this */
 answer[7] = person_chart_row["id"]; // we can't give them what they want (due to stupid constraints on their part), so the hell with what they want
 // this doesn't fit in 6 characters // person_chart_row["group_id"] + "-" + person_chart_row["chart_number"]; // TODO: 6 or 10 chars
 answer[8] = (StringToInt(person_row["current_id"]) > 2) ? "A" : "I"; // CHART_STATUS code, status of chart -- they meant of person, not of the chart number -- we're only sending "current_id > 2" chart numbers anyway
 return answer;
}

hl7_value hl7_ZRD (map<string, string> & person_row, list<map<string, string> > & person_name_rows, list<map<string, string> > & person_affiliation_rows, list<map<string, string> > & person_contact_info_rows)
{
 hl7_value answer;
 Pstamp last_row_update;
 Pstamp temp;

 temp.InStamp(person_row["last_update_stamp"], global_mode);
 last_row_update = max(last_row_update, temp);

 // change code:  default is A.  N trumps A.  Y trumps N.
 // Does anyone else notice these just so happen to be in alphabetical order by importance?  <EG>
 string change_code = "A";
 for (list<map<string, string> >::iterator i = person_name_rows.begin(); i != person_name_rows.end(); i++)
 {
  temp.InStamp((*i)["last_update_stamp"], global_mode);
  last_row_update = max(last_row_update, temp);
  change_code = max(change_code, (*i)["change_code"]);
 }

 for (list<map<string, string> >::iterator i = person_affiliation_rows.begin(); i != person_affiliation_rows.end(); i++)
 {
  temp.InStamp((*i)["last_update_stamp"], global_mode);
  last_row_update = max(last_row_update, temp);
  change_code = max(change_code, (*i)["change_code"]);
 }

 for (list<map<string, string> >::iterator i = person_contact_info_rows.begin(); i != person_contact_info_rows.end(); i++)
 {
  temp.InStamp((*i)["last_update_stamp"], global_mode);
  last_row_update = max(last_row_update, temp);
  change_code = max(change_code, (*i)["change_code"]);
 }

 answer[0] = "ZRD";
 answer[1] = hl7_PdateToDT(Pdate(last_row_update));
 answer[2] = hl7_PtimeToTM(Ptime(last_row_update));
 answer[3] = change_code;
 answer[8] = "A"; // active.
                  // TODO: for inactive (deleted or merged) people, write code elsewhere. need paul to create table of deleted/merged records for this.
 return answer;
}

hl7_value hl7_ZRL (map<string, string> & person_name_row)
{
 hl7_value answer;
 Pstamp last_row_update(person_name_row["last_update_stamp"], global_mode);
 answer[0] = "ZRL";
 answer[1] = hl7_PdateToDT(Pdate(last_row_update));
 answer[2] = hl7_PtimeToTM(Ptime(last_row_update));
 answer[3] = person_name_row["change_code"];
 answer[5] = person_name_row["id"];
 answer[6] = hl7_values(1, hl7_values("sssss", StripSurrounding(person_name_row["last_name"], " "),
                                               StripSurrounding(person_name_row["first_name"], " "),
                                               StripSurrounding(person_name_row["middle_name"], " "),
                                               StripSurrounding(person_name_row["suffix"], " "),
                                               StripSurrounding(person_name_row["prefix"], " ")));
 return answer;
}

hl7_value hl7_IN1 (unsigned int sequence_number, map<string, string> & person_insurance_row)
{
 hl7_value answer;
 answer[0] = "IN1";
 answer[1] = IntToString(sequence_number);
 answer[3] = person_insurance_row["t_taxid"];
 answer[4] = hl7_values(1, hl7_values("ss", hl7_PrometheusToHL7_insurance_types(person_insurance_row["t_insurance_id"]), string(person_insurance_row["t_name"])));
 answer[12] = hl7_PdateToDT(Pdate(person_insurance_row["p_start_date"], global_mode));
 answer[13] = hl7_PdateToDT(Pdate(person_insurance_row["p_stop_date"], global_mode));
 answer[15] = ""; // TODO: STATE code for medicaid eligibility + name of medicaid plan
 answer[16] = hl7_hack_split_name(person_insurance_row["p_name_on_policy"]); // TODO: name of insured as last^first^middle (name on policy)
 answer[17] = ""; // TODO: relationship to insured
                  // it'd be nice if we had a link between the policy and the "insured" person ...
                  // - name on policy pulled from person_names rather than hacked (above)
                  // - relationship determination
                  // - ...
 answer[47] = ""; // TODO: coverage type, A or B if medicare or railroad, 1- or 2-digit if medicaid, otherwise blank
 answer[49] = hl7_values(1, hl7_values("ss", string(person_insurance_row["p_policy_number"]), string("") /* prefix or suffix */)); // to end, also policy number & policy number suffix (not supported in prometheus as separate fields)
 return answer;
}

hl7_value hl7_ZIN (map<string, string> & person_insurance_row)
{
 hl7_value answer;
 Pstamp last_row_update;
 Pstamp last_coverage_update(person_insurance_row["c_last_update_stamp"], global_mode);
 Pstamp last_policy_update(person_insurance_row["p_last_update_stamp"], global_mode);
 Pstamp last_entity_update(person_insurance_row["t_last_update_stamp"], global_mode);
 last_row_update = max(last_coverage_update, max(last_policy_update, last_entity_update)); // three-way?
 answer[0] = "ZIN";
 answer[1] = hl7_PdateToDT(Pdate(last_row_update));
 if (!person_insurance_row["c_last_update_stamp"].empty() &&
     !person_insurance_row["p_last_update_stamp"].empty() &&
     !person_insurance_row["t_last_update_stamp"].empty())
  answer[2] = hl7_PtimeToTM(Ptime(last_row_update));
 answer[3] = person_insurance_row["change_code"];
 answer[5] = person_insurance_row["c_id"];
 answer[6] = answer[1].value; // to end; also copy of answer[1], says the spec
 return answer;
}
//---------------------------------------------------------------------------
hl7_value hl7_PV1 (hl7_batch & b, map<string, string> & visit_row, list<map<string, string> > & visit_condition_rows, list<map<string, string> > & visit_procedure_rows)
{
 hl7_value answer;
 answer[0] = "PV1";
 answer[1] = "0"; // only one is supported per message.

 // TODO -c HACK: hack hack hack. this export needs to be done and to them by the end of the day, so for now...
 string sending_facility = b.sending_facility;  // default is LAC
 if (!IsBlankOrZero(visit_row["parent_task_id"]))
  sending_facility = "555897";                  // those with parent tasks are COB visits, so "CHS Other"
 else if (!visit_row.empty() && visit_row["group_id"] == "29")
  sending_facility = "555816";                  // those with Shawnee Clinic use that facility code
                               // "Location of Encounter"
 answer[3] = sending_facility; // TODO -c MAGIC: two are "us", the other is "other CHS" (without specifying hospital vs. physician)
 answer[4] = "5" /* other */; // TODO: ADMISSION code -- we don't know, might "guess" by referrals, but that's no guarantee
 answer[7] = hl7_values(1, hl7_values("ccc", "" // (visit_row["group_id"] == "5" ? "2" /* contract */ : "3" /* tribal */)
                                           , ""
                                           , "" // string(visit_row["tax_entity_id"])
                                      )); // provider's: AFFILIATION code ^ PROVIDER_DISC ^ id of provider (3 characters -- not actual length according to CDC) -- this is somehow for direct hospitalization only?
 answer[8] = ""; // referring doctor (not necessary)
 // TODO -c HACK: Just so the IDs don't overlap, we're prefixing the "visit ID" with a 9 for pharmacy, since that ID comes from a different table (pharmacy_info) and could overlap with tasks.id's
 answer[19] = (visit_row["group_id"] != lookupTables->IdOf("groups", "Pharmacy"))
              ? hl7_values(1, hl7_values("ss", PadString(b.sending_facility_id, pad_left, '0', 5) + PadString(visit_row["id"], pad_left, '0', 10), string(visit_row["id"])))
              : hl7_values(1, hl7_values("ss", PadString(b.sending_facility_id, pad_left, '0', 5) + "9" + PadString(visit_row["id"], pad_left, '0', 9), string(visit_row["id"])));
 // various "amount fields"
 answer[36] = ""; // TODO: ER_DISPOS code
 answer[37] = ""; // FAC code (transferred to) -- we don't currently do transfers
 Pdate last_proc_cond = Pdate(visit_row["date_of_task"], global_mode);
 answer[44] = hl7_PdateToDT(last_proc_cond);
 for (list<map<string, string> >::iterator i = visit_condition_rows.begin(); i != visit_condition_rows.end(); i++)
  last_proc_cond = max(last_proc_cond, max(Pdate((*i)["start_date"], global_mode), Pdate((*i)["stop_date"], global_mode)));
 for (list<map<string, string> >::iterator i = visit_procedure_rows.begin(); i != visit_procedure_rows.end(); i++)
  last_proc_cond = max(last_proc_cond, max(Pdate((*i)["start_date"], global_mode), Pdate((*i)["stop_date"], global_mode)));
 answer[45] = hl7_PdateToDT(last_proc_cond);

 // more "amount fields"
 answer[52] = ""; // to end
 return answer;
}

hl7_value hl7_ZV1 (hl7_batch & b, map<string, string> & visit_row, list<map<string, string> > & visit_condition_rows, list<map<string, string> > & visit_procedure_rows)
{
 hl7_value answer;
 answer[0] = "ZV1";
 Pdate date_of_visit = Pdate(visit_row["date_of_task"], global_mode);
 Pstamp last_row_update = Pstamp(visit_row["last_update_stamp"], global_mode);
 answer[1] = hl7_PdateToDT(date_of_visit.IsValidDate() ? min(date_of_visit, Pdate(last_row_update)) : Pdate(last_row_update));

 // CLINIC code
 if (visit_row["auth_service_id"] == "") // we don't know the service
  answer[3] = hl7_PrometheusToHL7_task_lookups(visit_row);
 else
  answer[3] = hl7_PrometheusToHL7_service_types(visit_row["auth_service_id"]);

 answer[4] = hl7_PdateToDT(Pdate(last_row_update));
 answer[6] = ""; // SERVICE_LEVEL code // TODO: this would be nice
 answer[12] = ""; // number of consultations (might make more sense if we tracked several tasks together, or different providers for one task
 if (!IsBlankOrZero(visit_row["parent_task_id"]))
  answer[15] = hl7_values(1, hl7_values("ss", string(visit_row["auth_cob_number"]), b.sending_facility)); // authorization number ^ FAC code of authorizing facility
 else
  answer[15] = "";
 answer[17] = ""; // TODO: total cost (contract health)
 if (!IsBlankOrZero(visit_row["parent_task_id"]))
  answer[21] = hl7_PrometheusToHL7_vendor_types(visit_row["auth_service_id"]); // PROVIDER_TYPE code, "vendor" (no idea) // TODO: look into -- this is for in-house visits only, about our users/entities
 else
  answer[21] = "";

 // answer[22] = ""; // visit time in minutes
 // answer[23] = ""; // travel time in minutes
 /*
  group_id ... method
  2            minutes (a few large values in the thousands, 10 hours, etc.)
  4            hours (mostly, some really large values that aren't appropriate even for minutes)
  6            hours (clean)
  12           hours (clean)
  17           hours (assumed - only one value, "3")
  18           hours (mostly, one value of "60")
  26           hours (clean)
 */
 float f_total_time_spent = max(float(0.00), StringToFloat(visit_row["time_spent"]));
 float f_visit_time_spent = max(float(0.00), StringToFloat(visit_row["visit_time_spent"]));
 float f_other_time_spent = max(float(0.00), f_total_time_spent - f_visit_time_spent);
 if (visit_row["group_id"] != "2")
 {
  f_total_time_spent *= 60.0;
  f_visit_time_spent *= 60.0;
  f_other_time_spent *= 60.0;
 }
 int total_time_spent = f_total_time_spent;
 int visit_time_spent = f_visit_time_spent;
 int other_time_spent = f_other_time_spent;
 if (visit_row["time_spent"] == "" && visit_row["visit_time_spent"] != "")
 {
  answer[22] = IntToString(visit_time_spent);
 } else
 if (visit_row["time_spent"] != "" && visit_row["visit_time_spent"] != "")
 {
  answer[22] = IntToString(visit_time_spent);
  answer[23] = IntToString(other_time_spent);
 } else
 if (visit_row["time_spent"] != "" && visit_row["visit_time_spent"] == "")
 {
  answer[22] = IntToString(total_time_spent);
 }

 answer[24] = hl7_PtimeToTM(Ptime(last_row_update));
 answer[25] = (StringToInt(visit_row["current_id"]) > 2 ? "N" : "Y"); // record deleted?
 answer[26] = ""; // TODO: visit export date, but a timestamp (even though this is a date field and the next is a time, they want the whole thing here) -- research?
 answer[28] = ""; // visit_row["auth_cob_number"]; // PO number for CHS ...
 answer[29] = ""; // TODO: number of prescriptions dispensed (if medications, coming from pharmacy_info, this makes sense?)
 if (!visit_row["time_of_task"].empty())
  answer[30] = hl7_PtimeToTM(Ptime(visit_row["time_of_task"], global_mode));
 answer[31] = date_of_visit.IsValidDate() ? IntToString(date_of_visit.DayOfWeek()) : string(""); // DAY_OF_WEEK (bastards), sunday = 1, etc.

 // SVC_CATEGORY code -- (C)ontract, (T)ribe pays, (P) government pays (compacted)
 // yes. this field is marked as "encounter type" in the ADT file. Barbara says this is what goes there. tough.
 answer[32] = "P"; // default to compacted
 if (!IsBlankOrZero(visit_row["parent_task_id"]))
  answer[32] = "C"; // unless it's CHS

  // TODO -c MAGIC_NUMBER: jhaley 20100913 - hard-coded event program id #'s not necessarily future-safe
 if (visit_row["program_id"] == "1" || visit_row["program_id"] == "2" || visit_row["program_id"] == "4" || visit_row["program_id"] == "5")
  answer[32] = "T"; // unless it's a diab thing for REACH, "diabetes initiative", or the competitive grant -- then it's "T". but i only found coding for REACH ...
                    // Paul updated it to include DM Initiative and CVD (competitive grant) as well.  What about other grants; would Caregiver not count as tribe-pays?
                    // updated to include MSPI // TODO: should the logic simply be !=0 && != 3?

 Pdate last_proc_cond = Pdate(visit_row["date_of_task"], global_mode);
 for (list<map<string, string> >::iterator i = visit_condition_rows.begin(); i != visit_condition_rows.end(); i++)
  last_proc_cond = max(last_proc_cond, max(Pdate((*i)["start_date"], global_mode), Pdate((*i)["stop_date"], global_mode)));
 for (list<map<string, string> >::iterator i = visit_procedure_rows.begin(); i != visit_procedure_rows.end(); i++)
  last_proc_cond = max(last_proc_cond, max(Pdate((*i)["start_date"], global_mode), Pdate((*i)["stop_date"], global_mode)));

 answer[33] = "A"; // TODO: SVC_CAT_CD  (A)mbulatory, (H)ospitalization, (O)bservation, Day-(S)urgery
 // TODO -- guessing that it's a hospital visit? very vague.
 if (!IsBlankOrZero(visit_row["parent_task_id"]) &&
     !is_in(answer[21].GetRawValue(), "05", "06", "07", "12", "16", NULL) && // force exclude Physician, Optometrist, Dentist, Pharmacy, and All Other to outpatient
     (last_proc_cond > Pdate(visit_row["date_of_task"], global_mode) || answer[21].GetRawValue() == "03")) // force include "Hospital Psychiatric" to inpatient
  answer[33] = "H";

 answer[34] = ""; // cpt-4 code for admin/eval
 answer[35] = ""; // icd-9 of death
 answer[36] = ""; // DISCHARGE_SERVICE code
 answer[37] = ""; // DISCHARGE_TYPE code
 answer[38] = ""; // to end, length of stay in days (gotta calculate) // TODO: do we care?
 return answer;
}

hl7_value hl7_ZEN (list<map<string, string> > & visit_measurement_rows)
{
 unsigned int c = 0;
 hl7_value answer;
 answer[0] = "ZEN";
 // some eligibility fields (not supported, annoying for us to get)
 answer[4] = ""; // TODO: ever documented HTN (hypertension)? (gotta look in all condition codes)
 answer[5] = ""; // TODO: last documented HTN
 answer[6] = ""; // LMP -- research?
 answer[7] = ""; // last LMP noted
 answer[8] = ""; // ACE inhibitor filled
 answer[9] = ""; // urine protein test
 answer[10] = ""; // ... value

 // old algorithm would get 'first' response for each field, this one gets the last one. can be hacked to match, if anyone cares. there's no real order anyway.
 map<string, string> question_to_response;
 for (list<map<string, string> >::iterator i = visit_measurement_rows.begin(); i != visit_measurement_rows.end(); i++)
  question_to_response[(*i)["obs_question_id"]] = (*i)["response_metric"];

 string q;

 q = lookupTables->IdOf("obs_questions", "Micro Albumin");
 if (question_to_response.find(q) != question_to_response.end())
 {
  answer[11] = "Y"; // microalbuminuria test
  answer[12] = question_to_response[q];
  c++;
 }

 answer[13] = ""; // DM nutrition education

 q = lookupTables->IdOf("obs_questions", "Hemoglobin A1C");
 if (question_to_response.find(q) != question_to_response.end())
 {
  answer[14] = question_to_response[q]; // HGB A1C value
  c++;
 }

 answer[15] = ""; // pap

 q = lookupTables->IdOf("obs_questions", "Glucose");
 if (question_to_response.find(q) != question_to_response.end())
 {
  answer[16] = question_to_response[q]; // glucose (not m_glu)
  c++;
 }

 q = lookupTables->IdOf("obs_questions", "High Density Lipoprotein");
 if (question_to_response.find(q) != question_to_response.end())
 {
  answer[17] = "Y"; // HDL test
  answer[18] = question_to_response[q];
  c++;
 }

 q = lookupTables->IdOf("obs_questions", "Low Density Lipoprotein");
 if (question_to_response.find(q) != question_to_response.end())
 {
  answer[19] = "Y"; // LDL test
  answer[20] = question_to_response[q];
  c++;
 }

 q = lookupTables->IdOf("obs_questions", "Triglycerides");
 if (question_to_response.find(q) != question_to_response.end())
 {
  answer[21] = "Y"; // TRIG test
  answer[22] = question_to_response[q];
  c++;
 }

 answer[23] = ""; // PSA test
 answer[24] = ""; // fecal occult blood test
 answer[25] = (c) ? IntToString(c) : string(""); // to end, number of tests -- unless we want number of tests -regardless- of how many are sent?
 return answer;
}

hl7_value hl7_ZVP (unsigned int sequence_number, map<string, string> & visit_row)
{
 hl7_value answer;
 answer[0] = "ZVP";
 answer[1] = IntToString(sequence_number);
 answer[2] = ""; // TODO: AFFILIATION (provider aff.) ^ PROVIDER_DISC (discipline) // TODO: shall we always send the main phys. on the visit?
 if (IsBlankOrZero(visit_row["parent_task_id"]))
 {
  string the_id = visit_row["user_id"];
  answer[2] = hl7_values(1, hl7_values("css", "3" // provider affiliation, 2 = contract, 3 = tribal
                                            , PadString(visit_row["user_discipline"], pad_left, '0', 2) // not no mo' // hl7_PrometheusToHL7_user_discipline(the_id) // discipline -- we convert from the user id!
                                            , the_id // id string(visit_row["tax_entity_id"])
                                       )); // provider's: AFFILIATION code ^ PROVIDER_DISC ^ id of provider (3 characters -- not actual length according to CDC) -- this is somehow for direct hospitalization only?
  answer[3] = the_id; // provider id (local)
 }
 answer[4] = ""; // TODO: PROV_X12_CLASS (of provider)
 answer[5] = ""; // TODO: PROV_X12_SPEC (of provider)
 answer[6] = ""; // TODO: PROV_X12_TYPE (of provider)
 answer[7] = ""; // TODO: is midwifery? (Y/N)
 return answer;
}

hl7_value hl7_DG1 (unsigned int sequence_number, map<string, string> & visit_condition_row)
{
 hl7_value answer;
 answer[0] = "DG1";
 answer[1] = IntToString(sequence_number); // not sure if this means the index number on forms, or just any sequence?
 string valid_code = visit_condition_row["code"];
 size_t first_bad_char = valid_code.find_first_of(" ,;-");
 if (first_bad_char != string::npos) valid_code = valid_code.substr(0, first_bad_char);
 first_bad_char = UppercaseString(valid_code).find_first_of(upper_alphas, 1); // skip first character, for which alphas are allowed
 if (first_bad_char != string::npos) valid_code = valid_code.substr(0, first_bad_char); // alphas anywhere else make invalid ICD-9 codes...
 answer[3] = valid_code.substr(0,5); // a cleaned up diagnosis code; just in case, they're never longer than 5 characters, so this should be safe
 answer[4] = visit_condition_row["note"];
 answer[5] = ""; // TODO: actually a stamp in hl7 -- hl7_PdateToDT(Pdate(visit_condition_row["start_date"], global_mode));
 answer[19] = ""; // to end
 return answer;
}

hl7_value hl7_ZDX (map<string, string> & visit_condition_row)
{
 hl7_value answer;
 answer[0] = "ZDX";
 answer[4] = ""; // TODO: DX_CAUSE -- no idea.
 answer[6] = ""; // TODO: ICD9_EXT_INJ_CAUSE -- no idea.
 answer[7] = ""; // TODO: to end, also INJURY_PLACE -- nope, we don't have that either.
 return answer;
}

hl7_value hl7_PR1 (unsigned int sequence_number, map<string, string> & visit_procedure_row)
{
 hl7_value answer;
 answer[0] = "PR1";
 answer[1] = IntToString(sequence_number);
 // TODO:  WRONG!  This field is for ICD-9 procedure codes.  CPT/HCPC codes go in the OBX-CPT segment, which we don't currently send
 answer[3] = visit_procedure_row["cpt_hcpcs"];
 answer[4] = visit_procedure_row["local_use"];
 answer[5] = hl7_PdateToDT(Pdate(visit_procedure_row["start_date"], global_mode));
 answer[11] = ""; // AFFILIATION ^ PROVIDER_DISC code // TODO: seems good
 answer[15] = ""; // TODO: "associated diagnosis code" (CE, 250)
 answer[16] = visit_procedure_row["modifier"];
 answer[18] = ""; // to end
 return answer;
}

hl7_value hl7_ZDN (unsigned int sequence_number, map<string, string> & visit_procedure_row)
{
 hl7_value answer;
 answer[0] = "ZDN";
 answer[1] = IntToString(sequence_number);
 answer[2] = StripLeading(visit_procedure_row["cpt_hcpcs"], "D"); // yes, the field name -is- strange for an ADA code, eh? // TODO: clean up our metadata
 answer[3] = visit_procedure_row["days_units"];
 answer[4] = IntToString(StringToInt(visit_procedure_row["charges"])); // fee, to nearest dollar (thus conversions)
 answer[5] = "D"; // Delivery mode:  D - Direct; K - Contract
 answer[6] = ""; // visit_procedure_row["charges"]; // Why did Philip use these backward?
 answer[7] = ""; // to end, dentist's SSN -- we have that, but not per-procedure ... // TODO: try to get this
 return answer;
}

hl7_value hl7_ZPR (map<string, string> & visit_procedure_row, list<map<string, string> > & visit_reason_rows, list<map<string, string> > & visit_condition_rows)
{
 hl7_value answer;
 answer[0] = "ZPR";
 answer[1] = visit_procedure_row["days_units"];
 answer[3] = visit_procedure_row["tooth_surface"];
 answer[5] = ""; // infection? (Y/N)
 answer[8] = ""; // TODO: visit_procedure_row["cpt_hcpcs"];

 int least_condition_index = 0;
 int this_condition_index = 0;
 unsigned int least_condition_sequence = 0;
 for (list<map<string, string> >::iterator j = visit_reason_rows.begin(); j != visit_reason_rows.end(); j++)
 {
  if ((*j)["procedure_id"] == visit_procedure_row["id"]) // reason is for this procedure
  {
   unsigned int k = 1;
   for (list<map<string, string> >::iterator i = visit_condition_rows.begin(); i != visit_condition_rows.end(); i++, k++)
    if ((*j)["condition_id"] == (*i)["id"]) // reason is for this condition
    {
     this_condition_index = StringToInt((*i)["index_number"]);
     if (this_condition_index < least_condition_index || least_condition_sequence == 0)
     {
      least_condition_sequence = k;
      least_condition_index = this_condition_index;
     }
    }
  }
 }
 if (least_condition_sequence)
  answer[9] = IntToString(least_condition_sequence); // sequence number of diagnosis causing this procedure

 // TODO: examine possibility
 answer[10] = ""; // PROV_X12_CLASS code
 answer[11] = ""; // PROV_X12_SPEC code
 answer[12] = ""; // PROVIDER_TYPE code, to end
 return answer;
}

// TODO: we have public health nurses, this has to be somewhere ...
hl7_value hl7_ZPN (unsigned int sequence_number, map<string, string> & visit_row)
{
 hl7_value answer;
 answer[0] = "ZPN";
 answer[1] = IntToString(sequence_number); // but not applicable for DW-1?
 answer[2] = ""; // TODO: PHN_ACTIVITY code // TODO: might be able to scrounge this up
 answer[3] = ""; // TODO: PHN_INTERV_LEVEL code, to end
 return answer;
}

hl7_value hl7_ZDP (unsigned int sequence_number, map<string, string> & visit_procedure_row)
{
 hl7_value answer;
 answer[0] = "ZDP";
 answer[1] = IntToString(sequence_number);
 answer[2] = visit_procedure_row["tooth_position"]; // DENTAL_OPSITE code (tooth number? yes, and a few other values) // TODO: we have comma-separated values here for no good reason!
 answer[3] = visit_procedure_row["tooth_surface"]; // to end, expected to be (m)esial, (d)istal, (o)cclusial, (l)inqual, (b)uccal (field length is actually 5)
 return answer;
}

// TODO: are immunizations also procedures? (yes, but not all CVX or CPT have equivalents in the other)
// TODO: who at the tribe would do these? how do they record it currently?
// CPT-4 code 90471 = first vaccine/toxoid
// CPT-4 code 90472 = every other vaccine/toxoid
// CPT-4 code 90744 = Hep B kid/adolescent
// also:
// CPT-4 code 82947 = glucose lab test (lots of lab tests have CPT codes)
//
// http://www.cdc.gov/nip/registry/
// someone, somewhere, has a CPT <-> CVX mapping
hl7_value hl7_ZIM (unsigned int sequence_number, map<string, string> & visit_immunization_row)
{
 hl7_value answer;
 answer[0] = "ZIM";
 answer[1] = IntToString(sequence_number);
 answer[2] = ""; // CVX code (prod) // TODO: HL7_IMMUN_CVX code
 answer[3] = ""; // MVX code (mfg) // TODO: HL7_IMMUN_MVX code
 answer[4] = ""; // ihs mod // TODO: HL7_IMMUNIZATION code
 answer[5] = ""; // old ihs // TODO: IHS_IMMUNIZATION code
 answer[6] = ""; // to end
 return answer;
}

hl7_value hl7_ZMD (unsigned int sequence_number, map<string, string> & pharm_medication_row)
{
 hl7_value answer;
 answer[0] = "ZMD";
 answer[1] = IntToString(sequence_number);
 answer[2] = pharm_medication_row["drug_name"]; // drug name
 answer[3] = PadString(StripAllExcept(pharm_medication_row["ndc_number"], uints), pad_left, '0', 11); // NDC code
 answer[4] = ""; // TODO: VA_DRUG_CLASS code, isn't in our pharmacy_info table (but available from computer-rx?)
                 // is this the same as schedule-I, schedule-II, etc.?
                 // NO. looked at the code table online, this describes the -type- (intended effect) of the drug
                 // this is a very long list. not a good idea to hardcode. is not the same as generic/brand identification of "essential" drug in any given drug. this is by method/effect of action.
 answer[5] = pharm_medication_row["quantity_filled"]; // to end, medication quantity (decimal)
 return answer;
}

// TODO: we need education values too
// hl7_value hl7_OBX_LAB (unsigned int sequence, string LOINC, string result, string units, string low, string high);
// hl7_value hl7_OBX_MSR (unsigned int sequence, string code, string value);
// hl7_value hl7_OBX_PED (unsigned int sequence, string topic, string response, Ptime length);

vector<hl7_value> hl7_OBX_measurements (list<map<string, string> > & visit_measurement_rows)
{
 unsigned int k = 1;
 vector<hl7_value> answer;
 hl7_value temp;

 // measurements that have an MSR code
 for (list<map<string, string> >::iterator i = visit_measurement_rows.begin(); i != visit_measurement_rows.end(); i++)
 {
  if (!(*i)["msr_code"].empty())
  {
   temp[0] = "OBX";
   temp[1] = IntToString(k);
   temp[2] = "MSR";
   temp[3] = (*i)["msr_code"];
   temp[5] = (*i)["response_metric"];
   answer.push_back(temp);
   k++;
  }
 }

 // labs
 for (list<map<string, string> >::iterator i = visit_measurement_rows.begin(); i != visit_measurement_rows.end(); i++)
 {
  if (!(*i)["loinc_code"].empty())
  {
   hl7_value temp;
   temp[0] = "OBX";
   temp[1] = IntToString(k);
   temp[2] = "LAB";
   temp[3] = hl7_values(1, hl7_values("ss", (*i)["loinc_code"], (*i)["question_item"])); // LOINC ^ local lab test name
   temp[5] = (*i)["response_metric"];
   temp[6] = (*i)["unit_item"]; // unit of measure
   if ((*i)["min_response_metric"].empty() || (*i)["max_response_metric"].empty())
    temp[7] = "";
   else
    temp[7] = hl7_values(1, hl7_values("ss", (*i)["min_response_metric"], (*i)["max_response_metric"])); // low ^ high (reference values)
   answer.push_back(temp);
   k++;
  }
 }

 return answer;
}

hl7_value hl7_OBX_CPT (unsigned int sequence_number, map<string, string> & visit_procedure_row)
{
 hl7_value answer;
 answer[0] = "OBX";
 answer[1] = IntToString(sequence_number);
 answer[2] = "CPT";
 answer[3] = visit_procedure_row["cpt_hcpcs"];
 answer[5] = "1"; // CPT Quantity ... Michael Foley says "1" is a safe value from looking at HL7 files generated by RPMS
 return answer;
}

/*
vector<hl7_value> hl7_OBX_measurements (list<map<string, string> > & visit_measurement_rows)
{
 unsigned int k = 1;
 vector<hl7_value> answer;
 hl7_value temp;

 // measurements
 for (list<map<string, string> >::iterator i = visit_measurement_rows.begin(); i != visit_measurement_rows.end(); i++)
 {
  hl7_PrometheusToHL7_add_useful_msr(k, answer, "height", (*i)["height"]);
  hl7_PrometheusToHL7_add_useful_msr(k, answer, "weight", (*i)["weight"]);
  hl7_PrometheusToHL7_add_useful_msr(k, answer, "temperature", (*i)["temperature"]);
  hl7_PrometheusToHL7_add_useful_msr(k, answer, "pulse", (*i)["pulse"]);
  hl7_PrometheusToHL7_add_useful_msr(k, answer, "foc", (*i)["foc"]);
  hl7_PrometheusToHL7_add_useful_msr(k, answer, "respirations", (*i)["respirations"]);
  hl7_PrometheusToHL7_add_useful_msr(k, answer, "waist", (*i)["waist"]);

  // we don't support multi-field things above, blood pressure is likely multi-field one way or another
  if ((*i)["blood_pressure_hi"] != "" &&
      (*i)["blood_pressure_low"] != "")
  {
   hl7_value temp;
   temp[0] = "OBX";
   temp[1] = IntToString(k);
   temp[2] = "MSR";
   temp[3] = "04"; // blood pressure (single value?)
   temp[5] = (*i)["blood_pressure_hi"] + "/" + (*i)["blood_pressure_low"];
   answer.push_back(temp);
   k++;
  }
 }

 // labs
 for (list<map<string, string> >::iterator i = visit_measurement_rows.begin(); i != visit_measurement_rows.end(); i++)
 {
  hl7_PrometheusToHL7_add_useful_lab (k, answer, "glu", (*i)["glue"]);
  hl7_PrometheusToHL7_add_useful_lab (k, answer, "ha1c", (*i)["ha1c"]);
  hl7_PrometheusToHL7_add_useful_lab (k, answer, "hdl_c", (*i)["hdl_c"]);
  hl7_PrometheusToHL7_add_useful_lab (k, answer, "ldl_c", (*i)["ldl_c"]);
  hl7_PrometheusToHL7_add_useful_lab (k, answer, "trig", (*i)["trig"]);
 }

 return answer;
}
*/

/*
void hl7_PrometheusToHL7_add_useful_msr (unsigned int & sequence_number, vector<hl7_value> & measurements, string field_name, string field_value)
{
 string code = "";
 if (field_name == "height")
  code = "01";
 if (field_name == "weight")
  code = "02";
 if (field_name == "temperature")
  code = "03";
 if (field_name == "pulse")
  code = "05";
 if (field_name == "foc")
  code = "06";
 if (field_name == "respirations")
  code = "12";
 if (field_name == "waist")
  code = "50";

 if (field_value != "" && code != "")
 {
  hl7_value temp;
  temp[0] = "OBX";
  temp[1] = IntToString(sequence_number);
  temp[2] = "MSR";
  temp[3] = code;
  temp[5] = field_value;
  measurements.push_back(temp);
  sequence_number++;
 }
}
*/

// CLINICAL_MEASURE
/*
code    description                  prometheus field
01	HEIGHT                       height
02	WEIGHT                       weight
03	TEMPERATURE                  temperature
04	BLOOD PRESSURE               blood_pressure_hi / blood_pressure_low
05	PULSE                        pulse
06	HEAD CIRCUMFERENCE           foc // frontal-occipital circumference
12	RESPIRATIONS                 respirations
50	ABDOMINAL GIRTH              waist

25	CARDIAC EJECTION FRACTION    // %, via echocardiography
51	FUNDAL HEIGHT                // fetal
52	FETAL HEART TONES            // fetal
56	PRESENTATION                 // fetal (cephalic, oblique, transverse?) (presentation, breech, frank breech, complete, incomplete, footling, double footling, compound, shoulder, flexion, face, brow)
57	CERVIX DILATATION            // fetal
58	EFFACEMENT                   // fetal
59	STATION (PREGNANCY)          // fetal
09	HEARING                      // hearing
10	AUDIOMETRY                   // hearing
07	VISION UNCORRECTED           // vision
08	VISION CORRECTED             // vision
11	TONOMETRY                    // vision, pressure in the eye, hcpcs 92100 (why does this matter?)
15	REFRACTIVE ERROR             // vision
21	PEAK FLOW                    // breathing
20	O2 SATURATION                // not the same as spo2?
80	PAIN                         // ?
55	EDEMA                        // would describe swelling (compressible or not) caused by extra fluids in organs
*/

/*
void hl7_PrometheusToHL7_add_useful_lab (unsigned int & sequence_number, vector<hl7_value> & measurements, string field_name, string field_value)
{
 string code = "";
 string unit = "";
 if (field_name == "glu")
 {
  code = "2345-7";
  unit = "mg/dL";
 }
 if (field_name == "ha1c")
 {
  code = "4548-4";
  unit = ""; // percentage
 }
 if (field_name == "hdl_c")
 {
  code = "2085-9";
  unit = "mg/dL";
 }
 if (field_name == "ldl_c")
 {
  code = "2089-1";
  unit = "mg/dL";
 }
 if (field_name == "trig")
 {
  code = "2571-8";
  unit = "mg/dL";
 }

 if (field_value != "" && code != "")
 {
  hl7_value temp;
  temp[0] = "OBX";
  temp[1] = IntToString(sequence_number);
  temp[2] = "LAB";
  temp[3] = hl7_values(1, hl7_values("ss", code, field_name)); // LOINC ^ local lab test name
  temp[5] = field_value;
  temp[6] = unit; // unit of measure
  temp[7] = ""; // low ^ high (reference values)
  measurements.push_back(temp);
  sequence_number++;
 }
}
*/

// LOINC
/*
loinc   description                field
2345-7	Glucose                    glu // we have m_glu, but glu is what we used in other segment
4548-4	HgbA1C                     ha1c
2085-9	HDL Cholesterol            hdl_c
2089-1	LDL Cholesterol            ldl_c
2571-8	Triglycerides              trig
2857-1	Prostate Specific Antigen  //
2335-8	Fecal Occult Blood         //
19762-4	Pap Smear                  //
*/

//---------------------------------------------------------------------------

// temporary functions, should be replaced with some sort of table layout later
// this was easier (test vs. production servers, etc.) and more flexible (function vs. array), in case of oddity
string hl7_PrometheusToHL7_state_types_numbers (string prometheus_state_id)
{
 unsigned int _state_id = StringToInt(prometheus_state_id);
 switch (_state_id)
 {
  // the gaps in their numbers are not my fault. but they're there.
  case 154: return "01";
  case 155: return "02";
  case 156: return "04";
  case 157: return "05";
  case 158: return "06";
  case 159: return "08";
  case 160: return "09";
  case 161: return "10";
  case 162: return "11";
  case 163: return "12";
  case 164: return "13";
  case 165: return "15";
  case 166: return "16";
  case 167: return "17";
  case 168: return "18";
  case 169: return "19";
  case 170: return "20";
  case 171: return "21";
  case 172: return "22";
  case 173: return "23";
  case 174: return "24";
  case 175: return "25";
  case 176: return "26";
  case 177: return "27";
  case 178: return "28";
  case 179: return "29";
  case 180: return "30";
  case 181: return "31";
  case 182: return "32";
  case 183: return "33";
  case 184: return "34";
  case 185: return "35";
  case 186: return "36";
  case 187: return "37";
  case 188: return "38";
  case 189: return "39";
  case 190: return "40";
  case 191: return "41";
  case 192: return "42";
  case 193: return "44";
  case 194: return "45";
  case 195: return "46";
  case 196: return "47";
  case 197: return "48";
  case 198: return "49";
  case 199: return "50";
  case 200: return "51";
  case 201: return "53";
  case 202: return "54";
  case 203: return "55";
  case 204: return "56";
  default: return "99";
 }
}

string hl7_PrometheusToHL7_state_types_abbreviations (string prometheus_state_id)
{
 unsigned int _state_id = StringToInt(prometheus_state_id);
 switch (_state_id)
 {
  // the gaps in their numbers are not my fault. but they're there.
  case 154: return "AL";
  case 155: return "AK";
  case 156: return "AZ";
  case 157: return "AR";
  case 158: return "CA";
  case 159: return "CO";
  case 160: return "CT";
  case 161: return "DE";
  case 162: return "DC";
  case 163: return "FL";
  case 164: return "GA";
  case 165: return "HI";
  case 166: return "ID";
  case 167: return "IL";
  case 168: return "IN";
  case 169: return "IA";
  case 170: return "KS";
  case 171: return "KY";
  case 172: return "LA";
  case 173: return "ME";
  case 174: return "MD";
  case 175: return "MA";
  case 176: return "MI";
  case 177: return "MN";
  case 178: return "MS";
  case 179: return "MO";
  case 180: return "MT";
  case 181: return "NE";
  case 182: return "NV";
  case 183: return "NH";
  case 184: return "NJ";
  case 185: return "NM";
  case 186: return "NY";
  case 187: return "NC";
  case 188: return "ND";
  case 189: return "OH";
  case 190: return "OK";
  case 191: return "OR";
  case 192: return "PA";
  case 193: return "RI";
  case 194: return "SC";
  case 195: return "SD";
  case 196: return "TN";
  case 197: return "TX";
  case 198: return "UT";
  case 199: return "VT";
  case 200: return "VA";
  case 201: return "WA";
  case 202: return "WV";
  case 203: return "WI";
  case 204: return "WY";
  default: return "";
 }
}

/* DEPRECATED; use tribe_types.code
string hl7_PrometheusToHL7_tribe_types (string prometheus_tribe_id)
{
 unsigned int _tribe_id = StringToInt(prometheus_tribe_id);
 // official tribes.xls from IHS-NDW (second try at getting appendix C)
 switch (_tribe_id)
 {
  case 1: return "141"; // Absentee Shawnee Tribes Of Oklahoma
  case 16: return "031"; // Choctaw Nation Of Ok
  case 54: return "000"; // Non-Indian
  case 11: return "022"; // Cherokee Nation Of Ok
  case 81: return "137"; // Seminole Nation Of Oklahoma
  case 21: return "043"; // Creek Nation Of Ok Aka Muscogee (Creek) Tribe
  case 15: return "027"; // Chickasaw Nation Of Ok
  case 17: return "104"; // Citizen Band Pottawatomi
  case 39: return "059"; // Kickapoo Tribe Of Ok
  case 78: return "130"; // Sac And Fox Tribe Of Indians Of Oklahoma
  case 41: return "062"; // Kiowa Tribe Of Ok
  case 12: return "012"; // Cheyenne-Arapaho
  case 19: return "039"; // Comanche Tribe Of Ok
  case 47: return "032"; // Mississippi Band Choctaw
  case 63: return "091"; // Osage Tribe Of Ok
  case 69: return "097"; // Pawnee Tribe Of Ok
  case 50: return "084"; // Navajo Nation Of Arizona
  case 8: return "016"; // Caddo Tribe Of Ok
  case 93: return "170"; // Wichita Tribe Of Ok
  case 23: return "459"; // Delaware Tribe Of Ok
  case 65: return "079"; // Otoe-Missouria Tribe Of Ok
  case 72: return "102"; // Ponca Tribe Of Ok
  case 82: return "135"; // Seminole-Creek
  case 38: return "060"; // Kickapoo Tribe Of Indians Of Kansas
  case 84: return "019"; // Seneca-Cayuga Tribe Of Ok
  case 73: return "105"; // Prairie Band Pottawatomi Indians Of Kan.
  case 24: return "046"; // Delaware Tribe Of Western Ok
  case 35: return "056"; // Iowa Tribe Of Ok
  case 14: return "027"; // Chickasaw Choctaw
  case 87: return "285"; // Sisseton-Wahpeton Sioux Tribe Of Lake Traverse Res.
  case 2: return "223"; // Alabama & Coushatta Tribes Of Texas
  case 51: return "084"; // Navajo Nation Of New Mexico
  case 58: return "026"; // Northern Cheyenne Tribe
  case 83: return "138"; // Seneca Nation Of New York
  case 95: return "173"; // Wyandotte Tribe Of Ok
  case 5: return "011"; // Arapahoe
  case 28: return "142"; // Eastern Shawnee Tribe Of Oklahoma
  case 71: return "184"; // Peoria Tribe Of Ok
  case 61: return "089"; // Omaha Tribe Of Nebraska
  case 32: return "054"; // Hopi Tribe Of Az
  case 46: return "446"; // Minnesota Chippewa Tribe,mn-White Earth Band
  case 66: return "093"; // Ottawa Tribe Of Ok
  case 37: return "058"; // Kaw Tribe Of Ok
  case 26: return "023"; // Eastern Cherokee Tribe
  case 45: return "076"; // Miami Tribe Of Ok
  case 60: return "282"; // Oglala Sioux Tribe Of South Dakota
  case 4: return "231"; // Apache Tribe Of Ok
  case 94: return "171"; // Winnebago Tribe Of Winnebago Reservation Of Nebraska
  case 3: return "002"; // Aleut Tribe Of Alaska
  case 75: return "111"; // Pueblo Of Laguna
  case 59: return "084"; // Northern Navajo Tribe
  case 20: return "038"; // Confederated Tribes Of The Colville Reservation
  case 97: return "410"; // Yurok Tribe Of Hoopa Valley, Ca
  case 96: return "174"; // Yakima Tribe
  case 7: return "015"; // Blackfeet Tribe Of Montana
  case 79: return "033"; // Santa Ynez Chumash Mission Band Indian
  case 9: return "452"; // Catawba Indian Nation
  case 42: return "241"; // Lac Courte Oreilles Indian
  case 44: return "280"; // Lower Brule Sioux Tribe
  case 55: return "011"; // Northern Arapaho Tribe Of Wyoming
  case 18: return "037"; // Coeur D'alene Tribe Of Idaho
  case 22: return "044"; // Crow Nation Of Montana
  case 77: return "125"; // Quapaw Tribe Of Ok
  case 85: return "372"; // Shoshone
  case 89: return "161"; // Tonkawa Tribe Of Ok
  case 91: return "166"; // Ute Mountain Ute Tribe
  case 40: return "199"; // Kickapoo Tribe Of Texas
  case 56: return "011"; // Northern Arapahoe Tribe
  case 76: return "123"; // Pueblo Of Zia, New Mexico
  case 68: return "188"; // Passamaquoddy Tribe Of Maine
  case 6: return "003"; // Bering Straits Eskimo
  case 92: return "340"; // Viejas Indians Of Ca
  case 31: return "196"; // Grand Traverse Ottawa
  case 30: return "290"; // Fort Belknap Gros Ventre
  case 48: return "428"; // Miwok Indians Of Ca
  case 36: return "109"; // Isleta
  case 29: return "372"; // Eastern Shoshone Tribe Of Indians
  case 27: return "084"; // Eastern Navajo Tribe
  case 74: return "110"; // Pueblo Of Jemez, New Mexico
  case 70: return "097"; // Pawnee/Yakimo
  case 86: return "285"; // Sisseton-Wahpeton Dakotah Nation
  case 80: return "120"; // Santo Domingo Tribe Of Pueblo
  case 57: return "011"; // Northern Arapahoe Tribe Of Montana
  case 52: return "640"; // Nome Eskimo Community
  case 67: return "196"; // Ottawa/Chippewa
  case 62: return "089"; // Omaha Tribe Of South Dakota
  // various "other" values
  case 13: return "999"; // 022 Chickamauga Cherokee -- might not be pleased
  case 53: return "997"; // Non-Enrolled Tribal Member
  case 98: return "999"; // Unknown
  case 64: return "999"; // Other
  case 34: return "997"; // Indian-Non Tribal Member
  case 33: return "999"; // Indian-Non Enrolled
  case 90: return "999"; // Unlisted Tribe
  case 43: return "999"; // Little Shell Band Of Chippewah
  case 25: return "999"; // Doyton Tribe Of Alaska
  case 0: return "000"; //
  default: return "000";
  // 000 means non-indian (not recognized or not indian)
  // 999 means indian (not listed, etc.)
 }

 // this was apparently for a -different- list of IHS codes
 // no // switch (_tribe_id)
 {
  case 1: return "1";
  case 16: return "47";
  case 11: return "40";
  case 81: return "264";
  case 21: return "168";
  case 15: return "43";
  case 17: return "48";
  case 39: return "126";
  case 78: return "248";
  case 41: return "128";
  case 12: return "41";
  case 19: return "54";
  case 47: return "161";
  case 63: return "182";
  case 69: return "194";
  case 50: return "170";
  case 8: return "27";
  case 93: return "326";
  case 23: return "75";
  case 65: return "184";
  case 72: return "203";
  case 82: return "264";
  case 38: return "126";
  case 84: return "267";
  case 73: return "207";
  case 24: return "75";
  case 35: return "112";
  case 14: return "43";
  case 87: return "276";
  case 2: return "4";
  case 51: return "170";
  case 58: return "174";
  case 83: return "266";
  case 95: return "329";
  case 5: return "8";
  case 28: return "79";
  case 71: return "197";
  case 61: return "178";
  case 32: return "104";
  case 46: return "159";
  case 66: return "183";
  case 37: return "122";
  case 26: return "78";
  case 45: return "156";
  case 60: return "177";
  case 4: return "7";
  case 94: return "327";
  case 3: return "512";
  case 75: return "213";
  case 59: return "170";
  case 20: return "57";
  case 97: return "336";
  case 7: return "20";
  case 79: return "258";
  case 9: return "35";
  case 42: return "133";
  case 44: return "142";
  case 55: return "8";
  case 18: return "51";
  case 22: return "70";
  case 77: return "229";
  case 85: return "273";
  case 89: return "304";
  case 91: return "320";
  case 40: return "127";
  case 56: return "8";
  case 76: return "226";
  case 68: return "192";
  case 92: return "34";
  case 31: return "94";
  case 30: return "86";
  case 48: return "30";
  case 36: return "212";
  case 29: return "273";
  case 27: return "170";
  case 74: return "211";
  case 70: return "194";
  case 86: return "276";
  case 80: return "223";
  case 57: return "8";
  case 52: return "484";
  case 67: return "94";
  case 62: return "178";
  default: return "";
 }

}
*/

string hl7_PrometheusToHL7_insurance_types (string prometheus_insurance_id)
{
 unsigned int _insurance_id = StringToInt(prometheus_insurance_id);
 switch (_insurance_id)
 {
  case 12: return "PVT"; // ppo
  case 13: return "PVT"; // hmo
  case 14: return "MCD"; // hmo medicaid
  case 15: return "MCD"; // medicaid
  case 16: return "MCR"; // medicare A
  case 17: return "MCR"; // medicare B
  case 18: return "PVT"; // vet
  case 20: return "MCR"; // medicare D
  case 21: return "MCR"; // medicare advantage - jhaley 20110715
  default: return "";
 }
}

string hl7_PrometheusToHL7_service_types (string prometheus_service_id)
{
 unsigned int _service_id = StringToInt(prometheus_service_id);
 switch (_service_id)
 {
  case  54: return "A3"; // AMBULANCE
  case  56: return "35"; // AUDIOLOGY
  case  59: return "62"; // CANCER CHEMOTHERAPY
  case  57: return "02"; // CARDIOLOGY
  case  60: return "A6"; // CHIROPRACTIC
  case  74: return "56"; // DENTAL
  case  75: return "05"; // DERMATOLOGY
  case  76: return "63"; // RADIOLOGY
  case  78: return "92"; // DIALYSIS
  case  79: return "67"; // DIETARY
  case  81: return "07"; // ENT
  case  82: return "69"; // ENDOCRINOLOGY
  case  87: return "28"; // FAMILY PRACTICE
  case  88: return "B8"; // GASTROENTEROLOGY-HEPATOLOGY
  case  89: return "01"; // GENERAL
  case  90: return "11"; // HOME CARE
  case  99: return "13"; // INTERNAL MEDICINE
  case 101:
  case 102: return "76"; // LABORATORY SERVICES
  case 106:
  case 107: return "14"; // MENTAL HEALTH (PSYCHIATRY)
  case 108: return "49"; // NEPHROLOGY
  case 109: return "37"; // NEUROLOGY
  case 111: return "B5"; // NURSING
  case 114:
  case 115: return "B9"; // ONCOLOGY-HEMATOLOGY
  case 116: return "17"; // OPHTHALMOLOGY
  case 119: return "18"; // OPTOMETRY
  case 121: return "19"; // ORTHOPEDIC
  case 129: return "20"; // PEDIATRIC
  case 132: return "39"; // PHARMACY
  case 133: return "34"; // PHYSICAL THERAPY
  case 135: return "29"; // PLASTIC SURGERY
  case 136: return "65"; // PODIATRY
  case 142: return "C2"; // PULMONOLOGY
  case 144: return "63"; // RADIOLOGY
  case 145: return "21"; // REHABILIATION
  case 146: return "38"; // RHEUMATOLOGY
  case 147: return "B5"; // NURSING
  case 152:
  case 153: return "23"; // SURGICAL
  case 154: return "C1"; // NEUROSURGERY
  case 155:
  case 156:
  case 157:
  case 158:
  case 159:
  case 160:
  case 161: return "23"; // SURGICAL
  case 164: return "75"; // UROLOGY
  case 165: return "72"; // MAMMOGRAPHY
  case 170: return "C5"; // URGENT CARE
  case 171: return "80"; // URGENT CARE
  case 172: return "10"; // GYNECOLOGY
  case 173: return "23"; // SURGICAL
  case 175:
  case 176: return "23"; // SURGICAL
  case 184: return "16"; // OBSTETRICS
  case   0: return "XX"; // DEFAULT CODE FOR NULLS/BLANKS
   default: return "25"; // OTHER
 }
}

// DONE -c HACK:  not no mo' - we now link users to disciplines directly; fetched in dataset
string hl7_PrometheusToHL7_user_discipline (string prometheus_user_id)
{
 unsigned int _user_id = StringToInt(prometheus_user_id);
 switch (_user_id)
 {
  case  68: /* Maqbool */ return "00"; // MD
  case   8: /* Cooper  */ return "88"; // data-entry
  case  24: /* Thorpe  */ return "95"; // mental-health, master's [MS]
  case 129: /* Boren   */ return "00"; // MD
  case 125: /* Foreman */ return "88"; // data-entry
  case  45: /* Shannon */ return "94"; // mental-health, ba/bs
  case  36: /* Shirey  */ return "57"; // administrative [but why so many?]
  case  19: /* Putman  */ return "94"; // mental-health, ba/bs
  case  37: /* Lollar  */ return "21"; // nurse-pract [verify]
  case  33: /* Gillham */ return "21"; // nurse-pract [verify]
  case  89: /* Bridwel */ return "11"; // phys-assist
  case  67: /* Villine */ return "52"; // dentist
  case  43: /* States  */ return "13"; // public-health-nurse
  case 110: /* Sanders */ return "94"; // mental-health, master's (M Educ.)
  case 140: /* Waggone */ return "48"; // alcohol/substance counselor [BSW]
  case  40: /* Daugher */ return "57"; // administrative [LPN]
  case  42: /* Walters */ return "88"; // data-entry [third-party!]
  case 128: /* Warden  */ return "57"; // admin
  case  44: /* Smith   */ return "05"; // LPN
  case 118: /* Carpent */ return "21"; // nurse-pract [verify]
  case 131: /* Tilley  */ return "29"; // dietitian
  case  32: /* Glendin */ return "88"; // data-entry
  case  27: /* Little  */ return "05"; // LPN
  case 115: /* Shaw    */ return "46"; // dental-hygienist
  case 124: /* Gallego */ return "57"; // admin
  case 143: /* Gentry  */ return "05"; // COM, LPN
  case 112: /* Steves  */ return "05"; // COM, Student??? (LPN, say paul)
  case 165: /* Masilon */ return "88"; // LAC??? data-entry, says paul
  case 178: /* Cody    */ return "88"; // LAC??? data-entry, says paul
  case  48: /* Graham  */ return "57"; // DIAB, Admin
  case  10: /* Denniso */ return "29"; // DIAB, Dietitian
  case  7 : /* Harris  */ return "15"; // Dietetic
  case  11: /* Rea     */ return "15"; // DIAB, recreational therapist
  case 123: /* Steve D */ return "15"; // DIAB, wellness instruc.
  case 136: /* Baker   */ return "15"; // DIAB, wellness instr. [FIX USER's TAXENTITYID]
  case 155: /* Bread   */ return "15"; // DIAB, welness tech, Student???
  case 164: /* Kayci L */ return "15"; // DIAB wellness tech
  case 158: /* Lawrenc */ return "29"; // DIAB - dietitian ... but ... adrienne tilley? ENTITY?
  case 154: /* Albert  */ return "15"; // wellness tech
  case 153: /* Whitten */ return "15"; // wellness tech
  case 184: /* Tallbea */ return "15"; // DIAB???
  case  29: /* Beasley */ return "00"; // BEH, "Dr."
  case  38: /* Sloat   */ return "05"; // COM, Nurse
  case 152: /* Gibson  */ return "05"; // DIAB, RN
  case  26: /* Fillioe */ return "88"; // data-entry
  case  20: /* Grass   */ return "88"; // data-entry            [FIX USER's TAXENTITYID]
  case 185: /* Johnson */ return "57"; // 3rd party, nurse (admin?)
  case 161: /* Hale    */ return "88"; // data-entry: pharmacy tech, CHS specialist
  case 156: /* Harmon  */ return "88"; // data-entry
  case 163: /* E. John */ return "57"; // DIAB, admin
  case  13: /* Fran    */ return "57"; // DIAB, ... admin?
  case  63: /* Taihpea */ return "48"; // BEH??? alcohol BS
  case  65: /* Richard */ return "48"; // BEH??? alcohol BS
  case  96: /* Miller  */ return "19"; // BEH??? case worker,
  case  61: /* Mccune  */ return "19"; // BEH??? ....
  case  16: /* Gregory */ return "37"; // BEH??? family BS
  case  60: /* White   */ return "19"; // BEH??? ....
  case  62: /* Wheeler */ return "94"; // BEH??? .... BS
  case  50: /* Rena M  */ return "94"; // BEH??? social worker BS
  case  58: /* Chissoe */ return "19"; // BEH counselor .... [FIX USER's TAXENTITYID]
  case 135: /* Floyd   */ return "88"; // data-entry (CH)
  case 139: /* Knight  */ return "05"; // LPN
  case  31: /* Geraldi */ return "15"; // OTHER transportation
  case   5: /* Chancel */ return "66"; // RN, case manager
  case  22: /* C. Litt */ return "A8"; // pharmacy tech
  case  15: /* Mary P. */ return "57"; // admin
  case 167: /* Mcgeise */ return "57"; // admin (diab)
  case 149: /* Nevreka */ return "00"; // MD
  case  39: /* Leitka  */ return "88"; // data-entry -- radiographer? not for us ...
  case 137: /* Flanag  */ return "57"; // admin, but she's an RN also!
  case 159: /* Morris  */ return "23"; // lab technician
  case 150: /* Rossdeu */ return "66"; // admin, also RN, just like chancellor
  case 188: /* Kyte    */ return "11"; // phys-assist
  case 182: /* Hood    */ return "57"; // admin assist
  case 171: /* Moisier */ return "29"; // dietitian
  case 113: /* Vance   */ return "59"; // x-ray tech
  case 180: /* Maxwell */ return "66"; // case manager, LPN
  case  70: /* Taylor  */ return "23"; // lab tech
  case  95: /* Felton  */ return "57"; // nurse, clinic coord.
  case  12: /* Withrow */ return "57"; // admin (coord)
  case 195: /* Wood    */ return "00"; // DO, MD, Internal Medicine
  case 162: /* Burch   */ return "88"; // receptionist
  case 187: /* Silman  */ return "00"; // MD
  case  25: /* Hornsby */ return "09"; // pharmacist
  case 142: /* Gilmore */ return "57"; // admin
  case 101: /* Pahdoco */ return "88"; // data-entry recept.
  case  54: /* ------- */ return "57"; // --- matt!
  case  35: /* Knapp   */ return "88"; // data-entry [FIX TAX ENTITY]
  case 145: /* Estep   */ return "60"; // dental-assist
  case  52: /* Watson  */ return "88"; // data-entry
  case  69: /* ------- */ return "57"; // --- paul!
  case 141: /* Bettely */ return "88"; // receptionist
  case 100: /* Olzawsk */ return "46"; // hygienist [FIX TAX ENTITY]
  case 114: /* Ogden   */ return "09"; // pharmacist
  case  51: /* Francis */ return "88"; // data-entry; "office manager", my ass
  case 116: /* Wood    */ return "88"; // receptionist
  case  21: /* Romberg */ return "57"; // admin, of doom
  default: return "15"; // OTHER
 }
}

string hl7_PrometheusToHL7_vendor_types (string prometheus_service_id)
{
 unsigned int _service_id = StringToInt(prometheus_service_id);
 switch (_service_id)
 {
  // TODO: vague approximations!
  // we could also go by the ... name? on the entity? that seems nasty.

  case 84: // (ER physician)
  case 85: // (ER services)
  case 91: // (inpatient)
  case 92: // (outpatient)
  case 93: // (hospital services)
  case 94: // (hospital billing)
  case 171: // (urgent care)
   return "01"; // general hospital
                                                                               
  case 106: // (inpatient mental)
   return "03"; // psych hospital

  //case 168: // (respite care)
  // return "04"; // nursing home // what?  no.  let this be "All Other"

  case 87: // (family practice)
  case 89: // (general practice)
  case 138: // (primary phys.)
   return "05"; // physician

  case 119: // (optometry)
   return "06"; // optometrist

  case 74: // // (dentistry)
   return "07"; // dentist

  case 90: // (home health)
  case 111: // (nurse pract.)
  case 147: // (skilled nursing)
   return "08"; // nurse / home health

  case 77: // (diagnostic tests)
  case 101: // (lab)
  case 102: // (lab at little axe)
   //return "09"; // lab
   return "01"; // GM&S

  case 132: // (pharmacy)
   return "12"; // pharmacy

  case 54: // (ambulance)
  case 169: // (handicapped transportation)
   return "13"; // transportation

  case 76: // (diagnostic radiology)
  case 144: // (radiology)
   //return "15"; // x-ray
   return "01"; // GM&S

  case 60: // (chiropractor)
   return "17"; // chiropractor

  default:
   return "16";
 }
}

string hl7_PrometheusToHL7_task_lookups (map<string, string> visit_row)
{
 // everything I could think of at the moment, not guaranteed to be complete or absolutely accurate, sorry

 string answer = "25"; // "other"

 if (visit_row["group_id"] == "18") // supposedly not workload reportable, but when they loaded our last file, these were clearly counted
  answer = "56"; // dental
 else if (visit_row["group_id"] == "6")
  answer = "06"; // diabetic (there are group and individual education codes too)
 else if (visit_row["group_id"] == "2")
  answer = "C4"; // behavioral health; could also be "14" mental health (psychiatry)
 else if (visit_row["group_id"] == "10")
  answer = "39"; // pharmacy
 else if (visit_row["group_id"] == "17" || visit_row["group_id"] == "29")
  answer = "01"; // general (lac)
 /*else if (visit_row["group_id"] == "4")                    // 11 Home Care would fit better, but both are
  answer = "27"; // general preventative (community health)  // not workload reportable
 else if (visit_row["group_id"] == "12")
  answer = "27"; // general preventative (public health)*/
 else if (visit_row["group_id"] == "22")
  answer = "45"; // general preventative (public health nursing)

 // BEH
 if (visit_row["beh_service_id"] == "1") // mental health
  answer = "14"; // mental health
 if (visit_row["beh_service_id"] == "2") // substance abuse
  answer = "43"; // alcohol and substance
 if (visit_row["beh_service_id"] == "4") // residential care
  answer = "11"; // home care                                // not workload reportable

 // COM
 if (visit_row["com_dept_id"] == "4") // postpartum
  answer = "32"; // postpartum
 if (visit_row["com_dept_id"] == "1") // child               // TODO:  look at age for medical clinic visits
  answer = "20"; // pediatric

 if (visit_row["com_visit_id"] == "2") // education
  answer = "60"; // education classes

 // DIAB
 // TODO -c MAGIC_NUMBER: jhaley 20100913 - hard-coded event program id # not necessarily future-safe
 if (visit_row["program_id"] == "1") // diabetes education
 {
  if (visit_row["event_id"] != "0")
   answer = "98"; // diabetes education, group (REACH)
  else
   answer = "A1"; // diabetes education, individual (REACH)
 }

 // overrides!
 if (visit_row["method_id"] == "1" ||
     visit_row["visit_location_id"] == "3" ||
     visit_row["com_visit_id"] == "7")
  answer = "51"; // telephone
 if (visit_row["method_id"] == "9")
  answer = "42"; // mail

/*
 // TODO: not in tasks (is in appointments)
 if (visit_row["diab_activity_id"] == "59" || // exam
     visit_row["diab_activity_id"] == "61" || // clinic
     visit_row["diab_activity_id"] == "62" || // exam
     visit_row["diab_activity_id"] == "63" ||
     visit_row["diab_activity_id"] == "85" || // exam
     visit_row["appt_reason_id"] == "4"    || // exam
     visit_row["appt_reason_id"] == "37")     // treatment
  answer = "B7"; // foot clinic (we include exam, education)

 // TODO: not in tasks (is in appointments)
 if (visit_row["appt_reason_id"] == "35") // podiatry
  answer = "65"; // podiatry
 if (visit_row["appt_reason_id"] == "9" || // nutrition
     visit_row["appt_reason_id"] == "44") // health heart diet
  answer = "67"; // dietary

 // TODO: not in tasks (is in tax_entity)
 if (visit_row["beh_care_id"] == "3") // medical detox
  answer = "43"; // alcohol and substance
*/

 return answer;
}

string hl7_PrometheusToHL7_gender_types (string prometheus_gender_id)
{
 if (prometheus_gender_id == "0")
  return "U"; // unknown
 else if (prometheus_gender_id == "1")
  return "M"; // male
 else if (prometheus_gender_id == "2")
  return "F"; // female
 else
  return "O"; // other (XXY, XXXY, etc.)
}

hl7_value hl7_hack_split_name (string composed_name)
{
 hl7_value repetition_answer;
 hl7_value composite_answer;

 vector<string> split_name;
 SplitStringToVec(composed_name, " ", split_name);

 string suffix;
 string prefix;

 for (vector<string>::iterator i = split_name.begin(); i != split_name.end(); i++)
 {
  if (*i != "" && is_in(*i, "MR", "MR.", "MRS", "MRS.", "MS", "MS.", "SIR", "DR", "DR.", NULL))
  {
   prefix = *i;
   *i = "";
  }

  if (*i != "" && is_in(*i, "JR", "JR.", "SR", "SR.", "I", "II", "III", NULL))
  {
   suffix = *i;
   *i = "";
  }
 }

 // string debug_string;

 for (vector<string>::reverse_iterator i = split_name.rbegin(); i != split_name.rend(); i++)
  *i = StripAll(*i, ",.&!?'\"()*$%^#@");

 for (vector<string>::reverse_iterator i = split_name.rbegin(); i != split_name.rend(); i++)
  if (*i != "")
  {
   // debug_string += (*i + "^");
   composite_answer[0] = hl7_value(*i);
   *i = "";
   break;
  }

 unsigned int j = 1;
 for (vector<string>::iterator i = split_name.begin(); i != split_name.end(); i++)
  if (*i != "")
  {
   composite_answer[j++] = hl7_value(*i);
   // debug_string += (*i + "^");
  }

 // they wanted last^first^middle , no prefix and suffix? add later? whatever.
 // PSay(composed_name + "\n" + debug_string, m_debug, MB_OK, "");

 repetition_answer[0] = composite_answer;
 return repetition_answer;
}

string StripSurrounding(string incoming, string chars)
{
 return StripLeading(StripTrailing(incoming, chars), chars);
}
