#ifndef hl7_db_h
#define hl7_db_h

#include "hl7.h"

enum hl7_run_type {hl7_run_full, hl7_run_incremental};
void hl7_change_code (map<string, string> &entity, string have_sent_fieldname, bool look_at_sets, list<map<string, string> > &entity_sets, hl7_run_type full_or_inc);
bool hl7_assimilate_set(list<map<string, string> > a_set);
// define functions for converting particular database records into hl7 segments/values

// batch header
hl7_value hl7_BHS (hl7_batch & b, Pstamp export_stamp);
hl7_value hl7_ZHS (Pdate start, Pdate stop, string export_id);
// batch trailer
hl7_value hl7_ZTS (unsigned int record_count, unsigned int msh_count);
hl7_value hl7_BTS (unsigned int message_count, string message_comment);
// more generics
hl7_value hl7_MSH (hl7_batch & b, Pstamp event_stamp, string trigger_type, string message_id);
hl7_value hl7_EVN (string trigger_type, Pstamp event_stamp);

// common to A31 and A08 (person & visit)
hl7_value hl7_PID (hl7_batch & b, map<string, string> & person_row, map<string, string> & visit_row, list<map<string, string> > & person_name_rows, list<map<string, string> > & person_contact_info_rows);
hl7_value hl7_ZP2 (map<string, string> & person_row, list<map<string, string> > & person_name_rows, list<map<string, string> > & person_affiliation_rows, list<map<string, string> > & person_contact_info_rows);

// only in A31 (about person)
hl7_value hl7_ZRB (map<string, string> & person_row); // todo
hl7_value hl7_ZRD (map<string, string> & person_row, list<map<string, string> > & person_name_rows, list<map<string, string> > & person_affiliation_rows, list<map<string, string> > & person_contact_info_rows);
hl7_value hl7_ZRL (map<string, string> & person_name_row);
hl7_value hl7_ZRC (hl7_batch & b, map<string, string> & person_row, map<string, string> & person_chart_row);
hl7_value hl7_IN1 (unsigned int sequence_number, map<string, string> & person_insurance_row);
hl7_value hl7_ZIN (map<string, string> & person_insurance_row);

// only in A08 (about visit)
hl7_value hl7_PV1 (hl7_batch & b, map<string, string> & visit_row, list<map<string, string> > & visit_condition_rows, list<map<string, string> > & visit_procedure_rows);
hl7_value hl7_ZV1 (hl7_batch & b, map<string, string> & visit_row, list<map<string, string> > & visit_condition_rows, list<map<string, string> > & visit_procedure_rows);
hl7_value hl7_ZEN (list<map<string, string> > & visit_measurement_rows);
hl7_value hl7_ZVP (unsigned int sequence_number, map<string, string> & visit_row);
hl7_value hl7_DG1 (unsigned int sequence_number, map<string, string> & visit_condition_row);
hl7_value hl7_ZDX (map<string, string> & visit_condition_row);
hl7_value hl7_PR1 (unsigned int sequence_number, map<string, string> & visit_procedure_row);
hl7_value hl7_ZPR (map<string, string> & visit_procedure_row, list<map<string, string> > & visit_reason_rows, list<map<string, string> > & visit_condition_rows);
hl7_value hl7_ZPN (unsigned int sequence_number, map<string, string> & visit_row);
hl7_value hl7_ZDN (unsigned int sequence_number, map<string, string> & visit_procedure_row);
hl7_value hl7_ZDP (unsigned int sequence_number, map<string, string> & visit_procedure_row);
hl7_value hl7_ZIM (unsigned int sequence_number, map<string, string> & visit_immunization_row);
hl7_value hl7_ZMD (unsigned int sequence_number, map<string, string> & pharm_medication_row);

// hl7_value hl7_OBX_LAB (unsigned int sequence, string LOINC, string result, string units, string low, string high);
// hl7_value hl7_OBX_MSR (unsigned int sequence, string code, string value);
// hl7_value hl7_OBX_PED (unsigned int sequence, string topic, string response, Ptime length);

vector<hl7_value> hl7_OBX_measurements (list<map<string, string> > & visit_measurement_rows);
hl7_value hl7_OBX_CPT (unsigned int sequence_number, map<string, string> & visit_procedure_row);
void hl7_PrometheusToHL7_add_useful_msr (unsigned int & sequence_number, vector<hl7_value> & measurements, string field_name, string field_value);
void hl7_PrometheusToHL7_add_useful_lab (unsigned int & sequence_number, vector<hl7_value> & measurements, string field_name, string field_value);

// conversion functions from our codes to theirs
string hl7_PrometheusToHL7_state_types_numbers (string prometheus_state_id);
string hl7_PrometheusToHL7_state_types_abbreviations (string prometheus_state_id);
// DEPRECATED // string hl7_PrometheusToHL7_tribe_types (string prometheus_tribe_id);
string hl7_PrometheusToHL7_insurance_types (string prometheus_insurance_id);
string hl7_PrometheusToHL7_service_types (string prometheus_service_id);
string hl7_PrometheusToHL7_task_lookups (map<string, string> visit_row);
string hl7_PrometheusToHL7_gender_types (string prometheus_gender_id);
string hl7_PrometheusToHL7_user_discipline (string prometheus_user_id);
string hl7_PrometheusToHL7_vendor_types (string prometheus_service_id);

// hacking stuff for now
hl7_value hl7_hack_split_name (string composed_name);
string StripSurrounding(string incoming, string chars);

#endif
