#include "html_reporting.h"
#include "lookuptablewrapper.h"
#include "Filter.h"
#include "codeSpecialHtmlPreview.h"
#include "filter_export.h"

/*
 standard use:

 // get 'o' open for output (regular file, or any output stream, including the "append to in-memory string" stream)

 htmlStart(o, "My Report");
 for (...)
 {
  htmlSectionStart(o, should_page_break);
  htmlPrettyBar(o, "Section A");
  htmlSectionContinue(o);
  o << ...

   // embedded sub-sections
   htmlSectionStart(o, false);
   htmlPrettyBar(o, "SubSection 1");
   htmlSectionContinue(o);
   o << ...
   htmlSectionStop(o);

   htmlSectionStart(o, false);
   htmlPrettyBar(o, "SubSection 2");
   htmlSectionContinue(o);
   o << ...
   htmlSectionStop(o);

  htmlSectionStop(o);
 }
 htmlStop(o);

*/

//---------------------------------------------------------------------------
void htmlStart(ostream & o, string t, string s, bool show_time, bool show_user)
{
 o << "<html>"
      "<head>"
      "<style type=\"text/css\">"
      "<!--"
      "thead {display: table-header-group;}\n"
      "hr {height: 1px; border: 1px solid black; padding: 0px; margin: 0px;}\n"
      "th {text-align: left; overflow: hidden; font-weight: normal;}\n"
      "table {margin: 0px; border: 0px;}\n"
      "td {font: 11pt \"Arial\" sans-serif; overflow: hidden;}\n"
      ".datatable {border-collapse: collapse;}\n"
      ".datatable td {padding-right: 12px;}\n"
      ".underlined {border-bottom: 1px solid black;}\n"
      ".prettybar {width: 100%; font: 11pt \"Arial\" sans-serif; font-weight: bold; padding-bottom: 5px; border-bottom: 1px solid black;}\n"
      ".reporttime {text-align: left; vertical-align: top; white-space: nowrap;}\n"
      ".reportuser {text-align: right; vertical-align: top; white-space: nowrap;}\n"
      ".reportname {text-align: center; font: 13pt \"Arial\" sans-serif; font-weight: bold;}\n"
      ".reportsubname {text-align: center; font: 11pt \"Arial\" sans-serif; font-weight: normal;}\n"
      ".inblock {display: inline-block;}\n"
      ".inlabel {white-space: nowrap; padding-left: 5px; padding-right: 5px;}\n"
      ".invalue {background: #F0F0F0;}\n" // {border-bottom: 1px solid black;}\n"
      ".markup {border: 0px; padding: 0px; margin: 0px; margin-bottom: 5px;}\n"
      "@media print {.shaded {background-color: #E0E0E0;}}\n"
      "--></style>"
      "</head>"
      "<body>";
 htmlSectionStart(o, false);
 o << "<table cellspacing=0 width=100%><tr><td class='reporttime'>" << (show_time ? RightNow() : string("")) << "</td>"
      "<td class='reportname'>" << t << (s.empty() ? string("") : string("<br><span class='reportsubname'>" + s + "</span>")) << "</td>"
      "<td class='reportuser'>" << (show_user ? PrettyString(lookupTables->ValueOf("users", current_userid)) : string("")) << "</td></tr></table>";
 htmlSectionContinue(o);
}
//---------------------------------------------------------------------------
void htmlStop(ostream & o)
{
 htmlSectionStop(o);
 o << "</body>"
      "</html>";
}
//---------------------------------------------------------------------------
void htmlSectionStart(ostream & o, bool new_page)
{
 o << "<table cellspacing=0 width=100%"
   << (new_page ? " style='page-break-before: always;'" : "") << ">"
      "<thead>"
      "<tr><th align=left>";
}
//---------------------------------------------------------------------------
void htmlSectionContinue(ostream & o)
{
 o << "</th></tr>"
      "</thead>"
      "<tfoot>"
      "<tr>"
      "<td></td>"
      "</tr>"
      "</tfoot>"
      "<tbody><tr><td valign=top>";
}
//---------------------------------------------------------------------------
void htmlSectionStop(ostream & o)
{
 o << "</td></tr></tbody>"
      "</table>";
}
//---------------------------------------------------------------------------
void htmlPrettyBar(ostream & o, string t)
{
 o << "<div class='prettybar'>" << t << "</div>";
}
//---------------------------------------------------------------------------
bool htmlAutoGenerateFilterReport(classFilterObject * filter, string desc)
{
 if (!filter)
  return false;

 try
 {
  column_descriptions exp (filter);
  exp.ImportSettings(filter);

  // for those who don't get it -- i was raised in france. 'ze' makes fun of the french trying to say 'the' ... *sigh*
  ofstream ze_file;
  ze_file.open("autogenrep.html", ios::trunc | ios::out);
  if (!ze_file.is_open())
  {
   PSay("Failed to create temporary file for report output.", m_debug, MB_OK, "htmlAugoGenerateFilterReport");
   throw(97);
  }
  htmlStart(ze_file, desc);

  // go grab visible data with current settings
  vector<vector<string> > exported_data;
  vector<int>             max_col_sizes; // ignored, filled with zero
  vector<string>          header_data;
  vector<string>          val_cur;
/*
  val_cur.push_back("3");
  val_cur.push_back("4");
  val_cur.push_back("5");
*/
  int l = 0;
  for (vector<classCurrent>::iterator k = filter->CurrentVec.begin(); k != filter->CurrentVec.end(); k++, l++)
   if (k->showCurrent)
    val_cur.push_back(IntToString(l)/*k->currentItem*/);
  PSay(VecToCommaString(val_cur), m_debug, MB_OK, "html_report");

  exp.GetData(exported_data, max_col_sizes, false, val_cur);
  exp.GetHeaders(header_data);

  // start table
  ze_file << "<table cellspacing=0 width=100% class'datatable' style='margin-left: 0px;'><thead><tr class='underlined'>";
  for (vector<string>::iterator i = header_data.begin(); i != header_data.end(); i++)
   ze_file << "<th><i>" << *i << "</i></th>";
  ze_file << "</tr></thead><tbody>";

  for (vector<vector<string> >::iterator i = exported_data.begin(); i != exported_data.end(); i++)
  {
   ze_file << "<tr>";
   for (vector<string>::iterator j = i->begin(); j != i->end(); j++)
    ze_file << "<td>" << *j << "</td>";
   // PrettyDate, PrettyString, ... ?
   ze_file << "</tr>";
  }
  ze_file << "</tbody></table>\n";

  htmlStop(ze_file);
  ze_file.close();

  // apparently, we have to pass the fully-qualified path ... damn. might move this into the DoPrintPreview function? (always gonna be necessary? except not? i dunno. it's not much extra trouble here.)
  string real_cwd = GetCurrentDir().c_str();
  TformSpecialHtmlPreview::DoPrintPreview(real_cwd + "\\autogenrep.html");
 }
 catch (...)
 {
  return false;
 }
 return true;
}
//---------------------------------------------------------------------------

