//---------------------------------------------------------------------------

#include <vcl.h>
#pragma hdrstop

#include "launchARRAExemptionForm.h"
#include "reportARRAExemptionForm.h"
#include "codeSearchTaxEntity.h"
#include "codeSearchPerson.h"
//---------------------------------------------------------------------------
#pragma package(smart_init)
#pragma link "launchGeneric"
#pragma resource "*.dfm"
TformLaunchARRAExemptionForm *formLaunchARRAExemptionForm;
//---------------------------------------------------------------------------
__fastcall TformLaunchARRAExemptionForm::TformLaunchARRAExemptionForm(TComponent* Owner, string tax_entity_id,
                                                                      string person_id)
        : TformLaunchGeneric(Owner)
{
  CreateLink(txtProvider, btnProviderSearch, this, tax_entity_id, "tax_entities");
  CreateLink(txtPerson,   btnPersonSearch,   this, person_id,     "people");
  dateSigned->Date = Now();
}
//---------------------------------------------------------------------------
bool TformLaunchARRAExemptionForm::IsValidOnScreen()
{
  if (GetValidLinkId(txtProvider) == "")
  {
    PSay("Please select a valid provider to use as the addressee", m_missing_data, MB_OK);
    btnProviderSearch->SetFocus();
    return false;
  }
  if (GetValidLinkId(txtPerson) == "")
  {
    PSay("Please select a valid person to which this letter will be given", m_missing_data, MB_OK);
    btnPersonSearch->SetFocus();
    return false;
  }
  return true;
}
//---------------------------------------------------------------------------
bool TformLaunchARRAExemptionForm::RunReport()
{
  string provider_id = GetValidLinkId(txtProvider);
  map<string, string> provider_info;
  map<string, string> signer_info;

  string sql = "select tax_entities.name, tax_addresses.line1, tax_addresses.line2, tax_addresses.city, "
               "state_types.abbreviation, tax_addresses.zip, tax_addresses.phone_number "
               "from tax_entities "
               "left join tax_addresses on tax_addresses.id = case when tax_entities.physical_address_id > 0 "
               " then tax_entities.physical_address_id else tax_entities.billing_address_id end "
               "left join state_types on tax_addresses.state_id = state_types.id "
               "where tax_entities.id = '" + provider_id + "' ";

  if (!SqlToMap(report_transaction, sql, provider_info))
  {
    PSay("Failed to retrieve provider information", m_database_error, MB_OK);
    return false;
  }

  // get name/address/other info of ARRA user and tax entity for proper group (currently Contract Health only)
  sql = "select users.item, users.title, tax_entities.name, tax_addresses.line1, tax_addresses.city, "
        "state_types.abbreviation, tax_addresses.zip, tax_addresses.phone_number "
        "from groups "
        "inner join users on groups.arra_user_id = users.id "
        "inner join tax_entities on groups.arra_tax_entity_id = tax_entities.id "
        "left join tax_addresses on tax_addresses.id = case when tax_entities.physical_address_id > 0 "
        " then tax_entities.physical_address_id else tax_entities.billing_address_id end "
        "left join state_types on tax_addresses.state_id = state_types.id "
        "where groups.id = '" + lookupTables->IdOf("groups", "Contract Health") + "' ";

  if (!SqlToMap(report_transaction, sql, signer_info))
  {
    PSay("Failed to retrieve signer information", m_database_error, MB_OK);
    return false;
  }

  // jhaley 20110707: Create a Contract Health mailing task
  InsertCHSMailTask(NULL, GetValidLinkId(txtPerson),
                    string("ARRA form letter delivered to client for ") + txtProvider->Caption.c_str());

  TformReportARRAExemptionForm *reportForm = NULL;
  FORM_TRY
    reportForm = new TformReportARRAExemptionForm(NULL);
  FORM_CATCH

  // Header: Recipient name and address
  reportForm->lblProviderName->Caption = provider_info["name"].c_str();
  if (provider_info["line1"] != "")
    reportForm->memoProviderAddress->Lines->Add(FormatAddress(provider_info["line1"]).c_str());
  if (provider_info["line2"] != "")
    reportForm->memoProviderAddress->Lines->Add(FormatAddress(provider_info["line2"]).c_str());

  string city_state_zip = PrettyString(provider_info["city"]) + ", " + provider_info["abbreviation"] + "  " + provider_info["zip"];
  reportForm->memoProviderAddress->Lines->Add(city_state_zip.c_str());

  // Salutation: Recipient name
  reportForm->lblProviderSalutation->Caption = (provider_info["name"] + ":").c_str();

  // Add signing phone number to the end of the last sentence.
  int numlines = reportForm->richSummary->Lines->Count;
  string current_line = reportForm->richSummary->Lines->Strings[numlines - 1].c_str();
  current_line += " " + FormatPhoneNumber(signer_info["phone_number"]);
  reportForm->richSummary->Lines->Strings[numlines - 1] = current_line.c_str();

  // Footer
  // Use the indicated signing date
  reportForm->lblDateField->Caption = Pdate(dateSigned->Date).OutDate(mdy | yyyy).c_str();

  // Signing user/tax entity information
  reportForm->lblNameField->Caption     = signer_info["name"].c_str();
  reportForm->lblPositionField->Caption = (signer_info["item"] + ", " + signer_info["title"]).c_str();
  reportForm->lblAddressField->Caption  = (FormatAddress(signer_info["line1"]) + ", " + PrettyString(signer_info["city"]) + ", " +
                                           signer_info["abbreviation"] + "  " + FormatZipCode(signer_info["zip"])).c_str();
  reportForm->lblPhoneField->Caption    = FormatPhoneNumber(signer_info["phone_number"]).c_str();

  reportForm->reportARRA->Preview();

  return true;
}
//---------------------------------------------------------------------------
void __fastcall TformLaunchARRAExemptionForm::btnProviderSearchClick(TObject *Sender)
{
  vector<string> return_vector;

  FORM_TRY
    TformSearchTaxEntity *formProviderSearch = new TformSearchTaxEntity(NULL, &return_vector);
    formProviderSearch->ShowTaskModal(this);
  FORM_CATCH

  if (!return_vector.empty())
    CreateLink(txtProvider, btnProviderSearch, this, return_vector[0], "tax_entities");
}
//---------------------------------------------------------------------------
void __fastcall TformLaunchARRAExemptionForm::btnPersonSearchClick(TObject *Sender)
{
  vector<string> return_vector;

  FORM_TRY
    TformSearchPerson *formPersonSearch = new TformSearchPerson(NULL, &return_vector);
    formPersonSearch->ShowTaskModal(this);
  FORM_CATCH

  if (!return_vector.empty())
    CreateLink(txtPerson, btnPersonSearch, this, return_vector[0], "people");
}
//---------------------------------------------------------------------------

