//---------------------------------------------------------------------------

#include <vcl.h>
#pragma hdrstop

#include "launchAmountsByProviderService.h"
#include "reportAmountsByProviderService.h"
//---------------------------------------------------------------------------
#pragma package(smart_init)
#pragma link "codeSpecialGeneric"
#pragma resource "*.dfm"
TformLaunchAmountsByProviderService *formLaunchAmountsByProviderService;
//---------------------------------------------------------------------------
__fastcall TformLaunchAmountsByProviderService::TformLaunchAmountsByProviderService(TComponent* Owner)
        : TformSpecialGeneric(Owner)
{     
  dateBegin->Date = Now();
  dateEnd->Date = Now();

  services_filter.SetListViewHandle(listServices);
  services_filter.SetSecurity("services_provided");
  services_filter.AddColumnSpec("Service", "people", as_text, true, 180);
  services_filter.AddColumnSpec("Internal Id", "internal_id", as_int, false, 80);
  services_filter.FetchPreferences("launchamountsbyproviderservice.services_filter");
  services_filter.SetCheck(true);
  services_filter.SetCompress(true);
  services_filter.SetSQLStatement("select item, id from service_types where valid = 1");
  services_filter.RefreshDataSet();
  set<string> id_set;
  id_set.insert("0");
  id_set.insert(lookupTables->IdOf("service_types", "Respite Care")); // this is a special program under Community Health, and its information is seperate from Contract Health's services
  services_filter.SetCheckedWhereNotID(id_set, true);
}
//---------------------------------------------------------------------------
void __fastcall TformLaunchAmountsByProviderService::btnCloseClick(TObject *Sender)
{
 Close();
}
//---------------------------------------------------------------------------
void __fastcall TformLaunchAmountsByProviderService::btnRunClick(TObject *Sender)
{
  vector<string> checked_ids = services_filter.GetMultiID();
  if(! checked_ids.empty())
  {
    Processing(true);

    TIBTransaction * dbTransaction = new TIBTransaction(NULL);
    TIBDataSet * dbDataset = new TIBDataSet(NULL);
    string sql;
    string begin_date = Pdate(dateBegin->Date).OutDate(mdy | yyyy);
    string end_date =  Pdate(dateEnd->Date).OutDate(mdy | yyyy);
    string date_title = begin_date + " - " + end_date;

    try
    {
      dbTransaction->DefaultDatabase = dbDatabase;
      dbTransaction->StartTransaction();
      dbDataset->Database = dbDatabase;
      dbDataset->Transaction = dbTransaction;
      dbDataset->UniDirectional = false;

      ExecuteStatement(dbTransaction, "delete from temp_child_cob");
      ExecuteStatement(dbTransaction, "delete from temp_bill_amounts");
      ExecuteStatement(dbTransaction, "delete from temp_paid_visits");
      ExecuteStatement(dbTransaction, "delete from temp_obligated_report");
      // NOTE USES VIEW
      ExecuteStatement(dbTransaction, "insert into temp_child_cob select id, parent_task_id, tax_entity_id from visit_tasks "
                      "where visit_tasks.tax_entity_id != 0");
      ExecuteStatement(dbTransaction, "insert into temp_bill_amounts select bills.id, bills.task_id, sum(transactions.credit_amount - transactions.debit_amount) "
                      "from checks inner join purchase_reqs on purchase_reqs.check_id = checks.id "
                      "inner join bills on bills.purchase_req_id = purchase_reqs.id "
                      "left join transactions on transactions.bill_id = bills.id "
                      "and transactions.account_id in (select accounts.id from accounts where accounts.account_type_id = " + lookupTables->IdOf("account_types", "Cash") + ") "
                      "where checks.date_sent between '" + begin_date + "' and '" + end_date + "' "
                      "group by bills.id, bills.task_id, checks.date_sent");

      // COB tasks w children and has checks
      // NOTE USES VIEW
      sql = "insert into temp_paid_visits (service_provided_id, paid_visits, paid_members, paid_amount) "
            "select parent_cob.services_provided_id, "
            "count(child_cob.id) visit_count, count(distinct(parent_cob.person_id)) member_count, "
            "sum(bill_amounts.cash_amount) paid_invoices "
            "from cob_tasks parent_cob "
            "inner join temp_child_cob child_cob on parent_cob.id = child_cob.parent_task_id "
            "inner join temp_bill_amounts bill_amounts on bill_amounts.task_id = child_cob.id "
            "group by parent_cob.services_provided_id";
      ExecuteStatement(dbTransaction, sql);

      sql = "insert into temp_obligated_report (service_id, provider, service_type, paid_visit_count, paid_member_count, paid_amount) "
            "select service_types.id, tax_entities.name, service_types.item, "
            "sum(temp_paid_visits.paid_visits), "
            "sum(temp_paid_visits.paid_members), sum(temp_paid_visits.paid_amount) "
            "from tax_entities "
            "inner join services_provided on services_provided.tax_entity_id = tax_entities.id "
            "inner join service_types on services_provided.service_id = service_types.id "
            "left join temp_paid_visits on temp_paid_visits.service_provided_id = services_provided.id "
            "group by service_types.id, service_types.item, tax_entities.name";
      ExecuteStatement(dbTransaction, sql);

      // the quick reports stupidness has just about outdone itself. it refuses to show 0's in the report even
      // though i'm using the formatnumeric function. if i use the onprint with a regular text field, it doesn't
      // happen when the field is null, which is when i need it to. the expression field also refuses to do
      // the division for some reason. so, i'm selecting the final report into this unnecessary temp table
      // and running these update statemetns so i can select the whole thing at the end
      sql = "update temp_obligated_report set paid_visit_count = 0 where paid_visit_count is null";
      ExecuteStatement(dbTransaction, sql);
      sql = "update temp_obligated_report set paid_member_count = 0 where paid_member_count is null";
      ExecuteStatement(dbTransaction, sql);
      sql = "update temp_obligated_report set avg_paid_per_visit = (paid_amount / paid_visit_count) "
            "where paid_visit_count > 0";
      ExecuteStatement(dbTransaction, sql);
      sql = "update temp_obligated_report set avg_paid_per_visit = 0 where avg_paid_per_visit is null";
      ExecuteStatement(dbTransaction, sql);
      sql = "update temp_obligated_report set avg_paid_per_member = (paid_amount / paid_member_count) "
            "where paid_member_count > 0";
      ExecuteStatement(dbTransaction, sql);
      sql = "update temp_obligated_report set avg_paid_per_member = 0 where avg_paid_per_member is null";
      ExecuteStatement(dbTransaction, sql);

      sql = "select service_id, provider, service_type, "
            "paid_visit_count, paid_member_count, paid_amount, "
            "avg_paid_per_visit, avg_paid_per_member "
            "from temp_obligated_report "
            "where paid_member_count > 0 "
            "and service_id in (" + VecToCommaString(checked_ids) + ") "
            "order by service_id, service_type, provider";
      dbDataset->SelectSQL->Add(sql.c_str());
      dbDataset->Open();
      dbDataset->First();

      // create report form
      FORM_TRY
        TformReportAmountsByProviderService * report = new TformReportAmountsByProviderService(NULL);
        report->report->DataSet = dbDataset;
        report->lblDateTitle->Caption = date_title.c_str();
        report->lblUser->Caption = current_fullname.c_str();
        report->bandGroup->Expression = "service_id";
        report->txtProvider->DataSet = dbDataset;
        report->txtProvider->DataField = "provider";
        report->txtService->DataSet = dbDataset;
        report->txtService->DataField = "service_type";
        report->txtVisits->DataSet = dbDataset;
        report->txtVisits->DataField = "paid_visit_count";
        report->txtMembers->DataSet = dbDataset;
        report->txtMembers->DataField = "paid_member_count";
        report->exprPaid->Expression = "FORMATNUMERIC('$###,###,##0.00', paid_amount)";
        report->exprAvgPerVisit->Expression = "FORMATNUMERIC('$###,###,##0.00', avg_paid_per_visit)";
        report->exprAvgPerMember->Expression = "FORMATNUMERIC('$###,###,##0.00', avg_paid_per_member)";
        report->exprTotalVisits->Expression = "SUM(paid_visit_count)";
        report->exprTotalMembers->Expression = "SUM(paid_member_count)";
        report->exprTotalPaid->Expression = "FORMATNUMERIC('$###,###,##0.00', SUM(paid_amount))";
        report->exprTotalAvgPerVisit->Expression = "IF(SUM(paid_visit_count) > 0, FORMATNUMERIC('$###,###,##0.00', (SUM(paid_amount) / SUM(paid_visit_count))), '$0.00')";
        report->exprTotalAvgPerMember->Expression = "IF(SUM(paid_member_count) > 0, FORMATNUMERIC('$###,###,##0.00', (SUM(paid_amount) / SUM(paid_member_count))), '$0.00')";
        Processing(false);
        report->report->Preview();
      FORM_CATCH
      dbDataset->Close();
      dbTransaction->Commit();
      dbTransaction->StartTransaction();
      ExecuteStatement(dbTransaction, "delete from temp_child_cob");
      ExecuteStatement(dbTransaction, "delete from temp_bill_amounts");
      ExecuteStatement(dbTransaction, "delete from temp_paid_visits");
      ExecuteStatement(dbTransaction, "delete from temp_obligated_report");
      dbTransaction->Commit();
    }
    catch (...)
    {
      Processing(false);  
      PSay("Error running report", m_database_error, MB_OK);
      if (dbTransaction->Active)
        dbTransaction->Rollback();
    }

    delete dbTransaction;
    delete dbDataset;
  }
  else
    PSay("No services are checked", m_missing_data, MB_OK);
}
//---------------------------------------------------------------------------
void __fastcall TformLaunchAmountsByProviderService::FormShortCut(
      TWMKey &Msg, bool &Handled)
{
  Handled = true;
  if (Msg.CharCode == VK_RETURN)
    btnRun->Click();
  else
    Handled = false;

  if (!Handled)
    TformSpecialGeneric::FormShortCut(Msg, Handled);
}
//---------------------------------------------------------------------------
void __fastcall TformLaunchAmountsByProviderService::FormClose(
      TObject *Sender, TCloseAction &Action)
{
  TformSpecialGeneric::FormClose(Sender, Action);
}
//---------------------------------------------------------------------------

