//---------------------------------------------------------------------------

#include <vcl.h>
#pragma hdrstop

#include "launchAppointmentReport.h"
#include "reportClinicPatientLabels.h"
#include "codeSpecialSelectGroups.h"
#include "codeSearchTaxEntity.h"
#include "reportAppointmentListing.h"
#include "launchDiabRegistryReport.h"
#include "reportAppointmentOutcomeSummary.h"
#include "reportAppointmentListing.h"
#include "reportClinicAppointments.h"
#include "reportClinicSuperBill.h"
#include "reportDentalSuperBill.h"
#include "derivedheaders.h"
#include "launchIntakeFormReport.h"
#include "reportDiabAppointmentLetter.h"
//---------------------------------------------------------------------------
#pragma package(smart_init)
#pragma link "codeSpecialGeneric"
#pragma link "codeFrameCheckListDropDown"
#pragma link "codeBaseDropDown"
#pragma link "codeCheckListDropDown"
#pragma link "codePaddedBox"
#pragma resource "*.dfm"
TformLaunchAppointmentReport *formLaunchAppointmentReport;
//---------------------------------------------------------------------------
__fastcall TformLaunchAppointmentReport::TformLaunchAppointmentReport(TComponent* Owner, PApptOccurrenceMap *p_appt_set)
   : TformSpecialGeneric(Owner)
{
  refresh_what = refresh_none;
  dateBegin->Date = Date();
  dateEnd->Date = Date();
  all_appointments = NULL;

//  lookupTables->RegisterComboBox("users", cboEmployee);
//  lookupTables->RegisterComboBox("appt_reason_types", cboReasonLac);
  string groups_sql = "select id, item, valid from groups where valid = 1 and (has_appointments = 1 or id = 0) order by item";
  CreateLink(txtTaxEntity, btnTaxEntitySearch, this, "", "tax_entities");
  radioClinic->Enabled = false;

  if(optionsPermissions["table.tax_entities"]["view"] != "true")
    radioTaxEntity->Enabled = false;

  if (p_appt_set)
  {
    all_appointments = p_appt_set;
    has_scheduler_data = true;
  }
  else
  {
    all_appointments = new PApptOccurrenceMap();
    has_scheduler_data = false;
  }
  radioSchedule->Enabled = has_scheduler_data;
  radioSchedule->Checked = has_scheduler_data;
  WhichToPrintChange(NULL);

  // list of users
  string sql = "select users.item, groups.item, users.id from users, groups "
               "where users.group_id = groups.id and "
               "users.id > 0 and (users.valid = 1 or users.is_role = 1)";
  user_filter.SetListViewHandle(listUsers);
  user_filter.SetSecurity("tax_entities"); // TODO: yes, I know, that's wrong. but I don't want to give users.view to users ... close enough, right?
  user_filter.AddColumnSpec("User", "tax_entities", as_text, true, 120);
  user_filter.AddColumnSpec("Group", "tax_entities", as_text, true, 120);
  user_filter.AddColumnSpec("Internal Id", "internal_id", as_int, false, 80);
  user_filter.FetchPreferences("launchappointmentreport.user_filter");
  user_filter.SetIconStatus(false);
  user_filter.SetCheck(true);
  user_filter.SetSQLStatement(sql);
  user_filter.RefreshDataSet();
  // TODO: add+edit removed (don't care at the moment)
  user_request = atlas->RequestNotificationWhen("users.id", "0", rel_any, as_int, this,
                 RefreshHandler(&(TformLaunchAppointmentReport::AtlasRefresh)));

  // jhaley 20100921: Limit to appt_reason_types that are relevant to user's groups (this used to be just LAC reasons)
  set<string> group_perms = GetGroupPermissions("table.appointments", "", "view", "true");
  if (group_perms.find(lookupTables->IdOf("groups", "Shawnee Clinic")) != group_perms.end()) // SC must see LAC locations too
    group_perms.insert(lookupTables->IdOf("groups", "Little Axe Clinic"));
  string role_sql = "select item, " + CleverCurrentId("valid", "1", "Current", "Expired") + ", id "
                    "from appt_reason_types "
                    "where group_id = 0 or " + SafeInClause("group_id", SetToVec(group_perms));
  cbddRole->Filter->SetSecurity("");
  cbddRole->Filter->AddColumnSpec("Role", "", as_text, true, 200);
  cbddRole->Filter->AddColumnSpec("Current Id", "current_id", as_int, false, 40);
  cbddRole->Filter->AddColumnSpec("Internal Id", "internal_id", as_int, false, 60);
  cbddRole->Filter->SetSQLStatement(role_sql);
  cbddRole->Filter->FetchPreferences("appointment_report.role");
  cbddRole->Filter->SetIconStatus(true);
  cbddRole->Filter->ForceRefreshDataSet();
  cbddRole->Filter->CheckAll();
  cbddRole->DisplayFieldName = "Role";
  cbddRole->DropDownHeight = 144;


  lookupTables->RegisterComboBox("groups", cboGroup, groups_sql);
  if (cboGroup->Text == "") // we'll be nice and auto-populate
    if (lookupTables->IsValidId("groups", current_groupid, cboGroup))
      cboGroup->Text = lookupTables->ValueOf("groups", current_groupid).c_str();

  refresh_pending = true;
}
//---------------------------------------------------------------------------
void __fastcall TformLaunchAppointmentReport::FormShow(TObject *Sender)
{
  if (refresh_pending)
    RefreshDependencies();
}
//---------------------------------------------------------------------------
void TformLaunchAppointmentReport::RefreshDependencies()
{
  refresh_pending = false;
  WhichToPrintChange(NULL);
  // WhatToPrintChange(NULL); // now only necessary to call one or t'other
}
//---------------------------------------------------------------------------
void TformLaunchAppointmentReport::AtlasRefresh(int request_num, string matched_value)
{
 if (request_num == user_request)
  user_filter.RefreshDataSet();
}
//---------------------------------------------------------------------------
bool TformLaunchAppointmentReport::IsValidOnScreen()
{
  if(radioClinic->Checked)
  {
    /* // they want to be able to run it for ANY appointment they can see.  fine.  whatever.
    if((cboLocation->Text == "" || !lookupTables->IsInLookUpTable("groups", cboLocation)) &&
       (radioTaxEntity->Checked && IsBlankOrZero(tax_entity_id)))
    {
      PSay("For a daily appointment report you must either select a clinic location, a tax entity, or both.", m_missing_data, MB_OK);
      cboLocation->SetFocus();
      return false;
    } */

    if (!(checkDetailedListing->Enabled && checkDetailedListing->Checked) &&
        !(checkSuperbill->Enabled && checkSuperbill->Checked) &&
        !(checkProviderNotes->Enabled && checkProviderNotes->Checked) &&
        !(checkLabels->Enabled && checkLabels->Checked) &&
        !(checkPrintIntakesSignature->Enabled && checkPrintIntakesSignature->Checked) &&
        !(checkReminderLetters->Enabled && checkReminderLetters->Checked))
    {
      PSay("You must check off at least one clinic report you wish to run.", m_message, MB_OK);
      return false;
    }

    if (!lookupTables->IsInLookUpTable("groups", cboGroup))
    {
      PSay("Please select a valid department.", m_message, MB_OK);
      cboGroup->SetFocus();
      return false;
    }
    else if (checkLabels->Checked)
    {
      if (IsBlankOrZero(editLabelsPerPerson->Text.c_str()) || !IsInt(editLabelsPerPerson->Text.c_str()))
      {
        PSay("Please enter a valid number of labels to print per patient", m_missing_data, MB_OK);
        editLabelsPerPerson->SetFocus();
        return false;
      }
      if (IsBlankOrZero(editStartingLabel->Text.c_str()) || !IsInt(editLabelsPerPerson->Text.c_str()) ||
          StringToInt(editStartingLabel->Text.c_str()) < 0 || StringToInt(editStartingLabel->Text.c_str()) > 30)
      {
        PSay("Please enter a valid starting label", m_missing_data, MB_OK);
        editStartingLabel->SetFocus();
        return false;
      }
    }
    return true;
  }
  else
  {
/*
    if((radioEmployee->Checked) && (cboEmployee->Text == ""))
    {
      PSay("Please select an employee", m_missing_data, MB_OK);
      cboEmployee->SetFocus();
      return false;
    }
    else if((radioDepartment->Checked) && (cboDepartment->Text == ""))
    {
      PSay("Please select a department", m_missing_data, MB_OK);
      cboDepartment->SetFocus();
      return false;
    }
    else */
    if(radioTaxEntity->Checked && GetValidLinkId(txtTaxEntity) == "")
    {
      PSay("Please select a tax entity", m_missing_data, MB_OK);
      btnTaxEntitySearch->SetFocus();
      return false;
    }
    else if (radioUserList->Checked && user_filter.GetMultiID().size() == 0)
    {
      PSay("Please select at least one user", m_missing_data, MB_OK);
      listUsers->SetFocus();
      return false;
    }
    return true;
  }
}
//---------------------------------------------------------------------------
void __fastcall TformLaunchAppointmentReport::btnRunClick(TObject *Sender)
{
  if(IsValidOnScreen())
  {
    Processing(true);

    TIBTransaction * dbTransaction = new TIBTransaction(NULL);
    TIBDataSet * dbDataSet = new TIBDataSet(NULL);
    try
    {
      dbTransaction->DefaultDatabase = dbDatabase;
      dbDataSet->Database = dbDatabase;
      dbDataSet->Transaction = dbTransaction;
      dbDataSet->UniDirectional = false;
      dbTransaction->StartTransaction();
      LogIt("appointment_report.log", "begin 'delete from appointment_reports'");
      ExecuteStatement(dbTransaction, "delete from appointment_reports");
      LogIt("appointment_report.log", "done 'delete from appointment_reports'");

      Pdate begin_date;
      begin_date.InDate(dateBegin->Date);
      Pdate end_date;
      end_date.InDate(dateEnd->Date);
      string date_title = begin_date.OutDate(mdy | yyyy) + " - " + end_date.OutDate(mdy | yyyy);
      vector<string> user_ids;
      map<string, vector<string> > ids_to_match;
      map<string, string> insert_map;
      string local_tax_entity_id = "";
      string link_tax_entity_id = GetValidLinkId(txtTaxEntity);
      string title;
      string sql;

      if(radioUserList->Checked)
        user_ids = user_filter.GetMultiID();
      else if(radioGroup->Checked)
      {
        /*
        if(is_in(cboLocation->Text, "Little Axe Clinic", "Shawnee Clinic", "Dental Clinic", NULL))
          SqlToVec(dbTransaction, "select id from users where group_id = " + lookupTables->IdOf("groups", cboLocation->Text.c_str()), user_ids);
        else
          SqlToVec(dbTransaction, "select id from users", user_ids);
        */
        user_ids.clear();
        ids_to_match["group_id"].push_back(lookupTables->IdOf("groups", cboGroup));
      }
      else if(radioTaxEntity->Checked && !IsBlankOrZero(link_tax_entity_id))
        local_tax_entity_id = link_tax_entity_id;

      if (!radioOutcome->Checked)
      {
        ids_to_match["visit_outcome_id"].push_back("0");
        ids_to_match["visit_outcome_id"].push_back(lookupTables->IdOf("visit_outcome_types", "As Scheduled"));
        ids_to_match["visit_outcome_id"].push_back(lookupTables->IdOf("visit_outcome_types", "Walk-In"));
        ids_to_match["visit_outcome_id"].push_back(lookupTables->IdOf("visit_outcome_types", "Arrived Late"));
      }
      ids_to_match["current_id"].push_back("3");
      ids_to_match["current_id"].push_back("4");
      ids_to_match["current_id"].push_back("5");

      if (!has_scheduler_data)
      {
        LogIt("appointment_report.log", "begin DownloadAppointments()");
        DownloadAppointments(dbDatabase, begin_date, end_date, current_userid, user_ids, local_tax_entity_id, *all_appointments, ids_to_match,
                             (optionsPermissions["admin.all_appointments"]["enabled"] == "true"), false, "");
        LogIt("appointment_report.log", "done DownloadAppointments()");
      }
      LogIt("appointment_report.log", "begin multi-inserting into 'appointment_reports'");
      for(PApptOccurrenceMap::iterator itr = all_appointments->begin(); itr != all_appointments->end(); itr++)
      {
        if (itr->first >= begin_date && itr->first <= end_date)
          for(PApptOccurrenceSet::iterator set_itr = itr->second.begin(); set_itr != itr->second.end(); set_itr++)
          {
            insert_map["id"] = (*set_itr).id;
            insert_map["appt_date"] = itr->first.OutDate(mdy | yyyy);
            insert_map["appt_time"] = (*set_itr).range_begin.OutTime(military | withsec);
            ExecuteInsertStatement(dbTransaction, "appointment_reports", insert_map);
          }
      }
      LogIt("appointment_report.log", "done multi-inserting into 'appointment_reports'");

      if(radioListing->Checked)
      {
        sql = "select appointment_reports.appt_date, appointment_reports.appt_time, "
              "created_for_user.item created_for, created_by_user.item created_by, "
              "appointments.duration || ' ' || duration_types.item duration, "
              "person_names.last_name || ', ' || person_names.first_name || ' ' || person_names.suffix name, "
              "appointments.description, visit_outcome_types.item outcome "
              "from appointment_reports "
              "inner join appointments on appointment_reports.id = appointments.id "
              "left join users created_for_user on appointments.created_for = created_for_user.id "
              "left join users created_by_user on appointments.created_by = created_by_user.id "
              "left join duration_types on appointments.duration_id = duration_types.id "
              "left join visit_outcome_types on appointments.visit_outcome_id = visit_outcome_types.id "
              "left join people on appointments.person_id = people.id "
              "left join person_names on people.person_names_id = person_names.id ";
        if(radioPeople->Checked)
          sql += "where appointments.person_id > 0 ";
        if(checkExcludeBlocks->Checked)
        {
          if(radioPeople->Checked) // jhaley: ugh...
            sql += "and ";
          else
            sql += "where ";
          sql += "appointments.appt_reason_id <> " + lookupTables->IdOf("appt_reason_types", "Block Schedule") + " ";
        }
        if(radioDateSort->Checked)
          sql += "order by appointment_reports.appt_date, appointment_reports.appt_time";
        else if(radioPersonSort->Checked)
          sql += "order by person_names.last_name, person_names.first_name, appointment_reports.appt_date";
        title = "Appointment Report for ";
/*
        if(radioEmployee->Checked)
          title += cboEmployee->Text.c_str();
        else if(radioDepartment->Checked)
          title += cboDepartment->Text.c_str();
        else */
        if(radioTaxEntity->Checked)
          title += txtTaxEntity->Caption.c_str();

        DebugSql(this, &sql, other_forms, 9);
        dbDataSet->SelectSQL->Clear();
        dbDataSet->SelectSQL->Add(sql.c_str());
        LogIt("appointment_report.log", "begin select from 'appointment_reports' (appointment listing)");
        dbDataSet->Open();
        LogIt("appointment_report.log", "done select from 'appointment_reports' (appointment listing)");
        dbDataSet->First();

        TformReportAppointmentListing * report_form;
        FORM_TRY
          report_form = new TformReportAppointmentListing(NULL);
        FORM_CATCH

        report_form->report->DataSet = dbDataSet;
        report_form->lblTitle->Caption = title.c_str();
        report_form->lblUser->Caption = PrettyString(lookupTables->ValueOf("users", current_userid)).c_str();
        report_form->lblDateTitle->Caption = date_title.c_str();
        report_form->txtDate->DataSet = dbDataSet;
        report_form->txtDate->DataField = "appt_date";
        report_form->txtTime->DataSet = dbDataSet;
        report_form->txtTime->DataField = "appt_time";
        report_form->txtFor->DataSet = dbDataSet;
        report_form->txtFor->DataField = "created_for";
        report_form->txtBy->DataSet = dbDataSet;
        report_form->txtBy->DataField = "created_by";
        report_form->txtDuration->DataSet = dbDataSet;
        report_form->txtDuration->DataField = "duration";
        report_form->txtPerson->DataSet = dbDataSet;
        report_form->txtPerson->DataField = "name";
        report_form->txtDescription->DataSet = dbDataSet;
        report_form->txtDescription->DataField = "description";
        report_form->txtOutcome->DataSet = dbDataSet;
        report_form->txtOutcome->DataField = "outcome";
        Processing(false);
        report_form->report->Preview();
        Processing(true);
      }
      else if(radioSimplified->Checked)
      {
        sql = "select appointment_reports.appt_date, appointment_reports.appt_time, "
              "created_for_user.item created_for, "
              "appointments.duration, appointments.duration_id, "
              "person_names.last_name || ', ' || person_names.first_name || ' ' || person_names.suffix name, "
              "appointments.description, visit_outcome_types.item outcome "
              "from appointment_reports "
              "inner join appointments on appointment_reports.id = appointments.id "
              "left join users created_for_user on appointments.created_for = created_for_user.id "
              "left join duration_types on appointments.duration_id = duration_types.id "
              "left join visit_outcome_types on appointments.visit_outcome_id = visit_outcome_types.id "
              "left join people on appointments.person_id = people.id "
              "left join person_names on people.person_names_id = person_names.id ";
        if(radioPeople->Checked)
          sql += "where appointments.person_id > 0 ";
        if(checkExcludeBlocks->Checked)
        {
          if(radioPeople->Checked) // jhaley: ugh...
            sql += "and ";
          else
            sql += "where ";
          sql += "appointments.appt_reason_id <> " + lookupTables->IdOf("appt_reason_types", "Block Schedule") + " ";
        }
        sql += "order by appointment_reports.appt_date, appointment_reports.appt_time";

        DebugSql(this, &sql, other_forms, 9);
        dbDataSet->SelectSQL->Clear();
        dbDataSet->SelectSQL->Add(sql.c_str());
        LogIt("appointment_report.log", "begin select from 'appointment_reports' (simplified appointment per-day schedule)");
        dbDataSet->Open();
        LogIt("appointment_report.log", "done select from 'appointment_reports' (simplified appointment per-day schedule)");
        dbDataSet->First();

        LogIt("appointment_report.log", "begin launching simplified appointment listing");
        TformReportAppointmentListing * report_form;
        FORM_TRY
          report_form = new TformReportAppointmentListing(NULL);
        FORM_CATCH
        report_form->reportSimple->DataSet = dbDataSet;
        report_form->labelSimpleUser->Caption = PrettyString(lookupTables->ValueOf("users", current_userid)).c_str();
        report_form->labelSimpleRange->Caption = date_title.c_str();
        report_form->bandSimpleDate->Expression = "appt_date";
        report_form->textSimpleDate->DataSet = dbDataSet;
        report_form->textSimpleDate->DataField = "appt_date";
        report_form->textSimpleEmployee->DataSet = dbDataSet;
        report_form->textSimpleEmployee->DataField = "created_for";
        report_form->textSimplePerson->DataSet = dbDataSet;
        report_form->textSimplePerson->DataField = "name";
        report_form->textSimpleDescription->DataSet = dbDataSet;
        report_form->textSimpleDescription->DataField = "description";
        Processing(false);
        report_form->reportSimple->Preview();
        Processing(true);
        LogIt("appointment_report.log", "done launching simplified appointment listing");
      }
      else if(radioOutcome->Checked)
      {
        // TODO -c BUG : Why do these numbers not match the Patient Load by Appointment Outcome report?
        // I can't figure out what they're doing differently. They use the same ranges of current IDs, dates,
        // departments, etc., and come up with different numbers.

        sql = "select visit_outcome_types.item outcome, count(appointment_reports.id) the_count, "
              "count(case when AFF.id is not null and AFF.tribe_id = 1 then 1 else null end) ast_total, "
              "count(case when AFF.id is null or AFF.tribe_id <> 1 then 1 else null end) nonast_total "
              "from appointment_reports "
              "inner join appointments on appointment_reports.id = appointments.id "
              "inner join visit_outcome_types on appointments.visit_outcome_id = visit_outcome_types.id "
              "left join people P on appointments.person_id = P.id "
              "left join affiliations AFF on P.affiliations_id = AFF.id ";
        if(checkExcludeBlocks->Checked)
          sql += "where appointments.appt_reason_id <> " + lookupTables->IdOf("appt_reason_types", "Block Schedule") + " ";
        sql += "group by visit_outcome_types.item "
               "order by visit_outcome_types.item";

        title = "Appointment Report for ";
/*
        if(radioEmployee->Checked)
          title += cboEmployee->Text.c_str();
*/
        if(radioGroup->Checked)
          title += cboGroup->Text.c_str(); // jhaley 20101217: restored to functionality for proper group title
        else if(radioTaxEntity->Checked)
          title += txtTaxEntity->Caption.c_str();
        else
          title += "Selected Employees"; // jhaley 20110720: >_>

        DebugSql(this, &sql, other_forms, 9);
        dbDataSet->SelectSQL->Clear();
        dbDataSet->SelectSQL->Add(sql.c_str());
        LogIt("appointment_report.log", "begin select from 'appointment_reports' (appointments by outcome)");
        dbDataSet->Open();
        LogIt("appointment_report.log", "done select from 'appointment_reports' (appointments by outcome)");
        dbDataSet->First();

        TformReportAppointmentOutcomeSummary * report_form;
        FORM_TRY
          report_form = new TformReportAppointmentOutcomeSummary(NULL);
        FORM_CATCH
        report_form->report->DataSet = dbDataSet;
        report_form->lblTitle->Caption = title.c_str();
        report_form->lblUser->Caption = PrettyString(lookupTables->ValueOf("users", current_userid)).c_str();
        report_form->lblDateTitle->Caption = date_title.c_str();
        report_form->txtOutcome->DataSet = dbDataSet;
        report_form->txtOutcome->DataField = "outcome";
        report_form->txtTotal->DataSet = dbDataSet;
        report_form->txtTotal->DataField = "the_count";
        // jhaley 20101217: Guess AST/Non-AST was forgotten for this report; made functional.
        report_form->txtAST->DataSet = dbDataSet;
        report_form->txtAST->DataField = "ast_total";
        report_form->txtNonAST->DataSet = dbDataSet;
        report_form->txtNonAST->DataField = "nonast_total";
        report_form->exprTotal->Expression = "SUM(the_count)";
        Processing(false);
        report_form->report->Preview();
        Processing(true);
      }
      else if(radioClinic->Checked)
      {
        string chart_location = GetOneField(dbTransaction,
          "select chart_location_id from groups where id = '" + lookupTables->IdOf("groups", cboGroup) + "'");

        set<string> valid_consent_types;
        string consent_types_sql = "select id from consent_types where UPPER(item) like '%RELEASE%'";
        SqlToSet(dbDatabase, consent_types_sql, valid_consent_types);

        Pdate update_cutoff(DateToStr(Date()).c_str(), mdy | yyyy);
        update_cutoff.mod(-6, in_months);
        sql = "select appointments.person_id, appointment_reports.appt_date, appointment_reports.appt_time, appointments.id appt_id, "
              "appointments.duration || ' ' || duration_types.short_item duration, people.birth_date, people.id person_id, "
              "last_name || ', ' || (case when first_name != '' then ' ' else '' end) || first_name || (case when middle_name != '' then ' ' else '' end) || middle_name || (case when suffix != '' then ' ' else '' end) || suffix person_name, "
              //"person_names.last_name || ', ' || person_names.first_name || ' ' || person_names.suffix person_name, "
              "case when mcaid_cov.id is not null then 'Yes' else 'No' end has_medicaid, "
              "case when mcare_cov.id is not null then 'Yes' else 'No' end has_medicare, "
              "case when priv_cov.id is not null then 'Yes' else 'No' end has_private, "
              "case when mcaid_cov.id is null and mcare_cov.id is null and priv_cov.id is null "
                "then 'Uninsured' "
                "else case when mcaid_cov.id is not null then cast('Mcd ' as varchar(4)) else cast('' as varchar(4)) end || "
                     "case when mcare_cov.id is not null then cast('Mcr ' as varchar(4)) else cast('' as varchar(4)) end || "
                     "case when priv_cov.id is not null then cast('Priv ' as varchar(5)) else cast('' as varchar(5)) end "
                "end ins_cov_shortstr, "
              "people.social_security_number, people.birth_date, gender_types.abbreviation gender, tribe_types.code tribal_code, "
              "tax_entities.name provider_name, appointments.description, affiliations.cdib, visit_outcome_types.item visit_outcome, ";
        if (!IsBlankOrZero(chart_location))
        { /* DONE -c STUPID: It was potentially picking *other* chart numbers they had permission to see, if the patient didn't have
                             a chart established with the department that saw them.  We only care about the chart where seen though. */
          //set<string> can_see_charts = GetSpecialGroupPermissions("charts");
          sql += "(select first 1 charts.chart_number from charts "
                  "inner join groups on charts.location_id = groups.chart_location_id "
                  "where charts.person_id = people.id and groups.id = appointments.group_id "
                  "and charts.current_id > 2 and charts.location_id = " + chart_location + " "
                  "order by "
                  "case when appt_location_types.chart_location_id = appt_group.chart_location_id then 1 else 0 end desc"
                  ") chart_number, ";
        }
        else
          sql += " NULL chart_number, ";
        sql += "mail.line1 || case when mail.line2 is not null and mail.line2 != '' then ' / ' || mail.line2 else '' end address1, "
               "mail.city || ', ' || mail_st.abbreviation || '  ' || mail.zip address2, "
               "hp.phone_number home_phone, "
               "wp.phone_number work_phone, "
               "(select first 1 consents.id from consents where consents.person_id = people.id and "
               "consents.current_id >= 3 and consents.consent_id in (" + SetToCommaString(valid_consent_types) + ") and "
               " consents.signature_date >= '" + update_cutoff.OutDate(mdy | yyyy) + "' order by consents.signature_date desc) consent_id "
               "from appointment_reports "
               "inner join appointments on appointment_reports.id = appointments.id "
               "inner join users on appointments.created_for = users.id ";

        string medicaid_types = "'" + lookupTables->IdOf("insurance_types", "Medicaid") + "','"
                                    + lookupTables->IdOf("insurance_types", "HMO Medicaid") + "'";
        string medicare_types = "'" + lookupTables->IdOf("insurance_types", "Medicare A") + "','" // jhaley TODO: this is no good! should be a secondary lookup!
                                    + lookupTables->IdOf("insurance_types", "Medicare B") + "','"
                                    + lookupTables->IdOf("insurance_types", "Medicare D") + "','"
                                    + lookupTables->IdOf("insurance_types", "Medicare Advantage") + "'";

        /* This is the type of insurance benefit they care about; Dental cares about private dental insurance only,
           Medical about private medical only (presumably; they haven't asked for this feature), etc.  This is where you set it. */
        string pri_ins_type = (cboGroup->Text=="Dental Clinic")?"Dental":"";
        sql += "left join tax_entities on appointments.tax_entity_id = tax_entities.id "
               "left join duration_types on appointments.duration_id = duration_types.id "
               "left join appt_reason_types on appointments.appt_reason_id = appt_reason_types.id "
               "left join appt_location_types on appt_location_types.id = appointments.appt_location_id "
               "left join groups appt_group on appt_group.id = appointments.group_id "
               "left join people on appointments.person_id = people.id "
               "left join person_names on people.person_names_id = person_names.id "
               "left join affiliations on people.affiliations_id = affiliations.id "
               "left join tribe_types on affiliations.tribe_id = tribe_types.id "
               "left join gender_types on people.gender_id = gender_types.id "
               "left join insurance_coverage mcaid_cov on mcaid_cov.id = (select first 1 ic.id from insurance_coverage ic inner join insurance_policies ip on ip.id = ic.insurance_policy_id "
                  "where ic.person_id = people.id and ip.insurance_id in (" + medicaid_types + ") "
                  "and (ic.stop_date is null or ic.stop_date > current_date) and ip.current_id+1 >= 4 and ic.current_id+1 >= 4) "
               "left join insurance_policies mcaid_pol on mcaid_cov.insurance_policy_id = mcaid_pol.id "
               "left join insurance_coverage mcare_cov on mcare_cov.id = (select first 1 ic.id from insurance_coverage ic inner join insurance_policies ip on ip.id = ic.insurance_policy_id "
                  "where ic.person_id = people.id and ip.insurance_id in (" + medicare_types + ") "
                  "and (ic.stop_date is null or ic.stop_date > current_date) and ip.current_id+1 >= 4 and ic.current_id+1 >= 4) "
               "left join insurance_coverage priv_cov on priv_cov.id = (select first 1 ic.id from insurance_coverage ic inner join insurance_policies ip on ip.id = ic.insurance_policy_id ";
        if (pri_ins_type != "")
          sql +=  "inner join insurance_benefits ib on ib.insurance_policy_id = ip.id and ib.area_id = '" + lookupTables->IdOf("area_types", pri_ins_type) + "' ";
        sql +=    "where ic.person_id = people.id and ip.insurance_id not in (" + medicaid_types + "," + medicare_types + ") "
                  "and (ic.stop_date is null or ic.stop_date > current_date) and ip.current_id+1 >= 4 and ic.current_id+1 >= 4) "
               "left join visit_outcome_types on appointments.visit_outcome_id = visit_outcome_types.id "
               "left join person_contact_info mail on people.mailing_address_id = mail.id "
               "left join state_types mail_st on mail.state_id = mail_st.id "
               "left join person_contact_info hp on people.home_phone_id = hp.id "
               "left join person_contact_info wp on people.work_phone_id = wp.id ";

        vector<string> where_clause;
        if(radioPeople->Checked)
          where_clause.push_back("appointments.person_id > 0");
        if(checkExcludeBlocks->Checked)
          where_clause.push_back("appointments.appt_reason_id <> " + lookupTables->IdOf("appt_reason_types", "Block Schedule"));
        if(panelClinicDailyReport->Enabled && cbddRole->Text != "" &&
           //lookupTables->IsInLookUpTable("appt_reason_types", cboReasonLac->Text.c_str()) &&
           is_in(cboGroup->Text, "", "Little Axe Clinic", "Shawnee Clinic", NULL))
             where_clause.push_back("appointments.appt_reason_id in (" + VecToCommaString(cbddRole->Filter->GetMultiID())
              /*lookupTables->IdOf("appt_reason_types", cboReasonLac->Text.c_str())*/ + ")");
        string link_tax_entity_id2 = GetValidLinkId(txtTaxEntity);
        if(radioTaxEntity->Enabled && radioTaxEntity->Checked && !IsBlankOrZero(link_tax_entity_id2))
          where_clause.push_back("tax_entities.id = '" + link_tax_entity_id2 + "'");
        if (radioGroup->Enabled && radioGroup->Checked && cboGroup->Text != "")
        {
          sql += "and appointments.group_id = '" + lookupTables->IdOf("groups", cboGroup) + "' ";
          string facility_name = GetOneField(dbTransaction,
            "select name from tax_entities inner join groups on groups.facility_id = tax_entities.id "
            " and groups.id = '" + lookupTables->IdOf("groups", cboGroup) + "'");
          if (facility_name != "")
            title = "Appointment Report for " + facility_name;
          else
            title = "Appointment Report";
        }
        else title = "Appointment Report";

        if(!where_clause.empty())
          sql += (" where " + VecToDelimString(where_clause, "", "", " and ") + " ");

        if(radioDateSort->Checked)
          sql += "order by appointment_reports.appt_date, appointment_reports.appt_time";
        else if(radioPersonSort->Checked)
          sql += "order by person_names.last_name, person_names.first_name, appointment_reports.appt_date";

        string link_tax_entity_id3 = GetValidLinkId(txtTaxEntity);
        if(radioTaxEntity->Checked && !IsBlankOrZero(link_tax_entity_id3))
          title += (" for " + string(txtTaxEntity->Caption.c_str()));

        if(begin_date == end_date)
          date_title = begin_date.NameOfDay() + ", " + begin_date.NameOfMonth() + " " + IntToString(begin_date.day) +
                       ", " + IntToString(begin_date.year);

        if(debug_level[other_forms]>=9) QueryData(this, dbTransaction, sql);
        DebugSql(this, &sql, other_forms, 9);
        dbDataSet->SelectSQL->Clear();
        dbDataSet->SelectSQL->Add(sql.c_str());
        LogIt("appointment_report.log", "begin select from 'appointment_reports' (clinic appointment list)");
        dbDataSet->Open();
        LogIt("appointment_report.log", "end select from 'appointment_reports' (clinic appointment list)");

        if (checkDetailedListing->Checked)
        {
          dbDataSet->First();

          Application->ProcessMessages();
          TformReportClinicAppointments * report_form;
          FORM_TRY
            report_form = new TformReportClinicAppointments(NULL, dbDataSet, radioSummaryOutcomes->Checked?"visit_outcome":"", pri_ins_type);
          FORM_CATCH
          report_form->report->DataSet = dbDataSet;
          report_form->lblTitle->Caption = title.c_str();
          report_form->lblUser->Caption = PrettyString(lookupTables->ValueOf("users", current_userid)).c_str();
          report_form->lblDateTitle->Caption = date_title.c_str();
          report_form->txtDate->DataSet = dbDataSet;
          report_form->txtDate->DataField = "appt_date";
          report_form->txtTime->DataSet = dbDataSet;
          report_form->txtTime->DataField = "appt_time";
          report_form->txtDuration->DataSet = dbDataSet;
          report_form->txtDuration->DataField = "duration";
          report_form->txtPatient->DataSet = dbDataSet;
          report_form->txtPatient->DataField = "person_name";
          report_form->txtDOB->DataSet = dbDataSet;
          report_form->txtDOB->DataField = "birth_date";
          report_form->txtProvider->DataSet = dbDataSet;
          report_form->txtProvider->DataField = "provider_name";
          report_form->txtProvider->DataSet = dbDataSet;
          report_form->txtProvider->DataField = "provider_name";
          report_form->txtReason->DataSet = dbDataSet;
          report_form->txtReason->DataField = "description";
          report_form->txtTribalCode->DataSet = dbDataSet;
          report_form->txtTribalCode->DataField = "tribal_code";
          report_form->txtChart->DataSet = dbDataSet;
          report_form->txtChart->DataField = "chart_number";
          report_form->txtHomePhone->DataSet = dbDataSet;
          report_form->txtHomePhone->DataField = "home_phone";
          report_form->txtWorkPhone->DataSet = dbDataSet;
          report_form->txtWorkPhone->DataField = "work_phone";
          report_form->txtMedicaid->DataSet = dbDataSet;
          report_form->txtMedicaid->DataField = "has_medicaid";
          report_form->exprConsent->Expression = "IF(consent_id > 0, 'No', 'Yes')";
          Processing(false);
          report_form->report->Preview();
          Processing(true);
        } // end detailed listing

        if(checkSuperbill->Checked || checkProviderNotes->Checked)
        {
          dbDataSet->First();
          corner_types set1 = ct_none, set2 = ct_none;
          if (checkSuperbill->Checked)
          {
            if(radioUL1->Checked) set1 = ct_upper_left;
            if(radioUR1->Checked) set1 = ct_upper_right;
            if(radioLL1->Checked) set1 = ct_lower_left;
            if(radioLR1->Checked) set1 = ct_lower_right;
          }
          if (checkProviderNotes->Checked)
          {
            if(radioUL2->Checked) set2 = ct_upper_left;
            if(radioUR2->Checked) set2 = ct_upper_right;
            if(radioLL2->Checked) set2 = ct_lower_left;
            if(radioLR2->Checked) set2 = ct_lower_right;
          }
          // don't print chart # and tribal code for SC // why not?
          TformReportClinicSuperBill::Execute(this, dbDataSet, set1, set2, /* cboLocation->Text != "Shawnee Clinic" */ true);
        } // end demographic sheets

        if(checkLabels->Checked)
        {
          dbDataSet->First();
          TformReportClinicPatientLabels::Execute(this, dbDataSet, StringToInt(editLabelsPerPerson->Text.c_str()), StringToInt(editStartingLabel->Text.c_str())-1,
            /*cboLocation->Text != "Shawnee Clinic"*/ true, checkShowDateTime->Checked, radioSheetLabels->Checked ? print_label_sheets : print_label_spooled);
        }
        // end labels  :)

        if(checkPrintIntakesSignature->Enabled && checkPrintIntakesSignature->Checked)
        {
          dbDataSet->First();
          vector<string> person_ids;
          set<string> person_ids_set;
          while (!dbDataSet->Eof)
          {
            string p = dbDataSet->FieldByName("person_id")->AsString.c_str();
            bool needs_update = dbDataSet->FieldByName("consent_id")->IsNull;

            if (needs_update && person_ids_set.find(p) == person_ids_set.end())
            {
              person_ids.push_back(p);
              person_ids_set.insert(p);
            }
            dbDataSet->Next();
          }
          // dbDataSet->Close();
          TformLaunchIntakeFormReport::DoSilent(this, person_ids, true, false);
          Processing(true);
        } // end update forms

        if (checkReminderLetters->Enabled && checkReminderLetters->Checked)
        {
          string group_id = lookupTables->IdOf("groups", cboGroup);
          bool valid_group = false;
          if (group_id == lookupTables->IdOf("groups", "Little Axe Clinic") ||
              group_id == lookupTables->IdOf("groups", "Shawnee Clinic") ||
              group_id == lookupTables->IdOf("groups", "Dental Clinic"))
            valid_group = true;
          if (!valid_group)
            PSay("The appointment reminder letters can only be run for the medical or dental clinics from here.", m_missing_data, MB_OK);
          else
          {
            vector<string> appointments;
            dbDataSet->First();

            while (!dbDataSet->Eof)
            {
             appointments.push_back(dbDataSet->FieldByName("appt_id")->AsString.c_str());
             dbDataSet->Next();
            }

            TformReportDiabAppointmentLetter::Do(appointments, group_id, radioPersonSort->Checked, true, StringToInt(editStartingMailingLabel->Text.c_str()));
          }
        }
      }
      LogIt("appointment_report.log", "begin 'delete from appointment_reports'");
      ExecuteStatement(dbTransaction, "delete from appointment_reports");
      LogIt("appointment_report.log", "end 'delete from appointment_reports'");
      dbDataSet->Close();
      dbTransaction->Commit();
      Processing(false);
    }
    catch(...)
    {
      Processing(false);
      PSay("Error running report", m_database_error, MB_OK);
      if (dbTransaction->Active)
        dbTransaction->Rollback();
    }
    delete dbTransaction;
    delete dbDataSet;

    Processing(false); // just in case.
  }// end if isvalidonscreen
}
//---------------------------------------------------------------------------
void __fastcall TformLaunchAppointmentReport::btnTaxEntitySearchClick(TObject *Sender)
{
  // create and show new person search form
  vector<string> return_vector;
  FORM_TRY
    TformSearchTaxEntity * taxEntitySearch = new TformSearchTaxEntity(NULL,&return_vector);
    taxEntitySearch->ShowTaskModal(this);
  FORM_CATCH
  if (! return_vector.empty())
    CreateLink(txtTaxEntity, btnTaxEntitySearch, this, return_vector[0], "tax_entities");
}
//---------------------------------------------------------------------------
void __fastcall TformLaunchAppointmentReport::FormClose(TObject *Sender, TCloseAction &Action)
{
  lookupTables->UnregisterComboBox("groups", cboGroup);
  TformSpecialGeneric::FormClose(Sender, Action);
}
//---------------------------------------------------------------------------
void __fastcall TformLaunchAppointmentReport::btnCloseClick(TObject *Sender)
{
  Close();
}
//---------------------------------------------------------------------------
void __fastcall TformLaunchAppointmentReport::WhatToPrintChange(TObject *Sender)
{
 if (refresh_what == refresh_none) refresh_what = refresh_what_then_which;
/* DONE: we really need a bool or enum or something to indicate which side changed;
   if this side, run that side after; if that side, run this side after; that way
   we make sure Which reflects changes to What and What reflects changes to Which,
   all the time, without introducing an infinite loop
*/
/*
 radioEmployee->Enabled = !(radioClinic->Checked);
 cboEmployee->Enabled = !(radioClinic->Checked);
 radioDepartment->Enabled = !(radioClinic->Checked);
 cboDepartment->Enabled = !(radioClinic->Checked);
*/
 radioTaxEntity->Enabled = (optionsPermissions["table.tax_entities"]["view"] == "true");
 radioUserList->Enabled = !(radioClinic->Checked);

 radioPeople->Enabled = !(radioOutcome->Checked);

 if (!(radioPeople->Enabled))
  radioAll->Checked = true;
 if (!(radioAll->Enabled))
  radioPeople->Checked = true;

 radioDateSort->Enabled = !(radioSimplified->Checked) && !(radioOutcome->Checked);
 radioPersonSort->Enabled = !(radioOutcome->Checked);

 if (!(radioDateSort->Enabled))
  radioPersonSort->Checked = true;
 if (!(radioPersonSort->Enabled))
  radioDateSort->Checked = true;

 cboGroupChange(NULL);

 if (refresh_what == refresh_what_then_which)
   WhichToPrintChange(NULL);
 refresh_what = refresh_none;
}
//---------------------------------------------------------------------------
void __fastcall TformLaunchAppointmentReport::WhichToPrintChange(TObject *Sender)
{
 if (refresh_what == refresh_none) refresh_what = refresh_which_then_what;
// cboEmployee->Enabled        = radioEmployee->Enabled && radioEmployee->Checked;
 cboGroup->Enabled             = radioGroup->Enabled && radioGroup->Checked;
 txtTaxEntity->Enabled         = radioTaxEntity->Enabled && radioTaxEntity->Checked;
 RecurseEnabled(btnTaxEntitySearch, radioTaxEntity->Enabled && radioTaxEntity->Checked);
 listUsers->Enabled            = radioUserList->Enabled && radioUserList->Checked;
 listUsers->Repaint(); // I noticed some odd gray areas from re-enabling the control w/ checkboxes?

 radioListing->Enabled    = !radioSchedule->Checked;
 radioSimplified->Enabled = !radioSchedule->Checked;
 radioOutcome->Enabled    = !radioSchedule->Checked;
 radioClinic->Enabled     = ((radioTaxEntity->Enabled && radioTaxEntity->Checked) ||
                             (radioGroup->Enabled && radioGroup->Checked &&
                              cboGroup->Enabled && lookupTables->IsInLookUpTable("groups", cboGroup)));
 if (radioSchedule->Checked)
   radioClinic->Checked = true;
 if (refresh_what == refresh_which_then_what)
   WhatToPrintChange(NULL);
 refresh_what = refresh_none;
}
//---------------------------------------------------------------------------
void __fastcall TformLaunchAppointmentReport::FormShortCut(TWMKey &Msg,bool &Handled)
{
  Handled = true;
  if (Msg.CharCode == VK_RETURN)
    btnRun->Click();
  else
    Handled = false;

  if (!Handled)
    TformSpecialGeneric::FormShortCut(Msg, Handled);
}
//---------------------------------------------------------------------------
void __fastcall TformLaunchAppointmentReport::cboGroupChange(TObject *Sender)
{
  // Massive logic change - just disable if not in department mode, or if no valid department
  if(radioGroup->Enabled && radioGroup->Checked &&
     cboGroup->Enabled && lookupTables->IsInLookUpTable("groups", cboGroup))
  {
    if (cboGroup->Text == "Dental Clinic")
      cbddRole->Enabled = false;
  }
  RecurseEnabled(panelClinicDailyReport, (radioTaxEntity->Enabled && radioTaxEntity->Checked) ||
                                         (radioGroup->Enabled && radioGroup->Checked &&
                                         cboGroup->Enabled && lookupTables->IsInLookUpTable("groups", cboGroup)));
  panelClinicDailyReport->Enabled = (radioClinic->Enabled && radioClinic->Checked);
}
//---------------------------------------------------------------------------
void __fastcall TformLaunchAppointmentReport::staticAddUsersFromGroupsClick(TObject *Sender)
{
  /* Deprecated
  set<string> return_set;
  string id;
  FORM_TRY
    TformSpecialSelectGroups * select_group_form = new TformSpecialSelectGroups(NULL, &return_set, NULL);
    select_group_form->ShowTaskModal(this);
  FORM_CATCH
  if(!return_set.empty()) // if it's empty ... we don't do anything. which makes it annoying to unselect everyone. but at least our sql will be valid.
  {
   user_filter.SetCheckedWhereID(return_set, true);
   user_filter.SetCheckedWhereNotID(return_set, false);
  }
  */
}
//---------------------------------------------------------------------------


