//---------------------------------------------------------------------------

#include <vcl.h>
#pragma hdrstop

#include "launchApptOutcomeDetailReport.h"
#include "reportApptOutcomeDetails.h"

//---------------------------------------------------------------------------
#pragma package(smart_init)
#pragma link "launchGeneric"
#pragma resource "*.dfm"
TformLaunchApptOutcomeDetailReport *formLaunchApptOutcomeDetailReport;
//---------------------------------------------------------------------------
__fastcall TformLaunchApptOutcomeDetailReport::TformLaunchApptOutcomeDetailReport(TComponent* Owner)
        : TformLaunchGeneric(Owner)
{
  pair<Pdate, Pdate> last_month_dates = GetPriorMonthDateRange(Pdate(Date()));
  dateBegin->Date = last_month_dates.first.OutDate();  // first day of last month
  dateEnd->Date   = last_month_dates.second.OutDate(); // last day of last month

  // Add all groups to cboDepartment for which the user has view permissions
  set<string> group_perms = GetGroupPermissions("table.appointments", "", "view", "true");
  string group_perm_sql = "select id, item, valid from groups where groups.id > 0 and " + SafeInClause("groups.id", SetToVec(group_perms));
  lookupTables->RegisterComboBox("groups", cboDepartment, group_perm_sql);
}
//---------------------------------------------------------------------------
__fastcall TformLaunchApptOutcomeDetailReport::~TformLaunchApptOutcomeDetailReport()
{
  if (lookupTables->TableName(cboDepartment) == "groups")
    lookupTables->UnregisterComboBox("groups", cboDepartment);
}
//---------------------------------------------------------------------------
bool TformLaunchApptOutcomeDetailReport::IsValidOnScreen()
{
  if (cboDepartment->Text == "" || !lookupTables->IsInLookUpTable("groups", cboDepartment))
  {
    PSay("Please select a valid department", m_invalid_data, MB_OK);
    cboDepartment->SetFocus();
    return false;
  }
  else if (Pdate(dateBegin->Date) > Pdate(dateEnd->Date))
  {
    PSay("Please select a begin date prior or equal to the end date", m_invalid_data, MB_OK);
    dateBegin->SetFocus();
    return false;
  }
  return true;
}
//---------------------------------------------------------------------------
bool TformLaunchApptOutcomeDetailReport::BeforeRunReport()
{
  dbDataset = AddDataSet();
  return TformLaunchGeneric::BeforeRunReport();
}
//---------------------------------------------------------------------------
bool TformLaunchApptOutcomeDetailReport::RunReport()
{
  string sql;
  string group_id   = lookupTables->IdOf("groups", cboDepartment);
  string begin_date = Pdate(dateBegin->Date).OutDate(mdy | yyyy);
  string end_date   = Pdate(dateEnd->Date).OutDate(mdy | yyyy);
  string date_title = begin_date + " - " + end_date;
  bool   is_clinic  = (group_id == lookupTables->IdOf("groups", "Little Axe Clinic") ||
                       group_id == lookupTables->IdOf("groups", "Shawnee Clinic"));

    // Create the report form
  TformReportApptOutcomeDetails *report_form = NULL;
  FORM_TRY
    report_form = new TformReportApptOutcomeDetails(NULL);
  FORM_CATCH

  // Query is too complicated to inline here, see below.
  sql = GetSQLQuery(group_id, begin_date, end_date);

  InitDataSet(dbDataset, sql);

  report_form->reportDetails->DataSet = dbDataset;

  // Title band
  report_form->lblTitleDept->Caption = cboDepartment->Text; // show department name
  report_form->lblDateRange->Caption = date_title.c_str();  // display dates on title band

  // Page footer
  report_form->lblUser->Caption = current_fullname.c_str(); // display generating user's name on page footer

  // Header band
  if (!is_clinic)
  {
    report_form->lblColMedical->Enabled  = false;
    report_form->lblColObGyn->Enabled    = false;
    report_form->lblColPodiatry->Enabled = false;
    report_form->lblColEndocrinology->Enabled = false;
  }

  // Detail band
  SetupQRDBText(report_form->txtOutcome,    dbDataset, "outcome");
  SetupQRDBText(report_form->txtInsured,    dbDataset, "num_ins");
  SetupQRDBText(report_form->txtUninsured,  dbDataset, "num_unins");
  SetupQRDBText(report_form->txtAST,        dbDataset, "num_ast");
  SetupQRDBText(report_form->txtNonAST,     dbDataset, "num_nast");
  SetupQRDBText(report_form->txtASTDiab,    dbDataset, "num_ast_diab");
  SetupQRDBText(report_form->txtNonASTDiab, dbDataset, "num_nast_diab");
  if (is_clinic)
  {
    // These categorizations only make sense when applied to medical clinic appointments
    SetupQRDBText(report_form->txtMedical,  dbDataset, "num_med");
    SetupQRDBText(report_form->txtObGyn,    dbDataset, "num_obgyn");
    SetupQRDBText(report_form->txtPodiatry, dbDataset, "num_pod");
    SetupQRDBText(report_form->txtEndo,     dbDataset, "num_endo");
  }
  else
  {
    report_form->txtMedical->Enabled  = false;
    report_form->txtObGyn->Enabled    = false;
    report_form->txtPodiatry->Enabled = false;
    report_form->txtEndo->Enabled     = false;
  }

  // Summary band
  report_form->exprTotalInsured->Expression    = "SUM(num_ins)";
  report_form->exprTotalUninsured->Expression  = "SUM(num_unins)";
  report_form->exprTotalAST->Expression        = "SUM(num_ast)";
  report_form->exprTotalNonAST->Expression     = "SUM(num_nast)";
  report_form->exprTotalASTDiab->Expression    = "SUM(num_ast_diab)";
  report_form->exprTotalNonASTDiab->Expression = "SUM(num_nast_diab)";
  if (is_clinic)
  {
    report_form->exprTotalMedical->Expression  = "SUM(num_med)";
    report_form->exprTotalObGyn->Expression    = "SUM(num_obgyn)";
    report_form->exprTotalPodiatry->Expression = "SUM(num_pod)";
    report_form->exprTotalEndo->Expression     = "SUM(num_endo)";
  }
  else
  {
    report_form->exprTotalMedical->Enabled  = false;
    report_form->exprTotalObGyn->Enabled    = false;
    report_form->exprTotalPodiatry->Enabled = false;
    report_form->exprTotalEndo->Enabled     = false;
  }

  sql = "select count(consents.id) "
        "from consents "
        "inner join consent_types on consents.consent_id = consent_types.id "
        "where consents.group_id = '" + group_id + "' and consents.current_id > 2 and consent_types.item like '%RELEASE%' and "
        "consents.signature_date between '" + begin_date + "' and '" + end_date + "' ";

  string num_updates = GetOneField(report_transaction, sql);
  report_form->lblUpdateCount->Caption = IntToString(StringToInt(num_updates)).c_str();

  report_form->reportDetails->Preview();

  return true;
}
//---------------------------------------------------------------------------
string TformLaunchApptOutcomeDetailReport::GetEndocrinologyProvider()
{
  // TODO -c MAGIC : Where are we going to get this ID from? Dr. Copeland is the designated Endocrinology provider.
  return string("3161");
}
//---------------------------------------------------------------------------
string TformLaunchApptOutcomeDetailReport::GetSQLQuery(string &group_id, string &begin_date, string &end_date)
{
  return string(
    /* Outer Query - sum inner query rows on various criteria within each visit outcome */
    /* NOTE: as-integer casts required to workaround a QuickReport TLargeintField limitation */
    "select outcome, "
    "cast(sum(is_insured) as integer) num_ins, cast(sum((is_insured - 1) * -1) as integer) num_unins, "
    "cast(sum(is_ast) as integer) num_ast, cast(sum((is_ast - 1) * -1) as integer) num_nast, "
    "cast(sum(is_medical) as integer) num_med, cast(sum(is_obgyn) as integer) num_obgyn, "
    "cast(sum(is_podiatry) as integer) num_pod, cast(sum(is_endo) as integer) num_endo, "
    "cast(sum(case when is_ast = 1 and is_diab = 1 then 1 else 0 end) as integer) num_ast_diab, "
    "cast(sum(case when is_ast = 0 and is_diab = 1 then 1 else 0 end) as integer) num_nast_diab "
    "from "
    /* Inner Query */
    "(select case when VOT.item = '' then 'Unspecified' else VOT.item end outcome, "
    /* Affiliations: AST vs Non-AST */
    " case when TA.tribe_id = '" + lookupTables->IdOf("tribe_types", "Absentee Shawnee Tribes of Oklahoma") + "' then 1 else 0 end is_ast, "
    /* Insured Status */
    " case when Ins.id is not null then 1 else 0 end is_insured, "
    /* Diabetic conditions - 250 or 250.%, not "Relative", and task-attached */
    " case when exists (select COND.id from conditions COND where P.id = COND.person_id and (COND.code = '250' or COND.code like '250.%') "
    "  and COND.condition_id <> '") + lookupTables->IdOf("condition_types", "Relative") + "' and COND.task_id <> 0) then 1 else 0 end is_diab, "
    /* Medical Appointments - for this report, not Podiatry, Ob/Gyn, or Endocrinology */
    " case when ART.id not in (" + lookupTables->IdOf("appt_reason_types", "Podiatry") + ", " + lookupTables->IdOf("appt_reason_types", "Ob/Gyn") + ") "
    "  and PROV.id <> '" + GetEndocrinologyProvider() + "' then 1 else 0 end is_medical, "
    /* Ob/Gyn Appointments */
    " case when ART.id  = '" + lookupTables->IdOf("appt_reason_types", "Ob/Gyn")   + "' then 1 else 0 end is_obgyn, "
    /* Podiatry Appointments */
    " case when ART.id  = '" + lookupTables->IdOf("appt_reason_types", "Podiatry") + "' then 1 else 0 end is_podiatry, "
    /* Endocrinology Appointments */
    " case when PROV.id = '" + GetEndocrinologyProvider() + "' then 1 else 0 end is_endo "
    /* Tables */
    " from appointments A "
    " left join visit_outcome_types VOT on A.visit_outcome_id = VOT.id " /* Appointment outcomes, for grouping */
    " left join appt_reason_types ART on A.appt_reason_id = ART.id "     /* Appointment reason, for breakdown sums */
    " left join people P on A.person_id = P.id "                         /* People: needed for insurance, conditions, affiliations */
    " left join affiliations TA on P.affiliations_id = TA.id "           /* Affiliations: for AST vs Non-AST */
    " left join tax_entities PROV on A.tax_entity_id = PROV.id "         /* Appointment provider, because of Dr. Copeland */
    " left join insurance_coverage Ins on Ins.id = "                     /* Insurance: NOTE - group-specific as to what area is considered */
    " (select first 1 IIC.id "
    "  from insurance_coverage IIC "
    "  inner join insurance_policies IIP on IIC.insurance_policy_id = IIP.id "
    "  inner join insurance_benefits IIB on IIB.insurance_policy_id = IIP.id and IIB.is_covered = 1 and "
    "   ((IIB.area_id = '" + lookupTables->IdOf("area_types", "Dental") + "' "
    "     and A.group_id = '" + lookupTables->IdOf("groups", "Dental Clinic") + "') or "
    "    (IIB.area_id = '" + lookupTables->IdOf("area_types", "Medical") + "' "
    "     and A.group_id in (" + lookupTables->IdOf("groups", "Little Axe Clinic") + ", " + lookupTables->IdOf("groups", "Shawnee Clinic") + "))) "
    "  where IIC.person_id = A.person_id "
    "  and (IIC.start_date <= A.begin_date or (IIC.start_date is null and IIC.creation_stamp <= A.begin_date)) "
    "  and (IIC.stop_date  >  A.begin_date or (IIC.stop_date  is null and IIC.current_id+1 > 3)) "
    "  and (IIP.stop_date  >  A.begin_date or (IIP.stop_date  is null and IIP.current_id+1 > 3))) "
    /* Inner Query Conditions */
    " where "
    " A.current_id+1 > 3 and A.group_id = '" + group_id + "' and "       /* Incomplete or better, in the group, and within the date range specified */
    " A.begin_date between '" + begin_date + "' and '" + end_date + "' and "
    " A.appt_reason_id <> " + lookupTables->IdOf("appt_reason_types", "Block Schedule") + " " /* No block schedule appts */
    /* Group outer query sums by visit outcome */
    ") group by outcome";
}
//---------------------------------------------------------------------------

