//---------------------------------------------------------------------------

#include <vcl.h>
#pragma hdrstop

#include "launchBehProgramReport.h"
#include "derivedheaders.h"
//---------------------------------------------------------------------------
#pragma package(smart_init)
#pragma link "codeSpecialGeneric"
#pragma resource "*.dfm"
TformLaunchBehProgramReport *formLaunchBehProgramReport;
//---------------------------------------------------------------------------
__fastcall TformLaunchBehProgramReport::TformLaunchBehProgramReport(TComponent* Owner)
        : TformSpecialGeneric(Owner)
{
  Pdate begin_date = CurrentPdate();
  if(begin_date.month == 1)
    begin_date.mod(-1, in_years);
  begin_date.month = 1;
  begin_date.day = 1;
  dateBegin->Date = begin_date.OutDate();
  string end_date = "12/31/" + IntToString(begin_date.year);
  dateEnd->Date = StrToDate(end_date.c_str());
  lookupTables->RegisterComboBox("age_group_types", cboAgeGroup, "select min(id) id, group_name item, 1 valid from age_groups group by 2");
}
//---------------------------------------------------------------------------
bool TformLaunchBehProgramReport::IsValidOnScreen()
{
  if (cboAgeGroup->Text == "" || !lookupTables->IsInLookUpTable("age_group_types", cboAgeGroup))
  {
    PSay("Please select a valid age group type.", m_invalid_data, MB_OK);
    cboAgeGroup->SetFocus();
    return false;
  }
  // Note parent class doesn't implement. This is called from btnRunClick as on some other launch forms.
  // return TformSpecialGeneric::IsValidOnScreen();
  return true;
}
//---------------------------------------------------------------------------
void __fastcall TformLaunchBehProgramReport::btnCloseClick(TObject *Sender)
{
  Close();
}
//---------------------------------------------------------------------------

void __fastcall TformLaunchBehProgramReport::btnRunClick(TObject *Sender)
{
  if (!IsValidOnScreen())
    return;

  Processing(true);
  dbTransaction = new TIBTransaction(NULL);
  dbGenderDataSet = new TIBDataSet(NULL);
  dbAgeDataSet = new TIBDataSet(NULL);
  dbTribeDataSet = new TIBDataSet(NULL);
  dbRaceDataSet = new TIBDataSet(NULL);

  report_form = NULL;

  try
  {
    dbTransaction->DefaultDatabase = dbDatabase;
    dbGenderDataSet->Database = dbDatabase;
    dbGenderDataSet->Transaction = dbTransaction;
    dbGenderDataSet->UniDirectional = false;
    dbAgeDataSet->Database = dbDatabase;
    dbAgeDataSet->Transaction = dbTransaction;
    dbAgeDataSet->UniDirectional = false;
    dbTribeDataSet->Database = dbDatabase;
    dbTribeDataSet->Transaction = dbTransaction;
    dbTribeDataSet->UniDirectional = false;
    dbRaceDataSet->Database = dbDatabase;
    dbRaceDataSet->Transaction = dbTransaction;
    dbRaceDataSet->UniDirectional = false;
    dbTransaction->StartTransaction();

    FORM_TRY
      report_form = new TformReportBehProgram(NULL);
    FORM_CATCH

    string sql;
    string begin_date = Pdate(dateBegin->Date).OutDate(mdy | yyyy);
    string end_date =  Pdate(dateEnd->Date).OutDate(mdy | yyyy);
    string date_title = begin_date + " - " + end_date;
    string today = DateToStr(Date()).c_str();
    map<string, string> total_map;
    int total_visits = 0;

    try
    {
      report_form->lblDateTitle->Caption = date_title.c_str();

      sql = "select beh_service_types.item service_type, count(tasks.id) total_count "
            "from beh_service_types left join tasks on "
            "tasks.beh_service_id = beh_service_types.id and "
            "tasks.date_of_task >= '" + begin_date + "' and "
            "tasks.date_of_task <= '" + end_date + "' and "
            "tasks.group_id = " + lookupTables->IdOf("groups", "Behavioral Health") + " and "
            "tasks.current_id >= 3 "
            "where beh_service_types.id > 0 "
            "group by beh_service_types.item";
      SqlToValueMap(dbTransaction, sql, total_map);
      report_form->lblTotalMental->Caption = total_map["MENTAL HEALTH"].c_str();
      report_form->lblTotalSubstance->Caption = total_map["SUBSTANCE ABUSE"].c_str();
      report_form->lblTotalDomestic->Caption = total_map["DOMESTIC VIOLENCE"].c_str();
      report_form->lblTotalResidential->Caption = total_map["RESIDENTIAL CARE"].c_str();
      for(map<string, string>::iterator itr = total_map.begin(); itr != total_map.end(); itr++)
        total_visits += StringToInt(itr->second);
      report_form->lblTotalTotal->Caption = IntToString(total_visits).c_str();

      // gender report---------------------------------------------------------------
      ExecuteStatement(dbTransaction, "delete from temp_beh_report");
      ExecuteStatement(dbTransaction, "delete from temp_beh_report_update");
      sql = "insert into temp_beh_report "
           "select gender_types.id, tasks.beh_service_id, count(tasks.id) "
            "from gender_types left join (people inner join tasks on tasks.person_id = people.id and "
            "tasks.date_of_task >= '" + begin_date + "' and "
            "tasks.date_of_task <= '" + end_date + "' and "
            "tasks.current_id >= 3 and "
            "tasks.group_id = " + lookupTables->IdOf("groups", "Behavioral Health") +
            ") on people.gender_id = gender_types.id "
            "group by gender_types.id, tasks.beh_service_id";
      ExecuteStatement(dbTransaction, sql);

      sql = "insert into temp_beh_report_update "
            "select gender_types.item gender, sum(m.total_visits) m_count, sum(s.total_visits) s_count, "
            "sum(d.total_visits) d_count, sum(r.total_visits) r_count "
            "from gender_types "
            "left join temp_beh_report m on m.types_id = gender_types.id and "
            "m.other_id = " + lookupTables->IdOf("beh_service_types", "Mental Health") + " "
            "left join temp_beh_report s on s.types_id = gender_types.id and "
            "s.other_id = " + lookupTables->IdOf("beh_service_types", "Substance Abuse") + " "
            "left join temp_beh_report d on d.types_id = gender_types.id and "
            "d.other_id = " + lookupTables->IdOf("beh_service_types", "Domestic Violence") + " "
            "left join temp_beh_report r on r.types_id = gender_types.id and "
            "r.other_id = " + lookupTables->IdOf("beh_service_types", "Residential Care") + " "
            "where gender_types.id > 0 "
            "group by gender_types.item";
      ExecuteStatement(dbTransaction, sql);

      sql = "update temp_beh_report_update set m_count = 0 where m_count is null";
      ExecuteStatement(dbTransaction, sql);
      sql = "update temp_beh_report_update set s_count = 0 where s_count is null";
      ExecuteStatement(dbTransaction, sql);
      sql = "update temp_beh_report_update set d_count = 0 where d_count is null";
      ExecuteStatement(dbTransaction, sql);
      sql = "update temp_beh_report_update set r_count = 0 where r_count is null";
      ExecuteStatement(dbTransaction, sql);

     sql = "select group_name, m_count, s_count, d_count, r_count, "
            "(m_count + s_count + d_count + r_count) total_count "
            "from temp_beh_report_update";

      dbGenderDataSet->SelectSQL->Add(sql.c_str());
      dbGenderDataSet->Open();
      dbGenderDataSet->First();
      report_form->gender_report->DataSet = dbGenderDataSet;
      report_form->txtGender->DataSet = dbGenderDataSet;
      report_form->txtGender->DataField = "group_name";
      report_form->txtGenderMental->DataSet = dbGenderDataSet;
      report_form->txtGenderMental->DataField = "m_count";
      report_form->txtGenderSubstance->DataSet = dbGenderDataSet;
      report_form->txtGenderSubstance->DataField = "s_count";
      report_form->txtGenderDomestic->DataSet = dbGenderDataSet;
      report_form->txtGenderDomestic->DataField = "d_count";
      report_form->txtGenderResidential->DataSet = dbGenderDataSet;
      report_form->txtGenderResidential->DataField = "r_count";
      report_form->txtGenderTotal->DataSet = dbGenderDataSet;
      report_form->txtGenderTotal->DataField = "total_count";

      // age report---------------------------------------------------------------
      ExecuteStatement(dbTransaction, "delete from temp_beh_report");
      ExecuteStatement(dbTransaction, "delete from temp_beh_report_update");
      sql = "insert into temp_beh_report "
            "select age_groups.id, tasks.beh_service_id, count(tasks.id) "
            "from age_groups left join (people inner join tasks on tasks.person_id = people.id and "
            "tasks.date_of_task >= '" + begin_date + "' and "
            "tasks.date_of_task <= '" + end_date + "' and "
            "tasks.current_id >= 3 and "
            "tasks.group_id = " + lookupTables->IdOf("groups", "Behavioral Health") + ") on "
            "age_groups.group_name = '" + SafeSQLString(cboAgeGroup->Text.c_str()) + "' and "
            "(select p_age from calc_age(people.birth_date)) >= age_groups.age_min and "
            "(select p_age from calc_age(people.birth_date)) <= age_groups.age_max "
            "group by age_groups.id, tasks.beh_service_id "
            "having age_groups.id > 0";
      DebugSql(this, &sql, other_forms, 9, dbTransaction);
      ExecuteStatement(dbTransaction, sql);

      sql = "insert into temp_beh_report_update "
            "select age_groups.id, "
            "sum(m.total_visits) m_count, sum(s.total_visits) s_count, "
            "sum(d.total_visits) d_count, sum(r.total_visits) r_count "
            "from age_groups "
            "left join temp_beh_report m on m.types_id = age_groups.id and "
            "m.other_id = " + lookupTables->IdOf("beh_service_types", "Mental Health") + " "
            "left join temp_beh_report s on s.types_id = age_groups.id and "
            "s.other_id = " + lookupTables->IdOf("beh_service_types", "Substance Abuse") + " "
            "left join temp_beh_report d on d.types_id = age_groups.id and "
            "d.other_id = " + lookupTables->IdOf("beh_service_types", "Domestic Violence") + " "
            "left join temp_beh_report r on r.types_id = age_groups.id and "
            "r.other_id = " + lookupTables->IdOf("beh_service_types", "Residential Care") + " "
            "where age_groups.group_name = '" + SafeSQLString(cboAgeGroup->Text.c_str()) + "' "
            "group by age_groups.id";
      ExecuteStatement(dbTransaction, sql);

      sql = "update temp_beh_report_update set m_count = 0 where m_count is null";
      ExecuteStatement(dbTransaction, sql);
      sql = "update temp_beh_report_update set s_count = 0 where s_count is null";
      ExecuteStatement(dbTransaction, sql);
      sql = "update temp_beh_report_update set d_count = 0 where d_count is null";
      ExecuteStatement(dbTransaction, sql);
      sql = "update temp_beh_report_update set r_count = 0 where r_count is null";
      ExecuteStatement(dbTransaction, sql);

      sql = "select age_groups.id, age_groups.age_display, "
            "m_count, s_count, d_count, r_count, "
            "(m_count + s_count + d_count + r_count) total_count "
            "from temp_beh_report_update, age_groups "
            "where cast(temp_beh_report_update.group_name as integer) = age_groups.id "
            "order by age_groups.id";
      DebugSql(this, &sql, other_forms, 9, dbTransaction);

      dbAgeDataSet->SelectSQL->Clear();
      dbAgeDataSet->SelectSQL->Add(sql.c_str());
      dbAgeDataSet->Open();
      dbAgeDataSet->First();
      report_form->age_report->DataSet = dbAgeDataSet;
      report_form->txtAgeGroup->DataSet = dbAgeDataSet;
      report_form->txtAgeGroup->DataField = "age_display";
      report_form->txtAgeMental->DataSet = dbAgeDataSet;
      report_form->txtAgeMental->DataField = "m_count";
      report_form->txtAgeSubstance->DataSet = dbAgeDataSet;
      report_form->txtAgeSubstance->DataField = "s_count";
      report_form->txtAgeDomestic->DataSet = dbAgeDataSet;
      report_form->txtAgeDomestic->DataField = "d_count";
      report_form->txtAgeResidential->DataSet = dbAgeDataSet;
      report_form->txtAgeResidential->DataField = "r_count";
      report_form->txtAgeTotal->DataSet = dbAgeDataSet;
      report_form->txtAgeTotal->DataField = "total_count";

      // tribe report--------------------------------------------------------------
      ExecuteStatement(dbTransaction, "delete from temp_beh_report");
      ExecuteStatement(dbTransaction, "delete from temp_beh_report_update");
      sql = "insert into temp_beh_report "
            "select tribe_types.id, tasks.beh_service_id, count(tasks.id) "
            "from tribe_types inner join affiliations  on affiliations.tribe_id = tribe_types.id "
            "inner join people on people.affiliations_id = affiliations.id "
            "inner join tasks on tasks.person_id = people.id and "
            "tasks.date_of_task >= '" + begin_date + "' and "
            "tasks.date_of_task <= '" + end_date + "' and "
            "tasks.current_id >= 3 and "
            "tasks.group_id = " + lookupTables->IdOf("groups", "Behavioral Health") + " "
            "group by tribe_types.id, tasks.beh_service_id "
            "having beh_service_id > 0";
      ExecuteStatement(dbTransaction, sql);

      sql = "insert into temp_beh_report_update "
            "select tribe_types.item tribe, sum(m.total_visits) m_count, sum(s.total_visits) s_count, "
            "sum(d.total_visits) d_count, sum(r.total_visits) r_count "
            "from tribe_types "
            "left join temp_beh_report m on m.types_id = tribe_types.id and "
            "m.other_id = " + lookupTables->IdOf("beh_service_types", "Mental Health") + " "
            "left join temp_beh_report s on s.types_id = tribe_types.id and "
            "s.other_id = " + lookupTables->IdOf("beh_service_types", "Substance Abuse") + " "
            "left join temp_beh_report d on d.types_id = tribe_types.id and "
            "d.other_id = " + lookupTables->IdOf("beh_service_types", "Domestic Violence") + " "
            "left join temp_beh_report r on r.types_id = tribe_types.id and "
            "r.other_id = " + lookupTables->IdOf("beh_service_types", "Residential Care") + " "
            "where tribe_types.id > 0 "
            "group by tribe_types.item";
      ExecuteStatement(dbTransaction, sql);

      sql = "update temp_beh_report_update set m_count = 0 where m_count is null";
      ExecuteStatement(dbTransaction, sql);
      sql = "update temp_beh_report_update set s_count = 0 where s_count is null";
      ExecuteStatement(dbTransaction, sql);
      sql = "update temp_beh_report_update set d_count = 0 where d_count is null";
      ExecuteStatement(dbTransaction, sql);
      sql = "update temp_beh_report_update set r_count = 0 where r_count is null";
      ExecuteStatement(dbTransaction, sql);

      sql = "select group_name, m_count, s_count, d_count, r_count, "
            "(m_count + s_count + d_count + r_count) total_count "
            "from temp_beh_report_update "
            "where (m_count + s_count + d_count + r_count) > 0 "
            "order by group_name";

      dbTribeDataSet->SelectSQL->Clear();
      dbTribeDataSet->SelectSQL->Add(sql.c_str());
      dbTribeDataSet->Open();
      dbTribeDataSet->First();
      report_form->tribe_report->DataSet = dbTribeDataSet;
      report_form->txtTribe->DataSet = dbTribeDataSet;
      report_form->txtTribe->DataField = "group_name";
      report_form->txtTribeMental->DataSet = dbTribeDataSet;
      report_form->txtTribeMental->DataField = "m_count";
      report_form->txtTribeSubstance->DataSet = dbTribeDataSet;
      report_form->txtTribeSubstance->DataField = "s_count";
      report_form->txtTribeDomestic->DataSet = dbTribeDataSet;
      report_form->txtTribeDomestic->DataField = "d_count";
      report_form->txtTribeResidential->DataSet = dbTribeDataSet;
      report_form->txtTribeResidential->DataField = "r_count";
      report_form->txtTribeTotal->DataSet = dbTribeDataSet;
      report_form->txtTribeTotal->DataField = "total_count";

      // race report--------------------------------------------------------------
      ExecuteStatement(dbTransaction, "delete from temp_beh_report");
      ExecuteStatement(dbTransaction, "delete from temp_beh_report_update");
      sql = "insert into temp_beh_report "
            "select race_types.id, tasks.beh_service_id, count(tasks.id) "
            "from race_types inner join people on people.race_id = race_types.id "
            "inner join tasks on tasks.person_id = people.id and "
            "tasks.date_of_task >= '" + begin_date + "' and "
            "tasks.date_of_task <= '" + end_date + "' and "
            "tasks.current_id >= 3 and "
            "tasks.group_id = " + lookupTables->IdOf("groups", "Behavioral Health") + " "
            "group by race_types.id, tasks.beh_service_id "
            "having beh_service_id > 0";
      ExecuteStatement(dbTransaction, sql);

      sql = "insert into temp_beh_report_update "
            "select race_types.item, sum(m.total_visits) m_count, sum(s.total_visits) s_count, "
            "sum(d.total_visits) d_count, sum(r.total_visits) r_count "
            "from race_types "
            "left join temp_beh_report m on m.types_id = race_types.id and "
            "m.other_id = " + lookupTables->IdOf("beh_service_types", "Mental Health") + " "
            "left join temp_beh_report s on s.types_id = race_types.id and "
            "s.other_id = " + lookupTables->IdOf("beh_service_types", "Substance Abuse") + " "
            "left join temp_beh_report d on d.types_id = race_types.id and "
            "d.other_id = " + lookupTables->IdOf("beh_service_types", "Domestic Violence") + " "
            "left join temp_beh_report r on r.types_id = race_types.id and "
            "r.other_id = " + lookupTables->IdOf("beh_service_types", "Residential Care") + " "
            "where race_types.id > 0 "
            "group by race_types.item";
      ExecuteStatement(dbTransaction, sql);

      sql = "update temp_beh_report_update set m_count = 0 where m_count is null";
      ExecuteStatement(dbTransaction, sql);
      sql = "update temp_beh_report_update set s_count = 0 where s_count is null";
      ExecuteStatement(dbTransaction, sql);
      sql = "update temp_beh_report_update set d_count = 0 where d_count is null";
      ExecuteStatement(dbTransaction, sql);
      sql = "update temp_beh_report_update set r_count = 0 where r_count is null";
      ExecuteStatement(dbTransaction, sql);

      sql = "select group_name, m_count, s_count, d_count, r_count, "
            "(m_count + s_count + d_count + r_count) total_count "
            "from temp_beh_report_update "
            "where (m_count + s_count + d_count + r_count) > 0 "
            "order by group_name";

      dbRaceDataSet->SelectSQL->Clear();
      dbRaceDataSet->SelectSQL->Add(sql.c_str());
      dbRaceDataSet->Open();
      dbRaceDataSet->First();
      report_form->race_report->DataSet = dbRaceDataSet;
      report_form->txtRace->DataSet = dbRaceDataSet;
      report_form->txtRace->DataField = "group_name";
      report_form->txtRaceMental->DataSet = dbRaceDataSet;
      report_form->txtRaceMental->DataField = "m_count";
      report_form->txtRaceSubstance->DataSet = dbRaceDataSet;
      report_form->txtRaceSubstance->DataField = "s_count";
      report_form->txtRaceDomestic->DataSet = dbRaceDataSet;
      report_form->txtRaceDomestic->DataField = "d_count";
      report_form->txtRaceResidential->DataSet = dbRaceDataSet;
      report_form->txtRaceResidential->DataField = "r_count";
      report_form->txtRaceTotal->DataSet = dbRaceDataSet;
      report_form->txtRaceTotal->DataField = "total_count";
    }
    catch(...)
    {
      PSay("Error running report", m_database_error, MB_OK);
      if (dbTransaction->Active)
        dbTransaction->Rollback();
    }

    Processing(false);

    compositeReport->Prepare();
    compositeReport->Preview();

    dbGenderDataSet->Close();
    dbAgeDataSet->Close();
    dbTribeDataSet->Close();
    dbRaceDataSet->Close();
    ExecuteStatement(dbTransaction, "delete from temp_beh_report");
    ExecuteStatement(dbTransaction, "delete from temp_beh_report_update");
    dbTransaction->Commit();
    report_form->CloseNow();

  }
  catch (...)
  {
    Processing(false);
    PSay("Error running report", m_database_error, MB_OK);
    if (dbTransaction->Active)
      dbTransaction->Rollback();
  }

  delete dbTransaction;
  delete dbGenderDataSet;
  delete dbAgeDataSet;
  delete dbTribeDataSet;
  delete dbRaceDataSet;
}
//---------------------------------------------------------------------------
void __fastcall TformLaunchBehProgramReport::compositeReportAddReports(TObject *Sender)
{
  compositeReport->Reports->Add(report_form->gender_report);
  compositeReport->Reports->Add(report_form->age_report);
  compositeReport->Reports->Add(report_form->tribe_report);
  compositeReport->Reports->Add(report_form->race_report);
}
//---------------------------------------------------------------------------
void __fastcall TformLaunchBehProgramReport::FormShortCut(TWMKey &Msg, bool &Handled)
{
  Handled = true;
  if (Msg.CharCode == VK_RETURN)
    btnRun->Click();
  else
    Handled = false;

  if (!Handled)
    TformSpecialGeneric::FormShortCut(Msg, Handled);
}
//---------------------------------------------------------------------------
// jhaley: override to unregister combo box
void __fastcall TformLaunchBehProgramReport::FormClose(TObject *Sender, TCloseAction &Action)
{
  lookupTables->UnregisterComboBox("age_group_types", cboAgeGroup);
  TformSpecialGeneric::FormClose(Sender, Action);
}
//---------------------------------------------------------------------------

