//---------------------------------------------------------------------------

#include <vcl.h>
#pragma hdrstop

#include "launchBehWorkloadReport.h"
#include "reportBehWorkload.h"
//---------------------------------------------------------------------------
#pragma package(smart_init)
#pragma link "launchGeneric"
#pragma link "codeRubberContainer"
#pragma resource "*.dfm"
TformLaunchBehWorkloadReport *formLaunchBehWorkloadReport;
//---------------------------------------------------------------------------
__fastcall TformLaunchBehWorkloadReport::TformLaunchBehWorkloadReport(TComponent* Owner)
        : TformLaunchGeneric(Owner)
{
  // TODO: Generalize me to work for other groups?
  pair<Pdate, Pdate> last_month_dates = GetPriorMonthDateRange(Pdate(Date()));
  dateBegin->Date = last_month_dates.first.OutDate();  // first day of last month
  dateEnd->Date   = last_month_dates.second.OutDate(); // last day of last month
}
//---------------------------------------------------------------------------
void __fastcall TformLaunchBehWorkloadReport::FormShow(TObject *Sender)
{
  event_filter.SetListViewHandle(listEventTypes);
  event_filter.SetSecurity("");
  event_filter.AddColumnSpec("Event Type", "events", as_text, true, 184 /*, prettyString*/);
  event_filter.AddColumnSpec("Internal Id", "internal_id", as_int, false, 80);
  event_filter.FetchPreferences("launchBehWorkloadReport.event_types_filter");
  event_filter.SetIconStatus(false);
  event_filter.SetAutosizeColumns(false);
  event_filter.SetCheck(true);
  event_filter.SetHeader(true);
  event_filter.RefreshDataSet();
  event_filter.SetSQLStatement("select item, id from event_type_types where exists "
                               "(select * from events EV where EV.group_id = '" + lookupTables->IdOf("groups", "Behavioral Health") + "' "
                               " and EV.event_type_id = event_type_types.id) "
                               "order by item");
  set<string> just_one_id;
  string the_one_id = GetOneField(dbDatabase, "select id from event_type_types where upper(item) = 'DEVELOPMENTAL SCREENING'");
  just_one_id.insert(the_one_id);
  event_filter.ForceRefreshDataSet();
  event_filter.SetCheckedWhereID(just_one_id, true);

  TformLaunchGeneric::FormShow(Sender);
}
//---------------------------------------------------------------------------
bool TformLaunchBehWorkloadReport::IsValidOnScreen()
{
  if (Pdate(dateBegin->Date) > Pdate(dateEnd->Date))
  {
    PSay("Please select a begin date prior or equal to the end date", m_invalid_data, MB_OK);
    dateBegin->SetFocus();
    return false;
  }

  vector<string> id_vec = event_filter.GetMultiID();
  if(id_vec.empty())
  {
    PSay("Please select one or more event types to consider for attendance", m_missing_data, MB_OK);
    listEventTypes->SetFocus();
    return false;
  }

  return true;
}
//---------------------------------------------------------------------------
bool TformLaunchBehWorkloadReport::BeforeRunReport()
{
  dbDataset = AddDataSet();
  return TformLaunchGeneric::BeforeRunReport();
}
//---------------------------------------------------------------------------
bool TformLaunchBehWorkloadReport::RunReport()
{
  string sql;
  string group_id   = lookupTables->IdOf("groups", "Behavioral Health");
  string begin_date = Pdate(dateBegin->Date).OutDate(mdy | yyyy);
  string end_date   = Pdate(dateEnd->Date).OutDate(mdy | yyyy);
  string date_title = begin_date + " - " + end_date;

    // Create the report form
  TformReportBehWorkload *report_form = NULL;
  FORM_TRY
    report_form = new TformReportBehWorkload(NULL);
  FORM_CATCH

  // Main query - Count attendance by event type
  sql = "select ETT.item event_type, cast(sum(EV.number_attended) as integer) num_attended "
        "from events EV "
        "inner join event_type_types ETT on EV.event_type_id = ETT.id "
        "where EV.group_id = '" + group_id + "' and EV.current_id+1 >= 3 and "
        "EV.date_of_event between '" + begin_date + "' and '" + end_date + "' and "
        + SafeInClause("EV.event_type_id", event_filter.GetMultiID()) + " "
        "group by 1 ";

  InitDataSet(dbDataset, sql);

  report_form->reportBeh->DataSet = dbDataset;

  // Title band
  report_form->lblTitleDates->Caption = date_title.c_str();  // display dates on title band

  // Page footer
  report_form->lblUser->Caption = current_fullname.c_str(); // display generating user's name on page footer

  // Detail band
  SetupQRDBText(report_form->txtEventType,  dbDataset, "event_type");
  SetupQRDBText(report_form->txtAttendance, dbDataset, "num_attended");

  // Summary band
  report_form->exprTotalAttendance->Expression = "SUM(num_attended)";

  // Get count of successful (with a client being seen) appointments in the same time period
  string appts_sql =
    "select count(distinct A.id) from appointments A "
    "where A.group_id = '" + group_id + "' and A.current_id+1 >= 3 and A.person_id <> 0 and "
    "A.appt_reason_id <> " + lookupTables->IdOf("appt_reason_types", "Block Schedule") + " and " // jhaley 20110720
    "(A.appointment_date between '" + begin_date + "' and '" + end_date + "' or "
    " A.begin_date between '" + begin_date + "' and '" + end_date + "' or "
    " A.end_date between '" + begin_date + "' and '" + end_date + "') and "
    "A.visit_outcome_id in ("
    + lookupTables->IdOf("visit_outcome_types", "As Scheduled") + ", "
    + lookupTables->IdOf("visit_outcome_types", "Walk-In")      + ", "
    + lookupTables->IdOf("visit_outcome_types", "Arrived Late") + ") ";
  string appt_count = GetOneField(dbDatabase, appts_sql);
  report_form->lblTotalApptNum->Caption = IntToString(StringToInt(appt_count)).c_str();

  // Get count of tasks that are visits
  string tasks_sql =
    "select count(distinct T.id) from tasks T "
    "where T.group_id = '" + group_id + "' and T.current_id+1 >= 3 and T.person_id <> 0 and "
    "T.date_of_task between '" + begin_date + "' and '" + end_date + "' and "
    "T.method_id = '" + lookupTables->IdOf("method_types", "Visit") + "' ";
  string task_count = GetOneField(dbDatabase, tasks_sql);
  report_form->lblTotalVisitNum->Caption = IntToString(StringToInt(task_count)).c_str();

  report_form->reportBeh->Preview();

  return true;
}

