//---------------------------------------------------------------------------

#include <vcl.h>
#pragma hdrstop

#include "launchBudgetModsReport.h"
#include "reportBudgetMods.h"
//---------------------------------------------------------------------------
#pragma package(smart_init)
#pragma link "codeSpecialGeneric"
#pragma resource "*.dfm"
TformLaunchBudgetModsReport *formLaunchBudgetModsReport;
//---------------------------------------------------------------------------
__fastcall TformLaunchBudgetModsReport::TformLaunchBudgetModsReport(TComponent* Owner)
        : TformSpecialGeneric(Owner)
{
 // some defaults.
 Pdate today_date(Date());
 Pdate begin_date = today_date;
 begin_date.day   = 1;

 if (1 > begin_date.month)
  begin_date.mod(-1, in_years);
 begin_date.month = 1;

 Pdate end_date = begin_date;
 end_date.mod(1, in_years);
 end_date.mod(-1, in_days);

 dateBeginDate->Date = begin_date.OutDate();
 dateEndDate->Date   = end_date.OutDate();

 buttonAccountResetClick(NULL);
}
//---------------------------------------------------------------------------
void __fastcall TformLaunchBudgetModsReport::buttonRunClick(
      TObject *Sender)
{
 // run
 Processing(true);

 string sql;
 string date_title = Pdate(dateBeginDate->DateTime).OutDate(global_mode) + " - " + Pdate(dateEndDate->DateTime).OutDate(global_mode);

 TIBTransaction * dbTransaction = new TIBTransaction(NULL);
 TIBDataSet * dbDataSet = new TIBDataSet(NULL);

 try
 {
  StdTransaction(dbTransaction, dbDatabase);

  vector<map<string, string> > temporary_transaction_data;

  sql = "select transactions.id tx_id, transactions.fiscal_date, "
        "(transactions.debit_amount - transactions.credit_amount) tx_amount, "
        "accounts.id acc_id, accounts.account_type_id "
        "from transactions "
        "inner join accounts on accounts.id = transactions.account_id "
        "where accounts.is_budget = 1 "
        "order by accounts.id, transactions.fiscal_date, transactions.id";
  if (!SqlToVecMap(dbTransaction, sql, temporary_transaction_data))
   throw(1);

  // in-memory modifications
  map<string, vector<financial> > transaction_to_cumulative_amounts;
  string previous_account    = "";
  string current_account     = "";
  string current_tx          = "";
  financial previous_total   = 0.00;
  financial current_change   = 0.00;
  financial new_total        = 0.00;
  vector<financial> current_values(3);
  for (vector<map<string, string> >::iterator i_tx = temporary_transaction_data.begin(); i_tx != temporary_transaction_data.end(); i_tx++)
  {
   current_account = (*i_tx)["acc_id"];
   current_tx      = (*i_tx)["tx_id"];
   current_change = StringToFinancial((*i_tx)["tx_amount"]);
   if (current_account != previous_account)
    previous_total = 0.00;

   if (!IsDebitPositiveByAccountType(GetAccountTypeByTypeName((*i_tx)["account_type_id"])))
    current_change = current_change * financial(-1.00);
   new_total = previous_total + current_change;

   current_values[0] = previous_total;
   current_values[2] = new_total;
   current_values[1] = current_change;
   transaction_to_cumulative_amounts[current_tx] = current_values;

   previous_account = current_account;
   previous_total   = new_total;
  }

  // paper report
  FORM_TRY
   TformReportBudgetMods * bsReport = new TformReportBudgetMods(NULL, &transaction_to_cumulative_amounts);

   dbTransaction->DefaultDatabase = dbDatabase; // TODO, is this needed, considering the call to StdTransaction? (change launchBudgetReport as well)
   dbTransaction->Active = true;
   dbDataSet->Database = dbDatabase;
   dbDataSet->Transaction = dbTransaction;
   dbDataSet->UniDirectional = false;

   string chunk_acc = "";
   for (int i = 0; i < listAccountOrder->Items->Count; i++)
   {
    if (i)
     chunk_acc += ", ";
    chunk_acc += "acc_level_" + string(listAccountOrder->Items->Item[i]->SubItems->Strings[0].c_str()) + ".short_item";
   }
   string chunk_date = "transactions.fiscal_date";

   sql = "select transactions.id tx_id, transactions.transaction_number tx_num, transactions.fiscal_date, transactions.note, "
         "(acc_level_one.short_item || '-' || acc_level_two.short_item || '-' || acc_level_three.short_item || '-' || acc_level_four.short_item) acc_name "
         "from transactions "
         "inner join accounts on accounts.id = transactions.account_id "
         "inner join acc_level_one on acc_level_one.id = accounts.acc_level_one_id "
         "inner join acc_level_two on acc_level_two.id = accounts.acc_level_two_id "
         "inner join acc_level_three on acc_level_three.id = accounts.acc_level_three_id "
         "inner join acc_level_four on acc_level_four.id = accounts.acc_level_four_id "
         "where accounts.is_budget = 1 "
         "and transactions.fiscal_date between '" + Pdate(dateBeginDate->DateTime).OutDate(global_mode) + "' and '" + Pdate(dateEndDate->DateTime).OutDate(global_mode) + "' "
         "and accounts.account_type_id in (" + lookupTables->IdOf("account_types", "Expense") + ", " + lookupTables->IdOf("account_types", "Revenue") + ") "
         "order by " + (radioArrangeByAccount->Checked ? chunk_acc + ", " + chunk_date + ", transactions.id" : chunk_date + ", transactions.transaction_number, transactions.note, " + chunk_acc);

   dbDataSet->SelectSQL->Add(sql.c_str());
   dbDataSet->Open();
   dbDataSet->First();
   bsReport->_labelTitleDate->Caption   = date_title.c_str();
   bsReport->_labelUser->Caption        = PrettyString(lookupTables->ValueOf("users", current_userid)).c_str();
   bsReport->reportBudgetMods->DataSet  = dbDataSet;
   bsReport->txtGroupInfo->DataSet      = dbDataSet;
   bsReport->txtTxNum->DataSet          = dbDataSet;
   bsReport->txtDetailInfo->DataSet     = dbDataSet;
   bsReport->txtNote->DataSet           = dbDataSet;
   bsReport->txtGroupInfo->DataField    = (radioArrangeByAccount->Checked ? "acc_name" : "fiscal_date");
   bsReport->txtTxNum->DataField        = "tx_num";
   bsReport->txtDetailInfo->DataField   = (radioArrangeByAccount->Checked ? "fiscal_date" : "acc_name");
   bsReport->txtNote->DataField         = "note";
   bsReport->bandNumber->Expression     = "tx_num";
   bsReport->bandGroupInfo->Expression  = bsReport->txtGroupInfo->DataField;

   Processing(false);
   bsReport->reportBudgetMods->Preview();
  FORM_CATCH

  dbTransaction->Commit();
 }
 catch (int oops)
 {
  Processing(false);
  PSay("Error running report", m_database_error, MB_OK, "oops number: " + IntToString(oops) + "; last_sql_lib_error: " + last_sql_lib_error.message);
  if (dbTransaction->Active)
   dbTransaction->Rollback();
 }
 catch (...)
 {
  Processing(false);
  PSay("Error running report", m_database_error, MB_OK, "oops number: none; last_sql_lib_error: " + last_sql_lib_error.message);
  if (dbTransaction->Active)
   dbTransaction->Rollback();
 }
 delete dbTransaction;
 delete dbDataSet;

 Processing(false);
}
//---------------------------------------------------------------------------
void __fastcall TformLaunchBudgetModsReport::buttonCloseClick(
      TObject *Sender)
{
 // close
 Close();
}
//---------------------------------------------------------------------------
void __fastcall TformLaunchBudgetModsReport::FormShortCut(TWMKey &Msg,
      bool &Handled)
{
 Handled = true;
 if (Msg.CharCode == VK_RETURN)
  buttonRun->Click();
 else
  Handled = false;

 // this should be done elsewhere. most launch forms will ignore default specialgeneric shortcut keys! - jhaley: DONE 20100930
 TformSpecialGeneric::FormShortCut(Msg, Handled);
}
//---------------------------------------------------------------------------
void __fastcall TformLaunchBudgetModsReport::buttonAccountUpClick(
      TObject *Sender)
{
 if (listAccountOrder->Selected)
  if (listAccountOrder->Selected->Index > 0)
  {
   int curr_index = listAccountOrder->Selected->Index;
   TListItem * prev = listAccountOrder->Items->Item[curr_index - 1];
   TListItem * temp = listAccountOrder->Items->Insert(curr_index + 1);
   temp->Assign(prev);
   prev->Delete();
  }
}
//---------------------------------------------------------------------------
void __fastcall TformLaunchBudgetModsReport::buttonAccountResetClick(
      TObject *Sender)
{
 listAccountOrder->Items->Clear();
 TListItem * temp = NULL;

 temp = listAccountOrder->Items->Add();
 temp->Caption = "Fund";
 temp->SubItems->Add("one");

 temp = listAccountOrder->Items->Add();
 temp->Caption = "General Ledger";
 temp->SubItems->Add("two");

 temp = listAccountOrder->Items->Add();
 temp->Caption = "Department";
 temp->SubItems->Add("three");

 temp = listAccountOrder->Items->Add();
 temp->Caption = "Sub-Department";
 temp->SubItems->Add("four");
}
//---------------------------------------------------------------------------
void __fastcall TformLaunchBudgetModsReport::buttonAccountDownClick(
      TObject *Sender)
{
 if (listAccountOrder->Selected)
  if (listAccountOrder->Selected->Index < listAccountOrder->Items->Count - 1)
  {
   int curr_index = listAccountOrder->Selected->Index;
   TListItem * next = listAccountOrder->Items->Item[curr_index + 1];
   TListItem * temp = listAccountOrder->Items->Insert(curr_index);
   temp->Assign(next);
   next->Delete();
  }
}
//---------------------------------------------------------------------------

