//---------------------------------------------------------------------------

#include <vcl.h>
#pragma hdrstop

#include "launchCHSBillingWorkloadReport.h"
#include "reportCHSBillingWorkload.h"
#include "derivedheaders.h"
//---------------------------------------------------------------------------
#pragma package(smart_init)
#pragma link "launchGeneric"
#pragma link "codeBaseDropDown"
#pragma link "codeCheckListDropDown"
#pragma resource "*.dfm"
TformLaunchCHSBillingWorkloadReport *formLaunchCHSBillingWorkloadReport;
//---------------------------------------------------------------------------
__fastcall TformLaunchCHSBillingWorkloadReport::TformLaunchCHSBillingWorkloadReport(TComponent* Owner, const string &p_group_id)
        : TformLaunchGeneric(Owner), running_group_id()
{
  pair<Pdate, Pdate> last_month_dates = GetPriorMonthDateRange(Pdate(Date())); // jhaley - use GetPriorMonthDateRange
  dateBegin->Date = last_month_dates.first.OutDate();
  dateEnd->Date   = last_month_dates.second.OutDate();

  cbddBillTypes->Filter->SetSecurity("");
  cbddBillTypes->Filter->AddColumnSpec("Bill Type", "", as_text, true, 210 /*, rawString */);
  cbddBillTypes->Filter->AddColumnSpec("Internal Id", "internal_id", as_int, false, 60);
  cbddBillTypes->Filter->SetSQLStatement("select item, id from bill_types where id > 0 and valid = 1");
  cbddBillTypes->Filter->FetchPreferences("launchchsbillingworkload.bill_types_cbdd");

  lookupTables->RegisterComboBox("account_mask_types", cboAccount);

  running_batch = suppress_header = suppress_footer = false; // jhaley

  // jhaley 20120510: Support special behavior for Purchasing & Supply department
  running_group_id = p_group_id;

  if(running_group_id == lookupTables->IdOf("groups", "Purchasing/Supply"))
  {
    // Nope! Not for you, young grasshoppah.
    btnRunBatch->Enabled = false;
    btnRunBatch->Visible = false;
  }
}
//---------------------------------------------------------------------------
__fastcall TformLaunchCHSBillingWorkloadReport::~TformLaunchCHSBillingWorkloadReport()
{
  if (lookupTables->TableName(cboAccount) == "account_mask_types")
    lookupTables->UnregisterComboBox("account_mask_types", cboAccount);
}
//---------------------------------------------------------------------------
bool TformLaunchCHSBillingWorkloadReport::BeforeRunReport()
{
  dbDataset = NULL;
  dbDataset = new TIBDataSet(NULL);
  return TformLaunchGeneric::BeforeRunReport();
}
//---------------------------------------------------------------------------
bool TformLaunchCHSBillingWorkloadReport::RunReport()
{
  checkpoint_branch = "Main";
  checkpoint = 0;
  string sql;

  string begin_date = Pdate(dateBegin->Date).OutDate(mdy | yyyy);
  string end_date =  Pdate(dateEnd->Date).OutDate(mdy | yyyy);
  string date_title = "Bills processed within:  " + begin_date + " - " + end_date;

  dbDataset->Database = dbDatabase;
  dbDataset->Transaction = report_transaction;
  dbDataset->UniDirectional = false;

  TformReportCHSBillingWorkload *report = NULL;
  FORM_TRY
    report = new TformReportCHSBillingWorkload(NULL);
  FORM_CATCH

  string group_id = running_group_id; // jhaley - use running_group_id here.

  // Alternate titles for some departments
  if (group_id == lookupTables->IdOf("groups", "Purchasing/Supply"))
    report->lblTitle->Caption = "AST Health System Purchasing and Supply Workload";

  if (!suppress_header)
  {
    report->lblStaff->Enabled = true;
    report->memoEmployees->Enabled = true;
    vector<string> employee_vec;
    sql = "select coalesce(users.item, '') || ' - ' || coalesce(users.title, '') employee "
          "from users where users.valid = 1 and "
          "users.group_id = '" + group_id + "' "
          "order by users.i_level desc, users.item";
    SqlToVec(report_transaction, sql, employee_vec);
    for(vector<string>::iterator itr = employee_vec.begin(); itr != employee_vec.end(); itr++)
       report->memoEmployees->Lines->Add((*itr).c_str());
  }
  else
  {
    report->lblStaff->Enabled = false;
    report->memoEmployees->Enabled = false;
  }

  // jhaley 20110211: get bill types
  string bill_types_sql = "inner join bill_types on bills.bill_id = bill_types.id ";
  vector<map<string, string> > selected_vecmap = cbddBillTypes->Filter->GetAllData(true, true);
  if (selected_vecmap.size() > 0)
  {
    vector<string> id_vec;

    for (vector<map<string, string> >::iterator i_row = selected_vecmap.begin(); i_row != selected_vecmap.end(); i_row++)
      id_vec.push_back((*i_row)["id"]);

    bill_types_sql += "and " + SafeInClause("bill_types.id", id_vec) + " ";
  }

  // jhaley 20110211: restriction by account mask
  string account_sql = "select accounts.id from accounts where accounts.account_type_id = '" + lookupTables->IdOf("account_types", "Expense") + "' ";
  string account_mask_id = lookupTables->IdOf("account_mask_types", cboAccount);
  boolean join_account_mask_types = false;
  if (!IsBlankOrZero(account_mask_id))
  {
    join_account_mask_types = true;
    account_sql +=
      "and ( "
      "(case when account_mask_types.negate_mask = 1 and accounts.name not like account_mask_types.mask then 1 else 0 end) = 1 or "
      "(case when account_mask_types.negate_mask = 0 and accounts.name like account_mask_types.mask then 1 else 0 end) = 1) ";
  }

  sql  = "select purchase_reqs.purchase_req_number, bill_types.id bill_id, bill_types.item bill_type, area_types.id area_id, "
         " area_types.item area_type, tax_entities.id tax_entity_id, tax_entities.name, "
         " case when tin.identifier = '999999999' then '' else tin.identifier end identifier, "
         " sum(transactions.debit_amount - transactions.credit_amount) paid, count(distinct(bills.id)) num_bills, "
         " count(distinct(bills.purchase_req_id)) num_prs "
         "from tax_entities "
         " right join bills on bills.actual_tax_id = tax_entities.id "
         + bill_types_sql +
         //" inner join bill_types on bills.bill_id = bill_types.id "
         " left join purchase_reqs on bills.purchase_req_id = purchase_reqs.id ";
  if (join_account_mask_types)
    sql += "inner join account_mask_types on account_mask_types.id = '" + account_mask_id + "' ";
  sql += " inner join transactions on transactions.bill_id = bills.id and transactions.account_id in (" + account_sql + ") "
         " left join tasks visit on bills.task_id = visit.id and visit.parent_task_id > 0 "
         "   and visit.group_id = " + lookupTables->IdOf("groups", "Contract Health") + " and visit.services_provided_id = 0 "
         " left join tasks cob on visit.parent_task_id = cob.id and cob.parent_task_id = 0 "
         "   and cob.group_id = " + lookupTables->IdOf("groups", "Contract Health") + " and cob.services_provided_id > 0 "
         " left join services_provided on cob.services_provided_id = services_provided.id "
         " left join area_types on services_provided.area_id = area_types.id "
         " left join entity_ids tin on tin.tax_entity_id = tax_entities.id and tin.current_id >= 3 "
         "   and tin.entity_id_id = '" + lookupTables->IdOf("entity_id_types", "TIN/EIN") + "' "
         "where bills.purchase_req_id > 0 and tax_entities.id > 0 and purchase_reqs.date_sent between '" + begin_date + "' and '" + end_date + "' "
         " and purchase_reqs.last_updated_by in " // jhaley 20110331: Include PRs generated by Administration users as well (Executive Assistant may fill in for TPB)
         " (select users.id from users where users.group_id = " + group_id + " or users.group_id = '" + lookupTables->IdOf("groups", "Administration") + "') "
         "group by purchase_reqs.purchase_req_number, bill_types.id, bill_types.item, area_types.id, area_types.item, "
         " tax_entities.id, tax_entities.name, tin.identifier, tax_entities.current_id, tax_entities.id "
         "having sum(transactions.debit_amount - transactions.credit_amount) > 0.00 "
         "order by bill_types.item, bill_types.id, area_types.item, area_types.id, tax_entities.name, tax_entities.id ";

  DebugSql(this, &sql, other_forms, 9, report_transaction);

  dbDataset->SelectSQL->Clear();
  dbDataset->SelectSQL->Add(sql.c_str());
  dbDataset->Open();
  dbDataset->First();

  report->report->DataSet = dbDataset;
  report->lblDateTitle->Caption = date_title.c_str();
  report->lblUser->Caption = current_fullname.c_str();

  // jhaley 20110211: display account name on report
  if (cboAccount->Text == "")
    report->lblAccount->Caption = "All Accounts";
  else
    report->lblAccount->Caption = cboAccount->Text.c_str();

  report->txtPRNum->DataSet = dbDataset;
  report->txtPRNum->DataField = "purchase_req_number";
  report->txtBillType->DataSet = dbDataset;
  report->txtBillType->DataField = "bill_type";
  report->txtServiceArea->DataSet = dbDataset;
  report->txtServiceArea->DataField = "area_type";
  report->txtTaxEntity->DataSet = dbDataset;
  report->txtTaxEntity->DataField = "name";
  report->txtTaxID->DataSet = dbDataset;
  report->txtTaxID->DataField = "identifier";
  report->txtBills->DataSet = dbDataset;
  report->txtBills->DataField = "num_bills";
  report->txtPRs->DataSet = dbDataset;
  report->txtPRs->DataField = "num_prs";
  report->exprPaid->Expression = "FORMATNUMERIC('$###,###,##0.00', paid)";
  report->exprPaidServiceTotal->Expression = "FORMATNUMERIC('$###,###,##0.00', SUM(paid))";
  report->exprBillsServiceTotal->Expression = "FORMATNUMERIC('###,###,##0', SUM(num_bills))";
  report->exprPRsServiceTotal->Expression = "FORMATNUMERIC('###,###,##0', SUM(num_prs))";
  report->exprPaidBillTypeTotal->Expression = "FORMATNUMERIC('$###,###,##0.00', SUM(paid))";
  report->exprBillsBillTypeTotal->Expression = "FORMATNUMERIC('###,###,##0', SUM(num_bills))";
  report->exprPRsBillTypeTotal->Expression = "FORMATNUMERIC('###,###,##0', SUM(num_prs))";

  report->bandGroupHeader->Expression = "bill_id";
  report->bandGroupHeader->Height = 22;

  report->bandGroupHeader2->Expression = "area_id";
  report->bandGroupHeader2->Height = 22;

  if (!suppress_footer)
  {
    bool suppress_denials = false;

    // Purchasing/Supply is not concerned with CHS denials.
    if (running_group_id == lookupTables->IdOf("groups", "Purchasing/Supply"))
      suppress_denials = true;

    report->bandSummary->Enabled = true;

    if (!suppress_denials)
    {
      report->bandSummaryChild->Enabled = true;
      vector<map<string, string> > letters;
      sql = "select area_types.item area, count(denial_letters.id) num_letters "
            "from denial_letters "
            "inner join denials on denial_letters.denial_id = denials.id "
            "inner join services_provided on denials.services_provided_id = services_provided.id "
            "inner join area_types on services_provided.area_id = area_types.id "
            "where denial_letters.date_processed between '" + begin_date + "' and '" + end_date + "' "
            " and denial_letters.denial_authority_id = '" + lookupTables->IdOf("denial_authority_types", "Third Party Billing") + "' "
            "group by area_types.id, area_types.item "
            "having count(denial_letters.id) > 0 "
            "order by area_types.item ";
      DebugSql(this, &sql, other_forms, 9, report_transaction);
      SqlToVecMap(report_transaction, sql, letters);

      if (letters.empty())
      {
        report->bandSummary->Height = 64;
        report->lblDenialLetters->Enabled = false;
        report->memoLetterType->Enabled = false;
        report->memoLetterCount->Enabled = false;
        report->bandSummaryChild->Enabled = false;
        report->bandSummaryChild->Height = 0;
      }
      else
      {
        int total_letters = 0;
        for (vector<map<string, string> >::iterator itr = letters.begin(); itr != letters.end(); itr++)
        {
          report->memoLetterType->Lines->Add((PrettyString((*itr)["area"])).c_str());
          report->memoLetterCount->Lines->Add((PrettyString((*itr)["num_letters"])).c_str());
          total_letters += StringToInt((*itr)["num_letters"]);
        }
        report->lblLetterTotal->Caption = IntToString(total_letters).c_str();
      }
    }
    else
    {
      report->bandSummaryChild->Enabled = false;
      report->lblDenialLetters->Enabled = false;
      report->memoLetterType->Enabled   = false;
      report->memoLetterCount->Enabled  = false;
    }
  }
  else
  {
    report->bandSummary->Enabled = false;
    report->bandSummaryChild->Enabled = false;
  }

  Processing(false);

  if (running_batch) // jhaley
    report->report->PreviewModal();
  else
  {
    btnRunBatch->Enabled = false;
    report->report->Preview();
    btnRunBatch->Enabled = true;
  }

  dbDataset->Close();

  return true;
}
//---------------------------------------------------------------------------
bool TformLaunchCHSBillingWorkloadReport::AfterRunReport()
{
  delete dbDataset;

  bool all_is_well = TformLaunchGeneric::AfterRunReport();

  /*  err ... copy paste oversight?  This stuff is from the CHS obligated/paid (expensed) report...
  all_is_well = all_is_well && ExecuteStatement(report_transaction, "delete from temp_obpd_cobs"); checkpoint++;
  all_is_well = all_is_well && ExecuteStatement(report_transaction, "delete from temp_obpd_visits"); checkpoint++;
  all_is_well = all_is_well && ExecuteStatement(report_transaction, "delete from temp_obpd_cob_expense"); checkpoint++;
  all_is_well = all_is_well && ExecuteStatement(report_transaction, "delete from temp_obpd_bill_expense"); checkpoint++;
  all_is_well = all_is_well && ExecuteStatement(report_transaction, "delete from temp_obpd_totals"); checkpoint++;
  all_is_well = all_is_well && ExecuteStatement(report_transaction, "delete from temp_obpd_services_provided"); checkpoint++;
  */
  return all_is_well;
}
//---------------------------------------------------------------------------
void __fastcall TformLaunchCHSBillingWorkloadReport::btnRunBatchClick(TObject *Sender)
{
  running_batch = true;

  // Be sure the dropdown has been populated first
  cbddBillTypes->Filter->ForceRefreshDataSet();
  cbddBillTypes->DisplayFieldName = "Bill Type";

  // Remember current values
  vector<map<string, string> > selected_vecmap = cbddBillTypes->Filter->GetAllData(true, true);
  string account_mask = cboAccount->Text.c_str();

  // Batch reports run only for "Vendor Bills"
  set<string> bill_ids;
  bill_ids.insert(lookupTables->IdOf("bill_types", "Incoming Vendor Bill"));
  cbddBillTypes->Filter->SetCheckedWhereID(bill_ids, true);
  cbddBillTypes->Filter->SetCheckedWhereNotID(bill_ids, false);

  // Run report for AST Health
  cboAccount->Text = "AST Tribal Health Authority, Inc.";
  suppress_header = false;
  suppress_footer = true;
  btnRunReportClick(NULL);

  // Run report for LI-SI-WI-NWI
  cboAccount->Text = "LI-SI-WI-NWI Health, Inc.";
  suppress_header = true;
  suppress_footer = false;
  btnRunReportClick(NULL);

  // Restore values that were on the form at the beginning
  set<string> id_set;
  for (vector<map<string, string> >::iterator i_row = selected_vecmap.begin(); i_row != selected_vecmap.end(); ++i_row)
    id_set.insert((*i_row)["id"]);
  cbddBillTypes->Filter->SetCheckedWhereID(id_set, true);
  cbddBillTypes->Filter->SetCheckedWhereNotID(id_set, false);
  //cbddBillTypes->Filter->RefreshDataSet();
  cboAccount->Text = account_mask.c_str();

  running_batch = suppress_header = suppress_footer = false;
}
//---------------------------------------------------------------------------
void __fastcall TformLaunchCHSBillingWorkloadReport::cbddBillTypesDown(TObject *Sender)
{
  cbddBillTypes->Filter->ForceRefreshDataSet();
  cbddBillTypes->DisplayFieldName = "Bill Type";
}
//---------------------------------------------------------------------------
// Old "Details" Code:
/*
  if (radioDetail->Checked)
  {
    report->bandDetail->Height = 16;
    report->bandChild->Enabled = true;
    report->bandChild->Height = 16;

    report->txtPersonName->DataSet = dbDataset;
    report->txtPersonName->DataField = "person_name";
    report->txtCobDate->DataSet = dbDataset;
    report->txtCobDate->DataField = "cob_date";
    report->txtCobNumber->DataSet = dbDataset;
    report->txtCobNumber->DataField = "cob_number";
    report->txtCobExpense->DataSet = dbDataset;
    report->txtCobExpense->DataField = "cob_expense";                   // TODO:  Expense fields should be Expression's, FORMATNUMERIC'd
    report->txtUnpaidVisitDate->DataSet = dbDataset;
    report->txtUnpaidVisitDate->DataField = "unpaid_visit_date";
    report->txtUnpaidBillDate->DataSet = dbDataset;
    report->txtUnpaidBillDate->DataField = "unpaid_bill_date";
    report->txtUnpaidPrDate->DataSet = dbDataset;
    report->txtUnpaidPrDate->DataField = "unpaid_pr_date";
    report->txtUnpaidPrNumber->DataSet = dbDataset;
    report->txtUnpaidPrNumber->DataField = "unpaid_pr_number";
    report->txtUnpaidExpense->DataSet = dbDataset;
    report->txtUnpaidExpense->DataField = "unpaid_expense";
    report->txtPaidVisitDate->DataSet = dbDataset;
    report->txtPaidVisitDate->DataField = "paid_visit_date";
    report->txtPaidBillDate->DataSet = dbDataset;
    report->txtPaidBillDate->DataField = "paid_bill_date";
    report->txtPaidPrDate->DataSet = dbDataset;
    report->txtPaidPrDate->DataField = "paid_pr_date";
    report->txtPaidPrNumber->DataSet = dbDataset;
    report->txtPaidPrNumber->DataField = "paid_pr_number";
    report->txtPaidCheckDate->DataSet = dbDataset;
    report->txtPaidCheckDate->DataField = "paid_check_date";
    report->txtPaidCheckNumber->DataSet = dbDataset;
    report->txtPaidCheckNumber->DataField = "paid_check_number";
    report->txtPaidExpense->DataSet = dbDataset;
    report->txtPaidExpense->DataField = "paid_expense";

    report->exprTotalCobVisits->Expression = "SUM(cob_visits)";
    report->exprTotalCobPeople->Expression = "SUM(cob_people)";
    report->exprTotalCobAmount->Expression = "FORMATNUMERIC('$###,###,##0.00', SUM(cob_expense))";
    report->exprTotalUnpaidVisits->Expression = "SUM(unpaid_visits)";
    report->exprTotalUnpaidPeople->Expression = "SUM(unpaid_people)";
    report->exprTotalUnpaidAmount->Expression = "FORMATNUMERIC('$###,###,##0.00', SUM(unpaid_expense))";
    report->exprTotalPaidVisits->Expression = "SUM(paid_visits)";
    report->exprTotalPaidPeople->Expression = "SUM(paid_people)";
    report->exprTotalPaidAmount->Expression = "FORMATNUMERIC('$###,###,##0.00', SUM(paid_expense))";

    if (how_to_run == by_service || how_to_run == by_service_area)
    {
      report->txtProvider->Enabled = false;      // because this data isn't useful in by_service mode
      report->lblProvider->Enabled = false;      // because that column is now empty
      report->txtDetailProvider->Enabled = true; // because we don't have a provider on the group header rows so we put it on the per-visit rows
      report->txtDetailProvider->DataSet = dbDataset;
      report->txtDetailProvider->DataField = "provider"; // and populate it, yup
    }
  }
  else
  {
    report->bandDetail->Height = 0;
    report->bandChild->Enabled = false;
    report->bandChild->Height = 0;

    report->exprTotalCobVisits->Expression = "SUM(cob_visits)";
    report->exprTotalCobPeople->Expression = "SUM(cob_people)";
    report->exprTotalCobAmount->Expression = "FORMATNUMERIC('$###,###,##0.00', SUM(cob_amount))";
    report->exprTotalUnpaidVisits->Expression = "SUM(unpaid_visits)";
    report->exprTotalUnpaidPeople->Expression = "SUM(unpaid_people)";
    report->exprTotalUnpaidAmount->Expression = "FORMATNUMERIC('$###,###,##0.00', SUM(unpaid_amount))";
    report->exprTotalPaidVisits->Expression = "SUM(paid_visits)";
    report->exprTotalPaidPeople->Expression = "SUM(paid_people)";
    report->exprTotalPaidAmount->Expression = "FORMATNUMERIC('$###,###,##0.00', SUM(paid_amount))";
  }
*/

