//---------------------------------------------------------------------------

#include <vcl.h>
#pragma hdrstop

#include "launchCheckJustification.h"
#include "reportCheckJustification.h"
//---------------------------------------------------------------------------
#pragma package(smart_init)
#pragma link "codeSpecialGeneric"
#pragma resource "*.dfm"
TformLaunchCheckJustification *formLaunchCheckJustification;
//---------------------------------------------------------------------------
__fastcall TformLaunchCheckJustification::TformLaunchCheckJustification(TComponent* Owner, string id)
        : TformSpecialGeneric(Owner)
{
 check_id = id;
 string sql;
 sql = "select tax_entities.tax_entity_id "
       "from checks "
       "left join tax_entities on checks.tax_entity_id = tax_entities.id "
       "where checks.id = " + check_id;
}
//---------------------------------------------------------------------------

void __fastcall TformLaunchCheckJustification::btnCloseClick(TObject *Sender)
{
 Close();
}
//---------------------------------------------------------------------------
static void SetupQRDBText(TQRDBText *qrdbText, TIBDataSet *dataSet, const string &name)
{
  qrdbText->DataSet   = dataSet;
  qrdbText->DataField = name.c_str();
}
//---------------------------------------------------------------------------
void __fastcall TformLaunchCheckJustification::btnRunClick(TObject *Sender)
{
  TIBTransaction *dbTransaction = NULL;
  TIBDataSet     *dbDataSet     = NULL;
  TformReportCheckJustification *cj_report = NULL;

  // create report form
  FORM_TRY
    cj_report = new TformReportCheckJustification(NULL);
  FORM_CATCH_RETURN("TformReportCheckJustification", )

  try
  {
    dbTransaction = new TIBTransaction(NULL);
  }
  catch(...)
  {
  }

  try
  {
    dbDataSet = new TIBDataSet(NULL);
  }
  catch(...)
  {
  }

  if(!dbTransaction || !dbDataSet)
  {
    PSay("Failed to create one or more needed database objects", m_program_error, MB_OK);
    delete dbTransaction;
    delete dbDataSet;
    return;
  }

  if(!IsBlankOrZero(check_id))
  {
    try
    {
      dbTransaction->DefaultDatabase = dbDatabase;
      dbDataSet->Database = dbDatabase;
      dbDataSet->Transaction = dbTransaction;
      dbDataSet->UniDirectional = false;
      dbTransaction->StartTransaction();
    }
    catch(...)
    {
      PSay("Warning: failed to initialize transaction or dataset", m_database_error, MB_OK);
    }

    string sql =
      "select purchase_reqs.purchase_req_number, affiliations.cdib, tasks.patient_control_number, "
      " tasks.date_of_task, bills.invoice_amount, "
      " (sum(transactions.debit_amount) - sum(transactions.credit_amount)) amount_paid "
      "from checks, purchase_reqs "
      "left join bills on purchase_reqs.id = bills.purchase_req_id "
      "left join tasks on bills.task_id = tasks.id "
      "left join people on tasks.person_id = people.id "
      "left join affiliations on people.affiliations_id = affiliations.id "
      "left join transactions on bills.id = transactions.bill_id "
      "left join accounts on transactions.account_id = accounts.id "
      "where purchase_reqs.check_id = checks.id and "
      " accounts.account_type_id = '" + lookupTables->IdOf("account_types", "EXPENSE") + "' and "
      // TODO: possible misuse of effective_date, after changes (not necessarily "today")
      " checks.id = " + check_id + " and transactions.effective_date <= purchase_reqs.date_sent "
      "group by purchase_reqs.purchase_req_number, affiliations.cdib, tasks.patient_control_number, "
      " tasks.date_of_task, bills.invoice_amount";

    try
    {
      dbDataSet->SelectSQL->Add(sql.c_str());
    }
    catch(...)
    {
      PSay("Warning: failed to add SQL to dataset", m_program_error, MB_OK);
    }

    // sql for provider name
    string provider_name;
    try
    {
       sql = "select tax_entities.name "
             "from checks "
             "left join tax_entities on checks.tax_entity_id = tax_entities.id "
             "where checks.id = " + check_id;
       provider_name = GetOneField(dbTransaction, sql);
    }
    catch(...)
    {
      PSay("Warning: failed to get provider name for report", m_database_error, MB_OK);
      provider_name = "Unknown Provider";
    }

    string check_number;
    try
    {
      check_number = GetOneField(dbTransaction, "select check_number from checks where id = " + check_id);
    }
    catch(...)
    {
      PSay("Warning: failed to get check number for report", m_database_error, MB_OK);
      check_number = "Unknown Check";
    }

    string user = PrettyString(lookupTables->ValueOf("users", current_userid));

    // put info in the name quick report object
    try
    {
      dbDataSet->Open();
      dbDataSet->First();

      cj_report->qpCheckJustification->DataSet = dbDataSet;

      cj_report->qrpProviderName->Caption = provider_name.c_str();
      cj_report->qrpCheckNo->Caption      = check_number.c_str();
      cj_report->lblUser->Caption         = user.c_str();

      SetupQRDBText(cj_report->qrpPRNum,             dbDataSet, "purchase_req_number");
      SetupQRDBText(cj_report->qrpCDIB,              dbDataSet, "cdib");
      SetupQRDBText(cj_report->qrpPatientAccountNum, dbDataSet, "patient_control_number");
      SetupQRDBText(cj_report->qrpDOS,               dbDataSet, "date_of_task");

      cj_report->qreInvoiceAmount->Expression    = "FORMATNUMERIC('$ ###,##0.00', invoice_amount)";
      cj_report->qreAmountPaid->Expression       = "FORMATNUMERIC('$ ###,##0.00', amount_paid)";
      cj_report->qreWriteOff->Expression         = "FORMATNUMERIC('$ ###,##0.00', invoice_amount - amount_paid)";
      cj_report->qreSumInvoiceAmount->Expression = "FORMATNUMERIC('$ ###,##0.00', SUM(invoice_amount))";
      cj_report->qreSumAmountPaid->Expression    = "FORMATNUMERIC('$ ###,##0.00', SUM(amount_paid))";
      cj_report->qreSumWriteOff->Expression      = "FORMATNUMERIC('$ ###,##0.00', SUM(invoice_amount - amount_paid))";

      // jhaley 20120703: must use PreviewModal!
      cj_report->qpCheckJustification->PreviewModal();
    }
    catch(...)
    {
      PSay("Check justification report failed to initialize, please try again.", m_program_error, MB_OK);
    }
  }
  else
  {
    PSay("No checks were entered", m_missing_data, MB_OK);
  }

  delete dbTransaction;
  delete dbDataSet;
}
//---------------------------------------------------------------------------
void __fastcall TformLaunchCheckJustification::FormShortCut(TWMKey &Msg,
      bool &Handled)
{
  Handled = true;
  if (Msg.CharCode == VK_RETURN)
    btnRun->Click();
  else
    Handled = false;

  if (!Handled)
    TformSpecialGeneric::FormShortCut(Msg, Handled);
}
//---------------------------------------------------------------------------

