//---------------------------------------------------------------------------

#include <vcl.h>
#pragma hdrstop

#include "launchClinicPatientListReport.h"
#include "reportClinicPatientList.h"
//---------------------------------------------------------------------------
#pragma package(smart_init)
#pragma link "codeSpecialGeneric"
#pragma resource "*.dfm"
TformLaunchClinicPatientListReport *formLaunchClinicPatientListReport;
//---------------------------------------------------------------------------
__fastcall TformLaunchClinicPatientListReport::TformLaunchClinicPatientListReport(TComponent* Owner, string p_id)
        : TformSpecialGeneric(Owner)
{
  clinic_id = p_id;
  dateEnd->Date = Now();
  Pdate begin = Now(); begin.mod(-6, in_months);
  dateBegin->Date = begin.OutDate();
  dateInactiveEnd->Date = dateEnd->Date;
  dateInactiveBegin->Date = dateBegin->Date;

  set<string> can_view_charts = GetSpecialGroupPermissions("charts");

  radioSome->Enabled = ! can_view_charts.empty();

  radioInactive->Enabled = radioSome->Enabled;
  radioAllCharts->Enabled = radioSome->Enabled;
  radioActiveCharts->Enabled = radioSome->Enabled;
  radioInactiveCharts->Enabled = radioSome->Enabled;
  string location_groups_sql = "select id, item, valid from groups where " + SafeInClause("id", SetToVec(can_view_charts));
  lookupTables->RegisterComboBox("groups", cboClinic, location_groups_sql);
  // DONE : now checks to make sure the item is in the list before setting it
  AnsiString desired_group = lookupTables->ValueOf("groups", clinic_id).c_str();
  if (cboClinic->Items->IndexOf(desired_group) != -1)
    cboClinic->Text = desired_group;
  if (cboClinic->Text == "") // we'll be nice and fill in the clinic name for them
    // or I guess I could've done the above this way.  Two entirely different ways of checking if the item is in the list.  w00t?
    if (lookupTables->IsValidId("groups", current_groupid, cboClinic)) // is current user's group in the list?  if so, set the clinic automatically...
      cboClinic->Text = lookupTables->ValueOf("groups", current_groupid).c_str();
}
//---------------------------------------------------------------------------
void __fastcall TformLaunchClinicPatientListReport::btnCloseClick(TObject *Sender)
{
  lookupTables->UnregisterComboBox("groups", cboClinic);
  Close();
}
//---------------------------------------------------------------------------
bool TformLaunchClinicPatientListReport::IsValidOnScreen()
{
  if (radioSome->Checked && (editMinVisits->Text == "" || (StringToInt(editMinVisits->Text.c_str()) < 1) ))
  {
     PSay("Please enter a valid number of visits.", m_message, MB_OK);
     editMinVisits->SetFocus();
     return false;
  }
  else if (cboClinic->Text == "" || !lookupTables->IsInLookUpTable("groups", cboClinic))
  {
     PSay("Please select a clinic.", m_message, MB_OK);
     cboClinic->SetFocus();
     return false;
  }
  else if (!radioAllCharts->Checked && !radioActiveCharts->Checked && !radioInactiveCharts->Checked)
  {
     PSay("You have not selected an option under \"Charts\".  If these options are disabled, you do not have the required permissions to run this report.", m_message, MB_OK);
     cboClinic->SetFocus();
     return false;
  }
  else if (!HasLocationPermissions(GetSpecialGroupPermissions("general"), cboClinic->Text.c_str()))
  {
     PSay("You do not have permission to run this report for " + string(cboClinic->Text.c_str()) + ".", m_message, MB_OK);
     cboClinic->SetFocus();
     return false;
  }

  return true;
}
//---------------------------------------------------------------------------
void __fastcall TformLaunchClinicPatientListReport::btnRunClick(TObject *Sender)
{
  if (IsValidOnScreen())
  {
    Processing(true);

    clinic_id = lookupTables->IdOf("groups", cboClinic).c_str();

    string begin_date, end_date, date_title;
    if (radioSome->Checked)
    {
      begin_date = Pdate(dateBegin->Date).OutDate(mdy | yyyy);
      end_date   = Pdate(dateEnd->Date).OutDate(mdy | yyyy);
    }
    if (radioInactive->Checked)
    {
      begin_date = Pdate(dateInactiveBegin->Date).OutDate(mdy | yyyy);
      end_date   = Pdate(dateInactiveEnd->Date).OutDate(mdy | yyyy);
    }
    else if (radioAll->Checked)
    {
      begin_date = "";
      end_date   = "";
    }

    // jhaley 20101112: allow restriction by AST tribal affiliation status
    int tribal_affiliation;
    if (radioAffiliationsAST->Checked)
      tribal_affiliation = TformReportClinicPatientList::ast_only;
    else if(radioAffiliationsNonAST->Checked)
      tribal_affiliation = TformReportClinicPatientList::nonast_only;
    else
      tribal_affiliation = TformReportClinicPatientList::everyone;

    TformReportClinicPatientList::Execute(this, radioAll->Checked, radioSome->Checked, radioInactive->Checked,
      editMinVisits->Text.c_str(), begin_date, end_date, clinic_id, radioAllCharts->Checked, radioActiveCharts->Checked,
      radioInactiveCharts->Checked, tribal_affiliation);
    Processing(false);
  }
}
//---------------------------------------------------------------------------
void __fastcall TformLaunchClinicPatientListReport::FormShortCut(TWMKey &Msg, bool &Handled)
{
  Handled = true;
  if (Msg.CharCode == VK_RETURN)
    btnRun->Click();
  else
    Handled = false;

  if (!Handled)
    TformSpecialGeneric::FormShortCut(Msg, Handled);
}
//---------------------------------------------------------------------------
void __fastcall TformLaunchClinicPatientListReport::FormClose(TObject *Sender, TCloseAction &Action)
{
  TformSpecialGeneric::FormClose(Sender, Action);
}
//---------------------------------------------------------------------------
void __fastcall TformLaunchClinicPatientListReport::radioClick(TObject *Sender)
{
  panelSome->Enabled = radioSome->Checked;
  panelInactive->Enabled = radioInactive->Checked;
}
//---------------------------------------------------------------------------

