//---------------------------------------------------------------------------

#include <vcl.h>
#pragma hdrstop

#include "launchCobProductivityReport.h"
//#include "reportCobProductivity.h" - jhaley: no longer needed
#include "reportCHSServiceAreaBreakdown.h"
//---------------------------------------------------------------------------
#pragma package(smart_init)
#pragma link "codeSpecialGeneric"
#pragma resource "*.dfm"
TformLaunchCobProductivityReport *formLaunchCobProductivityReport;
//---------------------------------------------------------------------------
__fastcall TformLaunchCobProductivityReport::TformLaunchCobProductivityReport(TComponent* Owner)
        : TformSpecialGeneric(Owner)
{
  Pdate begin_date = CurrentPdate();
  begin_date.day = 1;
  dateBegin->Date = begin_date.OutDate();
  dateEnd->Date = Date();
}
//---------------------------------------------------------------------------
void __fastcall TformLaunchCobProductivityReport::btnCloseClick(TObject *Sender)
{
  Close();
}
//---------------------------------------------------------------------------
bool TformLaunchCobProductivityReport::DoServiceAreaBreakdown()
{
  string sql;
  TIBDataSet * dbDataSet = new TIBDataSet(NULL);
  bool success = true;
  string begin_date = Pdate(dateBegin->Date).OutDate(mdy | yyyy);
  string end_date   = Pdate(dateEnd->Date).OutDate(mdy | yyyy);
  string date_title = begin_date + " - " + end_date;

  try
  {
    dbDataSet->Database = dbDatabase;
    dbDataSet->Transaction = dbTransaction;
    dbDataSet->UniDirectional = false;

    // jhaley 20100928: Ideally, this needs to be generated dynamically in case of new service areas; although the report will still require
    // modification in that case because the current set of area types consume all available space even in landscape mode. So for now I am
    // just adding the extra area types that were previously missing as static columns, as a stop-gap.

    sql = "select (extract(year from date_of_task) || '-' || LPAD(extract(month from date_of_task), 2, '0')) yearmo, users.item user_name, "
          "count(distinct case when area_types.id = " + lookupTables->IdOf("area_types", "Dental")          + " then tasks.id else null end) dental, "
          "count(distinct case when area_types.id = " + lookupTables->IdOf("area_types", "Hearing")         + " then tasks.id else null end) hearing, "
          "count(distinct case when area_types.id = " + lookupTables->IdOf("area_types", "Medical")         + " then tasks.id else null end) medical, "
          "count(distinct case when area_types.id = " + lookupTables->IdOf("area_types", "Orthodontics")    + " then tasks.id else null end) ortho, "
          "count(distinct case when area_types.id = " + lookupTables->IdOf("area_types", "Vision")          + " then tasks.id else null end) vision, "
          "count(distinct case when area_types.id = " + lookupTables->IdOf("area_types", "Prescription")    + " then tasks.id else null end) prescrip, "
          "count(distinct case when area_types.id = " + lookupTables->IdOf("area_types", "Psych Inpatient") + " then tasks.id else null end) psych, "
          "count(distinct case when area_types.id = " + lookupTables->IdOf("area_types", "Out-Of-Network")  + " then tasks.id else null end) outofnet, "
          "count(distinct tasks.id) total "
          "from users "
          "inner join tasks on tasks.user_id = users.id "
          "inner join services_provided on tasks.services_provided_id = services_provided.id "
          "inner join area_types on services_provided.area_id = area_types.id "
          "where users.group_id in (" + lookupTables->IdOf("groups", "Contract Health") + "," + lookupTables->IdOf("groups", "Administration") + ") and "
          "tasks.group_id in (" + lookupTables->IdOf("groups", "Contract Health") + "," + lookupTables->IdOf("groups", "Administration") + ") and "
          "tasks.services_provided_id > 0 and tasks.current_id+1 > 3 and "
          "tasks.date_of_task between '" + begin_date + "' and '" + end_date + "' "
          "group by 1, 2 "
          "order by 1, 2 ";

    dbDataSet->SelectSQL->Add(sql.c_str());
    dbDataSet->Open();
    dbDataSet->First();

    FORM_TRY
      TformReportCHSServiceAreaBreakdown *report_form = new TformReportCHSServiceAreaBreakdown(NULL);
      report_form->reportBreakdown->DataSet = dbDataSet;
      report_form->lblDateTitle->Caption = date_title.c_str();
      report_form->lblUser->Caption = PrettyString(lookupTables->ValueOf("users", current_userid)).c_str();

      // setup group header band
      report_form->bandGroupHeader->Expression = "yearmo";
      report_form->exprMonth->Expression = "yearmo";

      // setup detail band
      report_form->txtDetailEmployee->DataSet   = dbDataSet;
      report_form->txtDetailEmployee->DataField = "user_name";
      report_form->txtDetailDental->DataSet     = dbDataSet;
      report_form->txtDetailDental->DataField   = "dental";
      report_form->txtDetailHearing->DataSet    = dbDataSet;
      report_form->txtDetailHearing->DataField  = "hearing";
      report_form->txtDetailMedical->DataSet    = dbDataSet;
      report_form->txtDetailMedical->DataField  = "medical";
      report_form->txtDetailOrtho->DataSet      = dbDataSet;
      report_form->txtDetailOrtho->DataField    = "ortho";
      report_form->txtDetailVision->DataSet     = dbDataSet;
      report_form->txtDetailVision->DataField   = "vision";
      report_form->txtDetailPrescription->DataSet     = dbDataSet;
      report_form->txtDetailPrescription->DataField   = "prescrip";
      report_form->txtDetailPsychInpatient->DataSet   = dbDataSet;
      report_form->txtDetailPsychInpatient->DataField = "psych";
      report_form->txtDetailOutOfNetwork->DataSet     = dbDataSet;
      report_form->txtDetailOutOfNetwork->DataField   = "outofnet";
      report_form->txtDetailTotal->DataSet   = dbDataSet;
      report_form->txtDetailTotal->DataField = "total";

      // setup group footer band
      report_form->exprGroupDental->Expression         = "SUM(dental)";
      report_form->exprGroupHearing->Expression        = "SUM(hearing)";
      report_form->exprGroupMedical->Expression        = "SUM(medical)";
      report_form->exprGroupOrtho->Expression          = "SUM(ortho)";
      report_form->exprGroupVision->Expression         = "SUM(vision)";
      report_form->exprGroupPrescription->Expression   = "SUM(prescrip)";
      report_form->exprGroupPsychInpatient->Expression = "SUM(psych)";
      report_form->exprGroupOutOfNetwork->Expression   = "SUM(outofnet)";
      report_form->exprGroupTotal->Expression          = "SUM(total)";

      // setup summary band
      report_form->exprSummaryDental->Expression         = "SUM(dental)";
      report_form->exprSummaryHearing->Expression        = "SUM(hearing)";
      report_form->exprSummaryMedical->Expression        = "SUM(medical)";
      report_form->exprSummaryOrtho->Expression          = "SUM(ortho)";
      report_form->exprSummaryVision->Expression         = "SUM(vision)";
      report_form->exprSummaryPrescription->Expression   = "SUM(prescrip)";
      report_form->exprSummaryPsychInpatient->Expression = "SUM(psych)";
      report_form->exprSummaryOutOfNetwork->Expression   = "SUM(outofnet)";
      report_form->exprSummaryGrandTotal->Expression     = "SUM(total)";

      // jhaley 20100928: moved denial letters to service area breakdown report to eliminate redundant CHS workload report
      vector<map<string, string> > letters;
      sql = "select area_types.item area, count(denial_letters.id) num_letters "
            "from denial_letters "
            "inner join denials on denial_letters.denial_id = denials.id "
            "inner join services_provided on denials.services_provided_id = services_provided.id "
            "inner join area_types on services_provided.area_id = area_types.id "
            "where denial_letters.date_processed between '" + begin_date + "' and '" + end_date + "' "
            " and denial_letters.denial_authority_id = '" + lookupTables->IdOf("denial_authority_types", "CMMR") + "' "
            "group by area_types.id, area_types.item "
            "having count(denial_letters.id) > 0 "
            "order by area_types.item ";
      SqlToVecMap(dbTransaction, sql, letters);
      if (letters.empty())
      {
        report_form->bandSummary->Height = 40;
        report_form->lblDenialLetters->Enabled = false;
        report_form->memoLetterType->Enabled = false;
        report_form->memoLetterCount->Enabled = false;
        report_form->bandSummaryChild->Enabled = false;
        report_form->bandSummaryChild->Height = 0;
      }
      else
      {
        int total_letters = 0;
        for (vector<map<string, string> >::iterator itr = letters.begin(); itr != letters.end(); itr++)
        {
          report_form->memoLetterType->Lines->Add(PrettyString((*itr)["area"]).c_str());
          report_form->memoLetterCount->Lines->Add(PrettyString((*itr)["num_letters"]).c_str());
          total_letters += StringToInt((*itr)["num_letters"]);
        }
        report_form->lblLetterTotal->Caption = IntToString(total_letters).c_str();
      }

      // jhaley 20100928: merged everything into breakdown report
      Processing(false);
      report_form->reportBreakdown->Preview();
    FORM_CATCH

    dbDataSet->Close();
  }
  catch(...)
  {
    PSay("Error running service breakdown report", m_database_error, MB_OK);
    success = false;
  }

  delete dbDataSet;

  return success;
}
//---------------------------------------------------------------------------
void __fastcall TformLaunchCobProductivityReport::btnRunClick(TObject *Sender)
{
  Processing(true);

  FORM_TRY
    dbTransaction = new TIBTransaction(NULL);
  FORM_CATCH

  try
  {
    dbTransaction->DefaultDatabase = dbDatabase;
    dbTransaction->StartTransaction();

    // jhaley 20100928: Run service area breakdown only (now includes denial letters info)
    DoServiceAreaBreakdown();

    dbTransaction->Commit();
  }
  catch (...)
  {
    Processing(false);
    PSay("Error running report", m_database_error, MB_OK);
    if (dbTransaction->Active)
      dbTransaction->Rollback();
  }

  delete dbTransaction;
}
//---------------------------------------------------------------------------
void __fastcall TformLaunchCobProductivityReport::FormShortCut(TWMKey &Msg, bool &Handled)
{
  Handled = true;
  if (Msg.CharCode == VK_RETURN)
    btnRun->Click();
  else
    Handled = false;

  if (!Handled)
    TformSpecialGeneric::FormShortCut(Msg, Handled);
}
//---------------------------------------------------------------------------

