//---------------------------------------------------------------------------

#include <vcl.h>
#pragma hdrstop

#include "launchComMeasurements.h"
#include "codeSearchPerson.h"
#include "reportComMeasurements.h"
//---------------------------------------------------------------------------
#pragma package(smart_init)
#pragma link "codeSpecialGeneric"
#pragma resource "*.dfm"
TformLaunchComMeasurementReport *formLaunchComMeasurementReport;
//---------------------------------------------------------------------------
__fastcall TformLaunchComMeasurementReport::TformLaunchComMeasurementReport(TComponent* Owner, string id)
        : TformSpecialGeneric(Owner)
{
  dateBegin->Date = Now();
  dateEnd->Date = Now();
  CreateLink(txtName, btnPersonSearch, this, id, "people");
}
//---------------------------------------------------------------------------
void __fastcall TformLaunchComMeasurementReport::btnPersonSearchClick(
      TObject *Sender)
{
   // create and show new person search form
  vector<string> return_vector;
  FORM_TRY
    TformSearchPerson * formPersonSearch = new TformSearchPerson(NULL,&return_vector);
    formPersonSearch->ShowTaskModal(this);
  FORM_CATCH
  if (! return_vector.empty())
    CreateLink(txtName, btnPersonSearch, this, return_vector[0], "people");
}
//---------------------------------------------------------------------------
void __fastcall TformLaunchComMeasurementReport::btnCloseClick(
      TObject *Sender)
{
  Close();
}
//---------------------------------------------------------------------------
void __fastcall TformLaunchComMeasurementReport::btnRunClick(
      TObject *Sender)
{
  string person_id = GetValidLinkId(txtName);
  if (person_id != "")
  {
    Processing(true);

    TIBTransaction * dbTransaction = new TIBTransaction(NULL);
    TIBDataSet * dbDataSet = new TIBDataSet(NULL);
    string sql;
    string begin_date = Pdate(dateBegin->Date).OutDate(mdy | yyyy);
    string end_date =  Pdate(dateEnd->Date).OutDate(mdy | yyyy);
    string date_title = begin_date + " - " + end_date;
    string title = "Measurement report for " + GetPersonName(dbDatabase, person_id, first_last);

    try
    {
      dbTransaction->DefaultDatabase = dbDatabase;
      dbDataSet->Database = dbDatabase;
      dbDataSet->Transaction = dbTransaction;
      dbDataSet->UniDirectional = false;


      sql = "select obs_response_sets.date_of_observation,  "
            "obs_weight.response_metric weight, "
            "obs_sys.response_metric blood_pressure_hi, "
            "obs_dia.response_metric blood_pressure_low, "
            "obs_temp.response_metric temperature, "
            "obs_pulse.response_metric pulse, "
            "obs_resp.response_metric respirations, "
            "obs_bs.response_metric blood_sugar, "
            "obs_fbs.response_metric fasting_blood_sugar "
            "from obs_response_sets "
            "left join obs_responses obs_weight on obs_weight.obs_response_set_id = obs_response_sets.id and obs_weight.obs_question_id = " + lookupTables->IdOf("obs_questions", "Weight") + " "
            "left join obs_responses obs_sys on obs_sys.obs_response_set_id = obs_response_sets.id and obs_sys.obs_question_id = " + lookupTables->IdOf("obs_questions", "Systolic Blood Pressure") + " "
            "left join obs_responses obs_dia on obs_dia.obs_response_set_id = obs_response_sets.id and obs_dia.obs_question_id = " + lookupTables->IdOf("obs_questions", "Diastolic Blood Pressure") + " "
            "left join obs_responses obs_temp on obs_temp.obs_response_set_id = obs_response_sets.id and obs_temp.obs_question_id = " + lookupTables->IdOf("obs_questions", "Temperature") + " "
            "left join obs_responses obs_pulse on obs_pulse.obs_response_set_id = obs_response_sets.id and obs_pulse.obs_question_id = " + lookupTables->IdOf("obs_questions", "Pulse") + " "
            "left join obs_responses obs_resp on obs_resp.obs_response_set_id = obs_response_sets.id and obs_resp.obs_question_id = " + lookupTables->IdOf("obs_questions", "Respirations") + " "
            "left join obs_responses obs_bs on obs_bs.obs_response_set_id = obs_response_sets.id and obs_bs.obs_question_id = " + lookupTables->IdOf("obs_questions", "Blood Sugar") + " "
            "left join obs_responses obs_fbs on obs_fbs.obs_response_set_id = obs_response_sets.id and obs_fbs.obs_question_id = " + lookupTables->IdOf("obs_questions", "Fasting Blood Sugar") + " "
            "where "
            "obs_response_sets.group_id = " + lookupTables->IdOf("groups", "Community Health") + " and "
            "obs_response_sets.date_of_observation >= '" + begin_date + "' and "
            "obs_response_sets.date_of_observation <= '" + end_date + "' and "
            "obs_response_sets.person_id = " + person_id + " and "
            "obs_response_sets.current_id > 2 "
            "order by "
            "obs_response_sets.date_of_observation";

      dbTransaction->StartTransaction();
      dbDataSet->SelectSQL->Add(sql.c_str());
      dbDataSet->Open();
      dbDataSet->First();

      FORM_TRY
        TformReportComMeasurement * measurementReport = new TformReportComMeasurement(NULL);
        measurementReport->measurementReport->DataSet = dbDataSet;
        measurementReport->lblTitle->Caption = title.c_str();
        measurementReport->lblDateTitle->Caption = date_title.c_str();
        measurementReport->txtDate->DataSet = dbDataSet;
        measurementReport->lblUser->Caption = PrettyString(lookupTables->ValueOf("users", current_userid)).c_str();
        measurementReport->txtDate->DataField = "date_of_observation";
        measurementReport->exprWeight->Expression = "FORMATNUMERIC('##0.00', weight)";
        measurementReport->exprHiBlood->Expression = "FORMATNUMERIC('##0.00', blood_pressure_hi)";
        measurementReport->exprLowBlood->Expression = "FORMATNUMERIC('##0.00', blood_pressure_low)";
        measurementReport->exprTemperature->Expression = "FORMATNUMERIC('##0.00', temperature)";
        measurementReport->exprPulse->Expression = "FORMATNUMERIC('##0', pulse)";
        measurementReport->exprRespiration->Expression = "FORMATNUMERIC('##0', respirations)";
        measurementReport->exprBloodSugar->Expression = "FORMATNUMERIC('##0', blood_sugar)";
        measurementReport->exprFastBloodSugar->Expression = "FORMATNUMERIC('##0', fasting_blood_sugar)";
        Processing(false);
        measurementReport->measurementReport->Preview();
      FORM_CATCH
      dbDataSet->Close();
      dbTransaction->Commit();
   }
   catch (...)
   {
     Processing(false);
     PSay("Error running report", m_database_error, MB_OK);
     if (dbTransaction->Active)
       dbTransaction->Rollback();
   }
    delete dbTransaction;
    delete dbDataSet;
  }
  else
  {
    PSay("Please enter a client name", m_missing_data, MB_OK);
    btnPersonSearch->SetFocus();
  }
}
//---------------------------------------------------------------------------
void __fastcall TformLaunchComMeasurementReport::FormShortCut(TWMKey &Msg,
      bool &Handled)
{
  Handled = true;
  if (Msg.CharCode == VK_RETURN)
    btnRun->Click();
  else
    Handled = false;

  if (!Handled)
    TformSpecialGeneric::FormShortCut(Msg, Handled);
}
//---------------------------------------------------------------------------

