//---------------------------------------------------------------------------

#include <vcl.h>
#pragma hdrstop

#include "launchDenialLettersReport.h"
#include "reportDenialLetter.h"
//---------------------------------------------------------------------------
#pragma package(smart_init)
#pragma link "launchGeneric"
#pragma link "codeBaseDropDown"
#pragma link "codeCheckListDropDown"
#pragma resource "*.dfm"
TformLaunchDenialLettersReport *formLaunchDenialLettersReport;
//---------------------------------------------------------------------------
__fastcall TformLaunchDenialLettersReport::TformLaunchDenialLettersReport(TComponent* Owner)
        : TformLaunchGeneric(Owner)
{
  Pdate today_date(Date());
  Pdate begin_date = today_date;
  begin_date.day   = 1;           // first day of this month
  Pdate end_date = begin_date;    // first day of this month
  begin_date.mod(-1, in_months);  // first day of last month
  end_date.mod(-1, in_days);      // last day of last month

  dateBegin->Date   = begin_date.OutDate();
  dateEnd->Date     = end_date.OutDate();
  dateOneDate->Date = today_date.OutDate();

  cbddAuthority->Filter->SetSecurity("");
  cbddAuthority->Filter->AddColumnSpec("Authority", "", as_text, true, 185, rawString);
  cbddAuthority->Filter->AddColumnSpec("Current Id", "current_id", as_int, false, 40);
  cbddAuthority->Filter->AddColumnSpec("Internal Id", "internal_id", as_int, false, 60);
  cbddAuthority->Filter->SetSQLStatement("select item, " + CleverCurrentId("valid", "1", "Current", "Expired") + ", id from denial_authority_types where id > 0");
  cbddAuthority->Filter->FetchPreferences("denial_letters_report.authority");
  //cbddAuthority->Filter->ShowHeaders(false);
  cbddAuthority->Filter->SetIconStatus(true);
  cbddAuthority->DropDownHeight = 96;
}
//---------------------------------------------------------------------------
bool TformLaunchDenialLettersReport::IsValidOnScreen()
{
  if (radioProcessed->Checked)
  {
    vector<string> authority = cbddAuthority->Filter->GetMultiID();
    if (authority.empty())
    {
      PSay("You must check at least one denial letter processing authority to use date processed.", m_message, MB_OK);
      cbddAuthority->SetFocus();
      return false;
    }
  }
  if (dateBegin->Date > dateEnd->Date)
  {
    PSay("End date must be greater than or equal to begin date.", m_message, MB_OK);
    dateEnd->SetFocus();
    return false;
  }

  return true;
}
//---------------------------------------------------------------------------
bool TformLaunchDenialLettersReport::BeforeRunReport()
{
  return TformLaunchGeneric::BeforeRunReport();
}
//---------------------------------------------------------------------------
bool TformLaunchDenialLettersReport::RunReport()
{
  pvecmap letters_vecmap;
  vector<string> letters;

  string sql = "select DL.id, DL.denial_status_id, DL.denial_authority_id, DL.date_processed, DN.person_id, TE.name "
               "from denial_letters DL "
               "inner join denials DN on DL.denial_id = DN.id "
               "inner join services_provided SP on DN.services_provided_id = SP.id "
               "inner join tax_entities TE on SP.tax_entity_id = TE.id ";

  // jhaley 20100712: join to denial_items when DOS is selected
  if (radioDOS->Checked)
    sql += "inner join denial_items DI on DI.denial_letter_id = DL.id ";

  // jhaley 20100716: Note that expired *is* included here because "Resolved" (aka "approved") letters
  // are given a status of "Expired".
  sql += "where DL.current_id >= 2 and ";

  string begin_date = "'" + Pdate(dateBegin->Date).OutDate(mdy | yyyy) + "'";
  string end_date   = "'" + Pdate(dateEnd->Date).OutDate(mdy | yyyy) + "'";

  // TODO:  when we add the table back in with DOS->amount, we can look at all dates, because
  //        right now, a BDOS < begin_date and EDOS > end_date MAY or MAY NOT have a DOS within
  //        the specified range
  // jhaley: ^^^^ == ???? - it seems to work as-is, but we'll see.

  if (radioProcessed->Checked)
    sql += SafeInClause("DL.denial_authority_id", cbddAuthority->Filter->GetMultiID()) + " and "
           "DL.date_processed between " + begin_date + " and " + end_date;
  else if (radioDOS->Checked)
    sql += "((DI.begin_date_of_service between " + begin_date + " and " + end_date + ") or "
           " (DI.end_date_of_service between " + begin_date + " and " + end_date + ") or "
           " (DI.begin_date_of_service < " + begin_date + " and DI.end_date_of_service > " + end_date + "))";
  else if (radioEffective->Checked)
    sql += "DL.effective_date between " + begin_date + " and " + end_date; // jhaley 20100712: changed to effective_date

  SqlToVecMap(dbDatabase, sql, letters_vecmap);

  // jhaley 20110714: Insert CHS mailing log tasks, and build letters vector while doing it.
  bool task_error = false;

  for(pvecmap::iterator i_letter = letters_vecmap.begin(); i_letter != letters_vecmap.end(); ++i_letter)
  {
    try
    {
      string msg =
        "Printed "
        + PrettyString(lookupTables->ValueOf("denial_authority_types", (*i_letter)["denial_authority_id"])) + " '"
        + PrettyString(lookupTables->ValueOf("denial_status_types",    (*i_letter)["denial_status_id"])) +
        "' letter for services by " + (*i_letter)["name"] + " processed on " + (*i_letter)["date_processed"];

      if(!InsertCHSMailTask(NULL, (*i_letter)["person_id"], msg))
        task_error = true;
    }
    catch(...) // Don't let anything throw out of this loop.
    {
      task_error = true;
    }

    letters.push_back((*i_letter)["id"]);
  }
  if(task_error)
    PSay("Failed to log one or more denial letter mailout tasks", m_warning, MB_OK);

  TformReportDenialLetter::Do(letters, string(radioOneDate->Checked ? dateOneDate->Date.DateString().c_str() : ""),
                              checkDuplicates->Checked); // jhaley 20110315: allow printing dupes w/CHS chart #'s

  return true;
}
//---------------------------------------------------------------------------
bool TformLaunchDenialLettersReport::AfterRunReport()
{
  return TformLaunchGeneric::AfterRunReport();
}
//---------------------------------------------------------------------------
// jhaley 20100920: As on TformSearchEvent, do the filter refresh here, on demand, rather than in the constructor, as
// for some reason it's too early to be run from there and causes seemingly phantom access violations (can't nail them
// down to a particular operation so far - crash seems to be inside a VCL routine).
void __fastcall TformLaunchDenialLettersReport::cbddAuthorityDown(TObject *Sender)
{
  cbddAuthority->Filter->ForceRefreshDataSet();
  cbddAuthority->DisplayFieldName = "Authority";
}
//---------------------------------------------------------------------------

