//---------------------------------------------------------------------------

#include <vcl.h>
#pragma hdrstop

#include "launchDentalBillingDataReport.h"
#include "reportDentalBillingData.h"
//---------------------------------------------------------------------------
#pragma package(smart_init)
#pragma link "launchGeneric"
#pragma resource "*.dfm"
TformLaunchDentalBillingDataReport *formLaunchDentalBillingDataReport;
//---------------------------------------------------------------------------
__fastcall TformLaunchDentalBillingDataReport::TformLaunchDentalBillingDataReport(TComponent* Owner)
        : TformLaunchGeneric(Owner)
{
  Pdate today_date(Date());
  Pdate begin_date = today_date;
  begin_date.day   = 1;           // first day of this month
  Pdate end_date = begin_date;    // first day of this month
  begin_date.mod(-1, in_months);  // first day of last month
  end_date.mod(-1, in_days);      // last day of last month

  dateBegin->Date = begin_date.OutDate();
  dateEnd->Date   = end_date.OutDate();
}
//---------------------------------------------------------------------------
bool TformLaunchDentalBillingDataReport::IsValidOnScreen()
{
  if (dateBegin->Date > dateEnd->Date)
  {
    PSay("End date must be greater than or equal to begin date.", m_invalid_data, MB_OK);
    dateEnd->SetFocus();
    return false;
  }

  return true;
}
//---------------------------------------------------------------------------
bool TformLaunchDentalBillingDataReport::BeforeRunReport()
{
  dbDataset = NULL;
  dbDataset = new TIBDataSet(NULL);
  return TformLaunchGeneric::BeforeRunReport();
}
//---------------------------------------------------------------------------
bool TformLaunchDentalBillingDataReport::RunReport()
{
  string begin_date = Pdate(dateBegin->Date).OutDate(mdy | yyyy);
  string end_date   = Pdate(dateEnd->Date).OutDate(mdy | yyyy);
  string date_title = begin_date + " - " + end_date;

  dbDataset->Database = dbDatabase;
  dbDataset->Transaction = report_transaction;
  dbDataset->UniDirectional = false;


  TformReportDentalBillingData *report_form = NULL;
  FORM_TRY
    report_form = new TformReportDentalBillingData(NULL);
  FORM_CATCH

  string join_type = radioInsured->Checked ? "inner" : "left";

  string sql = "select T.date_of_task DOS, "
               "(PN.last_name || ', ' || "
               " (case when PN.first_name = '' then '' else ' ' end) || PN.first_name || "
               " (case when PN.middle_name = '' then '' else ' ' end) || PN.middle_name || "
               " (case when PN.suffix = '' then '' else ' ' end) || PN.suffix) person_name, "
               "(case when DenIns.id is not null then 'Yes' else 'No' end) insured, "
               "(case when B.id is not null then 'Yes' else 'No' end) on_bill, "
               "Px.cpt_hcpcs, Px.charges, TE.name insurance, MT.id month_id, MT.item month_name "
               "from tasks T "
               "left join bills B on B.task_id = T.id and B.current_id+1 > 2 "
               "left join people P on T.person_id = P.id "
               "left join person_names PN on P.person_names_id = PN.id "
               "left join procedures Px on Px.task_id = T.id and Px.current_id+1 > 2 "
               "left join insurance_coverage IC on B.insurance_coverage_id = IC.id "
               "left join insurance_policies IP on IC.insurance_policy_id = IP.id "
               "left join tax_entities TE on IP.tax_entity_id = TE.id "
               "left join month_types MT on MT.id = extract(month from T.date_of_task) "
               + join_type + " join insurance_coverage DenIns on DenIns.id = "
                 "(select first 1 IIC.id "
                  "from insurance_coverage IIC "
                  "inner join insurance_policies IIP on IIC.insurance_policy_id = IIP.id "
                  "inner join insurance_benefits IIB on IIB.insurance_policy_id = IIP.id "
                    "and IIB.area_id = " + lookupTables->IdOf("area_types", "Dental") + " and IIB.is_covered = 1 "
                  "where IIC.person_id = T.person_id "
                  "and (IIC.start_date <= T.date_of_task or IIC.start_date is null) "
                  "and (IIP.start_date <= T.date_of_task or IIP.start_date is null) "
                  "and (IIC.stop_date > T.date_of_task or IIC.stop_date is null) "
                  "and (IIP.stop_date > T.date_of_task or IIP.stop_date is null) "
                  "and IIC.current_id+1 > 2 and IIP.current_id+1 > 2) "
               "where T.current_id+1 > 3 and T.group_id = '" + lookupTables->IdOf("groups", "Dental Clinic") + "' "
               "and T.date_of_task between '" + begin_date + "' and '" + end_date + "' ";
  if (radioUninsured->Checked)
    sql += "and DenIns.id is null ";
  sql += "order by 1, 2 ";

  dbDataset->SelectSQL->Clear();
  dbDataset->SelectSQL->Add(sql.c_str());
  dbDataset->Open();
  dbDataset->First();

  report_form->billReport->DataSet = dbDataset;
  report_form->lblDateTitle->Caption = date_title.c_str();  // display dates on title band
  report_form->lblUser->Caption = current_fullname.c_str(); // display generating user's name on page footer

  // detail band controls
  report_form->txtDOS->DataSet = dbDataset;
  report_form->txtDOS->DataField = "DOS";
  report_form->txtName->DataSet = dbDataset;
  report_form->txtName->DataField = "person_name";
  report_form->txtInsured->DataSet = dbDataset;
  report_form->txtInsured->DataField = "insured";
  report_form->txtOnBill->DataSet = dbDataset;
  report_form->txtOnBill->DataField = "on_bill";
  report_form->txtPxCode->DataSet = dbDataset;
  report_form->txtPxCode->DataField = "cpt_hcpcs";
  report_form->exprCharges->Expression = "FORMATNUMERIC('$###,###,##0.00', charges)";
  report_form->txtInsurance->DataSet = dbDataset;
  report_form->txtInsurance->DataField = "insurance";

  // setup group header bands
  report_form->bandGroupHeader->Expression = "month_id";
  report_form->exprMonth->Expression = "month_name";

  // group footer controls
  report_form->exprChargesPerMonth->Expression = "FORMATNUMERIC('$###,###,##0.00', SUM(charges))";

  // summary controls
  report_form->exprTotalCharges->Expression = "FORMATNUMERIC('$###,###,##0.00', SUM(charges))";

  Processing(false);
  report_form->billReport->Preview();

  dbDataset->Close();

  return true;
}
//---------------------------------------------------------------------------
bool TformLaunchDentalBillingDataReport::AfterRunReport()
{
  if (dbDataset)
  {
    delete dbDataset;
    dbDataset = NULL;
  }
  return TformLaunchGeneric::AfterRunReport();
}
//---------------------------------------------------------------------------
