//---------------------------------------------------------------------------

#include <vcl.h>
#pragma hdrstop

#include "launchDepartmentContactsReport.h"
#include "reportDepartmentContacts.h"
#include "derivedheaders.h"
//---------------------------------------------------------------------------
#pragma package(smart_init)
#pragma link "codeSpecialGeneric"
#pragma resource "*.dfm"
TformLaunchDepartmentContactsReport *formLaunchDepartmentContactsReport;
//---------------------------------------------------------------------------
__fastcall TformLaunchDepartmentContactsReport::TformLaunchDepartmentContactsReport(TComponent* Owner)
        : TformSpecialGeneric(Owner)
{
  dateBegin->Date = Now();
  dateEnd->Date = Now();
  lookupTables->RegisterComboBox("method_types", cboMethod);
}
//---------------------------------------------------------------------------
void __fastcall TformLaunchDepartmentContactsReport::btnCloseClick(TObject *Sender)
{
  Close();        
}
//---------------------------------------------------------------------------
void __fastcall TformLaunchDepartmentContactsReport::btnRunClick(TObject *Sender)
{
  if((cboMethod->Text != "") && (! lookupTables->IsInLookUpTable("method_types", cboMethod->Text.c_str())))
  {
    PSay("Please enter a valid contact method or leave it blank", m_missing_data, MB_OK);
    cboMethod->SetFocus();
    return;
  }
  if(!checkDepartment->Checked && !checkTribe->Checked)
  {
    PSay("You must check at least one of the checkboxes for department and tribe", m_missing_data, MB_OK);
    return;
  }

  Processing(true);
  TIBTransaction * dbTransaction = new TIBTransaction(NULL);
  TIBDataSet * dbDataSet = new TIBDataSet(NULL);
  string sql;
  string begin_date = Pdate(dateBegin->Date).OutDate(mdy | yyyy);
  string end_date =  Pdate(dateEnd->Date).OutDate(mdy | yyyy);
  string date_title = begin_date + " - " + end_date;
  string group_fields, group_by_fields;
  if (checkDepartment->Checked)
  {
     group_fields += "groups.item department, ";
     group_by_fields += "groups.item, ";
  }
  if (checkTribe->Checked)
  {
     group_fields += "tribe_types.item tribe, ";
     group_by_fields += "tribe_types.item, ";
  }
  if (group_fields.size() > 0)
     group_fields = group_fields.substr(0, group_fields.size() - 2);
  if (group_by_fields.size() > 0)
     group_by_fields = group_by_fields.substr(0, group_by_fields.size() - 2);

  try
  {
    dbTransaction->DefaultDatabase = dbDatabase;
    dbDataSet->Database = dbDatabase;
    dbDataSet->Transaction = dbTransaction;
    dbDataSet->UniDirectional = false;
    string task_method = "";
    if(cboMethod->Text != "")
      task_method += "and tasks.method_id = " + lookupTables->IdOf("method_types", cboMethod->Text.c_str()) + " ";
    sql = "select " + group_fields + ", count(tasks.id) total_tasks, "
          "count(distinct tasks.person_id) distinct_people "
          "from tasks ";
    if(checkDepartment->Checked)
       sql += "inner join groups on tasks.group_id = groups.id ";
    if (checkTribe->Checked)
    {
       sql += "inner join people on tasks.person_id = people.id "
              "inner join affiliations on people.affiliations_id = affiliations.id "
              "inner join tribe_types on affiliations.tribe_id = tribe_types.id ";
    }
    sql += " where tasks.parent_task_id = 0 and tasks.person_id > 0 and tasks.current_id >= 3 "
           "and tasks.date_of_task between '" + begin_date + "' and '" + end_date + "' " + task_method;
    if(group_fields.size() > 0)
      sql += "group by " + group_by_fields;

    DebugSql(this, &sql, other_forms, 9);
    dbTransaction->StartTransaction();
    dbDataSet->SelectSQL->Add(sql.c_str());
    dbDataSet->Open();
    dbDataSet->First();

    FORM_TRY
      TformReportDepartmentContacts * report_form = new TformReportDepartmentContacts(NULL);
      if(cboMethod->Text != "")
        report_form->lblTitle->Caption = ReplaceStringWithString(report_form->lblTitle->Caption.c_str(), "Contact", cboMethod->Text.c_str()).c_str();
      report_form->report->DataSet = dbDataSet;
      report_form->lblDateTitle->Caption = date_title.c_str();
      report_form->lblUser->Caption = PrettyString(lookupTables->ValueOf("users", current_userid)).c_str();
      report_form->txtDetail->DataSet = dbDataSet;
      if (checkDepartment->Checked)
      {
         if (checkTribe->Checked)
         {
            report_form->txtDetail->DataField = "tribe";
            report_form->txtGroupHeading->DataField = "department";
            report_form->txtGroupFooting->DataField = "department";
         }
         else
         {
            report_form->txtDetail->DataField = "department";
            report_form->GroupHeaderBand->Enabled = false;
            report_form->GroupFooterBand->Enabled = false;
         }
      }
      else
      {
         report_form->txtDetail->DataField = "tribe";
         report_form->GroupHeaderBand->Enabled = false;
         report_form->GroupFooterBand->Enabled = false;
      }
      if (report_form->GroupHeaderBand->Enabled && report_form->GroupFooterBand->Enabled)
      {
         report_form->txtGroupHeading->DataSet = dbDataSet;
         report_form->txtGroupFooting->DataSet = dbDataSet;
         report_form->GroupHeaderBand->Expression = "department";
         report_form->exprGroupContacts->Expression = "SUM(total_tasks)";
         report_form->exprGroupPeople->Expression = "SUM(distinct_people)";
      }
      report_form->txtTasks->DataSet = dbDataSet;
      report_form->txtTasks->DataField = "total_tasks";
      report_form->txtPeople->DataSet = dbDataSet;
      report_form->txtPeople->DataField = "distinct_people";
      report_form->exprTotal->Expression = "SUM(total_tasks)";
      Processing(false);
      report_form->report->Preview();
    FORM_CATCH

    dbDataSet->Close();
    dbTransaction->Commit();
  }
  catch (...)
  {
    Processing(false);
    if (dbTransaction->Active)
      dbTransaction->Rollback();
    PSay("Error running report", m_database_error, MB_OK);
  }
  delete dbTransaction;
  delete dbDataSet;
}
//---------------------------------------------------------------------------
void __fastcall TformLaunchDepartmentContactsReport::FormShortCut(TWMKey &Msg, bool &Handled)
{
  Handled = true;
  if (Msg.CharCode == VK_RETURN)
    btnRun->Click();
  else
    Handled = false;

  if (!Handled)
    TformSpecialGeneric::FormShortCut(Msg, Handled);
}
//---------------------------------------------------------------------------
void __fastcall TformLaunchDepartmentContactsReport::FormClose(
      TObject *Sender, TCloseAction &Action)
{
  lookupTables->UnregisterComboBox("method_types", cboMethod);
  TformSpecialGeneric::FormClose(Sender, Action);
}
//---------------------------------------------------------------------------

