//---------------------------------------------------------------------------

#include <vcl.h>
#pragma hdrstop

#include "launchDiabApptReport.h"
#include "reportDiabAppt.h"
//---------------------------------------------------------------------------
#pragma package(smart_init)
#pragma link "launchGeneric"
#pragma resource "*.dfm"
TformLaunchDiabApptReport *formLaunchDiabApptReport;
//---------------------------------------------------------------------------
__fastcall TformLaunchDiabApptReport::TformLaunchDiabApptReport(TComponent* Owner)
        : TformLaunchGeneric(Owner)
{
  // Set default dates
  pair<Pdate, Pdate> last_month_dates = GetPriorMonthDateRange(Pdate(Date()));
  dateBegin->Date = last_month_dates.first.OutDate();
  dateEnd->Date   = last_month_dates.second.OutDate();
}
//---------------------------------------------------------------------------
bool TformLaunchDiabApptReport::IsValidOnScreen()
{
  if (Pdate(dateBegin->Date) > Pdate(dateEnd->Date))
  {
    PSay("Please select a begin date prior or equal to the end date", m_invalid_data, MB_OK);
    dateBegin->SetFocus();
    return false;
  }

  return TformLaunchGeneric::IsValidOnScreen();
}
//---------------------------------------------------------------------------
bool TformLaunchDiabApptReport::BeforeRunReport()
{
  dbDataset = AddDataSet();
  return TformLaunchGeneric::BeforeRunReport();
}
//---------------------------------------------------------------------------
bool TformLaunchDiabApptReport::RunReport()
{
  string sql;
  string begin_date = Pdate(dateBegin->Date).OutDate(mdy | yyyy);
  string end_date   = Pdate(dateEnd->Date).OutDate(mdy | yyyy);
  string date_title = begin_date + " - " + end_date;

  // Create the report form
  TformReportDiabAppt *report_form = NULL;
  FORM_TRY
    report_form = new TformReportDiabAppt(NULL);
  FORM_CATCH
  if(!report_form)
    return false;

  // Main query
  sql =
    "select case when AST.item = '' then 'Dental' else AST.item end service_type, "
    "(PN.first_name || ' ' || PN.middle_name || ' ' || PN.last_name || ' ' || PN.suffix) pname, "
    "groups.item group_name, ALT.item location, A.appointment_date, A.appointment_time, "
    "case when ART.item <> '' then ART.item else A.description end reason, "
    "VOT.item outcome "
    "from appointments A "
    "inner join appt_reason_types ART on A.appt_reason_id = ART.id "
    "inner join appt_service_types AST on ART.appt_service_id = AST.id "
    "inner join appt_location_types ALT on A.appt_location_id = ALT.id "
    "inner join visit_outcome_types VOT on A.visit_outcome_id = VOT.id "
    "inner join groups on A.group_id = groups.id "
    "inner join people P on A.person_id = P.id "
    "left  join person_names PN on P.person_names_id = PN.id "
    "where A.current_id+1 >= 3 and P.id <> 0 and "                                               // current, w/person
    "A.appt_reason_id <> " + lookupTables->IdOf("appt_reason_types", "Block Schedule") + " and " // 20110720: not a block
    "(A.appointment_date between '" + begin_date + "' and '" + end_date + "' or "                // within date range
    " A.begin_date between '"       + begin_date + "' and '" + end_date + "' or "
    " A.end_date between '"         + begin_date + "' and '" + end_date + "') and "
    "A.visit_outcome_id in ("
    + lookupTables->IdOf("visit_outcome_types", "As Scheduled") + ", "                           // Specified outcomes
    + lookupTables->IdOf("visit_outcome_types", "Walk-In"     ) + ", "
    + lookupTables->IdOf("visit_outcome_types", "Arrived Late") + ", "
    + lookupTables->IdOf("visit_outcome_types", "No Show"     ) + ") and "
    "exists (select first 1 COND.id from conditions COND where COND.person_id = P.id and "       // Diabetic condition
    " (COND.code = '250' or COND.code like '250.%') and COND.current_id >= 3 and "
    " COND.condition_id = '" + lookupTables->IdOf("condition_types", "CONDITION") + "') and "
    "((groups.id = '" + lookupTables->IdOf("groups", "Shawnee Clinic") + "' and AST.id = '"      // 1. Optometry -or-
       + lookupTables->IdOf("appt_service_types", "Optometry") + "') or "
    "  groups.id = '" + lookupTables->IdOf("groups", "Dental Clinic") + "' or "                  // 2. Dental -or-
    "  AST.id = '" + lookupTables->IdOf("appt_service_types", "Podiatry") + "') "                // 3. Podiatry
    "order by 1, 2, 3 ";

  InitDataSet(dbDataset, sql);
  report_form->reportAppts->DataSet = dbDataset;

  // Title band
  report_form->lblTitleDates->Caption = date_title.c_str();  // display dates on title band

  // Page footer
  report_form->lblUser->Caption = current_fullname.c_str(); // display generating user's name on page footer

  // Group header band - group by appt_service_type
  report_form->bandGroupHeader->Expression = "service_type";
  SetupQRDBText(report_form->txtGroupService, dbDataset, "service_type");

  // Detail band
  SetupQRDBText(report_form->txtName,       dbDataset, "pname");
  SetupQRDBText(report_form->txtDepartment, dbDataset, "group_name");
  SetupQRDBText(report_form->txtLocation,   dbDataset, "location");
  SetupQRDBText(report_form->txtDate,       dbDataset, "appointment_date");
  SetupQRDBText(report_form->txtTime,       dbDataset, "appointment_time");
  SetupQRDBText(report_form->txtReason,     dbDataset, "reason");
  SetupQRDBText(report_form->txtOutcome,    dbDataset, "outcome");

  // Group footer band - count of appointments in the current grouping
  report_form->exprNumAppts->Expression = "COUNT()";

  report_form->reportAppts->Preview();

  return true;
}
//---------------------------------------------------------------------------
