//---------------------------------------------------------------------------

#include <vcl.h>
#pragma hdrstop

#include "launchDiabClientReport.h"
#include "reportMailingLabels.h"
#include "reportMailingLabelsPrinted.h"
#include "reportDiabClient.h"
#include "codeSearchTaxEntity.h"
#include "launchMailingLabelsWithTasksReport.h"
#include "derivedheaders.h"
//---------------------------------------------------------------------------
#pragma package(smart_init)
#pragma link "codeSpecialGeneric"
#pragma link "codeBaseDropDown"
#pragma link "codeCheckListDropDown"
#pragma resource "*.dfm"
TformLaunchDiabClientReport *formLaunchDiabClientReport;
//---------------------------------------------------------------------------
__fastcall TformLaunchDiabClientReport::TformLaunchDiabClientReport(TComponent* Owner)
        : TformSpecialGeneric(Owner)
{
  dateShoesOrdered->Date = Now();
  dateTask->Date = Now();
  
  lookupTables->RegisterComboBox("tribe_types", cboTribe);
  lookupTables->RegisterComboBox("gender_types", cboGender);
  lookupTables->RegisterComboBox("month_types", cboMonth);

  if(optionsPermissions["table.conditions"]["view"] != "true")
  {
    checkDiagnosis->Enabled = false;
    panelDiagnosis->Enabled = false;
    checkNoDiagnosis->Enabled = false;
    panelNoDiagnosis->Enabled = false;
  }
  if(optionsPermissions["table.tasks"]["view"] != "true")
  {
    checkTasks->Enabled = false;
  }
  if(optionsPermissions["table.tasks"]["insert"] != "true")
  {
    checkCreateTask->Enabled = false;
    panelTask->Enabled = false;
  }
  if(optionsPermissions["table.tasks.diab"]["insert"] != "true")
    radioPreview->Enabled = false;
  else
  {
    if((optionsPermissions["special.diab.general"]["enabled"] == "true") &&
      (optionsPermissions["table.tasks.diab"]["insert"] == "true"))
    {
      lookupTables->RegisterComboBox("users", cboEmployee);
      lookupTables->RegisterComboBox("diab_activity_types", cboActivity);

      string event_program_sql = "select id, item, valid from program_types "
                                 "where (group_id = 0 or group_id = '" + lookupTables->IdOf("groups", "Diabetes") + "') "
                                 "and valid = 1 order by item ";
      lookupTables->RegisterComboBox("program_types", cboProgram, event_program_sql);
    }
    else if((optionsPermissions["special.beh.general"]["enabled"] == "true") &&
            (optionsPermissions["table.tasks.diab"]["insert"] == "true"))
    {
      lookupTables->RegisterComboBox("users", cboEmployee);
      lookupTables->RegisterComboBox("beh_activity_types", cboActivity);
    }
    else if((optionsPermissions["special.com.general"]["enabled"] == "true") &&
            (optionsPermissions["table.tasks.diab"]["insert"] == "true"))
    {
      lookupTables->RegisterComboBox("users", cboEmployee);
      lookupTables->RegisterComboBox("com_activity_types", cboActivity);
    }
    else
    {
      checkCreateTask->Enabled = false;
      panelTask->Enabled = false;
    }

  }
  if(optionsPermissions["table.participations"]["view"] != "true")
  {
    checkParticipations->Enabled = false;
  }
  if(optionsPermissions["table.obs_response_sets"]["view"] != "true")
  {
    checkMeasurements->Enabled = false;
  }
  if(optionsPermissions["table.appointments.people"]["view"] != "true")
  {
    checkAppointments->Enabled = false;
  }
  if(optionsPermissions["table.referrals"]["view"] != "true")
  {
    checkReferrals->Enabled = false;
  }
  if(optionsPermissions["table.affiliations"]["view"] != "true")
  {
    checkTribe->Enabled = false;
    cboTribe->Enabled = false;
  }
  if(optionsPermissions["table.incentives"]["view"] != "true")
  {
    checkShoes->Enabled = false;
    panelShoes->Enabled = false;
  }
  if(optionsPermissions["table.person_notes"]["view"] != "true")
  {
    checkCreateNote->Enabled = false;
    panelNote->Enabled = false;
  }
  if(optionsPermissions["table.people.diab"]["view"] != "true")
  {
    checkFitness->Enabled = false;
    checkPrimary->Enabled = false;
    checkASCDiabetes->Enabled = false;
    checkLACDiabetes->Enabled = false;
    checkNeitherDiabetes->Enabled = false;
    if(optionsPermissions["table.tax_entities"]["view"] != "true")
      checkProvider->Enabled = false;
  }
  if(optionsPermissions["table.people.beh"]["view"] != "true")
    checkAdmitted->Enabled = false;
  checkMyTasks->Enabled = false;
  checkMyParticipations->Enabled = false;
  checkMyMeasurements->Enabled = false;

  // jhaley 20101029: setup enrollments CBDD - warning: duplicated code w/codeSearchPerson
  string enrollment_sql =
    "select enrollment_types.item || (case when enrollment_status_types.id = 0 then '' else (' - ' || enrollment_status_types.item) end) item, "
    "enrollment_types.id * 1000 + coalesce(enrollment_status_types.id, 0) internal_id "           // hack alert!
    "from enrollment_types "
    "inner join permissions P on P.s = enrollment_types.permission_name and P.k = 'VIEW' and P.v = 'TRUE' and P.user_id = '" + current_userid + "' "
    "left join enrollment_status_types on (enrollment_types.id = enrollment_status_types.enrollment_id or "
    " (enrollment_types.allow_blank = 1 and enrollment_status_types.id = 0)) and enrollment_status_types.valid = 1 "
    "where enrollment_types.id > 0 and enrollment_types.valid = 1 "
    "order by enrollment_types.permission_name, enrollment_types.item, enrollment_status_types.item ";
  cbddEnrollments->Filter->SetSecurity("");
  cbddEnrollments->Filter->AddColumnSpec("Enrollment Type", "", as_text, true, 210, rawString);
  cbddEnrollments->Filter->AddColumnSpec("Internal Id", "internal_id", as_int, false, 60);
  cbddEnrollments->Filter->SetSQLStatement(enrollment_sql);
  cbddEnrollments->Filter->FetchPreferences("diabclientreport.enrollment_filter");
}
//---------------------------------------------------------------------------
void __fastcall TformLaunchDiabClientReport::btnCloseClick(TObject *Sender)
{
  Close();
}
//---------------------------------------------------------------------------
void __fastcall TformLaunchDiabClientReport::btnRunClick(TObject *Sender)
{
  if (!IsValidOnScreen())
    return;

  Screen->Cursor = crHourGlass;

  TIBTransaction * dbTransaction = new TIBTransaction(NULL);
  TIBDataSet * dbDataSet = new TIBDataSet(NULL);
  string sql;
  string date_title = "Client Report - " + CurrentDate();
  string where;

  try
  {
    StdTransaction(dbTransaction, dbDatabase);
    dbDataSet->Database = dbDatabase;
    dbDataSet->Transaction = dbTransaction;
    dbDataSet->UniDirectional = false;

    sql = "select people.id, person_contact_info.line1 || ' ' || "
          "person_contact_info.city || ' ' || state_types.abbreviation || ' ' || person_contact_info.zip address_line "
          "from people left join affiliations on affiliations.id = people.affiliations_id "
          "left join tribe_types on affiliations.tribe_id = tribe_types.id "
          "left join person_names on person_names.id = people.person_names_id "
          "left join person_contact_info on person_contact_info.id = people.person_contact_info_id "
          "left join state_types on person_contact_info.state_id = state_types.id "
          "where people.current_id >= 3 ";
    DebugSql(this, &sql, other_forms, 9);

    set<string> can_see_charts = GetSpecialGroupPermissions("charts");
    if (checkAges->Checked)
    {
      int age1 = StringToInt(editAge1->Text.c_str());
      age1 -= age1 * 2;
      int age2 = StringToInt(editAge2->Text.c_str());
      age2 -= age2 * 2;
      Pdate date1 = CurrentPdate();
      Pdate date2 = CurrentPdate();
      date1.mod(age1, in_years);
      date2.mod(age2, in_years);
      sql += "and people.birth_date <= '" + date1.OutDate(mdy | yyyy) + "' and "
             "people.birth_date >= '" + date2.OutDate(mdy | yyyy) + "' and "
             "people.birth_date is not null ";
    }
    if ((checkDiagnosis->Checked) || (checkNoDiagnosis->Checked))
    {
      where = "";
      if (optionsPermissions["table.conditions.common"]["view"] == "true")
        where += "conditions.group_id = 0 or ";
      if (optionsPermissions["table.conditions.diab"]["view"] == "true")
        where += "conditions.group_id = " + lookupTables->IdOf("groups", "Diabetes") + " or ";
      if (optionsPermissions["table.conditions.beh"]["view"] == "true")
        where += "conditions.group_id = " + lookupTables->IdOf("groups", "Behavioral Health") + " or ";
      if (optionsPermissions["table.conditions.com"]["view"] == "true")
        where += "conditions.group_id = " + lookupTables->IdOf("groups", "Community Health") + " or ";
      if (optionsPermissions["table.conditions.cob"]["view"] == "true")
        where += "conditions.group_id = " + lookupTables->IdOf("groups", "Contract Health") + " or ";
      if (optionsPermissions["table.conditions.lac"]["view"] == "true")
        where += "conditions.group_id = " + lookupTables->IdOf("groups", "Little Axe Clinic") + " or ";
      if (optionsPermissions["table.conditions.sc"]["view"] == "true")
        where += "conditions.group_id = " + lookupTables->IdOf("groups", "Shawnee Clinic") + " or ";
      if (where != "")
      {
        where = where.substr(0, where.length()-4);
        sql += "and ";
        if(checkDiagnosis->Checked)
          sql += "exists ";
        else
          sql += "not exists ";
        sql += "(select conditions.id from conditions where conditions.person_id = people.id and "
               "conditions.code ";
        if(checkExact->Checked)
          sql += "= '" + SafeSQLString(editDiagnosis->Text.c_str()) + "' ";
        else
          sql += "like '" + SafeSQLString(editDiagnosis->Text.c_str()) + "%' ";
        sql += "and conditions.current_id >= 3 and conditions.condition_id != " + lookupTables->IdOf("condition_types", "Relative") +
               "and (conditions.start_date is null or conditions.start_date <= current_date) "
               "and (conditions.stop_date is null or conditions.stop_date > current_date) "
               "and (" + where + ")) ";
      }
    }
    if (checkHave->Checked)
    {
      string begin_date = "";
      string end_date = "";
      if(editBeginDate->Text != "")
        begin_date = "'" + Pdate(editBeginDate->Text.c_str(), mdy | yy).OutDate(mdy | yyyy) + "'";
      if(editEndDate->Text != "")
        end_date = "'" + Pdate(editEndDate->Text.c_str(), mdy | yy).OutDate(mdy | yyyy) + "'";
      if((! checkAll->Checked) && (! checkNone->Checked))
        sql += " and (";

      if(checkTasks->Checked)
      {
        if(checkAll->Checked)
          sql += "and ";
        else if(checkNone->Checked)
          sql += "and not ";
        else
          sql += "(";
        sql += "exists (select tasks.id from tasks where person_id = people.id and tasks.current_id >= 3 ";
        if (begin_date != "")
          sql += " and tasks.date_of_task >= " + begin_date;
        if (end_date != "")
          sql += " and tasks.date_of_task <= " + end_date;
        if(checkMyTasks->Checked)
          sql += " and tasks.group_id = " + current_groupid;
        if(checkProvider->Checked)
          sql += " and tasks.tax_entity_id = '" + GetValidLinkId(txtProvider) + "' ";
        if((checkAll->Checked) || (checkNone->Checked))
          sql += ") ";
        else
          sql += ")) or ";
      }

      if(checkParticipations->Checked)
      {
        if((begin_date == "") && (end_date == ""))
        {
          if(checkAll->Checked)
            sql += "and ";
          else if(checkNone->Checked)
            sql += "and not ";
          else
            sql += "(";
          sql += "exists (select participations.id from participations where person_id = people.id and participations.current_id >= 3";
          if(checkMyParticipations->Checked)
            sql += " and participations.group_id = " + current_groupid;
          if((checkAll->Checked) || (checkNone->Checked))
            sql += ") ";
          else
            sql += ")) or ";
        }
        else
        {
          if(checkAll->Checked)
            sql += "and ";
          else if(checkNone->Checked)
            sql += "and not ";
          else
            sql += "(";
          sql += "exists (select participations.id from participations, events where participations.event_id = events.id and "
                 "person_id = people.id and participations.current_id >= 3 ";
          if (begin_date != "")
            sql += " and events.date_of_event >= " + begin_date;
          if (end_date != "")
            sql += " and events.date_of_event <= " + end_date;
          if(checkMyParticipations->Checked)
            sql += " and participations.group_id = " + current_groupid;
          if((checkAll->Checked) || (checkNone->Checked))
            sql += ") ";
          else
            sql += ")) or ";
        }
      }

      if(checkMeasurements->Checked)
      {
        if(checkAll->Checked)
          sql += "and ";
        else if(checkNone->Checked)
          sql += "and not ";
        else
          sql += "(";
        sql += "exists (select obs_response_sets.id from obs_response_sets where obs_response_sets.person_id = people.id and obs_response_sets.current_id >= 3";
        if (begin_date != "")
          sql += " and obs_response_sets.date_of_observation >= " + begin_date;
        if (end_date != "")
          sql += " and obs_response_sets.date_of_observation <= " + end_date;
        if(checkMyMeasurements->Checked)
          sql += " and obs_response_sets.group_id = " + current_groupid; // we normally assume they have permission to their own group, yes? maybe?
        if((checkAll->Checked) || (checkNone->Checked))
          sql += ") ";
        else
          sql += ")) or ";
      }

      if(checkAppointments->Checked)
      {
        if(checkAll->Checked)
          sql += "and ";
        else if(checkNone->Checked)
          sql += "and not ";
        else
          sql += "(";
        sql += "exists (select appointments.id from appointments where person_id = people.id and appointments.current_id >= 3 ";
        if((checkAll->Checked) || (checkNone->Checked))
          sql += ") ";
        else
          sql += ")) or ";
      }

      if(checkReferrals->Checked)
      {
        if(checkAll->Checked)
          sql += "and ";
        else if(checkNone->Checked)
          sql += "and not ";
        else
          sql += "(";
        sql += "exists (select referrals.id from referrals where person_id = people.id and referrals.current_id >= 3";
        if (begin_date != "")
          sql += " and referrals.date_of_referral >= " + begin_date;
        if (end_date != "")
          sql += " and referrals.date_of_referral <= " + end_date;
        if((checkAll->Checked) || (checkNone->Checked))
          sql += ") ";
        else
          sql += ")) or ";
      }
      if((! checkAll->Checked) && (! checkNone->Checked))
      {
        sql = sql.substr(0, sql.length()-4);
        sql += ") ";
      }
    }
    if(checkTribe->Checked)
      sql += "and affiliations.tribe_id = " + lookupTables->IdOf("tribe_types", cboTribe->Text.c_str()) + " ";
    if(checkDeceased->Checked)
      sql += "and people.death_date is null ";
    if(checkGender->Checked)
      sql += "and people.gender_id = " + lookupTables->IdOf("gender_types", cboGender->Text.c_str()) + " ";
    if(checkBirthday->Checked)
      sql += "and cast(people.birth_date as varchar(255)) like '%-" +
              twodigit(StringToInt(lookupTables->IdOf("month_types", cboMonth->Text.c_str()))) + "-%' ";
    if (checkFitness->Checked)
      sql += "and exists (select fitness_people.person_id from fitness_people, fitness_memberships where "
             "people.id = fitness_people.person_id and fitness_memberships.id = "
             "fitness_people.fitness_membership_id and "
             "fitness_memberships.end_date is null and fitness_people.current_id >= 3 and "
             "fitness_memberships.current_id >= 3) ";
    if (checkPrimary->Checked)
      sql += "and exists (select fitness_people.person_id from fitness_people, fitness_memberships where "
             "people.id = fitness_people.person_id and fitness_memberships.id = "
             "fitness_people.fitness_membership_id and "
             "fitness_memberships.end_date is null and fitness_people.current_id >= 3 and "
             "fitness_memberships.current_id >= 3 and fitness_people.is_primary = 1) ";
    if (checkASCDiabetes->Checked)
      sql += "and people.asc_diab_client = 1 ";
    if (checkLACDiabetes->Checked)
      sql += "and people.lac_diab_client = 1 ";
    if (checkNeitherDiabetes->Checked)
      sql += "and (people.lac_diab_client = 0 and people.asc_diab_client = 0) ";
    if (checkProvider->Checked && !checkTasks->Checked)
      sql += "and people.diab_provider_id = " + GetValidLinkId(txtProvider) + " ";
    if (checkAdmitted->Checked)
      sql += "and exists (select beh_treatment.person_id from beh_treatment "
              "where beh_treatment.person_id = people.id and "
              "beh_treatment.admit_date is not null and beh_treatment.discharge_date is null "
              "and beh_treatment.current_id >= 3) ";
    if (checkBlankName->Checked)
      sql += "and person_names.last_name is not null and person_names.first_name is not null ";
    if (checkBlankAddress->Checked)
      sql += "and person_contact_info.line1 is not null and person_contact_info.city is not null and "
               "(person_contact_info.state_id is not null or person_contact_info.state_id != 0) and "
               "person_contact_info.zip is not null ";
    if (checkZip->Checked)
      sql += "and person_contact_info.zip in (select local_zip_types.item from local_zip_types) ";
    if (checkShoes->Checked)
    {
      sql += "and exists (select incentives.person_id from incentives where incentives.date_ordered = '" + Pdate(dateShoesOrdered->Date).OutDate(mdy | yyyy) + "' "
             "and incentives.person_id = people.id and incentives.incentive_id = '" + lookupTables->IdOf("incentive_types", "Shoes") + "' "
             "and incentives.current_id >= 3) ";
    }

    // jhaley 20101101: Add enrollments - warning: duplicated code w/codeSearchPerson
    vector<map<string, string> > selected_vecmap = cbddEnrollments->Filter->GetAllData(true, true);
    if (selected_vecmap.size() > 0)
    {
      sql += "and exists (select ENROLL.id from enrollment ENROLL where ENROLL.person_id = people.id and ENROLL.current_id+1 >= 4 and (";

      // These must all be added as a single condition, only one of which must be true (OR logic)
      for(vector<map<string, string> >::iterator i_row = selected_vecmap.begin(); i_row != selected_vecmap.end(); i_row++)
      {
        int combo_ids = StringToInt((*i_row)["internal_id"]);
        string enrollment_type_id   = IntToString(combo_ids / 1000); // hack alert! - can I do this better? probably...
        string enrollment_status_id = IntToString(combo_ids % 1000);

        // If not the first item, add an "or".
        if (i_row != selected_vecmap.begin())
          sql += "or ";

        sql += "(ENROLL.enrollment_id = '" + enrollment_type_id + "' ";
        if (!IsBlankOrZero(enrollment_status_id))
          sql += "and ENROLL.enrollment_status_id = '" + enrollment_status_id + "' ";

        // close this predicate
        sql += ") ";
      }
      // Close the set of predicates and the 'exists' subquery
      sql += ")) ";
    }

    vector<map<string, string> > result_vec;
    map<string, string> address_map;
    list<string> address_list;
    DebugSql(this, &sql, other_forms, 9);
    SqlToVecMap(dbTransaction, sql, result_vec);
    for(vector<map<string, string> >::iterator itr = result_vec.begin(); itr != result_vec.end(); itr++)
    {
      //Application->MessageBox(FormatAddress((*itr)["address_line"]).c_str(), ("FormatAddress(" + (*itr)["address_line"] + ")").c_str(), MB_OK);
      if(checkOneAddress->Checked)
        address_map[FormatAddress((*itr)["address_line"])] = (*itr)["id"];
      else
        address_list.push_back((*itr)["id"]);
    }
    ExecuteStatement(dbTransaction, "delete from mailing_label_report");
    map<string, string> to_insert;
    if(checkOneAddress->Checked)
    {
      for(map<string, string>::iterator itr = address_map.begin(); itr != address_map.end(); itr++)
      {
        to_insert["id"] = itr->second;
        ExecuteInsertStatement(dbTransaction, "mailing_label_report", to_insert);
      }
    }
    else
    {
      for(list<string>::iterator itr = address_list.begin(); itr != address_list.end(); itr++)
      {
        to_insert["id"] = (*itr);
        ExecuteInsertStatement(dbTransaction, "mailing_label_report", to_insert);
      }
    }

    if(radioLabels->Checked)
    {
      string id_string;
      vector<string> id_vec;
      SqlToVec(dbTransaction, "select id from mailing_label_report", id_vec);
      id_string = VecToCommaString(id_vec);
      if(id_string != "")
      {
        sql = "select people.id from people left join person_contact_info on "
              "people.person_contact_info_id = person_contact_info.id "
              "where people.id in (" + id_string + ") and "
              "person_contact_info.current_id < 4";
        SqlToVec(dbTransaction, sql, id_vec);
        id_string = VecToCommaString(id_vec);
        if(id_string != "")
        {
          sql = "delete from mailing_label_report where id in (" + id_string + ")";
          ExecuteStatement(dbTransaction, sql);
        }
      }
      sql = "select person_names.first_name || ' ' || person_names.last_name || ' ' || person_names.suffix name, "
            "person_contact_info.line1, person_contact_info.line2, "
            "person_contact_info.city || ', ' || state_types.abbreviation || '  ' || person_contact_info.zip city "
            "from mailing_label_report, people "
            "left join person_contact_info on people.person_contact_info_id = "
            "person_contact_info.id left join state_types on person_contact_info.state_id = state_types.id "
            "left join person_names on people.person_names_id = person_names.id "
            "where mailing_label_report.id = people.id "
            "order by person_names.last_name, person_names.first_name";
      DebugSql(this, &sql, other_forms, 9);
      dbDataSet->SelectSQL->Add(sql.c_str());
      dbDataSet->Open();
      dbDataSet->First();

      FORM_TRY
        TformReportMailingLabels * labels = new TformReportMailingLabels(NULL);
        labels->labelReport->DataSet = dbDataSet;
        labels->txtName->DataSet = dbDataSet;
        labels->txtName->DataField = "name";
        labels->txtAddress1->DataSet = dbDataSet;
        labels->txtAddress1->DataField = "line1";
        Screen->Cursor = crDefault;
        labels->labelReport->Preview();
      FORM_CATCH


      dbDataSet->Close();
      dbTransaction->Commit();

      sql = "select person_names.last_name || ', ' || person_names.first_name || ' ' || person_names.suffix name "
            "from mailing_label_report, people "
            "left join person_names on people.person_names_id = person_names.id "
            "left join person_contact_info on people.person_contact_info_id = person_contact_info.id "
            "where mailing_label_report.id = people.id "
            "order by person_names.last_name, person_names.first_name";
      DebugSql(this, &sql, other_forms, 9);
      dbTransaction->StartTransaction();
      dbDataSet->SelectSQL->Clear();
      dbDataSet->SelectSQL->Add(sql.c_str());
      dbDataSet->Open();
      dbDataSet->First();

      FORM_TRY
        TformReportMailingLabelsPrinted * labelReport = new TformReportMailingLabelsPrinted(NULL);
        labelReport->lblDateTitle->Caption = date_title.c_str();
        labelReport->labelReport->DataSet = dbDataSet;
        labelReport->lblUser->Caption = PrettyString(lookupTables->ValueOf("users", current_userid)).c_str();
        labelReport->txtName->DataSet = dbDataSet;
        labelReport->txtName->DataField = "name";
        Screen->Cursor = crDefault;
        labelReport->labelReport->Preview();
      FORM_CATCH

      dbDataSet->Close();
      dbTransaction->Commit();
    }
    else if(radioReport->Checked)
    {
      sql = "select person_names.last_name || ', ' || person_names.first_name || ' ' || person_names.suffix name, "
            "people.social_security_number, people.birth_date, affiliations.cdib, "
            "tribe_types.short_item tribe, "
            "(select first 1 charts.chart_number from charts "
             "inner join groups on charts.location_id = groups.chart_location_id and groups.has_charts = 1 and charts.location_id > 0 "
             "where charts.person_id = people.id and " + SafeInClause("groups.id", SetToVec(can_see_charts)) + " "
              "and charts.current_id+1 > 3 order by groups.hl7_priority asc nulls last) chart_num, "
            "person_contact_info.line1 || '  ' || "
            "person_contact_info.city || ', ' || state_types.abbreviation || '  ' || "
            "person_contact_info.zip address, "
            "person_contact_info.phone_number "
            "from mailing_label_report, people left join affiliations on affiliations.id = people.affiliations_id "
            "left join tribe_types on affiliations.tribe_id = tribe_types.id "
            "left join person_names on person_names.id = people.person_names_id "
            "left join person_contact_info on person_contact_info.id = people.person_contact_info_id "
            "left join state_types on person_contact_info.state_id = state_types.id "
            "where mailing_label_report.id = people.id "
            "order by person_names.last_name, person_names.first_name"; 
      DebugSql(this, &sql, other_forms, 9);
      dbDataSet->SelectSQL->Add(sql.c_str());
      dbDataSet->Open();
      dbDataSet->First();

      FORM_TRY
        TformReportDiabClient * clientReport = new TformReportDiabClient(NULL);
        clientReport->clientReport->DataSet = dbDataSet;
        clientReport->lblTitle->Caption = date_title.c_str();
        clientReport->lblUser->Caption = PrettyString(lookupTables->ValueOf("users", current_userid)).c_str();
        clientReport->txtName->DataSet = dbDataSet;
        clientReport->txtName->DataField = "name";
        clientReport->txtSSN->DataSet = dbDataSet;
        clientReport->txtSSN->DataField = "social_security_number";
        clientReport->txtDOB->DataSet = dbDataSet;
        clientReport->txtDOB->DataField = "birth_date";
        clientReport->txtCDIB->DataSet = dbDataSet;
        clientReport->txtCDIB->DataField = "cdib";
        clientReport->txtTribe->DataSet = dbDataSet;
        clientReport->txtTribe->DataField = "tribe";
        clientReport->txtChart->DataSet = dbDataSet;
        clientReport->txtChart->DataField = "chart_num";
        clientReport->txtAddress->DataSet = dbDataSet;
        clientReport->txtAddress->DataField = "address";
        clientReport->txtPhone->DataSet = dbDataSet;
        clientReport->txtPhone->DataField = "phone_number";
        Screen->Cursor = crDefault;
        clientReport->clientReport->Preview();
      FORM_CATCH

      dbDataSet->Close();
    }
    else if (radioPreview->Checked)
    {
      sql = "select mailing_label_report.id from mailing_label_report";
      vector<string> id_vec;
      SqlToVec(dbTransaction, sql, id_vec);
      map<string, string> temp_map;
      if(checkCreateTask->Checked)
      {
        temp_map["user_id"] = lookupTables->IdOf("users", cboEmployee->Text.c_str());
        temp_map["date_of_task"] = Pdate(dateTask->Date).OutDate(mdy | yyyy);
        temp_map["diab_activity_id"] = lookupTables->IdOf("diab_activity_types", cboActivity->Text.c_str());
        temp_map["task_comments"] = editTaskNote->Text.c_str();
        temp_map["method_id"] = lookupTables->IdOf("method_types", "Mail");
        temp_map["direction_id"] = lookupTables->IdOf("direction_types", "Outgoing");
        temp_map["check_box"] = "true";
      }
      temp_map["group_id"] = lookupTables->IdOf("groups", "Diabetes");

      Screen->Cursor = crDefault;
      FORM_TRY
        TformLaunchMailingLabelsWithTasksReport *preview_form = new TformLaunchMailingLabelsWithTasksReport(NULL, temp_map, id_vec);
        preview_form->Show();
      FORM_CATCH
    }

    if((checkCreateTask->Checked) && (! radioPreview->Checked))
    {
      Screen->Cursor = crHourGlass;
      if (optionsPermissions["special.diab.general"]["enabled"] == "true")
      {
        sql = "insert into tasks (id, current_id, last_update_stamp, last_updated_by, "
              "event_id, group_id, date_of_task, task_comments, "
              "method_id, direction_id, classification_id, "
              "beh_meeting_id, beh_visit_id, beh_activity_id, visit_location_id, is_educational, "
              "is_personal, diab_activity_id, diab_contact_id, diab_meeting_id, "
              "task_purpose_id, program_id, user_id, visit_outcome_id, com_meeting_id, "
              "com_activity_id, com_visit_id, new_client, event_location_id, beh_appt_id, beh_service_id, "
              "com_dept_id, tax_entity_id, parent_task_id, services_provided_id, hospitalized, "
              "given_room, cob_id, care_classification_id, care_meeting_id, person_id) select"
              "(select gen_id(tasks_gen, 1) from uno), 4, '" + RightNow() + "', " + current_userid +
              ", " + "0, " + lookupTables->IdOf("groups", "Diabetes") + ", '" +
              Pdate(dateTask->Date).OutDate(mdy | yyyy) + "', "
              "'" + SafeSQLString(editTaskNote->Text.c_str()) + "', " +
              lookupTables->IdOf("method_types", "Mail") + ", " + lookupTables->IdOf("direction_types", "Outgoing") +
              ", 0, 0, 0, 0, 0, 0, 0, " + lookupTables->IdOf("diab_activity_types", cboActivity->Text.c_str()) +
              ", " + lookupTables->IdOf("diab_contact_types", "Contact") + ", 0, 0, " +
              lookupTables->IdOf("program_types", cboProgram) + ", " +
              lookupTables->IdOf("users", cboEmployee->Text.c_str()) + ", 0, 0, 0, 0, 0, 0, 0, 0, "
              "0, 0, 0, 0, 0, 0, 0, 0, 0, "
              "mailing_label_report.id from mailing_label_report";
      }
      else if (optionsPermissions["special.beh.general"]["enabled"] == "true")
      {
        sql = "insert into tasks (id, current_id, last_update_stamp, last_updated_by, "
              "event_id, group_id, date_of_task, task_comments, "
              "method_id, direction_id, classification_id, "
              "beh_meeting_id, beh_visit_id, beh_activity_id, visit_location_id, is_educational, "
              "is_personal, diab_activity_id, diab_contact_id, diab_meeting_id, "
              "task_purpose_id, program_id, user_id, visit_outcome_id, com_meeting_id, "
              "com_activity_id, com_visit_id, new_client, event_location_id, beh_appt_id, beh_service_id, "
              "com_dept_id, tax_entity_id, parent_task_id, services_provided_id, hospitalized, "
              "given_room, cob_id, care_classification_id, care_meeting_id, person_id) select"
              "(select gen_id(tasks_gen, 1) from uno), 4, '" + RightNow() + "', " + current_userid +
              ", " + "0, " + lookupTables->IdOf("groups", "Behavioral Health") + ", '" +
              Pdate(dateTask->Date).OutDate(mdy | yyyy) + "', "
              "'" + SafeSQLString(editTaskNote->Text.c_str()) + "', " +
              lookupTables->IdOf("method_types", "Mail") + ", " + lookupTables->IdOf("direction_types", "Outgoing") +
              ", 0, 0, 0, 0, 0, 0, 0, " + lookupTables->IdOf("beh_activity_types", cboActivity->Text.c_str()) +
              ", 0, 0, 0, 0, " +
              lookupTables->IdOf("users", cboEmployee->Text.c_str()) + ", 0, 0, 0, 0, 0, 0, 0, 0, "
              "0, 0, 0, 0, 0, 0, 0, 0, 0, "
              "mailing_label_report.id from mailing_label_report";
      }
      else if (optionsPermissions["special.com.general"]["enabled"] == "true")
      {
        sql = "insert into tasks (id, current_id, last_update_stamp, last_updated_by, "
              "event_id, group_id, date_of_task, task_comments, "
              "method_id, direction_id, classification_id, "
              "beh_meeting_id, beh_visit_id, beh_activity_id, visit_location_id, is_educational, "
              "is_personal, diab_activity_id, diab_contact_id, diab_meeting_id, "
              "task_purpose_id, program_id, user_id, visit_outcome_id, com_meeting_id, "
              "com_activity_id, com_visit_id, new_client, event_location_id, beh_appt_id, beh_service_id, "
              "com_dept_id, tax_entity_id, parent_task_id, services_provided_id, hospitalized, "
              "given_room, cob_id, care_classification_id, "
              "care_meeting_id, person_id) select"
              "(select gen_id(tasks_gen, 1) from uno), 4, '" + RightNow() + "', " + current_userid +
              ", " + "0, " + lookupTables->IdOf("groups", "Community Health") + ", '" +
              Pdate(dateTask->Date).OutDate(mdy | yyyy) + "', "
              "'" + SafeSQLString(editTaskNote->Text.c_str()) + "', " +
              lookupTables->IdOf("method_types", "Mail") + ", " + lookupTables->IdOf("direction_types", "Outgoing") +
              ", 0, 0, 0, 0, 0, 0, 0, " + lookupTables->IdOf("com_activity_types", cboActivity->Text.c_str()) +
              ", 0, 0, 0, 0, " +
              lookupTables->IdOf("users", cboEmployee->Text.c_str()) + ", 0, 0, 0, 0, 0, 0, 0, 0, "
              "0, 0, 0, 0, 0, 0, 0, 0, 0, "
              "mailing_label_report.id from mailing_label_report";
      }
      ExecuteStatement(dbTransaction, sql);
    } // end if create task checked
    if(checkCreateNote->Checked)
    {
      // jhaley 20101229: Minor adjustments - use the proper id generator, and provide a 0 warning_id for clarity/safety.
      sql = "insert into person_notes (id, current_id, last_update_stamp, last_updated_by, group_id, note, person_id, warning_id) "
            "select (select gen_id(person_notes_gen, 1) from uno), 4, '" + RightNow() + "', " + current_userid + ", "
            + lookupTables->IdOf("groups", current_groupid) + ", '" + SafeSQLString(editPersonNote->Text.c_str()) + "', "
            "mailing_label_report.id, 0 "
            "from mailing_label_report";
      ExecuteStatement(dbTransaction, sql);
    }
    sql = "delete from mailing_label_report";
    ExecuteStatement(dbTransaction, sql);
    Screen->Cursor = crDefault;
    if(dbTransaction->Active)
      dbTransaction->Commit();
  } // ent try
  catch (...)
  {
    Screen->Cursor = crDefault;
    PSay("Error running report", m_database_error, MB_OK);
    if (dbTransaction->Active)
      dbTransaction->Rollback();
  }
  delete dbTransaction;
  delete dbDataSet;
}
//---------------------------------------------------------------------------
void __fastcall TformLaunchDiabClientReport::FormClose(TObject *Sender, TCloseAction &Action)
{
  lookupTables->UnregisterComboBox("tribe_types", cboTribe);
  lookupTables->UnregisterComboBox("gender_types", cboGender);
  lookupTables->UnregisterComboBox("month_types", cboMonth);

  if((optionsPermissions["special.diab.general"]["enabled"] == "true") &&
     (optionsPermissions["table.tasks.diab"]["insert"] == "true"))
  {
    lookupTables->UnregisterComboBox("users", cboEmployee);
    lookupTables->UnregisterComboBox("diab_activity_types", cboActivity);
    lookupTables->UnregisterComboBox("program_types", cboProgram);
  }
  else if((optionsPermissions["special.beh.general"]["enabled"] == "true") &&
          (optionsPermissions["table.tasks.diab"]["insert"] == "true"))
  {
    lookupTables->UnregisterComboBox("users", cboEmployee);
    lookupTables->UnregisterComboBox("beh_activity_types", cboActivity);
  }
  else if((optionsPermissions["special.com.general"]["enabled"] == "true") &&
          (optionsPermissions["table.tasks.diab"]["insert"] == "true"))
  {
    lookupTables->UnregisterComboBox("users", cboEmployee);
    lookupTables->UnregisterComboBox("com_activity_types", cboActivity);
  }
  TformSpecialGeneric::FormClose(Sender, Action);
}
//---------------------------------------------------------------------------
void __fastcall TformLaunchDiabClientReport::checkAgesClick(TObject *Sender)
{
  if(checkAges->Checked)
    panelAges->Enabled = true;
  else
    panelAges->Enabled = false;
}
//---------------------------------------------------------------------------
void __fastcall TformLaunchDiabClientReport::checkDiagnosisClick(TObject *Sender)
{
  if(checkDiagnosis->Checked)
    panelDiagnosis->Enabled = true;
  else
    panelDiagnosis->Enabled = false;
}
//---------------------------------------------------------------------------
void __fastcall TformLaunchDiabClientReport::checkHaveClick(TObject *Sender)
{
  if(checkHave->Checked)
    panelHave->Enabled = true;
  else
    panelHave->Enabled = false;
}
//---------------------------------------------------------------------------
void __fastcall TformLaunchDiabClientReport::checkTribeClick(TObject *Sender)
{
  if(checkTribe->Checked)
    panelTribe->Enabled = true;
  else
    panelTribe->Enabled = false;
}
//---------------------------------------------------------------------------
void __fastcall TformLaunchDiabClientReport::checkGenderClick(TObject *Sender)
{
  if(checkGender->Checked)
    panelGender->Enabled = true;
  else
    panelGender->Enabled = false;
}
//---------------------------------------------------------------------------
void __fastcall TformLaunchDiabClientReport::checkTasksClick(TObject *Sender)
{
  if(checkTasks->Checked)
  {
    checkMyTasks->Enabled = true;
    checkProvider->Caption = "clients having visits with this provider:";
  }
  else
  {
    checkMyTasks->Enabled = false;
    checkProvider->Caption = "clients with a primary provider of:";
  }
}
//---------------------------------------------------------------------------
void __fastcall TformLaunchDiabClientReport::checkParticipationsClick(TObject *Sender)
{
  if(checkParticipations->Checked)
    checkMyParticipations->Enabled = true;
  else
    checkMyParticipations->Enabled = false;
}
//---------------------------------------------------------------------------
void __fastcall TformLaunchDiabClientReport::checkMeasurementsClick(TObject *Sender)
{
  if(checkMeasurements->Checked)
    checkMyMeasurements->Enabled = true;
  else
    checkMyMeasurements->Enabled = false;
}
//---------------------------------------------------------------------------
bool TformLaunchDiabClientReport::IsValidOnScreen()
{
  if(checkAges->Checked)
  {
    if((editAge1->Text == "") || (editAge2->Text == ""))
    {
      PSay("Please enter both ages", m_invalid_data, MB_OK);
      editAge1->SetFocus();
      return false;
    }
    else if(! IsInt(editAge1->Text.c_str()))
    {
      PSay("Please enter a valid minimum age", m_invalid_data, MB_OK);
      editAge1->SetFocus();
      return false;
    }
    else if(! IsInt(editAge2->Text.c_str()))
    {
      PSay("Please enter a valid maximum age", m_invalid_data, MB_OK);
      editAge2->SetFocus();
      return false;
    }
    else if(StringToInt(editAge1->Text.c_str()) > StringToInt(editAge1->Text.c_str()))
    {
      PSay("Please enter the smaller age in the first age box", m_invalid_data, MB_OK);
      editAge1->SetFocus();
      return false;
    }
  }
  if(checkDiagnosis->Checked)
  {
    if(editDiagnosis->Text == "")
    {
      PSay("Please enter a diagnosis code", m_missing_data, MB_OK);
      editDiagnosis->SetFocus();
      return false;
    }
  }
  if(checkNoDiagnosis->Checked)
  {
    if(editNoDiagnosis->Text == "")
    {
      PSay("Please enter a diagnosis code", m_missing_data, MB_OK);
      editNoDiagnosis->SetFocus();
      return false;
    }
  }
  if(checkHave->Checked)
  {
    if((checkAll->Checked) && (! ((checkTasks->Checked) || (checkParticipations->Checked) || (checkMeasurements->Checked) ||
       (checkAppointments->Checked) || (checkReferrals->Checked))))
    {
      PSay("Please check at least one box", m_missing_data, MB_OK);
      checkTasks->SetFocus();
      return false;
    }
    else if((checkNone->Checked) && (! ((checkTasks->Checked) || (checkParticipations->Checked) || (checkMeasurements->Checked) ||
            (checkAppointments->Checked) || (checkReferrals->Checked))))
    {
      PSay("Please check at least one box", m_missing_data, MB_OK);
      checkTasks->SetFocus();
      return false;
    }
    else if((checkAll->Checked) && (checkNone->Checked))
    {
      PSay("You can only check one of \"Must have all of these\" or \"Does not have any of these\"", m_invalid_data, MB_OK);
      checkAll->SetFocus();
      return false;
    }
    else if(! Pdate (editBeginDate->Text.c_str(), mdy | yy).IsValidDate() && ! (editBeginDate->Text == ""))
    {
      PSay("Please enter a valid begin date", m_invalid_data, MB_OK);
      editBeginDate->SetFocus();
      return false;
    }
    else if(! Pdate (editEndDate->Text.c_str(), mdy | yy).IsValidDate() && ! (editEndDate->Text == ""))
    {
      PSay("Please enter a valid end date", m_invalid_data, MB_OK);
      editEndDate->SetFocus();
      return false;
    }
  }
  if(checkTribe->Checked)
  {
    if (! lookupTables->IsInLookUpTable("tribe_types", cboTribe->Text.c_str()))
    {
      PSay("Please enter a valid tribe", m_invalid_data, MB_OK);
      cboTribe->SetFocus();
      return false;
    }
  }
  if(checkGender->Checked)
  {
    if (! lookupTables->IsInLookUpTable("gender_types", cboGender->Text.c_str()))
    {
      PSay("Please enter a valid gender", m_invalid_data, MB_OK);
      cboGender->SetFocus();
      return false;
    }
  }
  if(checkBirthday->Checked)
  {
    if (! lookupTables->IsInLookUpTable("month_types", cboMonth->Text.c_str()))
    {
      PSay("Please enter a valid month", m_invalid_data, MB_OK);
      cboMonth->SetFocus();
      return false;
    }
  }
  if(checkProvider->Checked)
  {
    if(GetValidLinkId(txtProvider) == "")
    {
      PSay("Please enter a provider", m_missing_data, MB_OK);
      btnProviderSearch->SetFocus();
      return false;
    }
  }
  if(checkCreateTask->Checked)
  {
    if(optionsPermissions["special.diab.general"]["enabled"] == "true")
    {
      if (! lookupTables->IsInLookUpTable("users", cboEmployee->Text.c_str()))
      {
        PSay("Please enter a valid employee", m_invalid_data, MB_OK);
        cboEmployee->SetFocus();
        return false;
      }
      else if(cboEmployee->Text == "")
      {
        PSay("Please enter an employee", m_missing_data, MB_OK);
        cboEmployee->SetFocus();
        return false;
      }
      else if (! lookupTables->IsInLookUpTable("program_types", cboProgram))
      {
        PSay("Please enter a valid program", m_invalid_data, MB_OK);
        cboProgram->SetFocus();
        return false;
      }
      else if (! lookupTables->IsInLookUpTable("cboActivity", cboActivity->Text.c_str()))
      {
        PSay("Please enter a valid activity type", m_invalid_data, MB_OK);
        cboActivity->SetFocus();
        return false;
      }
    }
    else if(optionsPermissions["special.beh.general"]["enabled"] == "true")
    {
      if (! lookupTables->IsInLookUpTable("users", cboEmployee->Text.c_str()))
      {
        PSay("Please enter a valid employee", m_invalid_data, MB_OK);
        cboEmployee->SetFocus();
        return false;
      }
      else if(cboEmployee->Text == "")
      {
        PSay("Please enter an employee", m_missing_data, MB_OK);
        cboEmployee->SetFocus();
        return false;
      }
      else if (! lookupTables->IsInLookUpTable("beh_activity_types", cboProgram->Text.c_str()))
      {
        PSay("Please enter a valid activity type", m_invalid_data, MB_OK);
        cboActivity->SetFocus();
        return false;
      }
    }
    else if(optionsPermissions["special.com.general"]["enabled"] == "true")
    {
      if (! lookupTables->IsInLookUpTable("users", cboEmployee->Text.c_str()))
      {
        PSay("Please enter a valid employee", m_invalid_data, MB_OK);
        cboEmployee->SetFocus();
        return false;
      }
      else if(cboEmployee->Text == "")
      {
        PSay("Please enter an employee", m_missing_data, MB_OK);
        cboEmployee->SetFocus();
        return false;
      }
      else if (! lookupTables->IsInLookUpTable("com_activity_types", cboProgram->Text.c_str()))
      {
        PSay("Please enter a valid activity type", m_invalid_data, MB_OK);
        cboActivity->SetFocus();
        return false;
      }
    }
  } // end if checkCreateTask->Checked
  if(checkCreateNote->Checked)
  {
    if(editPersonNote->Text == "")
    {
      PSay("Please enter a note", m_missing_data, MB_OK);
      editPersonNote->SetFocus();
      return false;
    }
  }

  return true;
}
//---------------------------------------------------------------------------
void __fastcall TformLaunchDiabClientReport::checkClientTypeClick(TObject *Sender)
{
  /*
  if(checkClientType->Checked)
    panelClientType->Enabled = true;
  else
    panelClientType->Enabled = false;
  */
}
//---------------------------------------------------------------------------
void __fastcall TformLaunchDiabClientReport::checkBirthdayClick(TObject *Sender)
{
  if(checkBirthday->Checked)
    panelBirthday->Enabled = true;
  else
    panelBirthday->Enabled = false;
}
//---------------------------------------------------------------------------
void __fastcall TformLaunchDiabClientReport::checkShoesClick(TObject *Sender)
{
  if(checkShoes->Checked)
    panelShoes->Enabled = true;
  else
    panelShoes->Enabled = false;
}
//---------------------------------------------------------------------------
void __fastcall TformLaunchDiabClientReport::checkCreateTaskClick(TObject *Sender)
{
  if(checkCreateTask->Checked)
    panelTask->Enabled = true;
  else
    panelTask->Enabled = false;
}
//---------------------------------------------------------------------------
void __fastcall TformLaunchDiabClientReport::checkCreateNoteClick(TObject *Sender)
{
  if(checkCreateNote->Checked)
    panelNote->Enabled = true;
  else
    panelNote->Enabled = false;
}
//---------------------------------------------------------------------------
void __fastcall TformLaunchDiabClientReport::FormShortCut(TWMKey &Msg,bool &Handled)
{
  Handled = true;
  if (Msg.CharCode == VK_RETURN)
    btnRun->Click();
  else
    Handled = false;

  if (!Handled)
    TformSpecialGeneric::FormShortCut(Msg, Handled);
}
//---------------------------------------------------------------------------
void __fastcall TformLaunchDiabClientReport::checkNoDiagnosisClick(TObject *Sender)
{
  if(checkNoDiagnosis->Checked)
    panelNoDiagnosis->Enabled = true;
  else
    panelNoDiagnosis->Enabled = false;
}
//---------------------------------------------------------------------------
void __fastcall TformLaunchDiabClientReport::btnProviderSearchClick(TObject *Sender)
{
  // create and show new person search form
  vector<string> return_vector;
  FORM_TRY
    TformSearchTaxEntity * providerSearch = new TformSearchTaxEntity(NULL,&return_vector);
    providerSearch->ShowTaskModal(this);
  FORM_CATCH
  if (! return_vector.empty())
    CreateLink(txtProvider, btnProviderSearch, this, return_vector[0], "tax_entities");
}
//---------------------------------------------------------------------------
void __fastcall TformLaunchDiabClientReport::checkProviderClick(TObject *Sender)
{
  if(checkProvider->Checked)
    panelProvider->Enabled = true;
  else
    panelProvider->Enabled = false;
}
//---------------------------------------------------------------------------
// jhaley 20101029: OnDown handler for CBDD - JIT-refresh for the filter, to avoid access violations otherwise.
void __fastcall TformLaunchDiabClientReport::cbddEnrollmentsDown(TObject *Sender)
{
  cbddEnrollments->Filter->ForceRefreshDataSet();
  cbddEnrollments->DisplayFieldName = "Enrollment Type";
}
//---------------------------------------------------------------------------

