// I need to make the grouping better, now that the group title shows up on the report.  Specifically,
// I want to left join on the condition that lower_bound is null and age/improvement <= upper_bound,
// lower_bound < age/improvement <= upper_bound, or upper_bound is null and age/improvement > lower_bound

//---------------------------------------------------------------------------
#include <vcl.h>
#pragma hdrstop

#include "derivedheaders.h"
#include "launchDiabMeasurementComparisonReport.h"
#include "reportDiabMeasurementComparison.h"
//---------------------------------------------------------------------------
#pragma package(smart_init)
#pragma link "codeSpecialGeneric"
#pragma resource "*.dfm"
TformLaunchDiabMeasurementComparisonReport *formLaunchDiabMeasurementComparisonReport;
//---------------------------------------------------------------------------
__fastcall TformLaunchDiabMeasurementComparisonReport::TformLaunchDiabMeasurementComparisonReport(TComponent* Owner)
        : TformSpecialGeneric(Owner)
{
   string obs_type_sql = "select obs_questions.id, obs_questions.item, obs_questions.valid from obs_questions "
                         "where improvement_id > 0 "
                         "and obs_questions.obs_question_id = '" + lookupTables->IdOf("obs_question_types", "Metric") + "' "
                         "order by obs_questions.item ";
   lookupTables->RegisterComboBox("obs_questions", cboType, obs_type_sql);

   string group_perms = SetToCommaString(GetSpecialGroupPermissions("general"));
   string group_sql = "select id, item, valid from event_name_types "
                      "where group_id in (" + group_perms + ") and valid = 1 "
                      "order by item ";
   lookupTables->RegisterComboBox("event_name_types", cboEventName, group_sql, SetGroupIdForNewLookupTableItem);

   // Build SQL for event types
   string event_type_sql = "select id, item, valid from event_type_types where (group_id = 0 or ";

   // Public Health should be able to view COM event types, as they are allowed to enter them
   if (optionsPermissions["table.measurements.pub"]["view"] == "true")
     event_type_sql += "group_id = '" + lookupTables->IdOf("groups", "Community Health") + "' or ";

   event_type_sql += "group_id in (" + group_perms + ")) and valid = 1 order by item ";

   lookupTables->RegisterComboBox("event_type_types", cboEventType, event_type_sql);

   CriteriaChange(NULL);
   Pdate temp_date = Pdate(Now());
   editDateEnd2->Text = temp_date.OutDate(mdy|yyyy).c_str();
   temp_date.day = 1;
   editDateBegin2->Text = temp_date.OutDate(mdy|yyyy).c_str();
   temp_date.day = LastDayOf(temp_date.month, temp_date.year);
   editDateEnd1->Text = temp_date.OutDate(mdy|yyyy).c_str();
   temp_date.month = 1;
   temp_date.day = 1;
   editDateBegin1->Text = temp_date.OutDate(mdy|yyyy).c_str();

   vector<string> groups;   // group_id's of all the measurements the user has permission to see
   if(optionsPermissions["table.measurements.com"]["view"] == "true")
     groups.push_back(lookupTables->IdOf("groups", "Community Health"));
   if(optionsPermissions["table.measurements.common"]["view"] == "true")
     groups.push_back("0");
   if(optionsPermissions["table.measurements.diab"]["view"] == "true")
     groups.push_back(lookupTables->IdOf("groups", "Diabetes"));
   if(optionsPermissions["table.measurements.lac"]["view"] == "true")
     groups.push_back(lookupTables->IdOf("groups", "Little Axe Clinic"));
   if(optionsPermissions["table.measurements.pub"]["view"] == "true")
     groups.push_back(lookupTables->IdOf("groups", "Public Health"));

   /* TODO -c HACK: internal_id is added as the first column because below, I need the actual
      group id's for each checked item, but that function returns a vector of -the values in the first
      column- for each checked row.  How stupid!  Filter should be fixed.  :-D */
   string department_sql = "select id, item from groups where id in (" + VecToCommaString(groups) + ");";
   department_filter.SetListViewHandle(listDepartments);
   department_filter.SetSecurity("");
   department_filter.AddColumnSpec("Internal Id", "internal_id", as_text, false, 75);
   department_filter.AddColumnSpec("Department", "", as_text, true, 180);
   //department_filter.FetchPreferences("");
   department_filter.SetCheck(true);
   department_filter.SetIconStatus(false);
   department_filter.SetSQLStatement(department_sql);
   department_filter.RefreshDataSet();
   department_filter.CheckAll();
}
//---------------------------------------------------------------------------
void __fastcall TformLaunchDiabMeasurementComparisonReport::btnRunClick(TObject *Sender)
{
 if (IsValidOnScreen())
 {
  Processing(true);

  //string begin_date = Pdate(dateBegin1->Date).OutDate(mdy | yyyy);
  //string end_date =  Pdate(dateEnd1->Date).OutDate(mdy | yyyy);
  //string date_title = begin_date + " - " + end_date;

  TIBTransaction * dbTransaction = new TIBTransaction(NULL);
  TIBDataSet * dbDataSet = new TIBDataSet(NULL);

  progress_container z;

  try
  {
    // Set up transaction for use of temp tables
    dbTransaction->DefaultDatabase = dbDatabase;
    dbTransaction->StartTransaction();

    // Some flags to make things easier down below
    bool c1_take_meas_num = radioMeasurementNumber1->Checked;
    bool c2_take_meas_num = radioMeasurementNumber2->Checked;
    bool c1_take_min_date = radioNoMeasurementNumber1->Checked && radioFirstMeasurement1->Checked;
    bool c2_take_min_date = radioNoMeasurementNumber2->Checked && radioFirstMeasurement2->Checked;
    bool c1_take_max_date = radioNoMeasurementNumber1->Checked && radioLastMeasurement1->Checked;
    bool c2_take_max_date = radioNoMeasurementNumber2->Checked && radioLastMeasurement2->Checked;
    bool c1_take_all_date = radioNoMeasurementNumber1->Checked && radioAllMeasurements1->Checked;
    bool c2_take_all_date = radioNoMeasurementNumber2->Checked && radioAllMeasurements2->Checked;
    bool c1_use_date_range = checkDateRange1->Checked;
    bool c2_use_date_range = checkDateRange2->Checked;
    bool c1_use_begin_date = c1_use_date_range && editDateBegin1->Text != "";
    bool c1_use_end_date   = c1_use_date_range && editDateEnd1->Text != "";
    bool c2_use_begin_date = c2_use_date_range && editDateBegin2->Text != "";
    bool c2_use_end_date   = c2_use_date_range && editDateEnd2->Text != "";
    bool do_ages           = checkAges->Checked && editAges->Text != "";
    bool do_improvement    = checkImprovement->Checked && editImprovement->Text != "";
    bool do_grouping       = do_ages || do_improvement;

    //string session_id = current_groupid + "~" + computer_name + "~" + ReplaceStringWithString(RightNow(), " ", "~");
    //PSay("Your session id is '" + session_id + "'", m_debug, MB_OK);
    string session_id = GetNextId(dbTransaction, "temp_measure_comp");

    string obs_question_id = lookupTables->IdOf("obs_questions", cboType);
    string today = DateToStr(Date()).c_str();

    string lower_better = GetOneField(dbTransaction, "select improvement_id from obs_questions where id = " + obs_question_id);
    if (lower_better != lookupTables->IdOf("improvement_types", "Lower is better"))
     lower_better = "0";


    // TODO: deal with the following --
    // - diab_activity_types now has mixed-case entries
    // - these entries were renamed from "Session x" to "STAR Session x"
    string min_session_id, max_session_id;
    if (checkSessionsMax->Checked)
    {
      max_session_id = GetOneField(dbTransaction, "select id from diab_activity_types where upper(item) = 'STAR SESSION " + string(editSessionMax->Text.c_str()) + "'");
      if (max_session_id == "")
      {
        PSay("Invalid max STAR session number", m_message, MB_OK);
        Processing(false);
        return;
      }
    }
    if (checkSessionsMin->Checked)
    {
      min_session_id = GetOneField(dbTransaction, "select id from diab_activity_types where upper(item) = 'STAR SESSION " + string(editSessionMin->Text.c_str()) + "'");
      if (min_session_id == "")
      {
        PSay("Invalid min STAR session number", m_message, MB_OK);
        Processing(false);
        return;
      }
    }

    // Run the SQL to fill the temp tables the report will be using
    string sql;

/*    sql = "delete from temp_measurements";
    ExecuteStatement(dbTransaction, sql);
    sql = "delete from temp_measurement_comparison1";
    ExecuteStatement(dbTransaction, sql);
    sql = "delete from temp_measurement_comparison2";
    ExecuteStatement(dbTransaction, sql);
*/
    if (do_grouping) // dynamically create the grouping brackets in temp_measurement_groups table
    {                // note: expects user to enter upper bound except for last group (it already
                     // knows the correct lower and upper bound for ages, %improvement, etc.)
/*       sql = "delete from temp_measurement_groups";
       ExecuteStatement(dbTransaction, sql);
*/
       vector<string> temp_vec;
       map<string, string> insert_map;
       string lower_bound;
       if(do_ages)
       {
          lower_bound = "";
          temp_vec.clear();
          SplitStringToVec(editAges->Text.c_str(), " ", temp_vec);
          for(vector<string>::iterator itr = temp_vec.begin(); itr != temp_vec.end(); itr++)
          {
             insert_map.clear();
             insert_map["session"] = session_id;
             insert_map["lower_bound"] = lower_bound;
             insert_map["upper_bound"] = (*itr);
             insert_map["group_type"] = "AGES";
             ExecuteInsertStatement(dbTransaction, "temp_measurement_groups", insert_map);
             if(IsInt(*itr))
               lower_bound = (*itr);
             else
             {
               PSay((*itr) + " is not a valid integer", m_invalid_data, MB_OK);
               editAges->SetFocus();
               throw(1);
             }
          }
          insert_map.clear();
          insert_map["session"] = session_id;
          insert_map["lower_bound"] = lower_bound;
          insert_map["upper_bound"] = "";
          insert_map["group_type"] = "AGES";
          ExecuteInsertStatement(dbTransaction, "temp_measurement_groups", insert_map);
       }
       if(do_improvement)
       {
          lower_bound = "";
          temp_vec.clear();
          SplitStringToVec(editImprovement->Text.c_str(), " ", temp_vec);
          for(vector<string>::iterator itr = temp_vec.begin(); itr != temp_vec.end(); itr++)
          {
             insert_map.clear();
             insert_map["session"] = session_id;
             insert_map["lower_bound"] = lower_bound;
             insert_map["upper_bound"] = (*itr);
             insert_map["group_type"] = "IMPROVEMENT";
             ExecuteInsertStatement(dbTransaction, "temp_measurement_groups", insert_map);
             if(IsInt(*itr))
               lower_bound = (*itr);
             else
             {
               PSay((*itr) + " is not a valid integer", m_invalid_data, MB_OK);
               editAges->SetFocus();
               throw(1);
             }
          }
          insert_map.clear();
          insert_map["session"] = session_id;
          insert_map["lower_bound"] = lower_bound;
          insert_map["upper_bound"] = "";
          insert_map["group_type"] = "IMPROVEMENT";
          ExecuteInsertStatement(dbTransaction, "temp_measurement_groups", insert_map);
       }
    }

    vector<string> groups = department_filter.GetMultiID();
    string person_restrictions;
    if (checkDiabClients->Checked)
    {
       // jhaley: changed diab_client_id -> enrollment
       person_restrictions += " exists (select DCE.id from enrollment DCE "
           "where DCE.person_id = people.id and DCE.current_id+1 >= 4 and "
           "DCE.enrollment_id = '" + lookupTables->IdOf("enrollment_types", "Diabetes Client") + "') and";
    }
    if (checkStepsParticipants->Checked)
    {
       // jhaley: changed steps_member -> enrollment
       person_restrictions += " exists (select SE.id from enrollment SE "
           "where SE.person_id = people.id and SE.current_id+1 >= 4 and "
           "SE.enrollment_id = '" + lookupTables->IdOf("enrollment_types", "STEPS Participant") + "' and "
           "SE.enrollment_status_id = '" + lookupTables->IdOf("enrollment_status_types", "Member") + "') and";
    }
    if (checkStarParticipants->Checked)
    {
       // jhaley: changed star_participant -> enrollment
       person_restrictions += " exists (select SPE.id from enrollment SPE "
           "where SPE.person_id = people.id and SPE.current_id+1 >= 4 and "
           "SPE.enrollment_id = '" + lookupTables->IdOf("enrollment_types", "STAR Participant") + "') and";
    }
    if (checkFitnessMembers->Checked)
       person_restrictions += " exists (select * from fitness_people inner join fitness_memberships on fitness_people.fitness_membership_id = fitness_memberships.id "
                              "where fitness_people.person_id = people.id and fitness_people.current_id+1>=4 and fitness_memberships.is_waiting = 0) and";
    if (checkCVD->Checked)
    {
       // jhaley: changed cvd_participant -> enrollment
       person_restrictions += " exists (select CVDE.id from enrollment CVDE "
           "where CVDE.person_id = people.id and CVDE.current_id+1 >= 4 and "
           "CVDE.enrollment_id = '" + lookupTables->IdOf("enrollment_types", "Healthy Heart Participant") + "') and";
    }
    if (checkDiabetic->Checked)
       person_restrictions += " exists (select * from conditions C "
                              "where C.person_id = people.id and C.current_id+1>=4 and (C.code = '250' or C.code like '250.%') and C.condition_id = 5) and";
    if (person_restrictions.length() > 0)
       person_restrictions = person_restrictions.substr(0, person_restrictions.length() - 3);
/*
    map<int, int> star_session_ids;
    vector<string> session_sql_vec1, session_sql_vec2;
    if (checkSessions1->Checked || checkSessions2->Checked)
    {
      vector<map<string, string> > sessions_vec;
      string session_sql = "select id, item from diab_activity_types where item like 'SESSION _'";
      SqlToVecMap(dbTransaction, session_sql, sessions_vec);
      for (unsigned int i = 0; i < sessions_vec.size(); i++)
        star_session_ids[StringToInt(sessions_vec[i]["item"].substr(sessions_vec[i]["item"].length()-1))] = StringToInt(sessions_vec[i]["id"]);
      for (map<int, int>::iterator itr = star_session_ids.begin(); itr != star_session_ids.end(); itr++)
      {
        if (checkSessions1->Checked)
        {
          if (itr->first <= StringToInt(editSessionMin1->Text.c_str()))
             session_sql_vec1.push_back("exists (select tasks.id from tasks where tasks.person_id = people.id and tasks.diab_activity_id = " + IntToString(itr->second) + ")");
          if (itr->first > StringToInt(editSessionMax1->Text.c_str()))
             session_sql_vec1.push_back("not exists (select tasks.id from tasks where tasks.person_id = people.id and tasks.diab_activity_id = " + IntToString(itr->second) + ")");
        }
        if (checkSessions2->Checked)
        {
          if (itr->first <= StringToInt(editSessionMin2->Text.c_str()))
             session_sql_vec2.push_back("exists (select tasks.id from tasks where tasks.person_id = people.id and tasks.diab_activity_id = " + IntToString(itr->second) + ")");
          if (itr->first > StringToInt(editSessionMax2->Text.c_str()))
             session_sql_vec2.push_back("not exists (select tasks.id from tasks where tasks.person_id = people.id and tasks.diab_activity_id = " + IntToString(itr->second) + ")");
        }
      }
    }
*/

// Put subset of measurements table that we want to look at in a temp table (note: the get_measurement() stored procedure uses this table)
    sql = "insert into temp_measurements "
          "   select " + session_id + ", person_id, obs_responses.response_metric, date_of_observation, participations_id "
          "   from obs_response_sets "
          "   inner join obs_responses on obs_responses.obs_response_set_id = obs_response_sets.id and obs_responses.obs_question_id = " + obs_question_id + " "
          "   where obs_response_sets.current_id > 2 and obs_response_sets.group_id in (" + VecToCommaString(groups) + ") ";
    if (checkStepsMeasurements->Checked)
       sql += "   and is_steps_observation = 1 ";
    // have to be careful with the date restriction... if just criteria 1 or criteria 2 date range is being used then skip; however, if
    // both are going to be used, we can construct a ((date_range1) or (date_range2)) sort of restriction and speed up subsequent queries
    if((c1_use_begin_date || c1_use_end_date) && (c2_use_begin_date || c2_use_end_date))
    {
       sql += " and ((";
       if(c1_use_begin_date)
          sql += "   date_of_observation >= '" + string(editDateBegin1->Text.c_str()) + "' and";
       if(c1_use_end_date)
          sql += "   date_of_observation <= '" + string(editDateEnd1->Text.c_str()) + "' and";

       sql = sql.substr(0, sql.size() - 3);
       sql += ") or (";
       if(c2_use_begin_date)
          sql += "   date_of_observation >= '" + string(editDateBegin2->Text.c_str()) + "' and";
       if(c2_use_end_date)
          sql += "   date_of_observation <= '" + string(editDateEnd2->Text.c_str()) + "' and";
       sql = sql.substr(0, sql.size() - 3);
       sql += " ))";
    }
    DebugSql(this, &sql, other_forms, 9, dbTransaction);
    z.Mark("Start: total measurements");
    ExecuteStatement(dbTransaction, sql);
    z.Mark("Stop: total measurements");

    int current_statement1 = 1, current_statement2 = 1;

// Fetch the person_id, birth_date, full name, and dates of measurement for each criteria group
    sql =  "insert into temp_measurement_comparison1 "
           "   select " + session_id + ", M.person_id, birth_date, last_name || ', ' || first_name || ' ' || middle_name || ' ' || suffix name, date_of_measurement, null, " + IntToString(current_statement1) + " ";
    sql += "   from people inner join temp_measurements M on M.person_id = people.id and M.session = " + session_id + " ";
    if(c1_take_meas_num && editMeasurementNumber1->Text != "")
       sql += "        and M.date_of_measurement = (select param_measurement_date from get_measurement(person_id, " + string(editMeasurementNumber1->Text.c_str()) + ")) ";
    if(c1_use_begin_date)
       sql += "        and date_of_measurement >= '" + string(editDateBegin1->Text.c_str()) + "' ";
    if(c1_use_end_date)
       sql += "        and date_of_measurement <= '" + string(editDateEnd1->Text.c_str()) + "' ";
    sql += "        inner join person_names on people.person_names_id = person_names.id ";
    if(cboEventName->Text != "" || cboEventType->Text != "")
    {
       sql += "     inner join participations on M.participation_id = participations.id "
              "     inner join events on participations.event_id = events.id ";
       if (cboEventName->Text != "")
         sql += "and events.event_name_id = '" + lookupTables->IdOf("event_name_types", cboEventName) + "' ";
       if (cboEventType->Text != "")
         sql += "and events.event_type_id = '" + lookupTables->IdOf("event_type_types", cboEventType) + "' ";
    }
    if (person_restrictions.length() > 0 || checkSessionsMax->Checked)
    {
       sql += "   where ";
       if (person_restrictions.length() > 0)
         sql += person_restrictions + " and ";
       if (checkSessionsMax->Checked)
         sql += "exists (select max(tasks.date_of_task) from tasks where tasks.person_id = M.person_id and tasks.diab_activity_id = " + max_session_id + ") and ";
       sql = sql.substr(0, sql.length() - 4);
    }
    DebugSql(this, &sql, other_forms, 9, dbTransaction);
    z.Mark("Start: appropriate dates of measurement [comparison 1]");
    ExecuteStatement(dbTransaction, sql);
    z.Mark("Stop: appropriate dates of measurement [comparison 1]");

    current_statement1++;

    sql =  "insert into temp_measurement_comparison2 "
           "   select " + session_id + ", M.person_id, birth_date, last_name || ', ' || first_name || ' ' || middle_name || ' ' || suffix name, date_of_measurement, null, " + IntToString(current_statement2) + " ";
    sql += "   from people inner join temp_measurements M on M.person_id = people.id and M.session = " + session_id + " ";
    if(c2_take_meas_num && editMeasurementNumber2->Text != "")
       sql += "        and M.date_of_measurement = (select param_measurement_date from get_measurement(person_id, " + string(editMeasurementNumber1->Text.c_str()) + ")) ";
    if(c2_use_begin_date)
       sql += "        and date_of_measurement >= '" + string(editDateBegin2->Text.c_str()) + "' ";
    if(c2_use_end_date)
       sql += "        and date_of_measurement <= '" + string(editDateEnd2->Text.c_str()) + "' ";
    sql += "        inner join person_names on people.person_names_id = person_names.id ";
    if (person_restrictions.length() > 0 || checkSessionsMin->Checked)
    {
       sql += "   where ";
       if (person_restrictions.length() > 0)
         sql += person_restrictions + " and ";
       if (checkSessionsMin->Checked)
         sql += "exists (select min(tasks.date_of_task) from tasks where tasks.person_id = M.person_id and tasks.diab_activity_id = " + min_session_id + ") and ";
       sql = sql.substr(0, sql.length() - 4);
    }
    DebugSql(this, &sql, other_forms, 9, dbTransaction);
    z.Mark("Start: appropriate dates of measurement [comparison 2]");
    ExecuteStatement(dbTransaction, sql);
    z.Mark("Stop: appropriate dates of measurement [comparison 2]");

    current_statement2++;

// If restricting by STAR sessions, then do so now
    string get_max_session_date, get_min_session_date;

    if(checkSessionsMax->Checked || checkSessionsMin->Checked)
    {
/*      sql = "delete from temp_measurement_sessions";
      ExecuteStatement(dbTransaction, sql);
*/
    }

    if(checkSessionsMax->Checked)
    {
      get_max_session_date = "insert into temp_measurement_sessions select " + session_id + ", M.person_id, max(date_of_task), 1 "
                                                                   " from tasks inner join temp_measurement_comparison1 M on tasks.person_id = M.person_id and M.session = " + session_id + " "
                                                                      " and M.current_statement = 1 and tasks.diab_activity_id = " + max_session_id;
      if(c1_use_begin_date)
         get_max_session_date += (" and M.measurement_date >= '" + string(editDateBegin1->Text.c_str()) + "' ");
      if(c1_use_end_date)
         get_max_session_date += (" and M.measurement_date <= '" + string(editDateEnd1->Text.c_str()) + "' ");
      get_max_session_date += " group by M.person_id ";
      DebugSql(this, &get_max_session_date, other_forms, 9, dbTransaction);
      ExecuteStatement(dbTransaction, get_max_session_date);

      sql =  "insert into temp_measurement_comparison1 "
             "   select " + session_id + ", M.person_id, M.birth_date, M.name, M.measurement_date, M.measurement, " + IntToString(current_statement1) + " ";
      sql += "   from temp_measurement_comparison1 M inner join temp_measurement_sessions S on M.person_id = S.person_id and M.session = " + session_id + " and S.session = " + session_id + " and S.current_statement = 1 where M.measurement_date <= S.session_date ";
      DebugSql(this, &sql, other_forms, 9, dbTransaction);
      z.Mark("Start: restrict by STAR [comparison 1]");
      ExecuteStatement(dbTransaction, sql);
      z.Mark("Stop: restrict by STAR [comparison 1]");
      current_statement1++;
    }

    if(checkSessionsMin->Checked)
    {
      get_min_session_date = "insert into temp_measurement_sessions select " + session_id + ", M.person_id, min(date_of_task), 2 "
                                                                   " from tasks inner join temp_measurement_comparison2 M on tasks.person_id = M.person_id and M.session = " + session_id + " "
                                                                      " and M.current_statement = 1 and tasks.diab_activity_id = " + min_session_id;
      if(c2_use_begin_date)
         get_min_session_date += (" and M.measurement_date >= '" + string(editDateBegin2->Text.c_str()) + "' ");
      if(c2_use_end_date)
         get_min_session_date += (" and M.measurement_date <= '" + string(editDateEnd2->Text.c_str()) + "' ");
      get_min_session_date += " group by M.person_id ";
      DebugSql(this, &get_min_session_date, other_forms, 9, dbTransaction);
      ExecuteStatement(dbTransaction, get_min_session_date);

      sql =  "insert into temp_measurement_comparison2 "
             "   select " + session_id + ", M.person_id, M.birth_date, M.name, M.measurement_date, M.measurement, " + IntToString(current_statement2) + " ";
      sql += "   from temp_measurement_comparison2 M inner join temp_measurement_sessions S on M.person_id = S.person_id and M.session = " + session_id + " and S.session = " + session_id + " and S.current_statement = 2 where M.measurement_date >= S.session_date ";
      DebugSql(this, &sql, other_forms, 9, dbTransaction);
      z.Mark("Start: restrict by STAR [comparison 2]");
      ExecuteStatement(dbTransaction, sql);
      z.Mark("Stop: restrict by STAR [comparison 2]");
      current_statement2++;
    }

    // Apply any necessary min/max etc.
    if(c1_take_min_date || c1_take_max_date)
    {
      sql = "insert into temp_measurement_comparison1 "
            "   select " + session_id + ", T.person_id, T.birth_date, T.name, ";
      if(c1_take_min_date)
         sql += "min(T.measurement_date), ";
      else if(c1_take_max_date)
         sql += "max(T.measurement_date), ";
      else
         sql += "T.measurement_date, ";
      sql += ("null, " + IntToString(current_statement1) + " from temp_measurement_comparison1 T where current_statement = " + IntToString(current_statement1-1) + " and T.session = " + session_id + " "
             "and exists (select * from obs_response_sets M inner join obs_responses N on N.obs_response_set_id = M.id and N.obs_question_id = " + obs_question_id + " where M.person_id = T.person_id and M.date_of_observation = T.measurement_date) "
             "group by T.person_id, T.birth_date, T.name ");
      DebugSql(this, &sql, other_forms, 9, dbTransaction);
      z.Mark("Start: restrict by min/max date [comparison 1]");
      ExecuteStatement(dbTransaction, sql);
      z.Mark("Stop: restrict by min/max date [comparison 2]");
      current_statement1++;
    }

    if(c2_take_min_date || c2_take_max_date)
    {
      sql = "insert into temp_measurement_comparison2 "
            "   select " + session_id + ", T.person_id, T.birth_date, T.name, ";
      if(c2_take_min_date)
         sql += "min(T.measurement_date), ";
      else if(c2_take_max_date)
         sql += "max(T.measurement_date), ";
      else
         sql += "T.measurement_date, ";
      sql += ("null, " + IntToString(current_statement2) + " from temp_measurement_comparison2 T where current_statement = " + IntToString(current_statement2-1) + " and T.session = " + session_id + " "
             "and exists (select * from obs_response_sets M inner join obs_responses N on N.obs_response_set_id = M.id and N.obs_question_id = " + obs_question_id + " where M.person_id = T.person_id and M.date_of_observation = T.measurement_date) "
             "group by T.person_id, T.birth_date, T.name ");
      DebugSql(this, &sql, other_forms, 9, dbTransaction);
      z.Mark("Start: restrict by min/max date [comparison 2]");
      ExecuteStatement(dbTransaction, sql);
      z.Mark("Stop: restrict by min/max date [comparison 2]");
      current_statement2++;
    }

    // Now fetch the average measurement for each date... almost done!

    sql =  "insert into temp_measurement_comparison1 "
           "   select distinct " + session_id + ", T.person_id, T.birth_date, T.name, T.measurement_date, ";
    if(c1_take_min_date || c1_take_max_date)
       sql += " avg(N.response_metric)";
    else
       sql += " N.response_metric";
    sql += (", " + IntToString(current_statement1) + " "
           "   from temp_measurement_comparison1 T "
           "   inner join obs_response_sets M on T.person_id = M.person_id and T.measurement_date = M.date_of_observation and T.session = " + session_id +
           "   inner join obs_responses N on N.obs_response_set_id = M.id and N.obs_question_id = " + obs_question_id +
           "   where current_statement = " + IntToString(current_statement1-1) + " ");
    if(c1_take_min_date || c1_take_max_date)
       sql += " group by T.person_id, T.birth_date, T.name, T.measurement_date ";
    DebugSql(this, &sql, other_forms, 9, dbTransaction);
    z.Mark("Start: get final set [comparison 1]");
    ExecuteStatement(dbTransaction, sql);
    z.Mark("Stop: get final set [comparison 1]");

    sql = "insert into temp_measurement_comparison2 "
          "   select distinct " + session_id + ", T.person_id, T.birth_date, T.name, T.measurement_date, ";
    if(c2_take_min_date || c2_take_max_date)
       sql += " avg(N.response_metric)";
    else
       sql += " N.response_metric";
    sql += (", " + IntToString(current_statement2) + " "
           "   from temp_measurement_comparison2 T "
           "   inner join obs_response_sets M on T.person_id = M.person_id and T.measurement_date = M.date_of_observation and T.session = " + session_id +
           "   inner join obs_responses N on N.obs_response_set_id = M.id and N.obs_question_id = " + obs_question_id +
           "   where current_statement = " + IntToString(current_statement2-1) + " ");
    if(c2_take_min_date || c2_take_max_date)
       sql += " group by T.person_id, T.birth_date, T.name, T.measurement_date ";
    DebugSql(this, &sql, other_forms, 9, dbTransaction);
    z.Mark("Start: get final set [comparison 1]");
    ExecuteStatement(dbTransaction, sql);
    z.Mark("Stop: get final set [comparison 2]");

    // last statements inserted into temp tables.  Don't increment counters and pull current_statements (not minus 1)

    // Create the report
    TformReportDiabMeasurementComparison * mcReport;
    FORM_TRY
      mcReport = new TformReportDiabMeasurementComparison(NULL);
    FORM_CATCH

    mcReport->lblInfoTitle->Caption = "";

    dbDataSet->Database = dbDatabase;
    dbDataSet->Transaction = dbTransaction;
    dbDataSet->UniDirectional = false;

    string get_improvement = "(select p_improvement from get_improvement(A.measurement, B.measurement, " + lower_better + "))";
    sql =  "select A.name name, A.birth_date dob, A.measurement_date date1, A.measurement measurement1, "
           "     B.measurement_date date2, B.measurement measurement2, A.person_id person_id, " + get_improvement + " percent_improvement ";

    // grouping expression/title stuff
    string group_title = "";
    string group_expr = "";
    if (do_ages)
    {

      group_title = "'Ages ' || (select result from get_range_text(ages.lower_bound, ages.upper_bound)) ";
      group_expr = "ages.id ";
    }
    if (do_improvement)
    {
      if(group_title != "") group_title += "|| ', ' || ";
      group_title += "(select result from get_range_text(improvement.lower_bound, improvement.upper_bound)) || '% improvement ' ";
      if(group_expr != "") group_expr += "|| ', ' || ";
      group_expr += "improvement.id ";
    }
    // repeat as necessary for additional groups following model of do_improvement
    if (do_ages || do_improvement /*other groups must go here too!*/ )
    {
      sql += (", " + group_title + " group_title ");
      sql += (", " + group_expr + " group_expr ");
    }
    // end of grouping expression/title stuff

    sql += "from temp_measurement_comparison1 A inner join temp_measurement_comparison2 B "
           "        on A.person_id = B.person_id and A.current_statement = " + IntToString(current_statement1) + " and B.current_statement = " + IntToString(current_statement2) + " "
           "           and A.session = " + session_id + " and B.session = " + session_id + " ";
    //if(c1_take_min_date || c1_take_max_date || c2_take_min_date || c2_take_max_date)
    sql += "        and A.measurement_date < B.measurement_date ";
    /* TODO -c HACK: Yes, this is unnecessary duplication; should only call stored procedure once and use BETWEEN, but doing so causes the
       database to disconnect everyone who is connected, and casting doesn't help either.  Very odd glitch. */
    if (do_ages)
       sql += "     left join temp_measurement_groups ages on ages.session = " + session_id + " and "
                   "(ages.lower_bound is null or (select p_age from calc_age(A.birth_date)) >= ages.lower_bound) and "
                   "(ages.upper_bound is null or (select p_age from calc_age(A.birth_date)) < ages.upper_bound) and ages.group_type = 'AGES' ";
    if (do_improvement)
       sql += "     left join temp_measurement_groups improvement on improvement.session = " + session_id + " and "
                   "(improvement.lower_bound is null or " + get_improvement + " >= improvement.lower_bound) and "
                   "(improvement.upper_bound is null or " + get_improvement + " < improvement.upper_bound) and improvement.group_type = 'IMPROVEMENT' ";
    sql += "order by ";
    if (do_ages)
       sql += "ages.id, ";
    if (do_improvement)
       sql += "improvement.id, ";
    sql += "A.name ";

    z.Mark("Start: final join");

    DebugSql(this, &sql, other_forms, 9, dbTransaction);
    dbDataSet->SelectSQL->Add(sql.c_str());
    dbDataSet->Open();
    dbDataSet->First();

    mcReport->report->DataSet = dbDataSet;

    mcReport->lblUser->Caption = PrettyString(lookupTables->ValueOf("users", current_userid)).c_str();

    if(c1_take_meas_num)
       mcReport->lblMeasurement1->Caption = "Measurement " + editMeasurementNumber1->Text;
    else if(c1_take_min_date)
       mcReport->lblMeasurement1->Caption = "First measurement";
    else if(c1_take_max_date)
       mcReport->lblMeasurement1->Caption = "Last measurement";
    else if(c1_take_all_date)
       mcReport->lblMeasurement1->Caption = "All measurements";

    if(c2_take_meas_num)
       mcReport->lblMeasurement2->Caption = "Measurement " + editMeasurementNumber2->Text;
    else if(c2_take_min_date)
       mcReport->lblMeasurement2->Caption = "First measurement";
    else if(c2_take_max_date)
       mcReport->lblMeasurement2->Caption = "Last measurement";
    else if(c2_take_all_date)
       mcReport->lblMeasurement2->Caption = "All measurements";

    if(c1_use_date_range)
       mcReport->lblDateRange1->Caption = FormatDateLabel(editDateBegin1->Text.c_str(), editDateEnd1->Text.c_str()).c_str();
    else
       mcReport->lblDateRange1->Caption = FormatDateLabel("", "").c_str();
    if(c2_use_date_range)
       mcReport->lblDateRange2->Caption = FormatDateLabel(editDateBegin2->Text.c_str(), editDateEnd2->Text.c_str()).c_str();
    else
       mcReport->lblDateRange2->Caption = FormatDateLabel("", "").c_str();

    mcReport->lblMType1->Caption = cboType->Text;
    mcReport->lblMType2->Caption = cboType->Text;
    mcReport->txtName->DataSet = dbDataSet;
    mcReport->txtName->DataField = "name";
    mcReport->txtDOB->DataSet = dbDataSet;
    mcReport->txtDOB->DataField = "dob";
    mcReport->txtDate1->DataSet = dbDataSet;
    mcReport->txtDate1->DataField = "date1";
    mcReport->txtMType1->DataSet = dbDataSet;
    mcReport->txtMType1->DataField = "measurement1";
    mcReport->txtDate2->DataSet = dbDataSet;
    mcReport->txtDate2->DataField = "date2";
    mcReport->txtMType2->DataSet = dbDataSet;
    mcReport->txtMType2->DataField = "measurement2";
    mcReport->txtImprovement->DataSet = dbDataSet;
    mcReport->txtImprovement->DataField = "percent_improvement";
    mcReport->txtGroupTitle->DataSet = dbDataSet;
    mcReport->txtGroupTitle->DataField = "group_title";
    mcReport->txtGroupExpr->DataSet = dbDataSet;
    mcReport->txtGroupExpr->DataField = "group_expr";

    //if(lower_better=="1")
    //   mcReport->exprImprovement->Expression = "FORMATNUMERIC('##0.00', (measurement2 - measurement1) * (-100) / measurement1)";
    //else if(lower_better=="0")
    //   mcReport->exprImprovement->Expression = "FORMATNUMERIC('##0.00', (measurement2 - measurement1) * 100 / measurement1)";

    if (do_grouping)
    {
       mcReport->GroupFooterBand->Enabled = true;
/*       string group_expression = "";
       if (do_ages)
          group_expression += "STR(age_group)";
       if (do_ages && do_improvement)
          group_expression += " - ";
       if (do_improvement)
          group_expression += "STR(improvement_group)";
*/
       mcReport->GroupHeaderBand->Expression = "group_expr";
    }

    string sm1, sm2;
    sm1 = "select avg(cast(measurement as decimal(12,6))) from temp_measurement_comparison1 T1 "
          "where T1.session = " + session_id + " and measurement != '0' and measurement is not null and "
          "   exists (select * from temp_measurement_comparison2 T2 "
          "           where T2.session = " + session_id + " and T1.person_id = T2.person_id "
          "              and T1.measurement_date != T2.measurement_date and measurement != '0' and measurement is not null);";
    sm2 = "select avg(cast(measurement as decimal(12,6))) from temp_measurement_comparison2 T2 "
          "where T2.session = " + session_id + " and measurement != '0' and measurement is not null and "
          "   exists (select * from temp_measurement_comparison1 T1 "
          "           where T1.session = " + session_id + " and T1.person_id = T2.person_id "
          "              and T1.measurement_date != T2.measurement_date and measurement != '0' and measurement is not null);";
    float m1 = StringToFloat(GetOneField(dbTransaction, sm1));
    float m2 = StringToFloat(GetOneField(dbTransaction, sm2));
    float p_improve;
    if (m1 != 0)
       p_improve = (m2 - m1) * 100 / m1;

    mcReport->lblSummaryAvgMType1->Caption = FloatToString(m1).c_str();
    mcReport->lblSummaryAvgMType2->Caption = FloatToString(m2).c_str();
    if (m1 == 0)
      mcReport->lblSummaryAvgImprovement->Caption = "div by 0";
    else if(lower_better=="1")
      mcReport->lblSummaryAvgImprovement->Caption = FloatToString(-1 * p_improve).c_str();
    else if(lower_better=="0")
      mcReport->lblSummaryAvgImprovement->Caption = FloatToString(p_improve).c_str();

    mcReport->exprAvgMType1->Expression = "FORMATNUMERIC('#0.00', AVERAGE(measurement1))";
    mcReport->exprAvgMType2->Expression = "FORMATNUMERIC('#0.00', AVERAGE(measurement2))";
//    if(lower_better=="1")
//       mcReport->exprAvgImprovement->Expression = "FORMATNUMERIC('##0.00', AVERAGE(measurement2 - measurement1) * (-100) / measurement1))";
//    else if(lower_better=="0")
//       mcReport->exprAvgImprovement->Expression = "FORMATNUMERIC('##0.00', AVERAGE(measurement2 - measurement1) * 100 / measurement1))";
//    if(lower_better=="1")
//       mcReport->exprAvgImprovement->Expression = "FORMATNUMERIC('##0.00', (AVERAGE(measurement2) - AVERAGE(measurement1)) * (-100) / AVERAGE(measurement1))";
//    else if(lower_better=="0")
//       mcReport->exprAvgImprovement->Expression = "FORMATNUMERIC('##0.00', (AVERAGE(measurement2) - AVERAGE(measurement1)) * 100 / AVERAGE(measurement1))";
    if(lower_better=="1")
       mcReport->exprAvgImprovement->Expression = "FORMATNUMERIC('##0.00', (AVERAGE(measurement2) - AVERAGE(measurement1)) * (-100) / AVERAGE(measurement1))";
    else if(lower_better=="0")
       mcReport->exprAvgImprovement->Expression = "FORMATNUMERIC('##0.00', (AVERAGE(measurement2) - AVERAGE(measurement1)) * 100 / AVERAGE(measurement1))";

    mcReport->exprSummaryAvgMType1->Expression = "FORMATNUMERIC('#0.00', AVERAGE(measurement1))";
    mcReport->exprSummaryAvgMType2->Expression = "FORMATNUMERIC('#0.00', AVERAGE(measurement2))";
    if(lower_better=="1")
       mcReport->exprSummaryAvgImprovement->Expression = "FORMATNUMERIC('##0.00', (AVERAGE(measurement2) - AVERAGE(measurement1)) * (-100) / AVERAGE(measurement1))";
    else if(lower_better=="0")
       mcReport->exprSummaryAvgImprovement->Expression = "FORMATNUMERIC('##0.00', (AVERAGE(measurement2) - AVERAGE(measurement1)) * 100 / AVERAGE(measurement1))";

    /*
    mcReport->txtLineItem->DataSet = dbDataSet;
    bsReport->txtLineItem->DataField = "line_item";

    bsReport->exprBudgetAmount->Expression = "FORMATNUMERIC('$###,###,##0.00', budget_amount)";
    bsReport->exprAmountSpent->Expression = "FORMATNUMERIC('$###,###,##0.00', amount_spent_to_date)";
    bsReport->exprAmountRemaining->Expression = "FORMATNUMERIC('$###,###,##0.00', budget_amount - amount_spent_to_date)";
    bsReport->exprPercentSpent->Expression = "FORMATNUMERIC('##0.00', amount_spent_to_date / budget_amount * 100)";
    */

    Processing(false);
    mcReport->report->Preview();
    z.Mark("Stop: final join");

    ViewSql(z.Report());

    sql = "delete from temp_measurements where session = " + session_id;
    ExecuteStatement(dbTransaction, sql);
    sql = "delete from temp_measurement_comparison1 where session = " + session_id;
    ExecuteStatement(dbTransaction, sql);
    sql = "delete from temp_measurement_comparison2 where session = " + session_id;
    ExecuteStatement(dbTransaction, sql);
    sql = "delete from temp_measurement_groups where session = " + session_id;
    ExecuteStatement(dbTransaction, sql);
    sql = "delete from temp_measurement_sessions where session = " + session_id;
    ExecuteStatement(dbTransaction, sql);

    dbTransaction->Commit();
  }
  catch (...)
  {
    Processing(false);
    PSay("Error running report", m_database_error, MB_OK);
    if (dbTransaction->Active)
      dbTransaction->Rollback();
  }

  delete dbTransaction;
  delete dbDataSet;
 } // end if(IsValidOnScreen())
}
//---------------------------------------------------------------------------
bool TformLaunchDiabMeasurementComparisonReport::IsValidOnScreen()
{
  // Some validation in your eye!
  if (cboType->Text == "" || !lookupTables->IsInLookUpTable("obs_questions", cboType))
  {
    PSay("Please select a type of measurement to compare", m_invalid_data, MB_OK);
    cboType->SetFocus();
    return false;
  }
  if (!lookupTables->IsInLookUpTable("event_name_types", cboEventName))
  {
    PSay("Please select a valid event name", m_invalid_data, MB_OK);
    cboEventName->SetFocus();
    return false;
  }
  if (!lookupTables->IsInLookUpTable("event_type_types", cboEventType))
  {
    PSay("Please select a valid event type", m_invalid_data, MB_OK);
    cboEventType->SetFocus();
    return false;
  }
  else if (checkDateRange1->Checked && IsInvalidDate(editDateBegin1))
  {
    PSay("Please enter a valid criteria 1 begin date or leave it blank", m_invalid_data, MB_OK);
    editDateBegin1->SetFocus();
    return false;
  }
  else if (checkDateRange1->Checked && IsInvalidDate(editDateEnd1))
  {
    PSay("Please enter a valid criteria 1 end date or leave it blank", m_invalid_data, MB_OK);
    editDateEnd1->SetFocus();
    return false;
  }
  else if (checkDateRange2->Checked && IsInvalidDate(editDateBegin2))
  {
    PSay("Please enter a valid criteria 2 begin date or leave it blank", m_invalid_data, MB_OK);
    editDateBegin2->SetFocus();
    return false;
  }
  else if (checkDateRange2->Checked && IsInvalidDate(editDateEnd2))
  {
    PSay("Please enter a valid criteria 2 end date or leave it blank", m_invalid_data, MB_OK);
    editDateEnd2->SetFocus();
    return false;
  }
  else if (radioMeasurementNumber1->Checked &&
     (editMeasurementNumber1->Text=="" || !IsInt(editMeasurementNumber1->Text.c_str())))
  {
    PSay("Please enter a valid criteria 1 measurement number", m_invalid_data, MB_OK);
    editMeasurementNumber1->SetFocus();
    return false;
  }
  else if (radioMeasurementNumber2->Checked &&
     (editMeasurementNumber2->Text=="" || !IsInt(editMeasurementNumber2->Text.c_str())))
  {
    PSay("Please enter a valid criteria 2 measurement number", m_invalid_data, MB_OK);
    editMeasurementNumber2->SetFocus();
    return false;
  }
  vector<string> groups = department_filter.GetMultiID();
  if(groups.empty())
  {
    PSay("No departments are selected.  You must choose the departments whose measurements you wish to include.", m_invalid_data, MB_OK);
    return false;
  }

  return true;
}
//---------------------------------------------------------------------------
void __fastcall TformLaunchDiabMeasurementComparisonReport::btnCloseClick(TObject *Sender)
{
  Close();
}
//---------------------------------------------------------------------------
void __fastcall TformLaunchDiabMeasurementComparisonReport::FormShortCut(
      TWMKey &Msg, bool &Handled)
{
  Handled = true;
  if (Msg.CharCode == VK_RETURN)
    btnRun->Click();
  else
    Handled = false;

  if (!Handled)
    TformSpecialGeneric::FormShortCut(Msg, Handled);
}
//---------------------------------------------------------------------------
void __fastcall TformLaunchDiabMeasurementComparisonReport::CriteriaChange(TObject *Sender)
{
 panelMeasurementNumber1->Enabled = radioMeasurementNumber1->Checked;
 panelNoMeasurementNumber1->Enabled = radioNoMeasurementNumber1->Checked;
 panelDateRange1->Enabled = checkDateRange1->Checked;

 panelMeasurementNumber2->Enabled = radioMeasurementNumber2->Checked;
 panelNoMeasurementNumber2->Enabled = radioNoMeasurementNumber2->Checked;
 panelDateRange2->Enabled = checkDateRange2->Checked;
}
//---------------------------------------------------------------------------
void __fastcall TformLaunchDiabMeasurementComparisonReport::FormClose(
      TObject *Sender, TCloseAction &Action)
{
  lookupTables->UnregisterComboBox(cboType);
  lookupTables->UnregisterComboBox("event_name_types", cboEventName);
  lookupTables->UnregisterComboBox("event_type_types", cboEventType);
  TformSpecialGeneric::FormClose(Sender, Action);
}
//---------------------------------------------------------------------------
string TformLaunchDiabMeasurementComparisonReport::FormatDateLabel(string date1, string date2)
{
  string begin_date, end_date, date_title;
  if (date1 != "")
    begin_date = Pdate(date1, mdy | yyyy).OutDate(mdy | yyyy);
  if (date2 != "")
    end_date   = Pdate(date2, mdy | yyyy).OutDate(mdy | yyyy);
  if (date1 != "" && date2 != "")
    date_title = begin_date + " - " + end_date;
  else if (date1 != "" && date2 == "")
    date_title = "On or after " + begin_date;
  else if (date1 == "" && date2 != "")
    date_title = "On or before " + end_date;
  else if (date1 == "" && date2 == "")
    date_title = "Any date";
  return date_title;
}
//---------------------------------------------------------------------------

