//---------------------------------------------------------------------------

#include <vcl.h>
#pragma hdrstop

#include "launchDiabMeasurementReport.h"
#include "codeSearchPerson.h"
#include "reportDiabMeasurement.h"
//---------------------------------------------------------------------------
#pragma package(smart_init)
#pragma link "codeSpecialGeneric"
#pragma resource "*.dfm"
TformLaunchDiabMeasurementReport *formLaunchDiabMeasurementReport;
//---------------------------------------------------------------------------
__fastcall TformLaunchDiabMeasurementReport::TformLaunchDiabMeasurementReport(TComponent* Owner, string id)
        : TformSpecialGeneric(Owner)
{
  dateBegin->Date = Now();
  dateEnd->Date = Now();
  CreateLink(txtName, btnPersonSearch, this, id, "people");
}
//---------------------------------------------------------------------------
void __fastcall TformLaunchDiabMeasurementReport::btnPersonSearchClick(TObject *Sender)
{
  // create and show new person search form
  vector<string> return_vector;
  FORM_TRY
    TformSearchPerson * formPersonSearch = new TformSearchPerson(NULL,&return_vector);
    formPersonSearch->ShowTaskModal(this);
  FORM_CATCH
  if (! return_vector.empty())
    CreateLink(txtName, btnPersonSearch, this, return_vector[0], "people");
}
//---------------------------------------------------------------------------
void __fastcall TformLaunchDiabMeasurementReport::btnCloseClick(TObject *Sender)
{
  Close();        
}
//---------------------------------------------------------------------------
void __fastcall TformLaunchDiabMeasurementReport::btnRunClick(TObject *Sender)
{
  string person_id = GetValidLinkId(txtName);
  if (person_id != "")
  {
    Processing(true);

    TIBTransaction * dbTransaction = new TIBTransaction(NULL);
    TIBDataSet * dbDataSet = new TIBDataSet(NULL);
    string sql;
    string begin_date = Pdate(dateBegin->Date).OutDate(mdy | yyyy);
    string end_date =  Pdate(dateEnd->Date).OutDate(mdy | yyyy);
    string date_title = begin_date + " - " + end_date;
    string title = "Measurement report for " + GetPersonName(dbDatabase, person_id, first_last);

    try
    {
      StdTransaction(dbTransaction, dbDatabase);
      dbDataSet->Database = dbDatabase;
      dbDataSet->Transaction = dbTransaction;
      dbDataSet->UniDirectional = false;


      sql = "select obs_response_sets.date_of_observation, "
            "obs_height.response_metric height, "
            "obs_weight.response_metric weight, "
            "obs_waist.response_metric waist, "
            "obs_hip.response_metric hip, "
            "obs_tricep.response_metric tricep, "
            "obs_calf.response_metric calf, "
            "obs_bmi.response_metric bmi "
            "from obs_response_sets "
            "left join obs_responses obs_height on obs_height.obs_response_set_id = obs_response_sets.id and obs_height.obs_question_id = " + lookupTables->IdOf("obs_questions", "Height") + " "
            "left join obs_responses obs_weight on obs_weight.obs_response_set_id = obs_response_sets.id and obs_weight.obs_question_id = " + lookupTables->IdOf("obs_questions", "Weight") + " "
            "left join obs_responses obs_waist on obs_waist.obs_response_set_id = obs_response_sets.id and obs_waist.obs_question_id = " + lookupTables->IdOf("obs_questions", "Waist") + " "
            "left join obs_responses obs_hip on obs_hip.obs_response_set_id = obs_response_sets.id and obs_hip.obs_question_id = " + lookupTables->IdOf("obs_questions", "Hip") + " "
            "left join obs_responses obs_tricep on obs_tricep.obs_response_set_id = obs_response_sets.id and obs_tricep.obs_question_id = " + lookupTables->IdOf("obs_questions", "Tricep") + " "
            "left join obs_responses obs_calf on obs_calf.obs_response_set_id = obs_response_sets.id and obs_calf.obs_question_id = " + lookupTables->IdOf("obs_questions", "Calf") + " "
            "left join obs_responses obs_bmi on obs_bmi.obs_response_set_id = obs_response_sets.id and obs_bmi.obs_question_id = " + lookupTables->IdOf("obs_questions", "Body-Mass Index") + " "
            "where "
            "obs_response_sets.group_id = " + lookupTables->IdOf("groups", "Diabetes") + " and "
            "obs_response_sets.date_of_observation >= '" + begin_date + "' and "
            "obs_response_sets.date_of_observation <= '" + end_date + "' and "
            "obs_response_sets.person_id = " + person_id + " and " +
            "obs_response_sets.current_id > 2 "
            "order by "
            "obs_response_sets.date_of_observation";

      dbDataSet->SelectSQL->Add(sql.c_str());
      dbDataSet->Open();
      dbDataSet->First();

      FORM_TRY
        TformReportDiabMeasurement * measurementReport = new TformReportDiabMeasurement(NULL);
        measurementReport->measurementReport->DataSet = dbDataSet;
        measurementReport->lblTitle->Caption = title.c_str();
        measurementReport->lblDateTitle->Caption = date_title.c_str();
        measurementReport->txtDate->DataSet = dbDataSet;
        measurementReport->lblUser->Caption = PrettyString(lookupTables->ValueOf("users", current_userid)).c_str();
        measurementReport->txtDate->DataField = "date_of_observation";
        measurementReport->exprHeight->Expression = "FORMATNUMERIC('##0.00', height)";
        measurementReport->exprWeight->Expression = "FORMATNUMERIC('##0.00', weight)";
        measurementReport->exprWaist->Expression = "FORMATNUMERIC('##0.00', waist)";
        measurementReport->exprHip->Expression = "FORMATNUMERIC('##0.00', hip)";
        measurementReport->exprTricep->Expression = "FORMATNUMERIC('##0.00', tricep)";
        measurementReport->exprCalf->Expression = "FORMATNUMERIC('##0.00', calf)";
        measurementReport->exprBMI->Expression = "FORMATNUMERIC('##0.00', bmi)";
        Processing(false);
        measurementReport->measurementReport->Preview();
      FORM_CATCH
      dbDataSet->Close();
      dbTransaction->Commit();
    }
    catch (...)
    {
      Processing(false);
      PSay("Error running report", m_database_error, MB_OK);
      if (dbTransaction->Active)
        dbTransaction->Rollback();
    }
    delete dbTransaction;
    delete dbDataSet;
  }
  else
  {
    PSay("Please enter a client name", m_missing_data, MB_OK);
    btnPersonSearch->SetFocus();
  }        
}
//---------------------------------------------------------------------------
void __fastcall TformLaunchDiabMeasurementReport::FormShortCut(TWMKey &Msg,
      bool &Handled)
{
  Handled = true;
  if (Msg.CharCode == VK_RETURN)
    btnRun->Click();
  else
    Handled = false;

  if (!Handled)
    TformSpecialGeneric::FormShortCut(Msg, Handled);
}
//---------------------------------------------------------------------------

