//---------------------------------------------------------------------------

#include <vcl.h>
#pragma hdrstop

#include "launchDiabReminderLettersReport.h"
#include "codeSpecialSelectGroups.h"
#include "reportDiabAppointmentLetter.h"
#include "reportMailingLabels.h"
#include "scheduling.h"
//---------------------------------------------------------------------------
#pragma package(smart_init)
#pragma link "codeSpecialGeneric"
#pragma resource "*.dfm"
TformLaunchDiabReminderLettersReport *formLaunchDiabReminderLettersReport;
//---------------------------------------------------------------------------
__fastcall TformLaunchDiabReminderLettersReport::TformLaunchDiabReminderLettersReport(TComponent* Owner)
        : TformSpecialGeneric(Owner)
{
  checkTasks->Enabled = (optionsPermissions["table.tasks.diab"]["insert"] == "true");
  dateBegin->Date = Now();
  dateEnd->Date = Now();
  FillListView();

  // jhaley 20110706: Groups for which user has proper view permissions only - use custom SQL
  string groups_sql = "select groups.id, groups.item, groups.valid from groups where valid = 1 and id > 0 and exists "
                      "(select * from permissions where s = ('TABLE.APPOINTMENTS.' || groups.short_item) and "
                      " k = 'VIEW' and v = 'TRUE' and user_id = '" + current_userid + "') "
                      "order by groups.item ";
  lookupTables->RegisterComboBox("groups", cboDepartment, groups_sql);
}
//---------------------------------------------------------------------------
void __fastcall TformLaunchDiabReminderLettersReport::btnCloseClick(TObject *Sender)
{
  Close();
}
//---------------------------------------------------------------------------
void TformLaunchDiabReminderLettersReport::FillListView()
{
  // jhaley 20110706: Use checkbox filter
  string user_sql =
    "select users.item, groups.item, users.id "
    "from users "
    "inner join groups on users.group_id = groups.id "
    "where users.valid = 1 and users.id > 0 ";
  user_filter.SetListViewHandle(listUsers);
  user_filter.SetSecurity("");
  user_filter.AddColumnSpec("User Name",   "",            as_text, true,  190, prettyString);
  user_filter.AddColumnSpec("Department",  "",            as_text, true,  120, prettyString);
  user_filter.AddColumnSpec("Internal Id", "internal_id", as_int,  false,  80);
  user_filter.FetchPreferences("reminderlettersreport.user_filter");
  user_filter.SetIconStatus(false);
  user_filter.SetCheck(true);
  user_filter.SetSQLStatement(user_sql);
  user_filter.RefreshDataSet();
}
//---------------------------------------------------------------------------
void __fastcall TformLaunchDiabReminderLettersReport::menuSelectGroupsClick(TObject *Sender)
{
  // TODO: jhaley - not needed any more
}
//---------------------------------------------------------------------------
vector<string> TformLaunchDiabReminderLettersReport::GetCheckedUsers()
{
  // jhaley 20110706: considerably simplified via use of checkbox filter
  return user_filter.GetMultiID();
}
//---------------------------------------------------------------------------
void __fastcall TformLaunchDiabReminderLettersReport::btnRunClick(TObject *Sender)
{
  vector<string> user_ids;
  vector<string> group_ids;

  // jhaley 20110706: Completely separated functionality of "by employee" and "by department" due to problems
  if(radioByEmployee->Checked)
  {
    user_ids = GetCheckedUsers();
    if(user_ids.empty())
    {
      PSay("Please select one or more employees first.", m_missing_data, MB_OK);
      return;
    }
    string groups_sql = "select distinct group_id from users where " + SafeInClause("id", user_ids) + " ";
    SqlToVec(dbDatabase, groups_sql, group_ids);
    if(group_ids.size() > 1)
    {
      PSay("Please do not select employees in different departments at the same time.", m_invalid_data, MB_OK);
      return; // Can't do it. Sorry. The letter report logic is too brain-dead. I tried to fix it :( - jhaley
    }
  }
  else
  {
    if(!lookupTables->IsInLookUpTable("groups", cboDepartment))
    {
      PSay("Please select a valid department first.", m_missing_data, MB_OK);
      return;
    }
    string users_sql = "select id from users where valid = 1 and id > 0 and group_id = '"
                       + lookupTables->IdOf("groups", cboDepartment) + "' order by item ";
    SqlToVec(dbDatabase, users_sql, user_ids);
    group_ids.push_back(lookupTables->IdOf("groups", cboDepartment)); // one group only.
  }

  TIBTransaction *dbTransaction = new TIBTransaction(NULL);
  TIBDataSet     *dbDataset     = new TIBDataSet(NULL);
  string sql;
  string temp = "";

  try
  {
    Processing(true);
    dbTransaction->DefaultDatabase = dbDatabase;
    dbDataset->Database = dbDatabase;
    dbDataset->Transaction = dbTransaction;
    dbTransaction->StartTransaction();

    Pdate begin_date(dateBegin->Date);
    Pdate end_date(dateEnd->Date);
    PApptOccurrenceMap all_appointments;
    vector<string> appointments;

    map<string, vector<string> > ids_to_match;
    ids_to_match["visit_outcome_id"].push_back("0");
    ids_to_match["visit_outcome_id"].push_back(lookupTables->IdOf("visit_outcome_types", "As Scheduled"));
    ids_to_match["visit_outcome_id"].push_back(lookupTables->IdOf("visit_outcome_types", "Walk-In"));
    ids_to_match["visit_outcome_id"].push_back(lookupTables->IdOf("visit_outcome_types", "Arrived Late"));
    ids_to_match["current_id"].push_back("3");
    ids_to_match["current_id"].push_back("4");
    ids_to_match["current_id"].push_back("5");
    DownloadAppointments(dbDatabase, begin_date, end_date, current_userid, user_ids, vector<string>(), "",
                         all_appointments, ids_to_match,
                         (optionsPermissions["admin.all_appointments"]["enabled"] == "true"), false, "");

    for(PApptOccurrenceMap::iterator outer_itr = all_appointments.begin(); outer_itr != all_appointments.end(); outer_itr++)
    {
      for(PApptOccurrenceSet::iterator inner_itr = outer_itr->second.begin(); inner_itr != outer_itr->second.end(); inner_itr++)
      {
        appointments.push_back(inner_itr->id);
      }
    }

    if(appointments.size() > 0)
    {
      Processing(false);
      TformReportDiabAppointmentLetter::Do(appointments, group_ids[0], false, true, 1);

      if(checkTasks->Checked && optionsPermissions["table.tasks.diab"]["insert"] == "true")
      {
        Processing(true);
        // jhaley 20101118: repaired task creation ability
        sql = "insert into tasks (id, current_id, last_update_stamp, last_updated_by, group_id, "
              "user_id, person_id, date_of_task, method_id, direction_id, diab_activity_id, task_comments) "
              "select (select gen_id(tasks_gen, 1) from uno), 4, '" + RightNow() + "', " + current_userid + ", " +
              lookupTables->IdOf("groups", "Diabetes") + ", " + current_userid + ", appointments.person_id, '" + CurrentDate() + "', " +
              lookupTables->IdOf("method_types", "Mail") + ", " + lookupTables->IdOf("direction_types", "Outgoing") + ", " +
              lookupTables->IdOf("diab_activity_types", "Appointment letter") + ", 'APPOINTMENT REMINDER LETTER' "
              "from appointments where " + SafeInClause("appointments.id", appointments) + " ";
        ExecuteStatement(dbTransaction, sql);
        Processing(false);
      }
    }
    else
    {
      Processing(false);
      PSay("There are no appointments meeting the specified criteria for which to print reminder letters",
           m_message, MB_OK);
    }

    dbDataset->Close();
    dbTransaction->Commit();
  }
  catch(...)
  {
    Processing(false);
    if (dbTransaction->Active)
      dbTransaction->Rollback();
    PSay("Error printing letter", m_database_error, MB_OK);
  }

  delete dbDataset;
  delete dbTransaction;
}
//--------------------------------------------------------------------------- 
void __fastcall TformLaunchDiabReminderLettersReport::FormClose(TObject *Sender, TCloseAction &Action)
{
  // TODO: jhaley - restore reminder_letter preferences? They can actually be counter-productive when you do not want
  // to print out the same list of users every time...
  /*
  int row_count = listUsers->Items->Count;
  vector<string> id_vec;
  for(int i = 0; i < row_count; i++)
  {
    if(listUsers->Items->Item[i]->Checked)
      id_vec.push_back(listUsers->Items->Item[i]->SubItems->Strings[0].c_str());
  }
  if(! id_vec.empty())
    optionsPreferences["reminder_letters"]["user_id_string"] = VecToCommaString(id_vec);
  else
    optionsPreferences["reminder_letters"]["user_id_string"] = "";
  */
  
  lookupTables->UnregisterComboBox("groups", cboDepartment);
  TformSpecialGeneric::FormClose(Sender, Action);
}
//---------------------------------------------------------------------------
void __fastcall TformLaunchDiabReminderLettersReport::cboDepartmentChange(TObject *Sender)
{
  // TODO: jhaley - not needed any longer
}
//---------------------------------------------------------------------------
// buttonModeChangeClick - Allows switching back to "By Employee" mode with all the employees in that department auto-
// checked, which allows for customizing the list as was possible before I redesigned this sucker.
void __fastcall TformLaunchDiabReminderLettersReport::buttonModeChangeClick(TObject *Sender)
{
  if(!lookupTables->IsInLookUpTable("groups", cboDepartment))
  {
    PSay("Please select a department first", m_missing_data, MB_OK);
  }
  else
  {
    string group_id = lookupTables->IdOf("groups", cboDepartment);
    set<string> user_ids;
    string user_sql =
      "select id from users where valid = 1 and id > 0 and group_id = '" + group_id + "' ";
    SqlToSet(dbDatabase, user_sql, user_ids);
    user_filter.UncheckAll();
    user_filter.SetCheckedWhereID(user_ids, true);
    radioByEmployee->Checked = true;
    radioByDept->Checked = false;
  }
}
//---------------------------------------------------------------------------
// buttonClearClick - jhaley: Added 20110706 to allow easy obvious way to clear out the filter completely, since a
// blank Department value is not available in the dropbox. (While it is true that you can already right-click the
// filter and select 'Clear all', I am trying to ELIMINATE all dependence on hidden UI elements, not introduce yet more
// of it!)
void __fastcall TformLaunchDiabReminderLettersReport::buttonClearClick(TObject *Sender)
{
  user_filter.UncheckAll();        
}
//---------------------------------------------------------------------------

