//---------------------------------------------------------------------------

#include <vcl.h>
#pragma hdrstop

#include "launchDollarsWithProceduresReport.h"
#include "reportDollarsWithProceduresSummary.h"
#include "reportDollarsWithProceduresDetail.h"
//---------------------------------------------------------------------------
#pragma package(smart_init)
#pragma link "launchGeneric"
#pragma link "codeBaseDropDown"
#pragma link "codeCheckListDropDown"
#pragma resource "*.dfm"
TformLaunchDollarsWithProceduresReport *formLaunchDollarsWithProceduresReport;
//---------------------------------------------------------------------------
__fastcall TformLaunchDollarsWithProceduresReport::TformLaunchDollarsWithProceduresReport(TComponent* Owner)
        : TformLaunchGeneric(Owner)
{
  pair<Pdate, Pdate> last_month_dates = GetPriorMonthDateRange(Pdate(Date()));
  // note default begin date is in control's Date field (8/1/2008)
  dateEnd->Date = last_month_dates.second.OutDate(); // last day of last month

  cbddGroups->Filter->SetSecurity("");
  cbddGroups->Filter->AddColumnSpec("Departments", "", as_text, true, 200);
  cbddGroups->Filter->AddColumnSpec("Current Id", "current_id", as_int, false, 40);
  cbddGroups->Filter->AddColumnSpec("Internal Id", "internal_id", as_int, false, 60);

  // we want to view all groups for which the user has permission to view tasks
  set<string> group_perms = GetGroupPermissions("table.tasks", "", "view", "true");

  // jhaley 20100831: added departments filter dropdown as on event-by-client report launch form
  cbddGroups->Filter->SetSQLStatement("select distinct groups.item, " + CleverCurrentId("groups.valid", "1", "Current", "Expired") + ", groups.id "
                                      "from groups "
                                      "where groups.id > 0 and " + SafeInClause("groups.id", SetToVec(group_perms)) + " "
                                      "order by groups.item ");

  cbddGroups->Filter->FetchPreferences("dollarswithprocedures.groups");
  cbddGroups->Filter->SetIconStatus(true);
  cbddGroups->Filter->ForceRefreshDataSet();
  cbddGroups->DisplayFieldName = "Departments";
  cbddGroups->DropDownHeight = 218;

  // auto-check everything except Contract Health and Little Axe
  set<string> not_perms;
  not_perms.insert(lookupTables->IdOf("groups", "Contract Health"));
  not_perms.insert(lookupTables->IdOf("groups", "Little Axe Clinic"));

  cbddGroups->Filter->SetCheckedWhereNotID(not_perms, true);
  cbddGroups->RefreshText();
}
//---------------------------------------------------------------------------
bool TformLaunchDollarsWithProceduresReport::IsValidOnScreen()
{
  if (dateBegin->Date > dateEnd->Date)
  {
    PSay("End date must be greater than or equal to begin date.", m_invalid_data, MB_OK);
    dateEnd->SetFocus();
    return false;
  }
  return true;
}
//---------------------------------------------------------------------------
bool TformLaunchDollarsWithProceduresReport::BeforeRunReport()
{
  // get groups for which to view tasks
  vector<string> groups_vec = cbddGroups->Filter->GetMultiID();

  if (groups_vec.size() == 0)
  {
    PSay("Please select one or more departments", m_missing_data, MB_OK);
    cbddGroups->SetFocus();
    return false;
  }

  dbSummaryDataSet = NULL;
  dbSummaryDataSet = new TIBDataSet(NULL);
  dbDetailDataSet  = NULL;
  dbDetailDataSet  = new TIBDataSet(NULL);

  return TformLaunchGeneric::BeforeRunReport();
}
//---------------------------------------------------------------------------
bool TformLaunchDollarsWithProceduresReport::RunReport()
{
  // get groups for which to view tasks
  vector<string> groups_vec = cbddGroups->Filter->GetMultiID();

  // get dates from launch form
  Pdate dtBegin = Pdate(dateBegin->Date); // used later on
  Pdate dtEnd = Pdate(dateEnd->Date); // used later on
  string begin_date = dtBegin.OutDate(mdy | yyyy);
  string end_date   = dtEnd.OutDate(mdy | yyyy);
  string date_title = begin_date + " - " + end_date;

  // setup datasets
  dbSummaryDataSet->Database       = dbDatabase;
  dbSummaryDataSet->Transaction    = report_transaction;
  dbSummaryDataSet->UniDirectional = false;
  dbDetailDataSet->Database        = dbDatabase;
  dbDetailDataSet->Transaction     = report_transaction;
  dbDetailDataSet->UniDirectional  = false;

  // create the various objects needed to create the combined report
  TQRPageList *source = NULL, *destination = NULL; // page list objects
  TQRPrinter  *printer_thing = NULL;               // QR printer object

  FORM_TRY
    source = new TQRPageList();
  FORM_CATCH
  FORM_TRY
    destination = new TQRPageList();
  FORM_CATCH
  FORM_TRY
    destination->Stream = new TQRStream(100000);
  FORM_CATCH
  FORM_TRY
    printer_thing = InitTQRPrinter();
  FORM_CATCH

  //
  // *** Run the first report - Summary ***
  //
  TformReportDollarsWithProceduresSummary *summary_report = NULL;
  FORM_TRY
    summary_report = new TformReportDollarsWithProceduresSummary(NULL);
  FORM_CATCH
  summary_report->reportSummary->DataSet = dbSummaryDataSet;

  /* TODO -c FANCY : I was going to clean this up, then realized that they don't even create bills, so there's not much point...
      select G.item department, (extract(year from T.date_of_task) || '-' || LPAD(extract(month from T.date_of_task), 2, '0')) yearmo,
         sum(charges.to_bill) amt_to_bill, sum(coalesce(billed.dollars_billed, 0)) amt_billed,
         count(distinct(T.id)) visits, count(billed.bills) claims
      from tasks T
         inner join groups G on T.group_id = G.id
         left join
           (select Px.task_id, sum(case when C.clinic_cost > 0 then C.clinic_cost else Px.charges end) to_bill
            from procedures Px
            left join codes C on Px.cpt_hcpcs = C.code
            where Px.current_id+1 > 3 group by 1) charges on T.id = charges.task_id
         left join
           (select B.task_id, sum(Tx.debit_amount - Tx.credit_amount) dollars_billed, count(distinct(B.id)) bills
            from bills B
            left join transactions Tx on Tx.bill_id = B.id
            left join accounts A on Tx.account_id = A.id
            where A.account_type_id = 4 group by 1) billed on T.id = billed.task_id
      where T.group_id = 18 and T.current_id+1 > 3 and T.date_of_task between '8/1/2008' and current_date
      group by 1, 2
      order by 1, 2, 3
*/
  string summary_sql = "select groups.item department, (extract(year from T.date_of_task) || '-' || LPAD(extract(month from T.date_of_task), 2, '0')) yearmo, "
                       //"sum(case when codes.clinic_cost > 0 then codes.clinic_cost else procedures.charges end) amt_to_bill, "
                       "sum(charges.to_bill) amt_to_bill, "
                       //"sum(case when bills.id is not null then case when codes.clinic_cost > 0 then codes.clinic_cost else procedures.charges end else 0 end) amt_billed, "
                       "count(distinct(T.id)) visits, count(distinct(bills.id)) claims "
                       "from tasks T "
                       "inner join groups on T.group_id = groups.id "
                       "left join "
                       " (select Px.task_id, sum(case when C.clinic_cost > 0 then C.clinic_cost else Px.charges end) to_bill "
                       "  from procedures Px "
                       "  left join codes C on Px.cpt_hcpcs = C.code "
                       "  where Px.current_id+1 > 3 group by 1) charges on T.id = charges.task_id "
                       "left join bills on bills.task_id = T.id "
                       "left join transactions paid_transactions on paid_transactions.bill_id = bills.id "
                       "left join accounts paid_accounts on paid_transactions.account_id = paid_accounts.id "
                       "where T.date_of_task between '" + begin_date + "' and '" + end_date + "' and " + SafeInClause("T.group_id", groups_vec) + " "
                       "and T.current_id+1 > 3 "
                       "group by 1, 2 "
                       "order by 1, 2, 3";
  dbSummaryDataSet->SelectSQL->Clear();
  dbSummaryDataSet->SelectSQL->Add(summary_sql.c_str());
  dbSummaryDataSet->Open();
  string csv = "Dollars Associated With Procedures " + IntToString(dtBegin.year) + "-" + IntToString(dtBegin.month) +
               " through " + IntToString(dtEnd.year) + "-" + IntToString(dtEnd.month) + ".csv";
  if (!DataSetToCSV(dbSummaryDataSet, csv))
    PSay("Failed to write summary data to CSV file", m_message, MB_OK);
  dbSummaryDataSet->First();

  // setup stuff on the report form...
  summary_report->lblDateTitle->Caption = date_title.c_str();  // display dates on title band
  summary_report->lblUser->Caption = current_fullname.c_str(); // display generating user's name on page footer

  // detail band controls
  summary_report->txtDepartment->DataSet = dbSummaryDataSet;
  summary_report->txtDepartment->DataField = "department";
  summary_report->txtYearMo->DataSet = dbSummaryDataSet;
  summary_report->txtYearMo->DataField = "yearmo";
  summary_report->txtVisits->DataSet = dbSummaryDataSet;
  summary_report->txtVisits->DataField = "visits";
  summary_report->txtClaims->DataSet = dbSummaryDataSet;
  summary_report->txtClaims->DataField = "claims";
  summary_report->exprIncoming->Expression = "FORMATNUMERIC('$###,###,##0.00', amt_to_bill)";
  summary_report->exprBilled->Expression = "FORMATNUMERIC('$###,###,##0.00', amt_billed)";

  // group header bands
  summary_report->bandGroupHeader->Expression = "department";

  // group footer
  summary_report->exprDeptTotal->Expression  = "FORMATNUMERIC('$###,###,##0.00', SUM(amt_to_bill))";
  summary_report->exprDeptVisits->Expression = "SUM(visits)";
  summary_report->exprDeptBilled->Expression = "FORMATNUMERIC('$###,###,##0.00', SUM(amt_billed))";
  summary_report->exprDeptClaims->Expression = "SUM(claims)";

  // summary band
  summary_report->exprGrandTotal->Expression = "FORMATNUMERIC('$###,###,##0.00', SUM(amt_to_bill))";

  // write the first report to file and then load it into the destination page list
  string summary_filename = PMakeTempFilePath("summary.qrp");
  summary_report->reportSummary->Prepare();
  summary_report->reportSummary->QRPrinter->Save(summary_filename.c_str());
  source->Clear();
  source->LoadFromFile(summary_filename.c_str());
  for (int i = 1; i <= source->PageCount; i++)
    destination->AddPage(source->GetPage(i));

  // close the first report
  summary_report->CloseNow();
  dbSummaryDataSet->Close();

  //
  // *** Run the second report - Details ***
  //
  TformReportDollarsWithProceduresDetail *detail_report = NULL;
  FORM_TRY
    detail_report = new TformReportDollarsWithProceduresDetail(NULL);
  FORM_CATCH
  detail_report->reportDetail->DataSet = dbDetailDataSet;

  string detail_sql = "select groups.item department, (extract(year from date_of_task) || '-' || LPAD(extract(month from date_of_task), 2, '0')) yearmo, "
                      "tasks.date_of_task DOS, tasks.person_id, procedures.cpt_hcpcs px_code, "
                      "(case when codes.clinic_cost > 0 then codes.clinic_cost else procedures.charges end) amt_to_bill "
                      "from procedures "
                      "inner join tasks on procedures.task_id = tasks.id "
                      "inner join groups on tasks.group_id = groups.id "
                      "left join codes on procedures.cpt_hcpcs = codes.code "
                      "where start_date between '" + begin_date + "' and '" + end_date + "' and " + SafeInClause("tasks.group_id", groups_vec) + " "
                      "and procedures.current_id+1 > 3 and tasks.current_id+1 > 3 "
                      "order by 1, 2, 3";
  dbDetailDataSet->SelectSQL->Clear();
  dbDetailDataSet->SelectSQL->Add(detail_sql.c_str());
  dbDetailDataSet->Open();
  dbDetailDataSet->First();

  // setup stuff on the report form...
  detail_report->lblDateTitle->Caption = date_title.c_str();  // display dates on title band
  detail_report->lblUser->Caption = current_fullname.c_str(); // display generating user's name on page footer

  // detail band controls
  detail_report->txtDepartment->DataSet = dbDetailDataSet;
  detail_report->txtDepartment->DataField = "department";
  detail_report->txtYearMo->DataSet = dbDetailDataSet;
  detail_report->txtYearMo->DataField = "yearmo";
  detail_report->txtDOS->DataSet = dbDetailDataSet;
  detail_report->txtDOS->DataField = "DOS";
  detail_report->txtPersonID->DataSet = dbDetailDataSet;
  detail_report->txtPersonID->DataField = "person_id";
  detail_report->txtPxCode->DataSet = dbDetailDataSet;
  detail_report->txtPxCode->DataField = "px_code";
  detail_report->exprCharges->Expression = "FORMATNUMERIC('$###,###,##0.00', amt_to_bill)";

  // group header bands
  detail_report->bandGroupHeaderDept->Expression = "department"; // outer grouping
  detail_report->exprGroupDept->Expression = "department";
  detail_report->bandGroupHeaderMonth->Expression = "yearmo";    // inner grouping

  // group footer band
  detail_report->exprGroupDate->Expression = "yearmo";
  detail_report->exprGroupCharges->Expression = "FORMATNUMERIC('$###,###,##0.00', SUM(amt_to_bill))";

  // summary band
  detail_report->exprGrandTotal->Expression = "FORMATNUMERIC('$###,###,##0.00', SUM(amt_to_bill))";

  // write the second report to file and then load it into the destination page list
  string details_filename = PMakeTempFilePath("details.qrp");
  detail_report->reportDetail->Prepare();
  detail_report->reportDetail->QRPrinter->Save(details_filename.c_str());
  source->Clear();
  source->LoadFromFile(details_filename.c_str());
  for (int i = 1; i <= source->PageCount; i++)
    destination->AddPage(source->GetPage(i));

  // close the second report
  detail_report->CloseNow();
  dbDetailDataSet->Close();

  //
  // *** Display the concatenated reports ***
  //
  string complete_filepath = PMakeTempFilePath("complete.qrp");
  destination->Finish();
  destination->SaveToFile(complete_filepath.c_str());
  printer_thing->Load(complete_filepath.c_str());
  Processing(false);
  printer_thing->PreviewModal();

  // shut down actions - free all the stuff we created
  delete source;
  delete destination;
  delete printer_thing;

  return true;
}
//---------------------------------------------------------------------------
bool TformLaunchDollarsWithProceduresReport::AfterRunReport()
{
  if (dbSummaryDataSet)
  {
    delete dbSummaryDataSet;
    dbSummaryDataSet = NULL;
  }
  if (dbDetailDataSet)
  {
    delete dbDetailDataSet;
    dbDetailDataSet = NULL;
  }
  return TformLaunchGeneric::AfterRunReport();
}
//---------------------------------------------------------------------------
