//---------------------------------------------------------------------------

#include <vcl.h>
#pragma hdrstop

#include "launchEnrollment.h"
#include "reportEnrollment.h"
//---------------------------------------------------------------------------
#pragma package(smart_init)
#pragma link "launchGeneric"
#pragma link "codeFrameCheckListDropDown"
#pragma resource "*.dfm"
TformLaunchEnrollmentReport *formLaunchEnrollmentReport;
//---------------------------------------------------------------------------
__fastcall TformLaunchEnrollmentReport::TformLaunchEnrollmentReport(TComponent* Owner)
        : TformLaunchGeneric(Owner)
{
  // populate Type only with programs they have permission to pick
  // auto-select item if only one item in list

  string sql = "select ET.id, ET.item, ET.valid from "
               "enrollment_types ET "
               "inner join permissions P2 on P2.s = ET.permission_name and P2.k = 'VIEW' and P2.v = 'TRUE' and P2.user_id = " + current_userid + " ";
  lookupTables->RegisterComboBox("enrollment_types", cboType, sql);
  if (cboType->Items->Count == 1)
    cboType->ItemIndex = 0;

  cbddStatus->Filter->SetSecurity("");
  cbddStatus->Filter->AddColumnSpec("Status", "", as_text, true, 200);
  cbddStatus->Filter->AddColumnSpec("Current Id", "current_id", as_int, false, 40);
  cbddStatus->Filter->AddColumnSpec("Internal Id", "internal_id", as_int, false, 60);
  cbddStatus->Filter->FetchPreferences("enrollment_report.status");
  cbddStatus->Filter->SetIconStatus(true);
  cbddStatus->DisplayFieldName = "Status";
  cbddStatus->DropDownHeight = 144;

  editAdmit->Text = DateToStr(Date() - 180).c_str(); // back 6 months.  ish.
  editRemit->Text = DateToStr(Date()).c_str();
}
//---------------------------------------------------------------------------
void __fastcall TformLaunchEnrollmentReport::cboTypeChange(TObject *Sender)
{
  string sql = "select ES.item, " + CleverCurrentId("ES.valid", "1", "Current", "Expired") + ", ES.id "
               "from enrollment_status_types ES "
               "where ES.enrollment_id = '" + lookupTables->IdOf("enrollment_types", cboType) + "' or "
               "(ES.id = 0 and (select ET.allow_blank from enrollment_types ET where ET.id = '" + lookupTables->IdOf("enrollment_types", cboType) + "') = 1) ";

  cbddStatus->Filter->SetSQLStatement(sql);
  cbddStatus->Filter->ForceRefreshDataSet();
}
//---------------------------------------------------------------------------
bool TformLaunchEnrollmentReport::IsValidOnScreen()
{
  if (!lookupTables->IsInLookUpTable("enrollment_types", cboType))
  {
    PSay("Please select the type of enrollment you wish to enter.", m_invalid_data, MB_OK);
    cboType->SetFocus();
    return false;
  }

  // Verify dates
  if (!radioAll->Checked) // jhaley - Any option other than "All People" requires two valid dates
  {
    if (editAdmit->Text == "" || !Pdate(editAdmit->Text.c_str(), mdy | yy).IsValidDate())
    {
      PSay("Please enter a valid begin date", m_invalid_data, MB_OK);
      editAdmit->SetFocus();
      return false;
    }
    else if (editRemit->Text == "" || !Pdate(editRemit->Text.c_str(), mdy | yy).IsValidDate())
    {
      PSay("Please enter a valid end date", m_invalid_data, MB_OK);
      editRemit->SetFocus();
      return false;
    }
    else if (Pdate(editAdmit->Text.c_str(), mdy | yy) > Pdate(editRemit->Text.c_str(), mdy | yy))
    {
      PSay("Please enter a begin date that comes before the end date", m_invalid_data, MB_OK);
      editAdmit->SetFocus();
      return false;
    }
  }

  return true;
}
//---------------------------------------------------------------------------
bool TformLaunchEnrollmentReport::BeforeRunReport()
{
  if (TformLaunchGeneric::BeforeRunReport())
  {
    dbDataSet = new TIBDataSet(NULL);
    dbDataSet->Database = dbDatabase;
    dbDataSet->Transaction = report_transaction;
    dbDataSet->UniDirectional = false;
    return true;
  }
  return false;
}
//---------------------------------------------------------------------------
// jhaley 20101119: This was sitting around half-finished so I said "hey let's do it." This was originally looking very
// Diabetes-specific and would unconditionally display a lot of info that other departments aren't supposed to see, so
// I took account of that via permissions and it can now be enabled for all departments interested in enrollment programs.
bool TformLaunchEnrollmentReport::RunReport()
{
  string begin_date, end_date, date_title;
  string sql, fields, from, where, orderby;
  vector<string> status_ids = cbddStatus->Filter->GetMultiID();

  if (!radioAll->Checked) // Indicate the dates if they're relevant (otherwise it gets set to "")
  {
    begin_date = Pdate(editAdmit->Text.c_str(), mdy | yy).OutDate(mdy | yyyy);
    end_date   = Pdate(editRemit->Text.c_str(), mdy | yy).OutDate(mdy | yyyy);
    date_title = begin_date + " - " + end_date;
  }

  fields = "select (PN.last_name || ', ' || PN.first_name) name, P.birth_date, GT.item gender, E.enrollment_number, ES.item status, ";
  from   = "from enrollment E "
           "inner join people P on E.person_id = P.id "
           "inner join gender_types GT on P.gender_id = GT.id "
           "inner join person_names PN on P.person_names_id = PN.id "
           "inner join enrollment_status_types ES on E.enrollment_status_id = ES.id ";

  // if has Diabetes permissions, get data for diabetic diagnosis code, "Client", "STAR", and "STEPS" enrollments
  if (optionsPermissions["table.enrollment.diab"]["view"] == "true")
  {
    if (optionsPermissions["table.conditions.diab"]["view"] == "true")
    {
      // Display the first chronological diagnosis (this is what's done in a couple other places so I follow suit - we
      // could also display the *newest* diagnosis if somebody feels that's more relevant - also, other departments could
      // have some relevant diagnosis data display here but I don't know what one type of code they would consider to be
      // nearly all-inclusive with respect to their clients and/or prospects).

      fields += "COND.code diag_code, COND.start_date, ";
      from   += "left join conditions COND on COND.id = (select first 1 IC.id from conditions IC where IC.person_id = P.id and "
                " (IC.code = '250' or IC.code like '250.%') and IC.condition_id <> '" + lookupTables->IdOf("condition_types", "Relative") +
                "' and IC.current_id+1 >= 4 order by IC.start_date) ";
    }
    fields += "(case when exists (select DCE.id from enrollment DCE where DCE.person_id = P.id and DCE.current_id+1 >= 4 and "
              "  DCE.enrollment_id = '" + lookupTables->IdOf("enrollment_types", "Diabetes Client") + "') then 'Yes' else 'No' end) is_client, "
              "(case when exists (select STAR.id from enrollment STAR where STAR.person_id = P.id and STAR.current_id+1 >= 4 and "
              "  STAR.enrollment_id = '" + lookupTables->IdOf("enrollment_types", "STAR Participant") + "') then 'Yes' else 'No' end) is_star, "
              "(case when exists (select STEPS.id from enrollment STEPS where STEPS.person_id = P.id and STEPS.current_id+1 >= 4 and "
              "  STEPS.enrollment_id = '" + lookupTables->IdOf("enrollment_types", "STEPS Participant") + "') then 'Yes' else 'No' end) is_steps, ";
  }

  // Add number of visits - only count ones they should know about.
  set<string> can_see_tasks = GetSpecialGroupPermissions("general");
  fields += "(select count(T.id) from tasks T where T.current_id+1 >= 3 and T.person_id = P.id and "
            " T.method_id = '" + lookupTables->IdOf("method_types", "Visit") + "' and "
            + SafeInClause("T.group_id", SetToVec(can_see_tasks)) + ") visit_count ";

  // Any enrollment of the specified type not marked 'deleted' or 'incorrect'
  where = "where E.enrollment_id = '" + lookupTables->IdOf("enrollment_types", cboType) + "' and E.current_id+1 >= 3 ";

  // Additional Restrictions:
  if (radioAdmitted->Checked)
  {
    // Enrolled during the specified time period
    where += "and (exists (select EC.id from enrollment_changes EC where EC.enrollment_id = E.id and "
             " EC.enrollment_change_id = '" + lookupTables->IdOf("enrollment_change_types", "Enrolled") + "' and "
             " EC.new_admit_date between '" + begin_date + "' and '" + end_date + "') or "
             "E.admit_date between '" + begin_date + "' and '" + end_date + "') ";
  }
  else if (radioRemitted->Checked)
  {
    // Un-enrolled during the specified time period
    where += "and (exists (select EC.id from enrollment_changes EC where EC.enrollment_id = E.id and "
             " EC.enrollment_change_id = '" + lookupTables->IdOf("enrollment_change_types", "Un-Enrolled") + "' and "
             " EC.new_remit_date between '" + begin_date + "' and '" + end_date + "') or "
             "E.remit_date between '" + begin_date + "' and '" + end_date + "') ";
  }
  else if (radioOverlap->Checked) // TODO : These are a bit unsatisfactory because so many enrollments have no attached dates...
  {
    // For the enrollment period to overlap at all, that means:
    // * Admit is either unspecified (so we can't say), or it is before the end of the date range - AND -
    // * Remit is either unspecified (so we can't say), or it is after the beginning of the date range.
    where += "and ((E.admit_date is null or E.admit_date <= '" + end_date + "') and "
             "     (E.remit_date is null or E.remit_date > '" + begin_date + "')) ";
  }
  else if (radioContains->Checked)
  {
    // For the enrollment period to completely contain the specified dates, it means:
    // * Admit is either unspecified (so we can't say), or it is before/equal to the beginning of the date range -AND-
    // * Remit is either unspecified (so we can't say), or it is after/equal to the end of the date range.
    where += "and ((E.admit_date is null or E.admit_date <= '" + begin_date + "') and "
             "     (E.remit_date is null or E.remit_date >= '" + end_date + "')) ";
  }

  // Add restriction to particular enrollment status types, if any have been specified
  if (!status_ids.empty())
    where += "and " + SafeInClause("E.enrollment_status_id", status_ids) + " ";

  // Determine ordering
  if (radioPerson->Checked)
    orderby = "order by PN.last_name, PN.first_name ";
  else if (radioStatus->Checked)
    orderby = "order by ES.item ";
  else if (radioEnrolled->Checked)
    orderby = "order by E.admit_date ";
  else if (radioUnenrolled->Checked)
    orderby = "order by E.remit_date ";
  else if (radioEnrollmentNumber->Checked)
    orderby = "order by E.enrollment_number ";
  else if (radioPCP->Checked)
  {
    // This option is a bit odd considering the data isn't displayed on the report. ::Shrugs::
    from += "left join tax_entities on E.provider_id = tax_entities.id ";
    orderby = "order by tax_entities.name ";
  }

  // Build the query
  sql = fields + from + where + orderby;

  dbDataSet->SelectSQL->Clear();
  dbDataSet->SelectSQL->Add(sql.c_str());
  dbDataSet->Open();
  dbDataSet->First();

  TformReportEnrollment *report_form = NULL;
  FORM_TRY
    report_form = new TformReportEnrollment(NULL);
  FORM_CATCH
  report_form->report->DataSet = dbDataSet;
  report_form->lblUser->Caption = PrettyString(lookupTables->ValueOf("users", current_userid)).c_str();
  report_form->lblDateTitle->Caption = date_title.c_str();

  // Detail band controls
  report_form->txtName->DataSet = dbDataSet;
  report_form->txtName->DataField = "name";
  report_form->txtDOB->DataSet = dbDataSet;
  report_form->txtDOB->DataField = "birth_date";
  report_form->txtGender->DataSet = dbDataSet;
  report_form->txtGender->DataField = "gender";
  report_form->txtEnrollmentNumber->DataSet = dbDataSet;
  report_form->txtEnrollmentNumber->DataField = "enrollment_number";
  report_form->txtStatus->DataSet = dbDataSet;
  report_form->txtStatus->DataField = "status";

  if (optionsPermissions["table.enrollment.diab"]["view"] == "true")
  {
    if (optionsPermissions["table.conditions.diab"]["view"] == "true")
    {
      report_form->txtCode->DataSet = dbDataSet;
      report_form->txtCode->DataField = "diag_code";
      report_form->txtDiagDate->DataSet = dbDataSet;
      report_form->txtDiagDate->DataField = "start_date";
    }
    else
    {
      report_form->txtCode->Enabled = false;
      report_form->txtDiagDate->Enabled = false;
    }
    report_form->txtClient->DataSet = dbDataSet;
    report_form->txtClient->DataField = "is_client";
    report_form->txtStar->DataSet = dbDataSet;
    report_form->txtStar->DataField = "is_star";
    report_form->txtSteps->DataSet = dbDataSet;
    report_form->txtSteps->DataField = "is_steps";
  }
  else
  {
    // No permissions to view any of this info.
    report_form->txtCode->Enabled = false;
    report_form->txtDiagDate->Enabled = false;
    report_form->txtClient->Enabled = false;
    report_form->txtStar->Enabled = false;
    report_form->txtSteps->Enabled = false;
  }

  report_form->txtVisitCount->DataSet = dbDataSet;
  report_form->txtVisitCount->DataField = "visit_count";

  report_form->report->Preview();

  return true;
}
//---------------------------------------------------------------------------
bool TformLaunchEnrollmentReport::AfterRunReport()
{
  if (dbDataSet)
  {
    dbDataSet->Close();
    delete dbDataSet;
    dbDataSet = NULL;
  }
  return TformLaunchGeneric::AfterRunReport();
}
//---------------------------------------------------------------------------

