//---------------------------------------------------------------------------

#include <vcl.h>
#pragma hdrstop

#include "launchEventByClientReport.h"
#include "codeSearchPerson.h"
#include "reportEventByClient.h"
//---------------------------------------------------------------------------
#pragma package(smart_init)
#pragma link "codeSpecialGeneric"
#pragma link "codeBaseDropDown"
#pragma link "codeCheckListDropDown"
#pragma resource "*.dfm"
TformLaunchEventByClientReport *formLaunchEventByClientReport;
//---------------------------------------------------------------------------
__fastcall TformLaunchEventByClientReport::TformLaunchEventByClientReport(TComponent* Owner, string id, string group_id)
        : TformSpecialGeneric(Owner)
{
  dateBegin->Date = Now();
  dateEnd->Date = Now();
  CreateLink(txtName, btnPersonSearch, this, id, "people");

  string clever_current_id = "case when groups.valid = 1 then '" + lookupTables->IdOf("current_types", "Current") +
                             "' else '" + lookupTables->IdOf("current_types", "Expired") + "' end";
  cbddGroups->Filter->SetSecurity("");
  cbddGroups->Filter->AddColumnSpec("Departments", "", as_text, true, 200);
  cbddGroups->Filter->AddColumnSpec("Current Id", "current_id", as_int, false, 40);
  cbddGroups->Filter->AddColumnSpec("Internal Id", "internal_id", as_int, false, 60);

  string where_clause = "where ";
  set<string> group_perms = GetGroupPermissions("table.events", "", "view", "true");

  if (group_id != "") // if a specific group_id was passed into the constructor, then we want to view ONLY that group
    where_clause += "groups.id = '" + group_id + "' ";
  else // otherwise, with group_id == "", we want to view all groups for which the user has permission to view events
    where_clause += SafeInClause("groups.id", SetToVec(group_perms)) + " ";

  cbddGroups->Filter->SetSQLStatement("select distinct groups.item, " + clever_current_id + ", groups.id from groups "
                                      "inner join events on events.group_id = groups.id "
                                      + where_clause +
                                      "order by groups.item ");

  cbddGroups->Filter->FetchPreferences("eventbyclient_report.groups");
  cbddGroups->Filter->SetIconStatus(true);
  cbddGroups->Filter->ForceRefreshDataSet();
  cbddGroups->DisplayFieldName = "Departments";
  cbddGroups->DropDownHeight = 144;

  // auto-check all available groups (in the case of group_id != "", there is only one group, and the groups were
  // already limited to those with permissions otherwise)
  cbddGroups->Filter->SetCheckedWhereID(group_perms, true);
  cbddGroups->RefreshText();
}

//---------------------------------------------------------------------------
void __fastcall TformLaunchEventByClientReport::btnPersonSearchClick(
      TObject *Sender)
{
   // create and show new person search form
  vector<string> return_vector;
  FORM_TRY
    TformSearchPerson * formPersonSearch = new TformSearchPerson(NULL,&return_vector);
    formPersonSearch->ShowTaskModal(this);
  FORM_CATCH
  if (! return_vector.empty())
    CreateLink(txtName, btnPersonSearch, this, return_vector[0], "people");
}
//---------------------------------------------------------------------------
void __fastcall TformLaunchEventByClientReport::btnRunClick(TObject *Sender)
{
  vector<string> groups = cbddGroups->Filter->GetMultiID();

  string person_id = GetValidLinkId(txtName);
  if (person_id == "")
  {
    PSay("Please enter a client name", m_missing_data, MB_OK);
    btnPersonSearch->SetFocus();
  }
  else if (groups.size() == 0)
  {
    PSay("Please select one or more departments", m_missing_data, MB_OK);
    cbddGroups->SetFocus();
  }
  else
  {
    Processing(true);
    TIBTransaction * dbTransaction = new TIBTransaction(NULL);
    TIBDataSet * dbDataSet = new TIBDataSet(NULL);
    string sql;
    string begin_date = Pdate(dateBegin->Date).OutDate(mdy | yyyy);
    string end_date =  Pdate(dateEnd->Date).OutDate(mdy | yyyy);
    string date_title = begin_date + " - " + end_date;
    string title = "Event report for " + GetPersonName(dbDatabase, person_id, first_last);

    try
    {
      dbTransaction->DefaultDatabase = dbDatabase;
      dbDataSet->Database = dbDatabase;
      dbDataSet->Transaction = dbTransaction;
      dbDataSet->UniDirectional = false;


      sql = "select events.date_of_event, event_name_types.item event_name, event_type_types.item event_type, "
            "events.group_id, case "
            "when event_location_types.zip_code_id = 0 then event_location_types.item "
            "when event_location_types.item = '' then (zip_codes.city || ', ' || zip_codes.state) "
            "else (zip_codes.city || ', ' || zip_codes.state || ' - ' || event_location_types.item) end location "
            "from events "
            "inner join participations on participations.event_id = events.id "
            "inner join event_name_types on events.event_name_id = event_name_types.id "
            "inner join event_type_types on events.event_type_id = event_type_types.id "
            "inner join event_location_types on events.event_location_id = event_location_types.id "
            "inner join zip_codes on event_location_types.zip_code_id = zip_codes.id "
            "where participations.person_id = '" + person_id + "' and "
            "events.date_of_event >= '" + begin_date + "' and "
            "events.date_of_event <= '" + end_date + "' and "
            + SafeInClause("events.group_id", groups) + " and "
            "events.current_id >= 3 and participations.current_id >= 3 "
            "order by events.date_of_event ";

      dbTransaction->StartTransaction();
      dbDataSet->SelectSQL->Add(sql.c_str());
      dbDataSet->Open();
      dbDataSet->First();

      FORM_TRY
        TformReportEventByClient * eventReport = new TformReportEventByClient(NULL);
        eventReport->eventReport->DataSet = dbDataSet;
        eventReport->lblTitle->Caption = title.c_str();
        eventReport->lblDateTitle->Caption = date_title.c_str();
        eventReport->lblUser->Caption = PrettyString(lookupTables->ValueOf("users", current_userid)).c_str();
        eventReport->txtDate->DataSet = dbDataSet;
        eventReport->txtDate->DataField = "date_of_event";
        eventReport->txtEventName->DataSet = dbDataSet;
        eventReport->txtEventName->DataField = "event_name";
        eventReport->txtEventType->DataSet = dbDataSet;
        eventReport->txtEventType->DataField = "event_type";
        eventReport->txtEventDepartment->DataSet = dbDataSet;
        eventReport->txtEventDepartment->DataField = "group_id";
        eventReport->txtEventLocation->DataSet = dbDataSet;
        eventReport->txtEventLocation->DataField = "location";
        Processing(false);
        eventReport->eventReport->Preview();
      FORM_CATCH
      dbDataSet->Close();
      dbTransaction->Commit();
    }
    catch (...)
    {
      Processing(false);
      PSay("Error running report", m_database_error, MB_OK);
      if (dbTransaction->Active)
        dbTransaction->Rollback();
    }

    delete dbTransaction;
    delete dbDataSet;
  }
}
//---------------------------------------------------------------------------
void __fastcall TformLaunchEventByClientReport::btnCloseClick(
      TObject *Sender)
{
   Close();
}
//---------------------------------------------------------------------------
void __fastcall TformLaunchEventByClientReport::FormShortCut(TWMKey &Msg,
      bool &Handled)
{
  Handled = true;
  if (Msg.CharCode == VK_RETURN)
    btnRun->Click();
  else
    Handled = false;

  if (!Handled)
    TformSpecialGeneric::FormShortCut(Msg, Handled);
}
//---------------------------------------------------------------------------

