//---------------------------------------------------------------------------

#include <vcl.h>
#pragma hdrstop

#include "launchEventListingReport.h"
#include "reportEventListing.h"
#include "derivedheaders.h"
//---------------------------------------------------------------------------
#pragma package(smart_init)
#pragma link "codeSpecialGeneric"
#pragma link "codeRubberContainer"
#pragma link "codeBaseDropDown"
#pragma link "codeCheckListDropDown"
#pragma resource "*.dfm"
TformLaunchEventListingReport *formLaunchEventListingReport;
//---------------------------------------------------------------------------
__fastcall TformLaunchEventListingReport::TformLaunchEventListingReport(TComponent* Owner, string group_id)
        : TformSpecialGeneric(Owner)
{
  dateBegin->Date = Now();
  dateEnd->Date = Now();

  event_filter.SetListViewHandle(listEvents);
  event_filter.SetSecurity("events");
  event_filter.AddColumnSpec("Event Name", "events", as_text, true, 184, prettyString);
  event_filter.AddColumnSpec("Internal Id", "internal_id", as_int, false, 80);
  event_filter.FetchPreferences("launchDiabEventReport.event_filter");
  event_filter.SetIconStatus(false);
  event_filter.SetAutosizeColumns(false);
  event_filter.SetCheck(true);
  event_filter.SetHeader(true);
  event_filter.RefreshDataSet();
  event_filter.SetSQLStatement("select item, id from event_name_types");
  event_filter.ForceRefreshDataSet();
//  event_filter.AutosizeColumns();
  event_filter.CheckAll();

  // jhaley 20100811: added departments filter dropdown as on event-by-client report launch form
  cbddGroups->Filter->SetSecurity("");
  cbddGroups->Filter->AddColumnSpec("Departments", "", as_text, true, 200);
  cbddGroups->Filter->AddColumnSpec("Current Id", "current_id", as_int, false, 40);
  cbddGroups->Filter->AddColumnSpec("Internal Id", "internal_id", as_int, false, 60);

  string where_clause = "where ";
  set<string> group_perms = GetGroupPermissions("table.events", "", "view", "true");

  if (group_id != "") // if a specific group_id was passed into the constructor, then we want to view ONLY that group
    where_clause += "groups.id = '" + group_id + "' ";
  else // otherwise, with group_id == "", we want to view all groups for which the user has permission to view events
    where_clause += SafeInClause("groups.id", SetToVec(group_perms)) + " ";

  cbddGroups->Filter->SetSQLStatement("select distinct groups.item, " + CleverCurrentId("groups.valid", "1", "Current", "Expired") + ", groups.id from groups "
                                      "inner join events on events.group_id = groups.id "
                                      + where_clause +
                                      "order by groups.item ");

  cbddGroups->Filter->FetchPreferences("eventlisting_report.groups");
  cbddGroups->Filter->SetIconStatus(true);
  cbddGroups->Filter->ForceRefreshDataSet();
  cbddGroups->DisplayFieldName = "Departments";
  cbddGroups->DropDownHeight = 144;

  // auto-check all available groups (in the case of group_id != "", there is only one group, and the groups were
  // already limited to those with permissions otherwise)
  cbddGroups->Filter->SetCheckedWhereID(group_perms, true);
  cbddGroups->RefreshText();
}
//---------------------------------------------------------------------------
void __fastcall TformLaunchEventListingReport::btnCloseClick(TObject *Sender)
{
  Close();        
}
//---------------------------------------------------------------------------
void __fastcall TformLaunchEventListingReport::btnRunClick(TObject *Sender)
{
  vector<string> event_names_vec = event_filter.GetMultiID();
  vector<string> groups_vec = cbddGroups->Filter->GetMultiID();

  // jhaley 20100811: good old-fashioned error checking before we use these in an "in" clause.
  if (event_names_vec.size() == 0)
  {
    PSay("Please select one or more event types", m_missing_data, MB_OK);
    listEvents->SetFocus();
  }
  else if (groups_vec.size() == 0)
  {
    PSay("Please select one or more departments", m_missing_data, MB_OK);
    cbddGroups->SetFocus();
  }
  else
  {
    Processing(true);

    TIBTransaction * dbTransaction = new TIBTransaction(NULL);
    TIBDataSet * dbDataSet = new TIBDataSet(NULL);
    string sql;
    string begin_date = Pdate(dateBegin->Date).OutDate(mdy | yyyy);
    string end_date =  Pdate(dateEnd->Date).OutDate(mdy | yyyy);
    string date_title = begin_date + " - " + end_date;

    try
    {
      StdTransaction(dbTransaction, dbDatabase);
      dbDataSet->Database = dbDatabase;
      dbDataSet->Transaction = dbTransaction;
      dbDataSet->UniDirectional = false;

      sql = "select events.date_of_event, event_name_types.item event_name, "
            "event_type_types.item event_type, case "
            "when event_location_types.zip_code_id = 0 then event_location_types.item "
            "when event_location_types.item = '' then (zip_codes.city || ', ' || zip_codes.state) "
            "else (zip_codes.city || ', ' || zip_codes.state || ' - ' || event_location_types.item) end location, "
            "events.number_attended, events.group_id, groups.item group_name "
            "from events "
            "inner join event_name_types on events.event_name_id = event_name_types.id "
            "inner join event_type_types on events.event_type_id = event_type_types.id "
            "inner join event_location_types on events.event_location_id = event_location_types.id "
            "inner join zip_codes on event_location_types.zip_code_id = zip_codes.id "
            "inner join groups on events.group_id = groups.id "
            "where " + SafeInClause("events.group_id", groups_vec) + " and "
            "events.date_of_event >= '" + begin_date + "' and "
            "events.date_of_event <= '" + end_date + "' and "
            "events.current_id >= 3 and "
            + SafeInClause("events.event_name_id", event_names_vec) + " "
            "order by groups.item, events.date_of_event";

      DebugSql(this, &sql);
      dbDataSet->SelectSQL->Add(sql.c_str());
      dbDataSet->Open();
      dbDataSet->First();

      FORM_TRY
        TformEventListingReport * report = new TformEventListingReport(NULL);
        report->eventReport->DataSet = dbDataSet;
        report->lblDateTitle->Caption = date_title.c_str();
        report->lblUser->Caption = PrettyString(lookupTables->ValueOf("users", current_userid)).c_str();
        report->txtDate->DataSet = dbDataSet;
        report->txtDate->DataField = "date_of_event";
        report->txtName->DataSet = dbDataSet;
        report->txtName->DataField = "event_name";
        report->txtEventType->DataSet = dbDataSet;
        report->txtEventType->DataField = "event_type";
        report->txtLocation->DataSet = dbDataSet;
        report->txtLocation->DataField = "location";
        report->txtAttendance->DataSet = dbDataSet;
        report->txtAttendance->DataField = "number_attended";
        report->exprTotAttendance->Expression = "SUM(number_attended)";
        report->groupHeaderBand->Expression = "group_id";
        report->exprGroup->Expression = "group_name";
        report->exprGroupTotalAttendance->Expression = "SUM(number_attended)";
        Processing(false);
        report->eventReport->Preview();
      FORM_CATCH
      dbDataSet->Close();
      dbTransaction->Commit();
    }
    catch (...)
    {
      Processing(false);
      PSay("Error running report", m_database_error, MB_OK);
      if (dbTransaction->Active)
        dbTransaction->Rollback();
    }
    delete dbTransaction;
    delete dbDataSet;
  }
}
//---------------------------------------------------------------------------
void __fastcall TformLaunchEventListingReport::FormShortCut(TWMKey &Msg,
      bool &Handled)
{
  Handled = true;
  if (Msg.CharCode == VK_RETURN)
    btnRun->Click();
  else
    Handled = false;

  if (!Handled)
    TformSpecialGeneric::FormShortCut(Msg, Handled);
}
//---------------------------------------------------------------------------

