//---------------------------------------------------------------------------

#include <vcl.h>
#pragma hdrstop

#include "launchFitnessMembershipSummaryReport.h"
#include "reportFitnessMembershipSummary.h"
#include "derivedheaders.h"
//---------------------------------------------------------------------------
#pragma package(smart_init)
#pragma link "codeSpecialGeneric"
#pragma resource "*.dfm"
TformLaunchFitnessMembershipSummaryReport *formLaunchFitnessMembershipSummaryReport;
//---------------------------------------------------------------------------
__fastcall TformLaunchFitnessMembershipSummaryReport::TformLaunchFitnessMembershipSummaryReport(TComponent* Owner)
        : TformSpecialGeneric(Owner)
{
  dateBegin->Date = Now();
  dateEnd->Date = Now();
  lookupTables->RegisterComboBox("fit_facility_types", cboFacility);
}
//---------------------------------------------------------------------------
void __fastcall TformLaunchFitnessMembershipSummaryReport::btnCloseClick(TObject *Sender)
{
  Close();
}
//---------------------------------------------------------------------------
void __fastcall TformLaunchFitnessMembershipSummaryReport::btnRunClick(TObject *Sender)
{
  if ((lookupTables->IsInLookUpTable("fit_facility_types", cboFacility->Text.c_str())) &&
     (cboFacility->Text != ""))
  {
    Processing(true);

    TIBTransaction * dbTransaction = new TIBTransaction(NULL);
    TIBDataSet * dbDataSet = new TIBDataSet(NULL);
    string sql;
    string begin_date = Pdate(dateBegin->Date).OutDate(mdy | yyyy);
    string end_date =  Pdate(dateEnd->Date).OutDate(mdy | yyyy);
    string date_title = begin_date + " - " + end_date;
    string title = "Fitness Memberships - ";
    title += cboFacility->Text.c_str();
    string memberships_total;

    try
    {
      dbTransaction->DefaultDatabase = dbDatabase;
      dbDataSet->Database = dbDatabase;
      dbDataSet->Transaction = dbTransaction;
      dbDataSet->UniDirectional = false;
      dbTransaction->StartTransaction();

      string date_logic;
      if (radioNew->Checked)
        date_logic = "fitness_memberships.begin_date between '" + begin_date + "' and '" + end_date + "' and "
                     "(fitness_memberships.end_date is null or fitness_memberships.end_date >= '" + begin_date + "') and ";
      else if (radioActivePartial->Checked) // active at least part of the date range specified
        date_logic = "(fitness_memberships.begin_date is null or fitness_memberships.begin_date <= '" + end_date + "') and "
                     "(fitness_memberships.end_date is null or fitness_memberships.end_date >= '" + begin_date + "') and ";
      else if (radioActiveEntire->Checked) // active at least the entire date range specified
        date_logic = "(fitness_memberships.begin_date is null or fitness_memberships.begin_date <= '" + begin_date + "') and "
                     "(fitness_memberships.end_date is null or fitness_memberships.end_date >= '" + end_date + "') and ";

      sql = "select count(fitness_memberships.id) "
            "from fitness_memberships "
            "where fitness_memberships.fit_facility_id = " + lookupTables->IdOf("fit_facility_types", cboFacility->Text.c_str()) + " and ";
      sql += date_logic +
            "fitness_memberships.current_id >= 3 and fitness_memberships.is_waiting = 0";
      memberships_total = GetOneField(dbTransaction, sql);

      sql = "select person_names.last_name || ', ' || person_names.first_name || ' ' || "
            "person_names.suffix name, fitness_people.is_primary, fitness_memberships.id "
            "from people "
            "left  join person_names on person_names.id = people.person_names_id "
            "inner join fitness_people on fitness_people.person_id = people.id "
            "inner join fitness_memberships on fitness_people.fitness_membership_id = fitness_memberships.id "
            "where fitness_memberships.fit_facility_id = " + lookupTables->IdOf("fit_facility_types", cboFacility->Text.c_str()) + " and ";
      sql += date_logic +
            "fitness_memberships.current_id >= 3 and "
            "fitness_people.current_id >= 3 and people.current_id >= 3 and fitness_memberships.is_waiting = 0 "
            "order by fitness_memberships.id, fitness_people.is_primary descending";

      DebugSql(this, &sql);

      dbDataSet->SelectSQL->Add(sql.c_str());
      dbDataSet->Open();
      dbDataSet->First();

      FORM_TRY
        TformReportFitnessMembershipSummary * report = new TformReportFitnessMembershipSummary(NULL);
        report->membershipReport->DataSet = dbDataSet;
        report->lblTitle->Caption = title.c_str();
        report->lblDateTitle->Caption = date_title.c_str();
        report->lblUser->Caption = PrettyString(lookupTables->ValueOf("users", current_userid)).c_str();
        report->lblTotalMemberships->Caption = memberships_total.c_str();
        report->groupBand->Expression = "id";
        report->exprPrimary->Expression = "IF(is_primary = '1', 'x', '')";
        report->txtName->DataSet = dbDataSet;
        report->txtName->DataField = "name";
        Processing(false);
        report->membershipReport->Preview();
      FORM_CATCH
      dbDataSet->Close();
      dbTransaction->Commit();
    }
    catch (...)
    {
      Processing(false);
      PSay("Error running report", m_database_error, MB_OK);
      if (dbTransaction->Active)
        dbTransaction->Rollback();
    }
    delete dbTransaction;
    delete dbDataSet;
  }
  else
    PSay("Please enter a valid facility", m_invalid_data, MB_OK);
}
//---------------------------------------------------------------------------
void __fastcall TformLaunchFitnessMembershipSummaryReport::FormClose(TObject *Sender, TCloseAction &Action)
{
  lookupTables->UnregisterComboBox("fit_facility_types", cboFacility);
  TformSpecialGeneric::FormClose(Sender, Action);
}
//---------------------------------------------------------------------------
void __fastcall TformLaunchFitnessMembershipSummaryReport::FormShortCut(TWMKey &Msg, bool &Handled)
{
  Handled = true;
  if (Msg.CharCode == VK_RETURN)
    btnRun->Click();
  else
    Handled = false;

  if (!Handled)
    TformSpecialGeneric::FormShortCut(Msg, Handled);
}
//---------------------------------------------------------------------------

