//---------------------------------------------------------------------------

#include <vcl.h>
#pragma hdrstop

#include "launchFormLetter.h"
#include "reportFormLetter.h"
#include "codeSearchPerson.h"
//---------------------------------------------------------------------------
#pragma package(smart_init)
#pragma link "launchGeneric"
#pragma resource "*.dfm"
TformLaunchFormLetter *formLaunchFormLetter;
//---------------------------------------------------------------------------
__fastcall TformLaunchFormLetter::TformLaunchFormLetter(TComponent* Owner, string person_id, pmapstrs *pvariables)
        : TformLaunchGeneric(Owner)
{
  Pdate today_date(Date());

  // Person ID can be passed in optionally (pass "" if none is available)
  CreateLink(txtPerson, btnPersonSearch, this, person_id, "people");

  dateLetter->Date = today_date.OutDate();

  // Copy in any variable values that were passed in
  if(pvariables)
  {
    lookupTables->RegisterComboBox("form_letters", cboLetterType);
    passed_variables = *pvariables;
  }
  else
  {
    // If no external vars were passed in, remove those letters from the drop-down.
    string sql = "select id, item, valid from form_letters where valid = 1 and needs_external_vars = 0";
    lookupTables->RegisterComboBox("form_letters", cboLetterType, sql);
  }
}
//---------------------------------------------------------------------------
__fastcall TformLaunchFormLetter::~TformLaunchFormLetter()
{
  lookupTables->UnregisterComboBox("form_letters", cboLetterType);
}
//---------------------------------------------------------------------------
bool TformLaunchFormLetter::IsValidOnScreen()
{
  if(IsBlankOrZero(GetValidLinkId(txtPerson)))
  {
    PSay("Please select a valid person.", m_missing_data, MB_OK);
    btnPersonSearch->SetFocus();
    return false;
  }
  if(!lookupTables->IsInLookUpTable("form_letters", cboLetterType))
  {
    PSay("Please select a form letter to print.", m_missing_data, MB_OK);
    cboLetterType->SetFocus();
    return false;
  }
  
  return true;
}
//---------------------------------------------------------------------------
bool TformLaunchFormLetter::BeforeRunReport()
{
  dbLetterItemsSet = AddDataSet();
  return TformLaunchGeneric::BeforeRunReport();
}
//---------------------------------------------------------------------------
string TformLaunchFormLetter::DoVariableSubstitutions(const string &input)
{
  string outstr = input;

  for(pmapstrs::const_iterator itr = variables.begin(); itr != variables.end(); ++itr)
    outstr = ReplaceStringWithString(outstr, itr->first, itr->second);

  return outstr;
}
//---------------------------------------------------------------------------
bool TformLaunchFormLetter::RunReport()
{
  string letter_date    = Pdate(dateLetter->Date).OutDate(mdy | yyyy);
  string form_letter_id = lookupTables->IdOf("form_letters", cboLetterType);
  string person_id      = GetValidLinkId(txtPerson);
  string user_id;       // Comes from the form_letters record
  string group_id;      // Comes from the form_letters record

  pmapstrs form_letter_data;
  pmapstrs person_data;
  pmapstrs user_data;
  pmapstrs dept_info;
  
  // Get Form Letter data (selected by combo box):
  // * title text
  // * header text
  // * footer text
  // * enclosures text
  ++checkpoint; // 1
  string form_letter_sql =
    "select title_text, header_text, footer_text, enclosures_text, signing_user_id, sending_group_id, phone_number, "
    " use_dept_name "
    "from form_letters "
    "where id = '" + form_letter_id + "' ";
  if(!SqlToMap(dbDatabase, form_letter_sql, form_letter_data))
    return false;

  // Setup variable substitutions map

  // Local universal variables
  variables["$nl$"]           = "\n";
  variables["$phone_number$"] = FormatPhoneNumberNoBreak(form_letter_data["phone_number"]);

  // Merge in variables passed from the caller
  MergeMap(variables, passed_variables);

  // Get user id and group id
  user_id  = form_letter_data["signing_user_id"];
  group_id = form_letter_data["sending_group_id"];

  // Get Person Information:
  // * Name (First + Last + Suffix)
  // * Address (Line1, Line2 (if present), City, State, Zip
  // * Chart Number
  ++checkpoint; // 2
  string person_sql =
    "select (PN.first_name || ' ' || PN.last_name || ' ' || PN.suffix) person_name, "
    " PCI.line1, PCI.line2, PCI.city, ST.abbreviation state_name, PCI.zip, coalesce(CH.chart_number, 'No chart available') chart_num "
    "from people P "
    "left join person_names PN on P.person_names_id = PN.id "
    "left join person_contact_info PCI on PCI.id = coalesce(P.home_address_id, P.person_contact_info_id) "
    "left join state_types ST on PCI.state_id = ST.id "
    "left join charts CH on CH.id = "
    " (select first 1 charts.id from charts where charts.current_id >= 3 and charts.person_id = P.id and "
    "  charts.location_id = " + lookupTables->IdOf("location_types", "Contract Health") + " "
    "  order by charts.last_update_stamp desc) "
    "where P.id = '" + person_id + "' ";
  if(!SqlToMap(dbDatabase, person_sql, person_data))
    return false;

  // Get User Information (user_id in form_letters entry):
  // * User name
  // * User position
  ++checkpoint; // 3
  string user_sql =
    "select users.item, users.title from users where id = '" + user_id + "' ";
  if(!SqlToMap(dbDatabase, user_sql, user_data))
    return false;

  // Get Department Information (sending_group_id in form_letters record):
  // * tax entity name
  // * group or tax entity phone number
  // * group or tax entity fax number
  // * tax entity address, city, state, zip
  // TODO -c DUPLICATION: taken from reportDiabAppointmentLetter, mostly verbatim
  ++checkpoint; // 4
  string dept_sql =
    "select tax_entities.name, "
    "  coalesce(groups.phone_number, tax_addresses.phone_number) phn_no, " // jhaley - allow group-specific Phone #
    "  coalesce(groups.fax_number, tax_addresses.fax_number) fax_no, "     //        - and fax # as well
    "  coalesce(tax_addresses.line1, '') || ' ' || coalesce(tax_addresses.line2, '') address, "
    "  tax_addresses.city, "
    "  state_types.abbreviation state, "
    "  tax_addresses.zip, "
    "  groups.item "
    "from groups "
    "left join tax_entities on groups.facility_id = tax_entities.id "
    "left join tax_addresses on tax_addresses.id = "
    "  case when tax_entities.physical_address_id > 0 then "
    "    tax_entities.physical_address_id "
    "  else "
    "    tax_entities.billing_address_id "
    "  end "
    "left join state_types on tax_addresses.state_id = state_types.id "
    "where groups.id = " + group_id + " ";
  if(!SqlToMap(dbDatabase, dept_sql, dept_info))
    return false;

  // Get Letter Items in TIBDataSet
  // * Ordinals (order by field)
  // * Text
  ++checkpoint; // 5
  string items_sql =
    "select (ordinal || '.') ordinal_text , item_text "
    "from form_letter_items "
    "where form_letter_id = '" + form_letter_id + "' "
    "order by ordinal ";

  InitDataSet(dbLetterItemsSet, items_sql);

  // Create letter form
  ++checkpoint; // 6
  TformReportFormLetter *letter_form;
  FORM_TRY
    letter_form = new TformReportFormLetter(NULL, this);
  FORM_CATCH_RETURN("TformReportFormLetter", false)

  // Setup Report Controls
  ++checkpoint; // 7
  letter_form->reportForm->DataSet = dbLetterItemsSet;

  // Page Header Band

  // Address control in upper-right-hand corner (TODO -c DUPLICATION: stolen from TformReportDiabAppointmentLetter)
  letter_form->txtAstInfo->Lines->Clear();
  letter_form->txtAstInfo->Lines->Add(FormatAddress(dept_info["address"]).c_str());
  letter_form->txtAstInfo->Lines->Add((PrettyString(dept_info["city"]) + ", "
                                      + UppercaseString(dept_info["state"]) + "  "
                                      + dept_info["zip"]).c_str());
  letter_form->txtAstInfo->Lines->Add((FormatPhoneNumber(dept_info["phn_no"]) + " phone").c_str());
  letter_form->txtAstInfo->Lines->Add((FormatPhoneNumber(dept_info["fax_no"]) + " fax").c_str());

  letter_form->lblDate->Caption = letter_date.c_str();
  letter_form->lblPersonName->Caption = PrettyString(CollapseSpaces(person_data["person_name"])).c_str();

  string city_state_zip = PrettyString(person_data["city"]) + ", " + person_data["state_name"] + "  " + person_data["zip"];
  if(person_data["line1"] != "")
    letter_form->memoPersonAddress->Lines->Add(FormatAddress(person_data["line1"]).c_str());
  if(person_data["line2"] != "")
    letter_form->memoPersonAddress->Lines->Add(FormatAddress(person_data["line2"]).c_str());
  if(city_state_zip != "")
    letter_form->memoPersonAddress->Lines->Add(city_state_zip.c_str());

  if(form_letter_data["title_text"] != "")
    letter_form->lblTitleText->Caption = form_letter_data["title_text"].c_str();
  else
  {
    letter_form->lblTitleText->Caption = "";
    letter_form->lblTitleText->Enabled = false;
    letter_form->memoHeaderText->Top = letter_form->memoHeaderText->Top - 8;
  }

  letter_form->memoHeaderText->Lines->Text = DoVariableSubstitutions(form_letter_data["header_text"]).c_str();

  // Detail Band
  SetupQRDBText(letter_form->txtItemNo,   dbLetterItemsSet, "ordinal_text");
  SetupQRDBText(letter_form->txtItemText, dbLetterItemsSet, "item_text");

  if(dbLetterItemsSet->Eof == true)
  {
    letter_form->txtItemNo->Enabled = false;
    letter_form->txtItemText->Enabled = false;
    letter_form->bandDetail->Height = 0;

    letter_form->bandSummary->Top     = letter_form->bandPageHeader->Top + letter_form->bandPageHeader->Height;
    letter_form->bandFooterChild->Top = letter_form->bandSummary->Top + letter_form->bandSummary->Height;
  }

  // Page Footer Band
  if(form_letter_data["footer_text"] != "")
    letter_form->memoFooterText->Lines->Text = DoVariableSubstitutions(form_letter_data["footer_text"]).c_str();
  else
  {
    letter_form->memoFooterText->Enabled = false;
    letter_form->bandSummary->Height = 0;
    letter_form->bandFooterChild->Top = letter_form->bandSummary->Top;
  }

  // Footer Child Band
  letter_form->lblUserName->Caption = user_data["item"].c_str();

  // user position, or department name, depending on the letter's option
  if(form_letter_data["use_dept_name"] == "1")
    letter_form->lblUserPosition->Caption = PrettyString(dept_info["item"]).c_str();
  else
    letter_form->lblUserPosition->Caption = user_data["title"].c_str();

  letter_form->memoEnclosures->Lines->Text = DoVariableSubstitutions(form_letter_data["enclosures_text"]).c_str();
  letter_form->lblChartNo->Caption         = person_data["chart_num"].c_str();

  // Insert a CHS Mail Task
  string msg =
    "Printed a " + string(cboLetterType->Text.c_str()) + " form letter for " + person_data["person_name"];
  if(!InsertCHSMailTask(NULL, person_id, msg))
    PSay("Failed to log form letter mailout task", m_warning, MB_OK);

  // Preview it!
  ++checkpoint; // 8
  letter_form->reportForm->Preview();

  return true;
}
//---------------------------------------------------------------------------
void __fastcall TformLaunchFormLetter::btnPersonSearchClick(TObject *Sender)
{
  vector<string> return_vector;

  TformSearchPerson *formPersonSearch = NULL;
  FORM_TRY
    formPersonSearch = new TformSearchPerson(NULL, &return_vector);
  FORM_CATCH_RETURN("TformSearchPerson", )

  formPersonSearch->ShowTaskModal(this);

  if (!return_vector.empty())
    CreateLink(txtPerson, btnPersonSearch, this, return_vector[0], "people");
}
//---------------------------------------------------------------------------

