//---------------------------------------------------------------------------

#include <vcl.h>
#pragma hdrstop

#include "launchGeneric.h"
//---------------------------------------------------------------------------
#pragma package(smart_init)
#pragma link "codeSpecialGeneric"
#pragma resource "*.dfm"
#include <Qrctrls.hpp>
#include <QuickRpt.hpp>
#include <QRCtrls.hpp>
TformLaunchGeneric *formLaunchGeneric;
//---------------------------------------------------------------------------
__fastcall TformLaunchGeneric::TformLaunchGeneric(TComponent* Owner)
        : TformSpecialGeneric(Owner)
{
}
//---------------------------------------------------------------------------
void __fastcall TformLaunchGeneric::btnRunReportClick(TObject *Sender)
{
  if (IsValidOnScreen())
  { // set up global transaction
    report_transaction = NULL;
    report_transaction = new TIBTransaction (NULL);
    StdTransaction(report_transaction, dbDatabase);
    // reset global checkpoint counter
    checkpoint = -1;
    checkpoint_branch = "";
    extra_error_info = "";
    // reset run status
    run_status = lg_not_running;

    Processing(true);
    try
    {
      try
      {
        run_status = lg_running;
        report_start_time = Now();
        if (BeforeRunReport())
        {
          bool run_success = false;
          btnRunReport->Enabled = false;   // jhaley 20110124: Disable toolbar buttons while running a report - no reentrancy!
          btnCloseScreen->Enabled = false;
          AV_TRY
            run_success = RunReport();
          AV_CATCH
          if (run_success)
            run_status = lg_run_success;  // yay!
          else
            run_status = lg_run_failure;  // boo!
        }
        else
          run_status = lg_run_failure;    // hiss!
      }
      catch (...)
      {
        run_status = lg_run_failure;      // *rotten tomato lobbing*
      }
    }
    __finally
    {
      Processing(false);
      report_stop_time = Now();

      // AfterRunReport is executed before Cleanup so Cleanup can't muck up checkpoints
      // before After has a chance to report where the error occurred.  Yes, it's still
      // possible to override After, muck up checkpoints and THEN call the ancestor's
      // After, but that's a bit more obvious of a problem looking at your descendent's code
      AfterRunReport();

      Cleanup();

      if (report_transaction)
      {
        if(report_transaction->Active)
        {
          if(run_status==lg_run_success)
            report_transaction->Commit();
          else
            report_transaction->Rollback();
        }
        delete report_transaction;
      }
      // should we leave run_status as is, or set it back to not_running?  It doesn't
      // matter within this function, but what might others expect?
      // run_status = lg_not_running;

      // jhaley 20110124: Re-enable toolbar buttons (keep this in __finally and hopefully it *always* runs)
      btnRunReport->Enabled = true;
      btnCloseScreen->Enabled = true;
    }
  }
}
//---------------------------------------------------------------------------
void __fastcall TformLaunchGeneric::btnSaveClick(TObject *Sender)
{
  SaveReportCriteria();
}
//---------------------------------------------------------------------------
void __fastcall TformLaunchGeneric::btnLoadClick(TObject *Sender)
{
  LoadReportCriteria();
}
//---------------------------------------------------------------------------
void __fastcall TformLaunchGeneric::btnCloseScreenClick(TObject *Sender)
{
  CloseReport();
}
//---------------------------------------------------------------------------
bool TformLaunchGeneric::IsValidOnScreen()     // screen validation
{
  return true; // override me
}
//---------------------------------------------------------------------------
bool TformLaunchGeneric::BeforeRunReport()
{ // if you override me, remember to call your ancestor!

  // Go ahead and insert a row into reports_run indicating start time, who's running it,
  // and what options they're running it with, and commit and start the transaction again.

  // Then in AfterRunReport we'll update that row with stop time, etc.  This way we'll
  // easily be able to query what reports were taking so long the user just killed P
  // rather than waiting on it to finish.
  Processing(true);

  return true;
}
//---------------------------------------------------------------------------
bool TformLaunchGeneric::RunReport()
{
  return true; // override me
}
//---------------------------------------------------------------------------
bool TformLaunchGeneric::AfterRunReport()
{ // if you override me, remember to call your ancestor!
  Processing(false);

  if (run_status == lg_run_failure)
  {
    // -1 is what we set it to and indicates to us that they're not actually using the checkpoint-tracking
    // functionality, so we say N/A.  Alternately we could just omit the second line, but ... eh.
    AnsiString last_checkpoint = "N/A";
    if (checkpoint > -1) last_checkpoint = IntToString(checkpoint).c_str();
    if (checkpoint_branch != "") last_checkpoint = checkpoint_branch + "-" + last_checkpoint;
    ::PSay(("There was an error running the report.\n\nLast checkpoint reached: " + last_checkpoint).c_str(), m_database_error, MB_OK, "TformLaunchGeneric::AfterRunReport");
  }
  FreeDataSets(); // jhaley
  return true; // override me
}
//---------------------------------------------------------------------------
bool TformLaunchGeneric::SaveReportCriteria()
{
  return true; // override me
}
//---------------------------------------------------------------------------
bool TformLaunchGeneric::LoadReportCriteria()
{
  return true; // override me
}
//---------------------------------------------------------------------------
// jhaley: Do *not* override this to do resource release! It is only appropriate for this to be called from
// btnCloseScreenClick, and that is the only time it *is* called. If the form closes in any other way, all resources
// released by overriding this function will be leaked. Use the form's destructor instead.
bool TformLaunchGeneric::CloseReport()
{
  Close();
  return true; // will it even reach this point? - jhaley: VCL doesn't long-jump out of event handlers, so in fact, yes.
}
//---------------------------------------------------------------------------
void TformLaunchGeneric::Cleanup()
{
  extra_error_info = "";
  bool close_transaction_when_done = false;
  if (!report_transaction->Active)
  {
    report_transaction->StartTransaction();
    close_transaction_when_done = true;
  }

  try
  {
    // and what do we do with the returned bool, exactly?
    DoCleanup();
  }
  catch (...)
  {
    if (extra_error_info == "")
      PPretendToSay("Error caught in launchGeneric::Cleanup()", m_program_error);
    else
      PSay("Error during Cleanup:\n\n" + extra_error_info, m_program_error, MB_OK);
  }
}
//---------------------------------------------------------------------------
bool TformLaunchGeneric::DoCleanup()
{
  return true;
}
//---------------------------------------------------------------------------
void __fastcall TformLaunchGeneric::FormShortCut(TWMKey &Msg, bool &Handled)
{
  Handled = true;
  if (Msg.CharCode == VK_RETURN)
    btnRunReport->Click();
  else
    Handled = false;

  if (!Handled)
    TformSpecialGeneric::FormShortCut(Msg, Handled);
}
//---------------------------------------------------------------------------
void __fastcall TformLaunchGeneric::SetCheckpointBranch(AnsiString Value)
{
  // TODO: write to database
  Fcheckpoint_branch = Value;
}
//---------------------------------------------------------------------------
void __fastcall TformLaunchGeneric::SetCheckpoint(int Value)
{
  // TODO: write to database
  Fcheckpoint = Value;
}
//---------------------------------------------------------------------------
// jhaley 20110110: Data set management routines
// Add a managed dataset.
TIBDataSet *TformLaunchGeneric::AddDataSet()
{
  TIBDataSet *dbDataset = new TIBDataSet(NULL);
  data_sets.push_back(dbDataset);
  return dbDataset;
}
//---------------------------------------------------------------------------
// Initialize a dataset
void TformLaunchGeneric::InitDataSet(TIBDataSet *dataSet, const string &sql)
{
  dataSet->Database = dbDatabase;
  dataSet->Transaction = report_transaction;
  dataSet->UniDirectional = false;
  dataSet->SelectSQL->Clear();
  dataSet->SelectSQL->Add(sql.c_str());
  dataSet->Open();
  dataSet->First();
}
//---------------------------------------------------------------------------
// Free all datasets. This is called from AfterRunReport
void TformLaunchGeneric::FreeDataSets()
{
  for (vector<TIBDataSet *>::iterator i_ds = data_sets.begin(); i_ds != data_sets.end(); ++i_ds)
  {
    TIBDataSet *ds = *i_ds;
    delete ds;
  }
  data_sets.clear();
}
//---------------------------------------------------------------------------
// Utility - Setup a TQRDBText control
void TformLaunchGeneric::SetupQRDBText(TControl *txtControl, TIBDataSet *dataSet, const string &name)
{
  // I have to do this conditional downcast because TQRDBText isn't recognized as a valid class in the header for this
  // class, and adding the QuickReport headers there seems to make things go bonkers.
  if (txtControl->ClassType() == __classid(TQRDBText))
  {
    TQRDBText *qrdbText = (TQRDBText *)txtControl;
    qrdbText->DataSet   = dataSet;
    qrdbText->DataField = name.c_str();
  }
}
//---------------------------------------------------------------------------
void __fastcall TformLaunchGeneric::FormClose(TObject *Sender, TCloseAction &Action)
{
  // jhaley 20110124: Do not allow 'X' button to close the form while a report is running. This is distinguishable
  // by testing for Sender == this. When the form is closed externally, this condition is fortunately not true (we don't
  // want to refuse closing when the main menu form asks us to close, for example).
  if (Sender == static_cast<TObject *>(this) && btnCloseScreen->Enabled == false)
    Action = caNone;
  else
    TformSpecialGeneric::FormClose(Sender, Action);
}
//---------------------------------------------------------------------------

