//---------------------------------------------------------------------------

#include <vcl.h>
#pragma hdrstop

#include "launchInsuranceChargeAnalysisReport.h"
//---------------------------------------------------------------------------
#pragma package(smart_init)
#pragma link "launchGeneric"
#pragma resource "*.dfm"
TformLaunchInsuranceChargeAnalysisReport *formLaunchInsuranceChargeAnalysisReport;
//---------------------------------------------------------------------------
__fastcall TformLaunchInsuranceChargeAnalysisReport::TformLaunchInsuranceChargeAnalysisReport(TComponent* Owner)
        : TformLaunchGeneric(Owner)
{
  Pdate today_date(Date());
  Pdate begin_date = today_date;
  begin_date.day   = 1;           // first day of this month
  Pdate end_date = begin_date;    // first day of this month
  begin_date.mod(-1, in_months);  // first day of last month
  end_date.mod(-1, in_days);      // last day of last month

  dateBegin->Date = begin_date.OutDate();
  dateEnd->Date   = end_date.OutDate();

  // jhaley: get departments with view permissions for the tasks attached to the bills
  set<string> group_perms = GetGroupPermissions("table.tasks", "", "view", "true");
  string group_perm_sql = "select id, item, valid from groups where groups.id > 0 and " + SafeInClause("groups.id", SetToVec(group_perms));

  lookupTables->RegisterComboBox("groups", cboDepartment, group_perm_sql);
}
//---------------------------------------------------------------------------
// jhaley 20101119: CloseReport *cannot* be used as resource release because it's only an event handler for the Close button
// on TformLaunchGeneric (it really shouldn't even be virtual). Use the class destructor instead.
__fastcall TformLaunchInsuranceChargeAnalysisReport::~TformLaunchInsuranceChargeAnalysisReport()
{
  if (lookupTables->TableName(cboDepartment) == "groups") // Only if initialized successfully
    lookupTables->UnregisterComboBox("groups", cboDepartment);
}
//---------------------------------------------------------------------------
bool TformLaunchInsuranceChargeAnalysisReport::IsValidOnScreen()
{
  if (dateBegin->Date > dateEnd->Date)
  {
    PSay("End date must be greater than or equal to begin date.", m_invalid_data, MB_OK);
    dateEnd->SetFocus();
    return false;
  }
  else if (!lookupTables->IsInLookUpTable("groups", cboDepartment))
  {
    PSay("Please enter a valid department", m_invalid_data, MB_OK);
    cboDepartment->SetFocus();
    return false;
  }

  return true;
}
//---------------------------------------------------------------------------
bool TformLaunchInsuranceChargeAnalysisReport::BeforeRunReport()
{
  dbDataset = NULL;
  dbDataset = new TIBDataSet(NULL);
  return TformLaunchGeneric::BeforeRunReport();
}
//---------------------------------------------------------------------------
// jhaley 20100915: utility method
void TformLaunchInsuranceChargeAnalysisReport::PutNewLabel(TQRBand *parent_band, TQRDBText *make_like, int top, string caption)
{
  TQRLabel *new_label;
  FORM_TRY
    new_label = new TQRLabel(parent_band);
  FORM_CATCH

  // copy important characteristics from the "make_like" QRDBText control:
  new_label->Font = make_like->Font;
  new_label->Left = make_like->Left;
  new_label->Width = make_like->Width;
  new_label->Height = make_like->Height;
  new_label->Alignment = make_like->Alignment;

  // set other characteristics to constant values or passed-in parameters:
  new_label->AutoSize = false;
  new_label->Caption = caption.c_str();
  new_label->Parent = parent_band;
  new_label->Top = top;
  new_label->Transparent = true;
}
//---------------------------------------------------------------------------
bool TformLaunchInsuranceChargeAnalysisReport::RunReport()
{
  string begin_date = Pdate(dateBegin->Date).OutDate(mdy | yyyy);
  string end_date   = Pdate(dateEnd->Date).OutDate(mdy | yyyy);
  string date_title = begin_date + " - " + end_date;
  string group_id   = lookupTables->IdOf("groups", cboDepartment);

  try
  {
    // drop everything from the temp_ica_report table before starting
    if (!ExecuteStatement(report_transaction, "delete from temp_ica_report"))
      throw(1);

    // Query to insert items into the ICA temp table
    string insert_sql = "insert into temp_ica_report (insurance_item, total_charge, tax_entity_name) "
                        "select "
                        "case "
                        "when upper(insurance_types.item) like '%MEDICARE%' then 'MEDICARE' "
                        "when upper(insurance_types.item) like '%MEDICAID%' then 'MEDICAID' "
                        "else 'PRIVATE' "
                        "end, bills.total_charge, tax_entities.name "
                        "from bills "
                        "inner join tasks on bills.task_id = tasks.id "
                        "inner join insurance_coverage on bills.insurance_coverage_id = insurance_coverage.id "
                        "inner join insurance_policies on insurance_coverage.insurance_policy_id = insurance_policies.id "
                        "inner join insurance_types on insurance_policies.insurance_id = insurance_types.id "
                        "inner join tax_entities on bills.provider_tax_id = tax_entities.id "
                        "where "
                        "tasks.date_of_task between '" + begin_date + "' and '" + end_date + "' and "
                        "bills.current_id+1 > 3 and "
                        "tasks.group_id = '" + group_id + "' ";

    // Do the insert
    if (!ExecuteStatement(report_transaction, insert_sql))
      throw(2);

    // Get grand totals for charges and bills
    map<string, string> totals_map;
    if (!SqlToMap(report_transaction, "select sum(total_charge) grand_total_charges, count(id) grand_total_bills from temp_ica_report", totals_map))
      throw(3);

    // Do the report
    TformReportInsuranceChargeAnalysis *report_form = NULL;
    FORM_TRY
      report_form = new TformReportInsuranceChargeAnalysis(NULL);
    FORM_CATCH

    dbDataset->Database = dbDatabase;
    dbDataset->Transaction = report_transaction;
    dbDataset->UniDirectional = false;

    // Main report query - Group data from temp table by department
    string report_sql;
    if(!IsBlankOrZero(totals_map["grand_total_bills"]) && !IsBlankOrZero(totals_map["grand_total_charges"])) // don't divide by zero or try to use null/empty strings
    {
      report_sql = "select insurance_item, sum(total_charge) total_charges, (sum(total_charge) * 100 / " + totals_map["grand_total_charges"] + ") pct_charges, "
                   "count(id) total_bills, (count(id) * 100 / " + totals_map["grand_total_bills"] + ") pct_bills "
                   "from temp_ica_report "
                   "group by insurance_item ";
    }
    else
    {
      report_sql = "select insurance_item, 0 as total_charges, 0 as pct_charges, 0 as total_bils, 0 as pct_bills "
                   "from temp_ica_report "
                   "group by insurance_item ";
    }

    dbDataset->SelectSQL->Clear();
    dbDataset->SelectSQL->Add(report_sql.c_str());
    dbDataset->Open();
    dbDataset->First();

    report_form->analysisReport->DataSet = dbDataset;
    report_form->lblDateTitle->Caption = date_title.c_str();  // display dates on title band
    report_form->lblUser->Caption = current_fullname.c_str(); // display generating user's name on page footer
    report_form->lblDepartmentTitle->Caption = cboDepartment->Text; // show department name

    // detail band controls
    report_form->txtDetailInsurance->DataSet = dbDataset;
    report_form->txtDetailInsurance->DataField = "insurance_item";
    report_form->txtDetailDollarsBilled->DataSet = dbDataset;
    report_form->txtDetailDollarsBilled->DataField = "total_charges";
    report_form->txtDetailPercentBilled->DataSet = dbDataset;
    report_form->txtDetailPercentBilled->DataField = "pct_charges";
    report_form->txtDetailNumClaims->DataSet = dbDataset;
    report_form->txtDetailNumClaims->DataField = "total_bills";
    report_form->txtDetailPercentClaims->DataSet = dbDataset;
    report_form->txtDetailPercentClaims->DataField = "pct_bills";

    // summary band - manually create a lot of TQRLabels and populate them with information grouped by provider
    string provider_sql;
    if(!IsBlankOrZero(totals_map["grand_total_bills"]) && !IsBlankOrZero(totals_map["grand_total_charges"]))
    {
      provider_sql = "select tax_entity_name, sum(total_charge) total_charges, (sum(total_charge) * 100 / " + totals_map["grand_total_charges"] + ") pct_charges, "
                     "count(id) total_bills, (count(id) * 100 / " + totals_map["grand_total_bills"] + ") pct_bills "
                     "from temp_ica_report "
                     "group by tax_entity_name ";
    }
    else
    {
      provider_sql = "select tax_entity_name, 0 as total_charges, 0 as pct_charges, 0 as total_bills, 0 as pct_bills "
                     "from temp_ica_report "
                     "group by tax_entity_name ";
    }

    vector<map<string, string> > provider_vecmap;
    if(SqlToVecMap(report_transaction, provider_sql, provider_vecmap))
    {
      int current_top = report_form->lblProvider->Top + report_form->lblProvider->Height + 7;
      int row_height = report_form->bandDetail->Height;
      for(vector<map<string, string> >::iterator it_provider = provider_vecmap.begin(); it_provider != provider_vecmap.end(); it_provider++)
      {
        PutNewLabel(report_form->bandSummary, report_form->txtDetailInsurance,     current_top, (*it_provider)["tax_entity_name"]);
        PutNewLabel(report_form->bandSummary, report_form->txtDetailDollarsBilled, current_top, FormatString((*it_provider)["total_charges"], dollarString));
        PutNewLabel(report_form->bandSummary, report_form->txtDetailPercentBilled, current_top, FloatToString(StringToFloat((*it_provider)["pct_charges"]), 2) + "%");
        PutNewLabel(report_form->bandSummary, report_form->txtDetailNumClaims,     current_top, (*it_provider)["total_bills"]);
        PutNewLabel(report_form->bandSummary, report_form->txtDetailPercentClaims, current_top, ((*it_provider)["pct_bills"]) + "%");

        report_form->bandSummary->Height += row_height;
        current_top += row_height;
      }
    }

    // summary child band
    report_form->lblSummaryChildDollarsBilled->Caption = FormatString(totals_map["grand_total_charges"], dollarString).c_str();
    report_form->lblSummaryChildNumClaims->Caption = totals_map["grand_total_bills"].c_str();

    Processing(false);
    report_form->analysisReport->Preview();

    // drop everything from the temp_ica_report table after done
    ExecuteStatement(report_transaction, "delete from temp_ica_report");
  }
  catch(int sql_statement)
  {
    Processing(false);
    PSay("Error running report: database error #" + IntToString(sql_statement), m_database_error, MB_OK);
  }
  catch(...)
  {
    Processing(false);
    PSay("Error running report", m_database_error, MB_OK);
  }

  dbDataset->Close();

  return true;
}
//---------------------------------------------------------------------------
bool TformLaunchInsuranceChargeAnalysisReport::AfterRunReport()
{
  if (dbDataset)
  {
    delete dbDataset;
    dbDataset = NULL;
  }

  return TformLaunchGeneric::AfterRunReport();
}
//---------------------------------------------------------------------------
// jhaley: Nope, can't do this!
/*
bool TformLaunchInsuranceChargeAnalysisReport::CloseReport()
{
  lookupTables->UnregisterComboBox("groups", cboDepartment);
  return TformLaunchGeneric::CloseReport();
}
*/
