//---------------------------------------------------------------------------

#include <vcl.h>
#pragma hdrstop

#include "launchIntakeFormReport.h"
#include "codeSearchPerson.h"
#include "codeRecordPerson.h"
#include "reportIntakeForm.h"
#include "derivedheaders.h"
//---------------------------------------------------------------------------
#pragma package(smart_init)
#pragma link "codeSpecialGeneric"
#pragma resource "*.dfm"
TformLaunchIntakeFormReport *formLaunchIntakeFormReport;
//---------------------------------------------------------------------------
__fastcall TformLaunchIntakeFormReport::TformLaunchIntakeFormReport(TComponent* Owner)
        : TformSpecialGeneric(Owner)
{
  restrict_relationships = true;
  hide_signature = false;
}
//---------------------------------------------------------------------------
void __fastcall TformLaunchIntakeFormReport::btnCloseClick(TObject *Sender)
{
  Close();
}
//---------------------------------------------------------------------------
void __fastcall TformLaunchIntakeFormReport::btnRunClick(TObject *Sender)
{
  restrict_relationships = cboOnlyEmpAndEmer->Checked;
  if (!person_ids.empty() || !IsBlankOrZero(SelectedPersonId))
  {
    person_ids.clear();
    person_ids.push_back(SelectedPersonId);
    Run();
  }
  else
    PSay("You must select a person for whom to print an intake form.", m_missing_data, MB_OK);
}
//---------------------------------------------------------------------------
void __fastcall TformLaunchIntakeFormReport::lblNameClick(TObject *Sender)
{
  if (!IsBlankOrZero(SelectedPersonId))
  {
    map<string, string> t;
    t["id"] = SelectedPersonId;
    TformRecordPerson::Do(t);
  }
}
//---------------------------------------------------------------------------
void __fastcall TformLaunchIntakeFormReport::SetSelectedPersonId(string Value)
{
  //selected_person_id = Value;
  CreateLink(txtName, btnPersonSearch, this, Value, "people");
}
//---------------------------------------------------------------------------
// jhaley 20101028: Added to handle getting ID from link rather than storing it locally.
string __fastcall TformLaunchIntakeFormReport::GetSelectedPersonId()
{
  return GetLinkId(txtName); // Use GetLinkId to return the exact contents, since I assume that's what user code expects to be returned.
}
//---------------------------------------------------------------------------
void __fastcall TformLaunchIntakeFormReport::btnPersonSearchClick(TObject *Sender)
{
  vector<string> return_vector = TformSearchPerson::Do(this);
  if (! return_vector.empty())
    SelectedPersonId = return_vector[0];
  // if they cancel, it's blank. if they select nothing, it's 0.
}
//---------------------------------------------------------------------------
void TformLaunchIntakeFormReport::DoSilent(TComponent * _modal_parent, string _person_id, bool _restrict_relationships, bool _hide_signature)
{
  // TODO -c DUPLICATION: jhaley - combine with the code below.
  TformLaunchIntakeFormReport *z = NULL;
  FORM_TRY
    z = new TformLaunchIntakeFormReport(_modal_parent);
  FORM_CATCH_RETURN("TformLaunchIntakeFormReport", )

  try
  {
    z->person_ids.clear();
    z->person_ids.push_back(_person_id);
    z->SelectedPersonId = _person_id; // just for kicks?
    z->restrict_relationships = _restrict_relationships;
    z->hide_signature = _hide_signature;
    z->Run();
    z->Close();
  }
  catch(EAccessViolation &av) // jhaley 20110830: DEBUG - Something might be haywire in the above.
  {
    LogIt("launchintake.log", "Caught access violation while setting up form in DoSilent(1)");
  }
  catch(Exception &ex)
  {
    LogIt("launchintake.log", string("Caught an exception while setting up form in DoSilent(1): ") + ex.Message.c_str());
  }
  catch(...)
  {
    LogIt("launchintake.log", "Caught an unknown type of exception in DoSilent(1)");
  }
}
//---------------------------------------------------------------------------
void TformLaunchIntakeFormReport::DoSilent(TComponent * _modal_parent, string _person_id, string blah, bool _restrict_relationships, bool _hide_signature)
{
  DoSilent(_modal_parent, _person_id, _restrict_relationships, _hide_signature);
}
//---------------------------------------------------------------------------
void TformLaunchIntakeFormReport::DoSilent(TComponent * _modal_parent, vector<string> _person_ids, bool _restrict_relationships, bool _hide_signature)
{
  TformLaunchIntakeFormReport *z = NULL;
  FORM_TRY
    z = new TformLaunchIntakeFormReport(_modal_parent);
  FORM_CATCH_RETURN("TformLaunchIntakeFormReport", );

  try
  {
    z->person_ids = _person_ids;
    z->SelectedPersonId = "";
    z->restrict_relationships = _restrict_relationships;
    z->hide_signature = _hide_signature;
    z->Run();
    z->Close();
  }
  catch(EAccessViolation &av) // jhaley 20110830: DEBUG - Something might be haywire in the above.
  {
    LogIt("launchintake.log", "Caught access violation while setting up form in DoSilent(2)");
  }
  catch(Exception &ex)
  {
    LogIt("launchintake.log", string("Caught an exception while setting up form in DoSilent(2): ") + ex.Message.c_str());
  }
  catch(...)
  {
    LogIt("launchintake.log", "Caught an unknown type of exception in DoSilent(2)");
  }
}
//---------------------------------------------------------------------------
void TformLaunchIntakeFormReport::DoSilent(TComponent * _modal_parent, vector<string> _person_ids, string blah, bool _restrict_relationships, bool _hide_signature)
{
  DoSilent(_modal_parent, _person_ids, _restrict_relationships, _hide_signature);
}
//---------------------------------------------------------------------------
void __fastcall TformLaunchIntakeFormReport::Run()
{
  Processing(true);

  dbTransaction = new TIBTransaction(NULL);
  dataSetPerson = new TIBDataSet(NULL);
  dataSetAddresses = new TIBDataSet(NULL);
  dataSetRelationships = new TIBDataSet(NULL);
  dataSetInsurance = new TIBDataSet(NULL);
  dataSetExtraInsurance = new TIBDataSet(NULL);

  TQRPageList * source = new TQRPageList();
  TQRPageList * destination = new TQRPageList();
  destination->Stream = new TQRStream(100000);
  TQRPrinter * printer_thing = InitTQRPrinter();

  // Create report form
  FORM_TRY
    personIntakeForm = new TformIntakeReport(NULL);
  FORM_CATCH_RETURN("TformIntakeReport", )

  int report_stage = 0; // jhaley 20110830: Added for debugging
  try
  {
    report_stage = 1;
    dbTransaction->DefaultDatabase = dbDatabase;
    dbTransaction->StartTransaction();
    personIntakeForm->dbTransaction = dbTransaction;

    for (int i = 0; i < person_ids.size(); i++)
    {
      dataSetPerson->Database = dbDatabase;
      dataSetPerson->Transaction = dbTransaction;
      dataSetPerson->UniDirectional = false;

      dataSetAddresses->Database = dbDatabase;
      dataSetAddresses->Transaction = dbTransaction;
      dataSetAddresses->UniDirectional = false;

      dataSetRelationships->Database = dbDatabase;
      dataSetRelationships->Transaction = dbTransaction;
      dataSetRelationships->UniDirectional = false;

      dataSetInsurance->Database = dbDatabase;
      dataSetInsurance->Transaction = dbTransaction;
      dataSetInsurance->UniDirectional = false;

      dataSetExtraInsurance->Database = dbDatabase;
      dataSetExtraInsurance->Transaction = dbTransaction;
      dataSetExtraInsurance->UniDirectional = false;

      current_person_id = person_ids[i];
      // prepare composite report
      compositeReport->Prepare();
      string z = PMakeTempFilePath(string("page") + IntToString(i) + ".qrp");
      ((TQuickRep *)(compositeReport->Reports->First()))->QRPrinter->Save(z.c_str());

      dataSetPerson->Close();
      dataSetAddresses->Close();
      dataSetRelationships->Close();
      dataSetInsurance->Close();
      dataSetExtraInsurance->Close();
    }

    report_stage = 2;
    for (int i = 0; i < person_ids.size(); i++)
    {
      string z = PMakeTempFilePath(string("page") + IntToString(i) + ".qrp");
      source->Clear();
      source->LoadFromFile(z.c_str());
      for (int i = 1; i <= source->PageCount; i++)
        destination->AddPage(source->GetPage(i));
    }

    report_stage = 3;
    // combine pages
    destination->Finish();
    string temp_filepath = PMakeTempFilePath("complete.qrp");
    destination->SaveToFile(temp_filepath.c_str());
    // load combined report and preview
    printer_thing->Load(temp_filepath.c_str());

    report_stage = 4;
    // preview composite report
    Processing(false);
    printer_thing->PreviewModal();

    report_stage = 5;
    if (!personIntakeForm->CloseNow())
      PSay("Unable to close person intake print form (reportIntakeForm)", m_program_error, MB_OK);

    // shouldn't be needed, but doesn't cause a problem either ... meh.
    /*
    dataSetPerson->Close();
    dataSetAddresses->Close();
    dataSetRelationships->Close();
    dataSetInsurance->Close();
    dataSetExtraInsurance->Close();
    */
    dbTransaction->Commit();
  }
  catch(...)
  {
    Processing(false);
    PSay("Error running report - reached stage " + IntToString(report_stage), m_program_error, MB_OK);
  }
  delete dbTransaction;
  delete dataSetPerson;
  delete dataSetAddresses;
  delete dataSetRelationships;
  delete dataSetInsurance;
  delete dataSetExtraInsurance;
  delete printer_thing;
  delete source;
  delete destination;
}
//---------------------------------------------------------------------------
void __fastcall TformLaunchIntakeFormReport::compositeReportAddReports(TObject *Sender)
{
  try
  {
    // setup sql for datasets

    // sql for name dataset
    string sql;

    sql = "select id from people where id = " + current_person_id;
    dataSetPerson->SelectSQL->Clear();
    dataSetPerson->SelectSQL->Add(sql.c_str());

    // sql for address dataset
    sql = "select person_contact_info.line1, person_contact_info.line2, "
          "person_contact_info.city, state_types.item state, person_contact_info.zip, country_types.item country, "
          "person_contact_info.phone_number, person_contact_info.current_id, person_contact_info.trust_lands, contact_info_types.item addr_type " // jhaley: added trust lands
          "from person_contact_info "
          "left join state_types on state_types.id = person_contact_info.state_id "
          "left join country_types on country_types.id = person_contact_info.country_id "
          "left join contact_info_types on contact_info_types.id = person_contact_info.contact_info_id "
          "where "
          "person_contact_info.current_id >= 3 and person_contact_info.person_id = " + current_person_id + " ";
    vector<map<string, string> > pre_adds;
    SqlToVecMap(dbTransaction, sql, pre_adds);
    if (pre_adds.size() < 1)
    {
      sql += "union "
             "select cast('' as varchar(250)) line1, cast('' as varchar(250)) line2, "
             "cast('' as varchar(250)) city, cast('' as varchar(255)) state, cast('' as varchar(250)) zip, cast('' as varchar(255)) country, "
             "cast('' as varchar(250)) phone_number, 0 current_id, 0 trust_lands, cast('' as varchar(255)) addr_type from uno ";      // jhaley: added trust lands
    }
    if (pre_adds.size() < 2)
    {
      sql += "union "
             "select cast('' as varchar(250)) line1, cast('' as varchar(250)) line2, "
             "cast('' as varchar(250)) city, cast('' as varchar(255)) state, cast('' as varchar(250)) zip, cast('' as varchar(255)) country, "
             "cast('' as varchar(250)) phone_number, 1 current_id, 0 trust_lands, cast('' as varchar(255)) addr_type from uno ";       // jhaley: added trust lands
    }
    sql += " order by 8 desc";

    DebugSql(this, &sql, other_forms, 9);
    dataSetAddresses->SelectSQL->Clear();
    dataSetAddresses->SelectSQL->Add(sql.c_str());

    // sql for relationships (stolen from the codeRecordPerson file and then updated for my diabolical purposes :)  muhahahaha
    sql = "select "
          "cast(person_names.first_name || ' ' || person_names.last_name || ' ' || person_names.suffix as varchar(800)) name, "
          "person_contact_info.phone_number, "
          "relationship_types.item relationship_type, "
          "relationships.same_household, "
          "relationships.b_is_emergency is_emergency, "// relationships.a_is_emergency, "
          "relationships.b_is_banned is_banned, " // relationships.a_is_banned, "
          "relationships.b_is_responsible is_responsible " // relationships.a_is_responsible, "
          "from relationships left join people on relationships.person_b_id = people.id "
          "left join person_names on person_names.id = people.person_names_id "
          "left join relationship_types on relationship_types.id = relationships.rel_b_to_a_id "
          "left join person_contact_info on person_contact_info.id = people.person_contact_info_id "
          "where (relationships.person_a_id = '" + current_person_id + "' and "
                "relationships.current_id >= 3 and "
                "relationships.a_is_banned = 0) ";
    // if you add something here you must also add it on the other side of the union
    if(restrict_relationships)
    {
      sql +=  "and "
              "(relationship_types.id = " + lookupTables->IdOf("relationship_types", "EMPLOYER") + " "
              "or relationships.b_is_emergency = 1 "
              "or relationships.b_is_responsible = 1) ";
    }
    sql += "union "
           "select "
           "cast(person_names.first_name || ' ' || person_names.last_name || ' ' || person_names.suffix as varchar(800)) name, "
           "person_contact_info.phone_number, "
           "relationship_types.item relationship_type, "
           "relationships.same_household, "
           "relationships.a_is_emergency is_emergency, " // relationships.b_is_emergency, "
           "relationships.a_is_banned is_banned, " // relationships.b_is_banned, "
           "relationships.a_is_responsible is_responsible " // relationships.b_is_responsible, "
           "from relationships "
           "left join people on relationships.person_a_id = people.id "
           "left join person_names on person_names.id = people.person_names_id "
           "left join relationship_types on relationship_types.id = relationships.rel_a_to_b_id "
           "left join person_contact_info on person_contact_info.id = people.person_contact_info_id "
           "where (relationships.person_b_id = '" + current_person_id + "' and "
              "relationships.current_id >= 3 and "
              "relationships.b_is_banned = 0) ";
    if(restrict_relationships)
    {
      sql +=  "and "
              "(relationship_types.id = " + lookupTables->IdOf("relationship_types", "EMPLOYER") + " "
              "or relationships.b_is_emergency = 1 "
              "or relationships.b_is_responsible = 1) ";
    }

    vector<map<string, string> > pre_rels;
    SqlToVecMap(dbTransaction, sql, pre_rels);
    bool has_mother = false;
    bool has_father = false;
    bool has_emg = false;
    bool has_res = false;
    bool has_emp = false;
    for (vector<map<string, string> >::iterator i = pre_rels.begin(); i != pre_rels.end(); i++)
    {
      if ((*i)["relationship_type"] == "MOTHER")
        has_mother = true;
      if ((*i)["relationship_type"] == "FATHER")
        has_father = true;
      if ((*i)["relationship_type"] == "EMPLOYER")
        has_emp = true;
      if ((*i)["is_emergency"] == "1")
        has_emg = true;
      if ((*i)["is_responsible"] == "1")
        has_res = true;
    }

    if (!has_mother)
      sql += "union select cast('' as varchar(800)) name, cast('' as varchar(250)) phone_number, cast('MOTHER' as varchar(255)) relationship_type, 0, 0, 0, 0 from uno ";
    if (!has_father)
      sql += "union select cast('' as varchar(800)) name, cast('' as varchar(250)) phone_number, cast('FATHER' as varchar(255)) relationship_type, 0, 0, 0, 0 from uno ";
    if (!has_emg)
      sql += "union select cast('' as varchar(800)) name, cast('' as varchar(250)) phone_number, cast('EMERGENCY CONTACT' as varchar(255)) relationship_type, 0, 0, 0, 0 from uno ";
    if (!has_res)
    {
      string age_sql = "select (select p_age from calc_age(p.birth_date)) from people p where p.id = '" + current_person_id + "' ";
      int person_age = StringToInt(GetOneField(dbTransaction, age_sql));

      if(person_age < 18) // jhaley 20110207: is a minor child? cannot be responsible for self
      {
        sql += "union select cast('Unknown' as varchar(800)) name, cast('' as varchar(250)) phone_number, "
               "cast('RESPONSIBLE PARTY' as varchar(255)) relationship_type, 0, 0, 0, 0 from uno ";
      }
      else
      {
        //sql += "union select cast('' as varchar(800)) name, cast('' as varchar(250)) phone_number, cast('RESPONSIBLE PARTY' as varchar(255)) relationship_type, 0, 0, 0, 0 from uno ";
        // By default now, if they don't have a responsible party, they're responsible for themselves.  So sayeth the Jan.  Janice Maxwell, that is.  Black gold.  Texas tea. ... what???
        sql += "union select "
               "cast(person_names.first_name || ' ' || person_names.last_name || ' ' || person_names.suffix || "
               "case when people.gender_id = " + lookupTables->IdOf("gender_types", "Male") + " then ' (himself)' "
                  "when people.gender_id = " + lookupTables->IdOf("gender_types", "Female") + " then ' (herself)' else '' end as varchar(800)) name, "
               "person_contact_info.phone_number, "
               "cast('RESPONSIBLE PARTY' as varchar(255)) relationship_type, "
               "1 same_household, "
               "0 is_emergency, "
               "0 is_banned, "
               "0 is_responsible " // don't need it to say "Responsible party, responsible party" now do we?
               "from people "
               "left join person_names on person_names.id = people.person_names_id "
               "left join person_contact_info on person_contact_info.id = people.person_contact_info_id "
               "where people.id = '" + current_person_id + "' ";
      }
    }
    if (!has_emp)
       sql += "union select cast('' as varchar(800)) name, cast('' as varchar(250)) phone_number, cast('EMPLOYER' as varchar(255)) relationship_type, 0, 0, 0, 0 from uno ";

    DebugSql(this, &sql, other_forms, 9);
    dataSetRelationships->SelectSQL->Clear();
    dataSetRelationships->SelectSQL->Add(sql.c_str());

    string id_veteran      = lookupTables->IdOf("insurance_types", "Veteran");
    string id_medicare_a   = lookupTables->IdOf("insurance_types", "Medicare A");
    string id_medicare_b   = lookupTables->IdOf("insurance_types", "Medicare B");
    string id_medicare_d   = lookupTables->IdOf("insurance_types", "Medicare D");
    string id_medicare_adv = lookupTables->IdOf("insurance_types", "Medicare Advantage");
    string id_medicaid     = lookupTables->IdOf("insurance_types", "Medicaid");

    // sql for insurance dataset
    sql = "select tax_entities.name, "
               "tax_addresses.phone_number, "
               "insurance_types.item insurance_type, "
               "source_types.item source_type, "
               "coverage_types.item coverage_type, "
               "insurance_policies.name_on_policy, "
               "outer_table.name_on_card, " // it's funny how accidentally misplacing this (compared to the UNIONs below) produced "104: datatype unknown" rather than some sort of type-mismatch (varchar(250) vs. varchar(255)) ... grrr. took forever to figure out.
               "holders_address.line1 holder_line1, "
               "holders_address.line2 holder_line2, "
               "holders_address.city holder_city, "
               "holders_state.abbreviation holder_state, "
               "holders_address.zip holder_zip, "
               "holders_address.phone_number holder_phone, "
               "holders_gender.item holder_gender, "
               "policy_holder.birth_date holder_dob, "
               "policy_holder.social_security_number holder_ssn, "
               "insurance_policies.group_number, "
               "insurance_policies.policy_number, "
               "case when outer_table.person_id = policy_holder.id then cast('SELF' as VARCHAR(255)) else relationship_types.item end relationship_type, "
               "insurance_policies.bin_number, "
               "insurance_policies.processor_control_number, "
               "(select tax_entities.name "
                "from tax_entities, insurance_coverage inner_table "
                "where outer_table.id = inner_table.id and "
                "inner_table.tax_entity_id = tax_entities.id and "
                "inner_table.person_id = '" + current_person_id + "') doctor, "
              "insurance_policies.start_date, "
              "insurance_policies.stop_date, "
              "(select count(*) from insurance_benefits where insurance_benefits.area_id = " + /* medical */ lookupTables->IdOf("area_types", "Medical") + " and insurance_benefits.is_covered = 1 and insurance_benefits.insurance_policy_id = insurance_policies.id) is_medical, "
              "(select count(*) from insurance_benefits where insurance_benefits.area_id = " + /* dental */ lookupTables->IdOf("area_types", "Dental") + " and insurance_benefits.is_covered = 1 and insurance_benefits.insurance_policy_id = insurance_policies.id) is_dental, "
              "(select count(*) from insurance_benefits where insurance_benefits.area_id = " + /* vision */ lookupTables->IdOf("area_types", "Vision") + " and insurance_benefits.is_covered = 1 and insurance_benefits.insurance_policy_id = insurance_policies.id) is_vision, "
              "(select count(*) from insurance_benefits where insurance_benefits.area_id = " + /* prescr. */ lookupTables->IdOf("area_types", "Prescription") + " and insurance_benefits.is_covered = 1 and insurance_benefits.insurance_policy_id = insurance_policies.id) is_rx, "
              "outer_table.person_code, "
              "outer_table.current_id "
          "from insurance_coverage outer_table "
          "left join insurance_policies on outer_table.insurance_policy_id = insurance_policies.id "
          "left join tax_entities on insurance_policies.tax_entity_id = tax_entities.id "
          "left join tax_addresses on tax_entities.billing_address_id = tax_addresses.id "
          "left join insurance_types on insurance_policies.insurance_id = insurance_types.id "
          "left join source_types on insurance_policies.source_id = source_types.id "
          "left join coverage_types on insurance_policies.coverage_id = coverage_types.id "
          "left join people policy_holder on policy_holder.id = insurance_policies.policy_holder_id "
          "left join person_contact_info holders_address on holders_address.id = policy_holder.person_contact_info_id "
          "left join state_types holders_state on holders_state.id = holders_address.state_id "
          "left join gender_types holders_gender on holders_gender.id = policy_holder.gender_id "
          "left join relationships rel on rel.id = (select first 1 id from relationships_view rph where rph.person_a_id = outer_table.person_id and rph.person_b_id = policy_holder.id and rph.current_id+1>3) "
          "left join relationship_types on rel.rel_a_to_b_id = relationship_types.id "
          "where outer_table.person_id = '" + current_person_id + "' and "
              "outer_table.current_id >= 3 and insurance_types.id not in "
              "(" + id_veteran + ", " + id_medicare_a + ", " + id_medicare_b + ", " + id_medicare_adv + ", " + id_medicaid + ") ";

    vector<map<string, string> > pre_ins;
    SqlToVecMap(dbTransaction, sql, pre_ins);
    if (pre_ins.size() < 1)
      sql += "union select cast('' as varchar(250)) name, cast('' as varchar(255)) phone_number, "
             "cast('' as varchar(255)) insurance_type, cast('' as varchar(255)) source_type, "
             "cast('' as varchar(255)) coverage_type, cast('' as varchar(255)) name_on_policy, cast('' as varchar(255)) name_on_card, "
             "cast('' as varchar(250)) holder_line1, cast('' as varchar(250)) holder_line2, cast('' as varchar(250)) holder_city, "
             "cast('' as varchar(255)) holder_state, cast('' as varchar(250)) holder_zip, cast('' as varchar(250)) holder_phone, "
             "cast('' as varchar(255)) holder_gender, cast(NULL as date) holder_dob, cast('' as varchar(250)) holder_ssn, "
             "cast('' as varchar(255)) group_number, cast('' as varchar(255)) policy_number, cast('' as varchar(255)) relationship_type, "
             "cast('' as varchar(250)) bin_number, cast('' as varchar(250)) processor_control_number, cast('' as varchar(250)) doctor, "
             "cast(NULL as date) start_date, cast(NULL as date) stop_date, "
             "0 is_medical, 0 is_dental, 0 is_vision, 0 is_rx, cast(NULL as integer) person_code, 0 current_id from uno ";
    if (pre_ins.size() < 2)
      sql += "union select cast('' as varchar(250)) name, cast('' as varchar(255)) phone_number, "
             "cast('' as varchar(255)) insurance_type, cast('' as varchar(255)) source_type, "
             "cast('' as varchar(255)) coverage_type, cast('' as varchar(255)) name_on_policy, cast('' as varchar(255)) name_on_card, "
             "cast('' as varchar(250)) holder_line1, cast('' as varchar(250)) holder_line2, cast('' as varchar(250)) holder_city, "
             "cast('' as varchar(255)) holder_state, cast('' as varchar(250)) holder_zip, cast('' as varchar(250)) holder_phone, "
             "cast('' as varchar(255)) holder_gender, cast(NULL as date) holder_dob, cast('' as varchar(250)) holder_ssn, "
             "cast('' as varchar(255)) group_number, cast('' as varchar(255)) policy_number, cast('' as varchar(255)) relationship_type, "
             "cast('' as varchar(250)) bin_number, cast('' as varchar(250)) processor_control_number, cast('' as varchar(250)) doctor, "
             "cast(NULL as date) start_date, cast(NULL as date) stop_date, "
             "0 is_medical, 0 is_dental, 0 is_vision, 0 is_rx, cast(NULL as integer) person_code, 1 current_id from uno ";
    sql += "order by 29 desc";
    DebugSql(this, &sql, other_forms, 9);
    dataSetInsurance->SelectSQL->Clear();
    dataSetInsurance->SelectSQL->Add(sql.c_str());

    // gen info for one value fields
    map<string, string> field_vals;

    string ssn_bday_gender_sql =
      "select people.social_security_number, people.birth_date, gender_types.item "
      "from people "
      "left join gender_types on gender_types.id = people.gender_id "
      "where people.id = " + current_person_id;
    SqlToMap(dbTransaction, ssn_bday_gender_sql, field_vals);
    personIntakeForm->qrpSSN->Caption = field_vals["social_security_number"].c_str();
    personIntakeForm->qrpDOB->Caption = field_vals["birth_date"].c_str();
    personIntakeForm->qrpGender->Caption = PrettyString(field_vals["item"]).c_str();

    sql = "select tribe_types.item, affiliations.cdib "
          "from people left join affiliations on affiliations.id = people.affiliations_id left join tribe_types on tribe_types.id = affiliations.tribe_id "
          "where people.id = '" + current_person_id + "' and affiliations.current_id >= 3";
    SqlToMap(dbTransaction, sql, field_vals);
    personIntakeForm->qrpCDIB->Caption = field_vals["cdib"].c_str();
    personIntakeForm->qrpTribe->Caption = PrettyString(field_vals["item"]).c_str();

    map<string, string> consent_map;
    SqlToMap(dbTransaction, "select first 1 consents.signature_date consent_on, groups.item consent_at "
                            "from consents inner join consent_types on consent_types.id = consents.consent_id "
                            "left join groups on consents.group_id = groups.id "
                            "where consent_types.item like '%RELEASE%' and person_id = '" + current_person_id + "' and current_id > 2 "
                            "order by consents.signature_date desc", consent_map);
    consent_map["consent_on"];
    string time_since_last_consent;
    Pdate now_date(Date());
    Pdate then_date(consent_map["consent_on"], global_mode);
    /*
    if (then_date.IsValidDate())
    {
      // hey, see, i was feeling polite ... this could be move to a slightly prettier version of that function that -assumes- we're talking about 'today'
      // but what if we were talking about some random date? then we'd want "before" and "after" rather than "ago" and "hence", and "today", "tomorrow",
      // and "yesterday" would be far messier. ("that day", "the day before", "the day after"?)
      // i don't want to build right now.
      if (now_date == then_date)
        time_since_last_consent = " (today)";
      else if (now_date == then_date + 1)
        time_since_last_consent = " (yesterday)";
      else if (now_date == then_date - 1)
        time_since_last_consent = " (tomorrow)";
      else if (now_date > then_date)
        time_since_last_consent = " (" + GetTimeSince(now_date, then_date) + " ago)";
      else if (now_date < then_date)
        time_since_last_consent = " (" + GetTimeSince(now_date, then_date) + " hence)";
    }
    */
    string consent_at = consent_map["consent_at"] != "" ? (" at " + PrettyString(consent_map["consent_at"])) : string("");
    FriendlyDiffDate(now_date.OutDate(mdy|yyyy), then_date.OutDate(mdy|yyyy), time_since_last_consent, "", false);
    if (time_since_last_consent != "") time_since_last_consent = " (" + time_since_last_consent + ")";
      personIntakeForm->qrpLastConsentDate->Caption = (consent_map["consent_on"] + time_since_last_consent + consent_at).c_str();

    vector<string> names;
    sql = "select person_names.prefix || ' ' || person_names.first_name || ' ' || "
               "person_names.middle_name || ' ' || person_names.last_name || ' ' || "
               "person_names.suffix name "
          "from person_names where person_names.person_id = '" + current_person_id + "' and person_names.current_id >= 3 order by current_id desc";
    SqlToVec(dbTransaction, sql, names);
    for (vector<string>::iterator i = names.begin(); i != names.end(); i++)
      *i = StripLeading(StripTrailing(PrettyString(*i), " "), " ");
    personIntakeForm->qrpNames->Caption = VecToDelimString(names, "", "", " / ").c_str();
    personIntakeForm->lblOUOName->Caption = VecToDelimString(names, "", "", " / ").c_str();
    personIntakeForm->lblMedicalConsentPage->Caption = "Page 3 of 3";

    /* TODO -c feature:  it would be nice if the Office Use Only box on the signature page included */

    // sql for extra insurance dataset
    sql = "select insurance_types.id, "
               "insurance_policies.policy_number, "
               "insurance_policies.start_date, "
               "insurance_policies.stop_date "
          "from insurance_coverage outer_table "
          "left join insurance_policies on outer_table.insurance_policy_id = insurance_policies.id "
          "left join insurance_types on insurance_policies.insurance_id = insurance_types.id "
          "where outer_table.person_id = '" + current_person_id + "' and "
              "outer_table.current_id >= 3 and insurance_types.id in "
              "(" + id_veteran + ", " + id_medicare_a + ", " + id_medicare_b + ", " + id_medicare_adv + ", " + id_medicaid + ") ";
    vector<map<string, string> > pre_extras;
    SqlToVecMap(dbTransaction, sql, pre_extras);
    map<string, string> extra_results;
    for (vector<map<string, string> >::iterator i = pre_extras.begin(); i != pre_extras.end(); i++)
    {
      if ((*i)["id"] == id_veteran)
      {
        extra_results["has_vet"] = "1";
        extra_results["vet_date"] = (*i)["start_date"] + ((*i)["stop_date"].empty() ? "" : " - ") + (*i)["stop_date"];
      }
      if ((*i)["id"] == id_medicare_a)
      {
        extra_results["has_med_a"] = "1";
        if ((*i)["policy_number"] != "")
          extra_results["medicare_number"] = (*i)["policy_number"];
      }
      if ((*i)["id"] == id_medicare_b)
      {
        extra_results["has_med_b"] = "1";
        if ((*i)["policy_number"] != "")
          extra_results["medicare_number"] = (*i)["policy_number"];
      }
      if ((*i)["id"] == id_medicare_d)
      {
        extra_results["has_med_d"] = "1";
        if ((*i)["policy_number"] != "")
          extra_results["medicare_number"] = (*i)["policy_number"];
      }
      if ((*i)["id"] == id_medicaid)
      {
        extra_results["has_med"] = "1";
        extra_results["medicaid_number"] = (*i)["policy_number"];
      }
      if ((*i)["id"] == id_medicare_adv)
      {
        extra_results["has_med_adv"] = "1";
        if ((*i)["policy_number"] != "")
          extra_results["medicare_number"] = (*i)["policy_number"];
      }
    }

    sql = "select "
          "'" + SafeSQLString(extra_results["has_vet"]) + "' has_vet, "
          "'" + SafeSQLString(extra_results["has_med_a"]) + "' has_medicare_a, "
          "'" + SafeSQLString(extra_results["has_med_b"]) + "' has_medicare_b, "
          "'" + SafeSQLString(extra_results["has_med_d"]) + "' has_medicare_d, "
          "'" + SafeSQLString(extra_results["has_med_adv"]) + "' has_medicare_adv, "
          "'" + SafeSQLString(extra_results["has_med"]) + "' has_medicaid, "
          "'" + SafeSQLString(extra_results["vet_date"]) + "' vet_date, "
          "'" + SafeSQLString(extra_results["medicare_number"]) + "' medicare_number, "
          "'" + SafeSQLString(extra_results["medicaid_number"]) + "' medicaid_number "
          " from uno";
    DebugSql(this, &sql, other_forms, 9);
    dataSetExtraInsurance->SelectSQL->Clear();
    dataSetExtraInsurance->SelectSQL->Add(sql.c_str());

    if (hide_signature)
    {
      personIntakeForm->labelSignature->Enabled = !hide_signature;
      personIntakeForm->shapeSignature->Enabled = !hide_signature;
      personIntakeForm->labelDateSigned->Enabled = !hide_signature;
      personIntakeForm->shapeDateSigned->Enabled = !hide_signature;
      personIntakeForm->labelOfficeUse->Enabled = !hide_signature;
      personIntakeForm->shapeOfficeUse->Enabled = !hide_signature;
      // -- not needed? embedded? *shrug*
      personIntakeForm->labelEnteredBy->Enabled = !hide_signature;
      personIntakeForm->shapeEnteredBy->Enabled = !hide_signature;
    }

    // put info in the name quick report object
    dataSetPerson->Open();
    dataSetPerson->First();
    personIntakeForm->intakeReport1->DataSet = dataSetPerson;

    // put info in the address quick report object
    dataSetAddresses->Open();
    dataSetAddresses->First();
    personIntakeForm->intakeReport2->DataSet = dataSetAddresses;

    personIntakeForm->qrpAddressPhone->DataSet = dataSetAddresses;
    personIntakeForm->qrpAddressPhone->DataField = "phone_number";

    // put info in the relationship quick report object
    dataSetRelationships->Open();
    dataSetRelationships->First();
    personIntakeForm->intakeReport3->DataSet = dataSetRelationships;

    personIntakeForm->qrpRelName->DataSet = dataSetRelationships;
    personIntakeForm->qrpRelName->DataField = "name";

    personIntakeForm->qrpRelPhone->DataSet = dataSetRelationships;
    personIntakeForm->qrpRelPhone->DataField = "phone_number";

    // put info in the insurance quick report object
    dataSetInsurance->Open();
    dataSetInsurance->First();
    personIntakeForm->intakeReport4->DataSet = dataSetInsurance;

    personIntakeForm->qrpInsCo->DataSet = dataSetInsurance;
    personIntakeForm->qrpInsCo->DataField = "name";

    personIntakeForm->qrpInsPhone->DataSet = dataSetInsurance;
    personIntakeForm->qrpInsPhone->DataField = "phone_number";

    personIntakeForm->qrpInsPolicyHolder->DataSet = dataSetInsurance;
    personIntakeForm->qrpInsPolicyHolder->DataField = "name_on_policy";

    personIntakeForm->qrpInsGroup->DataSet = dataSetInsurance;
    personIntakeForm->qrpInsGroup->DataField = "group_number";

    personIntakeForm->qrpInsPolicy->DataSet = dataSetInsurance;
    personIntakeForm->qrpInsPolicy->DataField = "policy_number";

    personIntakeForm->qrpInsBIN->DataSet = dataSetInsurance;
    personIntakeForm->qrpInsBIN->DataField = "bin_number";

    personIntakeForm->qrpInsPCN->DataSet = dataSetInsurance;
    personIntakeForm->qrpInsPCN->DataField = "processor_control_number";

    personIntakeForm->qrpInsPC->DataSet = dataSetInsurance;
    personIntakeForm->qrpInsPC->DataField = "person_code";

    personIntakeForm->qrpInsDoctor->DataSet = dataSetInsurance;
    personIntakeForm->qrpInsDoctor->DataField = "doctor";

    personIntakeForm->qrpInsStart->DataSet = dataSetInsurance;
    personIntakeForm->qrpInsStart->DataField = "start_date";

    personIntakeForm->qrpInsEnd->DataSet = dataSetInsurance;
    personIntakeForm->qrpInsEnd->DataField = "stop_date";

    personIntakeForm->qrpRelOther->DataSet = dataSetInsurance;
    personIntakeForm->qrpRelOther->DataField = "relationship_type";

    personIntakeForm->qrpInsPolicyHolderGender->DataSet = dataSetInsurance;
    personIntakeForm->qrpInsPolicyHolderGender->DataField = "holder_gender";

    personIntakeForm->qrpInsPolicyHolderDOB->DataSet = dataSetInsurance;
    personIntakeForm->qrpInsPolicyHolderDOB->DataField = "holder_dob";

    personIntakeForm->qrpInsPolicyHolderSSN->DataSet = dataSetInsurance;
    personIntakeForm->qrpInsPolicyHolderSSN->DataField = "holder_ssn";

    personIntakeForm->qrpInsPolicyHolderPhone->DataSet = dataSetInsurance;
    personIntakeForm->qrpInsPolicyHolderPhone->DataField = "holder_phone";

    // extra insurance (veteran, medicare a, b, medicaid)
    dataSetExtraInsurance->Open();
    dataSetExtraInsurance->First();
    personIntakeForm->intakeReport6->DataSet = dataSetExtraInsurance;

    personIntakeForm->qrpVetDate->DataSet = dataSetExtraInsurance;
    personIntakeForm->qrpVetDate->DataField = "vet_date";

    personIntakeForm->qrpMedicareNum->DataSet = dataSetExtraInsurance;
    personIntakeForm->qrpMedicareNum->DataField = "medicare_number";

    personIntakeForm->qrpMedicaidNum->DataSet = dataSetExtraInsurance;
    personIntakeForm->qrpMedicaidNum->DataField = "medicaid_number";

    // add all quick report objects to the composite report
    compositeReport->Reports->Add(personIntakeForm->intakeReport1);
    compositeReport->Reports->Add(personIntakeForm->intakeReport2);
    compositeReport->Reports->Add(personIntakeForm->intakeReport3);
    compositeReport->Reports->Add(personIntakeForm->intakeReport4);
    compositeReport->Reports->Add(personIntakeForm->intakeReport6);
    compositeReport->Reports->Add(personIntakeForm->medicalConsent);
  }
  catch (...)
  {
    PSay("Error running report", m_database_error, MB_OK);
    if (dbTransaction->Active)
      dbTransaction->Rollback();
  }
}
//---------------------------------------------------------------------------
void __fastcall TformLaunchIntakeFormReport::FormShortCut(TWMKey &Msg,bool &Handled)
{
  Handled = true;
  if (Msg.CharCode == VK_RETURN)
    btnRun->Click();
  else
    Handled = false;

  if (!Handled)
    TformSpecialGeneric::FormShortCut(Msg, Handled);
}
//---------------------------------------------------------------------------



