//---------------------------------------------------------------------------

#include <vcl.h>
#pragma hdrstop

#include "launchMailingLabels.h"
#include "reportMailingLabels.h"
#include "codeRecordPerson.h"
//---------------------------------------------------------------------------
#pragma package(smart_init)
#pragma link "launchGeneric"
#pragma link "codeRubberContainer"
#pragma resource "*.dfm"
TformLaunchMailingLabels *formLaunchMailingLabels;
//---------------------------------------------------------------------------
__fastcall TformLaunchMailingLabels::TformLaunchMailingLabels(TComponent* Owner)
        : TformLaunchGeneric(Owner), person_search_filter(), person_select_filter(), person_id_set()
{
  // Add all groups to cboDepartment for which the user has view permissions for charts in that department
  set<string> group_perms = GetGroupPermissions("table.charts", "", "view", "true");
  string group_perm_sql =
    "select id, item, valid from groups where groups.id > 0 and " + SafeInClause("groups.id", SetToVec(group_perms)) + " "
    "order by item ";

  lookupTables->RegisterComboBox("groups", cboDepartment, group_perm_sql);

  // Set box to user's own department if it is a valid value 
  if(group_perms.find(current_groupid) != group_perms.end())
    cboDepartment->Text = PrettyString(lookupTables->ValueOf("groups", current_groupid)).c_str();

  // Init filters

  // search filter
  person_search_filter.SetListViewHandle(listPersonMiniSearch);
  person_search_filter.SetSecurity("people");
  person_search_filter.AddColumnSpec("Last Name",   "people",      as_text, true,  100);
  person_search_filter.AddColumnSpec("First Name",  "people",      as_text, true,  100);
  person_search_filter.AddColumnSpec("Middle Name", "people",      as_text, false, 100);
  person_search_filter.AddColumnSpec("Chart No.",   "charts",      as_text, true,  90);
  person_search_filter.AddColumnSpec("SSN",         "people",      as_text, true,  90, prettySSNString);
  person_search_filter.AddColumnSpec("Current Id",  "current_id",  as_int,  false, 80);
  person_search_filter.AddColumnSpec("Internal Id", "internal_id", as_int,  false, 80);
  person_search_filter.FetchPreferences("launchmailinglabels.searchfilter");
  person_search_filter.SetIconStatus(true);
  person_search_filter.SetSQLStatement("");
  person_search_filter.RefreshDataSet();

  person_search_filter.SetAddEdit(this, NULL, EditRecord(&TformLaunchMailingLabels::Zoom));

  // selection filter
  person_select_filter.SetListViewHandle(listPeopleToPrint);
  person_select_filter.AddColumnSpec("Last Name",   "people",      as_text, true,  100);
  person_select_filter.AddColumnSpec("First Name",  "people",      as_text, true,  100);
  person_select_filter.AddColumnSpec("Middle Name", "people",      as_text, false, 100);
  person_select_filter.AddColumnSpec("Chart No.",   "charts",      as_text, true,  90);
  person_select_filter.AddColumnSpec("SSN",         "people",      as_text, true,  90, prettySSNString);
  person_select_filter.AddColumnSpec("Current Id",  "current_id",  as_int,  false, 80);
  person_select_filter.AddColumnSpec("Internal Id", "internal_id", as_int,  false, 80);
  person_select_filter.FetchPreferences("launchmailinglabels.selectfilter");
  person_select_filter.SetIconStatus(true);
  person_select_filter.SetSQLStatement("");
  person_select_filter.RefreshDataSet();

  person_select_filter.SetAddEdit(this, NULL, EditRecord(&TformLaunchMailingLabels::Zoom));
}
//---------------------------------------------------------------------------
__fastcall TformLaunchMailingLabels::~TformLaunchMailingLabels()
{
  if(lookupTables->TableName(cboDepartment) == "groups")
    lookupTables->UnregisterComboBox("groups", cboDepartment);
}
//---------------------------------------------------------------------------
bool TformLaunchMailingLabels::IsValidOnScreen()
{
  if(person_id_set.empty())
  {
    PSay("Please add one or more people to the list of mailing labels to print.", m_missing_data, MB_OK);
    return false;
  }

  return TformLaunchGeneric::IsValidOnScreen();
}
//---------------------------------------------------------------------------
bool TformLaunchMailingLabels::BeforeRunReport()
{
  dbDataset = AddDataSet();
  return TformLaunchGeneric::BeforeRunReport();
}
//---------------------------------------------------------------------------
bool TformLaunchMailingLabels::RunReport()
{
  string labels_sql =
    "select (PN.first_name || ' ' || PN.last_name || ' ' || PN.suffix) name, PCI.line1, PCI.line2, "
    " (PCI.city || ', ' || ST.abbreviation || '  ' || PCI.zip) city "
    "from people P "
    "left join person_contact_info PCI on P.person_contact_info_id = PCI.id "
    "left join state_types ST on PCI.state_id = ST.id "
    "left join person_names PN on P.person_names_id = PN.id "
    "where " + SafeInClause("P.id", SetToVec(person_id_set)) + " "
    "order by PN.last_name, PN.first_name ";

  InitDataSet(dbDataset, labels_sql);

  TformReportMailingLabels *label_form = NULL;
  FORM_TRY
    label_form = new TformReportMailingLabels(NULL);
  FORM_CATCH_RETURN("TformReportMailingLabels", false)

  label_form->labelReport->DataSet = dbDataset;

  SetupQRDBText(label_form->txtName,     dbDataset, "name");
  SetupQRDBText(label_form->txtAddress1, dbDataset, "line1");

  label_form->labelReport->Preview();

  return true;
}
//---------------------------------------------------------------------------
void TformLaunchMailingLabels::doRefreshSelect(const string &group_id)
{
  string select_sql =
    "select PN.last_name, PN.first_name, PN.middle_name, "
    " (select first 1 CH.chart_number "
    "  from charts CH "
    "  inner join location_types LOC on CH.location_id = LOC.id "
    "  where CH.person_id = P.id and LOC.group_id = '" + group_id + "' "
    "  order by CH.last_update_stamp desc), "
    " P.social_security_number, P.current_id, P.id "
    "from people P "
    "left join person_names PN on P.person_names_id = PN.id "
    "where " + SafeInClause("P.id", SetToVec(person_id_set));

  person_select_filter.SetSQLStatement(select_sql);
  person_select_filter.RefreshDataSet();
}
//---------------------------------------------------------------------------
void __fastcall TformLaunchMailingLabels::btnRightClick(TObject *Sender)
{
  // Add person from search list to list of people to print
  string person_id = person_search_filter.GetSelectedRecordID();
  string group_id  = lookupTables->IdOf("groups", cboDepartment);

  if(IsBlankOrZero(group_id))
  {
    PSay("Please select a chart department first.", m_missing_data, MB_OK);
    return;
  }

  if(!IsBlankOrZero(person_id))
    person_id_set.insert(person_id);

  if(person_id_set.empty())
  {
    PSay("Please search for some people first.", m_missing_data, MB_OK);
    return;
  }

  doRefreshSelect(group_id);
}
//---------------------------------------------------------------------------
void __fastcall TformLaunchMailingLabels::btnLeftClick(TObject *Sender)
{
  // Remove selected person from list of people to print
  string person_id = person_select_filter.GetSelectedRecordID();
  string group_id  = lookupTables->IdOf("groups", cboDepartment);

  if(IsBlankOrZero(group_id))
  {
    PSay("Please select a chart department first.", m_missing_data, MB_OK);
    return;
  }

  if(IsBlankOrZero(person_id))
  {
    PSay("Please select a person to remove from the list first.", m_missing_data, MB_OK);
    return;
  }

  person_id_set.erase(person_id);

  doRefreshSelect(group_id);
}
//---------------------------------------------------------------------------
void __fastcall TformLaunchMailingLabels::btnClearClick(TObject *Sender)
{
  string group_id  = lookupTables->IdOf("groups", cboDepartment);

  if(IsBlankOrZero(group_id))
  {
    PSay("Please select a chart department first.", m_missing_data, MB_OK);
    return;
  }

  person_id_set.clear();
  doRefreshSelect(group_id);
}
//---------------------------------------------------------------------------
void __fastcall TformLaunchMailingLabels::btnSearchClick(TObject *Sender)
{
  string group_id  = lookupTables->IdOf("groups", cboDepartment);

  if(IsBlankOrZero(group_id))
  {
    PSay("Please select a chart department first.", m_missing_data, MB_OK);
    return;
  }

  // Need at least one condition to search for
  if(editFirst->Text == "" && editLast->Text == "" && editChartNo->Text == "")
  {
    PSay("Please specify some data to search for first (name and/or chart number).", m_missing_data, MB_OK);
    return;
  }

  // Search for person and refresh left listview
  string search_sql =
    "select PN.last_name, PN.first_name, PN.middle_name, "
    " (select first 1 CH.chart_number "
    "  from charts CH "
    "  inner join location_types LOC on CH.location_id = LOC.id "
    "  where CH.person_id = P.id and LOC.group_id = '" + group_id + "' "
    "  order by CH.last_update_stamp desc), "
    " P.social_security_number, P.current_id, P.id "
    "from people P "
    "left join person_names PN on P.person_names_id = PN.id "
    "where 1=1 ";

  if(editFirst->Text != "")
  {
    search_sql +=
      "and exists (select IPN.first_name from person_names IPN "
      "where IPN.person_id = P.id and IPN.current_id > 0 and "
      "IPN.first_name like '%" + SafeSQLString(ReplaceStringWithString(editFirst->Text.c_str(), " ", "%")) + "%') ";
  }

  if(editLast->Text != "")
  {
    search_sql +=
      "and exists (select IPN.last_name from person_names IPN "
      "where IPN.person_id = P.id and IPN.current_id > 0 and "
      "IPN.last_name like '%" + SafeSQLString(ReplaceStringWithString(editLast->Text.c_str(), " ", "%")) + "%') ";
  }

  if(editChartNo->Text != "")
  {
    search_sql +=
      "and exists "
      "(select IC.chart_number "
      " from charts IC "
      " inner join location_types ILOC on IC.location_id = ILOC.id "
      " where IC.person_id = P.id and "
      " ILOC.group_id = '" + group_id + "' and "
      " IC.chart_number like '%" + SafeSQLString(editChartNo->Text.c_str()) + "%') ";
  }

  person_search_filter.SetSQLStatement(search_sql);
  person_search_filter.RefreshDataSet();
}
//---------------------------------------------------------------------------
bool TformLaunchMailingLabels::Zoom(const string & p_id)
{
  pmapstrs temp_map;

  temp_map["id"] = p_id;

  TformRecordPerson *p_form = NULL;
  FORM_TRY
    p_form = new TformRecordPerson(NULL, temp_map);
  FORM_CATCH_RETURN("TformRecordPerson", false);

  p_form->Show();

  return true;
}
