//---------------------------------------------------------------------------

#include <vcl.h>
#pragma hdrstop

#include "launchMailingLabelsWithTasksReport.h"
#include "codeRecordPerson.h"
#include "reportMailingLabels.h"
//---------------------------------------------------------------------------
#pragma package(smart_init)
#pragma link "codeSpecialGeneric"
#pragma resource "*.dfm"
TformLaunchMailingLabelsWithTasksReport *formLaunchMailingLabelsWithTasksReport;
//---------------------------------------------------------------------------
__fastcall TformLaunchMailingLabelsWithTasksReport::TformLaunchMailingLabelsWithTasksReport(TComponent* Owner, map<string, string> &p_task_map, vector<string> &p_ids_to_print)
        : TformSpecialGeneric(Owner)
{
  task_map = p_task_map;
  id_vec = p_ids_to_print;

  lookupTables->RegisterComboBox("users", cboEmployee);
  lookupTables->RegisterComboBox("method_types", cboMethod);
  lookupTables->RegisterComboBox("direction_types", cboDirection);

  // group_id is strictly propagated from the task_map
  group_id = task_map["group_id"];

  // jhaley 20100803: Implemented for multiple groups; use the proper activities lookup
  if (group_id == lookupTables->IdOf("groups", "Diabetes"))
  {
    activities_table_name = "diab_activity_types";
    activities_id_name    = "diab_activity_id";
  }
  else if (group_id == lookupTables->IdOf("groups", "Behavioral Health"))
  {
    activities_table_name = "beh_activity_types";
    activities_id_name    = "beh_activity_id";
  }
  else if (group_id == lookupTables->IdOf("groups", "Community Health") ||
           group_id == lookupTables->IdOf("groups", "Public Health")    ||
           group_id == lookupTables->IdOf("groups", "Caregiver Support Program"))
  {
    activities_table_name = "com_activity_types";
    activities_id_name    = "com_activity_id";
  }
  if (activities_table_name != "")
    lookupTables->RegisterComboBox(activities_table_name, cboActivity);
  else
  {
    // hide activities when they are not relevant; all the corresponding IDs will be zeroed.
    cboActivity->Enabled = false;
    cboActivity->Visible = false;
    lblActivity->Visible = false;
  }

  // jhaley: get appropriate permission for group
  set<string> allowed_task_groups = GetGroupPermissions("table.tasks", "", "insert", "true");
  if (allowed_task_groups.find(group_id) != allowed_task_groups.end())
  {
    if (!task_map.empty())
    {
      if (task_map["check_box"] == "true")
        checkTasks->Checked = true;
      if (task_map["user_id"] != "")
        cboEmployee->Text = lookupTables->ValueOf("users", task_map["user_id"]).c_str();
      else
        cboEmployee->Text = lookupTables->ValueOf("users", current_userid).c_str();
      if (task_map["date_of_task"] != "")
        dateTask->Date = Pdate(task_map["date_of_task"], mdy | yyyy).OutDate();
      else
        dateTask->Date = Date();

      editTimeSpent->Text = task_map["time_spent"].c_str();
      editNotes->Text     = SentenceString(task_map["task_comments"]).c_str();
      cboMethod->Text     = lookupTables->ValueOf("method_types", task_map["method_id"]).c_str();
      cboDirection->Text  = lookupTables->ValueOf("direction_types", task_map["direction_id"]).c_str();
      if (activities_table_name != "")
        cboActivity->Text = lookupTables->ValueOf(activities_table_name, task_map[activities_id_name]).c_str();

      if (task_map["check_box"] == "true")
        RecurseEnabled(panelTask, true);
      else
      {
        editNotes->Enabled = false;
        RecurseEnabled(panelTask, false);
      }
    }
    else
      RecurseEnabled(panelTask, false);
  }
  else
  {
    checkTasks->Enabled = false;
    RecurseEnabled(panelTask, false);
  }


  string sql = "";
  if (! id_vec.empty())
    sql = "select person_names.last_name, person_names.first_name, person_names.suffix, "
          "people.birth_date, affiliations.cdib, tribe_types.item, people.current_id, people.id "
          "from people "
          "left join person_names on people.person_names_id = person_names.id "
          "left join affiliations on people.affiliations_id = affiliations.id "
          "left join tribe_types on affiliations.tribe_id = tribe_types.id "
          "where " + SafeInClause("people.id", id_vec);

  person_filter.SetListViewHandle(listPeople);
  person_filter.SetSecurity("people");
  person_filter.AddColumnSpec("Last Name", "people", as_text, true, 110);
  person_filter.AddColumnSpec("First Name", "people", as_text, true, 110);
  person_filter.AddColumnSpec("Suffix", "people", as_text, false, 100);
  person_filter.AddColumnSpec("Birthday", "people", as_date, true, 100);
  person_filter.AddColumnSpec("CDIB", "affiliations", as_text, true, 100);
  person_filter.AddColumnSpec("Tribe", "affiliations", as_text, true, 150);
  person_filter.AddColumnSpec("Current Id", "current_id", as_int, false, 80);
  person_filter.AddColumnSpec("Internal Id", "internal_id", as_int, false, 80);
  person_filter.FetchPreferences("launchmailinglabelswithtasks.person_filter");
  person_filter.SetCheck(true);
  person_filter.SetCompress(true);
  person_filter.SetSQLStatement(sql);
  person_filter.RefreshDataSet();
  person_filter.SetAddEdit(this, NULL, EditRecord(&(TformLaunchMailingLabelsWithTasksReport::EditPerson)));
}
//---------------------------------------------------------------------------
void TformLaunchMailingLabelsWithTasksReport::EditPerson()
{
  map<string, string> temp_map;
  temp_map["id"] = person_filter.GetSelectedRecordID();
  FORM_TRY
    TformRecordPerson *person_form = new TformRecordPerson(NULL, temp_map);
    person_form->Show();
  FORM_CATCH
}
//---------------------------------------------------------------------------
void __fastcall TformLaunchMailingLabelsWithTasksReport::FormClose(TObject *Sender, TCloseAction &Action)
{
  lookupTables->UnregisterComboBox("users", cboEmployee);
  lookupTables->UnregisterComboBox("method_types", cboMethod);
  lookupTables->UnregisterComboBox("direction_types", cboDirection);
  if (activities_table_name != "")
    lookupTables->UnregisterComboBox(activities_table_name, cboActivity);

  TformSpecialGeneric::FormClose(Sender, Action);
}
//---------------------------------------------------------------------------

void __fastcall TformLaunchMailingLabelsWithTasksReport::checkTasksClick(TObject *Sender)
{
  if (checkTasks->Checked)
    RecurseEnabled(panelTask, true);
  else
    RecurseEnabled(panelTask, false);
}
//---------------------------------------------------------------------------


void __fastcall TformLaunchMailingLabelsWithTasksReport::btnCheckAllClick(TObject *Sender)
{
  person_filter.CheckAll();        
}
//---------------------------------------------------------------------------

void __fastcall TformLaunchMailingLabelsWithTasksReport::btnUncheckAllClick(TObject *Sender)
{
  person_filter.UncheckAll();
}
//---------------------------------------------------------------------------
bool TformLaunchMailingLabelsWithTasksReport::IsValidOnScreen()
{
  if (!lookupTables->IsInLookUpTable("users", cboEmployee->Text.c_str()))
  {
    PSay("Please enter a valid employee", m_invalid_data, MB_OK);
    cboEmployee->SetFocus();
    return false;
  }
  else if (!IsFloat(editTimeSpent->Text.c_str()))
  {
    PSay("Please enter a valid time spent", m_invalid_data, MB_OK);
    editTimeSpent->SetFocus();
    return false;
  }
  else if (!lookupTables->IsInLookUpTable("method_types", cboMethod->Text.c_str()))
  {
    PSay("Please enter a valid method of contact", m_invalid_data, MB_OK);
    cboMethod->SetFocus();
    return false;
  }
  else if (!lookupTables->IsInLookUpTable("direction_types", cboDirection->Text.c_str()))
  {
    PSay("Please enter either incoming or outgoing", m_invalid_data, MB_OK);
    cboDirection->SetFocus();
    return false;
  }
  else if (activities_table_name != "" && !lookupTables->IsInLookUpTable(activities_table_name, cboActivity->Text.c_str()))
  {
    PSay("Please enter a valid activity type", m_invalid_data, MB_OK);
    cboActivity->SetFocus();
    return false;
  }

  return true;
}
//---------------------------------------------------------------------------
void __fastcall TformLaunchMailingLabelsWithTasksReport::btnRunClick(TObject *Sender)
{
  vector<string> id_vec = person_filter.GetMultiID();
  if (!id_vec.empty())
  {
    if (IsValidOnScreen())
    {
      Screen->Cursor = crHourGlass;
      string sql;
      TIBTransaction *dbTransaction = new TIBTransaction(NULL);
      TIBDataSet *dbDataset = new TIBDataSet(NULL);

      try
      {
        FORM_TRY
          TformReportMailingLabels *label_form = new TformReportMailingLabels(NULL);
          dbTransaction->DefaultDatabase = dbDatabase;
          dbDataset->Database = dbDatabase;
          dbDataset->Transaction = dbTransaction;
          dbTransaction->StartTransaction();
          sql = "select person_names.first_name || ' ' || person_names.last_name first_last, "
                "person_contact_info.line1, person_contact_info.line2, person_contact_info.city || ', ' || state_types.abbreviation || '  ' || "
                "person_contact_info.zip city "
                "from people "
                "inner join person_names on people.person_names_id = person_names.id "
                "inner join person_contact_info on people.person_contact_info_id = person_contact_info.id "
                "inner join state_types on person_contact_info.state_id = state_types.id "
                "where person_contact_info.line1 is not null and person_contact_info.city is not null "
                "and person_contact_info.state_id > 0 and person_contact_info.zip is not null "
                "and " + SafeInClause("people.id", id_vec) + " "
                "order by person_names.last_name, person_names.first_name";
          dbDataset->SelectSQL->Add(sql.c_str());
          dbDataset->Open();
          label_form->labelReport->DataSet = dbDataset;
          label_form->txtName->DataSet = dbDataset;
          label_form->txtName->DataField = "first_last";
          label_form->txtAddress1->DataSet = dbDataset;
          label_form->txtAddress1->DataField = "line1";
          Screen->Cursor = crDefault;
          label_form->labelReport->Preview();
        FORM_CATCH

        if (checkTasks->Checked)
        {
          Screen->Cursor = crHourGlass;
          string time_spent = editTimeSpent->Text.c_str();
          string activity_ids_str = "";
          if (time_spent == "")
            time_spent = "null";
          // jhaley: be careful and fill in all the IDs properly.
          if (activities_table_name == "beh_activity_types")
            activity_ids_str = lookupTables->IdOf(activities_table_name, cboActivity->Text.c_str()) + ", 0, 0";
          else if (activities_table_name == "com_activity_types")
            activity_ids_str = "0, " + lookupTables->IdOf(activities_table_name, cboActivity->Text.c_str()) + ", 0";
          else if (activities_table_name == "diab_activity_types")
            activity_ids_str = "0, 0, " + lookupTables->IdOf(activities_table_name, cboActivity->Text.c_str());
          else
            activity_ids_str = "0, 0, 0"; // clinic groups don't use activities at all.
          sql = "insert into tasks (id, current_id, last_update_stamp, last_updated_by, person_id, group_id, "
                "user_id, date_of_task, time_spent, method_id, direction_id, beh_activity_id, com_activity_id, "
                "diab_activity_id, task_comments) "
                "select (select gen_id(tasks_gen, 1) from uno), 4, '" + RightNow() + "', " +  current_userid  + ", people.id, " +
                group_id + ", " + lookupTables->IdOf("users", cboEmployee->Text.c_str()) + ", "
                "'" +  Pdate(dateTask->Date).OutDate(mdy | yyyy) + "', " +
                time_spent + ", " + lookupTables->IdOf("method_types", cboMethod->Text.c_str()) + ", " +
                lookupTables->IdOf("direction_types", cboDirection->Text.c_str()) + ", " + activity_ids_str + ", "
                "'" + SafeSQLString(editNotes->Text.c_str()) + "' "
                "from people where " + SafeInClause("id", id_vec);

          if (!ExecuteStatement(dbTransaction, sql))
            throw 1;
          Screen->Cursor = crDefault;
        }

        dbDataset->Close();
        dbTransaction->Commit();
      }
      catch(...)
      {
        Screen->Cursor = crDefault;
        if (dbTransaction->Active)
          dbTransaction->Rollback();
        PSay("Error printing labels", m_database_error, MB_OK);
      }
      delete dbDataset;
      delete dbTransaction;
    }
  }
  else
  {
    PSay("You have not selected any people", m_missing_data, MB_OK);
  }
}
//---------------------------------------------------------------------------

void __fastcall TformLaunchMailingLabelsWithTasksReport::btnCloseClick(TObject *Sender)
{
  Close();        
}
//---------------------------------------------------------------------------

