//---------------------------------------------------------------------------

#include <vcl.h>
#pragma hdrstop

#include "launchObligatedAccountsReport.h"
#include "reportObligatedAccounts.h"
#include "derivedheaders.h"
#include "codeSpecialDataQuery.h"
//---------------------------------------------------------------------------
#pragma package(smart_init)
#pragma link "codeSpecialGeneric"
#pragma resource "*.dfm"
TformLaunchObligatedAmountsReport *formLaunchObligatedAmountsReport;
//---------------------------------------------------------------------------
__fastcall TformLaunchObligatedAmountsReport::TformLaunchObligatedAmountsReport(TComponent* Owner)
        : TformSpecialGeneric(Owner)
{
  Pdate year_begin;
  year_begin.InDate(Date());
  year_begin.month = 1;
  year_begin.day = 1;
  dateBegin->Date = year_begin.OutDate();
  dateEnd->Date = Date();
  services_filter.SetListViewHandle(listServices);
  services_filter.SetSecurity("services_provided");
  services_filter.AddColumnSpec("Service", "people", as_text, true, 180);
  services_filter.AddColumnSpec("Internal Id", "internal_id", as_int, false, 80);
  services_filter.FetchPreferences("launchobligatedprovidereport.services_filter");
  services_filter.SetCheck(true);
  services_filter.SetCompress(true);
  services_filter.SetSQLStatement("select item, id from service_types where valid = 1");
  services_filter.RefreshDataSet();
  set<string> id_set;
  id_set.insert("0");
  id_set.insert(lookupTables->IdOf("service_types", "Respite Care")); // this is a special program under Community Health, and its information is seperate from Contract Health's services
  services_filter.SetCheckedWhereNotID(id_set, true);
}
//---------------------------------------------------------------------------
void __fastcall TformLaunchObligatedAmountsReport::btnCloseClick(TObject *Sender)
{
  Close();
}
//---------------------------------------------------------------------------
void __fastcall TformLaunchObligatedAmountsReport::FormShortCut(TWMKey &Msg, bool &Handled)
{
  Handled = true;
  if (Msg.CharCode == VK_RETURN)
    btnRun->Click();
  else
    Handled = false;

  if (!Handled)
    TformSpecialGeneric::FormShortCut(Msg, Handled);
}
//---------------------------------------------------------------------------
void __fastcall TformLaunchObligatedAmountsReport::FormClose(TObject *Sender, TCloseAction &Action)
{
  TformSpecialGeneric::FormClose(Sender, Action);
}
//---------------------------------------------------------------------------
void __fastcall TformLaunchObligatedAmountsReport::btnRunClick(TObject *Sender)
{
  vector<string> checked_ids = services_filter.GetMultiID();
  if(! checked_ids.empty())
  {
    Processing(true);

    TIBTransaction * dbTransaction = new TIBTransaction(NULL);
    TIBDataSet * dbDataset = new TIBDataSet(NULL);
    string sql;
    string begin_date = Pdate(dateBegin->Date).OutDate(mdy | yyyy);
    string end_date =  Pdate(dateEnd->Date).OutDate(mdy | yyyy);
    string date_title = begin_date + " - " + end_date;

    try
    {
      if(!StdTransaction(dbTransaction, dbDatabase))
        PSay("Error setting up transaction", m_database_error, MB_OK);
      dbDataset->Database = dbDatabase;
      dbDataset->Transaction = dbTransaction;
      dbDataset->UniDirectional = false;

      ExecuteStatement(dbTransaction, "delete from temp_child_cob");
      ExecuteStatement(dbTransaction, "delete from temp_obligated_cob");
      ExecuteStatement(dbTransaction, "delete from temp_bill_amounts");
      ExecuteStatement(dbTransaction, "delete from temp_bill_accounts");
      ExecuteStatement(dbTransaction, "delete from temp_obligated");
      ExecuteStatement(dbTransaction, "delete from temp_paid_visits");
      ExecuteStatement(dbTransaction, "delete from temp_obligated_report");

      // NOTE USES VIEW
      // All COB Visits
      ExecuteStatement(dbTransaction, "insert into temp_child_cob select id, parent_task_id, tax_entity_id from visit_tasks "
                      "where visit_tasks.tax_entity_id != 0");
      // All COB Tasks in date range without a child (COB Visit)
      ExecuteStatement(dbTransaction, "insert into temp_obligated_cob select id, person_id, services_provided_id "
                      "from cob_tasks where cob_tasks.date_of_task between '" + begin_date + "' and '" + end_date + "' "
                      " and not exists (select temp_child_cob.id from temp_child_cob "
                      "where temp_child_cob.parent_task_id = cob_tasks.id)");
      // All bills paid (by check date) in date range
      ExecuteStatement(dbTransaction, "insert into temp_bill_accounts select bills.id, bills.task_id, transactions.account_id, transactions.id, transactions.credit_amount, transactions.debit_amount "
                      "from checks inner join purchase_reqs on purchase_reqs.check_id = checks.id "
                      "inner join bills on bills.purchase_req_id = purchase_reqs.id "
                      "inner join transactions on transactions.bill_id = bills.id "
                      "and transactions.account_id in (select accounts.id from accounts where accounts.account_type_id = " + lookupTables->IdOf("account_types", "Cash") + ") "
                      "where checks.date_sent between '" + begin_date + "' and '" + end_date + "' and bills.task_id > 0 "
                      "group by bills.id, bills.task_id, checks.date_sent, transactions.account_id, transactions.id, transactions.credit_amount, transactions.debit_amount");

      // Crap.  Where does this go?
      //"service_id in (" + VecToCommaString(checked_ids) + ") "

      // Obligated
      // COB Tasks with no children -- get Expense transactions on them.  Expense on COBs == obligated amounts that are still obligated.  Get services_provided, # of visits, people, total amounts, and blah
      sql = "insert into temp_obligated (account_id, obligated_amount) "
            "select transactions.account_id, "
            "sum(transactions.debit_amount - transactions.credit_amount) obligated_amount "
            "from temp_obligated_cob obligated_cob "
            "inner join transactions on transactions.task_id = obligated_cob.id and "
            "transactions.account_id in (select accounts.id from accounts where "
            "accounts.account_type_id = " + lookupTables->IdOf("account_types", "Expense") + ") "
            "group by account_id";
      DebugSql(this, &sql, other_forms, 9);
      ExecuteStatement(dbTransaction, sql);

      // Paid
      // Take all COB Tasks with children (COB Visits), join to all bills paid in date range, getting services_provided, # of visits, people, total amounts, and blah
      sql = "insert into temp_paid_visits (account_id, paid_amount) "
            "select bill_accounts.account_id, "
                   "sum(bill_accounts.credit_amount - bill_accounts.debit_amount) paid_amount "
//            "from cob_tasks parent_cob "
//            "inner join temp_child_cob child_cob on parent_cob.id = child_cob.parent_task_id "
//            "inner join temp_bill_accounts bill_accounts on bill_accounts.task_id = child_cob.id "
            "from temp_bill_accounts bill_accounts where task_id > 0 "
            "group by account_id";
      DebugSql(this, &sql, other_forms, 9);
      ExecuteStatement(dbTransaction, sql);

      /*
      sql = "select nonbudget.id account_id, "
            "acc_level_one.id fund_id, acc_level_one.item      || ' (' || acc_level_one.short_item   || ')' fund_name, "
            "acc_level_three.id dept_id, acc_level_three.item  || ' (' || acc_level_three.short_item || ')' dept_name, "
            "acc_level_four.id subdept_id, acc_level_four.item || ' (' || acc_level_four.short_item  || ')' subdept_name, "
            "acc_level_two.id lineitem_id, acc_level_two.item  || ' (' || acc_level_two.short_item   || ')' lineitem_name, "
            "sum(tx.debit_amount - tx.credit_amount) budget_amount, sum(O.obligated_amount) obligated_amount, "
            "sum(P.paid_amount) paid_amount "
            "from accounts nonbudget "
            "inner join acc_level_one   on nonbudget.acc_level_one_id   = acc_level_one.id   "
            "inner join acc_level_two   on nonbudget.acc_level_two_id   = acc_level_two.id   "
            "inner join acc_level_three on nonbudget.acc_level_three_id = acc_level_three.id "
            "inner join acc_level_four  on nonbudget.acc_level_four_id  = acc_level_four.id  "
            "inner join accounts budget on nonbudget.budget_account_id = budget.id "
            "left join transactions tx on tx.account_id = budget.id "
            "left join temp_obligated O on O.account_id = nonbudget.id "
            "left join temp_paid_visits P on P.account_id = nonbudget.id "
            "where nonbudget.is_budget = 0 "
            "group by 1, 2, 3, 4, 5, 6, 7, 8, 9 "
            "having sum(O.obligated_amount) != 0 or sum(P.paid_amount) != 0 "
            "order by acc_level_one.id, acc_level_three.id, acc_level_four.id, acc_level_two.id ";
      */
      sql = "select nonbudget.id account_id, "
/*          "  acc_level_one.id fund_id, acc_level_one.item      || ' (' || acc_level_one.short_item   || ')' fund_name, "
            "  acc_level_three.id dept_id, acc_level_three.item  || ' (' || acc_level_three.short_item || ')' dept_name, "
            "  acc_level_four.id subdept_id, acc_level_four.item || ' (' || acc_level_four.short_item  || ')' subdept_name, "
            "  acc_level_two.id lineitem_id, acc_level_two.item  || ' (' || acc_level_two.short_item   || ')' lineitem_name, "
*/
            "  acc_level_one.id fund_id,     acc_level_one.short_item   || '-' || acc_level_one.item   fund_name, "
            "  acc_level_three.id dept_id,   acc_level_three.short_item || '-' || acc_level_three.item dept_name, "
            "  acc_level_four.id subdept_id, acc_level_four.short_item  || '-' || acc_level_four.item  subdept_name, "
            "  acc_level_two.id lineitem_id, acc_level_two.short_item   || '-' || acc_level_two.item   lineitem_name, "
            // This is pretty stupid, but it wouldn't let me GROUP by this column.  Oh well.
            "  min((select sum(tx.debit_amount - tx.credit_amount) "
            "       from transactions tx where tx.account_id = nonbudget.budget_account_id and tx.fiscal_date <= '" + end_date + "')) budget_amount, "
            "  sum(O.obligated_amount) obligated_amount, sum(P.paid_amount) paid_amount "
            "from accounts nonbudget inner join acc_level_one   on nonbudget.acc_level_one_id   = acc_level_one.id "
            "                        inner join acc_level_two   on nonbudget.acc_level_two_id   = acc_level_two.id "
            "                        inner join acc_level_three on nonbudget.acc_level_three_id = acc_level_three.id "
            "                        inner join acc_level_four  on nonbudget.acc_level_four_id  = acc_level_four.id "
            "                        left join temp_obligated O on O.account_id = nonbudget.id "
            "                        left join temp_paid_visits P on P.account_id = nonbudget.id "
            "group by 1, 2, 3, 4, 5, 6, 7, 8, 9 "
            "having sum(O.obligated_amount) != 0 or sum(P.paid_amount) != 0 "
            "order by acc_level_one.id, acc_level_three.id, acc_level_four.id, acc_level_two.id ";

      DebugSql(this, &sql, other_forms, 9);

      // QueryData(this, dbTransaction, sql);

      dbDataset->SelectSQL->Add(sql.c_str());
      dbDataset->Open();
      dbDataset->First();

      FORM_TRY
        TformReportObligatedAccounts *report = new TformReportObligatedAccounts(NULL);
        report->report->DataSet = dbDataset;
        report->lblDateTitle->Caption = date_title.c_str();
        report->lblUser->Caption = current_fullname.c_str();
/*        report->txtProvider->DataSet = dbDataset;
        report->txtProvider->DataField = "provider";
        report->txtService->DataSet = dbDataset;
        report->txtService->DataField = "service_type";
        report->txtObligatedVisits->DataSet = dbDataset;
        report->txtObligatedVisits->DataField = "obligated_cobs";
        report->txtObligatedMembers->DataSet = dbDataset;
        report->txtObligatedMembers->DataField = "obligated_members";
*/
        report->exprFund->Expression = "fund_name";
        report->exprDept->Expression = "dept_name";
        report->exprSubDept->Expression = "subdept_name";
        report->exprLineItem->Expression = "lineitem_name";
        report->groupFund->Expression = "fund_id";
        report->groupDept->Expression = "dept_id";
        report->groupSubDept->Expression = "subdept_id";
        report->groupLineItem->Expression = "lineitem_id";

        report->exprBudgetAmount->Expression = "FORMATNUMERIC('$###,###,##0.00', budget_amount)";
        report->exprObligatedAmount->Expression = "FORMATNUMERIC('$###,###,##0.00', obligated_amount)";
        report->exprPaidAmount->Expression = "FORMATNUMERIC('$###,###,##0.00', paid_amount)";
        report->exprExpensedAmount->Expression = "FORMATNUMERIC('$###,###,##0.00', sum(obligated_amount + paid_amount))";
        report->exprRemainingAmount->Expression = "FORMATNUMERIC('$###,###,##0.00', sum(budget_amount) - sum(obligated_amount + paid_amount))";

        report->exprTotalObligatedAmount->Expression = "FORMATNUMERIC('$###,###,##0.00', SUM(obligated_amount))";
        report->exprTotalPaidAmount->Expression = "FORMATNUMERIC('$###,###,##0.00', SUM(paid_amount))";

/*        acc_info the_budget = InfoById();  // TODO -c MAGIC_NUMBER: Governor wanted total budget for IHS->OSG->Contract Health Services
        financial the_budget_amount = the_budget.GetCurrentBalance();


        report->lblCHSBudgetTotalAmt->Caption = ("$" + FinancialToStringEx(CHS_budget_amount)).c_str();
        report->exprCHSBudgetExpensedAmt->Expression = "FORMATNUMERIC('$###,###,##0.00', (SUM(obligated_amount) + SUM(paid_amount)))";
        report->exprCHSBudgetRemainingAmt->Expression = ("FORMATNUMERIC('$###,###,##0.00', (" + FinancialToString(CHS_budget_amount, false, 2) + " - SUM(obligated_amount) - SUM(paid_amount)))").c_str();
*/
        Processing(false);
        report->report->Preview();
      FORM_CATCH

      dbDataset->Close();
      ExecuteStatement(dbTransaction, "delete from temp_child_cob");
      ExecuteStatement(dbTransaction, "delete from temp_obligated_cob");
      ExecuteStatement(dbTransaction, "delete from temp_bill_amounts");
      ExecuteStatement(dbTransaction, "delete from temp_bill_accounts");
      ExecuteStatement(dbTransaction, "delete from temp_obligated");
      ExecuteStatement(dbTransaction, "delete from temp_paid_visits");
      ExecuteStatement(dbTransaction, "delete from temp_obligated_report");

      dbTransaction->Commit();
    }
    catch (...)
    {
      Processing(false);
      if (dbTransaction->Active)
        dbTransaction->Rollback();
      PSay("Error running report", m_database_error, MB_OK);
    }
    delete dbTransaction;
    delete dbDataset;
  }
  else
    PSay("No services are checked", m_missing_data, MB_OK);
}
//---------------------------------------------------------------------------
