//---------------------------------------------------------------------------
#include <vcl.h>
#pragma hdrstop

#include "launchObligatedAndExpensedReports.h"
#include "reportObligatedAndExpensed.h"
#include "derivedheaders.h"
//---------------------------------------------------------------------------
#pragma package(smart_init)
#pragma link "launchGeneric"
#pragma link "codeFrameCheckListDropDown"
#pragma resource "*.dfm"
TformLaunchObligatedAndExpensedReports *formLaunchObligatedAndExpensedReports;
//---------------------------------------------------------------------------
__fastcall TformLaunchObligatedAndExpensedReports::TformLaunchObligatedAndExpensedReports(TComponent* Owner)
        : TformLaunchGeneric(Owner)
{
  Pdate year_begin;
  year_begin.InDate(Date());
  year_begin.month = 1;
  year_begin.day = 1;
  dateBegin->Date = year_begin.OutDate();
  dateEnd->Date = Date();
  services_filter.SetListViewHandle(listServices);
  services_filter.SetSecurity("services_provided");
  services_filter.AddColumnSpec("Service", "people", as_text, true, 180);
  services_filter.AddColumnSpec("Internal Id", "internal_id", as_int, false, 80);
  services_filter.FetchPreferences("launchobligatedprovidereport.services_filter");
  services_filter.SetCheck(true);
  services_filter.SetCompress(true);
  services_filter.SetSQLStatement("select item, id from service_types where valid = 1");
  services_filter.RefreshDataSet();
  set<string> id_set;
  id_set.insert("0");
  id_set.insert(lookupTables->IdOf("service_types", "Respite Care")); // this is a special program under Community Health, and its information is seperate from Contract Health's services
  services_filter.SetCheckedWhereNotID(id_set, true);
  services_filter.SetHint("Applies ONLY to \"by provider and service\" and \"by service provided\" reports.");

  string clever_current_id = "case when valid = 1 then '" + lookupTables->IdOf("current_types", "Current") + "' else '" + lookupTables->IdOf("current_types", "Expired") + "' end";
  cbddServiceArea->Filter->SetSecurity("");
  cbddServiceArea->Filter->AddColumnSpec("Service Area", "", as_text, true, 200);
  cbddServiceArea->Filter->AddColumnSpec("Current Id", "current_id", as_int, false, 40);
  cbddServiceArea->Filter->AddColumnSpec("Internal Id", "internal_id", as_int, false, 60);
  cbddServiceArea->Filter->SetSQLStatement("select item, " + clever_current_id + ", id from area_types where id > 0");
  cbddServiceArea->Filter->FetchPreferences("launchobligatedprovidereport.service_area_filter");
  cbddServiceArea->Filter->SetIconStatus(true);
  cbddServiceArea->Filter->ForceRefreshDataSet();
  cbddServiceArea->Filter->CheckAll();
  cbddServiceArea->DisplayFieldName = "Service Area";
  cbddServiceArea->DropDownHeight = 144;

  cboCatchment->ItemIndex = 0;
  cboState->ItemIndex = 0;
}
//---------------------------------------------------------------------------
bool TformLaunchObligatedAndExpensedReports::BeforeRunReport()
{
  dbDataset = NULL;
  dbDataset = new TIBDataSet(NULL);
  return TformLaunchGeneric::BeforeRunReport();
}
//---------------------------------------------------------------------------
bool TformLaunchObligatedAndExpensedReports::RunReport()
{
  service_ids = services_filter.GetMultiID();
  service_area_ids = cbddServiceArea->Filter->GetMultiID();

  checkpoint_branch = "Main";
  checkpoint = 0;
  string sql;

  string begin_date = Pdate(dateBegin->Date).OutDate(mdy | yyyy);
  string end_date =  Pdate(dateEnd->Date).OutDate(mdy | yyyy);
  string date_title = begin_date + " - " + end_date;

  ExecuteStatement(report_transaction, "delete from temp_obpd_cobs"); checkpoint++;
  ExecuteStatement(report_transaction, "delete from temp_obpd_visits"); checkpoint++;
  ExecuteStatement(report_transaction, "delete from temp_obpd_cob_expense"); checkpoint++;
  ExecuteStatement(report_transaction, "delete from temp_obpd_bill_expense"); checkpoint++;
  //ExecuteStatement(report_transaction, "delete from temp_obpd_totals"); checkpoint++; // is done at the appropriate time
  ExecuteStatement(report_transaction, "delete from temp_obpd_services_provided"); checkpoint++;

  string restrict_people;
  if (radioCatchment->Checked || radioState->Checked)
  {
//    restrict_people = "exists (select * from person_contact_info pci left join county_types on pci.county_id = county_types.id "
//                      "where pci.person_id = people.id and pci.current_id+1 > 3 and ";
    /* We have a foreign key that should accurately point to their home-address record; let's use it so as to prevent potential
       overlap between running the report "inside" vs. "outside" the catchment area. */
    restrict_people = "left join person_contact_info pci on people.home_address_id = pci.id and pci.current_id+1 > 3 and ";

    // more where clause
    if (radioCatchment->Checked)
    { // state is OK and county is one of the magic five
      string sub_sql = "pci.state_id = " + lookupTables->IdOf("state_types", "Oklahoma") + " and "
        "substring(pci.zip from 1 for 5) in (select zip_code from zip_codes_view where state = 'OK' and county in ('Pottawatomie', 'Cleveland', 'Lincoln', 'Logan', 'Oklahoma'))";
      /* We should probably count people whose address we don't know -somewhere-, so let's consider them, by default, to be
         outside the catchment area (I guess?).  We'll use "inside" criteria in the join, then if "outside" is what they
         selected, restrict to rows where the join returned nothing, via the where clause below */
      //if (cboCatchment->Text == "outside") // then we do the inverse
      //  sub_sql = "not (" + sub_sql + ")";
      restrict_people += sub_sql + " ";
    }
    else if (radioState->Checked)
    { // state is OK
      string sub_sql = "pci.state_id = " + lookupTables->IdOf("state_types", "Oklahoma");
      /* We should probably count people whose address we don't know -somewhere-, so let's consider them, by default, to be
         outside the catchment area (I guess?).  We'll use "inside" criteria in the join, then if "outside" is what they
         selected, restrict to rows where the join returned nothing, via the where clause below */
      //if (cboState->Text == "outside") // then we do the inverse
      //  sub_sql = "not (" + sub_sql + ")";
      restrict_people += sub_sql + " ";
    }
    // end where clause; now end the exists() subselect
    //restrict_people += ") ";
  }

  /* All visits we might care about */
  sql = "insert into temp_obpd_visits (id, parent_task_id, visit_date, bill_id, bill_date, pr_id, pr_date, pr_number, check_id, check_date, check_number) "
        "select Visit.id, Visit.parent_task_id, Visit.date_of_task, bills.id, bills.bill_date, purchase_reqs.id, purchase_reqs.date_sent, purchase_reqs.purchase_req_number, "
               "checks.id, checks.date_sent, checks.check_number "
        "from visit_tasks Visit "
        "inner join bills on bills.task_id = Visit.id " // must be on a bill
        "left  join purchase_reqs on bills.purchase_req_id = purchase_reqs.id "
        "left  join checks on purchase_reqs.check_id = checks.id ";
  if (radioCatchment->Checked || radioState->Checked)
    sql += "left join people on Visit.person_id = people.id " + restrict_people;
  sql += "where Visit.tax_entity_id > 0 and "
              "(bills.bill_date between '" + begin_date + "' and '" + end_date + "' or "
               "purchase_reqs.date_sent between '" + begin_date + "' and '" + end_date + "' or "
               "checks.date_sent between '" + begin_date + "' and '" + end_date + "') ";
  //if (radioCatchment->Checked || radioState->Checked)
  //  sql += " and " + restrict_people;
  /* We should probably count people whose address we don't know -somewhere-, so let's consider them, by default, to be
     outside the catchment area (I guess?).  We'll use "inside" criteria in the join, then if "outside" is what they
     selected, restrict to rows where the join returned nothing, via the where clause */
  if ((radioCatchment->Checked && cboCatchment->Text == "outside") ||
      (radioState->Checked && cboState->Text == "outside"))
    sql += " and pci.id is null ";
  if ((radioCatchment->Checked && cboCatchment->Text == "within") ||
      (radioState->Checked && cboState->Text == "within"))
    sql += " and pci.id is not null ";
  DebugSql(this, &sql, other_forms, 9, report_transaction);
  ExecuteStatement(report_transaction, sql); checkpoint++;

  /* parent COBs for everything we might care about */
  sql = "insert into temp_obpd_cobs (id, person_id, services_provided_id, cob_date, cob_number) "
        "select COB.id, COB.person_id, COB.services_provided_id, COB.date_of_task, COB.cob_number "
        "from cob_tasks COB ";
  if (radioCatchment->Checked || radioState->Checked)
    sql += "left join people on COB.person_id = people.id " + restrict_people;
  sql += "where (COB.date_of_task between '" + begin_date + "' and '" + end_date + "' " // COB in date range
               "or exists (select * from temp_obpd_visits Visits "
                          "where Visits.parent_task_id = COB.id)) ";                   // or has visits in date range
  //if (radioCatchment->Checked || radioState->Checked)
  //  sql += " and " + restrict_people;
  /* We should probably count people whose address we don't know -somewhere-, so let's consider them, by default, to be
     outside the catchment area (I guess?).  We'll use "inside" criteria in the join, then if "outside" is what they
     selected, restrict to rows where the join returned nothing, via the where clause */
  if ((radioCatchment->Checked && cboCatchment->Text == "outside") ||
      (radioState->Checked && cboState->Text == "outside"))
    sql += " and pci.id is null ";
  if ((radioCatchment->Checked && cboCatchment->Text == "within") ||
      (radioState->Checked && cboState->Text == "within"))
    sql += " and pci.id is not null ";
  DebugSql(this, &sql, other_forms, 9, report_transaction);
  ExecuteStatement(report_transaction, sql); checkpoint++;

  /* Expenses per COB */
  sql = "insert into temp_obpd_cob_expense (cob_id, service_id, tax_entity_id, area_id, person_id, account_id, cob_date, cob_number, expensed) "
        "select COB.id, SP.service_id, SP.tax_entity_id, SP.area_id, COB.person_id, account_id, COB.cob_date, COB.cob_number, sum(tx.debit_amount - tx.credit_amount) expensed "
        "from temp_obpd_cobs COB "
        "left join transactions tx on tx.task_id = COB.id and tx.account_id in (select id from accounts where accounts.account_type_id = " + lookupTables->IdOf("account_types", "Expense") + ") "
                                                                                "and (0=1 or tx.effective_date <= '" + end_date + "') "
        "inner join services_provided SP on COB.services_provided_id = SP.id "
        "where COB.cob_date between '" + begin_date + "' and '" + end_date + "' "
        "group by 1, 2, 3, 4, 5, 6, 7, 8 ";
  DebugSql(this, &sql, other_forms, 9, report_transaction);
  ExecuteStatement(report_transaction, sql); checkpoint++;

  /* Expenses per bill */
  sql = "insert into temp_obpd_bill_expense (bill_id, service_id, tax_entity_id, area_id, person_id, visit_date, bill_date, pr_date, pr_number, check_date, check_number, "
                                            "is_on_pr, is_on_check, account_id, cob_id, expensed) "
        "select Visit.bill_id, SP.service_id, SP.tax_entity_id, SP.area_id, COB.person_id, Visit.visit_date, Visit.bill_date, Visit.pr_date, Visit.pr_number, Visit.check_date, Visit.check_number, "
        "case when Visit.pr_date between '" + begin_date + "' and '" + end_date + "' then 1 else 0 end, "
        "case when Visit.check_date between '" + begin_date + "' and '" + end_date + "' then 1 else 0 end, tx.account_id, "
        "COB.id, sum(tx.debit_amount - tx.credit_amount) expensed "
        "from temp_obpd_cobs COB "
        "inner join services_provided SP on COB.services_provided_id = SP.id "
        "inner join temp_obpd_visits Visit on Visit.parent_task_id = COB.id "
        "inner join bills on bills.task_id = Visit.id "
        "inner join transactions tx on tx.bill_id = bills.id and tx.account_id in (select id from accounts where accounts.account_type_id = " + lookupTables->IdOf("account_types", "Expense") + ") "
                                                                                   "and (0=1 or tx.effective_date <= '" + end_date + "') "
        "where "
        "     (((Visit.pr_date is null or Visit.pr_date NOT between '" + begin_date + "' and '" + end_date + "') and Visit.bill_date between '" + begin_date + "' and '" + end_date + "') or "
              "((Visit.check_date is null or Visit.check_date NOT between '" + begin_date + "' and '" + end_date + "') and pr_date between '" + begin_date + "' and '" + end_date + "') or "
              "(Visit.check_date between '" + begin_date + "' and '" + end_date + "')) "
        "group by 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15 "
        "having sum(tx.debit_amount - tx.credit_amount) > 0 ";
  DebugSql(this, &sql, other_forms, 9, report_transaction);
  ExecuteStatement(report_transaction, sql); checkpoint++;

  sql = "insert into temp_obpd_services_provided (service_id, tax_entity_id) select distinct service_id, tax_entity_id from services_provided";
  DebugSql(this, &sql, other_forms, 9, report_transaction);
  ExecuteStatement(report_transaction, sql); checkpoint++;

  dbDataset->Database = dbDatabase;
  dbDataset->Transaction = report_transaction;
  dbDataset->UniDirectional = false;

  if (checkByProviderService->Checked) RunReport(by_provider_service);
  if (checkByService->Checked)         RunReport(by_service);
  if (checkByServiceArea->Checked)     RunReport(by_service_area);
  if (checkByAccount->Checked)         RunReport(by_account);
  return true;
}
//---------------------------------------------------------------------------
bool TformLaunchObligatedAndExpensedReports::AfterRunReport()
{
  delete dbDataset;

  bool all_is_well = TformLaunchGeneric::AfterRunReport();

  all_is_well = all_is_well && ExecuteStatement(report_transaction, "delete from temp_obpd_cobs"); checkpoint++;
  all_is_well = all_is_well && ExecuteStatement(report_transaction, "delete from temp_obpd_visits"); checkpoint++;
  all_is_well = all_is_well && ExecuteStatement(report_transaction, "delete from temp_obpd_cob_expense"); checkpoint++;
  all_is_well = all_is_well && ExecuteStatement(report_transaction, "delete from temp_obpd_bill_expense"); checkpoint++;
  all_is_well = all_is_well && ExecuteStatement(report_transaction, "delete from temp_obpd_totals"); checkpoint++;
  all_is_well = all_is_well && ExecuteStatement(report_transaction, "delete from temp_obpd_services_provided"); checkpoint++;
  return all_is_well;
}
//---------------------------------------------------------------------------
void TformLaunchObligatedAndExpensedReports::RunReport(obpd_rep_type how_to_run)
{
  /* build a totals table */

  switch (how_to_run)
  {
   case by_provider_service:  checkpoint_branch = "Provider-Service-Report"; break;
   case by_service:           checkpoint_branch = "Service-Report"; break;
   case by_service_area:      checkpoint_branch = "Service-Area-Report"; break;
   case by_account:           checkpoint_branch = "Account-Report"; break;
  }
  checkpoint = 0;

  string sql;
  // something else we get to do again, yay
  string begin_date = Pdate(dateBegin->Date).OutDate(mdy | yyyy);
  string end_date =  Pdate(dateEnd->Date).OutDate(mdy | yyyy);
  string date_title = begin_date + " - " + end_date;

  ExecuteStatement(report_transaction, "delete from temp_obpd_totals"); checkpoint++;

  string group1, group2, group_order;
  if (how_to_run == by_provider_service)
  {
    group1 = "tax_entity_id provider"; group2 = "service_id";
    group_order = "group by 2, 3 order by tax_entity_id, service_id ";
  }
  else if (how_to_run == by_service)
  {
    group1 = "service_id"; group2 = "NULL";
    group_order = "group by 2 order by service_id ";
  }
  else if (how_to_run == by_service_area)
  {
    group1 = "area_id"; group2 = "NULL";
    group_order = "group by 2 order by area_id ";
  }

  /* cob totals */
  sql = "insert into temp_obpd_totals (current_statement, group1_id, group2_id, cob_visits, cob_people, cob_amount) "
        "select 1, " + group1 + ", " + group2 + ", count(distinct(cob_id)), count(distinct(person_id)), sum(expensed) "
        "from temp_obpd_cob_expense where expensed > 0 " + group_order;
  DebugSql(this, &sql, other_forms, 9, report_transaction);
  ExecuteStatement(report_transaction, sql); checkpoint++;

  /* unpaid bill totals */
  sql = "insert into temp_obpd_totals (current_statement, group1_id, group2_id, unpaid_visits, unpaid_people, unpaid_amount) "
        "select 2, " + group1 + ", " + group2 + ", count(distinct(bill_id)), count(distinct(person_id)), sum(expensed) "
        "from temp_obpd_bill_expense where is_on_check = 0 and expensed > 0 " + group_order;
  DebugSql(this, &sql, other_forms, 9, report_transaction);
  ExecuteStatement(report_transaction, sql); checkpoint++;

  /* paid bill totals */
  sql = "insert into temp_obpd_totals (current_statement, group1_id, group2_id, paid_visits, paid_people, paid_amount) "
        "select 3, " + group1 + ", " + group2 + ", count(distinct(bill_id)), count(distinct(person_id)), sum(expensed) "
        "from temp_obpd_bill_expense where is_on_check = 1 and expensed > 0 " + group_order;
  DebugSql(this, &sql, other_forms, 9, report_transaction);
  ExecuteStatement(report_transaction, sql); checkpoint++;

  string detail_join;
  vector<string> extra_columns;
  if (how_to_run == by_provider_service)
  {
    extra_columns.push_back("OTE.name provider");
    extra_columns.push_back("service_types.item service");
    detail_join = ") on COB_SP.service_id = SP.service_id and COB_SP.tax_entity_id = SP.tax_entity_id "; // COB.services_provided_id = SP.id ";
  }
  else if (how_to_run == by_service)
  {
    if (radioDetail->Checked)
      extra_columns.push_back("OTE.name provider");
    extra_columns.push_back("service_types.item service");
    detail_join = ") on COB_SP.service_id = service_types.id ";
  }
  else if (how_to_run == by_service_area)
  {
    if (radioDetail->Checked)
      extra_columns.push_back("OTE.name provider");
    extra_columns.push_back("area_types.item area");
    detail_join = ") on COB_SP.area_id = area_types.id ";
  }

  /* final select */
  if (how_to_run == by_provider_service)
    sql = "select CAST(SP.tax_entity_id AS varchar(255)) || '--' || CAST(SP.service_id AS varchar(255)) to_group_by, ";
  else if (how_to_run == by_service)
    sql = "select CAST(service_types.id AS varchar(255)) to_group_by, ";
  else if (how_to_run == by_service_area)
    sql = "select CAST(area_types.id AS varchar(255)) to_group_by, ";
  sql +=       VecToCommaString(extra_columns) + ", "
               "cob_tot.cob_visits, cob_tot.cob_people, cob_tot.cob_amount, "
               "unpaid_tot.unpaid_visits, unpaid_tot.unpaid_people, unpaid_tot.unpaid_amount, "
               "paid_tot.paid_visits, paid_tot.paid_people, paid_tot.paid_amount ";
  if (radioDetail->Checked)
    sql += ", person_names.last_name || ', ' || person_names.first_name || case when person_names.middle_name != '' then ' ' else '' end || "
             "person_names.middle_name || case when person_names.suffix != '' then ' ' else '' end || person_names.suffix person_name, "
           "cob.cob_date, cob.cob_number, cob_exp.expensed cob_expense, "
           "unpaid.visit_date unpaid_visit_date, unpaid.bill_date unpaid_bill_date, unpaid.pr_date unpaid_pr_date, unpaid.pr_number unpaid_pr_number, unpaid.expensed unpaid_expense, "
           "paid.visit_date paid_visit_date, paid.bill_date paid_bill_date, paid.pr_date paid_pr_date, paid.pr_number paid_pr_number, paid.check_date paid_check_date, paid.check_number paid_check_number, paid.expensed paid_expense ";
  if (how_to_run == by_provider_service)
    sql += "from tax_entities OTE "
           "inner join temp_obpd_services_provided SP on SP.tax_entity_id = OTE.id "
           "inner join service_types on SP.service_id = service_types.id "
           "left join temp_obpd_totals cob_tot    on cob_tot.group1_id = SP.tax_entity_id and cob_tot.group2_id    = SP.service_id and "
                                                 "cob_tot.current_statement = 1 "
           "left join temp_obpd_totals unpaid_tot on unpaid_tot.group1_id = SP.tax_entity_id and unpaid_tot.group2_id = SP.service_id and "
                                                 "unpaid_tot.current_statement = 2 "
           "left join temp_obpd_totals paid_tot   on paid_tot.group1_id = SP.tax_entity_id and paid_tot.group2_id   = SP.service_id and "
                                                 "paid_tot.current_statement = 3 ";
  else if (how_to_run == by_service)
    sql += "from service_types "
           "left join temp_obpd_totals cob_tot    on cob_tot.group1_id    = service_types.id and cob_tot.current_statement = 1 "
           "left join temp_obpd_totals unpaid_tot on unpaid_tot.group1_id = service_types.id and unpaid_tot.current_statement = 2 "
           "left join temp_obpd_totals paid_tot   on paid_tot.group1_id   = service_types.id and paid_tot.current_statement = 3 ";
  else if (how_to_run == by_service_area)
    sql += "from area_types "
           "left join temp_obpd_totals cob_tot    on cob_tot.group1_id    = area_types.id and cob_tot.current_statement = 1 "
           "left join temp_obpd_totals unpaid_tot on unpaid_tot.group1_id = area_types.id and unpaid_tot.current_statement = 2 "
           "left join temp_obpd_totals paid_tot   on paid_tot.group1_id   = area_types.id and paid_tot.current_statement = 3 ";
  if (radioDetail->Checked)
  {
    sql += "inner join "
           "(temp_obpd_cobs COB "
            "left join temp_obpd_cob_expense cob_exp on cob_exp.cob_id = COB.id " // cob_exp.tax_entity_id = SP.tax_entity_id and cob_exp.service_id = SP.service_id "
            "left join temp_obpd_bill_expense unpaid on unpaid.cob_id = COB.id and " //unpaid.tax_entity_id = SP.tax_entity_id and unpaid.service_id = SP.service_id and "
                                                       "unpaid.is_on_check = 0 "
            "left join temp_obpd_bill_expense paid on paid.cob_id = COB.id and " //paid.tax_entity_id = SP.tax_entity_id and paid.service_id = SP.service_id and "
                                                     "paid.is_on_check = 1 "
            "left join services_provided COB_SP on COB_SP.id = COB.services_provided_id ";

    if (how_to_run != by_provider_service)
      sql += "left join tax_entities OTE on COB_SP.tax_entity_id = OTE.id ";

    sql  += detail_join + // and ( ... ) // not necessary?
            // nasty ... but we don't know which thing the person's attached to.... // nevermind...this became unexpectedly easy the new way
            "left join people on people.id = COB.person_id "
                                       //"case when cob_exp.cob_id is not null then cob_exp.person_id "
                                       //"else case when unpaid.bill_id is not null then unpaid.person_id "
                                       //     "else paid.person_id end end "
            "left join person_names on people.person_names_id = person_names.id ";
  }
  sql += "where (cob_tot.cob_visits is not null or unpaid_tot.unpaid_visits is not null or paid_tot.paid_visits is not null) ";
  if (how_to_run == by_provider_service || how_to_run == by_service)
    sql += "and " + SafeInClause("service_types.id", service_ids) + " ";
  if (how_to_run == by_service_area)
    sql += "and " + SafeInClause("area_types.id", service_area_ids) + " ";
  if (how_to_run == by_provider_service)
    sql += "order by OTE.name, service_types.item";
  else if (how_to_run == by_service)
    sql += "order by service_types.item";
  else if (how_to_run == by_service_area)
    sql += "order by area_types.item";

  if (radioDetail->Checked)
    sql += ", cob_exp.cob_date desc nulls last, last_name, first_name, middle_name, suffix, cob_exp.cob_number nulls last, "
           "unpaid.visit_date desc nulls last, unpaid.bill_date desc nulls last, unpaid.pr_date desc nulls last, unpaid.pr_number nulls last, "
           "paid.visit_date desc nulls last, paid.bill_date desc nulls last, "
           "paid.pr_date desc nulls last, paid.pr_number nulls last, paid.check_date desc nulls last, paid.check_number nulls last ";

  DebugSql(this, &sql, other_forms, 9, report_transaction);

  dbDataset->SelectSQL->Clear();
  dbDataset->SelectSQL->Add(sql.c_str());
  dbDataset->Open();
  dbDataset->First();

  TformReportObligatedAndExpensed *report = NULL;
  FORM_TRY
    report = new TformReportObligatedAndExpensed(NULL);
  FORM_CATCH

  string title = "(no title?)";
  switch (how_to_run)
  {
   case by_provider_service:  title = "Expensed Amounts xxx By Provider Name"; break;
   case by_service:           title = "Expensed Amounts xxx By Service"; break;
   case by_service_area:      title = "Expensed Amounts xxx By Service Area"; break;
   case by_account:           title = "Expensed Amounts xxx By Account"; break;
  }
  if (radioAllPeople->Checked) title += " - Including All Patients";
  else if (radioCatchment->Checked) title += " - Including Only Patients " + SentenceString(cboCatchment->Text.c_str()) + " Our Catchment Area";
  else if (radioState->Checked) title += " - Including Only Patients " + SentenceString(cboState->Text.c_str()) + " the State";
  report->lblTitle->Caption = ReplaceStringWithString(title, "xxx", radioSummary->Checked ? "Summarized" : "Detailed").c_str();

  report->reportByProviderService->DataSet = dbDataset;
  report->lblDateTitle->Caption = date_title.c_str();
  report->lblUser->Caption = current_fullname.c_str();
  report->txtProvider->DataSet = dbDataset;
  report->txtProvider->DataField = "provider";
  report->txtService->DataSet = dbDataset;
  if (how_to_run == by_service_area)
    report->txtService->DataField = "area";
  else
    report->txtService->DataField = "service";
  report->txtCobVisits->DataSet = dbDataset;
  report->txtCobVisits->DataField = "cob_visits";
  report->txtCobPeople->DataSet = dbDataset;
  report->txtCobPeople->DataField = "cob_people";
  report->exprCobAmount->Expression = "FORMATNUMERIC('$###,###,##0.00', cob_amount)";
  //report->exprAvgObligatedPerVisit->Expression = "FORMATNUMERIC('$###,###,##0.00', avg_obligated_per_visit)";
  //report->exprAvgObligatedPerMember->Expression = "FORMATNUMERIC('$###,###,##0.00', avg_obligated_per_member)";
  report->txtUnpaidVisits->DataSet = dbDataset;
  report->txtUnpaidVisits->DataField = "unpaid_visits";
  report->txtUnpaidPeople->DataSet = dbDataset;
  report->txtUnpaidPeople->DataField = "unpaid_people";
  report->exprUnpaidAmount->Expression = "FORMATNUMERIC('$###,###,##0.00', unpaid_amount)";
  report->txtPaidVisits->DataSet = dbDataset;
  report->txtPaidVisits->DataField = "paid_visits";
  report->txtPaidPeople->DataSet = dbDataset;
  report->txtPaidPeople->DataField = "paid_people";
  report->exprPaidAmount->Expression = "FORMATNUMERIC('$###,###,##0.00', paid_amount)";

  //report->exprTotalAvgObligatedPerVisit->Expression = "IF(SUM(obligated_cobs) > 0, FORMATNUMERIC('$###,###,##0.00', (SUM(obligated_amount) / SUM(obligated_cobs))), '$0.00')";
  //report->exprTotalAvgObligatedPerMember->Expression = "IF(SUM(obligated_members) > 0, FORMATNUMERIC('$###,###,##0.00', (SUM(obligated_amount) / SUM(obligated_members))), '$0.00')";
  //report->exprTotalAvgCostPerVisit->Expression = "IF(SUM(paid_visit_count) > 0, FORMATNUMERIC('$###,###,##0.00', (SUM(paid_amount) / SUM(paid_visit_count))), '$0.00')";
  //report->exprTotalAvgCostPerMember->Expression = "IF(SUM(paid_member_count) > 0, FORMATNUMERIC('$###,###,##0.00', (SUM(paid_amount) / SUM(paid_member_count))), '$0.00')";

  // awwwwww yeahhh
  string restrict_to = "";
  if (how_to_run == by_provider_service || how_to_run == by_service)
    restrict_to += "and " + SafeInClause("service_id", service_ids) + " ";
  if (how_to_run == by_service_area)
    restrict_to += "and " + SafeInClause("area_id", service_area_ids) + " ";
  sql = "select count(distinct(person_id)) from temp_obpd_cob_expense where expensed > 0 " + restrict_to;
  DebugSql(this, &sql, other_forms, 9, report_transaction);
  report->lblTotalCobPeople->Caption = GetOneField(report_transaction, sql).c_str();
  sql = "select count(distinct(person_id)) from temp_obpd_bill_expense where is_on_check = 0 and expensed > 0 " + restrict_to;
  DebugSql(this, &sql, other_forms, 9, report_transaction);
  report->lblTotalUnpaidPeople->Caption = GetOneField(report_transaction, sql).c_str();
  sql = "select count(distinct(person_id)) from temp_obpd_bill_expense where is_on_check = 1 and expensed > 0 " + restrict_to;
  DebugSql(this, &sql, other_forms, 9, report_transaction);
  report->lblTotalPaidPeople->Caption = GetOneField(report_transaction, sql).c_str();

  report->bandGroupHeader->Expression = "to_group_by";
  report->bandGroupHeader->Height = 22;

  if (how_to_run == by_service_area)
    report->lblService->Caption = "Service Area";

  if (radioDetail->Checked)
  {
    report->bandDetail->Height = 16;
    report->bandChild->Enabled = true;
    report->bandChild->Height = 16;

    report->txtPersonName->DataSet = dbDataset;
    report->txtPersonName->DataField = "person_name";
    report->txtCobDate->DataSet = dbDataset;
    report->txtCobDate->DataField = "cob_date";
    report->txtCobNumber->DataSet = dbDataset;
    report->txtCobNumber->DataField = "cob_number";
    report->txtCobExpense->DataSet = dbDataset;
    report->txtCobExpense->DataField = "cob_expense";                   // TODO:  Expense fields should be Expression's, FORMATNUMERIC'd
    report->txtUnpaidVisitDate->DataSet = dbDataset;
    report->txtUnpaidVisitDate->DataField = "unpaid_visit_date";
    report->txtUnpaidBillDate->DataSet = dbDataset;
    report->txtUnpaidBillDate->DataField = "unpaid_bill_date";
    report->txtUnpaidPrDate->DataSet = dbDataset;
    report->txtUnpaidPrDate->DataField = "unpaid_pr_date";
    report->txtUnpaidPrNumber->DataSet = dbDataset;
    report->txtUnpaidPrNumber->DataField = "unpaid_pr_number";
    report->txtUnpaidExpense->DataSet = dbDataset;
    report->txtUnpaidExpense->DataField = "unpaid_expense";
    report->txtPaidVisitDate->DataSet = dbDataset;
    report->txtPaidVisitDate->DataField = "paid_visit_date";
    report->txtPaidBillDate->DataSet = dbDataset;
    report->txtPaidBillDate->DataField = "paid_bill_date";
    report->txtPaidPrDate->DataSet = dbDataset;
    report->txtPaidPrDate->DataField = "paid_pr_date";
    report->txtPaidPrNumber->DataSet = dbDataset;
    report->txtPaidPrNumber->DataField = "paid_pr_number";
    report->txtPaidCheckDate->DataSet = dbDataset;
    report->txtPaidCheckDate->DataField = "paid_check_date";
    report->txtPaidCheckNumber->DataSet = dbDataset;
    report->txtPaidCheckNumber->DataField = "paid_check_number";
    report->txtPaidExpense->DataSet = dbDataset;
    report->txtPaidExpense->DataField = "paid_expense";

    report->exprTotalCobVisits->Expression = "SUM(cob_visits)";
    report->exprTotalCobPeople->Expression = "SUM(cob_people)";
    report->exprTotalCobAmount->Expression = "FORMATNUMERIC('$###,###,##0.00', SUM(cob_expense))";
    report->exprTotalUnpaidVisits->Expression = "SUM(unpaid_visits)";
    report->exprTotalUnpaidPeople->Expression = "SUM(unpaid_people)";
    report->exprTotalUnpaidAmount->Expression = "FORMATNUMERIC('$###,###,##0.00', SUM(unpaid_expense))";
    report->exprTotalPaidVisits->Expression = "SUM(paid_visits)";
    report->exprTotalPaidPeople->Expression = "SUM(paid_people)";
    report->exprTotalPaidAmount->Expression = "FORMATNUMERIC('$###,###,##0.00', SUM(paid_expense))";

    if (how_to_run == by_service || how_to_run == by_service_area)
    {
      report->txtProvider->Enabled = false;      // because this data isn't useful in by_service mode
      report->lblProvider->Enabled = false;      // because that column is now empty
      report->txtDetailProvider->Enabled = true; // because we don't have a provider on the group header rows so we put it on the per-visit rows
      report->txtDetailProvider->DataSet = dbDataset;
      report->txtDetailProvider->DataField = "provider"; // and populate it, yup
    }
  }
  else
  {
    report->bandDetail->Height = 0;
    report->bandChild->Enabled = false;
    report->bandChild->Height = 0;

    report->exprTotalCobVisits->Expression = "SUM(cob_visits)";
    report->exprTotalCobPeople->Expression = "SUM(cob_people)";
    report->exprTotalCobAmount->Expression = "FORMATNUMERIC('$###,###,##0.00', SUM(cob_amount))";
    report->exprTotalUnpaidVisits->Expression = "SUM(unpaid_visits)";
    report->exprTotalUnpaidPeople->Expression = "SUM(unpaid_people)";
    report->exprTotalUnpaidAmount->Expression = "FORMATNUMERIC('$###,###,##0.00', SUM(unpaid_amount))";
    report->exprTotalPaidVisits->Expression = "SUM(paid_visits)";
    report->exprTotalPaidPeople->Expression = "SUM(paid_people)";
    report->exprTotalPaidAmount->Expression = "FORMATNUMERIC('$###,###,##0.00', SUM(paid_amount))";
  }

  acc_info CHS_budget = InfoById("1007");  // TODO -c MAGIC_NUMBER: Governor wanted total budget for IHS->OSG->Contract Health Services
  financial CHS_budget_amount = CHS_budget.GetCurrentBalance();

  // report->lblCHSBudgetTotalAmt->Caption = ("$" + FinancialToStringEx(CHS_budget_amount)).c_str();
  // report->exprCHSBudgetExpensedAmt->Expression = "FORMATNUMERIC('$###,###,##0.00', (SUM(obligated_amount) + SUM(paid_amount)))";
  // report->exprCHSBudgetRemainingAmt->Expression = ("FORMATNUMERIC('$###,###,##0.00', (" + FinancialToString(CHS_budget_amount, false, 2) + " - SUM(obligated_amount) - SUM(paid_amount)))").c_str();

  Processing(false);
  report->reportByProviderService->Preview();

  dbDataset->Close();
}
//---------------------------------------------------------------------------


