//---------------------------------------------------------------------------

#include <vcl.h>
#pragma hdrstop

#include "launchObligatedProviderReport.h"
#include "reportObligatedProvider.h"
//---------------------------------------------------------------------------
#pragma package(smart_init)
#pragma link "codeSpecialGeneric"
#pragma resource "*.dfm"
TformLaunchObligatedProviderReport *formLaunchObligatedProviderReport;
//---------------------------------------------------------------------------
__fastcall TformLaunchObligatedProviderReport::TformLaunchObligatedProviderReport(TComponent* Owner)
        : TformSpecialGeneric(Owner)
{
  Pdate year_begin;
  year_begin.InDate(Date());
  year_begin.month = 1;
  year_begin.day = 1;
  dateBegin->Date = year_begin.OutDate();
  dateEnd->Date = Date();
  services_filter.SetListViewHandle(listServices);
  services_filter.SetSecurity("services_provided");
  services_filter.AddColumnSpec("Service", "people", as_text, true, 180);
  services_filter.AddColumnSpec("Internal Id", "internal_id", as_int, false, 80);
  services_filter.FetchPreferences("launchobligatedprovidereport.services_filter");
  services_filter.SetCheck(true);
  services_filter.SetCompress(true);
  services_filter.SetSQLStatement("select item, id from service_types where valid = 1");
  services_filter.RefreshDataSet();
  set<string> id_set;
  id_set.insert("0");
  id_set.insert(lookupTables->IdOf("service_types", "Respite Care")); // this is a special program under Community Health, and its information is seperate from Contract Health's services
  services_filter.SetCheckedWhereNotID(id_set, true);
}
//---------------------------------------------------------------------------
void __fastcall TformLaunchObligatedProviderReport::btnCloseClick(TObject *Sender)
{
  Close();
}
//---------------------------------------------------------------------------
void __fastcall TformLaunchObligatedProviderReport::FormShortCut(TWMKey &Msg, bool &Handled)
{
  Handled = true;
  if (Msg.CharCode == VK_RETURN)
    btnRun->Click();
  else
    Handled = false;

  if (!Handled)
    TformSpecialGeneric::FormShortCut(Msg, Handled);
}
//---------------------------------------------------------------------------
void __fastcall TformLaunchObligatedProviderReport::FormClose(TObject *Sender, TCloseAction &Action)
{
  TformSpecialGeneric::FormClose(Sender, Action);
}
//---------------------------------------------------------------------------
void __fastcall TformLaunchObligatedProviderReport::btnRunClick(TObject *Sender)
{
  vector<string> checked_ids = services_filter.GetMultiID();
  if(! checked_ids.empty())
  {
    Processing(true);

    TIBTransaction * dbTransaction = new TIBTransaction(NULL);
    TIBDataSet * dbDataset = new TIBDataSet(NULL);
    string sql;
    string begin_date = Pdate(dateBegin->Date).OutDate(mdy | yyyy);
    string end_date =  Pdate(dateEnd->Date).OutDate(mdy | yyyy);
    string date_title = begin_date + " - " + end_date;

    try
    {
      dbTransaction->DefaultDatabase = dbDatabase;
      dbDataset->Database = dbDatabase;
      dbDataset->Transaction = dbTransaction;
      dbDataset->UniDirectional = false;
      dbTransaction->StartTransaction();

      ExecuteStatement(dbTransaction, "delete from temp_child_cob");
      ExecuteStatement(dbTransaction, "delete from temp_obligated_cob");
      ExecuteStatement(dbTransaction, "delete from temp_bill_amounts");
      ExecuteStatement(dbTransaction, "delete from temp_obligated");
      ExecuteStatement(dbTransaction, "delete from temp_paid_visits");
      ExecuteStatement(dbTransaction, "delete from temp_obligated_report");

      // NOTE USES VIEW
      ExecuteStatement(dbTransaction, "insert into temp_child_cob select id, parent_task_id, tax_entity_id from visit_tasks "
                      "where visit_tasks.tax_entity_id != 0");
      // NOTE USES VIEW
      ExecuteStatement(dbTransaction, "insert into temp_obligated_cob select id, person_id, services_provided_id "
                      "from cob_tasks where cob_tasks.date_of_task between '" + begin_date + "' and '" + end_date + "' "
                      " and not exists (select temp_child_cob.id from temp_child_cob "
                      "where temp_child_cob.parent_task_id = cob_tasks.id)");
      ExecuteStatement(dbTransaction, "insert into temp_bill_amounts select bills.id, bills.task_id, sum(transactions.credit_amount - transactions.debit_amount) "
                      "from checks inner join purchase_reqs on purchase_reqs.check_id = checks.id "
                      "inner join bills on bills.purchase_req_id = purchase_reqs.id "
                      "left join transactions on transactions.bill_id = bills.id "
                      "and transactions.account_id in (select accounts.id from accounts where accounts.account_type_id = " + lookupTables->IdOf("account_types", "Cash") + ") "
                      "where checks.date_sent between '" + begin_date + "' and '" + end_date + "' "
                      "group by bills.id, bills.task_id, checks.date_sent");

      // COB tasks with no children
      sql = "insert into temp_obligated (service_provided_id, /* tax_entity_id, */ obligated_cobs, obligated_members, obligated_amount) "
            "select obligated_cob.services_provided_id, "
            "count(obligated_cob.id) visit_count, count(distinct(obligated_cob.person_id)) member_count, "
            "sum(transactions.debit_amount - transactions.credit_amount) obligated_amount "
            "from temp_obligated_cob obligated_cob "
            "inner join transactions on transactions.task_id = obligated_cob.id and "
            "transactions.account_id in (select accounts.id from accounts where "
            "accounts.account_type_id = " + lookupTables->IdOf("account_types", "Expense") + ") "
            "group by obligated_cob.services_provided_id";
      ExecuteStatement(dbTransaction, sql);

      // COB tasks w children and has checks
      // NOTE USES VIEW
      sql = "insert into temp_paid_visits (service_provided_id, paid_visits, paid_members, paid_amount) "
            "select parent_cob.services_provided_id, "
            "count(child_cob.id) visit_count, count(distinct(parent_cob.person_id)) member_count, "
            "sum(bill_amounts.cash_amount) paid_invoices "
            "from cob_tasks parent_cob "
            "inner join temp_child_cob child_cob on parent_cob.id = child_cob.parent_task_id "
            "inner join temp_bill_amounts bill_amounts on bill_amounts.task_id = child_cob.id "
            "group by parent_cob.services_provided_id";
      ExecuteStatement(dbTransaction, sql);

      sql = "insert into temp_obligated_report (service_id, provider, service_type, obligated_cobs, "
            "obligated_members, obligated_amount, paid_visit_count, paid_member_count, paid_amount) "
            "select service_types.id, tax_entities.name, service_types.item, "
            "sum(temp_obligated.obligated_cobs), sum(temp_obligated.obligated_members), "
            "sum(temp_obligated.obligated_amount), sum(temp_paid_visits.paid_visits), "
            "sum(temp_paid_visits.paid_members), sum(temp_paid_visits.paid_amount) "
            "from tax_entities "
            "inner join services_provided on services_provided.tax_entity_id = tax_entities.id "
            "inner join service_types on services_provided.service_id = service_types.id "
            "left join temp_paid_visits on temp_paid_visits.service_provided_id = services_provided.id "
            "left join temp_obligated on temp_obligated.service_provided_id = services_provided.id "

            "group by tax_entities.name, service_types.id, service_types.item";
      ExecuteStatement(dbTransaction, sql);

      // the quick reports stupidness has just about outdone itself. it refuses to show 0's in the report even
      // though i'm using the formatnumeric function. if i use the onprint with a regular text field, it doesn't
      // happen when the field is null, which is when i need it to. the expression field also refuses to do
      // the division for some reason. so, i'm selecting the final report into this unnecessary temp table
      // and running these update statemetns so i can select the whole thing at the end
      sql = "update temp_obligated_report set obligated_cobs = 0 where obligated_cobs is null";
      ExecuteStatement(dbTransaction, sql);
      sql = "update temp_obligated_report set obligated_members = 0 where obligated_members is null";
      ExecuteStatement(dbTransaction, sql);
      sql = "update temp_obligated_report set avg_obligated_per_visit = (obligated_amount / obligated_cobs) "
            "where obligated_cobs > 0";
      ExecuteStatement(dbTransaction, sql);
      sql = "update temp_obligated_report set avg_obligated_per_visit = 0 where avg_obligated_per_visit is null";
      ExecuteStatement(dbTransaction, sql);
      sql = "update temp_obligated_report set avg_obligated_per_member = (obligated_amount / obligated_members) "
            "where obligated_members > 0";
      ExecuteStatement(dbTransaction, sql);
      sql = "update temp_obligated_report set avg_obligated_per_visit = 0 where avg_obligated_per_member is null";
      ExecuteStatement(dbTransaction, sql);
      sql = "update temp_obligated_report set paid_visit_count = 0 where paid_visit_count is null";
      ExecuteStatement(dbTransaction, sql);
      sql = "update temp_obligated_report set paid_member_count = 0 where paid_member_count is null";
      ExecuteStatement(dbTransaction, sql);
      sql = "update temp_obligated_report set avg_paid_per_visit = (paid_amount / paid_visit_count) "
            "where paid_visit_count > 0";
      ExecuteStatement(dbTransaction, sql);
      sql = "update temp_obligated_report set avg_paid_per_visit = 0 where avg_paid_per_visit is null";
      ExecuteStatement(dbTransaction, sql);
      sql = "update temp_obligated_report set avg_paid_per_member = (paid_amount / paid_member_count) "
            "where paid_member_count > 0";
      ExecuteStatement(dbTransaction, sql);
      sql = "update temp_obligated_report set avg_paid_per_member = 0 where avg_paid_per_member is null";
      ExecuteStatement(dbTransaction, sql);

      sql = "select provider, service_type, obligated_cobs, obligated_members, obligated_amount, "
            "avg_obligated_per_visit, avg_obligated_per_member, "
            "paid_visit_count, paid_member_count, paid_amount, "
            "avg_paid_per_visit, avg_paid_per_member "
            "from temp_obligated_report "
            "where (obligated_members > 0 or "
            "paid_member_count > 0) and "
            "service_id in (" + VecToCommaString(checked_ids) + ") "
            "order by provider, service_type";

      dbDataset->SelectSQL->Add(sql.c_str());
      dbDataset->Open();
      dbDataset->First();

      FORM_TRY
        TformReportObligatedProvider *report = new TformReportObligatedProvider(NULL);
        report->report->DataSet = dbDataset;
        report->lblDateTitle->Caption = date_title.c_str();
        report->lblUser->Caption = current_fullname.c_str();
        report->txtProvider->DataSet = dbDataset;
        report->txtProvider->DataField = "provider";
        report->txtService->DataSet = dbDataset;
        report->txtService->DataField = "service_type";
        report->txtObligatedVisits->DataSet = dbDataset;
        report->txtObligatedVisits->DataField = "obligated_cobs";
        report->txtObligatedMembers->DataSet = dbDataset;
        report->txtObligatedMembers->DataField = "obligated_members";
        report->exprObligatedAmount->Expression = "FORMATNUMERIC('$###,###,##0.00', obligated_amount)";
        report->exprAvgObligatedPerVisit->Expression = "FORMATNUMERIC('$###,###,##0.00', avg_obligated_per_visit)";
        report->exprAvgObligatedPerMember->Expression = "FORMATNUMERIC('$###,###,##0.00', avg_obligated_per_member)";
        report->txtVisits->DataSet = dbDataset;
        report->txtVisits->DataField = "paid_visit_count";
        report->txtMembers->DataSet = dbDataset;
        report->txtMembers->DataField = "paid_member_count";
        report->exprPaidInvoices->Expression = "FORMATNUMERIC('$###,###,##0.00', paid_amount)";
        report->exprAvgCostPerVisit->Expression = "FORMATNUMERIC('$###,###,##0.00', avg_paid_per_visit)";
        report->exprAvgCostPerMember->Expression = "FORMATNUMERIC('$###,###,##0.00', avg_paid_per_member)";
        report->exprTotalObligatedVisits->Expression = "SUM(obligated_cobs)";
        report->exprTotalObligatedMembers->Expression = "SUM(obligated_members)";
        report->exprTotalObligatedAmount->Expression = "FORMATNUMERIC('$###,###,##0.00', SUM(obligated_amount))";
        report->exprTotalAvgObligatedPerVisit->Expression = "IF(SUM(obligated_cobs) > 0, FORMATNUMERIC('$###,###,##0.00', (SUM(obligated_amount) / SUM(obligated_cobs))), '$0.00')";
        report->exprTotalAvgObligatedPerMember->Expression = "IF(SUM(obligated_members) > 0, FORMATNUMERIC('$###,###,##0.00', (SUM(obligated_amount) / SUM(obligated_members))), '$0.00')";
        report->exprTotalVisits->Expression = "SUM(paid_visit_count)";
        report->exprTotalMembers->Expression = "SUM(paid_member_count)";
        report->exprTotalPaidAmount->Expression = "FORMATNUMERIC('$###,###,##0.00', SUM(paid_amount))";
        report->exprTotalAvgCostPerVisit->Expression = "IF(SUM(paid_visit_count) > 0, FORMATNUMERIC('$###,###,##0.00', (SUM(paid_amount) / SUM(paid_visit_count))), '$0.00')";
        report->exprTotalAvgCostPerMember->Expression = "IF(SUM(paid_member_count) > 0, FORMATNUMERIC('$###,###,##0.00', (SUM(paid_amount) / SUM(paid_member_count))), '$0.00')";

        acc_info CHS_budget = InfoById("1007");  // TODO -c MAGIC_NUMBER: Governor wanted total budget for IHS->OSG->Contract Health Services
        financial CHS_budget_amount = CHS_budget.GetCurrentBalance();

        // report->lblCHSBudgetTotalAmt->Caption = ("$" + FinancialToStringEx(CHS_budget_amount)).c_str();
        // report->exprCHSBudgetExpensedAmt->Expression = "FORMATNUMERIC('$###,###,##0.00', (SUM(obligated_amount) + SUM(paid_amount)))";
        // report->exprCHSBudgetRemainingAmt->Expression = ("FORMATNUMERIC('$###,###,##0.00', (" + FinancialToString(CHS_budget_amount, false, 2) + " - SUM(obligated_amount) - SUM(paid_amount)))").c_str();

        Processing(false);
        report->report->Preview();
      FORM_CATCH

      dbDataset->Close();
      // dbTransaction->Commit();
      // dbTransaction->StartTransaction();
      ExecuteStatement(dbTransaction, "delete from temp_child_cob");
      ExecuteStatement(dbTransaction, "delete from temp_obligated_cob");
      ExecuteStatement(dbTransaction, "delete from temp_bill_amounts");
      ExecuteStatement(dbTransaction, "delete from temp_obligated");
      ExecuteStatement(dbTransaction, "delete from temp_paid_visits");
      ExecuteStatement(dbTransaction, "delete from temp_obligated_report");
      dbTransaction->Commit();
    }
    catch (...)
    {
      Processing(false);
      if (dbTransaction->Active)
        dbTransaction->Rollback();
      PSay("Error running report", m_database_error, MB_OK);
    }
    delete dbTransaction;
    delete dbDataset;
  }
  else
    PSay("No services are checked", m_missing_data, MB_OK);
}
//---------------------------------------------------------------------------

