//---------------------------------------------------------------------------

#include <vcl.h>
#pragma hdrstop

#include "launchOptometryAppointments.h"
#include "reportOptometryAppointments.h"
//---------------------------------------------------------------------------
#pragma package(smart_init)
#pragma link "launchGeneric"
#pragma resource "*.dfm"
TformLaunchOptometryAppointments *formLaunchOptometryAppointments;
//---------------------------------------------------------------------------
__fastcall TformLaunchOptometryAppointments::TformLaunchOptometryAppointments(TComponent* Owner)
        : TformLaunchGeneric(Owner)
{
  // Set default dates
  pair<Pdate, Pdate> last_month_dates = GetPriorMonthDateRange(Pdate(Date()));
  dateBegin->Date = last_month_dates.first.OutDate();
  dateEnd->Date   = last_month_dates.second.OutDate();
}
//---------------------------------------------------------------------------
bool TformLaunchOptometryAppointments::IsValidOnScreen()
{
  if (Pdate(dateBegin->Date) > Pdate(dateEnd->Date))
  {
    PSay("Please select a begin date prior or equal to the end date", m_invalid_data, MB_OK);
    dateBegin->SetFocus();
    return false;
  }

  return TformLaunchGeneric::IsValidOnScreen();
}
//---------------------------------------------------------------------------
bool TformLaunchOptometryAppointments::BeforeRunReport()
{
  dbDataSetReasons  = AddDataSet();
  return TformLaunchGeneric::BeforeRunReport();
}
//---------------------------------------------------------------------------
bool TformLaunchOptometryAppointments::RunReport()
{
  string group_id   = lookupTables->IdOf("groups", "Shawnee Clinic"); // Optometry is inside Shawnee Clinic Dept.
  string begin_date = Pdate(dateBegin->Date).OutDate(mdy | yyyy);
  string end_date   = Pdate(dateEnd->Date).OutDate(mdy | yyyy);
  string date_title = begin_date + " - " + end_date;

  // Create the report form
  TformReportOptometryAppointments *report_form = NULL;
  FORM_TRY
    report_form = new TformReportOptometryAppointments(NULL);
  FORM_CATCH

  // Main query - Optometry appointments by reason, diabetic and non-diabetic
  string appts_by_reason_sql =
    "select case when ART.item = '' then '(Unspecified)' else ART.item end reason, "
    "cast(sum(case when COND.id is null then 1 else 0 end) as integer) sum_nondiabetic, "
    "cast(sum(case when COND.id is not null then 1 else 0 end) as integer) sum_diabetic, "
    "count(distinct A.id) count_total "
    "from appointments A "
    "inner join appt_reason_types ART on A.appt_reason_id = ART.id "
    "left  join conditions COND on COND.id = "               // Find first diabetic condition with valid status and type
    "(select first 1 ICOND.id from conditions ICOND where ICOND.person_id = A.person_id and "
    " (ICOND.code = '250' or ICOND.code like '250.%') and ICOND.current_id >= 3 and "
    " ICOND.condition_id = '" + lookupTables->IdOf("condition_types", "CONDITION") + "') "
    "where A.current_id > 1 and A.group_id = '" + group_id + "' and "
    "ART.appt_service_id = '" + lookupTables->IdOf("appt_service_types", "Optometry") + "' and "
    "ART.id <> " + lookupTables->IdOf("appt_reason_types", "Block Schedule") + " and " // jhaley 20110720
    "(A.appointment_date between '" + begin_date + "' and '" + end_date + "' or "
    " A.begin_date between '" + begin_date + "' and '" + end_date + "' or "
    " A.end_date between '" + begin_date + "' and '" + end_date + "') "
    "group by 1 "
    "order by 1 ";

  InitDataSet(dbDataSetReasons, appts_by_reason_sql);

  report_form->reportOpto->DataSet = dbDataSetReasons;

  // Title band
  report_form->lblTitleDates->Caption = date_title.c_str();  // display dates on title band

  // Page footer
  report_form->lblUser->Caption = current_fullname.c_str(); // display generating user's name on page footer

  // Detail band
  SetupQRDBText(report_form->txtDetailReason, dbDataSetReasons, "reason");
  SetupQRDBText(report_form->txtDetailNonDm,  dbDataSetReasons, "sum_nondiabetic");
  SetupQRDBText(report_form->txtDetailDM,     dbDataSetReasons, "sum_diabetic");
  SetupQRDBText(report_form->txtDetailTotal,  dbDataSetReasons, "count_total");

  // Summary band
  report_form->exprSummaryNonDM->Expression = "SUM(sum_nondiabetic)";
  report_form->exprSummaryDM->Expression    = "SUM(sum_diabetic)";
  report_form->exprSummaryTotal->Expression = "SUM(count_total)";

  // Do the appointments-by-outcome breakdown
  string appts_by_outcome_sql =
    "select case when VOT.item = '' then '(Unspecified)' else VOT.item end outcome, count(A.id) sum_outcome "
    "from appointments A "
    "inner join visit_outcome_types VOT on A.visit_outcome_id = VOT.id "
    "inner join appt_reason_types ART on A.appt_reason_id = ART.id "
    "where A.current_id > 1 and A.group_id = '" + group_id + "' and "
    "ART.appt_service_id = '" + lookupTables->IdOf("appt_service_types", "Optometry") + "' and "
    "(A.appointment_date between '" + begin_date + "' and '" + end_date + "' or "
    " A.begin_date between '" + begin_date + "' and '" + end_date + "' or "
    " A.end_date between '" + begin_date + "' and '" + end_date + "') "
    "group by 1 "
    "order by 1 ";

  vector<map<string, string> > appts_by_outcome;
  if(SqlToVecMap(dbDatabase, appts_by_outcome_sql, appts_by_outcome))
  {
    report_form->memoOutcomes->Lines->Clear();
    report_form->memoAppts->Lines->Clear();

    for(vector<map<string, string> >::iterator i_appt = appts_by_outcome.begin(); i_appt != appts_by_outcome.end(); ++i_appt)
    {
      report_form->memoOutcomes->Lines->Add(PrettyString((*i_appt)["outcome"]).c_str());
      report_form->memoAppts->Lines->Add((*i_appt)["sum_outcome"].c_str());
    }
  }

  report_form->reportOpto->Preview();

  return true;
}
//---------------------------------------------------------------------------


