//---------------------------------------------------------------------------

#include <vcl.h>
#pragma hdrstop

#include "launchPRReport.h"
// #include "reportRequsitionJustification.h"
#include "reportBulkEOB.h"
//---------------------------------------------------------------------------
#pragma package(smart_init)
#pragma link "codeSpecialGeneric"
#pragma resource "*.dfm"
TformLaunchPRReport *formLaunchPRReport;
//---------------------------------------------------------------------------
__fastcall TformLaunchPRReport::TformLaunchPRReport(TComponent* Owner, string p_id)
        : TformSpecialGeneric(Owner)
{
  id = p_id;
  if((id == "") || (!IsInt(id)) || (StringToInt(id) < 1))
  {
    PSay("The id of the PR is invalid", m_program_error, MB_OK, "id = " + id);
    btnRun->Enabled = false;
  }
  else
  {
    // PHILIP -- apparently, we determine ahead of time whether or not this PR has been modified
    //        -- probably needs to be changed, along with other code
    /*
    string sql = "select count(transactions.id) "
                 "from purchase_reqs "
                 "inner join bills on purchase_reqs.id = bills.purchase_req_id "
                 "inner join transactions on bills.id = transactions.bill_id "
                 "where purchase_reqs.id = " + id + " and "
                 "cast(transactions.last_update_stamp as date) > purchase_reqs.date_sent";
    int count = StringToInt(GetOneField(dbDatabase, sql));
    if (count > 0)
      transactions_modified = true;
    else
    */
    transactions_modified = false;

    // PHILIP -- we also determine ahead of time if this is a vendor or provider PR
    // PHILIP -- this changes the defaults on which extra sheets to print, and on labels to be shown on forms
    string sql_get_bill_type =
      "select first 1 bills.bill_id "
      "from purchase_reqs "
      "inner join bills on purchase_reqs.id = bills.purchase_req_id "
      "where purchase_reqs.id = " + id;

    string temp = GetOneField(dbDatabase, sql_get_bill_type);
    if(temp == lookupTables->IdOf("bill_types", "Incoming vendor bill"))
    {
      pr_type = vendor;

      _labelJustificationProcedures->Enabled = false;
      editJustificationProcedures->Enabled   = false;
      spinJustificationProcedures->Enabled   = false;

      _labelJustificationNoProcedures->Enabled = false;
      editJustificationNoProcedures->Enabled   = false;
      spinJustificationNoProcedures->Enabled   = false;

      _labelEOB->Enabled = false;
      editEOB->Enabled   = false;
      spinEOB->Enabled   = false;
    }
    else
    {
      pr_type = provider;
      // spinUnnamed->Position = 0;
      spinJustificationProcedures->Position = 2;
      spinJustificationNoProcedures->Position = 1;
      spinEOB->Position = 0; // temporary, until they switch named -> (bulk) EOB instead
    }

    // jhaley 20120712: "simple" PR print option
    if(optionsPreferences["accounting"]["simple_pr_print"] == "true")
    {
      spinOriginal->Position = 1;
      spinReturnToOriginator->Position = 0;
      spinProcurement->Position = 0;
      spinCopyForOriginator->Position = 0;
      spinJustificationProcedures->Position = 1;
      spinJustificationNoProcedures->Position = 0;
      spinEOB->Position = 0;
    }

    // PHILIP -- and then an extra link, in case you forget what you're doing and want to go see ...
    CreateLink(txtPrNumber, NULL, this, id, "purchase_reqs");
  }
}
//---------------------------------------------------------------------------
void __fastcall TformLaunchPRReport::btnCloseClick(TObject *Sender)
{
  Close();
}
//---------------------------------------------------------------------------
void __fastcall TformLaunchPRReport::btnRunClick(TObject *Sender)
{
  if(!IsValidOnScreen())
    return;

  // get objects ready (stuff for composite reports, null datasets and transactions)
  AutoPtr<TQRPageList, void> pSource      = AutoPtr<TQRPageList, void>();
  AutoPtr<TQRPageList, void> pDestination = AutoPtr<TQRPageList, void>();

  AutoPtr<TQRPrinter, TComponent *> pPrinterThing    = AutoPtr<TQRPrinter, TComponent *>(NULL);
  AutoPtr<TIBDataSet, TComponent *> pAccountsDataset = AutoPtr<TIBDataSet, TComponent *>(NULL);
  AutoPtr<TIBDataSet, TComponent *> pPrDataset       = AutoPtr<TIBDataSet, TComponent *>(NULL);
  AutoPtr<TIBDataSet, TComponent *> pJustifDataset   = AutoPtr<TIBDataSet, TComponent *>(NULL);
  AutoPtr<TIBDataSet, TComponent *> pNoJustifDataset = AutoPtr<TIBDataSet, TComponent *>(NULL);
  AutoPtr<TIBDataSet, TComponent *> pEobDataset      = AutoPtr<TIBDataSet, TComponent *>(NULL);
  AutoPtr<TIBTransaction, TComponent *> pTransaction = AutoPtr<TIBTransaction, TComponent *>(NULL);

  TQRPageList *source        = pSource.get();
  TQRPageList *destination   = pDestination.get();
  TQRPrinter  *printer_thing = pPrinterThing.get();

  accountsDataset        = pAccountsDataset.get();
  prDataset              = pPrDataset.get();
  justificationDataset   = pJustifDataset.get();
  nojustificationDataset = pNoJustifDataset.get();
  eobDataset             = pEobDataset.get();
  dbTransaction          = pTransaction.get();

  // Need all valid objects.
  if(!source || !destination || !accountsDataset || !prDataset || !justificationDataset || !nojustificationDataset ||
     !eobDataset || !dbTransaction || !printer_thing)
  {
    PSay("Construction of one or more required components failed. Please try again. If this message continues, "
         "contact a programmer.", m_program_error, MB_OK);
    return;
  }

  FORM_TRY
    pr_report = new TformReportPR(NULL);
  FORM_CATCH_RETURN("TformReportPR", )

  try
  {
    destination->Stream = new TQRStream(100000);
  }
  catch(...)
  {
    PSay("Construction of a printer stream failed. Please try again.", m_program_error, MB_OK);
    return;
  }

  Processing(true);

  // PHILIP -- shouldn't this just be a map of name to int? or something? or does order matter?
  // PHILIP -- order matters, in that they don't want it in order by name ... O, R, P, C ...
  vector<page_class> pr_copies_to_print;
  page_class temp_class;

  // PHILIP -- for each number of copies, add the name of the copy to use ... and how many.
  if(StringToInt(editOriginal->Text.c_str()) > 0)
  {
    temp_class.name = "Original";
    temp_class.copies = StringToInt(editOriginal->Text.c_str());
    pr_copies_to_print.push_back(temp_class);
  }
  if(StringToInt(editReturnToOriginator->Text.c_str()) > 0)
  {
    temp_class.name = "Return to Originator";
    temp_class.copies = StringToInt(editReturnToOriginator->Text.c_str());
    pr_copies_to_print.push_back(temp_class);
  }
  if(StringToInt(editProcurement->Text.c_str()) > 0)
  {
    temp_class.name = "Procurement Copy";
    temp_class.copies = StringToInt(editProcurement->Text.c_str());
    pr_copies_to_print.push_back(temp_class);
  }
  if(StringToInt(editCopyForOriginator->Text.c_str()) > 0)
  {
    temp_class.name = "Copy for Originator";
    temp_class.copies = StringToInt(editCopyForOriginator->Text.c_str());
    pr_copies_to_print.push_back(temp_class);
  }

  // PHILIP -- set up transactions (could use StdTransaction instead?) and datasets
  try
  {
    dbTransaction->DefaultDatabase = dbDatabase;
    accountsDataset->Database = dbDatabase;
    accountsDataset->Transaction = dbTransaction;
    accountsDataset->UniDirectional = false;
    prDataset->Database = dbDatabase;
    prDataset->Transaction = dbTransaction;
    prDataset->UniDirectional = false;
    justificationDataset->Database = dbDatabase;
    justificationDataset->Transaction = dbTransaction;
    justificationDataset->UniDirectional = false;
    nojustificationDataset->Database = dbDatabase;
    nojustificationDataset->Transaction = dbTransaction;
    nojustificationDataset->UniDirectional = false;
    eobDataset->Database = dbDatabase;
    eobDataset->Transaction = dbTransaction;
    eobDataset->UniDirectional = false;

    dbTransaction->StartTransaction();
  }
  catch(...)
  {
    PSay("Setup of transaction for report failed, please try again.", m_database_error, MB_OK);
    return;
  }

  string _provider_name = GetOneField(dbTransaction,
     "select tax_entities.name "
     "from purchase_reqs "
     "left join tax_entities on purchase_reqs.tax_entity_id = tax_entities.id "
     "where purchase_reqs.id = " + id);

  string _purchase_req_num = GetOneField(dbTransaction,
    "select purchase_reqs.purchase_req_number from purchase_reqs where purchase_reqs.id = " + id);

  string bill_to_sql =
     "select tax_entities.name, tax_addresses.line1, tax_addresses.line2, tax_addresses.city, state_types.abbreviation state, tax_addresses.zip "
     "from purchase_reqs "
     "inner join tax_entities on purchase_reqs.bill_to_entity_id = tax_entities.id "
     "inner join tax_addresses on tax_entities.billing_address_id = tax_addresses.id "
     "inner join state_types on tax_addresses.state_id = state_types.id "
     "where purchase_reqs.id = " + id;
  SqlToMap(dbTransaction, bill_to_sql, bill_to);

  // prepare temp table for conversion of transactions for this PR
  string sql_delete_related_transactions =
    "delete from temp_pr_transactions";

  string sql_insert_related_transactions =
    "insert into temp_pr_transactions "
    "select bills.purchase_req_id, bills.id, bills.actual_tax_id, transactions.tax_entity_id, transactions.account_id, "
    " outside_accounts.account_type_id, outside_accounts.pending_account_id, "
    " (select first 1 payable_accounts.id "
    "  from accounts payable_accounts "
    "  where payable_accounts.account_type_id = " + lookupTables->IdOf("account_types", "Payable") +
    "  and payable_accounts.cash_account_id = outside_accounts.id), "
    " (select first 1 receivable_accounts.id "
    "  from accounts receivable_accounts "
    "  where receivable_accounts.account_type_id = " + lookupTables->IdOf("account_types", "Receivable") +
    "  and receivable_accounts.cash_account_id = outside_accounts.id), "
    " debit_amount, credit_amount "
    "from purchase_reqs "
    "inner join bills on bills.purchase_req_id = purchase_reqs.id "
    "inner join transactions on transactions.bill_id = bills.id "
    "inner join accounts outside_accounts on transactions.account_id = outside_accounts.id "
    "where purchase_reqs.id = " + id;

  string sql_update_revenue_to_receivable =
    "update temp_pr_transactions set account_id = pending_id, account_type_id = " + lookupTables->IdOf("account_types", "Receivable") +
    " where account_type_id =" + lookupTables->IdOf("account_types", "Revenue") + " and entity_id != paid_entity_id";
  string sql_update_cash_to_payable =
    "update temp_pr_transactions set account_id = payable_id, account_type_id = " + lookupTables->IdOf("account_types", "Payable") +
    " where account_type_id =" + lookupTables->IdOf("account_types", "Cash") + " and entity_id = paid_entity_id";
  string sql_update_cash_to_receivable =
    "update temp_pr_transactions set account_id = receivable_id, account_type_id = " + lookupTables->IdOf("account_types", "Receivable") +
    " where account_type_id = " + lookupTables->IdOf("account_types", "Cash") + " and entity_id != paid_entity_id";

  if(!ExecuteStatement(dbTransaction, sql_delete_related_transactions))
  {
    PSay("Failed to clear temporary table for transaction transformation.", m_database_error, MB_OK);
    return;
  }

  if(!ExecuteStatement(dbTransaction, sql_insert_related_transactions))
  {
    PSay("Failed to insert transactions into temporary table for transformation.", m_database_error, MB_OK);
    return;
  }

  if(!ExecuteStatement(dbTransaction, sql_update_revenue_to_receivable) ||
     !ExecuteStatement(dbTransaction, sql_update_cash_to_payable) ||
     !ExecuteStatement(dbTransaction, sql_update_cash_to_receivable))
  {
    PSay("Failed to transform transactions for use on a PR report.", m_database_error, MB_OK);
    return;
  }

  // PHILIP -- for each thing to print (name of sheet changing and whatnot) add all pages of the 'original' as many times as is needed to the composite report
  for(unsigned int i = 0; i < pr_copies_to_print.size(); i++)
  {
    string file_name = PMakeTempFilePath(string("page") + IntToString(i) + ".qrp");
    current_pr_type = pr_copies_to_print[i].name;
    compositeReport->Prepare();
    ((TQuickRep *)(compositeReport->Reports->First()))->QRPrinter->Save(file_name.c_str());
    source->Clear();
    source->LoadFromFile(file_name.c_str());
    for(int k = 0; k < pr_copies_to_print[i].copies; k++)
    {
      for(int j = 1; j <= source->PageCount; j++)
        destination->AddPage(source->GetPage(j));
    }
  }

  // add other reports to destination here

  // PHILIP -- this page lists modifications to the PR (this information does NOT overlap the information on the front page)
  // (See Dead Code #1 below -jhaley)

  // begin justification reports
  // (See Dead Code #2 below for how it used to be done... -jhaley)

  int justification_procedure_copies = StringToInt(editJustificationProcedures->Text.c_str());
  if(justification_procedure_copies > 0)
  {
    string sql_get_justification =
      "select bills.id, people.id person_fk, "
      " person_names.last_name || ', ' || person_names.first_name || ' ' || person_names.suffix name, "
      " tasks.patient_control_number, procedures.start_date, procedures.place_of_service, procedures.cpt_hcpcs code, "
      " procedures.charges charged, procedure_payments.amount_charged billed, procedure_payments.amount_allowed allowed, "
      " procedure_payments.amount_paid paid "
      "from purchase_reqs "
      "left join bills on purchase_reqs.id = bills.purchase_req_id and bills.current_id + 1 > 2 + 1 "
      "left join tasks on tasks.id = bills.task_id "
      "left join procedures on procedures.task_id = tasks.id "
      "left join procedure_payments on procedure_payments.procedure_id = procedures.id "
      "left join people on tasks.person_id = people.id "
      "left join person_names on people.person_names_id = person_names.id "
      "where purchase_reqs.id = " + id + " "
      "order by person_names.last_name, person_names.first_name, tasks.date_of_task, procedures.start_date";

    try
    {
      justificationDataset->SelectSQL->Add(sql_get_justification.c_str());
      justificationDataset->Open();
      justificationDataset->First();

      TformReportBulkEOB *report = NULL;
      FORM_TRY
        report = new TformReportBulkEOB(NULL, true);
      FORM_CATCH

      report->reportJustification->DataSet = justificationDataset;
      report->textProvider->Caption = _provider_name.c_str();
      report->numPR->Caption        = _purchase_req_num.c_str();
      report->labelUser->Caption    = PrettyString(lookupTables->ValueOf("users", current_userid)).c_str();
      report->textDOS->DataSet = justificationDataset;
      report->textDOS->DataField = "start_date";
      report->textPOS->DataSet = justificationDataset;
      report->textPOS->DataField = "place_of_service";
      report->textProc->DataSet = justificationDataset;
      report->textProc->DataField = "code";
      report->bandPatientHeader->Expression = "id"; // per-bill rather than per-person (person_fk)

      report->reportJustification->Prepare();
      string pageb_name = PMakeTempFilePath("pageb.qrp");
      report->reportJustification->QRPrinter->Save(pageb_name.c_str());

      source->Clear();
      source->LoadFromFile(pageb_name.c_str());
      for(int k = 0; k < justification_procedure_copies; k++)
      {
        for(int j = 1; j <= source->PageCount; j++)
          destination->AddPage(source->GetPage(j));
      }
      report->CloseNow();

      justificationDataset->Close();
    }
    catch(...)
    {
       PSay("Warning: generation of justification report failed", m_program_error, MB_OK);
    }
  }

  int justification_no_procedure_copies = StringToInt(editJustificationNoProcedures->Text.c_str());
  if(justification_no_procedure_copies > 0)
  {                                                               
    string sql_get_justification =
      "select bills.id, people.id person_fk, '' name, tasks.patient_control_number, procedures.start_date, "
      " procedures.place_of_service, '' code, procedures.charges charged, procedure_payments.amount_charged billed, "
      " procedure_payments.amount_allowed allowed, procedure_payments.amount_paid paid "
      "from purchase_reqs "
      "left join bills on purchase_reqs.id = bills.purchase_req_id and bills.current_id + 1 > 2 + 1 "
      "left join tasks on tasks.id = bills.task_id "
      "left join procedures on procedures.task_id = tasks.id "
      "left join procedure_payments on procedure_payments.procedure_id = procedures.id "
      "left join people on tasks.person_id = people.id "
      "left join person_names on people.person_names_id = person_names.id "
      "where purchase_reqs.id = " + id + " "
      "order by person_names.last_name, person_names.first_name, tasks.date_of_task, procedures.start_date";

    try
    {
      nojustificationDataset->SelectSQL->Add(sql_get_justification.c_str());
      nojustificationDataset->Open();
      nojustificationDataset->First();

      TformReportBulkEOB *report = NULL;
      FORM_TRY
        report = new TformReportBulkEOB(NULL, true);
      FORM_CATCH

      report->reportJustification->DataSet = nojustificationDataset;
      report->textProvider->Caption = _provider_name.c_str();
      report->numPR->Caption        = _purchase_req_num.c_str();
      report->labelUser->Caption    = PrettyString(lookupTables->ValueOf("users", current_userid)).c_str();
      report->textDOS->DataSet = nojustificationDataset;
      report->textDOS->DataField = "start_date";
      report->textPOS->DataSet = nojustificationDataset;
      report->textPOS->DataField = "place_of_service";
      report->textProc->DataSet = nojustificationDataset;
      report->textProc->DataField = "code";
      report->_labelProc->Caption = "";
      report->_labelPersonName->Caption = ""; // they don't want names to go to finance either...
      report->bandPatientHeader->Expression = "id"; // still per-bill rather than per-person

      report->reportJustification->Prepare();
      string pagec_name = PMakeTempFilePath("pagec.qrp");
      report->reportJustification->QRPrinter->Save(pagec_name.c_str());

      source->Clear();
      source->LoadFromFile(pagec_name.c_str());
      for(int k = 0; k < justification_no_procedure_copies; k++)
      {
        for(int j = 1; j <= source->PageCount; j++)
          destination->AddPage(source->GetPage(j));
      }
      report->CloseNow();
      nojustificationDataset->Close();
    }
    catch(...)
    {
      PSay("Warning: generation of no-procedures justification report failed", m_program_error, MB_OK);
    }
  }

  int eob_copies = StringToInt(editEOB->Text.c_str());
  if(eob_copies > 0)
  {
    string sql_get_eob =
      "select bills.id, "
      " affiliations.cdib, person_names.last_name || ', ' || person_names.first_name || ' ' || person_names.suffix name, "
      " tasks.patient_control_number, procedures.start_date, procedures.cpt_hcpcs code, procedures.charges charged, "
      " procedure_payments.amount_charged billed, procedure_payments.amount_allowed allowed, procedure_payments.amount_paid paid "
      "from purchase_reqs "
      "left join bills on purchase_reqs.id = bills.purchase_req_id and bills.current_id + 1 > 2 + 1 "
      "left join tasks on tasks.id = bills.task_id "
      "left join procedures on procedures.task_id = tasks.id "
      "left join procedure_payments on procedure_payments.procedure_id = procedures.id "
      "left join people on tasks.person_id = people.id "
      "left join person_names on people.person_names_id = person_names.id "
      "left join affiliations on people.affiliations_id = affiliations.id "
      "where purchase_reqs.id = " + id + " "
      "order by person_names.last_name, person_names.first_name, tasks.date_of_task, procedures.start_date";

    try
    {
      eobDataset->SelectSQL->Add(sql_get_eob.c_str());
      eobDataset->Open();
      eobDataset->First();

      TformReportBulkEOB *report = NULL;
      FORM_TRY
         report = new TformReportBulkEOB(NULL);
      FORM_CATCH
      report->qpBulkEOB->DataSet = eobDataset;
      report->qrpProviderName->Caption = _provider_name.c_str();
      report->qrpPR->Caption = _purchase_req_num.c_str();
      report->qrpPR->Caption = GetOneField(dbTransaction,
        "select purchase_reqs.purchase_req_number from purchase_reqs where purchase_reqs.id = " + id).c_str();
      report->lblUser->Caption = PrettyString(lookupTables->ValueOf("users", current_userid)).c_str();
      report->qrpCDIB->DataSet = eobDataset;
      report->qrpCDIB->DataField = "cdib";
      report->qrpName->DataSet = eobDataset;
      report->qrpName->DataField = "name";
      report->qrpPatientAccountNum->DataSet = eobDataset;
      report->qrpPatientAccountNum->DataField = "patient_control_number";
      report->qrpDate->DataSet = eobDataset;
      report->qrpDate->DataField = "start_date";
      report->qrpProcedure->DataSet = eobDataset;
      report->qrpProcedure->DataField = "code";
      report->bandGroup->Expression = "id";

      report->qpBulkEOB->Prepare();
      string paged_name = PMakeTempFilePath("paged.qrp");
      report->qpBulkEOB->QRPrinter->Save(paged_name.c_str());

      source->Clear();
      source->LoadFromFile(paged_name.c_str());
      for(int k = 0; k < eob_copies; k++)
      {
        for(int j = 1; j <= source->PageCount; j++)
          destination->AddPage(source->GetPage(j));
      }
      report->CloseNow();

      eobDataset->Close();
    }
    catch(...)
    {
      PSay("Warning: generation of EOB report failed", m_program_error, MB_OK);
    }
  }

  // we've had reports of users having issues with running a PR report and getting weird results ... this might be the
  // cause. just in case, we delete here -also-. this -should- be standard behavior, btw.

  if(!ExecuteStatement(dbTransaction, "delete from temp_pr_bills"))
  {
    PSay("Failed cleaning up temporary tables after the report.\nYou might want to rerun the report and let it "
         "clean things up again.", m_database_error, MB_OK, "temp_pr_bills");
  }

  if(!ExecuteStatement(dbTransaction, "delete from temp_pr_transactions"))
  {
    PSay("Failed cleaning up temporary tables after the report.\nYou might want to rerun the report and let it "
         "clean things up again.", m_database_error, MB_OK, "temp_pr_transactions");
  }

  Processing(false);

  try
  {
    dbTransaction->Commit();
    destination->Finish();
  }
  catch(...)
  {
    PSay("Warning: cleanup process failed", m_program_error, MB_OK);
  }

  // PHILIP -- save the composite thingie, then load it ... and preview. woohoo!
  string temp_filepath = PMakeTempFilePath("complete.qrp");
  destination->SaveToFile(temp_filepath.c_str());
  printer_thing->Load(temp_filepath.c_str());
  printer_thing->PreviewModal();

  pr_report->Close();
}
//---------------------------------------------------------------------------
void __fastcall TformLaunchPRReport::compositeReportAddReports(TObject *Sender)
{
  // PHILIP -- setting up captions on the top of reportAccounts (address information, type, date, number)
  string sql;
  if(pr_type == vendor)
    pr_report->labelType->Caption = "Vendor:";
  else
    pr_report->labelType->Caption = "Provider paid:";

  if(bill_to["name"] != "")
  {
    pr_report->memoBillTo->Lines->Clear();
    pr_report->memoBillTo->Lines->Add(bill_to["name"].c_str());
    pr_report->memoBillTo->Lines->Add(FormatAddress(bill_to["line1"]).c_str());
    if(bill_to["line2"] != "")
      pr_report->memoBillTo->Lines->Add(FormatAddress(bill_to["line2"]).c_str());
    pr_report->memoBillTo->Lines->Add(
      (PrettyString(bill_to["city"]) + ", " +
       UppercaseString(bill_to["state"]) + " " +
       FormatZipCode(bill_to["zip"])).c_str());
  }

  pr_report->labelExpenditureReason->Caption =
    SentenceString(GetOneField(dbTransaction, "select exp_reason from purchase_reqs where id = " + id)).c_str();

  sql = "select tin.identifier taxid, tax_entities.name, tax_addresses.line1, tax_addresses.line2, "
        " tax_addresses.city || ', ' || state_types.abbreviation || '  ' || tax_addresses.zip city_line, "
        " purchase_reqs.purchase_req_number, purchase_reqs.date_sent "
        "from purchase_reqs "
        "inner join tax_entities on purchase_reqs.tax_entity_id = tax_entities.id "
        "inner join tax_addresses on tax_entities.billing_address_id = tax_addresses.id "
        "left join state_types on tax_addresses.state_id = state_types.id "
        "left join entity_ids tin on tin.tax_entity_id = tax_entities.id and "
        "  tin.current_id >= 3 and tin.entity_id_id = '" + lookupTables->IdOf("entity_id_types", "TIN/EIN") + "' "
        "where purchase_reqs.id = " + id;

  map<string, string> address_map;
  if(!SqlToMap(dbTransaction, sql, address_map))
  {
    PSay("Failed to get PR data", m_database_error, MB_OK);
    return;
  }

  pr_report->labelPRNumber->Caption   = PrettyString(address_map["purchase_req_number"]).c_str();
  pr_report->labelPRDate->Caption     = PrettyString(address_map["date_sent"]).c_str();
  pr_report->labelToTaxID->Caption    = PrettyString(address_map["taxid"]).c_str();
  pr_report->labelToName->Caption     = address_map["name"].c_str();
  pr_report->labelToAddress1->Caption = FormatAddress(address_map["line1"]).c_str();

  if(address_map["line2"] == "")
  {
    pr_report->labelToAddress2->Caption = PrettyString(address_map["city_line"]).c_str();
    pr_report->labelToAddress3->Caption = "";
  }
  else
  {
    pr_report->labelToAddress2->Caption = FormatAddress(address_map["line2"]).c_str();
    pr_report->labelToAddress3->Caption = PrettyString(address_map["city_line"]).c_str();
  }

  // See Dead Code #3 below - jhaley
  // PHILIP -- getting list of accounts used and their amounts, regardless of entity (only for certain dates)
  // PHILIP -- this needs to be changed to handle receivables, etc. correctly, and maybe for dates?

  string sql_get_accounts_used =
    "select temp_pr_transactions.account_id, "
    " acc_level_one.short_item || '-' || acc_level_three.short_item || '-' || acc_level_four.short_item || '-' || "
    "  acc_level_two.short_item || '  (' || "
    "  acc_level_one.item || '--' || acc_level_three.item || '--' || acc_level_four.item || '--' || "
    "  acc_level_two.item || ')' account_name, "
    " sum(temp_pr_transactions.debit - temp_pr_transactions.credit) account_amount "
    "from temp_pr_transactions "
    "inner join accounts on temp_pr_transactions.account_id = accounts.id "
    "inner join acc_level_one on accounts.acc_level_one_id = acc_level_one.id "
    "inner join acc_level_two on accounts.acc_level_two_id = acc_level_two.id "
    "inner join acc_level_three on accounts.acc_level_three_id = acc_level_three.id "
    "inner join acc_level_four on accounts.acc_level_four_id = acc_level_four.id "
    "where temp_pr_transactions.pr_id = " + id +
    " and temp_pr_transactions.account_type_id != " + /* NOT payable */ lookupTables->IdOf("account_types", "Payable") +
    "group by temp_pr_transactions.account_id, acc_level_one.short_item, acc_level_three.short_item, "
    " acc_level_four.short_item, acc_level_two.short_item,"
    " acc_level_one.item, acc_level_three.item, acc_level_four.item, acc_level_two.item";

  if(accountsDataset->Active)
    accountsDataset->Close();
  accountsDataset->SelectSQL->Clear();
  accountsDataset->SelectSQL->Add(sql_get_accounts_used.c_str());
  accountsDataset->Open();
  accountsDataset->First();
  pr_report->reportAccounts->DataSet = accountsDataset;
  pr_report->textAccountName->DataSet = accountsDataset;
  pr_report->textAccountName->DataField = "account_name";
  pr_report->exprAccountAmount->Expression = "FORMATNUMERIC('$ ###,###,###0.00', account_amount)";
  pr_report->exprAccountTotal->Expression = "FORMATNUMERIC('$ ###,###,###0.00', SUM(account_amount))";
  compositeReport->Reports->Add(pr_report->reportAccounts);
  // PHILIP -- first page is added here (not multiple times?)

  // PHILIP -- now set up reportBills
  if(pr_type == vendor)
  {
    // See Dead Code #4 below -jhaley
    // PHILIP -- for vendors only, select the names, etc.
    // PHILIP -- needs to be modified for list of fields, as well as pulling transactions (dates?)
    // PHILIP -- needs to set up the group band / expression so it'll group bills transactions together

    string sql_delete_temp_bills =
      "delete from temp_pr_bills ";

    string sql_insert_temp_bills =
      "insert into temp_pr_bills "
      "select bills.purchase_req_id, bills.id, bills.invoice_number, bills.note, "
      " (select sum(temp_pr_transactions.credit - temp_pr_transactions.debit) "
      "  from temp_pr_transactions where temp_pr_transactions.bill_id = bills.id and "
      /* payable ... or anything but payable if you flip the dr/cr, should be the same */
      "  temp_pr_transactions.account_type_id = " + lookupTables->IdOf("account_types", "Payable") +
      " ) "
      "from bills "
      "where bills.purchase_req_id = " + id;

    if(!ExecuteStatement(dbTransaction, sql_delete_temp_bills))
    {
      PSay("Failed to clear temporary table for vendor bill information.", m_database_error, MB_OK);
      return;
    }

    if(!ExecuteStatement(dbTransaction, sql_insert_temp_bills))
    {
      PSay("Failed to insert temporary vendor bill information.", m_database_error, MB_OK);
      return;
    }

    // jhaley 20101122: Modified for proper Firebird 2.x alias semantics
    string sql_get_vendor_bills_and_receivables =
      "select temp_pr_bills.id, temp_pr_bills.invoice_number, temp_pr_bills.note, temp_pr_bills.bill_payment, "
      " receivables.account_id, "
      " acc_level_one.short_item || '-' || acc_level_three.short_item || '-' || acc_level_four.short_item || '-' || "
      "  acc_level_two.short_item account_name, "
      " tax_entities.name entity_name, sum(receivables.debit - receivables.credit) account_amount "
      "from temp_pr_bills "
      "left join temp_pr_transactions receivables on receivables.bill_id = temp_pr_bills.id "
      " and receivables.account_type_id = " + /* receivable */ lookupTables->IdOf("account_types", "Receivable") +
      "left join accounts on receivables.account_id = accounts.id "
      "left join tax_entities on receivables.entity_id = tax_entities.id "
      "left join acc_level_one on accounts.acc_level_one_id = acc_level_one.id "
      "left join acc_level_two on accounts.acc_level_two_id = acc_level_two.id "
      "left join acc_level_three on accounts.acc_level_three_id = acc_level_three.id "
      "left join acc_level_four on accounts.acc_level_four_id = acc_level_four.id "
      "where temp_pr_bills.pr_id = " + id +
      "group by temp_pr_bills.id, temp_pr_bills.invoice_number, temp_pr_bills.note, temp_pr_bills.bill_payment, "
      " receivables.account_id, acc_level_one.short_item, acc_level_three.short_item, acc_level_four.short_item, "
      " acc_level_two.short_item, tax_entities.name ";

    if(prDataset->Active)
      prDataset->Close();

    prDataset->SelectSQL->Clear();
    prDataset->SelectSQL->Add(sql_get_vendor_bills_and_receivables.c_str());
    prDataset->Open();
    prDataset->First();
    pr_report->reportBills->DataSet = prDataset;

    // PHILIP -- testing here for modifications (extra page, label at bottom of this one)
    if(!transactions_modified)
      pr_report->labelSeeModifications->Caption = "";

    // PHILIP -- change the label at the bottom to indicate the type of PR
    pr_report->labelSheetType->Caption = current_pr_type.c_str();

    // PHILIP -- stupid, stupid thingie of group_or_user ... MUST GO AWAY
    // See Dead Code #5 below -jhaley
    // PHILIP -- now back to reasonable crap ...

    pr_report->_labelFacilityName->SendToBack();
    pr_report->_labelFacilityTaxID->SendToBack();
    pr_report->_labelProviderName->SendToBack();
    pr_report->_labelProviderTaxID->SendToBack();
    pr_report->_labelProviderBillCount->SendToBack();

    pr_report->bandGroupBills->Expression = "id";

    pr_report->textInvoiceNumber->DataSet = prDataset;
    pr_report->textInvoiceNumber->DataField = "invoice_number";
    pr_report->textDescription->DataSet = prDataset;
    pr_report->textDescription->DataField = "note";
    pr_report->exprBillAmount->Expression = "FORMATNUMERIC('$ ###,###,##0.00', bill_payment)";

    pr_report->textBillAccountName->DataSet = prDataset;
    pr_report->textBillAccountName->DataField = "account_name";
    pr_report->textBillEntityName->DataSet = prDataset;
    pr_report->textBillEntityName->DataField = "entity_name";
    pr_report->exprReceivableAmount->Expression = "FORMATNUMERIC('$ ###,###,##0.00', account_amount)";

    // don't bother ... with bands not printing, values get ignored. it won't sum correctly.
    // get the sum into that field ... 'cause it's a sucking bastard on a quick report.
    // wait, no, the bastard IS quick reports.
    pr_report->exprBillTotal->Expression =
      ("FORMATNUMERIC('$ ###,###,##0.00', " +
       GetOneField(dbTransaction, "select sum(bill_payment) from temp_pr_bills where temp_pr_bills.pr_id = " + id) +
       ")"
      ).c_str();
  }
  else
  {
    // PHILIP -- now for provider bills (woohoo!)
    // See Dead Code #6 below -jhaley

    string sql_get_provider_bills =
      "select facilities.id, facilities.name facility_name, facility_tin.identifier facility_taxid, providers.id, "
      " providers.name provider_name, provider_tin.identifier provider_taxid, "
      " (cast(facilities.id as varchar(32)) || cast(providers.id as varchar(32))) group_expression, "
      " count(distinct bills.id) bill_count, "
      " sum(credit-debit) bill_payment "
      "from temp_pr_transactions "
      "inner join accounts on temp_pr_transactions.account_id = accounts.id "
      "inner join bills on temp_pr_transactions.bill_id = bills.id "
      "inner join tax_entities facilities on bills.facility_tax_id = facilities.id "
      "left join entity_ids facility_tin on facility_tin.tax_entity_id = facilities.id and "
      " facility_tin.current_id >= 3 and facility_tin.entity_id_id = '" + lookupTables->IdOf("entity_id_types", "TIN/EIN") + "' "
      "inner join tax_entities providers on bills.provider_tax_id = providers.id "
      "left join entity_ids provider_tin on provider_tin.tax_entity_id = providers.id and provider_tin.current_id >= 3 "
      " and provider_tin.entity_id_id = '" + lookupTables->IdOf("entity_id_types", "TIN/EIN") + "' "
      "where temp_pr_transactions.pr_id = " + id + " and " /* payable, but not just today's payable */
      " temp_pr_transactions.account_type_id = " + lookupTables->IdOf("account_types", "Payable") +
      "group by facilities.name, providers.name, facility_tin.identifier, provider_tin.identifier, facilities.id, providers.id ";

    if(prDataset->Active)
      prDataset->Close();
    prDataset->SelectSQL->Clear();
    prDataset->SelectSQL->Add(sql_get_provider_bills.c_str());
    prDataset->Open();
    prDataset->First();
    pr_report->reportBills->DataSet = prDataset;

    // PHILIP -- modifications or no?
    if(!transactions_modified)
      pr_report->labelSeeModifications->Caption = "";

    // PHILIP -- type of tax entity
    pr_report->labelSheetType->Caption = current_pr_type.c_str();

    pr_report->_labelInvoiceNumber->SendToBack();
    pr_report->_labelDescription->SendToBack();

    pr_report->bandGroupBills->Expression = "group_expression";

    pr_report->textFacilityName->DataSet = prDataset;
    pr_report->textFacilityName->DataField = "facility_name";
    pr_report->textFacilityTaxId->DataSet = prDataset;
    pr_report->textFacilityTaxId->DataField = "facility_taxid";
    pr_report->textProviderName->DataSet = prDataset;
    pr_report->textProviderName->DataField = "provider_name";
    pr_report->textProviderTaxId->DataSet = prDataset;
    pr_report->textProviderTaxId->DataField = "provider_taxid";
    pr_report->textProviderBillCount->DataSet = prDataset;
    pr_report->textProviderBillCount->DataField = "bill_count";;
    pr_report->exprBillAmount->Expression = "FORMATNUMERIC('$ ###,###,##0.00', bill_payment)";
    pr_report->exprReceivableAmount->Expression = "FORMATNUMERIC('$ ###,###,##0.00', 0.00)";

    // and because we're not printing the detail ... the sum won't work. so we select it. damn QR.
    pr_report->exprBillTotal->Expression =
      ("FORMATNUMERIC('$ ###,###,##0.00', " +
       GetOneField(dbTransaction,
         "select sum(credit-debit) "
         "from temp_pr_transactions "
         "where temp_pr_transactions.pr_id = " + id + " and "
         " temp_pr_transactions.account_type_id = " + lookupTables->IdOf("account_types", "Payable")) +
       ")"
      ).c_str();
  }

  // PHILIP -- add this page
  compositeReport->Reports->Add(pr_report->reportBills);
}
//---------------------------------------------------------------------------
bool TformLaunchPRReport::IsValidOnScreen()
{
  // you know ... these aren't needed anymore, what with the spin-controls ... they can't enter non-ints. at all.

  if(!IsInt(editOriginal->Text.c_str()))
  {
    PSay("Please enter a valid number of copies ('original')", m_invalid_data, MB_OK);
    editOriginal->SetFocus();
    return false;
  }
  if(!IsInt(editReturnToOriginator->Text.c_str()))
  {
    PSay("Please enter a valid number of copies ('return to originator')", m_invalid_data, MB_OK);
    editReturnToOriginator->SetFocus();
    return false;
  }
  if(!IsInt(editProcurement->Text.c_str()))
  {
    PSay("Please enter a valid number of copies ('procurement')", m_invalid_data, MB_OK);
    editProcurement->SetFocus();
    return false;
  }
  if(!IsInt(editCopyForOriginator->Text.c_str()))
  {
    PSay("Please enter a valid number of copies ('copy for originator')", m_invalid_data, MB_OK);
    editCopyForOriginator->SetFocus();
    return false;
  }
  if(!IsInt(editJustificationProcedures->Text.c_str()))
  {
    PSay("Please enter a valid number of copies ('justification with procedures')", m_invalid_data, MB_OK);
    editJustificationProcedures->SetFocus();
    return false;
  }
  if(!IsInt(editJustificationNoProcedures->Text.c_str()))
  {
    PSay("Please enter a valid number of copies ('justification without procedures')", m_invalid_data, MB_OK);
    editJustificationNoProcedures->SetFocus();
    return false;
  }
  if(!IsInt(editEOB->Text.c_str()))
  {
    PSay("Please enter a valid number of copies ('explanation of benefits')", m_invalid_data, MB_OK);
    editEOB->SetFocus();
    return false;
  }
  if(StringToInt(editOriginal->Text.c_str()) +
     StringToInt(editReturnToOriginator->Text.c_str()) +
     StringToInt(editProcurement->Text.c_str()) +
     StringToInt(editCopyForOriginator->Text.c_str()) +
     StringToInt(editJustificationProcedures->Text.c_str()) +
     StringToInt(editJustificationNoProcedures->Text.c_str()) +
     StringToInt(editEOB->Text.c_str()) == 0
    )
  {
    PSay("Please request at least one page to be printed", m_invalid_data, MB_OK);
    return false;
  }

  return true;

  // See Dead Code #7 below -jhaley
}

//---------------------------------------------------------------------------------------------------------------------
// DEAD CODE
// jhaley 20120710 - moved down here to clean stuff the hell up, without necessarily losing some historically important
// code that explains the way things have changed.
//

// ---- Dead Code #1 - From btnRunClick ------------------------------------------------------
// PHILIP -- dangerous way of selecting transactions. based on the note? what the hell?
// PHILIP -- must be changed, use temp table of modified transactions (for the purpose of faking cash as pending, etc.)
/*
if (transactions_modified)
{
  string sql;
  sql = "select transactions.last_update_stamp, transactions.effective_date, "
        " accounts.name, tasks.parent_task_id cob_number, "
        " transactions.debit_amount, transactions.credit_amount, "
        " transactions.transaction_number, transactions.note "
        "from purchase_reqs "
        "inner join bills on bills.purchase_req_id = purchase_reqs.id "
        "inner join tasks on tasks.id = bills.task_id "
        "inner join transactions on transactions.bill_id = bills.id "
        "inner join accounts on transactions.account_id = accounts.id "
        "where purchase_reqs.id = " + id + " and cast(transactions.last_update_stamp as date) > purchase_reqs.date_sent and "
        " transactions.note not like 'PAYMENT OF CHECK%' "
        "order by transactions.last_update_stamp, transactions.transaction_number, tasks.parent_task_id, transactions.note";
  modificationDataset->SelectSQL->Add(sql.c_str());
  modificationDataset->Open();
  modificationDataset->First();
  FORM_TRY
    TformReportPRModification * report = new TformReportPRModification(NULL);
    // PHILIP -- get the name of the tax entity for this PR (don't we have that already?)
    report->qpPRModification->DataSet = modificationDataset;
    sql = "select tax_entities.name from purchase_reqs "
          "inner join tax_entities on purchase_reqs.tax_entity_id = tax_entities.id where purchase_reqs.id = " + id;
    report->qrpProviderName->Caption = PrettyString(GetOneField(dbTransaction, sql)).c_str();
    // PHILIP -- then go and get the PR number ...  (don't we have that already?)
    report->qrpPR->Caption = GetOneField(dbTransaction,
      "select purchase_reqs.purchase_req_number from purchase_reqs where purchase_reqs.id = " + id).c_str();
    report->lblUser->Caption = current_fullname.c_str();
    report->qrpLastUpdate->DataSet = modificationDataset;
    report->qrpLastUpdate->DataField = "last_update_stamp";
    report->qrpEffectiveDate->DataSet = modificationDataset;
    report->qrpEffectiveDate->DataField = "effective_date";
    report->qrpAccount->DataSet = modificationDataset;
    report->qrpAccount->DataField = "name";
    report->qrpCOB->DataSet = modificationDataset;
    report->qrpCOB->DataField = "cob_number";
    report->qreDebit->Expression = "FORMATNUMERIC('$ ###,##0.00', (debit_amount))";
    report->qreCredit->Expression = "FORMATNUMERIC('$ ###,##0.00', (credit_amount))";
    report->qrpTransNum->DataSet = modificationDataset;
    report->qrpTransNum->DataField = "transaction_number";
    report->qrpNote->DataSet = modificationDataset;
    report->qrpNote->DataField = "note";
    report->qpPRModification->Prepare();
    report->qpPRModification->QRPrinter->Save("pagea.qrp");
    report->CloseNow();
  FORM_CATCH
  source->Clear();
  source->LoadFromFile("pagea.qrp");
  for (int j = 1; j <= source->PageCount; j++)
  {
    destination->AddPage(source->GetPage(j));
  }
  modificationDataset->Close();
} // end transactions modified report
*/


// ---- Dead Code #2 - From btnRunClick ------------------------------------------------------
/*
int unnamed_copies = StringToInt(editUnnamed->Text.c_str());
int named_copies = StringToInt(editNamed->Text.c_str());
if((unnamed_copies > 0) || (named_copies > 0))
{
  string sql_get_justification =
    "select bills.id, affiliations.cdib, "
    " person_names.last_name || ', ' || person_names.first_name || ' ' || person_names.suffix name, "
    " tasks.patient_control_number, tasks.date_of_task, bills.total_charge, bills.invoice_amount, bills.paid_amount "
    "from purchase_reqs "
    "left join bills on purchase_reqs.id = bills.purchase_req_id and bills.current_id + 1 > 2 + 1 "
    "left join tasks on bills.task_id = tasks.id "
    "left join people on tasks.person_id = people.id "
    "left join person_names on people.person_names_id = person_names.id "
    "left join affiliations on people.affiliations_id = affiliations.id "
    "where purchase_reqs.id = " + id + " "
    "order by person_names.last_name, person_names.first_name";
  justificationDataset->SelectSQL->Add(sql_get_justification.c_str());
  justificationDataset->Open();
  justificationDataset->First();

  // PHILIP -- again, select the name of the tax entity for this PR (we already have this, i swear)
  // TODO
  string sql_get_name =
    "select tax_entities.name "
    "from purchase_reqs "
    "left join tax_entities on purchase_reqs.tax_entity_id = tax_entities.id "
    "where purchase_reqs.id = " + id;
  FORM_TRY
    // TODO -c TYPO: (well, sorta) ... this class name (likely file name too) is Tfrom* not Tform*
    TfromReportRequsitionJustification *report = new TfromReportRequsitionJustification(NULL);
    report->qpRequisitionJustification->DataSet = justificationDataset;
    report->qrpProviderName->Caption = PrettyString(GetOneField(dbTransaction, sql_get_name)).c_str();
    // PHILIP -- and the PR number too! good god! what the fuck?!
    report->qrpPR->Caption = GetOneField(dbTransaction,
      "select purchase_reqs.purchase_req_number from purchase_reqs where purchase_reqs.id = " + id).c_str();
    report->lblUser->Caption = PrettyString(lookupTables->ValueOf("users", current_userid)).c_str();
    report->qrpCDIB->DataSet = justificationDataset;
    report->qrpCDIB->DataField = "cdib";
    report->qrpDOS->DataSet = justificationDataset;
    report->qrpDOS->DataField = "date_of_task";
    report->qreTotalCharge->Expression = "FORMATNUMERIC('$ ###,###,##0.00', (total_charge))";
    report->qreInvoiceAmount->Expression = "FORMATNUMERIC('$ ###,###,##0.00', (invoice_amount))";
    report->qreAmountPaid->Expression = "FORMATNUMERIC('$ ###,###,##0.00', (paid_amount))";
    report->qreWriteOff->Expression = "FORMATNUMERIC('$ ###,###,##0.00', (total_charge - paid_amount))";
    report->qreSumTotalCharge->Expression = "FORMATNUMERIC('$ ###,###,##0.00', SUM(total_charge))";
    report->qreSumInvoiceAmount->Expression = "FORMATNUMERIC('$ ###,###,##0.00', SUM(invoice_amount))";
    report->qreSumAmountPaid->Expression = "FORMATNUMERIC('$ ###,###,##0.00', SUM(paid_amount))";
    report->qreSumWriteOff->Expression = "FORMATNUMERIC('$ ###,###,##0.00', SUM(total_charge - paid_amount))";

    // PHILIP -- use this dataset to run the un-named copy as many times as necessary
    if (unnamed_copies > 0)
    {
      report->lblName->Caption = "";
      report->qrpName->DataField = "";
      report->lblPatientAccountNum->Caption = "";
      report->qrpPatientAccountNum->DataField = "";
      report->qpRequisitionJustification->Prepare();
      report->qpRequisitionJustification->QRPrinter->Save("pageb.qrp"); // jhaley - wry!?!?
      for(int k = 0; k < unnamed_copies; k++)
      {
        source->Clear();
        source->LoadFromFile("pageb.qrp"); // jhaley - wry!?!?
        for (int j = 1; j <= source->PageCount; j++)
          destination->AddPage(source->GetPage(j));
      }
    }

    // PHILIP -- use this dataset again to run the named copy as many times as necessary
    if(named_copies > 0)
    {
      report->qrpName->DataSet = justificationDataset;
      report->qrpName->DataField = "name";
      report->qrpPatientAccountNum->DataSet = justificationDataset;
      report->qrpPatientAccountNum->DataField = "patient_control_number";
      report->qpRequisitionJustification->Prepare();
      report->qpRequisitionJustification->QRPrinter->Save("pagec.qrp");  // jhaley - wry!?!?
      source->Clear();
      source->LoadFromFile("pagec.qrp"); // jhaley - wry!?!?
      for(int k = 0; k < named_copies; k++)
      {
        for (int j = 1; j <= source->PageCount; j++)
          destination->AddPage(source->GetPage(j));
      }
    }
    report->CloseNow();
  FORM_CATCH
  justificationDataset->Close();
}
*/


// ---- Dead Code #3 - from compositeReportAddReports ----------------------------------------
// PHILIP -- getting list of accounts used and their amounts, regardless of entity (only for certain dates)
// PHILIP -- this needs to be changed to handle receivables, etc. correctly, and maybe for dates?
/*
sql = "select "
      "  acc_level_one.short_item || '-' || acc_level_three.short_item || '-' || "
      "  acc_level_four.short_item  || '-' || acc_level_two.short_item account_name, "
      " (sum(transactions.debit_amount) - sum(transactions.credit_amount)) account_amount "
      "from acc_level_one, acc_level_three, acc_level_four, acc_level_two, purchase_reqs "
      "inner join bills on purchase_reqs.id = bills.purchase_req_id "
      "inner join transactions on bills.id = transactions.bill_id "
      "inner join accounts on transactions.account_id = accounts.id "
      "where accounts.acc_level_one_id = acc_level_one.id and "
      " accounts.acc_level_three_id = acc_level_three.id and "
      " accounts.acc_level_four_id = acc_level_four.id and "
      " accounts.acc_level_two_id = acc_level_two.id and "
      " (accounts.account_type_id = '" + lookupTables->IdOf("account_types", "Expense") + "' or "
      " accounts.account_type_id = '" + lookupTables->IdOf("account_types", "Receivable") + "') and "
      " purchase_reqs.id = '" + id + "' and cast(transactions.last_update_stamp as date) <= purchase_reqs.date_sent "
      "group by accounts.name, acc_level_one.short_item, acc_level_three.short_item, acc_level_four.short_item, "
      " acc_level_two.short_item";
*/
/* pr_id, bill_id, paid_entity_id, entity_id, account_id, account_type_id, pending_id, payable_id, receivable_id, debit, credit*/


// ---- Dead Code #4 - from compositeReportAddReports ----------------------------------------
// PHILIP -- for vendors only, select the names, etc.
// PHILIP -- needs to be modified for list of fields, as well as pulling transactions (dates?)
// PHILIP -- needs to set up the group band / expression so it'll group bills transactions together
/*
sql = "select distinct(users.item) user_name, bills.groups groups, bills.group_or_user, "
      "tax_entities.name vendor_name, bills.invoice_number, bills.invoice_amount, bills.note "
      "from purchase_reqs "
      "left join tax_entities on purchase_reqs.tax_entity_id = tax_entities.id "
      "left join bills on purchase_reqs.id = bills.purchase_req_id "
      "left join users on bills.user_id = users.id "
      "left join tasks on bills.task_id = tasks.id "
      "left join transactions on bills.id = transactions.bill_id "
      "left join accounts on transactions.account_id = accounts.id "
      "where accounts.account_type_id = '" + lookupTables->IdOf("account_types", "Expense") + "' and "
      "purchase_reqs.id = " + id + " and cast(transactions.last_update_stamp as date) <= purchase_reqs.date_sent";
*/


// ---- Dead Code #5 - from compositeReportAddReports ----------------------------------------
// PHILIP -- stupid, stupid thingie of group_or_user ... MUST GO AWAY
/*
sql =
  "select bills.group_or_user "
  "from purchase_reqs "
  "inner join bills on purchase_reqs.id = bills.purchase_req_id "
  "where purchase_reqs.id = " + id;
int group_or_user = StringToInt(GetOneField(dbTransaction, sql));
pr_report->txtEmployee->DataSet = prDataset;
if(group_or_user == 0)
  pr_report->txtEmployee->DataField = "groups";
else
  pr_report->txtEmployee->DataField = "user_name";
*/
// PHILIP -- now back to reasonable crap ...


// ---- Dead Code #6 - from compositeReportAddReports ----------------------------------------
// PHILIP -- now for provider bills (woohoo!)
/*
sql =
  "select facilites.name facility_name, facilites.taxid facility_taxid, providers.name provider_name, "
  " providers.taxid provider_taxid, pr_report.total_charge, pr_report.amount_paid "
  "from pr_report, tax_entities providers, tax_entities facilites "
  "where pr_report.facility_id = facilites.id and pr_report.provider_id = providers.id "
  "order by providers.name, providers.taxid, facilites.name, facilites.taxid";
*/


// ---- Dead Code #7 - from IsValidOnScreen --------------------------------------------------
/*
  if((StringToInt(editOriginal->Text.c_str()) > 0) || (StringToInt(editReturnToOriginator->Text.c_str()) > 0) ||
     (StringToInt(editProcurement->Text.c_str()) > 0) || (StringToInt(editCopyForOriginator->Text.c_str()) > 0) ||
     (StringToInt(editUnnamed->Text.c_str()) > 0) || (StringToInt(editNamed->Text.c_str()) > 0))
    return true;
  else
    return false;
*/

