//---------------------------------------------------------------------------
#include <vcl.h>
#pragma hdrstop

#include "launchPatientDemographicsReport.h"
#include "reportClinicSuperBill.h"
#include "reportClinicPatientLabels.h"
#include "derivedheaders.h"
//---------------------------------------------------------------------------
#pragma package(smart_init)
#pragma link "codeSpecialGeneric"
#pragma resource "*.dfm"
TformLaunchPatientDemographicsReport *formLaunchPatientDemographicsReport;
//---------------------------------------------------------------------------
__fastcall TformLaunchPatientDemographicsReport::TformLaunchPatientDemographicsReport(TComponent* Owner,
    string p_source_table, string p_source_id, string p_group_id)
        : TformSpecialGeneric(Owner)
{
  source_table = LowercaseString(p_source_table);
  source_id = LowercaseString(p_source_id);

  if (source_table == "people")
  {
    checkSuperbill->Checked = false;
    checkSuperbill->Enabled = false;
    panelCorners1->Enabled = false;
    checkProviderNotes->Checked = false;
    checkProviderNotes->Enabled = false;
    panelCorners2->Enabled = false;
    checkLabels->Checked = true;
    checkShowDateTime->Checked = false;
    checkShowDateTime->Enabled = false;
  }
  else
  { /* ??? */ }

  // jhaley 20100720: get clincal groups for which the user has permission (has_charts implies is a clinic)
  // jhaley 20110222: clinic_id > 0 is too restrictive; it leaves out Behavioral Health et al. Evidently they were still
  // able to use this form previously via a glitch, through which their group would be put in the box even though it was
  // not in the drop down...
  set<string> group_permissions = GetSpecialGroupPermissions("general");
  string location_groups_sql = "select id, item, valid from groups where " /*clinic_id > 0 and */ " has_charts = 1 and "
                               + SafeInClause("id", SetToVec(group_permissions)) + " ";
  lookupTables->RegisterComboBox("groups", cboLocation, location_groups_sql);

  // set the group automatically - jhaley: allow use of a passed-in group_id for the sake of appointments
  if (p_group_id != "")
  {
    cboLocation->Text = lookupTables->ValueOf("groups", p_group_id).c_str(); // put value in box for custom SQL to kick in
    if (!lookupTables->IsInLookUpTable("groups", cboLocation))               // if it works, keep it; otherwise, no default
      cboLocation->Text = "";
  }
  else if (cboLocation->Items->Count == 1)
    cboLocation->ItemIndex = 0;
}
//---------------------------------------------------------------------------
void __fastcall TformLaunchPatientDemographicsReport::FormClose(TObject *Sender, TCloseAction &Action)
{
  lookupTables->UnregisterComboBox("groups", cboLocation);
  TformSpecialGeneric::FormClose(Sender, Action);
}
//---------------------------------------------------------------------------
void __fastcall TformLaunchPatientDemographicsReport::btnCloseClick(TObject *Sender)
{
  Close();
}
//---------------------------------------------------------------------------
void __fastcall TformLaunchPatientDemographicsReport::FormShortCut(TWMKey &Msg, bool &Handled)
{
  Handled = true;
  if (Msg.CharCode == VK_RETURN)
    btnRun->Click();
  else
    Handled = false;

  if (!Handled)
    TformSpecialGeneric::FormShortCut(Msg, Handled);
}
//---------------------------------------------------------------------------
bool TformLaunchPatientDemographicsReport::IsValidOnScreen()
{
  if(cboLocation->Text == "" || !lookupTables->IsInLookUpTable("groups", cboLocation))
  {
    PSay("Please select a clinic", m_missing_data, MB_OK);
    cboLocation->SetFocus();
    return false;
  }
  if (!checkSuperbill->Checked && !checkProviderNotes->Checked && !checkLabels->Checked)
  {
    PSay("Nothing to do!  Please check at least one of the three main checkboxes.", m_missing_data, MB_OK);
  }
  if (checkLabels->Checked)
  {
    if (IsBlankOrZero(editLabelsPerPerson->Text.c_str()) || !IsInt(editLabelsPerPerson->Text.c_str()))
    {
      PSay("Please enter a valid number of labels to print per patient", m_missing_data, MB_OK);
      editLabelsPerPerson->SetFocus();
      return false;
    }
    if (IsBlankOrZero(editStartingLabel->Text.c_str()) || !IsInt(editLabelsPerPerson->Text.c_str()) ||
        StringToInt(editStartingLabel->Text.c_str()) < 0 || StringToInt(editStartingLabel->Text.c_str()) > 30)
    {
      PSay("Please enter a valid starting label", m_missing_data, MB_OK);
      editStartingLabel->SetFocus();
      return false;
    }
  }
  return true;
}
//---------------------------------------------------------------------------
void __fastcall TformLaunchPatientDemographicsReport::btnRunClick(TObject *Sender)
{
  if(IsValidOnScreen())
  {
    LogIt("appt_demographics.log", "Starting transaction");
    Processing(true);

    TIBTransaction * dbTransaction = new TIBTransaction(NULL);
    TIBDataSet * dbDataSet = new TIBDataSet(NULL);
    try
    {
      dbTransaction->DefaultDatabase = dbDatabase;
      dbTransaction->StartTransaction();

      // insert blanks and repeatings as sets ito temp_repeat_data, pull from that to dataset
      dbDataSet->Database = dbDatabase;
      dbDataSet->Transaction = dbTransaction;
      dbDataSet->UniDirectional = false;

      string patient_sql, blank_sql, query_sql;

      LogIt("appt_demographics.log", "deleting from temp_repeat_data");
      ExecuteStatement(dbTransaction, "delete from temp_repeat_data");
      LogIt("appt_demographics.log", "deleting from temp_demographics");
      ExecuteStatement(dbTransaction, "delete from temp_demographics");

      int num_blanks = StringToInt(editStartingLabel->Text.c_str())-1;
      int num_repeats = StringToInt(editLabelsPerPerson->Text.c_str());
      bool print_chart_and_code = /* cboLocation->Text != "Shawnee Clinic" */ true;

      string medicaid_types =
        "'" + lookupTables->IdOf("insurance_types", "Medicaid") + "','"
            + lookupTables->IdOf("insurance_types", "HMO Medicaid") + "'";
      string medicare_types =
        "'" + lookupTables->IdOf("insurance_types", "Medicare A") + "','"
            + lookupTables->IdOf("insurance_types", "Medicare B") + "','"
            + lookupTables->IdOf("insurance_types", "Medicare D") + "','"
            + lookupTables->IdOf("insurance_types", "Medicare Advantage") + "'";

      /* This is the type of insurance benefit they care about; Dental cares about private dental insurance only,
         Medical about private medical only (presumably; they haven't asked for this feature), etc.  This is where you set it. */
      string pri_ins_type = (cboLocation->Text=="Dental Clinic")?"Dental":"";

      // TODO -c WRONG: This should use ExecuteInsertStatement!
      LogIt("appt_demographics.log", "inserting into temp_demographics");
      patient_sql =   "insert into temp_demographics (person_id, appt_date, appt_time, person_name, social_security_number, birth_date, gender, tribal_code, chart_number, ins_cov_shortstr, address1, address2, home_phone) "
                      "select people.id, ";
      if (source_table == "appointments")
       patient_sql += "appointments.end_date appt_date, appointments.appointment_time appt_time, ";
      else
       patient_sql += "NULL appt_date, NULL appt_time, ";
      set<string> can_see_charts = GetSpecialGroupPermissions("charts");
      patient_sql +=  "last_name || ', ' || (case when first_name != '' then ' ' else '' end) || first_name || (case when middle_name != '' then ' ' else '' end) || middle_name || (case when suffix != '' then ' ' else '' end) || suffix person_name, "
                      "people.social_security_number, people.birth_date, gender_types.abbreviation gender, tribe_types.code tribal_code, "
                      "(select first 1 charts.chart_number from charts "
                       "inner join groups on charts.location_id = groups.chart_location_id and groups.has_charts = 1 and charts.location_id > 0 "
                       "where charts.person_id = people.id and " + SafeInClause("groups.id", SetToVec(can_see_charts)) + " "
                      /*
                       if(optionsPermissions["special.lac.charts"]["enabled"] == "true")
                         sql += " and charts.group_id = " + lookupTables->IdOf("groups", "Little Axe Clinic");
                       else if(optionsPermissions["special.sc.charts"]["enabled"] == "true")
                         sql += " and charts.group_id = " + lookupTables->IdOf("groups", "Shawnee Clinic");
                       else if(optionsPermissions["special.den.charts"]["enabled"] == "true")
                         sql += " and charts.group_id = " + lookupTables->IdOf("groups", "Little Axe Clinic");
                       else if((optionsPermissions["special.cob.general"]["enabled"] == "true") ||
                         (optionsPermissions["special.bill.general"]["enabled"] == "true") ||
                         (optionsPermissions["special.res.general"]["enabled"] == "true"))
                         sql += " and charts.group_id = " + lookupTables->IdOf("groups", "Contract Health");
                      */
                       "and charts.current_id > 2 order by groups.hl7_priority asc nulls last) chart_number, "
                      "case when mcaid_cov.id is null and mcare_cov.id is null and priv_cov.id is null "
                        "then 'Uninsured' "
                        "else case when mcaid_cov.id is not null then cast('Mcd ' as varchar(4)) else cast('' as varchar(4)) end || "
                             "case when mcare_cov.id is not null then cast('Mcr ' as varchar(4)) else cast('' as varchar(4)) end || "
                             "case when priv_cov.id is not null then cast('Priv ' as varchar(5)) else cast('' as varchar(4)) end "
                        "end, "
                      "mail.line1 || case when mail.line2 is not null and mail.line2 != '' then ' / ' || mail.line2 else '' end address1, "
                      "mail.city || ', ' || mail_st.abbreviation || '  ' || mail.zip address2, "
                      "hp.phone_number home_phone "
                      "from ";
      // not really necessary, but makes more sense to me, logically, at the moment
      if (source_table == "appointments")
       patient_sql += "appointments left join people on appointments.person_id = people.id ";
      else if (source_table == "people")
       patient_sql += "people "; //left join appointments on people.id = appointments.person_id ";
      patient_sql +=  "left join person_names on people.person_names_id = person_names.id "
                      "left join affiliations on people.affiliations_id = affiliations.id "
                      "left join tribe_types on affiliations.tribe_id = tribe_types.id "
                      "left join gender_types on people.gender_id = gender_types.id "
                      "left join person_contact_info mail on people.mailing_address_id = mail.id "
                      "left join state_types mail_st on mail.state_id = mail_st.id "
                      "left join person_contact_info hp on people.home_phone_id = hp.id "
                      "left join insurance_coverage mcaid_cov on mcaid_cov.id = (select first 1 ic.id from insurance_coverage ic inner join insurance_policies ip on ip.id = ic.insurance_policy_id "
                         "where ic.person_id = people.id and ip.insurance_id in (" + medicaid_types + ") "
                         "and (ic.stop_date is null or ic.stop_date > current_date) and ip.current_id+1 >= 4 and ic.current_id+1 >= 4) "
                      "left join insurance_coverage mcare_cov on mcare_cov.id = (select first 1 ic.id from insurance_coverage ic inner join insurance_policies ip on ip.id = ic.insurance_policy_id "
                         "where ic.person_id = people.id and ip.insurance_id in (" + medicare_types + ") "
                         "and (ic.stop_date is null or ic.stop_date > current_date) and ip.current_id+1 >= 4 and ic.current_id+1 >= 4) "
                      "left join insurance_coverage priv_cov on priv_cov.id = (select first 1 ic.id from insurance_coverage ic inner join insurance_policies ip on ip.id = ic.insurance_policy_id ";
      if (pri_ins_type != "")
       patient_sql += "inner join insurance_benefits ib on ib.insurance_policy_id = ip.id and ib.area_id = '" + lookupTables->IdOf("area_types", pri_ins_type) + "' ";
      patient_sql +=  "where ic.person_id = people.id and ip.insurance_id not in (" + medicaid_types + "," + medicare_types + ") "
                      "and (ic.stop_date is null or ic.stop_date > current_date) and ip.current_id+1 >= 4 and ic.current_id+1 >= 4) "
                      "where " + source_table + ".id = '" + source_id + "'";

      patient_sql += "order by person_names.last_name, person_names.first_name";
      if (source_table == "appointments")
       patient_sql += ", appointments.end_date, appointments.appointment_time";

      DebugSql(this, &patient_sql, other_forms, 9);
      ExecuteStatement(dbTransaction, patient_sql);
      // initial report
      LogIt("appt_demographics.log", "selecting from temp_demographics");
      query_sql = "select person_name, appt_date, appt_time, social_security_number, birth_date, gender, tribal_code, chart_number, ins_cov_shortstr, address1, address2, home_phone "
                  "from temp_demographics "
                  "order by 1 nulls first, 2 nulls first, 3 nulls first, 4 nulls first, 5 nulls first, 6 nulls first, 7 nulls first, 8 nulls first, 9 nulls first, 10 nulls first, 11 nulls first, 12 nulls first ";

      //if(debug_level[other_forms]>=8) QueryData(this, dbTransaction, sql);
      DebugSql(this, &query_sql, other_forms, 9);
      dbDataSet->SelectSQL->Clear();
      dbDataSet->SelectSQL->Add(query_sql.c_str());

      dbDataSet->Open();
      dbDataSet->First();

      Application->ProcessMessages();
      if(checkSuperbill->Checked || checkProviderNotes->Checked)
      {
//          if(is_in(cboLocation->Text.c_str(), "Little Axe Clinic", "Shawnee Clinic", NULL))
        corner_types set1 = ct_none, set2 = ct_none;
        if (checkSuperbill->Checked)
        {
          if(radioUL1->Checked) set1 = ct_upper_left;
          if(radioUR1->Checked) set1 = ct_upper_right;
          if(radioLL1->Checked) set1 = ct_lower_left;
          if(radioLR1->Checked) set1 = ct_lower_right;
        }
        if (checkProviderNotes->Checked)
        {
          if(radioUL2->Checked) set2 = ct_upper_left;
          if(radioUR2->Checked) set2 = ct_upper_right;
          if(radioLL2->Checked) set2 = ct_lower_left;
          if(radioLR2->Checked) set2 = ct_lower_right;
        }
        // don't print chart # and tribal code for SC
        LogIt("appt_demographics.log", "Running superbill report");
        TformReportClinicSuperBill::Execute(this, dbDataSet, set1, set2, print_chart_and_code);
        LogIt("appt_demographics.log", "Done running superbill report");
      }

      dbDataSet->Close();

      if(checkLabels->Checked)
      {
        // insert the repetitions of the data row
        for (int i = 1; i < num_repeats; i++) // we already inserted i=0, so now we start at 1
        {
          LogIt("appt_demographics.log", "Inserting repetitions");
          ExecuteStatement(dbTransaction, patient_sql);
        }

        // insert blank values for blank labels
        // it doesn't matter which field we use; we just have to insert one non-null value so that QuickReport will
        // actually print the detail (if all values for a row are null, QR skips it)
        blank_sql = "insert into temp_demographics (ins_cov_shortstr) values (' ')";
        for (int i = 0; i < num_blanks; i++)
        {
          LogIt("appt_demographics.log", "Inserting blank values");
          ExecuteStatement(dbTransaction, blank_sql);
        }
        dbDataSet->Open();
        dbDataSet->First();

        Application->ProcessMessages(); // jhaley TODO: HERE is a sure-fire way to make things blow up :>
        LogIt("appt_demographics.log", "Running labels report");
        print_label_method how_to_print = (radioSheetLabels->Checked ? print_label_sheets : print_label_spooled);
        // you would think we'd pass the starting label to the report, wouldn't you?
        //TformReportClinicPatientLabels::Execute(this, dbDataSet, 1, StringToInt(editStartingLabel->Text.c_str())-1, print_chart_and_code, checkShowDateTime->Checked, how_to_print);
        // but we don't, because we've manually inserted blank rows, so we don't want the report to try to simulate blank rows for us, or we'll get way more blank labels than we need.
        TformReportClinicPatientLabels::Execute(this, dbDataSet, 1, 0, print_chart_and_code, checkShowDateTime->Checked, how_to_print);
        LogIt("appt_demographics.log", "Done running labels report");

        dbDataSet->Close();
      }

      LogIt("appt_demographics.log", "deleting from temp_repeat_data");
      ExecuteStatement(dbTransaction, "delete from temp_repeat_data");
      LogIt("appt_demographics.log", "deleting from temp_demographics");
      ExecuteStatement(dbTransaction, "delete from temp_demographics");

      dbTransaction->Commit();
    }
    catch(...)
    {
      Processing(false);
      PSay("Error running report", m_database_error, MB_OK);
      if (dbTransaction->Active)
        dbTransaction->Rollback();
    }
    delete dbTransaction;
    delete dbDataSet;
    Processing(false);
  }
}
//---------------------------------------------------------------------------
void __fastcall TformLaunchPatientDemographicsReport::radioLabelTypeClick(TObject *Sender)
{
  editStartingLabel->Enabled = radioSheetLabels->Checked;
}
//---------------------------------------------------------------------------

