//---------------------------------------------------------------------------

#include <vcl.h>
#pragma hdrstop

#include "launchPatientHealthSummary.h"
#include "codeSpecialHtmlPreview.h"
#include "derivedheaders.h"
//---------------------------------------------------------------------------
#pragma package(smart_init)
#pragma link "launchGeneric"
#pragma link "codeRubberContainer"
#pragma resource "*.dfm"
TformLaunchPatientHealthSummary *formLaunchPatientHealthSummary;
//---------------------------------------------------------------------------
__fastcall TformLaunchPatientHealthSummary::TformLaunchPatientHealthSummary(TComponent* Owner, set<string> &p_person_ids)
        : TformLaunchGeneric(Owner)
{
  person_ids = p_person_ids;
  radioMyDepartment->Caption = (ReplaceStringWithString(string(radioMyDepartment->Caption.c_str()), "X", PGlobals->current_groupname)).c_str();

  group_filter.SetListViewHandle(listGroups);
  group_filter.SetSecurity("");
  group_filter.AddColumnSpec("Group", "", as_text, true, 250);
  group_filter.AddColumnSpec("Internal Id", "internal_id", as_int, false, 80);
  group_filter.FetchPreferences("launchpatienthealthsummary.groups");
  group_filter.SetCheck(true);
  group_filter.SetCompress(true);
  general_group_permissions = GetSpecialGroupPermissions("general");
  group_filter.SetSQLStatement("select item, id from groups where id in ("+SetToCommaString(general_group_permissions)+") and valid = 1 order by item");
  group_filter.RefreshDataSet();
  group_filter.SetAddEdit(this, NULL, NULL);
  group_filter.CheckAll();

  // include id(0) so there is a "blank" option
  string location_groups_sql = "select id, item, valid from groups where ((clinic_id > 0 and has_charts = 1) or id = 0) order by item"; // clinics have charts
  lookupTables->RegisterComboBox("groups", cboClinic, location_groups_sql);
  if (cboClinic->Text == "") // we'll be nice and fill in the clinic name for them
    if (lookupTables->IsValidId("groups", current_groupid, cboClinic)) // is current user's group a clinic?  if so, set the clinic automatically...
      cboClinic->Text = lookupTables->ValueOf("groups", current_groupid).c_str();

  radioMyDepartment->Caption = (ReplaceStringWithString(radioMyDepartment->Caption.c_str(), "X", PGlobals->current_groupname)).c_str();
}
//---------------------------------------------------------------------------
// jhaley 20101119: CloseReport *cannot* be used as resource release because it's only an event handler for the Close button
// on TformLaunchGeneric (it really shouldn't even be virtual). Use the class destructor instead.
__fastcall TformLaunchPatientHealthSummary::~TformLaunchPatientHealthSummary()
{
  if (lookupTables->TableName(cboClinic) == "groups") // Only if initialized successfully
    lookupTables->UnregisterComboBox("groups", cboClinic);
}
//---------------------------------------------------------------------------
// jhaley: Nope! Can't do this.
/*
bool TformLaunchPatientHealthSummary::CloseReport()
{
  lookupTables->UnregisterComboBox("groups", cboClinic);
  return TformLaunchGeneric::CloseReport();
}
*/
//---------------------------------------------------------------------------
bool TformLaunchPatientHealthSummary::BeforeRunReport()
{
  /* not doing QR, don't need a dbDataset */
  return TformLaunchGeneric::BeforeRunReport();
}
//---------------------------------------------------------------------------
bool TformLaunchPatientHealthSummary::IsValidOnScreen()
{
  string num_visits = editVisits->Text.c_str();
  if (radioMyDepartment->Checked)
  {
    general_group_permissions.clear();
    general_group_permissions.insert(current_groupid);
  }
  else
    general_group_permissions = VecToSet(group_filter.GetMultiID());
  if (!IsInt(num_visits) || StringToInt(num_visits) < 0)
  {
    PSay("Please enter a positive integer for the number of visits", m_message, MB_OK);
    return false;
  }
  else if (cboClinic->Text == "" || !lookupTables->IsInLookUpTable("groups", cboClinic))
  {
    PSay("Please select a clinic.", m_message, MB_OK);
    cboClinic->SetFocus();
    return false;
  }
  else if (general_group_permissions.empty())
  {
    PSay("Please select at least one department whose visits you wish to include.", m_message, MB_OK);
    listGroups->SetFocus();
    return false;
  }
  return true;
}
//---------------------------------------------------------------------------
bool TformLaunchPatientHealthSummary::RunReport()
{
  checkpoint_branch = "Main";
  checkpoint = 0;

 Processing(true);

 string num_visits = editVisits->Text.c_str();
 string clinic_id = lookupTables->IdOf("groups", cboClinic).c_str();
 // we fetched general_group_permissions based on what's checked, in IsValidOnScreen()
 string sql;

 try
 {
   vector<map<string, string> > people_rows;
   vector<map<string, string> > visit_rows;
   vector<map<string, string> > condition_rows;
   vector<map<string, string> > procedure_rows;
   vector<map<string, string> > measurement_rows;

   ofstream html_file;
   html_file.open("patient_health_summary.html", ios::trunc | ios::out);
   if (!html_file.is_open())
   {
    PSay("Failed to create temporary file for report output.", m_debug, MB_OK);
    throw(97);
   }
   htmlStart(html_file, "Patient Health Summary Report");

   if ((optionsPermissions["table.people"]["view"] == "true"))
   {
    set<string> can_see_charts = GetSpecialGroupPermissions("charts");
    string people_sql = "select people.id, "
                        "people.social_security_number, people.birth_date, people.death_date, people.gender_id, "
                        "person_names.last_name || ', ' || person_names.first_name name, "
                        "person_contact_info.line1, person_contact_info.line2, "
                        "person_contact_info.city, state_types.abbreviation, person_contact_info.zip, "
                        "person_contact_info.phone_number, "
                        "(select first 1 charts.chart_number from charts "
                        "inner join groups on charts.location_id = groups.chart_location_id and groups.has_charts = 1 and charts.location_id > 0 "
                        "where charts.person_id = people.id and " + SafeInClause("groups.id", SetToVec(can_see_charts)) +
                        "and charts.current_id > 2 order by groups.hl7_priority asc nulls last) chart_number, "
                        "'" + clinic_id + "' location_id "
                        "from people "
                        "left join person_names on person_names.id = people.person_names_id "
                        "left join person_contact_info on person_contact_info.id = people.person_contact_info_id "
                        "left join state_types on state_types.id = person_contact_info.state_id ";
    if (!person_ids.empty())
      people_sql += "where people.id in (" + SetToCommaString(person_ids) + ") ";
    people_sql += "order by people.id";
    DebugSql(this, &people_sql, other_forms, 9, report_transaction);
    SqlToVecMap (report_transaction, people_sql, people_rows);
   }

   set<string> allowed_task_groups = GetGroupPermissions("table.tasks", "", "view", "true");
   // general permissions (used by the listview) should be a superset of tasks/conditions/measurements/etc...if their permissions are set up wrong, tough titty toenails
   if (allowed_task_groups.size() > 0)
   {
     set<string> temp;
     temp.clear();
     temp = intersect(general_group_permissions, allowed_task_groups);
     allowed_task_groups = temp;
   }

   set<string> allowed_condition_groups = GetGroupPermissions("table.conditions", "", "view", "true");
   if (allowed_condition_groups.size() > 0)
   {
     set<string> temp;
     temp.clear();
     temp = intersect(general_group_permissions, allowed_condition_groups);
     allowed_condition_groups = temp;
   }

   set<string> allowed_obs_groups = GetGroupPermissions("table.measurements", "", "view", "true");
   if (allowed_obs_groups.size() > 0)
   {
     set<string> temp;
     temp.clear();
     temp = intersect(general_group_permissions, allowed_obs_groups);
     allowed_obs_groups = temp;
   }

   for (unsigned int person_count = 0; person_count < people_rows.size(); person_count++)
   {
     if (!allowed_task_groups.empty())
     {
       sql = "select first " + num_visits + " tasks.id, "
             "tasks.person_id, "
             "tasks.date_of_task "
             "from tasks "
             "left join groups on groups.id = tasks.group_id "
             "where tasks.person_id = " + people_rows[person_count]["id"] + " "
             "and tasks.current_id + 1 > 3 "
             "and tasks.group_id in (" + SetToCommaString(allowed_task_groups) + ") "
             "and tasks.method_id = " + lookupTables->IdOf("method_types", "Visit") + " "
             "order by tasks.date_of_task desc";
       DebugSql(this, &sql, other_forms, 9, report_transaction);
       SqlToVecMap(report_transaction, sql, visit_rows);
     }

     if (!visit_rows.empty())
     {
       vector<string> visit_id_vec;
       VecMapColumnToVec(visit_rows, visit_id_vec, "id");
       string visit_ids = VecToCommaString(visit_id_vec);

       htmlAddPerson(html_file, people_rows[person_count], (person_count != 0)); // last parameter is 'new page'

       if (!allowed_condition_groups.empty())
       {
        sql = "select conditions.person_id, "
              "conditions.start_date, "
              "conditions.code, "
              "codes.description note, "
              "groups.short_item grp_name, "
              "case when task_id != 0 then 1 else 0 end is_diagnosed "
              "from conditions "
              "left join codes on codes.code = conditions.code and codes.code_id = 3 "
              "left join groups on groups.id = conditions.group_id "
              "where conditions.task_id in (" + visit_ids + ") "
              "and conditions.current_id + 1 > 3 "
              "and conditions.condition_id != 7 " /* 'relative' */
              "and conditions.group_id in (" + SetToCommaString(allowed_condition_groups) + ") "
              "order by conditions.start_date desc";
         DebugSql(this, &sql, other_forms, 9, report_transaction);
         SqlToVecMap(report_transaction, sql, condition_rows);
       }

       string sql = "select tasks.person_id, "
                    "procedures.start_date, "
                    "procedures.cpt_hcpcs, "
                    "codes.description note, "
                    "groups.short_item grp_name "
                    "from procedures inner join tasks on tasks.id = procedures.task_id "
                    "left join codes on codes.code = procedures.cpt_hcpcs and codes.code_id != 3 " // TODO: currently, ICD-9 is '3', and is the only 'condition' type
                    "left join groups on groups.id = tasks.group_id "
                    "where tasks.id in (" + visit_ids + ") "
                    "and procedures.cpt_hcpcs != '0' "
                    "and procedures.current_id + 1 > 3 "
                    "order by procedures.start_date desc";
       DebugSql(this, &sql, other_forms, 9, report_transaction);
       SqlToVecMap(report_transaction, sql, procedure_rows);

       if (!allowed_obs_groups.empty())
       {
        sql = "select "
               "obs_response_sets.person_id, "
               "obs_questions.item cat, "
               "obs_response_sets.date_of_observation, "
               "obs_questions.obs_question_id obs_question_type_id, "
               "obs_questions.obs_option_set_id, obs_option_types.item obs_ord_item, obs_option_types.rank obs_ord_val, "
               "obs_responses.response_text obs_txt_val, "
               "obs_responses.response_date obs_date_val, "
               "obs_responses.response_metric obs_met_val, units.item obs_met_unit, "
               "groups.short_item grp_name "
              "from obs_response_sets "
               "left join obs_responses on obs_responses.obs_response_set_id = obs_response_sets.id "
               "left join groups on groups.id = obs_response_sets.group_id "
               "left join obs_questions on obs_questions.id = obs_responses.obs_question_id "
               "left join obs_option_types on obs_option_types.id = obs_responses.obs_option_id "
               "left join units on units.id = obs_questions.unit_id "
              "where obs_response_sets.task_id in (" + visit_ids + ") "
               "and obs_response_sets.current_id + 1 > 3 "
               "and obs_response_sets.obs_question_set_id in "
               "(select oqs.id from obs_question_set_types oqs "
                "inner join permissions p on p.s = oqs.permission_name and p.k = 'VIEW' and p.v = 'TRUE' and p.user_id = " + current_userid + ") "
               "and obs_response_sets.obs_question_set_id in (1,2) " // body metrics and labs.  TODO:  suck it (x in (a,b,c) and x in (b,c,e) should only return x in (b,c) right?

               "and (1 = 0 or obs_response_sets.group_id in (" + SetToCommaString(allowed_obs_groups) + ")) "
              "order by "
               "obs_response_sets.person_id, obs_response_sets.date_of_observation desc";
         DebugSql(this, &sql, other_forms, 9, report_transaction);
         SqlToVecMap(report_transaction, sql, measurement_rows);
        }

       if (!condition_rows.empty())
        htmlAddConditions(html_file, condition_rows);

       if (!procedure_rows.empty())
        htmlAddProcedures(html_file, procedure_rows);

       if (!measurement_rows.empty())
        htmlAddMeasurements(html_file, measurement_rows);

       htmlSectionStop(html_file);
     } // end !visit_rows.empty()
   } // end for...people

   htmlStop(html_file);
   html_file.close();

   // apparently, we have to pass the fully-qualified path ... damn. might move this into the DoPrintPreview function? (always gonna be necessary? except not? i dunno. it's not much extra trouble here.)
   string real_cwd = GetCurrentDir().c_str();
   Processing(false);
   TformSpecialHtmlPreview::DoPrintPreview(real_cwd + "\\patient_health_summary.html");
   this->CloseNow();
 }
 catch (...)
 {
   Processing(false);
   PSay("Error running patient health summary report", m_database_error, MB_OK);
 }

 return true;
}
//---------------------------------------------------------------------------
bool TformLaunchPatientHealthSummary::AfterRunReport()
{
  return TformLaunchGeneric::AfterRunReport();
}
//---------------------------------------------------------------------------
void TformLaunchPatientHealthSummary::htmlAddPerson(ostream & o, map<string, string> person_row, bool new_page)
{
 htmlSectionStart(o, new_page);
 htmlPrettyBar(o, PrettyString(person_row["name"]));
 htmlSectionContinue(o);

 o << "<table cellspacing=0 width=100%>"
      "<tr>"
      "<td width=50% valign=top>"
       "<table cellspacing=0 width=100%>";
       // removed - it's in the header now
       // "<tr><td style='padding-right:8px;'><i>Name</i></td><td width=100%>" << PrettyString(person_row["name"]) << "</td></tr>"
       if (optionsPermissions["table.person_contact_info"]["view"] == "true")
       {
        o << "<tr><td valign=top><i>Address</i></td><td width=100%>"
          << FormatAddress(person_row["line1"]) << "<br>";
        if (!person_row["line2"].empty())
         o << FormatAddress(person_row["line2"]) << "<br>";
        o << PrettyString(person_row["city"]) << "<br>"
          << person_row["abbreviation"];
        if (!person_row["abbreviation"].empty() && !person_row["zip"].empty())
         o << ", " << person_row["zip"];
        else
         if (!person_row["abbreviation"].empty())
          o << person_row["zip"];
        o << "</td></tr>"
             "<tr><td><i>Phone</i></td><td width=100%>"
          << FormatPhoneNumber(person_row["phone_number"])
          << "</td></tr>";
       }
  o << "</table>"
      "</td>"
      "<td width=50% valign=top>"
       "<table cellspacing=0 width=100%>"
       "<tr><td><i>SSN</i></td><td width=100%>"
        << PrettySSN(person_row["social_security_number"])
        << "</td></tr>"
       "<tr><td><i>DOB</i></td><td width=100%>"
        << person_row["birth_date"];
       if (!person_row["birth_date"].empty())
        o << " ("
          << CalcAge(Pdate(person_row["birth_date"], global_mode))
          << ")";
  o << "</td></tr>"
       "<tr><td><i>Gender</i></td><td>"
    << lookupTables->ValueOf("gender_types", person_row["gender_id"])
    << "</td></tr>";
       if (optionsPermissions["table.charts"]["view"] == "true")
       {
        o << "<tr><td><i>Chart</i></td><td>"
          << person_row["chart_number"];
        if (!person_row["chart_number"].empty())
         o << " (" << lookupTables->ValueOf("groups", person_row["location_id"]) << ")";
        o << "</td></tr>";
       }
  o << "</table>"
      "</td>"
      "</tr>"
      "</table>";

 // historical --
 // now, this is vexing. i'd like to have the htmlSectionStop() here, but
 // - we'd have to add conditions, procedures, measurements in this function (and pass them to this function)
 // - we only conditionally call each of those functions, based on whether or not there are requests for them at all
 // - we'd have to pass the entire set of request structures into here to do the above
 // ... so oddly, it's much easier to just call htmlSectionStop() from the function calling us. funky, eh?
 // "Aw, boo hoo!"  Fixxored. -- Paul
}
//---------------------------------------------------------------------------
void TformLaunchPatientHealthSummary::htmlAddConditions(ostream & o, vector<map<string, string> > condition_rows)
{
 htmlSectionStart(o, false);
 htmlPrettyBar(o, "Conditions");
 htmlSectionContinue(o);
 o << "<table cellspacing=0 width=100% class='datatable' style='margin-left: 24px;'>"
      "<thead><tr>"
       //"<td><i>Category</i></td>"
       "<td><i>Date</i></td>"
       "<td><i>Code</i></td>"
       "<td><i>Dept.</i></td>"
       "<td><i>Desc.</i></td>"
      "</tr></thead><tbody>";

 for (vector<map<string, string> >::iterator i = condition_rows.begin(); i != condition_rows.end(); i++)
 {
  bool is_diag = ((*i)["is_diagnosed"] != "0");
  if (!is_diag)
   (*i)["note"] = "Unofficial: " + (*i)["note"];

  o << "<tr>"
        //"<td" << (is_diag ? "" : " style='color: lightgrey;'") << ">" << (*i)["cat"]                    << "</td>"
        "<td" << (is_diag ? "" : " style='color: lightgrey;'") << ">" << PrettyDate((*i)["start_date"]) << "</td>"
        "<td" << (is_diag ? "" : " style='color: lightgrey;'") << ">" << (*i)["code"]                   << "</td>"
        "<td" << (is_diag ? "" : " style='color: lightgrey;'") << ">" << PrettyString((*i)["grp_name"]) << "</td>"
        "<td" << (is_diag ? "" : " style='color: lightgrey;'") << ">" << PrettyString((*i)["note"])     << "</td>"
       "</tr>\n";
 }

 if (condition_rows.empty())
  o << "<tr>"
        "<td colspan=4><b>None</b></td>"
       "</tr>";

 o << "</tbody></table>\n";
 htmlSectionStop(o);
}
//---------------------------------------------------------------------------
void TformLaunchPatientHealthSummary::htmlAddProcedures(ostream & o, vector<map<string, string> > procedure_rows)
{
 htmlSectionStart(o, false);
 htmlPrettyBar(o, "Procedures");
 htmlSectionContinue(o);
 o << "<table cellspacing=0 width=100% class='datatable' style='margin-left: 24px;'>"
      "<thead><tr>"
       //"<td><i>Category</i></td>"
       "<td><i>Date</i></td>"
       "<td><i>Code</i></td>" // damn browser not supporting max-width correctly ... this'll seem wrong if they put long comments in the code field -- tried width:expression("121px") ... could give width:expression(true ? "121px" : "121px"), i suppose. (css, that is. IE-only feature.)
       "<td><i>Dept.</i></td>" // damn browser not supporting max-width correctly ... this'll seem wrong if they put long comments in the code field -- tried width:expression("121px") ... could give width:expression(true ? "121px" : "121px"), i suppose. (css, that is. IE-only feature.)
       "<td><i>Desc.</i></td>"
      "</tr></thead><tbody>";

 for (vector<map<string, string> >::iterator i = procedure_rows.begin(); i != procedure_rows.end(); i++)
 {
  o << "<tr>"
        //"<td>" << (*i)["cat"]                    << "</td>"
        "<td>" << PrettyDate((*i)["start_date"]) << "</td>"
        "<td>" << (*i)["cpt_hcpcs"]              << "</td>"
        "<td>" << PrettyString((*i)["grp_name"]) << "</td>"
        "<td>" << PrettyString((*i)["note"])     << "</td>"
       "</tr>\n";
 }

 if (procedure_rows.empty())
  o << "<tr>"
        "<td colspan=4><b>None</b></td>"
       "</tr>";

 o << "</tbody></table>\n";
 htmlSectionStop(o);
}
//---------------------------------------------------------------------------
void TformLaunchPatientHealthSummary::htmlAddMeasurements(ostream & o, vector<map<string, string> > measurement_rows)
{
 htmlSectionStart(o, false);
 htmlPrettyBar(o, "Observations");
 htmlSectionContinue(o);
 o << "<table cellspacing=0 class='datatable' style='margin-left: 24px;'>"
      "<thead><tr>"
       "<td><i>Metric</i></td>"
       "<td><i>Date</i></td>"
       "<td colspan=2 align=center><i>Value</i></td>"
       "<td><i>Dept.</i></td>"
      "</tr></thead><tbody>";

 for (vector<map<string, string> >::iterator i = measurement_rows.begin(); i != measurement_rows.end(); i++)
 {
  o << "<tr>"
        "<td>" << (*i)["cat"]                             << "</td>"
        "<td>" << PrettyDate((*i)["date_of_observation"]) << "</td>";

  if ((*i)["obs_question_type_id"] == "5") // TODO -c MAGIC: date
  {
   o << "<td colspan=2>" << (*i)["obs_date_val"] << "</td>";
  } else
  if ((*i)["obs_question_type_id"] == "4") // TODO -c MAGIC: metric
  {
   o << "<td align=right>" << StripTrailing(StripTrailing(FinancialToString(StringToFinancial((*i)["obs_met_val"])), "0"), ".") << "</td>"
        "<td>" << (*i)["obs_met_unit"] << "</td>";
  } else
  if ((*i)["obs_question_type_id"] == "3") // TODO -c MAGIC: ordinal
  {
   o << "<td colspan=2>" << (*i)["obs_ord_item"] << "</td>";
  } else
  if ((*i)["obs_question_type_id"] == "2") // TODO -c MAGIC: text
  {
   if ((*i)["date_of_observation"] == "Average")
    o << "<td colspan=2>Incalculable</td>";
   else
    o << "<td colspan=2>" << (*i)["obs_txt_val"] << "</td>";
  } else
  if ((*i)["obs_question_type_id"] == "1") // TODO -c MAGIC: choice
  {
   if ((*i)["date_of_observation"] == "Average")
    o << "<td colspan=2>Incalculable</td>";
   else
    o << "<td colspan=2>" << (*i)["obs_ord_item"] << "</td>";
  } else
   o << "<td colspan=2>Unknown question type</td>";
  o << "<td>" << PrettyString((*i)["grp_name"]) << "</td>"
       "</tr>\n";
 }

 if (measurement_rows.empty())
  o << "<tr>"
        "<td colspan=3><b>None</b></td>"
       "</tr>";

 o << "</tbody></table>\n";
 htmlSectionStop(o);
}
//---------------------------------------------------------------------------
void TformLaunchPatientHealthSummary::Do(set<string> &p_person_ids)
{
  FORM_TRY
    TformLaunchPatientHealthSummary *launch = new TformLaunchPatientHealthSummary(NULL, p_person_ids);
    launch->Show();
  FORM_CATCH
}
//---------------------------------------------------------------------------

