//---------------------------------------------------------------------------

#include <vcl.h>
#pragma hdrstop

#include "launchPendingAccountsReport.h"
#include "reportPendingAccounts.h"
#include "derivedheaders.h"
//---------------------------------------------------------------------------
#pragma package(smart_init)
#pragma link "codeSpecialGeneric"
#pragma resource "*.dfm"
TformLaunchPendingAccountsReport *formLaunchPendingAccountsReport;
//---------------------------------------------------------------------------
__fastcall TformLaunchPendingAccountsReport::TformLaunchPendingAccountsReport(TComponent* Owner)
        : TformSpecialGeneric(Owner)
{
 // get form ready ... go !
 edit_days.setup_listview(listDays, false);
 edit_days.add_or_modify_column("day", "Day number", true, 0, 200, as_int, rawString);
 edit_days.setup_callbacks((TForm*)this, InformForm(&(TformLaunchPendingAccountsReport::DayRowSelected)), NULL);

 edit_sources.setup_listview(listSources, false);
 edit_sources.add_or_modify_column("description",      "Description",       true, 0, 400, as_text, rawString);
 edit_sources.add_or_modify_column("tx_parent",        "tx_parent",        false, 1,  50,  as_int, rawString);
 edit_sources.add_or_modify_column("is_coords",        "is_coords",        false, 2,  50,  as_int, rawString);
 edit_sources.add_or_modify_column("is_visits",        "is_visits",        false, 3,  50,  as_int, rawString);
 edit_sources.add_or_modify_column("is_bills",         "is_bills",         false, 4,  50,  as_int, rawString);
 edit_sources.add_or_modify_column("is_purchase_reqs", "is_purchase_reqs", false, 5,  50,  as_int, rawString);
 edit_sources.add_or_modify_column("is_checks",        "is_checks",        false, 6,  50,  as_int, rawString);
 edit_sources.add_or_modify_column("date_context",     "date_context",     false, 7,  50,  as_int, rawString);
 edit_sources.setup_callbacks((TForm*)this, InformForm(&(TformLaunchPendingAccountsReport::DataSourceSelected)), NULL);

 filter_account_types.SetListViewHandle(listAccountTypes);
 filter_account_types.SetSecurity("accounts");
 filter_account_types.AddColumnSpec("Account Type", "accounts", as_text, true, 300);
 filter_account_types.AddColumnSpec("Current Id", "current_id", as_int, false, 80);
 filter_account_types.AddColumnSpec("Internal Id", "internal_id", as_int, false, 80);
 filter_account_types.FetchPreferences("launchpendingaccountsreport.account_types_filter");
 filter_account_types.SetCheck(true);
 filter_account_types.SetCompress(true);
 filter_account_types.SetSQLStatement("select account_types.item, 5, account_types.id from account_types where valid = 1");
 filter_account_types.RefreshDataSet();

 all_source_tables.insert (pair<string, recursive_data_source>("visits",             recursive_data_source("tasks visits", "visits.services_provided_id = 0 and visits.id != 0", "0, visits.id, visits.date_of_task", "0", false)));
 all_source_tables.insert (pair<string, recursive_data_source>("bills",              recursive_data_source("bills", "bills.id != 0", "bills.id, 0, bills.bill_date", "0", false)));
 all_source_tables.insert (pair<string, recursive_data_source>("coord. of benefits", recursive_data_source("tasks cobs", "cobs.services_provided_id != 0 and cobs.id != 0", "0, cobs.id, cobs.date_of_task", "0", false)));
 all_source_tables.insert (pair<string, recursive_data_source>("purchase_reqs",      recursive_data_source("purchase_reqs", "purchase_reqs.id != 0", "purchase_reqs.id", "0", false)));
 all_source_tables.insert (pair<string, recursive_data_source>("checks",             recursive_data_source("checks", "checks.id != 0", "checks.id", "0", false)));

 all_source_links.push_back (recursive_data_source_link("bills", "visits", "bills.task_id = visits.id"));
 all_source_links.push_back (recursive_data_source_link("visits", "coord. of benefits", "visits.parent_task_id = cobs.id"));
 all_source_links.push_back (recursive_data_source_link("bills", "purchase_reqs", "bills.purchase_req_id = purchase_reqs.id"));
 all_source_links.push_back (recursive_data_source_link("purchase_reqs", "checks", "purchase_reqs.check_id = checks.id"));

 lookupTables->RegisterComboBox("account_types", comboAccountType);

 DataSourceSelected();
}
//---------------------------------------------------------------------------
void __fastcall TformLaunchPendingAccountsReport::buttonDaysAddClick(
      TObject *Sender)
{
 // add new break point
 listDays->Selected = NULL;
}
//---------------------------------------------------------------------------
void __fastcall TformLaunchPendingAccountsReport::buttonDaysRemoveClick(
      TObject *Sender)
{
 // remove selected break point
 if (listDays->Selected)
  edit_days.remove_current_row(true);
}
//---------------------------------------------------------------------------
void __fastcall TformLaunchPendingAccountsReport::buttonDaysClearClick(
      TObject *Sender)
{
 // clear all points
 edit_days.clear_data();
}
//---------------------------------------------------------------------------
void TformLaunchPendingAccountsReport::DayRowSelected ()
{
 // fill in the edit box
 map<string, string> row_data;
 if (!edit_days.get_current_row(row_data))
  row_data["day"] = "0";

 editDayNumber->Text = row_data["day"].c_str();
}
//---------------------------------------------------------------------------
void __fastcall TformLaunchPendingAccountsReport::buttonDaysSaveClick(
      TObject *Sender)
{
 // save this break point
 map<string, string> new_row;
 set<string> unique_column;
 unique_column.insert("day");

 if (IsInt(editDayNumber->Text.c_str()))
 {
  new_row["day"] = editDayNumber->Text.c_str();
  if (!edit_days.is_future_duplicate(new_row, unique_column))
  {
   edit_days.add_or_modify_row_data(new_row, true);
   buttonDaysAdd->Click();
  }
  else
   PSay ("That break-point already exists. Please select a different day number.", m_duplicate_data, MB_OK);
 } else
  PSay ("You must enter a valid integer number.", m_invalid_data, MB_OK);
}
//---------------------------------------------------------------------------
void __fastcall TformLaunchPendingAccountsReport::buttonRunClick(TObject *Sender)
{
 // run report

 // create column headers and expression fields as we go along ... wee!
 // try to space them out equally along the time-line
 // given A, B, C, D, E (5)
 // the delimeters are:
 // -inf ... A ... B ... C ... D ... E ... +inf (7)
 // gives us
 // -inf to A ... A to B ... B to C ... C to D ... D to E ... E to +inf ... (6)
 //
 // then we have the total for covered items (+1)
 // possibly the total of all payables (+1)
 // possibly the total of all receivables (+1)

 // we'll get totals for everyone ...

 // then we'll get all the transactions and information about them, for the type of account selected

 // then we'll get, for each rule, the list of task_id or bill_id values that match the rule

 // then we'll take the distinct entries, inner join them with
 // the transaction summaries, to get the amounts that go with the items
 // outer join them with the per-entity amounts where those amounts are non-zero, for each type
 // and ... still need to inner-join with entities to get their name (maybe id)

 // distinct rule information inner join entities ... not a problem
 // we can get, in only a few (3) seconds, for all entities, all account types, current total ...
 // at most, there's gonna be around 3000 entities, with at most 6 types of stuff. likely much less.
 // we'll store that in memory instead, and pull it out of our arses ...
 // -but-
 // we still need to know about everyone with an outstanding thingie, no?
 // we might not be selecting all the types of things that cause receivable, while wanting to know about
 // anyone with receivables, period.
 // this is now an option: (o) based on items (o) based on totals (o) based on either

 // if we want (o) based on items
 // take all items we've found (distinct) and inner join them with transactions, inner joined with entities ...

 // if we want (o) based on totals
 // take all entities with any totals != 0 among those listed ... then left join that with transactions inner joined with items.

 // if we want (o) based on either
 // errr ... take all entities with any totals != 0 among those listed ... then outer join that with transactions inner joined with items?

 // (by "transactions", we mean the temp table of bill/task/entity/sum for a given account type)

 // which date to use? it was planned to each rule could specify the date to use -- but it'd be easy to do the following:
 // visits on COB's, using COB date
 // visits with bills not on purchase requests, using Visit date
 // those overlap -- although we can clean up the list of task_id's to use, we'll wind up with != dates
 // now what?

 // solution to the date problem -- jennifer assures me it's normal to just use the transaction date,
 // whatever that was at the time. therefore, the option to select a different source for the date has
 // been removed, and transaction dates will be used instead. hopefully, ... that'll be consistent.

 // wait, no ...
 // instead, we'll use the task or bill date, straight ... that way, for a given item, only one date is used.
 // that SHOULD be the same date as the transactions (at least most) and ... we won't get one item split
 // across several dates, or worse, have one date with negative stuff, the other with positive ... ew.

 // so. if it's a visit, or a cob, we use the task date. if it's a bill, we use the bill date. if the
 // transactions disagree ... too bad. if they want to use a different date, too bad. the reasoning is above.

 // now that we're all thoroughly confused ...

 // TODO: check for oddities in options selected
 string aging_account_type_id = lookupTables->IdOf("account_types", comboAccountType->Text.c_str());
 vector<map<string, string> > result_days;
 edit_days.export_data(result_days);
 vector<map<string, string> > data_sources;
 edit_sources.export_data(data_sources);
 vector<string> account_types_to_total = filter_account_types.GetMultiID();

 if (aging_account_type_id.empty())
 {
  PSay ("You must select a valid type of account to itemize from the drop-down.", m_invalid_data, MB_OK);
  return;
 }

 if (checkShowItemTotal->Checked && data_sources.empty())
 {
  PSay ("Showing item-totals isn't useful unless you have at least one data-source. Otherwise you'll get a column of 0.00 values.", m_invalid_data, MB_OK);
  return;
 }

 if (!radioShowByTotals->Checked)
 {
  // must have rules, otherwise this is stupid ...
  if (data_sources.empty())
  {
   PSay ("If you'd like to show information about entities with 'items', you're going to need to select at least one source of said items.\nSuggestion: to get all sources, create three rules: one for each type, with nothing checked or unchecked.", m_invalid_data, MB_OK);
   return;
  }
  // must have date periods (at least one division) ? maybe not. a total will do.
  // and even then ... maybe just warn? let them have no date periods and nothing shown ...
  // but still join by it. joining by items makes sense even if you don't show the data.
  if (result_days.size() == 0 && !checkShowItemTotal->Checked && !checkShowItems->Checked)
  {
   PSay ("You've decided to show any entities who at least have some dated items in the sources you've described.\nHowever, you're not showing any item totals, nor are you detailing the items. Please select at least one of these options.", m_invalid_data, MB_OK);
   return;
  }
 }

 if (!radioShowByItems->Checked)
 {
  // need at least one account type selected, yes? it would make sense to say
  // "show if expenses != 0, but don't show me the value of expenses" but ...
  // really rather not do that. just seems dorkier than the other. can be changed later.
  if (account_types_to_total.empty())
  {
   PSay ("You've decided to show any entities who at least have one non-zero absolute total.\nYou must, however, select at least one type of account for which to show an absolute total.", m_invalid_data, MB_OK);
   return;
  }
 }

 Processing(true);

 TIBTransaction * dbTransaction = new TIBTransaction(NULL);
 if (StdTransaction(dbTransaction, dbDatabase))
 {
  try
  {
   // temp table descriptions

   // temp_aging_entity_totals    (tax_entity_id, account_type_id, amount)
   // temp_aging_transactions     (bill_id, task_id, tax_entity_id, amount)
   // temp_aging_dated_items      (bill_id, task_id, item_date)
   // temp_aging_dist_dated_items (bill_id, task_id, item_date, tax_entity_id, amount)
   string sql_delete_entity_totals    = "delete from temp_aging_entity_totals";
   string sql_delete_transactions     = "delete from temp_aging_transactions";
   string sql_delete_dated_items      = "delete from temp_aging_dated_items";
   string sql_delete_dist_dated_items = "delete from temp_aging_dist_dated_items";

   string sql_insert_entity_totals =
    "insert into temp_aging_entity_totals "
    "select tax_entity_id, accounts.account_type_id, sum(credit_amount - debit_amount) "
    "from transactions "
    "inner join accounts on accounts.id = transactions.account_id "
    "group by tax_entity_id, account_type_id "
    "having sum(credit_amount - debit_amount) != 0.00";

   string sql_insert_transactions  =
    "insert into temp_aging_transactions "
    "select bill_id, task_id, tax_entity_id, sum(credit_amount - debit_amount) "
    "from transactions "
    "inner join accounts on accounts.id = transactions.account_id "
    "where accounts.account_type_id = " + aging_account_type_id +
    " group by tax_entity_id, bill_id, task_id "
    "having sum(credit_amount - debit_amount) != 0.00";

   // run the above stuff first, so we can join to it next
   if (!ExecuteStatement(dbTransaction, sql_delete_entity_totals))
   {
    PSay ("Failed to delete old temporary totals.", m_database_error, MB_OK);
    throw(1);
   }
   if (!ExecuteStatement(dbTransaction, sql_delete_transactions))
   {
    PSay ("Failed to delete old temporary transactions.", m_database_error, MB_OK);
    throw(2);
   }
   if (!ExecuteStatement(dbTransaction, sql_delete_dated_items))
   {
    PSay ("Failed to delete old date-categorized items.", m_database_error, MB_OK);
    throw(3);
   }
   if (!ExecuteStatement(dbTransaction, sql_delete_dist_dated_items))
   {
    PSay ("Failed to delete old distinct date-categorized items.", m_database_error, MB_OK);
    throw(4);
   }

   if (!ExecuteStatement(dbTransaction, sql_insert_entity_totals))
   {
    PSay ("Failed to insert new temporary totals.", m_database_error, MB_OK);
    throw(5);
   }
   if (!ExecuteStatement(dbTransaction, sql_insert_transactions))
   {
    PSay ("Failed to insert new temporary transactions.", m_database_error, MB_OK);
    throw(6);
   }

   string sql_insert_dated_items = "";
   for (vector<map<string, string> >::iterator i_source = data_sources.begin(); i_source != data_sources.end(); i_source++)
   {
    sql_insert_dated_items =
     "insert into temp_aging_dated_items ";
    string chosen_base = comboParent->Items->Strings[StringToInt((*i_source)["tx_parent"])].c_str();

    map<string, recursive_data_source> local_source_tables = all_source_tables;
    local_source_tables["visits"].exists_value             = (*i_source)["is_visits"];
    local_source_tables["bills"].exists_value              = (*i_source)["is_bills"];
    local_source_tables["coord. of benefits"].exists_value = (*i_source)["is_coords"];
    local_source_tables["purchase_reqs"].exists_value      = (*i_source)["is_purchase_reqs"];
    local_source_tables["checks"].exists_value             = (*i_source)["is_checks"];
    local_source_tables[chosen_base].exists_value          = "1";

   // was here. we've got a problem. those inserts are going to take bloody long. and i mean it.
   // we can probably minimize the damange by joining this crap to the transactions, for items where it's
   // already possible there's something left. that should help. it should be quick to get the list of
   // bill_id or task_id to check -- and we can run off that, going back and checking each one to see if it
   // matches one of the rules. still not fast, but faster than trying to get two sets of all possible values
   // just so we can join 'em back together again and get 5 rows.

    local_source_tables[chosen_base].table_and_alias += ", temp_aging_transactions";
    if (chosen_base == "bills")
     local_source_tables[chosen_base].where_clause   += " and bills.id = temp_aging_transactions.bill_id";
    else if (chosen_base == "visits")
     local_source_tables[chosen_base].where_clause   += " and visits.id = temp_aging_transactions.task_id";
    else
     local_source_tables[chosen_base].where_clause   += " and cobs.id = temp_aging_transactions.task_id";

    sql_insert_dated_items += RecursiveTableJoinSqlGenerator (local_source_tables, all_source_links, chosen_base, true);
    if (!ExecuteStatement(dbTransaction, sql_insert_dated_items))
    {
     PSay ("Failed to insert new dated items.", m_database_error, MB_OK);
     throw(7);
    }
   }

   // not only will we take the distinct ones, but we'll also only take the ones with anything outstanding,
   // and join it together ahead of time in a nice, convenient manner. (this could maybe have been done in
   // a query, but seeing as those queries would contain left/right/outer joins ... that seemed highly
   // dangerous from a semantic point of view. no, really. firebird gets confused about what you want.)

   // (note: these have already been matched up during per-rule selects, but nothing guaranteed that they
   // would come through unique ... and when we join them again to get the rest of the info, we don't want
   // duplicates appearing. so we distinct it again. that's the reason.)
   string sql_insert_distinct_dated_items =
    "insert into temp_aging_dist_dated_items "
    "select distinct temp_aging_transactions.bill_id, temp_aging_transactions.task_id, "
    "temp_aging_dated_items.item_date, temp_aging_transactions.tax_entity_id, "
    "temp_aging_transactions.amount "
    "from temp_aging_dated_items "
    "inner join temp_aging_transactions "
    "on temp_aging_dated_items.bill_id = temp_aging_transactions.bill_id and "
    "temp_aging_dated_items.task_id = temp_aging_transactions.task_id";

   if (!ExecuteStatement(dbTransaction, sql_insert_distinct_dated_items))
   {
    PSay ("Failed to insert distinct new dated items.", m_database_error, MB_OK);
    throw(8);
   }

   // stuff's been inserted ... now what?
   // must retrieve the grid of amounts, etc.
   // self-joins are rather slow. sub-selects aren't great either. multiple passes? i don't want to
   // modify the table layout for each pass, adding columns. it'd be best to do that at the end, during the
   // final select.
   // decision: we're going to pre-fetch a bunch. the select to give us a grid of per-date totals isn't
   // too bad, in my tests it took around 4 to 6 minutes for about 8 categories of dates. it's extremely
   // variable, however. number of items retrieved by previous stuff is gonna be a problem.
   // i've made changes to separate the first and second passes for item id's -- it was too dangerous
   // (speed-wise) to leave a pass_number in the table. that's thousands upon thousands of rows, with
   // a very non-specific index. it'd work, but ... ew?
   // okay. so we'll pull into memory ...
   // - the entity totals
   // - the entity per-date totals (for everyone with something on the 'right' account type, as determined above)
   // we'll pull the item information from the database
   // - entity_totals (for the specific type) left join (dist_dated_items) (based on totals)
   // - entity_totals (for the specific type) right join (dist_dated_items) (based on items)
   // - entity_totals (for the specific type) outer join (dist_dated_items) (based on both)
   // - if they don't want to see the details of the items, ... above still applies. (math is the same.)
   // when we pull from dist_dated_items, we need to left join to the tasks or bill to get more info.

   // pull in the per-entity-per-period totals
   map<string, string> dated_column_descriptions;
   vector<string> dated_column_order;
   Pdate today_date = Pdate(TDateTime::CurrentDate());
   Pdate prev_date;
   Pdate this_date;
   string column_name;

   // turn them into integers -- puts them in order by integer, rather than text, value. very neat.
   set<int> int_result_days;
   for (unsigned int i = 0; i < result_days.size(); i++)
    int_result_days.insert(StringToInt(result_days[i]["day"]));

   string sql_select_entity_period_totals =
    "select ";
   set<int>::iterator i_prev_day;
   for (set<int>::iterator i_day = int_result_days.begin(); i_day != int_result_days.end(); i_day++)
   {
    sql_select_entity_period_totals +=
     "(select sum(temp_aging_dist_dated_items.amount) "
     "from temp_aging_dist_dated_items "
     "where temp_aging_dist_dated_items.tax_entity_id = temp_aging_entity_totals.tax_entity_id ";
    if (i_day != int_result_days.begin())
     sql_select_entity_period_totals +=
      "and temp_aging_dist_dated_items.item_date <= '" + prev_date.OutDate(global_mode) + "'";

    this_date = today_date;
    this_date.mod(-(*i_day), in_days);
    prev_date = this_date;

    column_name = "column_" + IntToString(*i_day);
    sql_select_entity_period_totals +=
     "and temp_aging_dist_dated_items.item_date > '" + this_date.OutDate(global_mode) + "') " + column_name + ", ";

    dated_column_order.push_back(column_name);
    if (i_day != int_result_days.begin())
    {
     i_prev_day = i_day;
     i_prev_day--;
     dated_column_descriptions[column_name] = IntToString(*i_prev_day) + " to " + IntToString(*i_day - 1);
    } else
     dated_column_descriptions[column_name] = "... to " + IntToString(*i_day);
   }

   if (int_result_days.size())
   {
    sql_select_entity_period_totals +=
     "(select sum(temp_aging_dist_dated_items.amount) "
     "from temp_aging_dist_dated_items "
     "where temp_aging_dist_dated_items.tax_entity_id = temp_aging_entity_totals.tax_entity_id "
     "and temp_aging_dist_dated_items.item_date <= '" + this_date.OutDate(global_mode) + "') column_inf, ";
    dated_column_order.push_back("column_inf");
    dated_column_descriptions["column_inf"] = IntToString(*(--(int_result_days.end()))) + " to ...";
   }

   if (checkShowItemTotal->Checked)
   {
    sql_select_entity_period_totals +=
     "(select sum(temp_aging_dist_dated_items.amount) "
     "from temp_aging_dist_dated_items "
     "where temp_aging_dist_dated_items.tax_entity_id = temp_aging_entity_totals.tax_entity_id "
     ") column_all, ";
    dated_column_order.push_back("column_all");
    dated_column_descriptions["column_all"] = "Item Total";
   }

   sql_select_entity_period_totals +=
    "temp_aging_entity_totals.tax_entity_id "
    "from temp_aging_entity_totals "
    "inner join tax_entities "
    "on tax_entities.id = temp_aging_entity_totals.tax_entity_id "
    "where temp_aging_entity_totals.account_type_id = " + aging_account_type_id +
    " and temp_aging_entity_totals.amount != 0";

   vector<map<string, string> > cache_period_totals;
   if (!SqlToVecMap(dbTransaction, sql_select_entity_period_totals, cache_period_totals))
   {
    PSay ("Failed to pre-select period totals.", m_database_error, MB_OK);
    DebugSql(this, &sql_select_entity_period_totals, other_forms, 9, dbTransaction);
    throw(9);
   }

   // pull in the per-entity totals
   string sql_select_entity_totals =
    "select * from temp_aging_entity_totals";
   vector<map<string, string> > cache_entity_totals;
   if (!SqlToVecMap(dbTransaction, sql_select_entity_totals, cache_entity_totals))
   {
    PSay ("Failed to pre-select entity totals.", m_database_error, MB_OK);
    DebugSql(this, &sql_select_entity_totals, other_forms, 9, dbTransaction);
    throw(10);
   }

   // put it all into easy-to-retrieve entity_column_values;
   map<string, vector<financial> > entity_column_values;
   map<string, vector<financial> > final_entity_column_values;

   set<string> accounts_to_reverse;
   accounts_to_reverse.insert(lookupTables->IdOf("account_types", "Expense"));
   accounts_to_reverse.insert(lookupTables->IdOf("account_types", "Receivable"));
   accounts_to_reverse.insert(lookupTables->IdOf("account_types", "Cash"));

   unsigned int num_dated_columns = dated_column_order.size();
   unsigned int num_total_columns = account_types_to_total.size();
   unsigned int num_columns = num_dated_columns + num_total_columns;

   vector<financial> empty_vector;
   empty_vector.resize(num_columns);

   vector<financial> totals_vector;
   totals_vector.resize(num_columns);

   vector<string> column_names;

   for (unsigned int i = 0; i < num_dated_columns; i++)
    column_names.push_back(dated_column_descriptions[dated_column_order[i]]);
   for (unsigned int i = 0; i < num_total_columns; i++)
    column_names.push_back(PrettyString(lookupTables->ValueOf("account_types", account_types_to_total[i])));

   map<string, vector<financial> >::iterator i_entity;
   string entity_id;
   string account_type_id;
   unsigned int i_dated_col_num;
   unsigned int i_total_col_num;
   financial multiplier;

   // per-entity absolute totals
   for (vector<map<string, string> >::iterator i_total = cache_entity_totals.begin(); i_total != cache_entity_totals.end(); i_total++)
   {
    entity_id = (*i_total)["tax_entity_id"];
    i_entity = entity_column_values.find(entity_id);
    if (i_entity == entity_column_values.end())
    {
     entity_column_values[entity_id] = empty_vector;
     i_entity = entity_column_values.find(entity_id);
    }
    account_type_id = (*i_total)["account_type_id"];
    if (accounts_to_reverse.find(account_type_id) != accounts_to_reverse.end())
     multiplier = -1.00;
    else
     multiplier = +1.00;
    for (i_total_col_num = 0; i_total_col_num < num_total_columns; i_total_col_num++)
    {
     if (account_types_to_total[i_total_col_num] == account_type_id)
     {
      i_entity->second[i_total_col_num + num_dated_columns] = multiplier * StringToFinancial((*i_total)["amount"]);
      break;
     }
    }
   }

   // switched the two loops so i could do the following just once ... out of laziness. -- philip
   if (accounts_to_reverse.find(aging_account_type_id) != accounts_to_reverse.end())
    multiplier = -1.00;
   else
    multiplier = +1.00;

   // per-entity, per-period totals
   for (vector<map<string, string> >::iterator i_dated = cache_period_totals.begin(); i_dated != cache_period_totals.end(); i_dated++)
   {
    entity_id = (*i_dated)["tax_entity_id"];
    i_entity = entity_column_values.find(entity_id);
    if (i_entity == entity_column_values.end())
    {
     entity_column_values[entity_id] = empty_vector;
     i_entity = entity_column_values.find(entity_id);
    }
    for (i_dated_col_num = 0; i_dated_col_num < num_dated_columns; i_dated_col_num++)
     i_entity->second[i_dated_col_num] = multiplier * StringToFinancial((*i_dated)[dated_column_order[i_dated_col_num]]);
   }

   // now we clean it up based on their left/outer/right/inner choices
   // anything not in this list won't get printed. so there. ha. hahahaha.
   bool has_aging;
   bool has_total;
   unsigned int i_testing_col;
   for (i_entity = entity_column_values.begin(); i_entity != entity_column_values.end(); i_entity++)
   {
    has_aging = false;
    for (i_testing_col = 0; i_testing_col < num_dated_columns; i_testing_col++)
     if (i_entity->second[i_testing_col] != 0.00)
     {
      has_aging = true;
      break;
     }

    has_total = false;
    for (i_testing_col = num_dated_columns; i_testing_col < num_columns; i_testing_col++)
     if (i_entity->second[i_testing_col] != 0.00)
     {
      has_total = true;
      break;
     }

    if ((radioShowByItems->Checked && has_aging) ||
        (radioShowByTotals->Checked && has_total) ||
        (radioShowByEither->Checked && (has_aging || has_total)) ||
        (radioShowByBoth->Checked && has_aging && has_total))
    {
     final_entity_column_values.insert(pair<string, vector<financial> >(i_entity->first, i_entity->second));
     for (i_testing_col = 0; i_testing_col < num_columns; i_testing_col++)
      totals_vector[i_testing_col] += i_entity->second[i_testing_col];
    }
   }

   string sql_select_dated_items =
    "select distinct "
      "temp_aging_entity_totals.tax_entity_id entity_id, "
      "entities.name entity_name, "
      "tin.identifier entity_taxid, "
      "temp_aging_dist_dated_items.bill_id, "
      "temp_aging_dist_dated_items.task_id, "
      "temp_aging_dist_dated_items.item_date, "
      "temp_aging_dist_dated_items.amount, "
      "tasks.services_provided_id, "
      "person_names.last_name || ', ' || person_names.first_name person_name, "
      "bills.invoice_number, "
      "bills.note, "
      "service_types.item service_name, "
      "area_types.item area_name, "
      "serving.name service_provider, "
      "providing.name task_provider, "
      "purchase_reqs.purchase_req_number pr_number, "
      "purchase_reqs.date_sent pr_date, "
      "checks.check_number check_number, "
      "checks.date_sent check_date "
    "from temp_aging_entity_totals "
    "inner join tax_entities entities on temp_aging_entity_totals.tax_entity_id = entities.id "
    "left join entity_ids tin on tin.tax_entity_id = entities.id and tin.current_id >= 3 and tin.entity_id_id = '" + lookupTables->IdOf("entity_id_types", "TIN/EIN") + "' "
    "left join temp_aging_dist_dated_items on temp_aging_entity_totals.tax_entity_id = temp_aging_dist_dated_items.tax_entity_id "
    "left join tasks on temp_aging_dist_dated_items.task_id = tasks.id "
    "left join bills on temp_aging_dist_dated_items.bill_id = bills.id "
    "left join people on tasks.person_id = people.id "
    "left join person_names on people.person_names_id = person_names.id "
    "left join services_provided on tasks.services_provided_id = services_provided.id "
    "left join service_types on services_provided.service_id = service_types.id "
    "left join area_types on services_provided.area_id = area_types.id "
    "left join tax_entities serving on services_provided.tax_entity_id = serving.id "
    "left join tax_entities providing on tasks.tax_entity_id = providing.id "
    "left join purchase_reqs on bills.purchase_req_id = purchase_reqs.id and bills.purchase_req_id != 0 "
    "left join checks on purchase_reqs.check_id = checks.id and purchase_reqs.check_id != 0 "
    "where temp_aging_entity_totals.tax_entity_id != 0 "
    "order by entities.name, temp_aging_entity_totals.tax_entity_id, tasks.services_provided_id, temp_aging_dist_dated_items.item_date, temp_aging_dist_dated_items.amount";

    DebugSql(this, &sql_select_dated_items, other_forms, 9, dbTransaction);

    TIBDataSet * dbDataset = new TIBDataSet(NULL);

    FORM_TRY
     TformReportPendingAccounts * rep = new TformReportPendingAccounts(NULL, column_names, final_entity_column_values, totals_vector, checkShowItems->Checked, (multiplier < 0.00));

     if (!data_sources.empty())
      rep->labelAccountType->Caption = (lookupTables->ValueOf("account_types", aging_account_type_id) + " Amounts Itemized").c_str();

     dbDataset->Database = dbDatabase;
     dbDataset->Transaction = dbTransaction;
     dbDataset->UniDirectional = false;

     dbDataset->SelectSQL->Add(sql_select_dated_items.c_str());
     dbDataset->Open();
     dbDataset->First();
     rep->reportPendingAccounts->DataSet = dbDataset;
     rep->labelUser->Caption = PrettyString(lookupTables->ValueOf("users", current_userid)).c_str();

     rep->bandGroupEntity->Expression = "entity_id";
     rep->bandGroupType->Expression = "services_provided_id";

     rep->textTaxEntityName->DataSet = dbDataset;
     rep->textTaxEntityName->DataField = "entity_name";
     rep->textTaxEntityID->DataSet = dbDataset;
     rep->textTaxEntityID->DataField = "entity_taxid";
     rep->textItemDate->DataSet = dbDataset;
     rep->textItemDate->DataField = "item_date";
     rep->textItemAmount->DataSet = dbDataset;
     rep->textItemAmount->DataField = "amount";
     rep->textPerson->DataSet = dbDataset;
     rep->textPerson->DataField = "person_name";
     rep->textServiceProvider->DataSet = dbDataset;
     rep->textServiceProvider->DataField = "service_provider";
     rep->textTaskProvider->DataSet = dbDataset;
     rep->textTaskProvider->DataField = "task_provider";
     rep->textService->DataSet = dbDataset;
     rep->textService->DataField = "service_name";
     rep->textInvoice->DataSet = dbDataset;
     rep->textInvoice->DataField = "invoice_number";
     rep->textNote->DataSet = dbDataset;
     rep->textNote->DataField = "note";
     rep->textPurchaseReqNumber->DataSet = dbDataset;
     rep->textPurchaseReqNumber->DataField = "pr_number";
     rep->textPurchaseReqDate->DataSet = dbDataset;
     rep->textPurchaseReqDate->DataField = "pr_date";
     rep->textCheckNumber->DataSet = dbDataset;
     rep->textCheckNumber->DataField = "check_number";
     rep->textCheckDate->DataSet = dbDataset;
     rep->textCheckDate->DataField = "check_date";
     rep->textArea->DataSet = dbDataset;
     rep->textArea->DataField = "area_name";

     Processing(false);
     
     rep->reportPendingAccounts->Preview();
    FORM_CATCH

   delete dbDataset;

   ExecuteStatement(dbTransaction, "delete from temp_aging_entity_totals");
   ExecuteStatement(dbTransaction, "delete from temp_aging_transactions");
   ExecuteStatement(dbTransaction, "delete from temp_aging_dated_items");
   ExecuteStatement(dbTransaction, "delete from temp_aging_dist_dated_items");
   dbTransaction->Commit();
  }
  catch (...)
  {
   PSay ("An error has caused the reporting process to be aborted.", m_database_error, MB_OK);
   if (dbTransaction->InTransaction)
    dbTransaction->Rollback();
  }

  if (dbTransaction)
   delete dbTransaction;
 }

 Processing(false);
 return;
}
//---------------------------------------------------------------------------
void __fastcall TformLaunchPendingAccountsReport::buttonCloseClick(
      TObject *Sender)
{
 // close form
 Close();
}
//---------------------------------------------------------------------------
void __fastcall TformLaunchPendingAccountsReport::FormClose(
      TObject *Sender, TCloseAction &Action)
{
 // destroy objects
 Action = caFree;
 TformSpecialGeneric::FormClose(Sender, Action);
 if (Action == caFree)
 {
  lookupTables->UnregisterComboBox("account_types", comboAccountType);
  edit_days.setup_listview(NULL, false);
  edit_sources.setup_listview(NULL, false);
 }
}
//---------------------------------------------------------------------------

void __fastcall TformLaunchPendingAccountsReport::buttonSourcesAddClick(
      TObject *Sender)
{
 // add data source
 listSources->Selected = NULL;        
}
//---------------------------------------------------------------------------

void __fastcall TformLaunchPendingAccountsReport::buttonSourcesRemoveClick(
      TObject *Sender)
{
 // remove data source
 if (listSources->Selected)
  edit_sources.remove_current_row(true);
}
//---------------------------------------------------------------------------

void __fastcall TformLaunchPendingAccountsReport::buttonSourcesClearClick(
      TObject *Sender)
{
 // clear data sources
 edit_sources.clear_data();
}
//---------------------------------------------------------------------------

void __fastcall TformLaunchPendingAccountsReport::comboParentChange(
      TObject *Sender)
{
 // different parent -- switch group boxes! go!
 if (groupVisits->Enabled)
  RecurseEnabled(groupVisits, false);
 if (groupBills->Enabled)
  RecurseEnabled(groupBills, false);
 if (groupCoords->Enabled)
  RecurseEnabled(groupCoords, false);
 if (groupNone->Enabled)
  RecurseEnabled(groupNone, false);

 if (comboParent->Text == "visits")
 {
  groupVisits->BringToFront();
  RecurseEnabled(groupVisits, true);
 }
 else if (comboParent->Text == "bills")
 {
  groupBills->BringToFront();
  RecurseEnabled(groupBills, true);
 }
 else if (comboParent->Text == "coord. of benefits")
 {
  groupCoords->BringToFront();
  RecurseEnabled(groupCoords, true);
 }
 else
 {
  groupNone->BringToFront();
  RecurseEnabled(groupNone, true);
 }
}
//---------------------------------------------------------------------------

void __fastcall TformLaunchPendingAccountsReport::buttonSourcesSaveClick(
      TObject *Sender)
{
 // save data source

 // for any given value in the top drop-down, any combination of [x] makes sense
 // (coord. of benefits [ ] without visits [x] with bills [ ] without purchase_reqs [?] ...
 //  would mean "any cobs that don't have visits that themselves have bills that themselves have no pr's")

 // however, the date selection is trickier -- we can't take the date from something that shouldn't exist,
 // and further more, we can't take it from any of the dependent tables ... therefore, the date selected
 // must either be from the parent object, or from one of its parents (cob over a visit, if visit is the item)
 // however, for parent FK's, we can't take the date unless it's reachable in every case --
 // if you say [?] about a visit being on a cob, we can't use the cob's date. just can't.
 // (no two queries are identical, right? because depending on the parent, you get different transactions,
 // even if you stay within the same circumstances.)

 int index_of_tx_parent = comboParent->ItemIndex;

 /*
 string selected_date = comboDate->Text.c_str();
 int index_of_date_context = comboDate->ItemIndex;
 bool valid_date = false;

 if (comboParent->Text == "visits")
 {
  if (selected_date == "visit date")
   valid_date = true;
  else
  if (selected_date == "coord. of benefits date")
   if (checkVisitsCoords->State == cbChecked)
    valid_date = true;
   else
    PSay ("In order to use the date of the coordination of benefits,\nyou must select the coordination of benefits as a required reference.", m_invalid_data, MB_OK);
  else
   PSay ("You may only use the date of visit or, in some cases,\ndate of the coordination of benefits, when counting transactions on a visit.", m_invalid_data, MB_OK);
 }
 else if (comboParent->Text == "bills")
 {
  if (selected_date == "bill date")
   valid_date = true;
  else
  if (selected_date == "visit date")
   if (checkBillsVisits->State == cbChecked)
    valid_date = true;
   else
    PSay ("In order to use the date of the visit on a bill,\nyou must select the visit as a required reference.", m_invalid_data, MB_OK);
  else
  if (selected_date == "coord. of benefits date")
   if (checkBillsVisits->State == cbChecked && checkBillsCoords->State == cbChecked)
    valid_date = true;
   else
    PSay ("In order to use the date of the coordination of benefits,\nyou must select both the visit and the coordination of benefits as required references.", m_invalid_data, MB_OK);
  else
  if (selected_date == "purchase req. date")
   if (checkBillsPurchaseReqs->State == cbChecked)
    valid_date = true;
   else
    PSay ("In order to use the date of the purchase req.,\nyou must select the purchase req. as a required reference.", m_invalid_data, MB_OK);
  else
  if (selected_date == "check date")
   if (checkBillsPurchaseReqs->State == cbChecked && checkBillsChecks->State == cbChecked)
    valid_date = true;
   else
    PSay ("In order to use the date of the check,\nyou must select both the purchase req. and the check as required references.", m_invalid_data, MB_OK);
  else
   PSay ("You must select the date to use when categorizing transactions attached to these bills.", m_invalid_data, MB_OK);
 }
 else if (comboParent->Text == "coord. of benefits")
 {
  if (selected_date == "coord. of benefits date")
   valid_date = true;
  else
   PSay ("On coordinations of benefits, no other date is available, as there might be multiple dates available.\n(Multiple visits, thus multiple dates, or multiple bills with different dates, etc.)", m_invalid_data, MB_OK);
 }
 else
 {
  // never valid
  PSay ("No parent item has been selected for the transactions to count.", m_invalid_data, MB_OK);
 }

 if (!valid_date)
  return;
 */

 map<string, string> row_data;
 // edit_sources.get_current_row(row_data);

 // just blanking it, just 'cause ...
 row_data["is_coords"]        = "0";
 row_data["is_visits"]        = "0";
 row_data["is_bills"]         = "0";
 row_data["is_purchase_reqs"] = "0";
 row_data["is_checks"]        = "0";

 row_data["tx_parent"]        = IntToString(index_of_tx_parent);
 if (comboParent->Text == "visits")
 {
  row_data["is_coords"]        = NumberByState (checkVisitsCoords->State);
  row_data["is_bills"]         = NumberByState (checkVisitsBills->State);
  row_data["is_purchase_reqs"] = NumberByState (checkVisitsPurchaseReqs->State);
  row_data["is_checks"]        = NumberByState (checkVisitsChecks->State);

  row_data["description"]      = "visits";
  row_data["description"]     += (checkVisitsCoords->State == cbChecked) ? " on a COB" : "";
  row_data["description"]     += (checkVisitsCoords->State == cbUnchecked) ? " not on a COB" : "";
  if (checkVisitsBills->State != cbGrayed || checkVisitsPurchaseReqs->State != cbGrayed || checkVisitsChecks->State != cbGrayed)
   if (row_data["description"]!= "visits")
    row_data["description"]   += ", and";
  row_data["description"]     += (checkVisitsBills->State == cbChecked) ? " w/ bills" : "";
  row_data["description"]     += (checkVisitsBills->State == cbUnchecked) ? " w/o bills" : "";
  row_data["description"]     += (checkVisitsPurchaseReqs->State == cbChecked) ? " on a PR" : "";
  row_data["description"]     += (checkVisitsPurchaseReqs->State == cbUnchecked) ? " not on a PR" : "";
  row_data["description"]     += (checkVisitsChecks->State == cbChecked) ? " on a check" : "";
  row_data["description"]     += (checkVisitsChecks->State == cbUnchecked) ? " not on a check" : "";
 }
 else if (comboParent->Text == "bills")
 {
  row_data["is_coords"]        = NumberByState (checkBillsCoords->State);
  row_data["is_visits"]        = NumberByState (checkBillsVisits->State);
  row_data["is_purchase_reqs"] = NumberByState (checkBillsPurchaseReqs->State);
  row_data["is_checks"]        = NumberByState (checkBillsChecks->State);

  row_data["description"]      = "bills";
  row_data["description"]     += (checkBillsVisits->State == cbChecked) ? " w/ visits" : "";
  row_data["description"]     += (checkBillsVisits->State == cbUnchecked) ? " w/o visits" : "";
  row_data["description"]     += (checkBillsCoords->State == cbChecked) ? " on a COB" : "";
  row_data["description"]     += (checkBillsCoords->State == cbUnchecked) ? " not on a COB" : "";
  if (checkBillsPurchaseReqs->State != cbGrayed || checkBillsChecks->State != cbGrayed)
   if (row_data["description"]!= "bills")
    row_data["description"]   += ", and";
  row_data["description"]     += (checkBillsPurchaseReqs->State == cbChecked) ? " on a PR" : "";
  row_data["description"]     += (checkBillsPurchaseReqs->State == cbUnchecked) ? " not on a PR" : "";
  row_data["description"]     += (checkBillsChecks->State == cbChecked) ? " on a check" : "";
  row_data["description"]     += (checkBillsChecks->State == cbUnchecked) ? " not on a check" : "";
 }
 else if (comboParent->Text == "coord. of benefits")
 {
  row_data["is_visits"]        = NumberByState (checkCoordsVisits->State);
  row_data["is_bills"]         = NumberByState (checkCoordsBills->State);
  row_data["is_purchase_reqs"] = NumberByState (checkCoordsPurchaseReqs->State);
  row_data["is_checks"]        = NumberByState (checkCoordsChecks->State);

  row_data["description"]      = "COBs";
  row_data["description"]     += (checkCoordsVisits->State == cbChecked) ? " w/ visits" : "";
  row_data["description"]     += (checkCoordsVisits->State == cbUnchecked) ? " w/o visits" : "";
  row_data["description"]     += (checkCoordsBills->State == cbChecked) ? " w/ bills" : "";
  row_data["description"]     += (checkCoordsBills->State == cbUnchecked) ? " w/o bills" : "";
  row_data["description"]     += (checkCoordsPurchaseReqs->State == cbChecked) ? " on a PR" : "";
  row_data["description"]     += (checkCoordsPurchaseReqs->State == cbUnchecked) ? " not on a PR" : "";
  row_data["description"]     += (checkCoordsChecks->State == cbChecked) ? " on a check" : "";
  row_data["description"]     += (checkCoordsChecks->State == cbUnchecked) ? " not on a check" : "";
 }
 else
 {
  // shouldn't happen, but just a gentle, useless reminder here, staring back at you.
 }

 // row_data["description"]     += string(", using ") + comboDate->Text.c_str();
 // row_data["date_context"]     = IntToString(index_of_date_context);

 set<string> uniques;
 SplitStringToSet("description", ",", uniques);

 if (!edit_sources.is_future_duplicate(row_data, uniques))
 {
  edit_sources.add_or_modify_row_data(row_data, true);
  buttonSourcesAdd->Click();
 } else
  PSay ("A data source / rule already exists which does exactly this.", m_duplicate_data, MB_OK);
}
//---------------------------------------------------------------------------
void TformLaunchPendingAccountsReport::DataSourceSelected ()
{
 // fill in the edit box
 map<string, string> row_data;
 if (!edit_sources.get_current_row(row_data))
 {
  row_data["tx_parent"]        = "0"; // item index in list
  row_data["is_coords"]        = "0"; // -1, 0, 1 ...
  row_data["is_visits"]        = "0";
  row_data["is_bills"]         = "0";
  row_data["is_purchase_reqs"] = "0";
  row_data["is_checks"]        = "0";
  // row_data["date_context"]     = "0"; // item index in list
 }

 // top combo box
 int index_of_tx_parent = StringToInt(row_data["tx_parent"]);
 // was checking for < 0, but not necessary if using number from list
 comboParent->ItemIndex = index_of_tx_parent;

 comboParentChange(NULL);

 // group boxes of doom
 if (comboParent->Text == "visits")
 {
  checkVisitsCoords->State = StateByNumber(row_data["is_coords"]);
  checkVisitsBills->State = StateByNumber(row_data["is_bills"]);
  checkVisitsPurchaseReqs->State = StateByNumber(row_data["is_purchase_reqs"]);
  checkVisitsChecks->State = StateByNumber(row_data["is_checks"]);
 }
 else if (comboParent->Text == "bills")
 {
  checkBillsCoords->State = StateByNumber(row_data["is_coords"]);
  checkBillsVisits->State = StateByNumber(row_data["is_visits"]);
  checkBillsPurchaseReqs->State = StateByNumber(row_data["is_purchase_reqs"]);
  checkBillsChecks->State = StateByNumber(row_data["is_checks"]);
 }
 else if (comboParent->Text == "coord. of benefits")
 {
  checkCoordsBills->State = StateByNumber(row_data["is_bills"]);
  checkCoordsVisits->State = StateByNumber(row_data["is_visits"]);
  checkCoordsPurchaseReqs->State = StateByNumber(row_data["is_purchase_reqs"]);
  checkCoordsChecks->State = StateByNumber(row_data["is_checks"]);
 }
 else
 {
  // empty page, nothing to do.
 }

 /*
 // bottom combo box
 int index_of_date_context = StringToInt(row_data["date_context"]);
 // was checking for < 0, but not necessary if using number from list
 comboDate->ItemIndex = index_of_date_context;
 */
}
//---------------------------------------------------------------------------
TCheckBoxState TformLaunchPendingAccountsReport::StateByNumber (string c_num)
{
 if (c_num == "-1")
  return cbUnchecked;
 else if (c_num == "1")
  return cbChecked;
 else
  return cbGrayed;
}
//---------------------------------------------------------------------------
string TformLaunchPendingAccountsReport::NumberByState (TCheckBoxState c_state)
{
 if (c_state == cbUnchecked)
  return "-1";
 else if (c_state == cbChecked)
  return "1";
 else
  return "0";
}
//---------------------------------------------------------------------------

string RecursiveTableJoinSqlGenerator (map<string, recursive_data_source> & source_tables, vector<recursive_data_source_link> & source_joins, string this_source, bool is_root_node)
{
 // placeholder_where and placeholder_from
 string answer;

 if (source_tables[this_source].is_visited)
  return answer;
 else
  source_tables[this_source].is_visited = true;

 if (!is_root_node)
 {
  // add the first clause based on our current status
  if (source_tables[this_source].exists_value == "1")
   answer += " and exists(";
  else
  if (source_tables[this_source].exists_value == "-1")
   answer += " and not exists(";
 }

 // if we're not pass-through, start selecting stuff ... otherwise, we'll catch it on the way back.
 if (source_tables[this_source].exists_value != "0")
 {
  answer += "select " + source_tables[this_source].wanted_fields +
            " from " + source_tables[this_source].table_and_alias +
            " " + placeholder_from + " ";
  answer += "where (" + source_tables[this_source].where_clause + ") " + placeholder_where;
 }

 // for every unvisited direct relative, add a clause for that branch, recursively.
 // (this will either add the clauses to the above select statement, or to the overall select statement we can't see

 string temp_answer;
 for (vector<recursive_data_source_link>::iterator i_link = source_joins.begin(); i_link != source_joins.end(); i_link++)
 {
  temp_answer = "";
  // if it's a useful, unused source ... go for it.
  if (i_link->a_source_name == this_source)
  {
   if (!source_tables[i_link->b_source_name].is_visited)
    temp_answer = RecursiveTableJoinSqlGenerator (source_tables, source_joins, i_link->b_source_name, false);
  } else
  if (i_link->b_source_name == this_source)
  {
   if (!source_tables[i_link->a_source_name].is_visited)
    temp_answer = RecursiveTableJoinSqlGenerator (source_tables, source_joins, i_link->a_source_name, false);
  }
  // if we got something, while we're still on the link, finish cleaning up the stuff ...
  if (source_tables[this_source].exists_value == "0")
  {
   temp_answer = ReplaceStringWithString(temp_answer, placeholder_from, ", " + source_tables[this_source].table_and_alias + placeholder_from);
   temp_answer = ReplaceStringWithString(temp_answer, placeholder_where, " and (" + i_link->join_clause + ") and (" + source_tables[this_source].where_clause + ")" + placeholder_where);
  } else
  {
   temp_answer = ReplaceStringWithString(temp_answer, placeholder_from, "");
   temp_answer = ReplaceStringWithString(temp_answer, placeholder_where, " and (" + i_link->join_clause + ") ");
  }
  answer += temp_answer;
 }

 // add the matching ')' based on our current status
 if (!is_root_node)
 {
  if (source_tables[this_source].exists_value != "0")
   answer += ") ";
 }
 else
 {
  answer = ReplaceStringWithString(answer, placeholder_from, "");
  answer = ReplaceStringWithString(answer, placeholder_where, "");
 }

 // ViewSql("(current level is: " + this_source + ") " + answer);
 return answer;
}

//---------------------------------------------------------------------------



