//---------------------------------------------------------------------------

#include <vcl.h>
#pragma hdrstop

#include "launchPersonProviderInfoReport.h"
#include "codeSearchPerson.h"
#include "codeSearchTaxEntity.h"
//---------------------------------------------------------------------------
#pragma package(smart_init)
#pragma link "codeSpecialGeneric"
#pragma resource "*.dfm"
TformLaunchPersonProviderInfoReport *formLaunchPersonProviderInfoReport;
//---------------------------------------------------------------------------
__fastcall TformLaunchPersonProviderInfoReport::TformLaunchPersonProviderInfoReport(TComponent* Owner, string patient_id, string referrer_id, string provider_id)
        : TformSpecialGeneric(Owner)
{
 /* TODO: default to AST insurance if no other insurance is being printed --
          try to not use MAGIC numbers? */
 // TODO -c MAGIC : the above might happen. not currently true.
 /* TODO -c HACK : you'll see "x + 1 > 3" rather than "x > 2 " in the SQL.
    this is to force the optimizer to ignore the FK index on current_id, as it's not usually useful/important. */

 person_id          = patient_id;
 tax_entity_from_id = referrer_id;
 tax_entity_to_id   = provider_id;

 CreateLink(staticPerson, buttonSearchPerson, this, person_id, "people");
 CreateLink(staticTaxEntityFrom, buttonSearchTaxEntityFrom, this, tax_entity_from_id, "tax_entities");
 CreateLink(staticTaxEntityTo, buttonSearchTaxEntityTo, this, tax_entity_to_id, "tax_entities", "", false);
}
//---------------------------------------------------------------------------
void __fastcall TformLaunchPersonProviderInfoReport::buttonRunClick(TObject *Sender)
{
 if (person_id == "0" || person_id == "")
 {
  PSay("Please enter a patient name", m_missing_data, MB_OK);
  buttonSearchPerson->SetFocus();
 } else
 {
  Processing(true);

  dbTransaction  = new TIBTransaction(NULL);

  TQRPageList * source       = new TQRPageList();
  TQRPageList * destination  = new TQRPageList();
  destination->Stream        = new TQRStream(100000);
  TQRPrinter * printer_thing = InitTQRPrinter();

  try
  {
   // prepare composite report
   compositeReport->Prepare();
   string temp_name = PMakeTempFilePath("temporary.qrp");
   ((TQuickRep *)(compositeReport->Reports->First()))->QRPrinter->Save(temp_name.c_str());
   // add page 1
   source->Clear();
   source->LoadFromFile(temp_name.c_str());
   for (int i = 1; i <= source->PageCount; i++)
     destination->AddPage(source->GetPage(i));

   // combine pages
   destination->Finish();
   string temp_filepath = PMakeTempFilePath("complete.qrp");
   destination->SaveToFile(temp_filepath.c_str());
   // load combined report and preview
   printer_thing->Load(temp_filepath.c_str());

   // preview composite report
   Processing(false);
   printer_thing->PreviewModal();

   // close (hopefully deleting) forms used
   for (list<TformReportPersonProviderInfo *>::iterator i_form_used = report_forms.begin(); i_form_used != report_forms.end(); i_form_used++)
    if (!(*i_form_used)->CloseNow())
     PSay("Unable to close a form used to prepare this report (TformReportPersonProviderInfo)", m_program_error, MB_OK);
   report_forms.clear();

   // close and delete datasets
   for (list<TIBDataSet *>::iterator i_dataset_used = report_datasets.begin(); i_dataset_used != report_datasets.end(); i_dataset_used++)
   {
    (*i_dataset_used)->Close();
    delete (*i_dataset_used);
   }
   report_datasets.clear();

   // close transaction
   dbTransaction->Commit();
  }
  catch(...)
  {
   Processing(false);
   PSay("Error running report", m_database_error, MB_OK);
  }

  // transaction
  delete dbTransaction;

  // composite reports bits
  delete printer_thing;
  delete source;
  delete destination;
 }
}
//---------------------------------------------------------------------------
void __fastcall TformLaunchPersonProviderInfoReport::buttonCloseClick(TObject *Sender)
{
 Close();
}
//---------------------------------------------------------------------------
void __fastcall TformLaunchPersonProviderInfoReport::buttonSearchPersonClick(TObject *Sender)
{
 vector<string> return_vector;
 FORM_TRY
  TformSearchPerson * s = new TformSearchPerson(NULL,&return_vector);
  s->ShowTaskModal(this);
 FORM_CATCH
 if (!return_vector.empty())
 {
  person_id = return_vector[0];
  CreateLink(staticPerson, buttonSearchPerson, this, person_id, "people");
 }
}
//---------------------------------------------------------------------------
void __fastcall TformLaunchPersonProviderInfoReport::buttonSearchTaxEntityFromClick(TObject *Sender)
{
 vector<string> return_vector;
 FORM_TRY
  TformSearchTaxEntity * s = new TformSearchTaxEntity(NULL,&return_vector);
  s->ShowTaskModal(this);
 FORM_CATCH
 if (!return_vector.empty())
 {
  tax_entity_from_id = return_vector[0];
  CreateLink(staticTaxEntityFrom, buttonSearchTaxEntityFrom, this, tax_entity_from_id, "tax_entities");
 }
}
//---------------------------------------------------------------------------
void __fastcall TformLaunchPersonProviderInfoReport::buttonSearchTaxEntityToClick(TObject *Sender)
{
 vector<string> return_vector;
 FORM_TRY
  TformSearchTaxEntity * s = new TformSearchTaxEntity(NULL,&return_vector);
  s->ShowTaskModal(this);
 FORM_CATCH
 if (!return_vector.empty())
 {
  tax_entity_to_id = return_vector[0];
  CreateLink(staticTaxEntityTo, buttonSearchTaxEntityTo, this, tax_entity_to_id, "tax_entities", "", false);
 }
}
//---------------------------------------------------------------------------
void TformLaunchPersonProviderInfoReport::StartNewForm()
{
 TformReportPersonProviderInfo * report_form;
 FORM_TRY
  report_form = new TformReportPersonProviderInfo (NULL);
 FORM_CATCH
 report_forms.push_back(report_form);
 return;
}
//---------------------------------------------------------------------------
void TformLaunchPersonProviderInfoReport::StartNewDataset()
{
 TIBDataSet * report_dataset;
 report_dataset = new TIBDataSet (NULL);
 report_dataset->Database       = dbDatabase;
 report_dataset->Transaction    = dbTransaction;
 report_dataset->UniDirectional = false;
 report_datasets.push_back(report_dataset);
 return;
}
//---------------------------------------------------------------------------
void TformLaunchPersonProviderInfoReport::SetupTitle(TformReportPersonProviderInfo * report_form, TIBDataSet * report_dataset)
{
  report_form->reportTitle->DataSet   = report_dataset;
  report_form->qrpSSNField->DataSet   = report_dataset;
  report_form->qrpDOBField->DataSet   = report_dataset;
  report_form->qrpCDIBField->DataSet  = report_dataset;
  report_form->qrpTribeField->DataSet = report_dataset;
  report_form->qrpSSNField->DataField   = "social_security_number";
  report_form->qrpDOBField->DataField   = "birth_date";
  report_form->qrpCDIBField->DataField  = "cdib";
  report_form->qrpTribeField->DataField = "item";
}
//---------------------------------------------------------------------------
void TformLaunchPersonProviderInfoReport::SetupContact(TformReportPersonProviderInfo * report_form, TIBDataSet * report_dataset)
{
  report_form->reportContact->DataSet = report_dataset;
  report_form->qrpRelName->DataSet    = report_dataset;
  report_form->qrpRelAddr1->DataSet   = report_dataset;
  report_form->qrpRelAddr2->DataSet   = report_dataset;
  report_form->qrpRelCity->DataSet    = report_dataset;
  report_form->qrpRelState->DataSet   = report_dataset;
  report_form->qrpRelZip->DataSet     = report_dataset;
  report_form->qrpRelCountry->DataSet = report_dataset;
  report_form->qrpRelPhone->DataSet   = report_dataset;
  report_form->qrpRelFax->DataSet     = report_dataset;
  report_form->qrpRelEmail->DataSet   = report_dataset;
  report_form->qrpRelName->DataField    = "name";
  report_form->qrpRelAddr1->DataField   = "line1";
  report_form->qrpRelAddr2->DataField   = "line2";
  report_form->qrpRelCity->DataField    = "city";
  report_form->qrpRelState->DataField   = "state_name";
  report_form->qrpRelZip->DataField     = "zip";
  report_form->qrpRelCountry->DataField = "country_name";
  report_form->qrpRelPhone->DataField   = "phone_number";
  report_form->qrpRelFax->DataField     = "fax_number";
  report_form->qrpRelEmail->DataField   = "email_address";
}
//---------------------------------------------------------------------------
void TformLaunchPersonProviderInfoReport::SetupInsurance(TformReportPersonProviderInfo * report_form, TIBDataSet * report_dataset)
{
  report_form->reportInsurance->DataSet        = report_dataset;
  report_form->qrpInsCompany->DataSet          = report_dataset;
//  report_form->qrpInsCont->DataSet             = report_dataset;
//  report_form->qrpInsBin->DataSet              = report_dataset;
  report_form->qrpInsPolicyHolderName->DataSet = report_dataset;
  report_form->qrpInsStartDate->DataSet        = report_dataset;
  report_form->qrpInsEndDate->DataSet          = report_dataset;
  report_form->qrpInsGroupNum->DataSet         = report_dataset;
  report_form->qrpInsPolicyNum->DataSet        = report_dataset;
  report_form->qrpInsPersonCode->DataSet       = report_dataset;
  report_form->qrpInsDoctor->DataSet           = report_dataset;
  report_form->qrpInsCompany->DataField          = "ins_name";
//  report_form->qrpInsCont->DataField             = "processor_control_number";
//  report_form->qrpInsBin->DataField              = "bin_number";
  report_form->qrpInsPolicyHolderName->DataField = "name_on_policy";
  report_form->qrpInsStartDate->DataField        = "start_date";
  report_form->qrpInsEndDate->DataField          = "stop_date";
  report_form->qrpInsGroupNum->DataField         = "group_number";
  report_form->qrpInsPolicyNum->DataField        = "policy_number";
  report_form->qrpInsPersonCode->DataField       = "person_code";
  report_form->qrpInsDoctor->DataField           = "pcp_name";
  if (report_dataset->Eof)
   report_form->bandInsuranceDetail->Height = 0;
}
//---------------------------------------------------------------------------
void TformLaunchPersonProviderInfoReport::SetupBlanks(TformReportPersonProviderInfo * report_form, TIBDataSet * report_dataset)
{
  report_form->reportBlankInfo->DataSet = report_dataset;
  report_form->qrpItem->DataSet         = report_dataset;
  report_form->qrpItem->DataField = "thing";
  if (report_dataset->Eof)
   report_form->bandBlankDetail->Height = 0;
}
//---------------------------------------------------------------------------
void __fastcall TformLaunchPersonProviderInfoReport::compositeReportAddReports(TObject *Sender)
{
 try
 {
  // setup the transaction
  dbTransaction->DefaultDatabase = dbDatabase;
  dbTransaction->StartTransaction();

  TformReportPersonProviderInfo * report_form;
  TIBDataSet * report_dataset;
  string sql;

  // title, patient identifying information
  StartNewForm();
  report_form = report_forms.front();
  StartNewDataset();
  report_dataset = report_datasets.back();

  sql = "select people.social_security_number, people.birth_date, affiliations.cdib, tribe_types.item "
        "from people "
        "left join affiliations on affiliations.id = people.affiliations_id "
        " and affiliations.current_id + 1 > 3 "
        "left join tribe_types on affiliations.tribe_id = tribe_types.id "
        "where people.id = " + person_id;
  report_dataset->SelectSQL->Add(sql.c_str());
  report_dataset->Open();
  report_dataset->First();
  SetupTitle(report_form, report_dataset);
  compositeReport->Reports->Add(report_form->reportTitle);

  // patient contact information
  StartNewDataset();
  report_dataset = report_datasets.back();
  report_form = report_forms.front();

  sql = "select person_names.prefix || ' ' || person_names.first_name || ' ' || person_names.middle_name || ' ' || person_names.last_name || ' ' || person_names.suffix name, "
        "person_contact_info.line1, person_contact_info.line2, person_contact_info.city, state_types.item state_name, "
        "person_contact_info.zip, country_types.item country_name, "
        "person_contact_info.phone_number, '' fax_number, person_contact_info.email_address "
        "from people "
        "left join person_names on person_names.id = people.person_names_id "
        " and person_names.current_id + 1 > 3 "
        "left join person_contact_info on person_contact_info.id = people.person_contact_info_id "
        " and person_contact_info.current_id + 1 > 3 "
        "left join state_types on state_types.id = person_contact_info.state_id "
        "left join country_types on country_types.id = person_contact_info.country_id "
        "where people.id = " + person_id;
  report_dataset->SelectSQL->Add(sql.c_str());
  report_dataset->Open();
  report_dataset->First();
  report_form->bandContactTitle->Height = 0; // don't show?
  // report_form->qrpContactTitle->Caption = " Patient contact"; // doesn't matter, if invisible (above)
  SetupContact(report_form, report_dataset);
  compositeReport->Reports->Add(report_form->reportContact);

  // employer contact information
  set<string> employers;
  sql = "select relationships.person_a_id thing from relationships where person_b_id = " + person_id + " and (relationships.rel_a_to_b_id = " + lookupTables->IdOf("relationship_types", "Employer") +
        " or relationships.rel_b_to_a_id = " + lookupTables->IdOf("relationship_types", "Employee") + ") "
        " and relationships.current_id + 1 > 3 "
        " union "
        "select relationships.person_b_id thing from relationships where person_a_id = " + person_id + " and (relationships.rel_a_to_b_id = " + lookupTables->IdOf("relationship_types", "Employee") +
        " or relationships.rel_b_to_a_id = " + lookupTables->IdOf("relationship_types", "Employer") + ") "
        " and relationships.current_id + 1 > 3 ";
  if (!SqlToSet(dbTransaction, sql, employers))
  {
   PSay("Failed to retrieve list of employers.", m_database_error, MB_OK);
   return;
  }

  if (!employers.empty())
  {
   StartNewDataset();
   report_dataset = report_datasets.back();
   StartNewForm();
   report_form = report_forms.back();

   sql = "select person_names.prefix || ' ' || person_names.first_name || ' ' || person_names.middle_name || ' ' || person_names.last_name || ' ' || person_names.suffix name, "
         "person_contact_info.line1, person_contact_info.line2, person_contact_info.city, state_types.item state_name, "
         "person_contact_info.zip, country_types.item country_name, "
         "person_contact_info.phone_number, '' fax_number, person_contact_info.email_address "
         "from people "
         "left join person_names on person_names.id = people.person_names_id "
         " and person_names.current_id + 1 > 3 "
         "left join person_contact_info on person_contact_info.id = people.person_contact_info_id "
         " and person_contact_info.current_id + 1 > 3 "
         "left join state_types on state_types.id = person_contact_info.state_id "
         "left join country_types on country_types.id = person_contact_info.country_id "
         "where people.id in (" + SetToCommaString(employers) + ")";
   report_dataset->SelectSQL->Add(sql.c_str());
   report_dataset->Open();
   report_dataset->First();
   report_form->qrpContactTitle->Caption = " Employer contact(s)";
   SetupContact(report_form, report_dataset);
   compositeReport->Reports->Add(report_form->reportContact);
  }

  // emergency contact information
  set<string> emergencies;
  sql = "select relationships.person_a_id thing from relationships where person_b_id = " + person_id + " and a_is_emergency = 1 "
        " and relationships.current_id + 1 > 3 "
        " union "
        "select relationships.person_b_id thing from relationships where person_a_id = " + person_id + " and b_is_emergency = 1 "
        " and relationships.current_id + 1 > 3 ";
  if (!SqlToSet(dbTransaction, sql, emergencies))
  {
   PSay("Failed to retrieve list of emergency contacts.", m_database_error, MB_OK);
   return;
  }

  if (!emergencies.empty())
  {
   StartNewDataset();
   report_dataset = report_datasets.back();
   StartNewForm();
   report_form = report_forms.back();

   sql = "select person_names.prefix || ' ' || person_names.first_name || ' ' || person_names.middle_name || ' ' || person_names.last_name || ' ' || person_names.suffix name, "
         "person_contact_info.line1, person_contact_info.line2, person_contact_info.city, state_types.item state_name, "
         "person_contact_info.zip, country_types.item country_name, "
         "person_contact_info.phone_number, '' fax_number, person_contact_info.email_address "
         "from people "
         "left join person_names on person_names.id = people.person_names_id "
         " and person_names.current_id + 1 > 3 "
         "left join person_contact_info on person_contact_info.id = people.person_contact_info_id "
         " and person_contact_info.current_id + 1 > 3 "
         "left join state_types on state_types.id = person_contact_info.state_id "
         "left join country_types on country_types.id = person_contact_info.country_id "
         "where people.id in (" + SetToCommaString(emergencies) + ")";
   report_dataset->SelectSQL->Add(sql.c_str());
   report_dataset->Open();
   report_dataset->First();
   report_form->qrpContactTitle->Caption = " Emergency contact(s)";
   SetupContact(report_form, report_dataset);
   compositeReport->Reports->Add(report_form->reportContact);
  }

  // insurance
  StartNewDataset();
  report_dataset = report_datasets.back();
  report_form = report_forms.front();

  sql = "select ins_co.name ins_name, insurance_policies.processor_control_number, insurance_policies.bin_number, "
        "insurance_policies.name_on_policy, insurance_policies.start_date, insurance_policies.stop_date, "
        "insurance_policies.group_number, insurance_policies.policy_number, "
        "insurance_coverage.person_code, pcp.name pcp_name "
        "from insurance_coverage "
        "inner join insurance_policies on insurance_policies.id = insurance_coverage.insurance_policy_id "
        "left join tax_entities ins_co on ins_co.id = insurance_policies.tax_entity_id "
        "left join tax_entities pcp on pcp.id = insurance_coverage.tax_entity_id "
        "where insurance_coverage.person_id = " + person_id +
        " and insurance_coverage.current_id + 1 > 3 "
        " and insurance_policies.current_id + 1 > 3 "
        " and (insurance_policies.stop_date is null or insurance_policies.stop_date >= current_date) "
        " and (insurance_policies.start_date is null or insurance_policies.start_date <= current_date) ";
  report_dataset->SelectSQL->Add(sql.c_str());
  report_dataset->Open();
  report_dataset->First();
  if (report_dataset->Eof)
  {
   // okay, no normal insurance listed.
   // instead, we try to get something resembling what we'd have if the Tribe were listed as insurance
   sql = "select ins_co.name ins_name, '' processor_control_number, '' bin_number, "
         "'' name_on_policy, NULL start_date, NULL stop_date, "
         "'' group_number, '' policy_number, "
         "'' person_code, '' pcp_name "
         "from tax_entities ins_co "
         "where ins_co.id = 5"; // TODO -c MAGIC : shit. damn it. argh.
                                // tax entity #5 (just looked) is "Absentee Shawnee Contract Health"
   report_dataset->Close();
   report_dataset->SelectSQL->Clear();
   report_dataset->SelectSQL->Add(sql.c_str());
   report_dataset->Open();
   report_dataset->First();
  }
  SetupInsurance(report_form, report_dataset);
  compositeReport->Reports->Add(report_form->reportInsurance);

  // allergies
  StartNewDataset();
  report_dataset = report_datasets.back();
  report_form = report_forms.front();

  sql = "select distinct note thing from conditions inner join condition_types on condition_types.id = conditions.condition_id where condition_types.item like '%ALLERGY%' and conditions.person_id = " + person_id + " and conditions.current_id + 1 > 3";
  report_dataset->SelectSQL->Add(sql.c_str());
  report_dataset->Open();
  report_dataset->First();
  report_form->qrpInfoTitle->Caption = " Allergies";
  SetupBlanks(report_form, report_dataset);
  compositeReport->Reports->Add(report_form->reportBlankInfo);

  // diagnosis
  StartNewDataset();
  report_dataset = report_datasets.back();
  StartNewForm();
  report_form = report_forms.back();

  sql = "select * from uno where 0 = 1";
  report_dataset->SelectSQL->Add(sql.c_str());
  report_dataset->Open();
  report_dataset->First();
  report_form->qrpInfoTitle->Caption = " Diagnosis";
  SetupBlanks(report_form, report_dataset);
  compositeReport->Reports->Add(report_form->reportBlankInfo);

  // treatment
  StartNewDataset();
  report_dataset = report_datasets.back();
  StartNewForm();
  report_form = report_forms.back();

  sql = "select * from uno where 0 = 1";
  report_dataset->SelectSQL->Add(sql.c_str());
  report_dataset->Open();
  report_dataset->First();
  report_form->qrpInfoTitle->Caption = " Treatment requested";
  SetupBlanks(report_form, report_dataset);
  compositeReport->Reports->Add(report_form->reportBlankInfo);

  // medications
  StartNewDataset();
  report_dataset = report_datasets.back();
  StartNewForm();
  report_form = report_forms.back();

  sql = "select * from uno where 0 = 1";
  report_dataset->SelectSQL->Add(sql.c_str());
  report_dataset->Open();
  report_dataset->First();
  report_form->qrpInfoTitle->Caption = " Medications";
  SetupBlanks(report_form, report_dataset);
  compositeReport->Reports->Add(report_form->reportBlankInfo);

  // history
  StartNewDataset();
  report_dataset = report_datasets.back();
  StartNewForm();
  report_form = report_forms.back();

  sql = "select * from uno where 0 = 1";
  report_dataset->SelectSQL->Add(sql.c_str());
  report_dataset->Open();
  report_dataset->First();
  report_form->qrpInfoTitle->Caption = " Significant history";
  SetupBlanks(report_form, report_dataset);
  compositeReport->Reports->Add(report_form->reportBlankInfo);

  // requesting physician info
  if (tax_entity_from_id != "" && tax_entity_from_id != "0")
  {
   StartNewDataset();
   report_dataset = report_datasets.back();
   StartNewForm();
   report_form = report_forms.back();

   sql = "select tax_entities.name, tax_addresses.line1, tax_addresses.line2, tax_addresses.city, state_types.item state_name, "
         "tax_addresses.zip, country_types.item country_name, tax_addresses.phone_number, tax_addresses.fax_number, tax_addresses.email_address "
         "from tax_entities "
         "left join tax_addresses on tax_addresses.id = tax_entities.physical_address_id "
         " and tax_addresses.current_id + 1 > 3 "
         "left join state_types on state_types.id = tax_addresses.state_id "
         "left join country_types on country_types.id = tax_addresses.country_id "
         "where tax_entities.id = " + tax_entity_from_id;
   report_dataset->SelectSQL->Add(sql.c_str());
   report_dataset->Open();
   report_dataset->First();
   report_form->qrpContactTitle->Caption = " Requesting physician";
   SetupContact(report_form, report_dataset);
   compositeReport->Reports->Add(report_form->reportContact);
  } else
  {
   StartNewDataset();
   report_dataset = report_datasets.back();
   StartNewForm();
   report_form = report_forms.back();

   sql = "select * from uno where 0 = 1";
   report_dataset->SelectSQL->Add(sql.c_str());
   report_dataset->Open();
   report_dataset->First();
   report_form->qrpInfoTitle->Caption = " Requesting physician";
   SetupBlanks(report_form, report_dataset);
   compositeReport->Reports->Add(report_form->reportBlankInfo);
  }
  // end of report sections
 }
 catch (...)
 {
  PSay("Error running report", m_database_error, MB_OK);
  if (dbTransaction->Active)
   dbTransaction->Rollback();
 }
}
//---------------------------------------------------------------------------
void __fastcall TformLaunchPersonProviderInfoReport::compositeReportFinished(TObject *Sender)
{
 // not necessary?
}
//---------------------------------------------------------------------------
