//---------------------------------------------------------------------------
#include <vcl.h>
#pragma hdrstop

#include "launchPrintCobReport.h"
#include "cached_files.h"
//---------------------------------------------------------------------------
#pragma package(smart_init)
#pragma link "codeSpecialGeneric"
#pragma resource "*.dfm"
TformLaunchPrintCobReport *formLaunchPrintCobReport;
//---------------------------------------------------------------------------
__fastcall TformLaunchPrintCobReport::TformLaunchPrintCobReport(TComponent* Owner, string p_id)
        : TformSpecialGeneric(Owner)
{
  id = p_id;
  CreateLink(txtCob, NULL, this, id, "tasks");
}
//---------------------------------------------------------------------------
void __fastcall TformLaunchPrintCobReport::btnCloseClick(TObject *Sender)
{
  Close();
}
//---------------------------------------------------------------------------
void __fastcall TformLaunchPrintCobReport::btnRunClick(TObject *Sender)
{
  FORM_TRY
    report_form = new TformReportPrintCob(NULL);
  FORM_CATCH_RETURN("TformReportPrintCob", )

  Processing(true);

  dbTransaction = new TIBTransaction(NULL);
  dbApptDataSet = new TIBDataSet(NULL);
  dbTransDataSet = new TIBDataSet(NULL);
  dbReferralDataSet = new TIBDataSet(NULL);
  dbInsuranceDataSet = new TIBDataSet(NULL);
  dbLetterDataSet = new TIBDataSet(NULL);
  TQRPageList * source = new TQRPageList();
  TQRPageList * destination = new TQRPageList();
  destination->Stream = new TQRStream(100000);
  TQRPrinter * printer_thing = InitTQRPrinter();

  try
  {
    dbTransaction->DefaultDatabase = dbDatabase;
    dbApptDataSet->Database = dbDatabase;
    dbApptDataSet->Transaction = dbTransaction;
    dbApptDataSet->UniDirectional = false;
    dbTransDataSet->Database = dbDatabase;
    dbTransDataSet->Transaction = dbTransaction;
    dbTransDataSet->UniDirectional = false;
    dbReferralDataSet->Database = dbDatabase;
    dbReferralDataSet->Transaction = dbTransaction;
    dbReferralDataSet->UniDirectional = false;
    dbInsuranceDataSet->Database = dbDatabase;
    dbInsuranceDataSet->Transaction = dbTransaction;
    dbInsuranceDataSet->UniDirectional = false;
    dbLetterDataSet->Transaction = dbTransaction;
    dbLetterDataSet->UniDirectional = false;

    dbTransaction->StartTransaction();

    // jhaley 20120229: get the AST logo graphic from the file cache.
    FileCache::GetJPEGFromCache(report_form->QRImage1->Picture, dbDatabase, dbTransaction,
                                "Prometheus\\images", "ast_cob_logo.jpg");
    report_form->QRImage1->Width  = 73;
    report_form->QRImage1->Height = 73;

    compositeReport->Prepare();

    string file_name = PMakeTempFilePath("page1.qrp");
    ((TQuickRep *)(compositeReport->Reports->First()))->QRPrinter->Save(file_name.c_str());
    source->Clear();
    source->LoadFromFile(file_name.c_str());
    for (int i = 1; i <= source->PageCount; i++)
      destination->AddPage(source->GetPage(i));
    destination->Finish();
    string temp_filepath = PMakeTempFilePath("complete.qrp");
    destination->SaveToFile(temp_filepath.c_str());
    // load combined report and preview
    printer_thing->Load(temp_filepath.c_str());
    Processing(false);
    printer_thing->PreviewModal();

    dbApptDataSet->Close();
    dbTransDataSet->Close();
    dbReferralDataSet->Close();
    dbInsuranceDataSet->Close();
    dbLetterDataSet->Close();

    dbTransaction->Commit();
    report_form->CloseNow();
  }
  catch (...)
  {
    Processing(false);
    PSay("Error running report", m_database_error, MB_OK);
    if (dbTransaction->Active)
      dbTransaction->Rollback();
  }

  delete dbTransaction;
  delete printer_thing;
  delete source;
  delete destination;

  delete dbApptDataSet;
  delete dbTransDataSet;
  delete dbReferralDataSet;
  delete dbInsuranceDataSet;
  delete dbLetterDataSet;
}
//---------------------------------------------------------------------------
string TformLaunchPrintCobReport::DoVariableSubstitutions(const string &input, const pmapstrs &variables)
{
  string outstr = input;

  for(pmapstrs::const_iterator itr = variables.begin(); itr != variables.end(); ++itr)
    outstr = ReplaceStringWithString(outstr, itr->first, itr->second);

  return outstr;
}
//---------------------------------------------------------------------------
void __fastcall TformLaunchPrintCobReport::compositeReportAddReports(TObject *Sender)
{
  string sql;
  map<string, string> result_map;

  sql = "select tasks.cob_number, tasks.date_of_task, people.id person_id, "
        " person_names.first_name || ' ' || person_names.last_name || ' ' || person_names.suffix name, "
        " person_contact_info.line1, person_contact_info.city || ', ' || state_types.abbreviation || '  ' || "
        " person_contact_info.zip city_line, person_contact_info.phone_number, "
        " affiliations.cdib, people.social_security_number, people.birth_date, "
        " tax_entities.name provider, service_types.item service, area_types.item area, "
        " tasks.task_comments, tasks.visit_comments, cob_types.item cob_type, "
        " (select first 1 charts.chart_number from charts "
        "  where charts.person_id = people.id and charts.current_id+1>3 and "
        "  charts.location_id = '" + lookupTables->IdOf("location_types", "Contract Health") + "') chs_chart "
        "from tasks "
        " inner join services_provided on tasks.services_provided_id = services_provided.id "
        " inner join tax_entities on services_provided.tax_entity_id = tax_entities.id "
        " inner join service_types on services_provided.service_id = service_types.id "
        " inner join area_types on services_provided.area_id = area_types.id "
        " inner join people on tasks.person_id = people.id "
        " left join person_names on people.person_names_id = person_names.id "
        " left join affiliations on people.affiliations_id = affiliations.id "
        " left join person_contact_info on people.person_contact_info_id = person_contact_info.id "
        " left join state_types on person_contact_info.state_id = state_types.id "
        " left join cob_types on tasks.cob_id = cob_types.id "
        "where tasks.id = " + id;

  SqlToMap(dbTransaction, sql, result_map);

  report_form->lblCob->Caption = PrettyString(result_map["cob_number"]).c_str();
  report_form->lblChartNum->Caption = result_map["chs_chart"].c_str();
  string cob_type = result_map["cob_type"];
  if (cob_type != "" &&  cob_type != "0")
    report_form->lblCobTitle->Caption = cob_type.c_str();
  report_form->lblCobDate->Caption      = result_map["date_of_task"].c_str();
  report_form->lblName->Caption         = PrettyString(result_map["name"]).c_str();
  report_form->lblLine1->Caption        = FormatAddress(result_map["line1"]).c_str();
  report_form->lblCity->Caption         = PrettyString(result_map["city_line"]).c_str();
  report_form->lblPhone->Caption        = FormatPhoneNumber(result_map["phone_number"]).c_str();
  report_form->lblCdib->Caption         = PrettyString(result_map["cdib"]).c_str();
  report_form->lblSsn->Caption          = PrettySSN(result_map["social_security_number"]).c_str();
  report_form->lblDob->Caption          = result_map["birth_date"].c_str();
  report_form->lblProviderName->Caption = result_map["provider"].c_str();
  report_form->lblService->Caption      = PrettyString(result_map["service"]).c_str();
  report_form->lblArea->Caption         = PrettyString(result_map["area"]).c_str();

  /* Now let's report when their last consent for release of information was, eh? */
  map<string, string> consent_map;
  SqlToMap(dbTransaction, "select first 1 consents.signature_date consent_on, groups.item consent_at "
                          "from consents "
                          "inner join consent_types on consent_types.id = consents.consent_id "
                          "left join groups on consents.group_id = groups.id "
                          "where consent_types.item like '%RELEASE%' and person_id = '" + result_map["person_id"] + "' and current_id > 2 "
                          "order by consents.signature_date desc", consent_map);

  // jhaley 20120501: Is this just here to create a string at index "consent_on"?? Assignment to "" would be more clear...
  consent_map["consent_on"];

  /* for which we need to know the Date of Service ... we'll use the lattermost of the appointment dates marked as current */
  string DOS = GetOneField(dbTransaction, "select max(appointments.begin_date) from appointments where appointments.task_id = " + id + " and appointments.current_id+1 > 2");
  string time_since_last_consent;
  Pdate anchor_date(DOS, global_mode);
  Pdate relative_date(consent_map["consent_on"], global_mode);
  string consent_at = consent_map["consent_at"] != "" ? (" at " + PrettyString(consent_map["consent_at"])) : string("");
  if (FriendlyDiffDate(anchor_date.OutDate(mdy|yyyy), relative_date.OutDate(mdy|yyyy), time_since_last_consent, "DOS", true))
  {
    if (time_since_last_consent != "") time_since_last_consent = " (" + time_since_last_consent + ")";
    report_form->lblLastConsent->Caption = ReplaceStringWithString(report_form->lblLastConsent->Caption.c_str(),
                                                                   "Never",
                                                                   (consent_map["consent_on"] + time_since_last_consent + consent_at)).c_str();
  }

  // the following was done at the request of Cynthia Chancellor
  string reason = result_map["task_comments"];
  unsigned int s_reason;
  do
  {
    s_reason = reason.size();
    reason = StripTrailing(reason, " \n\t\r");
  } while (s_reason != reason.size());

  string usage = result_map["visit_comments"];
  unsigned int s_usage;
  do
  {
    s_usage = usage.size();
    usage = StripTrailing(usage, " \n\t\r");
  } while (s_usage != usage.size());

  report_form->lblReason->Caption = reason.c_str(); // /*SentenceString(*/result_map["task_comments"]/*)*/.c_str();
  report_form->lblUsage->Caption  = usage.c_str();  // /*SentenceString(*/result_map["visit_comments"]/*)*/.c_str();

  /* NOTE:  >= 2 is a little non-standard, but they explicitly WANTED "Expired" appointments (which is, correctly, IMO,
     how they are marking Cancelled and Rescheduled appointments) to show up.  */
  sql = "select appointments.begin_date, appointments.appointment_time, "
        " visit_outcome_types.item outcome from appointments, visit_outcome_types "
        "where appointments.visit_outcome_id = visit_outcome_types.id and "
        " appointments.task_id = " + id + " and "
        " appointments.current_id >= 2";

  dbApptDataSet->SelectSQL->Clear();
  dbApptDataSet->SelectSQL->Add(sql.c_str());
  dbApptDataSet->Open();
  dbApptDataSet->First();

  report_form->appt_report->DataSet = dbApptDataSet;
  report_form->txtApptDate->DataSet = dbApptDataSet;
  report_form->txtApptDate->DataField = "begin_date";
  report_form->txtApptTime->DataSet = dbApptDataSet;
  report_form->txtApptTime->DataField = "appointment_time";
  report_form->txtResult->DataSet = dbApptDataSet;
  report_form->txtResult->DataField = "outcome";

  compositeReport->Reports->Add(report_form->appt_report);

  // HACK: this was the slow query for printing COB's, so this one was hacked so the index optimization wouldn't suck
  // TODO: something still takes a couple of seconds, before any queries (according to the SQL monitor) before the
  //  report appears -- what? can it go away? do all reports take a few seconds, regardless?
  // i think i finally decided that was just my (Philip)'s computer -- running reports on citrix is almost
  //  instantaneous. there's some sort of extra overhead for me, everywhere. not ... so much O(N) being slower,
  //  just ... added time.
  sql = "select accounts.name, sum(transactions.debit_amount - transactions.credit_amount) trans_amount "
        "from transactions inner join accounts on accounts.id = transactions.account_id "
        "where transactions.task_id = " + id + " and "
        "transactions.current_id + 1 > 2 + 1 and "
        "accounts.account_type_id + 1 = " + lookupTables->IdOf("account_types", "Expense") + " + 1 "
        "group by accounts.name "
        "having sum(transactions.debit_amount - transactions.credit_amount) > 0.00";

  dbTransDataSet->SelectSQL->Clear();
  dbTransDataSet->SelectSQL->Add(sql.c_str());
  dbTransDataSet->Open();
  dbTransDataSet->First();

  report_form->limit_report->DataSet = dbTransDataSet;
  report_form->txtAccount->DataSet = dbTransDataSet;
  report_form->txtAccount->DataField = "name";
  report_form->exprAmount->Expression = "FORMATNUMERIC('$#,##0.00', trans_amount)";

  compositeReport->Reports->Add(report_form->limit_report);

  sql = "select referrals.date_of_referral, a.name to_provider, b.name by_provider "
        "from tasks, referrals, tax_entities a, tax_entities b "
        "where tasks.id = " + id + " and "
        "referrals.task_id = tasks.id and "
        "referrals.to_provider_id = a.id and "
        "referrals.by_provider_id = b.id and "
        "referrals.current_id >= 3";

  dbReferralDataSet->SelectSQL->Clear();
  dbReferralDataSet->SelectSQL->Add(sql.c_str());
  dbReferralDataSet->Open();
  dbReferralDataSet->First();

  report_form->referral_report->DataSet = dbReferralDataSet;
  report_form->txtReferralDate->DataSet = dbReferralDataSet;
  report_form->txtReferralDate->DataField = "date_of_referral";
  report_form->txtReferredTo->DataSet = dbReferralDataSet;
  report_form->txtReferredTo->DataField = "to_provider";
  report_form->txtReferredBy->DataSet = dbReferralDataSet;
  report_form->txtReferredBy->DataField = "by_provider";

  compositeReport->Reports->Add(report_form->referral_report);

  sql = "select tax_entities.name company, insurance_policies.policy_number, insurance_policies.group_number, "
        "insurance_policies.start_date, insurance_coverage.person_code "
        "from insurance_coverage "
        "inner join insurance_policies on insurance_coverage.insurance_policy_id = insurance_policies.id "
        "inner join tax_entities on insurance_policies.tax_entity_id = tax_entities.id "
        "inner join tasks on tasks.person_id = insurance_coverage.person_id "
        "where tasks.id = " + id + " and "
        "insurance_coverage.current_id >= 3 and "
        "insurance_policies.current_id >= 3 and "
        "(insurance_policies.stop_date > tasks.date_of_task or insurance_policies.stop_date is null)";

  dbInsuranceDataSet->SelectSQL->Clear();
  dbInsuranceDataSet->SelectSQL->Add(sql.c_str());
  dbInsuranceDataSet->Open();
  dbInsuranceDataSet->First();

  report_form->ins_report->DataSet = dbInsuranceDataSet;
  report_form->lblUser->Caption = PrettyString(lookupTables->ValueOf("users", current_userid)).c_str();
  report_form->txtCompany->DataSet = dbInsuranceDataSet;
  report_form->txtCompany->DataField = "company";
  report_form->txtPolicy->DataSet = dbInsuranceDataSet;
  report_form->txtPolicy->DataField = "policy_number";
  report_form->txtGroup->DataSet = dbInsuranceDataSet;
  report_form->txtGroup->DataField = "group_number";
  report_form->txtEffectiveDate->DataSet = dbInsuranceDataSet;
  report_form->txtEffectiveDate->DataField = "start_date";
  report_form->txtPersonCode->DataSet = dbInsuranceDataSet;
  report_form->txtPersonCode->DataField = "person_code";

  sql = "select conditions.note from conditions, tasks "
        "where tasks.id = " + id + " and "
        "tasks.person_id = conditions.person_id and "
        "conditions.current_id >= 3 and "
        "(conditions.condition_id = " + lookupTables->IdOf("condition_types", "Allergy") + " or "
        "conditions.condition_id = " + lookupTables->IdOf("condition_types", "Drug Allergy") + ")";
  list<string> condition_list;
  string allergies;
  SqlToList(dbTransaction, sql, condition_list);

  if(! condition_list.empty())
  {
    for(list<string>::iterator itr = condition_list.begin(); itr != condition_list.end(); itr++)
      allergies += (*itr) + ", ";
    allergies = allergies.substr(0, allergies.length() - 2);
    report_form->lblAllergies->Caption = PrettyString(allergies).c_str();
  }
  else
    report_form->lblAllergies->Caption = "None";

  compositeReport->Reports->Add(report_form->ins_report);

  // jhaley 20120202: Now they DON'T want this 9___9
/*
  // jhaley 20111007: ANOTHER CHS form letter that they are just dying to have so badly that I had to rush like a demon
  // to finish it >_>
  string tax_address_sql =
    "select first 1 TA.line1, TA.line2, TA.city, TAS.abbreviation state, TA.zip "
    "from tasks T "
    "inner join services_provided SP on T.services_provided_id = SP.id "
    "inner join tax_entities TE on SP.tax_entity_id = TE.id "
    "inner join tax_addresses TA on TA.id = coalesce(TE.billing_address_id, TE.physical_address_id) "
    "inner join state_types TAS on TA.state_id = TAS.id "
    "where T.id = " + id + " and TA.current_id >= 3 "
    "order by TA.current_id desc, TA.last_update_stamp desc ";
  pmapstrs letter_address_map;
  SqlToMap(dbTransaction, tax_address_sql, letter_address_map);

  if(letter_address_map["line1"] != "")
    report_form->memoLetterProviderAddress->Lines->Add(FormatAddress(letter_address_map["line1"]).c_str());
  if(letter_address_map["line2"] != "")
    report_form->memoLetterProviderAddress->Lines->Add(FormatAddress(letter_address_map["line2"]).c_str());

  string city_state_zip = PrettyString(letter_address_map["city"]) + ", " +
                                       letter_address_map["state"] + "  " + letter_address_map["zip"];
  report_form->memoLetterProviderAddress->Lines->Add(city_state_zip.c_str());

  string letter_outer_sql =
    "select TE.name provider_name, ('Dear ' || TE.name || ':') greeting, "
    " CHSPL.preamble_text, CHSPL.body_text, CHSPL.resolution_text, "
    " U.item signer_name, U.title signer_title, ATE.name signer_contact_name, ATEA.line1 signer_addr_line1, "
    " ATEA.line2 signer_addr_line2, ATEA.city signer_city, ATEAS.abbreviation signer_state, ATEA.zip signer_zip, "
    " ATEA.phone_number signer_phone "
    "from chs_provider_letter CHSPL "
    "inner join tasks T on T.id = " + id + " "
    "inner join services_provided SP on T.services_provided_id = SP.id "
    "inner join tax_entities TE on SP.tax_entity_id = TE.id "
    "inner join groups on groups.id = '" + lookupTables->IdOf("groups", "Contract Health") + "' "
    "inner join users U on groups.arra_user_id = U.id "
    "inner join tax_entities ATE on groups.arra_tax_entity_id = ATE.id "
    "left  join tax_addresses ATEA on ATEA.id = coalesce(ATE.physical_address_id, ATE.billing_address_id) "
    "left  join state_types ATEAS on ATEA.state_id = ATEAS.id "
    "where CHSPL.id = 0 ";
  pmapstrs letter_fields_map;
  SqlToMap(dbTransaction, letter_outer_sql, letter_fields_map);

  report_form->lblLetterDate->Caption          = DateToStr(Now()).c_str();
  report_form->lblLetterProviderName->Caption  = letter_fields_map["provider_name"].c_str();
  report_form->lblGreeting->Caption            = letter_fields_map["greeting"].c_str();
  report_form->memoPreamble->Lines->Text       = letter_fields_map["preamble_text"].c_str();
  report_form->memoBody->Lines->Text           = letter_fields_map["body_text"].c_str();
  report_form->lblLetterFooterName->Caption    = letter_fields_map["signer_contact_name"].c_str();
  report_form->lblLetterFooterContact->Caption = (letter_fields_map["signer_name"] + ", " +
                                                  letter_fields_map["signer_title"]).c_str();
  report_form->lblLetterFooterAddr->Caption    = (FormatAddress(letter_fields_map["signer_addr_line1"]) + ", " +
                                                  PrettyString(letter_fields_map["signer_city"]) + ", " +
                                                  letter_fields_map["signer_state"] + "  " +
                                                  FormatZipCode(letter_fields_map["signer_zip"])).c_str();

  string phone_number = FormatPhoneNumberNoBreak(letter_fields_map["signer_phone"]);
  report_form->lblLetterFooterPhone->Caption = phone_number.c_str();

  pmapstrs variables;
  variables["$phone_number$"] = phone_number;
  report_form->memoResolution->Lines->Text = DoVariableSubstitutions(letter_fields_map["resolution_text"], variables).c_str();

  string letter_inner_sql =
    "select CHSLI.item_title, CHSLI.item_text "
    "from chs_letter_items CHSLI "
    "where CHSLI.chs_provider_letter_id = 0 "
    "order by CHSLI.item_ordinal ";

  dbLetterDataSet->SelectSQL->Clear();
  dbLetterDataSet->SelectSQL->Add(letter_inner_sql.c_str());
  dbLetterDataSet->Open();
  dbLetterDataSet->First();

  report_form->reportLetter->DataSet   = dbLetterDataSet;
  report_form->txtItemTitle->DataSet   = dbLetterDataSet;
  report_form->txtItemTitle->DataField = "item_title";
  report_form->txtReason->DataSet      = dbLetterDataSet;
  report_form->txtReason->DataField    = "item_text";

  compositeReport->Reports->Add(report_form->reportLetter);

  // jhaley 20111007: Also log this letter to CHS mailing tasks
  InsertCHSMailTask(dbTransaction, result_map["person_id"],
                    string("ACA/IHCIA letter printed for provider ") + letter_fields_map["provider_name"]);
*/
}
//---------------------------------------------------------------------------
void __fastcall TformLaunchPrintCobReport::FormShortCut(TWMKey &Msg, bool &Handled)
{
  Handled = true;
  if (Msg.CharCode == VK_RETURN)
    btnRun->Click();
  else
    Handled = false;

  if (!Handled)
    TformSpecialGeneric::FormShortCut(Msg, Handled);
}
//---------------------------------------------------------------------------
void TformLaunchPrintCobReport::Do(string p_id, TformSpecialGeneric * parent_form)
{
 FORM_TRY
  TformLaunchPrintCobReport * z = new TformLaunchPrintCobReport(NULL, p_id);
  z->ShowTaskModal(parent_form);
 FORM_CATCH
}
//---------------------------------------------------------------------------
void TformLaunchPrintCobReport::DoSilently(string p_id)
{
 FORM_TRY
  TformLaunchPrintCobReport * z = new TformLaunchPrintCobReport(NULL, p_id);
  z->btnRun->Click();
  z->btnClose->Click();
 FORM_CATCH
}
//---------------------------------------------------------------------------

