//---------------------------------------------------------------------------

#include <vcl.h>
#pragma hdrstop

#include "launchProceduresWithoutChargesReport.h"
#include "reportProceduresWithoutCharges.h"
//---------------------------------------------------------------------------
#pragma package(smart_init)
#pragma link "launchGeneric"
#pragma link "codeBaseDropDown"
#pragma link "codeCheckListDropDown"
#pragma resource "*.dfm"
TformLaunchProceduresWithoutChargesReport *formLaunchProceduresWithoutChargesReport;
//---------------------------------------------------------------------------
__fastcall TformLaunchProceduresWithoutChargesReport::TformLaunchProceduresWithoutChargesReport(TComponent* Owner)
        : TformLaunchGeneric(Owner)
{
  pair<Pdate, Pdate> last_month_dates = GetPriorMonthDateRange(Pdate(Date()));
  dateBegin->Date = last_month_dates.first.OutDate();  // first day of last month
  dateEnd->Date   = last_month_dates.second.OutDate(); // last day of last month

  department_filter.SetSecurity("");
  department_filter.SetListViewHandle(listDepartments);
  department_filter.AddColumnSpec("Departments", "", as_text, true, 200);
  department_filter.AddColumnSpec("Current Id", "current_id", as_int, false, 40);
  department_filter.AddColumnSpec("Internal Id", "internal_id", as_int, false, 60);

  // Procedures viewed here are attached to tasks. The permission to view this form is based on procedure permissions,
  // which are not group-specific. The groups you can view procedure data for is therefore based on the task permission.
  set<string> group_perms = GetGroupPermissions("table.tasks", "", "view", "true");
  department_filter.SetSQLStatement("select distinct groups.item, " + CleverCurrentId("groups.valid", "1", "Current", "Expired") + ", groups.id "
                                    "from groups "
                                    "where " + SafeInClause("groups.id", SetToVec(group_perms)) + " and groups.id > 0 "
                                    "order by groups.item ");

  department_filter.FetchPreferences("procedureswocharges_report.groups");
  department_filter.SetIconStatus(true);
  department_filter.SetCheck(true);
  department_filter.ForceRefreshDataSet();

  // auto-check all available groups
  department_filter.SetCheckedWhereID(group_perms, true);
}
//---------------------------------------------------------------------------
bool TformLaunchProceduresWithoutChargesReport::IsValidOnScreen()
{
  if (dateBegin->Date > dateEnd->Date)
  {
    PSay("End date must be greater than or equal to begin date.", m_invalid_data, MB_OK);
    dateEnd->SetFocus();
    return false;
  }
  else if (department_filter.GetMultiID().size() == 0)
  {
    PSay("Please select one or more departments", m_invalid_data, MB_OK);
    listDepartments->SetFocus();
    return false;
  }

  return true;
}
//---------------------------------------------------------------------------
bool TformLaunchProceduresWithoutChargesReport::BeforeRunReport()
{
  dbDataset = NULL;
  dbDataset = new TIBDataSet(NULL);
  return TformLaunchGeneric::BeforeRunReport();
}
//---------------------------------------------------------------------------
bool TformLaunchProceduresWithoutChargesReport::RunReport()
{
  string begin_date = Pdate(dateBegin->Date).OutDate(mdy | yyyy);
  string end_date   = Pdate(dateEnd->Date).OutDate(mdy | yyyy);
  string date_title = begin_date + " - " + end_date;
  vector<string> group_ids = department_filter.GetMultiID();

  dbDataset->Database = dbDatabase;
  dbDataset->Transaction = report_transaction;
  dbDataset->UniDirectional = false;

  TformReportProceduresWithoutCharges *report_form = NULL;
  FORM_TRY
    report_form = new TformReportProceduresWithoutCharges(NULL);
  FORM_CATCH

  string sql = "select groups.item department, "
               "(extract(year from tasks.date_of_task) || '-' || LPAD(extract(month from tasks.date_of_task), 2, '0')) yearmo, "
               "case when procedures.cpt_hcpcs = '0' then procedures.local_use else procedures.cpt_hcpcs end px_code, "
               "coalesce(sum(codes.clinic_cost), 0.0) fee, "
               "count(procedures.id) procedure_count "
               "from procedures "
               "inner join tasks on procedures.task_id = tasks.id "
               "inner join groups on tasks.group_id = groups.id "
               "left join codes on procedures.cpt_hcpcs = codes.code "
               "where procedures.start_date between '" + begin_date + "' and '" + end_date + "' and "
               + SafeInClause("tasks.group_id", group_ids) + " and "
               "procedures.current_id+1 > 3 and tasks.current_id+1 > 3 and procedures.charges = 0 "
               "group by 1, 2, 3 "
               "order by 1, 2, 3 ";

  dbDataset->SelectSQL->Clear();
  dbDataset->SelectSQL->Add(sql.c_str());
  dbDataset->Open();
  dbDataset->First();

  report_form->reportProcedures->DataSet = dbDataset;
  report_form->lblDateTitle->Caption = date_title.c_str();  // display dates on title band
  report_form->lblUser->Caption = current_fullname.c_str(); // display generating user's name on page footer

  // detail band controls
  report_form->txtDepartment->DataSet = dbDataset;
  report_form->txtDepartment->DataField = "department";
  report_form->txtYearMo->DataSet = dbDataset;
  report_form->txtYearMo->DataField = "yearmo";
  report_form->txtPxCode->DataSet = dbDataset;
  report_form->txtPxCode->DataField = "px_code";
  report_form->txtNumProcedures->DataSet = dbDataset;
  report_form->txtNumProcedures->DataField = "procedure_count";
  report_form->exprDetailFee->Expression = "FORMATNUMERIC('$###,###,##0.00', fee)";

  // setup group header bands
  report_form->bandGroupDept->Expression = "department";
  report_form->bandGroupYearMo->Expression = "yearmo";
  report_form->exprGroupDept->Expression = "department";
  report_form->exprGroupMonth->Expression = "yearmo";

  // setup group footer and summary controls
  report_form->exprMonthProcedures->Expression   = "SUM(procedure_count)";
  report_form->exprDeptProcedures->Expression    = "SUM(procedure_count)";
  report_form->exprSummaryProcedures->Expression = "SUM(procedure_count)";
  report_form->exprMonthFee->Expression   = "FORMATNUMERIC('$###,###,##0.00', SUM(fee))";
  report_form->exprDeptFee->Expression    = "FORMATNUMERIC('$###,###,##0.00', SUM(fee))";
  report_form->exprSummaryFee->Expression = "FORMATNUMERIC('$###,###,##0.00', SUM(fee))";

  Processing(false);
  report_form->reportProcedures->Preview();

  dbDataset->Close();

  return true;
}
//---------------------------------------------------------------------------
bool TformLaunchProceduresWithoutChargesReport::AfterRunReport()
{
  if (dbDataset)
  {
    delete dbDataset;
    dbDataset = NULL;
  }
  return TformLaunchGeneric::AfterRunReport();
}
//---------------------------------------------------------------------------
