//---------------------------------------------------------------------------

#include <vcl.h>
#pragma hdrstop

#include "launchProcsByProvider.h"
#include "reportProcsByProvider.h"
#include "codeSearchTaxEntity.h"
//---------------------------------------------------------------------------
#pragma package(smart_init)
#pragma link "launchGeneric"
#pragma resource "*.dfm"
TformLaunchProcsByProvider *formLaunchProcsByProvider;
//---------------------------------------------------------------------------
__fastcall TformLaunchProcsByProvider::TformLaunchProcsByProvider(TComponent* Owner)
        : TformLaunchGeneric(Owner)
{
  CreateLink(txtProvider, btnProviderSearch, this, "", "tax_entities");

  pair<Pdate, Pdate> last_month_dates = GetPriorMonthDateRange(Pdate(Date()));
  dateBegin->Date = last_month_dates.first.OutDate();  // first day of last month
  dateEnd->Date   = last_month_dates.second.OutDate(); // last day of last month
}
//---------------------------------------------------------------------------
bool TformLaunchProcsByProvider::IsValidOnScreen()
{
  // Need a valid provider, first.
  if(IsBlankOrZero(GetValidLinkId(txtProvider)))
  {
    PSay("Please select a valid provider", m_missing_data, MB_OK);
    btnProviderSearch->SetFocus();
    return false;
  }

  // Permission is based on tasks.group view permission
  // (User must also have procedures and conditions permissions to even get in here so those aren't checked here)
  set<string> allowed_group_ids = GetGroupPermissions("table.tasks", "", "view", "true");

  string provider_group_sql =
    "select first 1 users.group_id "
    "from users "
    "where users.tax_entity_id = '" + GetValidLinkId(txtProvider) + "' and users.valid = 1 ";

  // Get the provider's user's group ID and check it for validity and for viewing user's permissions
  string provider_group_id = GetOneField(dbDatabase, provider_group_sql);
  if(IsBlankOrZero(provider_group_id))
  {
    PSay("The selected provider is not an employee.\n\n"
         "There is no local procedures data for this provider.", m_invalid_data, MB_OK);
    return false;
  }
  if(allowed_group_ids.find(provider_group_id) == allowed_group_ids.end())
  {
    PSay("You do not have permission to view visit-related information for this provider.\n\n"
         "Please speak with your supervisor if you need to view this provider's procedures via this report.",
         m_invalid_data, MB_OK);
    return false;
  }

  // Verify date range.
  if(Pdate(dateBegin->Date) > Pdate(dateEnd->Date))
  {
    PSay("Please select a begin date prior or equal to the end date", m_invalid_data, MB_OK);
    dateBegin->SetFocus();
    return false;
  }

  return true;
}
//---------------------------------------------------------------------------
bool TformLaunchProcsByProvider::BeforeRunReport()
{
  dbDataset = AddDataSet();
  return TformLaunchGeneric::BeforeRunReport();
}
//---------------------------------------------------------------------------
bool TformLaunchProcsByProvider::RunReport()
{
  string sql;
  string begin_date = Pdate(dateBegin->Date).OutDate(mdy | yyyy);
  string end_date   = Pdate(dateEnd->Date).OutDate(mdy | yyyy);
  string date_title = begin_date + " - " + end_date;

  // Create the report form.
  TformReportProcsByProvider *report_form = NULL;
  FORM_TRY
    report_form = new TformReportProcsByProvider(NULL);
  FORM_CATCH;

  // Main query.
  sql =
    "select (C.code || ' - ' || C.description) proc, count(P.id) num_procs, "
    "count(case when COND.id is not null then 1 else null end) dm, "
    "count(case when COND.id is null then 1 else null end) non_dm, "
    "sum(P.charges) charges "
    "from tasks T "
    "left join conditions COND on COND.id = "
    " (select first 1 ICOND.id from conditions ICOND "
    "  where (ICOND.code = '250' or ICOND.code like '250.%') and "
    "  ICOND.condition_id = " + lookupTables->IdOf("condition_types", "Condition") + " and "
    "  ICOND.person_id = T.person_id and ICOND.current_id >= 3) "
    "left join procedures P on P.task_id = T.id "
    "left join codes C on C.code = P.cpt_hcpcs "
    "where T.current_id >= 2 and T.person_id > 0 and T.tax_entity_id = '" + GetValidLinkId(txtProvider) + "' and "
    "T.date_of_task between '" + begin_date + "' and '" + end_date + "' "
    "group by 1 ";

  InitDataSet(dbDataset, sql);

  report_form->reportProcs->DataSet = dbDataset;

  // Title band
  report_form->lblTitleProvider->Caption = txtProvider->Caption; // show department name
  report_form->lblDateRange->Caption     = date_title.c_str();   // display dates on title band

  // Page footer
  report_form->lblUser->Caption = current_fullname.c_str(); // display generating user's name on page footer

  // Detail band
  SetupQRDBText(report_form->txtProcedure, dbDataset, "proc");
  SetupQRDBText(report_form->txtNumProcs,  dbDataset, "num_procs");
  SetupQRDBText(report_form->txtNumDM,     dbDataset, "dm");
  SetupQRDBText(report_form->txtNumNonDM,  dbDataset, "non_dm");
  SetupQRDBText(report_form->txtCharges,   dbDataset, "charges");

  // Summary band
  report_form->exprTotalNum->Expression     = "SUM(num_procs)";
  report_form->exprTotalDM->Expression      = "SUM(dm)";
  report_form->exprTotalNonDM->Expression   = "SUM(non_dm)";
  report_form->exprTotalCharges->Expression = "SUM(charges)";

  report_form->reportProcs->Preview();

  return true;
}
//---------------------------------------------------------------------------
void __fastcall TformLaunchProcsByProvider::btnProviderSearchClick(TObject *Sender)
{
  // create and show new person search form
  pvecstr return_vector;

  TformSearchTaxEntity *providerSearch = NULL;
  FORM_TRY
    providerSearch = new TformSearchTaxEntity(NULL, &return_vector);
  FORM_CATCH

  if(providerSearch)
  {
    providerSearch->ShowTaskModal(this);
    if(!return_vector.empty())
      CreateLink(txtProvider, btnProviderSearch, this, return_vector[0], "tax_entities");
  }
}
//---------------------------------------------------------------------------

