//---------------------------------------------------------------------------

#include <vcl.h>
#pragma hdrstop

#include "launchProviderLabels.h"
#include "reportMailingLabels.h"
#include "reportTaxEntityFileLabels.h"
#include "reportTaxEntityListing.h"
#include "derivedheaders.h"
//---------------------------------------------------------------------------
#pragma package(smart_init)
#pragma link "codeSpecialGeneric"
#pragma link "codeBaseDropDown"
#pragma link "codeCheckListDropDown"
#pragma resource "*.dfm"
TformLaunchProviderLabels *formLaunchProviderLabels;
//---------------------------------------------------------------------------
__fastcall TformLaunchProviderLabels::TformLaunchProviderLabels(TComponent* Owner)
        : TformSpecialGeneric(Owner)
{
  dateBegin->Date = Now();
  dateEnd->Date = Now();
  cbddTaxEntityTypes->Filter->SetSecurity("");
  cbddTaxEntityTypes->Filter->AddColumnSpec("Type", "", as_text, true, 200);
  cbddTaxEntityTypes->Filter->AddColumnSpec("Current Id", "current_id", as_int, false, 40);
  cbddTaxEntityTypes->Filter->AddColumnSpec("Internal Id", "internal_id", as_int, false, 60);
  string clever_current_id = "case when valid = 1 then '" + lookupTables->IdOf("current_types", "Current") + "' else '" + lookupTables->IdOf("current_types", "Expired") + "' end";
  cbddTaxEntityTypes->Filter->SetSQLStatement("select item, " + clever_current_id + ", id from tax_entity_types where id > 0");
  cbddTaxEntityTypes->Filter->FetchPreferences("launchProviderLabels.tax_entity_types");
  cbddTaxEntityTypes->Filter->SetIconStatus(true);
  cbddTaxEntityTypes->Filter->ForceRefreshDataSet();
  cbddTaxEntityTypes->Filter->CheckAll();
  cbddTaxEntityTypes->DisplayFieldName = "Type";
  cbddTaxEntityTypes->DropDownHeight = 144;
}
//---------------------------------------------------------------------------
void __fastcall TformLaunchProviderLabels::btnCloseClick(TObject *Sender)
{
  Close();        
}
//---------------------------------------------------------------------------
void __fastcall TformLaunchProviderLabels::btnRunClick(TObject *Sender)
{
  Processing(true);

  TIBTransaction * dbTransaction = new TIBTransaction(NULL);
  TIBDataSet * dbDataSet = new TIBDataSet(NULL);
  string sql;
  string begin_date = Pdate(dateBegin->Date).OutDate(mdy | yyyy);
  string end_date =  Pdate(dateEnd->Date).OutDate(mdy | yyyy);

  try
  {
    dbTransaction->DefaultDatabase = dbDatabase;
    dbDataSet->Database = dbDatabase;
    dbDataSet->Transaction = dbTransaction;
    dbDataSet->UniDirectional = false;

    bool group_by_service = radioReport->Enabled && radioReport->Checked && checkGroupByServices->Enabled && checkGroupByServices->Checked;
    bool exclude_invalid_addresses = (checkExcludeInvalidAddresses->Enabled && checkExcludeInvalidAddresses->Checked);

    vector<string> entity_types = cbddTaxEntityTypes->Filter->GetMultiID();

    sql =  "select tax_entities.name, tax_addresses.line1, tax_addresses.line2, "
           "tax_addresses.city || ', ' || state_types.abbreviation || '  ' || tax_addresses.zip city, "
           "tax_addresses.phone_number, tax_addresses.fax_number, tax_addresses.website, tax_addresses.email_address ";
    if (group_by_service)
       sql += ", service_types.id service_id, service_types.item service";
    sql += " from tax_entities inner join tax_addresses on tax_addresses.id = tax_entities.";
    if (radioBilling->Checked)
      sql += "billing_address_id ";
    else if (radioPhysical->Checked)
      sql += "physical_address_id ";
    if (group_by_service)
      sql += "inner join services_provided on services_provided.tax_entity_id = tax_entities.id "
             "inner join service_types on services_provided.service_id = service_types.id ";
    sql += "inner join state_types on tax_addresses.state_id = state_types.id where " + SafeInClause("tax_entities.tax_entity_id", entity_types) + " and ";
    if (exclude_invalid_addresses)
      sql += "tax_addresses.line1 is not null and tax_addresses.city is not null and "
             "tax_addresses.state_id > 0 and tax_addresses.zip is not null and ";
    if(checkBillDates->Checked)
      sql += "exists (select bills.id from bills where bills.actual_tax_id = tax_entities.id and "
          "bills.bill_date >= '" + begin_date + "' and "
          "bills.bill_date <= '" + end_date + "') and ";
    if(radioPrintOnly->Checked)
    {
      string temp = "";
      int count = 0;
      if(checkProviders->Checked)
      {
        temp += "exists (select bills.id from bills where bills.actual_tax_id = tax_entities.id and "
               "(bills.bill_id = " + lookupTables->IdOf("bill_types", "incoming provider bill") +
               " or bills.bill_id = " + lookupTables->IdOf("bill_types", "outgoing provider bill") + ")) or ";
        count++;
      }
      if(checkVendors->Checked)
      {
        temp += "exists(select bills.id from bills where bills.actual_tax_id = tax_entities.id and "
               "bills.bill_id = " + lookupTables->IdOf("bill_types", "incoming vendor bill") + ") or ";
        count++;
      }
      if(checkInsuranceCompanies->Checked)
      {
        /* TODO -c OVERSIGHT? : No date restriction here--what if they want to restrict this to insurance
           companies we've billed from the medical clinic, dental clinic, or one of the pharmacies,
           within a given date range? */
        temp += "exists(select insurance_policies.id from insurance_policies where "
               "insurance_policies.tax_entity_id = tax_entities.id) or ";
        count++;
      }
      if(temp != "")
      {
        temp = temp.substr(0, temp.length()-4);
        // sql += "and "; // er... no.
        if(count > 1)
          sql += "(" + temp + ") ";
        else
          sql += temp + " ";
      }
    }
    // Clean up any dangling SQL if necessary
    if (sql.substr(sql.length()-4, 3)=="and")
      sql = sql.substr(0, sql.length()-5);
    if (sql.substr(sql.length()-6, 5)=="where")
      sql = sql.substr(0, sql.length()-7);

    if(group_by_service) // id field shouldn't be necessary but I don't like relying on distinct text fields
      sql += " order by service_types.item, service_types.id, tax_entities.name ";
    else
      sql += " order by tax_entities.name";

    DebugSql(this, &sql, other_forms, 9);

    dbTransaction->StartTransaction();
    dbDataSet->SelectSQL->Add(sql.c_str());
    dbDataSet->Open();
    dbDataSet->First();

    if(radioMailing->Checked)
    {
      FORM_TRY
        TformReportMailingLabels *report = new TformReportMailingLabels(NULL);
        report->print_raw_names = true; // tax entity names are stored with user-defined capitalization; don't override
        report->labelReport->DataSet = dbDataSet;
        report->txtName->DataSet = dbDataSet;
        report->txtName->DataField = "name";
        report->txtAddress1->DataSet = dbDataSet;
        report->txtAddress1->DataField = "line1";
        Processing(false);
        report->labelReport->Preview();
      FORM_CATCH
    }
    else if(radioFile->Checked)
    {
      FORM_TRY
        TformReportTaxEntityFileLables *report = new TformReportTaxEntityFileLables(NULL);
        report->report->DataSet = dbDataSet;
        report->txtName->DataSet = dbDataSet;
        report->txtName->DataField = "name";
        Processing(false);
        report->report->Preview();
      FORM_CATCH
    }
    else if(radioReport->Checked)
    {
      FORM_TRY
        string begin_date = Pdate(dateBegin->Date).OutDate(mdy | yyyy);
        string end_date   = Pdate(dateEnd->Date).OutDate(mdy | yyyy);
        string date_title = begin_date + " - " + end_date;

        TformReportTaxEntityListing * listing = new TformReportTaxEntityListing(NULL);
        if(checkBillDates->Checked)
           listing->lblDateTitle->Caption = date_title.c_str();
        else
           listing->lblDateTitle->Caption = "";
        listing->report->DataSet = dbDataSet;

        if(group_by_service)
        {
          listing->groupHeaderBand->Expression = "service_id";
          listing->exprGroup->Expression = "service";
        }
        else
        {
          listing->groupHeaderBand->Enabled = false;
          listing->groupHeaderBand->Height = 0;
        }
        Processing(false);
        listing->report->Preview();
      FORM_CATCH
    }

    dbDataSet->Close();
    dbTransaction->Commit();

  }
  catch (...)
  {
    Processing(false);
    PSay("Error running report", m_database_error, MB_OK);
    if (dbTransaction->Active)
      dbTransaction->Rollback();
  }
  delete dbTransaction;
  delete dbDataSet;
}
//---------------------------------------------------------------------------
void __fastcall TformLaunchProviderLabels::FormShortCut(TWMKey &Msg, bool &Handled)
{
  Handled = true;
  if (Msg.CharCode == VK_RETURN)
    btnRun->Click();
  else
    Handled = false;

  if (!Handled)
    TformSpecialGeneric::FormShortCut(Msg, Handled);
}
//---------------------------------------------------------------------------
void __fastcall TformLaunchProviderLabels::checkBillDatesClick(TObject *Sender)
{
  if(checkBillDates->Checked)
  {
    lblBeginDate->Enabled = true;
    dateBegin->Enabled = true;
    lblEndDate->Enabled = true;
    dateEnd->Enabled = true;
  }
  else
  {
    lblBeginDate->Enabled = false;
    dateBegin->Enabled = false;
    lblEndDate->Enabled = false;
    dateEnd->Enabled = false;
  }
}
//---------------------------------------------------------------------------
void __fastcall TformLaunchProviderLabels::radioPrintOnlyClick(TObject *Sender)
{
  if(radioPrintOnly->Checked)
  {
    checkProviders->Enabled = true;
    checkVendors->Enabled = true;
    checkInsuranceCompanies->Enabled = true;
  }
  else
  {
    checkProviders->Enabled = false;
    checkVendors->Enabled = false;
    checkInsuranceCompanies->Enabled = false;
  }
}
//---------------------------------------------------------------------------
void __fastcall TformLaunchProviderLabels::changeHowToPrint(TObject *Sender)
{
   checkExcludeInvalidAddresses->Enabled = radioMailing->Checked;
   checkGroupByServices->Enabled = radioReport->Checked;
}
//---------------------------------------------------------------------------

/*
      sql = "select person_names.first_name || ' ' || person_names.last_name || ' ' || person_names.suffix name, "
            "person_contact_info.line1, person_contact_info.line2, "
            "person_contact_info.city || ', ' || state_types.abbreviation || '  ' || person_contact_info.zip city "
            "from mailing_label_report, people "
            "left join person_contact_info on people.person_contact_info_id = "
            "person_contact_info.id left join state_types on person_contact_info.state_id = state_types.id "
            "left join person_names on people.person_names_id = person_names.id "
            "where mailing_label_report.id = people.id "
            "order by person_names.last_name, person_names.first_name";
*/
