//---------------------------------------------------------------------------

#include <vcl.h>
#pragma hdrstop

#include "launchProviderSummaryReport.h"
#include "reportProviderSummary.h"
//---------------------------------------------------------------------------
#pragma package(smart_init)
#pragma link "codeSpecialGeneric"
#pragma resource "*.dfm"
TformLaunchProviderSummaryReport *formLaunchProviderSummaryReport;
//---------------------------------------------------------------------------
__fastcall TformLaunchProviderSummaryReport::TformLaunchProviderSummaryReport(TComponent* Owner)
        : TformSpecialGeneric(Owner)
{
  Pdate begin_date = CurrentPdate();
  begin_date.month = 1;
  begin_date.day = 1;
  dateBegin->Date = begin_date.OutDate();
  dateEnd->Date = Date();
}
//---------------------------------------------------------------------------
void __fastcall TformLaunchProviderSummaryReport::btnCloseClick(
      TObject *Sender)
{
  Close();        
}
//---------------------------------------------------------------------------
void __fastcall TformLaunchProviderSummaryReport::btnRunClick(
      TObject *Sender)
{
  Processing(true);

  TIBTransaction * dbTransaction = new TIBTransaction(NULL);
  TIBDataSet * dbDataSet = new TIBDataSet(NULL);
  string sql;
  string begin_date = Pdate(dateBegin->Date).OutDate(mdy | yyyy);
  string end_date =  Pdate(dateEnd->Date).OutDate(mdy | yyyy);
  string date_title = begin_date + " - " + end_date;

  try
  {
    dbTransaction->DefaultDatabase = dbDatabase;
    dbDataSet->Database = dbDatabase;
    dbDataSet->Transaction = dbTransaction;
    dbDataSet->UniDirectional = false;
    dbTransaction->StartTransaction();

    sql = "insert into temp_provider_paid select tax_entities.id, "
          "sum(paid_transactions.debit_amount - paid_transactions.credit_amount), "
          "count(visit_tasks.id) total_visits "
          "from tax_entities "
          "inner join tasks visit_tasks on visit_tasks.tax_entity_id = tax_entities.id "
          "inner join bills on bills.task_id = visit_tasks.id and "
          "bills.bill_id = " + lookupTables->IdOf("bill_types", "Incoming provider bill") + " "
          "inner join transactions paid_transactions on paid_transactions.bill_id = bills.id "
          "inner join accounts paid_accounts on paid_transactions.account_id = paid_accounts.id "
          "where visit_tasks.parent_task_id != 0 and "
          "paid_accounts.account_type_id = " + lookupTables->IdOf("account_types", "Expense") + " and "
          "visit_tasks.date_of_task between '" + begin_date + "' and '" + end_date + "' "
          "group by tax_entities.id";
    ExecuteStatement(dbTransaction, sql);

    sql = "insert into temp_provider_obligated "
          "select tax_entities.id, sum(ob_transactions.credit_amount - ob_transactions.debit_amount) "
          "from tax_entities "
          "inner join services_provided on services_provided.tax_entity_id = tax_entities.id "
          "inner join tasks ob_tasks on ob_tasks.services_provided_id = services_provided.id "
          "inner join transactions ob_transactions on ob_transactions.task_id = ob_tasks.id "
          "inner join accounts ob_accounts on ob_transactions.account_id = ob_accounts.id "
          "where ob_tasks.group_id = " + lookupTables->IdOf("groups", "Contract Health") + " and "
          "ob_accounts.account_type_id = " + lookupTables->IdOf("account_types", "Payable") + " and "
          "ob_tasks.date_of_task between '" + begin_date + "' and '" + end_date + "' "
          "group by tax_entities.id";
    ExecuteStatement(dbTransaction, sql);

    sql = "select tax_entities.name provider, temp_provider_paid.total_visits, "
          "temp_provider_obligated.amount_obligated, temp_provider_paid.amount_paid, 5 "
          "from (temp_provider_paid "
          "full outer join temp_provider_obligated on temp_provider_paid.tax_entity_id = temp_provider_obligated.tax_entity_id) "
          "inner join tax_entities on tax_entities.id = temp_provider_paid.tax_entity_id or "
          "tax_entities.id = temp_provider_obligated.tax_entity_id "
          "group by tax_entities.name, temp_provider_paid.total_visits, "
          "temp_provider_obligated.amount_obligated, temp_provider_paid.amount_paid";

    dbDataSet->SelectSQL->Add(sql.c_str());
    dbDataSet->Open();
    dbDataSet->First();

    FORM_TRY
      TformReportProviderSummary * report_form = new TformReportProviderSummary(NULL);
      report_form->report->DataSet = dbDataSet;
      report_form->lblDateTitle->Caption = date_title.c_str();
      report_form->lblUser->Caption = PrettyString(lookupTables->ValueOf("users", current_userid)).c_str();
      report_form->txtProvider->DataSet = dbDataSet;
      report_form->txtProvider->DataField = "provider";
      report_form->exprVisits->Expression = "FORMATNUMERIC('0', total_visits)";
      report_form->exprObligated->Expression = "FORMATNUMERIC('$#,###,###0.00', amount_obligated)";
      report_form->exprPaid->Expression = "FORMATNUMERIC('$#,###,###0.00', amount_paid)";
      report_form->exprTotal->Expression = "FORMATNUMERIC('$#,###,###0.00', SUM(amount_paid))";
      Processing(false);
      report_form->report->Preview();
    FORM_CATCH

    dbDataSet->Close();
    ExecuteStatement(dbTransaction, "delete from temp_provider_obligated");
    ExecuteStatement(dbTransaction, "delete from temp_provider_paid");    
    dbTransaction->Commit();

  }
  catch (...)
  {
    Processing(false);
    PSay("Error running report", m_database_error, MB_OK);
    if (dbTransaction->Active)
      dbTransaction->Rollback();
  }
  delete dbTransaction;
  delete dbDataSet;
}
//---------------------------------------------------------------------------
void __fastcall TformLaunchProviderSummaryReport::FormShortCut(TWMKey &Msg,
      bool &Handled)
{
  Handled = true;
  if (Msg.CharCode == VK_RETURN)
    btnRun->Click();
  else
    Handled = false;

  if (!Handled)
    TformSpecialGeneric::FormShortCut(Msg, Handled);
}
//---------------------------------------------------------------------------

