//---------------------------------------------------------------------------

#include <vcl.h>
#pragma hdrstop

#include "launchRollSheetReport.h"
#include "codeSearchEvent.h"
#include "reportRollSheet.h"
//---------------------------------------------------------------------------
#pragma package(smart_init)
#pragma link "launchGeneric"
#pragma resource "*.dfm"
TformLaunchRollSheetReport *formLaunchRollSheetReport;
//---------------------------------------------------------------------------
__fastcall TformLaunchRollSheetReport::TformLaunchRollSheetReport(TComponent* Owner, string &p_event_id, string &p_group_id)
        : TformLaunchGeneric(Owner)
{
  event_id = p_event_id;
  group_id = p_group_id;

  CreateLink(txtEventName, btnEventSearch, this, event_id, "events");
}
//---------------------------------------------------------------------------
bool TformLaunchRollSheetReport::IsValidOnScreen()
{
  if(GetValidLinkId(txtEventName) == "")
  {
    PSay("Please select an event", m_missing_data, MB_OK);
    btnEventSearch->SetFocus();
    return false;
  }

  return TformLaunchGeneric::IsValidOnScreen();
}
//---------------------------------------------------------------------------
bool TformLaunchRollSheetReport::BeforeRunReport()
{
  dbDataset = AddDataSet();
  return TformLaunchGeneric::BeforeRunReport();
}
//---------------------------------------------------------------------------
bool TformLaunchRollSheetReport::RunReport()
{
  string event_id = GetValidLinkId(txtEventName);

  // Create the report form
  TformReportRollSheet *report_form = NULL;
  FORM_TRY
    report_form = new TformReportRollSheet(NULL);
  FORM_CATCH

  // Get event information
  string event_info_sql =
    "select ENT.item event_name, EV.date_of_event, "
    "case when ELT.zip_code_id = 0 then ELT.item "
    "     when ELT.item = '' then (ZIP.city || ', ' || ZIP.state) "
    "     else (ZIP.city || ', ' || ZIP.state || ' - ' || ELT.item) end event_loc, "
    "case when EV.group_id = '" + lookupTables->IdOf("groups", "Caregiver Support Program") + "' then CST.item "
    "     else ETT.item end event_type, "
    "EV.number_attended "
    "from events EV "
    "left join event_name_types ENT on EV.event_name_id = ENT.id "
    "left join event_location_types ELT on EV.event_location_id = ELT.id "
    "left join zip_codes ZIP on ELT.zip_code_id = ZIP.id "
    "left join care_service_types CST on EV.care_service_id = CST.id "
    "left join event_type_types ETT on EV.event_type_id = ETT.id "
    "where EV.id = '" + event_id + "' ";

  map<string, string> event_info_map;
  if(SqlToMap(report_transaction, event_info_sql, event_info_map))
  {
    report_form->lblTitleEventName->Caption  = event_info_map["event_name"     ].c_str();
    report_form->lblTitleDate->Caption       = event_info_map["date_of_event"  ].c_str();
    report_form->lblTitleLocation->Caption   = event_info_map["event_loc"      ].c_str();
    report_form->lblTitleAttendance->Caption = event_info_map["number_attended"].c_str();

    if(group_id == lookupTables->IdOf("groups", "Caregiver Support Program"))
      report_form->lblTitleType->Caption = PrettyString(event_info_map["event_type"]).c_str();
    else
      report_form->lblTitleType->Caption = event_info_map["event_type"].c_str();
  }

  // Main query -- People on the roll sheet
  string event_roll_query =
    "select "
    "(coalesce((case when PN.id is not null then PN.prefix      else ERI.prefix      end), '') || ' ' || "
    " coalesce((case when PN.id is not null then PN.first_name  else ERI.first_name  end), '') || ' ' || "
    " coalesce((case when PN.id is not null then PN.middle_name else ERI.middle_name end), '') || ' ' || "
    " coalesce((case when PN.id is not null then PN.last_name   else ERI.last_name   end), '') || ' ' || "
    " coalesce((case when PN.id is not null then PN.suffix      else ERI.suffix      end), '')) person_name, "
    "(case when P.id   is not null then (select p_age from calc_age(P.birth_date)) else ERI.age end) person_age, "
    "coalesce(GT.item, '') person_gender "
    "from event_roll_sheets ERS "
    "left join event_roll_items ERI on ERI.event_roll_id = ERS.id and ERI.current_id >= 3 "
    "left join people P on ERI.person_id = P.id and P.id <> 0 "
    "left join person_names PN on P.person_names_id = PN.id "
    "left join gender_types GT on GT.id = (case when P.id is not null then P.gender_id else ERI.gender_id end) "
    "where ERS.event_id = '" + event_id + "' ";

  InitDataSet(dbDataset, event_roll_query);
  report_form->reportRoll->DataSet = dbDataset;

  // Page footer
  report_form->lblUser->Caption = current_fullname.c_str(); // display generating user's name on page footer

  // Detail band
  SetupQRDBText(report_form->txtName,   dbDataset, "person_name");
  SetupQRDBText(report_form->txtAge,    dbDataset, "person_age");
  SetupQRDBText(report_form->txtGender, dbDataset, "person_gender");

  report_form->reportRoll->Preview();

  return true;
}
//---------------------------------------------------------------------------
void __fastcall TformLaunchRollSheetReport::btnEventSearchClick(TObject *Sender)
{
  // create and show new event search form
  vector<string> return_vector;
  FORM_TRY
    TformSearchEvent *eventSearch = new TformSearchEvent(NULL, &return_vector, group_id);
    eventSearch->ShowTaskModal(this);
  FORM_CATCH
  if(!return_vector.empty())
    CreateLink(txtEventName, btnEventSearch, this, return_vector[0], "events");
}
//---------------------------------------------------------------------------

