//---------------------------------------------------------------------------

#include <vcl.h>
#pragma hdrstop

#include "launchSoonerCare.h"
#include "reportSoonerCare.h"
//---------------------------------------------------------------------------
#pragma package(smart_init)
#pragma link "launchGeneric"
#pragma resource "*.dfm"
TformLaunchSoonerCare *formLaunchSoonerCare;
//---------------------------------------------------------------------------
__fastcall TformLaunchSoonerCare::TformLaunchSoonerCare(TComponent* Owner)
        : TformLaunchGeneric(Owner)
{
  pair<Pdate, Pdate> last_month_dates = GetPriorMonthDateRange(Pdate(Date()));
  dateBegin->Date = last_month_dates.first.OutDate();  // first day of last month
  dateEnd->Date   = last_month_dates.second.OutDate(); // last day of last month
}
//---------------------------------------------------------------------------
bool TformLaunchSoonerCare::IsValidOnScreen()
{
  if(Pdate(dateBegin->Date) > Pdate(dateEnd->Date))
  {
    PSay("Please select a Begin Date before or equal to the End Date.", m_invalid_data, MB_OK);
    dateBegin->SetFocus();
    return false;
  }
  return true;
}
//---------------------------------------------------------------------------
bool TformLaunchSoonerCare::BeforeRunReport()
{
  dbDataSet = AddDataSet();
  return TformLaunchGeneric::BeforeRunReport();
}
//---------------------------------------------------------------------------
bool TformLaunchSoonerCare::RunReport()
{
  string sql;
  string begin_date = Pdate(dateBegin->Date).OutDate(mdy | yyyy);
  string end_date   = Pdate(dateEnd->Date).OutDate(mdy | yyyy);
  string date_title = begin_date + " - " + end_date;
  vector<string> group_ids;
  string soonerplan_id;

  // Create the report form
  TformReportSoonerCare *reportForm = NULL;
  FORM_TRY
    reportForm = new TformReportSoonerCare(NULL);
  FORM_CATCH_RETURN("TformReportSoonerCare", false)

  // NB: Count SoonerPlan Family Planning as coverage on this report
  sql = "select first 1 id from tax_entities where name = 'SoonerPlan Family Planning - Oklahoma Medicaid (Limited)'";
  soonerplan_id = GetOneField(dbDatabase, sql);

  // The report was specified to include visits at any department providing service at the clinic site.
  // People outside LAC/SC group see everything (mainly for my benefit).
  // This is not and shouldn't be conflated with a permissions issue, as none of the visit information is visible.
  if(current_groupid != lookupTables->IdOf("groups"," Little Axe Clinic"))
  {
    // Service groups @ SC:
    group_ids.push_back(lookupTables->IdOf("groups", "Shawnee Clinic"));
    group_ids.push_back(lookupTables->IdOf("groups", "Behavioral Health"));
    group_ids.push_back(lookupTables->IdOf("groups", "Community Health"));
  }
  if(current_groupid != lookupTables->IdOf("groups", "Shawnee Clinic"))
  {
    // Service groups @ LAC:
    group_ids.push_back(lookupTables->IdOf("groups", "Little Axe Clinic"));
    group_ids.push_back(lookupTables->IdOf("groups", "Dental Clinic"));
  }

  sql = "select "
        " (PN.last_name || ', ' || PN.first_name || ' ' || PN.middle_name || ' ' || PN.suffix) person_name, "
        " PCI.phone_number, "
        " case when exists "
        " (select IC.id "
        "  from insurance_coverage IC "
        "  inner join insurance_policies IP on IC.insurance_policy_id = IP.id "
        "  where IC.person_id = P.id and IC.current_id >= 3 and "
        "  (IC.stop_date is null or IC.stop_date > current_date) and "
        "  (IP.tax_entity_id = '" + soonerplan_id + "' or "
        "   (IP.insurance_id = '" + lookupTables->IdOf("insurance_types", "Medicaid") + "' and "
        "    exists (select * from insurance_benefits IB where IB.insurance_policy_id = IC.insurance_policy_id and "
        "            IB.is_covered = 1 and IB.area_id = '" + lookupTables->IdOf("area_types", "Medical") + "')))) "
        " then 'Yes' else 'No' end has_soonercare, "
        " case when exists "
        " (select IC.id "
        "  from insurance_coverage IC "
        "  inner join insurance_policies IP on IC.insurance_policy_id = IP.id "
        "  where IC.person_id = P.id and IC.current_id >= 3 and "
        "  (IC.stop_date is null or IC.stop_date > current_date) and "
        "  (IP.tax_entity_id = '" + soonerplan_id + "' or "
        "    exists (select * from insurance_benefits IB where IB.insurance_policy_id = IC.insurance_policy_id and "
        "            IB.is_covered = 1 and IB.area_id = '" + lookupTables->IdOf("area_types", "Medical") + "'))) "
        " then 'Yes' else 'No' end is_insured, "
        " (select p_age from calc_age(P.birth_date)) age, "
        " case when TT.id is null or TT.item = '' then 'Unknown' "
        "      when TT.id = 1 then 'Absentee Shawnee' "
        "      when TT.id in (0,33,34,53,54,64,90,98) then 'Non-Indian' "
        "      else 'Other Tribe' end aff_name, "
        " AFF.cdib, AFF.join_date, AFF.relinquish_date "
        "from people P "
        "left join person_names PN on P.person_names_id = PN.id "
        "left join person_contact_info PCI on P.person_contact_info_id = PCI.id ";

  // Non-Indians only?
  switch(radioGroupAffs->ItemIndex)
  {
  case 1:  // ASTs Only
    sql += "right join affiliations AFF on P.affiliations_id = AFF.id and AFF.tribe_id = 1 ";
    break;
  case 2:  // Non-Indians Only - Warning: more hard-coded non-tribal tribe_types ID's...
    sql += "right join affiliations AFF on P.affiliations_id = AFF.id and AFF.tribe_id in (0,33,34,53,54,64,90,98) ";
    break;
  default: // Everybody Dance Now (Unh, Unh-Unh, Unh - Unh)
    sql += "left join affiliations AFF on P.affiliations_id = AFF.id ";
    break;
  }

  sql += "left join tribe_types TT on AFF.tribe_id = TT.id "
         "where exists "
         "(select T.id from tasks T where T.person_id = P.id and "
         + SafeInClause("T.group_id", group_ids) + " and "
         " T.current_id >= 3 and T.date_of_task between '" + begin_date + "' and '" + end_date + "') ";

  // Restrictions by insurance status?
  switch(radioGroupSC->ItemIndex)
  {
  case 1: // SoonerCare Only
    sql += "and exists "
           " (select IC.id "
           "  from insurance_coverage IC "
           "  inner join insurance_policies IP on IC.insurance_policy_id = IP.id "
           "  where IC.person_id = P.id and IC.current_id >= 3 and "
           "  (IC.stop_date is null or IC.stop_date > current_date) and "
           "  (IP.tax_entity_id = '" + soonerplan_id + "' or "
           "   (IP.insurance_id = '" + lookupTables->IdOf("insurance_types", "Medicaid") + "' and "
           "    exists (select * from insurance_benefits IB where IB.insurance_policy_id = IC.insurance_policy_id and "
           "            IB.is_covered = 1 and IB.area_id = '" + lookupTables->IdOf("area_types", "Medical") + "')))) ";
    break;
  case 2: // Non-SoonerCare Only
    sql += "and not exists "
           " (select IC.id "
           "  from insurance_coverage IC "
           "  inner join insurance_policies IP on IC.insurance_policy_id = IP.id "
           "  where IC.person_id = P.id and IC.current_id >= 3 and "
           "  (IC.stop_date is null or IC.stop_date > current_date) and "
           "  (IP.tax_entity_id = '" + soonerplan_id + "' or "
           "   (IP.insurance_id = '" + lookupTables->IdOf("insurance_types", "Medicaid") + "' and "
           "    exists (select * from insurance_benefits IB where IB.insurance_policy_id = IC.insurance_policy_id and "
           "            IB.is_covered = 1 and IB.area_id = '" + lookupTables->IdOf("area_types", "Medical") + "')))) ";
    break;
  default:
    break;
  }

  sql += "order by 1 ";

  InitDataSet(dbDataSet, sql);

  reportForm->reportSC->DataSet = dbDataSet;

  // Title band
  reportForm->lblDateTitle->Caption = date_title.c_str();  // display dates on title band

  // Page footer
  reportForm->lblUser->Caption = current_fullname.c_str(); // display generating user's name on page footer

  // Detail band
  SetupQRDBText(reportForm->txtName,    dbDataSet, "person_name");
  SetupQRDBText(reportForm->txtPhoneNo, dbDataSet, "phone_number");
  SetupQRDBText(reportForm->txtHasSC,   dbDataSet, "has_soonercare");
  SetupQRDBText(reportForm->txtInsured, dbDataSet, "is_insured");
  SetupQRDBText(reportForm->txtAge,     dbDataSet, "age");
  SetupQRDBText(reportForm->txtAffName, dbDataSet, "aff_name");
  SetupQRDBText(reportForm->txtCDIB,    dbDataSet, "cdib");
  SetupQRDBText(reportForm->txtJoin,    dbDataSet, "join_date");
  SetupQRDBText(reportForm->txtEnd,     dbDataSet, "relinquish_date");


  reportForm->reportSC->Preview();

  return true;
}
//---------------------------------------------------------------------------
