//---------------------------------------------------------------------------

#include <vcl.h>
#pragma hdrstop

#include "launchTaskDemoReport.h"
#include "reportTaskDemographics.h"
//---------------------------------------------------------------------------
#pragma package(smart_init)
#pragma link "launchGeneric"
#pragma link "codeBaseDropDown"
#pragma link "codeCheckListDropDown"
#pragma resource "*.dfm"
TformLaunchTaskDemoReport *formLaunchTaskDemoReport;
//---------------------------------------------------------------------------
__fastcall TformLaunchTaskDemoReport::TformLaunchTaskDemoReport(TComponent* Owner)
        : TformLaunchGeneric(Owner)
{
  // Add all groups to cboDepartment for which the user has view permissions
  set<string> group_perms = GetGroupPermissions("table.tasks", "", "view", "true");
  string group_perm_sql =
    "select id, item, valid from groups where groups.id > 0 and " + SafeInClause("groups.id", SetToVec(group_perms)) + " "
    "order by item ";
  lookupTables->RegisterComboBox("groups", cboDepartment, group_perm_sql);

  // Set default dates
  pair<Pdate, Pdate> last_month_dates = GetPriorMonthDateRange(Pdate(Date()));
  dateBegin->Date = last_month_dates.first.OutDate();
  dateEnd->Date   = last_month_dates.second.OutDate();

  // Setup age groups
  lookupTables->RegisterComboBox("age_group_types", cboAgeGroups, "select min(id) id, group_name item, 1 valid from age_groups group by 2");

  // Setup enrollments CBDD - warning: duplicated code w/launchDiabClientReport, codeSearchPerson - should be a function probably at this point.
  string enrollment_sql =
    "select enrollment_types.item || (case when enrollment_status_types.id = 0 then '' else (' - ' || enrollment_status_types.item) end) item, "
    "enrollment_types.id * 1000 + coalesce(enrollment_status_types.id, 0) internal_id "           // hack alert!
    "from enrollment_types "
    "inner join permissions P on P.s = enrollment_types.permission_name and P.k = 'VIEW' and P.v = 'TRUE' and P.user_id = '" + current_userid + "' "
    "left join enrollment_status_types on (enrollment_types.id = enrollment_status_types.enrollment_id or "
    " (enrollment_types.allow_blank = 1 and enrollment_status_types.id = 0)) and enrollment_status_types.valid = 1 "
    "where enrollment_types.id > 0 and enrollment_types.valid = 1 "
    "order by enrollment_types.permission_name, enrollment_types.item, enrollment_status_types.item ";
  cbddEnrollment->Filter->SetSecurity("");
  cbddEnrollment->Filter->AddColumnSpec("Enrollment Type", "", as_text, true, 220, rawString);
  cbddEnrollment->Filter->AddColumnSpec("Internal Id", "internal_id", as_int, false, 60);
  cbddEnrollment->Filter->SetSQLStatement(enrollment_sql);
  cbddEnrollment->Filter->FetchPreferences("launchtaskdemo.enrollment_filter");
}
//---------------------------------------------------------------------------
__fastcall TformLaunchTaskDemoReport::~TformLaunchTaskDemoReport()
{
  if (lookupTables->TableName(cboDepartment) == "groups")
    lookupTables->UnregisterComboBox("groups", cboDepartment);
  if (lookupTables->TableName(cboAgeGroups) == "age_group_types")
    lookupTables->UnregisterComboBox("age_group_types", cboAgeGroups);
}
//---------------------------------------------------------------------------
bool TformLaunchTaskDemoReport::IsValidOnScreen()
{
  if (cboDepartment->Text == "" || !lookupTables->IsInLookUpTable("groups", cboDepartment))
  {
    PSay("Please select a valid department", m_invalid_data, MB_OK);
    cboDepartment->SetFocus();
    return false;
  }
  else if (cboAgeGroups->Text == "" || !lookupTables->IsInLookUpTable("age_group_types", cboAgeGroups))
  {
    PSay("Please select a valid age grouping", m_invalid_data, MB_OK);
    cboAgeGroups->SetFocus();
    return false;
  }
  else if (Pdate(dateBegin->Date) > Pdate(dateEnd->Date))
  {
    PSay("Please select a begin date prior or equal to the end date", m_invalid_data, MB_OK);
    dateBegin->SetFocus();
    return false;
  }
  else if (checkEnrollment->Checked && cbddEnrollment->Filter->GetMultiID().empty())
  {
    PSay("Please select one or more enrollment types, or uncheck 'Restrict by enrollment'", m_missing_data, MB_OK);
    cbddEnrollment->SetFocus();
    return false;
  }

  return true;
}
//---------------------------------------------------------------------------
bool TformLaunchTaskDemoReport::BeforeRunReport()
{
  dbDataset = new TIBDataSet(NULL);
  return TformLaunchGeneric::BeforeRunReport();
}
//---------------------------------------------------------------------------
bool TformLaunchTaskDemoReport::RunReport()
{
  string sql;
  string age_groups = UppercaseString(cboAgeGroups->Text.c_str());
  string group_id   = lookupTables->IdOf("groups", cboDepartment);
  string begin_date = Pdate(dateBegin->Date).OutDate(mdy | yyyy);
  string end_date   = Pdate(dateEnd->Date).OutDate(mdy | yyyy);
  string date_title = begin_date + " - " + end_date;

  // Create the report form
  TformReportTaskDemographics *report_form = NULL;
  FORM_TRY
    report_form = new TformReportTaskDemographics(NULL);
  FORM_CATCH

  sql = "select age_groups.id age_id, "
        "count(case when P.gender_id = '" + lookupTables->IdOf("gender_types", "Male")   + "' then 1 else null end) male_visits, "
        "count(case when P.gender_id = '" + lookupTables->IdOf("gender_types", "Female") + "' then 1 else null end) female_visits, "
        "count(case when P.gender_id = 0 then 1 else null end) unknown_visits, " // gender type indeterminate/unspecified
        "count(T.id) total_visits "
        "from tasks T "
        "inner join people P on T.person_id = P.id and P.id > 0 "
        "inner join age_groups on age_groups.group_name = '" + age_groups + "' and "
        " (select p_age from calc_age(P.birth_date)) >= age_groups.age_min and "
        " (select p_age from calc_age(P.birth_date)) <= age_groups.age_max and age_groups.id > 0 "
        "where T.group_id = '" + group_id + "' and T.date_of_task between '" + begin_date + "' and '" + end_date + "' and "
        " T.current_id+1 >= 3 ";

  report_form->memoRestrictionInfo->Lines->Add("* Count of tasks (people are not unique)");
  // Optional restriction by affiliations
  switch (radioGroupAffiliations->ItemIndex)
  {
  case 1: // AST people only
    sql += "and exists (select A.id from affiliations A where P.affiliations_id = A.id and "
           " A.tribe_id = '" + lookupTables->IdOf("tribe_types", "Absentee Shawnee Tribes of Oklahoma") + "') ";
    report_form->memoRestrictionInfo->Lines->Add("* People who are Absentee Shawnee tribal members");
    break;
  case 2: // Non-AST, but still tribal
    sql += "and exists (select A.id from affiliations A where P.affiliations_id = A.id and A.tribe_id not in (0, "
           + lookupTables->IdOf("tribe_types", "Absentee Shawnee Tribes of Oklahoma") + ", "
           + lookupTables->IdOf("tribe_types", "Indian-Non Tribal Member") + ", "
           + lookupTables->IdOf("tribe_types", "Non-Enrolled Tribal Member") + ", "
           + lookupTables->IdOf("tribe_types", "Non-Indian") + ", "
           + lookupTables->IdOf("tribe_types", "Unlisted (Not Federally-Recognized) Tribe") + ", "
           + lookupTables->IdOf("tribe_types", "Unknown (Unspecified Tribe)") + ", "
           + lookupTables->IdOf("tribe_types", "Other Federally-Recognized Tribe") + ")) ";
    report_form->memoRestrictionInfo->Lines->Add("* People who are members of tribes other than Absentee Shawnee Tribes of Oklahoma");
    break;
  case 3: // Non-tribal
    sql += "and exists (select A.id from affiliations A where P.affiliations_id = A.id and A.tribe_id in (0, "
           + lookupTables->IdOf("tribe_types", "Indian-Non Tribal Member") + ", "
           + lookupTables->IdOf("tribe_types", "Non-Enrolled Tribal Member") + ", "
           + lookupTables->IdOf("tribe_types", "Non-Indian") + ", "
           + lookupTables->IdOf("tribe_types", "Unlisted (Not Federally-Recognized) Tribe") + ", "
           + lookupTables->IdOf("tribe_types", "Unknown (Unspecified Tribe)") + ", "
           + lookupTables->IdOf("tribe_types", "Other Federally-Recognized Tribe") + ")) ";
    report_form->memoRestrictionInfo->Lines->Add("* People who are not tribal members");
    break;
  default: // All people (or anything else magically getting in here)
    report_form->memoRestrictionInfo->Lines->Add("* All people by affiliation");
    break;
  }

  // Optional restriction by enrollments
  if (checkEnrollment->Checked)
  {
    // jhaley 20101101: Add enrollments - warning: (mostly) duplicated code w/launchDiabClientReport and codeSearchPerson.
    vector<map<string, string> > selected_vecmap = cbddEnrollment->Filter->GetAllData(true, true);
    if (selected_vecmap.size() > 0)
    {
      sql += "and ";
      if (radioNotHaving->Checked) // unique to this form currently - allow negation
        sql += "not ";
      sql += "exists (select ENROLL.id from enrollment ENROLL where ENROLL.person_id = P.id and ENROLL.current_id+1 >= 4 and (";

      // These must all be added as a single condition, only one of which must be true (OR logic)
      for(vector<map<string, string> >::iterator i_row = selected_vecmap.begin(); i_row != selected_vecmap.end(); i_row++)
      {
        int combo_ids = StringToInt((*i_row)["internal_id"]);
        string enrollment_type_id   = IntToString(combo_ids / 1000); // hack alert! - can I do this better? probably...
        string enrollment_status_id = IntToString(combo_ids % 1000);

        // If not the first item, add an "or".
        if (i_row != selected_vecmap.begin())
          sql += "or ";

        sql += "(ENROLL.enrollment_id = '" + enrollment_type_id + "' ";
        if (!IsBlankOrZero(enrollment_status_id))
          sql += "and ENROLL.enrollment_status_id = '" + enrollment_status_id + "' ";

        // close this predicate
        sql += ") ";

        string restrict_string = "* People with";
        if (radioNotHaving->Checked)
          restrict_string += "out";
        restrict_string += " a current enrollment of type '" + (*i_row)["item"] + "'";
        report_form->memoRestrictionInfo->Lines->Add(restrict_string.c_str());
      }
      // Close the set of predicates and the 'exists' subquery
      sql += ")) ";
    }
  }

  // Optional visit restriction
  if (checkVisitsOnly->Checked)
  {
    sql += "and T.method_id = '" + lookupTables->IdOf("method_types", "Visit") + "' ";
    report_form->memoRestrictionInfo->Lines->Add("* Tasks marked as Visits");
  }

  // TODO : Other things that could be added on request...
  // * Restriction by (non)-attachment to an event with specified properties
  // * Restriction to specified method types (not just visit)
  // * Restriction by any of the other various task lookups (many have group-specific usage, though) 

  // Grouping/ordering by age groups
  sql += "group by 1 order by 1 ";

  dbDataset->Database = dbDatabase;
  dbDataset->Transaction = report_transaction;
  dbDataset->UniDirectional = false;
  dbDataset->SelectSQL->Clear();
  dbDataset->SelectSQL->Add(sql.c_str());
  dbDataset->Open();
  dbDataset->First();

  report_form->reportDemographics->DataSet = dbDataset;

  // Title band
  report_form->lblDepartmentTitle->Caption = cboDepartment->Text; // show department name
  report_form->lblDateTitle->Caption = date_title.c_str();  // display dates on title band

  // Page footer
  report_form->lblUser->Caption = current_fullname.c_str(); // display generating user's name on page footer

  // Detail band
  report_form->txtAgeGroup->DataSet   = dbDataset;
  report_form->txtAgeGroup->DataField = "age_id";
  report_form->txtMaleTasks->DataSet   = dbDataset;
  report_form->txtMaleTasks->DataField = "male_visits";
  report_form->txtFemaleTasks->DataSet   = dbDataset;
  report_form->txtFemaleTasks->DataField = "female_visits";
  report_form->txtUnspecifiedTasks->DataSet   = dbDataset;
  report_form->txtUnspecifiedTasks->DataField = "unknown_visits";
  report_form->txtAgeGroupTotal->DataSet   = dbDataset;
  report_form->txtAgeGroupTotal->DataField = "total_visits";

  // Summary band
  report_form->exprMaleSum->Expression        = "SUM(male_visits)";
  report_form->exprFemaleSum->Expression      = "SUM(female_visits)";
  report_form->exprUnspecifiedSum->Expression = "SUM(unknown_visits)";
  report_form->exprGrandTotal->Expression     = "SUM(total_visits)";

  report_form->reportDemographics->Preview();

  return true;
}
//---------------------------------------------------------------------------
bool TformLaunchTaskDemoReport::AfterRunReport()
{
  if (dbDataset)
  {
    delete dbDataset;
    dbDataset = NULL;
  }
  return TformLaunchGeneric::AfterRunReport();
}
//---------------------------------------------------------------------------
void __fastcall TformLaunchTaskDemoReport::cbddEnrollmentDown(TObject *Sender)
{
  cbddEnrollment->Filter->ForceRefreshDataSet();
  cbddEnrollment->DisplayFieldName = "Enrollment Type";
}
//---------------------------------------------------------------------------
