//---------------------------------------------------------------------------

#include <vcl.h>
#pragma hdrstop

#include "launchVendorBillsOverAmountReport.h"
#include "reportVendorBillsOverAmount.h"
//---------------------------------------------------------------------------
#pragma package(smart_init)
#pragma link "launchGeneric"
#pragma resource "*.dfm"
TformLaunchVendorBillsOverAmountReport *formLaunchVendorBillsOverAmountReport;
//---------------------------------------------------------------------------
__fastcall TformLaunchVendorBillsOverAmountReport::TformLaunchVendorBillsOverAmountReport(TComponent* Owner)
        : TformLaunchGeneric(Owner)
{
  pair<Pdate, Pdate> last_month_dates = GetPriorMonthDateRange(Pdate(Date()));

  dateBegin->Date = last_month_dates.first.OutDate();  // first day of last month
  dateEnd->Date   = last_month_dates.second.OutDate(); // last day of last month
}
//---------------------------------------------------------------------------
bool TformLaunchVendorBillsOverAmountReport::IsValidOnScreen()
{
  if (dateBegin->Date > dateEnd->Date)
  {
    PSay("End date must be greater than or equal to begin date.", m_invalid_data, MB_OK);
    dateEnd->SetFocus();
    return false;
  }
  else if (!IsFloat(editAmount->Text.c_str()))
  {
    PSay("Please enter a valid minimum expense in dollars", m_invalid_data, MB_OK);
    editAmount->SetFocus();
    return false;
  }

  return true;
}
//---------------------------------------------------------------------------
bool TformLaunchVendorBillsOverAmountReport::BeforeRunReport()
{
  dbDataset = NULL;
  dbDataset = new TIBDataSet(NULL);
  return TformLaunchGeneric::BeforeRunReport();
}
//---------------------------------------------------------------------------
bool TformLaunchVendorBillsOverAmountReport::RunReport()
{
  string begin_date = Pdate(dateBegin->Date).OutDate(mdy | yyyy);
  string end_date   = Pdate(dateEnd->Date).OutDate(mdy | yyyy);
  string date_title = begin_date + " - " + end_date;
  string min_amount = editAmount->Text.c_str();
  string sql;
  vector<string> tax_entities_vec;

  dbDataset->Database = dbDatabase;
  dbDataset->Transaction = report_transaction;
  dbDataset->UniDirectional = false;

  TformReportVendorBillsOverAmount *report_form = NULL;
  FORM_TRY
    report_form = new TformReportVendorBillsOverAmount(NULL);
  FORM_CATCH

  // first get the target set of tax entity id's, which is vendor entities with Expense accounts >= the cutoff
  sql = "select transactions.tax_entity_id "
        "from bills "
        "inner join transactions on bills.id = transactions.bill_id "
        "inner join accounts on transactions.account_id = accounts.id "
        "where bills.bill_id = '" + lookupTables->IdOf("bill_types", "INCOMING VENDOR BILL") + "' and "
        "bills.bill_date between '" + begin_date + "' and '" + end_date + "' and "
        "transactions.effective_date between '" + begin_date + "' and '" + end_date + "' and "
        "accounts.account_type_id = '" + lookupTables->IdOf("account_types", "EXPENSE") + "' "
        "group by transactions.tax_entity_id "
        "having sum(transactions.debit_amount - transactions.credit_amount) >= '" + min_amount + "' ";
  SqlToVec(report_transaction, sql, tax_entities_vec);

  // now do the report.
  sql = "select tax_entities.name provider, bills.bill_date, checks.check_number, "
        "sum(case when account_type_id = '" + lookupTables->IdOf("account_types", "PAYABLE") + "' then transactions.credit_amount - transactions.debit_amount else 0 end) amount_obligated, "
        "sum(case when account_type_id = '" + lookupTables->IdOf("account_types", "CASH") + "' then transactions.credit_amount - transactions.debit_amount else 0 end) amount_paid "
        "from bills "
        "inner join transactions on bills.id = transactions.bill_id "
        "inner join accounts on transactions.account_id = accounts.id "
        "inner join tax_entities on transactions.tax_entity_id = tax_entities.id "
        "left join purchase_reqs on bills.purchase_req_id = purchase_reqs.id "
        "left join checks on purchase_reqs.check_id = checks.id "
        "where bills.bill_id = '" + lookupTables->IdOf("bill_types", "INCOMING VENDOR BILL") + "' and "
        "bills.bill_date between '" + begin_date + "' and '" + end_date + "' and "
        "transactions.effective_date between '" + begin_date + "' and '" + end_date + "' and "
        "accounts.account_type_id in (" + lookupTables->IdOf("account_types", "PAYABLE") + ", " + lookupTables->IdOf("account_types", "CASH") + ") and "
        + SafeInClause("tax_entities.id", tax_entities_vec) + " "
        "group by 1, 3, 2 "
        "order by 1, 2 ";

  dbDataset->SelectSQL->Clear();
  dbDataset->SelectSQL->Add(sql.c_str());
  dbDataset->Open();
  dbDataset->First();

  report_form->reportVendors->DataSet = dbDataset;
  report_form->lblDateTitle->Caption = date_title.c_str();  // display dates on title band
  report_form->lblUser->Caption = current_fullname.c_str(); // display generating user's name on page footer

  string new_caption = report_form->lblTitle->Caption.c_str(); // add minimum cutoff amount to title
  new_caption += FormatString(min_amount, dollarString);
  report_form->lblTitle->Caption = new_caption.c_str();

  // detail band controls
  report_form->txtProvider->DataSet = dbDataset;
  report_form->txtProvider->DataField = "provider";
  report_form->txtBillDate->DataSet = dbDataset;
  report_form->txtBillDate->DataField = "bill_date";
  report_form->txtCheckNum->DataSet = dbDataset;
  report_form->txtCheckNum->DataField = "check_number";
  report_form->exprObAmount->Expression = "FORMATNUMERIC('$###,###0.00', amount_obligated)";
  report_form->exprPaidAmount->Expression = "FORMATNUMERIC('$###,###0.00', amount_paid)";
  report_form->exprExpensedAmount->Expression = "FORMATNUMERIC('$###,###0.00', amount_obligated + amount_paid)";

  Processing(false);
  report_form->reportVendors->Preview();

  dbDataset->Close();

  return true;
}
//---------------------------------------------------------------------------
bool TformLaunchVendorBillsOverAmountReport::AfterRunReport()
{
  if (dbDataset)
  {
    delete dbDataset;
    dbDataset = NULL;
  }
  return TformLaunchGeneric::AfterRunReport();
}
//---------------------------------------------------------------------------