#ifndef rel_vis_h
#define rel_vis_h

class rel_named
{
 public:
  string name;
  bool operator < (const rel_named & rhs) {return name < rhs.name;}
};

class rel_type : public rel_named
{
 public:
  // sorting function(s)
  // transformation function(s)
  // aggregate function(s)
};

class rel_field : public rel_named
{
 public:
  string type;
};

class rel_key : public rel_named
{
 public:
  set<string> fields;
  bool        unique;
};

class rel_table : public rel_named
{
 public:
  set<string> fields;
  set<string> keys;
};

class rel_cardinality : public rel_named
{
 public:
  string key_primary;
  string key_foreign;
};

class rel_schema : public rel_named
{
 public:
  set<rel_type>        types; // some sort of 'static'/'global'?
  set<rel_field>       fields;
  set<rel_key>         keys;
  set<rel_table>       tables;
  set<rel_cardinality> cardinalities;
};

class rel_row_value
{
 public:
  map<string, string> value;
};

class rel_index
{
 public:
  vector<string>           field_order;
  set<string>              field_set;
  vector<vector<string> *> index;
  unsigned int             GetIndexWorth(const set<string> & wanted_fields);
  bool                     IndexUsesField(const string & wanted_field);
  void                     TupleChanged(vector<string> * changed_tuple);
};

class rel_table_value : public rel_named
{
 public:
  // sorting is presentation-tier problem ...
  // as is tracking which row goes to which TTreeNode* or TListItem* ...
  // as we won't re-sort this list,
  // visual controls can keep track of a list/vector/map that includes the int/pointer of the row;
  // we get the benefits of a pointer without the worries of a memory re-allocation ...
  // but then, if we're not going to sort this vector here, do we need it to be a vector? a list would take up less space.
  // visual controls can then sort this index using a function object that looks up the original table value for each entry.
  // 'course, that seems slow ... but the look-up uses an int/pointer which means it's O(1), just overhead-like.
  vector<rel_index>      indexes;
  list<vector<string > > tuples;
};

class rel_schema_value : public rel_schema
{
 public:
  set<rel_table_value> table_values;
};

// realization: did i really want to build an in-memory database? it's a rather scary idea. but so many benefits ...

// data-holding variations:
// - vector<map<string, string> >
// - map<string, vector<string> >
// - map<string, vector<map<string, string> > >
// - map<string, map<string, vector<string> > >
// ...
// do we want to load a joined view, or the separate subsets of various tables?

// controls:
// - grouping listview (master/detail, possibly expanding/collapsing)
// - tree listview (actual row-to-row relationships involving parent/child data)
// - multi-column drop-down
// - directed graph
// - undirected graph
// - progress indicator (dependency, inclusion, %, count, status, etc.)
// - charts (bar, scatter, gantt, pie, timeline, geographical map, etc.)
//   - can we use a "default" representation for a given data set, but allow users to say "i want this on a timeline"?
// - reporting (incl. composite reports)
//   - reporting should include other, more generic, code for layout/paging/charts/graphics/etc.

/*

- interface
  - identifying fields (pk for fields)
  - identifying rows (pointer, index, pk)
  - information needed to transform rows into complex structures (trees), derived-class behavior
- cardinality
  - groups of fields
  - dependency between groups of fields (determinancy)
  - known uniqueness of fields (pk)
- security
  - which groups of fields may be seen
  - which fields in groups may be seen (not necessarily the same)
- layout
  - selection of fields
  - selection of rows
  - positioning of fields
  - sorting of rows/groups
  - grouping of rows for control-break processing
  - reporting-like arrangement with summary rows added
- value storage
  - simple strings with constant casts/constructors
  - built objects with pointers/inheritance
- value representation
  - complex (panels, controls)
  - symbiotic (listview with controls embedded vs. entire scrollbox full of panels)
  - simple (text transformations -- could be done early?)
- derived values
  - possible aggregate functions
  - rules for using them
- updatability
  - editing existing values
  - deleting existing values
  - inserting new values
  - checking for duplicates (constraints)
  - refreshing on external notifications (atlas)
- save/restore (data, for reports, export, import)
- save/restore (settings, for reports, listviews, etc.)
- concept of "drill-down"
  - pull more data
  - use preselected data
  - is a "person search" just finding people, for which we should then display all related info? (multiple names, addresses, etc.)
- selection using full candidate key information, possibly distinguishing between a person and a person's address even if in same list

- record generic
  - could start transaction immediately when creating new record
  - could pass transaction to children, handle savepoints for (partial) undo
  - make sure filters use transactions of containing form
  - atlas refreshes would need to succeed
  - (odd interaction of transactions)
  - interaction of transactions with non-record forms (launching wizards)
  - some way of knowing which forms are sharing a transaction
  - modal state of forms sharing transactions (because savepoints don't allow the freedom I'd need otherwise)

*/

#endif

