//---------------------------------------------------------------------------

#include <vcl.h>
#pragma hdrstop

#include "reportClinicPatientLabels.h"
//---------------------------------------------------------------------------
#pragma package(smart_init)
#pragma link "codeSpecialGeneric"
#pragma resource "*.dfm"
TformReportClinicPatientLabels *formReportClinicPatientLabels;
//---------------------------------------------------------------------------
__fastcall TformReportClinicPatientLabels::TformReportClinicPatientLabels(TComponent* Owner, int p_labels_per_person, int p_num_of_blanks, bool p_print_top_row, bool p_print_date_time_stamp, print_label_method p_how)
        : TformSpecialGeneric(Owner)
{
  num_label_repetitions = p_labels_per_person;
  // NOTE:  num_of_blanks is one less than the first label printed (start on label 5 == print 4 blank labels)
  num_of_blanks = p_num_of_blanks;
  if (num_of_blanks < 0) num_of_blanks = 0;
  print_top_row = p_print_top_row;
  print_date_time_stamp = p_print_date_time_stamp;
  how = p_how;

  if (!print_top_row)
  {
    int vert_ofs = 18;
    int top_row;
    switch (how)
    {
     case print_label_sheets:
      //switch SSN and code
      top_row = txtSsnSheet->Top;
      txtSsnSheet->Top = txtCodeSheet->Top;
      txtCodeSheet->Top = top_row;
      //disable the ones we don't want
      txtChartSheet->Enabled = false;
      txtCodeSheet->Enabled = false;
      // shift the rest up
      txtNameSheet->Top = txtNameSheet->Top - vert_ofs;
      txtDobSheet->Top = txtDobSheet->Top - vert_ofs;
      txtSsnSheet->Top = txtSsnSheet->Top - vert_ofs;
      txtGenderSheet->Top = txtGenderSheet->Top - vert_ofs;
      txtInsuranceSheet->Top = txtInsuranceSheet->Top - vert_ofs;
      lblDateTimeSheet->Top = lblDateTimeSheet->Top - vert_ofs;
      if (!print_date_time_stamp)
        lblDateTimeSheet->Enabled = false;
      break;
     case print_label_spooled:
      //switch SSN and code
      top_row = txtSsnSpooled->Top;
      txtSsnSpooled->Top = txtCodeSpooled->Top;
      txtCodeSpooled->Top = top_row;
      //disable the ones we don't want
      txtChartSpooled->Enabled = false;
      txtCodeSpooled->Enabled = false;
      // shift the rest up
      txtNameSpooled->Top = txtNameSpooled->Top - vert_ofs;
      txtDobSpooled->Top = txtDobSpooled->Top - vert_ofs;
      txtSsnSpooled->Top = txtSsnSpooled->Top - vert_ofs;
      txtGenderSpooled->Top = txtGenderSpooled->Top - vert_ofs;
      txtInsuranceSpooled->Top = txtInsuranceSpooled->Top - vert_ofs;
      lblDateTimeSpooled->Top = lblDateTimeSpooled->Top - vert_ofs;
      if (!print_date_time_stamp)
        lblDateTimeSpooled->Enabled = false;
      break;
    }
  }
}
//---------------------------------------------------------------------------
void __fastcall TformReportClinicPatientLabels::reportSheetLabelsBeforePrint(TCustomQuickRep *Sender, bool &PrintReport)
{
  label_number = 1;
  is_priming_pump = false;
  is_shooting_blanks = false;
}
//---------------------------------------------------------------------------
void __fastcall TformReportClinicPatientLabels::reportSheetLabelsAfterPreview(TObject *Sender)
{
  Close();
}
//---------------------------------------------------------------------------
void __fastcall TformReportClinicPatientLabels::infoBandBeforePrint(TQRCustomBand *Sender, bool &PrintBand)
{
  is_shooting_blanks = label_number <= num_of_blanks;

  TQuickRep* report = (how == print_label_sheets ? reportSheetLabels : reportSpooledLabels);
  if (is_priming_pump || is_shooting_blanks)
  {
    string msg = "reportClinicPatientLabels::infoBandBeforePrint going back one on label #" + IntToString(label_number) + "x" + IntToString((label_number-num_of_blanks)%num_label_repetitions);
    LogIt("appt_demographics.log", msg);
    if(debug_level[other_forms]==9 && message_router) message_router->SendMessageToAll("debug", msg);
    report->DataSet->Prior();
    //is_priming_pump = false;
  }
}
//---------------------------------------------------------------------------
void __fastcall TformReportClinicPatientLabels::infoBandAfterPrint(TQRCustomBand *Sender, bool BandPrinted)
{
  TQuickRep* report = (how == print_label_sheets ? reportSheetLabels : reportSpooledLabels);
  if (!report->DataSet->Bof)
  {
    string msg = "reportClinicPatientLabels::infoBandAfterPrint !BOF on label #" + IntToString(label_number) + "x" + IntToString((label_number-num_of_blanks)%num_label_repetitions);
    LogIt("appt_demographics.log", msg);
    if(debug_level[other_forms]==9 && message_router) message_router->SendMessageToAll("debug", msg);
    if (!is_shooting_blanks)
    {
      if ((label_number - num_of_blanks) % num_label_repetitions)
      {
        msg = "reportClinicPatientLabels::infoBandAfterPrint going back one on label #" + IntToString(label_number) + "x" + IntToString((label_number-num_of_blanks)%num_label_repetitions);
        LogIt("appt_demographics.log", msg);
        if(debug_level[other_forms]==9 && message_router) message_router->SendMessageToAll("debug", msg);
        report->DataSet->Prior();
      }
      else // if (!(report->DataSet->Eof))
        is_priming_pump = false;
    }
  }
  else
  {
    string msg = "reportClinicPatientLabels::infoBandAfterPrint BOF on label #" + IntToString(label_number) + "x" + IntToString((label_number-num_of_blanks)%num_label_repetitions);
    LogIt("appt_demographics.log", msg);
    if(debug_level[other_forms]==9 && message_router) message_router->SendMessageToAll("debug", msg);
    if (num_label_repetitions > 1 || (num_label_repetitions == 1 && num_of_blanks > 0))
      is_priming_pump = true;
  }

  label_number++;
}
//---------------------------------------------------------------------------
void __fastcall TformReportClinicPatientLabels::txtSsnPrint(TObject *sender, AnsiString &Value)
{
  if (is_shooting_blanks)
    Value = "";
  else
    Value = PrettySSN(Value.c_str()).c_str();
}
//---------------------------------------------------------------------------
void __fastcall TformReportClinicPatientLabels::txtTimePrint(TObject *sender, AnsiString &Value)
{
  Ptime temp(Value.c_str(), military | withsec);
  Value = temp.OutTime(ampm | withoutsec).c_str();
}
//---------------------------------------------------------------------------
void __fastcall TformReportClinicPatientLabels::OnLabelsPrint(TObject *sender,AnsiString &Value)
{
  if (is_shooting_blanks)
    Value = "";
  else
    Value = PrettyString(Value.c_str()).c_str();
}
//---------------------------------------------------------------------------
void __fastcall TformReportClinicPatientLabels::txtAddress1Print(TObject *sender, AnsiString &Value)
{
  if (is_shooting_blanks)
    Value = "";
  else
    Value = FormatAddress(Value.c_str()).c_str();
}
//---------------------------------------------------------------------------
void __fastcall TformReportClinicPatientLabels::txtAddress2Print(TObject *sender, AnsiString &Value)
{
  if (is_shooting_blanks)
    Value = "";
  else
  {
    string s = string(Value.c_str());
    size_t comma = s.find(",");
    if (comma != string::npos)
      s = PrettyString(s.substr(0, comma)) + UppercaseString(s.substr(comma));
    else
      s = PrettyString(s);
    Value = s.c_str();
  }
}
//---------------------------------------------------------------------------
void __fastcall TformReportClinicPatientLabels::txtPhonePrint(TObject *sender, AnsiString &Value)
{
  if (is_shooting_blanks)
    Value = "";
  else
    Value = FormatPhoneNumber(Value.c_str()).c_str();
}
//---------------------------------------------------------------------------
void __fastcall TformReportClinicPatientLabels::txtSsnSpooledPrint(TObject *sender, AnsiString &Value)
{
  if (is_shooting_blanks)
    Value = "";
  else
    Value = PrettySSN(Value.c_str()).c_str();
}
//---------------------------------------------------------------------------
void __fastcall TformReportClinicPatientLabels::lblDateTimePrint(TObject *sender, AnsiString &Value)
{
  TQuickRep* report = (how == print_label_sheets ? reportSheetLabels : reportSpooledLabels);
  Value = "";
  if (print_date_time_stamp && !is_shooting_blanks)
  {
    string d = (report->DataSet->FieldByName("appt_date")->AsString + " " +
              report->DataSet->FieldByName("appt_time")->AsString).c_str();
    if (d != " ")
    {
      Pstamp p(d, mdy | yyyy |datefirst | withsec);
      Value = p.OutStamp(datefirst | mdy | yy | military | withoutsec).c_str();
    }
  }
}
//---------------------------------------------------------------------------
void TformReportClinicPatientLabels::Execute(TformSpecialGeneric *calling_form, TDataSet * dbDataSet, int lpp, int nob, bool print_chart_and_code, bool p_print_date_time_stamp, print_label_method how)
{
  calling_form->Processing(true);
  Application->ProcessMessages();
  TformReportClinicPatientLabels * labels_form;
  if (how == print_label_spooled) nob = 0; // blank labels makes sense with sheet labels; not so much with spooled labels.  override whatever was passed if we're using a spool of labels.
  FORM_TRY
    labels_form = new TformReportClinicPatientLabels(NULL, lpp, nob, print_chart_and_code, p_print_date_time_stamp, how);
  FORM_CATCH
  //DataSetToCSV(dbDataSet, "appt_demographics.csv", true);
  switch (how)
  {
   case print_label_sheets:
    labels_form->reportSheetLabels->DataSet = dbDataSet;
    labels_form->txtNameSheet->DataSet = dbDataSet;
    labels_form->txtNameSheet->DataField = "person_name";
    labels_form->txtChartSheet->DataSet = dbDataSet;
    labels_form->txtChartSheet->DataField = "chart_number";
    labels_form->txtSsnSheet->DataSet = dbDataSet;
    labels_form->txtSsnSheet->DataField = "social_security_number";
    labels_form->txtGenderSheet->DataSet = dbDataSet;
    labels_form->txtGenderSheet->DataField = "gender";
    labels_form->txtInsuranceSheet->DataSet = dbDataSet;
    labels_form->txtInsuranceSheet->DataField = "ins_cov_shortstr";
    labels_form->txtAddress1Sheet->DataSet = dbDataSet;
    labels_form->txtAddress1Sheet->DataField = "address1";
    labels_form->txtAddress2Sheet->DataSet = dbDataSet;
    labels_form->txtAddress2Sheet->DataField = "address2";
    labels_form->txtPhoneSheet->DataSet = dbDataSet;
    labels_form->txtPhoneSheet->DataField = "home_phone";
    labels_form->txtDobSheet->DataSet = dbDataSet;
    labels_form->txtDobSheet->DataField = "birth_date";
    labels_form->txtCodeSheet->DataSet = dbDataSet;
    labels_form->txtCodeSheet->DataField = "tribal_code";
    calling_form->Processing(false);
    labels_form->reportSheetLabels->Preview();
    break;
   case print_label_spooled:
    labels_form->reportSpooledLabels->DataSet = dbDataSet;
    labels_form->txtNameSpooled->DataSet = dbDataSet;
    labels_form->txtNameSpooled->DataField = "person_name";
    labels_form->txtChartSpooled->DataSet = dbDataSet;
    labels_form->txtChartSpooled->DataField = "chart_number";
    labels_form->txtSsnSpooled->DataSet = dbDataSet;
    labels_form->txtSsnSpooled->DataField = "social_security_number";
    labels_form->txtGenderSpooled->DataSet = dbDataSet;
    labels_form->txtGenderSpooled->DataField = "gender";
    labels_form->txtInsuranceSpooled->DataSet = dbDataSet;
    labels_form->txtInsuranceSpooled->DataField = "ins_cov_shortstr";
    labels_form->txtAddress1Spooled->DataSet = dbDataSet;
    labels_form->txtAddress1Spooled->DataField = "address1";
    labels_form->txtAddress2Spooled->DataSet = dbDataSet;
    labels_form->txtAddress2Spooled->DataField = "address2";
    labels_form->txtPhoneSpooled->DataSet = dbDataSet;
    labels_form->txtPhoneSpooled->DataField = "home_phone";
    labels_form->txtDobSpooled->DataSet = dbDataSet;
    labels_form->txtDobSpooled->DataField = "birth_date";
    labels_form->txtCodeSpooled->DataSet = dbDataSet;
    labels_form->txtCodeSpooled->DataField = "tribal_code";
    calling_form->Processing(false);
    labels_form->reportSpooledLabels->Preview();
    break;
  }
}
//---------------------------------------------------------------------------
void __fastcall TformReportClinicPatientLabels::OnChartNumberPrint(TObject *sender, AnsiString &Value)
{
  if (is_shooting_blanks)
    Value = "";
  else
    Value = StripLeading(Value.c_str(), "0").c_str();
}
//---------------------------------------------------------------------------

