//---------------------------------------------------------------------------

#include <vcl.h>
#pragma hdrstop

#include "reportDaySchedule.h"
//---------------------------------------------------------------------------
#pragma package(smart_init)
#pragma link "codeSpecialGeneric"
#pragma resource "*.dfm"
TformReportDaySchedule *formReportDaySchedule;
//---------------------------------------------------------------------------
__fastcall TformReportDaySchedule::TformReportDaySchedule(TComponent* Owner)
        : TformSpecialGeneric(Owner)
{
}
//---------------------------------------------------------------------------
void TformReportDaySchedule::Do(PApptOccurrenceMap &p_appt_set, Pdate current_date, bool preview_modal)
{
  int report_stage;
  
  TformReportDaySchedule *report_form   = NULL;
  TIBTransaction         *dbTransaction = NULL;
  TIBDataSet             *dbDataSet     = NULL;

  FORM_TRY
    report_form = new TformReportDaySchedule(NULL);
  FORM_CATCH_RETURN("TformReportDaySchedule", )

  TQuickRep *the_report = report_form->reportDaySchedule;

  try
  {
    report_stage = 0;
    dbTransaction = new TIBTransaction(NULL);
    dbDataSet     = new TIBDataSet(NULL);

    // startup transaction and init dataset
    ++report_stage;
    StdTransaction(dbTransaction, dbDatabase);
    dbDataSet->Transaction    = dbTransaction;
    dbDataSet->UniDirectional = false;

    // clear out appointment_reports
    ++report_stage;
    ExecuteStatement(dbTransaction, "delete from appointment_reports");

    // insert records into temp table based on passed-in occurrences map
    ++report_stage;
    for(PApptOccurrenceMap::iterator itr = p_appt_set.begin(); itr != p_appt_set.end(); ++itr)
    {
      for(PApptOccurrenceSet::iterator set_itr = itr->second.begin(); set_itr != itr->second.end(); ++set_itr)
      {
        pmapstrs insert_map;
        insert_map["id"]            = (*set_itr).id;
        insert_map["appt_date"]     = itr->first.OutDate(mdy | yyyy);
        insert_map["appt_time"]     = (*set_itr).range_begin.OutTime(military | withsec);
        insert_map["appt_end_time"] = (*set_itr).range_end.OutTime(military | withsec);
        ExecuteInsertStatement(dbTransaction, "appointment_reports", insert_map);
      }
    }

    // build the sql query for the report
    ++report_stage;
    string report_sql =
      "select U.user_color color, AREP.appt_time begin_time, AREP.appt_end_time end_time, "
      " (case when P.id is null or P.id = 0 then 'No person' else (PN.first_name || ' ' || PN.last_name) end) person, "
      " APPT.description, "
      " (select first 1 CH.chart_number "
      "  from charts CH "
      "  where CH.person_id = P.id "
      "  and CH.current_id > 2 "
      "  order by "
      "    case when CH.group_id = APPT.group_id then 1 else 0 end desc, "
      "    case when ALT.chart_location_id = AGRP.chart_location_id then 1 else 0 end desc, "
      "    CH.last_update_stamp desc "
      " ) chart_number "
      "from appointment_reports AREP "
      "inner join appointments APPT on AREP.id = APPT.id "
      "inner join users U on APPT.created_for = U.id "
      "left  join people P on APPT.person_id = P.id "
      "left  join person_names PN on P.person_names_id = PN.id "
      "left  join appt_location_types ALT on APPT.appt_location_id = ALT.id "
      "left  join groups AGRP on APPT.group_id = AGRP.id ";

    // open the dataset
    dbDataSet->SelectSQL->Clear();
    dbDataSet->SelectSQL->Add(report_sql.c_str());
    dbDataSet->Open();
    dbDataSet->First();

    // setup the report controls
    ++report_stage;
    the_report->DataSet = dbDataSet;
    report_form->lblDate->Caption =
      (current_date.NameOfDay() + " - " + current_date.NameOfMonth() + " " +
       IntToString(current_date.day) + ", " + IntToString(current_date.year)).c_str();
    report_form->txtBeginTime->DataSet     = dbDataSet;
    report_form->txtBeginTime->DataField   = "begin_time";
    report_form->txtEndTime->DataSet       = dbDataSet;
    report_form->txtEndTime->DataField     = "end_time";
    report_form->txtName->DataSet          = dbDataSet;
    report_form->txtName->DataField        = "person";
    report_form->txtDescription->DataSet   = dbDataSet;
    report_form->txtDescription->DataField = "description";
    report_form->txtChartNo->DataSet       = dbDataSet;
    report_form->txtChartNo->DataField     = "chart_number";

    // preview report - pass preview_modal == true if the report could be run recursively!
    ++report_stage;
    if(preview_modal)
      the_report->PreviewModal();
    else
      the_report->Preview();
  }
  catch(...)
  {
    ::PSay("An error occurred while running the report (report stage " + IntToString(report_stage) + ")",
           m_program_error, MB_OK, "TformReportDaySchedule");
  }

  // Cleanup/Shutdown
  if(dbDataSet)
  {
    delete dbDataSet;
    dbDataSet = NULL;
  }
  if(dbTransaction)
  {
    if(dbTransaction->Active)
      dbTransaction->Rollback();
    delete dbTransaction;
    dbTransaction = NULL;
  }

  report_form->Close();
}
//---------------------------------------------------------------------------
void __fastcall TformReportDaySchedule::txtBeginTimePrint(TObject *sender, AnsiString &Value)
{
  Value = Ptime(Value.c_str(), withsec | military).OutTime(ampm | withoutsec).c_str();
}
//---------------------------------------------------------------------------
void __fastcall TformReportDaySchedule::txtEndTimePrint(TObject *sender, AnsiString &Value)
{
  Value = Ptime(Value.c_str(), withsec | military).OutTime(ampm | withoutsec).c_str();
}
//---------------------------------------------------------------------------
void __fastcall TformReportDaySchedule::txtNamePrint(TObject *sender, AnsiString &Value)
{
  Value = PrettyString(Value.c_str()).c_str();
}
//---------------------------------------------------------------------------
void __fastcall TformReportDaySchedule::txtChartNoPrint(TObject *sender, AnsiString &Value)
{
  Value = (string("chart # ") + Value.c_str()).c_str();        
}
//---------------------------------------------------------------------------
void __fastcall TformReportDaySchedule::bandDetailBeforePrint(TQRCustomBand *Sender, bool &PrintBand)
{
  if(!reportDaySchedule->DataSet->IsEmpty())
    shapeUserColor->Brush->Color = TColor(StringToInt(reportDaySchedule->DataSet->FieldByName("color")->AsString.c_str()));
}
//---------------------------------------------------------------------------
void __fastcall TformReportDaySchedule::txtDescriptionPrint(TObject *sender, AnsiString &Value)
{
  Value = SentenceString(Value.c_str()).c_str();
}
//---------------------------------------------------------------------------
