//---------------------------------------------------------------------------

#include <vcl.h>
#pragma hdrstop

#include "reportPendingAccounts.h"
//---------------------------------------------------------------------------
#pragma package(smart_init)
#pragma link "codeSpecialGeneric"
#pragma resource "*.dfm"
TformReportPendingAccounts *formReportPendingAccounts;
//---------------------------------------------------------------------------
__fastcall TformReportPendingAccounts::TformReportPendingAccounts(TComponent* Owner, vector<string> & p_column_names, map<string, vector<financial> > & p_entity_totals, vector<financial> & p_bottom_totals, bool p_show_detail_rows, bool p_reverse_amount_field)
        : TformSpecialGeneric(Owner), column_names(p_column_names), entity_totals(p_entity_totals), bottom_totals(p_bottom_totals), show_detail_rows(p_show_detail_rows), reverse_amount_field(p_reverse_amount_field)
{
 row_number = 0;
 today_date = Pdate(TDateTime::CurrentDate());
 // new TQRLabel -- headers
 // new TQRLabel -- fields (not really from db, thus not TQRDBText)
 // margin between items should = (labelTaxEntityID->Left - labelTaxEntityName->Right)
 // take space defined as (bandColumnHeader->Width - labelTaxEntityID->Right)) as space to use
 // divide that up among the columns equally, then each field is given its space, right-justified ...
 // with margin to the left (size specified above) included inside the divided space.
 // it -would- be too damned convenient of them to provide us with a ->Right on labels, hunh. grrr.
 unsigned int num_columns  = column_names.size();
 unsigned int start_at     = (labelTaxEntityID->Width + labelTaxEntityID->Left);
 unsigned int margin_space = (labelTaxEntityID->Left  - (labelTaxEntityName->Left + labelTaxEntityName->Width));
 unsigned int column_space = (bandColumnHeader->Width - start_at);
 double space_per_column   = (double)column_space / (double)num_columns;

 TQRLabel * newest_column_header;
 TQRLabel * newest_column_value;
 TQRLabel * newest_column_summary;
 for (unsigned int i = 0; i < num_columns; i++)
 {
  if(debug_level[other_forms]>=2 && message_router) message_router->SendMessageToAll("debug", "doing a column");
  unsigned int whole_left  = ((double)i * space_per_column) + start_at;
  unsigned int whole_right = ((double)(i + 1) * space_per_column) + start_at;
  unsigned int text_left   = whole_left + margin_space;

  newest_column_header               = new TQRLabel(this);
  newest_column_header->Parent       = bandColumnHeader;
  newest_column_header->Caption      = column_names[i].c_str();
  newest_column_header->Left         = text_left;
  newest_column_header->Width        = whole_right - text_left;
  newest_column_header->Top          = 2;
  newest_column_header->Height       = 18;
  newest_column_header->Transparent  = true;
  newest_column_header->Alignment    = taRightJustify;
  newest_column_header->AutoSize     = false;
  newest_column_header->AutoStretch  = false;

  newest_column_summary              = new TQRLabel(this);
  newest_column_summary->Parent      = bandSummary;
  newest_column_summary->Caption     = FinancialToString(bottom_totals[i], true, 2).c_str();
  newest_column_summary->Left        = whole_left;
  newest_column_summary->Width       = whole_right - whole_left;
  newest_column_summary->Top         = 2;
  newest_column_summary->Height      = 18;
  newest_column_summary->Transparent = true;
  newest_column_summary->Alignment   = taRightJustify;
  newest_column_summary->AutoSize    = false;
  newest_column_summary->AutoStretch = false;
  if (show_detail_rows)
   newest_column_summary->Font->Style = newest_column_summary->Font->Style << fsBold;

  newest_column_value                = new TQRLabel(this);
  newest_column_value->Parent        = bandGroupEntity;
  newest_column_value->Caption       = "bob";
  newest_column_value->Left          = whole_left;
  newest_column_value->Width         = whole_right - whole_left;
  newest_column_value->Top           = 0;
  newest_column_value->Height        = 17;
  newest_column_value->Transparent   = true;
  newest_column_value->Alignment     = taRightJustify;
  newest_column_value->AutoSize      = false;
  newest_column_value->AutoStretch   = false;
  if (show_detail_rows)
   newest_column_value->Font->Style   = newest_column_value->Font->Style << fsBold;

  value_labels.push_back(newest_column_value);
 }

 if (show_detail_rows)
 {
  textTaxEntityName->Font->Style = textTaxEntityName->Font->Style << fsBold;
  textTaxEntityID->Font->Style = textTaxEntityID->Font->Style << fsBold;
 }
}
//---------------------------------------------------------------------------

void __fastcall TformReportPendingAccounts::textItemAmountPrint(
      TObject *sender, AnsiString &Value)
{
 if (reverse_amount_field)
  Value = FinancialToString(-(StringToFinancial(Value.c_str()))).c_str();
 else
  Value = StringToAmount(Value.c_str()).c_str();
 return;
}
//---------------------------------------------------------------------------

void __fastcall TformReportPendingAccounts::bandGroupEntityBeforePrint(
      TQRCustomBand *Sender, bool &PrintBand)
{
 // fill in values with -stuff-!
 unsigned int num_columns = column_names.size();
 string tax_entity_id = textTaxEntityName->DataSet->FieldByName("entity_id")->AsString.c_str();
 map<string, vector<financial> >::iterator i_entity = entity_totals.find(tax_entity_id);
 if (i_entity != entity_totals.end())
 {
  for (unsigned int i = 0; i < num_columns; i++)
   value_labels[i]->Caption = FinancialToString(i_entity->second[i], true, 2).c_str();

  if (!show_detail_rows)
  {
   if (row_number % 2)
    bandGroupEntity->Color = TColor(0x00eeffee);
   else
    bandGroupEntity->Color = TColor(0x00ffffff);
   row_number++;
  }
 }
 else
  PrintBand = false;
 return;
}
//---------------------------------------------------------------------------

void __fastcall TformReportPendingAccounts::textPersonPrint(
      TObject *sender, AnsiString &Value)
{
 if (Value != ", ")
  Value = PrettyString(Value.c_str()).c_str();
 else
  Value = "";
 return;
}
//---------------------------------------------------------------------------

void __fastcall TformReportPendingAccounts::textServiceProviderPrint(
      TObject *sender, AnsiString &Value)
{
 Value = PrettyString(Value.c_str()).c_str();
 return;
}
//---------------------------------------------------------------------------

void __fastcall TformReportPendingAccounts::textPurchaseReqNumberPrint(
      TObject *sender, AnsiString &Value)
{
 if (Value != "")
  Value = "C" + Value;
 return;
}
//---------------------------------------------------------------------------

void __fastcall TformReportPendingAccounts::textItemDatePrint(
      TObject *sender, AnsiString &Value)
{
 if (Value != "")
 {
  Pdate temp_date;
  temp_date.InDate(Value.c_str(), mdy | yyyy);
  Value = temp_date.OutDate(mdy | yyyy | left_zeros).c_str();
 }
 return;
}
//---------------------------------------------------------------------------

void __fastcall TformReportPendingAccounts::bandGroupTypeBeforePrint(
      TQRCustomBand *Sender, bool &PrintBand)
{
 if (show_detail_rows &&
     (textTaxEntityName->DataSet->FieldByName("bill_id")->AsString != "" ||
      textTaxEntityName->DataSet->FieldByName("task_id")->AsString != ""))
 {
  PrintBand = true;

  string services_provided_id = textTaxEntityName->DataSet->FieldByName("services_provided_id")->AsString.c_str();
  if (services_provided_id == "")
  {
   labelItemType->Caption = "Bill";
   labelInvoiceNumber->Caption   = "Invoice Number";
   labelPatientName->Caption     = "";
   labelInvoiceNote->Caption     = "Invoice Note";
   labelTaskProvider->Caption    = "";
   labelServiceProvider->Caption = "";
   labelServiceName->Caption     = "";
   labelPRNumber->Caption        = "PR #";
   labelPRDate->Caption          = "PR Date";
   labelCheckNumber->Caption     = "Chk. #";
   labelCheckDate->Caption       = "Chk. Date";
   labelAreaName->Caption        = "";
  }
  else if (services_provided_id == "0")
  {
   labelItemType->Caption = "Visit";
   labelInvoiceNumber->Caption   = "";
   labelPatientName->Caption     = "Patient Name";
   labelInvoiceNote->Caption     = "";
   labelTaskProvider->Caption    = "Visit Provider Name";
   labelServiceProvider->Caption = "";
   labelServiceName->Caption     = "";
   labelPRNumber->Caption        = "";
   labelPRDate->Caption          = "";
   labelCheckNumber->Caption     = "";
   labelCheckDate->Caption       = "";
   labelAreaName->Caption        = "";
  }
  else
  {
   labelItemType->Caption = "Coord. of Benefits";
   labelInvoiceNumber->Caption   = "";
   labelPatientName->Caption     = "Patient Name";
   labelInvoiceNote->Caption     = "";
   labelTaskProvider->Caption    = "";
   labelServiceProvider->Caption = "Oblig. Provider Name";
   labelServiceName->Caption     = "Oblig. Service";
   labelPRNumber->Caption        = "";
   labelPRDate->Caption          = "";
   labelCheckNumber->Caption     = "";
   labelCheckDate->Caption       = "";
   labelAreaName->Caption        = "Oblig. Area";
  }
 }
 else
  PrintBand = false;
 return;
}
//---------------------------------------------------------------------------

void __fastcall TformReportPendingAccounts::bandDetailBeforePrint(
      TQRCustomBand *Sender, bool &PrintBand)
{
 if (show_detail_rows &&
     (textTaxEntityName->DataSet->FieldByName("bill_id")->AsString != "" ||
      textTaxEntityName->DataSet->FieldByName("task_id")->AsString != ""))
 {
  PrintBand = true;
  temp_date.InDate(textTaxEntityName->DataSet->FieldByName("item_date")->AsString.c_str(), mdy | yyyy);
  int num_days = today_date - temp_date;
  if (num_days == 1)
   labelDayCount->Caption = (IntToString(num_days) + " day ago").c_str();
  else
   labelDayCount->Caption = (IntToString(num_days) + " days ago").c_str();
 } else
  PrintBand = false;
}
//---------------------------------------------------------------------------

