//---------------------------------------------------------------------------

#include <vcl.h>
#pragma hdrstop

#include "reportPersonAudit.h"
#include "derivedheaders.h"
#include "codeSpecialHtmlPreview.h"
#include "html_reporting.h"
//---------------------------------------------------------------------------
#pragma package(smart_init)
#pragma link "codeSpecialGeneric"
#pragma resource "*.dfm"
TformReportPersonAudit *formReportPersonAudit;

/*
This report contains some experimentation. It was the first time I had to output stuff like this (other than HL7).
We have N people, and for each person, we want to show
- conditions
- procedures
- measurements
And for each of these three things, we actually have several sets of criteria, each different, and each restricted by count / subset (earliest, latest, etc.)
Guess what? QR doesn't like doing that, except with sub-selects (which were slow as fuck when I tried doing them.)

First attempt:
 Composite report of composite reports.
 Each composite report contained "people", "conditions", "procedures", "measurements" created by subselects
 The reports were then joined together.
 Took minutes, used subselects, wouldn't scale well to longer reports (many people.)

Second attempt:
 Paul had a method of creating a 'custom' report by attaching bands to each other via the ChildBand property.
 This creates (at runtime, you start with only a title band) a very long linked-list of bands.
 It works.
 It also took two hours to output 700 pages. But they were correct-ish.

Third method:
 Fuck QR, let's try HTML output.
 codeSpecialHtmlPreview is the result of this.
 CSS3 isn't available, so pagination is still painful.
 Page numbers are injected into reports by a hack against IE's print-preview, via the registry.
 (To know more about the hacks involved, go see the preview form. Things got ugly before they got better.)
 This version can output crap in seconds. That's what I'm talking about.
 Does require (at least at the moment -- could see about DOM-injection) a temp file.

What does using HTML mean?
 Well, for one thing, it means that in the future, we could change the appearance of all reports simultaneously by changing some CSS.
 But we have so many QR reports, the chances of this ever happening are slim.
 But it is -damned- fast, when needed. And IE's print-preview window is similar enough to QR's to not be confusing.
 Another feature is that IE's print-preview lets you set up the printer you want to send the report to. QR's only does for non-composite reports. Ouch.
 [Actually this isn't true.  I fixed Composite QR stupidness.  You were just doing it wrong. :-) --Paul]
 [Actually, it is true. I had to undo your changes to Composite reports because QR is still buggy and hates your code. We did it the ugly but *working* way, you did it the pretty but *crashy* way. Newer versions of QR may fix the whole printer thing, but your code still crashes. So sorry, but you may not. =) --Philip]
 But ... HTML isn't meant for reports. Per-page stuff is difficult/impossible to do. Headers and Footers are a stretch as it is.
 And you have to know HTML.
 And you have to deal with HTML hacks.
 And you have to deal with IE.
 And there's the issue of centralizing functions for doing HTML chunks, CSS, and ... flexibility across types of reports.
 And ... you get the idea.

Ongoing issue(s):
 What shall we do about these reports being *forms*? It's kinda stupid, neh?
 We don't need the form for anything, though a class (or two) is handy.
 Let's ... think on that at some point.

 Also to consider: making this report 'plugin'-capable. Sure, conditions, procedures, measurements are nice ... as chunks.
 But the report format lends itself nicely to having random chunks added (maybe several of each) at random.

 Also -- what about images, graphs, and so forth? If we can output images, we can drop them into the folder with the html.
 Some form of graphics embedded in the html would be better though (one-file-report) ... and generating images isn't easy.
 Might take 'gd' (used by php) and integrate it into P? (Note: likely a GPL violation if we do? Need to check into it.)
*/

//---------------------------------------------------------------------------
__fastcall TformReportPersonAudit::TformReportPersonAudit(TComponent* Owner)
        : TformSpecialGeneric(Owner)
{
}
//---------------------------------------------------------------------------
void TformReportPersonAudit::Do (TIBTransaction * dbTransaction, string source_view, xValue & xoptions)
{
 FORM_TRY
  TformReportPersonAudit* z = new TformReportPersonAudit(NULL);
  z->Run(dbTransaction, source_view, xoptions);
 FORM_CATCH
}
//---------------------------------------------------------------------------
person_audit_report_options::_code_category::_code_category(xValue init)
{
 if (xTag(init) == "map")
 {
  map<string, xValue> config_map = xToMap(init);
  if (xTag(config_map["category"]) == "string")
   category = xToString(config_map["category"]);
  if (xTag(config_map["codes"]) == "code_matches")
   codes = config_map["codes"]; // keep intact for later
  if (xTag(config_map["subset"]) == "int")
   type = xToInt(config_map["subset"]);
  if (xTag(config_map["count"]) == "int")
   count = xToInt(config_map["count"]);
 }
}
//---------------------------------------------------------------------------
person_audit_report_options::_meas_desired::_meas_desired(xValue init)
{
 if (xTag(init) == "map")
 {
  map<string, xValue> config_map = xToMap(init);
//  if (xTag(config_map["meas"]) == "string")
//   meas_field = xToString(config_map["meas"]);
  if (xTag(config_map["id"]) == "string")
   obs_id = xToString(config_map["id"]);
  if (xTag(config_map["subset"]) == "int")
   type = xToInt(config_map["subset"]); // mds_none shouldn't happen, as we exclude those when saving, but ... hope for the best?
  if (xTag(config_map["count"]) == "int")
   count = xToInt(config_map["count"]);
 }
}
//---------------------------------------------------------------------------
person_audit_report_options::person_audit_report_options(xValue & xoptions)
{
 if (xTag(xoptions) == "map")
 {
  map<string, xValue> config_map = xToMap(xoptions);

  if (xTag(config_map["person_chart_location_id"]) == "string")
   person.chart_location_id = xToString(config_map["person_chart_location_id"]);

  // lazy!
  if (person.chart_location_id.empty() || !IsInt(person.chart_location_id))
   person.chart_location_id = "0";

  if (xTag(config_map["conditions"]) == "vector")
  {
   vector<xValue> conditions_vec = xToVec(config_map["conditions"]);
   for (vector<xValue>::iterator i = conditions_vec.begin(); i != conditions_vec.end(); i++)
    conditions.requests.push_back(_code_category(*i));
  }

  if (xTag(config_map["procedures"]) == "vector")
  {
   vector<xValue> procedures_vec = xToVec(config_map["procedures"]);
   for (vector<xValue>::iterator i = procedures_vec.begin(); i != procedures_vec.end(); i++)
    procedures.requests.push_back(_code_category(*i));
  }

  if (xTag(config_map["measurements"]) == "vector")
  {
   vector<xValue> measurements_vec = xToVec(config_map["measurements"]);
   for (vector<xValue>::iterator i = measurements_vec.begin(); i != measurements_vec.end(); i++)
    measurements.requests.push_back(_meas_desired(*i));
  }
 }
}
//---------------------------------------------------------------------------
easy_streaming_dataset* person_audit_report_options::_code_category::create_condition_stream(TIBTransaction* dbTransaction, string source_view)
{
 // from recordPerson
 // TODO: "Third Party Resources" and "MIS" have a few conditions records of their own (a couple dozen) !!!
 vector<string> allowed_groups;
 if (optionsPermissions["table.conditions.common"]["view"] == "true")
   allowed_groups.push_back("0");
 if (optionsPermissions["table.conditions.diab"]["view"] == "true")
   allowed_groups.push_back(lookupTables->IdOf("groups", "Diabetes"));
 if (optionsPermissions["table.conditions.beh"]["view"] == "true")
   allowed_groups.push_back(lookupTables->IdOf("groups", "Behavioral Health"));
 if (optionsPermissions["table.conditions.com"]["view"] == "true")
   allowed_groups.push_back(lookupTables->IdOf("groups", "Community Health"));
 if (optionsPermissions["table.conditions.lac"]["view"] == "true")
   allowed_groups.push_back(lookupTables->IdOf("groups", "Little Axe Clinic"));
 if (optionsPermissions["table.conditions.den"]["view"] == "true")
   allowed_groups.push_back(lookupTables->IdOf("groups", "Dental Clinic"));
 if (optionsPermissions["table.conditions.cob"]["view"] == "true")
   allowed_groups.push_back(lookupTables->IdOf("groups", "Contract Health"));
 if (optionsPermissions["table.conditions.pub"]["view"] == "true")
   allowed_groups.push_back(lookupTables->IdOf("groups", "Public Health"));
 // TODO: possibly dangerous ... members of a group might need stuff taken away from them, even from their own group?
 allowed_groups.push_back(current_groupid);

 string sql = "select * from uno where 1 = 0";
 if (!allowed_groups.empty())
  sql = "select conditions.person_id, "
        "'" + ReplaceStringWithString(category, "'", "''") + "' cat, "
        "conditions.start_date, "
        "conditions.code, "
        "codes.description note, "
        "groups.short_item grp_name, "
        "case when task_id != 0 then 1 else 0 end is_diagnosed "
        "from conditions "
        "inner join " + source_view + " on " + source_view + ".id = conditions.person_id "
        "left join codes on codes.code = conditions.code and codes.code_id = 3 " // TODO: currently, ICD-9 is '3', and is the only 'condition' type
        "left join groups on groups.id = conditions.group_id "
        "where " + xToCodeMatchingSql(codes, "conditions.code") + " "
        "and conditions.current_id + 1 > 3 "
        "and conditions.condition_id != 7 " /* 'relative' */
        "and conditions.group_id in (" + VecToCommaString(allowed_groups) + ") "
        "order by conditions.person_id, conditions.start_date desc";
 return new easy_streaming_dataset(dbTransaction, sql);
}
//---------------------------------------------------------------------------
easy_streaming_dataset* person_audit_report_options::_code_category::create_procedure_stream(TIBTransaction* dbTransaction, string source_view)
{
 string sql = "select tasks.person_id, "
              "'" + ReplaceStringWithString(category, "'", "''") + "' cat, "
              "procedures.start_date, "
              "procedures.cpt_hcpcs, "
              "codes.description note, "
              "groups.short_item grp_name "
              "from procedures inner join tasks on tasks.id = procedures.task_id "
              "inner join " + source_view + " on " + source_view + ".id = tasks.person_id "
              "left join codes on codes.code = procedures.cpt_hcpcs and codes.code_id != 3 " // TODO: currently, ICD-9 is '3', and is the only 'condition' type
              "left join groups on groups.id = tasks.group_id "
              "where " + xToCodeMatchingSql (codes, "procedures.cpt_hcpcs") + " "
              "and procedures.cpt_hcpcs != '0' "
              "and procedures.current_id + 1 > 3 "
              "order by tasks.person_id, procedures.start_date desc";
 return new easy_streaming_dataset(dbTransaction, sql);
}
//---------------------------------------------------------------------------
easy_streaming_dataset* person_audit_report_options::_meas_desired::create_measurement_stream(TIBTransaction* dbTransaction, string source_view)
{
 // from recordPerson
 // TODO -- some of these permissions don't actually exist (yet), just used the ones from conditions (superset)
 // TODO -- change these to obs_response_sets.* permissions?
 vector<string> allowed_groups;
 if (optionsPermissions["table.measurements.common"]["view"] == "true")
   allowed_groups.push_back("0");
 if (optionsPermissions["table.measurements.diab"]["view"] == "true")
   allowed_groups.push_back(lookupTables->IdOf("groups", "Diabetes"));
 if (optionsPermissions["table.measurements.beh"]["view"] == "true")
   allowed_groups.push_back(lookupTables->IdOf("groups", "Behavioral Health"));
 if (optionsPermissions["table.measurements.com"]["view"] == "true")
   allowed_groups.push_back(lookupTables->IdOf("groups", "Community Health"));
 if (optionsPermissions["table.measurements.lac"]["view"] == "true")
   allowed_groups.push_back(lookupTables->IdOf("groups", "Little Axe Clinic"));
 if (optionsPermissions["table.measurements.den"]["view"] == "true")
   allowed_groups.push_back(lookupTables->IdOf("groups", "Dental Clinic"));
 if (optionsPermissions["table.measurements.cob"]["view"] == "true")
   allowed_groups.push_back(lookupTables->IdOf("groups", "Contract Health"));
 if (optionsPermissions["table.measurements.pub"]["view"] == "true")
   allowed_groups.push_back(lookupTables->IdOf("groups", "Public Health"));
 // TODO: possibly dangerous ... members of a group might need stuff taken away from them, even from their own group?
 allowed_groups.push_back(current_groupid);

 string sql = "select * from uno where 1 = 0";
/*
 if (!allowed_groups.empty())
  sql = "select measurements.person_id, "
        "'" + ReplaceStringWithString(meas_field, "'", "''") + "' cat, "
        "measurements.date_of_measurement, "
        "measurements." + meas_field + " obs_val, "
        "groups.short_item grp_name "
        "from measurements "
        "inner join " + source_view + " on " + source_view + ".id = measurements.person_id "
        "left join groups on groups.id = measurements.group_id "
        "where measurements." + meas_field + " is not null "
        "and measurements." + meas_field + " != 0 " // this doesn't take care of out-of-bounds! cleaned-up data is required for that ...
        "and measurements.current_id + 1 > 3 "
        "and measurements.group_id in (" + VecToCommaString(allowed_groups) + ") "
        "order by measurements.person_id, measurements.date_of_measurement desc";
*/
 if (!allowed_groups.empty())
  sql = "select "
         "obs_response_sets.person_id, "
         "obs_questions.item cat, "
         "obs_response_sets.date_of_observation, "
         "obs_questions.obs_question_id obs_question_type_id, "
         "obs_questions.obs_option_set_id, obs_option_types.item obs_ord_item, obs_option_types.rank obs_ord_val, "
         "obs_responses.response_text obs_txt_val, "
         "obs_responses.response_date obs_date_val, "
         "obs_responses.response_metric obs_met_val, units.item obs_met_unit, "
         "groups.short_item grp_name "
        "from obs_response_sets "
         "inner join " + source_view + " on " + source_view + ".id = obs_response_sets.person_id "
         "left join obs_responses on obs_responses.obs_response_set_id = obs_response_sets.id "
         "left join groups on groups.id = obs_response_sets.group_id "
         "left join obs_questions on obs_questions.id = obs_responses.obs_question_id "
         "left join obs_option_types on obs_option_types.id = obs_responses.obs_option_id "
         "left join units on units.id = obs_questions.unit_id "
        "where "
         "obs_responses.obs_question_id = " + obs_id + " "
         "and obs_response_sets.current_id + 1 > 3 "
         "and obs_response_sets.obs_question_set_id in "
         "(select oqs.id from obs_question_set_types oqs "
          "inner join permissions p on p.s = oqs.permission_name and p.k = 'VIEW' and p.v = 'TRUE' and p.user_id = " + current_userid + ") "
         "and (1 = 0 or obs_response_sets.group_id in (" + VecToCommaString(allowed_groups) + ")) "
        "order by "
         "obs_response_sets.person_id, obs_response_sets.date_of_observation desc";
 return new easy_streaming_dataset(dbTransaction, sql);
}
//---------------------------------------------------------------------------
void person_audit_report_options::_code_category::slice_up(list<map<string, string> > & all_items)
{
 if (type == ccs_all)
  return;

 int c = 0;
 list<map<string, string> > temp_copy;
 if (type == ccs_earliest)
  for (list<map<string, string> >::iterator i = all_items.begin(); i != all_items.end() && c < count; i++, c++)
   temp_copy.push_back(*i);

 if (type == ccs_latest)
  for (list<map<string, string> >::reverse_iterator i = all_items.rbegin(); i != all_items.rend() && c < count; i++, c++)
   temp_copy.push_front(*i);

 all_items = temp_copy;
}
//---------------------------------------------------------------------------
void person_audit_report_options::_meas_desired::slice_up(list<map<string, string> > & all_items)
{
 if (type == mds_all)
  return;

 if (type == mds_none)
 {
  all_items.clear();
  return;
 }

 if (type == mds_latest || type == mds_latest_avg)
 {
  int c = 0;
  list<map<string, string> > temp_copy;
  for (list<map<string, string> >::iterator i = all_items.begin(); i != all_items.end() && c < count; i++, c++)
   temp_copy.push_back(*i);
  all_items = temp_copy;
 }

 if (!all_items.empty() && (type == mds_all_avg || type == mds_latest_avg))
 {
  map<string, string> temp_copy;
  financial z = financial(0.00);
  unsigned int c = 0;

  temp_copy["person_id"]           = all_items.front()["person_id"];
  temp_copy["obs_question_type_id"]= all_items.front()["obs_question_type_id"];
  temp_copy["cat"]                 = all_items.front()["cat"];
  temp_copy["date_of_observation"] = "Average"; // date_of_measurement is oddly blank for averages, eh? "weird" ... what will I think of next?
  temp_copy["grp_name"]            = "";

  // TODO: -- uh ... what about "5": dates? can ... can we average dates? is that really a good idea?
  if (all_items.front()["obs_question_type_id"] == "4") // TODO -c MAGIC: metric
  {
   for (list<map<string, string> >::iterator i = all_items.begin(); i != all_items.end(); i++, c++)
    z += StringToFinancial((*i)["obs_met_val"]);
   if (c)
    z /= financial(c); // c shouldn't be able to be zero ... we -do- check for that up-front. really. i swear.
   else
    z = financial(0.00);
   temp_copy["obs_met_val"]         = FinancialToString(z);
   temp_copy["obs_met_unit"]        = all_items.front()["obs_met_unit"];
  } else
  if (all_items.front()["obs_question_type_id"] == "3") // TODO -c MAGIC: ordinal
  {
   for (list<map<string, string> >::iterator i = all_items.begin(); i != all_items.end(); i++, c++)
    z += StringToFinancial((*i)["obs_ord_val"]);
   if (c)
    z /= financial(c); // c shouldn't be able to be zero ... we -do- check for that up-front. really. i swear.
   else
    z = financial(0.00);
   int o = floorl(z.AsLongDouble() + 0.50);
   temp_copy["obs_ord_val"] = IntToString(o);
   temp_copy["obs_ord_item"] = GetOneField(dbDatabase, "select first 1 item from obs_option_types where obs_option_set_id = " + all_items.front()["obs_option_set_id"] + " and rank >= " + IntToString(o));
  }

  all_items.clear();
  all_items.push_front(temp_copy);
 }
}
//---------------------------------------------------------------------------
void TformReportPersonAudit::Run (TIBTransaction * dbTransaction, string source_view, xValue & xoptions)
{
 Processing(true);

 options = xoptions;

/*
 FORM_TRY
  dbTransaction = new TIBTransaction(NULL);
 FORM_CATCH
*/

 try
 {
//   StdTransaction(dbTransaction, dbDatabase);

   map<string, string> person_row;
   list<map<string, string> > condition_rows;
   list<map<string, string> > condition_rows_temp;
   list<map<string, string> > procedure_rows;
   list<map<string, string> > procedure_rows_temp;
   list<map<string, string> > measurement_rows;
   list<map<string, string> > measurement_rows_temp;

   // for those who don't get it -- i was raised in france. 'ze' makes fun of the french trying to say 'the' ... *sigh*
   ofstream ze_file;
   ze_file.open("audit.html", ios::trunc | ios::out);
   if (!ze_file.is_open())
   {
    PSay("Failed to create temporary file for report output.", m_debug, MB_OK);
    throw(97);
   }
   htmlStart(ze_file, "Audit Report");

   if ((optionsPermissions["table.people"]["view"] == "true"))
   {
    set<string> can_see_charts = GetSpecialGroupPermissions("charts");
    string people_sql = "select people.id, "
                        "people.social_security_number, people.birth_date, people.death_date, people.gender_id, "
                        "person_names.last_name || ', ' || person_names.first_name name, "
                        "person_contact_info.line1, person_contact_info.line2, "
                        "person_contact_info.city, state_types.abbreviation, person_contact_info.zip, "
                        "person_contact_info.phone_number, "
                        "(select first 1 charts.chart_number from charts "
                        "inner join groups on charts.location_id = groups.chart_location_id and groups.has_charts = 1 and charts.location_id > 0 "
                        "where charts.person_id = people.id and " + SafeInClause("groups.id", SetToVec(can_see_charts)) +
                        "and charts.current_id > 2 order by groups.hl7_priority asc nulls last) chart_number, "
                        "'" + options.person.chart_location_id + "' location_id "
                        "from people "
                        "left join person_names on person_names.id = people.person_names_id "
                        "left join person_contact_info on person_contact_info.id = people.person_contact_info_id "
                        "left join state_types on state_types.id = person_contact_info.state_id "
                        "inner join " + source_view + " on " + source_view + ".id = people.id "
                        "order by people.id";
    easy_streaming_dataset person_stream (dbTransaction, people_sql);

    vector<easy_streaming_dataset*> condition_streams;
    if (optionsPermissions["table.conditions"]["view"] == "true")
     for(vector<person_audit_report_options::_code_category>::iterator i = options.conditions.requests.begin(); i != options.conditions.requests.end(); i++)
      condition_streams.push_back(i->create_condition_stream(dbTransaction, source_view));

    vector<easy_streaming_dataset*> procedure_streams;
    if(optionsPermissions["table.procedures"]["view"] == "true")
     for(vector<person_audit_report_options::_code_category>::iterator i = options.procedures.requests.begin(); i != options.procedures.requests.end(); i++)
      procedure_streams.push_back(i->create_procedure_stream(dbTransaction, source_view));

    vector<easy_streaming_dataset*> measurement_streams;
    if(optionsPermissions["table.obs_response_sets"]["view"] == "true")
     for(vector<person_audit_report_options::_meas_desired>::iterator i = options.measurements.requests.begin(); i != options.measurements.requests.end(); i++)
      measurement_streams.push_back(i->create_measurement_stream(dbTransaction, source_view));

    unsigned int person_count = 0;
    while (person_stream.GetRow(person_row))
    {
     htmlAddPerson(ze_file, person_row, (person_count != 0)); // last parameter is 'new page'

     // build in-memory lists

     condition_rows.clear();
     unsigned int j = 0;
     for (vector<easy_streaming_dataset*>::iterator i = condition_streams.begin(); i != condition_streams.end(); i++, j++)
     {
      (*i)->AdvanceWhileMap(streaming_dataset_map_value_predicate<less_strings_as_ints>("person_id", less_strings_as_ints(), person_row["id"]));
      (*i)->GetRowsWhile(condition_rows_temp, streaming_dataset_map_value_predicate<equal_to_strings_as_ints>("person_id", equal_to_strings_as_ints(), person_row["id"]));
      options.conditions.requests[j].slice_up(condition_rows_temp);
      condition_rows.insert(condition_rows.end(), condition_rows_temp.begin(), condition_rows_temp.end());
     }
     if (!condition_streams.empty())
      htmlAddConditions(ze_file, condition_rows);

     procedure_rows.clear();
     j = 0;
     for (vector<easy_streaming_dataset*>::iterator i = procedure_streams.begin(); i != procedure_streams.end(); i++, j++)
     {
      (*i)->AdvanceWhileMap(streaming_dataset_map_value_predicate<less_strings_as_ints>("person_id", less_strings_as_ints(), person_row["id"]));
      (*i)->GetRowsWhile(procedure_rows_temp, streaming_dataset_map_value_predicate<equal_to_strings_as_ints>("person_id", equal_to_strings_as_ints(), person_row["id"]));
      options.procedures.requests[j].slice_up(procedure_rows_temp);
      procedure_rows.insert(procedure_rows.end(), procedure_rows_temp.begin(), procedure_rows_temp.end());
     }
     if (!procedure_streams.empty())
      htmlAddProcedures(ze_file, procedure_rows);

     measurement_rows.clear();
     j = 0;
     for (vector<easy_streaming_dataset*>::iterator i = measurement_streams.begin(); i != measurement_streams.end(); i++, j++)
     {
      (*i)->AdvanceWhileMap(streaming_dataset_map_value_predicate<less_strings_as_ints>("person_id", less_strings_as_ints(), person_row["id"]));
      (*i)->GetRowsWhile(measurement_rows_temp, streaming_dataset_map_value_predicate<equal_to_strings_as_ints>("person_id", equal_to_strings_as_ints(), person_row["id"]));
      options.measurements.requests[j].slice_up(measurement_rows_temp);
      measurement_rows.insert(measurement_rows.end(), measurement_rows_temp.begin(), measurement_rows_temp.end());
     }
     if (!measurement_streams.empty())
      htmlAddMeasurements(ze_file, measurement_rows);

     htmlSectionStop(ze_file);

     person_stream.Next();
     person_count++;
    }

    for (vector<easy_streaming_dataset*>::iterator i = condition_streams.begin(); i != condition_streams.end(); i++)
     delete *i;
    for (vector<easy_streaming_dataset*>::iterator i = procedure_streams.begin(); i != procedure_streams.end(); i++)
     delete *i;
    for (vector<easy_streaming_dataset*>::iterator i = measurement_streams.begin(); i != measurement_streams.end(); i++)
     delete *i;
   }

   htmlStop(ze_file);
   ze_file.close();

   // apparently, we have to pass the fully-qualified path ... damn. might move this into the DoPrintPreview function? (always gonna be necessary? except not? i dunno. it's not much extra trouble here.)
   string real_cwd = GetCurrentDir().c_str();
   Processing(false);
   TformSpecialHtmlPreview::DoPrintPreview(real_cwd + "\\audit.html");
   // dbTransaction->Commit();
   this->CloseNow();
 }
 catch (...)
 {
   Processing(false);
   PSay("Error running audit report", m_database_error, MB_OK);
   // if (dbTransaction->Active)
   //   dbTransaction->Rollback();
 }

// delete dbTransaction;
}
//---------------------------------------------------------------------------
void TformReportPersonAudit::htmlAddPerson(ostream & o, map<string, string> person_row, bool new_page)
{
 htmlSectionStart(o, new_page);
 htmlPrettyBar(o, PrettyString(person_row["name"]));
 htmlSectionContinue(o);

 o << "<table cellspacing=0 width=100%>"
      "<tr>"
      "<td width=50% valign=top>"
       "<table cellspacing=0 width=100%>";
       // removed - it's in the header now
       // "<tr><td style='padding-right:8px;'><i>Name</i></td><td width=100%>" << PrettyString(person_row["name"]) << "</td></tr>"
       if (optionsPermissions["table.person_contact_info"]["view"] == "true")
       {
        o << "<tr><td valign=top><i>Address</i></td><td width=100%>"
          << FormatAddress(person_row["line1"]) << "<br>";
        if (!person_row["line2"].empty())
         o << FormatAddress(person_row["line2"]) << "<br>";
        o << PrettyString(person_row["city"]) << "<br>"
          << person_row["abbreviation"];
        if (!person_row["abbreviation"].empty() && !person_row["zip"].empty())
         o << ", " << person_row["zip"];
        else
         if (!person_row["abbreviation"].empty())
          o << person_row["zip"];
        o << "</td></tr>"
             "<tr><td><i>Phone</i></td><td width=100%>"
          << FormatPhoneNumber(person_row["phone_number"])
          << "</td></tr>";
       }
  o << "</table>"
      "</td>"
      "<td width=50% valign=top>"
       "<table cellspacing=0 width=100%>"
       "<tr><td><i>SSN</i></td><td width=100%>"
        << PrettySSN(person_row["social_security_number"])
        << "</td></tr>"
       "<tr><td><i>DOB</i></td><td width=100%>"
        << person_row["birth_date"];
       if (!person_row["birth_date"].empty())
        o << " ("
          << CalcAge(Pdate(person_row["birth_date"], global_mode))
          << ")";
  o << "</td></tr>"
       "<tr><td><i>Gender</i></td><td>"
    << lookupTables->ValueOf("gender_types", person_row["gender_id"])
    << "</td></tr>";
       if (optionsPermissions["table.charts"]["view"] == "true")
       {
        o << "<tr><td><i>Chart</i></td><td>"
          << person_row["chart_number"];
        if (!person_row["chart_number"].empty())
         o << " (" << lookupTables->ValueOf("groups", person_row["location_id"]) << ")"; // TODO: should this pull from location_types?  I'm not sure what person.chart_location_id comes from
        o << "</td></tr>";
       }
  o << "</table>"
      "</td>"
      "</tr>"
      "</table>";

 // historical --
 // now, this is vexing. i'd like to have the htmlSectionStop() here, but
 // - we'd have to add conditions, procedures, measurements in this function (and pass them to this function)
 // - we only conditionally call each of those functions, based on whether or not there are requests for them at all
 // - we'd have to pass the entire set of request structures into here to do the above
 // ... so oddly, it's much easier to just call htmlSectionStop() from the function calling us. funky, eh?
}
void TformReportPersonAudit::htmlAddConditions(ostream & o, list<map<string, string> > condition_rows)
{
 htmlSectionStart(o, false);
 htmlPrettyBar(o, "Conditions");
 htmlSectionContinue(o);
 o << "<table cellspacing=0 width=100% class='datatable' style='margin-left: 24px;'>"
      "<thead><tr>"
       "<td><i>Category</i></td>"
       "<td><i>Date</i></td>"
       "<td><i>Code</i></td>"
       "<td><i>Dept.</i></td>"
       "<td><i>Desc.</i></td>"
      "</tr></thead><tbody>";

 for (list<map<string, string> >::iterator i = condition_rows.begin(); i != condition_rows.end(); i++)
 {
  bool is_diag = ((*i)["is_diagnosed"] != "0");
  if (!is_diag)
   (*i)["note"] = "Unofficial: " + (*i)["note"];

  o << "<tr>"
        "<td" << (is_diag ? "" : " style='color: lightgrey;'") << ">" << (*i)["cat"]                    << "</td>"
        "<td" << (is_diag ? "" : " style='color: lightgrey;'") << ">" << PrettyDate((*i)["start_date"]) << "</td>"
        "<td" << (is_diag ? "" : " style='color: lightgrey;'") << ">" << (*i)["code"]                   << "</td>"
        "<td" << (is_diag ? "" : " style='color: lightgrey;'") << ">" << PrettyString((*i)["grp_name"]) << "</td>"
        "<td" << (is_diag ? "" : " style='color: lightgrey;'") << ">" << PrettyString((*i)["note"])     << "</td>"
       "</tr>\n";
 }

 if (condition_rows.empty())
  o << "<tr>"
        "<td colspan=5><b>None</b></td>"
       "</tr>";

 o << "</tbody></table>\n";
 htmlSectionStop(o);
}
void TformReportPersonAudit::htmlAddProcedures(ostream & o, list<map<string, string> > procedure_rows)
{
 htmlSectionStart(o, false);
 htmlPrettyBar(o, "Procedures");
 htmlSectionContinue(o);
 o << "<table cellspacing=0 width=100% class='datatable' style='margin-left: 24px;'>"
      "<thead><tr>"
       "<td><i>Category</i></td>"
       "<td><i>Date</i></td>"
       "<td><i>Code</i></td>" // damn browser not supporting max-width correctly ... this'll seem wrong if they put long comments in the code field -- tried width:expression("121px") ... could give width:expression(true ? "121px" : "121px"), i suppose. (css, that is. IE-only feature.)
       "<td><i>Dept.</i></td>" // damn browser not supporting max-width correctly ... this'll seem wrong if they put long comments in the code field -- tried width:expression("121px") ... could give width:expression(true ? "121px" : "121px"), i suppose. (css, that is. IE-only feature.)
       "<td><i>Desc.</i></td>"
      "</tr></thead><tbody>";

 for (list<map<string, string> >::iterator i = procedure_rows.begin(); i != procedure_rows.end(); i++)
 {
  o << "<tr>"
        "<td>" << (*i)["cat"]                    << "</td>"
        "<td>" << PrettyDate((*i)["start_date"]) << "</td>"
        "<td>" << (*i)["cpt_hcpcs"]              << "</td>"
        "<td>" << PrettyString((*i)["grp_name"]) << "</td>"
        "<td>" << PrettyString((*i)["note"])     << "</td>"
       "</tr>\n";
 }

 if (procedure_rows.empty())
  o << "<tr>"
        "<td colspan=5><b>None</b></td>"
       "</tr>";

 o << "</tbody></table>\n";
 htmlSectionStop(o);
}
void TformReportPersonAudit::htmlAddMeasurements(ostream & o, list<map<string, string> > measurement_rows)
{
 htmlSectionStart(o, false);
 htmlPrettyBar(o, "Observations");
 htmlSectionContinue(o);
 o << "<table cellspacing=0 class='datatable' style='margin-left: 24px;'>"
      "<thead><tr>"
       "<td><i>Metric</i></td>"
       "<td><i>Date</i></td>"
       "<td colspan=2 align=right><i>Value</i></td>"
       "<td><i>Dept.</i></td>"
      "</tr></thead><tbody>";

 for (list<map<string, string> >::iterator i = measurement_rows.begin(); i != measurement_rows.end(); i++)
 {
  o << "<tr>"
        "<td>" << (*i)["cat"]                             << "</td>"
        "<td>" << PrettyDate((*i)["date_of_observation"]) << "</td>";

  if ((*i)["obs_question_type_id"] == "5") // TODO -c MAGIC: date
  {
   o << "<td colspan=2>" << (*i)["obs_date_val"] << "</td>";
  } else
  if ((*i)["obs_question_type_id"] == "4") // TODO -c MAGIC: metric
  {
   o << "<td align=right>" << StripTrailing(StripTrailing(FinancialToString(StringToFinancial((*i)["obs_met_val"])), "0"), ".") << "</td>"
        "<td>" << (*i)["obs_met_unit"] << "</td>";
  } else
  if ((*i)["obs_question_type_id"] == "3") // TODO -c MAGIC: ordinal
  {
   o << "<td colspan=2>" << (*i)["obs_ord_item"] << "</td>";
  } else
  if ((*i)["obs_question_type_id"] == "2") // TODO -c MAGIC: text
  {
   if ((*i)["date_of_observation"] == "Average")
    o << "<td colspan=2>Incalculable</td>";
   else
    o << "<td colspan=2>" << (*i)["obs_txt_val"] << "</td>";
  } else
  if ((*i)["obs_question_type_id"] == "1") // TODO -c MAGIC: choice
  {
   if ((*i)["date_of_observation"] == "Average")
    o << "<td colspan=2>Incalculable</td>";
   else
    o << "<td colspan=2>" << (*i)["obs_ord_item"] << "</td>";
  } else
   o << "<td colspan=2>Unknown question type</td>";
  o << "<td>" << PrettyString((*i)["grp_name"]) << "</td>"
       "</tr>\n";
 }

 if (measurement_rows.empty())
  o << "<tr>"
        "<td colspan=4><b>None</b></td>"
       "</tr>";

 o << "</tbody></table>\n";
 htmlSectionStop(o);
}

