//---------------------------------------------------------------------------

#include <vcl.h>
#pragma hdrstop

#include "reportYearlyMeasurementSummary.h"
#include "html_reporting.h"
#include "codeSpecialHtmlPreview.h"
#include "derivedheaders.h"
//---------------------------------------------------------------------------
#pragma package(smart_init)
#pragma link "codeSpecialGeneric"
#pragma resource "*.dfm"
TformReportYearlyMeasurementSummary *formReportYearlyMeasurementSummary;
//---------------------------------------------------------------------------
__fastcall TformReportYearlyMeasurementSummary::TformReportYearlyMeasurementSummary(TComponent* Owner)
        : TformSpecialGeneric(Owner)
{
}
//---------------------------------------------------------------------------

struct obs_stream
{
 string q_id;
 string name;
 string obs_type;
 string obs_opt_set_id;
 easy_streaming_dataset * data_stream;
 yms_methods m;
 TIBTransaction * given_tx;

 obs_stream(TIBTransaction * dbTransaction, string question_id, string source_view, string where_clause, yms_methods method)
 {
  given_tx = dbTransaction;
  q_id = question_id;
  map<string, string> t;
  SqlToMap(dbTransaction, "select item, obs_question_id, obs_option_set_id from obs_questions where id = " + q_id, t);
  name = t["item"];
  obs_type = t["obs_question_id"];
  obs_opt_set_id = t["obs_option_set_id"];
  m = method;

  // go!
  string obs_field;
  if (obs_type == "4")
   obs_field = "response_metric";
  else if (obs_type == "3")
   obs_field = "rank";

  string sql = "select extract(year from date_of_observation) meas_year, ";
  if (m == yms_average)
   sql += "avg(" + obs_field + ")";
  else
   sql += obs_field;
  sql += " meas_val "
         "from obs_responses "
         "inner join obs_response_sets on obs_responses.obs_response_set_id = obs_response_sets.id "
         "and obs_responses.obs_question_id = " + q_id + " "
         "left join " + source_view + " on " + source_view + ".id = obs_response_sets.person_id "
         "left join obs_option_types on obs_option_types.id = obs_responses.obs_option_id "
         "where " + where_clause +
         "and " + source_view + ".id is not null ";
  if (m == yms_average) // or any aggregate operator
   sql += "group by extract(year from date_of_observation) ";
  sql += "order by extract(year from date_of_observation) desc";
  data_stream = new easy_streaming_dataset(dbTransaction, sql);
 }

 ~obs_stream()
 {
  delete data_stream;
 }

 string get_year_data(string year)
 {
  data_stream->AdvanceWhileMap(streaming_dataset_map_value_predicate<great_strings_as_ints>("meas_year", great_strings_as_ints(), year));
  map<string, string> temp_row;
  vector<financial> all_meas;
  while (data_stream->GetRow(temp_row) && temp_row["meas_year"] == year)
  {
   all_meas.push_back(StringToFinancial(temp_row["meas_val"]));
   data_stream->Next();
  }
  if (all_meas.empty())
   return "";

  // this "works" for averages too (1-element array, will take the first element (1/2=0))
  sort(all_meas.begin(), all_meas.end());
  unsigned c = all_meas.size();
  financial r;
  if (c % 2) // odd, grab
  {
   c /= 2;
   r = all_meas[c];
  } else // even, interpolate
  {
   c /= 2;
   r = (all_meas[c] + all_meas[c - 1]) / financial(2.00);
  }

  // output would be nice!
  if (obs_type == "4") // metric
  {
   return FinancialToString(r, false, 1);
  } else
  if (obs_type == "3") // ordinal
  {
   // rounding!
   // string b = FinancialToString(r, false, 1);
   int n = (r.AsLongDouble() + 0.50);
   // find nearest option?
   return GetOneField(given_tx, "select first 1 item from obs_option_types where obs_option_set_id = " +  obs_opt_set_id + " and rank >= " + IntToString(n) + " order by rank asc"); // + " (" + b + ")";
  } else
   return "unsupported";
 }
};



void TformReportYearlyMeasurementSummary::Run (TIBTransaction * dbTransaction, string source_view, const vector<string> & obs_question_ids, const string & optional_date_start, const string & optional_date_stop, yms_methods method)
{
 Processing(true);
 try
 {
   ofstream ze_file;
   ze_file.open("yms.html", ios::trunc | ios::out);
   if (!ze_file.is_open())
   {
    PSay("Failed to create temporary file for report output.", m_debug, MB_OK);
    throw(97);
   }
   htmlStart(ze_file, "Yearly Measurement Summary", (method == yms_average) ? "Averaged (mean)" : "Averaged (median)");

   if ((optionsPermissions["table.people"]["view"] == "true"))
   {
    // TODO -- some of these permissions don't actually exist (yet), just used the ones from conditions (superset)
    // TODO -- gotta convert these to obs permissions?
    vector<string> allowed_groups;
    if (optionsPermissions["table.measurements.common"]["view"] == "true")
      allowed_groups.push_back("0");
    if (optionsPermissions["table.measurements.diab"]["view"] == "true")
      allowed_groups.push_back(lookupTables->IdOf("groups", "Diabetes"));
    if (optionsPermissions["table.measurements.beh"]["view"] == "true")
      allowed_groups.push_back(lookupTables->IdOf("groups", "Behavioral Health"));
    if (optionsPermissions["table.measurements.com"]["view"] == "true")
      allowed_groups.push_back(lookupTables->IdOf("groups", "Community Health"));
    if (optionsPermissions["table.measurements.lac"]["view"] == "true")
      allowed_groups.push_back(lookupTables->IdOf("groups", "Little Axe Clinic"));
    if (optionsPermissions["table.measurements.den"]["view"] == "true")
      allowed_groups.push_back(lookupTables->IdOf("groups", "Dental Clinic"));
    if (optionsPermissions["table.measurements.cob"]["view"] == "true")
      allowed_groups.push_back(lookupTables->IdOf("groups", "Contract Health"));
    if (optionsPermissions["table.measurements.pub"]["view"] == "true")
      allowed_groups.push_back(lookupTables->IdOf("groups", "Public Health"));
    // TODO: possibly dangerous ... members of a group might need stuff taken away from them, even from their own group?
    allowed_groups.push_back(current_groupid);

    string date_string = SqlOptionalBetween("date_of_observation", optional_date_start, optional_date_stop);

    string sql;

    string where_clause =
           "date_of_observation between '1/1/1995' and cast('now' as date) " // TODO -c HACK: ignoring stuff with out-of-bounds dates
           "and " + date_string + " "
           "and obs_response_sets.current_id + 1 > 3 "
           "and (1 = 0 or obs_response_sets.group_id in (" + VecToCommaString(allowed_groups) + ")) "
           "and (1 = 0 or obs_response_sets.obs_question_set_id in (select oqs.id from obs_question_set_types oqs inner join permissions p on p.s = oqs.permission_name and p.k = 'VIEW' and p.v = 'TRUE' and p.user_id = " + current_userid + ")) ";

    sql = "select extract(year from date_of_observation) meas_year, "
           "count(distinct obs_response_sets.id) meas_cnt, "
           "count(distinct obs_response_sets.person_id) meas_ppl "
           "from obs_responses "
           "inner join obs_response_sets on obs_responses.obs_response_set_id = obs_response_sets.id "
           "and obs_responses.obs_question_id in (" + VecToCommaString(obs_question_ids) + ") "
           "left join " + source_view + " on " + source_view + ".id = obs_response_sets.person_id "
           "where " + where_clause +
           "and " + source_view + ".id is not null "
           // "and obs_responses.id is not null "
           "group by extract(year from date_of_observation) "
           "order by extract(year from date_of_observation) desc";

    vector<obs_stream*> observation_streams;
    for (vector<string>::const_iterator i = obs_question_ids.begin(); i != obs_question_ids.end(); i++)
     observation_streams.push_back(new obs_stream(dbTransaction, *i, source_view, where_clause, method));

    ze_file << "<table class='datatable' width=100%>"
               "<thead>"
               "<tr>"
               "<td align=right><i>Year</i></td>"
               "<td align=right><i>Sessions</i></td>"
               "<td align=right><i>People</i></td>";

    for (vector<obs_stream*>::iterator i = observation_streams.begin(); i != observation_streams.end(); i++)
     ze_file << "<td align=right><i>" << (*i)->name << "</i></td>";
    ze_file << "</tr>"
               "</thead>"
               "<tbody>";

    vector<map<string, string> > results;
    unsigned int total_session_count = 0;

    if (SqlToVecMap(dbTransaction, sql, results))
    {
     for (vector<map<string, string> >::iterator j = results.begin(); j != results.end(); j++)
     {
      ze_file << "<tr>"
                 "<td align=right>" << (*j)["meas_year"] << "</td>"
                 "<td align=right>" << (*j)["meas_cnt"] << "</td>"
                 "<td align=right>" << (*j)["meas_ppl"] << "</td>";
      total_session_count += StringToInt((*j)["meas_cnt"]);
      for (vector<obs_stream*>::iterator k = observation_streams.begin(); k != observation_streams.end(); k++)
       ze_file << "<td align=right>" << (*k)->get_year_data((*j)["meas_year"]) << "</td>";
      ze_file << "</tr>";
     }
    } else
     ze_file << "<tr><td colspan=" << (3 + observation_streams.size()) << "><b>Failed to retrieve data!</b></td></tr>";

    ze_file << "</tbody>"
               "</table>";

    // totals at the bottom
    ze_file << "<br><i>Total relevant sessions:</i> " << total_session_count;

    sql = "select count(distinct obs_response_sets.person_id) meas_ppl "
           "from obs_responses "
           "inner join obs_response_sets on obs_responses.obs_response_set_id = obs_response_sets.id "
           "and obs_responses.obs_question_id in (" + VecToCommaString(obs_question_ids) + ") "
           "left join " + source_view + " on " + source_view + ".id = obs_response_sets.person_id "
           "where " + where_clause +
           "and obs_responses.id is not null "
           "and " + source_view + ".id is not null";
    ze_file << "<br><i>Unique people measured:</i> " << GetOneField(dbTransaction, sql);

    sql = "select count(distinct id) from " + source_view;
    ze_file << "<br><i>Unique people in input set:</i> " << GetOneField(dbTransaction, sql);

    for (vector<obs_stream*>::iterator i = observation_streams.begin(); i != observation_streams.end(); i++)
     delete *i;
   }

   htmlStop(ze_file);
   ze_file.close();

   string real_cwd = GetCurrentDir().c_str();
   Processing(false);
   TformSpecialHtmlPreview::DoPrintPreview(real_cwd + "\\yms.html");
   this->CloseNow();
 }
 catch (...)
 {
   Processing(false);
   PSay("Error running audit report", m_database_error, MB_OK);
 }
}
//---------------------------------------------------------------------------
void TformReportYearlyMeasurementSummary::Do (TIBTransaction * dbTransaction, string source_view, const vector<string> & obs_question_ids, const string & optional_date_start, const string & optional_date_stop, yms_methods method)
{
 FORM_TRY
  TformReportYearlyMeasurementSummary* z = new TformReportYearlyMeasurementSummary(NULL);
  z->Run(dbTransaction, source_view, obs_question_ids, optional_date_start, optional_date_stop, method);
 FORM_CATCH
}
//---------------------------------------------------------------------------
