#include "scheduling.h"
#include "derivedheaders.h"
// public functions ------------------------------------------------------------
classScheduling::classScheduling(TCanvas *p_canvas, int p_canvas_width, int p_canvas_height, Pdate p_current_date,
                                 Pdate p_today_date, map<string, TColor> p_user_color_map,
                                 float p_percentage_used, int p_font_size, bool p_to_printer, bool p_show_clock_icons)
{
//  canvas = p_canvas;
  passed_canvas = p_canvas;
  to_printer = p_to_printer;
  canvas_width = p_canvas_width;
  canvas_height = p_canvas_height;
  show_clock_icons = p_show_clock_icons; // jhaley 20100820: made clocks optional
  if(! to_printer)
  {
    FORM_TRY
     bitmap = new Graphics::TBitmap();
    FORM_CATCH
    canvas = bitmap->Canvas;
    FORM_TRY
     bitmap->Width = p_canvas_width;
     bitmap->Height = p_canvas_height;
    FORM_CATCH
  }
  else
  {
    canvas = passed_canvas;
  }
  percentage_used = p_percentage_used;
  current_date = p_current_date;
  today_date = p_today_date;
  used_width = canvas_width * percentage_used;
  used_height = canvas_height * percentage_used;
  x_start_pos = (canvas_width - used_width) / 2.0;
  x_end_pos = x_start_pos + used_width;
  y_start_pos = (canvas_height - used_height) / 2.0;
  y_end_pos = y_start_pos + used_height;

  canvas->Font->Size = p_font_size;
  title_bar_vert_margin = 0.6;
  title_bar_row_padding = 1.2;
  SetTitleBarSize(-1);

  title_bar_color = TColor(0x00d6aa94);
  background_color = TColor(0x00ffffff);
  grid_line_color = TColor(0x00808080);
  text_color = TColor(0x00000000);
  today_color = TColor(0x000000ca);
  week_titles_color = TColor(0x00bc6e48);
  month_appt_color = TColor(0x00800000);
  year_margin_color = TColor(0x00c8d0d4);
  user_color_map = p_user_color_map;
}
// ------------------------
classScheduling::~classScheduling()
{
  if((bitmap) && (! to_printer))
    delete bitmap;
}
// ------------------------
void classScheduling::SetCanvasDimensions(int p_canvas_width, int p_canvas_height)
{
  canvas_width = p_canvas_width;
  canvas_height = p_canvas_height;
  if(! to_printer)
  {
   FORM_TRY
    bitmap->Width = p_canvas_width;
    bitmap->Height = p_canvas_height;
   FORM_CATCH
  }
  used_width = p_canvas_width * percentage_used;
  used_height = p_canvas_height * percentage_used;
  x_start_pos = (canvas_width - used_width) / 2.0;
  x_end_pos = canvas_width - x_start_pos;
  y_start_pos = (canvas_height - used_height) / 2.0;
  y_end_pos = canvas_height - y_start_pos;
}
// ------------------------
void classScheduling::SetFontSize(int p_font_size, int p_current_calendar_view)
{
  canvas->Font->Size = p_font_size;
  SetTitleBarSize(p_current_calendar_view);
}
// ------------------------
void classScheduling::DrawDayCalendar(PApptOccurrenceMap &p_appointment_map)
{
  SetTitleBarSize(day_calendar);
  DrawBackground(day_calendar);
  DrawTitleBar(day_calendar);
  DrawCalendarTitle(day_calendar);
  DrawAppointmentsOnDayView(p_appointment_map);
  stamp_rect_vec.clear();
  stamp_rect temp_class;
  temp_class.rect = Rect(x_start_pos, y_start_pos, x_end_pos, y_end_pos);
  temp_class.the_stamp.InDate(current_date);
  temp_class.the_stamp.InTime("08:00:00", military | withsec);
  temp_class.column_name = "";
  stamp_rect_vec.push_back(temp_class);
  if(! to_printer)
    CopyBitmapImageToCanvas();
}
// ------------------------
void classScheduling::DrawWeekCalendar(PApptOccurrenceMap &p_appointment_map)
{
  SetTitleBarSize(week_calendar);
  DrawBackground(week_calendar);
  DrawTitleBar(week_calendar);
  DrawCalendarTitle(week_calendar);
  DrawAppointmentsOnWeekView(p_appointment_map);
  if(! to_printer)
    CopyBitmapImageToCanvas();
}
// ------------------------
void classScheduling::DrawMonthCalendar(PApptOccurrenceMap &p_appointment_map)
{
  SetTitleBarSize(month_calendar);
  DrawBackground(month_calendar);
  DrawTitleBar(month_calendar);
  int day_titles_height = title_bar_height / 2.0;
  int day_titles_end = title_bar_end + day_titles_height;
  DrawLine(x_start_pos, day_titles_end, x_end_pos, day_titles_end);
  DrawHorizontalLines(day_titles_end, y_end_pos, 5);
  DrawCalendarTitle(month_calendar);
  DrawDayNamesOnMonthCalendar();
  DrawDatesOnMonthCalendar();
  DrawAppointmentsOnMonthCalendar(p_appointment_map);
  if(! to_printer)
    CopyBitmapImageToCanvas();
}
// ------------------------
void classScheduling::DrawYearCalendar(TColor p_background_color)
{
  SetTitleBarSize(year_calendar);
  int between_calendar_margin = canvas->TextHeight("Sample") * 0.8;
  int each_calendar_width = (used_width - (between_calendar_margin * 3.0)) / 4.0;
  int each_calendar_height = (used_height - (between_calendar_margin * 2.0)) / 3.0;
  int temp_x;
  int temp_y;
  int current_x = x_start_pos;
  int current_y = y_start_pos;

  DrawBackground(year_calendar, p_background_color);
  stamp_rect_vec.clear();
  appointment_rect_vec.clear();
  for(int i = 1; i <= 12; i++)
  {
    if(i % 4 == 0)
    {
      temp_x = x_end_pos - each_calendar_width;
      if(i != 12)
        DrawSmallMonthCalendar(i, current_date.year, each_calendar_width, each_calendar_height, temp_x, current_y);
      else
      {
        temp_y = y_end_pos - each_calendar_height;
        DrawSmallMonthCalendar(i, current_date.year, each_calendar_width, each_calendar_height, temp_x, temp_y);
      }
    }
    else if(i >= 9)
    {
      temp_y = y_end_pos - each_calendar_height;
      DrawSmallMonthCalendar(i, current_date.year, each_calendar_width, each_calendar_height, current_x, temp_y);
    }
    else
      DrawSmallMonthCalendar(i, current_date.year, each_calendar_width, each_calendar_height, current_x, current_y);

    if(i % 4 != 0)
      current_x += each_calendar_width + between_calendar_margin;
    else
    {
      current_x = x_start_pos;
      current_y += each_calendar_height + between_calendar_margin;
    }
  }
  if(! to_printer)
    CopyBitmapImageToCanvas();
}
// ------------------------
void classScheduling::DrawBlockDayCalendar(PApptOccurrenceMap &p_appointment_map)
{
  SetTitleBarSize(block_day_calendar);
  appointment_rect_vec.clear();
  DrawBackground(block_day_calendar);
  DrawTitleBar(block_day_calendar);
  DrawCalendarTitle(day_calendar);
  int left_side_end;
  DrawDayBlockTimes(left_side_end, block_day_calendar);
  DrawDayBlockAppointments(left_side_end, x_end_pos, current_date, p_appointment_map, block_day_calendar);
  if(! to_printer)
    CopyBitmapImageToCanvas();
}
// ------------------------
void classScheduling::DrawBlockLocationCalendar(PApptOccurrenceMap &p_appointment_map)
{
  SetTitleBarSize(block_location_calendar);
  appointment_rect_vec.clear();
  DrawBackground(block_location_calendar);
  DrawTitleBar(block_location_calendar);
  DrawCalendarTitle(day_calendar);
  int left_side_end;
  DrawDayBlockTimes(left_side_end, block_location_calendar);
  // split up appointment map by location, find out how many locations we've got to display (# of columns),
  // calculate column width, and then call this function once for each column/location.  Fun!
  map<string, PApptOccurrenceMap> location_map;

  // loop through all appointments for current_date
  PApptOccurrenceMap::iterator i_day = p_appointment_map.find(current_date);
  if(i_day != p_appointment_map.end())
  {
    for(PApptOccurrenceSet::iterator itr = i_day->second.begin(); itr != i_day->second.end(); itr++)
    {
       // re-build appointments by location
       location_map[itr->location][current_date].insert(*itr);
    }
  }
  // loop through locations, passing each PApptOccurrenceMap to
  // the function to draw the column
  int this_x_start = left_side_end;
  int this_x_end;
  int i = 0;
  string title_text;
  canvas->Font->Color = text_color;
  vector<stamp_rect> temp_vec;

  for(map<string, PApptOccurrenceMap>::iterator itr = location_map.begin(); itr != location_map.end(); itr++)
  {
    title_text = itr->first;       // left_side_end is the rightmost part of the "column" that contains the time
    this_x_start = left_side_end + ((used_width - (left_side_end - x_start_pos)) * (i/float(location_map.size())));
    this_x_end = left_side_end + ((used_width - (left_side_end - x_start_pos)) * ((i+1)/float(location_map.size())));
    canvas->Brush->Style = bsClear;
    canvas->Font->Style = TFontStyles()<< fsBold;
    canvas->TextRect(Rect(this_x_start, y_start_pos, this_x_end, y_start_pos + title_bar_height),
                    (this_x_end) - ((this_x_end - this_x_start) / 2.0) - (canvas->TextWidth(title_text.c_str()) / 2.0),
                    (y_start_pos + TitleBarRowCenter(2)) - (canvas->TextHeight(title_text.c_str()) / 2.0),
                    title_text.c_str());
    canvas->Font->Style = TFontStyles();
    canvas->Brush->Style = bsSolid;
    int row_height = canvas->TextHeight("Sample");
    int row_one_bottom = y_start_pos + row_height * (1 + title_bar_vert_margin) +
                         (row_height * title_bar_row_padding / 2.0);
    DrawLine(this_x_start, row_one_bottom, this_x_start, y_end_pos);
    DrawDayBlockAppointments(this_x_start, this_x_end, current_date, itr->second, block_location_calendar);

    // previously stamp_rect_vec contained one region per "row" ... we're going to create a new region per column,
    // per row, then take that temporary structure and replace the original stamp_rect_vec with it.  Thus we
    // convert a per-row set of clickable regions to a grid of row/col clickable regions.
    stamp_rect temp_struct;
    for(vector<stamp_rect>::iterator itr = stamp_rect_vec.begin(); itr != stamp_rect_vec.end(); itr++)
    {
      temp_struct.rect = Rect(this_x_start, itr->rect.Top, this_x_end, itr->rect.Bottom);
      temp_struct.the_stamp = itr->the_stamp;
      temp_struct.column_name = title_text;
      temp_vec.push_back(temp_struct);
    }
    stamp_rect_vec = temp_vec;

    i++;
  }

  if(! to_printer)
    CopyBitmapImageToCanvas();
}
// ------------------------
void classScheduling::DrawBlockReasonCalendar(PApptOccurrenceMap &p_appointment_map)
{
  SetTitleBarSize(block_reason_calendar);
  appointment_rect_vec.clear();
  DrawBackground(block_reason_calendar);
  DrawTitleBar(block_reason_calendar);
  DrawCalendarTitle(day_calendar);
  int left_side_end;
  DrawDayBlockTimes(left_side_end, block_reason_calendar);
  // split up appointment map by reason, find out how many reasons we've got to display (# of columns),
  // calculate column width, and then call this function once for each column/reason.  Fun!
  map<string, PApptOccurrenceMap > reason_map;

  // loop through all appointments for current_date
  PApptOccurrenceMap::iterator i_day = p_appointment_map.find(current_date);
  if(i_day != p_appointment_map.end())
  {
    for(PApptOccurrenceSet::iterator itr = i_day->second.begin(); itr != i_day->second.end(); itr++)
    {
       // re-build appointments by reason
       reason_map[itr->reason][current_date].insert(*itr);
    }
  }
  // loop through reasons, passing each PApptOccurrenceMap to
  // the function to draw the column
  int this_x_start = left_side_end;
  int this_x_end;
  int i = 0;
  string title_text;
  canvas->Font->Color = text_color;
  vector<stamp_rect> temp_vec;

  for(map<string, PApptOccurrenceMap >::iterator itr = reason_map.begin(); itr != reason_map.end(); itr++)
  {
    title_text = itr->first;
    this_x_start = left_side_end + ((used_width - (left_side_end - x_start_pos)) * (i/float(reason_map.size())));
    this_x_end = left_side_end + ((used_width - (left_side_end - x_start_pos)) * ((i+1)/float(reason_map.size())));
    canvas->Brush->Style = bsClear;
    canvas->Font->Style = TFontStyles()<< fsBold;
    canvas->TextRect(Rect(this_x_start, y_start_pos, this_x_end, y_start_pos + title_bar_height),
                    (this_x_end) - ((this_x_end - this_x_start) / 2.0) - (canvas->TextWidth(title_text.c_str()) / 2.0),
                    (y_start_pos + TitleBarRowCenter(2)) - (canvas->TextHeight(title_text.c_str()) / 2.0),
                    title_text.c_str());
    canvas->Font->Style = TFontStyles();
    canvas->Brush->Style = bsSolid;
    int row_height = canvas->TextHeight("Sample");
    int row_one_bottom = y_start_pos + row_height * (1 + title_bar_vert_margin) +
                         (row_height * title_bar_row_padding / 2.0);
    DrawLine(this_x_start, row_one_bottom, this_x_start, y_end_pos);
    DrawDayBlockAppointments(this_x_start, this_x_end, current_date, itr->second, block_reason_calendar);

    // previously stamp_rect_vec contained one region per "row" ... we're going to create a new region per column,
    // per row, then take that temporary structure and replace the original stamp_rect_vec with it.  Thus we
    // convert a per-row set of clickable regions to a grid of row/col clickable regions.
    stamp_rect temp_struct;
    for(vector<stamp_rect>::iterator itr = stamp_rect_vec.begin(); itr != stamp_rect_vec.end(); itr++)
    {
      temp_struct.rect = Rect(this_x_start, itr->rect.Top, this_x_end, itr->rect.Bottom);
      temp_struct.the_stamp = itr->the_stamp;
      temp_struct.column_name = title_text;
      temp_vec.push_back(temp_struct);
    }
    stamp_rect_vec = temp_vec;

    i++;
  }

  if(! to_printer)
    CopyBitmapImageToCanvas();
}
// ------------------------
void classScheduling::DrawBlockUserPosCalendar(PApptOccurrenceMap &p_appointment_map)
{
  SetTitleBarSize(block_userpos_calendar);
  appointment_rect_vec.clear();
  DrawBackground(block_userpos_calendar);
  DrawTitleBar(block_userpos_calendar);
  DrawCalendarTitle(day_calendar);
  int left_side_end;
  DrawDayBlockTimes(left_side_end, block_userpos_calendar);
  // split up appointment map by userpos, find out how many userposs we've got to display (# of columns),
  // calculate column width, and then call this function once for each column/userpos.  Fun!
  map<string, PApptOccurrenceMap > userpos_map;

  // loop through all appointments for current_date
  PApptOccurrenceMap::iterator i_day = p_appointment_map.find(current_date);
  if(i_day != p_appointment_map.end())
  {
    for(PApptOccurrenceSet::iterator itr = i_day->second.begin(); itr != i_day->second.end(); itr++)
    {
       // re-build appointments by userpos
       userpos_map[itr->userpos][current_date].insert(*itr);
    }
  }
  // loop through userposs, passing each PApptOccurrenceMap to
  // the function to draw the column
  int this_x_start = left_side_end;
  int this_x_end;
  int i = 0;
  string title_text;
  canvas->Font->Color = text_color;
  vector<stamp_rect> temp_vec;

  for(map<string, PApptOccurrenceMap >::iterator itr = userpos_map.begin(); itr != userpos_map.end(); itr++)
  {
    title_text = itr->first;
    this_x_start = left_side_end + ((used_width - (left_side_end - x_start_pos)) * (i/float(userpos_map.size())));
    this_x_end = left_side_end + ((used_width - (left_side_end - x_start_pos)) * ((i+1)/float(userpos_map.size())));
    canvas->Brush->Style = bsClear;
    canvas->Font->Style = TFontStyles()<< fsBold;
    canvas->TextRect(Rect(this_x_start, y_start_pos, this_x_end, y_start_pos + title_bar_height),
                    (this_x_end) - ((this_x_end - this_x_start) / 2.0) - (canvas->TextWidth(title_text.c_str()) / 2.0),
                    (y_start_pos + TitleBarRowCenter(2)) - (canvas->TextHeight(title_text.c_str()) / 2.0),
                    title_text.c_str());
    canvas->Font->Style = TFontStyles();
    canvas->Brush->Style = bsSolid;
    int row_height = canvas->TextHeight("Sample");
    int row_one_bottom = y_start_pos + row_height * (1 + title_bar_vert_margin) +
                         (row_height * title_bar_row_padding / 2.0);
    DrawLine(this_x_start, row_one_bottom, this_x_start, y_end_pos);
    DrawDayBlockAppointments(this_x_start, this_x_end, current_date, itr->second, block_userpos_calendar);

    // previously stamp_rect_vec contained one region per "row" ... we're going to create a new region per column,
    // per row, then take that temporary structure and replace the original stamp_rect_vec with it.  Thus we
    // convert a per-row set of clickable regions to a grid of row/col clickable regions.
    stamp_rect temp_struct;
    for(vector<stamp_rect>::iterator itr = stamp_rect_vec.begin(); itr != stamp_rect_vec.end(); itr++)
    {
      temp_struct.rect = Rect(this_x_start, itr->rect.Top, this_x_end, itr->rect.Bottom);
      temp_struct.the_stamp = itr->the_stamp;
      temp_struct.column_name = title_text;
      temp_vec.push_back(temp_struct);
    }
    stamp_rect_vec = temp_vec;

    i++;
  }

  if(! to_printer)
    CopyBitmapImageToCanvas();
}
// ------------------------
void classScheduling::DrawBlockOutcomeCalendar(PApptOccurrenceMap &p_appointment_map)
{
  SetTitleBarSize(block_outcome_calendar);
  appointment_rect_vec.clear();
  DrawBackground(block_outcome_calendar);
  DrawTitleBar(block_outcome_calendar);
  DrawCalendarTitle(day_calendar);
  int left_side_end;
  DrawDayBlockTimes(left_side_end, block_outcome_calendar);
  // split up appointment map by outcome, find out how many outcomes we've got to display (# of columns),
  // calculate column width, and then call this function once for each column/outcome.  Fun!
  map<string, PApptOccurrenceMap > outcome_map;

  // loop through all appointments for current_date
  PApptOccurrenceMap::iterator i_day = p_appointment_map.find(current_date);
  if(i_day != p_appointment_map.end())
  {
    for(PApptOccurrenceSet::iterator itr = i_day->second.begin(); itr != i_day->second.end(); itr++)
    {
       // re-build appointments by outcome
       outcome_map[itr->outcome][current_date].insert(*itr);
    }
  }
  // loop through outcomes, passing each PApptOccurrenceMap to
  // the function to draw the column
  int this_x_start = left_side_end;
  int this_x_end;
  int i = 0;
  string title_text;
  canvas->Font->Color = text_color;
  vector<stamp_rect> temp_vec;

  for(map<string, PApptOccurrenceMap >::iterator itr = outcome_map.begin(); itr != outcome_map.end(); itr++)
  {
    title_text = itr->first;       // left_side_end is the rightmost part of the "column" that contains the time
    this_x_start = left_side_end + ((used_width - (left_side_end - x_start_pos)) * (i/float(outcome_map.size())));
    this_x_end = left_side_end + ((used_width - (left_side_end - x_start_pos)) * ((i+1)/float(outcome_map.size())));
    canvas->Brush->Style = bsClear;
    canvas->Font->Style = TFontStyles()<< fsBold;
    canvas->TextRect(Rect(this_x_start, y_start_pos, this_x_end, y_start_pos + title_bar_height),
                    (this_x_end) - ((this_x_end - this_x_start) / 2.0) - (canvas->TextWidth(title_text.c_str()) / 2.0),
                    (y_start_pos + TitleBarRowCenter(2)) - (canvas->TextHeight(title_text.c_str()) / 2.0),
                    title_text.c_str());
    canvas->Font->Style = TFontStyles();
    canvas->Brush->Style = bsSolid;
    int row_height = canvas->TextHeight("Sample");
    int row_one_bottom = y_start_pos + row_height * (1 + title_bar_vert_margin) +
                         (row_height * title_bar_row_padding / 2.0);
    DrawLine(this_x_start, row_one_bottom, this_x_start, y_end_pos);
    DrawDayBlockAppointments(this_x_start, this_x_end, current_date, itr->second, block_outcome_calendar);

    // previously stamp_rect_vec contained one region per "row" ... we're going to create a new region per column,
    // per row, then take that temporary structure and replace the original stamp_rect_vec with it.  Thus we
    // convert a per-row set of clickable regions to a grid of row/col clickable regions.
    stamp_rect temp_struct;
    for(vector<stamp_rect>::iterator itr = stamp_rect_vec.begin(); itr != stamp_rect_vec.end(); itr++)
    {
      temp_struct.rect = Rect(this_x_start, itr->rect.Top, this_x_end, itr->rect.Bottom);
      temp_struct.the_stamp = itr->the_stamp;
      temp_struct.column_name = title_text;
      temp_vec.push_back(temp_struct);
    }
    stamp_rect_vec = temp_vec;

    i++;
  }

  if(! to_printer)
    CopyBitmapImageToCanvas();
}
// ------------------------
void classScheduling::DrawBlockProviderCalendar(PApptOccurrenceMap &p_appointment_map)
{
  SetTitleBarSize(block_provider_calendar);
  appointment_rect_vec.clear();
  DrawBackground(block_provider_calendar);
  DrawTitleBar(block_provider_calendar);
  DrawCalendarTitle(day_calendar);
  int left_side_end;
  DrawDayBlockTimes(left_side_end, block_provider_calendar);
  // split up appointment map by provider, find out how many providers we've got to display (# of columns),
  // calculate column width, and then call this function once for each column/provider.  Fun!
  map<string, PApptOccurrenceMap > provider_map;

  // loop through all appointments for current_date
  PApptOccurrenceMap::iterator i_day = p_appointment_map.find(current_date);
  if(i_day != p_appointment_map.end())
  {
    for(PApptOccurrenceSet::iterator itr = i_day->second.begin(); itr != i_day->second.end(); itr++)
    {
       // re-build appointments by provider
       provider_map[itr->provider][current_date].insert(*itr);
    }
  }
  // loop through providers, passing each PApptOccurrenceMap to
  // the function to draw the column
  int this_x_start = left_side_end;
  int this_x_end;
  int i = 0;
  string title_text;
  canvas->Font->Color = text_color;
  vector<stamp_rect> temp_vec;

  for(map<string, PApptOccurrenceMap >::iterator itr = provider_map.begin(); itr != provider_map.end(); itr++)
  {
    title_text = itr->first;
    this_x_start = left_side_end + ((used_width - (left_side_end - x_start_pos)) * (i/float(provider_map.size())));
    this_x_end = left_side_end + ((used_width - (left_side_end - x_start_pos)) * ((i+1)/float(provider_map.size())));
    canvas->Brush->Style = bsClear;
    canvas->Font->Style = TFontStyles()<< fsBold;
    canvas->TextRect(Rect(this_x_start, y_start_pos, this_x_end, y_start_pos + title_bar_height),
                    (this_x_end) - ((this_x_end - this_x_start) / 2.0) - (canvas->TextWidth(title_text.c_str()) / 2.0),
                    (y_start_pos + TitleBarRowCenter(2)) - (canvas->TextHeight(title_text.c_str()) / 2.0),
                    title_text.c_str());
    canvas->Font->Style = TFontStyles();
    canvas->Brush->Style = bsSolid;
    int row_height = canvas->TextHeight("Sample");
    int row_one_bottom = y_start_pos + row_height * (1 + title_bar_vert_margin) +
                         (row_height * title_bar_row_padding / 2.0);
    DrawLine(this_x_start, row_one_bottom, this_x_start, y_end_pos);
    DrawDayBlockAppointments(this_x_start, this_x_end, current_date, itr->second, block_provider_calendar);

    // previously stamp_rect_vec contained one region per "row" ... we're going to create a new region per column,
    // per row, then take that temporary structure and replace the original stamp_rect_vec with it.  Thus we
    // convert a per-row set of clickable regions to a grid of row/col clickable regions.
    stamp_rect temp_struct;
    for(vector<stamp_rect>::iterator itr = stamp_rect_vec.begin(); itr != stamp_rect_vec.end(); itr++)
    {
      temp_struct.rect = Rect(this_x_start, itr->rect.Top, this_x_end, itr->rect.Bottom);
      temp_struct.the_stamp = itr->the_stamp;
      temp_struct.column_name = title_text;
      temp_vec.push_back(temp_struct);
    }
    stamp_rect_vec = temp_vec;

    i++;
  }

  if(! to_printer)
    CopyBitmapImageToCanvas();
}
// ------------------------
void classScheduling::DrawBlockWeekCalendar(PApptOccurrenceMap &p_appointment_map)
{
  SetTitleBarSize(block_week_calendar);
  appointment_rect_vec.clear();
  DrawBackground(block_week_calendar);
  DrawTitleBar(block_week_calendar);
  int left_side_end;
  DrawDayBlockTimes(left_side_end, block_week_calendar);
  DrawWeekBlockTitle(left_side_end, p_appointment_map, block_week_calendar);
  if(! to_printer)
    CopyBitmapImageToCanvas();
}
// ------------------------
Pdate classScheduling::GetCurrentDate()
{
  return current_date;
}
// ------------------------
void classScheduling::SetCurrentDate(Pdate p_new_date)
{
  current_date = p_new_date;
}
// ------------------------
TColor classScheduling::GetTitleBarColor()
{
  return title_bar_color;
}
// ------------------------
void classScheduling::GetBeginEndDates(Pdate &p_begin_date, Pdate &p_end_date, int calendar_view)
{
  if(calendar_view == day_calendar || calendar_view == block_day_calendar ||
     calendar_view == block_location_calendar || calendar_view == block_reason_calendar ||
     calendar_view == block_outcome_calendar || calendar_view == block_provider_calendar ||
     calendar_view == block_userpos_calendar)
  {
    p_begin_date = current_date;
    p_end_date = current_date;
  }
  else if((calendar_view == week_calendar) || (calendar_view == block_week_calendar))
  {
    Pdate temp_date = current_date;
    while(temp_date.DayOfWeek() != 1)
      temp_date.mod(-1, in_days);
    p_begin_date = temp_date;
    temp_date.mod(6, in_days);
    p_end_date = temp_date;
  }
  else if(calendar_view == month_calendar)
  {
    Pdate temp_date = current_date;
    temp_date.day = 1;
    p_begin_date = temp_date;
    temp_date.mod(LastDayOf(current_date.month, current_date.year)-1, in_days);
    p_end_date = temp_date;
  }
}
// ------------------------
int classScheduling::GetHeightNeeded(PApptOccurrenceMap &p_appointment_map, int calendar_view)
{
  int to_return = 0;
  if (!canvas)
    return to_return;
  int text_height = canvas->TextHeight("Sample");
  if(calendar_view == day_calendar)
  {
    to_return += CalcTitleBarHeight(calendar_view);
    PApptOccurrenceMap::iterator i_day = p_appointment_map.find(current_date);
    if(i_day != p_appointment_map.end())
    {
      to_return += i_day->second.size() * (text_height * 1.5);
      to_return += text_height; // for a little extra room at the bottom
    }
  }
  else if(calendar_view == week_calendar)
  {
    Pdate begin_date;
    Pdate end_date;
    GetBeginEndDates(begin_date, end_date, week_calendar);
    PApptOccurrenceMap::iterator i_day;
    to_return += CalcTitleBarHeight(calendar_view);
    while(begin_date <= end_date)
    {
      i_day = p_appointment_map.find(begin_date);
      to_return += text_height * 1.5; // for date/day title
      if (i_day != p_appointment_map.end()) // this is what was causing very-large heights to be requested -> resource errors -> week view not working
      {
        if(i_day->second.size() == 1)
          to_return += text_height * 3.0;
        else if(i_day->second.size() > 1)
          to_return += i_day->second.size() * (text_height * 1.5);
        else // if the date is present, but empty?
          to_return += text_height * 3.0;
      } else // date not found at all
        to_return += text_height * 3.0;
      begin_date.mod(1, in_days);
    }
    to_return += text_height * 1.2;
  }
  else if(calendar_view == block_day_calendar || calendar_view == block_week_calendar ||
          calendar_view == block_location_calendar || calendar_view == block_reason_calendar ||
          calendar_view == block_outcome_calendar || calendar_view == block_provider_calendar ||
          calendar_view == block_userpos_calendar)
  {
    to_return = GetPtimeYPosition(Ptime("19:30:00", military | withsec), calendar_view);
  }

  return to_return;
}
// ------------------------
string classScheduling::GetAppointmentIdAtPosition(int p_x_pos, int p_y_pos)
{
  string to_return = "";
  for(vector<appt_rect>::iterator itr = appointment_rect_vec.begin(); itr != appointment_rect_vec.end(); itr++)
  {
    if(IsInRectangle(p_x_pos, p_y_pos, itr->rect))
    {
      to_return = itr->id;
      break;
    }
  }
  return to_return;
}
// ------------------------
void classScheduling::GetStampAtPosition(int p_x_pos, int p_y_pos, bool &p_one_found, Pstamp &p_stamp_found, string &p_col_name)
{
  p_one_found = false;
  for(vector<stamp_rect>::iterator itr = stamp_rect_vec.begin(); itr != stamp_rect_vec.end(); itr++)
  {
    if(IsInRectangle(p_x_pos, p_y_pos, itr->rect))
    {
      p_one_found = true;
      p_stamp_found = itr->the_stamp;
      p_col_name = itr->column_name;
      break;
    }
  }
}
// ------------------------
int classScheduling::GetXStartPosition()
{
  return x_start_pos;
}
// ------------------------
int classScheduling::GetXEndPosition()
{
  return x_end_pos;
}
int classScheduling::GetYStartPosition()
{
  return y_start_pos;
}
// ------------------------
int classScheduling::GetYEndPosition()
{
  return y_end_pos;
}
// ------------------------

// private functions -----------------------------------------------------------
bool classScheduling::IsInRectangle(int p_x_pos, int p_y_pos, TRect p_rect)
{
  if((p_x_pos >= p_rect.Left) && (p_x_pos <= p_rect.Right) &&
    (p_y_pos >= p_rect.Top) && (p_y_pos <= p_rect.Bottom))
    return true;
  else
    return false;
}
// ------------------------
void classScheduling::CopyBitmapImageToCanvas()
{
  passed_canvas->Draw(0, 0, bitmap);
}
// ------------------------
void classScheduling::DrawTitleBar(int p_current_calendar_view)
{
  canvas->Brush->Color = title_bar_color;
  canvas->Pen->Color = grid_line_color;
  canvas->Rectangle(x_start_pos, y_start_pos, x_end_pos, title_bar_end);
  int rows_in_title_bar = GetRowsNeeded(p_current_calendar_view);
  int row_height = canvas->TextHeight("Sample");
  for (int i = 1; i < rows_in_title_bar; i++)
  {
    int y_pos = y_start_pos + (row_height * (1 + title_bar_vert_margin) +
                (i - 1) * row_height * (1 + title_bar_row_padding) +
                (row_height * title_bar_row_padding / 2.0));
    DrawLine(x_start_pos, y_pos, x_end_pos, y_pos);
  }
}
// ------------------------
void classScheduling::DrawBackground(int p_current_calendar_view, TColor p_background_color)
{

  canvas->Pen->Color = grid_line_color;
  canvas->Brush->Color = p_background_color;
  canvas->Rectangle(x_start_pos, y_start_pos, x_end_pos, y_end_pos);
//  if((to_printer) && ((p_current_calendar_view != day_calendar) ||
//    (p_current_calendar_view != month_calendar)))
//  {
//    canvas->Pen->Color = p_background_color;
//    DrawLine(x_start_pos, y_end_pos, x_end_pos, y_end_pos);
//  }

/*
  if((to_printer) && ((p_current_calendar_view != year_calendar) ||
     (p_current_calendar_view != month_calendar)))
  {
    canvas->Brush->Color = p_background_color;
    canvas->FillRect(Rect(x_start_pos, y_start_pos, x_end_pos, y_end_pos));
    canvas->Pen->Color = p_background_color;
    DrawLine(x_start_pos, y_start_pos, x_end_pos, y_start_pos);
    DrawLine(x_start_pos, y_start_pos, x_start_pos, y_end_pos);
    DrawLine(x_end_pos, y_start_pos, x_end_pos, y_end_pos);
  }
  else
  {
    canvas->Pen->Color = grid_line_color;
    canvas->Brush->Color = p_background_color;
    canvas->Rectangle(x_start_pos, y_start_pos, x_end_pos, y_end_pos);
  }
 */
}
// ------------------------
int classScheduling::CalcTitleBarHeight(int p_current_calendar_view)
{
  int rows_in_title_bar = GetRowsNeeded(p_current_calendar_view);
  int result = canvas->TextHeight("Sample") * (1 + 2 * title_bar_vert_margin) +
     (rows_in_title_bar - 1) * canvas->TextHeight("Sample") * (1 + title_bar_row_padding);
  return result;
}
// ------------------------
void classScheduling::SetTitleBarSize(int p_current_calendar_view)
{
  //title_bar_height = canvas->TextHeight("Sample") * 2.2;
  title_bar_height = CalcTitleBarHeight(p_current_calendar_view);
  title_bar_end = y_start_pos + title_bar_height + 1;
}
// ------------------------
void classScheduling::DrawCalendarTitle(int calendar_type)
{
  int hor_title_bar_center = (used_width / 2.0) + x_start_pos;
  int ver_title_bar_center = TitleBarRowCenter(1) + y_start_pos;
  string title_text;
  if(calendar_type == day_calendar)
  {
    title_text = current_date.NameOfDay() + " - " + current_date.NameOfMonth() + " " +
                 IntToString(current_date.day) + ", " + IntToString(current_date.year);
  }
  else if(calendar_type == week_calendar)
  {
    Pdate temp_date = current_date;
    while(temp_date.DayOfWeek() != 1)
      temp_date.mod(-1, in_days);
    title_text = temp_date.NameOfMonth() + " " +
                 IntToString(temp_date.day) + ", " + IntToString(temp_date.year) + " - ";
    temp_date.mod(6, in_days);
    title_text += temp_date.NameOfMonth() + " " +
                  IntToString(temp_date.day) + ", " + IntToString(temp_date.year);
  }
  else if(calendar_type == month_calendar)
  {
    title_text = current_date.NameOfMonth() + " " + IntToString(current_date.year);
  }
  canvas->Brush->Color = title_bar_color;
  canvas->Font->Color = text_color;
  canvas->Font->Style = TFontStyles()<< fsBold;
  int hor_text_center = hor_title_bar_center - (canvas->TextWidth(title_text.c_str()) / 2.0);
  int ver_text_center = ver_title_bar_center - (canvas->TextHeight(title_text.c_str()) / 2.0);
  canvas->TextOut(hor_text_center, ver_text_center, title_text.c_str());
  canvas->Font->Style = TFontStyles();
}
// ------------------------
void classScheduling::DrawHorizontalLines(int start_pos, int end_pos, int num_of_lines)
{
  int this_area_height = end_pos - start_pos;
  float current_y;
  for(int i = 1; i <= num_of_lines; i++)
  {
    current_y = this_area_height * (i / ((float)num_of_lines+1.0)) + start_pos;
    DrawLine(x_start_pos, (int)current_y, x_end_pos, (int)current_y);
  }
}
// ------------------------
void classScheduling::DrawLine(int x_begin, int y_begin, int x_end, int y_end)
{
  canvas->Pen->Color = grid_line_color;
  canvas->MoveTo(x_begin, y_begin);
  canvas->LineTo(x_end, y_end);
}
// ------------------------
void classScheduling::DrawDayNamesOnMonthCalendar()
{
  string display_text;
  float current_x;
  float previous_x = x_start_pos;
  int hor_column_center;
  int hor_center;
  int day_titles_height = title_bar_height / 2.0;
  int day_titles_end = title_bar_end + day_titles_height;
  int day_titles_start = day_titles_end - canvas->TextHeight("Sample");
  canvas->Brush->Color = background_color;
  canvas->Font->Color = week_titles_color;

  for(int i = 1; i < 7; i++)
  {
    current_x = used_width * (i / 7.0) + x_start_pos;
    DrawLine((int)current_x, day_titles_end, (int)current_x, y_end_pos);

    // put title on column
    hor_column_center = ((current_x - previous_x) / 2.0) + previous_x;
    if(i == 1)
      display_text = "Sun";
    else if(i == 2)
      display_text = "Mon";
    else if(i == 3)
      display_text = "Tues";
    else if(i == 4)
      display_text = "Wed";
    else if(i == 5)
      display_text = "Thur";
    else if(i == 6)
      display_text = "Fri";
    hor_center = hor_column_center - (canvas->TextWidth(display_text.c_str()) / 2.0);
    canvas->TextOut(hor_center, day_titles_start, display_text.c_str());

    previous_x = current_x;
  }
  display_text = "Sat";
  hor_column_center = ((x_end_pos - previous_x) / 2.0) + previous_x;
  hor_center = hor_column_center - (canvas->TextWidth(display_text.c_str()) / 2.0);
  canvas->TextOut(hor_center, day_titles_start, display_text.c_str());

  canvas->Font->Color = text_color;

}
// ------------------------
void classScheduling::OutputTodayText(int x_text_start, int y_text_start, string text_to_display)
{
  canvas->Font->Color = today_color;
  canvas->Font->Style = TFontStyles()<< fsBold;
  canvas->TextOut(x_text_start, y_text_start, text_to_display.c_str());
  canvas->Font->Style = TFontStyles();
  canvas->Font->Color = text_color;
}
// ------------------------
void classScheduling::DrawDayNameOnWeekCalendar(int x_pos, int y_pos, Pdate date_to_draw)
{
  string text_to_display = IntToString(date_to_draw.day) + "  " + date_to_draw.NameOfDay();
  if(date_to_draw == today_date)
    OutputTodayText(x_pos, y_pos, text_to_display);
  else
  {
    canvas->Pen->Color = text_color;
    canvas->Brush->Color = background_color;
    canvas->TextOut(x_pos, y_pos, text_to_display.c_str());
  }
}
// ------------------------
void classScheduling::DrawDatesOnMonthCalendar()
{
  Pdate temp_date = current_date;
  temp_date.day = 1;
  int begin_day_of_week = temp_date.DayOfWeek();
  int current_day = 1;
  int days_in_month = LastDayOf(current_date.month, current_date.year);
  int day_titles_height = title_bar_height / 2.0;
  int day_titles_end = title_bar_end + day_titles_height;
  int calendar_area = y_end_pos - day_titles_end;
  float current_y;
  float current_x;
  float next_x;
  float next_y;
  float top_left_margin = canvas->TextHeight("Sample") * 0.2;
  bool can_draw_date = false;
  stamp_rect temp_date_rect;

  stamp_rect_vec.clear();
  appointment_rect_vec.clear();
  for(int i = 0; i < 6; i++)
  {
    current_y = calendar_area * ((float)i / 6.0) + day_titles_end;
    next_y = calendar_area * ((i + 1.0) / 6.0) + day_titles_end;
    for(int j = 0; j < 7; j++)
    {
      current_x = used_width * ((float)j / 7.0) + x_start_pos;
      next_x = used_width * ((j + 1.0) / 7.0) + x_start_pos;
      if(! can_draw_date)
      {
        if ((begin_day_of_week - 1) == j)
          can_draw_date = true;
      }
      if(current_day > days_in_month)
        can_draw_date = false;
      if(can_draw_date)
      {
        temp_date_rect.the_stamp.InDate(temp_date);
        temp_date_rect.the_stamp.InTime("08:00", military | withoutsec);
        temp_date_rect.rect = Rect(current_x, current_y, next_x, next_y);
        temp_date_rect.column_name = "";
        stamp_rect_vec.push_back(temp_date_rect);
        if((current_day == today_date.day) && (current_date.month == today_date.month) && (current_date.year == today_date.year))
          OutputTodayText(current_x + top_left_margin, current_y + top_left_margin, IntToString(current_day));
        else
          canvas->TextOut(current_x + top_left_margin, current_y + top_left_margin, IntToString(current_day).c_str());
        current_day++;
        temp_date++;
      }
//      previous_x = current_x;
    }
//    previous_y = current_y;
  }
}
// ------------------------
string classScheduling::GetDayWeekViewDisplayText(appointment_occurrence p_occurrence, int p_current_calendar_view)
{
  string text_to_display = "";
  Ptime midnight("0:00", military | withoutsec);
  Ptime day_end("23:59", military | withoutsec);

  if((p_current_calendar_view == day_calendar) || (p_current_calendar_view == week_calendar))
  {
    if((p_occurrence.range_begin == midnight) && (p_occurrence.range_end == day_end))
      text_to_display = "all day";
    else if((p_occurrence.range_begin == midnight) && (p_occurrence.range_end != day_end))
      text_to_display = "until " + p_occurrence.range_end.OutTime(ampm | withoutsec);
    else if((p_occurrence.range_begin != midnight) && (p_occurrence.range_end == day_end))
      text_to_display  = "starting " + p_occurrence.range_begin.OutTime(ampm | withoutsec);
    else if(p_occurrence.range_begin == p_occurrence.range_end)
      text_to_display = p_occurrence.range_begin.OutTime(ampm | withoutsec);
    else
      text_to_display = p_occurrence.range_begin.OutTime(ampm | withoutsec) + " - " + p_occurrence.range_end.OutTime(ampm | withoutsec);

    if((p_occurrence.person != "") || (p_occurrence.description != ""))
      text_to_display += ": ";
  }
  if((p_occurrence.person != "") && (p_occurrence.person != " "))
  {
    text_to_display += p_occurrence.person;
    if(p_occurrence.description != "")
      text_to_display += " -";
  }
  if(p_occurrence.description != "")
    text_to_display += " " + p_occurrence.description;
  if((p_occurrence.chart_number != "") && ((p_current_calendar_view == day_calendar) ||
    (p_current_calendar_view == week_calendar)))
    text_to_display += "; chart # " + p_occurrence.chart_number;

  text_to_display = ReplaceStringWithString(ReplaceAnyCharWithString(text_to_display, "\a\f\n\r\t", "; "), "; ; ", "; ");
  return text_to_display;
}
// ------------------------
void classScheduling::DrawUserColorSquare(string user_id, int x_pos, int y_pos, int dimensions)
{
  canvas->Brush->Color = user_color_map[user_id];
  canvas->Pen->Color = grid_line_color;
  canvas->Rectangle(x_pos, y_pos, x_pos + dimensions, y_pos + dimensions);
}
// ------------------------
void classScheduling::DrawDayWeekAppointment(int this_x_start, int &current_y_pos, int text_height,
                                             int p_current_calendar_view, appointment_occurrence p_occurrence)
{
  string text_to_display;
  int current_x_pos = this_x_start;
  TRect rect_to_draw_in;
  appt_rect temp_appt_rect;
  int total_width_needed = 0;

  text_to_display = GetDayWeekViewDisplayText(p_occurrence, p_current_calendar_view);
  DrawUserColorSquare(p_occurrence.user, current_x_pos, current_y_pos, text_height);
  current_x_pos += text_height + (text_height/2.0);
  total_width_needed += this_x_start + text_height + (text_height/2.0);
  if(show_clock_icons) // jhaley 20100820: made icons optional
  {
    // i'm adding 1 to text height on this next line because if i don't, the icons look pointy
    DrawClockIcon(canvas, current_x_pos, current_y_pos, text_height+1, GetIconType(p_occurrence.outcome),
                  p_occurrence.range_begin.hour, p_occurrence.range_begin.minute);
    current_x_pos += text_height + (text_height/2.0);
    total_width_needed += text_height + (text_height/2.0);
  }
  canvas->Brush->Color = background_color;
  canvas->Pen->Color = text_color;
  int text_end_y_pos;
  if(current_y_pos + text_height > y_end_pos - 1)
    text_end_y_pos = y_end_pos - 1;
  else
    text_end_y_pos = current_y_pos + text_height;
  total_width_needed += canvas->TextWidth(text_to_display.c_str());
  if(total_width_needed < x_end_pos - 1)
    rect_to_draw_in = Rect(current_x_pos, current_y_pos, total_width_needed, text_end_y_pos);
  else
    rect_to_draw_in = Rect(current_x_pos, current_y_pos, x_end_pos - 1, text_end_y_pos);
  canvas->TextRect(rect_to_draw_in, current_x_pos, current_y_pos, text_to_display.c_str());
  temp_appt_rect.id = p_occurrence.id;
  rect_to_draw_in.Left -= (text_height * 3.0);
  temp_appt_rect.rect = rect_to_draw_in;
  appointment_rect_vec.push_back(temp_appt_rect);
  current_y_pos += text_height * 1.5;
}
// ------------------------
void classScheduling::DrawAppointmentsOnDayView(PApptOccurrenceMap &p_appointment_map)
{
  int text_height = canvas->TextHeight("Sample");
  int current_y_pos = y_start_pos + title_bar_height + (text_height/2.0);
  int this_x_start = x_start_pos + (text_height/2.0);

  stamp_rect_vec.clear();
  appointment_rect_vec.clear();
  PApptOccurrenceMap::iterator i_day = p_appointment_map.find(current_date);
  if(i_day != p_appointment_map.end())
  {
    for(PApptOccurrenceSet::iterator itr = i_day->second.begin();
        itr != i_day->second.end(); itr++)
      DrawDayWeekAppointment(this_x_start, current_y_pos, text_height, day_calendar, *itr);
  }
}
// ------------------------
void classScheduling::DrawAppointmentsOnWeekView(PApptOccurrenceMap &p_appointment_map)
{
  int text_height = canvas->TextHeight("Sample");
  int current_y_pos = y_start_pos + title_bar_height + (text_height/4.0);
  int x_titles_start = x_start_pos + (text_height/4.0);
  int this_x_start = x_start_pos + (text_height/2.0);
  int current_rect_y_start;
  stamp_rect temp_date_rect;
  Pdate begin_date;
  Pdate end_date;
  GetBeginEndDates(begin_date, end_date, week_calendar);
  PApptOccurrenceMap::iterator i_day;

  appointment_rect_vec.clear();
  stamp_rect_vec.clear();
  while(begin_date <= end_date)
  {
    current_rect_y_start = current_y_pos;
    DrawDayNameOnWeekCalendar(x_titles_start, current_y_pos, begin_date);
    current_y_pos += text_height * 1.5;
    i_day = p_appointment_map.find(begin_date);
    if(i_day != p_appointment_map.end())
    {
      for(PApptOccurrenceSet::iterator itr = i_day->second.begin(); itr != i_day->second.end(); itr++)
        DrawDayWeekAppointment(this_x_start, current_y_pos, text_height, week_calendar, *itr);
      if(i_day->second.size() == 1)
        current_y_pos += text_height * 1.5;
      DrawLine(x_start_pos, current_y_pos, x_end_pos, current_y_pos);
    }
    else
    {
      current_y_pos += text_height * 3.0;
      DrawLine(x_start_pos, current_y_pos, x_end_pos, current_y_pos);
    }
    temp_date_rect.the_stamp.InDate(begin_date);
    temp_date_rect.the_stamp.InTime("08:00", military | withoutsec);
    temp_date_rect.rect = Rect(x_start_pos, current_rect_y_start, x_end_pos, current_y_pos);
    temp_date_rect.column_name = "";
    stamp_rect_vec.push_back(temp_date_rect);

    current_y_pos += text_height/4.0;
    begin_date.mod(1, in_days);
  } // end while
}
// ------------------------
void classScheduling::DrawAppointmentsOnMonthCalendar(PApptOccurrenceMap &p_appointment_map)
{
  PApptOccurrenceMap::iterator i_day;
  int day_count;
  string text_to_display;
  int text_width;
  int text_height = canvas->TextHeight("(123)");
  int margin = text_height * 0.3;

  appointment_rect_vec.clear();
  canvas->Font->Color = month_appt_color;
  for(vector<stamp_rect>::iterator itr = stamp_rect_vec.begin(); itr != stamp_rect_vec.end(); itr++)
  {
    i_day = p_appointment_map.find((Pdate)itr->the_stamp);
    if(i_day != p_appointment_map.end())
    {
      day_count = p_appointment_map[(Pdate)itr->the_stamp].size();
      text_to_display = "(" + IntToString(day_count) + ")";
      text_width = canvas->TextWidth(text_to_display.c_str());
      canvas->TextOut(itr->rect.Right - text_width - margin, itr->rect.Bottom - text_height - margin, text_to_display.c_str());
    }
  }
}
// ------------------------
void classScheduling::DrawSmallMonthOutline(int p_x_start, int p_y_start, int p_calendar_width, int p_calendar_height)
{
  canvas->Brush->Color = background_color;
  canvas->Pen->Color = grid_line_color;
  canvas->Rectangle(p_x_start, p_y_start, p_x_start + p_calendar_width, p_y_start + p_calendar_height);
}
// ------------------------
void classScheduling::DrawSmallMonthTitleBar(int p_x_start, int p_y_start, int p_bar_width, int p_bar_height, string p_title)
{
  canvas->Brush->Color = title_bar_color;
  canvas->Pen->Color = grid_line_color;
  canvas->Font->Color = text_color;
  TRect title_bar_rect(p_x_start, p_y_start, p_x_start + p_bar_width, p_y_start + p_bar_height);
  canvas->Rectangle(title_bar_rect);

  canvas->Font->Style = TFontStyles()<< fsBold;
  int hor_center = p_x_start + (p_bar_width / 2.0);
  int text_width = canvas->TextWidth(p_title.c_str());
  int ver_center = p_y_start + (p_bar_height / 2.0);
  int text_height = canvas->TextHeight(p_title.c_str());
  title_bar_rect.Left = title_bar_rect.Left + 1;
  title_bar_rect.Right = title_bar_rect.Right - 1;
  title_bar_rect.Top = title_bar_rect.Top + 1;
  title_bar_rect.Bottom = title_bar_rect.Bottom - 1;
  canvas->TextRect(title_bar_rect, hor_center - (text_width / 2.0), ver_center - (text_height / 2.0), p_title.c_str());
  canvas->Font->Style = TFontStyles();

}
// ------------------------
void classScheduling::DrawSmallMonthDayAbbreviations(int p_x_start, int p_y_start, int p_cell_width, int p_row_height, int p_calendar_width)
{
  int ver_text_center =  p_y_start + (p_row_height / 2.0) - (canvas->TextHeight("1") / 2.0);
  int hor_center;
  int text_width;
  int current_x = p_x_start;
  TRect rect_to_draw_in;
  string to_display;

  canvas->Font->Color = week_titles_color;
  canvas->Brush->Style = bsClear;
  for(int i = 1; i <= 7; i++)
  {
    if(i == 1) to_display = "S";
    else if(i == 2) to_display = "M";
    else if(i == 3) to_display = "T";
    else if(i == 4) to_display = "W";
    else if(i == 5) to_display = "R";
    else if(i == 6) to_display = "F";
    else to_display = "S";
    rect_to_draw_in = Rect(current_x, p_y_start, current_x + p_cell_width, p_y_start + p_row_height);
    hor_center = (p_cell_width / 2.0) + current_x;
    text_width = canvas->TextWidth(to_display.c_str());
    canvas->TextRect(rect_to_draw_in, hor_center - (text_width / 2.0), ver_text_center, to_display.c_str());
    current_x += p_cell_width;
  }
  canvas->Pen->Color = text_color;
  int text_height = canvas->TextHeight("Sample");
  int side_margin = text_height * 0.3;
  int temp_y_pos = ver_text_center + text_height;
  canvas->MoveTo(p_x_start + side_margin, temp_y_pos);
  canvas->LineTo(p_x_start + p_calendar_width - side_margin, temp_y_pos);
  canvas->Brush->Style = bsSolid;
}
// ------------------------
void classScheduling::DrawSmallMonthCalendar(int p_month, int p_year, int p_calendar_width, int p_calendar_height, int p_x_start, int p_y_start)
{
  Pdate temp_date(IntToString(p_month) + "/1/" + IntToString(p_year), mdy | yyyy);
  int current_day = 1;
  int days_in_month = LastDayOf(temp_date.month, temp_date.year);
  int begin_day_of_week = temp_date.DayOfWeek();
  int text_height = canvas->TextHeight("Sample");
  int text_width = canvas->TextWidth("33");
  int title_bar_height = text_height * 2.2;
  int week_titles_height = text_height * 1.5;
  int each_row_height = (p_calendar_height - title_bar_height - week_titles_height) / 6.0;
  int each_column_width = p_calendar_width / 7.0;
  int current_x = p_x_start + 1;
  int current_y = p_y_start + title_bar_height;
  int next_x;
  int next_y;
  bool can_draw_date = false;
  bool is_today;
  int x_text_pos;
  int y_text_pos;
  int hor_cell_center;
  stamp_rect temp_date_rect;
  TRect rect_to_draw_in;
  string to_display;

  DrawSmallMonthOutline(p_x_start, p_y_start, p_calendar_width, p_calendar_height);
  DrawSmallMonthTitleBar(p_x_start, p_y_start, p_calendar_width, title_bar_height, temp_date.NameOfMonth() + " " + IntToString(temp_date.year));
  DrawSmallMonthDayAbbreviations(current_x, current_y, each_column_width, week_titles_height, p_calendar_width-2);
  current_y += week_titles_height;

  canvas->Brush->Style = bsClear;
  canvas->Font->Color = text_color;
  for(int i = 0; i < 6; i++)
  {
    current_y = p_y_start + title_bar_height + week_titles_height + (each_row_height * i);
    next_y = current_y + each_row_height;
    y_text_pos = current_y + (each_row_height / 2.0) - (text_height / 2.0);
    for(int j = 0; j < 7; j++)
    {
      current_x = p_x_start + (each_column_width * j);
      next_x = current_x + each_column_width;
      if(! can_draw_date)
      {
        if((begin_day_of_week - 1) == j)
          can_draw_date = true;
      }
      if(current_day > days_in_month)
        can_draw_date = false;
      if(can_draw_date)
      {
        temp_date_rect.the_stamp.InDate(temp_date);
        temp_date_rect.the_stamp.InTime("08:00", military | withoutsec);
        temp_date_rect.rect = Rect(current_x + 1, current_y, next_x - 1, next_y - 1);
        temp_date_rect.column_name = "";
        stamp_rect_vec.push_back(temp_date_rect);
        to_display = IntToString(current_day);
        is_today = ((temp_date.day == today_date.day) && (temp_date.month == today_date.month) && (temp_date.year == today_date.year));
        if(is_today)
        {
           canvas->Font->Color = today_color;
           canvas->Font->Style = TFontStyles()<< fsBold;
        }
        hor_cell_center = current_x + (each_column_width / 2.0);
        x_text_pos = hor_cell_center - (text_width / 2.0) + (text_width - canvas->TextWidth(to_display.c_str()));
        canvas->TextOut(x_text_pos, y_text_pos, to_display.c_str());
        if(is_today)
        {
          canvas->Font->Style = TFontStyles();
          canvas->Font->Color = text_color;
        }
        current_day++;
        temp_date++;
      }
    }
  }
  canvas->Brush->Style = bsSolid;
}
// ------------------------
int classScheduling::GetPtimeYPosition(Ptime p_the_time, int p_current_calendar_view)
{
  int text_height = canvas->TextHeight("Sample");
  int to_return;
  int seconds_since_begin;
  int each_block_height;
  if(to_printer)
    each_block_height = text_height * 1.30;
  else
    each_block_height = text_height * 1.7;
  Ptime begin_time("07:00:00", military | withsec);

  if(p_the_time < begin_time)
    seconds_since_begin = 0;
  else
    seconds_since_begin = p_the_time - begin_time;
  to_return = y_start_pos + CalcTitleBarHeight(p_current_calendar_view) + ((seconds_since_begin / 900.0) * each_block_height);

  return to_return;
}
// ------------------------
void classScheduling::DrawDayBlockTimes(int &p_left_side_end, int p_current_calendar_view)
{
  int text_height = canvas->TextHeight("Sample");
  int text_width = canvas->TextWidth("00:00 am");
  Ptime current_time("07:00:00", military | withsec);
  Ptime next_time("7:15:00", military | withsec);
  int next_y = GetPtimeYPosition(next_time, p_current_calendar_view);
  int current_y = GetPtimeYPosition(current_time, p_current_calendar_view);
  int left_side_end = x_start_pos + (text_width * 1.2);
  p_left_side_end = left_side_end;
  int hor_center;
  int ver_center;
  stamp_rect temp_stamp_rect;

  stamp_rect_vec.clear();
  canvas->Pen->Color = grid_line_color;
  canvas->Font->Color = text_color;
  canvas->Brush->Color = background_color;
  canvas->MoveTo(left_side_end, title_bar_end);
  canvas->LineTo(left_side_end, y_end_pos);
  for(int i = 0; i < 50; i++)
  {
    text_width = canvas->TextWidth(current_time.OutTime(ampm | withoutsec).c_str());
    hor_center = x_start_pos + ((left_side_end - x_start_pos) / 2.0) - (text_width / 2.0);
    ver_center = current_y + ((next_y - current_y) / 2.0) - (text_height / 2.0);
    canvas->TextOut(hor_center, ver_center, current_time.OutTime(ampm | withoutsec).c_str());
    canvas->MoveTo(x_start_pos, next_y);
    canvas->LineTo(x_end_pos, next_y);
    temp_stamp_rect.the_stamp.InDate(current_date);
    temp_stamp_rect.the_stamp.InTime(current_time);
    temp_stamp_rect.rect = Rect(x_start_pos, current_y + 1, x_end_pos, next_y - 1);
    temp_stamp_rect.column_name = "";
    stamp_rect_vec.push_back(temp_stamp_rect);
    current_y = next_y;
    current_time = next_time;
    next_time.mod(15, in_minutes);
    next_y = GetPtimeYPosition(next_time, p_current_calendar_view);
  }
}
// ------------------------
void classScheduling::DrawBlockText(TRect p_block_rect, string p_text, appointment_occurrence &p_occurrence)
{
  double text_height = canvas->TextHeight("Sample");
  if((p_block_rect.Height() >= text_height) && (p_block_rect.Width() >= text_height) && (p_text.length() > 0))
  {
    double margin;
    if((p_block_rect.Width() <= (text_height * 4.0)) || (p_block_rect.Height() <= (text_height * 4.0)))
      margin = 0;
    else
      margin = text_height * 0.75;
    TRect this_rect = Rect(p_block_rect.Left + margin, p_block_rect.Top + margin,
                           p_block_rect.Right - margin, p_block_rect.Bottom - margin);

    int avg_chars_per_line = RoundNearest((double)this_rect.Width() / (double)canvas->TextWidth("i"));
    string remaining_text = p_text;
    string temp_text;
    vector<string> lines_to_display;
    int current_text_width;
    int total_text_height = 0;
    int j = 0;

    while((remaining_text.length() > 0) && (total_text_height + text_height < this_rect.Height()))
    {
      temp_text = remaining_text.substr(0, avg_chars_per_line);
      current_text_width = canvas->TextWidth(temp_text.c_str());
        if(j == 0)
          current_text_width += RoundNearest(text_height * 1.3) + 1;
      while((temp_text.length() > 0) && (current_text_width > this_rect.Width()))
      {
        temp_text = temp_text.substr(0, temp_text.length() - 1);
        current_text_width = canvas->TextWidth(temp_text.c_str());
        if(j == 0)
          current_text_width += RoundNearest(text_height * 1.3) + 1;;
      }
      unsigned int space_pos = temp_text.find_last_of(' ');
      if((space_pos != string::npos) && (space_pos > 0) && (remaining_text.length() != temp_text.length()))
      {
        unsigned int i = temp_text.length() - 1;
        while(i >= space_pos)
        {
          temp_text = temp_text.substr(0, temp_text.length() - 1);
          i--;
        }
      }
      lines_to_display.push_back(temp_text);
      remaining_text = remaining_text.substr(temp_text.length(), remaining_text.length() - temp_text.length());
      remaining_text = StripLeading(remaining_text, " ");
      total_text_height += text_height * 1.5;
      j++;
    }

    canvas->Brush->Style = bsClear;
    canvas->Font->Color = text_color;
    total_text_height -= RoundNearest(text_height * 0.4);
    int current_y = this_rect.Top + RoundNearest(((double)this_rect.Height() / 2.0) - ((double)total_text_height / 2.0));
    int current_x;
    int i = 0;
    for(vector<string>::iterator itr = lines_to_display.begin(); itr != lines_to_display.end(); itr++, i++)
    {
      if(i == 0)
      {
        current_x = this_rect.Left + RoundNearest(((double)this_rect.Width() / 2.0) - ((double)(canvas->TextWidth((*itr).c_str()) + (text_height * 1.3)) / 2.0));
        if(show_clock_icons) // jhaley 20100820: made clocks optional
        {
          DrawClockIcon(canvas, current_x, current_y, text_height+1, GetIconType(p_occurrence.outcome),
                        p_occurrence.range_begin.hour, p_occurrence.range_begin.minute);
          current_x += RoundNearest(text_height * 1.3) + 1;
        }
      }
      else
        current_x = this_rect.Left + RoundNearest(((double)this_rect.Width() / 2.0) - ((double)canvas->TextWidth((*itr).c_str()) / 2.0));
      canvas->TextRect(this_rect, current_x, current_y, (*itr).c_str());
      current_y += RoundNearest(text_height * 1.5);
    }
    canvas->Brush->Style = bsSolid;
  }
}
// ------------------------
void classScheduling::DrawDayBlockAppointments(int p_this_day_start, int p_this_day_end, Pdate p_date_to_show,
                                               PApptOccurrenceMap &p_appointment_map, int p_current_calendar_view)
{
  PApptOccurrenceMap temp_appointment_map = p_appointment_map;
  PApptOccurrenceMap::iterator i_day = temp_appointment_map.find(p_date_to_show);
  if(i_day != temp_appointment_map.end())
  {
    Ptime temp_p_time;
    Ptime last_time;
    last_time.InTime("19:27:00", military | withsec);
    Ptime seven_thirty;
    seven_thirty.InTime("19:30:00", military | withsec);
    for(PApptOccurrenceSet::iterator itr = i_day->second.begin(); itr != i_day->second.end(); itr++)
    {
      if(itr->range_begin == itr->range_end)
      {
        if((itr->range_begin >= last_time) && (itr->range_begin <= seven_thirty))
          itr->range_end.InTime("19:30:00", military | withsec);
        else
        {
          temp_p_time = itr->range_begin;
          temp_p_time.mod(4, in_minutes);
          itr->range_end = temp_p_time;
        }
      }
    }
    map<range<Ptime>, vector<PApptOccurrenceSet> > block_layout_vec;
    range_group_block_layout(i_day->second, block_layout_vec);
    int this_y_start;
    int this_y_end;
    int this_x_start;
    int this_x_end;
    TRect rect_to_draw;
    appt_rect temp_appt_rect;
    double i = 0;

    for(map<range<Ptime>, vector<PApptOccurrenceSet> >::iterator itr = block_layout_vec.begin();
        itr != block_layout_vec.end(); itr++)
    {
      i = 0;
      for(vector<PApptOccurrenceSet>::iterator vec_itr = itr->second.begin(); vec_itr != itr->second.end(); vec_itr++, i++)
      {
        this_x_start = p_this_day_start + RoundNearest((double)(p_this_day_end - p_this_day_start) * (i/(double)itr->second.size()));
        this_x_end = p_this_day_start + RoundNearest((double)(p_this_day_end - p_this_day_start) * ((i+1.0)/(double)itr->second.size()));
        for(PApptOccurrenceSet::iterator appt_itr = vec_itr->begin(); appt_itr != vec_itr->end(); appt_itr++)
        {
          if(appt_itr->range_begin <= Ptime("19:30:00", military | withsec))
          {
            this_y_start = GetPtimeYPosition(appt_itr->range_begin, p_current_calendar_view);
            this_y_end = GetPtimeYPosition(appt_itr->range_end, p_current_calendar_view);
            canvas->Brush->Color = user_color_map[appt_itr->user];
            DrawLine(this_x_start, this_y_start, this_x_end, this_y_start);
            DrawLine(this_x_end, this_y_start, this_x_end, this_y_end);
            DrawLine(this_x_end, this_y_end, this_x_start, this_y_end);
            DrawLine(this_x_start, this_y_end, this_x_start, this_y_start);
            rect_to_draw = Rect(this_x_start+1, this_y_start+1, this_x_end, this_y_end);
            if(this_x_end == x_end_pos)
              rect_to_draw.Right = x_end_pos - 1;
            canvas->FillRect(rect_to_draw);
            temp_appt_rect.id = appt_itr->id;
            temp_appt_rect.rect = rect_to_draw;
            appointment_rect_vec.push_back(temp_appt_rect);
            DrawBlockText(rect_to_draw, GetDayWeekViewDisplayText((*appt_itr), block_day_calendar), (*appt_itr));
          }
        }
      }
    }
  } // end if
}
// ------------------------
void classScheduling::DrawWeekBlockTitle(int p_left_side_end, PApptOccurrenceMap &p_appointment_map, int p_current_calendar_view)
{
  Pdate begin_date;
  Pdate end_date;
  GetBeginEndDates(begin_date, end_date, block_week_calendar);
  begin_date.mod(1, in_days);
  int this_x_start = p_left_side_end;
  int this_x_end;
  int i = 0;
  string title_text;
  canvas->Font->Color = text_color;
  vector<stamp_rect> temp_vec;

  while(begin_date < end_date)
  {
    title_text = begin_date.OutDate(mdy | yyyy);
    this_x_start = p_left_side_end + ((used_width - (p_left_side_end - x_start_pos)) * (i/5.0));
    this_x_end = p_left_side_end + ((used_width - (p_left_side_end - x_start_pos)) * ((i+1)/5.0));
    canvas->Brush->Style = bsClear;
    canvas->Font->Style = TFontStyles()<< fsBold;
    canvas->TextRect(Rect(this_x_start, y_start_pos, this_x_end, y_start_pos + title_bar_height),
                    (this_x_end) - ((this_x_end - this_x_start) / 2.0) - (canvas->TextWidth(title_text.c_str()) / 2.0),
                    (y_start_pos + title_bar_height) - (title_bar_height / 2.0) - (canvas->TextHeight(title_text.c_str()) / 2.0),
                    title_text.c_str());
    canvas->Font->Style = TFontStyles();
    canvas->Brush->Style = bsSolid;
    DrawLine(this_x_start, y_start_pos, this_x_start, y_end_pos);
    DrawDayBlockAppointments(this_x_start, this_x_end, begin_date, p_appointment_map, p_current_calendar_view);

    // previously stamp_rect_vec contained one region per "row" ... we're going to create a new region per column,
    // per row, then take that temporary structure and replace the original stamp_rect_vec with it.  Thus we
    // convert a per-row set of clickable regions to a grid of row/col clickable regions.
    stamp_rect temp_struct;
    for(vector<stamp_rect>::iterator itr = stamp_rect_vec.begin(); itr != stamp_rect_vec.end(); itr++)
    {
      temp_struct.rect = Rect(this_x_start, itr->rect.Top, this_x_end, itr->rect.Bottom);
      temp_struct.the_stamp.InDate(begin_date);
      temp_struct.the_stamp.InTime((Ptime)itr->the_stamp);
      temp_struct.column_name = "";
      temp_vec.push_back(temp_struct);
    }
    stamp_rect_vec = temp_vec;
    begin_date.mod(1, in_days);
    i++;
  }
}
// ------------------------
int classScheduling::TitleBarRowCenter(int row)
{
  // note that returned value is an offset from... wherever the title bar is
  // add top margin always, row-1 * row+row_padding, and then half a row (return row center, remember?)
  int row_height = canvas->TextHeight("Sample");
  int result = (row_height * title_bar_vert_margin) +
               (row - 1) * (row_height * (1 + title_bar_row_padding)) +
               (row_height / 2.0);
  //Application->MessageBox(IntToString(result).c_str(), ("Gonna put stuff on row " + IntToString(row) + " at: ").c_str(), MB_OK);
  return result;
}
// ------------------------
int classScheduling::GetRowsNeeded(int p_current_calendar_view)
{
 switch (p_current_calendar_view)
 {
  case day_calendar:
  case week_calendar:
  case month_calendar:
  case year_calendar:
  case block_day_calendar:
  case block_week_calendar:
     return 1;
  case block_location_calendar:
  case block_reason_calendar:
  case block_userpos_calendar:
  case block_outcome_calendar:
  case block_provider_calendar:
     return 2;
  default:
     return 0;
 }
}
// ------------------------
// jhaley 20100820: Support for toggling display of clock icons on/off.
void classScheduling::SetShowClockIcons(bool p_show_clock_icons)
{
  show_clock_icons = p_show_clock_icons;
}
// ------------------------
// jhaley 20100820: Support for toggling display of clock icons on/off.
bool classScheduling::GetShowClockIcons()
{
  return show_clock_icons;
}
// end scheduling class --------------------------------------------------------
bool DownloadAppointments(TIBDatabase *dbApptDatabase, Pdate begin_download_date, Pdate end_download_date, string user_id, vector<string> user_ids,
                          vector<string> group_ids, string tax_entity_id, PApptOccurrenceMap &all_appointments,
                          PApptIDMapVec &ids_to_match, bool p_view_all, bool show_chart_num, string chart_location)
{
  if(debug_level[libraries]>=2 && message_router)
  {
    map<string, string> params;
    params["begin_date"] = begin_download_date.OutDate(global_mode);
    params["end_date"] = end_download_date.OutDate(global_mode);
    params["user_id"] = user_id;
    params["user_ids"] = VecToCommaString(user_ids);
    params["group_ids"] = VecToCommaString(group_ids);
    params["tax_entity_id"] = tax_entity_id;
    params["ids_to_match"] = "{";
    for(map<string, vector<string> >::iterator itr = ids_to_match.begin(); itr != ids_to_match.end(); itr++)
       params["ids_to_match"] += ("{" + itr->first + ",(" + VecToDelimString(itr->second, "", "", ",")) + ")}";
    params["ids_to_match"] += "}";
    params["p_view_all"] = IntToBoolString(BoolToInt(p_view_all));
    params["show_chart_num"] = IntToBoolString(BoolToInt(show_chart_num));
    params["chart_location"] = chart_location;
    message_router->SendMessageToAll("debug", "DownloadAppointments (" + MapToDelimString(params, "{", "}", "", "'", "'", "'", "'", ",") + ")");
  }
  string sql;
  /* TODO -c HACK: We need to do an intersection between the set of group_ids passed in, and the set of
     groups they have general permission to edit, and -then- get the list of corresponding users.
     Something like (untested!) :

     set<string> requested_groups, can_view, groups_to_fetch;
     requested_groups.clear();
     VecToSet(group_ids, requested_groups);
     can_view.clear();
     // by table.appointments.x permissions specifically ... ?
     can_view = GetGroupPermissions("table.appointments", "", "view", "true");
     // or just special.x.general permissions ... ?
     can_view = GetSpecialGroupPermissions("general");
     groups_to_fetch = intersect(requested_groups, can_view);
  */
  if (! group_ids.empty())
    SqlToVec(dbDatabase, "select id from users where " + SafeInClause("group_id", group_ids), user_ids);
  string user_id_string = VecToCommaString(user_ids);

  sql = "select appointments.id, appointments.description, appointments.repeating_string, "
        "appointments.appointment_time, appointments.created_for, "
        "person_names.first_name || ' ' || person_names.last_name person, "
        "appointments.duration, duration_types.item duration_type, "
        "appointments.begin_date, appointments.end_date, appointments.final_end_stamp, "
        "visit_outcome_types.item outcome, appt_location_types.item appt_location, ";
  if(show_chart_num)
  {
    /*sql += "(select first 1 charts.chart_number from charts where charts.person_id = people.id "
           "and charts.group_id = " + chart_location + " and charts.current_id > 2 order by charts.last_update_stamp desc) chart_number, ";
    */
    set<string> can_see_charts = GetSpecialGroupPermissions("charts");
    sql += "(select first 1 charts.chart_number from charts "
            //"charts.group_id = '" + lookupTables->IdOf("groups", cboClinic->Text.c_str()) + "' ";
            "where charts.person_id = people.id ";
    if (!IsBlankOrZero(chart_location))
      sql += "and charts.location_id = " + chart_location + " ";
    sql +=  "and charts.current_id > 2 "
            "order by case when charts.group_id = appointments.group_id then 1 else 0 end desc, "
            "case when appt_location_types.chart_location_id = appt_group.chart_location_id then 1 else 0 end desc, "
            "charts.last_update_stamp desc) chart_number, ";
  }
  sql += "appt_reason_types.item appt_reason, cuserpos.item userpos, "
         "coalesce(cast(appt_reason_types.lac_ordinal as varchar(10)) || ' - ' || appt_reason_types.item, '') lac_reason, "
         "appointments.within_24_hours, appointments.group_id, tax_entities.name provider_name "
         "from appointments ";

// start insert here
/*
  if((! user_ids.empty()) && (! p_view_all))
    sql += ", appointment_viewers ";
  sql += ", duration_types, visit_outcome_types, appt_location_types, "
        "people left join person_names on people.person_names_id = person_names.id "
        "where appointments.person_id = people.id and "
        "appointments.duration_id = duration_types.id and "
        "appointments.visit_outcome_id = visit_outcome_types.id and "
        "appointments.appt_location_id = appt_location_types.id and "
*/
  if(! p_view_all)
    sql += " left join appointment_viewers on appointment_viewers.appointment_id = appointments.id ";
  sql += " left join duration_types on duration_types.id = appointments.duration_id "
         " left join visit_outcome_types on visit_outcome_types.id = appointments.visit_outcome_id "
         " left join appt_location_types on appt_location_types.id = appointments.appt_location_id "
         " left join appt_reason_types on appt_reason_types.id = appointments.appt_reason_id "
         " left join users cuser on appointments.created_for = cuser.id "
         " left join user_position_types cuserpos on cuser.user_position_id = cuserpos.id "
         " left join groups appt_group on appt_group.id = appointments.group_id "
         " left join people on people.id = appointments.person_id "
         " left join person_names on person_names.id = people.person_names_id "
         " left join tax_entities on tax_entities.id = appointments.tax_entity_id "
         " where "
// end insert here
         "appointments.begin_date <= '" + end_download_date.OutDate(mdy | yyyy) + "' and " +
         "appointments.final_end_stamp >= '" + begin_download_date.OutDate(mdy | yyyy) + " 00:00:00' ";
  if(! ids_to_match.empty())
    for(PApptIDMapVec::iterator itr = ids_to_match.begin(); itr != ids_to_match.end(); itr++)
      if (!itr->second.empty())
        sql += "and (" + SafeInClause("appointments." + itr->first, itr->second) + ") ";

  /* DONE -c permissions : moved this block from inside the
     if (! user_ids.empty())
     below, to here, because this should be done globally,
     regardless of what we're searching by; permissions
     to view appointments returned should always be checked! */
  if(! p_view_all)
    sql += "and appointment_viewers.user_id = " + user_id + " ";

  if(! user_ids.empty())
  {
    sql += "and " + SafeInClause("appointments.created_for", user_ids) + " ";
  }
  else if((tax_entity_id != "") && (IsInt(tax_entity_id)) && (StringToInt(tax_entity_id) > 0))
    sql += "and appointments.tax_entity_id = " + tax_entity_id;
  else if (ids_to_match.find("group_id") != ids_to_match.end() && !ids_to_match["group_id"].empty())
    ; // this is already handled above, but if we have group_id(s), we don't want to exit!
  else
    return false;
  sql += " order by appointments.appointment_time";

  // debug
  //DebugSql(NULL, &sql, libraries, 3);

  TIBTransaction *dbTransaction = NULL;
  FORM_TRY
    dbTransaction = new TIBTransaction(NULL);
  FORM_CATCH

  TIBDataSet *dbDataset = NULL;
  FORM_TRY
    dbDataset = new TIBDataSet(NULL);
  FORM_CATCH
  repeat_spec<int> temp_repeat;
  vector<Pdate> date_matches;
  Pstamp final_end_stamp;
  Pdate begin_date;
  Pdate end_date;
  Pdate temp_date;
  Ptime appt_time;
  range<Pdate> shown_range = range<Pdate>(begin_download_date, end_download_date);
  appointment_occurrence one_occurrence;
  bool to_return = true;
  int duration;
  string duration_type;
  all_appointments.clear();

  try
  {
    dbTransaction->DefaultDatabase = dbApptDatabase;
    dbDataset->Database = dbApptDatabase;
    dbDataset->Transaction = dbTransaction;

    dbTransaction->StartTransaction();
    dbDataset->SelectSQL->Add(sql.c_str());
    dbDataset->Open();
    dbDataset->First();
    while(! dbDataset->Eof)
    {
      appt_time.InTime(dbDataset->FieldByName("appointment_time")->AsString.c_str(), military | withsec);
      begin_date.InDate(dbDataset->FieldByName("begin_date")->AsString.c_str(), mdy | yyyy);
      end_date.InDate(dbDataset->FieldByName("end_date")->AsString.c_str(), mdy | yyyy);
      temp_repeat.Load(dbDataset->FieldByName("repeating_string")->AsString.c_str(), begin_date, end_date);
      one_occurrence.id = dbDataset->FieldByName("id")->AsString.c_str();
      one_occurrence.person = PrettyString(dbDataset->FieldByName("person")->AsString.c_str());
      one_occurrence.group = dbDataset->FieldByName("group_id")->AsString.c_str();
      one_occurrence.description = SentenceString(dbDataset->FieldByName("description")->AsString.c_str());
      one_occurrence.user = dbDataset->FieldByName("created_for")->AsString.c_str();
      one_occurrence.outcome = LowercaseString(dbDataset->FieldByName("outcome")->AsString.c_str());
      one_occurrence.provider = dbDataset->FieldByName("provider_name")->AsString.c_str();
      one_occurrence.location = PrettyString(dbDataset->FieldByName("appt_location")->AsString.c_str());
      one_occurrence.userpos = dbDataset->FieldByName("userpos")->AsString.c_str();

      // display raw because that's the way it's stored.  "PA Appointment" etc. capitalized correctly.
      if (optionsPermissions["special.lac.general"]["enabled"] == "true" || optionsPermissions["special.sc.general"]["enabled"] == "true")
         one_occurrence.reason = dbDataset->FieldByName("lac_reason")->AsString.c_str();
      else
         one_occurrence.reason = PrettyString(dbDataset->FieldByName("appt_reason")->AsString.c_str());

      if(show_chart_num)
         one_occurrence.chart_number = PrettyString(dbDataset->FieldByName("chart_number")->AsString.c_str());
      final_end_stamp.InStamp(dbDataset->FieldByName("final_end_stamp")->AsString.c_str(), mdy | yyyy | military | withsec | datefirst);
      duration = StringToInt(dbDataset->FieldByName("duration")->AsString.c_str());
      duration_type = LowercaseString(dbDataset->FieldByName("duration_type")->AsString.c_str());

      date_matches = temp_repeat.AllMatches(date_range(begin_date, end_date));
      for(vector<Pdate>::iterator itr = date_matches.begin(); itr != date_matches.end(); itr++)
      {
        Pstamp match_begin;
        match_begin.InDate(*itr);
        match_begin.InTime(appt_time);
        Pstamp match_end = match_begin;

        if((duration > 0) && (duration_type != ""))
        {
          if(duration_type == "minutes")
            match_end.mod(duration, in_minutes);
          else if(duration_type == "hours")
            match_end.mod(duration, in_hours);
          else if(duration_type == "days")
            match_end.mod(duration, in_days);
          else if(duration_type == "weeks")
            match_end.mod(duration, in_weeks);
          else if(duration_type == "months")
            match_end.mod(duration, in_months);
          else if(duration_type == "years")
            match_end.mod(duration, in_years);
        }

        if ((Pdate)match_begin != (Pdate)match_end)
        {
         // split into first, last, and middle days
          if(shown_range.ContainEq((Pdate)match_begin))
          {
            one_occurrence.range_begin = (Ptime)match_begin;
            one_occurrence.range_end = Ptime("23:59", military | withoutsec);
            all_appointments[(Pdate)match_begin].insert(one_occurrence);
          }

          if(shown_range.ContainEq((Pdate)final_end_stamp))
          {
            one_occurrence.range_begin = Ptime("0:00", military | withoutsec);
            one_occurrence.range_end = (Ptime)match_end;
            all_appointments[(Pdate)match_end].insert(one_occurrence);
          }

          temp_date = (Pdate)match_begin;
          temp_date.mod(1, in_days);
          while(temp_date < (Pdate)match_end)
          {
            if(shown_range.ContainEq(temp_date))
            {
              one_occurrence.range_begin = Ptime("0:00", military | withoutsec);
              one_occurrence.range_end = Ptime("23:59", military | withoutsec);
              all_appointments[temp_date].insert(one_occurrence);
            }
            temp_date++;
          }
        }
        else
        {
         // all on one day
          if(shown_range.ContainEq(*itr))
          {
            one_occurrence.range_begin = (Ptime)match_begin;
            one_occurrence.range_end = (Ptime)match_end;
            all_appointments[*itr].insert(one_occurrence);
          }
        }
      } // end for date_matches
      dbDataset->Next();
    } // end while not eof
    dbDataset->Close();
    dbTransaction->Commit();
  }
  catch(...)
  {
    to_return = false;
    if(dbTransaction->Active)
      dbTransaction->Rollback();
  }

  delete dbTransaction;
  delete dbDataset;
  return to_return;
}
// ------------------------
bool DownloadAppointments(TIBDatabase *dbApptDatabase, Pdate begin_download_date, Pdate end_download_date,
                          string user_id, vector<string> user_ids, string tax_entity_id,
                          PApptOccurrenceMap &all_appointments, PApptIDMapVec &ids_to_match, bool p_view_all,
                          bool show_chart_num, string chart_location)
{
  /* without group_ids; just ignore it */
  return DownloadAppointments(dbApptDatabase, begin_download_date, end_download_date, user_id, user_ids,
                              vector<string>(), tax_entity_id, all_appointments, ids_to_match, p_view_all,
                              show_chart_num, chart_location);
}
// ------------------------
void DrawClockIcon (TCanvas *canvas, int icon_left, int icon_top, int icon_size, clock_icon_types icon,
                    int int_hour, int int_min)
{
 TColor old_font_color = canvas->Font->Color;
 int old_size = canvas->Font->Size;
 string old_font_name = canvas->Font->Name.c_str();
 switch (icon)
 {
  case clock_normal:
   DrawClock (canvas, icon_left, icon_top, icon_size / 2, clBlack, clWhite, clBlack, clBlack, true, false, int_hour,
              int_min, 0.50, 0.75, true, 12, clGray, 0.85, 0.95, true, clBlack); // normal
   break;
  case clock_rescheduled:
   DrawClock (canvas, icon_left, icon_top, icon_size * 0.375, clBlack, clWhite, clBlack, clBlack, true, false, int_hour,
              int_min, 0.50, 0.75, true, 12, clGray, 0.85, 0.95, true, clBlack); // resched
   DrawClock (canvas, icon_left + icon_size / 4, icon_top + icon_size / 8, icon_size * 0.375, clBlack, clWhite, clBlack,
              clBlack, true, false, int_hour, int_min, 0.50, 0.75, true, 12, clGray, 0.85, 0.95, true, clBlack);
   break;
  case clock_missed:
   DrawClock (canvas, icon_left, icon_top, icon_size / 2, clBlack, clWhite, clBlack, clBlack, false, false, int_hour,
              int_min, 0.50, 0.75, true, 12, clGray, 0.85, 0.95, false, clBlack); // ?
   canvas->Brush->Style = bsClear;
   canvas->Font->Name = "Arial";
   canvas->Font->Color = clRed;
   canvas->Font->Height = icon_size * 0.80;
   canvas->TextOutA(icon_left + icon_size / 2 - canvas->TextWidth("?") / 2, icon_top + (icon_size / 2) - (canvas->Font->Height / 2) , "?");
   break;
  case clock_cancelled:
   DrawClock (canvas, icon_left, icon_top, icon_size / 2, clBlack, clWhite, clBlack, clBlack, false, false, int_hour,
              int_min, 0.50, 0.75, true, 12, clGray, 0.85, 0.95, false, clBlack); // X
   canvas->Brush->Style = bsClear;
   canvas->Font->Name = "Arial";
   canvas->Font->Color = clRed;
   canvas->Font->Height = icon_size * 0.80;
   canvas->TextOutA(icon_left + icon_size / 2 - canvas->TextWidth("X") / 2, icon_top + (icon_size / 2) - (canvas->Font->Height / 2) , "X");
   break;
  default:
   DrawClock (canvas, icon_left, icon_top, icon_size / 2, clBlack, clWhite, clBlack, clBlack, false, false, int_hour,
              int_min, 0.50, 0.75, true, 12, clGray, 0.85, 0.95, true, clBlack); // unknown
   break;
 }
 canvas->Font->Size = old_size;
 canvas->Font->Color = old_font_color;
 canvas->Font->Name = old_font_name.c_str();
}
// ------------------------
void DrawClock (TCanvas *canvas, int int_x, int int_y, int int_r, TColor col_rim, TColor col_face,
                                 TColor col_hour, TColor col_min, bool bool_time, bool bool_follow, int int_hour, int int_min,
                                 double dbl_hour, double dbl_min, bool bool_ticks, int int_ticks, TColor col_ticks,
                                 double dbl_inner, double dbl_outer, bool bool_dot, TColor col_dot)
{
 // draw the face of a clock within the given area, with hour/minute hands and stuff

 // save ...
 TColor old_brush_color      = canvas->Brush->Color;
 TBrushStyle old_brush_style = canvas->Brush->Style;
 TColor old_pen_color        = canvas->Pen->Color;
 TPenMode old_pen_mod        = canvas->Pen->Mode;
 TPenStyle old_pen_style     = canvas->Pen->Style;
 int old_pen_width           = canvas->Pen->Width;

 if (canvas)
 {
  // draw the base face
  canvas->Brush->Color = col_face;
  canvas->Brush->Style = bsSolid;

  canvas->Pen->Color   = col_rim;
  canvas->Pen->Mode    = pmCopy;
  canvas->Pen->Style   = psSolid;
  canvas->Pen->Width   = 1;

  // face and rim
  canvas->Ellipse(int_x, int_y, int_x + int_r + int_r, int_y + int_r + int_r);

  if (bool_ticks)
  {
   double angle_tick;

   for (int int_i = 0; int_i < int_ticks; int_i++)
   {
    angle_tick  = (2.0 * M_PI * (double(int_i) / double(int_ticks))) - M_PI_2;

    // minute hand
    canvas->Pen->Color = col_ticks;
    canvas->MoveTo (int_x + int_r + (cos(angle_tick) * dbl_inner * int_r), int_y + int_r + (sin(angle_tick) * dbl_inner * int_r));
    canvas->LineTo (int_x + int_r + (cos(angle_tick) * dbl_outer * int_r), int_y + int_r + (sin(angle_tick) * dbl_outer * int_r));
   }
  }

  if (bool_time)
  {
   double angle_hour;
   if (bool_follow)
    angle_hour  = (2.0 * M_PI * (float((int_hour * 60.0) + int_min) / float(12.0 * 60.0))) - M_PI_2;
   else
    angle_hour  = (2.0 * M_PI * (float(int_hour) / float(12.0))) - M_PI_2;
   double angle_min   = (2.0 * M_PI * ((int_min) / (60.0))) - M_PI_2;

   // minute hand
   canvas->Pen->Color = col_min;
   canvas->MoveTo (int_x + int_r, int_y + int_r);
   canvas->LineTo (int_x + int_r + (cos(angle_min) * dbl_min * int_r), int_y + int_r + (sin(angle_min) * dbl_min * int_r));

   // hour hand
   canvas->Pen->Color = col_hour;
   canvas->MoveTo (int_x + int_r, int_y + int_r);
   canvas->LineTo (int_x + int_r + (cos(angle_hour) * dbl_hour * int_r), int_y + int_r + (sin(angle_hour) * dbl_hour * int_r));
  }

  if (bool_dot)
  {
   canvas->Pen->Color = col_dot;
   canvas->Pixels[int_x + int_r][int_y + int_r] = col_dot;
  }

  // redraw the rim, just in case
  if (bool_dot || bool_ticks)
  {
   canvas->Brush->Color = col_face;
   canvas->Brush->Style = bsClear;

   canvas->Pen->Color   = col_rim;
   canvas->Pen->Mode    = pmCopy;
   canvas->Pen->Style   = psSolid;
   canvas->Pen->Width   = 1;

   canvas->Ellipse(int_x, int_y, int_x + int_r + int_r, int_y + int_r + int_r);
  }

  // ... restore!
  canvas->Brush->Color = old_brush_color;
  canvas->Brush->Style = old_brush_style;
  canvas->Pen->Color   = old_pen_color;
  canvas->Pen->Mode    = old_pen_mod;
  canvas->Pen->Style   = old_pen_style;
  canvas->Pen->Width   = old_pen_width;
 }
 return;
}
// ------------------------
clock_icon_types GetIconType(string outcome)
{
  if((outcome == "") || (outcome == "as scheduled") || (outcome == "walk-in") || (outcome == "arrived late"))
    return clock_normal;
  else if((outcome == "no one home") || (outcome == "missed appointment") || (outcome == "no show"))
    return clock_missed;
  else if(outcome == "rescheduled")
    return clock_rescheduled;
  else if(outcome == "cancelled")
    return clock_cancelled;
  else
    return clock_unknown;
}
// ------------------------

