#ifndef classScheduling_h
#define classScheduling_h

#include "util.h"
#include "repeating.h"
#include "ranges.h"
#include "lookuptablewrapper.h" //check group for appointment location (block location view)

#define day_calendar 0
#define week_calendar 1
#define month_calendar 2
#define year_calendar 3
#define block_day_calendar 4
#define block_week_calendar 5
#define block_location_calendar 6
#define block_reason_calendar 7
#define block_userpos_calendar 8
#define block_outcome_calendar 9
#define block_provider_calendar 10

class appointment_occurrence : public range <Ptime>
{
 public:
  string id;
  string person;
  string group;
  string description;
  string user;
  string outcome;
  string location;
  string reason;
  string userpos;
  string provider;
  string chart_number;
};

class appt_rect
{
  public:
   string id;
   TRect rect;
};

class stamp_rect
{
  public:
   Pstamp the_stamp;
   TRect rect;
   string column_name;
};

enum clock_icon_types {clock_normal, clock_rescheduled, clock_missed, clock_cancelled, clock_unknown};

void DrawClock(TCanvas *canvas, int int_x, int int_y, int int_r, TColor col_rim,
               TColor col_face, TColor col_hour, TColor col_min, bool bool_time, bool bool_follow, int int_hour,
               int int_min, double dbl_hour, double dbl_min, bool bool_ticks, int int_ticks, TColor col_ticks,
               double dbl_inner, double dbl_outer, bool bool_dot, TColor col_dot);
void DrawClockIcon(TCanvas *canvas, int icon_left, int icon_top, int icon_size, clock_icon_types icon,
                   int int_hour, int int_min);
clock_icon_types GetIconType(string outcome);

// jhaley 20110720: Typedefs for hideous STL appointment composite types:
typedef multiset<appointment_occurrence> PApptOccurrenceSet;
typedef map<Pdate, PApptOccurrenceSet>   PApptOccurrenceMap;
typedef map<string, vector<string> >     PApptIDMapVec;

class classScheduling
{
  private:
    TCanvas *canvas;
    Graphics::TBitmap *bitmap;
    TCanvas *passed_canvas;
    int canvas_width;
    int canvas_height;
    float percentage_used;
    int used_width;
    int used_height;
    int x_start_pos;
    int y_start_pos;
    int x_end_pos;
    int y_end_pos;
    int title_bar_height;
    float title_bar_vert_margin;
    float title_bar_row_padding;
    int title_bar_end;
    Pdate current_date;
    Pdate today_date;
    TColor title_bar_color;
    TColor background_color;
    TColor grid_line_color;
    TColor text_color;
    TColor week_titles_color;
    TColor today_color;
    TColor month_appt_color;
    TColor year_margin_color;
    map<string, TColor> user_color_map;
    vector<appt_rect> appointment_rect_vec;
    vector<stamp_rect> stamp_rect_vec;
    bool to_printer;
    bool show_clock_icons;
    // int rows_in_title_bar;

    void DrawTitleBar(int p_current_calendar_view);
    void DrawBackground(int p_current_calendar_view, TColor p_background_color = (TColor)0x00ffffff);
    int CalcTitleBarHeight(int p_current_calendar_view);
    void SetTitleBarSize(int p_current_calendar_view);
    void DrawCalendarTitle(int calendar_type);
    void DrawHorizontalLines(int start_pos, int end_pos, int num_of_lines);
    void DrawLine(int x_begin, int y_begin, int x_end, int y_end);
    void DrawDayNamesOnMonthCalendar();
    void OutputTodayText(int x_text_start, int y_text_start, string text_to_display);
    void DrawDatesOnMonthCalendar();
    void CopyBitmapImageToCanvas();
    void DrawAppointmentsOnDayView(PApptOccurrenceMap &p_appointment_map);
    void DrawAppointmentsOnWeekView(PApptOccurrenceMap &p_appointment_map);
    void DrawAppointmentsOnMonthCalendar(PApptOccurrenceMap &p_appointment_map);
    void DrawDayNameOnWeekCalendar(int x_pos, int y_pos, Pdate date_to_draw);
    void DrawUserColorSquare(string user_id, int x_pos, int y_pos, int dimensions);
    string GetDayWeekViewDisplayText(appointment_occurrence p_occurrence, int p_current_calendar_view);
    void DrawDayWeekAppointment(int this_x_start, int &current_y_pos, int text_height, int p_current_calendar_view, appointment_occurrence p_occurrence);
    bool IsInRectangle(int p_x_pos, int p_y_pos, TRect p_rect);
    void DrawSmallMonthCalendar(int p_month, int p_year, int p_calendar_width, int p_calendar_height, int p_x_start, int p_y_start);
    void DrawSmallMonthTitleBar(int p_x_start, int p_y_start, int p_bar_width, int p_bar_height, string p_title);
    void DrawSmallMonthOutline(int p_x_start, int p_y_start, int p_calendar_width, int p_calendar_height);
    void DrawSmallMonthDayAbbreviations(int p_x_start, int p_y_start, int p_row_width, int p_row_height, int p_calendar_width);
    void DrawDayBlockTimes(int &p_left_side_end, int p_current_calendar_view);
    void DrawDayBlockAppointments(int p_this_day_start, int p_this_day_end, Pdate p_date_to_show, PApptOccurrenceMap &p_appointment_map, int p_current_calendar_view);
    int GetPtimeYPosition(Ptime p_the_time, int p_current_calendar_view);
    void DrawBlockText(TRect p_block_rect, string p_text, appointment_occurrence &p_occurrence);
    void DrawWeekBlockTitle(int p_left_side_end, PApptOccurrenceMap &p_appointment_map, int p_current_calendar_view);
    int TitleBarRowCenter(int row);
    int GetRowsNeeded(int p_current_calendar_view);

  public:
    classScheduling(TCanvas * canvas, int p_canvas_width, int p_canvas_height, Pdate p_current_date,
                    Pdate p_today_date, map<string, TColor> p_user_color_map,
                    float p_percentage_used = 1.0, int p_font_size = 9, bool p_to_printer = false, bool p_show_clock_icons = true);
    ~classScheduling();
    void SetCanvasDimensions(int p_canvas_width, int p_canvas_height);
    void DrawDayCalendar(PApptOccurrenceMap &p_appointment_map);
    void DrawWeekCalendar(PApptOccurrenceMap &p_appointment_map);
    void DrawMonthCalendar(PApptOccurrenceMap &p_appointment_map);
    void DrawYearCalendar(TColor p_background_color = (TColor)0x00ffffff);
    void DrawBlockDayCalendar(PApptOccurrenceMap &p_appointment_map);
    void DrawBlockLocationCalendar(PApptOccurrenceMap &p_appointment_map);
    void DrawBlockReasonCalendar(PApptOccurrenceMap &p_appointment_map);
    void DrawBlockUserPosCalendar(PApptOccurrenceMap &p_appointment_map);
    void DrawBlockOutcomeCalendar(PApptOccurrenceMap &p_appointment_map);
    void DrawBlockProviderCalendar(PApptOccurrenceMap &p_appointment_map);
    void DrawBlockWeekCalendar(PApptOccurrenceMap &p_appointment_map);
    Pdate GetCurrentDate();
    void SetFontSize(int p_font_size, int p_current_calendar_view);
    void SetCurrentDate(Pdate p_new_date);
    TColor GetTitleBarColor();
    void GetBeginEndDates(Pdate &p_begin_date, Pdate &p_end_date, int calendar_view);
    int GetHeightNeeded(PApptOccurrenceMap &p_appointment_map, int calendar_view);
    string GetAppointmentIdAtPosition(int p_x_pos, int p_y_pos);
    void GetStampAtPosition(int p_x_pos, int p_y_pos, bool &p_one_found, Pstamp &p_stamp_found, string &p_col_name = string());
    int GetXStartPosition();
    int GetXEndPosition();
    int GetYStartPosition();
    int GetYEndPosition();
    void SetShowClockIcons(bool p_show_clock_icons);
    bool GetShowClockIcons();
};

bool DownloadAppointments(TIBDatabase *dbApptDatabase, Pdate begin_download_date, Pdate end_download_date, string user_id, vector<string> user_ids,
                          vector<string> group_ids, string tax_entity_id, PApptOccurrenceMap &all_appointments,
                          PApptIDMapVec &ids_to_match, bool p_view_all, bool show_chart_num, string chart_location);

bool DownloadAppointments(TIBDatabase *dbApptDatabase, Pdate begin_download_date, Pdate end_download_date, string user_id, vector<string> user_ids,
                          string tax_entity_id, PApptOccurrenceMap &all_appointments,
                          PApptIDMapVec &ids_to_match, bool p_view_all, bool show_chart_num, string chart_location);

#endif
