#include "tinyrecord_concentrator.h"
#include "codeFrameTinyRecordMagic.h"

tr_magic_concentrator::tr_magic_concentrator ()
{
 // nada?
}
tr_magic_concentrator::~tr_magic_concentrator ()
{
 // avoid issues with iterators being invalided under our feet!
 set<TframeTinyRecordMagic*>::iterator c, n;
 for (c = FMagics.begin(); c != FMagics.end(); c = n)
 {
  n = c;
  n++;
  (*c)->Concentrator = NULL;
 }
}
void tr_magic_concentrator::Add (TframeTinyRecordMagic * to_add)
{
 FMagics.insert(to_add);
 Refresh();
}
void tr_magic_concentrator::Remove (TframeTinyRecordMagic * to_remove)
{
 FMagics.erase(to_remove);
 Refresh();
}
bool tr_magic_concentrator::Exists (TframeTinyRecordMagic * to_test)
{
 return (FMagics.find(to_test) != FMagics.end());
}
void tr_magic_concentrator::List (TframeTinyRecordMagic * list_into)
{
 if (!list_into)
  return;
 if (list_into->Magic != trm_same)
  return;

 TframeTinyRecordMagic * old_item = list_into->SameAs;
 list_into->comboSameAs->ItemIndex = -1;
 list_into->comboSameAs->Items->Clear();
 list_into->vectorSameAs.clear();
 int i;
 bool found = false;
 for(i = 0; i < FCachedList.size(); i++)
 {
  if (FCachedList[i].stn == list_into->SourceTableName)
  {
   list_into->comboSameAs->Items->Add(FCachedList[i].desc.c_str());
   list_into->vectorSameAs.push_back(FCachedList[i].item);
   if (FCachedList[i].item == old_item)
   {
    list_into->comboSameAs->ItemIndex = i;
    list_into->comboSameAs->Text = FCachedList[i].desc.c_str();
    found = true;
   }
  }
 }
 if (!found) // list_into->comboSameAs->ItemIndex == -1)
 {
  list_into->SameAs = NULL; // sorry, it's not there anymore?
  list_into->comboSameAs->Text = "";
 }
}
void tr_magic_concentrator::Refresh ()
{
 // cache the results partially so List() won't take as long on each call
 FCachedList.clear();
 for (set<TframeTinyRecordMagic*>::iterator i = FMagics.begin(); i != FMagics.end(); i++)
  if ((*i)->Magic == trm_new || ((*i)->Magic == trm_existing && (*i)->HasId))
   FCachedList.push_back(tr_cached_item((*i)->SourceTableName, (*i)->Desc, (*i)));
 sort(FCachedList.begin(), FCachedList.end());

 // now make everyone ask to be updated cleanly.
 for (set<TframeTinyRecordMagic*>::iterator i = FMagics.begin(); i != FMagics.end(); i++)
  (*i)->RefreshComboSameAs();
}

TframeTinyRecordMagic * tr_magic_concentrator::Find (string _stn, string _id, TframeTinyRecordMagic * not_this_one)
{
 for(int i = 0; i < FCachedList.size(); i++)
  if (FCachedList[i].stn == _stn && FCachedList[i].item->Magic == trm_existing && FCachedList[i].item->Id == _id && FCachedList[i].item != not_this_one)
   return FCachedList[i].item;
 return NULL;
}

// when a magic changes its description in some way ...
// - inform the concentrator
// - the concentrator finds all other magics of the same stn that are not same-as, etc.
// - the concentrator informs them that they need to refresh their list of magics
// - the magics in turn each get the list of available other magics (apart from themselves)
// - the magics try to maintain the currently-selected one
// -> magics need to know the TframeTinyRecordMagic * they point to
//    - changing it should auto-register/auto-unregister in the concentrator
//    - refreshing the list should get descriptions -and- pointers so it can match them up even if the description changes
// when a magic changes what it points to ...
// - unpoint from the old one
// - point to the new one
// - make sure we're not expanded ...
// when a magic is asked to save, and it points to another ...
// - it doesn't need to do anything.
// when a magic is asked for its id, because a parent record needs it (relationships.person_b_id, for example, or insurance)
// - we need the id of the thing we point to,
// - but it may not have saved yet either,
// - and it may not be safe for it to save:
// - new person
// - has new coverage
// - which points to new policy
// - which points to the person
// -> saving the person
//    - save person
//    - set person_id on names, affiliations, relationships, etc.
//    - save names, affiliations, ... relationships
//    - to save the relationship, we need the link to the other person
//    - save the person we contain, if it's 'same', it does nothing
//    - get the id from the person (if 'same', get it from the main person)
//    - save the relationship
//    - ... okay. that's not a problem.
// okay. so what if ...
// - they skip the relationship page
// - they enter insurance, including a new employer person
// - they then go back to relationships, and add a relationship for the employer
// -> wind up trying to save the relationships first,
//    - it needs to save the person so it can get the id, but the person isn't saved yet
//    - can it safely ask the person elsewhere to go ahead and save?
//    - ... yes. i think so. then get the id.
// - so: if a magic is asked for its id, and it can't get it because it points to an unsaved ... ask to save?
// - what about tax entities? no problem.
// - what about insurance policy? it points to people and tax entities, but neither can have insurance themselves ...
// so long as we avoid circular links? that's pretty much the issue.



// need to set transaction with which to save (in -all- components) before trying to save any (as it may ask others to save)
