/*

  Prometheus Stub Launcher

  Version Token File Processing

  A version token file resides on the file server, as well as on the local 
  machine. If the two don't match, the stub will download all the files from
  the file server and then update the local version token file provided it
  was successful. The local version token file resides inside the Prometheus
  file cache directory, which is always writable by the currently logged-in
  user.

*/

#include <cstdio>
#include <string>
#include "versiontoken.h"
#include "utils.h"

using namespace std;

string Version::localVersion;
string Version::remoteVersion;

//
// Version::ReadVersion
//
// Read a version token file. Depending on the value passed in the second 
// argument, it will be stored either in localVersion or remoteVersion.
//
void Version::ReadVersion(const char *filename, int versionType)
{
   char *text = Utils::LoadTextFile(filename);

   if(text)
   {
      switch(versionType)
      {
      case LOCAL:
         localVersion = text;
         break;
      case REMOTE:
         remoteVersion = text;
         break;
      }

      delete [] text;
   }
}

//
// Version::WriteVersion
//
// Update the local version file with the remote version number.
//
void Version::WriteVersion(const char *filename)
{
   if(remoteVersion != "")
   {
      FILE *f = NULL;

      if((f = fopen(filename, "w")))
      {
         fprintf(f, "%s", remoteVersion.c_str());
         fclose(f);
      }
   }
}

// EOF

