/*

   Prometheus Stub Launcher

   Utilties

*/

#include <windows.h>
#include <cstdio> 
#include <cstring>
#include <string>

#include "utils.h"
#include "cached_files.h"


using namespace std;

//
// Utils::LoadTextFile
//
// Load a text file from disk. Returns null if fails.
//
char *Utils::LoadTextFile(const char *filename)
{
   FILE *f;
   char *buffer;
   long size;

   if(!(f = fopen(filename, "rb")))
      return NULL; // can't open file

   fseek(f, 0, SEEK_END);
   size = ftell(f);
   fseek(f, 0, SEEK_SET);

   if(size <= 0)
   {
      fclose(f);
      return NULL; // nothing in the file
   }

   buffer = new char [size + 1]; // size+1 for null termination
   memset(buffer, 0, size + 1);

   if(fread(buffer, 1, size, f) != size)
   {
      delete [] buffer;
      fclose(f);
      return NULL; // can't read the file
   }

   fclose(f);

   return buffer; // success
}

//
// Utils::Launch
//
// Convenience wrapper around CreateProcess
//
bool Utils::Launch(const char *process, char *cmdline)
{
   PROCESS_INFORMATION pi;
   STARTUPINFO si;
   BOOL result;
   pair<string, string> filespec;

   memset(&si, 0, sizeof(si));

   si.cb = sizeof(si);

   filespec = FileCache::GetFileSpec(process);

   // NB: must set working directory or Prometheus itself will still copy itself and
   // launch from the Program Files directory!
   result = CreateProcess((LPCSTR)process, (LPSTR)cmdline, NULL, NULL, FALSE, 
                          0, NULL, (LPCSTR)filespec.first.c_str(), &si, &pi);

   CloseHandle(pi.hProcess);
   CloseHandle(pi.hThread);

   return (result != 0);
}

//
// Utils::FatalError
//
// Opens a MessageBox and then quits the program.
//
void Utils::FatalError(const char *text)
{
   MessageBox(NULL, (LPCTSTR)text, (LPCTSTR)"Error", MB_OK|MB_ICONERROR);
   ExitProcess(0);
}

// EOF

