{$ifdef ver100}
unit QRNew;

interface

procedure Register;

implementation

uses Windows, SysUtils, Classes, Controls, Forms, ExptIntf, ToolIntf, VirtIntf,
  IStreams, DsgnIntf, QuickRpt, QReport, QR3Const;

type
  TNewQuickReport = class(TIExpert)
    function GetName: string; override;
    function GetComment: string; override;
    function GetGlyph: HICON; override;
    function GetStyle: TExpertStyle; override;
    function GetState: TExpertState; override;
    function GetIDString: string; override;
    function GetAuthor: string; override;
    function GetPage: string; override;
    function GetMenuText: string; override;
    procedure Execute; override;
  end;

{ TNewQuickReport }

function TNewQuickReport.GetName: string;
begin
  Result := 'Report'; { <-- do not resource }
end;

function TNewQuickReport.GetComment: string;
begin
  Result := SqrDesignAReport;
end;

function TNewQuickReport.GetGlyph: HICON;
begin
  Result := LoadIcon(HInstance, 'QRNEW');
end;

function TNewQuickReport.GetStyle: TExpertStyle;
begin
  Result := esForm;
end;

function TNewQuickReport.GetState: TExpertState;
begin
  Result := [esEnabled];
end;

function TNewQuickReport.GetIDString: string;
begin
  Result := 'QuSoft.NewQuickReport'; { <-- do not resource }
end;

function TNewQuickReport.GetAuthor: string;
begin
  Result := 'QuSoft AS'; { <-- do not resource }
end;

function TNewQuickReport.GetPage: string;
begin
  Result := 'New';
end;

function TNewQuickReport.GetMenuText: string;
begin
  Result := '';
end;

{$ifdef BCB}
const
  CRLF = #13#10;
  DashLine =
  '//----------------------------------------------------------------------------';

  QuickReportCppSource =
    DashLine + CRLF +
    '#include <vcl\vcl.h>' + CRLF +
    '#pragma hdrstop' + CRLF +
    CRLF +
    '#include "%0:s.h"' + CRLF +
    DashLine + CRLF +
    '#pragma resource "*.dfm"' + CRLF +
    'T%1:s *%1:s;' + CRLF +
    DashLine + CRLF +
    '__fastcall T%1:s::T%1:s(TComponent* Owner)' + CRLF +
    '    : TQuickRep(Owner)' + CRLF +
    '{' + CRLF +
    '}' + CRLF +
    DashLine;

  QuickReportCppHeader =
    DashLine + CRLF +
    '#ifndef %0:sH' + CRLF +
    '#define %0:sH' + CRLF +
    DashLine + CRLF +
    '#include <vcl\Classes.hpp>' + CRLF +
    '#include <vcl\Controls.hpp>' + CRLF +
    '#include <vcl\StdCtrls.hpp>' + CRLF +
    '#include <vcl\Forms.hpp>' + CRLF +
    '#include <vcl\QuickRpt.hpp>' + CRLF +
    '#include <vcl\QRCtrls.hpp>' + CRLF +
    DashLine + CRLF +
    'class T%1:s : public TQuickRep' + CRLF +
    '{' + CRLF +
    '__published:' + CRLF +
    'private:' + CRLF +
    'public:' + CRLF +
    '   __fastcall T%1:s::T%1:s(TComponent* Owner);' + CRLF +
    '};' + CRLF +
    DashLine + CRLF +
    'extern T%1:s *%1:s;' + CRLF +
    DashLine + CRLF +
    '#endif';

  QuickReportUnitSource =
    'unit %0:s;'#13#10 +
    #13#10 +
    'interface'#13#10 +
    #13#10 +
    'uses Windows, SysUtils, Messages, Classes, Graphics, Controls,'#13#10 +
    '  StdCtrls, ExtCtrls, Forms, Quickrpt, QRCtrls;'#13#10 +
    #13#10 +
    'type'#13#10 +
    '  T%1:s = class(TQuickRep)'#13#10 +
    '  private'#13#10 +
    #13#10 +
    '  public'#13#10 +
    #13#10 +
    '  end;'#13#10 +
    #13#10 +
    'var'#13#10 +
    '  %1:s: T%1:s;'#13#10 +
    #13#10 +
    'implementation'#13#10 +
    #13#10 +
    '{$R *.DFM}'#13#10 +
    #13#10 +
    'end.'#13#10;

  QuickReportDfmSource = 'object %s: T%0:s end';

procedure TNewQuickReport.Execute;
var
  UnitIdent, Filename: string;
  ReportName: string;
  CodeStream, HdrStream: TIStream;
  DFMStream: TIStream;
  DFMString, DFMVCLStream: TStream;
  InCppBuilder: Boolean;
begin
  if not ToolServices.GetNewModuleName(UnitIdent, FileName) then Exit;
  ReportName := 'QuickReport' + Copy(UnitIdent, 5, 255);
  InCppBuilder := CompareText(HexDisplayPrefix, '0x') = 0;
  if InCppBuilder then
  begin
    HdrStream := TIStreamAdapter.Create(TStringStream.Create(Format(QuickReportCppHeader,
      [UnitIdent, ReportName])), True);
    CodeStream := TIStreamAdapter.Create(TStringStream.Create(Format(QuickReportCppSource,
      [UnitIdent, ReportName])), True);
  end else
  begin
    HdrStream := nil;
    CodeStream := TIStreamAdapter.Create(TStringStream.Create(Format(QuickReportUnitSource,
      [UnitIdent, ReportName])), True);
  end;
  try
    CodeStream.AddRef;
    if HdrStream <> nil then HdrStream.AddRef;
    DFMString := TStringStream.Create(Format(QuickReportDfmSource, [ReportName]));
    try
      DFMVCLStream := TMemoryStream.Create;
      try
        ObjectTextToResource(DFMString, DFMVCLStream);
        DFMVCLStream.Position := 0;
      except
        DFMVCLStream.Free;
      end;
      DFMStream := TIStreamAdapter.Create(DFMVCLStream, True);
      try
        DFMStream.AddRef;
        ToolServices.CreateCppModule(FileName, ReportName, 'TQuickRep', '',
          HdrStream, CodeStream, DFMStream, [cmAddToProject, cmShowSource, cmShowForm,
            cmUnNamed, cmMarkModified]);
      finally
        DFMStream.Free;
      end;
    finally
      DFMString.Free;
    end;
  finally
    CodeStream.Free;
    if HdrStream <> nil then HdrStream.Free
  end;
end;

{$ELSE}
const
  QuickReportUnitSource =
    'unit %0:s;'#13#10 +
    #13#10 +
    'interface'#13#10 +
    #13#10 +
    'uses Windows, SysUtils, Messages, Classes, Graphics, Controls,'#13#10 +
    '  StdCtrls, ExtCtrls, Forms, Quickrpt, QRCtrls;'#13#10 +
    #13#10 +
    'type'#13#10 +
    '  T%1:s = class(TQuickRep)'#13#10 +
    '  private'#13#10 +
    #13#10 +
    '  public'#13#10 +
    #13#10 +
    '  end;'#13#10 +
    #13#10 +
    'var'#13#10 +
    '  %1:s: T%1:s;'#13#10 +
    #13#10 +
    'implementation'#13#10 +
    #13#10 +
    '{$R *.DFM}'#13#10 +
    #13#10 +
    'end.'#13#10;
  QuickReportDfmSource = 'object %s: T%0:s end';

procedure TNewQuickReport.Execute;
var
  UnitIdent, Filename: string;
  ReportName: string;
  CodeStream: TIStream;
  DFMStream: TIStream;
  DFMString, DFMVCLStream: TStream;
begin
  if not ToolServices.GetNewModuleName(UnitIdent, FileName) then Exit;
  ReportName := 'QuickReport' + Copy(UnitIdent, 5, 255);
  CodeStream := TIStreamAdapter.Create(TStringStream.Create(Format(QuickReportUnitSource,
    [UnitIdent, ReportName])), True);
  try
    CodeStream.AddRef;
    DFMString := TStringStream.Create(Format(QuickReportDfmSource, [ReportName]));
    try
      DFMVCLStream := TMemoryStream.Create;
      try
        ObjectTextToResource(DFMString, DFMVCLStream);
        DFMVCLStream.Position := 0;
      except
        DFMVCLStream.Free;
      end;
      DFMStream := TIStreamAdapter.Create(DFMVCLStream, True);
      try
        DFMStream.AddRef;
        ToolServices.CreateModuleEx(FileName, ReportName, 'TQuickRep', '',
          CodeStream, DFMStream, [cmAddToProject, cmShowSource, cmShowForm,
            cmUnNamed, cmMarkModified]);
      finally
        DFMStream.Free;
      end;
    finally
      DFMString.Free;
    end;
  finally
    CodeStream.Free;
  end;
end;

{$endif}

type
  TQuickReportCustomModule = class(TCustomModule)
    function GetAttributes: TCustomModuleAttributes; override;
    procedure ValidateComponent(Component: TComponent); override;
    procedure ExecuteVerb(Index: Integer); override;
    function GetVerb(Index: Integer): string; override;
    function GetVerbCount: Integer; override;
  end;

function TQuickReportCustomModule.GetAttributes: TCustomModuleAttributes;
begin
  Result := [cmaVirtualSize];
end;

procedure TQuickReportCustomModule.ValidateComponent(Component: TComponent);
begin
  if (Component is TControl) and not (Component is TQRBasePanel)
    and not (Component is TQRPrintable) then
    raise Exception.CreateFmt(SqrCannotAdd, [Component.ClassName]);
end;

procedure TQuickReportCustomModule.ExecuteVerb(Index: Integer);
begin
  ExecuteDesignVerb(Index, Root as TQuickRep);
end;

function TQuickReportCustomModule.GetVerb(Index: Integer): string;
begin
  Result := GetDesignVerb(Index);
end;

function TQuickReportCustomModule.GetVerbCount: Integer;
begin
  Result := GetDesignVerbCount;
end;

procedure Register;
begin
  RegisterCustomModule(TQuickRep, TQuickReportCustomModule);
  RegisterLibraryExpert(TNewQuickReport.Create);
end;

end.
{$else}

{$I QRDEFS.INC}

unit QRNew;

interface

procedure Register;

implementation

uses Windows, ActiveX, SysUtils, Classes, Controls, Forms, ExptIntf, ToolIntf,
  VirtIntf, IStreams, DsgnIntf, QuickRpt, QReport, QR3Const;

type
  TNewQuickReport = class(TIExpert)
    function GetName: string; override;
    function GetComment: string; override;
    function GetGlyph: HICON; override;
    function GetStyle: TExpertStyle; override;
    function GetState: TExpertState; override;
    function GetIDString: string; override;
    function GetAuthor: string; override;
    function GetPage: string; override;
    function GetMenuText: string; override;
    procedure Execute; override;
  end;

{ TNewQuickReport }

function TNewQuickReport.GetName: string;
begin
  Result := SqrReport;
end;

function TNewQuickReport.GetComment: string;
begin
  Result := SqrDesignReport;
end;

function TNewQuickReport.GetGlyph: HICON;
begin
  Result := LoadIcon(HInstance, 'QRNEW');
end;

function TNewQuickReport.GetStyle: TExpertStyle;
begin
  Result := esForm;
end;

function TNewQuickReport.GetState: TExpertState;
begin
  Result := [esEnabled];
end;

function TNewQuickReport.GetIDString: string;
begin
  Result := 'QuSoft.NewQuickReport'; { <-- do not resource }
end;

function TNewQuickReport.GetAuthor: string;
begin
  Result := SqrQuSoft;
end;

function TNewQuickReport.GetPage: string;
begin
  Result := SqrNew;
end;

function TNewQuickReport.GetMenuText: string;
begin
  Result := '';
end;

const
  CRLF = #13#10;
  DashLine =
  '//----------------------------------------------------------------------------';

  QuickReportCppSource =
    DashLine + CRLF +
    '#include <vcl\vcl.h>' + CRLF +
    '#pragma hdrstop' + CRLF +
    CRLF +
    '#include "%0:s.h"' + CRLF +
    DashLine + CRLF +
    '#pragma resource "*.dfm"' + CRLF +
    'T%1:s *%1:s;' + CRLF +
    DashLine + CRLF +
    '__fastcall T%1:s::T%1:s(TComponent* Owner)' + CRLF +
    '    : TQuickRep(Owner)' + CRLF +
    '{' + CRLF +
    '}' + CRLF +
    DashLine;

  QuickReportCppHeader =
    DashLine + CRLF +
    '#ifndef %0:sH' + CRLF +
    '#define %0:sH' + CRLF +
    DashLine + CRLF +
    '#include <vcl\Classes.hpp>' + CRLF +
    '#include <vcl\Controls.hpp>' + CRLF +
    '#include <vcl\StdCtrls.hpp>' + CRLF +
    '#include <vcl\Forms.hpp>' + CRLF +
    '#include <vcl\QuickRpt.hpp>' + CRLF +
    '#include <vcl\QRCtrls.hpp>' + CRLF +
    DashLine + CRLF +
    'class T%1:s : public TQuickRep' + CRLF +
    '{' + CRLF +
    '__published:' + CRLF +
    'private:' + CRLF +
    'public:' + CRLF +
    '   __fastcall T%1:s::T%1:s(TComponent* Owner);' + CRLF +
    '};' + CRLF +
    DashLine + CRLF +
    'extern T%1:s *%1:s;' + CRLF +
    DashLine + CRLF +
    '#endif';

  QuickReportUnitSource =
    'unit %0:s;'#13#10 +
    #13#10 +
    'interface'#13#10 +
    #13#10 +
    'uses Windows, SysUtils, Messages, Classes, Graphics, Controls,'#13#10 +
    '  StdCtrls, ExtCtrls, Forms, Quickrpt, QRCtrls;'#13#10 +
    #13#10 +
    'type'#13#10 +
    '  T%1:s = class(TQuickRep)'#13#10 +
    '  private'#13#10 +
    #13#10 +
    '  public'#13#10 +
    #13#10 +
    '  end;'#13#10 +
    #13#10 +
    'var'#13#10 +
    '  %1:s: T%1:s;'#13#10 +
    #13#10 +
    'implementation'#13#10 +
    #13#10 +
    '{$R *.DFM}'#13#10 +
    #13#10 +
    'end.'#13#10;

  QuickReportDfmSource = 'object %s: T%0:s end';

procedure TNewQuickReport.Execute;
var
  UnitIdent, Filename: string;
  ReportName: string;
  CodeStream, HdrStream, DFMStream: IStream;
  DFMString, DFMVCLStream: TStream;
  InCppBuilder: Boolean;
begin
{$ifdef ver110}
  if not ToolServices.GetNewModuleName(UnitIdent, FileName) then Exit;
  ReportName := 'QuickReport' + Copy(UnitIdent, 5, 255);
  InCppBuilder := HexDisplayPrefix <> '$';
  if InCppBuilder then
  begin
    HdrStream := TIStreamAdapter.Create(TStringStream.Create(Format(QuickReportCppHeader,
      [UnitIdent, ReportName])), soOwned);
    CodeStream := TIStreamAdapter.Create(TStringStream.Create(Format(QuickReportCppSource,
      [UnitIdent, ReportName])), soOwned);
  end else
  begin
    HdrStream := nil;
    CodeStream := TIStreamAdapter.Create(TStringStream.Create(Format(QuickReportUnitSource,
      [UnitIdent, ReportName])), soOwned);
  end;
  DFMString := TStringStream.Create(Format(QuickReportDfmSource, [ReportName]));
  try
    DFMVCLStream := TMemoryStream.Create;
    DFMStream := TIStreamAdapter.Create(DFMVCLStream, soOwned);
    ObjectTextToResource(DFMString, DFMVCLStream);
    DFMVCLStream.Position := 0;
    ToolServices.CreateCppModule(FileName, ReportName, 'TQuickRep', '',
      HdrStream, CodeStream, DFMStream, [cmAddToProject, cmShowSource, cmShowForm,
      cmUnNamed, cmMarkModified]);
  finally
    DFMString.Free;
  end;
{$endif}
end;

type
  TQuickReportCustomModule = class(TCustomModule)
    function GetAttributes: TCustomModuleAttributes; override;
    procedure ValidateComponent(Component: IComponent); override;
    procedure ExecuteVerb(Index: Integer); override;
    function GetVerb(Index: Integer): string; override;
    function GetVerbCount: Integer; override;
  end;

function TQuickReportCustomModule.GetAttributes: TCustomModuleAttributes;
begin
  Result := [cmaVirtualSize];
end;

procedure TQuickReportCustomModule.ValidateComponent(Component: IComponent);
var
  Temp: TComponent;
begin
  Temp := TryExtractComponent(Component);
  if (Temp = nil) or ((Temp is TControl) and not (Temp is TQRBasePanel)
    and not (Temp is TQRPrintable)) then
    raise Exception.CreateFmt(SqrCannotAdd, [Temp.ClassName]);
end;

procedure TQuickReportCustomModule.ExecuteVerb(Index: Integer);
begin
  ExecuteDesignVerb(Index, ExtractComponent(Root) as TQuickRep);
end;

function TQuickReportCustomModule.GetVerb(Index: Integer): string;
begin
  Result := GetDesignVerb(Index);
end;

function TQuickReportCustomModule.GetVerbCount: Integer;
begin
  Result := GetDesignVerbCount;
end;

procedure Register;
begin
  RegisterCustomModule(TQuickRep, TQuickReportCustomModule);
  RegisterLibraryExpert(TNewQuickReport.Create);
end;
{$endif}
end.

