unit QRPDFFilt;
////////////////////////////////////////////////////////////////////////////
//  Unit : QRPDFFilt.pas
//
//  TQRGAbstractExportFilter -> TQRPDFDocumentFilter
//
//  The new PDF Export document filter class -
//
//  Must use the new QRHtmlImage, QRDBHtmlImage controls to export graphics
//
//  (c) 2001 QBS Software
//      created 28th September 2001
//   17/11 fix stretch ug with TQRGRImage - now stretchraw in control rect
//   25/11 Added proprtional control to PDF
//   11/01/02 added Inside proportional mode
//   14/02/02 add top margin adjust and left m adjust
//
//   REVISION to V2.0 - 21-06-02
//   QRDBImage, QRImage and QRShape now supported.
//   29-06-02 Add StartConcat to fix property bug in C++ 
////////////////////////////////////////////////////////////////////////////
{$define VER20}
{$define notUSE_DLL}    
interface
{$ifdef USE_DLL}
uses windows, classes, controls, stdctrls,sysutils, graphics, buttons,
     forms, extctrls, dialogs, printers, db, DBtables, ComCtrls, qrexport,
     QRPrntr, Quickrpt, QR3Const, qrctrls, grimgctrl;
{$else}
uses windows, classes, controls, stdctrls,sysutils, graphics, buttons,
     forms, extctrls, dialogs, printers, db, DBtables, ComCtrls, qrexport,
     QRPrntr, Quickrpt, QR3Const, qrctrls, grimgctrl, pdfobjs;
{$endif}
 
const
    PointsPerMM = 72.0 / 25.4;
  IT_TEXT = 1;
  IT_NEWPAGE = 2;
  IT_GRAPHIC = 3;
  IT_IMAGE = 4;
  // shapes
  S_BOX = 0;
  S_CIRCLE = 1;
  S_HLINE = 2;
  S_VLINE = 3;
  S_OBLIQUE = 4;
  S_TOPBOTTOM = 5;
  S_LEFTRIGHT = 6;
  S_ROUNDRECT = 7;
  // ascii
  ORD0 = ord('0');
  ORDA = ord('A');

type
{$ifdef USE_DLL}
  TRGBColor = record
      red, green, blue : byte;
  end;

    TPDFItemRec = record
      ItemType : byte;
      Xpos, Ypos : extended;
      Fontname, FText : pchar;
      fontsize : integer;
      fcolor : pchar;
      fbold, fitalic : boolean;
      fAlignment : byte;
      filled, staticimage : boolean;
      width, height, thickness, xscale, yscale : extended;
      imagesrc : pchar;
      shape : byte;
      pixelwidth, pixelheight : integer;
      rgbstrokecolor : TRGBColor;
      rgbfcolor : TRGBColor;
  end;
{$endif}

  TQRPAbstractExportFilter = class(TQRExportFilter)
  private
    FStream : TStream;
    FCharWidth,
    FCharHeight,
    FPaperWidth,
    FPaperHeight : extended;
    FLineCount,
    FColCount : integer;
    FPageProcessed : boolean;
  protected
    function GetText(X, Y : extended; var Font : TFont) : string;
    function GetFilterName : string; override;
    function GetDescription : string; override;
    function GetExtension : string; override;
    procedure WriteToStream(const AText : string);
    procedure WriteLnToStream(const AText : string);
    procedure CreateStream(Filename : string); virtual;
    procedure CloseStream; virtual;
    procedure ProcessPage; virtual;
    procedure StorePage; virtual;
    property Stream : TStream read FStream write FStream;
    property PageProcessed : boolean read FPageProcessed write FPageProcessed;
    property CharWidth : extended read FCharWidth write FCharWidth;
    property CharHeight : extended read FCharHeight write FCharHeight;
    property PaperWidth : extended read FPaperWidth write FPaperWidth;
    property PaperHeight : extended read FPaperHeight write FPaperHeight;
    property LineCount : integer read FLineCount write FLineCount;
    property ColCount : integer read FColCount write FColCount;
  public
    procedure Start(PaperWidth, PaperHeight : integer; Font : TFont); override;
    procedure Finish; override;
    procedure EndPage; override;
    procedure NewPage; override;
{$ifdef VER20}
    procedure AcceptGraphic( Xoff, Yoff : extended; GControl : TControl); override;
{$else}
    procedure AcceptGraphic( Xoff, Yoff : extended; GControl : TControl); virtual;
{$endif}
    procedure TextOut(X, Y : extended; Font : TFont; BGColor : TColor;
               Alignment : TAlignment; Text : string); override;
  end;

  TQRPDFDocumentFilter = class(TQRPAbstractExportFilter)
  private
    FFreeStream : boolean;
    FBodyColor : TColor;
    FExtraBody : String;
    FFirstPage : boolean;
    FPageLength : integer;
    FPageNumber : longint;
    FNextPicnum : longint;
    FPDFItem : TPDFItemRec;
    FTextOnTop : boolean;
    FConcatenating : boolean;
    FReportnum : integer;
    FLeftMargin : integer;
    FTopMargin : integer;
    procedure SetTextOnTop( bval : boolean );
  protected
    function GetFilterName : string; override;
    function GetDescription : string; override;
    function GetExtension : string; override;
    function GetStreaming : boolean; override;
    procedure CreateStream(Filename : string); override;
    procedure CloseStream; override;
  public
    constructor Create( filename : string );
    destructor Destroy; override;
    procedure StorePage; override;
    procedure TextOut(X, Y : extended; Font : TFont; BGColor : TColor;
               Alignment : TAlignment; Text : string); override;
    procedure ProcessPage; override;
    procedure Start(PaperWidth, PaperHeight : integer; Font : TFont); override;
    procedure Finish; override;
    procedure SetMargins( TopMarg, Leftmarg : integer );
    procedure NewPage; override;
    procedure EndPage; override;
    procedure EndConcat;
    procedure StartConcat;
    procedure AcceptGraphic( Xoff, Yoff : extended; GControl : TControl); override;
    procedure AddFontMap( ssmap : string );
    property PageLength : integer read FPageLength write FPageLength;
    property Stream;
    property FreeStream : boolean read FFreeStream write FFreeStream;
    property TextOnTop : boolean read FTextOnTop write SetTextOnTop;
    property Concatenating : boolean read FConcatenating write FConcatenating;
    property Reportnum : integer read FReportnum write FReportnum;
    property TopMargin : integer read FTopMargin write FTopMargin;
    property LeftMargin : integer read FLeftMargin write FLeftMargin;
  end;


implementation
var
    MMperpixelX, MMperpixelY, PointsPerPixelX, PointsPerPixelY : extended;
{$ifdef USE_DLL}
   procedure CloseDownLib; external 'QRPdfLib.dll';
   procedure InitLib( Mother : pointer ); external 'QRPdfLib.dll';
   procedure AddPDFItem( ItemRec : TPDFItemRec ); external 'QRPdfLib.dll';
   procedure AddImageItem( ItemRec : TPDFItemRec; imgdata : pointer ); external 'QRPdfLib.dll';
   Procedure SetPageParams( w, h, tm, tma, lma : extended ); external 'QRPdfLib.dll';
   Procedure AddFontSub( ssmap : string ); external 'QRPdfLib.dll';
   procedure FinishDoc( FOutFile : string ); external 'QRPdfLib.dll';
   procedure SetTextFirst( bval : boolean ); external 'QRPdfLib.dll';
{$endif}

{TQRPAbstractExportFilter}
procedure TQRPAbstractExportFilter.AcceptGraphic( Xoff, Yoff : extended; GControl : TControl);
begin
end;

function TQRPAbstractExportFilter.GetFilterName : string;
begin
  result := 'QRAbstract'; // Do not translate
end;

function TQRPAbstractExportFilter.GetDescription : string;
begin
  Result := '';
end;

function TQRPAbstractExportFilter.GetExtension : string;
begin
  Result := '';
end;

procedure TQRPAbstractExportFilter.Start(PaperWidth, PaperHeight : integer; Font : TFont);
begin
   // see override
   inherited Start( paperwidth, paperheight, font );
end;


procedure TQRPAbstractExportFilter.CreateStream(Filename : string);
begin
  FStream := TFileStream.Create(Filename, fmCreate);
end;

procedure TQRPAbstractExportFilter.CloseStream;
begin
  FStream.Free;
end;

procedure TQRPAbstractExportFilter.WriteToStream(const AText : string);
begin
  Stream.Write(AText[1], length(AText));
end;

procedure TQRPAbstractExportFilter.WriteLnToStream(const AText : string);
begin
  WriteToStream(AText + #13 + #10);
end;


procedure TQRPAbstractExportFilter.Finish;
begin
    // all in override
    inherited;
end;

procedure TQRPAbstractExportFilter.NewPage;
begin
   // all in override
end;

procedure TQRPAbstractExportFilter.EndPage;
begin
  // all in override
end;

procedure TQRPAbstractExportFilter.ProcessPage;
begin
  FPageProcessed := True;
end;

procedure TQRPAbstractExportFilter.StorePage;
begin
end;

procedure TQRPAbstractExportFilter.TextOut(X, Y : extended; Font : TFont;
                      BGColor : TColor; Alignment : TAlignment; Text : string);
begin
end;

function TQRPAbstractExportFilter.GetText(X, Y : extended; var Font : TFont) : string;
begin
end;


{TQRPDFDocumentFilter}
procedure TQRPDFDocumentFilter.AddFontMap( ssmap : string );
begin
     AddFontSub( ssmap );
end;

function TQRPDFDocumentFilter.GetFilterName : string;
begin
  Result := 'PDF document';
end;

function TQRPDFDocumentFilter.GetDescription : string;
begin
  Result := SqrHTMLDocumentForWeb;
end;

function TQRPDFDocumentFilter.GetExtension : string;
begin
  Result := 'PDF'; // Do not translate
end;

function TQRPDFDocumentFilter.GetStreaming : boolean;
begin
  Result := false;
end;

procedure TQRPDFDocumentFilter.CreateStream(Filename : string);
begin
  if Filename = '' then
  begin
    FStream := TMemoryStream.Create;
    FreeStream := false;
  end else
  begin
    FreeStream := true;
    inherited CreateStream(Filename);
  end;
end;

procedure TQRPDFDocumentFilter.CloseStream;
begin
  if FreeStream then
    inherited CloseStream;
end;

destructor TQRPDFDocumentFilter.destroy;
begin
    Closedownlib;// move to 'finish'
    inherited;
end;

constructor TQRPDFDocumentFilter.Create( filename : string );
begin
   // made this a full call
   inherited Create( filename );
   InitLib( self.Owner );
   FBodyColor := clWhite;
   FExtraBody := '';
   FFirstPage := true;
   FPageNumber := 1;
   FPageLength := 2500;
   FNextPicnum := 0;
   FConcatenating := false;
   FReportNum := 0;
   FLeftMargin := 0;
   FTopMargin := 0;
end;

procedure TQRPDFDocumentFilter.SetTextOnTop( bval : boolean );
begin
    settextfirst( not bval );
end;

procedure TQRPDFDocumentFilter.SetMargins( TopMarg, Leftmarg : integer );
begin
    FTopMargin := Topmarg;
    FLeftmargin := Leftmarg;
end;

// QBSS : override method
procedure TQRPDFDocumentFilter.ProcessPage;
begin
  FPageProcessed := True;
  StorePage;
end;

// Overridden Start
procedure TQRPDFDocumentFilter.Start(PaperWidth, PaperHeight : integer; Font : TFont);
var
    tm : extended;
begin
    if fconcatenating and (FReportnum > 0) then exit;
    inherited start( paperwidth, paperheight, Font );  // sets active true
    MMperpixelX := 1.0 / (10*TQuickrep(owner).QRPrinter.XFactor);
    MMperpixelY := 1.0 / (10*TQuickrep(owner).QRPrinter.YFactor);
    PointsPerPixelX := PointsPerMM  * MMperPixelX;
    PointsPerPixelY := PointsPerMM  * MMperPixelY;
    tm := TQuickrep(self.owner).page.TopMargin * 0.1 * PointsPerMM;
    SetPageParams( 0.1*PaperWidth*PointsPerMM, 0.1*paperheight*PointsPerMM, tm, FTopMargin, FLeftmargin );
    freportnum := 1;
end;

// overridden Finish
procedure TQRPDFDocumentFilter.Finish;
begin
    if fconcatenating then exit;
    // process the list of textitems and graphic items
    FinishDoc( filename );
    inherited;// sets active false
end;

procedure TQRPDFDocumentFilter.EndConcat;
begin
     fconcatenating := false;
     Finish;
end;

procedure TQRPDFDocumentFilter.StartConcat;
begin
     fconcatenating := true;
end;

procedure TQRPDFDocumentFilter.StorePage;
begin
     // do nothing.
end;

// add a text titem to the list
procedure TQRPDFDocumentFilter.TextOut(X, Y : extended; Font : TFont; BGColor : TColor;
               Alignment : TAlignment; Text : string);
var
    xp, yp, textsize :integer;
    parentrep : TQuickrep;
    tmap : TBitmap;
begin
     tmap := TBitmap.Create;
   try
     // make a pdf textitem
     fpdfitem.ItemType := IT_TEXT;
     parentrep := tquickrep( self.owner );
     // text pos is in 1/10 mm
     // map the same as graphic elements
     xp := parentrep.QRPrinter.XPos( X );
     yp := parentrep.QRPrinter.YPos( Y );
     //Alignment - recalculate positions
     case alignment of
            taLeftJustify : ;// do nothing
            taRightJustify :
            begin
                // the xpos is the right of the text pos
                tmap.canvas.Font.Assign( font );
                textsize := tmap.canvas.textwidth( text );
                xp := xp - textsize;
            end;
            taCenter:
            begin
                // the xpos is the center of the text pos
                tmap.canvas.Font.Assign( font );
                textsize := tmap.canvas.textwidth( text );
                xp := xp - (textsize div 2);
            end;
     end;
     fpdfitem.Xpos := xp * PointsperpixelX;
     fpdfitem.Ypos := yp * PointsperpixelY + font.size;
     fpdfitem.FText := allocmem( length(text)+1);
     strpcopy(  fpdfitem.FText,Text );
     fpdfitem.Fontname := allocmem( length(Font.Name)+1);
     strpcopy(  fpdfitem.Fontname,Font.Name );
     fpdfitem.fontsize := Font.Size;
     fpdfitem.rgbfcolor.blue := getBvalue(font.color);
     fpdfitem.rgbfcolor.red := getRvalue(font.color);
     fpdfitem.rgbfcolor.green := getGvalue(font.color);
     fpdfitem.fbold := fsBold in font.Style;
     fpdfitem.fitalic := fsItalic in font.Style;
     AddPDFItem( fpdfitem );
   finally
      tmap.free;
   end;
end;

procedure TQRPDFDocumentFilter.AcceptGraphic( Xoff, Yoff : extended; GControl : TControl);
const
     REC_SIZE = 2048;
var
    xdim, ydim : longint;
    conwidth, conheight, coffset : integer;
    imgfilename : string;
    tempmap : TBitmap;
    SCont : TQRPDFShape;
{$ifdef VER20}
    SSCont : TQRShape;
    qrdbcont : TQRDBImage; 
{$endif}    
    HCont : TQRHRule;
    drawpict : TPicture;
    aspectratio : double;
    dbcont : TQRGrDBImage;
    // turns a byte into 2 hex digits
    procedure Hexit( var c1 : char; var c2 : char; b : byte );
    var
       b1, b2 : byte;
    begin
      b1 := b shr 4;
      b2 := b and $0F;
      if b1 < 10 then
          c1 := chr( b1 + ORD0 )
      else
          c1 := chr( (b1-10) + ORDA );
      if b2 < 10 then
          c2 := chr( b2 + ORD0 )
      else
          c2 := chr( (b2-10) + ORDA );
    end;
    // this is the routine that takes all the time ...
    // compression and UTF85 filters must be applied here ...
    procedure CaptureImageBytes( srcmap : TBitmap);
    var
        k, j : longint;
        b : byte;
        pcolor : TColor;
{$ifdef BINARY_IMG}
        bitmode : TPixelFormat;
        imgstream : TFilestream;
        bbuff : array[0..3000] of byte;
        bp : integer;
{$else}
        imgfile : textfile;
        c1, c2 : char;
{$endif}
        outbuff : string;
    begin
         xdim := srcmap.width;
         ydim := srcmap.height;
         outbuff := '';
         imgfilename := format( 'TF%-6.6d.tmp', [FNextPicnum] );
{$ifdef BINARY_IMG}
         imgstream := TFilestream.Create( imgfilename, fmCreate );
         bp := 0;
{$else}
         assignfile( imgfile, imgfilename);
         rewrite( imgfile );
{$endif}
         //bitmode := srcmap.PixelFormat;
         for j := 0 to ydim-1 do
         begin
             for k := 0 to xdim-1 do
             begin
                   pcolor := srcmap.Canvas.Pixels[ k, j ];
{$ifdef BINARY_IMG}
                   b := byte(pcolor);
                   bbuff[bp] := b;
                   inc(bp);
                   b := byte(pcolor shr 8);
                   bbuff[bp] := b;
                   inc(bp);
                   b := byte(pcolor shr 16);
                   bbuff[bp] := b;
                   inc(bp);
                   if bp = 3000 then
                   begin
                       imgstream.WriteBuffer( bbuff, bp );
                       bp := 0;
                   end;
{$else}
                   b := byte(pcolor);
                   hexit( c1, c2 , b );
                   outbuff := outbuff + c1 + c2;

                   b := byte(pcolor shr 8);
                   hexit( c1, c2, b);
                   outbuff := outbuff + c1 + c2;

                   b := byte(pcolor shr 16);
                   hexit( c1, c2, b);
                   outbuff := outbuff + c1 + c2;
                   if length( outbuff) >= REC_SIZE then
                   begin
                       writeln( imgfile, outbuff );
                       outbuff := '';
                   end;
{$endif}
             end;

         end;
{$ifdef BINARY_IMG}
         if bp > 0 then
                imgstream.WriteBuffer( bbuff, bp );
         imgstream.Free;
{$else}
         if length( outbuff ) > 0 then
              writeln( imgfile, outbuff );
         closefile( imgfile );
{$endif}         
         inc( FNextPicnum );
    end;
begin
  tempmap := TBitmap.create;
  drawpict := TPicture.Create;
  fpdfitem.xscale := 1.0;
  fpdfitem.yscale := 1.0;
  try
    if (GControl is TQRHRule) then
    begin
         HCont := (GControl as TQRHRule);
         fpdfitem.ItemType := IT_GRAPHIC;
         fpdfitem.shape := S_HLINE;
         fpdfitem.Xpos := Xoff * PointsPerpixelX;
         fpdfitem.Ypos := Yoff * PointsPerpixelY;
         fpdfitem.thickness := HCont.Pen.Width;
         fpdfitem.rgbstrokecolor.red := getRvalue( HCont.Pen.Color);
         fpdfitem.rgbstrokecolor.green := getGvalue( HCont.Pen.Color);
         fpdfitem.rgbstrokecolor.blue := getBvalue( HCont.Pen.Color);
         fpdfitem.width := HCont.Width * PointsPerpixelX;
         fpdfitem.height := HCont.height * PointsPerpixelX;
         fpdfitem.filled := false;
         fpdfitem.Fontname := nil;
         fpdfitem.FText := nil;
         addpdfitem( fpdfitem );
         exit;
    end;
    if (GControl is TQRPDFShape) then
    begin
         SCont := (GControl as TQRPDFShape);
         fpdfitem.ItemType := IT_GRAPHIC;
         case scont.shapetype of
           qrsRectangle: fpdfitem.shape := S_BOX;
           qrsCircle: fpdfitem.shape := S_CIRCLE;
           qrsVertLine: fpdfitem.shape := S_VLINE;
           qrsHorLine: fpdfitem.shape := S_HLINE;
           qrsTopAndBottom: fpdfitem.shape := S_TOPBOTTOM;
           qrsRightAndLeft: fpdfitem.shape := S_LEFTRIGHT;
         end;
         fpdfitem.Xpos := Xoff * PointsPerpixelX;
         fpdfitem.Ypos := Yoff * PointsPerpixelY;
         fpdfitem.thickness := scont.Pen.Width;
         fpdfitem.rgbstrokecolor.red := getRvalue( scont.Pen.Color);
         fpdfitem.rgbstrokecolor.green := getGvalue( scont.Pen.Color);
         fpdfitem.rgbstrokecolor.blue := getBvalue( scont.Pen.Color);
         fpdfitem.rgbfcolor.red := getRvalue( scont.brush.Color);
         fpdfitem.rgbfcolor.green := getGvalue( scont.brush.Color);
         fpdfitem.rgbfcolor.blue := getBvalue( scont.brush.Color);
         fpdfitem.width := SCont.Width * PointsPerpixelX;
         fpdfitem.height := SCont.height * PointsPerpixelX;
         fpdfitem.filled := bsClear <> scont.Brush.Style;
         fpdfitem.Fontname := nil;
         fpdfitem.FText := nil;
         addpdfitem( fpdfitem );
         exit;
    end;
    if (GControl is TQRShape) then
    begin
         SSCont := (GControl as TQRShape);
         fpdfitem.ItemType := IT_GRAPHIC;
         fpdfitem.shape := S_HLINE; // default ??
         case SSCont.Shape of
           qrsRectangle: fpdfitem.shape := S_BOX;
           qrsCircle: fpdfitem.shape := S_CIRCLE;
           qrsVertLine: fpdfitem.shape := S_VLINE;
           qrsHorLine: fpdfitem.shape := S_HLINE;
           qrsTopAndBottom: fpdfitem.shape := S_TOPBOTTOM;
           qrsRightAndLeft: fpdfitem.shape := S_LEFTRIGHT;
           qrsRoundrect : fpdfitem.shape := S_OBLIQUE;
         end;
         fpdfitem.Xpos := Xoff * PointsPerpixelX;
         fpdfitem.Ypos := Yoff * PointsPerpixelY;
         fpdfitem.thickness := SSCont.Pen.Width;
         fpdfitem.rgbstrokecolor.red := getRvalue( SSCont.Pen.Color);
         fpdfitem.rgbstrokecolor.green := getGvalue( SSCont.Pen.Color);
         fpdfitem.rgbstrokecolor.blue := getBvalue( SSCont.Pen.Color);
         fpdfitem.rgbfcolor.red := getRvalue( SSCont.brush.Color);
         fpdfitem.rgbfcolor.green := getGvalue( SSCont.brush.Color);
         fpdfitem.rgbfcolor.blue := getBvalue( SSCont.brush.Color);
         fpdfitem.width := SSCont.Width * PointsPerpixelX;
         fpdfitem.height := SSCont.height * PointsPerpixelX;
         fpdfitem.filled := bsClear <> SSCont.Brush.Style;
         fpdfitem.Fontname := nil;
         fpdfitem.FText := nil;
         addpdfitem( fpdfitem );
         exit;
    end;
    //-------------------------------------------------------
    // It's a picture
    tempmap.width := GControl.Width;
    tempmap.height := GControl.height;
    conwidth := GControl.Width;
    conheight := Gcontrol.height;
    if (GControl is TQRGrImage) then
    begin
         if not TQRGrImage(GControl).ClipImage then
             tempmap.canvas.stretchdraw( Rect( 0, 0, tempmap.width, tempmap.height),
                                             TQRGrImage(GControl).Picture.graphic)
         else
         begin
             tempmap.canvas.draw(  0, 0, TQRGrImage(GControl).Picture.graphic)
         end;
         CaptureImageBytes( tempmap );
    end
{$ifdef VER20}
    else if (GControl is TQRImage) then
    begin
         if not TQRImage(GControl).stretch then
             tempmap.canvas.stretchdraw( Rect( 0, 0, tempmap.width, tempmap.height),
                                             TQRImage(GControl).Picture.graphic)
         else
         begin
             tempmap.canvas.draw(  0, 0, TQRImage(GControl).Picture.graphic)
         end;
         CaptureImageBytes( tempmap );
    end
    else if (GControl is TQRDBImage) then
    begin
       qrdbcont := GControl as TQRDBImage;
       if qrdbcont.field <> nil then
       begin
          drawpict.Assign( qrdbcont.field );
          //AspectRatio := 1.0 * drawpict.Width / drawpict.height;
          if not qrdbcont.Stretch then
          begin
              // the picture shrinks or grows to fit the control
              // this tiresome code is a mirror of that in the DBImage print
              if (drawpict.Width/conwidth) < (drawpict.Height/conHeight) then
              begin
                  // the height ratio is greater so shrink the width and center horiz
                  coffset := conwidth; // save the current width
                  conwidth := round(drawpict.width/(drawpict.Height/conHeight));
                  coffset := ( coffset - conwidth) div 2;
                  Xoff :=Xoff + coffset;
              end
              else
              begin
                  coffset := conheight;
                  conheight := round(drawpict.Height/(drawpict.Width/conwidth));
                  coffset := ( coffset - conheight) div 2;
                  Yoff := Yoff + coffset;
              end;
          end;
          //--------
          if qrdbcont.stretch then
               tempmap.assign( qrdbcont.field )
          else
          begin
                  tempmap.canvas.draw(  0, 0, drawpict.graphic);
          end;
          CaptureImageBytes( tempmap );
       end;
    end
{$endif}
    else if (GControl is TQRGrDBImage) then
    begin
       dbcont := GControl as TQRGrDBImage;
       if dbcont.field <> nil then
       begin
          drawpict.Assign( dbcont.field );
          AspectRatio := 1.0 * drawpict.Width / drawpict.height;
          if not dbcont.Stretch then
          begin
               if dbcont.FitMode=fmInsideProp then
               begin
                 // the picture shrinks or grows to fit the control
                 // this tiresome code is a mirror of that in the DBImage print
                 if (drawpict.Width/conwidth) < (drawpict.Height/conHeight) then
                 begin
                  // the height ratio is greater so shrink the width and center horiz
                  coffset := conwidth; // save the current width
                  conwidth := round(drawpict.width/(drawpict.Height/conHeight));
                  coffset := ( coffset - conwidth) div 2;
                  Xoff :=Xoff + coffset;
                 end
                 else
                 begin
                  coffset := conheight;
                  conheight := round(drawpict.Height/(drawpict.Width/conwidth));
                  coffset := ( coffset - conheight) div 2;
                  Yoff := Yoff + coffset;
                 end;
               end
               else if dbcont.FitMode=fmProportional then
               begin
                  if (drawpict.Width/conwidth) < (drawpict.Height/conHeight) then
                     conHeight := trunc( conwidth / aspectratio )
                  else
                     conwidth := trunc( conheight * aspectratio );
               end
               else if dbcont.FitMode=fmKeepWidth then
                   conHeight := trunc( conwidth / aspectratio )
               else if dbcont.FitMode=fmKeepHeight then
                   conwidth := trunc( conheight * aspectratio );
          end;
          //--------
          if not dbcont.ClipImage then
               tempmap.assign( dbcont.field )
          else
          begin
                  tempmap.canvas.draw(  0, 0, drawpict.graphic);
          end;
          CaptureImageBytes( tempmap );
       end;
    end;                                                      
    fpdfitem.ItemType := IT_IMAGE;
    fpdfitem.imagesrc := allocmem( length(imgfilename ) + 1 );
    strpcopy( fpdfitem.imagesrc, imgfilename );
    fpdfitem.Xpos := Xoff * PointsPerpixelX;
    fpdfitem.Ypos := (conheight + Yoff) * PointsperpixelY;
    fpdfitem.width := tempmap.Width * PointsPerpixelX;
    fpdfitem.height := tempmap.height * PointsPerpixelX;
    fpdfitem.pixelwidth := tempmap.Width;
    fpdfitem.pixelheight := tempmap.height;
    fpdfitem.xscale := 1.0 * conWidth / tempmap.Width;
    fpdfitem.yscale := 1.0 * conheight / tempmap.height;
    AddImageItem( fpdfitem, nil );
  finally
    tempmap.free;
    drawpict.free;
  end;
end;
{
    BitBuff : array of byte; // address of array for bitmap bits
    bminf : PBitmapInfo;     // address of structure with bitmap data
GetDIBits( (GControl as TQRImage).canvas.handle,
        (GControl as TQRImage).Picture.bitmap.handle,
        0, 0, nil, bminf, DIB_RGB_COLORS );
}

procedure TQRPDFDocumentFilter.EndPage;
begin
    // do nothing
end;

// Override method
procedure TQRPDFDocumentFilter.NewPage;
begin
    // add a newpage item
    if FFirstPage then
    begin
        FFirstPage := false;
        exit;
    end;
    FPDFItem.ItemType := IT_NEWPAGE;
    AddPDFItem( FPDFItem );
    inc(FPageNumber);
end;

end.

