unit QRWebFilt;
////////////////////////////////////////////////////////////////////////////
//  Unit : QRWebFilt.pas
//
//  TQRGAbstractExportFilter -> TQRGHTMLDocumentFilter
//
//  The new HTML Export document filter class -
//   No lines, columns or text entries.
//  Must use the new QRGrImage, QRGrDBImage controls to export graphics
//
//  (c) 2001 QBS Software
//      Upgraded to be multi-page  29th September 2001
//      Right align bug fixed 11th October
//      Center align bug fixed 14th October
//      Stuff spaces with &nbsp;  ..
//      Change text coords to pixels. Add VertAdjust to HRule   ..
//      Suppress 'next' on final page
//      Add 'FirstLastLinks', 'FinalPage' property
//      25/11/02 add proportional control
//      24th Feb 2002 Added JPEG file saves
//      3rd March 2002 Added concat, linkfontname
////////////////////////////////////////////////////////////////////////////
{$define VER36}
interface
uses windows, classes, controls, stdctrls,sysutils, graphics, buttons,
     forms, extctrls, dialogs, printers, db, DBtables, ComCtrls, qrexport,
     QRPrntr, Quickrpt, QR3Const, qrctrls, grimgctrl{$ifdef JPEGS}, jpeg{$endif};

type

  TQRGAbstractExportFilter = class(TQRExportFilter)
  private
    FStream : TStream;
    FCharWidth,
    FCharHeight,
    FPaperWidth,
    FPaperHeight : extended;
    FLineCount,
    FColCount : integer;
    FPageProcessed : boolean;
    FFont : TFont;
    FActiveFont : TFont;
  protected
    function GetText(X, Y : extended; var Font : TFont) : string;
    function GetFilterName : string; override;
    function GetDescription : string; override;
    function GetExtension : string; override;
    procedure WriteToStream(const AText : string);
    procedure WriteLnToStream(const AText : string);
    procedure CreateStream(Filename : string); virtual;
    procedure CloseStream; virtual;
    procedure ProcessPage; virtual;
    procedure StorePage; virtual;
    property Stream : TStream read FStream write FStream;
    property PageProcessed : boolean read FPageProcessed write FPageProcessed;
    property CharWidth : extended read FCharWidth write FCharWidth;
    property CharHeight : extended read FCharHeight write FCharHeight;
    property PaperWidth : extended read FPaperWidth write FPaperWidth;
    property PaperHeight : extended read FPaperHeight write FPaperHeight;
    property LineCount : integer read FLineCount write FLineCount;
    property ColCount : integer read FColCount write FColCount;
  public
    procedure Start(PaperWidth, PaperHeight : integer; Font : TFont); override;
    procedure Finish; override;
    procedure EndPage; override;
    procedure NewPage; override;
{$ifdef VER36}
    procedure AcceptGraphic( Xoff, Yoff : extended; GControl : TControl); override;
{$else}
    procedure AcceptGraphic( Xoff, Yoff : extended; GControl : TControl); virtual;
{$endif}
    procedure TextOut(X, Y : extended; Font : TFont; BGColor : TColor;
               Alignment : TAlignment; Text : string); override;
  end;

  TQRGHTMLDocumentFilter = class(TQRGAbstractExportFilter)
  private
    FFreeStream : boolean;
    FBodyColor : TColor;
    FBackgroundPic : String;
    FExtraBody : String;
    FTransparentTextBG : boolean;
    FFirstPage : boolean;
    FPageLength : integer;
    FFinalPage : integer;
    FMultiPage : boolean;
    FPageLinks : boolean;
    FConcatenate : boolean;
    FFirstReport : boolean;
    FReportNumber : integer;
    FConcatCount : integer;
    FFirstLastLinks : boolean;
    FPageNumber : longint;
    FLastPage : boolean;
    FPictureDir : string;
    FLinkFontName : string;
    FLinkFontHeight : integer;
    FNextPicnum : longint;
    FPageMaxY : integer;
  protected
    function GetFilterName : string; override;
    function GetDescription : string; override;
    function GetExtension : string; override;
    function GetStreaming : boolean; override;
    procedure CreateStream(Filename : string); override;
    procedure CloseStream; override;
  public
    constructor Create( filename : string );
    procedure StorePage; override;
    procedure TextOut(X, Y : extended; Font : TFont; BGColor : TColor;
               Alignment : TAlignment; Text : string); override;
    procedure ProcessPage; override;
    procedure Start(PaperWidth, PaperHeight : integer; Font : TFont); override;
    procedure Finish; override;
    procedure SetHTMLBody( BodyCol : TColor; BGPic, BExtra : string );
    property TransparentTextBG : boolean read FTransparentTextBG  write FTransparentTextBG;
    procedure NewPage; override;
    procedure EndPage; override;
    procedure AcceptGraphic( Xoff, Yoff : extended; GControl : TControl); override;
    property PictureDir : string read FPictureDir write FPictureDir;
    property MultiPage : boolean read FMultiPage  write FMultiPage;
    property PageLinks : boolean read FPageLinks  write FPageLinks;
    property PageLength : integer read FPageLength write FPageLength;
    property FinalPage : integer read FFinalPage write FFinalPage;
    property FirstLastLinks : boolean read FFirstLastLinks write FFirstLastLinks;
    property Stream;
    property FreeStream : boolean read FFreeStream write FFreeStream;
    property Concat : boolean read FConcatenate write FConcatenate;
    property ConcatCount : integer read FConcatCount write FConcatCount;
    property LinkFontSize : integer read FLinkFontHeight write FLinkFontHeight;
    property LinkFontName : string read FLinkFontName write FLinkFontName;
  end;

  function basename( fname : string ) : string;

implementation

{TQRGAbstractExportFilter}
procedure TQRGAbstractExportFilter.AcceptGraphic( Xoff, Yoff : extended; GControl : TControl);
begin
end;

function TQRGAbstractExportFilter.GetFilterName : string;
begin
  result := 'QRAbstract'; // Do not translate
end;

function TQRGAbstractExportFilter.GetDescription : string;
begin
  Result := '';
end;

function TQRGAbstractExportFilter.GetExtension : string;
begin
  Result := '';
end;

procedure TQRGAbstractExportFilter.Start(PaperWidth, PaperHeight : integer; Font : TFont);
begin
  CreateStream(Filename);
  FFont := TFont.Create;
  FActiveFont := TFont.Create;
  FFont.Assign(Font);
  CharHeight := Font.Size * (254 / 72);
  CharWidth := Font.Size * (254 / 72);
  FPaperHeight := PaperHeight;
  FPaperWidth := PaperWidth;
  LineCount := round(PaperHeight / CharHeight);
  FPageProcessed := false;
  inherited Start(PaperWidth, PaperHeight, Font);
end;


procedure TQRGAbstractExportFilter.CreateStream(Filename : string);
begin
  FStream := TFileStream.Create(Filename, fmCreate);
end;

procedure TQRGAbstractExportFilter.CloseStream;
begin
  FStream.Free;
end;

procedure TQRGAbstractExportFilter.WriteToStream(const AText : string);
begin
  Stream.Write(AText[1], length(AText));
end;

procedure TQRGAbstractExportFilter.WriteLnToStream(const AText : string);
begin
  WriteToStream(AText + #13 + #10);
end;


procedure TQRGAbstractExportFilter.Finish;
begin
  FFont.Free;
  FActiveFont.Free;
  CloseStream;
  inherited Finish;
end;

procedure TQRGAbstractExportFilter.NewPage;
begin
  FPageProcessed := False;
  FActiveFont.Free;
  FActiveFont := TFont.Create;
  inherited NewPage;
end;

procedure TQRGAbstractExportFilter.EndPage;
begin
  ProcessPage;
  inherited EndPage;
end;

procedure TQRGAbstractExportFilter.ProcessPage;
begin
  FPageProcessed := True;
end;

procedure TQRGAbstractExportFilter.StorePage;
begin
end;

procedure TQRGAbstractExportFilter.TextOut(X, Y : extended; Font : TFont;
                      BGColor : TColor; Alignment : TAlignment; Text : string);
begin
end;

function TQRGAbstractExportFilter.GetText(X, Y : extended; var Font : TFont) : string;
begin
end;


{TQRGHTMLDocumentFilter}
function TQRGHTMLDocumentFilter.GetFilterName : string;
begin
  Result := SqrHtmlDocument;
end;

function TQRGHTMLDocumentFilter.GetDescription : string;
begin
  Result := SqrHTMLDocumentForWeb;
end;

function TQRGHTMLDocumentFilter.GetExtension : string;
begin
  Result := 'HTM'; // Do not translate
end;

function TQRGHTMLDocumentFilter.GetStreaming : boolean;
begin
  Result := not MultiPage;// stream for 1 page report mode
end;

procedure TQRGHTMLDocumentFilter.CreateStream(Filename : string);
begin
  if Filename = '' then
  begin
    FStream := TMemoryStream.Create;
    FreeStream := false;
  end else
  begin
    FreeStream := true;
    inherited CreateStream(Filename);
  end;
end;

procedure TQRGHTMLDocumentFilter.CloseStream;
begin
  if FreeStream then
    inherited CloseStream;
end;

// QBS Upgraded HTML filter starts
constructor TQRGHTMLDocumentFilter.Create( filename : string );
begin
   inherited;
   FBackgroundpic := '';
   FBodyColor := clWhite;
   FExtraBody := '';
   FTransparentTextBG := false;
   FFirstPage := true;
   FFirstReport := true;
   FConcatenate := false;
   FMultiPage := false;
   FPageLinks := false;
   FPageNumber := 1;
   FLastPage := false;
   FFinalPage := 0;
   FPageLength := 2500;
   FPictureDir := '';
   FNextPicnum := 0;
   FLinkFontName := 'Arial';
   FLinkFontHeight := 12;
   FReportNumber := 1;
   FConcatCount := 1;

end;
// Give user access to new properties
procedure TQRGHTMLDocumentFilter.SetHTMLBody( BodyCol : TColor; BGPic, BExtra : string );
begin
    FBodyColor := BodyCol;
    FBackGroundPic := BGPic;
    FExtraBody := BExtra;
end;


// QBSS : override method to skip converttoxxx
procedure TQRGHTMLDocumentFilter.ProcessPage;
begin
  FPageProcessed := True;
  StorePage;
end;

function ColTrans( ct : TColor ) : string;
var
   tempstr : string;
begin
    if ct < 0 then
    begin
        result := '#000000';
        exit;
    end;
    tempstr := format ( '%6.6x', [longint(ct)]);
    result := '#' + copy( tempstr, 5, 2 ) +copy( tempstr, 3, 2 )  +copy( tempstr, 1, 2 ) ;
end;


// Overridden Start
procedure TQRGHTMLDocumentFilter.Start(PaperWidth, PaperHeight : integer; Font : TFont);
var
    propstr : string;
begin
  if not multipage then
  begin
    inherited;  // creates a stream
    propstr := ' bgcolor="'+coltrans( FBodycolor)+'" ';
    if trim(FBackgroundpic) <> '' then
          propstr := propstr + ' background="'+FBackgroundpic+'"';
    if trim(FExtraBody) <> '' then
          propstr := propstr + ' ' + FExtraBody;

    writetostream( '<html>' +chr($0D) + chr($0A) );
    writetostream( '<head>' +chr($0D) + chr($0A) );
    writetostream( ' <title>'+self.Filename+'</title>' +chr($0D) + chr($0A) );
    writetostream( '</head>' +chr($0D) + chr($0A)  );
    writetostream( '<body '+propstr+'>' +chr($0D) + chr($0A) );
  end;
end;

// overridden Finish
procedure TQRGHTMLDocumentFilter.Finish;
begin

  if not multipage then
  begin
    writetostream( '</body>' +chr($0D) + chr($0A) );
    writetostream( '</html>' +chr($0D) + chr($0A) );
  end;
  // inherited closes the stream. don't in Concat mode
  if not FConcatenate and not multipage then
      inherited finish
  else;
      Dec( FPagenumber);
  FFirstReport := false;
  inc(FReportNumber);
end;

procedure TQRGHTMLDocumentFilter.EndPage;
var
     Linkstr, hfilename, hdr, prstr, nextstr : string;
     firststr, laststr : string;
begin
    if not FMultiPage or FFirstPage then
    begin
        FFirstPage := false;
        exit;
    end;
    // put in links to other pages
    if fpagelinks and multipage then
    begin
      hdr := '<div style="position:absolute;top:%-dpx;left:10mm;font-family:' + flinkfontname +
                    ';font-size:%-dpt;'+ 'text-decoration:none;text-align:left;">';
      linkstr := format( hdr, [FPageMaxY + 30, flinkfontheight] );
      prstr := '';
      nextstr := '';
      if not FLastPage then
      begin
          hfilename := basename(filename) + format( '%-d.html', [FPageNumber+1] );
          nextstr := '<a href="'+ hfilename +'">Next page</a>';
      end;
      if fpagenumber > 1 then
      begin // put in a back link
         hfilename := basename(filename) + format( '%-d.html', [FPageNumber-1] );
         prstr := '<a href="'+ hfilename +'">Previous page</a>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;'
      end;
      if FFirstlastLinks then
      begin
          if (fpagenumber = 1) then // need to know last pageno
          begin
             hfilename := basename(filename) + format( '%-d.html', [FFinalPage] );
             laststr := '&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<a href="'+ hfilename +'">Last page</a>';
          end;
          if FLastPage then
          begin
             hfilename := basename(filename) + format( '%-d.html', [1] );
             firststr := '<a href="'+ hfilename +'">First page</a>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;';
          end;
      end;
      linkstr := linkstr + firststr + prstr + nextstr + laststr + '<br><br></div>';
    end;
    writetostream( linkstr + chr($0D) + chr($0A) );
    writetostream( '</body>' +chr($0D) + chr($0A) );
    writetostream( '</html>' +chr($0D) + chr($0A) );
    inc(FPageNumber);
end;

// QBSS : override method to skip converttoxxx
procedure TQRGHTMLDocumentFilter.NewPage;
var
     hfilename, propstr : string;
begin
    if not FMultiPage then
    begin
        FFirstPage := false; // if it's not multipage do nowt
        exit;
    end;
    FFirstPage := false;
    closestream;
    FPageMaxY := 0; // can be any small value
    hfilename := basename(filename) + format( '%-d.html', [FPageNumber] );
    createstream( hfilename );
    propstr := ' bgcolor="'+coltrans( FBodycolor)+'" ';
    if trim(FBackgroundpic) <> '' then
          propstr := propstr + ' background="'+FBackgroundpic+'"';
    if trim(FExtraBody) <> '' then
          propstr := propstr + ' ' + FExtraBody;
    writetostream( '<html>' +chr($0D) + chr($0A) );
    writetostream( '<head>' +chr($0D) + chr($0A) );
    writetostream( ' <title>' + hfilename + '</title>' +chr($0D) + chr($0A) );
    writetostream( '</head>' +chr($0D) + chr($0A)  );
    writetostream( '<body '+propstr+'>' +chr($0D) + chr($0A) );
end;
procedure TQRGHTMLDocumentFilter.StorePage;
begin
     // do nothing.
end;

// Using absolute positions we can output immediately
procedure TQRGHTMLDocumentFilter.TextOut(X, Y : extended; Font : TFont; BGColor : TColor;
               Alignment : TAlignment; Text : string);
//<DIV style="position:absolute;left:1px;top:184px;"><hr width="500" size="2" color="black"></div>
const
    // 12/10/01 top, left of <DIV now in pixels.
    DivStrDef = '<div style="position:absolute;top:!toppx;left:!leftpx;font-family:!fontname;'
                +'font-size:!fontsizept;'+
                 'color:!color;text-decoration:!deco;text-align:left;background-color:!bgcolor;'+
                 'font-style:!style;font-weight :!weight;">';
var
  I, K, afontheight, xp, yp, pixely : integer;
  textsize : integer;
  Fontcolor : TColor;
  AText, afontname, fitalic, fbold, funderline, fstrike, fbgcolor : string;
  NewDivStr, aLinkText, originaltext : string;
  parentrep : TQuickrep;
  CBand : TQRCustomband;
  break2 : boolean;
  cLinkType : TLinkDef;
  tmap : TBitmap;

  procedure FixDivStr;
  begin
        newdivstr := stringreplace( newdivstr, '!top', format( '%-d', [yp] ), [rfReplaceAll] );
        newdivstr := stringreplace( newdivstr, '!left', format( '%-d', [xp] ), [rfReplaceAll] );
        newdivstr := stringreplace( newdivstr, '!fontname', trim(afontname), [rfReplaceAll] );
        newdivstr := stringreplace( newdivstr, '!fontsize', format( '%-d', [afontheight] ), [rfReplaceAll] );
        newdivstr := stringreplace( newdivstr, '!color', trim(coltrans(fontcolor)), [rfReplaceAll] );
        newdivstr := stringreplace( newdivstr, '!bgcolor', fbgcolor, [rfReplaceAll] );
        newdivstr := stringreplace( newdivstr, '!style', fitalic, [rfReplaceAll] );
        newdivstr := stringreplace( newdivstr, '!weight', fbold, [rfReplaceAll] );
        newdivstr := stringreplace( newdivstr, '!deco', funderline, [rfReplaceAll] );
  end;

begin
      tmap := TBitmap.create;
      try
        atext := Text;
        originaltext := Text;
        aLinktext := '';
        clinktype := ldHTTPJump; // keep compiler quiet !
        parentrep := TQuickrep( self.owner );
        if parentrep.DataSet <> nil then
              FLastPage := ( parentrep.DataSet.Eof and ( FConcatcount = FReportNumber ));
        break2 := false;
        for i := 0 to parentrep.ControlCount-1 do
        begin
             if  parentrep.Controls[i].InheritsFrom( TQRCustomBand ) then
             begin
                  CBand := TQRCustomBand(parentrep.Controls[i]);
                  for k := 0 to CBand.controlcount-1 do
                    if atext = '%!'+CBand.controls[k].name+'!%' then
                    begin
                          if (CBand.controls[k] is TQRHTMLLabel ) then
                          begin
                               aLinkText := TQRHTMLLabel( CBand.controls[k]).linktext;
                               aText := TQRHTMLLabel( CBand.controls[k]).caption;
                               originaltext := aText;
                               cLinkType := TQRHTMLLabel( CBand.controls[k]).LinkType;
                               break2 := true;
                               break;
                          end;
                    end;
                    if break2 then break;
             end;
        end;
        // must be a standard text or label control
        newdivstr := divstrdef;
        atext := stringreplace( atext, '<', '&lt;', [rfReplaceAll] );
        atext := stringreplace( atext, '>', '&gt;', [rfReplaceAll] );
        // 15/10/01 stuff spaces in text
        atext := stringreplace( atext, ' ', '&nbsp;', [rfReplaceAll] );
        afontname := Font.Name;
        aFontheight := Font.Size;
        if alinktext <> '' then
        begin
              if cLinkType = ldHTTPJump then
                 atext := '<a href="' + aLinktext + '">' + aText + '</a>'
              else if cLinkType = ldNameJump then
                 atext := '<a href="#' + aLinktext + '">' + aText + '</a>'
              else
                 atext := '<a name="' + aLinktext + '">';
        end;
        // 12/10/01 try pixels ? map the same as graphic elements
        xp := parentrep.QRPrinter.XPos( X );
        yp := parentrep.QRPrinter.YPos( Y );
        // 08/10/01 Alignment - recalculate positions
        case alignment of
            taLeftJustify : ;// do nothing
            taRightJustify :
            begin
                // the xpos is the right of the text pos
                tmap.canvas.Font.Assign( font );
                // must use 'originaltext' not 'atext' because we may have added html
                textsize := tmap.canvas.textwidth( originaltext );
                xp := xp - textsize;
            end;
            taCenter:
            begin
                // the xpos is the center of the text pos
                tmap.canvas.Font.Assign( font );
                // must use 'originaltext' not 'atext' because we may have added html
                textsize := tmap.canvas.textwidth( originaltext );
                xp := xp - (textsize div 2);
            end;
        end;
        // used to position links at page bottom
        pixely := yp; // to pixels
        if pixely > FPageMaxY then FPageMaxY := pixely;

        Fontcolor := Font.Color;
        fBold := 'normal';
        fitalic := 'none';
        funderline := 'none';
        if FTransparentTextBG then
            fbgcolor := 'Transparent'
        else
            fbgcolor := 'white';
        if fsBold in Font.Style then
               fbold := 'Bold';
        if fsItalic in Font.Style then
               fitalic := 'Italic';
        if fsUnderline in Font.Style then
               funderline := 'Underline';
        if fsStrikeout in Font.Style then
               fstrike := 'Strikeout';

        FixDivStr;
        writeToStream( newdivstr + chr($0D)+chr($0A));
        writeToStream( atext+ chr($0D)+chr($0A) );
        writeToStream('</DIV>'+ chr($0D)+chr($0A));
      finally
          tmap.free;
      end;
end;

// strip off file extension
function basename( fname : string ) : string;
var
   p : integer;
begin
    basename := fname;
    p := pos( '.', fname );
    if p = 0 then exit;
    basename := copy( fname, 1, p - 1 );
end;

procedure TQRGHTMLDocumentFilter.AcceptGraphic( Xoff, Yoff : extended; GControl : TControl);
var
    xpos, ypos, xdim, ydim, conwidth, conheight : integer;
    imgfile, divstr, FText : string;
    tempmap : TBitmap;
    SCont : TQRHRule;
    dbcont : TQRGrDBImage;
    SSCont : TQRShape;
    qrdbcont : TQRDBImage;
    gdest : TRect;
    aspectratio : double;
    drawpict : TPicture;
{$ifdef JPEGS}
    jpeg : TJPEGImage;
{$endif}
begin
  tempmap := TBitmap.create;
  drawpict := TPicture.create;
{$ifdef JPEGS}
  jpeg := TJPEGImage.create;
{$endif}
  try
    gdest.Left := trunc(Xoff);
    gdest.Top := round(Yoff);
    gdest.Right := gdest.left + GControl.Width;
    gdest.Bottom := gdest.top + GControl.height;
    conheight := GControl.height;
    conwidth := GControl.Width;
    if GControl is TQRHRule then
    begin
         SCont := (GControl as TQRHRule);
         gdest.top := gdest.top - 8 - scont.VertAdjust;
         divstr := '<DIV style="position:absolute;left:'+
            format( '%-d', [gdest.left]) + 'px;top:' +
            format( '%-d', [gdest.top]) +
            format('px;height:%-dpx;"><hr width="', [ SCont.pen.width] ) +
            format( '%-d', [scont.width]) + '" size="' +
            format( '%-d', [SCont.pen.width]) + '" color="' +
            coltrans(scont.pen.color)+'"></div>';
         writeToStream( divstr + chr($0D)+chr($0A));
         if (gdest.top) > FPageMaxY then FPageMaxY := gdest.top;
         exit;
    end
{$ifdef VER36}
    else if GControl is TQRShape then
    begin
         SSCont := (GControl as TQRShape);
         if SSCont.Shape <> qrsHorLine then exit;
         gdest.top := gdest.top - 8 - sscont.VertAdjust;
         divstr := '<DIV style="position:absolute;left:'+
            format( '%-d', [gdest.left]) + 'px;top:' +
            format( '%-d', [gdest.top]) +
            format('px;height:%-dpx;"><hr width="', [ SSCont.pen.width] ) +
            format( '%-d', [sScont.width]) + '" size="' +
            format( '%-d', [SSCont.pen.width]) + '" color="' +
            coltrans(sscont.pen.color)+'"></div>';
         writeToStream( divstr + chr($0D)+chr($0A));
         if (gdest.top) > FPageMaxY then FPageMaxY := gdest.top;
         exit;
    end
{$endif}
    else if GControl is TQRGrImage then
    begin
       tempmap.width := GDest.right-gdest.left;
       tempmap.height := gdest.bottom-gdest.top;
{$ifdef JPEGS}
       imgfile := GControl.Name + '.jpg';
{$else}
       imgfile := GControl.Name + '.bmp';
{$endif}
       if trim(picturedir) <> '' then
                 imgfile := IncludeTrailingBackslash( FPictureDir ) + imgfile;
       if (GControl as TQRGrImage).Picture.Graphic is TBitmap then
       begin
{$ifdef JPEGS}}
            jpeg.Assign((GControl as TQRGrImage).Picture.bitmap);
            jpeg.savetofile(imgfile);
{$else}
             tempmap.assign( (GControl as TQRGrImage).Picture.bitmap);
             tempmap.savetofile(imgfile);
{$endif}
       end
       else
       begin
            tempmap.canvas.stretchdraw( Rect( 0, 0, tempmap.width, tempmap.height),
                                   (GControl as TQRGrImage).Picture.graphic);
{$ifdef JPEGS}}
            jpeg.Assign(tempmap);
            jpeg.savetofile(imgfile);
{$else}
             tempmap.savetofile(imgfile);
{$endif}
       end;
    end
{$ifdef VER36}
    else if GControl is TQRImage then
    begin
       tempmap.width := GDest.right-gdest.left;
       tempmap.height := gdest.bottom-gdest.top;
{$ifdef JPEGS}
       imgfile := GControl.Name + '.jpg';
{$else}
       imgfile := GControl.Name + '.bmp';
{$endif}
       if trim(picturedir) <> '' then
                 imgfile := IncludeTrailingBackslash( FPictureDir ) + imgfile;
       if (GControl as TQRImage).Picture.Graphic is TBitmap then
       begin
{$ifdef JPEGS}}
            jpeg.Assign((GControl as TQRGrImage).Picture.bitmap);
            jpeg.savetofile(imgfile);
{$else}
             tempmap.assign( (GControl as TQRGrImage).Picture.bitmap);
             tempmap.savetofile(imgfile);
{$endif}
       end
       else
       begin
            tempmap.canvas.stretchdraw( Rect( 0, 0, tempmap.width, tempmap.height),
                                   (GControl as TQRImage).Picture.graphic);
{$ifdef JPEGS}}
            jpeg.Assign(tempmap);
            jpeg.savetofile(imgfile);
{$else}
             tempmap.savetofile(imgfile);
{$endif}
       end;
    end
    else if (GControl is TQRDBImage) then
    begin
      qrdbcont := GControl as TQRDBImage;
      // proportional sizing
      drawpict.assign( qrdbcont.field );
      AspectRatio := 1.0 * drawpict.Width / drawpict.height;
      if not qrdbcont.stretch then
      begin
         if (drawpict.Width/conwidth) < (drawpict.Height/conHeight) then
               conHeight := trunc( conwidth / aspectratio )
         else
              conwidth := trunc( conheight * aspectratio );
      end;
      tempmap.width := conwidth;
      tempmap.height := conheight;

      inc( FNextPicnum );
{$ifdef JPEGS}
      imgfile := GControl.Name + format( '%-d', [FNextPicnum] ) + '.jpg';
{$else}
      imgfile := GControl.Name + format( '%-d', [FNextPicnum] ) + '.bmp';
{$endif}
      if trim(picturedir) <> '' then
            imgfile := IncludeTrailingBackslash( FPictureDir ) + imgfile;
      if qrdbcont.field <> nil then
      begin
            tempmap.assign( qrdbcont.field );
{$ifdef JPEGS}
            jpeg.Assign(tempmap);
{$endif}
      end;
{$ifdef JPEGS}
      jpeg.savetofile( imgfile );// blank pic for nil field
{$else}
      tempmap.savetofile( imgfile );
{$endif}
    end
{$endif}
    else if (GControl is TQRGrDBImage) then
    begin
      dbcont := GControl as TQRGrDBImage;
      // proportional sizing
      drawpict.assign( dbcont.field );
      AspectRatio := 1.0 * drawpict.Width / drawpict.height;
      if not dbcont.stretch then
      begin
       if dbcont.FitMode=fmProportional then
       begin
         if (drawpict.Width/conwidth) < (drawpict.Height/conHeight) then
               conHeight := trunc( conwidth / aspectratio )
         else
              conwidth := trunc( conheight * aspectratio );
       end
       else if dbcont.FitMode=fmKeepWidth then
               conHeight := trunc( conwidth / aspectratio )
       else if dbcont.FitMode=fmKeepHeight then
                conwidth := trunc( conheight * aspectratio );
      end;
      tempmap.width := conwidth;
      tempmap.height := conheight;

      inc( FNextPicnum );
{$ifdef JPEGS}
      imgfile := GControl.Name + format( '%-d', [FNextPicnum] ) + '.jpg';
{$else}
      imgfile := GControl.Name + format( '%-d', [FNextPicnum] ) + '.bmp';
{$endif}
      if trim(picturedir) <> '' then
            imgfile := IncludeTrailingBackslash( FPictureDir ) + imgfile;
      if dbcont.field <> nil then
      begin
            tempmap.assign( dbcont.field );
{$ifdef JPEGS}
            jpeg.Assign(tempmap);
{$endif}
      end;
{$ifdef JPEGS}
      jpeg.savetofile( imgfile );// blank pic for nil field
{$else}
      tempmap.savetofile( imgfile );
{$endif}
    end;
    FText := imgfile;
    XPos := GDest.left;
    YPos := GDest.top;
    XDim := conwidth;
    YDim := conheight;
    if (YPos+ydim) > FPageMaxY then FPageMaxY := (YPos+ydim);
    // graphic link : units pixels
    divstr := '<DIV style="position:absolute;left:'+
                       format( '%-d', [xpos]) +'px;top:'+
                       format( '%-d', [ypos]) +'px;">'+
                       '<img src="'+FText+'" width=' +
                       format( '%-d', [xdim])+' height=' +
                       format( '%-d', [ydim])+'></div>';
    writeToStream( divstr + chr($0D)+chr($0A));
  finally
    tempmap.free;
    drawpict.free;
{$ifdef JPEGS}
    jpeg.free;
{$endif}    
  end;
end;


end.

